// DrawShapes.r
// Copyright  1986-95 by Apple Computer, Inc. All rights reserved. 

/*
	Change History:
		 6/23/95	TWB		Hack. 
		 2/22/95	srf		use XAppName when including CODE resources. 
		 1/19/95	TWB		Change the shapes view to want to be target. 
		 1/13/95	TWB		Include DrawShapesPict. 
		 1/12/95	TWB		Make the tools palette view sizeSuperView instead of
							sizeFixed so we can dynamically resize the palette
							window. 
		10/14/94	TWB		Use qNoCode. 
*/

/*  Auto-Include the requirements for this source */
#ifndef __TYPES_R__
#include "Types.r"
#endif

#ifndef __MacAppTypes__
#include "MacAppTypes.r"
#endif

#ifndef __ViewTypes__
#include "ViewTypes.r"
#endif

#if qDebug | qPerform
include "Debug.rsrc" not 'ckid';
#endif
include "Dialog.rsrc" not 'ckid';
include "MacApp.rsrc" not 'ckid';
include "Printing.rsrc" not 'ckid';

#ifndef __FloatWindow__
#include "FloatWindow.r"
#endif

include $$Shell("ObjApp")"DrawShapesPict.rsrc";

#if !qNoCode
include $$Shell("ObjApp")$$Shell("XAppName") 'CODE';
#endif

/* Resource ids */
#define kShapeWindowRSRCID		1005			/* The main document window */
#define kShapeViewRSRCID		1006			/* the shape view resource */
#define kToolsPaletteRSRCID		1007			// the floating window tools palette
#define kPatternsPaletteRSRCID	1008			// the floating window patterns palette
#define kColorMenuBar			 131			/* Menu bar for a color system */
#define kNonColorMenuBar		 132			/* Menu bar for a black & white system */
#define kRainbowArrow			 140
#define kPickerPrompt			 256			/* 'STR ' resource for Color Picker */

#define mGoodies				   4			/* the goodies menu */
#define mTools					   5			/* The Tools menu resource id */
#define mPatterns				   6			/* The Patterns menu resource id */
#define mColor					   7			/* The Color menu resource id */
#define mMoreDebug				   8			/* Menu number for the 'More Debug' menu */
#define mHack					   9

/* Command numbers */
#define cArrow					1100
#define cBox					1101
#define cCircle					1102
#define cHBox					1103

#define cPatterns				1300			/* The 'Patterns' menu */

#define cPickColor				1005			/* Command to bring up Color Picker */

#define cNewShape				1010			/* Command to draw a new shape */
#define cMoveShape				1011			/* Command to move one or more shapes */
#define cChangeShade			1012			/* Buzz command for "Undo Shade Change" */
#define cChangeColor			1013			/* Buzz command for "Undo Color Change" */
#define cChangeTool				1014			/* Buzz command for "Undo Tool Change" */

#define cPasteReplacesSelection	 118			/* Three commands in the 'More Debug' menu */
#define cRecalcExtent			 119
#define cConstrainDrags 		 120
#define cBetterFeedback			 121


/* view constants */
#define kMaxCoord				30000			/* largest possible QuickDraw view coordinate
												(QuickDraw maximum minus slop for size of screen) */
#define kToolsPaletteWidth			41			// Width of the tools palette
#define kToolsPaletteHeight			160			// height of the tools palette 

#define kPatternsAcross				4			// how many patterns across
#define kPatternsDown 				10			// how many patterns down
#define kTotalPatterns				kPatternsAcross * kPatternsDown

#define kWPatternsCell 				18			// Width of patterns Cell
#define kHPatternsCell 				14			// Height of patterns Cell
#define kHorizontalFrames		kPatternsDown - 1		
#define kVerticalFrames			kPatternsAcross - 1		

// Width of the Patterns palette
#define kWPatternsChoice				kWPatternsCell * kPatternsAcross + kVerticalFrames
// height of the Patterns palette
#define kHPatternsChoice				kHPatternsCell * kPatternsDown + kHorizontalFrames

#define kSignature		'SS05'					// Application signature
#define kFileType		'SF05'					// Application file type
#define kStationery		'sF05'					// Application stationery type

resource 'DITL' (phAboutApp, purgeable) {
	 {
/* [ 1] */	{160, 182, 180, 262},
			Button {
				enabled,
				"OK"
			};
/* [ 2] */	{10, 75, 150, 320},
			StaticText {
				disabled,
					"This sample program demonstrates many features of MacApp.\n\n"
					"This program was written with MacApp  1985-95 by Apple Computer, Inc."
			};
/* [ 3] */	{10, 20, 42, 52},
			Icon {
				disabled,
				1
			}
	}
};

include "Defaults.rsrc"  'ALRT' (phAboutApp);	// Grab the default about box

resource 'View' (kShapeWindowRSRCID, purgeable) {MAThreeOh, 
	{
	ViewSignatureAndClassname
		{'wind', 531, "", 'WIND', enabled, noIdle, {}, MAThreeOh, 
		{50, 40}, {200, 396}, sizeVariable, sizeVariable, shown, doesntWantToBeTarget, 
		handlesCursor, letsSubViewsHandleCursor, noCursorID, 
		handlesHelp, letsSubViewsHandleHelp, noHelpID, 1, 
		NoDrawingEnvironment {}, 
		AdornerListSignatureAndClassname {NoIdentifier, AdornerList, AdornerElementSize, AdornerElementSizeShift, DynamicArrayAllocationIncrement, {
			AdornFirst, AdornerLocalObject {EraseAdorner}, 
			DrawView, AdornerLocalObject {DrawAdorner}, 
			AdornLast, AdornerLocalObject {ResizeIconAdorner}}}, emptyUserArea, 
		Window {zoomDocProc, 'SHAP', goAwayBox, resizable, ignoreFirstClick, 
			freeOnClosing, disposeOnFree, closesDocument, openWithDocument, dontAdaptToScreen, stagger, forceOnScreen, 
			dontCenter, doesntFloat, doesntHideOnSuspend, generateActivates, filler, 1006, 1}, 3}, 
	ViewSignatureAndClassname
		{'scrl', 183, "", 'SCLR', enabled, noIdle, {}, MAThreeOh, 
		{0, 0}, {185, 381}, sizeRelSuperView, sizeRelSuperView, shown, doesntWantToBeTarget, 
		handlesCursor, letsSubViewsHandleCursor, noCursorID, 
		handlesHelp, letsSubViewsHandleHelp, noHelpID, 1, 
		NoDrawingEnvironment {}, 
		NoAdorners {}, emptyUserArea, 
		Scroller {'vCLR', 'hCLR', {kMaxCoord, kMaxCoord}, {16, 16}, noVertConstrain, noHorzConstrain, noInset, respondsToFKeys}, 1}, 
	ViewSignatureAndClassname
		{'inc@', 72, "", noID, enabled, noIdle, {}, MAThreeOh, 
		{0, 0}, {kMaxCoord, kMaxCoord}, sizeFillPages, sizeFillPages, shown, doesntWantToBeTarget, 
		handlesCursor, letsSubViewsHandleCursor, noCursorID, 
		handlesHelp, letsSubViewsHandleHelp, noHelpID, 1, 
		NoDrawingEnvironment {}, 
		NoAdorners {}, emptyUserArea, 
		IncludeAt {kShapeViewRSRCID, {0, 0}}, 
		NoSubviews}, 
	ViewSignatureAndClassname
		{'ssbr', 101, "", 'hCLR', enabled, noIdle, {}, MAThreeOh, 
		{185, -1}, {16, 383}, sizeVariable, sizeVariable, shown, doesntWantToBeTarget, 
		handlesCursor, letsSubViewsHandleCursor, noCursorID, 
		handlesHelp, letsSubViewsHandleHelp, noHelpID, 1, 
		NoDrawingEnvironment {}, 
		NoAdorners {}, emptyUserArea, 
		ScrollerScrollBar {mHScrollBarHit, notHilited, notDimmed, sizeable, noInset, 128, dontPreferOutline, h, 0, 0, 29619}, 
		NoSubviews}, 
	ViewSignatureAndClassname
		{'ssbr', 101, "", 'vCLR', enabled, noIdle, {}, MAThreeOh, 
		{-1, 381}, {187, 16}, sizeVariable, sizeVariable, shown, doesntWantToBeTarget, 
		handlesCursor, letsSubViewsHandleCursor, noCursorID, 
		handlesHelp, letsSubViewsHandleHelp, noHelpID, 1, 
		NoDrawingEnvironment {}, 
		NoAdorners {}, emptyUserArea, 
		ScrollerScrollBar {mVScrollBarHit, notHilited, notDimmed, sizeable, noInset, 128, dontPreferOutline, v, 0, 0, 29815}, 
		NoSubviews}
	}
};

resource 'View' (kShapeViewRSRCID, purgeable) {MAThreeOh, 
	{
	ViewSignatureAndClassname
		{'view', 72, "TShapeView", 'SHAP', enabled, noIdle, {}, MAThreeOh, 
		{0, 0}, {kMaxCoord, kMaxCoord}, sizeFillPages, sizeFillPages, shown, wantsToBeTarget, 
		handlesCursor, letsSubViewsHandleCursor, noCursorID, 
		handlesHelp, letsSubViewsHandleHelp, noHelpID, 1, 
		NoDrawingEnvironment {}, 
		NoAdorners {}, emptyUserArea, 
		View {}, 
		NoSubviews}
	}
};

resource 'View' (kToolsPaletteRSRCID, purgeable) {MAThreeOh, 
	{
	ViewSignatureAndClassname
		{'fwnd', 214, "", 'TWND', enabled, noIdle, {}, MAThreeOh, 
		{80, 80}, {kToolsPaletteHeight, kToolsPaletteWidth}, sizeFixed, sizeFixed, shown, doesntWantToBeTarget, 
		handlesCursor, letsSubViewsHandleCursor, noCursorID, 
		handlesHelp, letsSubViewsHandleHelp, noHelpID, 1, 
		NoDrawingEnvironment {}, 
		AdornerListSignatureAndClassname {NoIdentifier, AdornerList, AdornerElementSize, AdornerElementSizeShift, DynamicArrayAllocationIncrement, {
			AdornFirst, AdornerLocalObject {EraseAdorner}, 
			DrawView, AdornerLocalObject {DrawAdorner}}}, emptyUserArea, 
		FloatWindow {kWindoidWDEF + kWindoidNotResizable, noID, goAwayBox, notResizable, doFirstClick, 
			dontFreeOnClosing, dontDisposeOnFree, doesntCloseDocument, dontOpenWithDocument, dontAdaptToScreen, dontStagger, forceOnScreen, 
			dontCenter, floats, hidesOnSuspend, doesntGenerateActivates, filler, 1006, 1}, 1}, 
	ViewSignatureAndClassname
		{'view', 75, "TToolsPalette", 'TPLT', enabled, noIdle, {}, MAThreeOh, 
		{0, 0}, {kToolsPaletteHeight, kToolsPaletteWidth}, sizeSuperView, sizeSuperView, shown, doesntWantToBeTarget, 
		handlesCursor, letsSubViewsHandleCursor, noCursorID, 
		handlesHelp, letsSubViewsHandleHelp, noHelpID, 1, 
		NoDrawingEnvironment {}, 
		NoAdorners {}, emptyUserArea, 
		View {}, 
		NoSubviews}
	}
};


resource 'View' (kPatternsPaletteRSRCID, purgeable) { MAThreeOh,
	{
	ViewSignatureAndClassname
		{'wind', 368, "TFloatWindow", 'PWND', enabled, noIdle, {}, MAThreeOh,
			{80, 80}, {149, 76}, sizeFixed, sizeFixed, shown, doesntWantToBeTarget,
			handlesCursor, letsSubViewsHandleCursor, noCursorID,
			handlesHelp, letsSubViewsHandleHelp, noHelpID, 1,
			NoDrawingEnvironment {},
			AdornerListSignatureAndClassname {
				NoIdentifier,
				AdornerList,
				AdornerElementSize,
				AdornerElementSizeShift,
				DynamicArrayAllocationIncrement,
				{	/* array AdornerElementArray: 2 elements */
					/* [1] */
					AdornFirst,
					AdornerLocalObject {
						EraseAdorner
					},
					/* [2] */
					DrawView,
					AdornerLocalObject {
						DrawAdorner
					}
				}
			},
			emptyUserArea,
			Window {
				52,
				noID,
				goAwayBox,
				notResizable,
				doFirstClick,
				dontFreeOnClosing,
				dontDisposeOnFree,
				doesntCloseDocument,
				dontOpenWithDocument,
				dontAdaptToScreen,
				dontStagger,
				forceOnScreen,
				dontCenter,
				floats,
				hidesOnSuspend,
				doesntGenerateActivates,
				filler,
				1006,
				1
			},
			1
		},
		/* [2] */
		ViewSignatureAndClassname {
			'view',
			229,
			"TPatternsPalette",
			'PPLT',
			enabled,
			noIdle,
			{	/* array BehaviorArray: 0 elements */
			},
			MAThreeOh,
			{	/* array: 2 elements */
				/* [1] */
				0,
				/* [2] */
				0
			},
			{	/* array: 2 elements */
				/* [1] */
				149,
				/* [2] */
				75
			},
			sizeFixed,
			sizeFixed,
			shown,
			doesntWantToBeTarget,
			handlesCursor,
			letsSubViewsHandleCursor,
			noCursorID,
			handlesHelp,
			letsSubViewsHandleHelp,
			noHelpID,
			1,
			NoDrawingEnvironment {

			},
			NoAdorners {

			},
			emptyUserArea,
			View {

			},
			NoSubviews
		}
	}
};

resource 'View' (1009, purgeable) {MAThreeOh, 
	{
	ViewSignatureAndClassname
		{'fwnd', 705, "", noID, enabled, noIdle, {}, MAThreeOh, 
		{80, 80}, {130, 300}, sizeVariable, sizeVariable, shown, doesntWantToBeTarget, 
		handlesCursor, letsSubViewsHandleCursor, noCursorID, 
		handlesHelp, letsSubViewsHandleHelp, noHelpID, 1, 
		NoDrawingEnvironment {}, 
		AdornerListSignatureAndClassname {NoIdentifier, AdornerList, AdornerElementSize, AdornerElementSizeShift, DynamicArrayAllocationIncrement, {
			AdornFirst, AdornerLocalObject {EraseAdorner}, 
			DrawView, AdornerLocalObject {DrawAdorner}}}, emptyUserArea, 
		FloatWindow {52, noID, goAwayBox, notResizable, doFirstClick, 
			dontFreeOnClosing, dontDisposeOnFree, doesntCloseDocument, dontOpenWithDocument, dontAdaptToScreen, dontStagger, forceOnScreen, 
			dontCenter, floats, hidesOnSuspend, doesntGenerateActivates, filler, 1002, -1}, 1}, 
	ViewSignatureAndClassname
		{'view', 578, "THackView", 'hack', enabled, noIdle, {}, MAThreeOh, 
		{0, 0}, {130, 300}, sizeSuperView, sizeSuperView, shown, doesntWantToBeTarget, 
		handlesCursor, letsSubViewsHandleCursor, noCursorID, 
		handlesHelp, letsSubViewsHandleHelp, noHelpID, 1, 
		NoDrawingEnvironment {}, 
		NoAdorners {}, emptyUserArea, 
		View {}, 4}, 
	ViewSignatureAndClassname
		{'butn', 92, "", 'bABO', enabled, noIdle, {}, MAThreeOh, 
		{90, 36}, {20, 64}, sizeFixed, sizeFixed, shown, doesntWantToBeTarget, 
		handlesCursor, letsSubViewsHandleCursor, noCursorID, 
		handlesHelp, letsSubViewsHandleHelp, noHelpID, 1, 
		NoDrawingEnvironment {}, 
		NoAdorners {}, emptyUserArea, 
		Button {mButtonHit, notHilited, notDimmed, notSizeable, noInset, 1000, dontPreferOutline, 1000, 1}, 
		NoSubviews}, 
	ViewSignatureAndClassname
		{'butn', 92, "", 'bRET', enabled, noIdle, {}, MAThreeOh, 
		{90, 125}, {20, 64}, sizeFixed, sizeFixed, shown, doesntWantToBeTarget, 
		handlesCursor, letsSubViewsHandleCursor, noCursorID, 
		handlesHelp, letsSubViewsHandleHelp, noHelpID, 1, 
		NoDrawingEnvironment {}, 
		NoAdorners {}, emptyUserArea, 
		Button {mButtonHit, notHilited, notDimmed, notSizeable, noInset, 1000, dontPreferOutline, 1000, 2}, 
		NoSubviews}, 
	ViewSignatureAndClassname
		{'butn', 203, "", 'bIGN', enabled, noIdle, {}, MAThreeOh, 
		{86, 215}, {28, 72}, sizeFixed, sizeFixed, shown, doesntWantToBeTarget, 
		handlesCursor, letsSubViewsHandleCursor, noCursorID, 
		handlesHelp, letsSubViewsHandleHelp, noHelpID, 1, 
		DrawingEnvironmentClassIDAndClassname {1, "TDrawingEnvironment", {3, 3}, 8, $"FFFF FFFF FFFF FFFF", {0, 0, 0}, {65535, 65535, 65535}}, 
		AdornerListSignatureAndClassname {NoIdentifier, AdornerList, AdornerElementSize, AdornerElementSizeShift, DynamicArrayAllocationIncrement, {
			DrawView, AdornerLocalObject {DrawAdorner}, 
			AdornAfter, AdornerSignatureAndClassname {'rrct', "", 'outl', freeOnDeletion, $""}}}, emptyUserArea, 
		Button {mDismiss, notHilited, notDimmed, notSizeable, {4, 4, 4, 4}, 1000, dontPreferOutline, 1000, 3}, 
		NoSubviews}, 
	ViewSignatureAndClassname
		{'stat', 96, "", noID, notEnabled, noIdle, {}, MAThreeOh, 
		{13, 19}, {55, 266}, sizeFixed, sizeFixed, shown, doesntWantToBeTarget, 
		handlesCursor, letsSubViewsHandleCursor, noCursorID, 
		handlesHelp, letsSubViewsHandleHelp, noHelpID, 1, 
		NoDrawingEnvironment {}, 
		NoAdorners {}, emptyUserArea, 
		StaticText {mStaticTextHit, notHilited, notDimmed, notSizeable, noInset, 1000, dontPreferOutline, autoWrap, dontEraseFirst, justSystem, 1001, 1}, 
		NoSubviews}
	}
};

resource 'TxSt' (1000) {
	tsPlain,
	0,
	{	/* array: 3 elements */
		/* [1] */
		0x0,
		/* [2] */
		0x0,
		/* [3] */
		0x0
	},
	""
};

//--------------------------------------------------------------------------------

resource 'SIZE' (-1) {
	saveScreen,
	acceptSuspendResumeEvents,
	enableOptionSwitch,
	canBackground,
	doesActivateOnFGSwitch,
	backgroundAndForeground,
	dontGetFrontClicks,
	ignoreAppDiedEvents,
	is32BitCompatible,
	isHighLevelEventAware,
	localAndRemoteHLEvents,
	isStationeryAware,				
	reserved,
	reserved,
	reserved,
	reserved,
#if qDebug
	1200 * 1024,
	1000 * 1024
#else
	800 * 1024,													// try to leave enough room for printing
	600 * 1024
#endif
};


resource 'mem!' (132,
#if qNames
	"Additional Stack Space",
#endif
	purgeable) {
	0,				// Add to object heap size
	0,				// Add to heap increment size
	0,				// Add to temporary reserve
	0,				// Add to permanent reserve
	1024			// Add to stack space
};

/*	Quitting and saving files is the most memory intensive part of this program.
	In addition to the resident segments, these segments are needed. */

resource 'res!' (256,
#if qNames
	"DrawShapes' resident segments",
#endif
	purgeable) {
	{	"ARes";
		"ShapeRes"
	};
};
resource 'seg!' (256,
#if qNames
	"DrawShapes' hi-water mark",
#endif
purgeable) {
	{	"ARes";
		"GNonRes";
		"GClose";
		"GDoCommand";
		"GFile";
		"GOpen";
		"GSelCommand";
		"GNonRes2";
		"GPrint";
		"GReadResource";
		"BBOpen";
		"ShapeRes"
	}
};

#if qPowerPC
#include "CodeFragmentTypes.r"

resource 'cfrg' (0, "Configuration Info") {
	{
		kPowerPC,
		kFullLib,
		kNoVersionNum,
		kNoVersionNum,
		0,
		0,
		kIsApp,
		kOnDiskFlat,
		kZeroOffset,
		kWholeFork,
		"DrawShapes"
	}
};
#endif

resource 'STR ' (kPickerPrompt) {
	"Pick a color, any color"
};

resource 'STR#' (1000) {
	{	/* array StringArray: 3 elements */
		/* [1] */
		"Abort",
		/* [2] */
		"Retry",
		/* [3] */
		"Ignore"
	}
};

resource 'STR#' (1001) {
	{	/* array StringArray: 1 elements */
		/* [1] */
		"An unexpected hack has ocurred. "
	}
};


/*--------------------------------------------------------------------------------
 Menus!
--------------------------------------------------------------------------------*/

include "Defaults.rsrc" 'CMNU' (mApple);	// Grab the default Apple/File menus
include "Defaults.rsrc" 'CMNU' (mFile);
include "Defaults.rsrc" 'CMNU' (mEdit);		// Grab the default Edit menu

resource 'CMNU' (mGoodies,
#if qNames
"mGoodies",
#endif
purgeable) {
	mGoodies,
	textMenuProc,
	EnablingManagedByMacApp,
	enabled,
	"Goodies",
	{
/* [1] */ "Show Page Breaks",
								noIcon, noKey, noMark, plain, cShowBreaks;
/* [2] */ "-",					noIcon, noKey,	noMark, plain, nocommand;
/* [3] */ "Better feedback desired",
								noIcon, noKey, noMark, plain, cBetterFeedback
	}
};

resource 'CMNU' (mTools,
#if qNames
"mTools",
#endif
purgeable) {
	mTools,
	textMenuProc,
	EnablingManagedByMacApp,
	enabled,
	"Tools",
	 {
/* [1] */	"Arrow",		noIcon, noKey, noMark, plain, cArrow;
/* [2] */	"Box",	noIcon, noKey, noMark, plain, cBox;
/* [3] */	"Circle",	noIcon, noKey, noMark, plain, cCircle;
/* [4] */	"Heavy Box",	noIcon, noKey, noMark, plain, cHBox
	 }
};

resource 'CMNU' (mPatterns,
#if qNames
"mPatterns",
#endif
purgeable) {
	mPatterns,
	textMenuProc,
	EnablingManagedByMacApp,
	enabled,
	"Patterns",
	{ "Patterns", noIcon, noKey, noMark, plain, cPatterns }
};

resource 'CMNU' (mColor,
#if qNames
"mColor",
#endif
purgeable) {
	mColor,
	textMenuProc,
	EnablingManagedByMacApp,
	enabled,
	"Colors",
	{	/* array: 9 elements */
/* [1] */	"Green",			noIcon, noKey, noMark, plain, noCommand;
/* [2] */	"Yellow",			noIcon, noKey, noMark, plain, noCommand;
/* [3] */	"Orange",			noIcon, noKey, noMark, plain, noCommand;
/* [4] */	"Red",				noIcon, noKey, noMark, plain, noCommand;
/* [5] */	"Purple",			noIcon, noKey, noMark, plain, noCommand;
/* [6] */	"Blue",				noIcon, noKey, noMark, plain, noCommand;
/* [7] */	"Black",			noIcon, noKey, noMark, plain, noCommand;
/* [8] */	"-",				noIcon, noKey, noMark, plain, noCommand;
/* [9] */	"Pick a Color",	noIcon, noKey, noMark, plain, cPickColor
	}
};

resource 'CMNU' (mMoreDebug,
#if qNames
"mMoreDebug",
#endif
purgeable) {
	mMoreDebug,
	textMenuProc,
	EnablingManagedByMacApp,
	enabled,
	"More Debug",
	 {	/* array: 3 elements */
/* [1] */ "Paste Replaces Selection",
								noIcon, noKey, noMark, plain, cPasteReplacesSelection;
/* [2] */ "Recompute view extent",
								noIcon, noKey, noMark, plain, cRecalcExtent;
/* [3] */ "Constrain dragging to view borders",
								noIcon, noKey, noMark, plain, cConstrainDrags
	}
};

resource 'mctb' (mColor) {
	{
	/* Green */
	mColor, 1,
		{
		0x0000, 0xDB00, 0x0000;			/* Mark color */
		0x0000, 0xDB00, 0x0000;			/* Name color */
		0x0000, 0xDB00, 0x0000;			/* Command Key color */
		0xFFFF, 0xFFFF, 0xFFFF			/* Background color */
		};
	/* Yellow */
	mColor, 2,
		{
		0xFFFF, 0xDB00, 0x0000;			/* Mark color */
		0xFFFF, 0xDB00, 0x0000;			/* Name color */
		0xFFFF, 0xDB00, 0x0000;			/* Command Key color */
		0xFFFF, 0xFFFF, 0xFFFF			/* Background color */
		};
	/* Orange */
	mColor, 3,
		{
		0xFFFF, 0x4700, 0x0000;			/* Mark color */
		0xFFFF, 0x4700, 0x0000;			/* Name color */
		0xFFFF, 0x4700, 0x0000;			/* Command Key color */
		0xFFFF, 0xFFFF, 0xFFFF			/* Background color */
		};
	/* Red */
	mColor, 4,
		{
		0xDB00, 0x0000, 0x0000;			/* Mark color */
		0xDB00, 0x0000, 0x0000;			/* Name color */
		0xDB00, 0x0000, 0x0000;			/* Command Key color */
		0xFFFF, 0xFFFF, 0xFFFF			/* Background color */
		};
	/* Purple */
	mColor, 5,
		{
		0x9200, 0x0000, 0xB000;			/* Mark color */
		0x9200, 0x0000, 0xB000;			/* Name color */
		0x9200, 0x0000, 0xB000;			/* Command Key color */
		0xFFFF, 0xFFFF, 0xFFFF			/* Background color */
		};
	/* Blue */
	mColor, 6,
		{
		0x0000, 0x0000, 0xFFFF;			/* Mark color */
		0x0000, 0x0000, 0xFFFF;			/* Name color */
		0x0000, 0x0000, 0xFFFF;			/* Command Key color */
		0xFFFF, 0xFFFF, 0xFFFF			/* Background color */
		};
	/* Black */
	mColor, 7,
		{
		0x0000, 0x0000, 0x0000;			/* Mark color */
		0x0000, 0x0000, 0x0000;			/* Name color */
		0x0000, 0x0000, 0x0000;			/* Command Key color */
		0xFFFF, 0xFFFF, 0xFFFF			/* Background color */
		}
	}
};

resource 'CMNU' (mBuzzwords,
#if qNames
"Buzzwords",
#endif
purgeable
) {
	mBuzzwords,
	textMenuProc,
	EnablingManagedByMacApp,
	enabled,
	"Buzzwords",
	 {	/* array: 5 elements */
/* [1] */	"Drawing",				noIcon, noKey, noMark, plain, cNewShape;
/* [2] */	"Move",					noIcon, noKey, noMark, plain, cMoveShape;
/* [3] */	"Page Setup Change",	noIcon, noKey, noMark, plain, cChangePrinterStyle;
/* [4] */	"Shade Change",			noIcon, noKey, noMark, plain, cChangeShade;
/* [5] */	"Color Change",			noIcon, noKey, noMark, plain, cChangeColor;
/* [6] */	"Tool Change",			noIcon, noKey, noMark, plain, cChangeTool
	}
};

resource 'CMNU' (mHack, purgeable) {
	mHack,
	textMenuProc,
	0x10,
	enabled,
	"Hack",
	{	/* array: 3 elements */
		/* [1] */
		"Abort", noIcon, noKey, noMark, plain, 1901,
		/* [2] */
		"Retry", noIcon, noKey, noMark, plain, 1902,
		/* [3] */
		"Ignore", noIcon, noKey, noMark, plain, 1903
	}
};


/* Menu bar for a color system */
resource 'MBAR' (kColorMenuBar,
#if qNames
"kColorMenuBar",
#endif
nonpurgeable) {
	{mApple; mFile; mEdit; mTools; mPatterns; mColor; mGoodies; mHack}
};

/* Menu bar for a non-color system */
resource 'MBAR' (kNonColorMenuBar,
#if qNames
"kNonColorMenuBar",
#endif
nonpurgeable) {
	{mApple; mFile; mEdit; mTools; mPatterns; mGoodies; mHack}
};

resource 'MBAR' (kMBarNotDisplayed,
#if qNames
"kMBarNotDisplayed",
#endif
nonpurgeable) {
	{mMoreDebug}
};

//--------------------------------------------------------------------------------

/* The color "rainbow" arrow cursor */
data 'crsr' (kRainbowArrow, "Rainbow Arrow") {
	$"8001 0000 0062 0000 0096 0000 0000 0000"        /* ....b......... */
	$"0000 0000 0000 4000 6000 7000 7800 7C00"        /* ......@.`.p.x.|. */
	$"7E00 7F00 7F80 7C00 6C00 4600 0600 0300"        /* ~....|.l.F..... */
	$"0100 0000 C000 E000 F000 F800 FC00 FE00"        /* ............... */
	$"FF00 FF80 FFC0 FFE0 FE00 EF00 CF00 8780"        /* ........... */
	$"0780 0380 0001 0001 0000 0000 0000 0000"        /* .............. */
	$"0000 0000 0000 8008 0000 0000 0010 0010"        /* ............... */
	$"0000 0000 0000 0000 0048 0000 0048 0000"        /* .........H...H.. */
	$"0000 0004 0000 0000 0000 0000 0000 0116"        /* ................ */
	$"0000 0000 0000 0000 0000 0000 0000 0100"        /* ................ */
	$"0000 0000 0000 0110 0000 0000 0000 0121"        /* ...............! */
	$"0000 0000 0000 0222 2000 0000 0000 0233"        /* ......." ......3 */
	$"3200 0000 0000 0333 3330 0000 0000 0344"        /* 2......330.....D */
	$"4445 0000 0000 0444 4445 5000 0000 0444"        /* DE.....DDEP....D */
	$"5500 0000 0000 0550 6600 0000 0000 0500"        /* U......Pf....... */
	$"0660 0000 0000 0000 0660 0000 0000 0000"        /* .`.......`...... */
	$"0066 0000 0000 0000 0066 0000 0000 0000"        /* .f.......f...... */
	$"0000 0000 0000 0000 0000 0000 0006 0000"        /* ................ */
	$"FFFF FFFF FFFF 0001 0000 DB00 0000 0002"        /* ................ */
	$"FFFF DB00 0000 0003 FFFF 4F00 0000 0004"        /* ..........O..... */
	$"DB00 0000 0000 0005 9200 0000 B000 0006"        /* .............. */
	$"0000 0000 FFFF"                                 /* ...... */
};

include "Defaults.rsrc"  'STR#' (kDefaultCredits);		// Grab the default credits

resource 'ICN#' (128, "DrawShapes Family", purgeable) {
	{	/* array: 2 elements */
		/* [1] */
		$"0000 0000 007F 00FE 0081 0102 0080 8203"
		$"0100 8403 0100 4803 0200 5003 0200 2003"
		$"0400 0003 0400 0003 0800 0003 0800 0003"
		$"1000 0003 1000 0003 2000 0003 23FF FE03"
		$"4200 0303 4200 0303 8200 0303 FA01 FF3F"
		$"7A06 019F 7A18 006F 03F0 0020 01E0 0010"
		$"0020 0018 0020 0018 0030 0038 0018 0070"
		$"000E 01E0 0007 FF80 0001 FE",
		/* [2] */
		$"0000 0000 00FF 01FE 00FF 83FE 01FF 87FF"
		$"01FF CFFF 03FF DFFF 03FF FFFF 07FF FFFF"
		$"07FF FFFF 0FFF FFFF 0FFF FFFF 1FFF FFFF"
		$"1FFF FFFF 3FFF FFFF 3FFF FFFF 7FFF FFFF"
		$"7FFF FFFF FFFF FFFF FFFF FFFF FFFF FFFF"
		$"7FFF FFFF 7FFF FFFF 03FF FFE0 01FF FFF8"
		$"003F FFF8 003F FFF8 003F FFF8 001F FFF8"
		$"001F FFF0 000F FFE0 0007 FF"
	}
};

resource 'ICN#' (129, "DrawShapes Document", purgeable) {
	{	/* array: 2 elements */
		/* [1] */
		$"1FFF FC00 1000 0600 1000 0500 1000 0480"
		$"1000 0440 1000 0420 1000 07F0 1000 0010"
		$"1000 0010 1000 0010 1000 0010 13FF F010"
		$"1200 1810 1200 1810 1200 1810 1201 F810"
		$"1206 0C10 1218 0310 13F0 0110 11E0 0090"
		$"1020 00D0 1020 00D0 1030 01D0 1018 0390"
		$"100E 0F10 1007 FC10 1001 F010 1000 0010"
		$"1000 0010 1000 0010 1000 0010 1FFF FFF0",
		/* [2] */
		$"1FFF FC00 1FFF FE00 1FFF FF00 1FFF FF80"
		$"1FFF FFC0 1FFF FFE0 1FFF FFF0 1FFF FFF0"
		$"1FFF FFF0 1FFF FFF0 1FFF FFF0 1FFF FFF0"
		$"1FFF FFF0 1FFF FFF0 1FFF FFF0 1FFF FFF0"
		$"1FFF FFF0 1FFF FFF0 1FFF FFF0 1FFF FFF0"
		$"1FFF FFF0 1FFF FFF0 1FFF FFF0 1FFF FFF0"
		$"1FFF FFF0 1FFF FFF0 1FFF FFF0 1FFF FFF0"
		$"1FFF FFF0 1FFF FFF0 1FFF FFF0 1FFF FFF0"
	}
};

resource 'ICN#' (130, "DrawShapes Stationery", purgeable) {
	{	/* array: 2 elements */
		/* [1] */
		$"3FFF FFE0 2000 0020 2000 0038 2000 0028"
		$"2000 0028 2000 0028 27FF E028 2400 3028"
		$"2400 3028 2400 3028 2403 F028 240C 1828"
		$"2430 0628 27E0 0228 23C0 0128 2040 01A8"
		$"2040 01A8 2060 03A8 2030 0728 201C 1E28"
		$"200F F828 2003 E028 2000 0028 2000 0FE8"
		$"2000 0848 2000 0888 2000 0908 2000 0A08"
		$"2000 0C08 3FFF F808 0800 0008 0FFF FFF8",
		/* [2] */
		$"3FFF FFE0 3FFF FFE0 3FFF FFF8 3FFF FFF8"
		$"3FFF FFF8 3FFF FFF8 3FFF FFF8 3FFF FFF8"
		$"3FFF FFF8 3FFF FFF8 3FFF FFF8 3FFF FFF8"
		$"3FFF FFF8 3FFF FFF8 3FFF FFF8 3FFF FFF8"
		$"3FFF FFF8 3FFF FFF8 3FFF FFF8 3FFF FFF8"
		$"3FFF FFF8 3FFF FFF8 3FFF FFF8 3FFF FFF8"
		$"3FFF FFF8 3FFF FFF8 3FFF FFF8 3FFF FFF8"
		$"3FFF FFF8 3FFF FFF8 0FFF FFF8 0FFF FFF8"
	}
};

resource 'ics#' (128, "DrawShapes Family", purgeable) {
	{	/* array: 2 elements */
		/* [1] */
		$"0707 0489 0851 0821 1001 1001 2001 2001"
		$"5FF1 5019 D1FB 5E05 0C02 0606 03FE 01FC",
		/* [2] */
		$"078F 0FDF 0FFF 1FFF 1FFF 3FFF 3FFF 7FFF"
		$"7FFF FFFF FFFF 7FFF 0FFE 07FE 03FE 03FC"
	}
};

resource 'ics#' (129, "DrawShapes Document", purgeable) {
	{	/* array: 2 elements */
		/* [1] */
		$"7FE0 4030 4028 403C 4004 5F84 50C4 51C4"
		$"5E24 4C14 4634 43F4 41E4 4004 4004 7FFC",
		/* [2] */
		$"7FE0 7FF0 7FF8 7FFC 7FFC 7FFC 7FFC 7FFC"
		$"7FFC 7FFC 7FFC 7FFC 7FFC 7FFC 7FFC 7FFC"
	}
};

resource 'ics#' (130, "DrawShapes Stationery", purgeable) {
	{	/* array: 2 elements */
		/* [1] */
		$"7FF8 4008 5F8E 50CA 51CA 5E2A 4C1A 463A"
		$"43FA 41EA 407A 4052 4062 7FC2 1002 1FFE",
		/* [2] */
		$"7FF8 7FF8 7FFE 7FFE 7FFE 7FFE 7FFE 7FFE"
		$"7FFE 7FFE 7FFE 7FFE 7FFE 7FFE 1FFE 1FFE"
	}
};

resource 'ics4' (128, "DrawShapes Family", purgeable) {
	$"0000 0666 7000 7666 0000 7677 6707 6776"
	$"0000 6777 7676 7776 0007 6777 7767 7776"
	$"0006 7777 7777 7776 0076 7777 7777 7776"
	$"0067 7777 7777 7776 0767 7777 7777 7776"
	$"067F FFFF FFFF 7776 767F CCCC CCCF F776"
	$"666F CCCF FFFF F666 0DDF FFF3 3333 3FDD"
	$"0000 FF33 3333 33F0 0000 0FF3 3333 3FF0"
	$"0000 00FF FFFF FFE0 0000 00CD EFFF ED"
};

resource 'ics4' (129, "DrawShapes Document", purgeable) {
	$"0FFF FFFF FFF0 0000 0F0C 0C0C 0CFF 0000"
	$"0FC0 C0C0 C0FC F000 0F0C 0C0C 0CFF FF00"
	$"0FC0 C0C0 C0C0 CF00 0F0F FFFF FC0C 0F00"
	$"0FCF CCCC FFC0 CF00 0F0F CCCF FF0C 0F00"
	$"0FCF FFF3 33F0 CF00 0F0C FF33 333F 0F00"
	$"0FC0 CFF3 33FF CF00 0F0C 0CFF FFFF 0F00"
	$"0FC0 C0CF FFF0 CF00 0F0C 0C0C 0C0C 0F00"
	$"0FC0 C0C0 C0C0 CF00 0FFF FFFF FFFF FF"
};

resource 'ics4' (130, "DrawShapes Stationery", purgeable) {
	$"0FFF FFFF FFFF F000 0F0C 0C0C 0C0C F000"
	$"0FCF FFFF F0C0 FFF0 0F0F CCCC FF0C FDF0"
	$"0FCF CCCF FFC0 FDF0 0F0F FFF3 33FC FDF0"
	$"0FC0 FF33 333F FDF0 0F0C 0FF3 33FF FDF0"
	$"0FC0 C0FF FFFF FDF0 0F0C 0C0F FFF0 FDF0"
	$"0FC0 C0C0 CFFF FDF0 0F0C 0C0C 0FCF DDF0"
	$"0FC0 C0C0 CFFD D0F0 0FFF FFFF FFDD 0CF0"
	$"000F DDDD DDD0 C0F0 000F FFFF FFFF FFF0"
};

resource 'ics8' (128, "DrawShapes Family", purgeable) {
	$"0000 0000 00EC ECEC C000 0000 C0EC ECEC"
	$"0000 0000 C0EC C0C0 ECC0 00C0 ECC0 C0EC"
	$"0000 0000 ECC0 C0C0 C0EC C0EC C0C0 C0EC"
	$"0000 00C0 ECC0 C0C0 C0C0 ECC0 C0C0 C0EC"
	$"0000 00EC C0C0 C0C0 C0C0 C0C0 C0C0 C0EC"
	$"0000 C0EC C0C0 C0C0 C0C0 C0C0 C0C0 C0EC"
	$"0000 ECC0 C0C0 C0C0 C0C0 C0C0 C0C0 C0EC"
	$"00C0 ECC0 C0C0 C0C0 C0C0 C0C0 C0C0 C0EC"
	$"00EC C0FF FFFF FFFF FFFF FFFF C0C0 C0EC"
	$"C0EC C0FF 3030 3030 3030 30FF FFC0 C0EC"
	$"ECEC ECFF 3030 30FF FFFF FFFF FFEC ECEC"
	$"00FA FAFF FFFF FFD8 D8D8 D8D8 D8FF FAFA"
	$"0000 0000 FFFF D8D8 D8D8 D8D8 D8D8 FF00"
	$"0000 0000 00FF FFD8 D8D8 D8D8 D8FF FF00"
	$"0000 0000 0000 FFFF FFFF FFFF FFFF FC00"
	$"0000 0000 0000 F6FA FCFF FFFF FCFA"
};

resource 'ics8' (129, "DrawShapes Document", purgeable) {
	$"00FF FFFF FFFF FFFF FFFF FF00 0000 0000"
	$"00FF F5F5 F5F5 F5F5 F5F5 FFFF 0000 0000"
	$"00FF F5F5 F5F5 F5F5 F5F5 FF2B FF00 0000"
	$"00FF F5F5 F5F5 F5F5 F5F5 FFFF FFFF 0000"
	$"00FF F5F5 F5F5 F5F5 F5F5 F5F5 F5FF 0000"
	$"00FF F5FF FFFF FFFF FFF5 F5F5 F5FF 0000"
	$"00FF F5FF 2A2A 2A2A FFFF F5F5 F5FF 0000"
	$"00FF F5FF 2A2A 2AFF FFFF F5F5 F5FF 0000"
	$"00FF F5FF FFFF FFD8 D8D8 FFF5 F5FF 0000"
	$"00FF F5F5 FFFF D8D8 D8D8 D8FF F5FF 0000"
	$"00FF F5F5 F5FF FFD8 D8D8 FFFF F5FF 0000"
	$"00FF F5F5 F5F5 FFFF FFFF FFFF F5FF 0000"
	$"00FF F5F5 F5F5 F5FF FFFF FFF5 F5FF 0000"
	$"00FF F5F5 F5F5 F5F5 F5F5 F5F5 F5FF 0000"
	$"00FF F5F5 F5F5 F5F5 F5F5 F5F5 F5FF 0000"
	$"00FF FFFF FFFF FFFF FFFF FFFF FFFF"
};

resource 'ics8' (130, "DrawShapes Stationery", purgeable) {
	$"00FF FFFF FFFF FFFF FFFF FFFF FF00 0000"
	$"00FF F5F5 F5F5 F5F5 F5F5 F5F5 FF00 0000"
	$"00FF F5FF FFFF FFFF FFF5 F5F5 FFFF FF00"
	$"00FF F5FF 2A2A 2A2A FFFF F5F5 FFF9 FF00"
	$"00FF F5FF 2A2A 2AFF FFFF F5F5 FFF9 FF00"
	$"00FF F5FF FFFF FFD8 D8D8 FFF5 FFF9 FF00"
	$"00FF F5F5 FFFF D8D8 D8D8 D8FF FFF9 FF00"
	$"00FF F5F5 F5FF FFD8 D8D8 FFFF FFF9 FF00"
	$"00FF F5F5 F5F5 FFFF FFFF FFFF FFF9 FF00"
	$"00FF F5F5 F5F5 F5FF FFFF FFF5 FFF9 FF00"
	$"00FF F5F5 F5F5 F5F5 F5FF FFFF FFF9 FF00"
	$"00FF F5F5 F5F5 F5F5 F5FF 2BFF F9F9 FF00"
	$"00FF F5F5 F5F5 F5F5 F5FF FFF9 F9F5 FF00"
	$"00FF FFFF FFFF FFFF FFFF F9F9 F5F5 FF00"
	$"0000 00FF F9F9 F9F9 F9F9 F9F5 F5F5 FF00"
	$"0000 00FF FFFF FFFF FFFF FFFF FFFF FF"
};

resource 'icl4' (128, "DrawShapes Family", purgeable) {
	$"0000 0000 0000 0000 0000 0000 0000 0000"
	$"0000 0000 7666 6666 0000 0007 6666 6660"
	$"0000 0000 6777 7776 7000 0076 7777 7760"
	$"0000 0007 6777 7777 6000 0767 7777 776D"
	$"0000 0006 7777 7777 6700 7677 7777 776D"
	$"0000 0076 7777 7777 7607 6777 7777 776D"
	$"0000 0067 7777 7777 7676 7777 7777 776D"
	$"0000 0767 7777 7777 7767 7777 7777 776D"
	$"0000 0677 7777 7777 7777 7777 7777 776D"
	$"0000 7677 7777 7777 7777 7777 7777 776D"
	$"0000 6777 7777 7777 7777 7777 7777 776D"
	$"0007 6777 7777 7777 7777 7777 7777 776D"
	$"0006 7777 7777 7777 7777 7777 7777 776D"
	$"0076 7777 7777 7777 7777 7777 7777 776D"
	$"0067 7777 7777 7777 7777 7777 7777 776D"
	$"0767 77FF FFFF FFFF FFFF FFF7 7777 776D"
	$"0677 77FC CCCC CCCC CCCC CCFF 7777 776D"
	$"7677 77FC CCCC CCCC CCCC CCFF 7777 776D"
	$"6777 77FC CCCC CCCC CCCC CCFF 7777 776D"
	$"6666 66FC CCCC CCCF FFFF FFFF 6666 666D"
	$"0DDD DDFC CCCC CFF3 3333 333F FDDD DDDD"
	$"0DDD DDFC CCCF F333 3333 3333 3FDD DDDD"
	$"0000 00FF FFFF 3333 3333 3333 33F0 0000"
	$"0000 000F FFF3 3333 3333 3333 333F C000"
	$"0000 0000 00F3 3333 3333 3333 333F F000"
	$"0000 0000 00F3 3333 3333 3333 333F F000"
	$"0000 0000 00DF 3333 3333 3333 33FF F000"
	$"0000 0000 000F F333 3333 3333 3FFF C000"
	$"0000 0000 000C FFF3 3333 333F FFFC 0000"
	$"0000 0000 0000 CDFF FFFF FFFF FCC0 0000"
	$"0000 0000 0000 0CCD FFFF FFDC"
};

resource 'icl4' (129, "DrawShapes Document", purgeable) {
	$"000F FFFF FFFF FFFF FFFF FF00 0000 0000"
	$"000F 0C0C 0C0C 0C0C 0C0C 0FF0 0000 0000"
	$"000F C0C0 C0C0 C0C0 C0C0 CFCF 0000 0000"
	$"000F 0C0C 0C0C 0C0C 0C0C 0FCC F000 0000"
	$"000F C0C0 C0C0 C0C0 C0C0 CFCC CF00 0000"
	$"000F 0C0C 0C0C 0C0C 0C0C 0FCC CCF0 0000"
	$"000F C0C0 C0C0 C0C0 C0C0 CFFF FFFF 0000"
	$"000F 0C0C 0C0C 0C0C 0C0C 0C0C 0C0F 0000"
	$"000F C0C0 C0C0 C0C0 C0C0 C0C0 C0CF 0000"
	$"000F 0C0C 0C0C 0C0C 0C0C 0C0C 0C0F 0000"
	$"000F C0C0 C0C0 C0C0 C0C0 C0C0 C0CF 0000"
	$"000F 0CFF FFFF FFFF FFFF 0C0C 0C0F 0000"
	$"000F C0FC CCCC CCCC CCCF F0C0 C0CF 0000"
	$"000F 0CFC CCCC CCCC CCCF FC0C 0C0F 0000"
	$"000F C0FC CCCC CCCC CCCF F0C0 C0CF 0000"
	$"000F 0CFC CCCC CCCF FFFF FC0C 0C0F 0000"
	$"000F C0FC CCCC CFF3 3333 FFC0 C0CF 0000"
	$"000F 0CFC CCCF F333 3333 33FF 0C0F 0000"
	$"000F C0FF FFFF 3333 3333 333F C0CF 0000"
	$"000F 0C0F FFF3 3333 3333 3333 FC0F 0000"
	$"000F C0C0 CF33 3333 3333 3333 3FCF 0000"
	$"000F 0C0C 0F33 3333 3333 3333 3F0F 0000"
	$"000F C0C0 CDF3 3333 3333 3333 FFCF 0000"
	$"000F 0C0C 0CFF 3333 3333 333F FF0F 0000"
	$"000F C0C0 C0CF FF33 3333 3FFF FCCF 0000"
	$"000F 0C0C 0C0C DFFF FFFF FFFC CC0F 0000"
	$"000F C0C0 C0C0 CCDF FFFF DCC0 C0CF 0000"
	$"000F 0C0C 0C0C 0C0C 0C0C 0C0C 0C0F 0000"
	$"000F C0C0 C0C0 C0C0 C0C0 C0C0 C0CF 0000"
	$"000F 0C0C 0C0C 0C0C 0C0C 0C0C 0C0F 0000"
	$"000F C0C0 C0C0 C0C0 C0C0 C0C0 C0CF 0000"
	$"000F FFFF FFFF FFFF FFFF FFFF FFFF"
};

resource 'icl4' (130, "DrawShapes Stationery", purgeable) {
	$"00FF FFFF FFFF FFFF FFFF FFFF FFF0 0000"
	$"00FC 0C0C 0C0C 0C0C 0C0C 0C0C 0CF0 0000"
	$"00F0 C0C0 C0C0 C0C0 C0C0 C0C0 C0FF F000"
	$"00FC 0C0C 0C0C 0C0C 0C0C 0C0C 0CFD F000"
	$"00F0 C0C0 C0C0 C0C0 C0C0 C0C0 C0FD F000"
	$"00FC 0C0C 0C0C 0C0C 0C0C 0C0C 0CFD F000"
	$"00F0 CFFF FFFF FFFF FFF0 C0C0 C0FD F000"
	$"00FC 0FCC CCCC CCCC CCFF 0C0C 0CFD F000"
	$"00F0 CFCC CCCC CCCC CCFF C0C0 C0FD F000"
	$"00FC 0FCC CCCC CCCC CCFF 0C0C 0CFD F000"
	$"00F0 CFCC CCCC CCFF FFFF C0C0 C0FD F000"
	$"00FC 0FCC CCCC FF33 333F FC0C 0CFD F000"
	$"00F0 CFCC CCFF 3333 3333 3FF0 C0FD F000"
	$"00FC 0FFF FFF3 3333 3333 33FC 0CFD F000"
	$"00F0 C0FF FF33 3333 3333 333F C0FD F000"
	$"00FC 0C0C F333 3333 3333 3333 FCFD F000"
	$"00F0 C0C0 F333 3333 3333 3333 F0FD F000"
	$"00FC 0C0C DF33 3333 3333 333F FCFD F000"
	$"00F0 C0C0 CFF3 3333 3333 33FF F0FD F000"
	$"00FC 0C0C 0CFF F333 3333 FFFF CCFD F000"
	$"00F0 C0C0 C0CD FFFF FFFF FFCC C0FD F000"
	$"00FC 0C0C 0C0C CDFF FFFD CC0C 0CFD F000"
	$"00F0 C0C0 C0C0 C0C0 C0C0 C0C0 C0FD F000"
	$"00FC 0C0C 0C0C 0C0C 0C0C FFFF FFFD F000"
	$"00F0 C0C0 C0C0 C0C0 C0C0 FCCC CFDC F000"
	$"00FC 0C0C 0C0C 0C0C 0C0C FCCC FDCC F000"
	$"00F0 C0C0 C0C0 C0C0 C0C0 FCCF DCCC F000"
	$"00FC 0C0C 0C0C 0C0C 0C0C FCFD CCCC F000"
	$"00F0 C0C0 C0C0 C0C0 C0C0 FFDC CCCC F000"
	$"00FF FFFF FFFF FFFF FFFF FDCC CCCC F000"
	$"0000 FDDD DDDD DDDD DDDD DCCC CCCC F000"
	$"0000 FFFF FFFF FFFF FFFF FFFF FFFF F0"
};

resource 'icl8' (128, "DrawShapes Family", purgeable) {
	$"0000 0000 0000 0000 0000 0000 0000 0000"
	$"0000 0000 0000 0000 0000 0000 0000 0000"
	$"0000 0000 0000 0000 C0EC ECEC ECEC ECEC"
	$"0000 0000 0000 00C0 ECEC ECEC ECEC EC00"
	$"0000 0000 0000 0000 ECC0 C0C0 C0C0 C0EC"
	$"C000 0000 0000 C0EC C0C0 C0C0 C0C0 EC00"
	$"0000 0000 0000 00C0 ECC0 C0C0 C0C0 C0C0"
	$"EC00 0000 00C0 ECC0 C0C0 C0C0 C0C0 ECFA"
	$"0000 0000 0000 00EC C0C0 C0C0 C0C0 C0C0"
	$"ECC0 0000 C0EC C0C0 C0C0 C0C0 C0C0 ECFA"
	$"0000 0000 0000 C0EC C0C0 C0C0 C0C0 C0C0"
	$"C0EC 00C0 ECC0 C0C0 C0C0 C0C0 C0C0 ECFA"
	$"0000 0000 0000 ECC0 C0C0 C0C0 C0C0 C0C0"
	$"C0EC C0EC C0C0 C0C0 C0C0 C0C0 C0C0 ECFA"
	$"0000 0000 00C0 ECC0 C0C0 C0C0 C0C0 C0C0"
	$"C0C0 ECC0 C0C0 C0C0 C0C0 C0C0 C0C0 ECFA"
	$"0000 0000 00EC C0C0 C0C0 C0C0 C0C0 C0C0"
	$"C0C0 C0C0 C0C0 C0C0 C0C0 C0C0 C0C0 ECFA"
	$"0000 0000 C0EC C0C0 C0C0 C0C0 C0C0 C0C0"
	$"C0C0 C0C0 C0C0 C0C0 C0C0 C0C0 C0C0 ECFA"
	$"0000 0000 ECC0 C0C0 C0C0 C0C0 C0C0 C0C0"
	$"C0C0 C0C0 C0C0 C0C0 C0C0 C0C0 C0C0 ECFA"
	$"0000 00C0 ECC0 C0C0 C0C0 C0C0 C0C0 C0C0"
	$"C0C0 C0C0 C0C0 C0C0 C0C0 C0C0 C0C0 ECFA"
	$"0000 00EC C0C0 C0C0 C0C0 C0C0 C0C0 C0C0"
	$"C0C0 C0C0 C0C0 C0C0 C0C0 C0C0 C0C0 ECFA"
	$"0000 C0EC C0C0 C0C0 C0C0 C0C0 C0C0 C0C0"
	$"C0C0 C0C0 C0C0 C0C0 C0C0 C0C0 C0C0 ECFA"
	$"0000 ECC0 C0C0 C0C0 C0C0 C0C0 C0C0 C0C0"
	$"C0C0 C0C0 C0C0 C0C0 C0C0 C0C0 C0C0 ECFA"
	$"00C0 ECC0 C0C0 FFFF FFFF FFFF FFFF FFFF"
	$"FFFF FFFF FFFF FFC0 C0C0 C0C0 C0C0 ECFA"
	$"00EC C0C0 C0C0 FF30 3030 3030 3030 3030"
	$"3030 3030 3030 FFFF C0C0 C0C0 C0C0 ECFA"
	$"C0EC C0C0 C0C0 FF30 3030 3030 3030 3030"
	$"3030 3030 3030 FFFF C0C0 C0C0 C0C0 ECFA"
	$"ECC0 C0C0 C0C0 FF30 3030 3030 3030 3030"
	$"3030 3030 3030 FFFF C0C0 C0C0 C0C0 ECFA"
	$"ECEC ECEC ECEC FF30 3030 3030 3030 30FF"
	$"FFFF FFFF FFFF FFFF ECEC ECEC ECEC ECFA"
	$"00FA FAFA FAFA FF30 3030 3030 30FF FFD8"
	$"D8D8 D8D8 D8D8 D8FF FFFA FAFA FAFA FAFA"
	$"00FA FAFA FAFA FF30 3030 30FF FFD8 D8D8"
	$"D8D8 D8D8 D8D8 D8D8 D8FF FAFA FAFA FAFA"
	$"0000 0000 0000 FFFF FFFF FFFF D8D8 D8D8"
	$"D8D8 D8D8 D8D8 D8D8 D8D8 FF00 0000 0000"
	$"0000 0000 0000 00FF FFFF FFD8 D8D8 D8D8"
	$"D8D8 D8D8 D8D8 D8D8 D8D8 D8FF F600 0000"
	$"0000 0000 0000 0000 0000 FFD8 D8D8 D8D8"
	$"D8D8 D8D8 D8D8 D8D8 D8D8 D8FF FF00 0000"
	$"0000 0000 0000 0000 0000 FFD8 D8D8 D8D8"
	$"D8D8 D8D8 D8D8 D8D8 D8D8 D8FF FF00 0000"
	$"0000 0000 0000 0000 0000 FAFF D8D8 D8D8"
	$"D8D8 D8D8 D8D8 D8D8 D8D8 FFFF FF00 0000"
	$"0000 0000 0000 0000 0000 00FF FFD8 D8D8"
	$"D8D8 D8D8 D8D8 D8D8 D8FF FFFF F600 0000"
	$"0000 0000 0000 0000 0000 00F6 FFFF FFD8"
	$"D8D8 D8D8 D8D8 D8FF FFFF FFF6 0000 0000"
	$"0000 0000 0000 0000 0000 0000 F6FA FFFF"
	$"FFFF FFFF FFFF FFFF FFF7 F600 0000 0000"
	$"0000 0000 0000 0000 0000 0000 00F6 F7FA"
	$"FFFF FFFF FFFF FAF7"
};

resource 'icl8' (129, "DrawShapes Document", purgeable) {
	$"0000 00FF FFFF FFFF FFFF FFFF FFFF FFFF"
	$"FFFF FFFF FFFF 0000 0000 0000 0000 0000"
	$"0000 00FF F5F5 F5F5 F5F5 F5F5 F5F5 F5F5"
	$"F5F5 F5F5 F5FF FF00 0000 0000 0000 0000"
	$"0000 00FF F5F5 F5F5 F5F5 F5F5 F5F5 F5F5"
	$"F5F5 F5F5 F5FF 2BFF 0000 0000 0000 0000"
	$"0000 00FF F5F5 F5F5 F5F5 F5F5 F5F5 F5F5"
	$"F5F5 F5F5 F5FF 2B2B FF00 0000 0000 0000"
	$"0000 00FF F5F5 F5F5 F5F5 F5F5 F5F5 F5F5"
	$"F5F5 F5F5 F5FF 2B2B 2BFF 0000 0000 0000"
	$"0000 00FF F5F5 F5F5 F5F5 F5F5 F5F5 F5F5"
	$"F5F5 F5F5 F5FF 2B2B 2B2B FF00 0000 0000"
	$"0000 00FF F5F5 F5F5 F5F5 F5F5 F5F5 F5F5"
	$"F5F5 F5F5 F5FF FFFF FFFF FFFF 0000 0000"
	$"0000 00FF F5F5 F5F5 F5F5 F5F5 F5F5 F5F5"
	$"F5F5 F5F5 F5F5 F5F5 F5F5 F5FF 0000 0000"
	$"0000 00FF F5F5 F5F5 F5F5 F5F5 F5F5 F5F5"
	$"F5F5 F5F5 F5F5 F5F5 F5F5 F5FF 0000 0000"
	$"0000 00FF F5F5 F5F5 F5F5 F5F5 F5F5 F5F5"
	$"F5F5 F5F5 F5F5 F5F5 F5F5 F5FF 0000 0000"
	$"0000 00FF F5F5 F5F5 F5F5 F5F5 F5F5 F5F5"
	$"F5F5 F5F5 F5F5 F5F5 F5F5 F5FF 0000 0000"
	$"0000 00FF F5F5 FFFF FFFF FFFF FFFF FFFF"
	$"FFFF FFFF F5F5 F5F5 F5F5 F5FF 0000 0000"
	$"0000 00FF F5F5 FF2A 2A2A 2A2A 2A2A 2A2A"
	$"2A2A 2AFF FFF5 F5F5 F5F5 F5FF 0000 0000"
	$"0000 00FF F5F5 FF2A 2A2A 2A2A 2A2A 2A2A"
	$"2A2A 2AFF FFF5 F5F5 F5F5 F5FF 0000 0000"
	$"0000 00FF F5F5 FF2A 2A2A 2A2A 2A2A 2A2A"
	$"2A2A 2AFF FFF5 F5F5 F5F5 F5FF 0000 0000"
	$"0000 00FF F5F5 FF2A 2A2A 2A2A 2A2A 2AFF"
	$"FFFF FFFF FFF5 F5F5 F5F5 F5FF 0000 0000"
	$"0000 00FF F5F5 FF2A 2A2A 2A2A 2AFF FFD8"
	$"D8D8 D8D8 FFFF F5F5 F5F5 F5FF 0000 0000"
	$"0000 00FF F5F5 FF2A 2A2A 2AFF FFD8 D8D8"
	$"D8D8 D8D8 D8D8 FFFF F5F5 F5FF 0000 0000"
	$"0000 00FF F5F5 FFFF FFFF FFFF D8D8 D8D8"
	$"D8D8 D8D8 D8D8 D8FF F5F5 F5FF 0000 0000"
	$"0000 00FF F5F5 F5FF FFFF FFD8 D8D8 D8D8"
	$"D8D8 D8D8 D8D8 D8D8 FFF5 F5FF 0000 0000"
	$"0000 00FF F5F5 F5F5 F5FF D8D8 D8D8 D8D8"
	$"D8D8 D8D8 D8D8 D8D8 D8FF F5FF 0000 0000"
	$"0000 00FF F5F5 F5F5 F5FF D8D8 D8D8 D8D8"
	$"D8D8 D8D8 D8D8 D8D8 D8FF F5FF 0000 0000"
	$"0000 00FF F5F5 F5F5 F5FA FFD8 D8D8 D8D8"
	$"D8D8 D8D8 D8D8 D8D8 FFFF F5FF 0000 0000"
	$"0000 00FF F5F5 F5F5 F5F5 FFFF D8D8 D8D8"
	$"D8D8 D8D8 D8D8 D8FF FFFF F5FF 0000 0000"
	$"0000 00FF F5F5 F5F5 F5F5 F6FF FFFF D8D8"
	$"D8D8 D8D8 D8FF FFFF FFF6 F5FF 0000 0000"
	$"0000 00FF F5F5 F5F5 F5F5 F5F6 FAFF FFFF"
	$"FFFF FFFF FFFF FFF7 F6F5 F5FF 0000 0000"
	$"0000 00FF F5F5 F5F5 F5F5 F5F5 F6F7 FAFF"
	$"FFFF FFFF FAF7 F5F5 F5F5 F5FF 0000 0000"
	$"0000 00FF F5F5 F5F5 F5F5 F5F5 F5F5 F5F5"
	$"F5F5 F5F5 F5F5 F5F5 F5F5 F5FF 0000 0000"
	$"0000 00FF F5F5 F5F5 F5F5 F5F5 F5F5 F5F5"
	$"F5F5 F5F5 F5F5 F5F5 F5F5 F5FF 0000 0000"
	$"0000 00FF F5F5 F5F5 F5F5 F5F5 F5F5 F5F5"
	$"F5F5 F5F5 F5F5 F5F5 F5F5 F5FF 0000 0000"
	$"0000 00FF F5F5 F5F5 F5F5 F5F5 F5F5 F5F5"
	$"F5F5 F5F5 F5F5 F5F5 F5F5 F5FF 0000 0000"
	$"0000 00FF FFFF FFFF FFFF FFFF FFFF FFFF"
	$"FFFF FFFF FFFF FFFF FFFF FFFF"
};

resource 'icl8' (130, "DrawShapes Stationery", purgeable) {
	$"0000 FFFF FFFF FFFF FFFF FFFF FFFF FFFF"
	$"FFFF FFFF FFFF FFFF FFFF FF00 0000 0000"
	$"0000 FFF5 F5F5 F5F5 F5F5 F5F5 F5F5 F5F5"
	$"F5F5 F5F5 F5F5 F5F5 F5F5 FF00 0000 0000"
	$"0000 FFF5 F5F5 F5F5 F5F5 F5F5 F5F5 F5F5"
	$"F5F5 F5F5 F5F5 F5F5 F5F5 FFFF FF00 0000"
	$"0000 FFF5 F5F5 F5F5 F5F5 F5F5 F5F5 F5F5"
	$"F5F5 F5F5 F5F5 F5F5 F5F5 FFF9 FF00 0000"
	$"0000 FFF5 F5F5 F5F5 F5F5 F5F5 F5F5 F5F5"
	$"F5F5 F5F5 F5F5 F5F5 F5F5 FFF9 FF00 0000"
	$"0000 FFF5 F5F5 F5F5 F5F5 F5F5 F5F5 F5F5"
	$"F5F5 F5F5 F5F5 F5F5 F5F5 FFF9 FF00 0000"
	$"0000 FFF5 F5FF FFFF FFFF FFFF FFFF FFFF"
	$"FFFF FFF5 F5F5 F5F5 F5F5 FFF9 FF00 0000"
	$"0000 FFF5 F5FF 2A2A 2A2A 2A2A 2A2A 2A2A"
	$"2A2A FFFF F5F5 F5F5 F5F5 FFF9 FF00 0000"
	$"0000 FFF5 F5FF 2A2A 2A2A 2A2A 2A2A 2A2A"
	$"2A2A FFFF F5F5 F5F5 F5F5 FFF9 FF00 0000"
	$"0000 FFF5 F5FF 2A2A 2A2A 2A2A 2A2A 2A2A"
	$"2A2A FFFF F5F5 F5F5 F5F5 FFF9 FF00 0000"
	$"0000 FFF5 F5FF 2A2A 2A2A 2A2A 2A2A FFFF"
	$"FFFF FFFF F5F5 F5F5 F5F5 FFF9 FF00 0000"
	$"0000 FFF5 F5FF 2A2A 2A2A 2A2A FFFF D8D8"
	$"D8D8 D8FF FFF5 F5F5 F5F5 FFF9 FF00 0000"
	$"0000 FFF5 F5FF 2A2A 2A2A FFFF D8D8 D8D8"
	$"D8D8 D8D8 D8FF FFF5 F5F5 FFF9 FF00 0000"
	$"0000 FFF5 F5FF FFFF FFFF FFD8 D8D8 D8D8"
	$"D8D8 D8D8 D8D8 FFF5 F5F5 FFF9 FF00 0000"
	$"0000 FFF5 F5F5 FFFF FFFF D8D8 D8D8 D8D8"
	$"D8D8 D8D8 D8D8 D8FF F5F5 FFF9 FF00 0000"
	$"0000 FFF5 F5F5 F5F5 FFD8 D8D8 D8D8 D8D8"
	$"D8D8 D8D8 D8D8 D8D8 FFF5 FFF9 FF00 0000"
	$"0000 FFF5 F5F5 F5F5 FFD8 D8D8 D8D8 D8D8"
	$"D8D8 D8D8 D8D8 D8D8 FFF5 FFF9 FF00 0000"
	$"0000 FFF5 F5F5 F5F5 FAFF D8D8 D8D8 D8D8"
	$"D8D8 D8D8 D8D8 D8FF FFF5 FFF9 FF00 0000"
	$"0000 FFF5 F5F5 F5F5 F5FF FFD8 D8D8 D8D8"
	$"D8D8 D8D8 D8D8 FFFF FFF5 FFF9 FF00 0000"
	$"0000 FFF5 F5F5 F5F5 F5F6 FFFF FFD8 D8D8"
	$"D8D8 D8D8 FFFF FFFF F6F5 FFF9 FF00 0000"
	$"0000 FFF5 F5F5 F5F5 F5F5 F6FA FFFF FFFF"
	$"FFFF FFFF FFFF F7F6 F5F5 FFF9 FF00 0000"
	$"0000 FFF5 F5F5 F5F5 F5F5 F5F6 F7FA FFFF"
	$"FFFF FFFA F7F5 F5F5 F5F5 FFF9 FF00 0000"
	$"0000 FFF5 F5F5 F5F5 F5F5 F5F5 F5F5 F5F5"
	$"F5F5 F5F5 F5F5 F5F5 F5F5 FFF9 FF00 0000"
	$"0000 FFF5 F5F5 F5F5 F5F5 F5F5 F5F5 F5F5"
	$"F5F5 F5F5 FFFF FFFF FFFF FFF9 FF00 0000"
	$"0000 FFF5 F5F5 F5F5 F5F5 F5F5 F5F5 F5F5"
	$"F5F5 F5F5 FF2B 2B2B F7FF F9F7 FF00 0000"
	$"0000 FFF5 F5F5 F5F5 F5F5 F5F5 F5F5 F5F5"
	$"F5F5 F5F5 FF2B 2BF7 FFF9 F72B FF00 0000"
	$"0000 FFF5 F5F5 F5F5 F5F5 F5F5 F5F5 F5F5"
	$"F5F5 F5F5 FF2B F7FF F9F7 2BF6 FF00 0000"
	$"0000 FFF5 F5F5 F5F5 F5F5 F5F5 F5F5 F5F5"
	$"F5F5 F5F5 FFF7 FFF9 F72B F6F6 FF00 0000"
	$"0000 FFF5 F5F5 F5F5 F5F5 F5F5 F5F5 F5F5"
	$"F5F5 F5F5 FFFF F9F7 2BF6 F6F6 FF00 0000"
	$"0000 FFFF FFFF FFFF FFFF FFFF FFFF FFFF"
	$"FFFF FFFF FFF9 F72B F6F6 F6F6 FF00 0000"
	$"0000 0000 FFF9 F9F9 F9F9 F9F9 F9F9 F9F9"
	$"F9F9 F9F9 F9F7 2BF6 F6F6 F6F6 FF00 0000"
	$"0000 0000 FFFF FFFF FFFF FFFF FFFF FFFF"
	$"FFFF FFFF FFFF FFFF FFFF FFFF FF"
};

type kSignature as 'STR ';
resource kSignature (0,
#if qNames
"Signature",
#endif
	purgeable) {
	"DrawShapes 3.0.1 Apple Computer, Inc. 1988-95"
};

resource 'FREF' (128,
#if qNames
"DrawShapes Application",
#endif
	purgeable) {
	'APPL',
	0,
	""
};

resource 'FREF' (129,
#if qNames
"DrawShapes Document",
#endif
	purgeable) {
	kFileType,
	1,
	""
};

resource 'FREF' (130,
#if qNames
"DrawShapes Stationery",
#endif
	purgeable) {
	kStationery,
	2,
	""
};

resource 'BNDL' (128,
#if qNames
"DrawShapes",
#endif
	purgeable) {
	kSignature,
	0,
		{
		'ICN#',
			{
			0, 128,
			1, 129,
			2, 130
			},
		'FREF',
			{
			0, 128,
			1, 129,
			2, 130
			}
		}
};

// Get the default Version resources
include "Defaults.rsrc"  'vers' (1);		// Application or file specific
include "Defaults.rsrc"  'vers' (2);		// Overall package

resource 'PAT#' (128, "Patterns", purgeable) {
	{	/* array PatArray: 40 elements */
		/* [1] */
		$"",
		/* [2] */
		$"8000 0000 08",
		/* [3] */
		$"8800 2200 8800 22",
		/* [4] */
		$"8888 2222 8888 2222",
		/* [5] */
		$"88AA 22AA 88AA 22AA",
		/* [6] */
		$"CCAA 33AA CCAA 33AA",
		/* [7] */
		$"EEAA BBAA EEAA BBAA",
		/* [8] */
		$"EEBB BBEE EEBB BBEE",
		/* [9] */
		$"FFBB FFEE FFBB FFEE",
		/* [10] */
		$"FFBB FFFF FFBB FFFF",
		/* [11] */
		$"8010 0220 0108 4004",
		/* [12] */
		$"FFFF FFFF FFFF FFFF",
		/* [13] */
		$"8822 8822 8822 8822",
		/* [14] */
		$"1122 4488 1122 4488",
		/* [15] */
		$"C480 0C68 4302 3026",
		/* [16] */
		$"B130 031B D8C0 0C8D",
		/* [17] */
		$"AA00 AA00 AA00 AA",
		/* [18] */
		$"8822 5522 8822 5522",
		/* [19] */
		$"8855 2255 8855 2255",
		/* [20] */
		$"77DD 77DD 77DD 77DD",
		/* [21] */
		$"80",
		/* [22] */
		$"AA55 AA55 AA55 AA55",
		/* [23] */
		$"0384 4830 0C02 0101",
		/* [24] */
		$"8244 3944 8201 0101",
		/* [25] */
		$"8814 2241 8841 2214",
		/* [26] */
		$"8080 413E 0808 14E3",
		/* [27] */
		$"2204 8C74 2217 9810",
		/* [28] */
		$"BE80 8808 EB08 8880",
		/* [29] */
		$"25C8 3289 6424 4C92",
		/* [30] */
		$"A29C 41BE 2AC9 14EB",
		/* [31] */
		$"40A0 0000 040A",
		/* [32] */
		$"8040 2000 0204 08",
		/* [33] */
		$"AA00 8000 8800 80",
		/* [34] */
		$"FF80 8080 8080 8080",
		/* [35] */
		$"081C 22C1 8001 0204",
		/* [36] */
		$"FF80 8080 FF08 0808",
		/* [37] */
		$"F874 2247 8F17 2271",
		/* [38] */
		$"BF00 BFBF B0B0 B0B0",
		/* [39] */
		$"FF7F BE5D A241 80",
		/* [40] */
		$"FAF5 FAF5 A050 A050"
	}
};

