/*------------------------------------------------------------------------------
#
#	Apple Macintosh Developer Technical Support
#
#	MultiFinder-Aware Simple TextEdit Sample Application
#
#	Hacked for Windows to only support vertical Help Scrolling... 
#
------------------------------------------------------------------------------*/

#include "MSWindows.h"
#include "Help.h"

#define kDoHyperText	true	// click on green text

extern MSWindowPtr	gHelpWindow;

extern PicHandle	*gPicts;
extern short		gButtonBarHeight;
extern short		gButtonBarID[];
extern Rect			gButtonBar[];

extern short		gHelpInx,gHelpID[];

/* Here are declarations for the local C routines. */

	void GetTERect( WindowPtr window, Rect *teRect );
	void GetLocalUpdateRgn( WindowPtr window, RgnHandle localRgn );
	void AdjustViewRect( TEHandle docTE );
	void AdjustTE( WindowPtr window );

	void ResetScrollBar( void );
	void CommonAction( ControlHandle control, short *amount );
	pascal void VActionProc( ControlHandle control, short part );
	void AdjustHV( Boolean isVert, ControlHandle control, TEHandle docTE,
					Boolean canRedraw );
	void AdjustScrollValues( WindowPtr window, Boolean canRedraw );
	void AdjustScrollSizes( WindowPtr window );
	void AdjustScrollbars( WindowPtr window, Boolean needsResize );

	void GetHelpText(short selStart,short selEnd,TEHandle theTE);
	void TrackHelp(Point where,TEHandle theTE);


/* Called when the window has been resized to fix up the controls and content. */
#pragma segment Main
void DoResizeHelp(WindowPtr window)
{
	if (((DocumentPeek)window)->docVScroll != nil) {	/* to avoid call at init time */
		AdjustScrollbars(window, true);
		AdjustTE(window);
		DoInvalRect(&window->portRect);
	}
} /* DoResizeHelp */


/*	This is called when a window is activated or deactivated.
	It calls TextEdit to deal with the selection. */

void DoActivateHelp(WindowPtr window,Boolean becomingActive)
{
	RgnHandle	tempRgn, clipRgn;
	DocumentPeek doc;
	
	doc = (DocumentPeek) window;
	if ( becomingActive ) {
		/*	since we dont want TEActivate to draw a selection in an area where
			were going to erase and redraw, well clip out the update region
			before calling it. */
		tempRgn = NewRgn();
		clipRgn = NewRgn();
		GetLocalUpdateRgn(window, tempRgn);			/* get localized update region */
		GetClip(clipRgn);
		DiffRgn(clipRgn, tempRgn, tempRgn);			/* subtract updateRgn from clipRgn */
		SetClip(tempRgn);
		TEActivate(doc->docTE);
		SetClip(clipRgn);							/* restore the full-blown clipRgn */
		DisposeRgn(tempRgn);
		DisposeRgn(clipRgn);
		/* the controls must be redrawn on activation: */
		(*doc->docVScroll)->contrlVis = kControlVisible;
		DoInvalRect(&(*doc->docVScroll)->contrlRect);
	}
	else {		
		TEDeactivate(doc->docTE);
		/* the controls must be hidden on deactivation: */
		HideControl(doc->docVScroll);
	}
} /*DoActivateHelp*/

Boolean SetHelpID(short textID)
{
	short	i,baseID,lastID = 0;

	baseID = (textID / 100) * 100;
	switch (baseID) {
	case kHelpMine:
		if (textID <= kHelpMineMax)
			lastID = kHelpMineMax;
		break;
	case kHelpPM:
		if (textID <= kHelpPMMax)
			lastID = kHelpPMMax;
		break;
	case kHelpHelp:
		if (textID <= kHelpHelpMax)
			lastID = kHelpHelpMax;
		break;
	}
	if (lastID == 0)	/* bad help resource ID */
		return(false);

	if (textID == baseID) {
		gButtonBarID[4] = kHelpPrevPictG;
		gButtonBarID[5] = kHelpNextPict;
	} else
	if (textID == lastID) {
		gButtonBarID[4] = kHelpPrevPict;
		gButtonBarID[5] = kHelpNextPictG;
	} else {
		gButtonBarID[4] = kHelpPrevPict;
		gButtonBarID[5] = kHelpNextPict;
	}
	if (0 < gHelpInx)
		gButtonBarID[2] = kHelpBackPict;
	else
		gButtonBarID[2] = kHelpBackPictG;

	if (gHelpInx < kMaxHelpInx) 
		gHelpID[++gHelpInx] = textID;
	else {
		for (i = 0; i < kMaxHelpInx; i++) {
			gHelpID[i] = gHelpID[i + 1];
		}
		gHelpID[gHelpInx] = textID;
	}
}

void DoGetHelp(short textID)
{
	short		 baseID;
	Handle		 hText = nil;
	StScrpHandle hStyle;
	Ptr			 title;

	if (SetHelpID(textID)) {
		hText = GetResource('TEXT', textID);
		hStyle = (StScrpHandle)GetResource('styl', textID);
	}
	if ( hText && hStyle ) {		/* set the new help text & style */
		HLock(hText);
		TESetText(*hText,GetHandleSize(hText),gHelpWindow->docTE);
		HUnlock(hText);
		SetStylScrap(0,GetHandleSize(hText),hStyle,false,gHelpWindow->docTE);
		baseID = (textID / 100) * 100;
		switch (baseID) {
		case kHelpMine:
			title = "\pMinesweeper Help";
			break;
		case kHelpPM:
			title = "\pProgram Manager Help";
			break;
		case kHelpHelp:
		default:
			title = "\pHelp";
			break;
		}
		BlockMove(title,gHelpWindow->child->title,title[0]+1);
		SetWTitle((WindowPtr)gHelpWindow,title);
		ResetScrollBar();
	} else
		DoMSAlert(kInfo, kHelpMsg);
}

void GetHelpText(short selStart,short selEnd,TEHandle theTE)
{
	Handle		 hText;
	short		 textID;
	ResType		 theType;
	StScrpHandle hStyle;
	Str255		 theName;

	theName[0] = selEnd - selStart;
	hText = (*theTE)->hText;
	BlockMove(&(*hText)[selStart],&theName[1],theName[0]);

	hText = GetNamedResource('TEXT', theName);
	hStyle = (StScrpHandle)GetNamedResource('styl', theName);

	if ( hText && hStyle ) {		/* set the new help text & style */
		GetResInfo(hText,&textID,&theType,theName);
		if (SetHelpID(textID)) {	/* we found it, so ID must be good */
			HLock(hText);
			TESetText(*hText,GetHandleSize(hText),theTE);
			HUnlock(hText);
			SetStylScrap(0,GetHandleSize(hText),hStyle,false,theTE);
			DoInvalRect(&(*theTE)->viewRect);
			ResetScrollBar();
		}
	} else
		DoMSAlert(kInfo, kHelpMsg);
}

void TrackHelp(Point where,TEHandle theTE)
{
	short		lineHeight,fontAscent;
	short		i,clickChar,selStart,selEnd;	
	TextStyle	theStyle;
	Ptr			text;

	text = *(*theTE)->hText;
	clickChar = TEGetOffset(where,theTE);
	i = selStart = clickChar;
	while (true) {
		i--;
		TEGetStyle(i,&theStyle,&lineHeight,&fontAscent,theTE);
		if ((0 <= i) && (' ' <= text[i]) &&
			(theStyle.tsColor.green == 0x8000)) {
			selStart = i;
		} else {
			break;
		}
	}
	selEnd = clickChar;
	while (true) {
		TEGetStyle(selEnd,&theStyle,&lineHeight,&fontAscent,theTE);
		if ((selEnd < kMaxTELength) &&  (' ' <= text[selEnd]) &&
			(theStyle.tsColor.green == 0x8000)) {
			selEnd++;
		} else {
			break;
		}
	}
	if (selStart < selEnd) {
	
reclick:
		do {
			if ((selStart <= clickChar) && (clickChar <= selEnd)) {
				TESetSelect(selStart,selEnd,theTE);
//				TEUpdate(&(*theTE)->viewRect, theTE);
				while (StillDown()) {
					GetMouse(&where);
					clickChar = TEGetOffset(where,theTE);
					if (!((selStart <= clickChar) && (clickChar <= selEnd))) {
						TESetSelect(0,0,theTE);	// dragged out
//						TEUpdate(&(*theTE)->viewRect, theTE);
						goto reclick;			// so, try again
					}
				}
				TESetSelect(0,0,theTE);
//				TEUpdate(&(*theTE)->viewRect, theTE);
				GetHelpText(selStart,selEnd,theTE);
				return;		// we did the hyperlink
			}
			GetMouse(&where);
			clickChar = TEGetOffset(where,theTE);
		} while (StillDown());
	}
		
	TESetSelect(0,0,theTE);
//	TEUpdate(&(*theTE)->viewRect, theTE);
	return;
}

/*	This is called when a mouseDown occurs in the content of a window. */

Boolean DoContentClickHelp(WindowPtr window,EventRecord *event)
{
	Point		mouse;
	ControlHandle control;
	short		part, value;
	Boolean		shiftDown;
	DocumentPeek doc;
	Rect		teRect;

	SetPort(window);
	mouse = event->where;							/* get the click position */
	GlobalToLocal(&mouse);
	doc = (DocumentPeek) window;
	/* see if we are in the viewRect. if so, we wont check the controls */
	teRect = (*doc->docTE)->viewRect;
	if ( PtInRect(mouse, &teRect) ) {
		if (kDoHyperText) {
			TrackHelp(mouse, doc->docTE);
		} else {		/* see if we need to extend the selection */
			shiftDown = (event->modifiers & shiftKey) != 0;	/* extend if Shift is down */
			TEClick(mouse, shiftDown, doc->docTE);
		}
	} else {
		part = FindControl(mouse, window, &control);
		switch ( part ) {
			case 0:			
				return(false);		/* do MS stuff for viewRect case */
			case inThumb:
				value = GetCtlValue(control);
				part = TrackControl(control, mouse, nil);
				if ( part != 0 ) {
					value -= GetCtlValue(control);
					/* value now has CHANGE in value; if value changed, scroll */
					if ( value != 0 ) {
						if ( control == doc->docVScroll )
							TEScroll(0, value, doc->docTE);
					}
				}
				break;
			default:				/* they clicked in an arrow, so track & scroll */
				if ( control == doc->docVScroll )
					value = TrackControl(control, mouse, (ProcPtr) VActionProc);
				break;
		}
	}
	return(true);		/* say we've handled it */

} /*DoContentClickHelp*/



void InitButtonBar()
{
	gButtonBarID[0] = kHelpContentPict;
	gButtonBarID[1] = kHelpSearchPict;
	gButtonBarID[2] = kHelpBackPict;
	gButtonBarID[3] = kHelpHistoryPict;
	gButtonBarID[4] = kHelpPrevPict;
	gButtonBarID[5] = kHelpNextPict;
	gButtonBarID[6] = kHelpGreyPict;
}

void SetButtonBar(WindowPtr window,Rect *viewRect)
{
	short		i, id, h[kMaxButtonBar], v, x, y;
	PicHandle	pict;

	x = viewRect->left; y = viewRect->top;
	for (i = 0; i < kMaxButtonBar; i++) {
		if ((id = gButtonBarID[i]) && (pict = gPicts[id])) {
			h[i] = (*pict)->picFrame.right - (*pict)->picFrame.left;
			v = (*pict)->picFrame.bottom - (*pict)->picFrame.top;
			if (y < v)
				y = v;
			x += h[i];
		}
	}

	gButtonBar[0].top = viewRect->top;
	gButtonBar[0].left = window->portRect.left + kGrowBorder;
	gButtonBar[0].right = gButtonBar[0].left + h[0];
	gButtonBar[0].bottom = gButtonBar[0].top + v;

	for (i = 1; i < kMaxButtonBar; i++) {
		if (gButtonBarID[i] != 0) {
			gButtonBar[i] = gButtonBar[i-1];
			gButtonBar[i].left = gButtonBar[i-1].right;
			gButtonBar[i].right = gButtonBar[i].left + h[i];
		}
	}
	gButtonBar[kMaxButtonBar-1].right = 
			window->portRect.right - kGrowBorder;	// grey filler
	gButtonBarHeight = v;
	viewRect->top += gButtonBarHeight;		// offset the view rect
}

void DrawButtonBar()
{
	short		i, id;
	PicHandle	pict;

	for (i = 0; i < kMaxButtonBar; i++) {
		if ((id = gButtonBarID[i]) && (pict = gPicts[id]))
			DrawPicture(pict,&gButtonBar[i]);
	}
}

/* Draw the contents of an application window. */

void DrawHelpWindow(WindowPtr window)
{
	TEHandle theTE;

	theTE = ((DocumentPeek) window)->docTE;
	DrawMSWindow(window);
	DrawButtonBar();
	DrawControls(window);
	EraseRect(&(*theTE)->viewRect);		// clear the old text
	TEUpdate(&(*theTE)->viewRect, theTE);
} /* DrawHelpWindow */


/* Create a new document and window. */

void DoNewHelp( WindowPtr window, short textID )
{
	Boolean		good = false;
	Rect		destRect, viewRect;
	DocumentPeek doc;

	SetPort(window);
	doc = (DocumentPeek) window;

	InitButtonBar();
	GetTERect(window, &viewRect);
	InsetRect(&viewRect, kTextMargin, kTextMargin);	/* adjust for margin */
	
	destRect = viewRect;
	doc->docTE = TEStylNew(&destRect, &viewRect);
	good = doc->docTE != nil;	/* if TENew succeeded, we have a good document */
	
	if ( good ) {				/* good document?  get scrollbar */
		AdjustViewRect(doc->docTE);
		doc->docVScroll = GetNewControl(rVScroll, window);
		good = (doc->docVScroll != nil);
	}
	
	if ( good ) {				/* good?  adjust & draw the controls, draw the window */
		 /* set the initial text & style */
		DoGetHelp(textID);
		gHelpInx--;				/* eat this one for 'Back' Button */
		/* false to AdjustScrollValues means musnt redraw; technically, of course,
		the window is hidden so it wouldnt matter whether we called ShowControl or not. */
		AdjustScrollSizes(window);
		AdjustScrollValues(window, false);
	} else {
		DoCloseHelp(window);	/* otherwise regret we ever created it... */
//		AlertUser(eNoWindow);			/* and tell user */
	}
} /* DoNewHelp */


/* Close a window. This handles desk accessory and application windows. */

/*	1.01 - At this point, if there was a document associated with a
	window, you could do any document saving processing if it is 'dirty'.
	DoCloseWindow would return true if the window actually closed, i.e.,
	the user didnt cancel from a save dialog. This result is handy when
	the user quits an application, but then cancels the save of a document
	associated with a window. */

void DoCloseHelp(WindowPtr window)
{
	TEHandle	theTE;

	theTE = ((DocumentPeek) window)->docTE;
	if ( theTE != nil )
		TEDispose(theTE);	/* dispose the TEHandle if we got far enough to make one */
} /*DoCloseHelp*/


/* Returns the update region in local coordinates */
void GetLocalUpdateRgn(WindowPtr window,RgnHandle localRgn)
{
	CopyRgn(((WindowPeek) window)->updateRgn, localRgn);	/* save old update region */
	OffsetRgn(localRgn, window->portBits.bounds.left, window->portBits.bounds.top);
} /* GetLocalUpdateRgn */


/* Return a rectangle that is inset from the portRect by the size of
	the scrollbars and a little extra margin. */

void GetTERect(WindowPtr window,Rect *teRect)
{
	*teRect = window->portRect;
	teRect->top += kPictHeight + kIconHdr + 4;
	InsetRect(teRect, kGrowBorder, kGrowBorder);	/* adjust for grow bounds */
	teRect->right -= kScrollbarWidth; 				/* and for the scrollbars */
	SetButtonBar(window,teRect);			/* move the view down by the button bar */
} /*GetTERect*/


/* No longer update the TERec's view rect so that it is the greatest multiple 
	of the lineHeight that still fits in the old viewRect, just scroll bits. */

void AdjustViewRect(TEHandle docTE)
{
#pragma unused (docTE)
} /*AdjustViewRect*/


/* Scroll the TERec around to match up to the potentially updated scrollbar
	values. This is really useful when the window has been resized such that the
	scrollbars became inactive but the TERec was already scrolled. */

void AdjustTE(WindowPtr window)
{
	TEPtr		te;
	
	te = *((DocumentPeek)window)->docTE;
	TEScroll(0, (te->viewRect.top - te->destRect.top) -
				GetCtlValue(((DocumentPeek)window)->docVScroll),
				((DocumentPeek)window)->docTE);
} /*AdjustTE*/

void ResetScrollBar(void)
{
	if (((DocumentPeek)gHelpWindow)->docVScroll != nil) {
		SetCtlValue(((DocumentPeek)gHelpWindow)->docVScroll, 0);
		AdjustTE((WindowPtr)gHelpWindow);
	}
}

/*	Common algorithm for pinning the value of a control. It returns the actual amount
	the value of the control changed. Note the pinning is done for the sake of returning
	the amount the control value changed. */

void CommonAction(ControlHandle control,short *amount)
{
	short		value, max;
	
	value = GetCtlValue(control);	/* get current value */
	max = GetCtlMax(control);		/* and maximum value */
	*amount = value - *amount;
	if ( *amount < 0 )
		*amount = 0;
	else if ( *amount > max )
		*amount = max;
	SetCtlValue(control, *amount);
	*amount = value - *amount;		/* calculate the real change */
} /* CommonAction */


/* Determines how much to change the value of the vertical scrollbar by and how
	much to scroll the TE record. */

pascal void VActionProc(ControlHandle control,short part)
{
	short		amount;
	WindowPtr	window;
	TEPtr		te;
	
	if ( part != 0 ) {				/* if it was actually in the control */
		window = (*control)->contrlOwner;
		te = *((DocumentPeek) window)->docTE;
		switch ( part ) {
			case inUpButton:
			case inDownButton:		/* one line */
				amount = kLineHeight;
				break;
			case inPageUp:			/* one page */
			case inPageDown:
				amount = te->viewRect.bottom - te->viewRect.top - kLineHeight;
				break;
		}
		if ( (part == inDownButton) || (part == inPageDown) )
			amount = -amount;		/* reverse direction for a downer */
		CommonAction(control, &amount);
		if ( amount != 0 )
			TEScroll(0, amount, ((DocumentPeek) window)->docTE);
	}
} /* VActionProc */



/* Calculate the new control maximum value and current value.
	The vertical max is calculated by comparing the number of lines to the vertical
	 size of the viewRect. The current values are set by comparing the offset between
	the view and destination rects. If necessary and we canRedraw, have the control be 
	re-drawn by calling ShowControl. */

void AdjustScrollValues(WindowPtr window,Boolean canRedraw)
{
	short		value, max;
	short		oldValue, oldMax;
	Point		pt;
	ControlHandle control;
	TEPtr		te;
	DocumentPeek doc;
	
	doc = (DocumentPeek)window;
	control = doc->docVScroll;

	oldValue = GetCtlValue(control);
	oldMax = GetCtlMax(control);
	te = *doc->docTE;							/* point to TERec for convenience */

	pt = TEGetPoint(kMaxTELength,doc->docTE);
	max = pt.v - (te->viewRect.bottom - te->viewRect.top);
	
	if ( max < 0 ) max = 0;
	SetCtlMax(control, max);
	
	/* Must deref. after SetCtlMax since, technically, it could draw and therefore move
		memory. This is why we dont just do it once at the beginning. */
	te = *doc->docTE;
	value = te->viewRect.top - te->destRect.top;
	
	if ( value < 0 ) value = 0;
	else if ( value >  max ) value = max;
	
	SetCtlValue(control, value);
	/* now redraw the control if it needs to be and can be */
	if ( canRedraw || (max != oldMax) || (value != oldValue) )
		ShowControl(control);

} /*AdjustScrollValues*/


/*	Re-calculate the position and size of the viewRect and the scrollbars. */

void AdjustScrollSizes(WindowPtr window)
{
	Rect		teRect;
	DocumentPeek doc;
	
	doc = (DocumentPeek) window;
	GetTERect(window, &teRect);							/* start with TERect */
	(*doc->docTE)->viewRect = teRect;
	InsetRect(&(*doc->docTE)->viewRect, kTextMargin, kTextMargin);	/* adjust for margin */
	AdjustViewRect(doc->docTE);							/* snap to nearest line */
	MoveControl(doc->docVScroll, teRect.right, teRect.top);
	SizeControl(doc->docVScroll, kScrollbarWidth, teRect.bottom - teRect.top);
} /*AdjustScrollSizes*/


/* Turn off the controls by jamming a zero into their contrlVis fields (HideControl erases them
	and we don't want that). If the controls are to be resized as well, call the procedure to do that,
	then call the procedure to adjust the maximum and current values. Finally re-enable the controls
	by jamming a $FF in their contrlVis fields. */

void AdjustScrollbars(WindowPtr window,Boolean needsResize)
{
	DocumentPeek doc;
	
	doc = (DocumentPeek) window;
	/* First, turn visibility of scrollbars off so we wont get unwanted redrawing */
	(*doc->docVScroll)->contrlVis = kControlInvisible;	/* turn them off */
	if ( needsResize )									/* move & size as needed */
		AdjustScrollSizes(window);
	AdjustScrollValues(window, needsResize);			/* fool with max and current value */
	/* Now, restore visibility in case we never had to ShowControl during adjustment */
	(*doc->docVScroll)->contrlVis = kControlVisible;	/* turn them on */
} /* AdjustScrollbars */
