#include <Quickdraw.h>
#include <Fonts.h>
#include <Menus.h>
#include <OSUtils.h>

//--------------------------- semi-global constants -----------------------------

#define mPlaceMsg			3			// popup placement IM V-248
#define TopMenuItem			3			// popup placement IM V-248

#define	kMItemHeight		16
#define kMenuOffsetLeft		13
#define kMenuOffsetRight	 4
#define kMenuTabOffset		20

#define kMenuUnderline		'&'
#define kMenuTab			'%'

void DoSizeMessage(MenuHandle myMenu);
void DoDrawMessage(MenuHandle myMenu,Rect *myRect);
short DoChooseMessage(MenuHandle myMenu,Rect *myRect,Point myPoint,short oldItem);
short DoPlaceMessage(MenuHandle myMenu,Rect *myRect,Point myPoint);

//----------------------------- MenuDef Procedure -------------------------------

pascal void
MSMenuDef(short message,		// what are we supposed to do?
		MenuHandle theMenu,		// what menu ?
		Rect *menuRect,			// in what rect?
		Point hitPt,			// where's the mouse?
		short *whichItem)		// what item is that?

{	//--------------------- Case on message and call procedure ----------------------
	switch (message) {
		case mSizeMsg :
				DoSizeMessage(theMenu);
				break;
		case mDrawMsg : 
				DoDrawMessage(theMenu,menuRect);
				break;
		case mChooseMsg :
				*whichItem = DoChooseMessage(theMenu,menuRect,hitPt,*whichItem);
				break;
		case mPlaceMsg :
				*whichItem = DoPlaceMessage(theMenu,menuRect,hitPt);
				break;
	}
}

//----------------------------- String parse procedures -------------------------------

short SizeMSMenuItem(StringPtr text)
{
	short	i,sz;
	char	c;

	sz = kMenuOffsetLeft + kMenuOffsetRight;
	for (i = 1; i <= text[0]; i++) {
		switch (c = text[i]) {
			case kMenuUnderline:
				break;
			case kMenuTab:
				sz += kMenuTabOffset;
				break;
			default:
				sz += CharWidth(c);
				break;
		}
	}
	return(sz);
}

void DrawMSMenuItem(StringPtr text,short left,short right,short v)
{
	short	i,sz = 0;
	char	c;
	Boolean	underLine = false;

	MoveTo(left + kMenuOffsetLeft,v);
	for (i = 1; i <= text[0]; i++) {
		switch (c = text[i]) {
			case kMenuUnderline:
				underLine = true;
				break;
			case kMenuTab:
				text[i] = text[0] - i;
				MoveTo(right - (StringWidth((StringPtr)&text[i]) + kMenuOffsetRight),v);
				DrawString((StringPtr)&text[i]);
				text[i] = kMenuTab;	// restore the tab
				return;				// and we're out of here
			default:
				sz = CharWidth(c);
				DrawChar(c);
				if (underLine) {
					Move(-1,1);
					Line(-(sz-1),0);
					Move(sz,-1);
					underLine = false;
				}
				break;
		}
	}
}

//----------------------------- DimRect procedure -------------------------------

void DimRect(Rect *theRect)
{
	Pattern	graypat;
	Ptr		pGrayPat;
	
	pGrayPat = (Ptr)&graypat;
	pGrayPat[0] = pGrayPat[2] = pGrayPat[4] = pGrayPat[6] = 0xAA;
	pGrayPat[1] = pGrayPat[3] = pGrayPat[5] = pGrayPat[7] = 0x55;
	PenPat(&graypat);
	PenMode(patBic);
	PaintRect(theRect);
	PenNormal();
}

//--------------------------- GetItemsRect Function -----------------------------
 
void GetItemsRect(MenuHandle myMenu,Rect *myRect,short theItem,Rect *itemRect)
{
#pragma unused (myMenu)
	*itemRect = *myRect; // initialize the current rect
	itemRect->top += kMItemHeight * (theItem - 1); // update the rect 
	itemRect->bottom = itemRect->top + kMItemHeight;
}
 
//--------------------------- DoDrawMessage Procedure ---------------------------

void DoDrawMessage(MenuHandle myMenu,Rect *myRect)
{
	short	currentItem;
	Rect	currentRect;
	Str255	itemString;
	short	itemMark;
	short	newVert;
	Pattern	graypat;
	Ptr		pGrayPat;
	
	pGrayPat = (Ptr)&graypat;
	pGrayPat[0] = pGrayPat[2] = pGrayPat[4] = pGrayPat[6] = 0xAA;
	pGrayPat[1] = pGrayPat[3] = pGrayPat[5] = pGrayPat[7] = 0x55;
	currentRect = *myRect; // initialize the current rect
	currentRect.bottom = currentRect.top + kMItemHeight;	// 1st menu items rect
	for (currentItem = 1; currentItem <= CountMItems(myMenu); currentItem++) { // draw every item
		GetItem(myMenu,currentItem,itemString);			// get info on each item
		GetItemMark(myMenu,currentItem,&itemMark);		// get the items marked char

		if ((itemString[0] == 1) && (itemString[1] == '-')) {	// special case '-' item
			PenPat(&graypat);
			MoveTo(currentRect.left,currentRect.top + kMItemHeight / 2);
			LineTo(currentRect.right,currentRect.top + kMItemHeight / 2);
			PenNormal();
		} else {	// draw normal item stuff
			newVert = ((currentRect.bottom - currentRect.top) / 2);	//	get baseline
			newVert += currentRect.top + 4; 
			MoveTo(currentRect.left + 2,newVert);
			
			if (itemMark != 0)
				DrawChar(itemMark);
	
			DrawMSMenuItem(itemString,currentRect.left,currentRect.right,newVert);

			if (((*myMenu)->enableFlags & 1) == 0)
				DimRect(&currentRect);		// menu id disabled
			if (((*myMenu)->enableFlags & (1 << currentItem)) == 0)
				DimRect(&currentRect);		// menu currentItem disabled
	
		}
		currentRect.top += kMItemHeight;	// update the rect 
		currentRect.bottom += kMItemHeight;
	}	
} // of DoDrawMessage

//-------------------------- DoChooseMessage Procedure --------------------------
	
short DoChooseMessage(MenuHandle myMenu,Rect *myRect,Point myPoint,short oldItem)
{
	short theItem,result;
	Rect itemRect;

	if (PtInRect(myPoint,myRect)) {
		theItem = 1;
		do {
			GetItemsRect(myMenu,myRect,theItem,&itemRect);
			theItem = theItem + 1;
		} while (!PtInRect(myPoint,&itemRect));
		theItem--; 	// undo last increment

		if (((*myMenu)->enableFlags & 1) == 0)
			theItem = 0;
		if (((*myMenu)->enableFlags & (1 << theItem)) == 0)
			theItem = 0;

		if (theItem != oldItem) {	//de-select old, select new
			if (oldItem != 0) {		// deselect old
				GetItemsRect(myMenu,myRect,oldItem,&itemRect);
				InvertRect(&itemRect);
			}
			if (theItem != 0) {
				GetItemsRect(myMenu,myRect,theItem,&itemRect);
				InvertRect(&itemRect);
			}
		}
		result = theItem; // return result
	} else {	// it was not in our menu
		if (oldItem != 0) { // we need to de-select old item
			GetItemsRect(myMenu, myRect,oldItem,&itemRect);
			InvertRect(&itemRect);
		}
		result = 0; // return result
	}
	return(result);
}

//-------------------------- DoPlaceMessage Procedure --------------------------
	
short DoPlaceMessage(MenuHandle myMenu,Rect *myRect,Point myPoint)
{
	myRect->left = myPoint.h;
	myRect->top = myPoint.v;
	myRect->right = myRect->left + (*myMenu)->menuWidth;
	myRect->bottom = myRect->top + (*myMenu)->menuHeight;
	return(0);
}

//--------------------------- DoSizeMessage Procedure ---------------------------

void DoSizeMessage(MenuHandle myMenu)
{
	short	maxWidth;		// keep track of the maximum width
	short	totalHeight;	// keep track of the total height
	short	currentItem;	// the menu item we are currently looking at
	Str255	itemString;		// text of the current menu item
	short	itemMark;		// the items mark
	short	tempWidth;		// the current items width
			
	maxWidth = 0;			// initailize width
	totalHeight = 0;		// initialize height
	for (currentItem = 1; currentItem <= CountMItems(myMenu); currentItem++ ) {	// look at every item
		GetItem(myMenu,currentItem,itemString);		// get the items text
		GetItemMark(myMenu,currentItem,&itemMark);	// get the items marked char
		
		tempWidth = SizeMSMenuItem(itemString);
		if (tempWidth > maxWidth)
			maxWidth = tempWidth;
		totalHeight += kMItemHeight;		// add just enough for text
	}
	(*myMenu)->menuWidth = maxWidth;		// save result in menu record
	(*myMenu)->menuHeight = totalHeight;	// ditto...
}


