#include <Types.h>
#include <Resources.h>
#include <Quickdraw.h>
#include <QDOffscreen.h>
#include <Palette.h>
#include <Fonts.h>
#include <Events.h>
#include <Windows.h>
#include <Menus.h>
#include <TextEdit.h>
#include <Dialogs.h>
#include <Desk.h>
#include <ToolUtils.h>
#include <Memory.h>
#include <SegLoad.h>
#include <Files.h>
#include <OSUtils.h>
#include <OSEvents.h>
#include <DiskInit.h>
#include <Sound.h>
#include <Packages.h>
#include <Script.h>
#include <Traps.h>
#include "Resource.h"

/* Define HiWrd and LoWrd macros for efficiency. */
#define HiWrd(aLong)	(((aLong) >> 16) & 0xFFFF)
#define LoWrd(aLong)	((aLong) & 0xFFFF)

/* Define TopLeft and BotRight macros for convenience. Notice the implicit
   dependency on the ordering of fields within a Rect */
#define TopLeft(aRect)	(* (Point *) &(aRect).top)
#define BotRight(aRect)	(* (Point *) &(aRect).bottom)


#define kMaxMasters	10		// MoreMasters() Blocks
#define kMaxSleep	20		// WNE sleep time

struct MSIconRecord {
	struct MSIconRecord *next;
	struct MSChildRecord *group;
	struct MSChildRecord *restore;
	short		id;
	short		selected;
	Rect		iconBox;
	Rect		textBox;
	Str31		text1;
	Str31		text2;
	short		action;
	long		refCon;
};
typedef struct MSIconRecord MSIconRecord;
typedef MSIconRecord *MSIconPtr;

struct MSMenuRecord {
	struct MSMenuRecord *next;
	MenuHandle	menu;
	Rect		titleBox;
	Str31		title;
	short		menuItem;
	Rect		menuRect;
};
typedef struct MSMenuRecord MSMenuRecord;
typedef MSMenuRecord *MSMenuPtr;

struct MSChildRecord {
	struct MSChildRecord *next;
	struct MSWindowRecord *window;
	MSMenuPtr	MSMenu;
	MSIconPtr	icon;
	MSIconPtr	iconList;
	Rect		portRect;
	Point		origin;
	Rect		restoreRect;
	Point		restoreOrigin;
	short		active;
	short		base;
	short		minmax;
	short		maximized;
	short		growBorder;
	Rect		controlMenuRect;
	Rect		menuBarRect;
	Rect 		controlMenuBox;
	Rect 		maximizeBox;
	Rect		minimizeBox;
	Str31		title;
};
typedef struct MSChildRecord MSChildRecord;
typedef MSChildRecord *MSChildPtr;

typedef void (*DrawProcPtr) (WindowPtr window);

struct MSWindowRecord {
	WindowRecord	window;
	MSChildPtr		child;
	MSMenuPtr		MSMenu;
	MSIconPtr		icon;
	Point			origin;
	short			savebits;
	GWorldPtr		GWorld;
	DrawProcPtr		DrawProc;
	TEHandle		docTE;
	ControlHandle	docVScroll;
	ProcPtr			docClik;
};
typedef struct MSWindowRecord MSWindowRecord;
typedef MSWindowRecord *MSWindowPtr;
typedef MSWindowRecord DocumentRecord, *DocumentPeek;

typedef struct ChildRec {
	short	xOffset, yOffset;
	short	maxCols, maxRows;
} ChildRec, *ChildPtr, **ChildHnd;

typedef struct IconRec {
	short	actionType;
	long	refCon;
} IconRec, *IconPtr, **IconHnd;

typedef struct FSSpecs {
	FSSpec	file;
	FSSpec	app;
	FInfo	fInfo;
} FSSpecs;

typedef struct PMChildParams {
	short	windInx, childInx, iconID;
	short	xOffset, yOffset;
	short	maxCols, maxRows;
	short	minimizeRow, minimizeCol;
	short	h, v;
	Boolean	iconSelected;
} ChildParams, *ChildParamPtr;

#define MSWindow	((MSWindowPtr)window)

/* Constants */

#define kDoDraw				0
#define kDoSave				1
#define kDoCopy				2

#define kBMPType			'BINA'
#define kCreator			'mdos'

#define kOffscreenOffset	0x4000

#define kNoMinMax			0
#define kMinMax				1

#define vWHITE				0x0FFFF
#define vLTGRAY				0x0C000
#define vGRAY				0x08000
#define vDKGRAY				0x04000
#define vBLACK				0x00000

#define	kMaxChildRows		4
#define	kMaxChildCols		5

//	Minesweeper Picts
#define kButtonPict				130		// Unpressed button
#define kEmptyPict			 kNumPict	// Pressed button is same as zero
#define kNumPict				140		// Microsoft numbers
#define kFlagPict				150		// Flagged mine
#define kBombPict				151		// Unexploded mine
#define kBoomPict				152		// The hit mine
#define kBadPict				153		// Wrong flag guess
#define kMarkPict				154		// Unknown flag
#define kLCDPict				160		// LCD numbers
#define kHappyFace				170		// Smile face
#define kPressedFace			171		// Smile face, pressed
#define kOwFace					172		// Smile face, owww 
#define kSadFace				173		// Sad face
#define kWinFace				174		// Winning face
#define kPressedWin				175		// Winning face, pressed

//	Windows Application Icons
#define	kProgramManager			300		// windows icon ??

#define	kInfoIcon				250
#define	kCautionIcon			251
#define	kErrorIcon				252
#define	kYesNoIcon				253

#define kOKPict					254		// MS OK button
#define kCancelPict				255		// MS Cancel button
#define kYesPict				256		// MS Yes button
#define kNoPict					257		// MS No button

//	Help Button bar
#define kHelpContentPict		270
#define kHelpSearchPict			271
#define kHelpBackPict			272
#define kHelpHistoryPict		273
#define kHelpPrevPict			274
#define kHelpNextPict			275
#define kHelpGreyPict			276
#define kHelpBackPictG			277
#define kHelpPrevPictG			278
#define kHelpNextPictG			279

#define kMaxButtonBar			7		/* includes inactive grey space Pict */
#define kMaxHelpInx				40		/* maximum Help Back stack */


//	Windows Cursors
#define	kHoriCursor				128
#define	kVertCursor				129
#define	kDRightCursor			130
#define	kURightCursor			131
#define	kArrowCursor			132
#define	kMoveCursor				133

#define kMaxChildIcons		(kMaxChildRows * kMaxChildCols)
#define kMaxPict			(400 + iMaxChild * kMaxChildIcons)

//	Window actions
#define vMaximize			1
#define vMinimize			2
#define vRestore			3
#define vCascade			4
#define vTile				5
#define vMove				6

//	Window modes
#define kNormalMode			0
#define kMoveMode			1

//	Window definition
#define kGrowBorder			4
#define kIconSize			32
#define kIconLineHeight		12
#define kIconHdr			12
#define kIconStepH			80
#define kIconStepV			64
#define kIconTextInset		3
#define kIconTextSize		10
#define kIconXOffset (kGrowBorder + (kIconStepH - kIconSize) / 2)

#define kOKBoxW				64
#define kOKBoxH				32

//	Window Icons
#define kMenuHeight			19
#define kPictHeight		(kMenuHeight + kGrowBorder)
#define kPictWidth			22
#define kPictWidth2			18
#define	kMenuBox			220
#define	kMaximizeBox		221
#define	kMinimizeBox		222
#define	kRestoreBox			225
#define kNoBox				223
#define	kPressOffset		10
#define	kInactiveOffset		(-20)
#define	kTop				224
#define	kBottomLeft			226
#define	kBottomRight		227

//	Game window border definition
#define kBox			16
#define kBorderTop	(kPictHeight + kMenuHeight + 1)
#define kBorder1		3
#define kBorder2		6
#define kBorder3		3
#define kBorder		(kBorder1 + kBorder2 + kBorder3)
#define kBorder3h	(kBorder3 - 1)
#define kBorderLCD		4
#define kHeightLCD		32
#define kWidthLCD		18
#define kInsetLCD	((kBorder - 1) + kBorderLCD)
#define kHdrSize	(kBorderTop + kInsetLCD + kHeightLCD + kBorderLCD + kBorder3h + kBorder2 + kBorder3)

//	Game table values
#define kMinRows		8
#define kMaxRows		20
#define kMinCols		8
#define kMaxCols		40
#define kMinMines		8		/* divisors */
#define kMaxMines		2
#define kMaxTimes		4		/* array bounds */

#define kMaxTime		999		/* initial scores */
#define kPlayer		"\pAnonymous"

#define minefield(r,c) 	pMineField[(r) * gCols + (c)]
#define mineview(r,c) 	pMineView[(r) * gCols + (c)]
#define	kNumMask		0x0F
#define	kMine			0x10
#define	kFlag			0x20
#define	kNoFlag			0x40

struct _GamePrefsEntry {
	short	rows;
	short	cols;
	short	mines;
	short	mintimes[kMaxTimes];
	Str255	players[kMaxTimes];
	long	flags;
};
typedef struct _GamePrefsEntry GamePrefsEntry;


/* Prototypes */

//	Windows Bitmap routines
Boolean getdoc(char *filename);
Boolean ReadBMP(StringPtr name);
void DrawBMP(WindowPtr window);
void SavePict(WindowPtr window,StringPtr name);
void OpenMSMenu(WindowPtr window,MSMenuPtr MSMenu,Point pt);

// BMP and Offscreen routines
void CreateGWorld(WindowPtr window);
void SaveBits(WindowPtr window);
void DrawGWorld(WindowPtr window);
void EndGWorld(WindowPtr window);
void DoInvalRect(Rect *rect);
Handle FindBMPFiles(short vRefNum);
void InitBkgndPixData(Ptr baseAddr,long rowBytes,long rows,long inBytes,long offset);
void InitBkgndGWorld(Rect *drawRect,short depth,CTabHandle pmTable);
void FillBMPGWorld(long offset,long inBytes);
void SetCorner(RGBColor *magicColor);

//	Application routines called from the shell
Boolean InitializeApp(void);
void InstallMenus(WindowPtr window);
void SetUpMenus(void);
void DoMenuCommand(MSChildPtr MSChild,MenuHandle hMenu,short theItem);
Boolean PtInGame(Point where);
Boolean ContentClick(EventRecord *event,WindowPtr window);
void ClickIcon(MSChildPtr MSChild, Point where, long modifiers);
void DoubleClickIcon(MSChildPtr MSChild,MSIconPtr icon);
void ClickBkgndIcon(WindowPtr window,EventRecord *event);
void DrawBkgndWindow(WindowPtr window);
void DrawPict(short id,short *x,short *y);
void DrawCell(short id,Rect *box);
void DrawIcon(MSIconPtr icon,short active);
void DrawWindow(WindowPtr window);
void GameTime(void);
void GetDragRect(WindowPtr window,Rect *dragRect);
Boolean DragChild(WindowPtr window,Point where);

//	Shell routines called from the application
Boolean IsAppWindow( WindowPtr window );
Boolean IsDAWindow( WindowPtr window );
void SetActive(WindowPtr window,Boolean becomingActive);
void CountClicks(EventRecord *macEvent);
void HideChild(MSChildPtr MSChild);
void BringChildFront(WindowPtr window,MSChildPtr MSChild,Point where,Boolean doUpdate);
void SetCursorRgns(void);
void SizeMSWindow(MSChildPtr MSChild, Boolean arrangeIcons);
Boolean DoCloseWindow( WindowPtr window );
void Terminate(void);

//	Application routines called within the application
void Initialize(void);
void EventLoop(void);
void DrawGame(BitMap *pixMap);
void DrawMSMenuBar(WindowPtr window);
void DrawMSWindow(WindowPtr	window);
void ClipMenuFromView(MSMenuPtr MSMenu,RgnHandle clipRgn);
void DrawAlertWindow(WindowPtr	window);
void DrawMineWindow(WindowPtr window);
void InitMineCursor(void);
void ResetGameGlobals(GamePrefsEntry *prefs);
void InitGameGlobals(void);
void NewGame(Boolean resize);
short InitCustomGame(GamePrefsEntry *prefs);
unsigned char minechar(short r,short c);
void InitGame(short rx,short cx);
void DrawFrame(void);
void DrawHdr(void);
void DrawMap(void);
void DrawNewMap(BitMap *pixMap);
void DrawCell(short id,Rect *box);
void ClickCell(Point where,short modifiers);
void ClickGame(Point where,short modifiers);
void doDialog(short dialogID);
Point CenterWithin(Rect *r, Rect *within);
void PositionWindow(WindowPtr window);	
void DoMSCommand(MSChildPtr MSChild,short command);
void DoControlMenu(MSChildPtr MSChild,Point where);
void DoMenuBar(MSChildPtr MSChild,Point where);
void DoubleClickIcon(MSChildPtr MSChild,MSIconPtr icon);

//	def procs
typedef pascal void (*menuProcPtr)(short message,MenuHandle menu,
									Rect *menuRect,Point pt,short *item);

pascal void MSMenuDef(short message,MenuHandle theMenu,
						Rect *menuRect,Point hitPt,short *whichItem);

void DoMSAlert(short severity, short item);
void HideMSWindow( WindowPtr window );

//	Help Window procs
void DoNewHelp( WindowPtr window, short textID );
void DrawHelpWindow( WindowPtr window );
void DoResizeHelp( WindowPtr window );
void DoActivateHelp( WindowPtr window, Boolean becomingActive );
Boolean DoContentClickHelp( WindowPtr window, EventRecord *event );
void DoGetHelp(short textID);
void DoCloseHelp( WindowPtr window) ;

// utility routines
Boolean SupportsCatSearch(short vRefNum);
void doCatSearch(void);
Boolean SupportsFileLaunch(void);
void doLaunchApp(FSSpec *launchApp);
void doLaunchFile(FSSpecs *launch);
PicHandle IconToPict(short iconResID);

MSChildPtr BuildPMChild(MSChildPtr PMChild, StringPtr childName, 
				ChildParamPtr childParams);
void BuildPMIcon(MSChildPtr PMChild, MSChildPtr MSChild, IconPtr pIcon, 
				ChildParamPtr childParams);
MSChildPtr AddPMChild(MSChildPtr PMChild, StringPtr childName);
void AddPMIcon(MSChildPtr PMChild, MSChildPtr MSChild, 
				short action, long refCon);
void AddPict(short id, Handle pict, StringPtr text);


