#include "MSWindows.h"

// Windows Shell Globals
extern short	 gSound;
extern SndChannelPtr gSoundChannnel;
extern Handle	 gSndHdl[];
extern WindowPtr gMineWindow;
extern RgnHandle gArrowRgn,gcArrowRgn,gcUpRgn,gMineRgn,gNoMineRgn;
extern Boolean	 gMagicMines;
extern Boolean	 gEmptyGame;
extern char		 gMagicStr[];
extern RGBColor	 gMagicColor;
extern RGBColor	 gBlack,gWhite,gLtGray,gDkGray;
extern PicHandle *gPicts;

// Game Globals
Rect			gGameRect,
				gHdrRect,
				gFaceRect;
Boolean			gMarks = false,
				gNewGame,
				gRunning = false;
short			gClearCells,
				gFace,
				gPressedFace;

unsigned long	gStartTime, gEndTime, gNow, gThen;

short			gRows, gCols, gMines;

unsigned char	*pMineField = nil;
unsigned char	*pMineView  = nil;

short			gGameType;
short			colrange[]  = {12,16,30,16};
short			rowrange[]  = {12,16,16,16};
short			minerange[] = {20,40,99,40};

GamePrefsEntry	gPrefs;


#pragma segment TheApp

void
WritePrefs(GamePrefsEntry *prefs)
{
	Handle h;

	if (h = GetResource(rScoreType, rScore)) {
		HNoPurge(h);
		BlockMove((Ptr)prefs, *h, sizeof(GamePrefsEntry));
		UseResFile(HomeResFile(h));
		ChangedResource(h);
		WriteResource(h);
		HPurge(h);
		UpdateResFile(HomeResFile(h));
	}
}

void
ResetGameGlobals(GamePrefsEntry *prefs)
{
	short i;

	for (i=0; i < kMaxTimes; i++) {
		prefs->mintimes[i] = kMaxTime;
		BlockMove((Ptr)kPlayer,(Ptr)&prefs->players[i],sizeof(kPlayer));
	}
	WritePrefs(prefs);
}

void
ReadPrefs(GamePrefsEntry *prefs)
{
	Handle h;

	if (h = GetResource(rScoreType, rScore))
		BlockMove(*h, (Ptr)prefs, sizeof(GamePrefsEntry));
	else {
		h = NewHandleClear(sizeof(GamePrefsEntry));
		AddResource(h, rScoreType, rScore, "\pScore");
		ResetGameGlobals(prefs);
	}
}

void
InitGameGlobals()
{
	GetDateTime(&gStartTime);
	qd.randSeed = gStartTime;

	gGameType = iIntermediate - iBeginner;

	ReadPrefs(&gPrefs);

	NewGame(false);
}

void
NewGame(Boolean resize)
{
	short	r,c;
	Rect	bounds;
	Point	pt;
	WindowPtr window = gMineWindow;

	GetDateTime(&gStartTime);
	gNow = gStartTime;
	gEndTime = 0;
	gRunning = false;
	gPrefs.cols  = gCols  = colrange[gGameType];
	gPrefs.rows  = gRows  = rowrange[gGameType];
	gPrefs.mines = gMines = minerange[gGameType];

	if (pMineField) {
		DisposPtr(pMineField);
		DisposPtr(pMineView);
	}
	pMineField = NewPtrClear(gRows*gCols);
	pMineView  = NewPtrClear(gRows*gCols);
	for (r = 0; r < gRows; r++)
	for (c = 0; c < gCols; c++)
		mineview(r,c) = kButtonPict;
	SetRect(&bounds,0,0,gCols * kBox + 2*kBorder,gRows * kBox + kHdrSize + kBorder);
	gHdrRect = bounds;
	gHdrRect.top = kBorderTop;	// skip MSWindow Hdr
	gHdrRect.bottom = kHdrSize;
	gFaceRect.top = kBorderTop + kInsetLCD;
	gFaceRect.bottom = gFaceRect.top + kIconSize;
	gFaceRect.left = (gHdrRect.right - kIconSize) / 2;
	gFaceRect.right = gFaceRect.left + kIconSize;
	gFace = kHappyFace;
	gPressedFace = kPressedFace;
	gNewGame = true;
	gClearCells = 0;
	SetPort(window);
	SizeWindow(window,bounds.right,bounds.bottom,true);	// fUpdate
	pt.h = window->portRect.right;
	pt.v = window->portRect.bottom;
	LocalToGlobal(&pt);
	if ((qd.screenBits.bounds.right - 4) < pt.h) // keep it on screen
		pt.h = qd.screenBits.bounds.right - 4;
	if ((qd.screenBits.bounds.bottom - 4) < pt.v)
		pt.v = qd.screenBits.bounds.bottom - 4;
	MoveWindow(window, pt.h - bounds.right, pt.v - bounds.bottom, true);
	MSWindow->child->portRect = window->portRect;
	SizeMSWindow(MSWindow->child,false);
	CreateGWorld(window);
	ClipRect(&bounds);
	if (resize)
		EraseRect(&bounds);
	DoInvalRect(&bounds);			// force an update now
}

short
InitCustomGame(GamePrefsEntry *prefs)
{
	if ((prefs->rows < kMinRows) || (prefs->rows > kMaxRows))
		return(iRows);
	if ((prefs->cols < kMinCols) || (prefs->cols > kMaxCols))
		return(iCols);
	if ((prefs->mines < ((prefs->rows * prefs->cols) / kMinMines)) ||
		(prefs->mines > ((prefs->rows * prefs->cols) / kMaxMines)))
		return(iMines);

	gGameType = iCustom - iBeginner;
	colrange[gGameType] = prefs->cols;
	rowrange[gGameType] = prefs->rows;
	minerange[gGameType] = prefs->mines;
	NewGame(true);
	return(0);
}

void
InitMineCursor(void)
{
	register short r,c,x,y;
	Point	pt;
	Rect	box;
	
	if (gMagicMines) {				// find the magic regions
		EmptyRgn(gMineRgn);
		CopyRgn(gArrowRgn,gNoMineRgn);
		pt.h = kBorder;
		pt.v = kHdrSize;
		LocalToGlobal(&pt);
		for (r = 0; r < gRows; r++)
		for (c = 0; c < gCols; c++) {
			x = c * kBox + pt.h;
			y = r * kBox + pt.v;
			SetRect(&box,x,y,x + kBox,y + kBox);
			RectRgn(gcArrowRgn,&box);
			if (minefield(r,c) & kMine) {
				UnionRgn(gMineRgn,gcArrowRgn,gMineRgn);
				DiffRgn(gNoMineRgn,gcArrowRgn,gNoMineRgn);
			}
		}
	}
}

void
InitGame(short rx,short cx)
{
	register short n,r,c,r1,c1,r2,c2;
	unsigned long x;
	
	for (r = 0; r < gRows; r++)
	for (c = 0; c < gCols; c++)
		minefield(r,c) = 0;

	minefield(rx,cx) = kFlag;		// flag the original click cell		
	n = gMines;
	while (0 < n) {
		x = Random();
		r1 = x & 0x7FFF;
		r = r1 % gRows;
		r1 = (x >> 8) & 0x7FFF;
		c = r1 % gCols;
		if ((minefield(r,c) & (~kNumMask)) == 0) {
			minefield(r,c) = kMine;
			r1 = (r == 0) ? 0 : r - 1;
			r2 = (r >= gRows - 1) ? gRows - 1 : r + 1;
			for ( ; r1 <= r2; r1++) {
				c1 = (c == 0) ? 0 : c - 1;
				c2 = (c >= gCols - 1) ? gCols - 1 : c + 1;
				for ( ; c1 <= c2; c1++) {
					if (minefield(r1,c1) != kMine)
						minefield(r1,c1)++;
				}
			}
			n--;
		}
	}
	minefield(rx,cx) -= kFlag;		// unflag the original click cell	

	if (gEmptyGame) {
		gMagicMines = ((gMagicStr[0] == 5) && (gMagicStr[1] == 'x') &&
					   (gMagicStr[2] == 'y') && (gMagicStr[3] == 'z') &&
					   (gMagicStr[4] == 'z') && (gMagicStr[5] == 'y'));
		gMagicStr[0] = 0;
		gEmptyGame = false;
	}
	InitMineCursor();
	GetDateTime(&gStartTime);		// start timing the game now
	gNow = gStartTime;
	gNewGame = false;
	gClearCells = gRows * gCols - gMines;
	gRunning = true;
}

void
GameTime(void)
{
	GrafPtr	svPort;
	WindowPtr window;

	if (gRunning) {
		GetDateTime(&gNow);
		if (gNow != gThen) {
			GetPort(&svPort);
			window = gMineWindow;
			SetPort(window);
			RectRgn(gcUpRgn,&gHdrRect);
			if (MSWindow->MSMenu != nil)	// clip out visible the menu
				ClipMenuFromView(MSWindow->MSMenu,gcUpRgn);
			SetClip(gcUpRgn);
			DrawHdr();
			ClipRect(&window->portRect);
			MSWindow->savebits = kDoSave;	
			SetPort(svPort);
		}
	}
}

void
DrawGame(BitMap *pixMap)
{
	if (gNewGame)
		DrawNewMap(pixMap);
	else {
		DrawFrame();
		DrawMap();
		DrawHdr();
	}
}

unsigned char
minechar(short r,short c)
{
	short v;
	
	v = minefield(r,c);
	return ((v >= kMine) ? kBombPict: (v & kNumMask) + kNumPict);
}

void
DrawPict(short id,short *x,short *y)
{
	PicHandle pict;
	Rect 	box;

	if (pict = gPicts[id]) {
		SetRect(&box,*x,*y,*x + (*pict)->picFrame.right,*y + (*pict)->picFrame.bottom);
		DrawPicture(pict,&box);
		*x += (*pict)->picFrame.right;
	}
}

void
DrawLCD(short n,short x,short y)
{
	short i,j;
	short v;
	Str255	text;

	NumToString(n,&text);
	for (i = 0; i < 3; i++) {
		j = text[0] + i - 2;
		v = (j > 0) ? text[j] - '0' : 0;
		DrawPict(v + kLCDPict,&x,&y);
	}
}

void
DrawBevel(Rect *box,short width,RGBColor *topColor,RGBColor *bottomColor)
{
	register short i,a,b1,b2;
	
	RGBForeColor(topColor);
	PenSize(width,width);
	FrameRect(box);
	RGBForeColor(bottomColor);
	PenNormal();
	a = box->bottom-1;
	b1 = box->left;
	b2 = box->right-1;
	for (i = 0; i < width; i++) {
		MoveTo(b1++,a);
		LineTo(b2--,a--);
	}
	a = box->right-1;
	b1 = box->top;
	b2 = box->bottom-1;
	for (i = 0; i < width; i++) {
		MoveTo(a,b1++);
		LineTo(a--,b2--);
	}
}

void
DrawFrame(void)
{
	short x,y;
	Rect box,box1;
	
	box = gMineWindow->portRect;
	box.top = kBorderTop;		// Header & menu bar
	RGBForeColor(&gLtGray);
	PaintRect(&box);

	DrawBevel(&box,kBorder1,&gWhite,&gDkGray);
	
	InsetRect(&box,kBorder1+kBorder2,kBorder1+kBorder2);
	box1 = box;
	box1.bottom = box1.top + 2*kBorderLCD + +2* kBorder3h + kHeightLCD;
	DrawBevel(&box1,kBorder3h,&gDkGray,&gWhite);
	
	box1 = box;
	box1.top = kHdrSize - kBorder3;
	DrawBevel(&box1,kBorder3,&gDkGray,&gWhite);
	
	RGBForeColor(&gDkGray);
	x = kInsetLCD;
	y = kBorderTop + kInsetLCD + kHeightLCD - 1;
	MoveTo(x,kBorderTop + kInsetLCD);
	LineTo(x,y);
	RGBForeColor(&gWhite);
	x += 3*kWidthLCD + 1;
	MoveTo(x,kBorderTop + kInsetLCD);
	LineTo(x,y);
	RGBForeColor(&gDkGray);
	x = gMineWindow->portRect.right - kInsetLCD - 3*kWidthLCD - 2;
	MoveTo(x,kBorderTop + kInsetLCD);
	LineTo(x,y);
	RGBForeColor(&gWhite);
	x += 3*kWidthLCD + 1;
	MoveTo(x,kBorderTop + kInsetLCD);
	LineTo(x,y);
	RGBForeColor(&gBlack);
}

void
DrawHdr(void)
{
	short time;

	DrawCell(gFace,&gFaceRect);

	DrawLCD(gMines,kInsetLCD + 1,kBorderTop + kInsetLCD);
	
	if (gEndTime) {
		time = gEndTime - gStartTime;
	} else
	if (gRunning) {
		GetDateTime(&gNow);
		time = gNow - gStartTime;
		if (time > 999)
			time = 999;
	} else
		time = 0;

	DrawLCD(time,gMineWindow->portRect.right - kInsetLCD - 3*kWidthLCD - 1,
		kBorderTop + kInsetLCD);
	gThen = gNow;
}

void
DrawCell(short id,Rect *box)
{
	PicHandle pict;
	
	if (pict = gPicts[id])
		DrawPicture(pict,box);
}

void
DrawNewMap(BitMap *pixMap)
{
	short	r,c;
	Rect	box,box1;
	
	DrawFrame();
	DrawHdr();
	SetRect(&box,kBorder,kHdrSize,kBorder+kBox,kHdrSize+kBox);
	r = 0;
	for (c = 0; c < gCols; c++) {
		DrawCell(mineview(r,c),&box);
		box.left += kBox;
		box.right += kBox;
	}
	box.left = kBorder;
	box.right -= kBox;
	gGameRect = box;
	for (r = 1; r < gRows; r++) {
		box1 = box;
		box.top += kBox;
		box.bottom += kBox;
		CopyBits(pixMap,pixMap,&box1,&box,srcCopy,nil);
	}
	gGameRect.bottom = box.bottom;
}

void
DrawMap(void)
{
	short	r,c;
	Rect	box;
	
	DrawHdr();
	SetRect(&box,kBorder,kHdrSize,kBorder+kBox,kHdrSize+kBox);
	for (r = 0; r < gRows; r++) {
		box.left = kBorder;
		box.right = box.left + kBox;
		for (c = 0; c < gCols; c++) {
			DrawCell(mineview(r,c),&box);
			box.left += kBox;
			box.right += kBox;
		}
		box.top += kBox;
		box.bottom += kBox;
	}
}

void
ClickMines(void)
{
	short	r,c,x,y;
	unsigned char v,v1;
	Rect	box;

	for (r = 0; r < gRows; r++) {
		for (c = 0; c < gCols; c++) {
			v = minechar(r,c);
			if (v == kBombPict)
				v1 = v;
			else
			if (mineview(r,c) == kFlagPict)	// misidentified
				v1 = kBadPict;
			else
				v1 = 0;
			if (v1 != 0) {
				mineview(r,c) = v1;
				x = c * kBox + kBorder;
				y = r * kBox + kHdrSize;
				SetRect(&box,x,y,x + kBox,y + kBox);
				DrawCell(v1,&box);
			}
		}
	}
	gFace = kSadFace;
	gEndTime = gNow;
	gRunning = false;
}

void
ClickNeighbors(r,c)
{
	short	r1,c1,r2,c2,x,y;
	unsigned char v;
	Rect	box;

	r1 = (r == 0) ? 0 : r - 1;
	r2 = (r >= gRows - 1) ? gRows - 1 : r + 1;
	for ( ; r1 <= r2; r1++) {
		c1 = (c == 0) ? 0 : c - 1;
		c2 = (c >= gCols - 1) ? gCols - 1 : c + 1;
		for ( ; c1 <= c2; c1++) {
			v = minechar(r1,c1);
			if (mineview(r1,c1) == kButtonPict) {	// currently unturned or unflagged
				mineview(r1,c1) = v;
				gClearCells--;			// count down good ones
				x = c1 * kBox + kBorder;
				y = r1 * kBox + kHdrSize;
				SetRect(&box,x,y,x + kBox,y + kBox);
				DrawCell(v,&box);
				if (v == kNumPict)
					ClickNeighbors(r1,c1);
			}
		}
	}
}

short
CountNeighborFlags(r,c)
{
	short	r1,c1,r2,c2;
	short	flags = 0;
	Boolean button = false;

	r1 = (r == 0) ? 0 : r - 1;
	r2 = (r >= gRows - 1) ? gRows - 1 : r + 1;
	for ( ; r1 <= r2; r1++) {
		c1 = (c == 0) ? 0 : c - 1;
		c2 = (c >= gCols - 1) ? gCols - 1 : c + 1;
		for ( ; c1 <= c2; c1++) {
			if (mineview(r1,c1) == kButtonPict) 
				button = true;
			else
			if (mineview(r1,c1) == kFlagPict) 
				flags++;
		}
	}
	return((button) ? flags : 9);
}

short
ClearNeighbors(r,c)
{
	short	r1,c1,r2,c2,x,y;
	unsigned char v;
	Rect	box;

	r1 = (r == 0) ? 0 : r - 1;
	r2 = (r >= gRows - 1) ? gRows - 1 : r + 1;
	for ( ; r1 <= r2; r1++) {
		c1 = (c == 0) ? 0 : c - 1;
		c2 = (c >= gCols - 1) ? gCols - 1 : c + 1;
		for ( ; c1 <= c2; c1++) {
			v = minechar(r1,c1);
			if (mineview(r1,c1) == kButtonPict) {	// currently unturned or unflagged
				if (v == kBombPict) {
					if (gSound) SndPlay(gSoundChannnel,gSndHdl[(gMines&1)?sLose1a:sLose2a],false);
					ClickMines();
					if (gSound) SndPlay(gSoundChannnel,gSndHdl[(gMines&1)?sLose1b:sLose2b],false);
					mineview(r1,c1) = kBoomPict;		// we blew up
					gClearCells += 8;	// don't count down good ones
				}
				mineview(r1,c1) = v;
				gClearCells--;			// count down good ones
				x = c1 * kBox + kBorder;
				y = r1 * kBox + kHdrSize;
				SetRect(&box,x,y,x + kBox,y + kBox);
				DrawCell(v,&box);
				if (v == kNumPict)
					ClickNeighbors(r1,c1);
			}
		}
	}
	return(minechar(r,c));
}

Boolean PtInGame(Point where)
{
	return (PtInRect(where,&gGameRect) || PtInRect(where,&gFaceRect));
}

void
ClickGame(Point where,short modifiers)
{
	Boolean result = false;
	
reclick:
	do {
		if (PtInRect(where,&gGameRect) && (!gEndTime))
			ClickCell(where,modifiers);
		else
		if (PtInRect(where,&gFaceRect)) {
			DrawCell(gPressedFace,&gFaceRect);
			while (StillDown()) {
				GetMouse(&where);
				if (!PtInRect(where,&gFaceRect)) {	// dragged out of the box
					DrawCell(gFace,&gFaceRect);
					goto reclick;					// try again
				}
			}
			if (gSound) SndPlay(gSoundChannnel,gSndHdl[sReset1],false);
			NewGame(false);
			if (gSound) SndPlay(gSoundChannnel,gSndHdl[sReset2],false);
		}
		GetMouse(&where);
	} while (StillDown());
}

void
DoWinner()
{
	gFace = kWinFace;
	gPressedFace = kPressedWin;
	DrawCell(kWinFace,&gFaceRect);
	if (gSound) SndPlay(gSoundChannnel,gSndHdl[sWin],false);
	gEndTime = gNow;
	if (gPrefs.mintimes[gGameType] > (gEndTime - gStartTime)) {
		gPrefs.mintimes[gGameType] = gEndTime - gStartTime;
		doDialog(rPlayer);
		WritePrefs(&gPrefs);
	}
	gRunning = false;
}

void
ClickCell(Point where,short modifiers)
{
	unsigned char v;
	short	r,c,x,y;
	Rect	box;
	
	DrawCell(kOwFace,&gFaceRect);
	where.h -= kBorder;
	where.v -= kHdrSize;
	r = where.v / kBox;
	c = where.h / kBox;
	if ((r < 0) || (gRows <= r) || (c < 0) || (gCols <= c))
		return;	// not in a cell
	x = c * kBox + kBorder;
	y = r * kBox + kHdrSize;
	SetRect(&box,x,y,x + kBox,y + kBox);
	v = mineview(r,c);
	if (v == kButtonPict)
		DrawCell(kEmptyPict,&box);
	
	while (StillDown()) {
		GetMouse(&where);
		if (!PtInRect(where,&box)) {	// dragged out of the box
			if (mineview(r,c) == kButtonPict)
				DrawCell(kButtonPict,&box);
			if (!PtInRect(where,&gGameRect))	// dragged out of the game
				DrawCell(gFace,&gFaceRect);
			return;								// try new box
		}
	}
	if (gNewGame) {
		InitGame(r,c);
	}

	switch (mineview(r,c)) {
	case kMarkPict:			// was a mark flag
		if (gSound) SndPlay(gSoundChannnel,gSndHdl[sClick2],false);
		v = kButtonPict;
		break;

	case kFlagPict:			// was a bomb flag
		if (gSound) SndPlay(gSoundChannnel,gSndHdl[sFlag],false);
		gMines++;
		v = (gMarks) ? kMarkPict : kButtonPict;
		break;

	case kButtonPict:		// is a button
		if (modifiers) {
			if ((gMarks) && (modifiers & optionKey)) {	// option key down
				if (gSound) SndPlay(gSoundChannnel,gSndHdl[sClick2],false);
				v = kMarkPict;
			} else {			// any modifier key down
				if (gSound) SndPlay(gSoundChannnel,gSndHdl[sClick2],false);
				v = kFlagPict;
				gMines--;
				if ((gClearCells <= 0) && (gMines == 0))
					DoWinner();
			}
		} else {					// just a clean click
			v = minechar(r,c);
			if (v == kBombPict) {
				if (gSound) SndPlay(gSoundChannnel,gSndHdl[(gMines&1)?sLose1a:sLose2a],false);
				ClickMines();
				if (gSound) SndPlay(gSoundChannnel,gSndHdl[(gMines&1)?sLose1b:sLose2b],false);
				v = kBoomPict;		// we blew up
			} else
			if (mineview(r,c) == kButtonPict) {	// currently unturned or unflagged
				if (((--gClearCells) <= 0) && (gMines == 0)) {	// count down good ones
					DoWinner();
				} else {
					if (gSound) SndPlay(gSoundChannnel,gSndHdl[sClick1],false);
				}
			}
		}
		break;

	default:
		v = minechar(r,c) - kNumPict;
		if (v == CountNeighborFlags(r,c)) {
			if (gSound) SndPlay(gSoundChannnel,gSndHdl[sReset2],false);
			v = ClearNeighbors(r,c);
			if ((gClearCells <= 0) && (gMines == 0))
				DoWinner();
		} else
			return;		// not a valid click location
	}
	mineview(r,c) = v;
	DrawCell(v,&box);
	if (v == kNumPict) {		// this cell has no surrounding bombs
		if (gSound) SndPlay(gSoundChannnel,gSndHdl[sReset2],false);
		ClickNeighbors(r,c);
	}
	DrawHdr();
	((MSWindowPtr)gMineWindow)->savebits = true;
}
