/*
**	Offscreen.c
**
**	Copyright (c) 1991, Bob Spence
**	All rights reserved.
*/

#include "MSWindows.h"

extern short		gDouble;
extern MSWindowPtr	gBkgndWindow;
extern RGBColor		gBlack, gWhite;

void
CreateGWorld(WindowPtr window)
{
	OSErr		err;

	if (MSWindow->GWorld != nil)
		EndGWorld(window);
	err = NewGWorld (&MSWindow->GWorld, 8, &window->portRect, nil, nil, nil);
	MSWindow->savebits = kDoDraw;
}

void
DrawGWorld(WindowPtr window)
{
	GDHandle	theGDevice;
	PixMapHandle thePixMap;
	short	h,v;
	Rect	srcRect,destRect;

	if (MSWindow->GWorld != nil) {
		theGDevice = GetGWorldDevice(MSWindow->GWorld);
	//	thePixMap = GetGWorldPixMap(MSWindow->GWorld);	// 7.0
		thePixMap = (*theGDevice)->gdPMap;
		srcRect = (*thePixMap)->bounds;
		destRect = srcRect;
		if ((window == gBkgndWindow) && gDouble) {
			h = destRect.right *= 2;
			v = destRect.bottom *= 2;
		} else {
			h = destRect.right;
			v = destRect.bottom;
		}
		RGBForeColor (&gBlack);
		RGBBackColor (&gWhite);
		LockPixels(thePixMap);
		while (destRect.top < window->portRect.bottom) {
			CopyBits((BitMap*)*thePixMap,&window->portBits, 
					  &srcRect,&destRect,srcCopy,nil);
			if (destRect.right < window->portRect.right)
				OffsetRect(&destRect,h,0);
			else
				OffsetRect(&destRect,-destRect.left,v);
		}
		UnlockPixels(thePixMap);
	}
}


void
EndGWorld(WindowPtr window)
{
	DisposeGWorld(MSWindow->GWorld);
	MSWindow->GWorld = nil;
}

void
createPalette(WindowPtr window,CTabHandle colors)
{
	PaletteHandle thePalette;

//	SelectWindow(window);
	if (thePalette = GetPalette(window))
		DisposePalette(thePalette);

	thePalette = NewPalette((*colors)->ctSize + 1,colors,pmCourteous,0);
	SetPalette(window,thePalette,false);
	ActivatePalette(window);
}

void
InitBkgndGWorld(Rect *drawRect,short depth,CTabHandle pmTable)
{
	OSErr		err;
	GDHandle	SaveGD;
	CGrafPtr	SavePort;

	GetGWorld (&SavePort,&SaveGD);
	if (gBkgndWindow->GWorld != nil)
		EndGWorld((WindowPtr)gBkgndWindow);
	createPalette((WindowPtr)gBkgndWindow,pmTable);
	err = NewGWorld (&gBkgndWindow->GWorld, depth, drawRect, pmTable, nil, nil);
	DisposHandle((Handle)pmTable);
	SetGWorld (SavePort,SaveGD);
}

void
FillBMPGWorld(long offset,long inBytes)
{
	GDHandle	theGDevice;
	PixMapHandle thePixMap;
	long		maxx,maxy;
	Ptr			bitsOut;

	theGDevice = GetGWorldDevice(gBkgndWindow->GWorld);
//	thePixMap = GetGWorldPixMap(gBkgndWindow->GWorld);	// 7.0
	thePixMap = (*theGDevice)->gdPMap;
	maxx = (*thePixMap)->rowBytes & 0x3FFF;
	maxy = (*thePixMap)->bounds.bottom - (*thePixMap)->bounds.top;
	LockPixels(thePixMap);
	bitsOut = GetPixBaseAddr(thePixMap);
	InitBkgndPixData(bitsOut,maxx,maxy,inBytes,offset);
	UnlockPixels(thePixMap);
}

void
PutPictFile(Handle theData,StringPtr name)
{
	Point	gLocation = {0x40,0x40};
	short	refNum;
	OSErr	err;
	SFReply	reply;
	long	len,theLength;
	Ptr		p;

	SFPutFile(gLocation,					/* location */
		(StringPtr)"\pSave Picture as:",	/* prompt string */
		name,								/* original name */
		nil,								/* dlgHook */
		&reply);							/* record for returned values */

	if (reply.good) {
		err = Create(reply.fName,reply.vRefNum,'8BIM','PICT');
		if (err != noErr) return;
		err = FSOpen(reply.fName,reply.vRefNum,&refNum); 
		if (err != noErr) return;
		p = NewPtrClear(len = 512);
		err = FSWrite(refNum,&len,p);	// the first block is empty ??
		DisposPtr(p);
		theLength = GetHandleSize(theData);
		HLock(theData);
		err = FSWrite(refNum,&theLength,(Ptr)*theData);
		HUnlock(theData);
		err = FSClose(refNum);
		FlushVol(nil,reply.vRefNum);
	}
}

short	resid = 5000;

void PutPictResource(Handle theData,StringPtr name)
{
	short refNum;
	
	CreateResFile("\pimages");
	refNum = OpenResFile("\pimages");
	AddResource(theData,'PICT',resid++,name);
	CloseResFile(refNum);
}


void
SavePict(WindowPtr window,StringPtr name)
{
	long		i,end,sz,row;
	Ptr			base,pt;
	Rect		srcRect;
	PicHandle	myPict = nil;
	CGrafPtr	SavePort;
	GDHandle	SaveGD;
	GDHandle	theGDevice;
	PixMapHandle thePixMap;

	if (MSWindow->GWorld != nil) {
		theGDevice = GetGWorldDevice(MSWindow->GWorld);
	//	thePixMap = GetGWorldPixMap(MSWindow->GWorld);	// 7.0
		thePixMap = (*theGDevice)->gdPMap;
		srcRect = (*thePixMap)->bounds;
		base = GetPixBaseAddr(thePixMap);
		row = (*thePixMap)->rowBytes & 0x3FFF; 	/*offset to next line*/
		sz = (*thePixMap)->pixelSize;

		pt = base + row;				// kludge, 1st row is null, 2nd is 0xFF ??
		end = srcRect.right * sz / 8;	// many pixels per byte
		for (i = 0; i < end; i++) {
			if ((*pt & 0xFF) == 0xFF)	// find 1st whitespace
				break;
			else
				pt++;
		}
		srcRect.right = i * 8 / sz;
		if ((sz < 8) && ((*(pt-1) & 0x0F) == 0x0F))
			srcRect.right--;

		pt = base + 1;
		end = srcRect.bottom;			// one row per pixel
		for (i = 0; i < end; i++) {
			if ((*pt & 0xFF) == 0xFF)	// find 1st whitespace
				break;
			else
				pt += row;
		}
		srcRect.bottom = i;

		GetGWorld (&SavePort,&SaveGD);
		SetPort((WindowPtr)gBkgndWindow);
		ClipRect(&srcRect);				// Set the view clip
		RGBForeColor (&gBlack);
		RGBBackColor (&gWhite);
		myPict = OpenPicture(&srcRect);
		LockPixels(thePixMap);
		CopyBits((BitMap*)*thePixMap,&((WindowPtr)gBkgndWindow)->portBits, 
				  &srcRect,&srcRect,srcCopy,nil);
		UnlockPixels(thePixMap);
		ClosePicture();
		ClipRect(&(*((CWindowPtr)gBkgndWindow)->portPixMap)->bounds);	// Restore the view clip
		SetGWorld (SavePort,SaveGD);
		if (false) {
			PutPictResource((Handle)myPict,name);
		} else {
			PutPictFile((Handle)myPict,name);
			DisposHandle((Handle)myPict);
		}
	}
}
