/* [Buffered_File_Routines.h] START OF FILE */

/* [---BEGIN---] FILE HEADER COMMENTS */

/**
 ** SCREEN VIEW = Palatino-12, Tabs=6 chars
 ** PRINTER VIEW = Palatino (Bold) -12, Tabs=0.30 inch
 ** ========================================================================
 ** Written by James Merkle
 ** Header file for main
 ** (C) Copyright 1990 by James Merkle
 ** ========================================================================
 **
 **/

/* [----END----] FILE HEADER COMMENTS */


/*=###################################################################=*/
/*=###################################################################=*/
#ifndef _H_Buffered_File_Routines

#define _H_Buffered_File_Routines
/*=###################################################################=*/
/*=###################################################################=*/


/*=###################################################################=*/
/* [---BEGIN---] FILE INCLUDE AREA */

/* [----END----] FILE INCLUDE AREA */
/*=###################################################################=*/


/*=###################################################################=*/
/* [---BEGIN---] DEFINE AREA */

/* file transfers will take place in 512 byte (MINIMUM_LOGICAL_SECTOR_SIZE) increments, when possible */
#define MINIMUM_LOGICAL_SECTOR_SIZE	512

#define MINIMUM_TRANSFER_FILE_BUFFER_SIZE	MINIMUM_LOGICAL_SECTOR_SIZE

/* MAX_OPEN_FORKS is used in setting a limit on the array that keeps track of open files */
/* 10 is set now, because on most systems the actual number of open files is  limited anyway!*/
/* in MS-DOS 3.1 the default number of File Control Blocks is only 4!!! anything beyond that is virtual */
/* in Mac OS 6.X, according to the IM2-page 87 alludes that File Manager also may treat the supply of FCBS virtually */ 
#define MAX_OPEN_FORKS	10

/* Paranoid-only safety keys used when allocating and destroying references to open file buffers */
#define BUFFER_NOT_VALID 		SAFE_FALSE
#define BUFFER_IN_USE				SAFE_TRUE

/* buffer size limitations for buffered file routines */
/* PRACTICAL_BUFFER_MINUMUM must not be less than MINIMUM_TRANSFER_FILE_BUFFER_SIZE */
#define 		PRACTICAL_BUFFER_MINUMUM 		4096

/* PRACTICAL_BUFFER_MAXIMUM is used for limiting the amount of memory that the file fork copy routine uses */
/* 102400 is not unreasonable since the application does not copy file forks in an asychronous manner, anyway*/
#define 		PRACTICAL_BUFFER_MAXIMUM 		102400

/* flags used with buffered file open routines */
#define 		DATA_FORK								1234
#define 		RESOURCE_FORK							4321

#define 		WRITE_MODE							1234
#define 		READ_MODE								4321

#define 		FLUSH_BUFFER							TRUE
#define 		DO_NOT_FLUSH_BUFFER				FALSE

/* [----END----] DEFINE AREA */
/*=###################################################################=*/


/*=###################################################################=*/
/* [---BEGIN---] PROTOTYPES-------------------------- */													
													
extern Boolean Create_File_Buffer_Structures(void);
													
extern Boolean Destroy_File_Buffer_Structures(void);

extern Boolean Read_File_Using_Buffer(	INTEGER_16 open_File_Fork
												, INTEGER_32	*data_Size_Read
												, Ptr data_Destination
												, INTEGER_16 *result_OS_Error);
							
							
extern Boolean Flush_File_Buffer( 	INTEGER_16 open_File_Fork
										, INTEGER_16 *result_OS_Error);
																				
									
extern Boolean Write_File_Using_Buffer(	INTEGER_16 		open_File_Fork
												, INTEGER_32 	*data_Size_To_Write
												, Ptr 				data_Source
												, INTEGER_16 	*result_OS_Error);
													
																																						
extern Boolean Open_File_Using_Buffer( INTEGER_16		fork_Desired
										, Str255			file_Name_P
										, INTEGER_16		working_Directory_Ref
										, INTEGER_16	 	*file_Reference_Num
										, INTEGER_32		minimum_Size
										, INTEGER_32		maximum_Size
										, INTEGER_32		desired_Size
										, INTEGER_32		*result_Size
										, Safe_Boolean	read_Only_Acceptable
										, INTEGER_16		*result_OS_Error);
												
extern Boolean Open_Well_Defined_File_Using_Buffer( INTEGER_16		fork_Desired
										, Str255			file_Name_P
										, INTEGER_16		working_Directory_Ref
										, INTEGER_16 	volume_Reference
										, INTEGER_32 	directory_ID
										, INTEGER_16	 	*file_Reference_Num
										, INTEGER_32		minimum_Size
										, INTEGER_32		maximum_Size
										, INTEGER_32		desired_Size
										, INTEGER_32		*result_Size
										, Safe_Boolean	read_Only_Acceptable
										, INTEGER_16		*result_OS_Error);
													
extern Boolean Close_File_With_Buffer(	INTEGER_16 		open_File_Fork
												, Boolean			flush_Buffer_Desired
												, INTEGER_16 	*result_OS_Error);
													
extern Boolean Locate_File_Buffer_Index(	INTEGER_16	 open_File_Fork
												, INTEGER_32	 *result_Buffer_Index);													
																																	
extern Boolean Copy_File_To_File( INTEGER_16		source_File_Fork
										,INTEGER_16		target_File_Fork
										, INTEGER_32		maximum_Memory_To_Utilize); 

extern Boolean Copy_File_To_File_Unbuffered( INTEGER_16 source_File_Fork
											, INTEGER_16 target_File_Fork
											, INTEGER_32 maximum_Memory_To_Utilize);
													
/* [----END----] PROTOTYPES-------------------------- */
/*=###################################################################=*/

/*=###################################################################=*/
/* [---BEGIN---] EXTERNAL DECLARATIONS----------------- */

typedef	struct file_Buffer_Def
  {
  INTEGER_16		buffer_Activated; /* treated as a "safe" flag */
  INTEGER_32		buffer_Size;
  char 				*buffer_Address;
  INTEGER_32 	read_Write_Flags; /* Read , Write, or Both */
  INTEGER_32		file_Position_Offset;
  INTEGER_32		buffer_Position_Offset;
  INTEGER_32		data_Position_Offset;
  INTEGER_32 	ticks_Since_Last_Purge;
  } file_Buffer, *file_Buffer_Ptr;

typedef struct file_Buffer_Array_Def
 {
 	INTEGER_32 		total_Open_Files;
	INTEGER_16		last_Accessed_Buffer_Index; /* previously used file_Buffer_Ptr*/
 	INTEGER_32		last_Accessed_OS_File_Reference; /* previously used file system ID */
	file_Buffer_Ptr 	list_Of_Open_Forks[MAX_OPEN_FORKS]; /* supporting structures */
	INTEGER_16 		list_Of_Open_Fork_IDs[MAX_OPEN_FORKS]; /* file system IDs of open files (-1 reserved as safety flag) */
  } file_Buffer_Array, *file_Buffer_Array_Ptr;



extern file_Buffer_Array_Ptr		file_Buffers; /* for controlling open files and buffering */
extern INTEGER_32				file_Buffers_Memory_In_Use;
extern Boolean					allow_Damaged_Data_To_Load; /* When True, bad data ignored on reads */

/* [---BEGIN---] EXTERNAL DECLARATIONS----------------- */
/*=###################################################################=*/

/*=###################################################################=*/
/*=###################################################################=*/
#endif
/*=###################################################################=*/
/*=###################################################################=*/


/* END OF FILE: Buffered_File_Routines.h */
