/* START OF FILE: Memory_Utilities.h */


/*=**************************************************************************=*/
/* [---BEGIN---] FILE HEADER COMMENTS */
/**
 ** SCREEN VIEW = [Adobe Syntax Black-10, Tabs = 6]
 ** PRINTER VIEW = [Adobe Syntax Black-10, Tabs = 6]
 ** =======================================================================
 ** Skeleton v1.0, Skeleton is an application program that does nothing.
 ** Written by Jim Wolff, and Jim Merkle
 ** Memory_Utilities.h - Header for Memory_Utilities.c
 ** current version (C) Copyright 1992 James Merkle
 ** =======================================================================
 **
 **
 ** =============
 ** Revision History
 ** =============
 ** SEE NS_Shell_Main.c for revision history and version number.
 **
 ** ====================
 ** OTHER INFORMATION
 ** ====================
 ** 
 **
 **/
/* [----END----] FILE HEADER COMMENTS */
/*=**************************************************************************=*/


/*=**************************************************************************=*/
#ifndef _H_Memory_Utilities

#define _H_Memory_Utilities
/*=**************************************************************************=*/


/*=**************************************************************************=*/
/* [---BEGIN---] FILE INCLUDE AREA */

#include "Shell_Globals.h"

/* [----END----] FILE INCLUDE AREA */
/*=**************************************************************************=*/


/*=**************************************************************************=*/
/* [---BEGIN---] DEFINE AREA */

#define ALLOW_PURGING					SAFE_TRUE
#define DISALLOW_PURGING				SAFE_FALSE

#define ALLOW_PARTIAL_AMOUNT		SAFE_TRUE
#define DISALLOW_PARTIAL_AMOUNT	SAFE_FALSE

#define MOVE_HIGH_REQUIRED			SAFE_TRUE
#define MOVE_HIGH_NOT_REQUIRED		SAFE_FALSE


/* [----END----] DEFINE AREA */
/*=**************************************************************************=*/


/*=**************************************************************************=*/
/* [---BEGIN---] TYPE AREA */

/* =======
typedef enum {
	MEMORY_NOT_INITIALIZED = SAFE_FALSE,
	MEMORY_INITIALIZED = SAFE_TRUE
} Memory_Allocation;
======= */

/* [----END----] TYPE AREA */
/*=**************************************************************************=*/


/*=**************************************************************************=*/
/* [---BEGIN---] PROTOTYPE AREA */


#pragma parameter Block_Move_No_Flush(__A0,__A1,__D0)
pascal void Block_Move_No_Flush(const void *srcPtr,void *destPtr,Size byteCount)
    = 0xA22E;   /* is not A02E */


extern Safe_Boolean Initialize_Memory_Utilities(void);

extern Safe_Boolean Arrange_Free_Memory(CARDINAL_32 memory_Requested
									, CARDINAL_32 memory_To_Preserve
									, Safe_Boolean purge_Allowed
									, Safe_Boolean partial_Allowed
									, CARDINAL_32 *memory_Prepared);

extern Safe_Boolean Arrange_Free_System_Memory(CARDINAL_32 memory_Requested
											, CARDINAL_32 memory_To_Preserve
											, Safe_Boolean purge_Allowed
											, Safe_Boolean partial_Allowed
											, CARDINAL_32 *memory_Prepared);

extern Safe_Boolean Create_Handle(CARDINAL_32 *memory_Requested
								, CARDINAL_32 memory_To_Preserve
								, Safe_Boolean purge_Allowed
								, Safe_Boolean partial_Allowed
								, Handle *data_Handle
								, Memory_Allocation *allocated);

extern Safe_Boolean Create_System_Handle(CARDINAL_32 *memory_Requested
										, CARDINAL_32 memory_To_Preserve
										, Safe_Boolean purge_Allowed
										, Safe_Boolean partial_Allowed
										, Handle *data_Handle
										, Memory_Allocation *allocated);

extern Safe_Boolean Create_Ptr(CARDINAL_32 *memory_Requested
							, CARDINAL_32 memory_To_Preserve
							, Safe_Boolean purge_Allowed
							, Safe_Boolean partial_Allowed
							, Ptr *data_Ptr
							, Memory_Allocation *allocated);

extern Safe_Boolean Destroy_Handle(Handle *data_Handle
									, Memory_Allocation *allocated);

extern Safe_Boolean Destroy_Ptr(Ptr *data_Ptr
							, Memory_Allocation *allocated);

extern Safe_Boolean Begin_Use_Of_Handle(Handle handle_To_Prepare
									, Safe_Boolean move_High_Vital
									, SignedByte *original_State);

extern Safe_Boolean End_Use_Of_Handle(Handle handle_To_Prepare
									, SignedByte original_State);

extern Safe_Boolean Reset_Handle_Size(Handle *old_Handle, CARDINAL_32 new_Size, INTEGER_16 *result_Code);
						
extern Safe_Boolean Reset_Handle_Size_Preserving_Contents(Handle *old_Handle
																	, CARDINAL_32 new_Size
																	, INTEGER_16 *result_Code);

extern Boolean Determine_Absolute_Free_Memory(INTEGER_32 *total_Bytes_Contiguous_Now
								,INTEGER_32	*total_Bytes_Now
								,INTEGER_32 	*total_Bytes_Contiguous_Later
								,INTEGER_32	*total_Bytes_Later);
							
extern void Calculate_Total_Free_Stack_Space(CARDINAL_32 *total_Stack_Space
				, CARDINAL_32 *current_Stack_Free
				, CARDINAL_32 *total_Appl_Zone);

extern Boolean Load_Desired_Memory_Options(CARDINAL_32 *dynamic_Stack_Desired_In_Options
												,CARDINAL_32 *application_Heap_Desired_In_Options );

extern void Limit_Memory_Value_To_Safer_Limit_If_Virtual_Memory(CARDINAL_32 *desired_Size
										, CARDINAL_32 minimum_Size_To_Shrink);

extern Boolean Move_Memory(Ptr data_Source, Ptr data_Destination, INTEGER_32 bytes_To_Copy);

extern Safe_Boolean Lock_And_Translate_Contiguous_Memory(Ptr logical_Address
																, CARDINAL_32 memory_Size
																, Ptr *physical_Address);
																
extern Safe_Boolean Unlock_Memory(Ptr logical_Address
									, CARDINAL_32 memory_Size);
									
extern Safe_Boolean Unhold_Memory(Ptr logical_Address
									, CARDINAL_32 memory_Size);

extern Safe_Boolean Hold_Memory(Ptr logical_Address
													, CARDINAL_32 memory_Size);

extern Safe_Boolean Create_Pointer_To_Held_RAM_Pages_In_Handle(Safe_Boolean allow_Temporary_Memory_If_Needed
																		, CARDINAL_32 memory_Ratio_To_Take
																		, CARDINAL_32 maximum_Useful_Memory
																		, CARDINAL_32 minimum_Useful_Memory
																		, Handle	*new_Buffer_Handle
																		, Memory_Allocation	*new_Buffer_Memory_In_Use
																		, Ptr	*new_Buffer_Pointer_To_Page_Start
																		, Safe_Boolean	*new_Memory_Is_Temporary_Heap
																		, CARDINAL_32	*usable_Buffer_Size
																		, Safe_Boolean	*pages_Held_And_Needs_Release);

extern Safe_Boolean Reset_System_Handle_Size_Preserving_Contents(Handle *old_Handle, CARDINAL_32 new_Size, INTEGER_16 *result_Code);


/* [----END----] PROTOTYPE AREA */
/*=**************************************************************************=*/


/*=**************************************************************************=*/
/* [---BEGIN---] EXTERN DECLARATION AREA */

/* [----END----] EXTERN DECLARATION AREA */
/*=**************************************************************************=*/


/*=**************************************************************************=*/
/* [---BEGIN---] LOCAL STATIC DEFINITION AREA */

/* [----END----] LOCAL STATIC DEFINITION AREA */
/*=**************************************************************************=*/


/*=**************************************************************************=*/
#endif
/*=**************************************************************************=*/


/* END OF FILE: Memory_Utilities.h */