/* START OF FILE: Shell_Assembly.h */


/*=**************************************************************************=*/
/* [---BEGIN---] FILE HEADER COMMENTS */
/**
 ** SCREEN VIEW = [Adobe Syntax Black-10, Tabs = 6]
 ** PRINTER VIEW = [Adobe Syntax Black-10, Tabs = 6]
 ** =======================================================================
 ** Written by Jim Wolff, and Jim Merkle
 ** Shell_Assembly.h - Header for Shell_Assembly.c
 ** current version (C) Copyright 1993 James Merkle
 ** =======================================================================
 **
 **
 ** =============
 ** Revision History
 ** =============
 ** SEE NS_Shell_Main.c for revision history and version number.
 **
 ** ====================
 ** OTHER INFORMATION
 ** ====================
 ** 
 **
 **/
/* [----END----] FILE HEADER COMMENTS */
/*=**************************************************************************=*/


/*=**************************************************************************=*/
#ifndef _H_Shell_Assembly

#define _H_Shell_Assembly
/*=**************************************************************************=*/


/*=**************************************************************************=*/
/* [---BEGIN---] FILE INCLUDE AREA */

#include "Shell_Globals.h"

/* [----END----] FILE INCLUDE AREA */
/*=**************************************************************************=*/


/*=**************************************************************************=*/
/* [---BEGIN---] DEFINE AREA */

/* [----END----] DEFINE AREA */
/*=**************************************************************************=*/


/*=**************************************************************************=*/
/* [---BEGIN---] TYPE AREA */

typedef struct object_Record
		{
			struct object_Record **next_Object;
			int					 object_Mode;
			Handle				 object_Driver;
			Handle				 object_Data;
			INTEGER_32				 user_Reference;
		} object_Record, *object_Ptr, **object_Hdl;


typedef struct LaunchStruct {
	INTEGER_8		*pfName;			/* pointer to the name of launchee */
	INTEGER_16		param;
	INTEGER_8		LC[2];				/*extended parameters:*/
	INTEGER_32		extBlockLen;		/*number of bytes in extension == 6*/
	INTEGER_16		fFlags;				/*Finder file info flags (see below)*/
	INTEGER_32		launchFlags;		/*bit 31,30==1 for sublaunch, others reserved*/
} *pLaunchStruct;

/* [----END----] TYPE AREA */
/*=**************************************************************************=*/


/*=**************************************************************************=*/
/* [---BEGIN---] PROTOTYPE AREA */


/*  */
/* This is the same as : movem.l	d1-d2/a0-a1,-(sp) */
/*  */
void Save_Normally_Corruptable_Registers(void) = {0x48E7, 0x60C0};

/*  */
/* This is the same as : movem.l	(sp)+, d1-d2/a0-a1 */
/*  */
void Restore_Normally_Corruptable_Registers(void) = {0x4CDF, 0x0306};

/*  */
/* This is the same as : movem.l	d1-d7/a0-a5,-(sp) */
/*  */
void Save_Extra_Registers(void) = {0x48E7, 0x7FFC};

/*  */
/* This is the same as : movem.l	(sp)+, d1-d7/a0-a5 */
/*  */
void Restore_Extra_Registers(void) = {0x4CDF, 0x3FFE};


extern Safe_Boolean Initialize_Shell_Assembly(void);

extern Safe_Boolean	Initialize_Shell_Assembly(void);

extern void	Setup_CRC_32_Table(void);

extern void	CalculateCRC32(void);

extern void	CalculateCRC32long(void);

extern void	Calculate_CRC_32(
					Ptr data_Buffer
					, CARDINAL_32 data_Size
					, CARDINAL_32 *current_Checksum);
					
extern pascal void	Fill_Word_Buffer(
					Ptr buffer
					, INTEGER_32 buffer_Size
					, INTEGER_16 value);
					
extern pascal void	Fill_Long_Buffer_Normal(
					Ptr buffer
					, INTEGER_32 buffer_Size
					, INTEGER_32 value);
					
extern pascal void	Fill_Large_Buffer(
					Ptr buffer
					, INTEGER_32 buffer_Size
					, INTEGER_32 value);
					
extern pascal OSErr	Launch_Application(
					pLaunchStruct pLnch);
					
extern pascal Boolean	Swap_Instruction_Cache(
					Boolean cache_Enable);
					
extern pascal void	Flush_Instruction_Cache(void);

extern pascal Boolean	Swap_Data_Cache(
					Boolean cache_Enable);
					
extern pascal void	Flush_Data_Cache(void);

extern pascal void	Flush_Code_Caches(void);

extern pascal OSErr	Range_Flush_Code_Caches(
					Ptr address
					, INTEGER_32 count);
					
extern pascal void	Enable_External_Cache(void);

extern pascal void	Disable_External_Cache(void);

extern pascal void	Flush_External_Cache(void);


#if (defined(__MWERKS__))

k_MW_ASM CARDINAL_32 Compact_System_Heap(INTEGER_32 contiguous_Memory_Desired);

k_MW_ASM CARDINAL_32 Get_Max_Contiguous_System_If_Compacted(INTEGER_32 contiguous_Memory_Desired);

k_MW_ASM void Get_Maximum_Free_System_If_Purged(INTEGER_32 *contiguous_Memory_Possible
																, INTEGER_32 *total_Memory_Possible);
k_MW_ASM OSErr Purge_System_Heap(INTEGER_32 contiguous_Memory_Desired);

k_MW_ASM CARDINAL_32 Compact_And_Purge_All_System_Heap(void);

#else
#if (defined(THINK_C))
pascal CARDINAL_32 Compact_System_Heap(INTEGER_32 contiguous_Memory_Desired);

pascal CARDINAL_32 Get_Max_Contiguous_System_If_Compacted(INTEGER_32 contiguous_Memory_Desired);

pascal void Get_Maximum_Free_System_If_Purged(INTEGER_32 *contiguous_Memory_Possible
																, INTEGER_32 *total_Memory_Possible);

pascal OSErr Purge_System_Heap(INTEGER_32 contiguous_Memory_Desired);

pascal CARDINAL_32 Compact_And_Purge_All_System_Heap(void);
#else
#error "\pUNKNOWN COMPILER (Neither Metroworks nor THINK C)"
#endif
#endif



#if (defined(__MWERKS__))

extern k_MW_ASM void Fill_Byte_Memory_With_Byte(Ptr start_Address
								, CARDINAL_32 byte_Total
								, CARDINAL_16 byte_Value);


#else 
#if (defined(THINK_C))

extern pascal void Fill_Byte_Memory_With_Byte(Ptr start_Address
								, CARDINAL_32 byte_Total
								, CARDINAL_16 byte_Value);

#else 
#error "\pUnknown Compiler (not THINK or Metroworks)"
#endif
#endif


#if (defined(__MWERKS__))

extern k_MW_ASM OSErr Install_Driver(INTEGER_16 driver_Reference);
extern k_MW_ASM OSErr Simulate_Driver_Open(CARDINAL_32 address_For_Open, CARDINAL_32 parameter_Block, DCtlPtr device_Driver_Ctl_Ptr);

#else 
#if (defined(THINK_C))

extern pascal OSErr Install_Driver(INTEGER_16 driver_Reference);
extern pascal OSErr Simulate_Driver_Open(CARDINAL_32 address_For_Open, CARDINAL_32 parameter_Block, DCtlPtr device_Driver_Ctl_Ptr);

#else 
#error "\pUnknown Compiler (not THINK or Metroworks)"
#endif
#endif



#if (defined(__MWERKS__))
extern k_MW_ASM void Copy_Long_Cache_Aligned_040_Memory(Ptr source_Buffer
									, Ptr destination_Buffer
									, CARDINAL_32 int amount);

extern k_MW_ASM void Copy_Cache_Aligned_040_Memory_Blocks(Ptr source_Buffer
									, Ptr destination_Buffer
									, CARDINAL_32 int amount);

#else 
extern pascal void Copy_Long_Cache_Aligned_040_Memory(Ptr source_Buffer
															, Ptr destination_Buffer
															, CARDINAL_32 int amount);

extern pascal void Copy_Cache_Aligned_040_Memory_Blocks(Ptr source_Buffer
									, Ptr destination_Buffer
									, CARDINAL_32 int amount);

#endif


/* [----END----] PROTOTYPE AREA */
/*=**************************************************************************=*/


/*=**************************************************************************=*/
/* [---BEGIN---] EXTERN DECLARATION AREA */

/* [----END----] EXTERN DECLARATION AREA */
/*=**************************************************************************=*/


/*=**************************************************************************=*/
/* [---BEGIN---] LOCAL STATIC DEFINITION AREA */

/* [----END----] LOCAL STATIC DEFINITION AREA */
/*=**************************************************************************=*/


/*=**************************************************************************=*/
#endif
/*=**************************************************************************=*/


/* END OF FILE: Shell_Assembly.h */