/* START OF FILE: String_Utilities.h */


/*=**************************************************************************=*/
/* [---BEGIN---] FILE HEADER COMMENTS */
/**
 ** SCREEN VIEW = [Adobe Syntax Black-10, Tabs = 6]
 ** PRINTER VIEW = [Adobe Syntax Black-10, Tabs = 6]
 ** =======================================================================
 ** Written by Jim Wolff, and Jim Merkle
 ** String_Utilities.h - Header file for string manipulation routines.
 ** current version (C) Copyright 1993 James Merkle
 ** =======================================================================
 **
 **
 ** ============
 ** Revision History
 ** ============
 ** SEE NS_Shell_Main.c for revision history and version number.
 **
 ** ===============
 ** OTHER INFORMATION
 ** ===============
 **
 **/
/* [----END----] FILE HEADER COMMENTS */
/*=**************************************************************************=*/


/*=**************************************************************************=*/
/* [---BEGIN---] FILE INCLUDE AREA */

#include "Shell_Globals.h"
#include "Memory_Utilities.h"

/* [----END----] FILE INCLUDE AREA */
/*=**************************************************************************=*/


/*=**************************************************************************=*/
#ifndef _H_String_Utilities

#define _H_String_Utilities
/*=**************************************************************************=*/


/*=**************************************************************************=*/
/* [---BEGIN---] DEFINE AREA */

#define NO_PAD		1
#define NO_LIMIT	0

#define COMPARE_GREATER			1
#define COMPARE_EQUAL				0
#define COMPARE_LESS_THAN		-1

/* [----END----] DEFINE AREA */
/*=********************************************************************=*/


/*=********************************************************************=*/
/* [---BEGIN---] TYPE AREA */

	/*=--------------------------------------------------------------------------=*/
	/* ASCII_Character will always usually be unsigned char. */
	/* For quick upgrading, replacing definition of "ASCII_Character" */
	/* to a bizzare struct typedef will make the compiler flush out any */
	/* references to string routines that are hiding and might need another */
	/* adjustment to another form in the future when text strings */
	/* become full Apple Script manager or retrograding to "char" */
	/* I would have used CARDINAL_8, instead of ASCII_Character */
	/* but it would mess up "pad_Length" and countless others if a  */
	/* replacement was ever done */
	/*=--------------------------------------------------------------------------=*/
	
	typedef	CARDINAL_8		ASCII_Character;

/* [----END----] TYPE AREA */
/*=**************************************************************************=*/
/* [---BEGIN---] PROTOTYPE AREA */

extern Boolean Initialize_String_Utilities(void);

extern void Mem_To_Hex_String_Hyper_Fast(register CARDINAL_8 *memory
											, CARDINAL_8 number_Of_Bytes
											, Boolean use_Spaces
											, Str255 hex_Result_String);

extern void Convert_Byte_To_Hex(CARDINAL_8 byte_Value
								, Str255 hex_Result_String);

extern void Convert_Integer_To_Hex(INTEGER_16 integer_Value
									, Str255 hex_Result_String);

extern void Convert_Long_To_Hex(INTEGER_32 long_Value
								, Str255 hex_Result_String);

extern ASCII_Character *Convert_Cardinal_32_To_String(CARDINAL_32 long_Value
								, ASCII_Character *long_String
								, CARDINAL_8 field_Size);

extern ASCII_Character *Convert_Long_To_String(INTEGER_32 long_Value
								, ASCII_Character *long_String
								, CARDINAL_8 field_Size);
								
extern ASCII_Character *Convert_Double_To_String(DIGITS_SPEED_REAL double_Value
									, CARDINAL_8 field_Size
									, CARDINAL_8 decimal_Places
									, ASCII_Character *double_String);
									
extern ASCII_Character *Convert_Boolean_To_String(Boolean boolean_Value
										, ASCII_Character *boolean_String);
										
extern INTEGER_32 Convert_String_To_Long(ASCII_Character *long_String);


extern Safe_Boolean Convert_String_To_Cardinal_32(ASCII_Character *string_Ptr
													, CARDINAL_32 *cardinal_32);

extern Safe_Boolean Convert_String_To_Cardinal_16(ASCII_Character *string_Ptr
													, CARDINAL_16 *cardinal_16_Ptr);

extern Safe_Boolean Convert_String_To_Cardinal_8(ASCII_Character *string_Ptr
													, ASCII_Character *cardinal_8_Ptr);

extern DIGITS_SPEED_REAL Convert_String_To_Double(ASCII_Character *double_String);

extern void Copy_Like_Pascal_String(ASCII_Character *source_String
									, ASCII_Character *destination_String);

extern void Copy_Pascal_String(Str255 source_String
									, Str255 destination_String);

extern void Concat_After_Pascal_String(Str255 pascal_String
									, Str255 pascal_String_To_Concat);

extern void Concat_Before_Pascal_String(Str255 pascal_String_To_Concat
										, Str255 pascal_String);

extern void Concat_String(Str255 pascal_String
					, ASCII_Character *pascal_String_To_Concat);

extern void Concat_Long_To_String(Str255 pascal_String
								, INTEGER_32 long_Value
								, INTEGER_16 field_Size);

extern void Concat_Many( Str255  result_String, INTEGER_32 total_Strings, ...);

extern void Concat_Cardinal_To_String(Str255 pascal_String
								, CARDINAL_32 cardinal_Value
								, CARDINAL_8 field_Size);

extern void Concat_Double_To_String(Str255 pascal_String
								, DIGITS_SPEED_REAL double_Value
								, INTEGER_16 field_Size
								, INTEGER_16 decimal_Places);

extern void Clear_String(Str255 string_To_Clear);

extern void Assign_String(Str255 string_To_Assign
							, ASCII_Character *pascal_String);

extern void Make_String_Long_String(Str255 string_To_Make
								, ASCII_Character *first_String
								, INTEGER_32 long_Value
								, INTEGER_16 field_Size
								, ASCII_Character *second_String);

void Make_String_Cardinal_String(Str255 string_To_Make
								, ASCII_Character *first_String
								, CARDINAL_32 cardinal_Value
								, CARDINAL_8 field_Size
								, ASCII_Character *second_String);

extern void Make_String_Double_String(Str255 string_To_Make
								, ASCII_Character *first_String
								, DIGITS_SPEED_REAL double_Value
								, INTEGER_16 field_Size
								, INTEGER_16 decimal_Places
								, ASCII_Character *second_String);

extern void Make_String_Hex_String(Str255 string_To_Make
								, ASCII_Character *first_String
								, CARDINAL_8 *memory
								, CARDINAL_8 number_Of_Bytes
								, Boolean use_Spaces
								, ASCII_Character *second_String);

extern ASCII_Character *Array_To_String(ASCII_Character *array_Ptr
						, CARDINAL_8 array_Size
						, ASCII_Character *string);

extern void Delete_Characters(ASCII_Character *string
							, CARDINAL_8 position
							, CARDINAL_8 number);

extern ASCII_Character *Strip_Outside_Controls(ASCII_Character *string);

extern void Zero_Pad_String(ASCII_Character *string_To_Pad
						, CARDINAL_8 pad_Length);

extern Str255 *Truncate_String_To_Field(Str255 string_To_Truncate
										, CARDINAL_8 field_Length);

extern ASCII_Character *Justify_String_Right(ASCII_Character *string
							, CARDINAL_8 field_Size);
extern ASCII_Character *Justify_String_Left(ASCII_Character *string
							, CARDINAL_8 field_Size);
							
extern void Get_String_Centered_Rectangle(INTEGER_16 horizontal_Center_Position
										, INTEGER_16 vertical_Position
										, Str255 message
										, Rect *enclosing_Rectange);

extern void Draw_String_Centered(INTEGER_16 horizontal_Center_Position
								, INTEGER_16 vertical_Position
								, Str255 message
								, Rect *enclosing_Rectangle);
												
extern void Get_NS_String(CARDINAL_16	list_ID
						, CARDINAL_16 index_Number
						, Str255 returned_String);
									
extern Safe_Boolean Compare_Pascal_Strings_Ignoring_Case(ASCII_Character *first_String
											, ASCII_Character *second_String);

extern Safe_Boolean Compare_Pascal_Strings(ASCII_Character *first_String
											, ASCII_Character *second_String);

extern INTEGER_16 Compare_Pascal_Strings_Fully(ASCII_Character *first_Input_String
												, ASCII_Character *second_Input_String);

extern void Increment_Numerical_Suffix(ASCII_Character *new_String);

extern void Truncate_Without_Numerical_Suffix(ASCII_Character *new_String);

/* [----END----] PROTOTYPE AREA */
/*=**************************************************************************=*/


/*=**************************************************************************=*/
/* [---BEGIN---] EXTERN DECLARATION AREA */

extern Memory_Allocation			hex_Table_Memory_In_Use;

/* [----END----] EXTERN DECLARATION AREA */
/*=**************************************************************************=*/


/*=**************************************************************************=*/
#endif
/*=**************************************************************************=*/
/* END OF FILE: String_Utilities.h */
