/* START OF FILE: Graphics_Primitives.h */


/*=**************************************************************************=*/
/* [---BEGIN---] FILE HEADER COMMENTS */
/**
 ** SCREEN VIEW = [Adobe Syntax Black-10, Tabs = 6]
 ** PRINTER VIEW = [Adobe Syntax Black-10, Tabs = 6]
 ** =======================================================================
 ** Written by Jim Merkle
 ** Graphics_Primitives.h - Header for home-brewed graphics routines.
 ** 1990 by Jim Merkle.
 ** =======================================================================
 **
 **
 ** ============
 ** Revision History
 ** ============
 ** SEE xxxx.c for revision history and version number.
 **
 ** =================
 ** OTHER INFORMATION
 ** =================
 ** 
 **
 **/
/* [----END----] FILE HEADER COMMENTS */
/*=**************************************************************************=*/


/*=**************************************************************************=*/
#ifndef _H_Graphics_Primitives

#define _H_Graphics_Primitives
/*=**************************************************************************=*/


/*=**************************************************************************=*/
/* [---BEGIN---] FILE INCLUDE AREA */

#include "Shell_Globals.h"
#include "Off_Screen_Color.h"

/* [----END----] FILE INCLUDE AREA */
/*=**************************************************************************=*/


/*=**************************************************************************=*/
/* [---BEGIN---] DEFINE AREA */

  #define	  MAX_BIT_PATTERN_MASKS   (4096)

 /* since one of my monitors (Viking 24") is 1288 pixels wide, I should at least allow another 50% */
 #define	MAXIMUM_SCREEN_WIDTH		1920
 
 /* 10 390 pixel picts in one tall offscreen port is 3900 */
 /* until 1994.09 was just 4400 */
 /* For no limits allow (32766 - 200) at least */
 /* HACK HACK HACK For quickdraw emulation */
 /* at highspeed set to 1/4 32k or 4400 to keep table in 32K */
 #define	MAXIMUM_SCREEN_HEIGHT		32566
 
 /* MAXIMUM_SAFETY_AREA is for programmers who misjudge non-clipped graphics boundaries */
  #define	MAXIMUM_SAFETY_AREA			200
 
 /* MAXIMUM_DEFAULT_POLYGON_MEMORY is for lazy memory wasting usage of default "Polygon_List" structure */
  #define	 MAXIMUM_DEFAULT_POLYGON_MEMORY  300
  
 /* BOUNDED , and UNBOUNDED are calling parameters used by Draw_Framed_Polygon */
  #define	  BOUNDED		SAFE_TRUE
  #define	  UNBOUNDED	SAFE_FALSE
  
  /* for "Place_Patterned_Data" : Four templates per combination,   32 start positions, 32 lengths */
  
  /* ((4L) * (32L) * (32L)) */
  
  
  /* [----END----] DEFINE AREA */
/*=**************************************************************************=*/


/*=**************************************************************************=*/
/* [---BEGIN---] TYPE AREA */

/* these tables are created by Create_Graphics_Primitives_Data_Structures */
/* these tables are filled by Reset_Current_Window_Structures */
/* these tables are destroyed by Destroy_Graphics_Primitives_Data_Structures */

typedef struct {
		INTEGER_32		safety_Low_Clipping_Area[MAXIMUM_SAFETY_AREA]; /* for bad indexing */
		INTEGER_32		Pixel_Offset_Table[MAXIMUM_SCREEN_WIDTH];
		INTEGER_32		safety_High_Clipping_Area[MAXIMUM_SAFETY_AREA]; /* for bad indexing */
	} Horizontal_Table,*Horizontal_Table_Ptr;

typedef struct {
		Ptr				safety_Low_Clipping_Area[MAXIMUM_SAFETY_AREA]; /* for bad indexing */
		Ptr				Pixel_Address_Table[MAXIMUM_SCREEN_HEIGHT];
		Ptr				safety_High_Clipping_Area[MAXIMUM_SAFETY_AREA]; /* for bad indexing */
	} Vertical_Table,*Vertical_Table_Ptr;

typedef struct {
		INTEGER_32	total_Polygons;
		INTEGER_32	total_Offset_Size;
		INTEGER_16	polygon_Data[MAXIMUM_DEFAULT_POLYGON_MEMORY];
	} Polygon_List,*Polygon_List_Ptr;

typedef struct {
		CARDINAL_32	pattern_Data[MAX_BIT_PATTERN_MASKS]; 
	} fill_Pattern_Data_Table,*fill_Pattern_Data_Table_Ptr;

/* [----END----] TYPE AREA */
/*=**************************************************************************=*/


/*=**************************************************************************=*/
/* [---BEGIN---] PROTOTYPE AREA */

extern Boolean Determine_Window_Obscured(GrafPtr port);

extern Boolean Create_Graphics_Primitives_Data_Structures( void );
extern Boolean Destroy_Graphics_Primitives_Data_Structures(void);
extern Boolean Initialize_Graphics_Primitives(Boolean	is_Port_In_Window , Boolean is_Port_Offscreen_Memory);
extern Boolean Reset_Current_Window_Structures(Boolean is_Port_In_Window, Boolean is_Port_Offscreen_Memory);

extern void Set_Current_Color_Primitive(RGBColor	*desired_Color);
extern void Draw_Line_To(INTEGER_16 new_X, INTEGER_16 new_Y);
extern void Old_Draw_Clipped_Line_To(INTEGER_16 new_X, INTEGER_16 new_Y, Rect *bounding_Rectangle);

extern void Draw_Copied_Bits(const BitMap *srcMap
				, const BitMap *destMap
				, Rect *srcRect
				, Rect *destRect
				, INTEGER_16 tMode
				, RgnHandle maskRgn);

extern void Draw_Clipped_Line_To(INTEGER_16 new_X, INTEGER_16 new_Y, Rect *bounding_Rectangle);
extern void Draw_Framed_Polygon(Polygon_List_Ptr current_Polygon_List, Safe_Boolean bounded, Rect *bounding_Rectangle);extern void Draw_Filled_Rectangle(Rect *element_Bounding_Rectangle, PixPatHandle current_Pattern);
extern void Draw_Ellipse(Rect *element_Bounding_Rectangle);
extern void Draw_Circle(Rect *element_Bounding_Rectangle);
extern void Draw_Filled_Circle(Rect *element_Bounding_Rectangle, PixPatHandle current_Pattern);
extern void Draw_Filled_Ellipse(Rect *element_Bounding_Rectangle, PixPatHandle current_Pattern);
extern void Draw_Filled_Arc(Rect *element_Bounding_Rectangle, CARDINAL_16 start_Angle, CARDINAL_16 arc_Angle, PixPatHandle current_Pattern);

extern void Get_Pixel_Index(INTEGER_32 horizontal_Coordinate
						, INTEGER_32 vertical_Coordinate
						, CARDINAL_32 *fetched_Color);

extern pascal void Get_32bits_Of_8_Bit_Pixel_Subprimitive(INTEGER_32 horizontal_Coordinate
																, INTEGER_32 vertical_Coordinate
																, CARDINAL_32 *fetched_Pixels);
																

extern Safe_Boolean Filter_Color_From_Image_Completely(Off_Screen_Ptr offscreen_Ptr
															, RGBColor *color_to_Kill
															, CARDINAL_32 index_To_Kill
															, Safe_Boolean use_Direct_Color_For_Input
															, Rect *offscreen_Rect);

/*  */
/* Helper routine */
/*  */
pascal void Get_Color_Data_Pixel_32_Bit_Subprimitive(INTEGER_32 horizontal_Coordinate
																, INTEGER_32 vertical_Coordinate
																, RGBColor *fetched_Color);

/* [----END----] PROTOTYPE AREA */
/*=**************************************************************************=*/


/*=**************************************************************************=*/
/* [---BEGIN---] EXTERN DECLARATION AREA */

extern		GrafPtr						primitives_Current_Port;
extern		GDHandle						g_Primitives_Current_Device;
extern		RGBColor						primitives_Color_In_Use;

extern		Horizontal_Table_Ptr		primitives_Horizontal_Lookup_Table;
extern		INTEGER_32					*primitives_Horizontal_Lookup_Table_Direct;
extern		Vertical_Table_Ptr			primitives_Vertical_Lookup_Table;
extern		Ptr								*primitives_Vertical_Lookup_Table_Direct;

extern		SPEED_DIGITS_REAL			primitives_Current_Pixels_Per_Byte;	
extern		CARDINAL_16				primitives_Current_Screen_Depth;


/* [----END----] EXTERN DECLARATION AREA */
/*=**************************************************************************=*/


/*=**************************************************************************=*/
/* [---BEGIN---] LOCAL STATIC DEFINITION AREA */

/* [----END----] LOCAL STATIC DEFINITION AREA */
/*=**************************************************************************=*/


/*=**************************************************************************=*/
#endif
/*=**************************************************************************=*/
/* END OF FILE: Graphics_Primitives.h */
