/* START OF FILE: Off_Screen_Color.h */


/*=**************************************************************************=*/
/* [---BEGIN---] FILE HEADER COMMENTS */
/**
 ** SCREEN VIEW = [Adobe Syntax Black-10, Tabs = 4]
 ** PRINTER VIEW = [Adobe Syntax Black-10, Tabs = 4]
 ** =======================================================================
 ** Written by James Merkle
 ** Off_Screen_Color.h - header for Off_Screen_Color.c
 ** (C) Copyright 1993 by James Merkle
 ** =======================================================================
 **
 **
 ** ============
 ** Revision History
 ** ============
 **
 **  Refer to "Off_Screen_Color.c"
 **						
 **
 ** ====================
 ** OTHER INFORMATION
 ** ====================
 ** 
 **
 **/
/* [----END----] FILE HEADER COMMENTS */
/*=**************************************************************************=*/



/*=**************************************************************************=*/
#ifndef _H_Off_Screen_Color

#define _H_Off_Screen_Color
/*=**************************************************************************=*/


/*=**************************************************************************=*/
/* [---BEGIN---] #include AREA */

#include "Shell_Globals.h"

/* [----END----] #include AREA */
/*=**************************************************************************=*/


/*=**************************************************************************=*/
/* [---BEGIN---] #define AREA */

#define k_ORIGINAL_QUICKDRAW				1
#define k_COLOR_QUICKDRAW					2
#define k_32_BIT_COLOR_QUICKDRAW	3

 /*  */
/* Macro to get topleft, bottomright points of a Rect */
 /*  */
#define TOP_LEFT(r)					(((Point *) &(r))[0])
#define BOTTOM_RIGHT(r)		(((Point *) &(r))[1])

 /*  */
 /* Classic 3*4 ratio with more detail than icon */
 /*  */
#define k_PREVIEW_WIDTH_MAXIMUM		(64L)
#define k_PREVIEW_HEIGHT_MAXIMUM		(48L)

/* [----END----] #define AREA */
/*=**************************************************************************=*/


/*=**************************************************************************=*/
/* [---BEGIN---] TYPE AREA */


	/*  */
	/* By defining the graphics port stuff as a union, the  */
	/* memory we save is significant with these large structures. */
	/*  */
	union Union_Port_Rec {
		CGrafPort			color_Port;
		GrafPort				old_Style_Port;
	};

/*  */
/* Structure for managing offscreen stuff. */
/* B&W uses only a few fields (offscreen_Port, image_Handle, */
/* initialized, and color_Type) */
/*  */
 typedef struct {
	 Safe_Boolean					initialized;
	 union Union_Port_Rec		offscreen_Port;
	 GDHandle							offscreen_GDevice_Hdl;
	 Handle								image_Handle;
	 CTabHandle						color_Table_Handle;
	 CARDINAL_16					color_Type;
 } Off_Screen_Port_Rec, *Off_Screen_Ptr;


typedef enum {
	k_USE_RESOURCE_INFO = 1,
	k_USE_RECT_LOCATION
} fill_Technique;


/* [----END----] TYPE AREA */
/*=**************************************************************************=*/


/*=**************************************************************************=*/
/* [---BEGIN---] PROTOTYPE AREA */

extern CARDINAL_32 Get_Bit_Depth_Of_Primary_Device(void);


extern void Determine_Information_Of_Device_Containing_Rect(Rect *global_Rect_Passed
													, INTEGER_16		*video_Monitor_Driver_Ref
													, INTEGER_8		*video_Monitor_Slot
													, INTEGER_8		*video_Monitor_ID
													, CARDINAL_32	*video_Optional_Base_Address
													, INTEGER_32		*bits_Per_Pixel
													, INTEGER_32		*bits_Per_Pixel_Greatest
													, CARDINAL_16 	*total_Intersected_Screens
													, Safe_Boolean	*rect_Is_Within_A_Screen
													, PixMapHandle	*video_Pix_Map_Handle
													, GDHandle			*graphics_Device
													, Safe_Boolean	*slot_Information_Valid);
																									
Safe_Boolean Load_Pict_File_Into_Color_Bit_Map(FSSpec *file_Info
																		, CARDINAL_16	desired_Color_Depth
																		, CTabHandle		offscreen_Color_Table_Handle
																		, CGrafPtr	offscreen_Graph_Port
																		, GDHandle	*offscreen_GDevice_Ptr
																		, Handle		*image_Handle_Ptr);

extern void Determine_Bit_Depth_Of_Device_Containing_Rect(Rect *global_Rect
													, CARDINAL_16  *total_Intersected_Screens
													, INTEGER_32	 *bits_Per_Pixel
													, Safe_Boolean	*rect_Is_Within_A_Screen);
													
extern OSErr Create_Off_Screen_Port(Rect					*offscreen_Bounds
												, INTEGER_16		desired_Bit_Depth
												, CTabHandle		color_Table_Handle
												, CGrafPtr			graphics_Port
												, GDHandle			*created_Graphics_Device
												, Handle				*image_Handle_Ptr);


extern OSErr Setup_Pix_Map_Structure(INTEGER_16				desired_Bit_Depth
															, Rect						*offscreen_Bounds
															, CTabHandle			color_Table_Handle
															, INTEGER_16			bytes_Per_Row
															, PixMapHandle		initialized_Bit_Map_Handle
															, Handle					*image_Handle_Ptr);

extern OSErr Create_GDevice_Structure( PixMapHandle bit_Map_For_Device_Handle
	, GDHandle	*new_GDevice_Handle);

extern void Get_System_Screen_Data(Rect *global_Rect_Location
													, INTEGER_16		*video_Monitor_Driver_Ref
													, INTEGER_8		*video_Monitor_Slot
													, INTEGER_8		*video_Monitor_ID
													, CARDINAL_32	*video_Optional_Base_Address
													, PixMapHandle	*video_Pix_Map_Handle
													, GDHandle			*video_Graphics_Device_Handle
													, INTEGER_32		*bits_Per_Pixel
													, Safe_Boolean	*slot_Information_Valid
													, Safe_Boolean	*something_Found);

extern void Dispose_Off_Screen_Port(CGrafPtr doomedPort
												, GDHandle doomedGDevice);

Safe_Boolean	Create_Clut_By_Stealing_From_Color_Port(PixMapHandle	video_Pix_Map_Handle
																				, CTabHandle *created_Clut);

extern Safe_Boolean Load_Pict_Into_BW_Bit_Map(INTEGER_16 source_Pict_Id
																		, GrafPort	*offscreen_Graph_Port
																		, Handle		*image_Handle_Ptr);
																		
extern Safe_Boolean Load_Pict_Into_Color_Bit_Map(INTEGER_16 source_Pict_Id
																		, CARDINAL_16	desired_Color_Depth
																		, CTabHandle		offscreen_Color_Table_Handle
																		, CGrafPtr	offscreen_Graph_Port
																		, GDHandle	*offscreen_GDevice_Ptr
																		, Handle		*image_Handle_Ptr);

Safe_Boolean Copy_Rect_Into_Color_Bit_Map(Rect *source_Rect
																		, CARDINAL_16	desired_Color_Depth
																		, CTabHandle		offscreen_Color_Table_Handle
																		, CGrafPtr	offscreen_Graph_Port
																		, GDHandle	*offscreen_GDevice_Ptr
																		, Handle		*image_Handle_Ptr);
																		
extern Safe_Boolean Copy_Rect_Into_BW_Bit_Map(Rect *source_Rect
																		, GrafPort	*offscreen_Graph_Port
																		, Handle		*image_Handle_Ptr);
																		
extern Safe_Boolean	Close_Offscreen_Pict_Port(Off_Screen_Ptr	offscreen);

extern void Lock_Video_Memory_For_Lazy_But_Safe_Use(Off_Screen_Ptr offscreen_Ptr);

extern Safe_Boolean	Copy_Off_Screen_Port(Off_Screen_Ptr		existing_Offscreen_Ptr
																, Off_Screen_Ptr offscreen_Ptr);

extern Safe_Boolean	Load_Pict_Into_Bit_Map(Off_Screen_Ptr offscreen_Ptr
														, INTEGER_16 pict_Resource_Id);


extern Safe_Boolean	Get_Data_Into_Bit_Map(Off_Screen_Ptr offscreen_Ptr
													, fill_Technique desired_Fill
													, INTEGER_16 pict_Resource_Id
													, Rect *source_Rect);
													
extern Safe_Boolean	Copy_Rect_Into_Bit_Map(Off_Screen_Ptr offscreen_Ptr
													, Rect *source_Rect);

Safe_Boolean	Get_Picts_Into_One_Bit_Map(Off_Screen_Ptr offscreen_Ptr
													, INTEGER_16 first_Pict_Id
													, INTEGER_16 total_Picts
													, Rect *frame_Locations
													, Rect *governing_Rect
													, CARDINAL_32	expected_Total_Pixel_Height
													, CARDINAL_32	expected_Total_Depth_If_Not_Main_Screen
													, Rect *expected_Target_Rect);

extern void Find_A_8_Bit_Non_System_Screen(Rect *global_Rect_Location
													, INTEGER_16		*video_Monitor_Driver_Ref
													, INTEGER_8		*video_Monitor_Slot
													, INTEGER_8		*video_Monitor_ID
													, CARDINAL_32	*video_Optional_Base_Address
													, PixMapHandle	*video_Pix_Map_Handle
													, GDHandle			*video_Graphics_Device_Handle
													, INTEGER_32		*bits_Per_Pixel
													, Safe_Boolean	*slot_Information_Valid
													, Safe_Boolean	*other_Screens_Found_Not_8_Bit
													, Safe_Boolean	*something_Found);
													
extern Safe_Boolean	Create_Clut_For_Direct_Mapping(CARDINAL_16 desired_Depth
																				, CTabHandle *created_Clut);

extern void Restore_Colors_Palette(void);

extern void Restore_Screen_Colors_To_Environment(void);

extern void Set_Color_Match_For_Pixel_Copy(Boolean set_Match, PixMapHandle pixel_Map);
						
extern Safe_Boolean Guarantee_File_Is_PICT_Type(INTEGER_16 source_Volume_Reference
						, INTEGER_32 source_Directory_ID
						, Str63 source_File_Name
						, INTEGER_32 *source_Bit_Width
						, INTEGER_32 *source_Bit_Height);

extern Safe_Boolean Guarantee_File_Is_PICT_Type_And_Parse_Depth(INTEGER_16 source_Volume_Reference
						, INTEGER_32 source_Directory_ID
						, Str63 source_File_Name
						, INTEGER_32 *source_Bit_Width
						, INTEGER_32 *source_Bit_Height
						, Safe_Boolean parse_For_Color_Depth
						, CARDINAL_32	*source_Bit_Color_Depth);

extern Safe_Boolean Copy_Bitmap_Into_Pict_Command(Off_Screen_Port_Rec *port_Ptr
																, Rect *source_Bit_Rect_Ptr
																, INTEGER_32		desired_Color_Depth
																, PicHandle			*picture_Handle
																, Boolean			tiny_Preview_Size_Desired);

Safe_Boolean Fetch_Extra_Video_Driver_Info(INTEGER_16 video_Monitor_Driver_Ref
																, CARDINAL_32 *video_Optional_Base_Address
																, INTEGER_8 *video_Monitor_Slot
																, INTEGER_8 *video_Monitor_ID);

/* [----END----] PROTOTYPE AREA */
/*=**************************************************************************=*/


/*=**************************************************************************=*/
/* [---BEGIN---] extern DECLARATIONS */



/* [----END----] extern DECLARATIONS */
/*=**************************************************************************=*/


/*=**************************************************************************=*/
/* [---BEGIN---] LOCAL STATIC DEFINITION AREA */


/* [----END----] LOCAL STATIC DEFINITION AREA */
/*=**************************************************************************=*/


/*=**************************************************************************=*/
#endif
/*=**************************************************************************=*/
/* END OF FILE: Off_Screen_Color.h */