/* START OF FILE: Ring_Buffer_Cache_Setup.h */


/*=**************************************************************************=*/
/* [---BEGIN---] FILE HEADER COMMENTS */
/**
 ** SCREEN VIEW = [Adobe Syntax Black-10, Tabs = 4]
 ** PRINTER VIEW = [Adobe Syntax Black-10, Tabs = 4]
 ** =======================================================================
 ** Written by James Merkle
 ** Ring_Buffer_Cache_Setup.h - Header for Ring_Buffer_Cache_Setup.c
 ** (C) Copyright 1994 James Merkle
 ** =======================================================================
 **
 **
 ** ============
 ** Revision History
 ** ============
 **
 **  Refer to "Ring_Buffer_Cache_Setup.c"
 **						
 **
 ** ====================
 ** OTHER INFORMATION
 ** ====================
 ** 
 **
 **/
/* [----END----] FILE HEADER COMMENTS */
/*=**************************************************************************=*/



/*=**************************************************************************=*/
#ifndef _H_Ring_Buffer_Cache_Setup

#define _H_Ring_Buffer_Cache_Setup
/*=**************************************************************************=*/


/*=**************************************************************************=*/
/* [---BEGIN---] #include AREA */

#include "Shell_Globals.h"

/* [----END----] #include AREA */
/*=**************************************************************************=*/


/*=**************************************************************************=*/
/* [---BEGIN---] #define AREA */

#define k_UNUSED_ENTRY										(0xFFFFFFFF)
#define k_MAX_CACHE_ENTRIES								(361L)

/* [----END----] #define AREA */
/*=**************************************************************************=*/


/*=**************************************************************************=*/
/* [---BEGIN---] TYPE AREA */

typedef struct {
	/*  */
	/* An offset of "0" is equal to beginning of ring buffer, but */
	/* an offset of "FFFFFFFF" is equal to NOT VALID */
	/* Vital that this structure be long aligned!!! */
	/*  */
	CARDINAL_32				offset;
	CARDINAL_32				composite_Search_Key; /* puppet_Source_Bin, activation, X, Y */
	CARDINAL_32				secondary_Search_Key; 
	CARDINAL_32				total_Size_Stored;
} Cache_Tag;

typedef struct {
	Safe_Boolean					actively_Caching;
	Safe_Boolean					temp_Memory_Suggested;
	
	Handle						file_Cache_Handle;
	CARDINAL_8					*file_Cache_Ptr;
	Memory_Allocation			file_Cache_Memory_Initialized;
	Safe_Boolean					file_Cache_Is_Temp_Memory;
	
	CARDINAL_32				total_Cache_FIFO_Entries;
	CARDINAL_32				max_Cache_FIFO_Entries;
	CARDINAL_32				head_FIFO_Entry_For_Addition;
	CARDINAL_32				tail_FIFO_Entry_For_Deletion;
	
	CARDINAL_32				max_Ring_Cache_Size;
	CARDINAL_32				ring_Cache_Head_Next_Byte;
	CARDINAL_32				ring_Cache_Tail_Byte;
	
	Cache_Tag					cache_Entries[k_MAX_CACHE_ENTRIES];
} Ring_Buffer_Cache_Rec, *Ring_Buffer_Cache_Ptr,**Ring_Buffer_Cache_Hdl;

/* [----END----] TYPE AREA */
/*=**************************************************************************=*/


/*=**************************************************************************=*/
/* [---BEGIN---] PROTOTYPE AREA */

extern Safe_Boolean Initialize_Ring_Buffer_Cache_Setup(void);

extern Safe_Boolean Destroy_Ring_Buffer_Cache_Setup(void);

extern Safe_Boolean Create_New_Ring_Cache(Ring_Buffer_Cache_Hdl *data_Handle
								, Memory_Allocation *allocated
								, Safe_Boolean use_Temporary_Memory);

extern Safe_Boolean Reset_And_Open_Ring_Buffer_If_Needed(Ring_Buffer_Cache_Ptr cache_Information_Ptr
													, CARDINAL_32 minimum_Extra_Temp_Ram_Remaining
													, CARDINAL_32 maximum_Acceptable_Cache
													, CARDINAL_32 minimum_Acceptable_Cache
													, CARDINAL_32 moderately_Acceptable_Cache
													, CARDINAL_32 minimum_Extra_App_Ram_Remaining);

extern Safe_Boolean Close_Down_Ring_Cache_If_Possible(Ring_Buffer_Cache_Ptr cache_Information_Ptr);

extern Safe_Boolean Destroy_Ring_Buffer_Cache(Ring_Buffer_Cache_Hdl *data_Handle
												, Memory_Allocation *allocated);

/* [----END----] PROTOTYPE AREA */
/*=**************************************************************************=*/


/*=**************************************************************************=*/
/* [---BEGIN---] extern DECLARATIONS */


/* [----END----] extern DECLARATIONS */
/*=**************************************************************************=*/


/*=**************************************************************************=*/
/* [---BEGIN---] LOCAL STATIC DEFINITION AREA */


/* [----END----] LOCAL STATIC DEFINITION AREA */
/*=**************************************************************************=*/


/*=**************************************************************************=*/
#endif
/*=**************************************************************************=*/
/* END OF FILE: Ring_Buffer_Cache_Setup.h */
