/* START OF FILE: Simple_Ring_Cache.h */


/*=**************************************************************************=*/
/* [---BEGIN---] FILE HEADER COMMENTS */
/**
 ** SCREEN VIEW = [Adobe Syntax Black-10, Tabs = 4]
 ** PRINTER VIEW = [Adobe Syntax Black-10, Tabs = 4]
 ** =======================================================================
 ** Written by James Merkle
 ** Simple_Ring_Cache.c - Routines to support a primitive ring cache of fixed size items
 ** (C) Copyright 1995 James Merkle
 ** =======================================================================
 **
 **
 ** ============
 ** Revision History
 ** ============
 **
 **  Refer to "Simple_Ring_Cache.c"
 **						
 **
 ** ====================
 ** OTHER INFORMATION
 ** ====================
 ** 
 **
 **/
/* [----END----] FILE HEADER COMMENTS */
/*=**************************************************************************=*/



/*=**************************************************************************=*/
#ifndef _H_Simple_Ring_Cache

#define _H_Simple_Ring_Cache
/*=**************************************************************************=*/


/*=**************************************************************************=*/
/* [---BEGIN---] #include AREA */

#include "Shell_Globals.h"

/* [----END----] #include AREA */
/*=**************************************************************************=*/


/*=**************************************************************************=*/
/* [---BEGIN---] #define AREA */

/*  */
/* Power of 2 needed. Entire list is always scanned so keep short */
/* It must also be less than 32K!! (otherwise mainsearch loop needs modification)*/
/*  */
#define k_MAX_CACHE_ENTRIES_POWER_2		(64)

/* [----END----] #define AREA */
/*=**************************************************************************=*/


/*=**************************************************************************=*/
/* [---BEGIN---] TYPE AREA */

typedef struct {
	CARDINAL_32				secondary_Search_Key; 
	CARDINAL_32				cached_Data;
} Cache_Info;

typedef struct {
	Safe_Boolean					structure_Initialized;
	CARDINAL_32				max_Cache_FIFO_Entries;
	CARDINAL_32				max_Entry_Bit_Mask;
	CARDINAL_32				head_FIFO_Entry_For_Addition;
	CARDINAL_32				composite_Search_Key[k_MAX_CACHE_ENTRIES_POWER_2];
	Cache_Info					cache_Entries[k_MAX_CACHE_ENTRIES_POWER_2];
} Simple_Ring_Buffer_Cache_Rec, *Simple_Ring_Buffer_Cache_Ptr;

/* [----END----] TYPE AREA */
/*=**************************************************************************=*/


/*=**************************************************************************=*/
/* [---BEGIN---] PROTOTYPE AREA */

extern Safe_Boolean Initialize_Simple_Ring_Cache(void);

extern Safe_Boolean Destroy_Simple_Ring_Cache(void);

extern void Clear_Simple_Ring_Cache_Entries(Simple_Ring_Buffer_Cache_Ptr cache_Master_Ptr);

extern Safe_Boolean Locate_Best_Frame_From_Simple_Ring_Cache(CARDINAL_32 simple_Key_1
																	, CARDINAL_32 simple_Key_2
																	, CARDINAL_32 *cached_Data);
																	
extern Safe_Boolean Record_Frame_Into_Simple_Ring_Cache(CARDINAL_32 simple_Key_1
																, CARDINAL_32 simple_Key_2
																, CARDINAL_32 index);


/* [----END----] PROTOTYPE AREA */
/*=**************************************************************************=*/


/*=**************************************************************************=*/
/* [---BEGIN---] extern DECLARATIONS */


/* [----END----] extern DECLARATIONS */
/*=**************************************************************************=*/


/*=**************************************************************************=*/
/* [---BEGIN---] LOCAL STATIC DEFINITION AREA */


/* [----END----] LOCAL STATIC DEFINITION AREA */
/*=**************************************************************************=*/


/*=**************************************************************************=*/
#endif
/*=**************************************************************************=*/
/* END OF FILE: Simple_Ring_Cache.h */