/* START OF FILE: Sound_Record.h */


/*=**************************************************************************=*/
/* [---BEGIN---] FILE HEADER COMMENTS */
/**
 ** SCREEN VIEW = [Adobe Syntax Black-10, Tabs = 4]
 ** PRINTER VIEW = [Adobe Syntax Black-10, Tabs = 4]
 ** =======================================================================
 ** Written by James Merkle
 ** Sound_Record.c - Routines to support Spin-O-Rama
 ** (C) Copyright 1994 James Merkle
 ** =======================================================================
 **
 **
 ** ============
 ** Revision History
 ** ============
 **
 **  Refer to "Sound_Record.c"
 **						
 **
 ** ====================
 ** OTHER INFORMATION
 ** ====================
 ** 
 **
 **/
/* [----END----] FILE HEADER COMMENTS */
/*=**************************************************************************=*/



/*=**************************************************************************=*/
#ifndef _H_Sound_Record

#define _H_Sound_Record
/*=**************************************************************************=*/


/*=**************************************************************************=*/
/* [---BEGIN---] #include AREA */


/* [----END----] #include AREA */
/*=**************************************************************************=*/


/*=**************************************************************************=*/
/* [---BEGIN---] #define AREA */


/* [----END----] #define AREA */
/*=**************************************************************************=*/

/*  */
/* use 2 64K buffers for sound recording. The buffers are sent */
/* as async requests for extra speed but we do not want to tie */
/* up hard disk incomming write buffers with requests larger than 64K. */
/* Disk I/O is eventiually in phase with 512 byte writes */
/* but for RAID array to prevent REad/write it might be better to align */
/* on 2K, 4K or 32K etc. but then each buffer would not always be filled. */
/*  */
#define k_TOTAL_SOUND_BUFFERS						2
#define k_MINIMUM_BUFFER_RESOLUTION		(64UL * 1024UL)

/*  */
/* You could use a smaller (1% to 25% smaller) cutoff point for */
/* interpreting a buffer as ready to dump but you shoul make it a */
/* divisor of 512 for disk use. You should also think about */
/* trying to play catch-up so that sound recording is aligned to */
/* 512 bytes after the odd sized AIFF header is written. This is easy to do */
/* in the fill routine especially if the optimal ratio is at least 512 bytes */
/* leeway. This is because the interrupt callback for an average SID */
/* (Sound Input Device) is bound to be a divisor of 1024 but an odd */
/* modulo lot of 512 or less is desired to be placed in another buffer at the */
/* start so that the file becomes aligned. This is achieved best in the first */
/* buffer addition but then the first buffer write would not be an optimal size. */
/* Therfore it is best to do at first buffer overflow/fill and transfer. */
/*  */
#define k_USEFUL_BUFFER_FILL_RATIO			(k_MINIMUM_BUFFER_RESOLUTION - 512UL)


#define	k_SWAP_BUFFERS								2

/*=**************************************************************************=*/
/* [---BEGIN---] TYPE AREA */

typedef enum {
	PRIMARY_SOUND_INPUT = 1
	, NATIONAL_INSTRUMENTS_DA
	, LOCAL_AUDIO_FILE_PLAYING
	, REMOTE_NETWORK_FEED
	, TOP_KEYBOARD_ROW
	, MIDI_INPUT
	, TELEBRIDGE_SID_HYBRID_INPUT
	, ANY_INPUT_AT_ALL
} Audio_Source_Type;


typedef struct {
	Safe_Boolean					initialized;
	Safe_Boolean					more_Recording_Desired;
	Safe_Boolean					stop_Recording_As_Soon_As_Possible;
	Safe_Boolean					data_Overflow_Corrupted_File;
	CARDINAL_32				maximum_Possible_File_Size;
	CARDINAL_32				maximum_Samples_To_Record;
	FSSpec						desired_File_Spec;
	INTEGER_16					file_Reference;
	CARDINAL_32				total_Samples_Written_To_Stream;
	CARDINAL_32				number_Of_Channels;
	CARDINAL_32				sample_Rate;
	CARDINAL_32				sample_Size;
	CARDINAL_32				compression_Type;
	CARDINAL_32				sound_File_Header_Size;
	
	/*  */
	/* An audio async ioBuffer can be in 4 common states */
	/* 1: It is openly accepting data still - internal_Double_Buffer_To_Use points to it
	/*    or internal_Double_Buffer_Total_Bytes is less than max possible size */
	/*   and internal_Double_Buffer_Pending_Write is not set to true. */
	/* 2: It is full waiting for dump to disk internal_Double_Buffer_Total_Bytes is equal to */
	/*    max size but internal_Double_Buffer_Pending_Write is not set to true */
	/* 3: It is in process of writing asyncronously : internal_Double_Buffer_Pending_Write is */
	/* set and ioResult is k_MAC_OS_ASYNC_BUSY_FLAG */
	/* 4 : It is finished writing but needs fags flipped to ready to fill state. */
	/*    in this case internal_Double_Buffer_Pending_Write is still set true even though */
	/* ioResult is no longer set to k_MAC_OS_ASYNC_BUSY_FLAG */
	/*  */
			
	CARDINAL_32				internal_Double_Buffer_To_Use;
	CARDINAL_32				internal_Double_Buffer_Total_Bytes[k_TOTAL_SOUND_BUFFERS];
	Safe_Boolean					internal_Double_Buffer_Pending_Write[k_TOTAL_SOUND_BUFFERS];
	IOParam						internal_Double_Async_Params[k_TOTAL_SOUND_BUFFERS];
	CARDINAL_8					internal_Double_Buffer_Storage[k_TOTAL_SOUND_BUFFERS][k_MINIMUM_BUFFER_RESOLUTION];
	
} Record_Sound_To_File_Rec, *Record_Sound_To_File_Ptr, **Record_Sound_To_File_Hdl;


 


 typedef struct {
	Safe_Boolean						initialized;
	INTEGER_16						sample_Size;
	
	CARDINAL_32					non_Vital_global_A5_World;
	
	volatile	Safe_Boolean		recording_Completed;
	CARDINAL_32					recording_Globals_Sentinel;
	CARDINAL_32					recording_Buffer_Odometer;
	
	CARDINAL_16					desired_Level_Meter_State;
	CARDINAL_16					previous_Level_Meter_State;
	OSType								compression_Type;
	OSType								previous_Compression_Type;
	Fixed									sample_Rate;
	Fixed									previous_Sample_Rate;
	Safe_Boolean						use_Specific_Channel_Mapping;
	CARDINAL_32					desired_Active_Channels;
	CARDINAL_32					previous_Activated_Channels;
	INTEGER_16						number_Of_Channels;
	INTEGER_16						previous_Activated_Channel_Total;
	Safe_Boolean						compression_Options_Altered;
	Safe_Boolean						twos_Complement_Storage_Option_Understood;
	INTEGER_16						twos_Complement_Storage_Option;
	INTEGER_16						original_Twos_Complement_Storage_Option;
	INTEGER_32						input_Reference;
	Str255									device_Name;
	INTEGER_16						asynchronous_Driver_Support_Option;
	CARDINAL_32					bytes_To_Record;
	CARDINAL_32					sound_Header_Size;
	
	/*  */
	/* These are for large recording double buffer cacpture/monitoring */
	/*  */
	
	SPB								sound_Recording_Block[k_SWAP_BUFFERS];
	Handle							recorded_Sound_Data_Handle[k_SWAP_BUFFERS];
	Memory_Allocation				recorded_Sound_Data_Memory_In_Use[k_SWAP_BUFFERS];
	CARDINAL_16					recorded_Sound_Data_Section;
	
	/*  */
	/* These are for time-accurate device-level double buffer capture/monitoring */
	/* (streaming to disk asyncronously via large double buffers) */
	/*  */
	
	Memory_Allocation				audio_Tapping_Structure_Initialized;
	Record_Sound_To_File_Hdl		audio_Tapping_Structure_Hdl;
	
	Safe_Boolean						record_To_New_Buffer_Directly_From_Completion;
 } Recording_Rec, *Recording_Ptr;


 typedef struct {
	Safe_Boolean						initialized;
	Safe_Boolean						tap_Possibly_Active;
	/*  */
	/* Warning : Never attach more than one item to an audio file */
	/* playback, and if required to do so, (multiple views of a scene) */
	/* then ensure that anything pointing to it is acceptable to swap back */
	/* with this one arbitrary "on standby" voice */
	/*  */

	Safe_Boolean						previous_Open_Voice_Index_Valid;
	CARDINAL_32					previous_Open_Voice_Index;
	CARDINAL_32					playback_Index;
 } Aufio_File_Rec, *Aufio_File_Ptr;

 typedef struct {
	Safe_Boolean						initialized;
	
/* ==
	Handle								recorded_Sound_Data_Handle;
	Memory_Allocation			recorded_Sound_Data_Memory_In_Use;
=== */
 } Sampler_Rec, *Sampler_Ptr;

 typedef struct {
	Safe_Boolean						initialized;
	
/* ===
	Handle								recorded_Sound_Data_Handle;
	Memory_Allocation			recorded_Sound_Data_Memory_In_Use;
=== */
 } Network_Rec, *Network_Ptr;

 typedef struct {
	Safe_Boolean						initialized;
	
/* ===
	Handle								recorded_Sound_Data_Handle;
	Memory_Allocation			recorded_Sound_Data_Memory_In_Use;
=== */
 } Manual_Rec, *Manual_Ptr;

union Union_Record{
	Recording_Rec		apple_Sound_Input;
	Sampler_Rec			sampler_Board;
	Aufio_File_Rec		audio_File;
	Network_Rec			remote;
	Manual_Rec			manual;
};

 typedef struct {
	Safe_Boolean						initialized;
	CARDINAL_32					current_Raw_Signal;
	CARDINAL_32					current_Interpolated_Signal;
	CARDINAL_32					signals_Dropping_Consecutively;
	
	CARDINAL_32					Max_Signal_Range;	
	SPEED_DIGITS_REAL			amplification_Response_Curve;	
	CARDINAL_32					maximum_Signal_Since_Last;
	CARDINAL_32					tickCount_Last_Updated;
	CARDINAL_16					audio_Source_Channel;
	Audio_Source_Type			audio_Source;
	union Union_Record			info;
 } Voice_Rec, *Voice_Ptr, **Voice_Hdl;


/* [----END----] TYPE AREA */
/*=**************************************************************************=*/


/*=**************************************************************************=*/
/* [---BEGIN---] PROTOTYPE AREA */

extern Safe_Boolean Initialize_Sound_Record(void);

extern Safe_Boolean Destroy_Sound_Record(void);

extern Safe_Boolean Start_Recording_To_New_AIFF_File(Record_Sound_To_File_Ptr record_Sound_Ptr
														, CARDINAL_32 maximum_Possible_File_Size
														, FSSpec	*desired_File_Spec
														, CARDINAL_32 number_Of_Channels
														, CARDINAL_32 sample_Rate
														, CARDINAL_32 sample_Size
														, CARDINAL_32 compression_Type);

extern Safe_Boolean Stop_Recording_To_New_AIFF_File(Record_Sound_To_File_Ptr record_Sound_Ptr);

extern Safe_Boolean Store_Possible_AIFF_Data_During_Interrupts(Record_Sound_To_File_Ptr record_Sound_Ptr
																, CARDINAL_32	desired_Buffer_Fill_Limit);

extern void Advance_Buffer_Marker_If_Possible(Record_Sound_To_File_Ptr record_Sound_Ptr, INTEGER_16 index);

extern Safe_Boolean Periodically_Save_To_New_AIFF_File_If_Needed(Record_Sound_To_File_Ptr record_Sound_Ptr);

extern Safe_Boolean Get_Next_AIFF_File_Specification( CARDINAL_32 expected_File_Minimum_Size
															, CARDINAL_32 *maximum_Possible_File_Size
															, FSSpec *direct_Disk_Recording_File_Ptr
															, Safe_Boolean	*user_Cancelled);
extern pascal Safe_Boolean Transfer_AIFF_Data_During_Interrupts(Record_Sound_To_File_Ptr record_Sound_Ptr
															, CARDINAL_32 source_Address
															, CARDINAL_32	 total_Remaining_Bytes);


/* [----END----] PROTOTYPE AREA */
/*=**************************************************************************=*/


/*=**************************************************************************=*/
/* [---BEGIN---] extern DECLARATIONS */

extern	volatile	Safe_Boolean		g_Severe_Audio_Record_Problem_Needs_Display_Soon;

/* [----END----] extern DECLARATIONS */
/*=**************************************************************************=*/


/*=**************************************************************************=*/
/* [---BEGIN---] LOCAL STATIC DEFINITION AREA */


/* [----END----] LOCAL STATIC DEFINITION AREA */
/*=**************************************************************************=*/


/*=**************************************************************************=*/
#endif
/*=**************************************************************************=*/
/* END OF FILE: Sound_Record.h */