/* START OF FILE: Sound_Tap_Channel.h */


/*=**************************************************************************=*/
/* [---BEGIN---] FILE HEADER COMMENTS */
/**
 ** SCREEN VIEW = [Adobe Syntax Black-10, Tabs = 4]
 ** PRINTER VIEW = [Adobe Syntax Black-10, Tabs = 4]
 ** =======================================================================
 ** Written by James Merkle
 ** Sound_Tap_Channel.h - Header file for Sound_Tap_Channel.c
 ** (C) Copyright 1995 James Merkle
 ** =======================================================================
 **
 **
 ** ============
 ** Revision History
 ** ============
 **
 **  Refer to "Sound_Tap_Channel.c"
 **						
 **
 ** ====================
 ** OTHER INFORMATION
 ** ====================
 ** 
 **
 **/
/* [----END----] FILE HEADER COMMENTS */
/*=**************************************************************************=*/



/*=**************************************************************************=*/
#ifndef _H_Sound_Tap_Channel

#define _H_Sound_Tap_Channel
/*=**************************************************************************=*/


/*=**************************************************************************=*/
/* [---BEGIN---] #include AREA */

#include <SoundComponents.h>

/* [----END----] #include AREA */
/*=**************************************************************************=*/


/*=**************************************************************************=*/
/* [---BEGIN---] #define AREA */

/*  */
/* Should be equal to number of monitored sounds (file playback etc) */
/*  */
#define k_SIMULTANEOUS_TAPS	3

/* [----END----] #define AREA */
/*=**************************************************************************=*/


/*=**************************************************************************=*/
/* [---BEGIN---] TYPE AREA */

	typedef struct {
		volatile CARDINAL_32		tick_Time_Last_Called;
		SoundComponentData		component_Recent_Info;
		volatile Safe_Boolean		data_untrusted_In_Transition;
		Handle						component_Recent_Buffer;
		Memory_Allocation			component_Recent_Buffer_Memory_In_Use;
		CARDINAL_32				component_Recent_Buffer_Max_Size;
	} Tap_Buffer_Rec, *Tap_Buffer_Ptr;

	typedef struct {
		volatile Safe_Boolean		active_And_In_Use;
		volatile CARDINAL_32		tick_Time_First_Setup;
		volatile Safe_Boolean		next_Buffer;
		volatile CARDINAL_32		tap_Buffer_Total_Samples_Counted;
		Tap_Buffer_Rec				tap_Buffer[2];		
		CARDINAL_32				user_Reference_Info;
	} Tap_Info_Rec, *Tap_Info_Ptr;
	
#if !defined(ORIGINAL_MAC_SYSTEM)
#error "UNDEFINED PARAMETER!"
#endif
#if (ORIGINAL_MAC_SYSTEM == 0)
	typedef struct {
		Tap_Info_Rec					info;
		volatile SoundParamUPP	original_Get_More_Data_UPP;
		
		SoundParamUPP				get_More_Data_Function_UPP; /* static memory structure! */
	} Audio_Progress_Tap_Rec, *Audio_Progress_Tap_Ptr;
#else
	typedef struct {
		Tap_Info_Rec					info;
		
		volatile ProcPtr				original_Get_More_Data_Function;
			
	} Audio_Progress_Tap_Rec, *Audio_Progress_Tap_Ptr;
#endif



/* [----END----] TYPE AREA */
/*=**************************************************************************=*/


/*=**************************************************************************=*/
/* [---BEGIN---] PROTOTYPE AREA */

extern Safe_Boolean Initialize_Sound_Tap_Channel(void);

extern Safe_Boolean Destroy_Sound_Tap_Channel(void);

extern Safe_Boolean Install_Tap_On_Get_More_Data(SndChannelPtr	active_Snd_Channel_Ptr
											, CARDINAL_32 user_Reference_Info
											, Audio_Progress_Tap_Ptr *resulting_Audio_Tap_Ptr);

extern Safe_Boolean Destroy_Audio_Tap(Audio_Progress_Tap_Ptr audio_Tap_Ptr);

extern Safe_Boolean Get_Current_Amplitude_From_Tap(Audio_Progress_Tap_Ptr audio_Tap_Ptr
														, CARDINAL_32 channel_Desired
														, CARDINAL_32 second_Ratio_For_Window
														, CARDINAL_8 *amplitude);
extern pascal void Set_Sound_Callback(SndChannelPtr active_Snd_Channel_Ptr, SndCommand *command);

extern Safe_Boolean Delay_Tap_Until_Good_Opportunity(Ptr	playback);

/* [----END----] PROTOTYPE AREA */
/*=**************************************************************************=*/


/*=**************************************************************************=*/
/* [---BEGIN---] extern DECLARATIONS */


/* [----END----] extern DECLARATIONS */
/*=**************************************************************************=*/


/*=**************************************************************************=*/
/* [---BEGIN---] LOCAL STATIC DEFINITION AREA */


/* [----END----] LOCAL STATIC DEFINITION AREA */
/*=**************************************************************************=*/


/*=**************************************************************************=*/
#endif
/*=**************************************************************************=*/
/* END OF FILE: Sound_Tap_Channel.h */