/* START OF FILE: Sound_Utilities.h */

/*=********************************************************************=*/
/* [---BEGIN---] FILE HEADER COMMENTS */
/**
 ** SCREEN VIEW = Adobe Syntax Black-10, Tabs=6 chars
 ** PRINTER VIEW = Adobe Syntax Black -10, Tabs=0.30 inch
 ** ========================================================================
 ** Written by James Merkle
 ** [Sound_Utilities.h] - Miscellaneous utilities related to digital sound recording and playback
 ** (C) Copyright 1992 James Merkle
 ** ========================================================================
 **
 **
 ** ============
 ** Revision History
 ** ============
 ** See Sound_Utilities.c for revision history and version number.
 **
 ** ===============
 ** OTHER INFORMATION
 ** ===============
 **			
 **
 **/
/* [----END----] FILE HEADER COMMENTS */
/*=**************************************************************************=*/


/*=**************************************************************************=*/
#ifndef _H_Sound_Utilities

#define _H_Sound_Utilities
/*=**************************************************************************=*/


/*=**************************************************************************=*/
/* [---BEGIN---] FILE INCLUDE AREA */

#include "Shell_Globals.h"
#include "Memory_Utilities.h"

/* [----END----] FILE INCLUDE AREA */
/*=**************************************************************************=*/


/*=**************************************************************************=*/
/* [---BEGIN---] DEFINE AREA */
#pragma mark DEFINE_AREA


/* [----END----] DEFINE AREA */
/*=**************************************************************************=*/


/*=**************************************************************************=*/
/* [---BEGIN---] TYPE AREA */
	
	typedef struct {
		INTEGER_16		volume_Reference_Num;			
		INTEGER_32		directory_ID;		
		Str63				file_Name;		
		INTEGER_16		file_Reference;				
	} File_Specification, *File_Specification_Ptr;

	typedef struct {
		/*=--------------------------------------------------------------------------=*/
		/* dangerous_Stack_Corruption_Handle will be set illegally! */
		/* Because "SPBGetDeviceInfo" can corrupt and write over adjacent */
		/* memory and is not properly documented in Inside Mac 6. */
		/* For example, when "SPBGetDeviceInfo" is called with "siCompressionAvailable" */
		/* It will corrupt the adjacent four bytes after the destination pointer by */
		/* trying to save a handle over it! Absolutely Idiotic ! Use NeXT Step soon. */
		/*=--------------------------------------------------------------------------=*/
		CARDINAL_32		buffer_Safety_Scrap_Memory_1;
		CARDINAL_32		buffer_Safety_Scrap_Memory_2;
		INTEGER_16			number_Of_Compression_Types;
		Handle				dangerous_Stack_Corruption_Handle;
		CARDINAL_32		buffer_Safety_Scrap_Memory_3;
		CARDINAL_32		buffer_Safety_Scrap_Memory_4;
		
		Memory_Allocation	dangerous_Stack_Corruption_Handle_Memory_In_Use;
	} Compression_Rec;


	typedef struct {
		/*=--------------------------------------------------------------------------=*/
		/* dangerous_Stack_Corruption_Handle will be set illegally! */
		/* Because "SPBGetDeviceInfo" can corrupt and write over adjacent */
		/* memory and is not properly documented in Inside Mac 6. */
		/* For example, when "SPBGetDeviceInfo" is called with "siSampleRate" */
		/* It will corrupt the adjacent four bytes before the destination pointer by */
		/* trying to save a handle over it! */
		/*=--------------------------------------------------------------------------=*/
		CARDINAL_32		buffer_Safety_Scrap_Memory_1;
		CARDINAL_32		buffer_Safety_Scrap_Memory_2;
		INTEGER_16			number_Of_Sample_Rates;
		Handle				dangerous_Stack_Corruption_Handle;
		CARDINAL_32		buffer_Safety_Scrap_Memory_3;
		CARDINAL_32		buffer_Safety_Scrap_Memory_4;
		
		Memory_Allocation	dangerous_Stack_Corruption_Handle_Memory_In_Use;
	} Sample_Rate_Rec;


/* [----END----] TYPE AREA */
/*=**************************************************************************=*/


/*=**************************************************************************=*/
/* [---BEGIN---] PROTOTYPE AREA */
#pragma mark PROTOTYPE_AREA

extern Safe_Boolean Create_Resource_Output_File(Handle resource_To_Create_Hdl
												, CARDINAL_32 file_Creator 
												, CARDINAL_32 file_Type
												, CARDINAL_32 resource_Type
												, INTEGER_16 resource_ID
												, Str255 default_File_Name
												, Str255 resource_Comment
												,  File_Specification_Ptr Direct_Disk_Recording_File_Ptr);

extern void Test_Sound_Input(void);

extern void Test_Playing_16_Bit_Stereo_AIFF(void);

/* [----END----] PROTOTYPE AREA */
/*=**************************************************************************=*/


/*=**************************************************************************=*/
/* [---BEGIN---] EXTERN DECLARATION AREA */

extern Safe_Boolean		recording_Completed_Global_Flag;
extern INTEGER_32		experimental_Global_Counter;

/* [----END----] EXTERN DECLARATION AREA */
/*=**************************************************************************=*/


/*=**************************************************************************=*/
/* [---BEGIN---] LOCAL STATIC DEFINITION AREA */

/* [----END----] LOCAL STATIC DEFINITION AREA */
/*=**************************************************************************=*/


/*=**************************************************************************=*/
#endif
/*=**************************************************************************=*/
/* END OF FILE: Sound_Utilities.h */
