/* [Buffered_File_Routines.c] START OF FILE */

/* [---BEGIN---] FILE HEADER COMMENTS */

/**
 ** SCREEN VIEW = Adobe Syntax Black-10, Tabs=6 chars
 ** PRINTER VIEW = Adobe Syntax Black -10, Tabs=0.30 inch
 ** ========================================================================
 ** Written by James Merkle
 ** [Buffered_File_Routines.c] - routines providing buffered files
 ** (C) Copyright 1990 by James Merkle
 ** All rights reserved, except those wishing to "copy" moderate sections of code or files may do so, so long as
 ** such code is never used directly either in animation-related tools or multimedia tools that are  
 ** used in live performance or live broadcast or recorded mass market broadcast or film. Desktop usage not restricted.
 ** Conventional desktop game or entertainment not restricted.
 ** ========================================================================
 **
 **
 ** =============
 ** Revision History
 ** =============
 **  90.02.27			(James Merkle)
 **						Initial coding started
 **
 **  90.05.09			(James Merkle)
 **						Slight modifications to buffer close routines for shutdown 
 **
 **  90.07.11			(James Merkle)
 **						Add test code to compare my routines vs Apple's OS.  (My overhead is 10 times less!!!) 
 **
 **  90.04.12			(James Merkle)
 **						Altered slightly for looks and portability 
 **
 **  91.04.17			(James Merkle)
 **						Altered slightly for looks and portability 
 **
 **  1991.07.05		(James Merkle)
 **						Altered slightly to import into NS-Shell, for frequent usage 
 **
 **  1994.06.05		(James Merkle)
 **						BUG FIX : bytes_Remaining_To_Fetch was not being checked for evenness in Read_File_Using_Buffer.
 **
 **  1995.04.09		(James Merkle)
 **						BUG FIX : bytes_Remaining_To_Fetch was not being checked for "0" in Read_File_Using_Buffer.
 **
 **  1995.05.10		(James Merkle)
 **						BUG FIX : in Open_Well_Defined_File_Using_Buffer result_Size was not set ever when returned
 **
 **
 **
 ** =================
 ** OTHER INFORMATION
 ** =================
 **	It is important to initialize special structures used for controlling buffers with "Create_File_Buffer_Structures"
 **
 ** This module is quite efficient, and well suited for high speed file format conversions, or file copying.
 ** It contains many different sized type definitions to assist in copying memory quickly when needed.
 **
 **/

/* [----END----] FILE HEADER COMMENTS */


/*=**************************************************************************=*/
/* [---BEGIN---] FILE INCLUDE AREA */

#include "Shell_Globals.h"
#include "Error_Utilities.h"
#include "Memory_Utilities.h"

#include "Buffered_File_Routines.h"

/* #include "System_Dependent_Hooks.h" */

/* [----END----] FILE INCLUDE AREA */
/*=**************************************************************************=*/
/* [---BEGIN---] PROTOTYPES-------------------------- */

/*--------------- LOCAL PROTOTYPES -------------------*/


/* extern Boolean Create_File_Buffer_Structures(void); */

/* extern Boolean Destroy_File_Buffer_Structures(void); */

/* extern Boolean Read_File_Using_Buffer(	INTEGER_16 	open_File_Fork
													, register INTEGER_32	*data_Size_Read
													, CARDINAL_8 				*data_Destination
													, INTEGER_16 	*result_OS_Error); */
													
/* extern Boolean Write_File_Using_Buffer(INTEGER_16 	open_File_Fork
													, INTEGER_32	*data_Size_To_Write
													, CARDINAL_8 				*data_Source
													, INTEGER_16	*result_OS_Error) */
													
/* extern Boolean Flush_File_Buffer( INTEGER_16	open_File_Fork
										, INTEGER_16	*result_OS_Error); */
											
/* extern Boolean Open_File_Using_Buffer( INTEGER_16	fork_Desired
												, Str255			file_Name_P
												, INTEGER_16		working_Directory_Ref
												, INTEGER_16		*file_Reference_Num
												, INTEGER_32		minimum_Size
												, INTEGER_32		maximum_Size
												, INTEGER_32		desired_Size
												, INTEGER_32		*result_Size
												, INTEGER_16		*result_OS_Error);*/
													
/* extern Boolean Close_File_With_Buffer(	INTEGER_16 		open_File_Fork
												, INTEGER_16 	*result_OS_Error); */

/* extern Boolean Copy_File_To_File( INTEGER_16		source_File_Fork
											,INTEGER_16		target_File_Fork
											, INTEGER_32		maximum_Memory_To_Utilize); */

/* extern Boolean Locate_File_Buffer_Index(	INTEGER_16	open_File_Fork
												, INTEGER_32	*result_Buffer_Index); */


Boolean Read_From_File(	INTEGER_16 	open_File_Fork
											, INTEGER_32	*data_Size_Read
											, Ptr data_Destination
											, INTEGER_16 	*result_OS_Error);

Boolean Read_Damaged_Data_Files_Anyway(	INTEGER_16 	open_File_Fork
											, INTEGER_32	*data_Size_Read
											, Ptr data_Destination
											, INTEGER_16 	*result_OS_Error);


Boolean Write_To_File(	INTEGER_16 		open_File_Fork
								, INTEGER_32		*data_Size_Write
								,Ptr data_Source
								, INTEGER_16 	*result_OS_Error);

/* [----END----] PROTOTYPES-------------------------- */
/*=**************************************************************************=*/

/*=**************************************************************************=*/
/* structures used within Buffered_File_Routines.c  */

typedef struct {
	CARDINAL_32	Full_Word_1;
	CARDINAL_16	Half_Word_1;
} structure_Size_6_Bytes, *structure_Size_6_Bytes_Ptr;

typedef struct {
	CARDINAL_32	Full_Word_1;
	CARDINAL_32	Full_Word_2;
} structure_Size_8_Bytes, *structure_Size_8_Bytes_Ptr;

typedef struct {
	CARDINAL_32	Full_Word_1;
	CARDINAL_32	Full_Word_2;
	CARDINAL_16	Half_Word_1;
} structure_Size_10_Bytes, *structure_Size_10_Bytes_Ptr;

typedef struct {
	CARDINAL_32	Full_Word_1;
	CARDINAL_32	Full_Word_2;
	CARDINAL_32	Full_Word_3;
} structure_Size_12_Bytes, *structure_Size_12_Bytes_Ptr;

typedef struct {
	CARDINAL_32	Full_Word_1;
	CARDINAL_32	Full_Word_2;
	CARDINAL_32	Full_Word_3;
	CARDINAL_16	Half_Word_1;
} structure_Size_14_Bytes, *structure_Size_14_Bytes_Ptr;

typedef struct {
	CARDINAL_32	Full_Word_1;
	CARDINAL_32	Full_Word_2;
	CARDINAL_32	Full_Word_3;
	CARDINAL_32	Full_Word_4;
} structure_Size_16_Bytes, *structure_Size_16_Bytes_Ptr;

typedef struct {
	CARDINAL_32	Full_Word_1;
	CARDINAL_32	Full_Word_2;
	CARDINAL_32	Full_Word_3;
	CARDINAL_32	Full_Word_4;
	CARDINAL_16	Half_Word_1;
} structure_Size_18_Bytes, *structure_Size_18_Bytes_Ptr;

typedef struct {
	CARDINAL_32	Full_Word_1;
	CARDINAL_32	Full_Word_2;
	CARDINAL_32	Full_Word_3;
	CARDINAL_32	Full_Word_4;
	CARDINAL_32	Full_Word_5;
} structure_Size_20_Bytes, *structure_Size_20_Bytes_Ptr;

typedef struct {
	CARDINAL_32	Full_Word_1;
	CARDINAL_32	Full_Word_2;
	CARDINAL_32	Full_Word_3;
	CARDINAL_32	Full_Word_4;
	CARDINAL_32	Full_Word_5;
	CARDINAL_16	Half_Word_1;
} structure_Size_22_Bytes, *structure_Size_22_Bytes_Ptr;

typedef struct {
	CARDINAL_32	Full_Word_1;
	CARDINAL_32	Full_Word_2;
	CARDINAL_32	Full_Word_3;
	CARDINAL_32	Full_Word_4;
	CARDINAL_32	Full_Word_5;
	CARDINAL_32	Full_Word_6;
} structure_Size_24_Bytes, *structure_Size_24_Bytes_Ptr;

typedef struct {
	CARDINAL_32	Full_Word_1;
	CARDINAL_32	Full_Word_2;
	CARDINAL_32	Full_Word_3;
	CARDINAL_32	Full_Word_4;
	CARDINAL_32	Full_Word_5;
	CARDINAL_32	Full_Word_6;
	CARDINAL_16	Half_Word_1;
} structure_Size_26_Bytes, *structure_Size_26_Bytes_Ptr;

typedef struct {
	CARDINAL_32	Full_Word_1;
	CARDINAL_32	Full_Word_2;
	CARDINAL_32	Full_Word_3;
	CARDINAL_32	Full_Word_4;
	CARDINAL_32	Full_Word_5;
	CARDINAL_32	Full_Word_6;
	CARDINAL_32	Full_Word_7;
} structure_Size_28_Bytes, *structure_Size_28_Bytes_Ptr;

typedef struct {
	CARDINAL_32	Full_Word_1;
	CARDINAL_32	Full_Word_2;
	CARDINAL_32	Full_Word_3;
	CARDINAL_32	Full_Word_4;
	CARDINAL_32	Full_Word_5;
	CARDINAL_32	Full_Word_6;
	CARDINAL_32	Full_Word_7;
	CARDINAL_16	Half_Word_1;
} structure_Size_30_Bytes, *structure_Size_30_Bytes_Ptr;

typedef struct {
	CARDINAL_32	Full_Word_1;
	CARDINAL_32	Full_Word_2;
	CARDINAL_32	Full_Word_3;
	CARDINAL_32	Full_Word_4;
	CARDINAL_32	Full_Word_5;
	CARDINAL_32	Full_Word_6;
	CARDINAL_32	Full_Word_7;
	CARDINAL_32	Full_Word_8;
} structure_Size_32_Bytes, *structure_Size_32_Bytes_Ptr;
/*=**************************************************************************=*/

/*=**************************************************************************=*/
/* variables used in and outside Buffered_File_Routines.c  */

file_Buffer_Array_Ptr		file_Buffers; /* for controlling open files and buffering */
INTEGER_32				file_Buffers_Memory_In_Use;
Boolean					allow_Damaged_Data_To_Load; /* When True, bad data ignored on reads */

/*=**************************************************************************=*/

/*=**************************************************************************=*/
/* variables used within Buffered_File_Routines.c  */
static CARDINAL_8	*unit_Name = "\pBuffered_File_Routines";


static void Copy_One_Byte(register CARDINAL_8 *source_Ptr, register CARDINAL_8 *destination_Ptr);
static void Copy_Two_Bytes(register CARDINAL_8 *source_Ptr, register CARDINAL_8 *destination_Ptr);
static void Copy_Four_Bytes(register CARDINAL_8 *source_Ptr, register CARDINAL_8 *destination_Ptr);
static void Copy_Six_Bytes(register CARDINAL_8 *source_Ptr, register CARDINAL_8 *destination_Ptr);
static void Copy_Eight_Bytes(register CARDINAL_8 *source_Ptr, register CARDINAL_8 *destination_Ptr);
static void Copy_Ten_Bytes(register CARDINAL_8 *source_Ptr, register CARDINAL_8 *destination_Ptr);
static void Copy_Twelve_Bytes(register CARDINAL_8 *source_Ptr, register CARDINAL_8 *destination_Ptr);
static void Copy_Fourteen_Bytes(register CARDINAL_8 *source_Ptr, register CARDINAL_8 *destination_Ptr);
static void Copy_Sixteen_Bytes(register CARDINAL_8 *source_Ptr, register CARDINAL_8 *destination_Ptr);
static void Copy_Eighteen_Bytes(register CARDINAL_8 *source_Ptr, register CARDINAL_8 *destination_Ptr);
static void Copy_Twenty_Bytes(register CARDINAL_8 *source_Ptr, register CARDINAL_8 *destination_Ptr);
static void Copy_Twenty_Two_Bytes(register CARDINAL_8 *source_Ptr, register CARDINAL_8 *destination_Ptr);
static void Copy_Twenty_Four_Bytes(register CARDINAL_8 *source_Ptr, register CARDINAL_8 *destination_Ptr);
static void Copy_Twenty_Six_Bytes(register CARDINAL_8 *source_Ptr, register CARDINAL_8 *destination_Ptr);
static void Copy_Twenty_Eight_Bytes(register CARDINAL_8 *source_Ptr, register CARDINAL_8 *destination_Ptr);
static void Copy_Thirty_Bytes(register CARDINAL_8 *source_Ptr, register CARDINAL_8 *destination_Ptr);
static void Copy_Thirty_Two_Bytes(register CARDINAL_8 *source_Ptr, register CARDINAL_8 *destination_Ptr);
static void Copy_Invalid_Bytes(register CARDINAL_8 *source_Ptr, register CARDINAL_8 *destination_Ptr);

static void Initialize_Jump_Table(void);

static void							(*function_Array[33])(...); /* Indexed 0-32, we don't use 0'th position */

/*=**************************************************************************=*/


/*=**************************************************************************=*/
Boolean Read_Damaged_Data_Files_Anyway(	INTEGER_16 	open_File_Fork
											, INTEGER_32	*data_Size_Read
											, Ptr data_Destination
											, INTEGER_16 	*result_OS_Error)
											
/*=--------------------------------------------------------------------------=*/
/* Intended for portability possibilities */
/* Allows partially damaged reading. Not currently implemented though */
/*=--------------------------------------------------------------------------=*/
{
	INTEGER_16	function_Result;
	INTEGER_16	os_Error;
/* END VAR */


	os_Error = FSRead(open_File_Fork, data_Size_Read, data_Destination);
	if ( os_Error == noErr) {
		function_Result = TRUE;
		*result_OS_Error =  noErr;
	}
	else
	{
		function_Result = FALSE;
		*result_OS_Error =  os_Error;
	}
	
	return(function_Result);
	
} /* Read_Damaged_Data_Files_Anyway */
/*=**************************************************************************=*/


/*=**************************************************************************=*/
Boolean Read_From_File(	INTEGER_16 	open_File_Fork
											, INTEGER_32	*data_Size_Read
											, Ptr data_Destination
											, INTEGER_16 	*result_OS_Error)
{
	INTEGER_16	function_Result;
	INTEGER_16	os_Error;
/* END VAR */


	os_Error = FSRead(open_File_Fork, data_Size_Read, data_Destination);
	if ( os_Error == noErr) {
		function_Result = TRUE;
		*result_OS_Error =  noErr;
	}
	else
	{
		function_Result = FALSE;
		*result_OS_Error =  os_Error;
	}
	
	return(function_Result);
	
} /* Read_From_File */
/*=**************************************************************************=*/


/*=**************************************************************************=*/
Boolean Write_To_File(	INTEGER_16 		open_File_Fork
								, INTEGER_32		*data_Size_Write
								, Ptr data_Source
								, INTEGER_16 	*result_OS_Error)
{
	INTEGER_16	function_Result;
	INTEGER_16	os_Error;
/* END VAR */


	os_Error = FSWrite(open_File_Fork, data_Size_Write, data_Source);
	if ( os_Error == noErr) {
		function_Result = TRUE;
		*result_OS_Error =  noErr;
	}
	else
	{
		function_Result = FALSE;
		*result_OS_Error =  os_Error;
	}
	
	return(function_Result);
	
} /* Write_To_File */
/*=**************************************************************************=*/

/*=**************************************************************************=*/
/* Note:	If the structure and method used by this routine is altered, the "Open_File_Using_Buffer"  */
/*			routine may have to be changed as well */
Boolean Locate_File_Buffer_Index(	INTEGER_16	open_File_Fork
														, INTEGER_32	*result_Buffer_Index)
{
	CARDINAL_8				*code_Name;
	Boolean			found_Position;
	INTEGER_32	current_Buffer;
	INTEGER_16	function_Result;
/* END VAR */


	code_Name = "\pLocate_File_Buffer_Index";
	function_Result = FALSE;
	
	current_Buffer = (file_Buffers->total_Open_Files - 1); /* remember, Base 0 */
	for(found_Position = FALSE; (current_Buffer >= 0); current_Buffer-- ) /* contains a break statement */
	{
		if (file_Buffers->list_Of_Open_Fork_IDs[current_Buffer] == open_File_Fork) {
			if (file_Buffers->list_Of_Open_Forks[current_Buffer]->buffer_Activated == BUFFER_IN_USE ) {
				found_Position = TRUE;
				file_Buffers->last_Accessed_OS_File_Reference = open_File_Fork;
				file_Buffers->last_Accessed_Buffer_Index = current_Buffer;
				break; /* break retains current value of current_Buffer */
			}
		}
	}
	
	if (found_Position != TRUE) {
		Display_Serious_Error_Message(unit_Name, code_Name, "\pFile fork buffer ID not located!", "\p"); /* This error could be handled by caller */
		*result_Buffer_Index = 0; /* "paranoia" set */
		function_Result = FALSE;
		goto END_OF_FUNCTION;
	}
	else
	{
		*result_Buffer_Index = current_Buffer;
	}

	function_Result = TRUE;

END_OF_FUNCTION:
	return(function_Result);
	
} /* Locate_File_Buffer_Index */
/*=**************************************************************************=*/


/*=**************************************************************************=*/
Boolean Create_File_Buffer_Structures(void)
{
	INTEGER_16		function_Result;
	CARDINAL_8				*code_Name;
	
	INTEGER_32	total_Bytes_Contiguous_Now,total_Bytes_Now;
	INTEGER_32	total_Bytes_Contiguous_Later,total_Bytes_Later;
/* END VAR */


	code_Name = "\pCreate_File_Buffer_Structures";
	function_Result = FALSE;
	
	Initialize_Jump_Table();
	
	if (Determine_Absolute_Free_Memory(&total_Bytes_Contiguous_Now,&total_Bytes_Now,&total_Bytes_Contiguous_Later,&total_Bytes_Later) != TRUE) {
		Display_Serious_Error_Message(unit_Name, code_Name, "\pcall to Determine_Absolute_Free_Memory", "\p");
		function_Result = FALSE;
		goto END_OF_FUNCTION;
	}

	if (total_Bytes_Contiguous_Later > sizeof(file_Buffer_Array)) {
		file_Buffers = (file_Buffer_Array_Ptr) NewPtr(sizeof(file_Buffer_Array));
		if (MemError() != noErr) {
			Display_Serious_Error_Message(unit_Name, code_Name, "\pmemory not available!", "\p");
			function_Result = FALSE;
			goto END_OF_FUNCTION;
		}
	}
	else
	{
		Display_Serious_Error_Message(unit_Name, code_Name, "\pmemory not available for file_Buffer_Array", "\p");
		function_Result = FALSE;
		goto END_OF_FUNCTION;
	}
	
	file_Buffers->total_Open_Files = 0;
	
	
	file_Buffers_Memory_In_Use = MEMORY_INITIALIZED;
	file_Buffers->list_Of_Open_Fork_IDs[0] == BUFFER_NOT_VALID; /* Mark 1st for safety */
	file_Buffers->last_Accessed_OS_File_Reference = 0;
	file_Buffers->last_Accessed_Buffer_Index = 0;

	allow_Damaged_Data_To_Load = FALSE;  /* Initialize.  When True, bad data ignored on reads */
	
	function_Result = TRUE;

END_OF_FUNCTION:
	return(function_Result);
	
} /* Create_File_Buffer_Structures */
/*=**************************************************************************=*/


/*=**************************************************************************=*/
/* Note: this routine does not flush any open file buffers. I did this on purpose, but it is trivial to modify it */
Boolean Destroy_File_Buffer_Structures( void )
{
	CARDINAL_8			*code_Name;
	INTEGER_16	function_Result;
	INTEGER_16 	result_OS_Error;
	INTEGER_32	current_Buffer;
/* END VAR */


	code_Name = "\pDestroy_File_Buffer_Structures";
	function_Result = FALSE;
	
	/* check if any files are still open for any odd reason and close them */
	if (file_Buffers_Memory_In_Use == MEMORY_INITIALIZED)
	{
		current_Buffer = file_Buffers->total_Open_Files;
		for(;(current_Buffer > 0);current_Buffer-- )
		{
			if (file_Buffers->list_Of_Open_Fork_IDs[current_Buffer] != BUFFER_NOT_VALID) {
				if (Close_File_With_Buffer(file_Buffers->list_Of_Open_Fork_IDs[current_Buffer], DO_NOT_FLUSH_BUFFER, &result_OS_Error) != TRUE) {
					Display_Serious_Error_Message(unit_Name, code_Name, "\pmemory not available?", "\p");
					function_Result = FALSE;
					goto END_OF_FUNCTION;
				}

				file_Buffers->list_Of_Open_Fork_IDs[current_Buffer] = BUFFER_NOT_VALID;
			}
		}
	}

	DisposPtr((Ptr)file_Buffers);
	if (MemError() != noErr) {
		Display_Serious_Error_Message(unit_Name, code_Name, "\pDisposPtr of file_Buffers failed", "\p");
		function_Result = FALSE;
		goto END_OF_FUNCTION;
	}

	function_Result = TRUE;

END_OF_FUNCTION:
	return(function_Result);
	
} /* Destroy_File_Buffer_Structures */
/*=**************************************************************************=*/


/*=**************************************************************************=*/
static void Initialize_Jump_Table(void)
{
	CARDINAL_8	*source_Ptr, *destination_Ptr;
/* END VAR */


	function_Array[0] = Copy_Invalid_Bytes;
	function_Array[3] = Copy_Invalid_Bytes;
	function_Array[5] = Copy_Invalid_Bytes;
	function_Array[7] = Copy_Invalid_Bytes;
	function_Array[9] = Copy_Invalid_Bytes;
	function_Array[11] = Copy_Invalid_Bytes;
	function_Array[13] = Copy_Invalid_Bytes;
	function_Array[15] = Copy_Invalid_Bytes;
	function_Array[17] = Copy_Invalid_Bytes;
	function_Array[19] = Copy_Invalid_Bytes;
	function_Array[21] = Copy_Invalid_Bytes;
	function_Array[23] = Copy_Invalid_Bytes;
	function_Array[25] = Copy_Invalid_Bytes;
	function_Array[27] = Copy_Invalid_Bytes;
	function_Array[29] = Copy_Invalid_Bytes;	
	function_Array[31] = Copy_Invalid_Bytes;
	
	function_Array[1] = Copy_One_Byte;
	function_Array[2] = Copy_Two_Bytes;
	function_Array[4] = Copy_Four_Bytes;
	function_Array[6] = Copy_Six_Bytes;
	function_Array[8] = Copy_Eight_Bytes;  
	function_Array[10] = Copy_Ten_Bytes;
	function_Array[12] = Copy_Twelve_Bytes;
	function_Array[14] = Copy_Fourteen_Bytes;
	function_Array[16] = Copy_Sixteen_Bytes;
	function_Array[18] = Copy_Eighteen_Bytes;
	function_Array[20] = Copy_Twenty_Bytes;
	function_Array[22] = Copy_Twenty_Two_Bytes;
	function_Array[24] = Copy_Twenty_Four_Bytes;
	function_Array[26] = Copy_Twenty_Six_Bytes;
	function_Array[28] = Copy_Twenty_Eight_Bytes;
	function_Array[30] = Copy_Thirty_Bytes;
	function_Array[32] = Copy_Thirty_Two_Bytes;
	
} /* static void Initialize_Jump_Table(void) */
/*=**************************************************************************=*/


/*=**************************************************************************=*/
static void Copy_One_Byte(register CARDINAL_8 *source_Ptr, register CARDINAL_8 *destination_Ptr)
{
	*destination_Ptr = *source_Ptr;
	
} /* static void Copy_One_Byte(register CARDINAL_8 *source_Ptr, register CARDINAL_8 *destination_Ptr) */
/*=**************************************************************************=*/


/*=**************************************************************************=*/
static void Copy_Two_Bytes(register CARDINAL_8 *source_Ptr, register CARDINAL_8 *destination_Ptr)
{
	*((CARDINAL_16 *)destination_Ptr) = *((CARDINAL_16 *)source_Ptr);

} /* static void Copy_Two_Bytes(register CARDINAL_8 *source_Ptr, register CARDINAL_8 *destination_Ptr) */
/*=**************************************************************************=*/


/*=**************************************************************************=*/
static void Copy_Four_Bytes(register CARDINAL_8 *source_Ptr, register CARDINAL_8 *destination_Ptr)
{
	*((CARDINAL_32 *)destination_Ptr) = *((CARDINAL_32 *)source_Ptr);
	
} /* static void Copy_Four_Bytes(register CARDINAL_8 *source_Ptr, register CARDINAL_8 *destination_Ptr) */
/*=**************************************************************************=*/


/*=**************************************************************************=*/
static void Copy_Six_Bytes(register CARDINAL_8 *source_Ptr, register CARDINAL_8 *destination_Ptr)
{
	*((structure_Size_6_Bytes_Ptr)destination_Ptr) = *((structure_Size_6_Bytes_Ptr)source_Ptr);
	
} /* static void Copy_Six_Bytes(register CARDINAL_8 *source_Ptr, register CARDINAL_8 *destination_Ptr) */
/*=**************************************************************************=*/


/*=**************************************************************************=*/
static void Copy_Eight_Bytes(register CARDINAL_8 *source_Ptr, register CARDINAL_8 *destination_Ptr)
{
	*((structure_Size_8_Bytes_Ptr)destination_Ptr) = *((structure_Size_8_Bytes_Ptr)source_Ptr);
	
} /* static void Copy_Eight_Bytes(register CARDINAL_8 *source_Ptr, register CARDINAL_8 *destination_Ptr) */
/*=**************************************************************************=*/


/*=**************************************************************************=*/
static void Copy_Ten_Bytes(register CARDINAL_8 *source_Ptr, register CARDINAL_8 *destination_Ptr)
{
	*((structure_Size_10_Bytes_Ptr)destination_Ptr) = *((structure_Size_10_Bytes_Ptr)source_Ptr);
	
} /* static void Copy_Ten_Bytes(register CARDINAL_8 *source_Ptr, register CARDINAL_8 *destination_Ptr) */
/*=**************************************************************************=*/


/*=**************************************************************************=*/
static void Copy_Twelve_Bytes(register CARDINAL_8 *source_Ptr, register CARDINAL_8 *destination_Ptr)
{
	*((structure_Size_12_Bytes_Ptr)destination_Ptr) = *((structure_Size_12_Bytes_Ptr)source_Ptr);
	
} /* static void Copy_Twelve_Bytes(register CARDINAL_8 *source_Ptr, register CARDINAL_8 *destination_Ptr) */
/*=**************************************************************************=*/


/*=**************************************************************************=*/
static void Copy_Fourteen_Bytes(register CARDINAL_8 *source_Ptr, register CARDINAL_8 *destination_Ptr)
{
	*((structure_Size_14_Bytes_Ptr)destination_Ptr) = *((structure_Size_14_Bytes_Ptr)source_Ptr);
	
} /* static void Copy_Fourteen_Bytes(register CARDINAL_8 *source_Ptr, register CARDINAL_8 *destination_Ptr) */
/*=**************************************************************************=*/


/*=**************************************************************************=*/
static void Copy_Sixteen_Bytes(register CARDINAL_8 *source_Ptr, register CARDINAL_8 *destination_Ptr)
{
	*((structure_Size_16_Bytes_Ptr)destination_Ptr) = *((structure_Size_16_Bytes_Ptr)source_Ptr);
	
} /* static void Copy_Sixteen_Bytes(register CARDINAL_8 *source_Ptr, register CARDINAL_8 *destination_Ptr) */
/*=**************************************************************************=*/


/*=**************************************************************************=*/
static void Copy_Eighteen_Bytes(register CARDINAL_8 *source_Ptr, register CARDINAL_8 *destination_Ptr)
{
	*((structure_Size_18_Bytes_Ptr)destination_Ptr) = *((structure_Size_18_Bytes_Ptr)source_Ptr);
	
} /* static void Copy_Eighteen_Bytes(register CARDINAL_8 *source_Ptr, register CARDINAL_8 *destination_Ptr) */
/*=**************************************************************************=*/


/*=**************************************************************************=*/
static void Copy_Twenty_Bytes(register CARDINAL_8 *source_Ptr, register CARDINAL_8 *destination_Ptr)
{
	*((structure_Size_20_Bytes_Ptr)destination_Ptr) = *((structure_Size_20_Bytes_Ptr)source_Ptr);
	
} /* static void Copy_Twenty_Bytes(register CARDINAL_8 *source_Ptr, register CARDINAL_8 *destination_Ptr) */
/*=**************************************************************************=*/


/*=**************************************************************************=*/
static void Copy_Twenty_Two_Bytes(register CARDINAL_8 *source_Ptr, register CARDINAL_8 *destination_Ptr)
{
	*((structure_Size_22_Bytes_Ptr)destination_Ptr) = *((structure_Size_22_Bytes_Ptr)source_Ptr);
	
} /* static void Copy_Twenty_Two_Bytes(register CARDINAL_8 *source_Ptr, register CARDINAL_8 *destination_Ptr) */
/*=**************************************************************************=*/


/*=**************************************************************************=*/
static void Copy_Twenty_Four_Bytes(register CARDINAL_8 *source_Ptr, register CARDINAL_8 *destination_Ptr)
{
	*((structure_Size_24_Bytes_Ptr)destination_Ptr) = *((structure_Size_24_Bytes_Ptr)source_Ptr);
	
} /* static void Copy_Twenty_Four_Bytes(register CARDINAL_8 *source_Ptr, register CARDINAL_8 *destination_Ptr) */
/*=**************************************************************************=*/


/*=**************************************************************************=*/
static void Copy_Twenty_Six_Bytes(register CARDINAL_8 *source_Ptr, register CARDINAL_8 *destination_Ptr)
{
	*((structure_Size_26_Bytes_Ptr)destination_Ptr) = *((structure_Size_26_Bytes_Ptr)source_Ptr);
	
} /* static void Copy_Twenty_Six_Bytes(register CARDINAL_8 *source_Ptr, register CARDINAL_8 *destination_Ptr) */
/*=**************************************************************************=*/


/*=**************************************************************************=*/
static void Copy_Twenty_Eight_Bytes(register CARDINAL_8 *source_Ptr, register CARDINAL_8 *destination_Ptr)
{
	*((structure_Size_28_Bytes_Ptr)destination_Ptr) = *((structure_Size_28_Bytes_Ptr)source_Ptr);
	
} /* static void Copy_Twenty_Eight_Bytes(register CARDINAL_8 *source_Ptr, register CARDINAL_8 *destination_Ptr) */
/*=**************************************************************************=*/


/*=**************************************************************************=*/
static void Copy_Thirty_Bytes(register CARDINAL_8 *source_Ptr, register CARDINAL_8 *destination_Ptr)
{
	*((structure_Size_30_Bytes_Ptr)destination_Ptr) = *((structure_Size_30_Bytes_Ptr)source_Ptr);
	
} /* static void Copy_Thirty_Bytes(register CARDINAL_8 *source_Ptr, register CARDINAL_8 *destination_Ptr) */
/*=**************************************************************************=*/


/*=**************************************************************************=*/
static void Copy_Thirty_Two_Bytes(register CARDINAL_8 *source_Ptr, register CARDINAL_8 *destination_Ptr)
{
	*((structure_Size_32_Bytes_Ptr)destination_Ptr) = *((structure_Size_32_Bytes_Ptr)source_Ptr);
	
} /* static void Copy_Thirty_Two_Bytes(register CARDINAL_8 *source_Ptr, register CARDINAL_8 *destination_Ptr) */
/*=**************************************************************************=*/


/*=**************************************************************************=*/
static void Copy_Invalid_Bytes(register CARDINAL_8 *source_Ptr, register CARDINAL_8 *destination_Ptr)
{
	CARDINAL_8	*code_Name;
/* END VAR */


	code_Name = "\pRead_File_Using_Buffer(Copy_Invalid_Bytes)";
	Display_Serious_Error_Message(unit_Name, code_Name, "\pError in Function Array Index Value!", "\p");
	
} /* static void Copy_Invalid_Bytes(register CARDINAL_8 *source_Ptr, register CARDINAL_8 *destination_Ptr) */
/*=**************************************************************************=*/


/*=**************************************************************************=*/
 Boolean Read_File_Using_Buffer(INTEGER_16 open_File_Fork
									, register INTEGER_32 *data_Size_Read
									, Ptr data_Destination
									, INTEGER_16 *result_OS_Error)
{
	INTEGER_16			function_Result;
	CARDINAL_8					*code_Name;
	register	Ptr		current_Data_Source;
	register	Ptr		current_Data_Destination;
	file_Buffer_Ptr			local_File_Buffers;
	INTEGER_32			current_Buffer;
	register	INTEGER_32	bytes_Remaining_To_Fetch;
	INTEGER_32			amount_To_Read;
	Boolean				result_Code; /* used for function results */
	INTEGER_16			os_Error;
	Boolean				stop_Looping;
/* END VAR */


		*result_OS_Error =  noErr;


		/* Default initialization for routine not performed at this place for performance gain */
		
		if (file_Buffers->last_Accessed_OS_File_Reference == open_File_Fork) {
			current_Buffer = file_Buffers->last_Accessed_Buffer_Index;
		}
		else
		{
			if (Locate_File_Buffer_Index(open_File_Fork, &current_Buffer) != TRUE) {
				code_Name = "\pRead_File_Using_Buffer";
				Display_Serious_Error_Message(unit_Name, code_Name, "\pFile fork buffer ID not located!", "\p");
				function_Result = FALSE;
				*result_OS_Error =  noErr;
				os_Error = noErr;
				goto END_OF_FUNCTION;
			}
		}

		local_File_Buffers = file_Buffers->list_Of_Open_Forks[current_Buffer];
		
		
		if (local_File_Buffers->read_Write_Flags == WRITE_MODE) {
			if ( Flush_File_Buffer(open_File_Fork, &os_Error) != TRUE) {
				code_Name = "\pRead_File_Using_Buffer";
				Display_Serious_Error_Message(unit_Name, code_Name, "\pCould not flush buffer!", "\p");
				function_Result = FALSE;
				*result_OS_Error =  noErr;
				os_Error = noErr;
				goto END_OF_FUNCTION;
			}
			local_File_Buffers->read_Write_Flags = READ_MODE; 
			/* Practical to go from WRITE to READ because flush resets to zero ?*/
		}
		
		
			/*-----------------------------------------------------------------------------------------------------------------------*/
			
		bytes_Remaining_To_Fetch = *data_Size_Read;

		amount_To_Read =   (local_File_Buffers->data_Position_Offset - local_File_Buffers->buffer_Position_Offset);

		if  (amount_To_Read >= bytes_Remaining_To_Fetch) {
			current_Data_Source = (Ptr)(local_File_Buffers->buffer_Address + local_File_Buffers->buffer_Position_Offset);	
			
			if (bytes_Remaining_To_Fetch <= 32) {
				if ( bytes_Remaining_To_Fetch == 1) {
					*data_Destination = *current_Data_Source;
				}
				else {
				
					/*  */
					/* 1995.04.09 JAM STUPID BUG FIX: "bytes_Remaining_To_Fetch" */
					/* was not being checked for "0"! null transfers had to fail */
					/*  */
					if (bytes_Remaining_To_Fetch == 0) { 
						/*  */
						/* How the hell did it get here? */
						/* Should put uip diagnostics warning if needed. */
						/*  */
					
					}
					else {
						/*  */
						/* 1994.06.05 JAM STUPID BUG FIX: "bytes_Remaining_To_Fetch" */
						/* was not being checked for evenness! Odd transfers had to fail */
						/*  */
						if (((((unsigned long)data_Destination) & 0x00000001) == 0)
							&& ((((unsigned long)current_Data_Source) & 0x00000001) == 0)
							&& ((bytes_Remaining_To_Fetch & 0x00000001) == 0)) {
							function_Array[bytes_Remaining_To_Fetch](current_Data_Source, data_Destination);
						}
						else
							Move_Memory((Ptr)current_Data_Source, data_Destination, bytes_Remaining_To_Fetch);
					}
				}
			}
			else {
				Move_Memory((Ptr)current_Data_Source, data_Destination, bytes_Remaining_To_Fetch);
			}
			
			local_File_Buffers->buffer_Position_Offset = local_File_Buffers->buffer_Position_Offset + bytes_Remaining_To_Fetch;
			*result_OS_Error =  noErr;
			/* Thats it! */		
				
		} /* if  (amount_To_Read >= bytes_Remaining_To_Fetch) */
		else
		{
		
		/* Initialization for balance of routine performed here */
			code_Name = "\pRead_File_Using_Buffer";
			function_Result = FALSE;
			*result_OS_Error =  noErr;
			os_Error = noErr;

			/*-----------------------------------------------------------------------------------------------------------------------*/
			current_Data_Destination = data_Destination;
			stop_Looping = FALSE;
			do /* while ( stop_Looping != TRUE ) */
			{
			
				amount_To_Read =   (local_File_Buffers->data_Position_Offset - local_File_Buffers->buffer_Position_Offset);
				
				if ( amount_To_Read > 0 ) {
				
					if ( bytes_Remaining_To_Fetch < amount_To_Read) {
						amount_To_Read = bytes_Remaining_To_Fetch;
					}
					
					current_Data_Source = (Ptr) (local_File_Buffers->buffer_Address + local_File_Buffers->buffer_Position_Offset); 
					Move_Memory(current_Data_Source, current_Data_Destination, amount_To_Read);
					current_Data_Destination = current_Data_Destination + amount_To_Read;
					local_File_Buffers->buffer_Position_Offset = local_File_Buffers->buffer_Position_Offset + amount_To_Read;
					bytes_Remaining_To_Fetch = (bytes_Remaining_To_Fetch - amount_To_Read);
					
				} /* if ( amount_To_Read > 0 )  */
				else
				{
					
					if (local_File_Buffers->buffer_Position_Offset == local_File_Buffers->data_Position_Offset) {
						if (local_File_Buffers->data_Position_Offset != 0) {
							/* advance file position and shift markers to bottom of array*/
							local_File_Buffers->file_Position_Offset= local_File_Buffers->file_Position_Offset + local_File_Buffers->buffer_Position_Offset ; 
							local_File_Buffers->buffer_Position_Offset = 0;
							local_File_Buffers->data_Position_Offset = 0;
						}
					}

						current_Data_Source = (Ptr) (local_File_Buffers->buffer_Address + local_File_Buffers->buffer_Position_Offset); 
						amount_To_Read = local_File_Buffers->buffer_Size;
						
						if ( allow_Damaged_Data_To_Load == TRUE ) {
							result_Code = Read_From_File(open_File_Fork, &amount_To_Read, current_Data_Source, &os_Error);
						}
						else {
							result_Code = Read_Damaged_Data_Files_Anyway(open_File_Fork, &amount_To_Read, current_Data_Source, &os_Error);
						}
						
						
						if (result_Code == TRUE) {
							local_File_Buffers->data_Position_Offset = amount_To_Read;
						}
						else
						{
							if (os_Error == eofErr) {
								stop_Looping = TRUE;
								local_File_Buffers->data_Position_Offset = amount_To_Read;

								if ( amount_To_Read > 0 ) {
									if ( bytes_Remaining_To_Fetch < amount_To_Read) {
										amount_To_Read = bytes_Remaining_To_Fetch;
									}

									current_Data_Source = (Ptr) (local_File_Buffers->buffer_Address + local_File_Buffers->buffer_Position_Offset); 
									Move_Memory(current_Data_Source, current_Data_Destination, amount_To_Read);
									current_Data_Destination = current_Data_Destination + amount_To_Read;
									local_File_Buffers->buffer_Position_Offset = local_File_Buffers->buffer_Position_Offset + amount_To_Read;
									bytes_Remaining_To_Fetch = (bytes_Remaining_To_Fetch - amount_To_Read);
								}	
														
							} 
							else
							{
								*result_OS_Error = os_Error;
								Display_Serious_Error_Message(unit_Name, code_Name, "\pError reading data for file.", "\p");
								function_Result = FALSE;
								goto END_OF_FUNCTION;
							} /* NOT if (os_Error == eofErr) */
						} /* NOT 	if (result_Code == TRUE) */
							
				
				}  /* else NOT if ( amount_To_Read > 0 )  */
				
				if ( bytes_Remaining_To_Fetch == 0 ) 
					stop_Looping = TRUE;
				
			} while ( stop_Looping != TRUE );

			if (bytes_Remaining_To_Fetch > 0) {
				*data_Size_Read = (*data_Size_Read - bytes_Remaining_To_Fetch);
				*result_OS_Error = eofErr;
				function_Result = FALSE;
				goto END_OF_FUNCTION;
			}

		} /* NOT if  (amount_To_Read > bytes_Remaining_To_Fetch) */

		function_Result = TRUE;

END_OF_FUNCTION:
	return(function_Result);
	} /* Read_File_Using_Buffer */
/*=**************************************************************************=*/


/*=**************************************************************************=*/
 Boolean Write_File_Using_Buffer(INTEGER_16 	open_File_Fork
									, INTEGER_32		*data_Size_To_Write
									, Ptr data_Source
									, INTEGER_16		*result_OS_Error)
{
	INTEGER_16	function_Result;
	CARDINAL_8				*code_Name;
	Ptr					current_Data_Destination;
	Ptr					current_Data_Source;
	file_Buffer_Ptr		local_File_Buffers;
	INTEGER_32		current_Buffer;
	INTEGER_32		bytes_Remaining_To_Write;
	INTEGER_32		room_To_Write;
	INTEGER_16		os_Error;
	Boolean			stop_Looping;
/* END VAR */


	code_Name = "\pWrite_File_Using_Buffer";
	function_Result = FALSE;
	*result_OS_Error =  noErr;
	os_Error = noErr;

	if (file_Buffers->last_Accessed_OS_File_Reference == open_File_Fork) {
		current_Buffer = file_Buffers->last_Accessed_Buffer_Index;
	}
	else
	{
		if (Locate_File_Buffer_Index(open_File_Fork, &current_Buffer) != TRUE) {
			Display_Serious_Error_Message(unit_Name, code_Name, "\pFile fork buffer ID not located!", "\p");
			function_Result = FALSE;
			goto END_OF_FUNCTION;
		}
	}

	local_File_Buffers = file_Buffers->list_Of_Open_Forks[current_Buffer];
	
	
	if (local_File_Buffers->read_Write_Flags == READ_MODE) {
		if ( Flush_File_Buffer(open_File_Fork, &os_Error) != TRUE) {
			Display_Serious_Error_Message(unit_Name, code_Name, "\pCould not flush buffer!", "\p");
			function_Result = FALSE;
			goto END_OF_FUNCTION;
		}
		local_File_Buffers->read_Write_Flags = WRITE_MODE; 
		/* Practical to go from WRITE to READ because flush resets to zero ?*/
	}
	
	bytes_Remaining_To_Write = *data_Size_To_Write;
	
		/*-----------------------------------------------------------------------------------------------------------------------*/
	
	room_To_Write =   (local_File_Buffers->buffer_Size - local_File_Buffers->data_Position_Offset);

	if  (room_To_Write >= bytes_Remaining_To_Write) {
		current_Data_Destination = (Ptr) (local_File_Buffers->buffer_Address + local_File_Buffers->data_Position_Offset);	
		Move_Memory( data_Source, current_Data_Destination, bytes_Remaining_To_Write);
		local_File_Buffers->data_Position_Offset = local_File_Buffers->data_Position_Offset + bytes_Remaining_To_Write;
		/* Thats it! */			
	}
	else
	{
		/*-----------------------------------------------------------------------------------------------------------------------*/
		current_Data_Source = data_Source;
		stop_Looping = FALSE;
		do /* while ( stop_Looping != TRUE ) */
		{
		
			room_To_Write =   (local_File_Buffers->buffer_Size - local_File_Buffers->data_Position_Offset);
			
			if ( room_To_Write > 0 ) {
			
				if ( bytes_Remaining_To_Write < room_To_Write) {
					room_To_Write = bytes_Remaining_To_Write;
				}
				
				current_Data_Destination = (Ptr)(local_File_Buffers->buffer_Address + local_File_Buffers->data_Position_Offset); 
				Move_Memory(current_Data_Source, current_Data_Destination, room_To_Write);
				current_Data_Source = current_Data_Source + room_To_Write;
				local_File_Buffers->data_Position_Offset = local_File_Buffers->data_Position_Offset + room_To_Write;
				bytes_Remaining_To_Write = (bytes_Remaining_To_Write - room_To_Write);
				
			} /* if ( room_To_Write > 0 )  */
			else
			{
 					current_Data_Destination = (Ptr) (local_File_Buffers->buffer_Address); 
				room_To_Write = local_File_Buffers->buffer_Size;
			
				if ( Write_To_File(	open_File_Fork, &room_To_Write, current_Data_Destination,&os_Error) == TRUE)
				{
					/* set file position and shift markers to begining of array*/
					local_File_Buffers->data_Position_Offset = 0;
					local_File_Buffers->buffer_Position_Offset = 0; /* "paranoia" safety */
					local_File_Buffers->file_Position_Offset= local_File_Buffers->file_Position_Offset + room_To_Write ; 
				}
				else
				{
					local_File_Buffers->data_Position_Offset = 0;
					local_File_Buffers->buffer_Position_Offset = 0; /* "paranoia" safety */
					local_File_Buffers->file_Position_Offset= local_File_Buffers->file_Position_Offset + room_To_Write ; 
					Display_Serious_Error_Message(unit_Name, code_Name, "\pError writing data for file. (full?) ", "\p"); /* !! use freindlier error dialog in future*/
		
					/* At this point, I want a typical "Abort,Retry,Fail" capability as MS-DOS typically utilizes .... later on*/
		
					function_Result = FALSE;
					*result_OS_Error = os_Error;

					goto END_OF_FUNCTION;
				} /* NOT if ( os_Error == noErr) */
					
		
			}  /* else NOT if ( room_To_Write > 0 )  */
			
			if ( bytes_Remaining_To_Write == 0 ) 
				stop_Looping = TRUE;
			
		} while ( stop_Looping != TRUE );

	} /* NOT if  (room_To_Write > bytes_Remaining_To_Write) */

	function_Result = TRUE;

	
END_OF_FUNCTION:
	return(function_Result);
	
} /* Write_File_Using_Buffer */
/*=**************************************************************************=*/

/*=**************************************************************************=*/
 Boolean Flush_File_Buffer(	INTEGER_16	open_File_Fork
								, INTEGER_16	*result_OS_Error)
{
	CARDINAL_8				*code_Name;
	INTEGER_16		function_Result, iO_Queue_Element_Size;
	INTEGER_32		current_Buffer, bytes_To_Flush, previous_Value;
	file_Buffer_Ptr		local_File_Buffers;
	INTEGER_16		os_Error;
	Ptr					current_Data_Address;
/* END VAR */


	code_Name = "\pFlush_File_Buffer";
	function_Result = FALSE;
	*result_OS_Error =  noErr;
	
	
		if (file_Buffers->last_Accessed_OS_File_Reference == open_File_Fork) {
			current_Buffer = file_Buffers->last_Accessed_Buffer_Index;
		}
		else
		{
			if (Locate_File_Buffer_Index(open_File_Fork, &current_Buffer) != TRUE) {
				Display_Serious_Error_Message(unit_Name, code_Name, "\pFile fork buffer ID not located!", "\p");
				function_Result = FALSE;
				goto END_OF_FUNCTION;
			}
		}

		local_File_Buffers = file_Buffers->list_Of_Open_Forks[current_Buffer];

		if (local_File_Buffers->buffer_Position_Offset != local_File_Buffers->data_Position_Offset) {
		
			if (local_File_Buffers->read_Write_Flags == WRITE_MODE) {

				bytes_To_Flush = local_File_Buffers->data_Position_Offset; /* use bytes_To_Flush as temporary value */
				if ((bytes_To_Flush >= local_File_Buffers->buffer_Size) || (bytes_To_Flush < 0 )) {
					Display_Serious_Error_Message(unit_Name, code_Name, "\pError with buffer structure usage", "\p");
					function_Result = FALSE;
					goto END_OF_FUNCTION;
				}
				
				previous_Value = bytes_To_Flush;
				/* write out the data we just got */
				current_Data_Address = (Ptr) (local_File_Buffers->buffer_Address);
				
				if ( Write_To_File(	open_File_Fork, &bytes_To_Flush, current_Data_Address,&os_Error) != TRUE)
				{
					Display_Serious_Error_Message(unit_Name, code_Name, "\pError writing data for file", "\p");
					function_Result = FALSE;
					*result_OS_Error = os_Error;
					goto END_OF_FUNCTION;
				}

				if (bytes_To_Flush > previous_Value) {
					Display_Serious_Error_Message(unit_Name, code_Name, "\pOperating System had Insane Write_To_File reply", "\p");
					function_Result = FALSE;
					goto END_OF_FUNCTION;
				}

				local_File_Buffers->file_Position_Offset = (local_File_Buffers->file_Position_Offset + bytes_To_Flush);
				local_File_Buffers->buffer_Position_Offset= 0;
								
				local_File_Buffers->ticks_Since_Last_Purge= TickCount();
/*				iO_Queue_Element_Size = FlushFile(open_File_Fork);       */


				if (bytes_To_Flush != previous_Value) {
					Display_Serious_Error_Message(unit_Name, code_Name, "\pFile not fully written!", "\p");
					function_Result = FALSE;
					*result_OS_Error = os_Error;
					goto END_OF_FUNCTION;
				}
				
			} /* if (local_File_Buffers->read_Write_Flags == WRITE_MODE) */
			else if (local_File_Buffers->read_Write_Flags == READ_MODE) {

			/* Destroy buffered data due to request from programmer */
			/* Note: flushing a READ_MODE buffer has several uses. Preserving the current OS file position is important */
				
				if (local_File_Buffers->data_Position_Offset != 0) {
					/* advance file position and shift markers to bottom of array*/
					local_File_Buffers->file_Position_Offset= local_File_Buffers->file_Position_Offset + local_File_Buffers->buffer_Position_Offset ; 
					local_File_Buffers->data_Position_Offset = 0;
				}
				
				local_File_Buffers->buffer_Position_Offset= 0; 

				local_File_Buffers->ticks_Since_Last_Purge= TickCount();
/*				iO_Queue_Element_Size = FlushFile(open_File_Fork);      */

			}
			else
			{
				Display_Serious_Error_Message(unit_Name, code_Name, "\pFile fork buffer mode invalid!", "\p");
				function_Result = FALSE;
				goto END_OF_FUNCTION;
			} /* NOT if (current_Buffer->read_Write_Flags == READ_MODE) */
		
		
		} /* if (local_File_Buffers->buffer_Position_Offset != local_File_Buffers->data_Position_Offset) */;
			
			function_Result = TRUE;
	
END_OF_FUNCTION:
	return(function_Result);
} /* Flush_File_Buffer */
/*=**************************************************************************=*/


/*=**************************************************************************=*/
 Boolean Open_File_Using_Buffer( INTEGER_16		fork_Desired
										, Str255			file_Name_P
										, INTEGER_16		working_Directory_Ref
										, INTEGER_16		*file_Reference_Num
										, INTEGER_32		minimum_Size
										, INTEGER_32		maximum_Size
										, INTEGER_32		desired_Size
										, INTEGER_32		*result_Size
										, Safe_Boolean	read_Only_Acceptable
										, INTEGER_16		*result_OS_Error)
{
	INTEGER_16	function_Result;
/* END VAR */


/*  */
/*  the "Well_Defined" routine will use working_Directory_Ref if */
/* it is set, otherwise uses volume_Reference & directory_ID */
/*  */

	function_Result = Open_Well_Defined_File_Using_Buffer( fork_Desired
										, file_Name_P
										, working_Directory_Ref
										, 0
										, 0
										, file_Reference_Num
										, minimum_Size
										, maximum_Size
										, desired_Size
										, result_Size
										, read_Only_Acceptable
										, result_OS_Error);
	
END_OF_FUNCTION:
	return(function_Result);
	
} /* Open_File_Using_Buffer */
/*=**************************************************************************=*/


/*=**************************************************************************=*/
 Boolean Open_Well_Defined_File_Using_Buffer( INTEGER_16		fork_Desired
										, Str255			file_Name_P
										, INTEGER_16		working_Directory_Ref
										, INTEGER_16 	volume_Reference
										, INTEGER_32 	directory_ID
										, INTEGER_16		*file_Reference_Num
										, INTEGER_32		minimum_Size
										, INTEGER_32		maximum_Size
										, INTEGER_32		desired_Size
										, INTEGER_32		*result_Size
										, Safe_Boolean	read_Only_Acceptable
										, INTEGER_16		*result_OS_Error)
{
/*  */
/*  The "Well_Defined" routine will use working_Directory_Ref if */
/* it is set, otherwise uses volume_Reference & directory_ID. */
/* Note : if "desired_Size" is passed as 0, then an arbitrary 20% */
/* of application heap free memory will be used */
/*  */

	CARDINAL_8			*code_Name;
	INTEGER_16	function_Result;
	Boolean		found_Position;
	INTEGER_32	current_Buffer, file_Buffer_Size;
	
	INTEGER_32			total_Bytes_Contiguous_Now, total_Bytes_Now; /* these variables are for memory management */
	INTEGER_32			total_Bytes_Contiguous_Later, total_Bytes_Later;
	
	INTEGER_16			os_Error;
	HParamBlockRec		hfs_Parameter_Block;
/* END VAR */


	code_Name = "\pOpen_Well_Defined_File_Using_Buffer";
	function_Result = FALSE;
	*result_OS_Error =  noErr;
	*result_Size = 0;

	if ((fork_Desired != DATA_FORK) && (fork_Desired != RESOURCE_FORK)) {
		Display_Serious_Error_Message(unit_Name, code_Name, "\pType of Fork desired invalid!", "\p");
		function_Result = FALSE;
		goto END_OF_FUNCTION;
	}

	if (minimum_Size < MINIMUM_TRANSFER_FILE_BUFFER_SIZE) {
		Display_Serious_Error_Message(unit_Name, code_Name, "\pError with specified buffer minimum.", "\p");
		function_Result = FALSE;
		goto END_OF_FUNCTION;
	}


	if (working_Directory_Ref != 0) {
	
		if (fork_Desired == DATA_FORK) {
			*result_OS_Error = FSOpen(file_Name_P, working_Directory_Ref, file_Reference_Num);
			if ( *result_OS_Error != noErr) {
				Display_Serious_Error_Message(unit_Name, code_Name, "\pError opening file data fork.", "\p");
				function_Result = FALSE;
				goto END_OF_FUNCTION;
			} 
		}
		else
		{
			*result_OS_Error = OpenRF((StringPtr)file_Name_P, working_Directory_Ref, file_Reference_Num);
			if ( *result_OS_Error != noErr) {
				Display_Serious_Error_Message(unit_Name, code_Name, "\pError opening file resource fork.", "\p");
				function_Result = FALSE;
				goto END_OF_FUNCTION;
			} 
		}
	} /* if (working_Directory_Ref != 0) */
	else {
	
			hfs_Parameter_Block.ioParam.ioNamePtr = &file_Name_P[0];
			hfs_Parameter_Block.ioParam.ioVRefNum = volume_Reference;
			hfs_Parameter_Block.fileParam.ioDirID = directory_ID;
			hfs_Parameter_Block.ioParam.ioMisc = nil;
			hfs_Parameter_Block.ioParam.ioCompletion = nil;
			
			if (read_Only_Acceptable == SAFE_TRUE)
				hfs_Parameter_Block.ioParam.ioPermssn = fsRdPerm;
			else
				hfs_Parameter_Block.ioParam.ioPermssn = fsRdWrPerm; /* Read and write is allowed */
			
			if (fork_Desired == DATA_FORK) {
				os_Error = PBHOpen(&hfs_Parameter_Block, FALSE);
			}
			else
			{
				os_Error = PBHOpenRF(&hfs_Parameter_Block, FALSE);

			} /* NOT if (fork_Desired == DATA_FORK) */

			if (os_Error != noErr) {
				if (os_Error == opWrErr) {
					Display_Serious_Error_Message(unit_Name, code_Name, "\pError opening file.", "\pIt is already in use!");
					function_Result = FALSE;
					goto END_OF_FUNCTION;
				}
				else {
					Display_Serious_Error_Message(unit_Name, code_Name, "\pError opening file fork.", "\pStrange. Reboot soon.");
					function_Result = FALSE;
					goto END_OF_FUNCTION;
				}
			}
			
			*file_Reference_Num = hfs_Parameter_Block.ioParam.ioRefNum;
			
			
		} /* NOT if (working_Directory_Ref != 0) */
		
		
		
	/* Note : the following section works in much the same way the "Locate_File_Buffer_Index" routine works */
	current_Buffer = (file_Buffers->total_Open_Files - 1); /* remember, Base 0 */
	for(found_Position = FALSE; (current_Buffer >= 0); current_Buffer-- )  /* contains a break statement */
	{
		if (file_Buffers->list_Of_Open_Fork_IDs[current_Buffer] == BUFFER_NOT_VALID) {
			found_Position = TRUE;
			break; 
		}
	}
	
	if (found_Position != TRUE) {
		if (file_Buffers->total_Open_Files < MAX_OPEN_FORKS) {
			current_Buffer = file_Buffers->total_Open_Files; /* covert to base-0 here */
		}
		else
		{
			Display_Serious_Error_Message(unit_Name, code_Name, "\pExceded MAX_OPEN_FORKS!", "\p");
			function_Result = FALSE;
			goto END_OF_FUNCTION;
		}
	}
	
	
	file_Buffer_Size = desired_Size;
	
	if (Determine_Absolute_Free_Memory(&total_Bytes_Contiguous_Now,&total_Bytes_Now,&total_Bytes_Contiguous_Later,&total_Bytes_Later) != TRUE) {
		Display_Serious_Error_Message(unit_Name, code_Name, "\pcall to Determine_Absolute_Free_Memory", "\p");
		function_Result = FALSE;
		goto END_OF_FUNCTION;
	}
	
	if (file_Buffer_Size == 0) {
	/* Unspecified size... grab a lot extra if possible. Try to use 20% of free memory for each unspecified size file buffer */
		file_Buffer_Size = (((total_Bytes_Contiguous_Later / 5) / MINIMUM_LOGICAL_SECTOR_SIZE) * MINIMUM_LOGICAL_SECTOR_SIZE); 
	}
			
	if (file_Buffer_Size < MINIMUM_TRANSFER_FILE_BUFFER_SIZE) {
		if (total_Bytes_Contiguous_Later > MINIMUM_TRANSFER_FILE_BUFFER_SIZE) {
			file_Buffer_Size = MINIMUM_TRANSFER_FILE_BUFFER_SIZE;
		}
		else
		{
			Display_Serious_Error_Message(unit_Name, code_Name, "\pMemory not available!", "\p");
			function_Result = FALSE;
			goto END_OF_FUNCTION;
		}
	
	}
	
	
	file_Buffers->list_Of_Open_Forks[current_Buffer] = (file_Buffer_Ptr) NewPtr(sizeof( file_Buffer));
	if (MemError() != noErr) {
		Display_Serious_Error_Message(unit_Name, code_Name, "\pmemory not available?", "\p");
		function_Result = FALSE;
		goto END_OF_FUNCTION;
	}
	
	file_Buffers->list_Of_Open_Forks[current_Buffer]->buffer_Address = NewPtr(file_Buffer_Size);
	if (MemError() != noErr) {
		Display_Serious_Error_Message(unit_Name, code_Name, "\pmemory not available?", "\p");
		function_Result = FALSE;
		goto END_OF_FUNCTION;
	}
	else {
		*result_Size = file_Buffer_Size;
	}
	
	if ((current_Buffer +1) > file_Buffers->total_Open_Files)
		file_Buffers->total_Open_Files = (current_Buffer +1);
	
	file_Buffers->list_Of_Open_Fork_IDs[current_Buffer] = *file_Reference_Num;
	file_Buffers->list_Of_Open_Forks[current_Buffer]->buffer_Activated =BUFFER_IN_USE;
	file_Buffers->list_Of_Open_Forks[current_Buffer]->buffer_Size= file_Buffer_Size;
	file_Buffers->list_Of_Open_Forks[current_Buffer]->read_Write_Flags = READ_MODE;
	file_Buffers->list_Of_Open_Forks[current_Buffer]->file_Position_Offset= 0; /* position of buffer to file */
	file_Buffers->list_Of_Open_Forks[current_Buffer]->data_Position_Offset= 0; /* bytes loaded into buffer */
	file_Buffers->list_Of_Open_Forks[current_Buffer]->buffer_Position_Offset= 0; /* position of buffer offset */
	file_Buffers->list_Of_Open_Forks[current_Buffer]->ticks_Since_Last_Purge= TickCount();

	file_Buffers->last_Accessed_OS_File_Reference = *file_Reference_Num; /* for optimization */
	file_Buffers->last_Accessed_Buffer_Index = current_Buffer;

	function_Result = TRUE;

END_OF_FUNCTION:

	if (function_Result != TRUE) {
		if (result_OS_Error == noErr) {
			*result_OS_Error = -1;
		}
	}

	return(function_Result);
} /* Open_Well_Defined_File_Using_Buffer */
/*=**************************************************************************=*/


/*=**************************************************************************=*/
 Boolean Close_File_With_Buffer(	INTEGER_16 		open_File_Fork
										, Boolean			flush_Buffer_Desired
										, INTEGER_16 	*result_OS_Error)
	{
	CARDINAL_8				*code_Name;
	INTEGER_16		function_Result;
	INTEGER_16		os_Error;
	INTEGER_32		current_Buffer;
	file_Buffer_Ptr		local_File_Buffers;
/* END VAR */


	code_Name = "\pClose_File_With_Buffer";

	function_Result = FALSE;
	*result_OS_Error =  noErr;
	
	if (flush_Buffer_Desired == TRUE) {
		if (Flush_File_Buffer( open_File_Fork, result_OS_Error) != TRUE) {
			Display_Serious_Error_Message(unit_Name, code_Name, "\pError flushing a buffered file fork.", "\p");
			*result_OS_Error = os_Error;
			function_Result = FALSE;
			goto END_OF_FUNCTION;
		}
	}
	
	
	/* --- The remainder of the Close_File_With_Buffer routine deletes the management overhead ---*/
	
	if (file_Buffers->last_Accessed_OS_File_Reference == open_File_Fork) {
		current_Buffer = file_Buffers->last_Accessed_Buffer_Index;
	}
	else
	{
		if (Locate_File_Buffer_Index(open_File_Fork, &current_Buffer) != TRUE) {
			Display_Serious_Error_Message(unit_Name, code_Name, "\pFile fork buffer ID not located!", "\p");
			function_Result = FALSE;
			goto END_OF_FUNCTION;
		}
	}

	local_File_Buffers = file_Buffers->list_Of_Open_Forks[current_Buffer];
	DisposPtr(local_File_Buffers->buffer_Address);
	if (MemError() != noErr) {
		Display_Serious_Error_Message(unit_Name, code_Name, "\pBuffer Pointer invalid!", "\p");
		function_Result = FALSE;
		goto END_OF_FUNCTION;
	}

	DisposPtr((Ptr)file_Buffers->list_Of_Open_Forks[current_Buffer]);
	if (MemError() != noErr) {
		Display_Serious_Error_Message(unit_Name, code_Name, "\pcurrent_Buffer pointer invalid!", "\p");
		function_Result = FALSE;
		goto END_OF_FUNCTION;
	}

	file_Buffers->list_Of_Open_Fork_IDs[current_Buffer] = BUFFER_NOT_VALID;
	if (file_Buffers->total_Open_Files == ( current_Buffer + 1)) {
		file_Buffers->total_Open_Files = file_Buffers->total_Open_Files - 1;
	}

	file_Buffers->last_Accessed_OS_File_Reference = 0;
	file_Buffers->last_Accessed_Buffer_Index = 0;

	os_Error = FSClose(open_File_Fork);
	if ( os_Error != noErr) {
		Display_Serious_Error_Message(unit_Name, code_Name, "\pError closing an open file fork", "\p");
		*result_OS_Error = os_Error;
		function_Result = FALSE;
		goto END_OF_FUNCTION;
	}
	
	function_Result = TRUE;


END_OF_FUNCTION:

	if (function_Result != TRUE) {
		if (result_OS_Error == noErr) {
			*result_OS_Error = -1;
		}
	}

	return(function_Result);
	
} /* Close_File_With_Buffer */
/*=**************************************************************************=*/


/*=**************************************************************************=*/
#define	BENCHMARK_SIZE	1

Boolean Copy_File_To_File( INTEGER_16		source_File_Fork
											,INTEGER_16		target_File_Fork
											, INTEGER_32		maximum_Memory_To_Utilize)
{
	CARDINAL_8	*code_Name;
	INTEGER_16		os_Error;
	INTEGER_32		data_Written, data_Read;
	INTEGER_32		transfer_File_Buffer_Size, remainder;
	Boolean			data_Remaining, function_Result;
	Ptr					transfer_File_Buffer;
	
	INTEGER_32		total_Bytes_Contiguous_Now,total_Bytes_Now;
	INTEGER_32		total_Bytes_Contiguous_Later,total_Bytes_Later;
	INTEGER_32		buffer_Read_Offset, work_Long;

	CARDINAL_8	result_String[256];
	CARDINAL_8	temp_String[256]; /*256 bytes each */
	INTEGER_32		loops;
	INTEGER_32		end_Tick_Time,Start_Tick_Time;
/* END VAR */


	code_Name="\pCopy_File_To_File";
	data_Remaining = TRUE;
	function_Result = TRUE;
	
	
	if (Determine_Absolute_Free_Memory(&total_Bytes_Contiguous_Now,&total_Bytes_Now,&total_Bytes_Contiguous_Later,&total_Bytes_Later) != TRUE) {
		Display_Serious_Error_Message(unit_Name, code_Name, "\pcall to Determine_Absolute_Free_Memory", "\p");
		function_Result = FALSE;
		goto END_OF_FUNCTION;
	}

	/* during this routine, grab a lot extra if possible. Try to use 50% of free memory */
	transfer_File_Buffer_Size = (((total_Bytes_Contiguous_Later / 2) / MINIMUM_LOGICAL_SECTOR_SIZE) * MINIMUM_LOGICAL_SECTOR_SIZE);

	if (transfer_File_Buffer_Size > maximum_Memory_To_Utilize)
		transfer_File_Buffer_Size = maximum_Memory_To_Utilize;

	if (transfer_File_Buffer_Size < MINIMUM_TRANSFER_FILE_BUFFER_SIZE) {
		if (total_Bytes_Contiguous_Later > MINIMUM_TRANSFER_FILE_BUFFER_SIZE) {
			transfer_File_Buffer_Size = MINIMUM_TRANSFER_FILE_BUFFER_SIZE;
		}
		else
		{
			Display_Serious_Error_Message(unit_Name, code_Name, "\pMemory not available for buffer!", "\p");
			function_Result = FALSE;
			goto END_OF_FUNCTION;
		}
	
	}

	transfer_File_Buffer = NewPtr(transfer_File_Buffer_Size);
	if (MemError() != noErr) {
		Display_Serious_Error_Message(unit_Name, code_Name, "\pmemory not available?", "\p");
		function_Result = FALSE;
		goto END_OF_FUNCTION;
	}

/*____________________________________________________________*/
	
	while (data_Remaining == TRUE)
	{

#ifdef USE_DEBUG_CODE

/*===================================================================== */
		loops = transfer_File_Buffer_Size / BENCHMARK_SIZE; /* BENCHMARK_SIZE must be divisor of MINIMUM_LOGICAL_SECTOR_SIZE, but > 2 */
		remainder = transfer_File_Buffer_Size - (loops * BENCHMARK_SIZE);
		
		data_Read = BENCHMARK_SIZE;
		buffer_Read_Offset = ( INTEGER_32 )transfer_File_Buffer;
		Start_Tick_Time = TickCount();
		

	while(loops--) {
		/*	if (os_Error = FSRead(source_File_Fork, &data_Read, (CARDINAL_8 *)buffer_Read_Offset) != noErr) { */
		if (Read_File_Using_Buffer(source_File_Fork, &data_Read, (CARDINAL_8 *)buffer_Read_Offset, &os_Error) != TRUE) {
			if ( os_Error != noErr) {
				
					if (os_Error == eofErr) {
						/* end of media */
						data_Remaining = FALSE;
						loops = 0; /* to force loop counter to stop */
					} 
					else
					{
						Display_Serious_Error_Message(unit_Name, code_Name, "\pError reading data for file.", "\p");
						function_Result = FALSE;
						goto END_OF_FUNCTION;
							
					} /* NOT if (os_Error == eofErr) */
				} 
				
				if (data_Read < 0) {
					Display_Serious_Error_Message(unit_Name, code_Name, "\pError reading data for file- Bad total amount read.", "\p");
				}
				
				buffer_Read_Offset = buffer_Read_Offset + data_Read;
				data_Read = BENCHMARK_SIZE; /* reset in case needed due to value change */
				
			} /* if (Read_File_Using_Buffer(source_File_Fork, &data_Read, (CARDINAL_8 *)buffer_Read_Offset, &os_Error) != TRUE) */
			else {
				buffer_Read_Offset = buffer_Read_Offset + BENCHMARK_SIZE;
				
			} /*NOT if (Read_File_Using_Buffer(source_File_Fork, &data_Read, (CARDINAL_8 *)buffer_Read_Offset, &os_Error) != TRUE) */

		} /* while(loops--) */
		
		if((data_Remaining == TRUE) && (remainder > 0)) {
					/* now read the remainder into the temporary buffer */
			data_Read = remainder;
			
			/* if (os_Error = FSRead(source_File_Fork, &data_Read, (CARDINAL_8 *)buffer_Read_Offset) != noErr) { */
			if (Read_File_Using_Buffer(source_File_Fork, &data_Read, (CARDINAL_8 *)buffer_Read_Offset, &os_Error) != TRUE) {
				if ( os_Error != noErr) {
				
					if (os_Error == eofErr) {
						/* end of media */
						data_Remaining = FALSE;
						loops = 0; /* to force loop counter to stop */
					} 
					else
					{
						Display_Serious_Error_Message(unit_Name, code_Name, "\pError reading data for file.", "\p");
						function_Result = FALSE;
						goto END_OF_FUNCTION;
							
					} /* NOT if (os_Error == eofErr) */
				} 
				
				if (data_Read < 0) {
					Display_Serious_Error_Message(unit_Name, code_Name, "\pError  reading data for file- Bad total amount read.", "\p");
				}
				
				buffer_Read_Offset = buffer_Read_Offset + data_Read;
				data_Read = BENCHMARK_SIZE; /* reset in case needed due to value change */
				
			} /* if (Read_File_Using_Buffer(source_File_Fork, &data_Read, (CARDINAL_8 *)buffer_Read_Offset, &os_Error) != TRUE) */
			else {
				buffer_Read_Offset = buffer_Read_Offset + data_Read;
				
			} /*NOT if (Read_File_Using_Buffer(source_File_Fork, &data_Read, (CARDINAL_8 *)buffer_Read_Offset, &os_Error) != TRUE) */

		} /* if((data_Remaining == TRUE)) && (remainder > 0)) */

		
			
		end_Tick_Time = TickCount();
		if ((end_Tick_Time > Start_Tick_Time) || ((end_Tick_Time < 0) && (Start_Tick_Time < 0 ))) {
			end_Tick_Time = end_Tick_Time - Start_Tick_Time;
		}
		else {
			end_Tick_Time = 0; /* no unanticipated answer, but at least a repeatable error ! */
		}
		
		data_Written = (INTEGER_32)(CARDINAL_32)buffer_Read_Offset - (CARDINAL_32)transfer_File_Buffer;

		
/* Data input rate was[xxx] (Bytes/Sec), fetching xxx (Bytes) */

		Copy_Pascal_String ("\pData input rate was [", result_String );
		if((end_Tick_Time != 0) && (data_Written != 0)){
			work_Long = (data_Written * 60.0 / (end_Tick_Time * 1.0));
			NumToString(work_Long , temp_String);
		}
		else {
			Copy_Pascal_String("\p0", temp_String);
		}
			
		Concat_After_Pascal_String(result_String,temp_String);
		Concat_After_Pascal_String(result_String,"\p] (Bytes/Sec), fetching ");
		NumToString(data_Written , temp_String);
		Concat_After_Pascal_String(result_String, temp_String);
		Concat_After_Pascal_String(result_String,"\p (Bytes)");
		
		
		Display_OK_Dialog(result_String);			

#else
/*===================================================================== */
		data_Read = transfer_File_Buffer_Size;
		if (Read_File_Using_Buffer(source_File_Fork, &data_Read, transfer_File_Buffer, &os_Error) != TRUE) {
			if ( os_Error != noErr) {
			
				if (os_Error == eofErr) {
					/* end of media */
					data_Remaining = FALSE;
				} 
				else {
					Display_Serious_Error_Message(unit_Name, code_Name, "\pError reading data for file.", "\p");
					function_Result = FALSE;
					goto END_OF_FUNCTION;
						
				} /* NOT if (os_Error == eofErr) */
			} 
		}
		
	data_Written = data_Read;

/*===================================================================== */
#endif  /*USE_DEBUG_CODE*/



		/* read in some data, try to fill buffer */


		/* write out the data we just got */
	/* if (os_Error = FSWrite(target_File_Fork, &data_Written, transfer_File_Buffer) != noErr) { */
		if (Write_File_Using_Buffer(target_File_Fork, &data_Written, transfer_File_Buffer, &os_Error) != TRUE) {

			/* "os_Error" may be of value for error routine... disk full? etc. */
			Display_Serious_Error_Message(unit_Name, code_Name, "\pError writing data for file", "\p");
			return(FALSE);
		}


/*======================
		if(true == Button()) {
			SysBeep(30);
			Display_Serious_Error_Message(unit_Name, code_Name, "\pUser aborted by pressing mouse during conversion", "\p");
			g_Shell.interface.abort_Program = TRUE;
			function_Result = FALSE;
			goto END_OF_FUNCTION;
		}
============================= */			
			
	} /*while (data_Remaining == TRUE)*/
	
/*____________________________________________________________*/

	
END_OF_FUNCTION:

	DisposPtr(transfer_File_Buffer);
	if (MemError() != noErr) {
		Display_Serious_Error_Message(unit_Name, code_Name, "\ptransfer_File_Buffer invalid!", "\p");
		function_Result = FALSE;
		goto END_OF_FUNCTION;
	}

	return(function_Result);
	
} /* Copy_File_To_File */
/*=**************************************************************************=*/


/*=**************************************************************************=*/
 Boolean Copy_File_To_File_Unbuffered( INTEGER_16 source_File_Fork
											, INTEGER_16 target_File_Fork
											, INTEGER_32 maximum_Memory_To_Utilize)
{
	CARDINAL_8			*code_Name;
	INTEGER_16	os_Error;
	INTEGER_32	data_Written, data_Read;
	INTEGER_32	transfer_File_Buffer_Size, remainder;
	Boolean		data_Remaining, function_Result;
	Ptr				transfer_File_Buffer;
	
	INTEGER_32	total_Bytes_Contiguous_Now,total_Bytes_Now;
	INTEGER_32	total_Bytes_Contiguous_Later,total_Bytes_Later;
/* END VAR */


	code_Name="\pCopy_File_To_File_Unbuffered";
	data_Remaining = TRUE;
	function_Result = TRUE;
	
	
	if (Determine_Absolute_Free_Memory(&total_Bytes_Contiguous_Now,&total_Bytes_Now,&total_Bytes_Contiguous_Later,&total_Bytes_Later) != TRUE) {
		Display_Serious_Error_Message(unit_Name, code_Name, "\pcall to Determine_Absolute_Free_Memory", "\p");
		function_Result = FALSE;
		goto END_OF_FUNCTION;
	}

	/* during this routine, grab a lot extra if possible. Try to use 50% of free memory */
	transfer_File_Buffer_Size = (((total_Bytes_Contiguous_Later / 2) / MINIMUM_LOGICAL_SECTOR_SIZE) * MINIMUM_LOGICAL_SECTOR_SIZE);

	if (transfer_File_Buffer_Size > maximum_Memory_To_Utilize)
		transfer_File_Buffer_Size = maximum_Memory_To_Utilize;

	if (transfer_File_Buffer_Size < MINIMUM_TRANSFER_FILE_BUFFER_SIZE) {
		if (total_Bytes_Contiguous_Later > MINIMUM_TRANSFER_FILE_BUFFER_SIZE) {
			transfer_File_Buffer_Size = MINIMUM_TRANSFER_FILE_BUFFER_SIZE;
		}
		else
		{
			Display_Serious_Error_Message(unit_Name, code_Name, "\pMemory not available for buffer!", "\p");
			function_Result = FALSE;
			goto END_OF_FUNCTION;
		}
	
	}

	transfer_File_Buffer = NewPtr(transfer_File_Buffer_Size);
	if (MemError() != noErr) {
		Display_Serious_Error_Message(unit_Name, code_Name, "\pmemory not available?", "\p");
		function_Result = FALSE;
		goto END_OF_FUNCTION;
	}

/*____________________________________________________________*/
	
	while (data_Remaining == TRUE)
	{
		/*  */
		/* read in some data, try to fill buffer */
		/*  */					
		data_Read = transfer_File_Buffer_Size;
		os_Error = FSRead(source_File_Fork, &data_Read, transfer_File_Buffer);
		if ( os_Error != noErr) {
		
			if (os_Error == eofErr) {
				/* end of media */
				data_Remaining = FALSE;
			} 
			else {
				Display_Serious_Error_Message(unit_Name, code_Name, "\pError reading data for file.", "\p");
				function_Result = FALSE;
				goto END_OF_FUNCTION;
					
			} /* NOT if (os_Error == eofErr) */
		} 
		
		data_Written = data_Read;

		/*  */
		/* write out the data we just got */
		/*  */					
		os_Error = FSWrite(target_File_Fork, &data_Written, transfer_File_Buffer);
		
		if ( os_Error != noErr) {
			Display_Serious_Error_Message(unit_Name, code_Name, "\pError writing data for file", "\p");
			return(FALSE);
		}
			
	} /*while (data_Remaining == TRUE)*/
	
/*____________________________________________________________*/

	
END_OF_FUNCTION:

	DisposPtr(transfer_File_Buffer);
	if (MemError() != noErr) {
		Display_Serious_Error_Message(unit_Name, code_Name, "\ptransfer_File_Buffer invalid!", "\p");
		function_Result = FALSE;
		goto END_OF_FUNCTION;
	}

	return(function_Result);
	
} /* Copy_File_To_File_Unbuffered */
/*=**************************************************************************=*/


/*=**************************************************************************=*/
/* END OF FILE: Buffered_File_Routines.c */

