/* START OF FILE: File_Resource_Utilities.c */


/*=**************************************************************************=*/
/* [---BEGIN---] FILE HEADER COMMENTS */
/**
 ** SCREEN VIEW = [Adobe Syntax Black-10, Tabs = 4]
 ** PRINTER VIEW = [Adobe Syntax Black-10, Tabs = 4]
 ** =======================================================================
 ** Written by James Merkle
 ** File_Resource_Utilities.c - Utility routines regarding resource files
 ** (C) Copyright 1995 James Merkle
 ** =======================================================================
 **
 **
 ** ============
 ** Revision History
 ** ============
 **
 **  1995.05.22		(James Merkle)
 **						Initial coding started from scratch
 **
 **
 **
 ** =================
 ** OTHER INFORMATION
 ** =================
 ** 
 ** 
 **
 **
 **/
/* [----END----] FILE HEADER COMMENTS */
/*=**************************************************************************=*/


/*=**************************************************************************=*/
/* [---BEGIN---] #include AREA */

#include "Shell_Globals.h"

#include "String_Utilities.h"
#include "Error_Utilities.h"
#include "Shell_Dialogs.h"
#include "Shell_Assembly.h"
#include "Volume_Utilities.h"

#include "File_Resource_Utilities.h"

/* [----END----] #include AREA */
/*=**************************************************************************=*/


/*=**************************************************************************=*/
/* [---BEGIN---] #define AREA */


/* [----END----] #define AREA */
/*=**************************************************************************=*/


/*=**************************************************************************=*/
/* [---BEGIN---] TYPE AREA */


/* [----END----] TYPE AREA */
/*=**************************************************************************=*/


/*=**************************************************************************=*/
/* [---BEGIN---] PROTOTYPE AREA */

Safe_Boolean	Save_Pict_As_New_File_BASE(FSSpec	*desired_File
												, PicHandle	picture_Handle
												, Safe_Boolean save_Into_Resource);

/* [----END----] PROTOTYPE AREA */
/*=**************************************************************************=*/


/*=**************************************************************************=*/
/* [---BEGIN---] GLOBAL DEFINITION AREA */


/* [----END----] GLOBAL DEFINITION AREA */
/*=**************************************************************************=*/


/*=**************************************************************************=*/
/* [---BEGIN---] LOCAL STATIC DEFINITION AREA */

static CARDINAL_8	*unit_Name = "\pFile_Resource_Utilities";


/* [----END----] LOCAL STATIC DEFINITION AREA */
/*=**************************************************************************=*/


/*=**************************************************************************=*/
Safe_Boolean Add_Resource_Handle(Handle resource_To_Create_Hdl
											, ResType resource_Type
											, INTEGER_16 resource_ID
											, Str255 resource_Comment)
{
	Safe_Boolean		function_Result;
	Handle			temp_Handle;
	Boolean			still_Deleting;
	CARDINAL_32	deletion_Total;
	CARDINAL_8		*code_Name;
	OSErr					os_Error;
/* END VAR */


	code_Name = "\pAdd_Resource_Handle";
	function_Result = SAFE_FALSE;

	/*  */
	/* Before adding, go ahead and delete any duplicates. */
	/* Keep looping in case multiple resources respond. */
	/* Only loop 10 times in case pathalogical though. */
	/*  */
	
	still_Deleting = TRUE;
	deletion_Total = 0;
	do {
		temp_Handle = Get1Resource(resource_Type, resource_ID);
		os_Error = ResError();
		if ((temp_Handle != nil) && (os_Error == noErr)) {
			RemoveResource(temp_Handle);
			os_Error = ResError();
			if ( os_Error != noErr) {
				Display_Serious_Error_Message(unit_Name
											, code_Name
											, "\pOS Error RemoveResource."
											, "\p");
				goto Error_Handler;
			}
			else {
				deletion_Total += 1;
			}
		}
		else {
			still_Deleting = FALSE;
		}
		
	}while ((still_Deleting == TRUE) && (deletion_Total < 10));
	

	AddResource(resource_To_Create_Hdl, resource_Type, resource_ID, resource_Comment);
	os_Error = ResError();
	if ( os_Error != noErr) {
		Display_Serious_Error_Message(unit_Name, code_Name, "\pOS Error AddResource.", "\p");
		goto Error_Handler;
	} 
		
	ChangedResource(resource_To_Create_Hdl);
	os_Error = ResError();
	if ( os_Error != noErr) {
		Display_Serious_Error_Message(unit_Name, code_Name, "\pOS Error OpenResFile.", "\p");
		goto Error_Handler;
	} 

	WriteResource(resource_To_Create_Hdl);
	os_Error = ResError();
	if ( os_Error != noErr) {
		Display_Serious_Error_Message(unit_Name, code_Name, "\pOS Error OpenResFile.", "\p");
		goto Error_Handler;
	} 
	
	DetachResource(resource_To_Create_Hdl);
	os_Error = ResError();
	if ( os_Error != noErr) {
		Display_Serious_Error_Message(unit_Name, code_Name, "\pOS Error WriteResouce.", "\p");
		goto Error_Handler;
	} 
	
	function_Result = SAFE_TRUE;
	goto Normal_Function_Exit;
	
	
Error_Handler:
	function_Result = SAFE_FALSE;
		
Normal_Function_Exit:
	return(function_Result);
	
}
/*=**************************************************************************=*/


/*=**************************************************************************=*/
Safe_Boolean	Save_Pict_Resource_As_New_File(FSSpec	*desired_File
													, PicHandle	picture_Handle)
{
	Safe_Boolean		function_Result;
/* END VAR */


	function_Result = Save_Pict_As_New_File_BASE(desired_File, picture_Handle, SAFE_TRUE);
		
	return(function_Result);
	
} /* Safe_Boolean	Save_Pict_Resource_As_New_File */
/*=**************************************************************************=*/


/*=**************************************************************************=*/
Safe_Boolean	Save_Pict_As_New_Pict_File(FSSpec	*desired_File
													, PicHandle	picture_Handle)
{
	Safe_Boolean		function_Result;
/* END VAR */


	function_Result = Save_Pict_As_New_File_BASE(desired_File, picture_Handle, SAFE_FALSE);
		
	return(function_Result);
	
} /* Safe_Boolean	Save_Pict_Resource_As_New_File */
/*=**************************************************************************=*/


/*=**************************************************************************=*/
Safe_Boolean	Save_Pict_As_New_File_BASE(FSSpec	*desired_File
												, PicHandle	picture_Handle
												, Safe_Boolean save_Into_Resource)
{
	INTEGER_16		file_Reference;
	INTEGER_16		resource_ID;
	WDPBRec			working_Directory_PB;
	INTEGER_16		working_Directory;
	CARDINAL_32	file_Creator;
	CARDINAL_32	file_Type;
	CARDINAL_32	resource_Type;
	CARDINAL_32	temp_Size;
	CARDINAL_32	file_Minimum_Size;
	CARDINAL_32	total_To_Write;
	PictInfo			pict_Data;
	OSErr				os_Error;
	INTEGER_32		transfer_Count;
	INTEGER_32		transfer_Count_Copy;
	INTEGER_16		scrap_Volume_Reference_Number;
	CARDINAL_32	volume_Space;
	CARDINAL_32	total_Handle_Size_To_Write;
	Safe_Boolean		function_Result;
	Safe_Boolean		working_Directory_In_Use;
	Str255				volume_Name;
	Str255				work_String_2;
	Str255				resource_Comment;
	Str255				note_String;
	CARDINAL_8		*code_Name;
/* END VAR */


	code_Name = "\pSave_Pict_Resource_As_New_File";
	function_Result = SAFE_FALSE;
	working_Directory_In_Use = SAFE_FALSE;

	if (picture_Handle == nil) {
		goto Error_Handler;
	}

	file_Minimum_Size = 0;
	total_Handle_Size_To_Write = 0;
	
	if (EmptyRect(&(**picture_Handle).picFrame) == TRUE) {
		goto Error_Handler;
	}
	else {
		/*  */
		/* Add in the sizes of Picture */
		/*  */
		temp_Size = GetHandleSize((Handle)picture_Handle);
		
		if (MemError() != noErr) {
			goto Error_Handler;
		}
		else {
			file_Minimum_Size += temp_Size;
			total_Handle_Size_To_Write = temp_Size;
		}
	}

	note_String[0] = 0;
	volume_Name[0] = 0;

	os_Error = GetVInfo((Get_Drive_Number(desired_File->vRefNum)), volume_Name, &scrap_Volume_Reference_Number, (INTEGER_32 *)&volume_Space);
	if (os_Error != noErr) {
		Concat_Many(note_String, 3, "\pGetVInfo for volume ", volume_Name, "\p ID = ");
		Concat_Long_To_String(note_String, os_Error, NO_LIMIT);
		goto Error_Handler_With_Message;
	} /* if (os_Error != noErr) */

	if (((CARDINAL_32)file_Minimum_Size + (CARDINAL_32)MINIMUM_DISK_SPACE_TO_PRESERVE) > volume_Space) {

			Concat_Many(note_String, 3, "\p", volume_Name, "\p is too full for the file. (");
			Make_String_Long_String(work_String_2, (CARDINAL_8 *)note_String, volume_Space, NO_LIMIT,"\p)bytes free, (" );
			Make_String_Long_String(note_String
										, (CARDINAL_8 *)work_String_2
										, (file_Minimum_Size + MINIMUM_DISK_SPACE_TO_PRESERVE)
										, NO_LIMIT
										,"\p)bytes needed.  Find additional space for file." );

		goto Error_Handler_With_Message;
	}


	file_Creator= '8BIM';

	if (save_Into_Resource == SAFE_TRUE) {
		file_Type = 'SCRN';
	}
	else {
		file_Type = 'PICT';
	}
	
	note_String[0] = 0;

	os_Error = HCreate(desired_File->vRefNum
									, desired_File->parID
									, desired_File->name
									, file_Creator
									, file_Type);
						
	if (os_Error == dupFNErr) {
		os_Error = HDelete(desired_File->vRefNum
									, desired_File->parID
									, desired_File->name);
		if (os_Error != noErr) {
			Concat_Many(note_String, 3,"\pFSDelete for file ", desired_File->name, "\p ID = ");
			Concat_Long_To_String(note_String, os_Error, NO_LIMIT);
			goto Error_Handler_With_Message;
		}

		os_Error = HCreate(desired_File->vRefNum
									, desired_File->parID
									, desired_File->name
							, file_Creator
							, file_Type);
	} /* if (os_Error == dupFNErr) */

	if (os_Error != noErr) {
		Concat_Many(note_String, 3,"\pCreate for file ", desired_File->name, "\p ID = ");
		Concat_Long_To_String(note_String, os_Error, NO_LIMIT);
		goto Error_Handler_With_Message;
	} 

	/*  */
	/* Crappy deficient OpenRFPerm needs ancient Working Directory */
	/*  */
	working_Directory_PB.ioCompletion = nil;
	working_Directory_PB.ioNamePtr = nil;
	working_Directory_PB.ioWDProcID = FILE_CREATOR;
	working_Directory_PB.ioVRefNum = desired_File->vRefNum;
	working_Directory_PB.ioWDDirID = desired_File->parID;
	
	os_Error = PBOpenWD(&working_Directory_PB, FALSE);
	if (os_Error != noErr) {
		goto Error_Handler;
	}
	working_Directory_In_Use = SAFE_TRUE;
	working_Directory = working_Directory_PB.ioVRefNum;

	if (save_Into_Resource == SAFE_TRUE) {
		HCreateResFile(desired_File->vRefNum
										, desired_File->parID
										, desired_File->name);
		os_Error = ResError();
		if (os_Error != noErr) {
			Concat_Many(note_String, 3,"\pCreateResFile for file ", desired_File->name, "\p ID = ");
			Concat_Long_To_String(note_String, os_Error, NO_LIMIT);
			goto Error_Handler_With_Message;
		} 
		
		/*=--------------------------------------------------------------------------=*/
		/* 'OpenRFPerm' used to prevent having to use 'SetVol' */
		/* 'StripAddress' called as per tech notes and THINK reference */
		/*=--------------------------------------------------------------------------=*/
		file_Reference = OpenRFPerm(((StringPtr)(StripAddress(
															&(desired_File->name[0]))))
															, working_Directory
															, fsRdWrPerm );
	
		os_Error = ResError();
		if ((file_Reference == -1) || ( os_Error != noErr)) {
			Display_Serious_Error_Message(unit_Name, code_Name, "\pOS Error OpenResFile.", "\pNo resource fork?");
			goto Error_Handler;
		}
	
		resource_Type = 'PICT';
		resource_ID = 128;
	
		if (Add_Resource_Handle((Handle)picture_Handle
												, resource_Type
												, resource_ID
												, "\pBackdrop picture") != SAFE_TRUE) {
			goto Error_Handler;
		}
	
		CloseResFile(file_Reference);
		os_Error = ResError();
		if ( os_Error != noErr) {
			Display_Serious_Error_Message(unit_Name, code_Name, "\pOS Error CloseResFile.", "\p");
		}
	
		file_Reference = 0;
	} /* if (save_Into_Resource == SAFE_TRUE) */
	else {
		/*  */
		/* Spool into pict file. */
		/*  */
	
		os_Error = HOpenDF(desired_File->vRefNum
								, desired_File->parID
								, desired_File->name
								, fsRdWrPerm
								, &file_Reference);
		if ( os_Error != noErr) {
			Display_Serious_Error_Message(unit_Name, code_Name, "\pOS Error HOpenDF.", "\p");
		}

		Fill_Byte_Memory_With_Byte((Ptr)&work_String_2[0], 256, 0);
		total_To_Write = 256;
		os_Error = FSWrite(file_Reference, (INTEGER_32 *)&total_To_Write, &work_String_2[0]);
		if ( os_Error != noErr) {
			goto Error_Handler_IO_With_Message;
		}
		
		total_To_Write = 256;
		os_Error = FSWrite(file_Reference, (INTEGER_32 *)&total_To_Write, &work_String_2[0]);
		if ( os_Error != noErr) {
			goto Error_Handler_IO_With_Message;
		}
		
		total_To_Write = total_Handle_Size_To_Write;
		os_Error = FSWrite(file_Reference, (INTEGER_32 *)&total_To_Write, *(Handle)picture_Handle);
		if ( os_Error != noErr) {
			goto Error_Handler_IO_With_Message;
		}
		
		/*  */
		/* Go back to start of file and write out file header. */
		/*  */
		os_Error = SetFPos(file_Reference, fsFromStart, 0);
		if ( os_Error != noErr) {
			Concat_Many(note_String, 3,"\pFSPos for file ", desired_File->name, "\p ID = ");
			Concat_Long_To_String(note_String, os_Error, NO_LIMIT);
			os_Error = FSClose(file_Reference);
			file_Reference = 0;
			goto Error_Handler_With_Message;
		}
		
		
		os_Error = GetPictInfo(picture_Handle
								, &pict_Data
								, 0
								, 0
								, systemMethod
								, 0);
		if ( os_Error == noErr) {
		
			pict_Data.thePalette = 0;
			pict_Data.theColorTable = 0;
			pict_Data.commentHandle = 0;
			pict_Data.fontHandle = 0;
			pict_Data.fontNamesHandle = 0;
		
			total_To_Write = sizeof(pict_Data);
			os_Error = FSWrite(file_Reference, (INTEGER_32 *)&total_To_Write, (Ptr)&pict_Data);
			if ( os_Error != noErr) {
				goto Error_Handler_IO_With_Message;
			}
		}
		
		os_Error = FSClose(file_Reference);
		if ( os_Error != noErr) {
			Display_Serious_Error_Message(unit_Name, code_Name, "\pOS Error FSClose.", "\p");
		}
		file_Reference = 0;
		
	} /* NOT  if (save_Into_Resource == SAFE_TRUE) */


	function_Result = SAFE_TRUE;
	goto Normal_Function_Exit;

Error_Handler_IO_With_Message:
	Concat_Many(note_String, 3,"\pFSWrite for file ", desired_File->name, "\p ID = ");
	Concat_Long_To_String(note_String, os_Error, NO_LIMIT);
	os_Error = FSClose(file_Reference);
	file_Reference = 0;
	goto Error_Handler_With_Message;

Error_Handler_With_Message:
	function_Result = SAFE_FALSE;
	Display_Serious_Error_Message(unit_Name, code_Name, note_String, "\p");
	goto Normal_Function_Exit;

Error_Handler:
	Display_Serious_Error_Message(unit_Name, code_Name, "\pInternal problem saving Pict file", "\p");
	function_Result = SAFE_FALSE;
		
Normal_Function_Exit:
		
	/*  */
	/* Placed here to ensure not accidentally ever left allocated */
	/*  */
	if (working_Directory_In_Use == SAFE_TRUE) {
		os_Error = CloseWD(working_Directory);
		if (os_Error != noErr) {
			goto Error_Handler;
		}
	}
			
	return(function_Result);
	
} /* Safe_Boolean	Save_Pict_Resource_As_New_File... */
/*=**************************************************************************=*/
	
	
/*=**************************************************************************=*/				
Safe_Boolean	Get_File_Specification_For_Save(FSSpec		*file_Specification
															, CARDINAL_8	*assumed_File_Name
															, CARDINAL_8	*message_String
															, Safe_Boolean *user_Abort)
{
	INTEGER_16 		number_Of_Types;
	Boolean				user_Clicked_Ok;
	Str63					original_Name;
	SFTypeList 			type_List;
	SFReply				sf_Reply;
	Safe_Boolean		function_Result;
/* END VAR */


	function_Result = SAFE_FALSE;
	*user_Abort = SAFE_FALSE;
	
	if ((assumed_File_Name == nil) || (message_String == nil)) {
		goto Error_Handler;
	}
	
	Assign_String(original_Name, assumed_File_Name);
	if(original_Name[0] >31) {
		original_Name[0] = 31;
	}

	user_Clicked_Ok = Perform_Put_File_Dialog(message_String
										, original_Name
										, &sf_Reply);
	
	if ((user_Clicked_Ok != TRUE) || (sf_Reply.good != TRUE)) {
		/*  */
		/* User Abort, no file name */
		/*  */
		*user_Abort = SAFE_TRUE;
		function_Result = SAFE_FALSE;
		goto Normal_Function_Exit;
	}

	Get_Last_Volume_Reference_And_Directory_ID(&(file_Specification->vRefNum)
																	, &(file_Specification->parID));
																	
	Copy_Pascal_String(sf_Reply.fName, file_Specification->name);
		
	function_Result = SAFE_TRUE;
	goto Normal_Function_Exit;

Error_Handler:
	function_Result = SAFE_FALSE;
		
Normal_Function_Exit:
	return(function_Result);
	
} /* Safe_Boolean	Get_File_Specification_For_Save... */
/*=**************************************************************************=*/


/*=**************************************************************************=*/
/* END OF FILE: File_Resource_Utilities.c */






