/* START OF FILE: OS_Extra_Utilities.c */


/*=**************************************************************************=*/
/* [---BEGIN---] FILE HEADER COMMENTS */
/**
 ** SCREEN VIEW = [Adobe Syntax Black-10, Tabs = 6]
 ** PRINTER VIEW = [Adobe Syntax Black-10, Tabs = 6]
 ** =======================================================================
 ** Written by James Merkle
 ** OS_Extra_Utilities.c - More Miscellaneous utilities related to operating system functions
 ** current version (C) Copyright 1993 James Merkle
 ** All rights reserved, except those wishing to "copy" moderate sections of code or files may do so, so long as
 ** such code is never used directly either in animation-related tools or multimedia tools that are  
 ** used in live performance or live broadcast or recorded mass market broadcast or film. Desktop usage not restricted.
 ** Conventional desktop game or entertainment not restricted.
 ** =======================================================================
 **
 **
 ** ============
 ** Revision History
 ** ============
 **
 **  1992.02.19		(James Merkle)
 **						Split from out of  OS_Utilities
 **
 **
 **
 **
 ** =================
 ** OTHER INFORMATION
 ** =================
 ** 
 **
 **/
/* [----END----] FILE HEADER COMMENTS */
/*=**************************************************************************=*/


/*=**************************************************************************=*/
/* [---BEGIN---] FILE INCLUDE AREA */

#include <LoMem.h>
#include <FixMath.h>
#include <GestaltEqu.h>
#include <Traps.h>

#include "Error_Utilities.h"
#include "OS_Utilities.h"
#include "Shell_Dialogs.h"
#include "Shell_Globals.h"
#include "Time_Utilities.h"
#include "String_Utilities.h"

#include "OS_Extra_Utilities.h"

/* [----END----] FILE INCLUDE AREA */
/*=**************************************************************************=*/


/*=**************************************************************************=*/
/* [---BEGIN---] DEFINE AREA */


#define 		GESTALT_FUNCTION_RES_TYPE	'GDEF'
#define		GESTALT_FUNCTION_RES_ID		129

/*=--------------------------------------------------------------------------=*/
/* page 193 TMON pro Reference to calculate TMON_OFFSET_FOR_ENTRY_ADDRESS */
/*=--------------------------------------------------------------------------=*/
#define	TMON_GESTALT_SELECTOR				'TMON'
#define	TMON_NORMAL_FULL_ENTRY				'MON '
#define	TMON_OFFSET_FOR_ENTRY_ADDRESS	24


/* [----END----] DEFINE AREA */
/*=**************************************************************************=*/


/*=**************************************************************************=*/
/* [---BEGIN---] TYPE AREA */


/* [----END----] TYPE AREA */
/*=**************************************************************************=*/


/*=**************************************************************************=*/
/* [---BEGIN---] PROTOTYPE AREA */

Safe_Boolean Blind_Search_TMON_Entry_Point ( Ptr	*address_Of_Monitor_Entry, Safe_Boolean	find_MacJmp_Location);

pascal void Invoke_TMON_Using_Monitor_Commands( Ptr	address_Of_Monitor_Entry
										, CARDINAL_32	*result_Code
										, Safe_Boolean	find_MacJmp_Location);
										
Safe_Boolean Determine_TMON_Entry_Point ( Ptr	*address_Of_Monitor_Entry );


/* [----END----] PROTOTYPE AREA */
/*=**************************************************************************=*/


/*=**************************************************************************=*/
/* [---BEGIN---] GLOBAL DEFINITION AREA */


/* [----END----] GLOBAL DEFINITION AREA */
/*=**************************************************************************=*/


/*=**************************************************************************=*/
/* [---BEGIN---] LOCAL STATIC DEFINITION AREA */

static CARDINAL_8				*unit_Name = "\pOS_Extra_Utilities";

/* [----END----] LOCAL STATIC DEFINITION AREA */
/*=**************************************************************************=*/


/*=**************************************************************************=*/
Safe_Boolean	Post_Gestalt_Breadcrumbs_If_Needed ( OSType Application_ID
									, Str63				stored_File_Name
									, INTEGER_16		stored_Volume_Reference_Num
									, INTEGER_32		stored_Directory_ID	
									, Safe_Boolean 	*already_Present )
{
	Safe_Boolean					function_Result;
	modify_GDEF_Code_Ptr		thePtr;
	INTEGER_16					os_Error;
	INTEGER_32					returned_Value;
	SysEnvRec					system_Environment;
	CARDINAL_32				memory_Requested;
	Handle						gestalt_Function_Hdl;
	Memory_Allocation			gestalt_Function_Memory_Initialized;
	Handle 						temp_Handle;
/* END VAR */

	function_Result = SAFE_FALSE;
	*already_Present = SAFE_FALSE;


	os_Error = SysEnvirons(SYSTEM_0605, &system_Environment);
	if (os_Error != noErr) {
		function_Result = SAFE_FALSE;
		goto Normal_Function_Exit;
	}
	
	if (Gestalt_Available() != TRUE) {
		function_Result = SAFE_FALSE;
		goto Normal_Function_Exit;
	}


	os_Error = Gestalt(Application_ID, &returned_Value) ;
	if (os_Error != noErr) {
		if (os_Error == gestaltUndefSelectorErr) {
			/*=--------------------------------------------------------------------------=*/
			/* It is not registered yet! good. continue */
			/*=--------------------------------------------------------------------------=*/
		}
		else {
			goto Error_Handler;
		}
	}
	else {
			/*=--------------------------------------------------------------------------=*/
			/* It is already registered! good. */
			/*=--------------------------------------------------------------------------=*/
			function_Result = SAFE_TRUE;
			*already_Present = SAFE_TRUE;
			goto Normal_Function_Exit;
	}


	/*=--------------------------------------------------------------------------=*/
	/* Note the GDEF does not need to have resource bits set for System heap any longer. */
	/* This is because multiple copies of the resource need to be made anyway because */
	/* the resource is modified and used by many potential programs. */
	/* It is not harmful to be set for system heap, but for logical use of compression code, */
	/* it should be in application heap */
	/*=--------------------------------------------------------------------------=*/
	
	temp_Handle = Get1Resource(GESTALT_FUNCTION_RES_TYPE, GESTALT_FUNCTION_RES_ID);
	if ((temp_Handle == nil) || (ResError() != noErr)){
		goto Error_Handler;
	}


	gestalt_Function_Memory_Initialized = MEMORY_NOT_INITIALIZED;
	gestalt_Function_Hdl = nil;
	memory_Requested = GetHandleSize(temp_Handle);
	if (MemError() != noErr)
		goto Error_Handler;

	if ( Create_System_Handle(&memory_Requested
									, 0L
									, ALLOW_PURGING
									, DISALLOW_PARTIAL_AMOUNT
									, &gestalt_Function_Hdl
									, &gestalt_Function_Memory_Initialized) != SAFE_TRUE ){
		goto Error_Handler;
	}

		/*=--------------------------------------------------------------------------=*/
		/* Simple paranoia check */
		/*=--------------------------------------------------------------------------=*/
		if (GetHandleSize(gestalt_Function_Hdl) != GetHandleSize(temp_Handle)) {
		
		}

		/*=--------------------------------------------------------------------------=*/
		/* copy GDEF from resource to system heap copy and release resource. */
		/*=--------------------------------------------------------------------------=*/
	
	BlockMove(*temp_Handle, *gestalt_Function_Hdl, memory_Requested);
	
	ReleaseResource(temp_Handle);
	if (ResError() != noErr)
		goto Error_Handler;
	
		/*=--------------------------------------------------------------------------=*/
		/* I, JM, have observed that the function MoveHHi will usually fail to move the handle */
		/* much higher than it is created originally in 32bit sys 7. In fact many unlocked handles */
		/* are above it, but Apple probably knows what those things are and why they are there. */
		/* Oddly resources can be loaded and locked Low in system heap though */ 
		/*=--------------------------------------------------------------------------=*/
	MoveHHi(gestalt_Function_Hdl);
	if (MemError() != noErr)
		goto Error_Handler;

	HNoPurge(gestalt_Function_Hdl);
	if (MemError() != noErr)
		goto Error_Handler;

	HLock(gestalt_Function_Hdl);
	if (MemError() != noErr)
		goto Error_Handler;

	thePtr = (modify_GDEF_Code_Ptr) *(gestalt_Function_Hdl);
	/*=--------------------------------------------------------------------------=*/
	/* Naughty self modifying code that saves our info into the GDEF */
	/*=--------------------------------------------------------------------------=*/
	thePtr->volume_Reference_Num = stored_Volume_Reference_Num;
	thePtr->directory_ID = stored_Directory_ID;
	BlockMove((Ptr) stored_File_Name, &(thePtr->file_Name), 64);

	
#if !defined(ORIGINAL_MAC_SYSTEM)
#error "UNDEFINED PARAMETER!"
#endif
#if (ORIGINAL_MAC_SYSTEM == 0)
XXXX 		os_Error = NewGestalt(Application_ID, (ProcPtr) StripAddress((Ptr) *gestalt_Function_Hdl));
#else
	os_Error = NewGestalt(Application_ID, (SelectorFunctionUPP) StripAddress((Ptr) *gestalt_Function_Hdl));
#endif 
	
	if (os_Error != noErr) {
		if (os_Error == gestaltDupSelectorErr) {
			/*=--------------------------------------------------------------------------=*/
			/* Although checked only a moment ago, It was registered behind us! ok. */
			/*=--------------------------------------------------------------------------=*/
			function_Result = SAFE_TRUE;
			*already_Present = SAFE_TRUE;
			
			DisposHandle(gestalt_Function_Hdl);
			if (MemError() != noErr) {
				goto Error_Handler;
			}

			goto Normal_Function_Exit;
		}
		else {
			function_Result = SAFE_FALSE;
			goto Normal_Function_Exit;
		}
	}

	*already_Present = SAFE_FALSE;
	function_Result = SAFE_TRUE;
	goto Normal_Function_Exit;

Error_Handler:
		/*=--------------------------------------------------------------------------=*/
		/* no room for elaborate error message in this program, call SysBeep. */
		/*=--------------------------------------------------------------------------=*/
		SysBeep(20);
		function_Result = SAFE_FALSE;

Normal_Function_Exit:
	return(function_Result);
	
} /* Post_Gestalt_Breadcrumbs_If_Needed */
/*=**************************************************************************=*/


/*=**************************************************************************=*/
void Reset_Parameter_Ram_Settings_For_Serial_Ports(void)
{
	SysParmType		system_Parameters;
/* END VAR */

	system_Parameters = *(GetSysPPtr());
	
	system_Parameters.aTalkA = 0x00;
	system_Parameters.aTalkB = 0x00;
	
	system_Parameters.portA = 0xCC0A;
	system_Parameters.portB = 0xCC0A;
	
	*(GetSysPPtr()) = system_Parameters;
	
	WriteParam();


} /* void Reset_Parameter_Ram_Settings_For_Serial_Ports(void) */
/*=**************************************************************************=*/


/*=**************************************************************************=*/
Safe_Boolean Blind_Search_TMON_Entry_Point ( Ptr	*address_Of_Monitor_Entry, Safe_Boolean	find_MacJmp_Location)
	/*=--------------------------------------------------------------------------=*/
	/* BRUTE FORCE SEARCH FOR RESIDUE OF PREVIOUS TMON */
	/* FROM EARLIER LIFE.  This is for very special circumstances. */
	/* Patches will not be present, but TMON 3.00 should still function if it*/
	/* is complete in memory and not written over yet. */
	/* This routine is meant for finding TMON after a warm reset */
	/*=--------------------------------------------------------------------------=*/
{
	CARDINAL_32	current_Address, final_Address;
	SysEnvRec		system_Environment;
	INTEGER_16		os_Error;
	Safe_Boolean		function_Result;
	CARDINAL_32	long_Word_1,long_Word_2,long_Word_3,long_Word_4;
/* END VAR */

#define BYTES_TO_SCREW_WITH_MEMTOP_CRAP	5000000L

#define TMON_PROPER_ENTRY_WORD_1	0x9EFC0028
#define TMON_PROPER_ENTRY_WORD_2	0x486F0028
#define TMON_PROPER_ENTRY_WORD_3	0x48E7FFFE
#define TMON_PROPER_ENTRY_WORD_4	0x244F42EA

#define TMON_RAW_ENTRY_WORD_1	0x4A6F0004
#define TMON_RAW_ENTRY_WORD_2	0x6616201F
#define TMON_RAW_ENTRY_WORD_3	0x544F4EF9
#define TMON_RAW_ENTRY_WORD_4	0x0009EFAC

#define MACSBUG_RAW_ENTRY_WORD_1	0x4A6F0004
#define MACSBUG_RAW_ENTRY_WORD_2	0x6616201F
#define MACSBUG_RAW_ENTRY_WORD_3	0x544F4EF9
#define MACSBUG_RAW_ENTRY_WORD_4	0x0009EFAC


#define USE_TMON_MAC_JUMP_NOT_MACSBUG	SAFE_TRUE


	if (find_MacJmp_Location != SAFE_TRUE) {
	
		long_Word_1 = TMON_PROPER_ENTRY_WORD_1;
		long_Word_2 = TMON_PROPER_ENTRY_WORD_2;
		long_Word_3 = TMON_PROPER_ENTRY_WORD_3;
		long_Word_4 = TMON_PROPER_ENTRY_WORD_4;
		
	}
	else {
		if ( USE_TMON_MAC_JUMP_NOT_MACSBUG == SAFE_TRUE) {
			long_Word_1 = TMON_RAW_ENTRY_WORD_1;
			long_Word_2 = TMON_RAW_ENTRY_WORD_2;
			long_Word_3 = TMON_RAW_ENTRY_WORD_3;
			long_Word_4 = TMON_RAW_ENTRY_WORD_4; /* Actually not compared */
		}
		else {
			long_Word_1 = MACSBUG_RAW_ENTRY_WORD_1;
			long_Word_2 = MACSBUG_RAW_ENTRY_WORD_2;
			long_Word_3 = MACSBUG_RAW_ENTRY_WORD_3;
			long_Word_4 = MACSBUG_RAW_ENTRY_WORD_4;
		}
	}
	


	*address_Of_Monitor_Entry = 0;

	os_Error = SysEnvirons(SYSTEM_0605, &system_Environment);
	if (os_Error != noErr) {
		final_Address = 30000000L; /* until good way to use sysenvirons to get (ABSOLUTELY REAL) memory size possible */
	}
	else {
		switch (system_Environment.machineType) {
			case gestaltClassic:
			case gestaltMacXL:
			case gestaltMac512KE:
			case gestaltMacSE:
					final_Address = (CARDINAL_32)MemTop + (CARDINAL_32)BYTES_TO_SCREW_WITH_MEMTOP_CRAP;
					if ( final_Address > 4194304L)
						final_Address = 4194304L; /* four meg max memory size possible */
				break;
				
			case gestaltMacClassic:
			case gestaltPortable:
					final_Address = (CARDINAL_32)MemTop + (CARDINAL_32)BYTES_TO_SCREW_WITH_MEMTOP_CRAP;
					if ( final_Address > 8388608L)
						final_Address = 8388608L; /* 8 meg max memory size desired to search */
				break;
	
			case gestaltMacII:
			case gestaltMacIIx:
			case gestaltMacIIcx:
			case gestaltMacSE030:
			case gestaltMacIIci:
			case gestaltMacIIfx:
			case gestaltMacIIsi:
			case gestaltMacLC:
			default:
					/* final_Address = (CARDINAL_32)MemTop + (CARDINAL_32)BYTES_TO_SCREW_WITH_MEMTOP_CRAP; */
					/* final_Address = (CARDINAL_32)(20971520L); */ /* 20 Meg */
					final_Address = (CARDINAL_32)(8388608L); /* 8 Meg */
					
					if ( final_Address > 8388608L)
						final_Address = 8388608L; /* 8 meg max memory size desired to search */
				break;
				
		} /* switch (machine_Type) */

	}
	
	final_Address = ((CARDINAL_32)final_Address - (CARDINAL_32)16); /* subtract LONG_WORD_TO_LOCATE's */

	
	for ( current_Address = 2048; (current_Address < final_Address); current_Address += 4 ) {
		
		if ( (*((CARDINAL_32 *)current_Address)) == long_Word_1) {
			if ( (*((CARDINAL_32 *)(current_Address + 4))) == long_Word_2) {
				if ( (*((CARDINAL_32 *)(current_Address + 8))) == long_Word_3) {
				
					if ((find_MacJmp_Location != SAFE_TRUE) || (USE_TMON_MAC_JUMP_NOT_MACSBUG != SAFE_TRUE)) {
						if ( (*((CARDINAL_32 *)(current_Address + 12))) == long_Word_4) {
						
							break;
						}
					}
					else {
							break;
					}
				}
			}
		}
	}

	if (current_Address >= final_Address) {
		goto Error_Handler;
	}

	/*=--------------------------------------------------------------------------=*/
	/* TMON is probably located at this point!. */
	/*=--------------------------------------------------------------------------=*/
	*address_Of_Monitor_Entry = (Ptr)current_Address;
	
	function_Result = SAFE_TRUE;
	goto Normal_Function_Exit;
	
Error_Handler:
	function_Result = SAFE_FALSE;

Normal_Function_Exit:
	return(function_Result);
	
} /* Safe_Boolean	Blind_Search_TMON_Entry_Point ( Ptr	*address_Of_Monitor_Entry ) */
/*=**************************************************************************=*/


/*=**************************************************************************=*/
Safe_Boolean Determine_TMON_Entry_Point ( Ptr	*address_Of_Monitor_Entry )
{
	Safe_Boolean					function_Result;
	INTEGER_16					os_Error;
	INTEGER_32					returned_Value;
	SysEnvRec					system_Environment;
/* END VAR */

	function_Result = SAFE_FALSE;
	*address_Of_Monitor_Entry = nil;

	os_Error = SysEnvirons(SYSTEM_0605, &system_Environment);
	if (os_Error != noErr) {
		function_Result = SAFE_FALSE;
		goto Normal_Function_Exit;
	}
	
	if (Gestalt_Available() != TRUE) {
		function_Result = SAFE_FALSE;
		goto Normal_Function_Exit;
	}

	os_Error = Gestalt(TMON_GESTALT_SELECTOR, &returned_Value) ;
	if (os_Error != noErr) {
		if (os_Error == gestaltUndefSelectorErr) {
			/*=--------------------------------------------------------------------------=*/
			/* It is not registered yet! good. continue */
			/*=--------------------------------------------------------------------------=*/
		}
		else {
			goto Error_Handler;
		}
	}
	
	/*=--------------------------------------------------------------------------=*/
	/* TMON is currently registered! good. */
	/*=--------------------------------------------------------------------------=*/
	*address_Of_Monitor_Entry = *((Ptr *)((CARDINAL_32)returned_Value + (CARDINAL_32)TMON_OFFSET_FOR_ENTRY_ADDRESS));
	
	function_Result = SAFE_TRUE;
	goto Normal_Function_Exit;
	
Error_Handler:
	function_Result = SAFE_FALSE;

Normal_Function_Exit:
	return(function_Result);
	
} /* Safe_Boolean	Determine_TMON_Entry_Point ( Ptr	*address_Of_Monitor_Entry ) */
/*=**************************************************************************=*/


/*=**************************************************************************=*/
pascal void Invoke_TMON_Using_Monitor_Commands( Ptr	address_Of_Monitor_Entry
										, CARDINAL_32	*result_Code
										, Safe_Boolean	find_MacJmp_Location)
{

#define DEFINITION_SEVars		3120
	asm
		{
		
; Output : NONE
; Purpose : To test Monitor commands with TMON, for eventual support of TMON at SCSI driver load time
; Usage : currently is just test code
; NOTE : Gestalt used, must eventually be hard coded into another high memory address with a Safe_Boolean
;			key so that SCSI driver can scan or jump to the address to get the true address of the Monitor entrypoints.
; GLOBALS : utilizes Gestalt entry for TMON, in system heap
; Also, The code is written this way to allow proper return to itself after monitor exit, though in real world, this is not critical.

	/*=--------------------------------------------------------------------------=*/
	/*  if find_MacJmp_Location then save ALLLLLLLLLL registers */
	/* in fact SEVars should contain all the registers */
	/*=--------------------------------------------------------------------------=*/
		movem.l	d0-d7/a0-a6,-(SP)				; Save all registers that will be altered
		
		move.w	find_MacJmp_Location,d0
		move.w	#SAFE_FALSE,d1
		sub.w		d0,d1
		beq		@Normal_Debugger_Entry_Point
				
		movem.l	(SP),d0-d7/a0-a6			; Restore altered registers for saving to SEVars (don't mess up a7!)
		move.l	a1,result_Code				; Save the real a1 (temporary)
		
		clr.l		d0								; Sacrifice value of d0 to load a 32 bit value for SEVars address
		addi.l		#DEFINITION_SEVars,d0		; Load SEVars address
		move.l	d0,a1
		movem.l	d0-d7/a0-a7,(a1)			; store all registers to SEVars area in correct order and locations
		addi.l		#36,a1						; Move address to a1 storage
		move.l	result_Code,(a1)				; Restore the real a1 into SEVars (the other a1 is not correct)
		addi.l		#28,a1						; Move address from a1 to end of SEVars for PC
		
		lea			@Final_Program_Counter_Location,a0	; Get program counter for storage in SEVars
		move.l	a0,(a1)						; Save program counter at end of SEVars

		move.l	address_Of_Monitor_Entry,a1			; Prepare address of TMON entrypoint
		jmp		(a1	)										; Jump to TMON entrypoint for Monitor commands
		
@Final_Program_Counter_Location:

		Debugger					; See if returns OK

		bra			@Finish_And_Exit
		
@Normal_Debugger_Entry_Point:
			
		move.l	TMON_NORMAL_FULL_ENTRY,d0		; Select desired TMON command
		move.l	address_Of_Monitor_Entry,a1			; Prepeare address of TMON entrypoint
		jsr			(a1	)										; Jump to TMON entrypoint for Monitor commands
		move.l	d0,result_Code							; Store result code of TMON action

		
Finish_And_Exit:	

		movem.l	(SP)+,d0-d7/a0-a6	; Restore altered registers
		unlk		a6						; Remove stack frame
		movea.l	(SP)+,a0				; Load invocation address
		lea			10(SP),SP				; Restore proper stack size ("4" was size of passed parameters)
		jmp		(a0)					; Jump back to caller. All done
		
	} /* asm */
	
} /* pascal static void Invoke_TMON_Using_Monitor_Commands(void) */
/*=**************************************************************************=*/


/*=**************************************************************************=*/
pascal void Debug_With_TMON(void)
{
	CARDINAL_32	result_Code_TMON;
	Ptr					address_Of_Monitor_Entry;
	Safe_Boolean		find_MacJmp_Location;
/* END VAR */
	
		
		find_MacJmp_Location = SAFE_TRUE;


	/*=--------------------------------------------------------------------------=*/
	/* Test TMON by calling directly. */
	/*=--------------------------------------------------------------------------=*/
	/*==
		if (Blind_Search_TMON_Entry_Point(&address_Of_Monitor_Entry, find_MacJmp_Location) == SAFE_TRUE) {
			Invoke_TMON_Using_Monitor_Commands(address_Of_Monitor_Entry, &result_Code_TMON, find_MacJmp_Location);
		}
 ==== */



	if (Determine_TMON_Entry_Point(&address_Of_Monitor_Entry) == SAFE_TRUE) {
		Invoke_TMON_Using_Monitor_Commands(address_Of_Monitor_Entry, &result_Code_TMON, SAFE_FALSE);
	}
	else {
		if (Blind_Search_TMON_Entry_Point(&address_Of_Monitor_Entry, find_MacJmp_Location) == SAFE_TRUE) {
			Invoke_TMON_Using_Monitor_Commands(address_Of_Monitor_Entry, &result_Code_TMON, find_MacJmp_Location);
		}
	}

}/* pascal void Debug_With_TMON(void) */
/*=**************************************************************************=*/


/*=**************************************************************************=*/
/* END OF FILE: OS_Extra_Utilities.c */
