/* START OF FILE: Physical_File_Mapping.c */


/*=**************************************************************************=*/
/* [---BEGIN---] FILE HEADER COMMENTS */
/**
 ** SCREEN VIEW = [Adobe Syntax Black-10, Tabs = 4]
 ** PRINTER VIEW = [Adobe Syntax Black-10, Tabs = 4]
 ** =======================================================================
 ** Written by James Merkle
 ** Physical_File_Mapping.c - Routines to Locate volume blocks
 ** (C) Copyright 1995 James Merkle
 ** =======================================================================
 **
 **
 ** ============
 ** Revision History
 ** ============
 **
 **  1995.05.10		(James Merkle)
 **						Initial coding started from scratch
 **
 **  1995.06.13		(James Merkle)
 **						Finished it and tested it for first time. Had bugs.
 **
 **
 **
 ** =================
 ** OTHER INFORMATION
 ** =================
 ** 
 ** 
 **
 **
 **/
/* [----END----] FILE HEADER COMMENTS */
/*=**************************************************************************=*/


/*=**************************************************************************=*/
/* [---BEGIN---] #include AREA */

#include <LowMem.h>
#include "Shell_Globals.h"

#if !defined(k_USER_ERROR_INTERFACE_ALLOWED)
#error "UNDEFINED PARAMETER!"
#endif
#if (k_USER_ERROR_INTERFACE_ALLOWED == 1)

#include "OS_Utilities.h"
#include "Error_Utilities.h"
#include "Memory_Utilities.h"

/* ===
#include "Main_Utilities.h"

=== */

#else

#include "Code_Resource_Utilities.h"

#endif


#include "Physical_File_Mapping.h"

/* [----END----] #include AREA */
/*=**************************************************************************=*/


/*=**************************************************************************=*/
/* [---BEGIN---] #define AREA */


#define	k_CONVERT_LOGICAL_TO_PHYSICAL	0xA0BF

#define HFS_SIGNATURE_WORD				0x4244
#define k_HFS_LOGICAL_BLOCK_SIZE			(512UL)
#define k_HFS_LOGICAL_BLOCK_SHIFT		(9)

/*  */
/* The larger this number, the less times the handle holding a */
/* growing extent list will need to expand. But too large is wasteful */
/* too, and might disrupt heap cleanliness.*/
/* 100 is probably a good value. */
/*  */
#define k_EXTENTS_TO_MAP_MEMORY_CHUNK		(2UL)

/*  */
/* This occupies last element in extent list. Helps to detect bad logic */
/* and validate for ovverruns and corruption. */
/* It is only for debugging logic */
/*  */
#define k_END_SENTINAL_SAFETY_MARKER				(0xFFFF1234)
#define k_END_SENTINAL_SAFETY_MARKERS_USED		1

/* [----END----] #define AREA */
/*=**************************************************************************=*/


/*=**************************************************************************=*/
/* [---BEGIN---] TYPE AREA */

#if defined(powerc) || defined (__powerc)
#pragma options align=mac68k
#endif

	typedef struct	{
		CARDINAL_32	fcbFile_ID;
		CARDINAL_16	fcbFile_Flags;
		CARDINAL_16	fcbSBlk;
		CARDINAL_32	fcbEOF;
		CARDINAL_32	fcbPLen;
		CARDINAL_32	fcbCurrent_Position;
		VCB				*fcbVPtr;
		Ptr					fcbBuffer_Address;
		CARDINAL_8		pad_1[2];
		CARDINAL_32	fcbClmpSize;
		CARDINAL_32	fcbBTCBPtr;
		CARDINAL_16	fcbExtRec_1_location;
		CARDINAL_16	fcbExtRec_1_range;
		CARDINAL_16	fcbExtRec_2_location;
		CARDINAL_16	fcbExtRec_2_range;
		CARDINAL_16	fcbExtRec_3_location;
		CARDINAL_16	fcbExtRec_3_range;
		CARDINAL_32	fcbFile_Type;
		CARDINAL_32	fcbCatalog_Position_Hint;
		CARDINAL_32	fcbParent_Directory_ID;
		Str31				fcbFile_Name;
	} FCB_Spoof, *FCB_Spoof_Ptr;
	
#if defined(powerc) || defined(__powerc)
#pragma options align=reset
#endif



/* [----END----] TYPE AREA */
/*=**************************************************************************=*/


/*=**************************************************************************=*/
/* [---BEGIN---] PROTOTYPE AREA */

Safe_Boolean Clear_File_Map_Information_Block(Map_Info_Ptr file_Map);

Safe_Boolean Finish_Mapping_File_Location(INTEGER_16 file_Reference
													, Map_Info_Ptr file_Map
													, Safe_Boolean	store_Data_In_System_Heap);


#if (defined(__MWERKS__))

k_MW_ASM INTEGER_16 Get_Physical_Blocks_For_File_Position(VCB		*open_Volume_Ptr
												, Ptr			 	file_Control_Blocks_Base_Start_Ptr
												, CARDINAL_16	file_Reference_Offset
												, CARDINAL_32	starting_Byte_Position
												, CARDINAL_32	total_Bytes_Desired
												, CARDINAL_32	*physical_Allocation_Block
												, CARDINAL_32	*total_Bytes_Possible);

#else
#if (defined(THINK_C))

pascal INTEGER_16 Get_Physical_Blocks_For_File_Position(VCB		*open_Volume_Ptr
												, Ptr			 	file_Control_Blocks_Base_Start_Ptr
												, CARDINAL_16	file_Reference_Offset
												, CARDINAL_32	starting_Byte_Position
												, CARDINAL_32	total_Bytes_Desired
												, CARDINAL_32	*physical_Allocation_Block
												, CARDINAL_32	*total_Bytes_Possible);
#else
#error "\pUNKNOWN COMPILER (Neither Metroworks nor THINK C)"
#endif
#endif


/* [----END----] PROTOTYPE AREA */
/*=**************************************************************************=*/


/*=**************************************************************************=*/
/* [---BEGIN---] GLOBAL DEFINITION AREA */


/* [----END----] GLOBAL DEFINITION AREA */
/*=**************************************************************************=*/


/*=**************************************************************************=*/
/* [---BEGIN---] LOCAL STATIC DEFINITION AREA */

static CARDINAL_8	*unit_Name;
	Ptr					g_Address_Of_Remap_Routine_MC68K;
	Safe_Boolean		g_Address_Of_Remap_Routine_Initialized;


/* [----END----] LOCAL STATIC DEFINITION AREA */
/*=**************************************************************************=*/


/*=**************************************************************************=*/
Safe_Boolean Initialize_Physical_File_Mapping(void)
{
	CARDINAL_8		*static_Unit_Name = "\pPhysical_File_Mapping";
	INTEGER_16		type;
	Safe_Boolean		function_Result;
/* END VAR */

/*  */
/* Global persistant structures are created here */
/*  */
	unit_Name = static_Unit_Name;
	
	g_Address_Of_Remap_Routine_MC68K = nil;
	g_Address_Of_Remap_Routine_Initialized = SAFE_FALSE;
	
	if (Trap_Available(k_CONVERT_LOGICAL_TO_PHYSICAL) != TRUE) {
		goto Error_Handler;
	}
	
	type = Get_Trap_Type(k_CONVERT_LOGICAL_TO_PHYSICAL);
	g_Address_Of_Remap_Routine_MC68K = (Ptr)NGetTrapAddress(k_CONVERT_LOGICAL_TO_PHYSICAL, type);
	g_Address_Of_Remap_Routine_Initialized = SAFE_TRUE;
	
	
	function_Result = SAFE_TRUE;
	goto Normal_Function_Exit;

Error_Handler:
	function_Result = SAFE_FALSE;
		
Normal_Function_Exit:
	return(function_Result);
	
}
/*=**************************************************************************=*/


/*=**************************************************************************=*/
Safe_Boolean Destroy_Physical_File_Mapping(void)
{
	Safe_Boolean		function_Result;
/* END VAR */

/*  */
/* Global persistant structures are deleted here */
/*  */

	function_Result = SAFE_TRUE;
	return(function_Result);
}
/*=**************************************************************************=*/ 


/*=**************************************************************************=*/ 
Safe_Boolean Map_Entire_File_Location(INTEGER_16 file_Reference
													, Map_Info_Ptr file_Map
													, Safe_Boolean	store_Data_In_System_Heap)
{
	Safe_Boolean		function_Result;
/* END VAR */


	if (Start_Mapping_File_Location(file_Reference, file_Map) != SAFE_TRUE) {
		goto Error_Handler;
	}
	
	if (file_Map->every_Extent_Mapped_Already != SAFE_TRUE) {
		/*  */
		/* Invoke fancy routine to map out entire file in another routine. */
		/*  */
		if (Finish_Mapping_File_Location(file_Reference, file_Map, store_Data_In_System_Heap) != SAFE_TRUE) {
			goto Error_Handler;
		}
	}
	
	function_Result = SAFE_TRUE;
	goto Normal_Function_Exit;

Error_Handler:
	function_Result = SAFE_FALSE;
	file_Map->initialized = SAFE_FALSE;
		
Normal_Function_Exit:
	return(function_Result);
}
/*=**************************************************************************=*/


/*=**************************************************************************=*/ 
Safe_Boolean Destroy_File_Map_Information_Block(Map_Info_Ptr file_Map)
{
	Safe_Boolean	function_Result;
/* END VAR */


	function_Result = SAFE_TRUE;
	
	if (file_Map->initialized == SAFE_TRUE) {
		if (file_Map->all_Extents_Mapped_Memory_Initialized == MEMORY_INITIALIZED) {
		
			if (Destroy_Handle((Handle *)&file_Map->all_Extents_Mapped_Hdl
									, &file_Map->all_Extents_Mapped_Memory_Initialized) != SAFE_TRUE) {
						
				function_Result = SAFE_FALSE;
			}
		}
	}
	
	file_Map->initialized = SAFE_FALSE;
	file_Map->every_Extent_Mapped_Already = SAFE_FALSE;
	file_Map->total_Extents_Mapped = 0;
	file_Map->all_Extents_Mapped_Hdl = nil;
	file_Map->all_Extents_Mapped_Memory_Initialized = MEMORY_NOT_INITIALIZED;
	
	return(function_Result);
}
/*=**************************************************************************=*/


/*=**************************************************************************=*/ 
Safe_Boolean Clear_File_Map_Information_Block(Map_Info_Ptr file_Map)
{

	file_Map->initialized = SAFE_FALSE;
	file_Map->every_Extent_Mapped_Already = SAFE_FALSE;
	file_Map->total_Extents_Mapped = 0;
	file_Map->all_Extents_Mapped_Hdl = nil;
	file_Map->all_Extents_Mapped_Memory_Initialized = MEMORY_NOT_INITIALIZED;
	
	/*  */
	/* For Paranoia clear all the rest too */
	/*  */
	file_Map->driver_Reference = 0;
	file_Map->physical_File_Size = 0;
	file_Map->logical_File_Size = 0;
	file_Map->start_Block_Byte = 0;
	file_Map->drive_Number = 0;
	file_Map->volume_Allocation_Block_Size = 0;
	file_Map->volume_Allocation_Block_Offset = 0;
	file_Map->file_Allocation_Clump_Size = 0;
	file_Map->file_B_Tree_Clump_Block_Ptr = 0;
	file_Map->extent_Byte_Location_1 = 0;
	file_Map->extent_Byte_Range_1 = 0;
	file_Map->extent_Byte_Location_2 = 0;
	file_Map->extent_Byte_Range_2 = 0;

	return(SAFE_TRUE);
	
} /* Safe_Boolean Clear_File_Map_Information_Block ... */
/*=**************************************************************************=*/


/*=**************************************************************************=*/  
Safe_Boolean Calculate_Drive_Offset(Map_Info_Ptr file_Map
									, CARDINAL_32 data_Start_Offset
									, CARDINAL_32 *actual_Drive_Offset)
{
	Safe_Boolean	function_Result;
/* END VAR */


	*actual_Drive_Offset = 0xFFFFFFFF;
	
	if (file_Map->initialized != SAFE_TRUE) {
		goto Error_Handler;
	}
	
	if (file_Map->every_Extent_Mapped_Already == SAFE_TRUE) {
		if (file_Map->total_Extents_Mapped == 0) {
			/*  */
			/* File fork possibly not created yet. */
			/*  */
			goto Error_Handler;
		}
		else {
			if (file_Map->total_Extents_Mapped == 1) {
				*actual_Drive_Offset = data_Start_Offset + file_Map->start_Block_Byte;
			}
			else {
				/*  */
				/* HACK HACK HACK HACK HACK HACK */
				/* HACK HACK HACK HACK HACK HACK */
				/* HACK HACK HACK HACK HACK HACK */
				/* HACK HACK HACK HACK HACK HACK */
				/* Should solve using other information and possible 2nd extent. */
				/* For now just jhack test. */
				/*  */
				*actual_Drive_Offset = data_Start_Offset + file_Map->start_Block_Byte;
			}
		}
	}
	else {
		/*  */
		/* HACK HACK HACK HACK HACK HACK */
		/* HACK HACK HACK HACK HACK HACK */
		/* HACK HACK HACK HACK HACK HACK */
		/* HACK HACK HACK HACK HACK HACK */
		/* Should error, but for now just do offset from first for test */
		/*  */
		*actual_Drive_Offset = data_Start_Offset + file_Map->start_Block_Byte;
	}
	

	function_Result = SAFE_TRUE;
	goto Normal_Function_Exit;

Error_Handler:
	function_Result = SAFE_FALSE;
		
Normal_Function_Exit:
	return(function_Result);
	
} /* Safe_Boolean Calculate_Drive_Offset ... */
/*=**************************************************************************=*/ 


/*=**************************************************************************=*/ 
Safe_Boolean Finish_Mapping_File_Location(INTEGER_16 file_Reference
													, Map_Info_Ptr file_Map
													, Safe_Boolean	store_Data_In_System_Heap)
{
	CARDINAL_8		*code_Name;
	Safe_Boolean		function_Result;
	OSErr				os_Error;
	CARDINAL_32	starting_Byte_Position;
	CARDINAL_32	total_Bytes_Desired;
	CARDINAL_32	physical_Byte_Position;
	CARDINAL_32	total_Bytes_Possible;
	CARDINAL_32	extent_Record;
	CARDINAL_32	preferred_Size;
	CARDINAL_32	handle_Record_Limit;
	CARDINAL_32	temp_Value;
	Boolean			still_Mapping;
/* END VAR */


	code_Name = "\pFinish_Mapping_File_Location";

	/*  */
	/* Use Lg2Phys technology to map file. */
	/*  */
	
	if (g_Address_Of_Remap_Routine_Initialized != SAFE_TRUE){
		goto Error_Handler;
	}
	
	starting_Byte_Position = 0;
	extent_Record = 0;
	still_Mapping = TRUE;
	
	do {
		total_Bytes_Desired = file_Map->physical_File_Size - starting_Byte_Position;
		
		if (total_Bytes_Desired != 0) {
				
			os_Error = -1;	
			os_Error = Get_Physical_Blocks_For_File_Position(file_Map->volume_Control_Block_Ptr
														, (Ptr)LMGetFCBSPtr()
														, file_Reference
														, starting_Byte_Position
														, total_Bytes_Desired
														, &physical_Byte_Position
														, &total_Bytes_Possible);
			
			if (os_Error != noErr) {
				still_Mapping = FALSE;
				goto Error_Handler;
			}
			
			/*  */
			/* Adjust physical_Byte_Position from allocation block into proper byte limit */
			/*  */
			physical_Byte_Position = physical_Byte_Position << k_HFS_LOGICAL_BLOCK_SHIFT;
	
			if (extent_Record == 0) {
				/*  */
				/* Double verify first extent start location against reference test value */
				/*  */
				if (physical_Byte_Position != file_Map->extent_Byte_Location_1) {
				
#if (k_USER_ERROR_INTERFACE_ALLOWED == 1)
					Display_Serious_Error_Message(unit_Name
												, code_Name
												, "\pThis system cannot map the file's extents using Apple's '_Lg2Phys' becaue it deviates from FCB. No alternative exists."
												, "\pBad system? Try optimizing the file into a contiguous block range.");
#endif
					goto Error_Handler;
				}
				
				/*  */
				/* Because this is the first time through, allocate handle for */
				/* extents. Allocate a large chunk at a time. Expand it as needed. */
				/* Expansion was tested already because expansion was set small in original tests. */
				/* It is resized smaller upon conclusion. */
				/*  */
				preferred_Size = sizeof(Extent_Record);
				preferred_Size = preferred_Size * k_EXTENTS_TO_MAP_MEMORY_CHUNK;
				temp_Value = preferred_Size;
				
				handle_Record_Limit = k_EXTENTS_TO_MAP_MEMORY_CHUNK;
				
				file_Map->all_Extents_Mapped_Memory_Initialized = MEMORY_NOT_INITIALIZED;
				file_Map->all_Extents_Mapped_Hdl = nil;
				
				/*  */
				/* JAM BUG FIX:
				/* Though never used before in a program, the handle was incorrectly passed. */
				/*  */
				if (store_Data_In_System_Heap == SAFE_TRUE) {
					if (Create_System_Handle(&preferred_Size
										, (CARDINAL_32)(10240L)
										, ALLOW_PURGING
										, DISALLOW_PARTIAL_AMOUNT
										, (Handle *)&file_Map->all_Extents_Mapped_Hdl
										, &file_Map->all_Extents_Mapped_Memory_Initialized) != SAFE_TRUE) {
										
#if (k_USER_ERROR_INTERFACE_ALLOWED == 1)
						Display_Low_Memory_Error_Message(unit_Name, code_Name, temp_Value);
#endif
						function_Result = SAFE_FALSE;
						goto Normal_Function_Exit;
					}
				}
				else {
					if (Create_Handle(&preferred_Size
										, (CARDINAL_32)(10240L)
										, ALLOW_PURGING
										, DISALLOW_PARTIAL_AMOUNT
										, (Handle *)&file_Map->all_Extents_Mapped_Hdl
										, &file_Map->all_Extents_Mapped_Memory_Initialized) != SAFE_TRUE) {
										
#if (k_USER_ERROR_INTERFACE_ALLOWED == 1)
						Display_Low_Memory_Error_Message(unit_Name, code_Name, temp_Value);
#endif

						function_Result = SAFE_FALSE;
						goto Normal_Function_Exit;
					}
				}
				
				
			} /* if (extent_Record == 0) */
			
			/*  */
			/* Save information regarding extent. */
			/* Expand handle, if needed, by another chunk. */
			/*  */

			if ((extent_Record + k_END_SENTINAL_SAFETY_MARKERS_USED) >= handle_Record_Limit) {
			
				handle_Record_Limit += k_EXTENTS_TO_MAP_MEMORY_CHUNK;
				preferred_Size = handle_Record_Limit * (CARDINAL_32)sizeof(Extent_Record);
				if (store_Data_In_System_Heap == SAFE_TRUE) {
					if (Reset_System_Handle_Size_Preserving_Contents((Handle *)&file_Map->all_Extents_Mapped_Hdl
																	, preferred_Size
																	, &os_Error) != SAFE_TRUE) {
#if (k_USER_ERROR_INTERFACE_ALLOWED == 1)
						Display_Low_Memory_Error_Message(unit_Name, code_Name, preferred_Size);
#endif

						function_Result = SAFE_FALSE;
						goto Normal_Function_Exit;
					}
				}
				else {
					if (Reset_Handle_Size_Preserving_Contents((Handle *)&file_Map->all_Extents_Mapped_Hdl
																	, preferred_Size
																	, &os_Error) != SAFE_TRUE) {
#if (k_USER_ERROR_INTERFACE_ALLOWED == 1)
						Display_Low_Memory_Error_Message(unit_Name, code_Name, preferred_Size);
#endif

						function_Result = SAFE_FALSE;
						goto Normal_Function_Exit;
					}
				}
			
			} /* if (extent_Record >= handle_Record_Limit) */
						
						
			if ((extent_Record + k_END_SENTINAL_SAFETY_MARKERS_USED) < handle_Record_Limit) {
			
				(*file_Map->all_Extents_Mapped_Hdl)[extent_Record].extent_Byte_Location = physical_Byte_Position;
				(*file_Map->all_Extents_Mapped_Hdl)[extent_Record].extent_Byte_Range = total_Bytes_Possible;
			}
			else {
				goto Error_Handler;
			}
			
			
			starting_Byte_Position += total_Bytes_Possible;
			
			/*  */
			/* Impossible, but important paranoia incase file grows */
			/* while being mapped */
			/*  */
			if (starting_Byte_Position > file_Map->physical_File_Size) {
				starting_Byte_Position = file_Map->physical_File_Size;
			}
			
			extent_Record += 1;
		}
		else {
			still_Mapping = FALSE;
		}
	} while (still_Mapping == TRUE);
	
	if (extent_Record > 0) {
		file_Map->total_Extents_Mapped = extent_Record;
		file_Map->every_Extent_Mapped_Already = SAFE_TRUE;
		
		/*  */
		/* extent_Record should be pointing past final entry. */
		/*  */
		if (extent_Record < handle_Record_Limit) {
			(*file_Map->all_Extents_Mapped_Hdl)[extent_Record].extent_Byte_Location = k_END_SENTINAL_SAFETY_MARKER;
			
			/*  */
			/* Much safer than using "0" in this case */
			/*  */
			(*file_Map->all_Extents_Mapped_Hdl)[extent_Record].extent_Byte_Range = 0xFFFFFFFF;
		}
		
		/*  */
		/* Shrink Handle size back down to contain only valid records, */
		/* plus the end sentinel. */
		/*  */
		preferred_Size = (extent_Record + 1) * (CARDINAL_32)sizeof(Extent_Record);
		if (store_Data_In_System_Heap == SAFE_TRUE) {
			if (Reset_System_Handle_Size_Preserving_Contents((Handle *)&file_Map->all_Extents_Mapped_Hdl
															, preferred_Size
															, &os_Error) != SAFE_TRUE) {
				goto Normal_Function_Exit;
			}
		}
		else {
			if (Reset_Handle_Size_Preserving_Contents((Handle *)&file_Map->all_Extents_Mapped_Hdl
															, preferred_Size
															, &os_Error) != SAFE_TRUE) {
				goto Normal_Function_Exit;
			}
		}
		
	} /* if (extent_Record > 0) */
	
	
	function_Result = SAFE_TRUE;
	goto Normal_Function_Exit;
	
Error_Handler:
	function_Result = SAFE_FALSE;
	
#if (k_USER_ERROR_INTERFACE_ALLOWED == 1)

	Display_Serious_Error_Message(unit_Name
								, code_Name
								, "\pThe file's extents location could not be  fully mapped out. Perhaps memory was too low or a serious problem exists.."
								, "\pBad system? Try optimizing the file into a contiguous block range.");
#endif

	/*  */
	/* Deallocate the extent handle if it was created? Nah, leave that */
	/* mess to the cleanup code. */
	/*  */

Normal_Function_Exit:
	return(function_Result);
	
} /* Safe_Boolean Finish_Mapping_File_Location ... */
/*=**************************************************************************=*/ 


/*=**************************************************************************=*/ 
Safe_Boolean Start_Mapping_File_Location(INTEGER_16 file_Reference, Map_Info_Ptr file_Map)
{
	VCB				*vcb_Ptr;
	FCB_Spoof_Ptr	active_FCB_Ptr;
	Safe_Boolean		function_Result;
	CARDINAL_16	temp_16_Bit_Value;
/* END VAR */


	function_Result = SAFE_FALSE;

	if (Clear_File_Map_Information_Block(file_Map) != SAFE_TRUE) {
		goto Error_Handler;
	}
	
	file_Map->initialized = SAFE_FALSE;
	file_Map->every_Extent_Mapped_Already = SAFE_FALSE;
	file_Map->total_Extents_Mapped = 0;
	file_Map->all_Extents_Mapped_Hdl = nil;
	file_Map->all_Extents_Mapped_Memory_Initialized = MEMORY_NOT_INITIALIZED;

	/*  */
	/* Get FCB Ptr from memory. Required to get the extent info*/
	/* Force File Reference to be treated as raw 16 bit cardinal offset, */
	/* yet ensure math is done in 32 bits */
	/*  */
	temp_16_Bit_Value = (CARDINAL_16)file_Reference;
	active_FCB_Ptr = (FCB_Spoof_Ptr)((CARDINAL_32)LMGetFCBSPtr() + (CARDINAL_32)temp_16_Bit_Value);
	
	vcb_Ptr = active_FCB_Ptr->fcbVPtr;
		
	if (vcb_Ptr->vcbSigWord != HFS_SIGNATURE_WORD) {
		goto Error_Handler;
	}

	/*  */
	/* It is unreasonably difficult to extract the extent records of a file. */
	/* Regrettably, because a driver may need to virtualize its storage and */
	/* redirect data to blocks stored in files, and because a storage driver can get */
	/* called during interrupts and while file manager is busy, any storage driver */
	/* that reads or writes from files really should use the Device Manager */
	/* and not the File Manager to fetch data from files, even if open files. */
	/* To do this it helps to know where the file is recorded! If the file is */
	/* contiguosly stored, even if across a complex hard disk array, the */
	/* mapping is simple, and the determination that the file is contiguous */
	/* is also easy to determine. A fe special fields just for that purpose are no */
	/* longer effective in System 7.0.0. Only "0" is found in "active_FCB_Ptr->fcbSBlk" */
	/* System 7.0 and file open or close status, and resource or data fork */
	/* being used has no effect. Routines such as "PBGetFCBInfo" and */
	/* "PBHGetFInfo" seem to offer no help. */
	/* Therefore we NEED to get the starting block location from the */
	/* extent records. Another routine maps out all extents, this routine */
	/* merely obtains the first three extents. */
	/*  */

	file_Map->driver_Reference = vcb_Ptr->vcbDRefNum;
	file_Map->physical_File_Size = active_FCB_Ptr->fcbPLen;
	file_Map->volume_Allocation_Block_Size = vcb_Ptr->vcbAlBlkSiz;
	file_Map->volume_Control_Block_Ptr = active_FCB_Ptr->fcbVPtr;

	file_Map->logical_File_Size = active_FCB_Ptr->fcbEOF;
	
	file_Map->volume_Allocation_Block_Offset = vcb_Ptr->vcbAlBlSt;

	file_Map->start_Block_Byte = (file_Map->volume_Allocation_Block_Size 
										* (CARDINAL_32)active_FCB_Ptr->fcbExtRec_1_location) ;

	/*  */
	/* Remember "volume_Allocation_Block_Offset" is part of calculation. */
	/*  */
	file_Map->start_Block_Byte += (file_Map->volume_Allocation_Block_Offset * k_HFS_LOGICAL_BLOCK_SIZE);
	
	file_Map->drive_Number = vcb_Ptr->vcbDrvNum;
	file_Map->file_Allocation_Clump_Size = active_FCB_Ptr->fcbClmpSize;
	file_Map->file_B_Tree_Clump_Block_Ptr = active_FCB_Ptr->fcbBTCBPtr;
	
	file_Map->extent_Byte_Location_1 = 0;
	file_Map->extent_Byte_Range_1 = 0;
	file_Map->extent_Byte_Location_2 = 0;
	file_Map->extent_Byte_Range_2 = 0;

	if (file_Map->physical_File_Size == 0) {
		file_Map->every_Extent_Mapped_Already = SAFE_TRUE;
		file_Map->total_Extents_Mapped = 0;
	}
	else {
	
		if ((active_FCB_Ptr->fcbExtRec_2_location != 0) 
			|| (active_FCB_Ptr->fcbExtRec_2_range != 0)) {
			
			/*  */
			/* The first two extents are used. Check if the third is too. */
			/* if so then we should just assume that there are more than */
			/* two extents and just make a note of it for another routine */
			/* to do a thorough search. */
			/*  */
			
			file_Map->extent_Byte_Location_1 = (file_Map->volume_Allocation_Block_Size 
													* (CARDINAL_32)active_FCB_Ptr->fcbExtRec_1_location);

			file_Map->extent_Byte_Location_1 += (file_Map->volume_Allocation_Block_Offset * k_HFS_LOGICAL_BLOCK_SIZE);

			file_Map->extent_Byte_Range_1 = (file_Map->volume_Allocation_Block_Size 
													* (CARDINAL_32)active_FCB_Ptr->fcbExtRec_1_range);
													
			file_Map->extent_Byte_Location_2 = (file_Map->volume_Allocation_Block_Size 
													* (CARDINAL_32)active_FCB_Ptr->fcbExtRec_2_location);
													
			file_Map->extent_Byte_Location_2 += (file_Map->volume_Allocation_Block_Offset * k_HFS_LOGICAL_BLOCK_SIZE);

			file_Map->extent_Byte_Range_2 = (file_Map->volume_Allocation_Block_Size 
													* (CARDINAL_32)active_FCB_Ptr->fcbExtRec_2_range);
			file_Map->total_Extents_Mapped = 2;
			
			if ((active_FCB_Ptr->fcbExtRec_3_location != 0) 
				|| (active_FCB_Ptr->fcbExtRec_3_range != 0)) {
				
				file_Map->every_Extent_Mapped_Already = SAFE_FALSE;
			}
			else {
				/*  */
				/* Good, only two extents for certain. */
				/*  */
				file_Map->every_Extent_Mapped_Already = SAFE_TRUE;
			}
		}
		else {
			/*  */
			/* Good, only one extent for certain. Nice and contiguous file. */
			/*  */
			file_Map->every_Extent_Mapped_Already = SAFE_TRUE;
			file_Map->total_Extents_Mapped = 1;

			file_Map->extent_Byte_Location_1 = (file_Map->volume_Allocation_Block_Size 
													* (CARDINAL_32)active_FCB_Ptr->fcbExtRec_1_location);
													
			file_Map->extent_Byte_Location_1 += (file_Map->volume_Allocation_Block_Offset * k_HFS_LOGICAL_BLOCK_SIZE);

			file_Map->extent_Byte_Range_1 = (file_Map->volume_Allocation_Block_Size 
													* (CARDINAL_32)active_FCB_Ptr->fcbExtRec_1_range);

			/*  */
			/* Double check to see if file's extent range is consistent */
			/* with its physical size */
			/*  */
			if (file_Map->extent_Byte_Range_1 != active_FCB_Ptr->fcbPLen) {
			
				/*  */
				/* JAM BUG FIX */
				/* If size differs it is OK if it is larger and is a multiple of  */
				/* the "volume_Allocation_Block_Offset" otherwise it is an */
				/* error.
				/*  */
				if ((file_Map->extent_Byte_Range_1 < active_FCB_Ptr->fcbPLen)
					|| (file_Map->volume_Allocation_Block_Size == 0)
					|| (active_FCB_Ptr->fcbExtRec_1_range == 0)) {
					goto Error_Handler;
				}
			}
		}
	
	} /* if (file_Map->physical_File_Size == 0) */
	
	file_Map->initialized = SAFE_TRUE;

	/*  */
	/* Invoke fancy routine to map out entire file in another routine, if needed. */
	/*  */

	function_Result = SAFE_TRUE;
	goto Normal_Function_Exit;

Error_Handler:
	function_Result = SAFE_FALSE;
	file_Map->initialized = SAFE_FALSE;

		
Normal_Function_Exit:
	return(function_Result);
	
} /* Safe_Boolean Start_Mapping_File_Location ... */
/*=**************************************************************************=*/


/*=**************************************************************************=*/
#if (defined(THINK_C))
pascal INTEGER_16 Get_Physical_Blocks_For_File_Position(VCB		*open_Volume_Ptr
												, Ptr			 	file_Control_Blocks_Base_Start_Ptr
												, CARDINAL_16	file_Reference_Offset
												, CARDINAL_32	starting_Byte_Position
												, CARDINAL_32	total_Bytes_Desired
												, CARDINAL_32	*physical_Allocation_Block
												, CARDINAL_32	*total_Bytes_Possible)
{
/*  */
/* Feel free to call this routine iteratively to determine the */
/* Device Manager block locations of each fragment of a */
/* file across a volume. Advance "starting_Byte_Position" only */
/* by "total_Bytes_Possible" each time until file is fully mapped. */
/* Do not request past file physical end! Also this is the best */
/* method of locating extent ranges because it works with */
/* MSF, HFS, ISO9660, and System 8. Using B Tree directly */
/* is not as portable. */
/*  */

	asm {
		movem.l	d1-d7/a1-a5,-(sp)		; Save registers that might be alterred (Hope highword d0 is safe)
		
		move.l	open_Volume_Ptr(a6),a2						; Copy VCB Ptr into a2
		move.l	file_Control_Blocks_Base_Start_Ptr(a6),a1		; Copy LMGetFCBSPtr() into a1
		move.w	file_Reference_Offset(a6), d1				; Copy FCB offset (reference) into d1
		move.l	starting_Byte_Position(a6),d5				; Copy file offset into d5
		andi.l		#0xFFFFFE00, d5							; Mask out lower 9 bits (redundant) for 512 byte alignment
		move.l	total_Bytes_Desired(a6),d4					; Copy max desired bytes into d4
		andi.l		#0xFFFFFE00, d4							; Mask out lower 9 bits (redundant) for 512 byte alignment
		clr.l		d6												; Clear returned total bytes result for paranoia
		clr.l		d3												; Clear physical start position result for paranoia


;		/*  */
;		/* A bug in Apple's ROM routines requires a6 to be valid and to */
;		/* point to some form of minor private stack. Create a larger one */
;		/* than needed, just for future safety, and move a7 out of the way */
;		/*  */

		movem.l	a4-a6,-(sp)	; Save a4 through a6 because they are alterred for a moment					
		move.l	sp, a6			; Copy start of new a6 based scratch stack
		subi.l		#4096,sp		; Create an a6 stack for buggy dependancy deep in ROM (doesn't like a7 stack!)
									; Note also, the a6 stack must be larger than 256 bytes. 4K seems safe minimum.
		
		
		lea				@Return_Position(pc),a3	; Copy return address into scratch register
		move.l		a3, -(a6)						; Push return address onto first element of a6 stack from scratch reg.
		move.l		a3, -(sp)						; Push return address onto first element of 'proper' a7 stack too


#if !defined(GLOBALS_ARE_A4_RELATIVE)
#error "UNDEFINED PARAMETER!"
#endif
#if (GLOBALS_ARE_A4_RELATIVE == 1)
		move.l		g_Address_Of_Remap_Routine_MC68K(a4),-(sp)		; Push trap entrypoint onto stack
#else
		move.l		g_Address_Of_Remap_Routine_MC68K(a5),-(sp)		; Push trap entrypoint onto stack
#endif /* NOT if (GLOBALS_ARE_A4_RELATIVE == 1) */

;		/*  */
;		/* For safety paranoia, a4 and a5 are deliberately nulified */
;		/*  */
		move.l		#0xFFF00000,a4				; Nullify a4 momentarily, its not suppossed to be referenced anyways
		move.l		#0xFFF00000,a5				; Nullify a5 momentarily, its not suppossed to be referenced anyways

;dc.w	0xA9FF										; (_Debugger)

		rts										; Call the block remapping routine! (_Lg2Phys trap. Mac OS itself uses vector)
		
@Return_Position:

		addi.l		#4096,sp			; Remove a6 stack for buggy dependancy deep in ROM
		movem.l	(sp)+,a4-a6		; Restore aalterred registers
					
		move.l	total_Bytes_Possible(a6), a1
		move.l	d6, (a1)
		
		move.l	physical_Allocation_Block(a6), a1
		move.l	d3, (a1)
		
@Finish_And_Exit:	
		movem.l	(sp)+,d1-d7/a1-a5		; Restore alterred registers

		unlk		a6							; Remove stack frame
		movea.l	(sp)+,a0					; Load invocation address
		
;dc.w	0xA9FF										; (_Debugger)

		/*  */
		/* JAM BUG FIX */
		/* Stack size in previously unused routine was incorrectly restored. */
		/*  */
		
		addi.l		#26,sp					; Restore proper stack size ("26" was size of passed parameters)
		move.w	d0, (sp)					; (Also sets flags, for whatever thats worth)
		jmp		(a0)						; Jump back to caller. All done
	}
	
} /* pascal INTEGER_16 Get_Physical_Blocks_For_File_Position...*/
#endif /* defined(THINK_C)) */
/*=**************************************************************************=*/


/*=**************************************************************************=*/
#if (defined(__MWERKS__))
k_MW_ASM INTEGER_16 Get_Physical_Blocks_For_File_Position(VCB		*open_Volume_Ptr
												, Ptr			 	file_Control_Blocks_Base_Start_Ptr
												, CARDINAL_16	file_Reference_Offset
												, CARDINAL_32	starting_Byte_Position
												, CARDINAL_32	total_Bytes_Desired
												, CARDINAL_32	*physical_Allocation_Block
												, CARDINAL_32	*total_Bytes_Possible)
{
/*  */
/* Feel free to call this routine iteratively to determine the */
/* Device Manager block locations of each fragment of a */
/* file across a volume. Advance "starting_Byte_Position" only */
/* by "total_Bytes_Possible" each time until file is fully mapped. */
/* Do not request past file physical end! Also this is the best */
/* method of locating extent ranges because it works with */
/* MSF, HFS, ISO9660, and System 8. Using B Tree directly */
/* is not as portable. */
/*  */

fralloc +

	movem.l	d1-d7/a1-a5,-(sp)		/* Save registers that might be alterred (Hope highword d0 is safe) */
	
	move.l	open_Volume_Ptr,a2					/* Copy VCB Ptr into a2 */
	move.l	file_Control_Blocks_Base_Start_Ptr,a1		/* Copy LMGetFCBSPtr() into a1 */
	move.w	file_Reference_Offset, d1				/* Copy FCB offset (reference) into d1 */
	move.l	starting_Byte_Position,d5				/* Copy file offset into d5 */
	andi.l		#0xFFFFFE00, d5					/* Mask out lower 9 bits (redundant) for 512 byte alignment */
	move.l	total_Bytes_Desired,d4					/* Copy max desired bytes into d4 */
	andi.l		#0xFFFFFE00, d4					/* Mask out lower 9 bits (redundant) for 512 byte alignment */
	clr.l		d6									/* Clear returned total bytes result for paranoia */
	clr.l		d3									/* Clear physical start position result for paranoia */


		/*  */
		/* A bug in Apple's ROM routines requires a6 to be valid and to */
		/* point to some form of minor private stack. Create a larger one */
		/* than needed, just for future safety, and move a7 out of the way */
		/*  */

	movem.l	a4-a6,-(sp)		/* Save a4 through a6 because they are alterred for a moment */				
	move.l	sp, a6			/* Copy start of new a6 based scratch stack */

	suba.l		#0x1000,a7		/* Create an a6 stack for buggy dependancy deep in ROM (doesn't like a7 stack!) */
							/* Note also, the a6 stack must be larger than 256 bytes. 4K seems safe minimum. */
	
	
	lea			@Return_Position,a3		/* Copy return address into scratch register */
	move.l		a3, -(a6)						/* Push return address onto first element of a6 stack from scratch reg. */
	move.l		a3, -(sp)						/* Push return address onto first element of 'proper' a7 stack too */

	move.l		g_Address_Of_Remap_Routine_MC68K,-(sp)		/* Push trap entrypoint onto stack */
	
	/*  */
	/* For safety paranoia, a4 and a5 are deliberately nulified */
	/*  */
	
	move.l		#0xFFF00000,a4				/* Nullify a4 momentarily, its not suppossed to be referenced anyways */
	move.l		#0xFFF00000,a5				/* Nullify a5 momentarily, its not suppossed to be referenced anyways */

/* dc.w	0xA9FF	*/							/* (_Debugger) */

	rts										/* Call the block remapping routine! (_Lg2Phys trap. Mac OS itself uses vector) */
	
@Return_Position:

	adda.l		#0x1000,a7			/* Remove a6 stack for buggy dependancy deep in ROM */
	movem.l	(sp)+,a4-a6			/* Restore aalterred registers */
				
	move.l	total_Bytes_Possible, a1
	move.l	d6, (a1)
	
	move.l	physical_Allocation_Block, a1
	move.l	d3, (a1)
	
@Finish_And_Exit:	
	movem.l	(sp)+,d1-d7/a1-a5		/* Restore alterred registers */

	
/*dc.w	0xA9FF	*/							/* (_Debugger) */

	
	frfree
	move.w	d0, 4(sp)					/* (Also sets flags, for whatever thats worth) */
	rts
	
} /* pascal INTEGER_16 Get_Physical_Blocks_For_File_Position...*/
#endif /* defined(__MWERKS__)) */
/*=**************************************************************************=*/


/*=**************************************************************************=*/
/* END OF FILE: Physical_File_Mapping.c */

