/* START OF FILE: Show_Help.c */
/*=**************************************************************************=*/
/* [---BEGIN---] FILE HEADER COMMENTS */

/**
 ** SCREEN VIEW = [Adobe Syntax Black-10, Tabs = 6]
 ** PRINTER VIEW = [Adobe Syntax Black-10, Tabs = 6]
 ** =======================================================================
 ** Written by JWWalker / rewritten by Jim Merkle
 ** Show_Help.c - displays scrolling text in a dialog box.
 ** current version (C) Copyright 1993 James Merkle
 ** =======================================================================
 **
 **
 ** ============
 ** Revision History
 ** ============
 **	1991.07.02		(Jim Merkle)
 **							Version 1.0
 **							Initial coding started by revamping pure public domain source
 **
 **
	Show_Help by James W. Walker, March 1991
	This code is freely usable.
	
	e-mail:
		Internet			76367.2271@compuserve.com
		CIS					76367,2271
		America Online		JWWalker
	
	This code displays scrolling text in a dialog box.  The text comes
	from TEXT/styl resources, which can be created with ResEdit 2.1.
	Pictures can be included in the text using the same scheme as
	TeachText: Each option-space character indicates where the top
	edge of a picture should go, and pictures are centered horizontally.
	Pictures come from consecutively-numbered PICT resources.  The text
	cannot be edited, but one can select text and copy it to the
	clipboard using command-C, or save it as a TeachText file.
	
	The fact that our window is not resizeable has its advantages.  We
	can compute where the pictures go, once and for all, except to take
	vertical scrolling into account.
	
	Prototype:
	
void Show_Help_With_Title( INTEGER_16 help_dlog_id, INTEGER_16 help_text_id
		, INTEGER_16 base_pict_id,Str255 message );
	
	help_dlog_id  is the resource ID of the DLOG resource.  The dialog
	              should have an OK button as item 1 and a userItem, to
	              display the text, in item 2.
	
	help_text_id is the resource ID of the TEXT and styl resources.
	
	base_pict_id is the resource ID of the first PICT resource.

 **
 **/

/* [----END----] FILE HEADER COMMENTS */
/*=**************************************************************************=*/


/*=**************************************************************************=*/
/* [---BEGIN---] FILE INCLUDE AREA */

#include "Build_Options.h"
#include "Window_Utilities.h"
#include "Cursor_Utilities.h"
#include "String_Utilities.h"
#include "Shell_Dialogs.h"
#include "Shell_Globals.h"

#include "Show_Help.h"
#include "Interface_Utilities.h"

/* [----END----] FILE INCLUDE AREA */
/*=**************************************************************************=*/


/*=**************************************************************************=*/
/* [---BEGIN---] DEFINE AREA */

/*=--------------------------------------------------------------------------=*/
/* Set to 1 for a code resource */
/*=--------------------------------------------------------------------------=*/

#define A4_PROJECT GLOBALS_ARE_A4_RELATIVE

#if A4_PROJECT
#include <SetUpA4.h>
#endif

#define CHECKPOINTS 0

#if CHECKPOINTS
#define CKPT(x)		DebugStr( x  );
#else
#define CKPT(x)
#endif


#define		SCROLLBAR_WIDTH	16
#define		TEXT_INSET		4
#define		PAGE_OVERLAP	12
#define		ASSERT(x,y)
#define		MIN_SCROLL		10


#define		SOMETHING_OK_DIALOG_OK_ITEM					1
#define		SOMETHING_OK_DIALOG_SCROLLING_ITEM			2
#define		CREATE_TEACH_TEXT_DOCUMENT_ITEM				3
#define 		SOMETHING_OK_HEADING_TEXT_ITEM				4

#define		SOMETHING_OK_DIALOG_OUTLINE_ITEM 			7
#define		SOMETHING_OK_INFO_ITEM				 			8


/* [----END----] DEFINE AREA */
/*=**************************************************************************=*/


/*=**************************************************************************=*/
/* [---BEGIN---] TYPE AREA */

	
	typedef struct {
		Rect			bounds;
		PicHandle		pict_Handle;
	} pict_info;
	
	typedef struct {
		DialogRecord		dialog;
		ControlHandle	scrollbar;
		Boolean			high_defer_flag;
		Rect				high_rect[3];
		INTEGER_16		high_waiting;
		INTEGER_16		pict_count;	/* how many pictures */
		pict_info			*pict_data;	/* pointer to an array */
	}	help_record, *help_ptr;
	

/* [----END----] TYPE AREA */
/*=**************************************************************************=*/


/*=**************************************************************************=*/
/* [---BEGIN---] PROTOTYPE AREA */

/*=--------------------------------------------------------------------------=*/
/* Prototypes of public routines */
/*=--------------------------------------------------------------------------=*/

/* void Flash_button( DialogPtr the_dialog, INTEGER_16 item_number ); */
/* void Show_Help_With_Title( INTEGER_16 help_dlog_id, INTEGER_16 help_text_id
		, INTEGER_16 base_pict_id,Str255 message ); */

/*=--------------------------------------------------------------------------=*/
/* Prototypes of private routines */
/*=--------------------------------------------------------------------------=*/
	static pascal Boolean Filter_Events_For_Dialog( DialogPtr dialog, EventRecord	*event, INTEGER_16 *itemHit);
	static pascal void	userItem_proc( WindowPtr the_window, INTEGER_16 item_num );
	static pascal void Scroll_text( ControlHandle the_bar, int part_code );
	static pascal Auto_scroll( void );
	static pascal void High_hook( Rect *high_rect );
	static void Do_deferred_hilites( help_ptr	hptr, Rect *update_rect );
	static void Handle_scroll( DialogPtr dialog, INTEGER_16 the_part, Point local_point );
	static void Adjust_text( DialogPtr dialog );
	static void Find_pictures( DialogPtr dlog, INTEGER_16 first_pict_id );
	static void Draw_picts( WindowPtr the_window, Rect *update_rect );
	static void High_hook_glue( void );
	static void Save_text( TEHandle the_text, INTEGER_16 base_pict_id,Str255 file_Name_Preferred );


/* [----END----] PROTOTYPE AREA */
/*=**************************************************************************=*/


/*=**************************************************************************=*/
/* [---BEGIN---] GLOBAL DEFINITION AREA */

static	StringPtr		something_OK_Dialog_Heading_Text_Ptr;

/* [----END----] GLOBAL DEFINITION AREA */
/*=**************************************************************************=*/


/*=**************************************************************************=*/
/* [---BEGIN---] LOCAL STATIC DEFINITION AREA */

static	RgnHandle	save_clip;
static	Boolean		standard_Titles;

/* [----END----] LOCAL STATIC DEFINITION AREA */
/*=**************************************************************************=*/


/*=**************************************************************************=*/
void Show_Help_With_Title( INTEGER_16 help_dlog_id, INTEGER_16 help_text_id
		, INTEGER_16 base_pict_id,Str255 message, Boolean non_Standard_Title )
{
	register			DialogPtr	dptr;
	register			TEHandle	the_text;
	INTEGER_16		itype, ihit;
	Handle			help_item;
	Rect				help_item_box;
	Handle			help_TEXT;
	StScrpHandle		help_styl;
	GrafPtr			save_port;
	Rect				dest, view;
	ControlHandle	the_bar;
	INTEGER_16		max_scroll, nLines;
	Point				place;
	
	
	INTEGER_16	item_Type;
	Rect			item_Rect;
	Handle		item_Handle;
	Rect			ok_Rect;
	Str255			heading_String; /* something_OK_Dialog_Heading_Text_Ptr points to it! */
	/* END VAR */


	if (non_Standard_Title == TRUE) {
		standard_Titles = FALSE;
		base_pict_id += 1;
	}
	else 
		standard_Titles = TRUE;

	dptr = (DialogPtr) NewPtr( sizeof(help_record) );
	dptr = GetNewDialog( help_dlog_id, (Ptr) dptr, (WindowPtr)-1L );
	GetPort( &save_port );
	Set_View_Port( dptr );
	
	help_TEXT = Get1Resource( 'TEXT', help_text_id );
	help_styl = (StScrpHandle) Get1Resource( 'styl', help_text_id );
	ASSERT( (dptr != nil) && (help_TEXT != nil) && (help_styl != nil),
		"\pResource error in Show_Help_With_Title" );
	HLock( help_TEXT );
	
	GetDItem( dptr, SOMETHING_OK_DIALOG_SCROLLING_ITEM, &itype, &help_item, &help_item_box );
	SetDItem( dptr, SOMETHING_OK_DIALOG_SCROLLING_ITEM, itype, (Handle) userItem_proc, &help_item_box );
	
	
	
	Center_Window_Unadjusted_For_Title_Bars(dptr);
	
	GetDItem(dptr, k_OK_Item, &item_Type, &item_Handle, &ok_Rect);
	GetDItem(dptr, SOMETHING_OK_DIALOG_OUTLINE_ITEM, &item_Type, &item_Handle, &item_Rect);
	item_Rect = ok_Rect;
	InsetRect(&item_Rect, -4, -4);
	SetDItem(dptr, SOMETHING_OK_DIALOG_OUTLINE_ITEM, item_Type, (Handle)userItem_proc, &item_Rect);


	GetDItem(dptr, SOMETHING_OK_HEADING_TEXT_ITEM, &item_Type, &item_Handle, &item_Rect);
	Assign_String(heading_String, (CARDINAL_8 *)message);
	if (standard_Titles == TRUE) {
		Concat_After_Pascal_String(heading_String, (StringPtr)"\p (Help)");
	}
	something_OK_Dialog_Heading_Text_Ptr = &heading_String[0];
	SetDItem(dptr, SOMETHING_OK_HEADING_TEXT_ITEM, item_Type, (Handle)userItem_proc, &item_Rect);

	GetDItem(dptr, SOMETHING_OK_INFO_ITEM, &item_Type, &item_Handle, &item_Rect);
	SetDItem(dptr, SOMETHING_OK_INFO_ITEM, item_Type, (Handle)userItem_proc, &item_Rect);
	
		
	
	view = help_item_box;
	InsetRect( &view, 1, 1 );
	view.right -= SCROLLBAR_WIDTH;
	dest = view;
	InsetRect( &dest, TEXT_INSET, 0 );
	the_text = TEStylNew( &dest, &view );
	ASSERT( the_text != nil, "\pFailed TEStylNew." );
	
	TEStylInsert( *help_TEXT, GetHandleSize(help_TEXT),
		help_styl, the_text );
		
/*=--------------------------------------------------------------------------=*/
/* Maybe not needed, but can't hurt. */
/*=--------------------------------------------------------------------------=*/
	TECalText( the_text );	
	
	TEActivate( the_text );
	ReleaseResource( (Handle) help_styl );
	ReleaseResource( help_TEXT );
	nLines = (**the_text).nLines;
	SetWRefCon( dptr, (INTEGER_32)the_text );
	max_scroll = TEGetHeight( (INTEGER_32) nLines, 1, the_text )
		- (view.bottom - view.top);
	
	help_item_box.left = help_item_box.right - SCROLLBAR_WIDTH;
	
	the_bar = NewControl( (WindowPtr)dptr, &help_item_box, (StringPtr)"\p", TRUE, 0, 0, max_scroll, scrollBarProc, 0L );
	
		
	ASSERT( the_bar != nil, "\pFailed NewControl for scroll bar." );
	((help_ptr) dptr)->scrollbar = the_bar;
	
	Find_pictures( dptr, base_pict_id );

/*=--------------------------------------------------------------------------=*/
/* Permit auto-scrolling */
/*=--------------------------------------------------------------------------=*/
	TEAutoView( TRUE, the_text );	
	CKPT( "\pInstalling ClikLoop" );
	
#if !defined(ORIGINAL_MAC_SYSTEM)
#error "UNDEFINED PARAMETER!"
#endif
#if (ORIGINAL_MAC_SYSTEM == 0)
XXXX do correctly 

	(**the_text).clikLoop = (ProcPtr)Auto_scroll;

	(**the_text).highHook = (ProcPtr) High_hook_glue;

#else
	(**the_text).clickLoop = (TEClickLoopUPP)Auto_scroll;
	(**the_text).highHook = (HighHookUPP) High_hook_glue;
#endif 
	
	((help_ptr)dptr)->high_defer_flag = FALSE;
	
/*=--------------------------------------------------------------------------=*/
/* Used in Draw_picts */
/*=--------------------------------------------------------------------------=*/
	save_clip = NewRgn();	
	
	ShowWindow( dptr );
	Arrow_Cursor();

	
#if A4_PROJECT
	RememberA4();
#endif
	do {

#if !defined(ORIGINAL_MAC_SYSTEM)
#error "UNDEFINED PARAMETER!"
#endif
#if (ORIGINAL_MAC_SYSTEM == 0)
XXXX 			ModalDialog( (ProcPtr) Filter_Events_For_Dialog, &ihit );
#else
	ModalDialog( (ModalFilterUPP) Filter_Events_For_Dialog, &ihit );
#endif 
		
		if (ihit == CREATE_TEACH_TEXT_DOCUMENT_ITEM)
			Save_text( the_text, base_pict_id, message );
	} while (ihit != SOMETHING_OK_DIALOG_OK_ITEM);
	
	
	DisposeRgn( save_clip );
	DisposPtr( (Ptr) ((help_ptr)dptr)->pict_data );
	DisposDialog( dptr );
	Update_Shell_Windows();
	Arrow_Cursor();
	TEDispose( the_text );
	Set_View_Port( save_port );
	
/*	function_Result = TRUE;
	return(function_Result); */
	
}
/*=**************************************************************************=*/


/*=**************************************************************************=*/
static void Save_text( TEHandle the_text, INTEGER_16 base_pict_id,Str255 file_Name_Preferred )
{
	static Str255				prompt = "\pName of TeachText file:";
	static Str255				default_name;
	static Str255				default_suffix = "\p Info TTEXT";
	SFReply					reply;
	OSErr						err;
	INTEGER_16				data_refnum, res_refnum, old_resfile;
	Handle					text_data;
	SignedByte				state;
	INTEGER_32				count;
	register INTEGER_16		num_picts, pict_id;
	Handle					old_pict, new_pict;
	
/* END VAR */
	
/*=--------------------------------------------------------------------------=*/
/* Avoidance of duplicated names among source and target is achieved, */
/* yet the target may still overwrite an existing file of same name at this time ! */
/*=--------------------------------------------------------------------------=*/

	Assign_String(default_name, (CARDINAL_8 *)file_Name_Preferred);
	if (default_name[0] <= (31 - default_suffix[0])) {
		Concat_After_Pascal_String(default_name, default_suffix);
	}
	else if ( (default_name[default_name[0]] == default_suffix[default_suffix[0]] )
			&& (default_name[0] == default_suffix[0] )) {
			default_name[default_name[0]] == '2';
	}
	
	Perform_Put_File_Dialog( prompt, default_name, &reply );
	
	if (reply.good)
	{
		old_resfile = CurResFile();
		num_picts = Count1Resources( 'PICT' );
		
		err = FSDelete( reply.fName, reply.vRefNum );
		err = Create( reply.fName, reply.vRefNum, 'ttxt', 'ttro' );
		err = FSOpen( reply.fName, reply.vRefNum, &data_refnum );
		text_data = (**the_text).hText;
		state = HGetState(text_data );
		HLock( text_data );
		count = GetHandleSize( text_data );
		err = FSWrite( data_refnum, &count, *text_data );
		err = FSClose( data_refnum );
		HSetState( text_data, state );
		
		if (num_picts > 0)
		{
			err = SetVol( nil, reply.vRefNum );
			CreateResFile( reply.fName );
			err = ResError();
			res_refnum = OpenResFile( reply.fName );
			err = ResError();
			for (pict_id = base_pict_id;
				pict_id < base_pict_id + num_picts; ++pict_id )
			{
				UseResFile( old_resfile );
				old_pict = Get1Resource( 'PICT', pict_id );
				if (old_pict == nil)
					break;
				new_pict = old_pict;
				err = HandToHand( &new_pict );
				UseResFile( res_refnum );
				
				AddResource( new_pict, 'PICT', (pict_id - base_pict_id + 1000), (StringPtr)"\p" );
				
				err = ResError();
			}
			CloseResFile( res_refnum );
			err = FlushVol( nil, reply.vRefNum );
		}
	}
}
/*=**************************************************************************=*/


/*=**************************************************************************=*/
static void High_hook_glue( void )
{
	asm {
		move.L	(SP)+, A0	; get address of rectangle
		movem.L	A2-A5/D3-D7, -(SP)	; save registers
		move.L	A0, -(SP)
	}
	CKPT( "\pHigh_hook_glue" );
	asm {
		JSR		High_hook
		movem.L	(SP)+, A2-A5/D3-D7	; restore registers
		RTS
	}
}
/*=**************************************************************************=*/


/*=**************************************************************************=*/
static pascal void High_hook( Rect *the_rect )
/*=--------------------------------------------------------------------------=*/
/*	This deferred highlighting scheme is used to ensure that highlighting */
/*	will be done after any pictures have been drawn, not before.	To do */
/*	otherwise can cause pictures to be incorrectly highlighted during        */
/*	auto-scrolling.	This effect can be seen in TeachText.                          */
/*=--------------------------------------------------------------------------=*/
{
	register help_ptr	front;

/* END VAR */

	
	CKPT("\pHigh_hook");
	front = (help_ptr) FrontWindow();
	if ( (front == nil) ||
		(GetPtrSize((Ptr)front) != sizeof(help_record)) )
		return;
	if (!front->high_defer_flag)
		InvertRect( the_rect );
	else
	{
		if ( front->high_waiting >= 3 )
			DebugStr((StringPtr)"\pHighlight overflow");
		else
		{
			front->high_rect[ front->high_waiting ]
				= *the_rect;
			front->high_waiting++;
		}
	}
}
/*=**************************************************************************=*/


/*=**************************************************************************=*/
static pascal Auto_scroll()
/*=--------------------------------------------------------------------------=*/
/*	This is a ClikLoop routine, called repeatedly by TEClick when the */
/*	mouse is down. */
/*=--------------------------------------------------------------------------=*/
{
	register	WindowPtr	the_display;
	register	ControlHandle	the_bar;
	Point					mouse_point;
	Rect					view_rect;
	register 	TEHandle	the_text;

/* END VAR */

	
	asm {
		movem.l		a1-a5/d1-d7, -(SP)
	}
	CKPT( "\pAuto_scroll");
	the_display = FrontWindow();
	if ( (the_display != nil) &&
		(GetPtrSize((Ptr)the_display) == sizeof(help_record)) )
	{
		the_text = (TEHandle) GetWRefCon( the_display );
		the_bar = ((help_ptr) the_display)->scrollbar;
		
		GetMouse( &mouse_point );
		view_rect = (**the_text).viewRect;
		if (mouse_point.v < view_rect.top)
			Scroll_text( the_bar, inUpButton );
		else if (mouse_point.v > view_rect.bottom)
			Scroll_text( the_bar, inDownButton );
	}
	asm {
		movem.L		(SP)+, a1-a5/d1-d7
		moveQ		#1, D0
	}
}
/*=**************************************************************************=*/


/*=**************************************************************************=*/
static void Draw_picts( WindowPtr the_window, Rect *update_rect )
/*=--------------------------------------------------------------------------=*/
/* Called by Adjust_text and userItem_proc to draw pictures. */
/*=--------------------------------------------------------------------------=*/
{
	register TEHandle	the_text;
	register INTEGER_16		pict_count, pict_index;
	PicHandle	the_pict;
	INTEGER_16 		v_offset;
	Rect		pict_loc, dummy;

/* END VAR */

	
	CKPT("\pDraw_picts");
	the_text = (TEHandle) GetWRefCon( the_window );
	v_offset = (**the_text).destRect.top - (**the_text).viewRect.top
		- TEXT_INSET;
	pict_count = ((help_ptr) the_window)->pict_count;
	for (pict_index = 0; pict_index < pict_count; pict_index++)
	{
		pict_loc = ((help_ptr) the_window)->pict_data[pict_index].bounds;
		OffsetRect( &pict_loc, 0, v_offset );
		if (!SectRect( &pict_loc, update_rect, &dummy ))
			continue;
		the_pict = ((help_ptr) the_window)->pict_data[pict_index].pict_Handle;
		LoadResource( (Handle) the_pict );
		HLock( (Handle) the_pict );
		GetClip( save_clip );
		ClipRect( update_rect );
		DrawPicture( the_pict, &pict_loc );
		SetClip( save_clip );
		HUnlock( (Handle) the_pict );
	}
}
/*=**************************************************************************=*/


/*=**************************************************************************=*/
static void Find_pictures( DialogPtr dlog, INTEGER_16 first_pict_id )
{
	register TEHandle	the_text;
	Handle		text_h;
	SignedByte	text_state;
	register INTEGER_32		offset;
	INTEGER_32		text_size;
	INTEGER_16		num_picts;
	register INTEGER_16		which_pict;
	CARDINAL_8		option_space[2] = "\xCA";
	pict_info	*pict_Info_Index;
	Point		place;
	INTEGER_16		line_height, font_ascent;
	TextStyle			what_style;

/* END VAR */

	
	CKPT("\pFind_pictures");
	the_text = (TEHandle) GetWRefCon( dlog );
	text_h = (**the_text).hText;
	text_state = HGetState( text_h );
	HLock( text_h );
	
/*=--------------------------------------------------------------------------=*/
/* Count option-space characters in the text. */
/*=--------------------------------------------------------------------------=*/
	text_size = GetHandleSize( text_h );
	offset = 0L;
	num_picts = 0;
	offset = Munger(text_h, offset, option_space, 1L, nil, 0L );
	while ( (offset >= 0L) && (offset <= text_size) )
	{
		num_picts++;
		offset++;
		offset = Munger(text_h, offset, option_space, 1L, nil, 0L );
		
	}
	
/*=--------------------------------------------------------------------------=*/
/* Allocate storage for an array of picture bounds. */
/*=--------------------------------------------------------------------------=*/
	pict_Info_Index = (pict_info *) NewPtr( sizeof(pict_info) * num_picts );
	((help_ptr)dlog)->pict_data = pict_Info_Index;
	
/*=--------------------------------------------------------------------------=*/
/* Initialize the picture info.  For each picture we record the   */
/* picture handle and its rectangle, in unscrolled window      */
/* coordinates.   */
/*=--------------------------------------------------------------------------=*/
	offset = 0L;
	for (which_pict = 0; which_pict < num_picts; which_pict++)
	{
		pict_Info_Index[which_pict].pict_Handle = (PicHandle) Get1Resource( 'PICT',
			first_pict_id + which_pict );
		if ( pict_Info_Index[which_pict].pict_Handle == nil )
			break;
		offset = Munger(text_h, offset, option_space, 1L, nil, 0L );
		place = TEGetPoint( (INTEGER_16)offset, the_text );
		TEGetStyle( offset, &what_style, &line_height,
			&font_ascent, the_text );
			
/*=--------------------------------------------------------------------------=*/
/* align picture with TOP of option-space */
/*=--------------------------------------------------------------------------=*/
		place.v -= line_height;	
		offset++;
		pict_Info_Index[which_pict].bounds = (**pict_Info_Index[which_pict].pict_Handle).picFrame;
		OffsetRect( &pict_Info_Index[which_pict].bounds,
			( ((**the_text).destRect.right + (**the_text).destRect.left) -
			(pict_Info_Index[which_pict].bounds.right + pict_Info_Index[which_pict].bounds.left)
			) / 2,
			- pict_Info_Index[which_pict].bounds.top + place.v );
	}
	((help_ptr)dlog)->pict_count = which_pict;
	
getout:
	HSetState( text_h, text_state );
}
/*=**************************************************************************=*/


/*=**************************************************************************=*/
static pascal void Scroll_text( ControlHandle the_bar, int part_code )
/*=--------------------------------------------------------------------------=*/
/* This is used as a TrackControl actionProc for scrolling, and also */
/* called by Auto_scroll for automatic scrolling. */
/*=--------------------------------------------------------------------------=*/
{
	register TEHandle	the_text;
	register INTEGER_16		delta;
	register WindowPtr		the_display;
	INTEGER_16				old_value;

/* END VAR */

	
	CKPT("\pScroll_text");
#if A4_PROJECT
	SetUpA4();
#endif
	if (part_code != 0)
	{
		the_display = (**the_bar).contrlOwner;
		the_text = (TEHandle) GetWRefCon( the_display );
		
		switch (part_code)
		{
			case inUpButton:
				delta = -MIN_SCROLL;
				break;
			case inDownButton:
				delta = MIN_SCROLL;
				break;
			case inPageUp:
				delta = - ( (**the_text).viewRect.bottom
					- (**the_text).viewRect.top - PAGE_OVERLAP );
				break;
			case inPageDown:
				delta = ( (**the_text).viewRect.bottom
					- (**the_text).viewRect.top - PAGE_OVERLAP );
				break;
		}
		old_value = GetCtlValue( the_bar );
		if ( ((delta < 0) && (old_value > 0)) ||
			((delta > 0) && (old_value < GetCtlMax(the_bar))) )
		{
			/*
				When this routine is called, TextEdit may have set the
				clipping region to the view rectangle, so we reset it
				here to make sure the scroll bar gets drawn.
			*/
			GetClip( save_clip );
			ClipRect( &the_display->portRect );
			SetCtlValue( the_bar, old_value + delta );
			SetClip( save_clip );
		}
		Adjust_text( the_display );
	}
#if A4_PROJECT
	RestoreA4();
#endif
}
/*=**************************************************************************=*/


/*=**************************************************************************=*/
static void Adjust_text( DialogPtr	dialog )
/*=--------------------------------------------------------------------------=*/
/*	Called by Handle_scroll and Scroll_text to scroll the text and */
/*	pictures into sync with the scroll bar's control value. */
/*=--------------------------------------------------------------------------=*/
{
	register				TEHandle	the_text;
	register INTEGER_16	scroll_down;
	INTEGER_16			old_scroll;
	Rect					update_rect;
	ControlHandle		the_bar;

/* END VAR */

	
	CKPT("\pAdjust_text");
	the_text = (TEHandle) GetWRefCon( dialog );
	the_bar = ((help_ptr) dialog)->scrollbar;
	old_scroll = (**the_text).viewRect.top - (**the_text).destRect.top;
	scroll_down = old_scroll - GetCtlValue( the_bar );
	if (scroll_down == 0)
		return;
	((help_ptr) dialog)->high_defer_flag = TRUE;
	((help_ptr) dialog)->high_waiting = 0;
	TEScroll( 0, scroll_down, the_text );
	update_rect = (**the_text).viewRect;
	if (scroll_down > 0)
	{
		if (scroll_down < (update_rect.bottom - update_rect.top))
			update_rect.bottom = update_rect.top + scroll_down;
	}
	else
		if (- scroll_down < (update_rect.bottom - update_rect.top))
			update_rect.top = update_rect.bottom + scroll_down;
	Draw_picts( dialog, &update_rect );
	Do_deferred_hilites( (help_ptr) dialog, &update_rect );
}
/*=**************************************************************************=*/


/*=**************************************************************************=*/
static void Do_deferred_hilites( help_ptr	hptr, Rect *update_rect )
{
	Rect			*hilite;

/* END VAR */

	
	while (hptr->high_waiting > 0)
	{
		hptr->high_waiting--;
		hilite =
			&hptr->high_rect[hptr->high_waiting];
		if (SectRect( hilite, update_rect, hilite ))
			InvertRect( hilite );
	}
	hptr->high_defer_flag = FALSE;
}
/*=**************************************************************************=*/


/*=**************************************************************************=*/
static void Handle_scroll( DialogPtr dialog, INTEGER_16 the_part, Point where )
/*=--------------------------------------------------------------------------=*/
/* Called by Filter_Events_For_Dialog to handle mouseDown events in the scroll bar. */
/*=--------------------------------------------------------------------------=*/
{
	register	ControlHandle the_bar;

/* END VAR */

	
	CKPT("\pHandle_scroll"); Set_View_Port( dialog );
	the_bar = ((help_ptr) dialog)->scrollbar;
	if (the_part == inThumb)
	{
		(void) TrackControl( the_bar, where, nil );
		Adjust_text( dialog );
	}
	else {
		
#if !defined(ORIGINAL_MAC_SYSTEM)
#error "UNDEFINED PARAMETER!"
#endif
#if (ORIGINAL_MAC_SYSTEM == 0)
XXXX 		(void) TrackControl( the_bar, where, (ProcPtr)Scroll_text );
#else
	(void) TrackControl( the_bar, where, (ControlActionUPP)Scroll_text );
#endif 
		
	}
	
}
/*=**************************************************************************=*/


/*=**************************************************************************=*/
static pascal Boolean Filter_Events_For_Dialog( DialogPtr dialog, EventRecord	*event, INTEGER_16 *itemHit)
/*=--------------------------------------------------------------------------=*/
/* This is the dialog event filter for our help window. */
/*=--------------------------------------------------------------------------=*/
{
	Point					local_point;
	INTEGER_16			the_part;
	ControlHandle		the_control;
	INTEGER_16			keycode, charcode;
	CARDINAL_8					the_char;
	register				TEHandle	the_text;
	Rect					item_box;
	INTEGER_16			cursor;
	
	INTEGER_16			item_type;
	Handle				help_item;

/* END VAR */

	
#if A4_PROJECT
	SetUpA4();
#endif

	GetDItem( dialog, SOMETHING_OK_DIALOG_SCROLLING_ITEM, &item_type, &help_item, &item_box );
	GetMouse(&local_point);
	item_box.right = item_box.right - SCROLLBAR_WIDTH;
	if (PtInRect(local_point, &item_box) == TRUE)
		Change_Cursor(iBeamCursor);
	else
		Arrow_Cursor();


	the_text = (TEHandle) GetWRefCon( dialog );
	TEIdle( the_text );
	
	
	switch (event->what) {
		case nullEvent:
			break;
		case mouseDown:
	CKPT("\pHelp_filter mousedown");
			local_point = event->where;
			GlobalToLocal( &local_point );
			the_part = FindControl( local_point, dialog, &the_control );
			if (the_part && ((**the_control).contrlMax > 1) )
			{
				Handle_scroll( dialog, the_part, local_point );	
				*itemHit = 2;
				break;
			}
			if (PtInRect( local_point, &(**the_text).viewRect ))
			{
				if (event->modifiers & shiftKey)
					TEClick( local_point, TRUE, the_text );
				else
					TEClick( local_point, FALSE, the_text );
			}
			break;
		case keyDown :	
		case autoKey :
			keycode = (event->message & keyCodeMask) >> 8;
			charcode = event->message & charCodeMask;
			/*
				There's no Cancel button, so we treat the OK button
				the same as a Cancel button.
			*/
			if ( (keycode == RETURN_KEYBOARD_CODE) || (keycode == ENTER_KEYBOARD_CODE) ||
				(keycode == TILDE_KEYBOARD_CODE) || (keycode == ESCAPE_KEYBOARD_CODE) ||
				((charcode == '.') && (event->modifiers & cmdKey)) )
			{
				*itemHit = SOMETHING_OK_DIALOG_OK_ITEM;	/* OK */
				Flash_button( dialog, *itemHit );
#if A4_PROJECT
				RestoreA4();
#endif
				return( TRUE );
			}
			if ( (charcode == 'c') && (event->modifiers & cmdKey) )
			{
				(void) ZeroScrap();
				TECopy( the_text );
				SystemEdit(3);
				local_point = (**the_text).selPoint;
				*(INTEGER_32 *)&local_point = PinRect( &(**the_text).viewRect,
					local_point );
				cursor = TEGetOffset( local_point, the_text );
				TESetSelect( (INTEGER_32)cursor, (INTEGER_32)cursor, the_text );
				event->what = nullEvent;
			}
			break;
	} /* end switch */
	
	/* tell the Dialog Manager that the event has NOT been handled and that it should
	** take further action on this event.
	*/
#if A4_PROJECT
	RestoreA4();
#endif
	return FALSE;
}
/*=**************************************************************************=*/


/*=**************************************************************************=*/
static pascal void	userItem_proc( WindowPtr dialog_Window, INTEGER_16 item_Number )
{
	Handle			item_h;
	Rect				item_box;
	INTEGER_16		item_type;
	TEHandle			the_text;
	
	INTEGER_16		item_Type;
	Handle			item_Handle;
	Rect				item_Rect;
	Str255				temp_String;

/* END VAR */

	
	asm {
		MOVEM.L	a1-a5/d0-d7, -(SP)
	}
	
	CKPT("\pUserItem_proc");
#if A4_PROJECT
	SetUpA4();
#endif



	switch (item_Number) {
	
			case SOMETHING_OK_DIALOG_SCROLLING_ITEM:
				the_text = (TEHandle) GetWRefCon( dialog_Window );
				item_box = (**the_text).viewRect;
				((help_ptr) dialog_Window)->high_defer_flag = TRUE;
				((help_ptr) dialog_Window)->high_waiting = 0;
				TEUpdate( &item_box, the_text );
				
				Draw_picts( dialog_Window, &item_box );
				Do_deferred_hilites( (help_ptr) dialog_Window, &item_box );
				
				/*
					Get the item's rectangle, and frame it.
				*/
				GetDItem( dialog_Window, item_Number, &item_type, &item_h, &item_box );
				FrameRect( &item_box );
			break;
			

		case SOMETHING_OK_DIALOG_OUTLINE_ITEM:
			if ((ns_Shell_Globals.operating_Conditions.options_Usage_Flags_1 & USAGE_FLAG_1_USE_MOTIF_MASK) == 0) {
				GetDItem(dialog_Window, SOMETHING_OK_DIALOG_OUTLINE_ITEM, &item_Type, &item_Handle, &item_Rect);
				PenSize(3, 3);
				FrameRoundRect(&item_Rect, 16, 16);
				PenSize(1, 1);
			}
			break;
			
		case SOMETHING_OK_HEADING_TEXT_ITEM:
			GetDItem(dialog_Window, SOMETHING_OK_HEADING_TEXT_ITEM, &item_Type, &item_Handle, &item_Rect);
		
			if (standard_Titles == TRUE) {
				TextFont(systemFont);
				TextFace(shadow);
				TextSize(12);
			}
			else {
				TextFont(geneva);
				TextSize(9);
			}
			
			TextBox ((Ptr)&something_OK_Dialog_Heading_Text_Ptr[1], something_OK_Dialog_Heading_Text_Ptr[0], &item_Rect, teJustCenter );
			TextFace(0);
			TextFont(systemFont);
			TextSize(12);

			break;
			
		case SOMETHING_OK_INFO_ITEM:
			GetDItem(dialog_Window, SOMETHING_OK_INFO_ITEM, &item_Type, &item_Handle, &item_Rect);
			
			Assign_String(temp_String, "\pwill copy selected text to the Clipboard.");
			TextFont(geneva);
			TextSize(9);
			TextBox ((Ptr)&temp_String[1], temp_String[0], &item_Rect, teJustLeft);
			TextFont(systemFont);
			TextSize(12);
			break;
			
	}



#if A4_PROJECT
	RestoreA4();
#endif
	asm {
		movem.l	(SP)+, a1-a5/d0-d7		; restore registers
	}
}
/*=**************************************************************************=*/


/*=**************************************************************************=*/
void Flash_button( DialogPtr the_dialog, INTEGER_16 item_number )
{
	ControlHandle	item_h;
	INTEGER_32		time;
	INTEGER_16		itype;
	Rect				box;

/* END VAR */

	
	GetDItem( the_dialog, item_number, &itype, (Handle *)&item_h, &box );
	HiliteControl( item_h, inButton );
	Delay( 9L, &time );
	HiliteControl( item_h, 0 );
}
/*=**************************************************************************=*/


/*=**************************************************************************=*/
/* END OF FILE: Show_Help.c */