/* START OF FILE: MOD_Utilities.c */


/*=**************************************************************************=*/
/* [---BEGIN---] FILE HEADER COMMENTS */
/**
 ** SCREEN VIEW = [Adobe Syntax Black-10, Tabs = 4]
 ** PRINTER VIEW = [Adobe Syntax Black-10, Tabs = 4]
 ** =======================================================================
 ** Written by James Merkle
 ** MOD_Utilities.c - Routines to support Playing MADF and MOD files
 ** (C) Copyright 1995 James Merkle
 ** =======================================================================
 **
 **
 ** ============
 ** Revision History
 ** ============
 **
 **  1995.06.08		(James Merkle)
 **						Initial coding started from example of Antoine Rosset's "Example.c"
 **
 **
 **
 ** =================
 ** OTHER INFORMATION
 ** =================
 ** 
 ** 
 **
 **
 **/
/* [----END----] FILE HEADER COMMENTS */
/*=**************************************************************************=*/


/*=**************************************************************************=*/
/* [---BEGIN---] #include AREA */

#include <gestaltequ.h>
#include <sound.h>

#include "MOD_Utilities_MAD.h"
#include "MOD_Utilities_RDriver.h"

#include "OS_Utilities.h"

#include "MOD_Utilities.h"

/* [----END----] #include AREA */
/*=**************************************************************************=*/


/*=**************************************************************************=*/
/* [---BEGIN---] #define AREA */


/* [----END----] #define AREA */
/*=**************************************************************************=*/


/*=**************************************************************************=*/
/* [---BEGIN---] TYPE AREA */


/* [----END----] TYPE AREA */
/*=**************************************************************************=*/


/*=**************************************************************************=*/
/* [---BEGIN---] PROTOTYPE AREA */


/* [----END----] PROTOTYPE AREA */
/*=**************************************************************************=*/


/*=**************************************************************************=*/
/* [---BEGIN---] GLOBAL DEFINITION AREA */


/* [----END----] GLOBAL DEFINITION AREA */
/*=**************************************************************************=*/


/*=**************************************************************************=*/
/* [---BEGIN---] LOCAL STATIC DEFINITION AREA */

static CARDINAL_8	*unit_Name;

Safe_Boolean		g_MOD_Utilities_Module_Initialized;
Safe_Boolean		g_MOD_Utilities_Driver_Initialized;
Safe_Boolean		g_MOD_Utilities_Music_Possibly_Playing;
CARDINAL_32	g_Active_Audio_Driver_Type;
CARDINAL_32	g_Active_Audio_Driver_Max_Channels;

Boolean		Stereo;
Boolean		StereoMixing;
Boolean		NewSoundManager;
Boolean		hasASC;

/* [----END----] LOCAL STATIC DEFINITION AREA */
/*=**************************************************************************=*/


/*=**************************************************************************=*/
Safe_Boolean Initialize_MOD_Utilities(void)
{
	CARDINAL_8		*static_Unit_Name = "\pMOD_Utilities";
	Safe_Boolean		function_Result;
	INTEGER_16		myBit;
	INTEGER_32	gestaltAnswer;
	CARDINAL_32	temp_Value;
	NumVersion		nVers;
/* END VAR */

	/*  */
	/* Global persistant structures are created here */
	/*  */
	unit_Name = static_Unit_Name;
	
	g_MOD_Utilities_Module_Initialized = SAFE_FALSE;
	g_MOD_Utilities_Driver_Initialized = SAFE_FALSE;
	g_MOD_Utilities_Music_Possibly_Playing = SAFE_FALSE;
	
	/*  */
	/* Paranoia only */
	/*  */
	g_Active_Audio_Driver_Type = 0;
	g_Active_Audio_Driver_Max_Channels = 0;
	
	/*  */
	/* HARDWARE IDENTIFICATION AND CHECK */
	/*  */
		
	/*  */
	/* ASC CHIP ? */
	/*  */
	
	Gestalt( gestaltHardwareAttr, &gestaltAnswer);
	myBit = gestaltHasASC;
	if( BitTst( &gestaltAnswer, 31-myBit) == false) {
		hasASC = false;
	}
	else {
		hasASC = true;
	}
	
	/*  */
	/* STEREO ? */
	/*  */
	
	Gestalt( gestaltSoundAttr, &gestaltAnswer);
	myBit = gestaltStereoCapability;
	Stereo = BitTst( &gestaltAnswer, 31-myBit);
	
	
	/*  */
	/*  */
	/*  */
	/*  */
	/*  */
	/*  */
	/* HACK HACK HACK !! Disable stereo. Most people have one speaker. */
	/*  */
	Stereo = FALSE;
	
	/*  */
	/* STEREO MIXING */
	/*  */
	
	myBit = gestaltStereoMixing;
	StereoMixing = BitTst( &gestaltAnswer, 31-myBit);
	
	/*  */
	/* SOUND MANAGER >= 3.0 ? */
	/*  */
	
	/*  */
	/* Think C 7.0.3 latest and MW C 7 1.3.1 do not have same */
	/* headers regarding version structure size!! Shitty MW C wont let me assign! */
	/*  */
	
#if (defined(__MWERKS__))
	temp_Value = SndSoundManagerVersion();
	BlockMoveData((Ptr)&temp_Value, (Ptr)&nVers, sizeof(CARDINAL_32));
#else 
	nVers = SndSoundManagerVersion();
#endif

	
	if( nVers.majorRev >= 3) {
		NewSoundManager = true;
	}
	else {
		NewSoundManager = false;
	}
		
	g_MOD_Utilities_Module_Initialized = SAFE_TRUE;
	
	function_Result = SAFE_TRUE;
	goto Normal_Function_Exit;

Error_Handler:
	function_Result = SAFE_FALSE;
		
Normal_Function_Exit:
	return(function_Result);
	
} /* Safe_Boolean Initialize_MOD_Utilities(void) */
/*=**************************************************************************=*/


/*=**************************************************************************=*/
Safe_Boolean Stop_MOD_Playback(void)
{
	Safe_Boolean		function_Result;
/* END VAR */


	if (g_MOD_Utilities_Module_Initialized != SAFE_TRUE) {
		goto Error_Handler;
	}

	if (g_MOD_Utilities_Music_Possibly_Playing == SAFE_TRUE) {
	
		/*  */
		/* Stop music */
		/*  */
		RStopMusic();
		CleanDriver();
		
		/*  */
		/* Dispose the current music */
		/*  */
		RClearMusic();
	
		g_MOD_Utilities_Music_Possibly_Playing = SAFE_FALSE;
	}
	
	function_Result = SAFE_TRUE;
	goto Normal_Function_Exit;

Error_Handler:
	function_Result = SAFE_FALSE;
		
Normal_Function_Exit:
	return(function_Result);
}
/*=**************************************************************************=*/


/*=**************************************************************************=*/
Safe_Boolean Start_MOD_Playback(void)
{
	Safe_Boolean		function_Result;
/* END VAR */


	if (g_MOD_Utilities_Module_Initialized != SAFE_TRUE) {
		goto Error_Handler;
	}

	if (g_MOD_Utilities_Driver_Initialized != SAFE_TRUE) {
		goto Error_Handler;
	}

	if (g_MOD_Utilities_Music_Possibly_Playing == SAFE_TRUE) {
	
		/*  */
		/* Stop music ? */
		/*  */
	
		g_MOD_Utilities_Music_Possibly_Playing = SAFE_FALSE;
	}
	
	RPlayMusic();
	
	/*  */
	/* "Reading" is a special external global. */
	/* I hope the driver treats it "volatile" because we change it here. */
	/*  */
	Reading = true;
	g_MOD_Utilities_Music_Possibly_Playing = SAFE_TRUE;
	
	function_Result = SAFE_TRUE;
	goto Normal_Function_Exit;

Error_Handler:
	function_Result = SAFE_FALSE;
		
Normal_Function_Exit:
	return(function_Result);
	
} /* Safe_Boolean Start_MOD_Playback(void) */
/*=**************************************************************************=*/


/*=**************************************************************************=*/
Safe_Boolean Destroy_MOD_Driver(void)
{
	Safe_Boolean		function_Result;
/* END VAR */


	if (g_MOD_Utilities_Module_Initialized != SAFE_TRUE) {
		goto Error_Handler;
	}

	if (g_MOD_Utilities_Driver_Initialized == SAFE_TRUE) {
		if (g_MOD_Utilities_Music_Possibly_Playing == SAFE_TRUE) {
			if (Stop_MOD_Playback() != SAFE_TRUE) {
				/*  */
				/* Ignore error, we must try and close driver. */
				/*  */
			}
		}
		
		/*  */
		/* Dispose music driver */
		/*  */
		RQuitMusic();
		
		g_MOD_Utilities_Driver_Initialized = SAFE_FALSE;
	}
	
	function_Result = SAFE_TRUE;
	goto Normal_Function_Exit;

Error_Handler:
	function_Result = SAFE_FALSE;
		
Normal_Function_Exit:
	return(function_Result);
}
/*=**************************************************************************=*/


/*=**************************************************************************=*/
Safe_Boolean Destroy_MOD_Utilities(void)
{
	Safe_Boolean		function_Result;
/* END VAR */

	if (g_MOD_Utilities_Module_Initialized == SAFE_TRUE) {
	
		if (Destroy_MOD_Driver() != SAFE_TRUE) {
			/*  */
			/* Ignore error, we must try and close driver. */
			/*  */
		}
	
		g_MOD_Utilities_Module_Initialized = SAFE_FALSE;
	}
	
	function_Result = SAFE_TRUE;
	return(function_Result);
}
/*=**************************************************************************=*/


/*=**************************************************************************=*/
Safe_Boolean Load_MOD_Utilities_Driver(void)
{
	Safe_Boolean		function_Result;
	CARDINAL_32	desired_Max_Channels;
	CARDINAL_32	desired_Audio_Driver;
/* END VAR */


	if (g_MOD_Utilities_Module_Initialized != SAFE_TRUE) {
		goto Error_Handler;
	}
	
	/*  */
	/* MAD Library Initialization, choose the best driver */
	/* for the current hardware. Standard initialization with 4 tracks */
	/* 	ASCMono */
	/* 	ASCStereo */
	/* 	SMMono */
	/* 	SMStereo */
	/* 	SMDSP */
	/* 	SMPolyPhonic */
	/*  */
	
	desired_Max_Channels = 4;
	desired_Audio_Driver = 0;
	
	if( (Stereo == true) && (StereoMixing == true))
	{		
		if( NewSoundManager == true)
		{
			desired_Audio_Driver = SMStereo;
		}
		else if( hasASC == true)
		{
			desired_Audio_Driver = ASCStereo;
		}
		else {
			desired_Audio_Driver = SMDSP;
		}
	}
	else
	{		
		if( NewSoundManager == true)
		{
			desired_Audio_Driver = SMMono;
		}
		else if( hasASC == true)
		{
			desired_Audio_Driver = ASCMono;
		}
		else {
			/*  */
			/* This mac cannot be used for music playback */
			/*  */
			goto Error_Handler;
		}
	}



	if (g_MOD_Utilities_Driver_Initialized == SAFE_TRUE) {
		/*  */
		/* If the driver is already loaded then it is shutdown if a */
		/* different set of features is needed.*/
		/*  */
		if ((desired_Audio_Driver != g_Active_Audio_Driver_Type)
			|| (desired_Max_Channels != g_Active_Audio_Driver_Max_Channels)){
		
			if (Destroy_MOD_Driver() != SAFE_TRUE) {
				/*  */
				/* Ignore error, it might not have really been open. */
				/*  */
			}
		}
	}

	
	if (g_MOD_Utilities_Driver_Initialized != SAFE_TRUE) {
	
		if( RInitMusic( desired_Audio_Driver, desired_Max_Channels) != noErr) {
			goto Error_Handler;
		}
		
		g_Active_Audio_Driver_Type = desired_Audio_Driver;
		g_Active_Audio_Driver_Max_Channels = desired_Max_Channels;
		g_MOD_Utilities_Driver_Initialized = SAFE_TRUE;
	}
	
	
	function_Result = SAFE_TRUE;
	goto Normal_Function_Exit;

Error_Handler:
	function_Result = SAFE_FALSE;
		
Normal_Function_Exit:
	return(function_Result);
	
} /* Safe_Boolean Load_MOD_Utilities_Driver(void) */
/*=**************************************************************************=*/


/*=**************************************************************************=*/
Safe_Boolean Start_Playing_MOD_Resource(INTEGER_16 resource_ID
												, MADPartition	*new_Music_Partition
												, Handle			*resource_Hdl)
{
	Safe_Boolean		function_Result;
	CARDINAL_32	sound_Size;
/* END VAR */


	if (g_MOD_Utilities_Module_Initialized != SAFE_TRUE) {
		goto Error_Handler;
	}

	if (Load_MOD_Utilities_Driver() != SAFE_TRUE) {
		goto Error_Handler;
	}
	
	*resource_Hdl = Get1Resource( 'MODm', resource_ID);
	if ((ResError() != noErr) || (*resource_Hdl == nil)) {
		goto Error_Handler;
	}
	
	MoveHHi(*resource_Hdl);
	if (MemError() != noErr) {
		if (MemError() != memLockedErr) {
			goto Error_Handler;
		}
	}

	HLock(*resource_Hdl);
	if (MemError() != noErr) {
		goto Error_Handler;
	}
	
	sound_Size = GetHandleSize(*resource_Hdl);
	if (MemError() != noErr) {
		goto Error_Handler;
	}
	
	ConvertMod2Mad(*(*resource_Hdl), sound_Size, new_Music_Partition);
	RInstallMADF(*new_Music_Partition);
	
	/*  */
	/* Update the tracks number ! */
	/*  */
	if( theFileInt->Tracks != DriveTrackNo)
	{
		DriveTrackNo = theFileInt->Tracks;
		ChangeTrackNo( DriveTrackNo);
	}

	if (Start_MOD_Playback() != SAFE_TRUE) {
		/*  */
		/* Ignore errorin this version. */
		/*  */
	}
	
	function_Result = SAFE_TRUE;
	goto Normal_Function_Exit;

Error_Handler:
	function_Result = SAFE_FALSE;
		
Normal_Function_Exit:
	return(function_Result);
	
} /* Safe_Boolean Start_Playing_About_Music(... */
/*=**************************************************************************=*/


/*=**************************************************************************=*/
Safe_Boolean Stop_Playing_MOD_Resource(MADPartition	*new_Music_Partition
													,Handle			*resource_Hdl)
{
	Safe_Boolean		function_Result;
/* END VAR */
	

	if (g_MOD_Utilities_Module_Initialized != SAFE_TRUE) {
		goto Error_Handler;
	}

	if (Stop_MOD_Playback() != SAFE_TRUE) {
		/*  */
		/* Ignore error, it might not have really been playing still. */
		/*  */
	}

	if (Destroy_MOD_Driver() != SAFE_TRUE) {
		/*  */
		/* Ignore error, it might not have really been open. */
		/*  */
	}
	
	ReleaseResource(*resource_Hdl);
	if (MemError() != noErr) {
		goto Error_Handler;
	}
	*resource_Hdl = nil;

	function_Result = SAFE_TRUE;
	goto Normal_Function_Exit;

Error_Handler:
	function_Result = SAFE_FALSE;
		
Normal_Function_Exit:
	return(function_Result);
}
/*=**************************************************************************=*/


/*=**************************************************************************=*/  
void Play_A_User_Selected_Music_File(void)
{
	long			sndSize;
	short			iFileRefI;
	Point			where = { -1, -1};
	SFReply			reply;
	SFTypeList		aType;
	OSErr			iErr = noErr;
	Ptr				tempPtr;
	MADPartition	aPartition;
	Boolean			End;
	
	
	if (g_MOD_Utilities_Module_Initialized != SAFE_TRUE) {
		goto Error_Handler;
	}

	if (Load_MOD_Utilities_Driver() != SAFE_TRUE) {
		goto Error_Handler;
	}

	End = false;
	
	while( End == false)
	{

		aType[ 0] = 'MADF';
		aType[ 1] = 'STrk';
		SFGetFile( where, "\p", 0L, 2, aType, 0L, &reply);
		
		if( !reply.good) End = true;
		else
		{
			SetVol( 0L, reply.vRefNum);
			
			if( reply.fType == 'STrk')			// **** MOD File ****
			{
				iErr = FSOpen( reply.fName, 0, &iFileRefI);
				if( iErr) DebugStr( "\pProblemo ??");
				
				GetEOF( iFileRefI, &sndSize);
					
				tempPtr = NewPtr( sndSize);
				FSRead( iFileRefI, &sndSize, tempPtr);
				FSClose( iFileRefI);
					
				ConvertMod2Mad( tempPtr, sndSize, &aPartition);
				RInstallMADF( aPartition);
						
				DisposPtr( tempPtr);	tempPtr = 0L;
			}
			else if( reply.fType == 'MADF') 	// **** MAD File ****
			{
				/*** To load a MAD File *********/
				if( RLoadMusic( reply.fName) != noErr) Debugger();
				
				/*
					*** TO OPEN A MAD RESOURCE ***
					
					To create a MAD Resource, use Auto-Exec export from Player PRO !!!!
					And load the resource with: by example...
				
					RLoadMusicRsrc( 'MADF', 3124); for example...
				*/
			}
			else DebugStr("\pUh Uh...");
			
			
			/****************************************/
			/**** Update the tracks number !! *******/
			/****************************************/
			if( theFileInt->Tracks != DriveTrackNo)
			{
				DriveTrackNo = theFileInt->Tracks;
				ChangeTrackNo( DriveTrackNo);
			}
			/****************************************/
			

			if (Start_MOD_Playback() != SAFE_TRUE) {
				/*  */
				/* Ignore errorin this version. */
				/*  */
			}
						
			while( !Button())
			{
				/** Do what you want here....	**/
				/** Bla bla...					**/
			}
			
			
			if (Stop_MOD_Playback() != SAFE_TRUE) {
				/*  */
				/* Ignore error, it might not have really been open. */
				/*  */
			}
			
		}
	}

	Error_Handler:
	
	if (Destroy_MOD_Driver() != SAFE_TRUE) {
		/*  */
		/* Ignore error, it might not have really been open. */
		/*  */
	}

	
} /* void Play_A_User_Selected_Music_File(void) */
/*=**************************************************************************=*/


/*=**************************************************************************=*/
/* END OF FILE: MOD_Utilities.c */






