/* START OF FILE: Dialog_Of_Background_Removal.c */


/*=**************************************************************************=*/
/* [---BEGIN---] FILE HEADER COMMENTS */
/**
 ** SCREEN VIEW = [Adobe Syntax Black-10, Tabs = 4]
 ** PRINTER VIEW = [Adobe Syntax Black-10, Tabs = 4]
 ** =======================================================================
 ** Written by James Merkle
 ** Dialog_Of_Background_Removal.c - Video Routines to support Poor Man's Video Phone
 ** (C) Copyright 1994 James Merkle
 ** All rights reserved, except those wishing to "copy" moderate sections of code or files may do so, so long as
 ** such code is never used directly either in animation-related tools or multimedia tools that are  
 ** used in live performance or live broadcast or recorded mass market broadcast or film. Desktop usage not restricted.
 ** Conventional desktop game or entertainment not restricted.
 ** =======================================================================
 **
 **
 ** ============
 ** Revision History
 ** ============
 **
 **  1995.04.14		(James Merkle)
 **						Initial coding started from scratch
 **
 **
 **
 ** =================
 ** OTHER INFORMATION
 ** =================
 ** 
 ** 
 **
 **
 **/
/* [----END----] FILE HEADER COMMENTS */
/*=**************************************************************************=*/


/*=**************************************************************************=*/
/* [---BEGIN---] #include AREA */


#include "Automatic_Dialogs.h"
#include "String_Utilities.h"
#include "Error_Utilities.h"
/* ===
#include "Command_Dialogs.h"
=== */

#include "Shell_Assembly.h"
#include "Dialog_Of_Background_Removal.h"

/* [----END----] #include AREA */
/*=**************************************************************************=*/


/*=**************************************************************************=*/
/* [---BEGIN---] #define AREA */

#define	BACKGROUND_OPTIONS_DIALOG_ID				7200
#define	BACKGROUND_OPTIONS_OUTLINE_ITEM			3

#define k_QUICKDRAW_16_BIT_SCALE	(8)

/*  */
/* The background will be eliminated if the last accepted background pixel upward */
/* is similar to this pixel. To enable this technology set to "1" instead of "0". */
/* It allows colors to change subtly across the background top to bottom. */
/* It can recover across a puppets head because it starts from last accepted pixel */
/* Normally (1) */
/*  */
#define k_ALLOW_HORIZONTAL_DRIFT_TRACKING				(1)

/*  */
/* The background will be eliminated if the last accepted background pixel leftward */
/* is similar to this pixel. To enable this technology set to "1" instead of "0" */
/* It allows colors to change subtly across the background left to right. */
/* It can recover across a puppets head because it starts from last accepted pixel */
/* Normally (1) */
/*  */
#define k_ALLOW_VERTICAL_DRIFT_TRACKING					(1)



#define k_BRIGHT_ENOUGH_TO_TRUST_FOR_BRIGHTEN		(64L * 256UL)

/*  */
/* To remove the pixel from consideration if it is pure chroma */
/* with color gun value from 8 to 41 inclusive and other channels */
/* too dark ( < 2 ) */
/*  */
#define k_ALLOW_COLOR_TRAPPING_DARK_BACKGROUND_CUTOFF		(100 * 256UL)
#define k_MAXIMUM_TRAP_DARK_BACKGROUND	(41 * 256UL)
#define k_MINIMUM_TRAP_DARK_BACKGROUND	(8 * 256UL)
#define k_BLACK_CHANNEL_GUN_LIMIT				(1 * 256UL)


/*  */
/* Allow another try at removal if deviance is high and brightness */
/* of main color is 70% and if other two colors are same deviance sign */
/* direction but not if one is deviant in just a minor way with opposite sign */
/*  */
#define k_MINIMUM_INTENSITY_TO_CONSIDER_LENIANCE		(140 * 256UL)
#define k_INSIGNIFICANT_DEVIANCE 								(10 * 256UL)



/*  */
/* These should be customizable but for now, hard wire them */
/* 5 Bit color only starts deviating in 8th value and higher*/
/* All deviations are Absolute value not relative */
/* They are adjusted against 16 bit torage format */
/*  */
#define k_MAXIMUM_5_BIT_RED_CHANNEL_DEVIANCE				(14 * 256UL)
#define k_MAXIMUM_5_BIT_BLUE_CHANNEL_DEVIANCE			(14 * 256UL)
#define k_MAXIMUM_5_BIT_GREEN_CHANNEL_DEVIANCE			(14 * 256UL)

/*  */
/* This value is used in an aggregate capacity. All deviations are added together */
/* after made absolute */
/* 9.6 = 10% for 5 bit channels */
/*  */
#define k_MAXIMUM_5_BIT_TOTAL_CHANNEL_DEVIANCE		(29 * 256UL)


/*  */
/* These should be customizable but for now, hard wire them */
/* 5 Bit color only starts deviating in 8th value and higher*/
/* All deviations are Absolute value not relative */
/* They are adjusted against 16 bit torage format */
/*  */
#define k_MAXIMUM_5_BIT_RED_CHANNEL_LENIENT_DEVIANCE				(14 * 256UL)
#define k_MAXIMUM_5_BIT_BLUE_CHANNEL_LENIENT_DEVIANCE				(14 * 256UL)
#define k_MAXIMUM_5_BIT_GREEN_CHANNEL_LENIENT_DEVIANCE			(14 * 256UL)

/*  */
/* This value is used in an aggregate capacity. All deviations are added together */
/* after made absolute */
/* 9.6 = 10% for 5 bit channels */
/*  */
#define k_MAXIMUM_5_BIT_TOTAL_CHANNEL_LENIENT_DEVIANCE		(29 * 256UL)



/*  */
/* These are for allowing drifting background color ramp removal */
/* It uses tighter match parameters */
/* Total color drift is allowed to be 12.5% */
/*  */
#define k_MAXIMUM_5_BIT_RED_DRIFT_DEVIANCE				(14 * 256UL)
#define k_MAXIMUM_5_BIT_BLUE_DRIFT_DEVIANCE			(14 * 256UL)
#define k_MAXIMUM_5_BIT_GREEN_DRIFT_DEVIANCE			(14 * 256UL)
#define k_MAXIMUM_5_BIT_TOTAL_DRIFT_DEVIANCE			(29 * 256UL)


/*  */
/* Allow only 20% brightness deviance ever in comparisons */
/*  */
#define k_MAX_BRIGHTNESS_DEVIANCE_5						(3UL *138UL * 256UL)
#define k_MAX_DARKNESS_DEVIANCE_5							(59L * 256UL)


/*  */
/* This is the maximum the colors can drift if the background is drifting. */
/* It is used only if drift deviance is enabled and only if a match is made */
/*  */
#define k_MAXIMUM_5_BIT_RED_CHANNEL_ABSOLUTE_HIGH_LIMIT				((5L * 8L) * 256UL)
#define k_MAXIMUM_5_BIT_BLUE_CHANNEL_ABSOLUTE_HIGH_LIMIT			((5L * 8L) * 256UL)
#define k_MAXIMUM_5_BIT_GREEN_CHANNEL_ABSOLUTE_HIGH_LIMIT			((5L * 8L) * 256UL)

/*  */
/* This value is used in an aggregate capacity. All deviations are added together */
/* after made absolute */
/*  */
#define k_MAXIMUM_5_BIT_TOTAL_CHANNEL_ABSOLUTE_HIGH_LIMIT			((8L * 8L) * 256UL)


/*  */
/* This is the maximum the colors can drift if the background is drifting. */
/* In this case the suspect pixel is darker than optimal */
/* It is used only if drift deviance is enabled and only if a match is made */
/*  */
#define k_MAXIMUM_5_BIT_RED_CHANNEL_ABSOLUTE_LOW_LIMIT				((3L * 8L) * 256UL)
#define k_MAXIMUM_5_BIT_BLUE_CHANNEL_ABSOLUTE_LOW_LIMIT				((3L * 8L) * 256UL)
#define k_MAXIMUM_5_BIT_GREEN_CHANNEL_ABSOLUTE_LOW_LIMIT			((3L * 8L) * 256UL)

/*  */
/* This value is used in an aggregate capacity. All deviations are added together */
/* after made absolute */
/*  */
#define k_MAXIMUM_5_BIT_TOTAL_CHANNEL_ABSOLUTE_LOW_LIMIT			((4L * 8L) * 256UL)



/*  */
/* These should be customizable but for now, hard wire them */
/* 8 = 3.1% for 8 Bit color channels */
/*  */
#define k_MAXIMUM_8_BIT_RED_CHANNEL_DEVIANCE			(14L * 256UL)
#define k_MAXIMUM_8_BIT_BLUE_CHANNEL_DEVIANCE		(14L * 256UL)
#define k_MAXIMUM_8_BIT_GREEN_CHANNEL_DEVIANCE		(14L * 256UL)

/*  */
/* This value is used in an aggregate capacity. All deviations are added together */
/* after made absolute */
/* 23.0 = 3% for three 8 bit channels */
/* NORMALLY 29, but allow it to be tighter (23,14?) */
/*  */	
#define k_MAXIMUM_8_BIT_TOTAL_CHANNEL_DEVIANCE		(23L * 256UL)


/*  */
/* These should be customizable but for now, hard wire them */
/* 8 = 3.1% for 8 Bit color channels */
/*  */
#define k_MAXIMUM_8_BIT_RED_CHANNEL_LENIENT_DEVIANCE			((27L) * 256UL)
#define k_MAXIMUM_8_BIT_BLUE_CHANNEL_LENIENT_DEVIANCE			((27L) * 256UL)
#define k_MAXIMUM_8_BIT_GREEN_CHANNEL_LENIENT_DEVIANCE		((27L) * 256UL)

/*  */
/* This value is used in an aggregate capacity. All deviations are added together */
/* after made absolute */
/* 23.0 = 3% for three 8 bit channels */
/*  */	

#define k_MAXIMUM_8_BIT_TOTAL_CHANNEL_LENIENT_DEVIANCE		((73L) * 256UL)


/*  */
/* These are for allowing drifting background color ramp removal */
/* It uses tighter match parameters */
/* Total color drift is allowed to be 12.5% */
/* was too aggressive at 24,24,24,29 / now is 8,8,8, 13 */
/*  */
#define k_MAXIMUM_8_BIT_RED_DRIFT_DEVIANCE				(8L * 256UL)
#define k_MAXIMUM_8_BIT_BLUE_DRIFT_DEVIANCE			(8L * 256UL)
#define k_MAXIMUM_8_BIT_GREEN_DRIFT_DEVIANCE			(8L * 256UL)
#define k_MAXIMUM_8_BIT_TOTAL_DRIFT_DEVIANCE			(13L * 256UL)


/*  */
/* This is the maximum the colors can drift if the background is drifting. */
/* It is used only if drift deviance is enabled and only if a match is made */
/*  */
#define k_MAXIMUM_8_BIT_RED_CHANNEL_ABSOLUTE_HIGH_LIMIT				((5L * 8L) * 256UL)
#define k_MAXIMUM_8_BIT_BLUE_CHANNEL_ABSOLUTE_HIGH_LIMIT			((5L * 8L) * 256UL)
#define k_MAXIMUM_8_BIT_GREEN_CHANNEL_ABSOLUTE_HIGH_LIMIT			((5L * 8L) * 256UL)

/*  */
/* This value is used in an aggregate capacity. All deviations are added together */
/* after made absolute */
/* 8 */
/*  */
#define k_MAXIMUM_8_BIT_TOTAL_CHANNEL_ABSOLUTE_HIGH_LIMIT			((9L * 8L) * 256UL)


/*  */
/* This is the maximum the colors can drift if the background is drifting. */
/* In this case the suspect pixel is darker than optimal */
/* It is used only if drift deviance is enabled and only if a match is made */
/*  */
#define k_MAXIMUM_8_BIT_RED_CHANNEL_ABSOLUTE_LOW_LIMIT				((3L * 8L) * 256UL)
#define k_MAXIMUM_8_BIT_BLUE_CHANNEL_ABSOLUTE_LOW_LIMIT				((3L * 8L) * 256UL)
#define k_MAXIMUM_8_BIT_GREEN_CHANNEL_ABSOLUTE_LOW_LIMIT			((3L * 8L) * 256UL)

/*  */
/* This value is used in an aggregate capacity. All deviations are added together */
/* after made absolute */
/*  */
#define k_MAXIMUM_8_BIT_TOTAL_CHANNEL_ABSOLUTE_LOW_LIMIT			((6L * 8L) * 256UL)

/*  */
/* This allows loose but not too loose luminance (brightness) */
/* Currently using 100% of 3 color channels of 50% each */
/*  */
#define k_MAX_BRIGHTNESS_DEVIANCE_8						(3UL *138UL * 256UL)

/*  */
/* It is important that we only intensify backgrounds about half as much */
/* as we tolerate brightly lit pixels */
/*  */
#define k_MAX_DARKNESS_DEVIANCE_8							(59UL * 256UL)

/* [----END----] #define AREA */
/*=**************************************************************************=*/


/*=**************************************************************************=*/ 
/* [---BEGIN---] TYPE AREA */

/* ======
#define ITEM_ANY				0
#define ITEM_BUTTON			1
#define ITEM_CHECK_BOX		2
#define ITEM_RADIO			3
#define ITEM_STATIC_TEXT	4
#define ITEM_EDIT_TEXT		5
#define ITEM_ICON				6
#define ITEM_PICT				7
#define ITEM_POPUP			8
#define ITEM_LIST				9
#define ITEM_USER				10
#define ITEM_REG_CONTROL		11
#define ITEM_AUTO_CONTROL	12
========= */

typedef struct {
	Item_Variables_Header	header;
	
#if (k_IGNORE_EXAMPLES == 0)

	Boolean_Item			example_Check_Box;
	
	Boolean_Item			example_Radio_String_Choice;
	Edit_Text_Item			example_Radio_String_Choice_Text;

	Boolean_Item			example_Radio_Value_Choice;
	Edit_Text_Item			example_Radio_Value_Choice_Value; /* 27 */

	Edit_Text_Item			example_Number_Info_String; /* 83 */
	
#endif /* #if (k_IGNORE_EXAMPLES == 0) */

	Boolean_Item	allow_Horizontal_Drift_Tracking_Item;
	Boolean_Item	allow_Vertical_Drift_Tracking_Item;
	
	Edit_Text_Item	bright_Enough_To_Trust_For_Brighten_Item;
	Edit_Text_Item	allow_Color_Trapping_Dark_Background_Cuttoff_Item;
	Edit_Text_Item	maximum_Trap_Dark_Background_Item;
	Edit_Text_Item	minimum_Trap_Dark_Background_Item;
	Edit_Text_Item	black_Channel_Gun_Limit_Item_Item;
	Edit_Text_Item	minimum_Intensity_To_Consider_Leniance_Item;
	Edit_Text_Item	insignificant_Deviance_Item;
	Edit_Text_Item	maximum_Red_Channel_Deviance_Item;
	Edit_Text_Item	maximum_Blue_Channel_Deviance_Item;
	Edit_Text_Item	maximum_Green_Channel_Deviance_Item;
	Edit_Text_Item	maximum_Total_Channel_Deviance_Item;
	Edit_Text_Item	maximum_Red_Channel_Lenient_Deviance_Item;
	Edit_Text_Item	maximum_Blue_Channel_Lenient_Deviance_Item;
	Edit_Text_Item	maximum_Green_Channel_Lenient_Deviance_Item;
	Edit_Text_Item	maximum_Total_Channel_Lenient_Deviance_Item;
	Edit_Text_Item	maximum_Red_Drift_Deviance_Item;
	Edit_Text_Item	maximum_Blue_Drift_Deviance_Item;
	Edit_Text_Item	maximum_Green_Drift_Deviance_Item;
	Edit_Text_Item	maximum_Total_Drift_Deviance_Item;
	Edit_Text_Item	maximum_Brightness_Deviance_Item;
	Edit_Text_Item	maximum_Darkness_Deviance_Item;
	Edit_Text_Item	maximum_Red_Channel_Absolute_High_Limit_Item;
	Edit_Text_Item	maximum_Blue_Channel_Absolute_High_Limit_Item;
	Edit_Text_Item	maximum_Green_Channel_Absolute_High_Limit_Item;
	Edit_Text_Item	maximum_Total_Channel_Absolute_High_Limit_Item;
	Edit_Text_Item	maximum_Red_Channel_Absolute_Low_Limit_Item;
	Edit_Text_Item	maximum_Blue_Channel_Absolute_Low_Limit_Item;
	Edit_Text_Item	maximum_Green_Channel_Absolute_Low_Limit_Item;
	Edit_Text_Item	maximum_Total_Channel_Absolute_Low_Limit_Item;
	
	User_Item					cancel_Button_Outline; /* 76 (70th item) */
} Background_Options_Item_Variables_Record;



/* [----END----] TYPE AREA */
/*=**************************************************************************=*/


/*=**************************************************************************=*/
/* [---BEGIN---] PROTOTYPE AREA */

Safe_Boolean Initialize_Defaults_For_Background_Removal(Background_Options_Ptr	options);

static Safe_Boolean Validate_And_Copy_Contents(Background_Options_Item_Variables_Record *items
													, Background_Options_Ptr options
													, CARDINAL_16 color_Resolution);

Safe_Boolean Scale_Input_Defaults_For_Background_Removal(Background_Options_Ptr	options);

Safe_Boolean Scale_Output_Defaults_For_Background_Removal(Background_Options_Ptr	options);

void Assign_Item_String_Value(Edit_Text_Item *item, INTEGER_32 value, CARDINAL_16 string_Width);

/* [----END----] PROTOTYPE AREA */
/*=**************************************************************************=*/


/*=**************************************************************************=*/
/* [---BEGIN---] GLOBAL DEFINITION AREA */


/* [----END----] GLOBAL DEFINITION AREA */
/*=**************************************************************************=*/


/*=**************************************************************************=*/
/* [---BEGIN---] LOCAL STATIC DEFINITION AREA */

static CARDINAL_8	*unit_Name = "\pDialog_Of_Background_Removal";


/* [----END----] LOCAL STATIC DEFINITION AREA */
/*=**************************************************************************=*/


/*=**************************************************************************=*/
Safe_Boolean Scale_Input_Defaults_For_Background_Removal(Background_Options_Ptr	options)
{	
	options->bright_Enough_To_Trust_For_Brighten >>= k_QUICKDRAW_16_BIT_SCALE;
	options->allow_Color_Trapping_Dark_Background_Cuttoff >>= k_QUICKDRAW_16_BIT_SCALE;
	options->maximum_Trap_Dark_Background >>= k_QUICKDRAW_16_BIT_SCALE;
	options->minimum_Trap_Dark_Background >>= k_QUICKDRAW_16_BIT_SCALE;
	options->black_Channel_Gun_Limit >>= k_QUICKDRAW_16_BIT_SCALE;
	options->minimum_Intensity_To_Consider_Leniance >>= k_QUICKDRAW_16_BIT_SCALE;
	options->insignificant_Deviance >>= k_QUICKDRAW_16_BIT_SCALE;
	options->maximum_5_Bit_Red_Channel_Deviance  >>= k_QUICKDRAW_16_BIT_SCALE;
	options->maximum_5_Bit_Blue_Channel_Deviance  >>= k_QUICKDRAW_16_BIT_SCALE;
	options->maximum_5_Bit_Green_Channel_Deviance  >>= k_QUICKDRAW_16_BIT_SCALE;
	options->maximum_5_Bit_Total_Channel_Deviance >>= k_QUICKDRAW_16_BIT_SCALE;
	options->maximum_5_Bit_Red_Channel_Lenient_Deviance  >>= k_QUICKDRAW_16_BIT_SCALE;
	options->maximum_5_Bit_Blue_Channel_Lenient_Deviance  >>= k_QUICKDRAW_16_BIT_SCALE;
	options->maximum_5_Bit_Green_Channel_Lenient_Deviance  >>= k_QUICKDRAW_16_BIT_SCALE;
	options->maximum_5_Bit_Total_Channel_Lenient_Deviance  >>= k_QUICKDRAW_16_BIT_SCALE;
	options->maximum_5_Bit_Red_Drift_Deviance  >>= k_QUICKDRAW_16_BIT_SCALE;
	options->maximum_5_Bit_Blue_Drift_Deviance  >>= k_QUICKDRAW_16_BIT_SCALE;
	options->maximum_5_Bit_Green_Drift_Deviance >>= k_QUICKDRAW_16_BIT_SCALE;
	options->maximum_5_Bit_Total_Drift_Deviance  >>= k_QUICKDRAW_16_BIT_SCALE;
	options->maximum_Brightness_Deviance_5  >>= k_QUICKDRAW_16_BIT_SCALE;
	options->maximum_Darkness_Deviance_5  >>= k_QUICKDRAW_16_BIT_SCALE;
	
	options->maximum_5_Bit_Red_Channel_Absolute_High_Limit  >>= k_QUICKDRAW_16_BIT_SCALE;
	options->maximum_5_Bit_Blue_Channel_Absolute_High_Limit  >>= k_QUICKDRAW_16_BIT_SCALE;
	options->maximum_5_Bit_Green_Channel_Absolute_High_Limit  >>= k_QUICKDRAW_16_BIT_SCALE;
	options->maximum_5_Bit_Total_Channel_Absolute_High_Limit  >>= k_QUICKDRAW_16_BIT_SCALE;
	options->maximum_5_Bit_Red_Channel_Absolute_Low_Limit  >>= k_QUICKDRAW_16_BIT_SCALE;
	options->maximum_5_Bit_Blue_Channel_Absolute_Low_Limit  >>= k_QUICKDRAW_16_BIT_SCALE;
	options->maximum_5_Bit_Green_Channel_Absolute_Low_Limit  >>= k_QUICKDRAW_16_BIT_SCALE;
	options->maximum_5_Bit_Total_Channel_Absolute_Low_Limit  >>= k_QUICKDRAW_16_BIT_SCALE;
	
	options->maximum_8_Bit_Red_Channel_Deviance  >>= k_QUICKDRAW_16_BIT_SCALE;
	options->maximum_8_Bit_Blue_Channel_Deviance  >>= k_QUICKDRAW_16_BIT_SCALE;
	options->maximum_8_Bit_Green_Channel_Deviance  >>= k_QUICKDRAW_16_BIT_SCALE;
	options->maximum_8_Bit_Total_Channel_Deviance  >>= k_QUICKDRAW_16_BIT_SCALE;
	options->maximum_8_Bit_Red_Channel_Lenient_Deviance  >>= k_QUICKDRAW_16_BIT_SCALE;
	options->maximum_8_Bit_Blue_Channel_Lenient_Deviance  >>= k_QUICKDRAW_16_BIT_SCALE;
	options->maximum_8_Bit_Green_Channel_Lenient_Deviance  >>= k_QUICKDRAW_16_BIT_SCALE;
	options->maximum_8_Bit_Total_Channel_Lenient_Deviance  >>= k_QUICKDRAW_16_BIT_SCALE;
	options->maximum_8_Bit_Red_Drift_Deviance  >>= k_QUICKDRAW_16_BIT_SCALE;
	options->maximum_8_Bit_Blue_Drift_Deviance >>= k_QUICKDRAW_16_BIT_SCALE;
	options->maximum_8_Bit_Green_Drift_Deviance  >>= k_QUICKDRAW_16_BIT_SCALE;
	options->maximum_8_Bit_Total_Drift_Deviance  >>= k_QUICKDRAW_16_BIT_SCALE;
	options->maximum_Brightness_Deviance_8  >>= k_QUICKDRAW_16_BIT_SCALE;
	options->maximum_Darkness_Deviance_8 >>= k_QUICKDRAW_16_BIT_SCALE;
	
	options->maximum_8_Bit_Red_Channel_Absolute_High_Limit >>= k_QUICKDRAW_16_BIT_SCALE;
	options->maximum_8_Bit_Blue_Channel_Absolute_High_Limit >>= k_QUICKDRAW_16_BIT_SCALE;
	options->maximum_8_Bit_Green_Channel_Absolute_High_Limit >>= k_QUICKDRAW_16_BIT_SCALE;
	options->maximum_8_Bit_Total_Channel_Absolute_High_Limit >>= k_QUICKDRAW_16_BIT_SCALE;
	options->maximum_8_Bit_Red_Channel_Absolute_Low_Limit >>= k_QUICKDRAW_16_BIT_SCALE;
	options->maximum_8_Bit_Blue_Channel_Absolute_Low_Limit >>= k_QUICKDRAW_16_BIT_SCALE;
	options->maximum_8_Bit_Green_Channel_Absolute_Low_Limit >>= k_QUICKDRAW_16_BIT_SCALE;
	options->maximum_8_Bit_Total_Channel_Absolute_Low_Limit >>= k_QUICKDRAW_16_BIT_SCALE;

	return(SAFE_TRUE);
	
} /* Safe_Boolean Scale_Input_Defaults_For_Background_Removal ... */
/*=**************************************************************************=*/


/*=**************************************************************************=*/
Safe_Boolean Scale_Output_Defaults_For_Background_Removal(Background_Options_Ptr	options)
{	
	options->bright_Enough_To_Trust_For_Brighten <<= k_QUICKDRAW_16_BIT_SCALE;
	options->allow_Color_Trapping_Dark_Background_Cuttoff <<= k_QUICKDRAW_16_BIT_SCALE;
	options->maximum_Trap_Dark_Background <<= k_QUICKDRAW_16_BIT_SCALE;
	options->minimum_Trap_Dark_Background <<= k_QUICKDRAW_16_BIT_SCALE;
	options->black_Channel_Gun_Limit <<= k_QUICKDRAW_16_BIT_SCALE;
	options->minimum_Intensity_To_Consider_Leniance <<= k_QUICKDRAW_16_BIT_SCALE;
	options->insignificant_Deviance <<= k_QUICKDRAW_16_BIT_SCALE;
	options->maximum_5_Bit_Red_Channel_Deviance  <<= k_QUICKDRAW_16_BIT_SCALE;
	options->maximum_5_Bit_Blue_Channel_Deviance  <<= k_QUICKDRAW_16_BIT_SCALE;
	options->maximum_5_Bit_Green_Channel_Deviance  <<= k_QUICKDRAW_16_BIT_SCALE;
	options->maximum_5_Bit_Total_Channel_Deviance <<= k_QUICKDRAW_16_BIT_SCALE;
	options->maximum_5_Bit_Red_Channel_Lenient_Deviance  <<= k_QUICKDRAW_16_BIT_SCALE;
	options->maximum_5_Bit_Blue_Channel_Lenient_Deviance  <<= k_QUICKDRAW_16_BIT_SCALE;
	options->maximum_5_Bit_Green_Channel_Lenient_Deviance  <<= k_QUICKDRAW_16_BIT_SCALE;
	options->maximum_5_Bit_Total_Channel_Lenient_Deviance  <<= k_QUICKDRAW_16_BIT_SCALE;
	options->maximum_5_Bit_Red_Drift_Deviance  <<= k_QUICKDRAW_16_BIT_SCALE;
	options->maximum_5_Bit_Blue_Drift_Deviance  <<= k_QUICKDRAW_16_BIT_SCALE;
	options->maximum_5_Bit_Green_Drift_Deviance <<= k_QUICKDRAW_16_BIT_SCALE;
	options->maximum_5_Bit_Total_Drift_Deviance  <<= k_QUICKDRAW_16_BIT_SCALE;
	options->maximum_Brightness_Deviance_5  <<= k_QUICKDRAW_16_BIT_SCALE;
	options->maximum_Darkness_Deviance_5  <<= k_QUICKDRAW_16_BIT_SCALE;
	
	options->maximum_5_Bit_Red_Channel_Absolute_High_Limit  <<= k_QUICKDRAW_16_BIT_SCALE;
	options->maximum_5_Bit_Blue_Channel_Absolute_High_Limit  <<= k_QUICKDRAW_16_BIT_SCALE;
	options->maximum_5_Bit_Green_Channel_Absolute_High_Limit  <<= k_QUICKDRAW_16_BIT_SCALE;
	options->maximum_5_Bit_Total_Channel_Absolute_High_Limit  <<= k_QUICKDRAW_16_BIT_SCALE;
	options->maximum_5_Bit_Red_Channel_Absolute_Low_Limit  <<= k_QUICKDRAW_16_BIT_SCALE;
	options->maximum_5_Bit_Blue_Channel_Absolute_Low_Limit  <<= k_QUICKDRAW_16_BIT_SCALE;
	options->maximum_5_Bit_Green_Channel_Absolute_Low_Limit  <<= k_QUICKDRAW_16_BIT_SCALE;
	options->maximum_5_Bit_Total_Channel_Absolute_Low_Limit  <<= k_QUICKDRAW_16_BIT_SCALE;
	
	options->maximum_8_Bit_Red_Channel_Deviance  <<= k_QUICKDRAW_16_BIT_SCALE;
	options->maximum_8_Bit_Blue_Channel_Deviance  <<= k_QUICKDRAW_16_BIT_SCALE;
	options->maximum_8_Bit_Green_Channel_Deviance  <<= k_QUICKDRAW_16_BIT_SCALE;
	options->maximum_8_Bit_Total_Channel_Deviance  <<= k_QUICKDRAW_16_BIT_SCALE;
	options->maximum_8_Bit_Red_Channel_Lenient_Deviance  <<= k_QUICKDRAW_16_BIT_SCALE;
	options->maximum_8_Bit_Blue_Channel_Lenient_Deviance  <<= k_QUICKDRAW_16_BIT_SCALE;
	options->maximum_8_Bit_Green_Channel_Lenient_Deviance  <<= k_QUICKDRAW_16_BIT_SCALE;
	options->maximum_8_Bit_Total_Channel_Lenient_Deviance  <<= k_QUICKDRAW_16_BIT_SCALE;
	options->maximum_8_Bit_Red_Drift_Deviance  <<= k_QUICKDRAW_16_BIT_SCALE;
	options->maximum_8_Bit_Blue_Drift_Deviance <<= k_QUICKDRAW_16_BIT_SCALE;
	options->maximum_8_Bit_Green_Drift_Deviance  <<= k_QUICKDRAW_16_BIT_SCALE;
	options->maximum_8_Bit_Total_Drift_Deviance  <<= k_QUICKDRAW_16_BIT_SCALE;
	options->maximum_Brightness_Deviance_8  <<= k_QUICKDRAW_16_BIT_SCALE;
	options->maximum_Darkness_Deviance_8 <<= k_QUICKDRAW_16_BIT_SCALE;
	
	options->maximum_8_Bit_Red_Channel_Absolute_High_Limit <<= k_QUICKDRAW_16_BIT_SCALE;
	options->maximum_8_Bit_Blue_Channel_Absolute_High_Limit <<= k_QUICKDRAW_16_BIT_SCALE;
	options->maximum_8_Bit_Green_Channel_Absolute_High_Limit <<= k_QUICKDRAW_16_BIT_SCALE;
	options->maximum_8_Bit_Total_Channel_Absolute_High_Limit <<= k_QUICKDRAW_16_BIT_SCALE;
	options->maximum_8_Bit_Red_Channel_Absolute_Low_Limit <<= k_QUICKDRAW_16_BIT_SCALE;
	options->maximum_8_Bit_Blue_Channel_Absolute_Low_Limit <<= k_QUICKDRAW_16_BIT_SCALE;
	options->maximum_8_Bit_Green_Channel_Absolute_Low_Limit <<= k_QUICKDRAW_16_BIT_SCALE;
	options->maximum_8_Bit_Total_Channel_Absolute_Low_Limit <<= k_QUICKDRAW_16_BIT_SCALE;

	return(SAFE_TRUE);
	
} /* Safe_Boolean Scale_Output_Defaults_For_Background_Removal ... */
/*=**************************************************************************=*/


/*=**************************************************************************=*/
Safe_Boolean Initialize_Defaults_For_Background_Removal(Background_Options_Ptr	options)
{

#if (k_IGNORE_EXAMPLES == 0)
	options->result_Check_Box = FALSE;
	
	options->result_Radio_String_Choice = 0;
	options->result_Radio_String_Choice_Text[0] = 0;
	Copy_Pascal_String("\pDefault", options->result_Radio_String_Choice_Text);
	
	options->result_Radio_Value_Choice = 0;
	
	options->result_Radio_Value_Choice_Value = 180;
#endif /* #if (k_IGNORE_EXAMPLES == 0) */


	if (k_ALLOW_HORIZONTAL_DRIFT_TRACKING != 0) {
		options->allow_Horizontal_Drift_Tracking = TRUE;
	}
	else {
		options->allow_Horizontal_Drift_Tracking = FALSE;
	}
	
	if (k_ALLOW_VERTICAL_DRIFT_TRACKING != 0) {
		options->allow_Vertical_Drift_Tracking = TRUE;
	}
	else {
		options->allow_Vertical_Drift_Tracking = FALSE;
	}
	
	options->bright_Enough_To_Trust_For_Brighten = k_BRIGHT_ENOUGH_TO_TRUST_FOR_BRIGHTEN;
	options->allow_Color_Trapping_Dark_Background_Cuttoff = k_ALLOW_COLOR_TRAPPING_DARK_BACKGROUND_CUTOFF;
	options->maximum_Trap_Dark_Background = k_MAXIMUM_TRAP_DARK_BACKGROUND;
	options->minimum_Trap_Dark_Background = k_MINIMUM_TRAP_DARK_BACKGROUND;
	options->black_Channel_Gun_Limit = k_BLACK_CHANNEL_GUN_LIMIT;
	options->minimum_Intensity_To_Consider_Leniance = k_MINIMUM_INTENSITY_TO_CONSIDER_LENIANCE;
	options->insignificant_Deviance = k_INSIGNIFICANT_DEVIANCE;
	options->maximum_5_Bit_Red_Channel_Deviance = k_MAXIMUM_5_BIT_RED_CHANNEL_DEVIANCE;
	options->maximum_5_Bit_Blue_Channel_Deviance = k_MAXIMUM_5_BIT_BLUE_CHANNEL_DEVIANCE;
	options->maximum_5_Bit_Green_Channel_Deviance = k_MAXIMUM_5_BIT_GREEN_CHANNEL_DEVIANCE;
	options->maximum_5_Bit_Total_Channel_Deviance = k_MAXIMUM_5_BIT_TOTAL_CHANNEL_DEVIANCE;
	options->maximum_5_Bit_Red_Channel_Lenient_Deviance = k_MAXIMUM_5_BIT_RED_CHANNEL_LENIENT_DEVIANCE;
	options->maximum_5_Bit_Blue_Channel_Lenient_Deviance = k_MAXIMUM_5_BIT_BLUE_CHANNEL_LENIENT_DEVIANCE;
	options->maximum_5_Bit_Green_Channel_Lenient_Deviance = k_MAXIMUM_5_BIT_GREEN_CHANNEL_LENIENT_DEVIANCE;
	options->maximum_5_Bit_Total_Channel_Lenient_Deviance = k_MAXIMUM_5_BIT_TOTAL_CHANNEL_LENIENT_DEVIANCE;
	options->maximum_5_Bit_Red_Drift_Deviance = k_MAXIMUM_5_BIT_RED_DRIFT_DEVIANCE;
	options->maximum_5_Bit_Blue_Drift_Deviance = k_MAXIMUM_5_BIT_BLUE_DRIFT_DEVIANCE;
	options->maximum_5_Bit_Green_Drift_Deviance = k_MAXIMUM_5_BIT_GREEN_DRIFT_DEVIANCE;
	options->maximum_5_Bit_Total_Drift_Deviance = k_MAXIMUM_5_BIT_TOTAL_DRIFT_DEVIANCE;
	options->maximum_Brightness_Deviance_5 = k_MAX_BRIGHTNESS_DEVIANCE_5;
	options->maximum_Darkness_Deviance_5 = k_MAX_DARKNESS_DEVIANCE_5;
	
	options->maximum_5_Bit_Red_Channel_Absolute_High_Limit = k_MAXIMUM_5_BIT_RED_CHANNEL_ABSOLUTE_HIGH_LIMIT;
	options->maximum_5_Bit_Blue_Channel_Absolute_High_Limit = k_MAXIMUM_5_BIT_BLUE_CHANNEL_ABSOLUTE_HIGH_LIMIT;
	options->maximum_5_Bit_Green_Channel_Absolute_High_Limit = k_MAXIMUM_5_BIT_GREEN_CHANNEL_ABSOLUTE_HIGH_LIMIT;
	options->maximum_5_Bit_Total_Channel_Absolute_High_Limit = k_MAXIMUM_5_BIT_TOTAL_CHANNEL_ABSOLUTE_HIGH_LIMIT;
	options->maximum_5_Bit_Red_Channel_Absolute_Low_Limit = k_MAXIMUM_5_BIT_RED_CHANNEL_ABSOLUTE_LOW_LIMIT;
	options->maximum_5_Bit_Blue_Channel_Absolute_Low_Limit = k_MAXIMUM_5_BIT_BLUE_CHANNEL_ABSOLUTE_LOW_LIMIT;
	options->maximum_5_Bit_Green_Channel_Absolute_Low_Limit = k_MAXIMUM_5_BIT_GREEN_CHANNEL_ABSOLUTE_LOW_LIMIT;
	options->maximum_5_Bit_Total_Channel_Absolute_Low_Limit = k_MAXIMUM_5_BIT_TOTAL_CHANNEL_ABSOLUTE_LOW_LIMIT;
	
	options->maximum_8_Bit_Red_Channel_Deviance = k_MAXIMUM_8_BIT_RED_CHANNEL_DEVIANCE;
	options->maximum_8_Bit_Blue_Channel_Deviance = k_MAXIMUM_8_BIT_BLUE_CHANNEL_DEVIANCE;
	options->maximum_8_Bit_Green_Channel_Deviance = k_MAXIMUM_8_BIT_GREEN_CHANNEL_DEVIANCE;
	options->maximum_8_Bit_Total_Channel_Deviance = k_MAXIMUM_8_BIT_TOTAL_CHANNEL_DEVIANCE;
	options->maximum_8_Bit_Red_Channel_Lenient_Deviance = k_MAXIMUM_8_BIT_RED_CHANNEL_LENIENT_DEVIANCE;
	options->maximum_8_Bit_Blue_Channel_Lenient_Deviance = k_MAXIMUM_8_BIT_BLUE_CHANNEL_LENIENT_DEVIANCE;
	options->maximum_8_Bit_Green_Channel_Lenient_Deviance = k_MAXIMUM_8_BIT_GREEN_CHANNEL_LENIENT_DEVIANCE;
	options->maximum_8_Bit_Total_Channel_Lenient_Deviance = k_MAXIMUM_8_BIT_TOTAL_CHANNEL_LENIENT_DEVIANCE;
	options->maximum_8_Bit_Red_Drift_Deviance = k_MAXIMUM_8_BIT_RED_DRIFT_DEVIANCE;
	options->maximum_8_Bit_Blue_Drift_Deviance = k_MAXIMUM_8_BIT_BLUE_DRIFT_DEVIANCE;
	options->maximum_8_Bit_Green_Drift_Deviance = k_MAXIMUM_8_BIT_GREEN_DRIFT_DEVIANCE;
	options->maximum_8_Bit_Total_Drift_Deviance = k_MAXIMUM_8_BIT_TOTAL_DRIFT_DEVIANCE;
	options->maximum_Brightness_Deviance_8 = k_MAX_BRIGHTNESS_DEVIANCE_8;
	options->maximum_Darkness_Deviance_8 = k_MAX_DARKNESS_DEVIANCE_8;
	
	options->maximum_8_Bit_Red_Channel_Absolute_High_Limit = k_MAXIMUM_8_BIT_RED_CHANNEL_ABSOLUTE_HIGH_LIMIT;
	options->maximum_8_Bit_Blue_Channel_Absolute_High_Limit = k_MAXIMUM_8_BIT_BLUE_CHANNEL_ABSOLUTE_HIGH_LIMIT;
	options->maximum_8_Bit_Green_Channel_Absolute_High_Limit = k_MAXIMUM_8_BIT_GREEN_CHANNEL_ABSOLUTE_HIGH_LIMIT;
	options->maximum_8_Bit_Total_Channel_Absolute_High_Limit = k_MAXIMUM_8_BIT_TOTAL_CHANNEL_ABSOLUTE_HIGH_LIMIT;
	options->maximum_8_Bit_Red_Channel_Absolute_Low_Limit = k_MAXIMUM_8_BIT_RED_CHANNEL_ABSOLUTE_LOW_LIMIT;
	options->maximum_8_Bit_Blue_Channel_Absolute_Low_Limit = k_MAXIMUM_8_BIT_BLUE_CHANNEL_ABSOLUTE_LOW_LIMIT;
	options->maximum_8_Bit_Green_Channel_Absolute_Low_Limit = k_MAXIMUM_8_BIT_GREEN_CHANNEL_ABSOLUTE_LOW_LIMIT;
	options->maximum_8_Bit_Total_Channel_Absolute_Low_Limit = k_MAXIMUM_8_BIT_TOTAL_CHANNEL_ABSOLUTE_LOW_LIMIT;

	return(SAFE_TRUE);
	
} /* Safe_Boolean Initialize_Defaults_For_Background_Removal... */
/*=**************************************************************************=*/


/*=**************************************************************************=*/
static Safe_Boolean Validate_And_Copy_Contents(Background_Options_Item_Variables_Record *items
													, Background_Options_Ptr options
													, CARDINAL_16 color_Resolution)
{
	Safe_Boolean 	function_Result;
	CARDINAL_32	current_Value;
	CARDINAL_16	string_Length;
	CARDINAL_8		*string_Ptr;
	Boolean 			good_Input;
/* END VAR */


	good_Input = TRUE;
	
	/*  */
	/* Convert all numerical strings first */
	/* Compare numerical values for acceptability */
	/*  */
	
#if (k_IGNORE_EXAMPLES == 0)
	string_Length = Strip_Outside_Controls(items->example_Radio_Value_Choice_Value.edit_String)[0];
	if ((string_Length < 1) || (string_Length > 3)) {
		good_Input = FALSE;
	}
	
	
	if (Validate_Ordinal_String_And_Copy(items->example_Radio_Value_Choice_Value.edit_String
								, 360
								, &(options->result_Radio_Value_Choice_Value)) != TRUE) {
		good_Input = FALSE;
	}
	
	
	string_Length = Strip_Outside_Controls(items->example_Radio_String_Choice_Text.edit_String)[0];
	if ((string_Length < 1) || (string_Length > 31)) {
		good_Input = FALSE;
	}
#endif /* #if (k_IGNORE_EXAMPLES == 0) */

	
	/*  */
	/* Assign numerical values */
	/*  */
	if (Validate_Ordinal_String_And_Copy(items->bright_Enough_To_Trust_For_Brighten_Item.edit_String
								, 255
								, (CARDINAL_32 *)&(options->bright_Enough_To_Trust_For_Brighten)) != TRUE) {
		good_Input = FALSE;
	}

	if (Validate_Ordinal_String_And_Copy(items->allow_Color_Trapping_Dark_Background_Cuttoff_Item.edit_String
								, 255
								, (CARDINAL_32 *)&(options->allow_Color_Trapping_Dark_Background_Cuttoff)) != TRUE) {
		good_Input = FALSE;
	}

	if (Validate_Ordinal_String_And_Copy(items->maximum_Trap_Dark_Background_Item.edit_String
								, 255
								, (CARDINAL_32 *)&(options->maximum_Trap_Dark_Background)) != TRUE) {
		good_Input = FALSE;
	}

	if (Validate_Ordinal_String_And_Copy(items->minimum_Trap_Dark_Background_Item.edit_String
								, 255
								, (CARDINAL_32 *)&(options->minimum_Trap_Dark_Background)) != TRUE) {
		good_Input = FALSE;
	}

	if (Validate_Ordinal_String_And_Copy(items->black_Channel_Gun_Limit_Item_Item.edit_String
								, 255
								, (CARDINAL_32 *)&(options->black_Channel_Gun_Limit)) != TRUE) {
		good_Input = FALSE;
	}

	if (Validate_Ordinal_String_And_Copy(items->minimum_Intensity_To_Consider_Leniance_Item.edit_String
								, 255
								, (CARDINAL_32 *)&(options->minimum_Intensity_To_Consider_Leniance)) != TRUE) {
		good_Input = FALSE;
	}

	if (Validate_Ordinal_String_And_Copy(items->insignificant_Deviance_Item.edit_String
								, 255
								, (CARDINAL_32 *)&(options->insignificant_Deviance)) != TRUE) {
		good_Input = FALSE;
	}


	
	if (color_Resolution < 8) {

		if (Validate_Ordinal_String_And_Copy(items->maximum_Red_Channel_Deviance_Item.edit_String
									, 255
									, (CARDINAL_32 *)&(options->maximum_5_Bit_Red_Channel_Deviance)) != TRUE) {
			good_Input = FALSE;
		}

		if (Validate_Ordinal_String_And_Copy(items->maximum_Blue_Channel_Deviance_Item.edit_String
									, 255
									, (CARDINAL_32 *)&(options->maximum_5_Bit_Blue_Channel_Deviance)) != TRUE) {
			good_Input = FALSE;
		}

		if (Validate_Ordinal_String_And_Copy(items->maximum_Green_Channel_Deviance_Item.edit_String
									, 255
									, (CARDINAL_32 *)&(options->maximum_5_Bit_Green_Channel_Deviance)) != TRUE) {
			good_Input = FALSE;
		}

		if (Validate_Ordinal_String_And_Copy(items->maximum_Total_Channel_Deviance_Item.edit_String
									, 255
									, (CARDINAL_32 *)&(options->maximum_5_Bit_Total_Channel_Deviance)) != TRUE) {
			good_Input = FALSE;
		}

		if (Validate_Ordinal_String_And_Copy(items->maximum_Red_Channel_Lenient_Deviance_Item.edit_String
									, 255
									, (CARDINAL_32 *)&(options->maximum_5_Bit_Red_Channel_Lenient_Deviance)) != TRUE) {
			good_Input = FALSE;
		}

		if (Validate_Ordinal_String_And_Copy(items->maximum_Blue_Channel_Lenient_Deviance_Item.edit_String
									, 255
									, (CARDINAL_32 *)&(options->maximum_5_Bit_Blue_Channel_Lenient_Deviance)) != TRUE) {
			good_Input = FALSE;
		}

		if (Validate_Ordinal_String_And_Copy(items->maximum_Green_Channel_Lenient_Deviance_Item.edit_String
									, 255
									, (CARDINAL_32 *)&(options->maximum_5_Bit_Green_Channel_Lenient_Deviance)) != TRUE) {
			good_Input = FALSE;
		}

		if (Validate_Ordinal_String_And_Copy(items->maximum_Total_Channel_Lenient_Deviance_Item.edit_String
									, 255
									, (CARDINAL_32 *)&(options->maximum_5_Bit_Total_Channel_Lenient_Deviance)) != TRUE) {
			good_Input = FALSE;
		}

		if (Validate_Ordinal_String_And_Copy(items->maximum_Red_Drift_Deviance_Item.edit_String
									, 255
									, (CARDINAL_32 *)&(options->maximum_5_Bit_Red_Drift_Deviance)) != TRUE) {
			good_Input = FALSE;
		}

		if (Validate_Ordinal_String_And_Copy(items->maximum_Blue_Drift_Deviance_Item.edit_String
									, 255
									, (CARDINAL_32 *)&(options->maximum_5_Bit_Blue_Drift_Deviance)) != TRUE) {
			good_Input = FALSE;
		}

		if (Validate_Ordinal_String_And_Copy(items->maximum_Green_Drift_Deviance_Item.edit_String
									, 255
									, (CARDINAL_32 *)&(options->maximum_5_Bit_Green_Drift_Deviance)) != TRUE) {
			good_Input = FALSE;
		}

		if (Validate_Ordinal_String_And_Copy(items->maximum_Total_Drift_Deviance_Item.edit_String
									, 255
									, (CARDINAL_32 *)&(options->maximum_5_Bit_Total_Drift_Deviance)) != TRUE) {
			good_Input = FALSE;
		}

		if (Validate_Ordinal_String_And_Copy(items->maximum_Brightness_Deviance_Item.edit_String
									, 767
									, (CARDINAL_32 *)&(options->maximum_Brightness_Deviance_5)) != TRUE) {
			good_Input = FALSE;
		}

		if (Validate_Ordinal_String_And_Copy(items->maximum_Darkness_Deviance_Item.edit_String
									, 767
									, (CARDINAL_32 *)&(options->maximum_Darkness_Deviance_5)) != TRUE) {
			good_Input = FALSE;
		}

		if (Validate_Ordinal_String_And_Copy(items->maximum_Red_Channel_Absolute_High_Limit_Item.edit_String
									, 255
									, (CARDINAL_32 *)&(options->maximum_5_Bit_Red_Channel_Absolute_High_Limit)) != TRUE) {
			good_Input = FALSE;
		}

		if (Validate_Ordinal_String_And_Copy(items->maximum_Blue_Channel_Absolute_High_Limit_Item.edit_String
									, 255
									, (CARDINAL_32 *)&(options->maximum_5_Bit_Blue_Channel_Absolute_High_Limit)) != TRUE) {
			good_Input = FALSE;
		}

		if (Validate_Ordinal_String_And_Copy(items->maximum_Green_Channel_Absolute_High_Limit_Item.edit_String
									, 255
									, (CARDINAL_32 *)&(options->maximum_5_Bit_Green_Channel_Absolute_High_Limit)) != TRUE) {
			good_Input = FALSE;
		}

		if (Validate_Ordinal_String_And_Copy(items->maximum_Total_Channel_Absolute_High_Limit_Item.edit_String
									, 255
									, (CARDINAL_32 *)&(options->maximum_5_Bit_Total_Channel_Absolute_High_Limit)) != TRUE) {
			good_Input = FALSE;
		}

		if (Validate_Ordinal_String_And_Copy(items->maximum_Red_Channel_Absolute_Low_Limit_Item.edit_String
									, 255
									, (CARDINAL_32 *)&(options->maximum_5_Bit_Red_Channel_Absolute_Low_Limit)) != TRUE) {
			good_Input = FALSE;
		}

		if (Validate_Ordinal_String_And_Copy(items->maximum_Blue_Channel_Absolute_Low_Limit_Item.edit_String
									, 255
									, (CARDINAL_32 *)&(options->maximum_5_Bit_Blue_Channel_Absolute_Low_Limit)) != TRUE) {
			good_Input = FALSE;
		}

		if (Validate_Ordinal_String_And_Copy(items->maximum_Green_Channel_Absolute_Low_Limit_Item.edit_String
									, 255
									, (CARDINAL_32 *)&(options->maximum_5_Bit_Green_Channel_Absolute_Low_Limit)) != TRUE) {
			good_Input = FALSE;
		}

		if (Validate_Ordinal_String_And_Copy(items->maximum_Total_Channel_Absolute_Low_Limit_Item.edit_String
									, 255
									, (CARDINAL_32 *)&(options->maximum_5_Bit_Total_Channel_Absolute_Low_Limit)) != TRUE) {
			good_Input = FALSE;
		}
	}
	else {

		if (Validate_Ordinal_String_And_Copy(items->maximum_Red_Channel_Deviance_Item.edit_String
									, 255
									, (CARDINAL_32 *)&(options->maximum_8_Bit_Red_Channel_Deviance)) != TRUE) {
			good_Input = FALSE;
		}

		if (Validate_Ordinal_String_And_Copy(items->maximum_Blue_Channel_Deviance_Item.edit_String
									, 255
									, (CARDINAL_32 *)&(options->maximum_8_Bit_Blue_Channel_Deviance)) != TRUE) {
			good_Input = FALSE;
		}

		if (Validate_Ordinal_String_And_Copy(items->maximum_Green_Channel_Deviance_Item.edit_String
									, 255
									, (CARDINAL_32 *)&(options->maximum_8_Bit_Green_Channel_Deviance)) != TRUE) {
			good_Input = FALSE;
		}

		if (Validate_Ordinal_String_And_Copy(items->maximum_Total_Channel_Deviance_Item.edit_String
									, 255
									, (CARDINAL_32 *)&(options->maximum_8_Bit_Total_Channel_Deviance)) != TRUE) {
			good_Input = FALSE;
		}

		if (Validate_Ordinal_String_And_Copy(items->maximum_Red_Channel_Lenient_Deviance_Item.edit_String
									, 255
									, (CARDINAL_32 *)&(options->maximum_8_Bit_Red_Channel_Lenient_Deviance)) != TRUE) {
			good_Input = FALSE;
		}

		if (Validate_Ordinal_String_And_Copy(items->maximum_Blue_Channel_Lenient_Deviance_Item.edit_String
									, 255
									, (CARDINAL_32 *)&(options->maximum_8_Bit_Blue_Channel_Lenient_Deviance)) != TRUE) {
			good_Input = FALSE;
		}

		if (Validate_Ordinal_String_And_Copy(items->maximum_Green_Channel_Lenient_Deviance_Item.edit_String
									, 255
									, (CARDINAL_32 *)&(options->maximum_8_Bit_Green_Channel_Lenient_Deviance)) != TRUE) {
			good_Input = FALSE;
		}

		if (Validate_Ordinal_String_And_Copy(items->maximum_Total_Channel_Lenient_Deviance_Item.edit_String
									, 255
									, (CARDINAL_32 *)&(options->maximum_8_Bit_Total_Channel_Lenient_Deviance)) != TRUE) {
			good_Input = FALSE;
		}

		if (Validate_Ordinal_String_And_Copy(items->maximum_Red_Drift_Deviance_Item.edit_String
									, 255
									, (CARDINAL_32 *)&(options->maximum_8_Bit_Red_Drift_Deviance)) != TRUE) {
			good_Input = FALSE;
		}

		if (Validate_Ordinal_String_And_Copy(items->maximum_Blue_Drift_Deviance_Item.edit_String
									, 255
									, (CARDINAL_32 *)&(options->maximum_8_Bit_Blue_Drift_Deviance)) != TRUE) {
			good_Input = FALSE;
		}

		if (Validate_Ordinal_String_And_Copy(items->maximum_Green_Drift_Deviance_Item.edit_String
									, 255
									, (CARDINAL_32 *)&(options->maximum_8_Bit_Green_Drift_Deviance)) != TRUE) {
			good_Input = FALSE;
		}

		if (Validate_Ordinal_String_And_Copy(items->maximum_Total_Drift_Deviance_Item.edit_String
									, 255
									, (CARDINAL_32 *)&(options->maximum_8_Bit_Total_Drift_Deviance)) != TRUE) {
			good_Input = FALSE;
		}

		if (Validate_Ordinal_String_And_Copy(items->maximum_Brightness_Deviance_Item.edit_String
									, 767
									, (CARDINAL_32 *)&(options->maximum_Brightness_Deviance_8)) != TRUE) {
			good_Input = FALSE;
		}

		if (Validate_Ordinal_String_And_Copy(items->maximum_Darkness_Deviance_Item.edit_String
									, 767
									, (CARDINAL_32 *)&(options->maximum_Darkness_Deviance_8)) != TRUE) {
			good_Input = FALSE;
		}

		if (Validate_Ordinal_String_And_Copy(items->maximum_Red_Channel_Absolute_High_Limit_Item.edit_String
									, 255
									, (CARDINAL_32 *)&(options->maximum_8_Bit_Red_Channel_Absolute_High_Limit)) != TRUE) {
			good_Input = FALSE;
		}

		if (Validate_Ordinal_String_And_Copy(items->maximum_Blue_Channel_Absolute_High_Limit_Item.edit_String
									, 255
									, (CARDINAL_32 *)&(options->maximum_8_Bit_Blue_Channel_Absolute_High_Limit)) != TRUE) {
			good_Input = FALSE;
		}

		if (Validate_Ordinal_String_And_Copy(items->maximum_Green_Channel_Absolute_High_Limit_Item.edit_String
									, 255
									, (CARDINAL_32 *)&(options->maximum_8_Bit_Green_Channel_Absolute_High_Limit)) != TRUE) {
			good_Input = FALSE;
		}

		if (Validate_Ordinal_String_And_Copy(items->maximum_Total_Channel_Absolute_High_Limit_Item.edit_String
									, 255
									, (CARDINAL_32 *)&(options->maximum_8_Bit_Total_Channel_Absolute_High_Limit)) != TRUE) {
			good_Input = FALSE;
		}

		if (Validate_Ordinal_String_And_Copy(items->maximum_Red_Channel_Absolute_Low_Limit_Item.edit_String
									, 255
									, (CARDINAL_32 *)&(options->maximum_8_Bit_Red_Channel_Absolute_Low_Limit)) != TRUE) {
			good_Input = FALSE;
		}

		if (Validate_Ordinal_String_And_Copy(items->maximum_Blue_Channel_Absolute_Low_Limit_Item.edit_String
									, 255
									, (CARDINAL_32 *)&(options->maximum_8_Bit_Blue_Channel_Absolute_Low_Limit)) != TRUE) {
			good_Input = FALSE;
		}

		if (Validate_Ordinal_String_And_Copy(items->maximum_Green_Channel_Absolute_Low_Limit_Item.edit_String
									, 255
									, (CARDINAL_32 *)&(options->maximum_8_Bit_Green_Channel_Absolute_Low_Limit)) != TRUE) {
			good_Input = FALSE;
		}

		if (Validate_Ordinal_String_And_Copy(items->maximum_Total_Channel_Absolute_Low_Limit_Item.edit_String
									, 255
									, (CARDINAL_32 *)&(options->maximum_8_Bit_Total_Channel_Absolute_Low_Limit)) != TRUE) {
			good_Input = FALSE;
		}
	}


	/*  */
	/* Copy strings compare lengths first. */
	/*  */

	/*  */
	/* Copy strings, truncate if needed */
	/*  */

	/*  */
	/* Copy over checkbox booleans */
	/*  */
	
#if (k_IGNORE_EXAMPLES == 0)
	options->result_Check_Box = items->example_Check_Box.checked;

	options->result_Radio_String_Choice = 0;
	options->result_Radio_String_Choice_Text[0] = 0;
	Copy_Pascal_String("\pDefault", options->result_Radio_String_Choice_Text);

	if (items->example_Radio_String_Choice.checked != FALSE) {
		options->result_Radio_String_Choice = 0;
		Copy_Pascal_String(items->example_Radio_String_Choice_Text.edit_String, options->result_Radio_String_Choice_Text);
	}
	else {
		good_Input = FALSE;
	}

#endif /* #if (k_IGNORE_EXAMPLES == 0) */

	if (items->allow_Horizontal_Drift_Tracking_Item.checked == TRUE) {
		options->allow_Horizontal_Drift_Tracking = TRUE;
	}
	else {
		options->allow_Horizontal_Drift_Tracking = FALSE;
	}

	if (items->allow_Vertical_Drift_Tracking_Item.checked == TRUE) {
		options->allow_Vertical_Drift_Tracking = TRUE;
	}
	else {
		options->allow_Vertical_Drift_Tracking = FALSE;
	}
	
	if  (good_Input != TRUE) {
		goto Error_Handler;
	}	


	/*  */
	/* Convert radio button selection groups select first choice */
	/*  */			
	
#if (k_IGNORE_EXAMPLES == 0)
	if (items->example_Radio_Value_Choice.checked != FALSE) {
		options->result_Radio_Value_Choice = 0;
	}
	else {
		good_Input = FALSE;
	}

#endif /* #if (k_IGNORE_EXAMPLES == 0) */


	if  (good_Input != TRUE) {
		goto Error_Handler;
	}	


	function_Result = SAFE_TRUE;
	goto Normal_Function_Exit;
	
Error_Handler:
	function_Result = SAFE_FALSE;
			
Normal_Function_Exit:


	return(function_Result);
}
/*=**************************************************************************=*/


/*=**************************************************************************=*/
void Assign_Item_String_Value(Edit_Text_Item *item, INTEGER_32 value, CARDINAL_16 string_Width)
{
	Str32				value_String;
/* END VAR */


	item->enabled = TRUE;
	NumToString(value,(CARDINAL_8 *)value_String);
	Copy_Pascal_String(value_String, item->edit_String);
	item->edit_Field_Type = longint_Field;
	item->edit_String_Edit_Length = string_Width;
	
} /* void Assign_Item_String_Value ... */
/*=**************************************************************************=*/


/*=**************************************************************************=*/
Boolean Get_Background_Removal_Options(Background_Options_Ptr	options
											, CARDINAL_32 example_Number_Info_Value
											, CARDINAL_16 color_Resolution)
{
	Background_Options_Item_Variables_Record	items;
	
	INTEGER_8		*work_String_Ptr;
	Str255				work_String;
	INTEGER_16		error_Code;
	INTEGER_32		length_1_Entered;
	Str32				value_String;
	Boolean			valid_Input;
	Boolean			keep_Inquiring;
	Boolean			function_Result;
/* END VAR */


	function_Result = FALSE;
	
	/*  */
	/* Allows all pointers to be set to "nil" lazy style */
	/*  */
	Fill_Byte_Memory_With_Byte((Ptr)&items, sizeof(items), 0);
	
#if !defined(ORIGINAL_MAC_SYSTEM)
#error "UNDEFINED PARAMETER!"
#endif
#if (ORIGINAL_MAC_SYSTEM == 0)
Dialogs.h
XXXX 		
	items.cancel_Button_Outline.setup_Function = (ProcPtr)Setup_Outline_New;
	items.cancel_Button_Outline.update_Function = (ProcPtr)Update_Outline;
#else
	items.cancel_Button_Outline.setup_Function = (void *)Setup_Outline_New;
	items.cancel_Button_Outline.update_Function = (void *)Update_Outline;
#endif 
	
	
	items.cancel_Button_Outline.action_Function = NULL;


	if (Initialize_Defaults_For_Background_Removal(options) != SAFE_TRUE) {
		goto Error_Handler;
	}
	
	if (Scale_Input_Defaults_For_Background_Removal(options) != SAFE_TRUE) {
		goto Error_Handler;
	}
	
	
	/*  */
	/* NOTE: Initialize_Defaults_For_Background_Removal filled in defaults */
	/* you may modify here */
	/*  */

#if (k_IGNORE_EXAMPLES == 0)
	items.example_Check_Box.enabled = TRUE;

	items.example_Radio_String_Choice.enabled = TRUE;

	switch(options->result_Radio_String_Choice) {
		case 0:
			items.example_Radio_String_Choice.checked = TRUE;
			break;
						
		default:
			items.example_Radio_String_Choice.checked = TRUE;
			break;
	} /* switch(options->result_Radio_String_Choice) */

	items.example_Radio_String_Choice_Text.enabled = TRUE;

	items.example_Radio_Value_Choice.enabled = TRUE;

	items.example_Radio_Value_Choice_Value.enabled = TRUE;
	
	items.example_Check_Box.checked = FALSE;
			
	items.example_Radio_Value_Choice.checked = TRUE;
	
	Copy_Pascal_String("\p180", items.example_Radio_Value_Choice_Value.edit_String);
	

	Copy_Pascal_String("\pHappy", items.example_Radio_String_Choice_Text.edit_String);

	items.example_Radio_String_Choice_Text.edit_String_Edit_Length = 31;
	
	items.example_Radio_Value_Choice_Value.edit_String_Edit_Length = 3;
	
	items.example_Radio_String_Choice_Text.edit_Field_Type = string_Field;
	
	items.example_Radio_Value_Choice_Value.edit_Field_Type = integer_Field;
	
	
	items.example_Number_Info_String.enabled = FALSE;
	NumToString(example_Number_Info_Value,(CARDINAL_8 *)value_String);
	Copy_Pascal_String(value_String, items.example_Number_Info_String.edit_String);
	items.example_Number_Info_String.edit_String_Edit_Length = 4;
#endif /* #if (k_IGNORE_EXAMPLES == 0) */


	items.allow_Horizontal_Drift_Tracking_Item.enabled = TRUE;
	items.allow_Vertical_Drift_Tracking_Item.enabled = TRUE;
	
	if (options->allow_Horizontal_Drift_Tracking == TRUE) {
		items.allow_Horizontal_Drift_Tracking_Item.checked = TRUE;
	}
	
	if (options->allow_Vertical_Drift_Tracking == TRUE) {
		items.allow_Vertical_Drift_Tracking_Item.checked = TRUE;
	}
	
	Assign_Item_String_Value(&(items.bright_Enough_To_Trust_For_Brighten_Item), options->bright_Enough_To_Trust_For_Brighten, 3);	
	Assign_Item_String_Value(&(items.allow_Color_Trapping_Dark_Background_Cuttoff_Item), options->allow_Color_Trapping_Dark_Background_Cuttoff, 3);	
	Assign_Item_String_Value(&(items.maximum_Trap_Dark_Background_Item), options->maximum_Trap_Dark_Background, 3);	
	Assign_Item_String_Value(&(items.minimum_Trap_Dark_Background_Item), options->minimum_Trap_Dark_Background, 3);	
	Assign_Item_String_Value(&(items.black_Channel_Gun_Limit_Item_Item), options->black_Channel_Gun_Limit, 3);	
	Assign_Item_String_Value(&(items.minimum_Intensity_To_Consider_Leniance_Item), options->minimum_Intensity_To_Consider_Leniance, 3);	
	Assign_Item_String_Value(&(items.insignificant_Deviance_Item), options->insignificant_Deviance, 3);	
	
	if (color_Resolution < 8) {
		Assign_Item_String_Value(&(items.maximum_Red_Channel_Deviance_Item), options->maximum_5_Bit_Red_Channel_Deviance, 3);	
		Assign_Item_String_Value(&(items.maximum_Blue_Channel_Deviance_Item), options->maximum_5_Bit_Blue_Channel_Deviance, 3);	
		Assign_Item_String_Value(&(items.maximum_Green_Channel_Deviance_Item), options->maximum_5_Bit_Green_Channel_Deviance, 3);	
		Assign_Item_String_Value(&(items.maximum_Total_Channel_Deviance_Item), options->maximum_5_Bit_Total_Channel_Deviance, 3);	
		Assign_Item_String_Value(&(items.maximum_Red_Channel_Lenient_Deviance_Item), options->maximum_5_Bit_Red_Channel_Lenient_Deviance, 3);	
		Assign_Item_String_Value(&(items.maximum_Blue_Channel_Lenient_Deviance_Item), options->maximum_5_Bit_Blue_Channel_Lenient_Deviance, 3);	
		Assign_Item_String_Value(&(items.maximum_Green_Channel_Lenient_Deviance_Item), options->maximum_5_Bit_Green_Channel_Lenient_Deviance, 3);	
		Assign_Item_String_Value(&(items.maximum_Total_Channel_Lenient_Deviance_Item), options->maximum_5_Bit_Total_Channel_Lenient_Deviance, 3);	
		Assign_Item_String_Value(&(items.maximum_Red_Drift_Deviance_Item), options->maximum_5_Bit_Red_Drift_Deviance, 3);	
		Assign_Item_String_Value(&(items.maximum_Blue_Drift_Deviance_Item), options->maximum_5_Bit_Blue_Drift_Deviance, 3);	
		Assign_Item_String_Value(&(items.maximum_Green_Drift_Deviance_Item), options->maximum_5_Bit_Green_Drift_Deviance, 3);	
		Assign_Item_String_Value(&(items.maximum_Total_Drift_Deviance_Item), options->maximum_5_Bit_Total_Drift_Deviance, 3);	
		Assign_Item_String_Value(&(items.maximum_Brightness_Deviance_Item), options->maximum_Brightness_Deviance_5, 3);	
		Assign_Item_String_Value(&(items.maximum_Darkness_Deviance_Item), options->maximum_Darkness_Deviance_5, 3);	
		Assign_Item_String_Value(&(items.maximum_Red_Channel_Absolute_High_Limit_Item), options->maximum_5_Bit_Red_Channel_Absolute_High_Limit, 3);	
		Assign_Item_String_Value(&(items.maximum_Blue_Channel_Absolute_High_Limit_Item), options->maximum_5_Bit_Blue_Channel_Absolute_High_Limit, 3);	
		Assign_Item_String_Value(&(items.maximum_Green_Channel_Absolute_High_Limit_Item), options->maximum_5_Bit_Green_Channel_Absolute_High_Limit, 3);	
		Assign_Item_String_Value(&(items.maximum_Total_Channel_Absolute_High_Limit_Item), options->maximum_5_Bit_Total_Channel_Absolute_High_Limit, 3);	
		Assign_Item_String_Value(&(items.maximum_Red_Channel_Absolute_Low_Limit_Item), options->maximum_5_Bit_Red_Channel_Absolute_Low_Limit, 3);	
		Assign_Item_String_Value(&(items.maximum_Blue_Channel_Absolute_Low_Limit_Item), options->maximum_5_Bit_Blue_Channel_Absolute_Low_Limit, 3);	
		Assign_Item_String_Value(&(items.maximum_Green_Channel_Absolute_Low_Limit_Item), options->maximum_5_Bit_Green_Channel_Absolute_Low_Limit, 3);	
		Assign_Item_String_Value(&(items.maximum_Total_Channel_Absolute_Low_Limit_Item), options->maximum_5_Bit_Total_Channel_Absolute_Low_Limit, 3);	
	}
	else {
		Assign_Item_String_Value(&(items.maximum_Red_Channel_Deviance_Item), options->maximum_8_Bit_Red_Channel_Deviance, 3);	
		Assign_Item_String_Value(&(items.maximum_Blue_Channel_Deviance_Item), options->maximum_8_Bit_Blue_Channel_Deviance, 3);	
		Assign_Item_String_Value(&(items.maximum_Green_Channel_Deviance_Item), options->maximum_8_Bit_Green_Channel_Deviance, 3);	
		Assign_Item_String_Value(&(items.maximum_Total_Channel_Deviance_Item), options->maximum_8_Bit_Total_Channel_Deviance, 3);	
		Assign_Item_String_Value(&(items.maximum_Red_Channel_Lenient_Deviance_Item), options->maximum_8_Bit_Red_Channel_Lenient_Deviance, 3);	
		Assign_Item_String_Value(&(items.maximum_Blue_Channel_Lenient_Deviance_Item), options->maximum_8_Bit_Blue_Channel_Lenient_Deviance, 3);	
		Assign_Item_String_Value(&(items.maximum_Green_Channel_Lenient_Deviance_Item), options->maximum_8_Bit_Green_Channel_Lenient_Deviance, 3);	
		Assign_Item_String_Value(&(items.maximum_Total_Channel_Lenient_Deviance_Item), options->maximum_8_Bit_Total_Channel_Lenient_Deviance, 3);	
		Assign_Item_String_Value(&(items.maximum_Red_Drift_Deviance_Item), options->maximum_8_Bit_Red_Drift_Deviance, 3);	
		Assign_Item_String_Value(&(items.maximum_Blue_Drift_Deviance_Item), options->maximum_8_Bit_Blue_Drift_Deviance, 3);	
		Assign_Item_String_Value(&(items.maximum_Green_Drift_Deviance_Item), options->maximum_8_Bit_Green_Drift_Deviance, 3);	
		Assign_Item_String_Value(&(items.maximum_Total_Drift_Deviance_Item), options->maximum_8_Bit_Total_Drift_Deviance, 3);	
		Assign_Item_String_Value(&(items.maximum_Brightness_Deviance_Item), options->maximum_Brightness_Deviance_8, 3);	
		Assign_Item_String_Value(&(items.maximum_Darkness_Deviance_Item), options->maximum_Darkness_Deviance_8, 3);	
		Assign_Item_String_Value(&(items.maximum_Red_Channel_Absolute_High_Limit_Item), options->maximum_8_Bit_Red_Channel_Absolute_High_Limit, 3);	
		Assign_Item_String_Value(&(items.maximum_Blue_Channel_Absolute_High_Limit_Item), options->maximum_8_Bit_Blue_Channel_Absolute_High_Limit, 3);	
		Assign_Item_String_Value(&(items.maximum_Green_Channel_Absolute_High_Limit_Item), options->maximum_8_Bit_Green_Channel_Absolute_High_Limit, 3);	
		Assign_Item_String_Value(&(items.maximum_Total_Channel_Absolute_High_Limit_Item), options->maximum_8_Bit_Total_Channel_Absolute_High_Limit, 3);	
		Assign_Item_String_Value(&(items.maximum_Red_Channel_Absolute_Low_Limit_Item), options->maximum_8_Bit_Red_Channel_Absolute_Low_Limit, 3);	
		Assign_Item_String_Value(&(items.maximum_Blue_Channel_Absolute_Low_Limit_Item), options->maximum_8_Bit_Blue_Channel_Absolute_Low_Limit, 3);	
		Assign_Item_String_Value(&(items.maximum_Green_Channel_Absolute_Low_Limit_Item), options->maximum_8_Bit_Green_Channel_Absolute_Low_Limit, 3);	
		Assign_Item_String_Value(&(items.maximum_Total_Channel_Absolute_Low_Limit_Item), options->maximum_8_Bit_Total_Channel_Absolute_Low_Limit, 3);	
	}

	current_Outline_Item_Number = BACKGROUND_OPTIONS_OUTLINE_ITEM;
	
	valid_Input = FALSE;
	keep_Inquiring = TRUE;
	
	do {					/* while (valid_Input == FALSE); */
		if (Perform_Dialog(BACKGROUND_OPTIONS_DIALOG_ID, (Item_Variables_Ptr)&items) == TRUE) {
		
			if (items.header.item_Hit == k_OK_Item) {
			
				if (Validate_And_Copy_Contents(&items, options, color_Resolution) == SAFE_TRUE) {
					valid_Input = TRUE;
					keep_Inquiring = FALSE;
				}
				else {
					/*  */
					/* Redundant, but placed here for debugging breaks */
					/*  */
					valid_Input = FALSE;
					keep_Inquiring = TRUE;
				}
								
			} /* if (items.header.item_Hit == k_OK_Item) */
			else {
				; /* NULL */
				
			} /* NOT if (items.header.item_Hit == k_OK_Item) */
				
		} /* if (Perform_Dialog(BACKGROUND_OPTIONS_DIALOG_ID, (Item_Variables_Ptr)&items) */
		else {
			valid_Input = FALSE;
			keep_Inquiring = FALSE;
		}
	
	} while (keep_Inquiring == TRUE);


	if (valid_Input != TRUE) {
		goto Error_Handler;
	}


	if (Scale_Output_Defaults_For_Background_Removal(options) != SAFE_TRUE) {
		goto Error_Handler;
	}
	

	function_Result = TRUE;
	goto Normal_Function_Exit;
	
Error_Handler:
	function_Result = FALSE;
			
Normal_Function_Exit:

	return(function_Result);
	
} /* Boolean Get_Background_Removal_Options(... */
/*=**************************************************************************=*/


/*=**************************************************************************=*/
/* END OF FILE: Dialog_Of_Background_Removal.c */






