/* START OF FILE: Copy_Compacted_Silloette.c */


/*=**************************************************************************=*/
/* [---BEGIN---] FILE HEADER COMMENTS */
/**
 ** SCREEN VIEW = [Adobe Syntax Black-10, Tabs = 4]
 ** PRINTER VIEW = [Adobe Syntax Black-10, Tabs = 4]
 ** =======================================================================
 ** Written by James Merkle
 ** Copy_Compacted_Silloette.c - Fast ability to draw a from a run length span compresed source using region 
 ** (C) Copyright 1993 by James Merkle
 ** All rights reserved, except those wishing to "copy" moderate sections of code or files may do so, so long as
 ** such code is never used directly either in animation-related tools or multimedia tools that are  
 ** used in live performance or live broadcast or recorded mass market broadcast or film. Desktop usage not restricted.
 ** Conventional desktop game or entertainment not restricted.
 ** =======================================================================
 **
 **
 ** ============
 ** Revision History
 ** ============
 **
 **  1993.03.16		(James Merkle)
 **						Initial coding started
 **
 **
 **
 **
 ** =================
 ** OTHER INFORMATION
 ** =================
 **  Non-overlapping memory required, same depth required, different offsets and rowbytes OK, region OK
 **
 **/
/* [----END----] FILE HEADER COMMENTS */
/*=**************************************************************************=*/


/*=**************************************************************************=*/
/* [---BEGIN---] #include AREA */

#include <QDOffscreen.h>

#include "Fast_Bit_Copy.h"
#include "Graphics_Primitives.h"
#include "Shell_Assembly.h"
#include "Error_Utilities.h"
#include "Memory_Utilities.h"
/* ===
#include "Dreaming_Stuff.h"
=== */
#include "Interface_Utilities.h"
#include "Copy_Compacted_Picture.h"

#include "Copy_Compacted_Silloette.h"


/* [----END----] #include AREA */
/*=**************************************************************************=*/


/*=**************************************************************************=*/
/* [---BEGIN---] #define AREA */

#define k_BUILD_SILLOETTE_COPY_VERSION		1

/* [----END----] #define AREA */
/*=**************************************************************************=*/


/*=**************************************************************************=*/
/* [---BEGIN---] TYPE AREA */


/* [----END----] TYPE AREA */
/*=**************************************************************************=*/


/*=**************************************************************************=*/
/* [---BEGIN---] PROTOTYPE AREA */


/* [----END----] PROTOTYPE AREA */
/*=**************************************************************************=*/


/*=**************************************************************************=*/
/* [---BEGIN---] GLOBAL DEFINITION AREA */


/* [----END----] GLOBAL DEFINITION AREA */
/*=**************************************************************************=*/


/*=**************************************************************************=*/
/* [---BEGIN---] LOCAL STATIC DEFINITION AREA */

static CARDINAL_8	*unit_Name = "\pCopy_Compacted_Silloette";

/* [----END----] LOCAL STATIC DEFINITION AREA */
/*=**************************************************************************=*/


/*=**************************************************************************=*/
void Fill_Compressed_Span_With_Simple_Silloette(CARDINAL_32	*row_Raw_Data_Ptr
															, CARDINAL_32	*skip_List_Data_Ptr
															, CARDINAL_32	*skip_List_Master_List_Ptr
															, CARDINAL_32	*destination_Word_Ptr
															, CARDINAL_32 destination_Rowbyte_Offset
															, CARDINAL_32	*source_Byte_Ptr
															, CARDINAL_32	source_Row_Bytes)
{

/* 1995.01.04 JAM Notes : Compressed region skip table now lacks */
/* extra offset capability (both inward jump mask and qty to skip in bytes */
/* Features not important now */

/* ======

we have source color pixmap, it may be in many small fragments with voids of unimportant areas
we have bounding region with frame rect, it must be equal or less than the source pixmap fragments

we want FAST copy bits for these objects

computation when loading not desired so sometimes it would be nice to 
force destination color table not source color table. Both color convert 
load balance directions needed.

Solution : convert (region & pixMap) to (fast_Region & skip_Map)

Advantages of this two part structure:

(1) Bidirectional in vertical directions allowing inverted image if needed
(2) Bidirectional in vertical directions allowing bottom up drawing if needed
(3) Might be faster than "byte"-biased unoptimized RLE because only 32 aligned  bit read/writes are used.
(4) Code cache not exhausted as with move immediates from pure shape blit compilers
(5) Has zero cycle load/setup time despite different rowbyte destination unlike pure shape blit compilers
(6) Could be used with 16 color, 4 color, and 1 color data with very little adjustment. Source&Dest equality needed.
(7) Extra sets of different regions can be used with one shared skip_Map (as long as skip_Map is combined regions of all)
This adds extra flexibility a standard RLE will not have because a standard RLE is one atomic structure.
(8) Region or Source is allowed to be infinitely sparse in vertical and horizontal directions with 0 cycle loss
(9) Source can be any arbitrary size withing confines of source rect at any non-normalized offset
(10) Source can be any number of fragments including "0"
(11) Copy can be modified to clip top and bottom very very easy.
(12) Secondary Copy routine could be created to clip horizontal directions, but not done yet.
(13) Everything is in handles and none need to be locked for ANY structures containing data during entire call
(14) Much faster than RLE technologies

Disadvantages (Minor):

(1) slower for small non-sparse sprites than an optimal arcade shape compiler lacking many advantages listed above
(2) For 8 bit data can only be used for non-32 bit aligned offset destinations (bit offset: 0,8,16,24) but slower
than pre shifted copies of those byte offsets. Motorola/Mac limit not source code enforced.
(3) For arbitrary pixel offset into destination at bit depths loser than 8, preshifted pre calculated
sprites at harmonic offsets allowing at least byte offsetting is needed. 1 bit would require 8 copies!
(4) Compression is not RLE but compression is memory friendly for unneeded masked out areas not part of any expected region.
(5) Because it is not RLE, images can be very complex, but cannot be index color mapped in transfer loop
(6) Because not RLE, rotation effects based on RLE technologies have to do a few more instructions

====== */

asm 68040 {
	movem.l d0-d7/a0-a4, -(sp)	; Save all registers to alter
	
	
	move.l	row_Raw_Data_Ptr(a6), a3	; Load row_Raw_Data_Ptr, already locked down ? not needed
	
	move.l	skip_List_Data_Ptr(a6), d5	; Load skip_List_Data_Ptr, already locked down ? not needed
																	; DO NOT avoid the safety "count" sentinel at start of list, this address is absolute.
	
	move.l	skip_List_Master_List_Ptr(a6), a4	; Load skip_List_Master_List_Ptr, 
														;  first element address of binary row/skip table triple list
	add.l		#4, a4									; avoid the safety "count" sentinel at start of list, 
	
	move.l	destination_Rowbyte_Offset(a6), d4	; Set destination rowbyte offset
	
	move.l	destination_Word_Ptr(a6), d6			; Load destination for picture (upper left of rect)

;REM	sub.l		d4, d6									; Pre-decrement to normalize for looping incrementer if modified for
															;  one line per offset span allowed.

#if (k_BUILD_SILLOETTE_COPY_VERSION == 1)

	move.l	source_Byte_Ptr(a6), d7					; Set source data offset (upper left of rect)
	move.l	a6, -(sp)												; Push a6 local stack frame onto stack! (We need the register)
	move.l	source_Row_Bytes(a6), a6				; Set source rowbyte offset (upper left of rect)

	move.l	d7, a3										; Spoof A3 to point to source bitmap instead (NOT NEEDED???)

;REM	sub.l		a6, d7									; Pre-decrement to normalize for looping incrementer
															;  If this drew in reverse order it would have been added
																	
																	
#endif /* #if (k_BUILD_SILLOETTE_COPY_VERSION == 1) */
			
	bra			@End_Of_Line							; Optimize out a branch test by starting loop here
	
	
/* - - - - - - - - - */
/* - - - - - - - - - */

; Alignment padding for speed (end loops)
		nop
		nop
			
/* - - - - - - - - - */
/* - - - - - - - - - */
  
; Group A : tokens 0 through 15 are for when next token is not a skip or space based token
; it encodes only 32 bits worth of data, it writes data out without using masked read
; these are very fast and are used if total span is 20 bytes or less. 

; Group B : tokens 16 through 31 are for when next token is skip or space based token
; it encodes only 32 bits worth of data

; Group C : tokens 32 through 47 are for when next token is skip or space based token
; it encodes 32 bits of leading data then a span of multiple 32 bit words (at least one) then trailing 32 if any
; note this must always be 64 bits or greater and 64 bits must be contiguous on two 32 bit words because
; at leat on loop and at least one leading 32 bit word must always be present.

; Group D : tokens 48 through 63 are for when next token is skip or space based token
; it encodes 32 bits of leading data if any then a span of multiples of 128 bytes (using 8 move16 68040, etc)
; then multiple 32 bit remainders then trailing 32 if any. Hopefully it is a common token because it is fast.
; the reminder is done by a jump into a table that does not loop.

; Group E : tokens 64 through 79 are for when next token is skip or space based token
; it encodes 32 bits of leading data if any then 1,2,or 3 leading 32bit words, 
; then a span of multiples of 128 bytes (using 8 move16 68040, etc)
; then multiple 32 bit remainders then trailing 32 if any. It is fast, but not as fast as Group D, but it
; is probably 3 times more common than Group D.
	
@Repeat_Start:
	
	move.l	(a4)+, d1			; Get Address of Row Start Raw Data, absolute offset
	add.l		d0, a1				; Adjust Skip List Entry by adding offset from triplet (could be optimized if needed)
	
#if (k_BUILD_SILLOETTE_COPY_VERSION == 0)	
	add.l		d1, a0				; This addition could be optimized into an absolute if needed in future
									;  a0 thus now equals raw start address of source 
									;  a0 already equals a3, a1 reset to d5, and d0 already was reset to via a4
#else 
	move.l	a3,a0		; NOT NEEEDED.
#endif
	
	move.l	(a4)+, d0			; Get Get total source-dest lines to skip

	bra			@Loop_End_Destination_Adjust	; Base already 0, Branch to adjust for "dbra"
@Loop_Start_Destination_Adjust:

#if (k_BUILD_SILLOETTE_COPY_VERSION == 1)	
	add.l		a6, d7				; Increment source pointer by rowbyte offset one time
	add.l		d4, d6				; Increment destination pointer by rowbyte offset one time
#else
	add.l		d4, d6				; Increment destination pointer by rowbyte offset one time
#endif

@Loop_End_Destination_Adjust:
	dbra		d0, @Loop_Start_Destination_Adjust
		
#if (k_BUILD_SILLOETTE_COPY_VERSION == 1)	
	move.l	d7, a0				; Copy d7 to a0 hijacking source
	move.l	d6, a2				; Copy d6 to a2 for use in prime loop, d6 is to be preserved in incremented form.
#else
	move.l	d6, a2				; Copy d6 to a2 for use in prime loop, d6 is to be preserved in incremented form.
#endif

/* ===

At this point we have:
a0 : points to Row Scan line raw data for this line of pixels to transfer (could be a packed picture)
a1 : points to Skip List Entry that will be used to skip over and select just what we want to copy
a2 : points to destination 32bit word unadjusted. Skip runs of this will appear simultaneous with source
a6 : points to source data 32bit word unadjusted (row pointer)

Those are the only three values we need to process a horizontal skip list. Also:

a3 : holds Row Raw Data main pointer for next loop through source, if any more
a4 : points to next skip list data tuplet for next loop through source, if any more
d4 : is equal to destination rowbyte offset for next loop through source, if any more
d5 : holds Skip List main pointer for next loop through source, if any more
d6 : holds virgin incremented copy of a2 for next loop through source, if any more
d7 : holds virgin incremented copy of a0 (source row) for next loop through source, if any more
 
  Free unused in routine : just d7, a6 available if needed
=== */

;dc.w	_Debugger
	;_Debugger
	
	move.l 	(a1)+,d0						; get Decompression token jump value
; 	adda.l 		(a1)+,a0						; get 32bit word source skip offset representing region additional offset
;	beq 		@End_Of_Line				; If d0 "0" value then the line is finished (OK to test late here)

#if (k_BUILD_SILLOETTE_COPY_VERSION == 1)	
 	move.l	(a1)+, d1
 	adda.l 		d1, a0						; adjust source 32bit word skip to be at parity with compressed source
 	adda.l 		d1, a2						; adjust destination 32bit word skip to be at parity with compressed source
#else
 	adda.l 		(a1)+,a2						; adjust destination 32bit word skip to be at parity with compressed source, if needed
#endif
	
	jmp		@Decompress_Jump_Table_Base(d0.w)	; Jump address should be ready by now in pipeline stage
	

	; In the following sections it is possible to assume d0,d2,d3 are trashable, a6&d7 could be obtained
	; (no masked read-in needed because all use direct bytes and such instead of masks)
	; d0 and d2 are used. d1 can be alterred.
	
	nop
	nop
	nop
	nop
	nop
	nop
	


#pragma mark -
#pragma mark Decompress_Jump_Table_Base

@Decompress_Jump_Table_Base
@Decode_Packet_0
										; Technique "0" =  no data means skip 4 bytes momentarily, rarely used

	addq.l		#4,a0					; Adjust source address to be only 8bit aligned
	addq.l		#4,a2					; Adjust destination address to be only 8bit aligned
	
	move.l 	(a1)+,d0						; get Decompression token jump value
	jmp		@Decompress_Jump_Table_Base(d0.w)	; Jump address should be ready by now in pipeline stage


	nop
	nop		; Space out to 48 bytes to allow 12 wasteful large PowerPC instructions in jump offset?
	nop		;  no... Space out to 24 bytes and have PowerPC code Shift the jump one power of 2 first.
	nop
	nop
	nop
	nop
	
;Decompress Technique 1 = simple byte mask pattern (0001) , no other pixel data
@Decode_Packet_1
	addq.l		#3,a0					; Adjust source address to be only 8bit aligned
	addq.l		#3,a2					; Adjust destination address to be only 8bit aligned
	move.b	(a0)+,(a2)+			; Copy 1 byte of data into destination (no masked read-in needed)

	move.l 	(a1)+,d0						; get Decompression token jump value
	jmp		@Decompress_Jump_Table_Base(d0.w)	; Jump address should be ready by now in pipeline stage
	
	nop
	nop		; Space out to 48 bytes to allow 12 wasteful large PowerPC instructions in jump offset?
	nop		;  no... Space out to 24 bytes and have PowerPC code Shift the jump one power of 2 first.
	nop
	nop
	nop

@Decode_Packet_2
;Decompress Technique 2 = simple byte mask pattern (0010) , no other pixel data
										; but data is in right side, no rotate needed
	move.b	2(a0),2(a2)			; Copy 1 byte of pixel data into destination
	addq.l		#4,a0					; Adjust destination address to normality
	addq.l		#4,a2					; Adjust destination address to normality

	move.l 	(a1)+,d0						; get Decompression token jump value
	jmp		@Decompress_Jump_Table_Base(d0.w)	; Jump address should be ready by now in pipeline stage

	nop
	nop		; Space out to 48 bytes to allow 12 wasteful large PowerPC instructions in jump offset?
	nop		;  no... Space out to 24 bytes and have PowerPC code Shift the jump one power of 2 first.
	nop

@Decode_Packet_3
;Decompress Technique 3 = simple byte mask pattern (0011) , no other pixel data
	addq.l		#2,a2					; Adjust destination address to be only 16bit aligned
	addq.l		#2,a0					; Adjust destination address to be only 16bit aligned
	move.w	(a0)+,(a2)+			; Copy 2 bytes of pixel data into destination

	move.l 	(a1)+,d0						; get Decompression token jump value
	jmp		@Decompress_Jump_Table_Base(d0.w)	; Jump address should be ready by now in pipeline stage

	nop
	nop		; Space out to 48 bytes to allow 12 wasteful large PowerPC instructions in jump offset?
	nop		;  no... Space out to 24 bytes and have PowerPC code Shift the jump one power of 2 first.
	nop
	nop
	nop
	

@Decode_Packet_4
;Decompress Technique 4 = simple byte mask pattern (0100) , no other pixel data
	move.b	1(a0),1(a2)			; Copy 1 byte of pixel data into destination
	addq.l		#4,a0					; Adjust source address to normality
	addq.l		#4,a2					; Adjust destination address to normality

	move.l 	(a1)+,d0						; get Decompression token jump value
	jmp		(@Decompress_Jump_Table_Base, PC,d0.w)	; Jump address should be ready by now in pipeline stage

	nop
	nop		; Space out to 48 bytes to allow 12 wasteful large PowerPC instructions in jump offset?
	nop		;  no... Space out to 24 bytes and have PowerPC code Shift the jump one power of 2 first.


@Decode_Packet_5
;Decompress Technique 5 = simple byte mask pattern (0101) , no other pixel data
	move.b	1(a0),1(a2)			; Copy 1 byte of pixel data into destination
	move.b	3(a0),3(a2)			; Copy 1 byte of pixel data into destination
	addq.l		#4,a0					; Adjust source address to normality
	addq.l		#4,a2					; Adjust destination address to normality


	move.l 	(a1)+,d0						; get Decompression token jump value
	jmp		(@Decompress_Jump_Table_Base, PC,d0.w)	; Jump address should be ready by now in pipeline stage

				; Space out to 48 bytes to allow 12 wasteful large PowerPC instructions in jump offset?
				;  no... Space out to 24 bytes and have PowerPC code Shift the jump one power of 2 first.

@Decode_Packet_6
;Decompress Technique 6 = simple byte mask pattern (0110) , no other pixel data, no unaligned writes
	move.b	1(a0),1(a2)			; Copy 1 byte of pixel data into destination
	move.b	2(a0),2(a2)			; Copy 1 byte of pixel data into destination
	addq.l		#4,a0					; Adjust source address to normality
	addq.l		#4,a2					; Adjust destination address to normality

	move.l 	(a1)+,d0						; get Decompression token jump value
	jmp		(@Decompress_Jump_Table_Base, PC,d0.w)	; Jump address should be ready by now in pipeline stage

	 			; Space out to 48 bytes to allow 12 wasteful large PowerPC instructions in jump offset?
				;  no... Space out to 24 bytes and have PowerPC code Shift the jump one power of 2 first.

@Decode_Packet_7
;Decompress Technique 7 = simple byte mask pattern (0111) , no other pixel data, no unaligned writes
	addq.l		#1,a0					; Adjust source address to normality
	addq.l		#1,a2					; Adjust destination address to normality
	move.b	(a0)+,(a2)+			; Copy 1 byte of pixel data into destination
	move.w	(a0)+,(a2)+			; Copy 2 bytes of pixel data into destination

	move.l 	(a1)+,d0						; get Decompression token jump value
	jmp		(@Decompress_Jump_Table_Base, PC,d0.w)	; Jump address should be ready by now in pipeline stage

	nop		; Space out to 48 bytes to allow 12 wasteful large PowerPC instructions in jump offset?
	nop		;  no... Space out to 24 bytes and have PowerPC code Shift the jump one power of 2 first.
	nop
	nop

@Decode_Packet_8
;Decompress Technique 8 = simple byte mask pattern (1000) , no other pixel data, no unaligned writes
	move.b	(a0),(a2)				; Copy 1 byte of pixel data into destination
	addq.l		#4,a0					; Adjust source address to normality
	addq.l		#4,a2					; Adjust destination address to normality

	move.l 	(a1)+,d0						; get Decompression token jump value
	jmp		(@Decompress_Jump_Table_Base, PC,d0.w)	; Jump address should be ready by now in pipeline stage

	nop
	nop		; Space out to 48 bytes to allow 12 wasteful large PowerPC instructions in jump offset?
	nop		;  no... Space out to 24 bytes and have PowerPC code Shift the jump one power of 2 first.
	nop
	nop


@Decode_Packet_9
;Decompress Technique 9 = simple byte mask pattern (1001) , no other pixel data, no unaligned writes
	move.b	(a0),(a2)				; Copy 1 byte of pixel data into destination
	move.b	3(a0),3(a2)			; Copy 1 byte of pixel data into destination
	addq.l		#4,a0					; Adjust source address to normality
	addq.l		#4,a2					; Adjust destination address to normality

	move.l 	(a1)+,d0						; get Decompression token jump value
	jmp		(@Decompress_Jump_Table_Base, PC,d0.w)	; Jump address should be ready by now in pipeline stage

	nop		; Space out to 48 bytes to allow 12 wasteful large PowerPC instructions in jump offset?
	nop		;  no... Space out to 24 bytes and have PowerPC code Shift the jump one power of 2 first.


@Decode_Packet_10
;Decompress Technique 10 = simple byte mask pattern (1010) , no other pixel data, no unaligned writes
	move.b	(a0),(a2)				; Copy 1 byte of pixel data into destination
	move.b	2(a0),2(a2)			; Copy 1 byte of pixel data into destination
	addq.l		#4,a0					; Adjust source address to normality
	addq.l		#4,a2					; Adjust destination address to normality

	move.l 	(a1)+,d0						; get Decompression token jump value
	jmp		(@Decompress_Jump_Table_Base, PC,d0.w)	; Jump address should be ready by now in pipeline stage

	nop		; Space out to 48 bytes to allow 12 wasteful large PowerPC instructions in jump offset?
	nop		;  no... Space out to 24 bytes and have PowerPC code Shift the jump one power of 2 first.


@Decode_Packet_11
;Decompress Technique 11 = simple byte mask pattern (1011) , no other pixel data, no unaligned writes
	move.b	(a0),(a2)				; Copy 1 byte of pixel data into destination
	move.w	2(a0),2(a2)			; Copy 2 bytes of pixel data into destination
	addq.l		#4,a0					; Adjust source address to normality
	addq.l		#4,a2					; Adjust destination address to normality

	move.l 	(a1)+,d0						; get Decompression token jump value
	jmp		(@Decompress_Jump_Table_Base, PC,d0.w)	; Jump address should be ready by now in pipeline stage
	nop
	nop


@Decode_Packet_12
;Decompress Technique 12 = simple byte mask pattern (1100) , no other pixel data, no unaligned writes
	move.w	(a0),(a2)				; Copy 2 bytes of pixel data into destination
	addq.l		#4,a0					; Adjust source address to normality
	addq.l		#4,a2					; Adjust destination address to normality

	move.l 	(a1)+,d0						; get Decompression token jump value
	jmp		(@Decompress_Jump_Table_Base, PC,d0.w)	; Jump address should be ready by now in pipeline stage
	nop
	nop
	nop
	nop
	nop


@Decode_Packet_13
;Decompress Technique 13 = simple byte mask pattern (1101) , no other pixel data, no unaligned writes
	move.w	(a0),(a2)				; Copy 2 bytes of pixel data into destination
	addq.l		#3,a0					; Adjust source address to normality
	addq.l		#3,a2					; Adjust destination address to normality
	move.b	(a0)+,(a2)+			; Copy 1 byte of pixel data into destination

	move.l 	(a1)+,d0						; get Decompression token jump value
	jmp		(@Decompress_Jump_Table_Base, PC,d0.w)	; Jump address should be ready by now in pipeline stage
	nop
	nop
	nop
	nop


@Decode_Packet_14
;Decompress Technique 14 = simple byte mask pattern (1110) , no other pixel data, no unaligned writes
	move.w	(a0)+,(a2)+			; Copy 2 bytes of pixel data into destination
	move.b	(a0),(a2)				; Copy 1 byte of pixel data into destination
	addq.l		#2,a0					; Adjust source address to normality
	addq.l		#2,a2					; Adjust destination address to normality

	move.l 	(a1)+,d0						; get Decompression token jump value
	jmp		(@Decompress_Jump_Table_Base, PC,d0.w)	; Jump address should be ready by now in pipeline stage
	nop
	nop
	nop
	nop


@Decode_Packet_15
;Decompress Technique 15 = simple byte mask pattern (1111) , no other pixel data, no unaligned writes
	move.l	(a0)+,(a2)+				; Copy 4 bytes of data into destination

	move.l 	(a1)+,d0					; get Decompression token jump value
	jmp		(@Decompress_Jump_Table_Base, PC,d0.w)	; Jump address should be ready by now in pipeline stage
	nop
	nop
	nop
	nop
	nop
	nop
	nop

/* ======================================================================== */
/* ======================================================================== */
/* ======================================================================== */

;Decompress Technique 16 = simple byte mask pattern (0000) , just a marker
@Decode_Packet_16
									; Technique "0" =  no data and means end of line
	move.l	a3, a0				; Get a fresh copy of Row Raw Data pointer (could be optimized out)
	move.l	d5, a1				; Get a fresh copy of Skip List pointer (could be optimized out)
	move.l	(a4)+, d0			; Get Address of Skip List Entry, absolute offset, cannot be "0" unless finished
	bne		@Repeat_Start	; If 0, exit routine, there are no entries left
	bra			@End_Of_All_Data
	nop
	nop
	nop		; Space out to 48 bytes to allow 12 wasteful large PowerPC instructions in jump offset?
	nop		;  no... Space out to 32 bytes and have PowerPC code Shift the jump one power of 2 first.
	nop
	nop
	nop
	nop
	nop
	
;Decompress Technique 17 = simple byte mask pattern (0001) , no other pixel data
@Decode_Packet_17
	move.b	3(a0),3(a2)			; Copy 1 byte of data into destination (no masked read-in needed)

	move.l	a3, a0				; Get a fresh copy of Row Raw Data pointer (could be optimized out)
	move.l	d5, a1				; Get a fresh copy of Skip List pointer (could be optimized out)
	move.l	(a4)+, d0			; Get Address of Skip List Entry, absolute offset, cannot be "0" unless finished
	bne		@Repeat_Start	; If 0, exit routine, there are no entries left
	bra			@End_Of_All_Data
	nop
	nop
	nop		; Space out to 48 bytes to allow 12 wasteful large PowerPC instructions in jump offset?
	nop		;  no... Space out to 32 bytes and have PowerPC code Shift the jump one power of 2 first.
	nop
	nop

@Decode_Packet_18
;Decompress Technique 18 = simple byte mask pattern (0010) , no other pixel data
										; but data is in right side, no rotate needed
	move.b	2(a0),2(a2)			; Copy 1 byte of pixel data into destination

	move.l	a3, a0				; Get a fresh copy of Row Raw Data pointer (could be optimized out)
	move.l	d5, a1				; Get a fresh copy of Skip List pointer (could be optimized out)
	move.l	(a4)+, d0			; Get Address of Skip List Entry, absolute offset, cannot be "0" unless finished
	bne		@Repeat_Start	; If 0, exit routine, there are no entries left
	bra			@End_Of_All_Data
	nop
	nop
	nop		; Space out to 48 bytes to allow 12 wasteful large PowerPC instructions in jump offset?
	nop		;  no... Space out to 32 bytes and have PowerPC code Shift the jump one power of 2 first.
	nop
	nop

@Decode_Packet_19
;Decompress Technique 19 = simple byte mask pattern (0011) , no other pixel data
	move.w	2(a0),2(a2)		; Copy 2 bytes of pixel data into destination

	move.l	a3, a0				; Get a fresh copy of Row Raw Data pointer (could be optimized out)
	move.l	d5, a1				; Get a fresh copy of Skip List pointer (could be optimized out)
	move.l	(a4)+, d0			; Get Address of Skip List Entry, absolute offset, cannot be "0" unless finished
	bne		@Repeat_Start	; If 0, exit routine, there are no entries left
	bra			@End_Of_All_Data
	nop
	nop
	nop
	nop
	nop
	nop
	

@Decode_Packet_20
;Decompress Technique 20 = simple byte mask pattern (0100) , no other pixel data
	move.b	1(a0),1(a2)			; Copy 1 byte of pixel data into destination


	move.l	a3, a0				; Get a fresh copy of Row Raw Data pointer (could be optimized out)
	move.l	d5, a1				; Get a fresh copy of Skip List pointer (could be optimized out)
	move.l	(a4)+, d0			; Get Address of Skip List Entry, absolute offset, cannot be "0" unless finished
	bne		@Repeat_Start	; If 0, exit routine, there are no entries left
	bra			@End_Of_All_Data
	nop
	nop
	nop
	nop
	nop
	nop


@Decode_Packet_21
;Decompress Technique 21 = simple byte mask pattern (0101) , no other pixel data
	move.b	1(a0),1(a2)			; Copy 1 byte of pixel data into destination
	move.b	3(a0),3(a2)			; Copy 1 byte of pixel data into destination

	move.l	a3, a0				; Get a fresh copy of Row Raw Data pointer (could be optimized out)
	move.l	d5, a1				; Get a fresh copy of Skip List pointer (could be optimized out)
	move.l	(a4)+, d0			; Get Address of Skip List Entry, absolute offset, cannot be "0" unless finished
	bne		@Repeat_Start	; If 0, exit routine, there are no entries left
	bra			@End_Of_All_Data
	nop
	nop
	nop

@Decode_Packet_22
;Decompress Technique 22 = simple byte mask pattern (0110) , no other pixel data, no unaligned writes
	move.b	1(a0),1(a2)			; Copy 1 byte of pixel data into destination
	move.b	2(a0),2(a2)			; Copy 1 byte of pixel data into destination

	move.l	a3, a0				; Get a fresh copy of Row Raw Data pointer (could be optimized out)
	move.l	d5, a1				; Get a fresh copy of Skip List pointer (could be optimized out)
	move.l	(a4)+, d0			; Get Address of Skip List Entry, absolute offset, cannot be "0" unless finished
	bne		@Repeat_Start	; If 0, exit routine, there are no entries left
	bra			@End_Of_All_Data
	nop
	nop
	nop

@Decode_Packet_23
;Decompress Technique 23 = simple byte mask pattern (0111) , no other pixel data, no unaligned writes
	move.b	1(a0),1(a2)		; Copy 1 byte of pixel data into destination
	move.w	2(a0),2(a2)		; Copy 2 bytes of pixel data into destination

	move.l	a3, a0				; Get a fresh copy of Row Raw Data pointer (could be optimized out)
	move.l	d5, a1				; Get a fresh copy of Skip List pointer (could be optimized out)
	move.l	(a4)+, d0			; Get Address of Skip List Entry, absolute offset, cannot be "0" unless finished
	bne		@Repeat_Start	; If 0, exit routine, there are no entries left
	bra			@End_Of_All_Data
	nop
	nop
	nop


@Decode_Packet_24
;Decompress Technique 24 = simple byte mask pattern (1000) , no other pixel data, no unaligned writes
	move.b	(a0),(a2)			; Copy 1 byte of pixel data into destination

	move.l	a3, a0				; Get a fresh copy of Row Raw Data pointer (could be optimized out)
	move.l	d5, a1				; Get a fresh copy of Skip List pointer (could be optimized out)
	move.l	(a4)+, d0			; Get Address of Skip List Entry, absolute offset, cannot be "0" unless finished
	bne		@Repeat_Start	; If 0, exit routine, there are no entries left
	bra			@End_Of_All_Data
	nop
	nop
	nop
	nop
	nop
	nop
	nop
	nop


@Decode_Packet_25
;Decompress Technique 25 = simple byte mask pattern (1001) , no other pixel data, no unaligned writes
	move.b	(a0),(a2)			; Copy 1 byte of pixel data into destination
	move.b	3(a0),3(a2)		; Copy 1 byte of pixel data into destination

	move.l	a3, a0				; Get a fresh copy of Row Raw Data pointer (could be optimized out)
	move.l	d5, a1				; Get a fresh copy of Skip List pointer (could be optimized out)
	move.l	(a4)+, d0			; Get Address of Skip List Entry, absolute offset, cannot be "0" unless finished
	bne		@Repeat_Start	; If 0, exit routine, there are no entries left
	bra			@End_Of_All_Data
	nop
	nop
	nop
	nop
	nop


@Decode_Packet_26
;Decompress Technique 26 = simple byte mask pattern (1010) , no other pixel data, no unaligned writes
	move.b	(a0),(a2)			; Copy 1 byte of pixel data into destination
	move.b	2(a0),2(a2)		; Copy 1 byte of pixel data into destination

	move.l	a3, a0				; Get a fresh copy of Row Raw Data pointer (could be optimized out)
	move.l	d5, a1				; Get a fresh copy of Skip List pointer (could be optimized out)
	move.l	(a4)+, d0			; Get Address of Skip List Entry, absolute offset, cannot be "0" unless finished
	bne		@Repeat_Start	; If 0, exit routine, there are no entries left
	bra			@End_Of_All_Data
	nop
	nop
	nop
	nop
	nop


@Decode_Packet_27
;Decompress Technique 27 = simple byte mask pattern (1011) , no other pixel data, no unaligned writes
	move.b	(a0),(a2)			; Copy 1 byte of pixel data into destination
	move.w	2(a0),2(a2)		; Copy 1 byte of pixel data into destination

	move.l	a3, a0				; Get a fresh copy of Row Raw Data pointer (could be optimized out)
	move.l	d5, a1				; Get a fresh copy of Skip List pointer (could be optimized out)
	move.l	(a4)+, d0			; Get Address of Skip List Entry, absolute offset, cannot be "0" unless finished
	bne		@Repeat_Start	; If 0, exit routine, there are no entries left
	bra			@End_Of_All_Data
	nop
	nop
	nop
	nop
	nop


@Decode_Packet_28
;Decompress Technique 28 = simple byte mask pattern (1100) , no other pixel data, no unaligned writes
	move.w	(a0),(a2)			; Copy 2 bytes of pixel data into destination

	move.l	a3, a0				; Get a fresh copy of Row Raw Data pointer (could be optimized out)
	move.l	d5, a1				; Get a fresh copy of Skip List pointer (could be optimized out)
	move.l	(a4)+, d0			; Get Address of Skip List Entry, absolute offset, cannot be "0" unless finished
	bne		@Repeat_Start	; If 0, exit routine, there are no entries left
	bra			@End_Of_All_Data
	nop
	nop
	nop
	nop
	nop
	nop
	nop
	nop


@Decode_Packet_29
;Decompress Technique 29 = simple byte mask pattern (1101) , no other pixel data, no unaligned writes
	move.w	(a0),(a2)			; Copy 2 bytes of pixel data into destination
	move.b	3(a0),3(a2)		; Copy 1 byte of pixel data into destination


	move.l	a3, a0				; Get a fresh copy of Row Raw Data pointer (could be optimized out)
	move.l	d5, a1				; Get a fresh copy of Skip List pointer (could be optimized out)
	move.l	(a4)+, d0			; Get Address of Skip List Entry, absolute offset, cannot be "0" unless finished
	bne		@Repeat_Start	; If 0, exit routine, there are no entries left
	bra			@End_Of_All_Data
	nop
	nop
	nop
	nop
	nop


@Decode_Packet_30
;Decompress Technique 30 = simple byte mask pattern (1110) , no other pixel data, no unaligned writes
	move.w	(a0)+,(a2)+		; Copy 2 bytes of pixel data into destination
	move.b	(a0),(a2)			; Copy 1 byte of pixel data into destination

	move.l	a3, a0				; Get a fresh copy of Row Raw Data pointer (could be optimized out)
	move.l	d5, a1				; Get a fresh copy of Skip List pointer (could be optimized out)
	move.l	(a4)+, d0			; Get Address of Skip List Entry, absolute offset, cannot be "0" unless finished
	bne		@Repeat_Start	; If 0, exit routine, there are no entries left
	bra			@End_Of_All_Data
	nop
	nop
	nop
	nop
	nop
	nop
	nop


@Decode_Packet_31
;Decompress Technique 31 = simple byte mask pattern (1111) , no other pixel data, no unaligned writes
	move.l	(a0),(a2)				; Copy 1 byte of data into destination

	move.l	a3, a0				; Get a fresh copy of Row Raw Data pointer (could be optimized out)
	move.l	d5, a1				; Get a fresh copy of Skip List pointer (could be optimized out)
	move.l	(a4)+, d0			; Get Address of Skip List Entry, absolute offset, cannot be "0" unless finished
	bne		@Repeat_Start	; If 0, exit routine, there are no entries left
	bra			@End_Of_All_Data
	nop
	nop
	nop
	nop
	nop
	nop
	nop
	nop

/* ======================================================================== */
/* ======================================================================== */
/* ======================================================================== */

@Decode_Packet_32
;Decompress Technique 32 = start byte mask pattern (0000), 32 bit, 32bit word loop, end mask (0000)

	move.l (a1)+, d0				; Get dbra count for long transfer loop (it is already BASE 0!)
@Loop_Copy_Decode_Packet_32:
	move.l	(a0)+, (a2)+		; Non-optimal 32Bit transfer loop, max of 31 ever done anyway
	dbra		d0, @Loop_Copy_Decode_Packet_32


	move.l	a3, a0				; Get a fresh copy of Row Raw Data pointer (could be optimized out)
	move.l	d5, a1				; Get a fresh copy of Skip List pointer (could be optimized out)
	move.l	(a4)+, d0			; Get Address of Skip List Entry, absolute offset, cannot be "0" unless finished
	bne		@Repeat_Start	; If 0, exit routine, there are no entries left
	bra			@End_Of_All_Data
	nop
	nop
	nop		; Space out to 48 bytes to allow 12 wasteful large PowerPC instructions in jump offset?
	nop		;  no... Space out to 40 bytes and have PowerPC code Shift the jump one power of 2 first.
	nop
	nop
	nop
	nop
	nop

@Decode_Packet_33
;Decompress Technique 33 = start byte mask pattern (0001), 32bit word loop, end mask (0000)
	addq.l		#3,a0					; Adjust source address to be only 8bit aligned
	addq.l		#3,a2					; Adjust destination address to be only 8bit aligned
	move.b	(a0)+,(a2)+			; Copy 1 byte of data into destination (no masked read-in needed)


	move.l (a1)+, d0				; Get dbra count for long transfer loop (it is already BASE 0!)
@Loop_Copy_Decode_Packet_33:
	move.l	(a0)+, (a2)+		; Non-optimal 32Bit transfer loop, max of 31 ever done anyway
	dbra		d0, @Loop_Copy_Decode_Packet_33


	move.l	a3, a0				; Get a fresh copy of Row Raw Data pointer (could be optimized out)
	move.l	d5, a1				; Get a fresh copy of Skip List pointer (could be optimized out)
	move.l	(a4)+, d0			; Get Address of Skip List Entry, absolute offset, cannot be "0" unless finished
	bne		@Repeat_Start	; If 0, exit routine, there are no entries left 
	bra			@End_Of_All_Data
	nop
	nop
	nop
	nop
	nop
	nop


@Decode_Packet_34
;Decompress Technique 34 = start byte mask pattern (0011), 32bit word loop, end mask (0000)
	addq.l		#2,a2					; Adjust destination address to be only 16bit aligned
	addq.l		#2,a0					; Adjust destination address to be only 16bit aligned
	move.w	(a0)+,(a2)+			; Copy 2 bytes of pixel data into destination


	move.l (a1)+, d0				; Get dbra count for long transfer loop (it is already BASE 0!)
@Loop_Copy_Decode_Packet_34:
	move.l	(a0)+, (a2)+		; Non-optimal 32Bit transfer loop, max of 31 ever done anyway
	dbra		d0, @Loop_Copy_Decode_Packet_34


	move.l	a3, a0				; Get a fresh copy of Row Raw Data pointer (could be optimized out)
	move.l	d5, a1				; Get a fresh copy of Skip List pointer (could be optimized out)
	move.l	(a4)+, d0			; Get Address of Skip List Entry, absolute offset, cannot be "0" unless finished
	bne		@Repeat_Start	; If 0, exit routine, there are no entries left
	bra			@End_Of_All_Data
	nop
	nop
	nop
	nop
	nop
	nop

@Decode_Packet_35
;Decompress Technique 35 = start byte mask pattern (0111), 32bit word loop, end mask (0000)
	addq.l		#1,a0					; Adjust source address to normality
	addq.l		#1,a2					; Adjust destination address to normality
	move.b	(a0)+,(a2)+			; Copy 1 byte of pixel data into destination
	move.w	(a0)+,(a2)+			; Copy 2 bytes of pixel data into destination


	move.l (a1)+, d0				; Get dbra count for long transfer loop (it is already BASE 0!)
@Loop_Copy_Decode_Packet_35:
	move.l	(a0)+, (a2)+		; Non-optimal 32Bit transfer loop, max of 31 ever done anyway
	dbra		d0, @Loop_Copy_Decode_Packet_35


	move.l	a3, a0				; Get a fresh copy of Row Raw Data pointer (could be optimized out)
	move.l	d5, a1				; Get a fresh copy of Skip List pointer (could be optimized out)
	move.l	(a4)+, d0			; Get Address of Skip List Entry, absolute offset, cannot be "0" unless finished
	bne		@Repeat_Start	; If 0, exit routine, there are no entries left
	bra			@End_Of_All_Data
	nop
	nop
	nop
	nop
	nop

@Decode_Packet_36
;Decompress Technique 36 = start byte mask pattern (0000), 32 bit, , 32bit word loop, end mask (1000)

	move.l (a1)+, d0				; Get dbra count for long transfer loop (it is already BASE 0!)
@Loop_Copy_Decode_Packet_36:
	move.l	(a0)+, (a2)+		; Non-optimal 32Bit transfer loop, max of 31 ever done anyway
	dbra		d0, @Loop_Copy_Decode_Packet_36

	move.b	(a0),(a2)				; Copy 1 byte of data into destination

	move.l	a3, a0				; Get a fresh copy of Row Raw Data pointer (could be optimized out)
	move.l	d5, a1				; Get a fresh copy of Skip List pointer (could be optimized out)
	move.l	(a4)+, d0			; Get Address of Skip List Entry, absolute offset, cannot be "0" unless finished
	bne		@Repeat_Start	; If 0, exit routine, there are no entries left
	bra			@End_Of_All_Data
	nop
	nop
	nop
	nop
	nop
	nop
	nop
	nop


@Decode_Packet_37
;Decompress Technique 37 = start byte mask pattern (0001), 32bit word loop, end mask (1000)
	addq.l		#3,a0					; Adjust source address to be only 8bit aligned
	addq.l		#3,a2					; Adjust destination address to be only 8bit aligned
	move.b	(a0)+,(a2)+			; Copy 1 byte of data into destination (no masked read-in needed)

	move.l (a1)+, d0				; Get dbra count for long transfer loop (it is already BASE 0!)
@Loop_Copy_Decode_Packet_37:
	move.l	(a0)+, (a2)+		; Non-optimal 32Bit transfer loop, max of 31 ever done anyway
	dbra		d0, @Loop_Copy_Decode_Packet_37

	move.b	(a0),(a2)				; Copy 1 byte of data into destination

	move.l	a3, a0				; Get a fresh copy of Row Raw Data pointer (could be optimized out)
	move.l	d5, a1				; Get a fresh copy of Skip List pointer (could be optimized out)
	move.l	(a4)+, d0			; Get Address of Skip List Entry, absolute offset, cannot be "0" unless finished
	bne		@Repeat_Start	; If 0, exit routine, there are no entries left
	bra			@End_Of_All_Data
	nop
	nop
	nop
	nop
	nop

@Decode_Packet_38
;Decompress Technique 38 = start byte mask pattern (0011), 32bit word loop, end mask (1000)
	addq.l		#2,a2					; Adjust destination address to be only 16bit aligned
	addq.l		#2,a0					; Adjust destination address to be only 16bit aligned
	move.w	(a0)+,(a2)+			; Copy 2 bytes of pixel data into destination

	move.l (a1)+, d0				; Get dbra count for long transfer loop (it is already BASE 0!)
@Loop_Copy_Decode_Packet_38:
	move.l	(a0)+, (a2)+		; Non-optimal 32Bit transfer loop, max of 31 ever done anyway
	dbra		d0, @Loop_Copy_Decode_Packet_38

	move.b	(a0),(a2)				; Copy 1 byte of data into destination

	move.l	a3, a0				; Get a fresh copy of Row Raw Data pointer (could be optimized out)
	move.l	d5, a1				; Get a fresh copy of Skip List pointer (could be optimized out)
	move.l	(a4)+, d0			; Get Address of Skip List Entry, absolute offset, cannot be "0" unless finished
	bne		@Repeat_Start	; If 0, exit routine, there are no entries left
	bra			@End_Of_All_Data
	nop
	nop
	nop
	nop
	nop


@Decode_Packet_39
;Decompress Technique 39 = start byte mask pattern (0111), 32bit word loop, end mask (1000)
	addq.l		#1,a0					; Adjust source address to normality
	addq.l		#1,a2					; Adjust destination address to normality
	move.b	(a0)+,(a2)+			; Copy 1 byte of pixel data into destination
	move.w	(a0)+,(a2)+			; Copy 2 bytes of pixel data into destination

	move.l (a1)+, d0				; Get dbra count for long transfer loop (it is already BASE 0!)
@Loop_Copy_Decode_Packet_39:
	move.l	(a0)+, (a2)+		; Non-optimal 32Bit transfer loop, max of 31 ever done anyway
	dbra		d0, @Loop_Copy_Decode_Packet_39

	move.b	(a0),(a2)				; Copy 1 byte of data into destination

	move.l	a3, a0				; Get a fresh copy of Row Raw Data pointer (could be optimized out)
	move.l	d5, a1				; Get a fresh copy of Skip List pointer (could be optimized out)
	move.l	(a4)+, d0			; Get Address of Skip List Entry, absolute offset, cannot be "0" unless finished
	bne		@Repeat_Start	; If 0, exit routine, there are no entries left
	bra			@End_Of_All_Data
	nop
	nop
	nop
	nop

@Decode_Packet_40
;Decompress Technique 40 = start byte mask pattern (0000), 32 bit, , 32bit word loop, end mask (1100)

	move.l (a1)+, d0					; Get dbra count for long transfer loop (it is already BASE 0!)
@Loop_Copy_Decode_Packet_40:
	move.l	(a0)+, (a2)+			; Non-optimal 32Bit transfer loop, max of 31 ever done anyway
	dbra		d0, @Loop_Copy_Decode_Packet_40

	move.w	(a0),(a2)				; Copy 2 bytes of data into destination

	move.l	a3, a0				; Get a fresh copy of Row Raw Data pointer (could be optimized out)
	move.l	d5, a1				; Get a fresh copy of Skip List pointer (could be optimized out)
	move.l	(a4)+, d0			; Get Address of Skip List Entry, absolute offset, cannot be "0" unless finished
	bne		@Repeat_Start	; If 0, exit routine, there are no entries left
	bra			@End_Of_All_Data
	nop
	nop
	nop
	nop
	nop
	nop
	nop
	nop


@Decode_Packet_41
;Decompress Technique 41 = start byte mask pattern (0001), 32bit word loop, end mask (1100)
	addq.l		#3,a0					; Adjust source address to be only 8bit aligned
	addq.l		#3,a2					; Adjust destination address to be only 8bit aligned
	move.b	(a0)+,(a2)+			; Copy 1 byte of data into destination (no masked read-in needed)

	move.l (a1)+, d0					; Get dbra count for long transfer loop (it is already BASE 0!)
@Loop_Copy_Decode_Packet_41:
	move.l	(a0)+, (a2)+			; Non-optimal 32Bit transfer loop, max of 31 ever done anyway
	dbra		d0, @Loop_Copy_Decode_Packet_41

	move.w	(a0),(a2)				; Copy 2 bytes of data into destination

	move.l	a3, a0				; Get a fresh copy of Row Raw Data pointer (could be optimized out)
	move.l	d5, a1				; Get a fresh copy of Skip List pointer (could be optimized out)
	move.l	(a4)+, d0			; Get Address of Skip List Entry, absolute offset, cannot be "0" unless finished
	bne		@Repeat_Start	; If 0, exit routine, there are no entries left
	bra			@End_Of_All_Data
	nop
	nop
	nop
	nop
	nop


@Decode_Packet_42
;Decompress Technique 41 = start byte mask pattern (0011), 32bit word loop, end mask (1100)
	addq.l		#2,a2					; Adjust destination address to be only 16bit aligned
	addq.l		#2,a0					; Adjust destination address to be only 16bit aligned
	move.w	(a0)+,(a2)+			; Copy 2 bytes of pixel data into destination

	move.l (a1)+, d0					; Get dbra count for long transfer loop (it is already BASE 0!)
@Loop_Copy_Decode_Packet_42:
	move.l	(a0)+, (a2)+			; Non-optimal 32Bit transfer loop, max of 31 ever done anyway
	dbra		d0, @Loop_Copy_Decode_Packet_42

	move.w	(a0),(a2)				; Copy 2 bytes of data into destination

	move.l	a3, a0				; Get a fresh copy of Row Raw Data pointer (could be optimized out)
	move.l	d5, a1				; Get a fresh copy of Skip List pointer (could be optimized out)
	move.l	(a4)+, d0			; Get Address of Skip List Entry, absolute offset, cannot be "0" unless finished
	bne		@Repeat_Start	; If 0, exit routine, there are no entries left
	bra			@End_Of_All_Data
	nop
	nop
	nop
	nop
	nop


@Decode_Packet_43
;Decompress Technique 43 = start byte mask pattern (0111), 32bit word loop, end mask (1100)
	addq.l		#1,a0					; Adjust source address to normality
	addq.l		#1,a2					; Adjust destination address to normality
	move.b	(a0)+,(a2)+			; Copy 1 byte of pixel data into destination
	move.w	(a0)+,(a2)+			; Copy 2 bytes of pixel data into destination

	move.l (a1)+, d0					; Get dbra count for long transfer loop (it is already BASE 0!)
@Loop_Copy_Decode_Packet_43:
	move.l	(a0)+, (a2)+			; Non-optimal 32Bit transfer loop, max of 31 ever done anyway
	dbra		d0, @Loop_Copy_Decode_Packet_43

	move.w	(a0),(a2)				; Copy 2 bytes of data into destination

	move.l	a3, a0				; Get a fresh copy of Row Raw Data pointer (could be optimized out)
	move.l	d5, a1				; Get a fresh copy of Skip List pointer (could be optimized out)
	move.l	(a4)+, d0			; Get Address of Skip List Entry, absolute offset, cannot be "0" unless finished
	bne		@Repeat_Start	; If 0, exit routine, there are no entries left
	bra			@End_Of_All_Data
	nop
	nop
	nop
	nop

@Decode_Packet_44
;Decompress Technique 44 = start byte mask pattern (0000), 32 bit, 32bit word loop, end mask (1110)

	move.l (a1)+, d0					; Get dbra count for long transfer loop (it is already BASE 0!)
@Loop_Copy_Decode_Packet_44:
	move.l	(a0)+, (a2)+			; Non-optimal 32Bit transfer loop, max of 31 ever done anyway
	dbra		d0, @Loop_Copy_Decode_Packet_44

	move.w	(a0)+,(a2)+		; Copy 2 bytes of pixel data into destination
	move.b	(a0),(a2)			; Copy 1 byte of pixel data into destination

	move.l	a3, a0				; Get a fresh copy of Row Raw Data pointer (could be optimized out)
	move.l	d5, a1				; Get a fresh copy of Skip List pointer (could be optimized out)
	move.l	(a4)+, d0			; Get Address of Skip List Entry, absolute offset, cannot be "0" unless finished
	bne		@Repeat_Start	; If 0, exit routine, there are no entries left
	bra			@End_Of_All_Data
	nop
	nop
	nop
	nop
	nop
	nop
	nop


@Decode_Packet_45
;Decompress Technique 45 = start byte mask pattern (0001), 32 bit, 32bit word loop, end mask (1110)
	addq.l		#3,a0					; Adjust source address to be only 8bit aligned
	addq.l		#3,a2					; Adjust destination address to be only 8bit aligned
	move.b	(a0)+,(a2)+			; Copy 1 byte of data into destination (no masked read-in needed)

	move.l (a1)+, d0					; Get dbra count for long transfer loop (it is already BASE 0!)
@Loop_Copy_Decode_Packet_45:
	move.l	(a0)+, (a2)+			; Non-optimal 32Bit transfer loop, max of 31 ever done anyway
	dbra		d0, @Loop_Copy_Decode_Packet_45

	move.w	(a0)+,(a2)+		; Copy 2 bytes of pixel data into destination
	move.b	(a0),(a2)			; Copy 1 byte of pixel data into destination

	move.l	a3, a0				; Get a fresh copy of Row Raw Data pointer (could be optimized out)
	move.l	d5, a1				; Get a fresh copy of Skip List pointer (could be optimized out)
	move.l	(a4)+, d0			; Get Address of Skip List Entry, absolute offset, cannot be "0" unless finished
	bne		@Repeat_Start	; If 0, exit routine, there are no entries left
	bra			@End_Of_All_Data
	nop
	nop
	nop
	nop


@Decode_Packet_46
;Decompress Technique 46 = start byte mask pattern (0011), 32 bit, 32bit word loop, end mask (1110)
	addq.l		#2,a2					; Adjust destination address to be only 16bit aligned
	addq.l		#2,a0					; Adjust destination address to be only 16bit aligned
	move.w	(a0)+,(a2)+			; Copy 2 bytes of pixel data into destination

	move.l (a1)+, d0					; Get dbra count for long transfer loop (it is already BASE 0!)
@Loop_Copy_Decode_Packet_46:
	move.l	(a0)+, (a2)+			; Non-optimal 32Bit transfer loop, max of 31 ever done anyway
	dbra		d0, @Loop_Copy_Decode_Packet_46

	move.w	(a0)+,(a2)+		; Copy 2 bytes of pixel data into destination
	move.b	(a0),(a2)			; Copy 1 byte of pixel data into destination

	move.l	a3, a0				; Get a fresh copy of Row Raw Data pointer (could be optimized out)
	move.l	d5, a1				; Get a fresh copy of Skip List pointer (could be optimized out)
	move.l	(a4)+, d0			; Get Address of Skip List Entry, absolute offset, cannot be "0" unless finished
	bne		@Repeat_Start	; If 0, exit routine, there are no entries left
	bra			@End_Of_All_Data
	nop
	nop
	nop
	nop

@Decode_Packet_47
;Decompress Technique 47 = start byte mask pattern (0111), 32 bit, 32bit word loop, end mask (1110)
	addq.l		#1,a0					; Adjust source address to normality
	addq.l		#1,a2					; Adjust destination address to normality
	move.b	(a0)+,(a2)+			; Copy 1 byte of pixel data into destination
	move.w	(a0)+,(a2)+			; Copy 2 bytes of pixel data into destination

	move.l (a1)+, d0					; Get dbra count for long transfer loop (it is already BASE 0!)
@Loop_Copy_Decode_Packet_47:
	move.l	(a0)+, (a2)+			; Non-optimal 32Bit transfer loop, max of 31 ever done anyway
	dbra		d0, @Loop_Copy_Decode_Packet_47

	move.w	(a0)+,(a2)+		; Copy 2 bytes of pixel data into destination
	move.b	(a0),(a2)			; Copy 1 byte of pixel data into destination

	move.l	a3, a0				; Get a fresh copy of Row Raw Data pointer (could be optimized out)
	move.l	d5, a1				; Get a fresh copy of Skip List pointer (could be optimized out)
	move.l	(a4)+, d0			; Get Address of Skip List Entry, absolute offset, cannot be "0" unless finished
	bne		@Repeat_Start	; If 0, exit routine, there are no entries left
	bra			@End_Of_All_Data
	nop
	nop
	nop

/* ======================================================================== */
/* ======================================================================== */
/* ======================================================================== */

@Decode_Packet_48
;Decompress Technique 48 = start byte mask pattern (0000), 128 byte loop, 32bit word loop, end mask (0000)

	move.l (a1)+, d2					; Get dbra count for 128 byte transfer loop (it is already BASE 0!)

				
#if !defined(k_ASSUME_68040_ALWAYS)
#error "UNDEFINED PARAMETER!"
#endif
#if  (k_ASSUME_68040_ALWAYS == 0)
	jsr		@Loop_Copy_128_Bytes_Old_Style
#else	
	jsr		@Loop_Copy_128_Bytes_New_Style
#endif /* if  (k_ASSUME_68040_ALWAYS == 0) */

	move.l (a1)+, d0				; Get dbra count for long transfer loop (it is already BASE 0!)
@Loop_Copy_Decode_Packet_48:
	move.l	(a0)+, (a2)+		; Non-optimal 32Bit transfer loop, max of 31 ever done anyway
	dbra		d0, @Loop_Copy_Decode_Packet_48


	move.l	a3, a0				; Get a fresh copy of Row Raw Data pointer (could be optimized out)
	move.l	d5, a1				; Get a fresh copy of Skip List pointer (could be optimized out)
	move.l	(a4)+, d0			; Get Address of Skip List Entry, absolute offset, cannot be "0" unless finished
	bne		@Repeat_Start	; If 0, exit routine, there are no entries left
	bra			@End_Of_All_Data
	
	nop		; Space out to 128 bytes to allow 32 wasteful large PowerPC instructions in jump offset?
	nop		;  no... Space out to 80 bytes and have PowerPC code Shift the jump one power of 2 first.
	nop
	nop
	nop
	nop


@Decode_Packet_49
;Decompress Technique 49 = start byte mask pattern (0001), 128 byte loop, 32bit word loop, end mask (0000)
	addq.l		#3,a0					; Adjust source address to be only 8bit aligned
	addq.l		#3,a2					; Adjust destination address to be only 8bit aligned
	move.b	(a0)+,(a2)+			; Copy 1 byte of data into destination (no masked read-in needed)

	move.l (a1)+, d2					; Get dbra count for 128 byte transfer loop (it is already BASE 0!)

#if  (k_ASSUME_68040_ALWAYS == 0)
	jsr		@Loop_Copy_128_Bytes_Old_Style
#else	
	jsr		@Loop_Copy_128_Bytes_New_Style
#endif /* if  (k_ASSUME_68040_ALWAYS == 0) */

	move.l (a1)+, d0				; Get dbra count for long transfer loop (it is already BASE 0!)
@Loop_Copy_Decode_Packet_49:
	move.l	(a0)+, (a2)+		; Non-optimal 32Bit transfer loop, max of 31 ever done anyway
	dbra		d0, @Loop_Copy_Decode_Packet_49


	move.l	a3, a0				; Get a fresh copy of Row Raw Data pointer (could be optimized out)
	move.l	d5, a1				; Get a fresh copy of Skip List pointer (could be optimized out)
	move.l	(a4)+, d0			; Get Address of Skip List Entry, absolute offset, cannot be "0" unless finished
	bne		@Repeat_Start	; If 0, exit routine, there are no entries left
	bra			@End_Of_All_Data
	nop
	nop		; Space out to 128 bytes to allow 32 wasteful large PowerPC instructions in jump offset?
	nop		;  no... Space out to 80 bytes and have PowerPC code Shift the jump one power of 2 first.

@Decode_Packet_50
;Decompress Technique 50 = start byte mask pattern (0011), 128 byte loop, 32bit word loop, end mask (0000)
	addq.l		#2,a2					; Adjust destination address to be only 16bit aligned
	addq.l		#2,a0					; Adjust destination address to be only 16bit aligned
	move.w	(a0)+,(a2)+			; Copy 2 bytes of pixel data into destination

	move.l (a1)+, d2					; Get dbra count for 128 byte transfer loop (it is already BASE 0!)

#if  (k_ASSUME_68040_ALWAYS == 0)
	jsr		@Loop_Copy_128_Bytes_Old_Style
#else	
	jsr		@Loop_Copy_128_Bytes_New_Style
#endif /* if  (k_ASSUME_68040_ALWAYS == 0) */

	move.l (a1)+, d0				; Get dbra count for long transfer loop (it is already BASE 0!)
@Loop_Copy_Decode_Packet_50:
	move.l	(a0)+, (a2)+		; Non-optimal 32Bit transfer loop, max of 31 ever done anyway
	dbra		d0, @Loop_Copy_Decode_Packet_50


	move.l	a3, a0				; Get a fresh copy of Row Raw Data pointer (could be optimized out)
	move.l	d5, a1				; Get a fresh copy of Skip List pointer (could be optimized out)
	move.l	(a4)+, d0			; Get Address of Skip List Entry, absolute offset, cannot be "0" unless finished
	bne		@Repeat_Start	; If 0, exit routine, there are no entries left
	bra			@End_Of_All_Data
	nop
	nop		; Space out to 128 bytes to allow 32 wasteful large PowerPC instructions in jump offset?
	nop		;  no... Space out to 80 bytes and have PowerPC code Shift the jump one power of 2 first.


@Decode_Packet_51
;Decompress Technique 51 = start byte mask pattern (0111), 128 byte loop, 32bit word loop, end mask (0000)
	addq.l		#1,a0					; Adjust source address to normality
	addq.l		#1,a2					; Adjust destination address to normality
	move.b	(a0)+,(a2)+			; Copy 1 byte of pixel data into destination
	move.w	(a0)+,(a2)+			; Copy 2 bytes of pixel data into destination

	move.l (a1)+, d2					; Get dbra count for 128 byte transfer loop (it is already BASE 0!)

#if  (k_ASSUME_68040_ALWAYS == 0)
	jsr		@Loop_Copy_128_Bytes_Old_Style
#else	
	jsr		@Loop_Copy_128_Bytes_New_Style
#endif /* if  (k_ASSUME_68040_ALWAYS == 0) */


	move.l (a1)+, d0				; Get dbra count for long transfer loop (it is already BASE 0!)
@Loop_Copy_Decode_Packet_51:
	move.l	(a0)+, (a2)+		; Non-optimal 32Bit transfer loop, max of 31 ever done anyway
	dbra		d0, @Loop_Copy_Decode_Packet_51


	move.l	a3, a0				; Get a fresh copy of Row Raw Data pointer (could be optimized out)
	move.l	d5, a1				; Get a fresh copy of Skip List pointer (could be optimized out)
	move.l	(a4)+, d0			; Get Address of Skip List Entry, absolute offset, cannot be "0" unless finished
	bne		@Repeat_Start	; If 0, exit routine, there are no entries left
	bra			@End_Of_All_Data
	nop		; Space out to 128 bytes to allow 32 wasteful large PowerPC instructions in jump offset?
	nop		;  no... Space out to 80 bytes and have PowerPC code Shift the jump one power of 2 first.


@Decode_Packet_52
;Decompress Technique 52 = start byte mask pattern (0000), 128 byte loop, 32bit word loop, end mask (1000)

	move.l (a1)+, d2					; Get dbra count for 128 byte transfer loop (it is already BASE 0!)

#if  (k_ASSUME_68040_ALWAYS == 0)
	jsr		@Loop_Copy_128_Bytes_Old_Style
#else	
	jsr		@Loop_Copy_128_Bytes_New_Style
#endif /* if  (k_ASSUME_68040_ALWAYS == 0) */

	move.l (a1)+, d0				; Get dbra count for long transfer loop (it is already BASE 0!)
@Loop_Copy_Decode_Packet_52:
	move.l	(a0)+, (a2)+		; Non-optimal 32Bit transfer loop, max of 31 ever done anyway
	dbra		d0, @Loop_Copy_Decode_Packet_52

	move.b	(a0),(a2)				; Copy 1 byte of data into destination

	move.l	a3, a0				; Get a fresh copy of Row Raw Data pointer (could be optimized out)
	move.l	d5, a1				; Get a fresh copy of Skip List pointer (could be optimized out)
	move.l	(a4)+, d0			; Get Address of Skip List Entry, absolute offset, cannot be "0" unless finished
	bne		@Repeat_Start	; If 0, exit routine, there are no entries left
	bra			@End_Of_All_Data
	nop
	nop
	nop		; Space out to 128 bytes to allow 32 wasteful large PowerPC instructions in jump offset?
	nop		;  no... Space out to 80 bytes and have PowerPC code Shift the jump one power of 2 first.
	nop


@Decode_Packet_53
;Decompress Technique 53 = start byte mask pattern (0001), 128 byte loop, 32bit word loop, end mask (1000)
	addq.l		#3,a0					; Adjust source address to be only 8bit aligned
	addq.l		#3,a2					; Adjust destination address to be only 8bit aligned
	move.b	(a0)+,(a2)+			; Copy 1 byte of data into destination (no masked read-in needed)

	move.l (a1)+, d2					; Get dbra count for 128 byte transfer loop (it is already BASE 0!)

#if  (k_ASSUME_68040_ALWAYS == 0)
	jsr		@Loop_Copy_128_Bytes_Old_Style
#else	
	jsr		@Loop_Copy_128_Bytes_New_Style
#endif /* if  (k_ASSUME_68040_ALWAYS == 0) */

	move.l (a1)+, d0				; Get dbra count for long transfer loop (it is already BASE 0!)
@Loop_Copy_Decode_Packet_53:
	move.l	(a0)+, (a2)+		; Non-optimal 32Bit transfer loop, max of 31 ever done anyway
	dbra		d0, @Loop_Copy_Decode_Packet_53

	move.b	(a0),(a2)				; Copy 1 byte of data into destination

	move.l	a3, a0				; Get a fresh copy of Row Raw Data pointer (could be optimized out)
	move.l	d5, a1				; Get a fresh copy of Skip List pointer (could be optimized out)
	move.l	(a4)+, d0			; Get Address of Skip List Entry, absolute offset, cannot be "0" unless finished
	bne		@Repeat_Start	; If 0, exit routine, there are no entries left
	bra			@End_Of_All_Data
	nop		; Space out to 128 bytes to allow 32 wasteful large PowerPC instructions in jump offset?
	nop		;  no... Space out to 80 bytes and have PowerPC code Shift the jump one power of 2 first.

@Decode_Packet_54
;Decompress Technique 54 = start byte mask pattern (0011), 128 byte loop, 32bit word loop, end mask (1000)
	addq.l		#2,a2					; Adjust destination address to be only 16bit aligned
	addq.l		#2,a0					; Adjust destination address to be only 16bit aligned
	move.w	(a0)+,(a2)+			; Copy 2 bytes of pixel data into destination

	move.l (a1)+, d2					; Get dbra count for 128 byte transfer loop (it is already BASE 0!)

#if  (k_ASSUME_68040_ALWAYS == 0)
	jsr		@Loop_Copy_128_Bytes_Old_Style
#else	
	jsr		@Loop_Copy_128_Bytes_New_Style
#endif /* if  (k_ASSUME_68040_ALWAYS == 0) */

	move.l (a1)+, d0				; Get dbra count for long transfer loop (it is already BASE 0!)
@Loop_Copy_Decode_Packet_54:
	move.l	(a0)+, (a2)+		; Non-optimal 32Bit transfer loop, max of 31 ever done anyway
	dbra		d0, @Loop_Copy_Decode_Packet_54

	move.b	(a0),(a2)				; Copy 1 byte of data into destination

	move.l	a3, a0				; Get a fresh copy of Row Raw Data pointer (could be optimized out)
	move.l	d5, a1				; Get a fresh copy of Skip List pointer (could be optimized out)
	move.l	(a4)+, d0			; Get Address of Skip List Entry, absolute offset, cannot be "0" unless finished
	bne		@Repeat_Start	; If 0, exit routine, there are no entries left
	bra			@End_Of_All_Data
	nop
	nop


@Decode_Packet_55
;Decompress Technique 55 = start byte mask pattern (0111), 128 byte loop, 32bit word loop, end mask (1000)
	addq.l		#1,a0					; Adjust source address to normality
	addq.l		#1,a2					; Adjust destination address to normality
	move.b	(a0)+,(a2)+			; Copy 1 byte of pixel data into destination
	move.w	(a0)+,(a2)+			; Copy 2 bytes of pixel data into destination

	move.l (a1)+, d2					; Get dbra count for 128 byte transfer loop (it is already BASE 0!)

#if  (k_ASSUME_68040_ALWAYS == 0)
	jsr		@Loop_Copy_128_Bytes_Old_Style
#else	
	jsr		@Loop_Copy_128_Bytes_New_Style
#endif /* if  (k_ASSUME_68040_ALWAYS == 0) */

	move.l (a1)+, d0				; Get dbra count for long transfer loop (it is already BASE 0!)
@Loop_Copy_Decode_Packet_55:
	move.l	(a0)+, (a2)+		; Non-optimal 32Bit transfer loop, max of 31 ever done anyway
	dbra		d0, @Loop_Copy_Decode_Packet_55

	move.b	(a0),(a2)				; Copy 1 byte of data into destination

	move.l	a3, a0				; Get a fresh copy of Row Raw Data pointer (could be optimized out)
	move.l	d5, a1				; Get a fresh copy of Skip List pointer (could be optimized out)
	move.l	(a4)+, d0			; Get Address of Skip List Entry, absolute offset, cannot be "0" unless finished
	bne		@Repeat_Start	; If 0, exit routine, there are no entries left
	bra			@End_Of_All_Data
	nop


@Decode_Packet_56
;Decompress Technique 56 = start byte mask pattern (0000), 128 byte loop, 32bit word loop, end mask (1100)

	move.l (a1)+, d2					; Get dbra count for 128 byte transfer loop (it is already BASE 0!)

#if  (k_ASSUME_68040_ALWAYS == 0)
	jsr		@Loop_Copy_128_Bytes_Old_Style
#else	
	jsr		@Loop_Copy_128_Bytes_New_Style
#endif /* if  (k_ASSUME_68040_ALWAYS == 0) */

	move.l (a1)+, d0					; Get dbra count for long transfer loop (it is already BASE 0!)
@Loop_Copy_Decode_Packet_56:
	move.l	(a0)+, (a2)+			; Non-optimal 32Bit transfer loop, max of 31 ever done anyway
	dbra		d0, @Loop_Copy_Decode_Packet_56

	move.w	(a0),(a2)				; Copy 2 bytes of data into destination

	move.l	a3, a0				; Get a fresh copy of Row Raw Data pointer (could be optimized out)
	move.l	d5, a1				; Get a fresh copy of Skip List pointer (could be optimized out)
	move.l	(a4)+, d0			; Get Address of Skip List Entry, absolute offset, cannot be "0" unless finished
	bne		@Repeat_Start	; If 0, exit routine, there are no entries left
	bra			@End_Of_All_Data
	nop
	nop
	nop
	nop
	nop


@Decode_Packet_57
;Decompress Technique 57 = start byte mask pattern (0001), 128 byte loop, 32bit word loop, end mask (1100)
	addq.l		#3,a0					; Adjust source address to be only 8bit aligned
	addq.l		#3,a2					; Adjust destination address to be only 8bit aligned
	move.b	(a0)+,(a2)+			; Copy 1 byte of data into destination (no masked read-in needed)

	move.l (a1)+, d2					; Get dbra count for 128 byte transfer loop (it is already BASE 0!)

#if  (k_ASSUME_68040_ALWAYS == 0)
	jsr		@Loop_Copy_128_Bytes_Old_Style
#else	
	jsr		@Loop_Copy_128_Bytes_New_Style
#endif /* if  (k_ASSUME_68040_ALWAYS == 0) */

	move.l (a1)+, d0					; Get dbra count for long transfer loop (it is already BASE 0!)
@Loop_Copy_Decode_Packet_57:
	move.l	(a0)+, (a2)+			; Non-optimal 32Bit transfer loop, max of 31 ever done anyway
	dbra		d0, @Loop_Copy_Decode_Packet_57

	move.w	(a0),(a2)				; Copy 2 bytes of data into destination

	move.l	a3, a0				; Get a fresh copy of Row Raw Data pointer (could be optimized out)
	move.l	d5, a1				; Get a fresh copy of Skip List pointer (could be optimized out)
	move.l	(a4)+, d0			; Get Address of Skip List Entry, absolute offset, cannot be "0" unless finished
	bne		@Repeat_Start	; If 0, exit routine, there are no entries left
	bra			@End_Of_All_Data
	nop
	nop


@Decode_Packet_58
;Decompress Technique 58 = start byte mask pattern (0011), 128 byte loop, 32bit word loop, end mask (1100)
	addq.l		#2,a2					; Adjust destination address to be only 16bit aligned
	addq.l		#2,a0					; Adjust destination address to be only 16bit aligned
	move.w	(a0)+,(a2)+			; Copy 2 bytes of pixel data into destination

	move.l (a1)+, d2					; Get dbra count for 128 byte transfer loop (it is already BASE 0!)

#if  (k_ASSUME_68040_ALWAYS == 0)
	jsr		@Loop_Copy_128_Bytes_Old_Style
#else	
	jsr		@Loop_Copy_128_Bytes_New_Style
#endif /* if  (k_ASSUME_68040_ALWAYS == 0) */

	move.l (a1)+, d0					; Get dbra count for long transfer loop (it is already BASE 0!)
@Loop_Copy_Decode_Packet_58:
	move.l	(a0)+, (a2)+			; Non-optimal 32Bit transfer loop, max of 31 ever done anyway
	dbra		d0, @Loop_Copy_Decode_Packet_58

	move.w	(a0),(a2)				; Copy 2 bytes of data into destination

	move.l	a3, a0				; Get a fresh copy of Row Raw Data pointer (could be optimized out)
	move.l	d5, a1				; Get a fresh copy of Skip List pointer (could be optimized out)
	move.l	(a4)+, d0			; Get Address of Skip List Entry, absolute offset, cannot be "0" unless finished
	bne		@Repeat_Start	; If 0, exit routine, there are no entries left
	bra			@End_Of_All_Data
	nop
	nop


@Decode_Packet_59
;Decompress Technique 59 = start byte mask pattern (0111), 128 byte loop, 32bit word loop, end mask (1100)
	addq.l		#1,a0					; Adjust source address to normality
	addq.l		#1,a2					; Adjust destination address to normality
	move.b	(a0)+,(a2)+			; Copy 1 byte of pixel data into destination
	move.w	(a0)+,(a2)+			; Copy 2 bytes of pixel data into destination

	move.l (a1)+, d2					; Get dbra count for 128 byte transfer loop (it is already BASE 0!)

#if  (k_ASSUME_68040_ALWAYS == 0)
	jsr		@Loop_Copy_128_Bytes_Old_Style
#else	
	jsr		@Loop_Copy_128_Bytes_New_Style
#endif /* if  (k_ASSUME_68040_ALWAYS == 0) */

	move.l (a1)+, d0					; Get dbra count for long transfer loop (it is already BASE 0!)
@Loop_Copy_Decode_Packet_59:
	move.l	(a0)+, (a2)+			; Non-optimal 32Bit transfer loop, max of 31 ever done anyway
	dbra		d0, @Loop_Copy_Decode_Packet_59

	move.w	(a0),(a2)				; Copy 2 bytes of data into destination

	move.l	a3, a0				; Get a fresh copy of Row Raw Data pointer (could be optimized out)
	move.l	d5, a1				; Get a fresh copy of Skip List pointer (could be optimized out)
	move.l	(a4)+, d0			; Get Address of Skip List Entry, absolute offset, cannot be "0" unless finished
	bne		@Repeat_Start	; If 0, exit routine, there are no entries left
	bra			@End_Of_All_Data
	nop


@Decode_Packet_60
;Decompress Technique 60 = start byte mask pattern (0000), 128 byte loop, 32bit word loop, end mask (1110)

	move.l (a1)+, d2					; Get dbra count for 128 byte transfer loop (it is already BASE 0!)

#if  (k_ASSUME_68040_ALWAYS == 0)
	jsr		@Loop_Copy_128_Bytes_Old_Style
#else	
	jsr		@Loop_Copy_128_Bytes_New_Style
#endif /* if  (k_ASSUME_68040_ALWAYS == 0) */

	move.l (a1)+, d0					; Get dbra count for long transfer loop (it is already BASE 0!)
@Loop_Copy_Decode_Packet_60:
	move.l	(a0)+, (a2)+			; Non-optimal 32Bit transfer loop, max of 31 ever done anyway
	dbra		d0, @Loop_Copy_Decode_Packet_60

	move.w	(a0)+,(a2)+		; Copy 2 bytes of pixel data into destination
	move.b	(a0),(a2)			; Copy 1 byte of pixel data into destination

	move.l	a3, a0				; Get a fresh copy of Row Raw Data pointer (could be optimized out)
	move.l	d5, a1				; Get a fresh copy of Skip List pointer (could be optimized out)
	move.l	(a4)+, d0			; Get Address of Skip List Entry, absolute offset, cannot be "0" unless finished
	bne		@Repeat_Start	; If 0, exit routine, there are no entries left
	bra			@End_Of_All_Data
	nop
	nop
	nop
	nop


@Decode_Packet_61
;Decompress Technique 61 = start byte mask pattern (0001), 128 byte loop, 32bit word loop, end mask (1110)
	addq.l		#3,a0					; Adjust source address to be only 8bit aligned
	addq.l		#3,a2					; Adjust destination address to be only 8bit aligned
	move.b	(a0)+,(a2)+			; Copy 1 byte of data into destination (no masked read-in needed)

	move.l (a1)+, d2					; Get dbra count for 128 byte transfer loop (it is already BASE 0!)

#if  (k_ASSUME_68040_ALWAYS == 0)
	jsr		@Loop_Copy_128_Bytes_Old_Style
#else	
	jsr		@Loop_Copy_128_Bytes_New_Style
#endif /* if  (k_ASSUME_68040_ALWAYS == 0) */

	move.l (a1)+, d0					; Get dbra count for long transfer loop (it is already BASE 0!)
@Loop_Copy_Decode_Packet_61:
	move.l	(a0)+, (a2)+			; Non-optimal 32Bit transfer loop, max of 31 ever done anyway
	dbra		d0, @Loop_Copy_Decode_Packet_61

	move.w	(a0)+,(a2)+		; Copy 2 bytes of pixel data into destination
	move.b	(a0),(a2)			; Copy 1 byte of pixel data into destination

	move.l	a3, a0				; Get a fresh copy of Row Raw Data pointer (could be optimized out)
	move.l	d5, a1				; Get a fresh copy of Skip List pointer (could be optimized out)
	move.l	(a4)+, d0			; Get Address of Skip List Entry, absolute offset, cannot be "0" unless finished
	bne		@Repeat_Start	; If 0, exit routine, there are no entries left
	bra			@End_Of_All_Data
	nop



@Decode_Packet_62
;Decompress Technique 62 = start byte mask pattern (0011), 128 byte loop, 32bit word loop, end mask (1110)
	addq.l		#2,a2					; Adjust destination address to be only 16bit aligned
	addq.l		#2,a0					; Adjust destination address to be only 16bit aligned
	move.w	(a0)+,(a2)+			; Copy 2 bytes of pixel data into destination

	move.l (a1)+, d2					; Get dbra count for 128 byte transfer loop (it is already BASE 0!)

#if  (k_ASSUME_68040_ALWAYS == 0)
	jsr		@Loop_Copy_128_Bytes_Old_Style
#else	
	jsr		@Loop_Copy_128_Bytes_New_Style
#endif /* if  (k_ASSUME_68040_ALWAYS == 0) */

	move.l (a1)+, d0					; Get dbra count for long transfer loop (it is already BASE 0!)
@Loop_Copy_Decode_Packet_62:
	move.l	(a0)+, (a2)+			; Non-optimal 32Bit transfer loop, max of 31 ever done anyway
	dbra		d0, @Loop_Copy_Decode_Packet_62

	move.w	(a0)+,(a2)+		; Copy 2 bytes of pixel data into destination
	move.b	(a0),(a2)			; Copy 1 byte of pixel data into destination

	move.l	a3, a0				; Get a fresh copy of Row Raw Data pointer (could be optimized out)
	move.l	d5, a1				; Get a fresh copy of Skip List pointer (could be optimized out)
	move.l	(a4)+, d0			; Get Address of Skip List Entry, absolute offset, cannot be "0" unless finished
	bne		@Repeat_Start	; If 0, exit routine, there are no entries left
	bra			@End_Of_All_Data
	nop


@Decode_Packet_63
;Decompress Technique 63 = start byte mask pattern (0111), 128 byte loop, 32bit word loop, end mask (1110)
	addq.l		#1,a0					; Adjust source address to normality
	addq.l		#1,a2					; Adjust destination address to normality
	move.b	(a0)+,(a2)+			; Copy 1 byte of pixel data into destination
	move.w	(a0)+,(a2)+			; Copy 2 bytes of pixel data into destination

	move.l (a1)+, d2					; Get dbra count for 128 byte transfer loop (it is already BASE 0!)

#if  (k_ASSUME_68040_ALWAYS == 0)
	jsr		@Loop_Copy_128_Bytes_Old_Style
#else	
	jsr		@Loop_Copy_128_Bytes_New_Style
#endif /* if  (k_ASSUME_68040_ALWAYS == 0) */

	move.l (a1)+, d0					; Get dbra count for long transfer loop (it is already BASE 0!)

@Loop_Copy_Decode_Packet_63:
	move.l	(a0)+, (a2)+			; Non-optimal 32Bit transfer loop, max of 31 ever done anyway
	dbra		d0, @Loop_Copy_Decode_Packet_63

	move.w	(a0)+,(a2)+		; Copy 2 bytes of pixel data into destination
	move.b	(a0),(a2)			; Copy 1 byte of pixel data into destination

	move.l	a3, a0				; Get a fresh copy of Row Raw Data pointer (could be optimized out)
	move.l	d5, a1				; Get a fresh copy of Skip List pointer (could be optimized out)
	move.l	(a4)+, d0			; Get Address of Skip List Entry, absolute offset, cannot be "0" unless finished
	bne		@Repeat_Start	; If 0, exit routine, there are no entries left
	bra			@End_Of_All_Data
	
	
; HACK HACK HACK	nop 1995.04.06 JAM removed

/* ======================================================================== */
/* ======================================================================== */
/* ======================================================================== */


@Decode_Packet_64
;Decompress Technique 64 = start byte mask pattern (0000), 32 to 96 bits, 128 byte loop, 32bit word loop, end mask (0000)

	move.l (a1)+, d0				; Get dbra count for long transfer loop (it is already BASE 0!)
@Loop_Align_For_Cache_Line_64:
	move.l	(a0)+, (a2)+		; 32Bit transfer loop, max of 3 ever done, achieves 128 bit alignment of source
	dbra		d0, @Loop_Align_For_Cache_Line_64

	move.l (a1)+, d2					; Get dbra count for 128 byte transfer loop (it is already BASE 0!)

#if !defined(k_ASSUME_68040_ALWAYS)
#error "UNDEFINED PARAMETER!"
#endif
#if  (k_ASSUME_68040_ALWAYS == 0)
	jsr		@Loop_Copy_128_Bytes_Old_Style
#else	
	jsr		@Loop_Copy_128_Bytes_New_Style
#endif /* if  (k_ASSUME_68040_ALWAYS == 0) */

	move.l (a1)+, d0				; Get dbra count for long transfer loop (it is already BASE 0!)
@Loop_Copy_Decode_Packet_64:
	move.l	(a0)+, (a2)+		; Non-optimal 32Bit transfer loop, max of 31 ever done anyway
	dbra		d0, @Loop_Copy_Decode_Packet_64


	move.l	a3, a0				; Get a fresh copy of Row Raw Data pointer (could be optimized out)
	move.l	d5, a1				; Get a fresh copy of Skip List pointer (could be optimized out)
	move.l	(a4)+, d0			; Get Address of Skip List Entry, absolute offset, cannot be "0" unless finished
	bne		@Repeat_Start	; If 0, exit routine, there are no entries left
	bra			@End_Of_All_Data
	
	nop		; Space out to 128 bytes to allow 32 wasteful large PowerPC instructions in jump offset?
	nop		;  no... Space out to 80 bytes and have PowerPC code Shift the jump one power of 2 first.
	nop
	nop
	nop
	nop


@Decode_Packet_65
;Decompress Technique 65 = start byte mask pattern (0001), 32 to 96 bits, 128 byte loop, 32bit word loop, end mask (0000)
	addq.l		#3,a0					; Adjust source address to be only 8bit aligned
	addq.l		#3,a2					; Adjust destination address to be only 8bit aligned
	move.b	(a0)+,(a2)+			; Copy 1 byte of data into destination (no masked read-in needed)

	move.l (a1)+, d0				; Get dbra count for long transfer loop (it is already BASE 0!)
@Loop_Align_For_Cache_Line_65:
	move.l	(a0)+, (a2)+		; 32Bit transfer loop, max of 3 ever done, achieves 128 bit alignment of source
	dbra		d0, @Loop_Align_For_Cache_Line_65

	move.l (a1)+, d2					; Get dbra count for 128 byte transfer loop (it is already BASE 0!)

#if  (k_ASSUME_68040_ALWAYS == 0)
	jsr		@Loop_Copy_128_Bytes_Old_Style
#else	
	jsr		@Loop_Copy_128_Bytes_New_Style
#endif /* if  (k_ASSUME_68040_ALWAYS == 0) */

	move.l (a1)+, d0				; Get dbra count for long transfer loop (it is already BASE 0!)
@Loop_Copy_Decode_Packet_65:
	move.l	(a0)+, (a2)+		; Non-optimal 32Bit transfer loop, max of 31 ever done anyway
	dbra		d0, @Loop_Copy_Decode_Packet_65


	move.l	a3, a0				; Get a fresh copy of Row Raw Data pointer (could be optimized out)
	move.l	d5, a1				; Get a fresh copy of Skip List pointer (could be optimized out)
	move.l	(a4)+, d0			; Get Address of Skip List Entry, absolute offset, cannot be "0" unless finished
	bne		@Repeat_Start	; If 0, exit routine, there are no entries left
	bra			@End_Of_All_Data
	nop
	nop		; Space out to 128 bytes to allow 32 wasteful large PowerPC instructions in jump offset?
	nop		;  no... Space out to 80 bytes and have PowerPC code Shift the jump one power of 2 first.

@Decode_Packet_66
;Decompress Technique 66 = start byte mask pattern (0011), 32 to 96 bits, 128 byte loop, 32bit word loop, end mask (0000)
	addq.l		#2,a2					; Adjust destination address to be only 16bit aligned
	addq.l		#2,a0					; Adjust destination address to be only 16bit aligned
	move.w	(a0)+,(a2)+			; Copy 2 bytes of pixel data into destination

	move.l (a1)+, d0				; Get dbra count for long transfer loop (it is already BASE 0!)
@Loop_Align_For_Cache_Line_66:
	move.l	(a0)+, (a2)+		; 32Bit transfer loop, max of 3 ever done, achieves 128 bit alignment of source
	dbra		d0, @Loop_Align_For_Cache_Line_66

	move.l (a1)+, d2					; Get dbra count for 128 byte transfer loop (it is already BASE 0!)

#if  (k_ASSUME_68040_ALWAYS == 0)
	jsr		@Loop_Copy_128_Bytes_Old_Style
#else	
	jsr		@Loop_Copy_128_Bytes_New_Style
#endif /* if  (k_ASSUME_68040_ALWAYS == 0) */

	move.l (a1)+, d0				; Get dbra count for long transfer loop (it is already BASE 0!)
@Loop_Copy_Decode_Packet_66:
	move.l	(a0)+, (a2)+		; Non-optimal 32Bit transfer loop, max of 31 ever done anyway
	dbra		d0, @Loop_Copy_Decode_Packet_66


	move.l	a3, a0				; Get a fresh copy of Row Raw Data pointer (could be optimized out)
	move.l	d5, a1				; Get a fresh copy of Skip List pointer (could be optimized out)
	move.l	(a4)+, d0			; Get Address of Skip List Entry, absolute offset, cannot be "0" unless finished
	bne		@Repeat_Start	; If 0, exit routine, there are no entries left
	bra			@End_Of_All_Data
	nop
	nop		; Space out to 128 bytes to allow 32 wasteful large PowerPC instructions in jump offset?
	nop		;  no... Space out to 80 bytes and have PowerPC code Shift the jump one power of 2 first.


@Decode_Packet_67
;Decompress Technique 67 = start byte mask pattern (0111), 32 to 96 bits, 128 byte loop, 32bit word loop, end mask (0000)
	addq.l		#1,a0					; Adjust source address to normality
	addq.l		#1,a2					; Adjust destination address to normality
	move.b	(a0)+,(a2)+			; Copy 1 byte of pixel data into destination
	move.w	(a0)+,(a2)+			; Copy 2 bytes of pixel data into destination

	move.l (a1)+, d0				; Get dbra count for long transfer loop (it is already BASE 0!)
@Loop_Align_For_Cache_Line_67:
	move.l	(a0)+, (a2)+		; 32Bit transfer loop, max of 3 ever done, achieves 128 bit alignment of source
	dbra		d0, @Loop_Align_For_Cache_Line_67

	move.l (a1)+, d2					; Get dbra count for 128 byte transfer loop (it is already BASE 0!)

#if  (k_ASSUME_68040_ALWAYS == 0)
	jsr		@Loop_Copy_128_Bytes_Old_Style
#else	
	jsr		@Loop_Copy_128_Bytes_New_Style
#endif /* if  (k_ASSUME_68040_ALWAYS == 0) */


	move.l (a1)+, d0				; Get dbra count for long transfer loop (it is already BASE 0!)
@Loop_Copy_Decode_Packet_67:
	move.l	(a0)+, (a2)+		; Non-optimal 32Bit transfer loop, max of 31 ever done anyway
	dbra		d0, @Loop_Copy_Decode_Packet_67


	move.l	a3, a0				; Get a fresh copy of Row Raw Data pointer (could be optimized out)
	move.l	d5, a1				; Get a fresh copy of Skip List pointer (could be optimized out)
	move.l	(a4)+, d0			; Get Address of Skip List Entry, absolute offset, cannot be "0" unless finished
	bne		@Repeat_Start	; If 0, exit routine, there are no entries left
	bra			@End_Of_All_Data
	nop		; Space out to 128 bytes to allow 32 wasteful large PowerPC instructions in jump offset?
	nop		;  no... Space out to 80 bytes and have PowerPC code Shift the jump one power of 2 first.


@Decode_Packet_68
;Decompress Technique 68 = start byte mask pattern (0000), 32 to 96 bits, 128 byte loop, 32bit word loop, end mask (1000)

	move.l (a1)+, d0				; Get dbra count for long transfer loop (it is already BASE 0!)
@Loop_Align_For_Cache_Line_68:
	move.l	(a0)+, (a2)+		; 32Bit transfer loop, max of 3 ever done, achieves 128 bit alignment of source
	dbra		d0, @Loop_Align_For_Cache_Line_68

	move.l (a1)+, d2					; Get dbra count for 128 byte transfer loop (it is already BASE 0!)

#if  (k_ASSUME_68040_ALWAYS == 0)
	jsr		@Loop_Copy_128_Bytes_Old_Style
#else	
	jsr		@Loop_Copy_128_Bytes_New_Style
#endif /* if  (k_ASSUME_68040_ALWAYS == 0) */

	move.l (a1)+, d0				; Get dbra count for long transfer loop (it is already BASE 0!)
@Loop_Copy_Decode_Packet_68:
	move.l	(a0)+, (a2)+		; Non-optimal 32Bit transfer loop, max of 31 ever done anyway
	dbra		d0, @Loop_Copy_Decode_Packet_68

	move.b	(a0),(a2)				; Copy 1 byte of data into destination

	move.l	a3, a0				; Get a fresh copy of Row Raw Data pointer (could be optimized out)
	move.l	d5, a1				; Get a fresh copy of Skip List pointer (could be optimized out)
	move.l	(a4)+, d0			; Get Address of Skip List Entry, absolute offset, cannot be "0" unless finished
	bne		@Repeat_Start	; If 0, exit routine, there are no entries left
	bra			@End_Of_All_Data
	nop
	nop
	nop		; Space out to 128 bytes to allow 32 wasteful large PowerPC instructions in jump offset?
	nop		;  no... Space out to 80 bytes and have PowerPC code Shift the jump one power of 2 first.
	nop


@Decode_Packet_69
;Decompress Technique 69 = start byte mask pattern (0001), 32 to 96 bits, 128 byte loop, 32bit word loop, end mask (1000)
	addq.l		#3,a0					; Adjust source address to be only 8bit aligned
	addq.l		#3,a2					; Adjust destination address to be only 8bit aligned
	move.b	(a0)+,(a2)+			; Copy 1 byte of data into destination (no masked read-in needed)

	move.l (a1)+, d0				; Get dbra count for long transfer loop (it is already BASE 0!)
@Loop_Align_For_Cache_Line_69:
	move.l	(a0)+, (a2)+		; 32Bit transfer loop, max of 3 ever done, achieves 128 bit alignment of source
	dbra		d0, @Loop_Align_For_Cache_Line_69

	move.l (a1)+, d2					; Get dbra count for 128 byte transfer loop (it is already BASE 0!)

#if  (k_ASSUME_68040_ALWAYS == 0)
	jsr		@Loop_Copy_128_Bytes_Old_Style
#else	
	jsr		@Loop_Copy_128_Bytes_New_Style
#endif /* if  (k_ASSUME_68040_ALWAYS == 0) */

	move.l (a1)+, d0				; Get dbra count for long transfer loop (it is already BASE 0!)
@Loop_Copy_Decode_Packet_69:
	move.l	(a0)+, (a2)+		; Non-optimal 32Bit transfer loop, max of 31 ever done anyway
	dbra		d0, @Loop_Copy_Decode_Packet_69

	move.b	(a0),(a2)				; Copy 1 byte of data into destination

	move.l	a3, a0				; Get a fresh copy of Row Raw Data pointer (could be optimized out)
	move.l	d5, a1				; Get a fresh copy of Skip List pointer (could be optimized out)
	move.l	(a4)+, d0			; Get Address of Skip List Entry, absolute offset, cannot be "0" unless finished
	bne		@Repeat_Start	; If 0, exit routine, there are no entries left
	bra			@End_Of_All_Data
	nop		; Space out to 128 bytes to allow 32 wasteful large PowerPC instructions in jump offset?
	nop		;  no... Space out to 80 bytes and have PowerPC code Shift the jump one power of 2 first.


@Decode_Packet_70
;Decompress Technique 70 = start byte mask pattern (0011), 32 to 96 bits, 128 byte loop, 32bit word loop, end mask (1000)
	addq.l		#2,a2					; Adjust destination address to be only 16bit aligned
	addq.l		#2,a0					; Adjust destination address to be only 16bit aligned
	move.w	(a0)+,(a2)+			; Copy 2 bytes of pixel data into destination

	move.l (a1)+, d0				; Get dbra count for long transfer loop (it is already BASE 0!)
@Loop_Align_For_Cache_Line_70:
	move.l	(a0)+, (a2)+		; 32Bit transfer loop, max of 3 ever done, achieves 128 bit alignment of source
	dbra		d0, @Loop_Align_For_Cache_Line_70

	move.l (a1)+, d2					; Get dbra count for 128 byte transfer loop (it is already BASE 0!)

#if  (k_ASSUME_68040_ALWAYS == 0)
	jsr		@Loop_Copy_128_Bytes_Old_Style
#else	
	jsr		@Loop_Copy_128_Bytes_New_Style
#endif /* if  (k_ASSUME_68040_ALWAYS == 0) */

	move.l (a1)+, d0				; Get dbra count for long transfer loop (it is already BASE 0!)
@Loop_Copy_Decode_Packet_70:
	move.l	(a0)+, (a2)+		; Non-optimal 32Bit transfer loop, max of 31 ever done anyway
	dbra		d0, @Loop_Copy_Decode_Packet_70

	move.b	(a0),(a2)				; Copy 1 byte of data into destination

	move.l	a3, a0				; Get a fresh copy of Row Raw Data pointer (could be optimized out)
	move.l	d5, a1				; Get a fresh copy of Skip List pointer (could be optimized out)
	move.l	(a4)+, d0			; Get Address of Skip List Entry, absolute offset, cannot be "0" unless finished
	bne		@Repeat_Start	; If 0, exit routine, there are no entries left
	bra			@End_Of_All_Data
	nop
	nop


@Decode_Packet_71
;Decompress Technique 71 = start byte mask pattern (0111), 32 to 96 bits, 128 byte loop, 32bit word loop, end mask (1000)
	addq.l		#1,a0					; Adjust source address to normality
	addq.l		#1,a2					; Adjust destination address to normality
	move.b	(a0)+,(a2)+			; Copy 1 byte of pixel data into destination
	move.w	(a0)+,(a2)+			; Copy 2 bytes of pixel data into destination

	move.l (a1)+, d0				; Get dbra count for long transfer loop (it is already BASE 0!)
@Loop_Align_For_Cache_Line_71:
	move.l	(a0)+, (a2)+		; 32Bit transfer loop, max of 3 ever done, achieves 128 bit alignment of source
	dbra		d0, @Loop_Align_For_Cache_Line_71

	move.l (a1)+, d2					; Get dbra count for 128 byte transfer loop (it is already BASE 0!)

#if  (k_ASSUME_68040_ALWAYS == 0)
	jsr		@Loop_Copy_128_Bytes_Old_Style
#else	
	jsr		@Loop_Copy_128_Bytes_New_Style
#endif /* if  (k_ASSUME_68040_ALWAYS == 0) */

	move.l (a1)+, d0				; Get dbra count for long transfer loop (it is already BASE 0!)
@Loop_Copy_Decode_Packet_71:
	move.l	(a0)+, (a2)+		; Non-optimal 32Bit transfer loop, max of 31 ever done anyway
	dbra		d0, @Loop_Copy_Decode_Packet_71

	move.b	(a0),(a2)				; Copy 1 byte of data into destination

	move.l	a3, a0				; Get a fresh copy of Row Raw Data pointer (could be optimized out)
	move.l	d5, a1				; Get a fresh copy of Skip List pointer (could be optimized out)
	move.l	(a4)+, d0			; Get Address of Skip List Entry, absolute offset, cannot be "0" unless finished
	bne		@Repeat_Start	; If 0, exit routine, there are no entries left
	bra			@End_Of_All_Data
	nop


@Decode_Packet_72
;Decompress Technique 72 = start byte mask pattern (0000), 32 to 96 bits, 128 byte loop, 32bit word loop, end mask (1100)

	move.l (a1)+, d0				; Get dbra count for long transfer loop (it is already BASE 0!)
@Loop_Align_For_Cache_Line_72:
	move.l	(a0)+, (a2)+		; 32Bit transfer loop, max of 3 ever done, achieves 128 bit alignment of source
	dbra		d0, @Loop_Align_For_Cache_Line_72

	move.l (a1)+, d2					; Get dbra count for 128 byte transfer loop (it is already BASE 0!)

#if  (k_ASSUME_68040_ALWAYS == 0)
	jsr		@Loop_Copy_128_Bytes_Old_Style
#else	
	jsr		@Loop_Copy_128_Bytes_New_Style
#endif /* if  (k_ASSUME_68040_ALWAYS == 0) */

	move.l (a1)+, d0					; Get dbra count for long transfer loop (it is already BASE 0!)
@Loop_Copy_Decode_Packet_72:
	move.l	(a0)+, (a2)+			; Non-optimal 32Bit transfer loop, max of 31 ever done anyway
	dbra		d0, @Loop_Copy_Decode_Packet_72

	move.w	(a0),(a2)				; Copy 2 bytes of data into destination

	move.l	a3, a0				; Get a fresh copy of Row Raw Data pointer (could be optimized out)
	move.l	d5, a1				; Get a fresh copy of Skip List pointer (could be optimized out)
	move.l	(a4)+, d0			; Get Address of Skip List Entry, absolute offset, cannot be "0" unless finished
	bne		@Repeat_Start	; If 0, exit routine, there are no entries left
	bra			@End_Of_All_Data
	nop
	nop
	nop
	nop
	nop


@Decode_Packet_73
;Decompress Technique 73 = start byte mask pattern (0001), 32 to 96 bits, 128 byte loop, 32bit word loop, end mask (1100)
	addq.l		#3,a0					; Adjust source address to be only 8bit aligned
	addq.l		#3,a2					; Adjust destination address to be only 8bit aligned
	move.b	(a0)+,(a2)+			; Copy 1 byte of data into destination (no masked read-in needed)

	move.l (a1)+, d0				; Get dbra count for long transfer loop (it is already BASE 0!)
@Loop_Align_For_Cache_Line_73:
	move.l	(a0)+, (a2)+		; 32Bit transfer loop, max of 3 ever done, achieves 128 bit alignment of source
	dbra		d0, @Loop_Align_For_Cache_Line_73

	move.l (a1)+, d2					; Get dbra count for 128 byte transfer loop (it is already BASE 0!)

#if  (k_ASSUME_68040_ALWAYS == 0)
	jsr		@Loop_Copy_128_Bytes_Old_Style
#else	
	jsr		@Loop_Copy_128_Bytes_New_Style
#endif /* if  (k_ASSUME_68040_ALWAYS == 0) */

	move.l (a1)+, d0					; Get dbra count for long transfer loop (it is already BASE 0!)
@Loop_Copy_Decode_Packet_73:
	move.l	(a0)+, (a2)+			; Non-optimal 32Bit transfer loop, max of 31 ever done anyway
	dbra		d0, @Loop_Copy_Decode_Packet_73

	move.w	(a0),(a2)				; Copy 2 bytes of data into destination

	move.l	a3, a0				; Get a fresh copy of Row Raw Data pointer (could be optimized out)
	move.l	d5, a1				; Get a fresh copy of Skip List pointer (could be optimized out)
	move.l	(a4)+, d0			; Get Address of Skip List Entry, absolute offset, cannot be "0" unless finished
	bne		@Repeat_Start	; If 0, exit routine, there are no entries left
	bra			@End_Of_All_Data
	nop
	nop


@Decode_Packet_74
;Decompress Technique 74 = start byte mask pattern (0011), 32 to 96 bits, 128 byte loop, 32bit word loop, end mask (1100)
	addq.l		#2,a2					; Adjust destination address to be only 16bit aligned
	addq.l		#2,a0					; Adjust destination address to be only 16bit aligned
	move.w	(a0)+,(a2)+			; Copy 2 bytes of pixel data into destination

	move.l (a1)+, d0				; Get dbra count for long transfer loop (it is already BASE 0!)
@Loop_Align_For_Cache_Line_74:
	move.l	(a0)+, (a2)+		; 32Bit transfer loop, max of 3 ever done, achieves 128 bit alignment of source
	dbra		d0, @Loop_Align_For_Cache_Line_74

	move.l (a1)+, d2					; Get dbra count for 128 byte transfer loop (it is already BASE 0!)

#if  (k_ASSUME_68040_ALWAYS == 0)
	jsr		@Loop_Copy_128_Bytes_Old_Style
#else	
	jsr		@Loop_Copy_128_Bytes_New_Style
#endif /* if  (k_ASSUME_68040_ALWAYS == 0) */

	move.l (a1)+, d0					; Get dbra count for long transfer loop (it is already BASE 0!)
@Loop_Copy_Decode_Packet_74:
	move.l	(a0)+, (a2)+			; Non-optimal 32Bit transfer loop, max of 31 ever done anyway
	dbra		d0, @Loop_Copy_Decode_Packet_74

	move.w	(a0),(a2)				; Copy 2 bytes of data into destination

	move.l	a3, a0				; Get a fresh copy of Row Raw Data pointer (could be optimized out)
	move.l	d5, a1				; Get a fresh copy of Skip List pointer (could be optimized out)
	move.l	(a4)+, d0			; Get Address of Skip List Entry, absolute offset, cannot be "0" unless finished
	bne		@Repeat_Start	; If 0, exit routine, there are no entries left
	bra			@End_Of_All_Data
	nop
	nop


@Decode_Packet_75
;Decompress Technique 75 = start byte mask pattern (0111), 32 to 96 bits, 128 byte loop, 32bit word loop, end mask (1100)
	addq.l		#1,a0					; Adjust source address to normality
	addq.l		#1,a2					; Adjust destination address to normality
	move.b	(a0)+,(a2)+			; Copy 1 byte of pixel data into destination
	move.w	(a0)+,(a2)+			; Copy 2 bytes of pixel data into destination

	move.l (a1)+, d0				; Get dbra count for long transfer loop (it is already BASE 0!)
@Loop_Align_For_Cache_Line_75:
	move.l	(a0)+, (a2)+		; 32Bit transfer loop, max of 3 ever done, achieves 128 bit alignment of source
	dbra		d0, @Loop_Align_For_Cache_Line_75

	move.l (a1)+, d2					; Get dbra count for 128 byte transfer loop (it is already BASE 0!)

#if  (k_ASSUME_68040_ALWAYS == 0)
	jsr		@Loop_Copy_128_Bytes_Old_Style
#else	
	jsr		@Loop_Copy_128_Bytes_New_Style
#endif /* if  (k_ASSUME_68040_ALWAYS == 0) */

	move.l (a1)+, d0					; Get dbra count for long transfer loop (it is already BASE 0!)
@Loop_Copy_Decode_Packet_75:
	move.l	(a0)+, (a2)+			; Non-optimal 32Bit transfer loop, max of 31 ever done anyway
	dbra		d0, @Loop_Copy_Decode_Packet_75

	move.w	(a0),(a2)				; Copy 2 bytes of data into destination

	move.l	a3, a0				; Get a fresh copy of Row Raw Data pointer (could be optimized out)
	move.l	d5, a1				; Get a fresh copy of Skip List pointer (could be optimized out)
	move.l	(a4)+, d0			; Get Address of Skip List Entry, absolute offset, cannot be "0" unless finished
	bne		@Repeat_Start	; If 0, exit routine, there are no entries left
	bra			@End_Of_All_Data
	nop


@Decode_Packet_76
;Decompress Technique 76 = start byte mask pattern (0000), 32 to 96 bits, 128 byte loop, 32bit word loop, end mask (1110)

	move.l (a1)+, d0				; Get dbra count for long transfer loop (it is already BASE 0!)
@Loop_Align_For_Cache_Line_76:
	move.l	(a0)+, (a2)+		; 32Bit transfer loop, max of 3 ever done, achieves 128 bit alignment of source
	dbra		d0, @Loop_Align_For_Cache_Line_76

	move.l (a1)+, d2					; Get dbra count for 128 byte transfer loop (it is already BASE 0!)

#if  (k_ASSUME_68040_ALWAYS == 0)
	jsr		@Loop_Copy_128_Bytes_Old_Style
#else	
	jsr		@Loop_Copy_128_Bytes_New_Style
#endif /* if  (k_ASSUME_68040_ALWAYS == 0) */

	move.l (a1)+, d0					; Get dbra count for long transfer loop (it is already BASE 0!)
@Loop_Copy_Decode_Packet_76:
	move.l	(a0)+, (a2)+			; Non-optimal 32Bit transfer loop, max of 31 ever done anyway
	dbra		d0, @Loop_Copy_Decode_Packet_76

	move.w	(a0)+,(a2)+		; Copy 2 bytes of pixel data into destination
	move.b	(a0),(a2)			; Copy 1 byte of pixel data into destination

	move.l	a3, a0				; Get a fresh copy of Row Raw Data pointer (could be optimized out)
	move.l	d5, a1				; Get a fresh copy of Skip List pointer (could be optimized out)
	move.l	(a4)+, d0			; Get Address of Skip List Entry, absolute offset, cannot be "0" unless finished
	bne		@Repeat_Start	; If 0, exit routine, there are no entries left
	bra			@End_Of_All_Data
	nop
	nop
	nop
	nop


@Decode_Packet_77
;Decompress Technique 77 = start byte mask pattern (0001), 32 to 96 bits, 128 byte loop, 32bit word loop, end mask (1110)
	addq.l		#3,a0					; Adjust source address to be only 8bit aligned
	addq.l		#3,a2					; Adjust destination address to be only 8bit aligned
	move.b	(a0)+,(a2)+			; Copy 1 byte of data into destination (no masked read-in needed)

	move.l (a1)+, d0				; Get dbra count for long transfer loop (it is already BASE 0!)
@Loop_Align_For_Cache_Line_77:
	move.l	(a0)+, (a2)+		; 32Bit transfer loop, max of 3 ever done, achieves 128 bit alignment of source
	dbra		d0, @Loop_Align_For_Cache_Line_77

	move.l (a1)+, d2					; Get dbra count for 128 byte transfer loop (it is already BASE 0!)

#if  (k_ASSUME_68040_ALWAYS == 0)
	jsr		@Loop_Copy_128_Bytes_Old_Style
#else	
	jsr		@Loop_Copy_128_Bytes_New_Style
#endif /* if  (k_ASSUME_68040_ALWAYS == 0) */

	move.l (a1)+, d0					; Get dbra count for long transfer loop (it is already BASE 0!)
@Loop_Copy_Decode_Packet_77:
	move.l	(a0)+, (a2)+			; Non-optimal 32Bit transfer loop, max of 31 ever done anyway
	dbra		d0, @Loop_Copy_Decode_Packet_77

	move.w	(a0)+,(a2)+		; Copy 2 bytes of pixel data into destination
	move.b	(a0),(a2)			; Copy 1 byte of pixel data into destination

	move.l	a3, a0				; Get a fresh copy of Row Raw Data pointer (could be optimized out)
	move.l	d5, a1				; Get a fresh copy of Skip List pointer (could be optimized out)
	move.l	(a4)+, d0			; Get Address of Skip List Entry, absolute offset, cannot be "0" unless finished
	bne		@Repeat_Start	; If 0, exit routine, there are no entries left
	bra			@End_Of_All_Data
	nop



@Decode_Packet_78
;Decompress Technique 78 = start byte mask pattern (0011), 32 to 96 bits, 128 byte loop, 32bit word loop, end mask (1110)
	addq.l		#2,a2					; Adjust destination address to be only 16bit aligned
	addq.l		#2,a0					; Adjust destination address to be only 16bit aligned
	move.w	(a0)+,(a2)+			; Copy 2 bytes of pixel data into destination

	move.l (a1)+, d0				; Get dbra count for long transfer loop (it is already BASE 0!)
@Loop_Align_For_Cache_Line_78:
	move.l	(a0)+, (a2)+		; 32Bit transfer loop, max of 3 ever done, achieves 128 bit alignment of source
	dbra		d0, @Loop_Align_For_Cache_Line_78

	move.l (a1)+, d2					; Get dbra count for 128 byte transfer loop (it is already BASE 0!)

#if  (k_ASSUME_68040_ALWAYS == 0)
	jsr		@Loop_Copy_128_Bytes_Old_Style
#else	
	jsr		@Loop_Copy_128_Bytes_New_Style
#endif /* if  (k_ASSUME_68040_ALWAYS == 0) */

	move.l (a1)+, d0					; Get dbra count for long transfer loop (it is already BASE 0!)
@Loop_Copy_Decode_Packet_78:
	move.l	(a0)+, (a2)+			; Non-optimal 32Bit transfer loop, max of 31 ever done anyway
	dbra		d0, @Loop_Copy_Decode_Packet_78

	move.w	(a0)+,(a2)+		; Copy 2 bytes of pixel data into destination
	move.b	(a0),(a2)			; Copy 1 byte of pixel data into destination

	move.l	a3, a0				; Get a fresh copy of Row Raw Data pointer (could be optimized out)
	move.l	d5, a1				; Get a fresh copy of Skip List pointer (could be optimized out)
	move.l	(a4)+, d0			; Get Address of Skip List Entry, absolute offset, cannot be "0" unless finished
	bne		@Repeat_Start	; If 0, exit routine, there are no entries left
	bra			@End_Of_All_Data
	nop


@Decode_Packet_79
;Decompress Technique 79 = start byte mask pattern (0111), 32 to 96 bits, 128 byte loop, 32bit word loop, end mask (1110)
	addq.l		#1,a0					; Adjust source address to normality
	addq.l		#1,a2					; Adjust destination address to normality
	move.b	(a0)+,(a2)+			; Copy 1 byte of pixel data into destination
	move.w	(a0)+,(a2)+			; Copy 2 bytes of pixel data into destination

	move.l (a1)+, d0				; Get dbra count for long transfer loop (it is already BASE 0!)
@Loop_Align_For_Cache_Line_79:
	move.l	(a0)+, (a2)+		; 32Bit transfer loop, max of 3 ever done, achieves 128 bit alignment of source
	dbra		d0, @Loop_Align_For_Cache_Line_79

	move.l (a1)+, d2					; Get dbra count for 128 byte transfer loop (it is already BASE 0!)

#if  (k_ASSUME_68040_ALWAYS == 0)
	jsr		@Loop_Copy_128_Bytes_Old_Style
#else	
	jsr		@Loop_Copy_128_Bytes_New_Style
#endif /* if  (k_ASSUME_68040_ALWAYS == 0) */

	move.l (a1)+, d0					; Get dbra count for long transfer loop (it is already BASE 0!)

@Loop_Copy_Decode_Packet_79:
	move.l	(a0)+, (a2)+			; Non-optimal 32Bit transfer loop, max of 31 ever done anyway
	dbra		d0, @Loop_Copy_Decode_Packet_79

	move.w	(a0)+,(a2)+		; Copy 2 bytes of pixel data into destination
	move.b	(a0),(a2)			; Copy 1 byte of pixel data into destination

	move.l	a3, a0				; Get a fresh copy of Row Raw Data pointer (could be optimized out)
	move.l	d5, a1				; Get a fresh copy of Skip List pointer (could be optimized out)
	move.l	(a4)+, d0			; Get Address of Skip List Entry, absolute offset, cannot be "0" unless finished
	bne		@Repeat_Start	; If 0, exit routine, there are no entries left
	bra			@End_Of_All_Data
	nop


/* ======================================================================== */
/* ======================================================================== */
/* ======================================================================== */

; nop
; Should be long aligned already
	
@Loop_Copy_128_Bytes_Old_Style
	move.l	(a0)+,(a2)+						; Transfer 32 bits of image
	move.l	(a0)+,(a2)+						; Transfer 32 bits of image
	move.l	(a0)+,(a2)+						; Transfer 32 bits of image
	move.l	(a0)+,(a2)+						; Transfer 32 bits of image
	
	move.l	(a0)+,(a2)+						; Transfer 32 bits of image
	move.l	(a0)+,(a2)+						; Transfer 32 bits of image
	move.l	(a0)+,(a2)+						; Transfer 32 bits of image
	move.l	(a0)+,(a2)+						; Transfer 32 bits of image
	
	move.l	(a0)+,(a2)+						; Transfer 32 bits of image
	move.l	(a0)+,(a2)+						; Transfer 32 bits of image
	move.l	(a0)+,(a2)+						; Transfer 32 bits of image
	move.l	(a0)+,(a2)+						; Transfer 32 bits of image
	
	move.l	(a0)+,(a2)+						; Transfer 32 bits of image
	move.l	(a0)+,(a2)+						; Transfer 32 bits of image
	move.l	(a0)+,(a2)+						; Transfer 32 bits of image
	move.l	(a0)+,(a2)+						; Transfer 32 bits of image
	
	move.l	(a0)+,(a2)+						; Transfer 32 bits of image
	move.l	(a0)+,(a2)+						; Transfer 32 bits of image
	move.l	(a0)+,(a2)+						; Transfer 32 bits of image
	move.l	(a0)+,(a2)+						; Transfer 32 bits of image
	
	move.l	(a0)+,(a2)+						; Transfer 32 bits of image
	move.l	(a0)+,(a2)+						; Transfer 32 bits of image
	move.l	(a0)+,(a2)+						; Transfer 32 bits of image
	move.l	(a0)+,(a2)+						; Transfer 32 bits of image
	
	move.l	(a0)+,(a2)+						; Transfer 32 bits of image
	move.l	(a0)+,(a2)+						; Transfer 32 bits of image
	move.l	(a0)+,(a2)+						; Transfer 32 bits of image
	move.l	(a0)+,(a2)+						; Transfer 32 bits of image
	
	move.l	(a0)+,(a2)+						; Transfer 32 bits of image
	move.l	(a0)+,(a2)+						; Transfer 32 bits of image
	move.l	(a0)+,(a2)+						; Transfer 32 bits of image
	move.l	(a0)+,(a2)+						; Transfer 32 bits of image
	dbra	d2, @Loop_Copy_128_Bytes_Old_Style
	rts

/* ======================================================================== */
/* ======================================================================== */
/* ======================================================================== */

 nop
; Should be long aligned now

@Loop_Copy_128_Bytes_New_Style

; dc.w	_Debugger

	dc.w 0xF620, 0xA000							; Move16 (a0+), (a2)+, Transfer 128 bits of image
	dc.w 0xF620, 0xA000							; Move16 (a0+), (a2)+, Transfer 128 bits of image
	dc.w 0xF620, 0xA000							; Move16 (a0+), (a2)+, Transfer 128 bits of image
	dc.w 0xF620, 0xA000							; Move16 (a0+), (a2)+, Transfer 128 bits of image
	dc.w 0xF620, 0xA000							; Move16 (a0+), (a2)+, Transfer 128 bits of image
	dc.w 0xF620, 0xA000							; Move16 (a0+), (a2)+, Transfer 128 bits of image
	dc.w 0xF620, 0xA000							; Move16 (a0+), (a2)+, Transfer 128 bits of image
	dc.w 0xF620, 0xA000							; Move16 (a0+), (a2)+, Transfer 128 bits of image
	dbra	d2, @Loop_Copy_128_Bytes_New_Style
	rts

/* ======================================================================== */
/* ======================================================================== */
/* ======================================================================== */
	

; ==========
	
@End_Of_Line:
	
	move.l	a3, a0				; Get a fresh copy of Row Raw Data pointer (could be optimized out)
	move.l	d5, a1				; Get a fresh copy of Skip List pointer (could be optimized out)
	move.l	(a4)+, d0			; Get Address of Skip List Entry, absolute offset, cannot be "0" unless finished
	
; The first entry will not be accidentally "0" because there are header fields at structure top
	bne		@Repeat_Start	; If 0, exit routine, there are no entries left
	
@End_Of_All_Data:
@End_Of_Routine:

#if (k_BUILD_SILLOETTE_COPY_VERSION == 1)	
	move.l		(sp)+, a6		; restore a6 back from stack, we needed it.
#endif
	movem.l		(sp)+, d0-d7/a0-a4 	; Restore all alterred registers

	} /* asm */
	
} /* Fill_Compressed_Span_With_Simple_Silloette */
/*=**************************************************************************=*/


/*=**************************************************************************=*/
/* END OF FILE: Copy_Compacted_Silloette.c */
