/* START OF FILE: Direct_Background_Removal.c */


/*=**************************************************************************=*/
/* [---BEGIN---] FILE HEADER COMMENTS */
/**
 ** SCREEN VIEW = [Adobe Syntax Black-10, Tabs = 6]
 ** PRINTER VIEW = [Adobe Syntax Black-10, Tabs = 6]
 ** =======================================================================
 ** Written by James Merkle
 ** Direct_Background_Removal.c - Creates region of direct color imagess less backgrounds. Very fast
 ** (C) Copyright 1993 by James Merkle
 ** All rights reserved, except those wishing to "copy" moderate sections of code or files may do so, so long as
 ** such code is never used directly either in animation-related tools or multimedia tools that are  
 ** used in live performance or live broadcast or recorded mass market broadcast or film. Desktop usage not restricted.
 ** Conventional desktop game or entertainment not restricted.
 ** =======================================================================
 **
 **
 ** ============
 ** Revision History
 ** ============
 **
 **  1993.03.16		(James Merkle)
 **						Initial coding started
 **
 **
 **
 **
 ** =================
 ** OTHER INFORMATION
 ** =================
 **
 **/
/* [----END----] FILE HEADER COMMENTS */
/*=**************************************************************************=*/


/*=**************************************************************************=*/
/* [---BEGIN---] #include AREA */

#include <QDOffscreen.h>

/* ===
#include "Core_Universal.h"
#include "Core_Specific.h"
#include "SysHooks.h"
#include "Core_Globals.h"
=== */

#include "Shell_Globals.h"
#include "Cursor_Utilities.h"
#include "Shell_Dialogs.h"
#include "Interface_Utilities.h"
#include "Error_Utilities.h"
#include "Dialog_Of_Background_Removal.h"

#include "Memory_Utilities.h"

#include "Off_Screen_Color.h"
#include "Graphics_Primitives.h"
/* ===
#include "Dialog_Of_Background_Removal.h"
=== */

#include "Direct_Background_Removal.h"


/* [----END----] #include AREA */
/*=**************************************************************************=*/


/*=**************************************************************************=*/
/* [---BEGIN---] #define AREA */




#define k_DEBUG_SAFE_MAX_PIXEL_WIDTH						(2200)

#define COLOR_PORT_PORTBITS_CHECK 		0xC000

/*  */
/* 3 or 4 would be reasonable. Must be 1 or greater */
/* was "4" */
/*  */
#define k_MAXIMUM_BACKGROUND_COLORS	6

/*  */
/* 4 or 5 would be reasonable. Can be 0 */
/* "4" represents a 16 pixel grid searchged for background colors */
/* "6" has been used successfully, trying 8 now */
/*  */
#define k_PIXELS_TO_SCAN_FOR_BACKGROUND_COLORS	8


/* ====

1> for each of four corners inset, average 4 pixels (divide by four after adding discrete channels)
2> compare against each other to vote on proper xtwo or three closest colors.
3> delete document backgound as follows

	 very loose outer parameters...
	 
	 FIRST : Determine is blue component is higher or equal to red and green components (blue screen bias)
	 (This reduces 2/3 of all pixels immediately before further tests) OPTIONAL SPEED ENHANCEMENT.
	 
	 SECOND : Reduce color into hue components, Not luminosity components.
	 The proper way is probably too slow so the way we do it is to Divide Color until its total power is
	 less than or equal to total power of standard spec color.
	 		Add R G B of test, if larger than R G B of sample Reduce all three using INTEGER division, goto next stage
	 		Add R G B of test, if less than R G B of sample Increase all three using INTEGER multiplication, goto next stage
	 
	THIRD : General luminosity (physical not biological effective luminosity) is now balanced. Compare BLUE color
	 last because it is most loose,
	
		if red deviance < red threshold deviance AND
		if green deviance < green threshold deviance AND
		if blue deviance < blue threshold deviance AND

			then it is a background pixel to properly discard,
			else if pixel above or to left that have already been processed are discarded, then
			compare against pixel to EITHER of the two ( allows color ramp along two axis to be discarded)
	
			Comparison against other two can be done via direct deviance comparisons.



			AGAINST leftmost (recent)
			if red deviance < red threshold deviance AND
			if green deviance < green threshold deviance AND
			if blue deviance < blue threshold deviance AND
			then it is a background pixel to properly discard
	
			**OR**
	
			AGAINST topmost (recent)
			if red deviance < red threshold deviance AND
			if green deviance < green threshold deviance AND
			if blue deviance < blue threshold deviance AND
			then it is a background pixel to properly discard
	
			Advance markers appropriately.
	

	FORTH : Go through all pixels again, based on BW bit map, and eliminate
	stragglers, isolated pixels.

	NOTE: This smart idea allows gradual color ramping up or down.

====== */

/* [----END----] #define AREA */
/*=**************************************************************************=*/


/*=**************************************************************************=*/
/* [---BEGIN---] TYPE AREA */


/* [----END----] TYPE AREA */
/*=**************************************************************************=*/


/*=**************************************************************************=*/
/* [---BEGIN---] PROTOTYPE AREA */


void Copy_Non_Masked_Bits(GrafPtr offscreen_Graph_Ptr
				, GrafPtr	simple_Bit_Map_Ptr
				, Rect *source_Rect
				, Rect *target_Rect
				, INTEGER_32 mask_Color_Total
				, Color_Type mask_Colors[]);

	
Safe_Boolean Check_Direct_Pixel_Neighbors_Are_Background(INTEGER_32 x_Position
																		,INTEGER_32 y_Position
																		,INTEGER_32 mask_Color_Total
																		,Color_Type mask_Colors[]
																		, Boolean use_Direct_Screen_Access);
																								
static pascal void Get_Single_1_Bit_Pixel_2(INTEGER_32 horizontal_Coordinate
												, INTEGER_32 vertical_Coordinate
												, CARDINAL_32 *fetched_Color);

static pascal  void Draw_Single_1_Bit_Pixel_2(INTEGER_32 horizontal_Coordinate, INTEGER_32 vertical_Coordinate, INTEGER_32 color_Desired);

Boolean Check_Upward_Previous_Background_Pixel_Matches(RGBColor *current_Color
																	, INTEGER_32 start_X
																	, INTEGER_32 start_Y
																	, INTEGER_32 target_X_Offset
																	, INTEGER_32 target_Y_Offset
																	, Boolean use_Direct_Screen_Access
																	, CARDINAL_32 stored_Bit_Depth
																	, CARDINAL_32	bytes_Per_Row
																	, CARDINAL_32	raw_Pixel_Address
																	, CARDINAL_8	predominant_Color);

Safe_Boolean Remove_Stray_Content_Pixels(INTEGER_32 current_Length
										, INTEGER_32 current_Height
										, INTEGER_32 target_X_Offset
										, INTEGER_32 target_Y_Offset
										, Boolean use_Direct_Screen_Access
										, CARDINAL_32	bytes_Per_Row
										, CARDINAL_32 raw_Pixel_Address);
										
Safe_Boolean Remove_Stray_Background_Pixels(INTEGER_32 current_Length
										, INTEGER_32 current_Height
										, INTEGER_32 target_X_Offset
										, INTEGER_32 target_Y_Offset
										, Boolean use_Direct_Screen_Access
										, CARDINAL_32	bytes_Per_Row
										, CARDINAL_32 raw_Pixel_Address);


/* [----END----] PROTOTYPE AREA */
/*=**************************************************************************=*/


/*=**************************************************************************=*/
/* [---BEGIN---] GLOBAL DEFINITION AREA */

Safe_Boolean				g_Background_Removal_Options_Ready;
Background_Options	g_Options;

/* [----END----] GLOBAL DEFINITION AREA */
/*=**************************************************************************=*/


/*=**************************************************************************=*/
/* [---BEGIN---] LOCAL STATIC DEFINITION AREA */

static CARDINAL_8	*unit_Name = "\pDirect_Background_Removal";


/* [----END----] LOCAL STATIC DEFINITION AREA */
/*=**************************************************************************=*/


/*=**************************************************************************=*/
Safe_Boolean Remove_Stray_Background_Pixels(INTEGER_32 current_Length
										, INTEGER_32 current_Height
										, INTEGER_32 target_X_Offset
										, INTEGER_32 target_Y_Offset
										, Boolean use_Direct_Screen_Access
										, CARDINAL_32	bytes_Per_Row
										, CARDINAL_32 raw_Pixel_Address)
/*  */
/* This routine somehow results in corrupting registers of the caller. */
/*  */
{
	register	INTEGER_32	current_X;
	register	INTEGER_32	current_Y;
	register	CARDINAL_8  pixel_Neighbors;
	Boolean			is_An_Isolated_Pixel;
	INTEGER_32		last_X_Position;
	CARDINAL_32	simple_Pixel_Color;
	CARDINAL_32	current_Screen_Address;
	Safe_Boolean		function_Result;
	INTEGER_32		x_Offset;
	INTEGER_32		y_Offset;
	CARDINAL_8		*code_Name;
/* END VAR */


	code_Name = "\pRemove_Stray_Background_Pixels";
	function_Result = SAFE_FALSE;

	
	if (((current_Length < 10) || (current_Length > k_DEBUG_SAFE_MAX_PIXEL_WIDTH))
		|| ((current_Height < 10) || (current_Height > k_DEBUG_SAFE_MAX_PIXEL_WIDTH))) {

		goto Error_Handler;
	}
		
	/*  */
	/* we do not want to examine the last column of pixels for inclusion */
	/*  */
	last_X_Position = (current_Length - 1);
	
	/*  */
	/* we do not want to examine the last column of pixels for inclusion */
	/*  */
	current_Height = (current_Height - 1);
	
	/*  */
	/* This code does not include using the left top and right */
	/* sides of the frame for its comparisons. Bottom is checked */
	/* for the hell of it*/
	/*  */
	current_Height += target_Y_Offset;
	last_X_Position += target_X_Offset;
	
	/*  */
	/* Important Flip-Flop Switch */
	/*  */
	is_An_Isolated_Pixel = FALSE;

	/*  */
	/* Remove isolated stragglers */
	/*  */
			
	for (current_Y = (1 + target_Y_Offset); (current_Y < current_Height); current_Y++) {

		for (current_X = (1 + target_X_Offset); (current_X < last_X_Position); current_X++) {

			/*  */
			/* Paranoia only */
			/*  */
			simple_Pixel_Color = 0;


			if (use_Direct_Screen_Access != TRUE) {
			/*  */
			/* Regular official QD technique, pretty slow */
			/* should already have a black colored pen */
			/*  */

	/* =====
				GetCPixel(x_Offset, y_Offset, &current_Simple_Color);
	====== */

				x_Offset = current_X;
				y_Offset = current_Y;

				current_Screen_Address = raw_Pixel_Address + (x_Offset >> 3);
				current_Screen_Address = current_Screen_Address 
										+ ((CARDINAL_32)(y_Offset) * bytes_Per_Row);
	
				pixel_Neighbors = *(CARDINAL_8 *)current_Screen_Address;
				if (pixel_Neighbors & (0x80 >> ((x_Offset) & 0x07))) {
					/*  */
					/* This is a non-background pixel */
					/*  */
					simple_Pixel_Color = 1;
				}
				else {
					simple_Pixel_Color = 0;
				}

				if (simple_Pixel_Color == 0) {
					/*  */
					/* Make certain at least one pixel is bordering it and has non-background color */
					/*  */
					
					is_An_Isolated_Pixel = TRUE;
		
					/*  */
					/* Make certain at least one pixel is bordering it and has non-background color */
					/* Check the top side */
					/*  */
					x_Offset = current_X;
					y_Offset = current_Y - 1;
					
					current_Screen_Address = raw_Pixel_Address + (x_Offset >> 3);
					current_Screen_Address = current_Screen_Address 
											+ ((CARDINAL_32)(y_Offset) * bytes_Per_Row);
		
					pixel_Neighbors = *(CARDINAL_8 *)current_Screen_Address;
					if ((pixel_Neighbors & (0x80 >> ((x_Offset) & 0x07))) == 0) {
						is_An_Isolated_Pixel = FALSE;
					}
					else {
					
						/*  */
						/* Check the bottom side */
						/*  */
						x_Offset = current_X;
						y_Offset = current_Y + 1;
						
						current_Screen_Address = raw_Pixel_Address + (x_Offset >> 3);
						current_Screen_Address = current_Screen_Address 
												+ ((CARDINAL_32)(y_Offset) * bytes_Per_Row);
			
						pixel_Neighbors = *(CARDINAL_8 *)current_Screen_Address;
						if ((pixel_Neighbors & (0x80 >> ((x_Offset) & 0x07))) == 0) {
							is_An_Isolated_Pixel = FALSE;
						}
						else {
						
							/*  */
							/* Check the Top left side */
							/*  */

							x_Offset = current_X - 1;
							y_Offset = current_Y - 1;
							
							current_Screen_Address = raw_Pixel_Address + (x_Offset >> 3);
							current_Screen_Address = current_Screen_Address 
													+ ((CARDINAL_32)(y_Offset) * bytes_Per_Row);
				
							pixel_Neighbors = *(CARDINAL_8 *)current_Screen_Address;
							if ((pixel_Neighbors & (0x80 >> ((x_Offset) & 0x07))) == 0) {
								is_An_Isolated_Pixel = FALSE;
							}
							else {
							
								/*  */
								/* Check the Bottom left side */
								/*  */
								
								x_Offset = current_X - 1;
								y_Offset = current_Y + 1;
								
								current_Screen_Address = raw_Pixel_Address + (x_Offset >> 3);
								current_Screen_Address = current_Screen_Address 
														+ ((CARDINAL_32)(y_Offset) * bytes_Per_Row);
					
								pixel_Neighbors = *(CARDINAL_8 *)current_Screen_Address;
								if ((pixel_Neighbors & (0x80 >> ((x_Offset) & 0x07))) == 0) {
									is_An_Isolated_Pixel = FALSE;
								}
								else {
								
									/*  */
									/* Check the top right side */
									/*  */
									
									x_Offset = current_X + 1;
									y_Offset = current_Y - 1;
									
									current_Screen_Address = raw_Pixel_Address + (x_Offset >> 3);
									current_Screen_Address = current_Screen_Address 
															+ ((CARDINAL_32)(y_Offset) * bytes_Per_Row);
						
									pixel_Neighbors = *(CARDINAL_8 *)current_Screen_Address;
									if ((pixel_Neighbors & (0x80 >> ((x_Offset) & 0x07))) == 0) {
										is_An_Isolated_Pixel = FALSE;
									}
									else {
									
										/*  */
										/* Check the bottom right side */
										/*  */
										x_Offset = current_X + 1;
										y_Offset = current_Y + 1;
										
										current_Screen_Address = raw_Pixel_Address + (x_Offset >> 3);
										current_Screen_Address = current_Screen_Address 
																+ ((CARDINAL_32)(y_Offset) * bytes_Per_Row);
							
										pixel_Neighbors = *(CARDINAL_8 *)current_Screen_Address;
										if ((pixel_Neighbors & (0x80 >> ((x_Offset) & 0x07))) == 0) {
											is_An_Isolated_Pixel = FALSE;
										}
										else {
										
											/*  */
											/* At this point all pixels in all directions except for left and right */
											/* have been compared. In this program no neighbors count to */
											/* left and right unless BOTH are set */
											/*  */
										
											x_Offset = current_X + 1;
											y_Offset = current_Y;
											
											current_Screen_Address = raw_Pixel_Address + (x_Offset >> 3);
											current_Screen_Address = current_Screen_Address 
																	+ ((CARDINAL_32)(y_Offset) * bytes_Per_Row);
								
											pixel_Neighbors = *(CARDINAL_8 *)current_Screen_Address;
											if ((pixel_Neighbors & (0x80 >> ((x_Offset) & 0x07))) == 0) {
											
												/*  */
												/* ENHANCEMENT : DOES NOT COUNT UNLESS OTHER SIDE IS PIXEL TOO */
												/* Check the left side */
												/*  */
												
												x_Offset = current_X - 1;
												y_Offset = current_Y;
												
												current_Screen_Address = raw_Pixel_Address + (x_Offset >> 3);
												current_Screen_Address = current_Screen_Address 
																		+ ((CARDINAL_32)(y_Offset) * bytes_Per_Row);
									
												pixel_Neighbors = *(CARDINAL_8 *)current_Screen_Address;
												if ((pixel_Neighbors & (0x80 >> ((x_Offset) & 0x07))) == 0) {
													is_An_Isolated_Pixel = FALSE;
												}
												
											} /* if (pixel_Neighbors & (0x80 >> ((x_Offset) & 0x07))) */
										
										} /* NOT if (pixel_Neighbors & (0x80 >> ((x_Offset) & 0x07))) */
									
									} /* NOT if (simple_Pixel_Color != 0) */
								
								} /* NOT if (simple_Pixel_Color != 0) */
							
							} /* NOT if (simple_Pixel_Color != 0) */
						
						} /* NOT if (simple_Pixel_Color != 0) */
					
					} /* NOT if (simple_Pixel_Color != 0) */
					
				}
				
			} /* if (use_Direct_Screen_Access != TRUE) */
			else {
				Get_Single_1_Bit_Pixel_2(current_X, current_Y, &simple_Pixel_Color);
				
				if (simple_Pixel_Color == 0) {
				
					/*  */
					/* Make certain at least one pixel is bordering it and has non-background color */
					/*  */

					is_An_Isolated_Pixel = TRUE;
					
					/*  */
					/* Check the top side */
					/*  */
					x_Offset = current_X;
					y_Offset = current_Y - 1;
					
					Get_Single_1_Bit_Pixel_2(x_Offset, y_Offset, &simple_Pixel_Color);
					if (simple_Pixel_Color == 0) {
						is_An_Isolated_Pixel = FALSE;
					}
					else {
					
						/*  */
						/* Check the bottom side */
						/*  */
						x_Offset = current_X;
						y_Offset = current_Y + 1;
						
						Get_Single_1_Bit_Pixel_2(x_Offset, y_Offset, &simple_Pixel_Color);
						if (simple_Pixel_Color == 0) {
							is_An_Isolated_Pixel = FALSE;
						}
						else {
						
							/*  */
							/* Check the Top left side */
							/*  */

							x_Offset = current_X - 1;
							y_Offset = current_Y - 1;
							
							Get_Single_1_Bit_Pixel_2(x_Offset, y_Offset, &simple_Pixel_Color);
							if (simple_Pixel_Color == 0) {
								is_An_Isolated_Pixel = FALSE;
							}
							else {
							
								/*  */
								/* Check the Bottom left side */
								/*  */
								
								x_Offset = current_X - 1;
								y_Offset = current_Y + 1;
								
								Get_Single_1_Bit_Pixel_2(x_Offset, y_Offset, &simple_Pixel_Color);
								if (simple_Pixel_Color == 0) {
									is_An_Isolated_Pixel = FALSE;
								}
								else {
								
									/*  */
									/* Check the top right side */
									/*  */
									
									x_Offset = current_X + 1;
									y_Offset = current_Y - 1;
									
									Get_Single_1_Bit_Pixel_2(x_Offset, y_Offset, &simple_Pixel_Color);
									if (simple_Pixel_Color == 0) {
										is_An_Isolated_Pixel = FALSE;
									}
									else {
									
										/*  */
										/* Check the bottom right side */
										/*  */
										x_Offset = current_X + 1;
										y_Offset = current_Y + 1;
										
										Get_Single_1_Bit_Pixel_2(x_Offset, y_Offset, &simple_Pixel_Color);
										if (simple_Pixel_Color == 0) {
											is_An_Isolated_Pixel = FALSE;
										}
										else {
										
											/*  */
											/* At this point all pixels in all directions except for left and right */
											/* have been compared. In this program no neighbors count to */
											/* left and right unless BOTH are set */
											/*  */
										
											x_Offset = current_X + 1;
											y_Offset = current_Y;
											
											Get_Single_1_Bit_Pixel_2(x_Offset, y_Offset, &simple_Pixel_Color);
											if (simple_Pixel_Color == 0) {
											
												/*  */
												/* ENHANCEMENT : DOES NOT COUNT UNLESS OTHER SIDE IS PIXEL TOO */
												/* Check the left side */
												/*  */
												
												x_Offset = current_X - 1;
												y_Offset = current_Y;
												
												Get_Single_1_Bit_Pixel_2(x_Offset, y_Offset, &simple_Pixel_Color);
												if (simple_Pixel_Color == 0) {
													is_An_Isolated_Pixel = FALSE;
												}
												
											} /* if (pixel_Neighbors & (0x80 >> ((x_Offset) & 0x07))) */
										
										} /* NOT if (simple_Pixel_Color == 0) */
									
									} /* NOT if (simple_Pixel_Color == 0) */
								
								} /* NOT if (simple_Pixel_Color == 0) */
							
							} /* NOT if (simple_Pixel_Color == 0) */
						
						} /* NOT if (simple_Pixel_Color == 0) */
					
					} /* NOT if (simple_Pixel_Color == 0) */
					
				} /* if (simple_Pixel_Color == 1) */
				
			} /* NOT if (use_Direct_Screen_Access == TRUE) */
			

			if (is_An_Isolated_Pixel == TRUE) {
			
				if (use_Direct_Screen_Access != TRUE) {  
					
					x_Offset = current_X;
					current_Screen_Address = raw_Pixel_Address + (x_Offset >> 3);
					current_Screen_Address = current_Screen_Address 
											+ ((CARDINAL_32)(current_Y) * bytes_Per_Row);
		
					pixel_Neighbors = *(CARDINAL_8 *)current_Screen_Address;
					*(CARDINAL_8 *)current_Screen_Address
														= pixel_Neighbors | ((0x80 >> ((x_Offset) & 0x07)));

				}
				else {
					Draw_Single_1_Bit_Pixel_2(current_X, current_Y, 1);
				}
				
				/*  */
				/* Reset Flip/Flop switch. Speed technique */
				/*  */
				is_An_Isolated_Pixel = FALSE;
			}
	
		} /* for (current_X = 0;  ... */
	} /* for (current_Y = 0;  ... */


	function_Result = SAFE_TRUE;
	goto Normal_Function_Exit;
	
Error_Handler:
	Display_Serious_Error_Message(unit_Name
										, code_Name
										, "\pProblem encountered"
										, "\pCorrupted? bad programming bounds");
										
	function_Result = SAFE_FALSE;
	
Normal_Function_Exit:
	return(function_Result);
	
} /* Safe_Boolean Remove_Stray_Background_Pixels */
/*=**************************************************************************=*/


/*=**************************************************************************=*/
static pascal  void Draw_Single_1_Bit_Pixel_2(INTEGER_32 horizontal_Coordinate, INTEGER_32 vertical_Coordinate, INTEGER_32 color_Desired)
{
	/*  */
	/* THIS IS A DUPLICATED COPY! */
	/* This routine returns the index color (1,0) at the specified address. */
	/* WARNING: needs 68020 addressing for speed */
	/*  */

	asm 68020 {
		move.l		d2,-(SP)						; Save all registers that will be altered (not including d0,d0,a0,a1)
		
		move.l		horizontal_Coordinate(a6),d1					; Fetch horizontal_Coordinate into a register for use later
		move.l		vertical_Coordinate(a6),d2						; Fetch vertical_Coordinate

		movea.l		primitives_Vertical_Lookup_Table_Direct(a5),a0		; Get vertical table address
		move.l		(a0,d2*4),a1												; Get vertical table indexed data (address of screen 0)
		movea.l		primitives_Horizontal_Lookup_Table_Direct(a5),a0	; Get horizontal table address
		add.l			(a0,d1*4),a1												; Add horizontal table indexed data for final address

		andi.l			#7,d1														; Mask out all but lowest pixel horizontal offset
		move.w		#7,d0														; Invert the pixel position ...
		sub.b			d1,d0														;  by subtracting from seven (famous trick)

		move.l		color_Desired(a6),d1
		beq.s			@Write_White_Pixel		

		bset.b			d0,(a1)													; Set the pixel data into the destination memory
		
		movem.l		(SP)+,d2													; Restore altered registers

		unlk			a6															; Remove stack frame
		movea.l		(sp)+,a0													; Load invocation address
		addi.l			#12,sp														; Restore proper stack size ("12" was size of passed parameters)
		jmp			(a0)														; Jump back to caller. All done


@Write_White_Pixel
		bclr.b			d0,(a1)													; Clear the pixel data into the destination memory
		
		movem.l		(SP)+,d2													; Restore altered registers
		
		unlk			a6															; Remove stack frame
		movea.l		(sp)+,a0													; Load invocation address
		addi.l			#12,sp														; Restore proper stack size ("12" was size of passed parameters)
		jmp			(a0)														; Jump back to caller. All done
		
	}		

} /* Draw_Single_1_Bit_Pixel_2 */
/*=**************************************************************************=*/


/*=**************************************************************************=*/
static pascal void Get_Single_1_Bit_Pixel_2(INTEGER_32 horizontal_Coordinate
																, INTEGER_32 vertical_Coordinate
																, CARDINAL_32 *fetched_Color)
{
	/*  */
	/* THIS IS A DUPLICATED COPY! */
	/* This routine returns the index color (1,0) at the specified address. */
	/* WARNING: needs 68020 addressing for speed */
	/*  */
	
/* END VAR */
	asm 68020 {		
		move.l		horizontal_Coordinate(a6),d1					; Fetch horizontal_Coordinate into a register for use later
		move.l		vertical_Coordinate(a6),d0						; Fetch vertical_Coordinate

		movea.l		primitives_Vertical_Lookup_Table_Direct(a5),a0		; Get vertical table address
		move.l		(a0,d0*4),a1												; Get vertical table indexed data (address of screen 0)
		movea.l		primitives_Horizontal_Lookup_Table_Direct(a5),a0	; Get horizontal table address
		add.l			(a0,d1*4),a1												; Add horizontal table indexed data for final address

		andi.l			#7,d1														; Mask out all but lowest pixel horizontal offset
		move.w		#7,d0														; Invert the pixel position ...
		sub.b			d1,d0														;  by subtracting from seven (famous trick)

		btst.b			d0,(a1)													; Set the pixel data into the destination memory
		beq.s			@Is_Black_Pixel
		move.l		fetched_Color(a6),a0
		move.l		#1,(a0)
		
		unlk			a6															; Remove stack frame
		movea.l		(sp)+,a0													; Load invocation address
		add.l			#12,sp													; Restore proper stack size ("12" was size of passed parameters)
		jmp			(a0)														; Jump back to caller. All done

@Is_Black_Pixel
		move.l		fetched_Color(a6),a0
		clr.l			(a0)
				
		unlk			a6															; Remove stack frame
		movea.l		(sp)+,a0													; Load invocation address
		add.l			#12,sp														; Restore proper stack size ("8" was size of passed parameters)
		jmp			(a0)														; Jump back to caller. All done
		
	}		

} /* Get_Single_1_Bit_Pixel_2 */
/*=**************************************************************************=*/


/*=**************************************************************************=*/
Boolean Check_Upward_Previous_Background_Pixel_Matches(RGBColor *current_Color
																	, INTEGER_32 start_X
																	, INTEGER_32 start_Y
																	, INTEGER_32 target_X_Offset
																	, INTEGER_32 target_Y_Offset
																	, Boolean use_Direct_Screen_Access
																	, CARDINAL_32 stored_Bit_Depth
																	, CARDINAL_32	bytes_Per_Row
																	, CARDINAL_32	raw_Pixel_Address
																	, CARDINAL_8	predominant_Color)
{
	register	INTEGER_32	current_X;
	register	INTEGER_32	current_Y;
	register	CARDINAL_8  pixel_Neighbors;
	Boolean			located_Something;
	CARDINAL_32	current_Screen_Address;
	RGBColor			upward_Last_Color;
	Boolean			function_result;
	INTEGER_32		red_Deviance;
	INTEGER_32		green_Deviance;
	INTEGER_32		blue_Deviance;
	CARDINAL_8		*code_Name;
/* END VAR */


	code_Name = "\pCheck_Upward_Previous_Background_Pixel_Matches";

	function_result = FALSE;
	current_Y = start_Y + target_Y_Offset;
	current_X = start_X + target_X_Offset;
	

	for (current_Y = start_Y; (current_Y >= 0); current_Y = current_Y - 1) {
	
	
		current_Screen_Address = raw_Pixel_Address + (current_X >> 3);
		current_Screen_Address = current_Screen_Address 
								+ ((CARDINAL_32)(current_Y) * bytes_Per_Row);

		pixel_Neighbors = *(CARDINAL_8 *)current_Screen_Address;
		if (pixel_Neighbors & (0x80 >> ((current_X) & 0x07))) {
			/*  */
			/* This is a non-background pixel */
			/*  */
		}
		else {
						
			if (use_Direct_Screen_Access == TRUE){
				/*  */
				/* This gets the pixel in a fast manner involving no multiplication */
				/*  */
			/* ===
				Get_Pixel_Index(x_Offset, y_Offset, &pixel_Color);
			=== */
				Get_Color_Data_Pixel_32_Bit_Subprimitive((current_X - target_X_Offset), (current_Y - target_Y_Offset), &upward_Last_Color);
			}
			else {
				GetCPixel((current_X - target_X_Offset), (current_Y - target_Y_Offset), &upward_Last_Color);
			}

			/*  */
			/* At this point we have proven that the most recent past */
			/* pixel UPWARD that was DETERMINED TO BE BACKGROUND */
			/* was the exact color of current_Color, yet our color upon entry was */
			/* upward_Last_Color. Compare to see if within tolerance */
			/*  */
						
			if (upward_Last_Color.red < current_Color->red) {
				red_Deviance = current_Color->red - upward_Last_Color.red;
			}
			else {
				red_Deviance = upward_Last_Color.red - current_Color->red;
			}
			
			if (upward_Last_Color.green < current_Color->green) {
				green_Deviance = current_Color->green - upward_Last_Color.green;
			}
			else {
				green_Deviance = upward_Last_Color.green - current_Color->green;
			}
			
			if (upward_Last_Color.blue < current_Color->blue) {
				blue_Deviance = current_Color->blue - upward_Last_Color.blue;
			}
			else {
				blue_Deviance = upward_Last_Color.blue - current_Color->blue;
			}
			
			/*  */
			/* Allow deviance for major color to differ by 200% vs other channels */
			/*  */

			switch(predominant_Color) {
				case 0:
					/*  */
					/* Blue is largest intensity, see if still valid */
					/*  */
					blue_Deviance = blue_Deviance >> 1; 
				break;

				case 1:
					/*  */
					/* Green is largest intensity, see if still valid */
					/*  */
					green_Deviance = green_Deviance >> 1; 
				break;

				case 2:
				default:
					/*  */
					/* Red is largest intensity, see if still valid */
					/*  */
					red_Deviance = red_Deviance >> 1;
					
			} /* switch(predominant_Color) */

			/*  */
			/* Contrast absolute values of color deviances  */
			/*  */
			if (stored_Bit_Depth < 24) {
				if ((red_Deviance <= g_Options.maximum_5_Bit_Red_Drift_Deviance)
					&& (blue_Deviance <= g_Options.maximum_5_Bit_Blue_Drift_Deviance)
					&& (green_Deviance <= g_Options.maximum_5_Bit_Green_Drift_Deviance)) {
					
					/*  */
					/* It meets all general parameters, but see if it matches the */
					/* maximum aggregate intensity value. */
					/* This is an important step for proper governing of color drift control */
					/*  */
					if ((red_Deviance + blue_Deviance + green_Deviance) <= g_Options.maximum_5_Bit_Total_Drift_Deviance) {
					
						function_result = TRUE;
					}
				}
			}
			else {
				if ((red_Deviance <= g_Options.maximum_8_Bit_Red_Drift_Deviance)
					&& (blue_Deviance <= g_Options.maximum_8_Bit_Blue_Drift_Deviance)
					&& (green_Deviance <= g_Options.maximum_8_Bit_Green_Drift_Deviance)) {
				
					/*  */
					/* It meets all general parameters, but see if it matches the */
					/* maximum aggregate intensity value. */
					/* This is an important step for proper governing of color drift control */
					/*  */
					if ((red_Deviance + blue_Deviance + green_Deviance) <= g_Options.maximum_8_Bit_Total_Drift_Deviance) {
					
						function_result = TRUE;
					}
				}
				
			} /* NOT if (stored_Bit_Depth < 24) */
						
			/*  */
			/* Jump out. Spaghetti jump instead of break. Oh well. */
			/*  */
			goto Normal_Function_Exit;
			
		} /* if (pixel_Neighbors & (0x80 >> ((x_Offset) & 0x07))) */
			
	
	} /* for (current_Y = start_Y; (current_Y >= 0); current_Y = current_Y - 1) */
	
	
Normal_Function_Exit:
	return(function_result);

} /* Boolean Check_Upward_Previous_Background_Pixel_Matches ... */
/*=**************************************************************************=*/


/*=**************************************************************************=*/
Safe_Boolean Remove_Stray_Content_Pixels(INTEGER_32 current_Length
										, INTEGER_32 current_Height
										, INTEGER_32 target_X_Offset
										, INTEGER_32 target_Y_Offset
										, Boolean use_Direct_Screen_Access
										, CARDINAL_32	bytes_Per_Row
										, CARDINAL_32 raw_Pixel_Address)
{
	register	INTEGER_32	current_X;
	register	INTEGER_32	current_Y;
	register	CARDINAL_8  pixel_Neighbors;
	Boolean			is_An_Isolated_Pixel;
	INTEGER_32		last_X_Position;
	CARDINAL_32	simple_Pixel_Color;
	CARDINAL_32	current_Screen_Address;
	Safe_Boolean		function_Result;
	INTEGER_32		x_Offset;
	INTEGER_32		y_Offset;
	CARDINAL_8		*code_Name;
/* END VAR */


	code_Name = "\pRemove_Stray_Content_Pixels";
	function_Result = SAFE_FALSE;
	
	if (((current_Length < 10) || (current_Length > k_DEBUG_SAFE_MAX_PIXEL_WIDTH))
		|| ((current_Height < 10) || (current_Height > k_DEBUG_SAFE_MAX_PIXEL_WIDTH))) {

		goto Error_Handler;
	}
	
	
	/*  */
	/* we do not want to examine the last column of pixels for inclusion */
	/*  */
	last_X_Position = (current_Length - 1);
	
	/*  */
	/* we do not want to examine the last column of pixels for inclusion */
	/*  */
	current_Height = (current_Height - 1);
	
	/*  */
	/* This code does not include using the left top and right */
	/* sides of the frame for its comparisons. Bottom is checked */
	/* for the hell of it*/
	/*  */
	current_Height += target_Y_Offset;
	last_X_Position += target_X_Offset;
	
	/*  */
	/* Important Flip-Flop Switch */
	/*  */
	is_An_Isolated_Pixel = FALSE;

	/*  */
	/* Remove isolated stragglers */
	/*  */
			
	for (current_Y = (1 + target_Y_Offset); (current_Y < current_Height); current_Y++) {

/* ===
		for (current_X = (1 + target_X_Offset); (current_X < last_X_Position); current_X++) {
=== */
		/*  */
		/* Ignore first + last position, Run Backwards to eliminate */
		/* right side dark lighting video alternating scan line residue. */
		/* Assumes bias lighting from upper left */
		/*  */
		for (current_X = (last_X_Position - 1); (current_X > 0); current_X--) {

			/*  */
			/* Paranoia only */
			/*  */
			simple_Pixel_Color = 0;


			if (use_Direct_Screen_Access != TRUE) {
			/*  */
			/* Regular official QD technique, pretty slow */
			/* should already have a black colored pen */
			/*  */

	/* =====
				GetCPixel(x_Offset, y_Offset, &current_Simple_Color);
	====== */

				x_Offset = current_X;
				y_Offset = current_Y;

				current_Screen_Address = raw_Pixel_Address + (x_Offset >> 3);
				current_Screen_Address = current_Screen_Address 
										+ ((CARDINAL_32)(y_Offset) * bytes_Per_Row);
	
				pixel_Neighbors = *(CARDINAL_8 *)current_Screen_Address;
				if (pixel_Neighbors & (0x80 >> ((x_Offset) & 0x07))) {
					/*  */
					/* This is a non-background pixel */
					/*  */
					simple_Pixel_Color = 1;
				}
				else {
					simple_Pixel_Color = 0;
				}

				if (simple_Pixel_Color == 1) {
					/*  */
					/* Make certain at least one pixel is bordering it and has non-background color */
					/*  */
					
					is_An_Isolated_Pixel = TRUE;
		
					/*  */
					/* Make certain at least one pixel is bordering it and has non-background color */
					/* Check the top side */
					/*  */
					x_Offset = current_X;
					y_Offset = current_Y - 1;
					
					current_Screen_Address = raw_Pixel_Address + (x_Offset >> 3);
					current_Screen_Address = current_Screen_Address 
											+ ((CARDINAL_32)(y_Offset) * bytes_Per_Row);
		
					pixel_Neighbors = *(CARDINAL_8 *)current_Screen_Address;
					if (pixel_Neighbors & (0x80 >> ((x_Offset) & 0x07))) {
						is_An_Isolated_Pixel = FALSE;
					}
					else {
					
						/*  */
						/* Check the bottom side */
						/*  */
						x_Offset = current_X;
						y_Offset = current_Y + 1;
						
						current_Screen_Address = raw_Pixel_Address + (x_Offset >> 3);
						current_Screen_Address = current_Screen_Address 
												+ ((CARDINAL_32)(y_Offset) * bytes_Per_Row);
			
						pixel_Neighbors = *(CARDINAL_8 *)current_Screen_Address;
						if (pixel_Neighbors & (0x80 >> ((x_Offset) & 0x07))) {
							is_An_Isolated_Pixel = FALSE;
						}
						else {
						
							/*  */
							/* Check the Top left side */
							/*  */

							x_Offset = current_X - 1;
							y_Offset = current_Y - 1;
							
							current_Screen_Address = raw_Pixel_Address + (x_Offset >> 3);
							current_Screen_Address = current_Screen_Address 
													+ ((CARDINAL_32)(y_Offset) * bytes_Per_Row);
				
							pixel_Neighbors = *(CARDINAL_8 *)current_Screen_Address;
							if (pixel_Neighbors & (0x80 >> ((x_Offset) & 0x07))) {
								is_An_Isolated_Pixel = FALSE;
							}
							else {
							
								/*  */
								/* Check the Bottom left side */
								/*  */
								
								x_Offset = current_X - 1;
								y_Offset = current_Y + 1;
								
								current_Screen_Address = raw_Pixel_Address + (x_Offset >> 3);
								current_Screen_Address = current_Screen_Address 
														+ ((CARDINAL_32)(y_Offset) * bytes_Per_Row);
					
								pixel_Neighbors = *(CARDINAL_8 *)current_Screen_Address;
								if (pixel_Neighbors & (0x80 >> ((x_Offset) & 0x07))) {
									is_An_Isolated_Pixel = FALSE;
								}
								else {
								
									/*  */
									/* Check the top right side */
									/*  */
									
									x_Offset = current_X + 1;
									y_Offset = current_Y - 1;
									
									current_Screen_Address = raw_Pixel_Address + (x_Offset >> 3);
									current_Screen_Address = current_Screen_Address 
															+ ((CARDINAL_32)(y_Offset) * bytes_Per_Row);
						
									pixel_Neighbors = *(CARDINAL_8 *)current_Screen_Address;
									if (pixel_Neighbors & (0x80 >> ((x_Offset) & 0x07))) {
										is_An_Isolated_Pixel = FALSE;
									}
									else {
									
										/*  */
										/* Check the bottom right side */
										/*  */
										x_Offset = current_X + 1;
										y_Offset = current_Y + 1;
										
										current_Screen_Address = raw_Pixel_Address + (x_Offset >> 3);
										current_Screen_Address = current_Screen_Address 
																+ ((CARDINAL_32)(y_Offset) * bytes_Per_Row);
							
										pixel_Neighbors = *(CARDINAL_8 *)current_Screen_Address;
										if (pixel_Neighbors & (0x80 >> ((x_Offset) & 0x07))) {
											is_An_Isolated_Pixel = FALSE;
										}
										else {
										
											/*  */
											/* At this point all pixels in all directions except for left and right */
											/* have been compared. In this program no neighbors count to */
											/* left and right unless BOTH are set */
											/*  */
										
											x_Offset = current_X + 1;
											y_Offset = current_Y;
											
											current_Screen_Address = raw_Pixel_Address + (x_Offset >> 3);
											current_Screen_Address = current_Screen_Address 
																	+ ((CARDINAL_32)(y_Offset) * bytes_Per_Row);
								
											pixel_Neighbors = *(CARDINAL_8 *)current_Screen_Address;
											if (pixel_Neighbors & (0x80 >> ((x_Offset) & 0x07))) {
											
												/*  */
												/* ENHANCEMENT : DOES NOT COUNT UNLESS OTHER SIDE IS PIXEL TOO */
												/* Check the left side */
												/*  */
												
												x_Offset = current_X - 1;
												y_Offset = current_Y;
												
												current_Screen_Address = raw_Pixel_Address + (x_Offset >> 3);
												current_Screen_Address = current_Screen_Address 
																		+ ((CARDINAL_32)(y_Offset) * bytes_Per_Row);
									
												pixel_Neighbors = *(CARDINAL_8 *)current_Screen_Address;
												if (pixel_Neighbors & (0x80 >> ((x_Offset) & 0x07))) {
													is_An_Isolated_Pixel = FALSE;
												}
												
											} /* if (pixel_Neighbors & (0x80 >> ((x_Offset) & 0x07))) */
										
										} /* NOT if (pixel_Neighbors & (0x80 >> ((x_Offset) & 0x07))) */
									
									} /* NOT if (simple_Pixel_Color != 0) */
								
								} /* NOT if (simple_Pixel_Color != 0) */
							
							} /* NOT if (simple_Pixel_Color != 0) */
						
						} /* NOT if (simple_Pixel_Color != 0) */
					
					} /* NOT if (simple_Pixel_Color != 0) */
					
				}
				
			} /* if (use_Direct_Screen_Access != TRUE) */
			else {
				Get_Single_1_Bit_Pixel_2(current_X, current_Y, &simple_Pixel_Color);
				
				if (simple_Pixel_Color == 1) {
				
					/*  */
					/* Make certain at least one pixel is bordering it and has non-background color */
					/*  */

					is_An_Isolated_Pixel = TRUE;
					
					/*  */
					/* Check the top side */
					/*  */
					x_Offset = current_X;
					y_Offset = current_Y - 1;
					
					Get_Single_1_Bit_Pixel_2(x_Offset, y_Offset, &simple_Pixel_Color);
					if (simple_Pixel_Color != 0) {
						is_An_Isolated_Pixel = FALSE;
					}
					else {
					
						/*  */
						/* Check the bottom side */
						/*  */
						x_Offset = current_X;
						y_Offset = current_Y + 1;
						
						Get_Single_1_Bit_Pixel_2(x_Offset, y_Offset, &simple_Pixel_Color);
						if (simple_Pixel_Color != 0) {
							is_An_Isolated_Pixel = FALSE;
						}
						else {
						
							/*  */
							/* Check the Top left side */
							/*  */

							x_Offset = current_X - 1;
							y_Offset = current_Y - 1;
							
							Get_Single_1_Bit_Pixel_2(x_Offset, y_Offset, &simple_Pixel_Color);
							if (simple_Pixel_Color != 0) {
								is_An_Isolated_Pixel = FALSE;
							}
							else {
							
								/*  */
								/* Check the Bottom left side */
								/*  */
								
								x_Offset = current_X - 1;
								y_Offset = current_Y + 1;
								
								Get_Single_1_Bit_Pixel_2(x_Offset, y_Offset, &simple_Pixel_Color);
								if (simple_Pixel_Color != 0) {
									is_An_Isolated_Pixel = FALSE;
								}
								else {
								
									/*  */
									/* Check the top right side */
									/*  */
									
									x_Offset = current_X + 1;
									y_Offset = current_Y - 1;
									
									Get_Single_1_Bit_Pixel_2(x_Offset, y_Offset, &simple_Pixel_Color);
									if (simple_Pixel_Color != 0) {
										is_An_Isolated_Pixel = FALSE;
									}
									else {
									
										/*  */
										/* Check the bottom right side */
										/*  */
										x_Offset = current_X + 1;
										y_Offset = current_Y + 1;
										
										Get_Single_1_Bit_Pixel_2(x_Offset, y_Offset, &simple_Pixel_Color);
										if (simple_Pixel_Color != 0) {
											is_An_Isolated_Pixel = FALSE;
										}
										else {
										
											/*  */
											/* At this point all pixels in all directions except for left and right */
											/* have been compared. In this program no neighbors count to */
											/* left and right unless BOTH are set */
											/*  */
										
											x_Offset = current_X + 1;
											y_Offset = current_Y;
											
											Get_Single_1_Bit_Pixel_2(x_Offset, y_Offset, &simple_Pixel_Color);
											if (simple_Pixel_Color != 0) {
											
												/*  */
												/* ENHANCEMENT : DOES NOT COUNT UNLESS OTHER SIDE IS PIXEL TOO */
												/* Check the left side */
												/*  */
												
												x_Offset = current_X - 1;
												y_Offset = current_Y;
												
												Get_Single_1_Bit_Pixel_2(x_Offset, y_Offset, &simple_Pixel_Color);
												if (simple_Pixel_Color != 0) {
													is_An_Isolated_Pixel = FALSE;
												}
												
											} /* if (pixel_Neighbors & (0x80 >> ((x_Offset) & 0x07))) */
										
										} /* NOT if (simple_Pixel_Color != 0) */
									
									} /* NOT if (simple_Pixel_Color != 0) */
								
								} /* NOT if (simple_Pixel_Color != 0) */
							
							} /* NOT if (simple_Pixel_Color != 0) */
						
						} /* NOT if (simple_Pixel_Color != 0) */
					
					} /* NOT if (simple_Pixel_Color != 0) */
					
				} /* if (simple_Pixel_Color == 1) */
				
			} /* NOT if (use_Direct_Screen_Access == TRUE) */
			

			if (is_An_Isolated_Pixel == TRUE) {
			
				if (use_Direct_Screen_Access != TRUE) {  
					
					x_Offset = current_X;
					current_Screen_Address = raw_Pixel_Address + (x_Offset >> 3);
					current_Screen_Address = current_Screen_Address 
											+ ((CARDINAL_32)(current_Y) * bytes_Per_Row);
		
					pixel_Neighbors = *(CARDINAL_8 *)current_Screen_Address;
					*(CARDINAL_8 *)current_Screen_Address
														= pixel_Neighbors & (~(0x80 >> ((x_Offset) & 0x07)));

				}
				else {
					Draw_Single_1_Bit_Pixel_2(current_X, current_Y, 0);
				}
				
				/*  */
				/* Reset Flip/Flop switch. Speed technique */
				/*  */
				is_An_Isolated_Pixel = FALSE;
			}
	
		} /* for (current_X = 0;  ... */
	} /* for (current_Y = 0;  ... */


	function_Result = SAFE_TRUE;
	goto Normal_Function_Exit;
	
Error_Handler:
	Display_Serious_Error_Message(unit_Name
										, code_Name
										, "\pProblem encountered"
										, "\pCorrupted? bad programming bounds");
										
	function_Result = SAFE_FALSE;
	
Normal_Function_Exit:
	return(function_Result);
	
} /* Safe_Boolean Remove_Stray_Content_Pixels */
/*=**************************************************************************=*/


/*=**************************************************************************=*/
void Copy_Direct_Non_Masked_Bits(GrafPtr offscreen_Graph_Ptr
				, Safe_Boolean		use_Fancy_Bit_Map
				, GrafPtr					simple_Bit_Map_Ptr
				, CGrafPtr				fancy_Bit_Map_Ptr
				, Rect *source_Rect
				, Rect *target_Rect
				, INTEGER_32 mask_Color_Total
				, Color_Type mask_Colors[])
/*  */
/* Creates mask but does not allow isolated pixels in the region */
/* for quality. takes a hell of a long time. GetCPixel is avoided if direct is possible */
/* This only uses the first color in the background map set, not the full list */
/*  */
{
	register	INTEGER_32	current_X;
	register	INTEGER_32	current_Y;
	register	INTEGER_32	current_Length;
	register	INTEGER_32	current_Height;
	register	CARDINAL_8  pixel_Neighbors;
	
	INTEGER_32		current_Left_Most_Background_Point_X;
	INTEGER_32		current_Left_Most_Background_Point_Y;
	INTEGER_32		tripled_Max_Color_Range;
	INTEGER_32		current_Intensity_Deviance;
	RGBColor			main_background_Color;
	RGBColor			current_Color;
	RGBColor			adjusted_Color;
	RGBColor			current_Left_Most_Background_Color;
	
	CARDINAL_32	brightened_Color;
	Boolean			red_Deviance_Negative_Slope;
	Boolean			green_Deviance_Negative_Slope;
	Boolean			blue_Deviance_Negative_Slope;

	INTEGER_32		main_background_Color_Intensity;
	INTEGER_32		current_Color_Intensity;
	INTEGER_32		red_Deviance;
	INTEGER_32		blue_Deviance;
	INTEGER_32		green_Deviance;
	INTEGER_32		red_Difference;
	INTEGER_32		blue_Difference;
	INTEGER_32		green_Difference;
	INTEGER_32		brightness_Deviance;
	
	INTEGER_32	target_X_Offset;
	INTEGER_32	target_Y_Offset;
	INTEGER_32	source_X_Offset;
	INTEGER_32	source_Y_Offset;
	INTEGER_32	x_Offset;
	INTEGER_32	y_Offset;
	INTEGER_32	stored_Bit_Depth;
	CARDINAL_32	raw_Pixel_Address;
	CARDINAL_32	bytes_Per_Row;
	Rect				bounding_Rect;
	CARDINAL_32	current_Screen_Address;
	Boolean				allow_Wider_Deviance;
	Boolean				use_Apples_Color_Routines;
	Boolean				use_Direct_Screen_Access;
	Boolean				pixel_Is_Main_Mask_Color;
	Boolean				pixel_Possible_Mask;
	INTEGER_8			predominant_Color;
	CARDINAL_8		*code_Name;
/* END VAR */


	code_Name = "\pCopy_Non_Masked_Bits";
	
	/*  */
	/* Ordinarily set to FALSE for safety */
	/*  */
	use_Apples_Color_Routines = TRUE;
	use_Direct_Screen_Access = FALSE;
	
	Set_View_Port(offscreen_Graph_Ptr);
	
	stored_Bit_Depth = (**(((CGrafPtr)offscreen_Graph_Ptr)->portPixMap)).pixelSize;	
	
	if (use_Apples_Color_Routines == TRUE) {
		if (Initialize_Graphics_Primitives(FALSE, TRUE) != TRUE) {
			Display_Serious_Error_Message(unit_Name, code_Name, "\pInitialize_Graphics_Primitives failed.  Program will use indirect access.", "\p");
		}
		else {
			use_Direct_Screen_Access = TRUE;
		}
	}

	if (((mask_Colors[0]).use_Raw_Index == SAFE_TRUE)
		 || ((stored_Bit_Depth != 16) && (stored_Bit_Depth != 24) && (stored_Bit_Depth != 32) && (stored_Bit_Depth != 48))){
		Display_Serious_Error_Message(unit_Name, code_Name, "\pBad Programming. Indexed color not expected", "\pBad Transfer.");
		goto Normal_Function_Exit;
	}
	
/* ===
	tripled_Max_Color_Range = 0x00010000;
	tripled_Max_Color_Range = tripled_Max_Color_Range * 3L;
===== */
	
	main_background_Color = (mask_Colors[0]).rgb_Color;
	
	if ((main_background_Color.blue >= main_background_Color.red) && (main_background_Color.blue >= main_background_Color.green)) {
		predominant_Color = 0;
	}
	else {
		if ((main_background_Color.green >= main_background_Color.red) && (main_background_Color.green >= main_background_Color.blue)) {
			predominant_Color = 1;
		}
		else {
			predominant_Color = 2;
		}
	}
	
	main_background_Color_Intensity = (CARDINAL_32)main_background_Color.red 
													+ (CARDINAL_32)main_background_Color.blue 
													+ (CARDINAL_32)main_background_Color.green;

	/*  */
	/* Initialize special color-ramp tracking markers */
	/* These are advanced as needed, when used */
	/*  */
	current_Left_Most_Background_Point_X = -1;
	current_Left_Most_Background_Point_Y = -1;
	
	/*  */
	/* For direct port writing into later */
	/*  */
	if (use_Fancy_Bit_Map == SAFE_TRUE) {
		raw_Pixel_Address = (CARDINAL_32)(GetPixBaseAddr(fancy_Bit_Map_Ptr->portPixMap));
		if (raw_Pixel_Address == 0) {
			goto Error_Handler;
		}
		bytes_Per_Row = ((*(fancy_Bit_Map_Ptr->portPixMap))->rowBytes & (~COLOR_PORT_PORTBITS_CHECK));
		bounding_Rect = (*(fancy_Bit_Map_Ptr->portPixMap))->bounds;				
	}
	else {
		raw_Pixel_Address = (CARDINAL_32)(*simple_Bit_Map_Ptr).portBits.baseAddr;
		bytes_Per_Row = (*simple_Bit_Map_Ptr).portBits.rowBytes;
		bounding_Rect = (*simple_Bit_Map_Ptr).portBits.bounds;					
	}
	
	target_X_Offset = (*target_Rect).left;
	target_Y_Offset = (*target_Rect).top;
	source_X_Offset = (*source_Rect).left;
	source_Y_Offset = (*source_Rect).top;
	current_Length = (*source_Rect).right - (*source_Rect).left;
	current_Height = (*source_Rect).bottom - (*source_Rect).top;
	
	/*  */
	/* Process the pixels removing obvious background and */
	/* trying to keep track of leftmost and top most deleted colors. */
	/* so that near matches of adjacent colors may drift across frame */
	/*  */
	for (current_Y = 0; (current_Y < current_Height); current_Y++) {
		
		if (current_Y == 44) {
			current_Y = 44;
		}
	
		for (current_X = 0; (current_X < current_Length); current_X++) {

			x_Offset = current_X + source_X_Offset;
			y_Offset = current_Y + source_Y_Offset;

			if (use_Direct_Screen_Access == TRUE){
				/*  */
				/* This gets the pixel in a fast manner involving no multiplication */
				/*  */
			/* ===
				Get_Pixel_Index(x_Offset, y_Offset, &pixel_Color);
			=== */
			
				Get_Color_Data_Pixel_32_Bit_Subprimitive(x_Offset, y_Offset, &current_Color);
			}
			else {
				GetCPixel(x_Offset, y_Offset, &current_Color);
			}
	
			if ((current_Color.red != main_background_Color.red)
				|| (current_Color.blue != main_background_Color.blue)
				|| (current_Color.green != main_background_Color.green)) {
					pixel_Is_Main_Mask_Color = FALSE;
			}
			else {
				pixel_Is_Main_Mask_Color = TRUE;
			}
	
			if (pixel_Is_Main_Mask_Color != TRUE) {
	
				/*  */
				/* Was not an obvious match, usually is the case. */
				/*  */
				pixel_Possible_Mask = TRUE;

				/*  */
				/* Try throwing out color based on BLUE NOT BEING PREDOMINANT */
				/*  */
				switch(predominant_Color) {
					case 0:
						/*  */
						/* Blue is largest intensity, see if still valid */
						/*  */
						if ((current_Color.blue < current_Color.red) || (current_Color.blue < current_Color.green)) {
							/*  */
							/* Eliminate this pixel, its not proper hue */
							/*  */
							pixel_Possible_Mask = FALSE;
						}
					break;
	
	
					case 1:
						/*  */
						/* Green is largest intensity, see if still valid */
						/*  */
						if ((current_Color.green < current_Color.red) || (current_Color.green < current_Color.blue)) {
							/*  */
							/* Eliminate this pixel, its not proper hue */
							/*  */
							pixel_Possible_Mask = FALSE;
						}
						else {
						
						}
					break;

					case 2:
					default:
						/*  */
						/* Red is largest intensity, see if still valid */
						/*  */
						if ((current_Color.red < current_Color.blue) || (current_Color.red < current_Color.green)) {
							/*  */
							/* Eliminate this pixel, its not proper hue */
							/*  */
							pixel_Possible_Mask = FALSE;
						}
				} /* switch(predominant_Color) */


				if (pixel_Possible_Mask == TRUE) {
				
					/*  */
					/* So far it all seems proper, pixel is in 1/3rd set of proper */
					/* channel color bias to be within set of background hues. */
					/* Now the intensity (power) of color is adjusted so */
					/* the sample we are using is in the same basic general intensity. */
					/*  */
														
					current_Color_Intensity = (CARDINAL_32)current_Color.red 
													+ (CARDINAL_32)current_Color.blue 
													+ (CARDINAL_32)current_Color.green;
					
					adjusted_Color = current_Color;
									
					if (current_Color_Intensity > main_background_Color_Intensity) {
						/*  */
						/* Darken color, it is brighter than reference test color. */
						/* Darken by simple integer division . */
						/* This technique avoids having to use floating point */
						/* Critical to remember to promote to 32 bit value in intermediate, naturally */
						/* because of C defects */
						/*  */
						brightness_Deviance = current_Color_Intensity - main_background_Color_Intensity;
						

						if (stored_Bit_Depth < 24) {
							if (brightness_Deviance > g_Options.maximum_Brightness_Deviance_5) {
								pixel_Possible_Mask = FALSE;
							}
						}
						else {
							if (brightness_Deviance > g_Options.maximum_Brightness_Deviance_8) {
								pixel_Possible_Mask = FALSE;
							}
						}
						
						if (pixel_Possible_Mask == TRUE) {
							/*  */
							/* Darken */
							/*  */
							if (adjusted_Color.red > 0) {
								adjusted_Color.red = ((CARDINAL_32)adjusted_Color.red * main_background_Color_Intensity) / current_Color_Intensity;
							}
							
							if (adjusted_Color.blue > 0) {
								adjusted_Color.blue = ((CARDINAL_32)adjusted_Color.blue * main_background_Color_Intensity) / current_Color_Intensity;
							}
							
							if (adjusted_Color.green > 0) {
								adjusted_Color.green = ((CARDINAL_32)adjusted_Color.green * main_background_Color_Intensity) / current_Color_Intensity;
							}
							
						} /* if (pixel_Possible_Mask == TRUE) */
						
					} /* if (current_Color_Intensity > main_background_Color_Intensity)  */
					else if (current_Color_Intensity < main_background_Color_Intensity) {
						/*  */
						/* Lighten color, it is darker than reference test color. */
						/* Lighten by simple integer multiplication. */
						/* This technique avoids having to use floating point */
						/*  */
						brightness_Deviance = main_background_Color_Intensity - current_Color_Intensity;
							
						if (stored_Bit_Depth < 24) {
							if (brightness_Deviance > g_Options.maximum_Darkness_Deviance_5) {
								if (current_Color_Intensity < g_Options.bright_Enough_To_Trust_For_Brighten) {
									pixel_Possible_Mask = FALSE;
								}
							}
						}
						else {
							if (brightness_Deviance > g_Options.maximum_Darkness_Deviance_8) {
								if (current_Color_Intensity < g_Options.bright_Enough_To_Trust_For_Brighten) {
									pixel_Possible_Mask = FALSE;
								}
							}
						}
							
						if (pixel_Possible_Mask == TRUE) {
						
							brightened_Color = adjusted_Color.red;
							if (brightened_Color > 0) {
								brightened_Color = (brightened_Color * main_background_Color_Intensity) / current_Color_Intensity;
								if ((brightened_Color & 0xFFFF0000) != 0) {
									brightened_Color = 0x0000FFFF;
								}
								adjusted_Color.red = brightened_Color;
							}
							
							brightened_Color = adjusted_Color.blue;
							if (brightened_Color > 0) {
								brightened_Color = (brightened_Color * main_background_Color_Intensity) / current_Color_Intensity;
								if ((brightened_Color & 0xFFFF0000) != 0) {
									brightened_Color = 0x0000FFFF;
								}
								adjusted_Color.blue = brightened_Color;
							}
							
							brightened_Color = adjusted_Color.green;
							if (brightened_Color > 0) {
								brightened_Color = (brightened_Color * main_background_Color_Intensity) / current_Color_Intensity;
								if ((brightened_Color & 0xFFFF0000) != 0) {
									brightened_Color = 0x0000FFFF;
								}
								adjusted_Color.green = brightened_Color;
							}
							
					/* ===		
							current_Intensity_Deviance = main_background_Color_Intensity - current_Color_Intensity;
							current_Intensity_Deviance = (tripled_Max_Color_Range + current_Intensity_Deviance);
							adjusted_Color.red = ((CARDINAL_32)adjusted_Color.red * current_Intensity_Deviance) / tripled_Max_Color_Range;
							adjusted_Color.blue = ((CARDINAL_32)adjusted_Color.blue * current_Intensity_Deviance) / tripled_Max_Color_Range;
							adjusted_Color.green = ((CARDINAL_32)adjusted_Color.green * current_Intensity_Deviance) / tripled_Max_Color_Range;
					==== */
						} /* if (pixel_Possible_Mask == TRUE) */

					}
					else {
						/*  */
						/* Color is correct intensity already */
						/*  */
						
					} /* NOT if (current_Color_Intensity < main_background_Color_Intensity) */
					
					
					if (pixel_Possible_Mask == TRUE) {
						/*  */
						/* We have an adjusted source color to compare now */
						/*  */					
						if (adjusted_Color.red < main_background_Color.red) {
							red_Deviance = main_background_Color.red - adjusted_Color.red;
							red_Deviance_Negative_Slope = TRUE;
						}
						else {
							red_Deviance = adjusted_Color.red - main_background_Color.red;
							red_Deviance_Negative_Slope = FALSE;
						}
						
						if (adjusted_Color.green < main_background_Color.green) {
							green_Deviance = main_background_Color.green - adjusted_Color.green;
							green_Deviance_Negative_Slope = TRUE;
						}
						else {
							green_Deviance = adjusted_Color.green - main_background_Color.green;
							green_Deviance_Negative_Slope = FALSE;
						}
						
						if (adjusted_Color.blue < main_background_Color.blue) {
							blue_Deviance = main_background_Color.blue - adjusted_Color.blue;
							blue_Deviance_Negative_Slope = TRUE;
						}
						else {
							blue_Deviance = adjusted_Color.blue - main_background_Color.blue;
							blue_Deviance_Negative_Slope = FALSE;
						}
						
						/*  */
						/* Allow deviance for major color to differ by 200% vs other channels */
						/*  */

						switch(predominant_Color) {
							case 0:
								/*  */
								/* Blue is largest intensity, drop it 50% */
								/*  */
								blue_Deviance = blue_Deviance >> 1; 
							break;
			
							case 1:
								/*  */
								/* Green is largest intensity, drop it 50% */
								/*  */
								green_Deviance = green_Deviance >> 1; 
							break;
			
							case 2:
							default:
								/*  */
								/* Red is largest intensity, drop it 50% */
								/*  */
								red_Deviance = red_Deviance >> 1;
								
						} /* switch(predominant_Color) */

						/*  */
						/* Contrast absolute values of color deviances  */
						/*  */
						if (stored_Bit_Depth < 24) {
							if ((red_Deviance <= g_Options.maximum_5_Bit_Red_Channel_Deviance)
								&& (blue_Deviance <= g_Options.maximum_5_Bit_Blue_Channel_Deviance)
								&& (green_Deviance <= g_Options.maximum_5_Bit_Green_Channel_Deviance)) {
								
								/*  */
								/* It meets all general parameters, but see if it matches the */
								/* maximum aggregate intensity value. */
								/* This is an important step for proper governing of color drift control */
								/*  */
								if ((red_Deviance + blue_Deviance + green_Deviance) <= g_Options.maximum_5_Bit_Total_Channel_Deviance) {
								
									pixel_Is_Main_Mask_Color =TRUE;
								}
							}
						}
						else {
							if ((red_Deviance <= g_Options.maximum_8_Bit_Red_Channel_Deviance)
								&& (blue_Deviance <= g_Options.maximum_8_Bit_Blue_Channel_Deviance)
								&& (green_Deviance <= g_Options.maximum_8_Bit_Green_Channel_Deviance)) {
							
								/*  */
								/* It meets all general parameters, but see if it matches the */
								/* maximum aggregate intensity value. */
								/* This is an important step for proper governing of color drift control */
								/*  */
								if ((red_Deviance + blue_Deviance + green_Deviance) <= g_Options.maximum_8_Bit_Total_Channel_Deviance) {
								
									pixel_Is_Main_Mask_Color =TRUE;
								}
							}
							
						} /* NOT if (stored_Bit_Depth < 24) */
						
						
						if (pixel_Is_Main_Mask_Color  != TRUE) {
							/*  */
							/* Still does not match... we need to eliminate */
							/* it using a wider tolerance if other two colors share same deviance */
							/* And primary color is above 40% */
							/*  */
							allow_Wider_Deviance = TRUE;
							switch(predominant_Color) {
								case 0:
									/*  */
									/* Blue is largest intensity, compare if deviances are similar */
									/*  */
									if (red_Deviance_Negative_Slope != green_Deviance_Negative_Slope) {
										if ((red_Deviance > g_Options.insignificant_Deviance)
											&& (green_Deviance > g_Options.insignificant_Deviance)) {
											allow_Wider_Deviance = FALSE;
										}
									}
									else {
										if (current_Color.blue < g_Options.minimum_Intensity_To_Consider_Leniance) {
											allow_Wider_Deviance = FALSE;
										}
									}
								break;
				
								case 1:
									/*  */
									/* Green is largest intensity, compare if deviances are similar */
									/*  */
									if (red_Deviance_Negative_Slope != blue_Deviance_Negative_Slope) {
										if ((red_Deviance > g_Options.insignificant_Deviance)
											&& (blue_Deviance > g_Options.insignificant_Deviance)) {
											allow_Wider_Deviance = FALSE;
										}
									}
									else {
										if (current_Color.green < g_Options.minimum_Intensity_To_Consider_Leniance) {
											allow_Wider_Deviance = FALSE;
										}
									}
								break;
				
								case 2:
								default:
									/*  */
									/* Red is largest intensity, compare if deviances are similar */
									/*  */
									if (green_Deviance_Negative_Slope != blue_Deviance_Negative_Slope) {
										if ((green_Deviance > g_Options.insignificant_Deviance)
											&& (blue_Deviance > g_Options.insignificant_Deviance)) {
											allow_Wider_Deviance = FALSE;
										}
									}
									else {
										if (current_Color.red < g_Options.minimum_Intensity_To_Consider_Leniance) {
											allow_Wider_Deviance = FALSE;
										}
									}
									
							} /* switch(predominant_Color) */
						
							if (allow_Wider_Deviance == TRUE) {
								/*  */
								/* Try again if deviance direction is same */
								/*  */
														
								/*  */
								/* Contrast absolute values of color deviances  */
								/*  */
								if (stored_Bit_Depth < 24) {
									if ((red_Deviance <= g_Options.maximum_5_Bit_Red_Channel_Lenient_Deviance)
										&& (blue_Deviance <= g_Options.maximum_5_Bit_Blue_Channel_Lenient_Deviance)
										&& (green_Deviance <= g_Options.maximum_5_Bit_Green_Channel_Lenient_Deviance)) {
										
										/*  */
										/* It meets all general parameters, but see if it matches the */
										/* maximum aggregate intensity value. */
										/* This is an important step for proper governing of color drift control */
										/*  */
										if ((red_Deviance + blue_Deviance + green_Deviance) <= g_Options.maximum_5_Bit_Total_Channel_Lenient_Deviance) {
										
											pixel_Is_Main_Mask_Color =TRUE;
										}
									}
								}
								else {
									if ((red_Deviance <= g_Options.maximum_8_Bit_Red_Channel_Lenient_Deviance)
										&& (blue_Deviance <= g_Options.maximum_8_Bit_Blue_Channel_Lenient_Deviance)
										&& (green_Deviance <= g_Options.maximum_8_Bit_Green_Channel_Lenient_Deviance)) {
									
										/*  */
										/* It meets all general parameters, but see if it matches the */
										/* maximum aggregate intensity value. */
										/* This is an important step for proper governing of color drift control */
										/*  */
										if ((red_Deviance + blue_Deviance + green_Deviance) <= g_Options.maximum_8_Bit_Total_Channel_Lenient_Deviance) {
										
											pixel_Is_Main_Mask_Color =TRUE;
										}
									}
									
								} /* NOT if (stored_Bit_Depth < 24) */
																
							} /* (allow_Wider_Deviance == TRUE) */
						
						} /* if (pixel_Is_Main_Mask_Color  != TRUE) */
						
					} /* if (pixel_Possible_Mask == TRUE) */
					
															
					/*  */
					/* Third to Last chance to ignore this pixel. */
					/* If pixel did not match tight deviances after intensity */
					/* adjustment, then spend time and effort comparing this pixel */
					/* with previous ACCEPTED background pixels to the LEFT and */
					/* UPWARDS to allow a steep gradual color ramp in the background */
					/* if needed. */
					/* Note that if brightness tolorance way out of deviance, this hue ramp */
					/* is **NOT** allowed */
					/*  */
					if (pixel_Possible_Mask == TRUE) {

						if (pixel_Is_Main_Mask_Color != TRUE) {
						
							/*  */
							/* First try leftmost pixel that was a previous acceptable */
							/* background match */
							/*  */

							if ((g_Options.allow_Horizontal_Drift_Tracking != 0) && (current_Left_Most_Background_Point_X >= 0)) {
								/*  */
								/* There is a topmost color we can use for ramping color comparison */
								/* Need to rely on underflow */
								/*  */	
								
								
								if (current_Color.red < current_Left_Most_Background_Color.red) {
									red_Difference = current_Left_Most_Background_Color.red - current_Color.red;
								}
								else {
									red_Difference = current_Color.red - current_Left_Most_Background_Color.red;
								}
								
								if (current_Color.green < current_Left_Most_Background_Color.green) {
									green_Difference = current_Left_Most_Background_Color.green - current_Color.green;
								}
								else {
									green_Difference = current_Color.green - current_Left_Most_Background_Color.green;
								}
								
								if (current_Color.blue < current_Left_Most_Background_Color.blue) {
									blue_Difference = current_Left_Most_Background_Color.blue - current_Color.blue;
								}
								else {
									blue_Difference = current_Color.blue - current_Left_Most_Background_Color.blue;
								}
								
								/*  */
								/* Allow deviance for major color to differ by 200% vs other channels */
								/*  */
		
								switch(predominant_Color) {
									case 0:
										/*  */
										/* Blue is largest intensity, see if still valid */
										/*  */
										blue_Difference = blue_Difference >> 1; 
									break;
					
									case 1:
										/*  */
										/* Green is largest intensity, see if still valid */
										/*  */
										green_Difference = green_Difference >> 1; 
									break;
					
									case 2:
									default:
										/*  */
										/* Red is largest intensity, see if still valid */
										/*  */
										red_Difference = red_Difference >> 1;
										
								} /* switch(predominant_Color) */
								
								/*  */
								/* Contrast absolute values of color deviances  */
								/*  */
								if (stored_Bit_Depth < 24) {
									if ((red_Difference <= g_Options.maximum_5_Bit_Red_Drift_Deviance)
										&& (blue_Difference <= g_Options.maximum_5_Bit_Blue_Drift_Deviance)
										&& (green_Difference <= g_Options.maximum_5_Bit_Green_Drift_Deviance)) {
										
										/*  */
										/* It meets all general parameters, but see if it matches the */
										/* maximum aggregate intensity value. */
										/* This is an important step for proper governing of color drift control */
										/*  */
										if ((red_Difference + blue_Difference + green_Difference) <= g_Options.maximum_5_Bit_Total_Drift_Deviance) {
											pixel_Is_Main_Mask_Color =TRUE;
										}
									}
								}
								else {
									if ((red_Difference <= g_Options.maximum_8_Bit_Red_Drift_Deviance)
										&& (blue_Difference <= g_Options.maximum_8_Bit_Blue_Drift_Deviance)
										&& (green_Difference <= g_Options.maximum_8_Bit_Green_Drift_Deviance)) {
									
										/*  */
										/* It meets all general parameters, but see if it matches the */
										/* maximum aggregate intensity value. */
										/* This is an important step for proper governing of color drift control */
										/*  */
										if ((red_Difference + blue_Difference + green_Difference) <= g_Options.maximum_8_Bit_Total_Drift_Deviance) {
											pixel_Is_Main_Mask_Color =TRUE;
										}
									}
								}
							} /* if (current_Left_Most_Background_Point_X >= 0) */
							
							/*  */
							/* Second to Last chance to ignore this pixel. */
							/* If still not a background color validate against the previous */
							/* TOPMOST color that was accepted as a background color */
							/* by hunting upward through all pixels vertically until */
							/* there is a gap (background) and thene get that color and then */
							/* compare it, for proper vertical color degradation. */
							/* That is vital. for proper real-world background removal. */
							/*  */
							if ((pixel_Is_Main_Mask_Color != TRUE) && (g_Options.allow_Vertical_Drift_Tracking != 0)){
									
								/*  */
								/* Use unadjusted pixel offests because b&W bitmap is being examined, */
								/* not the color one */
								/*  */
								if (Check_Upward_Previous_Background_Pixel_Matches(&current_Color
																						, current_X
																						, current_Y
																						, target_X_Offset
																						, target_Y_Offset
																						, use_Direct_Screen_Access
																						, stored_Bit_Depth
																						, bytes_Per_Row
																						, raw_Pixel_Address
																						, predominant_Color) == TRUE) {
									pixel_Is_Main_Mask_Color =TRUE;
								}
												
							} /* if (pixel_Is_Main_Mask_Color != TRUE) */
							
							/*  */
							/* If we just determined that this is a drift deviance background */
							/* pixel, we need to see if its drift properties are within */
							/* the limits of what we will accept. Otherwise there is no */
							/* control of the drift and it can be too aggressive. */
							/*  */
							if (pixel_Is_Main_Mask_Color == TRUE) {
							
							
								/*  */
								/* Design of logic preserved "red_Deviance" etc. and we */
								/* can now compare actual deviances from wider norm */
								/* of deviances. Disable match by default and force re-establishment */
								/*  */
								
								pixel_Is_Main_Mask_Color = FALSE;
								/*  */
								/* Contrast absolute values of color deviances  */
								/*  */
								
								if (current_Color_Intensity < main_background_Color_Intensity) {
								
									if (stored_Bit_Depth < 24) {
										if ((red_Deviance <= g_Options.maximum_5_Bit_Red_Channel_Absolute_Low_Limit)
											&& (blue_Deviance <= g_Options.maximum_5_Bit_Blue_Channel_Absolute_Low_Limit)
											&& (green_Deviance <= g_Options.maximum_5_Bit_Green_Channel_Absolute_Low_Limit)) {
											
											/*  */
											/* It meets all general parameters, but see if it matches the */
											/* maximum aggregate intensity value for low brightness cuttoff. */
											/* This is an important step for proper governing of color drift control */
											/*  */
											if ((red_Deviance + blue_Deviance + green_Deviance) <= g_Options.maximum_5_Bit_Total_Channel_Absolute_Low_Limit) {
											
												pixel_Is_Main_Mask_Color =TRUE;
											}
										}
									}
									else {
										if ((red_Deviance <= g_Options.maximum_8_Bit_Red_Channel_Absolute_Low_Limit)
											&& (blue_Deviance <= g_Options.maximum_8_Bit_Blue_Channel_Absolute_Low_Limit)
											&& (green_Deviance <= g_Options.maximum_8_Bit_Green_Channel_Absolute_Low_Limit)) {
										
											/*  */
											/* It meets all general parameters, but see if it matches the */
											/* maximum aggregate intensity value. */
											/* This is an important step for proper governing of color drift control */
											/*  */
											if ((red_Deviance + blue_Deviance + green_Deviance) <= g_Options.maximum_8_Bit_Total_Channel_Absolute_Low_Limit) {
											
												pixel_Is_Main_Mask_Color =TRUE;
											}
										}
										
									} /* NOT if (stored_Bit_Depth < 24) */
								}
								else {
								
									if (stored_Bit_Depth < 24) {
										if ((red_Deviance <= g_Options.maximum_5_Bit_Red_Channel_Absolute_High_Limit)
											&& (blue_Deviance <= g_Options.maximum_5_Bit_Blue_Channel_Absolute_High_Limit)
											&& (green_Deviance <= g_Options.maximum_5_Bit_Green_Channel_Absolute_High_Limit)) {
											
											/*  */
											/* It meets all general parameters, but see if it matches the */
											/* maximum aggregate intensity value for high brightness cuttoff */
											/* This is an important step for proper governing of color drift control */
											/*  */
											if ((red_Deviance + blue_Deviance + green_Deviance) <= g_Options.maximum_5_Bit_Total_Channel_Absolute_High_Limit) {
											
												pixel_Is_Main_Mask_Color =TRUE;
											}
										}
									}
									else {
										if ((red_Deviance <= g_Options.maximum_8_Bit_Red_Channel_Absolute_High_Limit)
											&& (blue_Deviance <= g_Options.maximum_8_Bit_Blue_Channel_Absolute_High_Limit)
											&& (green_Deviance <= g_Options.maximum_8_Bit_Green_Channel_Absolute_High_Limit)) {
										
											/*  */
											/* It meets all general parameters, but see if it matches the */
											/* maximum aggregate intensity value. */
											/* This is an important step for proper governing of color drift control */
											/*  */
											if ((red_Deviance + blue_Deviance + green_Deviance) <= g_Options.maximum_8_Bit_Total_Channel_Absolute_High_Limit) {
											
												pixel_Is_Main_Mask_Color =TRUE;
											}
										}
										
									} /* NOT if (stored_Bit_Depth < 24) */
								} /* NOT if (current_Color_Intensity < main_background_Color_Intensity) */
								
							}
							
						} /* if (pixel_Is_Main_Mask_Color != TRUE) */
													
					} /* if (pixel_Possible_Mask == TRUE) */
					
					/*  */
					/* Appropriate place to place last techniques for pixel matching */
					/* Luminosity test is not governing over these! */
					/*  */
					
					/*  */
					/* Almost absolute last chance to ignore this pixel, if dark. */
					/* If still not a background color as a special optimization hack, */
					/* remove the pixel from consideration if it is pure chroma */
					/* with color gun value from 8 to 16 inclusive and other channels */
					/* too dark ( < 2 ) */
					/*  */
					if ((pixel_Is_Main_Mask_Color != TRUE)
								&& (g_Options.allow_Color_Trapping_Dark_Background_Cuttoff >= main_background_Color_Intensity)){
							
						switch(predominant_Color) {
							case 0:
								/*  */
								/* Blue is largest intensity, see if it is dark but others nearly black */
								/*  */
								if ((current_Color.blue <= g_Options.maximum_Trap_Dark_Background)
									&& (current_Color.blue >= g_Options.minimum_Trap_Dark_Background)) {
									
									if ((current_Color.red <= g_Options.black_Channel_Gun_Limit)
										&& (current_Color.green <= g_Options.black_Channel_Gun_Limit)) {
										
										pixel_Is_Main_Mask_Color = TRUE;
									}
								}
							break;
			
							case 1:
								/*  */
								/* Green  is largest intensity, see if it is dark but others nearly black */
								/*  */
								if ((current_Color.green <= g_Options.maximum_Trap_Dark_Background)
									&& (current_Color.green >= g_Options.minimum_Trap_Dark_Background)) {
									
									if ((current_Color.red <= g_Options.black_Channel_Gun_Limit)
										&& (current_Color.blue <= g_Options.black_Channel_Gun_Limit)) {
										
										pixel_Is_Main_Mask_Color = TRUE;
									}
								}
							break;
										
							case 2:
							default:
								/*  */
								/* Red  is largest intensity, see if it is dark but others nearly black */
								/*  */
								if ((current_Color.red <= g_Options.maximum_Trap_Dark_Background)
									&& (current_Color.red >= g_Options.minimum_Trap_Dark_Background)) {
									
									if ((current_Color.green <= g_Options.black_Channel_Gun_Limit)
										&& (current_Color.blue <= g_Options.black_Channel_Gun_Limit)) {
										
										pixel_Is_Main_Mask_Color = TRUE;
									}
								}
								
						} /* switch(predominant_Color) */
										
					} /* if (pixel_Is_Main_Mask_Color != TRUE) */
			
			
					/*  */
					/* Final absolute last technique. Compare relative values */
					/* of three guns. Allow for Hot bright illumintation. If */
					/* current color is > 75% of max range and if other two */
					/* are > 40 % and if other two channels are in repective intensity order */
					/* then if other two channels are within each other in positive manner.... */
					/* IMPLEMENT later on.... */
					/*  */
					
					/*  */
					/* No more chances */
					/*  */
					
				} /* if (pixel_Possible_Mask == TRUE) */

			} /* if (pixel_Is_Main_Mask_Color != TRUE) */



			/*  */
			/* Mark the pixel for now, it looks like a keeper */
			/*  */
			if (pixel_Is_Main_Mask_Color != TRUE) {

				/*  */
				/* Regular official QD technique, pretty slow */
				/* should already have a black colored pen */
				/*  */
	/* =====
				Set_View_Port(simple_Bit_Map_Ptr);
				x_Offset = current_X + target_X_Offset;
				y_Offset = current_Y + target_Y_Offset;
				MoveTo(x_Offset, y_Offset);
				LineTo(x_Offset, y_Offset);
				Set_View_Port(offscreen_Graph_Ptr);
	====== */
				/*  */
				/* This is a much faster technique of setting a simple pixel black */
				/*  */
				x_Offset = current_X + target_X_Offset;
				current_Screen_Address = raw_Pixel_Address + (x_Offset >> 3);
				current_Screen_Address = current_Screen_Address 
										+ ((CARDINAL_32)(current_Y + target_Y_Offset) * bytes_Per_Row);
	
				pixel_Neighbors = *(CARDINAL_8 *)current_Screen_Address;
				*(CARDINAL_8 *)current_Screen_Address
													= pixel_Neighbors | (0x80 >> ((x_Offset) & 0x07));
	
			} /* if (pixel_Is_Main_Mask_Color == TRUE) */
			else {
				/*  */
				/* Update leftmost stored pixel information. */
				/*  */

				if (x_Offset > current_Left_Most_Background_Point_X) {
					current_Left_Most_Background_Point_X = x_Offset;
					current_Left_Most_Background_Point_Y = y_Offset;
					current_Left_Most_Background_Color = current_Color;
				}

			} /* NOT if (pixel_Is_Main_Mask_Color == TRUE) */
	
		} /* for (current_X = 0; (current_X < current_Length); current_X++)  */

		/*  */
		/* Have to reset the left side color ramp marker */
		/*  */
		current_Left_Most_Background_Point_X = -1;
		current_Left_Most_Background_Point_Y = -1;

	} /* for (current_Y = 0; (current_Y < current_Height); current_Y++) */
	
	/*  */
	/* We are done deleting pixels! */
	/* Now we must go through them a second time and eliminate */
	/* solitary pixels that might be artifacts of dithered background */
	/*  */
	
	if (use_Fancy_Bit_Map == SAFE_TRUE) {
		Set_View_Port((GrafPtr)fancy_Bit_Map_Ptr);
	}
	else {
		Set_View_Port(simple_Bit_Map_Ptr);
	}
	
	use_Direct_Screen_Access = FALSE;
	if (Initialize_Graphics_Primitives(FALSE, TRUE) != TRUE) {
		Display_Serious_Error_Message(unit_Name, code_Name, "\pInitialize_Graphics_Primitives failed.  Program will use indirect access.", "\p");
	}
	else {
		use_Direct_Screen_Access = TRUE;
	}
	
	if (Remove_Stray_Background_Pixels(current_Length
								, current_Height
								, target_X_Offset
								, target_Y_Offset
								, use_Direct_Screen_Access
								, bytes_Per_Row
								, raw_Pixel_Address) != SAFE_TRUE) {
	
	}

	if (Remove_Stray_Content_Pixels(current_Length
								, current_Height
								, target_X_Offset
								, target_Y_Offset
								, use_Direct_Screen_Access
								, bytes_Per_Row
								, raw_Pixel_Address) != SAFE_TRUE) {
	
	}
	
	Set_View_Port(offscreen_Graph_Ptr);

Error_Handler :
	
Normal_Function_Exit:
	; /* NULL */
	
} /* void Copy_Direct_Non_Masked_Bits... */
/*=**************************************************************************=*/


/*=**************************************************************************=*/					
Safe_Boolean	Create_Region_For_Direct_Port_Image(GrafPtr offscreen_Source_Port
															, Rect *source_Rect
															, Safe_Boolean		 	create_Color_Style_Port
															, GrafPort 					*offscreen_Graph_Port
															, Off_Screen_Ptr 			modern_Offscreen_Graph_Port)
{
	Safe_Boolean		function_Result;
	Safe_Boolean		port_Was_Set;
	CARDINAL_32	mask_Color_Total;
	GrafPtr				saved_Original_Port;
	PenState				saved_Pen_State;
	Rect						debug_Rect;
	CARDINAL_8		*code_Name;
	OSErr					os_Error;
	Color_Type			mask_Color[k_MAXIMUM_BACKGROUND_COLORS];
	Color_Type			*mask_Color_Ptr;
/* END VAR */


	code_Name = "\pCreate_Region_For_Direct_Port_Image";
	function_Result = SAFE_FALSE;
		
	GetPort(&saved_Original_Port);
	Set_View_Port(offscreen_Source_Port);
	port_Was_Set = SAFE_TRUE;
	GetPenState (&saved_Pen_State);
	
	/*  */
	/* Actually needed for CopyBits calls , so be certain. */
	/*  */
	ForeColor(blackColor);
	BackColor(whiteColor);

	mask_Color_Ptr = &(mask_Color[0]);
	mask_Color_Total = 0;
	
	if (g_Background_Removal_Options_Ready != SAFE_TRUE) {
		g_Background_Removal_Options_Ready = SAFE_TRUE;
		if (Get_Background_Removal_Options(&g_Options
												, 0
												, 8) != TRUE) {
			/*  */
			/* Do not honor cancel command. */
			/*  */
		}
	}

	
	function_Result = Determine_Masking_Colors(&mask_Color_Total
														, mask_Color_Ptr
														, source_Rect
														, SAFE_FALSE);
														
	if (function_Result != SAFE_TRUE) {
		Perform_OK_Dialog("\pProblem encountered : No identifiable background color. (over 6 colors within 8 pixels of corners?, background cropped from PICT?)");
		goto Error_Handler;
	}
		
	SetPort(saved_Original_Port);

	function_Result = Copy_Rect_Non_Mask_Into_BW_Bit_Map(
																		source_Rect
																		, create_Color_Style_Port
																		,  (offscreen_Graph_Port)
																		, &(modern_Offscreen_Graph_Port->offscreen_Port.color_Port)
																		, SAFE_FALSE
																		, mask_Color_Total
																		, mask_Color_Ptr
																		);
	if (function_Result != SAFE_TRUE) {
		goto Error_Handler;
	}
		
	function_Result = SAFE_TRUE;
	goto Normal_Function_Exit;
	
Error_Handler:
	if (port_Was_Set == SAFE_TRUE) {
		Set_View_Port(offscreen_Source_Port);
		SetPenState (&saved_Pen_State);
		Set_View_Port(saved_Original_Port);
		port_Was_Set = SAFE_FALSE;
	}
	Display_Serious_Error_Message(unit_Name, code_Name, "\pProblem encountered", "\p");
	function_Result = SAFE_FALSE;

Normal_Function_Exit:
	ShowCursor();

	if (port_Was_Set == SAFE_TRUE) {
		Set_View_Port(offscreen_Source_Port);
		SetPenState (&saved_Pen_State);
		Set_View_Port(saved_Original_Port);
	}
	return(function_Result);

} /* Safe_Boolean	Create_Region_For_Direct_Port_Image... */
/*=**************************************************************************=*/


/*=**************************************************************************=*/
/* END OF FILE: Direct_Background_Removal.c */
