/* START OF FILE: Fast_Bit_Copy.c */


/*=**************************************************************************=*/
/* [---BEGIN---] FILE HEADER COMMENTS */
/**
 ** SCREEN VIEW = [Adobe Syntax Black-10, Tabs = 4]
 ** PRINTER VIEW = [Adobe Syntax Black-10, Tabs = 4]
 ** =======================================================================
 ** Written by James Merkle
 ** Fast_Bit_Copy.c - Faster version of very limited copy bits for source mode, same rect, no region, etc.
 ** (C) Copyright 1993 by James Merkle
 ** All rights reserved, except those wishing to "copy" moderate sections of code or files may do so, so long as
 ** such code is never used directly either in animation-related tools or multimedia tools that are  
 ** used in live performance or live broadcast or recorded mass market broadcast or film. Desktop usage not restricted.
 ** Conventional desktop game or entertainment not restricted.
 ** =======================================================================
 **
 **
 ** ============
 ** Revision History
 ** ============
 **
 **  1993.03.16		(James Merkle)
 **						Initial coding started
 **
 **
 **
 **
 ** =================
 ** OTHER INFORMATION
 ** =================
 **  Non-overlapping memory required, same depth required, same rect size, no region, etc.
 **
 **/
/* [----END----] FILE HEADER COMMENTS */
/*=**************************************************************************=*/


/*=**************************************************************************=*/
/* [---BEGIN---] #include AREA */

#include <QDOffscreen.h>

#include "Interface_Utilities.h"
#include "Copy_Compacted_Picture.h"
#include "Fast_Bit_Copy.h"


/* [----END----] #include AREA */
/*=**************************************************************************=*/


/*=**************************************************************************=*/
/* [---BEGIN---] #define AREA */


#define COLOR_PORT_PORTBITS_CHECK 		0xC000

#define STATUS_REGISTER_SUPERVISOR_MODE_BIT  		0x2000
#define STATUS_REGISTER_ALL_INTERRUPTS_OFF_MASK 	0xF8FF
#define STATUS_REGISTER_ALL_INTERRUPTS					0x0700

	/*  */
	/* If true, then even when in user mode instead of supervisor */
	/* mode, the program will attemp illegal modification attepts */
	/* to the status register */
	/*  */
#define ASSUME_APPLE_SUPPLIED_EXCEPTION_HANDLER	TRUE

/* [----END----] #define AREA */
/*=**************************************************************************=*/


/*=**************************************************************************=*/
/* [---BEGIN---] TYPE AREA */


/* [----END----] TYPE AREA */
/*=**************************************************************************=*/


/*=**************************************************************************=*/
/* [---BEGIN---] PROTOTYPE AREA */

void Copy_Bits_Very_Quickly_Old(Ptr source_Byte_Ptr
					, Ptr destination_Byte_Ptr
					, INTEGER_32 source_Row_Bytes
					, INTEGER_32	dest_Row_Bytes
					, INTEGER_16 width
					, INTEGER_16 height
					, INTEGER_16 depth);


void Copy_Bits_From_Compressed_Pict_With_Region(CARDINAL_32	*row_Raw_Data_Ptr
															, CARDINAL_32	*skip_List_Data_Ptr
															, CARDINAL_32	*skip_List_Master_List_Ptr
															, CARDINAL_32	*destination_Word_Ptr
															, CARDINAL_32 destination_Rowbyte_Offset);
															

/* [----END----] PROTOTYPE AREA */
/*=**************************************************************************=*/


/*=**************************************************************************=*/
/* [---BEGIN---] GLOBAL DEFINITION AREA */


/* [----END----] GLOBAL DEFINITION AREA */
/*=**************************************************************************=*/


/*=**************************************************************************=*/
/* [---BEGIN---] LOCAL STATIC DEFINITION AREA */


/* [----END----] LOCAL STATIC DEFINITION AREA */
/*=**************************************************************************=*/


/*=**************************************************************************=*/
void Copy_Bits_Very_Quickly(Ptr source_Byte_Ptr
							, Ptr destination_Byte_Ptr
							, INTEGER_32 source_Row_Bytes
							, INTEGER_32 dest_Row_Bytes
							, INTEGER_16 width
							, INTEGER_16 height
							, INTEGER_16 depth)
{

	INTEGER_8	mode32;
	
/* =======
	if (depth > 8) {
	 	mode32 = true32b;
		SwapMMUMode(&mode32);
	}
======= */

	asm	 {
		move.w	width, d0		; Load pixel width
		mulu.w	depth, d0		; Multiply to get total bit width
		lsr.l		#3, d0			; Divide to get byte width
		
		move.w	d0, d1			; Copy Byte width
		lsr.w		#2, d1			; Divide to get 32 bit width
		
		moveq	#3, d2			; Load word mask
		and.w		d0, d2			; Mask off remaining bytes
				
		cmpi.w	#9, d1					; Compare 32 bit total
		bge.s		@copy_method_2	;  use normal routine if >= 9
		cmpi.w	#9, height				; Compare height total
		bge.s		@copy_method_2	;  use normal routine if >= 9
		
@copy_method_1
		;------------------------------
		movem.l	d3-d5,-(a7)						; Save additional registers to be altered
		
		move.l	source_Row_Bytes, d3			; Store source rowBytes size
		move.l	dest_Row_Bytes, d4				; Store destination rowBytes size
		sub.l		d0, d3								; Optimize source rowBytes to jump by subtracton 
		sub.l		d0, d4								; Optimize destination rowBytes to jump by subtracton
		movea.l	source_Byte_Ptr, a0				; Store offset into srcMap
		movea.l	destination_Byte_Ptr, a1		; Store offset into destMap
		move.w	height, d0							; Store height of copied rect
		
		bra			@Loop_For_Next_Row_End		; Branch to avoid adjusting for "dbra"

	@Loop_For_Next_Row:
	
		move.w	d1, d5									; Restore total_32_Bit_Words for decrement
		bra			@Loop_For_32_Bits_End				; Branch to avoid adjusting for "dbra"
			
	@Loop_For_32_Bits:
		move.l	(a0)+,(a1)+							; Transfer 32 bits of image
	
	@Loop_For_32_Bits_End
		dbra		d5, @Loop_For_32_Bits				; Decrement column loop count
		
		move.w	d2, d5									; Restore total_Extra_Bytes for decrement
		bra			@Loop_For_Final_8_Bits_End		; Branch to avoid adjusting for "dbra"

	@Loop_For_Final_8_Bits:
		move.b	(a2)+, (a3)+							;Transfer 8 bits of image
		
	@Loop_For_Final_8_Bits_End:							
		dbra	d5, @Loop_For_Final_8_Bits				; Decrement column loop count
		
		adda.l		d3, a0									; Increment source_Byte_Ptr by source_Row_Bytes
		adda.l		d4, a1									; Increment destination_Byte_Ptr by dest_Row_Bytes
		
		
	@Loop_For_Next_Row_End:							
		dbra	d0, @Loop_For_Next_Row				; Decrement row loop count		
		
		movem.l	(a7)+,d3-d5							; Restore all alterred registers
		
		bra		@Normal_Function_Exit
		;------------------------------
				
; ------------
	@Repeat_For_128_Big_Move_Bytes:
		dc.w 0xF620, 0x9000							; Move16 (a0+), (a1)+, Transfer 128 bits of image
		dc.w 0xF620, 0x9000							; Move16 (a0+), (a1)+, Transfer 128 bits of image
		dc.w 0xF620, 0x9000							; Move16 (a0+), (a1)+, Transfer 128 bits of image
		dc.w 0xF620, 0x9000							; Move16 (a0+), (a1)+, Transfer 128 bits of image
		
		dc.w 0xF620, 0x9000							; Move16 (a0+), (a1)+, Transfer 128 bits of image
		dc.w 0xF620, 0x9000							; Move16 (a0+), (a1)+, Transfer 128 bits of image
		dc.w 0xF620, 0x9000							; Move16 (a0+), (a1)+, Transfer 128 bits of image
		dc.w 0xF620, 0x9000							; Move16 (a0+), (a1)+, Transfer 128 bits of image
	@Loop_128_Big_Move_End:							
		dbra	d5, @Repeat_For_128_Big_Move_Bytes
		jmp			(a3)							; Jump to do remaining bytes
		
; ------------


@copy_method_2
		;------------------------------
		movem.l	d3-d5/a2-a3,-(a7)				; Save additional registers to be altered
		
		move.l	source_Row_Bytes, d3			; Store source rowBytes size
		move.l	dest_Row_Bytes, d4				; Store destination rowBytes size
		sub.l		d0, d3								; Optimize source rowBytes to jump by subtracton 
		sub.l		d0, d4								; Optimize destination rowBytes to jump by subtracton
		movea.l	source_Byte_Ptr, a0				; Store offset into srcMap
		movea.l	destination_Byte_Ptr, a1		; Store offset into destMap
		move.w	height, d0							; Store height of copied rect
		
		
		moveq	#(1<<5)-1,d5							; Create low 31 mask
		and.w		d1, d5									; Mask remaining 31 long words
		add.w		d5, d5									; Double long word remainder for jump address
		neg.w		d5										; Negate remainder offset for jump calculation
		lea			@Loop_128_End(d5.w), a2			; Load long word remainder jump address
		lsr.w		#5, d1									; Convert long word counter into 128 byte total
		move.l	#0x04000608, d5					; Load remaining bytes jump table
		lsl.w		#3, d2									; Multiply remaining bytes by 8 
		lsr.l		d2, d5									;  and adjust remainder jump table by it
		move.b	d5, d2									; Copy jump offset into a register with a "clean" word
		lea			@Repeat_Final_8_Bits(d2.w), a3	;  and Load byte remainder jump address with it
		
		bra			@Repeat_For_Next_Row_End		; Branch to avoid adjusting for "dbra"
	@Repeat_For_Next_Row:
	
		move.w	d1, d5									; Reload 128 byte total
		
		jmp		(a2)									; Jump to do remaining longs first (and avoid dbra adjust)

; ------------

	@Repeat_For_128_Bytes:
		move.l	(a0)+,(a1)+						; Transfer 32 bits of image
		move.l	(a0)+,(a1)+						; Transfer 32 bits of image
		move.l	(a0)+,(a1)+						; Transfer 32 bits of image
		move.l	(a0)+,(a1)+						; Transfer 32 bits of image
		
		move.l	(a0)+,(a1)+						; Transfer 32 bits of image
		move.l	(a0)+,(a1)+						; Transfer 32 bits of image
		move.l	(a0)+,(a1)+						; Transfer 32 bits of image
		move.l	(a0)+,(a1)+						; Transfer 32 bits of image
		
		move.l	(a0)+,(a1)+						; Transfer 32 bits of image
		move.l	(a0)+,(a1)+						; Transfer 32 bits of image
		move.l	(a0)+,(a1)+						; Transfer 32 bits of image
		move.l	(a0)+,(a1)+						; Transfer 32 bits of image
		
		move.l	(a0)+,(a1)+						; Transfer 32 bits of image
		move.l	(a0)+,(a1)+						; Transfer 32 bits of image
		move.l	(a0)+,(a1)+						; Transfer 32 bits of image
		move.l	(a0)+,(a1)+						; Transfer 32 bits of image
		
		move.l	(a0)+,(a1)+						; Transfer 32 bits of image
		move.l	(a0)+,(a1)+						; Transfer 32 bits of image
		move.l	(a0)+,(a1)+						; Transfer 32 bits of image
		move.l	(a0)+,(a1)+						; Transfer 32 bits of image
		
		move.l	(a0)+,(a1)+						; Transfer 32 bits of image
		move.l	(a0)+,(a1)+						; Transfer 32 bits of image
		move.l	(a0)+,(a1)+						; Transfer 32 bits of image
		move.l	(a0)+,(a1)+						; Transfer 32 bits of image
		
		move.l	(a0)+,(a1)+						; Transfer 32 bits of image
		move.l	(a0)+,(a1)+						; Transfer 32 bits of image
		move.l	(a0)+,(a1)+						; Transfer 32 bits of image
		move.l	(a0)+,(a1)+						; Transfer 32 bits of image
		
		move.l	(a0)+,(a1)+						; Transfer 32 bits of image
		move.l	(a0)+,(a1)+						; Transfer 32 bits of image
		move.l	(a0)+,(a1)+						; Transfer 32 bits of image
		move.l	(a0)+,(a1)+						; Transfer 32 bits of image
		
	@Loop_128_End:	
						
#if !defined(k_ASSUME_68040_ALWAYS)
#error "UNDEFINED PARAMETER!"
#endif
#if  (k_ASSUME_68040_ALWAYS == 0)
		dbra	d5, @Repeat_For_128_Bytes
#else	
		dbra	d5, @Repeat_For_128_Big_Move_Bytes
#endif
	
		jmp			(a3)							; Jump to do remaining bytes
		
		
	@Repeat_Final_8_Bits	
		move.w	(a0)+,(a1)+						; Transfer 16 bits of image
		bra.s		@Repeat_For_Final_8_Bits_End
		move.w	(a0)+,(a1)+						; Transfer part of 24 bits of image
		move.b	(a0)+,(a1)+						; Transfer 8 bits of image
		
	@Repeat_For_Final_8_Bits_End:							
	
		adda.l		d3, a0								; Increment source_Byte_Ptr by source_Row_Bytes
		adda.l		d4, a1								; Increment destination_Byte_Ptr by dest_Row_Bytes
				
	@Repeat_For_Next_Row_End:							
		dbra	d0, @Repeat_For_Next_Row			; Decrement row loop count
		movem.l	(a7)+,d3-d5/a2-a3				; Restore all alterred registers
		;------------------------------
		
@Normal_Function_Exit	
		}

/* =======
	if (depth > 8) {
		SwapMMUMode(&mode32);
	}
======= */

} /* void Copy_Bits_Very_Quickly... */
/*=**************************************************************************=*/


/*=**************************************************************************=*/
void Copy_Bits_Double_Even_Rows(Ptr source_Byte_Ptr
							, Ptr destination_Byte_Ptr
							, INTEGER_32 source_Row_Bytes
							, INTEGER_32 dest_Row_Bytes
							, INTEGER_16 width
							, INTEGER_16 height
							, INTEGER_16 depth)
/*  */
/* This interesting routine can draw the even lines of a structure that */
/* consists of pict data lacking odd field lines. It is unused and was a */
/* useful predecessor of the routine that draws even lines while awaiting */
/* odd data to finish loading from hard disk. It does not spinwait or do */
/* anything fancy. Can be compiled to utilize 68040 128 bit moves. */
/* Remember: Source data needs to consist of just even lines. */
/*  */
{
	INTEGER_8	mode32;
	
/* =======
	if (depth > 8) {
	 	mode32 = true32b;
		SwapMMUMode(&mode32);
	}
======= */

	asm	{
		move.w	width, d0		; Load pixel width
		mulu.w	depth, d0		; Multiply to get total bit width
		lsr.l		#3, d0			; Divide to get byte width
		
		move.w	d0, d1			; Copy Byte width
		lsr.w		#2, d1			; Divide to get 32 bit width
		
		moveq	#3, d2			; Load word mask
		and.w		d0, d2			; Mask off remaining bytes
				
		bra.s		@copy_method_2	;  skip over added area
		
; ------------
		
@copy_method_2
		;------------------------------
		movem.l	d3-d7/a2-a3,-(a7)				; Save additional registers to be altered
		

		move.l	source_Row_Bytes, d3			; Store source rowBytes size
		

		move.l	dest_Row_Bytes, d4				; Store destination rowBytes size
		
		move.l	d0, d7								; Store length offset (byte width) for line doubling up
		
		sub.l		d0, d3								; Optimize source rowBytes to jump by subtraction 
		sub.l		d0, d4								; Optimize destination rowBytes to jump by subtraction
		movea.l	source_Byte_Ptr, a0				; Store offset into srcMap
		movea.l	destination_Byte_Ptr, a1		; Store offset into destMap
		move.w	height, d0							; Store height of copied rect
		
		move.l	#0x80000000, d6				; Set flip flop flag for source line doubling
		
		moveq	#(1<<5)-1,d5							; Create low 31 mask
		and.w		d1, d5									; Mask remaining 31 long words
		add.w		d5, d5									; Double long word remainder for jump address
		neg.w		d5										; Negate remainder offset for jump calculation
		lea			@Loop_128_End(d5.w), a2			; Load long word remainder jump address
		lsr.w		#5, d1									; Convert long word counter into 128 byte total
		move.l	#0x04000608, d5					; Load remaining bytes jump table
		lsl.w		#3, d2									; Multiply remaining bytes by 8 
		lsr.l		d2, d5									;  and adjust remainder jump table by it
		move.b	d5, d2									; Copy jump offset into a register with a "clean" word
		lea			@Repeat_Final_8_Bits(d2.w), a3	;  and Load byte remainder jump address with it
		
		bra			@Repeat_For_Next_Row_End		; Branch to avoid adjusting for "dbra"
	@Repeat_For_Next_Row:
	
		move.w	d1, d5									; Reload 128 byte total
		
		jmp		(a2)									; Jump to do remaining longs first (and avoid dbra adjust)

; ------------

	@Repeat_For_128_Bytes:
		move.l	(a0)+,(a1)+						; Transfer 32 bits of image
		move.l	(a0)+,(a1)+						; Transfer 32 bits of image
		move.l	(a0)+,(a1)+						; Transfer 32 bits of image
		move.l	(a0)+,(a1)+						; Transfer 32 bits of image
		
		move.l	(a0)+,(a1)+						; Transfer 32 bits of image
		move.l	(a0)+,(a1)+						; Transfer 32 bits of image
		move.l	(a0)+,(a1)+						; Transfer 32 bits of image
		move.l	(a0)+,(a1)+						; Transfer 32 bits of image
		
		move.l	(a0)+,(a1)+						; Transfer 32 bits of image
		move.l	(a0)+,(a1)+						; Transfer 32 bits of image
		move.l	(a0)+,(a1)+						; Transfer 32 bits of image
		move.l	(a0)+,(a1)+						; Transfer 32 bits of image
		
		move.l	(a0)+,(a1)+						; Transfer 32 bits of image
		move.l	(a0)+,(a1)+						; Transfer 32 bits of image
		move.l	(a0)+,(a1)+						; Transfer 32 bits of image
		move.l	(a0)+,(a1)+						; Transfer 32 bits of image
		
		move.l	(a0)+,(a1)+						; Transfer 32 bits of image
		move.l	(a0)+,(a1)+						; Transfer 32 bits of image
		move.l	(a0)+,(a1)+						; Transfer 32 bits of image
		move.l	(a0)+,(a1)+						; Transfer 32 bits of image
		
		move.l	(a0)+,(a1)+						; Transfer 32 bits of image
		move.l	(a0)+,(a1)+						; Transfer 32 bits of image
		move.l	(a0)+,(a1)+						; Transfer 32 bits of image
		move.l	(a0)+,(a1)+						; Transfer 32 bits of image
		
		move.l	(a0)+,(a1)+						; Transfer 32 bits of image
		move.l	(a0)+,(a1)+						; Transfer 32 bits of image
		move.l	(a0)+,(a1)+						; Transfer 32 bits of image
		move.l	(a0)+,(a1)+						; Transfer 32 bits of image
		
		move.l	(a0)+,(a1)+						; Transfer 32 bits of image
		move.l	(a0)+,(a1)+						; Transfer 32 bits of image
		move.l	(a0)+,(a1)+						; Transfer 32 bits of image
		move.l	(a0)+,(a1)+						; Transfer 32 bits of image
		
	@Loop_128_End:	
						
#if !defined(k_ASSUME_68040_ALWAYS)
#error "UNDEFINED PARAMETER!"
#endif
#if  (k_ASSUME_68040_ALWAYS == 0)
		dbra	d5, @Repeat_For_128_Bytes
#else	
		dbra	d5, @Repeat_For_128_Big_Move_Bytes
#endif
	
		jmp			(a3)							; Jump to do remaining bytes
		
		
	@Repeat_Final_8_Bits	
		move.w	(a0)+,(a1)+						; Transfer 16 bits of image
		bra.s		@Repeat_For_Final_8_Bits_End
		move.w	(a0)+,(a1)+						; Transfer part of 24 bits of image
		move.b	(a0)+,(a1)+						; Transfer 8 bits of image
		
	@Repeat_For_Final_8_Bits_End:							
	
		;------------------------------
		; Only increment source every other row
		;------------------------------
		swap		d6									; Swap source rowbyte flag (only high bit set and used)
	
		adda.l		d4, a1								; Increment destination_Byte_Ptr by dest_Row_Bytes

		bmi.s		@Repeat_For_Next_Doubled_Even_Row_End	; Jump over source increment if Negative flag from d6 set
		
		adda.l		d3, a0							; Increment source_Byte_Ptr by source_Row_Bytes HALF the times

	@Repeat_For_Next_Row_End:							
		dbra	d0, @Repeat_For_Next_Row			; Decrement row loop count
		bra.s @Restore_Registers_And_Exit
		
		
	@Repeat_For_Next_Doubled_Even_Row_End:
		suba.l		d7, a0					
		dbra		d0, @Repeat_For_Next_Row			; Decrement row loop count
		bra.s 		@Restore_Registers_And_Exit
		
		
; ------------
	@Repeat_For_128_Big_Move_Bytes:
		dc.w 0xF620, 0x9000							; Move16 (a0+), (a1)+, Transfer 128 bits of image
		dc.w 0xF620, 0x9000							; Move16 (a0+), (a1)+, Transfer 128 bits of image
		dc.w 0xF620, 0x9000							; Move16 (a0+), (a1)+, Transfer 128 bits of image
		dc.w 0xF620, 0x9000							; Move16 (a0+), (a1)+, Transfer 128 bits of image
		dc.w 0xF620, 0x9000							; Move16 (a0+), (a1)+, Transfer 128 bits of image
		dc.w 0xF620, 0x9000							; Move16 (a0+), (a1)+, Transfer 128 bits of image
		dc.w 0xF620, 0x9000							; Move16 (a0+), (a1)+, Transfer 128 bits of image
		dc.w 0xF620, 0x9000							; Move16 (a0+), (a1)+, Transfer 128 bits of image
	@Loop_128_Big_Move_End:							
		dbra	d5, @Repeat_For_128_Big_Move_Bytes
		jmp			(a3)							; Jump to do remaining bytes
		
; ------------
		
@Restore_Registers_And_Exit
		movem.l	(a7)+,d3-d7/a2-a3				; Restore all alterred registers

@Normal_Function_Exit

		}

/* =======
	if (depth > 8) {
		SwapMMUMode(&mode32);
	}
======= */

} /* void Copy_Bits_Double_Even_Rows(Ptr source_Byte_Ptr... */
/*=**************************************************************************=*/

	
/*=**************************************************************************=*/
void Copy_Bits_Very_Quickly_Old(Ptr source_Byte_Ptr
					, Ptr destination_Byte_Ptr
					, INTEGER_32 source_Row_Bytes
					, INTEGER_32	dest_Row_Bytes
					, INTEGER_16 width
					, INTEGER_16 height
					, INTEGER_16 depth)
{

	INTEGER_8	mode32;
	INTEGER_16	total_32_Bit_Words;
	INTEGER_16	total_Extra_Bytes;
	INTEGER_16	total_Starting_Bytes;
	INTEGER_16	total_64_Byte_Chunks;
/* END VAR */

	
	total_32_Bit_Words = ((width * depth) >> 3);

	total_Starting_Bytes = ((CARDINAL_32)destination_Byte_Ptr & 0x0003);
	if (total_Starting_Bytes != 0) {
		total_Starting_Bytes = 4 - total_Starting_Bytes;
		total_32_Bit_Words = total_32_Bit_Words - total_Starting_Bytes;
	}

	total_Extra_Bytes = total_32_Bit_Words;
	total_32_Bit_Words = (total_32_Bit_Words >> 2);
	
	total_Extra_Bytes = (total_Extra_Bytes & 0x0003);
	
 /* ===
 	mode32 = true32b;
	SwapMMUMode(&mode32);
=== */


	/*  */
	/* Determine if smaller optimized one is wanted or the */
	/* more complex version is warrented */
	/*  */
	if ((total_32_Bit_Words < 9) && (height < 9)) {
	
		/*  */
		/* The more complex version is warrented. It has unrolled loops */
		/*  */
		asm {
			movem.l d0-d6 /a0-a3, -(sp)	; Save all registers to alter
			
			move.w	height, d0						; Store height of copied rect
			move.w	total_32_Bit_Words, d1		; Store total of 32-bit words
			move.w	total_Extra_Bytes, d2		; Store total of 8-bit words
			move.l	source_Row_Bytes, d3		; Store source rowBytes size
			move.l	dest_Row_Bytes, d4			; Store destination rowBytes size	
			movea.l 	source_Byte_Ptr, a0			; Store offset into srcMap
			movea.l	destination_Byte_Ptr, a1	; Store offset into destMap
			move.w	total_Starting_Bytes, d6		; Store total of 8-bit words

			
		/*  */
		/* d7, a4 are used in controlling the inner loops */
		/*  */
			
			bra			@Loop_For_Next_Row_End		; Branch to avoid adjusting for "dbra"

		@Loop_For_Next_Row:
			movea.L a0, a2				; Restore source_Byte_Ptr for increment
			movea.L a1, a3				; Restore destination_Byte_Ptr for increment
	
	
			move.w	d6, d5				; Restore total_Extra_Bytes for decrement
			
			bra			@Loop_For_Starting_8_Bits_End		; Branch to avoid adjusting for "dbra"

		@Loop_For_Starting_8_Bits:
			move.b	(a2)+, (a3)+								;Transfer 8 bits of image
			
		@Loop_For_Starting_8_Bits_End:							
			dbra	d5, @Loop_For_Starting_8_Bits			; Decrement column loop count

	
			move.w	d1, d5				; Restore total_32_Bit_Words for decrement
			
			bra			@Loop_For_32_Bits_End	; Branch to avoid adjusting for "dbra"
		@Loop_For_32_Bits:
			move.l	(a2)+, (a3)+					; Transfer 32 bits of image
			
		@Loop_For_32_Bits_End:							
			dbra		d5, @Loop_For_32_Bits		; Decrement column loop count
			
			
			move.w	d2, d5				; Restore total_Extra_Bytes for decrement
			
			bra			@Loop_For_Final_8_Bits_End		; Branch to avoid adjusting for "dbra"
			
		@Loop_For_Final_8_Bits:
			move.b	(a2)+, (a3)+				;Transfer 8 bits of image
			
		@Loop_For_Final_8_Bits_End:							
			dbra	d5, @Loop_For_Final_8_Bits		; Decrement column loop count
			
			adda.l		d3, a0				; Increment source_Byte_Ptr by source_Row_Bytes
			adda.l		d4, a1				; Increment destination_Byte_Ptr by dest_Row_Bytes
			
		@Loop_For_Next_Row_End:							
			dbra	d0, @Loop_For_Next_Row	; Decrement row loop count		
	
			movem.l (sp)+, d0-d6/a0-a3 	; Restore all alterred registers
		}
		 
	} /* if ((total_32_Bit_Words < 9) && (height < 9)) */
	else {

		total_64_Byte_Chunks = total_32_Bit_Words;
		total_64_Byte_Chunks = ((CARDINAL_16)total_64_Byte_Chunks >> 4);
		
		total_32_Bit_Words = (total_32_Bit_Words & 0x000F);
		
		/*  */
		/* The more complex version is warrented. It has unrolled loops */
		/*  */
		asm {
			movem.l d0-d7/a0-a3, -(sp)	; Save all registers to alter
			
			move.w	height, d0						; Store height of copied rect
			move.w	total_32_Bit_Words, d1		; Store total of 32-bit words
			move.w	total_Extra_Bytes, d2		; Store total of 8-bit words
			move.l	source_Row_Bytes, d3		; Store source rowBytes size
			move.l	dest_Row_Bytes, d4			; Store destination rowBytes size	
			movea.l 	source_Byte_Ptr, a0			; Store offset into srcMap
			movea.l	destination_Byte_Ptr, a1	; Store offset into destMap
			move.w	total_Starting_Bytes, d6		; Store initial 8-bit words total
			move.w	total_64_Byte_Chunks, d7	; Store total of 64 byte inner loops
			
			bra			@Repeat_For_Next_Row_End		; Branch to avoid adjusting for "dbra"

		@Repeat_For_Next_Row:
			movea.L a0, a2				; Restore source_Byte_Ptr for increment
			movea.L a1, a3				; Restore destination_Byte_Ptr for increment
	
	
			move.w	d6, d5				; Restore total_Extra_Bytes for decrement
			
			bra			@Repeat_For_Starting_8_Bits_End		; Branch to avoid adjusting for "dbra"

		@Repeat_For_Starting_8_Bits:
			move.b	(a2)+, (a3)+								;Transfer 8 bits of image
			
		@Repeat_For_Starting_8_Bits_End:							
			dbra	d5, @Repeat_For_Starting_8_Bits			; Decrement column loop count

			move.w	d7, d5				; Restore total_64_Byte_Chunks for decrement
			
			bra			@Repeat_For_64_Bytes_End		; Branch to avoid adjusting for "dbra"
			
		@Repeat_For_64_Bytes:
			move.l	(a2)+, (a3)+						; Transfer 32 bits of image
			move.l	(a2)+, (a3)+						; Transfer 32 bits of image
			move.l	(a2)+, (a3)+						; Transfer 32 bits of image
			move.l	(a2)+, (a3)+						; Transfer 32 bits of image
			move.l	(a2)+, (a3)+						; Transfer 32 bits of image
			move.l	(a2)+, (a3)+						; Transfer 32 bits of image
			move.l	(a2)+, (a3)+						; Transfer 32 bits of image
			move.l	(a2)+, (a3)+						; Transfer 32 bits of image
			move.l	(a2)+, (a3)+						; Transfer 32 bits of image
			move.l	(a2)+, (a3)+						; Transfer 32 bits of image
			move.l	(a2)+, (a3)+						; Transfer 32 bits of image
			move.l	(a2)+, (a3)+						; Transfer 32 bits of image
			move.l	(a2)+, (a3)+						; Transfer 32 bits of image
			move.l	(a2)+, (a3)+						; Transfer 32 bits of image
			move.l	(a2)+, (a3)+						; Transfer 32 bits of image
			move.l	(a2)+, (a3)+						; Transfer 32 bits of image
			
		@Repeat_For_64_Bytes_End:							
			dbra		d5, @Repeat_For_64_Bytes		; Decrement column loop count
				
			move.w	d1, d5				; Restore total_32_Bit_Words for decrement
			
			bra			@Repeat_For_32_Bits_End	; Branch to avoid adjusting for "dbra"
		@Repeat_For_32_Bits:
			move.l	(a2)+, (a3)+					; Transfer 32 bits of image
			
		@Repeat_For_32_Bits_End:							
			dbra		d5, @Repeat_For_32_Bits		; Decrement column loop count
			
			
			move.w	d2, d5				; Restore total_Extra_Bytes for decrement
			
			bra			@Repeat_For_Final_8_Bits_End		; Branch to avoid adjusting for "dbra"
			
		@Repeat_For_Final_8_Bits:
			move.b	(a2)+, (a3)+				;Transfer 8 bits of image
			
		@Repeat_For_Final_8_Bits_End:							
			dbra	d5, @Repeat_For_Final_8_Bits		; Decrement column loop count
			
			adda.l		d3, a0				; Increment source_Byte_Ptr by source_Row_Bytes
			adda.l		d4, a1				; Increment destination_Byte_Ptr by dest_Row_Bytes
			
		@Repeat_For_Next_Row_End:							
			dbra	d0, @Repeat_For_Next_Row	; Decrement row loop count		
	
			movem.l (sp)+, d0-d7/a0-a3 	; Restore all alterred registers
		}
	 	
	} /* NOT if ((total_32_Bit_Words < 9) && (height < 9)) */

 /* ===
	SwapMMUMode(&mode32);
=== */

} /* Copy_Bits_Very_Quickly_Old */
/*=**************************************************************************=*/


/*=**************************************************************************=*/
void Draw_Bits_Raw_To_Monitor(CARDINAL_32	source_Base_Address
				, CARDINAL_32	source_Row_Bytes
				, CARDINAL_32	compression
				, CARDINAL_32	source_Extra_Offset_X
				, CARDINAL_32	source_Extra_Offset_Y
				, PixMapHandle	destination_Pix_Map_Handle
				, Rect				*initial_Destination_Rect
				, Rect				*monitor_Max_Bounding_Rect
				, CARDINAL_32	preferred_Monitor_Depth)
{				
Fast_Copy_Bits_Rec	copy_Options_Rec;	
/* END VAR */
			
				
	if (Build_Draw_Bits_Raw_Parameters(source_Base_Address
											, source_Row_Bytes
											, compression
											, source_Extra_Offset_X
											, source_Extra_Offset_Y
											, destination_Pix_Map_Handle
											, initial_Destination_Rect
											, monitor_Max_Bounding_Rect
											, preferred_Monitor_Depth
											, &copy_Options_Rec) == SAFE_TRUE) {
		
		Draw_Bits_Raw_To_Monitor_Using_Params(&copy_Options_Rec);
	}
	
} /* void Draw_Bits_Raw_To_Monitor ... */
/*=**************************************************************************=*/


/*=**************************************************************************=*/
Safe_Boolean Build_Draw_Bits_Raw_Parameters(CARDINAL_32	source_Base_Address
				, CARDINAL_32	source_Row_Bytes
				, CARDINAL_32	compression
				, CARDINAL_32	source_Extra_Offset_X
				, CARDINAL_32	source_Extra_Offset_Y
				, PixMapHandle	destination_Pix_Map_Handle
				, Rect				*initial_Destination_Rect
				, Rect				*monitor_Max_Bounding_Rect
				, CARDINAL_32	preferred_Monitor_Depth
				, Fast_Copy_Bits_Ptr	copy_Options)
/*  */
/* Source and dest must be same depth. */
/* Destination Rect must be on one device, else is clipped. */
/* Destination Rect must be in GLOBAL coordinates. */
/* Extra Offsets are usually "0" */
/* This is a useful routine because it does not need source data */
/* to be in a real port. It is ideal for QuickTime Emulation routines. */
/*  */
{
	register	INTEGER_32	current_X;
	register	INTEGER_32	current_Y;
	register	INTEGER_32	current_Length;
	register	INTEGER_32	current_Height;
	Ptr					source_Byte_Address;
	Ptr					destination_Byte_Address;
	Rect				actual_Destination_Rect;
	Rect				*destination_Rect;
	Safe_Boolean		function_Result;
	Boolean			draw_Direct_To_Screen;
/* END VAR */


	function_Result = SAFE_FALSE;
	
	actual_Destination_Rect = *initial_Destination_Rect;
	destination_Rect = &actual_Destination_Rect;
		
	draw_Direct_To_Screen = TRUE;

	if (compression == k_BACKGROUND_FREE_COMPACT_DRAWING) {
		/*  */
		/* This type of picture storage has 4 special structures to deal with */
		/* including row_Raw_Data, skip_List_Data, skip_List_Master_List, and */
		/* scan_Line_Bounds. */
		/* Each is usually in a handle but when taken off a disk structure */
		/* directly they are stored as offsets in the header of a large piece of */
		/* conglomerated data */
		/*  */
		source_Byte_Address = (Ptr)source_Base_Address;
		if (source_Byte_Address == nil) {
			goto Error_Handler;
		}
		
		copy_Options->row_Raw_Data_Ptr = (CARDINAL_32 *)((CARDINAL_32)source_Byte_Address 
																		+ ((Packed_Pict_Ptr)source_Byte_Address)->row_Raw_Data_Offset);
																		
		copy_Options->skip_List_Data_Ptr = (CARDINAL_32 *)((CARDINAL_32)source_Byte_Address 
																		+ ((Packed_Pict_Ptr)source_Byte_Address)->skip_List_Data_Offset);
																		
		copy_Options->skip_List_Master_List_Ptr = (CARDINAL_32 *)((CARDINAL_32)source_Byte_Address 
																		+ ((Packed_Pict_Ptr)source_Byte_Address)->skip_List_Master_List_Offset);
		
		copy_Options->scan_Line_Bounds_Ptr = (CARDINAL_32 *)((CARDINAL_32)source_Byte_Address 
																		+ ((Packed_Pict_Ptr)source_Byte_Address)->scan_Line_Bounds_Offset);
																		
	} /* if ((INTEGER_8)copy_Options->compression == k_BACKGROUND_FREE_COMPACT_DRAWING) */

	if (preferred_Monitor_Depth > 1) {
	
		if (Rect_Is_In_Rect(monitor_Max_Bounding_Rect, destination_Rect) != SAFE_TRUE) {
			/*  */
			/* Readjust Rect and create smaller union bounds */
			/*  */
			if (Rect_Intersects_Rect(monitor_Max_Bounding_Rect, destination_Rect) != SAFE_TRUE) {
				goto Error_Handler;
			}
			else {				
				if (SectRect(monitor_Max_Bounding_Rect, destination_Rect, &actual_Destination_Rect) != TRUE) {
					goto Error_Handler;
				}
				/*  */
				/* The destination has been properly clipped, but now the */
				/* source must be adjusted if it was clipped on the left or top */
				/*  */
				if (actual_Destination_Rect.top > initial_Destination_Rect->top) {
					source_Extra_Offset_Y += (actual_Destination_Rect.top - initial_Destination_Rect->top);
				}	
				
				if (actual_Destination_Rect.left > initial_Destination_Rect->left) {
					source_Extra_Offset_X += (actual_Destination_Rect.left - initial_Destination_Rect->left);
				}				
			}
		}

		/*  */
		/* At this point we have global Rect and local source offsets */
		/* all ready. Now we convert global into local for memory */
		/* indexing, Problem is to not use LocalToGlobal nor GlobalToLocal */
		/* but rather to use "monitor_Max_Bounding_Rect" */
		/* And manually create offsets */
		/*  */
		
		actual_Destination_Rect.top = (actual_Destination_Rect.top - monitor_Max_Bounding_Rect->top);
		actual_Destination_Rect.left = (actual_Destination_Rect.left - monitor_Max_Bounding_Rect->left);
		actual_Destination_Rect.right = (actual_Destination_Rect.right - monitor_Max_Bounding_Rect->left);
		actual_Destination_Rect.bottom = (actual_Destination_Rect.bottom - monitor_Max_Bounding_Rect->top);


		current_Y = actual_Destination_Rect.top;
		current_Height = (actual_Destination_Rect.bottom - current_Y);
		
		current_X = actual_Destination_Rect.left;
		current_Length = (actual_Destination_Rect.right - current_X);

		if (preferred_Monitor_Depth < 8) {
			switch (preferred_Monitor_Depth) {
			
				case 0x1: 
					/*  */
					/* Check for determining if starts on byte boundary*/
					/*  */
					if ((current_X & 0x07) != 0) {
						draw_Direct_To_Screen = FALSE;
					}
					
					/*  */
					/* Check for determining if ends on byte boundary*/
					/*  */
					if ((current_Length & 0x07) != 0) {
						draw_Direct_To_Screen = FALSE;
					}
					
				break;
				
				case 0x2: 
					/*  */
					/* Check for determining if starts on byte boundary*/
					/*  */
					if ((current_X & 0x03) != 0) {
						draw_Direct_To_Screen = FALSE;
					}
					
					/*  */
					/* Check for determining if ends on byte boundary*/
					/*  */
					if ((current_Length & 0x03) != 0) {
						draw_Direct_To_Screen = FALSE;
					}
					
				break;
				
				case 0x4:
					/*  */
					/* Check for determining if starts on byte boundary*/
					/*  */
					if ((current_X & 0x01) != 0) {
						draw_Direct_To_Screen = FALSE;
					}
					
					/*  */
					/* Check for determining if ends on byte boundary*/
					/*  */
					if ((current_Length & 0x01) != 0) {
						draw_Direct_To_Screen = FALSE;
					}
					
				break;
				
				case 0x8: 
				case 16: 
				case 24: 
				case 32: 
					/*  */
					/* Nothing to check, its fine as is */
					/*  */
				break;
				
				default: 
					draw_Direct_To_Screen = FALSE;
					
			} /* switch (preferred_Monitor_Depth) */
		}
		
	}
	else {
		draw_Direct_To_Screen = FALSE;
	}
	
	
	if (draw_Direct_To_Screen != TRUE) {
		goto Error_Handler;
	}
	
	/*  */
	/* Rectangle is assumed to be in proper point orientation  */
	/*  */
	
	/* ===
			if(srcPixelSize != dstPixelSize){ 
				goto Choose_Regular_Copy_Bits;
			}
		==== */
			
	copy_Options->destination_Pix_Map_Handle = destination_Pix_Map_Handle;
	copy_Options->source_Base_Address = source_Base_Address;
	copy_Options->source_Row_Bytes = source_Row_Bytes;
	copy_Options->source_Extra_Offset_X = source_Extra_Offset_X;
	copy_Options->source_Extra_Offset_Y = source_Extra_Offset_Y;
	copy_Options->preferred_Monitor_Depth = preferred_Monitor_Depth;
	copy_Options->actual_Destination_Rect = actual_Destination_Rect;
	copy_Options->current_Length = current_Length;
	copy_Options->current_Height = current_Height;
	copy_Options->compression = compression;
	
	/*  */
	/* Fill default options here */
	/*  */
	copy_Options->disable_Interrupts = 0;
	
	
	function_Result = SAFE_TRUE;
	goto Normal_Function_Exit;
	
Error_Handler:
	function_Result = SAFE_FALSE;
		
Normal_Function_Exit:

	return(function_Result);

} /* void Build_Draw_Bits_Raw_Parameters(CARDINAL_32	source_Base_Address ... */
/*=**************************************************************************=*/


/*=**************************************************************************=*/
void Draw_Bits_Raw_To_Monitor_Using_Params(Fast_Copy_Bits_Ptr copy_Options)
				
/*  */
/* Source and dest must be same depth. */
/* Destination Rect must be on one device, else is clipped. */
/* Destination Rect must be in GLOBAL coordinates. */
/* Extra Offsets are usually "0" */
/* This is a useful routine because it does not need source data */
/* to be in a real port. It is ideal for QuickTime Emulation routines. */
/*  */
{
	Ptr		source_Byte_Address;
	Ptr		destination_Byte_Address;
	CARDINAL_16	condition_Codes, saved_Interrupt_Mask;
	CARDINAL_16	interrupts_Manipulated;
/* END VAR */
	
		
	destination_Byte_Address = nil;
	destination_Byte_Address = GetPixBaseAddr(copy_Options->destination_Pix_Map_Handle);
	if (destination_Byte_Address == nil) {
		goto Error_Handler;
	}
	
	*(CARDINAL_32 *)&destination_Byte_Address += 
						(CARDINAL_32)((CARDINAL_32)((*copy_Options->destination_Pix_Map_Handle)->rowBytes 
						& (~COLOR_PORT_PORTBITS_CHECK))
						* (CARDINAL_32)copy_Options->actual_Destination_Rect.top);

	*(CARDINAL_32 *)&destination_Byte_Address += 
							(((CARDINAL_32)copy_Options->actual_Destination_Rect.left * copy_Options->preferred_Monitor_Depth) >> 3);

		


	/*  */
	/* First Turn off all interrupts if possible, (will blow up on non-68000 if in user mode). */
	/* It will not blow up if Apple writes a future exception handler to run a software */
	/* routine as an exception handler for the privileged mode violation that */
	/* determines that the violation is merely a non-destructive read, and then fills in the info */
	/* for us as a courtesy and continues onward.  This is a likely prospect in the future. */
	/* I do not know of a way to easily read the SR on non-68000's in user mode! Ugh! */
	/*  */
	interrupts_Manipulated = 0;
	if (copy_Options->disable_Interrupts != 0) {
		condition_Codes = 0; /* paranoia init */
		asm{
			move		sr,condition_Codes;		/* reserved on all except 68000 */
		}
	
		if ((condition_Codes & STATUS_REGISTER_SUPERVISOR_MODE_BIT) !=0) {
			interrupts_Manipulated = 1;
			saved_Interrupt_Mask = (condition_Codes & STATUS_REGISTER_ALL_INTERRUPTS);
			asm{
				ori.w		#STATUS_REGISTER_ALL_INTERRUPTS,sr;				
			}
		}
		else {
			if (ASSUME_APPLE_SUPPLIED_EXCEPTION_HANDLER == TRUE){
				interrupts_Manipulated = 1;
				saved_Interrupt_Mask = (condition_Codes & STATUS_REGISTER_ALL_INTERRUPTS);
				asm{
					ori.w		#STATUS_REGISTER_ALL_INTERRUPTS,sr;				
				}
			}
		}
	}

/* ====
#define k_EVEN_ODD_ROW_COMPRESSION

#define k_EVEN_ROW_DOUBLE_DRAWING
#define k_ODD_ROW_SINGLE_DRAW
k_EVEN_ROW_DOUBLE_DRAWING
k_BACKGROUND_FREE_DRAWING
==== */
	/*  */
	/* Not exactly used at this time? */
	/*  */

	if ((INTEGER_8)copy_Options->compression == k_BACKGROUND_FREE_COMPACT_DRAWING) {
		
		/* NOT ACTUALLY USED YET! */
		
		Copy_Bits_From_Compressed_Pict_With_Region_NEW(copy_Options->row_Raw_Data_Ptr
												, copy_Options->skip_List_Data_Ptr
												, copy_Options->skip_List_Master_List_Ptr
												, (CARDINAL_32 *)destination_Byte_Address
												, ((*copy_Options->destination_Pix_Map_Handle)->rowBytes 
														& (~COLOR_PORT_PORTBITS_CHECK)));
	
	}
	else {
	
		/*  */
		/* DANGER : Passing a "Fake Handle" out of desperation (Apple sucks)*/
		/*  */
		
		source_Byte_Address = (Ptr)copy_Options->source_Base_Address;
		
		if (source_Byte_Address == nil) {
			goto Error_Handler;
		}
		
		*(CARDINAL_32 *)&source_Byte_Address += 
							(CARDINAL_32)((CARDINAL_32)(copy_Options->source_Row_Bytes 
							& (~COLOR_PORT_PORTBITS_CHECK))
							* (CARDINAL_32)copy_Options->source_Extra_Offset_Y);
	
		*(CARDINAL_32 *)&source_Byte_Address += 
								(((CARDINAL_32)copy_Options->source_Extra_Offset_X * copy_Options->preferred_Monitor_Depth) >> 3);
	
	
		switch((INTEGER_8)copy_Options->compression) {
		
			case k_EVEN_ROW_DOUBLE_DRAWING:
			
				Copy_Bits_Double_Even_Rows(source_Byte_Address
						, (Ptr)destination_Byte_Address
						, (copy_Options->source_Row_Bytes & (~COLOR_PORT_PORTBITS_CHECK))
						, ((*copy_Options->destination_Pix_Map_Handle)->rowBytes & (~COLOR_PORT_PORTBITS_CHECK))
						, copy_Options->current_Length
						, copy_Options->current_Height
						, copy_Options->preferred_Monitor_Depth);
			break;
			

			case k_NORMAL_RAW_BITMAP_DRAWING:
			default:
				Copy_Bits_Very_Quickly(source_Byte_Address
						, (Ptr)destination_Byte_Address
						, (copy_Options->source_Row_Bytes & (~COLOR_PORT_PORTBITS_CHECK))
						, ((*copy_Options->destination_Pix_Map_Handle)->rowBytes & (~COLOR_PORT_PORTBITS_CHECK))
						, copy_Options->current_Length
						, copy_Options->current_Height
						, copy_Options->preferred_Monitor_Depth);
						
		} /* switch */
			
	} /* NOT if ((INTEGER_8)copy_Options->compression == k_BACKGROUND_FREE_COMPACT_DRAWING) */
	
	

	/*  */
	/* Restore all interrupts to original state if  we are in , will blow up on non-68000 if in user mode */
	/*  */
	if (interrupts_Manipulated != 0) {
		condition_Codes = 0; /* paranoia init */
		asm{
			move		sr,condition_Codes;				/* reserved on all except 68000 */
		}
		if ((condition_Codes & STATUS_REGISTER_SUPERVISOR_MODE_BIT) !=0) {
			condition_Codes = (condition_Codes & STATUS_REGISTER_ALL_INTERRUPTS_OFF_MASK);
			condition_Codes = (condition_Codes | saved_Interrupt_Mask);			/* restore original interrupt_Level */
			asm{
				move		condition_Codes,sr;				
			}
		}
	}/* if (interrupts_Manipulated != 0) */
		
Error_Handler:

Normal_Function_Exit:
	; /* NULL */

} /* void Draw_Bits_Raw_To_Monitor_Using_Params(CARDINAL_32	source_Base_Address ... */
/*=**************************************************************************=*/


/*=**************************************************************************=*/
/* =====

Penalties = empty lines not accounted for (few expected)

 skip list subtraction routine
 
 
 index to previous cells region
 [+1]address of start pixel offset (line count)
 [+2]address of last pixel offset
 
 index to current cells region
 [+1]address of start pixel offset (line count)
 [+2] address of last pixel offset
 
 
load current 
load start of previous

===== */
/*=**************************************************************************=*/


/*=**************************************************************************=*/
void Copy_Bits_From_Compressed_Pict_With_Region(CARDINAL_32	*row_Raw_Data_Ptr
															, CARDINAL_32	*skip_List_Data_Ptr
															, CARDINAL_32	*skip_List_Master_List_Ptr
															, CARDINAL_32	*destination_Word_Ptr
															, CARDINAL_32 destination_Rowbyte_Offset)
{

/* ======

we have source color pixmap, it may be in many small fragments with voids of unimportant areas
we have bounding region with frame rect, it must be equal or less than the source pixmap fragments

we want FAST copy bits for these objects

computation when loading not desired so sometimes it would be nice to 
force destination color table not source color table. Both color convert 
load balance directions needed.

Solution : convert (region & pixMap) to (fast_Region & skip_Map)

Advantages of this two part structure:

(1) Bidirectional in vertical directions allowing inverted image if needed
(2) Bidirectional in vertical directions allowing bottom up drawing if needed
(3) Might be faster than "byte"-biased unoptimized RLE because only 32 aligned  bit read/writes are used.
(4) Code cache not exhausted as with move immediates from pure shape blit compilers
(5) Has zero cycle load/setup time despite different rowbyte destination unlike pure shape blit compilers
(6) Could be used with 16 color, 4 color, and 1 color data with very little adjustment. Source&Dest equality needed.
(7) Extra sets of different regions can be used with one shared skip_Map (as long
as skip_Map is combined regions of all) This adds extra flexibility a 
standard RLE will not have because a standard RLE is one atomic structure.
(8) Region or Source is allowed to be infinitely sparse in vertical and horizontal directions with 0 cycle loss
(9) Source can be any arbitrary size withing confines of source rect at any non-normalized offset
(10) Source can be any number of fragments including "0"
(11) Copy can be modified to clip top and bottom very very easy.
(12) Secondary Copy routine could be created to clip horizontal directions, but not done yet.
(13) Everything is in handles and none need to be locked for ANY structures containing data during entire call
(14) Much faster than RLE technologies

Disadvantages (Minor):
(1) slower for small non-sparse sprites than an optimal arcade shape compiler lacking many advantages listed above
(2) For 8 bit data can only be used for non-32 bit aligned offset destinations (bit offset: 0,8,16,24) but slower
than pre shifted copies of those byte offsets. Motorola/Mac limit not source code enforced.
(3) For arbitrary pixel offset into destination at bit depths loser than 8, preshifted pre calculated
sprites at harmonic offsets allowing at least byte offsetting is needed. 1 bit would require 8 copies!
(4) Compression is not RLE but compression is memory friendly for unneeded masked out areas not part of any expected region.
(5) Because it is not RLE, images can be very complex, but cannot be index color mapped in transfer loop
(6) Because not RLE, rotation effects based on RLE technologies have to do a few more instructions

====== */

asm {
	movem.l d0-d6/a0-a4, -(sp)	; Save all registers to alter
	
	
	move.l	row_Raw_Data_Ptr(a6), a3	; Load row_Raw_Data_Ptr, already locked down ? not needed
	move.l	skip_List_Data_Ptr(a6), d5	; Load skip_List_Data_Ptr, already locked down ? not needed
	move.l	skip_List_Master_List_Ptr(a6), a4	; Load skip_List_Master_List_Ptr, 
														;  first element address of binary row/skip table triple list
	add.l		#4, a4									; avoid the safety "count" sentinel at start of list, 
	
	move.l	destination_Rowbyte_Offset(a6), d4	; Set destination rowbyte offset
	move.l	destination_Word_Ptr(a6), d6			; Load destination for picture (upper left of rect), 
	sub.l		d4, d6										; Pre-decrement to normalize for looping incrementer
															;  If this drew in reverse order it would have been added
	
	bra.s		@End_Of_Line							; Optimize out a branch test by starting loop here
	
@Repeat_Start:
	
	move.l	(a4)+, d1			; Get Address of Row Start Raw Data, absolute offset
	add.l		d0, a1				; Adjust Skip List Entry by adding offset from triplet (could be optimized if needed)
	
	add.l		d1, a0				; This addition could be optimized into an absolute if needed in future
									;  a0 thus now equals raw start address of source 
									;  a0 already equals a3, a1 reset to d5, and d0 already was reset to via a4
	
	move.l	(a4)+, d0			; Get Get total source-dest lines to skip
	
/* ==
	beq		@Simple_Add_Row_Byte	; If 0, zero then add d4 to (d6) because this is base "0" else jump
=== */

	bra			@Loop_End_Adjust_Row_Destination	; Base already 0, Branch to adjust for "dbra"
@Loop_Start_Adjust_Row_Destination:
	add.l		d4, d6				; Increment destination pointer by rowbyte offset one time
@Loop_End_Adjust_Row_Destination:
	dbra		d0, @Loop_Start_Adjust_Row_Destination
		
	move.l	d6, a2				; Copy d6 to a2 for use in prime loop, d6 is to be preserved in incremented form.

/* ===

At this point we have:
a0 : points to Row Scan line raw data for this line of pixels to transfer (could be a packed picture)
a1 : points to Skip List Entry that will be used to skip over and select just what we want to copy
a2 : points to destination 32bit word unadjusted. Skip runs of this will appear simultaneous with source

Those are the only three values we need to process a horizontal skip list. Also:

a3 : holds Row Raw Data main pointer for next loop through source, if any more
a4 : points to next skip list data tuplet for next loop through source, if any more
d4 : is equal to destination rowbyte offset for next loop through source, if any more
d5 : holds Skip List main pointer for next loop through source, if any more
d6 : holds virgin incremented copy of a2 for next loop through source, if any more
 
  Free unused in routine : just d7
  
=== */

	;_Debugger

	move.l (a1)+,d0				; get front source "Or" mask
	beq @End_Of_Line			; If no source pixels at all exist then the line is finished
	add.l (a1)+,a0				; get 32bit word source skip offset representing region additional offset
	move.l d0, d1				; Preserve a copy of the source mask
	add.l (a1)+,a2				; adjust destination 32bit word skip to be at parity with compressed source, if needed
									; note : this value was precomputed as equal to itself in addition to a copy of previous value
	
	not.l d0							; Invert front mask to create write through mask
	beq @No_Front_Mask_Needed	; If this write through mask is zero then there are no bits to mask


	move.l	(a2), d2				; Get next 32bit word of destination
	and.l		d1, d2					; Mask dest area clear for placement of source pixels
	move.l	(a0)+,d3				; Get next source 32bit word
	and.l		d0, d3					; Mask off unwanted bits of source
	or.l			d3, d2					; assemble masked pieces
	move.l	d2, (a2)+				; Write finished pixels to destination and increment
	
	
@No_Front_Mask_Needed:
	move.l (a1)+, d0				; Get dbra count for long transfer loop
	dbgt	 d0, @Final_Skip_Section
	
; This could be sped up  several ways. We will precompute two integers for Loop and offset jump
; The long just loaded will be swapped and have two words in it.

@Loop_Copy_Central_Section:
	move.l	(a0)+, (a2)+		; Non-optimal 32Bit transfer loop, add move16 style soon
	dbf			d0, @Loop_Copy_Central_Section
	
@Final_Skip_Section: 			;(Could be optimized using theoretical 8 bit speedup technique also)

	move.l (a1)+,d0				; get next source "Or" mask
	beq @End_Of_Line			; Exit if no source pixels at all exist then the line is finished
	add.l (a1)+,a0				; get 32bit word source skip offset representing region additional offset
	move.l d0, d1				; Preserve a copy of the source mask
	add.l (a1)+,a2				; adjust destination 32bit word skip to be at parity with compressed source, if needed
									; note : this value was precomputed as equal to itself in addition to a copy of previous value
	
	not.l d0						; Invert front mask to create write through mask

	move.l	(a2), d2			; Get next 32bit word of destination
	and.l		d1, d2				; Mask dest area clear for placement of source pixels
	move.l	(a0)+,d3			; Get next source 32bit word
	and.l		d0, d3				; Mask off unwanted bits of source
	or.l			d3, d2				; assemble masked pieces
	move.l	d2, (a2)+			; Write finished pixels to destination and increment
	
@End_Of_Line:
	
	move.l	a3, a0				; Get a fresh copy of Row Raw Data pointer (could be optimized out)
	move.l	d5, a1				; Get a fresh copy of Skip List pointer (could be optimized out)
	move.l	(a4)+, d0			; Get Address of Skip List Entry, absolute offset, cannot be "0" unless finished
	
; The first entry will not be accidentally "0" because there are header fields at structure top
	bne		@Repeat_Start	; If 0, exit routine, there are no entries left
	
@End_Of_Routine:
	movem.l (sp)+, d0-d6/a0-a4 	; Restore all alterred registers

	} /* asm */
	
} /* Copy_Bits_From_Compressed_Pict_With_Region */
/*=**************************************************************************=*/


/*=**************************************************************************=*/
/* END OF FILE: Fast_Bit_Copy.c */
