/* START OF FILE: Fast_Image_Compaction.c */


/*=**************************************************************************=*/
/* [---BEGIN---] FILE HEADER COMMENTS */
/**
 ** SCREEN VIEW = [Adobe Syntax Black-10, Tabs = 4]
 ** PRINTER VIEW = [Adobe Syntax Black-10, Tabs = 4]
 ** =======================================================================
 ** Written by James Merkle
 ** Fast_Image_Compaction.c - Fast ability to draw a from a run length span compresed source using region 
 ** (C) Copyright 1993 by James Merkle
 ** All rights reserved, except those wishing to "copy" moderate sections of code or files may do so, so long as
 ** such code is never used directly either in animation-related tools or multimedia tools that are  
 ** used in live performance or live broadcast or recorded mass market broadcast or film. Desktop usage not restricted.
 ** Conventional desktop game or entertainment not restricted.
 ** =======================================================================
 **
 **
 ** ============
 ** Revision History
 ** ============
 **
 **  1993.03.16		(James Merkle)
 **						Initial coding started
 **
 **
 **
 **
 ** =================
 ** OTHER INFORMATION
 ** =================
 **  Non-overlapping memory required, same depth required, differennt offsets and rowbytes OK, region OK
 **
 **/
/* [----END----] FILE HEADER COMMENTS */
/*=**************************************************************************=*/


/*=**************************************************************************=*/
/* [---BEGIN---] #include AREA */

#include <QDOffscreen.h>

#include "Fast_Bit_Copy.h"
#include "Graphics_Primitives.h"
#include "Shell_Assembly.h"
#include "Error_Utilities.h"
#include "Memory_Utilities.h"
/* ==
#include "Dreaming_Stuff.h"
=== */
#include "Interface_Utilities.h"
#include "Fast_Image_Compaction.h"


/* [----END----] #include AREA */
/*=**************************************************************************=*/


/*=**************************************************************************=*/
/* [---BEGIN---] #define AREA */

/*  */
/* The following information are the vital object code offsets */
/* into the pixel expansion code. It is important to be 100% */
/* correct. The total object size is about 3K for Motorola 68K version */
/* To PowerPC offsets use these values but are alllowed to */
/* shift or multiply their values (but all must be uniformly increased! */
/*  */
#define k_EXPAND_TOKEN_GROUP_A_OFFSET			0x0000
#define k_EXPAND_TOKEN_GROUP_A_TOKEN_SPACE	0x0018

#define k_EXPAND_TOKEN_GROUP_B_OFFSET			0x0180
#define k_EXPAND_TOKEN_GROUP_B_TOKEN_SPACE	0x0020

#define k_EXPAND_TOKEN_GROUP_C_OFFSET			0x0380
#define k_EXPAND_TOKEN_GROUP_C_TOKEN_SPACE	0x0028

#define k_EXPAND_TOKEN_GROUP_D_OFFSET			0x0600
#define k_EXPAND_TOKEN_GROUP_D_TOKEN_SPACE	0x0028

#define k_EXPAND_TOKEN_GROUP_E_OFFSET				0x0880
#define k_EXPAND_TOKEN_GROUP_E_TOKEN_SPACE	0x0030

/*  */
/* MINIMUM is needed because there is a start, stop word, */
/* 32 words center and possibly a need for an end loop */
/* and a start loop. But because initial alignment may need */
/* three longs not just one, nut one can be eliminated, it leaves 37. */
/*  */
#define k_MINIMUM_WORDS_FOR_GROUP_D_SPAN		37

/*  */
/* Mask logic encodes using opposite of traditional Mac pixel values */
/* so that a "1" means a mask bit hit (black) not a miss (white) */
/*  */
#define	k_INVERTED_WHITE_NIBBLE 	(0)
#define	k_INVERTED_BLACK_NIBBLE 	(0x0F)

/*  */
/* Only used for padding source structure to align to 16 byte addresses */
/*  */
#define k_ARBITRARY_FILL_BYTES 0x99999999

/*  */
/* These are offsets into table of 64 different span encoders */
/* that are then reconverted to object code jump offsets */
/* that are then again at runtime shifted left for more dramatic */
/* offset on PowerPC (larger opcodes too big proportionately). */
/*  */
#define	k_GROUP_B_TOKEN_OFFSET 	(16)
#define	k_GROUP_C_TOKEN_OFFSET 	(32)
#define	k_GROUP_D_TOKEN_OFFSET 	(48)
#define	k_GROUP_E_TOKEN_OFFSET 	(64)

#define	k_ALL_4_BYTES_TOKEN 		(15)

#define COLOR_PORT_PORTBITS_CHECK 		0xC000

#define k_USE_FAST_TOKEN		TRUE
#define k_NOT_FAST_TOKEN		FALSE

/* [----END----] #define AREA */
/*=**************************************************************************=*/


/*=**************************************************************************=*/
/* [---BEGIN---] TYPE AREA */


/* [----END----] TYPE AREA */
/*=**************************************************************************=*/


/*=**************************************************************************=*/
/* [---BEGIN---] PROTOTYPE AREA */

Safe_Boolean Convert_Token_Value_To_Offset(CARDINAL_32	token, CARDINAL_32 *offset, CARDINAL_8 *token_Class);

pascal void Get_Four_1_Bit_Pixels(INTEGER_32 horizontal_Coordinate
											, INTEGER_32 vertical_Coordinate
											, CARDINAL_32 *fetched_Color);

Safe_Boolean Record_128_Bit_Aligned_Source_And_Dest_Token(CARDINAL_32 expected_Token_Value
																											,  CARDINAL_32 words_To_Loop_Copy
																											, CARDINAL_32	address_Of_Skip_List_Data
																											, CARDINAL_32	horizontal_Skip_Bytes);

/* ======
was declared with missing parameter but compilation alloed in THINK C 7.04!!
Safe_Boolean Close_A_Normal_Run_Of_Tokens(Boolean terminator_Is_Excluded
													, CARDINAL_32 total_32_Bit_Words
													, CARDINAL_32 *expected_Token_Value
													, CARDINAL_32 *expected_Token_Start_Position
													, INTEGER_32 vertical_Coordinate
													, INTEGER_32 horizontal_Coordinate
													, CARDINAL_32 fetched_Color
													, Boolean *searching_Black);
====== */
													
Safe_Boolean Close_A_Normal_Run_Of_Tokens(Boolean terminator_Is_Excluded
											, CARDINAL_32 total_32_Bit_Words
											, CARDINAL_32 *expected_Token_Value
											, CARDINAL_32 *expected_Token_Start_Position
											, INTEGER_32 vertical_Coordinate
											, INTEGER_32 horizontal_Coordinate
											, CARDINAL_32 fetched_Color
											, Boolean *searching_Black
											, Boolean *previous_Token_Is_Fast_Token);

											
Safe_Boolean Chop_Off_Leading_Token_For_Fast_Token_Compact_Region_Support(CARDINAL_32 *expected_Token_Value
																		, CARDINAL_32 *expected_Token_Start_Position
																		, INTEGER_32 vertical_Coordinate
																		, INTEGER_32 horizontal_Coordinate
																		, CARDINAL_32 total_32_Bit_Words
																		, CARDINAL_32 fetched_Color
																		, Boolean *searching_Black
																		, Boolean *previous_Token_Is_Fast_Token);
																					
Safe_Boolean Chop_Off_Leading_Non_Optimizable_Token_For_Compact_Region_Support(CARDINAL_32 *total_32_Bit_Words
																		, CARDINAL_32 *expected_Token_Value
																		, CARDINAL_32 *expected_Token_Start_Position
																		,  INTEGER_32 vertical_Coordinate
																		, INTEGER_32 horizontal_Coordinate
																		, CARDINAL_32 fetched_Color
																		, Boolean *searching_Black
																		, Boolean *previous_Token_Is_Fast_Token);

Safe_Boolean Back_Up_to_Previous_Line_And_Close_Pixel_Span(INTEGER_32 vertical_Coordinate
															, INTEGER_32 horizontal_Coordinate
															, CARDINAL_32 *expected_Token_Value
															, CARDINAL_32 *expected_Token_Start_Position
															, Boolean *previous_Token_Is_Fast_Token
															, Boolean *searching_Black
															, CARDINAL_16 max_Columns);
															
Safe_Boolean Record_Token(CARDINAL_32 expected_Token_Value
						, CARDINAL_32 horizontal_Coordinate
						, CARDINAL_32 vertical_Coordinate
						, CARDINAL_32 words_To_Loop_Copy
						, Boolean *previous_Token_Is_Fast_Token
						, Boolean this_Token_Is_Fast_Token);

Safe_Boolean Destroy_Any_Temp_Global_Compression_Handles(void);

Safe_Boolean Nullify_Temp_Global_Compression_Handles(void);

Safe_Boolean Prepare_Source_Pixel_Globals(const BitMap *source_Port);

Safe_Boolean Create_Temp_Global_Compression_Handles(Rect *offscreen_Size_Rect
																	, CARDINAL_32 source_Bit_Depth);

Safe_Boolean Copy_Memory_And_Resize(Handle	source_Handle
														, CARDINAL_32		size_Needed
														, Handle		*destination_Handle);

/* [----END----] PROTOTYPE AREA */
/*=**************************************************************************=*/


/*=**************************************************************************=*/
/* [---BEGIN---] GLOBAL DEFINITION AREA */


/* [----END----] GLOBAL DEFINITION AREA */
/*=**************************************************************************=*/


/*=**************************************************************************=*/
/* [---BEGIN---] LOCAL STATIC DEFINITION AREA */

static CARDINAL_8	*unit_Name = "\pFast_Image_Compaction";

	Handle					g_Temp_Row_Raw_Data_Hdl;
	Handle					g_Temp_Skip_List_Data_Hdl;
	Handle					g_Temp_Skip_List_Master_List_Hdl;
	Handle					g_Temp_Scan_Line_Bounds_Hdl;
	
	CARDINAL_32			g_Temp_Row_Raw_Data_Active_Size;
	CARDINAL_32			g_Temp_Skip_List_Data_Active_Size;
	CARDINAL_32			g_Temp_Skip_List_Master_List_Active_Size;
	CARDINAL_32			g_Temp_Scan_Line_Bounds_Active_Size;
	
	CARDINAL_32			g_Temp_Row_Raw_Data_Maximum_Size;
	CARDINAL_32			g_Temp_Skip_List_Data_Maximum_Size;
	CARDINAL_32			g_Temp_Skip_List_Master_List_Maximum_Size;
	CARDINAL_32			g_Temp_Scan_Line_Bounds_Maximum_Size;
	
	CARDINAL_32			g_Previous_Vertical_Coordinate_With_Data;

	Memory_Allocation		g_Temp_Row_Raw_Data_Hdl_Memory_In_Use;
	Memory_Allocation		g_Temp_Skip_List_Data_Hdl_Memory_In_Use;
	Memory_Allocation		g_Temp_Skip_List_Master_List_Hdl_Memory_In_Use;
	Memory_Allocation		g_Temp_Scan_Line_Bounds_Hdl_Memory_In_Use;

	CARDINAL_32			g_Active_Source_Byte_Address;
	CARDINAL_32			g_Active_Source_Row_Byte_Offset;
	CARDINAL_16			g_Active_Source_Bit_Depth;
	
	CARDINAL_32			g_Debug_Previous_Fast_Vertical_Offset;

/* [----END----] LOCAL STATIC DEFINITION AREA */
/*=**************************************************************************=*/


/*=**************************************************************************=*/
pascal void Get_Four_1_Bit_Pixels(INTEGER_32 horizontal_Coordinate
											, INTEGER_32 vertical_Coordinate
											, CARDINAL_32 *fetched_Color)
{
	/*  */
	/* This routine is meant to be indexed as 0 or 4 bit offsets */
	/* NOTHING ELSE. It is meant for high speed nibble extraction. */
	/* This routine returns the INVERTED index color (1,0) at the specified address. */
	/* Restrictions : If onscreen, the entire graphic region must be  */
	/* visible and unobscured, mouse must be temporarily */
	/* hidden, entire region must be upon a single device  */
	/* and utilize one screen depth. */
	/* If the pixel size is not a multiple of 8 bits, problems  */
	/* will occur if the window is not aligned so that pixel 0*/
	/* starts in a new byte address.  */
	/* To facilitate meeting these requirements utilize the */
	/* routines Begin_Direct_Video_Access and */
	/* Stop_Direct_Video_Access  */
	/* WARNING: needs 68020 addressing for speed. */
	/* If it is an offscreen grafport then all the above */
	/* restrictions can be ignored */
	/*  */
	
/* END VAR */
	asm 68020 {		
		move.l		horizontal_Coordinate(a6),d1					; Fetch horizontal_Coordinate into a register for use later
		move.l		vertical_Coordinate(a6),d0						; Fetch vertical_Coordinate

		movea.l		primitives_Vertical_Lookup_Table_Direct(a5),a0		; Get vertical table address
		move.l		(a0,d0*4),a1												; Get vertical table indexed data (address of screen 0)
		movea.l		primitives_Horizontal_Lookup_Table_Direct(a5),a0	; Get horizontal table address
		add.l			(a0,d1*4),a1												; Add horizontal table indexed data for final address

		clr.l				d0																; CLEAR RESULT : This is needed only because destination is not 8 bit
		move.b		(a1),d0
		move.l		fetched_Color(a6),a0
		
		andi.l			#4,d1														; Check if pixel horizontal offset is using upper 4 pixels of byte
		beq.s			@Low_Nibble_Needed
	
		andi.l			#15,d0													; Mask out all but lowest nibble
		move.l		d0, (a0)
		
		unlk			a6																; Remove stack frame
		movea.l		(sp)+,a0													; Load invocation address
		add.l			#12,sp													; Restore proper stack size ("12" was size of passed parameters)
		jmp			(a0)																; Jump back to caller. All done

@Low_Nibble_Needed
		lsr.b			#4,d0															; Shift to get high nibble. Could have been whole register
		move.l		d0, (a0)

		unlk			a6															; Remove stack frame
		movea.l		(sp)+,a0													; Load invocation address
		add.l			#12,sp														; Restore proper stack size ("8" was size of passed parameters)
		jmp			(a0)														; Jump back to caller. All done
		
	}		

} /* Get_Four_1_Bit_Pixels */
/*=**************************************************************************=*/


/*=**************************************************************************=*/
pascal void Get_Single_8_Bit_Byte_HACK_TEST(INTEGER_32 horizontal_Coordinate
																, INTEGER_32 vertical_Coordinate
																, CARDINAL_32 *fetched_Color);
pascal void Get_Single_8_Bit_Byte_HACK_TEST(INTEGER_32 horizontal_Coordinate
																, INTEGER_32 vertical_Coordinate
																, CARDINAL_32 *fetched_Color)
{
	/*  */
	/* Debugging routine. */
	/* This routine returns the byte containing the specified address. */
	/*  */
	
/* END VAR */
	asm 68020 {		
		move.l		horizontal_Coordinate(a6),d1					; Fetch horizontal_Coordinate into a register for use later
		move.l		vertical_Coordinate(a6),d0						; Fetch vertical_Coordinate

		movea.l		primitives_Vertical_Lookup_Table_Direct(a5),a0		; Get vertical table address
		move.l		(a0,d0*4),a1												; Get vertical table indexed data (address of screen 0)
		movea.l		primitives_Horizontal_Lookup_Table_Direct(a5),a0	; Get horizontal table address
		add.l			(a0,d1*4),a1												; Add horizontal table indexed data for final address
	
		move.l		fetched_Color(a6),a0
		clr.l				(a0)
		move.b		(a1),(a0)
		
		unlk			a6															; Remove stack frame
		movea.l		(sp)+,a0													; Load invocation address
		add.l			#12,sp													; Restore proper stack size ("12" was size of passed parameters)
		jmp			(a0)														; Jump back to caller. All done

	}		

} /* Get_Single_1_Bit_Pixel_HACK_TEST */
/*=**************************************************************************=*/


/*=**************************************************************************=*/
Safe_Boolean Prepare_Source_Pixel_Globals(const BitMap *source_Port)
{
	PixMapPtr		source_Pix_Map_Ptr;
	Ptr					source_Byte_Address;
	Safe_Boolean		function_Result;
/* END VAR */


	g_Active_Source_Byte_Address = 0xFFFFFFFF;
		
	if ((((PixMapPtr)source_Port)->rowBytes & COLOR_PORT_PORTBITS_CHECK) 
				!= 0) {
		/*  */
		/* We have proven that the destination is either a sneaky handle */
		/* being passed in the form of a fake BitMap, and is really */
		/* a pointer to a PixMap Handle, or it might also be a */
		/* valid pointer to a PixMap. To distinguish we check if any */
		/* low bits are set */
		/*  */
		
		if ((((PixMapPtr)source_Port)->rowBytes & (~COLOR_PORT_PORTBITS_CHECK)) != 0) {
			/*  */
			/* Probably is a real pointer to a PixMap because lowest 14 */
			/* bits of "rowBytes" are non-zero */ 
			/*  */
			source_Pix_Map_Ptr = (PixMapPtr)source_Port;
		}
		else {
			/*  */
			/* At this point for safety we could check to see if */
			/* (PixMapPtr)source_Port)->rowBytes has BOTH high bits set or not */
			/*  */
			source_Pix_Map_Ptr = **((PixMapHandle *)source_Port);
		}
		
	}
	else {
		/*  */
		/* Yucch! its an old Bit Map pointer for black and white */
		/*  */
		goto Error_Handler;
	}		
	
			
	/*  */
	/* DANGER : Passing a "Fake Handle" out of desperation (Apple sucks)*/
	/*  */
	source_Byte_Address = nil;
	source_Byte_Address = GetPixBaseAddr(&source_Pix_Map_Ptr);
	if (source_Byte_Address == nil) {
		goto Error_Handler;
	}
	
	g_Active_Source_Byte_Address = (CARDINAL_32)source_Byte_Address;
	g_Active_Source_Row_Byte_Offset = (source_Pix_Map_Ptr->rowBytes & (~COLOR_PORT_PORTBITS_CHECK));
	g_Active_Source_Bit_Depth = (*source_Pix_Map_Ptr).pixelSize;


	function_Result = SAFE_TRUE;
	goto Normal_Function_Exit;
	
Error_Handler:
	function_Result = SAFE_FALSE;

Normal_Function_Exit:
	return(function_Result);

} /* Safe_Boolean Prepare_Source_Pixel_Globals... */
/*=**************************************************************************=*/


/*=**************************************************************************=*/
Safe_Boolean Record_128_Bit_Aligned_Source_And_Dest_Token(CARDINAL_32 expected_Token_Value
																											,  CARDINAL_32 words_To_Loop_Copy
																											, CARDINAL_32	address_Of_Skip_List_Data
																											, CARDINAL_32	horizontal_Skip_Bytes)
/*  */
/* This records a single token representing a span of more */
/* than 128 bytes for certain. If long enough, (and it better be) */
/* a few tokens are consumed at the start to bring equilibrium */
/* to allow principle span to take place using Motorola Move16s. */
/* To allow this source and dest must be aligned to cache line addresses. */
/* It is easy to pad storage space (if needed) so that the pixels being */
/* copied in the fast span middle portion are from address start */
/* divisible by 16. It is also possible to draw a few longs (1 to 3 of them) */
/* so that the destination pixels at the start of a span that are not */
/* divisible by 16 can be preset. This is a strange algorithm and a little */
/* complex. But it is critical to allow tokens to represent 16 byte aligned */
/* information */
/* NOT DONE YET */
/*  */
{
	CARDINAL_32	destination_Byte_Address;
	CARDINAL_32	large_128_Byte_Loops;
	CARDINAL_32	extra_Leading_Source_Bytes_Drawn;
	CARDINAL_32	loops_Needed_For_Source_Alignment;
	CARDINAL_32	leading_Source_Address_Offset;
	CARDINAL_32	storage_Address_Alignment;
	CARDINAL_32	new_Offset_To_Use;
	Safe_Boolean		function_Result;
/* END VAR */


	function_Result = SAFE_FALSE;
	
	if ((expected_Token_Value & 0x03) != 0) {
		/*  */
		/* Four of the line spans do not have starting pixel leftovers */
		/* The rest need adjustment. Logic appears inverted but it works. */
		/*  */
		words_To_Loop_Copy = words_To_Loop_Copy - 1UL;
		
		/*  */
		/* One long will exist before start, so consider it (4ul). */
		/* Need to align "move16" source line over to next longword */
		/* to keep it at leat 64 bit aligned (chip wants to be 128 bit aligned) */
		/* This code guarantees 128 bit alignment though */
		/*  */
		extra_Leading_Source_Bytes_Drawn = 4;
	} /* if ((expected_Token_Value & 0x03) != 0) */
	else {
		/*  */
		/* Simple start example. */
		/* Need to align "move16" source line over to next longword */
		/* to keep it at leat 64 bit aligned (chip wants to be 128 bit aligned) */
		/* This code guarantees 128 bit alignment though */
		/*  */
		extra_Leading_Source_Bytes_Drawn = 0;
	} /* if ((expected_Token_Value & 0x03) != 0) */
	
	
	/*  */
	/* This is the section where calculations are made to determine */
	/* how source information is aligned */
	/*  */
	leading_Source_Address_Offset = (horizontal_Skip_Bytes & 0x000F);
	
	/*  */
	/* leading_Source_Address_Offset is either 0, 4, 8, or 12 bytes misaligned. */
	/* Aligned both structures to each other (Source + Dest) */
	/*  */	
	
	switch (leading_Source_Address_Offset) {
	
		case 4:
			loops_Needed_For_Source_Alignment = 3;
			if (extra_Leading_Source_Bytes_Drawn > 0) {
				loops_Needed_For_Source_Alignment = 2;
			}
			extra_Leading_Source_Bytes_Drawn += (loops_Needed_For_Source_Alignment * 4UL);
		break;
		
		case 8:
			loops_Needed_For_Source_Alignment = 2;
			if (extra_Leading_Source_Bytes_Drawn > 0) {
				loops_Needed_For_Source_Alignment = 1;
			}
			extra_Leading_Source_Bytes_Drawn += (loops_Needed_For_Source_Alignment * 4UL);
		break;
		
		case 12:
			loops_Needed_For_Source_Alignment = 1;
			if (extra_Leading_Source_Bytes_Drawn > 0) {
				loops_Needed_For_Source_Alignment = 0;
			}
			else {
				extra_Leading_Source_Bytes_Drawn += (loops_Needed_For_Source_Alignment * 4UL);
			}
		break;
		
		case 0:
			loops_Needed_For_Source_Alignment = 0;	
			if (extra_Leading_Source_Bytes_Drawn > 0) {
				/*  */
				/* Oh no! The destination starts on 128 bit but we are */
				/* about to make it unaligned to accommodate lead 32 bit word. */
				/* This means detination needs three padding (burned up) longs */
				/*  */
				loops_Needed_For_Source_Alignment = 3;
				extra_Leading_Source_Bytes_Drawn += (loops_Needed_For_Source_Alignment * 4UL);
			}
		break;
		
		default:
			goto Error_Handler;
			
	} /* switch (leading_Source_Address_Offset) */
	
	
	/*  */
	/* This is where padding is added to storage of pixels to */
	/* allow source 128 byte loop to be 128 bit aligned for */
	/* cache-line copys (Move16) */
	/* extra_Leading_Source_Bytes_Drawn is normally 0, 4, 8, or 12 at this point. */
	/* Note that "horizontal_Skip_Bytes" has not been examined yet. */
	/*  */
	storage_Address_Alignment = ((g_Temp_Row_Raw_Data_Active_Size + extra_Leading_Source_Bytes_Drawn) & 0x000F);

	if (storage_Address_Alignment != 0) {
	
		/*  */
		/* Gottta contend with main loop not aligned without some */
		/* storage padding */
		/*  */
	
		destination_Byte_Address = ((CARDINAL_32)*g_Temp_Row_Raw_Data_Hdl + g_Temp_Row_Raw_Data_Active_Size);
	
		switch(storage_Address_Alignment) {
			/*  */
			/* Main big loop loop will now start on 128 bit alignment guaranteed */
			/*  */
			case 4:
				*(CARDINAL_32 *)destination_Byte_Address = k_ARBITRARY_FILL_BYTES;
				*(CARDINAL_32 *)(destination_Byte_Address + 4ul) = k_ARBITRARY_FILL_BYTES;
				*(CARDINAL_32 *)(destination_Byte_Address + 8ul) = k_ARBITRARY_FILL_BYTES;
				g_Temp_Row_Raw_Data_Active_Size += 12;
			break;
			
			case 8:
				*(CARDINAL_32 *)destination_Byte_Address = k_ARBITRARY_FILL_BYTES;
				*(CARDINAL_32 *)(destination_Byte_Address + 4ul) = k_ARBITRARY_FILL_BYTES;
				g_Temp_Row_Raw_Data_Active_Size += 8;
			break;
			
			case 12:
				*(CARDINAL_32 *)destination_Byte_Address = k_ARBITRARY_FILL_BYTES;
				g_Temp_Row_Raw_Data_Active_Size += 4;
			break;
			
			default:
				goto Error_Handler;
		}
					
	} /* if ((storage_Address_Alignment != 0) */
	
	if ((expected_Token_Value - k_GROUP_D_TOKEN_OFFSET) >= 4) {
		/*  */
		/* The first four of the line spans do not have ending pixel leftovers */
		/* Logic appears inverted but it works */
		/*  */
		words_To_Loop_Copy = words_To_Loop_Copy - 1UL;
	}
		
	*(CARDINAL_32 *)(address_Of_Skip_List_Data + 4UL) = horizontal_Skip_Bytes;
	
	if (loops_Needed_For_Source_Alignment != 0) {
		/*  */
		/* Promote token to type "E" if token needs preamble before hitting */
		/* major 128 byte loop to align on cache-line boundary */
		/* This is a hack-ish way to do it. but it works. */
		/* decompose address and reconstruct it. */
		/*  */
		new_Offset_To_Use = *(CARDINAL_32 *)address_Of_Skip_List_Data;
		new_Offset_To_Use = new_Offset_To_Use - k_EXPAND_TOKEN_GROUP_D_OFFSET;
		new_Offset_To_Use = new_Offset_To_Use / k_EXPAND_TOKEN_GROUP_D_TOKEN_SPACE;
		new_Offset_To_Use = k_EXPAND_TOKEN_GROUP_E_OFFSET + (k_EXPAND_TOKEN_GROUP_E_TOKEN_SPACE * new_Offset_To_Use);
		*(CARDINAL_32 *)address_Of_Skip_List_Data = new_Offset_To_Use;
	
		words_To_Loop_Copy = words_To_Loop_Copy - loops_Needed_For_Source_Alignment;
		
		*(CARDINAL_32 *)(address_Of_Skip_List_Data + 8UL) = (loops_Needed_For_Source_Alignment - 1);
		
		/*  */
		/* BASE 0 is needed and active */
		/*  */				
		large_128_Byte_Loops = (words_To_Loop_Copy >> 5);
		*(CARDINAL_32 *)(address_Of_Skip_List_Data + 12UL) = large_128_Byte_Loops -1;
		
		/*  */
		/* BASE 0 is needed used (just modulo of 32 longs)*/
		/*  */
		*(CARDINAL_32 *)(address_Of_Skip_List_Data + 16UL) = ((words_To_Loop_Copy & 0x001F));

		g_Temp_Skip_List_Data_Active_Size += 20;
	}
	else {
	
		/*  */
		/* BASE 0 is needed and active */
		/*  */				
		large_128_Byte_Loops = (words_To_Loop_Copy >> 5);
		*(CARDINAL_32 *)(address_Of_Skip_List_Data + 8UL) = large_128_Byte_Loops -1;
		
		/*  */
		/* BASE 0 is needed used (just modulo of 32 longs)*/
		/*  */
		*(CARDINAL_32 *)(address_Of_Skip_List_Data + 12UL) = ((words_To_Loop_Copy & 0x001F));

		g_Temp_Skip_List_Data_Active_Size += 16;
	}
	

	function_Result = SAFE_TRUE;
	goto Normal_Function_Exit;
	
Error_Handler:
	function_Result = SAFE_FALSE;

Normal_Function_Exit:
	return(function_Result);
	
} /* Safe_Boolean Record_128_Bit_Aligned_Source_And_Dest_Token ... */
/*=**************************************************************************=*/


/*=**************************************************************************=*/
Safe_Boolean Record_Token(CARDINAL_32 expected_Token_Value
						, CARDINAL_32 horizontal_Coordinate
						, CARDINAL_32 vertical_Coordinate
						, CARDINAL_32 words_To_Loop_Copy
						, Boolean *previous_Token_Is_Fast_Token
						, Boolean this_Token_Is_Fast_Token)
/*  */
/* This records information into the graphics structures. */
/* If the token is a fast token its value is stored as a deviance */
/* from the normal type of token (or reverse) because it is a */
/* different class that decompresses a little faster but */
/* cannot skip or compress run lengths. */
/* If the previous token was fast then the master table is NOT */
/* updated and only the skip list table is appended to! */
/* This can be done numerous times if needed. */
/* The ability to do this is based on the previous value of */
/* the "previous_Token_Is_Fast_Token" flag, and that flag is reset */
/* each time through this routine. */
/*  */
{
	Safe_Boolean		function_Result;
	CARDINAL_8		*code_Name = "\pRecord_Token";
	CARDINAL_32 	offset;
	CARDINAL_32	address_Of_Tuplet;
	CARDINAL_32	source_Byte_Address;
	CARDINAL_32	row_Raw_Data_Bytes_Copied;
	CARDINAL_32	skip_List_Entry_Offset;
	CARDINAL_32	current_Left_Min_Position;
	CARDINAL_32	current_Right_Max_Position;
	CARDINAL_32	previous_Right_Max_Position;
	CARDINAL_32	address_Of_Skip_List_Data;
	CARDINAL_32	destination_Byte_Address;
	CARDINAL_32	large_128_Byte_Loops;
	CARDINAL_32	horizontal_Skip_Bytes;
	CARDINAL_8 		token_Class;
/* END VAR */



	if (expected_Token_Value > 63) {
		goto Error_Handler;
	}
	
	if (expected_Token_Value < k_GROUP_B_TOKEN_OFFSET) {
		if (this_Token_Is_Fast_Token != TRUE) {
			/*  */
			/* Promote to slower group set for future calls */
			/*  */
			expected_Token_Value += k_GROUP_B_TOKEN_OFFSET;
		}
	}
	else {
		if (this_Token_Is_Fast_Token == TRUE) {
			/*  */
			/* Not supposed to be promoted until inside here! */
			/*  */
			goto Error_Handler;
		}
	}
		
	if (Convert_Token_Value_To_Offset(expected_Token_Value, &offset, &token_Class) != SAFE_TRUE) {
		goto Error_Handler;
	}
	
	row_Raw_Data_Bytes_Copied = 0;
	
	/*  */
	/* Take Offset and encode another tokens worth of data */
	/* using current infornation */
	/*  */

	/*  */
	/* Byte offset for first start of pixel data run */
	/*  */
	source_Byte_Address = g_Active_Source_Byte_Address + (g_Active_Source_Row_Byte_Offset * vertical_Coordinate);
	horizontal_Skip_Bytes = ((horizontal_Coordinate * g_Active_Source_Bit_Depth) >> 3);
	source_Byte_Address += horizontal_Skip_Bytes;
	
	skip_List_Entry_Offset = g_Temp_Skip_List_Data_Active_Size;
	
	/*  */
	/* Start new span tuplet if the previous was not a fast */
	/* token */
	/*  */
	if (*previous_Token_Is_Fast_Token == TRUE) {
		/*  */
		/* Double verify that previous fast is compatible with this vertical offset */
		/*  */
		if (vertical_Coordinate != g_Debug_Previous_Fast_Vertical_Offset) {
			*previous_Token_Is_Fast_Token = FALSE;
		}
	}
	
	if (*previous_Token_Is_Fast_Token != TRUE) {
		/*  */
		/* Count is in triplets of 3 longs but, increment as single items */
		/*  */
		*(CARDINAL_32 *)*g_Temp_Skip_List_Master_List_Hdl += 1;
		
		/*  */
		/* Fill in Tuplet of information */
		/*  */
		address_Of_Tuplet = (CARDINAL_32)*g_Temp_Skip_List_Master_List_Hdl + g_Temp_Skip_List_Master_List_Active_Size;
	
		/*  */
		/* Fill in 1st - Address into skiplist entry */
		/*  */
		*(CARDINAL_32 *)address_Of_Tuplet = skip_List_Entry_Offset;
		
		/*  */
		/* Fill in 3rd - Total Lines to skip, may be 0 */
		/*  */
		if (vertical_Coordinate > g_Previous_Vertical_Coordinate_With_Data) {
			*(CARDINAL_32 *)(address_Of_Tuplet + 8UL) = (vertical_Coordinate - g_Previous_Vertical_Coordinate_With_Data);
		}
		else {
			/*  */
			/* Assume on same line */
			/*  */
			*(CARDINAL_32 *)(address_Of_Tuplet + 8UL) = 0;
		}

	}	
	
	g_Previous_Vertical_Coordinate_With_Data = vertical_Coordinate;
	
	address_Of_Skip_List_Data = g_Temp_Skip_List_Data_Active_Size + ((CARDINAL_32)*g_Temp_Skip_List_Data_Hdl);

	/*  */
	/* Store Compaction method token */
	/* May be reverse modified (promoted) from type "E" later on */
	/*  */
	*(CARDINAL_32 *)address_Of_Skip_List_Data = offset;
	
	/*  */
	/* Encode data into the structures now that type of */
	/* compaction is determined */
	/*  */
	switch (token_Class) {
	
		case 0:
			/*  */
			/* Though fast request was made, if previous is not fast, then we */
			/* MUST enter pict line offset to start "pipeline" pixel pump */
			/*  */
			row_Raw_Data_Bytes_Copied = 4;
			if (*previous_Token_Is_Fast_Token == TRUE) {
				g_Temp_Skip_List_Data_Active_Size += 4;
			}
			else {
				*(CARDINAL_32 *)(address_Of_Skip_List_Data + 4UL) = horizontal_Skip_Bytes;
				g_Temp_Skip_List_Data_Active_Size += 8;
			}
			break;
	
		case 1:
			/*  */
			/* To properly close down pixel pump, if previous */
			/* *previous_Token_Is_Fast_Token then interpretation of this WILL */
			/* not work correctly unless we record WITHOUT horizontal_Skip_Bytes */
			/*  */
			if (expected_Token_Value != 16) {
				row_Raw_Data_Bytes_Copied = 4;
			}
			else {
				/*  */
				/* This special token means that the span is concluded */
				/*  */
				row_Raw_Data_Bytes_Copied = 0;
			}

			if (*previous_Token_Is_Fast_Token == TRUE) {
				g_Temp_Skip_List_Data_Active_Size += 4;
			}
			else {
				*(CARDINAL_32 *)(address_Of_Skip_List_Data + 4UL) = horizontal_Skip_Bytes;
				g_Temp_Skip_List_Data_Active_Size += 8;
			}
			break;
	
		case 2:	
				row_Raw_Data_Bytes_Copied = (4UL * words_To_Loop_Copy);
				row_Raw_Data_Bytes_Copied += 4UL;
				
				if ((expected_Token_Value & 0x03) != 0) {
					/*  */
					/* Four of the line spans do not have starting pixel leftovers */
					/* The rest need adjustment. Logic appears inverted but it works. */
					/*  */
					words_To_Loop_Copy = words_To_Loop_Copy - 1UL;
				}
				
				if ((expected_Token_Value - k_GROUP_C_TOKEN_OFFSET) >= 4) {
					/*  */
					/* The first four of the line spans do not have ending pixel leftovers */
					/* Logic appears inverted but it works */
					/*  */
					words_To_Loop_Copy = words_To_Loop_Copy - 1UL;
				}
				
				*(CARDINAL_32 *)(address_Of_Skip_List_Data + 4UL) = horizontal_Skip_Bytes;
				
				*(CARDINAL_32 *)(address_Of_Skip_List_Data + 8UL) = words_To_Loop_Copy;
				g_Temp_Skip_List_Data_Active_Size += 12;
			break;
	
		case 3:
				row_Raw_Data_Bytes_Copied = (4UL * words_To_Loop_Copy);
				row_Raw_Data_Bytes_Copied += 4UL;
								
				if (Record_128_Bit_Aligned_Source_And_Dest_Token(expected_Token_Value
																											, words_To_Loop_Copy
																											, address_Of_Skip_List_Data
																											, horizontal_Skip_Bytes) != SAFE_TRUE) {
					goto Error_Handler;
				}

		break;
	
		default:
			goto Error_Handler;
			
	} /* switch (token_Class) */
	
	/*  */
	/* Copy over some pixels into the compacted source pixels */
	/*  */
	if (row_Raw_Data_Bytes_Copied > 0) {
	
		destination_Byte_Address = ((CARDINAL_32)*g_Temp_Row_Raw_Data_Hdl + g_Temp_Row_Raw_Data_Active_Size);
		BlockMove((Ptr)source_Byte_Address
							, (Ptr)destination_Byte_Address
							, row_Raw_Data_Bytes_Copied);
							
				
		/*  */
		/* Update scan line bounds data for right side of pixels */
		/*  */
		previous_Right_Max_Position = *(CARDINAL_32 *)((CARDINAL_32)&(((Scan_Bounds_Ptr)(*g_Temp_Scan_Line_Bounds_Hdl))->spans[0]) 
				+ (vertical_Coordinate * 8UL) + 4UL);
				
		if (previous_Right_Max_Position == 0) {
			/*  */
			/* If "0" then previous line information is virgin, therfore blindly stuff */
			/* and do not bother comparing for extreme ranges */
			/*  */

			*(CARDINAL_32 *)((CARDINAL_32)&(((Scan_Bounds_Ptr)(*g_Temp_Scan_Line_Bounds_Hdl))->spans[0]) 
				+ (vertical_Coordinate * 8UL) + 4UL) = (horizontal_Coordinate + ((CARDINAL_32)(row_Raw_Data_Bytes_Copied << 3) / g_Active_Source_Bit_Depth));
				
			*(CARDINAL_32 *)((CARDINAL_32)&(((Scan_Bounds_Ptr)(*g_Temp_Scan_Line_Bounds_Hdl))->spans[0]) 
				+ (vertical_Coordinate * 8UL)) = horizontal_Coordinate;
		}
		else { 
			/*  */
			/* Compare left and right sides for max values and only store max */
			/* values */
			/*  */			
			current_Right_Max_Position = (horizontal_Coordinate + ((CARDINAL_32)(row_Raw_Data_Bytes_Copied << 3) / g_Active_Source_Bit_Depth));
			
			if (previous_Right_Max_Position < current_Right_Max_Position) {
				*(CARDINAL_32 *)((CARDINAL_32)&(((Scan_Bounds_Ptr)(*g_Temp_Scan_Line_Bounds_Hdl))->spans[0]) 
						+ (vertical_Coordinate * 8UL) + 4UL) = current_Right_Max_Position;
			}
			
			current_Left_Min_Position = *(CARDINAL_32 *)((CARDINAL_32)&(((Scan_Bounds_Ptr)(*g_Temp_Scan_Line_Bounds_Hdl))->spans[0]) 
				+ (vertical_Coordinate * 8UL));
			
			if (current_Left_Min_Position > horizontal_Coordinate) {
				*(CARDINAL_32 *)((CARDINAL_32)&(((Scan_Bounds_Ptr)(*g_Temp_Scan_Line_Bounds_Hdl))->spans[0]) 
						+ (vertical_Coordinate * 8UL)) = horizontal_Coordinate;
			}
		
		} /* if (previous_Right_Max_Position == 0) */
	}
	else {
		destination_Byte_Address = 0;
		
	} /* NOT if (row_Raw_Data_Bytes_Copied > 0) */
	
	
	/*  */
	/* Fill in 2nd - Offset into Rowstart raw data */
	/* Use before increment */
	/*  */
	if (*previous_Token_Is_Fast_Token != TRUE) {
		*(CARDINAL_32 *)(address_Of_Tuplet + 4UL) = g_Temp_Row_Raw_Data_Active_Size;
		g_Temp_Skip_List_Master_List_Active_Size += 12UL;
	}
	
	g_Temp_Row_Raw_Data_Active_Size += row_Raw_Data_Bytes_Copied;
	
	*(CARDINAL_32 *)*g_Temp_Skip_List_Data_Hdl = g_Temp_Skip_List_Data_Active_Size;
	
	
	function_Result = SAFE_TRUE;
	goto Normal_Function_Exit;
	
Error_Handler:
	function_Result = SAFE_FALSE;
	Display_Serious_Error_Message(unit_Name
										, code_Name
										, "\pProblem in Record Token, Memory or bad token passed", "\pMust Abort compaction.");


Normal_Function_Exit:

	if (this_Token_Is_Fast_Token == TRUE) {
		*previous_Token_Is_Fast_Token = TRUE;
		g_Debug_Previous_Fast_Vertical_Offset = vertical_Coordinate;
	}
	else {
		*previous_Token_Is_Fast_Token = FALSE;
	}

	return(function_Result);

} /* Safe_Boolean Record_Token ... */
/*=**************************************************************************=*/


/*=**************************************************************************=*/
Safe_Boolean Close_A_Normal_Run_Of_Tokens(Boolean terminator_Is_Excluded
											, CARDINAL_32 total_32_Bit_Words
											, CARDINAL_32 *expected_Token_Value
											, CARDINAL_32 *expected_Token_Start_Position
											, INTEGER_32 vertical_Coordinate
											, INTEGER_32 horizontal_Coordinate
											, CARDINAL_32 fetched_Color
											, Boolean *searching_Black
											, Boolean *previous_Token_Is_Fast_Token)
{
	CARDINAL_32	sub_Token_Termination_Class;
	CARDINAL_8		*code_Name = "\pClose_A_Normal_Run_Of_Tokens";
	Safe_Boolean		function_Result;
/* END VAR */


	/*  */
	/* We could have as few as one token plus a terminator */
	/* or as many as 33 for small or infinite for large */
	/* "total_32_Bit_Words" is modified locally only */
	/*  */
	
	if (terminator_Is_Excluded == TRUE) {
	/*  */
	/* Terminating token may not be suited to end condition. */
	/* span may have to be severely limited */
	/*  */
	/*  */
	/* Now close off data run based on what is left */
	/* since first token is known to be solid 32 it can be 34 or */
	/* greater instead of 35 needed for long span */
	/*  */
	
		switch((CARDINAL_8)*expected_Token_Value) {
			case 0 :
			case 15 :
				*expected_Token_Value = 0;
			break;
			
			case 1 :
				*expected_Token_Value = 1;
			break;
		
			case 3 :
				*expected_Token_Value = 2;
			break;
		
			case 7 :
				*expected_Token_Value = 3;
			break;
		
			default :
				Display_Serious_Error_Message(unit_Name
																	, code_Name
																	, "\pUnexpected problem in picture compression."
																	, "\pError. 'expected_Token_Value' wrong usage?");

				goto Error_Handler;
		}
		
		if (total_32_Bit_Words > k_MINIMUM_WORDS_FOR_GROUP_D_SPAN) {
			if (Record_Token(*expected_Token_Value + k_GROUP_D_TOKEN_OFFSET
					, *expected_Token_Start_Position
					, vertical_Coordinate
					, total_32_Bit_Words
					, previous_Token_Is_Fast_Token
					, k_NOT_FAST_TOKEN) != SAFE_TRUE) {
				goto Error_Handler;
			}
		}
		else {
			if (Record_Token(*expected_Token_Value + k_GROUP_C_TOKEN_OFFSET
					, *expected_Token_Start_Position
					, vertical_Coordinate
					, total_32_Bit_Words
					, previous_Token_Is_Fast_Token
					, k_NOT_FAST_TOKEN) != SAFE_TRUE) {
				goto Error_Handler;
			}
		}
					
		if (fetched_Color != k_INVERTED_WHITE_NIBBLE) {
			/*  */
			/* Good, We found some pixels to use. Start a span */
			/*  */
			*expected_Token_Value = fetched_Color;
			*expected_Token_Start_Position = horizontal_Coordinate;
			*searching_Black = FALSE;
		}
		else {
			/*  */
			/* We exhausted the chain start high speed search again */
			/*  */
			*searching_Black = TRUE;
			*expected_Token_Value = 999; /* paranoia safety only */
		}
	}
	else {
		/*  */
		/* Our terminator is included in this token. This is good for speed */
		/* we just need to find out which type because one is a modifier of */
		/* the repeat span (it is all 32 bit filled) */
		/*  */
		
		switch ((CARDINAL_8)fetched_Color) {
		
			case 0: 
				/*  */
				/* Cool, this is a good one (optimal) it means there is no */
				/* teminal scrap bytes to this data run, gotta adjust lengths though. */
				/*  */
				total_32_Bit_Words = total_32_Bit_Words -1;
				sub_Token_Termination_Class = 0;
			break;
		
			case 8: 
				sub_Token_Termination_Class = 4;
			break;
		
			case 12: 
				sub_Token_Termination_Class = 8;
			break;
		
			case 14: 
				sub_Token_Termination_Class = 12;
			break;


			
			default:
				Display_Serious_Error_Message(unit_Name
													, code_Name
													, "\pCompact_Region_And_Bitmap failed. Bad token", "\pMust Abort.");
				goto Error_Handler;
		
		} /* switch ((CARDINAL_8)fetched_Color)  */

		switch((CARDINAL_8)*expected_Token_Value) {
			case 0 :
			case 15 :
				*expected_Token_Value = 0;
			break;
			
			case 1 :
				*expected_Token_Value = 1;
			break;
		
			case 3 :
				*expected_Token_Value = 2;
			break;
		
			case 7 :
				*expected_Token_Value = 3;
			break;
		
			default :
				Display_Serious_Error_Message(unit_Name
																	, code_Name
																	, "\pUnexpected problem in picture compression."
																	, "\pError. 'expected_Token_Value' wrong usage?");

				goto Error_Handler;
		}

		if (total_32_Bit_Words > k_MINIMUM_WORDS_FOR_GROUP_D_SPAN) {
			if (Record_Token(*expected_Token_Value + k_GROUP_D_TOKEN_OFFSET + sub_Token_Termination_Class
					, *expected_Token_Start_Position
					, vertical_Coordinate
					, total_32_Bit_Words
					, previous_Token_Is_Fast_Token
					, k_NOT_FAST_TOKEN) != SAFE_TRUE) {
				goto Error_Handler;
			}
		}
		else {
			if (Record_Token(*expected_Token_Value + k_GROUP_C_TOKEN_OFFSET + sub_Token_Termination_Class
					, *expected_Token_Start_Position
					, vertical_Coordinate
					, total_32_Bit_Words
					, previous_Token_Is_Fast_Token
					, k_NOT_FAST_TOKEN) != SAFE_TRUE) {
				goto Error_Handler;
			}
		}
	
		/*  */
		/* We exhausted the chain start high speed search again */
		/*  */
		*searching_Black = TRUE;
		*expected_Token_Value = 999; /* paranoia safety only */
		
	} /* NOT if (terminator_Is_Excluded == TRUE)  */


	function_Result = SAFE_TRUE;
	goto Normal_Function_Exit;
	
Error_Handler:
	function_Result = SAFE_FALSE;

Normal_Function_Exit:
	return(function_Result);

} /* void Close_A_Normal_Run_Of_Tokens(Boolean terminator_Is_Excluded... */
/*=**************************************************************************=*/


/*=**************************************************************************=*/
Safe_Boolean Chop_Off_Leading_Token_For_Fast_Token_Compact_Region_Support(CARDINAL_32 *expected_Token_Value
																					, CARDINAL_32 *expected_Token_Start_Position
																					, INTEGER_32 vertical_Coordinate
																					, INTEGER_32 horizontal_Coordinate
																					, CARDINAL_32 total_32_Bit_Words
																					, CARDINAL_32 fetched_Color
																					, Boolean *searching_Black
																					, Boolean *previous_Token_Is_Fast_Token)
{
	Boolean terminator_Is_Excluded;
	Safe_Boolean		function_Result;
/* END VAR */


	/*  */
	/* Goddamn it, we have to either reverse token and alter it */
	/* to non-fast mode, or we can chop off first token of this one */
	/* and use span (if there is a span) to encode as two more */
	/* fast tokens or record first as single slow and second as */
	/* initial marker of 32 bit full word starting marker. */
	/*  */
	
	/*  */
	/* "total_32_Bit_Words" gets modified locally only */
	/*  */
					
	/*  */
	/* If this was a piddly useless data run then encode first and */
	/* as fast token too and second as terminating "Full" token or  */
	/* fast token based on current token to be for next span start. */
	/*  */
	if (total_32_Bit_Words == 2) {
		/*  */
		/* Record token, use as fast token. Next token will be slow.*/
		/* Then start new data run */
		/*  */

		if (Record_Token(*expected_Token_Value
						, *expected_Token_Start_Position
						, vertical_Coordinate
						, 0
						, previous_Token_Is_Fast_Token
						, k_USE_FAST_TOKEN) != SAFE_TRUE) {
			goto Error_Handler;
		}
																
		if (fetched_Color != k_INVERTED_WHITE_NIBBLE) {

			if (Record_Token(k_ALL_4_BYTES_TOKEN
							, *expected_Token_Start_Position + 4
							, vertical_Coordinate
							, 0
							, previous_Token_Is_Fast_Token
							, k_USE_FAST_TOKEN) != SAFE_TRUE) {
				goto Error_Handler;
			}
							
			/*  */
			/* Good, We found some pixels to use. Start a span */
			/*  */
			*expected_Token_Value = fetched_Color;
			*expected_Token_Start_Position = horizontal_Coordinate;
			*searching_Black = FALSE;
		}
		else {
		
			/*  */
			/* Use slow token */
			/*  */
			if (Record_Token(k_ALL_4_BYTES_TOKEN
							, *expected_Token_Start_Position + 4
							, vertical_Coordinate
							, 0
							, previous_Token_Is_Fast_Token
							, k_NOT_FAST_TOKEN) != SAFE_TRUE) {
				goto Error_Handler;
			}
							
			/*  */
			/* We exhausted the chain start high speed search again */
			/*  */
			*searching_Black = TRUE;
			*expected_Token_Value = 999; /* paranoia safety only */
		}
	}
	else {
		/*  */
		/* First, we can assume (total_32_Bit_Words > 2) */
		/* This means after first token is knocked off, we have at least one */
		/* all 32 bit filled and then another scrap token at minimum */
		/* Record initial terminating token do not start use as fast token. */
		/*  */

		if (Record_Token(*expected_Token_Value
						, *expected_Token_Start_Position
						, vertical_Coordinate
						, 0
						, previous_Token_Is_Fast_Token
						, k_NOT_FAST_TOKEN) != SAFE_TRUE) {
			goto Error_Handler;
		}
						
		*expected_Token_Value = k_ALL_4_BYTES_TOKEN;
		*expected_Token_Start_Position += 4;
		
		total_32_Bit_Words = total_32_Bit_Words - 1;
		
		/*  */
		/* NOTE: fetched_Color is not valid unless logic flowed normally from a */
		/* core read. Beware moving code */
		/* Note, terminator will not ever actually be "15" (0 is 15 but previous)*/
		/*  */
		terminator_Is_Excluded = TRUE;
		if ((fetched_Color == 0) 
			|| (fetched_Color == 8) 
			|| (fetched_Color == 12) 
			|| (fetched_Color == 14)) {
			terminator_Is_Excluded = FALSE;
		}
						
		/*  */
		/* We could have as few as one token plus a terminator */
		/* or as many as 33 for small or infinite for large */
		/*  */
		
		if (Close_A_Normal_Run_Of_Tokens(terminator_Is_Excluded
											, total_32_Bit_Words
											, expected_Token_Value
											, expected_Token_Start_Position
											, vertical_Coordinate
											, horizontal_Coordinate
											, fetched_Color
											, searching_Black
											, previous_Token_Is_Fast_Token) != SAFE_TRUE) {
			goto Error_Handler;
										
		}

	} /* NOT if (total_32_Bit_Words == 2) */
	
	function_Result = SAFE_TRUE;
	goto Normal_Function_Exit;
	
Error_Handler:
	function_Result = SAFE_FALSE;

Normal_Function_Exit:
	return(function_Result);
	
} /* void Chop_Off_Leading_Token_For_Fast_Token_Compact_Region_Support ... */
/*=**************************************************************************=*/


/*=**************************************************************************=*/
Safe_Boolean Chop_Off_Leading_Non_Optimizable_Token_For_Compact_Region_Support(CARDINAL_32 *total_32_Bit_Words
																		, CARDINAL_32 *expected_Token_Value
																		, CARDINAL_32 *expected_Token_Start_Position
																		,  INTEGER_32 vertical_Coordinate
																		, INTEGER_32 horizontal_Coordinate
																		, CARDINAL_32 fetched_Color
																		, Boolean *searching_Black
																		, Boolean *previous_Token_Is_Fast_Token)
{
	Safe_Boolean		function_Result;
/* END VAR */


	/*  */
	/* First, kill off initial token, if not suited to data run start token */
	/* then allow logic to continue IF TWO TOKENS IN RUN STILL LEFT */
	/* otherwise kill and handle the token in the section that clears out */
	/* the initial problem and skip over. */
	/*  */
	if (*total_32_Bit_Words > 3) {

/* ===
		*previous_Token_Is_Fast_Token = FALSE;
==== */

		if (Record_Token(*expected_Token_Value
						, *expected_Token_Start_Position
						, vertical_Coordinate
						, 0
						, previous_Token_Is_Fast_Token
						, k_NOT_FAST_TOKEN) != SAFE_TRUE) {
			goto Error_Handler;
		}
						
		*expected_Token_Value = k_ALL_4_BYTES_TOKEN;
		*expected_Token_Start_Position += 4;
		
		*total_32_Bit_Words = *total_32_Bit_Words - 1;
	}
	else {
		/*  */
		/* Only 2 tokens exist in span, plus scrap terminator */
		/* This is more problematic, we need to kill all current tokens. */
		/* "*previous_Token_Is_Fast_Token" previous value unimportant */
		/*  */
		/*  */
		/* Record token, use as fast token. Next token will be fast*/
		/* also if another token shows promise */
		/*  */

		if (Record_Token(*expected_Token_Value
						, *expected_Token_Start_Position
						, vertical_Coordinate
						, 0
						, previous_Token_Is_Fast_Token
						, k_USE_FAST_TOKEN) != SAFE_TRUE) {
			goto Error_Handler;
		}
			
		if (fetched_Color != k_INVERTED_WHITE_NIBBLE) {
		
			if (Record_Token(k_ALL_4_BYTES_TOKEN
							, *expected_Token_Start_Position + 4
							, vertical_Coordinate
							, 0
							, previous_Token_Is_Fast_Token
							, k_USE_FAST_TOKEN) != SAFE_TRUE) {
				goto Error_Handler;
			}
						
			/*  */
			/* Good, We found some pixels to use. Start a span */
			/*  */
			*expected_Token_Value = fetched_Color;
			*expected_Token_Start_Position = horizontal_Coordinate;
			*searching_Black = FALSE;
		}
		else {
			/*  */
			/* Record slow token */
			/*  */
			if (Record_Token(k_ALL_4_BYTES_TOKEN
							, *expected_Token_Start_Position + 4
							, vertical_Coordinate
							, 0
							, previous_Token_Is_Fast_Token
							, k_NOT_FAST_TOKEN) != SAFE_TRUE) {
				goto Error_Handler;
			}


			/*  */
			/* We exhausted the chain start high speed search again */
			/*  */
			*searching_Black = TRUE;
			*expected_Token_Value = 999; /* paranoia safety only */
		}
		
		*total_32_Bit_Words = 0;
		
	} /* NOT if (*total_32_Bit_Words > 3) */
	
	function_Result = SAFE_TRUE;
	goto Normal_Function_Exit;
	
Error_Handler:
	function_Result = SAFE_FALSE;

Normal_Function_Exit:
	return(function_Result);
	
} /* void Chop_Off_Leading_Non_Optimizable_Token_For_Compact_Region_Support... */
/*=**************************************************************************=*/


/*=**************************************************************************=*/
Safe_Boolean Back_Up_to_Previous_Line_And_Close_Pixel_Span(INTEGER_32 vertical_Coordinate
															, INTEGER_32 horizontal_Coordinate
															, CARDINAL_32 *expected_Token_Value
															, CARDINAL_32 *expected_Token_Start_Position
															, Boolean *previous_Token_Is_Fast_Token
															, Boolean *searching_Black
															, CARDINAL_16 max_Columns)
{
	Safe_Boolean		function_Result;
	CARDINAL_32	fetched_Color;
	INTEGER_32		imaginary_Vertical_Marker;
	INTEGER_32		imaginary_Horizontal_Marker;
	CARDINAL_32	total_32_Bit_Words;
	Boolean			initiator_Is_Excluded;
	Boolean			terminator_Is_Excluded;
/* END VAR */


	/*  */
	/* Jump back to previous line (simulate end on previous line)
	/*  */
	imaginary_Vertical_Marker = vertical_Coordinate - 1;
	
	/*  */
	/* Jump back to end of previous line (simulate end+1 of previous line)
	/*  */
	imaginary_Horizontal_Marker = max_Columns;
	
	total_32_Bit_Words = ((imaginary_Horizontal_Marker - *expected_Token_Start_Position) >> 2);

	/*  */
	/* Spoof color to all white (we are backing up to previous line, */
	/* and still need to quit this span but are pretending to be to the right end (off edge). */
	/*  */
	fetched_Color = k_INVERTED_WHITE_NIBBLE;
	
	if (total_32_Bit_Words == 1) {
				
		/*  */
		/* Record token, do not use as fast token. */
		/*  */

		if (Record_Token(*expected_Token_Value
						, *expected_Token_Start_Position
						, imaginary_Vertical_Marker
						, 0
						, previous_Token_Is_Fast_Token
						, k_NOT_FAST_TOKEN) != SAFE_TRUE) {
			goto Error_Handler;
		}
		
		*expected_Token_Value = 999; /* paranoia safety only */
		*searching_Black = TRUE;
	
	}
	else {

		/*  */
		/* More than one 32 bit word in a row to encode. */
		/* depending on initial token, and current end token, and */
		/* whether "*previous_Token_Is_Fast_Token" this can result in many */
		/* different token outputs. Also "*expected_Token_Value" will never be "0" but may be "15" */
		/*  */
		
/* ===
		initiator_Is_Excluded = TRUE;
		if ((*expected_Token_Value == 15) 
			|| (*expected_Token_Value == 8) 
			|| (*expected_Token_Value == 12) 
			|| (*expected_Token_Value == 14)) {
			initiator_Is_Excluded = FALSE;
		}
	==== */
	
		initiator_Is_Excluded = TRUE;
		if ((*expected_Token_Value == 1) 
			|| (*expected_Token_Value == 3) 
			|| (*expected_Token_Value == 7) 
			|| (*expected_Token_Value == 15)) {
			initiator_Is_Excluded = FALSE;
		}

		if (initiator_Is_Excluded == TRUE) {
			/*  */
			/* First, kill off initial token, if not suited to data run start token */
			/* then allow logic to continue IF TWO TOKENS IN RUN STILL LEFT */
			/* otherwise kill and handle the token in the section that clears out */
			/* the initial problem and skip over with zeroing of "total_32_Bit_Words". */
			/*  */
			if (Chop_Off_Leading_Non_Optimizable_Token_For_Compact_Region_Support(&total_32_Bit_Words
																, expected_Token_Value
																, expected_Token_Start_Position
																, imaginary_Vertical_Marker
																, imaginary_Horizontal_Marker
																, fetched_Color
																, searching_Black
																, previous_Token_Is_Fast_Token) != SAFE_TRUE) {
				/*  */
				/* Should not fail */
				/*  */
				goto Error_Handler;
			}

		} /* if (initiator_Is_Excluded == TRUE)  */
		
		/*  */
		/* Total total_32_Bit_Words == 0 if span was already tokenized */
		/* above because too puny to do anything with */
		/*  */
		if (total_32_Bit_Words != 0) {
		
			if (*previous_Token_Is_Fast_Token == TRUE) {
				
				/*  */
				/* Goddamn it, we have to either reverse token and alter it */
				/* to non-fast mode, or we can chop off first token of this one */
				/* and use span (if there is a span) to encode as two more */
				/* fast tokens or record first as single slow and second as */
				/* initial marker of 32 bit full word starting marker. */
				/*  */
								
				/*  */
				/* If this was a piddly useless data run then encode first and */
				/* as fast token too and second as terminating "Full" token or  */
				/* fast token based on current token to be for next span start. */
				/*  */

				if (Chop_Off_Leading_Token_For_Fast_Token_Compact_Region_Support(expected_Token_Value
																	, expected_Token_Start_Position
																	, imaginary_Vertical_Marker
																	, imaginary_Horizontal_Marker
																	, total_32_Bit_Words
																	, fetched_Color
																	, searching_Black
																	, previous_Token_Is_Fast_Token) != SAFE_TRUE) {
					goto Error_Handler;
				}	
				
			} /* if (*previous_Token_Is_Fast_Token == TRUE) */
			else {
				/*  */
				/* previous token has no bearing. Perform normal closure logic */
				/*  */
				/*  */
				/* NOTE: fetched_Color is not valid unless logic flowed normally from a */
				/* corr read. Beware moving code */
				/* Note, terminator will not ever actually be "15" (0 is 15 but previous)*/
				/*  */
				terminator_Is_Excluded = TRUE;
				if ((fetched_Color == 0) 
					|| (fetched_Color == 8) 
					|| (fetched_Color == 12) 
					|| (fetched_Color == 14)) {
					terminator_Is_Excluded = FALSE;
				}
				
				/*  */
				/* We could have as few as one token plus a terminator */
				/* or as many as 33 for small or infinite for large */
				/* Terminating token may not be suited to end condition. */
				/* span may have to be severely limited */
				/*  */
				if (Close_A_Normal_Run_Of_Tokens(terminator_Is_Excluded
														, total_32_Bit_Words
														, expected_Token_Value
														, expected_Token_Start_Position
														, imaginary_Vertical_Marker
														, imaginary_Horizontal_Marker
														, fetched_Color
														, searching_Black
														, previous_Token_Is_Fast_Token) != SAFE_TRUE) {
					goto Error_Handler;
				}
											
			} /* if (*previous_Token_Is_Fast_Token == TRUE) */
		
		} /* if (total_32_Bit_Words != 0) */
		
	} /* if (total_32_Bit_Words <= 1) */
		
	function_Result = SAFE_TRUE;
	goto Normal_Function_Exit;
	
Error_Handler:
	function_Result = SAFE_FALSE;

Normal_Function_Exit:
	return(function_Result);
	
} /* Safe_Boolean Back_Up_to_Previous_Line_And_Close_Pixel_Span */
/*=**************************************************************************=*/


/*=**************************************************************************=*/
Safe_Boolean Nullify_Temp_Global_Compression_Handles(void)
{
	Safe_Boolean		function_Result;
	CARDINAL_32	preferred_Size;
/* END VAR */

	g_Temp_Row_Raw_Data_Hdl = nil;
	g_Temp_Skip_List_Data_Hdl = nil;
	g_Temp_Skip_List_Master_List_Hdl = nil;
	g_Temp_Scan_Line_Bounds_Hdl = nil;
	g_Temp_Row_Raw_Data_Hdl_Memory_In_Use = MEMORY_NOT_INITIALIZED;
	g_Temp_Skip_List_Data_Hdl_Memory_In_Use = MEMORY_NOT_INITIALIZED;
	g_Temp_Skip_List_Master_List_Hdl_Memory_In_Use = MEMORY_NOT_INITIALIZED;
	g_Temp_Scan_Line_Bounds_Hdl_Memory_In_Use = MEMORY_NOT_INITIALIZED;
	
	g_Temp_Row_Raw_Data_Active_Size = 0;
	g_Temp_Skip_List_Data_Active_Size = 0;
	g_Temp_Skip_List_Master_List_Active_Size = 0;
	g_Temp_Scan_Line_Bounds_Active_Size = 0;
	
	function_Result = SAFE_TRUE;
	goto Normal_Function_Exit;
	
Error_Handler:
	function_Result = SAFE_FALSE;

Normal_Function_Exit:
	return(function_Result);
	
} /* Safe_Boolean Create_Temp_Global_Compression_Handles... */
/*=**************************************************************************=*/


/*=**************************************************************************=*/
Safe_Boolean Create_Temp_Global_Compression_Handles(Rect *offscreen_Size_Rect
																	, CARDINAL_32 source_Bit_Depth)
{
	Safe_Boolean		function_Result;
	CARDINAL_32	preferred_Size;
	CARDINAL_32	requested_Size;
	CARDINAL_8		*code_Name = "\pCreate_Temp_Global_Compression_Handles";
/* END VAR */


	g_Temp_Row_Raw_Data_Hdl = nil;
	g_Temp_Skip_List_Data_Hdl = nil;
	g_Temp_Skip_List_Master_List_Hdl = nil;
	g_Temp_Scan_Line_Bounds_Hdl = nil;
	g_Temp_Row_Raw_Data_Hdl_Memory_In_Use = MEMORY_NOT_INITIALIZED;
	g_Temp_Skip_List_Data_Hdl_Memory_In_Use = MEMORY_NOT_INITIALIZED;
	g_Temp_Skip_List_Master_List_Hdl_Memory_In_Use = MEMORY_NOT_INITIALIZED;
	g_Temp_Scan_Line_Bounds_Hdl_Memory_In_Use = MEMORY_NOT_INITIALIZED;
	
	/*  */
	/* Calculate maximum space the raw pixel data could occupy */
	/* (only allows a tiny extra room for worst case) */
	/*  */
	preferred_Size = (offscreen_Size_Rect->right - offscreen_Size_Rect->left) + 4;
	preferred_Size += 1UL;
	preferred_Size = preferred_Size * (CARDINAL_32)(offscreen_Size_Rect->bottom - offscreen_Size_Rect->top + 1UL);
	preferred_Size = preferred_Size * source_Bit_Depth;
	preferred_Size = preferred_Size / 8UL;
	preferred_Size += 4;
	requested_Size = preferred_Size;

	if (Create_Handle(&preferred_Size
						, (CARDINAL_32)(10240L)
						, ALLOW_PURGING
						, DISALLOW_PARTIAL_AMOUNT
						, (Handle *)&g_Temp_Row_Raw_Data_Hdl
						, &g_Temp_Row_Raw_Data_Hdl_Memory_In_Use) != SAFE_TRUE) {
						
		Display_Low_Memory_Error_Message(unit_Name, code_Name, requested_Size);
		
		function_Result = SAFE_FALSE;
		goto Error_Handler;
	}
	g_Temp_Row_Raw_Data_Maximum_Size = requested_Size;

	/*  */
	/* HACK HACK HACK TEMP DEBUG LOGIC */
	/*  */
/* ===
	Fill_Byte_Memory_With_Byte(*g_Temp_Row_Raw_Data_Hdl, requested_Size, 0);
==== */
	
	/*  */
	/* Calculate maximum space the skip list data could occupy */
	/* (only allows a tiny extra room for worste case, each four pixels is one token*/
	/* unless in 16 or 32 bit then it gets worse for worst case */
	/*  */
	preferred_Size = (CARDINAL_32)(offscreen_Size_Rect->bottom - offscreen_Size_Rect->top);
	preferred_Size += 1UL;
	
	/*  */
	/* 1995.04.13 BUG FIX. Was dividing width by 4 incorrectly before. */
	/*  */
	preferred_Size = preferred_Size * (((offscreen_Size_Rect->right - offscreen_Size_Rect->left + 1UL)) + 16UL);
	if (source_Bit_Depth > 31) {
		preferred_Size = preferred_Size * 4UL;
	}
	else {
		if (source_Bit_Depth > 15) {
			preferred_Size = preferred_Size * 2UL;
		}
	}

	preferred_Size += 4;
	requested_Size = preferred_Size;
	
	if (Create_Handle(&preferred_Size
						, (CARDINAL_32)(10240L)
						, ALLOW_PURGING
						, DISALLOW_PARTIAL_AMOUNT
						, (Handle *)&g_Temp_Skip_List_Data_Hdl
						, &g_Temp_Skip_List_Data_Hdl_Memory_In_Use) != SAFE_TRUE) {
						
		Display_Low_Memory_Error_Message(unit_Name, code_Name, requested_Size);
		
		function_Result = SAFE_FALSE;
		goto Error_Handler;
	}
	
	g_Temp_Skip_List_Data_Maximum_Size = requested_Size;

	/*  */
	/* HACK HACK HACK TEMP DEBUG LOGIC */
	/*  */
/* ===
	Fill_Byte_Memory_With_Byte(*g_Temp_Skip_List_Data_Hdl, requested_Size, 0);
=== */
	
	/*  */
	/* Calculate maximum space for g_Temp_Skip_List_Master_List_Hdl */
	/* Three 4 byte components per line entry, plus a 4 byte count at front */
	/*  */
	preferred_Size = (CARDINAL_32)(offscreen_Size_Rect->bottom - offscreen_Size_Rect->top);
	preferred_Size += 1UL;
	preferred_Size = (preferred_Size * 3UL);
	preferred_Size = (preferred_Size * 4UL);
	preferred_Size += 4;
	
	/*  */
	/* HACK !!!! This structure is now sized to allow one span per*/
	/* scan line. Since it is indeterminent ahead of time, merely *
	/* allow 60 transitions per line, each line for now. FIX IT SOON */
	/* The skip list should contain the skips and byte spans not this structure!!! */
	/*  */
	preferred_Size = (preferred_Size * 60UL);
	
	requested_Size = preferred_Size;
	
	if (Create_Handle(&preferred_Size
						, (CARDINAL_32)(10240L)
						, ALLOW_PURGING
						, DISALLOW_PARTIAL_AMOUNT
						, (Handle *)&g_Temp_Skip_List_Master_List_Hdl
						, &g_Temp_Skip_List_Master_List_Hdl_Memory_In_Use) != SAFE_TRUE) {
						
		Display_Low_Memory_Error_Message(unit_Name, code_Name, requested_Size);
		
		function_Result = SAFE_FALSE;
		goto Error_Handler;
	}
	
	g_Temp_Skip_List_Master_List_Maximum_Size = requested_Size;

	/*  */
	/* HACK HACK HACK TEMP DEBUG LOGIC */
	/*  */
/* ===
	Fill_Byte_Memory_With_Byte(*g_Temp_Skip_List_Master_List_Hdl, requested_Size, 0);
=== */
	
	/*  */
	/* Calculate maximum space for g_Temp_Scan_Line_Bounds_Hdl */
	/* Two 4 byte components per line entry, plus 2 8 byte Rects at front */
	/* and a 4 byte line count */
	/* This is not oversized */
	/*  */
	preferred_Size = (CARDINAL_32)(offscreen_Size_Rect->bottom - offscreen_Size_Rect->top);
	preferred_Size += 1UL;
	preferred_Size = (preferred_Size * 8UL);
	preferred_Size += 20;
		
	requested_Size = preferred_Size;
	
	if (Create_Handle(&preferred_Size
						, (CARDINAL_32)(10240L)
						, ALLOW_PURGING
						, DISALLOW_PARTIAL_AMOUNT
						, (Handle *)&g_Temp_Scan_Line_Bounds_Hdl
						, &g_Temp_Scan_Line_Bounds_Hdl_Memory_In_Use) != SAFE_TRUE) {
						
		Display_Low_Memory_Error_Message(unit_Name, code_Name, requested_Size);
		
		function_Result = SAFE_FALSE;
		goto Error_Handler;
	}
	
	g_Temp_Scan_Line_Bounds_Maximum_Size = requested_Size;

	/*  */
	/* Initialize the four structures for imminent filling in */
	/*  */
	
	g_Temp_Row_Raw_Data_Active_Size = 0;
	g_Temp_Skip_List_Data_Active_Size = 0;
	g_Temp_Skip_List_Master_List_Active_Size = 0;
	g_Temp_Scan_Line_Bounds_Active_Size = 0;

	g_Previous_Vertical_Coordinate_With_Data = 0;

	/*  */
	/* Initialize the scan line optimized list of offsets. Having right side */
	/* equal "0" means line is not used */
	/*  */
	Fill_Byte_Memory_With_Byte(*g_Temp_Scan_Line_Bounds_Hdl, requested_Size, 0);
	
	SetRect(&(((Scan_Bounds_Ptr)(*g_Temp_Scan_Line_Bounds_Hdl))->bounds_Rect), 0, 0, 0 ,0);
	if (Move_Memory((Ptr)&offscreen_Size_Rect, (Ptr)&(((Scan_Bounds_Ptr)(*g_Temp_Scan_Line_Bounds_Hdl))->possible_Rect), sizeof(Rect)) != TRUE) {
	}
	((Scan_Bounds_Ptr)(*g_Temp_Scan_Line_Bounds_Hdl))->max_Lines = (offscreen_Size_Rect->bottom - offscreen_Size_Rect->top) + 1UL;
	
	/*  */
	/* Should probably still be equal to "20" */
	/*  */
	g_Temp_Scan_Line_Bounds_Active_Size = ((CARDINAL_32)&(((Scan_Bounds_Ptr)(*g_Temp_Scan_Line_Bounds_Hdl))->spans[0]) 
		- (CARDINAL_32)&(((Scan_Bounds_Ptr)(*g_Temp_Scan_Line_Bounds_Hdl))->bounds_Rect));
		
	g_Temp_Scan_Line_Bounds_Active_Size += (((Scan_Bounds_Ptr)(*g_Temp_Scan_Line_Bounds_Hdl))->max_Lines * 8UL);

	/*  */
	/* Initialize the master list */
	/*  */
	*(CARDINAL_32 *)*g_Temp_Skip_List_Master_List_Hdl = 0;
	g_Temp_Skip_List_Master_List_Active_Size = sizeof(CARDINAL_32);

	/*  */
	/* Initialize the skip list data (structure REQUIRES something at "0" so size is stored there) */
	/*  */
	*(CARDINAL_32 *)*g_Temp_Skip_List_Data_Hdl = 0;
	g_Temp_Skip_List_Data_Active_Size = sizeof(CARDINAL_32);
	


	function_Result = SAFE_TRUE;
	goto Normal_Function_Exit;
	
Error_Handler:
	function_Result = SAFE_FALSE;

Normal_Function_Exit:
	return(function_Result);
	
} /* Safe_Boolean Create_Temp_Global_Compression_Handles... */
/*=**************************************************************************=*/


/*=**************************************************************************=*/
Safe_Boolean Destroy_Any_Temp_Global_Compression_Handles(void)
{
	Safe_Boolean		function_Result;
	CARDINAL_8		*code_Name = "\pDestroy_Any_Temp_Global_Compression_Handles";
/* END VAR */


	if (g_Temp_Row_Raw_Data_Hdl_Memory_In_Use == MEMORY_INITIALIZED)	{
		if (Destroy_Handle((Handle *)&(g_Temp_Row_Raw_Data_Hdl)
											, &(g_Temp_Row_Raw_Data_Hdl_Memory_In_Use)) != SAFE_TRUE) {
			Display_Serious_Error_Message(unit_Name, code_Name, "\pError with DisposeHandle", "\pSystem Corrupted?");
			goto Error_Handler;
		}	
	}
	
	if (g_Temp_Skip_List_Data_Hdl_Memory_In_Use == MEMORY_INITIALIZED)	{
		if (Destroy_Handle((Handle *)&(g_Temp_Skip_List_Data_Hdl)
											, &(g_Temp_Skip_List_Data_Hdl_Memory_In_Use)) != SAFE_TRUE) {
			Display_Serious_Error_Message(unit_Name, code_Name, "\pError with DisposeHandle", "\pSystem Corrupted?");
			goto Error_Handler;
		}	
	}
	
	if (g_Temp_Skip_List_Master_List_Hdl_Memory_In_Use == MEMORY_INITIALIZED)	{
		if (Destroy_Handle((Handle *)&(g_Temp_Skip_List_Master_List_Hdl)
											, &(g_Temp_Skip_List_Master_List_Hdl_Memory_In_Use)) != SAFE_TRUE) {
			Display_Serious_Error_Message(unit_Name, code_Name, "\pError with DisposeHandle", "\pSystem Corrupted?");
			goto Error_Handler;
		}	
	}
	
	if (g_Temp_Scan_Line_Bounds_Hdl_Memory_In_Use == MEMORY_INITIALIZED)	{
		if (Destroy_Handle((Handle *)&(g_Temp_Scan_Line_Bounds_Hdl)
											, &(g_Temp_Scan_Line_Bounds_Hdl_Memory_In_Use)) != SAFE_TRUE) {
			Display_Serious_Error_Message(unit_Name, code_Name, "\pError with DisposeHandle", "\pSystem Corrupted?");
			goto Error_Handler;
		}	
	}
		
	function_Result = SAFE_TRUE;
	goto Normal_Function_Exit;
	
Error_Handler:
	function_Result = SAFE_FALSE;

Normal_Function_Exit:
	return(function_Result);
	
} /* Safe_Boolean Destroy_Any_Temp_Global_Compression_Handles... */
/*=**************************************************************************=*/


/*=**************************************************************************=*/
Safe_Boolean Copy_Memory_And_Resize(Handle	source_Handle
														, CARDINAL_32		size_Needed
														, Handle		*destination_Handle)
{
	Safe_Boolean		function_Result;
	CARDINAL_32	preferred_Size;
	Memory_Allocation	spoof_Hdl_Memory_In_Use;
	CARDINAL_8		*code_Name = "\pCopy_Memory_And_Resize";
/* END VAR */


	function_Result = SAFE_FALSE;


	if (GetHandleSize(source_Handle) < size_Needed) {
		Display_Serious_Error_Message(unit_Name, code_Name, "\pHandle overrun!", "\pBad programming?");
		goto Error_Handler;
	}
	
	preferred_Size = size_Needed;
	spoof_Hdl_Memory_In_Use = MEMORY_NOT_INITIALIZED;
	
	if (Create_Handle(&preferred_Size
						, (CARDINAL_32)(0L)
						, ALLOW_PURGING
						, DISALLOW_PARTIAL_AMOUNT
						, (Handle *)destination_Handle
						, &spoof_Hdl_Memory_In_Use) != SAFE_TRUE) {
						
		Display_Low_Memory_Error_Message(unit_Name, code_Name, size_Needed);
		
		function_Result = SAFE_FALSE;
		goto Error_Handler;
	}
	
	Move_Memory(*source_Handle, (Ptr)**destination_Handle, size_Needed);

	function_Result = SAFE_TRUE;
	goto Normal_Function_Exit;
	
Error_Handler:
	function_Result = SAFE_FALSE;
	Display_Serious_Error_Message(unit_Name, code_Name, "\pError with function", "\pSystem Corrupted or low memory?");

Normal_Function_Exit:
	return(function_Result);
}
/*=**************************************************************************=*/


/*=**************************************************************************=*/
Safe_Boolean Convert_Token_Value_To_Offset(CARDINAL_32	token, CARDINAL_32 *offset, CARDINAL_8 *token_Class)
/*  */
/* This is not table driven because it does not need to be lightning fast */
/* It is only used for image compaction */
/*  */
{
	CARDINAL_32	temp_Value;
	CARDINAL_8		*code_Name;
	Safe_Boolean		function_Result;
/* END VAR */


	/*  */
	/* Set result to end of line for safety (in case does not get set in logic) */
	/*  */
	*offset = k_EXPAND_TOKEN_GROUP_B_OFFSET;
	
	if (token > 63UL) {
		goto Error_Handler;
	}
	
	if (token < 16) {
		*offset = k_EXPAND_TOKEN_GROUP_A_OFFSET + (k_EXPAND_TOKEN_GROUP_A_TOKEN_SPACE * token);
		*token_Class = 0;
	}
	else {
		if (token < 32) {
			token = token - 16;
			*offset = k_EXPAND_TOKEN_GROUP_B_OFFSET + (k_EXPAND_TOKEN_GROUP_B_TOKEN_SPACE * token);
			*token_Class = 1;
		}
		else {
			if (token < 48) {
				token = token - 32;
				*offset = k_EXPAND_TOKEN_GROUP_C_OFFSET + (k_EXPAND_TOKEN_GROUP_C_TOKEN_SPACE * token);
				*token_Class = 2;
			}
			else {
				/*  */
				/* Assumed 63 or less by logic */
				/*  */
				token = token - 48;
				*offset = k_EXPAND_TOKEN_GROUP_D_OFFSET + (k_EXPAND_TOKEN_GROUP_D_TOKEN_SPACE * token);
				*token_Class = 3;
			}
		}
	}
	
	
	function_Result = SAFE_TRUE;
	goto Normal_Function_Exit;
	
Error_Handler:
	function_Result = SAFE_FALSE;
	
	code_Name = "\pConvert_Token_Value_To_Offset";
	Display_Serious_Error_Message("\p", code_Name, "\pUnexpected decompression token", "\pWrong usage?");

Normal_Function_Exit:
	return(function_Result);

}
/*=**************************************************************************=*/


/*=**************************************************************************=*/
Safe_Boolean Compact_Region_And_Bitmap(const BitMap *source_Port
													, Rect	*source_Rect
													, INTEGER_32	offset_Left
													, INTEGER_32	offset_Top
													, Safe_Boolean use_Mask_Input_Instead_Of_Bitmap
													, RgnHandle	clipping_Region
													, BitMapPtr		clipping_Bitmap
													, Handle	*row_Raw_Data_Hdl
													, Handle	*skip_List_Data_Hdl
													, Handle	*skip_List_Master_List_Hdl
													, Handle	*scan_Line_Bounds_Hdl)
{
	Rect								offscreen_Size_Rect;
	Off_Screen_Port_Rec	offscreen_Port;
	Off_Screen_Ptr 			offscreen_Ptr;
	RgnHandle					adjusted_Clipping_Region;
	CARDINAL_16			max_Rows;
	CARDINAL_16			max_Columns;
	INTEGER_32				test_Logic_Bit_Depth;
	GrafPtr						saved_Port;
	INTEGER_16				source_Bit_Depth;
	CARDINAL_32			source_Bit_Width;
	CARDINAL_32			source_Bit_Height;
	INTEGER_32				horizontal_Coordinate;
	INTEGER_32 				vertical_Coordinate;
	CARDINAL_32			fetched_Color;
	CARDINAL_32			temp_Color;
	CARDINAL_32			sub_Token_Termination_Class;
	CARDINAL_32			expected_Token_Value;
	CARDINAL_32			expected_Token_Start_Position;
	CARDINAL_32			total_32_Bit_Words;
	CARDINAL_32			debug_Value_1;
	Rect 							destination_Rect;
	Safe_Boolean				function_Result;
	Safe_Boolean				offscreen_Port_Created;
	Safe_Boolean				offscreen_Port_Selected;
	Memory_Allocation		row_Raw_Data_Hdl_Memory_In_Use;
	Memory_Allocation		skip_List_Data_Hdl_Memory_In_Use;
	Memory_Allocation		skip_List_Master_List_Hdl_Memory_In_Use;
	Memory_Allocation		scan_Line_Bounds_Hdl_Memory_In_Use;
	Boolean					searching_Black;
	Boolean					terminator_Is_Excluded;
	Boolean					start_Quick_Token_Run;
	Boolean					initiator_Is_Excluded;
	Boolean					previous_Token_Is_Fast_Token;
	CARDINAL_8		*code_Name;
/* END VAR */


	code_Name = "\pCompact_Region_And_Bitmap";
	function_Result = SAFE_FALSE;
	offscreen_Port_Created = SAFE_FALSE;
	offscreen_Port_Selected = SAFE_FALSE;
	
	/*  */
	/* These get initialized and filled in only at the function completion */
	/* Their sizes are unknown until then anyway. Other handles */
	/* are used instead while the structures are compacting. */
	/*  */
	*row_Raw_Data_Hdl = nil;
	row_Raw_Data_Hdl_Memory_In_Use = MEMORY_NOT_INITIALIZED;
	*skip_List_Data_Hdl = nil;
	skip_List_Data_Hdl_Memory_In_Use = MEMORY_NOT_INITIALIZED;
	*skip_List_Master_List_Hdl = nil;
	skip_List_Master_List_Hdl_Memory_In_Use = MEMORY_NOT_INITIALIZED;
	*scan_Line_Bounds_Hdl = nil;
	scan_Line_Bounds_Hdl_Memory_In_Use = MEMORY_NOT_INITIALIZED;

	if (source_Port->bounds.top > source_Port->bounds.bottom) {
		Display_Serious_Error_Message(unit_Name
																, code_Name
																, "\pProblem Lwith port to compress. (top > bottom)"
																, "\pCorrupted?");
		goto Error_Handler;
	}

	/*  */
	/* Zero out temp handles for proper early abortts. */
	/*  Cannot create then yet without knowing more */
	/*  */
	if (Nullify_Temp_Global_Compression_Handles() != SAFE_TRUE) {
		goto Error_Handler;
	}
	
	if (Prepare_Source_Pixel_Globals(source_Port) != SAFE_TRUE) {
		goto Error_Handler;
	}
	
	/*  */
	/*  Create initial bounds data based on Region rect */
	/*  */
	
		
	/*  */
	/*  Create Offscreen 8 bit pixel map for drawing opaque region */
	/* Span length tests will be using 8 bit flags, not one bit */
	/* Notice : we use "source_Rect" and *NOT* (**((*(CGrafPort *)source_Port).portPixMap)).bounds */
	/*  */
	source_Bit_Width = source_Rect->right - source_Rect->left;
	source_Bit_Height = source_Rect->bottom - source_Rect->top;
	
	/*  */
	/* SAFETY ENHANCEMENT!!! Adds extra 4 pixel PLUS rounds down to  4 */
	/* Force width to be accomodating if 1,2,or 3 pixels left. */
	/* Increment by 3 and get modulo using "4" */
	/*  */
	source_Bit_Width += 7;
	source_Bit_Width = (source_Bit_Width & 0xFFFFFFFC);

	source_Bit_Depth = g_Active_Source_Bit_Depth;
	test_Logic_Bit_Depth = 1;

	offscreen_Ptr = &offscreen_Port;
	
	SetRect(&offscreen_Size_Rect
				, 0
				, 0
				, source_Bit_Width
				, source_Bit_Height);

	offscreen_Ptr->color_Table_Handle = GetCTable(k_DEFAULT_2_COLOR_SYSTEM_CLUT);
	if (offscreen_Ptr->color_Table_Handle == nil) {
			Display_Serious_Error_Message(unit_Name
																, code_Name
																, "\pProblem Loading System CLUT (2.0)"
																, "\pCorrupted?");
		goto Error_Handler;
	}
	

	if (Create_Off_Screen_Port(&offscreen_Size_Rect
											, test_Logic_Bit_Depth
											, offscreen_Ptr->color_Table_Handle
											, (CGrafPtr)&(offscreen_Ptr->offscreen_Port.color_Port)
											, &(offscreen_Ptr->offscreen_GDevice_Hdl)
											, &(offscreen_Ptr->image_Handle)) != noErr) {
											
		Display_Serious_Error_Message(unit_Name
																, code_Name
																, "\pProblem with Create_Off_Screen_Port"
																, "\pToo little memory?");

		goto Error_Handler;
	}
	
	offscreen_Ptr->initialized =  SAFE_TRUE; 
	offscreen_Ptr->color_Type = k_COLOR_QUICKDRAW;
	Lock_Video_Memory_For_Lazy_But_Safe_Use(offscreen_Ptr);

	offscreen_Port_Created = SAFE_TRUE;
	
	/*  */
	/* Create temp handles to hold structures, some sized correctly, */
	/*  others oversized temporarily */
	/*  */
	if (Create_Temp_Global_Compression_Handles(&offscreen_Size_Rect, source_Bit_Depth) != SAFE_TRUE) {

		Display_Serious_Error_Message(unit_Name
																, code_Name
																, "\pProblem with Create_Temp_Global_Compression_Handles"
																, "\pToo conservative in prediction of complexity, or too little memory.");
		goto Error_Handler;
	}	
	
	/*  */
	/* Port might not be all white. Clear to all white for safety */
	/* before drawing blackened sillouette */
	/*  */
	GetPort(&saved_Port);
	
	Set_View_Port((GrafPtr)&(offscreen_Ptr->offscreen_Port.color_Port));
	offscreen_Port_Selected = SAFE_TRUE;

	/*  */
	/* Only needed really for CopyBits calls but why not be certain. */
	/*  */
	ForeColor(blackColor);
	BackColor(whiteColor);
	
	/*  */
	/* For safety to clear out the memory */
	/*  */
	EraseRect(&offscreen_Size_Rect); 
	
	/*  */
	/* We need to shift mask region because we used "source_Rect" and */
	/* *NOT* (**((*(CGrafPort *)source_Port).portPixMap)).bounds */
	/*  */
	
	
	if (use_Mask_Input_Instead_Of_Bitmap == SAFE_TRUE) {
		/*  */
		/* Offset mask not already done. Offset needed */
		/*  */
		adjusted_Clipping_Region = NewRgn();
		CopyRgn(clipping_Region, adjusted_Clipping_Region);
		
		OffsetRgn(adjusted_Clipping_Region, -(offset_Left), -(offset_Top));
		/*  */
		/* Now draw opaque outline */
		/*  */
		FillRgn(adjusted_Clipping_Region, &(qd.black));
		DisposeRgn(adjusted_Clipping_Region);
	}
	else {
		/*  */
		/* Use Simple bitmap as mask source information */
		/*  */

		destination_Rect = *source_Rect;
		OffsetRect(&destination_Rect, -(offset_Left), -(offset_Top));
	
		CopyBits(clipping_Bitmap
					, (const BitMap *)*(offscreen_Ptr->offscreen_Port.color_Port.portPixMap)	
					, source_Rect
					, &destination_Rect
					, srcOr
					, nil);
	
	} /* NOT if (use_Mask_Input_Instead_Of_Bitmap == SAFE_TRUE) */
	
	
	if (Initialize_Graphics_Primitives(FALSE, TRUE) != TRUE) {
		Display_Serious_Error_Message(unit_Name, code_Name, "\pInitialize_Graphics_Primitives failed.  Program cannot use indirect access.", "\pMust Abort.");
		goto Error_Handler;
	}
	
	max_Rows = offscreen_Size_Rect.bottom;
	max_Columns = offscreen_Size_Rect.right;
	
	searching_Black = TRUE;
	previous_Token_Is_Fast_Token = FALSE;
	for (vertical_Coordinate =0; (vertical_Coordinate < max_Rows); vertical_Coordinate++) {
	
		if (searching_Black != TRUE) {
			/*  */
			/* Will only get here if pixels border right edge of frame */
			/* This pixel Span is finished (end of scan line at least) . Close it out */
			/*  */
			if (Back_Up_to_Previous_Line_And_Close_Pixel_Span(vertical_Coordinate
																	, horizontal_Coordinate
																	, &expected_Token_Value
																	, &expected_Token_Start_Position
																	, &previous_Token_Is_Fast_Token
																	, &searching_Black
																	, max_Columns) != SAFE_TRUE) {
				goto Error_Handler;
			}
																	
			searching_Black = TRUE;
		}
		
		for (horizontal_Coordinate =0; (horizontal_Coordinate < max_Columns); horizontal_Coordinate +=4) {
				
			Get_Four_1_Bit_Pixels(horizontal_Coordinate
											, vertical_Coordinate
											, &fetched_Color);
											
			/*  */
			/*  */
			/*  */
			/*  */
			/* TEMP HACK HACK HACK!!! */
			/*  */
	/* ======
			if ((vertical_Coordinate == 16) && (horizontal_Coordinate == 36) && (fetched_Color == 0)) {
				Debugger();
				Get_Single_8_Bit_Byte_HACK_TEST(horizontal_Coordinate
											, vertical_Coordinate
											, &debug_Value_1);
				Get_Four_1_Bit_Pixels(horizontal_Coordinate
											, vertical_Coordinate
											, &fetched_Color);

			}
	==== */
											
											
											
			if (searching_Black == TRUE) {	
				if (fetched_Color != k_INVERTED_WHITE_NIBBLE) {
					/*  */
					/* Oh Boy! We found some pixels to use. Start a span */
					/*  */
					expected_Token_Value = fetched_Color;
					expected_Token_Start_Position = horizontal_Coordinate;
					searching_Black = FALSE;
				}

			}
			else {
				if (fetched_Color == k_INVERTED_BLACK_NIBBLE) {
					/*  */
					/* Great! Keep the span going! (Do nothing)*/
					/*  */
				}
				else {
				
					/*  */
					/* Damn, we have to stop scanning, but depending on what is */
					/* happening we might use the fast tokens to record this if */
					/* previous token is not fast token type */
					/*  */
					total_32_Bit_Words = ((horizontal_Coordinate - expected_Token_Start_Position) >> 2);
					if (total_32_Bit_Words == 1) {
					
						/*  */
						/* Crap, just one token, instead of decent span, it should be */
						/* validated as a quick token, by default if either this or */
						/* next nibble contains data */
						/* Possible values are post previous token fast or slow, start new token or not.*/
						/*  */
						
						start_Quick_Token_Run = FALSE;
						if (fetched_Color != k_INVERTED_WHITE_NIBBLE) {
							/*  */
							/* Has some pixels to use. Start a fast span */
							/*  */
							start_Quick_Token_Run = TRUE;
						}
						else {
							/*  */
							/* Check if the next one is not out of bounds and is */
							/* containing data */
							/*  */
							if ((horizontal_Coordinate + 4) < max_Columns) {
								Get_Four_1_Bit_Pixels((horizontal_Coordinate + 4)
														, vertical_Coordinate
														, &temp_Color);

								if (temp_Color != k_INVERTED_WHITE_NIBBLE) {
									/*  */
									/* Has some pixels to use. Start a fast span */
									/*  */
									start_Quick_Token_Run = TRUE;
								}

							} /* if (horizontal_Coordinate + 4) < max_Columns)  */

						} /* NOT if (fetched_Color != k_INVERTED_WHITE_NIBBLE) */

						if (start_Quick_Token_Run == TRUE) {
							/*  */
							/* Record token, use as fast token. */
							/*  */

							if (Record_Token(expected_Token_Value
											, expected_Token_Start_Position
											, vertical_Coordinate
											, 0
											, &previous_Token_Is_Fast_Token
											, k_USE_FAST_TOKEN) != SAFE_TRUE) {
								goto Error_Handler;
							}
							
							expected_Token_Value = fetched_Color;
							expected_Token_Start_Position = horizontal_Coordinate;
							searching_Black = FALSE;
							
						}
						else {
							/*  */
							/* Record token, do not use as fast token. */
							/*  */

							if (Record_Token(expected_Token_Value
											, expected_Token_Start_Position
											, vertical_Coordinate
											, 0
											, &previous_Token_Is_Fast_Token
											, k_NOT_FAST_TOKEN) != SAFE_TRUE) {
								goto Error_Handler;
							}

							expected_Token_Value = 999; /* paranoia safety only */
							searching_Black = TRUE;
						}

					} /* if (total_32_Bit_Words == 1) */
					else {
						/*  */
						/* More than one 32 bit word in a row to encode. */
						/* depending on initial token, and current end token, and */
						/* whether "previous_Token_Is_Fast_Token" this can result in many */
						/* different token outputs. Also "expected_Token_Value" will never be "0" but may be "15" */
						/*  */
						
						initiator_Is_Excluded = TRUE;
						if ((expected_Token_Value == 1) 
							|| (expected_Token_Value == 3) 
							|| (expected_Token_Value == 7) 
							|| (expected_Token_Value == 15)) {
							initiator_Is_Excluded = FALSE;
						}
				
						if (initiator_Is_Excluded == TRUE) {
							/*  */
							/* First, kill off initial token, if not suited to data run start token */
							/* then allow logic to continue IF TWO TOKENS IN RUN STILL LEFT */
							/* otherwise kill and handle the token in the section that clears out */
							/* the initial problem and skip over with zeroing of "total_32_Bit_Words". */
							/*  */
							if (Chop_Off_Leading_Non_Optimizable_Token_For_Compact_Region_Support(&total_32_Bit_Words
																				, &expected_Token_Value
																				, &expected_Token_Start_Position
																				, vertical_Coordinate
																				, horizontal_Coordinate
																				, fetched_Color
																				, &searching_Black
																				, &previous_Token_Is_Fast_Token) != SAFE_TRUE) {
								/*  */
								/* Should not fail */
								/*  */
								goto Error_Handler;
							}

						} /* if (initiator_Is_Excluded == TRUE)  */
						
						/*  */
						/* Total total_32_Bit_Words == 0 if span was already tokenized */
						/* above because too puny to do anything with */
						/*  */
						if (total_32_Bit_Words != 0) {
						
							if (previous_Token_Is_Fast_Token == TRUE) {
								
								/*  */
								/* Goddamn it, we have to either reverse token and alter it */
								/* to non-fast mode, or we can chop off first token of this one */
								/* and use span (if there is a span) to encode as two more */
								/* fast tokens or record first as single slow and second as */
								/* initial marker of 32 bit full word starting marker. */
								/*  */
												
								/*  */
								/* If this was a piddly useless data run then encode first and */
								/* as fast token too and second as terminating "Full" token or  */
								/* fast token based on current token to be for next span start. */
								/*  */

								if (Chop_Off_Leading_Token_For_Fast_Token_Compact_Region_Support(&expected_Token_Value
																					, &expected_Token_Start_Position
																					, vertical_Coordinate
																					, horizontal_Coordinate
																					, total_32_Bit_Words
																					, fetched_Color
																					, &searching_Black
																					, &previous_Token_Is_Fast_Token) != SAFE_TRUE) {
									goto Error_Handler;
								}	
								
							} /* if (previous_Token_Is_Fast_Token == TRUE) */
							else {
								/*  */
								/* previous token has no bearing. Perform normal closure logic */
								/*  */
								/*  */
								/* NOTE: fetched_Color is not valid unless logic flowed normally from a */
								/* corr read. Beware moving code */
								/* Note, terminator will not ever actually be "15" (0 is 15 but previous)*/
								/*  */
								terminator_Is_Excluded = TRUE;
								if ((fetched_Color == 0) 
									|| (fetched_Color == 8) 
									|| (fetched_Color == 12) 
									|| (fetched_Color == 14)) {
									terminator_Is_Excluded = FALSE;
								}
								
								/*  */
								/* We could have as few as one token plus a terminator */
								/* or as many as 33 for small or infinite for large */
								/* Terminating token may not be suited to end condition. */
								/* span may have to be severely limited */
								/*  */
								if (Close_A_Normal_Run_Of_Tokens(terminator_Is_Excluded
																		, total_32_Bit_Words
																		, &expected_Token_Value
																		, &expected_Token_Start_Position
																		, vertical_Coordinate
																		, horizontal_Coordinate
																		, fetched_Color
																		, &searching_Black
																		, &previous_Token_Is_Fast_Token) != SAFE_TRUE) {
									goto Error_Handler;
								}
															
							} /* if (previous_Token_Is_Fast_Token == TRUE) */
						
						} /* if (total_32_Bit_Words != 0) */
						
					} /* NOT if (total_32_Bit_Words == 1) */
				}
			}		
		} /* for (horizontal_Coordinate =0; ... */
	} /* for (vertical_Coordinate =0; ... */
								
	if (searching_Black != TRUE) {	
		/*  */
		/* Will only get here if pixels border right edge of frame */
		/* This pixel Span is finished (end of scan line at least) . Close it out */
		/*  */
		if (Back_Up_to_Previous_Line_And_Close_Pixel_Span(vertical_Coordinate
																, horizontal_Coordinate
																, &expected_Token_Value
																, &expected_Token_Start_Position
																, &previous_Token_Is_Fast_Token
																, &searching_Black
																, max_Columns) != SAFE_TRUE) {
			goto Error_Handler;
		}
																
		searching_Black = TRUE;
	} /* if (searching_Black != TRUE)  */
	
	
											
	Set_View_Port((GrafPtr)saved_Port);
	offscreen_Port_Selected = SAFE_FALSE;


	/*  */
	/* Ensure that end is zeroed out for sentinel safety logic */
	/*  */
	*(CARDINAL_32 *)((CARDINAL_32)(*g_Temp_Skip_List_Master_List_Hdl) + g_Temp_Skip_List_Master_List_Active_Size) = 0;
	*(CARDINAL_32 *)((CARDINAL_32)(*g_Temp_Skip_List_Master_List_Hdl) + g_Temp_Skip_List_Master_List_Active_Size + 4UL) = 0;
	*(CARDINAL_32 *)((CARDINAL_32)(*g_Temp_Skip_List_Master_List_Hdl) + g_Temp_Skip_List_Master_List_Active_Size + 8UL) = 0;
	g_Temp_Skip_List_Master_List_Active_Size += 12;
	
	/*  */
	/* Ensure that end is zeroed out for sentinel safety logic */
	/*  */
	*(CARDINAL_32 *)((CARDINAL_32)(*g_Temp_Skip_List_Data_Hdl) + g_Temp_Skip_List_Data_Active_Size) = 0;
	g_Temp_Skip_List_Data_Active_Size += 4;
	
	/*  */
	/* Unlock the four data handles so that memory copy and creation */
	/* does not interfere with them */
	/*  */
	HUnlock(g_Temp_Row_Raw_Data_Hdl);
	if (MemError() != noErr) {
		goto Corrupted_Memory_Error_Handler;
	}
	
	HUnlock(g_Temp_Skip_List_Data_Hdl);
	if (MemError() != noErr) {
		goto Corrupted_Memory_Error_Handler;
	}
	
	HUnlock(g_Temp_Skip_List_Master_List_Hdl);
	if (MemError() != noErr) {
		goto Corrupted_Memory_Error_Handler;
	}
	
	HUnlock(g_Temp_Scan_Line_Bounds_Hdl);
	if (MemError() != noErr) {
		goto Corrupted_Memory_Error_Handler;
	}
	
	
	/*  */
	/* Finish up back posting to structure headers*/
	/* Unlock, resize handles smaller if needed for those that were oversized. */
	/* To avoid Mac OS bias handles are not resized they are recopied. */
	/* This also avoids a few recompaction problems. */
	/*  */
	if (Copy_Memory_And_Resize(g_Temp_Row_Raw_Data_Hdl
														, g_Temp_Row_Raw_Data_Active_Size
														, row_Raw_Data_Hdl) != SAFE_TRUE) {
		goto Cannot_Duplicate_Memory_Error_Handler;
	}

	if (Copy_Memory_And_Resize(g_Temp_Skip_List_Data_Hdl
														, g_Temp_Skip_List_Data_Active_Size
														, skip_List_Data_Hdl) != SAFE_TRUE) {
		goto Cannot_Duplicate_Memory_Error_Handler;
	}
	
	if (Copy_Memory_And_Resize(g_Temp_Skip_List_Master_List_Hdl
														, g_Temp_Skip_List_Master_List_Active_Size
														, skip_List_Master_List_Hdl) != SAFE_TRUE) {
		goto Cannot_Duplicate_Memory_Error_Handler;
	}
	
	if (Copy_Memory_And_Resize(g_Temp_Scan_Line_Bounds_Hdl
														, g_Temp_Scan_Line_Bounds_Active_Size
														, scan_Line_Bounds_Hdl) != SAFE_TRUE) {
		goto Cannot_Duplicate_Memory_Error_Handler;
	}

	if (Destroy_Any_Temp_Global_Compression_Handles() != SAFE_TRUE) {
		/*  */
		/* Should not fail. Abort if so. */
		/*  */
		goto Error_Handler;
	}

	function_Result = SAFE_TRUE;
	goto Normal_Function_Exit;
	
	
Corrupted_Memory_Error_Handler:
	function_Result = SAFE_FALSE;
	Display_Serious_Error_Message(unit_Name, code_Name, "\pProblem during picture compression", "\pMemory structures OVERRAN. (Bad programming).");
	goto Normal_Function_Exit;
	
Cannot_Duplicate_Memory_Error_Handler:
	function_Result = SAFE_FALSE;
	Display_Serious_Error_Message(unit_Name, code_Name, "\pProblem during picture compression", "\pNot enough memory for handle duplication.");
	goto Normal_Function_Exit;
	
Error_Handler:
	function_Result = SAFE_FALSE;
	
	Display_Serious_Error_Message(unit_Name, code_Name, "\pUnexpected problem in picture compression", "\pLow memory, or Wrong usage?");

Normal_Function_Exit:

	if (offscreen_Port_Selected == SAFE_TRUE) {
		Set_View_Port((GrafPtr)saved_Port);
		offscreen_Port_Selected = SAFE_FALSE;
	}

	if (offscreen_Port_Created == SAFE_TRUE) {
		if (Close_Offscreen_Pict_Port(offscreen_Ptr) != SAFE_TRUE) {
		
		}
	}
	
	if (Destroy_Any_Temp_Global_Compression_Handles() != SAFE_TRUE) {
		/*  */
		/* Should not fail. Abort if so. */
		/*  */
	}

	return(function_Result);
	
} /* Safe_Boolean Compact_Region_And_Bitmap */
/*=**************************************************************************=*/


/*=**************************************************************************=*/
/* END OF FILE: Fast_Image_Compaction.c */
