/* START OF FILE: Graphics_Primitives_Support.c */


/*=**************************************************************************=*/
/* [---BEGIN---] FILE HEADER COMMENTS */
/**
 ** SCREEN VIEW = [Adobe Syntax Black-10, Tabs = 4]
 ** PRINTER VIEW = [Adobe Syntax Black-10, Tabs = 4]
 ** =======================================================================
 ** Written by James Merkle
 ** Graphics_Primitives_Support.c - Support routines for graphics programming
 ** (C) Copyright 1994 James Merkle
 ** All rights reserved, except those wishing to "copy" moderate sections of code or files may do so, so long as
 ** such code is never used directly either in animation-related tools or multimedia tools that are  
 ** used in live performance or live broadcast or recorded mass market broadcast or film. Desktop usage not restricted.
 ** Conventional desktop game or entertainment not restricted.
 ** =======================================================================
 **
 **
 ** ============
 ** Revision History
 ** ============
 **
 **  1995.03.04		(James Merkle)
 **						Initial coding started from scratch
 **
 **
 **
 ** =================
 ** OTHER INFORMATION
 ** =================
 ** 
 ** 
 **
 **
 **/
/* [----END----] FILE HEADER COMMENTS */
/*=**************************************************************************=*/


/*=**************************************************************************=*/
/* [---BEGIN---] #include AREA */

#include "Shell_Globals.h"
#include "Core_Globals.h"
#include "Shell_Dialogs.h"
#include "String_Utilities.h"

#include "Indexed_Background_Removal.h"
#include "Error_Utilities.h"
#include "Interface_Utilities.h"
#include "Copy_Compacted_Picture.h"

/* ===
#include "Core_Universal.h"
=== */

#include "Graphics_Primitives_Support.h"

/* [----END----] #include AREA */
/*=**************************************************************************=*/


/*=**************************************************************************=*/
/* [---BEGIN---] #define AREA */


/* [----END----] #define AREA */
/*=**************************************************************************=*/


/*=**************************************************************************=*/
/* [---BEGIN---] TYPE AREA */


/* [----END----] TYPE AREA */
/*=**************************************************************************=*/


/*=**************************************************************************=*/
/* [---BEGIN---] PROTOTYPE AREA */


/* [----END----] PROTOTYPE AREA */
/*=**************************************************************************=*/


/*=**************************************************************************=*/
/* [---BEGIN---] GLOBAL DEFINITION AREA */


/* [----END----] GLOBAL DEFINITION AREA */
/*=**************************************************************************=*/


/*=**************************************************************************=*/
/* [---BEGIN---] LOCAL STATIC DEFINITION AREA */

static CARDINAL_8	*unit_Name = "\pGraphics_Primitives_Support";


/* [----END----] LOCAL STATIC DEFINITION AREA */
/*=**************************************************************************=*/


/*=**************************************************************************=*/
Safe_Boolean Find_Similar_Index_Color_But_Different(CARDINAL_32 bit_Depth
															, CARDINAL_32 index_Color
															, CARDINAL_32 *replacement_Index_Color)
/*  */
/* This routine is incomplete! Eventually it will use current active color table for port */
/* and keep looking for looser and looser index values until it finds one */
/* different than the one it is trying to avoid. It will have to try three different */
/* techniques : changing just red channel, blue or green , either up or down */
/* or (prefereable if slight, changing brightness across all channels. */
/* very complex, so for now just some hack values are used. */
/* Reminder , index might not be a CLUT and could very well be a */
/* direct device that is going to be used for chroma (when AV definition is enhanced) */
/*  */
{
	Safe_Boolean		function_Result;
	CARDINAL_8		*code_Name;
/* END VAR */


	code_Name = "\pFind_Similar_Index_Color_But_Different";

	function_Result = SAFE_FALSE;

	switch(index_Color) {
		case 17:
			/*  */
			/* BRIGHT ORANGE */
			/*  */
			*replacement_Index_Color = 16;
		break;
	
		case k_PREFERRED_CHROMA_RESERVED_CHANNEL:
			/*  */
			/* Preferred Chroma index technology */
			/*  */
			*replacement_Index_Color = k_PREFERRED_CHROMA_RESERVED_CHANNEL_REPLACEMENT;
		break;
		
		goto Error_Handler;
	}


	function_Result = SAFE_TRUE;
	goto Normal_Function_Exit;
	
Error_Handler:
	Display_Serious_Error_Message(unit_Name, code_Name, "\pProblem is because Program is unfinished. It only allows a few chroma replacements", "\pUse index color 17, 254, or almost black direct");

	function_Result = SAFE_FALSE;
		
Normal_Function_Exit:	
	return(function_Result);
}
/*=**************************************************************************=*/


/*=**************************************************************************=*/
void Reposition_Along_Byte_Coordinates(WindowPtr	window_To_Reposition, Boolean mandatory)
{
	Rect 					global_Rect;
	INTEGER_32	 		bits_Per_Pixel_Union;
	INTEGER_32	 		primitives_Current_Bytes_Per_Pixel;
	CARDINAL_16  		total_Intersected_Screens;
	Safe_Boolean			rect_Is_Within_A_Screen;
	INTEGER_16			units_To_Shift;
	INTEGER_32			remainder;
	SPEED_DIGITS_REAL	units_Alignment_Per_Pixel;	
	INTEGER_32			offset_To_Global_Horizontal;
	INTEGER_32			left_Global, top_Global;
	Rect 					global_Source_Rect;
	INTEGER_32	 		bits_Per_Pixel_Greatest;
	INTEGER_16			video_Monitor_Driver_Ref;
	INTEGER_8			video_Monitor_ID;
	INTEGER_8			video_Monitor_Slot;
	CARDINAL_32		video_Optional_Base_Address;
	PixMapHandle		video_Pix_Map_Handle;
	Safe_Boolean			slot_Information_Valid;													
	GDHandle				graphics_Device;
	Str255					temp_String_1,temp_String_2;
/* END VAR */



	/*  */
	/* Positioned here just for the debugger */
	/*  */
	rect_Is_Within_A_Screen = SAFE_FALSE;
	
	global_Rect = (**((WindowPeek)window_To_Reposition)->contRgn).rgnBBox;
			
	Determine_Information_Of_Device_Containing_Rect(&global_Rect
												, &video_Monitor_Driver_Ref
												, &video_Monitor_Slot
												, &video_Monitor_ID
												, &video_Optional_Base_Address
												, &bits_Per_Pixel_Union
												, &bits_Per_Pixel_Greatest
												, &total_Intersected_Screens
												, &rect_Is_Within_A_Screen
												, &video_Pix_Map_Handle
												, &graphics_Device
												, &slot_Information_Valid);

	if ((rect_Is_Within_A_Screen != SAFE_TRUE) || (slot_Information_Valid != SAFE_TRUE)) {
		offset_To_Global_Horizontal = global_Rect.left;
		
		if (rect_Is_Within_A_Screen != SAFE_TRUE) {
			primitives_Current_Bytes_Per_Pixel = (**((CGrafPtr)window_To_Reposition)->portPixMap).pixelSize / 8.0;
		}
		else {
			primitives_Current_Bytes_Per_Pixel = bits_Per_Pixel_Union / 8.0;
		}
	}
	else {
		offset_To_Global_Horizontal = global_Rect.left - (*video_Pix_Map_Handle)->bounds.left;
		primitives_Current_Bytes_Per_Pixel = bits_Per_Pixel_Union / 8.0;
	}

	if (rect_Is_Within_A_Screen != SAFE_TRUE) {
		if (mandatory == TRUE) {
			Perform_OK_Dialog("\pThe window is not located completely within one device and therefore this version of the program will not draw to it. Move it!");
		}
		goto Function_Exit;
	}											


	if (g_Shell.library.force_Window_Alignment_To_32_Bits == TRUE) {
	
#if !defined(k_ASSUME_68040_ALWAYS)
#error "UNDEFINED PARAMETER!"
#endif
#if  (k_ASSUME_68040_ALWAYS == 0)
		units_Alignment_Per_Pixel = (bits_Per_Pixel_Union / 32.0);
#else	
		units_Alignment_Per_Pixel = (bits_Per_Pixel_Union / 128.0);
#endif /* if  (k_ASSUME_68040_ALWAYS == 0) */

	}
	else {
		units_Alignment_Per_Pixel = (bits_Per_Pixel_Union / 8.0);
	}

	units_To_Shift = (offset_To_Global_Horizontal * units_Alignment_Per_Pixel);
	
	if (units_Alignment_Per_Pixel < 1.0) {
							
		remainder = (INTEGER_32)( offset_To_Global_Horizontal 
						- ( (1.0 * units_To_Shift) / units_Alignment_Per_Pixel) ) ;
		
		if (remainder != 0) {
			left_Global = (**((WindowPeek)window_To_Reposition)->contRgn).rgnBBox.left - remainder;
			top_Global = (**((WindowPeek)window_To_Reposition)->contRgn).rgnBBox.top;
			MoveWindow(window_To_Reposition, left_Global, top_Global, FALSE);
		
			/*  */
			/* Try, try, again for safety, (illogical, AND does not ensure divisibility by 32) */
			/* But for peace of mind anyway */
			/*  */
			
			if ((rect_Is_Within_A_Screen == SAFE_TRUE) && (slot_Information_Valid == SAFE_TRUE)) {
			
				offset_To_Global_Horizontal = (**((WindowPeek)window_To_Reposition)->contRgn).rgnBBox.left
															- (*video_Pix_Map_Handle)->bounds.left;
			}
			else {
				offset_To_Global_Horizontal = (**((WindowPeek)window_To_Reposition)->contRgn).rgnBBox.left;
			}
			
			units_To_Shift = (offset_To_Global_Horizontal * units_Alignment_Per_Pixel);
			
			if (units_Alignment_Per_Pixel < 1.0) {
			
				remainder = (INTEGER_32)( offset_To_Global_Horizontal 
								- ( (1.0 * units_To_Shift) / units_Alignment_Per_Pixel) ) ;
				
				if (remainder != 0) {
					/*  */
					/* Impossible unless system being screwed with */
					/*  */
					if (mandatory == TRUE) {
						Convert_Long_To_String(remainder, temp_String_1 , NO_LIMIT);
						Copy_Pascal_String("\pWarning : For proper results, this window should be moved ",temp_String_2);
						Concat_After_Pascal_String(temp_String_2, temp_String_1 );
						Concat_After_Pascal_String(temp_String_2, "\p pixels leftward" );
						Perform_OK_Dialog(temp_String_2);
					}
				}
			}
		}
	}

		
Function_Exit:
; /* C crud */

} /* void Reposition_Along_Byte_Coordinates(WindowPtr	*window_To_Reposition) */
/*=**************************************************************************=*/


/*=**************************************************************************=*/
void Support_Update_By_Getting_Pict(PicHandle	*current_PICT, WindowPtr	*active_Window)
{
 Rect		source_Rect;
 Rect		destination_Rect;
/* END VAR */


	if (*current_PICT != nil) {
		KillPicture (*current_PICT);
	}
	
	/*=--------------------------------------------------------------------------=*/
	/* These color settings are probably unimportant except for tidiness */
	/*=--------------------------------------------------------------------------=*/
	RGBForeColor(&g_Shell.library.color_Black);
	RGBBackColor(&g_Shell.library.color_White);


	ClipRect(&((*((WindowPeek)(*active_Window))).port.portRect));

	source_Rect = ((*active_Window)->portRect);
	destination_Rect = ((*active_Window)->portRect);

	*current_PICT = OpenPicture(&((*active_Window)->portRect));

	/*=--------------------------------------------------------------------------=*/
	/* Because Source and Destination overlap, use screenBits ??? */
	/*=--------------------------------------------------------------------------=*/
/*============
	LocalToGlobal(&(((Point*)&(source_Rect))[0]));
	LocalToGlobal(&(((Point*)&(source_Rect))[1]));

	LocalToGlobal(&(((Point*)&(destination_Rect))[0]));
	LocalToGlobal(&(((Point*)&(destination_Rect))[1]));


	CopyBits (&screenBits
				,&screenBits
				,&source_Rect
				,&destination_Rect
				,srcCopy
				, nil);
==============*/

	CopyBits (&((*((WindowPeek)(*active_Window))).port.portBits)
				,&((*((WindowPeek)(*active_Window))).port.portBits)
				,&source_Rect
				,&destination_Rect
				,srcCopy
				, nil);
				
	ClosePicture();
	
} /* Support_Update_By_Getting_Pict */
/*=**************************************************************************=*/


/*=**************************************************************************=*/
void Support_Update_By_Putting_Pict(PicHandle	*current_PICT, WindowPtr	*active_Window)
{
	WindowPtr	tempPtr;
/* END VAR */

	GetPort (&tempPtr);
	Set_View_Port (*active_Window);
	DrawPicture (*current_PICT, &(*active_Window)->portRect);
	Set_View_Port (tempPtr);
} /* Support_Update_By_Putting_Pict */
/*=**************************************************************************=*/


/*=**************************************************************************=*/
/* END OF FILE: Graphics_Primitives_Support.c */






