/* START OF FILE: Indexed_Background_Removal.c */


/*=**************************************************************************=*/
/* [---BEGIN---] FILE HEADER COMMENTS */
/**
 ** SCREEN VIEW = [Adobe Syntax Black-10, Tabs = 4]
 ** PRINTER VIEW = [Adobe Syntax Black-10, Tabs = 4]
 ** =======================================================================
 ** Written by James Merkle
 ** Indexed_Background_Removal.c - Creates region of indexed color images less backgrounds. Very fast
 ** (C) Copyright 1993 by James Merkle
 ** All rights reserved, except those wishing to "copy" moderate sections of code or files may do so, so long as
 ** such code is never used directly either in animation-related tools or multimedia tools that are  
 ** used in live performance or live broadcast or recorded mass market broadcast or film. Desktop usage not restricted.
 ** Conventional desktop game or entertainment not restricted.
 ** =======================================================================
 **
 **
 ** ============
 ** Revision History
 ** ============
 **
 **  1993.03.16		(James Merkle)
 **						Initial coding started
 **
 **
 **
 **
 ** =================
 ** OTHER INFORMATION
 ** =================
 **	Feature: Votes using four corner arbitration.
 **	Also matches 8 neighbors and eliminates isolated pixels.
 **	Also matches against list of 6 kill colors or more.
 **	Also avoids Apple's routines if possible for speed.
 **
 **/
/* [----END----] FILE HEADER COMMENTS */
/*=**************************************************************************=*/


/*=**************************************************************************=*/
/* [---BEGIN---] #include AREA */

#include <QDOffscreen.h>

#include "Shell_Globals.h"

#if (7 == 11)
#include "Core_Universal.h" /* Non_Specific_Support_Routines */
#include "Core_Specific.h" /*System_Dependent_Hooks.h */
#include "SysHooks.h" /*System_Dependent_Hooks.h */
#include "Core_Globals.h" /*System_Dependent_Hooks.h */
#endif

#include "Shell_Globals.h"
#include "Cursor_Utilities.h"
#include "Shell_Dialogs.h"
#include "Interface_Utilities.h"
#include "Error_Utilities.h"

#include "Memory_Utilities.h"

#include "Off_Screen_Color.h"
#include "Graphics_Primitives.h"

#include "Direct_Background_Removal.h"
#include "Indexed_Background_Removal.h"


/* [----END----] #include AREA */
/*=**************************************************************************=*/


/*=**************************************************************************=*/
/* [---BEGIN---] #define AREA */

/*  */
/* 3 or 4 would be reasonable. Must be 1 or greater */
/* was "4" */
/*  */
#define k_MAXIMUM_BACKGROUND_COLORS	6

/*  */
/* 4 or 5 would be reasonable. Can be 0 */
/* "4" represents a 16 pixel grid searched for background colors */
/* "6" has been used successfully, trying 8 now */
/*  */
#define k_PIXELS_TO_SCAN_FOR_BACKGROUND_COLORS	8

/*  */
/* Some grabbing cards do not treat extreme left or right too well */
/* so for grabs eliminate the possibility of out of bounds colors used for */
/* sample base. */
/* WARNING : LEFT side is supposed to be at least 21, */
/* and right is needs only 10 but is paired for symmetry. */
/* Up and down margins are less critical */
/*  */
#define k_CROP_SAMPLE_X_INSET			(29)
#define k_CROP_SAMPLE_Y_INSET			(9)

/* ====
HACK HACK add brightness 
#define k_COLOR_DEVIANCE					(40L * 256L)
==== */
#define k_COLOR_DEVIANCE					(250UL * 256UL)


#define COLOR_PORT_PORTBITS_CHECK 		0xC000

/*  */
/* For paranoia only during debugging */
/* This may avoid problems of BW Bitmap being off by a few pixels */
/*  */
#define k_PIXEL_BORDER_PARANOIA_SAFETY_PAD		8


/* [----END----] #define AREA */
/*=**************************************************************************=*/


/*=**************************************************************************=*/
/* [---BEGIN---] TYPE AREA */


/* [----END----] TYPE AREA */
/*=**************************************************************************=*/


/*=**************************************************************************=*/
/* [---BEGIN---] PROTOTYPE AREA */


void Copy_Indexed_Non_Masked_Bits(GrafPtr offscreen_Graph_Ptr
				, Safe_Boolean		use_Fancy_Bit_Map
				, GrafPtr					simple_Bit_Map_Ptr
				, CGrafPtr				fancy_Bit_Map_Ptr
				, Rect *source_Rect
				, Rect *target_Rect
				, INTEGER_32 mask_Color_Total
				, Color_Type mask_Colors[]);
																			
Safe_Boolean Check_Pixel_Neighbors_Are_Background(INTEGER_32 x_Position
																								,INTEGER_32 y_Position
																								,INTEGER_32 mask_Color_Total
																								,Color_Type mask_Colors[]
																								, Boolean use_Direct_Screen_Access);
																								
Safe_Boolean	Create_Region_For_Indexed_Port_Image(GrafPtr offscreen_Source_Port
																	, Rect 									*source_Rect
																	, Safe_Boolean		 			create_Color_Style_Port
																	, GrafPort 							*offscreen_Graph_Port
																	, Off_Screen_Ptr 					modern_Offscreen_Graph_Port);

/* [----END----] PROTOTYPE AREA */
/*=**************************************************************************=*/


/*=**************************************************************************=*/
/* [---BEGIN---] GLOBAL DEFINITION AREA */


/* [----END----] GLOBAL DEFINITION AREA */
/*=**************************************************************************=*/


/*=**************************************************************************=*/
/* [---BEGIN---] LOCAL STATIC DEFINITION AREA */

static CARDINAL_8	*unit_Name = "\pIndexed_Background_Removal";

/* [----END----] LOCAL STATIC DEFINITION AREA */
/*=**************************************************************************=*/


/*=**************************************************************************=*/
Safe_Boolean Destroy_Simple_BW_Bit_Map(Safe_Boolean is_Color_Style_Port
																				, Handle		*image_Handle_Ptr
																				, GrafPort 		*offscreen_Graph_Port
																				, Off_Screen_Ptr modern_Offscreen_Graph_Port)
{
	Safe_Boolean		function_Result;
/* END VAR */


	function_Result = SAFE_FALSE;
	
	if (is_Color_Style_Port == SAFE_TRUE) {
		if (modern_Offscreen_Graph_Port->initialized == SAFE_TRUE) {
		 	if (Close_Offscreen_Pict_Port(modern_Offscreen_Graph_Port) != SAFE_TRUE) {
				/*  */
				/* might as well complain about it. */
				/*  */
		 		goto Error_Handler;
		 	}
		 }
	}
	else {
	
		if (image_Handle_Ptr != nil) {
			ClosePort(offscreen_Graph_Port);
			DisposeHandle(*image_Handle_Ptr);
			if (MemError() != noErr) {
				goto Error_Handler;
			}
		}
	
	} /* NOT if (is_Color_Style_Port == SAFE_TRUE) */
	
	
	function_Result = SAFE_TRUE;
	goto Normal_Function_Exit;
	
Error_Handler:
	function_Result = SAFE_FALSE;

Normal_Function_Exit:

	return(function_Result);

} /* Safe_Boolean Destroy_Simple_BW_Bit_Map ... */
/*=**************************************************************************=*/


/*=**************************************************************************=*/
Safe_Boolean Create_Simple_BW_Bit_Map(Rect *desired_Rect
																	, Handle		*image_Handle_Ptr
																	, Safe_Boolean create_Color_Style_Port
																	, GrafPort 		*offscreen_Graph_Port
																	, Off_Screen_Ptr modern_Offscreen_Graph_Port)
{
	INTEGER_32		bytes_Per_Row;
	GrafPtr				offscreen_Graph_Ptr;
	CARDINAL_32	total_Graphics_Storage;
	GrafPtr				saved_Port;
	Off_Screen_Ptr	offscreen_Ptr;
	OSErr					os_Error;
	Rect						current_Rect;
	INTEGER_32		source_Bit_Width;
	INTEGER_32		source_Bit_Height;
	PenState				saved_Pen_State;
	Safe_Boolean		function_Result;
	CARDINAL_8		*code_Name;
/* END VAR */


	code_Name = "\pCreate_Simple_BW_Bit_Map";
	function_Result = SAFE_FALSE;


	GetPenState (&saved_Pen_State);
	GetPort(&saved_Port);		


	source_Bit_Width = (*desired_Rect).right - (*desired_Rect).left;
	source_Bit_Height = (*desired_Rect).bottom - (*desired_Rect).top;
	
	/*  */
	/* Paranoia resizing. */
	/*  */
	source_Bit_Width += k_PIXEL_BORDER_PARANOIA_SAFETY_PAD;
	source_Bit_Height += k_PIXEL_BORDER_PARANOIA_SAFETY_PAD;
	
	SetRect(&current_Rect
					, 0
					, 0
					, source_Bit_Width
					, source_Bit_Height); 

	if (create_Color_Style_Port == SAFE_TRUE) {
	
		offscreen_Ptr = modern_Offscreen_Graph_Port;
		
		/*  */
		/* Kind of silly, because only used as BW */
		/*  */
		
		offscreen_Ptr->color_Table_Handle = GetCTable(k_DEFAULT_2_COLOR_SYSTEM_CLUT);
		if (offscreen_Ptr->color_Table_Handle == nil) {
				Display_Serious_Error_Message(unit_Name
																	, code_Name
																	, "\pProblem Loading System CLUT (2)"
																	, "\pCorrupted?");
			goto Error_Handler;
		}
					
		if (Create_Off_Screen_Port(&current_Rect
												, 1
												, offscreen_Ptr->color_Table_Handle
												, (CGrafPtr)&(offscreen_Ptr->offscreen_Port.color_Port)
												, &(offscreen_Ptr->offscreen_GDevice_Hdl)
												, &(offscreen_Ptr->image_Handle)) != noErr) {
			goto Error_Handler;
		}
		
		offscreen_Ptr->initialized =  SAFE_TRUE; 
		offscreen_Ptr->color_Type = k_COLOR_QUICKDRAW;
		Lock_Video_Memory_For_Lazy_But_Safe_Use(offscreen_Ptr);
	
		offscreen_Graph_Ptr = (GrafPtr)&(offscreen_Ptr->offscreen_Port.color_Port);
	}
	else {
		/*  */
		/* Round up to nearest long word and even align it */
		/* 32 bits wide  == 4 bytes total */
		/* Quickdraw works with even alignment but is FASTER for 4 byte */
		/*  */
		bytes_Per_Row = (((source_Bit_Width + 31) >> 5) << 2); 
		
		/*  */
		/* In the future, If we were safety-minded, we would multiply  */
		/* the byte width by the height to see if result was too big to fit */
		/* simple structures. No big deal.  */
		/*  */
		total_Graphics_Storage = source_Bit_Height * bytes_Per_Row;
		*image_Handle_Ptr = NewHandle(total_Graphics_Storage);
		if ((MemError() != noErr) || (*image_Handle_Ptr == nil)) {
			goto Error_Handler;
		}
			
		offscreen_Graph_Ptr = offscreen_Graph_Port;
		OpenPort(offscreen_Graph_Ptr);
		
		
		MoveHHi(*image_Handle_Ptr);
		os_Error = MemError();
		if ((os_Error != noErr) && (os_Error != -117)){
			goto Fatal_Memory_Handler;
		}
		
		/*  */
		/* Sadly, must remain locked forever until port is no longer needed */
		/*  */
		HLock(*image_Handle_Ptr);
		if (MemError() != noErr) {
			goto Fatal_Memory_Handler;
		}

		(*offscreen_Graph_Ptr).portBits.baseAddr = (Ptr)(**image_Handle_Ptr);
		(*offscreen_Graph_Ptr).portBits.rowBytes = bytes_Per_Row;
		(*offscreen_Graph_Ptr).portBits.bounds = current_Rect;
		
		/*  */
		/* HACK HACK HACK WAS NOT HERE before 1995.01 */
		/*  */
		(*offscreen_Graph_Ptr).portRect = current_Rect;
	
	} /* NOT if (create_Color_Style_Port == SAFE_TRUE) */

		
	Set_View_Port(offscreen_Graph_Ptr);
	
	/*  */
	/* Needed for CopyBits calls to be certain. */
	/*  */
	ForeColor(blackColor);
	BackColor(whiteColor);

	/*  */
	/* For safety to clear out the memory */
	/*  */
	EraseRect(&current_Rect); 
	
	Set_View_Port(saved_Port);
	SetPenState (&saved_Pen_State);
	
	function_Result = SAFE_TRUE;
	goto Normal_Function_Exit;
	
Fatal_Memory_Handler:

Error_Handler:
	function_Result = SAFE_FALSE;

Normal_Function_Exit:

	return(function_Result);

} /* Safe_Boolean Create_Simple_BW_Bit_Map ... */
/*=**************************************************************************=*/


/*=**************************************************************************=*/
Safe_Boolean Copy_Rect_Non_Mask_Into_BW_Bit_Map(
																		Rect *source_Rect
																		, Safe_Boolean		use_Fancy_Bit_Map
																		, GrafPtr					simple_Bit_Map_Ptr
																		, CGrafPtr				fancy_Bit_Map_Ptr
																		, Safe_Boolean image_Is_Indexed_Entry
																		, INTEGER_32 mask_Color_Total
																		, Color_Type mask_Colors[])
	/*  */
	/* This Allocates a handle and fills it with bitmap image */
	/* Set the correct port before calling. */
	/* It was written for animation purposes */
	/* invloving offscreen buffers (source bitmaps, result bitmap, etc. ) */
	/*  */
{
	Rect						source_Bit_Rect;
	Rect						current_Rect;
	
	INTEGER_32		source_Bit_Width;
	INTEGER_32		source_Bit_Height;
	Rect						destination_Rect;
	Safe_Boolean		function_Result;
	OSErr					os_Error;
	Rect						offscreen_Rect;
	GrafPtr				saved_Port;
	PenState				saved_Pen_State;
/* END VAR */


	function_Result = SAFE_FALSE;
			
	source_Bit_Width = (*source_Rect).right - (*source_Rect).left;
	source_Bit_Height = (*source_Rect).bottom - (*source_Rect).top;

	/*  */
	/* Actually needed for CopyBits calls , so be certain. */
	/*  */
	ForeColor(blackColor);
	BackColor(whiteColor);

	
	GetPenState (&saved_Pen_State);
	GetPort(&saved_Port);		
			
	if (use_Fancy_Bit_Map == SAFE_TRUE) {
		Set_View_Port((GrafPtr)fancy_Bit_Map_Ptr);
	}
	else {
		Set_View_Port(simple_Bit_Map_Ptr);
	}

	
	/*  */
	/* Needed for CopyBits calls to be certain. */
	/*  */
	ForeColor(blackColor);
	BackColor(whiteColor);

	/*  */
	/* For safety to clear out the memory */
	/*  */
	EraseRect(&current_Rect);
	
	/* ==== */
	
	SetPort(saved_Port);		
	

	SetRect(&destination_Rect, 0, 0, source_Bit_Width, source_Bit_Height);
	
	/*  */
	/* Instead of CopyBits, use a better transform that avoids */
	/* stray pixels */
	/*  */
	if (image_Is_Indexed_Entry == SAFE_TRUE) {

		Copy_Indexed_Non_Masked_Bits(saved_Port
					, use_Fancy_Bit_Map
					, simple_Bit_Map_Ptr
					, fancy_Bit_Map_Ptr
					, source_Rect
					, &destination_Rect
					, mask_Color_Total
					, mask_Colors);
	}
	else {
		Copy_Direct_Non_Masked_Bits(saved_Port
					, use_Fancy_Bit_Map
					, simple_Bit_Map_Ptr
					, fancy_Bit_Map_Ptr
					, source_Rect
					, &destination_Rect
					, mask_Color_Total
					, mask_Colors);
	}

/* ====
	CopyBits(&(saved_Port->portBits)
				, &(offscreen_Graph_Ptr->portBits)
				, source_Rect
				, &destination_Rect
				, srcCopy
				, nil);
=== */
	
	Set_View_Port(saved_Port);
	SetPenState (&saved_Pen_State);
	

	function_Result = SAFE_TRUE;
	goto Normal_Function_Exit;
		
		
Fatal_Memory_Handler:

Error_Handler:
	function_Result = SAFE_FALSE;

Normal_Function_Exit:

	return(function_Result);

} /* Safe_Boolean Copy_Rect_Non_Mask_Into_BW_Bit_Map(Rect *source_Rect... */
/*=**************************************************************************=*/


/*=**************************************************************************=*/
Safe_Boolean Check_Pixel_Neighbors_Are_Background(INTEGER_32 x_Position
																								,INTEGER_32 y_Position
																								,INTEGER_32 mask_Color_Total
																								,Color_Type mask_Colors[]
																								, Boolean use_Direct_Screen_Access)
{
	CARDINAL_32	pixel_Color;
	Safe_Boolean		function_Result;
	INTEGER_32		y_Offset;
	INTEGER_16		x_Offset;
	INTEGER_16		loop;
	RGBColor			sampled_Extra_Color;
	Boolean				still_Searching;
/* END VAR */
	
	
	function_Result = SAFE_FALSE;
	
	for (loop = 0; (loop < mask_Color_Total); loop++) {
		if ((mask_Colors[loop]).use_Raw_Index != SAFE_TRUE) {
			use_Direct_Screen_Access = FALSE;
		}
	}

	for(y_Offset=-1; (y_Offset < 2); y_Offset++ ) {
		for(x_Offset=-1; (x_Offset < 2); x_Offset++ ) {
			/*  */
			/* Scan grid looking for background color list matches IGNORE */
			/* center point obviously because it is not important to NOT match. */
			/*  */
			if ((y_Offset != 0) || (x_Offset != 0)) {
			
				if (use_Direct_Screen_Access == TRUE){
					
					/*  */
					/* This gets the pixel in a fast manner involving no multiplication */
					/*  */
					Get_Pixel_Index(x_Position + x_Offset, y_Position + y_Offset, &pixel_Color);
					
					still_Searching = TRUE;
					for (loop = 0; ((loop < mask_Color_Total) && still_Searching == TRUE); loop++) {
						if ((mask_Colors[loop]).raw_Index == pixel_Color) {
							still_Searching = FALSE;
						}
					}
				}
				else {
			
					GetCPixel(x_Position + x_Offset, y_Position + y_Offset, &sampled_Extra_Color);
								
					still_Searching = TRUE;
					for (loop = 0; ((loop < mask_Color_Total) && still_Searching == TRUE); loop++) {
						if ((sampled_Extra_Color.red == mask_Colors[loop].rgb_Color.red)
									&& (sampled_Extra_Color.green == mask_Colors[loop].rgb_Color.green)
									&& (sampled_Extra_Color.blue == mask_Colors[loop].rgb_Color.blue)) {
							still_Searching = FALSE;
						}
					}
					
				}
				
				if (still_Searching == TRUE) {
					goto Not_Definite_Background_Color;
				}

			}
		}
	}
	
	function_Result = SAFE_TRUE;
	goto Normal_Function_Exit;
	
Not_Definite_Background_Color:
	function_Result = SAFE_FALSE;

Normal_Function_Exit:

	return(function_Result);
	
}
/*=**************************************************************************=*/


/*=**************************************************************************=*/				
void Copy_Indexed_Non_Masked_Bits(GrafPtr offscreen_Graph_Ptr
				, Safe_Boolean		use_Fancy_Bit_Map
				, GrafPtr					simple_Bit_Map_Ptr
				, CGrafPtr				fancy_Bit_Map_Ptr
				, Rect *source_Rect
				, Rect *target_Rect
				, INTEGER_32 mask_Color_Total
				, Color_Type mask_Colors[])
/*  */
/* Creates mask but does not allow isolated pixels in the region */
/* for quality. takes a hell of a long time. GetCPixel is avoided if direct is possible */
/*  */
{
	register	INTEGER_32	current_X;
	register	INTEGER_32	current_Y;
	register	INTEGER_32	current_Length;
	register	INTEGER_32	current_Height;
	register	CARDINAL_8  pixel_Neighbors;
	RGBColor		current_Color;
	INTEGER_32	target_X_Offset;
	INTEGER_32	target_Y_Offset;
	INTEGER_32	source_X_Offset;
	INTEGER_32	source_Y_Offset;
	INTEGER_32	x_Offset;
	INTEGER_32	y_Offset;
	INTEGER_32	last_X_Position;
	CARDINAL_32	bytes_Per_Row;
	Boolean			is_An_Isolated_Pixel;
	CARDINAL_32	raw_Pixel_Address;
	Rect						bounding_Rect;
	CARDINAL_32	current_Screen_Address;
	Boolean				use_Apples_Color_Routines;
	Boolean				use_Direct_Screen_Access;
	Boolean				pixel_Is_Main_Mask_Color;
	CARDINAL_32	pixel_Color;
	CARDINAL_8		*code_Name;
/* END VAR */


	code_Name = "\pCopy_Non_Masked_Bits";
	
	/*  */
	/* Ordinarily set to FALSE for safety */
	/*  */
	use_Apples_Color_Routines = TRUE;
	use_Direct_Screen_Access = FALSE;
	
	
	Set_View_Port(offscreen_Graph_Ptr);
	
	if (use_Apples_Color_Routines == TRUE) {
		if (Initialize_Graphics_Primitives(FALSE, TRUE) != TRUE) {
			Display_Serious_Error_Message(unit_Name, code_Name, "\pInitialize_Graphics_Primitives failed.  Program will use indirect access.", "\p");
		}
		else {
			use_Direct_Screen_Access = TRUE;
		}
	}
	
	
	if ((mask_Colors[0]).use_Raw_Index != SAFE_TRUE) {
		use_Direct_Screen_Access = FALSE;
	}
	
	/*  */
	/* For direct port writing into later */
	/*  */
	if (use_Fancy_Bit_Map == SAFE_TRUE) {
		raw_Pixel_Address = (CARDINAL_32)(GetPixBaseAddr(fancy_Bit_Map_Ptr->portPixMap));
		if (raw_Pixel_Address == 0) {
			goto Error_Handler;
		}
		bytes_Per_Row = ((*(fancy_Bit_Map_Ptr->portPixMap))->rowBytes & (~COLOR_PORT_PORTBITS_CHECK));
		bounding_Rect = (*(fancy_Bit_Map_Ptr->portPixMap))->bounds;				
	}
	else {
		raw_Pixel_Address = (CARDINAL_32)(*simple_Bit_Map_Ptr).portBits.baseAddr;
		bytes_Per_Row = (*simple_Bit_Map_Ptr).portBits.rowBytes;
		bounding_Rect = (*simple_Bit_Map_Ptr).portBits.bounds;					
	}
	
	target_X_Offset = (*target_Rect).left;
	target_Y_Offset = (*target_Rect).top;
	source_X_Offset = (*source_Rect).left;
	source_Y_Offset = (*source_Rect).top;
	current_Length = (*source_Rect).right - (*source_Rect).left;
	current_Height = (*source_Rect).bottom - (*source_Rect).top;
	/*  */
	/* we do not want to examine the last column of pixels for inclusion */
	/*  */
	last_X_Position = (current_Length - 1);
	
	/*  */
	/* This code does not include using the left top and right */
	/* sides of the frame for its comparisons. Bottom is checked */
	/* for the hell of it*/
	/*  */
	for (current_Y = 1; (current_Y < current_Height); current_Y++) {
		for (current_X = 1; (current_X < last_X_Position); current_X++) {
		
			x_Offset = current_X + source_X_Offset;
			y_Offset = current_Y + source_Y_Offset;
			
			pixel_Is_Main_Mask_Color = TRUE;
			if (use_Direct_Screen_Access == TRUE){
				
				/*  */
				/* This gets the pixel in a fast manner involving no multiplication */
				/*  */
				Get_Pixel_Index(x_Offset, y_Offset, &pixel_Color);							
				if ((mask_Colors[0]).raw_Index != pixel_Color) {
					pixel_Is_Main_Mask_Color = FALSE;
				}
			}
			else {
				GetCPixel(x_Offset, y_Offset, &current_Color);
				
				if ((current_Color.red != (mask_Colors[0]).rgb_Color.red)
					|| (current_Color.blue != (mask_Colors[0]).rgb_Color.blue)
					|| (current_Color.green != (mask_Colors[0]).rgb_Color.green)) {
						pixel_Is_Main_Mask_Color = FALSE;
				}
			}

			
			if (pixel_Is_Main_Mask_Color != TRUE) {
				
				/*  */
				/* Destination port is already background color, check */
				/* if this needs to set a pixel to black */
				/*  */
				
				/*  */
				/* Make certain at least one pixel is bordering it and has non-background color */
				/* Check the right side */
				/*  */
				is_An_Isolated_Pixel = TRUE;
				x_Offset = current_X + source_X_Offset + 1;
				y_Offset = current_Y + source_Y_Offset;
				
				if (use_Direct_Screen_Access == TRUE){
					
					/*  */
					/* This gets the pixel in a fast manner involving no multiplication */
					/*  */
					Get_Pixel_Index(x_Offset, y_Offset, &pixel_Color);							
					if ((mask_Colors[0]).raw_Index != pixel_Color) {
						is_An_Isolated_Pixel = FALSE;
					}
				}
				else {
					GetCPixel(x_Offset, y_Offset, &current_Color);
					
					if ((current_Color.red != (mask_Colors[0]).rgb_Color.red)
						|| (current_Color.blue != (mask_Colors[0]).rgb_Color.blue)
						|| (current_Color.green != (mask_Colors[0]).rgb_Color.green)) {
						is_An_Isolated_Pixel = FALSE;
					}
				}				
				
				/*  */
				/* Make certain at least one pixel is bordering it and has non-background color */
				/* Check the left side */
				/*  */
				if (is_An_Isolated_Pixel == TRUE) {
					x_Offset = current_X + source_X_Offset - 1;
					y_Offset = current_Y + source_Y_Offset;
					GetCPixel(x_Offset, y_Offset, &current_Color);
					if ((current_Color.red != (mask_Colors[0]).rgb_Color.red)
						|| (current_Color.blue != (mask_Colors[0]).rgb_Color.blue)
						|| (current_Color.green != (mask_Colors[0]).rgb_Color.green)) {
						is_An_Isolated_Pixel = FALSE;
					}
				}
				
				/*  */
				/* Make certain at least one pixel is bordering it and has non-background color */
				/* Check the top side */
				/*  */
				if (is_An_Isolated_Pixel == TRUE) {
					x_Offset = current_X + source_X_Offset;
					y_Offset = current_Y + source_Y_Offset - 1;
					
					if (use_Direct_Screen_Access == TRUE){
						
						/*  */
						/* This gets the pixel in a fast manner involving no multiplication */
						/*  */
						Get_Pixel_Index(x_Offset, y_Offset, &pixel_Color);							
						if ((mask_Colors[0]).raw_Index != pixel_Color) {
							is_An_Isolated_Pixel = FALSE;
						}
					}
					else {
						GetCPixel(x_Offset, y_Offset, &current_Color);
						
						if ((current_Color.red != (mask_Colors[0]).rgb_Color.red)
							|| (current_Color.blue != (mask_Colors[0]).rgb_Color.blue)
							|| (current_Color.green != (mask_Colors[0]).rgb_Color.green)) {
							is_An_Isolated_Pixel = FALSE;
						}
					}				
					
				} /* if (is_An_Isolated_Pixel == TRUE) */
				
				/*  */
				/* Make certain at least one pixel is bordering it and has non-background color */
				/* Check the bottom side */
				/*  */
				if (is_An_Isolated_Pixel == TRUE) {
					x_Offset = current_X + source_X_Offset;
					y_Offset = current_Y + source_Y_Offset + 1;
					
					if (use_Direct_Screen_Access == TRUE){
						
						/*  */
						/* This gets the pixel in a fast manner involving no multiplication */
						/*  */
						Get_Pixel_Index(x_Offset, y_Offset, &pixel_Color);							
						if ((mask_Colors[0]).raw_Index != pixel_Color) {
							is_An_Isolated_Pixel = FALSE;
						}
					}
					else {
						GetCPixel(x_Offset, y_Offset, &current_Color);
						
						if ((current_Color.red != (mask_Colors[0]).rgb_Color.red)
							|| (current_Color.blue != (mask_Colors[0]).rgb_Color.blue)
							|| (current_Color.green != (mask_Colors[0]).rgb_Color.green)) {
							is_An_Isolated_Pixel = FALSE;
						}
					}				
					
					/*  */
					/* Check the Top left side */
					/*  */
					if (is_An_Isolated_Pixel == TRUE) {
						x_Offset = current_X + source_X_Offset - 1;
						y_Offset = current_Y + source_Y_Offset - 1;
						
						if (use_Direct_Screen_Access == TRUE){
							
							/*  */
							/* This gets the pixel in a fast manner involving no multiplication */
							/*  */
							Get_Pixel_Index(x_Offset, y_Offset, &pixel_Color);							
							if ((mask_Colors[0]).raw_Index != pixel_Color) {
								is_An_Isolated_Pixel = FALSE;
							}
						}
						else {
							GetCPixel(x_Offset, y_Offset, &current_Color);
							
							if ((current_Color.red != (mask_Colors[0]).rgb_Color.red)
								|| (current_Color.blue != (mask_Colors[0]).rgb_Color.blue)
								|| (current_Color.green != (mask_Colors[0]).rgb_Color.green)) {
								is_An_Isolated_Pixel = FALSE;
							}
						}				
						
						/*  */
						/* Check the Bottom left side */
						/*  */
						if (is_An_Isolated_Pixel == TRUE) {
							x_Offset = current_X + source_X_Offset - 1;
							y_Offset = current_Y + source_Y_Offset + 1;
							
							if (use_Direct_Screen_Access == TRUE){
								
								/*  */
								/* This gets the pixel in a fast manner involving no multiplication */
								/*  */
								Get_Pixel_Index(x_Offset, y_Offset, &pixel_Color);							
								if ((mask_Colors[0]).raw_Index != pixel_Color) {
									is_An_Isolated_Pixel = FALSE;
								}
							}
							else {
								GetCPixel(x_Offset, y_Offset, &current_Color);
								
								if ((current_Color.red != (mask_Colors[0]).rgb_Color.red)
									|| (current_Color.blue != (mask_Colors[0]).rgb_Color.blue)
									|| (current_Color.green != (mask_Colors[0]).rgb_Color.green)) {
									is_An_Isolated_Pixel = FALSE;
								}
							}				
							
							/*  */
							/* Check the top right side */
							/*  */
							if (is_An_Isolated_Pixel == TRUE) {
								x_Offset = current_X + source_X_Offset + 1;
								y_Offset = current_Y + source_Y_Offset - 1;
								
								if (use_Direct_Screen_Access == TRUE){
									
									/*  */
									/* This gets the pixel in a fast manner involving no multiplication */
									/*  */
									Get_Pixel_Index(x_Offset, y_Offset, &pixel_Color);							
									if ((mask_Colors[0]).raw_Index != pixel_Color) {
										is_An_Isolated_Pixel = FALSE;
									}
								}
								else {
									GetCPixel(x_Offset, y_Offset, &current_Color);
									
									if ((current_Color.red != (mask_Colors[0]).rgb_Color.red)
										|| (current_Color.blue != (mask_Colors[0]).rgb_Color.blue)
										|| (current_Color.green != (mask_Colors[0]).rgb_Color.green)) {
										is_An_Isolated_Pixel = FALSE;
									}
								}				
								
								
								/*  */
								/* Check the bottom right side */
								/*  */
								if (is_An_Isolated_Pixel == TRUE) {
									x_Offset = current_X + source_X_Offset + 1;
									y_Offset = current_Y + source_Y_Offset + 1;
									
									if (use_Direct_Screen_Access == TRUE){
										
										/*  */
										/* This gets the pixel in a fast manner involving no multiplication */
										/*  */
										Get_Pixel_Index(x_Offset, y_Offset, &pixel_Color);							
										if ((mask_Colors[0]).raw_Index != pixel_Color) {
											is_An_Isolated_Pixel = FALSE;
										}
									}
									else {
										GetCPixel(x_Offset, y_Offset, &current_Color);
										
										if ((current_Color.red != (mask_Colors[0]).rgb_Color.red)
											|| (current_Color.blue != (mask_Colors[0]).rgb_Color.blue)
											|| (current_Color.green != (mask_Colors[0]).rgb_Color.green)) {
											is_An_Isolated_Pixel = FALSE;
										}
									}				
									
								} /* if (is_An_Isolated_Pixel == TRUE) [BOTTOM RIGHT] */
								
							} /* if (is_An_Isolated_Pixel == TRUE) [TOP RIGHT] */
							
						} /* if (is_An_Isolated_Pixel == TRUE) [BOTTOM LEFT] */
							
					} /* if (is_An_Isolated_Pixel == TRUE) [TOP LEFT] */
					
				} /* if (is_An_Isolated_Pixel == TRUE) [BOTTOM] */
				
				if (is_An_Isolated_Pixel != TRUE) {
					/*  */
					/* Check if all bordering pixels are actually not one of the */
					/* background colors */
					/*  */
					x_Offset = current_X + source_X_Offset;
					y_Offset = current_Y + source_Y_Offset;
					if (Check_Pixel_Neighbors_Are_Background(x_Offset
																								, y_Offset
																								, mask_Color_Total
																								, mask_Colors
																								, use_Direct_Screen_Access) == SAFE_TRUE) {
						is_An_Isolated_Pixel = TRUE;																		
					}
				}				
				
				if (is_An_Isolated_Pixel != TRUE) {
					/*  */
					/* Regular official QD technique, pretty slow */
					/* should already have a black colored pen */
					/*  */
		/* =====
					Set_View_Port(simple_Bit_Map_Ptr);
					x_Offset = current_X + target_X_Offset;
					y_Offset = current_Y + target_Y_Offset;
					MoveTo(x_Offset, y_Offset);
					LineTo(x_Offset, y_Offset);
					Set_View_Port(offscreen_Graph_Ptr);
	====== */
					/*  */
					/* This is a much faster technique of setting a simple pixel black */
					/*  */
					x_Offset = current_X + target_X_Offset;
					current_Screen_Address = raw_Pixel_Address + (x_Offset >> 3);
					current_Screen_Address = current_Screen_Address 
											+ ((CARDINAL_32)(current_Y + target_Y_Offset) * bytes_Per_Row);
						
					pixel_Neighbors = *(CARDINAL_8 *)current_Screen_Address;
					*(CARDINAL_8 *)current_Screen_Address
														= pixel_Neighbors | (0x80 >> ((x_Offset) & 0x07));

				}
			}
			
		} /* for (current_X = 0;  ... */
	} /* for (current_Y = 0;  ... */
	
Error_Handler:

Normal_Function_Exit:
	; /* NULL */
}
/*=**************************************************************************=*/


/*=**************************************************************************=*/
Safe_Boolean	Determine_Masking_Colors(CARDINAL_32	*	mask_Color_Total
																		, Color_Type mask_Colors[]
																		, Rect				*source_Rect
																		, Safe_Boolean	indexed_Colors_Desired)
{
	Safe_Boolean		function_Result;
	RGBColor			color_1;
	RGBColor			color_2;
	RGBColor			color_3;
	RGBColor			color_4;
	RGBColor			color_1_Hold;
	RGBColor			color_2_Hold;
	RGBColor			color_3_Hold;
	RGBColor			color_4_Hold;
	INTEGER_16		red_Deviance;
	INTEGER_16		green_Deviance;
	INTEGER_16		blue_Deviance;
	RGBColor			final_Color;
	RGBColor			sampled_Extra_Color;
	INTEGER_32		final_Horizontal;
	INTEGER_32		final_Vertical;
	INTEGER_32		x_Offset;
	INTEGER_32		y_Offset;
	CARDINAL_16	total_Matches;
	INTEGER_16		current_Color;
	INTEGER_32		horizontal_Coordinate;
	INTEGER_32		vertical_Coordinate;
	GrafPtr				current_Port;
	Boolean				top_Two_Corners_Match;
	CARDINAL_8		*code_Name;
/* END VAR */


	code_Name = "\pDetermine_Masking_Colors";
	function_Result = SAFE_FALSE;
	GetPort(&current_Port);
	
	*mask_Color_Total = 0;
	current_Color = 0;
	
	(mask_Colors[current_Color]).use_Raw_Index = SAFE_FALSE;
	(mask_Colors[current_Color]).raw_Index = 0;
	
	/*  */
	/* Some grabbing cards do not treat extreme left or right too well */
	/* so for grabs eliminate the possibility of out of bounds colors used for */
	/* sample base. */
	/* WARNING : LEFT side is supposed to be at least 21, */
	/* and right is needs only 10 but is paired for symettry. */
	/* Up and down margins are less critical */
	/*  */
	
	horizontal_Coordinate = (*source_Rect).left;
	vertical_Coordinate = (*source_Rect).top;
	horizontal_Coordinate = horizontal_Coordinate + k_CROP_SAMPLE_X_INSET;
	vertical_Coordinate = vertical_Coordinate + k_CROP_SAMPLE_Y_INSET;
	GetCPixel(horizontal_Coordinate,vertical_Coordinate,&color_1);
	
	horizontal_Coordinate = (*source_Rect).right;
	horizontal_Coordinate = horizontal_Coordinate - 1;
	horizontal_Coordinate = horizontal_Coordinate - k_CROP_SAMPLE_X_INSET;
	GetCPixel(horizontal_Coordinate,vertical_Coordinate,&color_2);
	
	vertical_Coordinate = (*source_Rect).bottom;
	vertical_Coordinate = vertical_Coordinate - 1;
	vertical_Coordinate = vertical_Coordinate - k_CROP_SAMPLE_Y_INSET;
	GetCPixel(horizontal_Coordinate,vertical_Coordinate,&color_3);
	
	horizontal_Coordinate = (*source_Rect).left;
	horizontal_Coordinate = horizontal_Coordinate + k_CROP_SAMPLE_X_INSET;
	GetCPixel(horizontal_Coordinate,vertical_Coordinate,&color_4);
		

	total_Matches = 0;
	
	top_Two_Corners_Match = FALSE;

	if ((color_1.red == color_2.red)
		&& (color_1.green == color_2.green)
		&& (color_1.blue == color_2.blue)) {
		top_Two_Corners_Match = TRUE;
		total_Matches += 1;
	}
		
	if ((color_1.red == color_3.red)
		&& (color_1.green == color_3.green)
		&& (color_1.blue == color_3.blue)) {
		total_Matches += 1;
	}
		
	if ((color_1.red == color_4.red)
		&& (color_1.green == color_4.green)
		&& (color_1.blue == color_4.blue)) {
		total_Matches += 1;
	}
		
	if ((color_2.red == color_3.red)
		&& (color_2.green == color_3.green)
		&& (color_2.blue == color_3.blue)) {
		total_Matches += 1;
	}
	
	if ((color_2.red == color_4.red)
		&& (color_2.green == color_4.green)
		&& (color_2.blue == color_4.blue)) {
		total_Matches += 1;
	}
	
	if ((color_3.red == color_4.red)
		&& (color_3.green == color_4.green)
		&& (color_3.blue == color_4.blue)) {
		total_Matches += 1;
	}
	

	if ((total_Matches < 3) && (indexed_Colors_Desired != SAFE_TRUE)){
		/*  */
		/* Drop lowest 4 bits of color channel out of 8 bit  */
		/* And retry */
		/*  */
		color_1_Hold = color_1;
		color_2_Hold = color_2;
		color_3_Hold = color_3;
		color_4_Hold = color_4;
		
		color_1.red = (color_1.red & 0xFFFFF000);
		color_1.green = (color_1.green & 0xFFFFF000);
		color_1.blue = (color_1.blue & 0xFFFFF000);

		color_2.red = (color_2.red & 0xFFFFF000);
		color_2.green = (color_2.green & 0xFFFFF000);
		color_2.blue = (color_2.blue & 0xFFFFF000);

		color_3.red = (color_3.red & 0xFFFFF000);
		color_3.green = (color_3.green & 0xFFFFF000);
		color_3.blue = (color_3.blue & 0xFFFFF000);

		color_4.red = (color_4.red & 0xFFFFF000);
		color_4.green = (color_4.green & 0xFFFFF000);
		color_4.blue = (color_4.blue & 0xFFFFF000);

		total_Matches = 0;
		
		top_Two_Corners_Match = FALSE;
	
		if ((color_1.red == color_2.red)
			&& (color_1.green == color_2.green)
			&& (color_1.blue == color_2.blue)) {
			top_Two_Corners_Match = TRUE;
			total_Matches += 1;
		}
			
		if ((color_1.red == color_3.red)
			&& (color_1.green == color_3.green)
			&& (color_1.blue == color_3.blue)) {
			total_Matches += 1;
		}
			
		if ((color_1.red == color_4.red)
			&& (color_1.green == color_4.green)
			&& (color_1.blue == color_4.blue)) {
			total_Matches += 1;
		}
			
		if ((color_2.red == color_3.red)
			&& (color_2.green == color_3.green)
			&& (color_2.blue == color_3.blue)) {
			total_Matches += 1;
		}
		
		if ((color_2.red == color_4.red)
			&& (color_2.green == color_4.green)
			&& (color_2.blue == color_4.blue)) {
			total_Matches += 1;
		}
		
		if ((color_3.red == color_4.red)
			&& (color_3.green == color_4.green)
			&& (color_3.blue == color_4.blue)) {
			total_Matches += 1;
		}
		
		/*  */
		/* Restore correct colors */
		/*  */
		color_1 = color_1_Hold;
		color_2 = color_2_Hold;
		color_3 = color_3_Hold;
		color_4 = color_4_Hold;

		
		/* =================================== */
		
		if ((total_Matches < 3) && (indexed_Colors_Desired != SAFE_TRUE)){
			/*  */
			/* Drop lowest two bits of color channel out of 5 channels */
			/* And retry */
			/*  */
	
			total_Matches = 0;
			
			top_Two_Corners_Match = FALSE;
		
			/*  */
			/* Compare 1 and 2 */
			/*  */
			if (color_1.red > color_2.red) {
				red_Deviance = color_1.red - color_2.red;
			}
			else {
				red_Deviance = color_2.red - color_1.red;
			}
			
			if (color_1.green > color_2.green) {
				green_Deviance = color_1.green - color_2.green;
			}
			else {
				green_Deviance = color_2.green - color_1.green;
			}
			
			if (color_1.blue > color_2.blue) {
				blue_Deviance = color_1.blue - color_2.blue;
			}
			else {
				blue_Deviance = color_2.blue - color_1.blue;
			}
			
			if ((red_Deviance <= k_COLOR_DEVIANCE)
				&& (green_Deviance <= k_COLOR_DEVIANCE)
				&& (blue_Deviance <= k_COLOR_DEVIANCE)) {
				top_Two_Corners_Match = TRUE;
				total_Matches += 1;
			}
					
			/*  */
			/* Compare 1 and 3 */
			/*  */
			if (color_1.red > color_3.red) {
				red_Deviance = color_1.red - color_3.red;
			}
			else {
				red_Deviance = color_3.red - color_1.red;
			}
			
			if (color_1.green > color_3.green) {
				green_Deviance = color_1.green - color_3.green;
			}
			else {
				green_Deviance = color_3.green - color_1.green;
			}
			
			if (color_1.blue > color_3.blue) {
				blue_Deviance = color_1.blue - color_3.blue;
			}
			else {
				blue_Deviance = color_3.blue - color_1.blue;
			}
			
			if ((red_Deviance <= k_COLOR_DEVIANCE)
				&& (green_Deviance <= k_COLOR_DEVIANCE)
				&& (blue_Deviance <= k_COLOR_DEVIANCE)) {
				total_Matches += 1;
			}
		
			/*  */
			/* Compare 1 and 4 */
			/*  */
			if (color_1.red > color_4.red) {
				red_Deviance = color_1.red - color_4.red;
			}
			else {
				red_Deviance = color_4.red - color_1.red;
			}
			
			if (color_1.green > color_4.green) {
				green_Deviance = color_1.green - color_4.green;
			}
			else {
				green_Deviance = color_4.green - color_1.green;
			}
			
			if (color_1.blue > color_4.blue) {
				blue_Deviance = color_1.blue - color_4.blue;
			}
			else {
				blue_Deviance = color_4.blue - color_1.blue;
			}
			
			if ((red_Deviance <= k_COLOR_DEVIANCE)
				&& (green_Deviance <= k_COLOR_DEVIANCE)
				&& (blue_Deviance <= k_COLOR_DEVIANCE)) {
				total_Matches += 1;
			}
		
			/*  */
			/* Compare 2 and 3 */
			/*  */
			if (color_2.red > color_3.red) {
				red_Deviance = color_2.red - color_3.red;
			}
			else {
				red_Deviance = color_3.red - color_2.red;
			}
			
			if (color_2.green > color_3.green) {
				green_Deviance = color_2.green - color_3.green;
			}
			else {
				green_Deviance = color_3.green - color_2.green;
			}
			
			if (color_2.blue > color_3.blue) {
				blue_Deviance = color_2.blue - color_3.blue;
			}
			else {
				blue_Deviance = color_3.blue - color_2.blue;
			}
			
			if ((red_Deviance <= k_COLOR_DEVIANCE)
				&& (green_Deviance <= k_COLOR_DEVIANCE)
				&& (blue_Deviance <= k_COLOR_DEVIANCE)) {
				total_Matches += 1;
			}
		
			/*  */
			/* Compare 2 and 4 */
			/*  */
			if (color_2.red > color_4.red) {
				red_Deviance = color_2.red - color_4.red;
			}
			else {
				red_Deviance = color_4.red - color_2.red;
			}
			
			if (color_2.green > color_4.green) {
				green_Deviance = color_2.green - color_4.green;
			}
			else {
				green_Deviance = color_4.green - color_2.green;
			}
			
			if (color_2.blue > color_4.blue) {
				blue_Deviance = color_2.blue - color_4.blue;
			}
			else {
				blue_Deviance = color_4.blue - color_2.blue;
			}
			
			if ((red_Deviance <= k_COLOR_DEVIANCE)
				&& (green_Deviance <= k_COLOR_DEVIANCE)
				&& (blue_Deviance <= k_COLOR_DEVIANCE)) {
				total_Matches += 1;
			}
		
			/*  */
			/* Compare 3 and 4 */
			/*  */
			if (color_3.red > color_4.red) {
				red_Deviance = color_3.red - color_4.red;
			}
			else {
				red_Deviance = color_4.red - color_3.red;
			}
			
			if (color_3.green > color_4.green) {
				green_Deviance = color_3.green - color_4.green;
			}
			else {
				green_Deviance = color_4.green - color_3.green;
			}
			
			if (color_3.blue > color_4.blue) {
				blue_Deviance = color_3.blue - color_4.blue;
			}
			else {
				blue_Deviance = color_4.blue - color_3.blue;
			}
			
			if ((red_Deviance <= k_COLOR_DEVIANCE)
				&& (green_Deviance <= k_COLOR_DEVIANCE)
				&& (blue_Deviance <= k_COLOR_DEVIANCE)) {
				total_Matches += 1;
			}
			
			
		} /* if (total_Matches < 3)  */
		
		/* =================================== */
		
	} /* if (total_Matches < 3)  */
	
	if (total_Matches < 3) {
		if (indexed_Colors_Desired == SAFE_TRUE) {
			goto No_Matches_Error;
		}
		else {
			if ((total_Matches == 2) && (top_Two_Corners_Match == TRUE)) {
				/*  */
				/* GOOD ENOUGH FOR US! */
				/* MIGHT NOT BE PERFECT BUT ITS SOMETHING TO CHEW ON */
				/*  */
				
			}
			else {
				goto No_Matches_Error;
			}
		}
	}

	if (top_Two_Corners_Match == TRUE) {
		(mask_Colors[current_Color]).rgb_Color = color_1;
	}
	else {
		(mask_Colors[current_Color]).rgb_Color = color_3;
	}
	
	if (indexed_Colors_Desired == SAFE_TRUE) {
		(mask_Colors[current_Color]).raw_Index = Get_Index_For_Color(&((mask_Colors[current_Color]).rgb_Color));
		(mask_Colors[current_Color]).use_Raw_Index = SAFE_TRUE;
	}
	else {
		(mask_Colors[current_Color]).use_Raw_Index = SAFE_FALSE;
		(mask_Colors[current_Color]).raw_Index = 0;
	}
	
	/*  */
	/* We are done, we found one suspected background color. */
	/* Now we scan around one of the paired matching corners */
	/* and introduce more background colors into our list. */
	/* We will ask the user if the extra colors are OK to add, later on */
	/*  */
	
	final_Color = (mask_Colors[current_Color]).rgb_Color;
	
	if (top_Two_Corners_Match == TRUE) {
		final_Horizontal = (*source_Rect).left;
		final_Vertical = (*source_Rect).top;
	}
	else {
		final_Horizontal = (*source_Rect).right - 1;
		final_Vertical = (*source_Rect).bottom -1;
		/*  */
		/* We will be scanning 16 pixel area (duplicating current match) */
		/* For simplicity the current position will be in the UPPER LEFT */
		/* of this 4*4 block so position the current point to that location */
		/*  */
		final_Horizontal = final_Horizontal - k_PIXELS_TO_SCAN_FOR_BACKGROUND_COLORS;
		final_Vertical = final_Horizontal - k_PIXELS_TO_SCAN_FOR_BACKGROUND_COLORS;
	}
	
	/*  */
	/* Scan grid looking for new colors to add to background color list */
	/*  */
	if (indexed_Colors_Desired == SAFE_TRUE) {
		for(y_Offset=0; (y_Offset < k_PIXELS_TO_SCAN_FOR_BACKGROUND_COLORS); y_Offset++ ) {
			for(x_Offset=0; (x_Offset < k_PIXELS_TO_SCAN_FOR_BACKGROUND_COLORS); x_Offset++ ) {
				if ((current_Color + 1) < k_MAXIMUM_BACKGROUND_COLORS) {
					GetCPixel(final_Horizontal + x_Offset, final_Vertical + y_Offset, &sampled_Extra_Color);
					if ((sampled_Extra_Color.red != final_Color.red)
									|| (sampled_Extra_Color.green != final_Color.green)
									|| (sampled_Extra_Color.blue != final_Color.blue)) {
						
						current_Color += 1;
						(mask_Colors[current_Color]).rgb_Color = sampled_Extra_Color;
						(mask_Colors[current_Color]).raw_Index = Get_Index_For_Color(&((mask_Colors[current_Color]).rgb_Color));
						(mask_Colors[current_Color]).use_Raw_Index = SAFE_FALSE;
					}
				}
			}
		}
	} /* if (indexed_Colors_Desired == SAFE_TRUE) */
	
	*mask_Color_Total = current_Color + 1;
	
	function_Result = SAFE_TRUE;
	goto Normal_Function_Exit;
		
No_Matches_Error:

Error_Handler:
	function_Result = SAFE_FALSE;

Normal_Function_Exit:

	return(function_Result);
	
} /* Safe_Boolean	Determine_Masking_Colors ... */
/*=**************************************************************************=*/


/*=**************************************************************************=*/																	
Safe_Boolean	Create_Region_For_Indexed_Port_Image(GrafPtr offscreen_Source_Port
																	, Rect 									*source_Rect
																	, Safe_Boolean		 			create_Color_Style_Port
																	, GrafPort 							*offscreen_Graph_Port
																	, Off_Screen_Ptr 					modern_Offscreen_Graph_Port)
{
	Safe_Boolean		function_Result;
	Safe_Boolean		port_Was_Set;
	GrafPtr				saved_Original_Port;
	PenState				saved_Pen_State;

	CARDINAL_32	mask_Color_Total;
	
	CARDINAL_8		*code_Name;
	OSErr					os_Error;
	Color_Type			mask_Color[k_MAXIMUM_BACKGROUND_COLORS];
	Color_Type			*mask_Color_Ptr;
/* END VAR */


	code_Name = "\pCreate_Region_For_Indexed_Port_Image";
	function_Result = SAFE_FALSE;
		
	GetPort(&saved_Original_Port);
	Set_View_Port(offscreen_Source_Port);
	port_Was_Set = SAFE_TRUE;
	GetPenState (&saved_Pen_State);

	/*  */
	/* Actually needed for CopyBits calls , so be certain. */
	/*  */
	ForeColor(blackColor);
	BackColor(whiteColor);


	mask_Color_Ptr = &(mask_Color[0]);
	mask_Color_Total = 0;
	
	function_Result = Determine_Masking_Colors(
																		&mask_Color_Total
																		, mask_Color_Ptr
																		, source_Rect
																		, SAFE_TRUE);
	if (function_Result != SAFE_TRUE) {
		Perform_OK_Dialog("\pProblem encountered : No identifiable background color. (over 6 colors within 8 pixels of corners?, background cropped from PICT?)");
		goto Error_Handler;
	}
		
	SetPort(saved_Original_Port);
	
	function_Result = Copy_Rect_Non_Mask_Into_BW_Bit_Map(
																		source_Rect
																		, create_Color_Style_Port
																		, offscreen_Graph_Port
																		, &(modern_Offscreen_Graph_Port->offscreen_Port.color_Port)
																		, SAFE_TRUE
																		, mask_Color_Total
																		, mask_Color_Ptr);
	if (function_Result != SAFE_TRUE) {
		goto Error_Handler;
	}
		
	function_Result = SAFE_TRUE;
	goto Normal_Function_Exit;
	
Error_Handler:
	if (port_Was_Set == SAFE_TRUE) {
		Set_View_Port(offscreen_Source_Port);
		SetPenState (&saved_Pen_State);
		Set_View_Port(saved_Original_Port);
		port_Was_Set = SAFE_FALSE;
	}

	Display_Serious_Error_Message(unit_Name, code_Name, "\pProblem encountered", "\p");
	function_Result = SAFE_FALSE;
		
Normal_Function_Exit:

	if (port_Was_Set == SAFE_TRUE) {
		Set_View_Port(offscreen_Source_Port);
		SetPenState (&saved_Pen_State);
		Set_View_Port(saved_Original_Port);
	}

	return(function_Result);
		
} /* Safe_Boolean	Create_Region_For_Indexed_Port_Image... */
/*=**************************************************************************=*/


/*=**************************************************************************=*/																	
Safe_Boolean	Create_Region_For_Port_Image(GrafPtr offscreen_Source_Port
																	, Rect *source_Rect
																	, RgnHandle *mask)
{
	Safe_Boolean		function_Result;
	CARDINAL_32	stored_Bit_Depth;
	
	Safe_Boolean					create_Color_Style_Port;
	Handle							image_Handle;
	GrafPort 							offscreen_Graph_Port;
	Off_Screen_Port_Rec		modern_Offscreen_Graph_Port;
	OSErr								os_Error;
	Safe_Boolean					port_Was_Set;
	
	GrafPtr							saved_Original_Port;
	PenState							saved_Pen_State;
	CARDINAL_8					*code_Name;
/* END VAR */

	
	create_Color_Style_Port = SAFE_FALSE;
	code_Name = "\pCreate_Region_For_Port_Image";

	*mask = nil;

	GetPort(&saved_Original_Port);
	Set_View_Port(offscreen_Source_Port);
	GetPenState (&saved_Pen_State);
	port_Was_Set = SAFE_TRUE;
	
	function_Result = Create_Simple_BW_Bit_Map(source_Rect
																	, &image_Handle
																	, create_Color_Style_Port
																	, &offscreen_Graph_Port
																	, &modern_Offscreen_Graph_Port);
	
	if (function_Result != SAFE_TRUE) {
		Perform_OK_Dialog("\pProblem encountered : The simple mask background was not created (low memory?), import failing");
		goto Error_Handler;
	}
		
	stored_Bit_Depth = (**(((CGrafPtr)offscreen_Source_Port)->portPixMap)).pixelSize;
	if (stored_Bit_Depth > 8) {
		function_Result = Create_Region_For_Direct_Port_Image(offscreen_Source_Port
																										, source_Rect
																										, create_Color_Style_Port
																										, (&offscreen_Graph_Port)
																										, &modern_Offscreen_Graph_Port);
	}
	else{ 
		function_Result = Create_Region_For_Indexed_Port_Image(offscreen_Source_Port
																										, source_Rect
																										, create_Color_Style_Port
																										, (&offscreen_Graph_Port)
																										, &modern_Offscreen_Graph_Port);
	}

	/*  */
	/* Paranoia safety redundancy */
	/*  */
	if (create_Color_Style_Port == SAFE_TRUE) {
		Set_View_Port((GrafPtr)&(modern_Offscreen_Graph_Port.offscreen_Port.color_Port));
	}
	else {
		Set_View_Port((&offscreen_Graph_Port));
	}
	
	/*  */
	/* Convert it to a region by casting it to a BitMap*/
	/*  */
	*mask = NewRgn();
		
/*  */
/* Just set to entire region  for TEST */
/*  */
/* ====
	SetRectRgn(*mask
					, source_Rect->left
					, source_Rect->top
					, source_Rect->right
					, source_Rect->bottom);
					
	os_Error = noErr;
=== */
	
	if (create_Color_Style_Port == SAFE_TRUE) {
		os_Error = BitMapToRegion(*mask, (BitMap *)(*(modern_Offscreen_Graph_Port.offscreen_Port.color_Port.portPixMap)));
	}
	else {
		os_Error = BitMapToRegion(*mask, &(offscreen_Graph_Port.portBits));
	}

	if (os_Error != noErr) {
		if (os_Error == rgnTooBigErr) {
			if (port_Was_Set == SAFE_TRUE) {
				SetPenState (&saved_Pen_State);
				Set_View_Port(saved_Original_Port);
				port_Was_Set = SAFE_FALSE;
			}
			Perform_OK_Dialog("\pRegion too complex for this type of program to use (> 64K) Too many holes");
			goto Error_Handler;
		}
		else {
			goto Error_Handler;
		}
	}
		
	if (Destroy_Simple_BW_Bit_Map(create_Color_Style_Port
															, &image_Handle
															, &offscreen_Graph_Port
															, &modern_Offscreen_Graph_Port) != SAFE_TRUE) {
		/*  */
		/* Ignore err, exiting soon anyway */
		/*  */
	}

	function_Result = SAFE_TRUE;
	goto Normal_Function_Exit;
	
Error_Handler:
	Display_Serious_Error_Message(unit_Name, code_Name, "\pProblem encountered", "\p");
	function_Result = SAFE_FALSE;
		
Normal_Function_Exit:
	ShowCursor();

	if (port_Was_Set == SAFE_TRUE) {
		SetPenState (&saved_Pen_State);
		Set_View_Port(saved_Original_Port);
	}
	return(function_Result);
	
} /* Safe_Boolean	Create_Region_For_Port_Image ... */
/*=**************************************************************************=*/


/*=**************************************************************************=*/																	
Safe_Boolean	Create_Bit_Mask_For_Port_Image(GrafPtr offscreen_Source_Port
																	, Rect *source_Rect
																	, GrafPort *offscreen_Graph_Port
																	, Handle *image_Handle)
{
	Safe_Boolean		function_Result;
	CARDINAL_32	stored_Bit_Depth;
	
	OSErr								os_Error;
	Safe_Boolean					port_Was_Set;
	Safe_Boolean					create_Color_Style_Port;
	GrafPtr							saved_Original_Port;
	PenState							saved_Pen_State;
	CARDINAL_8					*code_Name;
/* END VAR */

	
	create_Color_Style_Port = SAFE_FALSE;
	code_Name = "\pCreate_Region_For_Port_Image";

	GetPort(&saved_Original_Port);
	Set_View_Port(offscreen_Source_Port);
	GetPenState (&saved_Pen_State);
	port_Was_Set = SAFE_TRUE;
	
	function_Result = Create_Simple_BW_Bit_Map(source_Rect
																	, image_Handle
																	, create_Color_Style_Port
																	, offscreen_Graph_Port
																	, nil);
	
	if (function_Result != SAFE_TRUE) {
		Perform_OK_Dialog("\pProblem encountered : The simple mask background was not created (low memory?), import failing");
		goto Error_Handler;
	}
		
	stored_Bit_Depth = (**(((CGrafPtr)offscreen_Source_Port)->portPixMap)).pixelSize;
	if (stored_Bit_Depth > 8) {
		function_Result = Create_Region_For_Direct_Port_Image(offscreen_Source_Port
																										, source_Rect
																										, create_Color_Style_Port
																										, (offscreen_Graph_Port)
																										, nil);
	}
	else{ 
		function_Result = Create_Region_For_Indexed_Port_Image(offscreen_Source_Port
																										, source_Rect
																										, create_Color_Style_Port
																										, (offscreen_Graph_Port)
																										, nil);
	}


	function_Result = SAFE_TRUE;
	goto Normal_Function_Exit;
	
Error_Handler:
	Display_Serious_Error_Message(unit_Name, code_Name, "\pProblem encountered", "\p");
	function_Result = SAFE_FALSE;
		
Normal_Function_Exit:
	ShowCursor();

	if (port_Was_Set == SAFE_TRUE) {
		SetPenState (&saved_Pen_State);
		Set_View_Port(saved_Original_Port);
	}
	return(function_Result);
	
} /* Safe_Boolean	Create_Bit_Mask_For_Port_Image ... */
/*=**************************************************************************=*/


/*=**************************************************************************=*/
/* END OF FILE: Indexed_Background_Removal.c */
