/* START OF FILE: Off_Screen_Color.c */


/*=**************************************************************************=*/
/* [---BEGIN---] FILE HEADER COMMENTS */
/**
 ** SCREEN VIEW = [Adobe Syntax Black-10, Tabs = 4]
 ** PRINTER VIEW = [Adobe Syntax Black-10, Tabs = 4]
 ** =======================================================================
 ** Written by James Merkle
 ** Off_Screen_Color.c - Routines to support offscreen QuickDraw
 ** (C) Copyright 1993 by James Merkle
 ** All rights reserved, except those wishing to "copy" moderate sections of code or files may do so, so long as
 ** such code is never used directly either in animation-related tools or multimedia tools that are  
 ** used in live performance or live broadcast or recorded mass market broadcast or film. Desktop usage not restricted.
 ** Conventional desktop game or entertainment not restricted.
 ** =======================================================================
 **
 **
 ** ============
 ** Revision History
 ** ============
 **
 **  1993.01.07		(James Merkle)
 **						Initial coding started, much taken from my own existing source
 **
 **  1993.08.23		(James Merkle)
 **							BUG FIX!!!! was off by one row for height!
 **
 **
 **
 ** =================
 ** OTHER INFORMATION
 ** =================
 ** 
 ** These are the general offscreen color routines created from Apple code.
 **
 **
 **/
/* [----END----] FILE HEADER COMMENTS */
/*=**************************************************************************=*/


/*=**************************************************************************=*/
/* [---BEGIN---] #include AREA */


#include <GestaltEqu.h>
#include <Palettes.h>


/* ==
#include "Core_Universal.h" 
#include "Core_Specific.h"
#include "SysHooks.h"
=== */

#include "Shell_Globals.h"
#include "Buffered_File_Routines.h"
#include "Interface_Utilities.h"
#include "Shell_Dialogs.h"
#include "Memory_Utilities.h"
#include "String_Utilities.h"
#include "Picture_Spooling.h"

#include "Interface_Utilities.h"
#include "Error_Utilities.h"

#include "Off_Screen_Color.h"


/* [----END----] #include AREA */
/*=**************************************************************************=*/


/*=**************************************************************************=*/
/* [---BEGIN---] #define AREA */

/*  */
/* Maximum number of bytes in a row of pixels */
/*  */
#define k_MAXIMUM_ROW_BYTES		0x3FFE 

/*  */
 /* Default resolution is 72 DPI; Fixed type */
/*  */
#define k_DEFAULT_RESOULUTION				0x00480000

/*  */
 /* Inverse-table resolution */
 /*  */
#define k_INVERSE_TABLE_RESOULUTION		4

 /*  */
 /* Number of bits per pixel in off-screen environment */
 /*  */
#define k_TEST_OFFSCREEN_DEPTH  	8

 /*  */
 /* Resource ID of gray-scale clut */
 /*  */
#define k_GRAY_CLUT_ID  1600 

 /*  */
  /* Resource ID of full-color clut */
 /*  */
#define k_COLOR_CLUT_ID 1601

#define k_PICT_FILE_DRAWING_BUFFER_SIZE		(40480L)

#define COLOR_PORT_PORTBITS_CHECK 		0xC000

#define k_DIRECT_16_BIT_COLOR		15
#define k_DIRECT_32_BIT_COLOR		24
#define k_DIRECT_48_BIT_COLOR		48

#define k_MATCH_COLOR_SEED_FLAG		0x4000

#define topLeft(r)		(((Point *) &(r))[0])
#define botRight(r)		(((Point *) &(r))[1])


/* [----END----] #define AREA */
/*=**************************************************************************=*/


/*=**************************************************************************=*/
/* [---BEGIN---] TYPE AREA */

	
/* [----END----] TYPE AREA */
/*=**************************************************************************=*/


/*=**************************************************************************=*/
/* [---BEGIN---] PROTOTYPE AREA */

void			Test_Off_Screen_Routines(void);
													

/* [----END----] PROTOTYPE AREA */
/*=**************************************************************************=*/


/*=**************************************************************************=*/
/* [---BEGIN---] GLOBAL DEFINITION AREA */


/* [----END----] GLOBAL DEFINITION AREA */
/*=**************************************************************************=*/


/*=**************************************************************************=*/
/* [---BEGIN---] LOCAL STATIC DEFINITION AREA */

static		 CARDINAL_8			*unit_Name = "\pOff_Screen_Color";

/* [----END----] LOCAL STATIC DEFINITION AREA */
/*=**************************************************************************=*/


/*=**************************************************************************=*/
void Determine_Bit_Depth_Of_Device_Containing_Rect(Rect *global_Rect
													, CARDINAL_16  *total_Intersected_Screens
													, INTEGER_32	 *bits_Per_Pixel
													, Safe_Boolean	*rect_Is_Within_A_Screen)
 	/*=--------------------------------------------------------------------------=*/
 	/* "Determine_Bit_Depth_Of_Device_Containing_Rect" is used */
 	/* when calling "Determine_Information_Of_Device_Containing_Rect" returns */
 	/* more information than desired. */
 	/* returns total number of intersected video devices (if any) */
 	/* and returns the smallest bit depth of the screens that do intersect */
 	/* the "global_Rect" (passed in local coordinates WHEN PORT IS ACTIVE)*/
 	/* Also, for video purposes, indicates if rect is found completely within a screen */
 	/*=--------------------------------------------------------------------------=*/
{
	INTEGER_16		video_Monitor_Driver_Ref;
	INTEGER_8			video_Monitor_ID;
	INTEGER_8			video_Monitor_Slot;
	CARDINAL_32	video_Optional_Base_Address;
	PixMapHandle	video_Pix_Map_Handle;
	INTEGER_32		bits_Per_Pixel_Greatest;
	Safe_Boolean		slot_Information_Valid;													
	GDHandle			graphics_Device;											
/* END VAR */


	Determine_Information_Of_Device_Containing_Rect(global_Rect
													, &video_Monitor_Driver_Ref
													, &video_Monitor_Slot
													, &video_Monitor_ID
													, &video_Optional_Base_Address
													, bits_Per_Pixel
													, &bits_Per_Pixel_Greatest
													, total_Intersected_Screens
													, rect_Is_Within_A_Screen
													, &video_Pix_Map_Handle
													, &graphics_Device
													, &slot_Information_Valid);
													
													
} /* void Determine_Bit_Depth_Of_Device_Containing_Rect(Rect *global_Rect ... */
/*=**************************************************************************=*/


/*=**************************************************************************=*/
Safe_Boolean Fetch_Extra_Video_Driver_Info(INTEGER_16 video_Monitor_Driver_Ref
																, CARDINAL_32 *video_Optional_Base_Address
																, INTEGER_8 *video_Monitor_Slot
																, INTEGER_8 *video_Monitor_ID)
{
	OSErr						current_Error;
	INTEGER_32			handle_Size;
	Safe_Boolean			function_Result, slot_Data_Located;
	DCtlHandle				device_Control_Entry_Hdl;
/* END VAR */	


	slot_Data_Located = SAFE_FALSE;
	function_Result = SAFE_FALSE;
/*  */
/* Simple paranoia inittialization of extra parameters.*/
/*  */
	*video_Optional_Base_Address = (CARDINAL_32)nil;
	*video_Monitor_Slot = -1;
	*video_Monitor_ID = -1;

	device_Control_Entry_Hdl = nil;
	device_Control_Entry_Hdl = GetDCtlEntry(video_Monitor_Driver_Ref);
	
	/*  */
	/* Check if handle is legitimately pointing to what we need */
	/*  */
	if (device_Control_Entry_Hdl != nil) {
		if (video_Monitor_Driver_Ref == (*device_Control_Entry_Hdl)->dCtlRefNum) {
					
			handle_Size = GetHandleSize((Handle)device_Control_Entry_Hdl);
			current_Error = MemError();
			if (current_Error != noErr) {
				goto Error_Handler;
			}

			if (handle_Size >= sizeof(AuxDCE)) {
				*video_Monitor_ID = (*((AuxDCEHandle)device_Control_Entry_Hdl))->dCtlSlotId;
				*video_Monitor_Slot = (*((AuxDCEHandle)device_Control_Entry_Hdl))->dCtlSlot;
				*video_Optional_Base_Address = (*((AuxDCEHandle)device_Control_Entry_Hdl))->dCtlDevBase;
				slot_Data_Located = SAFE_TRUE;
			}
		}
		
	} /* if (device_Control_Entry_Hdl != nil) */


	if (slot_Data_Located == SAFE_TRUE) {
		function_Result = SAFE_TRUE;
	}
	else {
		function_Result = SAFE_FALSE;
	}

	goto Normal_Function_Exit;
	
Error_Handler:
	function_Result = SAFE_FALSE;

Normal_Function_Exit:
	return(function_Result);
	
} /* Safe_Boolean Fetch_Extra_Video_Driver_Info(*video_Monitor_Driver_Ref ... */
/*=**************************************************************************=*/


/*=**************************************************************************=*/
CARDINAL_32 Get_Bit_Depth_Of_Primary_Device(void)
{
	GDHandle			primary_Device;
	CARDINAL_32	bits_Per_Pixel;
/* END VAR */


	bits_Per_Pixel = 1;
	if (ns_Shell_Globals.system_Environment.hasColorQD == TRUE) {
		primary_Device = nil;
		primary_Device = GetMainDevice();
		bits_Per_Pixel = (**(**primary_Device).gdPMap).pixelSize;
	}
	
	return(bits_Per_Pixel);

} /* INTEGER_32 Get_Bit_Depth_Of_Primary_Device(void) */
/*=**************************************************************************=*/


/*=**************************************************************************=*/
void Find_A_8_Bit_Non_System_Screen(Rect *global_Rect_Location
													, INTEGER_16		*video_Monitor_Driver_Ref
													, INTEGER_8		*video_Monitor_Slot
													, INTEGER_8		*video_Monitor_ID
													, CARDINAL_32	*video_Optional_Base_Address
													, PixMapHandle	*video_Pix_Map_Handle
													, GDHandle			*video_Graphics_Device_Handle
													, INTEGER_32		*bits_Per_Pixel
													, Safe_Boolean	*slot_Information_Valid
													, Safe_Boolean	*other_Screens_Found_Not_8_Bit
													, Safe_Boolean	*something_Found)
 	/*=--------------------------------------------------------------------------=*/
 	/* returns global rect of first non system device with 8 bit selected */
 	/*=--------------------------------------------------------------------------=*/
{
	Rect						screen_Rect;
	INTEGER_16		screen_Is_Color;
	INTEGER_16		screen_Is_Main;
	INTEGER_16		screen_Is_Video;
	INTEGER_32		screen_Depth;
	INTEGER_16		screen_Is_Active;
	INTEGER_16		count;
	GDHandle			gDevHand;
	GrafPtr				window_Manager_Port;
	Boolean				non_System_Found_Yet;
/* END VAR */
	

	*slot_Information_Valid	 =  SAFE_FALSE;
	*other_Screens_Found_Not_8_Bit = SAFE_FALSE;
	*video_Pix_Map_Handle = nil;
	*video_Graphics_Device_Handle = nil;
	non_System_Found_Yet = FALSE;
	*something_Found = SAFE_FALSE;

/*  */
/* Simple paranoia initialization of extra parameters.*/
/*  */
	*video_Optional_Base_Address = (CARDINAL_32)nil;
	*video_Monitor_Driver_Ref = 0;
	*video_Monitor_Slot = -1;
	*video_Monitor_ID = -1;

/* ===
	global_Rect = *local_Rect;
	LocalToGlobal(&TOP_LEFT(global_Rect));
	LocalToGlobal(&BOTTOM_RIGHT(global_Rect));
=== */
	
	if (ns_Shell_Globals.system_Environment.hasColorQD != TRUE) {
		goto Black_And_White_Assumed;
	}
	
	gDevHand = nil;
	count = 0;
			
	gDevHand = GetDeviceList();
	if (gDevHand == nil)
		goto Black_And_White_Assumed;

	do {
	
		screen_Is_Video = (INTEGER_16)TestDeviceAttribute(gDevHand, screenDevice);
		screen_Is_Active = (INTEGER_16)TestDeviceAttribute(gDevHand, screenActive);

		if ((screen_Is_Active != 0) && (screen_Is_Video != 0)) { 

				screen_Is_Main = (INTEGER_16)TestDeviceAttribute(gDevHand, mainScreen);
				
			/* ===
				screen_Is_Color = (INTEGER_16)TestDeviceAttribute(gDevHand, gdDevType);
			==== */
	
					
					screen_Depth = (*(*gDevHand)->gdPMap)->pixelSize; /* Bits per Pixel */
				
				
					if (screen_Depth == 8) {
											
						if (screen_Is_Main == 0) {
							/*  */
							/* Done! We found it ! */
							/*  */
							non_System_Found_Yet = TRUE;
							
							/*  */
							/* Only count screens without titlebars */
							/*  */
							*something_Found = SAFE_TRUE;
						}

						*bits_Per_Pixel = screen_Depth;
						
						screen_Rect = (*(*gDevHand)->gdPMap)->bounds;
						*global_Rect_Location = screen_Rect;
											
						*video_Monitor_Driver_Ref = (*gDevHand)->gdRefNum;
						*video_Pix_Map_Handle = (*gDevHand)->gdPMap;
						*video_Graphics_Device_Handle = gDevHand;
						
						if ( Fetch_Extra_Video_Driver_Info(*video_Monitor_Driver_Ref
																, video_Optional_Base_Address
																, video_Monitor_Slot
																, video_Monitor_ID) == SAFE_TRUE) {
							*slot_Information_Valid	 =  SAFE_TRUE;
						}
						else {
							*slot_Information_Valid	 =  SAFE_FALSE;
						}
					
					} /* if (screen_Depth == 8) */
					else {
						if (screen_Is_Main == 0) {
							*other_Screens_Found_Not_8_Bit = SAFE_TRUE;
						}
					}
									
			} /*if ((screen_Is_Active != 0) && (screen_Is_Video != 0)) */
		
		count += 1;
		
		gDevHand = GetNextDevice(gDevHand);
	
	} while ((gDevHand != nil) && (non_System_Found_Yet != TRUE));
	
		
	goto Function_Exit;
	
Black_And_White_Assumed:

Function_Exit:
	; /*NULL */

} /*void Find_A_8_Bit_Non_System_Screen(Rect *global_Rect ... */
/*=**************************************************************************=*/


/*=**************************************************************************=*/
void Get_System_Screen_Data(Rect *global_Rect_Location
													, INTEGER_16		*video_Monitor_Driver_Ref
													, INTEGER_8		*video_Monitor_Slot
													, INTEGER_8		*video_Monitor_ID
													, CARDINAL_32	*video_Optional_Base_Address
													, PixMapHandle	*video_Pix_Map_Handle
													, GDHandle			*video_Graphics_Device_Handle
													, INTEGER_32		*bits_Per_Pixel
													, Safe_Boolean	*slot_Information_Valid
													, Safe_Boolean	*something_Found)
 	/*=--------------------------------------------------------------------------=*/
 	/* returns global rect of first non system device with 8 bit selected */
 	/*=--------------------------------------------------------------------------=*/
{
	Rect						screen_Rect;
	INTEGER_16		screen_Is_Color;
	INTEGER_16		screen_Is_Main;
	INTEGER_16		screen_Is_Video;
	INTEGER_16		screen_Is_Active;
	INTEGER_16		count;
	GDHandle			gDevHand;
	GrafPtr				window_Manager_Port;
	Boolean				found_System_Screen;
/* END VAR */
	

	*slot_Information_Valid	 =  SAFE_FALSE;
	*video_Pix_Map_Handle = nil;
	*video_Graphics_Device_Handle = nil;
	found_System_Screen = FALSE;
	*something_Found = SAFE_FALSE;

/*  */
/* Simple paranoia initialization of extra parameters.*/
/*  */
	*video_Optional_Base_Address = (CARDINAL_32)nil;
	*video_Monitor_Driver_Ref = 0;
	*video_Monitor_Slot = -1;
	*video_Monitor_ID = -1;
	
	if (ns_Shell_Globals.system_Environment.hasColorQD != TRUE) {
		goto Black_And_White_Assumed;
	}
	
	gDevHand = nil;
	count = 0;
			
	gDevHand = GetDeviceList();
	if (gDevHand == nil)
		goto Black_And_White_Assumed;

	do {
	
		screen_Is_Video = (INTEGER_16)TestDeviceAttribute(gDevHand, screenDevice);
		screen_Is_Active = (INTEGER_16)TestDeviceAttribute(gDevHand, screenActive);

		if ((screen_Is_Active != 0) && (screen_Is_Video != 0)) { 

			screen_Is_Main = (INTEGER_16)TestDeviceAttribute(gDevHand, mainScreen);
			
		/* ===
			screen_Is_Color = (INTEGER_16)TestDeviceAttribute(gDevHand, gdDevType);
		==== */
													
			if (screen_Is_Main != 0) {
				/*  */
				/* Done! We found it ! */
				/*  */
				found_System_Screen = TRUE;
				
				*bits_Per_Pixel = (*(*gDevHand)->gdPMap)->pixelSize; /* Bits per Pixel */
				
				screen_Rect = (*(*gDevHand)->gdPMap)->bounds;
				*global_Rect_Location = screen_Rect;
									
				*video_Monitor_Driver_Ref = (*gDevHand)->gdRefNum;
				*video_Pix_Map_Handle = (*gDevHand)->gdPMap;
				*video_Graphics_Device_Handle = gDevHand;
				
				if ( Fetch_Extra_Video_Driver_Info(*video_Monitor_Driver_Ref
														, video_Optional_Base_Address
														, video_Monitor_Slot
														, video_Monitor_ID) == SAFE_TRUE) {
					*slot_Information_Valid	 =  SAFE_TRUE;
				}
				else {
					*slot_Information_Valid	 =  SAFE_FALSE;
				}
				
				/*  */
				/* Only count screens without titlebars */
				/*  */
				*something_Found = SAFE_TRUE;
			}
									
		} /*if ((screen_Is_Active != 0) && (screen_Is_Video != 0)) */
		
		count += 1;
		
		gDevHand = GetNextDevice(gDevHand);
	
	} while ((gDevHand != nil) && (found_System_Screen != TRUE));
	
		
	goto Function_Exit;
	
Black_And_White_Assumed:

Function_Exit:
	; /*NULL */

} /*void Get_System_Screen_Data(Rect *global_Rect ... */
/*=**************************************************************************=*/


/*=**************************************************************************=*/
void Determine_Information_Of_Device_Containing_Rect(Rect *global_Rect_Passed
													, INTEGER_16		*video_Monitor_Driver_Ref
													, INTEGER_8		*video_Monitor_Slot
													, INTEGER_8		*video_Monitor_ID
													, CARDINAL_32	*video_Optional_Base_Address
													, INTEGER_32		*bits_Per_Pixel
													, INTEGER_32		*bits_Per_Pixel_Greatest
													, CARDINAL_16 	*total_Intersected_Screens
													, Safe_Boolean	*rect_Is_Within_A_Screen
													, PixMapHandle	*video_Pix_Map_Handle
													, GDHandle			*graphics_Device
													, Safe_Boolean	*slot_Information_Valid)
													
 	/*=--------------------------------------------------------------------------=*/
 	/* returns total number of intersected video devices (if any) */
 	/* and returns the smallest bit depth of the screens that do intersect */
 	/* the "global_Rect" (passed in local coordinates WHEN PORT IS ACTIVE)*/
 	/* Also, for video purposes, indicates if rect is found completely within a screen */
 	/*=--------------------------------------------------------------------------=*/

{
	Rect						screen_Rect;
	Rect						global_Rect;
	INTEGER_16		screen_Is_Color;
	INTEGER_16		screen_Is_Main;
	INTEGER_16		screen_Is_Video;
	INTEGER_32		smallest_Bit_Depth_So_Far;
	INTEGER_32		screen_Depth;
	INTEGER_16		screen_Is_Active;
	INTEGER_16		count;
	GDHandle			gDevHand;
	GrafPtr				window_Manager_Port;
/* END VAR */
	

	*slot_Information_Valid	 =  SAFE_FALSE;
	*total_Intersected_Screens = 0;
	*rect_Is_Within_A_Screen = SAFE_FALSE;
	*video_Pix_Map_Handle = nil;
	*graphics_Device = nil;

/*  */
/* Simple paranoia inittialization of extra parameters.*/
/*  */
	*video_Optional_Base_Address = (CARDINAL_32)nil;
	*video_Monitor_Driver_Ref = 0;
	*video_Monitor_Slot = -1;
	*video_Monitor_ID = -1;
	*bits_Per_Pixel_Greatest = 1;

/* ===
	global_Rect = *local_Rect;
	LocalToGlobal(&TOP_LEFT(global_Rect));
	LocalToGlobal(&BOTTOM_RIGHT(global_Rect));
=== */
	global_Rect = *global_Rect_Passed;
	
	if (ns_Shell_Globals.system_Environment.hasColorQD != TRUE) {
		goto Black_And_White_Assumed;
	}
	
	gDevHand = nil;
	count = 0;
			
	gDevHand = GetDeviceList();
	if (gDevHand == nil)
		goto Black_And_White_Assumed;

 	/*  */
 	/* set big on purpose, will decrement as devices found */
 	/*  */
	smallest_Bit_Depth_So_Far = INTEGER_32_MAX; 

	do {
	
		screen_Is_Video = (INTEGER_16)TestDeviceAttribute(gDevHand, screenDevice);
		screen_Is_Active = (INTEGER_16)TestDeviceAttribute(gDevHand, screenActive);

		if ((screen_Is_Active != 0) && (screen_Is_Video != 0)) { 
	/* ===========
				screen_Is_Main = (INTEGER_16)TestDeviceAttribute(gDevHand, mainScreen);
				screen_Is_Color = (INTEGER_16)TestDeviceAttribute(gDevHand, gdDevType);
	============ */
	
				screen_Rect = (*(*gDevHand)->gdPMap)->bounds;
								
				if ( Rect_Intersects_Rect(&screen_Rect, &global_Rect) == SAFE_TRUE) {
					
					*graphics_Device	= gDevHand;
					
					*total_Intersected_Screens +=1;
					
					screen_Depth = (*(*gDevHand)->gdPMap)->pixelSize; /* Bits per Pixel */
					if (screen_Depth < smallest_Bit_Depth_So_Far)
						smallest_Bit_Depth_So_Far = screen_Depth;
						
					if (screen_Depth > *bits_Per_Pixel_Greatest)
						*bits_Per_Pixel_Greatest = screen_Depth;
						
					if ( Rect_Is_In_Rect(&screen_Rect, &global_Rect) == SAFE_TRUE) {
						*rect_Is_Within_A_Screen = SAFE_TRUE;
						*video_Monitor_Driver_Ref = (*gDevHand)->gdRefNum;
						*video_Pix_Map_Handle = (*gDevHand)->gdPMap;
						
						if ( Fetch_Extra_Video_Driver_Info(*video_Monitor_Driver_Ref
																, video_Optional_Base_Address
																, video_Monitor_Slot
																, video_Monitor_ID) == SAFE_TRUE) {
							*slot_Information_Valid	 =  SAFE_TRUE;
						}
						else {
							*slot_Information_Valid	 =  SAFE_FALSE;
						}
						
					}
					
				} /* if ( Rect_Intersects_Rect... */
				
			} /*if ((screen_Is_Active != 0) && (screen_Is_Video != 0)) */
		
		count += 1;
		
		gDevHand = GetNextDevice(gDevHand);
	
	} while (gDevHand != nil);
	
	if (*total_Intersected_Screens == 0)
		goto Black_And_White_Assumed;
		
	*bits_Per_Pixel = smallest_Bit_Depth_So_Far;
		
	goto Function_Exit;
	
	
Black_And_White_Assumed:

	GetWMgrPort(&window_Manager_Port);
	screen_Rect = window_Manager_Port->portRect;
	
	if ( Rect_Intersects_Rect(&screen_Rect, &global_Rect) == SAFE_TRUE) {
		*total_Intersected_Screens +=1;
		
		if ( Rect_Is_In_Rect(&screen_Rect, &global_Rect) == SAFE_TRUE)
			*rect_Is_Within_A_Screen = SAFE_TRUE;
	}

	*bits_Per_Pixel = 1;


Function_Exit:
	; /*NULL */

} /*void Determine_Information_Of_Device_Containing_Rect(Rect *global_Rect ... */
/*=**************************************************************************=*/


/*=**************************************************************************=*/
OSErr Create_Off_Screen_Port(Rect					*offscreen_Bounds
												, INTEGER_16		desired_Bit_Depth
												, CTabHandle		color_Table_Handle
												, CGrafPtr			graphics_Port
												, GDHandle			*created_Graphics_Device
												, Handle				*image_Handle_Ptr) 
/*  */
/* If it fails, Create_Off_Screen_Port will delete all structures */
/* it creates AND it WILL DELETE THE PASSED HANDLE */
/*  */
{

	PixMapHandle	newPixMap;	/* Handle to the new off-screen PixMap */
	GDHandle			newDevice;	/* Handle to the new off-screen GDevice */
	INTEGER_32		qdVersion;	/* Version of QuickDraw currently in use */
	GrafPtr				savedPort;	/* Pointer to GrafPort used for save/restore */
	INTEGER_16		bytesPerRow; /* Number of bytes per row in the PixMap */
	OSErr					error;		/* Returns error code */
/* END VAR */
	
	
	/* Initialize a few things before we begin */
	newPixMap = nil;
	newDevice = nil;
	error = noErr;

	if (color_Table_Handle != nil){
		HNoPurge((Handle)color_Table_Handle);
		if (MemError() != noErr) {
			error = -1;
			goto Fatal_Memory_Handler;
		}
		
		/*  */
		/* This should not need to be locked but it is for paranoia. */
		/* Move high first in future? */
		/*  */
		MoveHHi((Handle)color_Table_Handle);
		
		HLock((Handle)color_Table_Handle);
		if (MemError() != noErr) {
			error = -1;
			goto Fatal_Memory_Handler;
		}
	}
	else {
		error = -1;
		goto Fatal_Memory_Handler;
	}

	/*  */
	/* Round up to nearest long word and even align it 32 bits */
	/* wide  == 4 bytes total */
	/* Quickdraw works with even alignment but is FASTER for 4 byte */
	/*  */
	bytesPerRow = (((desired_Bit_Depth * (offscreen_Bounds->right - offscreen_Bounds->left)) + 31) >> 5) << 2;

	/* Get the current QuickDraw version */
	(void)Gestalt(gestaltQuickdrawVersion, &qdVersion);

	/* Make sure desired_Bit_Depth is indexed or depth is direct and 32-Bit QD installed */
	if (desired_Bit_Depth == 1 || desired_Bit_Depth == 2 || desired_Bit_Depth == 4 || desired_Bit_Depth == 8 ||
			((desired_Bit_Depth == 16 || desired_Bit_Depth == 32) && qdVersion >= gestalt32BitQD))
	{
		/* Maximum number of bytes per row is 16,382; make sure within range */
		if (bytesPerRow > k_MAXIMUM_ROW_BYTES)
		{
			/* # of bytes per row is more than 16,382; is parameter error */
			error = paramErr;
		}
	}
	else
		/* Pixel depth isnt valid; is parameter error */
		error = paramErr;

	/* If sanity checks succeed, then allocate a new CGrafPort */
	if (error == noErr)
	{
		/* Save the current port */
		GetPort(&savedPort);

		/* Initialize the new CGrafPort and make it the current port */
		OpenCPort(graphics_Port);

		/* Set portRect, visRgn, and clipRgn to the given offscreen_Bounds rect */
		graphics_Port->portRect = *offscreen_Bounds;
		RectRgn(graphics_Port->visRgn, offscreen_Bounds);
		ClipRect(offscreen_Bounds);

		/* Initialize the new PixMap for off-screen drawing */
		error = Setup_Pix_Map_Structure(desired_Bit_Depth
														, offscreen_Bounds
														, color_Table_Handle
														, bytesPerRow
														, graphics_Port->portPixMap
														, image_Handle_Ptr);
														
		if (error == noErr)
		{
			/* Grab the initialized PixMap handle */
			newPixMap = graphics_Port->portPixMap;

			/* Allocate and initialize a new GDevice */
			error = Create_GDevice_Structure(newPixMap, &newDevice);
		}

		/* Restore the saved port */
		Set_View_Port(savedPort);
	}

	/* One Last Look Around The House Before We Go */
	if (error != noErr)
	{
		/* Some error occurred; dispose of everything we allocated */
		if (newPixMap != nil) {
			DisposeHandle((Handle)*image_Handle_Ptr);
			if (MemError() != noErr) {
				/*  */
				/* Ignore error in this implementation */
				/*  */
			}
			DisposCTable(color_Table_Handle);
			if (MemError() != noErr) {
				/*  */
				/* Ignore error in this implementation */
				/*  */
			}
		}
		
		if (newDevice != nil) {
			DisposHandle((Handle)(**newDevice).gdITable);
			if (MemError() != noErr) {
				/*  */
				/* Ignore error in this implementation */
				/*  */
			}
			DisposHandle((Handle)newDevice);
			if (MemError() != noErr) {
				/*  */
				/* Ignore error in this implementation */
				/*  */
			}
			*created_Graphics_Device = nil;
		}
	}
	else
	{
		/* Everythings OK; return GDevice */
		*created_Graphics_Device = newDevice;
	}
	
		
Fatal_Memory_Handler:

	/* Return the error code */
	
	return(error);
	
} /* OSErr Create_Off_Screen_Port(Rect *offscreen_Bounds ... */ 
/*=**************************************************************************=*/


/*=**************************************************************************=*/
void Set_Color_Match_For_Pixel_Copy(Boolean set_Match, PixMapHandle pixel_Map)
{
PixMapPtr	pixel_Map_Ptr;
/* END VAR */


	/*  */
	/* Not surefire, but is a normal technique to request a non-match */
	/* of color tables to be honored for extra copyBits speed */ 
	/*  */
	if (pixel_Map != nil) {
	
		pixel_Map_Ptr = *pixel_Map;
		
		if (pixel_Map_Ptr->pixelType == 0) {
			if (pixel_Map_Ptr->pmTable != nil) {
				if (set_Match != TRUE) {
					(**pixel_Map_Ptr->pmTable).ctFlags = (**pixel_Map_Ptr->pmTable).ctFlags | k_MATCH_COLOR_SEED_FLAG;
				}
				else {
					(**pixel_Map_Ptr->pmTable).ctFlags = (**pixel_Map_Ptr->pmTable).ctFlags & (~k_MATCH_COLOR_SEED_FLAG);
				}
	
			}
		}
	}
}
/*=**************************************************************************=*/


/*=**************************************************************************=*/
OSErr Setup_Pix_Map_Structure(INTEGER_16				desired_Bit_Depth
															, Rect						*offscreen_Bounds
															, CTabHandle			color_Table_Handle
															, INTEGER_16			bytes_Per_Row
															, PixMapHandle		initialized_Bit_Map_Handle
															, Handle					*image_Handle_Ptr)

{
	Ptr		offBaseAddr; /* Pointer to the off-screen pixel image */
	CARDINAL_32		total_Graphics_Storage;
	OSErr						error;		/* Returns error code */
	PixMap					*map_Ptr;
	CARDINAL_32		debug_Value_1;
/* END VAR */
	
	
	error = noErr;
	offBaseAddr = nil;
		
	/*  */
	/*  */
	/*  */
	/*  */
	/*  */
	/*  TEMP HACK */
	/*  */
	/*  */
/* ===
	debug_Value_1 = desired_Bit_Depth;
	if (debug_Value_1 < 2) {
		Get_Debug("\pHEY HEY HEY its low bit (re : d0");
	}
	else {
		Get_Debug("\pHEY  Regular (re : d0");
	}
	
	asm {
		movem.l		d0, -(SP)
		nop
		nop
		move.l		debug_Value_1, d0
		movem.l		(sp)+, d0						
	}
==== */
	
	
	/* Allocate pixel image; CARDINAL_32 multiplication avoids overflow */		
	
	/*  */
	/*  */
	/*  */
	/*  */
	/* 1993.08.21 JAM BUG FIX */
	/* BUG FIX!!!! was off by one row for height!!!!!! */
	/* This is serious! */
	/*  */
	total_Graphics_Storage = (CARDINAL_32)bytes_Per_Row * (offscreen_Bounds->bottom - offscreen_Bounds->top + 1L);
	
	*image_Handle_Ptr = NewHandle(total_Graphics_Storage);
	if ((MemError() != noErr) || (*image_Handle_Ptr == nil)) {
		error = MemError();
		goto Fatal_Memory_Handler;
	}
	
	MoveHHi(*image_Handle_Ptr);
	error = MemError();
	if ((error != noErr) && (error != -117)){
		goto Fatal_Memory_Handler;
	}
	
	/*  */
	/* Sadly, must remain locked forever until port is no longer needed */
	/*  */
	HLock(*image_Handle_Ptr);
	if (MemError() != noErr) {
		error = MemError();
		goto Fatal_Memory_Handler;
	}
	
	offBaseAddr = **image_Handle_Ptr;
	
	
	map_Ptr = *initialized_Bit_Map_Handle;
			
	/*  */
	/* Initialize fields common to indexed and direct PixMaps */
	/* Because "initialized_Bit_Map_Handle" is not locked down, */
	/* DO NOT call traps that move memory*/
	/*  */

	map_Ptr->baseAddr = offBaseAddr;  /* Point to image */
	map_Ptr->rowBytes = bytes_Per_Row | /* MSB set for PixMap */
			0x8000;
	map_Ptr->bounds = *offscreen_Bounds;		/* Use given offscreen_Bounds */
	map_Ptr->pmVersion = 0;			/* No special stuff */
	map_Ptr->packType = 0;			/* Default PICT pack */
	map_Ptr->packSize = 0;			/* Always zero in mem */
	map_Ptr->hRes = k_DEFAULT_RESOULUTION;		/* 72 DPI default res */
	map_Ptr->vRes = k_DEFAULT_RESOULUTION;		/* 72 DPI default res */
	map_Ptr->pixelSize = desired_Bit_Depth;		/* Set # bits/pixel */
	map_Ptr->planeBytes = 0;			/* Not used */
	map_Ptr->pmReserved = 0;			/* Reserved and ABSOLUTELY MUST BE ZERO!!!*/

	/* Initialize fields specific to indexed and direct PixMaps */
	if (desired_Bit_Depth <= 8)
	{
		/* PixMap is indexed */
		map_Ptr->pixelType = 0;		/* Indicates indexed */
		map_Ptr->cmpCount = 1;		/* Have 1 component */
		map_Ptr->cmpSize = desired_Bit_Depth;	/* Component size=desired_Bit_Depth */
		map_Ptr->pmTable = color_Table_Handle; /* Handle to CLUT */
	}
	else
	{
		/* PixMap is direct */
		map_Ptr->pixelType = RGBDirect; /* Indicates direct */
		map_Ptr->cmpCount = 3;			/* Have 3 components */
		if (desired_Bit_Depth == 48) {
			map_Ptr->cmpSize = 16;		/* 16 bits/component */
		}
		else {
			if (desired_Bit_Depth == 16) {
				map_Ptr->cmpSize = 5;		/* 5 bits/component */
			}
			else {
				map_Ptr->cmpSize = 8;		/* 8 bits/component */
			}
		}
			
		/*  */
		/* The following have already been set, but just in case, we set them */
		/*  */
		(**color_Table_Handle).ctSeed = 3 * map_Ptr->cmpSize;
		(**color_Table_Handle).ctFlags = 0;
		(**color_Table_Handle).ctSize = 0;
		
		map_Ptr->pmTable = color_Table_Handle;
	}
			
	
		
Fatal_Memory_Handler:

	/* Return the error code */
	return error;
	
} /* OSErr Setup_Pix_Map_Structure(INTEGER_16 desired_Bit_Depth ... */
/*=**************************************************************************=*/


/*=**************************************************************************=*/
OSErr Create_GDevice_Structure( PixMapHandle bit_Map_For_Device_Handle
	, GDHandle	*new_GDevice_Handle)
{
	GDHandle	newDevice;  /* Handle to the new GDevice */
	ITabHandle embryoITab; /* Handle to the embryonic inverse table */
	Rect		deviceRect; /* Rectangle of GDevice */
	OSErr		error;		/* Error code */
/* END VAR */
	
	
	/* Initialize a few things before we begin */
	error = noErr;
	newDevice = nil;
	embryoITab = nil;

	/* Allocate memory for the new GDevice */
	newDevice = (GDHandle)NewHandle(sizeof (GDevice));
	if (newDevice != nil)
	{
		/* Allocate the embryonic inverse table */
		embryoITab = (ITabHandle)NewHandleClear(2);
		if (embryoITab != nil)
		{
			/* Set rectangle of device to PixMap bounds */
			deviceRect = (**bit_Map_For_Device_Handle).bounds;

										/* Initialize the new GDevice fields */
										
			(**newDevice).gdRefNum = 0;					/* Only used for screens */
			(**newDevice).gdID = 0;							/* Wont normally use */
			
			if ((**bit_Map_For_Device_Handle).pixelSize <= 8)
				(**newDevice).gdType = clutType;			/* Depth8; clut device */
			else
				(**newDevice).gdType = directType;		/* Depth>8; direct device */
				
			(**newDevice).gdITable = embryoITab;											/* 2-byte handle for now */
			(**newDevice).gdResPref = k_INVERSE_TABLE_RESOULUTION;	/* Normal inv table res */
			(**newDevice).gdSearchProc = nil;		/* No color-search proc */
			(**newDevice).gdCompProc = nil;		/* No complement proc */
			(**newDevice).gdFlags = 0;					/* Will set these later */
			(**newDevice).gdPMap = bit_Map_For_Device_Handle;				/* Reference our PixMap */
			(**newDevice).gdRefCon = 0;				/* Wont normally use */
			(**newDevice).gdNextGD = nil;			/* Not in GDevice list */
			(**newDevice).gdRect = deviceRect;	/* Use PixMap dimensions */
			(**newDevice).gdMode = -1;				/* For nonscreens */
			(**newDevice).gdCCBytes = 0;				/* Only used for screens */
			(**newDevice).gdCCDepth = 0;			/* Only used for screens */
			(**newDevice).gdCCXData = 0;			/* Only used for screens */
			(**newDevice).gdCCXMask = 0;			/* Only used for screens */
			(**newDevice).gdReserved = 0;			/* Currently unused */

			/* Set color-device bit if PixMap isnt black & white */
			if ((**bit_Map_For_Device_Handle).pixelSize > 1)
				SetDeviceAttribute(newDevice, gdDevType, true);

			/* Set bit to indicate that the GDevice has no video driver */
			SetDeviceAttribute(newDevice, noDriver, true);

			/* Initialize the inverse table */
			if ((**bit_Map_For_Device_Handle).pixelSize <= 8) {
				MakeITable((**bit_Map_For_Device_Handle).pmTable
								, (**newDevice).gdITable
								, (**newDevice).gdResPref);
				error = QDError();
			}
		}
		else
			error = MemError();
	}
	else
		error = MemError();

	/* Handle any errors along the way */
	if (error != noErr)
	{
		if (embryoITab != nil)
			DisposHandle((Handle)embryoITab);
		if (newDevice != nil)
			DisposHandle((Handle)newDevice);
	}
	else
		*new_GDevice_Handle = newDevice;

	/* Return a handle to the new GDevice */
	return error;
	
} /* OSErr Create_GDevice_Structure(PixMapHandle bit_Map_For_Device_Handle ... */
/*=**************************************************************************=*/


/*=**************************************************************************=*/
void Dispose_Off_Screen_Port(
	CGrafPtr doomedPort,	/* Pointer to the CGrafPort to be disposed of */
	GDHandle doomedGDevice) /* Handle to the GDevice to be disposed of */
	/*  */
	/* Note this does not dispose of the bit map memory, for versatility */
	/*  */
{
	CGrafPtr currPort;	/* Pointer to the current port */
	GDHandle currGDevice; /* Handle to the current GDevice */
/* END VAR */
	
	
	/* Check to see whether the doomed CGrafPort is the current port */
	GetPort((GrafPtr *)&currPort);
	if (currPort == doomedPort)
	{
		/* It is; set current port to Window Manager CGrafPort */
		GetCWMgrPort(&currPort);
		Set_View_Port((GrafPtr)currPort);
	}

	/* Check to see whether the doomed GDevice is the current GDevice */
	currGDevice = GetGDevice();
	if (currGDevice == doomedGDevice)
		/* It is; set current GDevice to the main screens GDevice */
		Set_View_Device(GetMainDevice());

	/* Throw everything away */
	(**doomedGDevice).gdPMap = nil;
	DisposGDevice(doomedGDevice);
	/*  */
	/* Our Port stores its memory in an external structure */
	/* (such as a locked handle). Therefore DO NOT dispose of */
	/* memory this way */
	/*  */
/* ====
	DisposPtr((**doomedPort->portPixMap).baseAddr);
==== */


	/*  */
	/* Our Clut is stored external structure */
	/* (such as a shared common CLUT). Therefore DO NOT dispose of */
	/* memory this way */
	/*  */
/* ===
	if ((**doomedPort->portPixMap).pmTable != nil)
		DisposCTable((**doomedPort->portPixMap).pmTable);
=== */
		
	CloseCPort(doomedPort);
	
	/*  */
	/* Our Port is stored external structure */
	/* (such as a global structure). Therefore DO NOT dispose of */
	/* memory this way */
	/*  */
/* ===
	DisposPtr((Ptr)doomedPort);
=== */
	
} /* void Dispose_Off_Screen_Port(CGrafPtr doomedPort ... */
/*=**************************************************************************=*/


/*=**************************************************************************=*/
Safe_Boolean	Close_Offscreen_Pict_Port(Off_Screen_Ptr	offscreen)
{
	Safe_Boolean		function_Result;
	CARDINAL_32	paranoia_Test_Size;
/* END VAR */


	function_Result = SAFE_FALSE;

	if (offscreen->initialized == SAFE_TRUE) {
		switch (offscreen->color_Type) {
		
			case k_COLOR_QUICKDRAW:
			case k_32_BIT_COLOR_QUICKDRAW:
				if (offscreen->image_Handle != nil) {
				
					Dispose_Off_Screen_Port(&(offscreen->offscreen_Port.color_Port)
															, offscreen->offscreen_GDevice_Hdl);
																			
					DisposeHandle(offscreen->image_Handle);
					if (MemError() != noErr) {
						/*  */
						/* We are closing up anyway. Ignore this serious error. */
						/*  */
					}

					/*  */
					/* The pmTable of the portPixMap is externally stoored */
					/* and disposed of, right here in fact we do it */
					/*  */
										
					if ((offscreen->color_Table_Handle != nil) 
								&& (*(offscreen->color_Table_Handle) != nil)) {
								
						DisposCTable(offscreen->color_Table_Handle);
						if (MemError() != noErr) {
							/*  */
							/* We are closing up anyway. Ignore this serious error. */
							/*  */
						}
						
						/*  */
						/* Extra paranoia */
						/*  */
						if ((offscreen->color_Table_Handle != nil) && (*(offscreen->color_Table_Handle) != nil)){
							paranoia_Test_Size = GetHandleSize((Handle)offscreen->color_Table_Handle);
							if (MemError() == noErr) {
								DisposeHandle((Handle)offscreen->color_Table_Handle);
								if (MemError() != noErr) {
									/*  */
									/* We are closing up anyway. Ignore this serious error. */
									/*  */
								}
							}
						}
						
						offscreen->color_Table_Handle = nil;
						
					}
					
				}
			break;
			
			case k_ORIGINAL_QUICKDRAW:
			default:
				if (offscreen->image_Handle != nil) {
					ClosePort(&offscreen->offscreen_Port.old_Style_Port);
					DisposeHandle(offscreen->image_Handle);
					if (MemError() != noErr) {
						/*  */
						/* We are closing up anyway. Ignore this serious error. */
						/*  */
					}
				}
		} /*  switch (offscreen->color_Type) */
			
		offscreen->initialized = SAFE_FALSE;

	} /* if (offscreen->initialized == SAFE_TRUE) */
	
	
	function_Result = SAFE_TRUE;
	goto Normal_Function_Exit;
	
Error_Handler:
	function_Result = SAFE_FALSE;
	offscreen->initialized = SAFE_FALSE;
		
Normal_Function_Exit:

	return(function_Result);
}
/*=**************************************************************************=*/


/*=**************************************************************************=*/
Safe_Boolean	Create_Clut_For_Direct_Mapping(CARDINAL_16 desired_Depth
																				, CTabHandle *created_Clut)
{
	CTabPtr				table_Ptr;
	Safe_Boolean		function_Result;
	INTEGER_16		direct_Color_Clump;
	CARDINAL_32	new_Size_Needed;
/* END VAR */


	function_Result = SAFE_FALSE;

	
	switch (desired_Depth) {
	
		case 16:
			direct_Color_Clump = k_DIRECT_16_BIT_COLOR;
			break;
		
		case 24:
		case 32:
			direct_Color_Clump = k_DIRECT_32_BIT_COLOR;
			break;
		
		case 48:
			direct_Color_Clump = k_DIRECT_48_BIT_COLOR;
			break;
		
		default :
			goto Error_Handler;
	}
	
	/*  */
	/* Assume that this is direct device, and treat as special case for extra safety */
	/* We are doing this for purity only. */
	/*  */
	
	*created_Clut = (CTabHandle)NewHandle(sizeof (ColorTable) - sizeof (CSpecArray));
	if (MemError() != noErr) {
		goto Error_Handler;
	}

	(***created_Clut).ctSeed = direct_Color_Clump;
	(***created_Clut).ctFlags = 0;
	(***created_Clut).ctSize = 0;
	
	
	function_Result = SAFE_TRUE;
	goto Normal_Function_Exit;
	
Error_Handler:
	function_Result = SAFE_FALSE;

Normal_Function_Exit:

	return(function_Result);
	
} /* Safe_Boolean	Create_Clut_For_Direct_Mapping... */
/*=**************************************************************************=*/


/*=**************************************************************************=*/
Safe_Boolean	Copy_Off_Screen_Port(Off_Screen_Ptr		existing_Offscreen_Ptr
																, Off_Screen_Ptr offscreen_Ptr)
{
	Safe_Boolean		function_Result;
	Rect						backdrop_Size_Rect;
	INTEGER_16		stored_Bit_Depth;
	OSErr					os_Error;
/* END VAR */


	function_Result = SAFE_FALSE;

	if (existing_Offscreen_Ptr->initialized != SAFE_TRUE) {
		goto Normal_Function_Exit;
	}

	offscreen_Ptr->color_Table_Handle = existing_Offscreen_Ptr->color_Table_Handle;
	os_Error = HandToHand((Handle *)&(offscreen_Ptr->color_Table_Handle));
	if ((MemError() != noErr) || (os_Error != noErr)){
		goto Error_Handler;
	}
	
	backdrop_Size_Rect = (**(existing_Offscreen_Ptr->offscreen_Port.color_Port.portPixMap)).bounds;
	stored_Bit_Depth = (**(existing_Offscreen_Ptr->offscreen_Port.color_Port.portPixMap)).pixelSize;

	if (Create_Off_Screen_Port(&backdrop_Size_Rect
											, stored_Bit_Depth
											, offscreen_Ptr->color_Table_Handle
											, (CGrafPtr)&(offscreen_Ptr->offscreen_Port.color_Port)
											, &(offscreen_Ptr->offscreen_GDevice_Hdl)
											, &(offscreen_Ptr->image_Handle)) != noErr) {
		goto Error_Handler;
	}
	
	offscreen_Ptr->initialized =  SAFE_TRUE; 
	offscreen_Ptr->color_Type = existing_Offscreen_Ptr->color_Type;
	Lock_Video_Memory_For_Lazy_But_Safe_Use(offscreen_Ptr);

	function_Result = SAFE_TRUE;
	goto Normal_Function_Exit;
	
Error_Handler:
	function_Result = SAFE_FALSE;

Normal_Function_Exit:

	return(function_Result); 
	
} /* Safe_Boolean	Copy_Off_Screen_Port... */
/*=**************************************************************************=*/


/*=**************************************************************************=*/
void Lock_Video_Memory_For_Lazy_But_Safe_Use(Off_Screen_Ptr offscreen_Ptr)
{	
	CARDINAL_8			*code_Name;
	Str255						message_String;
	Boolean					corrupted_Memory;
	OSErr						os_Error;
/* END VAR */


	code_Name = "\pLock_Video_Memory_For_Lazy_But_Safe_Use";
	corrupted_Memory = FALSE;

	MoveHHi(offscreen_Ptr->image_Handle);
	os_Error = MemError();
	if ((os_Error != noErr) && (os_Error != -117)){
		corrupted_Memory = TRUE;
	}
	HLock(offscreen_Ptr->image_Handle);
	if (MemError() != noErr) {
		corrupted_Memory = TRUE;
	}

	MoveHHi((Handle)offscreen_Ptr->color_Table_Handle);
	os_Error = MemError();
	if ((os_Error != noErr) && (os_Error != -117)){
		corrupted_Memory = TRUE;
	}
	HLock((Handle)offscreen_Ptr->color_Table_Handle);
	if (MemError() != noErr) {
		corrupted_Memory = TRUE;
	}

	MoveHHi((Handle)offscreen_Ptr->offscreen_GDevice_Hdl);
	os_Error = MemError();
	if ((os_Error != noErr) && (os_Error != -117)){
		corrupted_Memory = TRUE;
	}
	HLock((Handle)offscreen_Ptr->offscreen_GDevice_Hdl);
	if (MemError() != noErr) {
		corrupted_Memory = TRUE;
	}
	
	if (corrupted_Memory == TRUE) {
	/* ===
		Copy_Pascal_String("\pDanger: memory heap seems corrupted or bad programming", message_String);
		Display_Serious_Error_Message(unit_Name, code_Name, message_String, "\pCorruption?");
	==== */
	}

}
/*=**************************************************************************=*/


/*=**************************************************************************=*/
Safe_Boolean	Create_Clut_By_Stealing_From_Color_Port(PixMapHandle	video_Pix_Map_Handle
																				, CTabHandle *created_Clut)
{
	CTabPtr				table_Ptr;
	Safe_Boolean		function_Result;
	Handle				source_Handle;
	Boolean				direct_Color_Assumed;
	INTEGER_16		direct_Color_Clump;
	CARDINAL_32	new_Size_Needed;
/* END VAR */


	function_Result = SAFE_FALSE;

	source_Handle = (Handle)(**(video_Pix_Map_Handle)).pmTable;
	
	switch ((**(CTabHandle)source_Handle).ctSeed) {
	
		case k_DIRECT_16_BIT_COLOR:
			direct_Color_Assumed = TRUE;
			direct_Color_Clump = k_DIRECT_16_BIT_COLOR;
			break;
		
		case k_DIRECT_32_BIT_COLOR:
			direct_Color_Assumed = TRUE;
			direct_Color_Clump = k_DIRECT_32_BIT_COLOR;
			break;
		
		case k_DIRECT_48_BIT_COLOR:
			direct_Color_Assumed = TRUE;
			direct_Color_Clump = k_DIRECT_48_BIT_COLOR;
			break;
		
		default :
			direct_Color_Assumed = FALSE;
	}
	
	if (direct_Color_Assumed == TRUE) {
		/*  */
		/* Assume that this is direct device, and treat as special case for extra safety */
		/* We are doing this for purity only. */
		/*  */
		
		*created_Clut = (CTabHandle)NewHandle(sizeof (ColorTable) - sizeof (CSpecArray));
		if (MemError() != noErr) {
			goto Error_Handler;
		}

		(***created_Clut).ctSeed = direct_Color_Clump;
		(***created_Clut).ctFlags = 0;
		(***created_Clut).ctSize = 0;
		
	}
	else {
	
		new_Size_Needed = GetHandleSize(source_Handle);
		if (MemError() != noErr) {
			goto Error_Handler;
		}
	
		*created_Clut = (CTabHandle)NewHandle(new_Size_Needed);
		if (MemError() != noErr) {
			goto Error_Handler;
		}
	
	
		BlockMove(*source_Handle, (Ptr)**created_Clut, new_Size_Needed);
		
		/*  */
		/* Violate convention by deliberately setting seed value to match another */
		/* clut. Should this clut be shared? should they be different? */
		/*  */
	/* ===
		table_Ptr = **created_Clut;
		table_Ptr->ctSeed = GetCTSeed();
	=== */
	
	} /* NOT if ((**source_Handle).ctSeed == (3 * ( ... */
	
	function_Result = SAFE_TRUE;
	goto Normal_Function_Exit;
	
Error_Handler:
	function_Result = SAFE_FALSE;

Normal_Function_Exit:

	return(function_Result);
	
} /* Safe_Boolean	Create_Clut_By_Stealing_From_Color_Port... */
/*=**************************************************************************=*/
	
	
/*=**************************************************************************=*/
Safe_Boolean Load_Pict_Into_Color_Bit_Map(INTEGER_16 source_Pict_Id
																		, CARDINAL_16	desired_Color_Depth
																		, CTabHandle		offscreen_Color_Table_Handle
																		, CGrafPtr	offscreen_Graph_Port
																		, GDHandle	*offscreen_GDevice_Ptr
																		, Handle		*image_Handle_Ptr)
	/*  */
	/* This draws a Pict RESOURCE into a new port */
	/* This Allocates a handle and fills it with bitmap image and uses color*/
	/*  */
{
	PicHandle			source_Pict_Hdl;
	INTEGER_32		source_Bit_Width;
	INTEGER_32		source_Bit_Height;
	Rect						source_Bit_Rect;
	GrafPtr				saved_Port;
	GDHandle			saved_Device;
	Safe_Boolean		function_Result;
	OSErr					os_Error;
	Rect						offscreen_Rect;
/* END VAR */


	function_Result = SAFE_FALSE;
	
	source_Pict_Hdl = nil;
	source_Pict_Hdl = GetPicture(source_Pict_Id);
	if ((ResError() != noErr) || (source_Pict_Hdl == nil)) {
		goto Error_Handler;
	}
		
	source_Bit_Rect = (*source_Pict_Hdl)->picFrame;
	source_Bit_Width = source_Bit_Rect.right - source_Bit_Rect.left;
	source_Bit_Height = source_Bit_Rect.bottom - source_Bit_Rect.top;
		
	/*  */
	/* Set up the rectangle for the off-screen graphics environments */
	/*  */
	SetRect(&offscreen_Rect, 0, 0, source_Bit_Width, source_Bit_Height);
	
	os_Error= Create_Off_Screen_Port(&offscreen_Rect
															, desired_Color_Depth
															, offscreen_Color_Table_Handle
															, offscreen_Graph_Port
															, offscreen_GDevice_Ptr
															, image_Handle_Ptr);

	if (os_Error != noErr) {
		goto Error_Handler;
	}

	GetPort(&saved_Port);
	saved_Device = GetGDevice();
	
	Set_View_Port((GrafPtr)offscreen_Graph_Port);
	Set_View_Device(*offscreen_GDevice_Ptr);
	
	/*  */
	/* For safety to clear out the memory */
	/*  */
	EraseRect(&offscreen_Rect); 
	
	/*  */
	/* Only needed really for CopyBits calls but why not be certain. */
	/*  */
	ForeColor(blackColor);
	BackColor(whiteColor);
	
	SetRect(&source_Bit_Rect, 0, 0, source_Bit_Width, source_Bit_Height);
	DrawPicture(source_Pict_Hdl, &source_Bit_Rect);
	
	Set_View_Port(saved_Port);
	Set_View_Device(saved_Device);
	
	/*  */
	/* Release resource */
	/*  */
	if (source_Pict_Hdl != nil) {
		ReleaseResource((Handle)source_Pict_Hdl);
	}
	
	function_Result = SAFE_TRUE;
	goto Normal_Function_Exit;
		
Fatal_Memory_Handler:

Error_Handler:
	function_Result = SAFE_FALSE;

Normal_Function_Exit:

	return(function_Result);

}	/* Safe_Boolean Load_Pict_Into_Color_Bit_Map */
/*=**************************************************************************=*/


/*=**************************************************************************=*/
Safe_Boolean Guarantee_File_Is_PICT_Type(INTEGER_16 source_Volume_Reference
						, INTEGER_32 source_Directory_ID
						, Str63 source_File_Name
						, INTEGER_32 *source_Bit_Width
						, INTEGER_32 *source_Bit_Height)
{
	Safe_Boolean		function_Result;
	CARDINAL_32	source_Bit_Color_Depth;
/* END VAR */


	function_Result = Guarantee_File_Is_PICT_Type_And_Parse_Depth(source_Volume_Reference
																					, source_Directory_ID
																					, source_File_Name
																					, source_Bit_Width
																					, source_Bit_Height
																					, SAFE_FALSE
																					, &source_Bit_Color_Depth);
	
	return(function_Result);
																		
}
/*=**************************************************************************=*/


/*=**************************************************************************=*/
Safe_Boolean Guarantee_File_Is_PICT_Type_And_Parse_Depth(INTEGER_16 source_Volume_Reference
						, INTEGER_32 source_Directory_ID
						, Str63 source_File_Name
						, INTEGER_32 *source_Bit_Width
						, INTEGER_32 *source_Bit_Height
						, Safe_Boolean parse_For_Color_Depth
						, CARDINAL_32	*source_Bit_Color_Depth)
{
	Safe_Boolean		function_Result;
	Safe_Boolean		file_Is_Open;
	OSErr					os_Error;
	INTEGER_16		source_Pict_File_Ref;
	Rect						source_Bit_Rect;
	INTEGER_32		source_File_Size;
	HFileParam		file_Info_Params;
	FSSpec				new_File_Info;
	FSSpec				*file_Info;
/* END VAR */


	function_Result = SAFE_FALSE;
	file_Is_Open = SAFE_FALSE;
	source_Pict_File_Ref = 0;
	
	/*  */
	/* Paranoia */
	/*  */
	*source_Bit_Width = 1;
	*source_Bit_Height = 1;
	
	file_Info_Params.ioCompletion = nil;
	file_Info_Params.ioVRefNum = source_Volume_Reference;
	file_Info_Params.ioFVersNum = 0;
	file_Info_Params.ioFDirIndex = 0;
	file_Info_Params.ioNamePtr = &(source_File_Name[0]);
	file_Info_Params.ioDirID = source_Directory_ID;
	
	os_Error = PBHGetFInfo((HParmBlkPtr)&file_Info_Params, FALSE);
	if (os_Error != noErr) {
		goto Error_Handler;
	}
	
	if (file_Info_Params.ioFlFndrInfo.fdType != 'PICT') {
		/*  */
		/* Only files of actual type PICT are acceptable to us. */
		/*  */
		goto Error_Handler;
	}
		
#if !defined(k_USE_BUFFERED_FILE_IO)
#error "UNDEFINED PARAMETER!"
#endif	
#if (k_USE_BUFFERED_FILE_IO == 1)
		source_File_Size = 0;
		if (Open_Well_Defined_File_Using_Buffer(DATA_FORK
										, &(source_File_Name[0])
										, 0
										, source_Volume_Reference
										, source_Directory_ID
										, &source_Pict_File_Ref
										, (512L)
										, 4096
										, 4096
										, &source_File_Size
										, SAFE_TRUE
										, &os_Error) != TRUE) {
			os_Error = -1;
		}
#else	
	file_Info = &new_File_Info;
	os_Error = FSMakeFSSpec(source_Volume_Reference,source_Directory_ID, &(source_File_Name[0]), file_Info);
	if (os_Error != noErr) {
		/*  */
		/* (File not found) fnfErr is most common, but not acceptable here */
		/*  */
	}
	else {
		os_Error = FSpOpenDF(&(file_Info->file_Info),fsCurPerm,&source_Pict_File_Ref);
	}

#endif


	if (os_Error == noErr){
		file_Is_Open = SAFE_TRUE;
		/*  */
		/* Only needed really for CopyBits calls but why not be certain. */
		/*  */
		ForeColor(blackColor);
		BackColor(whiteColor);
																		
		/*  */
		/* NOTE: Get_File_Picture_Rect relies on "k_USE_BUFFERED_FILE_IO" method */
		/*  */
		os_Error = Get_File_Picture_Rect_And_Depth(source_Pict_File_Ref
																, &source_Bit_Rect
																, parse_For_Color_Depth
																, source_Bit_Color_Depth);
		if (os_Error != noErr) {
			/* ignore, will skip over soon */
		}
		
		/*  */
		/* source_Bit_Rect is ready! nor we can re-read and re-process the PICT */
		/*  */
		if (os_Error == noErr){
			*source_Bit_Width = source_Bit_Rect.right - source_Bit_Rect.left;
			*source_Bit_Height = source_Bit_Rect.bottom - source_Bit_Rect.top;
		}
	}
	
	if (os_Error != noErr) {
		goto Error_Handler;
	}
	
	
	function_Result = SAFE_TRUE;
	goto Normal_Function_Exit;
		
Error_Handler:
	function_Result = SAFE_FALSE;

Normal_Function_Exit:

	if (file_Is_Open == SAFE_TRUE) {
#if (k_USE_BUFFERED_FILE_IO == 1)
		if (Close_File_With_Buffer(source_Pict_File_Ref
									, FLUSH_BUFFER
									, &os_Error) != TRUE) {
			/*  */
			/* Hmmm. Not expected. Ignore anyway. */
			/*  */
			Perform_OK_Dialog("\pProblem in Guarantee_File_Is_PICT_Type -  Close_File_With_Buffer (2.0)");
		}
		source_Pict_File_Ref = 0;
#else
		os_Error = FSClose(source_Pict_File_Ref);
		source_Pict_File_Ref = 0;

#endif
		file_Is_Open = SAFE_FALSE;
	}
	
	return(function_Result);
	
} /* Safe_Boolean Guarantee_File_Is_PICT_Type */
/*=**************************************************************************=*/


/*=**************************************************************************=*/
Safe_Boolean Load_Pict_File_Into_Color_Bit_Map(FSSpec *file_Info
																		, CARDINAL_16	desired_Color_Depth
																		, CTabHandle		offscreen_Color_Table_Handle
																		, CGrafPtr	offscreen_Graph_Port
																		, GDHandle	*offscreen_GDevice_Ptr
																		, Handle		*image_Handle_Ptr)
	/*  */
	/* This draws a Pict FILE into a new port */
	/* This Allocates a handle and fills it with bitmap image and uses color*/
	/*  */
{
	INTEGER_32		source_Bit_Width;
	INTEGER_32		source_Bit_Height;
	INTEGER_16		source_Pict_Id;
	INTEGER_16		source_Pict_File_Ref;
	INTEGER_32		source_File_Size;
	Safe_Boolean		file_Is_Open;
	GrafPtr				saved_Port;
	GDHandle			saved_Device;
	Safe_Boolean		function_Result;
	OSErr					os_Error;
	Rect						offscreen_Rect;
/* END VAR */


	function_Result = SAFE_FALSE;
	file_Is_Open = SAFE_FALSE;

	GetPort(&saved_Port);
	saved_Device = GetGDevice();
		

	if (Guarantee_File_Is_PICT_Type(file_Info->vRefNum
						, file_Info->parID
						, &(file_Info->name[0])
						, &source_Bit_Width
						, &source_Bit_Height) == SAFE_TRUE) {
	
		/*  */
		/* Set up the rectangle for the off-screen graphics environments */
		/*  */
		SetRect(&offscreen_Rect, 0, 0, source_Bit_Width, source_Bit_Height);
		
		os_Error= Create_Off_Screen_Port(&offscreen_Rect
																, desired_Color_Depth
																, offscreen_Color_Table_Handle
																, offscreen_Graph_Port
																, offscreen_GDevice_Ptr
																, image_Handle_Ptr);
	
		if (os_Error != noErr) {
			goto Error_Handler;
		}
	
		
		Set_View_Port((GrafPtr)offscreen_Graph_Port);
		Set_View_Device(*offscreen_GDevice_Ptr);
		
		/*  */
		/* For safety to clear out the memory */
		/*  */
		EraseRect(&offscreen_Rect); 
		
		/*  */
		/* Only needed really for CopyBits calls but why not be certain. */
		/*  */
		ForeColor(blackColor);
		BackColor(whiteColor);

		Set_View_Port((GrafPtr)offscreen_Graph_Port);

#if (k_USE_BUFFERED_FILE_IO == 1)
		if (Open_Well_Defined_File_Using_Buffer(DATA_FORK
										, &(file_Info->name[0])
										, 0
										, (file_Info->vRefNum)
										, (file_Info->parID)
										, &source_Pict_File_Ref
										, (512L)
										, k_PICT_FILE_DRAWING_BUFFER_SIZE
										, k_PICT_FILE_DRAWING_BUFFER_SIZE
										, &source_File_Size
										, SAFE_TRUE
										, &os_Error) != TRUE) {
			Perform_OK_Dialog("\pProblem with Open_Well_Defined_File_Using_Buffer (1)");

			if (os_Error == noErr) {
				os_Error = -1;
			}
		}
#else
os_Error = FSpOpenDF(file_Info,fsCurPerm,&source_Pict_File_Ref);
#endif
		if (os_Error == noErr) {
		
			file_Is_Open = SAFE_TRUE;
			
			os_Error = Draw_File_Picture(source_Pict_File_Ref, &offscreen_Rect);
			if (os_Error != noErr) {
				Perform_OK_Dialog("\pNote: Draw_File_Picture failed");
			}
			
			/*  */
			/* Release File */
			/*  */
#if (k_USE_BUFFERED_FILE_IO == 1)
			if (Close_File_With_Buffer(source_Pict_File_Ref
										, FLUSH_BUFFER
										, &os_Error) != TRUE) {
				/*  */
				/* Hmmm. Not expected. Ignore anyway. */
				/*  */
				Perform_OK_Dialog("\pProblem with Close_File_With_Buffer (1)");
			}
#else
			os_Error = FSClose(source_Pict_File_Ref);
#endif
			source_Pict_File_Ref = 0;
			file_Is_Open = SAFE_FALSE;
		}
					
	} /* if (os_Error == noErr) */
	else {
		goto Error_Handler;
	}
	
	function_Result = SAFE_TRUE;
	goto Normal_Function_Exit;
		
Fatal_Memory_Handler:

Error_Handler:
	function_Result = SAFE_FALSE;

Normal_Function_Exit:

	if (file_Is_Open == SAFE_TRUE) {
#if (k_USE_BUFFERED_FILE_IO == 1)
		if (source_Pict_File_Ref != 0) {
			if (Close_File_With_Buffer(source_Pict_File_Ref
										, FLUSH_BUFFER
										, &os_Error) != TRUE) {
				/*  */
				/* Hmmm. Not expected. Ignore anyway. */
				/*  */
				Perform_OK_Dialog("\pProblem with Close_File_With_Buffer (2)");
			}
			source_Pict_File_Ref = 0;
		}
#else
		os_Error = FSClose(source_Pict_File_Ref);
		source_Pict_File_Ref = 0;

#endif
		file_Is_Open = SAFE_FALSE;
	}

	Set_View_Port(saved_Port);
	Set_View_Device(saved_Device);

	return(function_Result);

}	/* Safe_Boolean Load_Pict_Into_Color_Bit_Map */																	
/*=**************************************************************************=*/
	
	
/*=**************************************************************************=*/
Safe_Boolean Copy_Rect_Into_Color_Bit_Map(Rect *source_Rect
																		, CARDINAL_16	desired_Color_Depth
																		, CTabHandle		offscreen_Color_Table_Handle
																		, CGrafPtr	offscreen_Graph_Port
																		, GDHandle	*offscreen_GDevice_Ptr
																		, Handle		*image_Handle_Ptr)

	/*  */
	/* This Allocates a handle and fills it with bitmap image and uses color*/
	/* Set the correct port before calling. */
	/* It was written for animation purposes */
	/* invloving offscreen buffers (source bitmaps, result bitmap, etc. ) */
	/*  */
{
	INTEGER_32		source_Bit_Width;
	INTEGER_32		source_Bit_Height;
	Rect						destination_Rect;
	GrafPtr				saved_Port;
	GDHandle			saved_Device;
	Safe_Boolean		function_Result;
	OSErr					os_Error;
	Rect						offscreen_Rect;
	PenState				saved_Pen_State;
/* END VAR */


	function_Result = SAFE_FALSE;
			
	source_Bit_Width = (*source_Rect).right - (*source_Rect).left;
	source_Bit_Height = (*source_Rect).bottom - (*source_Rect).top;
		
	/*  */
	/* Set up the rectangle for the off-screen graphics environments */
	/*  */
	SetRect(&offscreen_Rect, 0, 0, source_Bit_Width, source_Bit_Height);
	
	os_Error= Create_Off_Screen_Port(&offscreen_Rect
															, desired_Color_Depth
															, offscreen_Color_Table_Handle
															, offscreen_Graph_Port
															, offscreen_GDevice_Ptr
															, image_Handle_Ptr);

	if (os_Error != noErr) {
		goto Error_Handler;
	}
		
	
	GetPenState (&saved_Pen_State);
	/*  */
	/* Actually needed for CopyBits calls , so be certain. */
	/*  */
	ForeColor(blackColor);
	BackColor(whiteColor);
		
	GetPort(&saved_Port);
	saved_Device = GetGDevice();
	
	Set_View_Port((GrafPtr)offscreen_Graph_Port);
	Set_View_Device(*offscreen_GDevice_Ptr);
	
	/*  */
	/* For safety to clear out the memory */
	/*  */
	EraseRect(&offscreen_Rect); 
	
	/*  */
	/* Needed for CopyBits calls to be certain. */
	/*  */
	ForeColor(blackColor);
	BackColor(whiteColor);
	
	SetRect(&destination_Rect, 0, 0, source_Bit_Width, source_Bit_Height);
	
	CopyBits (&(saved_Port->portBits)
				, (const BitMap *)*(offscreen_Graph_Port->portPixMap)
				, source_Rect
				, &destination_Rect
				, srcCopy
				, nil);
	
	Set_View_Port(saved_Port);
	Set_View_Device(saved_Device);
	SetPenState (&saved_Pen_State);
	
	function_Result = SAFE_TRUE;
	goto Normal_Function_Exit;
		
Fatal_Memory_Handler:

Error_Handler:
	function_Result = SAFE_FALSE;

Normal_Function_Exit:

	return(function_Result);

}	/* Safe_Boolean Copy_Rect_Into_Color_Bit_Map */																	
/*=**************************************************************************=*/
	
	
/*=**************************************************************************=*/
Safe_Boolean Load_Pict_Into_BW_Bit_Map(INTEGER_16 source_Pict_Id
																		, GrafPort	*offscreen_Graph_Port
																		, Handle		*image_Handle_Ptr)
	/*  */
	/* This Allocates a handle and fills it with bitmap image */
	/*  */
{
	PicHandle			source_Pict_Hdl;
	INTEGER_32		source_Bit_Width;
	INTEGER_32		source_Bit_Height;
	Rect						source_Bit_Rect;
	Rect						current_Rect;
	INTEGER_32		bytes_Per_Row;
	CARDINAL_32	total_Graphics_Storage;
	GrafPtr				offscreen_Graph_Ptr;
	GrafPtr				saved_Port;
	Safe_Boolean		function_Result;
/* END VAR */


	function_Result = SAFE_FALSE;
	
	source_Pict_Hdl = nil;
	source_Pict_Hdl = GetPicture(source_Pict_Id);
	if ((ResError() != noErr) || (source_Pict_Hdl == nil)) {
		goto Error_Handler;
	}
		
	source_Bit_Rect = (*source_Pict_Hdl)->picFrame;
	source_Bit_Width = source_Bit_Rect.right - source_Bit_Rect.left;
	source_Bit_Height = source_Bit_Rect.bottom - source_Bit_Rect.top;
		
	/*  */
	/* Round up to nearest long word and even align it */
	/* 32 bits wide  == 4 bytes total */
	/* Quickdraw works with even alignment but is FASTER for 4 byte */
	/*  */
	bytes_Per_Row = (((source_Bit_Width + 31) >> 5) << 2); 
	
	/*  */
	/* In the future, If we were safety-minded, we would multiply  */
	/* the byte width by the height to see if result was too big to fit */
	/* simple structures. No big deal.  */
	/*  */
	total_Graphics_Storage = source_Bit_Height * bytes_Per_Row;
	*image_Handle_Ptr = NewHandle(total_Graphics_Storage);
	if ((MemError() != noErr) || (*image_Handle_Ptr == nil)) {
		goto Error_Handler;
	}

	GetPort(&saved_Port);

	offscreen_Graph_Ptr = offscreen_Graph_Port;
	OpenPort(offscreen_Graph_Ptr);
	
	
	SetRect(&current_Rect,0,0,source_Bit_Width,source_Bit_Height); /* sub by one??? */
	
	MoveHHi(*image_Handle_Ptr);
	if (MemError() != noErr) {
		goto Fatal_Memory_Handler;
	}
	
	/*  */
	/* Sadly, must remain locked forever until port is no longer needed */
	/*  */
	HLock(*image_Handle_Ptr);
	if (MemError() != noErr) {
		goto Fatal_Memory_Handler;
	}
	
	(*offscreen_Graph_Ptr).portBits.baseAddr = (Ptr)(**image_Handle_Ptr);
	(*offscreen_Graph_Ptr).portBits.rowBytes = bytes_Per_Row;
	(*offscreen_Graph_Ptr).portBits.bounds = current_Rect;
	
	/*  */
	/* HACK HACK HACK WAS NOT HERE before 1995.01 */
	/*  */
	(*offscreen_Graph_Ptr).portRect = current_Rect;

		
	Set_View_Port(offscreen_Graph_Ptr);
	
	/*  */
	/* For safety to clear out the memory */
	/*  */
	EraseRect(&current_Rect); 
	
	/*  */
	/* Only needed really for CopyBits calls but why not be certain. */
	/*  */
	ForeColor(blackColor);
	BackColor(whiteColor);
	
	SetRect(&source_Bit_Rect, 0, 0, source_Bit_Width, source_Bit_Height);
	DrawPicture(source_Pict_Hdl, &source_Bit_Rect);
	
/* ===
	ClosePort(offscreen_Graph_Ptr);
=== */
	Set_View_Port(saved_Port);

	if (source_Pict_Hdl != nil) {
		ReleaseResource((Handle)source_Pict_Hdl);
	}
	
	function_Result = SAFE_TRUE;
	goto Normal_Function_Exit;
		
		
Fatal_Memory_Handler:

Error_Handler:
	function_Result = SAFE_FALSE;

Normal_Function_Exit:

	return(function_Result);

} /* Safe_Boolean Load_Pict_Into_BW_Bit_Map(Handle	*image_Handle_Ptr) */
/*=**************************************************************************=*/
	
	
/*=**************************************************************************=*/
Safe_Boolean Copy_Rect_Into_BW_Bit_Map(Rect *source_Rect
																		, GrafPort	*offscreen_Graph_Port
																		, Handle		*image_Handle_Ptr)
	/*  */
	/* This Allocates a handle and fills it with bitmap image */
	/* Set the correct port before calling. */
	/* It was written for animation purposes */
	/* invloving offscreen buffers (source bitmaps, result bitmap, etc. ) */
	/*  */
{
	Rect						source_Bit_Rect;
	Rect						current_Rect;
	INTEGER_32		bytes_Per_Row;
	CARDINAL_32	total_Graphics_Storage;
	GrafPtr				offscreen_Graph_Ptr;
	
	INTEGER_32		source_Bit_Width;
	INTEGER_32		source_Bit_Height;
	Rect						destination_Rect;
	GrafPtr				saved_Port;
	Safe_Boolean		function_Result;
	OSErr					os_Error;
	Rect						offscreen_Rect;
	PenState				saved_Pen_State;
/* END VAR */


	function_Result = SAFE_FALSE;
			
	source_Bit_Width = (*source_Rect).right - (*source_Rect).left;
	source_Bit_Height = (*source_Rect).bottom - (*source_Rect).top;
		
	/*  */
	/* Round up to nearest long word and even align it */
	/* 32 bits wide  == 4 bytes total */
	/* Quickdraw qorks with even alignment but is FASTER for 4 byte */
	/*  */
	bytes_Per_Row = (((source_Bit_Width + 31) >> 5) << 2); 
	
	/*  */
	/* In the future, If we were safety-minded, we would multiply  */
	/* the byte width by the height to see if result was too big to fit */
	/* simple structures. No big deal.  */
	/*  */
	total_Graphics_Storage = source_Bit_Height * bytes_Per_Row;
	*image_Handle_Ptr = NewHandle(total_Graphics_Storage);
	if ((MemError() != noErr) || (*image_Handle_Ptr == nil)) {
		goto Error_Handler;
	}

	GetPenState (&saved_Pen_State);
	
	/*  */
	/* Actually needed for CopyBits calls , so be certain. */
	/*  */
	ForeColor(blackColor);
	BackColor(whiteColor);

	GetPort(&saved_Port);

	offscreen_Graph_Ptr = offscreen_Graph_Port;
	OpenPort(offscreen_Graph_Ptr);
	
	
	SetRect(&current_Rect,0,0,source_Bit_Width,source_Bit_Height); /* sub by one??? */
	
	MoveHHi(*image_Handle_Ptr);
	if (MemError() != noErr) {
		goto Fatal_Memory_Handler;
	}
	
	/*  */
	/* Sadly, must remain locked forever until port is no longer needed */
	/*  */
	HLock(*image_Handle_Ptr);
	if (MemError() != noErr) {
		goto Fatal_Memory_Handler;
	}
	
	(*offscreen_Graph_Ptr).portBits.baseAddr = (Ptr)(**image_Handle_Ptr);
	(*offscreen_Graph_Ptr).portBits.rowBytes = bytes_Per_Row;
	(*offscreen_Graph_Ptr).portBits.bounds = current_Rect;
		
	/*  */
	/* HACK HACK HACK WAS NOT HERE before 1995.01 */
	/*  */
	(*offscreen_Graph_Ptr).portRect = current_Rect;

	Set_View_Port(offscreen_Graph_Ptr);
	
	/*  */
	/* For safety to clear out the memory */
	/*  */
	EraseRect(&current_Rect); 
	
	/*  */
	/* Needed for CopyBits calls to be certain. */
	/*  */
	ForeColor(blackColor);
	BackColor(whiteColor);
	
	SetRect(&destination_Rect, 0, 0, source_Bit_Width, source_Bit_Height);
	
	CopyBits (&(saved_Port->portBits)
				, &(offscreen_Graph_Ptr->portBits)
				, source_Rect
				, &destination_Rect
				, srcCopy
				, nil);
	
	Set_View_Port(saved_Port);
	SetPenState (&saved_Pen_State);
	

	function_Result = SAFE_TRUE;
	goto Normal_Function_Exit;
		
		
Fatal_Memory_Handler:

Error_Handler:
	function_Result = SAFE_FALSE;

Normal_Function_Exit:

	return(function_Result);

} /* Safe_Boolean Copy_Rect_Into_BW_Bit_Map(Handle	*image_Handle_Ptr) */
/*=**************************************************************************=*/


/*=**************************************************************************=*/
Safe_Boolean	Load_Pict_Into_Bit_Map(Off_Screen_Ptr offscreen_Ptr
													, INTEGER_16 pict_Resource_Id)
{
	GrafPtr				active_Port;
	Rect						source_Rect;
	Safe_Boolean		function_Result;
	CARDINAL_32	debug_Value_1;
	CARDINAL_32	debug_Value_2;
	CARDINAL_32	debug_Value_3;
	CARDINAL_32	debug_Value_4;
/* END VAR */


	/*  */
	/* Not used. Paranoia only. */
	/*  */
		GetPort(&active_Port);
		
	/*  */
	/* Could use this instead */
	/*  */
/* ===
		GetPort(&g_Current_Window);
	=== */
	
	
	/*  */
	/* If a desired device to match is unneeded , then use */
	/* a rect set to an arbitrary location */
	/*  */
/* ====
	SetRect(&source_Rect, 0, 0, 1, 1);
==== */

/* ==
	local_Rect = ((WindowPeek)g_Current_Window)->port.portRect;
	LocalToGlobal(&TOP_LEFT(local_Rect));
	LocalToGlobal(&BOTTOM_RIGHT(local_Rect));
	g_Most_Recent_Window_Location = local_Rect;
	g_Most_Recent_Window_Location_Valid = SAFE_TRUE;
== */

	/*  */
	/* Check if the active port (the one we will copy the depth from, */
	/* was created as a color port or not a color port */
	/* ARRRRGH these techniques do not work and neither does */
	/* "bounds". The only way I know how is via global set from */
	/* the current window */
	/*  */
	if ((active_Port->portBits.rowBytes 
						& COLOR_PORT_PORTBITS_CHECK) != COLOR_PORT_PORTBITS_CHECK) {
						
		source_Rect = active_Port->portRect;
	}
	else {
		source_Rect = ((CGrafPtr)active_Port)->portRect;
	}


	/*  */
	/*  */
	/*  */
	/*  */
	/*  */
	/*  TEMP HACK */
	/*  */
	/*  */
	/*  SEEMS TO BE VITAL!! WHY can't I use another technique  */
	/*  */
	
/* ===
	source_Rect = g_Most_Recent_Window_Location;
=== */
	
	/*  */
	/*  */
	/*  */
	



	function_Result = Get_Data_Into_Bit_Map(offscreen_Ptr
													, k_USE_RESOURCE_INFO
													, pict_Resource_Id
													, &source_Rect);
													
	return(function_Result);

}
/*=**************************************************************************=*/


/*=**************************************************************************=*/
Safe_Boolean	Copy_Rect_Into_Bit_Map(Off_Screen_Ptr offscreen_Ptr
													, Rect *source_Rect)
{
	Safe_Boolean		function_Result;
	Rect						global_Rect;
/* END VAR */

	global_Rect = *source_Rect;
	LocalToGlobal(&TOP_LEFT(global_Rect));
	LocalToGlobal(&BOTTOM_RIGHT(global_Rect));

	function_Result = Get_Data_Into_Bit_Map(offscreen_Ptr
													, k_USE_RECT_LOCATION
													, 0
													, &global_Rect);
													
	return(function_Result);

}
/*=**************************************************************************=*/


/*=**************************************************************************=*/
Safe_Boolean	Get_Data_Into_Bit_Map(Off_Screen_Ptr offscreen_Ptr
													, fill_Technique desired_Fill
													, INTEGER_16 pict_Resource_Id
													, Rect *global_Source_Rect)
{
	GrafPtr				source_Port;
	CARDINAL_16	desired_Bit_Depth;
	Safe_Boolean		function_Result;
	Boolean				use_Old_Quick_Draw;
	
	Rect 					local_Rect;
	CARDINAL_16  	total_Intersected_Screens;
	INTEGER_32	 	bits_Per_Pixel_Union;
	INTEGER_32	 	bits_Per_Pixel_Greatest;
	Safe_Boolean		rect_Is_Within_A_Screen;
	INTEGER_16		video_Monitor_Driver_Ref;
	INTEGER_8			video_Monitor_ID;
	INTEGER_8			video_Monitor_Slot;
	CARDINAL_32	video_Optional_Base_Address;
	PixMapHandle	video_Pix_Map_Handle;
	GDHandle			graphics_Device;											
	Safe_Boolean		slot_Information_Valid;													
	
	
	CARDINAL_32	debug_Value_1;
	CARDINAL_32	debug_Value_2;
	CARDINAL_32	debug_Value_3;
	CARDINAL_32	debug_Value_4;
	CARDINAL_32	debug_Value_5;
	CARDINAL_8		*code_Name;
/* END VAR */


	code_Name = "\pGet_Data_Into_Bit_Map";


/* ===
	global_Rect = source_Rect;
	LocalToGlobal(&TOP_LEFT(global_Rect));
	LocalToGlobal(&BOTTOM_RIGHT(global_Rect));
=== */
	
	
	Determine_Information_Of_Device_Containing_Rect(global_Source_Rect
													, &video_Monitor_Driver_Ref
													, &video_Monitor_Slot
													, &video_Monitor_ID
													, &video_Optional_Base_Address
													, &bits_Per_Pixel_Union
													, &bits_Per_Pixel_Greatest
													, &total_Intersected_Screens
													, &rect_Is_Within_A_Screen
													, &video_Pix_Map_Handle
													, &graphics_Device
													, &slot_Information_Valid);
	
	desired_Bit_Depth = bits_Per_Pixel_Union;

	if(bits_Per_Pixel_Union < 2) {
		use_Old_Quick_Draw = TRUE;
	}
	else {
		use_Old_Quick_Draw = FALSE;
		
		GetPort(&source_Port);
		
		/*  */
		/* Check if the current port (the one we will copy the CLUT from, */
		/* was created as a color port or not a color port */
		/*  */
		if ((source_Port->portBits.rowBytes 
							& COLOR_PORT_PORTBITS_CHECK) != COLOR_PORT_PORTBITS_CHECK) {
														
			use_Old_Quick_Draw = TRUE;
			desired_Bit_Depth = 1;
		}
		else {
			/*  */
			/* For color uses we want the deepest CLUT as recommended by apple */
			/*  */
			desired_Bit_Depth = bits_Per_Pixel_Greatest;
		}
	}
	
	if(use_Old_Quick_Draw == TRUE) {

		if (desired_Fill == k_USE_RESOURCE_INFO) {
			if (Load_Pict_Into_BW_Bit_Map(pict_Resource_Id
														, &(offscreen_Ptr->offscreen_Port.old_Style_Port)
														, &(offscreen_Ptr->image_Handle)) != SAFE_TRUE) {
														
				Display_Serious_Error_Message(unit_Name
																	, code_Name
																	, "\pProblem Loading Indicator resources (Memory?) ,1"
																	, "\pMemory not available?");
				goto Error_Handler;
			}
		}
		else {
			local_Rect = *global_Source_Rect;
			GlobalToLocal(&TOP_LEFT(local_Rect));
			GlobalToLocal(&BOTTOM_RIGHT(local_Rect));

			if (Copy_Rect_Into_BW_Bit_Map(&local_Rect
														, &(offscreen_Ptr->offscreen_Port.old_Style_Port)
														, &(offscreen_Ptr->image_Handle)) != SAFE_TRUE) {
														
				Display_Serious_Error_Message(unit_Name
																	, code_Name
																	, "\pProblem Loading Indicator resources (Memory?) ,2"
																	, "\pMemory not available?");
				goto Error_Handler;
			}
		}
	
		offscreen_Ptr->initialized = SAFE_TRUE;
		offscreen_Ptr->offscreen_GDevice_Hdl = nil;
		offscreen_Ptr->color_Type = k_ORIGINAL_QUICKDRAW;
	
	}
	else {
	
	
		
#if !defined(k_USE_DEFAULT_CLUT_RESOURCE)
#error "UNDEFINED PARAMETER!"
#endif
		if (k_USE_DEFAULT_CLUT_RESOURCE == 0) {
			
			if (video_Pix_Map_Handle == nil) {
			/*  */
			/* create Palette, use CLUT of current port (set to puppets window) */
			/* This is dependant on the validity of "TEST_DREAMING_WINDOW_INDEX" */
			/*  */
				GetPort(&source_Port);
/* ====
				if ((**active_Window_List[TEST_DREAMING_WINDOW_INDEX]).structure_Initialized != SAFE_TRUE) {
					GetPort(&source_Port);
				}
				else {
					source_Port	= (**active_Window_List[TEST_DREAMING_WINDOW_INDEX]).status_Window;
				}
==== */
			
				/*  */
				/* Regretfully, if the destination rect is not within one complete device */
				/* we just take the PixMap used by the current port, whatever that is */
				/*  */
				video_Pix_Map_Handle = ((CGrafPtr)source_Port)->portPixMap;
			}	
		
			if (Create_Clut_By_Stealing_From_Color_Port(video_Pix_Map_Handle
														, &(offscreen_Ptr->color_Table_Handle)) != SAFE_TRUE) {
														
					Display_Serious_Error_Message(unit_Name
																		, code_Name
																		, "\pProblem getting CLUT ,1.0"
																		, "\p");
				goto Error_Handler;
			}
			
		}
		else {

			offscreen_Ptr->color_Table_Handle = GetCTable(k_DEFAULT_256_COLOR_SYSTEM_CLUT);
			if (offscreen_Ptr->color_Table_Handle == nil) {
					Display_Serious_Error_Message(unit_Name
																		, code_Name
																		, "\pProblem Loading System CLUT (2.0)"
																		, "\pCorrupted?");
				goto Error_Handler;
			}
			
		}
		
		/*  */
		/* Color table does not need to be locked high. Or at all. */
		/* Lock anyway for paranoia */
		/*  */
		MoveHHi((Handle)offscreen_Ptr->color_Table_Handle);
		HLock((Handle)offscreen_Ptr->color_Table_Handle);
		if (MemError() != noErr) {
				Display_Serious_Error_Message(unit_Name
																	, code_Name
																	, "\pProblem locking handle (2.0)"
																	, "\pCorrupted!");
			goto Error_Handler;
		}	
		
/*  */
/* At this point we HOPE we have the CLUT from the device */
/* containing the global source Rect */
/*  */
/*  */
/*  */
/*  */
/* ====
	debug_Value_1 = (**offscreen_Ptr->color_Table_Handle).ctSize;
	debug_Value_2 = (**offscreen_Ptr->color_Table_Handle).ctTable[0].rgb.red;
	debug_Value_3 = (**offscreen_Ptr->color_Table_Handle).ctTable[0].rgb.blue;
	debug_Value_4 = (**offscreen_Ptr->color_Table_Handle).ctTable[0].rgb.green;
	debug_Value_5 = desired_Bit_Depth;
	Get_Debug("\pHEY  Check CLUT (re : d0");
	asm {
		movem.l		d0, -(SP)
		nop
		nop
		move.l		debug_Value_1, d0
		move.l		debug_Value_2, d0
		move.l		debug_Value_3, d0
		move.l		debug_Value_4, d0
		move.l		debug_Value_5, d0
		movem.l		(sp)+, d0						
	}
=== */
/*  */
/*  */
/*  */
/*  */
/*  */
/*  */
/*  */



		
		if (desired_Fill == k_USE_RESOURCE_INFO) {
			if (Load_Pict_Into_Color_Bit_Map(pict_Resource_Id
																, desired_Bit_Depth
																, offscreen_Ptr->color_Table_Handle
																, (CGrafPtr)&(offscreen_Ptr->offscreen_Port.color_Port)
																, &(offscreen_Ptr->offscreen_GDevice_Hdl)
																, &(offscreen_Ptr->image_Handle)) != SAFE_TRUE) {
														
				Display_Serious_Error_Message(unit_Name
																	, code_Name
																	, "\pProblem Loading pict resources (Memory?) ,3"
																	, "\pMemory not available?");
				goto Error_Handler;
			}
		}
		else {
		
			local_Rect = *global_Source_Rect;
			GlobalToLocal(&TOP_LEFT(local_Rect));
			GlobalToLocal(&BOTTOM_RIGHT(local_Rect));
		
			if (Copy_Rect_Into_Color_Bit_Map(&local_Rect
																, desired_Bit_Depth
																, offscreen_Ptr->color_Table_Handle
																, (CGrafPtr)&(offscreen_Ptr->offscreen_Port.color_Port)
																, &(offscreen_Ptr->offscreen_GDevice_Hdl)
																, &(offscreen_Ptr->image_Handle)) != SAFE_TRUE) {
														
				Display_Serious_Error_Message(unit_Name
																	, code_Name
																	, "\pProblem Loading pict resources (Memory?) ,4"
																	, "\pMemory not available?");
				goto Error_Handler;
			}
		}
		
		offscreen_Ptr->initialized = SAFE_TRUE;
		offscreen_Ptr->color_Type = k_COLOR_QUICKDRAW;
	}
	
	function_Result = SAFE_TRUE;
	goto Normal_Function_Exit;
		
		
Fatal_Memory_Handler:

Error_Handler:
	SysBeep(20);
	Display_Serious_Error_Message(unit_Name
														, code_Name
														, "\pAction aborting because of failure to create offscreen PICT"
														, "\pMemory not available?");
	function_Result = SAFE_FALSE;

Normal_Function_Exit:

	return(function_Result);

}
/*=**************************************************************************=*/


/*=**************************************************************************=*/
Safe_Boolean	Get_Picts_Into_One_Bit_Map(Off_Screen_Ptr offscreen_Ptr
													, INTEGER_16 first_Pict_Id
													, INTEGER_16 total_Picts
													, Rect *frame_Locations
													, Rect *governing_Rect
													, CARDINAL_32	expected_Total_Pixel_Height
													, CARDINAL_32	expected_Total_Depth_If_Not_Main_Screen
													, Rect *expected_Target_Rect)
/*  */
/* This loads a sequence of PICTs (from a PICS file) into one */
/* large tall graphics port. It stores the bounding rects of the */
/* Picts into an array that is passed to it. It clips the picts to a */
/* maximum controlling rect governed by the rect passed to it */
/* called "governing_Rect" */
/* Deficiency... is expecting current port to be set */
/* "expected_Total_Depth_If_Not_Main_Screen" honored only if > 0 */
/*  */
{
	GrafPtr				source_Port;
	CARDINAL_32	desired_Bit_Depth;
	Safe_Boolean		function_Result;
	Boolean				use_Old_Quick_Draw;
	INTEGER_32		current_Frame_Offset;
	INTEGER_32		pict_Resource_Id;
	Rect 					local_Rect;
	CARDINAL_16  	total_Intersected_Screens;
	INTEGER_32	 	video_Bits_Per_Pixel_Union;
	INTEGER_32	 	video_Bits_Per_Pixel_Greatest;
	Safe_Boolean		rect_Is_Within_A_Screen;
	INTEGER_16		video_Monitor_Driver_Ref;
	INTEGER_8			video_Monitor_ID;
	INTEGER_8			video_Monitor_Slot;
	CARDINAL_32	video_Optional_Base_Address;
	PixMapHandle	video_Pix_Map_Handle;
	Safe_Boolean		slot_Information_Valid;													
	
	
	CARDINAL_32	debug_Value_1;
	CARDINAL_32	debug_Value_2;
	CARDINAL_32	debug_Value_3;
	CARDINAL_32	debug_Value_4;
	CARDINAL_32	debug_Value_5;
	
	Rect						offscreen_Rect;
	INTEGER_32		current_Vertical_Offset;
	OSErr					os_Error;
	GrafPtr				saved_Port;
	GDHandle			saved_Device;
	PicHandle			source_Pict_Hdl;
	Rect						source_Bit_Rect;
	INTEGER_32		bytes_Per_Row;
	CARDINAL_32	total_Graphics_Storage;
	GrafPtr				offscreen_Graph_Ptr;
	Safe_Boolean		ports_Are_Still_Swapped;
	GDHandle			graphics_Device;											
	CARDINAL_8		*code_Name;

	Str255					error_String_1;
	Str255					error_String_2;
/* END VAR */


	code_Name = "\pGet_Picts_Into_One_Bit_Map";
	ports_Are_Still_Swapped = SAFE_FALSE;
	
	Determine_Information_Of_Device_Containing_Rect(expected_Target_Rect
													, &video_Monitor_Driver_Ref
													, &video_Monitor_Slot
													, &video_Monitor_ID
													, &video_Optional_Base_Address
													, &video_Bits_Per_Pixel_Union
													, &video_Bits_Per_Pixel_Greatest
													, &total_Intersected_Screens
													, &rect_Is_Within_A_Screen
													, &video_Pix_Map_Handle
													, &graphics_Device
													, &slot_Information_Valid);
	
	desired_Bit_Depth = video_Bits_Per_Pixel_Union;

	if(video_Bits_Per_Pixel_Union < 2) {
		use_Old_Quick_Draw = TRUE;
	}
	else {
		use_Old_Quick_Draw = FALSE;
		
		GetPort(&source_Port);
		
		/*  */
		/* Check if the current port (the one we will copy the CLUT from, */
		/* was created as a color port or not a color port */
		/*  */
		if ((source_Port->portBits.rowBytes 
							& COLOR_PORT_PORTBITS_CHECK) != COLOR_PORT_PORTBITS_CHECK) {
														
			use_Old_Quick_Draw = TRUE;
			desired_Bit_Depth = 1;
		}
		else {
			/*  */
			/* For color uses we want the deepest CLUT as recommended by apple */
			/*  */
			desired_Bit_Depth = video_Bits_Per_Pixel_Greatest;
		}
	}
	
	
	if (expected_Total_Depth_If_Not_Main_Screen != 0) {
		desired_Bit_Depth = expected_Total_Depth_If_Not_Main_Screen;
		use_Old_Quick_Draw = FALSE;
	}
	
	/*  */
	/* Equalize governing rect just for safety */
	/*  */
	governing_Rect->right = (governing_Rect->right - governing_Rect->left);
	governing_Rect->left = 0;
	governing_Rect->bottom = (governing_Rect->bottom - governing_Rect->top);
	governing_Rect->top = 0;
	
	offscreen_Rect = *governing_Rect;
	offscreen_Rect.bottom = expected_Total_Pixel_Height;
	
	current_Vertical_Offset = 0;
	
	if (use_Old_Quick_Draw != TRUE) {


		if (desired_Bit_Depth <= 8) {

#if !defined(k_USE_DEFAULT_CLUT_RESOURCE)
#error "UNDEFINED PARAMETER!"
#endif
			if (k_USE_DEFAULT_CLUT_RESOURCE == 0) {
				
				if (video_Pix_Map_Handle == nil) {
				/*  */
				/* create Palette, use CLUT of current port (set to puppets window) */
				/* This is dependant on the validity of "TEST_DREAMING_WINDOW_INDEX" */
				/*  */
					GetPort(&source_Port);
	/* ====
					if ((**active_Window_List[TEST_DREAMING_WINDOW_INDEX]).structure_Initialized != SAFE_TRUE) {
						GetPort(&source_Port);
					}
					else {
						source_Port	= (**active_Window_List[TEST_DREAMING_WINDOW_INDEX]).status_Window;
					}
	==== */
				
					/*  */
					/* Regretfully, if the destination rect is not within one complete device */
					/* we just take the PixMap used by the current port, whatever that is */
					/*  */
					video_Pix_Map_Handle = ((CGrafPtr)source_Port)->portPixMap;
				}	
			
				if (Create_Clut_By_Stealing_From_Color_Port(video_Pix_Map_Handle
															, &(offscreen_Ptr->color_Table_Handle)) != SAFE_TRUE) {
															
						Display_Serious_Error_Message(unit_Name
																			, code_Name
																			, "\pProblem getting CLUT ,1"
																			, "\p");
					goto Error_Handler;
				}
				
				
			}
			else {
	
				offscreen_Ptr->color_Table_Handle = GetCTable(k_DEFAULT_256_COLOR_SYSTEM_CLUT);
				if (offscreen_Ptr->color_Table_Handle == nil) {
						Display_Serious_Error_Message(unit_Name
																			, code_Name
																			, "\pProblem Loading System CLUT (2)"
																			, "\pCorrupted?");
					goto Error_Handler;
				}
				
			}
		
		} /* if (desired_Bit_Depth <= 8) */
		else {
				if (Create_Clut_For_Direct_Mapping(expected_Total_Depth_If_Not_Main_Screen
															, &(offscreen_Ptr->color_Table_Handle)) != SAFE_TRUE) {
															
						Display_Serious_Error_Message(unit_Name
																			, code_Name
																			, "\pProblem creating direct map CLUT"
																			, "\p");
					goto Error_Handler;
				}
		}
		
		/*  */
		/* Color table does not need to be locked high. Or at all. */
		/* Lock anyway for paranoia */
		/*  */
		MoveHHi((Handle)offscreen_Ptr->color_Table_Handle);
		HLock((Handle)offscreen_Ptr->color_Table_Handle);
		if (MemError() != noErr) {
				Display_Serious_Error_Message(unit_Name
																	, code_Name
																	, "\pProblem locking handle (2)"
																	, "\pCorrupted!");
			goto Error_Handler;
		}	

	/*  */
	/* At this point we HOPE we have the CLUT from the device */
	/* containing the global source Rect */
	/*  */
	/*  */
	/*  */
	/*  */
	/* ====
		debug_Value_1 = (**offscreen_Ptr->color_Table_Handle).ctSize;
		debug_Value_2 = (**offscreen_Ptr->color_Table_Handle).ctTable[0].rgb.red;
		debug_Value_3 = (**offscreen_Ptr->color_Table_Handle).ctTable[0].rgb.blue;
		debug_Value_4 = (**offscreen_Ptr->color_Table_Handle).ctTable[0].rgb.green;
		debug_Value_5 = desired_Bit_Depth;
		Get_Debug("\pHEY  Check CLUT (re : d0");
		asm {
			movem.l		d0, -(SP)
			nop
			nop
			move.l		debug_Value_1, d0
			move.l		debug_Value_2, d0
			move.l		debug_Value_3, d0
			move.l		debug_Value_4, d0
			move.l		debug_Value_5, d0
			movem.l		(sp)+, d0						
		}
	=== */
	/*  */
	/*  */
	/*  */
	/*  */
	/*  */
	/*  */
	/*  */
		
		os_Error= Create_Off_Screen_Port(&offscreen_Rect
																, desired_Bit_Depth
																, offscreen_Ptr->color_Table_Handle
																, (CGrafPtr)&(offscreen_Ptr->offscreen_Port.color_Port)
																, &(offscreen_Ptr->offscreen_GDevice_Hdl)
																, &(offscreen_Ptr->image_Handle));
	
		if (os_Error != noErr) {
			Display_Serious_Error_Message(unit_Name
																, code_Name
																, "\pProblem Creating a large offscreen color port"
																, "\pMemory not available for sequence?");
		
			goto Error_Handler;
		}
			
		GetPort(&saved_Port);
		saved_Device = GetGDevice();
		
		Set_View_Port((GrafPtr)&(offscreen_Ptr->offscreen_Port.color_Port));
		Set_View_Device((offscreen_Ptr->offscreen_GDevice_Hdl));
		ports_Are_Still_Swapped = SAFE_TRUE;

		/*  */
		/* For safety to clear out the memory */
		/*  */
		EraseRect(&offscreen_Rect); 
		
		/*  */
		/* Only needed really for CopyBits calls but why not be certain. */
		/*  */
		ForeColor(blackColor);
		BackColor(whiteColor);
	
	} /* if (use_Old_Quick_Draw != TRUE) */
	else {
			
		/*  */
		/* Round up to nearest long word and even align it */
		/* 32 bits wide  == 4 bytes total */
		/* Quickdraw qorks with even alignment but is FASTER for 4 byte */
		/*  */
		bytes_Per_Row = (((governing_Rect->right + 31) >> 5) << 2); 
		
		/*  */
		/* In the future, If we were safety-minded, we would multiply  */
		/* the byte width by the height to see if result was too big to fit */
		/* simple structures. No big deal.  */
		/*  */
		total_Graphics_Storage = expected_Total_Pixel_Height * bytes_Per_Row;
		offscreen_Ptr->image_Handle = NewHandle(total_Graphics_Storage);
		if ((MemError() != noErr) || (offscreen_Ptr->image_Handle == nil)) {
			if (ports_Are_Still_Swapped == SAFE_TRUE) {
				if (use_Old_Quick_Draw == TRUE) {
						Set_View_Port(saved_Port);
				} 
				else {
						Set_View_Port(saved_Port);
						Set_View_Device(saved_Device);
				} /* NOT  if(use_Old_Quick_Draw == TRUE) */
				ports_Are_Still_Swapped = SAFE_FALSE;
			}

			Display_Serious_Error_Message(unit_Name
																, code_Name
																, "\pProblem Creating a large offscreen B&W port"
																, "\pMemory not available for sequence?");
			goto Error_Handler;
		}
		
		offscreen_Graph_Ptr = &(offscreen_Ptr->offscreen_Port.old_Style_Port);
		OpenPort(offscreen_Graph_Ptr);
		
		MoveHHi(offscreen_Ptr->image_Handle);
		if (MemError() != noErr) {
			goto Fatal_Memory_Handler;
		}
		
		/*  */
		/* Sadly, must remain locked forever until port is no longer needed */
		/*  */
		HLock(offscreen_Ptr->image_Handle);
		if (MemError() != noErr) {
			goto Fatal_Memory_Handler;
		}
		
		(*offscreen_Graph_Ptr).portBits.baseAddr = (Ptr)(*(offscreen_Ptr->image_Handle));
		(*offscreen_Graph_Ptr).portBits.rowBytes = bytes_Per_Row;
		(*offscreen_Graph_Ptr).portBits.bounds.left = 0;
		(*offscreen_Graph_Ptr).portBits.bounds.top = 0;
		(*offscreen_Graph_Ptr).portBits.bounds.right = governing_Rect->right;
		(*offscreen_Graph_Ptr).portBits.bounds.bottom = expected_Total_Pixel_Height;
		
	/*  */
	/* HACK HACK HACK WAS NOT HERE before 1995.01 */
	/*  */
		(*offscreen_Graph_Ptr).portRect = (*offscreen_Graph_Ptr).portBits.bounds;

				
		GetPort(&saved_Port);
		Set_View_Port(offscreen_Graph_Ptr);
		ports_Are_Still_Swapped = SAFE_TRUE;
		
		/*  */
		/* Only needed really for CopyBits calls but why not be certain. */
		/*  */
		ForeColor(blackColor);
		BackColor(whiteColor);
	
		/*  */
		/* For safety to clear out the memory */
		/*  */
		EraseRect(&((*offscreen_Graph_Ptr).portBits.bounds)); 
	
	} /* NOT if (use_Old_Quick_Draw != TRUE) */
							
	current_Frame_Offset = 0;
						
	for (pict_Resource_Id = first_Pict_Id
															; (pict_Resource_Id < (first_Pict_Id + total_Picts))
															; pict_Resource_Id++ ) {
					
		source_Pict_Hdl = nil;
		source_Pict_Hdl = GetPicture(pict_Resource_Id);
		if ((ResError() != noErr) || (source_Pict_Hdl == nil)) {
	
			if (ports_Are_Still_Swapped == SAFE_TRUE) {
				if (use_Old_Quick_Draw == TRUE) {
						Set_View_Port(saved_Port);
				} 
				else {
						Set_View_Port(saved_Port);
						Set_View_Device(saved_Device);
				} /* NOT  if(use_Old_Quick_Draw == TRUE) */
				ports_Are_Still_Swapped = SAFE_FALSE;
			}
						
			Make_String_Long_String(error_String_1
														, "\pProblem importing PICT resource. Resource ID : "
														, first_Pict_Id
														, NO_LIMIT
														,"\p, Total Picts : ");
														
			Make_String_Long_String(error_String_2
														, error_String_1
														, total_Picts
														, NO_LIMIT
														,"\p.");
														
			Display_Serious_Error_Message(unit_Name
																, code_Name
																, error_String_2
																, "\pResource not available? Low Memory?");
			goto Error_Handler;
		}
			
		source_Bit_Rect = (*source_Pict_Hdl)->picFrame;
		/*  */
		/* Equalize Rect just in case */
		/*  */
		source_Bit_Rect.right = (source_Bit_Rect.right - source_Bit_Rect.left);
		source_Bit_Rect.left = 0;
		source_Bit_Rect.bottom = (source_Bit_Rect.bottom - source_Bit_Rect.top);
		source_Bit_Rect.top = 0;
					
		OffsetRect(&source_Bit_Rect, 0, current_Vertical_Offset);
		
		DrawPicture(source_Pict_Hdl, &source_Bit_Rect);
		current_Vertical_Offset += (source_Bit_Rect.bottom - source_Bit_Rect.top);

		/*  */
		/* Release resource */
		/*  */
		if (source_Pict_Hdl != nil) {
			ReleaseResource((Handle)source_Pict_Hdl);
		}
					
		frame_Locations[current_Frame_Offset] = source_Bit_Rect;
		current_Frame_Offset += 1;

	} /* for (pict_Resource_Id = first_Pict_Id */
		
	if (use_Old_Quick_Draw == TRUE) {
			offscreen_Ptr->initialized = SAFE_TRUE;
			offscreen_Ptr->offscreen_GDevice_Hdl = nil;
			offscreen_Ptr->color_Type = k_ORIGINAL_QUICKDRAW;

	} 
	else {
			offscreen_Ptr->initialized = SAFE_TRUE;
			offscreen_Ptr->color_Type = k_COLOR_QUICKDRAW;
	
	} /* NOT  if(use_Old_Quick_Draw == TRUE) */

	
	function_Result = SAFE_TRUE;
	goto Normal_Function_Exit;
		
		
Fatal_Memory_Handler:

Error_Handler:

	if (ports_Are_Still_Swapped == SAFE_TRUE) {
		if (use_Old_Quick_Draw == TRUE) {
				Set_View_Port(saved_Port);
		} 
		else {
				Set_View_Port(saved_Port);
				Set_View_Device(saved_Device);
		} /* NOT  if(use_Old_Quick_Draw == TRUE) */
		ports_Are_Still_Swapped = SAFE_FALSE;
	}

	SysBeep(20);
	Display_Serious_Error_Message(unit_Name
														, code_Name
														, "\pAction aborting because of failure to create offscreen PICT"
														, "\pMemory not available?");
	function_Result = SAFE_FALSE;

Normal_Function_Exit:
	if (ports_Are_Still_Swapped == SAFE_TRUE) {
		if (use_Old_Quick_Draw == TRUE) {
				Set_View_Port(saved_Port);
		} 
		else {
				Set_View_Port(saved_Port);
				Set_View_Device(saved_Device);
		} /* NOT  if(use_Old_Quick_Draw == TRUE) */
		ports_Are_Still_Swapped = SAFE_FALSE;
	}
	
	return(function_Result);

}
/*=**************************************************************************=*/


/*=**************************************************************************=*/
/******************************************************************************\
* NAME & SYNOPSIS:
* RestoreColorsPalette: Restore all screens to the default color table
*
* DESCRIPTION:
* To change the color table of any screen using the Palette Manager, a window
* must be used.  But all we want to do is reset the color tables, not display a
* window.  The obvious solution is to make a very small, inconspicuous window
* thats tough to notice, and thats what RestoreColorsPalette does.  A 1 pixel
* by 1 pixel is created, and a 1-entry palette is made and attached to the
* window, just so that it has a palette that can be set to the right size and
* colors later.
*
* Each screen can have different depths, and each can be gray-scale or colors,
* so every screen has have its colors set individually.  Since all we have is a
* 1 pixel by 1 pixel window, theres not much choice here anyway.  To run
* through each screen, the GDevice list is used.  The GDevice list is a linked
* list of GDevice records that the system maintains, and theres one GDevice
* record for every screen thats attached to a Macintosh.  Each screens GDevice
* can be used to find out the screens depth, and whether its set to Grays or
* Colors mode.  See the Graphics Device Manager chapter of Inside Macintosh
* Volume VI for details. GetDeviceList is used to find the first GDevice in the
* GDevice list, and GetNextDevice is used to get the next GDevice list the list.
*
* For each GDevice in the list, its first tested to see if it belongs to an
* active screen.  If it doesnt, then we just ignore it and go to the next
* GDevice in the list.  If it does, then the GDevices PixMap is checked to see
* what the depth of the screen is.  This depth can be passed to the GetCTable
* routine (see the Color Manager chapter of Inside Macintosh Volume V for
* details about this routine) to get a color table filled with the default
* colors for the screen depth.  As of 32-Bit QuickDraw 1.0 and/or system
* software version 6.0.5, the color table of a screen is guaranteed to contain
* the highlight color, and if you add 64 to the pixel depth and pass the result
* to GetCTable, GetCTable returns a color table filled with the default colors
* including the highlight color, and so thats what I pass to GetCTable for a
* screen set to Colors mode.  For gray-scale screens, you add 32 to the pixel
* depth to have GetCTable return a color table filled with the default gray-
* scale color table.
*
* I pass whatever color table GetCTable returns to CTab2Palette, and I use a
* usage mode of pmTolerant + pmExplicit.  This mode is just like pmTolerant, but
* it guarantees that the palette colors are put into the screens color table in
* the same order that they appear in the palette when that palette is activated.
* If you just use pmTolerant, the order of colors in the screens color table
* will likely be scrambled, so the screen has the default colors, but not in the
* default order.
*
* Once the palette is set up with the default colors for the screen were
* working on, the 1 pixel by 1 pixel window is moved to the extreme upper-left
* corner of the screen, then its shown and immediately hidden.  The moment the
* window is shown, the screens color table is filled with the default colors in
* the default order.  Then we move on to the next screen until all screens have
* been handled.
*
\******************************************************************************/
void Restore_Colors_Palette(void)
{
	WindowPtr     tinyWindow;    /* Pointer to tiny window */
	CTabHandle    defaultColors; /* Default color table for the screen */
	PaletteHandle tinyPalette;   /* Palette to express defaultColors */
	Rect          tinyRect;      /* Rectangle of tiny window */
	GDHandle      screenDevice;  /* Handle to each screens GDevice */
	Point         screenCorner;  /* Global coordinate of top-left of each screen */
	short         clutID;        /* ID of table of default colors */

/*  */
/* Make the tiny window for attaching our palette */
/*  */
	SetRect( &tinyRect, 0, 0, 1, 1 );
	tinyWindow = NewCWindow(
			nil,
			&tinyRect,
			(StringPtr)"\P",
			false,
			plainDBox,
			(WindowPtr)-1,
			false,
			0L );

/*  */
/* Make a palette; its just a placeholder for the moment */
/*  */
	tinyPalette = NewPalette( 1, nil, pmTolerant + pmExplicit, 0 );
	SetPalette( tinyWindow, tinyPalette, false );

	/* Loop through each screen GDevice to reset its screens colors */
	screenDevice = GetDeviceList();
	while (screenDevice != nil) {
		if (TestDeviceAttribute( screenDevice, screenActive )) {
		
		/*  */
		/* Grab default color table for the screen */
		/*  */
			clutID = (**(**screenDevice).gdPMap).pixelSize;
			if (TestDeviceAttribute (screenDevice, gdDevType ))
				clutID += 64;
			else
				clutID += 32;
			defaultColors = GetCTable( clutID );

		/*  */
		/* Convert default screen colors to a tolerant palette */
		/*  */
			CTab2Palette( defaultColors, tinyPalette, pmTolerant + pmExplicit, 0 );

		/*  */
		/* Dont need that clut any more */
		/*  */
			DisposeCTable( defaultColors );
			
		/*  */
		/* Move the tiny window into the upper left corner of the screen */
		/*  */
			screenCorner = topLeft( (**screenDevice).gdRect );
			MoveWindow( tinyWindow, screenCorner.h, screenCorner.v, false );
			ShowWindow( tinyWindow );
			HideWindow( tinyWindow );
		}

		/*  */
		/* Go to the next screen in the device list */
		/*  */
		screenDevice = GetNextDevice( screenDevice );
	}
	
	/*  */
	/* Get rid of the window and palette */
	/*  */
	DisposePalette( tinyPalette );
	DisposeWindow( tinyWindow );
	
}
/*=**************************************************************************=*/


/*=**************************************************************************=*/
/******************************************************************************\
* NAME & SYNOPSIS:
* RestoreColorsSlam: Restore all screens to the default color table
*
* DESCRIPTION:
* 32-Bit QuickDraw 1.0 and system software version 6.0.5 introduced a new
* routine called RestoreDeviceClut which sets the color table of specific
* screens or all the screens to the default set of colors.  RestoreColorsSlam
* calls RestoreDeviceClut with a nil parameter, which restores the color table
* of all attached screens.  At this point, the screen could appear in strange
* colors because RestoreDeviceClut doesnt cause update events to cause the
* screen to be redrawn in the new colors.  So, RestoreColorsSlam forces all the
* screens to be redrawn by calling the Window Manager routine, PaintBehind.
* This causes the entire desktop area and all windows to be redrawn.  The only
* part of the screen that isnt guaranteed to be redrawn is the menu bar, so
* DrawMenuBar is called to make sure the menu bar is redrawn in the new colors.
*
\******************************************************************************/
void Restore_Screen_Colors_To_Environment(void)
{
	RestoreDeviceClut( nil );
	PaintBehind( nil, GetGrayRgn() );
	DrawMenuBar();
}
/*=**************************************************************************=*/
	
	
/*=**************************************************************************=*/				
Safe_Boolean Copy_Bitmap_Into_Pict_Command(Off_Screen_Port_Rec *port_Ptr
																, Rect *source_Bit_Rect_Ptr
																, INTEGER_32		desired_Color_Depth
																, PicHandle			*picture_Handle
																, Boolean			tiny_Preview_Size_Desired)
{
	INTEGER_32					source_Bit_Width;
	INTEGER_32					source_Bit_Height;
	INTEGER_32					preview_Width;
	INTEGER_32					preview_Height;
	
	Safe_Boolean					port_Was_Set;
	Off_Screen_Ptr				offscreen_Ptr;
	GrafPtr							saved_Original_Port;
	PenState							saved_Pen_State;
	GDHandle						saved_Device;
	Rect									source_Bit_Rect;
	Rect									offscreen_Rect;
	Rect									recording_Area_Rect;

	CARDINAL_8					*code_Name;
	OSErr								os_Error;
	Str255								resource_Comment;
	Safe_Boolean					function_Result;
	Off_Screen_Port_Rec		our_Temp_Port;
	SPEED_DIGITS_REAL		desired_Rough_Ratio;
	SPEED_DIGITS_REAL		other_Ratio;
/* END VAR */


	code_Name = "\pCopy_Bitmap_Into_Pict_Command";
	function_Result = SAFE_FALSE;
	port_Was_Set = SAFE_FALSE;

	/*  */
	/* Copy in case pointer into an unlocked handle */
	/*  */
	source_Bit_Rect = *source_Bit_Rect_Ptr;
	
	source_Bit_Width = source_Bit_Rect.right - source_Bit_Rect.left;
	source_Bit_Height = source_Bit_Rect.bottom - source_Bit_Rect.top;
	
	if (tiny_Preview_Size_Desired == TRUE) {
		
		preview_Width = k_PREVIEW_WIDTH_MAXIMUM;
		preview_Height = k_PREVIEW_HEIGHT_MAXIMUM;
		
		if ((source_Bit_Width > preview_Width) || (source_Bit_Height > preview_Height)) {
		
			if (source_Bit_Width <= preview_Width) {
				desired_Rough_Ratio = (SPEED_DIGITS_REAL)(preview_Height * 1.0) / (SPEED_DIGITS_REAL)(source_Bit_Height * 1.0);
			}
			else {
				desired_Rough_Ratio = (SPEED_DIGITS_REAL)(preview_Width * 1.0) / (SPEED_DIGITS_REAL)(source_Bit_Width * 1.0);

				if (source_Bit_Height > preview_Height) {
					other_Ratio = ((SPEED_DIGITS_REAL)(preview_Height * 1.0) / (SPEED_DIGITS_REAL)(source_Bit_Height * 1.0));
					if (desired_Rough_Ratio > other_Ratio) {
						desired_Rough_Ratio = other_Ratio;
					}
				}
			}
		
			source_Bit_Width = (INTEGER_32)(desired_Rough_Ratio * ((SPEED_DIGITS_REAL)source_Bit_Width * 1.0));
			source_Bit_Height = (INTEGER_32)(desired_Rough_Ratio * ((SPEED_DIGITS_REAL)source_Bit_Height * 1.0));
			
			if (source_Bit_Width > preview_Width) {
				source_Bit_Width = preview_Width;
			}
			
			if (source_Bit_Height > preview_Height) {
				source_Bit_Height = preview_Height;
			}
			
			if (source_Bit_Height < 1) {
				source_Bit_Height = 1;
			}
			if (source_Bit_Width < 1) {
				source_Bit_Width = 1;
			}			
		}
	}
	
	SetRect(&offscreen_Rect
					, 0
					, 0
					, source_Bit_Width
					, source_Bit_Height);
					
					
	offscreen_Ptr = &our_Temp_Port;
	offscreen_Ptr->initialized = SAFE_FALSE;
	offscreen_Ptr->offscreen_GDevice_Hdl = nil;
	offscreen_Ptr->image_Handle = nil;
	offscreen_Ptr->color_Table_Handle = nil;
	offscreen_Ptr->color_Type = k_32_BIT_COLOR_QUICKDRAW;		 


	if (desired_Color_Depth <= 8) {
	
		offscreen_Ptr->color_Table_Handle = port_Ptr->color_Table_Handle;
		
		HandToHand((Handle *)&(offscreen_Ptr->color_Table_Handle));
		if (offscreen_Ptr->color_Table_Handle == nil) {
			goto Error_Handler;
		}
	} /* if (desired_Color_Depth <= 8) */
	else {
		/*  */
		/* Self repair paranoia */
		/*  */
		if (Create_Clut_For_Direct_Mapping(desired_Color_Depth, &(offscreen_Ptr->color_Table_Handle)) != SAFE_TRUE) {
 			goto Error_Handler;
		}
	
	}


	MoveHHi((Handle)offscreen_Ptr->color_Table_Handle);
	HLock((Handle)offscreen_Ptr->color_Table_Handle);
	if (MemError() != noErr) {
			Display_Serious_Error_Message(unit_Name
																, code_Name
																, "\pProblem locking handle (2)"
																, "\pCorrupted!");
		goto Error_Handler;
	}
								
	os_Error= Create_Off_Screen_Port(&offscreen_Rect
															, desired_Color_Depth
															, offscreen_Ptr->color_Table_Handle
															, (CGrafPtr)&(offscreen_Ptr->offscreen_Port.color_Port)
															, &(offscreen_Ptr->offscreen_GDevice_Hdl)
															, &(offscreen_Ptr->image_Handle));

	if (os_Error != noErr) {
		goto Error_Handler;
	}
	
	offscreen_Ptr->initialized = SAFE_TRUE;
	offscreen_Ptr->color_Type = k_COLOR_QUICKDRAW;
	


	GetPort(&saved_Original_Port);
	saved_Device = GetGDevice();

	Set_View_Port((GrafPtr)&(offscreen_Ptr->offscreen_Port.color_Port));
	Set_View_Device(offscreen_Ptr->offscreen_GDevice_Hdl);

	port_Was_Set = SAFE_TRUE;
	
	GetPenState (&saved_Pen_State);

	/*  */
	/* Copy picture to an erased offscreen port, preferably using */
	/* the original background color used for framing */
	/*  */
				
	/*  */
	/* Set background erasing color */
	/*  */
	ForeColor(blackColor);
	BackColor(whiteColor);

	/*  */
	/* erase port rect completely */
	/*  */
	ClipRect(&(offscreen_Ptr->offscreen_Port.color_Port.portRect));
	EraseRect(&(offscreen_Ptr->offscreen_Port.color_Port.portRect));

	/*  */
	/* Actually needed for CopyBits calls , so be certain. */
	/*  */
	ForeColor(blackColor);
	BackColor(whiteColor);
														
	ClipRect(&offscreen_Rect);
	
	if (tiny_Preview_Size_Desired == TRUE) {
	
	
	}

	
	switch (port_Ptr->color_Type) {
		case k_COLOR_QUICKDRAW:
		case k_32_BIT_COLOR_QUICKDRAW:
			CopyBits ((const BitMap *)*(port_Ptr->offscreen_Port.color_Port.portPixMap)
							, (const BitMap *)*(offscreen_Ptr->offscreen_Port.color_Port.portPixMap)
							, &source_Bit_Rect
							, &offscreen_Rect
							, srcCopy
							, nil);

		break;
		
		case k_ORIGINAL_QUICKDRAW:
		default:
		
			CopyBits (&(port_Ptr->offscreen_Port.old_Style_Port.portBits)
							, &(offscreen_Ptr->offscreen_Port.old_Style_Port.portBits)
							, &source_Bit_Rect
							, &offscreen_Rect
							, srcCopy
							, nil);
					
	} /*  switch (g_Indicator.color_Type) */
			
	/*  */
	/* start recording (record copy from and to itself) otherwise would be LARGE! */
	/*  */
	recording_Area_Rect = offscreen_Rect;
	*picture_Handle = OpenPicture(&recording_Area_Rect );
	
	switch (port_Ptr->color_Type) {
		case k_COLOR_QUICKDRAW:
		case k_32_BIT_COLOR_QUICKDRAW:
			CopyBits ((const BitMap *)*(offscreen_Ptr->offscreen_Port.color_Port.portPixMap)
							, (const BitMap *)*(offscreen_Ptr->offscreen_Port.color_Port.portPixMap)
							, &offscreen_Rect
							, &offscreen_Rect
							, srcCopy
							, nil);

		break;
		
		case k_ORIGINAL_QUICKDRAW:
		default:
		
			CopyBits (&(offscreen_Ptr->offscreen_Port.old_Style_Port.portBits)
							, &(offscreen_Ptr->offscreen_Port.old_Style_Port.portBits)
							, &offscreen_Rect
							, &offscreen_Rect
							, srcCopy
							, nil);
					
	} /*  switch (g_Indicator.color_Type) */

	ClosePicture();	
		
	function_Result = SAFE_TRUE;
	goto Normal_Function_Exit;

Error_Handler:
	Display_Serious_Error_Message(unit_Name, code_Name, "\pInternal problem converting pict", "\p");
	function_Result = SAFE_FALSE;
		
Normal_Function_Exit:
	
	if (port_Was_Set == SAFE_TRUE) {
		SetPenState (&saved_Pen_State);
		Set_View_Port(saved_Original_Port);
		Set_View_Device(saved_Device);
		Close_Offscreen_Pict_Port(offscreen_Ptr);
	}
	
	return(function_Result);
	
} /* Safe_Boolean Copy_Bitmap_Into_Pict_Command ... */
/*=**************************************************************************=*/


/*=**************************************************************************=*/
void Test_Off_Screen_Routines(void)
{
	CGrafPtr			gray_Offscreen_Port;
	GDHandle		gray_Device;
	CGrafPtr			color_Offscreen_Port;
	GDHandle		color_Device;
	GrafPtr			savedPort;
	GDHandle		savedDevice;
	CTabHandle	offscreen_Colors;
	Rect					offscreen_Rect;
	Rect					circle_Rect;
	INTEGER_16	loop;
	RGBColor		temp_Color;
	CGrafPort		full_Color_Port;
	CGrafPort		gray_Color_Port;
	Handle			gray_Bit_Map_Handle;
	Handle			color_Bit_Map_Handle;
	OSErr				os_Error;	
/* END VAR */
	
	
	gray_Bit_Map_Handle = nil;
	color_Bit_Map_Handle = nil;
	
	gray_Offscreen_Port = &gray_Color_Port;
	color_Offscreen_Port = &full_Color_Port;
	
	
	/* Set up the rectangle for the off-screen graphics environments */
	SetRect(&offscreen_Rect, 0, 0, 256, 256);

	/* Get the color table for the gray off-screen graphics environment */
	offscreen_Colors = GetCTable(k_GRAY_CLUT_ID);

	/* Create the gray off-screen graphics environment */
	os_Error = Create_Off_Screen_Port(&offscreen_Rect, k_TEST_OFFSCREEN_DEPTH, offscreen_Colors,
			gray_Offscreen_Port, &gray_Device, &gray_Bit_Map_Handle);

	if (os_Error == noErr)
	{
		/* Get the color table for the color off-screen graphics environment */
		offscreen_Colors = GetCTable(k_COLOR_CLUT_ID);

		/* Create the color off-screen graphics environment */
		os_Error = Create_Off_Screen_Port(&offscreen_Rect, k_TEST_OFFSCREEN_DEPTH, offscreen_Colors,
				color_Offscreen_Port, &color_Device, &color_Bit_Map_Handle);

		if (os_Error == noErr)
		{
			/* Save the current graphics environment */
			GetPort(&savedPort);
			savedDevice = GetGDevice();

			/* Set the current graphics environment to the gray one */
			Set_View_Port((GrafPtr)gray_Offscreen_Port);
			Set_View_Device(gray_Device);

			/* Draw gray-scale ramp into the gray off-screen environment */
			for (loop = 0; loop < 256; ++loop)
			{
				temp_Color.red = temp_Color.green = temp_Color.blue = loop * 257;
				RGBForeColor(&temp_Color);
				MoveTo(0, loop);
				LineTo(255, loop);
			}

			/* Copy gray ramp into color off-screen colorized with green */
			Set_View_Port((GrafPtr)color_Offscreen_Port);
			Set_View_Device(color_Device);
			temp_Color.red = 0x0000; temp_Color.green = 0xFFFF; temp_Color.blue = 0x0000;
			RGBForeColor(&temp_Color);
			CopyBits(&((GrafPtr)gray_Offscreen_Port)->portBits,
					&((GrafPtr)color_Offscreen_Port)->portBits,
					&gray_Offscreen_Port->portRect,
					&color_Offscreen_Port->portRect,
					srcCopy | ditherCopy, nil);

			/* Draw red, green, and blue circles */
			PenSize(8, 8);
			temp_Color.red = 0xFFFF; temp_Color.green = 0x0000; temp_Color.blue = 0x0000;
			RGBForeColor(&temp_Color);
			
			circle_Rect = color_Offscreen_Port->portRect;
			FrameOval(&circle_Rect);
			temp_Color.red = 0x0000; temp_Color.green = 0xFFFF; temp_Color.blue = 0x0000;
			RGBForeColor(&temp_Color);
			InsetRect(&circle_Rect, 20, 20);
			FrameOval(&circle_Rect);
			temp_Color.red = 0x0000; temp_Color.green = 0x0000; temp_Color.blue = 0xFFFF;
			RGBForeColor(&temp_Color);
			InsetRect(&circle_Rect, 20, 20);
			FrameOval(&circle_Rect);

			/* Copy the color off-screen environment to the current port */
			Set_View_Port(savedPort);
			Set_View_Device(savedDevice);
			
			CopyBits(&((GrafPtr)color_Offscreen_Port)->portBits, &savedPort->portBits,
					&color_Offscreen_Port->portRect, &savedPort->portRect,
					srcCopy, nil);

			if ((gray_Bit_Map_Handle != nil) && (*gray_Bit_Map_Handle)) {
				/* Dispose of the off-screen graphics environments */
				Dispose_Off_Screen_Port(gray_Offscreen_Port, gray_Device);
				
				DisposeHandle(gray_Bit_Map_Handle);
				if (MemError() != noErr) {
					/*  */
					/* We are closing up anyway. Ignore this serious error. */
					/*  */
				}
			}
			
			
			if ((color_Bit_Map_Handle != nil) && (*color_Bit_Map_Handle)) {
				/* Dispose of the off-screen graphics environments */
				Dispose_Off_Screen_Port(color_Offscreen_Port, color_Device);
				
				DisposeHandle(color_Bit_Map_Handle);
				if (MemError() != noErr) {
					/*  */
					/* We are closing up anyway. Ignore this serious error. */
					/*  */
				}
			}
			
		}
	}
	
} /* void Test_Off_Screen_Routines(void) */
/*=**************************************************************************=*/


/*=**************************************************************************=*/
/* Number of color table entries */
/* ===
#define kNumColors 256
=== */

/* Handle to the new color table */
/* ====
CTabHandle created_Clut; 
=== */

/* Index into the table of colors */
/* ===
INTEGER_16		index;
==== */

/* Allocate memory for the color table */
/* ===========

created_Clut = (CTabHandle)NewHandleClear(sizeof (ColorTable) +
		sizeof (ColorSpec) * (kNumColors - 1));
if (created_Clut != nil)
{

	table_Ptr = **created_Clut;

	table_Ptr->ctSeed = GetCTSeed();
	table_Ptr->ctFlags = 0;
	table_Ptr->ctSize = kNumColors - 1;

	for (index = 0; index < kNumColors; index++)
	{
		table_Ptr->ctTable[index].value = index;
		table_Ptr->ctTable[index].rgb.red = someRedValue;
		table_Ptr->ctTable[index].rgb.green = someGreenValue;
		table_Ptr->ctTable[index].rgb.blue = someBlueValue;
	}
}

=============== */
/*=**************************************************************************=*/


/*=**************************************************************************=*/
/* END OF FILE: Off_Screen_Color.c */
