/* START OF FILE: Picture_Spooling.c */


/*=**************************************************************************=*/
/* [---BEGIN---] FILE HEADER COMMENTS */
/**
 ** SCREEN VIEW = [Adobe Syntax Black-10, Tabs = 4]
 ** PRINTER VIEW = [Adobe Syntax Black-10, Tabs = 4]
 ** =======================================================================
 ** Written by James Merkle
 ** Picture_Spooling.c - Routines spooling PICT data from disk in chunks direct to port.
 ** (C) Copyright 1995 James Merkle
 ** All rights reserved, except those wishing to "copy" moderate sections of code or files may do so, so long as
 ** such code is never used directly either in animation-related tools or multimedia tools that are  
 ** used in live performance or live broadcast or recorded mass market broadcast or film. Desktop usage not restricted.
 ** Conventional desktop game or entertainment not restricted.
 ** =======================================================================
 **
 **
 ** ============
 ** Revision History
 ** ============
 **
 **  1995.01.09		(James Merkle)
 **						Initial coding started
 **
 **
 **
 ** =================
 ** OTHER INFORMATION
 ** =================
 **
 **
 **/
/* [----END----] FILE HEADER COMMENTS */
/*=**************************************************************************=*/


/*=**************************************************************************=*/
/* [---BEGIN---] #include AREA */

#include "Shell_Dialogs.h"
#include "Shell_Assembly.h"
#include "Memory_Utilities.h"
#include "String_Utilities.h"
#include "Buffered_File_Routines.h"


#include "Picture_Spooling.h"

/* [----END----] #include AREA */
/*=**************************************************************************=*/


/*=**************************************************************************=*/
/* [---BEGIN---] #define AREA */

/*=--------------------------------------------------------------------------=*/
/* Unique little THINK C constructs to save and restore A5 worlds */
/*=--------------------------------------------------------------------------=*/
#define RememberA5()	do { __Get_A5(); asm { move.l A5,(a1) } } while (0)

#define Setup_A5()			do { asm { move.l A5,-(sp) } __Get_A5(); asm { move.l (a1),A5 } } while (0)
#define Restore_A5()		do { asm { move.l (sp)+,A5 } } while (0)

/* [----END----] #define AREA */
/*=**************************************************************************=*/


/*=**************************************************************************=*/
/* [---BEGIN---] TYPE AREA */

OSErr Draw_File_Picture_BASE(short pictFileRef
								, Rect *destRect
								, Boolean get_Pict_Rect_And_Depth
								, Rect *source_Pict_Rect
								, Safe_Boolean parse_For_Color_Depth
								, CARDINAL_32 *source_Pict_Max_Depth);

pascal void Get_Quick_Draw_Data(Ptr dataPtr, INTEGER_16 byteCount);

static void __Get_A5(void);

/* [----END----] TYPE AREA */
/*=**************************************************************************=*/


/*=**************************************************************************=*/
/* [---BEGIN---] PROTOTYPE AREA */
					

/* [----END----] PROTOTYPE AREA */
/*=**************************************************************************=*/


/*=**************************************************************************=*/
/* [---BEGIN---] GLOBAL DEFINITION AREA */


/* [----END----] GLOBAL DEFINITION AREA */
/*=**************************************************************************=*/


/*=**************************************************************************=*/
/* [---BEGIN---] LOCAL STATIC DEFINITION AREA */

static CARDINAL_8			*unit_Name = "\pPicture_Spooling";

static volatile	INTEGER_16		g_Spooled_Pict_File_Ref;

/* [----END----] LOCAL STATIC DEFINITION AREA */
/*=**************************************************************************=*/


/*=**************************************************************************=*/
static void __Get_A5(void)
{
	asm {
		bsr.s	@1
		dc.l	0			;  store A5 here
@1		move.l	(sp)+,a1
	}
}
/*=**************************************************************************=*/


/*=**************************************************************************=*/
OSErr Draw_File_Picture(short pictFileRef, Rect *dest_Rect)
{
	CARDINAL_32 source_Pict_Max_Depth;
	
	
	return(Draw_File_Picture_BASE(pictFileRef, dest_Rect, FALSE, nil, SAFE_FALSE, &source_Pict_Max_Depth));
}
/*=**************************************************************************=*/


/*=**************************************************************************=*/
OSErr Get_File_Picture_Rect(short pictFileRef
												, Rect *source_Pict_Rect)
/*  */
/* NOTE: Get_File_Picture_Rect relies on "k_USE_BUFFERED_FILE_IO" method */
/*  */
{
	CARDINAL_32 source_Pict_Max_Depth;

	return(Draw_File_Picture_BASE(pictFileRef, nil, TRUE, source_Pict_Rect, SAFE_FALSE, &source_Pict_Max_Depth));
}
/*=**************************************************************************=*/


/*=**************************************************************************=*/
OSErr Get_File_Picture_Rect_And_Depth(short pictFileRef
												, Rect *source_Pict_Rect
												, Safe_Boolean parse_For_Color_Depth
												, CARDINAL_32 *source_Pict_Max_Depth)
/*  */
/* NOTE: Get_File_Picture_Rect relies on "k_USE_BUFFERED_FILE_IO" method */
/*  */
{
	return(Draw_File_Picture_BASE(pictFileRef, nil, TRUE, source_Pict_Rect, parse_For_Color_Depth, source_Pict_Max_Depth));
}
/*=**************************************************************************=*/


/*=**************************************************************************=*/
OSErr Draw_File_Picture_BASE(short pictFileRef
								, Rect *destRect
								, Boolean get_Pict_Rect_And_Depth
								, Rect *source_Pict_Rect
								, Safe_Boolean parse_For_Color_Depth
								, CARDINAL_32 *source_Pict_Max_Depth)
/*=***********
* NAME & SYNOPSIS:
* DrawFilePicture: Read a PICT file into the current GrafPort OR just analyze its
* rectangular max area for future call.
*
* PARAMETERS:
* short pictFileRef: File reference number of PICT file to open
* Rect  *destRect:   Rectangle to draw the picture into
*
* DEFINITION:
* This routine is used to read the picture data from the PICT file specified by
* pictFileRef and draw the picture into the rectangle specified by the destRect
* parameter in the current GrafPort.  Its just like DrawPicture, except the
* file reference number of a PICT file is specified instead of a PicHandle.
*
* DESCRIPTION:
* DrawFilePicture works by using the picture spooling method described in Inside
* Macintosh V on pages 87 through 90.  In this method, the getPicProc GrafProc is
* replaced by my routine called Get_Quick_Draw_Data which reads PICT information from
* disk.  By doing this, the DrawPicture command effectively reads PICT files.
*
* After setting up the GrafProcs record, this routine installs the GrafProcs
* into the current port.  Then, a temporary picture is manually allocated and
* placed into spool_Pict_Hdl.  This is essentially a placeholder to hold the picture
* header (which is different from the 512-byte PICT file header) and is always
* the size of the Picture record type without any picture data.
*
*
* SetFPos is called to skip the first 512 bytes
* of the file.  The first 512 bytes of a PICT file is called the PICT file
* header (not to be confused with the picture header, which contains the picSize
* and picFrame fields of the Picture type) and holds application-specific
* information.  This application doesnt care whats there so it just skips it.
* The picture header follows the PICT file header and is read into spool_Pict_Hdl.
*
* Finally (almost), DrawPicture is called.  Because this routine replaced the
* current ports GrafProcs with my own, DrawPicture gets the picture information
* from the PICT file rather than from spool_Pict_Hdl.  It draws the picture into the
* current port.
*
* RETURN VALUES:
* Result:   Returns an error code.
* destRect: Returns the picFrame of the picture in the PICT file if requested.
*
* NOTE: Get_File_Picture_Rect relies on "k_USE_BUFFERED_FILE_IO" method
*******=*/
	{
	CGrafPtr				current_Draw_Port;
	PicHandle				spool_Pict_Hdl;
	Memory_Allocation	spool_Pict_Hdl_Memory_Initialized;
	INTEGER_32			 bytes_To_Read;
	CARDINAL_32		 version_Of_QuickDraw;
	OSErr					error;
	QDProcsPtr			saved_QD_Draw_Procs_Ptr;	
	CARDINAL_32		preferred_Size;
	CARDINAL_32		original_Amount;
	CARDINAL_8			scrap_Debug_Pad_0[80];
	INTEGER_16			hfs_Block_Buffer[256];
	CARDINAL_8			scrap_Debug_Pad_1[200];
	CQDProcs				custom_QD_Draw_Procs;
	CQDProcs				stored_Original_QD_Draw_Procs;
	CARDINAL_8			scrap_Debug_Pad_2[200];
	OSErr					function_Result;
/* END VAR */


	function_Result = -1;

	current_Draw_Port = nil;
	spool_Pict_Hdl = nil;
	spool_Pict_Hdl_Memory_Initialized = MEMORY_NOT_INITIALIZED;
	saved_QD_Draw_Procs_Ptr = nil;

	/*  */
	/* Get the current version of QuickDraw */
	/*  */
	(void)Gestalt( gestaltQuickdrawVersion, (INTEGER_32 *)&version_Of_QuickDraw );

	/*  */
	/* Get a pointer to the current port and determine its type */
	/*  */
	GetPort((GrafPtr *)&current_Draw_Port );

/* ===
	SetPort((GrafPtr)current_Draw_Port );
	PortChanged((GrafPtr)current_Draw_Port );
=== */


	/*  */
	/* Initialize the GrafProcs for reading from a PICT file */
	/*  */
	if (version_Of_QuickDraw >= gestalt8BitQD) {
	
		/*  */
		/* Initialize defaults */
		/*  */
	
		SetStdCProcs( &custom_QD_Draw_Procs );
	/* ===
		custom_QD_Draw_Procs = *(CQDProcs *)(((CGrafPtr)current_Draw_Port)->grafProcs);
	=== */
	}
	else {
		SetStdProcs( /*<*/(QDProcsPtr) &custom_QD_Draw_Procs );
	}
		
#if !defined(ORIGINAL_MAC_SYSTEM)
#error "UNDEFINED PARAMETER!"
#endif
#if (ORIGINAL_MAC_SYSTEM == 0)
xxx do Pointer correctly for PPC
	custom_QD_Draw_Procs.getPicProc = (QDGetPicUPP)&Get_Quick_Draw_Data;
#else

	custom_QD_Draw_Procs.getPicProc = (QDGetPicUPP)&Get_Quick_Draw_Data;
	
	/*  */
	/* Store A5 using our normal technique */
	/* It will be needed as soon as custom GrafProcs installed */
	/*  */
	RememberA5();

#endif

	g_Spooled_Pict_File_Ref = pictFileRef;

	/*  */
	/* Install the custom GrafProcs into the tempPort */
	/*  */

	if (current_Draw_Port != nil) {
		saved_QD_Draw_Procs_Ptr = (QDProcsPtr)current_Draw_Port->grafProcs;
		
		/*  */
		/* 1995.05.21 JAM enhancement */
		/*  */
		BlockMoveData(saved_QD_Draw_Procs_Ptr, &stored_Original_QD_Draw_Procs, sizeof(stored_Original_QD_Draw_Procs));
	}
	else {
		Perform_OK_Dialog("\pTemp note: unexpected problem with port not being set/retrieved (1)");
	}
	
	
/* ===
	((CGrafPtr)current_Draw_Port)->grafProcs = (QDProcsPtr)&custom_QD_Draw_Procs;
==== */
	((CGrafPtr)current_Draw_Port)->grafProcs = (CQDProcsPtr)&custom_QD_Draw_Procs;

/* ==
	PortChanged((GrafPtr)current_Draw_Port );
=== */

	/*  */
	/* Create the temporary picture used for spooling */
	/*  */
	spool_Pict_Hdl = nil;
	spool_Pict_Hdl_Memory_Initialized = MEMORY_NOT_INITIALIZED;
			
	preferred_Size = sizeof (Picture)  + 200L;
	original_Amount = preferred_Size;
	
	if (Create_Handle(&preferred_Size
						, (CARDINAL_32)(0L)
						, ALLOW_PURGING
						, DISALLOW_PARTIAL_AMOUNT
						, (Handle *)&spool_Pict_Hdl
						, &spool_Pict_Hdl_Memory_Initialized) != SAFE_TRUE) {
								
		function_Result = memFullErr;
		goto Error_Handler;
	}
	
	HLock((Handle)spool_Pict_Hdl);
	if (MemError() != noErr) {
	
	}
	
	/*  */
	/* Paranoia only */
	/*  */
	Fill_Byte_Memory_With_Byte((Ptr)*spool_Pict_Hdl
								, original_Amount
								, 0);
								
	
	/*  */
	/* Skip the 512-byte PICT file header */
	/*  */
	
	bytes_To_Read = 512L;
	
#if !defined(k_USE_BUFFERED_FILE_IO)
#error "UNDEFINED PARAMETER!"
#endif
#if (k_USE_BUFFERED_FILE_IO == 1)
	if (Read_File_Using_Buffer(pictFileRef
								, &bytes_To_Read
								, (Ptr)&hfs_Block_Buffer[0]
								, &error) != TRUE) {
		if (error == noErr) {
			error = -1;
		}
	}
#else
	error = FSRead(pictFileRef
								, &bytes_To_Read
								, (Ptr)&hfs_Block_Buffer[0]);
#endif
	
	if (error != noErr) {
		function_Result = error;
		goto Error_Handler;
	}
	
	/*  */
	/* Read the picture header (not PICT file header) into spool_Pict_Hdl */
	/*  */
	
	bytes_To_Read = sizeof (Picture);
	
#if (k_USE_BUFFERED_FILE_IO == 1)
	if (Read_File_Using_Buffer(pictFileRef
								, &bytes_To_Read
								, (Ptr)*spool_Pict_Hdl
								, &error) != TRUE) {
		if (error == noErr) {
			error = -1;
		}
	}
#else
	error = FSRead(pictFileRef
								, &bytes_To_Read
								, (Ptr)*spool_Pict_Hdl);
#endif
	

	if (error != noErr) {
		function_Result = error;
		goto Error_Handler;
	}
	
	if (get_Pict_Rect_And_Depth == TRUE) {
		*source_Pict_Rect = (**spool_Pict_Hdl).picFrame;
		
		/*  */
		/* Assume 16 bit max info */
		/*  */
		*source_Pict_Max_Depth = 16;
		
		if (parse_For_Color_Depth == SAFE_TRUE) {
	/* =====
			os_Error = GetPictInfo((PicHandle)temp_Resource
					, &picture_Info
					, (0)
					, 0
					, popularMethod
					, 0);
					
			if (os_Error != noErr) {
				*source_Pict_Max_Depth = 16;
			}
			else {
				*source_Pict_Max_Depth = picture_Info.depth;	
			}
	
	===== */
		} /* if (parse_For_Color_Depth == SAFE_TRUE)  */

	}
	else {
		/*  */
		/* Spool the picture in */
		/*  */
		g_Spooled_Pict_File_Ref = pictFileRef;
		DrawPicture( spool_Pict_Hdl, destRect );
	}

	function_Result = noErr;
	goto Normal_Function_Exit;

Error_Handler:
	if (function_Result == noErr) {
		function_Result = -1;
	}
			
Normal_Function_Exit:

	if (saved_QD_Draw_Procs_Ptr != nil) {
		/*  */
		/* 1995.05.21 JAM enhancement? */
		/*  */
/* ===
		BlockMoveData(&stored_Original_QD_Draw_Procs, saved_QD_Draw_Procs_Ptr, sizeof(stored_Original_QD_Draw_Procs));	
==== */

		current_Draw_Port->grafProcs = (CQDProcsPtr)saved_QD_Draw_Procs_Ptr;
	
		/*  */
		/* 1995.05.21 JAM enhancement. Remind QD for paranoia.*/
		/*  */
		PortChanged((GrafPtr)current_Draw_Port );

		
		saved_QD_Draw_Procs_Ptr = nil;
	}
	else {
		current_Draw_Port->grafProcs = nil;
	}

	if (spool_Pict_Hdl_Memory_Initialized == MEMORY_INITIALIZED) {
		if (Destroy_Handle((Handle *)&spool_Pict_Hdl
							, &spool_Pict_Hdl_Memory_Initialized) != SAFE_TRUE) {
							
			/*  */
			/* Disregard memory error, we are exiting anyway */
			/*  */
		}
	}

	return(function_Result);
}
/*=**************************************************************************=*/


/*=**************************************************************************=*/
pascal void Get_Quick_Draw_Data(Ptr dataPtr, INTEGER_16 byteCount)
/******************************************************************************\
* NAME & SYNOPSIS:
* Get_Quick_Draw_Data: Get QuickDraw picture information from a file
*
* PARAMETERS:
* Ptr   dataPtr:   Points to location that picture data should be placed
* short byteCount: Number of bytes of picture data to read
*
* DEFINITION:
* This routine is a custom QuickDraw graphics procedure which replaces the
* standard GetPicProc.  QuickDraw graphics procedures are routines that
* QuickDraw calls to do almost everything that it does, like drawing rectangles,
* or getting picture data.  They can be replaced by setting up a QDProcs record
* and setting the appropriate field to point to the custom QuickDraw graphics
* procedure.  In this case, the getPicProc field is set to point to Get_Quick_Draw_Data
* so that any DrawPicture call within the GrafPort causes Get_Quick_Draw_Data to be
* called to get picture data.  Instead of getting picture data from memory,
* Get_Quick_Draw_Data gets picture data from the disk file whose file reference number is
* stored in the g_Spooled_Pict_File_Ref global variable.
*
* DESCRIPTION:
* Get_Quick_Draw_Data simply calls FSRead to read data from the file whose file reference
* number is stored in g_Spooled_Pict_File_Ref into the area pointed to by dataPtr, which is
* an area of memory thats set up by QuickDraw to receive picture data.  The
* number of bytes of picture data that QuickDraw wants is specified by the
* byteCount parameter.
*
* RETURN VALUES:
* None
\******************************************************************************/
{
	INTEGER_32 total_To_Read; /* Number of bytes to read from the PICT file */
	OSErr	error;
/* END VAR */


	/*  */
	/* Warning!!! Globals (A5) MUST BE VALID entering this routine */
	/*  */
	asm {
		movem.l	d0-d1/a0-a2/a5,  -(SP)		; Save all registers that may be altered
	}
	Setup_A5();
	
	total_To_Read = byteCount;
	
#if (k_USE_BUFFERED_FILE_IO == 1)
	if (Read_File_Using_Buffer(g_Spooled_Pict_File_Ref
								, &total_To_Read
								, (Ptr)dataPtr
								, &error) != TRUE) {
		if (error == noErr) {
			error = -1;
		}
	}
#else
	error = FSRead(g_Spooled_Pict_File_Ref
								, &total_To_Read
								, (Ptr)dataPtr);

#endif

	Restore_A5();
	asm {
		movem.l	(SP)+,d0-d1/a0-a2/a5	; Restore altered registers
	}
}
/*=**************************************************************************=*/


/*=**************************************************************************=*/
/* END OF FILE: Picture_Spooling.c */
