/* START OF FILE: Ring_Buffer_Cache.c */


/*=**************************************************************************=*/
/* [---BEGIN---] FILE HEADER COMMENTS */
/**
 ** SCREEN VIEW = [Adobe Syntax Black-10, Tabs = 4]
 ** PRINTER VIEW = [Adobe Syntax Black-10, Tabs = 4]
 ** =======================================================================
 ** Written by James Merkle
 ** Ring_Buffer_Cache.c - Routines for FIFO simple style (ring buffer) with Variable size large records
 ** (C) Copyright 1995 James Merkle
 ** All rights reserved, except those wishing to "copy" moderate sections of code or files may do so, so long as
 ** such code is never used directly either in animation-related tools or multimedia tools that are  
 ** used in live performance or live broadcast or recorded mass market broadcast or film. Desktop usage not restricted.
 ** Conventional desktop game or entertainment not restricted.
 ** =======================================================================
 **
 **
 ** ============
 ** Revision History
 ** ============
 **
 **  1995.01.09		(James Merkle)
 **						Initial coding started
 **
 **
 **
 ** =================
 ** OTHER INFORMATION
 ** =================
 **
 ** This ring buffer has occlusion of older elements as needed. It is not 100% optimal, but it is solid.
 **
 **/
/* [----END----] FILE HEADER COMMENTS */
/*=**************************************************************************=*/


/*=**************************************************************************=*/
/* [---BEGIN---] #include AREA */

#include "Error_Utilities.h"
#include "Memory_Utilities.h"
#include "Shell_Assembly.h"
#include "String_Utilities.h"

#include "Shell_Dialogs.h"
#include "Interface_Utilities.h"
#include "Shell_Globals.h"

#include "Ring_Buffer_Cache_Setup.h"
#include "Ring_Buffer_Cache.h"

/* [----END----] #include AREA */
/*=**************************************************************************=*/


/*=**************************************************************************=*/
/* [---BEGIN---] #define AREA */


/* [----END----] #define AREA */
/*=**************************************************************************=*/


/*=**************************************************************************=*/
/* [---BEGIN---] TYPE AREA */


/* [----END----] TYPE AREA */
/*=**************************************************************************=*/


/*=**************************************************************************=*/
/* [---BEGIN---] PROTOTYPE AREA */
					

/* [----END----] PROTOTYPE AREA */
/*=**************************************************************************=*/


/*=**************************************************************************=*/
/* [---BEGIN---] GLOBAL DEFINITION AREA */


/* [----END----] GLOBAL DEFINITION AREA */
/*=**************************************************************************=*/


/*=**************************************************************************=*/
/* [---BEGIN---] LOCAL STATIC DEFINITION AREA */

static CARDINAL_8			*unit_Name = "\pRing_Buffer_Cache";


/* [----END----] LOCAL STATIC DEFINITION AREA */
/*=**************************************************************************=*/


/*=**************************************************************************=*/
void Clear_Ring_Buffer_Entries(Ring_Buffer_Cache_Ptr cache_Master_Ptr)
{
	CARDINAL_32		index;
	CARDINAL_32		max_Entries;
/* END VAR */


	max_Entries = cache_Master_Ptr->max_Cache_FIFO_Entries;
	for (index = 0; (index < max_Entries); index++) {
		cache_Master_Ptr->cache_Entries[index].offset = k_UNUSED_ENTRY; 
		
		/* Paranoia clearing only */
	/* ===
		cache_Master_Ptr->cache_Entries[index].composite_Search_Key = 0;  
		cache_Master_Ptr->cache_Entries[index].secondary_Search_Key = 0;  
		cache_Master_Ptr->cache_Entries[index].total_Size_Stored = 0; 
	==== */
	}
	
	cache_Master_Ptr->total_Cache_FIFO_Entries = 0;
	cache_Master_Ptr->head_FIFO_Entry_For_Addition = 0;
	cache_Master_Ptr->tail_FIFO_Entry_For_Deletion = 0;
	
	/*  */
	/* 1995.05.30 JAM Possible BUG FIX. */
	/* previously if this routine was ever called caching would stop */
	/* because max_Ring_Cache_Size was cleared? */
	/*  */
/* ==
	cache_Master_Ptr->max_Ring_Cache_Size = 0;
=== */
	
	cache_Master_Ptr->ring_Cache_Head_Next_Byte = 0;
	cache_Master_Ptr->ring_Cache_Tail_Byte = 0;
	
}
/*=**************************************************************************=*/


/*=**************************************************************************=*/
Safe_Boolean Retrieve_Element_From_Buffer(Ring_Buffer_Cache_Ptr cache_Information_Ptr
											, CARDINAL_32 simple_Key_1
											, CARDINAL_32 simple_Key_2
											, CARDINAL_32 *storage_Location_To_Use)
{
	CARDINAL_32		cache_Entry_Address;
	CARDINAL_32		index;
	CARDINAL_32		last_Item;
	CARDINAL_32		first_Item;
	Safe_Boolean			function_Result;
	Boolean				still_Searching;
	register Ring_Buffer_Cache_Ptr cache_Master_Ptr;
/* END VAR */


	function_Result = SAFE_FALSE;
	
	/*  */
	/* Move to local variable. */
	/* Parameters passed to Think C 7.03 pointers will always take two memory accesses */
	/* and under no circumstances will stay put in a register unless they */
	/* are relocated to a local variable within the routine. Honest */
	/* This doubles acccess speed. */
	/*  */
	cache_Master_Ptr = cache_Information_Ptr;
	
	if (cache_Master_Ptr->actively_Caching == SAFE_TRUE) {
	
		if (cache_Master_Ptr->max_Cache_FIFO_Entries > 0) {
		
			first_Item = cache_Master_Ptr->head_FIFO_Entry_For_Addition;
			last_Item = cache_Master_Ptr->tail_FIFO_Entry_For_Deletion;
			still_Searching = TRUE;
			
			if (last_Item > first_Item) {
				/*  */
				/* Head is wrapped around for certain. Start scanning from */
				/* tail to last index in bin. */
				/* Then repeat search from "0" to current head. */
				/*  */
				cache_Entry_Address = (CARDINAL_32)&(cache_Master_Ptr->cache_Entries[last_Item]);
				for (index = last_Item; ((index < k_MAX_CACHE_ENTRIES) && (still_Searching == TRUE)); index++) {
					/*  */
					/* Should redesign so that search key is used for search but then */
					/* key could not be any value desired so two compares needed. */
					/* (Validation of "in use" or not for safety) */
					/*  */
					
					if (((Cache_Tag *)cache_Entry_Address)->offset != k_UNUSED_ENTRY) {
						if ((((Cache_Tag *)cache_Entry_Address)->composite_Search_Key == simple_Key_1) 
							&& (((Cache_Tag *)cache_Entry_Address)->secondary_Search_Key == simple_Key_2)) {
						
							*storage_Location_To_Use = (CARDINAL_32)(((Cache_Tag *)cache_Entry_Address)->offset 
															+ (CARDINAL_32)(cache_Master_Ptr->file_Cache_Ptr));
															
							still_Searching = FALSE;
						}
					}
					cache_Entry_Address += sizeof(Cache_Tag);
				}
				
				if (still_Searching != TRUE) {
					function_Result = SAFE_TRUE;
				}
				else {
					/*  */
					/* Repeat from start to head.*/
					/*  */
					cache_Entry_Address = (CARDINAL_32)&(cache_Master_Ptr->cache_Entries[0]);
					for (index = 0; ((index <= first_Item) && (still_Searching == TRUE)); index++) {
						/*  */
						/* Should redesign so that search key is used for search but then */
						/* key could not be any value desired so two compares needed. */
						/*  */
						
						if (((Cache_Tag *)cache_Entry_Address)->offset != k_UNUSED_ENTRY) {
							if ((((Cache_Tag *)cache_Entry_Address)->composite_Search_Key == simple_Key_1) 
								&& (((Cache_Tag *)cache_Entry_Address)->secondary_Search_Key == simple_Key_2)) {
							
								*storage_Location_To_Use = (CARDINAL_32)(((Cache_Tag *)cache_Entry_Address)->offset 
															+ (CARDINAL_32)(cache_Master_Ptr->file_Cache_Ptr));
								still_Searching = FALSE;
							}
						}
						cache_Entry_Address += sizeof(Cache_Tag);
					}
					if (still_Searching != TRUE) {
						function_Result = SAFE_TRUE;
					}
				} /* NOT if (still_Searching != TRUE) */
			}
			else {
				/*  */
				/* Head is simply before tail in direct linear address space */
				/* Merely scan from tail through to head.*/
				/*  */
				cache_Entry_Address = (CARDINAL_32)&(cache_Master_Ptr->cache_Entries[last_Item]);
				for (index = last_Item; ((index <= first_Item) && (still_Searching == TRUE)); index++) {
					/*  */
					/* Should redesign so that search key is used for search but then */
					/* key could not be any value desired so two compares needed. */
					/*  */
					
					if (((Cache_Tag *)cache_Entry_Address)->offset != k_UNUSED_ENTRY) {
						if ((((Cache_Tag *)cache_Entry_Address)->composite_Search_Key == simple_Key_1) 
							&& (((Cache_Tag *)cache_Entry_Address)->secondary_Search_Key == simple_Key_2)) {
						
							*storage_Location_To_Use = (CARDINAL_32)(((Cache_Tag *)cache_Entry_Address)->offset 
															+ (CARDINAL_32)(cache_Master_Ptr->file_Cache_Ptr));
															
							still_Searching = FALSE;
						}
					}
					cache_Entry_Address += sizeof(Cache_Tag);
				}
				if (still_Searching != TRUE) {
					function_Result = SAFE_TRUE;
				}
			}
			
		} /* if (cache_Master_Ptr->max_Cache_FIFO_Entries > 0) */
		
	} /* if (cache_Master_Ptr->actively_Caching == SAFE_TRUE)  ... */

	return(function_Result);
	
} /* Safe_Boolean Retrieve_Element_From_Buffer ... */
/*=**************************************************************************=*/


/*=**************************************************************************=*/
Safe_Boolean Prepare_Location_For_Buffer_Write(Ring_Buffer_Cache_Ptr cache_Information_Ptr
											, CARDINAL_32 simple_Key_1
											, CARDINAL_32 simple_Key_2
											, CARDINAL_32 total_Size
											, CARDINAL_32 *storage_Location_To_Use)
/*  */
/* This routine must be atomically treated. perhaps add */
/* guard flag or disable interrupts. */
/*  */
{
	CARDINAL_32		destination_Address;
	CARDINAL_32		location_Of_New_Head_Next_Byte;
	CARDINAL_32		location_Of_Lead_Byte_of_Next_Store;
	CARDINAL_16		index;
	Cache_Tag			*cache_Entry_Ptr;
	Cache_Tag			*tail_Entry_Ptr;
	Safe_Boolean			function_Result;
	Boolean				tail_Still_Needs_Deletion;
	Boolean				ignore_Check_For_First_Index;
	CARDINAL_8			*code_Name;
	register Ring_Buffer_Cache_Ptr cache_Master_Ptr;
/* END VAR */


	function_Result = SAFE_FALSE;
	
	code_Name = "\pPrepare_Location_For_Buffer_Write";
	
	/*  */
	/* Move to local variable. */
	/* Parameters passed to Think C 7.03 pointers will always take two memory accesses */
	/* and under no circumstances will stay put in a register unless they */
	/* are relocated to a local variable within the routine. Honest */
	/* This doubles acccess speed. */
	/*  */
	cache_Master_Ptr = cache_Information_Ptr;
	
	if (cache_Master_Ptr->actively_Caching == SAFE_TRUE) {
	
		if (cache_Master_Ptr->max_Ring_Cache_Size >= (total_Size + k_MEM_ALIGNMENT_ROUNDED_MAX)) {
		
			if (cache_Master_Ptr->max_Cache_FIFO_Entries == 0) {
				/*  */
				/* Simple case, just store at front of ring and at front of data buffer */
				/*  */
				cache_Entry_Ptr = &(cache_Master_Ptr->cache_Entries[0]);
								
				destination_Address = 0;
				
				/*  */
				/* Round up to nearest 32 byte alignment for next hint entry */
				/* This step would not normally be needed for 50% of handles created */
				/* because they probably are on 16 byte paragraph boundaries */
				/* Not needed buffer start was created offset alighned for convenience */
				/*  */
			/* ====
				destination_Address += (k_MEM_ALIGNMENT_ROUNDED_MAX - 1);
				destination_Address = (destination_Address & 0xFFFFFFE0);
			==== */
				
				cache_Entry_Ptr->offset = destination_Address;
				cache_Entry_Ptr->composite_Search_Key = simple_Key_1;
				cache_Entry_Ptr->secondary_Search_Key = simple_Key_2;
				cache_Entry_Ptr->total_Size_Stored = total_Size;
				
				/*  */
				/* Defer storage until outside of routine */
				/*  */
				*storage_Location_To_Use = destination_Address + (CARDINAL_32)cache_Master_Ptr->file_Cache_Ptr;
						
				cache_Master_Ptr->total_Cache_FIFO_Entries += 1;
				cache_Master_Ptr->head_FIFO_Entry_For_Addition = 0;
				cache_Master_Ptr->tail_FIFO_Entry_For_Deletion = 0;
				
				cache_Master_Ptr->ring_Cache_Head_Next_Byte = (cache_Entry_Ptr->offset + total_Size);
				cache_Master_Ptr->ring_Cache_Tail_Byte = cache_Entry_Ptr->offset;
			}
			else {
				/*  */
				/* General case where ring management is needed and some or none */
				/* of the oldest buffer entries need deletion to make room for head */
				/*  */
			
				index = cache_Master_Ptr->head_FIFO_Entry_For_Addition;
				index += 1;
				if (index >= k_MAX_CACHE_ENTRIES) {
					index = 0;
					
				/* ====
					cache_Master_Ptr->head_FIFO_Entry_For_Addition = 0;
				=== */
					
					if (cache_Master_Ptr->cache_Entries[0].offset != k_UNUSED_ENTRY) {
					
						/*  */
						/* Delete Tail Entry first before continuing IF NEEDED */
						/*  */
					
						if (cache_Master_Ptr->tail_FIFO_Entry_For_Deletion != 0) {
						
							/*  */
							/* This is just a paranoia test. */
							/*  */

							Display_Serious_Error_Message(unit_Name
								, code_Name
								, "\pInternal problem with Ring Buffer, tail entry lost"
								, "\pCaching not optimal");
							goto Error_Handler;
						}
						else {
						
							/*  */
							/* This is just a paranoia test. */
							/*  */
							if (cache_Master_Ptr->total_Cache_FIFO_Entries != k_MAX_CACHE_ENTRIES) {
								Display_Serious_Error_Message(unit_Name
									, code_Name
									, "\pInternal problem with Ring Buffer, total index count not proper"
									, "\pCaching not optimal");
								goto Error_Handler;
							}
						}

						cache_Master_Ptr->cache_Entries[0].offset = k_UNUSED_ENTRY;
						cache_Master_Ptr->total_Cache_FIFO_Entries = cache_Master_Ptr->total_Cache_FIFO_Entries - 1;
						
						/*  */
						/* This just expects the next slot is filled with valid data */
						/*  */
						cache_Master_Ptr->tail_FIFO_Entry_For_Deletion = 1;
						cache_Master_Ptr->ring_Cache_Tail_Byte = cache_Master_Ptr->cache_Entries[1].offset;
				
					} /* if (cache_Master_Ptr->cache_Entries[0].offset != k_UNUSED_ENTRY) */
					
				} /* if (index >= k_MAX_CACHE_ENTRIES) */
			
				cache_Entry_Ptr = &(cache_Master_Ptr->cache_Entries[index]);
				
				/*  */
				/* This is just a paranoia test. */
				/*  */
				if (cache_Entry_Ptr->offset != k_UNUSED_ENTRY) {
					Display_Serious_Error_Message(unit_Name
						, code_Name
						, "\pInternal problem with Ring Buffer, Tail residue collided with Head"
						, "\pCaching not optimal");
					goto Error_Handler;
				}
			
				ignore_Check_For_First_Index = FALSE;

				/*  */
				/* At this point we just delete linearly until the tail is moved up */
				/* out of the way of the data we want to write. It may even */
				/* have to wrap keys around and back, and may even have to delete */
				/* all the keys before a situation is satisfied to let us store into buffer. */
				/*  */
				if (cache_Master_Ptr->total_Cache_FIFO_Entries == 0) {
					location_Of_New_Head_Next_Byte = total_Size;
					location_Of_Lead_Byte_of_Next_Store = 0;
					ignore_Check_For_First_Index = TRUE;

				}
				else {
				
					location_Of_New_Head_Next_Byte = cache_Master_Ptr->ring_Cache_Head_Next_Byte;
					
					/*  */
					/* Round up to nearest 32 byte alignment for next hint entry */
					/* This step would not normally be needed for 50% of handles created */
					/* because they probably are on 16 byte paragraph boundaries */
					/*  */
					
					location_Of_New_Head_Next_Byte += (k_MEM_ALIGNMENT_ROUNDED_MAX - 1);
					location_Of_New_Head_Next_Byte = (location_Of_New_Head_Next_Byte & 0xFFFFFFE0);
					location_Of_Lead_Byte_of_Next_Store = location_Of_New_Head_Next_Byte;
					location_Of_New_Head_Next_Byte = location_Of_New_Head_Next_Byte + total_Size;
					if (location_Of_New_Head_Next_Byte > cache_Master_Ptr->max_Ring_Cache_Size) {
					
						/*  */
						/* Semi-common problem, the storage ring buffer needs to wrap around */
						/*  */
						location_Of_New_Head_Next_Byte = total_Size;
						location_Of_Lead_Byte_of_Next_Store = 0;
						
						ignore_Check_For_First_Index = TRUE;

						/*  */
						/* HACK HACK HACK : Needs attention to be wary of jump-across holes */
						/*  */
					}
				}
				
				tail_Still_Needs_Deletion = TRUE; 

				do {
					/*  */
					/* This is just a paranoia test. */
					/*  */
					tail_Entry_Ptr = &(cache_Master_Ptr->cache_Entries[cache_Master_Ptr->tail_FIFO_Entry_For_Deletion]);
					if (tail_Entry_Ptr->offset == k_UNUSED_ENTRY) {
						if (ignore_Check_For_First_Index != TRUE) {
							Display_Serious_Error_Message(unit_Name
								, code_Name
								, "\pInternal problem with Ring Buffer, Tail residue not holding data"
								, "\pCaching not optimal");
							goto Error_Handler;
						}
					}
					
					if (tail_Entry_Ptr->offset >= location_Of_New_Head_Next_Byte) {
						/*  */
						/* We are done! Deletion is complete */
						/*  */
						tail_Still_Needs_Deletion = FALSE;
					}
					else {
						/*  */
						/* Delete this entry if it is not fully behind us, otherwise we are done */
						/*  */
						
						if ((tail_Entry_Ptr->offset + tail_Entry_Ptr->total_Size_Stored) <= location_Of_Lead_Byte_of_Next_Store) {
							/*  */
							/* We are done! Deletion is complete */
							/*  */
							tail_Still_Needs_Deletion = FALSE;
						}
						else {
							tail_Entry_Ptr->offset = k_UNUSED_ENTRY;
							cache_Master_Ptr->total_Cache_FIFO_Entries = cache_Master_Ptr->total_Cache_FIFO_Entries - 1;
							
							if (cache_Master_Ptr->total_Cache_FIFO_Entries > 0) {
								/*  */
								/* This just expects the next slot is filled with valid data */
								/*  */
								cache_Master_Ptr->tail_FIFO_Entry_For_Deletion += 1;
								if (cache_Master_Ptr->tail_FIFO_Entry_For_Deletion >= k_MAX_CACHE_ENTRIES) {
									cache_Master_Ptr->tail_FIFO_Entry_For_Deletion = 0;
								}
								cache_Master_Ptr->ring_Cache_Tail_Byte = cache_Master_Ptr->cache_Entries[cache_Master_Ptr->tail_FIFO_Entry_For_Deletion].offset;
							}
							else {
								cache_Master_Ptr->tail_FIFO_Entry_For_Deletion = 0;
								
								cache_Master_Ptr->head_FIFO_Entry_For_Addition = 0;
								cache_Master_Ptr->tail_FIFO_Entry_For_Deletion = 0;
								cache_Master_Ptr->ring_Cache_Head_Next_Byte = 0;
								cache_Master_Ptr->ring_Cache_Tail_Byte = 0;
							}
						}
					}

				} while (tail_Still_Needs_Deletion == TRUE);
				
				/*  */
				/* Thats it! Merely record the work now */
				/*  */
				cache_Master_Ptr->head_FIFO_Entry_For_Addition = index;

				/*  */
				/* Just a paranoia test! */
				/*  */
				if (cache_Master_Ptr->head_FIFO_Entry_For_Addition >= k_MAX_CACHE_ENTRIES) {
					Display_Serious_Error_Message(unit_Name
						, code_Name
						, "\pInternal problem with Ring Buffer, head_FIFO_Entry_For_Addition overran index (s8)"
						, "\pCaching not optimal");
				}
				

				cache_Master_Ptr->ring_Cache_Head_Next_Byte = location_Of_New_Head_Next_Byte;
				if (cache_Master_Ptr->total_Cache_FIFO_Entries == 0) {
					cache_Master_Ptr->tail_FIFO_Entry_For_Deletion = index;
					cache_Master_Ptr->ring_Cache_Tail_Byte = location_Of_Lead_Byte_of_Next_Store;
				}
				cache_Master_Ptr->total_Cache_FIFO_Entries += 1;
				
				cache_Entry_Ptr->offset = location_Of_Lead_Byte_of_Next_Store;
				cache_Entry_Ptr->composite_Search_Key = simple_Key_1;
				cache_Entry_Ptr->secondary_Search_Key = simple_Key_2;
				cache_Entry_Ptr->total_Size_Stored = total_Size;
				
				/*  */
				/* Defer storage until outside of routine */
				/*  */
				*storage_Location_To_Use = location_Of_Lead_Byte_of_Next_Store + (CARDINAL_32)cache_Master_Ptr->file_Cache_Ptr;
	
			
			} /* NOT if (cache_Master_Ptr->total_Cache_FIFO_Entries == 0) */
			
		} /* if (cache_Master_Ptr->max_Ring_Cache_Size >= total_Size) */
		
		
		/*  */
		/* Just a paranoia test! */
		/*  */
		if (cache_Master_Ptr->tail_FIFO_Entry_For_Deletion >= k_MAX_CACHE_ENTRIES) {
			Display_Serious_Error_Message(unit_Name
				, code_Name
				, "\pInternal problem with Ring Buffer, tail_FIFO_Entry_For_Deletion overran index"
				, "\pCaching not optimal");
		}
		
		
		/*  */
		/* Just a paranoia test! */
		/*  */
		if (cache_Master_Ptr->head_FIFO_Entry_For_Addition >= k_MAX_CACHE_ENTRIES) {
			Display_Serious_Error_Message(unit_Name
				, code_Name
				, "\pInternal problem with Ring Buffer, head_FIFO_Entry_For_Addition overran index (s9)"
				, "\pCaching not optimal");
		}
		
	} /* if if (cache_Master_Ptr->actively_Caching == SAFE_TRUE)  ... */

 	function_Result = SAFE_TRUE;
	goto Normal_Function_Exit;
	
Error_Handler:
	function_Result = SAFE_FALSE;
		
Normal_Function_Exit:
	return(function_Result);
	
} /* Safe_Boolean Prepare_Location_For_Buffer_Write ... */
/*=**************************************************************************=*/


/*=**************************************************************************=*/
/* END OF FILE: Ring_Buffer_Cache.c */
