/* START OF FILE: Ring_Buffer_Cache_Setup.c */

/*=**************************************************************************=*/
/* [---BEGIN---] FILE HEADER COMMENTS */
/**
 ** SCREEN VIEW = [Adobe Syntax Black-10, Tabs = 4]
 ** PRINTER VIEW = [Adobe Syntax Black-10, Tabs = 4]
 ** =======================================================================
 ** Written by James Merkle
 ** Ring_Buffer_Cache_Setup.c - Setup code for FIFO simple style (ring buffer) with Variable size large records
 ** (C) Copyright 1995 James Merkle
 ** All rights reserved, except those wishing to "copy" moderate sections of code or files may do so, so long as
 ** such code is never used directly either in animation-related tools or multimedia tools that are  
 ** used in live performance or live broadcast or recorded mass market broadcast or film. Desktop usage not restricted.
 ** Conventional desktop game or entertainment not restricted.
 ** =======================================================================
 **
 **
 ** ============
 ** Revision History
 ** ============
 **
 **  1995.01.09		(James Merkle)
 **						Initial coding started
 **
 **
 **
 ** =================
 ** OTHER INFORMATION
 ** =================
 **
 ** This ring buffer has occlusion of older elements as needed.
 **
 **/
/* [----END----] FILE HEADER COMMENTS */
/*=**************************************************************************=*/


/*=**************************************************************************=*/
/* [---BEGIN---] #include AREA */

#include "Error_Utilities.h"
#include "Memory_Utilities.h"

#include "Shell_Dialogs.h"

#include "Ring_Buffer_Cache.h"
#include "Ring_Buffer_Cache_Setup.h"

/* [----END----] #include AREA */
/*=**************************************************************************=*/


/*=**************************************************************************=*/
/* [---BEGIN---] #define AREA */


/* [----END----] #define AREA */
/*=**************************************************************************=*/


/*=**************************************************************************=*/
/* [---BEGIN---] TYPE AREA */


/* [----END----] TYPE AREA */
/*=**************************************************************************=*/


/*=**************************************************************************=*/
/* [---BEGIN---] PROTOTYPE AREA */
					
Safe_Boolean Calculate_And_Get_Cache_Ram(CARDINAL_32 minimum_Extra_Temp_Ram_Remaining
													, CARDINAL_32 maximum_Acceptable_Cache
													, CARDINAL_32 minimum_Acceptable_Cache
													, CARDINAL_32 moderately_Acceptable_Cache
													, CARDINAL_32 minimum_Extra_App_Ram_Remaining
													, Ring_Buffer_Cache_Ptr cache_Information_Ptr
													, Safe_Boolean *use_Temporary_Memory);

CARDINAL_32 Get_Cache_Portion_Of_Application_Heap_Ram(CARDINAL_32 moderately_Acceptable_Cache
																		, CARDINAL_32 maximum_Acceptable_Cache
																		, CARDINAL_32 minimum_Acceptable_Cache
																		, CARDINAL_32 minimum_Extra_App_Ram_Remaining);

/* [----END----] PROTOTYPE AREA */
/*=**************************************************************************=*/


/*=**************************************************************************=*/
/* [---BEGIN---] GLOBAL DEFINITION AREA */

Safe_Boolean					g_Ring_Buffer_Cache_Initialized;

/* [----END----] GLOBAL DEFINITION AREA */
/*=**************************************************************************=*/


/*=**************************************************************************=*/
/* [---BEGIN---] LOCAL STATIC DEFINITION AREA */

static CARDINAL_8			*unit_Name;

/* [----END----] LOCAL STATIC DEFINITION AREA */
/*=**************************************************************************=*/


/*=**************************************************************************=*/
Safe_Boolean Initialize_Ring_Buffer_Cache_Setup(void)
{
	Safe_Boolean		function_Result;
	CARDINAL_8		*static_Unit_Name;
/* END VAR */


	unit_Name = "\pRing_Buffer_Cache_Setup";
	
/*  */
/* Global persistant structures are created here */
/*  */
	
	
		
	g_Ring_Buffer_Cache_Initialized = SAFE_TRUE;
	
	function_Result = SAFE_TRUE;
	goto Normal_Function_Exit;
	
Error_Handler:
	function_Result = SAFE_FALSE;
		
Normal_Function_Exit:
	return(function_Result);
	
} /* Safe_Boolean Initialize_Ring_Buffer_Cache_Setup( */
/*=**************************************************************************=*/


/*=**************************************************************************=*/
Safe_Boolean Destroy_Ring_Buffer_Cache_Setup(void)
{
	Safe_Boolean		function_Result;
/* END VAR */

/*  */
/* Global persistant structures are deleted here */
/*  */	
	function_Result = SAFE_TRUE;
	
	if (g_Ring_Buffer_Cache_Initialized == SAFE_TRUE) {

		; /* NULL */

	}

	return(function_Result);
	
} /* Safe_Boolean Destroy_Ring_Buffer_Cache_Setup ... */
/*=**************************************************************************=*/


/*=**************************************************************************=*/
Safe_Boolean Create_New_Ring_Cache(Ring_Buffer_Cache_Hdl *data_Handle
								, Memory_Allocation *allocated
								, Safe_Boolean use_Temporary_Memory)
{
	Safe_Boolean		function_Result;
	CARDINAL_32	memory_Requested;
	CARDINAL_8			*code_Name;
/* END VAR */


	code_Name = "\pCreate_New_Ring_Cache";
	function_Result = SAFE_TRUE;

/*  */
/* Global persistant structures are deleted here */
/*  */	
	
	memory_Requested = sizeof(Ring_Buffer_Cache_Rec);
	
	if (Create_Handle(&memory_Requested
								, (100L * 1024L)
								, ALLOW_PURGING
								, DISALLOW_PARTIAL_AMOUNT
								, (Handle *)data_Handle
								, allocated) != SAFE_TRUE) {
								
		Display_Low_Memory_Error_Message(unit_Name, code_Name, sizeof(Ring_Buffer_Cache_Rec));
		goto Error_Handler;
	}
	
	/*  */
	/* This is actually initialized later on when files are openned up */
	/*  */
	(**data_Handle)->actively_Caching = SAFE_FALSE;
	(**data_Handle)->temp_Memory_Suggested = use_Temporary_Memory;

	function_Result = SAFE_TRUE;
	goto Normal_Function_Exit;
	
Error_Handler:
	function_Result = SAFE_FALSE;
		
Normal_Function_Exit:
	return(function_Result);
	
} /* Safe_Boolean Destroy_Ring_Buffer_Cache_Setup ... */
/*=**************************************************************************=*/


/*=**************************************************************************=*/
Safe_Boolean Close_Down_Ring_Cache_If_Possible(Ring_Buffer_Cache_Ptr cache_Information_Ptr)
{
	CARDINAL_8			*code_Name;
	OSErr					os_Error;
	Safe_Boolean			function_Result;
	Safe_Boolean			problem_Encounterred;
/* END VAR */


	code_Name = "\pClose_Down_Ring_Cache_If_Possible";

	function_Result = SAFE_FALSE;
	problem_Encounterred = SAFE_FALSE;
	if (cache_Information_Ptr->actively_Caching == SAFE_TRUE) {
	
		cache_Information_Ptr->actively_Caching = SAFE_FALSE;

 		if (cache_Information_Ptr->file_Cache_Memory_Initialized == MEMORY_INITIALIZED) {
 		
 			if (cache_Information_Ptr->file_Cache_Is_Temp_Memory == SAFE_TRUE) {
				MFTempDisposHandle((Handle)cache_Information_Ptr->file_Cache_Handle, &os_Error);
				if (os_Error != noErr) {
					/*  */
					/* We should bitch, but its closing down anyways */
					/*  */
					problem_Encounterred = SAFE_TRUE;
				}
 			}
 			else {
				DisposeHandle((Handle)cache_Information_Ptr->file_Cache_Handle);
				if (MemError() != noErr) {
					/*  */
					/* We should bitch, but its closing down anyways */
					/*  */
					problem_Encounterred = SAFE_TRUE;
				}
 			}
 			
			cache_Information_Ptr->file_Cache_Memory_Initialized = MEMORY_NOT_INITIALIZED;
			cache_Information_Ptr->file_Cache_Handle = nil;

			/*  */
			/* Just paranoia clearing. */
			/*  */
			cache_Information_Ptr->file_Cache_Ptr = nil;
			cache_Information_Ptr->total_Cache_FIFO_Entries = 0;
			
 		} /* if (cache_Information_Ptr->file_Cache_Memory_Initialized == MEMORY_INITITALIZED) */

	} /* if (cache_Information_Ptr->actively_Caching == SAFE_TRUE) */


	if (problem_Encounterred == SAFE_TRUE) {
		goto Error_Handler;
	}
	
	function_Result = SAFE_TRUE;
	goto Normal_Function_Exit;
	
Error_Handler:
	Display_Serious_Error_Message(unit_Name, code_Name, "\pMemory Deletion Failed", "\pPerhaps System is corrupt?");
	function_Result = SAFE_FALSE;
		
Normal_Function_Exit:

	return(function_Result);
}
/*=**************************************************************************=*/


/*=**************************************************************************=*/
Safe_Boolean Destroy_Ring_Buffer_Cache(Ring_Buffer_Cache_Hdl *data_Handle
												, Memory_Allocation *allocated)
{
	Safe_Boolean		function_Result;
	OSErr				os_Error;
	CARDINAL_8			*code_Name;
/* END VAR */


	code_Name = "\pDestroy_Ring_Buffer_Cache";

	/*  */
	/* Global persistant structures are deleted here */
	/*  */	
	function_Result = SAFE_TRUE;

	/*  */	
	/* Closing down cache is a temporary and reversable manuever, */
	/* but in this case it is used in preparation of true and total shutdown. */
	/*  */	
	if (Close_Down_Ring_Cache_If_Possible(*(*data_Handle)) != SAFE_TRUE) {
		function_Result = SAFE_FALSE;
	}
	else {
			
		if (Destroy_Handle((Handle *)data_Handle, allocated) != SAFE_TRUE) {
			Display_Serious_Error_Message(unit_Name, code_Name, "\pError with Destroy_Handle", "\pSystem Corrupted?");
			function_Result = SAFE_FALSE;
		}

	}
		
	return(function_Result);
	
} /* Safe_Boolean Destroy_Ring_Buffer_Cache ... */
/*=**************************************************************************=*/


/*=**************************************************************************=*/
CARDINAL_32 Get_Cache_Portion_Of_Application_Heap_Ram(CARDINAL_32 moderately_Acceptable_Cache
																		, CARDINAL_32 maximum_Acceptable_Cache
																		, CARDINAL_32 minimum_Acceptable_Cache
																		, CARDINAL_32 minimum_Extra_App_Ram_Remaining)
/*  */
/* This simple calculation returns either "0" if no good or returns the */
/* maximum acceptable amount of memory to use for caching. */
/* The maximum is a percentage (75%) of the memory available AFTER */
/* the assumed amount of RAM needed for the three stage mix buffers */
/* is reserved. Those buffers are about 3 * the normal screen view and are */
/* again multiplied by 3 of them. 9 * 640 * 480 = 2764800 bytes. An additional */
/* 900K is added to that amount to allow a bare bones scene and backdrop to */
/* load. That total is "3686400" or 3600K. */
/* This routine also reserves the application bare minimum to reserve and */
/* is altogether a fair way to divide leftover memory. */
/*  */
{
	CARDINAL_32		function_Result;
	CARDINAL_32 		total_Bytes_Contiguous_Now;
	CARDINAL_32		total_Bytes_Now;
	CARDINAL_32 		total_Bytes_Contiguous_Later;
	CARDINAL_32		total_Bytes_Later;
	CARDINAL_32		memory_Requested;
	CARDINAL_8			*code_Name;
	Boolean				good_Enough_Amount;
/* END VAR */


	code_Name = "\pGet_Cache_Portion_Of_Application_Heap_Ram";
	function_Result = 0;
	
	
	if (Determine_Absolute_Free_Memory((INTEGER_32 *)&total_Bytes_Contiguous_Now
													,(INTEGER_32 *)&total_Bytes_Now
													,(INTEGER_32 *)&total_Bytes_Contiguous_Later
													,(INTEGER_32 *)&total_Bytes_Later) != TRUE) {
		goto Error_Handler;										
	}
	
	good_Enough_Amount = FALSE;
	
	memory_Requested = total_Bytes_Contiguous_Later;
	
	/*  */
	/* First subtract the reserved memory */
	/*  */
	if (memory_Requested > (PROGRAM_MINIMUM_MEMORY_TO_PRESERVE + minimum_Extra_App_Ram_Remaining)) {
		memory_Requested = (memory_Requested 
									- (PROGRAM_MINIMUM_MEMORY_TO_PRESERVE + minimum_Extra_App_Ram_Remaining));
		
		function_Result = memory_Requested;
		/*  */
		/* Take 75% of remaining RAM for ring buffer */
		/* leave the other 25% to the application */
		/*  */
		
		memory_Requested = (memory_Requested >> 1);
		memory_Requested = memory_Requested + (memory_Requested >> 1);
		
		if (memory_Requested >= minimum_Acceptable_Cache) {
		
			/*  */
			/* If memory_Requested > moderately_Acceptable_Cache */
			/* then divide memory 50-50 instead of the 75% ration */
			/* for all data past moderately_Acceptable_Cache limit. */
			/* Note that at this point, 25% has already been tucked away so */
			/* in theory, more RAM than you would expect is reserved for the */
			/* Application heap at this point. */
			/*  */
			
			if (memory_Requested > moderately_Acceptable_Cache) {
				memory_Requested = memory_Requested - moderately_Acceptable_Cache;
				memory_Requested = (memory_Requested >> 1);
				memory_Requested = memory_Requested + moderately_Acceptable_Cache;
			}
		
			/*  */
			/* Limit cache to max (100 MB) if required.*/
			/*  */
			if (memory_Requested > maximum_Acceptable_Cache) {
				memory_Requested = maximum_Acceptable_Cache;
			}

			function_Result = memory_Requested;
			good_Enough_Amount = TRUE;
		}
		else {
			if (function_Result >= minimum_Acceptable_Cache) {
				good_Enough_Amount = TRUE;
			}
		}
	}
	else {
		/*  */
		/* Just give up. Not enough memory */
		/*  */
		;
	}					
	
	if (good_Enough_Amount != TRUE) {
		function_Result = 0;
	}
	
	goto Normal_Function_Exit;
	
Error_Handler:
	Display_Serious_Error_Message(unit_Name, code_Name, "\pInternal problem with memory? System Mem functions failing", "\pUnexpected. Low Memory?");
	function_Result = 0;
		
Normal_Function_Exit:

	return(function_Result);
	
} /*CARDINAL_32 Get_Cache_Portion_Of_Application_Heap_Ram... */
/*=**************************************************************************=*/


/*=**************************************************************************=*/
Safe_Boolean Calculate_And_Get_Cache_Ram(CARDINAL_32 minimum_Extra_Temp_Ram_Remaining
													, CARDINAL_32 maximum_Acceptable_Cache
													, CARDINAL_32 minimum_Acceptable_Cache
													, CARDINAL_32 moderately_Acceptable_Cache
													, CARDINAL_32 minimum_Extra_App_Ram_Remaining
													, Ring_Buffer_Cache_Ptr cache_Information_Ptr
													, Safe_Boolean *use_Temporary_Memory)
/*  */
/* This Routine creates cache Ram handles. It relies on existing */
/* globals that probably should have been passed as parameters. */
/* It is fairly aggressive at guaranteeing some useful single buffer */
/* from either temporary or Application heap RAM */
/*  */
{
	CARDINAL_32		memory_Requested;
	INTEGER_32 			total_Bytes_Contiguous_Now;
	INTEGER_32			total_Bytes_Now;
	INTEGER_32 			total_Bytes_Contiguous_Later;
	INTEGER_32			total_Bytes_Later;
	INTEGER_32			work_Integer_32;
	CARDINAL_32		good_Enough_Amount;
	CARDINAL_32 		minimum_Size_To_Shrink;
	CARDINAL_32		desired_Temp_Memory_Size;
	CARDINAL_32		preferred_Size;
	CARDINAL_32		possible_Available_App_RAM;
	CARDINAL_32		deviation;
	CARDINAL_32		aligned_Start_Address;
	CARDINAL_8			*code_Name;
	Safe_Boolean			function_Result;
	Boolean				retry_Getting_Any_Temporary_RAM;
	OSErr					os_Error;
/* END VAR */


	function_Result = SAFE_FALSE;
	code_Name = "\pClose_Down_Ring_Cache_If_Possible";

	/*  */
	/* Preflight feasability of using temporary memory */
	/*  */
	if (*use_Temporary_Memory == SAFE_TRUE) {
			
		work_Integer_32 = maximum_Acceptable_Cache; 
		preferred_Size = MFMaxMem(&work_Integer_32);
		if (preferred_Size < minimum_Extra_Temp_Ram_Remaining) {
			preferred_Size = 0;
		}
		else {
			preferred_Size = preferred_Size - minimum_Extra_Temp_Ram_Remaining;
		}
		
		if (preferred_Size < moderately_Acceptable_Cache) {

	 		if (preferred_Size < Get_Cache_Portion_Of_Application_Heap_Ram(moderately_Acceptable_Cache
																		, maximum_Acceptable_Cache
																		, minimum_Acceptable_Cache
																		, minimum_Extra_App_Ram_Remaining)) {
				*use_Temporary_Memory = SAFE_FALSE;
	 		}
	 		else {
	 			if (preferred_Size == 0) {
					/*  */
					/* Both RAM areas are useless! (Both are "0") */
					/*  */
					goto Low_Memory_Error_Handler;										
	 			}
	 		}
		}
	}
	

	retry_Getting_Any_Temporary_RAM = FALSE;
	
	/*  */
	/* Get Temp Handle RAM if still desired */
	/*  */
	if (*use_Temporary_Memory == SAFE_TRUE) {
		
		minimum_Size_To_Shrink = minimum_Acceptable_Cache;
		desired_Temp_Memory_Size = preferred_Size;
		
		/*  */
		/* Invoke an important routine that tries to eliminate accidental use of */
		/* virtual memory */
		/*  */
		Limit_Memory_Value_To_Safer_Limit_If_Virtual_Memory(&desired_Temp_Memory_Size
											, minimum_Size_To_Shrink);

		if (desired_Temp_Memory_Size > maximum_Acceptable_Cache) {
			desired_Temp_Memory_Size = maximum_Acceptable_Cache;
		}
		
		if (desired_Temp_Memory_Size >= minimum_Acceptable_Cache) {
			cache_Information_Ptr->file_Cache_Handle = TempNewHandle((Size)desired_Temp_Memory_Size, &os_Error);
			if (cache_Information_Ptr->file_Cache_Handle == nil) {
				retry_Getting_Any_Temporary_RAM = TRUE;
			}
			else {
				cache_Information_Ptr->max_Ring_Cache_Size = desired_Temp_Memory_Size;
				cache_Information_Ptr->file_Cache_Is_Temp_Memory = SAFE_TRUE;
				
				MoveHHi((Handle)cache_Information_Ptr->file_Cache_Handle);
				os_Error = MemError();
				if ((os_Error != noErr) && (os_Error != -117)){
					goto Error_Handler;
				}
				
				/*  */
				/* Lock memory down, hold in RAM too (belongs non-virtual anyway) */
				/*  */
				HLock(cache_Information_Ptr->file_Cache_Handle);
				if (MemError() != noErr) {
					goto Error_Handler;
				}
			}
		}
		else {
			*use_Temporary_Memory = SAFE_FALSE;
		}
	}
	
	/*  */
	/* Retry getting Temp Handle RAM if still desired. */
	/* Iterate smaller and smaller until something fits that works. */
	/*  */
	if (retry_Getting_Any_Temporary_RAM == TRUE) {
	
		possible_Available_App_RAM = Get_Cache_Portion_Of_Application_Heap_Ram(moderately_Acceptable_Cache
																		, maximum_Acceptable_Cache
																		, minimum_Acceptable_Cache
																		, minimum_Extra_App_Ram_Remaining);
							
		do {
		
			/*  */
			/* Keep reducing request for temporary memory by another */
			/* 25% and reiterate through */
			/*  */
			desired_Temp_Memory_Size = (desired_Temp_Memory_Size >> 1);
			desired_Temp_Memory_Size = desired_Temp_Memory_Size + (desired_Temp_Memory_Size >> 1);
		
			if ((desired_Temp_Memory_Size >= minimum_Acceptable_Cache) 
				&& (desired_Temp_Memory_Size >= possible_Available_App_RAM)){
				
				cache_Information_Ptr->file_Cache_Handle = TempNewHandle((Size)desired_Temp_Memory_Size, &os_Error);
				
				if ((os_Error == noErr) && (cache_Information_Ptr->file_Cache_Handle != nil)) {
					retry_Getting_Any_Temporary_RAM = FALSE;
					cache_Information_Ptr->max_Ring_Cache_Size = desired_Temp_Memory_Size;
					cache_Information_Ptr->file_Cache_Is_Temp_Memory = SAFE_TRUE;
					
					MoveHHi((Handle)cache_Information_Ptr->file_Cache_Handle);
					os_Error = MemError();
					if ((os_Error != noErr) && (os_Error != -117)){
						goto Error_Handler;
					}
					
					/*  */
					/* Lock memory down, hold in RAM too (belongs non-virtual anyway) */
					/*  */
					HLock(cache_Information_Ptr->file_Cache_Handle);
					if (MemError() != noErr) {
						goto Error_Handler;
					}
				}
				else {
					/*  */
					/* Just keep going */
					/*  */
				}
			}
			else {
				*use_Temporary_Memory = SAFE_FALSE;
				retry_Getting_Any_Temporary_RAM = FALSE;
			}
			
		} while (retry_Getting_Any_Temporary_RAM == TRUE);
		
	}
	
	/*  */
	/* Give up on Temp RAM and use app Regular RAM */
	/*  */
	if (*use_Temporary_Memory != SAFE_TRUE) {
			
		memory_Requested = Get_Cache_Portion_Of_Application_Heap_Ram(moderately_Acceptable_Cache
																		, maximum_Acceptable_Cache
																		, minimum_Acceptable_Cache
																		, minimum_Extra_App_Ram_Remaining);
		
		if (memory_Requested != 0) {
			
			if (Create_Handle(&memory_Requested
										, minimum_Extra_App_Ram_Remaining
										, DISALLOW_PURGING
										, ALLOW_PARTIAL_AMOUNT
										, (Handle *)&(cache_Information_Ptr->file_Cache_Handle)
										, &(cache_Information_Ptr->file_Cache_Memory_Initialized)) != SAFE_TRUE) {
										
				Display_Low_Memory_Error_Message(unit_Name, code_Name, sizeof(Ring_Buffer_Cache_Rec));
				function_Result = SAFE_FALSE;
				goto Normal_Function_Exit;
			}
			
			/*  */
			/* Note: memory_Requested may be smaller than expected. */
			/*  */
			cache_Information_Ptr->max_Ring_Cache_Size = memory_Requested;

		} /* if (total_Bytes_Contiguous_Later >= (1024L * 1024L))  */
		else {
			goto Low_Memory_Error_Handler;
		}
		
		MoveHHi((Handle)cache_Information_Ptr->file_Cache_Handle);
		os_Error = MemError();
		if ((os_Error != noErr) && (os_Error != -117)){
			goto Error_Handler;
		}

		/*  */
		/* Lock memory down, hold in RAM too (belongs non-virtual anyway) */
		/*  */
		HLock(cache_Information_Ptr->file_Cache_Handle);
		if (MemError() != noErr) {
			goto Error_Handler;
		}
		
	} /* if (*use_Temporary_Memory != SAFE_TRUE)*/


	cache_Information_Ptr->file_Cache_Ptr = (CARDINAL_8 *)*(cache_Information_Ptr->file_Cache_Handle);

	aligned_Start_Address = (CARDINAL_32)cache_Information_Ptr->file_Cache_Ptr;
	aligned_Start_Address += (k_MEM_ALIGNMENT_ROUNDED_MAX - 1);
	aligned_Start_Address = (aligned_Start_Address & 0xFFFFFFE0);
	
	/*  */
	/* Adjust table to start at offset desired by programmer */
	/* most handles will be at least 16 byte aligned but we may */
	/* want 32 or exact RAM pages. */
	/*  */
	if (aligned_Start_Address != (CARDINAL_32)cache_Information_Ptr->file_Cache_Ptr) {
		deviation = (aligned_Start_Address - (CARDINAL_32)cache_Information_Ptr->file_Cache_Ptr);
		cache_Information_Ptr->max_Ring_Cache_Size = cache_Information_Ptr->max_Ring_Cache_Size - deviation;
																	
		cache_Information_Ptr->file_Cache_Ptr = (CARDINAL_8 *)((CARDINAL_32)cache_Information_Ptr->file_Cache_Ptr + deviation);											
	}
	
	/*  */
	/* Round Cache Available Size Down to nearest multiple of 32 bytes */
	/* only 4 byte alignment needed but this is better anyway */
	/*  */
	cache_Information_Ptr->max_Ring_Cache_Size = (cache_Information_Ptr->max_Ring_Cache_Size & 0xFFFFFFE0UL);
	

	function_Result = SAFE_TRUE;
	goto Normal_Function_Exit;
	
Error_Handler:
	Display_Serious_Error_Message(unit_Name, code_Name, "\pInternal problem with memory?", "\pCaching is not activated");
		
Low_Memory_Error_Handler:

	function_Result = SAFE_FALSE;

Normal_Function_Exit:

	return(function_Result);
	
}/* Safe_Boolean Calculate_And_Get_Cache_Ram... */
/*=**************************************************************************=*/


/*=**************************************************************************=*/
Safe_Boolean Reset_And_Open_Ring_Buffer_If_Needed(Ring_Buffer_Cache_Ptr cache_Information_Ptr
													, CARDINAL_32 minimum_Extra_Temp_Ram_Remaining
													, CARDINAL_32 maximum_Acceptable_Cache
													, CARDINAL_32 minimum_Acceptable_Cache
													, CARDINAL_32 moderately_Acceptable_Cache
													, CARDINAL_32 minimum_Extra_App_Ram_Remaining)
{
	Safe_Boolean			function_Result;
	Safe_Boolean			temp_Desired;
	CARDINAL_8			*code_Name;
/* END VAR */


	code_Name = "\pReset_And_Open_Ring_Buffer_If_Needed";

	function_Result = SAFE_FALSE;
	if (cache_Information_Ptr->actively_Caching != SAFE_TRUE) {

		cache_Information_Ptr->file_Cache_Handle = nil;
		cache_Information_Ptr->file_Cache_Memory_Initialized = MEMORY_NOT_INITIALIZED;
		cache_Information_Ptr->file_Cache_Is_Temp_Memory = SAFE_FALSE;
		cache_Information_Ptr->file_Cache_Ptr = nil;
		cache_Information_Ptr->max_Cache_FIFO_Entries = k_MAX_CACHE_ENTRIES;
		cache_Information_Ptr->total_Cache_FIFO_Entries = 0;
			
		cache_Information_Ptr->total_Cache_FIFO_Entries = 0;
		cache_Information_Ptr->head_FIFO_Entry_For_Addition = 0;
		cache_Information_Ptr->tail_FIFO_Entry_For_Deletion = 0;
		
		cache_Information_Ptr->max_Ring_Cache_Size = 0;
		cache_Information_Ptr->ring_Cache_Head_Next_Byte = 0;
		cache_Information_Ptr->ring_Cache_Tail_Byte = 0;
				
		Clear_Ring_Buffer_Entries(cache_Information_Ptr);
			
		temp_Desired = cache_Information_Ptr->temp_Memory_Suggested;
		if (Calculate_And_Get_Cache_Ram(minimum_Extra_Temp_Ram_Remaining
												, maximum_Acceptable_Cache
												, minimum_Acceptable_Cache
												, moderately_Acceptable_Cache
												, minimum_Extra_App_Ram_Remaining
												, cache_Information_Ptr
												, &temp_Desired) != SAFE_TRUE) {
			goto Error_Handler;
		}
		cache_Information_Ptr->actively_Caching = SAFE_TRUE;
		
	} /* if (cache_Information_Ptr->actively_Caching... */

	function_Result = SAFE_TRUE;
	goto Normal_Function_Exit;
	
Error_Handler:
	Display_Serious_Error_Message(unit_Name, code_Name, "\pProblem creating large disk RAM cache", "\pCaching is not activated, yet is vital for program to function. Aborting.");
	function_Result = SAFE_FALSE;
		
Normal_Function_Exit:

	return(function_Result);
}
/*=**************************************************************************=*/


/*=**************************************************************************=*/
/* END OF FILE: Ring_Buffer_Cache_Setup.c */
