/* START OF FILE: Simple_Ring_Cache.c */


/*=**************************************************************************=*/
/* [---BEGIN---] FILE HEADER COMMENTS */
/**
 ** SCREEN VIEW = [Adobe Syntax Black-10, Tabs = 6]
 ** PRINTER VIEW = [Adobe Syntax Black-10, Tabs = 6]
 ** =======================================================================
 ** Written by James Merkle
 ** Simple_Ring_Cache.c - Routines to support a primitive ring cache of fixed size items
 ** (C) Copyright 1995 James Merkle
 ** =======================================================================
 **
 **
 ** ============
 ** Revision History
 ** ============
 **
 **  1995.05.29		(James Merkle)
 **						Initial coding started from scratch
 **
 **
 **
 ** =================
 ** OTHER INFORMATION
 ** =================
 ** 
 ** 
 **
 **
 **/
/* [----END----] FILE HEADER COMMENTS */
/*=**************************************************************************=*/


/*=**************************************************************************=*/
/* [---BEGIN---] #include AREA */

#include "Shell_Globals.h"
#include "Simple_Ring_Cache.h"

/* [----END----] #include AREA */
/*=**************************************************************************=*/


/*=**************************************************************************=*/
/* [---BEGIN---] #define AREA */


/* [----END----] #define AREA */
/*=**************************************************************************=*/


/*=**************************************************************************=*/
/* [---BEGIN---] TYPE AREA */


/* [----END----] TYPE AREA */
/*=**************************************************************************=*/


/*=**************************************************************************=*/
/* [---BEGIN---] PROTOTYPE AREA */


/* [----END----] PROTOTYPE AREA */
/*=**************************************************************************=*/


/*=**************************************************************************=*/
/* [---BEGIN---] GLOBAL DEFINITION AREA */


/* [----END----] GLOBAL DEFINITION AREA */
/*=**************************************************************************=*/


/*=**************************************************************************=*/
/* [---BEGIN---] LOCAL STATIC DEFINITION AREA */

static CARDINAL_8	*unit_Name;

/*  */
/* One small, global (for speed), frame history list is declared. */
/*  */
Simple_Ring_Buffer_Cache_Rec	g_Recent_Frame_List;

/* [----END----] LOCAL STATIC DEFINITION AREA */
/*=**************************************************************************=*/


/*=**************************************************************************=*/
Safe_Boolean Initialize_Simple_Ring_Cache(void)
{
	CARDINAL_8			*static_Unit_Name = "\pSimple_Ring_Cache";
/* END VAR */

/*  */
/* Global persistant structures are created here */
/*  */
	
	/*  */
	/* k_MAX_CACHE_ENTRIES must be a power of 2 */
	/*  */
	g_Recent_Frame_List.max_Cache_FIFO_Entries = k_MAX_CACHE_ENTRIES_POWER_2;
	g_Recent_Frame_List.max_Entry_Bit_Mask = (k_MAX_CACHE_ENTRIES_POWER_2 - 1UL);
	g_Recent_Frame_List.structure_Initialized = SAFE_TRUE;

	Clear_Simple_Ring_Cache_Entries(&g_Recent_Frame_List);

	unit_Name = static_Unit_Name;
	
	return(SAFE_TRUE);
}
/*=**************************************************************************=*/


/*=**************************************************************************=*/
Safe_Boolean Destroy_Simple_Ring_Cache(void)
{
	Safe_Boolean		function_Result;
/* END VAR */

/*  */
/* Global persistant structures are deleted here */
/*  */

	function_Result = SAFE_TRUE;
	return(function_Result);
}
/*=**************************************************************************=*/


/*=**************************************************************************=*/
void Clear_Simple_Ring_Cache_Entries(Simple_Ring_Buffer_Cache_Ptr cache_Master_Ptr)
{
	CARDINAL_32		index;
	CARDINAL_32		max_Entries;
/* END VAR */
	
	
	if (cache_Master_Ptr->structure_Initialized == SAFE_TRUE) {
	
		max_Entries = cache_Master_Ptr->max_Cache_FIFO_Entries;
		
		for (index = 0; (index < max_Entries); index++) {
			cache_Master_Ptr->composite_Search_Key[index] = 0xFFFFFFFF;
			cache_Master_Ptr->cache_Entries[index].secondary_Search_Key = 0xFFFFFFFF; 
		}
		cache_Master_Ptr->head_FIFO_Entry_For_Addition = 0;
	}
	
}
/*=**************************************************************************=*/		

		
/*=**************************************************************************=*/		
Safe_Boolean Locate_Best_Frame_From_Simple_Ring_Cache(CARDINAL_32 simple_Key_1
																	, CARDINAL_32 simple_Key_2
																	, CARDINAL_32 *cached_Data)
{
	CARDINAL_32			current_List_Marker;
	CARDINAL_32			max_Entries;
	SIMPLE_LOOP			index;
	Safe_Boolean				function_Result;
	Boolean					found_Item;
/* END VAR */
	
	if (g_Recent_Frame_List.structure_Initialized == SAFE_TRUE) {
		current_List_Marker = g_Recent_Frame_List.head_FIFO_Entry_For_Addition;
		
		/*  */
		/* Start By adding one and looping until we reach where we started */
		/*  */
		
		current_List_Marker += 1;
		/*  */
		/* We may have wrapped around, mask back to first position if so. */
		/*  */
		current_List_Marker = (current_List_Marker & g_Recent_Frame_List.max_Entry_Bit_Mask);

		/*  */
		/* Process the list in two ascending lists. Starting from "current_List_Marker" */
		/* and travelling to end of list. If not found a second search from front to */
		/* original start "current_List_Marker" is made. */
		/*  */
		found_Item = FALSE;
		max_Entries = g_Recent_Frame_List.max_Cache_FIFO_Entries;
		for (index = current_List_Marker; (index < max_Entries) ; index++) {
			if (g_Recent_Frame_List.composite_Search_Key[index] == simple_Key_1) {
				/*  */
				/* It matched the main volatile key, now match the other */
				/*  */
				if (g_Recent_Frame_List.cache_Entries[index].secondary_Search_Key == simple_Key_2) {
					found_Item = TRUE;
					*cached_Data = g_Recent_Frame_List.cache_Entries[index].cached_Data;
					/*  */
					/* It is found, so break out of for loop */
					/*  */
					break;
				}
			}
		}
		
		if (found_Item != TRUE) {
			/*  */
			/* Process other half of the list from the start. */
			/*  */
			for (index = 0; (index < current_List_Marker) ; index++) {
				if (g_Recent_Frame_List.composite_Search_Key[index] == simple_Key_1) {
					/*  */
					/* It matched the main volatile key, now match the other */
					/*  */
					if (g_Recent_Frame_List.cache_Entries[index].secondary_Search_Key == simple_Key_2) {
						found_Item = TRUE;
						*cached_Data = g_Recent_Frame_List.cache_Entries[index].cached_Data;
						/*  */
						/* It is found, so break out of for loop */
						/*  */
						break;
					}
				}
			}
		}
	
		if (found_Item == TRUE) {
			function_Result = SAFE_TRUE;
		}
		else {
			function_Result = SAFE_FALSE;
		}
	}
	else {
		function_Result = SAFE_FALSE;
	}
	
	return(function_Result);
	
} /* Safe_Boolean Locate_Best_Frame_From_Simple_Ring_Cache */
/*=**************************************************************************=*/		

		
/*=**************************************************************************=*/		
Safe_Boolean Record_Frame_Into_Simple_Ring_Cache(CARDINAL_32 simple_Key_1
																, CARDINAL_32 simple_Key_2
																, CARDINAL_32 index)
{
Safe_Boolean		function_Result;
CARDINAL_32	head_FIFO_Entry_For_Addition;
Simple_Ring_Buffer_Cache_Rec	cache_Master_Ptr;
/* END VAR */

	
	if (g_Recent_Frame_List.structure_Initialized == SAFE_TRUE) {
		head_FIFO_Entry_For_Addition = g_Recent_Frame_List.head_FIFO_Entry_For_Addition;
	
		g_Recent_Frame_List.composite_Search_Key[head_FIFO_Entry_For_Addition] = simple_Key_1;
		g_Recent_Frame_List.cache_Entries[head_FIFO_Entry_For_Addition].secondary_Search_Key = simple_Key_2;
		g_Recent_Frame_List.cache_Entries[head_FIFO_Entry_For_Addition].cached_Data = index;
		
		/*  */
		/* Increment to next position, and wrap around buffer using mask instead of */
		/* subtraction. NOTE: Increment is done in REVERSE so that the*/
		/* search may travel trhough memory in successive contiguous 32 bit accesses */
		/* in a forward direction for speed on interleaved memory setups. */
		/* incrementing in reverse is OK because when it wraps around it merely */
		/* is masked back to high end of power of 2 size of array */
		/*  */
		head_FIFO_Entry_For_Addition -= 1UL;
		head_FIFO_Entry_For_Addition = (head_FIFO_Entry_For_Addition & g_Recent_Frame_List.max_Entry_Bit_Mask);
		g_Recent_Frame_List.head_FIFO_Entry_For_Addition = head_FIFO_Entry_For_Addition;
	
		function_Result = SAFE_TRUE;
	}
	else {
		function_Result = SAFE_FALSE;
	}
	
	return(function_Result);
	
} /* Safe_Boolean Record__Frame_Into_Simple_Ring_Cache */
/*=**************************************************************************=*/


/*=**************************************************************************=*/
/* END OF FILE: Simple_Ring_Cache.c */
