/* START OF FILE: Sound_Input_Device.c */


/*=**************************************************************************=*/
/* [---BEGIN---] FILE HEADER COMMENTS */
/**
 ** SCREEN VIEW = [Adobe Syntax Black-10, Tabs = 4]
 ** PRINTER VIEW = [Adobe Syntax Black-10, Tabs = 4]
 ** =======================================================================
 ** Written by James Merkle
 ** Sound_Input_Device.c - Audio Routines to support Poor Man's Video Phone
 ** (C) Copyright 1993 James Merkle
 ** All rights reserved, except those wishing to "copy" moderate sections of code or files may do so, so long as
 ** such code is never used directly either in animation-related tools or multimedia tools that are  
 ** used in live performance or live broadcast or recorded mass market broadcast or film. Desktop usage not restricted.
 ** Conventional desktop game or entertainment not restricted.
 ** =======================================================================
 **
 **
 ** ============
 ** Revision History
 ** ============
 **
 **  1993.05.03		(James Merkle)
 **						Initial coding started, some from other routines
 **
 **
 **
 ** =================
 ** OTHER INFORMATION
 ** =================
 ** 
 ** 
 **
 **
 **/
/* [----END----] FILE HEADER COMMENTS */
/*=**************************************************************************=*/


/*=**************************************************************************=*/
/* [---BEGIN---] #include AREA */


#include <GestaltEqu.h>
#include <Sound.h>
#include <SoundInput.h>

/* THINK C */
#include <Asm.h>
#include <Files.h>
#include <Traps.h>

#include "Error_Utilities.h"
#include "OS_Utilities.h"
#include "Shell_Dialogs.h"
#include "Shell_Globals.h"
#include "Memory_Utilities.h"

#include "Sound_Utilities.h"
#include "Sound_Play.h"
#include "String_Utilities.h"
#include "Volume_Utilities.h"
#include "Window_Utilities.h"
#include "Simple_Popup_Dialog.h"

#include "Sound_Input_Device.h"

/* [----END----] #include AREA */
/*=**************************************************************************=*/


/*=**************************************************************************=*/
/* [---BEGIN---] #define AREA */

/*  */
/* Fixes made for mediavision card */
/*  */
#define k_TRY_SPOON_FEEDING_PAS16_CARD		1

/*  */
/* If set to "1" "SPBGetRecordingStatus" is used instead of */
/* more practical routine (SPBGetDeviceInfo using siActiveLevels) */
/*  */
#define k_USE_SLOW_FULL_STATUS_FOR_SID_LEVELS		0

/*  */
/* For creating resource files */
/*  */
#define APPLE_RESOURCE_FILE_TYPE					'rsrc'
#define APPLE_RESOURCE_FILE_CREATOR			'RSED'

#define SOUND_RESOURCE_TYPE							'snd '


/*  */
/* Samples minimum voice acceptable */
/* 7.4Khz =  (0x1CFA2E8B)*/
/* 11.1Khz =  (0x2B7745D1)*/
/* 14.8Khz =  (0x39F45D16)*/
/* 22.3Khz =  (0x56EE8BA3)*/
/*  */
#define	MINIMUM_ACCEPTABLE_VOICE_RATE		(0x2B7745D1)

/*  */
/* siGoodQuality, siBestQuality */
/*  */

#define	k_EXAMPLE_DESIRED_QUALITY	siGoodQuality

#define k_TOTAL_MAX_EXPECTED_CHANNELS_EVER						32

/*  */
/* Apple reserves IDs 0 to 8191 */
/*  */
#define	MINIMUM_LEGAL_USER_SOUND_RESOURCE_ID	8192

/*  */
/* This buffer size is not critical. It will be reused again and again */
/* as a double buffer (20480, or 40960, 163840)*/
/* usually only big for mediavision */
/*  */
#define	k_VOICE_BUFFER_MONITOR_SIZE		40960


#define	k_RECORDING_SENTINEL_C									0x12348765

/*  */
/* For paranoia safety in selecting an input source from a popup list. */
/*  */
#define k_AUDIO_INPUT_SOURCE_SELECTION						999

#define k_BUILT_IN_AUDIO_NAME					"\pBuilt-in"
#define k_MEDIAVISION_PAS_16_NAME		"\pPAS16"
#define k_MEDIAVISION_PAS_17_NAME		"\pPAS17"
		
		
/*  */
/* Unique little THINK C constructs to save and restore A5 worlds */
/*  */
#define RememberA5()	do { __Get_A5(); asm { move.l A5,(a1) } } while (0)

#define Setup_A5()			do { asm { move.l A5,-(sp) } __Get_A5(); asm { move.l (a1),A5 } } while (0)
#define Restore_A5()		do { asm { move.l (sp)+,A5 } } while (0)
		
/* [----END----] #define AREA */
/*=**************************************************************************=*/


/*=**************************************************************************=*/
/* [---BEGIN---] TYPE AREA */

	typedef struct {
		/*  */
		/* Because "SPBGetDeviceInfo" might corrupt (it does) and write over adjacent */
		/* memory, for safety this large rect was made to set amplitude level */
		/* information */
		/*  */
		CARDINAL_32		buffer_Safety_Scrap_Memory_1;
		CARDINAL_32		buffer_Safety_Scrap_Memory_2;
		INTEGER_16			level_Meter_State;
		INTEGER_16			level_Meter_Setting;
		CARDINAL_32		buffer_Safety_Scrap_Memory_3;
		CARDINAL_32		buffer_Safety_Scrap_Memory_4;
	} Level_Meter_Setting_Rec;
	
	typedef struct {
		/*  */
		/* Because "SPBGetDeviceInfo" can corrupt and write over adjacent */
		/* memory for safety this large rect was made to receive amplitude level */
		/* information */
		/*  */
		CARDINAL_32		buffer_Safety_Scrap_Memory_1;
		CARDINAL_32		buffer_Safety_Scrap_Memory_2;
		INTEGER_16			level_Meter[k_TOTAL_MAX_EXPECTED_CHANNELS_EVER];
		CARDINAL_32		buffer_Safety_Scrap_Memory_3;
		CARDINAL_32		buffer_Safety_Scrap_Memory_4;
	} Level_Meter_Rec;


/* [----END----] TYPE AREA */
/*=**************************************************************************=*/


/*=**************************************************************************=*/
/* [---BEGIN---] PROTOTYPE AREA */

static void	__Get_A5(void);

pascal Safe_Boolean Prepare_Menu_List_For_Sound(MenuHandle	popup_Menu
													, INTEGER_16 *preferred_Mark
													, CARDINAL_32	*total_Listed_Items
													, CARDINAL_16	type_of_List
													, CARDINAL_16 *popup_References);

/* [----END----] PROTOTYPE AREA */
/*=**************************************************************************=*/


/*=**************************************************************************=*/
/* [---BEGIN---] GLOBAL DEFINITION AREA */


/* [----END----] GLOBAL DEFINITION AREA */
/*=**************************************************************************=*/


/*=**************************************************************************=*/
/* [---BEGIN---] LOCAL STATIC DEFINITION AREA */

static CARDINAL_8	*unit_Name = "\pSound_Input_Device";


/* [----END----] LOCAL STATIC DEFINITION AREA */
/*=**************************************************************************=*/


/*=**************************************************************************=*/
static void __Get_A5(void)
{
	asm {
		bsr.s	@1
		dc.l	0			;  store A5 here
@1		move.l	(sp)+,a1
	}
}
/*=**************************************************************************=*/


/*=**************************************************************************=*/
void Stop_Async_If_Completion_Illegally_Skipped(SPBPtr sound_Recording_Parameters_Ptr)
	/*  */
	/* In a perfect universe this routine is not needed, but because of */
	/* incomplete sound drivers such as the ASI Voice Navigator 2*/
	/* driver that Apple distributed a few times (Quicktime) , it is needed because when */
	/* recording to disk, asynchronously, the completion routine is not called! */
	/*  */
{
	Recording_Ptr	monitor;
	INTEGER_16		recording_Status;
	INTEGER_16		meter_Level;
	INTEGER_16		os_Error;
	CARDINAL_32	total_Samples_To_Record;
	CARDINAL_32	number_Of_Samples_Recorded;
	CARDINAL_32	total_Milliseconds_To_Record;
	CARDINAL_32	total_Milliseconds_Recorded;
/* END VAR */


	monitor = (Recording_Ptr)sound_Recording_Parameters_Ptr->userLong;
	
	if (monitor->recording_Completed != SAFE_TRUE) {

		SPBGetRecordingStatus(sound_Recording_Parameters_Ptr->inRefNum
						, &recording_Status
						, &meter_Level
						, &total_Samples_To_Record
						, &number_Of_Samples_Recorded
						, &total_Milliseconds_To_Record
						, &total_Milliseconds_Recorded);
		if (os_Error != noErr) {
			if (os_Error == siDeviceBusyErr) {
				/*  */
				/* If driver does not support 'SPBGetRecordingStatus' while in use, */
				/* then just spoof 'recording_Status' to a positive (busy) state */
				/*  */
				recording_Status = 1;
			}
			else {
				SysBeep(20);
				Perform_OK_Dialog("\pNOTE: Unexpected, non-fatal error in 'Stop_Async_If_Completion_Illegally_Skipped' when calling 'SPBGetRecordingStatus'!");
			}
		}
		else {
			if (number_Of_Samples_Recorded > 0) {
				if (total_Samples_To_Record <= number_Of_Samples_Recorded) {
					/*  */
					/* Completed the recording, assumedly, so force quit */
					/*  */
					recording_Status = 0;
				}
			}
		}

		if (recording_Status <= 0) {
			Force_Asynchronous_Recording_To_Stop(sound_Recording_Parameters_Ptr);
		}
	}


} /* pascal void Stop_Async_If_Completion_Illegally_Skipped(SPBPtr sound_Recording_Parameters) */
/*=**************************************************************************=*/


/*=**************************************************************************=*/
void Force_Asynchronous_Recording_To_Stop(SPBPtr sound_Recording_Parameters_Ptr)
{
	INTEGER_16		os_Error;
	Recording_Ptr	monitor;
/* END VAR */

	if (sound_Recording_Parameters_Ptr->error != noErr) {
	
	}
	
	monitor = (Recording_Ptr)sound_Recording_Parameters_Ptr->userLong;
		
	monitor->recording_Completed = SAFE_TRUE;

	os_Error = SPBStopRecording(sound_Recording_Parameters_Ptr->inRefNum);
	if (os_Error != noErr) {
	/* ===
		SysBeep(20);
		Perform_OK_Dialog("\pNOTE: Unexpected, non-fatal error in 'Force_Asynchronous_Recording_To_Stop' when calling 'SPBStopRecording'!");
	==== */
	} 

} /* pascal void Force_Asynchronous_Recording_To_Stop(SPBPtr sound_Recording_Parameters) */
/*=**************************************************************************=*/


/*=**************************************************************************=*/
pascal Safe_Boolean Process_Interrupt_Capable_Tasks(SPBPtr sound_Recording_Parameters_Ptr
																				, Ptr buffer_Start
																				, CARDINAL_32 peak_Volume
																				, CARDINAL_32 device_Buffer_Size)
{
	Recording_Ptr	monitor;
/* END VAR */


	monitor = (Recording_Ptr)sound_Recording_Parameters_Ptr->userLong;
		
	if (monitor->audio_Tapping_Structure_Initialized == MEMORY_INITIALIZED) {
	
		if (Transfer_AIFF_Data_During_Interrupts(*monitor->audio_Tapping_Structure_Hdl
															, (CARDINAL_32)buffer_Start
															, device_Buffer_Size) != SAFE_TRUE) {
			/*  */
			/* Ignore problem. Logic controls exceptions properly. */
			/*  */
		}
	}
		
} /* Safe_Boolean Process_Interrupt_Capable_Tasks */
/*=**************************************************************************=*/


/*=**************************************************************************=*/
pascal void Perform_Recording_Interrupt_Task(void)
{
SPBPtr						sound_Recording_Parameters_Ptr;
Ptr							buffer_Start;
CARDINAL_32		peak_Volume;
CARDINAL_32		device_Buffer_Size;
volatile CARDINAL_16		sentinel_Seems_Legitimate;
/* END VAR */


/*===
	Setup_A5();
	==== */

	asm
		{
		
; Output : NONE
; Purpose : Currently only a shell
; Usage : restart sound monitor using alternate buffer, a5 needs to be active in the 
; NOTE : Will eventually be used for siphoning data to our own occiliscope and level monitors 
		
		; WARNING: The following is no good with ASI drivers because the Voice Navigator driver does not 
		; fill in the registers during interrupts as they should

		movem.l	d0-d2/a0-a2/a5,  -(SP)		; Save all registers that may be altered

		move.l		a0, sound_Recording_Parameters_Ptr(a6)
		move.l		a1, buffer_Start(a6)
		move.l 		d0, peak_Volume(a6)
		move.l		d1, device_Buffer_Size(a6)
		
		; WARNING: The following is no good with ASI drivers because the Voice Navigator driver does not 
		; fill in the registers during interrupts as they should
		
		move.l	OFFSET(SPB, userLong)(a0), a1;
		
		/* Restore Global Variable referencing if desired */
		
		move.l	OFFSET(Recording_Rec, non_Vital_global_A5_World)(a1), a5;
		
		clr.w			sentinel_Seems_Legitimate(a6)	; Clear flag used for deciding if C based tasks need examination
		
		cmpi.l			#k_RECORDING_SENTINEL_C, OFFSET(Recording_Rec, recording_Globals_Sentinel)(a1)
		bne				@Error_Handler
		
		addq.l			#1,OFFSET(Recording_Rec, recording_Buffer_Odometer)(a1)	; Increment interrupt level counter
		
		addq.w		#1, sentinel_Seems_Legitimate(a6)
		
		bra				@Finish_And_Exit

@Error_Handler:	
		; Nothing to do. We can't reliably set a global because that is what is failing

@Finish_And_Exit:	

	}
	
	if (sentinel_Seems_Legitimate != 0) {
		if (Process_Interrupt_Capable_Tasks(sound_Recording_Parameters_Ptr, buffer_Start, peak_Volume, device_Buffer_Size) != SAFE_TRUE) {
			/*  */
			/* Ignore Errors. */
			/*  */
		}
	}
		
	asm{
		movem.l	(SP)+,d0-d2/a0-a2/a5	; Restore altered registers
		unlk		a6							; Remove stack frame
		movea.l	(SP)+,a0				; Load invocation address
		jmp		(a0)							; Jump back to caller, All Done
		
	} /* asm */
	

} /* pascal void Perform_Recording_Interrupt_Task(SPBPtr sound_Recording_Parameters) */
/*=**************************************************************************=*/


/*=**************************************************************************=*/
pascal void Perform_Asynch_Recording_Completion(SPBPtr sound_Recording_Parameters_Ptr)
{
OSErr					os_Error;
CARDINAL_16	current_Section;
Recording_Ptr	monitor;
/* END VAR */


	asm {
		movem.l	d0-d1/a0-a2/a5,  -(SP)		; Save all registers that will be altered
	
		move.l 	sound_Recording_Parameters_Ptr(a6), a0

		move.l	OFFSET(SPB, userLong)(a0), a1;
		move.l	a1, monitor(a6)
		
		/* Restore Global Variable referencing if desired */
		
		move.l	OFFSET(Recording_Rec, non_Vital_global_A5_World)(a1), a5;
		
		
		cmpi.l		#k_RECORDING_SENTINEL_C, OFFSET(Recording_Rec, recording_Globals_Sentinel)(a1)

		beq			@Continue_Routine

dc.w	_Debugger
		
@Continue_Routine:
		
	}
	
/* ===
	Setup_A5();
==== */

		
	if ((monitor->recording_Completed == SAFE_TRUE) 
		&& (sound_Recording_Parameters_Ptr->error == abortErr)) {
		
		sound_Recording_Parameters_Ptr->error = noErr;
		
	}
	else if ((monitor->recording_Completed != SAFE_TRUE) 
				&& (sound_Recording_Parameters_Ptr->error == abortErr)) {
				
		monitor->recording_Completed = SAFE_TRUE;
	}
	else {
	
		if (monitor->record_To_New_Buffer_Directly_From_Completion == SAFE_TRUE) {
		
			if (Restart_Recording_Alternating_Buffer(monitor) != TRUE) {
			
				/*  */
				/* Internal recovery routine. We should jot down fact that interrupt time */
				/* callbacks fail and present error at program completion.  */
				/*  */
				monitor->recording_Completed = SAFE_TRUE;
				
				/*  */
				/* Display a warning from main event loop when appropriate */
				/*  */
				g_Severe_Audio_Record_Problem_Needs_Display_Soon	= SAFE_TRUE;
					
			/* ====
				goto Error_Handler;
			=== */
			
			}
		}
		else {
			/*  */
			/* I want to chain a second recording but I cannot do that with the */
			/* Voice Navigator driver, so a lossy method in the main event loop is used */
			/* instead */
			/*  */
			monitor->recording_Completed = SAFE_TRUE;
		}
	}

	goto Finish_And_Exit;
	
Error_Handler:
	monitor->recording_Completed = SAFE_TRUE;
	sound_Recording_Parameters_Ptr->error = abortErr;

Finish_And_Exit:	

	asm {
		movem.l	(SP)+,d0-d1/a0-a2/a5	; Restore altered registers
	} /* asm */

} /* pascal void Perform_Asynch_Recording_Completion(SPBPtr sound_Recording_Parameters) */
/*=**************************************************************************=*/


/*=**************************************************************************=*/
pascal Boolean Restart_Recording_Alternating_Buffer(Recording_Ptr	monitor)
{
OSErr					os_Error;
CARDINAL_16	current_Section;
CARDINAL_32	current_a5_World;
Boolean				function_Result;
/* END VAR */


	function_Result = FALSE;
	if (monitor->recorded_Sound_Data_Section < (k_SWAP_BUFFERS -1)) {
		monitor->recorded_Sound_Data_Section += 1;
	}
	else {
		monitor->recorded_Sound_Data_Section = 0;
	}
	
	current_Section = monitor->recorded_Sound_Data_Section;
	
	monitor->sound_Recording_Block[current_Section].bufferPtr 
					= (Ptr)((CARDINAL_32)*monitor->recorded_Sound_Data_Handle[current_Section] 
					+ monitor->sound_Header_Size);


	monitor->sound_Recording_Block[current_Section].unused1 = 0;

	monitor->sound_Recording_Block[current_Section].inRefNum = monitor->input_Reference;
	
	monitor->sound_Recording_Block[current_Section].count
		= monitor->bytes_To_Record;
		
	monitor->sound_Recording_Block[current_Section].bufferLength
		= monitor->bytes_To_Record;
	
	/*  */
	/* 'milliseconds' will automatically be set to proper value via 'count' and 'sample_Rate' */
	/*  */
	monitor->sound_Recording_Block[current_Section].milliseconds = 0;
	
	/*  */
	/* Set 'error' for safety. (Not actually required) */
	/*  */
	monitor->sound_Recording_Block[current_Section].error = noErr;

	/*  */
	/* Side Note: some (ASIs) sound drivers do not call interrupt tasks when saving to disk */
	/*  */
	if (monitor->asynchronous_Driver_Support_Option != 0){
	
		monitor->sound_Recording_Block[current_Section].completionRoutine
					= &Perform_Asynch_Recording_Completion;
											
											
#if !defined(ORIGINAL_MAC_SYSTEM)
#error "UNDEFINED PARAMETER!"
#endif
#if (ORIGINAL_MAC_SYSTEM == 0)
Dialogs.h
XXXX 		
		monitor->sound_Recording_Block[current_Section].interruptRoutine
					= &Perform_Recording_Interrupt_Task;
#else
		monitor->sound_Recording_Block[current_Section].interruptRoutine
					= (SIInterruptUPP)&Perform_Recording_Interrupt_Task;
#endif 

																		
	}
	else {
		monitor->sound_Recording_Block[current_Section].completionRoutine = nil;
		monitor->sound_Recording_Block[current_Section].interruptRoutine = nil;
	}
			
	/*  */
	/* Store A5 using our normal technique */
	/*  */
	RememberA5();
	
	asm{
		move.l	a5, current_a5_World
	}
	

	monitor->non_Vital_global_A5_World = current_a5_World;
	
	monitor->sound_Recording_Block[current_Section].userLong = (INTEGER_32)monitor;
	monitor->sound_Recording_Block[current_Section].unused1 = 0;


	monitor->recording_Buffer_Odometer = 0;
	monitor->recording_Globals_Sentinel = k_RECORDING_SENTINEL_C;


	/*  */
	/*  */
	/*  */
	/*  */
	/*  */
	/*  */
	/*  */
	/*  */
	/*  */
	/* HACK HACK HACK */
	/*  */
#if !defined(k_TRY_SPOON_FEEDING_PAS16_CARD)
#error "UNDEFINED PARAMETER!"
#endif
#if (k_TRY_SPOON_FEEDING_PAS16_CARD == 1)
	
		if (monitor->record_To_New_Buffer_Directly_From_Completion != SAFE_TRUE) {
		
			os_Error = SPBStopRecording(monitor->input_Reference);
			if (os_Error != noErr) {
				/*  */
				/* Ignore error. This damn stop command was only added because */
				/* PAS16 card kept reporting "input busy -227" if any further commands issued */
				/*  */
			} 
		
		
		
	/* ===	
			os_Error = SPBCloseDevice(monitor->input_Reference);
			if (os_Error != noErr) {
				goto Error_Handler;
			}
	
			os_Error = SPBOpenDevice(monitor->device_Name, siWritePermission, &(monitor->input_Reference));
			if (os_Error != noErr) {
				goto Error_Handler;
			}
	
			if (monitor->input_Reference ==  0) {
				goto Error_Handler;
			}
	==== */		
	
		} /* if (monitor->record_To_New_Buffer_Directly_From_Completion != SAFE_TRUE) */
	
		
#endif /* #if (k_TRY_SPOON_FEEDING_PAS16_CARD ==1) */



	/*  */
	/* Record synchronously or asyncronously to memory, or to */
	/* disk using the sound input device */
	/*  */
	if (monitor->asynchronous_Driver_Support_Option == 0){
		os_Error = SPBRecord(&(monitor->sound_Recording_Block[current_Section]), FALSE);
	}
	else {
		os_Error = SPBRecord(&(monitor->sound_Recording_Block[current_Section]), TRUE);
	}
	
	if (os_Error != noErr) {
		goto Error_Handler;
	}
	
	monitor->recording_Completed = SAFE_FALSE;
	
	/*  */
	/* "monitor->recording_Completed" is normally set*/
	/* by completion routines, but if not asynch, then set manually here */
	/* because all the recording has just concluded */
	/*  */
	if (monitor->asynchronous_Driver_Support_Option == 0){
		monitor->recording_Completed = SAFE_TRUE;
	}	
		
	function_Result = TRUE;
	goto Finish_And_Exit;
	
Error_Handler:	
	function_Result = FALSE;

Finish_And_Exit:	

	return(function_Result);

} /* pascal void Restart_Recording_Alternating_Buffer(SPBPtr sound_Recording_Parameters) */
/*=**************************************************************************=*/


/*=**************************************************************************=*/
Safe_Boolean Extract_Recent_Amplitude_From_SID(Voice_Hdl	voice, CARDINAL_32	*raw_Signal)
{
	Safe_Boolean			function_Result;
	Level_Meter_Rec	levels;
	Recording_Ptr		monitor;
	CARDINAL_32		current_Time;
	INTEGER_16			recording_Status;
	INTEGER_16			meter_Level;
	CARDINAL_32		total_Samples_To_Record;
	CARDINAL_32		number_Samples_Recorded;
	CARDINAL_32		total_Milliseconds_To_Record;
	CARDINAL_32		number_Msecs_Recorded;
	Ptr							response_Ptr;
	OSErr						os_Error;
/* END VAR */


	function_Result = SAFE_FALSE;
		
	*raw_Signal = 0;
	
	if ((**voice).audio_Source == PRIMARY_SOUND_INPUT) {
	
		if ((**voice).info.apple_Sound_Input.recording_Completed != SAFE_TRUE) {
			
			if ((**voice).info.apple_Sound_Input.use_Specific_Channel_Mapping != SAFE_TRUE) {
			
#if !defined(k_USE_SLOW_FULL_STATUS_FOR_SID_LEVELS)
#error "UNDEFINED PARAMETER!"
#endif
#if (k_USE_SLOW_FULL_STATUS_FOR_SID_LEVELS == 1)

				os_Error = SPBGetRecordingStatus( (**voice).info.apple_Sound_Input.input_Reference,
										&recording_Status,
										&meter_Level,
										&total_Samples_To_Record,
										&number_Samples_Recorded,
										&total_Milliseconds_To_Record,
										&number_Msecs_Recorded );
										
				if (os_Error != noErr) {
					goto Error_Handler;
				}
				
				/*  */
				/* Mediavision shows two levels even in mono mode */
				/* strip off second channel */
				/*  */
				*raw_Signal = meter_Level & 0x00FF;

#else

				levels.level_Meter[0] = 0;
				levels.level_Meter[1] = 0;
				levels.level_Meter[2] = 0;
				levels.level_Meter[3] = 0;
				response_Ptr = (Ptr)&(levels.level_Meter[0]);
				os_Error = SPBGetDeviceInfo((**voice).info.apple_Sound_Input.input_Reference
						, siActiveLevels
						, response_Ptr);
						
				/*  */
				/* Mediavision shows two levels even in mono mode */
				/* strip off second channel */
				/*  */
				*raw_Signal = levels.level_Meter[0] & 0x00FF;

						
#endif /* if (k_USE_SLOW_FULL_STATUS_FOR_SID_LEVELS == 1) */
				
			}
			else {
				
				levels.level_Meter[0] = 0;
				levels.level_Meter[1] = 0;
				levels.level_Meter[2] = 0;
				levels.level_Meter[3] = 0;
				response_Ptr = (Ptr)&(levels.level_Meter[0]);
				os_Error = SPBGetDeviceInfo((**voice).info.apple_Sound_Input.input_Reference
						, siActiveLevels
						, response_Ptr);
						
				if (os_Error != noErr) {
					if (os_Error == siUnknownInfoType) {
						os_Error = SPBGetRecordingStatus( (**voice).info.apple_Sound_Input.input_Reference,
												&recording_Status,
												&meter_Level,
												&total_Samples_To_Record,
												&number_Samples_Recorded,
												&total_Milliseconds_To_Record,
												&number_Msecs_Recorded );
						if (os_Error != noErr) {
							goto Error_Handler;
						}
								
						/*  */
						/* Mediavision shows two levels even in mono mode */
						/* strip off second channel */
						/*  */
						*raw_Signal = meter_Level & 0x00FF;
					}
					else {
						goto Error_Handler;
					}
				}
				else {
					*raw_Signal = levels.level_Meter[(**voice).audio_Source_Channel];
				}
				
			} /* NOT if ((**voice).info.apple_Sound_Input.use_Specific_Channel_Mapping ...  */

		}
		else {
			goto Error_Handler;
			
		} /* NOT if ((**voice).info.apple_Sound_Input.recording_Completed != SAFE_TRUE) */
		
	}
	else {
		/*  */
		/* This was improperly called */
		/*  */
		goto Error_Handler;
		
	} /* NOT if ((**voice).audio_Source == PRIMARY_SOUND_INPUT) */

	function_Result = SAFE_TRUE;
	goto Normal_Function_Exit;
	
Error_Handler:
	function_Result = SAFE_FALSE;
	*raw_Signal = 0;
		
Normal_Function_Exit:

	return(function_Result);

} /* Safe_Boolean Extract_Recent_Amplitude_From_SID... */
/*=**************************************************************************=*/


/*=**************************************************************************=*/
Safe_Boolean Start_Sound_Input_Monitoring(Recording_Ptr	monitor
			, Safe_Boolean *activated
			, CARDINAL_32	desired_Active_Channels
			, Safe_Boolean select_Any_Input)
	/*  */
	/* This allows selection and enaction of continuous Sound */
	/* input monitoring, it is not used for setting up the single or double */
	/* mediavision cards */
	/*  */
{
	INTEGER_32			gestalt_Response;
	OSErr						os_Error;
	CARDINAL_32		memory_Requested;
	Str255						note_String;
	Handle					device_Icon_Handle;
	Ptr							response_Ptr;
	SIMPLE_LOOP		count, loop;
	INTEGER_16			driver_Reference;
	INTEGER_16			connected_Info;
	CARDINAL_16		temp_Short;
	Safe_Boolean			avoid_Asynchronous_Recording_Deliberately;
	
	CARDINAL_32		*cardinal_32_Ptr;
	CARDINAL_32		resulting_Index;
	
	Safe_Boolean			lower_Rate_Located;
	Safe_Boolean			sample_Rate_Options_Altered;
	Safe_Boolean			use_Lowest_Standard_Voice_Sample_Rate;
	Fixed						current_Lowest_Rate	;
	Sample_Rate_Rec	sample_Rate_Info;

	Safe_Boolean			valid_Result;
	Safe_Boolean			use_Standard_Voice_Compression;
	Safe_Boolean			capable_Of_Compression_NONE;
	Safe_Boolean			capable_Of_Compression_MAC3;
	Safe_Boolean			capable_Of_Compression_MAC6;
	Compression_Rec	compression_Info;
	
	INTEGER_16			total_Count_Of_Input_Drivers;
	INTEGER_16			buffer_Fragment;
	CARDINAL_32		buffer_Size_For_Playback;
	
	IOParam					parameter_Block;
	INTEGER_32			buffer_Size_To_Record;

	INTEGER_32			response;
	INTEGER_32			error_Position;
	Safe_Boolean			function_Result;
	Boolean					at_Least_One_Channel_Found;
	CARDINAL_32		temp_Mask;
	CARDINAL_32		one_Channel_Selected_Mask;
	CARDINAL_32		current_a5_World;
	Level_Meter_Setting_Rec		level_Meter_Switch;
/* END VAR */


	function_Result = SAFE_FALSE;
	*activated = SAFE_FALSE;
	
	/*  */
	/* Set 'avoid_Asynchronous_Recording_Deliberately' to avoid */
	/* using asynchronous recording because of suspicions of driver problems */
	/* when testing */
	/*  */
	avoid_Asynchronous_Recording_Deliberately = SAFE_FALSE;
	
	/*  */
	/* Set 'use_Standard_Voice_Compression' if disk space */
	/* or network transfer time is critical. It will make sound worse though.*/ 
	/*  */
	use_Standard_Voice_Compression = SAFE_FALSE;

	/*  */
	/* Set 'use_Lowest_Standard_Voice_Sample_Rate' if disk space */
	/* or network transfer time is critical. It will make sound worse though.*/ 
	/*  */
	use_Lowest_Standard_Voice_Sample_Rate = SAFE_FALSE;
	
	
	if (monitor->initialized != SAFE_TRUE) {
			SysBeep(20);
			Perform_OK_Dialog("\pError. Sound not initialized correctly, Naughty programmer.");
			error_Position = 2;
			goto Error_Handler;
	}
	
	for (loop = 0; (loop < k_SWAP_BUFFERS) ; loop++) {
		monitor->recorded_Sound_Data_Memory_In_Use[loop] = MEMORY_NOT_INITIALIZED;
	}

	if (Gestalt_Available() == TRUE) {
		os_Error = Gestalt(gestaltSoundAttr, &gestalt_Response);
		if (os_Error != noErr){
			SysBeep(20);
			Perform_OK_Dialog("\pStrange! Your operating sytem is unaware of IM6 'gestaltSoundAttr' (call Apple! Ha Ha)");
			error_Position = 10;
			goto Error_Handler;
		}
				
		
		if ((gestalt_Response & (0x01 << (gestaltHasSoundInputDevice))) == (0x01 << (gestaltHasSoundInputDevice))) {
		
			/*  */
			/* Cool! at this point we can query the device for speed, */
			/* compression, quality, etc */
			/*  */
		}
		else {
			SysBeep(20);
			Perform_OK_Dialog("\pAccording to 'Gestalt' You have no working or selectable sound input devices");
			function_Result = SAFE_FALSE;
			*activated = SAFE_FALSE;
			goto Normal_Function_Exit;
		}
		
		
		/*  */
		/* Select last entry, not first device. */
		/*  */
		os_Error = noErr;
		for (count =1; ((os_Error == noErr) && (count < 32000)); count++ ){
			os_Error = SPBGetIndexedDevice(count, monitor->device_Name, &device_Icon_Handle);
			if (os_Error != noErr) {
				if (count ==1) {
					error_Position = 20;
					goto Error_Handler;
				}
			}
		}
		
		/*  */
		/* Back up by one for total count of devices */
		/*  */
		total_Count_Of_Input_Drivers = count -1;
		
		if (total_Count_Of_Input_Drivers == 0) {
			function_Result = SAFE_FALSE;
			goto Normal_Function_Exit;
		}
		
		if (select_Any_Input == SAFE_TRUE) {
			resulting_Index = 1;
			valid_Result = SAFE_TRUE;
		}
		else {
			/*  */
			/* Ask user to select a device to use */
			/*  */
			if (Get_Popup_Menu_Choice(k_AUDIO_INPUT_SOURCE_SELECTION
													, &resulting_Index
													, &valid_Result
													, &Prepare_Menu_List_For_Sound) != SAFE_TRUE) {
													
				function_Result = SAFE_FALSE;
				goto Normal_Function_Exit;
			}
		}
		
		if (valid_Result != SAFE_TRUE) {
			function_Result = SAFE_FALSE;
			goto Normal_Function_Exit;
		}
		else {
			os_Error = SPBGetIndexedDevice(resulting_Index, monitor->device_Name, &device_Icon_Handle);
			if (os_Error != noErr) {
				error_Position = 20;
				goto Error_Handler;
			}
		}

		if ((Compare_Pascal_Strings(k_MEDIAVISION_PAS_16_NAME, monitor->device_Name) == SAFE_TRUE)
			|| (Compare_Pascal_Strings(k_MEDIAVISION_PAS_17_NAME, monitor->device_Name) == SAFE_TRUE)) {
			
			/*  */
			/* Nothing special, but eliminate chance of async call back */
			/*  */
			
		}

		for (loop = 0; (loop < k_SWAP_BUFFERS) ; loop++) {
			monitor->recorded_Sound_Data_Handle[loop] = nil;
			monitor->recorded_Sound_Data_Memory_In_Use[loop] = MEMORY_NOT_INITIALIZED;
			
			/*  */
			/* get 60 % of "free" memory, if fragmented and crashes... */
			/* too goddamn bad. But needs to be even number of bytes */
			/*  */
	/* ===
			memory_Requested = ((((FreeMem() * 8) / 10) / 2) * 2);
									, ALLOW_PARTIAL_AMOUNT
	=== */
			memory_Requested = k_VOICE_BUFFER_MONITOR_SIZE;
			if (Create_Handle( (CARDINAL_32 *)&memory_Requested
									, (CARDINAL_32)0L
									, ALLOW_PURGING
									, DISALLOW_PARTIAL_AMOUNT
									, &monitor->recorded_Sound_Data_Handle[loop]
									, &monitor->recorded_Sound_Data_Memory_In_Use[loop]) != SAFE_TRUE ) {
				Perform_OK_Dialog("\pMemory error allocating buffer_Memory (not enough?)");
				
				goto Normal_Function_Exit;
			} 
			
		} /* for (loop = 0; (loop < k_SWAP_BUFFERS) ... */
				
		
		/*  */
		/* Record directly from a sound input device */
		/*  */
	
		monitor->recording_Completed = SAFE_FALSE;
			
		monitor->input_Reference = 0;
		os_Error = SPBOpenDevice(monitor->device_Name, siWritePermission, &(monitor->input_Reference));
		if (os_Error != noErr) {
			error_Position = 30;
			goto Device_Busy_Error_Handler;
		}

		if (monitor->input_Reference ==  0) {
			error_Position = 40;
			goto Error_Handler;
		}
		
		response_Ptr = (Ptr)&connected_Info;
		os_Error = SPBGetDeviceInfo(monitor->input_Reference, siDeviceConnected, response_Ptr);
		if (os_Error != noErr) {
			error_Position = 50;
			goto Error_Handler;
		}

		if (connected_Info != siDeviceIsConnected) {
			/*  */
			/* Will fail for some bizzare reason if device is "Built-in" and not another type*/
			/*  */
			if (Compare_Pascal_Strings(monitor->device_Name, k_BUILT_IN_AUDIO_NAME) != SAFE_TRUE) {
				error_Position = 60;
				goto Error_Handler;
			}
		}
		
/* -  -  -  -  -  -  -  -  -  -  -  -  -  -  -  -  -  -  -  -  -  -  -  -  -  -  -  -  -  -  -  -  -  -  -  -  -  -  -  -  -  -  -  -  -  -  -  -  -  -  - */
/* -  -  -  -  -  -  -  -  -  -  -  -  -  -  -  -  -  -  -  -  -  -  -  -  -  -  -  -  -  -  -  -  -  -  -  -  -  -  -  -  -  -  -  -  -  -  -  -  -  -  - */

		response_Ptr = (Ptr)&(monitor->compression_Type);
		os_Error = SPBGetDeviceInfo(monitor->input_Reference, siCompressionType, response_Ptr);
		if (os_Error != noErr) {
			error_Position = 82;
			goto Error_Handler;
		}

		/*  */
		/* Searching for compression options, and setting them, is */
		/* a little messy, but this is the most direct method. */
		/*  */
		capable_Of_Compression_NONE = SAFE_FALSE;
		capable_Of_Compression_MAC3 = SAFE_FALSE;
		capable_Of_Compression_MAC6 = SAFE_FALSE;
		monitor->compression_Options_Altered = SAFE_FALSE;
		if (use_Standard_Voice_Compression == SAFE_TRUE) {
			compression_Info.dangerous_Stack_Corruption_Handle = nil;
			response_Ptr = (Ptr)&(compression_Info.number_Of_Compression_Types);
			os_Error = SPBGetDeviceInfo(monitor->input_Reference, siCompressionAvailable, response_Ptr);
			if (os_Error != noErr) {
				error_Position = 70;
				goto Error_Handler;
			}
			
			if (compression_Info.number_Of_Compression_Types > 0) {
				if (compression_Info.dangerous_Stack_Corruption_Handle != nil) {
				
					compression_Info.dangerous_Stack_Corruption_Handle_Memory_In_Use = MEMORY_INITIALIZED;
					
					MoveHHi(compression_Info.dangerous_Stack_Corruption_Handle);
					if (MemError() != noErr) {
						error_Position = 71;
						goto Error_Handler;
					}
					
					HLock(compression_Info.dangerous_Stack_Corruption_Handle);
					if (MemError() != noErr) {
						error_Position = 72;
						goto Error_Handler;
					}
				
					cardinal_32_Ptr = (CARDINAL_32 *)(*compression_Info.dangerous_Stack_Corruption_Handle);

					/*  */
					/* Search for 'NONE' 'MAC3' and 'MAC6' setting option flags */
					/* Remember that "number_Of_Compression_Types" does not */
					/* already include the NONE as part of its count. */ 
					/*  */
					for (loop = 0; (loop < (compression_Info.number_Of_Compression_Types + 1)); loop++) {
					
						if (*cardinal_32_Ptr == 'NONE') {
							capable_Of_Compression_NONE = SAFE_TRUE;
						}
						else if (*cardinal_32_Ptr == 'MAC3') {
							capable_Of_Compression_MAC3 = SAFE_TRUE;
						}
						else if (*cardinal_32_Ptr == 'MAC6') {
							capable_Of_Compression_MAC6 = SAFE_TRUE;
						}
						
						/*  */
						/* Increments list pointer address by four bytes */
						/*  */
						cardinal_32_Ptr += 1; 
					}
									
					if (Destroy_Handle(&compression_Info.dangerous_Stack_Corruption_Handle
							, &compression_Info.dangerous_Stack_Corruption_Handle_Memory_In_Use) != SAFE_TRUE) {
						error_Position = 74;
						goto Error_Handler;
					}
				
				} /* if (compression_Info.dangerous_Stack_Corruption_Handle != nil) */
				
			} /* if (compression_Info.number_Of_Compression_Types > 0) */
			
			
			/*  */
			/* Alter compression options if necessary */
			/*  */
			if (capable_Of_Compression_MAC3 == SAFE_TRUE) {
				if (monitor->compression_Type != 'MAC3') {
				
					monitor->previous_Compression_Type = monitor->compression_Type;
					monitor->compression_Type = 'MAC3';
					response_Ptr = (Ptr)&(monitor->compression_Type);
					os_Error = SPBSetDeviceInfo(monitor->input_Reference, siCompressionType, response_Ptr);
					if (os_Error != noErr) {
						error_Position = 75;
						goto Error_Handler;
					}
			
					response_Ptr = (Ptr)&(monitor->compression_Type);
					os_Error = SPBGetDeviceInfo(monitor->input_Reference, siCompressionType, response_Ptr);
					if (os_Error != noErr) {
						error_Position = 76;
						goto Error_Handler;
					}
					
					if (monitor->compression_Type != 'MAC3') {
						error_Position = 77;
						goto Error_Handler;
					}
					
					monitor->compression_Options_Altered = SAFE_TRUE;
				
				} /* if (monitor->compression_Type != 'MAC3') */
				
			} /* if (capable_Of_Compression_MAC3 == SAFE_TRUE) */
			
		} /* if (use_Standard_Voice_Compression == SAFE_TRUE) */
/* -  -  -  -  -  -  -  -  -  -  -  -  -  -  -  -  -  -  -  -  -  -  -  -  -  -  -  -  -  -  -  -  -  -  -  -  -  -  -  -  -  -  -  -  -  -  -  -  -  -  - */
/* -  -  -  -  -  -  -  -  -  -  -  -  -  -  -  -  -  -  -  -  -  -  -  -  -  -  -  -  -  -  -  -  -  -  -  -  -  -  -  -  -  -  -  -  -  -  -  -  -  -  - */

		response_Ptr = (Ptr)&(monitor->sample_Rate);
		os_Error = SPBGetDeviceInfo(monitor->input_Reference, siSampleRate, response_Ptr);
		if (os_Error != noErr) {
			error_Position = 80;
			goto Error_Handler;
		}
						
						
		/*  */
		/* Searching for sample_Rate options, and setting them, is */
		/* a little messy, but this is the most direct method. */
		/*  */
		lower_Rate_Located = SAFE_FALSE;
		sample_Rate_Options_Altered = SAFE_FALSE;
		if (use_Lowest_Standard_Voice_Sample_Rate == SAFE_TRUE) {
			sample_Rate_Info.dangerous_Stack_Corruption_Handle = nil;
			response_Ptr = (Ptr)&(sample_Rate_Info.number_Of_Sample_Rates);
			os_Error = SPBGetDeviceInfo(monitor->input_Reference, siSampleRateAvailable, response_Ptr);
			if (os_Error != noErr) {
				error_Position = 81;
				goto Error_Handler;
			}
			
			if (sample_Rate_Info.number_Of_Sample_Rates > 0) {
				if (sample_Rate_Info.dangerous_Stack_Corruption_Handle != nil) {
				
					sample_Rate_Info.dangerous_Stack_Corruption_Handle_Memory_In_Use = MEMORY_INITIALIZED;
					
					
					MoveHHi(sample_Rate_Info.dangerous_Stack_Corruption_Handle);
					if (MemError() != noErr) {
						error_Position = 82;
						goto Error_Handler;
					}

					HLock(sample_Rate_Info.dangerous_Stack_Corruption_Handle);
					if (MemError() != noErr) {
						error_Position = 82;
						goto Error_Handler;
					}
				
					cardinal_32_Ptr = (CARDINAL_32 *)(*sample_Rate_Info.dangerous_Stack_Corruption_Handle);

					/*  */
					/* Search for acceptable sample rate for voice digitization */
					/*  */
					current_Lowest_Rate = 0xFFFFFFFF;
					for (loop = 0; (loop < sample_Rate_Info.number_Of_Sample_Rates ); loop++) {
					
						if ((CARDINAL_32)(*cardinal_32_Ptr) < (CARDINAL_32)current_Lowest_Rate) {
							if ((CARDINAL_32)*cardinal_32_Ptr >= (CARDINAL_32)MINIMUM_ACCEPTABLE_VOICE_RATE ) {
								current_Lowest_Rate = (CARDINAL_32)*cardinal_32_Ptr;
								lower_Rate_Located = SAFE_TRUE;
							}
						}
						
						/*  */
						/* Increments list pointer address by four bytes */
						/*  */
						cardinal_32_Ptr += 1; 
					}
									
					if (Destroy_Handle(&sample_Rate_Info.dangerous_Stack_Corruption_Handle
							, &sample_Rate_Info.dangerous_Stack_Corruption_Handle_Memory_In_Use) != SAFE_TRUE) {
						error_Position = 83;
						goto Error_Handler;
					}
				
				} /* if (sample_Rate_Info.dangerous_Stack_Corruption_Handle != nil) */
				
			} /* if (sample_Rate_Info.number_Of_Sample_Rates > 0) */
			
			
			/*  */
			/* Alter sample_Rate options if necessary */
			/*  */
			if (lower_Rate_Located == SAFE_TRUE) {
			
				monitor->previous_Sample_Rate = monitor->sample_Rate;
				monitor->sample_Rate = current_Lowest_Rate;
				response_Ptr = (Ptr)&(monitor->sample_Rate);
				os_Error = SPBSetDeviceInfo(monitor->input_Reference, siSampleRate, response_Ptr);
				if (os_Error != noErr) {
					error_Position = 84;
					goto Error_Handler;
				}
		
				response_Ptr = (Ptr)&(monitor->sample_Rate);
				os_Error = SPBGetDeviceInfo(monitor->input_Reference, siSampleRate, response_Ptr);
				if (os_Error != noErr) {
					error_Position = 85;
					goto Error_Handler;
				}
				
				if (monitor->sample_Rate != current_Lowest_Rate) {
					error_Position = 86;
					goto Error_Handler;
				}
				
				sample_Rate_Options_Altered = SAFE_TRUE;
				
			} /* if (lower_Rate_Located == SAFE_TRUE) */
			else {
				monitor->previous_Sample_Rate = monitor->sample_Rate;
			}
			
		} /* if (use_Lowest_Standard_Voice_Sample_Rate == SAFE_TRUE) */
		else {
			monitor->previous_Sample_Rate = monitor->sample_Rate;
		}
						
						
/* -  -  -  -  -  -  -  -  -  -  -  -  -  -  -  -  -  -  -  -  -  -  -  -  -  -  -  -  -  -  -  -  -  -  -  -  -  -  -  -  -  -  -  -  -  -  -  -  -  -  - */
/* -  -  -  -  -  -  -  -  -  -  -  -  -  -  -  -  -  -  -  -  -  -  -  -  -  -  -  -  -  -  -  -  -  -  -  -  -  -  -  -  -  -  -  -  -  -  -  -  -  -  - */


		response_Ptr = (Ptr)&(level_Meter_Switch.level_Meter_State);
		os_Error = SPBGetDeviceInfo(monitor->input_Reference, siLevelMeterOnOff, response_Ptr);
		if (os_Error != noErr) {
			monitor->use_Specific_Channel_Mapping = SAFE_FALSE;
		}
		
		monitor->previous_Level_Meter_State = level_Meter_Switch.level_Meter_State;
		
		monitor->desired_Level_Meter_State = 1;
		
		if (monitor->desired_Level_Meter_State != monitor->previous_Level_Meter_State) {
			
			level_Meter_Switch.level_Meter_State = monitor->desired_Level_Meter_State;
			/*  */
			/*  Just in case, this uses two adjacent locations (like many other SID */
			/* commands do, set both */
			/*  */
			level_Meter_Switch.level_Meter_Setting = monitor->desired_Level_Meter_State;
			
			
			response_Ptr = (Ptr)&(level_Meter_Switch.level_Meter_State);
			os_Error = SPBSetDeviceInfo(monitor->input_Reference, siLevelMeterOnOff, response_Ptr);
			if (os_Error != noErr) {
				monitor->use_Specific_Channel_Mapping = SAFE_FALSE;
			}
		}

		response_Ptr = (Ptr)&(monitor->previous_Activated_Channel_Total);
		os_Error = SPBGetDeviceInfo(monitor->input_Reference, siNumberChannels, response_Ptr);
		if (os_Error != noErr) {
			error_Position = 52;
			goto Error_Handler;
		}

/* ======		
		response_Ptr = (Ptr)&(monitor->previous_Active_Channel_Total);
		os_Error = SPBGetDeviceInfo(monitor->input_Reference, siActiveChannels, response_Ptr);
		if (os_Error != noErr) {
			error_Position = 52;
			goto Error_Handler;
		}
===== */		
		
		if (monitor->previous_Activated_Channel_Total > k_TOTAL_MAX_EXPECTED_CHANNELS_EVER) {
			error_Position = 53;
			goto Error_Handler;
		}
		
		monitor->use_Specific_Channel_Mapping = SAFE_TRUE;
		response_Ptr = (Ptr)&(monitor->previous_Activated_Channels);
		os_Error = SPBGetDeviceInfo(monitor->input_Reference, siActiveChannels, response_Ptr);
		if (os_Error != noErr) {
			monitor->use_Specific_Channel_Mapping = SAFE_FALSE;
		}
		
		if (monitor->use_Specific_Channel_Mapping != SAFE_TRUE) {
			desired_Active_Channels = 0x80000000;
			monitor->desired_Active_Channels = desired_Active_Channels;
			monitor->previous_Activated_Channels = desired_Active_Channels;
		}
		else {
		
			/*  */
			/* "0" is allowed and it means anything (1st channel located) */
			/*  */
			if (desired_Active_Channels == 0) {
				desired_Active_Channels = monitor->previous_Activated_Channels;
			}
			
			/*  */
			/* Restrict the recording to activate only one channel */
			/*  */
			temp_Mask = 0x80000000;
			at_Least_One_Channel_Found = FALSE;
			for (loop = 0; ((loop < 31) && (at_Least_One_Channel_Found != TRUE)); loop++) {
				if (temp_Mask & desired_Active_Channels) {
					at_Least_One_Channel_Found = TRUE;
					one_Channel_Selected_Mask = temp_Mask;
				}
				else {
					temp_Mask = ((CARDINAL_32)temp_Mask >> 1);
				}
			}
	
			if (at_Least_One_Channel_Found == TRUE) {
				if (one_Channel_Selected_Mask != monitor->previous_Activated_Channels) {
				
					response_Ptr = (Ptr)&(one_Channel_Selected_Mask);
					os_Error = SPBSetDeviceInfo(monitor->input_Reference, siActiveChannels, response_Ptr);
					if (os_Error != noErr) {
						error_Position = 55;
						goto Error_Handler;
					}
					
					response_Ptr = (Ptr)&(monitor->desired_Active_Channels);
					os_Error = SPBGetDeviceInfo(monitor->input_Reference, siActiveChannels, response_Ptr);
					if (os_Error != noErr) {
						error_Position = 56;
						goto Error_Handler;
					}
				}
				
			} /* if (at_Least_One_Channel_Found == TRUE) */
			else {
				monitor->desired_Active_Channels = monitor->previous_Activated_Channels;
			}
					
		} /* NOT if (monitor->use_Specific_Channel_Mapping != SAFE_TRUE) */
		
		
		response_Ptr = (Ptr)&(monitor->number_Of_Channels);
		os_Error = SPBGetDeviceInfo(monitor->input_Reference, siNumberChannels, response_Ptr);
		if (os_Error != noErr) {
			error_Position = 87;
			goto Error_Handler;
		}


		response_Ptr = (Ptr)&(monitor->sample_Size);
		os_Error = SPBGetDeviceInfo(monitor->input_Reference, siSampleSize, response_Ptr);
		if (os_Error != noErr) {
			/*  */
			/* NOTE: Error is possible if compression is enabled, in theory */
			/*  */
			error_Position = 89;
			goto Error_Handler;
		}

		response_Ptr = (Ptr)&(monitor->asynchronous_Driver_Support_Option);
		os_Error = SPBGetDeviceInfo(monitor->input_Reference, siAsync, response_Ptr);
		if (os_Error != noErr) {
			if (os_Error == siUnknownInfoType) {
				/*  */
				/* If request is not programmed into driver then assume it is not capable */
				/*  */
				monitor->asynchronous_Driver_Support_Option = 0;
			}
			else {
				error_Position = 90;
				goto Error_Handler;
			}
		}
		
		/*  */
		/* For debugging purposes, honor "avoid_Asynchronous_Recording_Deliberately" */
		/*  */
		if (avoid_Asynchronous_Recording_Deliberately == SAFE_TRUE) {
			monitor->asynchronous_Driver_Support_Option = 0;
		}
		
		if (monitor->asynchronous_Driver_Support_Option == 0){ 
		
			Concat_Many(note_String, 3, "\pYour device's recording driver [ ", monitor->device_Name , "\p ] \
is currently incapable of background recording (not full-featured) . You will not be able to halt \
the recording until fully recorded. Acceptable?");

			if (Perform_Yes_No_Dialog(note_String ) != TRUE) {
				Perform_OK_Dialog("\pRecording to Disk/Memory aborting.");
				goto Normal_Function_Exit;
			}
		}

		monitor->twos_Complement_Storage_Option_Understood = SAFE_TRUE;
		response_Ptr = (Ptr)&(monitor->twos_Complement_Storage_Option);
		os_Error = SPBGetDeviceInfo(monitor->input_Reference, siTwosComplementOnOff, response_Ptr);
		if (os_Error != noErr) {
			if (os_Error == siUnknownInfoType) {
				/*  */
				/* If request is not programmed into driver then assume it is not set */
				/*  */
				monitor->twos_Complement_Storage_Option = 0;
				monitor->twos_Complement_Storage_Option_Understood = SAFE_FALSE;
			}
			else {
				error_Position = 92;
				goto Error_Handler;
			}
		}
		monitor->original_Twos_Complement_Storage_Option = monitor->twos_Complement_Storage_Option;
		

		/*  */
		/* If recording to memory (an snd resource) then twos complement */
		/* is unwanted because the 8 bit sample centerline is supposed to  */
		/* be 0x7F not 0x00. Check the setting and switch if needed */
		/* If recording to disk (an AIFF file) then twos complement */
		/* is desired because the 8 bit sample centerline is supposed to  */
		/* be 0x00 not 0x7F. Check the setting and switch if needed */
		/* Right now this is set for AIFF only if device is Apple SID type */
		/*  */
		if (Compare_Pascal_Strings(monitor->device_Name, k_BUILT_IN_AUDIO_NAME) == SAFE_TRUE) {

			if (monitor->twos_Complement_Storage_Option == 0) {
				monitor->twos_Complement_Storage_Option = 1;
				response_Ptr = (Ptr)&(monitor->twos_Complement_Storage_Option);
				os_Error = SPBSetDeviceInfo(monitor->input_Reference, siTwosComplementOnOff, response_Ptr);
				if (os_Error != noErr) {
					error_Position = 94;
					goto Error_Handler;
				}
			}
		}


		/*  */
		/* This loop initializes each section though only the first will be started */
		/*  */
		for (buffer_Fragment = 0; (buffer_Fragment < k_SWAP_BUFFERS) ; buffer_Fragment++) {
	
			temp_Short = monitor->sound_Header_Size;
			os_Error = SetupSndHeader((SndListHandle)monitor->recorded_Sound_Data_Handle[buffer_Fragment]
									, monitor->number_Of_Channels
									, monitor->sample_Rate
									, monitor->sample_Size
									, monitor->compression_Type
									, 60
									, 0
									, (INTEGER_16 *)&temp_Short);
			monitor->sound_Header_Size = temp_Short;
			if (os_Error != noErr) {
				error_Position = 110;
				goto Error_Handler;
			}
	
			buffer_Size_To_Record = GetHandleSize(monitor->recorded_Sound_Data_Handle[buffer_Fragment]);
			if (MemError() != noErr) {
				error_Position = 120;
				goto Error_Handler;
			}
			
			/*  */
			/* Reserve room for the sound files header */
			/*  */
			buffer_Size_To_Record = buffer_Size_To_Record - monitor->sound_Header_Size;
					
			/*  */
			/* HACK HACK HACK to limit size */
			/*  */
			/* buffer_Size_To_Record = ((64L) * (1024L)); */
	
			monitor->bytes_To_Record = buffer_Size_To_Record;
			
			/*  */
			/* Important to lock memory down before starting the recording */
			/*  */
			MoveHHi(monitor->recorded_Sound_Data_Handle[buffer_Fragment]);
			os_Error = MemError();
			if ((os_Error != noErr) && (os_Error != -117)){
				error_Position = 128;
				goto Error_Handler;
			}

			HLock(monitor->recorded_Sound_Data_Handle[buffer_Fragment]);
			if (MemError() != noErr) {
				error_Position = 130;
				goto Error_Handler;
			}
	
			monitor->sound_Recording_Block[buffer_Fragment].inRefNum = monitor->input_Reference;
			
			/*  */
			/* Side Note: some (ASIs) sound drivers do not call interrupt tasks when saving to disk */
			/*  */
			if (monitor->asynchronous_Driver_Support_Option != 0){
				monitor->sound_Recording_Block[buffer_Fragment].completionRoutine
							= &Perform_Asynch_Recording_Completion;
													
													
#if !defined(ORIGINAL_MAC_SYSTEM)
#error "UNDEFINED PARAMETER!"
#endif
#if (ORIGINAL_MAC_SYSTEM == 0)
Dialogs.h
XXXX 		
				monitor->sound_Recording_Block[buffer_Fragment].interruptRoutine
							= &Perform_Recording_Interrupt_Task;
#else
				monitor->sound_Recording_Block[buffer_Fragment].interruptRoutine
							= (SIInterruptUPP)&Perform_Recording_Interrupt_Task;
#endif 

																				
			}
			else {
				monitor->sound_Recording_Block[buffer_Fragment].completionRoutine = nil;
				monitor->sound_Recording_Block[buffer_Fragment].interruptRoutine = nil;
			}
							
			monitor->sound_Recording_Block[buffer_Fragment].userLong = (INTEGER_32)monitor;
			monitor->sound_Recording_Block[buffer_Fragment].unused1 = 0;
				
			monitor->sound_Recording_Block[buffer_Fragment].count = buffer_Size_To_Record;
			monitor->sound_Recording_Block[buffer_Fragment].bufferLength = buffer_Size_To_Record;
			
			monitor->sound_Recording_Block[buffer_Fragment].bufferPtr 
				= (Ptr)((CARDINAL_32)*monitor->recorded_Sound_Data_Handle[buffer_Fragment] 
					+ monitor->sound_Header_Size);
			
			/*  */
			/* 'milliseconds' will automatically be set to proper value via 'count' and 'sample_Rate' */
			/*  */
			monitor->sound_Recording_Block[buffer_Fragment].milliseconds = 0;
			
			/*  */
			/* Set 'error' for safety. (Not actually required) */
			/*  */
			monitor->sound_Recording_Block[buffer_Fragment].error = noErr;
			
		} /* for (buffer_Fragment = 0; (buffer_Fragment < k_SWAP_BUFFERS) ... */
		
		
		/*  */
		/* If recording directly from a sound input device that allows interrupt */
		/* time initiation of recording into second buffer from completion callback then do so. */
		/*  */
		
		if (Compare_Pascal_Strings(monitor->device_Name, k_BUILT_IN_AUDIO_NAME) == SAFE_TRUE) {
		
			monitor->record_To_New_Buffer_Directly_From_Completion = SAFE_TRUE;
		}
		else {
			monitor->record_To_New_Buffer_Directly_From_Completion = SAFE_FALSE;
		}
			
		/*  */
		/* Store A5 using our normal technique */
		/*  */
		RememberA5();
		
		asm{
			move.l	a5, current_a5_World
		}
		
		monitor->non_Vital_global_A5_World = current_a5_World;
		
		/*  */
		/* This marker toggles between the various buffers */
		/*  */
		monitor->recorded_Sound_Data_Section = 0;
		
		monitor->recording_Buffer_Odometer = 0;
		monitor->recording_Globals_Sentinel = k_RECORDING_SENTINEL_C;
		
		monitor->audio_Tapping_Structure_Initialized = MEMORY_NOT_INITIALIZED;
		monitor->audio_Tapping_Structure_Hdl = nil;
		
		monitor->initialized = SAFE_TRUE;
		
		/*  */
		/* Record synchronously or asyncronously to memory, or to */
		/* disk using the sound input device */
		/*  */
		if (monitor->asynchronous_Driver_Support_Option == 0){
			os_Error = SPBRecord(&(monitor->sound_Recording_Block[monitor->recorded_Sound_Data_Section]), FALSE);
		}
		else {
			os_Error = SPBRecord(&(monitor->sound_Recording_Block[monitor->recorded_Sound_Data_Section]), TRUE);
		}
		
		if (os_Error != noErr) {
			error_Position = 140;
			goto Error_Handler;
		}
		
		/*  */
		/* "monitor->recording_Completed" is normally set*/
		/* by completion routines, but if not asynch, then set manually here */
		/* because all the recording has just concluded */
		/*  */
		if (monitor->asynchronous_Driver_Support_Option == 0){
			monitor->recording_Completed = SAFE_TRUE;
		}	
		
		*activated = SAFE_TRUE;
				
	} /* if (Gestalt_Available() == TRUE) */

	function_Result = SAFE_TRUE;
	goto Normal_Function_Exit;
	
	
Device_Busy_Error_Handler:
	SysBeep(20);
	Perform_OK_Dialog("\pSorry, the Sound Input Device that was selected reported that it was already selected for exclusive control. Try another if possible.");
	function_Result = SAFE_FALSE;
	goto Error_Handler_Completion;

Error_Handler:
	SysBeep(20);
	Make_String_Long_String(note_String, "\pStart_Sound_Input_Monitoring failed! At location [",error_Position, NO_LIMIT,"\p ], unexpected error.");
	Perform_OK_Dialog(note_String);
	function_Result = SAFE_FALSE;

Error_Handler_Completion:

	for (loop = 0; (loop < k_SWAP_BUFFERS) ; loop++) {
		if (monitor->recorded_Sound_Data_Memory_In_Use[loop] == MEMORY_INITIALIZED) {
			if (Destroy_Handle(&monitor->recorded_Sound_Data_Handle[loop], &monitor->recorded_Sound_Data_Memory_In_Use[loop]) != SAFE_TRUE) {
				/*  */
				/* Ignore errors in this loop */
				/*  */
			}
		}
	}

Normal_Function_Exit:
	return(function_Result);

} /* void Start_Sound_Input_Monitoring(void) */
/*=**************************************************************************=*/


/*=**************************************************************************=*/
Safe_Boolean Stop_Sound_Input_Monitoring(Recording_Ptr	monitor)
{
	Safe_Boolean		function_Result;
	OSErr					os_Error;
	INTEGER_32		error_Position;
	Ptr						response_Ptr;
	SndChannelPtr	current_Snd_Channel_Ptr;
	Str255					note_String;
	CARDINAL_32	total_Delay_Time;
	CARDINAL_32	start_Time;
	CARDINAL_32	total_Internal_Buffers;
	CARDINAL_16	temp_Short;
	SIMPLE_LOOP	loop;
	INTEGER_16		voice;
	Level_Meter_Setting_Rec		level_Meter_Switch;
/* END VAR */


	function_Result = SAFE_FALSE;


	if (monitor->recording_Completed != SAFE_TRUE) {
				Force_Asynchronous_Recording_To_Stop(
						&(monitor->sound_Recording_Block[monitor->recorded_Sound_Data_Section]));
	}
	total_Delay_Time = (CARDINAL_32)TickCount() - start_Time;
	total_Internal_Buffers = monitor->recording_Buffer_Odometer;
	
	for (loop = 0; (loop < k_SWAP_BUFFERS) ; loop++) {

		if (monitor->recorded_Sound_Data_Memory_In_Use[loop] == MEMORY_INITIALIZED) {
			HUnlock(monitor->recorded_Sound_Data_Handle[loop]);
			if (MemError() != noErr) {
				error_Position = 160;
				goto Error_Handler;
			}
			
			/*  */
			/* Must use 'SetupSndHeader' to set the resulting size because three different headers exist! */
			/*  */
			temp_Short = monitor->sound_Header_Size;
			os_Error = SetupSndHeader((SndListHandle)monitor->recorded_Sound_Data_Handle[loop]
									, monitor->number_Of_Channels
									, monitor->sample_Rate
									, monitor->sample_Size
									, monitor->compression_Type
									, 60
									, monitor->sound_Recording_Block[loop].count
									, (INTEGER_16 *)&temp_Short);
			monitor->sound_Header_Size = (CARDINAL_16)temp_Short;
			if (os_Error != noErr) {
				error_Position = 160;
				goto Error_Handler;
			}
			
		} /* if (monitor->recorded_Sound_Data_Memory_In_Use[loop] == MEMORY_INITIALIZED) */
	}
	

	/*  */
	/* Reset it to the original settings it was found with, for courtesy */
	/*  */
/* ======	
	
	os_Error = SPBCloseDevice(monitor->input_Reference);
	if (os_Error != noErr){
		error_Position = 170;
		goto Error_Handler;
	}
	

	monitor->input_Reference = 0;
	os_Error = SPBOpenDevice(monitor->device_Name, siWritePermission, &(monitor->input_Reference));
	if (os_Error != noErr) {
		error_Position = 189;
		goto Error_Handler;
	}

	if (monitor->input_Reference ==  0) {
		error_Position = 190;
		goto Error_Handler;
	}
	
===== */

	/*  */
	/* Restore compression options back to original setting */
	/*  */
	if (monitor->compression_Options_Altered == SAFE_TRUE) {
		
			monitor->compression_Type = monitor->previous_Compression_Type;
			response_Ptr = (Ptr)&(monitor->compression_Type);
			os_Error = SPBSetDeviceInfo(monitor->input_Reference, siCompressionType, response_Ptr);
			if (os_Error != noErr) {
				Concat_Many(note_String, 3,"\pError while trying to reset the previous compression settings for ", monitor->device_Name, "\p ID = ");
				Concat_Long_To_String(note_String, os_Error, NO_LIMIT);
				Perform_OK_Dialog(note_String);
				goto Error_Handler;
			}

			response_Ptr = (Ptr)&(monitor->compression_Type);
			os_Error = SPBGetDeviceInfo(monitor->input_Reference, siCompressionType, response_Ptr);
			if (os_Error != noErr) {
				error_Position = 195;
				goto Error_Handler;
			}
			
			if (monitor->compression_Type != monitor->previous_Compression_Type) {
				error_Position = 200;
				goto Error_Handler;
			}
							
	} /* if (monitor->compression_Options_Altered == SAFE_TRUE) */


	/*  */
	/* Restore setting of twos complement back to original setting */
	/*  */
	if (monitor->twos_Complement_Storage_Option != monitor->original_Twos_Complement_Storage_Option) {
		if (monitor->twos_Complement_Storage_Option_Understood == SAFE_TRUE) {
			monitor->twos_Complement_Storage_Option = monitor->original_Twos_Complement_Storage_Option;
			response_Ptr = (Ptr)&(monitor->twos_Complement_Storage_Option);
			os_Error = SPBSetDeviceInfo(monitor->input_Reference, siTwosComplementOnOff, response_Ptr);
			if (os_Error != noErr) {
				error_Position = 210;
				goto Error_Handler;
			}
		}
	}
	
	if (monitor->sample_Rate != monitor->previous_Sample_Rate) {

			monitor->sample_Rate = monitor->previous_Sample_Rate;
			response_Ptr = (Ptr)&(monitor->sample_Rate);
			os_Error = SPBSetDeviceInfo(monitor->input_Reference, siSampleRate, response_Ptr);
			if (os_Error != noErr) {
				error_Position = 192;
				goto Error_Handler;
			}
	}
	
	if (monitor->desired_Active_Channels != monitor->previous_Activated_Channels) {
		response_Ptr = (Ptr)&(monitor->previous_Activated_Channels);
		os_Error = SPBSetDeviceInfo(monitor->input_Reference, siActiveChannels, response_Ptr);
		if (os_Error != noErr) {
			error_Position = 53;
			goto Error_Handler;
		}
	}
		

/* ====
	if (monitor->number_Of_Channels != monitor->previous_Activated_Channel_Total) {
	
			monitor->number_Of_Channels = monitor->previous_Activated_Channel_Total;
			response_Ptr = (Ptr)&(monitor->number_Of_Channels);
			
			siActiveChannels
			xxx
			
			os_Error = SPBSetDeviceInfo(monitor->input_Reference, siNumberChannels, response_Ptr);
			if (os_Error != noErr) {
				error_Position = 193;
				goto Error_Handler;
			}
	}
	
==== */

	if (monitor->desired_Level_Meter_State != monitor->previous_Level_Meter_State) {
		
		level_Meter_Switch.level_Meter_State = monitor->previous_Level_Meter_State;
		/*  */
		/*  Just in case, this uses two adjacent locations (like many other SID */
		/* commands do, set both */
		/*  */
		level_Meter_Switch.level_Meter_Setting = monitor->previous_Level_Meter_State;
		
		
		response_Ptr = (Ptr)&(level_Meter_Switch.level_Meter_State);
		os_Error = SPBSetDeviceInfo(monitor->input_Reference, siLevelMeterOnOff, response_Ptr);
		if (os_Error != noErr) {
			monitor->use_Specific_Channel_Mapping = SAFE_FALSE;
		}
	}
	
	/*  */
	/* Mediavision card is problematic about closing when active */
	/*  */
	SystemTask();
	SystemTask();
	os_Error = SPBCloseDevice(monitor->input_Reference);
	if (os_Error != noErr){
		error_Position = 220;
		goto Error_Handler;
	}
	SystemTask();

	/*  */
	/* Offer debugging option of replaying the last buffer sample */
	/*  */

#if (7 == 11)	
	if (Perform_Yes_No_Dialog("\pWould you like to hear the last sound buffer segment?") == TRUE) {
	
		current_Snd_Channel_Ptr = nil;
		
		os_Error = SndNewChannel(&current_Snd_Channel_Ptr, sampledSynth, initMono, nil);
		if (os_Error != noErr){
			Perform_OK_Dialog("\pSound playback not possible at this moment due to system limitations");
		}
		else {
		
			for (loop = 0; (loop < k_SWAP_BUFFERS) ; loop++) {
				os_Error = SndPlay(current_Snd_Channel_Ptr, monitor->recorded_Sound_Data_Handle[loop], FALSE);
				if (os_Error != noErr){
					error_Position = 180;
					goto Error_Handler;
				}
			}
	
			os_Error = SndDisposeChannel(current_Snd_Channel_Ptr, TRUE);
			if (os_Error != noErr){
				error_Position = 186;
				goto Error_Handler;
			}
			
			current_Snd_Channel_Ptr = nil;
	
		} /* NOT if (os_Error != noErr) */
	
	} /* if (Perform_Yes_No_Dialog("\pWould you like to hear the last sound buffer segment?") == TRUE) */
#endif /* (7 == 11) */


	
	function_Result = SAFE_TRUE;
	goto Normal_Function_Exit;
	
Error_Handler:
	SysBeep(20);
	Make_String_Long_String(note_String, "\pStop_Sound_Input_Monitoring failed! At location [",error_Position, NO_LIMIT,"\p ], unexpected error.");
	Perform_OK_Dialog(note_String);
	function_Result = SAFE_FALSE;

Normal_Function_Exit:

	for (loop = 0; (loop < k_SWAP_BUFFERS) ; loop++) {
		if (monitor->recorded_Sound_Data_Memory_In_Use[loop] == MEMORY_INITIALIZED) {
			if (Destroy_Handle(&monitor->recorded_Sound_Data_Handle[loop]
					, &monitor->recorded_Sound_Data_Memory_In_Use[loop]) != SAFE_TRUE) {
				function_Result = SAFE_FALSE;
			}
		}
	}

	return(function_Result);

} /* Safe_Boolean Stop_Sound_Input_Monitoring ... */
/*=**************************************************************************=*/


/*=**************************************************************************=*/
pascal Safe_Boolean Prepare_Menu_List_For_Sound(MenuHandle	popup_Menu
													, INTEGER_16 *preferred_Mark
													, CARDINAL_32	*total_Listed_Items
													, CARDINAL_16	type_of_List
													, CARDINAL_16 *popup_References)
{
	Safe_Boolean			function_Result;
	CARDINAL_32		items_Found;
	CARDINAL_32		total_Items;
	INTEGER_16			index;
	OSErr						os_Error;
	Str255						device_Name;
	INTEGER_32			gestalt_Response;
	Handle					device_Icon_Handle;
	CARDINAL_8			hot_Key;
	Str255						menu_String;
/* END VAR */


	function_Result = SAFE_FALSE;
	*preferred_Mark = 1;
	*total_Listed_Items = 0;
	
	popup_References[*total_Listed_Items] = 0;
	
	switch(type_of_List) {
	
		case k_AUDIO_INPUT_SOURCE_SELECTION:
			*total_Listed_Items = 0;
				

			if (Gestalt_Available() == TRUE) {
			
				os_Error = Gestalt(gestaltSoundAttr, &gestalt_Response);
				if (os_Error == noErr){						
					
					if ((gestalt_Response & (0x01 << (gestaltHasSoundInputDevice))) == (0x01 << (gestaltHasSoundInputDevice))) {
					
						os_Error = noErr;
						for (index =1; ((os_Error == noErr) && (index < 32000)); index++ ){
						
							os_Error = SPBGetIndexedDevice(index, device_Name, &device_Icon_Handle);
							if (os_Error == noErr) {
							
								/*  */
								/* Craig Marciniak indicated an important filter technique */
								/* to eliminate macros from menu names. It involves copying strings into markers. */
								/*  */
								menu_String[0] = 1;
								menu_String[1] = 'X';
								AppendMenu(popup_Menu, menu_String);
								popup_References[*total_Listed_Items] = *total_Listed_Items + 1;
								Copy_Pascal_String(&(device_Name[0]), menu_String);
								*total_Listed_Items += 1;
								SetItem(popup_Menu, *total_Listed_Items, &menu_String[0]);
							}
						}
						
					}
					else {
						SysBeep(20);
						Perform_OK_Dialog("\pAccording to 'Gestalt' You have no working or selectable sound input devices");
					}
					
				} /* if (os_Error == noErr) */
				
			} /* if (Gestalt_Available() */

		break;

	
		default:
			goto Error_Handler;
	}		
			
	/*  */
	/* Locate new position of original selection */
	/*  */
	
	function_Result = SAFE_TRUE;
	goto Normal_Function_Exit;
	
Error_Handler:
	function_Result = SAFE_FALSE;

Normal_Function_Exit:
	return(function_Result);

} /* Safe_Boolean Prepare_Menu_List_For_Sound(MenuHandle	popup_Menu) */
/*=**************************************************************************=*/


/*=**************************************************************************=*/
/* END OF FILE: Sound_Input_Device.c */
