/* START OF FILE: Sound_Play.c */


/*=**************************************************************************=*/
/* [---BEGIN---] FILE HEADER COMMENTS */
/**
 ** SCREEN VIEW = [Adobe Syntax Black-10, Tabs = 4]
 ** PRINTER VIEW = [Adobe Syntax Black-10, Tabs = 4]
 ** =======================================================================
 ** Written by James Merkle
 ** Sound_Play.c - Play AIFF from Hard disk and allow repeated sampling of audio using 3 buffers.
 ** (C) Copyright 1995 James Merkle
 ** =======================================================================
 **
 **
 ** ============
 ** Revision History
 ** ============
 **
 **  1995.05.19		(James Merkle)
 **						Initial coding started from scratch. 
 **
 **
 **
 ** =================
 ** OTHER INFORMATION
 ** =================
 ** 
 ** 
 **
 **
 **/
/* [----END----] FILE HEADER COMMENTS */
/*=**************************************************************************=*/


/*=**************************************************************************=*/
/* [---BEGIN---] #include AREA */


#include <AIFF.h>
#include <Sound.h>

#include "Shell_Globals.h"
#include "Error_Utilities.h"
#include "Memory_Utilities.h"
#include "Time_Utilities.h"

#include "OS_Utilities.h"
/* ====
#include "Dreaming_Stuff.h"
=== */

#include "Shell_Globals.h"

#include "Sound_Play.h"

/* [----END----] #include AREA */
/*=**************************************************************************=*/


/*=**************************************************************************=*/
/* [---BEGIN---] #define AREA */

/*  */
/* A smaller buffer seems better for streaming hard disks */
/* use two 56K buffers. Many disks stream when under 64K */
/* and 32K is insufficient if streaming more than one file at a time. */
/*  */
#define k_USEFUL_DISK_BUFFER_MAX						(56UL * 1024UL *2UL)

#define k_USEFUL_DISK_BUFFER_MIN						(32UL * 1024UL *2UL)

#define k_VALID_PLAYBACK_BUFFER_SIZE_MINIMUM	(32UL * 1024UL *2UL)


/* ===
#define k_USEFUL_DISK_BUFFER_MAX			(1048576UL)
==== */

/* [----END----] #define AREA */
/*=**************************************************************************=*/


/*=**************************************************************************=*/
/* [---BEGIN---] TYPE AREA */


/* [----END----] TYPE AREA */
/*=**************************************************************************=*/


/*=**************************************************************************=*/
/* [---BEGIN---] PROTOTYPE AREA */

pascal void Set_Play_Completion_Callback(SndChannelPtr	active_Snd_Channel_Ptr);

Safe_Boolean	Check_If_Digital_Audio_File_Play_Capable(void);

Safe_Boolean Destroy_AIFF_Playback_Structure(Async_AIFF_Ptr	 playback);

static void Mark_Playback_Flag_Completed(volatile Safe_Boolean *playing_AIFF_Now);

Safe_Boolean Add_Callback_Command_To_Channel(SndChannelPtr	active_Snd_Channel_Ptr, Async_AIFF_Ptr	playback);

/* [----END----] PROTOTYPE AREA */
/*=**************************************************************************=*/


/*=**************************************************************************=*/
/* [---BEGIN---] GLOBAL DEFINITION AREA */

	volatile CARDINAL_32		g_AIFF_Structures_Opened;
	Async_AIFF_Rec	g_Playback[k_MAX_FILES_ALLOWED_PLAYING];


/* [----END----] GLOBAL DEFINITION AREA */
/*=**************************************************************************=*/


/*=**************************************************************************=*/
/* [---BEGIN---] LOCAL STATIC DEFINITION AREA */

static CARDINAL_8	*unit_Name;

pascal void (*g_Address_Of_Inform_Main_Routine_Audio_Closing)(Async_AIFF_Ptr);

#if !defined(ORIGINAL_MAC_SYSTEM)
#error "UNDEFINED PARAMETER!"
#endif
#if (ORIGINAL_MAC_SYSTEM == 0)

FilePlayCompletionUPP		g_File_Play_Completion_UPP;

SndCallBackUPP		g_Sound_Callback_UPP;

#endif 


/* [----END----] LOCAL STATIC DEFINITION AREA */
/*=**************************************************************************=*/


/*=**************************************************************************=*/
Safe_Boolean Initialize_Sound_Play(Ptr file_Completion_Notice_Callback)
{
	CARDINAL_8			*static_Unit_Name = "\pSound_Play";
	SIMPLE_LOOP		loop;
/* END VAR */



	g_AIFF_Structures_Opened = 0;	
	for (loop = 0; (loop < k_MAX_FILES_ALLOWED_PLAYING); loop++) {
		g_Playback[loop].structure_Initialized = SAFE_FALSE;
	}
	
	
	unit_Name = static_Unit_Name;
	

	/*  */
	/* UPP not needed in ANSI compliant PPC compiler. Just use Ptr. */
	/* This is not touched by Mac OS. */
	/*  */
	g_Address_Of_Inform_Main_Routine_Audio_Closing = (void *)file_Completion_Notice_Callback;
	
	
#if (ORIGINAL_MAC_SYSTEM == 0)
	g_File_Play_Completion_UPP = (FilePlayCompletionUPP)NewRoutineDescriptor(
					(ProcPtr)(&Set_Play_Completion_Callback)
					, uppFilePlayCompletionProcInfo
					, GetCurrentISA());
				
	g_Sound_Callback_UPP = (SndCallBackUPP)NewRoutineDescriptor(
					(ProcPtr)(&Set_Play_Completion_Callback)
					, uppSndCallBackProcInfo
					, GetCurrentISA());
#endif

	return(SAFE_TRUE);
}
/*=**************************************************************************=*/


/*=**************************************************************************=*/
Safe_Boolean Destroy_Sound_Play(void)
{
	Safe_Boolean		function_Result;
/* END VAR */


/*  */
/* Global persistant structures are deleted here */
/*  */

	if (g_AIFF_Structures_Opened > 0) {
		Stop_Playing_Any_AIFF_Asynchronously(SAFE_FALSE);
	}

	function_Result = SAFE_TRUE;
	return(function_Result);
}
/*=**************************************************************************=*/


/*=**************************************************************************=*/
Safe_Boolean	Check_If_Digital_Audio_File_Play_Capable(void)
{
	CARDINAL_32	gestalt_Result;
	CARDINAL_32	features_Needed;
	Safe_Boolean		function_Result;
	Safe_Boolean		equipment_Acceptable;
	OSErr				os_Error;
/* END VAR */


	function_Result = SAFE_FALSE;
	
	equipment_Acceptable = SAFE_FALSE;
	
	if (Gestalt_Available() == TRUE) {
		
		os_Error = Gestalt((OSType)gestaltSoundAttr, (INTEGER_32 *)&gestalt_Result);
		if (os_Error == noErr) {

			/*  */
			/* Determine if Sound Manager has 16 bit file streaming */
			/*  */

			features_Needed = ((1L) << gestaltSoundIOMgrPresent);
			features_Needed = (features_Needed | ((1L) << gestaltMultiChannels));
			features_Needed = (features_Needed | ((1L) << gestaltSndPlayDoubleBuffer));
			features_Needed = (features_Needed | ((1L) << gestalt16BitAudioSupport));
			
			if ((gestalt_Result & features_Needed) == features_Needed){
				equipment_Acceptable = SAFE_TRUE;
			}
		}
		
		
		if (equipment_Acceptable == SAFE_TRUE) {
			/*  */
			/* Determine if real and trackable memory esists (multifinder or 7.0+) */
			/*  */
			equipment_Acceptable = SAFE_FALSE;
			os_Error = Gestalt((OSType)gestaltOSAttr, (INTEGER_32 *)&gestalt_Result);
			if (os_Error == noErr) {
				
				if ((CARDINAL_32)((gestalt_Result >> 4) & 0x0007) == 7){								
					equipment_Acceptable = SAFE_TRUE;
				}
			}
		}
		
		if (equipment_Acceptable == SAFE_TRUE) {
			/*  */
			/* Determine if FSSpec calls exists (QuickTime or 7.0+) */
			/*  */
			equipment_Acceptable = SAFE_FALSE;
			os_Error = Gestalt((OSType)gestaltFSAttr, (INTEGER_32 *)&gestalt_Result);
			if (os_Error == noErr) {
				
				if ((gestalt_Result & 0x02) != 0){
					equipment_Acceptable = SAFE_TRUE;
				}
			}
		}
				
	} /* if (Gestalt_Available() == TRUE)  */
	
	if (equipment_Acceptable == SAFE_TRUE) {
		function_Result = SAFE_TRUE;
	}
	else {
		function_Result = SAFE_FALSE;
	}

	return(function_Result);
	
}/* Safe_Boolean	Check_If_Digital_Audio_File_Play_Capable ... */
/*=**************************************************************************=*/


/*=**************************************************************************=*/
Safe_Boolean Find_Structure_Index_For_AIFF_Pointer(Async_AIFF_Ptr file_Control_Ptr, CARDINAL_32 *playback_Index)
{
	CARDINAL_32	items_Found;
	CARDINAL_32	items_To_Locate;
	SIMPLE_LOOP	loop;
	Safe_Boolean		function_Result;
	Boolean			item_Located;
/* END VAR */


	*playback_Index = 0;
	
	if (g_AIFF_Structures_Opened < 1) {
		goto Error_Handler;
	}

	items_To_Locate = g_AIFF_Structures_Opened;
	items_Found = 0;
	item_Located = FALSE;
	for (loop = 0; ((items_Found < items_To_Locate) && (loop < k_MAX_FILES_ALLOWED_PLAYING)); loop++) {
		if (g_Playback[loop].structure_Initialized == SAFE_TRUE) {
			items_Found += 1;
			
			if (&g_Playback[loop] == file_Control_Ptr) {
				item_Located = TRUE;
				*playback_Index = loop;
			}
		}
	}

	function_Result = SAFE_TRUE;
	goto Normal_Function_Exit;
	
Error_Handler:
	function_Result = SAFE_FALSE;
	
Normal_Function_Exit:

	return(function_Result);
}
/*=**************************************************************************=*/


/*=**************************************************************************=*/
Safe_Boolean Play_AIFF_Audio_File_Asyncronously(FSSpec *selected_File
															, Safe_Boolean install_Tap_On_Channel
															, Async_AIFF_Ptr	 *structure_Selected)
/*  */
/* This async file routine will work on a Macs with */
/* Trackable memory, or System 7, Quicktime (if system 6) */
/* and Sound Manager 3.0 abilities */
/* Most 1991 macs are fine. */
/*  */
{
	Ptr					call_Back_Routine_Address;
	OSErr				os_Error;
	CARDINAL_32	saved_Global_Reference;
	CARDINAL_32	memory_Requested;
	Safe_Boolean		function_Result;
	SIMPLE_LOOP	loop;
	Boolean			found_Item;
	Async_AIFF_Ptr	 playback;
	CARDINAL_8		*code_Name;
	CARDINAL_32	starting_Tick_Time;
	CARDINAL_32	elapsed_Time;
/* END VAR */


	code_Name = "\pPlay_AIFF_Audio_File_Asyncronously";
	function_Result = SAFE_FALSE;
	*structure_Selected = nil;
	
	if (g_AIFF_Structures_Opened >= k_MAX_FILES_ALLOWED_PLAYING) {
		Stop_Playing_Any_AIFF_Asynchronously(SAFE_TRUE);
	}
	
	/*  */
	/* Check if remedied and a slot open now. */
	/*  */
	if (g_AIFF_Structures_Opened >= k_MAX_FILES_ALLOWED_PLAYING) {
		goto Normal_Function_Exit;
	}
	
	/*  */
	/* Find an open slot */
	/*  */
	found_Item = FALSE;
	for (loop = 0; ((loop < k_MAX_FILES_ALLOWED_PLAYING) && (found_Item != TRUE)); loop++) {
		if (g_Playback[loop].structure_Initialized != SAFE_TRUE) {
			found_Item = TRUE;
			playback = &g_Playback[loop];
		}
	}

	if (found_Item != TRUE) {
		goto Normal_Function_Exit;
	}

	g_AIFF_Structures_Opened += 1;
	
	playback->structure_Initialized = SAFE_TRUE;
	playback->playing_AIFF_Now = SAFE_FALSE;
	
	playback->buffered_Sound_Data_Handle = nil;
	playback->buffered_Sound_Data_Memory_In_Use = SAFE_FALSE;
	playback->active_Snd_Channel_Ptr = nil;
	playback->file_Reference = 0;

	if (Check_If_Digital_Audio_File_Play_Capable() != SAFE_TRUE) {
		goto Normal_Function_Exit;
	}
	
	playback->audio_Tap_Initialized = SAFE_FALSE;
	playback->callback_Count = 0;
	playback->callback_Debug_Info_More_Data_Ptr = 0;
	
	playback->active_Snd_Channel_Ptr = nil;
	
	
#if !defined(ORIGINAL_MAC_SYSTEM)
#error "UNDEFINED PARAMETER!"
#endif
#if (ORIGINAL_MAC_SYSTEM == 1)
	call_Back_Routine_Address = nil;
	call_Back_Routine_Address = (Ptr)&Set_Sound_Callback;
	
	os_Error = SndNewChannel(&(playback->active_Snd_Channel_Ptr), sampledSynth, initStereo, (SndCallBackUPP)call_Back_Routine_Address);
#else
	/*  */
	/* pass nill for g_Sound_Callback_UPP if no callback needed */
	/*  */
	os_Error = SndNewChannel(&(playback->active_Snd_Channel_Ptr), sampledSynth, initStereo, (SndCallBackUPP)g_Sound_Callback_UPP);
#endif /* NOT if (ORIGINAL_MAC_SYSTEM == 1) */

	if (os_Error != noErr){
		Display_Serious_Error_Message(unit_Name, code_Name, "\pSound playback not possible at this moment due to system limitations", "\pToo many channels open?");
		goto Error_Handler;
	}
	else {
	
		if (Create_Pointer_To_Held_RAM_Pages_In_Handle(SAFE_TRUE
													, 3
													, k_USEFUL_DISK_BUFFER_MAX
													, k_USEFUL_DISK_BUFFER_MIN
													, &playback->buffered_Sound_Data_Handle
													, &playback->buffered_Sound_Data_Memory_In_Use
													, &playback->buffered_Sound_Data_Start_Page_Ptr
													, &playback->buffered_Sound_Data_Handle_In_Temporary
													, &playback->playback_Buffer_Size
													, &playback->pages_Held_And_Needs_Release
													) != SAFE_TRUE) {
			Display_Low_Memory_Error_Message(unit_Name, code_Name, k_USEFUL_DISK_BUFFER_MIN);
			goto Error_Handler;
		}

		os_Error = FSpOpenDF(selected_File, fsRdPerm, &(playback->file_Reference));
		if (os_Error != noErr) {
			goto Error_Handler;
		}
		
		/*  */
		/* Playback sound, but make sure the buffer is big enough to use */
		/* otherwise allow the Operating System to manage the double buffers */
		/*  */
			
#if !defined(k_USES_A5_FOR_GLOBALS)
#error "UNDEFINED PARAMETER!"
#endif
#if (k_USES_A5_FOR_GLOBALS == 1)
		/*  */
		/* Save global reference into "userInfo" for callback later */
		/* note that it is A4 (code resource) not A5 if not Appl. */
		/* PowerMac does not need it */
		/*  */
#if (ORIGINAL_MAC_SYSTEM == 1)
		asm {
			move.l 	a5, saved_Global_Reference(a6)				; Save original a5
		}
		
		playback->saved_Global_Reference = saved_Global_Reference;
		
		playback->active_Snd_Channel_Ptr->userInfo = (CARDINAL_32)playback;
#endif
#else
		asm {
			move.l 	a4, saved_Global_Reference(a6)				; Save original a4
		}
		
		playback->saved_Global_Reference = saved_Global_Reference;
		
		playback->active_Snd_Channel_Ptr->userInfo = (CARDINAL_32)playback;
		
#endif /* if (k_USES_A5_FOR_GLOBALS == 1) */		
		
		playback->playing_AIFF_Now = SAFE_TRUE;

		if (playback->playback_Buffer_Size > k_VALID_PLAYBACK_BUFFER_SIZE_MINIMUM) {
		
		
#if !defined(ORIGINAL_MAC_SYSTEM)
#error "UNDEFINED PARAMETER!"
#endif
#if (ORIGINAL_MAC_SYSTEM == 1)
			os_Error = SndStartFilePlay(playback->active_Snd_Channel_Ptr
								, playback->file_Reference
								, 0
								, playback->playback_Buffer_Size
								, (*playback->buffered_Sound_Data_Handle)
								, nil
								, (FilePlayCompletionUPP)&Set_Play_Completion_Callback
								, TRUE);
#else

			os_Error = SndStartFilePlay(playback->active_Snd_Channel_Ptr
								, playback->file_Reference
								, 0
								, playback->playback_Buffer_Size
								, (*playback->buffered_Sound_Data_Handle)
								, nil
								, (FilePlayCompletionUPP)g_File_Play_Completion_UPP
								, TRUE);
#endif /* NOT if (ORIGINAL_MAC_SYSTEM == 1) */

		}
		else {				

#if (ORIGINAL_MAC_SYSTEM == 1)
			os_Error = SndStartFilePlay(playback->active_Snd_Channel_Ptr
								, playback->file_Reference
								, 0
								, k_VALID_PLAYBACK_BUFFER_SIZE_MINIMUM
								, nil
								, nil
								, (FilePlayCompletionUPP)&Set_Play_Completion_Callback
								,TRUE);
#else 

			os_Error = SndStartFilePlay(playback->active_Snd_Channel_Ptr
								, playback->file_Reference
								, 0
								, k_VALID_PLAYBACK_BUFFER_SIZE_MINIMUM
								, nil
								, nil
								, (FilePlayCompletionUPP)g_File_Play_Completion_UPP
								, TRUE);

#endif /* NOT if (ORIGINAL_MAC_SYSTEM == 1) */
								
								
		}
		
		if (os_Error != noErr){
			goto Error_Handler;
		}

						
	} /* NOT if (os_Error != noErr) */
	
	
	if (install_Tap_On_Channel == SAFE_TRUE) {
		if (Install_Tap_On_Get_More_Data(playback->active_Snd_Channel_Ptr, 0, &playback->audio_Tap_Ptr) != SAFE_TRUE) {
			/*  */
			/* Not normally expected to fail if number of taps avaiable */
			/* is equal to number of open sounds playing. */
			/* Try again with one second of max delay for channel to get ready. */
			/* to play sound file. */
			/*  */
			if (Delay_Tap_Until_Good_Opportunity((Ptr)playback) != SAFE_TRUE) {
				/*  */
				/* Ignore. Not normally expected to fail? */
				/* causes no great harm. Cn fail if all taps are already busy */
				/*  */
				playback->audio_Tap_Initialized = SAFE_TRUE;
			}
			else {
				playback->audio_Tap_Initialized = SAFE_TRUE;
			}
		}
		else {
			playback->audio_Tap_Initialized = SAFE_TRUE;
		}
	}

	*structure_Selected = playback;

	function_Result = SAFE_TRUE;
	goto Normal_Function_Exit;
	
Error_Handler:
	function_Result = SAFE_FALSE;
	
Normal_Function_Exit:

	return(function_Result);
	
} /* Safe_Boolean Play_AIFF_Audio_File_Asyncronously... */
/*=**************************************************************************=*/


/*=**************************************************************************=*/
Safe_Boolean Destroy_AIFF_Playback_Structure(Async_AIFF_Ptr	 playback)
{
	Safe_Boolean	function_Result;
	OSErr				os_Error;
	CARDINAL_8		*code_Name;
/* END VAR */


	code_Name = "\pDestroy_AIFF_Playback_Structure";
	function_Result = SAFE_FALSE;
	
	if (playback->structure_Initialized != SAFE_TRUE) {
		goto Error_Handler;
	}
	
	if (playback->playing_AIFF_Now == SAFE_TRUE) {
		goto Error_Handler;
	}

	if ((CARDINAL_32)g_Address_Of_Inform_Main_Routine_Audio_Closing != 0) {
		/*  */
		/* Inform main client of closure */
		/* This is a normal inter-PPC-segment jump and should NOT need a ProcPtr */
		/*  */
		g_Address_Of_Inform_Main_Routine_Audio_Closing(playback);
	}

	if (playback->active_Snd_Channel_Ptr != nil) {
		os_Error = SndDisposeChannel(playback->active_Snd_Channel_Ptr, TRUE);
		if (os_Error != noErr){
			/*  */
			/* Ignore error in this version, we are quitting anyways*/
			/*  */
		}
		playback->active_Snd_Channel_Ptr = nil;
	}
	
	if (playback->file_Reference != 0) {
		os_Error = FSClose(playback->file_Reference);
		if (os_Error != noErr){
			/*  */
			/* Ignore error in this version, we are quitting anyways*/
			/*  */
		}
		playback->file_Reference = 0;
	}


	if (playback->audio_Tap_Initialized == SAFE_TRUE) {
		if (Destroy_Audio_Tap(playback->audio_Tap_Ptr) != SAFE_TRUE) {
			/*  */
			/* Ignore problems releasing tap structure. */
			/*  */
		}
	}

	if (playback->buffered_Sound_Data_Memory_In_Use == SAFE_TRUE) {
	
		if (playback->pages_Held_And_Needs_Release == SAFE_TRUE) {
			if (Unhold_Memory(playback->buffered_Sound_Data_Start_Page_Ptr
										, playback->playback_Buffer_Size) != SAFE_TRUE) {
										
				/*  */
				/* Ignore problems unholding pages. */
				/*  */
			}
		}
		
		/*  */
		/* In future version this will probably be RAM locked into memory */
		/* to avoid virtual memory swapping at start of playback */
		/* For now its just a place marker to remind me */
		/*  */
		if (playback->buffered_Sound_Data_Handle_In_Temporary == SAFE_TRUE) {
		
			MFTempHUnlock(playback->buffered_Sound_Data_Handle,&os_Error);
			if (os_Error != noErr) {
				/*  */
				/* Ignore error in this version, we are quitting anyways*/
				/*  */
			}
			
			MFTempDisposHandle(playback->buffered_Sound_Data_Handle,&os_Error);
			if (os_Error != noErr){
				/*  */
				/* Ignore error in this version, we are quitting anyways*/
				/*  */
			}
			playback->buffered_Sound_Data_Memory_In_Use = SAFE_FALSE;
			playback->buffered_Sound_Data_Handle = nil;
		}
		else {
			if (Destroy_Handle((Handle *)&playback->buffered_Sound_Data_Handle
								, &playback->buffered_Sound_Data_Memory_In_Use) != SAFE_TRUE) {
								
				Display_Serious_Error_Message(unit_Name
								, code_Name
								, "\pError with DisposHandle"
								, "\pSystem Corrupted?");
			}
		}
	} /* if (playback->buffered_Sound_Data_Memory_In_Use == SAFE_TRUE) */

	function_Result = SAFE_TRUE;
	goto Normal_Function_Exit;
	
Error_Handler:
	function_Result = SAFE_FALSE;
	
Normal_Function_Exit:

	return(function_Result);

} /* Safe_Boolean Destroy_AIFF_Playback_Structure... */
/*=**************************************************************************=*/


/*=**************************************************************************=*/
void Close_Any_Sound_Files_If_Needed(void)
{
	CARDINAL_32	items_Found;
	CARDINAL_32	items_To_Locate;
	SIMPLE_LOOP	loop;
/* END VAR */


	items_To_Locate = g_AIFF_Structures_Opened;
	items_Found = 0;
	for (loop = 0; ((items_Found < items_To_Locate) && (loop < k_MAX_FILES_ALLOWED_PLAYING)); loop++) {
		if (g_Playback[loop].structure_Initialized == SAFE_TRUE) {
			items_Found += 1;
			if (g_Playback[loop].playing_AIFF_Now != SAFE_TRUE) {
						
				if (Destroy_AIFF_Playback_Structure(&g_Playback[loop]) != SAFE_TRUE) {
					/*  */
					/* Ignore error in this version, we are quitting anyways*/
					/*  */
				}				
				
				if (g_AIFF_Structures_Opened > 0) {
					g_AIFF_Structures_Opened = g_AIFF_Structures_Opened - 1;
				}
				
				g_Playback[loop].structure_Initialized = SAFE_FALSE;
			}
		}
	}

} /* void Close_Any_Sound_Files_If_Needed... */
/*=**************************************************************************=*/


/*=**************************************************************************=*/
Safe_Boolean Stop_Playing_AIFF_Asynchronously(Async_AIFF_Ptr	 playback)
{
	Safe_Boolean		function_Result;
	OSErr				os_Error;
	CARDINAL_8		*code_Name;
/* END VAR */


	function_Result = SAFE_FALSE;
	
	if (playback->structure_Initialized != SAFE_TRUE) {
		goto Error_Handler;
	}

	if (playback->playing_AIFF_Now == SAFE_TRUE) {
		os_Error = SndStopFilePlay(playback->active_Snd_Channel_Ptr,TRUE);
		if (os_Error != noErr){
			/*  */
			/* Ignore error in this version, we are quitting anyways*/
			/*  */
		}
		Mark_Playback_Flag_Completed(&playback->playing_AIFF_Now);
	}
	
	if (Destroy_AIFF_Playback_Structure(playback) != SAFE_TRUE) {
		/*  */
		/* Ignore error in this version, we are quitting anyways*/
		/*  */
	}				
	
	if (g_AIFF_Structures_Opened > 0) {
		g_AIFF_Structures_Opened = g_AIFF_Structures_Opened - 1;
	}
	
	playback->structure_Initialized = SAFE_FALSE;

	function_Result = SAFE_TRUE;
	goto Normal_Function_Exit;
	
Error_Handler:
	function_Result = SAFE_FALSE;
	
Normal_Function_Exit:

	return(function_Result);

} /* Safe_Boolean Stop_Playing_AIFF_Asynchronously... */
/*=**************************************************************************=*/


/*=**************************************************************************=*/
void Stop_Playing_Any_AIFF_Asynchronously(Safe_Boolean stop_Oldest_Item_Found)
{
	CARDINAL_32	items_Found;
	CARDINAL_32	items_To_Locate;
	CARDINAL_32	oldest_Elasped_Time;
	CARDINAL_32	oldest_Index;
	CARDINAL_32	elapsed_Time;
	CARDINAL_32	current_Time;
	Async_AIFF_Ptr	 playback;
	OSErr				os_Error;
	Safe_Boolean		result;
	CARDINAL_8		*code_Name;
	SIMPLE_LOOP	loop;
/* END VAR */


	code_Name = "\pStop_Playing_Any_AIFF_Asynchronously";

	oldest_Elasped_Time = 0;
	oldest_Index = 0;
	
	if (stop_Oldest_Item_Found == SAFE_TRUE){
		/*  */
		/* Locate oldest otherwise all are stopped*/
		/*  */
		current_Time = TickCount();
		
		items_To_Locate = g_AIFF_Structures_Opened;
		items_Found = 0;
		for (loop = 0; ((loop < k_MAX_FILES_ALLOWED_PLAYING) && (items_Found < items_To_Locate)); loop++) {
			if (g_Playback[loop].structure_Initialized == SAFE_TRUE) {
			
				if (items_Found == 0) {
					/*  */
					/* Point to something for safety. */
					/*  */
					oldest_Index = loop;
				}
				
				items_Found += 1;
								
				elapsed_Time = Calculate_Total_Ticks(g_Playback[loop].starting_Tick_Time
								, current_Time);
				
				if (oldest_Elasped_Time < elapsed_Time) {
					oldest_Elasped_Time = elapsed_Time;
					oldest_Index = loop;
				}
			}
		}
	}
		
	items_To_Locate = g_AIFF_Structures_Opened;
	items_Found = 0;
	for (loop = 0; ((loop < k_MAX_FILES_ALLOWED_PLAYING) && (items_Found < items_To_Locate)); loop++) {
		if (g_Playback[loop].structure_Initialized == SAFE_TRUE) {
			items_Found += 1;
			
			playback = &g_Playback[loop];
			
			if ((stop_Oldest_Item_Found != SAFE_TRUE) || ((stop_Oldest_Item_Found == SAFE_TRUE) && (oldest_Index == loop))){
			
				result = Stop_Playing_AIFF_Asynchronously(playback);
				if (result != SAFE_TRUE) {
					/*  */
					/* Ignore error. It was handled as best as can be. */
					/*  */
				}

			} /* if ((stop_Oldest_Item_Found != SAFE_TRUE) || ((stop_Oldest_Item_Found == SAFE_TRUE) */
			
		} /* if (g_Playback[loop].structure_Initialized == SAFE_TRUE) */
		
	} /* for (loop = 0; ((loop < k_MAX_FILES_ALLOWED_PLAYING)  ... */
		
	
} /* void Stop_Playing_Any_AIFF_Asynchronously ... */
/*=**************************************************************************=*/


/*=**************************************************************************=*/
static void Mark_Playback_Flag_Completed(volatile Safe_Boolean *playing_AIFF_Now)
{
	/*  */
	/* This Bottleneck is just for Debugging to study when */
	/* Set_Play_Completion_Callback is called. I was hoping it is */
	/* called when file is fully played or is mostly fully played and */
	/* not called right at beginning of playback */
	/* Ensure it is not "Extern" to this file. */ 
	/*  */

/* ===
	Debugger();
=== */
	
	*playing_AIFF_Now = SAFE_FALSE;
	
}
/*=**************************************************************************=*/


/*=**************************************************************************=*/
Safe_Boolean Unpause_Halted_Channel(SndChannelPtr	active_Snd_Channel_Ptr)
/*  */
/* This is not currently used, but it is useful if needed */
/*  */
{
	Safe_Boolean		function_Result;
	SndCommand	command;
	OSErr				os_Error;
/* END VAR */



	command.cmd = resumeCmd;
	command.param1 = 0;
	command.param2 = 0;
	
	os_Error = SndDoImmediate(active_Snd_Channel_Ptr, &command);
	if (os_Error != noErr) {
		goto Error_Handler;
	}
	
	function_Result = SAFE_TRUE;
	goto Normal_Function_Exit;
	
Error_Handler:
	function_Result = SAFE_FALSE;
	
Normal_Function_Exit:

	return(function_Result);
} 
/*=**************************************************************************=*/


/*=**************************************************************************=*/
Safe_Boolean Add_Callback_Command_To_Channel(SndChannelPtr	active_Snd_Channel_Ptr, Async_AIFF_Ptr	playback)
/*  */
/* This is not currently used, but it is useful if needed */
/* because it can install a callback in the channel to mark */
/* a border point where a new tap should begin one day if */
/* a single channel is shared between overlapping sound effects on */
/* two or more channels. Or even one channel. This marker can be useful. */
/*  */
{
	Safe_Boolean		function_Result;
	SndCommand	command;
	OSErr				os_Error;
/* END VAR */


#if (7 == 11)

	command.cmd = waitCmd;
	command.param1 = 100UL;
	command.param2 = 0;
	
	os_Error = SndDoCommand(active_Snd_Channel_Ptr, &command, TRUE);
	if (os_Error != noErr) {
		goto Error_Handler;
	}

	command.cmd = pauseCmd;
	command.param1 = 0;
	command.param2 = 0;
	
	os_Error = SndDoCommand(active_Snd_Channel_Ptr, &command, TRUE);
	if (os_Error != noErr) {
		goto Error_Handler;
	}
	
	command.cmd = waitCmd;
	command.param1 = 200UL; 
	command.param2 = 0;
	
	os_Error = SndDoCommand(active_Snd_Channel_Ptr, &command, TRUE);
	if (os_Error != noErr) {
		goto Error_Handler;
	}

#endif /* (#if (7 == 11) */

	command.cmd = callBackCmd;
	command.param1 = (CARDINAL_32)playback;
	command.param2 = 0;
	
	os_Error = SndDoCommand(active_Snd_Channel_Ptr, &command, TRUE);
	if (os_Error != noErr) {
		goto Error_Handler;
	}
	

	function_Result = SAFE_TRUE;
	goto Normal_Function_Exit;
	
Error_Handler:
	function_Result = SAFE_FALSE;
	
Normal_Function_Exit:

	return(function_Result);
} 
/*=**************************************************************************=*/


/*=**************************************************************************=*/
#if (ORIGINAL_MAC_SYSTEM == 1)
pascal void Set_Play_Completion_Callback(SndChannelPtr	active_Snd_Channel_Ptr)
/*  */
/* Globals not actually relied upon in this routine yet. */
/* But are avaliable if needed */
/*  */
{
	CARDINAL_32	previous_Global_Reference;
	CARDINAL_32	saved_Global_Reference;
	Async_AIFF_Ptr	playback;
/* END VAR */


	playback = (Async_AIFF_Ptr)active_Snd_Channel_Ptr->userInfo;
	previous_Global_Reference = playback->saved_Global_Reference;

#if (k_USES_A5_FOR_GLOBALS == 1)

	asm {
		move.l 	a5, saved_Global_Reference(a6)		; Save original a5
		move.l	previous_Global_Reference(a6), A5
	}
	
	Mark_Playback_Flag_Completed(&playback->playing_AIFF_Now);
	
	asm {
		move.l	saved_Global_Reference(a6), A5
	}

#else
	asm {
		move.l 	a4, saved_Global_Reference(a6)		; Save original a4
		move.l	previous_Global_Reference(a6), A4
	}

	Mark_Playback_Flag_Completed(&playback->playing_AIFF_Now);
	
	asm {
		move.l	saved_Global_Reference(a6), A4
	}
#endif /* NOT #if (k_USES_A5_FOR_GLOBALS == 1) */

	
} /* pascal void Set_Play_Completion_Callback ... */
#else
/*=**************************************************************************=*/


/*=**************************************************************************=*/
pascal void Set_Play_Completion_Callback(SndChannelPtr	active_Snd_Channel_Ptr)
{
	Async_AIFF_Ptr	playback;
/* END VAR */


	playback = active_Snd_Channel_Ptr->userInfo;
	
	Mark_Playback_Flag_Completed(&playback->playing_AIFF_Now);
		
} /* FilePlayCompletionUPP Set_Play_Completion_Callback ... */
#endif /* NOT if (ORIGINAL_MAC_SYSTEM == 1) */
/*=**************************************************************************=*/


/*=**************************************************************************=*/
/* END OF FILE: Sound_Play.c */






