/* START OF FILE: Sound_Record.c */


/*=**************************************************************************=*/
/* [---BEGIN---] FILE HEADER COMMENTS */
/**
 ** SCREEN VIEW = [Adobe Syntax Black-10, Tabs = 4]
 ** PRINTER VIEW = [Adobe Syntax Black-10, Tabs = 4]
 ** =======================================================================
 ** Written by James Merkle
 ** Sound_Record.c - Cool sound bufferring for tapping SID devices to record to AIFF in nice disk chunks.
 ** (C) Copyright 1994 James Merkle
 ** =======================================================================
 **
 **
 ** ============
 ** Revision History
 ** ============
 **
 **  1995.06.04		(James Merkle)
 **						Initial coding started from scratch
 **
 **  1995.06.07		(James Merkle)
 **						Finished. Its goddamn awesome.
 **
 **
 **
 ** =================
 ** OTHER INFORMATION
 ** =================
 ** 
 ** 
 **
 **
 **/
/* [----END----] FILE HEADER COMMENTS */
/*=**************************************************************************=*/


/*=**************************************************************************=*/
/* [---BEGIN---] #include AREA */

#include <Sound.h>
#include <SoundInput.h>

#include "Volume_Utilities.h"
#include "Shell_Assembly.h"
#include "Sound_Input_Device.h"

#include "Shell_Dialogs.h"
#include "Shell_Globals.h"
#include "String_Utilities.h"
#include "Error_Utilities.h"

#include "Sound_Record.h"

/* [----END----] #include AREA */
/*=**************************************************************************=*/


/*=**************************************************************************=*/
/* [---BEGIN---] #define AREA */

/*  */
/* Allow file to dump buffers if buffering is all jammed up*/
/* because OS is too busy to call Main event loop processes */
/* often enough or mouse is held down in a menu too long, etc. */
/* If computer blows up for some reason either rewrite for device manager writes */
/* or else set to "0" and recompile. */
/* normally "0" */
/*  */
#define k_ALLOW_ASYNC_BLOCK_ALIGNED_FILE_WRITE_ON_SID_INTERRUPT		0


#define	AIFF_FILE_CREATOR					'jBox'
#define	AIFF_FILE_TYPE						'AIFF'
#define	AIFF_C_FILE_TYPE					'AIFC'

/*  */
/* Supposed to be same as a bset #5 (6th bit) */
/*  */
#define k_FORCE_NO_CACHE_BIT		0x20

/*  */
/* Must be 1, should be possitive at least. */
/*  */
#define	k_MAC_OS_ASYNC_BUSY_FLAG	1

/* [----END----] #define AREA */
/*=**************************************************************************=*/


/*=**************************************************************************=*/
/* [---BEGIN---] TYPE AREA */


/* [----END----] TYPE AREA */
/*=**************************************************************************=*/


/*=**************************************************************************=*/
/* [---BEGIN---] PROTOTYPE AREA */

Safe_Boolean Close_Sound_Output_File(INTEGER_16 *file_Reference);

Safe_Boolean Create_And_Open_Sound_Output_File( Safe_Boolean compressed_AIF_Type
															, FSSpec *direct_Disk_Recording_File_Ptr
															, INTEGER_16		*open_File_Reference);
															

Safe_Boolean Stop_Recording_To_AIFF_File(Record_Sound_To_File_Ptr record_Sound_Ptr);
															
Safe_Boolean Clear_Sound_Double_Buffers(Record_Sound_To_File_Ptr record_Sound_Ptr);

/* [----END----] PROTOTYPE AREA */
/*=**************************************************************************=*/


/*=**************************************************************************=*/
/* [---BEGIN---] GLOBAL DEFINITION AREA */

volatile	Safe_Boolean		g_Severe_Audio_Record_Problem_Needs_Display_Soon;
/* ==
		g_Severe_Audio_Record_Problem_Shown = SAFE_FALSE;
		g_Severe_Audio_Async_Record_Problem_Shown = SAFE_FALSE;
=== */

/* [----END----] GLOBAL DEFINITION AREA */
/*=**************************************************************************=*/


/*=**************************************************************************=*/
/* [---BEGIN---] LOCAL STATIC DEFINITION AREA */

static CARDINAL_8	*unit_Name;


/* [----END----] LOCAL STATIC DEFINITION AREA */
/*=**************************************************************************=*/


/*=**************************************************************************=*/
Safe_Boolean Initialize_Sound_Record(void)
{
	CARDINAL_8			*static_Unit_Name = "\pSound_Record";
/* END VAR */

/*  */
/* Global persistant structures are created here */
/*  */
	
	unit_Name = static_Unit_Name;
	
	return(SAFE_TRUE);
}
/*=**************************************************************************=*/


/*=**************************************************************************=*/
Safe_Boolean Destroy_Sound_Record(void)
{
	Safe_Boolean		function_Result;
/* END VAR */

/*  */
/* Global persistant structures are deleted here */
/*  */

	function_Result = SAFE_TRUE;
	return(function_Result);
}
/*=**************************************************************************=*/


/*=**************************************************************************=*/
Safe_Boolean Stop_Recording_To_New_AIFF_File(Record_Sound_To_File_Ptr record_Sound_Ptr)
{
	Safe_Boolean				function_Result;
	CARDINAL_8				*code_Name;
/* END VAR */


	code_Name = "\pStop_Recording_To_New_AIFF_File";
	function_Result = SAFE_FALSE;
	
	
	if (record_Sound_Ptr->initialized == SAFE_TRUE) {
	
		if (Stop_Recording_To_AIFF_File(record_Sound_Ptr) != SAFE_TRUE) {
			record_Sound_Ptr->initialized = SAFE_FALSE;
			record_Sound_Ptr->file_Reference = 0;
			record_Sound_Ptr->total_Samples_Written_To_Stream = 0;
			goto Error_Handler;
		}
	}
	
	record_Sound_Ptr->initialized = SAFE_FALSE;
	record_Sound_Ptr->file_Reference = 0;
	record_Sound_Ptr->total_Samples_Written_To_Stream = 0;
	
	function_Result = SAFE_TRUE;
	goto Normal_Function_Ending;
	
Error_Handler:
	Display_Serious_Error_Message(unit_Name, code_Name, "\pSerious problem trying to save a new AIFF output file.", "\pProgramming error?");
	function_Result = SAFE_FALSE;

Normal_Function_Ending:

	return(function_Result);
	
}/* Safe_Boolean Stop_Recording_To_New_AIFF_File */
/*=**************************************************************************=*/


/*=**************************************************************************=*/
void Advance_Buffer_Marker_If_Possible(Record_Sound_To_File_Ptr record_Sound_Ptr, INTEGER_16 index)
{

	if (record_Sound_Ptr->internal_Double_Buffer_Pending_Write[index] == SAFE_TRUE) {
	
		/*  */
		/* looks busy the only chance we have is if this one is completed with transfer. */
		/*  */
		
		if (record_Sound_Ptr->internal_Double_Async_Params[index].ioResult < k_MAC_OS_ASYNC_BUSY_FLAG) {
			/*  */
			/* It is the 4th state of the 4 states a buffer is usually in. */
			/* This buffer is FINISHED so advance marker to it. */
			/* Check if ioResult was an error though. */
			/*  */
			if (record_Sound_Ptr->internal_Double_Async_Params[index].ioResult < 0) {
				record_Sound_Ptr->data_Overflow_Corrupted_File = SAFE_TRUE;
			}
			record_Sound_Ptr->internal_Double_Buffer_Total_Bytes[index] = 0;
			record_Sound_Ptr->internal_Double_Buffer_Pending_Write[index] = SAFE_FALSE;
			record_Sound_Ptr->internal_Double_Buffer_To_Use = index;
		}
		else {
			/*  */
			/* It is the 3rd state of the 4 states a buffer is usually in. */
			/* The buffer is still pending an asyncronous write to a data fork of a file. */
			/* We cannot advance the marker now. If any real data came in right now we would have */
			/* to toss out the packet and record a data overflow using "data_Overflow_Corrupted_File" */
			/*  */
			; /* NULL */
		}
	}
	else {
		/*  */
		/* This next buffer does not seem to be writing. Check if it full or empty. */
		/*  */
		
		if (record_Sound_Ptr->internal_Double_Buffer_Total_Bytes[index] >= k_USEFUL_BUFFER_FILL_RATIO) {
			/*  */
			/* It is the 2nd state of the 4 states a buffer is usually in. */
			/* It is full and is of no more use for storage. */
			/* We cannot advance the marker now. If any real data came in right now we would have */
			/* to toss out the packet and record a data overflow using "data_Overflow_Corrupted_File" */
			/*  */
			; /* NULL */
		}
		else {
			/*  */
			/* It is the 1st state of the 4 states a buffer is usually in. */
			/* This buffer is available so advance marker to it. */
			/*  */
			
			/*  */
			/* Safety modification added. If buffer has any bytes in it */
			/* they are old and from the past and we cannot add to the end of */
			/* buffer if it contains any bytes? No we can even if wierd because */
			/* it would get ritten correctly after the bytes stored in it. */
			/* And it is the next buffer to use anyway. This is good logic especially */
			/* if there is only one buffer anyway. No harm to point to it. */
			/* also by pointing to it now if we got here because of interrupt race */
			/* paradox it is SAFE TO ASSIGN THE VOLATILE MARKER TO THIS BUFFER */
			/*  */
			record_Sound_Ptr->internal_Double_Buffer_To_Use = index;
		}
	}

} /* void Advance_Buffer_Marker_If... */
/*=**************************************************************************=*/


/*=**************************************************************************=*/
pascal Safe_Boolean Transfer_AIFF_Data_During_Interrupts(Record_Sound_To_File_Ptr record_Sound_Ptr
															, CARDINAL_32 source_Address
															, CARDINAL_32	 total_Remaining_Bytes)
/*  */
/* Call this from the async interrupt task of a SID driver in action */
/*  */
{
	CARDINAL_32	alignment_Leeway;
	CARDINAL_32	available_Bytes;
	CARDINAL_32	filled_Bytes;
	CARDINAL_32	destination_Address;
	Safe_Boolean		function_Result;
	CARDINAL_8		buffer_To_Use;
/* END VAR */


	if (record_Sound_Ptr->more_Recording_Desired == SAFE_TRUE) {
	
		/*  */
		/* 1: It is openly accepting data still - internal_Double_Buffer_To_Use points to it
		/*    or internal_Double_Buffer_Total_Bytes is less than max possible size */
		/*   and internal_Double_Buffer_Pending_Write is not set to true. */
		/*  */
		
		buffer_To_Use = record_Sound_Ptr->internal_Double_Buffer_To_Use;
		
		/*  */
		/* The buffer we are using (pointing to) may be saved completely and might */
		/* be possible to be reused right now and if it cannot be then we are */
		/* EXPECTED to just try to move to the next bufferr and assume that there */
		/* is more than one buffer. If the next buffer is unavailable we are screwed. */
		/*  */
		
		if (record_Sound_Ptr->internal_Double_Buffer_Pending_Write[buffer_To_Use] == SAFE_TRUE) {
			if (record_Sound_Ptr->internal_Double_Async_Params[buffer_To_Use].ioResult < k_MAC_OS_ASYNC_BUSY_FLAG) {
			
				/*  */
				/* It is the 4th state of the 4 states a buffer is usually in. */
				/* This buffer is FINISHED so maker is OK to leave pointing here. */
				/*  */
				if (record_Sound_Ptr->internal_Double_Async_Params[buffer_To_Use].ioResult < 0) {
					record_Sound_Ptr->data_Overflow_Corrupted_File = SAFE_TRUE;
				}
				record_Sound_Ptr->internal_Double_Buffer_Total_Bytes[buffer_To_Use] = 0;
				record_Sound_Ptr->internal_Double_Buffer_Pending_Write[buffer_To_Use] = SAFE_FALSE;
			}
		}

		if (record_Sound_Ptr->internal_Double_Buffer_Total_Bytes[buffer_To_Use] >= k_USEFUL_BUFFER_FILL_RATIO) {
			/*  */
			/* Usually this code is not encounterred. */
			/* This buffer is filled. Advance pointer to next buffer. */
			/* In this programs design if more than buffer exist this is a futile task */
			/* because the marker is already advanced. */
			/*  */
			if ((buffer_To_Use + 1) < k_TOTAL_SOUND_BUFFERS) {
				/*  */
				/* Next buffer to be utilized is after this one. Check if dormant */
				/*  */
				Advance_Buffer_Marker_If_Possible(record_Sound_Ptr, (buffer_To_Use + 1));
			}
			else {
				/*  */
				/* Next buffer to be utilized is at beginning of list. Check if first one dormant */
				/*  */
				Advance_Buffer_Marker_If_Possible(record_Sound_Ptr, 0);
			}
			
			if (buffer_To_Use == record_Sound_Ptr->internal_Double_Buffer_To_Use) {
				/*  */
				/* It failed! No way to save data! Overflow. Exit. */
				/*  */
				goto Data_Overrun_Error;
			}
			
			buffer_To_Use = record_Sound_Ptr->internal_Double_Buffer_To_Use;
		}
		
		filled_Bytes = record_Sound_Ptr->internal_Double_Buffer_Total_Bytes[buffer_To_Use];
		
		if ((record_Sound_Ptr->total_Samples_Written_To_Stream + record_Sound_Ptr->sound_File_Header_Size) 
					< k_USEFUL_BUFFER_FILL_RATIO) {
					
					
			if ((record_Sound_Ptr->total_Samples_Written_To_Stream + record_Sound_Ptr->sound_File_Header_Size + total_Remaining_Bytes)
						<= k_USEFUL_BUFFER_FILL_RATIO) {
				/*  */
				/* Nothing to worry about yet, buffer not full enough yet for alighnment. */
				/*  */
				available_Bytes = k_MINIMUM_BUFFER_RESOLUTION - filled_Bytes;
			}
			else {
				/*  */
				/* This section of code is only reached one time per file creation. */
				/* The first buffer has never been written out to disk yet */
				/* This means that the available amount should be decreased */
				/* by the amount of the (header modulo 512) to help align the */
				/* second and future buffer writes on 512 byte block boundaries. */
				/* This furthers chance that async writes at interrupt will work */
				/* and that streaming occurs faster. */
				/*  */
				alignment_Leeway = (k_MINIMUM_BUFFER_RESOLUTION - k_USEFUL_BUFFER_FILL_RATIO);
				
				/*  */
				/* Paranoia only. */
				/*  */
				alignment_Leeway = alignment_Leeway & 0xFFFFFE00;
			
				/*  */
				/* Alignment leeway is usually 512 or more bytes. We will align */
				/* this file to 512, 2K, 8K, or whatever is freindly to the disk array. */
				/* The values are constants in this program though and is usually just */
				/* the minimum of 512 (0x0200). */
				/*  */
				
				if (record_Sound_Ptr->sound_File_Header_Size <= alignment_Leeway) {
					available_Bytes = ((k_USEFUL_BUFFER_FILL_RATIO 
										+ (alignment_Leeway - record_Sound_Ptr->sound_File_Header_Size)) 
										- filled_Bytes);
				}
				else {
					/*  */
					/* Permits giant header to be aligned as well */
					/*  */
					available_Bytes = ((k_USEFUL_BUFFER_FILL_RATIO 
										+ (alignment_Leeway - (record_Sound_Ptr->sound_File_Header_Size % alignment_Leeway))) 
										- filled_Bytes);
				}
			}
		}
		else {
			available_Bytes = k_MINIMUM_BUFFER_RESOLUTION - filled_Bytes;
		}
		
		/*  */
		/* We now have the amount of available bytes in this buffer we can use. */
		/* It is guranteed to be greater than 0 by design. */
		/* We may need to write in one buffer and advance and write in another buffer. */
		/*  */
		destination_Address = (CARDINAL_32)&record_Sound_Ptr->internal_Double_Buffer_Storage[buffer_To_Use];
		destination_Address += filled_Bytes;
		
		if (total_Remaining_Bytes <= available_Bytes) {
			Block_Move_No_Flush((Ptr)source_Address, (Ptr)destination_Address, total_Remaining_Bytes);
			record_Sound_Ptr->internal_Double_Buffer_Total_Bytes[buffer_To_Use] += total_Remaining_Bytes;
		}
		else {
		
			/*  */
			/* Loop through as many buffers as needed to consume */
			/*  */
			do {
				Block_Move_No_Flush((Ptr)source_Address, (Ptr)destination_Address, available_Bytes);
				record_Sound_Ptr->internal_Double_Buffer_Total_Bytes[buffer_To_Use] += available_Bytes;
				total_Remaining_Bytes = total_Remaining_Bytes - available_Bytes;
				
				if (total_Remaining_Bytes > 0) {
					/*  */
					/* Now advance buffer if possible and write remainder */
					/*  */
					if ((buffer_To_Use + 1) < k_TOTAL_SOUND_BUFFERS) {
						/*  */
						/* Next buffer to be utilized is after this one. Check if dormant */
						/*  */
						Advance_Buffer_Marker_If_Possible(record_Sound_Ptr, (buffer_To_Use + 1));
					}
					else {
						/*  */
						/* Next buffer to be utilized is at beginning of list. Check if first one dormant */
						/*  */
						Advance_Buffer_Marker_If_Possible(record_Sound_Ptr, 0);
					}
					
					if (buffer_To_Use == record_Sound_Ptr->internal_Double_Buffer_To_Use) {
						/*  */
						/* It failed! No way to save data! Overflow? Exit? */
						/* Or if we are smart, we would force the buffers to flush right now and use */
						/* whatever buffer we find available after the flush completes! */
						/*  */
		
#if !defined(k_ALLOW_ASYNC_BLOCK_ALIGNED_FILE_WRITE_ON_SID_INTERRUPT)
#error "UNDEFINED PARAMETER!"
#endif
#if  (k_ALLOW_ASYNC_BLOCK_ALIGNED_FILE_WRITE_ON_SID_INTERRUPT == 1)
						if (Store_Possible_AIFF_Data_During_Interrupts(record_Sound_Ptr, 1) != SAFE_TRUE) {
							goto Data_Overrun_Error;
						}
#else
						goto Data_Overrun_Error;
#endif				
					}
					
					buffer_To_Use = record_Sound_Ptr->internal_Double_Buffer_To_Use;
					
					/*  */
					/* Now write another portion of remaining data */
					/* and hope that we did not create much of an audio loss */
					/* with this stupid dump of ours, because the SID is frozen */
					/* and we were required to try async I/O that might have gone */
					/* syncronously. */
					/*  */
					source_Address += available_Bytes;
					
					filled_Bytes = record_Sound_Ptr->internal_Double_Buffer_Total_Bytes[buffer_To_Use];
					destination_Address = (CARDINAL_32)&record_Sound_Ptr->internal_Double_Buffer_Storage[buffer_To_Use];
					destination_Address += filled_Bytes;
		
					available_Bytes = k_MINIMUM_BUFFER_RESOLUTION - filled_Bytes;
					if (available_Bytes > total_Remaining_Bytes) {
						available_Bytes = total_Remaining_Bytes;
					}
					
				} /* if (total_Remaining_Bytes > 0) */
				
			} while (total_Remaining_Bytes > 0);

		} /* NOIT if (total_Remaining_Bytes <= available_Bytes) */
			
	} /* if (record_Sound_Ptr->more_Recording_Desired == SAFE_TRUE)  */
	
	function_Result = SAFE_TRUE;
	goto Normal_Function_Ending;
	
Data_Overrun_Error:
	function_Result = SAFE_FALSE;
	record_Sound_Ptr->data_Overflow_Corrupted_File = SAFE_TRUE;

Error_Handler:
	function_Result = SAFE_FALSE;

Normal_Function_Ending:

	return(function_Result);

} /* Safe_Boolean Transfer_AIFF_Data_During_Interrupts... */
/*=**************************************************************************=*/


/*=**************************************************************************=*/
Safe_Boolean Store_Possible_AIFF_Data_During_Interrupts(Record_Sound_To_File_Ptr record_Sound_Ptr
																, CARDINAL_32	desired_Buffer_Fill_Limit)
/*  */
/* This is preferably called from non interrupt level but is */
/* called from interrupt always for testing and for situation where */
/* a buffer just filled and the last buffer is in use. */
/* In current usage I use only two buffers and last buffer in use is highly common. */
/* If many buffers are waiting to save data, then this code will write all the ready ones. */
/* it is only OK for desired_Buffer_Fill_Limit to be less than or equal to */
/* desired_Buffer_Fill_Limit, and is always equal to "k_USEFUL_BUFFER_FILL_RATIO" */
/* except when closing down the file capture and last buffer scraps are desired. */
/*  */
{
	IOParam			*parameter_Ptr;
	CARDINAL_32	 total_To_Transfer;
	SIMPLE_LOOP	loop;
	SIMPLE_LOOP	original_Boundary_Marker;
	Safe_Boolean		function_Result;
	OSErr				os_Error;
/* END VAR */
		
											
	if (record_Sound_Ptr->more_Recording_Desired == SAFE_TRUE) {

		/*  */
		/* An audio async ioBuffer can be in 4 common states */
		/* 1: It is openly accepting data still - internal_Double_Buffer_To_Use points to it
		/*    or internal_Double_Buffer_Total_Bytes is less than max possible size */
		/*   and internal_Double_Buffer_Pending_Write is not set to true. */
		/* 2: It is full waiting for dump to disk internal_Double_Buffer_Total_Bytes is equal to */
		/*    max size but internal_Double_Buffer_Pending_Write is not set to true */
		/* 3: It is in process of writing asyncronously : internal_Double_Buffer_Pending_Write is */
		/* set and ioResult is k_MAC_OS_ASYNC_BUSY_FLAG */
		/* At this point size is set back to "0" once transfer is queued */
		/* 4 : It is finished writing but needs fags flipped to ready to fill state. */
		/*    in this case internal_Double_Buffer_Pending_Write is still set true even though */
		/* ioResult is no longer set to k_MAC_OS_ASYNC_BUSY_FLAG */
		/*  */
		
		/*  */
		/* Always start looping through all buffers starting from current marker and going */
		/* around circular queue in a forward manner. Logic may even work with */
		/* one buffer but more than one is the only condition tested. */
		/* All previously filled buffers must be totally filled and not partially filled. */
		/*  */
				
		/*  */
		/* Save marker because it may change while we are saving buffers */
		/* if other lower level interrupt filling routines stream some more data. */
		/*  */
		original_Boundary_Marker = record_Sound_Ptr->internal_Double_Buffer_To_Use;
		if ((original_Boundary_Marker + 1) < k_TOTAL_SOUND_BUFFERS) {
			/*  */
			/* Oldest buffer recorded is after this one. Check if dormant */
			/*  */
			original_Boundary_Marker += 1;
		}
		else {
			/*  */
			/* Oldest buffer recorded is at beginning of list */
			/*  */
			original_Boundary_Marker = original_Boundary_Marker - 1;
		}

		
		for (loop = original_Boundary_Marker; (loop >= 0); loop = loop - 1) {
		
			if (record_Sound_Ptr->internal_Double_Buffer_Total_Bytes[loop] >= desired_Buffer_Fill_Limit) {
				/*  */
				/* Safety double check */
				/*  */
				if (record_Sound_Ptr->internal_Double_Buffer_Pending_Write[loop] != SAFE_TRUE) {
				
					/*  */
					/* Write this buffer and keep loking for more but if possible */
					/* advance marker to next position if it is the start point else leave it here */
					/* The need for this code here is not too common unless buffers fill */
					/* up in such a way that they cleanly fill the double buffers perfectly and */
					/* beyond that, that the marker couild not be previously advanced because */
					/* the oldest buffer was still pending a write and was not finished. */
					/*  */
					
					if (loop == record_Sound_Ptr->internal_Double_Buffer_To_Use) {
						if ((loop + 1) < k_TOTAL_SOUND_BUFFERS) {
							/*  */
							/* Next buffer to be utilized is after this one. Check if dormant */
							/*  */
							Advance_Buffer_Marker_If_Possible(record_Sound_Ptr, (loop + 1));
						}
						else {
							/*  */
							/* Next buffer to be utilized is at beginning of list. Check if first one dormant */
							/*  */
							Advance_Buffer_Marker_If_Possible(record_Sound_Ptr, 0);
						}
						
					} /* if (loop == original_Boundary_Marker) */
					
					/*  */
					/* Now that the active buffer was incremented if possible, write the buffer out. */
					/* During this write, even if not really asyncronously instantaneous, */
					/* interrupt fed data will write to a new buffer without having to already */
					/* worry about this condition (unless all buffers are filled.) */
					/*  */
					total_To_Transfer = record_Sound_Ptr->internal_Double_Buffer_Total_Bytes[loop];
					
					parameter_Ptr = &record_Sound_Ptr->internal_Double_Async_Params[loop];
					parameter_Ptr->ioResult = k_MAC_OS_ASYNC_BUSY_FLAG;
					parameter_Ptr->ioBuffer = (Ptr)&record_Sound_Ptr->internal_Double_Buffer_Storage[loop][0];
					parameter_Ptr->ioReqCount = total_To_Transfer;
					parameter_Ptr->ioPosOffset = (record_Sound_Ptr->sound_File_Header_Size 
															+ record_Sound_Ptr->total_Samples_Written_To_Stream);
															
					/*  */
					/* Advance marker now in case this I/O stalls and another happens behind it. */
					/* and if it does it needs to be POSITIONED correctly. To */
					/* position correctly it needs to have this advanced right now */
					/*  */
					record_Sound_Ptr->total_Samples_Written_To_Stream += total_To_Transfer;
					record_Sound_Ptr->internal_Double_Buffer_Pending_Write[loop] = SAFE_TRUE;

					/*  */
					/* Request async write request (we might bee in SID hardware interrupt) */
					/* If we mapped the file out we might have been write more safely. */
					/*  */
					os_Error = PBWrite((ParmBlkPtr)parameter_Ptr, TRUE);
					
					/*  */
					/* we do not need to check if data syncronously wrote and if so what the */
					/* result was. That stuff can be checked later on in normal buffer reusage. */
					/* The only error we need to check is the queuing error created for an */
					/* impossible async call. If so let this one go but corrupt the ioResult to prevent */
					/* our own logic from racing forever and locking oddly. */
					/*  */
					if (os_Error != noErr) {
						parameter_Ptr->ioResult = -1;
						
						/*  */
						/* Free up buffer completely now instead of later. */
						/* This allows file count and position to avoid any audio */
						/* noise in the file (but this sound will be missing instead) */
						/*  */
						record_Sound_Ptr->data_Overflow_Corrupted_File = SAFE_TRUE;
						record_Sound_Ptr->internal_Double_Buffer_Total_Bytes[loop] = 0;
						record_Sound_Ptr->internal_Double_Buffer_Pending_Write[loop] = SAFE_FALSE;
						
						/*  */
						/* It should be safe to back this off now because real Async */
						/* failure is pretty quick. if not safe then audio will just loose a good */
						/* buffer but keep a bad buffer. This is a infinitely small possibility. */
						/*  */
						record_Sound_Ptr->total_Samples_Written_To_Stream =
															record_Sound_Ptr->total_Samples_Written_To_Stream - total_To_Transfer;
															
					}
					
				} /* if (parameter_Ptr->internal_Double_Buffer_Pending_Write != SAFE_TRUE) */
				
			} /* if (parameter_Ptr->total_Samples_Written_To_Stream >= desired_Buffer_Fill_Limit) */
		
		} /* for (loop = original_Boundary_Marker; (loop >= 0); loop = loop - 1) */
		
		/*  */
		/* Now scan older half of circular buffer queue*/
		/*  */
		
		for (loop = (k_TOTAL_SOUND_BUFFERS - 1); (loop > original_Boundary_Marker); loop = loop - 1) {
		
			/*  */
			/* For incredibly detailed comments just look at the previous "for" */
			/* loop because it is identical to this but commented. */
			/*  */
			
			if (record_Sound_Ptr->internal_Double_Buffer_Total_Bytes[loop] >= desired_Buffer_Fill_Limit) {
			

				if (record_Sound_Ptr->internal_Double_Buffer_Pending_Write[loop] != SAFE_TRUE) {
									
					if (loop == record_Sound_Ptr->internal_Double_Buffer_To_Use) {
						if ((loop + 1) < k_TOTAL_SOUND_BUFFERS) {
							Advance_Buffer_Marker_If_Possible(record_Sound_Ptr, (loop + 1));
						}
						else {
							Advance_Buffer_Marker_If_Possible(record_Sound_Ptr, 0);
						}
						
					} /* if (loop == original_Boundary_Marker) */
					
					total_To_Transfer = record_Sound_Ptr->internal_Double_Buffer_Total_Bytes[loop];
					
					record_Sound_Ptr->internal_Double_Buffer_Pending_Write[loop] = SAFE_TRUE;
					
					parameter_Ptr = &record_Sound_Ptr->internal_Double_Async_Params[loop];
					parameter_Ptr->ioResult = k_MAC_OS_ASYNC_BUSY_FLAG;
					parameter_Ptr->ioBuffer = (Ptr)&record_Sound_Ptr->internal_Double_Buffer_Storage[loop][0];
					parameter_Ptr->ioReqCount = total_To_Transfer;
					parameter_Ptr->ioPosOffset = (record_Sound_Ptr->sound_File_Header_Size 
															+ record_Sound_Ptr->total_Samples_Written_To_Stream);
															
					record_Sound_Ptr->total_Samples_Written_To_Stream += total_To_Transfer;
					record_Sound_Ptr->internal_Double_Buffer_Pending_Write[loop] = SAFE_TRUE;
					os_Error = PBWrite((ParmBlkPtr)parameter_Ptr, TRUE);

					if (os_Error != noErr) {
						parameter_Ptr->ioResult = -1;
						
						record_Sound_Ptr->data_Overflow_Corrupted_File = SAFE_TRUE;
						record_Sound_Ptr->internal_Double_Buffer_Total_Bytes[loop] = 0;
						record_Sound_Ptr->internal_Double_Buffer_Pending_Write[loop] = SAFE_FALSE;
						record_Sound_Ptr->total_Samples_Written_To_Stream =
															record_Sound_Ptr->total_Samples_Written_To_Stream - total_To_Transfer;
					}
					
				} /* if (parameter_Ptr->internal_Double_Buffer_Pending_Write != SAFE_TRUE) */
				
			} /* if (parameter_Ptr->internal_Double_Buffer_Total_Bytes[loop] >= desired_Buffer_Fill_Limit) */
		
		} /* for (loop = k_TOTAL_SOUND_BUFFERS; (loop > original_Boundary_Marker); loop = loop - 1) */

	} /* if (record_Sound_Ptr->more_Recording_Desired == SAFE_TRUE)  */
	

	function_Result = SAFE_TRUE;
	
	return(function_Result);
	
} /* Safe_Boolean Store_Possible_AIFF_Data_During_Interrupts */
/*=**************************************************************************=*/


/*=**************************************************************************=*/
Safe_Boolean Periodically_Save_To_New_AIFF_File_If_Needed(Record_Sound_To_File_Ptr record_Sound_Ptr)
/*  */
/* This is called from main process periodic task loop */
/* It tries to outrace writing files at safer I/O time */
/* and tries to close and save files if a flag requests such an action */
/*  */
{
	Safe_Boolean				function_Result;
	CARDINAL_8				*code_Name;
/* END VAR */


	if (record_Sound_Ptr->initialized == SAFE_TRUE) {
	
		code_Name = "\pPeriodically_Save_To_New_AIFF_File_If_Needed";
		
		/*  */
		/* Using medium sized double buffers keep adding output to */
		/* open audio files that are in process of being saved. */
		/*  */
		
		if (record_Sound_Ptr->more_Recording_Desired == SAFE_TRUE) {
		
			if (Store_Possible_AIFF_Data_During_Interrupts(record_Sound_Ptr, k_USEFUL_BUFFER_FILL_RATIO) != SAFE_TRUE) {
				/*  */
				/* Close it down because there was a problem */
				/*  */
				record_Sound_Ptr->stop_Recording_As_Soon_As_Possible = SAFE_TRUE;
			}

		} /* if (record_Sound_Ptr->more_Recording_Desired == SAFE_TRUE) */
		
		if (record_Sound_Ptr->total_Samples_Written_To_Stream > record_Sound_Ptr->maximum_Samples_To_Record) {
			record_Sound_Ptr->stop_Recording_As_Soon_As_Possible = SAFE_TRUE;
		}

		if (record_Sound_Ptr->stop_Recording_As_Soon_As_Possible == SAFE_TRUE) {
		
			/*  */
			/* Closing down. Accept every single byte in buffer as useful to store */
			/*  */
			
			if (Store_Possible_AIFF_Data_During_Interrupts(record_Sound_Ptr, 1) != SAFE_TRUE) {
				/*  */
				/* Ignore closure problems if any. No harm */
				/*  */
			}
		
			if (Stop_Recording_To_AIFF_File(record_Sound_Ptr) != SAFE_TRUE) {
				record_Sound_Ptr->initialized = SAFE_FALSE;
				record_Sound_Ptr->file_Reference = 0;
				record_Sound_Ptr->more_Recording_Desired = SAFE_FALSE;
				record_Sound_Ptr->stop_Recording_As_Soon_As_Possible = SAFE_FALSE;
				goto Error_Handler;
			}
		}
		
	} /* if (record_Sound_Ptr->initialized == SAFE_TRUE) */
	
	function_Result = SAFE_TRUE;
	goto Normal_Function_Ending;
	
Error_Handler:
	Display_Serious_Error_Message(unit_Name, code_Name, "\pSerious problem trying to save a new AIFF output file.", "\pProgramming error?");
	function_Result = SAFE_FALSE;

Normal_Function_Ending:

	return(function_Result);
	
}/* Safe_Boolean Stop_Recording_To_New_AIFF_File */
/*=**************************************************************************=*/


/*=**************************************************************************=*/
Safe_Boolean Start_Recording_To_New_AIFF_File(Record_Sound_To_File_Ptr record_Sound_Ptr
														, CARDINAL_32 maximum_Possible_File_Size
														, FSSpec	*desired_File_Spec
														, CARDINAL_32 number_Of_Channels
														, CARDINAL_32 sample_Rate
														, CARDINAL_32 sample_Size
														, CARDINAL_32 compression_Type)
/*  */
/* A valid FSSpec-ish file path must be passed to this routine */
/*  */
{
	FSSpec *	direct_Disk_Recording_File_Ptr;
	OSErr						os_Error;
	Safe_Boolean				user_Cancelled;
	CARDINAL_8				*code_Name;
	Safe_Boolean				function_Result;
/* END VAR */


	code_Name = "\pStart_Recording_To_New_AIFF_File";
												
	function_Result = SAFE_FALSE;
	
	record_Sound_Ptr->initialized = SAFE_FALSE;
	record_Sound_Ptr->more_Recording_Desired = SAFE_FALSE;
	record_Sound_Ptr->stop_Recording_As_Soon_As_Possible = SAFE_FALSE;
	record_Sound_Ptr->file_Reference = 0;
	record_Sound_Ptr->data_Overflow_Corrupted_File = SAFE_FALSE;

	record_Sound_Ptr->total_Samples_Written_To_Stream = 0;
	
	record_Sound_Ptr->maximum_Possible_File_Size = maximum_Possible_File_Size;
	record_Sound_Ptr->desired_File_Spec = *desired_File_Spec;
	
	if (sample_Size < 8) {
		goto Error_Handler;
	}
	
	if (Create_And_Open_Sound_Output_File(SAFE_FALSE
													, &record_Sound_Ptr->desired_File_Spec
													, &record_Sound_Ptr->file_Reference) != SAFE_TRUE) {
		goto Error_Handler;
	}
	else {

		record_Sound_Ptr->number_Of_Channels = number_Of_Channels;
		record_Sound_Ptr->sample_Rate = sample_Rate;
		record_Sound_Ptr->sample_Size = sample_Size;
		record_Sound_Ptr->compression_Type = compression_Type;

		/*  */
		/* Record arbitrary empty header with No bytes declared yet. */
		/*  */
		os_Error = SetFPos(record_Sound_Ptr->file_Reference, fsFromStart, 0);
		if (os_Error != noErr) {
			goto Error_Handler;
		}

		os_Error = SetupAIFFHeader(record_Sound_Ptr->file_Reference
								, record_Sound_Ptr->number_Of_Channels
								, record_Sound_Ptr->sample_Rate
								, record_Sound_Ptr->sample_Size
								, record_Sound_Ptr->compression_Type
								, 0
								, 0 );
								
		if (os_Error != noErr) {
			goto Error_Handler;
		}			

		/*  */
		/* Reserve room for the sound files header . 'SetupAIFFHeader'  */
		/* already moved the file pointer */
		/*  */
		os_Error = GetFPos(record_Sound_Ptr->file_Reference, (INTEGER_32 *)&record_Sound_Ptr->sound_File_Header_Size);
		if ((os_Error != noErr ) || (record_Sound_Ptr->sound_File_Header_Size == 0)) {
			goto Error_Handler;
		}
				
		/*  */
		/* This sets up the double buffers for recording to disk.*/
		/*  */
		if (Clear_Sound_Double_Buffers(record_Sound_Ptr) != SAFE_TRUE) {
			goto Error_Handler;
		}
		
		record_Sound_Ptr->maximum_Samples_To_Record = ((1.0 * record_Sound_Ptr->maximum_Possible_File_Size) / (1.0 * (record_Sound_Ptr->sample_Size / 8.0)));
		record_Sound_Ptr->maximum_Samples_To_Record = record_Sound_Ptr->maximum_Samples_To_Record / record_Sound_Ptr->number_Of_Channels;
		
		/*  */
		/* Create Even number of samples */
		/*  */
		record_Sound_Ptr->maximum_Samples_To_Record = record_Sound_Ptr->maximum_Samples_To_Record & 0xFFFFFFFE;

		record_Sound_Ptr->initialized = SAFE_TRUE;
		record_Sound_Ptr->more_Recording_Desired = SAFE_TRUE;

	} /* NOT if (Create_And_Open_Sound_Output_File(... */


	function_Result = SAFE_TRUE;
	goto Normal_Function_Ending;
	
Error_Handler:
	Display_Serious_Error_Message(unit_Name, code_Name, "\pSerious problem trying to open a new AIFF output file.", "\pNot enough space?");
	function_Result = SAFE_FALSE;

Normal_Function_Ending:

	return(function_Result);
	
} /* Safe_Boolean Start_Recording_To_New_AIFF_File... */
/*=**************************************************************************=*/


/*=**************************************************************************=*/
Safe_Boolean Clear_Sound_Double_Buffers(Record_Sound_To_File_Ptr record_Sound_Ptr)
{
	SIMPLE_LOOP			loop;
	Safe_Boolean				function_Result;
/* END VAR */


	record_Sound_Ptr->stop_Recording_As_Soon_As_Possible = SAFE_FALSE;

	record_Sound_Ptr->internal_Double_Buffer_To_Use = 0;
	
	for (loop = 0; (loop < k_TOTAL_SOUND_BUFFERS); loop++) {
		record_Sound_Ptr->internal_Double_Buffer_Total_Bytes[loop] = 0;
		record_Sound_Ptr->internal_Double_Buffer_Pending_Write[loop] = SAFE_FALSE;
		
		Fill_Byte_Memory_With_Byte((Ptr)&record_Sound_Ptr->internal_Double_Async_Params[loop]
							, sizeof(record_Sound_Ptr->internal_Double_Async_Params[loop])
							, 0);
							
		/*  */
		/* For efficiency, pre-initialize part of structure. The only */
		/* fields changed when actively asyncronously recording will */
		/* be ioResult, ioBuffer, ioReqCount, and ioPosOffset. */
		/*  */
		record_Sound_Ptr->internal_Double_Async_Params[loop].ioRefNum = record_Sound_Ptr->file_Reference;
		record_Sound_Ptr->internal_Double_Async_Params[loop].ioPosMode = fsFromStart;
		
		/*  */
		/* Force it through to driver for speed and evenness of system jitter. Or flag */
		/*  */
		record_Sound_Ptr->internal_Double_Async_Params[loop].ioPosMode |= k_FORCE_NO_CACHE_BIT;
		
		record_Sound_Ptr->internal_Double_Async_Params[loop].ioCompletion = nil;
	}

	function_Result = SAFE_TRUE;

	return(function_Result);
	
} /* Safe_Boolean Start_Recording_To_New_AIFF_File... */
/*=**************************************************************************=*/


/*=**************************************************************************=*/
Safe_Boolean Stop_Recording_To_AIFF_File(Record_Sound_To_File_Ptr record_Sound_Ptr)
{
	OSErr						os_Error;
	CARDINAL_8				*code_Name;
	Safe_Boolean				function_Result;
/* END VAR */


	code_Name = "\pStop_Recording_To_AIFF_File";

	if (record_Sound_Ptr->initialized == SAFE_TRUE) {
	
		os_Error = SetFPos(record_Sound_Ptr->file_Reference, fsFromStart, 0);
		if (os_Error != noErr) {
			goto Error_Handler;
		}
	
		os_Error = SetupAIFFHeader(record_Sound_Ptr->file_Reference
										, record_Sound_Ptr->number_Of_Channels
										, record_Sound_Ptr->sample_Rate
										, record_Sound_Ptr->sample_Size
										, record_Sound_Ptr->compression_Type
										, ((record_Sound_Ptr->total_Samples_Written_To_Stream
											* record_Sound_Ptr->number_Of_Channels
											* record_Sound_Ptr->sample_Size) / (8L))
										, 0 );
		if (os_Error != noErr) {
			goto Error_Handler;
		}
		record_Sound_Ptr->file_Reference = 0;

	}
	else {
		goto Error_Handler;
	}
	
	record_Sound_Ptr->initialized = SAFE_FALSE;
	record_Sound_Ptr->file_Reference = 0;
	record_Sound_Ptr->more_Recording_Desired = SAFE_FALSE;
	record_Sound_Ptr->stop_Recording_As_Soon_As_Possible = SAFE_FALSE;

	function_Result = SAFE_TRUE;
	goto Normal_Function_Ending;
	
Error_Handler:
	Display_Serious_Error_Message(unit_Name
										, code_Name
										, "\pSerious problem trying to close an open sound file"
										, "\pFile not previously open?");
	function_Result = SAFE_FALSE;

Normal_Function_Ending:

	return(function_Result);
	
} /* Safe_Boolean Start_Recording_To_New_AIFF_File... */
/*=**************************************************************************=*/


/*=**************************************************************************=*/
Safe_Boolean Get_Next_AIFF_File_Specification( CARDINAL_32 expected_File_Minimum_Size
															, CARDINAL_32 *maximum_Possible_File_Size
															, FSSpec *direct_Disk_Recording_File_Ptr
															, Safe_Boolean	*user_Cancelled)
	/*  */
	/* This routine is intended to be used for opening an AIFF-C file */
	/* when used with direct-to-disk recording */
	/*  */
{
	INTEGER_16				scrap_Volume_Reference_Number;
	INTEGER_16				os_Error;
	Safe_Boolean				function_Result;
	Safe_Boolean				still_Searching_For_Location;
	CARDINAL_32			volume_Space;
	CARDINAL_32			file_Type;
	SFReply					sf_Reply;
	Str255						volume_Name;
	Str255						work_String_2;
	Str255						note_String;
	CARDINAL_8				*code_Name;
/* END VAR */


	code_Name = "\pGet_Next_AIFF_File_Specification";
	*user_Cancelled = SAFE_FALSE;
	*maximum_Possible_File_Size =0;

	function_Result = SAFE_FALSE;
	direct_Disk_Recording_File_Ptr->vRefNum = 0;
	direct_Disk_Recording_File_Ptr->parID = 0;
	direct_Disk_Recording_File_Ptr->name[0] = 0;

	Copy_Pascal_String("\pSound File.AIF" ,note_String);
	
	still_Searching_For_Location = SAFE_TRUE;
	do {
	
		if( Perform_Put_File_Dialog((StringPtr)"\pName the AIFF sound file:", note_String, &sf_Reply) != TRUE) {
		
			function_Result = SAFE_FALSE;
			*user_Cancelled = SAFE_TRUE;
			goto Normal_Function_Ending;
		}
			
		Copy_Pascal_String(sf_Reply.fName, direct_Disk_Recording_File_Ptr->name);
		Get_Last_Volume_Reference_And_Directory_ID(&(direct_Disk_Recording_File_Ptr->vRefNum), &(direct_Disk_Recording_File_Ptr->parID));
	
	
		note_String[0] = 0;
		
		os_Error = GetVInfo((Get_Drive_Number(direct_Disk_Recording_File_Ptr->vRefNum)), volume_Name, &scrap_Volume_Reference_Number, (INTEGER_32 *)&volume_Space);
		if (os_Error != noErr) {
				Concat_Many(note_String, 3, "\pGetVInfo for volume ", volume_Name, "\p ID = ");
				Concat_Long_To_String(note_String, os_Error, NO_LIMIT);
			goto Error_Handler;
		} /* if (os_Error != noErr) */
		
	
		if (((CARDINAL_32)expected_File_Minimum_Size + (CARDINAL_32)MINIMUM_DISK_SPACE_TO_PRESERVE) > volume_Space) {
	
			Concat_Many(note_String, 3, "\p", volume_Name, "\p is too full for the file. (");
			Make_String_Long_String(work_String_2, (CARDINAL_8 *)note_String, volume_Space, NO_LIMIT,"\p)bytes free, (" );
			Make_String_Long_String(note_String
										, (CARDINAL_8 *)work_String_2
										, (expected_File_Minimum_Size + MINIMUM_DISK_SPACE_TO_PRESERVE)
										, NO_LIMIT
										,"\p)bytes needed or more.  Find additional space for file." );
	
			still_Searching_For_Location = SAFE_TRUE;
		}
		else {
			still_Searching_For_Location = SAFE_FALSE;
		}
	
	} while (still_Searching_For_Location == SAFE_TRUE);

	*maximum_Possible_File_Size = ((CARDINAL_32)volume_Space - (CARDINAL_32)(MINIMUM_DISK_SPACE_TO_PRESERVE));

	function_Result = SAFE_TRUE;
	goto Normal_Function_Ending;
	
Error_Handler:
	Display_Serious_Error_Message(unit_Name, code_Name, note_String, "\p");
	function_Result = SAFE_FALSE;

Normal_Function_Ending:

	return(function_Result);
	
} /* Safe_Boolean Get_Next_AIFF_File_Specification */
/*=**************************************************************************=*/


/*=**************************************************************************=*/
Safe_Boolean Create_And_Open_Sound_Output_File( Safe_Boolean compressed_AIF_Type
															, FSSpec *direct_Disk_Recording_File_Ptr
															, INTEGER_16		*open_File_Reference)
	/*  */
	/* This routine is intended to be used for opening an AIFF-C file */
	/* when used with direct-to-disk recording */
	/* You must pass a useful FSSpec that is ready to create. */
	/*  */
{
	INTEGER_16				os_Error;
	Safe_Boolean				function_Result;
	CARDINAL_32			volume_Space;
	CARDINAL_32			file_Type;
	SFReply					sf_Reply;
	Str255						volume_Name;
	Str255						work_String_2;
	Str255						note_String;
	CARDINAL_8				*code_Name;
/* END VAR */


	code_Name = "\pCreate_And_Open_Sound_Output_File";
	function_Result = SAFE_FALSE;
	
	/*  */
	/* Paranoia init */
	/*  */
	*open_File_Reference = 0;
		
	if (compressed_AIF_Type == SAFE_TRUE) {
		file_Type = AIFF_C_FILE_TYPE;
	}
	else {
		file_Type = AIFF_FILE_TYPE;
	}	
		
	os_Error = HCreate(direct_Disk_Recording_File_Ptr->vRefNum
						, direct_Disk_Recording_File_Ptr->parID
						, direct_Disk_Recording_File_Ptr->name
						, AIFF_FILE_CREATOR
						, file_Type);
						
	if (os_Error == dupFNErr) {
		os_Error = HDelete(direct_Disk_Recording_File_Ptr->vRefNum
								, direct_Disk_Recording_File_Ptr->parID
								, direct_Disk_Recording_File_Ptr->name);
		if (os_Error != noErr) {
			Concat_Many(note_String, 3,"\pFSDelete for file ", direct_Disk_Recording_File_Ptr->name, "\p ID = ");
			Concat_Long_To_String(note_String, os_Error, NO_LIMIT);
			goto Error_Handler;
		}

		os_Error = HCreate(direct_Disk_Recording_File_Ptr->vRefNum
							, direct_Disk_Recording_File_Ptr->parID
							, direct_Disk_Recording_File_Ptr->name
							, AIFF_FILE_CREATOR
							, file_Type);
	} /* if (os_Error == dupFNErr) */

	if (os_Error != noErr) {
		Concat_Many(note_String, 3,"\pCreate for file ", direct_Disk_Recording_File_Ptr->name, "\p ID = ");
		Concat_Long_To_String(note_String, os_Error, NO_LIMIT);
		goto Error_Handler;
	} 

	os_Error = HOpen( direct_Disk_Recording_File_Ptr->vRefNum
							, direct_Disk_Recording_File_Ptr->parID
							, direct_Disk_Recording_File_Ptr->name
							, fsRdWrPerm
							, open_File_Reference);
	if (os_Error != noErr) {
		Concat_Many(note_String, 3,"\pFSOpen for file ", direct_Disk_Recording_File_Ptr->name, "\p ID = ");
		Concat_Long_To_String(note_String, os_Error, NO_LIMIT);
		goto Error_Handler;
	} 

		
	function_Result = SAFE_TRUE;
	goto Normal_Function_Ending;
	
Error_Handler:
	Display_Serious_Error_Message(unit_Name, code_Name, note_String, "\p");
	function_Result = SAFE_FALSE;

Normal_Function_Ending:

	return(function_Result);

} /* Safe_Boolean Create_And_Open_Sound_Output_File(FSSpec * direct_Disk_Recording_File_Ptr) */
/*=**************************************************************************=*/


/*=**************************************************************************=*/
Safe_Boolean Close_Sound_Output_File(INTEGER_16 *file_Reference)
{
	INTEGER_16				os_Error;
	Safe_Boolean				function_Result;
	CARDINAL_8				*code_Name;
/* END VAR */


	code_Name = "\pClose_Sound_Output_File";
	function_Result = SAFE_FALSE;
	
	os_Error = FSClose(*file_Reference);
	if (os_Error != noErr) {
		Display_Serious_Error_Message(unit_Name, code_Name, "\pFSClose for sound output file failed!", "\p");
		goto Error_Handler;
	} 
		
	*file_Reference = 0;
	
	function_Result = SAFE_TRUE;
	goto Normal_Function_Ending;
	
Error_Handler:
	function_Result = SAFE_FALSE;

Normal_Function_Ending:
	
	return(function_Result);

} /* Safe_Boolean Close_Sound_Output_File(FSSpec * direct_Disk_Recording_File_Ptr) */
/*=**************************************************************************=*/


/*=**************************************************************************=*/
/* END OF FILE: Sound_Record.c */
