/* START OF FILE: Dialog_Of_Background_Removal.c */


/*=**************************************************************************=*/
/* [---BEGIN---] FILE HEADER COMMENTS */
/**
 ** SCREEN VIEW = [Adobe Syntax Black-10, Tabs = 4]
 ** PRINTER VIEW = [Adobe Syntax Black-10, Tabs = 4]
 ** =======================================================================
 ** Written by James Merkle
 ** Dialog_Of_Background_Removal.c - Video Routines to support Spin-O-Rama
 ** (C) Copyright 1994 James Merkle
 ** =======================================================================
 **
 **
 ** ============
 ** Revision History
 ** ============
 **
 **  1995.04.14		(James Merkle)
 **						Initial coding started from scratch
 **
 **
 **
 ** =================
 ** OTHER INFORMATION
 ** =================
 ** 
 ** 
 **
 **
 **/
/* [----END----] FILE HEADER COMMENTS */
/*=**************************************************************************=*/


/*=**************************************************************************=*/
/* [---BEGIN---] #include AREA */


#include "Automatic_Dialogs.h"
#include "String_Utilities.h"
#include "Command_Dialogs.h"
#include "Shell_Assembly.h"
#include "Dialog_Of_Background_Removal.h"

/* [----END----] #include AREA */
/*=**************************************************************************=*/


/*=**************************************************************************=*/
/* [---BEGIN---] #define AREA */

#define	BACKGROUND_OPTIONS_DIALOG_ID				7200
#define	BACKGROUND_OPTIONS_OUTLINE_ITEM			76

/* [----END----] #define AREA */
/*=**************************************************************************=*/


/*=**************************************************************************=*/ 
/* [---BEGIN---] TYPE AREA */

typedef struct {
	Item_Variables_Header	header;
	
	Boolean_Item			example_Check_Box;
	
	Boolean_Item			example_Radio_String_Choice;
	Edit_Text_Item			example_Radio_String_Choice_Text;

	Boolean_Item			example_Radio_Value_Choice;
	Edit_Text_Item			example_Radio_Value_Choice_Value; /* 27 */

	Edit_Text_Item			example_Number_Info_String; /* 83 */
	
	User_Item					cancel_Button_Outline; /* 76 (70th item) */
} Background_Options_Item_Variables_Record;

typedef struct {
	Boolean				example_Check_Box;
	
	INTEGER_16			result_Radio_String_Choice;
	Str255					result_Radio_String_Choice_Text;
	
	INTEGER_16			result_Radio_Value_Choice;
	CARDINAL_32		result_Radio_Value_Choice_Value;

}Background_Options, *Background_Options_Ptr;


/* [----END----] TYPE AREA */
/*=**************************************************************************=*/


/*=**************************************************************************=*/
/* [---BEGIN---] PROTOTYPE AREA */

Safe_Boolean Initialize_Defaults_For_Background_Removal(Background_Options_Ptr	options);

static Safe_Boolean Validate_And_Copy_Contents(Background_Options_Item_Variables_Record *items
													, Background_Options_Ptr options);

Boolean Get_Background_Removal_Options(Background_Options_Ptr	options, CARDINAL_32 total_Picts);

/* [----END----] PROTOTYPE AREA */
/*=**************************************************************************=*/


/*=**************************************************************************=*/
/* [---BEGIN---] GLOBAL DEFINITION AREA */


/* [----END----] GLOBAL DEFINITION AREA */
/*=**************************************************************************=*/


/*=**************************************************************************=*/
/* [---BEGIN---] LOCAL STATIC DEFINITION AREA */

static CARDINAL_8	*unit_Name = "\pDialog_Of_Background_Removal";


/* [----END----] LOCAL STATIC DEFINITION AREA */
/*=**************************************************************************=*/


/*=**************************************************************************=*/
Safe_Boolean Initialize_Defaults_For_Background_Removal(Background_Options_Ptr	options)
{

	options->example_Check_Box = FALSE;
	
	options->result_Radio_String_Choice = 0;
	options->result_Radio_String_Choice_Text[0] = 0;
	Copy_Pascal_String("\pDefault", options->result_Radio_String_Choice_Text);
	
	options->result_Radio_Value_Choice = 0;
	
	options->result_Radio_Value_Choice_Value = 180;


	return(SAFE_TRUE);
}
/*=**************************************************************************=*/


/*=**************************************************************************=*/
static Safe_Boolean Validate_And_Copy_Contents(Background_Options_Item_Variables_Record *items
													, Background_Options_Ptr options)
{
	Safe_Boolean 	function_Result;
	CARDINAL_32	current_Value;
	CARDINAL_16	string_Length;
	CARDINAL_8		*string_Ptr;
	Boolean 			good_Input;
/* END VAR */


	good_Input = TRUE;
	
	/*  */
	/* Convert all numerical strings first */
	/* Compare numerical values for acceptability */
	/*  */
	
	string_Length = Strip_Outside_Controls(items->example_Radio_Value_Choice_Value.edit_String)[0];
	if ((string_Length < 1) || (string_Length > 3)) {
		good_Input = FALSE;
	}
	
	
	if (Validate_Ordinal_String_And_Copy(items->example_Radio_Value_Choice_Value.edit_String
								, 360
								, &(options->result_Radio_Value_Choice_Value)) != TRUE) {
		good_Input = FALSE;
	}
	
	
	string_Length = Strip_Outside_Controls(items->example_Radio_String_Choice_Text.edit_String)[0];
	if ((string_Length < 1) || (string_Length > 31)) {
		good_Input = FALSE;
	}
	
	/*  */
	/* Assign numerical values */
	/*  */

	/*  */
	/* Copy strings compare lengths first. */
	/*  */

	/*  */
	/* Copy strings, truncate if needed */
	/*  */

	/*  */
	/* Copy over checkbox booleans */
	/*  */
	
	options->example_Check_Box = items->example_Check_Box.checked;

	options->result_Radio_String_Choice = 0;
	options->result_Radio_String_Choice_Text[0] = 0;
	Copy_Pascal_String("\pDefault", options->result_Radio_String_Choice_Text);

	if (items->example_Radio_String_Choice.checked != FALSE) {
		options->result_Radio_String_Choice = 0;
		Copy_Pascal_String(items->example_Radio_String_Choice_Text.edit_String, options->result_Radio_String_Choice_Text);
	}
	else {
		good_Input = FALSE;
	}


	if  (good_Input != TRUE) {
		goto Error_Handler;
	}	

	/*  */
	/* Convert radio button selection groups select first choice */
	/*  */			
	
	if (items->example_Radio_Value_Choice.checked != FALSE) {
		options->result_Radio_Value_Choice = 0;
	}
	else {
		good_Input = FALSE;
	}


	if  (good_Input != TRUE) {
		goto Error_Handler;
	}	


	function_Result = SAFE_TRUE;
	goto Normal_Function_Exit;
	
Error_Handler:
	function_Result = SAFE_FALSE;
			
Normal_Function_Exit:


	return(function_Result);
}
/*=**************************************************************************=*/


/*=**************************************************************************=*/
Boolean Get_Background_Removal_Options(Background_Options_Ptr	options
											, CARDINAL_32 example_Number_Info_Value)
{
	Background_Options_Item_Variables_Record	items;
	
	INTEGER_8		*work_String_Ptr;
	Str255				work_String;
	INTEGER_16		error_Code;
	INTEGER_32		length_1_Entered;
	Str32				value_String;
	Boolean			valid_Input;
	Boolean			keep_Inquiring;
	Boolean			function_Result;
/* END VAR */


	function_Result = FALSE;
	
	/*  */
	/* Allows all pointers to be set to "nil" lazy style */
	/*  */
	Fill_Byte_Buffer((Ptr)&items, sizeof(items), 0);
	
#if !defined(ORIGINAL_MAC_SYSTEM)
#error "UNDEFINED PARAMETER!"
#endif
#if (ORIGINAL_MAC_SYSTEM == 0)
Dialogs.h
XXXX 		
	items.cancel_Button_Outline.setup_Function = (ProcPtr)Setup_Outline_New;
	items.cancel_Button_Outline.update_Function = (ProcPtr)Update_Outline;
#else
	items.cancel_Button_Outline.setup_Function = (void *)Setup_Outline_New;
	items.cancel_Button_Outline.update_Function = (void *)Update_Outline;
#endif 
	
	
	items.cancel_Button_Outline.action_Function = NULL;


	if (Initialize_Defaults_For_Background_Removal(options) != SAFE_TRUE) {
		goto Error_Handler;
	}
	/*  */
	/* NOTE: Initialize_Defaults_For_Background_Removal filled in defaults */
	/* you may modify here */
	/*  */

	items.example_Check_Box.enabled = TRUE;

	items.example_Radio_String_Choice.enabled = TRUE;

	switch(options->result_Radio_String_Choice) {
		case 0:
			items.example_Radio_String_Choice.checked = TRUE;
			break;
						
		default:
			items.example_Radio_String_Choice.checked = TRUE;
			break;
	} /* switch(options->result_Radio_String_Choice) */

	items.example_Radio_String_Choice_Text.enabled = TRUE;

	items.example_Radio_Value_Choice.enabled = TRUE;

	items.example_Radio_Value_Choice_Value.enabled = TRUE;
	
	items.example_Check_Box.checked = FALSE;
			
	items.example_Radio_Value_Choice.checked = TRUE;
	
	Copy_Pascal_String("\p180", items.example_Radio_Value_Choice_Value.edit_String);
	

	Copy_Pascal_String("\pHappy", items.example_Radio_String_Choice_Text.edit_String);

	items.example_Radio_String_Choice_Text.edit_String_Edit_Length = 31;
	
	items.example_Radio_Value_Choice_Value.edit_String_Edit_Length = 3;
	
	items.example_Radio_String_Choice_Text.edit_Field_Type = string_Field;
	
	items.example_Radio_Value_Choice_Value.edit_Field_Type = integer_Field;
	
	
	items.example_Number_Info_String.enabled = FALSE;
	NumToString(example_Number_Info_Value,(CARDINAL_8 *)value_String);
	Copy_Pascal_String(value_String, items.example_Number_Info_String.edit_String);
	items.example_Number_Info_String.edit_String_Edit_Length = 4;

	current_Outline_Item_Number = BACKGROUND_OPTIONS_OUTLINE_ITEM;
	
	valid_Input = FALSE;
	keep_Inquiring = TRUE;
	
	do {					/* while (valid_Input == FALSE); */
		if (Perform_Dialog(BACKGROUND_OPTIONS_DIALOG_ID, (Item_Variables_Ptr)&items) == TRUE) {
		
			if (items.header.item_Hit == OK) {
			
				if (Validate_And_Copy_Contents(&items, options) == SAFE_TRUE) {
					valid_Input = TRUE;
					keep_Inquiring = FALSE;
				}
				else {
					/*  */
					/* Redundant, but placed here for debugging breaks */
					/*  */
					valid_Input = FALSE;
					keep_Inquiring = TRUE;
				}
								
			} /* if (items.header.item_Hit == OK) */
			else {
				; /* NULL */
				
			} /* NOT if (items.header.item_Hit == OK) */
				
		} /* if (Perform_Dialog(BACKGROUND_OPTIONS_DIALOG_ID, (Item_Variables_Ptr)&items) */
		else {
			valid_Input = FALSE;
			keep_Inquiring = FALSE;
		}
	
	} while (keep_Inquiring == TRUE);


	if (valid_Input != TRUE) {
		goto Error_Handler;
	}

	function_Result = TRUE;
	goto Normal_Function_Exit;
	
Error_Handler:
	function_Result = FALSE;
			
Normal_Function_Exit:

	return(function_Result);
	
} /* Boolean Get_Background_Removal_Options(... */
/*=**************************************************************************=*/


/*=**************************************************************************=*/
/* END OF FILE: Dialog_Of_Background_Removal.c */






