/* START OF FILE: about.c */


/*=**************************************************************************=*/
/* [---BEGIN---] FILE HEADER COMMENTS */
/**
 ** SCREEN VIEW = [Adobe Syntax Black-10, Tabs = 4]
 ** PRINTER VIEW = [Adobe Syntax Black-10, Tabs = 4]
 ** =======================================================================
 ** Written by James Merkle
 ** about.c - Routines to support about box
 ** (C) Copyright 1992 by James Merkle.
 ** =======================================================================
 **
 **
 ** ============
 ** Revision History
 ** ============
 **
 **  1992.12.09		(James Merkle)
 **						Initial coding started
 **
 **
 **
 ** =================
 ** OTHER INFORMATION
 ** =================
 ** 
 **
 **
 **
 **
 **/
/* [----END----] FILE HEADER COMMENTS */
/*=**************************************************************************=*/


/*=**************************************************************************=*/
/* [---BEGIN---] #include AREA */

#include "Interface_Utilities.h"
#include "Window_Utilities.h"


#include "Core_Globals.h"
#include "Core_Universal.h"
#include "Off_Screen_Color.h"
#include "MOD_Utilities.h"

#include "Interface_Utilities.h"

#include "Sprite_World.h"
#include "Sprite_World_Sprite_Layer.h"
#include "Sprite_World_Sprite.h"
#include "Sprite_World_Frame.h"
#include "Sprite_World_Utilities.h"

#include "Sprite_World_Blit_Pixie.h"

/* ===
#include "DialogUtils.h"
#include "SpriteTest.h"
=== */

#include "about.h"

/* [----END----] #include AREA */
/*=**************************************************************************=*/


/*=**************************************************************************=*/
/* [---BEGIN---] #define AREA */


#define k_MOD_MUSIC_FOR_ABOUT		128

#define k_VERSION_STRING_INDEX			1
#define k_AUTHOR_STRING_INDEX			2
#define k_COPYRIGHT_STRING_INDEX		3

#define k_VERSION_USER_ITEM					6
#define k_AUTHOR_USER_ITEM					7
#define k_COPYRIGHT_USER_ITEM				8


#define TICKS_BEFORE_ABOUT_BOX_CLOSES		480

#define k_BIG_SPRITE_PICT_ID								4300
#define k_BIG_SPRITE_MASK_ID								4310
	
#define k_SPRITE_MAIN_ICON_ID							300
#define k_SPRITE_MAIN_ICON_SHADOW_ID			301
#define k_SPRITE_SHADOW_FRAMES					3

#define k_SPRITE_SHADOW_LOCATION_ITEM		9
#define k_SPRITE_START_LOCATION_ITEM			10
#define k_BIG_SPRITE_START_LOCATION_ITEM	2


/*  */
/* These things are related? Must be one apart? */
/*  */
#define k_SPRITE_START_SPEED							14
#define k_SPRITE_MAX_SPEED							15

/*  */
/* Was "55" */
/*  */
#define k_SPRITE_MOVEMENT_SPEED				55

#define k_USE_BLIT_PIXIE_CODE							0

/* [----END----] #define AREA */
/*=**************************************************************************=*/


/*=**************************************************************************=*/
/* [---BEGIN---] TYPE AREA */
	
	typedef struct {
		Rect						copyright_Rect, version_Rect, author_Rect;
		Str255					copyright_String, version_String, author_String;
	} Info_String_Rec, *Info_String_Ptr;

/* [----END----] TYPE AREA */
/*=**************************************************************************=*/


/*=**************************************************************************=*/
/* [---BEGIN---] PROTOTYPE AREA */


static void Get_Click(void);

void EarthMoveProc(SpritePtr srcSpriteP, Point* spritePoint);

void ShadowFrameProc(SpritePtr srcSpriteP, FramePtr curFrameP, long* curFrameIndex);

Safe_Boolean Force_Redraw_User_Dialog_Items(DialogPtr about_Dialog, Info_String_Ptr info_Strings);

INTEGER_16 Get_Owned_Resource_ID(INTEGER_16 resource_Selector);

/* [----END----] PROTOTYPE AREA */
/*=**************************************************************************=*/


/*=**************************************************************************=*/
/* [---BEGIN---] GLOBAL DEFINITION AREA */



/* [----END----] GLOBAL DEFINITION AREA */
/*=**************************************************************************=*/


/*=**************************************************************************=*/
/* [---BEGIN---] LOCAL STATIC DEFINITION AREA */

/*  */
/* Not required to be global, but I thought it would be good so that music could */
/* play across several tasks if neede */
/*  */
			MADPartition	g_New_Music_Partition;

/* [----END----] LOCAL STATIC DEFINITION AREA */
/*=**************************************************************************=*/


/*=**************************************************************************=*/
static void Get_Click(void)
{
	EventRecord event;
/* END VAR */


	while (!GetNextEvent(mDownMask, &event)) ;
	
	while (WaitMouseUp()) {
		; /* NULL */
	}
}
/*=**************************************************************************=*/


/*=**************************************************************************=*/
INTEGER_16 Get_Owned_Resource_ID(INTEGER_16 resource_Selector)
{


/* ===
	return( 0xC000 + ((~(g_DA_dce->dCtlRefNum))<<5) + resource_Selector );
=== */

	return(resource_Selector);
}
/*=**************************************************************************=*/


/*=**************************************************************************=*/
void doAbout(void)
{
	DialogPtr			about_Dialog;
	Handle				work_Handle;
	INTEGER_16		about_ID;
	CARDINAL_32	ending_Time;
	Boolean				done;
	GrafPtr				saved_Port, saved_Update_Port;
	EventRecord		wait_Event;
	INTEGER_16		item_Kind;
	FontInfo				font_Info;
	CARDINAL_32	font_Height;
	OSErr					os_Error;
	Safe_Boolean		play_Demo_Song;
	Handle				item_Handle;
	Handle				music_Resource_Hdl;
	Info_String_Rec		about_Strings;
	SpriteWorldPtr		sprite_World_Ptr;
	SpriteLayerPtr		earth_Sprite_Layer_Ptr;
	SpriteLayerPtr		big_Sprite_Layer_Ptr;
	SpritePtr 				big_Sprite;
	SpritePtr 				earth_Sprite;
	SpritePtr 				shadow_Sprite;
	Rect							sprite_Item_Rect;
	Safe_Boolean			sprite_Code_Still_In_Usage;
	Safe_Boolean			sprite_World_Existant;
	Rect							move_Bounds_Rect;
/* END VAR */

	sprite_World_Ptr = nil;
	big_Sprite = nil;
	big_Sprite_Layer_Ptr = nil;
	earth_Sprite_Layer_Ptr = nil;
	earth_Sprite = nil;
	shadow_Sprite = nil;
	sprite_World_Existant = SAFE_FALSE;
	sprite_Code_Still_In_Usage = SAFE_FALSE;
	os_Error = SWEnterSpriteWorld();
	if (os_Error == noErr) {
		sprite_World_Existant = SAFE_TRUE;
	}
	
	GetPort(&saved_Port);
	
	about_Dialog = nil;
	
	if(Get_Bit_Depth_Of_Primary_Device() < 3) {
		about_ID = Get_Owned_Resource_ID(k_ABOUT_BW_ID);
	}
	else {
		about_ID = Get_Owned_Resource_ID(k_ABOUT_COLOR_ID);
	}

	work_Handle = nil;
	work_Handle = GetResource('DLOG', about_ID);
	
	if ((work_Handle != nil) && (ResError() == noErr)) {
		ReleaseResource(work_Handle);
		about_Dialog = GetNewDialog(about_ID, (Ptr)nil, (WindowPtr)-1L);
	}
	
	if ((about_Dialog == nil) || (ResError() != noErr)|| (MemError() != noErr)) {

		/* ===
		DebugStr("\pGetNewDialog Failed. Possible missing DLOG/DITL resource ID");
		=== */
	}
	else {
		Set_View_Port(about_Dialog);
		Center_Window(about_Dialog);

		GetIndString(about_Strings.copyright_String, Get_Owned_Resource_ID(k_ABOUT_STRING_LIST_ID)
						, k_COPYRIGHT_STRING_INDEX);
		
		GetIndString(about_Strings.version_String, Get_Owned_Resource_ID(k_ABOUT_STRING_LIST_ID)
						, k_VERSION_STRING_INDEX);
		
		GetIndString(about_Strings.author_String, Get_Owned_Resource_ID(k_ABOUT_STRING_LIST_ID)
						, k_AUTHOR_STRING_INDEX);
	
		GetDItem(about_Dialog, k_COPYRIGHT_USER_ITEM, &item_Kind, &item_Handle, &about_Strings.copyright_Rect);
		GetDItem(about_Dialog, k_VERSION_USER_ITEM, &item_Kind, &item_Handle, &about_Strings.version_Rect);
		GetDItem(about_Dialog, k_AUTHOR_USER_ITEM, &item_Kind, &item_Handle, &about_Strings.author_Rect);
	
		if (sprite_World_Existant == SAFE_TRUE) {
			sprite_Code_Still_In_Usage = SAFE_TRUE;
			
			/*  */
			/* Create the sprite world */
			/*  */
			os_Error = SWCreateSpriteWorldFromWindow(&sprite_World_Ptr, (CWindowPtr)about_Dialog, nil);
		
			if (os_Error == noErr) {	
				/*  */
				/* Create the sprite layer */
				/*  */
				os_Error = SWCreateSpriteLayer(&earth_Sprite_Layer_Ptr);
				if (os_Error != noErr) {
					sprite_Code_Still_In_Usage = SAFE_FALSE;
				}
				
				os_Error = SWCreateSpriteLayer(&big_Sprite_Layer_Ptr);
				if (os_Error != noErr) {
					sprite_Code_Still_In_Usage = SAFE_FALSE;
				}
				
				
#if !defined(k_USE_BLIT_PIXIE_CODE)
#error "UNDEFINED PARAMETER!"
#endif
#if  (k_USE_BLIT_PIXIE_CODE == 1)
				HideCursor();
				SWSetSpriteWorldEraseProc(sprite_World_Ptr, BlitPixieEraseProc);
				SWSetSpriteWorldDrawProc(sprite_World_Ptr, BlitPixieDrawProc);
#endif
			
						
				if (sprite_Code_Still_In_Usage == SAFE_TRUE) {
					os_Error = SWCreateSpriteFromCIconResource(&earth_Sprite
											, nil, k_SPRITE_MAIN_ICON_ID
											, 1
											, kRegionMask);
											
					if (os_Error != noErr) {
						sprite_Code_Still_In_Usage = SAFE_FALSE;
					}
				}
				
				if (sprite_Code_Still_In_Usage == SAFE_TRUE) {
					os_Error = SWCreateSpriteFromCIconResource(&shadow_Sprite
														, nil
														, k_SPRITE_MAIN_ICON_SHADOW_ID
														, k_SPRITE_SHADOW_FRAMES
														, kRegionMask);
														
					if (os_Error != noErr) {
						sprite_Code_Still_In_Usage = SAFE_FALSE;
					}
				}
	
				if (sprite_Code_Still_In_Usage == SAFE_TRUE) {
					os_Error = SWCreateSpriteFromPictResource(&big_Sprite
														, nil
														, k_BIG_SPRITE_PICT_ID
														, k_BIG_SPRITE_MASK_ID
														, 1
														, kRegionMask);
														
					if (os_Error != noErr) {
						sprite_Code_Still_In_Usage = SAFE_FALSE;
					}
				}
	
			} /* if (os_Error == noErr) */
	
			if (sprite_Code_Still_In_Usage == SAFE_TRUE) {
			
				shadow_Sprite->userData = (long)earth_Sprite;
	
				SWAddSprite(earth_Sprite_Layer_Ptr, shadow_Sprite);
				SWAddSprite(earth_Sprite_Layer_Ptr, earth_Sprite);
				SWAddSpriteLayer(sprite_World_Ptr, earth_Sprite_Layer_Ptr);
				
				SWAddSprite(big_Sprite_Layer_Ptr, big_Sprite);
				SWAddSpriteLayer(sprite_World_Ptr, big_Sprite_Layer_Ptr);

	/* ===
				SWLockSpriteWorld(sprite_World_Ptr);
	==== */
	
				GetDItem(about_Dialog
									, k_SPRITE_SHADOW_LOCATION_ITEM
									, &item_Kind
									, &item_Handle
									, &sprite_Item_Rect);
	
				move_Bounds_Rect = about_Dialog->portRect;
				move_Bounds_Rect.bottom = sprite_Item_Rect.top - (k_SPRITE_MAX_SPEED / 2);
				
				/*  */
				/* set the sprites movement characteristics */
				/*  */
				SWSetSpriteMoveBounds(earth_Sprite, &move_Bounds_Rect);
				SWSetSpriteMoveDelta(earth_Sprite, 0, k_SPRITE_START_SPEED);
				SWSetSpriteMoveProc(earth_Sprite, EarthMoveProc);
				SWSetSpriteMoveTime(earth_Sprite, k_SPRITE_MOVEMENT_SPEED);
				SWSetSpriteMoveTime(shadow_Sprite, -1);		/* never move */
				SWSetSpriteFrameTime(shadow_Sprite, 0);
				SWSetSpriteFrameRange(shadow_Sprite, 0, 2);
				SWSetSpriteFrameProc(shadow_Sprite, ShadowFrameProc);
				SWSetCurrentFrameIndex(shadow_Sprite, 2);
				SWSetSpriteMoveBounds(shadow_Sprite, &move_Bounds_Rect);
				
				/*  */
				/* set the sprites initial location */
				/*  */
				SWSetSpriteLocation(shadow_Sprite, sprite_Item_Rect.left, sprite_Item_Rect.top);
				
				GetDItem(about_Dialog
									, k_SPRITE_START_LOCATION_ITEM
									, &item_Kind
									, &item_Handle
									, &sprite_Item_Rect);
									
				SWSetSpriteLocation(earth_Sprite, sprite_Item_Rect.left, sprite_Item_Rect.top);
				
				GetDItem(about_Dialog
									, k_BIG_SPRITE_START_LOCATION_ITEM
									, &item_Kind
									, &item_Handle
									, &sprite_Item_Rect);
				SWSetSpriteLocation(big_Sprite, sprite_Item_Rect.left, sprite_Item_Rect.top);
				
				((WindowPeek)about_Dialog)->refCon = (long)sprite_World_Ptr;
		
			} /* 	if (sprite_Code_Still_In_Usage == SAFE_TRUE) */
			
		} /* if (sprite_World_Existant == SAFE_TRUE) */
		
		ShowWindow(about_Dialog);
		SelectWindow(about_Dialog);
					
		TextSize(9);
		TextFont(geneva);
		TextFace(bold);
		GetFontInfo(&font_Info);
		font_Height = font_Info.descent +font_Info.ascent + font_Info.leading;
		
		SWLockSpriteWorld(sprite_World_Ptr);
		
		play_Demo_Song = SAFE_TRUE;
		if (play_Demo_Song == SAFE_TRUE) {
		
			if (Start_Playing_MOD_Resource(k_MOD_MUSIC_FOR_ABOUT, &g_New_Music_Partition, &music_Resource_Hdl) != SAFE_TRUE) {
				play_Demo_Song = SAFE_FALSE;
			}
			
			
		}
		
		if (sprite_Code_Still_In_Usage == SAFE_TRUE) {
					
			if (Force_Redraw_User_Dialog_Items(about_Dialog, &about_Strings) != SAFE_TRUE) {
				/*  */
				/* Ignore errors */
				/*  */
			}
			
			ValidRect(&(about_Dialog->portRect));

			CopyBits(&about_Dialog->portBits
						, (BitMapPtr)sprite_World_Ptr->backFrameP->framePix.bitMapP
						, &about_Dialog->portRect
						, &about_Dialog->portRect
						, srcCopy
						, nil);
						
			/*  */
			/* Debug only, copy back for verification
			/*  */
			/*  */
			/*  */
			/*  */
			CopyBits((BitMapPtr)sprite_World_Ptr->backFrameP->framePix.bitMapP
						, &about_Dialog->portBits
						, &about_Dialog->portRect
						, &about_Dialog->portRect
						, srcCopy
						, nil);
						
			/*  */
			/* Run first animation frame for debugging */
			/*  */
			SWLockSpriteWorld(sprite_World_Ptr);
			SWProcessSpriteWorld(sprite_World_Ptr);
			SWAnimateSpriteWorld(sprite_World_Ptr);
						
		} /* 	if (sprite_Code_Still_In_Usage == SAFE_TRUE) */
				
		ending_Time = (TICKS_BEFORE_ABOUT_BOX_CLOSES + TickCount());
		done = FALSE;
		
		/*=--------------------------------------------------------------------------=*/
		/* SystemTask deliberately not called in loop because this could be in a DA */
		/*=--------------------------------------------------------------------------=*/
		/*  while((EventAvail(mDownMask + keyDownMask + autoKeyMask, &wait_Event) == FALSE) 
						&& (done == FALSE)); */

		do {
		
		
		/*  */
		/*  */
		/*  */
		/*  */
		/* HACK HACK HACK POSSIBLE BUG! wait_Event not set */
		/*  */
		
			if(EventAvail(updateMask, &wait_Event) == TRUE) {
				if(wait_Event.message == (INTEGER_32)about_Dialog) {

					GetPort(&saved_Update_Port);
					Set_View_Port(about_Dialog);
					BeginUpdate(about_Dialog);
	
					if (Force_Redraw_User_Dialog_Items(about_Dialog, &about_Strings) != SAFE_TRUE) {
						/*  */
						/* Ignore errors */
						/*  */
					}

					if (sprite_Code_Still_In_Usage == SAFE_TRUE) {
						SWLockSpriteWorld(sprite_World_Ptr);
						SWUpdateSpriteWorld(sprite_World_Ptr);
					}
							
					EndUpdate(about_Dialog);
					Set_View_Port(saved_Update_Port);

				}
				else {
					SelectWindow(about_Dialog);
				}
			}
				
			if(TickCount() > ending_Time) {
			
		/*===
			CheckPlayingChannel(g_New_Music_Partition) == TRUE
			=== */
			
				if (play_Demo_Song == SAFE_TRUE) {
					ending_Time = (TICKS_BEFORE_ABOUT_BOX_CLOSES + TickCount());
				}
				else {
					done = TRUE;
				}
			}
			else {
				/*  */
				/* Run the about box animation */
				/*  */

				SWLockSpriteWorld(sprite_World_Ptr);
				SWProcessSpriteWorld(sprite_World_Ptr);
				SWAnimateSpriteWorld(sprite_World_Ptr);
				
				/*  */
				/* Keep the title animation going! */
				/*  */
			/* ===
				RunSpriteTest(gSpriteTestP);
			=== */
			}
						
			if (GetNextEvent(mDownMask + keyDownMask + autoKeyMask, &wait_Event) == TRUE)	{
			
				switch (wait_Event.what) {			
					case keyDown:
					case autoKey:
						done = TRUE;
						break;
			
					case mouseDown:
						done = TRUE;
						break;
			
				} /* switch (wait_Event->what)  */
			}

		} while (done == FALSE);
			
		
		if (play_Demo_Song == SAFE_TRUE) {
				
			if (Stop_Playing_MOD_Resource(&g_New_Music_Partition, &music_Resource_Hdl) != SAFE_TRUE) {
				/*  */
				/* Ignore errors in this implementation */
				/*  */
			}
		}
		
		DisposDialog(about_Dialog);
		
	} /* NOT if (about_Dialog == nil) */
	
	if (sprite_World_Existant == SAFE_TRUE) {
	
		if (earth_Sprite != nil)
			SWDisposeSprite(earth_Sprite, true);

		if (shadow_Sprite != nil)
			SWDisposeSprite(shadow_Sprite, true);

		if (big_Sprite != nil)
			SWDisposeSprite(big_Sprite, true);

		
#if !defined(k_USE_BLIT_PIXIE_CODE)
#error "UNDEFINED PARAMETER!"
#endif
#if  (k_USE_BLIT_PIXIE_CODE == 1)
		SWSetSpriteWorldEraseProc(sprite_World_Ptr, SWStdDrawProc);
		SWSetSpriteWorldDrawProc(sprite_World_Ptr, SWStdMaskDrawProc);
		ShowCursor();
#endif
					
		if (earth_Sprite_Layer_Ptr != nil)
			SWDisposeSpriteLayer(earth_Sprite_Layer_Ptr);
			
		if (big_Sprite_Layer_Ptr != nil)
			SWDisposeSpriteLayer(big_Sprite_Layer_Ptr);

		if (sprite_World_Ptr != nil)
			SWDisposeSpriteWorld(sprite_World_Ptr);
	
		SWExitSpriteWorld();
	}
	
	Set_View_Port(saved_Port);


} /* static void Perform_About_Command(void) */
/*=**************************************************************************=*/


/*=**************************************************************************=*/
Safe_Boolean Force_Redraw_User_Dialog_Items(DialogPtr about_Dialog, Info_String_Ptr info_Strings)
/*  */
/* Deliberate limitation : current port must already be "about_Dialog" */
/*  */
{
	Safe_Boolean		function_Result;
/* END VAR */


	function_Result = SAFE_FALSE;
	
	TextFont(systemFont);		/* applFont */
	TextSize(12);
	TextFace(0);

	TextFont(geneva);
	TextSize(9);
	TextFace(bold);
	
	DrawDialog(about_Dialog);
	
	TextBox(&info_Strings->author_String[1]
		, info_Strings->author_String[0]
		, &info_Strings->author_Rect
		, teJustCenter);
		
	TextBox(&info_Strings->copyright_String[1]
		, info_Strings->copyright_String[0]
		, &info_Strings->copyright_Rect
		, teJustCenter);
		
	TextBox(&info_Strings->version_String[1]
		, info_Strings->version_String[0]
		, &info_Strings->version_Rect
		, teJustCenter);
								
								
	function_Result = SAFE_TRUE;
	return(function_Result);
	
} /* Safe_Boolean Force_Redraw_User_Dialog_Items ... */
/*=**************************************************************************=*/


/*=**************************************************************************=*/
void EarthMoveProc(SpritePtr srcSpriteP, Point* spritePoint)
{
		/* gravity */
	if (srcSpriteP->vertMoveDelta < (short)k_SPRITE_MAX_SPEED)
		srcSpriteP->vertMoveDelta++;

		/* bounce */
	if (srcSpriteP->destFrameRect.bottom > srcSpriteP->moveBoundsRect.bottom)
	{
		srcSpriteP->vertMoveDelta = -srcSpriteP->vertMoveDelta;
		
		spritePoint->v = srcSpriteP->moveBoundsRect.bottom -
			(srcSpriteP->curFrameP->frameRect.bottom - srcSpriteP->curFrameP->frameRect.top);
	}
}
/*=**************************************************************************=*/


/*=**************************************************************************=*/
void ShadowFrameProc(SpritePtr srcSpriteP, FramePtr curFrameP, long* curFrameIndex)
{
	SpritePtr earth_Sprite = (SpritePtr)srcSpriteP->userData;
	short distanceFromTop = earth_Sprite->destFrameRect.top - srcSpriteP->moveBoundsRect.top;
	short height = srcSpriteP->moveBoundsRect.bottom - srcSpriteP->moveBoundsRect.top;

	*curFrameIndex = (long)((distanceFromTop * 3) / height);

		/* lets not index past the last frame! */
		/* (even though SpriteWorld will not allow it to happen) */
	if (*curFrameIndex > 2L) *curFrameIndex = 2L;
}
/*=**************************************************************************=*/


/*=**************************************************************************=*/
/* END OF FILE: about.c */
