
/**
 * NI_DAQ_MAC.h
 *
 * Copyright (c) 1990 National Instruments Corporation.  All rights reserved.
 *
 * This file should be included in the source files that call NB NI_DAQ_MAC functions.
 *
 * This file contains:
 *  1.	NI_DAQ_MAC Function Prototypes - Provide extra type checking and type casting when 
 *		calling NI_DAQ_MAC functions.  
 *	2. 	NI_DAQ_MAC error code definitions.
 *	3. 	Definitions for certain macros used by the example programs. 
 *   
 **/
 
 /*	The following definitions of int16, int32 are valid for MPW C and THINK C compilers */
typedef short int int16; 
typedef long  int int32;
typedef unsigned int  Uint; 
typedef unsigned long Ulong; 

/* 	LDSysError holds the result of any call to the Device Manager if a NI_DAQ_MAC 
	function returns an error code of -1 */
extern int16 LDSysError;

/* NB-Series Board Types */
#define NB_MIO_16		0x107
#define NB_DIO_32F		0x108
#define NB_GPIB			0x109
#define NB_DMA_8		0x10A
#define NB_AO_6			0x10B
#define NB_DIO_24		0x10C
#define NB_MIO_16X		0x198
#define NB_A2000		0x1C9
#define NB_PRL			0x1DB
#define LAB_NB          0x1C8
#define NB_DMA2800		0x1CA
#define NB_A2100		0x1CB	
#define NB_A2150		0x36F
#define NB_TIO_10		0x374
#define Lab_LC			0x3D2
#define NB_DIO_96		0x3D3

/* Miscellaneous Constants */
#define	FALSE	0
#define	TRUE	1

/* Misc. */
 int16 Master_Slave_Config (int16 slot, int16 numSlaves, int16 *SlaveList);
 
 int16 Board_Reset (int16 slot);
 
 void Board_ID (int16 slot, int16 *brd_id);
 
 void Board_Name(int16 slot, char *brdName);

/* NB-A2000 board specific functions */
 int16 A2000_Config (int16 slot, int16 sclock_src, int16 sclock_drv, int16 dither, int16 mem_type);
 
 int16 A2000_Calibrate (int16 slot, int16 save_new_vals, int16 cal_method, int16 channel, float ext_ref_voltage);
 
/* NB-A2100 board specific functions */
 int16 A2100_Calibrate (int16 slot, int16 AD_group, int16 reference);
 
 int16 A2100_Config (int16 slot, int16 DA_source, int16 trigger);

/* NB-A2150 board specific functions */
 int16 A2150_Config (int16 slot, int16 trig_drv, int16 master_clock, int16 num_slaves, int16 *slave_list); 
 
 
 /* NB-MIO-16X board specific function */
 int16 MIO_16X_Config (int16 slot, int16 adc_type);
  
 /* Analog input functions  */
  int16 AI_Config(int16 slot, int16 input_mode, int16 input_range, int16 polarity);
  
  int16 AI_Read(int16 slot, int16 chan, int16 gain, int16 *reading);
  
  int16 AI_Scale(int16 slot, int16 gain, int16 reading, float *voltage); 
  
  int16 AI_Setup(int16 slot, int16 chan, int16 gain);
  
  int16 AI_Check(int16 slot, int16 *status, int16 *reading);
  
  int16 AI_Clear(int16 slot);
  
  int16 AI_Mux_Config(int16 slot, int16 muxNum);
  
  int16 MAI_Coupling(int16 slot, int16 num_chans, int16 *coupling);
  
  int16 MAI_Setup(int16 slot, int16 num_chans, int16* chan_list, int16* gain_list, int32 mux_int, int16 timebase, int16 mux_mode);

  int16 MAI_Arm(int16 slot, int16 on_off);
  
  int16 MAI_Read(int16 slot, int16 *reading);
  
  int16 MAI_Scale(int16 slot, int32 count, int16 *values, float *volts);
  
  int16 MAI_Clear(int16 brd);
   
/* Analog output functions */
 int16 AO_Config(int16 slot, int16 chan, int16 output_mode, float output_range, int16 update_mode);

 int16 AO_Write(int16 slot, int16 chan, int16 value);

 int16 AO_Update(int16 slot);
 
 int16 AO_VScale(int16 slot, int16 chan, float voltage, int16 *bin_val);
 
 int16 AO_Setup(int16 slot, int16 chan, int16 output_mode, float output_range, int16 update_mode, int16 update_signal, int16 update_edge);

/* Counter/Timer functions */
 int16 CTR_Config(int16 slot, int16 ctr, int16 edge_mode, int16 gate_mode, int16 out_type, int16 out_polar);

 int16 CTR_Period(int16 slot, int16 ctr, int16 timebase);

 int16 CTR_Stop(int16 slot, int16 ctr);
 
 int16 CTR_Restart(int16 slot, int16 ctr);
 
 int16 CTR_Reset(int16 slot, int16 ctr, int16 outpt);
 
 int16 CTR_State(int16 slot, int16 ctr, int16 *out_state);
 
 int16 CTR_EvCount(int16 slot, int16 ctr, int16 timebase, int16 cont);
 
 int16 CTR_EvRead(int16 slot, int16 ctr, int16 *overflow, int32 *Count);
 
 int16 CTR_Pulse(int16 slot, int16 ctr, int16 timebase, int32 delay, int32 pulse_width);
 
 int16 CTR_Square(int16 slot, int16 ctr, int16 timebase, int32 period1, int32 period2);
 
 int16 CTR_Clock(int16 slot, int16 mode, int16 timebase, int16 division);
 
 int16 CTR_FOUT_Config(int16 slot, int16 FOUT_port, int16 mode, int16 timebase, int16 division);
 
 int16 ICTR_Reset(int16 slot, int16 ctr, int16 outpt);

 int16 ICTR_Read(int16 slot, int16 ctr, int32 *Count);
 
 int16 ICTR_Setup(int16 slot, int16 ctr, int16 mode, int32 Count, int16 bin_bcd);
 
/* Data acquisition functions */
 int16 DAQ_Config(int16 slot, int16 ext_trig, int16 ext_gate, int16 ext_conv);
 
 int16 DAQ_Start(int16 slot, int16 chan, int16 gain, int16 *buffer, int32 Count, int16 timebase, int32 samp_interval);
 
 int16 DAQ_Check(int16 slot, int16 *status);
 
 int16 DAQ_Clear(int16 slot);
 
 int16 DAQ_Scale(int16 slot, int16 gain, int32 Count, int16 *bin_array, float *volt_array);
 
 int16 SCAN_Setup(int16 slot, int16 num_chans, int16 *chan_vector, int16 *gain_vector, int16 mux_mode);
 
 int16 SCAN_Start(int16 slot, int16 *buffer, int32 Count, int16 timebase, int32 samp_interval);
 
 int16 SCAN_Check(int16 slot, int16 *status);
 
 int16 SCAN_Demux(int16 slot, int16 *buffer, int32 Count, int16 *parray[]);
 
 int16 Lab_SCAN_Start(int16 slot, int16 num_chans, int16 gain, int16 *buffer, int32 count_per_chan, int16 timebase, int32 samp_interval);
 
 int16 Lab_SCAN_Check(int16 slot, int16 *status, int32 *retrieved, int16 *SCAN_order);
 
 int16 DAQ2Config (int16 slot, int16 mode, int32 bufSize, int32 blkCount);
 
 int16 DAQ2Get (int16 slot, int16 trig_chan, int16 slope, int16 value, int16 *buffer, int32 blkCount, int32 timeout);
 
 int16 DAQ2Tap (int16 slot, int16 trig_chan, int16 slope, int16 value, int16 *buffer, int32 blkCount);
 
 int16 DAQ2TGet (int16 slot, int16 trig_chan, int16 slope, int16 value, int16 *buffer, int32 trigPos, int32 blkCount, int32 *actualCnt, int32 timeout);
 
 int16 DAQ2TTap (int16 slot, int16 trig_chan, int16 slope, int16 value, int16 *buffer, int32 trigPos, int32 blkCount);
 
 int16 DAQ2Clear (int16 slot);
 
 int16 DAQ_Trigger (int16 slot, int16 trig_chan, int16 slope, int16 value, int32 timeout);
 
 int16 DAQ2MemConfig (int16 slot, int16 value, int16 mode);
 
 int16 DAQ_PreTrig (int16 slot, int16 altTrig, int32 cnt);
 
 int16 SCAN_IntStart (int16 slot, int16 *buffer, int32 cnt, int16 sampTime, int32 sampInt, int16 scanTime, int32 scanInt);
 
/* Digital input/output functions */
 int16 DIG_Out_Port(int16 slot, int16 port, char pattern);
 
 int16 DIG_Out_Line(int16 slot, int16 port, int16 line, int16 state);
 
 int16 DIG_In_Port(int16 slot, int16 port, char *pattern);
 
 int16 DIG_In_Line(int16 slot, int16 port, int16 line, int16 *state);
 
 int16 DIG_Prt_Config(int16 slot, int16 port, int16 dir, int16 mode);
 
 int16 DIG_Line_Config(int16 slot, int16 port, int16 line, int16 dir);
 
 int16 DIG_Prt_Status(int16 slot, int16 port, int16 *status);
 
 int16 DIG_Grp_Config(int16 slot, int16 grp, int16 grp_size,int16 port, int16 dir);
 
 int16 DIG_Grp_Status(int16 slot, int16 grp, int16 *status);
 
 int16 DIG_In_Group(int16 slot, int16 grp, int32 *grp_pattern);
 
 int16 DIG_Out_Group(int16 slot, int16 grp, int32 grp_pattern);
 
 int16 DIG_Grp_Mode(int16 slot, int16 grp, int16 Pulse, int16 edge, int16 req_pol, int16 ack_pol, int16 set_time);
 
 int16 DIG_Blk_Start(int16 slot, int16 grp, int16 dir, char *buffer, int32 cnt, int32 interval, int16 timebase);
 
 int16 DIG_Blk_Check(int16 slot, int16 grp, int16 *status);
 
 int16 DIG_Blk_Clear(int16 slot, int16 grp);
 
 int16 DIG_Scan_Setup(int16 slot, int16 grp, int16 grp_size, int16* port_list, int16 dir);
 
/* Multiple-channel analog input functions */
 int16 MAI_Arm (int16 slot, int16 on_off);
 
 int16 MAI_Clear (int16 slot);
 
 int16 MAI_Coupling (int16 slot, int16 num_chans, int16 *coupling);
 
 int16 MAI_Read (int16 slot, int16 *reading);
 
 int16 MAI_Scale(int16 slot, int32 count, int16 *unscaled, float *scaled);
 
 int16 MAI_Setup (int16 slot, int16 num_chans, int16 *chan_list, int16 *gain_list, int32 mux_interval, int16 timebase, int16 mux_mode);

/* Multiple-channel data acquisition functions */
 int16 MDAQ_Check (int16 slot, int16 full_check, int16 *acq_done, int32 *current_frame, int32 *current_scan);
 
 int16 MDAQ_Clear (int16 slot);

 int16 MDAQ_Get (int16 slot, int16 scans_or_frames, int16 get_tap, int32 num_to_get, int32 start_frame, int32 start_scan, 
               int32 timeout, int16 *buffer, int32 *num_gotten, int32 *current_frame, int32 *current_sample, int16 *acq_done);

 int16 MDAQ_ScanRate (int16 slot, int32 scan_interval, int16 timebase);
 
 int16 MDAQ_Setup (int16 slot, int32 buffer_size, int16 scans_or_frames, int32 pretrig_scans, int32 posttrig_scans, int16 *buffer);
 
 int16 MDAQ_Start (int16 slot, int32 num_frames);
 
 int16 MDAQ_Stop (int16 slot);
 
 int16 MDAQ_Trig_Delay (int16 slot, int32 delay_interval, int16 timebase);
 
 int16 MDAQ_Trig_Select (int16 slot, int16 digital_trig, int16 edge, int16 analog_trig, int16 slope, int16 atrig_level, int16 atrig_source);

 int16 MDAQ_Trig_Config (int16 slot, int16 digital_trig, int16 edge, int16 analog_trig, int32 num_trigs,int16 atrig_source, int16 slope, 
 int16 atrig_level, Uint hyst_window, int32 timeout);
 
 int16 MDAQ_Op (int16 brd, int16 num_chan, int16 *chan_list, int16 *gain_list, int32 scan_int, int16 scan_timebase, int32 mux_int,
  			  int16 mux_timebase, int32 pre_scans, int32 post_scans, int16 mux_mode, int16 *buffer, int32 timeout);

/* RTSI bus trigger functions */
 int16 RTSI_Conn(int16 slot, int16 sig_code, int16 trig_line, int16 dir);
 
 int16 RTSI_Clear(int16 slot);

 int16 RTSI_DisConn(int16 slot, int16 sig_code, int16 trig_line);

/* Waveform generation functions */
 int16 WF_Setup (int16 slot, int16 channel, int32 update_interval, int16 timebase);

 int16 WF_Load (int16 slot, int16 channel, int16 *buffer, int32 cnt, int16 cont);

 int16 WF_DBLoad (int16 slot, int16 channel, int16 *buffer, int32 cnt, int16 cont, int32 timeout);

 int16 WF_Start (int16 slot, int16 channel);

 int16 WF_Check (int16 slot, int16 channel, int16 *status);

 int16 WF_Stop (int16 slot, int16 channel);

 int16 WF_Reset (int16 slot, int16 channel);

 int16 WF_Offset(int16 slot, int16 channel, int32 cnt, int16 *buffer);

 int16 WF_Grp_Setup (int16 slot, int16 num_chans, int16 *chan_vector, int32 update_interval, int16 timebase);

 int16 WF_Grp_Start (int16 slot);

 int16 WF_Grp_Stop (int16 slot);

 int16 WF_Grp_Reset (int16 slot);

/* Buffered waveform functions */

 int16 BWF_BlkLoad (int16 slot, int16 group, int16 channel, int32 buf_number, int16 copy_mode, int16 *data, int32 num_pts, int32 start_pt, int32 block_reps, int32 timeout, int32 *pts_copied, int32 *last_point, int32 *reps_done);
 
 int16 BWF_BufLoad (int16 slot, int16 group, int16 channel, int32 buf_number, int32 cycles, int16 *data, int32 count, int32 update_int, int16 timebase, int16 trig_mode, int32 block_size, int32 data_regen);

 int16 BWF_Check (int16 slot, int16 group, int16 channel, int16 check_mode, int16 *status, int32 *buf_number, int32 *cycles_output, int32 *points_output, int32 *update_int, int16 *timebase);
 
 int16 BWF_Clear (int16 slot, int16 group, int16 channel);
 
 int16 BWF_Rate (int16 slot, int16 group, int16 channel, int32 update_int, int16 timebase, int16 mode);
 
 int16 BWF_Start (int16 slot, int16 group, int16 channel, int32 *buf_numbers, int16 start_mode);
 
 int16 BWF_Stop (int16 slot, int16 group, int16 channel, int16 stop_mode);
 
 int16 BWF_Resume (int16 slot, int16 group, int16 channel);
 /* SCXI functions */

 int16 SCXI_Load_Config(int16 chassisID);
 
 int16 SCXI_Set_Config(int16 chassisID, int16 chassisType, int16 chassisAddr,
 				int16 commMode, int16 commPath, int16 numSlots, long *modules,
				int16 *opModes, int16 *DAQboardMap);

 int16 SCXI_Get_Chassis_Info(int16 chassisID, int16 *chassisType, int16 *chassisAddr,
 				int16 *commMode, int16 *commPath, int16 *numSlots);

 int16 SCXI_Get_Module_Info(int16 chassisID, int16 moduleSlot, long *moduleType,
 				int16 *opMode, int16 *DAQboard);
 				
 int16 SCXI_Reset(int16 chassisID, int16 moduleSlot);
 
 int16 SCXI_Single_Chan_Setup(int16 chassisID, int16 moduleSlot, int16 chan, int16 DAQboard);
 
 int16 SCXI_Set_Module_Gain(int16 chassisID, int16 moduleSlot, int16 gain);
 
 int16 SCXI_SCAN_Setup(int16 chassisID, int16 numModules, int16 *moduleList,
 				int16 *numChansList, int16 *startChansList, int16 DAQboard, int16 scanMode);
 				
 int16 SCXI_Track_Hold_Setup(int16 chassisID, int16 moduleSlot, int16 inputMode,
 				int16 source, int16 send, int16 holdCount, int16 DAQboard);
 				
 int16 SCXI_Change_Single_Chan(int16 chassisID, int16 moduleSlot, int16 chan);
 
 int16 SCXI_Calibrate_Setup(int16 chassisID, int16 moduleSlot, int16 calOp);
 
 int16 SCXI_Track_Hold_Control(int16 chassisID, int16 moduleSlot, int16 state, int16 DAQboard);

 int16 SCXI_MuxCtr_Setup(int16 slot, int16 enable, int16 scanDiv, int16 muxCtrValue);

 int16 SCXI_Get_State(int16 chassisID, int16 moduleSlot, int16 port, int16 channel, long *data);

 int16 SCXI_Get_Status(int16 chassisID, int16 moduleSlot, int16 wait, long *data);
 
 int16 SCXI_Set_State(int16 chassisID, int16 moduleSlot, int16 port, int16 channel, long data);
 
/*
	NI_DAQ_MAC Error Codes
*/
#define labErr			-36		/* NI_DAQ_MAC error occurred (system ioErr) */
#define notOurBrdErr	-60		/* not an NB-SERIES board */
#define badBrdNumErr	-61		/* error in slot input */
#define badGainErr		-62		/* error in gain input */
#define badChanErr		-63		/* error in channel input */
#define noSupportErr	-64		/* function not supported on this board */
#define badPortErr		-65		/* port out of range (brd dependent) */
#define badOutPortErr	-66		/* port not configured for output, in case of write */
#define noLatchModeErr	-67		/* port does not support latched mode (config call) */
#define noGroupAssign	-68		/* port cannot be assigned to group (config call) */
#define badInputValErr	-69		/* illegal input parameter value */
#define timeOutErr		-70		
#define outOfRangeErr 	-71	
#define daqInProgErr	-72		/* no config allowed when data acquisition in 
								   progress */	
#define counterInUseErr -73		/* counter already in use error */
#define noDAQErr		-74		/* no data acquisition in progress */
#define overFlowErr		-75
#define overRunErr		-76
#define badCntErr		-77		/* scan count must be multiple of num_chans */
#define brdTypeErr		-78
#define noCountOpErr	-79
#define ctrReservedErr  -80
#define portAssignToGrp	-81
#define noPortAssignErr	-82		/* no port asigned to group error */
#define noGrpOutErr		-83		/* group not configured for output error */
#define noGrpInErr		-84		/* group not configured for input error */
#define DIOinProgErr    -85		/* block digital I/O already in progress */
#define noDIOinProgErr  -86		/* no block digital I/O in progress */
#define illegalDIOErr   -87		/* one group may not use external handshaking
								   while the other group is doing timed I/O */
#define notDmaGrpErr    -88		/* DMA transfer requires port 0 or 1 in 8-bit grp,
								   ports 0 and 1 in 16-bit grp, or all ports in 32-bit grp. */
#define noLabSCANErr    -89     /* Lab-SCAN-Check call not executable */
#define badSigDirErr   	-90		/* bad RTSI signal direction */
#define trigInUseErr	-91		/* RTSI trigger line is in use */
#define noTrigAvailErr	-92		/* no RTSI trigger lines are available */
#define	sigInUseErr		-93		/* signal is being used */
#define noDmaCardErr	-94		/* a dma card is needed for waveform generation */
#define noDmaAvailErr	-95		/* no dma channel is available */
#define noSetupErr		-96 	/* a setup call is required */	
#define dacChanBusyErr	-97		/* analog output channel is in use */
#define noLoadErr		-98 	/* a waveform load call is required */
#define ChanInGrpErr	-99		/* channel has been assigned to a group
								   only synchronous calls allowed until
								   WF_Grp_Reset is called */
#define	noWvFmErr			-100	/* no waveform operation has been executed */
#define grpSetupErr			-101	/* Only one WF_Grp_Setup call per board is allowed */
#define WFGrpInProgErr		-102	/* Waveform generation group operation is in progress */
#define badAnlgTrigModeErr  -103	/* Specified analog trig mode is not supported 
										Used in MDAQ_Trig_Config, current valid values 0,1 */
#define badNumAnlgTrigsErr	-104	/* the specified number of analog triggers is not supported */
#define badHysWindowErr		-105	/* the specified hystresis window is out of range from
									the specified analog trigger level */ 
#define sharedTrigEnabledErr -106	/* cannot start data acquisition or waveform 
											generation w/o ext-trig if shared trig enabled */
#define contModeNotAllowedErr -107	/* continuous daq or wf not allowed on NB-A2100 if PIO */
#define notEnoughMemErr		-108	/* not enough physical memory */
#define badDMABlockSizeErr	-109	/* DMA block size in DAQ2Config invalid for the sampling rate set in DAQ_Start (when using DMA only) */
#define noBlkModeErr	-110
#define overWriteErr	-111
#define cntNotAvailErr	-112
#define noDAQAvailErr	-113	/* acquisition ended before cnt was available */
#define badBlkSizeErr	-114
#define blkModeConfErr	-115	/* no mem card config when in block mode */
#define notConnectedErr	-116	/* trigger line and signal code are not connected */
#define noIOTrigDrvErr	-117	/* cannot drive a trigger signal to the I/O connector in
									the current trigger configuration of the NB-A2100 & NB-A2150 */
#define trigDrvErr 		-118	/* can't enable both RTSI and ext. trigger (A2100 & A2150) */
#define trigDelayErr	-119	/* delayed trigger mode not supported in current configuration */
#define noTrigValErr	-120	/* no trigger value was found DAQTapBlock */
#define trigTimeOutErr	-121	/* timeout occurred before trigger was */
#define preTrigErr		-122	/*  MIO, A2100: Pre-triggering not allowed in combination with DAQ_Trigger,
									A2000: No analog or digital trigger enabled for Pre-Trigger,
									       or continuous sampling specified for Pre-Trigger. */
#define trigDelayIgnoredErr	-123	/*  A2000: Trigger signal source must be on-board for Post-Trigger */
#define preTrigCntErr   -124    /* The number of samples after the pretrigger in the DAQ_PreTrig
                                   function is not an integer multiple of the number of channels */
#define noArmErr		-125	/* A2000: must call SAI_Arm before SAI_Read for external sample clock */
#define RTSIConnErr		-126	/* Illegal RTSI connection. */
#define noDataErr		-127 	/* Expected data not available in FIFO */
#define noTrigEnabledErr -128	/* No trigger source enabled */
#define brdIsArmedErr	-129	/* Invalid operation while board is armed */
#define noPreSampsErr	-130	/* not enough pre-samples were collected */
#define scanIntErr 		-131	/* SCN2 interval must be at least the length 
								   of (num.chans * samp int) + 2 microsecs  */
#define trigSampErr		-132	/* # of samples following trigger must be <= total # */
#define oddSampsErr		-133	/* total number of samples must be an even number # */ 
#define badTotSampsErr	-133	/* total scans must be is either an odd numer or invalid */
#define pretrigScansErr	-134	/* Invalid number of pretrigger scans specified */
#define	posttrigScansErr -135	/* Invalid number of posttrigger scans specified */
#define frameTooBigErr	-136	/* Max frame size with DMA: 2^23  without DMA: uns long */
#define scanRateErr		-137	/* Invalid scan rate specified */
#define scanRateIgnoredErr -138 /* Scan rate is ignored when using an external sample clock */
#define invalidGetErr	-139	/* MDAQ_Get: trying to get 0 bytes or more bytes than were acquired */
#define dmaCfgErr		-140	/* dmaConfig error */
#define badDACValErr	-141	/* gain/offset DAC value out of range during calibration */
#define noConvergeErr	-142	/* gain/offset calibration can't converge to good value */
#define eeReadErr		-143	/* unable to read value from NB-A2000's EEPROM */
#define eeWriteErr		-144	/* unable to write value to NB-A2000's EEPROM */
#define eeHWErr			-145	/* hardware error when reading value from NB-A2000's EEPROM */
#define eeAddrErr		-146	/* invalid address for NB-A2000's EEPROM read/write */
#define badExtRefErr	-147	/* external reference value out of range during initial calibration */
#define badIntRefErr	-148	/* internal reference value out of range during initial calibration */
#define noCalResponseErr -149	/* no response while calibrating - unable to collect cal. data from board */
#define HWInitErr		-150	/* hardware initialization error */
#define clockSrcErr		-151	/* sample clock source must be consistent with operation */
#define sampClkDrvErr	-152	/* can't receive and drive external sample clock line simultaneously */
#define blkModeBufErr	-153	/* buffer size or base address is not 16 32-bit word aligned for NuBus block DMA xfers */
#define brdInUseForMasterSlaveErr 	-154  /* cannot setup a board for master-slave when it
											 has been set for master-slave */
#define sysOwnedRsrcErr     -155    /* operation failed because resource
                                    is reserved by NI-DAQ */
#define userOwnedRsrcErr    -156    /* operation failed because resource
                                    is reserved by the user */
#define brdInMasterSlaveClockErr -157	/* operation failed because an A2150 or A2100 slot in Master Slave Clock Configuration */
#define badPointsErr		-159	/* Specified points for waveform buffer or block are invalid */
#define daFifoDisabledErr 	-160	/* cannot do D/A output since FIFO is disabled */
#define badBufNumErr		-161	/* specified buffer number is bad */
#define badGrpNumErr		-162	/* specified group number is bad */
#define bwfInProgErr		-163	/* cannot execute operation since waveform in prog */
#define bwfEndedErr			-164	/* cannot execute operation since waveform has ended */	
#define underFlowErr		-165	/* a D/A FIFO underflowed */
#define dmaChainingErr		-166	/* dma chaining operation is halted, increase block size*/
#define underWriteErr		-167	/* some data being updated generated more times than specified */
#define endBlkLoadedErr		-168	/* cannot do further block loads since last block has been loaded */
#define regenStopErr		-169	/* waveform generation has been stopped to prevent regen */
#define oldNewInterfaceErr	-170	/* old and new interface calls are mixed up */
#define badOutLineErr       -175	/* line not configured for output in case of write */
#define noMemCardSupportErr	-179	/* either DMA channel is not available or machine is not in 32 bit mode */
#define cannotMakeContigMemErr  -180 	/* Could not make the virtual memory buffer to be
											 used in a DMA data acquisition or waveform 
											 generation contiguous */
#define trigBlkErr			-181    /* analog triggering and pre-triggering flags in single
									buffer mode must be disabled before starting double 
									buffered mode data acquisition */
#define extSignalErr        -182    /* since external signal is used for later update 
									   mode. This call of AO_Update had no effect */
#define invalidChassisIDErr -183	 /*  the SCXI chassis ID does not
                                    correspond to a configured chassis */
#define invalidModuleSlotErr  -184	 /*the SCXI module slot specified is
                                    invalid or corresponds to an empty slot */
#define configFileErr         -185   /*  the NI-DAQ Preferences file could
                                    not be found	or has been corrupted  */
#define SCXIConfigErr         -186   /* the SCXI configuration parameters
                                    specified indicate an invalid SCXI
                                    configuration, or the current
                                    function cannot be executed because
                                    of the current SCXI configuration */
#define SCXICommErr           -187  /*  SCXI communication error; either the
                                    chassis communication is disabled,
                                    or the driver could not successfully
                                    communicate with the chassis */
#define invalidOpModeErr      -188   /* either the operating mode specified
                                    in an SCXI config call is invalid,
                                    or a module is in the wrong opMode
                                    to execute the current function */
#define moduleNotSupported    -189  /*  one of the SCXI modules specified
                                    for a function is not supported for
                                    the operation; the rest of the
                                    function was executed for the
                                    specified modules that are supported */
#define DAQboardNotSupported  -190   /* the data acquisition board specified
                                    for an SCXI operation is the wrong
                                    board type for the operation */
#define SCXITrackHoldErr      -191   /* an attempt was made to drive the
                                    Track/Hold trigger line on the
                                    SCXIbus with more than one module,
                                    or a SCXI_Track_Hold_Control call
									was made and the Track/Hold setup is
                                    not correct */
#define outOnSomeInLineswarn  -300	/* writing to a port where some lines are configured for input */

#define UnexpErr              -500   /* unexpected error*/

#define toolboxErr	    -1		/* used in C interface code, detects that toolboxErr occurred */	

