/* START OF FILE: Dreaming_Animation.c */


/*=**************************************************************************=*/
/* [---BEGIN---] FILE HEADER COMMENTS */
/**
 ** SCREEN VIEW = [Adobe Syntax Black-10, Tabs = 4]
 ** PRINTER VIEW = [Adobe Syntax Black-10, Tabs = 4]
 ** =======================================================================
 ** Written by James Merkle
 ** Dreaming_Animation.c - Routines to support Poor Man's Video Phone
 ** (C) Copyright 1993 James Merkle
 ** All rights reserved, except those wishing to "copy" moderate sections of code or files may do so, so long as
 ** such code is never used directly either in animation-related tools or multimedia tools that are  
 ** used in live performance or live broadcast or recorded mass market broadcast or film. Desktop usage not restricted.
 ** Conventional desktop game or entertainment not restricted.
 ** =======================================================================
 **
 **
 ** ============
 ** Revision History
 ** ============
 **
 **  1994.07.01		(James Merkle)
 **						Initial coding started, moved over from "Dreaming_Stuff.c"
 **
 **
 **
 **
 **
 ** =================
 ** OTHER INFORMATION
 ** =================
 ** 
 ** 
 **
 **
 **/
/* [----END----] FILE HEADER COMMENTS */
/*=**************************************************************************=*/


/*=**************************************************************************=*/
/* [---BEGIN---] #include AREA */

#include <PictUtil.h>
#include <QuickDraw.h>

#include "Core_Universal.h" /* Non_Specific_Support_Routines */
#include "Core_Specific.h" /*System_Dependent_Hooks.h */
#include "SysHooks.h" /*System_Dependent_Hooks.h */
#include "Shell_Globals.h"
#include "Vertical_Blanking_Interrupt.h"
#include "Core_Globals.h" /*System_Dependent_Hooks.h */
#include "Memory_Utilities.h"
#include "String_Utilities.h"
#include "Shell_Dialogs.h"
#include "Shell_Assembly.h"
#include "Dreaming_Backdrop_Files.h"
#include "Spin_O_Rama_Files.h"
#include "Graphics_Primitives.h"
#include "Stage_View_Manager.h"

#include "Interface_Utilities.h"
#include "Error_Utilities.h"
#include "Window_Environment.h"

#include "Dreaming_Audio.h"
#include "Dreaming_Analog_Card.h"
#include "Dreaming_File_Loading.h"
#include "Dreaming_Scene_Files.h"
#include "Dreaming_Color_Stuff.h"
#include "Dreaming_Animation.h"
#include "Fast_Image_Compaction.h"
#include "Copy_Compacted_Picture.h"
#include "Ring_Buffer_Cache.h"
#include "Spin_O_Rama_Cache.h"

#include "Slide_Show_Routines.h"
#include "Video_Wipes.h"
#include "Copy_Compacted_Silloette.h"

#include "Graphics_Primitives.h"

/* [----END----] #include AREA */
/*=**************************************************************************=*/


/*=**************************************************************************=*/
/* [---BEGIN---] #define AREA */

/*  */
/* When developing, for benchmark effect, you MIGHT want all the */
/* background drawn around individual puppets but we really only should */
/* fill the pixels around the frame that we need. */
/* Set to "1" when finished programming */
/*  */
#define k_ONLY_FILL_CRITICAL_PIXELS_AROUND_PUPPET		1

/*  */
/* Set to "0" because technique does not work two levels deep? */
/* The puppet drawing itself needs to swap around stuctures. */
/*  */
#define k_PREVENT_QUICKDRAW_CLUT_MATCH_IN_BENCHMARK		0

#define k_BENCHMARK_RESULT_POSITION_X		576
#define k_BENCHMARK_RESULT_POSITION_Y		16

#define COLOR_PORT_PORTBITS_CHECK 		0xC000

#define k_BOGUS_ROLL_POSITION_TO_FORCE_MISMATCH	0xF9999999

#define k_MAXIMUM_FOR_12_BIT_RANGE	(4095UL)
#define k_SHIFT_FOR_12_BIT_RANGE		(12)

/* [----END----] #define AREA */
/*=**************************************************************************=*/


/*=**************************************************************************=*/
/* [---BEGIN---] TYPE AREA */


/* [----END----] TYPE AREA */
/*=**************************************************************************=*/


/*=**************************************************************************=*/
/* [---BEGIN---] PROTOTYPE AREA */

void Increment_Bobbing_Cycles(INTEGER_32 *x_Phase
													, INTEGER_32 *y_Phase
													, INTEGER_32 *x_Offset
													, INTEGER_32 *y_Offset
													, INTEGER_32 *cycle_Phase_Rate
													, INTEGER_32 *cycle_Phase_Acceleration
													, INTEGER_32 *cycle_Acceleration_Direction);

Safe_Boolean Bring_View_Into_Ready_To_Composite_Phase(void);

Safe_Boolean	Update_Controller_Settings_For_Item(Stage_Item_Ptr	item_Ptr
																				, CARDINAL_32	*horizontal_12_Bit_Percentage
																				, CARDINAL_32	*vertical_12_Bit_Percentage
																				, CARDINAL_32	*roll_12_Bit_Percentage);

Safe_Boolean	Check_If_Item_Position_Moved(Stage_Item_Ptr	item_Ptr
																				, Boolean *frame_Changed);

/* [----END----] PROTOTYPE AREA */
/*=**************************************************************************=*/


/*=**************************************************************************=*/
/* [---BEGIN---] GLOBAL DEFINITION AREA */


/* [----END----] GLOBAL DEFINITION AREA */
/*=**************************************************************************=*/


/*=**************************************************************************=*/
/* [---BEGIN---] LOCAL STATIC DEFINITION AREA */

static CARDINAL_8	*unit_Name = "\pDreaming_Animation";

/* [----END----] LOCAL STATIC DEFINITION AREA */

/*=**************************************************************************=*/


/*=**************************************************************************=*/
Safe_Boolean Bring_View_Into_Ready_To_Composite_Phase(void)	
/*  */
/* Note: This cycles through Stage_View_Manager until it */
/* apears that it has finally enterred k_DISPLAY_PHASE_READY_TO_COMPOSITE */
/*  */
{
	Safe_Boolean						function_Result;
	Item_Location_List_Ptr		list_Ptr;
	CARDINAL_8		*code_Name;
/* END VAR */


	code_Name = "\pBring_View_Into_Ready_To_Composite_Phase";

	if (dream.stage_View.stage_View_Buffers_Initialized == SAFE_TRUE) {
		list_Ptr = &dream.stage_View.double_Buffered_Data[dream.stage_View.next_Buffer_To_Use];
		
		if (list_Ptr->total_Remaining_Erasable_Items > 0) {
		
			/*  */
			/* Darn, we have things that need to be erased from the buffer */
			/* before we use it.
			/*  */
			if (dream.stage_View.display_Phase > k_DISPLAY_PHASE_BLASTING_PORT_ACTIVE) {
				/*  */
				/* Do something special to repair buffers? */
				/*  */
				dream.stage_View.display_Phase = k_DISPLAY_PHASE_ERASE_ACTIVE;
			}
			else {
				dream.stage_View.display_Phase = k_DISPLAY_PHASE_ERASE_ACTIVE;
			}
			
			while (dream.stage_View.display_Phase != k_DISPLAY_PHASE_READY_TO_COMPOSITE) {
				if (Process_Animation_Tasks_For_Main_Stage(&dream.stage_View) != SAFE_TRUE) {
					goto Error_Handler;
				}
			}
		}
	}
	else {
		goto Error_Handler;
	}

	function_Result = SAFE_TRUE;
	goto Normal_Function_Exit;
	
Error_Handler:
	Display_Serious_Error_Message(unit_Name, code_Name, "\pProblem encountered", "\p");
	function_Result = SAFE_FALSE;
		
Normal_Function_Exit:
	return(function_Result);
	
}
/*=**************************************************************************=*/


/*=**************************************************************************=*/
Safe_Boolean Fill_Selected_Port_With_All_Scene_Items(Off_Screen_Ptr	modern_Destination_Bitmap
																		, const BitMap	*destination_Bitmap
																		, Safe_Boolean	avoid_Modern_Bitmap
																		, CARDINAL_16	scene_To_Use
																		, Boolean show_Closed_Mouths
																		, INTEGER_16		audio_Index
																		, INTEGER_32		extra_Offset_X
																		, INTEGER_32		extra_Offset_Y)	
/*  */
/* Note: This should be used by setting destination port before calling */
/* (if any apple routines expected to be used) */
/* This does not modify or change any stage compositing information */
/* but does add to list of items if it seems appropriate. */
/* Addition to deletion lists are performed inside "Draw_Specific_Frame_Of_Item" */
/* But list is false until port is copied back into proper area. */
/* please clear all that needs clearing. */
/*  */
{
	CARDINAL_32	located_Items;
	CARDINAL_32	total_Items;
	Puppet_Ptr			current_Puppet;
	INTEGER_16		item_Index;
	INTEGER_16		new_Frame;
	INTEGER_16		current_Frame;
	Safe_Boolean		function_Result;
	Boolean				frame_Changed;
/* END VAR */


	located_Items = 0;
	total_Items = (**(dream.play.scene[scene_To_Use])).total_Items_In_Scene;
	for (item_Index = 0; ((item_Index < k_MAX_STAGE_ITEMS) && (located_Items < total_Items)); item_Index++) {

		current_Puppet = (**(dream.play.scene[scene_To_Use])).item[item_Index].puppet;

		if (current_Puppet != nil) {
			located_Items += 1;
			
			/*  */
			/* Add in future: use current amplitude for total redraws, */
			/* not just beginning frames */
			/*  */
			
			if ((show_Closed_Mouths == TRUE) || (audio_Index != EFFECT_NOTHING)){
			
				if (current_Puppet->puppet_File_Is_Old_Self_Contained_Type == SAFE_TRUE) {
					current_Frame = current_Puppet->direct_Art.talk_Start_Frame;
				}
				else {
				
					current_Frame = 0;
					
					if (current_Puppet->redirected_Information_Ptr[current_Puppet->current_Selection].array_Index_Valid == SAFE_TRUE) {
						current_Frame = (**(dream.spin_Objects.spin_Object[current_Puppet->redirected_Information_Ptr[current_Puppet->current_Selection].array_Index])).minimum_Intensity_Representing_Idle;
					}
				
				}
				
			}
			else {
				current_Frame = (**(dream.play.scene[scene_To_Use])).item[item_Index].current_Displayed_Frame[dream.stage_View.next_Buffer_To_Use].intensity;
				if (current_Frame == 0xFFFFFFFF) {
					if (current_Puppet->puppet_File_Is_Old_Self_Contained_Type == SAFE_TRUE) {
						current_Frame = current_Puppet->direct_Art.talk_Start_Frame;
					}
					else {
						if (current_Puppet->redirected_Information_Ptr[current_Puppet->current_Selection].array_Index_Valid == SAFE_TRUE) {
							current_Frame = (**(dream.spin_Objects.spin_Object[current_Puppet->redirected_Information_Ptr[current_Puppet->current_Selection].array_Index])).minimum_Intensity_Representing_Idle;
						}
					}
				}
				
				if ((**(dream.play.scene[scene_To_Use])).item[item_Index].controllers_Attached == SAFE_TRUE) {
								
					if (Get_Current_Volume_Frame(&((**(dream.play.scene[scene_To_Use])).item[item_Index])
													, &frame_Changed
													, (**(dream.play.scene[scene_To_Use])).item[item_Index].current_Displayed_Frame[dream.stage_View.next_Buffer_To_Use].intensity
													, &new_Frame) == SAFE_TRUE) {
													
						/*  */
						/* Ignore "frame_Changed" not important to us in full redraws */
						/*  */
						current_Frame = new_Frame;
					}
				}
			}

			/*  */
			/* If this is the new type of puppet, then for the purposes of this scene */
			/* invalidate the optimization cache art information. */
			/* This is acceptable because
			/*  */
			if (current_Puppet->puppet_File_Is_Old_Self_Contained_Type == SAFE_TRUE) {
			
			}
			
			/*  */
			/* Only change the intensity, leave the orientation */
			/*  */
			(**(dream.play.scene[scene_To_Use])).item[item_Index].current_Displayed_Frame[dream.stage_View.next_Buffer_To_Use].intensity = current_Frame;
			
			(**(dream.play.scene[scene_To_Use])).item[item_Index].last_Image.roll_Angle = k_BOGUS_ROLL_POSITION_TO_FORCE_MISMATCH;
			(**(dream.play.scene[scene_To_Use])).item[item_Index].frame_Needing_Erasure[dream.stage_View.next_Buffer_To_Use].valid_Information = SAFE_FALSE;
	
	
			if (Draw_Specific_Frame_Of_Item(modern_Destination_Bitmap
																		, destination_Bitmap
																		, avoid_Modern_Bitmap
																		, scene_To_Use
																		, item_Index
																		, &(**(dream.play.scene[scene_To_Use])).item[item_Index].current_Displayed_Frame[dream.stage_View.next_Buffer_To_Use]
																		, TOP_LEFT(((**(dream.play.scene[scene_To_Use])).item[item_Index].desired_Start_Location))
																		, extra_Offset_X
																		, extra_Offset_Y) != SAFE_TRUE) {
				/*  */
				/* Ignore errors in this implementation */
				/*  */
			}
			
		}
		
	} /* for (item_Index = 0; ((item_Index < k_MAX_STAGE_ITEMS)... */
	
	function_Result = SAFE_TRUE;
	return(function_Result);
	
} /* Safe_Boolean Fill_Selected_Port_With_All_Scene_Items(scene_To_Use ... */
/*=**************************************************************************=*/


/*=**************************************************************************=*/
Safe_Boolean	Redraw_Idle_Screen_Completely(Boolean actual_System_Update, Boolean show_Closed_Mouths)
	/*  */
	/* Draws whole frame, not just deviance */
	/*  */
{
	Safe_Boolean		function_Result;
	Safe_Boolean		port_Was_Set;
	CARDINAL_16	scene_To_Use;
	Boolean				background_Can_Be_Read;
	CARDINAL_32	back_Index;
	CARDINAL_16	color_Type_Of_Port;
	INTEGER_16		audio_Index;
	RgnHandle			clipping_Region;
	Safe_Boolean			device_Was_Set;
	GDHandle				saved_Device;
	GrafPtr				saved_Original_Port;
	PenState				saved_Pen_State;
	Rect					destination_Rect;
	Rect					source_Rect;
	Boolean				use_Direct_Screen_Access;
	WindowPtr			current_Graphics_Window_Ptr;
	INTEGER_32		current_Bit_Height;
	Rect				current_Rect;
	CARDINAL_32	buffer_Rect_Height;
	INTEGER_32		extra_Offset_X;
	INTEGER_32		extra_Offset_Y;
	const BitMap 	*selected_Port;
	const BitMap 	*destination_Bitmap;
	Primary_Status_Hdl		status_Globals_Hdl;
	Boolean				screen_Updated_By_Wiper;
	Safe_Boolean		something_Changed_In_View;
	Safe_Boolean		update_Events_Disabled_For_Recursve_Errors;
	CARDINAL_8		*code_Name;
/* END VAR */


	code_Name = "\pRedraw_Idle_Screen_Completely";
	port_Was_Set = SAFE_FALSE;
	device_Was_Set = SAFE_FALSE;
	update_Events_Disabled_For_Recursve_Errors = SAFE_FALSE;
	
	/*  */
	/* For background PICT */
	/*  */
	use_Direct_Screen_Access = FALSE;

	/*  */
	/*  */
	/*  */
	/*  */
	/*  */
	/*  */
	/*  */
	/* HACK HACK HACK: If Prompt window large, clear to black on forced */
	/* scene redraws not from updates */
	/*  */
	
	/*  */
	/* This is dependant on the validity of "g_Shell.interface.current_Graphics_Window" */
	/*  */
	if ((active_Window_List[TEST_DREAMING_WINDOW_INDEX] == nil)
		|| ((**active_Window_List[TEST_DREAMING_WINDOW_INDEX]).structure_Initialized != SAFE_TRUE)) {
		goto Error_Handler;
	}
	else {
		current_Graphics_Window_Ptr	= (**active_Window_List[TEST_DREAMING_WINDOW_INDEX]).status_Window;
	}
	
	
	if ((**(active_Window_List[TEST_DREAMING_WINDOW_INDEX])).update_Events_Permissible == SAFE_TRUE) {
		update_Events_Disabled_For_Recursve_Errors = SAFE_TRUE;
		(**(active_Window_List[TEST_DREAMING_WINDOW_INDEX])).update_Events_Permissible = SAFE_FALSE;
	}
	else {
		update_Events_Disabled_For_Recursve_Errors = SAFE_FALSE;
	}
	
	

	GetPort(&saved_Original_Port);
	Set_View_Port(current_Graphics_Window_Ptr);
	port_Was_Set = SAFE_TRUE;

	GetPenState (&saved_Pen_State);
	/*  */
	/* Actually needed for CopyBits calls , so be certain. */
	/*  */
	ForeColor(blackColor);
	BackColor(whiteColor);
	
#if !defined(k_ONLY_ALLOW_DIRECT_SCREEN_ANIMATION)
#error "UNDEFINED PARAMETER!"
#endif
#if  (k_ONLY_ALLOW_DIRECT_SCREEN_ANIMATION == 0)

	/*  */
	/* This is not really used unless scene contains 3D or New style */
	/* artwork but update it anyway the first time possible. */
	/*  */

	if (dream.stage_View.stage_View_Buffers_Initialized != SAFE_TRUE) {
		if (Reset_Stage_View_Manager_If_Needed() != SAFE_TRUE) {
			goto Error_Handler;
		}
	}
#endif

	
	if (actual_System_Update != TRUE) {
		status_Globals_Hdl = active_Window_List[GENERAL_INFO_WINDOW_INDEX];
		
		if ((status_Globals_Hdl != nil)
			&& ((**status_Globals_Hdl).structure_Initialized == SAFE_TRUE)) {
			current_Rect = ((WindowPeek)(**status_Globals_Hdl).status_Window)->port.portRect; /* WAS BOUNDS */
			current_Bit_Height = (current_Rect).bottom - (current_Rect).top;
			if (current_Bit_Height > 100) {
			
				if (g_Clear_Screen_Index != g_Active_Pict_Index) {
					if (Select_Slide(g_Clear_Screen_Index) != SAFE_TRUE) {
					
					}
					
					SelectWindow(current_Graphics_Window_Ptr);
					Set_View_Port(current_Graphics_Window_Ptr);
				}
			}
		}
	}

#if  (k_ONLY_ALLOW_DIRECT_SCREEN_ANIMATION == 0)
		Set_View_Port((GrafPtr)&(dream.stage_View.mix_View_Area_Port_Ptr[0]->offscreen_Port.color_Port));
		
		/*  */
		/* HACK HACK HACK */
		/* Might be a good idea to do something serious about */
		/* this unexpected size deviation here! */
		/*  */
#endif

	/*  */
	/*  */
	/*  */
	/*  */
	/*  */
	/*  */
	/*  */
	/*  */
	/*  */
	


	if (g_Dreaming_Module_Globals_Initialized != SAFE_TRUE) {
		function_Result = SAFE_TRUE;
		goto Normal_Function_Exit;
	}	

	if (dream.play.total_Scenes < 1) {
	
#if  (k_ONLY_ALLOW_DIRECT_SCREEN_ANIMATION == 1)
		SetRect(&destination_Rect,0,0,2000,2000);
#else
		destination_Rect = dream.stage_View.stage_View_Buffers_Total_Rect;
#endif
		
		BackColor(blackColor);
		FillRect(&destination_Rect, &(qd.black));
		BackColor(whiteColor);
		
#if  (k_ONLY_ALLOW_DIRECT_SCREEN_ANIMATION == 0)

		destination_Rect = current_Graphics_Window_Ptr->portRect;
		OffsetRect(&destination_Rect
						, -destination_Rect.left
						, -destination_Rect.top);
		
		source_Rect = destination_Rect;
		OffsetRect(&source_Rect
						, dream.stage_View.stage_Active_View_Rect.left
						, dream.stage_View.stage_Active_View_Rect.top);
		
		CopyBits ((const BitMap *)*(dream.stage_View.mix_View_Area_Port_Ptr[0]->offscreen_Port.color_Port.portPixMap)
					, &(current_Graphics_Window_Ptr->portBits)
					, &source_Rect
					, &destination_Rect
					, srcCopy
					, nil);
#endif

		function_Result = SAFE_TRUE;
		goto Normal_Function_Exit;
	}
	
	scene_To_Use = dream.current_Scene;
	
	if (dream.play.scene_Memory_In_Use[scene_To_Use] != MEMORY_INITIALIZED) {
		goto Error_Handler;
	}
	
	if ((**(dream.play.scene[scene_To_Use])).initialized != SAFE_TRUE) {
		goto Error_Handler;
	}
	
	/*  */
	/* ReCopy the background bits if they are present */
	/*  */
	background_Can_Be_Read = FALSE;
	if ((**(dream.play.scene[scene_To_Use])).backdrop_Index_Valid == SAFE_TRUE) {
		back_Index = (**(dream.play.scene[scene_To_Use])).backdrop_Index;
		if (dream.backdrops.backdrop_Memory_In_Use[back_Index] == MEMORY_INITIALIZED) {
			background_Can_Be_Read = TRUE;
		}
	}

#if  (k_ONLY_ALLOW_DIRECT_SCREEN_ANIMATION == 1)

	if ((**(dream.play.scene[scene_To_Use])).scene_Port.initialized == SAFE_TRUE) {
		saved_Device = GetGDevice();
		Set_View_Port((GrafPtr)&((**(dream.play.scene[scene_To_Use])).scene_Port.offscreen_Port.color_Port));
		Set_View_Device((**(dream.play.scene[scene_To_Use])).scene_Port.offscreen_GDevice_Hdl);
		device_Was_Set = SAFE_TRUE;
	
		source_Rect = (**(dream.play.scene[scene_To_Use])).scene_Background_Rect;
		selected_Port = (const BitMap *)*((**(dream.play.scene[scene_To_Use])).scene_Port.offscreen_Port.color_Port.portPixMap);
	}
	else {
		background_Can_Be_Read = FALSE;
		selected_Port = nil;
	}
	destination_Rect = source_Rect;
	
#else
		saved_Device = GetGDevice();
		Set_View_Device(dream.stage_View.mix_View_Area_Port_Ptr[0]->offscreen_GDevice_Hdl);
		device_Was_Set = SAFE_TRUE;
		
		selected_Port = (const BitMap *)*(dream.stage_View.mix_View_Area_Port_Ptr[0]->offscreen_Port.color_Port.portPixMap);
		
		source_Rect = dream.stage_View.stage_Active_View_Rect;
		destination_Rect = source_Rect;
		OffsetRect(&source_Rect, -source_Rect.left, -source_Rect.top);
		
#endif

	
	clipping_Region = nil;
	
	if (background_Can_Be_Read == TRUE) {
	
		if ((**(dream.backdrops.backdrop[back_Index])).backdrop_Port.initialized == SAFE_TRUE){
		
			/*  */
			/* Fill screen with deafult pict from memory */
			/*  */
			switch ((**(dream.backdrops.backdrop[back_Index])).backdrop_Port.color_Type) {
				case k_COLOR_QUICKDRAW:
				case k_32_BIT_COLOR_QUICKDRAW:
				
					if ((use_Direct_Screen_Access == TRUE) && (clipping_Region == nil)){
					
						Draw_Copied_Bits((const BitMap *)*((**(dream.backdrops.backdrop[back_Index])).backdrop_Port.offscreen_Port.color_Port.portPixMap)
									, (const BitMap *)selected_Port
									, &source_Rect
									, &destination_Rect
									, srcCopy
									, nil);
					}
					else {
						CopyBits ((const BitMap *)*((**(dream.backdrops.backdrop[back_Index])).backdrop_Port.offscreen_Port.color_Port.portPixMap)
									, (const BitMap *)selected_Port
									, &source_Rect
									, &destination_Rect
									, srcCopy
									, clipping_Region);
					}
					
				break;
				
				case k_ORIGINAL_QUICKDRAW:
				default:
				
#if  (k_ONLY_ALLOW_DIRECT_SCREEN_ANIMATION == 1)
					CopyBits (&((**(dream.backdrops.backdrop[back_Index])).backdrop_Port.offscreen_Port.old_Style_Port.portBits)
									, &((**(dream.play.scene[scene_To_Use])).scene_Port.offscreen_Port.old_Style_Port.portBits)
									, &source_Rect
									, &destination_Rect
									, srcCopy
									, clipping_Region);
#else

					CopyBits (&((**(dream.backdrops.backdrop[back_Index])).backdrop_Port.offscreen_Port.old_Style_Port.portBits)
									, &(dream.stage_View.mix_View_Area_Port_Ptr[0]->offscreen_Port.old_Style_Port.portBits)
									, &source_Rect
									, &destination_Rect
									, srcCopy
									, clipping_Region);
#endif
														
			} /*  switch (g_Indicator.color_Type) */
			
		 }
		 else {
			/*  */
			/* Just erase to black */
			/*  */
			BackColor(blackColor);
			FillRect(&destination_Rect, &(qd.black));
			BackColor(whiteColor);
		 }
	}
	else {
		/*  */
		/* Ignore offscreen port for now. HACK  */
		/*  */
	}
	
	
	 if (selected_Port != nil) {
	 	audio_Index = dream.play.transition_Effect;
	 	
		/*  */
		/* Even if there is or isn't a wipe, check if default introduction wipe is on */
		/*  */
		if (actual_System_Update != TRUE) {
		 	if (dream.play.transition_Effect_For_Introductions != EFFECT_NOTHING) {
		 		if ((**(dream.play.scene[scene_To_Use])).scene_Updated_To_Screen_Display_Once != SAFE_TRUE) {
		 			audio_Index = dream.play.transition_Effect_For_Introductions;
		 			(**(dream.play.scene[scene_To_Use])).scene_Updated_To_Screen_Display_Once = SAFE_TRUE;
		 		}
		 	}
	 	}
	 	else {
	 		audio_Index = EFFECT_NOTHING;
	 	}
	 }
	

	 if (selected_Port != nil) {
	 
#if  (k_ONLY_ALLOW_DIRECT_SCREEN_ANIMATION == 1)
		extra_Offset_X = 0;
		extra_Offset_Y = 0;
		
	/* ===
		destination_Bitmap = &(current_Graphics_Window_Ptr->portBits);
	=== */
	/* ===
		destination_Bitmap = *((**scene_To_Revise).scene_Port.offscreen_Port.color_Port.portPixMap);
	=== */
	
		Set_View_Port((GrafPtr)&((**(dream.play.scene[scene_To_Use])).scene_Port.offscreen_Port.color_Port));
		
		if (Fill_Selected_Port_With_All_Scene_Items((**scene_To_Revise).scene_Port
																				, nil
																				, SAFE_FALSE
																				, scene_To_Use
																				, show_Closed_Mouths
																				, audio_Index
																				, extra_Offset_X
																				, extra_Offset_Y) != SAFE_TRUE) {
			/*  */
			/* Ignore */
			/*  */
		}

#else
		extra_Offset_X = dream.stage_View.stage_Active_View_Rect.left;
		extra_Offset_Y = dream.stage_View.stage_Active_View_Rect.top;
																						
		/*  */
		/* Copy to first buffer. Mark first buffer as the active buffer. */
		/* Zero out its activity list. Mark its phase to start point of composition. */
		/* nullify both mix buffers because they will be both set to same soon. */
		/*  */
		dream.stage_View.next_Buffer_To_Use = 0;
		dream.stage_View.display_Phase = k_DISPLAY_PHASE_COMPOSITING_ACTIVE;
		dream.stage_View.display_Minor_Phase = 0;
		dream.stage_View.double_Buffered_Data[0].total_Remaining_Erasable_Items = 0;
		dream.stage_View.double_Buffered_Data[1].total_Remaining_Erasable_Items = 0;

		Set_View_Port((GrafPtr)&(dream.stage_View.mix_View_Area_Port_Ptr[1]->offscreen_Port.color_Port));
		CopyBits ((const BitMap *)*(dream.stage_View.mix_View_Area_Port_Ptr[0]->offscreen_Port.color_Port.portPixMap)
					, (const BitMap *)*(dream.stage_View.mix_View_Area_Port_Ptr[1]->offscreen_Port.color_Port.portPixMap)
					, &dream.stage_View.stage_Active_View_Rect
					, &dream.stage_View.stage_Active_View_Rect
					, srcCopy
					, nil);
					
		if ((**(dream.play.scene[scene_To_Use])).scene_Might_Contain_Movie_Ports == SAFE_TRUE) {
			if (Draw_Current_Stage_Background_Layers(&dream.stage_View
																						, (1 - k_BASE_OFFSET_FOR_MIX)
																						, &something_Changed_In_View) != SAFE_TRUE) {
				/*  */
				/* Ignore errors in this implementation */
				/*  */
			}
		}
		
		/*  */
		/* HACK HACK HACK warning, the buffer being used is forced as "1" */
		/* but really should be set to be equal to the default buffer in use. */
		/* "dream.stage_View.next_Buffer_To_Use should be "0" not "1" */
		/* and it is so thats OK */
		/*  */

		if (Fill_Selected_Port_With_All_Scene_Items(dream.stage_View.mix_View_Area_Port_Ptr[1]
																				, nil
																				, SAFE_FALSE
																				, scene_To_Use
																				, show_Closed_Mouths
																				, audio_Index
																				, extra_Offset_X
																				, extra_Offset_Y) != SAFE_TRUE) {
			/*  */
			/* Ignore */
			/*  */
		}
#endif

	} /*  if (selected_Port != nil)  */
	
	/*  */
	/* ReCopy the background bits if they are present */
	/*  */

	if (device_Was_Set == SAFE_TRUE) {
		Set_View_Device(saved_Device);
		device_Was_Set = SAFE_FALSE;
	}
	
	Set_View_Port(current_Graphics_Window_Ptr);
	
	source_Rect = (**(dream.play.scene[scene_To_Use])).scene_Background_Rect;
	destination_Rect = source_Rect;
	clipping_Region = nil;
	
	 
	 if ((**(dream.play.scene[scene_To_Use])).scene_Port.initialized == SAFE_TRUE) {
	 
		screen_Updated_By_Wiper = FALSE;
			 	
	 	if ((audio_Index != EFFECT_NOTHING) && (audio_Index <= k_MAXIMUM_WIPE_EFFECTS)) {
	 	
#if  (k_ONLY_ALLOW_DIRECT_SCREEN_ANIMATION == 1)		
		 	Show_Wipe_Screen_Effect(audio_Index
					 	, (GrafPtr)&(**(dream.play.scene[scene_To_Use])).scene_Port.offscreen_Port
					 	, (GrafPtr)current_Graphics_Window_Ptr);
#else

		/*  */
		/* Video effects do not honor offsets yet. They do not honor */
		/* sizes either but soon they will */
		/* That is why and extra CopyBits is wasted here */
		/*  */
		current_Rect = dream.stage_View.stage_Active_View_Rect;
		OffsetRect(&current_Rect, -current_Rect.left, -current_Rect.top);
		
		CopyBits ((const BitMap *)*(dream.stage_View.mix_View_Area_Port_Ptr[1]->offscreen_Port.color_Port.portPixMap)
					, (const BitMap *)*(dream.stage_View.mix_View_Area_Port_Ptr[2]->offscreen_Port.color_Port.portPixMap)
					, &dream.stage_View.stage_Active_View_Rect
					, &current_Rect
					, srcCopy
					, nil);

		Set_View_Port((GrafPtr)&(dream.stage_View.mix_View_Area_Port_Ptr[2]->offscreen_Port.color_Port));

		Show_Wipe_Screen_Effect(audio_Index
					 	, (GrafPtr)&(dream.stage_View.mix_View_Area_Port_Ptr[2]->offscreen_Port.color_Port)
					 	, (GrafPtr)current_Graphics_Window_Ptr);
					 	
		Set_View_Port(current_Graphics_Window_Ptr);

#endif 						 	
			screen_Updated_By_Wiper = TRUE;
	 	}

		if (screen_Updated_By_Wiper != TRUE) {
				
#if  (k_ONLY_ALLOW_DIRECT_SCREEN_ANIMATION == 1)	
			selected_Port = (const BitMap *)*((**(dream.play.scene[scene_To_Use])).scene_Port.offscreen_Port.color_Port.portPixMap);
			color_Type_Of_Port = (**(dream.play.scene[scene_To_Use])).scene_Port.color_Type;
#else	
			selected_Port = (const BitMap *)*(dream.stage_View.mix_View_Area_Port_Ptr[1]->offscreen_Port.color_Port.portPixMap);
			color_Type_Of_Port = dream.stage_View.mix_View_Area_Port_Ptr[1]->color_Type;
			source_Rect = dream.stage_View.stage_Active_View_Rect;
			destination_Rect = source_Rect;
			OffsetRect(&destination_Rect, -source_Rect.left, -source_Rect.top);
#endif
		
			/*  */
			/* Fill screen with deafult pict from memory */
			/*  */
			switch (color_Type_Of_Port) {
				case k_COLOR_QUICKDRAW:
				case k_32_BIT_COLOR_QUICKDRAW:
				
					if ((use_Direct_Screen_Access == TRUE) && (clipping_Region == nil)){
						Draw_Copied_Bits(selected_Port
									, &(current_Graphics_Window_Ptr->portBits)
									, &source_Rect
									, &destination_Rect
									, srcCopy
									, nil);
					}
					else {
						CopyBits (selected_Port
									, &(current_Graphics_Window_Ptr->portBits)
									, &source_Rect
									, &destination_Rect
									, srcCopy
									, clipping_Region);
					}
				break;
				
				case k_ORIGINAL_QUICKDRAW:
				default:
				
#if  (k_ONLY_ALLOW_DIRECT_SCREEN_ANIMATION == 1)	
					CopyBits (&((**(dream.play.scene[scene_To_Use])).scene_Port.offscreen_Port.old_Style_Port.portBits)
									, &(current_Graphics_Window_Ptr->portBits)
									, &source_Rect
									, &destination_Rect
									, srcCopy
									, clipping_Region);
#else	
					CopyBits (selected_Port
									, &(current_Graphics_Window_Ptr->portBits)
									, &source_Rect
									, &destination_Rect
									, srcCopy
									, clipping_Region);
#endif

														
			} /*  switch (g_Indicator.color_Type) */
		} /* if (screen_Updated_By_Wiper != TRUE) */		
	 }
	 else {
		/*  */
		/* Just erase to black */
		/*  */
		BackColor(blackColor);
		FillRect(&destination_Rect, &(qd.black));
		BackColor(whiteColor);
	 }	

	/*  */
	/* If needed, now copy CLEAN buffer to secondary update buffer. */
	/* We need to copy two buffers because animation system uses two */
	/*  */
	
#if  (k_ONLY_ALLOW_DIRECT_SCREEN_ANIMATION == 0)		
					
		dream.stage_View.display_Phase = k_DISPLAY_PHASE_IDLE_START_ERASE;
		dream.stage_View.display_Minor_Phase = 0;
		
		if (k_BUFFERS_REQUIRED > 2) {
			Set_View_Port((GrafPtr)&(dream.stage_View.mix_View_Area_Port_Ptr[2]->offscreen_Port.color_Port));
			CopyBits ((const BitMap *)*(dream.stage_View.mix_View_Area_Port_Ptr[0]->offscreen_Port.color_Port.portPixMap)
						, (const BitMap *)*(dream.stage_View.mix_View_Area_Port_Ptr[2]->offscreen_Port.color_Port.portPixMap)
						, &dream.stage_View.stage_Active_View_Rect
						, &dream.stage_View.stage_Active_View_Rect
						, srcCopy
						, nil);
						
			dream.stage_View.next_Buffer_To_Use = 1;
		}
#endif

	
	function_Result = SAFE_TRUE;
	goto Normal_Function_Exit;
	
Error_Handler:
	function_Result = SAFE_FALSE;
		
Normal_Function_Exit:

	if (device_Was_Set == SAFE_TRUE) {
		Set_View_Device(saved_Device);
		device_Was_Set = SAFE_FALSE;
	}

	if (port_Was_Set == SAFE_TRUE) {
		SetPenState (&saved_Pen_State);
		Set_View_Port(saved_Original_Port);
	}
	
	if (update_Events_Disabled_For_Recursve_Errors == SAFE_TRUE) {
		(**(active_Window_List[TEST_DREAMING_WINDOW_INDEX])).update_Events_Permissible = SAFE_TRUE;
	}

	return(function_Result);
	
} /* Safe_Boolean	Redraw_Idle_Screen_Completely */
/*=**************************************************************************=*/


/*=**************************************************************************=*/
Safe_Boolean	Update_Controller_Settings_For_Item(Stage_Item_Ptr	item_Ptr
																				, CARDINAL_32	*horizontal_12_Bit_Percentage
																				, CARDINAL_32	*vertical_12_Bit_Percentage
																				, CARDINAL_32	*roll_12_Bit_Percentage)
{
	CARDINAL_32	range;
	WindowPtr			current_Graphics_Window_Ptr;
	GrafPtr				saved_Original_Port;
	Rect						global_Rect;
	Point					local_point;
	Point					global_Point;
	Safe_Boolean		port_Was_Set;
	Safe_Boolean		function_Result;
/* END VAR */


	port_Was_Set = SAFE_FALSE;
	
	/*  */
	/* item_Ptr should indicate what controller to get values from. */
	/* It should indicate the type of controller as well as which controller. */
	/*  */
	
	/*  */
	/* If Caps Lock down...  */
	/*  */

	/*  */
	/* use window as relative control limit */
	/*  */
	
	if (active_Window_List[TEST_DREAMING_WINDOW_INDEX] != nil) {
	
		if ((**active_Window_List[TEST_DREAMING_WINDOW_INDEX]).structure_Initialized == SAFE_TRUE) {
			
			current_Graphics_Window_Ptr = (**active_Window_List[TEST_DREAMING_WINDOW_INDEX]).status_Window;
		
			GetPort(&saved_Original_Port);
			Set_View_Port(current_Graphics_Window_Ptr);
			port_Was_Set = SAFE_TRUE;
			
			GetMouse(&local_point);
			
			global_Rect = (**((WindowPeek)(**active_Window_List[TEST_DREAMING_WINDOW_INDEX]).status_Window)->contRgn).rgnBBox;
			
			global_Point = local_point;
			LocalToGlobal(&global_Point);
		
			if (PtInRect(global_Point, &global_Rect) != TRUE) {
				/*  */
				/* Mouse is out of bounds. */
				/* When mouse is out of bounds use the pinned value, or one day */
				/* add code to programmatically travel to the new position indicated and leave */
				/* controller settings as they are until legal control again? */
				/*  */
				
				/*  */
				/* Examine horizontal control first */
				/*  */
				
				if (global_Point.h < global_Rect.left) {
					*horizontal_12_Bit_Percentage = 0;
				}
				else if (global_Point.h >= global_Rect.right) {
					*horizontal_12_Bit_Percentage = k_MAXIMUM_FOR_12_BIT_RANGE;
				}
				else {
					range = (global_Rect.right - global_Rect.left);
					if (range > 1) {
						*horizontal_12_Bit_Percentage = (((CARDINAL_32)local_point.h * k_MAXIMUM_FOR_12_BIT_RANGE) 
																					/ (CARDINAL_32)(range - 1UL));
					}
					else {
						*horizontal_12_Bit_Percentage = 0;
					}
				}

				/*  */
				/* Examine vertical control */
				/*  */
				if (global_Point.v < global_Rect.top) {
					*vertical_12_Bit_Percentage = 0;
				}
				else if (global_Point.v >= global_Rect.bottom) {
					*vertical_12_Bit_Percentage = k_MAXIMUM_FOR_12_BIT_RANGE;
				}
				else {
					range = (global_Rect.bottom - global_Rect.top);
					if (range > 1) {
						*vertical_12_Bit_Percentage = (((CARDINAL_32)local_point.v * k_MAXIMUM_FOR_12_BIT_RANGE) 
																					/ (CARDINAL_32)(range - 1UL));
					}
					else {
						*vertical_12_Bit_Percentage = 0;
					}
				}
		
			}
			else {
			
				/*  */
				/* Examine horizontal control first */
				/*  */
				range = (global_Rect.right - global_Rect.left);
				if (range > 1) {
					*horizontal_12_Bit_Percentage = (((CARDINAL_32)local_point.h * k_MAXIMUM_FOR_12_BIT_RANGE) 
																				/ (CARDINAL_32)(range - 1UL));
				}
				else {
					*horizontal_12_Bit_Percentage = 0;
				}
				
				/*  */
				/* Examine vertical control */
				/*  */
				range = (global_Rect.bottom - global_Rect.top);
				if (range > 1) {
					*vertical_12_Bit_Percentage = (((CARDINAL_32)local_point.v * k_MAXIMUM_FOR_12_BIT_RANGE) 
																				/ (CARDINAL_32)(range - 1UL));
				}
				else {
					*vertical_12_Bit_Percentage = 0;
				}

			} /* if (PtInRect(global_Point, &global_Rect) != TRUE)  */
			
			
			/*  */
			/* Set Roll control to default */
			/*  */
			*roll_12_Bit_Percentage = 0;
			
		} /* if ((**active_Window_List[TEST_DREAMING_WINDOW_INDEX]).structure_Initialized == SAFE_TRUE) */
		
	}
	else {
		*horizontal_12_Bit_Percentage = 0;
		*vertical_12_Bit_Percentage = 0;
		*roll_12_Bit_Percentage = 0;
	}

	
	function_Result = SAFE_TRUE;
	goto Normal_Function_Exit;
	
Error_Handler:
	function_Result = SAFE_FALSE;
		
Normal_Function_Exit:

	if (port_Was_Set == SAFE_TRUE) {
		Set_View_Port(saved_Original_Port);
	}
	return(function_Result);
	
} /* Safe_Boolean	Update_Controller_Settings_For_Item( ... */
/*=**************************************************************************=*/


/*=**************************************************************************=*/
Safe_Boolean	Check_If_Item_Position_Moved(Stage_Item_Ptr	item_Ptr
												, Boolean *frame_Changed)
	/*  */
	/* Looks at controller settings to see if puppet has moved or not */
	/* *frame_Changed is reset so do not call unless already false, */
	/* or if you do, then remember that it returns valid change info. */
	/* I think this should be called for every animation frame to see if puppet has moved. */
	/*  */
{
	CARDINAL_32		horizontal_12_Bit_Percentage;
	CARDINAL_32		vertical_12_Bit_Percentage;
	CARDINAL_32		roll_12_Bit_Percentage;
	CARDINAL_32		new_Value;
	
	Puppet_Ptr				current_Puppet;
	INTEGER_32			range;
	Safe_Boolean			function_Result;
	Boolean					use_Keyboard_Control;
	Spin_Object_Ptr		spin_Art_Ptr;
	
	Puppet_Orientation_Ptr			orientation;
	Puppet_Raw_Orientation_Ptr	previous_Raw_Orientation;
/* END VAR */

	
	function_Result = SAFE_FALSE;
	*frame_Changed = FALSE;
	
	current_Puppet = item_Ptr->puppet;
	
	orientation = &item_Ptr->current_Displayed_Frame[dream.stage_View.next_Buffer_To_Use];
	previous_Raw_Orientation  = &item_Ptr->previous_Raw_Controls[dream.stage_View.next_Buffer_To_Use];

	if (current_Puppet->redirected_Information_Ptr[current_Puppet->current_Selection].array_Index_Valid != SAFE_TRUE) {
		/*  */
		/* Just exit  */
		/*  */
		function_Result = SAFE_TRUE;
		goto Normal_Function_Exit;
	}
	
	
	spin_Art_Ptr= (*(dream.spin_Objects.spin_Object[current_Puppet->redirected_Information_Ptr[current_Puppet->current_Selection].array_Index]));

	/*  */
	/* Look at horizontal, vertical and roll positions for the puppet */
	/* by extracting values from stored values kept in global structures. */
	/* these structures are either controlled from script, network feed, or */
	/* by direct involvement with interface elements */
	/*  */
	

	/*  */
	/* First, Update the controllers, or fetch controls from cached value */
	/*  */

	if (Update_Controller_Settings_For_Item(item_Ptr
																		, &horizontal_12_Bit_Percentage
																		, &vertical_12_Bit_Percentage
																		, &roll_12_Bit_Percentage) != SAFE_TRUE){
		/*  */
		/* Just exit  */
		/*  */
		function_Result = SAFE_TRUE;
		goto Normal_Function_Exit;
	}
	
	if (horizontal_12_Bit_Percentage != previous_Raw_Orientation->horizontal_Angle) {

		range = spin_Art_Ptr->max_Horizontal_Angle + 1;
		new_Value = ((horizontal_12_Bit_Percentage * range) >> k_SHIFT_FOR_12_BIT_RANGE);
		if (new_Value != orientation->horizontal_Angle) {
			orientation->horizontal_Angle = new_Value;
			*frame_Changed = TRUE;
		}
	}

	if (vertical_12_Bit_Percentage != previous_Raw_Orientation->vertical_Angle) {

		range = spin_Art_Ptr->max_Vertical_Angle + 1;
		new_Value = ((vertical_12_Bit_Percentage * range) >> k_SHIFT_FOR_12_BIT_RANGE);
		if (new_Value != orientation->vertical_Angle) {
			orientation->vertical_Angle = new_Value;
			*frame_Changed = TRUE;
		}
	}

	if (roll_12_Bit_Percentage != previous_Raw_Orientation->roll_Angle) {

		range = spin_Art_Ptr->max_Roll_Angle + 1;
		new_Value = ((roll_12_Bit_Percentage * range) >> k_SHIFT_FOR_12_BIT_RANGE);
		if (new_Value != orientation->roll_Angle) {
			orientation->roll_Angle = new_Value;
			*frame_Changed = TRUE;
		}
	}
	
	previous_Raw_Orientation->horizontal_Angle = horizontal_12_Bit_Percentage;
	previous_Raw_Orientation->vertical_Angle = vertical_12_Bit_Percentage;
	previous_Raw_Orientation->roll_Angle = roll_12_Bit_Percentage;
	
	function_Result = SAFE_TRUE;
	goto Normal_Function_Exit;
	
Error_Handler:
	function_Result = SAFE_FALSE;
		
Normal_Function_Exit:
	return(function_Result);
	
} /* Safe_Boolean	Check_If_Item_Position_Moved(Stage_Item_Ptr	item_Ptr (
/*=**************************************************************************=*/


/*=**************************************************************************=*/
Safe_Boolean	Get_Current_Volume_Frame(Stage_Item_Ptr	item_Ptr
												, Boolean *frame_Changed
												, INTEGER_16 old_Frame
												, INTEGER_16 *new_Frame)
	/*  */
	/* Draws whole frame, not just deviance */
	/* *new_Frame is not alterred unless needed. */
	/*  */
{
	CARDINAL_32	current_Time;
	CARDINAL_32	the_256_Percentage;
	Puppet_Ptr		current_Puppet;
	INTEGER_16		current_Frame;
	INTEGER_32		range;
	Safe_Boolean		function_Result;
	Boolean			use_Keyboard_Control;
	CARDINAL_32	keyboard_Amplitude;
	CARDINAL_32	minimum_Intensity_Representing_Idle;
	CARDINAL_32	max_Intensity;
	Spin_Object_Ptr		spin_Art_Ptr;
/* END VAR */


	function_Result = SAFE_FALSE;
	*frame_Changed = FALSE;
	
	*new_Frame = old_Frame;
		
	/*  */
	/* item_Ptr is assumed to be locked down!! */
	/*  */
	
	current_Puppet = item_Ptr->puppet;
	
	if (item_Ptr->controllers_Attached == SAFE_TRUE) {
						
		/*  */
		/* We should only use keyboard amplitudes if desperate */
		/* and if in a special keyboard monitoring mode selected in the */
		/* menu */
		/*  */
		use_Keyboard_Control = FALSE;
		
		if (use_Keyboard_Control == TRUE) {
			if (Fetch_Keyboard_Amplitude(&keyboard_Amplitude) == TRUE) {
				the_256_Percentage = (keyboard_Amplitude << 8) / 9;
			}
			else {
				the_256_Percentage = 0;
			}
		}
		else {
			the_256_Percentage = 0;
			
			/*  */
			/* This is where the actual volume is input. */
			/* if the puppet to draw is the same frame as the one shown */
			/* we should skip over it */
			/*  */
			current_Time = TickCount();
			
			if ((**item_Ptr->voice).initialized == SAFE_TRUE) {

				if (item_Ptr->last_Volume_Check > (**item_Ptr->voice).tickCount_Last_Updated) {
					/*  */
					/* Sound sample has not changed yet !!!!! This is guaranteed*/
					/* Use previous value */
					/*  */
				/* ====
					item_Ptr->current_Volume = 
				====== */
				}
				else {
					item_Ptr->current_Volume = (**item_Ptr->voice).current_Interpolated_Signal;
					item_Ptr->last_Volume_Check = current_Time;
				}
				
				the_256_Percentage = item_Ptr->current_Volume;
			/* ====
				the_256_Percentage = (Random() & 0x00FF);
			 ==== */
			}
			
			item_Ptr->last_Display_Time = current_Time;


		} /* NOT if (Fetch_Keyboard_Amplitude(&keyboard_Amplitude) == TRUE) */


		/*  */
		/* Adjust gain for Labview card */
		/*  */
		if ((**item_Ptr->voice).audio_Source == NATIONAL_INSTRUMENTS_DA) {					
			if ((**item_Ptr->voice).amplification_Response_Curve != (SPEED_DIGITS_REAL)0.0) {
				Convert_Audio_To_Non_Linear_Gain(the_256_Percentage, &((**item_Ptr->voice).amplification_Response_Curve), &the_256_Percentage);
			}
			if (the_256_Percentage > 255) {
				the_256_Percentage = 255;
			}
		}

		/*  */
		/* Adjust gain for Direct Data Stream Tap of a AIFF file playing */
		/*  */
/* ===
		if ((**item_Ptr->voice).audio_Source == LOCAL_AUDIO_FILE_PLAYING) {					
			if ((**item_Ptr->voice).amplification_Response_Curve != (SPEED_DIGITS_REAL)0.0) {
				Convert_Audio_To_Non_Linear_Gain(the_256_Percentage, &((**item_Ptr->voice).amplification_Response_Curve), &the_256_Percentage);
			}
			if (the_256_Percentage > 255) {
				the_256_Percentage = 255;
			}
		}
	=== */

		
		
		if (current_Puppet->puppet_File_Is_Old_Self_Contained_Type == SAFE_TRUE) {
			range = (current_Puppet->direct_Art.talk_Stop_Frame - current_Puppet->direct_Art.talk_Start_Frame) + 1;
			
			/*  */
			/* Remember that range is number of values and is 1 based but answer is 0 based */
			/* use answer as it is or adjust it upwards if needed */
			/* For example if positions 0 to 3 are desired (range of 4) then */
			/* an 8 bit activation results in four equal sized sets of : */
			/* 000-063 = 0 */
			/* 064-127 = 1 */
			/* 127-191 = 2 */
			/* 192-255 = 3 */
			/*  */
			current_Frame = ((the_256_Percentage * range) >> 8);
			
			/*  */
			/* Just paranoia limits */
			/*  */
			if(current_Frame < current_Puppet->direct_Art.talk_Start_Frame) {
				current_Frame = current_Puppet->direct_Art.talk_Start_Frame;
			}
			else if (current_Frame > current_Puppet->direct_Art.talk_Stop_Frame) {
				current_Frame = current_Puppet->direct_Art.talk_Stop_Frame;
			}
		}
		else {
		
			if (current_Puppet->redirected_Information_Ptr[current_Puppet->current_Selection].array_Index_Valid == SAFE_TRUE) {
				spin_Art_Ptr= (*(dream.spin_Objects.spin_Object[current_Puppet->redirected_Information_Ptr[current_Puppet->current_Selection].array_Index]));
			
				minimum_Intensity_Representing_Idle = spin_Art_Ptr->minimum_Intensity_Representing_Idle;
				range = spin_Art_Ptr->max_Intensity + 1 - minimum_Intensity_Representing_Idle;
				
				current_Frame = ((the_256_Percentage * range) >> 8);
				
				/*  */
				/* HACK HACK HACK : I know I need to adjust upwards but */
				/* It would be nice if the frames below idle were used as substitute */
				/* frames if the volume is low (idle) and the mouth is rapidly moving */
				/* allowing the lips to thrust out a little while talking instead of */
				/* merely touching each other. */
				/*  */
				current_Frame += minimum_Intensity_Representing_Idle;
			}
			else {
				current_Frame = 0;
			}
		}

			
#if (7 == 11)
		if (the_256_Percentage == 0) {
			/*  */
			/* Hack!!! Never trust absolute zero measurements as valid */
			/*  */
			current_Frame = old_Frame;
		}
#endif /* 7 == 11 */
	
	
		*new_Frame = current_Frame;
	
		if (current_Frame != old_Frame) {
			*frame_Changed = TRUE;
		}
		
	} /* 	if (item_Ptr->controllers_Attached == SAFE_TRUE)  */
	else {
		*new_Frame = current_Puppet->direct_Art.talk_Start_Frame;

		if (current_Puppet->puppet_File_Is_Old_Self_Contained_Type == SAFE_TRUE) {
			*new_Frame = current_Puppet->direct_Art.talk_Start_Frame;
		}
		else {
		
			*new_Frame = 0;
			
			if (current_Puppet->redirected_Information_Ptr[current_Puppet->current_Selection].array_Index_Valid == SAFE_TRUE) {
				*new_Frame = (**(dream.spin_Objects.spin_Object[current_Puppet->redirected_Information_Ptr[current_Puppet->current_Selection].array_Index])).minimum_Intensity_Representing_Idle;
			}
		
		}
	}

	function_Result = SAFE_TRUE;
	goto Normal_Function_Exit;
	
Error_Handler:
	function_Result = SAFE_FALSE;
		
Normal_Function_Exit:
	return(function_Result);
	
} /* Safe_Boolean	Get_Current_Volume_Frame... */
/*=**************************************************************************=*/


/*=**************************************************************************=*/
Safe_Boolean	Revise_Current_Scene(CARDINAL_32	window_Index, CARDINAL_32 scene_Index)
	/*  */
	/* Draws whole frame, not just deviance */
	/*  */
{
	Scene_Handle 	scene_To_Revise;
	CARDINAL_32	current_Time;
	CARDINAL_32	total_Items_For_Debug;
	Safe_Boolean		function_Result;
	Off_Screen_Ptr	modern_Destination_Bitmap;
	const BitMap		*destination_Bitmap;
	Safe_Boolean		avoid_Modern_Bitmap	;
	Safe_Boolean		port_Was_Set;
	INTEGER_16		index;
	INTEGER_16		current_Frame;
	GrafPtr				saved_Original_Port;
	PenState				saved_Pen_State;
	Puppet_Ptr			current_Puppet;
	WindowPtr			current_Graphics_Window_Ptr;
	CARDINAL_32	located_Items;
	CARDINAL_32	total_Items;
	INTEGER_32		extra_Offset_X;
	INTEGER_32		extra_Offset_Y;
	Safe_Boolean		problem_Encounterred;
	Safe_Boolean		alteration_Detected;
	Boolean				valid;
	Boolean				frame_Changed;
	Boolean				something_Changed_In_View;
	Boolean				position_Moved;
	CARDINAL_8		*code_Name;
/* END VAR */


	code_Name = "\pRevise_Current_Scene";
	problem_Encounterred = SAFE_FALSE;
	port_Was_Set = SAFE_FALSE;
	
	/*  */
	/* This is dependant on the validity of "g_Shell.interface.current_Graphics_Window" */
	/*  */
	if ((**active_Window_List[window_Index]).structure_Initialized != SAFE_TRUE) {
		goto Error_Handler;
	}
	else {
		current_Graphics_Window_Ptr	= (**active_Window_List[window_Index]).status_Window;
	}
	
	scene_To_Revise = dream.play.scene[scene_Index];
	
	located_Items = 0;
	total_Items = (**(scene_To_Revise)).total_Items_In_Scene;
	something_Changed_In_View = FALSE;
	
	
#if  (k_ONLY_ALLOW_DIRECT_SCREEN_ANIMATION == 0)
	if (total_Items > 0) {
		/*  */
		/* Its important that we advance into the compositing mode */
		/* before continuing, "k_DISPLAY_PHASE_IDLE_START_ERASE" is not especially safe. */
		/* All stages require analization for frames remaining. */
		/* All frames for THIS frame need to be erased. */
		/*  */
		if ((**(scene_To_Revise)).scene_Must_Use_Compositing_Buffers == SAFE_TRUE) {
		
			if (Bring_View_Into_Ready_To_Composite_Phase() != SAFE_TRUE) {
				/*  */
				/* Eeeks! Problem not anticipated. Fatal? Bail out */
				/*  */
				goto Error_Handler;
			}
			
		}
	}
	
	if ((**(scene_To_Revise)).scene_Might_Contain_Movie_Ports == SAFE_TRUE) {
	
		if (Draw_Current_Stage_Background_Layers(&dream.stage_View
																		, dream.stage_View.next_Buffer_To_Use
																		, &alteration_Detected) != SAFE_TRUE) {
			/*  */
			/* Ignore errors in this implementation */
			/*  */
		}
		else {
			if (alteration_Detected == SAFE_TRUE) {
				something_Changed_In_View = TRUE;
			}
		}
		
	}
	
#endif /* #if  (k_ONLY_ALLOW_DIRECT_SCREEN_ANIMATION == 0) */
	
	
	for (index = 0; ((index < k_MAX_STAGE_ITEMS) && (located_Items < total_Items)); index++) {
		
		valid = FALSE;
		if ((**(scene_To_Revise)).item[index].puppet != nil) {
			if ((**(scene_To_Revise)).item[index].valid_Pointer == SAFE_TRUE) {
				valid = TRUE;
			}
			else {
				Display_Serious_Error_Message(unit_Name, code_Name, "\pCorrupt Pointer (4)", "\pBad program");
				(**(scene_To_Revise)).item[index].puppet = nil;
			}
			
			/*  */
			/* At this point scene_To_Revise is = (dream.play.scene[scene_Index]) */
			/*  */
			
			current_Puppet = (**(scene_To_Revise)).item[index].puppet;
		}

		if (valid == TRUE) {
			located_Items += 1;
			
			/*  */
			/* Add in future: use current amplitude for total redraws, */
			/* not just beginning frames */
			/*  */
			if ((**(scene_To_Revise)).item[index].controllers_Attached == SAFE_TRUE) {
								
				if (Get_Current_Volume_Frame(&((**(scene_To_Revise)).item[index])
												, &frame_Changed
												, (**scene_To_Revise).item[index].current_Displayed_Frame[dream.stage_View.next_Buffer_To_Use].intensity
												, &current_Frame) != SAFE_TRUE) {
					/*  */
					/* ignore */
					/*  */
				}
				else {
					(**scene_To_Revise).item[index].current_Displayed_Frame[dream.stage_View.next_Buffer_To_Use].intensity = current_Frame;
				}
			
				
				/*  */
				/* Check if controller has moved and therefore redraw is needed. */
				/* This also updates correct orientation. */
				/*  */
				if (Check_If_Item_Position_Moved(&((**(scene_To_Revise)).item[index])
																		, &position_Moved) != SAFE_TRUE) {
					/*  */
					/* ignore */
					/*  */
					position_Moved = FALSE;
				}
				
				if (position_Moved == TRUE) {
					frame_Changed = TRUE;
				}
				
				if (frame_Changed != TRUE) {
					(**(scene_To_Revise)).item[index].item_Needs_Redraw = SAFE_FALSE;
				}
				else {
					(**(scene_To_Revise)).item[index].item_Needs_Redraw = SAFE_TRUE;
					something_Changed_In_View = TRUE;
				}
				
			} /* if ((**(scene_To_Revise)).item[index].controllers_Attached == SAFE_TRUE) */
			else {
			
				if (current_Puppet->puppet_File_Is_Old_Self_Contained_Type == SAFE_TRUE) {
					current_Frame = current_Puppet->direct_Art.talk_Start_Frame;
				}
				else {
				
					current_Frame = 0;
					
					if (current_Puppet->redirected_Information_Ptr[current_Puppet->current_Selection].array_Index_Valid == SAFE_TRUE) {
						current_Frame = (**(dream.spin_Objects.spin_Object[current_Puppet->redirected_Information_Ptr[current_Puppet->current_Selection].array_Index])).minimum_Intensity_Representing_Idle;
					}
				}
				
				frame_Changed = FALSE;
				
				/*  */
				/* HACK HACK HACK : Need to accomodate puppet type change */
				/* and changes in angle rotation or other modifications besides voice */
				/*  */
				if (current_Frame != (**scene_To_Revise).item[index].current_Displayed_Frame[dream.stage_View.next_Buffer_To_Use].intensity) {
					(**scene_To_Revise).item[index].current_Displayed_Frame[dream.stage_View.next_Buffer_To_Use].intensity = current_Frame;
					frame_Changed = TRUE;
				}
									

				/*  */
				/* Check if controller has moved and therefore redraw is needed. */
				/* This also updates correct orientation. */
				/*  */
				
				if (Check_If_Item_Position_Moved(&((**(scene_To_Revise)).item[index])
																		, &position_Moved) != SAFE_TRUE) {
					/*  */
					/* ignore */
					/*  */
					position_Moved = FALSE;
				}
				
				if (position_Moved == TRUE) {
					frame_Changed = TRUE;
				}
				
			
				if (frame_Changed != TRUE) {
					(**(scene_To_Revise)).item[index].item_Needs_Redraw = SAFE_FALSE;
				}
				else {
					(**(scene_To_Revise)).item[index].item_Needs_Redraw = SAFE_TRUE;
					something_Changed_In_View = TRUE;
				}
			
			} /* NOT if ((**(scene_To_Revise)).item[index].controllers_Attached == SAFE_TRUE) */
			
		} /* if (valid == TRUE) */
		
	} /* for (index = 0; ((index < k_MAX_STAGE_ITEMS) && (located_Items < total_Items)); index++) */
		

	/*  */
	/* Regrettably, if compositing, the old data was erased and it is required to redraw */
	/* all items. Only do so if something changed though. */
	/* Two times through loop is slightly innefficient for non-compositing setups */
	/*  */
	
	
	located_Items = 0;
	
	if (something_Changed_In_View == TRUE) {

		for (index = 0; ((index < k_MAX_STAGE_ITEMS) && (located_Items < total_Items)); index++) {
			
			valid = FALSE;
			if ((**(scene_To_Revise)).item[index].puppet != nil) {
				if ((**(scene_To_Revise)).item[index].valid_Pointer == SAFE_TRUE) {
					valid = TRUE;
				}
				else {
					Display_Serious_Error_Message(unit_Name, code_Name, "\pCorrupt Pointer (4)", "\pBad program");
					(**(scene_To_Revise)).item[index].puppet = nil;
				}
				
				/*  */
				/* At this point scene_To_Revise is = (dream.play.scene[scene_Index]) */
				/*  */
				
				current_Puppet = (**(scene_To_Revise)).item[index].puppet;
			}
	
			if (valid == TRUE) {
				located_Items += 1;
															
				/*  */
				/* Regrettably, if compositing, the old data was probably erased */
				/* all items are required to redraw themselves again if ANYTHING has changed. */
				/*  */
				if (((**(scene_To_Revise)).item[index].item_Needs_Redraw == SAFE_TRUE)
					|| ((**(scene_To_Revise)).scene_Must_Use_Compositing_Buffers == SAFE_TRUE)) {
					
#if  (k_ONLY_ALLOW_DIRECT_SCREEN_ANIMATION == 1)
					extra_Offset_X = 0;
					extra_Offset_Y = 0;
					destination_Bitmap = &(current_Graphics_Window_Ptr->portBits);
					modern_Destination_Bitmap = nil;
					avoid_Modern_Bitmap	 = SAFE_TRUE;


					if (port_Was_Set != SAFE_TRUE) {
						GetPort(&saved_Original_Port);
						
						Set_View_Port(current_Graphics_Window_Ptr);
						port_Was_Set = SAFE_TRUE;
						GetPenState (&saved_Pen_State);
						/*  */
						/* Actually needed for CopyBits calls , so be certain. */
						/*  */
						ForeColor(blackColor);
						BackColor(whiteColor);
					}

#else
					if ((**(scene_To_Revise)).scene_Must_Use_Compositing_Buffers == SAFE_TRUE) {
						extra_Offset_X = dream.stage_View.stage_Active_View_Rect.left;
						extra_Offset_Y = dream.stage_View.stage_Active_View_Rect.top;
												
						destination_Bitmap = nil;	
						modern_Destination_Bitmap = dream.stage_View.mix_View_Area_Port_Ptr[(dream.stage_View.next_Buffer_To_Use + k_MIX_AREA_BUFFER_0)];
						avoid_Modern_Bitmap	 = SAFE_FALSE;

	
						if (port_Was_Set != SAFE_TRUE) {
							GetPort(&saved_Original_Port);
							
							Set_View_Port((GrafPtr)&(dream.stage_View.mix_View_Area_Port_Ptr[
																(dream.stage_View.next_Buffer_To_Use + k_MIX_AREA_BUFFER_0)]->offscreen_Port.color_Port));
							port_Was_Set = SAFE_TRUE;
							GetPenState (&saved_Pen_State);
							/*  */
							/* Actually needed for CopyBits calls , so be certain. */
							/*  */
							ForeColor(blackColor);
							BackColor(whiteColor);
						}

					}
					else {
						extra_Offset_X = 0;
						extra_Offset_Y = 0;
						destination_Bitmap = &(current_Graphics_Window_Ptr->portBits);
						modern_Destination_Bitmap = nil;
						avoid_Modern_Bitmap = SAFE_TRUE;
						
						if (port_Was_Set != SAFE_TRUE) {
							GetPort(&saved_Original_Port);
							
							Set_View_Port(current_Graphics_Window_Ptr);
							port_Was_Set = SAFE_TRUE;
							GetPenState (&saved_Pen_State);
							/*  */
							/* Actually needed for CopyBits calls , so be certain. */
							/*  */
							ForeColor(blackColor);
							BackColor(whiteColor);
						}

					}
					
#endif /* NOT #if  (k_ONLY_ALLOW_DIRECT_SCREEN_ANIMATION == 1) */

					/*  */
					/* HACK HACK HACK : Need to accomodate puppet type change */
					/* and changes in angle rotation or other modifications besides voice */
					/*  */
					
					/*  */
					/* Force scene to change, because voice just changed. */
					/* there was a slight chance t would have cached otherwise. */
					/* But this clears the memory of any past frame. */
					/* we know intensity changed for this scene. */
					/*  */
					(**scene_To_Revise).item[index].last_Image.roll_Angle = k_BOGUS_ROLL_POSITION_TO_FORCE_MISMATCH;
					(**scene_To_Revise).item[index].frame_Needing_Erasure[dream.stage_View.next_Buffer_To_Use].valid_Information = SAFE_FALSE;
										
					if (Draw_Specific_Frame_Of_Item(modern_Destination_Bitmap
																	, destination_Bitmap
																	, avoid_Modern_Bitmap
																	, scene_Index
																	, index
																	, &(**scene_To_Revise).item[index].current_Displayed_Frame[dream.stage_View.next_Buffer_To_Use]
																	, TOP_LEFT(((**scene_To_Revise).item[index].desired_Start_Location))
																	, extra_Offset_X
																	, extra_Offset_Y) != SAFE_TRUE) {
						/*  */
						/* Ignore errors in this implementation */
						/*  */
						problem_Encounterred = SAFE_TRUE;
					}
					
				} /* if ((**(scene_To_Revise)).item[index].item_Needs_Redraw == SAFE_TRUE) */
				
			} /*if (valid == TRUE) */
			
		} /* for (index = 0; ((index < k_MAX_STAGE_ITEMS) && (located_Items < total_Items)); index++) */
		
	} /* if (something_Changed_In_View == TRUE) */
	
	if ((**(scene_To_Revise)).scene_Must_Use_Compositing_Buffers == SAFE_TRUE) {
		
		if (something_Changed_In_View == TRUE) {
		
			if (Copy_Composition_Buffer_To_Screen(&dream.stage_View
																				, dream.stage_View.next_Buffer_To_Use
																				, current_Graphics_Window_Ptr
																				, 0
																				, 0) != SAFE_TRUE) {
				/*  */
				/* Ignore errors in this implementation */
				/*  */
			}
		}
	}

	function_Result = SAFE_TRUE;
	goto Normal_Function_Exit;

Error_Handler:
	Display_Serious_Error_Message(unit_Name, code_Name, "\pProblem encountered", "\p");
	function_Result = SAFE_FALSE;
		
Normal_Function_Exit:
	if (port_Was_Set == SAFE_TRUE) {
		SetPenState (&saved_Pen_State);
		Set_View_Port(saved_Original_Port);
	}
	return(function_Result);
	
} /* Safe_Boolean	Revise_Current_Scene(Scene_Handle scene_To_Revise) */
/*=**************************************************************************=*/


/*=**************************************************************************=*/
Boolean Fetch_Keyboard_Amplitude(CARDINAL_32 *keyboard_Amplitude)
{
	Boolean				function_Result;
	Boolean				a_Key_Found;
	CARDINAL_8		key_Map[16];
	CARDINAL_16	keyboard_Scan_Codes[9];
	INTEGER_16		total_Codes;
	KeyMap				*key_Map_Ptr;
	SIMPLE_LOOP	loop;
/* END VAR */
	
	
	function_Result = FALSE;
	key_Map_Ptr = (KeyMap *)(&key_Map[0]);
	GetKeys(*key_Map_Ptr);
		
	total_Codes = 9;
	keyboard_Scan_Codes[0] = k_KEYCODE_1;
	keyboard_Scan_Codes[1] = k_KEYCODE_2;
	keyboard_Scan_Codes[2] = k_KEYCODE_3;
	keyboard_Scan_Codes[3] = k_KEYCODE_4;
	keyboard_Scan_Codes[4] = k_KEYCODE_5;
	keyboard_Scan_Codes[5] = k_KEYCODE_6;
	keyboard_Scan_Codes[6] = k_KEYCODE_7;
	keyboard_Scan_Codes[7] = k_KEYCODE_8;
	keyboard_Scan_Codes[8] = k_KEYCODE_9;
	
	
	a_Key_Found = FALSE;
	
	for (loop = 0; ((loop < total_Codes) && (a_Key_Found != TRUE)); loop++) {
		if ((( key_Map[keyboard_Scan_Codes[loop]>>3] >> (keyboard_Scan_Codes[loop] & 7) ) & 1) != 0) {
			a_Key_Found = TRUE;
			*keyboard_Amplitude = loop;
		}
	}

	if (a_Key_Found == TRUE) {
		function_Result = TRUE;
	}
	else {
		function_Result = FALSE;
		*keyboard_Amplitude = 0;
	}

Normal_Function_Exit:
	return(function_Result);
	
} /* Boolean Fetch_Keyboard_Amplitude(CARDINAL_32 ... */
/*=**************************************************************************=*/


/*=**************************************************************************=*/
Safe_Boolean	Draw_Specific_Frame_Of_Item(Off_Screen_Ptr	modern_Destination_Bitmap
																		, const BitMap	*destination_Bitmap
																		, Safe_Boolean	avoid_Modern_Bitmap
																		, INTEGER_16 	scene_Index
																		, INTEGER_16 	index_Into_Scene
																		, Puppet_Orientation_Ptr puppet_Orientation
																		, Point		scene_Offset
																		, INTEGER_32		extra_Offset_X
																		, INTEGER_32		extra_Offset_Y)
	/*  */
	/* Does not draw whole frame, just deviance. Does more if redraw needed */
	/*  */
{
	Puppet_Ptr					current_Puppet;
	Scene_Handle 			scene_To_Revise;
	Rect								source_Rect;
	Rect								destination_Rect;
	Off_Screen_Port_Rec *copy_Back_Port_Ptr;
	Spin_Object_Handle	spin_Object_Handle;
	Rect								*copy_Back_Port_Rect_Ptr;
	INTEGER_16				backdrop_Location;
	Safe_Boolean				function_Result;
	Boolean						use_Direct_Screen_Access;
	Boolean						region_Needs_Adjustment_Back;
	RgnHandle					clipping_Region;
	CARDINAL_32			previous_Index;
	CARDINAL_8				*code_Name;
	CARDINAL_8				drawing_Technique;
	Redirected_Art_Ptr	redirected_Art;
	INTEGER_16 				intensity;
	OSErr							os_Error;
	Boolean						same_Probable_Frame;
	Last_Image_Rec			*last_Image_Ptr;
	CARDINAL_32			composite_Search_Key;
	CARDINAL_32			spin_Info_Address;
	INTEGER_32				offset_Offscreen_Pixel_X;
	INTEGER_32				offset_Offscreen_Pixel_Y;
	Previously_Drawn_Frame_Rec		*previous_Frame_Ptr;
	Str255							note_String;
/* END VAR */


/* ==
	use_Direct_Screen_Access = TRUE;
== */

	use_Direct_Screen_Access = FALSE;
	code_Name = "\pDraw_Specific_Frame_Of_Item";
	function_Result = SAFE_TRUE;
	
	scene_To_Revise = dream.play.scene[scene_Index];
	current_Puppet = (**scene_To_Revise).item[index_Into_Scene].puppet;
	intensity = puppet_Orientation->intensity;

	if (intensity == 0xFFFFFFFF){
		goto Bad_Puppet_Frame_Offset_Error_Handler;
	}
	
	/*  */
	/* These are important to check */
	/*  */
	if (current_Puppet->puppet_File_Is_Old_Self_Contained_Type == SAFE_TRUE) {
	
		if (current_Puppet->direct_Art.frame_Total == 0) {
			goto Error_Handler;
		}
	
		source_Rect = current_Puppet->direct_Art.frame_Locations[intensity];
		destination_Rect = source_Rect;
		OffsetRect(&destination_Rect, -(source_Rect.left), -(source_Rect.top));
			
		OffsetRect(&destination_Rect
					, scene_Offset.h
					, scene_Offset.v);
	
		clipping_Region = nil;
			
		/*  */
		/* Determine drawing method needed */
		/*  */
		
		if (g_Debugging_Options[1] == SAFE_TRUE) {
			drawing_Technique = k_DRAW_MAXIMUM_SPAN_OF_ALL_FRAMES;
		}
		else {
			if (g_Debugging_Options[0] == SAFE_TRUE) {
				drawing_Technique = k_DRAW_ONLY_SINGLE_FRAME;
			}
			else {
				if (g_Debugging_Options[2] == SAFE_TRUE) {
					drawing_Technique = k_DRAW_BOUNDING_RECT_OF_FRAME;
				}
				else {
					if (g_Debugging_Options[3] == SAFE_TRUE) {
						/*  */
						/* This is automatically selected instead of */
						/* k_DRAW_MAXIMUM_SPAN_OF_ALL_FRAMES if the scene contains */
						/* more than one item and any overlap exists between items */
						/* OR if "(**(scene_To_Revise)).scene_Must_Use_Compositing_Buffers" */
						/*  */
						drawing_Technique = k_DRAW_BUFFERRED_SINGLE_FRAMES;
					}
					else {
						drawing_Technique = k_DRAW_ENTIRE_MAXIMUM_CLIPPED_FRAME;
					}
				}
			}
		}
		
		switch (drawing_Technique) {
		
			case k_DRAW_MAXIMUM_SPAN_OF_ALL_FRAMES:
				clipping_Region = current_Puppet->direct_Art.buffered_Animation_Total_Region;
		
				if (clipping_Region != nil) {
		
					if (*(CARDINAL_32 *)&(current_Puppet->direct_Art.frame_Buffered_Animation_Region_Offset[intensity]) != *(CARDINAL_32 *)&(scene_Offset)) {
		
						/*  */
						/* It needs replacement to new cached location (in use already, first time? etc. */
						/*  */
		
						/*  */
						/* Many CPU cycles on current architecture */
						/*  */
		
						if ((**scene_To_Revise).backdrop_Index_Valid == SAFE_TRUE) {
							backdrop_Location = (**scene_To_Revise).backdrop_Index;
				
							if ((dream.backdrops.backdrop_Memory_In_Use[backdrop_Location]) == SAFE_TRUE){
								copy_Back_Port_Ptr = &((**(dream.backdrops.backdrop[backdrop_Location])).backdrop_Port);
								copy_Back_Port_Rect_Ptr = &((**(dream.backdrops.backdrop[backdrop_Location])).backdrop_Background_Rect);
							}
							else {
								/*  */
								/* Impossible screwup! Scream? No. Just ignore backdrop */
								/*  */
								copy_Back_Port_Ptr = &((**scene_To_Revise).scene_Port);
								copy_Back_Port_Rect_Ptr = &((**scene_To_Revise).scene_Background_Rect);
							}
		
						}
						else {
							copy_Back_Port_Ptr = &((**scene_To_Revise).scene_Port);
							copy_Back_Port_Rect_Ptr = &((**scene_To_Revise).scene_Background_Rect);
						}
						
						if (Repaint_Background_Portion(current_Puppet
								, intensity
								, copy_Back_Port_Ptr
								, &(scene_Offset)
								, copy_Back_Port_Rect_Ptr) != SAFE_TRUE) {
							/*  */
							/* Ignore error in this version */
							/*  */
						}
						
						/*  */
						/* Negligible time on current archetecture */
						/*  */
						CopyRgn(current_Puppet->direct_Art.frame_Regions[intensity], current_Puppet->direct_Art.frame_Buffered_Animation_Regions[intensity]);
						current_Puppet->direct_Art.frame_Buffered_Animation_Region_Offset[intensity] = scene_Offset;
						OffsetRgn(current_Puppet->direct_Art.frame_Buffered_Animation_Regions[intensity], scene_Offset.h, scene_Offset.v);
					}
					
					if (*(CARDINAL_32 *)&(current_Puppet->direct_Art.buffered_Animation_Total_Region_Offset) != *(CARDINAL_32 *)&(scene_Offset)) {
						/*  */
						/* Negligible time on current archetecture */
						/*  */
						CopyRgn(current_Puppet->direct_Art.total_Lazy_Active_Region, current_Puppet->direct_Art.buffered_Animation_Total_Region);
						current_Puppet->direct_Art.buffered_Animation_Total_Region_Offset = scene_Offset;
						OffsetRgn(current_Puppet->direct_Art.buffered_Animation_Total_Region, scene_Offset.h, scene_Offset.v);
					}
					
				} /* if (clipping_Region != nil) */
			break; /* case k_DRAW_MAXIMUM_SPAN_OF_ALL_FRAMES */
		
			case k_DRAW_BOUNDING_RECT_OF_FRAME:
			
				source_Rect = (**(current_Puppet->direct_Art.frame_Regions[intensity])).rgnBBox;
				
				destination_Rect = source_Rect;
				
				OffsetRect(&destination_Rect
					, scene_Offset.h
					, scene_Offset.v);
				
				OffsetRect(&source_Rect
						, current_Puppet->direct_Art.frame_Locations[intensity].left
						, current_Puppet->direct_Art.frame_Locations[intensity].top);
						
			break; /* case k_DRAW_BOUNDING_RECT_OF_FRAME */
		
		
			case k_DRAW_ENTIRE_MAXIMUM_CLIPPED_FRAME:
				; /* NULL */
			break; /* case k_DRAW_ENTIRE_MAXIMUM_CLIPPED_FRAME */
		
			case k_DRAW_BUFFERRED_SINGLE_FRAMES:
			case k_DRAW_ONLY_SINGLE_FRAME:
			default:
				clipping_Region = current_Puppet->direct_Art.frame_Buffered_Animation_Regions[intensity];
				
				if (clipping_Region != nil) {
				
					if (*(CARDINAL_32 *)&(current_Puppet->direct_Art.frame_Buffered_Animation_Region_Offset[intensity]) != *(CARDINAL_32 *)&(scene_Offset)) {
					
						/*  */
						/* It needs replacement to new cached location (in use already, first time? etc.) */
						/* Few CPU cycles on current archetecture */
						/*  */
						CopyRgn(current_Puppet->direct_Art.frame_Regions[intensity], current_Puppet->direct_Art.frame_Buffered_Animation_Regions[intensity]);
						current_Puppet->direct_Art.frame_Buffered_Animation_Region_Offset[intensity] = scene_Offset;
						OffsetRgn(current_Puppet->direct_Art.frame_Buffered_Animation_Regions[intensity], scene_Offset.h, scene_Offset.v);
					}
				}
		} /*switch (drawing_Technique) */
	
		
		if ((extra_Offset_X != 0) || (extra_Offset_Y != 0)){
			OffsetRect(&destination_Rect, extra_Offset_X, extra_Offset_Y);
			OffsetRgn(clipping_Region, extra_Offset_X, extra_Offset_Y);
			region_Needs_Adjustment_Back = TRUE;
		}
		else {
			region_Needs_Adjustment_Back = FALSE;
		}
		
		/*  */
		/*  */
		/*  */
		/*  */
		/*  */
		/*  */
		/*  */
		/* HACK HACK HACK */
		/*  */
	/* ===
		if (g_Preferred_Monitor_Values_Valid == SAFE_TRUE) {
			Wait_For_Next_Video_Monitor_VBL(g_Preferred_Monitor_Slots);
		}
	===== */
			
		switch (current_Puppet->direct_Art.port.color_Type) {
			case k_COLOR_QUICKDRAW:
			case k_32_BIT_COLOR_QUICKDRAW:
						
				if ((clipping_Region == nil) && (use_Direct_Screen_Access == TRUE)){
					/*  */
					/* Danger:  beware of moving pixmap memory, though Copybits may in theory as well */
					/*  */
					if ((avoid_Modern_Bitmap != SAFE_TRUE) || (destination_Bitmap == nil)){
						destination_Bitmap = (const BitMap *)*(modern_Destination_Bitmap->offscreen_Port.color_Port.portPixMap);
					}
					Draw_Copied_Bits((const BitMap *)*(current_Puppet->direct_Art.port.offscreen_Port.color_Port.portPixMap)
							, destination_Bitmap
							, &source_Rect
							, &destination_Rect
							, srcCopy
							, nil);
								
				}
				else {
					if ((avoid_Modern_Bitmap != SAFE_TRUE) || (destination_Bitmap == nil)){
						destination_Bitmap = (const BitMap *)*(modern_Destination_Bitmap->offscreen_Port.color_Port.portPixMap);
					}
					CopyBits ((const BitMap *)*(current_Puppet->direct_Art.port.offscreen_Port.color_Port.portPixMap)
								, destination_Bitmap
								, &source_Rect
								, &destination_Rect
								, srcCopy
								, clipping_Region);
				}
			break;
			
			case k_ORIGINAL_QUICKDRAW:
			default:
				if ((avoid_Modern_Bitmap != SAFE_TRUE) || (destination_Bitmap == nil)){
					destination_Bitmap = (const BitMap *)*(modern_Destination_Bitmap->offscreen_Port.color_Port.portPixMap);
				}
				CopyBits (&(current_Puppet->direct_Art.port.offscreen_Port.old_Style_Port.portBits)
								, destination_Bitmap
								, &source_Rect
								, &destination_Rect
								, srcCopy
								, clipping_Region);
													
		} /*  switch (g_Indicator.color_Type) */
		
		if (region_Needs_Adjustment_Back == TRUE) {
			OffsetRgn(clipping_Region, -extra_Offset_X, -extra_Offset_Y);
		}
		
	} /* if (current_Puppet->puppet_File_Is_Old_Self_Contained_Type == SAFE_TRUE) */
	else {
	
		drawing_Technique = k_DRAW_REDIRECTED_DISK_ART_FRAME;
			
		redirected_Art = &current_Puppet->redirected_Information_Ptr[current_Puppet->current_Selection];
		if (redirected_Art->array_Index_Valid != SAFE_TRUE) {
			function_Result = SAFE_FALSE;
			goto Normal_Function_Exit;
		}
		
		/*  */
		/* These do not draw direct to screen. Mix buffers are used. */
		/* these mix buffers are always modern type. Accept nothing else. */
		/*  */
		if ((avoid_Modern_Bitmap == SAFE_TRUE) || (modern_Destination_Bitmap == nil)){
				goto Error_Handler;
		}
					
		spin_Object_Handle = dream.spin_Objects.spin_Object[redirected_Art->array_Index];
		
		if ((*spin_Object_Handle)->initialized == SAFE_TRUE) {
			if ((*spin_Object_Handle)->spin_Object_Remaining_To_Load != SAFE_TRUE) {
			
						
	/* ===
				destination_Bitmap
				minimum_Intensity_Representing_Idle;
	=== */


				offset_Offscreen_Pixel_X = redirected_Art->offset_Point_Position.h;
				offset_Offscreen_Pixel_Y = redirected_Art->offset_Point_Position.v;
				
				offset_Offscreen_Pixel_X += scene_Offset.h;
				offset_Offscreen_Pixel_Y += scene_Offset.v;
				
				offset_Offscreen_Pixel_X += extra_Offset_X;
				offset_Offscreen_Pixel_Y += extra_Offset_Y;
				
				/*  */
				/* For safety, truncate location of art offset in horizontal direction to */
				/* be equal to pixel address divisible by 128 bits (cache line) */
				/* eventually the program will use a different routine if needed */
				/*  */
				offset_Offscreen_Pixel_X = (offset_Offscreen_Pixel_X & 0xFFFFFFF0);
								
				puppet_Orientation->vertical_Angle = puppet_Orientation->vertical_Angle;
				
				/*  */
				/* Note that the drawing actually uses vertical angles usuially divisible by 5 */
				/* but this nonsense is resolved by the search routine actually using the closest match */ 
				/*  */

				last_Image_Ptr = &((**scene_To_Revise).item[index_Into_Scene].last_Image);
				
				if (last_Image_Ptr->valid_Information == SAFE_TRUE) {
					if (last_Image_Ptr->last_Spinfile_Index != redirected_Art->array_Index) {
						last_Image_Ptr->valid_Information = SAFE_FALSE;
					}
				}

				same_Probable_Frame = FALSE;
				if (last_Image_Ptr->valid_Information != SAFE_TRUE) {
					last_Image_Ptr->last_Spinfile_Index = redirected_Art->array_Index;
					last_Image_Ptr->valid_Information = SAFE_TRUE;
					last_Image_Ptr->intensity = puppet_Orientation->intensity;
					last_Image_Ptr->horizontal_Angle = puppet_Orientation->horizontal_Angle;
					last_Image_Ptr->vertical_Angle = puppet_Orientation->vertical_Angle;
					last_Image_Ptr->roll_Angle = puppet_Orientation->roll_Angle;
					
					last_Image_Ptr->avoidance_Image_Index = k_BOGUS_INDEX_FRAME;

					last_Image_Ptr->use_Fast_Copy = SAFE_FALSE;
				}
				else {
					if (last_Image_Ptr->last_Spinfile_Index == redirected_Art->array_Index) {
						if (last_Image_Ptr->horizontal_Angle == puppet_Orientation->horizontal_Angle) {
							if (last_Image_Ptr->vertical_Angle == puppet_Orientation->vertical_Angle) {
								if (last_Image_Ptr->roll_Angle == puppet_Orientation->roll_Angle) {
									if (last_Image_Ptr->intensity == puppet_Orientation->intensity){
										same_Probable_Frame = TRUE;
									}
								}
							}
						}
					}
					
					if (same_Probable_Frame != TRUE) {
						last_Image_Ptr->last_Spinfile_Index = redirected_Art->array_Index;
						last_Image_Ptr->valid_Information = SAFE_TRUE;
						last_Image_Ptr->intensity = intensity;
						last_Image_Ptr->horizontal_Angle = puppet_Orientation->horizontal_Angle;
						last_Image_Ptr->vertical_Angle = puppet_Orientation->vertical_Angle;
						last_Image_Ptr->roll_Angle = puppet_Orientation->roll_Angle;
					}
					else {
						goto No_Draw_Needed;
					}
				} /* if (last_Image_Ptr->valid_Information != SAFE_TRUE)  */
				
				/*  */
				/* Open file if needed. Closing can happen anytime if too many */
				/* files open. */
				/*  */
				if ((*spin_Object_Handle)->file_Reference == 0) {
					os_Error = HOpen((*spin_Object_Handle)->spin_Object_Location.vRefNum
													, (*spin_Object_Handle)->spin_Object_Location.parID
													, &((*spin_Object_Handle)->spin_Object_Location.name[0])
													, fsRdPerm
													, &((*spin_Object_Handle)->file_Reference));
													
					if (os_Error != noErr) {
						Concat_Many(note_String, 3,"\pFail to open file. HOpen for file ", &((*spin_Object_Handle)->spin_Object_Location.name[0]), "\p ID = ");
						Concat_Long_To_String(note_String, os_Error, NO_LIMIT);
						goto Error_Handler_With_Message;
					}
					
				}
								
				if (modern_Destination_Bitmap->initialized != SAFE_TRUE) {
						goto Error_Handler;
				}
						
				/*  */
				/* HACK HACK HACK : Note, assumes "dream.stage_View.next_Buffer_To_Use" */
				/* is valid, and correct buffer is being drawn into. */
				/*  */
				previous_Frame_Ptr = &(**scene_To_Revise).item[index_Into_Scene].frame_Needing_Erasure[dream.stage_View.next_Buffer_To_Use];
			
				/*  */
				/* Flush info for hell of it. Other technologies guard against double drawing. */
				/*  */
				previous_Frame_Ptr->valid_Information = SAFE_FALSE;
				previous_Frame_Ptr->search_Key_2 = 0;
				
				if (Draw_Spin_Object_From_File((*spin_Object_Handle)->file_Reference
								, modern_Destination_Bitmap
								, (dream.stage_View.mix_View_Area_Port_Ptr[0])
								, (*spin_Object_Handle)->spin_Info
								, previous_Frame_Ptr
								, puppet_Orientation->intensity
								, puppet_Orientation->horizontal_Angle
								, puppet_Orientation->vertical_Angle
								, puppet_Orientation->roll_Angle
								, TRUE
								, &(last_Image_Ptr->avoidance_Image_Index)
								, offset_Offscreen_Pixel_X
								, offset_Offscreen_Pixel_Y) != SAFE_TRUE) {
								
					/*  */
					/* Silent failure */
					/*  */
					function_Result = SAFE_FALSE;
					goto Normal_Function_Exit;
				}
				else {
				
					if (previous_Frame_Ptr->valid_Information == SAFE_TRUE) {
						composite_Search_Key = previous_Frame_Ptr->search_Key_1;
						spin_Info_Address = previous_Frame_Ptr->search_Key_2;
						previous_Index = previous_Frame_Ptr->previous_Index;
					}
					else {
						/*  */
						/* No future erasure needed? Odd. I guess becaue it is a movie */
						/* it is rectangular and in general, its shape can be deleted. */
						/*  */
						drawing_Technique = k_DRAW_REDIRECTED_DISK_SQUARE_FRAME;
					}
					
				}
				
			}
			else {
				function_Result = SAFE_FALSE;
				goto Normal_Function_Exit;
			}
		}
		else {
			function_Result = SAFE_FALSE;
			goto Normal_Function_Exit;
			
		} /* if ((*spin_Object_Handle)->initialized == SAFE_TRUE) */
		
	} /* NOT if (current_Puppet->puppet_File_Is_Old_Self_Contained_Type == SAFE_TRUE) */

	if ((**(scene_To_Revise)).scene_Must_Use_Compositing_Buffers == SAFE_TRUE) {
	
		/*  */
		/* Don't bother keeping track of what to delete if it uses */
		/* the combined rect way of drawing, only bother if it is */
		/* using individual item overlap mode */
		/*  */
		if ((drawing_Technique == k_DRAW_BUFFERRED_SINGLE_FRAMES) 
			|| (drawing_Technique == k_DRAW_ONLY_SINGLE_FRAME)) {
						
			if (Add_Old_Puppet_To_Removal_List(&dream.stage_View
																				, (Ptr)current_Puppet
																				, intensity
																				, extra_Offset_X
																				, extra_Offset_Y
																				, scene_Offset) != SAFE_TRUE) {
				/*  */
				/* Error not expected, but just tumble out. */
				/*  */
				goto Error_Handler;
			}
		}
		else {
			if (drawing_Technique == k_DRAW_REDIRECTED_DISK_ART_FRAME) {

				/*  */
				/* Paranoia double check. */
				/*  */
			
				if (spin_Info_Address == 0) {
					goto Error_Handler;
				}
				
				if ((*(Spin_Pict_Info_Handle)spin_Info_Address)->valid != SAFE_TRUE) {
					goto Error_Handler;
				}

				if (Add_Spin_Item_To_Removal_List(&dream.stage_View
																					, composite_Search_Key
																					, spin_Info_Address
																					, previous_Index
																					, (*spin_Object_Handle)->file_Reference
																					, offset_Offscreen_Pixel_X
																					, offset_Offscreen_Pixel_Y) != SAFE_TRUE) {
					/*  */
					/* Error not expected, but just tumble out. */
					/*  */
					goto Error_Handler;
				}
			}
			else if (drawing_Technique == k_DRAW_REDIRECTED_DISK_SQUARE_FRAME) {
				/*  */
				/* Ignore movie layer, no removal needed. */
				/*  */
			
			
			
			}
		}

	} /* if ((**(scene_To_Revise)).scene_Must_Use_Compositing_Buffers == SAFE_TRUE) */
	
No_Draw_Needed:

	function_Result = SAFE_TRUE;
	goto Normal_Function_Exit;
	
Bad_Puppet_Frame_Offset_Error_Handler:
	Display_Serious_Error_Message(unit_Name, code_Name, "\pProblem encountered. Illegal Puppet offset Index (-1)", "\pBad programming?");
	function_Result = SAFE_FALSE;
	goto Normal_Function_Exit;
	
Error_Handler_With_Message:
	Display_Serious_Error_Message(unit_Name, code_Name, note_String, "\pBad programming?");
	function_Result = SAFE_FALSE;
	goto Normal_Function_Exit;

Error_Handler:
	Display_Serious_Error_Message(unit_Name, code_Name, "\pProblem encountered", "\p");
	function_Result = SAFE_FALSE;
		
Normal_Function_Exit:

	return(function_Result);
	
} /* Safe_Boolean	Draw_Specific_Frame_Of_Item */
/*=**************************************************************************=*/

			
/*=**************************************************************************=*/
void Increment_Bobbing_Cycles(INTEGER_32 *x_Phase
													, INTEGER_32 *y_Phase
													, INTEGER_32 *x_Offset
													, INTEGER_32 *y_Offset
													, INTEGER_32 *cycle_Phase_Rate
													, INTEGER_32 *cycle_Phase_Acceleration
													, INTEGER_32 *cycle_Acceleration_Direction)
{
INTEGER_16	x_Index;
INTEGER_16	y_Index;
INTEGER_16	x_Range;
INTEGER_16	y_Range;
INTEGER_16	deviation;
INTEGER_16	x_Phase_Range;
INTEGER_16	y_Phase_Range;
/* END VAR */


	if (dream.jiggle_Active != SAFE_TRUE) {
		*x_Offset = 0;
		*y_Offset = 0;
		goto Normal_Function_Exit;
	}

	*cycle_Phase_Acceleration = *cycle_Phase_Acceleration % 16;
	if (*cycle_Phase_Acceleration == 0) {
		*cycle_Phase_Acceleration = ((CARDINAL_32)Random() % 16);
		if  (((CARDINAL_32)Random() & 0x0001) != 0) {
			*cycle_Acceleration_Direction = -1;
		}
		else {
			*cycle_Acceleration_Direction = 1;
		}
	}
	
	if (*cycle_Phase_Acceleration != 0) {
		*cycle_Phase_Rate = *cycle_Phase_Rate % (*cycle_Phase_Acceleration);
	}

	
	if (*cycle_Phase_Rate == 0) {	
	
		x_Range = dream.horizontal_Jiggle_Maximum;
		if (x_Range > 5) {
			x_Range = 5;
			switch(x_Range) {
				
				case 1:
					x_Phase_Range = 4;
				break;
				
				case 2:
					x_Phase_Range = 8;
				break;
				
				case 3:
					x_Phase_Range = 12;
				break;
				
				case 4:
					x_Phase_Range = 16;
				break;
				
				case 5:
					x_Phase_Range = 20;
				break;
				
				case 0:
				default:
					x_Phase_Range = 0;
			}
		}
		
/* ===
0
1,0,-1,0
1, 2, 1, 0,-1,-2,-1,0
1, 2, 3, 2, 1, 0,-1,-2,-3,-2,-1,0
1, 2, 3,4,3, 2, 1, 0,-1,-2,-3,-4,-3,-2,-1,0
1, 2, 3,4,5,4,3, 2, 1, 0,-1,-2,-3,-4,-5,-4,-3,-2,-1,0
=== */

		if (x_Range > 0) {
			x_Index = *x_Phase % x_Phase_Range;
			switch(x_Phase_Range) {
			
				case 4:
					switch (x_Index) {
						case 1:
							deviation = 1;
						break;
						
						case 2:
							deviation = 0;
						break;
						
						case 3:
							deviation = -1;
						break;
									
						default:
						case 0:
							deviation = 0;
					}
				break;
				
				case 8:
					switch (x_Index) {
						case 1:
							deviation = 1;
						break;
						
						case 2:
							deviation = 2;
						break;
						
						case 3:
							deviation = 1;
						break;
						
						case 4:
							deviation = 0;
						break;
						
						case 5:
							deviation = -1;
						break;
									
						case 6:
							deviation = -2;
						break;
									
						case 7:
							deviation = -1;
						break;
									
						default:
						case 0:
							deviation = 0;
					}
				break;
				
				case 12:
					switch (x_Index) {
						case 1:
							deviation = 1;
						break;
						
						case 2:
							deviation = 2;
						break;
						
						case 3:
							deviation = 3;
						break;
						
						case 4:
							deviation = 2;
						break;
						
						case 5:
							deviation = 1;
						break;
						
						case 6:
							deviation = 0;
						break;
						
						case 7:
							deviation = -1;
						break;
									
						case 8:
							deviation = -2;
						break;
									
						case 9:
							deviation = -3;
						break;
									
						case 10:
							deviation = -2;
						break;
									
						case 11:
							deviation = -1;
						break;
																		
						default:
						case 0:
							deviation = 0;
					}
				break;
				
				case 16:
					switch (x_Index) {
						case 1:
							deviation = 1;
						break;
						
						case 2:
							deviation = 2;
						break;
						
						case 3:
							deviation = 3;
						break;
						
						case 4:
							deviation = 4;
						break;
						
						case 5:
							deviation = 3;
						break;
						
						case 6:
							deviation = 2;
						break;
						
						case 7:
							deviation = 1;
						break;
						
						case 8:
							deviation = 0;
						break;
						
						case 9:
							deviation = -1;
						break;
									
						case 10:
							deviation = -2;
						break;
									
						case 11:
							deviation = -3;
						break;
									
						case 12:
							deviation = -4;
						break;
									
						case 13:
							deviation = -3;
						break;
									
						case 14:
							deviation = -2;
						break;
									
						case 15:
							deviation = -1;
						break;
																		
						default:
						case 0:
							deviation = 0;
					}
				break;
				
				case 20:
					switch (x_Index) {
						case 1:
							deviation = 1;
						break;
						
						case 2:
							deviation = 2;
						break;
						
						case 3:
							deviation = 3;
						break;
						
						case 4:
							deviation = 4;
						break;
						
						case 5:
							deviation = 5;
						break;
						
						case 6:
							deviation = 4;
						break;
						
						case 7:
							deviation = 3;
						break;
						
						case 8:
							deviation = 2;
						break;
						
						case 9:
							deviation = 1;
						break;
						
						case 10:
							deviation = 0;
						break;
						
						case 11:
							deviation = -1;
						break;
									
						case 12:
							deviation = -2;
						break;
									
						case 13:
							deviation = -3;
						break;
									
						case 14:
							deviation = -4;
						break;
									
						case 15:
							deviation = -5;
						break;
									
						case 16:
							deviation = -4;
						break;
									
						case 17:
							deviation = -3;
						break;
									
						case 18:
							deviation = -2;
						break;
									
						case 19:
							deviation = -1;
						break;
																		
						default:
						case 0:
							deviation = 0;
					}
				break;
				
			} /* switch(x_Range) */
		}
		else {
			deviation = 0;
		}
		*x_Offset = deviation;
	
	
/* ====================================================== */
/* ====================================================== */
	
	
		y_Range = dream.vertical_Jiggle_Maximum;
		if (y_Range > 5) {
			y_Range = 5;
			switch(y_Range) {
				
				case 1:
					y_Phase_Range = 4;
				break;
				
				case 2:
					y_Phase_Range = 8;
				break;
				
				case 3:
					y_Phase_Range = 12;
				break;
				
				case 4:
					y_Phase_Range = 16;
				break;
				
				case 5:
					y_Phase_Range = 20;
				break;
				
				case 0:
				default:
					y_Phase_Range = 0;
			}
		}

	
		if (y_Range > 0) {
			y_Index = *y_Phase % y_Phase_Range;
			switch(y_Phase_Range) {
			
				case 4:
					switch (y_Index) {
						case 1:
							deviation = 1;
						break;
						
						case 2:
							deviation = 0;
						break;
						
						case 3:
							deviation = -1;
						break;
									
						default:
						case 0:
							deviation = 0;
					}
				break;
				
				case 8:
					switch (y_Index) {
						case 1:
							deviation = 1;
						break;
						
						case 2:
							deviation = 2;
						break;
						
						case 3:
							deviation = 1;
						break;
						
						case 4:
							deviation = 0;
						break;
						
						case 5:
							deviation = -1;
						break;
									
						case 6:
							deviation = -2;
						break;
									
						case 7:
							deviation = -1;
						break;
									
						default:
						case 0:
							deviation = 0;
					}
				break;
				
				case 12:
					switch (y_Index) {
						case 1:
							deviation = 1;
						break;
						
						case 2:
							deviation = 2;
						break;
						
						case 3:
							deviation = 3;
						break;
						
						case 4:
							deviation = 2;
						break;
						
						case 5:
							deviation = 1;
						break;
						
						case 6:
							deviation = 0;
						break;
						
						case 7:
							deviation = -1;
						break;
									
						case 8:
							deviation = -2;
						break;
									
						case 9:
							deviation = -3;
						break;
									
						case 10:
							deviation = -2;
						break;
									
						case 11:
							deviation = -1;
						break;
																		
						default:
						case 0:
							deviation = 0;
					}
				break;
				
				case 16:
					switch (y_Index) {
						case 1:
							deviation = 1;
						break;
						
						case 2:
							deviation = 2;
						break;
						
						case 3:
							deviation = 3;
						break;
						
						case 4:
							deviation = 4;
						break;
						
						case 5:
							deviation = 3;
						break;
						
						case 6:
							deviation = 2;
						break;
						
						case 7:
							deviation = 1;
						break;
						
						case 8:
							deviation = 0;
						break;
						
						case 9:
							deviation = -1;
						break;
									
						case 10:
							deviation = -2;
						break;
									
						case 11:
							deviation = -3;
						break;
									
						case 12:
							deviation = -4;
						break;
									
						case 13:
							deviation = -3;
						break;
									
						case 14:
							deviation = -2;
						break;
									
						case 15:
							deviation = -1;
						break;
																		
						default:
						case 0:
							deviation = 0;
					}
				break;
				
				case 20:
					switch (y_Index) {
						case 1:
							deviation = 1;
						break;
						
						case 2:
							deviation = 2;
						break;
						
						case 3:
							deviation = 3;
						break;
						
						case 4:
							deviation = 4;
						break;
						
						case 5:
							deviation = 5;
						break;
						
						case 6:
							deviation = 4;
						break;
						
						case 7:
							deviation = 3;
						break;
						
						case 8:
							deviation = 2;
						break;
						
						case 9:
							deviation = 1;
						break;
						
						case 10:
							deviation = 0;
						break;
						
						case 11:
							deviation = -1;
						break;
									
						case 12:
							deviation = -2;
						break;
									
						case 13:
							deviation = -3;
						break;
									
						case 14:
							deviation = -4;
						break;
									
						case 15:
							deviation = -5;
						break;
									
						case 16:
							deviation = -4;
						break;
									
						case 17:
							deviation = -3;
						break;
									
						case 18:
							deviation = -2;
						break;
									
						case 19:
							deviation = -1;
						break;
																		
						default:
						case 0:
							deviation = 0;
					}
				break;
				
			} /* switch(y_Range) */
		}
		else {
			deviation = 0;
		}
		*y_Offset = deviation;
		
		
	
	
		*x_Phase += 1;
		*y_Phase += 1;
	}
	*cycle_Phase_Rate += 1;
	
	*cycle_Phase_Acceleration += *cycle_Acceleration_Direction;

Normal_Function_Exit:
	; /* NULL */
}
/*=**************************************************************************=*/
	 
	 
/*=**************************************************************************=*/
Safe_Boolean Repaint_Background_Portion(Puppet_Ptr puppet
						, CARDINAL_32 current_frame_Offset
						, Off_Screen_Port_Rec *scene_Port
						, Point *position_In_Scene_Point
						, Rect *scene_Background_Rect)
{
	Safe_Boolean		port_Was_Set;
	Boolean			embedded_Background_In_Puppet;
	GrafPtr			saved_Original_Port;
	Rect				source_Rect;
	Rect				destination_Rect;
	PenState			saved_Pen_State;
	GDHandle			saved_Device;
	RgnHandle		clipping_Region_Reference;
	RgnHandle		all_Except_Puppet_Region;
	RgnHandle		puppet_Region;
	RgnHandle		puppet_Max_Region;
	Safe_Boolean		function_Result;
	INTEGER_32		source_Distance;
	INTEGER_32		destination_Distance;
	CARDINAL_8		*code_Name;
/* END VAR */


	code_Name = "\pRepaint_Background_Portion";
	port_Was_Set = SAFE_FALSE;
	function_Result = SAFE_FALSE;
		
	if (puppet->puppet_Initialized != SAFE_TRUE) {
		goto Error_Handler;
	}
	
	if (puppet->puppet_File_Is_Old_Self_Contained_Type != SAFE_TRUE) {
		function_Result = SAFE_TRUE;
		goto Normal_Function_Exit;
	}

	if (puppet->direct_Art.port.initialized != SAFE_TRUE) {
		goto Puppet_Port_Not_Ready_Error;
	}
	
	if (current_frame_Offset == 0xFFFFFFFF) {
		goto Puppet_Offset_Illegal_Error;
	}
	
	GetPort(&saved_Original_Port);
	saved_Device = GetGDevice();

	Set_View_Port((GrafPtr)&(puppet->direct_Art.port.offscreen_Port.color_Port));
	Set_View_Device((*scene_Port).offscreen_GDevice_Hdl);

	port_Was_Set = SAFE_TRUE;

	GetPenState (&saved_Pen_State);
	/*  */
	/* Actually needed for CopyBits calls , so be certain. */
	/*  */
	ForeColor(blackColor);
	BackColor(whiteColor);
	
	source_Rect = *scene_Background_Rect;
	destination_Rect= puppet->direct_Art.frame_Locations[current_frame_Offset];
	
	source_Distance = (source_Rect.right - source_Rect.left);
	destination_Distance = (destination_Rect.right - destination_Rect.left);
	
	if (destination_Distance != source_Distance) {
		destination_Rect.right = destination_Rect.left + source_Distance;
	}
	
	source_Distance = (source_Rect.bottom - source_Rect.top);
	destination_Distance = (destination_Rect.bottom - destination_Rect.top);
	
	if (destination_Distance != source_Distance) {
		destination_Rect.bottom = destination_Rect.top + source_Distance;
	}
	
	/*  */
	/* mask out the area around the puppet and paste background */
	/*  */
	
	OffsetRect(&source_Rect
				, position_In_Scene_Point->h
				, position_In_Scene_Point->v);
		
	puppet_Region = NewRgn();
	CopyRgn(puppet->direct_Art.frame_Regions[current_frame_Offset], puppet_Region);
	OffsetRgn(puppet_Region, (destination_Rect.left), (destination_Rect.top));
	
	puppet_Max_Region = NewRgn();
	CopyRgn(puppet->direct_Art.total_Lazy_Active_Region, puppet_Max_Region);
	OffsetRgn(puppet_Max_Region, (destination_Rect.left), (destination_Rect.top));
	
	embedded_Background_In_Puppet = FALSE;
	if ((**puppet_Max_Region).rgnSize == (**puppet_Region).rgnSize) {
		if ( EqualRgn(puppet_Max_Region,puppet_Region) == TRUE) {
			/*  */
			/* REGIONS IDENTICAL!! This means embedded picture. */
			/* We must not try to find differences because it results in */
			/* region obliterating our art */
			/*  */
			embedded_Background_In_Puppet = TRUE;
		}
	}
	
	if (embedded_Background_In_Puppet != TRUE) {
		if ((**puppet_Region).rgnSize == 10) {
			if (((**puppet_Region).rgnSize == (**puppet_Max_Region).rgnSize) 
					|| (((**puppet_Max_Region).rgnBBox.bottom - (**puppet_Max_Region).rgnBBox.top) > (g_Window_Limits.specific.display_Item_Height_Max - 30L))){
				embedded_Background_In_Puppet = TRUE;
			}
		}
	}
	
	/*  */
	/*	If square puppet frame, eliminate fill stuff */
	/* But ONLY if the square puppet is equal to union bounds ! */
	/* Optimize referencing soon */
	/*  */
	if (embedded_Background_In_Puppet != TRUE) {
		if ((**(puppet->direct_Art.frame_Regions[current_frame_Offset])).rgnSize == 10) {
		
			if (((**puppet_Max_Region).rgnBBox.bottom - (**puppet_Max_Region).rgnBBox.top)
				== ((**(puppet->direct_Art.frame_Regions[current_frame_Offset])).rgnBBox.bottom 
					- (**(puppet->direct_Art.frame_Regions[current_frame_Offset])).rgnBBox.top)) {
					
				if (((**puppet_Max_Region).rgnBBox.right - (**puppet_Max_Region).rgnBBox.left)
					== ((**(puppet->direct_Art.frame_Regions[current_frame_Offset])).rgnBBox.right 
						- (**(puppet->direct_Art.frame_Regions[current_frame_Offset])).rgnBBox.left)) {
						
					embedded_Background_In_Puppet = TRUE;
				}
			}
		}
		
	} /* if (embedded_Background_In_Puppet != TRUE) */

	all_Except_Puppet_Region = NewRgn();
	RectRgn(all_Except_Puppet_Region, &destination_Rect);
	
	if (embedded_Background_In_Puppet != TRUE) {
	
		DiffRgn(all_Except_Puppet_Region, puppet_Region, all_Except_Puppet_Region);
		DiffRgn(puppet_Max_Region, puppet_Region, puppet_Max_Region);
		
		/*  */
		/* When developing, for benchmark effect, you MIGHT want all the */
		/* background drawn around individual puppets but we really only should */
		/* fill the pixels around the frame that we need */
		/*  */
		if (k_ONLY_FILL_CRITICAL_PIXELS_AROUND_PUPPET == 1) {
			clipping_Region_Reference = puppet_Max_Region;
		}
		else {
			clipping_Region_Reference = all_Except_Puppet_Region;
		}
				
		switch (puppet->direct_Art.port.color_Type) {
			case k_COLOR_QUICKDRAW:
			case k_32_BIT_COLOR_QUICKDRAW:
			
				CopyBits ((const BitMap *)*((*scene_Port).offscreen_Port.color_Port.portPixMap)
								, (const BitMap *)*(puppet->direct_Art.port.offscreen_Port.color_Port.portPixMap)
								, &source_Rect
								, &destination_Rect
								, srcCopy
								, clipping_Region_Reference);
			break;
			
			case k_ORIGINAL_QUICKDRAW:
			default:
			
				CopyBits (&((*scene_Port).offscreen_Port.old_Style_Port.portBits)
								, &(puppet->direct_Art.port.offscreen_Port.old_Style_Port.portBits)
								, &source_Rect
								, &destination_Rect
								, srcCopy
								, clipping_Region_Reference);
						
		} /*  switch (g_Indicator.color_Type) */
		
	} /* if (embedded_Background_In_Puppet != TRUE) */

	DisposeRgn(all_Except_Puppet_Region);
	DisposeRgn(puppet_Region);
	DisposeRgn(puppet_Max_Region);
	
	function_Result = SAFE_TRUE;
	goto Normal_Function_Exit;
	
Puppet_Offset_Illegal_Error:
	Display_Serious_Error_Message(unit_Name, code_Name, "\pPuppet offset invalid value!", "\pBad programming");
	function_Result = SAFE_FALSE;
	goto Normal_Function_Exit;
	
Puppet_Port_Not_Ready_Error:
	Display_Serious_Error_Message(unit_Name, code_Name, "\pPuppet port not 'initialized'", "\pBad programming");
	function_Result = SAFE_FALSE;
	goto Normal_Function_Exit;

Error_Handler:
	Display_Serious_Error_Message(unit_Name, code_Name, "\pProblem encountered", "\p");
	function_Result = SAFE_FALSE;
		
Normal_Function_Exit:
	if (port_Was_Set == SAFE_TRUE) {
		SetPenState (&saved_Pen_State);
		Set_View_Port(saved_Original_Port);
		Set_View_Device(saved_Device);
	}
	return(function_Result);
}
/*=**************************************************************************=*/


/*=**************************************************************************=*/
Safe_Boolean	Benchmark_Animation_Frames(CARDINAL_32 puppet_Index
																		, CARDINAL_32 scene_Index
																		, INTEGER_16 index_Into_Scene)
{
	Safe_Boolean		function_Result;
	Safe_Boolean		port_Was_Set;
	Scene_Handle 	scene_To_Revise;
	GrafPtr				saved_Original_Port;
	PenState				saved_Pen_State;
	Puppet_Ptr			current_Puppet;
	INTEGER_32		current_Frame;
	Rect						source_Rect;
	Rect						destination_Rect;
	WindowPtr			current_Graphics_Window_Ptr;
	Off_Screen_Ptr	modern_Destination_Bitmap;
	const BitMap		*destination_Bitmap;
	Safe_Boolean		avoid_Modern_Bitmap	;
	SIMPLE_LOOP	loop;
	CARDINAL_32	end_Speed;
	RgnHandle			clipping_Region;
	SIMPLE_LOOP	total_Test_Loops;
	CARDINAL_32	temp_Hack_Real_Seed;
	CARDINAL_32	start_Time;
	Rect						result_Rect;
	Boolean				use_Direct_Screen_Access;
	Safe_Boolean		color_Table_Seeds_Spoofed;
	Str255					timing_String;
	DIGITS_SPEED_REAL		frames_Per_Second;
	Boolean				test_Movement;
	Rect						saved_Rect;
	CARDINAL_8		*code_Name;
	INTEGER_32		extra_Offset_X;
	INTEGER_32		extra_Offset_Y;
	INTEGER_32 		x_Phase;
	INTEGER_32		y_Phase;
	INTEGER_32 		x_Offset;
	INTEGER_32 		y_Offset;
	INTEGER_32 		cycle_Phase_Rate;
	INTEGER_32 		cycle_Phase_Acceleration;
	INTEGER_32 		cycle_Acceleration_Direction;
	Safe_Boolean		something_Changed_In_View;
	CARDINAL_32	animation_Total_Frames;
	Boolean				continue_Looping;
/* END VAR */


	code_Name = "\pBenchmark_Animation_Frames";
	port_Was_Set = SAFE_FALSE;
	test_Movement = TRUE;
		
	animation_Total_Frames = 0;
	
	scene_To_Revise = dream.play.scene[scene_Index];
	
	x_Phase = 0;
	y_Phase = 0;
	cycle_Phase_Rate = 0;
	cycle_Phase_Acceleration = 0;
	cycle_Acceleration_Direction = 1;
	
/* =====
	if (Initialize_Graphics_Primitives(TRUE, FALSE) != TRUE) {
		Display_Serious_Error_Message(unit_Name, code_Name, "\pInitialize_Graphics_Primitives failed.  Program will abort.", "\p");
		g_Shell.interface.abort_Program = TRUE;
		function_Result = SAFE_FALSE;
		goto Normal_Function_Exit;
	}
	
===*/
	use_Direct_Screen_Access = TRUE;

	HideCursor();
	
	if (puppet_Index > dream.active_Puppets_Total) {
		goto Error_Handler;
	}
	
	current_Puppet = *(dream.puppet[puppet_Index]);
	
	GetPort(&saved_Original_Port);
	
	/*  */
	/* This is dependant on the validity of "g_Shell.interface.current_Graphics_Window" */
	/*  */
	if ((**active_Window_List[TEST_DREAMING_WINDOW_INDEX]).structure_Initialized != SAFE_TRUE) {
		goto Error_Handler;
	}
	else {
		current_Graphics_Window_Ptr	= (**active_Window_List[TEST_DREAMING_WINDOW_INDEX]).status_Window;
	}
	
#if  (k_ONLY_ALLOW_DIRECT_SCREEN_ANIMATION == 1)
	extra_Offset_X = 0;
	extra_Offset_Y = 0;
	destination_Bitmap = &(current_Graphics_Window_Ptr->portBits);
	modern_Destination_Bitmap = nil;
	avoid_Modern_Bitmap = SAFE_TRUE;
	
	Set_View_Port(current_Graphics_Window_Ptr);
	port_Was_Set = SAFE_TRUE;
#else
	if ((**(scene_To_Revise)).scene_Must_Use_Compositing_Buffers == SAFE_TRUE) {
		extra_Offset_X = dream.stage_View.stage_Active_View_Rect.left;
		extra_Offset_Y = dream.stage_View.stage_Active_View_Rect.top;
		destination_Bitmap = nil;
		modern_Destination_Bitmap = dream.stage_View.mix_View_Area_Port_Ptr[(dream.stage_View.next_Buffer_To_Use + k_MIX_AREA_BUFFER_0)];
		avoid_Modern_Bitmap = SAFE_FALSE;

	
		Set_View_Port((GrafPtr)&(dream.stage_View.mix_View_Area_Port_Ptr[
								(dream.stage_View.next_Buffer_To_Use + k_MIX_AREA_BUFFER_0)]->offscreen_Port.color_Port));
	/* ===
		Set_View_Device();
		device_Was_Set = SAFE_TRUE;
	=== */
		port_Was_Set = SAFE_TRUE;
	}
	else {
		extra_Offset_X = 0;
		extra_Offset_Y = 0;
		destination_Bitmap = &(current_Graphics_Window_Ptr->portBits);
		modern_Destination_Bitmap = nil;
		avoid_Modern_Bitmap = SAFE_TRUE;

		Set_View_Port(current_Graphics_Window_Ptr);
		port_Was_Set = SAFE_TRUE;
	}
#endif

	

	GetPenState (&saved_Pen_State);
	/*  */
	/* Actually needed for CopyBits calls , so be certain. */
	/*  */
	ForeColor(blackColor);
	BackColor(whiteColor);

	/*  */
	/* these few are important to clear */
	/*  */
	if (current_Puppet->direct_Art.frame_Total == 0) {
		goto Error_Handler;
	}
	
	/*  */
	/* Tell quickdraw not to try matching */
	/*  */
	
#if !defined(k_PREVENT_QUICKDRAW_CLUT_MATCH_IN_BENCHMARK)
#error "UNDEFINED PARAMETER!"
#endif
#if (k_PREVENT_QUICKDRAW_CLUT_MATCH_IN_BENCHMARK == 1)
		Set_Color_Match_For_Pixel_Copy(FALSE, current_Puppet->direct_Art.port.offscreen_Port.color_Port.portPixMap);
		
		color_Table_Seeds_Spoofed = SAFE_FALSE;
		temp_Hack_Real_Seed = (**(**(current_Puppet->direct_Art.port.offscreen_Port.color_Port.portPixMap)).pmTable).ctSeed;
		if (temp_Hack_Real_Seed != (**(**(((CWindowPeek)current_Graphics_Window_Ptr)->port.portPixMap)).pmTable).ctSeed) {
			color_Table_Seeds_Spoofed = SAFE_TRUE;
			(**(**(current_Puppet->direct_Art.port.offscreen_Port.color_Port.portPixMap)).pmTable).ctSeed
				= (**(**(((CWindowPeek)current_Graphics_Window_Ptr)->port.portPixMap)).pmTable).ctSeed;
		}
#endif

		
	total_Test_Loops = (300 / current_Puppet->direct_Art.frame_Total);
	start_Time = TickCount();
	continue_Looping = TRUE;
		
	for (loop = 0; ((loop < total_Test_Loops) && (continue_Looping == TRUE)); loop++) {
			
	/*  */
	/* First, progress forward through the pictures */
	/*  */
		for (current_Frame = 0
										; (current_Frame < current_Puppet->direct_Art.frame_Total)
										; current_Frame++) {
			
			
			/*  */
			/* This is for debugging */
			/*  */
/* =====
			ForeColor(greenColor);
			FrameRgn(current_Puppet->direct_Art.frame_Regions[current_Frame]);
			ForeColor(blackColor);
===== */

/* ===
			if (clipping_Region != nil) {
				ForeColor(redColor);
				FrameRgn(clipping_Region);
				ForeColor(blackColor);
			}
			else {
				ForeColor(redColor);
				FrameRect(&destination_Rect);
				ForeColor(blackColor);
			}
=== */

			if (test_Movement == TRUE) {
				Increment_Bobbing_Cycles(&x_Phase, &y_Phase, &x_Offset, &y_Offset, &cycle_Phase_Rate, &cycle_Phase_Acceleration, &cycle_Acceleration_Direction);
				saved_Rect = (**scene_To_Revise).item[index_Into_Scene].desired_Start_Location;
				OffsetRect(&((**scene_To_Revise).item[index_Into_Scene].desired_Start_Location), x_Offset, y_Offset);
			}
			
#if  (k_ONLY_ALLOW_DIRECT_SCREEN_ANIMATION == 0)
			/*  */
			/* might need to ping pong the mix buffers! */
			/*  */
			if ((**(scene_To_Revise)).scene_Must_Use_Compositing_Buffers == SAFE_TRUE) {
				destination_Bitmap = nil;
				modern_Destination_Bitmap = dream.stage_View.mix_View_Area_Port_Ptr[(dream.stage_View.next_Buffer_To_Use + k_MIX_AREA_BUFFER_0)];
				avoid_Modern_Bitmap = SAFE_FALSE;

				Set_View_Port((GrafPtr)&(dream.stage_View.mix_View_Area_Port_Ptr[
										(dream.stage_View.next_Buffer_To_Use + k_MIX_AREA_BUFFER_0)]->offscreen_Port.color_Port));
			}
#endif

			if ((**(scene_To_Revise)).scene_Might_Contain_Movie_Ports == SAFE_TRUE) {
				if (Draw_Current_Stage_Background_Layers(&dream.stage_View
								, dream.stage_View.next_Buffer_To_Use
								, &something_Changed_In_View) != SAFE_TRUE) {
					/*  */
					/* Ignore errors in this implementation */
					/*  */
				}
			}

			(**(dream.play.scene[scene_Index])).item[index_Into_Scene].current_Displayed_Frame[dream.stage_View.next_Buffer_To_Use].intensity = current_Frame;

			if (Draw_Specific_Frame_Of_Item(modern_Destination_Bitmap
																		, destination_Bitmap
																		, avoid_Modern_Bitmap
																		, scene_Index
																		, index_Into_Scene
																		, &(**(dream.play.scene[scene_Index])).item[index_Into_Scene].current_Displayed_Frame[dream.stage_View.next_Buffer_To_Use]
																		, TOP_LEFT(((**scene_To_Revise).item[index_Into_Scene].desired_Start_Location))
																		, extra_Offset_X
																		, extra_Offset_Y) != SAFE_TRUE) {
				/*  */
				/* Ignore errors */
				/*  */
			}
			
			if ((**(scene_To_Revise)).scene_Must_Use_Compositing_Buffers == SAFE_TRUE) {
				if (Copy_Composition_Buffer_To_Screen(&dream.stage_View
																					, dream.stage_View.next_Buffer_To_Use
																					, current_Graphics_Window_Ptr
																					, 0
																					, 0) != SAFE_TRUE) {
					/*  */
					/* Ignore errors in this implementation */
					/*  */
				}
			}
			
			animation_Total_Frames += 1;
			
			if (test_Movement == TRUE) {
				(**scene_To_Revise).item[index_Into_Scene].desired_Start_Location = saved_Rect;
			}

		} /* for (current_Frame = 0; (current_Frame < */
		
		if (((CARDINAL_32)TickCount() - start_Time) > 1200) {
			continue_Looping = FALSE;
		}
		else {
			if (Button() != 0) {
				continue_Looping = FALSE;
			}
		}

	/*  */
	/* Go backward through the pictures, skipping the current one */
	/*  */
	
		if (continue_Looping == TRUE) {
		
			for (current_Frame = (current_Puppet->direct_Art.frame_Total - 2)
											; (current_Frame >= 0)
											; current_Frame--) {
				
				
				/*  */
				/* This should speed up operation quite a bit */
				/*  */
	/* ===
				ForeColor(greenColor);
				FrameRgn(current_Puppet->direct_Art.frame_Regions[current_Frame]);
				ForeColor(blackColor);
	=== */
	
	/* ====
				if (clipping_Region != nil) {
					ForeColor(redColor);
					FrameRgn(clipping_Region);
					ForeColor(blackColor);
				}
				else {
					ForeColor(redColor);
					FrameRect(&destination_Rect);
					ForeColor(blackColor);
				}
	=== */
				if (test_Movement == TRUE) {
					Increment_Bobbing_Cycles(&x_Phase, &y_Phase, &x_Offset, &y_Offset, &cycle_Phase_Rate, &cycle_Phase_Acceleration, &cycle_Acceleration_Direction);
					saved_Rect = (**scene_To_Revise).item[index_Into_Scene].desired_Start_Location;
					OffsetRect(&((**scene_To_Revise).item[index_Into_Scene].desired_Start_Location), x_Offset, y_Offset);
				}
	
#if  (k_ONLY_ALLOW_DIRECT_SCREEN_ANIMATION == 0)
				/*  */
				/* might need to ping pong the mix buffers! */
				/*  */
				if ((**(scene_To_Revise)).scene_Must_Use_Compositing_Buffers == SAFE_TRUE) {

					destination_Bitmap = nil;
					modern_Destination_Bitmap = dream.stage_View.mix_View_Area_Port_Ptr[(dream.stage_View.next_Buffer_To_Use + k_MIX_AREA_BUFFER_0)];
					avoid_Modern_Bitmap = SAFE_FALSE;

				
					Set_View_Port((GrafPtr)&(dream.stage_View.mix_View_Area_Port_Ptr[
											(dream.stage_View.next_Buffer_To_Use + k_MIX_AREA_BUFFER_0)]->offscreen_Port.color_Port));
				}
#endif
	
				if ((**(scene_To_Revise)).scene_Might_Contain_Movie_Ports == SAFE_TRUE) {
					if (Draw_Current_Stage_Background_Layers(&dream.stage_View
																								, dream.stage_View.next_Buffer_To_Use
																								, &something_Changed_In_View) != SAFE_TRUE) {
						/*  */
						/* Ignore errors in this implementation */
						/*  */
					}
				}

				(**(dream.play.scene[scene_Index])).item[index_Into_Scene].current_Displayed_Frame[dream.stage_View.next_Buffer_To_Use].intensity = current_Frame;
				
				if (Draw_Specific_Frame_Of_Item(modern_Destination_Bitmap
																			, destination_Bitmap
																			, avoid_Modern_Bitmap
																			, scene_Index
																			, index_Into_Scene
																			, &(**(dream.play.scene[scene_Index])).item[index_Into_Scene].current_Displayed_Frame[dream.stage_View.next_Buffer_To_Use]
																			, TOP_LEFT(((**scene_To_Revise).item[index_Into_Scene].desired_Start_Location))
																			, extra_Offset_X
																			, extra_Offset_Y) != SAFE_TRUE) {
					/*  */
					/* Ignore errors */
					/*  */
				}
				
				if ((**(scene_To_Revise)).scene_Must_Use_Compositing_Buffers == SAFE_TRUE) {
					if (Copy_Composition_Buffer_To_Screen(&dream.stage_View
																						, dream.stage_View.next_Buffer_To_Use
																						, current_Graphics_Window_Ptr
																						, 0
																						, 0) != SAFE_TRUE) {
						/*  */
						/* Ignore errors in this implementation */
						/*  */
					}
				}
				
				animation_Total_Frames += 1;

				if (test_Movement == TRUE) {
					(**scene_To_Revise).item[index_Into_Scene].desired_Start_Location = saved_Rect;
				}
			} /* for (current_Frame = 0; (current_Frame < */
		} /* continue_Looping == TRUE) */

				
		if (((CARDINAL_32)TickCount() - start_Time) > 1200) {
			continue_Looping = FALSE;
		}
		else {
			if (Button() != 0) {
				continue_Looping = FALSE;
			}
		}

	} /* for (loop = 0; (loop < total_Test_Loops); loop++) */
	
	end_Speed = ((CARDINAL_32)TickCount() - start_Time);
	frames_Per_Second = end_Speed / 60.0;
	frames_Per_Second = ((DIGITS_SPEED_REAL)animation_Total_Frames) / frames_Per_Second;

	
#if (k_PREVENT_QUICKDRAW_CLUT_MATCH_IN_BENCHMARK == 1)

	if (color_Table_Seeds_Spoofed == SAFE_TRUE) {
		(**(**(current_Puppet->direct_Art.port.offscreen_Port.color_Port.portPixMap)).pmTable).ctSeed
			= temp_Hack_Real_Seed;
	}

	Set_Color_Match_For_Pixel_Copy(TRUE, current_Puppet->direct_Art.port.offscreen_Port.color_Port.portPixMap);
#endif


	Convert_Double_To_String(frames_Per_Second
									, 7
									, 3
									, (ASCII_Character *)&timing_String[0]);
									
	Concat_After_Pascal_String(timing_String, "\p F/Sec");
	
	ForeColor(blackColor);
	
	SetRect(&result_Rect
			, k_BENCHMARK_RESULT_POSITION_X
			, k_BENCHMARK_RESULT_POSITION_Y - 16
			, k_BENCHMARK_RESULT_POSITION_X + 82
			, k_BENCHMARK_RESULT_POSITION_Y);
					
	EraseRect(&result_Rect);
	
	ForeColor(whiteColor);
	BackColor(blackColor);
	
	TextFont(DEFAULT_TEXT_FONT);
	TextSize(DEFAULT_TEXT_SIZE);
	TextFace(bold);
		
	MoveTo(k_BENCHMARK_RESULT_POSITION_X + 2
					,k_BENCHMARK_RESULT_POSITION_Y - 3);
					
	DrawString(timing_String);
	
	Perform_OK_Dialog(timing_String);
	
	ForeColor(blackColor);
	BackColor(whiteColor);
	
	(**(dream.play.scene[scene_Index])).item[index_Into_Scene].current_Displayed_Frame[dream.stage_View.next_Buffer_To_Use].intensity = current_Frame;

	if (Redraw_Idle_Screen_Completely(FALSE, FALSE) != SAFE_TRUE) {
		/*  */
		/* Ignore error */
		/*  */
	}
	
	function_Result = SAFE_TRUE;
	goto Normal_Function_Exit;
	
Error_Handler:
	Display_Serious_Error_Message(unit_Name, "\p", "\pProblem encountered", "\p");
	function_Result = SAFE_FALSE;
		
Normal_Function_Exit:
	ShowCursor();

	if (port_Was_Set == SAFE_TRUE) {
		SetPenState (&saved_Pen_State);
		Set_View_Port(saved_Original_Port);
	}
	return(function_Result);
	
} /* Safe_Boolean	Benchmark_Animation_Frames */
/*=**************************************************************************=*/


/*=**************************************************************************=*/
Safe_Boolean	Erase_Old_Item_From_Mix_Buffer(Off_Screen_Port_Rec *port_Ptr
																		, Puppet_Ptr current_Puppet
																		, INTEGER_16 current_Frame
																		, Point		scene_Offset
																		, INTEGER_32		extra_Offset_X
																		, INTEGER_32		extra_Offset_Y)
																		
	/*  */
	/* This only erases a region with a chunk of background. */
	/* it is needed for animations that require full deletion*/
	/* of all items explicitely to allow multiple overlapping items */
	/* in a compositing buffer */
	/*  */
{
	Safe_Boolean			function_Result;
	Rect					source_Rect;
	Rect					destination_Rect;
	Boolean					use_Direct_Screen_Access;
	Boolean					region_Needs_Adjustment_Back;
	RgnHandle				clipping_Region;
	CARDINAL_8			*code_Name;
	CARDINAL_8			drawing_Technique;
/* END VAR */


/* ==
	use_Direct_Screen_Access = TRUE;
== */
	use_Direct_Screen_Access = FALSE;
	code_Name = "\pErase_Old_Item_From_Mix_Buffer";
	function_Result = SAFE_TRUE;
	
					
	if (current_Frame == 0xFFFFFFFF){
		goto Error_Handler;
	}
	
	/*  */
	/* These are important to check */
	/*  */
	if ((current_Puppet->puppet_File_Is_Old_Self_Contained_Type != SAFE_TRUE)
		 || (current_Puppet->direct_Art.frame_Total == 0)) {
		goto Error_Handler;
	}


	source_Rect = current_Puppet->direct_Art.frame_Locations[current_Frame];
	OffsetRect(&source_Rect, -(source_Rect.left), -(source_Rect.top));
		
	OffsetRect(&source_Rect
				, scene_Offset.h
				, scene_Offset.v);
				
	/*  */
	/* Source and destination are equal in this type of backdrop undate */
	/* routine. */
	/*  */
	destination_Rect = source_Rect;

	clipping_Region = nil;
		
	if (g_Debugging_Options[3] == SAFE_TRUE) {
		/*  */
		/* This is automatically selected instead of */
		/* k_DRAW_MAXIMUM_SPAN_OF_ALL_FRAMES if the scene contains */
		/* more than one item and any overlap exists between items */
		/* OR if "(**(scene_To_Revise)).scene_Must_Use_Compositing_Buffers" */
		/*  */
	/*===
		drawing_Technique = k_DRAW_ONLY_SINGLE_FRAME;
	=== */
		drawing_Technique = k_DRAW_BOUNDING_RECT_OF_FRAME;
	}
	else {
		function_Result = SAFE_TRUE;
		goto Normal_Function_Exit;
	}

	switch (drawing_Technique) {
	
		case k_DRAW_BOUNDING_RECT_OF_FRAME:
			destination_Rect = (**(current_Puppet->direct_Art.frame_Regions[current_Frame])).rgnBBox;
			OffsetRect(&destination_Rect
					, current_Puppet->direct_Art.frame_Locations[current_Frame].left
					, current_Puppet->direct_Art.frame_Locations[current_Frame].top);
					
			OffsetRect(&destination_Rect
					, source_Rect.left
					, source_Rect.top);
					
			source_Rect = destination_Rect;
					
		break; /* case k_DRAW_BOUNDING_RECT_OF_FRAME */
	
		case k_DRAW_ONLY_SINGLE_FRAME:
		default:
			clipping_Region = current_Puppet->direct_Art.frame_Buffered_Animation_Regions[current_Frame];
			
			if (clipping_Region != nil) {
			
				if (*(CARDINAL_32 *)&(current_Puppet->direct_Art.frame_Buffered_Animation_Region_Offset[current_Frame]) != *(CARDINAL_32 *)&(scene_Offset)) {
				
					/*  */
					/* It needs replacement to new cached location (in use already, first time? etc.) */
					/* Few CPU cycles on current archetecture */
					/*  */
					CopyRgn(current_Puppet->direct_Art.frame_Regions[current_Frame], current_Puppet->direct_Art.frame_Buffered_Animation_Regions[current_Frame]);
					current_Puppet->direct_Art.frame_Buffered_Animation_Region_Offset[current_Frame] = scene_Offset;
					OffsetRgn(current_Puppet->direct_Art.frame_Buffered_Animation_Regions[current_Frame], scene_Offset.h, scene_Offset.v);
				}
			}
	} /*switch (drawing_Technique) */

	if ((extra_Offset_X != 0) || (extra_Offset_Y != 0)){
		OffsetRect(&destination_Rect, extra_Offset_X, extra_Offset_Y);
		OffsetRgn(clipping_Region, extra_Offset_X, extra_Offset_Y);
		region_Needs_Adjustment_Back = TRUE;
	}
	else {
		region_Needs_Adjustment_Back = FALSE;
	}
			
	switch (current_Puppet->direct_Art.port.color_Type) {
		case k_COLOR_QUICKDRAW:
		case k_32_BIT_COLOR_QUICKDRAW:
					
			if ((clipping_Region == nil) && (use_Direct_Screen_Access == TRUE)){
				/*  */
				/* Danger:  beware of moving pixmap memory, though Copybits may in theory as well */
				/*  */
				Draw_Copied_Bits((const BitMap *)*(dream.stage_View.mix_View_Area_Port_Ptr[0]->offscreen_Port.color_Port.portPixMap)
						, (const BitMap *)*(port_Ptr->offscreen_Port.color_Port.portPixMap)
						, &source_Rect
						, &destination_Rect
						, srcCopy
						, nil);
							
			}
			else {
				CopyBits ((const BitMap *)*(dream.stage_View.mix_View_Area_Port_Ptr[0]->offscreen_Port.color_Port.portPixMap)
							, (const BitMap *)*(port_Ptr->offscreen_Port.color_Port.portPixMap)
							, &source_Rect
							, &destination_Rect
							, srcCopy
							, clipping_Region);
			}
		break;
		
		case k_ORIGINAL_QUICKDRAW:
		default:
		
			CopyBits (&(dream.stage_View.mix_View_Area_Port_Ptr[0]->offscreen_Port.old_Style_Port.portBits)
							, (const BitMap *)*(port_Ptr->offscreen_Port.color_Port.portPixMap)
							, &source_Rect
							, &destination_Rect
							, srcCopy
							, clipping_Region);
												
	} /*  switch (g_Indicator.color_Type) */
	
	if (region_Needs_Adjustment_Back == TRUE) {
		OffsetRgn(clipping_Region, -extra_Offset_X, -extra_Offset_Y);
	}
	
	function_Result = SAFE_TRUE;
	goto Normal_Function_Exit;
		
Error_Handler:
	Display_Serious_Error_Message(unit_Name, code_Name, "\pProblem encountered", "\p");
	function_Result = SAFE_FALSE;
		
Normal_Function_Exit:

	return(function_Result);
	
} /* Safe_Boolean	Erase_Old_Item_From_Mix_Buffer... */
/*=**************************************************************************=*/


/*=**************************************************************************=*/
Safe_Boolean	Erase_Spin_Item_From_Mix_Buffer( Off_Screen_Port_Rec *port_Ptr
																					, Off_Screen_Port_Rec *optional_Background_Port_Ptr
																					, CARDINAL_32	composite_Search_Key
																					, CARDINAL_32	spin_Info_Address
																					, CARDINAL_32 previous_Index
																					, INTEGER_16 file_Reference
																					, INTEGER_16 offset_X
																					, INTEGER_16 offset_Y)
	/*  */
	/* This only erases a region covered by a puppet with a */
	/* chunk of background. It is needed for animations that require*/
	/* full deletion of all items explicitely to allow multiple overlapping */
	/* items in a compositing buffer . Erasure cannot be done in an atomic */
	/* action in that case.*/
	/*  */
{
	Rect						backdrop_Size_Rect;
	CARDINAL_32	picture_Row_Byte_Offset;
	CARDINAL_32	stored_Bit_Depth;
	CARDINAL_32	previous_Data_Start_Offset;
	CARDINAL_32	previous_Compression_Type;
	INTEGER_32		previous_Frame_Total_Size;
	CARDINAL_32 	old_Storage_Location_To_Use;

	Rect					 	optional_Background_Rect;
	CARDINAL_32 	optional_Background_Row_Byte_Offset;
		
	Safe_Boolean			function_Result;
	IOParam					sync_Params;
	CARDINAL_8			*code_Name;
	CARDINAL_32		*row_Raw_Data_Ptr;
	CARDINAL_32		*skip_List_Data_Ptr;
	CARDINAL_32		*skip_List_Master_List_Ptr;
	CARDINAL_32		*destination_Word_Ptr;
	CARDINAL_32		*scan_Line_Bounds_Ptr;
	compression_Data_Header_Ptr	compression_Info;
	CARDINAL_32		*optional_Background_Word_Ptr;
	Boolean					old_Frame_Ready_To_Fetch;
	OSErr						os_Error;
	Str255						note_String;
	Spin_Pict_Info_Handle	spin_Info_Hdl;
/* END VAR */


	code_Name = "\pErase_Spin_Item_From_Mix_Buffer";
	function_Result = SAFE_TRUE;

	spin_Info_Hdl = (Spin_Pict_Info_Handle)spin_Info_Address;
	
	if ((*spin_Info_Hdl)->valid != SAFE_TRUE) {
		goto Error_Handler;
	}

	previous_Compression_Type = (*spin_Info_Hdl)->cell_Info[previous_Index].compression_Type;

	if (previous_Compression_Type == k_NO_COMPRESSION) {
		/*  */
		/* HACK HACK HACK : it is not critical to erase because erasure is usually */
		/* required for other purposes such as static animation. A Movie would */
		/* be crippled if it had to erase each frame for no reason. Add */
		/* a special flag to indicate that real erasure is required (such as a moving */
		/* rectangular area) */
		/*  */

		function_Result = SAFE_TRUE;
		goto Normal_Function_Exit;
	}

	if (previous_Compression_Type != k_MERKLE_TIME_COMPRESSION) {
		goto Error_Handler;
	}
	
	/*  */
	/* "g_Spin_Ring_Buffer_Initialized" should be checked? */
	/* not really needed. but if is not SAFE_TRUE, then set */
	/* g_Spin_Cache_Master_Ptr to point to nil for safety. */
	/*  */
	if ((g_Spin_Ring_Buffer_Initialized != SAFE_TRUE) || (g_Spin_Cache_Master_Ptr == nil)){
		goto Error_Handler;
	}

	backdrop_Size_Rect = (**(port_Ptr->offscreen_Port.color_Port.portPixMap)).bounds;
	picture_Row_Byte_Offset = ((**(port_Ptr->offscreen_Port.color_Port.portPixMap)).rowBytes & (~COLOR_PORT_PORTBITS_CHECK));
	stored_Bit_Depth = (**(port_Ptr->offscreen_Port.color_Port.portPixMap)).pixelSize;

	previous_Data_Start_Offset = (*spin_Info_Hdl)->cell_Info[previous_Index].data_Start_Offset;
	previous_Frame_Total_Size = (*spin_Info_Hdl)->cell_Info[previous_Index].data_Total_Size;
	
	old_Frame_Ready_To_Fetch = TRUE;
	
	if (Retrieve_Element_From_Buffer(g_Spin_Cache_Master_Ptr
												, composite_Search_Key
												, spin_Info_Address
												,&old_Storage_Location_To_Use) != SAFE_TRUE) {
												
		if (Prepare_Location_For_Buffer_Write(g_Spin_Cache_Master_Ptr
											, composite_Search_Key
											, spin_Info_Address
											, previous_Frame_Total_Size
											, &old_Storage_Location_To_Use) == SAFE_TRUE) {
											
			old_Frame_Ready_To_Fetch = FALSE;
			
		}
		else {
			Display_Serious_Error_Message(unit_Name, code_Name, "\pPrepare_Location_For_Buffer_Write failed to find node", "\pBad programming or cache size?");
			goto Error_Handler;
		}
	}
	
	if (previous_Compression_Type == k_MERKLE_TIME_COMPRESSION) {
	
		if (old_Frame_Ready_To_Fetch != TRUE) {
			/*  */
			/* Reload the old picture, and recheck is required of previously located cache value */
			/*  */
			
			/*  */
			/* NOTE: this has no worthwhile reason to be asyncronous because */
			/* it NORMALLY is in the cache. */
			/*  */
			
			sync_Params.ioRefNum = file_Reference;
			sync_Params.ioPosMode = fsFromStart;
			sync_Params.ioPosMode = (sync_Params.ioPosMode | k_FORCE_NO_CACHE_BIT);
			sync_Params.ioCompletion = nil;
			sync_Params.ioResult = 1;
			sync_Params.ioBuffer = (Ptr)old_Storage_Location_To_Use;
			sync_Params.ioReqCount = previous_Frame_Total_Size;
			sync_Params.ioPosOffset = previous_Data_Start_Offset;
			
			os_Error = PBRead((ParmBlkPtr)&sync_Params,	FALSE);
			if (os_Error != noErr) {
				Assign_String(note_String , "\pFSRead failed to read data. Error = ");
				Concat_Long_To_String(note_String, os_Error, NO_LIMIT);
			
				Display_Serious_Error_Message(unit_Name, code_Name, note_String, "\pBad file or drive?");
				goto Error_Handler;
			}

		}
		else {
			/*  */
			/* Was in cache already. Good, its ready to use for erasure. */
			/*  */
		}
				
		compression_Info = (compression_Data_Header_Ptr)old_Storage_Location_To_Use;

		if (compression_Info->sentinel_1 != SAFE_TRUE) {
			goto Error_Handler;
		}

		row_Raw_Data_Ptr = (CARDINAL_32 *)((CARDINAL_32)compression_Info + compression_Info->row_Raw_Data_Start_Offset);
		skip_List_Data_Ptr = (CARDINAL_32 *)((CARDINAL_32)compression_Info + compression_Info->skip_List_Data_Start_Offset);
		skip_List_Master_List_Ptr = (CARDINAL_32 *)((CARDINAL_32)compression_Info + compression_Info->skip_List_Master_List_Start_Offset);
		scan_Line_Bounds_Ptr = (CARDINAL_32 *)((CARDINAL_32)compression_Info + compression_Info->scan_Line_Bounds_Offset); 
					
		/*  */
		/* Avoid GetPixBaseAddr for a little speed */
		/*  */
		destination_Word_Ptr = (CARDINAL_32 *)*(port_Ptr->image_Handle);
					
		if (destination_Word_Ptr == nil) {
			goto Error_Handler;
		}
		
		*(CARDINAL_32 *)&destination_Word_Ptr += 
							(CARDINAL_32)((CARDINAL_32)(picture_Row_Byte_Offset)
							* (CARDINAL_32)(offset_Y));
	
		*(CARDINAL_32 *)&destination_Word_Ptr += 
								(((CARDINAL_32)(offset_X) 
									* (CARDINAL_32)(stored_Bit_Depth)) >> 3);

		optional_Background_Rect = (**(optional_Background_Port_Ptr->offscreen_Port.color_Port.portPixMap)).bounds;
		optional_Background_Row_Byte_Offset = ((**(optional_Background_Port_Ptr->offscreen_Port.color_Port.portPixMap)).rowBytes & (~COLOR_PORT_PORTBITS_CHECK));
					
	/* ====
		if ((optional_Background_Rect.left >= backdrop_Size_Rect.left)
			&& (optional_Background_Rect.right <= backdrop_Size_Rect.right)
			&& (optional_Background_Rect.top >= backdrop_Size_Rect.top)
			&& (optional_Background_Rect.bottom <= backdrop_Size_Rect.bottom)) {
			
		}
	==== */

		optional_Background_Word_Ptr = (CARDINAL_32 *)*(optional_Background_Port_Ptr->image_Handle);
	
		*(CARDINAL_32 *)&optional_Background_Word_Ptr += 
							(CARDINAL_32)((CARDINAL_32)(optional_Background_Row_Byte_Offset)
							* (CARDINAL_32)(offset_Y));
	
		*(CARDINAL_32 *)&optional_Background_Word_Ptr += 
								(((CARDINAL_32)(offset_X) 
									* (CARDINAL_32)(stored_Bit_Depth)) >> 3);

		/*  */
		/* This is background erasure step it could be faster, but the way it */
		/* works it erases the entire object filling in from a source picture */
		/*  */
		Fill_Compressed_Span_With_Simple_Silloette(row_Raw_Data_Ptr
														, skip_List_Data_Ptr
														, skip_List_Master_List_Ptr
														, destination_Word_Ptr
														, picture_Row_Byte_Offset
														, optional_Background_Word_Ptr
														, optional_Background_Row_Byte_Offset);
	
	} /* if (previous_Compression_Type == k_MERKLE_TIME_COMPRESSION) */


	function_Result = SAFE_TRUE;
	goto Normal_Function_Exit;
		
Error_Handler:
	Display_Serious_Error_Message(unit_Name, code_Name, "\pProblem encountered", "\pBad Pointer?");
	function_Result = SAFE_FALSE;
		
Normal_Function_Exit:

	return(function_Result);

} /* Safe_Boolean	Safe_Boolean	Erase_Spin_Item_From_Mix_Buffer ... */
/*=**************************************************************************=*/


/*=**************************************************************************=*/
void Reset_Graphics_Tables_For_Main_Screen_If_Needed(void)
{
	WindowPtr			current_Graphics_Window_Ptr;
	GrafPtr				saved_Port;
/* END VAR */


	if (active_Window_List[TEST_DREAMING_WINDOW_INDEX] != nil) {
		if ((**active_Window_List[TEST_DREAMING_WINDOW_INDEX]).structure_Initialized == SAFE_TRUE) {
			if ((**active_Window_List[TEST_DREAMING_WINDOW_INDEX]).draw_To_Screen_Direct_Plausable == SAFE_TRUE) {
				GetPort(&saved_Port);
				Set_View_Port((**active_Window_List[TEST_DREAMING_WINDOW_INDEX]).status_Window);
				if (Reset_Current_Window_Structures(TRUE, FALSE) != TRUE) {
					/*  */
					/* HACK HACK : need to raise panic alert */
					/*  */
				}
				Set_View_Port(saved_Port);

			}
		}
	}

}
/*=**************************************************************************=*/


/*=**************************************************************************=*/
/* END OF FILE: Dreaming_Animation.c */
