/* START OF FILE: Dreaming_Audio.c */


/*=**************************************************************************=*/
/* [---BEGIN---] FILE HEADER COMMENTS */
/**
 ** SCREEN VIEW = [Adobe Syntax Black-10, Tabs = 4]
 ** PRINTER VIEW = [Adobe Syntax Black-10, Tabs = 4]
 ** =======================================================================
 ** Written by James Merkle
 ** Dreaming_Audio.c - Audio Routines to support Poor Man's Video Phone
 ** (C) Copyright 1993 James Merkle
 ** All rights reserved, except those wishing to "copy" moderate sections of code or files may do so, so long as
 ** such code is never used directly either in animation-related tools or multimedia tools that are  
 ** used in live performance or live broadcast or recorded mass market broadcast or film. Desktop usage not restricted.
 ** Conventional desktop game or entertainment not restricted.
 ** =======================================================================
 **
 **
 ** ============
 ** Revision History
 ** ============
 **
 **  1993.05.03		(James Merkle)
 **						Initial coding started, some from other routines
 **
 **
 **
 ** =================
 ** OTHER INFORMATION
 ** =================
 ** 
 ** 
 **
 **
 **/
/* [----END----] FILE HEADER COMMENTS */
/*=**************************************************************************=*/


/*=**************************************************************************=*/
/* [---BEGIN---] #include AREA */


#include <GestaltEqu.h>
#include <Sound.h>
#include <SoundInput.h>

/* THINK C */
#include <Asm.h>
#include <Files.h>
#include <Traps.h>

#include "Error_Utilities.h"
#include "OS_Utilities.h"
#include "Shell_Dialogs.h"
#include "Shell_Globals.h"
#include "Memory_Utilities.h"
#include "Dreaming_Popup_Dialogs.h"
#include "Sound_Utilities.h"
#include "Sound_Play.h"
#include "String_Utilities.h"
#include "Time_Utilities.h"
#include "Volume_Utilities.h"
#include "Window_Utilities.h"
#include "Dreaming_Audio_Record.h"
/* ===
#include "Sound_Record.h"
=== */

#include "Dreaming_Analog_Card.h"

#include "Core_Universal.h" /* Non_Specific_Support_Routines */
#include "Shell_Globals.h"

#include "Dreaming_Stuff.h"
#include "Sound_Input_Device.h"
#include "Sound_Tap_Channel.h"

#include "Dreaming_Audio.h"

/* [----END----] #include AREA */
/*=**************************************************************************=*/


/*=**************************************************************************=*/
/* [---BEGIN---] #define AREA */



/*  */
/* Governers: Limits monitoring to 60 (1) times per second maximum */
/* Would be more efficient at 30 (2) but sound status is paramount and is */
/* useful as soon as it changes state. */
/*  */
#define k_TICK_DELAY_BETWEEN_SID_RESAMPLING						1
#define k_TICK_DELAY_BETWEEN_ANALOG_CARD_TAPPING		1
#define k_TICK_DELAY_BETWEEN_AUDIO_FILE_STATUS					1


/*  */
/* These Variables control dropoff logic */
/*  */
#define k_REDUCTION_JITTER_ALLOWANCE						(2L)
#define k_SECOND_STAGE_REDUCTION								(35L)
#define k_FIRST_STAGE_REDUCTION										(20L)
#define k_ITERATIONS_BEFORE_DROP_INTERPOLATION	(3)

/*  */
/* 140 too high, 90 too low */
/*  */
#define k_ALMOST_MAX_VOLUME										(100L)

/*  */
/* Only set to 1 if non multiline scanning is performed */
/* Normally "0". Wastes lots of CPU if set.  */
/*  */
#define ALLOW_CARD_UPDATE_ITERATIVELY					0
		
#define k_MAX_SID_8_BIT_AMPLITUDE												(255L)

				
/* [----END----] #define AREA */
/*=**************************************************************************=*/


/*=**************************************************************************=*/
/* [---BEGIN---] TYPE AREA */


/* [----END----] TYPE AREA */
/*=**************************************************************************=*/


/*=**************************************************************************=*/
/* [---BEGIN---] PROTOTYPE AREA */

Safe_Boolean Initialize_Audio_Monitoring_If_Needed(void);

Safe_Boolean Notify_Async_Audio_Problem_Found_If_First_Time(void);
	
Safe_Boolean Initialize_Sound_Input_Fields(Recording_Ptr input);

Safe_Boolean Initialize_Sampler_Card_Sampling_Fields(Sampler_Ptr input);

Safe_Boolean Initialize_Audio_File_Tracking_Fields(Aufio_File_Ptr input);

Safe_Boolean Extract_Recent_Amplitude(Voice_Hdl	voice, CARDINAL_32	*raw_Signal);

Safe_Boolean Create_Sound_Inputs_For_Card(Audio_Source_Type type
																, CARDINAL_32	total_Channels_Scanning);

Safe_Boolean Extract_Recent_Amplitude_From_Multiplexed_Lines(CARDINAL_32	*raw_Signal);

Safe_Boolean Process_Sound_Updating_From_Multiplexed_Lines(void);

Safe_Boolean Process_Sound_Updating_For_SID(Voice_Hdl	voice);

Safe_Boolean Process_Sound_Updating_For_Sampler(Voice_Hdl	voice);

Safe_Boolean Process_Sound_Updating_For_Audio_File(Voice_Hdl	voice);

Safe_Boolean Extract_Recent_Amplitude_From_Audio_File(Voice_Hdl	voice, CARDINAL_32	*raw_Signal);


/* [----END----] PROTOTYPE AREA */
/*=**************************************************************************=*/


/*=**************************************************************************=*/
/* [---BEGIN---] GLOBAL DEFINITION AREA */

	Voice_Monitoring_Rec		g_Voices;                 
	Safe_Boolean						g_Voice_Globals_Initialized;


/* [----END----] GLOBAL DEFINITION AREA */
/*=**************************************************************************=*/


/*=**************************************************************************=*/
/* [---BEGIN---] LOCAL STATIC DEFINITION AREA */

static CARDINAL_8	*unit_Name = "\pDreaming_Audio";

	/*  */
	/* These are only used for momentary extraction from */
	/* sampling cards to calculate peaks. Only one set needed*/
	/* because it is not reentrant or recursive. */
	/*  */
	INTEGER_16						g_Samples[k_MAXIMUM_MINI_SAMPLES * 4L];
	INTEGER_16						*g_Sample_Ptr;

	Safe_Boolean		g_Severe_Audio_Record_Problem_Shown;
	Safe_Boolean		g_Severe_Audio_Async_Record_Problem_Shown;

/* [----END----] LOCAL STATIC DEFINITION AREA */
/*=**************************************************************************=*/


/*=**************************************************************************=*/
Safe_Boolean	Initialize_Audio_Monitoring_If_Needed(void)
{
	Voice_Monitoring_Ptr		voices_Ptr;
	Safe_Boolean		function_Result;
	CARDINAL_32	index;
/* END VAR */

	function_Result = SAFE_FALSE;
	
	if (g_Voice_Globals_Initialized != SAFE_TRUE) {
	
		voices_Ptr = &g_Voices;
		
		voices_Ptr->monitor_Microphone_For_Speech = SAFE_FALSE;
		voices_Ptr->monitor_Sampler_For_Speech = SAFE_FALSE;
		voices_Ptr->monitor_Keyboard_As_Speech = SAFE_FALSE;
		voices_Ptr->monitor_Network_As_Speech = SAFE_FALSE;
		voices_Ptr->monitor_MIDI_For_Sound = SAFE_FALSE;
		voices_Ptr->Monitor_Phonebridge_For_Sound = SAFE_FALSE;
		voices_Ptr->total_Voices_Active = 0;
		
		voices_Ptr->tickCount_MIO_DAQ_Last_Updated = 0;

		voices_Ptr->phone_Rec.phoneBridge_Enabled = FALSE;
		voices_Ptr->phone_Rec.phoneBridge_ADB_Active = FALSE;
		
		voices_Ptr->phone_Rec.phoneBridge_Off_Hook = FALSE;
		voices_Ptr->phone_Rec.phoneBridge_Ringing = FALSE;
		voices_Ptr->phone_Rec.phoneBridge_Ref = 0;

		for (index = 0; (index < k_MAXIMUM_VOICE_MONITORS); index++) {
			voices_Ptr->voice_Active[index] = SAFE_FALSE;
			voices_Ptr->voice[index] = nil;
			voices_Ptr->voice_Memory_Initialized[index] = MEMORY_NOT_INITIALIZED;
		}
	
		g_Sample_Ptr = &(g_Samples[0]);

		voices_Ptr->initialized = SAFE_TRUE;
		g_Voice_Globals_Initialized = SAFE_TRUE;
		
		g_Severe_Audio_Record_Problem_Needs_Display_Soon = SAFE_FALSE;
		g_Severe_Audio_Record_Problem_Shown = SAFE_FALSE;
		g_Severe_Audio_Async_Record_Problem_Shown = SAFE_FALSE;
				
	} /* if (g_Voice_Globals_Initialized != SAFE_TRUE) */
	
	function_Result = SAFE_TRUE;
	goto Normal_Function_Exit;
	
Error_Handler:
	function_Result = SAFE_FALSE;
			
Normal_Function_Exit:
	return(function_Result);
	
} /* Safe_Boolean	Initialize_Dreaming_Globals... */
/*=**************************************************************************=*/


/*=**************************************************************************=*/
Safe_Boolean Stop_All_Sound_Monitoring_Of_Type(Audio_Source_Type type_Of_Service_To_Kill)
{
	CARDINAL_32		max_Items;
	CARDINAL_32		total_Items_Seen;
	Safe_Boolean			function_Result;
	CARDINAL_8			*code_Name;
	INTEGER_16			voice;
/* END VAR */


	code_Name = "\pStop_All_Sound_Monitoring_Of_Type";	
	function_Result = SAFE_TRUE;

	switch(type_Of_Service_To_Kill) {
	
		case REMOTE_NETWORK_FEED:
		
		break;
	
		case PRIMARY_SOUND_INPUT:
			if (g_Voices.monitor_Microphone_For_Speech == SAFE_TRUE) {
			
				max_Items = g_Voices.total_Voices_Active;
				total_Items_Seen = 0;
				
				for (voice=0; ((voice < k_MAXIMUM_VOICE_MONITORS) && (total_Items_Seen < max_Items)); voice++) {
					if (g_Voices.voice_Memory_Initialized[voice] == SAFE_TRUE) {
						total_Items_Seen += 1;
						if ((**g_Voices.voice[voice]).audio_Source == PRIMARY_SOUND_INPUT) {
							if (Shutdown_Voice(voice, PRIMARY_SOUND_INPUT) != SAFE_TRUE) {
								/*  */
								/* Error exception not implemented. No need to? */
								/*  */
							}
						}
					}
				}
			}
		break;
	
		case NATIONAL_INSTRUMENTS_DA:
			if (g_Voices.monitor_Sampler_For_Speech == SAFE_TRUE){
				
				max_Items = g_Voices.total_Voices_Active;
				total_Items_Seen = 0;

				for (voice=0; ((voice < k_MAXIMUM_VOICE_MONITORS) && (total_Items_Seen < max_Items)); voice++) {
					if (g_Voices.voice_Memory_Initialized[voice] == SAFE_TRUE) {
						total_Items_Seen += 1;
						if ((**g_Voices.voice[voice]).audio_Source == NATIONAL_INSTRUMENTS_DA) {
							if (Shutdown_Voice(voice, NATIONAL_INSTRUMENTS_DA) != SAFE_TRUE) {
								/*  */
								/* Error exception not implemented. No need to? */
								/*  */
							}
						}
					}
				}
			}
		break;

		case LOCAL_AUDIO_FILE_PLAYING:				
				max_Items = g_Voices.total_Voices_Active;
				total_Items_Seen = 0;

				for (voice=0; ((voice < k_MAXIMUM_VOICE_MONITORS) && (total_Items_Seen < max_Items)); voice++) {
					if (g_Voices.voice_Memory_Initialized[voice] == SAFE_TRUE) {
						total_Items_Seen += 1;
						
						if ((**g_Voices.voice[voice]).audio_Source == LOCAL_AUDIO_FILE_PLAYING) {
							
							if (Shutdown_Voice(voice, LOCAL_AUDIO_FILE_PLAYING) != SAFE_TRUE) {
								/*  */
								/* Error exception not implemented. No need to? */
								/*  */
							}
						}
					}
				}
		break;


				
		case TOP_KEYBOARD_ROW:
		
		break;
		
		case MIDI_INPUT:
		
		break;
		
		default:
			Display_Serious_Error_Message(unit_Name, code_Name, "\pundefined type", "\pBad programming");
			goto Error_Handler;
	}


	function_Result = SAFE_TRUE;
	goto Normal_Function_Exit;
	
Error_Handler:
	function_Result = SAFE_FALSE;
			
Normal_Function_Exit:
	return(function_Result);
	
} /* Safe_Boolean Stop_All_Sound_Monitoring_Of_Type... */
/*=**************************************************************************=*/


/*=**************************************************************************=*/
Safe_Boolean Initialize_Sound_Input_Fields(Recording_Ptr input)
{
	Safe_Boolean		function_Result;
/* END VAR */

	input->initialized = SAFE_TRUE;

	function_Result = SAFE_TRUE;
		
Normal_Function_Exit:

	return(function_Result);
}
/*=**************************************************************************=*/


/*=**************************************************************************=*/
Safe_Boolean Initialize_Sampler_Card_Sampling_Fields(Sampler_Ptr input)
{
	Safe_Boolean		function_Result;
/* END VAR */

	input->initialized = SAFE_TRUE;

	function_Result = SAFE_TRUE;
		
Normal_Function_Exit:

	return(function_Result);
}
/*=**************************************************************************=*/


/*=**************************************************************************=*/
Safe_Boolean Initialize_Audio_File_Tracking_Fields(Aufio_File_Ptr input)
{
	Safe_Boolean		function_Result;
/* END VAR */

	input->initialized = SAFE_TRUE;
	input->tap_Possibly_Active = SAFE_FALSE;
	input->previous_Open_Voice_Index_Valid = SAFE_FALSE;
	input->previous_Open_Voice_Index = 0;
	input->playback_Index = 0;
	

	function_Result = SAFE_TRUE;
		
Normal_Function_Exit:

	return(function_Result);
}
/*=**************************************************************************=*/


/*=**************************************************************************=*/
Safe_Boolean Shutdown_Voice(CARDINAL_32		index, Audio_Source_Type type_To_Destroy)
{
	Safe_Boolean		function_Result;
	Recording_Ptr	monitor;
	Safe_Boolean		voice_Closed;
	INTEGER_16		input_Reference;
	OSErr					os_Error;
	Audio_Source_Type	input_Type;
	Boolean				problem_Encounterred;
/* END VAR */


	function_Result = SAFE_FALSE;

	if ((**g_Voices.voice[index]).initialized != SAFE_TRUE) {
		goto Error_Handler;
	}

	input_Type = (**g_Voices.voice[index]).audio_Source;
	voice_Closed = SAFE_FALSE;

	if (input_Type == PRIMARY_SOUND_INPUT) {
	
		if ((type_To_Destroy == PRIMARY_SOUND_INPUT) || (type_To_Destroy == ANY_INPUT_AT_ALL)) {
		
			if (Stop_Recording_To_Voice_Audio_File(g_Voices.voice[index]) != SAFE_TRUE) {
				/*  */
				/* Stop monitoring and flush and close files before stopping voice completely */
				/* This is usually done at beginning of application shutdown process and this */
				/* is for safety only under that situation. */
				/*  */
			}
		
			problem_Encounterred = FALSE;
			
			input_Reference = ((**g_Voices.voice[index]).info.apple_Sound_Input).input_Reference;
			
			if (Stop_Sound_Input_Monitoring(&((**g_Voices.voice[index]).info.apple_Sound_Input)) != SAFE_TRUE) {
				problem_Encounterred = TRUE;
			}
			else {
				voice_Closed = SAFE_TRUE;
			}
			
			/*  */
			/* Retry closure for extra safety if Stop_Sound_Input_Monitoring */
			/* did not finish the job */
			/*  */
			os_Error = SPBCloseDevice(input_Reference);
			if (os_Error != noErr) {
				/*  */
				/* Ignore error, especially -229 because might already be closed. */
				/*  */
				/* ==
					if (os_Error != -229) {
						problem_Encounterred = TRUE;
					}
				== *(/
			}
			else {
				voice_Closed = SAFE_TRUE;
			}
 
			if (problem_Encounterred == TRUE) {
				goto Error_Handler;
			}
			
		}
	}
	else 	if (input_Type == NATIONAL_INSTRUMENTS_DA) {

		if ((type_To_Destroy == NATIONAL_INSTRUMENTS_DA) || (type_To_Destroy == ANY_INPUT_AT_ALL)) {
			voice_Closed = SAFE_TRUE;
			if (Stop_Sampling_From_Card() != SAFE_TRUE) {
				/*  */
				/* Error exception not implemented. No need to? */
				/*  */
			}
		}
		
	}
	else 	if (input_Type == LOCAL_AUDIO_FILE_PLAYING) {
		if ((type_To_Destroy == LOCAL_AUDIO_FILE_PLAYING) || (type_To_Destroy == ANY_INPUT_AT_ALL)) {
		
			/*  */
			/* We were spying and did not create memory, so just */
			/* kill all references. We could not allocate anyway because it */
			/* is all interrupt fed and hardware manipulated. This is why we just spy */
			/* on other global structures. */
			/* We should change the internal flags but the structure needs to*/
			/* have valid fields when they detatched from active puppets in this case. */
			/* The reason: voice may need to be swapped back */
			/*  */
	
			/*  */
			/* Immediately issue close and kill request to already playing sound. */
			/* This also disposes of its memory. This is usually not needed. */
			/*  */
			if (g_AIFF_Structures_Opened > 0) {
				if ((**g_Voices.voice[index]).info.audio_File.initialized == SAFE_TRUE) {
					if ((**g_Voices.voice[index]).info.audio_File.tap_Possibly_Active == SAFE_TRUE) {
	
						if (Stop_Playing_AIFF_Asynchronously(&g_Playback[(**g_Voices.voice[index]).info.audio_File.playback_Index]) != SAFE_TRUE) {
							/*  */
							/* Ignore error, if any. Problems of this type sort themselves out properly. */
							/*  */
						}
					}
					(**g_Voices.voice[index]).info.audio_File.tap_Possibly_Active = SAFE_FALSE;
				}
			}
			
			/*  */
			/* This is also usually not needed in our design */
			/*  */
			if ((**g_Voices.voice[index]).info.audio_File.tap_Possibly_Active == SAFE_TRUE) {
				if (Revert_All_Voices_In_Items_To_Paused_Original_Sources(g_Voices.voice[index]) != SAFE_TRUE) {
					/*  */
					/* Ignore error, if any in this version. */
					/*  */
				}
			}
	
			voice_Closed = SAFE_TRUE;
		} /* if ((type_To_Destroy == LOCAL_AUDIO_FILE_PLAYING)... */
		
	}
	else {
		/*  */
		/* Other sound types are not supported yet */
		/*  */
	/* ===
		if ((type_To_Destroy == PRIMARY_SOUND_INPUT) || (type_To_Destroy == ANY_INPUT_AT_ALL)) {
		
		}
	=== */
		Perform_OK_Dialog("\pProgram error. unknown sound type to close");
	}

	if (voice_Closed == SAFE_TRUE) {

		/*  */
		/* Setting to nil is almost good enough, but now we need to */
		/* hunt through all scenes and all objects */
		/* to kill any audio connection references to this one */
		/*  */
		
		if (Delete_A_Specific_Input_From_Any_Scene_Puppets(g_Voices.voice[index]) != SAFE_TRUE) {
			/* Not important */
			Perform_OK_Dialog("\pNote: Delete_A_Specific_Input_From_Any_Scene_Puppets failed.");
		}

		(**g_Voices.voice[index]).initialized = SAFE_FALSE;
	
		g_Voices.voice_Active[index] = SAFE_FALSE;
		if (g_Voices.total_Voices_Active > 0) {
			g_Voices.total_Voices_Active -= 1;
		}
		
		if (Destroy_Handle((Handle *)&(g_Voices.voice[index])
				, &(g_Voices.voice_Memory_Initialized[index])) != SAFE_TRUE) {
			goto Error_Handler;
		}
	}

	function_Result = SAFE_TRUE;
	goto Normal_Function_Exit;
	
Error_Handler:
	function_Result = SAFE_FALSE;
		
Normal_Function_Exit:

	return(function_Result);
	
} /* Safe_Boolean Shutdown_Voice... */
/*=**************************************************************************=*/


/*=**************************************************************************=*/
Safe_Boolean Create_Sound_Inputs_For_Card(Audio_Source_Type type
																, CARDINAL_32	total_Channels_Scanning)
{
	Safe_Boolean		function_Result;
	SIMPLE_LOOP	loop;
	CARDINAL_32	position;
/* END VAR */

	function_Result = SAFE_TRUE;

	for (loop = 0; (loop < total_Channels_Scanning) ; loop++) {
		if (Create_Sound_Input_Voice(type, &position) == SAFE_TRUE) {
			if ((**g_Voices.voice[position]).initialized == SAFE_TRUE) {
				(**g_Voices.voice[position]).audio_Source_Channel = loop;
			}
			else {
				function_Result = SAFE_FALSE;
			}
		}
		else {
			function_Result = SAFE_FALSE;
		}
	}


	goto Normal_Function_Exit;

Error_Handler:
	function_Result = SAFE_FALSE;

Normal_Function_Exit:

	return(function_Result);
}
/*=**************************************************************************=*/


/*=**************************************************************************=*/
Safe_Boolean Create_Sound_Input_Voice(Audio_Source_Type type
					, CARDINAL_32	*position)
/*  */
/* Generally designed for all types of input devices */
/*  */
{
	Safe_Boolean		function_Result;
	CARDINAL_32	memory_Requested;
	INTEGER_16		voice;
	Boolean				located_Microphone_Input;
	Boolean				located_Free_Spot;
	SignedByte 		original_State;	
/* END VAR */


	function_Result = SAFE_FALSE;
	*position = 0;

	if (Initialize_Audio_Monitoring_If_Needed() != SAFE_TRUE) {
		goto Error_Handler;
	}

	if (g_Voices.total_Voices_Active >= k_MAXIMUM_VOICE_MONITORS) {
		goto Error_Handler;
	}

	located_Free_Spot = FALSE;
	for (voice=0; ((voice < k_MAXIMUM_VOICE_MONITORS)
			&& (located_Free_Spot != TRUE)); voice++) {

		if (g_Voices.voice_Memory_Initialized[voice] != MEMORY_INITIALIZED) {

			g_Voices.voice_Memory_Initialized[voice] = MEMORY_NOT_INITIALIZED;
			memory_Requested = sizeof(Voice_Rec);
			if (Create_Handle( (CARDINAL_32 *)&memory_Requested
									, (CARDINAL_32)0L
									, ALLOW_PURGING
									, DISALLOW_PARTIAL_AMOUNT
									, (Handle *)&(g_Voices.voice[voice])
									, &(g_Voices.voice_Memory_Initialized[voice])) != SAFE_TRUE ) {
									
				Perform_OK_Dialog("\pMemory error allocating buffer_Memory (not enough?)");

				goto Normal_Function_Exit;
			}
			
			if (Begin_Use_Of_Handle((Handle)g_Voices.voice[voice]
									, MOVE_HIGH_REQUIRED
									, &original_State) != SAFE_TRUE) {
				/*  */
				/* Ignore error in implementation */
				/*  */
			}

			(**g_Voices.voice[voice]).initialized = SAFE_TRUE;
			(**g_Voices.voice[voice]).audio_Source = type;
			(**g_Voices.voice[voice]).current_Raw_Signal = 0;
			
			(**g_Voices.voice[voice]).signals_Dropping_Consecutively = 0;
			(**g_Voices.voice[voice]).current_Interpolated_Signal = 0;
			
			(**g_Voices.voice[voice]).Max_Signal_Range = k_MAX_SID_8_BIT_AMPLITUDE;
			/*  */
			/* Only used in card gain amplification and AIFF curve amplification */
			/*  */
			(**g_Voices.voice[voice]).amplification_Response_Curve = ((SPEED_DIGITS_REAL)k_CARD_AMPLIFICATION_RESPONSE);
			(**g_Voices.voice[voice]).maximum_Signal_Since_Last = 0;
			(**g_Voices.voice[voice]).tickCount_Last_Updated = 0;
			(**g_Voices.voice[voice]).audio_Source_Channel = 0;
			
			g_Voices.voice_Memory_Initialized[voice] = SAFE_TRUE;
						
			switch ((**g_Voices.voice[voice]).audio_Source) {
			
				case PRIMARY_SOUND_INPUT:
					if (Initialize_Sound_Input_Fields(
								&((**(g_Voices.voice[voice])).info.apple_Sound_Input)) != SAFE_TRUE) {
								
						goto Error_Handler;
					}
				break;
				
				case NATIONAL_INSTRUMENTS_DA:
					if (Initialize_Sampler_Card_Sampling_Fields(
								&((**(g_Voices.voice[voice])).info.sampler_Board)) != SAFE_TRUE) {
								
						goto Error_Handler;
					}
				break;
								
				case LOCAL_AUDIO_FILE_PLAYING:
					if (Initialize_Audio_File_Tracking_Fields(
								&((**(g_Voices.voice[voice])).info.audio_File)) != SAFE_TRUE) {
								
						goto Error_Handler;
					}
				break;
								
				default:
					; /* NULL */
			}
			
			g_Voices.voice_Active[voice] = SAFE_TRUE;
			g_Voices.total_Voices_Active += 1;

			located_Free_Spot = TRUE;
			*position = voice;
		}
	}

	if (located_Free_Spot != TRUE) {
		goto Error_Handler;
	}

	function_Result = SAFE_TRUE;
	goto Normal_Function_Exit;
	
Error_Handler:
	function_Result = SAFE_FALSE;
		
Normal_Function_Exit:

	return(function_Result);
	
} /* Safe_Boolean Create_Sound_Input_Voice ... */
/*=**************************************************************************=*/


/*=**************************************************************************=*/
extern Safe_Boolean Get_Sound_Input_Voice(Audio_Source_Type type
					, Recording_Ptr *voice_Ptr)
/*  */
/* Generally designed for all types of input devices */
/*  */
{
	CARDINAL_32	max_Items;
	CARDINAL_32	total_Items_Seen;
	Safe_Boolean		function_Result;
	CARDINAL_32	position;
	INTEGER_16		voice;
	Boolean				located_Microphone_Input;
/* END VAR */


	function_Result = SAFE_FALSE;
	*voice_Ptr = nil;
	
	located_Microphone_Input = FALSE;
	
	if (g_Voices.initialized == SAFE_TRUE) {
	
		do {
	
			if (g_Voices.total_Voices_Active > 0) {
				if (g_Voices.monitor_Microphone_For_Speech == SAFE_TRUE) {
				
				
					max_Items = g_Voices.total_Voices_Active;
					total_Items_Seen = 0;
					for (voice=0; ((voice < k_MAXIMUM_VOICE_MONITORS)
									&& (total_Items_Seen < max_Items)
									&& (located_Microphone_Input != TRUE)); voice++) {
							
						if (g_Voices.voice_Memory_Initialized[voice] == SAFE_TRUE) {
							total_Items_Seen += 1;
							
							if ((**g_Voices.voice[voice]).initialized == SAFE_TRUE) {
								if ((**g_Voices.voice[voice]).audio_Source == type) {
		
									located_Microphone_Input = TRUE;
									position = voice;
								}
							}
						}
					}
				}
			} /* if (g_Voices.total_Voices_Active > 0) */
			
			if (located_Microphone_Input != TRUE) {
				/*  */
				/* Create storage for a new voice and initialize it and set it to be */
				/* equal to the current "voice" we are searching for */
				/*  */
				if (Create_Sound_Input_Voice(type, &position) != SAFE_TRUE) {
					goto Error_Handler;
				}
				else {
					/* ===
					voice = position;
					located_Microphone_Input = TRUE;
					=== */
				}
			}
			
		} while (located_Microphone_Input != TRUE);
	}
	
	if (located_Microphone_Input != TRUE) {
		goto Error_Handler;
	}

	voice = position;
	*voice_Ptr = &((**g_Voices.voice[voice]).info.apple_Sound_Input);


	if (Attach_Default_Inputs_To_All_Scene_Puppets(dream.play.default_Start_Audio_Point) != SAFE_TRUE) {
	
	}
	
	function_Result = SAFE_TRUE;
	goto Normal_Function_Exit;
	
Error_Handler:
	function_Result = SAFE_FALSE;
		
Normal_Function_Exit:

	return(function_Result);
	
} /* Safe_Boolean Get_Sound_Input_Voice ... */
/*=**************************************************************************=*/


/*=**************************************************************************=*/
Safe_Boolean Get_Sampler_Voice(Audio_Source_Type type)
/*  */
/* Generally designed for all types of input devices */
/*  */
{
	CARDINAL_32	max_Items;
	CARDINAL_32	total_Items_Seen;
	Safe_Boolean		function_Result;
	CARDINAL_32	position;
	INTEGER_16		voice;
	Boolean				located_Analog_Card_Input;
/* END VAR */


	function_Result = SAFE_FALSE;
	
	located_Analog_Card_Input = FALSE;
	
	if (g_Voices.initialized == SAFE_TRUE) {
	
		do {
	
			if (g_Voices.total_Voices_Active > 0) {
				if (g_Voices.monitor_Sampler_For_Speech == SAFE_TRUE) {
				
					max_Items = g_Voices.total_Voices_Active;
					total_Items_Seen = 0;
					for (voice=0; ((voice < k_MAXIMUM_VOICE_MONITORS) && (total_Items_Seen < max_Items)
							&& (located_Analog_Card_Input != TRUE)); voice++) {
							
						if (g_Voices.voice_Memory_Initialized[voice] == SAFE_TRUE) {
							total_Items_Seen += 1;

							if ((**g_Voices.voice[voice]).initialized == SAFE_TRUE) {
								if ((**g_Voices.voice[voice]).audio_Source == type) {
		
									located_Analog_Card_Input = TRUE;
									position = voice;
								}
							}
						}
					}
				}
			} /* if (g_Voices.total_Voices_Active > 0) */
			
			if (located_Analog_Card_Input != TRUE) {
				/*  */
				/* Create storage for a new voice and initialize it and set it to be */
				/* equal to the current "voice" we are searching for */
				/*  */
				if (Create_Sound_Inputs_For_Card(type, g_Voices.audio_Channels_Needed_On_Cards) != SAFE_TRUE) {
					goto Error_Handler;
				}
				else {
					/* ===
					located_Analog_Card_Input = TRUE;
					=== */
				}
			}
			
		} while (located_Analog_Card_Input != TRUE);
	}
	
	
	if (located_Analog_Card_Input != TRUE) {
		goto Error_Handler;
	}


	if (Attach_Default_Inputs_To_All_Scene_Puppets(dream.play.default_Start_Audio_Point) != SAFE_TRUE) {
	
	}
	
	function_Result = SAFE_TRUE;
	goto Normal_Function_Exit;
	
Error_Handler:
	function_Result = SAFE_FALSE;
		
Normal_Function_Exit:

	return(function_Result);
	
} /* Safe_Boolean Get_Sampler_Voice ... */
/*=**************************************************************************=*/


/*=**************************************************************************=*/
Safe_Boolean Reattach_Inputs_To_Scene_Puppets(CARDINAL_32 scene_To_Use, CARDINAL_32 start_Offset_Into_Voices, Boolean reset_Voices)
{
	Safe_Boolean		function_Result;
	Boolean				problems_Found;
	CARDINAL_32	resulting_Index;
	Safe_Boolean 	valid_Result;
	Boolean				valid;
	Stage_Item_Ptr	item_Ptr;
	CARDINAL_8		*code_Name;
/* END VAR */


	function_Result = SAFE_FALSE;
	code_Name = "\pReattach_Inputs_To_Scene_Puppets";
	
	if (g_Dreaming_Module_Globals_Initialized == SAFE_TRUE) {
		if (dream.play.total_Scenes > 0) {
			if (dream.play.scene_Memory_In_Use[scene_To_Use] == MEMORY_INITIALIZED) {
				if ((**(dream.play.scene[scene_To_Use])).initialized == SAFE_TRUE) {
					if ((**(dream.play.scene[scene_To_Use])).items_Remaining_To_Load == 0) {
					
						if ((**(dream.play.scene[scene_To_Use])).total_Items_In_Scene > 1) {
						
							if (Get_Dream_Popup_Menu_Choice(k_ANIMATED_PUPPET_IN_SCENE_SELECTION
																	, &resulting_Index
																	, &valid_Result) != SAFE_TRUE) {
								function_Result = SAFE_FALSE;
							}
							else {
								if (valid_Result != SAFE_TRUE) {
									function_Result = SAFE_FALSE;
								}
								else {

									function_Result = SAFE_TRUE;

									if (g_Voices.voice_Active[start_Offset_Into_Voices] == SAFE_TRUE) {
									
										valid = FALSE;
										if ((**(dream.play.scene[scene_To_Use])).item[resulting_Index].puppet != nil) {
											if ((**(dream.play.scene[scene_To_Use])).item[resulting_Index].valid_Pointer == SAFE_TRUE) {
												valid = TRUE;
											}
											else {
												Display_Serious_Error_Message(unit_Name, code_Name, "\pCorrupt Pointer (3)", "\pBad program");
												(**(dream.play.scene[scene_To_Use])).item[resulting_Index].puppet = nil;
											}
										}
						
										if (valid == TRUE) {
						
											/*  */
											/* NOTICE: This assumes it is already locked down */
											/*  */
											item_Ptr = &((**(dream.play.scene[scene_To_Use])).item[resulting_Index]);
									
											if (item_Ptr->controllers_Attached == SAFE_TRUE) {
												if (Force_Scene_Item_To_Mute_Status(item_Ptr) != SAFE_TRUE) {
													/*  */
													/* Not important to worry about failures of this command */
													/*  */
												}
											}
										
											if (item_Ptr->controllers_Attached != SAFE_TRUE) {
												
												if (item_Ptr->voice != nil) {
													Display_Serious_Error_Message(unit_Name, code_Name, "\pWarning, not clearing 'controllers_Attached'", "\pBad programming, remedied.");
													item_Ptr->voice = nil;
												}
												
												if (Force_Scene_Item_To_Mute_Status(item_Ptr) != SAFE_TRUE) {
													/*  */
													/* Not important to worry about failures of this command */
													/*  */
												}
						
												if (item_Ptr->voice == nil) {						
						
													if (g_Voices.voice_Active[start_Offset_Into_Voices] == SAFE_TRUE) {
														item_Ptr->voice = g_Voices.voice[start_Offset_Into_Voices];
														item_Ptr->controllers_Attached = SAFE_TRUE;
													}
						
												}
												
											} /* if (item_Ptr->controllers_Attached == SAFE_TRUE)  */
						
										} /* if (valid == TRUE) */
									
									} /*if (g_Voices.voice_Active[start_Offset_Into_Voices] == SAFE_TRUE) */
								
								} 
								
							}
						}
						else {
							function_Result	= Attach_Default_Inputs_To_Scene_Puppets(scene_To_Use, start_Offset_Into_Voices, reset_Voices);
						}
					}
				}
			}
		}
	}
		


	goto Normal_Function_Exit;
	
Error_Handler:
	function_Result = SAFE_FALSE;
		
Normal_Function_Exit:

	return(function_Result);
	
} /* Safe_Boolean Reattach_Inputs_To_Scene_Puppets ... */
/*=**************************************************************************=*/


/*=**************************************************************************=*/
Safe_Boolean Attach_Default_Inputs_To_All_Scene_Puppets(CARDINAL_32 start_Offset_Into_Voices)
{
	Safe_Boolean		function_Result;
	Boolean				problems_Found;
	INTEGER_16		scene_To_Use;
	CARDINAL_8		*code_Name;
/* END VAR */


	code_Name = "\pAttach_Default_Inputs_To_All_Scene_Puppets";
	function_Result = SAFE_FALSE;
	
	if (g_Dreaming_Module_Globals_Initialized != SAFE_TRUE) {
		function_Result = SAFE_TRUE;
		goto Normal_Function_Exit;
	}	
	
	if (dream.play.total_Scenes < 1) {
		function_Result = SAFE_TRUE;
		goto Normal_Function_Exit;
	}

	problems_Found = FALSE;
	for (scene_To_Use = 0; (scene_To_Use < k_MAX_ACTIVE_SCENES); scene_To_Use++) {
		if (Attach_Default_Inputs_To_Scene_Puppets(scene_To_Use, start_Offset_Into_Voices, FALSE) != SAFE_TRUE) {
			problems_Found = TRUE;
		}
	}
	
	if (problems_Found == TRUE) {
		function_Result = SAFE_FALSE;
	}
	else {
		function_Result = SAFE_TRUE;
	}
	
	goto Normal_Function_Exit;
	
Error_Handler:
	function_Result = SAFE_FALSE;
		
Normal_Function_Exit:

	return(function_Result);
}
/*=**************************************************************************=*/


/*=**************************************************************************=*/
Safe_Boolean Attach_Default_Inputs_To_Scene_Puppets(INTEGER_32 scene_To_Use, CARDINAL_32 start_Offset_Into_Voices, Boolean reset_Voices)
{
	Safe_Boolean		function_Result;
	INTEGER_16		voice;
	INTEGER_16		position;
	SIMPLE_LOOP	loop;
	Boolean				valid;
	CARDINAL_32	max_Sorted_Items;
	Boolean				found_Voice;
	CARDINAL_32	voices_Used;
	CARDINAL_32	desired_Voice;
	Stage_Item_Ptr	item_Ptr;
	CARDINAL_8		*code_Name;
/* END VAR */


	code_Name = "\pAttach_Default_Inputs_To_Scene_Puppets";
	function_Result = SAFE_FALSE;
	
	if (g_Dreaming_Module_Globals_Initialized != SAFE_TRUE) {
		function_Result = SAFE_TRUE;
		goto Normal_Function_Exit;
	}	
	
	if (dream.play.total_Scenes < 1) {
		function_Result = SAFE_TRUE;
		goto Normal_Function_Exit;
	}
	
	if (dream.play.scene_Memory_In_Use[scene_To_Use] == MEMORY_INITIALIZED) {

		if ((**(dream.play.scene[scene_To_Use])).initialized == SAFE_TRUE) {

			max_Sorted_Items = (**(dream.play.scene[scene_To_Use])).total_Items_In_Scene;

			desired_Voice = start_Offset_Into_Voices;
			voices_Used = 0;
			for (position = 0; (position < max_Sorted_Items); position++ ) {
			
				/*  */
				/* HACK: We should let the user select voice inputs, but */
				/* use default each input located and apportioned out. */
				/*  */
				if (g_Voice_Globals_Initialized == SAFE_TRUE) {
					if (g_Voices.total_Voices_Active > 0) {
											
						found_Voice = FALSE;
						for (loop = desired_Voice
												; ((loop < k_MAXIMUM_VOICE_MONITORS) 
												&& (voices_Used < g_Voices.total_Voices_Active) 
												&& (found_Voice != TRUE))
															; loop++) {
												
							if (g_Voices.voice_Active[loop] == SAFE_TRUE) {
								desired_Voice = loop;
								voices_Used += 1;
								found_Voice = TRUE;
							}
						}
						
						if (found_Voice != TRUE) {
							desired_Voice = 0;
							voices_Used = 0;
							
							found_Voice = FALSE;
							for (loop = desired_Voice
													; ((loop < k_MAXIMUM_VOICE_MONITORS) 
													&& (voices_Used < g_Voices.total_Voices_Active) 
													&& (found_Voice != TRUE))
																; loop++) {
													
								if (g_Voices.voice_Active[loop] == SAFE_TRUE) {
									desired_Voice = loop;
									voices_Used += 1;
									found_Voice = TRUE;
								}
							}
						}						
					}
				}
			
			
				valid = FALSE;
				if ((**(dream.play.scene[scene_To_Use])).item[position].puppet != nil) {
					if ((**(dream.play.scene[scene_To_Use])).item[position].valid_Pointer == SAFE_TRUE) {
						valid = TRUE;
					}
					else {
						Display_Serious_Error_Message(unit_Name, code_Name, "\pCorrupt Pointer (3)", "\pBad program");
						(**(dream.play.scene[scene_To_Use])).item[position].puppet = nil;
					}
				}

				if (valid == TRUE) {

					/*  */
					/* NOTICE: This assumes it is already locked down */
					/*  */
					item_Ptr = &((**(dream.play.scene[scene_To_Use])).item[position]);
			
					if (reset_Voices == TRUE) {
						if (item_Ptr->controllers_Attached == SAFE_TRUE) {
							if (Force_Scene_Item_To_Mute_Status(item_Ptr) != SAFE_TRUE) {
								/*  */
								/* Not important to worry about failures of this command */
								/*  */
							}
						}
					}
					
					if (item_Ptr->controllers_Attached != SAFE_TRUE) {
						
						if (item_Ptr->voice != nil) {
							Display_Serious_Error_Message(unit_Name, code_Name, "\pWarning, not clearing 'controllers_Attached'", "\pBad programming, remedied.");
							item_Ptr->voice = nil;
						}
						
						if (Force_Scene_Item_To_Mute_Status(item_Ptr) != SAFE_TRUE) {
							/*  */
							/* Not important to worry about failures of this command */
							/*  */
						}

						if (item_Ptr->voice == nil) {						

							if (g_Voices.voice_Active[desired_Voice] == SAFE_TRUE) {
								item_Ptr->voice = g_Voices.voice[desired_Voice];
								item_Ptr->controllers_Attached = SAFE_TRUE;
								desired_Voice += 1;
								
							}

						}
						
					} /* if (item_Ptr->controllers_Attached == SAFE_TRUE)  */

				} /* if ((**(dream.play.scene[scene_To_Use])).item.puppet != nil) */
				
			} /* for (position = 0; (position < max_Sorted_Items)  ... */
			
		} /* if ((**(dream.play.scene[scene_To_Use])).initialized... */
		
	} /*if (dream.play.scene_Memory_In_Use[scene_To_Use] ... */
				
	
	function_Result = SAFE_TRUE;
	goto Normal_Function_Exit;
	
Error_Handler:
	function_Result = SAFE_FALSE;
		
Normal_Function_Exit:

	return(function_Result);
	
} /* Safe_Boolean Attach_Default_Inputs_To_Scene_Puppets ... */
/*=**************************************************************************=*/


/*=**************************************************************************=*/
Safe_Boolean Delete_A_Specific_Input_From_Any_Scene_Puppets(Voice_Hdl voice_To_Eliminate_Soon_Hdl)
{
	Safe_Boolean		function_Result;
/* END VAR */


	function_Result = Swap_Or_Delete_An_Input_From_Any_Scene_Puppets(voice_To_Eliminate_Soon_Hdl, SAFE_FALSE, nil);
	
	return(function_Result);
}
/*=**************************************************************************=*/


/*=**************************************************************************=*/
Safe_Boolean Swap_Or_Delete_An_Input_From_Any_Scene_Puppets(Voice_Hdl voice_To_Eliminate_Soon_Hdl
																														, Safe_Boolean		swap_With_Another_Voice
																														, Voice_Hdl voice_To_Swap_With_Hdl)
{
	Safe_Boolean		function_Result;
	INTEGER_16		position;
	SIMPLE_LOOP	loop;
	Boolean				valid;
	CARDINAL_32	located_Scenes;
	CARDINAL_32	scene_To_Use;
	CARDINAL_32	max_Sorted_Items;
	Boolean				found_Voice;
	CARDINAL_32	voices_Used;
	CARDINAL_32	desired_Voice;
	CARDINAL_32	located_Items;
	Stage_Item_Ptr	item_Ptr;
	CARDINAL_8		*code_Name;
/* END VAR */


	code_Name = "\pDelete_A_Specific_Input_From_Any_Scene_Puppets";
	function_Result = SAFE_FALSE;
	
	if (g_Dreaming_Module_Globals_Initialized != SAFE_TRUE) {
		function_Result = SAFE_TRUE;
		goto Normal_Function_Exit;
	}	
	
	if (dream.play.total_Scenes < 1) {
		function_Result = SAFE_TRUE;
		goto Normal_Function_Exit;
	}

	if (swap_With_Another_Voice == SAFE_TRUE) {
		/*  */
		/* Validate that the voice we are swapping exists else set */
		/* items to no sound connection. */
		/*  */
		swap_With_Another_Voice = SAFE_FALSE;
		if ((voice_To_Swap_With_Hdl != nil) && (*voice_To_Swap_With_Hdl != nil)) {
			if ((**voice_To_Swap_With_Hdl).initialized == SAFE_TRUE) {
				swap_With_Another_Voice = SAFE_TRUE;
			}
		}
	}


	located_Scenes = 0;
	for (scene_To_Use = 0; ((scene_To_Use < k_MAX_ACTIVE_SCENES) && (located_Scenes < dream.play.total_Scenes)); scene_To_Use++) {
		if (dream.play.scene_Memory_In_Use[scene_To_Use] == MEMORY_INITIALIZED) {
		
			located_Scenes += 1;
	
			if ((**(dream.play.scene[scene_To_Use])).initialized == SAFE_TRUE) {
	
				max_Sorted_Items = (**(dream.play.scene[scene_To_Use])).total_Items_In_Scene;
	
				located_Items = 0;
				for (position = 0; ((position < k_MAX_STAGE_ITEMS) && (located_Items < max_Sorted_Items)); position++ ) {
					located_Items += 1;
		
					/*  */
					/* NOTICE: This assumes it is already locked down */
					/*  */
					item_Ptr = &((**(dream.play.scene[scene_To_Use])).item[position]);
	
					if (item_Ptr->controllers_Attached == SAFE_TRUE) {
		
						if (item_Ptr->voice != nil) {
							if (*(item_Ptr->voice) == *voice_To_Eliminate_Soon_Hdl) {
							
								if (swap_With_Another_Voice == SAFE_TRUE) {
								
									/*  */
									/* Swapping is usually performed whenever a sound source is */
									/* temporarily swapped for input from an AIFF file. */
									/*  */
									
									item_Ptr->voice = voice_To_Swap_With_Hdl;

								}
								else {
									if (Force_Scene_Item_To_Mute_Status(item_Ptr) != SAFE_TRUE) {
										/*  */
										/* Not important to worry about failures of this command */
										/*  */
									}
								}
							}
						}
					}
				}
			}
		}
	}

	
	function_Result = SAFE_TRUE;
	goto Normal_Function_Exit;
	
Error_Handler:
	function_Result = SAFE_FALSE;
		
Normal_Function_Exit:

	return(function_Result);
	
} /* Safe_Boolean Delete_A_Specific_Input_From_Any_Scene_Puppets ... */
/*=**************************************************************************=*/


/*=**************************************************************************=*/
pascal void Inform_Main_Routine_Audio_File_Is_Closing(Async_AIFF_Ptr file_Control_Ptr)
{
/*  */
/* This is for placing utility routines that cleanup when an audio file */
/* is closed. It is for detatching voicees to items in scenes that */
/* might have been controlled by AIFF files momentarily and need */
/* closure. It is called several milliseconds after the file has stopped playing, */
/* because this routine is called only in response to polling. */
/*  */
	CARDINAL_32	max_Items;
	CARDINAL_32	total_Items_Seen;
	CARDINAL_32	playback_Index;
	CARDINAL_32	original_Voice_Index;
	Boolean				still_Searching;
/* END VAR */



	if (Find_Structure_Index_For_AIFF_Pointer(file_Control_Ptr, &playback_Index) == SAFE_TRUE) {
	
		/*  */
		/* Now that we know the index we need to hunt through */
		/* all voices looking for any that are using that index and need detatchment. */
		/*  */
	
		if (g_Voices.total_Voices_Active > 0) {
		
			still_Searching = TRUE;
			max_Items = g_Voices.total_Voices_Active;
			total_Items_Seen = 0;

			for (original_Voice_Index=0; ((original_Voice_Index < k_MAXIMUM_VOICE_MONITORS) 
													&& (total_Items_Seen < max_Items) 
													&& (still_Searching == TRUE)); original_Voice_Index++) {
													
				if (g_Voices.voice_Memory_Initialized[original_Voice_Index] == SAFE_TRUE) {
					total_Items_Seen += 1;
					
					if ((**g_Voices.voice[original_Voice_Index]).audio_Source == LOCAL_AUDIO_FILE_PLAYING) {
					
						if ((**g_Voices.voice[original_Voice_Index]).info.audio_File.initialized == SAFE_TRUE) {
						
							/*  */
							/* If it is our sound then detatch it (all instances ) */
							/*  */
							if ((**g_Voices.voice[original_Voice_Index]).info.audio_File.playback_Index == playback_Index) {
							
								still_Searching = FALSE;
								
								if ((**g_Voices.voice[original_Voice_Index]).info.audio_File.previous_Open_Voice_Index_Valid == SAFE_TRUE) {
									if (g_Voices.voice_Memory_Initialized[(**g_Voices.voice[original_Voice_Index]).info.audio_File.previous_Open_Voice_Index] == SAFE_TRUE) {
								
										/*  */
										/* Swap back any pointers possible still. */
										/*  */
										if (Swap_Or_Delete_An_Input_From_Any_Scene_Puppets(g_Voices.voice[original_Voice_Index]
																						, SAFE_TRUE
																						, g_Voices.voice[(**g_Voices.voice[original_Voice_Index]).info.audio_File.previous_Open_Voice_Index]) != SAFE_TRUE) {
											
											/*  */
											/* Ignore Error. */
											/*  */
										}
										
										(**g_Voices.voice[original_Voice_Index]).info.audio_File.previous_Open_Voice_Index_Valid = SAFE_FALSE;
										(**g_Voices.voice[original_Voice_Index]).info.audio_File.tap_Possibly_Active = SAFE_FALSE;
										
										
										/*  */
										/* Close the voice. We are done with it. */
										/*  */
										
										if (Shutdown_Voice(original_Voice_Index, LOCAL_AUDIO_FILE_PLAYING) != SAFE_TRUE) {
											/*  */
											/* Ignore error, if any in this version. */
											/*  */
										}
										
									}
								}
							}
						}
					}
				}
			}
		}	
		
	} /* if (Find_Structure_Index_For_AIFF_Pointer(file_Control_Ptr, &playback_Index) == SAFE_TRUE) */

	
} /* pascal void Inform_Main_Routine_Audio_File_Is_Closing... */
/*=**************************************************************************=*/


/*=**************************************************************************=*/
Safe_Boolean Revert_All_Voices_In_Items_To_Paused_Original_Sources(Voice_Hdl original_Voice)
/*  */
/* This is only a maintenance routine for AIFF voice sources */
/*  */
{
	Safe_Boolean		function_Result;
/* END VAR */


	if ((**original_Voice).audio_Source == LOCAL_AUDIO_FILE_PLAYING) {
	

		/*  */
		/* Interesting bit : swap previous voice to save back to */
		/* current level, just as if the file finished by itself. It will be saved again next */
		/* If this is being closed in order to play another file and preempt this one. */
		/*  */
		
		if ((**original_Voice).info.audio_File.previous_Open_Voice_Index_Valid == SAFE_TRUE) {
		
			/*  */
			/* hunt through all items in all scenes */
			/*  */
			
			if (g_Voices.total_Voices_Active > 0) {
				if (g_Voices.voice_Memory_Initialized[(**original_Voice).info.audio_File.previous_Open_Voice_Index] == SAFE_TRUE) {
								
					if (Swap_Or_Delete_An_Input_From_Any_Scene_Puppets(original_Voice
																	, SAFE_TRUE
																	, g_Voices.voice[(**original_Voice).info.audio_File.previous_Open_Voice_Index]) != SAFE_TRUE) {
						
						/*  */
						/* Ignore Error. */
						/*  */
					}
				}
			}
			
			/*  */
			/* Its done. No more need of this redirection info. */
			/*  */
			(**original_Voice).info.audio_File.previous_Open_Voice_Index_Valid = SAFE_FALSE;
			
		} /* if ((**original_Voice).info.audio_File.previous_Open_Voice_Index_Valid == SAFE_TRUE) */
		
	} /* if ((**original_Voice).audio_Source == LOCAL_AUDIO_FILE_PLAYING) */
	
	function_Result = SAFE_TRUE;
	
	return(function_Result);
	
} /* Safe_Boolean Revert_All_Voices_In_Items_To_Paused_Original_Sources */
/*=**************************************************************************=*/


/*=**************************************************************************=*/
Safe_Boolean Notify_Sync_Audio_Problem_Found_If_First_Time(void)
{
	CARDINAL_8		*code_Name;
/* END VAR */


	code_Name = "\pNotify_Sync_Audio_Problem_Found_If_First_Time";

	if (g_Severe_Audio_Record_Problem_Shown != SAFE_TRUE) {
	
		Display_Serious_Error_Message(unit_Name, code_Name, "\pWarning, severe problem encounterred in updating audio recording", "\pBad programming?, not remedied.");
		
		g_Severe_Audio_Record_Problem_Shown = SAFE_TRUE;
	}

	return(SAFE_TRUE);
	
}
/*=**************************************************************************=*/


/*=**************************************************************************=*/
Safe_Boolean Notify_Async_Audio_Problem_Found_If_First_Time(void)
{
	CARDINAL_8		*code_Name;
/* END VAR */


	code_Name = "\pNotify_Async_Audio_Problem_Found_If_First_Time";

	if (g_Severe_Audio_Record_Problem_Needs_Display_Soon == SAFE_TRUE) {
	
		if (g_Severe_Audio_Async_Record_Problem_Shown != SAFE_TRUE) {
		
			Display_Serious_Error_Message(unit_Name, code_Name, "\pWarning, a problem was encounterred in starting a request to record to a buffer asyncronously from an asyncronous completion routine", "\pBad SID driver programming? Bad System? Use another SID.");
			
			g_Severe_Audio_Async_Record_Problem_Shown = SAFE_TRUE;
		}
		g_Severe_Audio_Record_Problem_Needs_Display_Soon = SAFE_FALSE;
	}
	
	return(SAFE_TRUE);
	
}
/*=**************************************************************************=*/


/*=**************************************************************************=*/
Safe_Boolean Update_Sound_Input_Monitoring(void)
{
	CARDINAL_32	max_Items;
	CARDINAL_32	total_Items_Seen;
	Safe_Boolean		function_Result;
	INTEGER_16		voice;
/* END VAR */


	function_Result = SAFE_FALSE;

	if (g_Voices.initialized == SAFE_TRUE) {
		if (g_Voices.total_Voices_Active > 0) {
		
			max_Items = g_Voices.total_Voices_Active;
			total_Items_Seen = 0;
			for (voice=0; ((voice < k_MAXIMUM_VOICE_MONITORS) && (total_Items_Seen < max_Items)) ; voice++) {
				if (g_Voices.voice_Memory_Initialized[voice] == SAFE_TRUE) {
					total_Items_Seen += 1;
					
					if (Process_Sound_Updating(g_Voices.voice[voice]) != SAFE_TRUE) {
					
						if (Notify_Sync_Audio_Problem_Found_If_First_Time() != SAFE_TRUE) {
							/*  */
							/* Ignore error.*/
							/*  */
						}
											
						if (Shutdown_Voice(voice, ANY_INPUT_AT_ALL) != SAFE_TRUE) {
							/*  */
							/* Error exception not implemented. No need to? */
							/*  */
						}
					}
					
				}
			}
			
#if (ALLOW_CARD_UPDATE_ITERATIVELY == 0)
			if (Process_Sound_Updating_From_Multiplexed_Lines() != SAFE_TRUE) {
				/*  */
				/* Ignore failures. Lines closed and dealt with already */
				/*  */
			}
#endif
		}
	}
	
	if (g_Severe_Audio_Record_Problem_Needs_Display_Soon == SAFE_TRUE) {
		Notify_Async_Audio_Problem_Found_If_First_Time();
	}

	function_Result = SAFE_TRUE;
	goto Normal_Function_Exit;
	
Error_Handler:
	function_Result = SAFE_FALSE;
		
Normal_Function_Exit:

	return(function_Result);
}
/*=**************************************************************************=*/


/*=**************************************************************************=*/
Safe_Boolean Shutdown_Sound_Input_Monitoring(void)
{
	CARDINAL_32	max_Items;
	CARDINAL_32	total_Items_Seen;
	Safe_Boolean		function_Result;
	INTEGER_16		voice;
/* END VAR */


	function_Result = SAFE_FALSE;

	if (g_Voices.initialized == SAFE_TRUE) {
		if (g_Voices.total_Voices_Active > 0) {
		
			if ((g_Voices.monitor_Microphone_For_Speech == SAFE_TRUE) 
				|| (g_Voices.monitor_Sampler_For_Speech == SAFE_TRUE)){
				
				max_Items = g_Voices.total_Voices_Active;
				total_Items_Seen = 0;
				
				for (voice=0; ((voice < k_MAXIMUM_VOICE_MONITORS) && (total_Items_Seen < max_Items)) ; voice++) {
					if (g_Voices.voice_Memory_Initialized[voice] == SAFE_TRUE) {
						total_Items_Seen += 1;
						if (Shutdown_Voice(voice, ANY_INPUT_AT_ALL) != SAFE_TRUE) {
							/*  */
							/* Error exception not implemented. No need to? */
							/*  */
						}
					}
				}
			}
		}
	}

	function_Result = SAFE_TRUE;
	goto Normal_Function_Exit;
	
Error_Handler:
	function_Result = SAFE_FALSE;
		
Normal_Function_Exit:

	return(function_Result);
}
/*=**************************************************************************=*/


/*=**************************************************************************=*/
Safe_Boolean Start_Sampler_Monitoring(Safe_Boolean *activated)
{
	Safe_Boolean		function_Result;
/* END VAR */


	*activated = SAFE_FALSE;
	
	if (Start_Sampling_From_Card(g_Voices.audio_Channels_Needed_On_Cards) != SAFE_TRUE) {
		if (Stop_Sampling_From_Card() != SAFE_TRUE) {
		
		}
		goto Error_Handler;
	}
	
	*activated = SAFE_TRUE;
	function_Result = SAFE_TRUE;
	goto Normal_Function_Exit;
	
Error_Handler:
	function_Result = SAFE_FALSE;
		
Normal_Function_Exit:

	return(function_Result);
}
/*=**************************************************************************=*/

#pragma mark -

/*=**************************************************************************=*/
void Convert_Audio_To_Non_Linear_Gain(CARDINAL_32 input, SPEED_DIGITS_REAL *effect_Ratio, CARDINAL_32 *output)
{
SPEED_DIGITS_REAL		temp_Value;
/* END VAR */

/*  */
/* Output is not linear it is the type Y = X + g(X - X^2) */
/* "effect_Ratio" controls deviation from x Axis */
/*  */

	temp_Value = (SPEED_DIGITS_REAL)((SPEED_DIGITS_REAL)1.0 * input);
	temp_Value = (temp_Value / (SPEED_DIGITS_REAL)256.0);
	
	temp_Value = ((temp_Value) + (((SPEED_DIGITS_REAL)*effect_Ratio) * (temp_Value - (temp_Value * temp_Value))));
	temp_Value = temp_Value * (SPEED_DIGITS_REAL)256.0;
	*output = (CARDINAL_32)temp_Value;
}
/*=**************************************************************************=*/


/*=**************************************************************************=*/
Safe_Boolean Extract_Recent_Amplitude_From_Multiplexed_Lines(CARDINAL_32	*raw_Signal)
/*  */
/* A real time saver. This uses multiplexed data and extracts from interlaced stream in one pass */
/*  */
{
	CARDINAL_32		large_Positive_Summation;
	INTEGER_32			large_Signed_Deviance;
	INTEGER_32			software_Eighths_Amplifier;
	SIMPLE_LOOP		loop;
	CARDINAL_32		samples_Grabbed;
	Ptr							response_Ptr;
	OSErr						os_Error;
	Boolean					no_New_Data_Yet;
	CARDINAL_16		audio_Source_Channel;
	CARDINAL_16		*mini_Buffer_Sample_Ptr;
	Safe_Boolean			function_Result;
/* END VAR */


	function_Result = SAFE_FALSE;
	
	mini_Buffer_Sample_Ptr = (CARDINAL_16 *)&(g_Sample_Ptr[0]);
			
	samples_Grabbed = (k_MAXIMUM_MINI_SAMPLES * (CARDINAL_32)g_Voices.audio_Channels_Needed_On_Cards);

	/*  */
	/* Always fetch using phase of "0" for channel */
	/*  */
	if (Fetch_Tap_Of_Digital_Audio_Card(0
																, mini_Buffer_Sample_Ptr
																, samples_Grabbed
																, &no_New_Data_Yet
																) != SAFE_TRUE) {
		/*  */
		/* IGNORE ERRORS IN THIS VERSION */
		/*  */
		goto Error_Handler;
	}
	else {
	
		if (no_New_Data_Yet != TRUE) {

			for(audio_Source_Channel = 0; (audio_Source_Channel < g_Voices.audio_Channels_Needed_On_Cards); audio_Source_Channel++) {
				/*  */
				/* Calculate each sound line whether or not active in a scene */
				/* Optimization to avoid this can be made some other time */
				/*  */
			
				large_Positive_Summation = 0;
					
				/*  */
				/* Gotta ignore demuxed unwanted values  */
				/*  */
				for (loop= audio_Source_Channel; (loop < samples_Grabbed); loop+= g_Voices.audio_Channels_Needed_On_Cards) {
					large_Signed_Deviance = g_Sample_Ptr[loop];
					if (large_Signed_Deviance < 0) {
						large_Signed_Deviance = (- large_Signed_Deviance);
					}
					
					if (large_Signed_Deviance > 0x07FF) {
						large_Signed_Deviance = 0x07FF;
					}
					large_Positive_Summation += large_Signed_Deviance;
				}
										
				software_Eighths_Amplifier = k_SOFTWARE_EIGHTHS_AMPLIFIER_FACTOR;
				
				if (software_Eighths_Amplifier != 0) {
					large_Positive_Summation
								= (large_Positive_Summation * (software_Eighths_Amplifier + 8L)) / 8L;
				}
				
				large_Positive_Summation = (large_Positive_Summation 
														/ (k_MAXIMUM_MINI_SAMPLES - k_SKIP_SAMPLES));
														
				large_Positive_Summation = (large_Positive_Summation >> k_SAMPLER_DIVISOR);
														
				/*  */
				/* Adjust high point, clip to top bounds if needed */
				/*  */
				if (large_Positive_Summation > 0x00FF) {
					large_Positive_Summation = 0x00FF;
				}
				raw_Signal[audio_Source_Channel] = large_Positive_Summation;
			}
			
		}
		else {
			goto Error_Handler;
		}
	}
	
	
	function_Result = SAFE_TRUE;
	goto Normal_Function_Exit;
	
Error_Handler:
	function_Result = SAFE_FALSE;
		
Normal_Function_Exit:

	return(function_Result);
}
/*=**************************************************************************=*/


/*=**************************************************************************=*/
Safe_Boolean Extract_Recent_Amplitude(Voice_Hdl	voice, CARDINAL_32	*raw_Signal)
{
	Safe_Boolean			function_Result;
/* END VAR */


	function_Result = SAFE_FALSE;
		
	*raw_Signal = 0;

	switch ((**voice).audio_Source) {
	
		case PRIMARY_SOUND_INPUT:
			function_Result = Extract_Recent_Amplitude_From_SID(voice, raw_Signal);
		break;
	
		case NATIONAL_INSTRUMENTS_DA:
			function_Result = Extract_Recent_Amplitude_From_Sampler(voice, raw_Signal);
		break;
	
		case LOCAL_AUDIO_FILE_PLAYING:
			function_Result = Extract_Recent_Amplitude_From_Audio_File(voice, raw_Signal);
		break;
	
		case REMOTE_NETWORK_FEED:
		case TOP_KEYBOARD_ROW:
		case MIDI_INPUT:
		default:
			/*  */
			/* Other sound types are not supported yet */
			/*  */
			*raw_Signal = 0;
			function_Result = SAFE_FALSE;
	}
	
	return(function_Result);

}/* Safe_Boolean Extract_Recent_Amplitude */
/*=**************************************************************************=*/


/*=**************************************************************************=*/
Safe_Boolean Extract_Recent_Amplitude_From_Audio_File(Voice_Hdl	voice, CARDINAL_32	*raw_Signal)
{
	Safe_Boolean			function_Result;
	CARDINAL_32		channel_Desired;
	CARDINAL_32		second_Ratio_For_Window;
	CARDINAL_8 			amplitude;
	CARDINAL_16		buffer_Index;
	CARDINAL_32		total_Buffers_Available;
	Async_AIFF_Ptr		file_Playback_Status;
/* END VAR */


	function_Result = SAFE_FALSE;
			
	if ((**voice).audio_Source == LOCAL_AUDIO_FILE_PLAYING) {
		
		if ((**voice).info.audio_File.initialized != SAFE_TRUE) {
			goto Error_Handler;
		}
		
		if ((**voice).info.audio_File.tap_Possibly_Active != SAFE_TRUE) {
			/*  */
			/* It either finished, never started, or something fatal. */
			/*  */
			goto Error_Handler;
		}
				
		if (g_AIFF_Structures_Opened <= 0) {
			goto Error_Handler;
		}
		
		file_Playback_Status = &g_Playback[(**voice).info.audio_File.playback_Index];
		
		if (file_Playback_Status->structure_Initialized != SAFE_TRUE) {
			goto Error_Handler;
		}

		if (file_Playback_Status->audio_Tap_Initialized != SAFE_TRUE) {
			/*  */
			/* No tap was installed, though file did play audio. This happens if */
			/* dangerously low memory condition prevents creationg of 64K tap. */
			/*  */
			goto Error_Handler;
		}

		if (file_Playback_Status->audio_Tap_Ptr->info.active_And_In_Use != SAFE_TRUE) {
			/*  */
			/* Tap is pointing to dead item. Not expected! */
			/*  */
			goto Error_Handler;
		}

		if (file_Playback_Status->playing_AIFF_Now != SAFE_TRUE) {
			/*  */
			/* Its finished! We should skip all logic if more than 12 seconds has passed */
			/* This is an orphaned voice channel that was never closed after completion*/
			/* because of programming error or starvation of periodic-time cleanup task */
			/* that closes files at non-interrupt time. */
			/* We trust using buffer otherwise because even though file is closed the last buffer */
			/* has not finished being fed into the sound channel. */
			/*  */
			
			if (file_Playback_Status->audio_Tap_Ptr->info.tap_Buffer_Total_Samples_Counted == 0) {
				goto Error_Handler;
			}
			else {
				
				total_Buffers_Available = 2;
				if (file_Playback_Status->audio_Tap_Ptr->info.tap_Buffer[1].tick_Time_Last_Called == 0) {
					/*  */
					/* Eliminate 1 in 4 billion chance of error when Mac running many days */
					/* by looking at a second field. */
					/*  */
					if (file_Playback_Status->audio_Tap_Ptr->info.tap_Buffer[1].component_Recent_Info.sampleCount == 0) {
						total_Buffers_Available = 1;
					}
				}
				
				if (total_Buffers_Available > 1) {
				
					buffer_Index = file_Playback_Status->audio_Tap_Ptr->info.next_Buffer;
					buffer_Index += 1;
					buffer_Index = buffer_Index & 0x01;

					if (Calculate_Total_Ticks(file_Playback_Status->audio_Tap_Ptr->info.tap_Buffer[buffer_Index].tick_Time_Last_Called, TickCount()) > (60L * 12L)) {
						goto Error_Handler;
					}
				}
				else {
					if (Calculate_Total_Ticks(file_Playback_Status->audio_Tap_Ptr->info.tap_Buffer[0].tick_Time_Last_Called, TickCount()) > (60L * 12L)) {
						goto Error_Handler;
					}
				}
			}
			
		} /* if (file_Playback_Status->playing_AIFF_Now != SAFE_TRUE) */

		channel_Desired = 0;
		
		/*  */
		/* Look at current recent 30th of a second. */
		/* from buffer of sound that is playing or is about to play. */
		/* Time is calculated into the buffer to find bytes needed to sample */
		/*  */
		second_Ratio_For_Window = 30;
		
		
		if (Get_Current_Amplitude_From_Tap(file_Playback_Status->audio_Tap_Ptr
														, channel_Desired
														, second_Ratio_For_Window
														, &amplitude) != SAFE_TRUE) {
			*raw_Signal = 0;
			goto Error_Handler;
		}
		
		*raw_Signal = amplitude;

	}
	else {
		/*  */
		/* This was improperly called */
		/*  */
		goto Error_Handler;
		
	} /* NOT if ((**voice).audio_Source == LOCAL_AUDIO_FILE_PLAYING) */

	function_Result = SAFE_TRUE;
	goto Normal_Function_Exit;
	
Error_Handler:
	function_Result = SAFE_FALSE;
	*raw_Signal = 0;
		
Normal_Function_Exit:

	return(function_Result);

} /* Safe_Boolean Extract_Recent_Amplitude_From_Audio_File... */
/*=**************************************************************************=*/


/*=**************************************************************************=*/
Safe_Boolean Extract_Recent_Amplitude_From_Sampler(Voice_Hdl	voice, CARDINAL_32	*raw_Signal)
{
	Safe_Boolean			function_Result;
	CARDINAL_32		samples_Grabbed;
	INTEGER_16			meter_Level;
	CARDINAL_32		total_Samples_To_Record;
	CARDINAL_32		number_Samples_Recorded;
	CARDINAL_32		total_Milliseconds_To_Record;
	CARDINAL_32		number_Msecs_Recorded;
	CARDINAL_32		large_Positive_Summation;
	INTEGER_32			large_Signed_Deviance;
	INTEGER_32			software_Eighths_Amplifier;
	SIMPLE_LOOP		loop;
	Ptr							response_Ptr;
	OSErr						os_Error;
	Boolean					no_New_Data_Yet;
	CARDINAL_16		*mini_Buffer_Sample_Ptr;
/* END VAR */


	function_Result = SAFE_FALSE;
		
	*raw_Signal = 0;
	
	if ((**voice).audio_Source == NATIONAL_INSTRUMENTS_DA) {
		
#if (ALLOW_CARD_UPDATE_ITERATIVELY == 1)

		mini_Buffer_Sample_Ptr = (CARDINAL_16 *)&(g_Sample_Ptr[0]);
				
		samples_Grabbed = (k_MAXIMUM_MINI_SAMPLES * (CARDINAL_32)g_Voices.audio_Channels_Needed_On_Cards);
		
		if (Fetch_Tap_Of_Digital_Audio_Card((**voice).audio_Source_Channel
																	, mini_Buffer_Sample_Ptr
																	, samples_Grabbed
																	, &no_New_Data_Yet
																	) != SAFE_TRUE) {
			/*  */
			/* IGNORE ERRORS IN THIS VERSION */
			/*  */
			goto Error_Handler;
		}
		else {
		
			if (no_New_Data_Yet != TRUE) {
			
				/*  */
				/*  */
				/*  */
				/*  */
				/*  */
				/* AVERAGING 100 SAMPLES, CONVERTING NEGATIVES */
				/* starting from the fifth sample only */
				/*  */
				
				/*  */
				/* HACK HACK HACK : It is better to use EOR operation to */
				/* just flip high bits of all guard bits plus MSBit!!!! */
				/* Rewrite all this code after proving this is so. */
				/* To flip high bit in an 8 bit byte (not 12 bit in a 16 bit as we have) */
				/* you would write the C code as : */
				/* "data_Sample ^= 0x80;" to convert to Apple storage from two's complement. */
				/*  */
				
				
				large_Positive_Summation = 0;
					
				if ((k_POSITIVE_SIDE_SKEW !=  0 ) || (k_SKIP_SAMPLES != 0)) {
					for (loop= (k_SKIP_SAMPLES + 0); (loop < samples_Grabbed); loop++) {
						if (g_Sample_Ptr[loop] > (2047 + k_POSITIVE_SIDE_SKEW)) {
							
							/*  */
							/* not expected ever */
							/*  */
						
							large_Signed_Deviance = (g_Sample_Ptr[loop] - (2047 + k_POSITIVE_SIDE_SKEW));
							if (large_Signed_Deviance > 0x07FF) {
								large_Signed_Deviance = 0x07FF;
							}
							large_Positive_Summation += large_Signed_Deviance;
						}
						else {
							if (g_Sample_Ptr[loop] >= k_POSITIVE_SIDE_SKEW) {
								large_Signed_Deviance = g_Sample_Ptr[loop] - k_POSITIVE_SIDE_SKEW;
							}
							else {
								large_Signed_Deviance = (- g_Sample_Ptr[loop]) + k_POSITIVE_SIDE_SKEW;
							}
							
							if (large_Signed_Deviance > 0x07FF) {
								large_Signed_Deviance = 0x07FF;
							}
							large_Positive_Summation += large_Signed_Deviance;
						}
					}
				}
				else {
					/*  */
					/* Traditional loop through data */
					/*  */
					if (g_Voices.audio_Channels_Needed_On_Cards == 0) {
						for (loop= 0; (loop < samples_Grabbed); loop++) {
							large_Signed_Deviance = g_Sample_Ptr[loop];
							if (large_Signed_Deviance < 0) {
								large_Signed_Deviance = (- large_Signed_Deviance);
							}
							
							if (large_Signed_Deviance > 0x07FF) {
								large_Signed_Deviance = 0x07FF;
							}
							large_Positive_Summation += large_Signed_Deviance;
						}
					}
					else {
					
						/*  */
						/* Gotta ignore demuxed unwanted values  */
						/*  */
						
						for (loop= (**voice).audio_Source_Channel; (loop < samples_Grabbed); loop+= g_Voices.audio_Channels_Needed_On_Cards) {
							large_Signed_Deviance = g_Sample_Ptr[loop];
							if (large_Signed_Deviance < 0) {
								large_Signed_Deviance = (- large_Signed_Deviance);
							}
							
							if (large_Signed_Deviance > 0x07FF) {
								large_Signed_Deviance = 0x07FF;
							}
							large_Positive_Summation += large_Signed_Deviance;
						}
					
					
					}
				}
								
				software_Eighths_Amplifier = k_SOFTWARE_EIGHTHS_AMPLIFIER_FACTOR;
				
				
				if (software_Eighths_Amplifier != 0) {
					large_Positive_Summation
								= (large_Positive_Summation * (software_Eighths_Amplifier + 8L)) / 8L;
				}
				
				large_Positive_Summation = (large_Positive_Summation 
														/ (k_MAXIMUM_MINI_SAMPLES - k_SKIP_SAMPLES));
														
				large_Positive_Summation = (large_Positive_Summation >> k_SAMPLER_DIVISOR);
														
				/*  */
				/* Adjust high point, clip to top bounds if needed */
				/*  */
				if (large_Positive_Summation > 0x00FF) {
					large_Positive_Summation = 0x00FF;
				}
				*raw_Signal = large_Positive_Summation;
			}
			else {
				goto Error_Handler;
			}
		} 
#endif /* (ALLOW_CARD_UPDATE_ITERATIVELY == 1) */
		
	}
	else {
		/*  */
		/* This was improperly called */
		/*  */
		goto Error_Handler;
		
	} /* NOT if ((**voice).audio_Source == NATIONAL_INSTRUMENTS_DA) */

	function_Result = SAFE_TRUE;
	goto Normal_Function_Exit;
	
Error_Handler:
	function_Result = SAFE_FALSE;
	*raw_Signal = 0;
		
Normal_Function_Exit:

	return(function_Result);

} /* Safe_Boolean Extract_Recent_Amplitude_From_Sampler... */
/*=**************************************************************************=*/


/*=**************************************************************************=*/
Safe_Boolean Process_Sound_Updating_From_Multiplexed_Lines(void)
{
	CARDINAL_32	max_Items;
	CARDINAL_32	total_Items_Seen;
	Safe_Boolean		function_Result;
	CARDINAL_32	current_Time;
	CARDINAL_16	voice;
	Boolean				searching_For_Card_Support;
	INTEGER_8			source_Channel;
	Voice_Ptr			current_Voice;
	CARDINAL_32	raw_Signals[16];
/* END VAR */


	function_Result = SAFE_FALSE;

	current_Time = TickCount();
	if (g_Voices.tickCount_MIO_DAQ_Last_Updated + k_TICK_DELAY_BETWEEN_ANALOG_CARD_TAPPING <= current_Time) {
	
			searching_For_Card_Support = TRUE;
			max_Items = g_Voices.total_Voices_Active;
			total_Items_Seen = 0;
			for (voice=0; ((voice < k_MAXIMUM_VOICE_MONITORS)
								&& (total_Items_Seen < max_Items) 
								&& (searching_For_Card_Support == TRUE)); voice++) {
								
				if (g_Voices.voice_Memory_Initialized[voice] == SAFE_TRUE) {
					total_Items_Seen += 1;

					if ((**(g_Voices.voice[voice])).audio_Source == NATIONAL_INSTRUMENTS_DA) {
						searching_For_Card_Support = FALSE;
					}
				}
			}
	
		if (searching_For_Card_Support != TRUE) {
			if (Extract_Recent_Amplitude_From_Multiplexed_Lines(&(raw_Signals[0])) == SAFE_TRUE) {
				max_Items = g_Voices.total_Voices_Active;
				total_Items_Seen = 0;
				for (voice=0; ((voice < k_MAXIMUM_VOICE_MONITORS) && (total_Items_Seen < max_Items)) ; voice++) {
				
					if (g_Voices.voice_Memory_Initialized[voice] == SAFE_TRUE) {
						total_Items_Seen += 1;
						if ((**(g_Voices.voice[voice])).audio_Source == NATIONAL_INSTRUMENTS_DA) {
						
							current_Voice = (*(g_Voices.voice[voice]));
							
							source_Channel = (*current_Voice).audio_Source_Channel;
							
							if (raw_Signals[source_Channel] > (*current_Voice).Max_Signal_Range) {
								raw_Signals[source_Channel] =(*current_Voice).Max_Signal_Range;
							}
							
							if ((raw_Signals[source_Channel] <= ((*current_Voice).current_Raw_Signal) + k_REDUCTION_JITTER_ALLOWANCE)
								&& (raw_Signals[source_Channel] < k_ALMOST_MAX_VOLUME)){
							
								(*current_Voice).signals_Dropping_Consecutively += 1;
								
								if ((*current_Voice).signals_Dropping_Consecutively > k_ITERATIONS_BEFORE_DROP_INTERPOLATION) {
									if ((*current_Voice).current_Interpolated_Signal > k_SECOND_STAGE_REDUCTION) {
										(*current_Voice).current_Interpolated_Signal = (*current_Voice).current_Interpolated_Signal - k_SECOND_STAGE_REDUCTION;
									}
									else {
										(*current_Voice).current_Interpolated_Signal = 0;
									}
								}
								else if ((*current_Voice).signals_Dropping_Consecutively >= k_ITERATIONS_BEFORE_DROP_INTERPOLATION) {
									if (raw_Signals[source_Channel] > k_FIRST_STAGE_REDUCTION) {
										(*current_Voice).current_Interpolated_Signal = raw_Signals[source_Channel] - k_FIRST_STAGE_REDUCTION;
									}
									else {
										(*current_Voice).current_Interpolated_Signal = 0;
									}
								}
								else {
									(*current_Voice).current_Interpolated_Signal = raw_Signals[source_Channel];
								}
								
							
							}
							else {
								(*current_Voice).signals_Dropping_Consecutively = 0;
								(*current_Voice).current_Interpolated_Signal = raw_Signals[source_Channel];
							}
							
							(*current_Voice).current_Raw_Signal = raw_Signals[source_Channel];
							
							/*  */
							/* Experiment only.... only accumulate maximum_Signal_Since_Last */
							/*  */
							if (raw_Signals[source_Channel] > (*current_Voice).maximum_Signal_Since_Last) {
								(*current_Voice).maximum_Signal_Since_Last = raw_Signals[source_Channel];
							}
											
							current_Voice->tickCount_Last_Updated = current_Time;
						}
					}
				}
				
				g_Voices.tickCount_MIO_DAQ_Last_Updated = current_Time;
			}
			else {
				/*  */
				/* We could quit at this point, but one not too rare reason there was a failure */
				/* is because the buffer just recently faetched and there is no new levels to report */
				/*  */
				;
			}
		}
	}


	function_Result = SAFE_TRUE;
	goto Normal_Function_Exit;
	
Error_Handler:
	function_Result = SAFE_FALSE;
			
Normal_Function_Exit:
	return(function_Result);
	
} /* Safe_Boolean Process_Sound_Updating_From_Multiplexed_Lines... */
/*=**************************************************************************=*/


/*=**************************************************************************=*/
Safe_Boolean Process_Sound_Updating(Voice_Hdl	voice)
{
	Safe_Boolean			function_Result;
/* END VAR */


	function_Result = SAFE_FALSE;
	
	if ((**voice).initialized != SAFE_TRUE) {
		goto Error_Handler;
	}

	switch ((**voice).audio_Source) {
	
		case PRIMARY_SOUND_INPUT:
			function_Result = Process_Sound_Updating_For_SID(voice);
		break;
	
		case NATIONAL_INSTRUMENTS_DA:
			function_Result = Process_Sound_Updating_For_Sampler(voice);
		break;
	
		case LOCAL_AUDIO_FILE_PLAYING:
			function_Result = Process_Sound_Updating_For_Audio_File(voice);
		break;
	
		case REMOTE_NETWORK_FEED:
		case TOP_KEYBOARD_ROW:
		case MIDI_INPUT:
		default:
			/*  */
			/* Other sound types are not supported yet */
			/*  */
			function_Result = SAFE_FALSE;
	}
	
	goto Normal_Function_Exit;
	
Error_Handler:
	function_Result = SAFE_FALSE;
			
Normal_Function_Exit:
	return(function_Result);

}/* Safe_Boolean Process_Sound_Updating(Voice_Hdl	voice) */
/*=**************************************************************************=*/


/*=**************************************************************************=*/
Safe_Boolean Process_Sound_Updating_For_SID(Voice_Hdl	voice)
{
	Safe_Boolean			function_Result;
	Recording_Ptr	monitor;
	Record_Sound_To_File_Ptr audio_Tapping_Ptr;
	CARDINAL_32	current_Time;
	CARDINAL_32	raw_Signal;
/* END VAR */


	function_Result = SAFE_FALSE;
	
	if ((**voice).initialized != SAFE_TRUE) {
		goto Error_Handler;
	}

	if ((**voice).audio_Source != PRIMARY_SOUND_INPUT) {
		goto Error_Handler;
	}
	
	if ((**voice).info.apple_Sound_Input.recording_Completed == SAFE_TRUE) {
		if (Restart_Recording_Alternating_Buffer(&((**voice).info.apple_Sound_Input)) != TRUE) {
			goto Error_Handler;
		}
	}
	else {
		current_Time = TickCount();
		
		/*  */
		/* Limits monitoring to 60 times per second maximum when set to "1" */
		/*  */
		if ((**voice).tickCount_Last_Updated + k_TICK_DELAY_BETWEEN_SID_RESAMPLING <= current_Time) {
			
			/*  */
			/* Check for buffer writing to hard disk here instead of too frequently. */
			/* be aware that "current_Time" is aging meanwhile and is not reset after this */
			/* point, though that is OK or else voice monitoring might get too infrequent. */
			/*  */
			if ((**voice).info.apple_Sound_Input.audio_Tapping_Structure_Initialized == MEMORY_INITIALIZED) {
			
				/*  */
				/* Handle is already locked by design (because interrupts need to */
				/* access its contents */
				/*  */
				audio_Tapping_Ptr = *((**voice).info.apple_Sound_Input.audio_Tapping_Structure_Hdl);
				
				if (audio_Tapping_Ptr->initialized == SAFE_TRUE) {
					if (Periodically_Save_To_New_AIFF_File_If_Needed(audio_Tapping_Ptr) != SAFE_TRUE) {
						/*  */
						/* Ignore errors, the logic later handles it all */
						/*  */
					}
					
					if (audio_Tapping_Ptr->initialized != SAFE_TRUE) {
					
						/*  */
						/* This routine deletes the handle after possible extracting information */
						/*  */

						if (Inform_Program_Of_Recording_Completion(&(**voice).info.apple_Sound_Input.audio_Tapping_Structure_Hdl
																										, &(**voice).info.apple_Sound_Input.audio_Tapping_Structure_Initialized)) {
							/*  */
							/* Ignore errors, the logic later handles it all */
							/*  */
						}
					}
					
				} /* if (audio_Tapping_Ptr->initialized == SAFE_TRUE) */
			} /* if ((**voice).info.apple_Sound_Input.audio_Tapping_Structure_Initialized == MEMORY_INITIALIZED) */
			
			if (Extract_Recent_Amplitude(voice
					, &raw_Signal) != SAFE_TRUE) {
					
				/*  */
				/* We could quit at this point, but one not too rare reason there was a failure */
				/* is because the buffer just finally finished and there is no level to report */
				/*  */
			/* ===
				(**voice).maximum_Signal_Since_Last = 0;
				(**voice).current_Raw_Signal = 0;
				(*current_Voice).current_Interpolated_Signal = 0;
			==== */
				
				/*  */
				/* DO NOT update "tickCount_Last_Updated", so that we get a */
				/* retry as soon as possible */
				/*  */
			}
			else {
				if (raw_Signal > (**voice).Max_Signal_Range) {
					raw_Signal =(**voice).Max_Signal_Range;
				}
				
				(**voice).current_Raw_Signal = raw_Signal;
				(**voice).current_Interpolated_Signal = raw_Signal;

				/*  */
				/* Experiment only.... only accumulate maximum_Signal_Since_Last */
				/*  */
				if (raw_Signal > (**voice).maximum_Signal_Since_Last) {
					(**voice).maximum_Signal_Since_Last = raw_Signal;
				}
							
				(**voice).tickCount_Last_Updated = current_Time;
			}
		}
	}
	
	function_Result = SAFE_TRUE;
	goto Normal_Function_Exit;
	
Error_Handler:
	function_Result = SAFE_FALSE;
			
Normal_Function_Exit:
	return(function_Result);

}/* Safe_Boolean Process_Sound_Updating_For_SID(Voice_Hdl	voice) */
/*=**************************************************************************=*/


/*=**************************************************************************=*/
Safe_Boolean Process_Sound_Updating_For_Sampler(Voice_Hdl	voice)
{
	Safe_Boolean		function_Result;
	CARDINAL_32	raw_Signal;
/* END VAR */


	function_Result = SAFE_FALSE;
	
	if ((**voice).initialized != SAFE_TRUE) {
		goto Error_Handler;
	}

	if ((**voice).audio_Source != NATIONAL_INSTRUMENTS_DA) {
		goto Error_Handler;
	}
	
#if (ALLOW_CARD_UPDATE_ITERATIVELY == 1)

	current_Time = TickCount();
	/*  */
	/* Limits monitoring to 60 times per second maximum when set to "1" */
	/*  */
	if ((**voice).tickCount_Last_Updated + k_TICK_DELAY_BETWEEN_ANALOG_CARD_TAPPING <= current_Time) {
	
		if (Extract_Recent_Amplitude(voice
				, &raw_Signal) != SAFE_TRUE) {
				/*  */
				/* We could quit at this point, but one not too rare reason there was a failure */
				/* is because the buffer just finally finished and there is no level to report */
				/*  */
			/* ===
				(**voice).maximum_Signal_Since_Last = 0;
				(**voice).current_Raw_Signal = 0;
				(**voice).current_Interpolated_Signal = 0;
			==== */
				
				/*  */
				/* DO NOT update "tickCount_Last_Updated", so that we get a */
				/* retry as soon as possible */
				/*  */
		}
		else {
		
			if (raw_Signal > (**voice).Max_Signal_Range) {
				raw_Signal =(**voice).Max_Signal_Range;
			}
			
			(**voice).current_Raw_Signal = raw_Signal;
			(**voice).current_Interpolated_Signal = raw_Signal;

			/*  */
			/* Experiment only.... only accumulate maximum_Signal_Since_Last */
			/*  */
			if (raw_Signal > (**voice).maximum_Signal_Since_Last) {
				(**voice).maximum_Signal_Since_Last = raw_Signal;
			}
							
			(**voice).tickCount_Last_Updated = current_Time;
		}
	}
		
#endif /*  (ALLOW_CARD_UPDATE_ITERATIVELY == 1) */
	
	function_Result = SAFE_TRUE;
	goto Normal_Function_Exit;
	
Error_Handler:
	function_Result = SAFE_FALSE;
			
Normal_Function_Exit:
	return(function_Result);

}/* Safe_Boolean Process_Sound_Updating_For_Sampler(Voice_Hdl	voice) */
/*=**************************************************************************=*/


/*=**************************************************************************=*/
Safe_Boolean Process_Sound_Updating_For_Audio_File(Voice_Hdl	voice)
{
	Safe_Boolean			function_Result;
	CARDINAL_32		current_Time;
	CARDINAL_32	raw_Signal;
/* END VAR */


	function_Result = SAFE_FALSE;
	
	if ((**voice).initialized != SAFE_TRUE) {
		goto Error_Handler;
	}

	if ((**voice).audio_Source != LOCAL_AUDIO_FILE_PLAYING) {
		goto Error_Handler;
	}
	
	current_Time = TickCount();
	/*  */
	/* Limits monitoring to 60 times per second maximum when set to "1" */
	/*  */
	if ((**voice).tickCount_Last_Updated + k_TICK_DELAY_BETWEEN_AUDIO_FILE_STATUS <= current_Time) {
	
		if (Extract_Recent_Amplitude(voice, &raw_Signal) != SAFE_TRUE) {
				/*  */
				/* We could quit at this point, but one not too rare reason there was a failure */
				/* is because the buffer just finally finished and there is no level to report */
				/*  */
			/* ===
				(**voice).maximum_Signal_Since_Last = 0;
				(**voice).current_Raw_Signal = 0;
				(**voice).current_Interpolated_Signal = 0;
			==== */
				
				/*  */
				/* DO NOT update "tickCount_Last_Updated", so that we get a */
				/* retry as soon as possible */
				/*  */
		}
		else {
		
			if (raw_Signal > (**voice).Max_Signal_Range) {
				raw_Signal =(**voice).Max_Signal_Range;
			}
			
			(**voice).current_Raw_Signal = raw_Signal;
			(**voice).current_Interpolated_Signal = raw_Signal;

			/*  */
			/* Experiment only.... only accumulate maximum_Signal_Since_Last */
			/*  */
			if (raw_Signal > (**voice).maximum_Signal_Since_Last) {
				(**voice).maximum_Signal_Since_Last = raw_Signal;
			}
							
			(**voice).tickCount_Last_Updated = current_Time;
		}
	}
	
	function_Result = SAFE_TRUE;
	goto Normal_Function_Exit;
	
Error_Handler:
	function_Result = SAFE_FALSE;
			
Normal_Function_Exit:
	return(function_Result);

}/* Safe_Boolean Process_Sound_Updating_For_Audio_File(Voice_Hdl	voice) */
/*=**************************************************************************=*/


/*=**************************************************************************=*/
/* END OF FILE: Dreaming_Audio.c */
