/* START OF FILE: Dreaming_Audio_Record.c */


/*=**************************************************************************=*/
/* [---BEGIN---] FILE HEADER COMMENTS */
/**
 ** SCREEN VIEW = [Adobe Syntax Black-10, Tabs = 4]
 ** PRINTER VIEW = [Adobe Syntax Black-10, Tabs = 4]
 ** =======================================================================
 ** Written by James Merkle
 ** Dreaming_Audio_Record.c -Routines used for recording audio to AIFF files from tapping SIDs
 ** (C) Copyright 1995 James Merkle
 ** =======================================================================
 **
 **
 ** ============
 ** Revision History
 ** ============
 **
 **  1995.05.08		(James Merkle)
 **						Initial coding started from scratch
 **
 **
 **
 ** =================
 ** OTHER INFORMATION
 ** =================
 ** 
 ** 
 **
 **
 **/
/* [----END----] FILE HEADER COMMENTS */
/*=**************************************************************************=*/


/*=**************************************************************************=*/
/* [---BEGIN---] #include AREA */

#include "Error_Utilities.h"
#include "Memory_Utilities.h"
#include "File_Utilities.h"

#include "Dreaming_Stuff.h"
#include "Shell_Globals.h"
#include "Sound_Record.h"
#include "Object_Inventory.h"

#include "Dreaming_Audio_Record.h"

/* [----END----] #include AREA */
/*=**************************************************************************=*/


/*=**************************************************************************=*/
/* [---BEGIN---] #define AREA */


/* [----END----] #define AREA */
/*=**************************************************************************=*/


/*=**************************************************************************=*/
/* [---BEGIN---] TYPE AREA */


/* [----END----] TYPE AREA */
/*=**************************************************************************=*/


/*=**************************************************************************=*/
/* [---BEGIN---] PROTOTYPE AREA */


/* [----END----] PROTOTYPE AREA */
/*=**************************************************************************=*/


/*=**************************************************************************=*/
/* [---BEGIN---] GLOBAL DEFINITION AREA */


/* [----END----] GLOBAL DEFINITION AREA */
/*=**************************************************************************=*/


/*=**************************************************************************=*/
/* [---BEGIN---] LOCAL STATIC DEFINITION AREA */

static CARDINAL_8	*unit_Name = "\pDreaming_Audio_Record";


/* [----END----] LOCAL STATIC DEFINITION AREA */
/*=**************************************************************************=*/


/*=**************************************************************************=*/
Safe_Boolean Initialize_Dreaming_Audio_Record(void)
{
	CARDINAL_8			*static_Unit_Name = "\pDreaming_Audio_Record";
/* END VAR */

/*  */
/* Global persistant structures are created here */
/*  */
	
	unit_Name = static_Unit_Name;
	
	return(SAFE_TRUE);
}
/*=**************************************************************************=*/


/*=**************************************************************************=*/
Safe_Boolean Destroy_Dreaming_Audio_Record(void)
{
	Safe_Boolean		function_Result;
/* END VAR */

/*  */
/* Global persistant structures are deleted here */
/*  */

	function_Result = SAFE_TRUE;
	return(function_Result);
}
/*=**************************************************************************=*/


/*=**************************************************************************=*/
Safe_Boolean Start_Recording_To_AIFF_File(Record_Sound_To_File_Hdl *audio_Tapping_Hdl
												, Memory_Allocation *audio_Tapping_Initialized
												, Safe_Boolean create_File_Without_User_Inquiry
												, CARDINAL_32 number_Of_Channels
												, CARDINAL_32 sample_Rate
												, CARDINAL_32 sample_Size
												, CARDINAL_32 compression_Type)
/*  */
/* This routine deletes the allocated memory of a newly stopped */
/* sound tap to file. */
/* It also adds the file to the list of available AIFF files available to play. */
/*  */
{
	CARDINAL_32			expected_File_Minimum_Size;
	CARDINAL_32 	maximum_Possible_File_Size;
	CARDINAL_32	preferred_Size;
	FSSpec			desired_File_Spec;
	CARDINAL_8		*code_Name;
	Safe_Boolean		function_Result;
	Safe_Boolean		creation_Result;
	SignedByte 		original_State;
	Safe_Boolean		user_Cancelled;
/* END VAR */


	code_Name = "\pStart_Recording_To_AIFF_File";
	*audio_Tapping_Initialized = MEMORY_NOT_INITIALIZED;

	/*  */
	/* Hack Hack Hack : Add to AIFF database list soon*/
	/*  */

	/*  */
	/* 100K seems about right as minimum before starting. */
	/*  */
	expected_File_Minimum_Size = 100UL * 1024UL;
	

/* ===
	file_Specification->vRefNum = source_Volume_Reference;
	file_Specification->parID = source_Directory_ID;
	Copy_Pascal_String(sf_Reply.fName, file_Specification->name);
==== */

	/*  */
	/* To create the AIFF we can either ask the user for the name or just */
	/* give it a default untiled name and store in default file location. */
	/*  */
		
	if (create_File_Without_User_Inquiry == SAFE_TRUE) {
		creation_Result = Get_New_File_Specification_Arbitrarily(expected_File_Minimum_Size
															, &"\pSound File 1"[0]
															, &maximum_Possible_File_Size
															, &desired_File_Spec);
	}
	else {
		creation_Result = Get_Next_AIFF_File_Specification( expected_File_Minimum_Size
											, &maximum_Possible_File_Size
											, &desired_File_Spec
											, &user_Cancelled);
	}
											
	if (creation_Result != SAFE_TRUE) {
															
		if (user_Cancelled == SAFE_TRUE) {
			function_Result = SAFE_FALSE;
			goto Normal_Function_Exit;
		}
		else {
			Display_Serious_Error_Message(unit_Name, code_Name, "\pProblem trying to open a new AIFF output file.", "\pNot enough space, unreplacable file or Locked?");
			function_Result = SAFE_FALSE;
			goto Normal_Function_Exit;
		}
	}


	*audio_Tapping_Initialized = MEMORY_NOT_INITIALIZED;
	*audio_Tapping_Hdl = nil;
	preferred_Size = sizeof(Record_Sound_To_File_Rec);
	
	if (Create_Handle(&preferred_Size
						, (CARDINAL_32)(10240L)
						, ALLOW_PURGING
						, DISALLOW_PARTIAL_AMOUNT
						, (Handle *)audio_Tapping_Hdl
						, audio_Tapping_Initialized) != SAFE_TRUE) {
						
		Display_Low_Memory_Error_Message(unit_Name, code_Name, sizeof(Record_Sound_To_File_Rec));
		function_Result = SAFE_FALSE;
		goto Normal_Function_Exit;
	}

	/*  */
	/* Handle is accessed during interrupt level reads over a long duration. */
	/* This locks it and moves it high */
	/*  */
	if (Begin_Use_Of_Handle((Handle)*audio_Tapping_Hdl
								, MOVE_HIGH_REQUIRED
								, &original_State) != SAFE_TRUE) {
		goto Error_Handler;
	}
		
	if (Start_Recording_To_New_AIFF_File(**audio_Tapping_Hdl
												, maximum_Possible_File_Size
												, &desired_File_Spec
												, number_Of_Channels
												, sample_Rate
												, sample_Size
												, compression_Type) != SAFE_TRUE) {
		goto Error_Handler;										
	}


	function_Result = SAFE_TRUE;
	goto Normal_Function_Exit;
	
Error_Handler:
	function_Result = SAFE_FALSE;
	
	if (*audio_Tapping_Initialized == MEMORY_INITIALIZED) {
		/*  */
		/* Kill the handle memory now */
		/*  */
		if (Destroy_Handle((Handle *)audio_Tapping_Hdl, audio_Tapping_Initialized) != SAFE_TRUE) {
			Display_Serious_Error_Message(unit_Name, code_Name, "\pError with Destroy_Handle", "\pSystem Corrupted?");
		}
	}
		
Normal_Function_Exit:

	return(function_Result);
	
} /* Safe_Boolean Start_Recording_To_AIFF_File */
/*=**************************************************************************=*/


/*=**************************************************************************=*/
Safe_Boolean Inform_Program_Of_Recording_Completion(Record_Sound_To_File_Hdl *audio_Tapping_Hdl
																, Memory_Allocation *audio_Tapping_Initialized)
/*  */
/* This routine deletes the allocated memory of a newly stopped */
/* sound tap to file. */
/* It also adds the file to the list of available AIFF files available to play. */
/*  */
{
	Safe_Boolean		function_Result;
	CARDINAL_8		*code_Name;
	Boolean			database_Full;
/* END VAR */

	code_Name = "\pInform_Program_Of_Recording_Completion";

	if (*audio_Tapping_Initialized == MEMORY_INITIALIZED) {
		if ((***audio_Tapping_Hdl).total_Samples_Written_To_Stream > 1000) {
			if (Add_Potential_File_Item_To_Database('AIFF', &((***audio_Tapping_Hdl).desired_File_Spec), &database_Full) != SAFE_TRUE) {
				/*  */
				/* Ignore Add to AIFF database list error*/
				/*  */
			}
		}
	}

	if (Destroy_Handle((Handle *)audio_Tapping_Hdl, audio_Tapping_Initialized) != SAFE_TRUE) {
		Display_Serious_Error_Message(unit_Name, code_Name, "\pError with Destroy_Handle", "\pSystem Corrupted?");
	}

	function_Result = SAFE_TRUE;
	return(function_Result);
}
/*=**************************************************************************=*/


/*=**************************************************************************=*/
Safe_Boolean Stop_Recording_To_Voice_Audio_File(Voice_Hdl	voice)
{
	Record_Sound_To_File_Ptr audio_Tapping_Ptr;
	Safe_Boolean		function_Result;
	CARDINAL_8		*code_Name;
/* END VAR */

	code_Name = "\pStop_Recording_To_Any_Open_Audio_Files";
	
	if ((**voice).audio_Source == PRIMARY_SOUND_INPUT) {
	
		if ((**voice).info.apple_Sound_Input.audio_Tapping_Structure_Initialized == MEMORY_INITIALIZED) {
		
			/*  */
			/* Handle is already locked by design (because interrupts need to */
			/* access its contents */
			/*  */
			audio_Tapping_Ptr = *((**voice).info.apple_Sound_Input.audio_Tapping_Structure_Hdl);
			
			if (audio_Tapping_Ptr->initialized == SAFE_TRUE) {
			
				/*  */
				/* This results in stopping filling of buffers, dumping of all partial buffers */
				/* and writing and closing the open file. */
				/*  */
				audio_Tapping_Ptr->more_Recording_Desired = SAFE_FALSE;
				audio_Tapping_Ptr->stop_Recording_As_Soon_As_Possible = SAFE_TRUE;
	
				if (Periodically_Save_To_New_AIFF_File_If_Needed(audio_Tapping_Ptr) != SAFE_TRUE) {
					/*  */
					/* Ignore errors, the logic later handles it all */
					/*  */
				}
				
				if (audio_Tapping_Ptr->initialized != SAFE_TRUE) {
				
					/*  */
					/* This routine deletes the handle after possible extracting information */
					/*  */
	
					if (Inform_Program_Of_Recording_Completion(&(**voice).info.apple_Sound_Input.audio_Tapping_Structure_Hdl
																									, &(**voice).info.apple_Sound_Input.audio_Tapping_Structure_Initialized) != SAFE_TRUE){
						/*  */
						/* Ignore errors, the logic later handles it all */
						/*  */
					}
				}
				
			} /* if (audio_Tapping_Ptr->initialized == SAFE_TRUE) */
		} /* if ((**voice).info.apple_Sound_Input.audio_Tapping_Structure_Initialized == MEMORY_INITIALIZED) */
	}
	
	function_Result = SAFE_TRUE;
	return(function_Result);
	
} /* Safe_Boolean Stop_Recording_To_Voice_Audio_File */
/*=**************************************************************************=*/


/*=**************************************************************************=*/
Safe_Boolean Stop_Recording_To_Any_Open_Audio_Files(void)
/*  */
/* Should be called at application shutdown time */
/*  */
{
	CARDINAL_32	max_Items;
	CARDINAL_32	total_Items_Seen;
	Record_Sound_To_File_Ptr audio_Tapping_Ptr;
	Safe_Boolean		function_Result;
	SIMPLE_LOOP	voice;
	CARDINAL_8		*code_Name;
/* END VAR */

	code_Name = "\pStop_Recording_To_Any_Open_Audio_Files";


	if (g_Voices.initialized == SAFE_TRUE) {
		if (g_Voices.total_Voices_Active > 0) {
			max_Items = g_Voices.total_Voices_Active;
			total_Items_Seen = 0;
			for (voice=0; ((voice < k_MAXIMUM_VOICE_MONITORS) && (total_Items_Seen < max_Items)) ; voice++) {
				if (g_Voices.voice_Memory_Initialized[voice] == SAFE_TRUE) {
					total_Items_Seen += 1;
				
					if (Stop_Recording_To_Voice_Audio_File(g_Voices.voice[voice]) != SAFE_TRUE) {
						/*  */
						/* Notify if error, keep searching */
						/*  */
						
						if (Notify_Sync_Audio_Problem_Found_If_First_Time() != SAFE_TRUE) {
							/*  */
							/* Ignore error.*/
							/*  */
						}
					}
				}
			}
		}
	}


	function_Result = SAFE_TRUE;
	return(function_Result);
	
} /* Safe_Boolean Stop_Recording_To_Any_Open_Audio_Files */
/*=**************************************************************************=*/


/*=**************************************************************************=*/
Safe_Boolean Start_Recording_From_First_Talking_Item_In_Current_Scene(Safe_Boolean create_File_Without_User_Inquiry)
/*  */
/* Simple shortcut for a hot key that starts recording. */
/* Hot key is option-space to start and */
/* control option space to stop */
/*  */
{
	Record_Sound_To_File_Ptr audio_Tapping_Ptr;
	Safe_Boolean		function_Result;
	CARDINAL_8		*code_Name;
	CARDINAL_32 	scene_To_Use;
	CARDINAL_16 	puppet;
	INTEGER_16 		position;
	Stage_Item_Ptr	item_Ptr;
	Boolean			found;
	SIMPLE_LOOP	loop;
	INTEGER_16		items_Found;
	Scene_Ptr			scene;
	CARDINAL_32	total_Scenes;
	CARDINAL_32	total_To_Search;
	
	CARDINAL_32	number_Of_Channels;
	CARDINAL_32	sample_Rate;
	CARDINAL_32	sample_Size;
	CARDINAL_32	compression_Type;
	
	Voice_Hdl			voice;
/* END VAR */


	code_Name = "\pStop_Recording_To_Any_Open_Audio_Files";

	scene = *(dream.play.scene[dream.current_Scene]);
	found = FALSE;
	if ((*scene).total_Items_In_Scene > 0) {
		items_Found = 0;
		total_To_Search = (*scene).total_Items_In_Scene;
		
		for (loop = 0; (((loop < k_MAX_STAGE_ITEMS) && (found != TRUE) && (items_Found < total_To_Search))); loop++) {
			if ((*scene).item[loop].valid_Pointer == SAFE_TRUE) {
				items_Found += 1;
				
				if ((*scene).item[loop].controllers_Attached == SAFE_TRUE) {
				
					item_Ptr = &(*scene).item[loop];
					if ((**item_Ptr->voice).initialized == SAFE_TRUE) {
						if ((**item_Ptr->voice).audio_Source == PRIMARY_SOUND_INPUT) {					
							found = TRUE;
							position = loop;
							voice = item_Ptr->voice;
						}
					}
				}
			}
		}
	}

	if (found == TRUE) {

		if (Start_Recording_To_AIFF_File(&(**voice).info.apple_Sound_Input.audio_Tapping_Structure_Hdl
												, &(**voice).info.apple_Sound_Input.audio_Tapping_Structure_Initialized
												, create_File_Without_User_Inquiry
												, (**voice).info.apple_Sound_Input.number_Of_Channels
												, (**voice).info.apple_Sound_Input.sample_Rate
												, (**voice).info.apple_Sound_Input.sample_Size
												, (**voice).info.apple_Sound_Input.compression_Type) != SAFE_TRUE) {
			goto Error_Handler;
		}
	}

	function_Result = SAFE_TRUE;
	goto Normal_Function_Exit;
	
Error_Handler:
	function_Result = SAFE_FALSE;

Normal_Function_Exit:

	return(function_Result);
	
} /* Safe_Boolean Start_Recording_From_First_Talking_Item_In_Current_Scene... */
/*=**************************************************************************=*/


/*=**************************************************************************=*/
/* END OF FILE: Dreaming_Audio_Record.c */






