/* START OF FILE: Dreaming_Backdrop_Files.c */


/*=**************************************************************************=*/
/* [---BEGIN---] FILE HEADER COMMENTS */
/**
 ** SCREEN VIEW = [Adobe Syntax Black-10, Tabs = 4]
 ** PRINTER VIEW = [Adobe Syntax Black-10, Tabs = 4]
 ** =======================================================================
 ** Written by James Merkle
 ** Dreaming_Backdrop_Files.c - FIle management routines to load or convert Backdrop PICTs
 ** (C) Copyright 1994 James Merkle
 ** All rights reserved, except those wishing to "copy" moderate sections of code or files may do so, so long as
 ** such code is never used directly either in animation-related tools or multimedia tools that are  
 ** used in live performance or live broadcast or recorded mass market broadcast or film. Desktop usage not restricted.
 ** Conventional desktop game or entertainment not restricted.
 ** =======================================================================
 **
 **
 ** ============
 ** Revision History
 ** ============
 **
 **  1994.02.01		(James Merkle)
 **						Initial coding started
 **
 **
 **
 **
 ** =================
 ** OTHER INFORMATION
 ** =================
 **  
 **
 **/
/* [----END----] FILE HEADER COMMENTS */
/*=**************************************************************************=*/


/*=**************************************************************************=*/
/* [---BEGIN---] #include AREA */

#include "Core_Universal.h" /* Non_Specific_Support_Routines */
#include "Core_Specific.h"
#include "Core_Globals.h"
#include "String_Utilities.h"
#include "Error_Utilities.h"
#include "Shell_Dialogs.h"
#include "Shell_Globals.h"
#include "Volume_Utilities.h"
#include "SysHooks.h"
#include "Cursor_Utilities.h"
#include "Dreaming_Scene_Files.h"
#include "Interface_Utilities.h"
#include "Window_Environment.h"
#include "File_Resource_Utilities.h"


#include "Dreaming_Stuff.h"
#include "Dreaming_Color_Stuff.h"
#include "Dreaming_File_Loading.h"

#include "Dreaming_Backdrop_Files.h"

/* [----END----] #include AREA */
/*=**************************************************************************=*/


/*=**************************************************************************=*/
/* [---BEGIN---] #define AREA */

#define	k_EXTRA_PAD_AMOUNT								8


#define	k_BACKDROP_HEADER_TEMPLATE_NAME			"\pBak1"
#define	k_BACKDROP_HEADER_TEMPLATE_ID					3005

#define k_SEARCH_BUFFER_SIZE		(16384L)
#define k_MAXIMUM_FILE_MATCHES		10

#define	k_BACKDROP_PALLETE_RESOURCE_BASE		128
#define	k_BACKDROP_CLUT_RESOURCE_BASE			128

#define	k_BASE_REGION_RESOURCE_ID		k_BASE_BACKDROP_RESOURCE_ID

/*  */
/* For this version, total items is the same as the RAM-based limit */
/*  */
#define 	k_MAX_FILE_STAGE_ITEMS		k_MAX_STAGE_ITEMS

/*  */
/* There are no sages in loading the backdrops in this version, but the */
/* code is ready */
/*  */
#define	k_LAST_BACKGROUND_STAGE		1


/* [----END----] #define AREA */
/*=**************************************************************************=*/


/*=**************************************************************************=*/
/* [---BEGIN---] TYPE AREA */


/* [----END----] TYPE AREA */
/*=**************************************************************************=*/


/*=**************************************************************************=*/
/* [---BEGIN---] PROTOTYPE AREA */
															
															
Safe_Boolean	Initialize_Empty_Backdrop(CARDINAL_32 *located_Position);

/* [----END----] PROTOTYPE AREA */
/*=**************************************************************************=*/


/*=**************************************************************************=*/
/* [---BEGIN---] GLOBAL DEFINITION AREA */


/* [----END----] GLOBAL DEFINITION AREA */
/*=**************************************************************************=*/


/*=**************************************************************************=*/
/* [---BEGIN---] LOCAL STATIC DEFINITION AREA */

static CARDINAL_8	*unit_Name;

/* [----END----] LOCAL STATIC DEFINITION AREA */
/*=**************************************************************************=*/


/*=**************************************************************************=*/
Safe_Boolean Initialize_Dreaming_Backdrop_Files(void)
{
	CARDINAL_8			*static_Unit_Name = "\pDreaming_Backdrop_Files";
/* END VAR */

/*  */
/* Global persistant structures are created here */
/*  */

	unit_Name = static_Unit_Name;
	return(SAFE_TRUE);
}
/*=**************************************************************************=*/


/*=**************************************************************************=*/
Safe_Boolean Destroy_Dreaming_Backdrop_Files(void)
{
	Safe_Boolean		function_Result;
/* END VAR */

/*  */
/* Global persistant structures are deleted here */
/*  */

	function_Result = SAFE_TRUE;
	return(function_Result);
	
}
/*=**************************************************************************=*/


/*=**************************************************************************=*/
Safe_Boolean	Export_Backdrop_Sequence(Export_Backdrop_Rec	*params_For_Export_Backdrop)
{
	INTEGER_16		scrap_Volume_Reference_Number;
	INTEGER_32		destination_Directory_ID;
	INTEGER_32		directory_Reference;
	Rect				backdrop_Size_Rect;
	INTEGER_16		stored_Bit_Depth;
	Backdrop_Ptr		current_Backdrop;
	Safe_Boolean		function_Result;
	CARDINAL_32	temp_Size;
	CARDINAL_32	file_Minimum_Size;
	Boolean			problem_Getting_Picture;
	PicHandle			picture_Handle;
	INTEGER_32		volume_Space;
	Str255				volume_Name;
	Str255				work_String_2;
	Str255				note_String;
	OSErr				os_Error;
	INTEGER_32		transfer_Count;
	INTEGER_32		transfer_Count_Copy;
	CARDINAL_8		*code_Name;
/* END VAR */


	code_Name = "\pSave_Backdrop_Sequence";
	function_Result = SAFE_FALSE;
	problem_Getting_Picture = FALSE;
	picture_Handle = nil;
		
	/*  */
	/* current_Backdrop uses a handle assumed to always be locked */
	/*  */
	current_Backdrop = *(dream.backdrops.backdrop[params_For_Export_Backdrop->desired_Backdrop]);
	
	if ((current_Backdrop->initialized != SAFE_TRUE) 
		|| ((current_Backdrop->special_Effect_Backdrop != 0) && (current_Backdrop->backdrop_Port.initialized != SAFE_TRUE))) {

		Get_NS_String(k_DREAM_STRINGS, k_ERROR_BACKDROP_UNSAVABLE, work_String_2);
		Perform_OK_Dialog(work_String_2);
		goto Error_Handler;
	}

	if ((dream.backdrops.backdrop_Memory_In_Use[params_For_Export_Backdrop->desired_Backdrop]) != SAFE_TRUE) {
		function_Result = SAFE_FALSE;
		goto Normal_Function_Exit;
	}
	
	note_String[0] = 0;
	volume_Name[0] = 0;
	
	os_Error = GetVInfo((Get_Drive_Number(params_For_Export_Backdrop->desired_File.vRefNum)), volume_Name, &scrap_Volume_Reference_Number, (INTEGER_32 *)&volume_Space);
	if (os_Error != noErr) {
		Concat_Many(note_String, 3, "\pGetVInfo for volume ", volume_Name, "\p ID = ");
		Concat_Long_To_String(note_String, os_Error, NO_LIMIT);
		goto Error_Handler_With_Message;
	} /* if (os_Error != noErr) */
		

	switch (current_Backdrop->backdrop_Port.color_Type) {
		case k_COLOR_QUICKDRAW:
		case k_32_BIT_COLOR_QUICKDRAW:
		
			if (current_Backdrop->backdrop_Port.image_Handle != nil) {
				backdrop_Size_Rect = (**(current_Backdrop->backdrop_Port.offscreen_Port.color_Port.portPixMap)).bounds;
				stored_Bit_Depth = (**(current_Backdrop->backdrop_Port.offscreen_Port.color_Port.portPixMap)).pixelSize;
			}
			else {
				/*  */
				/* Cannot get here. paranoia code */
				/*  */
				SetRect(&(backdrop_Size_Rect), 0,0,1,1);
				stored_Bit_Depth = 8;
			}
		break;
		
		case k_ORIGINAL_QUICKDRAW:
		default:
			backdrop_Size_Rect = current_Backdrop->backdrop_Port.offscreen_Port.old_Style_Port.portBits.bounds;
			stored_Bit_Depth = 1;
	} /* switch (current_Backdrop->backdrop_Port.color_Type) */
	
					
	if (backdrop_Size_Rect.left < current_Backdrop->backdrop_Background_Rect.left) {
		backdrop_Size_Rect.left = current_Backdrop->backdrop_Background_Rect.left;
	}
					
	if (backdrop_Size_Rect.top < current_Backdrop->backdrop_Background_Rect.top) {
		backdrop_Size_Rect.top = current_Backdrop->backdrop_Background_Rect.top;
	}
					
	if (backdrop_Size_Rect.right > current_Backdrop->backdrop_Background_Rect.right) {
		backdrop_Size_Rect.right = current_Backdrop->backdrop_Background_Rect.right;
	}
					
	if (backdrop_Size_Rect.bottom > current_Backdrop->backdrop_Background_Rect.bottom) {
		backdrop_Size_Rect.bottom = current_Backdrop->backdrop_Background_Rect.bottom;
	}
	
	if (EmptyRect(&(backdrop_Size_Rect)) == TRUE) {
		Display_Serious_Error_Message(unit_Name, code_Name, "\pNo Valid Rect", "\pInversed points?");
		goto Error_Handler;
	}

	file_Minimum_Size = 0;
	
	if (current_Backdrop->special_Effect_Backdrop != 0) {
		/*  */
		/* Nothing to do */
		/*  */
		Perform_OK_Dialog("\pSorry. The backdrop cannot be exported because it is algorithmic");
		function_Result = SAFE_TRUE;
		goto Normal_Function_Exit;
	}

	if (current_Backdrop->special_Effect_Backdrop == 0) {
		
		if (Copy_Bitmap_Into_Pict_Command(&(current_Backdrop->backdrop_Port)
																	, &backdrop_Size_Rect
																	, stored_Bit_Depth
																	, &picture_Handle
																	, FALSE) != SAFE_TRUE) {
			goto Error_Handler;
		}
				
		if (EmptyRect(&(**picture_Handle).picFrame) == TRUE) {
			problem_Getting_Picture = TRUE;
		}
		else {
			/*  */
			/* Add in the sizes of Picture */
			/*  */
			temp_Size = GetHandleSize((Handle)picture_Handle);
			if (MemError() != noErr) {
				goto Error_Handler;
			}
			else {
				file_Minimum_Size += temp_Size;
			}
		}
	}
	
	if (((CARDINAL_32)file_Minimum_Size + (CARDINAL_32)MINIMUM_DISK_SPACE_TO_PRESERVE) > volume_Space) {

			Concat_Many(note_String, 3, "\p", volume_Name, "\p is too full for the file. (");
			Make_String_Long_String(work_String_2, (CARDINAL_8 *)note_String, volume_Space, NO_LIMIT,"\p)bytes free, (" );
			Make_String_Long_String(note_String
										, (CARDINAL_8 *)work_String_2
										, (file_Minimum_Size + MINIMUM_DISK_SPACE_TO_PRESERVE)
										, NO_LIMIT
										,"\p)bytes needed.  Find additional space for file." );

		goto Error_Handler_With_Message;
	}


	if (problem_Getting_Picture != TRUE) {
		if (Save_Pict_Resource_As_New_File(&params_For_Export_Backdrop->desired_File, picture_Handle) != SAFE_TRUE) {
			goto Error_Handler;
		}
	}
	else {
		goto Error_Handler_No_Memory;
	}
	
	function_Result = SAFE_TRUE;
	goto Normal_Function_Exit;

Error_Handler_No_Memory:
	function_Result = SAFE_FALSE;
	Display_Serious_Error_Message(unit_Name, code_Name, "\pInternal problem saving Backdrop", "\pNo memory");
	goto Normal_Function_Exit;

Error_Handler_With_Message:
	function_Result = SAFE_FALSE;
	Display_Serious_Error_Message(unit_Name, code_Name, note_String, "\p");
	goto Normal_Function_Exit;

Error_Handler:
	Display_Serious_Error_Message(unit_Name, code_Name, "\pInternal problem saving backdrop", "\p");
	function_Result = SAFE_FALSE;
		
Normal_Function_Exit:
				
	/*  */
	/* Delete picture if any used */
	/*  */
	if (current_Backdrop->special_Effect_Backdrop != 0) {
		if (picture_Handle != nil) {
			KillPicture(picture_Handle);
		}
	}
	
	return(function_Result);
	
} /* Safe_Boolean	Save_Backdrop_Sequence... */
/*=**************************************************************************=*/


/*=**************************************************************************=*/
Safe_Boolean	Save_Backdrop_Sequence(Save_Backdrop_Rec	*params_For_Save_Backdrop)
{
	INTEGER_16		scrap_Volume_Reference_Number;
	INTEGER_32		destination_Directory_ID;
	INTEGER_32		directory_Reference;
	INTEGER_16		file_Reference;
	WDPBRec			working_Directory_PB;
	INTEGER_16		working_Directory;
	Safe_Boolean		working_Directory_In_Use;
	Boolean			problem_Getting_Picture;
	Rect				rect_To_Save;
	CTabHandle		temp_Clut_Handle;
	PicHandle			picture_Handle;
	OSErr				os_Error;
	INTEGER_32		volume_Space;
	CARDINAL_32	file_Creator;
	CARDINAL_32	file_Type;
	CARDINAL_32	resource_Type;
	CARDINAL_32	file_Minimum_Size;
	CARDINAL_32	temp_Size;
	INTEGER_16		resource_ID;
	Backdrop_Ptr		current_Backdrop;
	Safe_Boolean		function_Result;
	CARDINAL_32	preferred_Size;

	Backdrop_File_Header_Ptr				header;
	Backdrop_File_Header_Hdl				resource_To_Create_Hdl;
	Memory_Allocation							resource_To_Create_Hdl_Memory_Initialized;
	
	Handle				template_Hdl;
	SIMPLE_LOOP	loop;
	INTEGER_16		index;
	CARDINAL_32	current_File_Offset;
	CARDINAL_8		*code_Name;
	Str255					resource_Comment;
	Str255					note_String;
	Str255					volume_Name;
	Str255					work_String_2;
	
	INTEGER_32		transfer_Count;
	INTEGER_32		transfer_Count_Copy;
/* END VAR */


	code_Name = "\pSave_Backdrop_Sequence";
	function_Result = SAFE_FALSE;
	working_Directory_In_Use = SAFE_FALSE;
	resource_To_Create_Hdl_Memory_Initialized = MEMORY_NOT_INITIALIZED;
	problem_Getting_Picture = FALSE;
	picture_Handle = nil;
		
	/*  */
	/* current_Backdrop uses a handle assumed to always be locked */
	/*  */
	current_Backdrop = *(dream.backdrops.backdrop[params_For_Save_Backdrop->desired_Backdrop]);
	
	if ((current_Backdrop->initialized != SAFE_TRUE) 
		|| ((current_Backdrop->special_Effect_Backdrop != 0) && (current_Backdrop->backdrop_Port.initialized != SAFE_TRUE))) {

		Get_NS_String(k_DREAM_STRINGS, k_ERROR_BACKDROP_UNSAVABLE, work_String_2);
		Perform_OK_Dialog(work_String_2);
		goto Error_Handler;
	}

	if ((dream.backdrops.backdrop_Memory_In_Use[params_For_Save_Backdrop->desired_Backdrop]) != SAFE_TRUE) {
		function_Result = SAFE_FALSE;
		goto Normal_Function_Exit;
	}
	
	
	file_Creator= FILE_CREATOR;
	file_Type = k_BACKDROP_FILE_TYPE;
	note_String[0] = 0;
	
	os_Error = GetVInfo((Get_Drive_Number(params_For_Save_Backdrop->desired_File.vRefNum)), volume_Name, &scrap_Volume_Reference_Number, (INTEGER_32 *)&volume_Space);
	if (os_Error != noErr) {
		Concat_Many(note_String, 3, "\pGetVInfo for volume ", volume_Name, "\p ID = ");
		Concat_Long_To_String(note_String, os_Error, NO_LIMIT);
		goto Error_Handler_With_Message;
	} /* if (os_Error != noErr) */
		
	preferred_Size = sizeof(Backdrop_File_Header_Rec);
	resource_To_Create_Hdl = nil;
	resource_To_Create_Hdl_Memory_Initialized = MEMORY_NOT_INITIALIZED;

	if (Create_Handle(&preferred_Size
						, (CARDINAL_32)(10240L)
						, ALLOW_PURGING
						, DISALLOW_PARTIAL_AMOUNT
						, (Handle *)&resource_To_Create_Hdl
						, &resource_To_Create_Hdl_Memory_Initialized) != SAFE_TRUE) {
						
		Display_Low_Memory_Error_Message(unit_Name, code_Name, sizeof(Backdrop_File_Header_Rec));
		
		function_Result = SAFE_FALSE;
		goto Error_Handler;
	}

	/*  */
	/* Start a section, handle not locked: not calling memory move indirectly */
	/*  */
	header = (Backdrop_File_Header_Ptr)*resource_To_Create_Hdl;
	header->valid = SAFE_TRUE;
	header->version = k_BACKDROP_VERSION_NOW;
	header->backward_Compatible_Version = k_BACKDROP_VERSION_NOW;
	header->backdrop_Size_Rect = current_Backdrop->backdrop_Background_Rect;


	switch (current_Backdrop->backdrop_Port.color_Type) {
		case k_COLOR_QUICKDRAW:
		case k_32_BIT_COLOR_QUICKDRAW:
		
			if (current_Backdrop->backdrop_Port.image_Handle != nil) {
				header->backdrop_Size_Rect = (**(current_Backdrop->backdrop_Port.offscreen_Port.color_Port.portPixMap)).bounds;
				header->stored_Bit_Depth = (**(current_Backdrop->backdrop_Port.offscreen_Port.color_Port.portPixMap)).pixelSize;
			}
			else {
				/*  */
				/* Cannot get here. paranoia code */
				/*  */
				SetRect(&(header->backdrop_Size_Rect), 0,0,1,1);
				header->stored_Bit_Depth = 8;
			}
		break;
		
		case k_ORIGINAL_QUICKDRAW:
		default:
			header->backdrop_Size_Rect = current_Backdrop->backdrop_Port.offscreen_Port.old_Style_Port.portBits.bounds;
			header->stored_Bit_Depth = 1;
	} /* switch (current_Backdrop->backdrop_Port.color_Type) */
	
					
	if (header->backdrop_Size_Rect.left < current_Backdrop->backdrop_Background_Rect.left) {
		header->backdrop_Size_Rect.left = current_Backdrop->backdrop_Background_Rect.left;
	}
					
	if (header->backdrop_Size_Rect.top < current_Backdrop->backdrop_Background_Rect.top) {
		header->backdrop_Size_Rect.top = current_Backdrop->backdrop_Background_Rect.top;
	}
					
	if (header->backdrop_Size_Rect.right > current_Backdrop->backdrop_Background_Rect.right) {
		header->backdrop_Size_Rect.right = current_Backdrop->backdrop_Background_Rect.right;
	}
					
	if (header->backdrop_Size_Rect.bottom > current_Backdrop->backdrop_Background_Rect.bottom) {
		header->backdrop_Size_Rect.bottom = current_Backdrop->backdrop_Background_Rect.bottom;
	}
	
	if (EmptyRect(&(header->backdrop_Size_Rect)) == TRUE) {
		Display_Serious_Error_Message(unit_Name, code_Name, "\pNo Valid Rect", "\pInversed points?");
		goto Error_Handler;
	}
					
	/*  */
	/* Clean up strings for file tidiness */
	/*  */	
	for (loop = 0; (loop < 64) ; loop++) {
		header->file_Name[loop] = 0;
	}
			
	Copy_Pascal_String(current_Backdrop->backdrop_Name
		, header->file_Name);
			
	rect_To_Save = header->backdrop_Size_Rect;
				
	/*  */
	/* Determine free space, first get header size */
	/*  */
	file_Minimum_Size = GetHandleSize((Handle)resource_To_Create_Hdl);
	if (MemError() != noErr) {
		goto Error_Handler;
	}
	
	/*  */
	/* End the section not calling memory move indirectly */
	/*  */

	/*  */
	/* Memory can indirectly move now and "header" needs */
	/* to be locked momentarily? No, forget it, just */
	/* reset right before Copy_Bitmap_Into_Pict_Command */
	/*  */
	
	MoveHHi((Handle)resource_To_Create_Hdl);
	if (MemError() != noErr) {
		goto Error_Handler;
	}
	
	HLock((Handle)resource_To_Create_Hdl);
	if (MemError() != noErr) {
		goto Error_Handler;
	}
	
	/*  */
	/* Update "header" in case it just moved */
	/*  */
	header = (Backdrop_File_Header_Ptr)*resource_To_Create_Hdl;

	if (current_Backdrop->special_Effect_Backdrop == 0) {
		
		if (Copy_Bitmap_Into_Pict_Command(&(current_Backdrop->backdrop_Port)
																	, &(header->backdrop_Size_Rect)
																	, header->stored_Bit_Depth
																	, &picture_Handle
																	, TRUE) != SAFE_TRUE) {
			goto Error_Handler;
		}
				
		if (EmptyRect(&(**picture_Handle).picFrame) == TRUE) {
			problem_Getting_Picture = TRUE;
		}
		else {
			/*  */
			/* Add in the sizes of Picture */
			/*  */
			temp_Size = GetHandleSize((Handle)picture_Handle);
			if (MemError() != noErr) {
				goto Error_Handler;
			}
			else {
				file_Minimum_Size += temp_Size;
			}
		}
	}
	
	if (((CARDINAL_32)file_Minimum_Size + (CARDINAL_32)MINIMUM_DISK_SPACE_TO_PRESERVE) > volume_Space) {

			Concat_Many(note_String, 3, "\p", volume_Name, "\p is too full for the file. (");
			Make_String_Long_String(work_String_2, (CARDINAL_8 *)note_String, volume_Space, NO_LIMIT,"\p)bytes free, (" );
			Make_String_Long_String(note_String
										, (CARDINAL_8 *)work_String_2
										, (file_Minimum_Size + MINIMUM_DISK_SPACE_TO_PRESERVE)
										, NO_LIMIT
										,"\p)bytes needed.  Find additional space for file." );

		goto Error_Handler_With_Message;
	}

	os_Error = HCreate(params_For_Save_Backdrop->desired_File.vRefNum
									, params_For_Save_Backdrop->desired_File.parID
									, params_For_Save_Backdrop->desired_File.name
									, file_Creator
									, file_Type);
						
	if (os_Error == dupFNErr) {
		os_Error = HDelete(params_For_Save_Backdrop->desired_File.vRefNum
									, params_For_Save_Backdrop->desired_File.parID
									, params_For_Save_Backdrop->desired_File.name);
		if (os_Error != noErr) {
			Concat_Many(note_String, 3,"\pFSDelete for file ", params_For_Save_Backdrop->desired_File.name, "\p ID = ");
			Concat_Long_To_String(note_String, os_Error, NO_LIMIT);
			goto Error_Handler_With_Message;
		}

		os_Error = HCreate(params_For_Save_Backdrop->desired_File.vRefNum
									, params_For_Save_Backdrop->desired_File.parID
									, params_For_Save_Backdrop->desired_File.name
							, file_Creator
							, file_Type);
	} /* if (os_Error == dupFNErr) */

	if (os_Error != noErr) {
		Concat_Many(note_String, 3,"\pCreate for file ", params_For_Save_Backdrop->desired_File.name, "\p ID = ");
		Concat_Long_To_String(note_String, os_Error, NO_LIMIT);
		goto Error_Handler_With_Message;
	} 

	HCreateResFile(params_For_Save_Backdrop->desired_File.vRefNum
									, params_For_Save_Backdrop->desired_File.parID
									, params_For_Save_Backdrop->desired_File.name);
	os_Error = ResError();
	if (os_Error != noErr) {
		Concat_Many(note_String, 3,"\pCreateResFile for file ", params_For_Save_Backdrop->desired_File.name, "\p ID = ");
		Concat_Long_To_String(note_String, os_Error, NO_LIMIT);
		goto Error_Handler_With_Message;
	} 

	/*  */
	/* Crappy deficient OpenRFPerm needs ancient Working Directory */
	/*  */
	working_Directory_PB.ioCompletion = nil;
	working_Directory_PB.ioNamePtr = nil;
	working_Directory_PB.ioWDProcID = FILE_CREATOR;
	working_Directory_PB.ioVRefNum = params_For_Save_Backdrop->desired_File.vRefNum;
	working_Directory_PB.ioWDDirID = params_For_Save_Backdrop->desired_File.parID;
	
	os_Error = PBOpenWD(&working_Directory_PB, FALSE);
	if (os_Error != noErr) {
		goto Error_Handler;
	}
	working_Directory_In_Use = SAFE_TRUE;
	working_Directory = working_Directory_PB.ioVRefNum;

	/*=--------------------------------------------------------------------------=*/
	/* 'OpenRFPerm' used to prevent having to use 'SetVol' */
	/* 'StripAddress' called as per tech notes and THINK reference */
	/*=--------------------------------------------------------------------------=*/
	file_Reference = OpenRFPerm(((StringPtr)(StripAddress(
																	&(params_For_Save_Backdrop->desired_File.name[0]))))
																	, working_Directory
																	, fsRdWrPerm );

	os_Error = ResError();
	if ((file_Reference == -1) || ( os_Error != noErr)) {
		Display_Serious_Error_Message(unit_Name, code_Name, "\pOS Error OpenResFile.", "\pNo resource fork?");
		goto Error_Handler;
	}

	resource_Type = k_BACKDROP_MAIN_RESOURCE_TYPE;
	resource_ID = k_BASE_BACKDROP_RESOURCE_ID;
	Get_NS_String(k_DREAM_STRINGS, k_DONT_EDIT_RESOURCE_COMMENT, resource_Comment);

	if (Add_Resource_Handle((Handle)resource_To_Create_Hdl
											, resource_Type
											, resource_ID
											, resource_Comment) != SAFE_TRUE) {
		goto Error_Handler;
	}

	if (current_Backdrop->special_Effect_Backdrop == 0) {
		if (problem_Getting_Picture != TRUE) {
			resource_Type = 'PICT';
			resource_ID = k_BASE_BACKDROP_RESOURCE_ID;
			Get_NS_String(k_DREAM_STRINGS, k_RESOURCE_IS_JUST_THUMBNAIL_COMMENT, resource_Comment);

			if (Add_Resource_Handle((Handle)picture_Handle
													, resource_Type
													, resource_ID
													, resource_Comment) != SAFE_TRUE) {
				goto Error_Handler;
			}
		}
	}

	/*  */
	/* Now add the template for the header */
	/*  */

	resource_Type = 'TMPL';
	resource_ID = k_BACKDROP_HEADER_TEMPLATE_ID;
	
	template_Hdl = GetResource(resource_Type, resource_ID);
	if (template_Hdl != nil) {
	
		DetachResource(template_Hdl);
		os_Error = ResError();
		if ( os_Error != noErr) {
			Display_Serious_Error_Message(unit_Name, code_Name, "\pOS Error DetachResource.", "\p");
			goto Error_Handler;
		} 
	
		if (Add_Resource_Handle((Handle)template_Hdl
												, resource_Type
												, resource_ID
												, k_BACKDROP_HEADER_TEMPLATE_NAME) != SAFE_TRUE) {
			goto Error_Handler;
		}
		
		DisposeHandle(template_Hdl);
		os_Error = MemError();
		if ( os_Error != noErr) {
			Display_Serious_Error_Message(unit_Name, code_Name, "\pOS Error DisposeHandle.", "\p");
			goto Error_Handler;
		} 
		
	} /* if (template_Hdl != nil) */
	
	CloseResFile(file_Reference);
	os_Error = ResError();
	if ( os_Error != noErr) {
		Display_Serious_Error_Message(unit_Name, code_Name, "\pOS Error CloseResFile.", "\p");
	}

	file_Reference = 0;


	/* = = = = = = = = = = = = = = = = = = = = = = = = = = = = = = = = = = = = = = = = = = = = = = = = = = = = = = = = */
	/* = = = = = = = = = = = = = = = = = = = = = = = = = = = = = = = = = = = = = = = = = = = = = = = = = = = = = = = = */
	/* = = = = = = = = = = = = = = = = = = = = = = = = = = = = = = = = = = = = = = = = = = = = = = = = = = = = = = = = */

	/*  */
	/* Now open data fork and fill it with our structure */
	/* The data fork is used because we might want to read */
	/* portions in during interrupts, as a background task, or */
	/* because we need to load it into memory in a specific */
	/* spot and there may not be enough room for two copies */
	/* to exchange from a loaded monster resource. It is true */
	/* that system 7 allows partial resource loading, but some */
	/* resource compressors, and other things fail when these are used. */
	/*  */									
	os_Error = HOpen(params_For_Save_Backdrop->desired_File.vRefNum
									, params_For_Save_Backdrop->desired_File.parID
									, &(params_For_Save_Backdrop->desired_File.name[0])
									, fsRdWrPerm
									, &file_Reference);
									
	if (os_Error != noErr) {
		Concat_Many(note_String, 3,"\pHOpen for file ", &(params_For_Save_Backdrop->desired_File.name[0]), "\p ID = ");
		Concat_Long_To_String(note_String, os_Error, NO_LIMIT);
		goto Error_Handler_With_Message;
	} 
	
	transfer_Count_Copy = 0;
	switch (current_Backdrop->backdrop_Port.color_Type) {
		case k_COLOR_QUICKDRAW:
		case k_32_BIT_COLOR_QUICKDRAW:
			if (current_Backdrop->backdrop_Port.image_Handle != nil) {
				transfer_Count_Copy = GetHandleSize(current_Backdrop->backdrop_Port.image_Handle);
				os_Error = MemError();
				if ( os_Error != noErr) {
					Display_Serious_Error_Message(unit_Name, code_Name, "\pBad Handle for Port.", "\p");
					goto Error_Handler;
				}
			}
		break;
		
		case k_ORIGINAL_QUICKDRAW:
		default:
			if (current_Backdrop->backdrop_Port.image_Handle != nil) {
				transfer_Count_Copy = GetHandleSize(current_Backdrop->backdrop_Port.image_Handle);
				os_Error = MemError();
				if ( os_Error != noErr) {
					Display_Serious_Error_Message(unit_Name, code_Name, "\pBad Handle for Port.", "\p");
					goto Error_Handler;
				}
			}
	} /* switch (current_Backdrop->backdrop_Port.color_Type) */

	transfer_Count = sizeof(transfer_Count_Copy);
	os_Error = FSWrite(file_Reference, &transfer_Count, (Ptr)&transfer_Count_Copy);
	if (os_Error != noErr) {
		Concat_Many(note_String, 3,"\pFSWrite for file ", &(params_For_Save_Backdrop->desired_File.name[0]), "\p ID = ");
		Concat_Long_To_String(note_String, os_Error, NO_LIMIT);
		goto Error_Handler_With_Message;
	}

	transfer_Count = transfer_Count_Copy;
	os_Error = FSWrite(file_Reference, &transfer_Count, (Ptr)*current_Backdrop->backdrop_Port.image_Handle);
	if (os_Error != noErr) {

		os_Error = FSClose(file_Reference);
		if ( os_Error != noErr) {
			/*  */
			/* Ignore, this was just an abortion tactic anyway */
			/*  */
		}

		Concat_Many(note_String, 3,"\pFSWrite for file ", &(params_For_Save_Backdrop->desired_File.name[0]), "\p ID = ");
		Concat_Long_To_String(note_String, os_Error, NO_LIMIT);
		goto Error_Handler_With_Message;
	}

	os_Error = FSClose(file_Reference);
	if ( os_Error != noErr) {
		Display_Serious_Error_Message(unit_Name, code_Name, "\pOS Error FSClose.", "\p");
	}
	file_Reference = 0;

	/* = = = = = = = = = = = = = = = = = = = = = = = = = = = = = = = = = = = = = = = = = = = = = = = = = = = = = = = = */
	/* = = = = = = = = = = = = = = = = = = = = = = = = = = = = = = = = = = = = = = = = = = = = = = = = = = = = = = = = */
	/* = = = = = = = = = = = = = = = = = = = = = = = = = = = = = = = = = = = = = = = = = = = = = = = = = = = = = = = = */


	function_Result = SAFE_TRUE;
	if (problem_Getting_Picture == TRUE) {
		goto Error_Handler_No_Memory;
	}
	
	goto Normal_Function_Exit;

Error_Handler_No_Memory:
	function_Result = SAFE_FALSE;
	Display_Serious_Error_Message(unit_Name, code_Name, "\pInternal problem saving Backdrop", "\pNo memory");
	goto Normal_Function_Exit;

Error_Handler_With_Message:
	function_Result = SAFE_FALSE;
	Display_Serious_Error_Message(unit_Name, code_Name, note_String, "\p");
	goto Normal_Function_Exit;

Error_Handler:
	Display_Serious_Error_Message(unit_Name, code_Name, "\pInternal problem saving backdrop", "\p");
	function_Result = SAFE_FALSE;
		
Normal_Function_Exit:
	
	if (resource_To_Create_Hdl_Memory_Initialized == MEMORY_INITIALIZED) {
		if (Destroy_Handle((Handle *)&resource_To_Create_Hdl
							, &resource_To_Create_Hdl_Memory_Initialized) != SAFE_TRUE) {
							
			Display_Serious_Error_Message(unit_Name, code_Name, "\pError with Destroy_Handle", "\pSystem Corrupted?");
		}
	}
	
	/*  */
	/* Placed here to ensure not accidentally ever left allocated */
	/*  */
	if (working_Directory_In_Use == SAFE_TRUE) {
		os_Error = CloseWD(working_Directory);
		if (os_Error != noErr) {
			goto Error_Handler;
		}
	}
		
	/*  */
	/* Delete picture if any used */
	/*  */
	if (current_Backdrop->special_Effect_Backdrop != 0) {
		if (picture_Handle != nil) {
			KillPicture(picture_Handle);
		}
	}
	
	return(function_Result);
	
} /* Safe_Boolean	Save_Backdrop_Sequence... */
/*=**************************************************************************=*/
	
	
/*=**************************************************************************=*/
Safe_Boolean Load_Raw_Pict_Fork_From_Backdrop(FSSpec *file_Info
																		, CARDINAL_16	desired_Color_Depth
																		, CTabHandle		offscreen_Color_Table_Handle
																		, CGrafPtr	offscreen_Graph_Port
																		, GDHandle	*offscreen_GDevice_Ptr
																		, Handle		*image_Handle_Ptr)
	/*  */
	/* This draws a raw data stream of frozen pixels a new port of same size.*/
	/* This Allocates a handle and fills it with bitmap image and uses color*/
	/* It is similar to another verswion called "Load_Raw_Pict_Fork_From_Spin_File" */
	/* in another file */
	/*  */
{
	Backdrop_File_Header_Ptr				header;
	SignedByte 										header_Original_State;
	Backdrop_File_Header_Hdl				backdrop_Header_Hdl;
	Memory_Allocation							backdrop_Header_Memory_In_Use;
	INTEGER_16										total_Puppet_Resources;
	INTEGER_32							total_Bytes_Contiguous_Now;
	INTEGER_32							total_Bytes_Now;
	INTEGER_32 							total_Bytes_Contiguous_Later;
	INTEGER_32							total_Bytes_Later;
	Rect						source_Bit_Rect;
	CARDINAL_32	handle_Size;
	INTEGER_16		current_Pict_Sequence;
	GrafPtr				saved_Port;
	GDHandle			saved_Device;
	Safe_Boolean		function_Result;
	OSErr					os_Error;
	Rect						offscreen_Rect;
	Safe_Boolean		resource_Sources_Swapped;
	INTEGER_16		input_File_Reference;
	INTEGER_16		save_Ref;
	INTEGER_32		transfer_Count;
	INTEGER_32		transfer_Count_Copy;
	INTEGER_32		file_Item_Offset;
	Boolean				still_Searching;
	CARDINAL_8		*code_Name;
	Str255					note_String;
	Str255					message_String;
/* END VAR */


	code_Name = "\pLoad_Raw_Pict_Fork_From_Backdrop";
	file_Item_Offset = 0;
	function_Result = SAFE_FALSE;
	backdrop_Header_Memory_In_Use = MEMORY_NOT_INITIALIZED;
	resource_Sources_Swapped = SAFE_FALSE;
	
	/*  */
	/* We need to open up the file's resource fork and get the information */
	/* to tell us the port size. We also need the information for */
	/* the version to see if compatible or not */
	/*  */
	
	save_Ref = CurResFile();
	input_File_Reference = HOpenResFile (file_Info->vRefNum
									, file_Info->parID
									, file_Info->name
									, fsRdPerm );
						
	if ((input_File_Reference == -1) || (ResError() != noErr)) {
		goto Error_Handler;
	}
	
	
	total_Puppet_Resources = Count1Resources(k_BACKDROP_MAIN_RESOURCE_TYPE);
	if (ResError() != noErr) {
		goto Error_Handler;
	}
	
	if (total_Puppet_Resources < 1) {
		Get_NS_String(k_DREAM_STRINGS, k_SORRY_WRONG_FILE_INDEX, message_String);
		Perform_OK_Dialog(message_String);
		goto Error_Handler;
	}

	if (Determine_Absolute_Free_Memory_Hook(&total_Bytes_Contiguous_Now
													,&total_Bytes_Now
													,&total_Bytes_Contiguous_Later
													,&total_Bytes_Later) != TRUE) {
		goto Error_Handler;
	}

	backdrop_Header_Hdl = (Backdrop_File_Header_Hdl)Get1Resource(k_BACKDROP_MAIN_RESOURCE_TYPE
																			, k_BASE_BACKDROP_RESOURCE_ID);
	if ((ResError() != noErr) || (backdrop_Header_Hdl == nil)) {
		goto Error_Handler;
	}
	backdrop_Header_Memory_In_Use = MEMORY_INITIALIZED;
	HNoPurge((Handle)backdrop_Header_Hdl);
	if (MemError() != noErr) {
		/*  */
		/* This cannot fail, but check anyway */
		/*  */
		goto Error_Handler;
	}
		
	if ((**backdrop_Header_Hdl).valid != SAFE_TRUE) {
		Copy_Pascal_String("\pThis file has corrupted resources and cannot be used safely", message_String);
		Perform_OK_Dialog(message_String);
		goto Error_Handler;
	}
	
	if (Begin_Use_Of_Handle((Handle)backdrop_Header_Hdl
									, MOVE_HIGH_NOT_REQUIRED
									, &header_Original_State) != SAFE_TRUE) {
		goto Error_Handler;
	}
	
	header = *backdrop_Header_Hdl;

	if ((header->backward_Compatible_Version & 0xFFFF0000) > (k_BACKDROP_VERSION_NOW & 0xFFFF0000)) {
		Copy_Pascal_String("\pThis file is too new to be used safely", message_String);
		Perform_OK_Dialog(message_String);
		goto Error_Handler;
	}
	
	if ((header->backward_Compatible_Version & 0xFFFF0000) < (k_BACKDROP_BASE_VERSION_NOW & 0xFFFF0000)) {
		Copy_Pascal_String("\pThis file is too old to be used safely", message_String);
		Perform_OK_Dialog(message_String);
		goto Error_Handler;
	}
	
	if (desired_Color_Depth != header->stored_Bit_Depth) {
		Copy_Pascal_String("\pThis file has unanticipated color depth. This program cannot convert it.", message_String);
		Perform_OK_Dialog(message_String);
		goto Error_Handler;
	}
	
	offscreen_Rect = header->backdrop_Size_Rect;

	/*  */
	/* Set up the rectangle for the off-screen graphics environments */
	/*  */
	SetRect(&offscreen_Rect, 0, 0, (offscreen_Rect.right - offscreen_Rect.left), offscreen_Rect.bottom - offscreen_Rect.top);
	
	os_Error= Create_Off_Screen_Port(&offscreen_Rect
															, desired_Color_Depth
															, offscreen_Color_Table_Handle
															, offscreen_Graph_Port
															, offscreen_GDevice_Ptr
															, image_Handle_Ptr);

	if (os_Error != noErr) {
		goto Error_Handler;
	}

	GetPort(&saved_Port);
	saved_Device = GetGDevice();
	
	Set_View_Port((GrafPtr)offscreen_Graph_Port);
	Set_View_Device(*offscreen_GDevice_Ptr);
	
	/*  */
	/* For safety to clear out the memory */
	/*  */
	EraseRect(&offscreen_Rect); 
	
	/*  */
	/* Only needed really for CopyBits calls but why not be certain. */
	/*  */
	ForeColor(blackColor);
	BackColor(whiteColor);
	
	/*  */
	/* Load data into memory here */
	/*  */
	Set_View_Port(saved_Port);
	Set_View_Device(saved_Device);
		
	
	if (backdrop_Header_Memory_In_Use == MEMORY_INITIALIZED) {
		ReleaseResource((Handle)backdrop_Header_Hdl);
		if (ResError() != noErr) {
			/*  */
			/* Ignore error, we are exiting anyway  */
			/*  */
		}
		backdrop_Header_Memory_In_Use = MEMORY_NOT_INITIALIZED;
	}
		
	if (resource_Sources_Swapped == SAFE_TRUE) {
		CloseResFile(input_File_Reference);
		if (ResError() != noErr) {

		}
		
		UseResFile(save_Ref);
		if (ResError() != noErr) {
			/*  */
			/* This should be recoverable from, enhance one day  */
			/*  */
		}
		resource_Sources_Swapped = SAFE_FALSE;
	}
		
	/*  */
	/* Now open data fork and get our large bit map structure. */
	/* The data fork is used because we might want to read */
	/* portions in during interrupts, as a background task, or */
	/* because we need to load it into memory in a specific */
	/* spot and there may not be enough room for two copies */
	/* to exchange from a loaded monster resource. It is true */
	/* that system 7 allows partial resource loading, but some */
	/* resource compressors, and other things fail when these are used. */
	/*  */

	os_Error = HOpen(file_Info->vRefNum
									,file_Info->parID
									,file_Info->name
									,fsRdPerm
									,&input_File_Reference);
	if (os_Error != noErr) {
		Concat_Many(note_String, 3,"\pHOpen for file ", file_Info->name, "\p ID = ");
		Concat_Long_To_String(note_String, os_Error, NO_LIMIT);
		goto Error_Handler_With_Message;
	} 
	
	
	current_Pict_Sequence = 0;
	still_Searching = TRUE;
	do {
		transfer_Count = sizeof(transfer_Count_Copy);
		os_Error = FSRead(input_File_Reference, &transfer_Count, (Ptr)&transfer_Count_Copy);
		if (os_Error != noErr) {
			Concat_Many(note_String, 3,"\pFSRead for file ", file_Info->name, "\p ID = ");
			Concat_Long_To_String(note_String, os_Error, NO_LIMIT);
			goto Error_Handler_With_Message;
		}
	
		/*  */
		/* Skip this puppet and go to the next one */
		/*  */
		
		if (current_Pict_Sequence == file_Item_Offset) {
			still_Searching = FALSE;
		}
		else {
			os_Error = SetFPos(input_File_Reference,fsFromMark,transfer_Count_Copy);
			if (os_Error != noErr) {
				Concat_Many(note_String, 3,"\pSetFPos for file ", file_Info->name, "\p ID = ");
				Concat_Long_To_String(note_String, os_Error, NO_LIMIT);
				goto Error_Handler_With_Message;
			}
			
			current_Pict_Sequence += 1;
		}
		
	} while (still_Searching == TRUE);
	

	transfer_Count = transfer_Count_Copy;
	
	handle_Size = GetHandleSize(*image_Handle_Ptr);
	if (MemError() != noErr) {
		Display_Serious_Error_Message(unit_Name, code_Name, "\pGetHandleSize (1)", "\p");
	}
	else {
	
		if (transfer_Count > handle_Size) {
			transfer_Count = handle_Size;
		}
		
		os_Error = FSRead(input_File_Reference, &transfer_Count, (Ptr)*(*image_Handle_Ptr));
		if (os_Error != noErr) {
			Concat_Many(note_String, 3,"\pFSRead for file ", file_Info->name, "\p ID = ");
			Concat_Long_To_String(note_String, os_Error, NO_LIMIT);
			goto Error_Handler_With_Message;
		}
	
		os_Error = FSClose(input_File_Reference);
		if ( os_Error != noErr) {
			Display_Serious_Error_Message(unit_Name, code_Name, "\pOS Error FSClose.", "\p");
		}
		
	}
	
	function_Result = SAFE_TRUE;
	goto Normal_Function_Exit;
		
		
Fatal_Memory_Handler:

Error_Handler_With_Message:
	function_Result = SAFE_FALSE;
	Display_Serious_Error_Message(unit_Name, code_Name, note_String, "\p");
	goto Normal_Function_Exit;
	
Error_Handler:
	Display_Serious_Error_Message(unit_Name, code_Name, "\pInternal problem getting Backdrop", "\p");
	function_Result = SAFE_FALSE;
		
Normal_Function_Exit:

	if (backdrop_Header_Memory_In_Use == MEMORY_INITIALIZED) {
		ReleaseResource((Handle)backdrop_Header_Hdl);
		if (ResError() != noErr) {
			/*  */
			/* Ignore error, we are exiting anyway  */
			/*  */
		}
		backdrop_Header_Memory_In_Use = MEMORY_NOT_INITIALIZED;
	}
		
	if (resource_Sources_Swapped == SAFE_TRUE) {
		CloseResFile(input_File_Reference);
		if (ResError() != noErr) {

		}
		
		UseResFile(save_Ref);
		if (ResError() != noErr) {
			/*  */
			/* This should be recoverable from, enhance one day  */
			/*  */
		}
		resource_Sources_Swapped = SAFE_FALSE;
	}
	
	return(function_Result);
	
}	/* Safe_Boolean Load_Raw_Pict_Fork_From_Backdrop */
/*=**************************************************************************=*/		

		
/*=**************************************************************************=*/		
Safe_Boolean Load_Some_More_Background(void)
/*  */
/* Called as a slow idle loop event task distributed as part */
/* of background I/O while backgrounds are loading. */
/* Governor should have tuning based on whether any animation */
/* is really occuring before deciding how often t call this routine */
/*  */
{
	Safe_Boolean		function_Result;
	CARDINAL_8		*code_Name;
	Backdrop_Ptr		current_Backdrop;
	Boolean				still_Searching;
	CARDINAL_16	index_Used;
	CARDINAL_16	current_Backdrop_Index;
/* END VAR */


	code_Name = "\pLoad_Some_More_Background";
	function_Result = SAFE_FALSE;

	if (dream.backdrops.backdrops_Still_Loading == 0) {
		function_Result = SAFE_TRUE;
		goto Normal_Function_Exit;
	}


	/*  */
	/* Search for a Backdrop to load some data into, ugly but not too slow */
	/* a tradeoff for now */
	/*  */
	still_Searching = TRUE;
	current_Backdrop_Index = 0;
	for(index_Used = 0; ((index_Used < k_MAX_ACTIVE_BACKDROPS) && (still_Searching == TRUE)); index_Used++) {
		
		if (dream.backdrops.backdrop_Memory_In_Use[index_Used] == SAFE_TRUE) {
		current_Backdrop = *(dream.backdrops.backdrop[index_Used]);
			if (current_Backdrop->initialized == SAFE_TRUE) {
			
				/*  */
				/* NOTE: Also check and compare here for background load needed */
				/* before skipping over them */
				/*  */
			
				if (current_Backdrop->backdrop_Remaining_To_Load == SAFE_TRUE) {
				
					/*  */
					/* Found a potentially good Backdrop to work from */
					/*  */			
					
					if (current_Backdrop->backdrop_Loading_Last_Index >= k_LAST_BACKGROUND_STAGE) {
						/*  */
						/* No more items left to load. skip to next Backdrop */
						/* Correction made here. no complaint issued though */
						/*  */
						current_Backdrop->backdrop_Remaining_To_Load = SAFE_FALSE;
					}
					else {
						still_Searching = FALSE;
						current_Backdrop_Index = index_Used;
					}
				}
				else {
						/*  */
						/* This one is already laoded */
						/*  */
				}
			}
		}
	} /* for(index_Used = 0; ... */
	
	if (still_Searching == TRUE) {
		/*  */
		/* Kill task, nothing left to safely load */
		/*  */
		dream.backdrops.backdrops_Still_Loading = 0;

		function_Result = SAFE_TRUE;
		goto Normal_Function_Exit;
	}
	
	
	/*  */
	/* Load some more backdrop done here. In this version, */
	/* we merely promote the status to closure.
	/*  */
	current_Backdrop->backdrop_Remaining_To_Load = SAFE_FALSE;
	current_Backdrop->backdrop_Loading_Last_Index = k_LAST_BACKGROUND_STAGE;
	

	function_Result = SAFE_TRUE;
	goto Normal_Function_Exit;
	
Error_Handler:
	function_Result = SAFE_FALSE;

Normal_Function_Exit:

	return(function_Result);
	
} /* Safe_Boolean Load_Some_More_Background(void) */
/*=**************************************************************************=*/
	
	
/*=**************************************************************************=*/
Safe_Boolean	Initialize_Empty_Backdrop(CARDINAL_32 *located_Position)
{
	Safe_Boolean		function_Result;
	CARDINAL_32	memory_Requested;
	INTEGER_16		index;
	INTEGER_16		position;
	Backdrop_Ptr		current_Backdrop;
	SignedByte			original_State;
	CARDINAL_8		*code_Name;
	Boolean				found_Location;
/* END VAR */


	code_Name = "\pInitialize_Empty_Backdrop";
	function_Result = SAFE_FALSE;
	*located_Position = 0;
	
	if (g_Dreaming_Module_Globals_Initialized == SAFE_TRUE) {
		if (dream.backdrops.total_Backdrops < k_MAX_ACTIVE_BACKDROPS) {
			found_Location = FALSE;
			for (index = 0; ((index < k_MAX_ACTIVE_BACKDROPS) && (found_Location != TRUE)); index++) {
				if (dream.backdrops.backdrop_Memory_In_Use[index] != MEMORY_INITIALIZED) {
					found_Location = TRUE;
					position = index;					
				}
			}
			
			if (found_Location != TRUE) {
				goto No_More_Backdrops_Error_Handler;
			}
			else {
			
				dream.backdrops.backdrop[position] = nil;
				dream.backdrops.backdrop_Memory_In_Use[position] = MEMORY_NOT_INITIALIZED;
				memory_Requested = sizeof(Backdrop_Rec);
				if (Create_Handle( (CARDINAL_32 *)&memory_Requested
										, (CARDINAL_32)0L
										, ALLOW_PURGING
										, DISALLOW_PARTIAL_AMOUNT
										, (Handle *)&(dream.backdrops.backdrop[position])
										, &(dream.backdrops.backdrop_Memory_In_Use[position])) != SAFE_TRUE ) {
										
					Perform_OK_Dialog("\pMemory error allocating buffer_Memory (not enough?) (4)");
		
					goto Normal_Function_Exit;
				}
				
				if (Begin_Use_Of_Handle((Handle)dream.backdrops.backdrop[position]
										, MOVE_HIGH_REQUIRED
										, &original_State) != SAFE_TRUE) {
					/*  */
					/* Ignore error in implementation */
					/*  */
				}
				
				/*  */
				/* Because locked, we can access using a pointer */
				/*  */		
				current_Backdrop = *(dream.backdrops.backdrop[position]);
				
				
				current_Backdrop->backdrop_Port.initialized = SAFE_FALSE;
				/*  */
				/* Gets properly set later onward */
				/*  */
				SetRect(&(current_Backdrop->backdrop_Background_Rect), 0, 0, 0, 0);
				
				/*  */
				/* Mostly paranoia only */
				/*  */
			/* ==
				current_Backdrop->version = k_BACKDROP_VERSION_NOW;
				current_Backdrop->backward_Compatible_Version = k_BACKDROP_BASE_VERSION_NOW;
				current_Backdrop->backdrop_Location.
			=== */

				current_Backdrop->special_Effect_Backdrop = 0;
				current_Backdrop->backdrop_Name[0] = 0;
				current_Backdrop->backdrop_Remaining_To_Load = SAFE_FALSE;
				current_Backdrop->backdrop_Loading_Last_Index = 0;
				
				current_Backdrop->initialized = SAFE_FALSE;

/* ===					
				dream.backdrops.total_Backdrops += 1;
==== */
				
				*located_Position = position;
				function_Result = SAFE_TRUE;
				
			}
		
		}

	} /* if (g_Dreaming_Module_Globals_Initialized ... */

	if (function_Result != SAFE_TRUE) {
		goto Error_Handler;
	}	
	
	goto Normal_Function_Exit;
	
No_More_Backdrops_Error_Handler:
	Display_Serious_Error_Message(unit_Name, code_Name, "\pNo more Backdrops can be loaded", "\pDelete some");

Error_Handler:
	function_Result = SAFE_FALSE;
			
Normal_Function_Exit:
	return(function_Result);
	
} /* Safe_Boolean	Initialize_Empty_Backdrop... */
/*=**************************************************************************=*/
	
	
/*=**************************************************************************=*/
Safe_Boolean	Destroy_A_Backdrop(CARDINAL_32 backdrop_To_Destroy)
{
	Safe_Boolean		function_Result;
	INTEGER_16		index;
	CARDINAL_8		*code_Name;
	Boolean				found_Location;
/* END VAR */


	code_Name = "\pDestroy_A_Backdrop";
	function_Result = SAFE_FALSE;
	
	if (g_Dreaming_Module_Globals_Initialized != SAFE_TRUE) {
		goto Error_Handler;
	}	
	
	if (dream.backdrops.total_Backdrops < 1) {
		goto Error_Handler;
	}
	
	/*  */
	/* In future add logic to not delete backdrop if depended upon */
	/*  */
	
	 if ((**(dream.backdrops.backdrop[backdrop_To_Destroy])).special_Effect_Backdrop == 0) {
	 
		 if ((**(dream.backdrops.backdrop[backdrop_To_Destroy])).backdrop_Port.initialized == SAFE_TRUE) {
		 	if (Close_Offscreen_Pict_Port(&((**(dream.backdrops.backdrop[backdrop_To_Destroy])).backdrop_Port)) != SAFE_TRUE) {
				/*  */
				/* might as well complain about it? No not in this version */
				/*  */
				/* ===
		 		goto Error_Handler;
		 		==== */
		 	}
		 }
		
		if (Destroy_Handle((Handle *)&(dream.backdrops.backdrop[backdrop_To_Destroy])
											, &(dream.backdrops.backdrop_Memory_In_Use[backdrop_To_Destroy])) != SAFE_TRUE) {
			Display_Serious_Error_Message(unit_Name, code_Name, "\pError with DisposHandle", "\pSystem Corrupted?");
			goto Error_Handler;
		}
		
		dream.backdrops.total_Backdrops = dream.backdrops.total_Backdrops -1;
	}
	
	
	function_Result = SAFE_TRUE;
	goto Normal_Function_Exit;

Error_Handler:
	function_Result = SAFE_FALSE;

Normal_Function_Exit:

	return(function_Result);

} /* Safe_Boolean	Destroy_A_Backdrop( ... */
/*=**************************************************************************=*/
	
	
/*=**************************************************************************=*/
Safe_Boolean	Convert_Backdrop_File_Immediate_Style(Convert_Backdrop_Rec	*params_For_Convert_Backdrop)
{
	Safe_Boolean		function_Result;
	Load_Backdrop_Rec				fake;
/* END VAR */

	function_Result = Load_Convert_Backdrop_File_Immediate_Style(&fake
																							, params_For_Convert_Backdrop
																							, TRUE);
																							
	return(function_Result);
}
/*=**************************************************************************=*/
	
	
/*=**************************************************************************=*/
Safe_Boolean	Load_Backdrop_File_Immediate_Style(Load_Backdrop_Rec	*params_For_Load_Backdrop)
{
	Safe_Boolean		function_Result;
	Convert_Backdrop_Rec				fake;
/* END VAR */

	function_Result = Load_Convert_Backdrop_File_Immediate_Style(params_For_Load_Backdrop
																							, &fake
																							, FALSE);
																							
	return(function_Result);
}
/*=**************************************************************************=*/		

		
/*=**************************************************************************=*/
Safe_Boolean	Load_Convert_Backdrop_File_Immediate_Style(Load_Backdrop_Rec	*params_For_Load_Backdrop
													, Convert_Backdrop_Rec	*params_For_Convert_Backdrop
													, Boolean		importation_Desired)
{
	CARDINAL_32		index_Used;
	Safe_Boolean			user_Abort;
	CARDINAL_32 		located_Position;
	Safe_Boolean			function_Result;
	CARDINAL_8			*code_Name;
/* END VAR */


	code_Name = "\pLoad_Convert_Backdrop_File_Immediate_Style";
	function_Result = SAFE_FALSE;
	
	if (Initialize_Empty_Backdrop(&located_Position) == SAFE_TRUE) {
	
		params_For_Load_Backdrop->desired_Backdrop = located_Position;
		params_For_Convert_Backdrop->desired_Backdrop = located_Position;
		
		if (Load_Or_Import_Backdrop_File(params_For_Load_Backdrop
												, params_For_Convert_Backdrop
												, importation_Desired) != SAFE_TRUE) {
		
			if (Destroy_A_Backdrop(located_Position) != SAFE_TRUE) {
				/*  */
				/* Not a big problem. We are quitting anyway. */
				/*  */
				Display_Serious_Error_Message(unit_Name, code_Name, "\pDestroy_A_Backdrop failed", "\pCorruption?");
			}
			
			goto Error_Handler;
		}
		else {
			/*  */
			/* Add to counter. Finished with it. */
			/* Already incremented */
			/*  */
		/* ===
			dream.backdrops.total_Backdrops += 1;
		==== */

			dream.backdrops.backdrops_Still_Loading += 1;
			
			/*  */
			/* Cycle through loading the Items in the Backdrop. Normally done in stages */
			/* in the event loop */
			/*  */
			while (dream.backdrops.backdrops_Still_Loading > 0) {
			
				if (Load_Some_More_Background() != SAFE_TRUE) {
					/*  */
					/* Kill task, nothing left to safely load */
					/*  */
					dream.backdrops.backdrops_Still_Loading = 0;
				}
				
			}
	
			/*  */
			/* If This backdrop that was loaded just replaced another.... */
			/* Naw, impracticle to swap atomically. */
			/*  */
			
		
		} /* NOT if (Load_Backdrop_File(located_Position, ... */
	}
	else {
		goto No_More_Backdrops_Error_Handler;
	}

	/*  */
	/* Something was loaded, enable all items */
	/*  */
	if (Reset_Menu_Control_Selectability() != SAFE_TRUE) {
		goto Error_Handler;
	}

	function_Result = SAFE_TRUE;
	goto Normal_Function_Exit;
	
No_More_Backdrops_Error_Handler:
	Display_Serious_Error_Message(unit_Name, code_Name, "\pNo more Backdrops can be added", "\pRemove some");

Error_Handler:
	function_Result = SAFE_FALSE;

Normal_Function_Exit:

	return(function_Result);
	
} /* Safe_Boolean	Load_Convert_Backdrop_File_Immediate_Style */
/*=**************************************************************************=*/
	

/*=**************************************************************************=*/				
Safe_Boolean	Specify_Backdrop_To_Load(FSSpec		*file_Specification
															, Safe_Boolean *user_Abort)
{
	INTEGER_16 		number_Of_Types;
	Boolean				user_Clicked_Ok;
	INTEGER_32		source_Directory_ID;
	INTEGER_16		source_Volume_Reference;
	SFTypeList 			type_List;
	SFReply				sf_Reply;
	Safe_Boolean		function_Result;
/* END VAR */


	function_Result = SAFE_FALSE;
	*user_Abort = SAFE_FALSE;

	number_Of_Types = 1;
	type_List[0] = k_BACKDROP_FILE_TYPE;

	user_Clicked_Ok = Perform_Get_File_Dialog(number_Of_Types
										, &(type_List)
										, &sf_Reply);
											
	if ((user_Clicked_Ok != TRUE) || (sf_Reply.good != TRUE)) {
		/*  */
		/* User Abort, no file name */
		/*  */
		*user_Abort = SAFE_TRUE;
		function_Result = SAFE_FALSE;
		goto Normal_Function_Exit;
	}
	
	Get_Last_Volume_Reference_And_Directory_ID(&source_Volume_Reference, &source_Directory_ID);

	file_Specification->vRefNum = source_Volume_Reference;
	file_Specification->parID = source_Directory_ID;
	Copy_Pascal_String(sf_Reply.fName, file_Specification->name);
		
	function_Result = SAFE_TRUE;
	goto Normal_Function_Exit;

Error_Handler:
	function_Result = SAFE_FALSE;
		
Normal_Function_Exit:
	return(function_Result);
	
} /* Safe_Boolean	Specify_Backdrop_To_Load... */
/*=**************************************************************************=*/
	
	
/*=**************************************************************************=*/				
Safe_Boolean	Specify_Pict_To_Import(FSSpec		*file_Specification
															, Safe_Boolean	*file_Is_Pict_File_Type
															, INTEGER_16		*resource_Index
															, Safe_Boolean *user_Abort
															, CARDINAL_32	*selected_Pict_Width
															, CARDINAL_32	*selected_Pict_Height)
{
	INTEGER_16 		number_Of_Types;
	INTEGER_16		input_File_Reference;
	Boolean				user_Clicked_Ok;
	INTEGER_32		source_Directory_ID;
	INTEGER_16		source_Volume_Reference;
	SFTypeList 			type_List;
	INTEGER_16		save_Ref;
	SFReply				sf_Reply;
	Safe_Boolean		resource_Sources_Swapped;
	OSErr					os_Error;
	INTEGER_16		resource_Id;
	ResType				resource_Type;
	PictInfo				picture_Info;
	Safe_Boolean		function_Result;
	INTEGER_32		default_Index;
	INTEGER_32		lowest_Index;
	INTEGER_32		highest_Index;
	CARDINAL_8		*code_Name;
	Rect						selected_Pict_Rect;
	Handle				temp_Resource;
	Boolean				still_Expecting_Pict_Resources;
	INTEGER_16		total_Picts;
	Str255					message_String;
	Str255					temp_String;
/* END VAR */


	function_Result = SAFE_FALSE;
	code_Name = "\pSpecify_Pict_To_Import";
	*user_Abort = SAFE_FALSE;
	input_File_Reference = 0;
	resource_Sources_Swapped = SAFE_FALSE;
	*file_Is_Pict_File_Type = SAFE_FALSE;
	temp_Resource = nil;


	number_Of_Types = -1;
	type_List[0] = 0;

	user_Clicked_Ok = Perform_Get_File_Dialog(number_Of_Types
										, &(type_List)
										, &sf_Reply);
											
	if ((user_Clicked_Ok != TRUE) || (sf_Reply.good != TRUE)) {
		/*  */
		/* User Abort, no file name */
		/*  */
		*user_Abort = SAFE_TRUE;
		function_Result = SAFE_FALSE;
		goto Normal_Function_Exit;
	}
	
	Get_Last_Volume_Reference_And_Directory_ID(&source_Volume_Reference, &source_Directory_ID);

	file_Specification->vRefNum = source_Volume_Reference;
	file_Specification->parID = source_Directory_ID;
	Copy_Pascal_String(sf_Reply.fName, file_Specification->name);
	
	/*  */
	/* Verify That a PICT resource exists to import or not */
	/*  */
	
	save_Ref = CurResFile();
	
	/*  */
	/* Open for read only permission */
	/*  */
	still_Expecting_Pict_Resources = TRUE;
	
	input_File_Reference = HOpenResFile (source_Volume_Reference
						, source_Directory_ID
						, sf_Reply.fName
						, fsRdPerm );
						
	if ((input_File_Reference == -1) || (ResError() != noErr)) {
		still_Expecting_Pict_Resources = FALSE;
	}
	
	total_Picts = 0;
	
	if (still_Expecting_Pict_Resources == TRUE) {												
		UseResFile(input_File_Reference);
		os_Error = ResError();
		if (os_Error != noErr) {
			still_Expecting_Pict_Resources = FALSE;
		}
		else {
		
			resource_Sources_Swapped = SAFE_TRUE;
				
			total_Picts = Count1Resources('PICT');
			if (ResError() != noErr) {
				still_Expecting_Pict_Resources = FALSE;
			}
			else {
				
				if (total_Picts < 1) {
					still_Expecting_Pict_Resources = FALSE;
				}
			}
		}
	}
	
	if (still_Expecting_Pict_Resources != TRUE) {
		/*  */
		/* Aha! now we try to look at the data fork instead. */
		/*  */
		if (resource_Sources_Swapped == SAFE_TRUE) {
			
			UseResFile(save_Ref);
			if (ResError() != noErr) {
				/*  */
				/* This should be recoverable from, enhance one day  */
				/*  */
			}
			resource_Sources_Swapped = SAFE_FALSE;
		}

		if (input_File_Reference != 0) {
			CloseResFile(input_File_Reference);
			if (ResError() != noErr) {

			}
			input_File_Reference = 0;
		}


		if (Guarantee_File_Is_PICT_Type(source_Volume_Reference
						, source_Directory_ID
						, sf_Reply.fName
						, (INTEGER_32 *)selected_Pict_Width
						, (INTEGER_32 *)selected_Pict_Height) == SAFE_TRUE) {
			total_Picts = 1;
			*file_Is_Pict_File_Type = SAFE_TRUE;
		}
		else {
			total_Picts = 0;
		}

	}/* if (still_Expecting_Pict_Resources != TRUE) */	
	
	if (total_Picts < 1) {
		Get_NS_String(k_DREAM_STRINGS, k_ERROR_NO_PICT_RESOURCES_IN_FILE, message_String);
		Perform_OK_Dialog(message_String);
		goto Error_Handler;
	}
	
	if (still_Expecting_Pict_Resources == TRUE) {

		if (total_Picts > 1) {
	
			lowest_Index = 1;
			highest_Index = total_Picts;
			default_Index = highest_Index;
			
			if (Get_Long_Dialog("\pOddly, there is more than one PICT resource available (choose one) :"
										, &default_Index
										, lowest_Index
										, highest_Index) != TRUE) {
				goto Error_Handler;		
			}
			
		}
		else {
			default_Index = 1;
		}
			
		temp_Resource = Get1IndResource ('PICT', default_Index);
		if (ResError() != noErr) {
			Display_Serious_Error_Message(unit_Name
																, code_Name
																, "\pInternal problem with resources"
																, "\pCorrupted?");
			goto Error_Handler;
		}
			
		GetResInfo(temp_Resource
							, &resource_Id
							, &resource_Type
							, &temp_String[0]);
							
		if (ResError() != noErr) {
			goto Error_Handler;
		}
			
		*resource_Index = resource_Id;
		
		os_Error = GetPictInfo((PicHandle)temp_Resource
						, &picture_Info
						, (0)
						, 0
						, popularMethod
						, 0);
						
		if (os_Error == noErr) {
			
			/*  */
			/* Using the first frame (The master puppet frame) */
			/* set limits on the puppet about to be created if needed */
			/* Note though that cropping is arbitrary in this version */
			/* and is not self center adjusting */
			/*  */
			selected_Pict_Rect = (*(PicHandle)temp_Resource)->picFrame;
			*selected_Pict_Width = selected_Pict_Rect.right - selected_Pict_Rect.left;
			*selected_Pict_Height = selected_Pict_Rect.bottom - selected_Pict_Rect.top;
		/* ===
			*selected_Pict_Depth =picture_Info.depth;
		=== */
			
		}
		else {
			goto Error_Handler;
		}
	
		
	} /* if (still_Expecting_Pict_Resources == TRUE) */
	
	
	/*  */
	/* At this point we have our pict size info obtained from either */
	/* a PICT resource or a PICT file */
	/*  */
	
	if (*selected_Pict_Width > g_Window_Limits.specific.display_Item_Width_Max) {
		selected_Pict_Rect.right = selected_Pict_Rect.right 
										- (*selected_Pict_Width - g_Window_Limits.specific.display_Item_Width_Max);
		*selected_Pict_Width = g_Window_Limits.specific.display_Item_Width_Max;
		
		Get_NS_String(k_DREAM_ERROR_STRINGS, k_NOTE_BACKDROP_WIDTH_WILL_CLIP, message_String);
		Perform_OK_Dialog(message_String);
		
	}
	
	if (*selected_Pict_Height > g_Window_Limits.specific.display_Item_Height_Max) {
		selected_Pict_Rect.bottom = selected_Pict_Rect.bottom 
										- (*selected_Pict_Height - g_Window_Limits.specific.display_Item_Height_Max);
		*selected_Pict_Height = g_Window_Limits.specific.display_Item_Height_Max;
		
		Get_NS_String(k_DREAM_ERROR_STRINGS, k_NOTE_BACKDROP_HEIGHT_WILL_CLIP, message_String);
		Perform_OK_Dialog(message_String);
		
	}

	if ((*selected_Pict_Width < g_Window_Limits.specific.display_Item_Width_Min)
		|| (*selected_Pict_Height < g_Window_Limits.specific.display_Item_Height_Min)){
		
		Get_NS_String(k_DREAM_ERROR_STRINGS, k_ERROR_BACKDROP_TOO_SMALL, message_String);
		Perform_OK_Dialog(message_String);
		
		goto Error_Handler;
	}
	
	/*  */
	/* Put away the resource, we extracted the information we wanted */
	/*  */
	if (still_Expecting_Pict_Resources == TRUE) {

		ReleaseResource(temp_Resource);
		if (ResError() != noErr) {
			goto Error_Handler;
		}
	}
	
	if (resource_Sources_Swapped == SAFE_TRUE) {
		
		UseResFile(save_Ref);
		if (ResError() != noErr) {
			/*  */
			/* This should be recoverable from, enhance one day  */
			/*  */
		}
		resource_Sources_Swapped = SAFE_FALSE;
	}

	if (input_File_Reference != 0) {
		CloseResFile(input_File_Reference);
		if (ResError() != noErr) {

		}
		input_File_Reference = 0;
	}
		
	function_Result = SAFE_TRUE;
	goto Normal_Function_Exit;

Error_Handler:
	function_Result = SAFE_FALSE;
		
Normal_Function_Exit:

	if (input_File_Reference != 0) {
		CloseResFile(input_File_Reference);
	}

	if (resource_Sources_Swapped == SAFE_TRUE) {
		CloseResFile(input_File_Reference);
		if (ResError() != noErr) {
			/*  */
			/* Ignore day  */
			/*  */
		}
		
		UseResFile(save_Ref);
		if (ResError() != noErr) {
			/*  */
			/* This should be recoverable from, enhance one day  */
			/*  */
		}
		resource_Sources_Swapped = SAFE_FALSE;
	}

	return(function_Result);
	
} /* Safe_Boolean	Specify_Backdrop_To_Load... */
/*=**************************************************************************=*/
	
	
/*=**************************************************************************=*/				
Safe_Boolean	Specify_Backdrop_To_Save(FSSpec		*file_Specification
															, Str63				*assumed_File_Name
															, Safe_Boolean *user_Abort)
{
	INTEGER_16 		number_Of_Types;
	Boolean				user_Clicked_Ok;
	Str63					original_Name;
	Str63					suffix_String;
	Str255					message_String;
	SFTypeList 			type_List;
	SFReply				sf_Reply;
	Safe_Boolean		function_Result;
/* END VAR */


	function_Result = SAFE_FALSE;
	*user_Abort = SAFE_FALSE;

	Get_NS_String(k_DREAM_STRINGS, k_SAVE_BACKDROP_PROMPT, message_String);
	Get_NS_String(k_DREAM_STRINGS, k_SAVE_BACKDROP_FILE_SUFFIX, suffix_String);

	Assign_String(original_Name, *assumed_File_Name);
	Strip_Off_Popular_Suffixes(&original_Name);
	/*  */
	/* Suffix not appended if just space character */
	/*  */
	if ((suffix_String[0] != 1) || (suffix_String[1] != ASCII_SPACE)) {
		Concat_String(original_Name, suffix_String);
	}
	
	if(original_Name[0] >31) {
		original_Name[0] = 31;
	}


	user_Clicked_Ok = Perform_Put_File_Dialog(message_String
										, original_Name
										, &sf_Reply);
	
	if ((user_Clicked_Ok != TRUE) || (sf_Reply.good != TRUE)) {
		/*  */
		/* User Abort, no file name */
		/*  */
		*user_Abort = SAFE_TRUE;
		function_Result = SAFE_FALSE;
		goto Normal_Function_Exit;
	}

	Get_Last_Volume_Reference_And_Directory_ID(&(file_Specification->vRefNum)
																	, &(file_Specification->parID));
																	
	Copy_Pascal_String(sf_Reply.fName, file_Specification->name);
		
	function_Result = SAFE_TRUE;
	goto Normal_Function_Exit;

Error_Handler:
	function_Result = SAFE_FALSE;
		
Normal_Function_Exit:
	return(function_Result);
	
} /* Safe_Boolean	Specify_Backdrop_To_Save... */
/*=**************************************************************************=*/
	
	
/*=**************************************************************************=*/				
Safe_Boolean	Specify_Backdrop_To_Export(FSSpec		*file_Specification
															, Str63				*assumed_File_Name
															, Safe_Boolean *user_Abort)
{
	INTEGER_16 		number_Of_Types;
	Boolean				user_Clicked_Ok;
	Str63					original_Name;
	Str63					suffix_String;
	Str255					message_String;
	SFReply				sf_Reply;
	Safe_Boolean		function_Result;
/* END VAR */


	function_Result = SAFE_FALSE;
	*user_Abort = SAFE_FALSE;

	Get_NS_String(k_DREAM_STRINGS, k_EXPORT_BACKDROP_PROMPT, message_String);
	Get_NS_String(k_DREAM_STRINGS, k_EXPORT_BACKDROP_BACKDROP_FILE_SUFFIX, suffix_String);

	Assign_String(original_Name, *assumed_File_Name);
	Strip_Off_Popular_Suffixes(&original_Name);
	/*  */
	/* Suffix not appended if just space character */
	/*  */
	if ((suffix_String[0] != 1) || (suffix_String[1] != ASCII_SPACE)) {
		Concat_String(original_Name, suffix_String);
	}
	
	if(original_Name[0] >31) {
		original_Name[0] = 31;
	}


	user_Clicked_Ok = Perform_Put_File_Dialog(message_String
										, original_Name
										, &sf_Reply);
	
	if ((user_Clicked_Ok != TRUE) || (sf_Reply.good != TRUE)) {
		/*  */
		/* User Abort, no file name */
		/*  */
		*user_Abort = SAFE_TRUE;
		function_Result = SAFE_FALSE;
		goto Normal_Function_Exit;
	}

	Get_Last_Volume_Reference_And_Directory_ID(&(file_Specification->vRefNum)
																	, &(file_Specification->parID));
																	
	Copy_Pascal_String(sf_Reply.fName, file_Specification->name);
		
	function_Result = SAFE_TRUE;
	goto Normal_Function_Exit;

Error_Handler:
	function_Result = SAFE_FALSE;
		
Normal_Function_Exit:
	return(function_Result);
	
} /* Safe_Boolean	Specify_Backdrop_To_Export... */
/*=**************************************************************************=*/
	
	
/*=**************************************************************************=*/
Safe_Boolean	Load_Or_Import_Backdrop_File(Load_Backdrop_Rec	*params_For_Load_Backdrop
													, Convert_Backdrop_Rec	*params_For_Convert_Backdrop
													, Boolean		importation_Desired)
{

	Backdrop_File_Header_Ptr				header;
	SignedByte 								header_Original_State;
	Backdrop_File_Header_Hdl				backdrop_Header_Hdl;
	Memory_Allocation					backdrop_Header_Memory_In_Use;
			
	CTabHandle								temp_Clut_Handle;
	Resource_Identifier					*first_Item_Ptr;
	Safe_Boolean 							*end_Sentinel_Ptr;
	Off_Screen_Ptr 							offscreen_Ptr;
	Rect												offscreen_Rect;
	CARDINAL_16							desired_Bit_Depth;
	CARDINAL_32	total_Items_Desired;
	CARDINAL_32	memory_Needed;
	INTEGER_32		transfer_Count;
	INTEGER_32		transfer_Count_Copy;
	WindowPtr			current_Graphics_Window_Ptr;
	Safe_Boolean		function_Result;
	Safe_Boolean		resource_Sources_Swapped;
	Safe_Boolean		file_Is_Pict_File;
	INTEGER_16		save_Ref;
	INTEGER_16		item_To_Fetch;
	
	INTEGER_16		input_File_Reference;
	INTEGER_16		resource_To_Load;
	
	OSErr					os_Error;
	INTEGER_32		total_Bytes_Contiguous_Now;
	INTEGER_32		total_Bytes_Now;
	INTEGER_32 		total_Bytes_Contiguous_Later;
	INTEGER_32		total_Bytes_Later;
	CTabHandle		temp_Clut_Table;
	Backdrop_Ptr		current_Backdrop;
	INTEGER_32		loop;
	INTEGER_16		total_Puppet_Resources;
	INTEGER_16		current_Pict_Sequence;
	Boolean				still_Searching;
	CARDINAL_8		*code_Name;
	Str255					message_String;
	Str255					note_String;
/* END VAR */


	code_Name = "\pLoad_Or_Import_Backdrop_File";
	function_Result = SAFE_FALSE;
	file_Is_Pict_File = SAFE_FALSE;
		
	backdrop_Header_Memory_In_Use = MEMORY_NOT_INITIALIZED;
			
	resource_Sources_Swapped = SAFE_FALSE;
										
	if (importation_Desired != TRUE) {
		params_For_Convert_Backdrop->desired_File = params_For_Load_Backdrop->desired_File;
		params_For_Convert_Backdrop->desired_Backdrop = params_For_Load_Backdrop->desired_Backdrop;
		
		params_For_Convert_Backdrop->desired_Resource_Index = k_BASE_BACKDROP_RESOURCE_ID;
		
		/*  */
		/* Useless paranoia */
		/*  */
		Copy_Pascal_String(&(params_For_Convert_Backdrop->desired_File.name[0]), params_For_Convert_Backdrop->desired_Name);
		params_For_Convert_Backdrop->selected_Pict_Width = 64;
		params_For_Convert_Backdrop->selected_Pict_Height = 64;
	}
	else {
		if (params_For_Convert_Backdrop->file_Is_Pict_File_Type == SAFE_TRUE) {
			/*  */
			/* This file is a PICT file, not a PICT resource to load. */
			/*  */
			file_Is_Pict_File = SAFE_TRUE;
		}
	}
										
	/*  */
	/* Open for read only permission */
	/*  */
	if (file_Is_Pict_File != SAFE_TRUE) {
		save_Ref = CurResFile();
		input_File_Reference = HOpenResFile (params_For_Convert_Backdrop->desired_File.vRefNum
										, params_For_Convert_Backdrop->desired_File.parID
										, params_For_Convert_Backdrop->desired_File.name
										, fsRdPerm );
							
		if ((input_File_Reference == -1) || (ResError() != noErr)) {
			goto Error_Handler;
		}
	}
				
	/*  */
	/* current_Backdrop uses a handle assumed to always be locked */
	/*  */
	current_Backdrop = *(dream.backdrops.backdrop[params_For_Convert_Backdrop->desired_Backdrop]);
	
	if ((current_Backdrop->initialized == SAFE_TRUE) 
		&& (current_Backdrop->backdrop_Port.initialized == SAFE_TRUE)) {
		Get_NS_String(k_DREAM_STRINGS, k_OLD_BACKDROP_NOT_DISPOSED_YET_ERROR, message_String);
		Perform_OK_Dialog(message_String);
		goto Error_Handler;
	}
	
	if (file_Is_Pict_File != SAFE_TRUE) {
	
		UseResFile(input_File_Reference);
		os_Error = ResError();
		if (os_Error != noErr) {
			Display_Serious_Error_Message(unit_Name, code_Name, "\pInternal using resource", "\pCorrupted?");
			goto Error_Handler;
		}
		
		resource_Sources_Swapped = SAFE_TRUE;
			
		/*  */
		/* Load images into memory, allocating a graphics port */
		/* big enough to hold the images into one tall graphics device port */
		/*  */
			
		if (importation_Desired != TRUE) {
			total_Puppet_Resources = Count1Resources(k_BACKDROP_MAIN_RESOURCE_TYPE);
			if (ResError() != noErr) {
				goto Error_Handler;
			}
			
			if (total_Puppet_Resources < 1) {
				Get_NS_String(k_DREAM_STRINGS, k_SORRY_WRONG_FILE_INDEX, message_String);
				Perform_OK_Dialog(message_String);
				goto Error_Handler;
			}
		}
		
	} /* if (file_Is_Pict_File != SAFE_TRUE) */

		
	if (Determine_Absolute_Free_Memory_Hook(&total_Bytes_Contiguous_Now
													,&total_Bytes_Now
													,&total_Bytes_Contiguous_Later
													,&total_Bytes_Later) != TRUE) {
		goto Error_Handler;
	}
		
	if ((dream.backdrops.backdrop_Memory_In_Use[params_For_Convert_Backdrop->desired_Backdrop]) != SAFE_TRUE) {
		function_Result = SAFE_FALSE;
		goto Normal_Function_Exit;
	}
		

	if (importation_Desired != TRUE) {

		backdrop_Header_Hdl = (Backdrop_File_Header_Hdl)Get1Resource(k_BACKDROP_MAIN_RESOURCE_TYPE
																				, k_BASE_BACKDROP_RESOURCE_ID);
		if ((ResError() != noErr) || (backdrop_Header_Hdl == nil)) {
			goto Error_Handler;
		}
		backdrop_Header_Memory_In_Use = MEMORY_INITIALIZED;
		HNoPurge((Handle)backdrop_Header_Hdl);
		if (MemError() != noErr) {
			/*  */
			/* This cannot fail, but check anyway */
			/*  */
			goto Error_Handler;
		}
			
		if ((**backdrop_Header_Hdl).valid != SAFE_TRUE) {
			Copy_Pascal_String("\pThis file has corrupted resources and cannot be used safely", message_String);
			Perform_OK_Dialog(message_String);
			goto Error_Handler;
		}
		
		if (Begin_Use_Of_Handle((Handle)backdrop_Header_Hdl
										, MOVE_HIGH_NOT_REQUIRED
										, &header_Original_State) != SAFE_TRUE) {
			goto Error_Handler;
		}
		
		header = *backdrop_Header_Hdl;
		
	}
	else {
		/*  */
		/* Paranoia only. 'backdrop_Header_Memory_In_Use' is flag*/
		/*  */
		backdrop_Header_Hdl = nil;
		header = nil;
	}
	
	/*  */
	/* Initialize most fields */
	/*  */
	current_Backdrop->initialized = SAFE_TRUE;
	current_Backdrop->backdrop_Port.initialized = SAFE_FALSE;
	SetRect(&(current_Backdrop->backdrop_Background_Rect), 0, 0, 1, 1);
	current_Backdrop->special_Effect_Backdrop = 0;
	current_Backdrop->backdrop_Name[0] = 0;
	current_Backdrop->backdrop_Remaining_To_Load = SAFE_FALSE;
	current_Backdrop->backdrop_Loading_Last_Index = 0;
	
	os_Error = FSMakeFSSpec(params_For_Convert_Backdrop->desired_File.vRefNum
									, params_For_Convert_Backdrop->desired_File.parID
									, params_For_Convert_Backdrop->desired_File.name
									,&(current_Backdrop->backdrop_Location));
	if (os_Error != noErr) {
		goto Error_Handler;
	}
	
	if (importation_Desired == TRUE) {
		
		current_Backdrop->backdrop_Name[0] = 0;
		SetRect (&offscreen_Rect
						, 0
						, 0
						, params_For_Convert_Backdrop->selected_Pict_Width
						, params_For_Convert_Backdrop->selected_Pict_Height );

		/*  */
		/* Properly handled below  */
		/*  */
		desired_Bit_Depth = 8;
		
		Copy_Pascal_String(params_For_Convert_Backdrop->desired_Name
										, current_Backdrop->backdrop_Name);
		Strip_Off_Popular_Suffixes(&(current_Backdrop->backdrop_Name));

	}
	else {
	
		Copy_Pascal_String(header->file_Name, current_Backdrop->backdrop_Name);
	
		if ((header->backward_Compatible_Version & 0xFFFF0000) > (k_BACKDROP_VERSION_NOW & 0xFFFF0000)) {
			Copy_Pascal_String("\pThis file is too new to be used safely", message_String);
			Perform_OK_Dialog(message_String);
			goto Error_Handler;
		}
		
		if ((header->backward_Compatible_Version & 0xFFFF0000) < (k_BACKDROP_BASE_VERSION_NOW & 0xFFFF0000)) {
			Copy_Pascal_String("\pThis file is too old to be used safely", message_String);
			Perform_OK_Dialog(message_String);
			goto Error_Handler;
		}
		
		offscreen_Rect = header->backdrop_Size_Rect;
		desired_Bit_Depth = header->stored_Bit_Depth;
	
	} /* NOT if (importation_Desired == TRUE) */
	
	current_Backdrop->backdrop_Background_Rect = offscreen_Rect;
		
	/*  */
	/* Now create the offscreen picture!  */
	/*  */
	
	 if (current_Backdrop->backdrop_Port.initialized == SAFE_TRUE) {
	 	if (Close_Offscreen_Pict_Port(&(current_Backdrop->backdrop_Port)) != SAFE_TRUE) {
			/*  */
			/* might as well complain about it. */
			/*  */
	 		goto Error_Handler;
	 	}
	 }
	 
	offscreen_Ptr = &(current_Backdrop->backdrop_Port);
	offscreen_Ptr->initialized = SAFE_FALSE;
	offscreen_Ptr->offscreen_GDevice_Hdl = nil;
	offscreen_Ptr->image_Handle = nil;
	offscreen_Ptr->color_Table_Handle = nil;
	offscreen_Ptr->color_Type = k_COLOR_QUICKDRAW;		 


	if ((importation_Desired == TRUE) && ((g_Preferred_Monitor_Depth > 8) && (g_Preferred_Monitor_Depth <= 48))){
			
		Get_NS_String(k_DREAM_STRINGS
						, k_IMPORT_BACKDROP_ALLOW_NON_256_COLOR
						, message_String);
		
		Concat_Long_To_String(message_String, g_Preferred_Monitor_Depth, NO_LIMIT);
			
		if (Perform_Yes_No_Dialog(message_String) != TRUE) {
			desired_Bit_Depth = 8;
			offscreen_Ptr->color_Table_Handle = GetCTable(k_DEFAULT_256_COLOR_SYSTEM_CLUT);
		}
		else {
			desired_Bit_Depth = g_Preferred_Monitor_Depth;
			if (Create_Clut_For_Direct_Mapping(g_Preferred_Monitor_Depth, &(offscreen_Ptr->color_Table_Handle)) != SAFE_TRUE) {
	 			goto Error_Handler;
			}
		}		
	}
	else {
	
		if ((g_Preferred_Monitor_Depth > 8) && (g_Preferred_Monitor_Depth <= 48)) {
			desired_Bit_Depth = g_Preferred_Monitor_Depth;
			if (Create_Clut_For_Direct_Mapping(g_Preferred_Monitor_Depth, &(offscreen_Ptr->color_Table_Handle)) != SAFE_TRUE) {
	 			goto Error_Handler;
			}
		}
		else {
			desired_Bit_Depth = 8;
			offscreen_Ptr->color_Table_Handle = GetCTable(k_DEFAULT_256_COLOR_SYSTEM_CLUT);
		}
	}

	
	if (offscreen_Ptr->color_Table_Handle == nil) {
			Display_Serious_Error_Message(unit_Name
																, code_Name
																, "\pProblem Loading System CLUT (2)"
																, "\pCorrupted?");
		goto Error_Handler;
	}	

	MoveHHi((Handle)offscreen_Ptr->color_Table_Handle);
	if (MemError() != noErr) {
			Display_Serious_Error_Message(unit_Name
																, code_Name
																, "\pProblem locking handle (2.0)"
																, "\pCorrupted!");
		goto Error_Handler;
	}
	
	HLock((Handle)offscreen_Ptr->color_Table_Handle);
	if (MemError() != noErr) {
			Display_Serious_Error_Message(unit_Name
																, code_Name
																, "\pProblem locking handle (2.1)"
																, "\pCorrupted!");
		goto Error_Handler;
	}
	
	if (Determine_Absolute_Free_Memory_Hook(&total_Bytes_Contiguous_Now
													,&total_Bytes_Now
													,&total_Bytes_Contiguous_Later
													,&total_Bytes_Later) != TRUE) {
		goto Error_Handler;
	}
	
	memory_Needed = desired_Bit_Depth;
	memory_Needed = memory_Needed * (CARDINAL_32)(offscreen_Rect.right - offscreen_Rect.left);
	memory_Needed = memory_Needed * (CARDINAL_32)(offscreen_Rect.bottom - offscreen_Rect.top);
	memory_Needed = memory_Needed >> 3;
	
	if (total_Bytes_Contiguous_Later < (k_MINIMUM_RAM_TO_PRESERVE_FOR_BIG_STUFF + memory_Needed)) {
		total_Bytes_Contiguous_Later = MaxMem(&total_Bytes_Now);
		if (total_Bytes_Contiguous_Later < (k_MINIMUM_RAM_TO_PRESERVE_FOR_BIG_STUFF + memory_Needed)) {
			total_Bytes_Contiguous_Later = MaxMem(&total_Bytes_Now);
	
			Get_NS_String(k_DREAM_ERROR_STRINGS, k_ERROR_NOT_ENOUGH_MEMORY_TO_SAFELY_LOAD, note_String);
			Make_String_Long_String(&message_String[0], &note_String[0], memory_Needed, NO_LIMIT, "\p bytes.");
			Perform_OK_Dialog(message_String);
			goto Error_Handler;
		}
	}

	if (((importation_Desired != TRUE)) && (header->backdrop_Special_Effect == SAFE_TRUE)) {

		os_Error= Create_Off_Screen_Port(&offscreen_Rect
																, desired_Bit_Depth
																, offscreen_Ptr->color_Table_Handle
																, (CGrafPtr)&(offscreen_Ptr->offscreen_Port.color_Port)
																, &(offscreen_Ptr->offscreen_GDevice_Hdl)
																, &(offscreen_Ptr->image_Handle));
		
		if (os_Error != noErr) {
			Copy_Pascal_String("\pA graphics port for this puppet file failed to be created (memory?)", message_String);
			Perform_OK_Dialog(message_String);
			goto Error_Handler;
		}
		
		/*  */
		/* Redesign to avoid locked crap one day */
		/*  */
		
		offscreen_Ptr->initialized =  SAFE_TRUE; 
		offscreen_Ptr->color_Type = k_COLOR_QUICKDRAW;
		
		Lock_Video_Memory_For_Lazy_But_Safe_Use(offscreen_Ptr);


		current_Backdrop->special_Effect_Backdrop = header->effect_Type;
		current_Backdrop->backdrop_Remaining_To_Load = SAFE_FALSE;
		current_Backdrop->backdrop_Loading_Last_Index = 0;
		
		/*  */
		/* Fill with solid effect color */
		/*  */
		/* ===
			"current_Backdrop->backdrop_Background_Rect"
		==== */
		
	}
	else{
		current_Backdrop->special_Effect_Backdrop = 0;
		current_Backdrop->backdrop_Remaining_To_Load = SAFE_TRUE;
		current_Backdrop->backdrop_Loading_Last_Index = 0;
		dream.backdrops.backdrops_Still_Loading += 1;
		
		/*  */
		/* The background can now loaded, preferably in stages */
		/* so that animation can proceed */
		/*  */
		
		
		/*  */
		/* At this point the other two stages have been completed */
		/* All that remains is to create the port and then */
		/* load the pict into memory. */
		/*  */	
		offscreen_Ptr = &(current_Backdrop->backdrop_Port);
		
		if (file_Is_Pict_File != SAFE_TRUE) {
	
			if (importation_Desired == TRUE) {
				resource_To_Load = params_For_Convert_Backdrop->desired_Resource_Index;
				
				/*  */
				/* 1994.06.05 JAM This better create all needed items! */
				/*  */
				if (Load_Pict_Into_Color_Bit_Map(resource_To_Load
																, desired_Bit_Depth
																, offscreen_Ptr->color_Table_Handle
																, (CGrafPtr)&(offscreen_Ptr->offscreen_Port.color_Port)
																, &(offscreen_Ptr->offscreen_GDevice_Hdl)
																, &(offscreen_Ptr->image_Handle)) != SAFE_TRUE) {
					goto Error_Handler;
				}
			}
			else {
				
				resource_To_Load = 0;
				if (Load_Raw_Pict_Fork_From_Backdrop(&(params_For_Load_Backdrop->desired_File)
																, desired_Bit_Depth
																, offscreen_Ptr->color_Table_Handle
																, (CGrafPtr)&(offscreen_Ptr->offscreen_Port.color_Port)
																, &(offscreen_Ptr->offscreen_GDevice_Hdl)
																, &(offscreen_Ptr->image_Handle)) != SAFE_TRUE) {
															
					/*  */
					/* Failure. Load default backdrop (not a PICT file?) */
					/*  */
					resource_To_Load = k_BASE_BACKDROP_RESOURCE_ID;
					if (Load_Pict_Into_Color_Bit_Map(resource_To_Load
															, desired_Bit_Depth
															, offscreen_Ptr->color_Table_Handle
															, (CGrafPtr)&(offscreen_Ptr->offscreen_Port.color_Port)
															, &(offscreen_Ptr->offscreen_GDevice_Hdl)
															, &(offscreen_Ptr->image_Handle)) != SAFE_TRUE) {
																
						Display_Serious_Error_Message(unit_Name
																			, code_Name
																			, "\pProblem Loading pict resources (Memory?) ,2"
																			, "\pMemory not available?");
						goto Error_Handler;
					}			
				}
			}
		}
		else {
			resource_To_Load = 0;
			
			if (Load_Pict_File_Into_Color_Bit_Map(&(params_For_Convert_Backdrop->desired_File)
															, desired_Bit_Depth
															, offscreen_Ptr->color_Table_Handle
															, (CGrafPtr)&(offscreen_Ptr->offscreen_Port.color_Port)
															, &(offscreen_Ptr->offscreen_GDevice_Hdl)
															, &(offscreen_Ptr->image_Handle)) != SAFE_TRUE) {
				/*  */
				/* If it fails just load the deafult background from App resources*/
				/*  */
				resource_To_Load = k_BASE_BACKDROP_RESOURCE_ID;
				if (Load_Pict_Into_Color_Bit_Map(resource_To_Load
																, desired_Bit_Depth
																, offscreen_Ptr->color_Table_Handle
																, (CGrafPtr)&(offscreen_Ptr->offscreen_Port.color_Port)
																, &(offscreen_Ptr->offscreen_GDevice_Hdl)
																, &(offscreen_Ptr->image_Handle)) != SAFE_TRUE) {
					goto Error_Handler;
				}
			}			
		}		

		offscreen_Ptr->initialized =  SAFE_TRUE; 
		offscreen_Ptr->color_Type = k_COLOR_QUICKDRAW;
		Lock_Video_Memory_For_Lazy_But_Safe_Use(offscreen_Ptr);

		/*  */
		/* Done loading */
		/*  */
		current_Backdrop->backdrop_Remaining_To_Load = SAFE_FALSE;
		current_Backdrop->backdrop_Loading_Last_Index = 0;
		
		if (importation_Desired == TRUE) {
			current_Backdrop->backdrop_Needs_Saving = SAFE_TRUE;
		}
		else {
			current_Backdrop->backdrop_Needs_Saving = SAFE_FALSE;
		}
				
		if (dream.backdrops.backdrops_Still_Loading > 0) {
			dream.backdrops.backdrops_Still_Loading = dream.backdrops.backdrops_Still_Loading - 1;
		}
		
	} /* NOT if (backdrop_Special_Effect->backdrop_Special_Effect == SAFE_TRUE) */
	
				
	if (backdrop_Header_Memory_In_Use == MEMORY_INITIALIZED) {
		ReleaseResource((Handle)backdrop_Header_Hdl);
		if (ResError() != noErr) {
			/*  */
			/* Ignore error, we are exiting anyway  */
			/*  */
		}
		backdrop_Header_Memory_In_Use = MEMORY_NOT_INITIALIZED;
	}
		
	if (resource_Sources_Swapped == SAFE_TRUE) {
		CloseResFile(input_File_Reference);
		if (ResError() != noErr) {
			goto Error_Handler;
		}
		
		UseResFile(save_Ref);
		if (ResError() != noErr) {
			/*  */
			/* This should be recoverable from, enhance one day  */
			/*  */
		}
		resource_Sources_Swapped = SAFE_FALSE;
	}

	/*  */
	/* Add to counter. Finished with it. */
	/*  */
	dream.backdrops.total_Backdrops += 1;
		
		
	function_Result = SAFE_TRUE;
	goto Normal_Function_Exit;
	
Error_Handler_With_Message:
	function_Result = SAFE_FALSE;
	Display_Serious_Error_Message(unit_Name, code_Name, note_String, "\p");
	goto Normal_Function_Exit;
	
Error_Handler:
	Display_Serious_Error_Message(unit_Name, code_Name, "\pInternal problem getting Backdrop", "\p");
	function_Result = SAFE_FALSE;
		
Normal_Function_Exit:

	if (backdrop_Header_Memory_In_Use == MEMORY_INITIALIZED) {
		ReleaseResource((Handle)backdrop_Header_Hdl);
		if (ResError() != noErr) {
			/*  */
			/* Ignore error, we are exiting anyway  */
			/*  */
		}
		backdrop_Header_Memory_In_Use = MEMORY_NOT_INITIALIZED;
	}
		
	if (resource_Sources_Swapped == SAFE_TRUE) {
		CloseResFile(input_File_Reference);
		if (ResError() != noErr) {

		}
		
		UseResFile(save_Ref);
		if (ResError() != noErr) {
			/*  */
			/* This should be recoverable from, enhance one day  */
			/*  */
		}
		resource_Sources_Swapped = SAFE_FALSE;
	}
	
	return(function_Result);
	
} /* Safe_Boolean	Load_Or_Import_Backdrop_File... */
/*=**************************************************************************=*/


/*=**************************************************************************=*/
/* END OF FILE: Dreaming_Backdrop_Files.c */
