/* START OF FILE: Dreaming_Color_Stuff.c */


/*=**************************************************************************=*/
/* [---BEGIN---] FILE HEADER COMMENTS */
/**
 ** SCREEN VIEW = [Adobe Syntax Black-10, Tabs = 4]
 ** PRINTER VIEW = [Adobe Syntax Black-10, Tabs = 4]
 ** =======================================================================
 ** Written by James Merkle
 ** Dreaming_Color_Stuff.c - Color Routines to support Poor Man's Video Phone
 ** (C) Copyright 1993 James Merkle
 ** All rights reserved, except those wishing to "copy" moderate sections of code or files may do so, so long as
 ** such code is never used directly either in animation-related tools or multimedia tools that are  
 ** used in live performance or live broadcast or recorded mass market broadcast or film. Desktop usage not restricted.
 ** Conventional desktop game or entertainment not restricted.
 ** =======================================================================
 **
 **
 ** ============
 ** Revision History
 ** ============
 **
 **  1993.02.01		(James Merkle)
 **						Initial coding started, from scratch
 **
 **
 **
 ** =================
 ** OTHER INFORMATION
 ** =================
 ** 
 ** 
 **
 **
 **/
/* [----END----] FILE HEADER COMMENTS */
/*=**************************************************************************=*/


/*=**************************************************************************=*/
/* [---BEGIN---] #include AREA */

#include <GestaltEqu.h>
#include "OS_Utilities.h"
#include "Core_Specific.h"
#include "Core_Universal.h" /* Non_Specific_Support_Routines */
#include "String_Utilities.h"
#include "Error_Utilities.h"
#include "Shell_Assembly.h"
#include "Shell_Dialogs.h"

#include "Shell_Globals.h"

#include "Dreaming_Color_Stuff.h"

/* [----END----] #include AREA */
/*=**************************************************************************=*/


/*=**************************************************************************=*/
/* [---BEGIN---] #define AREA */

#define	k_ANIMATION_CLUT_RES_ID				132
#define	k_ANIMATED_CLUT_TOTAL_COLORS		256

/* [----END----] #define AREA */
/*=**************************************************************************=*/


/*=**************************************************************************=*/
/* [---BEGIN---] TYPE AREA */

	
/* [----END----] TYPE AREA */
/*=**************************************************************************=*/


/*=**************************************************************************=*/
/* [---BEGIN---] PROTOTYPE AREA */


/* [----END----] PROTOTYPE AREA */
/*=**************************************************************************=*/


/*=**************************************************************************=*/
/* [---BEGIN---] GLOBAL DEFINITION AREA */

Safe_Boolean		g_Permit_Palette_Colors;
Safe_Boolean		g_Force_All_To_Use_Main_Puppet_Colors;

/* [----END----] GLOBAL DEFINITION AREA */
/*=**************************************************************************=*/


/*=**************************************************************************=*/
/* [---BEGIN---] LOCAL STATIC DEFINITION AREA */

static CARDINAL_8	*unit_Name = "\pDreaming_Color_Stuff";

/* [----END----] LOCAL STATIC DEFINITION AREA */
/*=**************************************************************************=*/


/*=**************************************************************************=*/
Safe_Boolean Force_Pallete_And_Clut_To_Match_Puppet(WindowPtr window_Ptr
																								, Puppet_Ptr current_Puppet)
/*  */
/* Used nowadays to force screen to equal puppet CLUT */
/* This is needed before background or any other art is loaded. */
/* Ideally another routine will hunt out other art and remap pixel values. */
/* Ideally another process could remap Puppet pixels. */
/* Ideally a simple solution would be to always use a special NTSC */
/* CLUT with a few colors at the high end reserved for pallette animation and */
/* special setting */
/*  */
/* "g_Permit_Palette_Colors" and "g_Force_All_To_Use_Main_Puppet_Colors"
/* are somewhat related to how this routine gets called. */
/*  */
{
	Safe_Boolean		function_Result;
	CARDINAL_8		*code_Name;
	INTEGER_16		total_Clut_Entries;
	Off_Screen_Ptr	offscreen_Ptr;
	PaletteHandle	palette_Hdl;
/* END VAR */

	code_Name = "\pForce_Pallete_And_Clut_To_Match_Puppet";
	function_Result = SAFE_FALSE;
			
	if (Check_If_Legal_Direct_Art_Or_Complain(current_Puppet) != SAFE_TRUE) {
		goto Error_Handler;
	}

	offscreen_Ptr = &(current_Puppet->direct_Art.port);
	
	if (offscreen_Ptr->initialized != SAFE_TRUE) {
		goto Error_Handler;
	}
		
	total_Clut_Entries = (**(offscreen_Ptr->color_Table_Handle)).ctSize + 1;
	if (total_Clut_Entries <= 1) {
		/*  */
		/* What the hell? Should be 256 colors at least anyways */
		/*  */
		goto Error_Handler;
	}
		
	/*  */
	/* "pmExplicit" could also be used here instead of "pmTolerant" */
	/* Better yet we could search for colors we do actually need and set them */
	/* to explicit instead of Tolerant */
	/*  */
	palette_Hdl = NewPalette(total_Clut_Entries
												, offscreen_Ptr->color_Table_Handle
												, pmExplicit
												, 0 );
			
	if (g_Permit_Palette_Colors == SAFE_TRUE) {
		NSetPalette(window_Ptr, palette_Hdl, pmFgUpdates);
	}
	
	/*  */
	/* Now force CLUT */
	/*  */
	
	if (g_Permit_Palette_Colors == SAFE_TRUE) {
		ActivatePalette(window_Ptr);
	}
	
	DisposePalette(palette_Hdl);
	
	function_Result = SAFE_TRUE;
	goto Normal_Function_Exit;
	
Error_Handler:
	function_Result = SAFE_FALSE;
	Display_Serious_Error_Message(unit_Name, code_Name, "\pUnexpected empty color table", "\pWrong usage orgder?");

Normal_Function_Exit:

	return(function_Result);

} /* Safe_Boolean Force_Pallete_And_Clut_To_Match_Puppet ... */
/*=**************************************************************************=*/


/*=**************************************************************************=*/
Safe_Boolean Create_Palette_From_Clut(CTabHandle source_Clut, PaletteHandle	*source_Hdl)
{
	Safe_Boolean		function_Result;
	CARDINAL_8		*code_Name;
	INTEGER_16		total_Clut_Entries;
/* END VAR */


	code_Name = "\pCreate_Pallette_From_Clut";
	function_Result = SAFE_FALSE;

	if (source_Clut == nil) {
		goto Error_Handler;
	}
	
	/*  */
	/* This is not off by one. IM specifies it this way */
	/*  */
	total_Clut_Entries = (**source_Clut).ctSize + 1;
	
	*source_Hdl = nil;
	*source_Hdl = NewPalette(total_Clut_Entries 
									,source_Clut
									, pmTolerant 
									, k_PUPPET_INTOLERENT);	
	
	if (*source_Hdl == nil) {
		goto Error_Handler;
	}
	
	
	function_Result = SAFE_TRUE;
	goto Normal_Function_Exit;
	
Error_Handler:
	function_Result = SAFE_FALSE;
		
Normal_Function_Exit:

	return(function_Result);
	
} /* Safe_Boolean Create_Palette_From_Clut... */
/*=**************************************************************************=*/


/*=**************************************************************************=*/
Safe_Boolean Activate_Palette_And_Verify_Intolerant(WindowPtr window_Ptr
																								, PaletteHandle	palette_Hdl)
/*  */
/* Not used during fast loads. Only used when importing PICS */
/*  */
{
	Safe_Boolean		function_Result;
	INTEGER_16		total_Colors;
	SIMPLE_LOOP	loop;
	CARDINAL_8		*code_Name;
	Boolean				keep_Looking;
	INTEGER_16		total_Clut_Entries;
	Str255					message_String;
/* END VAR */

	code_Name = "\pActivate_Palette_And_Verify_Intolerant";
	function_Result = SAFE_FALSE;

	if (palette_Hdl == nil) {
		goto Error_Handler;
	}
	
	/*  */
	/* If there are colors to be aware of,  force them into the */
	/* current palette now, after forcing the non-special colors */
	/* to be more tolerant. */
	/*  */
	total_Colors = (**palette_Hdl).pmEntries;
	if (total_Colors != 0) {
		if (g_Permit_Palette_Colors == SAFE_TRUE) {
			NSetPalette(window_Ptr, palette_Hdl, pmFgUpdates);
			ActivatePalette(window_Ptr);
		}
	}
	
	/*  */
	/* Verify that all colors are in the current CLUT */
	/*  */
	keep_Looking = TRUE;
	for (loop = 0
		; ((loop < total_Colors) && (keep_Looking == TRUE))
			; loop++) {
			
		keep_Looking = RealColor(&((**(palette_Hdl)).pmInfo[loop].ciRGB));
		
		if (keep_Looking != TRUE) {
			if ((**(palette_Hdl)).pmInfo[loop].ciUsage == pmTolerant) {
				if ((**(palette_Hdl)).pmInfo[loop].ciTolerance > k_PUPPET_INTOLERENT) {
					/*  */
					/* We just proved this color was not intolerant so we can ignore its absence */
					/*  */
					keep_Looking = TRUE;
				}
			}
		}
		
	}
	
	if (keep_Looking != TRUE) {
		SysBeep(10);
		Get_NS_String(k_DREAM_STRINGS, k_MISSING_COLOR_INDEX, message_String);
		Perform_OK_Dialog(message_String);
	}
	
	function_Result = SAFE_TRUE;
	goto Normal_Function_Exit;
	
Error_Handler:
	function_Result = SAFE_FALSE;
		
Normal_Function_Exit:

	return(function_Result);

} /* Safe_Boolean Activate_Palette_And_Verify_Intolerant ... */
/*=**************************************************************************=*/


/*=**************************************************************************=*/
Safe_Boolean Ask_User_For_Another_Clut(CTabHandle *located_Clut)
{
	Safe_Boolean		function_Result;
	Str255					temp_Part_1;
	Safe_Boolean		resource_Sources_Swapped;	
	Str255					message_String;
	INTEGER_16 		number_Of_Types;
	INTEGER_16		save_Ref;
	INTEGER_16		total_Cluts;
	SFTypeList 			type_List;
	SFReply				sf_Reply;
	INTEGER_16		source_Volume_Reference;
	INTEGER_16		input_File_Reference;
	INTEGER_32		source_Directory_ID;
	Boolean				user_Clicked_Ok;
	OSErr					os_Error;
	CARDINAL_8		*code_Name;
/* END VAR */


	code_Name = "\pAsk_User_For_Another_Clut";
	function_Result = SAFE_FALSE;
	*located_Clut = nil;

	Get_NS_String(k_DREAM_STRINGS, k_LOCATE_PICT_CONTAINING_A_CLUT_INDEX, temp_Part_1);

	number_Of_Types = 1;
	type_List[0] = 'PICT';
	
	user_Clicked_Ok = Perform_Get_File_Dialog(number_Of_Types
										, &(type_List)
										, &sf_Reply);
											
	if ((user_Clicked_Ok != TRUE) || (sf_Reply.good != TRUE)) {
		/*  */
		/* User Abort, no file name */
		/*  */
		function_Result = SAFE_FALSE;
		goto Normal_Function_Exit;
	}
	
	Get_Last_Volume_Reference_And_Directory_ID(&source_Volume_Reference, &source_Directory_ID);
										
	/*  */
	/* Open for read only permission */
	/*  */
	
	save_Ref = CurResFile();
	
	input_File_Reference = HOpenResFile (source_Volume_Reference
						, source_Directory_ID
						, sf_Reply.fName
						, fsRdPerm );
						
	if ((input_File_Reference == -1) || (ResError() != noErr)) {
		goto Error_Handler;
	}
													
	UseResFile(input_File_Reference);
	os_Error = ResError();
	if (os_Error != noErr) {
		
		Display_Serious_Error_Message(unit_Name, code_Name, "\pInternal using resource", "\pCorrupted?");
		goto Error_Handler;
	}
	
	resource_Sources_Swapped = SAFE_TRUE;
			
	total_Cluts = Count1Resources('clut');
	if (ResError() != noErr) {
		goto Error_Handler;
	}
		
	if (total_Cluts < 1) {
		Copy_Pascal_String("\pThat file Did not contain a separate color table. Sorry.", message_String);
		Perform_OK_Dialog(message_String);
		goto Error_Handler;
	}
	
	*located_Clut = nil;
	*located_Clut = (CTabHandle)Get1IndResource ('clut', 1);
	if (*located_Clut == nil){
		Display_Serious_Error_Message(unit_Name
															, code_Name
															, "\pInternal problem with resources ('clut')"
															, "\pCorrupted?");
		goto Error_Handler;
	}
	
	DetachResource((Handle)*located_Clut);
	if (ResError() != noErr) {
		goto Error_Handler;
	}
	
	function_Result = SAFE_TRUE;
	goto Normal_Function_Exit;
	
Error_Handler:
	function_Result = SAFE_FALSE;
		
Normal_Function_Exit:
	if (resource_Sources_Swapped == SAFE_TRUE) {
		CloseResFile(input_File_Reference);
		if (ResError() != noErr) {
			goto Error_Handler;
		}
		
		UseResFile(save_Ref);
		if (ResError() != noErr) {
			/*  */
			/* This should be recoverable from, enhance one day  */
			/*  */
		}
		resource_Sources_Swapped = SAFE_FALSE;
	}

	return(function_Result);
	
} /* Safe_Boolean Ask_User_For_Another_Clut ... */
/*=**************************************************************************=*/


/*=**************************************************************************=*/
Safe_Boolean Determine_If_Another_Clut_Wanted(CARDINAL_32 total_Colors_In_File
																					, CARDINAL_32  actual_Total_Of_Special_Colors)
{
	Safe_Boolean		function_Result;
	Str255					output_Part_1;
	Str255					output_Part_2;
	Str255					temp_Part_1;
	Str255					temp_Part_2;
/* END VAR */


	function_Result = SAFE_FALSE;

	Get_NS_String(k_DREAM_STRINGS, k_UNIQUE_COLORS_1_INDEX, temp_Part_1);
	Get_NS_String(k_DREAM_STRINGS, k_UNIQUE_COLORS_2_INDEX, temp_Part_2);
	Make_String_Long_String(&output_Part_1[0]
												, temp_Part_1
												, total_Colors_In_File
												, NO_LIMIT
												, temp_Part_2);
												
	Get_NS_String(k_DREAM_STRINGS, k_TOTAL_COLORS_1_INDEX, temp_Part_1);
	Get_NS_String(k_DREAM_STRINGS, k_TOTAL_COLORS_2_INDEX, temp_Part_2);
	Make_String_Long_String(&output_Part_2[0]
												, temp_Part_1
												, actual_Total_Of_Special_Colors
												, NO_LIMIT
												, temp_Part_2);
												

	Concat_After_Pascal_String(output_Part_1, output_Part_2);
	
	if (Perform_Yes_No_Dialog(output_Part_1) == TRUE) {
		function_Result = SAFE_TRUE;
	}
	else {
		function_Result = SAFE_FALSE;
	}

	goto Normal_Function_Exit;
	
Error_Handler:
	function_Result = SAFE_FALSE;
		
Normal_Function_Exit:

	return(function_Result);
	
} /* Safe_Boolean Determine_If_Another_Clut_Wanted ... */
/*=**************************************************************************=*/


/*=**************************************************************************=*/
Safe_Boolean Tally_Colors_Not_In_Default_Clut(PaletteHandle	source_Hdl
																		, CARDINAL_32 *total_Colors
																		, Safe_Boolean	reset_Common_To_Tolerant)
{	
	Safe_Boolean		function_Result;
	RGBColor			*color_1_Ptr;
	RGBColor			*color_2_Ptr;
	CTabHandle		default_Clut;
	ColorSpecPtr		color_Spec_Ptr;
	CARDINAL_32	color_Table_Base;
	INTEGER_16		total_Palette_Entries;
	INTEGER_16		total_Clut_Entries;
	SIMPLE_LOOP	loop;
	INTEGER_16		clut_Index;
	CARDINAL_16	total_Unique_Colors;
	Boolean				found_Match;
	CARDINAL_8		*code_Name;
/* END VAR */


	code_Name = "\pTally_Colors_Not_In_Default_Clut";
	function_Result = SAFE_FALSE;
	
	default_Clut = nil;
	default_Clut = (CTabHandle)Get1Resource ('clut', k_DEFAULT_256_COLOR_SYSTEM_CLUT);
	if (default_Clut == nil){
		Display_Serious_Error_Message(unit_Name
															, code_Name
															, "\pInternal problem with resources ('clut')"
															, "\pCorrupted?");
		goto Error_Handler;
	}
	
	total_Clut_Entries = (**default_Clut).ctSize + 1;
	color_Table_Base = (CARDINAL_32)&(**default_Clut).ctTable;
	
	total_Unique_Colors = 0;
	total_Palette_Entries = (**source_Hdl).pmEntries;
	for (loop = 0; (loop < total_Palette_Entries); loop++) {
			
		color_1_Ptr = &((**source_Hdl).pmInfo[loop].ciRGB);
		
		found_Match = FALSE;
		for (clut_Index = 0; ((clut_Index < total_Clut_Entries) && (found_Match != TRUE)); clut_Index++) {
			color_Spec_Ptr = (ColorSpecPtr)(color_Table_Base + (CARDINAL_32)(clut_Index * sizeof(ColorSpec)));
			color_2_Ptr = &(color_Spec_Ptr->rgb);
			
			if ((color_1_Ptr->red == color_2_Ptr->red)
					&& (color_1_Ptr->green == color_2_Ptr->green)
					&& (color_1_Ptr->blue == color_2_Ptr->blue)) {
					
				found_Match = TRUE;
				
				if (reset_Common_To_Tolerant == SAFE_TRUE) {
					SetEntryUsage(source_Hdl, loop, pmTolerant, k_PUPPET_TOLERENT);
				}
			}
		}
		
		if (found_Match != TRUE) {	
			total_Unique_Colors += 1;
		}
		
	} /* 	for (loop = 0; (loop < total_Palette_Entries) ... */
	
	/*  */
	/* Put away the resource, we extracted the information we wanted */
	/*  */
	ReleaseResource((Handle)default_Clut);
	if (ResError() != noErr) {
		goto Error_Handler;
	}
	
	*total_Colors = total_Unique_Colors;
	function_Result = SAFE_TRUE;
	goto Normal_Function_Exit;
	
Error_Handler:
	function_Result = SAFE_FALSE;
		
Normal_Function_Exit:

	return(function_Result);
	
} /* Safe_Boolean Tally_Colors_Not_In_Default_Clut ... */
/*=**************************************************************************=*/
	
	
/*=**************************************************************************=*/
Safe_Boolean Setup_Pallette_Animation(CARDINAL_32	window_Index)
{
	OSErr						os_Result;
	INTEGER_32			feature;
	RGBColor				black = {0,0,0};
	WindowPtr 			window_Ptr;
	Safe_Boolean			function_Result;
	Color_Animation_Ptr	color_Flow;
	CARDINAL_16		total_Entries;
	CARDINAL_32		memory_Requested;
	INTEGER_8				original_State;
/* END VAR */


	function_Result = SAFE_FALSE;
	
	/*  */
	/* HACK HACK HACK */
	/*  */
	goto Normal_Function_Exit;
	


/*  */
/* Use Gestalt to find 32 Bit Color QD */
/*  */
	if (Gestalt_Available() == TRUE) {
		os_Result = Gestalt(gestaltQuickdrawVersion, &feature);
		if (os_Result == noErr) {
			if ((feature & 0x0F00) != 0x0200)
				goto Error_Handler;
			}
		else {
			goto Error_Handler;
		}
	}
	else {
		goto Error_Handler;
	}

	/*  */
	/* This is dependant on the validity of "window_Index" */
	/*  */
	if ((active_Window_List[window_Index] == nil)
		|| ((**active_Window_List[window_Index]).structure_Initialized != SAFE_TRUE)) {
		goto Error_Handler;
	}
	else {
		window_Ptr	= (**active_Window_List[window_Index]).status_Window;
	}
	
	color_Flow = &(**active_Window_List[window_Index]).flow_Colors;
	
	if (color_Flow->animation_Active != SAFE_TRUE) {
	
		color_Flow->animation_Color_Table = nil;
		color_Flow->animation_Palette = nil;
	
		/*  */
		/* Create a new empty color table */
		/*  */
	
		memory_Requested = (sizeof(ColorTable) - sizeof(CSpecArray));
		color_Flow->animation_Color_Table_In_Use = MEMORY_NOT_INITIALIZED;
		if (Create_Handle( (CARDINAL_32 *)&memory_Requested
								, (CARDINAL_32)0L
								, ALLOW_PURGING
								, DISALLOW_PARTIAL_AMOUNT
								, (Handle *)&(color_Flow->animation_Color_Table)
								, &(color_Flow->animation_Color_Table_In_Use)) != SAFE_TRUE ) {
								
			Perform_OK_Dialog("\pMemory error allocating buffer_Memory (not enough?) (3)");

			goto Normal_Function_Exit;
		}
		
		/*  */
		/* 1995.05.21 JAM wierd Think C 7.0.3 bug with headers 2.0 */
		/* Apple declares (SignedByte is a signed char, but it seems incompatible with INTEGER_8 define! */
		/* Fucking hell! The reason is Think C thinks a "char *" is not equal to a "signed char *" */
		/* and that means it is not ANSI C compliant! */
		/*  */

		if (Begin_Use_Of_Handle((Handle)color_Flow->animation_Color_Table
								, MOVE_HIGH_REQUIRED
								, (signed char *)&original_State) != SAFE_TRUE) {
			/*  */
			/* Ignore error in implementation */
			/*  */
		}

		Fill_Byte_Memory_With_Byte((Ptr)*(color_Flow->animation_Color_Table), memory_Requested, 0);
		color_Flow->animation_Palette = GetPalette(window_Ptr);
		if (color_Flow->animation_Palette == nil) {
			color_Flow->animation_Palette = GetPalette((WindowPtr)-1);
			if ((**(color_Flow->animation_Palette)).pmEntries != 0) {	
				if (g_Permit_Palette_Colors == SAFE_TRUE) {
					NSetPalette(window_Ptr, color_Flow->animation_Palette, pmFgUpdates);
					ActivatePalette(window_Ptr);
				}
			}
			else {
				goto Normal_Function_Exit;
			}
		}
		
		Palette2CTab(color_Flow->animation_Palette, color_Flow->animation_Color_Table);
		/*  */
		/* Count colors in the CTabHandle for safety in cycling usage */
		/*  */
		total_Entries = (**(color_Flow->animation_Color_Table)).ctSize + 1;
		if (total_Entries <= 2) {
				goto Normal_Function_Exit;
		}
		
		color_Flow->animation_Color_Increment = 1;
		color_Flow->animation_Rotation_Point = 0;
		color_Flow->animation_Active = SAFE_TRUE;
	}
	else {
		/*  */
		/* Already setup !!! Nothing to do. Strange */
		/*  */
	}
	
	function_Result = SAFE_TRUE;
	goto Normal_Function_Exit;

Error_Handler:
	function_Result = SAFE_FALSE;
		
Normal_Function_Exit:

	return(function_Result);
	
} /* Safe_Boolean Setup_Pallette_Animation(CARDINAL_32	window_Index) */
/*=**************************************************************************=*/
	
	
/*=**************************************************************************=*/
Safe_Boolean Animate_Color_Table_For_Window(CARDINAL_32	window_Index)
{
	WindowPtr 			window_Ptr;
	RGBColor				animation_Change_Color;
	Color_Animation_Ptr	color_Flow;
	Safe_Boolean			function_Result;
	PaletteHandle		color_Pallette;
	CARDINAL_16		total_Entries;
	CARDINAL_16		first_Entry;
	CARDINAL_16		last_Entry;
	CARDINAL_16		current_Entry;
/* END VAR */


	function_Result = SAFE_FALSE;
	
	
	/*  */
	/* HACK HACK HACK */
	/*  */
	goto Normal_Function_Exit;


	/*  */
	/* This is dependant on the validity of "window_Index" */
	/*  */
	if ((active_Window_List[window_Index] == nil)
		|| ((**active_Window_List[window_Index]).structure_Initialized != SAFE_TRUE)) {
		goto Error_Handler;
	}
	else {
		window_Ptr	= (**active_Window_List[window_Index]).status_Window;
	}
	
	color_Flow = &(**active_Window_List[window_Index]).flow_Colors;

	/*  */
	/* Animate the colors one step for each invocaton */
	/*  */
	if (color_Flow->animation_Active == SAFE_TRUE) {
	
		color_Pallette = GetPalette(window_Ptr);
		if (color_Pallette == nil) {
			color_Pallette = GetPalette((WindowPtr)-1);
			if ((**(color_Pallette)).pmEntries != 0) {	
				if (g_Permit_Palette_Colors == SAFE_TRUE) {
					NSetPalette(window_Ptr, color_Pallette, pmFgUpdates);
					ActivatePalette(window_Ptr);
				}
			}
			else {
				goto Normal_Function_Exit;
			}
		}
		
		/*  */
		/* Reset for no good reason? */
		/*  */
		color_Flow->animation_Palette = color_Pallette;
		
		total_Entries = (**(color_Flow->animation_Color_Table)).ctSize + 1;
		
		/*  */
		/* Preserve white */
		/*  */
		first_Entry = 1;
		
		/*  */
		/* Preserve black */
		/*  */
		last_Entry = total_Entries - 1;
		
		/*  */
		/* At this point move up the color ramps to equal the range we want */
		/* ADD LATER... */
		/*  */
		if (last_Entry <= first_Entry) {
			goto Error_Handler;
		}
		else {
			total_Entries = (last_Entry - first_Entry);
		}

		current_Entry = color_Flow->animation_Rotation_Point;

		if (current_Entry < first_Entry) {
			current_Entry = first_Entry;
		}
		else if (current_Entry > last_Entry) {
			current_Entry = last_Entry;
		}

		AnimatePalette(window_Ptr
			, color_Flow->animation_Color_Table
			, current_Entry
			, first_Entry
			, ((last_Entry + 1) - current_Entry));
		
		AnimatePalette(window_Ptr
			, color_Flow->animation_Color_Table
			, first_Entry
			, first_Entry + ((last_Entry + 1) - current_Entry)
			, (current_Entry - first_Entry));
		
		current_Entry += color_Flow->animation_Color_Increment;
		
		/*  */
		/* Increment rotation point */
		/*  */
		while (current_Entry < first_Entry) {
			current_Entry = last_Entry - ((first_Entry - current_Entry) % total_Entries);
		}
		
		while (current_Entry > last_Entry) {
			current_Entry = ((current_Entry - last_Entry) % total_Entries) + first_Entry;
		}
		
		color_Flow->animation_Rotation_Point = current_Entry;
		
		/* ======
		
		GetEntryColor (color_Flow->animation_Palette, 1, &animation_Change_Color);

		AnimatePalette(window_Ptr
			, color_Flow->animation_Color_Table
			, 2
			, 1
			, k_ANIMATED_CLUT_TOTAL_COLORS - 2);
		
		AnimateEntry(window_Ptr
			, k_ANIMATED_CLUT_TOTAL_COLORS - 1
			, &animation_Change_Color);

		Palette2CTab (color_Flow->animation_Palette
			, color_Flow->animation_Color_Table);
		===== */
	}
	
	function_Result = SAFE_TRUE;
	goto Normal_Function_Exit;

Error_Handler:
	function_Result = SAFE_FALSE;
		
Normal_Function_Exit:

	return(function_Result);
	
} /* Safe_Boolean Animate_Color_Table_For_Window ... */
/*=**************************************************************************=*/


/*=**************************************************************************=*/
/* END OF FILE: Dreaming_Color_Stuff.c */
