/* START OF FILE: Dreaming_File_Loading.c */


/*=**************************************************************************=*/
/* [---BEGIN---] FILE HEADER COMMENTS */
/**
 ** SCREEN VIEW = [Adobe Syntax Black-10, Tabs = 4]
 ** PRINTER VIEW = [Adobe Syntax Black-10, Tabs = 4]
 ** =======================================================================
 ** Written by James Merkle
 ** Dreaming_File_Loading.c - File Routines to support Poor Man's Video Phone
 ** (C) Copyright 1993 James Merkle
 ** All rights reserved, except those wishing to "copy" moderate sections of code or files may do so, so long as
 ** such code is never used directly either in animation-related tools or multimedia tools that are  
 ** used in live performance or live broadcast or recorded mass market broadcast or film. Desktop usage not restricted.
 ** Conventional desktop game or entertainment not restricted.
 ** =======================================================================
 **
 **
 ** ============
 ** Revision History
 ** ============
 **
 **  1993.05.23		(James Merkle)
 **						Initial coding started, from scratch
 **
 **
 **
 ** =================
 ** OTHER INFORMATION
 ** =================
 ** 
 ** 
 **
 **
 **/
/* [----END----] FILE HEADER COMMENTS */
/*=**************************************************************************=*/


/*=**************************************************************************=*/
/* [---BEGIN---] #include AREA */

#include "Core_Universal.h" /* Non_Specific_Support_Routines */
#include "Core_Specific.h"
#include "Core_Globals.h"
#include "String_Utilities.h"
#include "Error_Utilities.h"
#include "Shell_Dialogs.h"
#include "Shell_Globals.h"
#include "Volume_Utilities.h"
#include "SysHooks.h"
#include "Command_Dialogs.h"
#include "Window_Environment.h"
#include "Cursor_Utilities.h"
#include "Dreaming_Scene_Files.h"
#include "Interface_Utilities.h"
#include "Spin_O_Rama_Files.h"
#include "File_Resource_Utilities.h"
#include "Dreaming_Stuff.h"
#include "Dreaming_Color_Stuff.h"
#include "Dreaming_Popup_Dialogs.h"
#include "Spin_O_Rama_Files.h"
#include "Object_Inventory.h"

#include "Dreaming_File_Loading.h"

/* [----END----] #include AREA */
/*=**************************************************************************=*/


/*=**************************************************************************=*/
/* [---BEGIN---] #define AREA */


#define	k_EXTRA_PAD_AMOUNT								8
	
#define	k_PUPPET_EXPANDABLE_RESOURCE_TYPE				'Pup2'
#define	k_PUPPET_REDIRECTED_FILES_RESOURCE_TYPE		'Pup3'

#define	k_PUPPET_HEADER_TEMPLATE_NAME						"\pPupp"
#define	k_PUPPET_BODY_TEMPLATE_NAME							"\pPup2"
#define	k_PUPPET_REDIRECTED_FILES_TEMPLATE_NAME	"\pPup3"

#define	k_PUPPET_APPLE_REGION_RESOURCE_TYPE	'Rgn1'
#define	k_PUPPET_PALLETE_RESOURCE_TYPE				'pltt'
#define	k_PUPPET_CLUT_RESOURCE_TYPE						'clut'
#define	k_PUPPET_HEADER_TEMPLATE_ID						3000
#define	k_PUPPET_BODY_TEMPLATE_ID							3001
#define	k_PUPPET_REDIRECTED_FILES_TEMPLATE_ID	3009


#define	k_PUPPET_PALLETE_RESOURCE_BASE		128
#define	k_PUPPET_CLUT_RESOURCE_BASE				128


	/*  */
	/* "128" resevered for composite region */
	/*  */
#define	k_BASE_REGION_RESOURCE_ID		k_BASE_PUPPET_RESOURCE_ID

#define	k_PUPPET_VERSION_010000		0x00010000
#define	k_PUPPET_VERSION_010100		0x00010100

	/*  */
	/* Version "0x00010200" created 1995.06.07 and allows a */
	/* Pup3 list of redirected files to be present. */
	/*  */
#define	k_PUPPET_VERSION_010200		0x00010200

#define	k_PUPPET_VERSION_NOW					k_PUPPET_VERSION_010200

/*  */
/* k_PUPPET_BASE_VERSION_NOW must never be greater than */
/* k_PUPPET_VERSION_NOW. */
/*  */
#define	k_PUPPET_BASE_VERSION_NOW		k_PUPPET_VERSION_010000


/* [----END----] #define AREA */
/*=**************************************************************************=*/


/*=**************************************************************************=*/
/* [---BEGIN---] TYPE AREA */

	
/* [----END----] TYPE AREA */
/*=**************************************************************************=*/


/*=**************************************************************************=*/
/* [---BEGIN---] PROTOTYPE AREA */

Safe_Boolean	Enforce_Selected_Colors_If_Visible(CARDINAL_32	desired_Puppet);

Safe_Boolean	Export_Puppet_Sequence(CARDINAL_32 index_Used
															, Safe_Boolean *user_Abort);


/* [----END----] PROTOTYPE AREA */
/*=**************************************************************************=*/


/*=**************************************************************************=*/
/* [---BEGIN---] GLOBAL DEFINITION AREA */


/* [----END----] GLOBAL DEFINITION AREA */
/*=**************************************************************************=*/


/*=**************************************************************************=*/
/* [---BEGIN---] LOCAL STATIC DEFINITION AREA */

static CARDINAL_8	*unit_Name = "\pDreaming_File_Loading";

/* [----END----] LOCAL STATIC DEFINITION AREA */
/*=**************************************************************************=*/


/*=**************************************************************************=*/
Safe_Boolean	Export_Puppet_Sequence(CARDINAL_32 index_Used
															, Safe_Boolean *user_Abort)
{
	Str63					destination_File_Name;
	INTEGER_16		destination_Volume;
	INTEGER_16		scrap_Volume_Reference_Number;
	INTEGER_32		destination_Directory_ID;
	INTEGER_32		directory_Reference;
	INTEGER_16		file_Reference;
	CARDINAL_32	resource_Type;
	INTEGER_16		resource_ID;
	SIMPLE_LOOP	loop;
	CARDINAL_32	file_Creator;
	CARDINAL_32	file_Type;
	INTEGER_32		volume_Space;
	Boolean				user_Clicked_Ok;
	SFReply				sf_Reply;
	Puppet_Ptr			current_Puppet;
	PicHandle			picture_Handle;
	Boolean				problem_Getting_Picture;
	OSErr					os_Error;
	CARDINAL_8		*code_Name;
	Safe_Boolean		function_Result;
	
	Off_Screen_Port_Rec 		our_Temp_Port;
	Off_Screen_Ptr				offscreen_Ptr;

	CARDINAL_32 	pad_Width_Amount;
	CARDINAL_32 	pad_Height_Amount;
	INTEGER_32		desired_Color_Depth;
	CARDINAL_32	memory_Needed;
	INTEGER_32		total_Bytes_Contiguous_Now;
	INTEGER_32		total_Bytes_Now;
	INTEGER_32 		total_Bytes_Contiguous_Later;
	INTEGER_32		total_Bytes_Later;

	Rect						offscreen_Rect;
	Rect						source_Bit_Rect;
	Rect						recording_Area_Rect;
	INTEGER_32		source_Bit_Width;
	INTEGER_32		source_Bit_Height;

	Safe_Boolean		port_Was_Set;
	GrafPtr				saved_Original_Port;
	Rect						source_Rect;
	Rect						destination_Rect;
	Rect						puppet_Source_Rect;
	PenState				saved_Pen_State;
	GrafPtr				offscreen_Source_Port;
	GDHandle			saved_Device;
	RgnHandle			clipping_Region;
	
	
	Str255					message_String;
	Str255					note_String;
	Str63					suffix_String;
	Str63					new_Suffix_String;
	Str63					original_Name;
	Str63					volume_Name;
	Str63					resource_Comment;
/* END VAR */



	code_Name = "\pExport_Puppet_Sequence";
	function_Result = SAFE_FALSE;
	port_Was_Set = SAFE_FALSE;
	*user_Abort = SAFE_FALSE;
	clipping_Region = NewRgn();

		
	/*  */
	/* current_Puppet uses to a handle assumed to always be locked */
	/*  */
	current_Puppet = *(dream.puppet[index_Used]);
	
	if (Check_If_Legal_Direct_Art_Or_Complain(current_Puppet) != SAFE_TRUE) {
		goto Error_Handler;
	}

	/*  */
	/* Should ask these also */
	/*  */
/* ===
	current_Puppet->screen_Name;
	current_Puppet->screen_Short_Name;
=== */
		
	Get_NS_String(k_DREAM_STRINGS, k_EXPORT_PUPPET_PICS_PROMPT, message_String);
	Get_NS_String(k_DREAM_STRINGS, k_SAVE_PUPPET_FILE_SUFFIX, suffix_String);
	Get_NS_String(k_DREAM_STRINGS, k_EXPORT_PUPPET_PICS_FILE_SUFFIX, new_Suffix_String);

	
	file_Creator= FILE_CREATOR;
	file_Type = k_PUPPET_PICS_FILE_TYPE;

	Assign_String(original_Name, current_Puppet->file_Name);
	Strip_Off_Popular_Suffixes(&original_Name);
	/*  */
	/* Suffix not appeded if just space character */
	/*  */
	if ((suffix_String[0] != 1) || (suffix_String[1] != ASCII_SPACE)) {
		Concat_String(original_Name, new_Suffix_String);
	}
	
	if(original_Name[0] >31) {
		original_Name[0] = 31;
	}

	user_Clicked_Ok = Perform_Put_File_Dialog(message_String
										, original_Name
										, &sf_Reply);
	
	if ((user_Clicked_Ok != TRUE) || (sf_Reply.good != TRUE)) {
		/*  */
		/* User Abort, no file name */
		/*  */
		*user_Abort = SAFE_TRUE;
		function_Result = SAFE_FALSE;
		goto Normal_Function_Exit;
	}
	
	directory_Reference = sf_Reply.vRefNum;
	Copy_Pascal_String(sf_Reply.fName, destination_File_Name);
	Get_Last_Volume_Reference_And_Directory_ID(&(destination_Volume), &(destination_Directory_ID));

	note_String[0] = 0;
	
	os_Error = GetVInfo((Get_Drive_Number(destination_Volume)), volume_Name, &scrap_Volume_Reference_Number, (INTEGER_32 *)&volume_Space);
	if (os_Error != noErr) {
		Concat_Many(note_String, 3, "\pGetVInfo for volume ", volume_Name, "\p ID = ");
		Concat_Long_To_String(note_String, os_Error, NO_LIMIT);
		goto Error_Handler_With_Message;
	} /* if (os_Error != noErr) */
			
	os_Error = HCreate(destination_Volume
						, destination_Directory_ID
						, destination_File_Name
						, file_Creator
						, file_Type);
						
	if (os_Error == dupFNErr) {
		os_Error = HDelete(destination_Volume
								, destination_Directory_ID
								, destination_File_Name);
		if (os_Error != noErr) {
			Concat_Many(note_String, 3,"\pFSDelete for file ", destination_File_Name, "\p ID = ");
			Concat_Long_To_String(note_String, os_Error, NO_LIMIT);
			goto Error_Handler_With_Message;
		}

		os_Error = HCreate(destination_Volume
							, destination_Directory_ID
							, destination_File_Name
							, file_Creator
							, file_Type);
	} /* if (os_Error == dupFNErr) */

	if (os_Error != noErr) {
		Concat_Many(note_String, 3,"\pCreate for file ", destination_File_Name, "\p ID = ");
		Concat_Long_To_String(note_String, os_Error, NO_LIMIT);
		goto Error_Handler_With_Message;
	} 


	HCreateResFile(destination_Volume
						, destination_Directory_ID
						, destination_File_Name);
	os_Error = ResError();
	if (os_Error != noErr) {
		Concat_Many(note_String, 3,"\pCreateResFile for file ", destination_File_Name, "\p ID = ");
		Concat_Long_To_String(note_String, os_Error, NO_LIMIT);
		goto Error_Handler_With_Message;
	} 

	/*=--------------------------------------------------------------------------=*/
	/* 'OpenRFPerm' used to prevent having to use 'SetVol' */
	/* 'StripAddress' called as per tech notes and THINK reference */
	/*=--------------------------------------------------------------------------=*/
	file_Reference = OpenRFPerm(((StringPtr)(StripAddress(&destination_File_Name[0]))), directory_Reference, fsRdWrPerm );

	os_Error = ResError();
	if ((file_Reference == -1) || ( os_Error != noErr)) {
		Display_Serious_Error_Message(unit_Name, code_Name, "\pOS Error OpenResFile.", "\pNo resource fork?");
		goto Error_Handler;
	}

	resource_Type = 'PICT';
	resource_ID = k_BASE_PUPPET_RESOURCE_ID;
	Get_NS_String(k_DREAM_STRINGS, k_DONT_EDIT_RESOURCE_COMMENT, resource_Comment);
	
	problem_Getting_Picture = FALSE;

	if (Check_If_Legal_Direct_Art_Or_Complain(current_Puppet) == SAFE_TRUE) {
		goto Error_Handler;
	}

	if ((current_Puppet->direct_Art.frame_Total < 1) || (current_Puppet->direct_Art.original_Total_Lazy_Active_Region == nil)) {
		goto Error_Handler;
	}
		
	source_Bit_Rect = (**current_Puppet->direct_Art.original_Total_Lazy_Active_Region).rgnBBox;
	source_Bit_Width = source_Bit_Rect.right - source_Bit_Rect.left;
	source_Bit_Height = source_Bit_Rect.bottom - source_Bit_Rect.top;
		
	/*  */
	/* Set up the rectangle for the off-screen graphics environments */
	/* Stretch it a bit if there is room for border */
	/*  */

	if ((source_Bit_Width + k_EXTRA_PAD_AMOUNT) < g_Window_Limits.specific.display_Item_Width_Max) {
		pad_Width_Amount = k_EXTRA_PAD_AMOUNT;
	}
	else {
		pad_Width_Amount = 0;
	}
	
	if ((source_Bit_Height + k_EXTRA_PAD_AMOUNT) < g_Window_Limits.specific.display_Item_Height_Max) {
		pad_Height_Amount = k_EXTRA_PAD_AMOUNT;
	}
	else {
		pad_Height_Amount = 0;
	}
				
	SetRect(&offscreen_Rect
					, 0
					, 0
					, source_Bit_Width + pad_Width_Amount
					, source_Bit_Height + pad_Height_Amount);
		
	switch (current_Puppet->direct_Art.port.color_Type) {
		case k_COLOR_QUICKDRAW:
		case k_32_BIT_COLOR_QUICKDRAW:
			if (current_Puppet->direct_Art.port.image_Handle != nil) {
				desired_Color_Depth = (**(current_Puppet->direct_Art.port.offscreen_Port.color_Port.portPixMap)).pixelSize;
			}
			else {
				/*  */
				/* Cannot get here. paranoia code */
				/*  */
				desired_Color_Depth = 8;
			}
		break;
		
		case k_ORIGINAL_QUICKDRAW:
		default:
			desired_Color_Depth = 1;
	} /* switch (current_Puppet->direct_Art.port.color_Type) */

	offscreen_Ptr = &our_Temp_Port;
	offscreen_Ptr->color_Table_Handle = current_Puppet->direct_Art.port.color_Table_Handle;
	
	HandToHand((Handle *)&(offscreen_Ptr->color_Table_Handle));
	if (offscreen_Ptr->color_Table_Handle == nil) {
		goto Error_Handler;
	}
	
	if (Determine_Absolute_Free_Memory_Hook(&total_Bytes_Contiguous_Now
													,&total_Bytes_Now
													,&total_Bytes_Contiguous_Later
													,&total_Bytes_Later) != TRUE) {
		goto Error_Handler;
	}
	
	memory_Needed = desired_Color_Depth;
	memory_Needed = memory_Needed * (CARDINAL_32)(offscreen_Rect.right - offscreen_Rect.left);
	memory_Needed = memory_Needed * (CARDINAL_32)(offscreen_Rect.bottom - offscreen_Rect.top);
	memory_Needed = memory_Needed >> 3;
	
	if (total_Bytes_Contiguous_Later < (k_MINIMUM_RAM_TO_PRESERVE_FOR_BIG_STUFF + memory_Needed)) {
		total_Bytes_Contiguous_Later = MaxMem(&total_Bytes_Now);
		if (total_Bytes_Contiguous_Later < (k_MINIMUM_RAM_TO_PRESERVE_FOR_BIG_STUFF + memory_Needed)) {
			total_Bytes_Contiguous_Later = MaxMem(&total_Bytes_Now);
	
			Get_NS_String(k_DREAM_ERROR_STRINGS, k_ERROR_EXPORTING_PUPPETS_USES_MEMORY, note_String);
			Make_String_Long_String(&message_String[0], &note_String[0], memory_Needed, NO_LIMIT, "\p bytes.");
			Perform_OK_Dialog(message_String);
			function_Result = SAFE_FALSE;
			goto Normal_Function_Exit;
		}
	}
	
	os_Error= Create_Off_Screen_Port(&offscreen_Rect
															, desired_Color_Depth
															, offscreen_Ptr->color_Table_Handle
															, (CGrafPtr)&(offscreen_Ptr->offscreen_Port.color_Port)
															, &(offscreen_Ptr->offscreen_GDevice_Hdl)
															, &(offscreen_Ptr->image_Handle));

	if (os_Error != noErr) {
		goto Error_Handler;
	}
	
	offscreen_Ptr->initialized = SAFE_TRUE;
	offscreen_Ptr->color_Type = k_COLOR_QUICKDRAW;
	
/* ====
	Animate_Cursor(STANDARD_ACUR_ID, USE_VBL);
===== */
	Animate_Cursor(STANDARD_ACUR_ID, DO_NOT_USE_VBL);


	GetPort(&saved_Original_Port);
	saved_Device = GetGDevice();

	Set_View_Port((GrafPtr)&(offscreen_Ptr->offscreen_Port.color_Port));
	Set_View_Device(offscreen_Ptr->offscreen_GDevice_Hdl);

	port_Was_Set = SAFE_TRUE;
	
	GetPenState (&saved_Pen_State);

	if (current_Puppet->direct_Art.frame_Total > 0) {
	
		if (current_Puppet->direct_Art.original_Total_Lazy_Active_Region != nil) {
							
			for (loop = 0; (loop < current_Puppet->direct_Art.frame_Total) ; loop++) {
			
			/*  */
			/* Copy picture to an erased offscreen port, preferably using */
			/* the original background color used for framing */
			/*  */
						
				/*  */
				/* Set background erasing color */
				/*  */
				ForeColor(blackColor);
				BackColor(whiteColor);

				/*  */
				/* erase port rect completely */
				/*  */
				ClipRect(&(offscreen_Ptr->offscreen_Port.color_Port.portRect));
				EraseRect(&(offscreen_Ptr->offscreen_Port.color_Port.portRect));

				/*  */
				/* Actually needed for CopyBits calls , so be certain. */
				/*  */
				ForeColor(blackColor);
				BackColor(whiteColor);

				/*  */
				/* Important: shrink clipRgn. Also, "source_Bit_Rect" is already */
				/* guaranteed to be big enough */
				/*  */
					
				SetRect(&destination_Rect
								, 0
								, 0
								, (current_Puppet->direct_Art.frame_Locations[loop].right - current_Puppet->direct_Art.frame_Locations[loop].left)
								, (current_Puppet->direct_Art.frame_Locations[loop].bottom - current_Puppet->direct_Art.frame_Locations[loop].top));
					
				puppet_Source_Rect = current_Puppet->direct_Art.frame_Locations[loop];
																		
				OffsetRect(&puppet_Source_Rect
							, source_Bit_Rect.left
							, source_Bit_Rect.top);
									
				OffsetRect(&destination_Rect
									, pad_Width_Amount / 2
									, pad_Height_Amount / 2);
									
				CopyRgn(current_Puppet->direct_Art.frame_Regions[loop], clipping_Region);
				OffsetRgn(clipping_Region
					, ((pad_Width_Amount / 2) - source_Bit_Rect.left)
					, ((pad_Height_Amount / 2) - source_Bit_Rect.top));
									
				ClipRect(&destination_Rect);
				
				/*  */
				/* start recording */
				/*  */
				recording_Area_Rect = offscreen_Rect;
				picture_Handle = OpenPicture(&recording_Area_Rect );
										
				switch (current_Puppet->direct_Art.port.color_Type) {
					case k_COLOR_QUICKDRAW:
					case k_32_BIT_COLOR_QUICKDRAW:
						CopyBits ((const BitMap *)*(current_Puppet->direct_Art.port.offscreen_Port.color_Port.portPixMap)
										, (const BitMap *)*(offscreen_Ptr->offscreen_Port.color_Port.portPixMap)
										, &puppet_Source_Rect
										, &destination_Rect
										, srcCopy
										, clipping_Region);
		
					break;
					
					case k_ORIGINAL_QUICKDRAW:
					default:
					
						CopyBits (&(current_Puppet->direct_Art.port.offscreen_Port.old_Style_Port.portBits)
										, &(offscreen_Ptr->offscreen_Port.old_Style_Port.portBits)
										, &puppet_Source_Rect
										, &destination_Rect
										, srcCopy
										, clipping_Region);
								
				} /*  switch (g_Indicator.color_Type) */
						
				ClosePicture();
					
				if (Add_Resource_Handle((Handle)picture_Handle
														, resource_Type
														, resource_ID
														, resource_Comment) != SAFE_TRUE) {
					goto Error_Handler;
				}
				
				if (EmptyRect(&(**picture_Handle).picFrame) == TRUE) {
					problem_Getting_Picture = TRUE;
				}
				
				KillPicture(picture_Handle);
		
				resource_ID += 1;
						
			} /* for (loop = 0; (loop < current_Puppet->direct_Art.frame_Total) ; loop++) */
			
		} /* if ((current_Puppet->direct_Art.original_Total_Lazy_Active_Region != nil) */
		
	} /* if (current_Puppet->direct_Art.frame_Total > 0) */
	
	
	CloseResFile(file_Reference);
	os_Error = ResError();
	if ( os_Error != noErr) {
		Display_Serious_Error_Message(unit_Name, code_Name, "\pOS Error CloseResFile.", "\p");
	}
	
	if (problem_Getting_Picture == TRUE) {
		goto Error_Handler_No_Memory;
	}
	
	goto Normal_Function_Exit;

Error_Handler_No_Memory:
	Stop_Animating_Cursor();
	function_Result = SAFE_FALSE;
	Display_Serious_Error_Message(unit_Name, code_Name, "\pInternal problem saving PICS", "\pNo memory");
	goto Normal_Function_Exit;
	
Error_Handler_With_Message:
	Stop_Animating_Cursor();
	function_Result = SAFE_FALSE;
	Display_Serious_Error_Message(unit_Name, code_Name, note_String, "\p");
	goto Normal_Function_Exit;

Error_Handler:
	Stop_Animating_Cursor();
	Display_Serious_Error_Message(unit_Name, code_Name, "\pInternal problem saving PICS", "\p");
	function_Result = SAFE_FALSE;
		
Normal_Function_Exit:

	Stop_Animating_Cursor();
	if (port_Was_Set == SAFE_TRUE) {
		SetPenState (&saved_Pen_State);
		Set_View_Port(saved_Original_Port);
		Set_View_Device(saved_Device);
		Close_Offscreen_Pict_Port(offscreen_Ptr);
	}
	DisposeRgn(clipping_Region);
	return(function_Result);
	
} /* Safe_Boolean	Export_Puppet_Sequence ... */
/*=**************************************************************************=*/
	
	
/*=**************************************************************************=*/				
Safe_Boolean	Specify_Puppet_To_Load(FSSpec		*file_Specification
															, Safe_Boolean *user_Abort)
{
	INTEGER_16 		number_Of_Types;
	Boolean				user_Clicked_Ok;
	INTEGER_32		source_Directory_ID;
	INTEGER_16		source_Volume_Reference;
	SFTypeList 			type_List;
	SFReply				sf_Reply;
	Safe_Boolean		function_Result;
/* END VAR */


	function_Result = SAFE_FALSE;
	*user_Abort = SAFE_FALSE;

	number_Of_Types = 1;
	type_List[0] = k_PUPPET_FILE_TYPE;

	user_Clicked_Ok = Perform_Get_File_Dialog(number_Of_Types
										, &(type_List)
										, &sf_Reply);
											
	if ((user_Clicked_Ok != TRUE) || (sf_Reply.good != TRUE)) {
		/*  */
		/* User Abort, no file name */
		/*  */
		*user_Abort = SAFE_TRUE;
		function_Result = SAFE_FALSE;
		goto Normal_Function_Exit;
	}
	
	Get_Last_Volume_Reference_And_Directory_ID(&source_Volume_Reference, &source_Directory_ID);

	file_Specification->vRefNum = source_Volume_Reference;
	file_Specification->parID = source_Directory_ID;
	Copy_Pascal_String(sf_Reply.fName, file_Specification->name);
		
	function_Result = SAFE_TRUE;
	goto Normal_Function_Exit;

Error_Handler:
	function_Result = SAFE_FALSE;
		
Normal_Function_Exit:
	return(function_Result);
	
} /* Safe_Boolean	Specify_Puppet_To_Load... */
/*=**************************************************************************=*/
	
	
/*=**************************************************************************=*/				
Safe_Boolean	Load_Puppet_Sequence(CARDINAL_32 index_Used
															, INTEGER_16	puppet_Offset
															, FSSpec		*file_Specification)
{

	Puppet_File_Header_Ptr			header;
	SignedByte 								header_Original_State;
	Puppet_File_Header_Hdl			puppet_Header_Hdl;
	Memory_Allocation					puppet_Header_Memory_In_Use;
	Puppet_File_Ptr							body;
	SignedByte 								body_Original_State;
	Puppet_File_Hdl						puppet_Body_Hdl;
	Memory_Allocation					puppet_Body_Memory_In_Use;
	CTabHandle								temp_Clut_Handle;
	Rect												*first_Rect_Ptr;
	CARDINAL_16 							*resource_ID_Count_2_Ptr;
	CARDINAL_16 							*first_Resource_ID_Ptr;
	Safe_Boolean 							*end_Sentinel_Ptr;
	Off_Screen_Ptr 							offscreen_Ptr;
	Rect												offscreen_Rect;
	CARDINAL_16							desired_Bit_Depth;
	CARDINAL_32	handle_Size;
	INTEGER_16		save_Ref;
	INTEGER_32		transfer_Count;
	INTEGER_32		transfer_Count_Copy;
	WindowPtr			current_Graphics_Window_Ptr;
	Safe_Boolean		function_Result;
	Safe_Boolean		resource_Sources_Swapped;
	
	INTEGER_16		input_File_Reference;
	OSErr					os_Error;
	INTEGER_32		total_Bytes_Contiguous_Now;
	INTEGER_32		total_Bytes_Now;
	INTEGER_32 		total_Bytes_Contiguous_Later;
	INTEGER_32		total_Bytes_Later;
	CARDINAL_32	memory_Needed;
	CTabHandle		temp_Clut_Table;
	Puppet_Ptr			current_Puppet;
	INTEGER_32		loop;
	INTEGER_16		total_Puppet_Resources;
	INTEGER_16		current_Pict_Sequence;
	Boolean				still_Searching;
	CARDINAL_8		*code_Name;
	
	Puppet_Redirected_File_Hdl	puppet_Redirected_File_List_Hdl;
	Puppet_Redirected_File_Ptr		redirected_Info;
	SignedByte 								redirected_Info_Original_State;
	Memory_Allocation					puppet_Redirected_File_List_Memory_In_Use;
	Safe_Boolean								puppet_Contains_All_Artwork;
	Redirected_File_Ptr					file_Info_Ptr;
	Redirected_Art_Ptr					loaded_Info_Ptr;
	CARDINAL_32							preferred_Size;
	Load_Spin_Object_Rec				params_For_Load_Spin_Object;
	
	Safe_Boolean		address_Instead_Of_File;
	CARDINAL_32	item_Address;

	Str255					message_String;
	Str255					note_String;
/* END VAR */


	code_Name = "\pLoad_Puppet_Sequence";
	function_Result = SAFE_FALSE;
		
	puppet_Header_Memory_In_Use = MEMORY_NOT_INITIALIZED;
	puppet_Body_Memory_In_Use = MEMORY_NOT_INITIALIZED;
	puppet_Redirected_File_List_Memory_In_Use = MEMORY_NOT_INITIALIZED;

	resource_Sources_Swapped = SAFE_FALSE;
		

	if (Determine_Absolute_Free_Memory_Hook(&total_Bytes_Contiguous_Now
													,&total_Bytes_Now
													,&total_Bytes_Contiguous_Later
													,&total_Bytes_Later) != TRUE) {
		goto Error_Handler_Early_Stages;
	}
	
	if (total_Bytes_Contiguous_Later < k_MINIMUM_RAM_TO_PRESERVE_FOR_BIG_STUFF) {
		Get_NS_String(k_DREAM_ERROR_STRINGS, k_ERROR_NOT_ENOUGH_MEMORY_TO_SAFELY_LOAD, message_String);
		Perform_OK_Dialog(message_String);
		function_Result = SAFE_FALSE;
		goto Normal_Function_Exit;
	}

	/*  */
	/* Open for read only permission */
	/*  */
	
	save_Ref = CurResFile();
	input_File_Reference = HOpenResFile (file_Specification->vRefNum
									, file_Specification->parID
									, file_Specification->name
									, fsRdPerm );
						
	if ((input_File_Reference == -1) || (ResError() != noErr)) {
		goto Error_Handler_Early_Stages;
	}
													
	UseResFile(input_File_Reference);
	os_Error = ResError();
	if (os_Error != noErr) {
		Display_Serious_Error_Message(unit_Name, code_Name, "\pInternal using resource", "\pCorrupted?");
		goto Error_Handler_Early_Stages;
	}
	
	resource_Sources_Swapped = SAFE_TRUE;
	
	/*  */
	/* Load images into memory, allocating a graphics port */
	/* big enough to hold the images into one tall graphics device port */
	/*  */
		
	total_Puppet_Resources = Count1Resources(k_PUPPET_MAIN_RESOURCE_TYPE);
	if (ResError() != noErr) {
		goto Error_Handler_Early_Stages;
	}
	
	if (total_Puppet_Resources < 1) {
		Get_NS_String(k_DREAM_STRINGS, k_SORRY_WRONG_FILE_INDEX, message_String);
		Perform_OK_Dialog(message_String);
		goto Error_Handler_Early_Stages;
	}	
				
	/*  */
	/* current_Puppet uses to a handle assumed to always be locked */
	/*  */
	current_Puppet = *(dream.puppet[index_Used]);
	if (current_Puppet->direct_Art.frame_Total != 0 ) {
		Copy_Pascal_String("\pUNIMPLEMENTED: This version cannot append new Pict data into existing list", message_String);
		Perform_OK_Dialog(message_String);
		goto Error_Handler_Early_Stages;
	}
	
	puppet_Header_Hdl = (Puppet_File_Header_Hdl)Get1Resource(k_PUPPET_MAIN_RESOURCE_TYPE
																			, k_BASE_PUPPET_RESOURCE_ID + puppet_Offset);
	if ((ResError() != noErr) || (puppet_Header_Hdl == nil)) {
		goto Error_Handler_Early_Stages;
	}
	puppet_Header_Memory_In_Use = MEMORY_INITIALIZED;
	HNoPurge((Handle)puppet_Header_Hdl);
	if (MemError() != noErr) {
		/*  */
		/* This cannot fail, but check anyway */
		/*  */
		goto Error_Handler_Early_Stages;
	}
	
	puppet_Body_Hdl = (Puppet_File_Hdl)Get1Resource(k_PUPPET_EXPANDABLE_RESOURCE_TYPE
																			, k_BASE_PUPPET_RESOURCE_ID + puppet_Offset);
	if ((ResError() != noErr) || (puppet_Body_Hdl == nil)) {
		goto Error_Handler_Early_Stages;
	}
	puppet_Body_Memory_In_Use = MEMORY_INITIALIZED;
	HNoPurge((Handle)puppet_Body_Hdl);
	if (MemError() != noErr) {
		/*  */
		/* This cannot fail, but check anyway */
		/*  */
		goto Error_Handler_Early_Stages;
	}
	
	if (((**puppet_Header_Hdl).valid != SAFE_TRUE) || ((**puppet_Body_Hdl).valid != SAFE_TRUE)) {
		Copy_Pascal_String("\pThis file has corrupted resources and cannot be used safely", message_String);
		Perform_OK_Dialog(message_String);
		goto Error_Handler_Early_Stages;
	}

	/*  */
	/* Get information regarding redirected artwork. Redirected */
	/* puppets can be large multifile puppets that stream direct from */
	/* disk and are used for 3D, multi-emotion artwork, movies or titles effects */
	/*  */

	puppet_Contains_All_Artwork = SAFE_FALSE;
	puppet_Redirected_File_List_Hdl = (Puppet_Redirected_File_Hdl)Get1Resource(k_PUPPET_REDIRECTED_FILES_RESOURCE_TYPE
																			, k_BASE_PUPPET_RESOURCE_ID + puppet_Offset);

	if ((ResError() != noErr) || (puppet_Redirected_File_List_Hdl == nil)) {
		/*  */
		/* If this resource is missing assume that the puppet is old style 2D and */
		/* is not the new type that uses Spin files */
		/*  */
		puppet_Contains_All_Artwork = SAFE_TRUE;
	}
	else {
		puppet_Redirected_File_List_Memory_In_Use = MEMORY_INITIALIZED;
		HNoPurge((Handle)puppet_Redirected_File_List_Hdl);
		if (MemError() != noErr) {
			/*  */
			/* This cannot fail, but check anyway */
			/*  */
			goto Error_Handler_Early_Stages;
		}
		if (((*puppet_Redirected_File_List_Hdl)->valid == SAFE_TRUE)
			 && ((*puppet_Redirected_File_List_Hdl)->puppet_File_Contains_No_Internal_Art == SAFE_TRUE)) {
			 
			/*  */
			/* Good, The puppet is the redirected type. Ignore internal art if any */
			/*  */
		}
		else {
			puppet_Contains_All_Artwork = SAFE_TRUE;
		}
		
	} /* NOT if ((ResError() != noErr) ... */		
	
	
	
	if (Begin_Use_Of_Handle((Handle)puppet_Header_Hdl
									, MOVE_HIGH_NOT_REQUIRED
									, &header_Original_State) != SAFE_TRUE) {
		goto Error_Handler_Early_Stages;
	}
	
	header = *puppet_Header_Hdl;
	
	Copy_Pascal_String(header->screen_Name, current_Puppet->screen_Name);
	Copy_Pascal_String(header->screen_Short_Name, current_Puppet->screen_Short_Name);
	Copy_Pascal_String(header->file_Name, current_Puppet->file_Name);

	if ((header->backward_Compatible_Version & 0xFFFF0000) > (k_PUPPET_VERSION_NOW & 0xFFFF0000)) {
		Copy_Pascal_String("\pThis file is too new to be used safely", message_String);
		Perform_OK_Dialog(message_String);
		goto Error_Handler_Early_Stages;
	}
	
	if ((header->backward_Compatible_Version & 0xFFFF0000) < (k_PUPPET_BASE_VERSION_NOW & 0xFFFF0000)) {
		Copy_Pascal_String("\pThis file is too old to be used safely", message_String);
		Perform_OK_Dialog(message_String);
		goto Error_Handler_Early_Stages;
	}

	current_Puppet->total_Special_Colors_To_Honor = header->total_Special_Colors_To_Honor;
	current_Puppet->total_Colors_Used_In_Pallette = header->total_Colors_Used_In_Pallette;
	
	offscreen_Rect = header->stored_Offscreen_Rect;
	desired_Bit_Depth = header->stored_Bit_Depth;
	
	if (header->special_Pallette_ID != 0) {
		current_Puppet->special_Pallete_To_Use = (PaletteHandle)GetNewPalette(header->special_Pallette_ID);
		if (current_Puppet->special_Pallete_To_Use == nil) {
			Copy_Pascal_String("\pThis file is corrupted (no pallette)", message_String);
			Perform_OK_Dialog(message_String);
			goto Error_Handler_Early_Stages;
		}
		
		HNoPurge((Handle)current_Puppet->special_Pallete_To_Use);
		os_Error = MemError();
		if (os_Error != noErr) {
			/*  */
			/* This cannot fail */
			/*  */
			goto Error_Handler_Early_Stages;
		}
		
		MoveHHi((Handle)current_Puppet->special_Pallete_To_Use);
		os_Error = MemError();
		if ((os_Error != noErr) && (os_Error != -117)){
			/*  */
			/* This cannot fail, but check anyway */
			/*  */
			goto Error_Handler;
		}
		
		HLock((Handle)current_Puppet->special_Pallete_To_Use);
		if (MemError() != noErr) {
			/*  */
			/* This cannot fail, but check anyway */
			/*  */
			goto Error_Handler;
		}
		
	}
	else {
		current_Puppet->special_Pallete_To_Use = nil;
	}
	
	
	/*  */
	/* Continue loading, but a big difference in the wayt to load is */
	/* whether this file actually contains art or is a container of refernces to */
	/* other art files (the new type of animation file that displays from disk */
	/*  */
	if (puppet_Contains_All_Artwork == SAFE_TRUE) {
	
		if (header->current_Clut_ID != 0) {
		
			temp_Clut_Handle = GetCTable(header->current_Clut_ID);
			if (temp_Clut_Handle == nil) {
					Display_Serious_Error_Message(unit_Name
																		, code_Name
																		, "\pProblem Loading System CLUT (0.1), Nonredirected Art requires it."
																		, "\pCorrupted?");
				goto Error_Handler;
			}	
		
			HLock((Handle)temp_Clut_Handle);
			if (MemError() != noErr) {
					Display_Serious_Error_Message(unit_Name
																		, code_Name
																		, "\pProblem locking handle (0.2)"
																		, "\pCorrupted!");
				goto Error_Handler;
			}
		
		
	#if (7 == 11)	
			temp_Clut_Handle = (CTabHandle)Get1Resource(
																					k_PUPPET_CLUT_RESOURCE_TYPE
																					, header->current_Clut_ID);
			if ((ResError() != noErr) || (temp_Clut_Handle == nil)) {
				Copy_Pascal_String("\pThis file is corrupted (no Color Table)", message_String);
				Perform_OK_Dialog(message_String);
				goto Error_Handler;
			}
			
			DetachResource((Handle)temp_Clut_Handle);
			if (ResError() != noErr) {
				/*  */
				/* This cannot fail, but check anyway */
				/*  */
				goto Error_Handler;
			}
			
			HNoPurge((Handle)temp_Clut_Handle);
			if (MemError() != noErr) {
				/*  */
				/* This cannot fail, but check anyway */
				/*  */
				goto Error_Handler;
			}
	#endif  /* (7 == 11)	*/
			
		}
		else {
			temp_Clut_Handle = nil;
		}

		current_Puppet->direct_Art.frame_Total = header->file_Frame_Total;
		
		current_Puppet->direct_Art.talk_Total_Frames = header->talk_Total_Frames;
		current_Puppet->direct_Art.talk_Start_Frame = header->talk_Start_Frame;
		current_Puppet->direct_Art.talk_Stop_Frame = header->talk_Stop_Frame;
	
		current_Puppet->puppet_File_Is_Old_Self_Contained_Type = SAFE_TRUE;
		
		current_Puppet->redirected_Information_Hdl = nil;
		current_Puppet->redirected_Information_Ptr = nil;
		current_Puppet->redirected_Information_Memory_In_Use = MEMORY_NOT_INITIALIZED;
		current_Puppet->redirected_Information_Max_Size = 0;
		current_Puppet->redirected_Information_Total_Used = 0;
		
		/*  */
		/* Important. This referes to selected emotion or effect or view. */
		/*  */
		current_Puppet->current_Selection = 0;
		
	
		if (Begin_Use_Of_Handle((Handle)puppet_Body_Hdl
										, MOVE_HIGH_NOT_REQUIRED
										, &body_Original_State) != SAFE_TRUE) {
			goto Error_Handler;
		}
			
		body = (Puppet_File_Ptr)*puppet_Body_Hdl;
		
		if (current_Puppet->direct_Art.frame_Total != body->total_Frame_Safety){ 
			goto Error_Handler;
		}
		
		if (current_Puppet->direct_Art.frame_Total > k_MAXIMUM_PICS_FRAMES){ 
			goto Error_Handler;
		}
		
		first_Rect_Ptr = &(body->file_Frame_Locations[0]);
		
		resource_ID_Count_2_Ptr = (CARDINAL_16 *)((CARDINAL_32)first_Rect_Ptr 
												+ ((CARDINAL_32)sizeof(Rect) 
													* (CARDINAL_32)body->resource_Template_Count_1));
		
		first_Resource_ID_Ptr = (CARDINAL_16 *)((CARDINAL_32)resource_ID_Count_2_Ptr
													+ sizeof(INTEGER_16));
		
		end_Sentinel_Ptr = (Safe_Boolean *)((CARDINAL_32)first_Resource_ID_Ptr 
												+ ((CARDINAL_32)sizeof(INTEGER_16) 
													* (CARDINAL_32)body->resource_Template_Count_1));
		
		if (*end_Sentinel_Ptr != SAFE_TRUE) {
			Copy_Pascal_String("\pThis file is corrupted, or program is designed wrong (bad end sentinel)", message_String);
			Perform_OK_Dialog(message_String);
			goto Error_Handler;
		}
		
		if ((current_Puppet->direct_Art.frame_Total > body->resource_Template_Count_1)
			|| (current_Puppet->direct_Art.frame_Total > *resource_ID_Count_2_Ptr)) {
			goto Error_Handler;
		}
		
		for (loop = 0; (loop < current_Puppet->direct_Art.frame_Total) ; loop++) {
		
			current_Puppet->direct_Art.frame_Locations[loop] = *(Rect *)((CARDINAL_32)first_Rect_Ptr 
																						+ ((CARDINAL_32)sizeof(Rect) * (CARDINAL_32)loop));
			/*  */
			/* Abuse the structure for just a moment */
			/*  */
			current_Puppet->direct_Art.frame_Regions[loop] = (RgnHandle)(*(CARDINAL_16 *)((CARDINAL_32)first_Resource_ID_Ptr 
																				+ ((CARDINAL_32)sizeof(Resource_Identifier) 
																					* (CARDINAL_32)loop)));
		}
		
		/*  */
		/* Abuse the structure for just a moment */
		/*  */
		current_Puppet->direct_Art.total_Lazy_Active_Region = (RgnHandle)body->file_Total_Lazy_Active_Region;
		
		/*  */
		/* Now load the frame regions */
		/*  */
		for (loop = 0; (loop < current_Puppet->direct_Art.frame_Total) ; loop++) {
		
			if (current_Puppet->direct_Art.frame_Regions[loop] != 0) {
				current_Puppet->direct_Art.frame_Regions[loop] = (RgnHandle)Get1Resource(
																						k_PUPPET_APPLE_REGION_RESOURCE_TYPE
																						, (INTEGER_16)current_Puppet->direct_Art.frame_Regions[loop]);
				if ((ResError() != noErr) || (current_Puppet->direct_Art.frame_Regions[loop] == nil)) {
					Copy_Pascal_String("\pThis file is corrupted (missing region)", message_String);
					Perform_OK_Dialog(message_String);
					goto Error_Handler;
				}
				
				DetachResource((Handle)current_Puppet->direct_Art.frame_Regions[loop] );
				if (ResError() != noErr) {
					/*  */
					/* This cannot fail, but check anyway */
					/*  */
					goto Error_Handler;
				}
				
				HNoPurge((Handle)current_Puppet->direct_Art.frame_Regions[loop] );
				if (MemError() != noErr) {
					/*  */
					/* This cannot fail, but check anyway */
					/*  */
					goto Error_Handler;
				}
				
			}
		}
		
		if (current_Puppet->direct_Art.total_Lazy_Active_Region != 0) {
			current_Puppet->direct_Art.total_Lazy_Active_Region = (RgnHandle)Get1Resource(
																		k_PUPPET_APPLE_REGION_RESOURCE_TYPE
																		, (INTEGER_16)current_Puppet->direct_Art.total_Lazy_Active_Region);
			if ((ResError() != noErr) || (current_Puppet->direct_Art.total_Lazy_Active_Region == nil)) {
				Copy_Pascal_String("\pThis file is corrupted (missing region 2)", message_String);
				Perform_OK_Dialog(message_String);
				goto Error_Handler;
			}
			
			DetachResource((Handle)current_Puppet->direct_Art.total_Lazy_Active_Region);
			if (ResError() != noErr) {
				/*  */
				/* This cannot fail, but check anyway */
				/*  */
				goto Error_Handler;
			}
			
			HLock((Handle)current_Puppet->direct_Art.total_Lazy_Active_Region);
			if (MemError() != noErr) {
				/*  */
				/* This cannot fail, but check anyway */
				/*  */
				goto Error_Handler;
			}
			
			/*  */
			/* This is top help fix an old bug I created in older versions */
			/* when importing solid backgrounds */
			/*  */
			if (Repair_Region_Defect(current_Puppet->direct_Art.total_Lazy_Active_Region, current_Puppet) != SAFE_TRUE) {
				/*  */
				/* Ignore errors in this version. Not designed to fail. */
				/*  */
			}
		
		}
	
		/*  */
		/* Make a copy of "total_Lazy_Active_Region" so that it */
		/* can be modiifed larger for jiggle effect */
		/*  */
		if (current_Puppet->direct_Art.total_Lazy_Active_Region != nil) {
			current_Puppet->direct_Art.original_Total_Lazy_Active_Region = NewRgn();
			CopyRgn(current_Puppet->direct_Art.total_Lazy_Active_Region, current_Puppet->direct_Art.original_Total_Lazy_Active_Region);
		}
		else {
			current_Puppet->direct_Art.original_Total_Lazy_Active_Region = nil;
		}
	
		/*  */
		/* Modify it larger by four pixels for jiggle and motion effect */
		/*  */
		InsetRgn(current_Puppet->direct_Art.total_Lazy_Active_Region, -(dream.horizontal_Jiggle_Maximum), -(dream.vertical_Jiggle_Maximum));
	
	
		/*  */
		/* Initialize buffered copies of specialized resources */
		/*  */
		for (loop = 0; (loop < current_Puppet->direct_Art.frame_Total) ; loop++) {
			if (current_Puppet->direct_Art.frame_Regions[loop] != nil) {
				current_Puppet->direct_Art.frame_Buffered_Animation_Regions[loop] = NewRgn();
				CopyRgn(current_Puppet->direct_Art.frame_Regions[loop], current_Puppet->direct_Art.frame_Buffered_Animation_Regions[loop]);
			}
			else {
				current_Puppet->direct_Art.frame_Buffered_Animation_Regions[loop] = nil;
			}
			current_Puppet->direct_Art.frame_Buffered_Animation_Region_Offset[loop].h = 0;
			current_Puppet->direct_Art.frame_Buffered_Animation_Region_Offset[loop].v = 0;
		}
		
		if (current_Puppet->direct_Art.total_Lazy_Active_Region != nil) {
			current_Puppet->direct_Art.buffered_Animation_Total_Region = NewRgn();
			CopyRgn(current_Puppet->direct_Art.total_Lazy_Active_Region, current_Puppet->direct_Art.buffered_Animation_Total_Region);
		}
		else {
			current_Puppet->direct_Art.buffered_Animation_Total_Region = nil;
		}
		current_Puppet->direct_Art.buffered_Animation_Total_Region_Offset.h = 0;
		current_Puppet->direct_Art.buffered_Animation_Total_Region_Offset.v = 0;
	
		
		
		if (Determine_Absolute_Free_Memory_Hook(&total_Bytes_Contiguous_Now
														,&total_Bytes_Now
														,&total_Bytes_Contiguous_Later
														,&total_Bytes_Later) != TRUE) {
			goto Error_Handler;
		}
		
		memory_Needed = desired_Bit_Depth;
		memory_Needed = memory_Needed * (CARDINAL_32)(offscreen_Rect.right - offscreen_Rect.left);
		memory_Needed = memory_Needed * (CARDINAL_32)(offscreen_Rect.bottom - offscreen_Rect.top);
		memory_Needed = memory_Needed >> 3;
		
		if (total_Bytes_Contiguous_Later < (k_MINIMUM_RAM_TO_PRESERVE_FOR_BIG_STUFF + memory_Needed)) {
			total_Bytes_Contiguous_Later = MaxMem(&total_Bytes_Now);
			if (total_Bytes_Contiguous_Later < (k_MINIMUM_RAM_TO_PRESERVE_FOR_BIG_STUFF + memory_Needed)) {
				total_Bytes_Contiguous_Later = MaxMem(&total_Bytes_Now);
		
				Get_NS_String(k_DREAM_ERROR_STRINGS, k_ERROR_EXPORTING_PUPPETS_USES_MEMORY, note_String);
				Make_String_Long_String(&message_String[0], &note_String[0], memory_Needed, NO_LIMIT, "\p bytes.");
				Perform_OK_Dialog(message_String);
				function_Result = SAFE_FALSE;
				goto Normal_Function_Exit;
			}
		}
	
	
		/*  */
		/* Now create the offscreen picture!  */
		/*  */
			
		offscreen_Ptr = &(current_Puppet->direct_Art.port);
		
		offscreen_Ptr->color_Table_Handle = temp_Clut_Handle;
		temp_Clut_Handle = nil;
		
	
		offscreen_Ptr->offscreen_GDevice_Hdl = nil;
		offscreen_Ptr->image_Handle = nil;
		/*  */
		/* Paranoia debugging only */
		/*  */
		MoveHHi((Handle)offscreen_Ptr->color_Table_Handle);
		os_Error = MemError();
		if ((os_Error != noErr) && (os_Error != -117)){
		
		}
		
		HLock((Handle)offscreen_Ptr->color_Table_Handle);
		offscreen_Ptr->color_Type = k_COLOR_QUICKDRAW;
	
		os_Error= Create_Off_Screen_Port(&offscreen_Rect
																, desired_Bit_Depth
																, offscreen_Ptr->color_Table_Handle
																, (CGrafPtr)&(offscreen_Ptr->offscreen_Port.color_Port)
																, &(offscreen_Ptr->offscreen_GDevice_Hdl)
																, &(offscreen_Ptr->image_Handle));
		
		if (os_Error != noErr) {
			Copy_Pascal_String("\pA graphics port for this puppet file failed to be created (low memory?)", message_String);
			Perform_OK_Dialog(message_String);
			goto Error_Handler;
		}
		
		/*  */
		/*  */
		/*  */
		/*  */
		/*  */
		/* 1993.08.01 JAM ADDED TO FIX BUG? */
		/*  */
		
		offscreen_Ptr->initialized =  SAFE_TRUE; 
		offscreen_Ptr->color_Type = k_COLOR_QUICKDRAW;
		
		Lock_Video_Memory_For_Lazy_But_Safe_Use(offscreen_Ptr);
			
		/*  */
		/*  */
		/*  */
	
	} /* if (puppet_Contains_All_Artwork == SAFE_TRUE) */
	else {
	
		current_Puppet->puppet_File_Is_Old_Self_Contained_Type = SAFE_FALSE;
		
		/*  */
		/* HACK HACK HACK initialize all fields to a safe value for now. */
		/*  */
		temp_Clut_Handle = nil;

		current_Puppet->direct_Art.port.initialized =  SAFE_FALSE; 
		current_Puppet->direct_Art.port.offscreen_GDevice_Hdl = nil;
		current_Puppet->direct_Art.port.image_Handle = nil;

		current_Puppet->direct_Art.frame_Total = 0;
		current_Puppet->direct_Art.talk_Total_Frames = 0;
		current_Puppet->direct_Art.talk_Start_Frame = 0;
		current_Puppet->direct_Art.talk_Stop_Frame = 0;
		current_Puppet->direct_Art.total_Lazy_Active_Region = nil;
		current_Puppet->direct_Art.original_Total_Lazy_Active_Region = nil;
		current_Puppet->direct_Art.buffered_Animation_Total_Region_Offset.h = 0;
		current_Puppet->direct_Art.buffered_Animation_Total_Region_Offset.v = 0;
		
		current_Puppet->direct_Art.frame_Locations[0].left = 0;
		current_Puppet->direct_Art.frame_Locations[0].right = 1;
		current_Puppet->direct_Art.frame_Locations[0].top = 0;
		current_Puppet->direct_Art.frame_Locations[0].bottom = 1;
		current_Puppet->direct_Art.frame_Regions[0] = nil;
		current_Puppet->direct_Art.total_Lazy_Active_Region = nil;
		current_Puppet->direct_Art.frame_Buffered_Animation_Regions[0] = nil;
		
		/*  */
		/* We should now move over information that we do need so that the files */
		/* for animation are openned and ready. (The puppet would not be openned */
		/* unless there was a reason to do so, so preflight it all and get headers loaded). */
		/* Note though that files are actually OPEN when puppet is loaded, and therefore */
		/* multiple read-only permission opens are active simultaneously perhaps. */
		/* But all open files are governed by data base. */
		/* Analyze the "puppet_Redirected_File_List_Hdl" information */
		/*  */
		
		if (current_Puppet->redirected_Information_Memory_In_Use == MEMORY_INITIALIZED) {
			goto Error_Handler;
		}
		
		if (Begin_Use_Of_Handle((Handle)puppet_Redirected_File_List_Hdl
										, MOVE_HIGH_NOT_REQUIRED
										, &redirected_Info_Original_State) != SAFE_TRUE) {
			goto Error_Handler;
		}
			
		redirected_Info = *puppet_Redirected_File_List_Hdl;

		
		current_Puppet->redirected_Information_Hdl = nil;
		current_Puppet->redirected_Information_Ptr = nil;
		current_Puppet->redirected_Information_Memory_In_Use = MEMORY_NOT_INITIALIZED;
		current_Puppet->redirected_Information_Max_Size = 0;
		current_Puppet->redirected_Information_Total_Used = 0;
		
		/*  */
		/* Important. This referes to selected emotion or effect or view. */
		/*  */
		current_Puppet->current_Selection = 0;

		if (redirected_Info->valid != SAFE_TRUE) {
			goto Error_Handler;
		}
		
		if (redirected_Info->puppet_File_Contains_No_Internal_Art != SAFE_TRUE) {
			goto Error_Handler;
		}
		
		if ((redirected_Info->redirection_Version & 0xFFFF0000) > (k_PUPPET_VERSION_NOW & 0xFFFF0000)) {
			Copy_Pascal_String("\pThis file is too new to be used safely. (Specifically, the redirection Pup3 resource)", message_String);
			Perform_OK_Dialog(message_String);
			goto Error_Handler;
		}
		
		if ((redirected_Info->redirection_Version & 0xFFFF0000) < (k_PUPPET_BASE_VERSION_NOW & 0xFFFF0000)) {
			Copy_Pascal_String("\pThis file is too old to be used safely. (Specifically, the redirection Pup3 resource)", message_String);
			Perform_OK_Dialog(message_String);
			goto Error_Handler;
		}
		
		current_Puppet->redirected_Information_Total_Used = redirected_Info->resource_Template_Count_1;
		
		file_Info_Ptr = &(redirected_Info->arbitrary_List_Of_File_Information[redirected_Info->resource_Template_Count_1]);
		if (*(Safe_Boolean *)file_Info_Ptr != SAFE_TRUE) {
			Copy_Pascal_String("\pThis file has a corrupted, undersized, resource. (Specifically, the redirection Pup3 resource)", message_String);
			Perform_OK_Dialog(message_String);
			goto Error_Handler;
		}
		
		if (current_Puppet->redirected_Information_Total_Used > 0) {
			/*  */
			/* First, create the memory for the puppet to hold the file redirection */
			/* information. */
			/*  */
					
			current_Puppet->redirected_Information_Max_Size = current_Puppet->redirected_Information_Total_Used;
			/*  */
			/* Add an extra one for convenience sake in case needed for transient effect. */
			/*  */
			current_Puppet->redirected_Information_Max_Size += 1;
			
			preferred_Size = ((current_Puppet->redirected_Information_Max_Size) * sizeof(Redirected_Art_Rec));
			current_Puppet->redirected_Information_Hdl = nil;
			current_Puppet->redirected_Information_Memory_In_Use = MEMORY_NOT_INITIALIZED;
	
			if (Create_Handle(&preferred_Size
								, (CARDINAL_32)(10240L)
								, ALLOW_PURGING
								, DISALLOW_PARTIAL_AMOUNT
								, (Handle *)&current_Puppet->redirected_Information_Hdl
								, &current_Puppet->redirected_Information_Memory_In_Use) != SAFE_TRUE) {
								
				preferred_Size = ((current_Puppet->redirected_Information_Max_Size) * sizeof(Redirected_Art_Rec));

				Display_Low_Memory_Error_Message(unit_Name, code_Name, preferred_Size);
				function_Result = SAFE_FALSE;
				goto Error_Handler;
			}
	
			/*  */
			/* Move High and lock it. Its not so bad. Puppets used to have to load */
			/* entire mix buffer ports in the past. */
			/*  */
			if (Begin_Use_Of_Handle((Handle)current_Puppet->redirected_Information_Hdl
											, MOVE_HIGH_REQUIRED
											, &redirected_Info_Original_State) != SAFE_TRUE) {
				goto Error_Handler;
			}
			current_Puppet->redirected_Information_Ptr = (Redirected_Art_Ptr)*current_Puppet->redirected_Information_Hdl;
		
			for (loop = 0; (loop < current_Puppet->redirected_Information_Total_Used) ; loop++) {
				file_Info_Ptr = &(redirected_Info->arbitrary_List_Of_File_Information[loop]);
				loaded_Info_Ptr = &(current_Puppet->redirected_Information_Ptr[loop]);
				
				/*  */
				/* First, copy over file information for future "saves". */
				/*  */
				
				Copy_Pascal_String(&file_Info_Ptr->file_Name[0], &loaded_Info_Ptr->file_Name[0]);
				loaded_Info_Ptr->file_Creation_Time = file_Info_Ptr->file_Creation_Time;
				loaded_Info_Ptr->emotion_Key_Code = file_Info_Ptr->emotion_Key_Code;
				loaded_Info_Ptr->cache_Technique_Desired = file_Info_Ptr->cache_Technique_Desired;
				loaded_Info_Ptr->special_Effect_Classification = file_Info_Ptr->special_Effect_Classification;
				loaded_Info_Ptr->special_Effect_Value = file_Info_Ptr->special_Effect_Value;
				loaded_Info_Ptr->special_Effect_Extra_Value = file_Info_Ptr->special_Effect_Extra_Value;
				loaded_Info_Ptr->offset_Point_Position = file_Info_Ptr->offset_Point_Position;

				/*  */
				/* Fill in information for loaded states here, including the sentinel flag. */
				/*  */
				loaded_Info_Ptr->valid = SAFE_TRUE;
								
				loaded_Info_Ptr->array_Index_Valid = SAFE_FALSE;
				loaded_Info_Ptr->array_Index = 0;
			}

		} /* 	if (current_Puppet->redirected_Information_Total_Used > 0) */
		
		/*  */
		/* Carefully set the end sentinel to ensure that resource is secure. */
		/*  */
	
	} /* NOT if (puppet_Contains_All_Artwork == SAFE_TRUE) */
	
	/*  */
	/* Close resource fork and open the data fork */
	/*  */
	
	if (puppet_Header_Memory_In_Use == MEMORY_INITIALIZED) {
		ReleaseResource((Handle)puppet_Header_Hdl);
		if (ResError() != noErr) {
			/*  */
			/* Ignore error, we are exiting anyway  */
			/*  */
		}
		puppet_Header_Memory_In_Use = MEMORY_NOT_INITIALIZED;
	}
	
	if (puppet_Body_Memory_In_Use == MEMORY_INITIALIZED) {
		ReleaseResource((Handle)puppet_Body_Hdl);
		if (ResError() != noErr) {
			/*  */
			/* Ignore error, we are exiting anyway  */
			/*  */
		}
		puppet_Body_Memory_In_Use = MEMORY_NOT_INITIALIZED;
	}

	if (puppet_Redirected_File_List_Memory_In_Use == MEMORY_INITIALIZED) {
		ReleaseResource((Handle)puppet_Redirected_File_List_Hdl);
		if (ResError() != noErr) {
			/*  */
			/* Ignore error, we are exiting anyway  */
			/*  */
		}
		puppet_Redirected_File_List_Memory_In_Use = MEMORY_NOT_INITIALIZED;
	}
	
	if (resource_Sources_Swapped == SAFE_TRUE) {
		CloseResFile(input_File_Reference);
		if (ResError() != noErr) {
			goto Error_Handler_Video_Ready;
		}
		
		UseResFile(save_Ref);
		if (ResError() != noErr) {
			/*  */
			/* This should be recoverable from, enhance one day  */
			/*  */
		}
		resource_Sources_Swapped = SAFE_FALSE;
	}
	

	if (puppet_Contains_All_Artwork == SAFE_TRUE) {
	
		/*  */
		/* Now open data fork and get our large bit map structure. */
		/* The data fork is used because we might want to read */
		/* portions in during interrupts, as a background task, or */
		/* because we need to load it into memory in a specific */
		/* spot and there may not be enough room for two copies */
		/* to exchange from a loaded monster resource. It is true */
		/* that system 7 allows partial resource loading, but some */
		/* resource compressors, and other things fail when these are used. */
		/*  */
	
		os_Error = HOpen(file_Specification->vRefNum
										,file_Specification->parID
										,file_Specification->name
										,fsRdPerm
										,&input_File_Reference);
		if (os_Error != noErr) {
			Concat_Many(note_String, 3,"\pHOpen for file ", file_Specification->name, "\p ID = ");
			Concat_Long_To_String(note_String, os_Error, NO_LIMIT);
			goto Error_Handler_With_Message;
		} 
		
		
		current_Pict_Sequence = 0;
		still_Searching = TRUE;
		do {
			transfer_Count = sizeof(transfer_Count_Copy);
			os_Error = FSRead(input_File_Reference, &transfer_Count, (Ptr)&transfer_Count_Copy);
			if (os_Error != noErr) {
				Concat_Many(note_String, 3,"\pFSRead for file ", file_Specification->name, "\p ID = ");
				Concat_Long_To_String(note_String, os_Error, NO_LIMIT);
				goto Error_Handler_With_Message;
			}
		
			/*  */
			/* Skip this puppet and go to the next one */
			/*  */
			
			if (current_Pict_Sequence == puppet_Offset) {
				still_Searching = FALSE;
			}
			else {
				os_Error = SetFPos(input_File_Reference,fsFromMark,transfer_Count_Copy);
				if (os_Error != noErr) {
					Concat_Many(note_String, 3,"\pSetFPos for file ", file_Specification->name, "\p ID = ");
					Concat_Long_To_String(note_String, os_Error, NO_LIMIT);
					goto Error_Handler_With_Message;
				}
				
				current_Pict_Sequence += 1;
			}
			
		} while (still_Searching == TRUE);
		
	
		transfer_Count = transfer_Count_Copy;
		
		handle_Size = GetHandleSize(offscreen_Ptr->image_Handle);
		if (MemError() != noErr) {
			Display_Serious_Error_Message(unit_Name, code_Name, "\pGetHandleSize (1)", "\p");
		}
		else {
		
			if (transfer_Count > handle_Size) {
				transfer_Count = handle_Size;
			}
			
			os_Error = FSRead(input_File_Reference, &transfer_Count, (Ptr)*(offscreen_Ptr->image_Handle));
			if (os_Error != noErr) {
				Concat_Many(note_String, 3,"\pFSWrite for file ", file_Specification->name, "\p ID = ");
				Concat_Long_To_String(note_String, os_Error, NO_LIMIT);
				goto Error_Handler_With_Message;
			}
		
			os_Error = FSClose(input_File_Reference);
			if ( os_Error != noErr) {
				Display_Serious_Error_Message(unit_Name, code_Name, "\pOS Error FSClose.", "\p");
			}
			
			
			if (Enforce_Selected_Colors_If_Visible(index_Used) != SAFE_TRUE) {
				goto Error_Handler_Video_Ready;
			}
		}
		
	} /* if (puppet_Contains_All_Artwork == SAFE_TRUE) */
	else {
		
		for (loop = 0; (loop < current_Puppet->redirected_Information_Total_Used) ; loop++) {
			loaded_Info_Ptr = &(current_Puppet->redirected_Information_Ptr[loop]);
			
			
		/* ====
			loaded_Info_Ptr->file_Name
			loaded_Info_Ptr->file_Creation_Time
			
			loaded_Info_Ptr->emotion_Key_Code
			loaded_Info_Ptr->cache_Technique_Desired
			loaded_Info_Ptr->special_Effect_Classification
			loaded_Info_Ptr->special_Effect_Value
			loaded_Info_Ptr->special_Effect_Extra_Value
			loaded_Info_Ptr->offset_Point_Position
		=== */
		
			/*  */
			/* Warning : file_Creation_Time is not used for safety. Any */
			/* first matched file is accepted. */
			/*  */
			
			if (Locate_Item_Location(k_SPIN_OBJECT_SELECTION
														, (Str63 *)&(loaded_Info_Ptr->file_Name[0])
														, &params_For_Load_Spin_Object.desired_File
														, &item_Address
														, &address_Instead_Of_File) != SAFE_TRUE) {
				/*  */
				/* Goddamn file not available. We need to skip item and bitch */
				/* Preflight should have been done elsewhere. */
				/*  */
				goto Misssing_Item_Error_Handler;
			}
			
			if (address_Instead_Of_File == SAFE_TRUE) {
			
				if (dream.spin_Objects.spin_Object_Memory_In_Use[item_Address] != MEMORY_INITIALIZED) {
					goto Error_Handler;
				}
				
				loaded_Info_Ptr->array_Index_Valid = SAFE_TRUE;
				loaded_Info_Ptr->array_Index = item_Address;

			}
			else {
				/*  */
				/* Load spin file puppet, initialize it somewhat and lock it down */
				/* Then get index to structure in preparation and store it. */
				/*  */
												
				params_For_Load_Spin_Object.async_Task_Desired = SAFE_FALSE;
				params_For_Load_Spin_Object.load_File_With_Self_Contained_Large_Ports = SAFE_FALSE;
	
				if (Load_Spin_Object_File_Immediate_Style(&params_For_Load_Spin_Object) != SAFE_TRUE) {
					goto Error_Handler_Puppet_Files_Not_Found;
				}
				else {
					loaded_Info_Ptr->array_Index_Valid = SAFE_TRUE;
					loaded_Info_Ptr->array_Index = params_For_Load_Spin_Object.desired_Spin_Object;
				}

			}
			
			/*  */
			/* Note that scenes that use this puppet will have */
			/* "scene_Must_Use_Compositing_Buffers" set for them when adding to item list */
			/*  */

		}	
	}
	
		
	/*  */
	/* Something was loaded, enable all items */
	/*  */
	if (Reset_Menu_Control_Selectability() != SAFE_TRUE) {
		goto Error_Handler_Video_Ready;
	}
		
	function_Result = SAFE_TRUE;
	goto Normal_Function_Exit;
	
Error_Handler_With_Message:
	function_Result = SAFE_FALSE;
	Display_Serious_Error_Message(unit_Name, code_Name, note_String, "\p");
	goto Normal_Function_Exit;
	
Misssing_Item_Error_Handler:
	function_Result = SAFE_FALSE;
	Display_Serious_Error_Message(unit_Name, code_Name, "\pSpecified Redirected Puppet in Puppet definition not loaded. File missing below program load point", "\pSkipped. Future version allows replace.");

	/*  */
	/* Ignore Item. Increment to next item. Ignore if overrun */
	/*  */
	goto Normal_Function_Exit;

Error_Handler_Video_Ready:
	function_Result = SAFE_FALSE;
	Perform_OK_Dialog("\pThough puppet memory mostly is available, something prevented its loading. Try closing scenes.");
	goto Normal_Function_Exit;
	
Error_Handler_Puppet_Files_Not_Found:
	function_Result = SAFE_FALSE;
	Display_Serious_Error_Message(unit_Name, code_Name, "\pInternal problem getting Puppet loaded info, probably cannot locate the file", "\pInternal file names and creation dates are vital");
	goto Normal_Function_Exit;
	
Error_Handler_Early_Stages:
	function_Result = SAFE_FALSE;
	Display_Serious_Error_Message(unit_Name, code_Name, "\pInternal problem getting Puppet loaded info, early stages", "\pMemory?");
	goto Normal_Function_Exit;
	
Error_Handler:
	Display_Serious_Error_Message(unit_Name, code_Name, "\pInternal problem getting Puppet loaded info, past midpoint", "\pMemory?");
	function_Result = SAFE_FALSE;
		
Normal_Function_Exit:

	if (puppet_Header_Memory_In_Use == MEMORY_INITIALIZED) {
		ReleaseResource((Handle)puppet_Header_Hdl);
		if (ResError() != noErr) {
			/*  */
			/* Ignore error, we are exiting anyway  */
			/*  */
		}
		puppet_Header_Memory_In_Use = MEMORY_NOT_INITIALIZED;
	}
	
	if (puppet_Body_Memory_In_Use == MEMORY_INITIALIZED) {
		ReleaseResource((Handle)puppet_Body_Hdl);
		if (ResError() != noErr) {
			/*  */
			/* Ignore error, we are exiting anyway  */
			/*  */
		}
		puppet_Body_Memory_In_Use = MEMORY_NOT_INITIALIZED;
	}
	
	if (resource_Sources_Swapped == SAFE_TRUE) {
		CloseResFile(input_File_Reference);
		if (ResError() != noErr) {
			goto Error_Handler;
		}
		
		UseResFile(save_Ref);
		if (ResError() != noErr) {
			/*  */
			/* This should be recoverable from, enhance one day  */
			/*  */
		}
		resource_Sources_Swapped = SAFE_FALSE;
	}
	
	return(function_Result);
	
} /* Safe_Boolean	Load_Puppet_Sequence... */
/*=**************************************************************************=*/


/*=**************************************************************************=*/
Safe_Boolean	Enforce_Selected_Colors_If_Visible(CARDINAL_32	desired_Puppet)
{
	Safe_Boolean		function_Result;
	SignedByte			status_Globals_State;
	WindowPtr			current_Graphics_Window_Ptr;
	CARDINAL_8		*code_Name;
	Puppet_Ptr			current_Puppet;
/* END VAR */


	function_Result = SAFE_FALSE;
	code_Name = "\pSelect_Scene_And_Display_First_Time";
	
	if (active_Window_List[g_Shell.interface.current_Graphics_Window] == nil) {
		function_Result = SAFE_TRUE;
		goto Normal_Function_Exit;
	}

	/*  */
	/*  */
	/*  */
	/*  */
	/*  */
	/*  */
	/*  */
	/*  */
	/*  */
	/*  */
	/* HACK */
	/* For consistancy.... shunt off logic all the time */
	/*  */
	function_Result = SAFE_TRUE;
	goto Normal_Function_Exit;

	
	/*  */
	/* current_Puppet uses to a handle assumed to always be locked */
	/*  */
	current_Puppet = *(dream.puppet[desired_Puppet]);
	if (current_Puppet->puppet_Initialized != SAFE_TRUE) {
		goto Error_Handler;
	}

	/*  */
	/* This is dependant on the validity of "current_Window_Index" */
	/*  */
	if ((active_Window_List[g_Shell.interface.current_Graphics_Window] == nil)
		|| ((**active_Window_List[g_Shell.interface.current_Graphics_Window]).structure_Initialized != SAFE_TRUE)) {
		SysBeep(20); 
		Assign_String(ns_Shell_Globals.error_Description_String, "\pError. Improperly initialized window structure");
		Assign_String(ns_Shell_Globals.error_Note_String, "\pProgrammer flaw");
		Display_Error_Message(unit_Name, code_Name);
		Setup_Error_Handling();
		goto Error_Handler;
	}
	else {
		current_Graphics_Window_Ptr	= (**active_Window_List[g_Shell.interface.current_Graphics_Window]).status_Window;
	}
	
	/*  */
	/* If there are colors to be aware of,  force them into the */
	/* current palette now, the non-special colors */
	/* were already forced to be more tolerant. */
	/*  */
	
	/*  */
	/*  */
	/*  */
	/*  */
	/*  */
	/*  */
	/*  */
	/* HACK avoid pallette for now? No, use it. */
	/* We MUST add black and white before using this! */
	/*  */
	Set_View_Port(current_Graphics_Window_Ptr);
	
	if (g_Force_All_To_Use_Main_Puppet_Colors == SAFE_TRUE) {
	
		if (Force_Pallete_And_Clut_To_Match_Puppet(
																		current_Graphics_Window_Ptr
																		,current_Puppet) != SAFE_TRUE) {
		
		
		}
	
	}
	else {
		if (current_Puppet->total_Special_Colors_To_Honor > 0) {
			if ((**(current_Puppet->special_Pallete_To_Use)).pmEntries > 0) {	
				if (g_Permit_Palette_Colors == SAFE_TRUE) {
		
					NSetPalette(current_Graphics_Window_Ptr
										, current_Puppet->special_Pallete_To_Use
										, pmFgUpdates);
										
					ActivatePalette(current_Graphics_Window_Ptr);
				}
			}
		}
	}
	
	function_Result = SAFE_TRUE;
	goto Normal_Function_Exit;

Error_Handler:
	function_Result = SAFE_FALSE;
		
Normal_Function_Exit:
	return(function_Result);

} /* Safe_Boolean	Enforce_Selected_Colors_If_Visible ... */
/*=**************************************************************************=*/


/*=**************************************************************************=*/
Safe_Boolean	Save_Puppet_Sequence(CARDINAL_32 index_Used
															, Safe_Boolean *user_Abort)
{
	Str63					destination_File_Name;
	INTEGER_16		destination_Volume;
	INTEGER_16		scrap_Volume_Reference_Number;
	INTEGER_32		destination_Directory_ID;
	INTEGER_32		directory_Reference;
	INTEGER_16		file_Reference;
	Handle				temp_Clut_Handle;
	Boolean				user_Clicked_Ok;
	OSErr					os_Error;
	INTEGER_32		volume_Space;
	CARDINAL_32	file_Creator;
	CARDINAL_32	file_Type;
	CARDINAL_32	resource_Type;
	CARDINAL_32	file_Minimum_Size;
	INTEGER_32		transfer_Count;
	INTEGER_32		transfer_Count_Copy;
	INTEGER_16		resource_ID;
	Puppet_Ptr			current_Puppet;
	Safe_Boolean		function_Result;
	CARDINAL_32	preferred_Size;
	
	Puppet_File_Header_Ptr			header;
	Puppet_File_Header_Hdl			resource_To_Create_Hdl;
	Memory_Allocation					resource_To_Create_Hdl_Memory_Initialized;
	Puppet_File_Ptr							body;
	Puppet_File_Hdl						expandable_Resource_Hdl;
	Memory_Allocation					expandable_Resource_Hdl_Memory_Initialized;

	Puppet_Redirected_File_Hdl	puppet_Redirected_File_List_Hdl;
	Puppet_Redirected_File_Ptr		redirected_Info;
	SignedByte 								redirected_Info_Original_State;
	Memory_Allocation					puppet_Redirected_File_List_Memory_In_Use;
	Redirected_File_Ptr					file_Info_Ptr;
	Redirected_Art_Ptr					loaded_Info_Ptr;

	Handle				template_Hdl;
	SFReply				sf_Reply;
	SIMPLE_LOOP	loop;
	CARDINAL_8		*code_Name;
	Str255					resource_Comment;
	Str255					message_String;
	Str255					note_String;
	Str255					volume_Name;
	Str255					work_String_2;
	Str63					suffix_String;
	Str63					original_Name;
/* END VAR */


	code_Name = "\pSave_Puppet_Sequence";
	function_Result = SAFE_FALSE;
	*user_Abort = SAFE_FALSE;
		
	resource_To_Create_Hdl_Memory_Initialized = MEMORY_NOT_INITIALIZED;
	expandable_Resource_Hdl_Memory_Initialized = MEMORY_NOT_INITIALIZED;
	puppet_Redirected_File_List_Memory_In_Use = MEMORY_NOT_INITIALIZED;

	/*  */
	/* current_Puppet uses to a handle assumed to always be locked */
	/*  */
	current_Puppet = *(dream.puppet[index_Used]);
	
	/*  */
	/* Should ask these also */
	/*  */
	Get_NS_String(k_DREAM_STRINGS, k_ENTER_PUPPET_NAMES_PROMPT, message_String);

	if (Edit_Fancy_Filename_Information(message_String
								, &(current_Puppet->file_Name[0])
								, &(current_Puppet->screen_Short_Name[0])
								, &(current_Puppet->screen_Name[0])) != TRUE) {
		*user_Abort = SAFE_TRUE;
		function_Result = SAFE_FALSE;
		goto Normal_Function_Exit;
	}
		
	Get_NS_String(k_DREAM_STRINGS, k_SAVE_PUPPET_PROMPT, message_String);
	Get_NS_String(k_DREAM_STRINGS, k_SAVE_PUPPET_FILE_SUFFIX, suffix_String);

	
	file_Creator= FILE_CREATOR;
	file_Type = k_PUPPET_FILE_TYPE;

	Assign_String(original_Name, current_Puppet->file_Name);
	Strip_Off_Popular_Suffixes(&original_Name);

	/*  */
	/* Suffix not appended if just space character */
	/*  */
	if ((suffix_String[0] != 1) || (suffix_String[1] != ASCII_SPACE)) {
		Concat_String(original_Name, suffix_String);
	}
	
	if(original_Name[0] >31) {
		original_Name[0] = 31;
	}

	user_Clicked_Ok = Perform_Put_File_Dialog(message_String
										, original_Name
										, &sf_Reply);
	
	if ((user_Clicked_Ok != TRUE) || (sf_Reply.good != TRUE)) {
		/*  */
		/* User Abort, no file name */
		/*  */
		*user_Abort = SAFE_TRUE;
		function_Result = SAFE_FALSE;
		goto Normal_Function_Exit;
	}
	
	directory_Reference = sf_Reply.vRefNum;
	Copy_Pascal_String(sf_Reply.fName, destination_File_Name);
	Copy_Pascal_String(destination_File_Name, current_Puppet->file_Name);

	Get_Last_Volume_Reference_And_Directory_ID(&(destination_Volume), &(destination_Directory_ID));


	note_String[0] = 0;
	
	os_Error = GetVInfo((Get_Drive_Number(destination_Volume)), volume_Name, &scrap_Volume_Reference_Number, (INTEGER_32 *)&volume_Space);
	if (os_Error != noErr) {
		Concat_Many(note_String, 3, "\pGetVInfo for volume ", volume_Name, "\p ID = ");
		Concat_Long_To_String(note_String, os_Error, NO_LIMIT);
		goto Error_Handler_With_Message;
	} /* if (os_Error != noErr) */
		

	preferred_Size = sizeof(Puppet_File_Header_Rec);
	resource_To_Create_Hdl = nil;

	if (Create_Handle(&preferred_Size
						, (CARDINAL_32)(10240L)
						, ALLOW_PURGING
						, DISALLOW_PARTIAL_AMOUNT
						, (Handle *)&resource_To_Create_Hdl
						, &resource_To_Create_Hdl_Memory_Initialized) != SAFE_TRUE) {

		Display_Low_Memory_Error_Message(unit_Name, code_Name, sizeof(Puppet_File_Header_Rec));
		function_Result = SAFE_FALSE;
		goto Error_Handler;
	}
	
	preferred_Size = sizeof(Puppet_File_Rec);
	expandable_Resource_Hdl = nil;

	if (Create_Handle(&preferred_Size
						, (CARDINAL_32)(10240L)
						, ALLOW_PURGING
						, DISALLOW_PARTIAL_AMOUNT
						, (Handle *)&expandable_Resource_Hdl
						, &expandable_Resource_Hdl_Memory_Initialized) != SAFE_TRUE) {
						
		Display_Low_Memory_Error_Message(unit_Name, code_Name, sizeof(Puppet_File_Rec));
		function_Result = SAFE_FALSE;
		goto Error_Handler;
	}

	/*  */
	/* Start a section not calling memory move indirectly */
	/*  */
	header = (Puppet_File_Header_Ptr)*resource_To_Create_Hdl;
	header->valid = SAFE_TRUE;
	header->version = k_PUPPET_VERSION_NOW;
	header->backward_Compatible_Version = k_PUPPET_VERSION_NOW;
	header->total_Special_Colors_To_Honor = current_Puppet->total_Special_Colors_To_Honor;
	
	
	if (current_Puppet->puppet_File_Is_Old_Self_Contained_Type == SAFE_TRUE) {
		switch (current_Puppet->direct_Art.port.color_Type) {
			case k_COLOR_QUICKDRAW:
			case k_32_BIT_COLOR_QUICKDRAW:
			
				if (current_Puppet->direct_Art.port.image_Handle != nil) {
					header->stored_Offscreen_Rect = (**(current_Puppet->direct_Art.port.offscreen_Port.color_Port.portPixMap)).bounds;
					header->stored_Bit_Depth = (**(current_Puppet->direct_Art.port.offscreen_Port.color_Port.portPixMap)).pixelSize;
				}
				else {
					/*  */
					/* Cannot get here. paranoia code */
					/*  */
					SetRect(&(header->stored_Offscreen_Rect), 0,0,1,1);
					header->stored_Bit_Depth = 8;
				}
			break;
			
			case k_ORIGINAL_QUICKDRAW:
			default:
				header->stored_Offscreen_Rect = current_Puppet->direct_Art.port.offscreen_Port.old_Style_Port.portBits.bounds;
				header->stored_Bit_Depth = 1;
		} /* switch (current_Puppet->direct_Art.port.color_Type) */
	}
	else {
		/*  */
		/* Art is indirect file and not controlled by this header. */
		/*  */

		SetRect(&(header->stored_Offscreen_Rect), 0,0,1,1);
		header->stored_Bit_Depth = 8;
		/*  */
		/* HACK HACK HACK we should get arbitrary size of first */
		/* puppet sequence? Shoul I get depth of first puppet sequence? */
		/*  */
	}

	
	
	
	
	header->total_Colors_Used_In_Pallette
		= current_Puppet->total_Colors_Used_In_Pallette;
		
	if (current_Puppet->special_Pallete_To_Use != nil) {
		/*  */
		/* the special pallete needs minding. Set the resource value */
		/*  */
		header->special_Pallette_ID = k_PUPPET_PALLETE_RESOURCE_BASE;
	}
	else {
		header->special_Pallette_ID = 0;
	}

	header->current_Clut_ID = k_PUPPET_CLUT_RESOURCE_BASE;
		
	header->talk_Total_Frames = current_Puppet->direct_Art.talk_Total_Frames;
	header->talk_Start_Frame = current_Puppet->direct_Art.talk_Start_Frame;
	header->talk_Stop_Frame = current_Puppet->direct_Art.talk_Stop_Frame;
	
	/*  */
	/* Clean up strings for file tidiness */
	/*  */
	for (loop = 0; (loop < 256) ; loop++) {
		header->screen_Name[loop] = 0;
	}
	
	for (loop = 0; (loop < 64) ; loop++) {
		header->screen_Short_Name[loop] = 0;
	}
	
	for (loop = 0; (loop < 64) ; loop++) {
		header->file_Name[loop] = 0;
	}
	
	Copy_Pascal_String(current_Puppet->screen_Name, header->screen_Name);
	
	Copy_Pascal_String(current_Puppet->screen_Short_Name
		, header->screen_Short_Name);
	
	Copy_Pascal_String(current_Puppet->file_Name
		, header->file_Name);
			
	header->file_Frame_Total = current_Puppet->direct_Art.frame_Total;
	
	if ((header->file_Frame_Total == 0) && (header->talk_Total_Frames > 0)){
		Make_String_Long_String(message_String
									, (CARDINAL_8 *)"\pBizzare puppet problem: Frames specified ["
									, (**resource_To_Create_Hdl).file_Frame_Total
									, NO_LIMIT
									, (CARDINAL_8 *)"\p] but that number was less than the number of talk frames and was fixed.");
		
		Display_Serious_Error_Message(unit_Name, code_Name, (CARDINAL_8 *)message_String, "\pMemory not available?");
		header->file_Frame_Total = header->talk_Total_Frames;
		
		/*  */
		/* Refresh handle pointer because handle may have just */
		/* moved because of the dialog */
		/*  */
		header = (Puppet_File_Header_Ptr)*resource_To_Create_Hdl;
	}
	
	body = (Puppet_File_Ptr)*expandable_Resource_Hdl;

	body->valid = SAFE_TRUE;
	
	body->total_Frame_Safety = current_Puppet->direct_Art.frame_Total;
	body->file_Total_Lazy_Active_Region = 0;
	for (loop = 0; (loop < k_MAXIMUM_PICS_FRAMES) ; loop++) {
		body->file_Frame_Locations[loop].top = 0;
		body->file_Frame_Locations[loop].left = 0;
		body->file_Frame_Locations[loop].right = 0;
		body->file_Frame_Locations[loop].bottom = 0;
		body->file_Frame_Regions[loop] = 0;
	}

	for (loop = 0; (loop < current_Puppet->direct_Art.frame_Total) ; loop++) {
		body->file_Frame_Locations[loop] = current_Puppet->direct_Art.frame_Locations[loop];
		body->file_Frame_Regions[loop] = k_BASE_REGION_RESOURCE_ID + loop;
	}

	body->file_Total_Lazy_Active_Region = (k_BASE_REGION_RESOURCE_ID - 1);

	body->resource_Template_Count_1 = k_MAXIMUM_PICS_FRAMES;
	body->resource_Template_Count_2 = k_MAXIMUM_PICS_FRAMES;
	body->end_Sentinel = SAFE_TRUE;
	
	/*  */
	/* End the section not calling memory move indirectly */
	/*  */
		
		
	/*  */
	/* Determine free space, first get header size */
	/*  */
	file_Minimum_Size = GetHandleSize((Handle)resource_To_Create_Hdl);
	if (MemError() != noErr) {
		goto Error_Handler;
	}
	
	/*  */
	/* Add in the size of the puppet control body */
	/*  */
	file_Minimum_Size = file_Minimum_Size + GetHandleSize((Handle)expandable_Resource_Hdl);
	if (MemError() != noErr) {
		goto Error_Handler;
	}
	
	/*  */
	/* Add in the sizes of the puppets regions */
	/*  */
	for (loop = 0; (loop < current_Puppet->direct_Art.frame_Total) ; loop++) {
		if (current_Puppet->direct_Art.frame_Regions[loop] != nil) {
			file_Minimum_Size = (file_Minimum_Size 
					+ GetHandleSize((Handle)current_Puppet->direct_Art.frame_Regions[loop]));
			if (MemError() != noErr) {
				goto Error_Handler;
			}
		}
	}
	

	if (((CARDINAL_32)file_Minimum_Size + (CARDINAL_32)MINIMUM_DISK_SPACE_TO_PRESERVE) > volume_Space) {

			Concat_Many(note_String, 3, "\p", volume_Name, "\p is too full for the file. (");
			Make_String_Long_String(work_String_2, (CARDINAL_8 *)note_String, volume_Space, NO_LIMIT,"\p)bytes free, (" );
			Make_String_Long_String(note_String
										, (CARDINAL_8 *)work_String_2
										, (file_Minimum_Size + MINIMUM_DISK_SPACE_TO_PRESERVE)
										, NO_LIMIT
										,"\p)bytes needed.  Find additional space for file." );

		goto Error_Handler_With_Message;
	}

	os_Error = HCreate(destination_Volume
						, destination_Directory_ID
						, destination_File_Name
						, file_Creator
						, file_Type);
						
	if (os_Error == dupFNErr) {
		os_Error = HDelete(destination_Volume
								, destination_Directory_ID
								, destination_File_Name);
		if (os_Error != noErr) {
			Concat_Many(note_String, 3,"\pFSDelete for file ", destination_File_Name, "\p ID = ");
			Concat_Long_To_String(note_String, os_Error, NO_LIMIT);
			goto Error_Handler_With_Message;
		}

		os_Error = HCreate(destination_Volume
							, destination_Directory_ID
							, destination_File_Name
							, file_Creator
							, file_Type);
	} /* if (os_Error == dupFNErr) */

	if (os_Error != noErr) {
		Concat_Many(note_String, 3,"\pCreate for file ", destination_File_Name, "\p ID = ");
		Concat_Long_To_String(note_String, os_Error, NO_LIMIT);
		goto Error_Handler_With_Message;
	} 


	HCreateResFile(destination_Volume
						, destination_Directory_ID
						, destination_File_Name);
	os_Error = ResError();
	if (os_Error != noErr) {
		Concat_Many(note_String, 3,"\pCreateResFile for file ", destination_File_Name, "\p ID = ");
		Concat_Long_To_String(note_String, os_Error, NO_LIMIT);
		goto Error_Handler_With_Message;
	} 

	/*=--------------------------------------------------------------------------=*/
	/* 'OpenRFPerm' used to prevent having to use 'SetVol' */
	/* 'StripAddress' called as per tech notes and THINK reference */
	/*=--------------------------------------------------------------------------=*/
	file_Reference = OpenRFPerm(((StringPtr)(StripAddress(&destination_File_Name[0]))), directory_Reference, fsRdWrPerm );

	os_Error = ResError();
	if ((file_Reference == -1) || ( os_Error != noErr)) {
		Display_Serious_Error_Message(unit_Name, code_Name, "\pOS Error OpenResFile.", "\pNo resource fork?");
		goto Error_Handler;
	}

	resource_Type = k_PUPPET_MAIN_RESOURCE_TYPE;
	resource_ID = k_BASE_PUPPET_RESOURCE_ID;
	Get_NS_String(k_DREAM_STRINGS, k_DONT_EDIT_RESOURCE_COMMENT, resource_Comment);

	if (Add_Resource_Handle((Handle)resource_To_Create_Hdl
											, resource_Type
											, resource_ID
											, resource_Comment) != SAFE_TRUE) {
		goto Error_Handler;
	}
	
	resource_Type = k_PUPPET_EXPANDABLE_RESOURCE_TYPE;
	
	if (Add_Resource_Handle((Handle)expandable_Resource_Hdl
											, resource_Type
											, resource_ID
											, resource_Comment) != SAFE_TRUE) {
		goto Error_Handler;
	}
	
	
	
	if (current_Puppet->puppet_File_Is_Old_Self_Contained_Type != SAFE_TRUE) {
	
		if (current_Puppet->redirected_Information_Memory_In_Use != MEMORY_INITIALIZED) {
			goto Error_Handler;
		}
	
		preferred_Size = sizeof(Puppet_Redirected_File_Rec);
		if (current_Puppet->redirected_Information_Total_Used == 0) {
			/*  */
			/* Absolutley gotta subtract the default initial item, list will be zeroed out */
			/* and end sentinel will ocupy where first position would be */
			/*  */
			preferred_Size = preferred_Size - sizeof(Redirected_File_Rec);
		}
		else {
			preferred_Size = preferred_Size + ((current_Puppet->redirected_Information_Total_Used - 1) * sizeof(Redirected_File_Rec));
		}
		
		/*  */
		/* Add safety sentinel to end of dynamically sized list. It is not part of structure. */
		/*  */
		preferred_Size = preferred_Size + sizeof(Safe_Boolean);
		
		puppet_Redirected_File_List_Hdl = nil;
		puppet_Redirected_File_List_Memory_In_Use = MEMORY_NOT_INITIALIZED;
	
		if (Create_Handle(&preferred_Size
							, (CARDINAL_32)(10240L)
							, ALLOW_PURGING
							, DISALLOW_PARTIAL_AMOUNT
							, (Handle *)&puppet_Redirected_File_List_Hdl
							, &puppet_Redirected_File_List_Memory_In_Use) != SAFE_TRUE) {
							
			Display_Low_Memory_Error_Message(unit_Name, code_Name, sizeof(Puppet_File_Rec));
			function_Result = SAFE_FALSE;
			goto Error_Handler;
		}

		if (Begin_Use_Of_Handle((Handle)puppet_Redirected_File_List_Hdl
										, MOVE_HIGH_NOT_REQUIRED
										, &redirected_Info_Original_State) != SAFE_TRUE) {
			goto Error_Handler;
		}
		redirected_Info = *puppet_Redirected_File_List_Hdl;
	
		redirected_Info->valid = SAFE_TRUE;
		redirected_Info->puppet_File_Contains_No_Internal_Art = SAFE_TRUE;
		redirected_Info->redirection_Version = k_PUPPET_BASE_VERSION_NOW;
		
		redirected_Info->resource_Template_Count_1 = current_Puppet->redirected_Information_Total_Used;
			
		for (loop = 0; (loop < redirected_Info->resource_Template_Count_1) ; loop++) {
			file_Info_Ptr = &(redirected_Info->arbitrary_List_Of_File_Information[loop]);
			loaded_Info_Ptr = &(current_Puppet->redirected_Information_Ptr[loop]);
			
			/*  */
			/* Paranoia truncation in case needed. Ideally we would not */
			/* tuncate our source! All my programs honor 63 character file names, */
			/* Though it might be better to support either infinite (NeXT OS), or 260 */
			/* characters (MS Windows 95) */
			/*  */
			if (loaded_Info_Ptr->file_Name[0] > 63) {
				loaded_Info_Ptr->file_Name[0] = 63;
			}
			Copy_Pascal_String(&loaded_Info_Ptr->file_Name[0], &file_Info_Ptr->file_Name[0]);
			file_Info_Ptr->file_Creation_Time = loaded_Info_Ptr->file_Creation_Time;
			file_Info_Ptr->emotion_Key_Code = loaded_Info_Ptr->emotion_Key_Code;
			file_Info_Ptr->cache_Technique_Desired = loaded_Info_Ptr->cache_Technique_Desired;
			file_Info_Ptr->offset_Point_Position = loaded_Info_Ptr->offset_Point_Position;
			file_Info_Ptr->special_Effect_Classification = loaded_Info_Ptr->special_Effect_Classification;
			file_Info_Ptr->special_Effect_Value = loaded_Info_Ptr->special_Effect_Value;
			file_Info_Ptr->special_Effect_Extra_Value = loaded_Info_Ptr->special_Effect_Extra_Value;
		}
		
		/*  */
		/* Carefully set the end sentinel to ensure that resource is secure. */
		/*  */
		file_Info_Ptr = &(redirected_Info->arbitrary_List_Of_File_Information[redirected_Info->resource_Template_Count_1]);
		*(Safe_Boolean *)file_Info_Ptr = SAFE_TRUE;
		
		/*  */
		/* End the section not calling memory move indirectly but was locked anyway*/
		/*  */

		
		resource_ID = k_BASE_PUPPET_RESOURCE_ID;
		Get_NS_String(k_DREAM_STRINGS, k_DONT_EDIT_RESOURCE_COMMENT, resource_Comment);
		resource_Type = k_PUPPET_REDIRECTED_FILES_RESOURCE_TYPE;
		
		if (Add_Resource_Handle((Handle)puppet_Redirected_File_List_Hdl
												, resource_Type
												, resource_ID
												, resource_Comment) != SAFE_TRUE) {
			goto Error_Handler;
		}
		
	} /* if (current_Puppet->puppet_File_Is_Old_Self_Contained_Type != SAFE_TRUE) */
	
	
	/*  */
	/* Save the puppets composite region (This is not cross platform compatible) */
	/*  */
	resource_Type = k_PUPPET_APPLE_REGION_RESOURCE_TYPE;

	if (current_Puppet->direct_Art.frame_Total > 0) {
		if ((current_Puppet->direct_Art.original_Total_Lazy_Active_Region != nil)
			&& ((**expandable_Resource_Hdl).file_Total_Lazy_Active_Region != 0)){
			
			resource_ID = (**expandable_Resource_Hdl).file_Total_Lazy_Active_Region;
			
			if (Add_Resource_Handle((Handle)current_Puppet->direct_Art.original_Total_Lazy_Active_Region
													, resource_Type
													, resource_ID
													, "\pComposite region") != SAFE_TRUE) {
				goto Error_Handler;
			}
		}
	}
	
	/*  */
	/* Save all puppet frame  regions (This is not cross platform compatible) */
	/*  */
	for (loop = 0; (loop < current_Puppet->direct_Art.frame_Total) ; loop++) {
		if (current_Puppet->direct_Art.frame_Regions[loop] != nil) {
			
			resource_ID = (**expandable_Resource_Hdl).file_Frame_Regions[loop];
			
			if (Add_Resource_Handle((Handle)current_Puppet->direct_Art.frame_Regions[loop]
													, resource_Type
													, resource_ID
													, "\pFrame region") != SAFE_TRUE) {
				goto Error_Handler;
			}
		}
	}
	
	/*  */
	/* Save pallete listing all used colors (except B&W) and all special colors. */
	/*  */
	if ((**resource_To_Create_Hdl).special_Pallette_ID != 0) {		
		resource_ID = (**resource_To_Create_Hdl).special_Pallette_ID;
		resource_Type = k_PUPPET_PALLETE_RESOURCE_TYPE;
		
		if (Add_Resource_Handle((Handle)current_Puppet->special_Pallete_To_Use
												, resource_Type
												, resource_ID
												, "\p") != SAFE_TRUE) {
			goto Error_Handler;
		}
	}
	
	/*  */
	/* Save current puppet offscreen port's color lookup table */
	/*  */
	if ((**resource_To_Create_Hdl).current_Clut_ID != 0) {		
		resource_ID = (**resource_To_Create_Hdl).current_Clut_ID;
		resource_Type = k_PUPPET_CLUT_RESOURCE_TYPE;
				
		if (current_Puppet->puppet_File_Is_Old_Self_Contained_Type == SAFE_TRUE) {
		
			switch (current_Puppet->direct_Art.port.color_Type) {
				case k_COLOR_QUICKDRAW:
				case k_32_BIT_COLOR_QUICKDRAW:
					if (current_Puppet->direct_Art.port.image_Handle != nil) {
						temp_Clut_Handle = (Handle)current_Puppet->direct_Art.port.color_Table_Handle;
					}
					else {
						(**resource_To_Create_Hdl).current_Clut_ID = 0;
					}
				break;
				
				case k_ORIGINAL_QUICKDRAW:
				default:
					(**resource_To_Create_Hdl).current_Clut_ID = 0;
			} /* switch (current_Puppet->direct_Art.port.color_Type) */
		}
		else {
			(**resource_To_Create_Hdl).current_Clut_ID = 0;
		}

		if ((**resource_To_Create_Hdl).current_Clut_ID != 0) {
			if (Add_Resource_Handle(temp_Clut_Handle
													, resource_Type
													, resource_ID
													, "\p") != SAFE_TRUE) {
				goto Error_Handler;
			}
		}
		
	} /* if ((**resource_To_Create_Hdl).current_Clut_ID != 0) ... */
	
	/*  */
	/* Now add the template for the header */
	/*  */

	resource_Type = 'TMPL';
	resource_ID = k_PUPPET_HEADER_TEMPLATE_ID;
	
	template_Hdl = GetResource(resource_Type, resource_ID);
	if (template_Hdl != nil) {
	
		DetachResource(template_Hdl);
		os_Error = ResError();
		if ( os_Error != noErr) {
			Display_Serious_Error_Message(unit_Name, code_Name, "\pOS Error DetachResource.", "\p");
			goto Error_Handler;
		} 
	
		if (Add_Resource_Handle((Handle)template_Hdl
												, resource_Type
												, resource_ID
												, k_PUPPET_HEADER_TEMPLATE_NAME) != SAFE_TRUE) {
			goto Error_Handler;
		}
		
		DisposeHandle(template_Hdl);
		os_Error = MemError();
		if ( os_Error != noErr) {
			Display_Serious_Error_Message(unit_Name, code_Name, "\pOS Error DetachResource.", "\p");
			goto Error_Handler;
		} 
		
	} /* if (template_Hdl != nil) */

	/*  */
	/* Now add the template for the puppet body */
	/*  */

	resource_Type = 'TMPL';
	resource_ID = k_PUPPET_BODY_TEMPLATE_ID;
	
	template_Hdl = GetResource(resource_Type, resource_ID);
	if (template_Hdl != nil) {
	
		DetachResource(template_Hdl);
		os_Error = ResError();
		if ( os_Error != noErr) {
			Display_Serious_Error_Message(unit_Name, code_Name, "\pOS Error DetachResource.", "\p");
			goto Error_Handler;
		} 
	
		if (Add_Resource_Handle((Handle)template_Hdl
												, resource_Type
												, resource_ID
												, k_PUPPET_BODY_TEMPLATE_NAME) != SAFE_TRUE) {
			goto Error_Handler;
		}
		
		DisposeHandle(template_Hdl);
		os_Error = MemError();
		if ( os_Error != noErr) {
			Display_Serious_Error_Message(unit_Name, code_Name, "\pOS Error DetachResource.", "\p");
			goto Error_Handler;
		} 
		
	} /* if (template_Hdl != nil) */


	/*  */
	/* Now add the template for the redirected file information */
	/*  */

	resource_Type = 'TMPL';
	resource_ID = k_PUPPET_REDIRECTED_FILES_TEMPLATE_ID;
	
	template_Hdl = GetResource(resource_Type, resource_ID);
	if (template_Hdl != nil) {
	
		DetachResource(template_Hdl);
		os_Error = ResError();
		if ( os_Error != noErr) {
			Display_Serious_Error_Message(unit_Name, code_Name, "\pOS Error DetachResource.", "\p");
			goto Error_Handler;
		} 
	
		if (Add_Resource_Handle((Handle)template_Hdl
												, resource_Type
												, resource_ID
												, k_PUPPET_REDIRECTED_FILES_TEMPLATE_NAME) != SAFE_TRUE) {
			goto Error_Handler;
		}
		
		DisposeHandle(template_Hdl);
		os_Error = MemError();
		if ( os_Error != noErr) {
			Display_Serious_Error_Message(unit_Name, code_Name, "\pOS Error DetachResource.", "\p");
			goto Error_Handler;
		} 
		
	} /* if (template_Hdl != nil) */


	
	CloseResFile(file_Reference);
	os_Error = ResError();
	if ( os_Error != noErr) {
		Display_Serious_Error_Message(unit_Name, code_Name, "\pOS Error CloseResFile.", "\p");
	}
	file_Reference = 0;
	

	if (current_Puppet->puppet_File_Is_Old_Self_Contained_Type == SAFE_TRUE) {
		/*  */
		/* If direct art, open data fork and fill it with our structure */
		/* The data fork is used because we might want to read */
		/* portions in during interrupts, as a background task, or */
		/* because we need to load it into memory in a specific */
		/* spot and there may not be enough room for two copies */
		/* to exchange from a loaded monster resource. It is true */
		/* that system 7 allows partial resource loading, but some */
		/* resource compressors, and other things fail when these are used. */
		/*  */
		os_Error = HOpen(destination_Volume
										,destination_Directory_ID
										,destination_File_Name
										,fsRdWrPerm
										,&file_Reference);
		if (os_Error != noErr) {
			Concat_Many(note_String, 3,"\pHOpen for file ", destination_File_Name, "\p ID = ");
			Concat_Long_To_String(note_String, os_Error, NO_LIMIT);
			goto Error_Handler_With_Message;
		} 
		
		transfer_Count_Copy = 0;
		switch (current_Puppet->direct_Art.port.color_Type) {
			case k_COLOR_QUICKDRAW:
			case k_32_BIT_COLOR_QUICKDRAW:
				if (current_Puppet->direct_Art.port.image_Handle != nil) {
					transfer_Count_Copy = GetHandleSize(current_Puppet->direct_Art.port.image_Handle);
					os_Error = MemError();
					if ( os_Error != noErr) {
						Display_Serious_Error_Message(unit_Name, code_Name, "\pBad Handle for Port.", "\p");
						goto Error_Handler;
					}
				}
			break;
			
			case k_ORIGINAL_QUICKDRAW:
			default:
				if (current_Puppet->direct_Art.port.image_Handle != nil) {
					transfer_Count_Copy = GetHandleSize(current_Puppet->direct_Art.port.image_Handle);
					os_Error = MemError();
					if ( os_Error != noErr) {
						Display_Serious_Error_Message(unit_Name, code_Name, "\pBad Handle for Port.", "\p");
						goto Error_Handler;
					}
				}
		} /* switch (current_Puppet->direct_Art.port.color_Type) */
	
		transfer_Count = sizeof(transfer_Count_Copy);
		os_Error = FSWrite(file_Reference, &transfer_Count, (Ptr)&transfer_Count_Copy);
		if (os_Error != noErr) {
			Concat_Many(note_String, 3,"\pFSWrite for file ", destination_File_Name, "\p ID = ");
			Concat_Long_To_String(note_String, os_Error, NO_LIMIT);
			goto Error_Handler_With_Message;
		}
	
		transfer_Count = transfer_Count_Copy;
		os_Error = FSWrite(file_Reference, &transfer_Count, (Ptr)*current_Puppet->direct_Art.port.image_Handle);
		if (os_Error != noErr) {
	
			os_Error = FSClose(file_Reference);
			if ( os_Error != noErr) {
				/*  */
				/* Ignore, this was just an abortion tactic anyway */
				/*  */
			}
	
			Concat_Many(note_String, 3,"\pFSWrite for file ", destination_File_Name, "\p ID = ");
			Concat_Long_To_String(note_String, os_Error, NO_LIMIT);
			goto Error_Handler_With_Message;
		}
	
		os_Error = FSClose(file_Reference);
		if ( os_Error != noErr) {
			Display_Serious_Error_Message(unit_Name, code_Name, "\pOS Error FSClose.", "\p");
		}
		file_Reference = 0;
	} /* if (current_Puppet->puppet_File_Is_Old_Self_Contained_Type == SAFE_TRUE) */


	function_Result = SAFE_TRUE;
	goto Normal_Function_Exit;

Error_Handler_With_Message:
	function_Result = SAFE_FALSE;
	Display_Serious_Error_Message(unit_Name, code_Name, note_String, "\p");
	goto Normal_Function_Exit;

Error_Handler:
	Display_Serious_Error_Message(unit_Name, code_Name, "\pInternal problem saving PICS", "\p");
	function_Result = SAFE_FALSE;
		
Normal_Function_Exit:

	if (expandable_Resource_Hdl_Memory_Initialized == MEMORY_INITIALIZED) {
		if (Destroy_Handle((Handle *)&expandable_Resource_Hdl
							, &expandable_Resource_Hdl_Memory_Initialized) != SAFE_TRUE) {
							
			Display_Serious_Error_Message(unit_Name, code_Name, "\pError with DisposHandle", "\pSystem Corrupted?");
		}
	}
	
	if (resource_To_Create_Hdl_Memory_Initialized == MEMORY_INITIALIZED) {
		if (Destroy_Handle((Handle *)&resource_To_Create_Hdl
							, &resource_To_Create_Hdl_Memory_Initialized) != SAFE_TRUE) {
							
			Display_Serious_Error_Message(unit_Name, code_Name, "\pError with DisposHandle", "\pSystem Corrupted?");
		}
	}
	
	if (puppet_Redirected_File_List_Memory_In_Use == MEMORY_INITIALIZED) {
		if (Destroy_Handle((Handle *)&puppet_Redirected_File_List_Hdl
							, &puppet_Redirected_File_List_Memory_In_Use) != SAFE_TRUE) {
							
			Display_Serious_Error_Message(unit_Name, code_Name, "\pError with DisposHandle", "\pSystem Corrupted?");
		}
	}
	
	return(function_Result);
	
} /* Safe_Boolean	Save_Puppet_Sequence... */
/*=**************************************************************************=*/


/*=**************************************************************************=*/
/* END OF FILE: Dreaming_File_Loading.c */
