/* START OF FILE: Dreaming_Scene_Files.c */


/*=**************************************************************************=*/
/* [---BEGIN---] FILE HEADER COMMENTS */
/**
 ** SCREEN VIEW = [Adobe Syntax Black-10, Tabs = 4]
 ** PRINTER VIEW = [Adobe Syntax Black-10, Tabs = 4]
 ** =======================================================================
 ** Written by James Merkle
 ** Dreaming_Scene_Files.c - Upkeep for lists of Scenes, Puppets, and Props. Both loaded or on Disk
 ** (C) Copyright 1994 James Merkle
 ** All rights reserved, except those wishing to "copy" moderate sections of code or files may do so, so long as
 ** such code is never used directly either in animation-related tools or multimedia tools that are  
 ** used in live performance or live broadcast or recorded mass market broadcast or film. Desktop usage not restricted.
 ** Conventional desktop game or entertainment not restricted.
 ** =======================================================================
 **
 **
 ** ============
 ** Revision History
 ** ============
 **
 **  1994.02.01		(James Merkle)
 **						Initial coding started
 **
 **
 **
 **
 ** =================
 ** OTHER INFORMATION
 ** =================
 **  
 **
 **/
/* [----END----] FILE HEADER COMMENTS */
/*=**************************************************************************=*/


/*=**************************************************************************=*/
/* [---BEGIN---] #include AREA */

#include "Core_Universal.h" /* Non_Specific_Support_Routines */
#include "Core_Specific.h"
#include "Core_Globals.h"
#include "String_Utilities.h"
#include "Error_Utilities.h"
#include "Shell_Dialogs.h"
#include "Shell_Globals.h"
#include "Volume_Utilities.h"
#include "SysHooks.h"
#include "Interface_Utilities.h"
#include "File_Resource_Utilities.h"

#include "Dreaming_Animation.h"

#include "Dreaming_Popup_Dialogs.h"
#include "Cursor_Utilities.h"
#include "Dreaming_Backdrop_Files.h"
#include "Spin_O_Rama_Files.h"
#include "Object_Inventory.h"

#include "Dreaming_Stuff.h"
#include "Dreaming_Color_Stuff.h"
#include "Dreaming_File_Loading.h"

#include "Dreaming_Scene_Files.h"

/* [----END----] #include AREA */
/*=**************************************************************************=*/


/*=**************************************************************************=*/
/* [---BEGIN---] #define AREA */


#define	k_EXTRA_PAD_AMOUNT								8

#define	k_SCENE_EXPANDABLE_RESOURCE_TYPE		'Scn2'
#define	k_SCENE_ITEM_RESOURCE_TYPE					'Item'

#define	k_SCENE_HEADER_TEMPLATE_NAME			"\pScn1"
#define	k_SCENE_HEADER_TEMPLATE_ID					3002
#define	k_SCENE_BODY_TEMPLATE_NAME				"\pScn2"
#define	k_SCENE_BODY_TEMPLATE_ID						3003
#define	k_SCENE_ITEM_TEMPLATE_NAME					"\pItem"
#define	k_SCENE_ITEM_TEMPLATE_ID							3004

#define	k_SCENE_APPLE_REGION_RESOURCE_TYPE	'Rgn1'
#define	k_SCENE_PALLETE_RESOURCE_TYPE				'pltt'
#define	k_SCENE_CLUT_RESOURCE_TYPE					'clut'

#define	k_SCENE_PALLETE_RESOURCE_BASE		128
#define	k_SCENE_CLUT_RESOURCE_BASE			128

#define	k_BASE_REGION_RESOURCE_ID		k_BASE_SCENE_RESOURCE_ID

/*  */
/* For this version, total items is the same as the RAM-based limit */
/*  */
#define 	k_MAX_FILE_STAGE_ITEMS		k_MAX_STAGE_ITEMS

/* [----END----] #define AREA */
/*=**************************************************************************=*/


/*=**************************************************************************=*/
/* [---BEGIN---] TYPE AREA */

 typedef struct {
	Safe_Boolean						valid;
	Str63									item_Name;
	Rect										scene_Position;
 } Item_File_Reference_Rec, *Item_File_Reference_Ptr, **Item_File_Reference_Hdl;

 typedef struct {
	Safe_Boolean						valid;
	CARDINAL_32					total_Items_In_Scene;
	INTEGER_16						resource_Template_Count_1;
	Resource_Identifier			file_Scene_Items[k_MAX_FILE_STAGE_ITEMS];
	Safe_Boolean						end_Sentinel;
 } Scene_File_Rec, *Scene_File_Ptr, **Scene_File_Hdl;

/* [----END----] TYPE AREA */
/*=**************************************************************************=*/


/*=**************************************************************************=*/
/* [---BEGIN---] PROTOTYPE AREA */
															
Safe_Boolean Load_Some_Item_Data(void);
																
Safe_Boolean	Select_Scene_And_Display_First_Time(CARDINAL_32	desired_Scene);

/* [----END----] PROTOTYPE AREA */
/*=**************************************************************************=*/


/*=**************************************************************************=*/
/* [---BEGIN---] GLOBAL DEFINITION AREA */


/* [----END----] GLOBAL DEFINITION AREA */
/*=**************************************************************************=*/


/*=**************************************************************************=*/
/* [---BEGIN---] LOCAL STATIC DEFINITION AREA */

static CARDINAL_8	*unit_Name;

/* [----END----] LOCAL STATIC DEFINITION AREA */
/*=**************************************************************************=*/


/*=**************************************************************************=*/
Safe_Boolean Initialize_Dreaming_Scene_Files(void)
{
	CARDINAL_8			*static_Unit_Name = "\pDreaming_Scene_Files";
/* END VAR */

/*  */
/* Global persistant structures are created here */
/*  */

	unit_Name = static_Unit_Name;
	return(SAFE_TRUE);
}
/*=**************************************************************************=*/


/*=**************************************************************************=*/
Safe_Boolean Destroy_Dreaming_Scene_Files(void)
{
	Safe_Boolean		function_Result;
/* END VAR */

/*  */
/* Global persistant structures are deleted here */
/*  */

	function_Result = SAFE_TRUE;
	return(function_Result);
}
/*=**************************************************************************=*/		

		
/*=**************************************************************************=*/
Safe_Boolean Reset_Menu_Control_Selectability(void)
/*  */
/* Invoke after any command that adds or removes scenes and Items */
/* To update menubar items */
/*  */
{
	Safe_Boolean					function_Result;
/* END VAR */


	function_Result = SAFE_FALSE;

	/*  */
	/* WATEFUL SECTION! This is the tiny section that sets menu states */
	/* in an annoyingly inneficient method because two many states are needed. */
	/* Because of this, it gets to go first, before menu knockover flags start flipping */
	/*  */
	if ((dream.play.total_Scenes > 0) || (dream.spin_Objects.total_Spin_Objects > 0)) {
			/*  */
			/* Set this state regardless of flag. tricky without extra flag */
			/*  */
			EnableItem(g_Shell.interface.program_Menus[WINDOWS_MENU] , MENU_COMMAND_SETUP_CHROMAKEY_LIST);
	}
	else {
			DisableItem(g_Shell.interface.program_Menus[WINDOWS_MENU] , MENU_COMMAND_SETUP_CHROMAKEY_LIST);
	}


	if (dream.backdrops.total_Backdrops > 0){
		if (g_Menus_Assume_No_Backgrounds == SAFE_TRUE) {
			EnableItem(g_Shell.interface.program_Menus[BACKDROPS_MENU] , MENU_COMMAND_CLOSE_BACKDROP);
			EnableItem(g_Shell.interface.program_Menus[BACKDROPS_MENU] , MENU_COMMAND_SAVE_BACKDROP);
			EnableItem(g_Shell.interface.program_Menus[BACKDROPS_MENU] , MENU_COMMAND_EXPORT_BACKDROP);
			g_Menus_Assume_No_Backgrounds = SAFE_FALSE;
		}
	}
	else {
		if (g_Menus_Assume_No_Backgrounds != SAFE_TRUE) {
			DisableItem(g_Shell.interface.program_Menus[BACKDROPS_MENU] , MENU_COMMAND_CLOSE_BACKDROP);
			DisableItem(g_Shell.interface.program_Menus[BACKDROPS_MENU] , MENU_COMMAND_SAVE_BACKDROP);			
			DisableItem(g_Shell.interface.program_Menus[BACKDROPS_MENU] , MENU_COMMAND_EXPORT_BACKDROP);			
			
			g_Menus_Assume_No_Backgrounds = SAFE_TRUE;
		}
	}


	if (dream.spin_Objects.total_Spin_Objects > 0){
		if (g_Menus_Assume_No_Spin_Objects == SAFE_TRUE) {
			EnableItem(g_Shell.interface.program_Menus[ITEMS_MENU] , MENU_COMMAND_CLOSE_SPIN_OBJECT);
			EnableItem(g_Shell.interface.program_Menus[ITEMS_MENU] , MENU_COMMAND_SAVE_SPIN_OBJECT);
			EnableItem(g_Shell.interface.program_Menus[ITEMS_MENU] , MENU_COMMAND_EXPORT_SPIN_OBJECT);
			EnableItem(g_Shell.interface.program_Menus[ITEMS_MENU] , MENU_COMMAND_RECOMPRESS_SPIN_OBJECT);

			EnableItem(g_Shell.interface.program_Menus[ITEMS_MENU] , MENU_COMMAND_TIME_VOICE);

			g_Menus_Assume_No_Spin_Objects = SAFE_FALSE;
		}
	}
	else {
		if (g_Menus_Assume_No_Spin_Objects != SAFE_TRUE) {
			DisableItem(g_Shell.interface.program_Menus[ITEMS_MENU] , MENU_COMMAND_CLOSE_SPIN_OBJECT);
			DisableItem(g_Shell.interface.program_Menus[ITEMS_MENU] , MENU_COMMAND_SAVE_SPIN_OBJECT);			
			DisableItem(g_Shell.interface.program_Menus[ITEMS_MENU] , MENU_COMMAND_EXPORT_SPIN_OBJECT);			
			DisableItem(g_Shell.interface.program_Menus[ITEMS_MENU] , MENU_COMMAND_RECOMPRESS_SPIN_OBJECT);			
			
			g_Menus_Assume_No_Spin_Objects = SAFE_TRUE;
		}
	}

	if (dream.play.total_Scenes > 0){
		if (g_Menus_Assume_No_Scenes == SAFE_TRUE) {
			EnableItem(g_Shell.interface.program_Menus[FILE_MENU] , MENU_COMMAND_CLOSE_SCENE);
			EnableItem(g_Shell.interface.program_Menus[FILE_MENU] , MENU_COMMAND_CLOSE_ALL_SCENE);
			EnableItem(g_Shell.interface.program_Menus[FILE_MENU] , MENU_COMMAND_SAVE_SCENE);
			EnableItem(g_Shell.interface.program_Menus[FILE_MENU] , MENU_COMMAND_SAVE_SCENE_AS);
			EnableItem(g_Shell.interface.program_Menus[FILE_MENU] , MENU_COMMAND_ADD_ITEM);
			
			EnableItem(g_Shell.interface.program_Menus[FILE_MENU] , MENU_COMMAND_REMOVE_ITEM);
			EnableItem(g_Shell.interface.program_Menus[FILE_MENU] , MENU_COMMAND_REPOSITION_ITEM);

			
			EnableItem(g_Shell.interface.program_Menus[FILE_MENU] , MENU_COMMAND_PAGE_SETUP);
			EnableItem(g_Shell.interface.program_Menus[FILE_MENU] , MENU_COMMAND_PAGE_PRINT);
			EnableItem(g_Shell.interface.program_Menus[FILE_MENU] , MENU_COMMAND_PRINT_SAVED_FILES);
			EnableItem(g_Shell.interface.program_Menus[FILE_MENU] , MENU_COMMAND_CHANGE_BACKDROP);
			
			EnableItem(g_Shell.interface.program_Menus[WINDOWS_MENU] , MENU_COMMAND_SET_SCENE_AS_DEFAULT);
			
			g_Menus_Assume_No_Scenes = SAFE_FALSE;
		}
	}
	else {
		if (g_Menus_Assume_No_Scenes != SAFE_TRUE) {
			DisableItem(g_Shell.interface.program_Menus[FILE_MENU] , MENU_COMMAND_CLOSE_SCENE);
			DisableItem(g_Shell.interface.program_Menus[FILE_MENU] , MENU_COMMAND_CLOSE_ALL_SCENE);
			DisableItem(g_Shell.interface.program_Menus[FILE_MENU] , MENU_COMMAND_SAVE_SCENE);
			DisableItem(g_Shell.interface.program_Menus[FILE_MENU] , MENU_COMMAND_SAVE_SCENE_AS);
			
			DisableItem(g_Shell.interface.program_Menus[FILE_MENU] , MENU_COMMAND_ADD_ITEM);
			
			DisableItem(g_Shell.interface.program_Menus[FILE_MENU] , MENU_COMMAND_REMOVE_ITEM);
			DisableItem(g_Shell.interface.program_Menus[FILE_MENU] , MENU_COMMAND_REPOSITION_ITEM);

			DisableItem(g_Shell.interface.program_Menus[FILE_MENU] , MENU_COMMAND_PAGE_SETUP);
			DisableItem(g_Shell.interface.program_Menus[FILE_MENU] , MENU_COMMAND_PAGE_PRINT);
			DisableItem(g_Shell.interface.program_Menus[FILE_MENU] , MENU_COMMAND_PRINT_SAVED_FILES);
			
			DisableItem(g_Shell.interface.program_Menus[FILE_MENU] , MENU_COMMAND_CHANGE_BACKDROP);
			
			DisableItem(g_Shell.interface.program_Menus[WINDOWS_MENU] , MENU_COMMAND_SET_SCENE_AS_DEFAULT);
			
			g_Menus_Assume_No_Scenes = SAFE_TRUE;
		}
	}
	
	if ((dream.play.total_Scenes > 0) && (dream.active_Puppets_Total > 0)) {
			/*  */
			/* Set this state regardless of flag. tricky without extra flag */
			/*  */
			
			EnableItem(g_Shell.interface.program_Menus[ITEMS_MENU] , MENU_COMMAND_TIME_VOICE);

	}
	else {
		if (dream.active_Puppets_Total == 0) {
			if (g_Menus_Assume_No_Items != SAFE_TRUE) {
				
				if (dream.spin_Objects.total_Spin_Objects == 0){
					DisableItem(g_Shell.interface.program_Menus[ITEMS_MENU] , MENU_COMMAND_TIME_VOICE);
				}

				g_Menus_Assume_No_Items = SAFE_TRUE;
			}
		}
		else {
			/*  */
			/* Set this state regardless of flag. They are unselectable because no scenes */
			/*  */

			DisableItem(g_Shell.interface.program_Menus[FILE_MENU] , MENU_COMMAND_ADD_ITEM);
			
			DisableItem(g_Shell.interface.program_Menus[FILE_MENU] , MENU_COMMAND_REMOVE_ITEM);
			DisableItem(g_Shell.interface.program_Menus[FILE_MENU] , MENU_COMMAND_REPOSITION_ITEM);
			
			DisableItem(g_Shell.interface.program_Menus[ITEMS_MENU] , MENU_COMMAND_TIME_VOICE);
		}
	}
		
	if (dream.active_Puppets_Total > 0) {
		if (g_Menus_Assume_No_Items == SAFE_TRUE) {
			EnableItem(g_Shell.interface.program_Menus[ITEMS_MENU] , MENU_COMMAND_SAVE_PUPPET);
			EnableItem(g_Shell.interface.program_Menus[ITEMS_MENU] , MENU_COMMAND_CLOSE_PUPPET);
			EnableItem(g_Shell.interface.program_Menus[ITEMS_MENU] , MENU_COMMAND_EXPORT_PUPPET);
			EnableItem(g_Shell.interface.program_Menus[ITEMS_MENU] , MENU_COMMAND_ADD_ANPOTHER_SPINFILE);
			EnableItem(g_Shell.interface.program_Menus[ITEMS_MENU] , MENU_COMMAND_EXPORT_PROP);
			
			g_Menus_Assume_No_Items = SAFE_FALSE;
		}
	}
	else {
		if (g_Menus_Assume_No_Items != SAFE_TRUE) {
			DisableItem(g_Shell.interface.program_Menus[ITEMS_MENU] , MENU_COMMAND_SAVE_PUPPET);
			DisableItem(g_Shell.interface.program_Menus[ITEMS_MENU] , MENU_COMMAND_CLOSE_PUPPET);
			DisableItem(g_Shell.interface.program_Menus[ITEMS_MENU] , MENU_COMMAND_EXPORT_PUPPET);
			DisableItem(g_Shell.interface.program_Menus[ITEMS_MENU] , MENU_COMMAND_ADD_ANPOTHER_SPINFILE);
			DisableItem(g_Shell.interface.program_Menus[ITEMS_MENU] , MENU_COMMAND_EXPORT_PROP);
			
			g_Menus_Assume_No_Items = SAFE_TRUE;
		}
	}


	
	function_Result = SAFE_TRUE;
	goto Normal_Function_Exit;
	
Error_Handler:
	function_Result = SAFE_FALSE;

Normal_Function_Exit:

	return(function_Result);
}
/*=**************************************************************************=*/


/*=**************************************************************************=*/
Safe_Boolean Advance_Scene(CARDINAL_32	direction)
{
Safe_Boolean			function_Result;
INTEGER_32			index;
INTEGER_32			total_Listed_Items;
Boolean					located;
CARDINAL_32		previous_Scene;
CARDINAL_32		scene;
/* END VAR */
	
		
	function_Result = SAFE_FALSE;
	previous_Scene = dream.current_Scene;

	located = FALSE;
	scene = 0;
	
	if (direction == 0) {
	
		total_Listed_Items = 0;
		for (index = ((INTEGER_32)dream.current_Scene - 1L); ((index >= 0) 
			&& (total_Listed_Items < dream.play.total_Scenes)
			&& (located != TRUE)); index = index - 1) {
						
			if (dream.play.scene_Memory_In_Use[index] == MEMORY_INITIALIZED) {
				total_Listed_Items += 1;
				
				/*  */
				/* If criteria matches (always does), set exit condition */
				/*  */
				located = TRUE;
				scene = index;
			}
			
		}
		
		if (located != TRUE) {
			/*  */
			/* Wrap around!, select last one found in list */
			/*  */
			total_Listed_Items = 0;
			for (index = (k_MAX_ACTIVE_SCENES - 1); ((index >= 0) 
				&& (total_Listed_Items < dream.play.total_Scenes)
				&& (located != TRUE)); index = index - 1) {
							
				if (dream.play.scene_Memory_In_Use[index] == MEMORY_INITIALIZED) {
					total_Listed_Items += 1;
					
					/*  */
					/* If criteria matches (always does), set exit condition */
					/*  */
					located = TRUE;
					scene = index;
				}
				
			}
		}		
		
		if (located == TRUE) {
			if (dream.current_Scene != scene) {
				dream.previously_Displayed_Scene = dream.current_Scene;
				dream.current_Scene = scene;
			}
			else {
				located = FALSE;
			}
		}
		
	}
	else if (direction == 1) {
	
		total_Listed_Items = 0;
		for (index = (dream.current_Scene + 1); ((index < k_MAX_ACTIVE_SCENES) 
			&& (total_Listed_Items < dream.play.total_Scenes)
			&& (located != TRUE)); index++) {
						
			if (dream.play.scene_Memory_In_Use[index] == MEMORY_INITIALIZED) {
				total_Listed_Items += 1;
				
				/*  */
				/* If criteria matches (always does), set exit condition */
				/*  */
				located = TRUE;
				scene = index;
			}
			
		}
		
		if (located != TRUE) {
			/*  */
			/* Wrap around!, select first one found in list */
			/*  */
			total_Listed_Items = 0;
			for (index = 0; ((index < k_MAX_ACTIVE_SCENES) 
				&& (total_Listed_Items < dream.play.total_Scenes)
				&& (located != TRUE)); index++) {
							
				if (dream.play.scene_Memory_In_Use[index] == MEMORY_INITIALIZED) {
					total_Listed_Items += 1;
					
					/*  */
					/* If criteria matches (always does), set exit condition */
					/*  */
					located = TRUE;
					scene = index;
				}
				
			}
		}		
		
		if (located == TRUE) {
			if (dream.current_Scene != scene) {
				dream.previously_Displayed_Scene = dream.current_Scene;
				dream.current_Scene = scene;
			}
			else {
				located = FALSE;
			}
		}
			
	}
	else {
		/* do nothing, dream.current_Scene is active */
		located = FALSE;
	}
	
	
	if (located == TRUE) {
		if (Redraw_Idle_Screen_Completely(FALSE, FALSE) != SAFE_TRUE) {
			function_Result = SAFE_FALSE;
			
			dream.current_Scene = previous_Scene;
			if (Redraw_Idle_Screen_Completely(FALSE, FALSE) != SAFE_TRUE) {
			}
		}
	}


	function_Result = SAFE_TRUE;
	goto Normal_Function_Exit;

Error_Handler:
	function_Result = SAFE_FALSE;
		
Normal_Function_Exit:
	return(function_Result);
}
/*=**************************************************************************=*/		

		
/*=**************************************************************************=*/		
Safe_Boolean Load_Some_Item_Data(void)
/*  */
/* Called as a slow idle loop event task distributed as part */
/* of background I/O while puppets are animating. */
/* governor should have tuning based on whether any animation */
/* is really occuring before deciding how often t call this routine */
/*  */
{
	Safe_Boolean		function_Result;
	FSSpec				file_Specification;
	CARDINAL_32	current_Item_Index;
	Scene_Ptr			active_Scene_Ptr;
	CARDINAL_32	item_Address;
	Load_Backdrop_Rec	params_For_Load_Backdrop;
	Safe_Boolean		address_Instead_Of_File;
	Safe_Boolean		abort_Desired;
	Safe_Boolean		result;
	Safe_Boolean		puppet_Structures_Created;
	SIMPLE_LOOP	loop;
	INTEGER_16		position_Of_Item_In_Scene;
	INTEGER_16		current_Scene_Index;
	INTEGER_16		pict_Resource_Id;
	INTEGER_16		index_Used;
	INTEGER_16		current_Item;
	Boolean				still_Searching;
	Boolean				puppets_All_Loaded;
	Boolean				stages_Finished;
	INTEGER_16		 current_Stage;
	CARDINAL_8		*code_Name;
	FSSpec				pict_File_Info;
/* END VAR */


	code_Name = "\pLoad_Some_Item_Data";
	function_Result = SAFE_FALSE;
	puppet_Structures_Created = SAFE_FALSE;

	if (dream.play.scenes_Still_Loading == 0) {
		function_Result = SAFE_TRUE;
		goto Normal_Function_Exit;
	}

	/*  */
	/* Search for a scene to load some data into, ugly but not too slow */
	/* a tradeoff for now */
	/*  */
	still_Searching = TRUE;
	puppets_All_Loaded = TRUE;
	current_Scene_Index = 0;
	for(index_Used = 0; ((index_Used < k_MAX_ACTIVE_SCENES) && (still_Searching == TRUE)); index_Used++) {
		if (dream.play.scene_Memory_In_Use[index_Used] == SAFE_TRUE) {
		active_Scene_Ptr = *(dream.play.scene[index_Used]);
			if (active_Scene_Ptr->initialized == SAFE_TRUE) {
			
				/*  */
				/* NOTE: Also check and compare here for background load needed */
				/* before skipping over them */
				/*  */
			
				if (active_Scene_Ptr->items_Remaining_To_Load != 0) {
					/*  */
					/* Found a potentially good scene to work from */
					/*  */			
					if (active_Scene_Ptr->item_Loading_Last_Index >= active_Scene_Ptr->total_Items_In_Scene) {
						/*  */
						/* No more items left to load. skip to next Scene */
						/* Correction made here. no complaint issued though */
						/*  */
						active_Scene_Ptr->items_Remaining_To_Load = 0;
					
						/*  */
						/* 1994.06.18 JAM add */
						/*  */
						/* ===
							active_Scene_Ptr->item_Loading_Last_Index = 0;
						==== */
					}
					else {
						still_Searching = FALSE;
						puppets_All_Loaded = FALSE;
						current_Scene_Index = index_Used;
					}
				}
				else {
					if (active_Scene_Ptr->backdrop_Needs_Attaching == SAFE_TRUE) {
						if (active_Scene_Ptr->item_Loading_Last_Index >= active_Scene_Ptr->total_Items_In_Scene) {
							still_Searching = FALSE;
							puppets_All_Loaded = TRUE;
							current_Scene_Index = index_Used;
						}
					}
				}
			}
		}
	} /* for(index_Used = 0; ... */


	if (still_Searching != TRUE) {
	
		if (puppets_All_Loaded != TRUE) {
	
			current_Item = active_Scene_Ptr->item_Loading_Last_Index;
			if ((active_Scene_Ptr->item[current_Item].valid_Pointer != SAFE_TRUE)
				&& (active_Scene_Ptr->item[current_Item].desired_File_Loading_Now == SAFE_TRUE)){
	
				/*  */
				/* Puppet not even starting to load yet. */
				/* We need to loacate the source of the Item from data base. */
				/* Fetch filled copy of a FSSpec for the puppet to load */
				/* or at least a pointer to a loaded puppet shared. */
				/*  */																
				if (Locate_Item_Location(k_PUPPET_SELECTION
															, (Str63 *)&(active_Scene_Ptr->item[current_Item].desired_File_Name[0])
															, &file_Specification
															, &item_Address
															, &address_Instead_Of_File) != SAFE_TRUE) {
					/*  */
					/* Goddamn file not available. We need to skip item and bitch */
					/* Preflight should have been done elsewhere. */
					/*  */
					goto Misssing_Item_Error_Handler;
				}

				/*  */
				/* These are used for determining Spinfile caching info */
				/*  */
				active_Scene_Ptr->item[current_Item].last_Image.valid_Information = SAFE_FALSE;
				active_Scene_Ptr->item[current_Item].last_Image.avoidance_Image_Index = k_BOGUS_INDEX_FRAME;

				for (loop = 0; (loop < (k_BUFFERS_REQUIRED - k_BASE_OFFSET_FOR_MIX)); loop++) {
					active_Scene_Ptr->item[current_Item].frame_Needing_Erasure[loop].valid_Information = SAFE_FALSE;
					active_Scene_Ptr->item[current_Item].current_Displayed_Frame[loop].intensity = 0xFFFFFFFF;
					active_Scene_Ptr->item[current_Item].current_Displayed_Frame[loop].horizontal_Angle = 0;
					active_Scene_Ptr->item[current_Item].current_Displayed_Frame[loop].vertical_Angle = 0;
					active_Scene_Ptr->item[current_Item].current_Displayed_Frame[loop].roll_Angle = 0;
				}

				/*  */
				/* These may already have been reset from file load but */
				/* no harm in ensuring it is OK */
				/*  */
				if (address_Instead_Of_File == SAFE_TRUE) {
				
					current_Item_Index = item_Address;
					if (dream.puppet_Memory_Initialized[current_Item_Index] != MEMORY_INITIALIZED) {
						goto Error_Handler;
					}
					
					active_Scene_Ptr->item[current_Item].puppet = *(dream.puppet[current_Item_Index]);			
					active_Scene_Ptr->item[current_Item].valid_Pointer = SAFE_TRUE;
					active_Scene_Ptr->item[current_Item].desired_File_Already_In_Ram = SAFE_TRUE;
					
					active_Scene_Ptr->item[current_Item].desired_File_Loading_Now = SAFE_FALSE;
					active_Scene_Ptr->item[current_Item].desired_File_Loading_Counter = 0;
					
				}
				else {
					/*  */
					/* Create a new puppet, initialize it somewhat and lock it down */
					/* Then set pointers to structure in preparation for final load. */
					/*  */
					if (Create_New_Puppet_Structures(&current_Item_Index) != SAFE_TRUE) {
					
						/*  */
						/* Error should cascade to point where scene deletes itself */
						/* Or puppets not controlled by a scene dispose themselves */
						/* With a re-attempt made */
						/*  */
						goto Error_Handler;
					}
					puppet_Structures_Created = SAFE_TRUE;
					
					if (dream.puppet_Memory_Initialized[current_Item_Index] != MEMORY_INITIALIZED) {
						goto Error_Handler;
					}
					
					active_Scene_Ptr->item[current_Item].puppet = *(dream.puppet[current_Item_Index]);
					active_Scene_Ptr->item[current_Item].valid_Pointer = SAFE_TRUE;
					active_Scene_Ptr->item[current_Item].desired_File_Already_In_Ram = SAFE_FALSE;
					
					active_Scene_Ptr->item[current_Item].desired_File_Loading_Now = SAFE_TRUE;
					active_Scene_Ptr->item[current_Item].desired_File_Loading_Counter = 0;
					
				} /* NOT if (address_Instead_Of_File == SAFE_TRUE) */
				
			}
			else {
				/*  */
				/* Hmmmm...  puppet load in progress already. */
				/*  */
			}
			
			/*  */
			/* Load a portion of a puppet file into the puppet structure. */
			/* Portion to load is based on index counter "desired_File_Loading_Counter" */
			/* TEMP HACK!!!! This loads it in one atomic step instead of several small stages */
			/*  (desired_File_Loading_Counter) */
			/*  */
			if (active_Scene_Ptr->item[current_Item].desired_File_Already_In_Ram != SAFE_TRUE) {
				
				if (Load_Puppet_Sequence(current_Item_Index
														, 0
														, &file_Specification) != SAFE_TRUE ) {
														
					/*  */
					/* Add deletion in future version? */
					/*  */
					
					goto Error_Handler;
				}
				
				/*  */
				/* HACK: Soon make final stage of "Load_Puppet_Sequence" */
				/* trigger setting this. */
				/*  */
				active_Scene_Ptr->item[current_Item].puppet = *(dream.puppet[current_Item_Index]);
				active_Scene_Ptr->item[current_Item].valid_Pointer = SAFE_TRUE;
				active_Scene_Ptr->item[current_Item].desired_File_Already_In_Ram = SAFE_TRUE;
				
			}
			
			active_Scene_Ptr->item[current_Item].desired_File_Loading_Now = SAFE_FALSE;
		
			if (active_Scene_Ptr->item[current_Item].desired_File_Already_In_Ram == SAFE_TRUE) {
	
				if (Force_Scene_Item_To_Mute_Status(&(active_Scene_Ptr->item[current_Item])) != SAFE_TRUE) {
					/*  */
					/* Not important to worry about failures of this command */
					/*  */
				}
			
				if (Add_Puppet_Sequence_To_Scene(current_Scene_Index, current_Item_Index, current_Item) != SAFE_TRUE) {
					/*  */
					/* Ignore error at this stage, and let it tumble out normally? */
					/*  */
					Display_Serious_Error_Message(unit_Name, code_Name, "\pAdd_Puppet_Sequence_To_Scene failed", "\pHandled.");
				}
				
				/*  */
				/* Dual use design of "Add_Puppet_Sequence_To_Scene" requires */
				/* decrement of count for each addition */
				/*  */
			/* ===
				DebugStr("\pAck!! Check total ");
			=== */
				active_Scene_Ptr->total_Items_In_Scene = active_Scene_Ptr->total_Items_In_Scene - 1;
				
				/*  */
				/* Item loaded fully. Increment to next item. Ignore if overrun */
				/*  */
				if (active_Scene_Ptr->items_Remaining_To_Load > 0) {
			 		active_Scene_Ptr->items_Remaining_To_Load = active_Scene_Ptr->items_Remaining_To_Load - 1;
			 	}
			 	active_Scene_Ptr->item_Loading_Last_Index += 1;
		
			} /* if (active_Scene_Ptr->item[current_Item].desired_File_Already_In_Ram ... */
		
		}
		else { 
			/*  */
			/* Kill task, nothing left to safely load */
			/*  */
			dream.play.scenes_Still_Loading = 0;
			
		} /* NOT if (puppets_All_Loaded != TRUE) */
		
		
		if (active_Scene_Ptr->item_Loading_Last_Index >= active_Scene_Ptr->total_Items_In_Scene) {
			/*  */
			/* No more items left to load. Get background now */
			/*  */		
			
			/*  */
			/* This probably should be cached global. Oh well. */
			/*  */
			
			if ((**(dream.play.scene[current_Scene_Index])).backdrop_Needs_Attaching == SAFE_TRUE) {
				pict_Resource_Id = k_DEFAULT_BACKGROUND_RES_ID;
				
				if ((**(dream.play.scene[current_Scene_Index])).backdrop_Selected == SAFE_TRUE) {
				
					pict_Resource_Id = 0;
				
					/*  */
					/* Puppet not even starting to load yet. */
					/* We need to loacate the source of the Item from data base. */
					/* Fetch filled copy of a FSSpec for the puppet to load */
					/* or at least a pointer to a loaded puppet shared. */
					/*  */																
					if (Locate_Item_Location(k_BACKDROP_SELECTION
																, (Str63 *)&((**(dream.play.scene[current_Scene_Index])).backdrop_Name[0])
																, &pict_File_Info
																, &item_Address
																, &address_Instead_Of_File) != SAFE_TRUE) {
						/*  */
						/* Goddamn file not available. We need to skip item and bitch */
						/* Preflight should have been done elsewhere. */
						/*  */
						pict_Resource_Id = k_DEFAULT_BACKGROUND_RES_ID;
		
						/* ===
						goto Misssing_Item_Error_Handler;
						=== */
					}
					else {
						pict_Resource_Id = 0;
					}
					
					if (pict_Resource_Id == 0) {
					
						/*  */
						/* These may already have been reset from file load but */
						/* no harm in ensuring it is OK */
						/*  */
						if (address_Instead_Of_File != SAFE_TRUE) {
							/*  */
							/* Gotta load it. "pict_File_Info" should be active. */
							/*  */					
							params_For_Load_Backdrop.desired_File = pict_File_Info;
							params_For_Load_Backdrop.desired_Backdrop = 0;
							if (Load_Backdrop_File_Immediate_Style(&params_For_Load_Backdrop) != SAFE_TRUE) {
								/*  */
								/* SHIT! Big problem ! */				
								/*  */					
								pict_Resource_Id = k_DEFAULT_BACKGROUND_RES_ID;
							}
							else {
								/*  */
								/* Goodie! Its loaded. Now allow search logic to work as designed. */
								/*  */					
								if (Locate_Item_Location(k_BACKDROP_SELECTION
																			, (Str63 *)&((**(dream.play.scene[current_Scene_Index])).backdrop_Name[0])
																			, &pict_File_Info
																			, &item_Address
																			, &address_Instead_Of_File) != SAFE_TRUE) {
									pict_Resource_Id = k_DEFAULT_BACKGROUND_RES_ID;
					
									/* ===
									goto Misssing_Item_Error_Handler;
									=== */
								}
								else {
									pict_Resource_Id = 0;
								}
							}
						}
					}
				}
				
				/*  */
				/* Cycle through iteratively, add iterative cycling to this level */
				/* later on as well (desired_File_Loading_Counter) */
				/*  */
				current_Stage = 0;
				do {
					result = Add_Background_To_Scene(current_Scene_Index
																				, pict_Resource_Id
																				, &pict_File_Info
																				, item_Address
																				, address_Instead_Of_File
																				, current_Stage
																				, &stages_Finished);
					current_Stage += 1;															
				} while ((result == SAFE_TRUE) && (stages_Finished != TRUE));
	
				/*  */
				/* Whether or not complete mark as finished. */
				/*  */
				(**(dream.play.scene[current_Scene_Index])).backdrop_Needs_Attaching = SAFE_FALSE;
				
			} /* if ((**(dream.play.scene[current_Scene_Index])).backdrop_Remaining_To_Load ... */
			else {
				result = SAFE_TRUE;
			}
			
			/*( result from a few levels above */
	
			if (result == SAFE_TRUE) {
				if ((**(dream.play.scene[current_Scene_Index])).backdrop_Index_Valid == SAFE_TRUE) {
					current_Stage = 0;
					do {
						result = Add_Background_Of_Scene_To_Puppets(current_Scene_Index
																					, current_Stage
																					, &stages_Finished);
						current_Stage += 1;															
					} while ((result == SAFE_TRUE) && (stages_Finished != TRUE));
				}
			}
			
			if (result != SAFE_TRUE) {
				/*  */
				/* Ignore. */
				/*  */
				Display_Serious_Error_Message(unit_Name, code_Name, "\punexpected problem (Add_Background_Of_Scene_To_Puppets failed) ", "\pMemory?");
			}
			
			if (Attach_Default_Inputs_To_Scene_Puppets(current_Scene_Index, dream.play.default_Start_Audio_Point, FALSE) != SAFE_TRUE) {
				Display_Serious_Error_Message(unit_Name, code_Name, "\punexpected problem (Attach_Default_Inputs_To_Scene_Puppets failed)", "\pMemory?");
			}
	
		} /* if (active_Scene_Ptr->item_Loading_Last_Index >= active_Scene_Ptr->total_Items_In_Scene) */
	
	} /* if (still_Searching != TRUE) */
	else {
		dream.play.scenes_Still_Loading = 0;
	}

	function_Result = SAFE_TRUE;
	goto Normal_Function_Exit;
	
Misssing_Item_Error_Handler:

	Display_Serious_Error_Message(unit_Name, code_Name, "\pSubstitute Scene item is needed. File missing", "\pSkipped. Future version allows replace.");
	/*  */
	/* Ignore Item. Increment to next item. Ignore if overrun */
	/*  */
	if (active_Scene_Ptr->items_Remaining_To_Load > 0) {
 		active_Scene_Ptr->items_Remaining_To_Load = active_Scene_Ptr->items_Remaining_To_Load - 1;
 	}
 	active_Scene_Ptr->item_Loading_Last_Index += 1;


Error_Handler:

	function_Result = SAFE_FALSE;

	if (active_Scene_Ptr->initialized == SAFE_TRUE) {
	
		if (puppet_Structures_Created == SAFE_TRUE) {
			if (Delete_A_Loaded_Puppet(current_Item_Index) != SAFE_TRUE) {
				/*  */
				/* Error should cascade to point where scene deletes itself */
				/* Or puppets not controlled by a scene dispose themselves */
				/* With a re-attempt made */
				/*  */
			}
		}

		active_Scene_Ptr->item[current_Item].puppet = nil;
		active_Scene_Ptr->item[current_Item].valid_Pointer = SAFE_FALSE;
		active_Scene_Ptr->item[current_Item].desired_File_Already_In_Ram = SAFE_FALSE;
		active_Scene_Ptr->item[current_Item].desired_File_Loading_Now = SAFE_FALSE;
		active_Scene_Ptr->item[current_Item].desired_File_Loading_Counter = 0;
		
		/*  */
		/* These are used for determining Spinfile caching info */
		/*  */
		active_Scene_Ptr->item[current_Item].last_Image.valid_Information = SAFE_FALSE;
		active_Scene_Ptr->item[current_Item].last_Image.avoidance_Image_Index = k_BOGUS_INDEX_FRAME;
		
		for (loop = 0; (loop < (k_BUFFERS_REQUIRED - k_BASE_OFFSET_FOR_MIX)); loop++) {
			active_Scene_Ptr->item[current_Item].frame_Needing_Erasure[loop].valid_Information = SAFE_FALSE;
			active_Scene_Ptr->item[current_Item].current_Displayed_Frame[loop].intensity = 0xFFFFFFFF;
			active_Scene_Ptr->item[current_Item].current_Displayed_Frame[loop].horizontal_Angle = 0;
			active_Scene_Ptr->item[current_Item].current_Displayed_Frame[loop].vertical_Angle = 0;
			active_Scene_Ptr->item[current_Item].current_Displayed_Frame[loop].roll_Angle = 0;
		}

		/*  */
		/* Ignore Item. Increment to next item. Ignore if overrun */
		/*  */
		if (active_Scene_Ptr->items_Remaining_To_Load > 0) {
	 		active_Scene_Ptr->items_Remaining_To_Load = active_Scene_Ptr->items_Remaining_To_Load - 1;
	 	}
	 	active_Scene_Ptr->item_Loading_Last_Index += 1;
	 	
	} /* if (active_Scene_Ptr->initialized == SAFE_TRUE) */


Normal_Function_Exit:

	return(function_Result);
	
} /* Safe_Boolean Load_Some_Item_Data */
/*=**************************************************************************=*/		

		
/*=**************************************************************************=*/				
Safe_Boolean	Add_Background_Immediately_To_Scene(CARDINAL_32 current_Scene_Index, Str255 name_String)
{
	Safe_Boolean			function_Result;
	Safe_Boolean			backdrop_May_Need_Deletion;
	Str63						previous_Backdrop_Name;
	CARDINAL_32		previous_Backdrop_Index;
	Scene_Ptr				active_Scene_Ptr;
	CARDINAL_8			*code_Name;
/* END VAR */


	code_Name = "\pAdd_Background_Immediately_To_Scene";
	backdrop_May_Need_Deletion = SAFE_FALSE;
	active_Scene_Ptr = *(dream.play.scene[current_Scene_Index]);
	if (active_Scene_Ptr->initialized == SAFE_TRUE) {
	
		if ((**(dream.play.scene[current_Scene_Index])).backdrop_Selected == SAFE_TRUE) {
			/*  */
			/* Kill old scene memory first? */
			/* Naw. Its deleted at end of this */
			/*  */

			if ((**(dream.play.scene[current_Scene_Index])).backdrop_Index_Valid == SAFE_TRUE) {
				backdrop_May_Need_Deletion = SAFE_TRUE;
				previous_Backdrop_Index = (**(dream.play.scene[current_Scene_Index])).backdrop_Index;
				Copy_Pascal_String(&((**(dream.play.scene[current_Scene_Index])).backdrop_Name[0]), previous_Backdrop_Name);
			}
		}
			
		/*  */
		/* Prepare to start */
		/*  */
		(**(dream.play.scene[current_Scene_Index])).backdrop_Selected = SAFE_TRUE;
		(**(dream.play.scene[current_Scene_Index])).backdrop_Needs_Attaching = SAFE_TRUE;
		
		
		if ((**(dream.play.scene[current_Scene_Index])).backdrop_Selected == SAFE_TRUE) {
		
			Copy_Pascal_String(&(name_String[0]), &(**(dream.play.scene[current_Scene_Index])).backdrop_Name[0]);
			active_Scene_Ptr->backdrop_Needs_Attaching = SAFE_TRUE;

			active_Scene_Ptr->items_Remaining_To_Load = 0;
			active_Scene_Ptr->item_Loading_Last_Index = active_Scene_Ptr->total_Items_In_Scene;
			
						
			dream.play.scenes_Still_Loading += 1;
			
			while (dream.play.scenes_Still_Loading > 0) {
			
				if (Load_Some_Item_Data() != SAFE_TRUE) {
					/*  */
					/* Kill task, nothing left to safely load */
					/*  */
					dream.play.scenes_Still_Loading = 0;
				}
			}
			
			if (Redraw_Idle_Screen_Completely(FALSE, FALSE) != SAFE_TRUE) {
				; /* NULL */
			}
		}
	}
	

	if (backdrop_May_Need_Deletion == SAFE_TRUE) {
		/*  */
		/* We must kill the offscreen backdrop. Its not needed now */
		/*  */
		if (((**(dream.play.scene[current_Scene_Index])).backdrop_Selected == SAFE_TRUE)
			&& ((**(dream.play.scene[current_Scene_Index])).backdrop_Index_Valid == SAFE_TRUE)
			&& ((**(dream.play.scene[current_Scene_Index])).backdrop_Needs_Attaching != SAFE_TRUE)) {
			
			/*  */
			/* Its safe to delete old backdrop, if not in other scenes */
			/*  */				
	
			if (Determine_Backdrop_In_Another_Scene(current_Scene_Index, previous_Backdrop_Index) != SAFE_TRUE) {
				if (Destroy_A_Backdrop(previous_Backdrop_Index) != SAFE_TRUE) {
					/*  */
					/* Not important to worry about failures of this command */
					/* But since so rare, we should post warning */
					/*  */				
					Display_Serious_Error_Message(unit_Name, code_Name, "\pDestroy_A_Backdrop fails", "\pSystem memory problems? Bad programming?");
				}
			}
		}
		else {
			/*  */
			/* Re-Attach old backdrop bacause of failure to get new one */
			/*  */				
			(**(dream.play.scene[current_Scene_Index])).backdrop_Selected = SAFE_TRUE;
			(**(dream.play.scene[current_Scene_Index])).backdrop_Index_Valid = SAFE_TRUE;
			(**(dream.play.scene[current_Scene_Index])).backdrop_Needs_Attaching = SAFE_FALSE;
			Copy_Pascal_String(previous_Backdrop_Name,&((**(dream.play.scene[current_Scene_Index])).backdrop_Name[0]));
			(**(dream.play.scene[current_Scene_Index])).backdrop_Index= previous_Backdrop_Index;
		}
	}

	function_Result = SAFE_TRUE;
	return(function_Result);
	
}
/*=**************************************************************************=*/		

		
/*=**************************************************************************=*/				
Safe_Boolean	Remove_Item_Immediately_From_Scene(CARDINAL_32 scene_To_Use, CARDINAL_32 item_To_Remove)
{
	Safe_Boolean			function_Result;
	Scene_Ptr				active_Scene_Ptr;
	INTEGER_16			position;
	SIMPLE_LOOP		loop;
	CARDINAL_32		actual_Puppet_Index;
	CARDINAL_8			*code_Name;
/* END VAR */


	code_Name = "\pRemove_Item_Immediately_From_Scene";
	function_Result = SAFE_FALSE;
	
	active_Scene_Ptr = *(dream.play.scene[scene_To_Use]);
	if (active_Scene_Ptr->initialized != SAFE_TRUE) {
		goto Error_Handler;
	}

	if ((active_Scene_Ptr->item[item_To_Remove].valid_Pointer != SAFE_TRUE)
		|| (active_Scene_Ptr->item[item_To_Remove].puppet == nil)) {
		goto Error_Handler;
	}
	
	if (active_Scene_Ptr->item[item_To_Remove].controllers_Attached == SAFE_TRUE) {
		if (Force_Scene_Item_To_Mute_Status(&(active_Scene_Ptr->item[item_To_Remove])) != SAFE_TRUE) {
			/*  */
			/* Not important to worry about failures of this command */
			/*  */
		}
	}
	
	if (Determine_Item_Needed_Elsewhere(scene_To_Use, item_To_Remove, &actual_Puppet_Index) != SAFE_TRUE) {
		
		/*  */
		/* Double check existence before deletion paranoia */
		/*  */
		if (Find_Puppet_Position_In_Scene(scene_To_Use
												, actual_Puppet_Index
												, &position) == SAFE_TRUE) {
												
			/*  */
			/* Delete puppet, it is not in use by other loaded scenes. */
			/*  */
			
			if (Delete_A_Loaded_Puppet(actual_Puppet_Index) != SAFE_TRUE) {
			
				/*  */
				/* Not important to worry about failures of this command */
				/*  */
				Display_Serious_Error_Message(unit_Name, code_Name, "\pNote: Scene puppet not erased (Delete_A_Loaded_Puppet)", "\pMay cause memory leak");
			}
		}
		else {
			/*  */
			/* Not found because just erased via another instantiation of same item in this scene */
			/*  */
			Display_Serious_Error_Message(unit_Name, code_Name, "\pNote: Scene puppet not erased (Find_Puppet_Position_In_Scene)", "\pMay cause memory leak");
		}
	}
	
	active_Scene_Ptr->item[item_To_Remove].puppet = nil;
	active_Scene_Ptr->item[item_To_Remove].valid_Pointer = SAFE_FALSE;
	active_Scene_Ptr->total_Items_In_Scene = active_Scene_Ptr->total_Items_In_Scene - 1;
	
	/*  */
	/* These are used for determining Spinfile caching info */
	/*  */
	active_Scene_Ptr->item[item_To_Remove].last_Image.valid_Information = SAFE_FALSE;
	active_Scene_Ptr->item[item_To_Remove].last_Image.avoidance_Image_Index = k_BOGUS_INDEX_FRAME;
	
	for (loop = 0; (loop < (k_BUFFERS_REQUIRED - k_BASE_OFFSET_FOR_MIX)); loop++) {
		active_Scene_Ptr->item[item_To_Remove].frame_Needing_Erasure[loop].valid_Information = SAFE_FALSE;
		active_Scene_Ptr->item[item_To_Remove].current_Displayed_Frame[loop].intensity = 0xFFFFFFFF;
		active_Scene_Ptr->item[item_To_Remove].current_Displayed_Frame[loop].horizontal_Angle = 0;
		active_Scene_Ptr->item[item_To_Remove].current_Displayed_Frame[loop].vertical_Angle = 0;
		active_Scene_Ptr->item[item_To_Remove].current_Displayed_Frame[loop].roll_Angle = 0;
	}
	
	if (Compact_Scene_Item_List(scene_To_Use) != SAFE_TRUE) {
			/*  */
			/* Not expected to ever fail. Abort. */
			/*  */
			goto Error_Handler;
	}
	
	/*  */
	/* Only redraw if active scene */
	/*  */
	if (scene_To_Use == dream.current_Scene) {
		if (Redraw_Idle_Screen_Completely(FALSE, FALSE) != SAFE_TRUE) {
			; /* NULL */
		}
	}
	
	function_Result = SAFE_TRUE;
	goto Normal_Function_Exit;
	
Error_Handler:
	function_Result = SAFE_FALSE;

Normal_Function_Exit:
	return(function_Result);
	
} /* Safe_Boolean	Remove_Item_Immediately_From_Scene ... */
/*=**************************************************************************=*/		

		
/*=**************************************************************************=*/				
Safe_Boolean	Add_Item_Immediately_To_Scene(CARDINAL_32 scene_To_Use, Str255 name_String)
{
	Safe_Boolean			function_Result;
	Scene_Ptr				active_Scene_Ptr;
	SIMPLE_LOOP		loop;
	INTEGER_16 			position;
/* END VAR */
	
	function_Result = SAFE_FALSE;

	active_Scene_Ptr = *(dream.play.scene[scene_To_Use]);
	if (active_Scene_Ptr->initialized != SAFE_TRUE) {
		goto Error_Handler;
	}
	
	if (Find_Empty_Position_In_Scene(scene_To_Use
									, &position) != SAFE_TRUE) {
		goto Error_Handler;
	}

/* ====
	Perform_OK_Dialog("\pSorry. This program lacks a way to add an item using this command because not debugged. Use Load Puppet instead. ");
	goto Error_Handler;
===== */
	
	/*  */
	/* Paranoia only */
	/*  */
	active_Scene_Ptr->item[position].valid_Pointer = SAFE_FALSE;
	active_Scene_Ptr->item[position].puppet = nil;
	
	/*  */
	/* These are used for determining Spinfile caching info */
	/*  */
	active_Scene_Ptr->item[position].last_Image.valid_Information = SAFE_FALSE;
	active_Scene_Ptr->item[position].last_Image.avoidance_Image_Index = k_BOGUS_INDEX_FRAME;

	for (loop = 0; (loop < (k_BUFFERS_REQUIRED - k_BASE_OFFSET_FOR_MIX)); loop++) {
		active_Scene_Ptr->item[position].frame_Needing_Erasure[loop].valid_Information = SAFE_FALSE;
		active_Scene_Ptr->item[position].current_Displayed_Frame[loop].intensity = 0xFFFFFFFF;
		active_Scene_Ptr->item[position].current_Displayed_Frame[loop].horizontal_Angle = 0;
		active_Scene_Ptr->item[position].current_Displayed_Frame[loop].vertical_Angle = 0;
		active_Scene_Ptr->item[position].current_Displayed_Frame[loop].roll_Angle = 0;
	}

	if (Force_Scene_Item_To_Mute_Status(&(active_Scene_Ptr->item[position])) != SAFE_TRUE) {
		/*  */
		/* Not important to worry about failures of this command */
		/*  */
	}

	active_Scene_Ptr->item[position].desired_File_Loading_Now = SAFE_TRUE;
	active_Scene_Ptr->item[position].desired_File_Loading_Counter = 0;
	Copy_Pascal_String(&(name_String[0]), &(active_Scene_Ptr->item[position].desired_File_Name[0]));
	
	active_Scene_Ptr->item[position].desired_Start_Location.top = 0;
	active_Scene_Ptr->item[position].desired_Start_Location.left = 0;
	active_Scene_Ptr->item[position].desired_Start_Location.bottom = 1;
	active_Scene_Ptr->item[position].desired_Start_Location.right = 1;

	active_Scene_Ptr->items_Remaining_To_Load += 1;
	active_Scene_Ptr->total_Items_In_Scene += 1;
	active_Scene_Ptr->item_Loading_Last_Index = active_Scene_Ptr->total_Items_In_Scene - 1;
	
	dream.play.scenes_Still_Loading += 1;
	
	while (dream.play.scenes_Still_Loading > 0) {
	
		if (Load_Some_Item_Data() != SAFE_TRUE) {
			/*  */
			/* Kill task, nothing left to safely load */
			/*  */
			dream.play.scenes_Still_Loading = 0;
		}
	}
	
	/*  */
	/* Only redraw if active scene */
	/*  */
	if (scene_To_Use == dream.current_Scene) {
		if (Redraw_Idle_Screen_Completely(FALSE, FALSE) != SAFE_TRUE) {
			; /* NULL */
		}
	}
	
	function_Result = SAFE_TRUE;
	goto Normal_Function_Exit;
	
Error_Handler:
	function_Result = SAFE_FALSE;

Normal_Function_Exit:
	return(function_Result);
	
}
/*=**************************************************************************=*/		

		
/*=**************************************************************************=*/				
Safe_Boolean	Create_New_Scene(Create_Scene_Rec	*params_For_Create_Scene)
{
	CARDINAL_32		item_Address;
	Safe_Boolean			address_Instead_Of_File;
	CARDINAL_32		current_Stage;
	Safe_Boolean			user_Abort;
	CARDINAL_32 		located_Position;
	Safe_Boolean			function_Result;
	Safe_Boolean			scene_Created;
	Safe_Boolean			result;
	Boolean					stages_Finished;
	CARDINAL_8			*code_Name;
	FSSpec					pict_File_Info;
	Scene_Ptr				active_Scene_Ptr;
/* END VAR */


	code_Name = "\pCreate_New_Scene";
	function_Result = SAFE_FALSE;
	scene_Created = SAFE_FALSE;
	
	if (Create_New_Empty_Scene(&located_Position) == SAFE_TRUE) {
	
		/*  */
		/* Setup sizes names, etc. Assumed locked down by design */
		/*  */
		active_Scene_Ptr = *(dream.play.scene[located_Position]);

		active_Scene_Ptr->scene_Background_Rect = params_For_Create_Scene->desired_Scene_Size;
		
		OffsetRect(&(active_Scene_Ptr->scene_Background_Rect)
								,-(active_Scene_Ptr->scene_Background_Rect.left)
								,-(active_Scene_Ptr->scene_Background_Rect.top));
								
		if (active_Scene_Ptr->scene_Background_Rect.right < k_MINIMUM_SCENE_WIDTH) {
			active_Scene_Ptr->scene_Background_Rect.right = k_MINIMUM_SCENE_WIDTH;
		}
			
		if (active_Scene_Ptr->scene_Background_Rect.bottom < k_MINIMUM_SCENE_HEIGHT) {
			active_Scene_Ptr->scene_Background_Rect.bottom = k_MINIMUM_SCENE_HEIGHT;
		}
			
		Copy_Pascal_String(params_For_Create_Scene->scene_Description, active_Scene_Ptr->screen_Name);
		Copy_Pascal_String(params_For_Create_Scene->scene_Short_Name, active_Scene_Ptr->screen_Short_Name);
		Copy_Pascal_String(params_For_Create_Scene->scene_File_Name, active_Scene_Ptr->file_Name);	
	
		/*  */
		/* We need to create offscreen scene graphics port and stuff now? */
		/*  */
	
		scene_Created = SAFE_TRUE;
		
		address_Instead_Of_File = SAFE_FALSE;
		item_Address = 0;
		
		/*  */
		/* Cycle through iteratively, add iterative cycling to this level */
		/* later on as well */
		/*  */
		current_Stage = 0;
		do {
			/*  */
			/* Only the backdrops based on actual file references can */
			/* utilize "address_Instead_Of_File" or "item_Address" */
			/* and only when the second parameter is "0" */
			/*  */
			result = Add_Background_To_Scene(located_Position
																		, k_DEFAULT_BACKGROUND_RES_ID
																		, &pict_File_Info
																		, item_Address
																		, address_Instead_Of_File
																		, current_Stage
																		, &stages_Finished);
			current_Stage += 1;															
		} while ((result == SAFE_TRUE) && (stages_Finished != TRUE));
		
		if (result == SAFE_TRUE) {
			current_Stage = 0;
			do {
				result = Add_Background_Of_Scene_To_Puppets(located_Position
																			, current_Stage
																			, &stages_Finished);
				current_Stage += 1;															
			} while ((result == SAFE_TRUE) && (stages_Finished != TRUE));
		}
		
		if (result != SAFE_TRUE) {
			/*  */
			/* Ignore. */
			/*  */
			Display_Serious_Error_Message(unit_Name, code_Name, "\punexpected problem", "\pMemory?");
			goto Error_Handler;
		}
				
		if (Select_Scene_And_Display_First_Time(located_Position) != SAFE_TRUE) {
			/*  */
			/* Ignore problem but set selected scene to innocent port? */
			/*  */
		}
	}
	else {
		goto No_More_Scenes_Error_Handler;
	}

	/*  */
	/* Something was loaded, enable all items */
	/*  */
	if (Reset_Menu_Control_Selectability() != SAFE_TRUE) {
		goto Error_Handler;
	}

	function_Result = SAFE_TRUE;
	goto Normal_Function_Exit;
	
No_More_Scenes_Error_Handler:
	Display_Serious_Error_Message(unit_Name, code_Name, "\pNo more scenes can be added", "\pRemove some");

Error_Handler:
	function_Result = SAFE_FALSE;

Normal_Function_Exit:
	if (function_Result != SAFE_TRUE) {
		if (scene_Created == SAFE_TRUE) {
			if (Destroy_A_Scene(located_Position) != SAFE_TRUE) {
				/*  */
				/* Not a big problem. We are quitting anyway. */
				/*  */
				Display_Serious_Error_Message(unit_Name, code_Name, "\pDestroy_A_Scene failed", "\pCorruption?");
			}
		}
	}

	return(function_Result);
	
} /* Safe_Boolean	Create_New_Scene */
/*=**************************************************************************=*/		

		
/*=**************************************************************************=*/				
Safe_Boolean	Load_Scene_File_Immediate_Style(Open_Scene_Rec	*params_For_Open_Scene)
{
	Safe_Boolean			user_Abort;
	Boolean					problem_In_Loading;
	CARDINAL_32 		located_Position;
	Safe_Boolean			function_Result;
	Close_Scenes_Rec	params_For_Close_Scenes;
	CARDINAL_8			*code_Name;
/* END VAR */


	code_Name = "\pLoad_Scene_File_Immediate_Style";
	function_Result = SAFE_FALSE;
	
	if (Create_New_Empty_Scene(&located_Position) == SAFE_TRUE) {
	
		params_For_Open_Scene->desired_Scene = located_Position;
	
		if (Load_Scene_File(params_For_Open_Scene) != SAFE_TRUE) {
		
			if (Destroy_A_Scene(located_Position) != SAFE_TRUE) {
				/*  */
				/* Not a big problem. We are quitting anyway. */
				/*  */
			}
			
			goto Error_Handler;
		}
		else {
			dream.play.scenes_Still_Loading += 1;
			
			/*  */
			/* Cycle through loading the Items in the scene. Normally done in stages */
			/* in the event loop */
			/*  */
			problem_In_Loading = FALSE;
			while (dream.play.scenes_Still_Loading > 0) {
			
				if (Load_Some_Item_Data() != SAFE_TRUE) {
					/*  */
					/* Kill task, nothing left to safely load */
					/*  */
					dream.play.scenes_Still_Loading = 0;
					problem_In_Loading = TRUE;
				}
			}
			
			if (problem_In_Loading == TRUE) {
				params_For_Close_Scenes.desired_Scene = params_For_Open_Scene->desired_Scene;
				if (Close_Scenes(&params_For_Close_Scenes) != SAFE_TRUE) {
					/*  */
					/*  */
				}
			}
			else {
			
				/*  */
				/* If first one, then display */
				/*  */
				if (dream.play.total_Scenes == 1) {
				
					if (Select_Scene_And_Display_First_Time(located_Position) != SAFE_TRUE) {
						/*  */
						/* Ignore problem but set selected scene to innocent port? */
						/*  */
					}
					
				}
			}
		
		} /* NOT if (Load_Scene_File(located_Position, ... */
	}
	else {
		goto No_More_Scenes_Error_Handler;
	}

	function_Result = SAFE_TRUE;
	goto Normal_Function_Exit;
	
No_More_Scenes_Error_Handler:
	Display_Serious_Error_Message(unit_Name, code_Name, "\pNo more scenes can be added", "\pRemove some");

Error_Handler:
	function_Result = SAFE_FALSE;

Normal_Function_Exit:

	return(function_Result);
}
/*=**************************************************************************=*/
	
	
/*=**************************************************************************=*/				
Safe_Boolean	Specify_Scene_To_Load(FSSpec		*file_Specification
															, Safe_Boolean *user_Abort)
{
	INTEGER_16 		number_Of_Types;
	Boolean				user_Clicked_Ok;
	INTEGER_32		source_Directory_ID;
	INTEGER_16		source_Volume_Reference;
	SFTypeList 			type_List;
	SFReply				sf_Reply;
	Safe_Boolean		function_Result;
/* END VAR */


	function_Result = SAFE_FALSE;
	*user_Abort = SAFE_FALSE;

	number_Of_Types = 1;
	type_List[0] = k_SCENE_FILE_TYPE;

	user_Clicked_Ok = Perform_Get_File_Dialog(number_Of_Types
										, &(type_List)
										, &sf_Reply);
											
	if ((user_Clicked_Ok != TRUE) || (sf_Reply.good != TRUE)) {
		/*  */
		/* User Abort, no file name */
		/*  */
		*user_Abort = SAFE_TRUE;
		function_Result = SAFE_FALSE;
		goto Normal_Function_Exit;
	}
	
	Get_Last_Volume_Reference_And_Directory_ID(&source_Volume_Reference, &source_Directory_ID);

	file_Specification->vRefNum = source_Volume_Reference;
	file_Specification->parID = source_Directory_ID;
	Copy_Pascal_String(sf_Reply.fName, file_Specification->name);
		
	function_Result = SAFE_TRUE;
	goto Normal_Function_Exit;

Error_Handler:
	function_Result = SAFE_FALSE;
		
Normal_Function_Exit:
	return(function_Result);
	
} /* Safe_Boolean	Specify_Scene_To_Load... */
/*=**************************************************************************=*/
	
	
/*=**************************************************************************=*/				
Safe_Boolean Determine_If_Scene_Loadable(Scene_Ptr active_Scene_Ptr, Str255 *missing_Item)
{
	Safe_Boolean		function_Result;
	SIMPLE_LOOP	loop;
	FSSpec				pict_File_Info;
	CARDINAL_32	item_Address;
	Safe_Boolean		address_Instead_Of_File;
/* END VAR */


	function_Result = SAFE_TRUE;
	(*missing_Item)[0] = 0;
	if (active_Scene_Ptr->backdrop_Selected == SAFE_TRUE) {
		
		/*  */
		/* Puppet not even starting to load yet. */
		/* We need to loacate the source of the Item from data base. */
		/* Fetch filled copy of a FSSpec for the puppet to load */
		/* or at least a pointer to a loaded puppet shared. */
		/*  */																
		if (Locate_Item_Location(k_BACKDROP_SELECTION
													, (Str63 *)&(active_Scene_Ptr->backdrop_Name[0])
													, &pict_File_Info
													, &item_Address
													, &address_Instead_Of_File) != SAFE_TRUE) {
			/*  */
			/* Goddamn file not available. We need to skip item and bitch */
			/*  */
			Copy_Pascal_String(&(active_Scene_Ptr->backdrop_Name[0]), (*missing_Item));
			function_Result = SAFE_FALSE;
			goto Misssing_Item_Error_Handler;
		}
	}

	if (function_Result == SAFE_TRUE) {
		if (active_Scene_Ptr->total_Items_In_Scene > 0) {

			for (loop = 0; (loop < active_Scene_Ptr->total_Items_In_Scene) ; loop++) {
							
				if (Locate_Item_Location(k_PUPPET_SELECTION
															, (Str63 *)&(active_Scene_Ptr->item[loop].desired_File_Name[0])
															, &pict_File_Info
															, &item_Address
															, &address_Instead_Of_File) != SAFE_TRUE) {
															
					Copy_Pascal_String(&(active_Scene_Ptr->item[loop].desired_File_Name[0]), (*missing_Item));
					/*  */
					/* Goddamn file not available. We need to skip item and bitch */
					/*  */
					function_Result = SAFE_FALSE;
					goto Misssing_Item_Error_Handler;
				}
			}
		}
	}

	
	function_Result = SAFE_TRUE;
	goto Normal_Function_Exit;

Misssing_Item_Error_Handler:

Error_Handler:
	function_Result = SAFE_FALSE;
		
Normal_Function_Exit:
	return(function_Result);
	
}
/*=**************************************************************************=*/
	
	
/*=**************************************************************************=*/				
Safe_Boolean	Load_Scene_File(Open_Scene_Rec	*params_For_Open_Scene)
{

	Scene_File_Header_Ptr				header;
	SignedByte 								header_Original_State;
	Scene_File_Header_Hdl				scene_Header_Hdl;
	Memory_Allocation					scene_Header_Memory_In_Use;
	
	Scene_File_Hdl							scene_Body_Hdl;
	Memory_Allocation					scene_Body_Memory_In_Use;
	Scene_File_Ptr							body;
	SignedByte 								body_Original_State;
	
	Item_File_Reference_Ptr			item;
	Item_File_Reference_Hdl			item_Resource_Hdl;
	Memory_Allocation					item_Resource_Hdl_Memory_Initialized;
	
	CTabHandle								temp_Clut_Handle;
	Resource_Identifier					*first_Item_Ptr;
	Safe_Boolean 							*end_Sentinel_Ptr;
	Off_Screen_Ptr 							offscreen_Ptr;
	Rect												offscreen_Rect;
	CARDINAL_16							desired_Bit_Depth;
	CARDINAL_32	total_Items_Desired;
	INTEGER_32		transfer_Count;
	INTEGER_32		transfer_Count_Copy;
	WindowPtr			current_Graphics_Window_Ptr;
	Safe_Boolean		function_Result;
	Safe_Boolean		resource_Sources_Swapped;
	CARDINAL_32	memory_Needed;
	INTEGER_16		save_Ref;
	INTEGER_16		item_To_Fetch;
	
	INTEGER_16		input_File_Reference;
	SIMPLE_LOOP	index;
	OSErr					os_Error;
	INTEGER_32		total_Bytes_Contiguous_Now;
	INTEGER_32		total_Bytes_Now;
	INTEGER_32 		total_Bytes_Contiguous_Later;
	INTEGER_32		total_Bytes_Later;
	CTabHandle		temp_Clut_Table;
	Scene_Ptr			active_Scene_Ptr;
	INTEGER_32		loop;
	INTEGER_16		total_Puppet_Resources;
	INTEGER_16		current_Pict_Sequence;
	Boolean				still_Searching;
	CARDINAL_8		*code_Name;
	Str255					message_String;
	Str255					note_String;
	Str63					item_Missing_Name;
/* END VAR */


	code_Name = "\pLoad_Scene_File";
	function_Result = SAFE_FALSE;
		
	scene_Header_Memory_In_Use = MEMORY_NOT_INITIALIZED;
	scene_Body_Memory_In_Use = MEMORY_NOT_INITIALIZED;
	item_Resource_Hdl_Memory_Initialized = MEMORY_NOT_INITIALIZED;
		
	resource_Sources_Swapped = SAFE_FALSE;
				
										
	/*  */
	/* Open for read only permission */
	/*  */
	
	save_Ref = CurResFile();
	input_File_Reference = HOpenResFile (params_For_Open_Scene->desired_File.vRefNum
									, params_For_Open_Scene->desired_File.parID
									, params_For_Open_Scene->desired_File.name
									, fsRdPerm );
						
	if ((input_File_Reference == -1) || (ResError() != noErr)) {
		goto Error_Handler;
	}
													
	UseResFile(input_File_Reference);
	os_Error = ResError();
	if (os_Error != noErr) {
		Display_Serious_Error_Message(unit_Name, code_Name, "\pInternal using resource", "\pCorrupted?");
		goto Error_Handler;
	}
	
	resource_Sources_Swapped = SAFE_TRUE;
	
	/*  */
	/* Load images into memory, allocating a graphics port */
	/* big enough to hold the images into one tall graphics device port */
	/*  */
		
	total_Puppet_Resources = Count1Resources(k_SCENE_MAIN_RESOURCE_TYPE);
	if (ResError() != noErr) {
		goto Error_Handler;
	}
	
	if (total_Puppet_Resources < 1) {
		Get_NS_String(k_DREAM_STRINGS, k_SORRY_WRONG_FILE_INDEX, message_String);
		Perform_OK_Dialog(message_String);
		goto Error_Handler;
	}	
		
	if (Determine_Absolute_Free_Memory_Hook(&total_Bytes_Contiguous_Now
													,&total_Bytes_Now
													,&total_Bytes_Contiguous_Later
													,&total_Bytes_Later) != TRUE) {
		goto Error_Handler;
	}
		
	if ((dream.play.scene_Memory_In_Use[params_For_Open_Scene->desired_Scene]) != SAFE_TRUE) {
		function_Result = SAFE_FALSE;
		goto Normal_Function_Exit;
	}
	
	
	/*  */
	/* active_Scene_Ptr uses a handle assumed to always be locked */
	/*  */
	active_Scene_Ptr = *(dream.play.scene[params_For_Open_Scene->desired_Scene]);
	if (active_Scene_Ptr->total_Items_In_Scene != 0 ) {
		Copy_Pascal_String("\pUNIMPLEMENTED: This version cannot append into a scene this way", message_String);
		Perform_OK_Dialog(message_String);
		goto Error_Handler;
	}
	
	/*  */
	/* These probably were already set but are vital to have set for safety. */
	/*  */
	active_Scene_Ptr->total_Items_In_Scene = 0;
	active_Scene_Ptr->scene_Port.initialized = SAFE_FALSE;
	active_Scene_Ptr->backdrop_Selected = SAFE_FALSE;

	scene_Header_Hdl = (Scene_File_Header_Hdl)Get1Resource(k_SCENE_MAIN_RESOURCE_TYPE
																			, k_BASE_SCENE_RESOURCE_ID);
	if ((ResError() != noErr) || (scene_Header_Hdl == nil)) {
		goto Error_Handler;
	}
	scene_Header_Memory_In_Use = MEMORY_INITIALIZED;
	HNoPurge((Handle)scene_Header_Hdl);
	if (MemError() != noErr) {
		/*  */
		/* This cannot fail, but check anyway */
		/*  */
		goto Error_Handler;
	}
	
	

	scene_Body_Hdl = (Scene_File_Hdl)Get1Resource(k_SCENE_EXPANDABLE_RESOURCE_TYPE
																			, k_BASE_SCENE_RESOURCE_ID);
	if ((ResError() != noErr) || (scene_Body_Hdl == nil)) {
		goto Error_Handler;
	}
	scene_Body_Memory_In_Use = MEMORY_INITIALIZED;
	HNoPurge((Handle)scene_Body_Hdl);
	if (MemError() != noErr) {
		/*  */
		/* This cannot fail, but check anyway */
		/*  */
		goto Error_Handler;
	}
	
	if (((**scene_Header_Hdl).valid != SAFE_TRUE) || ((**scene_Body_Hdl).valid != SAFE_TRUE)) {
		Copy_Pascal_String("\pThis file has corrupted resources and cannot be used safely", message_String);
		Perform_OK_Dialog(message_String);
		goto Error_Handler;
	}
	
	if (Begin_Use_Of_Handle((Handle)scene_Header_Hdl
									, MOVE_HIGH_NOT_REQUIRED
									, &header_Original_State) != SAFE_TRUE) {
		goto Error_Handler;
	}
	
	header = *scene_Header_Hdl;
	
	
	
	Copy_Pascal_String(header->screen_Name, active_Scene_Ptr->screen_Name);
	Copy_Pascal_String(header->screen_Short_Name, active_Scene_Ptr->screen_Short_Name);
	Copy_Pascal_String(header->file_Name, active_Scene_Ptr->file_Name);

	if ((header->backward_Compatible_Version & 0xFFFF0000) > (k_SCENE_VERSION_NOW & 0xFFFF0000)) {
		Copy_Pascal_String("\pThis file is too new to be used safely", message_String);
		Perform_OK_Dialog(message_String);
		goto Error_Handler;
	}
	
	if ((header->backward_Compatible_Version & 0xFFFF0000) < (k_SCENE_BASE_VERSION_NOW & 0xFFFF0000)) {
		Copy_Pascal_String("\pThis file is too old to be used safely", message_String);
		Perform_OK_Dialog(message_String);
		goto Error_Handler;
	}

	active_Scene_Ptr->total_Special_Colors_To_Honor = header->total_Special_Colors_To_Honor;
	active_Scene_Ptr->total_Colors_Used_In_Pallette = header->total_Colors_Used_In_Pallette;
	
	offscreen_Rect = header->scene_Size_Rect;
	desired_Bit_Depth = header->stored_Bit_Depth;
	
	if (header->special_Pallette_ID != 0) {
		active_Scene_Ptr->scene_Special_Pallete_To_Use = (PaletteHandle)GetNewPalette(header->special_Pallette_ID);
		if (active_Scene_Ptr->scene_Special_Pallete_To_Use == nil) {
			Copy_Pascal_String("\pThis file is corrupted (no pallette)", message_String);
			Perform_OK_Dialog(message_String);
			goto Error_Handler;
		}
		
		HNoPurge((Handle)active_Scene_Ptr->scene_Special_Pallete_To_Use);
		os_Error = MemError();
		if (os_Error != noErr) {
			/*  */
			/* This cannot fail */
			/*  */
			goto Error_Handler;
		}
		
		MoveHHi((Handle)active_Scene_Ptr->scene_Special_Pallete_To_Use);
		os_Error = MemError();
		if ((os_Error != noErr) && (os_Error != -117)){
			/*  */
			/* This cannot fail, but check anyway */
			/*  */
			goto Error_Handler;
		}
		
		HLock((Handle)active_Scene_Ptr->scene_Special_Pallete_To_Use);
		if (MemError() != noErr) {
			/*  */
			/* This cannot fail, but check anyway */
			/*  */
			goto Error_Handler;
		}
		
	}
	else {
		active_Scene_Ptr->scene_Special_Pallete_To_Use = nil;
	}
	
	/*  */
	/* GetCTable is now used !!! 1994.05.22 JAM !!  */
	/*  */
	if (header->current_Clut_ID != 0) {
	
		temp_Clut_Handle = GetCTable(header->current_Clut_ID);
		if (temp_Clut_Handle == nil) {
				Display_Serious_Error_Message(unit_Name
																	, code_Name
																	, "\pProblem Loading System CLUT (0.1)"
																	, "\pCorrupted?");
			goto Error_Handler;
		}	
	
		HLock((Handle)temp_Clut_Handle);
		if (MemError() != noErr) {
				Display_Serious_Error_Message(unit_Name
																	, code_Name
																	, "\pProblem locking handle (0.2)"
																	, "\pCorrupted!");
			goto Error_Handler;
		}
	
#if (7 == 11)
		temp_Clut_Handle = (CTabHandle)Get1Resource(
																				k_SCENE_CLUT_RESOURCE_TYPE
																				, header->current_Clut_ID);
		if ((ResError() != noErr) || (temp_Clut_Handle == nil)) {
			Copy_Pascal_String("\pThis file is corrupted (no Color Table)", message_String);
			Perform_OK_Dialog(message_String);
			goto Error_Handler;
		}
		
		DetachResource((Handle)temp_Clut_Handle);
		if (ResError() != noErr) {
			/*  */
			/* This cannot fail, but check anyway */
			/*  */
			goto Error_Handler;
		}
		
		HNoPurge((Handle)temp_Clut_Handle);
		if (MemError() != noErr) {
			/*  */
			/* This cannot fail, but check anyway */
			/*  */
			goto Error_Handler;
		}
#endif /* (7 == 11) */
		
		
	}
	else {
		temp_Clut_Handle = nil;
	}

	active_Scene_Ptr->total_Items_In_Scene = header->file_Item_Total;
	
	if (Begin_Use_Of_Handle((Handle)scene_Body_Hdl
									, MOVE_HIGH_NOT_REQUIRED
									, &body_Original_State) != SAFE_TRUE) {
		goto Error_Handler;
	}
		
	body = (Scene_File_Ptr)*scene_Body_Hdl;
	
	if (active_Scene_Ptr->total_Items_In_Scene != body->total_Items_In_Scene){ 
		goto Error_Handler;
	}
	
	if (active_Scene_Ptr->total_Items_In_Scene > k_MAX_STAGE_ITEMS){ 
		goto Error_Handler;
	}
	
	first_Item_Ptr = &(body->file_Scene_Items[0]);
	
	end_Sentinel_Ptr = (Safe_Boolean *)((CARDINAL_32)first_Item_Ptr 
											+ ((CARDINAL_32)sizeof(INTEGER_16) 
												* (CARDINAL_32)body->resource_Template_Count_1));
	
	if (*end_Sentinel_Ptr != SAFE_TRUE) {
		Copy_Pascal_String("\pThis file is corrupted, or program is designed wrong (bad end sentinel)", message_String);
		Perform_OK_Dialog(message_String);
		goto Error_Handler;
	}
	
	if (active_Scene_Ptr->total_Items_In_Scene > body->resource_Template_Count_1) {
		goto Error_Handler;
	}
	
	total_Items_Desired = 0;
	for (loop = 0; (loop < active_Scene_Ptr->total_Items_In_Scene) ; loop++) {
	
		item_To_Fetch = *(Resource_Identifier *)((CARDINAL_32)first_Item_Ptr 
																					+ ((CARDINAL_32)sizeof(Resource_Identifier) * (CARDINAL_32)loop));
		
		item_Resource_Hdl = nil;
		item_Resource_Hdl = (Item_File_Reference_Hdl)Get1Resource(
																					k_SCENE_ITEM_RESOURCE_TYPE
																					, item_To_Fetch);
		if ((ResError() != noErr) || (item_Resource_Hdl== nil)) {
			Copy_Pascal_String("\pThis file is corrupted (missing item ref)", message_String);
			Perform_OK_Dialog(message_String);
			goto Error_Handler;
		}
		
		HNoPurge((Handle)item_Resource_Hdl);
		if (MemError() != noErr) {
			/*  */
			/* This cannot fail, but check anyway */
			/*  */
			goto Error_Handler;
		}

		/*  */
		/* Loading pupet information here */
		/*  */
		if ((**item_Resource_Hdl).valid == SAFE_TRUE) {
			if (active_Scene_Ptr->item[total_Items_Desired].valid_Pointer == SAFE_TRUE) {
				Display_Serious_Error_Message(unit_Name, code_Name, "\pInternal problem getting Scene (dup)", "\p");
			}
			
			active_Scene_Ptr->item[total_Items_Desired].valid_Pointer = SAFE_FALSE;
			active_Scene_Ptr->item[total_Items_Desired].puppet = nil;

			/*  */
			/* These are used for determining Spinfile caching info */
			/*  */
			active_Scene_Ptr->item[total_Items_Desired].last_Image.valid_Information = SAFE_FALSE;
			active_Scene_Ptr->item[total_Items_Desired].last_Image.avoidance_Image_Index = k_BOGUS_INDEX_FRAME;
		
			for (index = 0; (index < (k_BUFFERS_REQUIRED - k_BASE_OFFSET_FOR_MIX)); index++) {
				active_Scene_Ptr->item[total_Items_Desired].frame_Needing_Erasure[index].valid_Information = SAFE_FALSE;
				active_Scene_Ptr->item[total_Items_Desired].current_Displayed_Frame[index].intensity = 0xFFFFFFFF;
				active_Scene_Ptr->item[total_Items_Desired].current_Displayed_Frame[index].horizontal_Angle = 0;
				active_Scene_Ptr->item[total_Items_Desired].current_Displayed_Frame[index].vertical_Angle = 0;
				active_Scene_Ptr->item[total_Items_Desired].current_Displayed_Frame[index].roll_Angle = 0;
			}

			if (Force_Scene_Item_To_Mute_Status(&(active_Scene_Ptr->item[total_Items_Desired])) != SAFE_TRUE) {
				/*  */
				/* Not important to worry about failures of this command */
				/*  */
			}
			
			active_Scene_Ptr->item[total_Items_Desired].desired_File_Loading_Now = SAFE_TRUE;
			active_Scene_Ptr->item[total_Items_Desired].desired_File_Loading_Counter = 0;
			Copy_Pascal_String(&((**item_Resource_Hdl).item_Name[0]), &(active_Scene_Ptr->item[total_Items_Desired].desired_File_Name[0]));
			active_Scene_Ptr->item[total_Items_Desired].desired_Start_Location = (**item_Resource_Hdl).scene_Position;
			
			total_Items_Desired += 1;
		}

		ReleaseResource((Handle)item_Resource_Hdl);
		if (ResError() != noErr) {
			/*  */
			/* This cannot fail, but check anyway */
			/*  */
			goto Error_Handler;
		}

	} /* for (loop = 0; (loop < active_Scene_Ptr->total_Items_In_Scene) ... */
	
	if (active_Scene_Ptr->total_Items_In_Scene != total_Items_Desired) {
		Copy_Pascal_String("\pThis scene contained more items than expected. Merely odd", message_String);
		Perform_OK_Dialog(message_String);
	}
	
	active_Scene_Ptr->total_Items_In_Scene = total_Items_Desired;
	active_Scene_Ptr->items_Remaining_To_Load = active_Scene_Ptr->total_Items_In_Scene;
	
	/* 1994.06.19 JAM alterred from "params_For_Open_Scene->desired_Scene" */
	
	active_Scene_Ptr->item_Loading_Last_Index = 0;
	
	
	active_Scene_Ptr->backdrop_Selected = header->backdrop_Selected;
	if (header->backdrop_Selected == SAFE_TRUE) {
		Copy_Pascal_String(header->backdrop_Name, active_Scene_Ptr->backdrop_Name);
		active_Scene_Ptr->backdrop_Needs_Attaching = SAFE_TRUE;
	}
	
	if (Determine_If_Scene_Loadable(active_Scene_Ptr, &item_Missing_Name) != SAFE_TRUE) {
		Concat_Many(message_String, 3, "\pThis scene is unable to locate a required file : ", item_Missing_Name, "\pLoad Anyway? (Not reccommended)");
		if (Perform_Boolean_Dialog(message_String) != TRUE) {
			goto Error_Handler_Without_Message;
		}
	}
	
	/*  */
	/* Now create the offscreen picture!  */
	/*  */
	
	 if (active_Scene_Ptr->scene_Port.initialized == SAFE_TRUE) {
	 	if (Close_Offscreen_Pict_Port(&(active_Scene_Ptr->scene_Port)) != SAFE_TRUE) {
			/*  */
			/* might as well complain about it. */
			/*  */
	 		goto Error_Handler;
	 	}
	 	
	 	if (active_Scene_Ptr->scene_Special_Pallete_To_Use != nil) {
	 		DisposePalette(active_Scene_Ptr->scene_Special_Pallete_To_Use);
	 	}
	 }

	if (Determine_Absolute_Free_Memory_Hook(&total_Bytes_Contiguous_Now
													,&total_Bytes_Now
													,&total_Bytes_Contiguous_Later
													,&total_Bytes_Later) != TRUE) {
		goto Error_Handler;
	}
	

	offscreen_Ptr = &(active_Scene_Ptr->scene_Port);
	offscreen_Ptr->initialized = SAFE_FALSE;
	offscreen_Ptr->color_Type = k_COLOR_QUICKDRAW;
	offscreen_Ptr->color_Table_Handle = nil;
	offscreen_Ptr->offscreen_GDevice_Hdl = nil;
	offscreen_Ptr->image_Handle = nil;

	memory_Needed = desired_Bit_Depth;
	memory_Needed = memory_Needed * (CARDINAL_32)(offscreen_Rect.right - offscreen_Rect.left);
	memory_Needed = memory_Needed * (CARDINAL_32)(offscreen_Rect.bottom - offscreen_Rect.top);
	memory_Needed = memory_Needed >> 3;
	
	if (total_Bytes_Contiguous_Later < (k_MINIMUM_RAM_TO_PRESERVE_FOR_BIG_STUFF + memory_Needed)) {
		total_Bytes_Contiguous_Later = MaxMem(&total_Bytes_Now);
		if (total_Bytes_Contiguous_Later < (k_MINIMUM_RAM_TO_PRESERVE_FOR_BIG_STUFF + memory_Needed)) {
			total_Bytes_Contiguous_Later = MaxMem(&total_Bytes_Now);
	
			Get_NS_String(k_DREAM_ERROR_STRINGS, k_ERROR_NOT_ENOUGH_MEMORY_TO_SAFELY_LOAD, note_String);
			Make_String_Long_String(&message_String[0], &note_String[0], memory_Needed, NO_LIMIT, "\p bytes.");
			Perform_OK_Dialog(message_String);
			goto Error_Handler_Without_Message;
		}
	}
	
	/*  */
	/* Paranoia debugging only */
	/*  */
	offscreen_Ptr->color_Table_Handle = temp_Clut_Handle;
	temp_Clut_Handle = nil;
	MoveHHi((Handle)offscreen_Ptr->color_Table_Handle);
	HLock((Handle)offscreen_Ptr->color_Table_Handle);


	os_Error= Create_Off_Screen_Port(&offscreen_Rect
															, desired_Bit_Depth
															, offscreen_Ptr->color_Table_Handle
															, (CGrafPtr)&(offscreen_Ptr->offscreen_Port.color_Port)
															, &(offscreen_Ptr->offscreen_GDevice_Hdl)
															, &(offscreen_Ptr->image_Handle));
	
	if (os_Error != noErr) {
		Copy_Pascal_String("\pA graphics port for this puppet file failed to be created (memory?)", message_String);
		Perform_OK_Dialog(message_String);
		goto Error_Handler;
	}
	
	/*  */
	/* Redesign to avoid locked crap one day */
	/*  */
	
	offscreen_Ptr->initialized =  SAFE_TRUE; 
	offscreen_Ptr->color_Type = k_COLOR_QUICKDRAW;
	Lock_Video_Memory_For_Lazy_But_Safe_Use(offscreen_Ptr);
				
	if (scene_Header_Memory_In_Use == MEMORY_INITIALIZED) {
		ReleaseResource((Handle)scene_Header_Hdl);
		if (ResError() != noErr) {
			/*  */
			/* Ignore error, we are exiting anyway  */
			/*  */
		}
		scene_Header_Memory_In_Use = MEMORY_NOT_INITIALIZED;
	}
	
	if (scene_Body_Memory_In_Use == MEMORY_INITIALIZED) {
		ReleaseResource((Handle)scene_Body_Hdl);
		if (ResError() != noErr) {
			/*  */
			/* Ignore error, we are exiting anyway  */
			/*  */
		}
		scene_Body_Memory_In_Use = MEMORY_NOT_INITIALIZED;
	}
	
	if (resource_Sources_Swapped == SAFE_TRUE) {
		CloseResFile(input_File_Reference);
		if (ResError() != noErr) {
			goto Error_Handler;
		}
		
		UseResFile(save_Ref);
		if (ResError() != noErr) {
			/*  */
			/* This should be recoverable from, enhance one day  */
			/*  */
		}
		resource_Sources_Swapped = SAFE_FALSE;
	}

	/*  */
	/* Something was loaded, enable all items */
	/*  */
	if (Reset_Menu_Control_Selectability() != SAFE_TRUE) {
		goto Error_Handler;
	}

		
	function_Result = SAFE_TRUE;
	goto Normal_Function_Exit;
	
Error_Handler_With_Message:
	function_Result = SAFE_FALSE;
	Display_Serious_Error_Message(unit_Name, code_Name, note_String, "\p");
	goto Normal_Function_Exit;
	
Error_Handler_Without_Message:
	function_Result = SAFE_FALSE;
	goto Normal_Function_Exit;

Error_Handler:
	function_Result = SAFE_FALSE;
	Display_Serious_Error_Message(unit_Name, code_Name, "\pInternal problem getting Scene", "\p");
			
Normal_Function_Exit:

	if (function_Result != SAFE_TRUE) {
		if (Destroy_A_Scene(params_For_Open_Scene->desired_Scene) != SAFE_TRUE) {
			/*  */
			/* Ignore scene deletion. Not a critical problem */
			/*  */
		}
	}
	
	if (scene_Header_Memory_In_Use == MEMORY_INITIALIZED) {
		ReleaseResource((Handle)scene_Header_Hdl);
		if (ResError() != noErr) {
			/*  */
			/* Ignore error, we are exiting anyway  */
			/*  */
		}
		scene_Header_Memory_In_Use = MEMORY_NOT_INITIALIZED;
	}
	
	if (scene_Body_Memory_In_Use == MEMORY_INITIALIZED) {
		ReleaseResource((Handle)scene_Body_Hdl);
		if (ResError() != noErr) {
			/*  */
			/* Ignore error, we are exiting anyway  */
			/*  */
		}
		scene_Body_Memory_In_Use = MEMORY_NOT_INITIALIZED;
	}
	
	if (resource_Sources_Swapped == SAFE_TRUE) {
		CloseResFile(input_File_Reference);
		if (ResError() != noErr) {
			goto Error_Handler;
		}
		
		UseResFile(save_Ref);
		if (ResError() != noErr) {
			/*  */
			/* This should be recoverable from, enhance one day  */
			/*  */
		}
/* ===		
		resource_Sources_Swapped = SAFE_FALSE;
	=== */	
		
	}
	
	return(function_Result);
	
} /* Safe_Boolean	Load_Scene_File... */
/*=**************************************************************************=*/
	
	
/*=**************************************************************************=*/
Safe_Boolean	Select_Scene_And_Display_First_Time(CARDINAL_32	desired_Scene)
{
	Safe_Boolean		function_Result;
	SignedByte			status_Globals_State;
	WindowPtr			current_Graphics_Window_Ptr;
	CARDINAL_8		*code_Name;
	Scene_Ptr			active_Scene_Ptr;
/* END VAR */


	function_Result = SAFE_FALSE;
	code_Name = "\pSelect_Scene_And_Display_First_Time";
		
		
	active_Scene_Ptr = *(dream.play.scene[desired_Scene]);
	if (active_Scene_Ptr->initialized != SAFE_TRUE) {
		goto Error_Handler;
	}
		
	Select_Graphics_Window(TEST_DREAMING_WINDOW_INDEX);
	
	/*  */
	/* Lock down because structure content addresses are being passed */
	/*  */
	if (Begin_Use_Of_Handle((Handle) active_Window_List[TEST_DREAMING_WINDOW_INDEX]
								, MOVE_HIGH_REQUIRED
								, &status_Globals_State) != SAFE_TRUE) {
								
		goto Error_Handler;
	}

	current_Graphics_Window_Ptr	= (**active_Window_List[TEST_DREAMING_WINDOW_INDEX]).status_Window;

	SetWTitle(current_Graphics_Window_Ptr, (StringPtr)WINDOW_TITLE_FOR_TEST_DREAMING);
	
	ForeColor(blackColor);
	BackColor(blackColor);
	BackPat(&(qd.black));
/* ===
	EraseRect(&(current_Graphics_Window_Ptr->portRect));
=== */
	BackColor(whiteColor);
					
		
	/*  */
	/* If there are colors to be aware of,  force them into the */
	/* current palette now, the non-special colors */
	/* were already forced to be more tolerant. */
	/*  */
		
	/*  */
	/*  */
	/*  */
	/*  */
	/*  */
	/*  */
	
	/*  */
	/* This is dependant on the validity of "current_Window_Index" */
	/*  */
	
	if ((active_Window_List[g_Shell.interface.current_Graphics_Window] == nil)
		|| ((**active_Window_List[g_Shell.interface.current_Graphics_Window]).structure_Initialized != SAFE_TRUE)) {
		SysBeep(20); 
		Assign_String(ns_Shell_Globals.error_Description_String, "\pError. Improperly initialized window structure");
		Assign_String(ns_Shell_Globals.error_Note_String, "\pProgrammer flaw");
		Display_Error_Message(unit_Name, code_Name);
		Setup_Error_Handling();
		goto Error_Handler;
	}
	else {
		current_Graphics_Window_Ptr	= (**active_Window_List[g_Shell.interface.current_Graphics_Window]).status_Window;
	}
	
	
	/*  */
	/* Add B&W for certain before using!?! */
	/*  */
	Set_View_Port(current_Graphics_Window_Ptr);
	
	if (active_Scene_Ptr->total_Special_Colors_To_Honor > 0) {
		if ((**(active_Scene_Ptr->scene_Special_Pallete_To_Use)).pmEntries > 0) {	
			if (g_Permit_Palette_Colors == SAFE_TRUE) {
	
				NSetPalette(current_Graphics_Window_Ptr
									, active_Scene_Ptr->scene_Special_Pallete_To_Use
									, pmFgUpdates);
									
				ActivatePalette(current_Graphics_Window_Ptr);
			}
		}
	}
	
	/*  */
	/*  */
	/*  */
	/*  */
	/*  */
	
	/*  */
	/* Fully draw the frame now */
	/*  */

	dream.current_Scene = desired_Scene;
	
	if (Redraw_Idle_Screen_Completely(FALSE, FALSE) == SAFE_TRUE) {
		/*  */
		/* Save window to a PICT for use during updates */
		/*  */
		
	/* ===
		Support_Update_By_Getting_Pict(&((**active_Window_List[TEST_DREAMING_WINDOW_INDEX]).update_Pict)
											,&((**active_Window_List[TEST_DREAMING_WINDOW_INDEX]).status_Window));
	==== */
								
	}
	else {
		/*  */
		/* Ignore error in drawing? */
		/*  */
	}

	/*  */
	/* Set window to a not use a PICT for use during updates */
	/*  */
	(**active_Window_List[TEST_DREAMING_WINDOW_INDEX]).update_Pict_Permissable_To_Use = SAFE_FALSE;
	(**active_Window_List[TEST_DREAMING_WINDOW_INDEX]).update_Pict = nil;
										
	if (End_Use_Of_Handle((Handle) active_Window_List[TEST_DREAMING_WINDOW_INDEX], status_Globals_State) != SAFE_TRUE) {
		goto Error_Handler;
	}
		
	function_Result = SAFE_TRUE;
	goto Normal_Function_Exit;

Error_Handler:
	function_Result = SAFE_FALSE;
		
Normal_Function_Exit:
	return(function_Result);
		
} /* Safe_Boolean	Select_Scene_And_Display_First_Time */
/*=**************************************************************************=*/
	
	
/*=**************************************************************************=*/				
Safe_Boolean	Specify_Scene_To_Save(FSSpec		*file_Specification
															, Str63				*assumed_File_Name
															, Safe_Boolean *user_Abort);
Safe_Boolean	Specify_Scene_To_Save(FSSpec		*file_Specification
															, Str63				*assumed_File_Name
															, Safe_Boolean *user_Abort)
{
	INTEGER_16 		number_Of_Types;
	Boolean				user_Clicked_Ok;
	Str63					original_Name;
	Str63					suffix_String;
	Str255					message_String;
	SFTypeList 			type_List;
	SFReply				sf_Reply;
	Safe_Boolean		function_Result;
/* END VAR */


	function_Result = SAFE_FALSE;
	*user_Abort = SAFE_FALSE;

	Get_NS_String(k_DREAM_STRINGS, k_SAVE_SCENE_PROMPT, message_String);
	Get_NS_String(k_DREAM_STRINGS, k_SAVE_SCENE_FILE_SUFFIX, suffix_String);

	Assign_String(original_Name, *assumed_File_Name);
	Strip_Off_Popular_Suffixes(&original_Name);
	
	/*  */
	/* Suffix not appended if just space character */
	/*  */
	if ((suffix_String[0] != 1) || (suffix_String[1] != ASCII_SPACE)) {
		Concat_String(original_Name, suffix_String);
	}
	
	if(original_Name[0] >31) {
		original_Name[0] = 31;
	}

	user_Clicked_Ok = Perform_Put_File_Dialog(message_String
										, original_Name
										, &sf_Reply);
	
	if ((user_Clicked_Ok != TRUE) || (sf_Reply.good != TRUE)) {
		/*  */
		/* User Abort, no file name */
		/*  */
		*user_Abort = SAFE_TRUE;
		function_Result = SAFE_FALSE;
		goto Normal_Function_Exit;
	}

	Get_Last_Volume_Reference_And_Directory_ID(&(file_Specification->vRefNum)
																	, &(file_Specification->parID));
																	
	Copy_Pascal_String(sf_Reply.fName, file_Specification->name);
		
	function_Result = SAFE_TRUE;
	goto Normal_Function_Exit;

Error_Handler:
	function_Result = SAFE_FALSE;
		
Normal_Function_Exit:
	return(function_Result);
	
} /* Safe_Boolean	Specify_Scene_To_Save... */
/*=**************************************************************************=*/


/*=**************************************************************************=*/
Safe_Boolean	Save_Scene_Sequence(Save_Scene_Rec	*params_For_Save_Scene)
{
	INTEGER_16		scrap_Volume_Reference_Number;
	INTEGER_32		destination_Directory_ID;
	INTEGER_16		file_Reference;
	CTabHandle		temp_Clut_Handle;
	OSErr					os_Error;
	INTEGER_32		volume_Space;
	CARDINAL_32	file_Creator;
	CARDINAL_32	file_Type;
	CARDINAL_32	resource_Type;
	CARDINAL_32	file_Minimum_Size;
	INTEGER_16		resource_ID;
	Scene_Ptr			active_Scene_Ptr;
	Safe_Boolean		function_Result;
	CARDINAL_32	preferred_Size;
	
	Scene_File_Header_Ptr				header;
	Scene_File_Header_Hdl				resource_To_Create_Hdl;
	Memory_Allocation					resource_To_Create_Hdl_Memory_Initialized;
	
	Scene_File_Ptr							body;
	Scene_File_Hdl							expandable_Resource_Hdl;
	Memory_Allocation					expandable_Resource_Hdl_Memory_Initialized;

	Item_File_Reference_Ptr			item;
	Item_File_Reference_Hdl			item_Resource_Hdl;
	Memory_Allocation					item_Resource_Hdl_Memory_Initialized;

	Handle				template_Hdl;
	SIMPLE_LOOP	loop;
	INTEGER_16		index;
	Safe_Boolean		working_Directory_In_Use;
	INTEGER_16		working_Directory;
	WDPBRec			working_Directory_PB;
	CARDINAL_32	current_File_Offset;
	CARDINAL_8		*code_Name;
	Str255					resource_Comment;
	Str255					note_String;
	Str255					volume_Name;
	Str255					work_String_2;
/* END VAR */


	code_Name = "\pSave_Scene_Sequence";
	function_Result = SAFE_FALSE;
	working_Directory_In_Use = SAFE_FALSE;
	resource_To_Create_Hdl_Memory_Initialized = MEMORY_NOT_INITIALIZED;
	expandable_Resource_Hdl_Memory_Initialized = MEMORY_NOT_INITIALIZED;
	item_Resource_Hdl_Memory_Initialized = MEMORY_NOT_INITIALIZED;
		
	if ((dream.play.scene_Memory_In_Use[params_For_Save_Scene->desired_Scene]) != SAFE_TRUE) {
		function_Result = SAFE_FALSE;
		goto Normal_Function_Exit;
	}
	
	/*  */
	/* active_Scene_Ptr uses a handle assumed to always be locked */
	/*  */
	active_Scene_Ptr = *(dream.play.scene[params_For_Save_Scene->desired_Scene]);
			
	file_Creator= FILE_CREATOR;
	file_Type = k_SCENE_FILE_TYPE;
	note_String[0] = 0;
	
	os_Error = GetVInfo((Get_Drive_Number(params_For_Save_Scene->desired_File.vRefNum)), volume_Name, &scrap_Volume_Reference_Number, (INTEGER_32 *)&volume_Space);
	if (os_Error != noErr) {
		Concat_Many(note_String, 3, "\pGetVInfo for volume ", volume_Name, "\p ID = ");
		Concat_Long_To_String(note_String, os_Error, NO_LIMIT);
		goto Error_Handler_With_Message;
	} /* if (os_Error != noErr) */
		
	preferred_Size = sizeof(Scene_File_Header_Rec);
	resource_To_Create_Hdl = nil;
	resource_To_Create_Hdl_Memory_Initialized = MEMORY_NOT_INITIALIZED;

	if (Create_Handle(&preferred_Size
						, (CARDINAL_32)(10240L)
						, ALLOW_PURGING
						, DISALLOW_PARTIAL_AMOUNT
						, (Handle *)&resource_To_Create_Hdl
						, &resource_To_Create_Hdl_Memory_Initialized) != SAFE_TRUE) {
						
		Display_Low_Memory_Error_Message(unit_Name, code_Name, sizeof(Scene_File_Header_Rec));
		
		function_Result = SAFE_FALSE;
		goto Error_Handler;
	}
	
	preferred_Size = sizeof(Scene_File_Rec);
	expandable_Resource_Hdl = nil;
	expandable_Resource_Hdl_Memory_Initialized = MEMORY_NOT_INITIALIZED;

	if (Create_Handle(&preferred_Size
						, (CARDINAL_32)(10240L)
						, ALLOW_PURGING
						, DISALLOW_PARTIAL_AMOUNT
						, (Handle *)&expandable_Resource_Hdl
						, &expandable_Resource_Hdl_Memory_Initialized) != SAFE_TRUE) {
						
		Display_Low_Memory_Error_Message(unit_Name, code_Name, sizeof(Scene_File_Rec));
		function_Result = SAFE_FALSE;
		goto Error_Handler;
	}
	
	preferred_Size = sizeof(Item_File_Reference_Rec);
	item_Resource_Hdl = nil;
	item_Resource_Hdl_Memory_Initialized = MEMORY_NOT_INITIALIZED;
	if (Create_Handle(&preferred_Size
						, (CARDINAL_32)(10240L)
						, ALLOW_PURGING
						, DISALLOW_PARTIAL_AMOUNT
						, (Handle *)&item_Resource_Hdl
						, &item_Resource_Hdl_Memory_Initialized) != SAFE_TRUE) {
						
		Display_Low_Memory_Error_Message(unit_Name, code_Name, sizeof(Item_File_Reference_Rec));
		function_Result = SAFE_FALSE;
		goto Error_Handler;
	}

	/*  */
	/* Start a section, handle not locked: not calling memory move indirectly */
	/*  */
	header = (Scene_File_Header_Ptr)*resource_To_Create_Hdl;
	header->valid = SAFE_TRUE;
	header->version = k_SCENE_VERSION_NOW;
	header->backward_Compatible_Version = k_SCENE_VERSION_NOW;
	header->total_Special_Colors_To_Honor = active_Scene_Ptr->total_Special_Colors_To_Honor;
	header->scene_Size_Rect = active_Scene_Ptr->scene_Background_Rect;
	
	header->backdrop_Selected = active_Scene_Ptr->backdrop_Selected;
	for (loop = 0; (loop < 64) ; loop++) {
		header->backdrop_Name[loop] = 0;
	}
	if (header->backdrop_Selected == SAFE_TRUE) {
	Copy_Pascal_String(active_Scene_Ptr->backdrop_Name
		, header->backdrop_Name);
	}
	
	switch (active_Scene_Ptr->scene_Port.color_Type) {
		case k_COLOR_QUICKDRAW:
		case k_32_BIT_COLOR_QUICKDRAW:
		
			if (active_Scene_Ptr->scene_Port.image_Handle != nil) {
			/* ===
				header->stored_Offscreen_Rect = (**(active_Scene_Ptr->scene_Port.offscreen_Port.color_Port.portPixMap)).bounds;
			=== */
				header->stored_Bit_Depth = (**(active_Scene_Ptr->scene_Port.offscreen_Port.color_Port.portPixMap)).pixelSize;
			}
			else {
				/*  */
				/* Cannot get here. paranoia code */
				/*  */
				header->stored_Bit_Depth = 8;
			}
		break;
		
		case k_ORIGINAL_QUICKDRAW:
		default:
			header->stored_Bit_Depth = 1;
	} /* switch (active_Scene_Ptr->scene_Port.color_Type) */
	
	header->total_Colors_Used_In_Pallette
		= active_Scene_Ptr->total_Colors_Used_In_Pallette;
		
		
		
	if (active_Scene_Ptr->scene_Special_Pallete_To_Use != nil) {
		/*  */
		/* the special pallete needs minding. Set the resource value */
		/*  */
		header->special_Pallette_ID = k_SCENE_PALLETE_RESOURCE_BASE;
	}
	else {
		header->special_Pallette_ID = 0;
	}

	header->current_Clut_ID = k_SCENE_CLUT_RESOURCE_BASE;
					
	/*  */
	/* Clean up strings for file tidiness */
	/*  */
	for (loop = 0; (loop < 256) ; loop++) {
		header->screen_Name[loop] = 0;
	}
	
	for (loop = 0; (loop < 64) ; loop++) {
		header->screen_Short_Name[loop] = 0;
	}
	
	for (loop = 0; (loop < 64) ; loop++) {
		header->file_Name[loop] = 0;
	}
	
	Copy_Pascal_String(active_Scene_Ptr->screen_Name, header->screen_Name);
	
	Copy_Pascal_String(active_Scene_Ptr->screen_Short_Name
		, header->screen_Short_Name);
	
	Copy_Pascal_String(active_Scene_Ptr->file_Name
		, header->file_Name);
			
				
	body = (Scene_File_Ptr)*expandable_Resource_Hdl;

	body->valid = SAFE_TRUE;
	
	for (loop = 0; (loop < k_MAX_FILE_STAGE_ITEMS) ; loop++) {
		body->file_Scene_Items[loop] = 0;
	}
	
	header->file_Item_Total = 0;
	if ((*active_Scene_Ptr).total_Items_In_Scene > 0) {
		for (loop = 0; (loop < k_MAX_STAGE_ITEMS); loop++) {
			if ((*active_Scene_Ptr).item[loop].valid_Pointer == SAFE_TRUE) {
				header->file_Item_Total += 1;
			}
		}
	}	
	
	/*  */
	/* Truncate total if too many! Add warning one day. Safe. */
	/*  */
	if (header->file_Item_Total > k_MAX_FILE_STAGE_ITEMS) {
		header->file_Item_Total = k_MAX_FILE_STAGE_ITEMS;
	}
		
	current_File_Offset = 0;
	for (loop = 0; ((loop < k_MAX_STAGE_ITEMS) 
			&& (current_File_Offset < header->file_Item_Total)); loop++) {
			
		if ((*active_Scene_Ptr).item[loop].valid_Pointer == SAFE_TRUE) {

			body->file_Scene_Items[current_File_Offset] = k_BASE_REGION_RESOURCE_ID + current_File_Offset;
			
			current_File_Offset += 1;
		}
	}
	
	
	body->resource_Template_Count_1 = k_MAX_FILE_STAGE_ITEMS;
	body->valid = SAFE_TRUE;
	body->end_Sentinel = SAFE_TRUE;
	body->total_Items_In_Scene = header->file_Item_Total;
	
	/*  */
	/* End the section not calling memory move indirectly */
	/*  */
		
		
	/*  */
	/* Determine free space, first get header size */
	/*  */
	file_Minimum_Size = GetHandleSize((Handle)resource_To_Create_Hdl);
	if (MemError() != noErr) {
		goto Error_Handler;
	}
	
	/*  */
	/* Add in the size of the second part of the resource */
	/*  */
	file_Minimum_Size = file_Minimum_Size + GetHandleSize((Handle)expandable_Resource_Hdl);
	if (MemError() != noErr) {
		goto Error_Handler;
	}
	
	/*  */
	/* Add in the sizes of the scene resources */
	/*  */
	file_Minimum_Size = (file_Minimum_Size 
			+ ((CARDINAL_32)sizeof(Item_File_Reference_Rec) * active_Scene_Ptr->total_Items_In_Scene));	

	if (((CARDINAL_32)file_Minimum_Size + (CARDINAL_32)MINIMUM_DISK_SPACE_TO_PRESERVE) > volume_Space) {

			Concat_Many(note_String, 3, "\p", volume_Name, "\p is too full for the file. (");
			Make_String_Long_String(work_String_2, (CARDINAL_8 *)note_String, volume_Space, NO_LIMIT,"\p)bytes free, (" );
			Make_String_Long_String(note_String
										, (CARDINAL_8 *)work_String_2
										, (file_Minimum_Size + MINIMUM_DISK_SPACE_TO_PRESERVE)
										, NO_LIMIT
										,"\p)bytes needed.  Find additional space for file." );

		goto Error_Handler_With_Message;
	}

	os_Error = HCreate(params_For_Save_Scene->desired_File.vRefNum
									, params_For_Save_Scene->desired_File.parID
									, params_For_Save_Scene->desired_File.name
									, file_Creator
									, file_Type);
						
	if (os_Error == dupFNErr) {
		os_Error = HDelete(params_For_Save_Scene->desired_File.vRefNum
									, params_For_Save_Scene->desired_File.parID
									, params_For_Save_Scene->desired_File.name);
		if (os_Error != noErr) {
			Concat_Many(note_String, 3,"\pFSDelete for file ", params_For_Save_Scene->desired_File.name, "\p ID = ");
			Concat_Long_To_String(note_String, os_Error, NO_LIMIT);
			goto Error_Handler_With_Message;
		}

		os_Error = HCreate(params_For_Save_Scene->desired_File.vRefNum
									, params_For_Save_Scene->desired_File.parID
									, params_For_Save_Scene->desired_File.name
							, file_Creator
							, file_Type);
	} /* if (os_Error == dupFNErr) */

	if (os_Error != noErr) {
		Concat_Many(note_String, 3,"\pCreate for file ", params_For_Save_Scene->desired_File.name, "\p ID = ");
		Concat_Long_To_String(note_String, os_Error, NO_LIMIT);
		goto Error_Handler_With_Message;
	} 

	HCreateResFile(params_For_Save_Scene->desired_File.vRefNum
									, params_For_Save_Scene->desired_File.parID
									, params_For_Save_Scene->desired_File.name);
	os_Error = ResError();
	if (os_Error != noErr) {
		Concat_Many(note_String, 3,"\pCreateResFile for file ", params_For_Save_Scene->desired_File.name, "\p ID = ");
		Concat_Long_To_String(note_String, os_Error, NO_LIMIT);
		goto Error_Handler_With_Message;
	} 

	/*  */
	/* Crappy deficient OpenRFPerm needs ancient Working Directory */
	/*  */
	working_Directory_PB.ioCompletion = nil;
	working_Directory_PB.ioNamePtr = nil;
	working_Directory_PB.ioWDProcID = FILE_CREATOR;
	working_Directory_PB.ioVRefNum = params_For_Save_Scene->desired_File.vRefNum;
	working_Directory_PB.ioWDDirID = params_For_Save_Scene->desired_File.parID;
	
	os_Error = PBOpenWD(&working_Directory_PB, FALSE);
	if (os_Error != noErr) {
		goto Error_Handler;
	}
	working_Directory_In_Use = SAFE_TRUE;
	working_Directory = working_Directory_PB.ioVRefNum;

	/*=--------------------------------------------------------------------------=*/
	/* 'OpenRFPerm' used to prevent having to use 'SetVol' */
	/* but still needs to rely on annoying "PBOpenWD" */
	/* 'StripAddress' called as per tech notes and THINK reference */
	/*=--------------------------------------------------------------------------=*/
	
	file_Reference = OpenRFPerm(((StringPtr)(StripAddress(
																	&(params_For_Save_Scene->desired_File.name[0]))))
																	, working_Directory
																	, fsRdWrPerm );

	os_Error = ResError();
	if ((file_Reference == -1) || ( os_Error != noErr)) {
		Display_Serious_Error_Message(unit_Name, code_Name, "\pOS Error OpenResFile.", "\pNo resource fork?");
		goto Error_Handler;
	}

	resource_Type = k_SCENE_MAIN_RESOURCE_TYPE;
	resource_ID = k_BASE_SCENE_RESOURCE_ID;
	Get_NS_String(k_DREAM_STRINGS, k_DONT_EDIT_RESOURCE_COMMENT, resource_Comment);

	if (Add_Resource_Handle((Handle)resource_To_Create_Hdl
											, resource_Type
											, resource_ID
											, resource_Comment) != SAFE_TRUE) {
		goto Error_Handler;
	}
	
	resource_Type = k_SCENE_EXPANDABLE_RESOURCE_TYPE;
	
	if (Add_Resource_Handle((Handle)expandable_Resource_Hdl
											, resource_Type
											, resource_ID
											, resource_Comment) != SAFE_TRUE) {
		goto Error_Handler;
	}
	
		
	/*  */
	/* Save all scene item records */
	/*  */
	resource_Type = k_SCENE_ITEM_RESOURCE_TYPE;
	current_File_Offset = 0;
	(**item_Resource_Hdl).valid = SAFE_TRUE;
	for (loop = 0; ((loop < k_MAX_STAGE_ITEMS) 
			&& (current_File_Offset < active_Scene_Ptr->total_Items_In_Scene)); loop++) {
			
		if ((*active_Scene_Ptr).item[loop].valid_Pointer == SAFE_TRUE) {
			resource_ID = k_BASE_REGION_RESOURCE_ID + current_File_Offset;
			for (index = 0; (index < 64) ; index++) {
				(**item_Resource_Hdl).item_Name[index] = 0;
			}
			
			if ((*active_Scene_Ptr).item[loop].desired_File_Name[0] != 0) {
				(**item_Resource_Hdl).scene_Position = (*active_Scene_Ptr).item[loop].desired_Start_Location;
								
				Copy_Pascal_String(&((*active_Scene_Ptr).item[loop].desired_File_Name[0])
					, (**item_Resource_Hdl).item_Name);
						
				if (Add_Resource_Handle((Handle)item_Resource_Hdl
														, resource_Type
														, resource_ID
														, "\pItem in Scene") != SAFE_TRUE) {
					goto Error_Handler;
				}
			
			}
			else if ((*active_Scene_Ptr).item[loop].puppet->puppet_Initialized == SAFE_TRUE) {
				/*  */
				/* Bizarre error, take name from last loaded puppet. */
				/* not expected to ever happen that way */
				/*  */
				
				/*  */
				/* Unexpected! Oh well, mark it as a blank space */
				/*  */
				SetRect(&((**item_Resource_Hdl).scene_Position),0,0,1,1);
	
				Copy_Pascal_String((*active_Scene_Ptr).item[loop].puppet->file_Name
					, (**item_Resource_Hdl).item_Name);
						
				if (Add_Resource_Handle((Handle)item_Resource_Hdl
														, resource_Type
														, resource_ID
														, "\pLast Item in Scene") != SAFE_TRUE) {
					goto Error_Handler;
				}
				
			}
			else {
				/*  */
				/* Unexpected! Oh well, mark it as a blank space */
				/*  */
				SetRect(&((**item_Resource_Hdl).scene_Position),0,0,1,1);
				(**item_Resource_Hdl).valid = SAFE_FALSE;
				if (Add_Resource_Handle((Handle)item_Resource_Hdl
														, resource_Type
														, resource_ID
														, "\pUndefinable Item in Scene") != SAFE_TRUE) {
					goto Error_Handler;
				}
				(**item_Resource_Hdl).valid = SAFE_TRUE;
			}
			
			current_File_Offset += 1;
		}
	}
	
	
	/*  */
	/* Save pallete listing all used colors (except B&W) and all special colors. */
	/*  */
	if ((**resource_To_Create_Hdl).special_Pallette_ID != 0) {		
		resource_ID = (**resource_To_Create_Hdl).special_Pallette_ID;
		resource_Type = k_SCENE_PALLETE_RESOURCE_TYPE;
		
		if (Add_Resource_Handle((Handle)active_Scene_Ptr->scene_Special_Pallete_To_Use
												, resource_Type
												, resource_ID
												, "\p") != SAFE_TRUE) {
			goto Error_Handler;
		}
	}
	
	/*  */
	/* Save current puppet offscreen port's color lookup table */
	/*  */
	if ((**resource_To_Create_Hdl).current_Clut_ID != 0) {		
		resource_ID = (**resource_To_Create_Hdl).current_Clut_ID;
		resource_Type = k_SCENE_CLUT_RESOURCE_TYPE;
				
		switch (active_Scene_Ptr->scene_Port.color_Type) {
			case k_COLOR_QUICKDRAW:
			case k_32_BIT_COLOR_QUICKDRAW:
				if (active_Scene_Ptr->scene_Port.image_Handle != nil) {
					temp_Clut_Handle = active_Scene_Ptr->scene_Port.color_Table_Handle;
				}
				else {
					(**resource_To_Create_Hdl).current_Clut_ID = 0;
				}
			break;
			
			case k_ORIGINAL_QUICKDRAW:
			default:
				(**resource_To_Create_Hdl).current_Clut_ID = 0;
		} /* switch (active_Scene_Ptr->scene_Port.color_Type) */


		if ((**resource_To_Create_Hdl).current_Clut_ID != 0) {
			if (Add_Resource_Handle((Handle)temp_Clut_Handle
													, resource_Type
													, resource_ID
													, "\p") != SAFE_TRUE) {
				goto Error_Handler;
			}
		}
		
	} /* if ((**resource_To_Create_Hdl).current_Clut_ID != 0) ... */
	
	/*  */
	/* Now add the template for the header */
	/*  */

	resource_Type = 'TMPL';
	resource_ID = k_SCENE_HEADER_TEMPLATE_ID;
	
	template_Hdl = GetResource(resource_Type, resource_ID);
	if (template_Hdl != nil) {
	
		DetachResource(template_Hdl);
		os_Error = ResError();
		if ( os_Error != noErr) {
			Display_Serious_Error_Message(unit_Name, code_Name, "\pOS Error DetachResource.", "\p");
			goto Error_Handler;
		} 
	
		if (Add_Resource_Handle((Handle)template_Hdl
												, resource_Type
												, resource_ID
												, k_SCENE_HEADER_TEMPLATE_NAME) != SAFE_TRUE) {
			goto Error_Handler;
		}
		
		DisposeHandle(template_Hdl);
		os_Error = MemError();
		if ( os_Error != noErr) {
			Display_Serious_Error_Message(unit_Name, code_Name, "\pOS Error DisposeHandle.", "\p");
			goto Error_Handler;
		} 
		
	} /* if (template_Hdl != nil) */

	/*  */
	/* Now add the template for the scene body (expanding list) */
	/*  */

	resource_Type = 'TMPL';
	resource_ID = k_SCENE_BODY_TEMPLATE_ID;
	
	template_Hdl = GetResource(resource_Type, resource_ID);
	if (template_Hdl != nil) {
	
		DetachResource(template_Hdl);
		os_Error = ResError();
		if ( os_Error != noErr) {
			Display_Serious_Error_Message(unit_Name, code_Name, "\pOS Error DetachResource.", "\p");
			goto Error_Handler;
		} 
	
		if (Add_Resource_Handle((Handle)template_Hdl
												, resource_Type
												, resource_ID
												, k_SCENE_BODY_TEMPLATE_NAME) != SAFE_TRUE) {
			goto Error_Handler;
		}
		
		DisposeHandle(template_Hdl);
		os_Error = MemError();
		if ( os_Error != noErr) {
			Display_Serious_Error_Message(unit_Name, code_Name, "\pOS Error DetachResource.", "\p");
			goto Error_Handler;
		} 
		
	} /* if (template_Hdl != nil) */

	/*  */
	/* Now add the template for the items */
	/*  */

	resource_Type = 'TMPL';
	resource_ID =k_SCENE_ITEM_TEMPLATE_ID;
	
	template_Hdl = GetResource(resource_Type, resource_ID);
	if (template_Hdl != nil) {
	
		DetachResource(template_Hdl);
		os_Error = ResError();
		if ( os_Error != noErr) {
			Display_Serious_Error_Message(unit_Name, code_Name, "\pOS Error DetachResource.", "\p");
			goto Error_Handler;
		} 
	
		if (Add_Resource_Handle((Handle)template_Hdl
												, resource_Type
												, resource_ID
												, k_SCENE_ITEM_TEMPLATE_NAME) != SAFE_TRUE) {
			goto Error_Handler;
		}
		
		DisposeHandle(template_Hdl);
		os_Error = MemError();
		if ( os_Error != noErr) {
			Display_Serious_Error_Message(unit_Name, code_Name, "\pOS Error DisposeHandle.", "\p");
			goto Error_Handler;
		} 
		
	} /* if (template_Hdl != nil) */
	
	CloseResFile(file_Reference);
	os_Error = ResError();
	if ( os_Error != noErr) {
		Display_Serious_Error_Message(unit_Name, code_Name, "\pOS Error CloseResFile.", "\p");
	}

	file_Reference = 0;

	function_Result = SAFE_TRUE;
	goto Normal_Function_Exit;

Error_Handler_With_Message:
	function_Result = SAFE_FALSE;
	Display_Serious_Error_Message(unit_Name, code_Name, note_String, "\p");
	goto Normal_Function_Exit;

Error_Handler:
	Display_Serious_Error_Message(unit_Name, code_Name, "\pInternal problem saving scene", "\p");
	function_Result = SAFE_FALSE;
		
Normal_Function_Exit:

	/*  */
	/* Placed here to ensure not accidentally ever left allocated */
	/*  */
	if (working_Directory_In_Use == SAFE_TRUE) {
		os_Error = CloseWD(working_Directory);
		if (os_Error != noErr) {
			goto Error_Handler;
		}
	}

	if (expandable_Resource_Hdl_Memory_Initialized == MEMORY_INITIALIZED) {
		if (Destroy_Handle((Handle *)&expandable_Resource_Hdl
							, &expandable_Resource_Hdl_Memory_Initialized) != SAFE_TRUE) {
							
			Display_Serious_Error_Message(unit_Name, code_Name, "\pError with Destroy_Handle", "\pSystem Corrupted?");
		}
	}
	
	if (resource_To_Create_Hdl_Memory_Initialized == MEMORY_INITIALIZED) {
		if (Destroy_Handle((Handle *)&resource_To_Create_Hdl
							, &resource_To_Create_Hdl_Memory_Initialized) != SAFE_TRUE) {
							
			Display_Serious_Error_Message(unit_Name, code_Name, "\pError with Destroy_Handle", "\pSystem Corrupted?");
		}
	}
	
	if (item_Resource_Hdl_Memory_Initialized == MEMORY_INITIALIZED) {
		if (Destroy_Handle((Handle *)&item_Resource_Hdl
							, &item_Resource_Hdl_Memory_Initialized) != SAFE_TRUE) {
							
			Display_Serious_Error_Message(unit_Name, code_Name, "\pError with Destroy_Handle", "\pSystem Corrupted?");
		}
	}

	return(function_Result);
	
} /* Safe_Boolean	Save_Scene_Sequence... */
/*=**************************************************************************=*/


/*=**************************************************************************=*/
/* END OF FILE: Dreaming_Scene_Files.c */
