/* START OF FILE: Dreaming_Stuff.c */


/*=**************************************************************************=*/
/* [---BEGIN---] FILE HEADER COMMENTS */
/**
 ** SCREEN VIEW = [Adobe Syntax Black-10, Tabs = 4]
 ** PRINTER VIEW = [Adobe Syntax Black-10, Tabs = 4]
 ** =======================================================================
 ** Written by James Merkle
 ** Dreaming_Stuff.c - Miscellaneous Routines to support Poor Man's Video Phone
 ** (C) Copyright 1993 James Merkle
 ** All rights reserved, except those wishing to "copy" moderate sections of code or files may do so, so long as
 ** such code is never used directly either in animation-related tools or multimedia tools that are  
 ** used in live performance or live broadcast or recorded mass market broadcast or film. Desktop usage not restricted.
 ** Conventional desktop game or entertainment not restricted.
 ** =======================================================================
 **
 **
 ** ============
 ** Revision History
 ** ============
 **
 **  1993.02.01		(James Merkle)
 **						Initial coding started, from scratch
 **
 **  1994.06.09		(James Merkle)
 **						REM: Remember to not set "current" scene when possibly loading while another is animating!
 **
 **
 **
 **
 **
 **
 **
 ** =================
 ** OTHER INFORMATION
 ** =================
 ** 
 ** 
 **
 **
 **/
/* [----END----] FILE HEADER COMMENTS */
/*=**************************************************************************=*/


/*=**************************************************************************=*/
/* [---BEGIN---] #include AREA */

#include <PictUtil.h>
#include <QuickDraw.h>

#include "Core_Universal.h" /* Non_Specific_Support_Routines */
#include "Core_Specific.h" /*System_Dependent_Hooks.h */
#include "SysHooks.h" /*System_Dependent_Hooks.h */
#include "Shell_Globals.h"
#include "Cursor_Utilities.h"

#include "Core_Globals.h" /*System_Dependent_Hooks.h */
#include "Memory_Utilities.h"
#include "String_Utilities.h"
#include "Shell_Dialogs.h"
#include "Shell_Assembly.h"
#include "Dreaming_Popup_Dialogs.h"
#include "Dreaming_Backdrop_Files.h"
#include "Spin_O_Rama_Files.h"
#include "Graphics_Primitives.h"

#include "Interface_Utilities.h"
#include "Error_Utilities.h"
#include "Spin_O_Rama.h"
#include "Dreaming_Audio_Record.h"

#include "Indexed_Background_Removal.h"
#include "Dreaming_Audio.h"
#include "Dreaming_Animation.h"
#include "Dreaming_Analog_Card.h"
#include "Dreaming_File_Loading.h"
#include "Dreaming_Scene_Files.h"
#include "Dreaming_Color_Stuff.h"
#include "Object_Inventory.h"
#include "Dreaming_Stuff.h"
#include "Video_Wipes.h"
#include "Window_Environment.h"

#include "Slide_Show_Routines.h"

#include "Graphics_Primitives.h"

/* [----END----] #include AREA */
/*=**************************************************************************=*/


/*=**************************************************************************=*/
/* [---BEGIN---] #define AREA */

#define k_MAX_POSSIBLE_POSITIVE_PICT_ID		129

/*  */
/* These can be set to anything but the importer expects */
/* all puppets to be a uniform maximum size. */
/* They are only used if resource missing though. */
/*  */
#define	k_ULTIMATE_PUPPET_WIDTH		640
/* was 390 */
#define	k_ULTIMATE_PUPPET_HEIGHT	480

/*  */
/* These should be re-examined and adjusted */
/* They are only used if resource missing though */
/*  */
#define MINIMUM_WINDOW_HEIGHT		64
#define MINIMUM_WINDOW_WIDTH		64

#define MAIN_WINDOW_HEIGHT	k_ULTIMATE_PUPPET_HEIGHT
#define MAIN_WINDOW_WIDTH	k_ULTIMATE_PUPPET_WIDTH


#define k_SUFFIX_PICT_STR			"\p.Pict"
#define k_SUFFIX_PICR_STR			"\p.Picr"
#define k_SUFFIX_PUP_STR			"\p.pup"
#define k_SUFFIX_BAK_STR			"\p.Bak"
#define k_SUFFIX_PICS_STR			"\p.pics"
#define k_SUFFIX_TXT_STR			"\p.Txt"
#define k_SUFFIX_AIFF_STR			"\p.Aiff"


#define k_KEYCODE_ASCII_TILDE			0x0032
#define k_KEYCODE_ASCII_ESCAPE		0x0035


#define k_STARTUP_SCENE_DEFAULT_FILE_NAME	"\pStartup Scene"

/* [----END----] #define AREA */
/*=**************************************************************************=*/


/*=**************************************************************************=*/
/* [---BEGIN---] TYPE AREA */


/* [----END----] TYPE AREA */
/*=**************************************************************************=*/


/*=**************************************************************************=*/
/* [---BEGIN---] PROTOTYPE AREA */

Safe_Boolean	Create_Region_For_Frame(CARDINAL_32 puppet_Index
																	, CARDINAL_32 frame
																	, RgnHandle *mask
																	, Boolean	sequence_Has_Background);

Safe_Boolean	Initialize_Backdrop_Information(void);

Safe_Boolean Find_Empty_Position_For_Scene(INTEGER_16 *position);


Safe_Boolean	Interpret_Mouse_In_Scene(Primary_Status_Hdl current_Window_Globals_Hdl
												, WindowPtr event_Window
												, EventRecord	*event_Record);

Safe_Boolean Find_Puppet_Index_Using_Address(Puppet_Ptr	address_Of_Desired
																, INTEGER_16 *position);

Safe_Boolean	Initialize_Spin_Object_Information(void);

Safe_Boolean Load_Raw_Pict_Fork_From_Backdrop(FSSpec *file_Info
																		, CARDINAL_16	desired_Color_Depth
																		, CTabHandle		offscreen_Color_Table_Handle
																		, CGrafPtr	offscreen_Graph_Port
																		, GDHandle	*offscreen_GDevice_Ptr
																		, Handle		*image_Handle_Ptr);


/* [----END----] PROTOTYPE AREA */
/*=**************************************************************************=*/


/*=**************************************************************************=*/
/* [---BEGIN---] GLOBAL DEFINITION AREA */

	Dreaming_Globals_Rec		dream;
	Safe_Boolean						g_Dreaming_Module_Globals_Initialized;
	Safe_Boolean						g_Debugging_Options[4];
	
	Safe_Boolean						g_Startup_Initialization_Complete;
	
	/*  */
	/* Used for making menu setting more intelligent and less CPU wasteful */
	/*  */
	Safe_Boolean						g_Menus_Assume_No_Scenes;
	Safe_Boolean						g_Menus_Assume_No_Spin_Objects;
	Safe_Boolean						g_Menus_Assume_No_Backgrounds;
	Safe_Boolean						g_Menus_Assume_No_Items;


/* [----END----] GLOBAL DEFINITION AREA */
/*=**************************************************************************=*/


/*=**************************************************************************=*/
/* [---BEGIN---] LOCAL STATIC DEFINITION AREA */

static CARDINAL_8	*unit_Name = "\pDreaming_Stuff";

	Dreaming_Globals_Ptr		dreaming_Ptr;

/* [----END----] LOCAL STATIC DEFINITION AREA */
/*=**************************************************************************=*/


/*=**************************************************************************=*/
Safe_Boolean	Initialize_Dreaming_If_Needed(void)
{
	Safe_Boolean		function_Result;
	CARDINAL_32	index;
/* END VAR */


	function_Result = SAFE_FALSE;

	if (g_Dreaming_Module_Globals_Initialized != SAFE_TRUE) {
	
		/*  */
		/* PROG NOTE: This should be put somewhere else in a better structure. */
		/*  */
		g_Startup_Initialization_Complete = SAFE_FALSE;
		
		g_Permit_Palette_Colors = SAFE_TRUE;
		
		g_Voices.audio_Channels_Needed_On_Cards = k_DEFAULT_CARD_CHANNELS;
		
		/*  */
		/* TEMP HACK TECHNOLOGY */
		/* Done mainly because a ONE-CLUT system is needed if */
		/* puppet art pixels cannot be renegotiated at loads for */
		/* speed */
		/*  */
		g_Force_All_To_Use_Main_Puppet_Colors = SAFE_TRUE;
		
		dreaming_Ptr = &dream;
		
		for (index = 0; (index < k_MAXIMUM_PUPPETS); index++) {
			dreaming_Ptr->puppet[index] = nil;
			dreaming_Ptr->puppet_Memory_Initialized[index] = MEMORY_NOT_INITIALIZED;
		}
		
		dreaming_Ptr->main_Active_Scene_Specified = SAFE_FALSE;

		/*  */
		/* Simulate shaky camera or active puppet */
		/*  */
		dreaming_Ptr->horizontal_Jiggle_Maximum = 0;
		dreaming_Ptr->vertical_Jiggle_Maximum = 0;
		dreaming_Ptr->jiggle_Active = SAFE_FALSE;
		dreaming_Ptr->main_Window_Preferred_Top_Offset = 1;

		dreaming_Ptr->main_Window_Preferred_Height = g_Window_Limits.main_Window_Height;
		dreaming_Ptr->main_Window_Preferred_Width = g_Window_Limits.main_Window_Width;
		
		dreaming_Ptr->active_Puppets_Total = 0;
		dreaming_Ptr->initialized = SAFE_TRUE;
		g_Dreaming_Module_Globals_Initialized = SAFE_TRUE;
		dreaming_Ptr->play.total_Scenes = 0;
		dreaming_Ptr->play.scenes_Still_Loading = 0;
		dreaming_Ptr->play.default_Start_Audio_Point = 1;
		
		dreaming_Ptr->current_Scene = 0;
		dreaming_Ptr->previously_Displayed_Scene = 0;
		
		dreaming_Ptr->play.transition_Effect = EFFECT_NOTHING;
		dreaming_Ptr->play.transition_Effect_For_Introductions = EFFECT_CIRCLE_BULGE;
				
		for (index = 0; (index < k_MAX_ACTIVE_SCENES); index++) {
			dreaming_Ptr->play.scene[index] = nil;
			dreaming_Ptr->play.scene_Memory_In_Use[index] = MEMORY_NOT_INITIALIZED;
			dreaming_Ptr->play.scene_Hot_Key[index] = 'A';
		}
		
		if (Initialize_Audio_Monitoring_If_Needed() != SAFE_TRUE) {
			goto Error_Handler;
		}
		
		if (Initialize_Backdrop_Information() != SAFE_TRUE) {
			goto Error_Handler;
		}
		
		if (Initialize_Spin_Object_Information() != SAFE_TRUE) {
			goto Error_Handler;
		}
		
		g_Menus_Assume_No_Scenes = SAFE_TRUE;
		g_Menus_Assume_No_Backgrounds = SAFE_TRUE;
		g_Menus_Assume_No_Spin_Objects = SAFE_TRUE;
		g_Menus_Assume_No_Items = SAFE_TRUE;	
		
	} /* if (g_Dreaming_Module_Globals_Initialized != SAFE_TRUE) */
	
	

	function_Result = SAFE_TRUE;
	goto Normal_Function_Exit;
	
Error_Handler:
	function_Result = SAFE_FALSE;
			
Normal_Function_Exit:
	return(function_Result);
	
} /* Safe_Boolean	Initialize_Dreaming_Globals... */
/*=**************************************************************************=*/


/*=**************************************************************************=*/
Safe_Boolean Check_If_Legal_Direct_Art_Or_Complain(Puppet_Ptr	current_Puppet)
{
	Safe_Boolean				function_Result;
/* END VAR */


	function_Result = SAFE_TRUE;

	if (current_Puppet->puppet_Initialized == SAFE_TRUE) {
		if (current_Puppet->puppet_File_Is_Old_Self_Contained_Type != SAFE_TRUE) {
			function_Result = SAFE_FALSE;
			Warn_About_Unfinished_Program_And_New_Art_Unexpected();
		}
	}
	
	return(function_Result);
}
/*=**************************************************************************=*/


/*=**************************************************************************=*/
void Warn_About_Unfinished_Program_And_New_Art_Unexpected(void)
{
	Str255		error_String;
/* END VAR */

	
	Get_NS_String(k_DREAM_ERROR_STRINGS, k_ERROR_PUPPET_SPIN_FILE_TYPE_UNEXPECTED, error_String);
	Perform_OK_Dialog(error_String);
}
/*=**************************************************************************=*/


/*=**************************************************************************=*/
Safe_Boolean	Define_Puppet_Using_Spin_File(CARDINAL_32 index_Used
															, Safe_Boolean *user_Abort
															, Safe_Boolean	add_To_Existing_Puppet)
{
	INTEGER_16 		number_Of_Types;
	INTEGER_16		save_Ref;
	SFTypeList 			type_List;
	SFReply				sf_Reply;
	INTEGER_16		source_Volume_Reference;
	INTEGER_16		input_File_Reference;
	INTEGER_32		source_Directory_ID;
	OSErr					os_Error;
	Puppet_Ptr			current_Puppet;
	Handle				temp_Resource;
	INTEGER_16		index;
	CARDINAL_32	preferred_Size;
	CARDINAL_32	loop;
	SignedByte 		redirected_Info_Original_State;
	HFileParam		parameters;

	CARDINAL_8							*code_Name;
	CARDINAL_32						current_Frame;
	CARDINAL_32						file_Creation_Time;
	Redirected_Art_Ptr				loaded_Info_Ptr;
	Boolean									user_Clicked_Ok;
	Import_Options						options;
	Spin_Object_File_Header_Hdl	spin_Object_Header_Hdl;
	Safe_Boolean							function_Result;
	Safe_Boolean							resource_Sources_Swapped;
	Str255										message_String;
	Str63										file_Name;
	Str63										internal_Frozen_File_Name;
	Load_Spin_Object_Rec			params_For_Load_Spin_Object;
	
	Safe_Boolean		address_Instead_Of_File;
	CARDINAL_32	item_Address;
/* END VAR */



	code_Name = "\pLoad_PICS_Sequence";
	function_Result = SAFE_FALSE;
	*user_Abort = SAFE_FALSE;
	
	resource_Sources_Swapped = SAFE_FALSE;
	
	
	Get_NS_String(k_DREAM_STRINGS, k_SELECT_SPIN_FILE_TO_ADD_TO_DEFINITION_INDEX, message_String);
	Perform_OK_Dialog(message_String);
	
	number_Of_Types = 1;
	type_List[0] = k_SPIN_OBJECT_FILE_TYPE;
/* ====
	type_List[1] = FILE_CREATOR;
=== */

	user_Clicked_Ok = Perform_Get_File_Dialog(number_Of_Types
										, &(type_List)
										, &sf_Reply);
											
	
	if ((user_Clicked_Ok != TRUE) || (sf_Reply.good != TRUE)) {
		/*  */
		/* User Abort, no file name */
		/*  */
		*user_Abort = SAFE_TRUE;
		function_Result = SAFE_FALSE;
		goto Normal_Function_Exit;
	}
	
	Get_Last_Volume_Reference_And_Directory_ID(&source_Volume_Reference, &source_Directory_ID);
										
	/*  */
	/* Open for read only permission */
	/*  */
	
	save_Ref = CurResFile();
	
	input_File_Reference = HOpenResFile (source_Volume_Reference
						, source_Directory_ID
						, sf_Reply.fName
						, fsRdPerm );
						
	if ((input_File_Reference == -1) || (ResError() != noErr)) {
		goto Error_Handler;
	}
	
	UseResFile(input_File_Reference);
	os_Error = ResError();
	if (os_Error != noErr) {
		Display_Serious_Error_Message(unit_Name, code_Name, "\pInternal using resource", "\pCorrupted?");
		goto Error_Handler;
	}
	
	resource_Sources_Swapped = SAFE_TRUE;

	/*  */
	/* current_Puppet uses to a handle assumed to always be locked */
	/*  */
	current_Puppet = *(dreaming_Ptr->puppet[index_Used]);
	if (current_Puppet->direct_Art.frame_Total != 0 ) {
		Copy_Pascal_String("\pUNIMPLEMENTED: This version cannot append new Pict data into existing list", message_String);
		Perform_OK_Dialog(message_String);
		goto Error_Handler;
	}


	parameters.ioNamePtr = &file_Name[0];	/* buffer to get/receive name */
	Copy_Pascal_String(&(sf_Reply.fName[0]), &file_Name[0]);
	parameters.ioDirID = source_Directory_ID;
	parameters.ioVRefNum = source_Volume_Reference;
	parameters.ioCompletion = nil;
	parameters.ioFDirIndex = 0;

	os_Error = PBHGetFInfo((HParmBlkPtr)&parameters, FALSE);
	if (os_Error == noErr) {
		file_Creation_Time = parameters.ioFlCrDat;
	}
	else {
		goto Error_Handler;
	}


	Strip_Off_Popular_Suffixes(&(sf_Reply.fName));
	
	if (add_To_Existing_Puppet != SAFE_TRUE) {
		Assign_String(current_Puppet->file_Name, sf_Reply.fName);
		Assign_String(current_Puppet->screen_Name, sf_Reply.fName);
		Assign_String(current_Puppet->screen_Short_Name, sf_Reply.fName);
	}
	

	/*  */
	/* Use spin file information to create default puppet information */
	/*  */
	spin_Object_Header_Hdl = (Spin_Object_File_Header_Hdl)Get1Resource (k_SPIN_OBJECT_MAIN_RESOURCE_TYPE, k_BASE_SPIN_OBJECT_RESOURCE_ID);
	if (ResError() != noErr) {
		Display_Serious_Error_Message(unit_Name
															, code_Name
															, "\pInternal problem with resources"
															, "\pCorrupted?");
		goto Error_Handler;
	}
	
	if ((*spin_Object_Header_Hdl)->valid != SAFE_TRUE) {
		Display_Serious_Error_Message(unit_Name
															, code_Name
															, "\pInternal problem with Spin File  resource header"
															, "\pCorrupted?");
		goto Error_Handler;
	}
	
	Copy_Pascal_String(&((*spin_Object_Header_Hdl)->file_Name[0]), &(internal_Frozen_File_Name[0]));

	ReleaseResource((Handle)spin_Object_Header_Hdl);
	if (ResError() != noErr) {
		goto Error_Handler;
	}
		
		
	if (add_To_Existing_Puppet == SAFE_TRUE) {
		if (current_Puppet->puppet_File_Is_Old_Self_Contained_Type == SAFE_TRUE) {
			Display_Serious_Error_Message(unit_Name
																, code_Name
																, "\pCannot add another puppet to this old style puppet."
																, "\pRecreate using new 3D Spin files?");
			goto Error_Handler;
		}
		
		if ((current_Puppet->puppet_Initialized != SAFE_TRUE)  || (current_Puppet->redirected_Information_Memory_In_Use != MEMORY_INITIALIZED)) {
			Display_Serious_Error_Message(unit_Name
																, code_Name
																, "\pCannot add another puppet to this old style puppet, It is missing previous definitions."
																, "\pBad Programming");
			goto Error_Handler;
		}
	}
	
	if (add_To_Existing_Puppet != SAFE_TRUE) {
	
		current_Puppet->puppet_Initialized = SAFE_TRUE;
		current_Puppet->puppet_File_Is_Old_Self_Contained_Type = SAFE_FALSE;
		current_Puppet->special_Pallete_To_Use = nil;
		current_Puppet->total_Special_Colors_To_Honor = 0;
		current_Puppet->total_Colors_Used_In_Pallette = 0;
	
		current_Puppet->puppet_File_Is_Old_Self_Contained_Type = SAFE_FALSE;
		
		current_Puppet->direct_Art.port.initialized =  SAFE_FALSE; 
		current_Puppet->direct_Art.port.offscreen_GDevice_Hdl = nil;
		current_Puppet->direct_Art.port.image_Handle = nil;
	
		current_Puppet->direct_Art.frame_Total = 0;
		current_Puppet->direct_Art.talk_Total_Frames = 0;
		current_Puppet->direct_Art.talk_Start_Frame = 0;
		current_Puppet->direct_Art.talk_Stop_Frame = 0;
		current_Puppet->direct_Art.total_Lazy_Active_Region = nil;
		current_Puppet->direct_Art.original_Total_Lazy_Active_Region = nil;
		current_Puppet->direct_Art.buffered_Animation_Total_Region_Offset.h = 0;
		current_Puppet->direct_Art.buffered_Animation_Total_Region_Offset.v = 0;
		
		current_Puppet->direct_Art.frame_Locations[0].left = 0;
		current_Puppet->direct_Art.frame_Locations[0].right = 1;
		current_Puppet->direct_Art.frame_Locations[0].top = 0;
		current_Puppet->direct_Art.frame_Locations[0].bottom = 1;
		current_Puppet->direct_Art.frame_Regions[0] = nil;
		current_Puppet->direct_Art.total_Lazy_Active_Region = nil;
		current_Puppet->direct_Art.frame_Buffered_Animation_Regions[0] = nil;
		current_Puppet->direct_Art.frame_Buffered_Animation_Region_Offset[0].h = 0;
		current_Puppet->direct_Art.frame_Buffered_Animation_Region_Offset[0].v = 0;
		
		current_Puppet->redirected_Information_Hdl = nil;
		current_Puppet->redirected_Information_Ptr = nil;
		current_Puppet->redirected_Information_Memory_In_Use = MEMORY_NOT_INITIALIZED;

		current_Puppet->redirected_Information_Total_Used = 1;
		current_Puppet->current_Selection = 0;
		
		/*  */
		/* First, create the memory for the puppet to hold the file redirection */
		/* information. */
		/*  */
				
		current_Puppet->redirected_Information_Max_Size = current_Puppet->redirected_Information_Total_Used;
		/*  */
		/* Add an extra one for convenience sake in case needed for transient effect. */
		/*  */
		current_Puppet->redirected_Information_Max_Size += 1;
		
		preferred_Size = ((current_Puppet->redirected_Information_Max_Size) * sizeof(Redirected_Art_Rec));
		current_Puppet->redirected_Information_Hdl = nil;
		current_Puppet->redirected_Information_Memory_In_Use = MEMORY_NOT_INITIALIZED;
	
		if (Create_Handle(&preferred_Size
							, (CARDINAL_32)(10240L)
							, ALLOW_PURGING
							, DISALLOW_PARTIAL_AMOUNT
							, (Handle *)&current_Puppet->redirected_Information_Hdl
							, &current_Puppet->redirected_Information_Memory_In_Use) != SAFE_TRUE) {
							
			preferred_Size = ((current_Puppet->redirected_Information_Max_Size) * sizeof(Redirected_Art_Rec));
	
			Display_Low_Memory_Error_Message(unit_Name, code_Name, preferred_Size);
			function_Result = SAFE_FALSE;
			goto Error_Handler;
		}
	
		/*  */
		/* Move High and lock it. Its not so bad. Puppets used to have to load */
		/* entire mix buffer ports in the past. */
		/*  */
		if (Begin_Use_Of_Handle((Handle)current_Puppet->redirected_Information_Hdl
										, MOVE_HIGH_REQUIRED
										, &redirected_Info_Original_State) != SAFE_TRUE) {
			goto Error_Handler;
		}
		current_Puppet->redirected_Information_Ptr = (Redirected_Art_Ptr)*current_Puppet->redirected_Information_Hdl;
		
		
	} /* if (add_To_Existing_Puppet != SAFE_TRUE) ... */
	else {
	
		current_Puppet->current_Selection = current_Puppet->redirected_Information_Total_Used;
		current_Puppet->redirected_Information_Total_Used += 1;
		
		if (current_Puppet->redirected_Information_Total_Used > current_Puppet->redirected_Information_Max_Size) {
			/*  */
			/* Check if expanded larger than max and then stretch it */
			/*  */
			current_Puppet->redirected_Information_Max_Size = current_Puppet->redirected_Information_Total_Used;
			/*  */
			/* Add an extra one for convenience sake in case needed for transient effect. */
			/*  */
			current_Puppet->redirected_Information_Max_Size += 1;
			
			preferred_Size = ((current_Puppet->redirected_Information_Max_Size) * sizeof(Redirected_Art_Rec));
			
			HUnlock(current_Puppet->redirected_Information_Hdl);
			os_Error = MemError();
			if(os_Error != noErr) {
				goto Error_Handler;
			}
			
			if (Reset_Handle_Size_Preserving_Contents((Handle *)&current_Puppet->redirected_Information_Hdl
															, preferred_Size
															, &os_Error) != SAFE_TRUE) {
				goto Normal_Function_Exit;
			}
			/*  */
			/* Move High and lock it. Its not so bad. Puppets used to have to load */
			/* entire mix buffer ports in the past. */
			/*  */
			if (Begin_Use_Of_Handle((Handle)current_Puppet->redirected_Information_Hdl
											, MOVE_HIGH_REQUIRED
											, &redirected_Info_Original_State) != SAFE_TRUE) {
				goto Error_Handler;
			}
	
			current_Puppet->redirected_Information_Ptr = (Redirected_Art_Ptr)*current_Puppet->redirected_Information_Hdl;
		}
		
		
	} /* NOT  if (add_To_Existing_Puppet != SAFE_TRUE) ...  */

	loaded_Info_Ptr = &(current_Puppet->redirected_Information_Ptr[current_Puppet->redirected_Information_Total_Used - 1]);

	/*  */
	/* First, copy over file information for future "saves". */
	/*  */
/* ===
	Copy_Pascal_String(&file_Name[0], &loaded_Info_Ptr->file_Name[0]);
=== */

	Copy_Pascal_String(&internal_Frozen_File_Name[0], &loaded_Info_Ptr->file_Name[0]);
	
	loaded_Info_Ptr->file_Creation_Time = file_Creation_Time;
	
	loaded_Info_Ptr->emotion_Key_Code = 0;
	loaded_Info_Ptr->cache_Technique_Desired = 0;
	loaded_Info_Ptr->special_Effect_Classification = 0;
	loaded_Info_Ptr->special_Effect_Value = 0;
	loaded_Info_Ptr->special_Effect_Extra_Value = 0;
	loaded_Info_Ptr->offset_Point_Position.h = 0;
	loaded_Info_Ptr->offset_Point_Position.v = 0;

	/*  */
	/* Fill in information for loaded states here, including the sentinel flag. */
	/*  */
	loaded_Info_Ptr->valid = SAFE_TRUE;
	
	/*  */
	/* Image opened and initialized later in a loop process. */
	/* That way puppet header resources are shared and so are spin file */
	/* header structures. That way many scenes share RAM with less waste */
	/*  */
		
	loaded_Info_Ptr->array_Index_Valid = SAFE_FALSE;
	loaded_Info_Ptr->array_Index = 0;
			
	
	if (resource_Sources_Swapped == SAFE_TRUE) {
		CloseResFile(input_File_Reference);
		if (ResError() != noErr) {
			goto Error_Handler;
		}
		
		UseResFile(save_Ref);
		if (ResError() != noErr) {
			/*  */
			/* This should be recoverable from, enhance one day  */
			/*  */
		}
		resource_Sources_Swapped = SAFE_FALSE;
	}
	
	for (loop = 0; (loop < current_Puppet->redirected_Information_Total_Used) ; loop++) {
		loaded_Info_Ptr = &(current_Puppet->redirected_Information_Ptr[loop]);
		
		
	/* ====
		loaded_Info_Ptr->file_Name
		loaded_Info_Ptr->file_Creation_Time
		
		loaded_Info_Ptr->emotion_Key_Code
		loaded_Info_Ptr->cache_Technique_Desired
		loaded_Info_Ptr->special_Effect_Classification
		loaded_Info_Ptr->special_Effect_Value
		loaded_Info_Ptr->special_Effect_Extra_Value
		loaded_Info_Ptr->offset_Point_Position
	=== */
	
		/*  */
		/* Warning : file_Creation_Time is not used for safety. Any */
		/* first matched file is accepted. */
		/*  */
		
		if (Locate_Item_Location(k_SPIN_OBJECT_SELECTION
													, (Str63 *)&(loaded_Info_Ptr->file_Name[0])
													, &params_For_Load_Spin_Object.desired_File
													, &item_Address
													, &address_Instead_Of_File) != SAFE_TRUE) {
			/*  */
			/* Goddamn file not available. We need to skip item and bitch */
			/* Preflight should have been done elsewhere. */
			/*  */
			goto Misssing_Item_Error_Handler;
		}
		
		if (address_Instead_Of_File == SAFE_TRUE) {
		
			if (dream.spin_Objects.spin_Object_Memory_In_Use[item_Address] != MEMORY_INITIALIZED) {
				goto Error_Handler;
			}
			
			loaded_Info_Ptr->array_Index_Valid = SAFE_TRUE;
			loaded_Info_Ptr->array_Index = item_Address;

		}
		else {
			/*  */
			/* Load spin file puppet, initialize it somewhat and lock it down */
			/* Then get index to structure in preparation and store it. */
			/*  */
											
			params_For_Load_Spin_Object.async_Task_Desired = SAFE_FALSE;
			params_For_Load_Spin_Object.load_File_With_Self_Contained_Large_Ports = SAFE_FALSE;

			if (Load_Spin_Object_File_Immediate_Style(&params_For_Load_Spin_Object) != SAFE_TRUE) {
				goto Error_Handler;
			}
			else {
				loaded_Info_Ptr->array_Index_Valid = SAFE_TRUE;
				loaded_Info_Ptr->array_Index = params_For_Load_Spin_Object.desired_Spin_Object;
			}

		}
		
		/*  */
		/* Note that scenes that use this puppet will have */
		/* "scene_Must_Use_Compositing_Buffers" set for them when adding to item list */
		/*  */

	}	
					
	/*  */
	/* Something was loaded, enable all items */
	/*  */
	if (Reset_Menu_Control_Selectability() != SAFE_TRUE) {
		goto Error_Handler;
	}	
	
	function_Result = SAFE_TRUE;
	goto Normal_Function_Exit;
	
Misssing_Item_Error_Handler:
	function_Result = SAFE_FALSE;
	Display_Serious_Error_Message(unit_Name, code_Name, "\pOops! Specified Redirected Puppet in not loaded because not officially 'found'. File not in prescanned list though obvious", "\pSorry. Future version will auto update list.");

	/*  */
	/* Ignore Item. Increment to next item. Ignore if overrun */
	/*  */
	goto Normal_Function_Exit;

Error_Handler:
	Display_Serious_Error_Message(unit_Name, code_Name, "\pInternal problem using Spin file", "\p");

	function_Result = SAFE_FALSE;
		
Normal_Function_Exit:
	if (resource_Sources_Swapped == SAFE_TRUE) {
		CloseResFile(input_File_Reference);
		if (ResError() != noErr) {
			goto Error_Handler;
		}
		
		UseResFile(save_Ref);
		if (ResError() != noErr) {
			/*  */
			/* This should be recoverable from, enhance one day  */
			/*  */
		}
		resource_Sources_Swapped = SAFE_FALSE;
	}

	return(function_Result);
	
} /* Safe_Boolean	Define_Puppet_Using_Spin_File ... */
/*=**************************************************************************=*/
	
	
/*=**************************************************************************=*/
Strip_Off_Popular_Suffixes(Str63 *filename)
{
	CARDINAL_32	length;
	SIMPLE_LOOP	loop;
	INTEGER_16		index;
	Boolean				searching_For_Period;
	Boolean				still_Searching_For_Match;
	CARDINAL_32	kill_Point;
	Str255					result_String;
/* END VAR */




	/*  */
	/* Check end for last period, then extract all string after the period */
	/*  */
	kill_Point = 0;
	length = (*filename)[0];
	searching_For_Period = TRUE;
	result_String[0] = 0;
	for (loop = length; ((loop > 0) && (searching_For_Period == TRUE)); loop--) {
		if ( (*filename)[loop] == ASCII_PERIOD) {
			kill_Point = loop;
			for (index = loop; (index <= (*filename)[0]); index++) {
				result_String[0] += 1;
				result_String[result_String[0]] = (*filename)[index];
			}
			searching_For_Period = FALSE;
		}
	}
	

	/*  */
	/* Check if suffix is on list of known suffixes and kill it. */
	/*  */

	if (searching_For_Period != TRUE) {
		still_Searching_For_Match = TRUE;

		if (still_Searching_For_Match == TRUE) {
			if (Compare_Pascal_Strings_Ignoring_Case(result_String, k_SUFFIX_PICT_STR) == SAFE_TRUE) {
				if (kill_Point > 1) {
					(*filename)[0] = kill_Point - 1;
				}
				still_Searching_For_Match = FALSE;
			}
		}
		
		if (still_Searching_For_Match == TRUE) {
			if (Compare_Pascal_Strings_Ignoring_Case(result_String, k_SUFFIX_PICR_STR) == SAFE_TRUE) {
				if (kill_Point > 1) {
					(*filename)[0] = kill_Point - 1;
				}
				still_Searching_For_Match = FALSE;
			}
		}
		
		if (still_Searching_For_Match == TRUE) {
			if (Compare_Pascal_Strings_Ignoring_Case(result_String, k_SUFFIX_PUP_STR) == SAFE_TRUE) {
				if (kill_Point > 1) {
					(*filename)[0] = kill_Point - 1;
				}
				still_Searching_For_Match = FALSE;
			}
		}
		
		if (still_Searching_For_Match == TRUE) {
			if (Compare_Pascal_Strings_Ignoring_Case(result_String, k_SUFFIX_BAK_STR) == SAFE_TRUE) {
				if (kill_Point > 1) {
					(*filename)[0] = kill_Point - 1;
				}
				still_Searching_For_Match = FALSE;
			}
		}
		
		if (still_Searching_For_Match == TRUE) {
			if (Compare_Pascal_Strings_Ignoring_Case(result_String, k_SUFFIX_PICS_STR) == SAFE_TRUE) {
				if (kill_Point > 1) {
					(*filename)[0] = kill_Point - 1;
				}
				still_Searching_For_Match = FALSE;
			}
		}
		
		if (still_Searching_For_Match == TRUE) {
			if (Compare_Pascal_Strings_Ignoring_Case(result_String, k_SUFFIX_TXT_STR) == SAFE_TRUE) {
				if (kill_Point > 1) {
					(*filename)[0] = kill_Point - 1;
				}
				still_Searching_For_Match = FALSE;
			}
		}
		
		if (still_Searching_For_Match == TRUE) {
			if (Compare_Pascal_Strings_Ignoring_Case(result_String, k_SUFFIX_AIFF_STR) == SAFE_TRUE) {
				if (kill_Point > 1) {
					(*filename)[0] = kill_Point - 1;
				}
				still_Searching_For_Match = FALSE;
			}
		}
		
		

	} /* if (searching_For_Period != TRUE) */

}
/*=**************************************************************************=*/
	
	
/*=**************************************************************************=*/
Safe_Boolean	Interpret_Mouse_In_Content(Primary_Status_Hdl current_Window_Globals_Hdl
												, WindowPtr event_Window
												,EventRecord	*event_Record)
{
	Safe_Boolean			function_Result;
/* END VAR */


	function_Result = SAFE_FALSE;


	switch ((**current_Window_Globals_Hdl).type_Of_Window_For_Updates) {
	
		case ABOUT_PROGRAM_WINDOW_INDEX:
			break;
		
		case GENERAL_INFO_WINDOW_INDEX:
			break;
		
		case TEST_RECT_WINDOW_INDEX:
		case TEST_CIRCLE_INDEX:
		case TEST_FILLED_CIRCLE_INDEX:
		case TEST_ELIPSE_WINDOW_INDEX:
		case TEST_FILLED_ELIPSE_WINDOW_INDEX:
		case TEST_LINE_WINDOW_INDEX:
		case FRAMED_POLYGONS_WINDOW_INDEX:
		case TIME_LINE_SPEED_WINDOW_INDEX:
		case TEST_THREE_D_WINDOW_INDEX:
			break;

		case TEST_DREAMING_WINDOW_INDEX:
				function_Result = Interpret_Mouse_In_Scene(current_Window_Globals_Hdl
												, event_Window
												, event_Record);
	
			break;
	
		case SHOW_NETWORK_ZONES_WINDOW_INDEX:
			break;
	
		default:
		/*=--------------------------------------------------------------------------=*/
		/* !!!!!!! This is not expected, but just in case...*/
		/*=--------------------------------------------------------------------------=*/\
			;
/* ===
		SysBeep(20);
		Display_Serious_Error_Message(unit_Name, code_Name, "\pError with Interpret_Mouse_In_Content", "\pBad Programming (new case)?");
=== */
	}

	goto Normal_Function_Exit;
	
Error_Handler:
	function_Result = SAFE_FALSE;
			
Normal_Function_Exit:
	return(function_Result);
}
/*=**************************************************************************=*/
	
	
/*=**************************************************************************=*/
Safe_Boolean	Interpret_Mouse_In_Scene(Primary_Status_Hdl current_Window_Globals_Hdl
												, WindowPtr event_Window
												, EventRecord	*event_Record)
{
	Safe_Boolean		function_Result;
	Safe_Boolean		port_Was_Set;
	const BitMap *destination_Bitmap;
	GrafPtr			saved_Port;
	WindowPtr		current_Graphics_Window_Ptr;
	Boolean			found_Item;
	Boolean			frame_Changed;
	Puppet_Ptr		current_Puppet;
	CARDINAL_32	scene_To_Use;
	CARDINAL_32	located_Items;
	CARDINAL_32	total_Items;
	CARDINAL_32	item_To_Alter;
	CARDINAL_32	index;
	CARDINAL_32	item_To_Move;
	INTEGER_32		extra_Offset_X;
	INTEGER_32		extra_Offset_Y;
	Safe_Boolean		something_Changed_In_View;
	INTEGER_16		current_Frame;
	Rect				start_Rect;
	Point				oldPt, newPt;
/* END VAR */


	function_Result = SAFE_FALSE;
	port_Was_Set = SAFE_FALSE;

	/*  */
	/* Our dragging is done only when option key is held */
	/*  */
	if ((((*event_Record).modifiers) & (optionKey)) != 0) {
		
		item_To_Move = 1;
		
		if ((((*event_Record).modifiers) & (optionKey | cmdKey)) == (optionKey | cmdKey)) {
			item_To_Move = 2;
		}
		else {
			if ((((*event_Record).modifiers) & (optionKey | controlKey)) == (optionKey | controlKey)) {
				item_To_Move = 3;
			}
			else {
				if ((((*event_Record).modifiers) & (optionKey | shiftKey)) == (optionKey | shiftKey)) {
					item_To_Move = 4;
				}
			}
		}
	
	
		if (g_Dreaming_Module_Globals_Initialized != SAFE_TRUE) {
			function_Result = SAFE_TRUE;
			goto Normal_Function_Exit;
		}	
	
		if (dreaming_Ptr->play.total_Scenes < 1) {
			function_Result = SAFE_TRUE;
			goto Normal_Function_Exit;
		}
	
		scene_To_Use = dreaming_Ptr->current_Scene;
		
		if (dreaming_Ptr->play.scene_Memory_In_Use[scene_To_Use] != MEMORY_INITIALIZED) {
			goto Error_Handler;
		}
		
		if ((**(dreaming_Ptr->play.scene[scene_To_Use])).initialized != SAFE_TRUE) {
			goto Error_Handler;
		}
	
		/*  */
		/* Look for a puppet to grab */
		/*  */
		found_Item = FALSE;
		
		if ((**(dreaming_Ptr->play.scene[scene_To_Use])).scene_Port.initialized == SAFE_TRUE) {
			located_Items = 0;
			total_Items = (**(dreaming_Ptr->play.scene[scene_To_Use])).total_Items_In_Scene;
			for (index = 0; ((index < k_MAX_STAGE_ITEMS) && (located_Items < total_Items)); index++) {
				current_Puppet = (**(dreaming_Ptr->play.scene[scene_To_Use])).item[index].puppet;
		
				if (current_Puppet != nil) {
					located_Items += 1;
									
					if (item_To_Move == located_Items) {
						found_Item = TRUE;
						item_To_Alter = index;
					}
				}
			}
		}
		
		if (found_Item == TRUE) {
		
			current_Graphics_Window_Ptr	= (**active_Window_List[TEST_DREAMING_WINDOW_INDEX]).status_Window;
					
			GetPort(&saved_Port);
			Set_View_Port(current_Graphics_Window_Ptr);
			port_Was_Set = SAFE_TRUE;
					
			/*  */
			/* Actually needed for CopyBits calls , so be certain. */
			/* We do not set them back though */
			/* nor do we set pen states */
			/*  */
			ForeColor(blackColor);
			BackColor(whiteColor);

			/*  */
			/* While mouse is still held down we need to track movement of item */
			/*  */
			
			oldPt= (*event_Record).where;
			GlobalToLocal( &oldPt );
			if ( StillDown() ) {

				start_Rect = (**(dreaming_Ptr->play.scene[scene_To_Use])).item[item_To_Alter].desired_Start_Location;
			/* ===
				current_Puppet
				item_To_Alter
			=== */

				while ( WaitMouseUp() ) {
					GetMouse( &newPt);
					if ( DeltaPoint(oldPt, newPt) ) {	
					
						/*  */
						/* First remove puppet by erasing it. NOT IMPLEMENTED */
						/*  */
						
						
						/*  */
						/* Now move scene offset by amount mouse has moved */
						/*  */
						OffsetRect( &start_Rect, newPt.h-oldPt.h, newPt.v-oldPt.v );
						(**(dreaming_Ptr->play.scene[scene_To_Use])).item[item_To_Alter].desired_Start_Location = start_Rect;
					
						/*  */
						/* NOW DRAW NEW POSITION (done by drawing all )*/
						/*  */
#if (11 == 11 )
						if (Redraw_Idle_Screen_Completely(TRUE, FALSE) != SAFE_TRUE) {
							/*  */
							/* Ignore error at this stage, and let it tumble out normally */
							/*  */
						}
#else 

						if (Update_Sound_Input_Monitoring() != SAFE_TRUE) {
							/*  */
							/* Ignore errors in this implementation */
							/*  */
						}

						if (Get_Current_Volume_Frame(&((**(dreaming_Ptr->play.scene[scene_To_Use])).item[item_To_Alter])
														, &frame_Changed
														, &current_Frame) != SAFE_TRUE) {
							/*  */
							/* ignore */
							/*  */
						}
							
						current_Puppet = (**(dreaming_Ptr->play.scene[scene_To_Use])).item[item_To_Alter].puppet;
						
#if  (k_ONLY_ALLOW_DIRECT_SCREEN_ANIMATION == 1)
						extra_Offset_X = 0;
						extra_Offset_Y = 0;
						destination_Bitmap = &(current_Graphics_Window_Ptr->portBits);
#else

						if ((**(dreaming_Ptr->play.scene[scene_To_Use])).scene_Must_Use_Compositing_Buffers == SAFE_TRUE) {
							extra_Offset_X = dream.stage_View.stage_Active_View_Rect.left;
							extra_Offset_Y = dream.stage_View.stage_Active_View_Rect.top;
							destination_Bitmap = (const BitMap *)*(dream.stage_View.mix_View_Area_Port_Ptr[
												(dream.stage_View.next_Buffer_To_Use + k_MIX_AREA_BUFFER_0)]->offscreen_Port.color_Port.portPixMap);
	
							/*  */
							/* must set and restore port around "Draw_Specific_Frame_Of_Item" */
							/* (restored at function end) */
							/*  */
							Set_View_Port((GrafPtr)&(dream.stage_View.mix_View_Area_Port_Ptr[
															(dream.stage_View.next_Buffer_To_Use + k_MIX_AREA_BUFFER_0)]->offscreen_Port.color_Port));
						}
						else {
							extra_Offset_X = 0;
							extra_Offset_Y = 0;
							destination_Bitmap = &(current_Graphics_Window_Ptr->portBits);
						}

						if ((**(scene_To_Revise)).scene_Might_Contain_Movie_Ports == SAFE_TRUE) {
							if (Draw_Current_Stage_Background_Layers(&dream.stage_View
															, dream.stage_View.next_Buffer_To_Use
															, &something_Changed_In_View) != SAFE_TRUE) {
								/*  */
								/* Ignore errors in this implementation */
								/*  */
							}
						}

#endif /* NOT if (11 == 11 ) */

						if (Draw_Specific_Frame_Of_Item(destination_Bitmap
																, scene_To_Use
																, item_To_Alter
																, current_Frame
																, TOP_LEFT(((**(dreaming_Ptr->play.scene[scene_To_Use])).item[item_To_Alter].desired_Start_Location))
																, extra_Offset_X
																, extra_Offset_Y) != SAFE_TRUE) {
							/*  */
							/* Ignore errors in this implementation */
							/*  */
						}

#endif
						
						oldPt=newPt;
					}
				}
			}
		}
	}
	
	function_Result = SAFE_TRUE;
	goto Normal_Function_Exit;
	
Error_Handler:
	function_Result = SAFE_FALSE;
			
Normal_Function_Exit:
	if (port_Was_Set == SAFE_TRUE) {
		Set_View_Port(saved_Port);
	}

	return(function_Result);
}
/*=**************************************************************************=*/
	
	
/*=**************************************************************************=*/
Safe_Boolean	Interpret_Key_Command(CARDINAL_32 message, CARDINAL_32 modifiers)
{
	Safe_Boolean			function_Result;
	Select_Scene_Rec		params_For_Select_Scene;
	CARDINAL_16		key;
	Boolean				command_Found;
	CARDINAL_32		raw_Command;
/* END VAR */


	function_Result = SAFE_FALSE;
		
	key = (message & keyCodeMask);
	key = key >> 8;
	
	command_Found = TRUE;
	switch (key) {
		case k_KEYCODE_1:
			raw_Command = 1;
		break;
		
		case k_KEYCODE_2:
			raw_Command = 2;
		break;
		
		case k_KEYCODE_3:
			raw_Command = 3;
		break;
		
		case k_KEYCODE_4:
			raw_Command = 4;
		break;
		
		case k_KEYCODE_5:
			raw_Command = 5;
		break;
		
		case k_KEYCODE_6:
			raw_Command = 6;
		break;
		
		case k_KEYCODE_7:
			raw_Command = 7;
		break;
		
		case k_KEYCODE_8:
			raw_Command = 8;
		break;
		
		case k_KEYCODE_9:
			raw_Command = 9;
		break;
		
		case k_KEYCODE_0:
			raw_Command = 10;
		break;
		
		default :		
			command_Found = FALSE;
	}
	
	if (command_Found != TRUE) {
		command_Found = TRUE;
		switch (key) {

		case k_KEYCODE_ASCII_P:
			raw_Command = 11;
		break;

		case k_KEYCODE_ASCII_O:
			raw_Command = 12;
		break;

		case k_KEYCODE_ASCII_I:
			raw_Command = 13;
		break;

		case k_KEYCODE_ASCII_U:
			raw_Command = 14;
		break;

		case k_KEYCODE_ASCII_Y:
			raw_Command = 15;
		break;

		case k_KEYCODE_ASCII_T:
			raw_Command = 16;
		break;

		case k_KEYCODE_ASCII_R:
			raw_Command = 17;
		break;

		case k_KEYCODE_ASCII_E:
			raw_Command = 18;
		break;

		case k_KEYCODE_ASCII_W:
			raw_Command = 19;
		break;

		case k_KEYCODE_ASCII_Q:
			raw_Command = 20;
		break;

		case k_KEYCODE_ASCII_A:
			raw_Command = 21;
		break;

		case k_KEYCODE_ASCII_S:
			raw_Command = 22;
		break;

		case k_KEYCODE_ASCII_D:
			raw_Command = 23;
		break;

		case k_KEYCODE_ASCII_F:
			raw_Command = 24;
		break;

		case k_KEYCODE_ASCII_G:
			raw_Command = 25;
		break;

		case k_KEYCODE_ASCII_H:
			raw_Command = 26;
		break;

		case k_KEYCODE_ASCII_J:
			raw_Command = 27;
		break;

		case k_KEYCODE_ASCII_K:
			raw_Command = 28;
		break;

		case k_KEYCODE_ASCII_L:
			raw_Command = 29;
		break;

		case k_KEYCODE_ASCII_PERIOD:
			raw_Command = 30;
		break;

		case k_KEYCODE_COMMA:
			raw_Command = 31;
		break;

		case k_KEYCODE_ASCII_M:
			raw_Command = 32;
		break;

		case k_KEYCODE_ASCII_N:
			raw_Command = 33;
		break;

		case k_KEYCODE_ASCII_B:
			raw_Command = 34;
		break;

		case k_KEYCODE_ASCII_V:
			raw_Command = 35;
		break;

		case k_KEYCODE_ASCII_C:
			raw_Command = 36;
		break;

		case k_KEYCODE_ASCII_X:
			raw_Command = 37;
		break;

		case k_KEYCODE_ASCII_Z:
			raw_Command = 38;
		break;

		case k_KEYCODE_ASCII_TILDE:
		case k_KEYCODE_ASCII_ESCAPE:
			raw_Command = 99;
		break;
		
		default :		
			command_Found = FALSE;
		}
	}
	
	/*  */
	/* Numeric key choice areas */
	/*  */
	if (command_Found == TRUE) {
		/*  */
		/* This section is when the option key is not held down */
		/*  */
		if ((raw_Command >= 1) && (raw_Command <= 9) 
				&& ((modifiers & (cmdKey | controlKey)) == 0)
				&& ((modifiers & (optionKey)) != 0)) {
				
			
			if (g_Dreaming_Module_Globals_Initialized == SAFE_TRUE) {
				if (dream.play.total_Scenes > 0) {
					if (dream.play.scene_Memory_In_Use[dream.current_Scene] == MEMORY_INITIALIZED) {
						if ((**(dream.play.scene[dream.current_Scene])).initialized == SAFE_TRUE) {
							if ((**(dream.play.scene[dream.current_Scene])).items_Remaining_To_Load == 0) {

								if (Reattach_Inputs_To_Scene_Puppets(dream.current_Scene, (raw_Command - 1), TRUE) != SAFE_TRUE) {
									/*  */
									/* Ignore. No beep wanted */
									/*  */
								}
							}
						}
					}
				}
			}
			
		}
		else {
			if (((raw_Command >= 1) && (raw_Command <= 38) 
				|| (raw_Command == 99)) && ((modifiers & (cmdKey | optionKey | controlKey)) == 0)) {
				
				params_For_Select_Scene.async_Task_Desired = SAFE_FALSE;
				
				if (raw_Command == 99) {
					if ((dream.main_Active_Scene_Specified == SAFE_TRUE) && (dream.current_Scene != dream.main_Active_Scene)){
						/*  */
						/* Ignore if not working any longer because self correcting */
						/*  */
						params_For_Select_Scene.desired_Scene = dream.main_Active_Scene;
					}
					else {
						params_For_Select_Scene.desired_Scene = dream.previously_Displayed_Scene;
					}
				}
				else {
					params_For_Select_Scene.desired_Scene = (raw_Command - 1);
				}
		
				if (g_Dreaming_Module_Globals_Initialized == SAFE_TRUE) {
					if (dream.play.total_Scenes > 0) {
						if (dream.play.scene_Memory_In_Use[params_For_Select_Scene.desired_Scene] == MEMORY_INITIALIZED) {
							if ((**(dream.play.scene[params_For_Select_Scene.desired_Scene])).initialized == SAFE_TRUE) {
								if ((**(dream.play.scene[params_For_Select_Scene.desired_Scene])).items_Remaining_To_Load == 0) {
									if (Select_Scene(&params_For_Select_Scene) != SAFE_TRUE) {
										/*  */
										/* Ignore. No beep wanted */
										/*  */
									}
								}
							}
						}
					}
				}
				
			}
		
		}
	}
	else {
	
		switch (key) {
		case k_KEYCODE_ASCII_SPACE:
		
			if ((modifiers & (cmdKey | optionKey | controlKey)) != 0) {
			
				if (((modifiers & cmdKey) == 0) && ((modifiers & optionKey) != 0) && ((modifiers & controlKey) == 0)) {
					/*  */
					/* Start Recording first located SID of first character talking in  */
					/* current scene to disk file (space + option) */
					/*  */
					
					if (Start_Recording_From_First_Talking_Item_In_Current_Scene(SAFE_TRUE) != SAFE_TRUE) {
						/*  */
						/* Ignore errors in this implementation */
						/*  */
					}
				}
				
				if (((modifiers & cmdKey) == 0) && ((modifiers & optionKey) != 0) && ((modifiers & controlKey) != 0)) {
				
					/*  */
					/* Stop Recording any opennly recording files (space + option + Control) */
					/*  */
					
					if (Stop_Recording_To_Any_Open_Audio_Files() != SAFE_TRUE) {
						/*  */
						/* Ignore errors in this implementation */
						/*  */
					}
				}
				
			}
			else {
				/*  */
				/* Expand screen size view */
				/*  */

				if (g_Set_Spin_Window_Max_Monitor_Size == SAFE_TRUE) {
					if (g_Menu_Bar_Obscured == SAFE_TRUE) {
						if (Restore_Menu_Bar() != SAFE_TRUE) {
						
						}
					}
					else {
						if (Hide_Menu_Bar() != SAFE_TRUE) {
						
						}
					}
				}
			}
			break;

			default:
				; /* NULL */
		}
	}
	

	function_Result = SAFE_TRUE;
	goto Normal_Function_Exit;
	
Error_Handler:
	function_Result = SAFE_FALSE;
			
Normal_Function_Exit:
	return(function_Result);
	
} /* Safe_Boolean	Interpret_Key_Command... */
/*=**************************************************************************=*/
	
	
/*=**************************************************************************=*/
Safe_Boolean	Initialize_Backdrop_Information(void)
{
	Safe_Boolean		function_Result;
	CARDINAL_32	index;
/* END VAR */

	function_Result = SAFE_FALSE;
	
	dreaming_Ptr->backdrops.total_Backdrops = 0;
	dreaming_Ptr->backdrops.backdrops_Still_Loading = 0;
	
	for (index = 0; (index < k_MAX_ACTIVE_BACKDROPS); index++) {
		dreaming_Ptr->backdrops.backdrop[index] = nil;
		dreaming_Ptr->backdrops.backdrop_Memory_In_Use[index] = MEMORY_NOT_INITIALIZED;
	}

	function_Result = SAFE_TRUE;
	goto Normal_Function_Exit;
	
Error_Handler:
	function_Result = SAFE_FALSE;
			
Normal_Function_Exit:
	return(function_Result);
	
} /* Safe_Boolean	Initialize_Backdrop_Information... */
/*=**************************************************************************=*/
	
	
/*=**************************************************************************=*/
Safe_Boolean	Initialize_Spin_Object_Information(void)
{
	Safe_Boolean		function_Result;
	CARDINAL_32	index;
/* END VAR */

	function_Result = SAFE_FALSE;
	
	dreaming_Ptr->spin_Objects.total_Spin_Objects = 0;
	dreaming_Ptr->spin_Objects.spin_Objects_Still_Loading = 0;
	
	for (index = 0; (index < k_MAX_ACTIVE_SPIN_OBJECTS); index++) {
		dreaming_Ptr->spin_Objects.spin_Object[index] = nil;
		dreaming_Ptr->spin_Objects.spin_Object_Memory_In_Use[index] = MEMORY_NOT_INITIALIZED;
	}

	function_Result = SAFE_TRUE;
	goto Normal_Function_Exit;
	
Error_Handler:
	function_Result = SAFE_FALSE;
			
Normal_Function_Exit:
	return(function_Result);
	
} /* Safe_Boolean	Initialize_Spin_Object_Information... */
/*=**************************************************************************=*/
	
	
/*=**************************************************************************=*/
Safe_Boolean	Add_Background_To_Scene(CARDINAL_32 scene_To_Use
																			, INTEGER_16 pict_Resource_Id
																			, FSSpec		*pict_File_Info
																			, CARDINAL_32	item_Address
																			, Safe_Boolean		address_Instead_Of_File
																			, INTEGER_16 current_Stage
																			, Boolean *stages_Finished)
/*  */
/* This can load either pict resources from the current resource fork, */
/* or it can load PICT files. If a Pict file is specified, set pict_Resource_Id to */
/* "0". It can also copy from a index into a table of pre loaded PICTs */
/* and to specifiy that you would utilize the address_Instead_Of_File flag */
/* and set "pict_Resource_Id" to "0" */
/*  */
{
	GrafPtr			saved_Port;
	Safe_Boolean		function_Result;
	INTEGER_16		index;
	INTEGER_16		position;
	GrafPtr			source_Port;
	Off_Screen_Ptr 	offscreen_Ptr;
	INTEGER_8		desired_Bit_Depth;
	PixMapHandle	video_Pix_Map_Handle;
	GDHandle			video_Graphics_Device_Handle;
	Rect				background_Rect;
	Boolean			found_Location;
	Rect				source_Rect;
	CARDINAL_8		*code_Name;
/* END VAR */


	code_Name = "\pAdd_Background_To_Scene";
	function_Result = SAFE_FALSE;
	*stages_Finished = FALSE;
	
	if (g_Dreaming_Module_Globals_Initialized != SAFE_TRUE) {
		goto Error_Handler;
	}	
	
	if (dreaming_Ptr->play.total_Scenes < 1) {
		goto Error_Handler;
	}
	
	if (dreaming_Ptr->play.scene_Memory_In_Use[scene_To_Use] != MEMORY_INITIALIZED) {
		goto Error_Handler;
	}
	
	if ((**(dreaming_Ptr->play.scene[scene_To_Use])).initialized != SAFE_TRUE) {
		goto Error_Handler;
	}
			 
	/*  */
	/* create new port if needed, destroy old one if it is present */
	/* and then recreate a new one */
	/* HACK HACK HACK change soon */
	/*  */
	 
	 if (current_Stage == 0) {
		 if ((**(dreaming_Ptr->play.scene[scene_To_Use])).scene_Port.initialized == SAFE_TRUE) {
		 
		 	if (Close_Offscreen_Pict_Port(&((**(dreaming_Ptr->play.scene[scene_To_Use])).scene_Port)) != SAFE_TRUE) {
				/*  */
				/* might as well complain about it. */
				/*  */
		 		goto Error_Handler;
		 	}
		 	
		 	if ((**(dreaming_Ptr->play.scene[scene_To_Use])).scene_Special_Pallete_To_Use != nil) {
		 		DisposePalette((**(dreaming_Ptr->play.scene[scene_To_Use])).scene_Special_Pallete_To_Use);
		 	}
		 }
		 
		offscreen_Ptr = &((**(dreaming_Ptr->play.scene[scene_To_Use])).scene_Port);
		offscreen_Ptr->initialized = SAFE_FALSE;
		offscreen_Ptr->offscreen_GDevice_Hdl = nil;
		offscreen_Ptr->image_Handle = nil;
		offscreen_Ptr->color_Table_Handle = nil;
		
		offscreen_Ptr->color_Type = 3;		 
	 }
	 else if (current_Stage == 1) {

		offscreen_Ptr = &((**(dreaming_Ptr->play.scene[scene_To_Use])).scene_Port);
		
		if (g_Preferred_Monitor_Depth <=8) {
			if (k_USE_DEFAULT_CLUT_RESOURCE == 0) {
				/*  */
				/* create Palette, use CLUT of current port (set to puppets window) */
				/* This is dependant on the validity of "TEST_DREAMING_WINDOW_INDEX" */
				/*  */
				if ((**active_Window_List[TEST_DREAMING_WINDOW_INDEX]).structure_Initialized != SAFE_TRUE) {
					GetPort(&source_Port);
				}
				else {
					source_Port	= (**active_Window_List[TEST_DREAMING_WINDOW_INDEX]).status_Window;
				}
				video_Pix_Map_Handle = ((CGrafPtr)source_Port)->portPixMap;
			
				if (Create_Clut_By_Stealing_From_Color_Port(video_Pix_Map_Handle
															, &(offscreen_Ptr->color_Table_Handle)) != SAFE_TRUE) {
															
						Display_Serious_Error_Message(unit_Name
																			, code_Name
																			, "\pProblem getting CLUT ,2"
																			, "\p");
					goto Error_Handler;
				}
			}
			else {
	
				offscreen_Ptr->color_Table_Handle = GetCTable(k_DEFAULT_256_COLOR_SYSTEM_CLUT);
				if (offscreen_Ptr->color_Table_Handle == nil) {
						Display_Serious_Error_Message(unit_Name
																			, code_Name
																			, "\pProblem Loading System CLUT (2)"
																			, "\pCorrupted?");
					goto Error_Handler;
				}	
			}
		}
		else {
			if (Create_Clut_For_Direct_Mapping(g_Preferred_Monitor_Depth, &(offscreen_Ptr->color_Table_Handle)) != SAFE_TRUE) {
	 			goto Error_Handler;
			}
			
		} /* if (g_Preferred_Monitor_Depth <=8) */
				
		HLock((Handle)offscreen_Ptr->color_Table_Handle);
		if (MemError() != noErr) {
				Display_Serious_Error_Message(unit_Name
																	, code_Name
																	, "\pProblem locking handle (2)"
																	, "\pCorrupted!");
			goto Error_Handler;
		}	
		
	} /* else if (current_Stage == 1) */
	else if (current_Stage == 2) {
		/*  */
		/* At this point the other two stages have been completed */
		/* All that remains is to create the port and then */
		/* load the pict into memory. */
		/*  */
	
		desired_Bit_Depth = g_Preferred_Monitor_Depth;
		offscreen_Ptr = &((**(dreaming_Ptr->play.scene[scene_To_Use])).scene_Port);

		if (pict_Resource_Id != 0) {


			/*  */
			/* If pict resource is non-zero then we can use either */
			/* "pict_File_Info" */
			/*  */
			/*  */
			/* A pict resource from application is the source, not a PICT file */
			/*  */
			if (Load_Pict_Into_Color_Bit_Map(pict_Resource_Id
																, desired_Bit_Depth
																, offscreen_Ptr->color_Table_Handle
																, (CGrafPtr)&(offscreen_Ptr->offscreen_Port.color_Port)
																, &(offscreen_Ptr->offscreen_GDevice_Hdl)
																, &(offscreen_Ptr->image_Handle)) != SAFE_TRUE) {
														
				Display_Serious_Error_Message(unit_Name
																	, code_Name
																	, "\pProblem Loading pict resources (Memory?) ,2"
																	, "\pMemory not available?");
				goto Error_Handler;
														
			}
		}
		else {
			
			if (address_Instead_Of_File == SAFE_TRUE) {
			
				if (dream.backdrops.total_Backdrops < 1) {
					goto Error_Handler;
				}
				
				 if ((**(dream.backdrops.backdrop[item_Address])).special_Effect_Backdrop == 0) {
					 if ((**(dream.backdrops.backdrop[item_Address])).backdrop_Port.initialized == SAFE_TRUE) {

						GetPort(&saved_Port);
						source_Rect = (**(dream.backdrops.backdrop[item_Address])).backdrop_Background_Rect;
						
						Set_View_Port((GrafPtr)&((**(dream.backdrops.backdrop[item_Address])).backdrop_Port.offscreen_Port));
						
						/*  */
						/* A pict resource from application is the source, not a PICT file */
						/*  */
						if (Copy_Rect_Into_Color_Bit_Map(&source_Rect
																, desired_Bit_Depth
																, offscreen_Ptr->color_Table_Handle
																, (CGrafPtr)&(offscreen_Ptr->offscreen_Port.color_Port)
																, &(offscreen_Ptr->offscreen_GDevice_Hdl)
																, &(offscreen_Ptr->image_Handle)) != SAFE_TRUE) {
																	
							Display_Serious_Error_Message(unit_Name
																, code_Name
																, "\pProblem Loading pict resources (Memory?) ,3"
																, "\pMemory not available?");
							goto Error_Handler;
																	
						}
						
						Set_View_Port(saved_Port);
						
						/*  */
						/* Cool! We copied it and it is from a shared backdrop */
						/*  */
						(**(dreaming_Ptr->play.scene[scene_To_Use])).backdrop_Index_Valid = SAFE_TRUE;
						(**(dreaming_Ptr->play.scene[scene_To_Use])).backdrop_Index = item_Address;
					 }
					 else {
						Display_Serious_Error_Message(unit_Name
															, code_Name
															, "\pProblem Loading Pict effect."
															, "\pReferenced backdrop is corrupt (1). Naughty programmer.");
						goto Error_Handler;
					 }
				}
				else {
					Display_Serious_Error_Message(unit_Name
														, code_Name
														, "\pProblem Loading Pict effect."
														, "\pEfffects not supported in version yet. Bad file for this version");
					goto Error_Handler;
				}
										
			}
			else {
				pict_Resource_Id = 0;
				if (Load_Raw_Pict_Fork_From_Backdrop(pict_File_Info
																	, desired_Bit_Depth
																	, offscreen_Ptr->color_Table_Handle
																	, (CGrafPtr)&(offscreen_Ptr->offscreen_Port.color_Port)
																	, &(offscreen_Ptr->offscreen_GDevice_Hdl)
																	, &(offscreen_Ptr->image_Handle)) != SAFE_TRUE) {
															
					if (pict_Resource_Id == 0) {
						/*  */
						/* Failure. Load default backdrop (not a PICT file?) */
						/*  */
						pict_Resource_Id = k_BASE_BACKDROP_RESOURCE_ID;
						if (Load_Pict_Into_Color_Bit_Map(pict_Resource_Id
																	, desired_Bit_Depth
																	, offscreen_Ptr->color_Table_Handle
																	, (CGrafPtr)&(offscreen_Ptr->offscreen_Port.color_Port)
																	, &(offscreen_Ptr->offscreen_GDevice_Hdl)
																	, &(offscreen_Ptr->image_Handle)) != SAFE_TRUE) {
																	
							Display_Serious_Error_Message(unit_Name
																, code_Name
																, "\pProblem Loading pict resources (Memory?) ,2"
																, "\pMemory not available?");
							goto Error_Handler;
						}			
					}
					else {
						goto Error_Handler;
					}
				}
				
			} /* if (address_Instead_Of_File == SAFE_TRUE) */
		}

			
		SetRect(&background_Rect, 0, 0, g_Window_Limits.specific.display_Item_Width_Max, g_Window_Limits.specific.display_Item_Height_Max);
		(**(dreaming_Ptr->play.scene[scene_To_Use])).scene_Background_Rect = background_Rect;
	
		offscreen_Ptr->initialized = SAFE_TRUE;
		offscreen_Ptr->color_Type = k_COLOR_QUICKDRAW;
		
	}  /* else if (current_Stage == 2) */
	else {
		 
		/*  */
		/* There are no more items, end this loop */
		/*  */
		*stages_Finished = TRUE;
	
	} /* NOT else if (current_Stage == 1) */
	
	function_Result = SAFE_TRUE;
	goto Normal_Function_Exit;
	
Error_Handler:
	function_Result = SAFE_FALSE;
	Display_Serious_Error_Message(unit_Name, code_Name, "\punexpected problem (1)", "\pMemory?");

Normal_Function_Exit:

	return(function_Result);

} /* Safe_Boolean	Add_Background_To_Scene(CARDINAL_32 scene_To_Use... */
/*=**************************************************************************=*/
	
	
/*=**************************************************************************=*/
Safe_Boolean	Add_Background_Of_Scene_To_Puppets(CARDINAL_32 scene_To_Use
																			, INTEGER_16 current_Stage
																			, Boolean *stages_Finished) 
/*  */
/* This fills in the area around a puppets frames to be equal to the new location */
/* it has been moved into or is appearing in. It fills in most of the frames */
/*  */
{
	Point				position_In_Scene_Offset;
	Stage_Item_Ptr	item_Ptr;
	Off_Screen_Port_Rec *copy_Back_Port_Ptr;
	Rect					*copy_Back_Port_Rect_Ptr;
	INTEGER_32		current_frame_Offset;
	INTEGER_16		this_Final_Stage_Loop;
	INTEGER_16		total_Items;
	INTEGER_16		position;
	INTEGER_16		backdrop_Location;
	Scene_Handle	scene_To_Revise;
	Safe_Boolean		function_Result;
	CARDINAL_8		*code_Name;
/* END VAR */


	code_Name = "\pAdd_Background_To_Scene";
	function_Result = SAFE_FALSE;
	*stages_Finished = FALSE;
	
	if (g_Dreaming_Module_Globals_Initialized != SAFE_TRUE) {
		goto Error_Handler;
	}	
	
	if (dreaming_Ptr->play.total_Scenes < 1) {
		goto Error_Handler;
	}
	
	if (dreaming_Ptr->play.scene_Memory_In_Use[scene_To_Use] != MEMORY_INITIALIZED) {
		goto Error_Handler;
	}
	
	scene_To_Revise = dreaming_Ptr->play.scene[scene_To_Use];
	
	if ((**(scene_To_Revise)).initialized != SAFE_TRUE) {
		goto Error_Handler;
	}
	

	/*  */
	/* Hurray, the picture is loaded! Now revise the puppet backgrounds */
	/*  */
	this_Final_Stage_Loop = 0;
	
	if (current_Stage < this_Final_Stage_Loop) {
		goto Error_Handler;
	}		
	
	total_Items = (**(scene_To_Revise)).total_Items_In_Scene;
	if (total_Items > k_MAX_STAGE_ITEMS) {
		total_Items = k_MAX_STAGE_ITEMS;
	}
	
	/*  */
	/* Check if we just got set to first part of sequence */
	/* If so, reset main counter and first item (if any) */
	/*  */
	if ((current_Stage - this_Final_Stage_Loop) == 0) {
		(**(scene_To_Revise)).scrap_Index_Global_Counter = 0;
		position = 0;
		if (total_Items > 0) {
			((**(scene_To_Revise)).item[position]).scrap_Loop_Global_Counter = 0;
		}
	}
	else {
		position = (**(scene_To_Revise)).scrap_Index_Global_Counter;
	}
	
	if (position < total_Items) {
		/*  */
		/* NOTICE: This assumes it is already locked down */
		/*  */
		item_Ptr = &((**(scene_To_Revise)).item[position]);

		current_frame_Offset = item_Ptr->scrap_Loop_Global_Counter;
		
		if ((item_Ptr->valid_Pointer != SAFE_TRUE) 
				|| (item_Ptr->puppet->puppet_Initialized != SAFE_TRUE)) {
			Display_Serious_Error_Message(unit_Name, code_Name, "\punexpected problem (1.1)", "\pLooped too many");
			goto Error_Handler;
		}
		
		if (item_Ptr->puppet->puppet_File_Is_Old_Self_Contained_Type == SAFE_TRUE) {
			if (current_frame_Offset > (item_Ptr->puppet->direct_Art.talk_Stop_Frame - item_Ptr->puppet->direct_Art.talk_Start_Frame)) {
				/*  */
				/* Increment if possible, else go to next, else finish up */
				/*  */
				item_Ptr->scrap_Loop_Global_Counter = 0;
				(**(scene_To_Revise)).scrap_Index_Global_Counter += 1;
				((**(scene_To_Revise)).item[((**(scene_To_Revise)).scrap_Index_Global_Counter)]).scrap_Loop_Global_Counter = 0;
				
				if (((**(scene_To_Revise)).scrap_Index_Global_Counter) >= total_Items) {
					(**(scene_To_Revise)).scrap_Index_Global_Counter = 0;
					*stages_Finished = TRUE;
				}
				else {
					position = (**(scene_To_Revise)).scrap_Index_Global_Counter;
					item_Ptr = &((**(scene_To_Revise)).item[position]);
					
					if ((item_Ptr->valid_Pointer != SAFE_TRUE) 
							|| (item_Ptr->puppet->puppet_Initialized != SAFE_TRUE)) {
						Display_Serious_Error_Message(unit_Name, code_Name, "\punexpected problem (1.3)", "\pLooped too many");
						goto Error_Handler;
					}
					
					/* 1994.06.10 JAM */
					current_frame_Offset = item_Ptr->scrap_Loop_Global_Counter;
					
				}
			}
			
			if (*stages_Finished != TRUE) {
				current_frame_Offset += item_Ptr->puppet->direct_Art.talk_Start_Frame;
				
				if (current_frame_Offset > item_Ptr->puppet->direct_Art.talk_Stop_Frame) {
					*stages_Finished = TRUE;
					Display_Serious_Error_Message(unit_Name, code_Name, "\punexpected problem (1.4)", "\pLooped too many");
					goto Error_Handler;
				}
				else {
				
					if ((**(scene_To_Revise)).scene_Port.initialized != SAFE_TRUE) {
						Display_Serious_Error_Message(unit_Name, code_Name, "\punexpected problem", "\pPort not available");
					}	
					
				/* ==
					position_In_Scene_Offset = TOP_LEFT((**(scene_To_Revise)).item[index_Into_Scene].desired_Start_Location);
				=== */
					position_In_Scene_Offset = TOP_LEFT(item_Ptr->desired_Start_Location);
									
					if ((**scene_To_Revise).backdrop_Index_Valid == SAFE_TRUE) {
						backdrop_Location = (**scene_To_Revise).backdrop_Index;
											
						if ((dream.backdrops.backdrop_Memory_In_Use[backdrop_Location]) == SAFE_TRUE){
							copy_Back_Port_Ptr = &((**(dream.backdrops.backdrop[backdrop_Location])).backdrop_Port);
							copy_Back_Port_Rect_Ptr = &((**(dream.backdrops.backdrop[backdrop_Location])).backdrop_Background_Rect);
						}
						else {
							/*  */
							/* Impossible screwup! Scream? No. Just ignore backdrop */
							/*  */
							copy_Back_Port_Ptr = &((**scene_To_Revise).scene_Port);
							copy_Back_Port_Rect_Ptr = &((**scene_To_Revise).scene_Background_Rect);
						}
					}
					else {
						copy_Back_Port_Ptr = &((**scene_To_Revise).scene_Port);
						copy_Back_Port_Rect_Ptr = &((**scene_To_Revise).scene_Background_Rect);
					}
					
					if (Repaint_Background_Portion(item_Ptr->puppet
							, current_frame_Offset
							, copy_Back_Port_Ptr
							, &position_In_Scene_Offset
							, copy_Back_Port_Rect_Ptr) != SAFE_TRUE) {
							
						/*  */
						/* Ignore error in this version */
						/*  */
					}
					
					
				}
	
				item_Ptr->scrap_Loop_Global_Counter += 1;
			}
		} /* if (item_Ptr->puppet->puppet_File_Is_Old_Self_Contained_Type == SAFE_TRUE) */
		else {
			/*  */
			/* 1995.06.13 JAM Enhanced now that new 3D art is used. */
			/*  */
			*stages_Finished = TRUE;
		}
		

	}
	else {
		/*  */
		/* There are no more items, end this loop */
		/*  */
		*stages_Finished = TRUE;
	}
	
	
		
	function_Result = SAFE_TRUE;
	goto Normal_Function_Exit;
	
Error_Handler:
	function_Result = SAFE_FALSE;
	Display_Serious_Error_Message(unit_Name, code_Name, "\punexpected problem (1)", "\pMemory?");

Normal_Function_Exit:

	return(function_Result);

} /* Safe_Boolean	Add_Background_Of_Scene_To_Puppets(CARDINAL_32 scene_To_Use... */
/*=**************************************************************************=*/
	
	
/*=**************************************************************************=*/
Safe_Boolean Repair_Region_Defect(RgnHandle total_Lazy_Active_Region, Puppet_Ptr current_Puppet)
/*  */
/* Written to fix a bug in old versions of program */
/* with embedded backgrounds*/
/*  */
{
	Safe_Boolean		function_Result;
	RgnHandle 		first_Region;
	SIMPLE_LOOP	loop;
/* END VAR */


	function_Result = SAFE_TRUE;
	
	if (current_Puppet->puppet_File_Is_Old_Self_Contained_Type == SAFE_TRUE) {
	
		first_Region = current_Puppet->direct_Art.frame_Regions[0];
		
		if ((first_Region != nil) && (total_Lazy_Active_Region != nil)) {
			if ((**first_Region).rgnSize == 10) {
				if (((**first_Region).rgnSize == (**total_Lazy_Active_Region).rgnSize) 
						|| (((**total_Lazy_Active_Region).rgnBBox.bottom - (**total_Lazy_Active_Region).rgnBBox.top) > (g_Window_Limits.specific.display_Item_Height_Max - 30L))){
						
					/*  */
					/* Damned regions are simple rects! Maybe these are the kind */
					/* that were problematic and oversized, if tall */
					/*  */
					if (((**total_Lazy_Active_Region).rgnBBox.bottom - (**total_Lazy_Active_Region).rgnBBox.top) > (g_Window_Limits.specific.display_Item_Height_Max * 2L)) {
					
						CopyRgn(first_Region, total_Lazy_Active_Region);
										
						for (loop = 1; (loop < current_Puppet->direct_Art.frame_Total) ; loop++) {
							CopyRgn(first_Region, current_Puppet->direct_Art.frame_Regions[loop]);
						}
					}
				}
			}
		}
	}
	
	function_Result = SAFE_TRUE;
	return(function_Result);
	
} /* Safe_Boolean Repair_Region_Defect */
/*=**************************************************************************=*/		

		
/*=**************************************************************************=*/
Safe_Boolean Compact_Scene_Item_List(CARDINAL_32 scene_To_Use)
/*  */
/* If order is unimportant we could just swap last item for first */
/* located gap index and 99% of time this would be optimal speed. */
/* This does not do that optimization step yet though. All */
/* data is repacked in one pass though. */
/*  */
{
	CARDINAL_32		total_Items_To_Search;
	CARDINAL_32		total_Items_Found;
	CARDINAL_32		index;
	CARDINAL_32		empty_Slot;
	SIMPLE_LOOP		loop;
	Scene_Ptr				active_Scene_Ptr;
	Safe_Boolean			function_Result;
	Boolean					empty_Slot_Found;
/* END VAR */


	active_Scene_Ptr = *(dream.play.scene[scene_To_Use]);
	if (active_Scene_Ptr->initialized != SAFE_TRUE) {
		goto Error_Handler;
	}
		
	if (active_Scene_Ptr->total_Items_In_Scene != 0) {
		total_Items_To_Search = active_Scene_Ptr->total_Items_In_Scene;
		total_Items_Found = 0;
		empty_Slot_Found = FALSE;

		for (index = 0; ((index < k_MAX_STAGE_ITEMS) && (total_Items_Found < total_Items_To_Search)); index++) {
			if ((active_Scene_Ptr->item[index].valid_Pointer == SAFE_TRUE)
				&& (active_Scene_Ptr->item[index].puppet != nil)) {

				total_Items_Found += 1;
				
				if (empty_Slot_Found == TRUE) {
					/*  */
					/* Transfer located data down into empty slot marker */
					/*  */
					active_Scene_Ptr->item[empty_Slot] = active_Scene_Ptr->item[index];
					
					active_Scene_Ptr->item[index].valid_Pointer = SAFE_FALSE;
					active_Scene_Ptr->item[index].puppet = nil;
					active_Scene_Ptr->item[index].desired_File_Loading_Now = SAFE_FALSE;
					
					/*  */
					/* These are used for determining Spinfile caching info */
					/*  */
					active_Scene_Ptr->item[index].last_Image.valid_Information = SAFE_FALSE;
					active_Scene_Ptr->item[index].last_Image.avoidance_Image_Index = k_BOGUS_INDEX_FRAME;
					
					for (loop = 0; (loop < (k_BUFFERS_REQUIRED - k_BASE_OFFSET_FOR_MIX)); loop++) {
						active_Scene_Ptr->item[index].frame_Needing_Erasure[loop].valid_Information = SAFE_FALSE;
						active_Scene_Ptr->item[index].current_Displayed_Frame[loop].intensity = 0xFFFFFFFF;
						active_Scene_Ptr->item[index].current_Displayed_Frame[loop].horizontal_Angle = 0;
						active_Scene_Ptr->item[index].current_Displayed_Frame[loop].vertical_Angle = 0;
						active_Scene_Ptr->item[index].current_Displayed_Frame[loop].roll_Angle = 0;
					}
					
					empty_Slot += 1;
				}
			}
			else {
				if (empty_Slot_Found != TRUE) {
					empty_Slot_Found = TRUE;
					empty_Slot = index;
				}
			}
		}
	}
	
	
	function_Result = SAFE_TRUE;
	goto Normal_Function_Exit;
	
Error_Handler:
	function_Result = SAFE_FALSE;

Normal_Function_Exit:
	return(function_Result);
	
}
/*=**************************************************************************=*/		

		
/*=**************************************************************************=*/
Safe_Boolean Compact_Puppet_List(void)
/*  */
/* If order is unimportant we could just swap last item for first */
/* located gap index and 99% of time this would be optimal speed. */
/* This does not do that optimization step yet though. All */
/* data is repacked in one pass though. */
/*  */
{
	CARDINAL_32		total_Items_To_Search;
	CARDINAL_32		total_Items_Found;
	CARDINAL_32		index;
	CARDINAL_32		empty_Slot;
	Safe_Boolean			function_Result;
	Boolean					empty_Slot_Found;
/* END VAR */

		
	if (dream.active_Puppets_Total != 0) {
		total_Items_To_Search = dream.active_Puppets_Total;
		total_Items_Found = 0;
		empty_Slot_Found = FALSE;

		for (index = 0; ((index < k_MAX_STAGE_ITEMS) && (total_Items_Found < total_Items_To_Search)); index++) {
			if ((dream.puppet_Memory_Initialized[index] == MEMORY_INITIALIZED)
				&& (dream.puppet[index] != nil)) {

				total_Items_Found += 1;
				
				if (empty_Slot_Found == TRUE) {
					/*  */
					/* Transfer located data down into empty slot marker */
					/*  */
					dream.puppet_Memory_Initialized[empty_Slot] = dream.puppet_Memory_Initialized[index];
					dream.puppet[empty_Slot] = dream.puppet[index];
					
					dream.puppet_Memory_Initialized[index]= MEMORY_NOT_INITIALIZED;
					dream.puppet[index] = nil;
					
					empty_Slot += 1;
				}
			}
			else {
				if (empty_Slot_Found != TRUE) {
					empty_Slot_Found = TRUE;
					empty_Slot = index;
				}
			}
		}
	}
	
	function_Result = SAFE_TRUE;
	goto Normal_Function_Exit;
	
Error_Handler:
	function_Result = SAFE_FALSE;

Normal_Function_Exit:
	return(function_Result);
	
}
/*=**************************************************************************=*/
	
	
/*=**************************************************************************=*/
Safe_Boolean Find_Empty_Position_In_Scene(CARDINAL_32 scene_To_Use
																, INTEGER_16 *position)
{
	Boolean				found;
	SIMPLE_LOOP	loop;
	Scene_Ptr			scene;
	Safe_Boolean		function_Result;
/* END VAR */

	*position = 0;

	scene = *(dreaming_Ptr->play.scene[scene_To_Use]);
	
	found = FALSE;
	if ((*scene).total_Items_In_Scene > 0) {
		for (loop = 0; ((loop < k_MAX_STAGE_ITEMS) && (found != TRUE)); loop++) {
			if ((*scene).item[loop].valid_Pointer != SAFE_TRUE) {
				if ((*scene).item[loop].desired_File_Loading_Now != SAFE_TRUE) {
					found = TRUE;
					*position = loop;					
				}
			}
		}
	}
	else {
		found = TRUE;
		*position = 0;
	}
	
	if (found != TRUE) {
		function_Result = SAFE_FALSE;
	}
	else {
		function_Result = SAFE_TRUE;
	}
	
	return(function_Result);

} /* Safe_Boolean Find_Empty_Position_In_Scene ... */
/*=**************************************************************************=*/
	
	
/*=**************************************************************************=*/
Safe_Boolean Find_Empty_Position_For_Scene(INTEGER_16 *position)
{
	Boolean			found;
	INTEGER_16		index;
	Safe_Boolean		function_Result;
/* END VAR */

	*position = 0;

	found = FALSE;
	if (g_Dreaming_Module_Globals_Initialized == SAFE_TRUE) {
		for (index = 0; ((index < k_MAX_ACTIVE_SCENES) && (found != TRUE)); index++) {
			if (dream.play.scene_Memory_In_Use[index] != MEMORY_INITIALIZED) {
				
				found = TRUE;
				*position = index;					
			}
		}
	}

	if (found != TRUE) {
		function_Result = SAFE_FALSE;
	}
	else {
		function_Result = SAFE_TRUE;
	}
	
	return(function_Result);

} /* Safe_Boolean Find_Empty_Position_In_Scene ... */
/*=**************************************************************************=*/
	
	
/*=**************************************************************************=*/
Safe_Boolean Find_Empty_Position_For_Puppet(INTEGER_16 *position)
{
	Boolean				found;
	SIMPLE_LOOP	loop;
	Puppet_Ptr			current_Puppet;
	Safe_Boolean		function_Result;
/* END VAR */

	*position = 0;
	
	found = FALSE;
	if (dreaming_Ptr->active_Puppets_Total != 0) {
		for (loop = 0; ((loop < k_MAXIMUM_PUPPETS) && (found != TRUE)); loop++) {			
			if (dreaming_Ptr->puppet_Memory_Initialized[loop] != MEMORY_INITIALIZED) {
				found = TRUE;
				*position = loop;
			}
		}
	}
	else {
		found = TRUE;
		*position = 0;
	}
	
	if (found != TRUE) {
		function_Result = SAFE_FALSE;
	}
	else {
		function_Result = SAFE_TRUE;
	}
	
	return(function_Result);

} /* Safe_Boolean Find_Empty_Position_For_Puppet ... */
/*=**************************************************************************=*/
	
	
/*=**************************************************************************=*/
Safe_Boolean Find_Puppet_Position_In_Scene(CARDINAL_32 scene_To_Use
																, CARDINAL_16 puppet
																, INTEGER_16 *position)
{
	Boolean			found;
	SIMPLE_LOOP	loop;
	INTEGER_16		items_Found;
	Scene_Ptr			scene;
	CARDINAL_32	total_Scenes;
	CARDINAL_32	total_To_Search;
	Puppet_Ptr		address_Of_Desired;
	Safe_Boolean		function_Result;
/* END VAR */


	address_Of_Desired = (*dreaming_Ptr->puppet[puppet]);
	*position = 0;

	scene = *(dreaming_Ptr->play.scene[scene_To_Use]);
	
	found = FALSE;
	if ((*scene).total_Items_In_Scene > 0) {
		items_Found = 0;
		total_To_Search = (*scene).total_Items_In_Scene;
		
		for (loop = 0; (((loop < k_MAX_STAGE_ITEMS) && (found != TRUE) && (items_Found < total_To_Search))); loop++) {
			if ((*scene).item[loop].valid_Pointer == SAFE_TRUE) {
				items_Found += 1;
				
				if ((*scene).item[loop].puppet == address_Of_Desired) {
					found = TRUE;
					*position = loop;
				}
			}
		}
	}
	
	if (found != TRUE) {
		function_Result = SAFE_FALSE;
	}
	else {
		function_Result = SAFE_TRUE;
	}
	
	return(function_Result);

} /* Safe_Boolean Find_Puppet_Position_In_Scene ... */
/*=**************************************************************************=*/
	
	
/*=**************************************************************************=*/
Safe_Boolean Find_Puppet_Index_Using_Address(Puppet_Ptr	address_Of_Desired
																, INTEGER_16 *position)
{
	Boolean			found;
	SIMPLE_LOOP	loop;
	CARDINAL_32	total_To_Search;
	CARDINAL_32	items_Found;
	Safe_Boolean		function_Result;
/* END VAR */


	*position = 0;
	found = FALSE;
	
	total_To_Search = dreaming_Ptr->active_Puppets_Total;
	if (total_To_Search != 0) {
		items_Found = 0;
		for (loop = 0; ((loop < k_MAXIMUM_PUPPETS) && (found != TRUE) && (items_Found < total_To_Search)); loop++) {			
			if (dreaming_Ptr->puppet_Memory_Initialized[loop] == MEMORY_INITIALIZED) {
				items_Found += 1;
				
				if (*(dreaming_Ptr->puppet[loop]) == address_Of_Desired) {
					found = TRUE;
					*position = loop;
				}
			}
		}
	}
	
	if (found != TRUE) {
		function_Result = SAFE_FALSE;
	}
	else {
		function_Result = SAFE_TRUE;
	}
	
	return(function_Result);

} /* Safe_Boolean Find_Puppet_Index_From_Its_Address ... */
/*=**************************************************************************=*/
	
	
/*=**************************************************************************=*/
Safe_Boolean Determine_Item_Needed_Elsewhere(CARDINAL_32 scene_To_Ignore
				, CARDINAL_16 item_Index
				, CARDINAL_32	*actual_Puppet_Index)
/*  */
/* This routine is not allowed to fail by design. Redesign usage */
/* if actual failure needed */
/*  */
{
	INTEGER_16		position;
	Scene_Ptr			scene;
	INTEGER_32		scene_Index;
	CARDINAL_32	items_Found;
	CARDINAL_32	loop;
	CARDINAL_32	total_To_Search;
	Safe_Boolean		function_Result;
	INTEGER_16		scenes_Found;
	Boolean			something_Located;
	CARDINAL_8		*code_Name;
/* END VAR */


	code_Name = "\pDetermine_Item_Needed_Elsewhere";
	function_Result = SAFE_FALSE;
	something_Located = FALSE;
	
	*actual_Puppet_Index = 0;

	if (dream.play.scene_Memory_In_Use[scene_To_Ignore] == MEMORY_INITIALIZED) {
		
		if ((**dream.play.scene[scene_To_Ignore]).item[item_Index].valid_Pointer == SAFE_TRUE) {
		
			/*  */
			/* Before looking in other scenes check if this puppet is */
			/* only used only once in this scene first */
			/*  */
		
			scene = *(dreaming_Ptr->play.scene[scene_To_Ignore]);
			
			items_Found = 0;
			total_To_Search = (*scene).total_Items_In_Scene;
			
			for (loop = 0; (((loop < k_MAX_STAGE_ITEMS) && (something_Located != TRUE) && (items_Found < total_To_Search))); loop++) {
				if ((*scene).item[loop].valid_Pointer == SAFE_TRUE) {
					items_Found += 1;
					
					if (loop != item_Index) {
						if ((*scene).item[loop].puppet == (*scene).item[item_Index].puppet) {
							/*  */
							/* This puppet is used more than one time in this scene. */
							/*  */
							something_Located = TRUE;
						}
					}
				}
			}
		
			/*  */
			/* Now check if in other scenes. */
			/*  */
			if (something_Located != TRUE) {
				if (Find_Puppet_Index_Using_Address((**dream.play.scene[scene_To_Ignore]).item[item_Index].puppet, &position) == SAFE_TRUE) {
					
					/*  */
					/* Convert "item_Index" into actual puppet reference */
					/*  */
					item_Index = position;
					
					*actual_Puppet_Index = item_Index;
					
					if (g_Dreaming_Module_Globals_Initialized == SAFE_TRUE) {
						scenes_Found = 0;
						total_To_Search = dream.play.total_Scenes;
						for (scene_Index = 0; ((scene_Index < k_MAX_ACTIVE_SCENES) 
								&& (scenes_Found < total_To_Search) 
								&& (something_Located != TRUE)); scene_Index++) {
								
							if (dream.play.scene_Memory_In_Use[scene_Index] == MEMORY_INITIALIZED) {
								scenes_Found += 1;
								
								if (scene_Index != scene_To_Ignore) {
								
									if (Find_Puppet_Position_In_Scene(scene_Index
																			, item_Index
																			, &position) == SAFE_TRUE) {
																			
										something_Located = TRUE;
									}
								}
							}
						}
					}
				}
				else {
					/*  */
					/* Bitch, invalid structure*/
					/*  */
					Display_Serious_Error_Message(unit_Name, code_Name, "\pInvalid scene structure (or duplicate in scene)", "\pFind_Puppet_Index_Using_Address did not work (2)");
				}
			} /* if (something_Located != TRUE) */
		}
		else {
			/*  */
			/* Bitch, invalid structure*/
			/*  */
			Display_Serious_Error_Message(unit_Name, code_Name, "\pInvalid scene structure", "\pNaughty (1)");
		}
	}


	if (something_Located == TRUE) {
		function_Result = SAFE_TRUE;
	}

	return(function_Result);
	
} /* Safe_Boolean Determine_Item_Needed_Elsewhere ... */
/*=**************************************************************************=*/
	
	
/*=**************************************************************************=*/
Safe_Boolean Determine_Backdrop_In_Another_Scene(CARDINAL_32 scene_To_Ignore, CARDINAL_32	match_Index)
{
	INTEGER_16		position;
	CARDINAL_32	total_To_Search;
	INTEGER_32		scene_Index;
	Boolean			need_To_Hunt_For_Backdrop;
	Safe_Boolean		function_Result;
	INTEGER_16		scenes_Found;
	Boolean			something_Located;
/* END VAR */


	function_Result = SAFE_FALSE;
	something_Located = FALSE;
	
	match_Index = 0;
	
	if (dream.play.scene_Memory_In_Use[scene_To_Ignore] == MEMORY_INITIALIZED) {
		need_To_Hunt_For_Backdrop = TRUE;
	}
	else {
		need_To_Hunt_For_Backdrop = FALSE;
	}
	
	if (need_To_Hunt_For_Backdrop == TRUE) {
		if (g_Dreaming_Module_Globals_Initialized == SAFE_TRUE) {
			scenes_Found = 0;
			total_To_Search = dream.play.total_Scenes;
			for (scene_Index = 0; ((scene_Index < k_MAX_ACTIVE_SCENES) 
					&& (scenes_Found < total_To_Search) 
					&& (something_Located != TRUE)); scene_Index++) {
					
				if (dream.play.scene_Memory_In_Use[scene_Index] == MEMORY_INITIALIZED) {
					scenes_Found += 1;
					
					if (scene_Index != scene_To_Ignore) {
						if ((**(dream.play.scene[scene_Index])).backdrop_Selected == SAFE_TRUE) {
							if ((**(dream.play.scene[scene_Index])).backdrop_Index_Valid == SAFE_TRUE) {
								if ((**(dream.play.scene[scene_Index])).backdrop_Index == match_Index) {
								
									something_Located = TRUE;
								}
							}
						}
					}
				}
			}
		}
	}

	if (something_Located == TRUE) {
		function_Result = SAFE_TRUE;
	}

	return(function_Result);

} /* Safe_Boolean Determine_Backdrop_In_Another_Scene ... */
/*=**************************************************************************=*/


/*=**************************************************************************=*/
Safe_Boolean	Add_Puppet_Sequence_To_Scene(CARDINAL_32 scene_To_Use, CARDINAL_16 puppet, CARDINAL_16	position)
{
	CARDINAL_32	max_Items;
	CARDINAL_32	total_Items_Seen;
	Safe_Boolean		function_Result;
	INTEGER_16		index;
	SIMPLE_LOOP	loop;
	CARDINAL_8		*code_Name;
	Stage_Item_Ptr	item_Ptr;
	Boolean				found_Location;
/* END VAR */


	code_Name = "\pAdd_Puppet_Sequence_To_Scene";
	function_Result = SAFE_FALSE;
	
	if (g_Dreaming_Module_Globals_Initialized != SAFE_TRUE) {
		goto Error_Handler;
	}	
	
	if (dreaming_Ptr->play.total_Scenes < 1) {
		goto Error_Handler;
	}
	
	if (dreaming_Ptr->play.scene_Memory_In_Use[scene_To_Use] != MEMORY_INITIALIZED) {
		goto Error_Handler;
	}
	
	if ((**(dreaming_Ptr->play.scene[scene_To_Use])).initialized != SAFE_TRUE) {
		goto Error_Handler;
	}
		
	if (dreaming_Ptr->puppet_Memory_Initialized[puppet] != MEMORY_INITIALIZED) {
		goto Error_Handler;
	}
	
	/*  */
	/* NOTICE: This assumes it is already locked down */
	/*  */
	item_Ptr = &((**(dreaming_Ptr->play.scene[scene_To_Use])).item[position]);

	item_Ptr->puppet = (*dreaming_Ptr->puppet[puppet]);

	if (item_Ptr->puppet->puppet_Initialized != SAFE_TRUE) {
		Display_Serious_Error_Message(unit_Name, code_Name, "\pInvalid Pointer passed (8)", "\pBad program");
	}

	Copy_Pascal_String(item_Ptr->puppet->file_Name, item_Ptr->desired_File_Name);
/* ====
	SetRect(&(item_Ptr->desired_Start_Location), 0,0,0,0);
==== */

	item_Ptr->valid_Pointer = SAFE_TRUE;
	
	if ((**(dreaming_Ptr->play.scene[scene_To_Use])).scene_Must_Use_Compositing_Buffers != SAFE_TRUE) {
		/*  */
		/* If we are adding a puppet that is redirected artwork then */
		/* insist on compositing buffer instead of direct to screen */ 
		/*  */
		if (item_Ptr->puppet->puppet_File_Is_Old_Self_Contained_Type != SAFE_TRUE) {
			(**(dreaming_Ptr->play.scene[scene_To_Use])).scene_Must_Use_Compositing_Buffers = SAFE_TRUE;
		}
	}
	
	/*  */
	/* HACK: We should let the user select voice input, but use default a little later on */
	/*  */
	
	if (Force_Scene_Item_To_Mute_Status(item_Ptr) != SAFE_TRUE) {
		/*  */
		/* Not important to worry about failures of this command */
		/*  */
	}
		
	/*  */
	/* HACK: We should let the user select voice input, but use default first one for now */
	/*  */
	
/* ===
	if (g_Voice_Globals_Initialized == SAFE_TRUE) {
		if (g_Voices.total_Voices_Active > 0) {
			max_Items = g_Voices.total_Voices_Active;
			total_Items_Seen = 0;
			for (loop = 0; ((loop < k_MAXIMUM_VOICE_MONITORS) && (total_Items_Seen < max_Items)); loop++) {
				if (g_Voices.voice_Active[loop] == SAFE_TRUE) {
					total_Items_Seen += 1;

					item_Ptr->voice = g_Voices.voice[loop];
					item_Ptr->controllers_Attached = SAFE_TRUE;
				}
			}
		}
	}
==== */
			
	(**(dreaming_Ptr->play.scene[scene_To_Use])).total_Items_In_Scene += 1;

	function_Result = SAFE_TRUE;
	goto Normal_Function_Exit;
	
No_More_Scenes_Error_Handler:
	Display_Serious_Error_Message(unit_Name, code_Name, "\pNo more items can be added", "\pRemove some");

Error_Handler:
	function_Result = SAFE_FALSE;

Normal_Function_Exit:

	return(function_Result);

} /* Safe_Boolean	Add_Puppet_Sequence_To_Scene( ... */
/*=**************************************************************************=*/
	
	
/*=**************************************************************************=*/
Safe_Boolean	Remove_Puppet_Sequence_From_Scene(CARDINAL_32 scene_To_Use, CARDINAL_16 puppet)
{
	Safe_Boolean		function_Result;
	INTEGER_16		index;
	INTEGER_16		position;
	SIMPLE_LOOP	loop;
	CARDINAL_8		*code_Name;
	Stage_Item_Ptr	item_Ptr;
	Boolean				found_Location;
/* END VAR */


	code_Name = "\pRemove_Puppet_Sequence_From_Scene";
	function_Result = SAFE_FALSE;
	
	if (g_Dreaming_Module_Globals_Initialized != SAFE_TRUE) {
		goto Error_Handler;
	}	
	
	if (dreaming_Ptr->play.total_Scenes < 1) {
		goto Error_Handler;
	}
	
	if (dreaming_Ptr->play.scene_Memory_In_Use[scene_To_Use] != MEMORY_INITIALIZED) {
		goto Error_Handler;
	}
	
	if ((**(dreaming_Ptr->play.scene[scene_To_Use])).initialized != SAFE_TRUE) {
		goto Error_Handler;
	}
	
	if (dreaming_Ptr->puppet_Memory_Initialized[puppet] != MEMORY_INITIALIZED) {
		goto Error_Handler;
	}

	if (Find_Puppet_Position_In_Scene(scene_To_Use, puppet, &position) != SAFE_TRUE) {
		/*  */
		/* Not a failure if already missing. Exit */
		/*  */
		function_Result = SAFE_TRUE;
		goto Normal_Function_Exit;
	}
	
	/*  */
	/* NOTICE: This assumes it is already locked down */
	/*  */
	item_Ptr = &((**(dreaming_Ptr->play.scene[scene_To_Use])).item[position]);

	item_Ptr->puppet = (*dreaming_Ptr->puppet[puppet]);

	if (item_Ptr->puppet->puppet_Initialized != SAFE_TRUE) {
		Display_Serious_Error_Message(unit_Name, code_Name, "\pInvalid Pointer passed (8)", "\pBad program");
	}
	
	if (Force_Scene_Item_To_Mute_Status(item_Ptr) != SAFE_TRUE) {
		/*  */
		/* Not important to worry about failures of this command */
		/*  */
	}

	item_Ptr->valid_Pointer = SAFE_FALSE;
	item_Ptr->desired_File_Loading_Now = SAFE_FALSE;

	(**(dreaming_Ptr->play.scene[scene_To_Use])).total_Items_In_Scene = 
			(**(dreaming_Ptr->play.scene[scene_To_Use])).total_Items_In_Scene - 1;

	
	function_Result = SAFE_TRUE;
	goto Normal_Function_Exit;
	
Error_Handler:
	function_Result = SAFE_FALSE;

Normal_Function_Exit:

	return(function_Result);

} /* Safe_Boolean	Remove_Puppet_Sequence_From_Scene( ... */
/*=**************************************************************************=*/
	
	
/*=**************************************************************************=*/
Safe_Boolean	Force_Scene_Item_To_Mute_Status(Stage_Item_Ptr	item_Ptr)
{
	Safe_Boolean		function_Result;
/* END VAR */

	
	item_Ptr->last_Display_Time = 0;
	item_Ptr->last_Volume_Check = 0;
	item_Ptr->current_Volume = 0;
	item_Ptr->voice = nil;
	item_Ptr->controllers_Attached = SAFE_FALSE;

	function_Result = SAFE_TRUE;

	return(function_Result);
}
/*=**************************************************************************=*/
	
	
/*=**************************************************************************=*/
Safe_Boolean	Destroy_A_Scene(CARDINAL_32 scene_To_Destroy)
{
	Safe_Boolean		function_Result;
	CARDINAL_32	total_Items;
	CARDINAL_32	items_Found;
	CARDINAL_32	backdrop_Index_To_Delete;
	Safe_Boolean		backdrop_In_Another_Scene;
	INTEGER_16		index;
	INTEGER_16		position;
	CARDINAL_32	actual_Puppet_Index;
	CARDINAL_8		*code_Name;
	Boolean				found_Location;
/* END VAR */


	code_Name = "\pDestroy_A_Scene";
	function_Result = SAFE_FALSE;
	
	if (g_Dreaming_Module_Globals_Initialized != SAFE_TRUE) {
		goto Error_Handler;
	}	
	
	if ((dreaming_Ptr->play.total_Scenes == 0)
		|| ((dreaming_Ptr->play.scene_Memory_In_Use[scene_To_Destroy] != MEMORY_INITIALIZED))){
		function_Result = SAFE_TRUE;
		goto Normal_Function_Exit;
	}
		
	if ((**(dreaming_Ptr->play.scene[scene_To_Destroy])).initialized != SAFE_TRUE) {
		Display_Serious_Error_Message(unit_Name, code_Name, "\pIGNORE: Warning, scene deletion request on non initialized structure", "\pBad programming");
	}
	
	total_Items = (**(dreaming_Ptr->play.scene[scene_To_Destroy])).total_Items_In_Scene;
	items_Found = 0;
	
	for (index = 0; ((index < k_MAX_STAGE_ITEMS) && (items_Found < total_Items)); index++) {
	
		if (((**(dreaming_Ptr->play.scene[scene_To_Destroy])).item[index].valid_Pointer == SAFE_TRUE)
			&& ((**(dreaming_Ptr->play.scene[scene_To_Destroy])).item[index].puppet != nil)) {
			
			/*  */
			/* May not resolve all if scene mangled but should be good enough */
			/*  */
			items_Found += 1;
			
			if ((**(dreaming_Ptr->play.scene[scene_To_Destroy])).item[index].controllers_Attached == SAFE_TRUE) {
			
				if (Force_Scene_Item_To_Mute_Status(&((**(dreaming_Ptr->play.scene[scene_To_Destroy])).item[index])) != SAFE_TRUE) {
					/*  */
					/* Not important to worry about failures of this command */
					/*  */
				}
			}
			
			if (Determine_Item_Needed_Elsewhere(scene_To_Destroy, index, &actual_Puppet_Index) != SAFE_TRUE) {
				
				/*  */
				/* Double check existence before deletion paranoia */
				/*  */
				if (Find_Puppet_Position_In_Scene(scene_To_Destroy
														, actual_Puppet_Index
														, &position) == SAFE_TRUE) {
														
					/*  */
					/* Delete puppet, it is not in use by other loaded scenes. */
					/*  */
					
					if (Delete_A_Loaded_Puppet(actual_Puppet_Index) != SAFE_TRUE) {
					
						/*  */
						/* Not important to worry about failures of this command */
						/*  */
						Display_Serious_Error_Message(unit_Name, code_Name, "\pNote: Scene puppet not erased (Delete_A_Loaded_Puppet)", "\pMay cause memory leak");
					}
				}
				else {
					/*  */
					/* Not found because just erased via another instantiation of same item in this scene */
					/*  */
					Display_Serious_Error_Message(unit_Name, code_Name, "\pNote: Scene puppet not erased (Find_Puppet_Position_In_Scene)", "\pMay cause memory leak");
				}
			}
			
			(**(dreaming_Ptr->play.scene[scene_To_Destroy])).item[index].puppet = nil;
			(**(dreaming_Ptr->play.scene[scene_To_Destroy])).item[index].valid_Pointer = SAFE_FALSE;
			(**(dreaming_Ptr->play.scene[scene_To_Destroy])).total_Items_In_Scene -= 1;
		}
		
	} /* for (index = 0; ((index < k_MAX_STAGE_ITEMS) ... */
	
		
	/*  */
	/* Paranoia only */
	/*  */
	backdrop_Index_To_Delete = 0;
	
	backdrop_In_Another_Scene = SAFE_TRUE;
	if ((**(dream.play.scene[scene_To_Destroy])).backdrop_Selected == SAFE_TRUE) {
		if ((**(dream.play.scene[scene_To_Destroy])).backdrop_Index_Valid == SAFE_TRUE) {
			backdrop_Index_To_Delete = ((**(dream.play.scene[scene_To_Destroy])).backdrop_Index);
			backdrop_In_Another_Scene = Determine_Backdrop_In_Another_Scene(scene_To_Destroy, backdrop_Index_To_Delete);
		}
	}
	
	
	if (backdrop_In_Another_Scene != SAFE_TRUE) {
		/*  */
		/* We must kill the offscreen backdrop. Its not needed now */
		/*  */
		if (Destroy_A_Backdrop(backdrop_Index_To_Delete) != SAFE_TRUE) {
			/*  */
			/* Not important to worry about failures of this command */
			/* But since so rare, we should post warning */
			/*  */				
			Display_Serious_Error_Message(unit_Name, code_Name, "\pDestroy_A_Backdrop fails", "\pSystem memory problems? Bad programming?");
		}
	}
	
	 if ((**(dreaming_Ptr->play.scene[scene_To_Destroy])).scene_Port.initialized == SAFE_TRUE) {
	 	if (Close_Offscreen_Pict_Port(&((**(dreaming_Ptr->play.scene[scene_To_Destroy])).scene_Port)) != SAFE_TRUE) {
			/*  */
			/* might as well complain about it? No not in this version */
			/*  */
			/* ===
	 		goto Error_Handler;
	 		==== */
	 	}
	 	
	 	if ((**(dreaming_Ptr->play.scene[scene_To_Destroy])).scene_Special_Pallete_To_Use != nil) {
	 		DisposePalette((**(dreaming_Ptr->play.scene[scene_To_Destroy])).scene_Special_Pallete_To_Use);
	 	}
	 }	
	
	(**(dreaming_Ptr->play.scene[scene_To_Destroy])).initialized = SAFE_FALSE;
	
	if (Destroy_Handle((Handle *)&(dreaming_Ptr->play.scene[scene_To_Destroy])
										, &(dreaming_Ptr->play.scene_Memory_In_Use[scene_To_Destroy])) != SAFE_TRUE) {
		Display_Serious_Error_Message(unit_Name, code_Name, "\pError with DisposHandle", "\pSystem Corrupted?");
		goto Error_Handler;
	}	
	dreaming_Ptr->play.scene[scene_To_Destroy] = nil;

	dreaming_Ptr->play.total_Scenes = dreaming_Ptr->play.total_Scenes -1;
	
	function_Result = SAFE_TRUE;
	goto Normal_Function_Exit;

Error_Handler:
	function_Result = SAFE_FALSE;

Normal_Function_Exit:

	return(function_Result);

} /* Safe_Boolean	Destroy_A_Scene( ... */
/*=**************************************************************************=*/
	
	
/*=**************************************************************************=*/
Safe_Boolean	Create_New_Empty_Scene(CARDINAL_32 *located_Position)
{
	Safe_Boolean		function_Result;
	CARDINAL_32	memory_Requested;
	SIMPLE_LOOP	loop;
	INTEGER_16		index;
	INTEGER_16		position;
	Scene_Ptr			current_Scene;
	SignedByte			original_State;
	CARDINAL_8		*code_Name;
	Boolean				found_Location;
/* END VAR */


	code_Name = "\pCreate_New_Empty_Scene";
	function_Result = SAFE_FALSE;
	*located_Position = 0;
	
	if (g_Dreaming_Module_Globals_Initialized == SAFE_TRUE) {
		
		if (dreaming_Ptr->play.total_Scenes < k_MAX_ACTIVE_SCENES) {
		
			if (Find_Empty_Position_For_Scene(&position) != SAFE_TRUE) {
				goto No_More_Scenes_Error_Handler;
			}
		
			found_Location = FALSE;
				
		
			dreaming_Ptr->play.scene[position] = nil;
			dreaming_Ptr->play.scene_Memory_In_Use[position] = MEMORY_NOT_INITIALIZED;
			memory_Requested = sizeof(Scene_Rec);
			if (Create_Handle( (CARDINAL_32 *)&memory_Requested
									, (CARDINAL_32)0L
									, ALLOW_PURGING
									, DISALLOW_PARTIAL_AMOUNT
									, (Handle *)&(dreaming_Ptr->play.scene[position])
									, &(dreaming_Ptr->play.scene_Memory_In_Use[position])) != SAFE_TRUE ) {
									
				Perform_OK_Dialog("\pMemory error allocating buffer_Memory (not enough?) (4)");
	
				goto Normal_Function_Exit;
			}
			
			if (Begin_Use_Of_Handle((Handle)dreaming_Ptr->play.scene[position]
									, MOVE_HIGH_REQUIRED
									, &original_State) != SAFE_TRUE) {
				/*  */
				/* Ignore error in implementation */
				/*  */
			}
			
			/*  */
			/* Because locked, we can access using a pointer */
			/*  */		
			current_Scene = *(dreaming_Ptr->play.scene[position]);
			
			for (index = 0; (index < k_MAX_STAGE_ITEMS); index++) {
				current_Scene->item[index].puppet = nil;
				current_Scene->item[index].valid_Pointer = SAFE_FALSE;
				current_Scene->item[index].desired_File_Loading_Now = SAFE_FALSE;
				
				
				/*  */
				/* These are used for determining Spinfile caching info */
				/*  */
				current_Scene->item[index].last_Image.valid_Information = SAFE_FALSE;
				current_Scene->item[index].last_Image.avoidance_Image_Index = k_BOGUS_INDEX_FRAME;
			
				for (loop = 0; (loop < (k_BUFFERS_REQUIRED - k_BASE_OFFSET_FOR_MIX)); loop++) {
					current_Scene->item[index].frame_Needing_Erasure[loop].valid_Information = SAFE_FALSE;
					current_Scene->item[index].current_Displayed_Frame[loop].intensity = 0xFFFFFFFF;
					current_Scene->item[index].current_Displayed_Frame[loop].horizontal_Angle = 0;
					current_Scene->item[index].current_Displayed_Frame[loop].vertical_Angle = 0;
					current_Scene->item[index].current_Displayed_Frame[loop].roll_Angle = 0;
				}
				
				if (Force_Scene_Item_To_Mute_Status(&(current_Scene->item[index])) != SAFE_TRUE) {
					/*  */
					/* Not important to worry about failures of this command */
					/*  */
				}

				/*  */
				/* Paranoia only */		
				/*  */		
				current_Scene->item[index].desired_File_Loading_Counter = 0;
				current_Scene->item[index].desired_File_Name[0] = 0;
				
				/*  */
				/* Don't know if important or not */
				/*  */		
				current_Scene->item[index].desired_Start_Location.top = 0;
				current_Scene->item[index].desired_Start_Location.left = 0;
				current_Scene->item[index].desired_Start_Location.bottom = 1;
				current_Scene->item[index].desired_Start_Location.right = 1;
			}
			
			dreaming_Ptr->play.scene_Hot_Key[position] = 'A';
			dreaming_Ptr->play.scene_Hot_Key[position] += dreaming_Ptr->play.total_Scenes;
			
			current_Scene->scene_Port.initialized = SAFE_FALSE;
			/*  */
			/* Gets properly set later onward */
			/*  */
			SetRect(&(current_Scene->scene_Background_Rect), 0, 0, 0, 0);
			
			current_Scene->scene_Special_Pallete_To_Use = nil;	
			
			current_Scene->total_Items_In_Scene = 0;
			current_Scene->scene_Updated_To_Screen_Display_Once = SAFE_FALSE;
			
			/*  */
			/* Is set to true when spin file art or text overlays, etc are used */
			/* otheriwse allows direct-to-screen animation */
			/* Set to "SAFE_FALSE" as default except fpr rare performace tests */
			/*  */
			current_Scene->scene_Must_Use_Compositing_Buffers = SAFE_FALSE;
						
			/*  */
			/* Is set to true when video underlays are used (movies: either */
			/* QuickTime or QuickestTime) */
			/* prevents traversing list needlessly in search */
			/*  */
			current_Scene->scene_Might_Contain_Movie_Ports = SAFE_FALSE;
			
			current_Scene->items_Remaining_To_Load = 0;
			current_Scene->item_Loading_Last_Index = 0;

			/*  */
			/* Mostly paranoia only */
			/*  */
			current_Scene->version = k_SCENE_VERSION_NOW;
			current_Scene->backward_Compatible_Version = k_SCENE_BASE_VERSION_NOW;

			current_Scene->total_Colors_Used_In_Pallette = 0;
			current_Scene->total_Special_Colors_To_Honor = 0;
			current_Scene->stored_Bit_Depth = 8;
			current_Scene->screen_Name[0] = 0;
			current_Scene->screen_Short_Name[0] = 0;
			current_Scene->file_Name[0] = 0;
			current_Scene->backdrop_Selected = SAFE_FALSE;
			current_Scene->backdrop_Needs_Attaching = SAFE_FALSE;
			current_Scene->backdrop_Name[0] = 0;
			current_Scene->backdrop_Index = 0;
			current_Scene->backdrop_Index_Valid = SAFE_FALSE;
				
			current_Scene->initialized = SAFE_TRUE;
			dreaming_Ptr->play.total_Scenes += 1;
			*located_Position = position;
			function_Result = SAFE_TRUE;
			
		}

	} /* if (g_Dreaming_Module_Globals_Initialized ... */

	if (function_Result != SAFE_TRUE) {
		goto Error_Handler;
	}	
	
	goto Normal_Function_Exit;
	
No_More_Scenes_Error_Handler:
	Display_Serious_Error_Message(unit_Name, code_Name, "\pNo more Scenes can be loaded", "\pDelete some");

Error_Handler:
	function_Result = SAFE_FALSE;
			
Normal_Function_Exit:
	return(function_Result);
	
} /* Safe_Boolean	Create_New_Empty_Scene... */
/*=**************************************************************************=*/


/*=**************************************************************************=*/
Safe_Boolean	Reset_Debugging_Controls(void)
{
	Safe_Boolean		function_Result;
/* END VAR */

	g_Debugging_Options[1] = SAFE_TRUE;
	
	g_Debugging_Options[0] = SAFE_FALSE;
	g_Debugging_Options[2] = SAFE_FALSE;
	g_Debugging_Options[3] = SAFE_FALSE;

	EnableItem(g_Shell.interface.program_Menus[DEBUG_MENU] , MENU_COMMAND_SET_DEBUG_1);
	EnableItem(g_Shell.interface.program_Menus[DEBUG_MENU] , MENU_COMMAND_SET_DEBUG_2);
	EnableItem(g_Shell.interface.program_Menus[DEBUG_MENU] , MENU_COMMAND_SET_DEBUG_3);
	EnableItem(g_Shell.interface.program_Menus[DEBUG_MENU] , MENU_COMMAND_SET_DEBUG_4);
	
	if (g_Debugging_Options[0] == SAFE_TRUE) {
		SetItemMark(g_Shell.interface.program_Menus[DEBUG_MENU]
					, MENU_COMMAND_SET_DEBUG_1
					, checkMark);
	}
	else {
		SetItemMark(g_Shell.interface.program_Menus[DEBUG_MENU]
					, MENU_COMMAND_SET_DEBUG_1
					, noMark);
	}
	
	if (g_Debugging_Options[1] == SAFE_TRUE) {
		SetItemMark(g_Shell.interface.program_Menus[DEBUG_MENU]
					, MENU_COMMAND_SET_DEBUG_2
					, checkMark);
	}
	else {
		SetItemMark(g_Shell.interface.program_Menus[DEBUG_MENU]
					, MENU_COMMAND_SET_DEBUG_2
					, noMark);
	}

	if (g_Debugging_Options[2] == SAFE_TRUE) {
		SetItemMark(g_Shell.interface.program_Menus[DEBUG_MENU]
					, MENU_COMMAND_SET_DEBUG_3
					, checkMark);
	}
	else {
		SetItemMark(g_Shell.interface.program_Menus[DEBUG_MENU]
					, MENU_COMMAND_SET_DEBUG_3
					, noMark);
	}

	if (g_Debugging_Options[3] == SAFE_TRUE) {
		SetItemMark(g_Shell.interface.program_Menus[DEBUG_MENU]
					, MENU_COMMAND_SET_DEBUG_4
					, checkMark);
	}
	else {
		SetItemMark(g_Shell.interface.program_Menus[DEBUG_MENU]
					, MENU_COMMAND_SET_DEBUG_4
					, noMark);
	}
	
	function_Result = SAFE_TRUE;
			
Normal_Function_Exit:
	return(function_Result);
	
} /* Safe_Boolean	Reset_Debugging_Controls... */
/*=**************************************************************************=*/


/*=**************************************************************************=*/
Safe_Boolean	Enable_Minimal_Puppet_Controls(void)
{
	Safe_Boolean		function_Result;
/* END VAR */


	EnableItem(g_Shell.interface.program_Menus[ITEMS_MENU] , MENU_COMMAND_LOAD_PUPPET);
	EnableItem(g_Shell.interface.program_Menus[ITEMS_MENU] , MENU_COMMAND_IMPORT_PUPPET);
	EnableItem(g_Shell.interface.program_Menus[ITEMS_MENU] , MENU_COMMAND_SET_LIFE_JIGGLE);
	EnableItem(g_Shell.interface.program_Menus[ITEMS_MENU] , MENU_COMMAND_CREATE_PUPPET_VIA_SPINFILE);
	
	EnableItem(g_Shell.interface.program_Menus[ITEMS_MENU] , MENU_COMMAND_LOAD_SPIN_OBJECT);
	EnableItem(g_Shell.interface.program_Menus[ITEMS_MENU] , MENU_COMMAND_IMPORT_SPIN_OBJECT);	
	
	function_Result = SAFE_TRUE;
			
Normal_Function_Exit:
	return(function_Result);
	
} /* Safe_Boolean	Enable_Minimal_Puppet_Controls... */
/*=**************************************************************************=*/


/*=**************************************************************************=*/
Safe_Boolean	Enable_Audio_Controls(void)
{
	Safe_Boolean		function_Result;
/* END VAR */
			
	EnableItem(g_Shell.interface.program_Menus[AUDIO_MENU] , MENU_COMMAND_MONITOR_MICROPHONE);
	EnableItem(g_Shell.interface.program_Menus[AUDIO_MENU] , MENU_COMMAND_MONITOR_SAMPLER);
	EnableItem(g_Shell.interface.program_Menus[AUDIO_MENU] , MENU_COMMAND_MONITOR_MANUAL);
	EnableItem(g_Shell.interface.program_Menus[AUDIO_MENU] , MENU_COMMAND_MONITOR_PHONEBRIDGE);

/* ==
	EnableItem(g_Shell.interface.program_Menus[AUDIO_MENU] , MENU_COMMAND_MONITOR_REMOTE);
=== */

	if (g_Voices.monitor_Microphone_For_Speech == SAFE_TRUE) {
		SetItemMark(g_Shell.interface.program_Menus[AUDIO_MENU]
					, MENU_COMMAND_MONITOR_MICROPHONE
					, checkMark);
	}
	else {
		SetItemMark(g_Shell.interface.program_Menus[AUDIO_MENU]
					, MENU_COMMAND_MONITOR_MICROPHONE
					, noMark);
	}
		
	if (g_Voices.monitor_Sampler_For_Speech == SAFE_TRUE) {
		SetItemMark(g_Shell.interface.program_Menus[AUDIO_MENU]
					, MENU_COMMAND_MONITOR_SAMPLER
					, checkMark);
	}
	else {
		SetItemMark(g_Shell.interface.program_Menus[AUDIO_MENU]
					, MENU_COMMAND_MONITOR_SAMPLER
					, noMark);
	}
	
	if (g_Voices.monitor_Keyboard_As_Speech == SAFE_TRUE) {
		SetItemMark(g_Shell.interface.program_Menus[AUDIO_MENU]
					, MENU_COMMAND_MONITOR_MANUAL
					, checkMark);
	}
	else {
		SetItemMark(g_Shell.interface.program_Menus[AUDIO_MENU]
					, MENU_COMMAND_MONITOR_MANUAL
					, noMark);
	}
	
	if (g_Voices.monitor_Network_As_Speech == SAFE_TRUE) {
		SetItemMark(g_Shell.interface.program_Menus[AUDIO_MENU]
					, MENU_COMMAND_MONITOR_REMOTE
					, checkMark);
	}
	else {
		SetItemMark(g_Shell.interface.program_Menus[AUDIO_MENU]
					, MENU_COMMAND_MONITOR_REMOTE
					, noMark);
	}
	
	if (g_Voices.monitor_MIDI_For_Sound == SAFE_TRUE) {
		SetItemMark(g_Shell.interface.program_Menus[AUDIO_MENU]
					, MENU_COMMAND_MONITOR_REAL_MIDI
					, checkMark);
	}
	else {
		SetItemMark(g_Shell.interface.program_Menus[AUDIO_MENU]
					, MENU_COMMAND_MONITOR_REAL_MIDI
					, noMark);
	}
	
	if (g_Voices.Monitor_Phonebridge_For_Sound == SAFE_TRUE) {
		SetItemMark(g_Shell.interface.program_Menus[AUDIO_MENU]
					, MENU_COMMAND_MONITOR_PHONEBRIDGE
					, checkMark);
	}
	else {
		SetItemMark(g_Shell.interface.program_Menus[AUDIO_MENU]
					, MENU_COMMAND_MONITOR_PHONEBRIDGE
					, noMark);
	}
	
	
	function_Result = SAFE_TRUE;
			
Normal_Function_Exit:
	return(function_Result);
	
} /* Safe_Boolean	Enable_Audio_Controls... */
/*=**************************************************************************=*/


/*=**************************************************************************=*/
Safe_Boolean	Destroy_A_Puppet_Structure(CARDINAL_32 index)
/*  */
/* Warning: Compact_Puppet_List called within so reference is invalid once */
/* this is called! Don't try to clear reference externally! */
/*  */
{
	Safe_Boolean		function_Result;
	CARDINAL_8		*code_Name;
/* END VAR */

	code_Name = "\pDestroy_A_Puppet_Structure";
	function_Result = SAFE_FALSE;

	if (dreaming_Ptr->puppet_Memory_Initialized[index] == MEMORY_INITIALIZED) {
	
		
		if ((**(dreaming_Ptr->puppet[index])).direct_Art.port.initialized == SAFE_TRUE) {
			Display_Serious_Error_Message(unit_Name, code_Name, "\pPort still present!", "\pSystem Corrupted?");
		}

		if ((**(dreaming_Ptr->puppet[index])).direct_Art.total_Lazy_Active_Region != nil){
			Display_Serious_Error_Message(unit_Name, code_Name, "\pRegions still present!", "\pSystem Corrupted?");
		}
		
		(**(dreaming_Ptr->puppet[index])).puppet_Initialized = SAFE_FALSE;
					
		if (Destroy_Handle((Handle *)&(dreaming_Ptr->puppet[index])
												, &dreaming_Ptr->puppet_Memory_Initialized[index]) != SAFE_TRUE) {
			Display_Serious_Error_Message(unit_Name, code_Name, "\pError with DisposHandle", "\pSystem Corrupted?");
			goto Error_Handler;
		}
		
		if (dreaming_Ptr->active_Puppets_Total > 0) {
			dreaming_Ptr->active_Puppets_Total = dreaming_Ptr->active_Puppets_Total - 1;
		}
	}
	else {
		Display_Serious_Error_Message(unit_Name, code_Name, "\pAlready assumed deleted", "\pCorrupted?");
		goto Error_Handler;
	}

	if (Compact_Puppet_List() != SAFE_TRUE) {
		goto Error_Handler;
	}

	function_Result = SAFE_TRUE;
	goto Normal_Function_Exit;
	
Error_Handler:
	function_Result = SAFE_FALSE;
	
Normal_Function_Exit:
	return(function_Result);
}
/*=**************************************************************************=*/


/*=**************************************************************************=*/
Safe_Boolean	Create_New_Puppet_Structures(CARDINAL_32 *index_Used)
{
	CARDINAL_32	puppet_Index;
	CARDINAL_32	index;
	Puppet_Ptr			current_Puppet;
	CARDINAL_32	preferred_Size;
	Safe_Boolean		function_Result;
	SignedByte			handle_State;
	INTEGER_16 		position;
	Boolean				found_It;
	CARDINAL_8		*code_Name;
	Str255					message_String;
/* END VAR */

	code_Name = "\pCreate_New_Puppet_Structures";
	function_Result = SAFE_FALSE;
	
	*index_Used = 0;
	
	if (g_Dreaming_Module_Globals_Initialized != SAFE_TRUE) {
		goto Error_Handler;
	}
		
	if (Find_Empty_Position_For_Puppet(&position) != SAFE_TRUE) {
		goto Error_Handler;
	}
	puppet_Index = position;
	
	/*  */
	/* Allocate puppet memory. Graphics port memory is */
	/* allocated later on in another routine */
	/*  */
	preferred_Size = sizeof(Puppet_Rec);
	dreaming_Ptr->puppet_Memory_Initialized[puppet_Index] = MEMORY_NOT_INITIALIZED;
	dreaming_Ptr->puppet[puppet_Index] = nil;
	
	if (Create_Handle(&preferred_Size
						, (CARDINAL_32)(10240L)
						, ALLOW_PURGING
						, DISALLOW_PARTIAL_AMOUNT
						, (Handle *)&dreaming_Ptr->puppet[puppet_Index]
						, &dreaming_Ptr->puppet_Memory_Initialized[puppet_Index]) != SAFE_TRUE) {

		Display_Low_Memory_Error_Message(unit_Name, code_Name, sizeof(Puppet_Rec));

		function_Result = SAFE_FALSE;
		goto Normal_Function_Exit;
	}
	/*  */
	/* Move it high and lock it down for good (until deleted) */
	/*  */
	if (Begin_Use_Of_Handle((Handle) dreaming_Ptr->puppet[puppet_Index]
					, MOVE_HIGH_REQUIRED
					, &handle_State) != SAFE_TRUE) {
		goto Error_Handler;
	}
	
	/*  */
	/* Clear to "FF" for paranoia safety debugging (help create crash) */
	/* NOTE WAS "0" before 1995.06.10 */
	/*  */
	Fill_Byte_Memory_With_Byte((Ptr )*(dreaming_Ptr->puppet[puppet_Index])
								, sizeof(Puppet_Rec)
								, 0xFF);
	
	current_Puppet = *(dreaming_Ptr->puppet[puppet_Index]);
	/*  */
	/* these few are important to clear */
	/*  */
	current_Puppet->direct_Art.frame_Total = 0;
	current_Puppet->direct_Art.talk_Total_Frames = 0;
		
	current_Puppet->direct_Art.talk_Start_Frame = 0;
	current_Puppet->direct_Art.talk_Stop_Frame = 0;
		
	for (index = 0; (index < k_MAXIMUM_PICS_FRAMES); index++) {
		/*  */
		/* Paranoia only, **NOT** needed in finished version at all */
		/*  */
		current_Puppet->direct_Art.frame_Locations[index].left = 0;
		current_Puppet->direct_Art.frame_Locations[index].right = 1;
		current_Puppet->direct_Art.frame_Locations[index].top = 0;
		current_Puppet->direct_Art.frame_Locations[index].bottom = 1;
		current_Puppet->direct_Art.frame_Regions[index] = nil;
		current_Puppet->direct_Art.frame_Buffered_Animation_Regions[index] = nil;
		current_Puppet->direct_Art.frame_Buffered_Animation_Region_Offset[index].h = 0;
		current_Puppet->direct_Art.frame_Buffered_Animation_Region_Offset[index].v = 0;
	}
	

	current_Puppet->direct_Art.total_Lazy_Active_Region = nil;
	current_Puppet->direct_Art.original_Total_Lazy_Active_Region = nil;
	current_Puppet->direct_Art.buffered_Animation_Total_Region = nil;
	current_Puppet->direct_Art.buffered_Animation_Total_Region_Offset.h = 0;
	current_Puppet->direct_Art.buffered_Animation_Total_Region_Offset.v = 0;

	current_Puppet->direct_Art.port.initialized = SAFE_FALSE;
	
	current_Puppet->puppet_File_Is_Old_Self_Contained_Type = SAFE_TRUE;
	current_Puppet->redirected_Information_Hdl = nil;
	current_Puppet->redirected_Information_Ptr = nil;
	current_Puppet->redirected_Information_Memory_In_Use = MEMORY_NOT_INITIALIZED;
	current_Puppet->redirected_Information_Max_Size = 0;
	current_Puppet->redirected_Information_Total_Used = 0;
	
	current_Puppet->total_Colors_Used_In_Pallette = 0;
	current_Puppet->total_Special_Colors_To_Honor = 0;
	current_Puppet->special_Pallete_To_Use = nil;

	current_Puppet->puppet_Initialized = SAFE_TRUE;

	dreaming_Ptr->active_Puppets_Total += 1;
	
	
	*index_Used = puppet_Index;

	function_Result = SAFE_TRUE;
	goto Normal_Function_Exit;
	
Error_Handler:
	function_Result = SAFE_FALSE;
		
Normal_Function_Exit:

	return(function_Result);
	
} /* Safe_Boolean	Create_New_Puppet_Structures... */
/*=**************************************************************************=*/


/*=**************************************************************************=*/
Safe_Boolean	Load_A_Puppet_Into_Any_Scene(Boolean import_PICS_Sequence, CARDINAL_16 import_Source)
{
	CARDINAL_8		*code_Name;
	Safe_Boolean		result;
	Safe_Boolean		function_Result;
	Safe_Boolean		abort_Desired;
	CARDINAL_32	item_Address_Index;
	Safe_Boolean		address_Instead_Of_File;
	CARDINAL_32	current_Index;
	CARDINAL_32	item_Address;
	INTEGER_16		position_Of_Item_In_Scene;
	Safe_Boolean		new_Scene_Was_Created;
	INTEGER_16		current_Stage;
	Scene_Ptr			active_Scene_Ptr;
	Boolean 				stages_Finished;
	INTEGER_16		pict_Resource_Id;
	FSSpec				file_Specification;
	FSSpec				pict_File_Info;
	CARDINAL_32 	located_Position;
/* END VAR */


	code_Name = "\pLoad_A_Puppet_Into_Any_Scene";
	function_Result = SAFE_FALSE;
	
	if (Create_New_Puppet_Structures(&current_Index) != SAFE_TRUE) {
		goto Error_Handler;
	}
	
	if (import_PICS_Sequence != TRUE) {
	
		if (Specify_Puppet_To_Load(&file_Specification
												, &abort_Desired) != SAFE_TRUE ) {
																									
			if (abort_Desired == SAFE_TRUE) {
				/*  */
				/* Ignore, we are quitting */
				/* IN FUTURE, delete puppet structure? */
				/*  */
				function_Result = SAFE_TRUE;
				goto Normal_Function_Exit;
			}
			else {
					goto Error_Handler;
			}
			
		}
											
		if (Load_Puppet_Sequence(current_Index
												, 0
												, &file_Specification) != SAFE_TRUE ) {

			/*  */
			/* IN FUTURE, delete puppet structure? */
			/*  */
			goto Error_Handler;
		}
	}
	else {
	
		if (import_Source == k_IMPORT_FROM_PICS_SEQUENCE) {
			if (Load_PICS_Sequence(current_Index, &abort_Desired) != SAFE_TRUE) {
			
				if (abort_Desired == SAFE_TRUE) {
					/*  */
					/* Ignore, we are quitting */
					/*  */
					function_Result = SAFE_TRUE;
					goto Normal_Function_Exit;
				}
				else {
					goto Error_Handler;
				}
			}
		}
		else {
			if (Define_Puppet_Using_Spin_File(current_Index, &abort_Desired, SAFE_FALSE) != SAFE_TRUE) {
			
				if (abort_Desired == SAFE_TRUE) {
					/*  */
					/* Ignore, we are quitting */
					/*  */
					function_Result = SAFE_TRUE;
					goto Normal_Function_Exit;
				}
				else {
					goto Error_Handler;
				}
			}
		}
		
	} /* if (import_PICS_Sequence != TRUE) ... */
	
	if (dreaming_Ptr->play.total_Scenes < 1) {
		if (Create_New_Empty_Scene(&located_Position) != SAFE_TRUE) {
			/*  */
			/* Ignore, puppet loaded, even if no scene available. */
			/* IN FUTURE DESTROY LOADED ITEM!!! */
			/*  */
			function_Result = SAFE_TRUE;
			goto Normal_Function_Exit;
		}
		
		new_Scene_Was_Created = SAFE_TRUE;		
	}
	else {
		/*  */
		/* Ask in future? */
		/* Merely add puppet to end of current scene's item list */
		/* because a scene is available. */
		/*  */
		located_Position = dreaming_Ptr->current_Scene;
		new_Scene_Was_Created = SAFE_FALSE;		
	}
	

	if (dreaming_Ptr->play.scene_Memory_In_Use[located_Position]
			== MEMORY_INITIALIZED) {
			
			if (Find_Empty_Position_In_Scene(located_Position, &position_Of_Item_In_Scene) != SAFE_TRUE) {
				/*  */
				/* Not a terrible failure if filled. Exit with warming */
				/*  */
				goto No_More_Scenes_Error_Handler;
			}
			
			active_Scene_Ptr = (*(dreaming_Ptr->play.scene[located_Position]));
			
			if (active_Scene_Ptr->item[position_Of_Item_In_Scene].desired_File_Already_In_Ram != SAFE_TRUE) {
				/*  */
				/* HACK: Soon make final stage of "Load_Puppet_Sequence" */
				/* trigger setting this. */
				/*  */
				active_Scene_Ptr->item[position_Of_Item_In_Scene].puppet = *(dream.puppet[current_Index]);
				active_Scene_Ptr->item[position_Of_Item_In_Scene].valid_Pointer = SAFE_TRUE;
				active_Scene_Ptr->item[position_Of_Item_In_Scene].desired_File_Already_In_Ram = SAFE_TRUE;
			}
			
			if (Force_Scene_Item_To_Mute_Status(&(active_Scene_Ptr->item[position_Of_Item_In_Scene])) != SAFE_TRUE) {
				/*  */
				/* Not important to worry about failures of this command */
				/*  */
			}

			if (Add_Puppet_Sequence_To_Scene(located_Position, current_Index, position_Of_Item_In_Scene) != SAFE_TRUE) {
				/*  */
				/* Ignore error at this stage, and let it tumble out normally? */
				/*  */
				
				/*  */
				/* local position already points to current scene now */
				/* so set the global scene pointer index */
				/*  */
				dreaming_Ptr->current_Scene = located_Position;
			}
			else {
				/*  */
				/* Draw sample first frame/ background. DRAWS WHOLE SCENE*/
				/*  */
			
				/*  */
				/* local position already points to current scene now */
				/* so set the global scene pointer index */
				/* NO!!!! only if not a single image system! */
				/*  */
				/*  */
				/*  */
				/*  */
				/*  */
				/*  */
				/*  */
				/*  */
				/*  */
				/*  */
				/*  */
				/*  */
				/* Hack temp */
				/*  */
				/*  */
				/*  */
				/*  */
				/*  */
				/*  */
				/*  */
				/*  */
				/*  */
				/*  */
				/*  */
				/*  */
				/*  */
				/*  */
				/*  */
				dreaming_Ptr->current_Scene = located_Position;
			}
	}
	else {
		/*  */
		/* Ignore. IN FUTURE, delete puppet structure? */
		/*  */
	}
	
	/*  */
	/* Add background if new_Scene_Was_Created (or OLD right now)*/
	/*  */
	if (new_Scene_Was_Created == SAFE_TRUE) {
		if (dreaming_Ptr->play.scene_Memory_In_Use[located_Position]
				== MEMORY_INITIALIZED) {
								
				/*  */
				/* Ths probably should be cached globals. Oh well. */
				/*  */
				pict_Resource_Id = k_DEFAULT_BACKGROUND_RES_ID;
				
				
				item_Address_Index = 0;
				address_Instead_Of_File = SAFE_FALSE;
				
				if ((**(dreaming_Ptr->play.scene[located_Position])).backdrop_Selected == SAFE_TRUE) {
				
					pict_Resource_Id = 0;
				
					/*  */
					/* Puppet not even starting to load yet. */
					/* We need to locate the source of the Item from data base. */
					/* Fetch filled copy of a FSSpec for the puppet to load */
					/* or at least a pointer to a loaded puppet shared. */
					/*  */																
					if (Locate_Item_Location(k_BACKDROP_SELECTION
												, (Str63 *)&((**(dreaming_Ptr->play.scene[located_Position])).backdrop_Name[0])
												, &pict_File_Info
												, &item_Address
												, &address_Instead_Of_File) != SAFE_TRUE) {
						/*  */
						/* Goddamn file not available. We need to skip item and bitch */
						/* Preflight should have been done elsewhere. */
						/*  */
						pict_Resource_Id = k_DEFAULT_BACKGROUND_RES_ID;

						/* ===
						goto Misssing_Item_Error_Handler;
						=== */
					}
					
					if (pict_Resource_Id == 0) {
					
						/*  */
						/* These may already have been reset from file load but */
						/* no harm in ensuring it is OK */
						/*  */
						if (address_Instead_Of_File != SAFE_TRUE) {
							/*  */
							/* We cannot share the pict image! */
							/* Gotta load it. "pict_File_Info" should be active. */
							/*  */
							/* pict_Resource_Id = 0; */
						}
					}
				}
				current_Stage = 0;
				do {
					result = Add_Background_To_Scene(located_Position
																			, pict_Resource_Id
																			, &pict_File_Info
																			, item_Address_Index
																			, address_Instead_Of_File
																			, current_Stage
																			, &stages_Finished);
				
					current_Stage += 1;															
				} while ((result == SAFE_TRUE) && (stages_Finished != TRUE));
				
				if (result == SAFE_TRUE) {
					current_Stage = 0;
					do {
						result = Add_Background_Of_Scene_To_Puppets(located_Position
																					, current_Stage
																					, &stages_Finished);
						current_Stage += 1;															
					} while ((result == SAFE_TRUE) && (stages_Finished != TRUE));
				}
				
				if (result != SAFE_TRUE) {
					/*  */
					/* Ignore. */
					/*  */
					Display_Serious_Error_Message(unit_Name, code_Name, "\punexpected problem", "\pMemory?");
				}

				if (Attach_Default_Inputs_To_Scene_Puppets(located_Position, dream.play.default_Start_Audio_Point, FALSE) != SAFE_TRUE) {
					Display_Serious_Error_Message(unit_Name, code_Name, "\punexpected problem (Attach_Default_Inputs_To_Scene_Puppets failed)", "\pMemory?");
				}
		}
		
		if (Redraw_Idle_Screen_Completely(FALSE, FALSE) != SAFE_TRUE) {
			/*  */
			/* Ignore error at this stage, and let it tumble out normally */
			/*  */
		}
		
	} /* (new_Scene_Was_Created == SAFE_TRUE) */
	else {
	
		if (Redraw_Idle_Screen_Completely(FALSE, FALSE) != SAFE_TRUE) {
			/*  */
			/* Ignore error at this stage, and let it tumble out normally */
			/*  */
		}
	}

	function_Result = SAFE_TRUE;
	goto Normal_Function_Exit;
	
No_More_Scenes_Error_Handler:
Error_Handler:
	Display_Serious_Error_Message(unit_Name, code_Name, "\pProblem encountered", "\p");
	function_Result = SAFE_FALSE;
		
Normal_Function_Exit:

	return(function_Result);
	
} /* Safe_Boolean	Load_A_Puppet_Into_Any_Scene(void) */
/*=**************************************************************************=*/


/*=**************************************************************************=*/
Safe_Boolean	Delete_A_Loaded_Puppet(CARDINAL_32	current_Index)
/*  */
/* Warning: Compact_Puppet_List called within as side effect, */
/* so reference is invalid once this is called! */
/* Don't try to clear reference externally! */
/*  */
{
	Safe_Boolean		function_Result;
	CARDINAL_8		*code_Name;
	CARDINAL_32	current_Frame;
	CARDINAL_32	last_Frame;
	CARDINAL_32	redirected_Information_Total_Used;
	CARDINAL_32	loop;
	OSErr					os_Error;
	Redirected_Art_Ptr		loaded_Info_Ptr;
	RgnHandle			*temp_Rgn_Ptr;
/* END VAR */


	code_Name = "\pDelete_A_Loaded_Puppet";
	function_Result = SAFE_FALSE;

	if (dreaming_Ptr->puppet_Memory_Initialized[current_Index] == MEMORY_INITIALIZED) {
		
	/* ===
			if ((**(dreaming_Ptr->puppet[current_Index])).initialized == SAFE_TRUE) {
	=== */
		
			if ((**(dreaming_Ptr->puppet[current_Index])).direct_Art.port.initialized == SAFE_TRUE) {
			
				if ((**(dreaming_Ptr->puppet[current_Index])).puppet_File_Is_Old_Self_Contained_Type != SAFE_TRUE) {
					Display_Serious_Error_Message(unit_Name, code_Name, "\pPossible Error with Disposing port because not standard", "\pSystem Corrupted?");
				}
			
				if (Close_Offscreen_Pict_Port(&(**(dreaming_Ptr->puppet[current_Index])).direct_Art.port) != SAFE_TRUE) {
					goto Error_Handler;
				}
				
			}
			
			(**(dreaming_Ptr->puppet[current_Index])).direct_Art.port.initialized = SAFE_FALSE;
			
			if ((**(dreaming_Ptr->puppet[current_Index])).special_Pallete_To_Use != nil) {
				if ((**((**(dreaming_Ptr->puppet[current_Index])).special_Pallete_To_Use)).pmEntries > 0) {	
					DisposePalette((**(dreaming_Ptr->puppet[current_Index])).special_Pallete_To_Use);
				}
			}

			if ((**(dreaming_Ptr->puppet[current_Index])).puppet_File_Is_Old_Self_Contained_Type == SAFE_TRUE) {
			
				last_Frame = (**(dreaming_Ptr->puppet[current_Index])).direct_Art.frame_Total;
				for (current_Frame = 0
										; (current_Frame < last_Frame)
										; current_Frame++) {
										
					temp_Rgn_Ptr = &((**(dreaming_Ptr->puppet[current_Index])).direct_Art.frame_Regions[current_Frame]);
					if (temp_Rgn_Ptr != nil) {
						DisposeRgn(*temp_Rgn_Ptr);
					}
					
					temp_Rgn_Ptr = &((**(dreaming_Ptr->puppet[current_Index])).direct_Art.frame_Buffered_Animation_Regions[current_Frame]);
					if (temp_Rgn_Ptr != nil) {
						DisposeRgn(*temp_Rgn_Ptr);
					}
				}
				
				temp_Rgn_Ptr = &((**(dreaming_Ptr->puppet[current_Index])).direct_Art.total_Lazy_Active_Region);
				if (temp_Rgn_Ptr != nil) {
					DisposeRgn(*temp_Rgn_Ptr);
					(**(dreaming_Ptr->puppet[current_Index])).direct_Art.total_Lazy_Active_Region = nil;
				}
				
				temp_Rgn_Ptr = &((**(dreaming_Ptr->puppet[current_Index])).direct_Art.original_Total_Lazy_Active_Region);
				if (temp_Rgn_Ptr != nil) {
					DisposeRgn(*temp_Rgn_Ptr);
				}
				
				temp_Rgn_Ptr = &((**(dreaming_Ptr->puppet[current_Index])).direct_Art.buffered_Animation_Total_Region);
				if (temp_Rgn_Ptr != nil) {
					DisposeRgn(*temp_Rgn_Ptr);
				}
			}
			else {
			
				/*  */
				/* We need to close any open files if this is actually in use. */
				/* Hopefully that was just done a moment ago. */
				/*  */
				if ((**(dreaming_Ptr->puppet[current_Index])).redirected_Information_Memory_In_Use == SAFE_TRUE) {

					redirected_Information_Total_Used = (**(dreaming_Ptr->puppet[current_Index])).redirected_Information_Total_Used;
					if (redirected_Information_Total_Used > 0) {
						
						for (loop = 0; (loop < redirected_Information_Total_Used) ; loop++) {
							loaded_Info_Ptr = &((**(dreaming_Ptr->puppet[current_Index])).redirected_Information_Ptr[loop]);
							
							if (loaded_Info_Ptr->valid == SAFE_TRUE) {
								/*  */
								/* Nothing special to do. */
								/*  */
							}
							
							loaded_Info_Ptr->valid = SAFE_FALSE;
						}
					}		
				}	
					
			} /* NOT if ((**(dreaming_Ptr->puppet[current_Index])).puppet_File_Is_Old_Self_Contained_Type ... */
			


			if ((**(dreaming_Ptr->puppet[current_Index])).redirected_Information_Memory_In_Use == SAFE_TRUE) {
						
				/*  */
				/* We need to close any open files if this is actually in use. */
				/* Hopefully that was just done a moment ago. */
				/*  */

				if (Destroy_Handle((Handle *)&(**(dreaming_Ptr->puppet[current_Index])).redirected_Information_Hdl
									, &(**(dreaming_Ptr->puppet[current_Index])).redirected_Information_Memory_In_Use) != SAFE_TRUE) {
									
					Display_Serious_Error_Message(unit_Name, code_Name, "\pError with DisposHandle", "\pSystem Corrupted?");
					goto Error_Handler;
				}
				else {
					(**(dreaming_Ptr->puppet[current_Index])).redirected_Information_Ptr = nil;
				}
			}

			if (Destroy_A_Puppet_Structure(current_Index) != SAFE_TRUE) {
				SysBeep(20);
				goto Error_Handler;
			}
			
		} /* if (dreaming_Ptr->puppet_Memory_Initialized[ ... */
		

	function_Result = SAFE_TRUE;
	goto Normal_Function_Exit;
	
Error_Handler:
	Display_Serious_Error_Message(unit_Name, code_Name, "\pProblem encountered", "\p");
	function_Result = SAFE_FALSE;
		
Normal_Function_Exit:

	return(function_Result);
}
/*=**************************************************************************=*/


/*=**************************************************************************=*/
Safe_Boolean	Destroy_Dream_Information(void)
{
	Safe_Boolean		function_Result;
	INTEGER_32		current_Index;
	INTEGER_32		total_Puppets;
	CARDINAL_8		*code_Name;
	INTEGER_16		puppets_Found;
/* END VAR */


	code_Name = "\pDestroy_Dream_Information";
	function_Result = SAFE_FALSE;

	if (g_Dreaming_Module_Globals_Initialized != SAFE_TRUE) {
		function_Result = SAFE_TRUE;
		goto Normal_Function_Exit;
	}

	if (dreaming_Ptr->initialized == SAFE_TRUE) {
		total_Puppets = dreaming_Ptr->active_Puppets_Total;
		
		puppets_Found = 0;
		for (current_Index = k_MAXIMUM_PUPPETS; ((current_Index >= 0) && (puppets_Found < total_Puppets)); current_Index -= 1) {
		
			if (dreaming_Ptr->puppet_Memory_Initialized[current_Index] == MEMORY_INITIALIZED) {
				puppets_Found += 1;
			
				if (Delete_A_Loaded_Puppet(current_Index) != SAFE_TRUE) {
					/*  */
					/* We are quitting anyway */
					/*  */
				}	
			}		
				
		} /* for (current_Index = 0; (current_Index < total_Puppets) ... */
		
	} /* if (dreaming_Ptr->initialized == SAFE_TREU) */
	
	dreaming_Ptr->active_Puppets_Total = 0;

	if (Reset_Menu_Control_Selectability() != SAFE_TRUE) {
		/*  */
		/* Not normal to ever fail */
		/*  */
		function_Result = SAFE_FALSE;
	}

	function_Result = SAFE_TRUE;
	goto Normal_Function_Exit;
	
Error_Handler:
	Display_Serious_Error_Message(unit_Name, code_Name, "\pProblem encountered", "\p");
	function_Result = SAFE_FALSE;
		
Normal_Function_Exit:

	return(function_Result);
	
} /* Safe_Boolean	Destroy_Dream_Information ... */
/*=**************************************************************************=*/


/*=**************************************************************************=*/																	
Safe_Boolean	Create_Region_For_Frame(CARDINAL_32 puppet_Index
																	, CARDINAL_32 frame
																	, RgnHandle *mask
																	, Boolean	sequence_Has_Background)
{
	Safe_Boolean		function_Result;
	Puppet_Ptr			current_Puppet;
	GrafPtr				offscreen_Source_Port;
	Rect 					source_Rect;
	INTEGER_32		vertical;
	INTEGER_16		horizontal;
/* END VAR */
	
	
	if (puppet_Index > dreaming_Ptr->active_Puppets_Total) {
		goto Error_Handler;
	}
	
	current_Puppet = *(dreaming_Ptr->puppet[puppet_Index]);
	
	if (Check_If_Legal_Direct_Art_Or_Complain(current_Puppet) != SAFE_TRUE) {
		goto Error_Handler;
	}

	if (current_Puppet->direct_Art.port.initialized != SAFE_TRUE) {
		goto Error_Handler;
	}
	
	offscreen_Source_Port	= (GrafPtr)&(current_Puppet->direct_Art.port.offscreen_Port);
	
	source_Rect  = current_Puppet->direct_Art.frame_Locations[frame];

	if (sequence_Has_Background != TRUE) {
	
		/*  */
		/* Remove the background (create region) */
		/*  */
		
		if (Create_Region_For_Port_Image(offscreen_Source_Port
																	, &source_Rect
																	, mask) != SAFE_TRUE) {
			goto Error_Handler;													
		}
	
	} /* if (sequence_Has_Background != TRUE) */
	else {
		*mask = NewRgn();
		/*  */
		/* 1994.06.09 - Its vital to adjust the rect! */
		/*  */
		vertical = current_Puppet->direct_Art.frame_Locations[0].top - source_Rect.top;
		horizontal = current_Puppet->direct_Art.frame_Locations[0].left - source_Rect.left;
		OffsetRect(&source_Rect, vertical, horizontal);
		RectRgn(*mask, &source_Rect);
	}
	
	function_Result = SAFE_TRUE;
	goto Normal_Function_Exit;
		
Error_Handler:
	function_Result = SAFE_FALSE;

Normal_Function_Exit:

	return(function_Result);
	
} /* Safe_Boolean	Create_Region_For_Frame ... */
/*=**************************************************************************=*/


/*=**************************************************************************=*/
void Perform_Startup_Actions_For_Scan_Completion(void)
{
	Monitor_Microphone_Rec		params_For_Monitor_Microphone;
	Load_Or_Select_Scene_Rec		params_For_Load_Or_Select_Scene;
	Safe_Boolean								problems_Encounterred;
	Boolean										found_Item;
	INTEGER_16								index;
	CARDINAL_32							total_Listed_Items;
	CARDINAL_32							items_Found;
	CARDINAL_32							total_Items;
	Str255											startup_Scene_File_Name;
/* END VAR */


	/*  */
	/* This procedure is meant to be called after every full disk scan completion. */
	/* Note that it is currently only intended to do something at initial startup. */
	/* It SHOULD open the file most recently opened and SHOULD open */
	/* the default sound inputs most recently opened. */
	/* Currently it tries to open internal sound input device if present. */
	/* That device can have two different names. */
	/* "DSP Sound Chip Support." or "???" */
	/* It also tries to open Default scene. Currently the default scene is setup */
	/* to "Startup Scene" */
	/*  */

	problems_Encounterred = SAFE_FALSE;
	if (g_Startup_Initialization_Complete != SAFE_TRUE) {
		g_Startup_Initialization_Complete = SAFE_TRUE;
		
		
		params_For_Monitor_Microphone.async_Task_Desired = SAFE_FALSE;
		params_For_Monitor_Microphone.select_Any_Input = SAFE_TRUE;
		
		if (Monitor_Microphone(&params_For_Monitor_Microphone) != SAFE_TRUE) {
			/*  */
			/* Ignore SID initialization. It is not important. */
			/*  */
			problems_Encounterred = SAFE_TRUE;
		}
	
		params_For_Load_Or_Select_Scene.async_Task_Desired = SAFE_FALSE;
		
		if (g_Dreaming_Module_Globals_Initialized == SAFE_TRUE) {
			if (g_Remote_Scenes.total_Available > 0) {
				Copy_Pascal_String(k_STARTUP_SCENE_DEFAULT_FILE_NAME, startup_Scene_File_Name);
				found_Item = FALSE;
				total_Listed_Items = 0;

				for (index = 0; ((index < k_MAXIMUM_DATABASE_SCENES) && (total_Listed_Items < g_Remote_Scenes.total_Available)); index++) {
								
					if ((**g_Remote_Scenes.info.scenes.handle).scenes[index].active == SAFE_TRUE) {

						total_Listed_Items += 1;				
						if (Compare_Pascal_Strings_Ignoring_Case((**g_Remote_Scenes.info.scenes.handle).scenes[index].scene_Name
										, startup_Scene_File_Name) == SAFE_TRUE) {
								found_Item = TRUE;
								params_For_Load_Or_Select_Scene.desired_Scene = index;
						}
					}
				}
				
				if (found_Item == TRUE) {
					if (Load_Or_Select_Scene(&params_For_Load_Or_Select_Scene) != SAFE_TRUE) {
						problems_Encounterred = SAFE_TRUE;
					}
				}
			}
		}
		
	
	} /* if (g_Startup_Initialization_Complete != SAFE_TRUE) */
}
/*=**************************************************************************=*/


/*=**************************************************************************=*/
Safe_Boolean	Load_PICS_Sequence(CARDINAL_32 index_Used
															, Safe_Boolean *user_Abort)
{
	WindowPtr			current_Graphics_Window_Ptr;
	Safe_Boolean		function_Result;
	Safe_Boolean		resource_Sources_Swapped;
	Safe_Boolean		swap_Palettes;
	INTEGER_16 		number_Of_Types;
	INTEGER_16		save_Ref;
	SFTypeList 			type_List;
	SFReply				sf_Reply;
	INTEGER_16		source_Volume_Reference;
	INTEGER_16		input_File_Reference;
	INTEGER_32		source_Directory_ID;
	OSErr					os_Error;
	INTEGER_32		total_Bytes_Contiguous_Now;
	INTEGER_32		total_Bytes_Now;
	INTEGER_32 		total_Bytes_Contiguous_Later;
	INTEGER_32		total_Bytes_Later;
	CTabHandle		temp_Clut_Table;
	PaletteHandle	temp_Palette;
	PaletteHandle	final_Special_Pallete_To_Use;
	CARDINAL_32	special_Colors_In_Clut;
	Puppet_Ptr			current_Puppet;
	Handle				temp_Resource;
	RgnHandle			temp_Region;
	INTEGER_32		loop;
	INTEGER_32		first_Pict_Id;
	INTEGER_32		unit_Width;
	INTEGER_32		unit_Height;
	INTEGER_16		resource_Id;
	INTEGER_16		desired_Depth;
	PictInfo				picture_Info;
	ResType				resource_Type;
	Str255					message_String;
	Str255					temp_String;
	Str255					temp_String_2;
	Str255					status_String;
	Boolean				user_Clicked_Ok;
	INTEGER_16		total_Picts;
	Rect						expected_Target_Rect;
	Rect						master_Puppet_Rect;
	CARDINAL_16	desired_Depth_If_Over_8_Bits;
	CARDINAL_32	actual_Total_Of_Special_Colors;
	CARDINAL_32	master_Puppet_Width;                                                                                                                                                                                                                                        
	CARDINAL_32	master_Puppet_Height;
	CARDINAL_32	expected_Total_Pixel_Height;
	Boolean				sequence_Has_Background;
	CARDINAL_8		*code_Name;
	CARDINAL_32	current_Frame;
	Import_Options	options;
/* END VAR */


	code_Name = "\pLoad_PICS_Sequence";
	function_Result = SAFE_FALSE;
	*user_Abort = SAFE_FALSE;
	
	resource_Sources_Swapped = SAFE_FALSE;
	
	
	Get_NS_String(k_DREAM_STRINGS, k_LOAD_PICS_FILE_INDEX, message_String);
	Perform_OK_Dialog(message_String);
	
	number_Of_Types = 1;
	type_List[0] = 'PICS';
/* ====
	type_List[1] = FILE_CREATOR;
=== */



	user_Clicked_Ok = Perform_Get_File_Dialog(number_Of_Types
										, &(type_List)
										, &sf_Reply);
											
	
	if ((user_Clicked_Ok != TRUE) || (sf_Reply.good != TRUE)) {
		/*  */
		/* User Abort, no file name */
		/*  */
		*user_Abort = SAFE_TRUE;
		function_Result = SAFE_FALSE;
		goto Normal_Function_Exit;
	}
	
	Get_Last_Volume_Reference_And_Directory_ID(&source_Volume_Reference, &source_Directory_ID);
										
	/*  */
	/* Open for read only permission */
	/*  */
	
	save_Ref = CurResFile();
	
	input_File_Reference = HOpenResFile (source_Volume_Reference
						, source_Directory_ID
						, sf_Reply.fName
						, fsRdPerm );
						
	if ((input_File_Reference == -1) || (ResError() != noErr)) {
		goto Error_Handler;
	}
	
													
	UseResFile(input_File_Reference);
	os_Error = ResError();
	if (os_Error != noErr) {
		Display_Serious_Error_Message(unit_Name, code_Name, "\pInternal using resource", "\pCorrupted?");
		goto Error_Handler;
	}
	
	resource_Sources_Swapped = SAFE_TRUE;

	
	/*  */
	/* Load PICS into memory, allocating a graphics port */
	/* big enough to hold all images vertically into one graphics device port */
	/*  */
	
		
	total_Picts = Count1Resources('PICT');
	if (ResError() != noErr) {
		goto Error_Handler;
	}
	
	if (total_Picts < 1) {
		Get_NS_String(k_DREAM_STRINGS, k_SORRY_WRONG_FILE_INDEX, message_String);
		Perform_OK_Dialog(message_String);
		goto Error_Handler;
	}
	
	if (total_Picts > k_MAXIMUM_PICS_FRAMES) {
		total_Picts = k_MAXIMUM_PICS_FRAMES;
	}
		
	if (Determine_Absolute_Free_Memory_Hook(&total_Bytes_Contiguous_Now
													,&total_Bytes_Now
													,&total_Bytes_Contiguous_Later
													,&total_Bytes_Later) != TRUE) {
		goto Error_Handler;
	}
		
	/*  */
	/* This is dependant on the validity of "current_Window_Index" */
	/*  */
	if ((**active_Window_List[TEST_DREAMING_WINDOW_INDEX]).structure_Initialized != SAFE_TRUE) {
		SysBeep(20); 
		Assign_String(ns_Shell_Globals.error_Description_String, "\pError. Improperly initialized window structure");
		Assign_String(ns_Shell_Globals.error_Note_String, "\pProgrammer flaw");
		Display_Error_Message(unit_Name, code_Name);
		Setup_Error_Handling();
		goto Error_Handler;
	}
	else {
		current_Graphics_Window_Ptr	= (**active_Window_List[TEST_DREAMING_WINDOW_INDEX]).status_Window;
	}
		
	/*  */
	/* current_Puppet uses to a handle assumed to always be locked */
	/*  */
	current_Puppet = *(dreaming_Ptr->puppet[index_Used]);
	if (current_Puppet->direct_Art.frame_Total != 0 ) {
		Copy_Pascal_String("\pUNIMPLEMENTED: This version cannot append new Pict data into existing list", message_String);
		Perform_OK_Dialog(message_String);
		goto Error_Handler;
	}

	
	Strip_Off_Popular_Suffixes(&(sf_Reply.fName));
	Assign_String(current_Puppet->file_Name, sf_Reply.fName);
	Assign_String(current_Puppet->screen_Name, sf_Reply.fName);
	Assign_String(current_Puppet->screen_Short_Name, sf_Reply.fName);

	temp_Resource = Get1IndResource ('PICT', 1);
	if (ResError() != noErr) {
		Display_Serious_Error_Message(unit_Name
															, code_Name
															, "\pInternal problem with resources"
															, "\pCorrupted?");
		goto Error_Handler;
	}
		
	GetResInfo(temp_Resource
						, &resource_Id
						, &resource_Type
						, &temp_String[0]);
						
	if (ResError() != noErr) {
		goto Error_Handler;
	}
	
	first_Pict_Id = resource_Id;
	
	current_Puppet->total_Special_Colors_To_Honor = 0;
	current_Puppet->total_Colors_Used_In_Pallette = 0;

/* ===
	os_Error = GetPictInfo((PicHandle)temp_Resource
					, &picture_Info
					, (returnPalette + suppressBlackAndWhite)
					, k_MAXIMUM_SPECIAL_COLORS
					, popularMethod
					, 0);
===== */

	os_Error = GetPictInfo((PicHandle)temp_Resource
					, &picture_Info
					, (returnPalette)
					, k_MAXIMUM_SPECIAL_COLORS
					, popularMethod
					, 0);

	if (Get_Import_Options(&	options, total_Picts) != TRUE) {
		goto Error_Handler;
	}

	desired_Depth = g_Preferred_Monitor_Depth; /* HACK HACK HACK screen depth */
	if (os_Error == noErr) {
		if (picture_Info.depth > 8 ) {
			
			Get_NS_String(k_DREAM_STRINGS
							, k_IMPORT_PUPPET_FILE_MAX_COLORS_SUFFIX
							, message_String);
			
			Concat_Long_To_String(message_String, picture_Info.depth, NO_LIMIT);
				
			if (Perform_Yes_No_Dialog(message_String) != TRUE) {
				desired_Depth = picture_Info.depth;
			}
		}
	}
	else {
		Get_NS_String(k_DREAM_STRINGS
						, k_IMPORT_PUPPET_NO_PICT_ANALYSIS_POSSIBLE
						, message_String);
											
		Perform_OK_Dialog(message_String);
		goto Error_Handler;

	}

	if (desired_Depth <= 8) {
		if ((os_Error == noErr) && ((**(picture_Info.thePalette)).pmEntries > 0)){
			/*  */
			/* Information can be utilized. */
			/* First set ALL to intolerant, soon we will reset those unneeded, that */
			/* are not special colors, MAYBE */
			/*  */
			current_Puppet->total_Colors_Used_In_Pallette = (**(picture_Info.thePalette)).pmEntries;
			if (current_Puppet->total_Colors_Used_In_Pallette <= picture_Info.uniqueColors) {
				for (loop = 0; (loop < current_Puppet->total_Colors_Used_In_Pallette) ; loop++) {
					SetEntryUsage(picture_Info.thePalette, loop, pmTolerant, k_PUPPET_INTOLERENT);
				}
			}
			else {
				Display_Serious_Error_Message(unit_Name
																		, code_Name
																		, "\ptotal_Colors_Used_In_Pallette seems wierd"
																		, "\pExtra entries");
			
			}
		}
		else {
			current_Puppet->total_Colors_Used_In_Pallette = 0;
		}
	}
	else {
		current_Puppet->total_Colors_Used_In_Pallette = 0;
	}
		
	/*  */
	/* Using the first frame (The master puppet frame) */
	/* set limits on the puppet about to be created if needed */
	/* Note though that cropping is arbitrary in this version */
	/* and is not self center adjusting */
	/*  */
	master_Puppet_Rect = (*(PicHandle)temp_Resource)->picFrame;
	master_Puppet_Width = master_Puppet_Rect.right - master_Puppet_Rect.left;
	master_Puppet_Height = master_Puppet_Rect.bottom - master_Puppet_Rect.top;
	
	if (master_Puppet_Width > g_Window_Limits.specific.display_Item_Width_Max) {
		master_Puppet_Rect.right = master_Puppet_Rect.right 
										- (master_Puppet_Width - g_Window_Limits.specific.display_Item_Width_Max);
		master_Puppet_Width = g_Window_Limits.specific.display_Item_Width_Max;
	}
	
	if (master_Puppet_Height > g_Window_Limits.specific.display_Item_Height_Max) {
		master_Puppet_Rect.bottom = master_Puppet_Rect.bottom 
										- (master_Puppet_Height - g_Window_Limits.specific.display_Item_Height_Max);
		master_Puppet_Height = g_Window_Limits.specific.display_Item_Height_Max;
	}
	
	/*  */
	/* Put away the resource, we extracted the information we wanted */
	/*  */
	ReleaseResource(temp_Resource);
	if (ResError() != noErr) {
		goto Error_Handler;
	}
		

	/*  */
	/* Determine if the special colors being reserved are really special or not */
	/* in a normal system. */
	/* First, we must restore the current resource search location */
	/*  */
	
	if (resource_Sources_Swapped == SAFE_TRUE) {
		
		UseResFile(save_Ref);
		if (ResError() != noErr) {
			/*  */
			/* This should be recoverable from, enhance one day  */
			/*  */
		}
		resource_Sources_Swapped = SAFE_FALSE;
	}
		
	final_Special_Pallete_To_Use = nil;
	
	if (desired_Depth <= 8) {
		if (Tally_Colors_Not_In_Default_Clut(picture_Info.thePalette
								, &actual_Total_Of_Special_Colors
								, SAFE_TRUE) 
							== SAFE_TRUE) {
	
	
			if (Activate_Palette_And_Verify_Intolerant(current_Graphics_Window_Ptr
																					, picture_Info.thePalette) != SAFE_TRUE) {
				/*  */
				/* Nothing special to do in this version for errors, just continue */
				/*  */
			}
			
			if (current_Puppet->total_Colors_Used_In_Pallette != 0) {
			
				final_Special_Pallete_To_Use = picture_Info.thePalette;
				if (HandToHand((Handle *)&final_Special_Pallete_To_Use) != noErr) {
					Display_Serious_Error_Message(unit_Name
																			, code_Name
																			, "\pHandToHand failed"
																			, "\pLow memory?");
					goto Error_Handler;
				}
			}
			else {
				final_Special_Pallete_To_Use = nil;
				actual_Total_Of_Special_Colors = 0;
			}		
	
			current_Puppet->total_Special_Colors_To_Honor = actual_Total_Of_Special_Colors;
	
			/*  */
			/* Ask User If any file containing a CLUT should be used as */
			/* a reference CLUT file. */
			/*  */
			if (Determine_If_Another_Clut_Wanted(current_Puppet->total_Colors_Used_In_Pallette
																				, actual_Total_Of_Special_Colors)
																== SAFE_TRUE) {
																
				if (Ask_User_For_Another_Clut(&temp_Clut_Table) == SAFE_TRUE) {
				
					if (Create_Palette_From_Clut(temp_Clut_Table, &temp_Palette) == SAFE_TRUE) {
	
						if (Tally_Colors_Not_In_Default_Clut(temp_Palette
									, &special_Colors_In_Clut
									, SAFE_TRUE) == SAFE_TRUE) {
								
							if (special_Colors_In_Clut > 1) {
							/*  */
							/* Finally, we have proved that there are some special colors */
							/* in a desired Clut, before using it we should check to see if it */
							/* has more special colors than the pallete we are eliminating */
							/* otherwise this is foolish */
							/*  */
								swap_Palettes = SAFE_FALSE;
	
								if (special_Colors_In_Clut < actual_Total_Of_Special_Colors) {
							
									Get_NS_String(k_DREAM_STRINGS
													, k_SPECIAL_COLORS_WAS_LESS_INDEX
													, message_String);
									
									Concat_Long_To_String(message_String, special_Colors_In_Clut, NO_LIMIT);
										
									if (Perform_Boolean_Dialog(message_String) == TRUE) {
										swap_Palettes = SAFE_TRUE;
									}
								}
								else {
									swap_Palettes = SAFE_TRUE;
								}
								
								if (swap_Palettes == SAFE_TRUE) {
									/*  */
									/* I don't know what to do about the original palette. Dispose after? */
									/*  */
									if (Activate_Palette_And_Verify_Intolerant(current_Graphics_Window_Ptr
																											, temp_Palette) != SAFE_TRUE) {
										/*  */
										/* Nothing special to do in this version for errors, just continue */
										/*  */
									}
									
									final_Special_Pallete_To_Use = temp_Palette;
									if (HandToHand((Handle *)&final_Special_Pallete_To_Use) != noErr) {
										Display_Serious_Error_Message(unit_Name
																								, code_Name
																								, "\pHandToHand failed"
																								, "\pLow memory?");
										goto Error_Handler;
									}
																	
								} /* if (swap_Palettes == SAFE_TRUE) */
							}
							else {
								Get_NS_String(k_DREAM_STRINGS
												, k_SELECTED_CLUT_NOT_SPECIAL_INDEX
												, message_String);
																	
								Perform_OK_Dialog(message_String);
								
							} /* NOT if (special_Colors_In_Clut > 1) */
							
							current_Puppet->total_Special_Colors_To_Honor = special_Colors_In_Clut;
							
						} /* if (Tally_Colors_Not_In_Default_Clut(temp_Palette ... */
						
						DisposePalette(temp_Palette);
						
					} /* if (Create_Pallette_From_Clut(&temp_Palette) */
					
					DisposeCTable(temp_Clut_Table);
					
				} /* ( if (Ask_User_For_Another_Clut(&temp_Clut_Table) ... */
				
			} /* if (Determine_If_Another_Clut_Wanted(current_Puppet->total_Colors_Used_In_Pallette... */
			
		} /* if (Tally_Colors_Not_In_Default_Clut(picture_Info.thePalette ... */
	}
	
	/*  */
	/* 1994.07.04 JAM possible bug fix */
	/*  */
	if ((picture_Info.thePalette != nil) && ((**(picture_Info.thePalette)).pmEntries)) {
		DisposePalette(picture_Info.thePalette);
		picture_Info.thePalette = nil;
	}
		
	/*  */
	/* Restore the resource source location */
	/*  */
	save_Ref = CurResFile();
	
	UseResFile(input_File_Reference);
	os_Error = ResError();
	if (os_Error != noErr) {
		
		Display_Serious_Error_Message(unit_Name, code_Name, "\pInternal using resource", "\pCorrupted? 3");
		goto Error_Handler;
	}
	
	resource_Sources_Swapped = SAFE_TRUE;
				
	expected_Target_Rect = ((WindowPeek)current_Graphics_Window_Ptr)->port.portRect;
	LocalToGlobal(&TOP_LEFT(expected_Target_Rect));
	LocalToGlobal(&BOTTOM_RIGHT(expected_Target_Rect));
	
	/*  */
	/* HACK HACK HACK, In the final version, precompute the exact */
	/* height in pixels that is expected to be used */
	/*  */
	expected_Total_Pixel_Height = (master_Puppet_Rect.bottom - master_Puppet_Rect.top);
	expected_Total_Pixel_Height = expected_Total_Pixel_Height * total_Picts;
	
	if ( expected_Total_Pixel_Height > (32767L)) {
	
		if (Perform_Yes_No_Dialog("\pThis is a HUGE thing. Are you sure?") != TRUE) {
			goto Error_Handler;
		}
		expected_Total_Pixel_Height = 32767L;
	}
	
	Get_NS_String(k_DREAM_STRINGS
				, k_LOADING_AND_CONVERTING_TO_CURRENT_COLORS
				, message_String); 
				
	Display_Message_Low(message_String);
	
	
	
	Animate_Cursor(STANDARD_ACUR_ID, DO_NOT_USE_VBL);
	
/* ===
	Animate_Cursor_Tickle();
	Animate_Cursor_Tickle();
	Animate_Cursor_Tickle();
	Animate_Cursor_Tickle();
	Animate_Cursor_Tickle();
	Animate_Cursor_Tickle();
	Stop_Animating_Cursor();
	
	Animate_Cursor(STANDARD_ACUR_ID, USE_VBL);
==== */
	
	if (desired_Depth > 8) {
		desired_Depth_If_Over_8_Bits = desired_Depth;
	}
	else {
		desired_Depth_If_Over_8_Bits = 0;
	}
	
	if (Get_Picts_Into_One_Bit_Map(&(current_Puppet->direct_Art.port)
													, first_Pict_Id
													, total_Picts
													, &(current_Puppet->direct_Art.frame_Locations[0])
													, &master_Puppet_Rect
													, expected_Total_Pixel_Height
													, desired_Depth_If_Over_8_Bits
													, &expected_Target_Rect) != SAFE_TRUE) {
													
			SysBeep(20);
			Stop_Animating_Cursor();
			Dispose_Message();
			goto Error_Handler;
	}
	
	Stop_Animating_Cursor();
	
	
	current_Puppet->special_Pallete_To_Use = final_Special_Pallete_To_Use;
	
	/*  */
	/* The Picts are loaded */
	/*  */	
	current_Puppet->direct_Art.frame_Total += total_Picts;
	
	/*  */
	/* HACK HACK HACK !!!! set to be full total in this version!!! */
	/*  */
	current_Puppet->direct_Art.talk_Total_Frames +=total_Picts;
	
	current_Puppet->direct_Art.talk_Start_Frame = 0;
	current_Puppet->direct_Art.talk_Stop_Frame = current_Puppet->direct_Art.talk_Start_Frame 
																	+ (current_Puppet->direct_Art.talk_Total_Frames  -1);
																	
								
	/*  */
	/* Validate that all rectangles are identical sizes, otherwise PICS */
	/* file for source was a composite PICS file or hand made */
	/*  */
	if (current_Puppet->direct_Art.frame_Total > 0) {
	
		unit_Width = current_Puppet->direct_Art.frame_Locations[current_Puppet->direct_Art.talk_Start_Frame].right;
		unit_Width = unit_Width - current_Puppet->direct_Art.frame_Locations[current_Puppet->direct_Art.talk_Start_Frame].left;
		
		unit_Height = current_Puppet->direct_Art.frame_Locations[current_Puppet->direct_Art.talk_Start_Frame].bottom;
		unit_Height = unit_Height - current_Puppet->direct_Art.frame_Locations[current_Puppet->direct_Art.talk_Start_Frame].top;
		
		for (current_Frame = 1
					; (current_Frame < current_Puppet->direct_Art.frame_Total)
					; current_Frame++) {
			
			if ((unit_Width != (current_Puppet->direct_Art.frame_Locations[current_Frame].right 
						- current_Puppet->direct_Art.frame_Locations[current_Frame].left))
				|| (unit_Height != (current_Puppet->direct_Art.frame_Locations[current_Frame].bottom 
						- current_Puppet->direct_Art.frame_Locations[current_Frame].top)) ) {
					
				/*  */
				/* Final step: prepare masks */
				/*  */
				Get_NS_String(k_DREAM_ERROR_STRINGS
							, k_ERROR_PICS_FILE_POSSIBLY_NOT_COMPOSITE_OR_RECT_DEVIANCE
							, message_String);
							
				Perform_OK_Dialog(message_String);
				goto Error_Handler;
			}
		}
		
	} /* if (current_Puppet->direct_Art.frame_Total > 0) */

																	
	/*  */
	/* Final step: prepare masks */
	/*  */
	Get_NS_String(k_DREAM_STRINGS
				, k_PROCESSING_PICTURES_FOR_BACKGROUND_EDGES
				, message_String);
				
	Get_NS_String(k_DREAM_STRINGS
				, k_REMAINING_FRAMES
				, temp_String);
				
	sequence_Has_Background = Perform_Yes_No_Dialog(
					"\pDoes this sequence have a background embedded in it?");
				
/* ===
	Animate_Cursor(STANDARD_ACUR_ID, USE_VBL);
=== */
	Animate_Cursor(STANDARD_ACUR_ID, DO_NOT_USE_VBL);
	
	
	for (current_Frame = 0
									; (current_Frame < current_Puppet->direct_Art.frame_Total)
									; current_Frame++) {
		
		current_Puppet->direct_Art.frame_Regions[current_Frame] = nil;
		
		Convert_Long_To_String((current_Puppet->direct_Art.frame_Total - current_Frame), temp_String_2, NO_LIMIT);
		Concat_Many(status_String, 3, message_String, temp_String_2, temp_String);
		Display_Message_Low(status_String);
		
		if (Create_Region_For_Frame(index_Used
				, current_Frame
				, &(current_Puppet->direct_Art.frame_Regions[current_Frame])
				, sequence_Has_Background) != SAFE_TRUE) {
			Stop_Animating_Cursor();
			Dispose_Message();
			goto Error_Handler;
		}
	}
	
	/*  */
	/* Determine maximum bounding mask of animation area */
	/*  */
	current_Puppet->direct_Art.total_Lazy_Active_Region = NewRgn();
	if (current_Puppet->direct_Art.total_Lazy_Active_Region == nil) {
		goto Error_Handler;
	}
	
	temp_Region = NewRgn();
	if (temp_Region == nil) {
		goto Error_Handler;
	}
	
	for (current_Frame = 0
									; (current_Frame < current_Puppet->direct_Art.frame_Total)
									; current_Frame++) {
		/*  */
		/* Copy region before using it for a paranoia related reason */
		/* too abstract to even bother really explaining here */
		/*  */
		CopyRgn(current_Puppet->direct_Art.frame_Regions[current_Frame], temp_Region);
		UnionRgn(temp_Region
							, current_Puppet->direct_Art.total_Lazy_Active_Region
							, current_Puppet->direct_Art.total_Lazy_Active_Region);
	}
	DisposeRgn(temp_Region);
	
	
	Stop_Animating_Cursor();
	Dispose_Message();

	/*  */
	/* Make a copy of "total_Lazy_Active_Region" so that it */
	/* can be modiifed larger for jiggle effect */
	/*  */
	if (current_Puppet->direct_Art.total_Lazy_Active_Region != nil) {
		current_Puppet->direct_Art.original_Total_Lazy_Active_Region = NewRgn();
		CopyRgn(current_Puppet->direct_Art.total_Lazy_Active_Region, current_Puppet->direct_Art.original_Total_Lazy_Active_Region);
	}
	else {
		current_Puppet->direct_Art.original_Total_Lazy_Active_Region = nil;
	}

	/*  */
	/* Modify it larger by four pixels for jiggle and motion effect */
	/*  */
	InsetRgn(current_Puppet->direct_Art.total_Lazy_Active_Region, -(dream.horizontal_Jiggle_Maximum), -(dream.vertical_Jiggle_Maximum));


	/*  */
	/* Initialize buffered copies of specialized resources */
	/*  */
	for (loop = 0; (loop < current_Puppet->direct_Art.frame_Total) ; loop++) {
		if (current_Puppet->direct_Art.frame_Regions[loop] != nil) {
			current_Puppet->direct_Art.frame_Buffered_Animation_Regions[loop] = NewRgn();
			CopyRgn(current_Puppet->direct_Art.frame_Regions[loop], current_Puppet->direct_Art.frame_Buffered_Animation_Regions[loop]);
		}
		else {
			current_Puppet->direct_Art.frame_Buffered_Animation_Regions[loop] = nil;
		}
		current_Puppet->direct_Art.frame_Buffered_Animation_Region_Offset[loop].h = 0;
		current_Puppet->direct_Art.frame_Buffered_Animation_Region_Offset[loop].v = 0;
	}
	
	if (current_Puppet->direct_Art.total_Lazy_Active_Region != nil) {
		current_Puppet->direct_Art.buffered_Animation_Total_Region = NewRgn();
		CopyRgn(current_Puppet->direct_Art.total_Lazy_Active_Region, current_Puppet->direct_Art.buffered_Animation_Total_Region);
	}
	else {
		current_Puppet->direct_Art.buffered_Animation_Total_Region = nil;
	}
	current_Puppet->direct_Art.buffered_Animation_Total_Region_Offset.h = 0;
	current_Puppet->direct_Art.buffered_Animation_Total_Region_Offset.v = 0;

	
	/*  */
	/* Something was loaded, enable all items */
	/*  */
	if (Reset_Menu_Control_Selectability() != SAFE_TRUE) {
		goto Error_Handler;
	}	
	
	
	function_Result = SAFE_TRUE;
	goto Normal_Function_Exit;
	
Error_Handler:
	Display_Serious_Error_Message(unit_Name, code_Name, "\pInternal problem getting PICS", "\p");

	function_Result = SAFE_FALSE;
		
Normal_Function_Exit:
	if (resource_Sources_Swapped == SAFE_TRUE) {
		CloseResFile(input_File_Reference);
		if (ResError() != noErr) {
			goto Error_Handler;
		}
		
		UseResFile(save_Ref);
		if (ResError() != noErr) {
			/*  */
			/* This should be recoverable from, enhance one day  */
			/*  */
		}
		resource_Sources_Swapped = SAFE_FALSE;
	}

	return(function_Result);
	
} /* Safe_Boolean	Load_PICS_Sequence... */
/*=**************************************************************************=*/


/*=**************************************************************************=*/
/* END OF FILE: Dreaming_Stuff.c */
