/* START OF FILE: Dreaming_Stuff.h */


/*=**************************************************************************=*/
/* [---BEGIN---] FILE HEADER COMMENTS */
/**
 ** SCREEN VIEW = [Adobe Syntax Black-10, Tabs = 4]
 ** PRINTER VIEW = [Adobe Syntax Black-10, Tabs = 4]
 ** =======================================================================
 ** Written by James Merkle
 ** Dreaming_Stuff.h - Header to support Poor Man's Video Phone
 ** (C) Copyright 1993 James Merkle
 ** =======================================================================
 **
 **
 ** ============
 ** Revision History
 ** ============
 **
 **  Refer to "Dreaming_Stuff.c"
 **						
 **
 ** ====================
 ** OTHER INFORMATION
 ** ====================
 ** 
 **
 **/
/* [----END----] FILE HEADER COMMENTS */
/*=**************************************************************************=*/



/*=**************************************************************************=*/
#ifndef _H_Dreaming_Stuff

#define _H_Dreaming_Stuff
/*=**************************************************************************=*/


/*=**************************************************************************=*/
/* [---BEGIN---] #include AREA */

#include <PictUtil.h>


#include "Application_Limits.h"
#include "Core_Specific.h"
#include "Shell_Globals.h"
#include "Off_Screen_Color.h"
#include "AsyncListManagerLib.h"
#include "Dreaming_Audio.h"
#include "Stage_View_Manager.h"
#include "Spin_O_Rama.h"


/* [----END----] #include AREA */
/*=**************************************************************************=*/


/*=**************************************************************************=*/
/* [---BEGIN---] #define AREA */


/*  */
/* Default background PICT */
/*  */
#define k_DEFAULT_BACKGROUND_RES_ID			140



#define k_DREAM_ERROR_STRINGS													1201
#define k_NOTE_BACKDROP_WIDTH_WILL_CLIP									1
#define k_NOTE_BACKDROP_HEIGHT_WILL_CLIP								2
#define k_ERROR_BACKDROP_TOO_SMALL											3
#define k_ERROR_NO_CARD_LOCATED													4
#define k_ERROR_NO_CARD_AT_ALL_LOCATED									5
#define k_ERROR_CARD_WOULD_NOT_CLEAR_DAQ							6
#define k_ERROR_OFFSCREEN_NOT_8_BIT_SO_SYSTEM_USED			7
#define k_ERROR_LAB_CARD_TOO_MANY_RESTARTS_NEEDED_1	8
#define k_ERROR_LAB_CARD_TOO_MANY_RESTARTS_NEEDED_2	9
#define k_ERROR_NOT_ENOUGH_MEMORY_TO_SAFELY_LOAD		10
#define k_ERROR_NOT_ENOUGH_MEMORY_PUPPETS_USE_MEMORY		11
#define k_ERROR_EXPORTING_PUPPETS_USES_MEMORY					12
#define k_ERROR_SCXI_MODULE_AND_CHASSIS_NOT_DETECTED	13
#define k_ERROR_SYSTEM_SCREEN_IF_USED_MUST_BE_256_COLOR	14
#define k_ERROR_PICS_FILE_POSSIBLY_NOT_COMPOSITE_OR_RECT_DEVIANCE	15
#define k_ERROR_YOU_NEED_QUICKTIME_INSTALLED						16
#define k_ERROR_YOU_NEED_QUICKTIME_2_0_INSTALLED				17
#define k_ERROR_NO_ACCEPTABLE_DIGITAL_VIDEO_IN					18
#define k_NOTE_MOST_ANIMATION_IS_NOT_POWERPC_NATIVE	19
#define k_ERROR_SOURCE_NOT_ALIGNED_TO_16_BYTE					20
#define k_ERROR_DESTINATION_NOT_ALIGNED_TO_16_BYTE			21
#define k_ERROR_PUPPET_IS_NOT_SPIN_FILE_TYPE							22
#define k_ERROR_PUPPET_SPIN_FILE_TYPE_UNEXPECTED					23



#define k_MINIMUM_RAM_TO_PRESERVE_FOR_BIG_STUFF				(600000L)

#define k_DREAM_PROMPT_STRINGS													1202
#define k_ENTER_MAXIMUM_JITTER_DISTANCE_HORIZONTAL			1
#define k_ENTER_MAXIMUM_JITTER_DISTANCE_VERTICAL					2


#define k_IMPORT_FROM_NORMAL_LOAD			0
#define k_IMPORT_FROM_PICS_SEQUENCE			1
#define k_IMPORT_FROM_SPIN_FILE						2

/* [----END----] #define AREA */
/*=**************************************************************************=*/


/*=**************************************************************************=*/
/* [---BEGIN---] TYPE AREA */

typedef struct {
	CARDINAL_32			intensity;
	CARDINAL_8				horizontal_Angle;
	CARDINAL_8				vertical_Angle;
	CARDINAL_8				roll_Angle;
} Puppet_Orientation_Rec, *Puppet_Orientation_Ptr;

/*  */
/* These are actually stored in 12 bit maximum form */
/*  */
typedef struct {
	CARDINAL_32			intensity;
	CARDINAL_32			horizontal_Angle;
	CARDINAL_32			vertical_Angle;
	CARDINAL_32			roll_Angle;
} Puppet_Raw_Orientation_Rec, *Puppet_Raw_Orientation_Ptr;

 typedef struct {
	Safe_Boolean					valid_Information;
	CARDINAL_32				search_Key_1;
	CARDINAL_32				search_Key_2;
	Off_Screen_Port_Rec 		*port_Ptr;
	INTEGER_32					offset_Offscreen_Pixel_X;
	INTEGER_32					offset_Offscreen_Pixel_Y;
	CARDINAL_32				previous_Index;
 }Previously_Drawn_Frame_Rec, *Previously_Drawn_Frame_Ptr;

/*  */
/* These are stored in file within a Pup3 resource */
/* This resource must be sized divisible by MOD 4 bytes or array */
/* logic for variable length array in resource will fail!!! */
/*  */
 typedef struct {
	Str63									file_Name;
	CARDINAL_32					file_Creation_Time;
	CARDINAL_32					emotion_Key_Code;
	CARDINAL_16					cache_Technique_Desired;
	CARDINAL_16					special_Effect_Classification;
	CARDINAL_32					special_Effect_Value;
	CARDINAL_32					special_Effect_Extra_Value;
	Point									offset_Point_Position;
 }Redirected_File_Rec, *Redirected_File_Ptr;
 
 
 typedef struct {
 	Safe_Boolean						valid;
	/*  */
	/* Externally saved fields */
	/*  */
	Str63									file_Name;
	CARDINAL_32					file_Creation_Time;
	CARDINAL_32					emotion_Key_Code;
	CARDINAL_16					cache_Technique_Desired;
	CARDINAL_16					special_Effect_Classification;
	CARDINAL_32					special_Effect_Value;
	CARDINAL_32					special_Effect_Extra_Value;
	Point									offset_Point_Position;
	
	Safe_Boolean						array_Index_Valid;
	CARDINAL_32					array_Index;
	
 }Redirected_Art_Rec, *Redirected_Art_Ptr;

/*  */
/* These are all used in old style animation that stored all art in */
/* data fork of main file, and wated space by not compacting around */
/* the frames */
/*  */
typedef struct {
	Off_Screen_Port_Rec		port;
	CARDINAL_32				frame_Total;
	Rect									frame_Locations[k_MAXIMUM_PICS_FRAMES];
	RgnHandle						frame_Regions[k_MAXIMUM_PICS_FRAMES];
	RgnHandle						frame_Buffered_Animation_Regions[k_MAXIMUM_PICS_FRAMES];
	Point								frame_Buffered_Animation_Region_Offset[k_MAXIMUM_PICS_FRAMES];
	RgnHandle						total_Lazy_Active_Region;
	RgnHandle						original_Total_Lazy_Active_Region;
	RgnHandle						buffered_Animation_Total_Region;
	Point								buffered_Animation_Total_Region_Offset;
	CARDINAL_32				talk_Total_Frames;
	CARDINAL_32				talk_Start_Frame;
	CARDINAL_32				talk_Stop_Frame;
} Direct_Art_Rec;


/*  */
/* The Puppet_Hdl MUST be locked down at all times */
/* because it contains a graphics port (for old style graphics) ! */
/* (Hidden inside direct_Art for old style art */
/*  */

 typedef struct {
	Safe_Boolean					puppet_Initialized;
	Safe_Boolean					puppet_File_Is_Old_Self_Contained_Type;
	
/*  */
/* This is used for holding information optionally available if */
/* the new style of animation is being used. */
/* They are only used when "puppet_File_Is_Old_Self_Contained_Type" is */
/* false */
/*  */
	Handle							redirected_Information_Hdl;
	Redirected_Art_Ptr		redirected_Information_Ptr;
	Memory_Allocation		redirected_Information_Memory_In_Use;
	CARDINAL_32				redirected_Information_Max_Size;
	CARDINAL_32				redirected_Information_Total_Used;
	CARDINAL_32				current_Selection;
	
/*  */
/* These is used in old style animation that stored all art in */
/* data fork of main file, and wasted space by not compacting around */
/* the frames. This structure should be avoided if  */
/* "puppet_File_Is_Old_Self_Contained_Type" is false */
/*  */
	Direct_Art_Rec				direct_Art;
		
	CARDINAL_32				total_Colors_Used_In_Pallette;
	CARDINAL_32				total_Special_Colors_To_Honor;
	PaletteHandle				special_Pallete_To_Use;
	
	
	Str255								screen_Name;
	Str63								screen_Short_Name;
	Str63								file_Name;
 } Puppet_Rec, *Puppet_Ptr, **Puppet_Hdl;


 typedef INTEGER_16	Resource_Identifier;
 
 typedef struct {
	Safe_Boolean						valid;
	CARDINAL_32					version;
	CARDINAL_32					backward_Compatible_Version;
	CARDINAL_32					total_Colors_Used_In_Pallette;
	CARDINAL_32					total_Special_Colors_To_Honor;
	Rect										stored_Offscreen_Rect;
	CARDINAL_16					stored_Bit_Depth;
	INTEGER_16						special_Pallette_ID;
	INTEGER_16						current_Clut_ID;
	CARDINAL_32					talk_Total_Frames;
	CARDINAL_32					talk_Start_Frame;
	CARDINAL_32					talk_Stop_Frame;
	Str255									screen_Name;
	Str63									screen_Short_Name;
	Str63									file_Name;
	CARDINAL_32					file_Frame_Total;
 } Puppet_File_Header_Rec, *Puppet_File_Header_Ptr, **Puppet_File_Header_Hdl;

 typedef struct {
	Safe_Boolean						valid;
	CARDINAL_32					total_Frame_Safety;
	Resource_Identifier			file_Total_Lazy_Active_Region;
	INTEGER_16						resource_Template_Count_1;
	Rect										file_Frame_Locations[k_MAXIMUM_PICS_FRAMES];
	INTEGER_16						resource_Template_Count_2;
	Resource_Identifier			file_Frame_Regions[k_MAXIMUM_PICS_FRAMES];
	Safe_Boolean						end_Sentinel;
 } Puppet_File_Rec, *Puppet_File_Ptr, **Puppet_File_Hdl;

 
/*  */
/* Template approximation of Pup3 resource (when present) */
/*  */
 typedef struct {
	Safe_Boolean						valid;
	Safe_Boolean						puppet_File_Contains_No_Internal_Art;
	CARDINAL_32					redirection_Version;
	INTEGER_16						resource_Template_Count_1;
	/*  */
	/* Note, This puppet file actually uses a dynamically sized array */
	/* of redirected file information */
	/*  */
	Redirected_File_Rec			arbitrary_List_Of_File_Information[1];

	/*  */
	/* Note, There is an end sentinel (Safe_Boolean) at end of resource. */
	/*  */

 } Puppet_Redirected_File_Rec, *Puppet_Redirected_File_Ptr, **Puppet_Redirected_File_Hdl;

 typedef struct {
	Safe_Boolean					initialized;
	Off_Screen_Port_Rec		backdrop_Port;
	Rect									backdrop_Background_Rect;
	CARDINAL_32				special_Effect_Backdrop;
	
	Str63								backdrop_Name;
	FSSpec							backdrop_Location;
	Safe_Boolean					backdrop_Remaining_To_Load;
	CARDINAL_32				backdrop_Loading_Last_Index;
	Safe_Boolean					backdrop_Needs_Saving;
}Backdrop_Rec, *Backdrop_Ptr, **Backdrop_Handle;
	
 typedef struct {
	CARDINAL_32				total_Backdrops;
	CARDINAL_32				backdrops_Still_Loading;
	Memory_Allocation		backdrop_Memory_In_Use[k_MAX_ACTIVE_BACKDROPS];
	Backdrop_Handle			backdrop[k_MAX_ACTIVE_BACKDROPS];
} Backdrop_Sets; 


/*  */
/* 24 bytes per frame. 5 minutes at 30 per sec = 1/3 megabyte */
/* which can be viewed as VERY wasteful for space conscious */
/* people but I feel this is fair for plenty of reasons */
/*  */
 typedef struct {
	CARDINAL_32				data_Start_Offset;
	CARDINAL_32				data_Total_Size;
	CARDINAL_16				compression_Type;
	CARDINAL_8					activation;
	CARDINAL_8					horizontal_Angle;
	CARDINAL_8					vertical_Angle;
	CARDINAL_8					roll_Angle;
	CARDINAL_16				color_Depth;
	CARDINAL_32				picture_Row_Byte_Offset;
	Rect									max_Picture_Bounds;
}Spin_Cell_Index, *Spin_Cell_Index_Ptr;


/*  */
/* For file output, do not alter. */
/*  */
 typedef struct {
	Safe_Boolean					valid;
	CARDINAL_32				array_Actual_Count;
	CARDINAL_32				version;
	CARDINAL_32				max_Depth;
	Rect									max_Bounds;
	CARDINAL_32				array_Max_Count_Size;
	Spin_Cell_Index				cell_Info[];
}Spin_Pict_Info_Rec, *Spin_Pict_Info_Ptr, **Spin_Pict_Info_Handle;


/*  */
/* The following is only used when importing or when */
/* not used as conventional puppet. Otherwise the ports */
/* are ignored. */
/*  */
typedef struct {
	Off_Screen_Port_Rec		spin_Object_Port_1;
	Off_Screen_Port_Rec		spin_Object_Port_2;
	INTEGER_32					offset_Offscreen_Pixel_X;
	INTEGER_32					offset_Offscreen_Pixel_Y;
	INTEGER_32					onscreen_Offscreen_Pixel_X;
	INTEGER_32					onscreen_Offscreen_Pixel_Y;
	Previously_Drawn_Frame_Rec		frame_Needing_Erasure;
}Isolated_Draw_Ability_Rec;

 
 typedef struct {
	Safe_Boolean					initialized;

	Memory_Allocation		spin_Info_Memory_In_Use;
	Spin_Pict_Info_Handle	spin_Info;

	Safe_Boolean							isolated_Port_Drawing_Active;
	Isolated_Draw_Ability_Rec	port_Info;

	CARDINAL_32				special_Effect_Spin_Object;
	
	/*  */
	/* When this is "0" the file is assumed closed. */
	/*  */
	INTEGER_16					file_Reference;
/* ===
	Volume_Map_Rec			*file_Blocks;
=== */

	/*  */
	/* Minimum_Intensity_Representing_Idle is usually "0" else it */
	/* is the frame to use when not "talking". Other reserved frames */
	/* are only used when talking to purse lips and slam them together */
	/* between syllables. (such frames are too "pouty" for representing */
	/* closed puppet mouth conditions */
	/*  */
	
	CARDINAL_32				minimum_Intensity_Representing_Idle;
	
	CARDINAL_32				max_Intensity;
	CARDINAL_32				max_Horizontal_Angle;
	CARDINAL_32				horizontal_Angle_Periodicity;
	CARDINAL_32				max_Vertical_Angle;
	CARDINAL_32				vertical_Angle_Periodicity;
	CARDINAL_32				max_Roll_Angle;
	CARDINAL_32				roll_Angle_Periodicity;
	Rect									spin_Object_Background_Rect;

	Str63								spin_Object_Name;
	FSSpec							spin_Object_Location;
	Safe_Boolean					spin_Object_Remaining_To_Load;
	CARDINAL_32				spin_Object_Loading_Last_Index;
	Safe_Boolean					spin_Object_Needs_Saving;
	
}Spin_Object_Rec, *Spin_Object_Ptr, **Spin_Object_Handle;


 typedef struct {
	CARDINAL_32				total_Spin_Objects;
	CARDINAL_32				spin_Objects_Still_Loading;
	Memory_Allocation		spin_Object_Memory_In_Use[k_MAX_ACTIVE_SPIN_OBJECTS];
	Spin_Object_Handle		spin_Object[k_MAX_ACTIVE_SPIN_OBJECTS];
} Spin_Object_Sets; 

 typedef struct {
	Puppet_Ptr						puppet;
	Safe_Boolean					valid_Pointer;
	
	Safe_Boolean					controllers_Attached;
	CARDINAL_32				last_Display_Time;
	CARDINAL_32				last_Volume_Check;
	CARDINAL_32				current_Volume;
	Voice_Hdl						voice;
	
	/*  */
	/* "last_Image" is needed for (Spin Files) and can */
	/* also be eventually designed for regular direct puppet art */
	/*  */
	Last_Image_Rec					last_Image;
	
	/*  */
	/* "frame_Needing_Erasure" is used with (Spin Files) but */
	/* it is essentially ignored. It could be designed to be used by */
	/* animation buffers, and could record specific details separate from *
	/* the official deletion lists. In current design they are not heeded, but */
	/* to be heeded better control over an "ignore data" flag needs */
	/* management. */
	/* There are more than one of them because there is more than one */
	/* stage mix buffer. */
	/*  */
	Previously_Drawn_Frame_Rec		frame_Needing_Erasure[(k_BUFFERS_REQUIRED - k_BASE_OFFSET_FOR_MIX)];

	/*  */
	/* This represents the orientation in the scene. It cannot be soley */
	/* used for recording if scene needs redraw but combined with */
	/* "frame_Needing_Erasure" it could be contrasted for optimization. */
	/* The only thing that can be optimized are short oscillations between */
	/* two states. */
	/*  */	
	Puppet_Orientation_Rec				current_Displayed_Frame[(k_BUFFERS_REQUIRED - k_BASE_OFFSET_FOR_MIX)];
	
	/*  */
	/* This represents the measured controls in the scene. It is */
	/* similar to current_Displayed_Frame but is raw and is not */
	/* yet related to exact item position because it has not been */
	/* multiplied against the boundaries. It is more volatile than */
	/* current_Displayed_Frame, but if it does not change then */
	/* current_Displayed_Frame is not examined for motion and */
	/* drawing does not take place if not needed. */
	/* The only thing that can be optimized are short oscillations between */
	/* two states or lack of any change at all. */
	/*  */	
	Puppet_Raw_Orientation_Rec		previous_Raw_Controls[(k_BUFFERS_REQUIRED - k_BASE_OFFSET_FOR_MIX)];
	
	/*  */	
	/* This marker only is used as a one-shot flag in single routines. */
	/* It is stored in  this structure to avoid creating strange memory structures. */
	/*  */	
	Safe_Boolean					item_Needs_Redraw;
	
	CARDINAL_32				scrap_Loop_Global_Counter;

	Safe_Boolean					desired_File_Loading_Now;
	Safe_Boolean					desired_File_Already_In_Ram;
	CARDINAL_32				desired_File_Loading_Counter;
	Str63								desired_File_Name;
	Rect									desired_Start_Location;
 } Stage_Item, *Stage_Item_Ptr;
    	
 typedef struct {
	Safe_Boolean					initialized;
	Safe_Boolean					scene_Updated_To_Screen_Display_Once;
	Safe_Boolean					scene_Must_Use_Compositing_Buffers;
	Safe_Boolean					scene_Might_Contain_Movie_Ports;
	
	CARDINAL_32				total_Items_In_Scene;
	Off_Screen_Port_Rec		scene_Port;
	Rect									scene_Background_Rect;
	PaletteHandle				scene_Special_Pallete_To_Use;
	Stage_Item						item[k_MAX_STAGE_ITEMS];
	CARDINAL_32				scrap_Index_Global_Counter;
	
/*  */
/* 1994.02.22 Alterations added. */
/*  */
	CARDINAL_32				items_Remaining_To_Load;
	CARDINAL_32				item_Loading_Last_Index;

	CARDINAL_32				version;
	CARDINAL_32				backward_Compatible_Version;
	CARDINAL_32				total_Colors_Used_In_Pallette;
	CARDINAL_32				total_Special_Colors_To_Honor;
	CARDINAL_16				stored_Bit_Depth;
	Str255								screen_Name;
	Str63								screen_Short_Name;
	Str63								file_Name;
	
	Safe_Boolean					backdrop_Needs_Attaching;
	Safe_Boolean					backdrop_Selected;
	Str63								backdrop_Name;
	CARDINAL_32				backdrop_Index;
	Safe_Boolean					backdrop_Index_Valid;
 } Scene_Rec, *Scene_Ptr, **Scene_Handle;
  
 typedef struct {
	CARDINAL_32				total_Scenes;
	CARDINAL_32				scenes_Still_Loading;
	CARDINAL_32				transition_Effect;
	CARDINAL_32				transition_Effect_For_Introductions;
	CARDINAL_32				default_Start_Audio_Point;
	Memory_Allocation		scene_Memory_In_Use[k_MAX_ACTIVE_SCENES];
	Scene_Handle				scene[k_MAX_ACTIVE_SCENES];
 	CARDINAL_32				scene_Hot_Key[k_MAX_ACTIVE_SCENES];
} Theater_Company;  

 typedef struct {
 	Safe_Boolean				list_Active;
 	CARDINAL_32			list_Control_Reference;
 	SkankyListHandle		list_Handle;
 	INTEGER_16				list_Font;
 	INTEGER_16				list_Font_Size;
 	Rect								list_Rect;
 } Scrolling_List, *Scrolling_List_Ptr;

 typedef struct {
	Safe_Boolean					initialized;
	CARDINAL_32				total_Controls;
	Rect									control_Location[k_MAX_MAIN_CONTROLS];
	Str31								control_Description[k_MAX_MAIN_CONTROLS];
	INTEGER_32					control_Value[k_MAX_MAIN_CONTROLS];
	Boolean							control_Highlighted[k_MAX_MAIN_CONTROLS];
	Boolean							control_Draw_Outline_Rect[k_MAX_MAIN_CONTROLS];
	CARDINAL_32				scrolling_Total;
	Scrolling_List					scroll[k_SCROLLING_CONTROL_TOTAL];
} Main_Controller;


/*  */
/* Structure for managing dreaming stuff. */
/*  */
 typedef struct {
	Safe_Boolean						initialized;
	
	Safe_Boolean						main_Active_Scene_Specified;
	CARDINAL_32					main_Active_Scene;
	
	CARDINAL_32					active_Puppets_Total;
	CARDINAL_32					main_Window_Preferred_Height;
	CARDINAL_32					main_Window_Preferred_Width;
	INTEGER_32						main_Window_Preferred_Top_Offset;
	Stage_View_Rec					stage_View;
	Memory_Allocation			puppet_Memory_Initialized[k_MAXIMUM_PUPPETS];
	Puppet_Hdl						puppet[k_MAXIMUM_PUPPETS];
	Theater_Company			play;
	Main_Controller				main_Controls;
	Backdrop_Sets					backdrops;
	Spin_Object_Sets				spin_Objects;
	CARDINAL_16					current_Scene;
	CARDINAL_16					previously_Displayed_Scene;
	INTEGER_16						horizontal_Jiggle_Maximum;
	INTEGER_16						vertical_Jiggle_Maximum;
	Safe_Boolean						jiggle_Active;
	Safe_Boolean						async_List_Manager_Initialized;
 } Dreaming_Globals_Rec, *Dreaming_Globals_Ptr;

/* [----END----] TYPE AREA */
/*=**************************************************************************=*/


/*=**************************************************************************=*/
/* [---BEGIN---] PROTOTYPE AREA */

extern Safe_Boolean	Initialize_Dreaming_If_Needed(void);

extern Safe_Boolean	Destroy_A_Scene(CARDINAL_32 scene_To_Destroy);

extern Safe_Boolean Compact_Scene_Item_List(CARDINAL_32 scene_To_Use);

extern Safe_Boolean Compact_Puppet_List(void);

extern Safe_Boolean	Add_Puppet_Sequence_To_Scene(CARDINAL_32 scene_To_Use, CARDINAL_16 puppet, CARDINAL_16	position);

extern Safe_Boolean	Remove_Puppet_Sequence_From_Scene(CARDINAL_32 scene_To_Use, CARDINAL_16 puppet);

extern Safe_Boolean	Create_New_Empty_Scene(CARDINAL_32 *located_Position);

extern Safe_Boolean	Destroy_A_Puppet_Structure(CARDINAL_32 index);

extern Safe_Boolean	Delete_A_Loaded_Puppet(CARDINAL_32	current_Index);

extern Safe_Boolean	Create_New_Puppet_Structures(CARDINAL_32 *index_Used);

extern Safe_Boolean Repair_Region_Defect(RgnHandle total_Lazy_Active_Region, Puppet_Ptr current_Puppet);

extern Safe_Boolean	Load_PICS_Sequence(CARDINAL_32 index_Used
																		, Safe_Boolean *user_Abort);

extern Safe_Boolean	Add_Background_To_Scene(CARDINAL_32 scene_To_Use
																			, INTEGER_16 pict_Resource_Id
																			, FSSpec		*pict_File_Info
																			, CARDINAL_32	item_Address
																			, Safe_Boolean		address_Instead_Of_File
																			, INTEGER_16 current_Stage
																			, Boolean *stages_Finished);

extern Safe_Boolean	Generate_Dream_Test_Display(void);

extern Safe_Boolean	Destroy_Dream_Information(void);

extern Safe_Boolean	Enable_Minimal_Puppet_Controls(void);

extern Safe_Boolean	Enable_Audio_Controls(void);

extern Safe_Boolean Find_Empty_Position_For_Puppet(INTEGER_16 *position);

extern Safe_Boolean	Reset_Debugging_Controls(void);

extern Safe_Boolean	Load_A_Puppet_Into_Any_Scene(Boolean import_PICS_Sequence, CARDINAL_16 import_Source);

extern Strip_Off_Popular_Suffixes(Str63 *filename);

extern Safe_Boolean	Force_Scene_Item_To_Mute_Status(Stage_Item_Ptr	item_Ptr);

extern Safe_Boolean Find_Puppet_Position_In_Scene(CARDINAL_32 scene_To_Use
																, CARDINAL_16 puppet
																, INTEGER_16 *position);

extern Safe_Boolean	Add_Background_Of_Scene_To_Puppets(CARDINAL_32 scene_To_Use
																			, INTEGER_16 current_Stage
																			, Boolean *stages_Finished);

extern Safe_Boolean Determine_Item_Needed_Elsewhere(CARDINAL_32 scene_To_Ignore
				, CARDINAL_16 item_Index
				, CARDINAL_32	*actual_Puppet_Index);

extern Safe_Boolean	Interpret_Key_Command(CARDINAL_32 message, CARDINAL_32 modifiers);

extern Safe_Boolean Find_Empty_Position_In_Scene(CARDINAL_32 scene_To_Use
																, INTEGER_16 *position);

extern Safe_Boolean	Interpret_Mouse_In_Content(Primary_Status_Hdl current_Window_Globals_Hdl
												, WindowPtr event_Window
												,EventRecord	*event_Record);

extern Safe_Boolean Determine_Backdrop_In_Another_Scene(CARDINAL_32 scene_To_Ignore, CARDINAL_32	match_Index);

extern Safe_Boolean	Define_Puppet_Using_Spin_File(CARDINAL_32 index_Used
															, Safe_Boolean *user_Abort
															, Safe_Boolean	add_To_Existing_Puppet);

extern Safe_Boolean Check_If_Legal_Direct_Art_Or_Complain(Puppet_Ptr	current_Puppet);

extern void Warn_About_Unfinished_Program_And_New_Art_Unexpected(void);

extern void Perform_Startup_Actions_For_Scan_Completion(void);

/* [----END----] PROTOTYPE AREA */
/*=**************************************************************************=*/


/*=**************************************************************************=*/
/* [---BEGIN---] extern DECLARATIONS */

extern Dreaming_Globals_Rec		dream;
extern Safe_Boolean						g_Dreaming_Module_Globals_Initialized;
extern Safe_Boolean						g_Debugging_Options[4];

extern Safe_Boolean						g_Menus_Assume_No_Spin_Objects;
extern Safe_Boolean						g_Menus_Assume_No_Backgrounds;
extern Safe_Boolean						g_Menus_Assume_No_Scenes;
extern Safe_Boolean						g_Menus_Assume_No_Items;


/* [----END----] extern DECLARATIONS */
/*=**************************************************************************=*/


/*=**************************************************************************=*/
/* [---BEGIN---] LOCAL STATIC DEFINITION AREA */


/* [----END----] LOCAL STATIC DEFINITION AREA */
/*=**************************************************************************=*/


/*=**************************************************************************=*/
#endif
/*=**************************************************************************=*/
/* END OF FILE: Dreaming_Stuff.h */