/* START OF FILE: Animation_Utilities.c */


/*=**************************************************************************=*/
/* [---BEGIN---] FILE HEADER COMMENTS */
/**
 ** SCREEN VIEW = [Adobe Syntax Black-10, Tabs = 6]
 ** PRINTER VIEW = [Adobe Syntax Black-10, Tabs = 6]
 ** ==============================================================
 ** NS-Shell, The shell from hell
 ** A Merkle & Wolff Production
 ** Animation_Utilities.c - XXX.
 ** 1990 by Pico, Inc.
 ** ==============================================================
 **
 **
 ** ============
 ** Revision History
 ** ============
 ** SEE xxxx.c for revision history and version number.
 **
 ** =================
 ** OTHER INFORMATION
 ** =================
 ** 
 **
 **/
/* [----END----] FILE HEADER COMMENTS */
/*=**************************************************************************=*/


/*=**************************************************************************=*/
/* [---BEGIN---] FILE INCLUDE AREA */

#include <math.h>

#include "Automatic_Dialogs.h"
#include "Error_Utilities.h"
#include "Shell_Dialogs.h"
#include "String_Utilities.h"
#include "Memory_Utilities.h"

#include "Animation_Utilities.h"

/* [----END----] FILE INCLUDE AREA */
/*=**************************************************************************=*/


/*=**************************************************************************=*/
/* [---BEGIN---] DEFINE AREA */

#define VIDEO_OPTIONS_DIALOG_ID				133
#define VIDEO_OPTIONS_OUTLINE_ITEM			3

/* [----END----] DEFINE AREA */
/*=**************************************************************************=*/


/*=**************************************************************************=*/
/* [---BEGIN---] TYPE AREA */

typedef struct {
	Item_Variables_Header	item_Header;
	User_Item					ok_Button_Outline;
	Edit_Text_Item			video_Mode_Text;
	Edit_Text_Item			video_Page_Text;
} Video_Options_Item_Variables;

/* [----END----] TYPE AREA */
/*=**************************************************************************=*/


/*=**************************************************************************=*/
/* [---BEGIN---] PROTOTYPE AREA */

/* [----END----] PROTOTYPE AREA */
/*=**************************************************************************=*/


/*=**************************************************************************=*/
/* [---BEGIN---] GLOBAL DEFINITION AREA */	
		
/* [----END----] GLOBAL DEFINITION AREA */
/*=**************************************************************************=*/


/*=**************************************************************************=*/
/* [---BEGIN---] LOCAL STATIC DEFINITION AREA */

static char		*unit_Name = "\pAnimation_Utilities";

/* [----END----] LOCAL STATIC DEFINITION AREA */
/*=**************************************************************************=*/


/*=**************************************************************************=*/
void Perform_Change_Video_Command(void)
{
	char							*code_Name = "\pPerform_Change_Video_Command";
	
	Str255							work_String, hex_String;
	char							*work_Ptr;
	INTEGER_16					video_Mode, video_Page;
	INTEGER_32					work_Long;
	Ptr								base_Address_Ptr;
	Safe_Boolean					work_Boolean;

	Video_Options_Item_Variables		item_Variables;
/* END VAR */


	Setup_Error_Handling();

	item_Variables.ok_Button_Outline.setup_Function = (ProcPtr)Setup_Outline;
	item_Variables.ok_Button_Outline.update_Function = (ProcPtr)Update_Outline;
	item_Variables.ok_Button_Outline.action_Function = NULL;
	current_Outline_Item_Number = VIDEO_OPTIONS_OUTLINE_ITEM;

	item_Variables.video_Mode_Text.enabled = TRUE;
	Assign_String(item_Variables.video_Mode_Text.edit_String, "\p");
	item_Variables.video_Mode_Text.edit_String_Edit_Length = 4;
	item_Variables.video_Mode_Text.edit_Field_Type = string_Field;
	
	item_Variables.video_Page_Text.enabled = TRUE;
	Assign_String(item_Variables.video_Page_Text.edit_String, "\p");
	item_Variables.video_Page_Text.edit_String_Edit_Length = 4;
	item_Variables.video_Page_Text.edit_Field_Type = string_Field;

Execute_Dialog:
	if(Perform_Dialog(VIDEO_OPTIONS_DIALOG_ID, (Item_Variables_Ptr)&item_Variables) == FALSE)
		goto Normal_Function_Exit;


	if (item_Variables.video_Mode_Text.edit_String[0] == ASCII_NULL) {
		Assign_String(work_String, "\pSorry, you must enter a video mode!");
		Perform_OK_Dialog(work_String);
		goto Execute_Dialog;
	}

	if (item_Variables.video_Page_Text.edit_String[0] == ASCII_NULL) {
		Assign_String(work_String, "\pSorry, you must enter a video page!");
		Perform_OK_Dialog(work_String);
		goto Execute_Dialog;
	}

	work_Long = Convert_String_To_Long((char *)item_Variables.video_Mode_Text.edit_String);
	if ((work_Long < 1) || (work_Long > 6)) {
		Assign_String(work_String, "\pSorry, Invalid video mode.  Quickdraw only support 6 video modes at this time, reenter!");
		Perform_OK_Dialog(work_String);
		goto Execute_Dialog;
	}
	
	video_Mode = work_Long + 127;

	work_Long = Convert_String_To_Long((char *)item_Variables.video_Page_Text.edit_String);
	video_Page = work_Long;

	work_Boolean = Set_Video_Mode(video_Mode, video_Page, &base_Address_Ptr);
	if (work_Boolean != SAFE_TRUE) {
		Assign_String(ns_Shell_Globals.error_Description_String, "\pSet_Video_Mode failed.");
		goto Error_Handler;
	}

	Assign_String(work_String, "\pThe base address: ");
	Convert_Long_To_Hex((CARDINAL_32)base_Address_Ptr, hex_String);
	Concat_After_Pascal_String((char *)work_String, (char *)hex_String);
	Perform_OK_Dialog(work_String);
	
	goto Normal_Function_Exit;

Error_Handler:
	Assign_String(ns_Shell_Globals.error_Note_String, "\pUnable to complete Perform_Change_Video_Command.");
	Display_Error_Message(unit_Name, code_Name);

Normal_Function_Exit:
	return;

} /* void Perform_Change_Video_Command(void) */
/*=**************************************************************************=*/


/*=**************************************************************************=*/
Safe_Boolean Set_Video_Mode(INTEGER_16 video_Mode
									, INTEGER_16 video_Page
									, Ptr *base_Address_Ptr)
{
	char						*code_Name = "\pSet_Video_Mode";
	Str255						video_Driver_Name;
	INTEGER_16				video_Driver_Reference;
	INTEGER_32				work_Integer_32;
	VDPageInfo				new_Video_Mode, *new_Video_Mode_Ptr;
	VDSetEntryRecord		new_Color_Specifications;		/* for cscSetEntries call (new CLUT) */
	CntrlParam				video_Control;
	DCtlHandle				device_Control_Handle;
	CARDINAL_32			parameter_Array[11];
	Safe_Boolean				function_Result;
/* END VAR */


	Setup_Error_Handling();
	function_Result = SAFE_FALSE;

	Assign_String(video_Driver_Name, "\p.Display_Video_Apple_TFB");
	ns_Shell_Globals.os_Error = OpenDriver(video_Driver_Name, &video_Driver_Reference);
	if (ns_Shell_Globals.os_Error != noErr) {
		Make_String_Long_String(ns_Shell_Globals.error_Description_String
									, "\pCall to OpenDriver failed, ID = "
									, ns_Shell_Globals.os_Error
									, NO_LIMIT
									, "\p.");
		goto Error_Handler;
	}
	
	
	
	device_Control_Handle = GetDCtlEntry(video_Driver_Reference);
	if (device_Control_Handle != NULL) {
		work_Integer_32 = 0;
		work_Integer_32 = GetHandleSize((**device_Control_Handle).dCtlDriver);
	}
	
	
	
	new_Video_Mode.csMode = video_Mode;
	new_Video_Mode.csData = 0;
	new_Video_Mode.csPage = video_Page;
	new_Video_Mode.csBaseAddr = NULL;
	
	parameter_Array[0] = (CARDINAL_32)&new_Video_Mode;
	
	ns_Shell_Globals.os_Error = Control(video_Driver_Reference, cscSetMode, parameter_Array);
	if (ns_Shell_Globals.os_Error != noErr) {
		Make_String_Long_String(ns_Shell_Globals.error_Description_String
									, "\pCall to Control(cscSetMode) failed, ID = "
									, ns_Shell_Globals.os_Error
									, NO_LIMIT
									, "\p.");
		goto Error_Handler;
	}
	
	*base_Address_Ptr = new_Video_Mode.csBaseAddr;
	
	ns_Shell_Globals.os_Error = Control(video_Driver_Reference, cscGrayScreen, parameter_Array);
	if (ns_Shell_Globals.os_Error != noErr) {
		Make_String_Long_String(ns_Shell_Globals.error_Description_String
									, "\pCall to Control(cscGrayScreen) failed, ID = "
									, ns_Shell_Globals.os_Error
									, NO_LIMIT
									, "\p.");
		goto Error_Handler;
	}
	
	ns_Shell_Globals.os_Error = Control(video_Driver_Reference, cscSetGray, parameter_Array);
	if (ns_Shell_Globals.os_Error != noErr) {
		Make_String_Long_String(ns_Shell_Globals.error_Description_String
									, "\pCall to Control(cscSetGray) failed, ID = "
									, ns_Shell_Globals.os_Error
									, NO_LIMIT
									, "\p.");
		goto Error_Handler;
	}
	
/*	ns_Shell_Globals.os_Error = Control(video_Driver_Reference, cscSetEntries, parameter_Array); */
	ns_Shell_Globals.os_Error = Control(video_Driver_Reference, cscReset, parameter_Array);
	if (ns_Shell_Globals.os_Error != noErr) {
		Make_String_Long_String(ns_Shell_Globals.error_Description_String
									, "\pCall to Control(cscReset) failed, ID = "
									, ns_Shell_Globals.os_Error
									, NO_LIMIT
									, "\p.");
		goto Error_Handler;
	}
	





	function_Result = SAFE_TRUE;
	goto Normal_Function_Exit;
	
Error_Handler:
	function_Result = SAFE_FALSE;
	Assign_String(ns_Shell_Globals.error_Note_String, "\pUnable to Set_Video_Mode.");
	Display_Error_Message(unit_Name, code_Name);

Normal_Function_Exit:
	return(function_Result);
	
} /* Safe_Boolean Set_Video_Mode(INTEGER_16 video_Mode... */
/*=**************************************************************************=*/


/* END OF FILE: Animation_Utilities.c */
