/* START OF FILE: Three_D_Utilities.c */


/*=**************************************************************************=*/
/* [---BEGIN---] FILE HEADER COMMENTS */
/**
 ** SCREEN VIEW = [Adobe Syntax Black-10, Tabs = 6]
 ** PRINTER VIEW = [Adobe Syntax Black-10, Tabs = 6]
 ** ==============================================================
 ** Written by Jim Merkle
 ** [Three_D_Utilities.c] - supporting routines
 ** (C) Copyright 1992 by Jim Merkle.
 ** ==============================================================
 **
 **
 ** ============
 ** Revision History
 ** ============
 **
 **  1992.01.03		(James Merkle)
 **						Initial coding started
 **
 **
 ** =================
 ** OTHER INFORMATION
 ** =================
 ** 
 **
 **/
/* [----END----] FILE HEADER COMMENTS */
/*=**************************************************************************=*/


/*=**************************************************************************=*/
/* [---BEGIN---] FILE INCLUDE AREA */

#include <Sane.h>

#include <profile.h>
#include <stdio.h>


#include "Core_Universal.h" /* Non_Specific_Support_Routines */
#include "Core_Specific.h" /*System_Dependent_Hooks.h */
#include "SysHooks.h" /*System_Dependent_Hooks.h */
#include "Vertical_Blanking_Interrupt.h"

#include "Shell_Globals.h"
#include "Memory_Utilities.h"
#include "Core_Globals.h"
#include "Graphics_Primitives.h"


#include "Three_D_Utilities.h"

/* [----END----] FILE INCLUDE AREA */
/*=**************************************************************************=*/


/*=**************************************************************************=*/
/* [---BEGIN---] DEFINE AREA */

#define k_USE_PROFILER		0

#define k_LINE_SHAPE			0x01
#define k_LINE_TRIANGLE		0x02

#define MAXIMUM_TRIG_ELEMENTS				2048
#define LAST_TRIG_ELEMENT_INDEX				2047
#define MAXIMUM_TRIG_ELEMENTS_SIGNIFICANT_BIT_MASK		0x07FF

#define STANDARD_X_COMPRESSION			0.9
#define STANDARD_Y_COMPRESSION			0.9
#define STANDARD_Z_COMPRESSION				0.9

#define LETTER_R 									0x52

#define k_TEST_IF_RETRACE_WORKING	0
#define k_TEST_RELATIVE_SPEEDS			0

/* [----END----] DEFINE AREA */
/*=**************************************************************************=*/


/*=**************************************************************************=*/
/* [---BEGIN---] TYPE AREA */


typedef struct {
	SPEED_DIGITS_REAL			xx, yy, zz;
} Point_3D;

typedef struct {
	SPEED_DIGITS_REAL		lookup_Table[MAXIMUM_TRIG_ELEMENTS];
} Trig_Lookup_Table, *Trig_Lookup_Table_Ptr, **Trig_Lookup_Table_Hdl;

	
typedef struct {
	CARDINAL_32 			object_Vertices_Total_Elements;
	CARDINAL_32			object_Shape_Total_Elements; 
	SPEED_DIGITS_REAL		object_Vertices[800]; /*object_Vertices are the non-transformed origin referenced points */
	CARDINAL_16			object_Shape[800]; /*Refers to object_Vertices by index*/
} object_Definition, *object_Definition_Ptr, **object_Definition_Hdl;

typedef struct {
	CARDINAL_32 			entity_Total_Elements; /* FUTURE: Optimization */
	object_Definition_Ptr	entity_Objects[]; /*pointers to objects connected together as one solid mobile entity */
	CARDINAL_32			entity_Total_Objects; 
	
	CARDINAL_16			entity_Heading_Value,	entity_Pitch_Value,	entity_Bank_Value;
	CARDINAL_16			entity_Heading_Rate_Of_Change, entity_Pitch_Rate_Of_Change, entity_Bank_Rate_Of_Change;
	
	CARDINAL_16			entity_X_Location,	entity_Y_Location,	entity_Z_Location; /* center of entity is 1/2 
	CARDINAL_16			entity_X_Rate_Of_Change,	entity_Y_Rate_Of_Change,	entity_Z_Rate_Of_Change;

	CARDINAL_16			entity_Previous_Heading_Value,	entity_Previous_Pitch_Value,	entity_Previous_Bank_Value; /* FUTURE: Optimization */
	CARDINAL_16			entity_Previous_X_Location, entity_Previous_Y_Location, entity_Previous_Z_Location; /* FUTURE: Optimization */
} entity_Definition, *entity_Definition_Ptr, **entity_Definition_Hdl;


/* [----END----] TYPE AREA */
/*=**************************************************************************=*/


/*=**************************************************************************=*/
/* [---BEGIN---] PROTOTYPE AREA */

void Initialize_Sine_Table( Trig_Lookup_Table_Hdl table_To_Fill, CARDINAL_16 elements_Needed);

void Initialize_Cosine_Table( Trig_Lookup_Table_Hdl	table_To_Fill, CARDINAL_16 elements_Needed);

void Initialize_Three_D_Structures( void );

void Create_Current_Universal_Rotaion_Matrix(CARDINAL_16	heading_Value,CARDINAL_16	pitch_Value,CARDINAL_16	bank_Value);

void Old_Create_Projection_Of_Object(SPEED_DIGITS_REAL *current_Object, SPEED_DIGITS_REAL *current_Projection, CARDINAL_32	number_Of_Elements);

void Create_Projection_Of_Object(SPEED_DIGITS_REAL *current_Object, SPEED_DIGITS_REAL *current_Projection, CARDINAL_32	number_Of_Elements);

void	Build_Object_Shape( CARDINAL_16	object_ID, object_Definition_Hdl	object_Handle);

void Display_Projected_Object_Assuming_Segments(SPEED_DIGITS_REAL *current_Projection
											, object_Definition_Hdl	object_Handle);
				
void Resize_Object_Shape( SPEED_DIGITS_REAL resizing_Factor, SPEED_DIGITS_REAL *object_Shape,CARDINAL_32 *object_Points);
	
void Center_Object_Shape_To_Itself(SPEED_DIGITS_REAL *object_Shape,CARDINAL_32 *object_Points);
				

/* [----END----] PROTOTYPE AREA */
/*=**************************************************************************=*/


/*=**************************************************************************=*/
/* [---BEGIN---] GLOBAL DEFINITION AREA */	
	
	SPEED_DIGITS_REAL			projection_Scale_X, projection_Scale_Y , projection_Scale_Z; /* scaling for projection */
	SPEED_DIGITS_REAL			universal_a, universal_b, universal_c, universal_d, universal_e, universal_f, universal_g, universal_h, universal_i; /* current Matrix */

	Trig_Lookup_Table_Hdl		cosine_Table_Hdl, sine_Table_Hdl;
	Memory_Allocation			cosine_Table_allocated, sine_Table_allocated;
	SignedByte					original_Cosine_Table_State, original_Sine_Table_State; 
	SPEED_DIGITS_REAL			*cosine_Table;
	SPEED_DIGITS_REAL			*sine_Table;
	
/* [----END----] GLOBAL DEFINITION AREA */
/*=**************************************************************************=*/


/*=**************************************************************************=*/
/* [---BEGIN---] LOCAL STATIC DEFINITION AREA */

static CARDINAL_8		*unit_Name = "\pThree_D_Utilities";

/* [----END----] LOCAL STATIC DEFINITION AREA */
/*=**************************************************************************=*/


/*=**************************************************************************=*/
void Initialize_Cosine_Table( Trig_Lookup_Table_Hdl	table_To_Fill, CARDINAL_16 elements_Needed)
{
	CARDINAL_8					*code_Name = "\pInitialize_Cosine_Table";
	SPEED_DIGITS_REAL			degrees_Per_Increment;
	SPEED_DIGITS_REAL			current_Angle;
	SPEED_DIGITS_REAL			current_Value;
	CARDINAL_16				index;
	SPEED_DIGITS_REAL			temp_Value;
	extended						temp_Extended;
/* END VAR */

	
#if !defined(k_ENABLE_DIRECT_FPU_ACCESS)
#error "UNDEFINED PARAMETER!"
#endif 
#if (k_ENABLE_DIRECT_FPU_ACCESS == 1)

	x80tox96(&pi(),&degrees_Per_Increment);

#else

	degrees_Per_Increment= pi();

#endif /* k_ENABLE_DIRECT_FPU_ACCESS */

	
	degrees_Per_Increment += degrees_Per_Increment;
	
	degrees_Per_Increment = (degrees_Per_Increment / elements_Needed);
	current_Angle = 0.0;
	for (  index = 0; (index < elements_Needed); index++) {
		current_Angle = degrees_Per_Increment * index;
		
/* ======
		current_Value = cos((0.0 * pi_Value ));		
		current_Value = cos((0.10 * pi_Value ));
		current_Value = cos((0.15 * pi_Value ));
		current_Value = cos((0.20 * pi_Value ));
		current_Value = cos((0.30 * pi_Value ));
		current_Value = cos((0.45 * pi_Value ));
		current_Value = cos((0.5 * pi_Value ));	
		current_Value = cos((0.75 * pi_Value ));
		current_Value = cos((0.9 * pi_Value ));	
		current_Value = cos((1.0 * pi_Value ));	
		current_Value = cos((1.2 * pi_Value ));	
		current_Value = cos((1.5 * pi_Value ));	
		current_Value = cos((1.8 * pi_Value ));	
		current_Value = cos((2.0 * pi_Value ));	
====== */


		
#if !defined(k_ENABLE_DIRECT_FPU_ACCESS)
#error "UNDEFINED PARAMETER!"
#endif 
#if (k_ENABLE_DIRECT_FPU_ACCESS == 1)

		x96tox80(&current_Angle,&temp_Extended);
		temp_Extended = cos(temp_Extended);
		x80tox96(&temp_Extended, &current_Value);

#else

		current_Value = cos(current_Angle);

#endif /* k_ENABLE_DIRECT_FPU_ACCESS */

		
		(**table_To_Fill).lookup_Table[index] = current_Value;
	}
	
} /* Initialize_Cosine_Table */
/*=**************************************************************************=*/


/*=**************************************************************************=*/
void Initialize_Sine_Table( Trig_Lookup_Table_Hdl table_To_Fill, CARDINAL_16 elements_Needed)
{
	CARDINAL_8				*code_Name = "\pInitialize_Sine_Table";
	SPEED_DIGITS_REAL		degrees_Per_Increment, current_Angle;
	SPEED_DIGITS_REAL		current_Value, pi_Value;
	extended					temp_Extended;
	CARDINAL_16			index;
/* END VAR */

	
#if !defined(k_ENABLE_DIRECT_FPU_ACCESS)
#error "UNDEFINED PARAMETER!"
#endif 
#if (k_ENABLE_DIRECT_FPU_ACCESS == 1)

	x80tox96(&pi(),&pi_Value);

#else

	pi_Value = pi();

#endif /* k_ENABLE_DIRECT_FPU_ACCESS */
			
	degrees_Per_Increment = ((2.0 * pi_Value ) / elements_Needed);
	current_Angle = 0.0;
	for (  index = 0; index < elements_Needed; index++) {
		current_Angle = degrees_Per_Increment * index;
		
#if !defined(k_ENABLE_DIRECT_FPU_ACCESS)
#error "UNDEFINED PARAMETER!"
#endif 
#if (k_ENABLE_DIRECT_FPU_ACCESS == 1)

		x96tox80(&current_Angle,&temp_Extended);
		temp_Extended = sin(temp_Extended);
		x80tox96(&temp_Extended, &current_Value);

#else

		current_Value = sin(current_Angle);

#endif /* k_ENABLE_DIRECT_FPU_ACCESS */
		
		
		
		(**table_To_Fill).lookup_Table[index] = current_Value;
	}
	
} /* Initialize_Cosine_Table */
/*=**************************************************************************=*/


/*=**************************************************************************=*/
void Initialize_Three_D_Structures( void )
{
	CARDINAL_8					*code_Name = "\pInitialize_Three_D_Structures";
	Safe_Boolean					work_Boolean;
	CARDINAL_32 				requested_Size;

/* END VAR */
	Setup_Error_Handling_Hook();

	projection_Scale_X = STANDARD_X_COMPRESSION; /* should be close to our "1" value */
	projection_Scale_Y = STANDARD_Y_COMPRESSION;
	projection_Scale_Z = STANDARD_Z_COMPRESSION;
	
	cosine_Table_allocated = MEMORY_NOT_INITIALIZED;
	requested_Size = sizeof(Trig_Lookup_Table);
	work_Boolean = Create_Handle_Hook(&requested_Size
								, 10240L
								, ALLOW_PURGING_HOOK
								, DISALLOW_PARTIAL_AMOUNT_HOOK
								, (Handle *)&cosine_Table_Hdl
								, &cosine_Table_allocated);
								
	sine_Table_allocated = MEMORY_NOT_INITIALIZED;
	requested_Size = sizeof(Trig_Lookup_Table);
	work_Boolean = Create_Handle_Hook(&requested_Size
								, 10240L
								, ALLOW_PURGING_HOOK
								, DISALLOW_PARTIAL_AMOUNT_HOOK
								, (Handle *)&sine_Table_Hdl
								, &sine_Table_allocated);


	if (work_Boolean != SAFE_TRUE) {
		Assign_String_Hook(ns_Shell_Globals.error_Description_String, "\pCreate_Handle_Hook failed. (Low Memory?)");
		goto Error_Handler;
	}
	
	if (Begin_Use_Of_Handle_Hook((Handle)sine_Table_Hdl, MOVE_HIGH_REQUIRED_HOOK, &original_Sine_Table_State)  != SAFE_TRUE) {
		Assign_String_Hook(ns_Shell_Globals.error_Description_String, "\pBegin_Use_Of_Handle_Hook failed. ");
		goto Error_Handler;
	}
	sine_Table = (SPEED_DIGITS_REAL	*)*sine_Table_Hdl;
	
	if (Begin_Use_Of_Handle_Hook((Handle)cosine_Table_Hdl, MOVE_HIGH_REQUIRED_HOOK, &original_Cosine_Table_State)  != SAFE_TRUE) {
		Assign_String_Hook(ns_Shell_Globals.error_Description_String, "\pBegin_Use_Of_Handle_Hook failed. ");
		goto Error_Handler;
	}
	cosine_Table = (SPEED_DIGITS_REAL	*)*cosine_Table_Hdl;
	
	Initialize_Cosine_Table(cosine_Table_Hdl, MAXIMUM_TRIG_ELEMENTS);
	Initialize_Sine_Table(sine_Table_Hdl, MAXIMUM_TRIG_ELEMENTS);


	goto Normal_Function_Exit;

Error_Handler:
	Assign_String_Hook(ns_Shell_Globals.error_Note_String, "\pUnable to complete Initialize_Three_D_Structures.");
	Display_Error_Message_Hook(unit_Name, code_Name);

Normal_Function_Exit:
	return;
} /* Initialize_Three_D_Structures /
/*=**************************************************************************=*/


/*=**************************************************************************=*/
void Create_Current_Universal_Rotaion_Matrix(CARDINAL_16	heading_Value,CARDINAL_16	pitch_Value,CARDINAL_16	bank_Value)
{

	if (heading_Value > LAST_TRIG_ELEMENT_INDEX ) {
		heading_Value = (heading_Value & MAXIMUM_TRIG_ELEMENTS_SIGNIFICANT_BIT_MASK);
	}
	else if (heading_Value < 0 ) {
		heading_Value = (heading_Value & MAXIMUM_TRIG_ELEMENTS_SIGNIFICANT_BIT_MASK);
	}
	
	if (pitch_Value > LAST_TRIG_ELEMENT_INDEX ) {
		pitch_Value = (pitch_Value & MAXIMUM_TRIG_ELEMENTS_SIGNIFICANT_BIT_MASK);
	}
	else if (heading_Value < 0 ) {
		pitch_Value = (pitch_Value & MAXIMUM_TRIG_ELEMENTS_SIGNIFICANT_BIT_MASK);
	}
	
	if (bank_Value > LAST_TRIG_ELEMENT_INDEX ) {
		bank_Value = (bank_Value & MAXIMUM_TRIG_ELEMENTS_SIGNIFICANT_BIT_MASK);
	}
	else if (heading_Value < 0 ) {
		bank_Value = (bank_Value & MAXIMUM_TRIG_ELEMENTS_SIGNIFICANT_BIT_MASK);
	}
		
	/*  */
	/*  No Axis rotation matrix example (identity matrix)*/
	/*  */
/* ============	

	universal_a = 1.0;
	universal_b = 0.0;
	universal_c = 0.0;
	universal_d = 0.0;
	universal_e = 1.0;
	universal_f = 0.0;
	universal_g = 0.0;
	universal_h = 0.0;
	universal_i = 1.0;
============== */


	/*=--------------------------------------------------------------------=*/
	/*  Y Axis rotation matrix example */
	/*=--------------------------------------------------------------------=*/
/* ============	
	universal_a = cosine_Table[heading_Value];
	universal_b = 0.0;
	universal_c = sine_Table[heading_Value];
	universal_d =  0.0;
	universal_e = 1.0; 
	universal_f = 0.0;
	universal_g = - sine_Table[heading_Value];
	universal_h = 0.0;
	universal_i = cosine_Table[heading_Value];
============== */

	/*=--------------------------------------------------------------------=*/
	/* X Axis rotation matrix example */
	/*=--------------------------------------------------------------------=*/
/* ============	
	universal_a = 1.0;
	universal_b = 0.0;
	universal_c = 0.0;
	universal_d =  0.0;
	universal_e = cosine_Table[pitch_Value];
	universal_f = 0.0 - sine_Table[pitch_Value];
	universal_g = 0.0;
	universal_h = sine_Table[pitch_Value];
	universal_i = cosine_Table[pitch_Value];
============== */


	/*=--------------------------------------------------------------------=*/
	/*  Z Axis rotation matrix example*/
	/*=--------------------------------------------------------------------=*/
/*=====
	universal_a = cosine_Table[bank_Value];
	universal_b = 0.0 - sine_Table[bank_Value];
	universal_c = 0.0;
	universal_d = sine_Table[bank_Value];
	universal_e = cosine_Table[bank_Value];
	universal_f = 0.0;
	universal_g = 0.0;
	universal_h = 0.0;
	universal_i = 1.0;
====== */

	/*=--------------------------------------------------------------------=*/






	/*=--------------------------------------------------------------------=*/
	/*  All Axis rotation matrix example*/
	/*=--------------------------------------------------------------------=*/

	universal_a = (cosine_Table[heading_Value] * cosine_Table[bank_Value]) + (sine_Table[heading_Value] * sine_Table[pitch_Value] * sine_Table[bank_Value]);
	universal_b = (sine_Table[heading_Value] * sine_Table[pitch_Value] * cosine_Table[bank_Value]) - (cosine_Table[heading_Value] * sine_Table[bank_Value]);
	universal_c = sine_Table[heading_Value] * cosine_Table[pitch_Value];
	universal_d = sine_Table[bank_Value] * cosine_Table[pitch_Value];
	universal_e = cosine_Table[bank_Value] * cosine_Table[pitch_Value];
	universal_f = 0.0 - sine_Table[pitch_Value];
	universal_g = ((cosine_Table[heading_Value] * sine_Table[pitch_Value] * sine_Table[bank_Value]) - (sine_Table[heading_Value]* cosine_Table[bank_Value]) );
	universal_h = (sine_Table[bank_Value] * sine_Table[heading_Value]) + (cosine_Table[heading_Value] * sine_Table[pitch_Value] * cosine_Table[bank_Value]);
	universal_i = cosine_Table[heading_Value] * cosine_Table[pitch_Value];




	/*=--------------------------------------------------------------------=*/
	/*  Apply Scaling after rotation, before translation,  matrix*/
	/*=--------------------------------------------------------------------=*/
	universal_a = universal_a * projection_Scale_X;
	universal_b = universal_b * projection_Scale_Y;
	universal_c = universal_c * projection_Scale_Z;

	universal_d = universal_d * projection_Scale_X;
	universal_e = universal_e * projection_Scale_Y;
	universal_f = universal_f * projection_Scale_Z;

	universal_g = universal_g * projection_Scale_X;
	universal_h = universal_h * projection_Scale_Y;
	universal_i = universal_i * projection_Scale_Z;
	
	
	/*=--------------------------------------------------------------------=*/
	/*  Optimization trick for latter Matrice Multiplication:*/
	/* Add unity to universal_d */
	/*=--------------------------------------------------------------------=*/
/* ===
	universal_d = (universal_d +1.0);
=== */

} /* Create_Current_Universal_Rotaion_Matrix */	
/*=**************************************************************************=*/


/*=**************************************************************************=*/
void Old_Create_Projection_Of_Object(SPEED_DIGITS_REAL *current_Object
										, SPEED_DIGITS_REAL *current_Projection
										, CARDINAL_32	number_Of_Elements)
{
register INTEGER_16	element_Offset;
/* END VAR */

	number_Of_Elements = number_Of_Elements * 3; /* Three times as many values per point */
 	for ( element_Offset = 0; (element_Offset < number_Of_Elements) ; ) {
 	
		/*=--------------------------------------------------------------------=*/
		/*  Z point,  matrix multiply*/
		/*=--------------------------------------------------------------------=*/
 		current_Projection[element_Offset] = (current_Object[element_Offset] * universal_c) + (current_Object[element_Offset+1] * universal_f) 
 		 											+ (current_Object[element_Offset+2] * universal_i);
 	
		/*=--------------------------------------------------------------------=*/
		/*  X point,  matrix multiply*/
		/*=--------------------------------------------------------------------=*/
 		current_Projection[element_Offset+1] = ((current_Object[element_Offset] * universal_a) 
 		 											+ (current_Object[(element_Offset +2 )] * universal_g));
 		current_Projection[element_Offset+1] += (current_Object[element_Offset +1] * universal_d);


		/*=--------------------------------------------------------------------=*/
		/*  Y point,  matrix multiply*/
		/*=--------------------------------------------------------------------=*/
 		current_Projection[element_Offset+2] = ((current_Object[element_Offset] * universal_b) + 
 		 											+ (current_Object[(element_Offset +2 )] * universal_h));
 		current_Projection[element_Offset+2] += (current_Object[element_Offset+1] * universal_e);
 		
  		element_Offset += 3;
 	}
 
} /* Old_Create_Projection_Of_Object */
/*=**************************************************************************=*/


/*=**************************************************************************=*/
void Create_Projection_Of_Object(SPEED_DIGITS_REAL *current_Object
											, SPEED_DIGITS_REAL *current_Projection
											, CARDINAL_32	number_Of_Elements)
{
register INTEGER_16	element_Offset;
/* END VAR */

	number_Of_Elements = number_Of_Elements * 3; /* Three times as many values per point */
 	for ( element_Offset = 0; (element_Offset < number_Of_Elements) ; ) {
 	
		/*=--------------------------------------------------------------------=*/
		/*  Z point,  matrix multiply*/
		/*=--------------------------------------------------------------------=*/
 		current_Projection[element_Offset] = (current_Object[element_Offset] * universal_c) 
 													+ (current_Object[element_Offset+1] * universal_f) 
 		 											+ (current_Object[element_Offset+2] * universal_i);
 	
		/*=--------------------------------------------------------------------=*/
		/*  X point,  matrix multiply*/
		/*=--------------------------------------------------------------------=*/
 		current_Projection[element_Offset+1] = ((current_Object[element_Offset] * universal_a) 
 		 											+ (current_Object[(element_Offset +2 )] * universal_g));
 		current_Projection[element_Offset+1] += (current_Object[element_Offset +1] * universal_d);


		/*=--------------------------------------------------------------------=*/
		/*  Y point,  matrix multiply*/
		/*=--------------------------------------------------------------------=*/
 		current_Projection[element_Offset+2] = ((current_Object[element_Offset] * universal_b) + 
 		 											+ (current_Object[(element_Offset +2 )] * universal_h));
 		current_Projection[element_Offset+2] += (current_Object[element_Offset+1] * universal_e);
 		
  		element_Offset += 3;
 	}
 
} /* Create_Projection_Of_Object */
/*=**************************************************************************=*/


/*=**************************************************************************=*/
void Resize_Object_Shape( SPEED_DIGITS_REAL resizing_Factor, SPEED_DIGITS_REAL *object_Vertices,CARDINAL_32 *object_Points)
{
	INTEGER_32				loop, last_point;
/* END VAR */

	/*=--------------------------------------------------------------------=*/
	/* Round off for safety */
	/*=--------------------------------------------------------------------=*/
	last_point = *object_Points /3;
	last_point = last_point * 3;
	
	for (loop = 0; (loop < last_point) ; loop++) {
		object_Vertices[loop] = object_Vertices[loop]  * resizing_Factor;
	}

}/* void Resize_Object_Shape( SPEED_DIGITS_REAL resizing_Factor, SPEED_DIGITS_REAL *object_Vertices,CARDINAL_32 *object_Points) */
/*=**************************************************************************=*/


/*=**************************************************************************=*/
void Center_Object_Shape_To_Itself(SPEED_DIGITS_REAL *object_Vertices,CARDINAL_32 *object_Points)
{
	INTEGER_32				loop, last_point;
	SPEED_DIGITS_REAL		horizontal_Offset, vertical_Offset, depth_Offset;
	SPEED_DIGITS_REAL		horizontal_Max, vertical_Max, depth_Max;
	SPEED_DIGITS_REAL		horizontal_Min, vertical_Min, depth_Min;
	SPEED_DIGITS_REAL		horizontal_Adjustment, vertical_Adjustment, depth_Adjustment;	
	SPEED_DIGITS_REAL		imaging_System_Maximum, imaging_System_Minimum;
/* END VAR */

	/*=--------------------------------------------------------------------=*/
	/* Round off for safety */
	/*=--------------------------------------------------------------------=*/
	last_point = *object_Points /3;
	last_point = last_point * 3;
		
	/*=--------------------------------------------------------------------=*/
	/* The limits are not truly the maximums possible for the data type, */
	/* but will serve, for this application. (reversed to increment them)*/
	/*=--------------------------------------------------------------------=*/
	imaging_System_Maximum = INTEGER_32_MIN;
	imaging_System_Minimum = INTEGER_32_MAX;
	
	horizontal_Max = imaging_System_Maximum;
	vertical_Max = imaging_System_Maximum;
	depth_Max = imaging_System_Maximum;
	horizontal_Min = imaging_System_Minimum;
	vertical_Min = imaging_System_Minimum;
	depth_Min = imaging_System_Minimum;
	
	
	/*=--------------------------------------------------------------------=*/
	/* Find limits being used */
	/*=--------------------------------------------------------------------=*/
	for (loop = 0; (loop < last_point) ; loop += 3) {
	
		if (object_Vertices[loop] > depth_Max) {
			depth_Max = object_Vertices[loop];
		}
		else if (object_Vertices[loop] < depth_Min) {
			depth_Min = object_Vertices[loop];
		}
			
		if (object_Vertices[loop+1] > horizontal_Max) {
			horizontal_Max = object_Vertices[loop+1];
		}
		else if (object_Vertices[loop+1] < horizontal_Min) {
			horizontal_Min = object_Vertices[loop+1];
		}
			
		if (object_Vertices[loop+2] > vertical_Max) {
			vertical_Max = object_Vertices[loop+2];
		}
		else if (object_Vertices[loop+2] < vertical_Min) {
			vertical_Min = object_Vertices[loop+2];
		}
	}
	
	depth_Adjustment = - (depth_Min + ((depth_Max -  depth_Min) / 2));
	horizontal_Adjustment = - (horizontal_Min + ((horizontal_Max -  horizontal_Min) / 2));
	vertical_Adjustment = - (vertical_Min + ((vertical_Max -  vertical_Min) / 2));

	for (loop = 0; (loop < last_point) ; loop += 3) {
		object_Vertices[loop] = object_Vertices[loop]  + depth_Adjustment;
		object_Vertices[loop+1] = object_Vertices[loop+1]  + horizontal_Adjustment;
		object_Vertices[loop+2] = object_Vertices[loop+2]  + vertical_Adjustment;
	}

}/* void Resize_Object_Shape( SPEED_DIGITS_REAL resizing_Factor, SPEED_DIGITS_REAL *object_Vertices,CARDINAL_32 *object_Points) */
/*=**************************************************************************=*/


/*=**************************************************************************=*/
void Display_Projected_Object_Assuming_Segments(SPEED_DIGITS_REAL *current_Projection
											, object_Definition_Hdl	object_Handle)
{
	GrafPtr					current_Port;
	INTEGER_32	 			loop;
	INTEGER_32	 			total_Points;
	INTEGER_32	 			next_X, next_Y;
	INTEGER_32	 			horizontal_Offset, vertical_Offset;
	CARDINAL_32 			current_Index;
	Rect						clipping_Rect;
	CARDINAL_16			*object_Shape;
/* END VAR */



	total_Points = ((**object_Handle).object_Shape_Total_Elements /3);
	object_Shape = ((**object_Handle).object_Shape);


	GetPort( &current_Port);
	
	clipping_Rect.top = (**((WindowPeek)current_Port)->contRgn).rgnBBox.top;
	clipping_Rect.bottom = (**((WindowPeek)current_Port)->contRgn).rgnBBox.bottom;
	
	/*=--------------------------------------------------------------------=*/
	/* Actual limit is really one pixel inward! Else bad things will happen */
	/*=--------------------------------------------------------------------=*/
	clipping_Rect.bottom = clipping_Rect.bottom - 1;
	
	clipping_Rect.left = (**((WindowPeek)current_Port)->contRgn).rgnBBox.left;
	clipping_Rect.right = (**((WindowPeek)current_Port)->contRgn).rgnBBox.right;
	
	/*=--------------------------------------------------------------------=*/
	/* Actual limit is really one pixel inward! Else bad things will happen */
	/*=--------------------------------------------------------------------=*/
	clipping_Rect.right = clipping_Rect.right - 1;
	

	/*=--------------------------------------------------------------------=*/
	/* Convert to local coordinates (0-based) */
	/*=--------------------------------------------------------------------=*/
	clipping_Rect.bottom = clipping_Rect.bottom - clipping_Rect.top;
	clipping_Rect.right = clipping_Rect.right - clipping_Rect.left;
	clipping_Rect.left =0;
	clipping_Rect.top = 0;
	

	/*=--------------------------------------------------------------------=*/
	/* Set object origin near window center */
	/*=--------------------------------------------------------------------=*/
	vertical_Offset = clipping_Rect.bottom - clipping_Rect.top;
	vertical_Offset = vertical_Offset / 2;
	horizontal_Offset = clipping_Rect.right - clipping_Rect.left;
	horizontal_Offset = horizontal_Offset / 2;


	current_Index = 1;
	
	for (loop = 0; (loop < total_Points) ; loop++) {
	
	
		if (object_Shape[current_Index -1] != k_LINE_SHAPE) {
			/* Debug Halt */
			total_Points = ((**object_Handle).object_Shape_Total_Elements /3);
		}
	
		next_X = current_Projection[object_Shape[current_Index]+1] + horizontal_Offset;
		next_Y = current_Projection[object_Shape[current_Index]+2] + vertical_Offset;
		current_Index = current_Index + 1;
	
/* ===
		next_X = current_Projection[current_Index] + horizontal_Offset;
		next_Y = current_Projection[current_Index + 1] + vertical_Offset;
		current_Index = current_Index + 3;
=== */	
		
		
		MoveTo(next_X, next_Y);
		
		next_X = current_Projection[object_Shape[current_Index]+1] + horizontal_Offset;
		next_Y = current_Projection[object_Shape[current_Index]+2] + vertical_Offset;
		current_Index = current_Index + 2;
		
		
/*===
		next_X = current_Projection[current_Index] + horizontal_Offset;
		next_Y = current_Projection[current_Index + 1]+ vertical_Offset;
		current_Index = current_Index + 3;
=== */

		Draw_Clipped_Line_To(next_X, next_Y, &clipping_Rect);


//		Draw_Line_To(next_X, next_Y);


	} /* for (loop = 0; (loop < total_Points) ; loop++) */

}
/*=**************************************************************************=*/


/*=**************************************************************************=*/
void Demonstrate_Three_D(void)
{
	object_Definition_Hdl	object_Handle;
	SignedByte 				original_State;
	SPEED_DIGITS_REAL 	*object_Vertices;

	
	
	CARDINAL_32 			*total_Vertices;
	
	CARDINAL_16			current_Heading;
	CARDINAL_16			current_Pitch;
	CARDINAL_16			current_Bank;
	SPEED_DIGITS_REAL		current_Heading_Degrees;
	SPEED_DIGITS_REAL		current_Pitch_Degrees;
	SPEED_DIGITS_REAL		current_Bank_Degrees;
	CARDINAL_8				*code_Name;
	INTEGER_16				loop, outer_Loop, step_Factor;
	Rect						active_Port_Sample_Global_Rect;
	CARDINAL_8	 			video_Device_Slot_To_Synchronize_With;
	
	CARDINAL_32			total_VBL_Accuracy_Time;

	SPEED_DIGITS_REAL 	current_Projection_1[800];
	SPEED_DIGITS_REAL 	current_Projection_2[800];
	
	SPEED_DIGITS_REAL		padding[100];
	
	
	CARDINAL_32	total_VBL_Time;
	CARDINAL_32	total_Matrix_Time;
	CARDINAL_32	total_Color_Time;
	CARDINAL_32	total_Projection_Time;
	CARDINAL_32	total_Display_Time;
/* END VAR */


#if (k_USE_PROFILER == 1)
	InitProfile(700, 20);
	start_VIA_timer();
	_profile = FALSE;
	 _trace = FALSE;
	freopen("profiler report.note", "w", stdout);
	_profile = TRUE;
#endif

	Setup_Error_Handling_Hook();
	
	code_Name = "\pDemonstrate_Three_D";
	step_Factor = 10;

	Initialize_Three_D_Structures();
	
	/*=--------------------------------------------------------------------=*/
	/* Setup VBL Slot monitor for this port, actual rect unimportant in this case */
	/*=--------------------------------------------------------------------=*/
	active_Port_Sample_Global_Rect.top = 0;
	active_Port_Sample_Global_Rect.left = 0;
	active_Port_Sample_Global_Rect.right = 1;
	active_Port_Sample_Global_Rect.bottom = 1;
	if (Install_VBL_Synchronizer_For_Slot_If_Needed(active_Port_Sample_Global_Rect, &video_Device_Slot_To_Synchronize_With) != SAFE_TRUE) {
		goto Error_Handler;
	}
	
	/*=--------------------------------------------------------------------=*/
	/* Test accuracy of Video monitoring (To see if interrupts are being */
	/* serviced properly */
	/*=--------------------------------------------------------------------=*/
#if (k_TEST_IF_RETRACE_WORKING == 1)
	total_VBL_Accuracy_Time = TickCount();
	for (loop = 0; (loop < 60) ; loop++) {
		Wait_For_Next_Video_Monitor_VBL(video_Device_Slot_To_Synchronize_With);
	}
	total_VBL_Accuracy_Time =  TickCount() - total_VBL_Accuracy_Time;
	if ((total_VBL_Accuracy_Time > (60 + 30))
		|| (total_VBL_Accuracy_Time < (60 - 30))) {
		SysBeep(10);
		goto Error_Handler;
	}
#endif
	
	
	object_Handle = (object_Definition_Hdl)NewHandle(6000);
	if (MemError() != noErr) {
		goto Error_Handler;
	}
		
	if (Begin_Use_Of_Handle((Handle)object_Handle
								, MOVE_HIGH_NOT_REQUIRED
								, &original_State)	!= SAFE_TRUE) {
		goto Error_Handler;
	}
		
		
	Build_Object_Shape( LETTER_R, object_Handle);
	
	object_Vertices = ((**object_Handle).object_Vertices);
	total_Vertices	= &((**object_Handle).object_Vertices_Total_Elements);
	
	
	Resize_Object_Shape( 20.0, object_Vertices, total_Vertices);
	
	Center_Object_Shape_To_Itself( object_Vertices, total_Vertices);
	
		
/*=====	
	Display_Projected_Object_Assuming_Segments(object_Vertices, object_Handle);
====== */
	
	projection_Scale_X = 1.0; /* should be close to our "1" value */
	projection_Scale_Y = 1.0;
	projection_Scale_Z = 1.0;
	
	
	
	current_Heading_Degrees = 70.0;
	current_Pitch_Degrees = 20.0;
	current_Bank_Degrees = 20.0;
	
	current_Heading_Degrees = ((current_Heading_Degrees /360.0) * MAXIMUM_TRIG_ELEMENTS);
	current_Pitch_Degrees = ((current_Pitch_Degrees /360.0) * MAXIMUM_TRIG_ELEMENTS);
	current_Bank_Degrees = ((current_Bank_Degrees /360.0) * MAXIMUM_TRIG_ELEMENTS);
	
	current_Heading = current_Heading_Degrees;
	current_Pitch = current_Pitch_Degrees;
	current_Bank = current_Bank_Degrees;
	
	Create_Current_Universal_Rotaion_Matrix(current_Heading, current_Pitch, current_Bank);
	
	
/* ===
	Create_Projection_Of_Object(object_Vertices, current_Projection_1, *total_Vertices );
	Create_Projection_Of_Object(object_Vertices, current_Projection_2, *total_Vertices );
==== */


#if (k_TEST_RELATIVE_SPEEDS == 1)
	current_Bank = (20 * step_Factor);
	current_Pitch = (20 * step_Factor);
	current_Heading = (20 * step_Factor);
	
	total_VBL_Time = TickCount();
	for (loop = 0; (loop < 300) ; loop++) {
		Wait_For_Next_Video_Monitor_VBL(video_Device_Slot_To_Synchronize_With);
	}
	total_VBL_Time =  TickCount() - total_VBL_Time;
	
	total_Matrix_Time = TickCount();
	for (loop = 0; (loop < 300) ; loop++) {
		Create_Current_Universal_Rotaion_Matrix(current_Heading, current_Pitch, current_Bank); 
	}
	total_Matrix_Time =  TickCount() - total_Matrix_Time;
	
	total_Color_Time = TickCount();
	for (loop = 0; (loop < 300) ; loop++) {
		Set_Current_Color_Primitive(&g_Shell.library.color_Black);
	}
	total_Color_Time =  TickCount() - total_Color_Time;
	

	total_Projection_Time = TickCount();
	for (loop = 0; (loop < 300) ; loop++) {
		Create_Projection_Of_Object(object_Vertices, current_Projection_1, *total_Vertices );
	}
	total_Projection_Time =  TickCount() - total_Projection_Time;
	
	
	total_Display_Time = TickCount();
	for (loop = 0; (loop < 300) ; loop++) {
		Display_Projected_Object_Assuming_Segments(current_Projection_1, object_Handle);
	}
	total_Display_Time =  TickCount() - total_Display_Time;
	
#endif
	
	
	
	
	/*=--------------------------------------------------------------------=*/
	/* Rotate through complete "Bank" "Heading" and "Pitch" axis */
	/* Draw max of one frame per update (if even possible). */
	/* Do erase during VBL of slot queue */
	/*=--------------------------------------------------------------------=*/
	for (outer_Loop = 0; (outer_Loop < 1) ; outer_Loop++) {
		for (loop = 0; (loop < (2 * (MAXIMUM_TRIG_ELEMENTS / step_Factor))) ; loop++) {
			current_Bank = (loop * step_Factor);
			current_Pitch = (loop * step_Factor);
			current_Heading = (loop * step_Factor);
			
			Create_Current_Universal_Rotaion_Matrix(current_Heading, current_Pitch, current_Bank); 
			
			if ((loop & 0x00000001) == 0) {
				Create_Projection_Of_Object(object_Vertices, current_Projection_1, *total_Vertices ); 
				Wait_For_Next_Video_Monitor_VBL(video_Device_Slot_To_Synchronize_With);
				
				if (loop > 0) {
					Set_Current_Color_Primitive(&g_Shell.library.color_White);
					Display_Projected_Object_Assuming_Segments(current_Projection_2, object_Handle);
				}
	
				Set_Current_Color_Primitive(&g_Shell.library.color_Black);
				Display_Projected_Object_Assuming_Segments(current_Projection_1, object_Handle);
			}
			else {
				Create_Projection_Of_Object(object_Vertices, current_Projection_2, *total_Vertices );  
			
				Wait_For_Next_Video_Monitor_VBL(video_Device_Slot_To_Synchronize_With);

				Set_Current_Color_Primitive(&g_Shell.library.color_White);
				Display_Projected_Object_Assuming_Segments(current_Projection_1, object_Handle);
	
				Set_Current_Color_Primitive(&g_Shell.library.color_Black);
				Display_Projected_Object_Assuming_Segments(current_Projection_2, object_Handle);
			}
		}
	}
	
	current_Bank_Degrees = 20.0;
	current_Bank_Degrees = ((current_Bank_Degrees /360.0) * MAXIMUM_TRIG_ELEMENTS);
	current_Bank = current_Bank_Degrees;

	current_Pitch_Degrees = 20.0;
	current_Pitch_Degrees = ((current_Pitch_Degrees /360.0) * MAXIMUM_TRIG_ELEMENTS);
	current_Pitch = current_Pitch_Degrees;

	current_Heading_Degrees = 70.0;
	current_Heading_Degrees = ((current_Heading_Degrees /360.0) * MAXIMUM_TRIG_ELEMENTS);
	current_Heading = current_Heading_Degrees;
	

	
	if (End_Use_Of_Handle_Hook((Handle)cosine_Table_Hdl,original_Cosine_Table_State)  != SAFE_TRUE) {
		Assign_String_Hook(ns_Shell_Globals.error_Description_String, "\pEnd_Use_Of_Handle_Hook failed. ");
		goto Error_Handler;
	}
	
	if (End_Use_Of_Handle_Hook((Handle)sine_Table_Hdl,original_Sine_Table_State)  != SAFE_TRUE) {
		Assign_String_Hook(ns_Shell_Globals.error_Description_String, "\pEnd_Use_Of_Handle_Hook failed. ");
		goto Error_Handler;
	}
	
	if (Destroy_Handle_Hook((Handle *)&cosine_Table_Hdl,&cosine_Table_allocated)  != SAFE_TRUE) {
		Assign_String_Hook(ns_Shell_Globals.error_Description_String, "\pDestroy_Handle failed. ");
		goto Error_Handler;
	}
	
	if (Destroy_Handle_Hook((Handle *)&sine_Table_Hdl,&sine_Table_allocated)  != SAFE_TRUE) {
		Assign_String_Hook(ns_Shell_Globals.error_Description_String, "\pDestroy_Handle failed. ");
		goto Error_Handler;
	}
	
	/*=--------------------------------------------------------------------=*/
	/* Remove VBL Slot monitor for this port, actual rect unimportant in this case */
	/*=--------------------------------------------------------------------=*/
	if (Remove_VBL_Synchronizer_For_Slot_If_Needed(active_Port_Sample_Global_Rect) != SAFE_TRUE) {
		goto Error_Handler;
	}
	
	
#if (k_USE_PROFILER == 1)
	_profile = FALSE;
	 _trace = FALSE;
	DumpProfile();
#endif

	goto Normal_Function_Exit;

Error_Handler:
	Assign_String_Hook(ns_Shell_Globals.error_Note_String, "\pUnable to complete Initialize_Three_D_Structures.");
	Display_Error_Message_Hook(unit_Name, code_Name);

Normal_Function_Exit:
	return;
}/* Demonstrate_Three_D */
/*=**************************************************************************=*/


/*=**************************************************************************=*/
void	Build_Object_Shape( CARDINAL_16	object_ID, object_Definition_Hdl	object_Handle)
{
	CARDINAL_8				*code_Name = "\pBuild_Object_Shape";
	INTEGER_32				loop, current_Point, current_Item;
	
	SPEED_DIGITS_REAL 	*object_Vertices;
	CARDINAL_16			*object_Shape;
	SignedByte 				original_State;
	
	SPEED_DIGITS_REAL		horizontal_Offset,vertical_Offset;
/* END VAR */



	/* if (object_ID == LETTER_R) */
		
		
	if (Begin_Use_Of_Handle((Handle)object_Handle
								, MOVE_HIGH_NOT_REQUIRED
								, &original_State)	!= SAFE_TRUE) {
		goto Error_Handler;
	}
	
	
	
	object_Vertices = ((**object_Handle).object_Vertices);
	object_Shape = ((**object_Handle).object_Shape);
	
	
	
	current_Point = 0;
	current_Item = 0;
	
	/*=--------------------------------------------------------------------=*/
	/* Draw the letter 'R' at another Z level of '1.0' */
	/*=--------------------------------------------------------------------=*/
	


	object_Vertices[current_Point] = 1.0;	/* 0 */
	current_Point +=1;
	object_Vertices[current_Point] = 0.0;
	current_Point +=1;
	object_Vertices[current_Point] = 0.0;
	current_Point +=1;

	
	object_Vertices[current_Point] = 1.0;	/* 1 */
	current_Point +=1;
	object_Vertices[current_Point] = 8.0;
	current_Point +=1;
	object_Vertices[current_Point] = 0.0;
	current_Point +=1;
	
	
	object_Shape[current_Item] = k_LINE_SHAPE;
	current_Item +=1;
	object_Shape[current_Item] = 0;
	current_Item +=1;
	object_Shape[current_Item] = 1;
	current_Item +=1;

		
	object_Vertices[current_Point] = 1.0;	/* 2 */
	current_Point +=1;
	object_Vertices[current_Point] = 8.0;
	current_Point +=1;
	object_Vertices[current_Point] = 1.0;
	current_Point +=1;
	
	
	object_Shape[current_Item] = k_LINE_SHAPE;
	current_Item +=1;
	object_Shape[current_Item] = 1;
	current_Item +=1;
	object_Shape[current_Item] = 2;
	current_Item +=1;

	
	
	object_Vertices[current_Point] = 1.0;	/* 3 */
	current_Point +=1;
	object_Vertices[current_Point] = 5.0;
	current_Point +=1;
	object_Vertices[current_Point] = 1.0;
	current_Point +=1;
	
	
	
	object_Shape[current_Item] = k_LINE_SHAPE;
	current_Item +=1;
	object_Shape[current_Item] = 2;
	current_Item +=1;
	object_Shape[current_Item] = 3;
	current_Item +=1;
	
	

	
	object_Vertices[current_Point] = 1.0;	/* 4 */
	current_Point +=1;
	object_Vertices[current_Point] = 8.0;
	current_Point +=1;
	object_Vertices[current_Point] = 4.0;
	current_Point +=1;
	
	
	object_Shape[current_Item] = k_LINE_SHAPE;
	current_Item +=1;
	object_Shape[current_Item] = 3;
	current_Item +=1;
	object_Shape[current_Item] = 4;
	current_Item +=1;
	
	
	object_Vertices[current_Point] = 1.0;	/* 5 */
	current_Point +=1;
	object_Vertices[current_Point] = 8.0;
	current_Point +=1;
	object_Vertices[current_Point] = 5.0;
	current_Point +=1;
	
	object_Shape[current_Item] = k_LINE_SHAPE;
	current_Item +=1;
	object_Shape[current_Item] = 4;
	current_Item +=1;
	object_Shape[current_Item] = 5;
	current_Item +=1;
	
	
	
	object_Vertices[current_Point] = 1.0;	/* 6 */
	current_Point +=1;
	object_Vertices[current_Point] = 5.0;
	current_Point +=1;
	object_Vertices[current_Point] = 2.0;
	current_Point +=1;
	
	
	object_Shape[current_Item] = k_LINE_SHAPE;
	current_Item +=1;
	object_Shape[current_Item] = 5;
	current_Item +=1;
	object_Shape[current_Item] = 6;
	current_Item +=1;
	
	
	
	object_Vertices[current_Point] = 1.0;	/* 7 */
	current_Point +=1;
	object_Vertices[current_Point] = 5.0;
	current_Point +=1;
	object_Vertices[current_Point] = 3.0;
	current_Point +=1;
	
	
	object_Shape[current_Item] = k_LINE_SHAPE;
	current_Item +=1;
	object_Shape[current_Item] = 6;
	current_Item +=1;
	object_Shape[current_Item] = 7;
	current_Item +=1;
	
	
	object_Vertices[current_Point] = 1.0;	/* 8 */
	current_Point +=1;
	object_Vertices[current_Point] = 3.0;
	current_Point +=1;
	object_Vertices[current_Point] = 5.0;
	current_Point +=1;
	
	object_Shape[current_Item] = k_LINE_SHAPE;
	current_Item +=1;
	object_Shape[current_Item] = 7;
	current_Item +=1;
	object_Shape[current_Item] = 8;
	current_Item +=1;
	
		
	object_Vertices[current_Point] = 1.0;	/* 9 */
	current_Point +=1;
	object_Vertices[current_Point] = 2.0;
	current_Point +=1;
	object_Vertices[current_Point] = 5.0;
	current_Point +=1;
	
	object_Shape[current_Item] = k_LINE_SHAPE;
	current_Item +=1;
	object_Shape[current_Item] = 8;
	current_Item +=1;
	object_Shape[current_Item] = 9;
	current_Item +=1;
	
	
	
	
	object_Vertices[current_Point] = 1.0;	/* 10 */
	current_Point +=1;
	object_Vertices[current_Point] = 0.0;
	current_Point +=1;
	object_Vertices[current_Point] = 3.0;
	current_Point +=1;
	
	object_Shape[current_Item] = k_LINE_SHAPE;
	current_Item +=1;
	object_Shape[current_Item] = 9;
	current_Item +=1;
	object_Shape[current_Item] = 10;
	current_Item +=1;
	
	
	
	
	object_Shape[current_Item] = k_LINE_SHAPE;
	current_Item +=1;
	object_Shape[current_Item] = 10;
	current_Item +=1;
	object_Shape[current_Item] = 0;
	current_Item +=1;
	

	/*=--------------------------------------------------------------------=*/
	/* Now repeat the letter 'R' at another Z level of '0.0' */
	/*=--------------------------------------------------------------------=*/
	

	object_Vertices[current_Point] = 0.0;	/* 11 */
	current_Point +=1;
	object_Vertices[current_Point] = 0.0;
	current_Point +=1;
	object_Vertices[current_Point] = 0.0;
	current_Point +=1;

	
	object_Vertices[current_Point] = 0.0;	/* 12 */
	current_Point +=1;
	object_Vertices[current_Point] = 8.0;
	current_Point +=1;
	object_Vertices[current_Point] = 0.0;
	current_Point +=1;
	
	
	object_Shape[current_Item] = k_LINE_SHAPE;
	current_Item +=1;
	object_Shape[current_Item] = 11;
	current_Item +=1;
	object_Shape[current_Item] = 12;
	current_Item +=1;
	
	
	
	object_Vertices[current_Point] = 0.0;	/* 13 */
	current_Point +=1;
	object_Vertices[current_Point] = 8.0;
	current_Point +=1;
	object_Vertices[current_Point] = 1.0;
	current_Point +=1;
	
	object_Shape[current_Item] = k_LINE_SHAPE;
	current_Item +=1;
	object_Shape[current_Item] = 12;
	current_Item +=1;
	object_Shape[current_Item] = 13;
	current_Item +=1;
	

	
	object_Vertices[current_Point] = 0.0;	/* 14 */
	current_Point +=1;
	object_Vertices[current_Point] = 5.0;
	current_Point +=1;
	object_Vertices[current_Point] = 1.0;
	current_Point +=1;
	
	object_Shape[current_Item] = k_LINE_SHAPE;
	current_Item +=1;
	object_Shape[current_Item] = 13;
	current_Item +=1;
	object_Shape[current_Item] = 14;
	current_Item +=1;
	
	
	
	
	object_Vertices[current_Point] = 0.0;	/* 15 */
	current_Point +=1;
	object_Vertices[current_Point] = 8.0;
	current_Point +=1;
	object_Vertices[current_Point] = 4.0;
	current_Point +=1;
	
	object_Shape[current_Item] = k_LINE_SHAPE;
	current_Item +=1;
	object_Shape[current_Item] = 14;
	current_Item +=1;
	object_Shape[current_Item] = 15;
	current_Item +=1;
	
	
	
	
	object_Vertices[current_Point] = 0.0;	/* 16 */
	current_Point +=1;
	object_Vertices[current_Point] = 8.0;
	current_Point +=1;
	object_Vertices[current_Point] = 5.0;
	current_Point +=1;
	
	object_Shape[current_Item] = k_LINE_SHAPE;
	current_Item +=1;
	object_Shape[current_Item] = 15;
	current_Item +=1;
	object_Shape[current_Item] = 16;
	current_Item +=1;
	
	
	
		
	object_Vertices[current_Point] = 0.0;	/* 17 */
	current_Point +=1;
	object_Vertices[current_Point] = 5.0;
	current_Point +=1;
	object_Vertices[current_Point] = 2.0;
	current_Point +=1;
	
	object_Shape[current_Item] = k_LINE_SHAPE;
	current_Item +=1;
	object_Shape[current_Item] = 16;
	current_Item +=1;
	object_Shape[current_Item] = 17;
	current_Item +=1;
	
	
	
	
	object_Vertices[current_Point] = 0.0;	/* 18 */
	current_Point +=1;
	object_Vertices[current_Point] = 5.0;
	current_Point +=1;
	object_Vertices[current_Point] = 3.0;
	current_Point +=1;
	
	object_Shape[current_Item] = k_LINE_SHAPE;
	current_Item +=1;
	object_Shape[current_Item] = 17;
	current_Item +=1;
	object_Shape[current_Item] = 18;
	current_Item +=1;
	
	
	
	
	object_Vertices[current_Point] = 0.0;	/* 19 */
	current_Point +=1;
	object_Vertices[current_Point] = 3.0;
	current_Point +=1;
	object_Vertices[current_Point] = 5.0;
	current_Point +=1;
	
	object_Shape[current_Item] = k_LINE_SHAPE;
	current_Item +=1;
	object_Shape[current_Item] = 18;
	current_Item +=1;
	object_Shape[current_Item] = 19;
	current_Item +=1;
	
	
	
	object_Vertices[current_Point] = 0.0;	/* 20 */
	current_Point +=1;
	object_Vertices[current_Point] = 2.0;
	current_Point +=1;
	object_Vertices[current_Point] = 5.0;
	current_Point +=1;
	
	object_Shape[current_Item] = k_LINE_SHAPE;
	current_Item +=1;
	object_Shape[current_Item] = 19;
	current_Item +=1;
	object_Shape[current_Item] = 20;
	current_Item +=1;
	
	
	object_Vertices[current_Point] = 0.0;	/* 21 */
	current_Point +=1;
	object_Vertices[current_Point] = 0.0;
	current_Point +=1;
	object_Vertices[current_Point] = 3.0;
	current_Point +=1;
	
	object_Shape[current_Item] = k_LINE_SHAPE;
	current_Item +=1;
	object_Shape[current_Item] = 20;
	current_Item +=1;
	object_Shape[current_Item] = 21;
	current_Item +=1;
	
	
	
	object_Shape[current_Item] = k_LINE_SHAPE;
	current_Item +=1;
	object_Shape[current_Item] = 21;
	current_Item +=1;
	object_Shape[current_Item] = 11;
	current_Item +=1;
	
	

	
	/*=--------------------------------------------------------------------=*/
	/* Draw support braces for the letter 'R'  */
	/*=--------------------------------------------------------------------=*/

	
	object_Shape[current_Item] = k_LINE_SHAPE;
	current_Item +=1;
	object_Shape[current_Item] = 11;
	current_Item +=1;
	object_Shape[current_Item] = 0;
	current_Item +=1;
	
	
	object_Shape[current_Item] = k_LINE_SHAPE;
	current_Item +=1;
	object_Shape[current_Item] = 12;
	current_Item +=1;
	object_Shape[current_Item] = 1;
	current_Item +=1;
	
	
	object_Shape[current_Item] = k_LINE_SHAPE;
	current_Item +=1;
	object_Shape[current_Item] = 13;
	current_Item +=1;
	object_Shape[current_Item] = 2;
	current_Item +=1;
	
	
	object_Shape[current_Item] = k_LINE_SHAPE;
	current_Item +=1;
	object_Shape[current_Item] = 14;
	current_Item +=1;
	object_Shape[current_Item] = 3;
	current_Item +=1;
	
	
	object_Shape[current_Item] = k_LINE_SHAPE;
	current_Item +=1;
	object_Shape[current_Item] = 15;
	current_Item +=1;
	object_Shape[current_Item] = 4;
	current_Item +=1;
	
	object_Shape[current_Item] = k_LINE_SHAPE;
	current_Item +=1;
	object_Shape[current_Item] = 16;
	current_Item +=1;
	object_Shape[current_Item] = 5;
	current_Item +=1;
	
	object_Shape[current_Item] = k_LINE_SHAPE;
	current_Item +=1;
	object_Shape[current_Item] = 17;
	current_Item +=1;
	object_Shape[current_Item] = 6;
	current_Item +=1;
	
	object_Shape[current_Item] = k_LINE_SHAPE;
	current_Item +=1;
	object_Shape[current_Item] = 18;
	current_Item +=1;
	object_Shape[current_Item] = 7;
	current_Item +=1;
	
	object_Shape[current_Item] = k_LINE_SHAPE;
	current_Item +=1;
	object_Shape[current_Item] = 19;
	current_Item +=1;
	object_Shape[current_Item] = 8;
	current_Item +=1;
	
	object_Shape[current_Item] = k_LINE_SHAPE;
	current_Item +=1;
	object_Shape[current_Item] = 20;
	current_Item +=1;
	object_Shape[current_Item] = 9;
	current_Item +=1;
		
	object_Shape[current_Item] = k_LINE_SHAPE;
	current_Item +=1;
	object_Shape[current_Item] = 21;
	current_Item +=1;
	object_Shape[current_Item] = 10;
	current_Item +=1;
	
	
	
	/*=--------------------------------------------------------------------=*/
	/* Adjust all vertice references to be equal to the beginning */
	/* location in the vertice lists (multiply each by 3)  */
	/*=--------------------------------------------------------------------=*/
	for (loop = 0; (loop < (current_Item )) ; loop = loop + 3) {
		object_Shape[loop +1] *= 3;
		object_Shape[loop +2] *= 3;
	}
	
	
	(**object_Handle).object_Vertices_Total_Elements = current_Point;
	(**object_Handle).object_Shape_Total_Elements = current_Item;


	
	/* Test crud */
/* =======
	horizontal_Offset = 60.0;
	vertical_Offset = 60.0;

	for (loop = 0; (loop < (current_Point /3 )) ; loop++) {
		object_Vertices[loop +1] += horizontal_Offset;
		object_Vertices[loop +2] += vertical_Offset;
	}
======== */

		
	if (End_Use_Of_Handle((Handle)object_Handle
								, original_State)	!= SAFE_TRUE) {
		goto Error_Handler;
	}

	goto Normal_Function_Exit;

Error_Handler:
	Assign_String_Hook(ns_Shell_Globals.error_Note_String, "\pUnable to complete function.");
	Display_Error_Message_Hook(unit_Name, code_Name);

Normal_Function_Exit:
	return;

}/* Build_Object_Shape */
/*=**************************************************************************=*/


/*=**************************************************************************=*/
/* END OF FILE: Three_D_Utilities.c */
