/* [Vertical_Blanking_Interrupt.c] START OF FILE */

/* [---BEGIN---] FILE HEADER COMMENTS */

/**
 ** SCREEN VIEW = Adobe Syntax Black-10, Tabs=6
 ** PRINTER VIEW = Adobe Syntax Black -10, Tabs=0.30 inch
 ** ========================================================================
 ** Vertical_Blanking_Interrupt
 ** Written by Jim Merkle
 ** [Vertical_Blanking_Interrupt.c] - supporting routines animating (during VBL: copy bitmaps or 
 ** 			swap pages or erase previous sprite)
 ** (C) Copyright 1992 by Jim Merkle
 ** All rights reserved, except those wishing to "copy" moderate sections of code or files may do so, so long as
 ** such code is never used directly either in animation-related tools or multimedia tools that are  
 ** used in live performance or live broadcast or recorded mass market broadcast or film. Desktop usage not restricted.
 ** Conventional desktop game or entertainment not restricted.
 ** ========================================================================
 **
 **
 ** =============
 ** Revision History
 ** =============
 **
 **  1992.03.05		(James Merkle)
 **						Initial coding started
 **
 **
 **
 **
 ** =================
 ** OTHER INFORMATION
 ** =================
 **
 **
 **
 **/

/* [----END----] FILE HEADER COMMENTS */
/*=**************************************************************************=*/



/*=**************************************************************************=*/

/* [---BEGIN---] FILE INCLUDE AREA */

#include <LoMem.h>
#include <GestaltEqu.h>

#include "Core_Globals.h"
#include "SysHooks.h"

#include "Vertical_Blanking_Interrupt.h"


/* [----END----] FILE INCLUDE AREA */
/*=**************************************************************************=*/


/*=**************************************************************************=*/
/* [---BEGIN---] DEFINE AREA */

#define 		GESTALT_FUNCTION_RES_TYPE			'GDEF'
#define 		GESTALT_FUNCTION_ID_SELECTOR		'GaGx'
#define		GESTALT_VBL_SYNC_RES_ID				130

/* [----END----] DEFINE AREA */
/*=**************************************************************************=*/


/*=**************************************************************************=*/
/* [---BEGIN---] TYPE AREA */



/* [----END----] TYPE AREA */
/*=**************************************************************************=*/


/*=**************************************************************************=*/
/* [---BEGIN---] PROTOTYPES-------------------------- */
#pragma mark PROTOTYPE_AREA

/*------------ LOCAL PROTOTYPES -------------------*/


/* [----END----] PROTOTYPES-------------------------- */
/*=**************************************************************************=*/


/*=**************************************************************************=*/
/* [---BEGIN---] GLOBAL DEFINITION AREA */

	Safe_Boolean										routine_Monitoring_VBL_Currently_Installed[MAXIMUM_DEVICE_SLOTS];
	modify_GDEF_VBL_Synchronizing_Code_Ptr	routine_Monitoring_VBL_Data_Address[MAXIMUM_DEVICE_SLOTS];
	CARDINAL_32									*routine_Monitoring_VBL_Current_Count[MAXIMUM_DEVICE_SLOTS];
	CARDINAL_32									routine_Monitoring_VBL_Total_Of_Dependant_Ports[MAXIMUM_DEVICE_SLOTS];

/* [----END----] GLOBAL DEFINITION AREA */
/*=**************************************************************************=*/


/*=**************************************************************************=*/
/* [---BEGIN---] LOCAL STATIC DEFINITION AREA */

static CARDINAL_8				*unit_Name = "\pGutsUniv";

/* [----END----] LOCAL STATIC DEFINITION AREA */
/*=**************************************************************************=*/


/*=**************************************************************************=*/
Safe_Boolean	Initialize_Structures_Allowing_VBL_Slot_Monitoring(void)
{
	Safe_Boolean		function_Result;
	INTEGER_16		loop;
/* END VAR */

	function_Result = SAFE_FALSE;

	for (loop =0; (loop < MAXIMUM_DEVICE_SLOTS); loop++) {
		routine_Monitoring_VBL_Currently_Installed[loop] = SAFE_FALSE;
	}

	function_Result = SAFE_TRUE;
	goto Normal_Function_Exit;

Error_Handler:
	function_Result = SAFE_FALSE;

Normal_Function_Exit:
	return(function_Result);
	
} /* Safe_Boolean	Initialize_Structures_Allowing_VBL_Slot_Monitoring(void) */
/*=**************************************************************************=*/


/*=**************************************************************************=*/
Safe_Boolean	Destroy_Structures_Allowing_VBL_Slot_Monitoring(void)
{
	Safe_Boolean		function_Result;
	INTEGER_16		outer_Loop, inner_Loop;
	OSErr						os_Error;
	CARDINAL_32			total_Of_Active_Participants_When_Starting;
	CARDINAL_32			current_Total_Participants;
	static pascal OSErr		(*address_Of_Routine_To_Enter)(void);
	modify_GDEF_VBL_Synchronizing_Code_Ptr		vbl_Synch_Ptr;
/* END VAR */

	function_Result = SAFE_FALSE;

	for (outer_Loop =0; (outer_Loop < MAXIMUM_DEVICE_SLOTS); outer_Loop++) {
		if (routine_Monitoring_VBL_Currently_Installed[outer_Loop] == SAFE_TRUE) {
			total_Of_Active_Participants_When_Starting = (routine_Monitoring_VBL_Data_Address[outer_Loop])->current_VBL_Slot_Slot_Participants;
			for (inner_Loop =0; (inner_Loop < total_Of_Active_Participants_When_Starting); inner_Loop++) {

				/*=--------------------------------------------------------------------------=*/
				/* Though *VERY* improbable, "current_Total_Participants" can change */
				/* externally, so re-check inside this loop */
				/*=--------------------------------------------------------------------------=*/
				current_Total_Participants = (routine_Monitoring_VBL_Data_Address[outer_Loop])->current_VBL_Slot_Slot_Participants;
				if (current_Total_Participants > 0) {
				
					address_Of_Routine_To_Enter = ( pascal OSErr	(*)(void))(routine_Monitoring_VBL_Data_Address[outer_Loop])->remove_VBL_Tasks_Address;
				
					if (address_Of_Routine_To_Enter() != noErr) {
						goto Error_Handler;						
					}
					
					routine_Monitoring_VBL_Total_Of_Dependant_Ports[outer_Loop] = vbl_Synch_Ptr->current_VBL_Slot_Slot_Participants;
				}

			} /* for (inner_Loop =0; (inner_Loop < total_Of_Active_Participants_When_Starting); inner_Loop++)  */
			
			routine_Monitoring_VBL_Currently_Installed[outer_Loop] = SAFE_FALSE;
		}
	}

	function_Result = SAFE_TRUE;
	goto Normal_Function_Exit;

Error_Handler:
	function_Result = SAFE_FALSE;

Normal_Function_Exit:
	return(function_Result);
	
} /* Safe_Boolean	Destroy_Structures_Allowing_VBL_Slot_Monitoring(void) */
/*=**************************************************************************=*/


/*=**************************************************************************=*/
Safe_Boolean	Post_Gestalt_VBL_Slot_Synch_If_Needed( OSType Application_ID
									, CARDINAL_16	desired_VBL_Slot_Number	
									, Safe_Boolean 	*already_Present )
{
	Safe_Boolean					function_Result;
	INTEGER_16					os_Error;
	INTEGER_32					returned_Value;
	SysEnvRec					system_Environment;
	CARDINAL_32				memory_Requested;
	Handle						gestalt_Function_Hdl;
	Memory_Allocation			gestalt_Function_Memory_Initialized;
	Handle 						temp_Handle;
	modify_GDEF_VBL_Synchronizing_Code_Ptr		vbl_Synch_Ptr;
/* END VAR */

	function_Result = SAFE_FALSE;
	*already_Present = SAFE_FALSE;


	os_Error = SysEnvirons(SYSTEM_0605, &system_Environment);
	if (os_Error != noErr) {
		function_Result = SAFE_FALSE;
		goto Normal_Function_Exit;
	}
	
	if (Gestalt_Available_Hook() != TRUE) {
		function_Result = SAFE_FALSE;
		goto Normal_Function_Exit;
	}


	os_Error = Gestalt(Application_ID, &returned_Value) ;
	if (os_Error != noErr) {
		if (os_Error == gestaltUndefSelectorErr) {
			/*=--------------------------------------------------------------------------=*/
			/* It is not registered yet! good. continue */
			/*=--------------------------------------------------------------------------=*/
		}
		else {
			goto Error_Handler;
		}
	}
	else {
			/*=--------------------------------------------------------------------------=*/
			/* It is already registered! good. */
			/*=--------------------------------------------------------------------------=*/
			function_Result = SAFE_TRUE;
			*already_Present = SAFE_TRUE;
			goto Normal_Function_Exit;
	}


	/*=--------------------------------------------------------------------------=*/
	/* Note the GDEF does not need to have resource bits set for System heap any longer. */
	/* This is because multiple copies of the resource need to be made anyway because */
	/* the resource is modified and used by many potential programs. */
	/* It is not harmful to be set for system heap, but for logical use of compression code, */
	/* it should be in application heap */
	/*=--------------------------------------------------------------------------=*/
	
	temp_Handle = Get1Resource(GESTALT_FUNCTION_RES_TYPE, GESTALT_VBL_SYNC_RES_ID);
	if ((temp_Handle == nil) || (ResError() != noErr)){
		goto Error_Handler;
	}


	gestalt_Function_Memory_Initialized = MEMORY_NOT_INITIALIZED;
	gestalt_Function_Hdl = nil;
	memory_Requested = GetHandleSize(temp_Handle);
	if (MemError() != noErr)
		goto Error_Handler;

	if ( Create_System_Handle_Hook(&memory_Requested
									, 0L
									, ALLOW_PURGING_HOOK
									, DISALLOW_PARTIAL_AMOUNT_HOOK
									, &gestalt_Function_Hdl
									, &gestalt_Function_Memory_Initialized) != SAFE_TRUE ){
		goto Error_Handler;
	}

		/*=--------------------------------------------------------------------------=*/
		/* Simple paranoia check */
		/*=--------------------------------------------------------------------------=*/
		if (GetHandleSize(gestalt_Function_Hdl) != GetHandleSize(temp_Handle)) {
			goto Error_Handler;
		}

		/*=--------------------------------------------------------------------------=*/
		/* copy GDEF from resource to system heap copy and release resource. */
		/*=--------------------------------------------------------------------------=*/
	
	BlockMove(*temp_Handle, *gestalt_Function_Hdl, memory_Requested);
	
	ReleaseResource(temp_Handle);
	if (ResError() != noErr)
		goto Error_Handler;
	
		/*=--------------------------------------------------------------------------=*/
		/* I, JM, have observed that the function MoveHHi will usually fail to move the handle */
		/* much higher than it is created originally in 32bit sys 7. In fact many unlocked handles */
		/* are above it, but Apple probably knows what those things are and why they are there. */
		/* Oddly resources can be loaded and locked Low in system heap though */ 
		/*=--------------------------------------------------------------------------=*/
	MoveHHi(gestalt_Function_Hdl);
	if (MemError() != noErr)
		goto Error_Handler;

	HNoPurge(gestalt_Function_Hdl);
	if (MemError() != noErr)
		goto Error_Handler;

	HLock(gestalt_Function_Hdl);
	if (MemError() != noErr)
		goto Error_Handler;

	vbl_Synch_Ptr = (modify_GDEF_VBL_Synchronizing_Code_Ptr) *(gestalt_Function_Hdl);
	/*=--------------------------------------------------------------------------=*/
	/* Insert our info into the GDEF, before calling initialization routine */
	/*=--------------------------------------------------------------------------=*/
	vbl_Synch_Ptr->current_VBL_Slot_Number = desired_VBL_Slot_Number;
	vbl_Synch_Ptr->current_VBL_Slot_Task_Initialized = SAFE_FALSE;
	
	/*=--------------------------------------------------------------------------=*/
	/* Paranoia initialization only */
	/*=--------------------------------------------------------------------------=*/
	vbl_Synch_Ptr->start_VBL_Tasks_Address = 0;
	vbl_Synch_Ptr->remove_VBL_Tasks_Address = 0;
	vbl_Synch_Ptr->current_VBL_Slot_Count = 0;
	vbl_Synch_Ptr->current_VBL_Slot_Slot_Participants = 0;
	



#if !defined(ORIGINAL_MAC_SYSTEM)
#error "UNDEFINED PARAMETER!"
#endif
#if (ORIGINAL_MAC_SYSTEM == 0)
Dialogs.h
XXXX 		
	os_Error = NewGestalt(Application_ID, (ProcPtr) StripAddress((Ptr) *gestalt_Function_Hdl));
#else
	os_Error = NewGestalt(Application_ID, (SelectorFunctionUPP) StripAddress((Ptr) *gestalt_Function_Hdl));
#endif 

	if (os_Error != noErr) {
		if (os_Error == gestaltDupSelectorErr) {
			/*=--------------------------------------------------------------------------=*/
			/* Although checked only a moment ago, It was registered behind us! ok. */
			/*=--------------------------------------------------------------------------=*/
			function_Result = SAFE_TRUE;
			*already_Present = SAFE_TRUE;
			
			DisposHandle(gestalt_Function_Hdl);
			if (MemError() != noErr) {
				goto Error_Handler;
			}

			goto Normal_Function_Exit;
		}
		else {
			function_Result = SAFE_FALSE;
			goto Normal_Function_Exit;
		}
	}

	*already_Present = SAFE_FALSE;
	function_Result = SAFE_TRUE;
	goto Normal_Function_Exit;

Error_Handler:
		/*=--------------------------------------------------------------------------=*/
		/* no room for elaborate error message in this program (yet), call SysBeep. */
		/*=--------------------------------------------------------------------------=*/
		SysBeep(20);
		function_Result = SAFE_FALSE;

Normal_Function_Exit:
	return(function_Result);
	
} /* Post_Gestalt_Breadcrumbs_If_Needed */
/*=**************************************************************************=*/


/*=**************************************************************************=*/
Safe_Boolean	Install_VBL_Synchronizer_For_Slot( CARDINAL_8	video_Device_Slot_To_Install_Synchronizer_On)
{
	Safe_Boolean				function_Result;
	OSType 					gestalt_Application_ID;
	CARDINAL_16			desired_VBL_Slot_Number;
	INTEGER_32				returned_Value;
	Safe_Boolean 			already_Present;
	CARDINAL_32			old_Participant_Count;
	OSErr						os_Error;
	static pascal OSErr		(*address_Of_Routine_To_Enter)(void);
	modify_GDEF_VBL_Synchronizing_Code_Ptr		vbl_Synch_Ptr;
/* END VAR */


	function_Result = SAFE_FALSE;
	desired_VBL_Slot_Number = video_Device_Slot_To_Install_Synchronizer_On;
	routine_Monitoring_VBL_Currently_Installed[video_Device_Slot_To_Install_Synchronizer_On] = SAFE_FALSE;

	
/* =====
	gestalt_Application_ID = GESTALT_FUNCTION_ID_SELECTOR;
====== */
	/*=--------------------------------------------------------------------------=*/
	/* gestalt entry WAS based on three character identifier with byte at end for slot */
	/* but now, though perhaps riskier, my VBL monitoring will use the low 24 bits of the */
	/* current stack base to allow multiple copies of this program to run simultaneously */
	/*=--------------------------------------------------------------------------=*/
	gestalt_Application_ID = (OSType)(*CurStackBase);
	gestalt_Application_ID = (gestalt_Application_ID << 8);
	

	/*=--------------------------------------------------------------------------=*/
	/* Clear out lowest byte of selector, to prepare for installation*/
	/*=--------------------------------------------------------------------------=*/
	gestalt_Application_ID = (gestalt_Application_ID & 0xFFFFFF00);
	
	/*=--------------------------------------------------------------------------=*/
	/* Insert the VBL device slot over the last byte of the selector */
	/*=--------------------------------------------------------------------------=*/
	gestalt_Application_ID = (gestalt_Application_ID | desired_VBL_Slot_Number);


	if (	Post_Gestalt_VBL_Slot_Synch_If_Needed(  gestalt_Application_ID
								, (CARDINAL_16)	video_Device_Slot_To_Install_Synchronizer_On	
								, &already_Present ) != SAFE_TRUE) {
		goto Error_Handler;						
	}

	os_Error = Gestalt(gestalt_Application_ID, &returned_Value) ;
	if (os_Error != noErr) {
		/*=--------------------------------------------------------------------------=*/
		/* It should already be registered! fail. */
		/*=--------------------------------------------------------------------------=*/
		goto Error_Handler;						
	}
	else {
		/*=--------------------------------------------------------------------------=*/
		/* It is registered. Good, now determine if it has been initialized and initialize it if needed */
		/*=--------------------------------------------------------------------------=*/
		
		vbl_Synch_Ptr = (modify_GDEF_VBL_Synchronizing_Code_Ptr)returned_Value;
		/*=--------------------------------------------------------------------------=*/
		/* Validate the GDEF, before continuing */
		/*=--------------------------------------------------------------------------=*/
		if (vbl_Synch_Ptr->current_VBL_Slot_Number != desired_VBL_Slot_Number) {
			goto Error_Handler;						
		}
		else {
			/*=--------------------------------------------------------------------------=*/
			/* If not initialized, initialize it*/
			/*=--------------------------------------------------------------------------=*/
			if (vbl_Synch_Ptr->current_VBL_Slot_Task_Initialized != SAFE_TRUE){

				address_Of_Routine_To_Enter = ( pascal OSErr	(*)(void))(&(vbl_Synch_Ptr->branch_To_Install_Synchronizing));
				
				if (address_Of_Routine_To_Enter() != noErr) {
					goto Error_Handler;						
				}
				
				routine_Monitoring_VBL_Current_Count[video_Device_Slot_To_Install_Synchronizer_On] =&(vbl_Synch_Ptr->current_VBL_Slot_Count);
				routine_Monitoring_VBL_Currently_Installed[video_Device_Slot_To_Install_Synchronizer_On] = SAFE_TRUE;
				routine_Monitoring_VBL_Data_Address[video_Device_Slot_To_Install_Synchronizer_On] = vbl_Synch_Ptr;
			}
			else {
				/*=--------------------------------------------------------------------------=*/
				/* Perhaps it was externally created with same gestalt selector and never killed yet. */
				/* Though rare, because the selector is based partly on "CurStackBase" value, it is */
				/* possible, so just reset the globals in this application to avoid re-installing it */
				/*=--------------------------------------------------------------------------=*/
				routine_Monitoring_VBL_Current_Count[video_Device_Slot_To_Install_Synchronizer_On] =&(vbl_Synch_Ptr->current_VBL_Slot_Count);
				routine_Monitoring_VBL_Currently_Installed[video_Device_Slot_To_Install_Synchronizer_On] = SAFE_TRUE;
				routine_Monitoring_VBL_Data_Address[video_Device_Slot_To_Install_Synchronizer_On] = vbl_Synch_Ptr;
			}

			/*=--------------------------------------------------------------------------=*/
			/* At this point, we know that the system is initialized already, */
			/* and has 0 or more active participants.*/
			/* Adding a participant when there are none, will automatically */
			/* create a VBL slot queue task */
			/*=--------------------------------------------------------------------------=*/
			old_Participant_Count = vbl_Synch_Ptr->current_VBL_Slot_Slot_Participants;

			address_Of_Routine_To_Enter = ( pascal OSErr	(*)(void))vbl_Synch_Ptr->start_VBL_Tasks_Address;
			
			os_Error = address_Of_Routine_To_Enter();
			if (os_Error != noErr) {
				goto Error_Handler;						
			}
			
			if (vbl_Synch_Ptr->current_VBL_Slot_Slot_Participants != (old_Participant_Count +1)) {
				goto Error_Handler;						
			}
			
			routine_Monitoring_VBL_Total_Of_Dependant_Ports[video_Device_Slot_To_Install_Synchronizer_On] = vbl_Synch_Ptr->current_VBL_Slot_Slot_Participants;
				
		} /* NOT if (vbl_Synch_Ptr->current_VBL_Slot_Number != desired_VBL_Slot_Number) */
		
	} /* NOT if (os_Error != noErr) */


	function_Result = SAFE_TRUE;
	goto Normal_Function_Exit;

Error_Handler:
		/*=--------------------------------------------------------------------------=*/
		/* no room for elaborate error message in this program (yet), call SysBeep. */
		/*=--------------------------------------------------------------------------=*/
		SysBeep(20);
		function_Result = SAFE_FALSE;

Normal_Function_Exit:
	return(function_Result);
	
} /* Safe_Boolean	Install_VBL_Synchronizer_For_Slot( CARDINAL_8	video_Device_Slot_To_Install_Synchronizer_On) */
/*=**************************************************************************=*/


/*=**************************************************************************=*/
Safe_Boolean	Install_VBL_Synchronizer_For_Slot_If_Needed( Rect	active_Port_Sample_Global_Rect, CARDINAL_8	 *video_Device_Slot_To_Synchronize_With)
{
	Safe_Boolean		function_Result;
	INTEGER_32	 	bits_Per_Pixel;
	INTEGER_32	 	bits_Per_Pixel_Greatest;
	CARDINAL_16 	 total_Intersected_Screens;
	Safe_Boolean		rect_Is_Within_A_Screen;
	Safe_Boolean		slot_Information_Valid;												
	INTEGER_16		video_Monitor_Driver_Ref;
	INTEGER_8		video_Monitor_Slot;
	INTEGER_8		video_Monitor_ID;
	PixMapHandle	video_Pix_Map_Handle;
	GDHandle			graphics_Device;
	CARDINAL_32	video_Optional_Base_Address;
	OSErr				os_Error;
	static pascal OSErr									(*address_Of_Routine_To_Enter)(void);
	modify_GDEF_VBL_Synchronizing_Code_Ptr		vbl_Synch_Ptr;
/* END VAR */


	function_Result = SAFE_FALSE;

	Determine_Information_Of_Device_Containing_Rect_Hook_NEW(&active_Port_Sample_Global_Rect
													, &video_Monitor_Driver_Ref
													, &video_Monitor_Slot
													, &video_Monitor_ID
													, &video_Optional_Base_Address
													, &bits_Per_Pixel
													, &bits_Per_Pixel_Greatest
													, &total_Intersected_Screens
													, &rect_Is_Within_A_Screen
													, &video_Pix_Map_Handle
													, &graphics_Device
													, &slot_Information_Valid);												

	if (rect_Is_Within_A_Screen == SAFE_TRUE) {
	
		if (slot_Information_Valid == SAFE_TRUE) {
			*video_Device_Slot_To_Synchronize_With = video_Monitor_Slot;
			
			if ((*video_Device_Slot_To_Synchronize_With < 0) || (*video_Device_Slot_To_Synchronize_With > MAXIMUM_DEVICE_SLOTS)) {
				/*  */
				/* This error should not happen for many years */
				/*  */
				goto Error_Handler;
			}
			
	
			if (routine_Monitoring_VBL_Currently_Installed[*video_Device_Slot_To_Synchronize_With] != SAFE_TRUE) {
	
				if (Install_VBL_Synchronizer_For_Slot(*video_Device_Slot_To_Synchronize_With) != SAFE_TRUE) {
					goto Error_Handler;
				}

			}
			else {

				address_Of_Routine_To_Enter = ( pascal OSErr	(*)(void))(routine_Monitoring_VBL_Data_Address[*video_Device_Slot_To_Synchronize_With])->start_VBL_Tasks_Address;
			
				os_Error = address_Of_Routine_To_Enter();
				if (os_Error != noErr) {
					goto Error_Handler;						
				}
				
				routine_Monitoring_VBL_Total_Of_Dependant_Ports[*video_Device_Slot_To_Synchronize_With] = vbl_Synch_Ptr->current_VBL_Slot_Slot_Participants;
			}
	
		} /* if (slot_Information_Valid == SAFE_TRUE) */
	
	} /* if (rect_Is_Within_A_Screen == SAFE_TRUE) */
	else {
		/*  */
		/* Nothing can be done, the window should be moved. */
		/* But just in case a Radius "pivot" just took place, this */
		/* routine SHOULD WARN USER TO MOVE WINDOW */
		/*  */
	
		/*  */
		/* !!! Add warning for user soon */
		/*  */
	
	}

	function_Result = SAFE_TRUE;
	goto Normal_Function_Exit;
	
Error_Handler:
	function_Result = SAFE_FALSE;

Normal_Function_Exit:
	return(function_Result);

} /* Safe_Boolean	Install_VBL_Synchronizer_For_Slot_If_Needed( Rect	active_Port_Sample_Local_Rect) ... */
/*=**************************************************************************=*/


/*=**************************************************************************=*/
Safe_Boolean	Remove_VBL_Synchronizer_For_Slot_If_Needed( Rect	active_Port_Sample_Local_Rect)
{
	Safe_Boolean		function_Result;
	INTEGER_32	 	bits_Per_Pixel;
	INTEGER_32	 	bits_Per_Pixel_Greatest;
	CARDINAL_16 	 total_Intersected_Screens;
	Safe_Boolean		rect_Is_Within_A_Screen;
	Safe_Boolean		slot_Information_Valid;												
	INTEGER_16		video_Monitor_Driver_Ref;
	INTEGER_8		video_Monitor_Slot;
	INTEGER_8		video_Monitor_ID;
	CARDINAL_32	video_Optional_Base_Address;
	INTEGER_16		video_Device_Slot_To_Synchronize_With;
	CARDINAL_32	current_Total_Participants;
	PixMapHandle	video_Pix_Map_Handle;
	GDHandle			graphics_Device;
	OSErr				os_Error;
	static pascal OSErr									(*address_Of_Routine_To_Enter)(...);
	modify_GDEF_VBL_Synchronizing_Code_Ptr		vbl_Synch_Ptr;
/* END VAR */


	function_Result = SAFE_FALSE;

	Determine_Information_Of_Device_Containing_Rect_Hook_NEW(&active_Port_Sample_Local_Rect
													, &video_Monitor_Driver_Ref
													, &video_Monitor_Slot
													, &video_Monitor_ID
													, &video_Optional_Base_Address
													, &bits_Per_Pixel
													, &bits_Per_Pixel_Greatest
													, &total_Intersected_Screens
													, &rect_Is_Within_A_Screen
													, &video_Pix_Map_Handle
													, &graphics_Device
													, &slot_Information_Valid);													

	if (rect_Is_Within_A_Screen == SAFE_TRUE) {
	
		if (slot_Information_Valid == SAFE_TRUE) {
			video_Device_Slot_To_Synchronize_With = video_Monitor_Slot;
			
			if ((video_Device_Slot_To_Synchronize_With < 0) || (video_Device_Slot_To_Synchronize_With > MAXIMUM_DEVICE_SLOTS)) {
				/*  */
				/* This error should not happen for many years */
				/*  */
				goto Error_Handler;
			}
			
	
			if (routine_Monitoring_VBL_Currently_Installed[video_Device_Slot_To_Synchronize_With] != SAFE_TRUE) {

				/*  */
				/* This error could happen if desktop "pivoted" or other things such as overlapping windows */
				/*  */
				function_Result = SAFE_TRUE;
				goto Normal_Function_Exit;
			}
			else {

				current_Total_Participants = (routine_Monitoring_VBL_Data_Address[video_Device_Slot_To_Synchronize_With])->current_VBL_Slot_Slot_Participants;
				if (current_Total_Participants > 0) {
				
					address_Of_Routine_To_Enter = ( pascal OSErr	(*)(...))(routine_Monitoring_VBL_Data_Address[video_Device_Slot_To_Synchronize_With])->remove_VBL_Tasks_Address;
				
					os_Error = address_Of_Routine_To_Enter();
					if (os_Error != noErr) {
						goto Error_Handler;						
					}
					
					
					routine_Monitoring_VBL_Total_Of_Dependant_Ports[video_Device_Slot_To_Synchronize_With] = vbl_Synch_Ptr->current_VBL_Slot_Slot_Participants;
				}
			}
	
		} /* if (slot_Information_Valid == SAFE_TRUE) */
	
	} /* if (rect_Is_Within_A_Screen == SAFE_TRUE) */
	else {
		/*  */
		/* This error could happen if desktop "pivoted" or other things such as overlapping windows */
		/*  */
		function_Result = SAFE_TRUE;
		goto Normal_Function_Exit;
	
	}

	function_Result = SAFE_TRUE;
	goto Normal_Function_Exit;
	
Error_Handler:
	function_Result = SAFE_FALSE;

Normal_Function_Exit:
	return(function_Result);

} /* Safe_Boolean	Remove_VBL_Synchronizer_For_Slot_If_Needed( Rect	active_Port_Sample_Local_Rect) */
/*=**************************************************************************=*/


/*=**************************************************************************=*/
void Wait_For_Next_Video_Monitor_VBL( CARDINAL_8	video_Device_Slot_To_Synchronize_With )
{
	register 		CARDINAL_32	vbl_Count_At_Entry;
	register 		CARDINAL_32	dead_Loop_Counter;
	 volatile 		CARDINAL_32	*vbl_Count_Ptr;
/* END VAR */

	dead_Loop_Counter = 0;
	
	if (routine_Monitoring_VBL_Currently_Installed[video_Device_Slot_To_Synchronize_With] == SAFE_TRUE) {
		vbl_Count_Ptr =  routine_Monitoring_VBL_Current_Count[video_Device_Slot_To_Synchronize_With];
	
		vbl_Count_At_Entry = *vbl_Count_Ptr;
	
		/*  */
		/* Because THINK C 5.02 violates ANSI guidelines for "volatile" */
		/* (and is non-ANSI compliant) the following loop will NOT compare */
		/* memory for external modification when all optimization is on */
		/*  */
/* ========
		while (vbl_Count_At_Entry == (volatile CARDINAL_32)*vbl_Count_Ptr) {
			dead_Loop_Counter += 1;
		}
======== */

		asm {
			move.l			dead_Loop_Counter,d1
			move.l			vbl_Count_Ptr,a0
			move.l			vbl_Count_At_Entry,d0
			
@Loop_Back_And_Test_For_VBL
			addq.l				#1,d1
			cmp.l				(a0),d0
			beq.s				@Loop_Back_And_Test_For_VBL
			move.l			d1,dead_Loop_Counter
		}	
	
	}
	else {
		vbl_Count_At_Entry = TickCount();
		/*  */
		/* Note that "TickCount" is called repeatedly as a trap and */
		/* not the nasty way of accessing memory directly. */
		/* The reason is very complex to explain in this small comment */
		/* but the reasons to do so are many. JM */
		/*  */
		while (vbl_Count_At_Entry == TickCount()) {
			dead_Loop_Counter += 1;
		}
	}
	
	
	
} /* void Wait_For_Next_Video_Monitor_VBL */
/*=**************************************************************************=*/


/*=**************************************************************************=*/
/* END OF FILE: Vertical_Blanking_Interrupt.c */
