/* START OF FILE: Object_Inventory.h */


/*=**************************************************************************=*/
/* [---BEGIN---] FILE HEADER COMMENTS */
/**
 ** SCREEN VIEW = [Adobe Syntax Black-10, Tabs = 4]
 ** PRINTER VIEW = [Adobe Syntax Black-10, Tabs = 4]
 ** =======================================================================
 ** Written by James Merkle
 ** Object_Inventory.h - Header for Object_Inventory.c
 ** (C) Copyright 1994 James Merkle
 ** =======================================================================
 **
 **
 ** ============
 ** Revision History
 ** ============
 **
 **  Refer to "Object_Inventory.c"
 **						
 **
 ** ====================
 ** OTHER INFORMATION
 ** ====================
 ** 
 **
 **/
/* [----END----] FILE HEADER COMMENTS */
/*=**************************************************************************=*/



/*=**************************************************************************=*/
#ifndef _H_Object_Inventory

#define _H_Object_Inventory
/*=**************************************************************************=*/


/*=**************************************************************************=*/
/* [---BEGIN---] #include AREA */

#include "Application_Limits.h"
#include "Shell_Globals.h"

/* [----END----] #include AREA */
/*=**************************************************************************=*/


/*=**************************************************************************=*/
/* [---BEGIN---] #define AREA */

#define k_PICT_FILE_TYPE		'PICT'

#define k_SEARCH_TIMEOUT							(6L)

#define k_SEARCH_ACTIVE_YIELD_TIMEOUT			k_SEARCH_TIMEOUT

#define k_RECURSIVE_SEARCH_ACTIVE_TIMEOUT	k_SEARCH_TIMEOUT


/* [----END----] #define AREA */
/*=**************************************************************************=*/


/*=**************************************************************************=*/
/* [---BEGIN---] TYPE AREA */


typedef struct {
	Safe_Boolean		active;
	CARDINAL_16	type_Of_Source;
	CARDINAL_32	algorithm_Id;
	INTEGER_16		resource_Id;
	FSSpec			file_Info;
	Str31				scene_Name;
} Scene_Database_Rec, *Scene_Database_Ptr;

	
typedef struct {
	Scene_Database_Rec		scenes[k_MAXIMUM_DATABASE_SCENES];
}Scenes_Array, *Scenes_Ptr, **Scenes_Hdl;


typedef struct {
	Safe_Boolean		active;
	CARDINAL_16	type_Of_Source;
	CARDINAL_32	algorithm_Id;
	INTEGER_16		resource_Id;
	FSSpec			file_Info;
	Str31				puppet_Name;
} Puppet_Database_Rec, *Puppet_Database_Ptr;

	
typedef struct {
	Puppet_Database_Rec		puppets[k_MAXIMUM_DATABASE_PUPPETS];
}Puppets_Array, *Puppets_Ptr, **Puppets_Hdl;

typedef struct {
	Safe_Boolean		active;
	CARDINAL_16	type_Of_Source;
	CARDINAL_32	algorithm_Id;
	INTEGER_16		resource_Id;
	FSSpec			file_Info;
	Str31				backdrop_Name;
} Backdrop_Database_Rec, *Backdrop_Database_Ptr;

	
typedef struct {
	Backdrop_Database_Rec		backdrops[k_MAXIMUM_DATABASE_BACKDROPS];
}Backdrops_Array, *Backdrops_Ptr, **Backdrops_Hdl;



typedef struct {
	Safe_Boolean		active;
	CARDINAL_16	type_Of_Source;
	CARDINAL_32	algorithm_Id;
	INTEGER_16		resource_Id;
	FSSpec			file_Info;
	Str31				spin_Object_Name;
} Spin_Object_Database_Rec, *Spin_Object_Database_Ptr;

typedef struct {
	Spin_Object_Database_Rec		spin_Objects[k_MAXIMUM_DATABASE_SPIN_OBJECTS];
}Spin_Objects_Array, *Spin_Objects_Ptr, **Spin_Objects_Hdl;


typedef struct {
	Safe_Boolean		active;
	CARDINAL_16	type_Of_Source;
	CARDINAL_32	algorithm_Id;
	INTEGER_16		resource_Id;
	FSSpec			file_Info;
	Str31				sound_Object_Name;
} Sound_Database_Rec, *Sound_Database_Ptr;

typedef struct {
	Sound_Database_Rec		sound_Objects[k_MAXIMUM_DATABASE_SOUND_OBJECTS];
}Sounds_Array, *Sounds_Ptr, **Sounds_Hdl;


typedef struct {
	Scenes_Ptr			pointer;
	Scenes_Hdl			handle;
} Remote_Scenes_Rec;

typedef struct {
	Puppets_Ptr			pointer;
	Puppets_Hdl			handle;
} Puppet_List_Rec;

typedef struct {
	Backdrops_Ptr			pointer;
	Backdrops_Hdl			handle;
} Backdrop_List_Rec;

typedef struct {
	Spin_Objects_Ptr			pointer;
	Spin_Objects_Hdl		handle;
} Spin_Object_List_Rec;

typedef struct {
	Sounds_Ptr			pointer;
	Sounds_Hdl			handle;
} Sound_List_Rec;

union Union_List_Record{
	Remote_Scenes_Rec			scenes;
	Puppet_List_Rec				puppets;
	Backdrop_List_Rec			backdrops;
	Spin_Object_List_Rec		spin_Objects;
	Sound_List_Rec				sounds;
};

typedef struct {
	Safe_Boolean			recent_Valid;
	Handle				recent_Hdl;
	Memory_Allocation	recent_Memory_In_Use;
	CARDINAL_32		recent_Length;
	
	CARDINAL_32		total_Available;
	Memory_Allocation	memory_In_Use;
	union Union_List_Record	info;
} Database_Control;



/* [----END----] TYPE AREA */
/*=**************************************************************************=*/


/*=**************************************************************************=*/
/* [---BEGIN---] PROTOTYPE AREA */

extern Safe_Boolean Initialize_Object_Inventory(void);

extern Safe_Boolean Destroy_Object_Inventory(void);

extern  Safe_Boolean Locate_Item_Location(CARDINAL_32		object_Classification
																, Str63			*item_Name
																, FSSpec		*file_Specification
																, CARDINAL_32	*item_Address
																, Safe_Boolean	*address_Instead_Of_File);

extern  Safe_Boolean	Start_Scanning_For_Files(CARDINAL_32		object_Classification);
extern  Safe_Boolean	Continue_Scanning_For_Files(void);
extern  Safe_Boolean	Shutdown_File_Scanning(void);

extern Safe_Boolean	Add_Potential_File_Item_To_Database(CARDINAL_32 file_Type, FSSpec *file_Specification, Boolean	*database_Full);

/* [----END----] PROTOTYPE AREA */
/*=**************************************************************************=*/


/*=**************************************************************************=*/
/* [---BEGIN---] extern DECLARATIONS */

extern Safe_Boolean		g_Scanning_Active;
extern Safe_Boolean		g_Scanning_Needs_Run;

extern Database_Control	g_Remote_Scenes;
extern Database_Control	g_Puppets;
extern Database_Control	g_Backdrops;
extern Database_Control	g_Spin_Objects;
extern	Database_Control	g_Audio_Files;

/* Check for accidental assignments of "g_Remote_Scenes.info.scenes.handle" */
/* istead of g_Remote_Scenes.recent_Hdl !!! */ 


/* [----END----] extern DECLARATIONS */
/*=**************************************************************************=*/


/*=**************************************************************************=*/
/* [---BEGIN---] LOCAL STATIC DEFINITION AREA */


/* [----END----] LOCAL STATIC DEFINITION AREA */
/*=**************************************************************************=*/


/*=**************************************************************************=*/
#endif
/*=**************************************************************************=*/

/* END OF FILE: Object_Inventory.h */
