//
//	File:	PBDriverInterface.h
//
//	Public constants and structures for communicating with the .PhoneBridge driver
//
//	Written by:	Tim Dierks
//	May 08, 1994
//

#ifndef __PBDRIVERINTERFACE__
#define __PBDRIVERINTERFACE__

#define	kProtocolVersion	1
#define	kSoftwareVersion	1

// temporary audio matrix stuff
enum {
	MacSndToPBSpeakerPTMicEnabled = 16,
	EarphoneToMacSoundIO,
	PhoneLineToMacSoundIO,					// hook switch must be done in separate command
	PhoneLineToMacSoundIOwPBSpeaker,		// hook switch must be done in separate command
	EarphoneToPhoneLineMacToPBSpeaker,		// hook switch must be done in separate command
	EarphoneToPhoneLineMacSndToPhoneLine,	// hook switch must be done in separate command
	OnHookMonitoringEnableCallerID,			// use when decoding Caller ID data (w/amp boost)
	OnHookMonitoringEnableCallerIDNoGain	// use when decoding Caller ID data (no gain)
};

// For format for csParam control call data, see PhoneBridgeControlParam structure
enum {
	pbControlGetDriverProtocolVersion = 1000,	// #
	pbControlGetSoftwareVersion,				// #
	pbControlCountLines,						// #
	pbControlGoOnHook,
	pbControlGoOffHook,
	pbControlDialPulseDigits,
	pbControlEnableDTMFDetect,
	pbControlDisableDTMFDetect,
	pbControlFlashSwitchhook,
	pbControlSetFlashTiming,					// *
	pbControlSetPauseTiming,					// *
	pbControlEnableMessageLight,
	pbControlDisableMessageLight,
	pbControlSetMessageLight,					// *
	pbControlEnableCPCDetect,
	pbControlDisableCPCDetect,
	pbControlSetCPCDetect,						// *
	pbControlEnableHookDetect,
	pbControlDisableHookDetect,
	pbControlSetHookDetect,						// *
	pbControlEnableVoiceExpander,
	pbControlDisableVoiceExpander,
	pbControlSetVoiceExpander,					// *
	pbControlEnableFeedbackSuppress,
	pbControlDisableFeedbackSuppress,
	pbControlSetFeedbackSuppress,				// *
	pbControlEnableDistinctiveRingDecode,
	pbControlDisableDistinctiveRingDecode,
	pbControlSetDistinctiveRingDecode,			// *
	pbControlEnableCallerIDDecode,
	pbControlDisableCallerIDDecode,
	pbControlSetCallerIDDecode,					// *
	pbControlGetStatus,							// #
	pbControlRegisterCallback,					// #
	pbControlUnregisterCallback,				// #
	pbControlDialMultipleDigits,
	pbControlEnableCallerID,
	pbControlSetAudioMatrix
};

// software based Control codes (DTMF dialing, caller ID, etc.)
enum
{
	pbControlDialToneDigit	=	2000
};

enum
{	messageHandsetStatus,
	messageDTMFDetected,
	messageRingDetected,
	messageLoopCurrentTransition,
	messageCallProgressState
};

enum
{	progressNoDialtone,
	progressDialtone,
	progressDialing,
	progressRinging,
	progressConnected,
	progressUglyTone,		// ???
	progressBusy,
	progressSilence,
	progressCNGTone
};

typedef pascal OSErr (*PhoneBridgeCallBackProc)(short msg, short mType, short value,
					long otherInfo, short lineNo, long reserved);

// This represents the structure of the csParam data for control calls
typedef struct
{	short	line;								// What line number to affect, for future expansion (first line = 1)
	union
	{	void					*retValPtr;		// Pointer to location to return result (for pbControlGetDriverProtocolVersion, pbControlGetSoftwareVersion, pbControlCountLines, pbControlGetStatus)
		PhoneBridgeCallBackProc	callback;		// Callback pointer (for pbControlRegisterCallback and pbControlUnregisterCallback)
		short					value;			// Value (for ...Set... functions which take a short or Boolean)
		unsigned char			buffer[8];		// for digit passing
		Ptr		callerIDBuffer;		// for digit passing
	} u;
	long	callbackRefcon;						// Value to be passed back to callback procedure (for pbControlRegisterCallback).
} PhoneBridgeControlParam;

typedef struct
{	Boolean		distinctiveRing;
	Boolean		callerID;
	Boolean		deviceOffHook;
	Boolean		loopCurrentPresent;
	Boolean 	localPhoneOffHook;
} PhoneBridgeStatus;

#define errDeviceNotFound			-13602	// Returned by Open when it couldn't find the PhoneBridge
#define errDriverNotReady			-13603	// Returned when the driver can't be used
											//  (for example, it couldn't allocate its RAM)

#endif // __PBDRIVERINTERFACE__

/****************************
 * 
 * Here's three sample calls
 
// take the phone on or off hook
void Hook(short hookValue)
{	OSErr						err;
	PhoneBridgeControlParam		pbcp;
	
	pbcp.line = 1;
	
	if (hookValue)
		err = Control(gRefNum,pbControlGoOffHook,&pbcp);
	else
		err = Control(gRefNum,pbControlGoOnHook,&pbcp);
}

// Set Flash timing
void SetFlashTiming(short flashTime)
{	OSErr						err;
	PhoneBridgeControlParam		pbcp;
	
	pbcp.line = 1;

	pbcp.u.value = flashTime;
	
	err = Control(gRefNum,pbControlSetFlashTiming,&pbcp);
}

// Flash
void Flash(short flashTime)
{	OSErr						err;
	PhoneBridgeControlParam		pbcp;
	
	pbcp.line = 1;
	
	err = Control(gRefNum,pbControlFlashSwitchhook,&pbcp);
}

 ****************************/