/* START OF FILE: Play_Movie_Fast_Benchmark.c */


/*=**************************************************************************=*/
/* [---BEGIN---] FILE HEADER COMMENTS */
/**
 ** SCREEN VIEW = [Adobe Syntax Black-10, Tabs = 4]
 ** PRINTER VIEW = [Adobe Syntax Black-10, Tabs = 4]
 ** =======================================================================
 ** Written by James Merkle
 ** Play_Movie_Fast_Benchmark.c - Video Routines to support Poor Man's Video Phone
 ** (C) Copyright 1994 James Merkle
 ** All rights reserved, except those wishing to "copy" moderate sections of code or files may do so, so long as
 ** such code is never used directly either in animation-related tools or multimedia tools that are  
 ** used in live performance or live broadcast or recorded mass market broadcast or film. Desktop usage not restricted.
 ** Conventional desktop game or entertainment not restricted.
 ** =======================================================================
 **
 **
 ** ============
 ** Revision History
 ** ============
 **
 **  1995.04.14		(James Merkle)
 **						Initial coding started from scratch
 **
 **
 **
 ** =================
 ** OTHER INFORMATION
 ** =================
 ** 
 ** Note: there are two sets of source code in this section. Old and New.
 **
 **
 **/
/* [----END----] FILE HEADER COMMENTS */
/*=**************************************************************************=*/


/*=**************************************************************************=*/
/* [---BEGIN---] #include AREA */

#include <LowMem.h>

#include "Dreaming_Audio.h"
#include "Dreaming_Stuff.h"
#include "String_Utilities.h"
#include "Fast_Bit_Copy.h"
#include "Error_Utilities.h"
#include "Interface_Utilities.h"
#include "Spin_O_Rama.h"
#include "Physical_File_Mapping.h"
#include "Vertical_Blanking_Interrupt.h"
#include "OS_Utilities.h"
#include "Shell_Dialogs.h"
#include "Spin_O_Rama_Files.h"
#include "Spin_O_Rama.h"
#include "Spin_O_Rama_Cache.h"
#include "Window_Environment.h"

#include "Play_Movie_Fast_Benchmark.h"

/* [----END----] #include AREA */
/*=**************************************************************************=*/


/*=**************************************************************************=*/
/* [---BEGIN---] #define AREA */

/*  */
/* The more frames the faster the video will play back. */
/* if the system is not asyncronous it should be set to "1". */
/* If it is set to one, but it is asyncronous, the ioResult fields will */
/* be looked at and no speacial magic hack is used, otherwise a */
/* faster technique that keeps track of buffer fill progress is used. */
/* If it needs more RAM, increase "k_MINIMUM_EXTRA_APP_RAM_REMAINING" */
/*  */
#define k_MULTIFRAME_BUFFER_COUNT			(2)

/*  */
/* This is arbitrary stack based list of time delay profile for */
/* studying frame time delays */
/*  */
#define k_MAX_FRAMES_TO_TIME_STUDY			(40)

/*  */
/* This is only set to 1 if debugging the data throughput. Leave "0" */
/*  */
#define k_DEBUG_OPTION_NEVER_UPDATE_VIDEO			0

/*  */
/* This is only set to 1 if comparing my copybits vs. Apples. Leave "0" */
/*  */
#define k_DEBUG_OPTION_ONLY_USE_COPY_BITS_ONE_FRAME	0

/*  */
/* This is only set to 1 if transferring from device manager */
/* as test instead of file manager. Leave "0" */
/*  */
#define k_DEBUG_OPTION_AVOID_FILE_MANAGER_TEST		0


#define COLOR_PORT_PORTBITS_CHECK 		0xC000

/*  */
/* To lock down pages to prevent virtual thrash, and allow faster I/O */
/* make transfer page aligned (perhaps contiguous if possible) */
/* Mac Quadra 840AV uses 8K pages, not 4K as many */
/* other computers do. */
/*  */
#define k_PAGE_ALIGN_SIZE	(8192UL)
#define k_PAGE_ALIGN_MASK	(0xFFFFE000)

/*  */
/* There is a 1 in 2^64 chance (very tiny) that we may accidentally */
/* read one frame ahead before frame is finished loading */
/* if multiple frames are being read at one time. */
/*  */
#define k_NOT_READY_LOADING_FIRST_KEY			0x9876DEAD
#define k_NOT_READY_LOADING_SECOND_KEY		0xFADFACE9

/*  */
/* This must be "1" for async to work correctly */
/*  */
#define k_MAC_OS_ASYNC_BUSY_FLAG				1


/* [----END----] #define AREA */
/*=**************************************************************************=*/


/*=**************************************************************************=*/
/* [---BEGIN---] TYPE AREA */


/* [----END----] TYPE AREA */
/*=**************************************************************************=*/


/*=**************************************************************************=*/
/* [---BEGIN---] PROTOTYPE AREA */

void Waste_Cycles_Without_Touching_RAM(void);


/* [----END----] PROTOTYPE AREA */
/*=**************************************************************************=*/


/*=**************************************************************************=*/
/* [---BEGIN---] GLOBAL DEFINITION AREA */


/* [----END----] GLOBAL DEFINITION AREA */
/*=**************************************************************************=*/


/*=**************************************************************************=*/
/* [---BEGIN---] LOCAL STATIC DEFINITION AREA */

static CARDINAL_8	*unit_Name = "\pPlay_Movie_Fast_Benchmark";


/* [----END----] LOCAL STATIC DEFINITION AREA */
/*=**************************************************************************=*/


/*=**************************************************************************=*/ 
void Waste_Cycles_Without_Touching_RAM(void)
{
/*  */
/* Typically, a 640* 480 frame or 16 bit video takes 25 of these */
/* cpu loops to delay before data is ready (assuming no drawing) */
/* On 840AV while two cards provide 18.3 MB per second average. */
/* Therefore to not fall behind we need to draw to the screen at a */
/* rate of (30 * 200 * 25) = 150000 nops to transfer 38,400 cachelines of 16 */
/* bytes apiece. Regrettably, it can do it fast but I wish we could keep up */
/* with 18.3 megabyte per second hard disk (usually 14.9 MB). */
/* This routine is only so that the motherboard RAM is not being hit */
/* while in a deadloop waiting for hard disk. But it NEVER IS USED */
/* with the current Copybits strategy because it always falls behind drives! */
/*  */
	asm {
		move.l	d0, -(SP)
		move.l	#200, d0
@Loop_Start_For_CPU_Delay:
		nop
		nop
		nop
		nop
		nop
		nop
		nop
		nop
		nop
		nop
		nop
		nop
		nop
		nop
		nop
		nop
		nop
		nop
		nop
		nop
		nop
		nop
		nop
		nop
		nop
		nop
		nop
		nop
		nop
		dbra		d0, @Loop_Start_For_CPU_Delay
		move.l	(SP)+,d0
	}
	
} /* void Waste_Cycles_Without_Touching_RAM(void) */
/*=**************************************************************************=*/


/*=**************************************************************************=*/  
Safe_Boolean	Benchmark_Spinning_Window_Completely(Boolean allow_VLB_Delays, Boolean show_Statistics)
	/*  */
	/* Another gigantic complex looking performance oriented routine. */
	/* Linear though. Pascal allows encapsulating routines and scope, and */
	/* so do other real languages, so blame the crappy "C" compiler not me. */
	/* Plays video from hard disk. Only uses QuickestTime files. */
	/* Draws whole frame, not just deviance, but could be alterred to honor */
	/* partial frames easily. */
	/* NOTICE! This routine is shared by benchmark code as well as code */
	/* to blast video through to screen in loop, once through, or pause/unpause. */
	/* It is much faster than QuickTime 2.0 on Quadra840AV with fast */
	/* disk drives. It is in fact 8 or 9 times faster and provides 8 to 9 times */
	/* as many frames per second over slow QuickTime 2.0 for highest */
	/* quality TOTALLY LOSSLESS VIDEO (non-compressed even). */
	/*  */
{
	CARDINAL_32	data_Start_Offset;
	CARDINAL_32	last_Index;
	CARDINAL_32	start_Time;
	CARDINAL_32	end_Speed;
	CARDINAL_32	preferred_Size;
	CARDINAL_32	desired_Size;
	CARDINAL_32	start_Frame;
	CARDINAL_32	current_Frame_Load_Point;
	CARDINAL_32	offset_Offscreen_Pixel_X;
	CARDINAL_32	offset_Offscreen_Pixel_Y;
	CARDINAL_32	picture_Row_Byte_Offset;
	CARDINAL_32	buffer_To_Get_Data_From;
	CARDINAL_32	source_Buffer_Raw_Data;
	CARDINAL_32	fetched_Data_Sentinel;
	CARDINAL_32	storage_Buffer_1_Frame_Start;
	CARDINAL_32	storage_Buffer_1_Frames_Fetched;
	CARDINAL_32	storage_Buffer_2_Frame_Start;
	CARDINAL_32	storage_Buffer_2_Frames_Fetched;
	CARDINAL_32	storage_Buffer_1_Data_Start_Offset;
	CARDINAL_32	storage_Buffer_2_Data_Start_Offset;
	CARDINAL_32	storage_Buffer_1_Next_Frame_Index;
	CARDINAL_32	storage_Buffer_2_Next_Frame_Index;
	CARDINAL_32	storage_Buffer_1_Next_Frame_Offset;
	CARDINAL_32	storage_Buffer_2_Next_Frame_Offset;
	CARDINAL_32	just_Starting_Playback_Phase;
	CARDINAL_32	next_Size;
	CARDINAL_32	total_Frames_To_Load;
	CARDINAL_32	max_buffer_Size;
	CARDINAL_32	start_Load_Point;
	CARDINAL_32	page_Aligned_Start_Address;
	CARDINAL_32	animation_Total_Frames;
	CARDINAL_32	dead_Cycle_Loops_Waiting_For_Data;
	CARDINAL_32	start_Tick_Time_Of_Read;
	CARDINAL_32	total_Tick_Time_Of_Issued_Read;
	CARDINAL_32	longest_Tick_Time_Of_Issued_Read;
	CARDINAL_32	list_Of_Dead_Cycle_Loops_Waiting_For_Data[k_MAX_FRAMES_TO_TIME_STUDY];
	CARDINAL_32	average_Of_Dead_Cycle_Loops;
	CARDINAL_32	accepted_Frames_Used_In_Average;
	CARDINAL_32	longest_Observed_Delay;
	INTEGER_32		transfer_Count;
	INTEGER_32		transfer_Count_Copy;
	INTEGER_32		loop;
	Rect				destination_Rect;
	Rect				source_Rect;
	Rect				global_Detination_Rect;
	Rect				active_Area_Rect;
	Ptr							storage_Buffer_1_Ptr;
	Ptr							storage_Buffer_2_Ptr;
	Ptr							physical_Address;
	IOParam					params_1;
	IOParam					params_2;
	Map_Info_Rec			file_Map;
	DIGITS_SPEED_REAL		delay_Loops_Per_Frame;
	DIGITS_SPEED_REAL		frames_Per_Second;
	compression_Data_Header_Ptr		compression_Info;

	Safe_Boolean		asynch_Transfer_Possibly_Works;
	Safe_Boolean		function_Result;
	Safe_Boolean		unlock_Frozen_Memory_Needed_At_Quit;
	Safe_Boolean		spin_Handle_Info_Locked_Down;
	Safe_Boolean		reallocate_Spin_Cache_When_Finished;
	SignedByte		status_Globals_State;
	Spin_Pict_Info_Ptr	spin_Info_Ptr;
	SignedByte			spin_Info_Handle_State;
	BitMap			*destination_Bitmap;
	WindowPtr		current_Graphics_Window_Ptr;
	Spin_Object_Handle		spin_Object_Handle;
	Boolean					found_Location;
	const BitMap 	*source_Bitmap;	
	INTEGER_16		index;
	INTEGER_16		position;
	Str255				note_String;
	OSErr				os_Error;
	Boolean			ports_Swapped;
	Boolean			devices_Swapped;
	Boolean			continue_Looping;
	Str255				timing_String;
	Str63				value_String;
	Boolean			benchmark_Multiple_Frames;
	Boolean			use_First_Buffer;
	GrafPtr			saved_Port;
	GDHandle			saved_Device;
	CARDINAL_8		*code_Name;
	Fast_Copy_Bits_Rec		copy_Options_Rec;
	Safe_Boolean				use_Fast_Screen_Drawing;
	Handle					video_Scratch_Buffer_1_Hdl;
	Memory_Allocation		video_Scratch_Buffer_1_Hdl_Memory_Initialized;
	Handle					video_Scratch_Buffer_2_Hdl;
	Memory_Allocation		video_Scratch_Buffer_2_Hdl_Memory_Initialized;
	Boolean					buffers_Have_One_Frame_Ready;
/* END VAR */


/* HACK HACK HACK HACK */
/* HACK HACK HACK HACK */
/* HACK HACK HACK HACK */
/* HACK HACK HACK HACK */
/* HACK HACK HACK HACK */



#if !defined(k_FORCE_VBL_METERING)
#error "UNDEFINED PARAMETER!"
#endif
#if (k_FORCE_VBL_METERING == 1)
	allow_VLB_Delays = TRUE;
#endif


	
#if !defined(k_DEBUG_OPTION_ONLY_USE_COPY_BITS_ONE_FRAME)
#error "UNDEFINED PARAMETER!"
#endif
#if (k_DEBUG_OPTION_ONLY_USE_COPY_BITS_ONE_FRAME == 0)
	/*  */
	/* If false, it merely ping pongs between graphics buffers, otherwise it */
	/* will ping pong between two large multiframe buffers. */
	/*  */
	benchmark_Multiple_Frames = TRUE;
#else
	benchmark_Multiple_Frames = FALSE;
#endif
	

	/*  */
	/* Note : For PowerPC, just read one frame at a time and use copybits. */
	/*  */
	if (ns_Shell_Globals.has_Real_68K_Available != TRUE) {
		benchmark_Multiple_Frames = FALSE;
	}



	ports_Swapped = FALSE;
	devices_Swapped = FALSE;
	animation_Total_Frames = 0;

	video_Scratch_Buffer_1_Hdl_Memory_Initialized = MEMORY_NOT_INITIALIZED;
	video_Scratch_Buffer_2_Hdl_Memory_Initialized = MEMORY_NOT_INITIALIZED;
	unlock_Frozen_Memory_Needed_At_Quit = SAFE_FALSE;
	asynch_Transfer_Possibly_Works = SAFE_FALSE;
	spin_Handle_Info_Locked_Down = SAFE_FALSE;
	reallocate_Spin_Cache_When_Finished = SAFE_FALSE;

	HideCursor();

	GetPort(&saved_Port);
	saved_Device = GetGDevice();
	
	function_Result = SAFE_FALSE;
	code_Name = "\pRedraw_Spinning_Window_Completely";

	if ((active_Window_List[MAIN_SPINNING_WINDOW_INDEX] == nil)
		|| ((**active_Window_List[MAIN_SPINNING_WINDOW_INDEX]).structure_Initialized != SAFE_TRUE)) {
		
		SysBeep(20); 
		Assign_String(ns_Shell_Globals.error_Description_String, "\pError. Improperly initialized window structure");
		Assign_String(ns_Shell_Globals.error_Note_String, "\pProgrammer flaw");
		Display_Error_Message(unit_Name, code_Name);
		Setup_Error_Handling();
		goto Error_Handler;
	}
	else {
	
		/*  */
		/* Lock down because structure content addresses are being passed */
		/*  */
		if (Begin_Use_Of_Handle((Handle) active_Window_List[MAIN_SPINNING_WINDOW_INDEX]
									, MOVE_HIGH_REQUIRED
									, &status_Globals_State) != SAFE_TRUE) {
									
			goto Error_Handler;
		}

		current_Graphics_Window_Ptr	= (**active_Window_List[MAIN_SPINNING_WINDOW_INDEX]).status_Window;
		destination_Bitmap = &(current_Graphics_Window_Ptr->portBits);
	}
	
	Set_View_Port(current_Graphics_Window_Ptr);
	ports_Swapped = TRUE;


	found_Location = FALSE;
	
	if (g_Dreaming_Module_Globals_Initialized == SAFE_TRUE) {
		if (dream.spin_Objects.total_Spin_Objects > 0) {
			found_Location = FALSE;
			for (index = 0; ((index < k_MAX_ACTIVE_SPIN_OBJECTS) && (found_Location != TRUE)); index++) {
				if (dream.spin_Objects.spin_Object_Memory_In_Use[index] == MEMORY_INITIALIZED) {
					if ((*dream.spin_Objects.spin_Object[index])->isolated_Port_Drawing_Active == SAFE_TRUE) {
						found_Location = TRUE;
						position = index;					
					}				
				}
			}
		}
	}
	
	spin_Object_Handle = dream.spin_Objects.spin_Object[position];
	
	if ((*spin_Object_Handle)->isolated_Port_Drawing_Active != SAFE_TRUE) {
		Assign_String(&note_String[0], "\pError, the arbitrarily chosen first SpinFile in memory is not meant to be used for direct Spin Window Useage");
		goto Error_Handler_With_Message;
	}
	
	if (Begin_Use_Of_Handle((Handle)(*spin_Object_Handle)->spin_Info
								, MOVE_HIGH_NOT_REQUIRED
								, &spin_Info_Handle_State) != SAFE_TRUE) {
								
		goto Error_Handler;
	}
	spin_Handle_Info_Locked_Down = SAFE_TRUE;
	spin_Info_Ptr = *(*spin_Object_Handle)->spin_Info;

	if (found_Location != TRUE) {
		goto Out_Of_Bounds;
	}

	if (benchmark_Multiple_Frames == TRUE) {
	
		if (g_Spin_Ring_Buffer_Initialized == SAFE_TRUE) {
			reallocate_Spin_Cache_When_Finished = SAFE_TRUE;
			/*  */	
			/* Closing down cache is a temporary and reversable manuever */
			/*  */	
			if (Close_Down_Ring_Cache_If_Possible(g_Spin_Cache_Master_Ptr) != SAFE_TRUE) {
				/* Ignore, not critical nor expected */
			}
		}

		preferred_Size =  (spin_Info_Ptr)->cell_Info[0].data_Total_Size;
		preferred_Size = ((preferred_Size + k_PAGE_ALIGN_SIZE) * k_MULTIFRAME_BUFFER_COUNT);
		preferred_Size += (k_PAGE_ALIGN_SIZE + k_PAGE_ALIGN_SIZE);
		
		/*  */
		/* if file might contain compressed images then add at least */
		/* double the buffer! No telling what sizes are really needed! */
		/*  */
		if ((spin_Info_Ptr)->cell_Info[0].compression_Type != k_NO_COMPRESSION) {
			preferred_Size = preferred_Size + preferred_Size;
		}
		
		video_Scratch_Buffer_1_Hdl_Memory_Initialized = MEMORY_NOT_INITIALIZED;
		video_Scratch_Buffer_1_Hdl = nil;
		desired_Size = preferred_Size;
		
		if (Create_Handle(&desired_Size
							, (CARDINAL_32)(10240L)
							, ALLOW_PURGING
							, DISALLOW_PARTIAL_AMOUNT
							, (Handle *)&video_Scratch_Buffer_1_Hdl
							, &video_Scratch_Buffer_1_Hdl_Memory_Initialized) != SAFE_TRUE) {
							
			Display_Low_Memory_Error_Message(unit_Name, code_Name, preferred_Size);
			function_Result = SAFE_FALSE;
			goto Normal_Function_Exit;
		}
	
		video_Scratch_Buffer_2_Hdl_Memory_Initialized = MEMORY_NOT_INITIALIZED;
		video_Scratch_Buffer_2_Hdl = nil;
		desired_Size = preferred_Size;
		
		if (Create_Handle(&desired_Size
							, (CARDINAL_32)(10240L)
							, ALLOW_PURGING
							, DISALLOW_PARTIAL_AMOUNT
							, (Handle *)&video_Scratch_Buffer_2_Hdl
							, &video_Scratch_Buffer_2_Hdl_Memory_Initialized) != SAFE_TRUE) {
							
			Display_Low_Memory_Error_Message(unit_Name, code_Name, preferred_Size);
			function_Result = SAFE_FALSE;
			goto Normal_Function_Exit;
		}
			
		MoveHHi(video_Scratch_Buffer_1_Hdl);
		if (MemError() != noErr) {
			/*  */
			/* In this simple version, Ignore this serious error. */
			/*  */
			goto Error_Handler;
		}
	
		HLock(video_Scratch_Buffer_1_Hdl);
		if (MemError() != noErr) {
			/*  */
			/* In this simple version, Ignore this serious error. */
			/*  */
			goto Error_Handler;
		}
		
		MoveHHi(video_Scratch_Buffer_2_Hdl);
		if (MemError() != noErr) {
			/*  */
			/* In this simple version, Ignore this serious error. */
			/*  */
			goto Error_Handler;
		}
	
		HLock(video_Scratch_Buffer_2_Hdl);
		if (MemError() != noErr) {
			/*  */
			/* In this simple version, Ignore this serious error. */
			/*  */
			goto Error_Handler;
		}
				
	} /* if (benchmark_Multiple_Frames == TRUE)  */
	
	if (g_Last_Image.valid_Information == SAFE_TRUE) {
		if (g_Last_Image.last_Spinfile_Index != position) {
			g_Last_Image.valid_Information = SAFE_FALSE;
		}
	}

	if ((*spin_Object_Handle)->file_Reference == 0) {
		os_Error = HOpen((*spin_Object_Handle)->spin_Object_Location.vRefNum
										, (*spin_Object_Handle)->spin_Object_Location.parID
										, &((*spin_Object_Handle)->spin_Object_Location.name[0])
										, fsRdPerm
										, &((*spin_Object_Handle)->file_Reference));
										
		if (os_Error != noErr) {
			Concat_Many(note_String, 3,"\pFail to open file. HOpen for file ", &((*spin_Object_Handle)->spin_Object_Location.name[0]), "\p ID = ");
			Concat_Long_To_String(note_String, os_Error, NO_LIMIT);
			goto Error_Handler_With_Message;
		}
		else {
		/* ===
			if (Recalculate_File_Volume_Blocks((*spin_Object_Handle)->file_Reference, &(g_Last_Image.file_Blocks)) == SAFE_TRUE) {
			
			}
		==== */
		}
	}

	for (index = 0; (index < k_MAX_FRAMES_TO_TIME_STUDY); index++) {
		list_Of_Dead_Cycle_Loops_Waiting_For_Data[index] = 0xFFFFFFFF;
	}

	Set_View_Port(current_Graphics_Window_Ptr);
	
	source_Rect =  (**((*spin_Object_Handle)->port_Info.spin_Object_Port_1.offscreen_Port.color_Port.portPixMap)).bounds;
	destination_Rect = source_Rect;

	global_Detination_Rect = destination_Rect;
	LocalToGlobal(&TOP_LEFT(global_Detination_Rect));
	LocalToGlobal(&BOTTOM_RIGHT(global_Detination_Rect));

	ForeColor(blackColor);
	BackColor(whiteColor);
		
	InvalRect(&destination_Rect);
	MoveTo(0,0);
	
	last_Index = (spin_Info_Ptr)->array_Actual_Count;
	
	
#if !defined(k_DEBUG_OPTION_AVOID_FILE_MANAGER_TEST)
#error "UNDEFINED PARAMETER!"
#endif
#if (k_DEBUG_OPTION_AVOID_FILE_MANAGER_TEST == 1)

	/*  */
	/* This is just a hack. It does not actually follow the file yet. */
	/* Point to arbitrary block. Get drive and volume ready */
	/*  */

	if (Map_Entire_File_Location(g_Last_Image.file_Reference, &file_Map) != SAFE_TRUE) {
		goto Error_Handler;
	}
	
	/*  */	
	/* Store driver reference number, and drive number */
	/*  */
	params_1.ioRefNum = file_Map.driver_Reference;		
	params_1.ioVRefNum = file_Map.drive_Number;
	params_2.ioRefNum = file_Map.driver_Reference;		
	params_2.ioVRefNum = file_Map.drive_Number;

#else
	params_1.ioRefNum = (*spin_Object_Handle)->file_Reference;
	params_2.ioRefNum = (*spin_Object_Handle)->file_Reference;
#endif

	params_1.ioCompletion = nil;
	params_1.ioPosMode = fsFromStart;
	params_1.ioPosMode = (params_1.ioPosMode | k_FORCE_NO_CACHE_BIT);
	params_2.ioCompletion = nil;
	params_2.ioPosMode = fsFromStart;
	params_2.ioPosMode = (params_2.ioPosMode | k_FORCE_NO_CACHE_BIT);
	
	source_Buffer_Raw_Data = (CARDINAL_32)*(((*spin_Object_Handle)->port_Info.spin_Object_Port_1).image_Handle);
	
	use_Fast_Screen_Drawing = SAFE_FALSE;
	
		
	if ((benchmark_Multiple_Frames != TRUE)
		&& ((spin_Info_Ptr)->cell_Info[0].compression_Type == k_NO_COMPRESSION) 
		&& (ns_Shell_Globals.has_Real_68K_Available != TRUE)
		){

		/*  */
		/* Temp Hack hack hack :
		/* We should avoid using 68K data transfer and use Apples PowerPC *
		/* CopyBits until my routiones are PowerPC assembly, or Native C */ 
		/*  */
		; /* NULL */
	}
	else {
		if (g_Preferred_Monitor_Values_Valid == SAFE_TRUE) {
		
			/*  */
			/* Build cached structure for video drawing. */
			/* The first 3 values will be force updated each animation frame */
			/* So any representative frame is OK */
			/*  */
			
			if ((g_Set_Spin_Window_Max_Monitor_Size == SAFE_TRUE) && (benchmark_Multiple_Frames != TRUE)){
				/*  */
				/* If benchmark_Multiple_Frames is true, we must not use this */
				/* redirection method */
				/*  */
				offset_Offscreen_Pixel_X = (*spin_Object_Handle)->port_Info.offset_Offscreen_Pixel_X;
				offset_Offscreen_Pixel_Y = (*spin_Object_Handle)->port_Info.offset_Offscreen_Pixel_Y;
				picture_Row_Byte_Offset = ((**(((*spin_Object_Handle)->port_Info.spin_Object_Port_1).offscreen_Port.color_Port.portPixMap)).rowBytes & (~COLOR_PORT_PORTBITS_CHECK));
			}
			else {
				offset_Offscreen_Pixel_X = 0;
				offset_Offscreen_Pixel_Y = 0;
				picture_Row_Byte_Offset = (spin_Info_Ptr)->cell_Info[0].picture_Row_Byte_Offset;
			}
			
			if (g_Set_Spin_Window_Max_Monitor_Size == SAFE_TRUE) {
			/* === 
				backdrop_Size_Rect = (**(((*spin_Object_Handle)->port_Info.spin_Object_Port_1).color_Port.portPixMap)).bounds;
				stored_Bit_Depth = (**(((*spin_Object_Handle)->port_Info.spin_Object_Port_1).color_Port.portPixMap)).pixelSize;
			==== */
			
				active_Area_Rect = (spin_Info_Ptr)->max_Bounds;
				OffsetRect(&active_Area_Rect, global_Detination_Rect.left, global_Detination_Rect.top);
				OffsetRect(&active_Area_Rect, (*spin_Object_Handle)->port_Info.offset_Offscreen_Pixel_X, (*spin_Object_Handle)->port_Info.offset_Offscreen_Pixel_Y);
	
				if (Build_Draw_Bits_Raw_Parameters((CARDINAL_32)*(((*spin_Object_Handle)->port_Info.spin_Object_Port_1).image_Handle)
						, picture_Row_Byte_Offset
						, (spin_Info_Ptr)->cell_Info[0].compression_Type
						, offset_Offscreen_Pixel_X
						, offset_Offscreen_Pixel_Y
						, g_Preferred_Monitor_Video_Pix_Map_Handle
						, &active_Area_Rect
						, &g_Preferred_Monitor_Max_Rect
						, g_Preferred_Monitor_Depth
						, &copy_Options_Rec) == SAFE_TRUE) {
					use_Fast_Screen_Drawing = SAFE_TRUE;
				}
			}
			else {
				if (Build_Draw_Bits_Raw_Parameters((CARDINAL_32)*(((*spin_Object_Handle)->port_Info.spin_Object_Port_1).image_Handle)
						, picture_Row_Byte_Offset
						, (spin_Info_Ptr)->cell_Info[0].compression_Type
						, offset_Offscreen_Pixel_X
						, offset_Offscreen_Pixel_Y
						, g_Preferred_Monitor_Video_Pix_Map_Handle
						, &global_Detination_Rect
						, &g_Preferred_Monitor_Max_Rect
						, g_Preferred_Monitor_Depth
						, &copy_Options_Rec) == SAFE_TRUE) {
					use_Fast_Screen_Drawing = SAFE_TRUE;
				}
			}
			
			copy_Options_Rec.disable_Interrupts = 0;
		}
	}


		
	if (use_Fast_Screen_Drawing != SAFE_TRUE) {
		benchmark_Multiple_Frames = FALSE;
	}
	
	if (benchmark_Multiple_Frames == TRUE){
	
		/*  */
		/* NOTE! These handles need to be 128 bit aligned */
		/* or better. Do it on 8K page boundaries so that they */
		/* can be locked properly. Small slowdown otherwise on */
		/* Raided JackHammer using two boards? */
		/*  */
		storage_Buffer_1_Ptr = *video_Scratch_Buffer_1_Hdl;
		storage_Buffer_2_Ptr = *video_Scratch_Buffer_2_Hdl;
		
		max_buffer_Size = GetHandleSize(video_Scratch_Buffer_1_Hdl);
		/*  */
		/* HACK HACK HACK not optimal method.
		/* Either assume both buffers are same length or else just limit max */
		/* of buffer to minimum of both. Else rewrite to honor both. */
		/*  */
		if ((MemError() != noErr) || (max_buffer_Size == 0)) {
			goto Error_Handler;
		}
		
		page_Aligned_Start_Address = (CARDINAL_32)storage_Buffer_1_Ptr;
		page_Aligned_Start_Address +=  k_PAGE_ALIGN_SIZE;
		page_Aligned_Start_Address = (page_Aligned_Start_Address & k_PAGE_ALIGN_MASK);
		max_buffer_Size = max_buffer_Size - (page_Aligned_Start_Address - (CARDINAL_32)storage_Buffer_1_Ptr);
		storage_Buffer_1_Ptr = (Ptr)page_Aligned_Start_Address;
		max_buffer_Size = (max_buffer_Size & k_PAGE_ALIGN_MASK);


		page_Aligned_Start_Address = (CARDINAL_32)storage_Buffer_2_Ptr;
		page_Aligned_Start_Address +=  k_PAGE_ALIGN_SIZE;
		page_Aligned_Start_Address = (page_Aligned_Start_Address & k_PAGE_ALIGN_MASK);
		max_buffer_Size = max_buffer_Size - (page_Aligned_Start_Address - (CARDINAL_32)storage_Buffer_2_Ptr);
		storage_Buffer_2_Ptr = (Ptr)page_Aligned_Start_Address;
		max_buffer_Size = (max_buffer_Size & k_PAGE_ALIGN_MASK);
		
		if (show_Statistics != TRUE) {
			unlock_Frozen_Memory_Needed_At_Quit = SAFE_FALSE;
		}
		else {
			unlock_Frozen_Memory_Needed_At_Quit = SAFE_TRUE;
					
			/*  */
			/* Memory is page aligned. Now Lock them all down */
			/*  */
			if (Lock_And_Translate_Contiguous_Memory(storage_Buffer_1_Ptr
																	, max_buffer_Size
																	, &physical_Address) != SAFE_TRUE) {
				/*  */
				/* Ignore problem, Damned PowerPCs with 24 meg can only */
				/* lock 6 Meg at boot anyways. Migrate to PrepareMemoryForIO */
				/*  */
			}
			
			if (Lock_And_Translate_Contiguous_Memory(storage_Buffer_2_Ptr
																	, max_buffer_Size
																	, &physical_Address) != SAFE_TRUE) {
			}
		}
	}
	else {
		storage_Buffer_1_Ptr = *(((*spin_Object_Handle)->port_Info.spin_Object_Port_1).image_Handle);
		storage_Buffer_2_Ptr = *(((*spin_Object_Handle)->port_Info.spin_Object_Port_2).image_Handle);
		max_buffer_Size = GetHandleSize(((*spin_Object_Handle)->port_Info.spin_Object_Port_1).image_Handle);
		/*  */
		/* HACK HACK HACK not optimal method.
		/* Either assume both buffers are same length or else just limit max */
		/* of buffer to minimum of both. Else rewrite to honor both. */
		/*  */
		if ((MemError() != noErr) || (max_buffer_Size == 0)) {
			goto Error_Handler;
		}
	}

	
	if (use_First_Buffer == TRUE) {
		copy_Options_Rec.source_Base_Address = (CARDINAL_32)storage_Buffer_1_Ptr;
	}
	else {
		copy_Options_Rec.source_Base_Address = (CARDINAL_32)storage_Buffer_2_Ptr;
	}
		
	buffer_To_Get_Data_From = 0;
	
	storage_Buffer_1_Frame_Start = 0;
	storage_Buffer_1_Frames_Fetched = 0;
	storage_Buffer_2_Frame_Start = 0;
	storage_Buffer_2_Frames_Fetched = 0;
	
	current_Frame_Load_Point = 0;
	longest_Tick_Time_Of_Issued_Read = 0;

	start_Time = TickCount();
	continue_Looping = TRUE;
	animation_Total_Frames = 0;
	
	start_Frame = g_Last_Movie_Frame_Paused;
	if (start_Frame >= last_Index) {
		start_Frame = 0;
	}
		
	do {
		buffers_Have_One_Frame_Ready = FALSE;
		just_Starting_Playback_Phase = 2;
		use_First_Buffer = TRUE;
		
		current_Frame_Load_Point = start_Frame;
				
		for (loop = start_Frame; ((loop < last_Index) && (continue_Looping == TRUE)); loop++) {
	
			if (buffers_Have_One_Frame_Ready != TRUE) {
			
				do {
									
					data_Start_Offset = (spin_Info_Ptr)->cell_Info[current_Frame_Load_Point].data_Start_Offset;
					next_Size = (spin_Info_Ptr)->cell_Info[current_Frame_Load_Point].data_Total_Size;
					total_Frames_To_Load = 0;
					start_Load_Point = current_Frame_Load_Point;
					
					do {
						transfer_Count_Copy = next_Size;
						
						/*  */
						/* Lay down vital markers to allow us to determine if buffers are ready */
						/*  */
						if (use_First_Buffer == TRUE) {
							fetched_Data_Sentinel = (CARDINAL_32)storage_Buffer_1_Ptr + next_Size - sizeof(CARDINAL_32);
						}
						else {
							fetched_Data_Sentinel = (CARDINAL_32)storage_Buffer_2_Ptr + next_Size - sizeof(CARDINAL_32);
						}
						*(CARDINAL_32 *)fetched_Data_Sentinel = k_NOT_READY_LOADING_SECOND_KEY;
						fetched_Data_Sentinel -= sizeof(CARDINAL_32);
						*(CARDINAL_32 *)fetched_Data_Sentinel = k_NOT_READY_LOADING_FIRST_KEY;

						current_Frame_Load_Point += 1;
						total_Frames_To_Load += 1;
						next_Size = (((spin_Info_Ptr)->cell_Info[current_Frame_Load_Point].data_Start_Offset
										- data_Start_Offset) 
									+ (spin_Info_Ptr)->cell_Info[current_Frame_Load_Point].data_Total_Size);
					
					} while ((next_Size < max_buffer_Size) && (current_Frame_Load_Point < last_Index));
					
					/*  */
					/* Round up transfer to next page, but not if there are no */
					/* more frames after this one */
					/*  */
					if (current_Frame_Load_Point < last_Index) {
						transfer_Count_Copy +=  k_PAGE_ALIGN_SIZE;
						transfer_Count_Copy = (transfer_Count_Copy & k_PAGE_ALIGN_MASK);
						/*  */
						/* If roundup exceeds buffer, then forget it. Truncation*/
						/* will end on nice boundary though, at least 512 byte aligned. */
						/*  */
						if (transfer_Count_Copy > max_buffer_Size) {
							transfer_Count_Copy = max_buffer_Size;
						}
					}

					if (total_Frames_To_Load != 0) {
					
						start_Tick_Time_Of_Read = (CARDINAL_32)LMGetTicks();
						
						if (use_First_Buffer == TRUE) {
												
							params_1.ioCompletion = nil;
							params_1.ioResult = k_MAC_OS_ASYNC_BUSY_FLAG;
							params_1.ioBuffer = storage_Buffer_1_Ptr;

#if !defined(k_DEBUG_OPTION_AVOID_FILE_MANAGER_TEST)
#error "UNDEFINED PARAMETER!"
#endif
#if (k_DEBUG_OPTION_AVOID_FILE_MANAGER_TEST == 1)
							params_1.ioReqCount = transfer_Count_Copy;
							
							/*  */
							/* Round up blocks if programmed wrong. Buffers ahould */
							/* be able to take it, but this is wrong to do, especially if */
							/* buffers cannot take it. Should check first. Oh well. */
							/*  */
							if ((params_1.ioReqCount & 0x000001FF) != 0) {
								params_1.ioReqCount += 511;
								params_1.ioReqCount = (params_1.ioReqCount & 0xFFFFFE00);
							}
														
							 if (Calculate_Drive_Offset(&file_Map
							 						, data_Start_Offset
							 						, (CARDINAL_32 *)&params_1.ioPosOffset) != SAFE_TRUE) {
							 	goto Error_Handler;
							 }
#else
							params_1.ioReqCount = transfer_Count_Copy;
							params_1.ioPosOffset = data_Start_Offset;
#endif

							use_First_Buffer = FALSE;
							
							storage_Buffer_1_Frame_Start = start_Load_Point;
							
							/*  */
							/* first check if no need to read from buffer because */
							/* impossible but already caught up */
							/*  */
							if (storage_Buffer_1_Frames_Fetched != 0) {
								Display_Serious_Error_Message(unit_Name, code_Name, "\pBuffers ran away! Data not used as should", "\pBad program?");
								goto Error_Handler;
							}
							
							storage_Buffer_1_Frames_Fetched = total_Frames_To_Load;
							storage_Buffer_1_Data_Start_Offset = data_Start_Offset;
							storage_Buffer_1_Next_Frame_Index = 0;
							storage_Buffer_1_Next_Frame_Offset = data_Start_Offset;
	
							os_Error = PBRead((ParmBlkPtr)&params_1,	TRUE);
							if (os_Error != noErr) {
								Display_Serious_Error_Message(unit_Name, code_Name, "\pFSRead failed to read data", "\pBad file or drive?");
								goto Error_Handler;
							}
							else {
								if (params_1.ioResult == k_MAC_OS_ASYNC_BUSY_FLAG) {
									asynch_Transfer_Possibly_Works = SAFE_TRUE;
								}
							}
						} /* if (use_First_Buffer == TRUE) ... */
						else {		
				
							params_2.ioCompletion = nil;
							params_2.ioResult = k_MAC_OS_ASYNC_BUSY_FLAG;
							params_2.ioBuffer = storage_Buffer_2_Ptr;
							
#if !defined(k_DEBUG_OPTION_AVOID_FILE_MANAGER_TEST)
#error "UNDEFINED PARAMETER!"
#endif
#if (k_DEBUG_OPTION_AVOID_FILE_MANAGER_TEST == 1)

							params_2.ioReqCount = transfer_Count_Copy;
							
							if ((params_2.ioReqCount & 0x000001FF) != 0) {
								params_2.ioReqCount += 511;
								params_2.ioReqCount = (params_2.ioReqCount & 0xFFFFFE00);
							}
														
							 if (Calculate_Drive_Offset(&file_Map
							 						, data_Start_Offset
							 						, (CARDINAL_32 *)&params_2.ioPosOffset) != SAFE_TRUE) {
							 	goto Error_Handler;
							 }
#else
							params_2.ioReqCount = transfer_Count_Copy;
							params_2.ioPosOffset = data_Start_Offset;
#endif
							
							use_First_Buffer = TRUE;
							
							storage_Buffer_2_Frame_Start = start_Load_Point;
							
							if (storage_Buffer_2_Frames_Fetched != 0) {
								Display_Serious_Error_Message(unit_Name, code_Name, "\pBuffers ran away (2)! Data not used as should", "\pBad program?");
								goto Error_Handler;
							}
	
							storage_Buffer_2_Frames_Fetched = total_Frames_To_Load;
							storage_Buffer_2_Data_Start_Offset = data_Start_Offset;
							storage_Buffer_2_Next_Frame_Index = 0;
							storage_Buffer_2_Next_Frame_Offset = data_Start_Offset;
							
							os_Error = PBRead((ParmBlkPtr)&params_2,	TRUE);
							if (os_Error != noErr) {
								Display_Serious_Error_Message(unit_Name, code_Name, "\pFSRead failed to read data", "\pBad file or drive?");
								goto Error_Handler;
							}
							else {
								if (params_2.ioResult == k_MAC_OS_ASYNC_BUSY_FLAG) {
									asynch_Transfer_Possibly_Works = SAFE_TRUE;
								}
							}
						} /* NOT if (use_First_Buffer == TRUE) */
						
						if (just_Starting_Playback_Phase > 0) {
							/*  */
							/* The first time through we force buffer to fill. The second time */
							/* we exit while I/O is occuring */
							/*  */
							if (just_Starting_Playback_Phase == 2) {
								if (use_First_Buffer != TRUE) {
									if (params_1.ioResult == 1) {
										while((volatile OSErr)params_1.ioResult == k_MAC_OS_ASYNC_BUSY_FLAG) {
											; /* NULL */
										}
									}
								}
								else {
									if (params_2.ioResult == 1) {
										while((volatile OSErr)params_2.ioResult == k_MAC_OS_ASYNC_BUSY_FLAG) {
											; /* NULL */
										}
									}
								}
							}
							else {
								/*  */
								/* Assume just_Starting_Playback_Phase == 1 and that */
								/* a buffer is ready to read from and set a marker to point to */
								/* it. */
								/*  */
								if (use_First_Buffer != TRUE) {
									buffer_To_Get_Data_From = 1;
								}
								else {
									buffer_To_Get_Data_From = 0;
								}
							}
							
							just_Starting_Playback_Phase = just_Starting_Playback_Phase - 1;
							
						} /* if (just_Starting_Playback_Phase > 0) */
						
					} /* if (total_Frames_To_Load != 0) */
					else {
						/*  */
						/* No more frames to load. */
						/* Just use whats left in buffer, if anything. */
						/*  */
						if (buffer_To_Get_Data_From == 0) {
							if (storage_Buffer_1_Next_Frame_Index == 0) {
								Display_Serious_Error_Message(unit_Name, code_Name, "\pData underrun error, Frames ended exhauseted (1)", "\pbad programming?");
								goto Error_Handler;
							}
						}
						else {
							if (storage_Buffer_2_Next_Frame_Index == 0) {
								Display_Serious_Error_Message(unit_Name, code_Name, "\pData underrun error, Frames ended exhauseted (2)", "\pbad programming?");
								goto Error_Handler;
							}
						}
					}
						
				} while (just_Starting_Playback_Phase != 0);
								
				buffers_Have_One_Frame_Ready = TRUE;

			} /* if (buffers_Have_One_Frame_Ready != TRUE) */
			
			
			if (buffers_Have_One_Frame_Ready == TRUE) {
				/*  */
				/* Wait until buffer seems to have frame ready to feed from */
				/* eventually data will cross boundary asynchronously and it */
				/* will be ready to snatch */
				/*  */
				
				if (buffer_To_Get_Data_From == 0) {
				
					if (storage_Buffer_1_Next_Frame_Index < storage_Buffer_1_Frames_Fetched) {
						/*  */
						/* Wait for the oldest outstanding frame to complete reading */
						/* We might be way too fast again and may have consumed */
						/* all our buffers up */
						/*  */
						
						source_Buffer_Raw_Data = (CARDINAL_32)storage_Buffer_1_Ptr;
						source_Buffer_Raw_Data += (storage_Buffer_1_Next_Frame_Offset - storage_Buffer_1_Data_Start_Offset);
						
						if (use_Fast_Screen_Drawing != SAFE_TRUE) {
							source_Bitmap = (const BitMap *)*(((*spin_Object_Handle)->port_Info.spin_Object_Port_1).offscreen_Port.color_Port.portPixMap);
						}
						
						if ((storage_Buffer_1_Next_Frame_Index + 1) == storage_Buffer_1_Frames_Fetched) {
							/*  */
							/* if only one frame loaded per buffer, use proper Mac OS */
							/* flag and not async sneaky sentinel that assumes low to */
							/* high data read order */
							/*  */
							if (params_1.ioResult == 1) {
								while(params_1.ioResult == 1) {
									; /* NULL */
								}
							}
						}
						else {
							/*  */
							/* A different type of dead loop is needed. This one will */
							/* wait for markers to get written over by incomming */
							/* data stream. */
							/*  */
							
							dead_Cycle_Loops_Waiting_For_Data = 0;
							fetched_Data_Sentinel = (source_Buffer_Raw_Data 
										+ (spin_Info_Ptr)->cell_Info[storage_Buffer_1_Frame_Start + storage_Buffer_1_Next_Frame_Index].data_Total_Size);
																	
							fetched_Data_Sentinel = fetched_Data_Sentinel - sizeof(CARDINAL_32);
							while(*(volatile CARDINAL_32 *)fetched_Data_Sentinel == k_NOT_READY_LOADING_SECOND_KEY) {
								Waste_Cycles_Without_Touching_RAM();
								dead_Cycle_Loops_Waiting_For_Data += 1;
							}
							fetched_Data_Sentinel = fetched_Data_Sentinel - sizeof(CARDINAL_32);
							while(*(volatile CARDINAL_32 *)fetched_Data_Sentinel == k_NOT_READY_LOADING_FIRST_KEY) {
								Waste_Cycles_Without_Touching_RAM();
							}
							
							if (loop < k_MAX_FRAMES_TO_TIME_STUDY) {
								list_Of_Dead_Cycle_Loops_Waiting_For_Data[loop] = dead_Cycle_Loops_Waiting_For_Data;
							}
						}
						
						copy_Options_Rec.source_Row_Bytes = (spin_Info_Ptr)->cell_Info[(storage_Buffer_1_Frame_Start + storage_Buffer_1_Next_Frame_Index)].picture_Row_Byte_Offset;
						copy_Options_Rec.compression = (spin_Info_Ptr)->cell_Info[(storage_Buffer_1_Frame_Start + storage_Buffer_1_Next_Frame_Index)].compression_Type;
						
						if (storage_Buffer_1_Next_Frame_Index == 0) {
							/*  */
							/* This represents the tick time delay between last issued */
							/* request to read some data, and how long it tool to */
							/* allow the other buffer to be read from. This timing is */
							/* logical if sync or asyn system is operating. */
							/*  */
							total_Tick_Time_Of_Issued_Read = (CARDINAL_32)LMGetTicks();
							total_Tick_Time_Of_Issued_Read = total_Tick_Time_Of_Issued_Read - start_Tick_Time_Of_Read;
							if (total_Tick_Time_Of_Issued_Read > longest_Tick_Time_Of_Issued_Read) {
								longest_Tick_Time_Of_Issued_Read = total_Tick_Time_Of_Issued_Read;
							}
						}
						
						storage_Buffer_1_Next_Frame_Index += 1;
			
						if (storage_Buffer_1_Next_Frame_Index >= storage_Buffer_1_Frames_Fetched) {
							/*  */
							/* Last frame was just taken, point to other buffer and */
							/* mark flag to start another feed going (if it hasn't already started)*/
							/*  */
							buffers_Have_One_Frame_Ready = FALSE;
							buffer_To_Get_Data_From = 1;
							
							/*  */
							/* Mark structure as "consumed" totally */
							/*  */
							storage_Buffer_1_Frames_Fetched = 0;
						}
						else {
							storage_Buffer_1_Next_Frame_Offset = (spin_Info_Ptr)->cell_Info[(storage_Buffer_1_Frame_Start + storage_Buffer_1_Next_Frame_Index)].data_Start_Offset;
						}
					}
					else {
						/*  */
						/* OH NO!!!! DATA UNDERRUN!!!! NOT POSSIBLE */
						/*  */
						goto Error_Handler;
					}
				}
				else {
				
					if (storage_Buffer_2_Next_Frame_Index < storage_Buffer_2_Frames_Fetched) {
						/*  */
						/* Wait for the oldest outstanding frame to complete reading */
						/* We might be way too fast again and may have consumed */
						/* all our buffers up */
						/*  */
						
						source_Buffer_Raw_Data = (CARDINAL_32)storage_Buffer_2_Ptr;
						source_Buffer_Raw_Data += (storage_Buffer_2_Next_Frame_Offset - storage_Buffer_2_Data_Start_Offset);
						
						if (use_Fast_Screen_Drawing != SAFE_TRUE) {
							source_Bitmap = (const BitMap *)*(((*spin_Object_Handle)->port_Info.spin_Object_Port_2).offscreen_Port.color_Port.portPixMap);
						}

						if ((storage_Buffer_2_Next_Frame_Index + 1) == storage_Buffer_2_Frames_Fetched) {
							/*  */
							/* if only one frame loaded per buffer, use proper Mac OS */
							/* flag and not async sneaky sentinel that assumes low to */
							/* high data read order */
							/*  */
							if (params_2.ioResult == 1) {
								while(params_2.ioResult == 1) {
									; /* NULL */
								}
							}
						}
						else {
							/*  */
							/* A different type of dead loop is needed. This one will */
							/* wait for markers to get written over by incomming */
							/* data stream. */
							/*  */
							
							fetched_Data_Sentinel = (source_Buffer_Raw_Data 
										+ (spin_Info_Ptr)->cell_Info[storage_Buffer_2_Frame_Start + storage_Buffer_2_Next_Frame_Index].data_Total_Size);
							
							dead_Cycle_Loops_Waiting_For_Data = 0;
							
							fetched_Data_Sentinel = fetched_Data_Sentinel - sizeof(CARDINAL_32);
							while(*(volatile CARDINAL_32 *)fetched_Data_Sentinel == k_NOT_READY_LOADING_SECOND_KEY) {
								Waste_Cycles_Without_Touching_RAM();
								dead_Cycle_Loops_Waiting_For_Data += 1;
							}
							fetched_Data_Sentinel = fetched_Data_Sentinel - sizeof(CARDINAL_32);
							while(*(volatile CARDINAL_32 *)fetched_Data_Sentinel == k_NOT_READY_LOADING_FIRST_KEY) {
								Waste_Cycles_Without_Touching_RAM();
							}
							if (loop < k_MAX_FRAMES_TO_TIME_STUDY) {
								list_Of_Dead_Cycle_Loops_Waiting_For_Data[loop] = dead_Cycle_Loops_Waiting_For_Data;
							}
						}
						
						copy_Options_Rec.source_Row_Bytes = (spin_Info_Ptr)->cell_Info[(storage_Buffer_2_Frame_Start + storage_Buffer_2_Next_Frame_Index)].picture_Row_Byte_Offset;
						copy_Options_Rec.compression = (spin_Info_Ptr)->cell_Info[(storage_Buffer_2_Frame_Start + storage_Buffer_2_Next_Frame_Index)].compression_Type;
						
						if (storage_Buffer_2_Next_Frame_Index == 0) {
							/*  */
							/* This represents the tick time delay between last issued */
							/* request to read some data, and how long it tool to */
							/* allow the other buffer to be read from. This timing is */
							/* logical if sync or asyn system is operating. */
							/*  */
							total_Tick_Time_Of_Issued_Read = (CARDINAL_32)LMGetTicks();
							total_Tick_Time_Of_Issued_Read = total_Tick_Time_Of_Issued_Read - start_Tick_Time_Of_Read;
							if (total_Tick_Time_Of_Issued_Read > longest_Tick_Time_Of_Issued_Read) {
								longest_Tick_Time_Of_Issued_Read = total_Tick_Time_Of_Issued_Read;
							}
						}

						storage_Buffer_2_Next_Frame_Index += 1;
			
						if (storage_Buffer_2_Next_Frame_Index >= storage_Buffer_2_Frames_Fetched) {
							/*  */
							/* Last frame was just taken, point to other buffer and */
							/* mark flag to start another feed going (if it hasn't already started)*/
							/*  */
							buffers_Have_One_Frame_Ready = FALSE;
							buffer_To_Get_Data_From = 0;
							
							/*  */
							/* Mark structure as "consumed" totally */
							/*  */
							storage_Buffer_2_Frames_Fetched = 0;
						}
						else {
							storage_Buffer_2_Next_Frame_Offset = (spin_Info_Ptr)->cell_Info[(storage_Buffer_2_Frame_Start + storage_Buffer_2_Next_Frame_Index)].data_Start_Offset;
						}
					}
					else {
						/*  */
						/* OH NO!!!! DATA UNDERRUN!!!! NOT POSSIBLE */
						/*  */
						goto Error_Handler;
					}
				} /* NOT if (buffer_To_Get_Data_From == 0) */
				
				
				
#if !defined(k_DEBUG_OPTION_NEVER_UPDATE_VIDEO)
#error "UNDEFINED PARAMETER!"
#endif
#if (k_DEBUG_OPTION_NEVER_UPDATE_VIDEO == 0)

				/*  */
				/* Transfer to screen at VBL Time */
				/*  */
				if (allow_VLB_Delays == TRUE) {
					if (g_Preferred_Monitor_Values_Valid == SAFE_TRUE) {
						Wait_For_Next_Video_Monitor_VBL(g_Preferred_Monitor_Slots);
					}
				}
				
				if (use_Fast_Screen_Drawing == SAFE_TRUE) {
				
				/* ===
					OffsetRect(&source_Rect
						, position_In_Scene_Point->h
						, position_In_Scene_Point->v);
				==== */
					
					copy_Options_Rec.source_Base_Address = source_Buffer_Raw_Data;
					
					if (copy_Options_Rec.compression == k_MERKLE_TIME_COMPRESSION) {
					
						copy_Options_Rec.compression = k_BACKGROUND_FREE_COMPACT_DRAWING;
						compression_Info = (compression_Data_Header_Ptr)copy_Options_Rec.source_Base_Address;
			
						if (compression_Info->sentinel_1 != SAFE_TRUE) {
							goto Error_Handler;
						}

						copy_Options_Rec.row_Raw_Data_Ptr = (CARDINAL_32 *)((CARDINAL_32)compression_Info + compression_Info->row_Raw_Data_Start_Offset);
						copy_Options_Rec.skip_List_Data_Ptr = (CARDINAL_32 *)((CARDINAL_32)compression_Info + compression_Info->skip_List_Data_Start_Offset);
						copy_Options_Rec.skip_List_Master_List_Ptr = (CARDINAL_32 *)((CARDINAL_32)compression_Info + compression_Info->skip_List_Master_List_Start_Offset);
						copy_Options_Rec.scan_Line_Bounds_Ptr = (CARDINAL_32 *)((CARDINAL_32)compression_Info + compression_Info->scan_Line_Bounds_Offset); 
					}
					else {
						copy_Options_Rec.compression = k_NORMAL_RAW_BITMAP_DRAWING;
					}
					
					Draw_Bits_Raw_To_Monitor_Using_Params(&copy_Options_Rec);
									
				}
				else {
					CopyBits (source_Bitmap
							, destination_Bitmap
							, &source_Rect
							, &destination_Rect
							, srcCopy
							, nil);
				}
#endif

				
									
			} /* if (buffers_Have_One_Frame_Ready != TRUE) */
			else {
				/*  */
				/* Assume serious problem preventing display and skip the frame */
				/*  */
			}
					
			animation_Total_Frames += 1;
			
			if ((animation_Total_Frames & 0x00000003) == 0) {
			
				if (Button() != 0) {
					continue_Looping = FALSE;
				}
				else {
					if (Check_Keyboard_For_Key(k_KEYCODE_ASCII_S) == SAFE_TRUE) {
						if (Check_Keyboard_For_Key(CONTROL_KEYBOARD_CODE) == SAFE_TRUE) {
							if (Check_Keyboard_For_Key(SHIFT_OR_LEFT_SHIFT_KEYBOARD_CODE) == SAFE_TRUE) {
								continue_Looping = FALSE;
								g_Last_Movie_Frame_Paused = loop;
							}
						}
					}
				}
			} /* if ((animation_Total_Frames & 0x00000001) == 0) */
			
		} /* for (loop = 0; ((loop < last_Index) && (continue_Looping == TRUE)); loop++) */
	
		start_Frame = 0;
	} while ((g_Loop_Movie_Forever == SAFE_TRUE) && (continue_Looping == TRUE));
	
	if (continue_Looping == TRUE) {
		g_Last_Movie_Frame_Paused = 0;
	}
	

	
	end_Speed = ((CARDINAL_32)TickCount() - start_Time);
	
	frames_Per_Second = end_Speed / 60.0;
	frames_Per_Second = ((DIGITS_SPEED_REAL)animation_Total_Frames) / frames_Per_Second;

/* - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - */
/* - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - */
/* - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - */

	Convert_Double_To_String(frames_Per_Second
									, 7
									, 3
									, (ASCII_Character *)&timing_String[0]);
									
	if (asynch_Transfer_Possibly_Works == SAFE_TRUE) {
		Concat_After_Pascal_String(timing_String, "\p complete Frames per Sec (ASYNC). (Could be faster with direct to screen and no file manager)");
	}
	else {
		Concat_After_Pascal_String(timing_String, "\p complete Frames per Sec. (Could be faster with direct to screen and no file manager)");
	}
	
	average_Of_Dead_Cycle_Loops = 0;
	accepted_Frames_Used_In_Average = 0;
	longest_Observed_Delay = 0;
	for (index = 0; (index < k_MAX_FRAMES_TO_TIME_STUDY); index++) {
		if (list_Of_Dead_Cycle_Loops_Waiting_For_Data[index] != 0xFFFFFFFF) {
			average_Of_Dead_Cycle_Loops += list_Of_Dead_Cycle_Loops_Waiting_For_Data[index];
			accepted_Frames_Used_In_Average += 1;
			
			if (list_Of_Dead_Cycle_Loops_Waiting_For_Data[index] > longest_Observed_Delay) {
				longest_Observed_Delay = list_Of_Dead_Cycle_Loops_Waiting_For_Data[index];
			}
		}
	}
	
	if (accepted_Frames_Used_In_Average > 0) {
		delay_Loops_Per_Frame = ((DIGITS_SPEED_REAL)average_Of_Dead_Cycle_Loops) / accepted_Frames_Used_In_Average;
	}
	else {
		delay_Loops_Per_Frame = 0.0;
	}
	
	Convert_Double_To_String(delay_Loops_Per_Frame
									, 8
									, 3
									, (ASCII_Character *)&value_String[0]);
									
	Concat_After_Pascal_String(timing_String, "\p. Ave. delay wait per frame = ");
	Concat_After_Pascal_String(timing_String, value_String);
	
	Convert_Cardinal_32_To_String(longest_Observed_Delay
									, (ASCII_Character *)&value_String[0]
									, NO_LIMIT);
	
	Concat_After_Pascal_String(timing_String, "\p. Max delay wait observed = ");
	Concat_After_Pascal_String(timing_String, value_String);
	Concat_After_Pascal_String(timing_String, "\p. Max Seconds waiting for Read ISSUANCE = ");

	if (longest_Tick_Time_Of_Issued_Read > 0) {
		delay_Loops_Per_Frame = ((DIGITS_SPEED_REAL)longest_Tick_Time_Of_Issued_Read) / 60;
	}
	else {
		delay_Loops_Per_Frame = 0.0;
	}

	Convert_Double_To_String(delay_Loops_Per_Frame
									, 7
									, 4
									, (ASCII_Character *)&value_String[0]);

	Concat_After_Pascal_String(timing_String, value_String);

	/*  */
	/* Should show times for dead loops and longest tick time */
	/* on screen. For now just use a debugger. */
	/*  */
/* ===
	xxx =list_Of_Dead_Cycle_Loops_Waiting_For_Data[xx]
	xxx = longest_Tick_Time_Of_Issued_Read
=== */
/* - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - */
/* - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - */
/* - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - */

	ValidRect(&((current_Graphics_Window_Ptr)->portRect));
		
		
No_Draw_Needed:
Out_Of_Bounds:

	if (End_Use_Of_Handle((Handle) active_Window_List[MAIN_SPINNING_WINDOW_INDEX], status_Globals_State) != SAFE_TRUE) {
		goto Error_Handler;
	}

	function_Result = SAFE_TRUE;
	goto Normal_Function_Exit;

Error_Handler_With_Message:
	function_Result = SAFE_FALSE;
	Display_Serious_Error_Message(unit_Name, code_Name, note_String, "\p");
	goto Normal_Function_Exit;

Error_Handler:
	function_Result = SAFE_FALSE;
		
Normal_Function_Exit:
	g_Last_Image.valid_Information = SAFE_FALSE;
	
	if (spin_Handle_Info_Locked_Down == SAFE_TRUE) {
		if (End_Use_Of_Handle((Handle)(*spin_Object_Handle)->spin_Info, spin_Info_Handle_State) != SAFE_TRUE) {
			/*  */
			/* Ignore */
			/*  */
		}
	}

	if (unlock_Frozen_Memory_Needed_At_Quit == SAFE_TRUE) {
		if (Unlock_Memory(storage_Buffer_1_Ptr, max_buffer_Size) != SAFE_TRUE) {
			/*  */
			/* Ignore problem for this version */
			/*  */
		}
		if (Unhold_Memory(storage_Buffer_1_Ptr, max_buffer_Size) != SAFE_TRUE) {
			/*  */
			/* Ignore problem for this version */
			/*  */
		}
		if (Unlock_Memory(storage_Buffer_2_Ptr, max_buffer_Size) != SAFE_TRUE) {
			/*  */
			/* Ignore problem for this version */
			/*  */
		}
		if (Unhold_Memory(storage_Buffer_2_Ptr, max_buffer_Size) != SAFE_TRUE) {
			/*  */
			/* Ignore problem for this version */
			/*  */
		}
	} /* if (unlock_Frozen_Memory_Needed_At_Quit == SAFE_TRUE) */

	if (video_Scratch_Buffer_1_Hdl_Memory_Initialized == MEMORY_INITIALIZED) {
		if (Destroy_Handle((Handle *)&video_Scratch_Buffer_1_Hdl, &video_Scratch_Buffer_1_Hdl_Memory_Initialized) != SAFE_TRUE) {
			Display_Serious_Error_Message(unit_Name, code_Name, "\pError with DisposHandle", "\pSystem Corrupted?");
			goto Error_Handler;
		}
	}

	if (video_Scratch_Buffer_2_Hdl_Memory_Initialized == MEMORY_INITIALIZED) {
		if (Destroy_Handle((Handle *)&video_Scratch_Buffer_2_Hdl, &video_Scratch_Buffer_2_Hdl_Memory_Initialized) != SAFE_TRUE) {
			Display_Serious_Error_Message(unit_Name, code_Name, "\pError with DisposHandle", "\pSystem Corrupted?");
			goto Error_Handler;
		}
	}

	if (devices_Swapped == TRUE) {
		Set_View_Device(saved_Device);
	}
	
	if (ports_Swapped == TRUE) {
		Set_View_Port(saved_Port);
	}
	ShowCursor();

	if (show_Statistics == TRUE) {
		if (animation_Total_Frames > 0) {
			Perform_OK_Dialog(timing_String);
		}
	}
	
	if (reallocate_Spin_Cache_When_Finished == SAFE_TRUE) {
		if (Reset_And_Open_Spin_Cache_If_Needed() != SAFE_TRUE) {
			/* Ignore, not critical nor expected */
		}
	}

	return(function_Result);
		
} /* Safe_Boolean	Benchmark_Spinning_Window_Completely ... */
/*=**************************************************************************=*/


/*=**************************************************************************=*/
/* END OF FILE: Play_Movie_Fast_Benchmark.c */






