
/*
 *  VIA_timer.c
 *
 *  Copyright (c) 1991-1994 Symantec Corporation.  All rights reserved.
 *
 */

#include "LowMem.h"
#include "stdlib.h"
#include "profile.h"

#define vT1C		(512*4)
#define vT1CH		(512*5)
#define vIER		(512*14)

struct words { short hi, lo; };
struct bytes { char hi, lo; };

/*  tick count  */
static union {
	long			dword;
	struct words	word;
} elapsed;

/*  VIA countdown timer #1  */
static union {
	short			word;
	struct bytes	byte;
} timer;

static struct {
	ProcPtr			vector;
	char			enable;
} save;

static Boolean timer_running, exithook_installed;

/*	The universal headers do not define the low-mem global Lvl1DT, so define it here.	*/
#ifdef __CONDITIONALMACROS__
#define LMGetLvl1DT(vectorNumber) (*(((ProcPtr *) 0x0192) + (vectorNumber)))
#define LMSetLvl1DT(Lvl1DTValue, vectorNumber) (*(((ProcPtr *) 0x0192) + (vectorNumber)) = (Lvl1DTValue))
#endif


/*
 *  VIA_ticks - return tick count
 *
 *  Each tick represents approximately 1.28 micro-seconds.
 *
 */

long
VIA_ticks(void)
{
	Ptr	VIAbase;
	
#ifdef __CONDITIONALMACROS__
	VIAbase = LMGetVIA();
#else
	VIAbase = VIA;
#endif
	if (!timer_running)
		start_VIA_timer();
	do {
		timer.byte.hi = VIAbase[vT1CH];
		timer.byte.lo = VIAbase[vT1C];
	} while (timer.byte.hi != VIAbase[vT1CH]);
	elapsed.word.lo = ~timer.word;
	return(elapsed.dword);
}


/*
 *  start_VIA_timer
 *
 *  It is not necessary to call this routine, as it is automatically
 *  invoked by "VIA_ticks".
 *
 */

void
start_VIA_timer(void)
{	
	Ptr VIAbase;
#ifdef __CONDITIONALMACROS__
	ProcPtr t;
#endif
		/*  install rollover interrupt  */

#ifdef __CONDITIONALMACROS__
	VIAbase = LMGetVIA();
#else
	VIAbase = VIA;
#endif

	VIAbase[vT1C] = VIAbase[vT1CH] = ~0;
#ifdef __CONDITIONALMACROS__
	save.vector = LMGetLvl1DT(6);
#else
	save.vector = Lvl1DT[6];
#endif
	save.enable = VIAbase[vIER];
	asm {
		lea		@A5,a0
		move.l	a5,(a0)
		lea		@rollover,a0
#ifdef __CONDITIONALMACROS__
		move.l	a0,t
#else
		move.l	a0,Lvl1DT[6]
#endif
	}
#ifdef __CONDITIONALMACROS__
		LMSetLvl1DT(t,6);
#endif
	VIAbase[vIER] = 0xC0;
	
		/*  arrange for timer to be stopped at exit  */
		
	timer_running = true;
	if (!exithook_installed) {
		_atexit(stop_VIA_timer);
		exithook_installed = true;
	}
	return;
		
		/*  interrupt handler  */

rollover:
	asm {
		move.l	a5,-(sp)
		movea.l	@A5,a5
	}
	++elapsed.word.hi;
	VIAbase[vT1C] = VIAbase[vT1CH] = ~0;
	asm {
		movea.l	(sp)+,a5
		rts
A5:		dc.l	0				;  store app's A5 here
	}
}


/*
 *  stop_VIA_timer
 *
 *  It is not necessary to call this routine, as it will be called
 *  automatically at program exit.
 *
 */

void
stop_VIA_timer(void)
{
	Ptr VIAbase;

#ifdef __CONDITIONALMACROS__
	VIAbase = LMGetVIA();
#else
	VIAbase = VIA;
#endif
	if (timer_running) {
		if (!(save.enable & 0x40))
			VIAbase[vIER] = 0x40;
#ifdef __CONDITIONALMACROS__
		LMSetLvl1DT(save.vector,6);
#else
		Lvl1DT[6] = save.vector;
#endif
		timer_running = false;
	}
}
