/* START OF FILE: Slide_Show_Routines.c */


/*=**************************************************************************=*/
/* [---BEGIN---] FILE HEADER COMMENTS */
/**
 ** SCREEN VIEW = [Adobe Syntax Black-10, Tabs = 4]
 ** PRINTER VIEW = [Adobe Syntax Black-10, Tabs = 4]
 ** =======================================================================
 ** Written by James Merkle
 ** Slide_Show_Routines.c - Slide viewer
 ** (C) Copyright 1994 James Merkle
 ** All rights reserved, except those wishing to "copy" moderate sections of code or files may do so, so long as
 ** such code is never used directly either in animation-related tools or multimedia tools that are  
 ** used in live performance or live broadcast or recorded mass market broadcast or film. Desktop usage not restricted.
 ** Conventional desktop game or entertainment not restricted.
 ** =======================================================================
 **
 **
 ** ============
 ** Revision History
 ** ============
 **
 **  1994.04.22		(James Merkle)
 **						Initial coding started
 **
 **
 **
 **
 ** =================
 ** OTHER INFORMATION
 ** =================
 **  
 **
 **/
/* [----END----] FILE HEADER COMMENTS */
/*=**************************************************************************=*/


/*=**************************************************************************=*/
/* [---BEGIN---] #include AREA */

/* ===
#include "About.h"
#include "Dreaming_File_Loading.h"
#include "Dreaming_Slide_Files.h"
#include "Dreaming_Backdrop_Files.h"
#include "Dreaming_Stuff.h"
=== */
#include "Dreaming_Stuff.h"


#include "Error_Utilities.h"
#include "Memory_Utilities.h"
#include "Shell_Dialogs.h"
#include "Off_Screen_Color.h"
#include "Interface_Utilities.h"
#include "Program_Globals.h"
#include "String_Utilities.h"
#include "Volume_Utilities.h"
#include "Dreaming_Popup_Dialogs.h"

#include "Command_Executor.h"

#include "Core_Globals.h"
#include "Core_Specific.h"
#include "Core_Globals.h"
#include "Command_Redirector.h"
#include "Picture_Spooling.h"

#include "Buffered_File_Routines.h"

#include "Slide_Show_Routines.h"

/* [----END----] #include AREA */
/*=**************************************************************************=*/


/*=**************************************************************************=*/
/* [---BEGIN---] #define AREA */

/* ===
#define k_PICT_FILE_DRAWING_BUFFER_SIZE		(20480L)
=== */
#define k_PICT_FILE_DRAWING_BUFFER_SIZE		(40480L)


#define k_RESOURCE_PICT_BASE		30000

/* 100 is too restrictive (Studio 32 limit >= 100 ) */
#define k_PROMPT_MAX_CUTOFF		(102L)


/* [----END----] #define AREA */
/*=**************************************************************************=*/


/*=**************************************************************************=*/
/* [---BEGIN---] TYPE AREA */


/* [----END----] TYPE AREA */
/*=**************************************************************************=*/


/*=**************************************************************************=*/
/* [---BEGIN---] PROTOTYPE AREA */
															

Safe_Boolean Add_Resources_Into_List(void);

pascal void Get_Quick_Draw_Data(Ptr dataPtr, INTEGER_16 byteCount);

Safe_Boolean	Reset_Window_Size_If_Needed(Rect *source_Bit_Rect, Safe_Boolean *resize_Occured, Boolean from_Official_Update);

/* [----END----] PROTOTYPE AREA */
/*=**************************************************************************=*/


/*=**************************************************************************=*/
/* [---BEGIN---] GLOBAL DEFINITION AREA */

	Safe_Boolean			g_Recent_Remote_Slides_Valid;
	Handle				g_Recent_Remote_Slides_Hdl;
	Memory_Allocation	g_Recent_Remote_Slides_Memory_In_Use;
	CARDINAL_32		g_Recent_Remote_Slides_Length;
	CARDINAL_32		g_Total_Picts_Available;
	
	Memory_Allocation		g_Slides_Memory_In_Use;
	Slides_Ptr					g_Slides_Ptr;
	Slides_Hdl					g_Slides_Hdl;
		
		
	CARDINAL_32		g_Active_Pict_Index;
	CARDINAL_32	g_Clear_Screen_Index;


/* [----END----] GLOBAL DEFINITION AREA */
/*=**************************************************************************=*/


/*=**************************************************************************=*/
/* [---BEGIN---] LOCAL STATIC DEFINITION AREA */

static CARDINAL_8	*unit_Name;

	CARDINAL_32	g_Previously_Drawn_Pict_Index;
		
	Rect						g_Last_Rect;
		
/* [----END----] LOCAL STATIC DEFINITION AREA */
/*=**************************************************************************=*/


/*=**************************************************************************=*/
Safe_Boolean Initialize_Slide_Show_Routines(void)
{
	CARDINAL_8			*static_Unit_Name = "\pSlide_Show_Routines";
	CARDINAL_32		loop;
	SignedByte			original_State;
	CARDINAL_32		memory_Requested;
	Safe_Boolean			function_Result;
/* END VAR */


/*  */
/* Global persistant structures are created here */
/*  */
	unit_Name = static_Unit_Name;
	function_Result = SAFE_FALSE;

	g_Slides_Hdl = nil;
	g_Slides_Memory_In_Use = MEMORY_NOT_INITIALIZED;
	g_Slides_Ptr = nil;
	
	memory_Requested = sizeof(Slides_Array);
	if (Create_Handle( (CARDINAL_32 *)&memory_Requested
							, (CARDINAL_32)0L
							, ALLOW_PURGING
							, DISALLOW_PARTIAL_AMOUNT
							, (Handle *)&g_Slides_Hdl
							, &g_Slides_Memory_In_Use) != SAFE_TRUE ) {
							
		Perform_OK_Dialog("\pMemory error allocating slide memory (not enough?) (4)");
		goto Error_Handler;
	}
	
	if (Begin_Use_Of_Handle((Handle)g_Slides_Hdl
							, MOVE_HIGH_REQUIRED
							, &original_State) != SAFE_TRUE) {
		goto Error_Handler;
	}
		
	/*  */
	/* Because locked, we can access using a pointer */
	/*  */		
	g_Slides_Ptr = *g_Slides_Hdl;
	

	g_Recent_Remote_Slides_Hdl = nil;
	g_Recent_Remote_Slides_Memory_In_Use = MEMORY_NOT_INITIALIZED;
	memory_Requested = k_MAXIMUM_CACHED_REMOTE_LIST_SIZE;
	if (Create_Handle( (CARDINAL_32 *)&memory_Requested
							, (CARDINAL_32)0L
							, ALLOW_PURGING
							, DISALLOW_PARTIAL_AMOUNT
							, (Handle *)&g_Recent_Remote_Slides_Hdl
							, &g_Recent_Remote_Slides_Memory_In_Use) != SAFE_TRUE ) {
							
		Perform_OK_Dialog("\pMemory error allocating slide memory (not enough?) (4)");
		goto Error_Handler;
	}
	
	if (Begin_Use_Of_Handle((Handle)g_Recent_Remote_Slides_Hdl
							, MOVE_HIGH_REQUIRED
							, &original_State) != SAFE_TRUE) {
		goto Error_Handler;
	}
	g_Recent_Remote_Slides_Length = 0;
	g_Recent_Remote_Slides_Valid = SAFE_FALSE;

	g_Previously_Drawn_Pict_Index = 99999;
	
	g_Total_Picts_Available = 0;

	for(loop= 0; (loop < k_MAXIMUM_SLIDES); loop++) {
		g_Slides_Ptr->slides[loop].active = SAFE_FALSE;
		
		g_Slides_Ptr->slides[loop].type_Of_Source = k_FROM_ALGORITHM;
		g_Slides_Ptr->slides[loop].algorithm_Id = k_BLACK_WINDOW;
		
/* ===
		g_Slides_Ptr->slides[loop].resource_Id = 140;
		g_Slides_Ptr->slides[loop].active = SAFE_FALSE;
		g_Slides_Ptr->slides[loop].active = SAFE_FALSE;
		g_Slides_Ptr->slides[loop].active = SAFE_FALSE;

=== */

	}
	
	/*  */
	/* Setup Empty Screen  */
	/*  */
	g_Slides_Ptr->slides[g_Total_Picts_Available].active = SAFE_TRUE;
	g_Slides_Ptr->slides[g_Total_Picts_Available].type_Of_Source = k_FROM_ALGORITHM;
	g_Slides_Ptr->slides[g_Total_Picts_Available].algorithm_Id = k_NO_WINDOW;
	
	/*  */
	/* Names MUSt be 31 or less characters */
	/*  */
	Copy_Pascal_String(&k_NO_WINDOW_NAME[0], (g_Slides_Ptr->slides[g_Total_Picts_Available].slide_Name));
	g_Total_Picts_Available += 1;

	
	/*  */
	/* Setup Blue Screen  */
	/*  */
	g_Slides_Ptr->slides[g_Total_Picts_Available].active = SAFE_TRUE;
	g_Slides_Ptr->slides[g_Total_Picts_Available].type_Of_Source = k_FROM_ALGORITHM;
	g_Slides_Ptr->slides[g_Total_Picts_Available].algorithm_Id = k_BLUESCREEN_WINDOW;
	Copy_Pascal_String(&k_BLUESCREEN_WINDOW_NAME[0], (g_Slides_Ptr->slides[g_Total_Picts_Available].slide_Name));
	g_Total_Picts_Available += 1;

	/*  */
	/* Setup Blue Screen  */
	/*  */
	g_Slides_Ptr->slides[g_Total_Picts_Available].active = SAFE_TRUE;
	g_Slides_Ptr->slides[g_Total_Picts_Available].type_Of_Source = k_FROM_ALGORITHM;
	g_Slides_Ptr->slides[g_Total_Picts_Available].algorithm_Id = k_GREENSCREEN_WINDOW;
	Copy_Pascal_String(&k_GREENSCREEN_WINDOW_NAME[0], (g_Slides_Ptr->slides[g_Total_Picts_Available].slide_Name));
	g_Total_Picts_Available += 1;

	
	/*  */
	/* Setup Black Pict Item */
	/*  */
	g_Slides_Ptr->slides[g_Total_Picts_Available].active = SAFE_TRUE;
	g_Slides_Ptr->slides[g_Total_Picts_Available].type_Of_Source = k_FROM_ALGORITHM;
	g_Slides_Ptr->slides[g_Total_Picts_Available].algorithm_Id = k_BLACK_WINDOW;
	Copy_Pascal_String(&k_BLACK_WINDOW_NAME[0], (g_Slides_Ptr->slides[g_Total_Picts_Available].slide_Name));

	/*  */
	/* Choose this as default */
	/*  */
	g_Clear_Screen_Index = g_Total_Picts_Available;
	g_Total_Picts_Available += 1;
	
	
	if (Add_Resources_Into_List() != SAFE_TRUE) {
		goto Error_Handler;
	}
	
	SetRect(&g_Last_Rect, -10, -10, 800, 800);
	
	/*  */
	/*  no need to re-draw */
	/*  */
	g_Active_Pict_Index = g_Clear_Screen_Index;

	function_Result = SAFE_TRUE;
	goto Normal_Function_Exit;

Error_Handler:
	function_Result = SAFE_FALSE;
		
Normal_Function_Exit:
	return(function_Result);
}
/*=**************************************************************************=*/


/*=**************************************************************************=*/
Safe_Boolean Destroy_Slide_Show_Routines(void)
{
	Safe_Boolean		function_Result;
/* END VAR */

/*  */
/* Global persistant structures are deleted here */
/*  */

	function_Result = SAFE_TRUE;
	return(function_Result);
}
/*=**************************************************************************=*/


/*=**************************************************************************=*/
Safe_Boolean Select_Slide(CARDINAL_32	slide)
{
Safe_Boolean		function_Result;
/* END VAR */
	
	
	function_Result = SAFE_FALSE;

	g_Active_Pict_Index = slide;
	
	if (Redraw_General_Screen_Completely(FALSE) != SAFE_TRUE) {
		goto Error_Handler;
	}
	
	function_Result = SAFE_TRUE;
	goto Normal_Function_Exit;

Error_Handler:
	function_Result = SAFE_FALSE;
		
Normal_Function_Exit:
	return(function_Result);
}
/*=**************************************************************************=*/


/*=**************************************************************************=*/
Safe_Boolean Convert_Text_List_Of_Slides_To_Menu(Handle list_Data_Hdl
											, CARDINAL_32 utilized_Bytes
											, MenuHandle	popup_Menu
											, CARDINAL_32	*total_Listed_Items
											, CARDINAL_16 *popup_References)
{
Safe_Boolean		function_Result;
CARDINAL_8		*data;
CARDINAL_32	current_Index;
Str63				temp_String;
CARDINAL_32	position_Value;
Str255				menu_String;
/* END VAR */
	
	
	function_Result = SAFE_FALSE;
	
	/*  */
	/* list_Data_Hdl is assumed locked by design. */
	/*  */
	data = (CARDINAL_8 *)*list_Data_Hdl;

	current_Index = 0;
	/*  */
	/* Ignore and skip length and first colon */
	/*  */
	current_Index += 11;
	
	while (current_Index < utilized_Bytes) {
	
		/*  */
		/* Force string into word alignment by copying it */
		/*  */
		BlockMove(&data[current_Index], &temp_String[0], (data[current_Index] + 1L));
		
		if (Convert_String_To_Cardinal_32(&temp_String[0]
														, &position_Value) != SAFE_TRUE) {
			/*  */
			/* Ugh! might as well make entire list invalid and abort? */
			/* No, assume no more data */
			/*  */
			function_Result = SAFE_TRUE;
			goto Normal_Function_Exit;
		}
	
		menu_String[0] = 1;
		menu_String[1] = 'X';
		AppendMenu(popup_Menu, menu_String);
		popup_References[*total_Listed_Items] = position_Value;
		
		current_Index += (temp_String[0] + 1UL);
		current_Index += 1;
	
		BlockMove(&data[current_Index], &menu_String[0], (data[current_Index] + 1UL));
				
		current_Index += (data[current_Index] + 1UL);
		current_Index += 1;

		*total_Listed_Items += 1;
		SetItem(popup_Menu, *total_Listed_Items, &menu_String[0]);
	
	} /* while (current_Index < utilized_Bytes) */
		
	
	function_Result = SAFE_TRUE;
	goto Normal_Function_Exit;

Error_Handler:
	function_Result = SAFE_FALSE;
		
Normal_Function_Exit:
	return(function_Result);
}
/*=**************************************************************************=*/


/*=**************************************************************************=*/
Safe_Boolean Create_Text_List_Of_Slides(CARDINAL_8 *list_Data
											, CARDINAL_32 available_Bytes
											, CARDINAL_32 *utilized_Bytes)
{
Safe_Boolean		function_Result;
CARDINAL_32	total_Bytes;
CARDINAL_32	index;
CARDINAL_32	total_Listed_Items;
Str63				long_String;
CARDINAL_32	length_To_Copy;
CARDINAL_32	longest_Possible_Element;
/* END VAR */
	
	
	function_Result = SAFE_FALSE;
	
	*utilized_Bytes = 0;
	
	total_Bytes = 0;
	
	if (available_Bytes < 11) {
		goto Error_Handler;
	}
	
	longest_Possible_Element = 10 + 1 + 32 + 1;
	
	/*  */
	/* Paranoia only */
	/*  */
	longest_Possible_Element += 1; 
	
	/*  */
	/* Create a field at beginning to hold string representation */
	/* of entire count of records to process. Nicety of design, not vital. */
	/*  */
	for (index = 0; (index < 10); index++) {
		list_Data[index] = ASCII_SPACE;
	}
	
	total_Bytes += 10;
	
	/*  */
	/* Use semicolons to separate fields of data (they will not be occuring in this data) */
	/*  */
	list_Data[total_Bytes] = ':';
	total_Bytes += 1;
	
	total_Listed_Items = 0;
	for (index = 0; ((index < k_MAXIMUM_SLIDES) 
		&& (total_Listed_Items < g_Total_Picts_Available)
		&& ((total_Bytes + longest_Possible_Element) < available_Bytes)); index++) {
					
		if ((**g_Slides_Hdl).slides[index].active == SAFE_TRUE) {
			total_Listed_Items += 1;
			
			/*  */
			/* Convert and copy index position into data list */
			/*  */
			Convert_Cardinal_32_To_String(index
								, &long_String[0]
								, NO_LIMIT);
									
			length_To_Copy = long_String[0];
			length_To_Copy += 1;
			
			BlockMove(&(long_String[0])
						, &(list_Data[total_Bytes])
						, length_To_Copy);
						
			total_Bytes += length_To_Copy;
			
			/*  */
			/* Use semicolons to separate fields of data */
			/*  */
			list_Data[total_Bytes] = ':';
			total_Bytes += 1;

			/*  */
			/* Copy Display string into data list */
			/*  */
			length_To_Copy = (**g_Slides_Hdl).slides[index].slide_Name[0];
			length_To_Copy += 1;
			
			BlockMove(&((**g_Slides_Hdl).slides[index].slide_Name[0])
						, &(list_Data[total_Bytes])
						, length_To_Copy);
						
			total_Bytes += length_To_Copy;

			/*  */
			/* Use semicolons to separate fields of data */
			/*  */
			list_Data[total_Bytes] = ':';
			total_Bytes += 1;
		}
	}

	/*  */
	/* Prefix padded numeric total of length to data list front section*/
	/*  */
	Convert_Cardinal_32_To_String(total_Bytes
						, &long_String[0]
						, NO_LIMIT);
						
	length_To_Copy = long_String[0];
	BlockMove(&(long_String[1])
				, &(list_Data[0 + (10 - length_To_Copy)])
				, length_To_Copy);
						
	*utilized_Bytes = total_Bytes;
	
	function_Result = SAFE_TRUE;
	goto Normal_Function_Exit;

Error_Handler:
	function_Result = SAFE_FALSE;
		
Normal_Function_Exit:
	return(function_Result);
}
/*=**************************************************************************=*/


/*=**************************************************************************=*/
Safe_Boolean Advance_Slide(CARDINAL_32	direction)
{
Safe_Boolean		function_Result;
INTEGER_32		index;
INTEGER_32		total_Listed_Items;
Boolean			located;
CARDINAL_32		slide;
/* END VAR */
	
	
	function_Result = SAFE_FALSE;
	located = FALSE;
	slide = 0;
	
	if (direction == 0) {
	
		total_Listed_Items = 0;
		for (index = ((INTEGER_32)g_Active_Pict_Index - 1L); ((index >= 0) 
			&& (total_Listed_Items < g_Total_Picts_Available)
			&& (located != TRUE)); index = index - 1) {
						
			if ((**g_Slides_Hdl).slides[index].active == SAFE_TRUE) {
				total_Listed_Items += 1;
				
				/*  */
				/* If criteria matches (always does), set exit condition */
				/*  */
				located = TRUE;
				slide = index;
			}
			
		}
		
		if (located != TRUE) {
			/*  */
			/* Wrap around!, select last one found in list */
			/*  */
			total_Listed_Items = 0;
			for (index = (k_MAXIMUM_SLIDES - 1); ((index >= 0) 
				&& (total_Listed_Items < g_Total_Picts_Available)
				&& (located != TRUE)); index = index - 1) {
							
				if ((**g_Slides_Hdl).slides[index].active == SAFE_TRUE) {
					total_Listed_Items += 1;
					
					/*  */
					/* If criteria matches (always does), set exit condition */
					/*  */
					located = TRUE;
					slide = index;
				}
				
			}
		}		
		
		if (located == TRUE) {
			if (g_Active_Pict_Index != slide) {
				g_Active_Pict_Index = slide;
			}
			else {
				located = FALSE;
			}
		}
		
	}
	else if (direction == 1) {
	
		total_Listed_Items = 0;
		for (index = (g_Active_Pict_Index + 1); ((index < k_MAXIMUM_SLIDES) 
			&& (total_Listed_Items < g_Total_Picts_Available)
			&& (located != TRUE)); index++) {
						
			if ((**g_Slides_Hdl).slides[index].active == SAFE_TRUE) {
				total_Listed_Items += 1;
				
				/*  */
				/* If criteria matches (always does), set exit condition */
				/*  */
				located = TRUE;
				slide = index;
			}
			
		}
		
		if (located != TRUE) {
			/*  */
			/* Wrap around!, select first one found in list */
			/*  */
			total_Listed_Items = 0;
			for (index = 0; ((index < k_MAXIMUM_SLIDES) 
				&& (total_Listed_Items < g_Total_Picts_Available)
				&& (located != TRUE)); index++) {
							
				if ((**g_Slides_Hdl).slides[index].active == SAFE_TRUE) {
					total_Listed_Items += 1;
					
					/*  */
					/* If criteria matches (always does), set exit condition */
					/*  */
					located = TRUE;
					slide = index;
				}
				
			}
		}		
		
		if (located == TRUE) {
			if (g_Active_Pict_Index != slide) {
				g_Active_Pict_Index = slide;
			}
			else {
				located = FALSE;
			}
		}
			
		g_Active_Pict_Index = slide;
	}
	else {
		/* do nothing, g_Active_Pict_Index is active */
		located = FALSE;
	}
	
	if (located == TRUE) {
		if (Redraw_General_Screen_Completely(FALSE) != SAFE_TRUE) {
			goto Error_Handler;
		}
	}
	
	function_Result = SAFE_TRUE;
	goto Normal_Function_Exit;

Error_Handler:
	function_Result = SAFE_FALSE;
		
Normal_Function_Exit:
	return(function_Result);
	
} /* Safe_Boolean Advance_Slide ... */
/*=**************************************************************************=*/


/*=**************************************************************************=*/
Safe_Boolean Change_Window_Size_BASE(Change_Window_Size_Rec	*params_For_Change_Window_Size)
{
	Safe_Boolean		function_Result;
	GrafPtr			window_Manager_Port;
	WindowPtr		current_Graphics_Window_Ptr;
	GrafPtr			saved_Update_Port;
	Rect				valid_Screen_Area;
	INTEGER_16		window_Index;
	INTEGER_16		width;
	INTEGER_16		height;
	INTEGER_32		left_Global;
	INTEGER_32		top_Global;
	Rect						temp_Rect;
	Rect						desired_Screen_Area;
	Primary_Status_Hdl		status_Globals_Hdl;
	Safe_Boolean				window_Needs_Unhide;
/* END VAR */

	
	function_Result = SAFE_TRUE;
	window_Needs_Unhide = SAFE_FALSE;
	GetPort(&saved_Update_Port);

	window_Index= params_For_Change_Window_Size->window_Index;
	status_Globals_Hdl = active_Window_List[window_Index];
	
	if ((status_Globals_Hdl != nil)
		&& ((**status_Globals_Hdl).structure_Initialized == SAFE_TRUE)) {
		current_Graphics_Window_Ptr	= (**status_Globals_Hdl).status_Window;
	}
	else {
		goto Error_Handler;
	}

	GetWMgrPort(&window_Manager_Port);
/* ==
	valid_Screen_Area = window_Manager_Port->portRect;
	valid_Screen_Area.top = valid_Screen_Area.top + GetMBarHeight();
=== */

	valid_Screen_Area = (qd.screenBits).bounds;
	
/* ===
	params_For_Change_Window_Size->new_Rect
=== */

	switch(params_For_Change_Window_Size->window_Type) {
		
		case k_WINDOW_SIZE_640_390:
			width = 640;
			height = 390;
		break;
		
		case k_WINDOW_SIZE_638_455:
			width = 638;
			height = 455;
		break;
		
		case k_WINDOW_SIZE_640_480:
			width = 640;
			height = 480;
		break;
		
		case k_WINDOW_SIZE_960_720:
			width = 960;
			height = 720;
		break;
		
		case k_WINDOW_SIZE_FILLSCREEN:
			width = 640;
			height = 390;
		break;
		
		case k_WINDOW_SIZE_DEFAULT:
		default:
			/*  */
			/* Sizes will be reset properly for "k_WINDOW_SIZE_DEFAULT" soon */
			/*  */
			width = 640;
			height = 70;
	}
	
	if (params_For_Change_Window_Size->window_Type == k_WINDOW_SIZE_DEFAULT) {
		temp_Rect = (**status_Globals_Hdl).window_Starting_Port_Rect_Position;
		width = temp_Rect.right - temp_Rect.left;
		height = temp_Rect.bottom - temp_Rect.top;
		
		/*  */
		/*  */
		/*  */
		/*  */
		/*  */
		/* HACK HACK HACK : Need to find out largest legal workspace. */
		/* what command? what function? How? */
		/* For now, negate usefulness of valid_Screen_Area (our sanity checker ) */
		/*  */
			valid_Screen_Area = temp_Rect;
		
		/*  */
		/*  */
		/*  */
		
		if ((valid_Screen_Area.left <= temp_Rect.right) && (valid_Screen_Area.right >= temp_Rect.left)) {
			/*  */
			/* Good, one side or other in bounds so far */
			/*  */
			if ((valid_Screen_Area.top <= temp_Rect.bottom) && (valid_Screen_Area.bottom >= temp_Rect.top)) {
				/*  */
				/* Good, window top portion in bounds (though maybe not on a monitor */
				/*  */
				
				ForeColor(blackColor);
				BackColor(blackColor);
				EraseRect (&(((**status_Globals_Hdl).status_Window)->portRect));
				ForeColor(blackColor);
				BackColor(whiteColor);
				
				if ((**status_Globals_Hdl).window_Is_Hidden != SAFE_TRUE) {
					ShowHide((**status_Globals_Hdl).status_Window, FALSE);
					window_Needs_Unhide = SAFE_TRUE;
				}
								
				(**status_Globals_Hdl).update_Events_Permissible = SAFE_FALSE;
				MoveWindow((**status_Globals_Hdl).status_Window, temp_Rect.left, temp_Rect.top, FALSE);
				(**status_Globals_Hdl).update_Events_Permissible = SAFE_TRUE;
			}
		}
	}
	else {
		/*  */
		/* Center to center of "startup" screen if possible, else center left right */
		/*  */
		if ((**status_Globals_Hdl).global_Device_Startup_Max_Rect_Valid == SAFE_TRUE) {
		
			desired_Screen_Area = (**status_Globals_Hdl).global_Device_Startup_Max_Rect;
			
		/*  */
		/*  */
		/*  */
		/*  */
		/*  */
		/* HACK HACK HACK : Need to find out largest legal workspace. */
		/* what command? what function? How? */
		/* For now, negate usefulness of valid_Screen_Area (our sanity checker ) */
		/*  */
			valid_Screen_Area = desired_Screen_Area;
		
		/*  */
		/*  */
		/*  */

			
			if ((desired_Screen_Area.left >= valid_Screen_Area.left)
				&& (desired_Screen_Area.top >= valid_Screen_Area.top)
				&& (desired_Screen_Area.right <= valid_Screen_Area.right)
				&& (desired_Screen_Area.bottom <= valid_Screen_Area.bottom)) {
				/*  */
				/* Nothing odd has happened. Continue */
				/*  */
				left_Global = desired_Screen_Area.left + ((desired_Screen_Area.right - desired_Screen_Area.left) / 2) - (width / 2);
			/* ===
				if (left_Global < desired_Screen_Area.left) {
					left_Global = desired_Screen_Area.left;
				}
			==== */
	
				if ((**status_Globals_Hdl).global_Device_Startup_Max_Rect_Contains_Menubar == SAFE_TRUE) {
					desired_Screen_Area.top = desired_Screen_Area.top + GetMBarHeight();
				}
				
				top_Global = desired_Screen_Area.top;
				
				/*  */
				/* We will not be centering vertically in this implementation */
				/* But we will instead be honoring a resource preference */
				/* positive or negative */
				/*  */
				top_Global += dream.main_Window_Preferred_Top_Offset;

			/* ===
				top_Global = top_Global + ((desired_Screen_Area.bottom - desired_Screen_Area.top) / 2) - (height / 2);
			==== */
			
				/*  */
				/* TOP MUST be inside if on a Macintosh */
				/*  */
				if (top_Global < desired_Screen_Area.top) {
					top_Global = desired_Screen_Area.top;
				}
				
				ForeColor(blackColor);
				BackColor(blackColor);
				EraseRect (&(((**status_Globals_Hdl).status_Window)->portRect));
				ForeColor(blackColor);
				BackColor(whiteColor);
				
				(**status_Globals_Hdl).update_Events_Permissible = SAFE_FALSE;
				MoveWindow((**status_Globals_Hdl).status_Window, left_Global, top_Global, FALSE);
				(**status_Globals_Hdl).update_Events_Permissible = SAFE_TRUE;
				
			}
			else {
				/*  */
				/* Monitor configuration changed! Ignore cached values. Rewrite */
				/* reset code one day? */
				/*  */
				; /* NULL */
			}
		}
	}
	

	if ((**status_Globals_Hdl).window_Is_Hidden != SAFE_TRUE) {
		SizeWindow(current_Graphics_Window_Ptr,width,height,TRUE);
	}
	else {
		SizeWindow(current_Graphics_Window_Ptr,width,height,FALSE);
	}
	
	if (window_Needs_Unhide == SAFE_TRUE){
		ShowHide((**status_Globals_Hdl).status_Window, TRUE);
	}
			
	function_Result = SAFE_TRUE;
	goto Normal_Function_Exit;
	
Error_Handler:
	function_Result = SAFE_FALSE;

Normal_Function_Exit:
	Set_View_Port(saved_Update_Port);
	return(function_Result);
	
} /* Safe_Boolean Change_Window_Size_BASE ... */
/*=**************************************************************************=*/


/*=**************************************************************************=*/
Safe_Boolean	Reset_Window_Size_If_Needed(Rect *source_Bit_Rect, Safe_Boolean *resize_Occured, Boolean from_Official_Update)
{
	Safe_Boolean						function_Result;
	Change_Window_Size_Rec			params_For_Change_Window_Size;
	INTEGER_32						source_Bit_Width;
	INTEGER_32						source_Bit_Height;
	Rect								current_Rect;
	INTEGER_32						current_Bit_Width;
	INTEGER_32						current_Bit_Height;
	Primary_Status_Hdl				status_Globals_Hdl;
	Boolean							needs_Resize;
/* END VAR */
	
	
	function_Result = SAFE_FALSE;
	*resize_Occured = SAFE_FALSE;

	source_Bit_Width = (*source_Bit_Rect).right - (*source_Bit_Rect).left;
	source_Bit_Height = (*source_Bit_Rect).bottom - (*source_Bit_Rect).top;

	status_Globals_Hdl = active_Window_List[GENERAL_INFO_WINDOW_INDEX];
	
	if ((status_Globals_Hdl != nil)
		&& ((**status_Globals_Hdl).structure_Initialized == SAFE_TRUE)) {
	}
	else {
		goto Error_Handler;
	}

	needs_Resize = FALSE;
	if (source_Bit_Height > k_PROMPT_MAX_CUTOFF) {
		/*  */
		/* Check if window is already less than k_PROMPT_MAX_CUTOFF. It should be */
		/* if normal prompt small window of 70 pixels high (or 100 )*/
		/*  */
		
		current_Rect = ((WindowPeek)(**status_Globals_Hdl).status_Window)->port.portRect;
		current_Bit_Width = (current_Rect).right - (current_Rect).left;
		current_Bit_Height = (current_Rect).bottom - (current_Rect).top;
		
		/*  */
		/* Only height is compared */
		/*  */
		if (current_Bit_Height < k_PROMPT_MAX_CUTOFF) {
			needs_Resize = TRUE;
			params_For_Change_Window_Size.window_Type = k_WINDOW_SIZE_640_480;
		}
	}
	else {
		if (source_Bit_Height < k_PROMPT_MAX_CUTOFF) {
			/*  */
			/* Check if window is already more than k_PROMPT_MAX_CUTOFF. It should be */
			/* if not the prompt window. */
			/*  */
		
			current_Rect = ((WindowPeek)(**status_Globals_Hdl).status_Window)->port.portRect;
			current_Bit_Width = (current_Rect).right - (current_Rect).left;
			current_Bit_Height = (current_Rect).bottom - (current_Rect).top;
			
			/*  */
			/* Only height is compared */
			/*  */
			if (current_Bit_Height > k_PROMPT_MAX_CUTOFF) {
				needs_Resize = TRUE;
				params_For_Change_Window_Size.window_Type = k_WINDOW_SIZE_DEFAULT;
			}
		}
	}

	if (from_Official_Update != TRUE) {
		if (needs_Resize == TRUE) {
			params_For_Change_Window_Size.window_Index = GENERAL_INFO_WINDOW_INDEX;
			SetRect(&(params_For_Change_Window_Size.new_Rect), 0, 0, 1, 1);
					
			if (Change_Window_Size_BASE(&params_For_Change_Window_Size) != SAFE_TRUE) {
				goto Error_Handler;
			}
			*resize_Occured = SAFE_TRUE;
			
			/*  */
			/* Bring to foreground if not default, else bring puppet window to */
			/* foreground and let this thing go to the back */
			/*  */
			if (params_For_Change_Window_Size.window_Type == k_WINDOW_SIZE_DEFAULT) {
			
				if ((active_Window_List[TEST_DREAMING_WINDOW_INDEX] != nil)
					|| ((**active_Window_List[TEST_DREAMING_WINDOW_INDEX]).structure_Initialized == SAFE_TRUE)) {
					SelectWindow((**(active_Window_List[TEST_DREAMING_WINDOW_INDEX])).status_Window);
					
					/*  */
					/* Gotta reset port we were in if any */
					/*  */
					
				}
				
			}
			else {
				SelectWindow((**status_Globals_Hdl).status_Window);
			}
			
		} /* if (needs_Resize == TRUE) */
		else {
			/*  */
			/* Paranoia and semi-non paranoia. Always select large windows for */
			/* safety even if not officially changing sizes */
			/*  */
			SelectWindow((**status_Globals_Hdl).status_Window);
		}
	}

	function_Result = SAFE_TRUE;
	goto Normal_Function_Exit;

Error_Handler:
	function_Result = SAFE_FALSE;
		
Normal_Function_Exit:
	return(function_Result);
}
/*=**************************************************************************=*/


/*=**************************************************************************=*/
Safe_Boolean Redraw_General_Screen_Completely(Boolean from_Official_Update)
{
	SignedByte		status_Globals_State;
	WindowPtr		current_Graphics_Window_Ptr;
	Safe_Boolean		function_Result;
	PicHandle			source_Pict_Hdl;
	INTEGER_32		source_Bit_Width;
	INTEGER_32		source_Bit_Height;
	INTEGER_16		source_Pict_Id;
	INTEGER_16		source_Pict_File_Ref;
	INTEGER_32		source_File_Size;
	Rect				source_Bit_Rect;
	Boolean			new_Display;
	Boolean			redisplay_Window;
	Safe_Boolean		picture_Failure;
	RgnHandle		save_Region;
	Safe_Boolean		region_Swapped;
	Safe_Boolean		resize_Occured;
	INTEGER_32		dest_Height;
	INTEGER_32		dest_Width;
	INTEGER_32		adjust_Top;
	INTEGER_32		adjust_Left;
	OSErr				os_Error;
	RGBColor			element_Color;
	RGBColor			previous_Color;
/* END VAR */
	
	
	function_Result = SAFE_FALSE;
	picture_Failure = SAFE_FALSE;
	region_Swapped = SAFE_FALSE;
	resize_Occured = SAFE_FALSE;
	source_Pict_File_Ref = 0;
	
/* ===
	if (Begin_Use_Of_Handle((Handle) active_Window_List[GENERAL_INFO_WINDOW_INDEX]
								, MOVE_HIGH_REQUIRED
								, &status_Globals_State) != SAFE_TRUE) {
								
		goto Error_Handler;
	}
=== */
	HLock((Handle) active_Window_List[GENERAL_INFO_WINDOW_INDEX]);



	current_Graphics_Window_Ptr	= (**active_Window_List[GENERAL_INFO_WINDOW_INDEX]).status_Window;

	redisplay_Window = FALSE;
	
	if ((**active_Window_List[GENERAL_INFO_WINDOW_INDEX]).window_Is_Hidden == SAFE_TRUE) {
		redisplay_Window = TRUE;
		if (g_Slides_Ptr->slides[g_Active_Pict_Index].type_Of_Source == k_FROM_ALGORITHM) {
			if (g_Slides_Ptr->slides[g_Active_Pict_Index].algorithm_Id == k_NO_WINDOW) {
				redisplay_Window = FALSE;
			}
		}
	}
	
	if (redisplay_Window == TRUE) {
		/*  */
		/* NOTE: Do not select window! */
		/*  */
		ShowWindow(current_Graphics_Window_Ptr);
		(**active_Window_List[GENERAL_INFO_WINDOW_INDEX]).window_Is_Hidden = SAFE_FALSE;
	}
	
	Set_View_Port (current_Graphics_Window_Ptr);

	if (from_Official_Update != TRUE) {
		/*  */
		/* Force marked valid at end */
		/*  */
		InvalRect(&((current_Graphics_Window_Ptr)->portRect));
		
		save_Region = nil;
		save_Region = NewRgn();
		if (save_Region != nil) {
			GetClip(save_Region);
			ClipRect(&((current_Graphics_Window_Ptr)->portRect));
			region_Swapped = SAFE_TRUE;
		}
	}

	if (from_Official_Update == TRUE) {
		/*  */
		/* We must clear immediately! because might be ugly white areas */
		/*  */
		
		ForeColor(blackColor);
		BackColor(blackColor);
		EraseRect (&((current_Graphics_Window_Ptr)->portRect));
		ForeColor(blackColor);
		BackColor(whiteColor);
	}

	if (g_Total_Picts_Available > 0) {
		if (g_Active_Pict_Index != g_Previously_Drawn_Pict_Index) {
			new_Display = TRUE;
			g_Previously_Drawn_Pict_Index = g_Active_Pict_Index;
		}
		else {
			/*  */
			/* Already is being displayed! we should only redraw what we need to */
			/* We should NOT black out entire rect area. to prevent flicker */
			/*  */
			new_Display = FALSE;
		}
			
		if (g_Slides_Ptr->slides[g_Active_Pict_Index].type_Of_Source == k_FROM_ALGORITHM) {
			
			switch (g_Slides_Ptr->slides[g_Active_Pict_Index].algorithm_Id) {
				
				case k_NO_WINDOW:
					if (new_Display == TRUE) {
						current_Graphics_Window_Ptr	= (**active_Window_List[GENERAL_INFO_WINDOW_INDEX]).status_Window;
						if ((**active_Window_List[GENERAL_INFO_WINDOW_INDEX]).window_Is_Hidden != SAFE_TRUE) {
							HideWindow(current_Graphics_Window_Ptr);
							(**active_Window_List[GENERAL_INFO_WINDOW_INDEX]).window_Is_Hidden = SAFE_TRUE;
						}
					}
					else {
						/*  */
						/* Nothing to do special */
						/*  */
					}
					
					ForeColor(blackColor);
					BackColor(blackColor);
					EraseRect (&((current_Graphics_Window_Ptr)->portRect));
					ForeColor(blackColor);
					BackColor(whiteColor);
				break;
				
				case k_BLUESCREEN_WINDOW:
					element_Color.blue = 0xFFFF;
					element_Color.red = 0;
					element_Color.green = 0;
					
					GetBackColor(&previous_Color);
					RGBBackColor(&element_Color);
					EraseRect (&((current_Graphics_Window_Ptr)->portRect));
					RGBBackColor(&previous_Color);
				break;
				
				case k_GREENSCREEN_WINDOW:
					element_Color.blue = 0;
					element_Color.red = 0;
					element_Color.green = 0xFFFF;
					
					GetBackColor(&previous_Color);
					RGBBackColor(&element_Color);
					EraseRect (&((current_Graphics_Window_Ptr)->portRect));
					RGBBackColor(&previous_Color);
				break;
				
				case k_BLACK_WINDOW:
				default:
					ForeColor(blackColor);
					BackColor(blackColor);
					EraseRect (&((current_Graphics_Window_Ptr)->portRect));
					ForeColor(blackColor);
					BackColor(whiteColor);
			}
						
			g_Last_Rect = (((current_Graphics_Window_Ptr)->portRect));
						
		}
		else if (g_Slides_Ptr->slides[g_Active_Pict_Index].type_Of_Source == k_FROM_RESOURCE) {
		
			/*  */
			/* Get resource ready if possible */
			/*  */	
			source_Pict_Id = g_Slides_Ptr->slides[g_Active_Pict_Index].resource_Id;
			
			source_Pict_Hdl = GetPicture(source_Pict_Id);
			if ((ResError() != noErr) || (source_Pict_Hdl == nil)) {
				/*  */
				/* Release resource */
				/*  */
				if (source_Pict_Hdl != nil) {
					ReleaseResource((Handle)source_Pict_Hdl);
				}
				picture_Failure = SAFE_TRUE;

			}
			else {
				source_Bit_Rect = (*source_Pict_Hdl)->picFrame;
				source_Bit_Width = source_Bit_Rect.right - source_Bit_Rect.left;
				source_Bit_Height = source_Bit_Rect.bottom - source_Bit_Rect.top;
				SetRect(&source_Bit_Rect, 0, 0, source_Bit_Width, source_Bit_Height);
				
				
				if (Reset_Window_Size_If_Needed(&source_Bit_Rect, &resize_Occured, from_Official_Update) != SAFE_TRUE) {
				
					Set_View_Port (current_Graphics_Window_Ptr);
					
					/*  */
					/* No consequence if failure */
					/*  */
				}
				else {
					Set_View_Port (current_Graphics_Window_Ptr);

					if (resize_Occured == SAFE_TRUE) {
						/*  */
						/* Reset the forced update infor if windows changed in size */
						/*  */
						if (from_Official_Update != TRUE) {
							InvalRect(&((current_Graphics_Window_Ptr)->portRect));
							
							if (region_Swapped == SAFE_TRUE) {
								ClipRect(&((current_Graphics_Window_Ptr)->portRect));
							}
						}
						/*  */
						/* We need to change clip rect! */
						/* the saved version is evil now */
						/* Done at bottom */
						/*  */
						
						ForeColor(blackColor);
						BackColor(blackColor);
						EraseRect (&((current_Graphics_Window_Ptr)->portRect));
						ForeColor(blackColor);
						BackColor(whiteColor);
					}
				}
				

				
				/*  */
				/* Only needed really for CopyBits calls but why not be certain. */
				/*  */
				ForeColor(blackColor);
				BackColor(whiteColor);
				
				
				dest_Width = (current_Graphics_Window_Ptr)->portRect.right - (current_Graphics_Window_Ptr)->portRect.left;
				dest_Height = (current_Graphics_Window_Ptr)->portRect.bottom - (current_Graphics_Window_Ptr)->portRect.top;
				
				if (source_Bit_Height <= dest_Height){
					/*  */
					/* Recenter destination Rect height*/
					/*  */
					adjust_Top = dest_Height - source_Bit_Height;
					adjust_Top = adjust_Top / 2;
					
					OffsetRect(&source_Bit_Rect,0,adjust_Top);
				}
				
				if (source_Bit_Width <= dest_Width){
					/*  */
					/* Recenter destination Rect width */
					/*  */
					adjust_Left = dest_Width - source_Bit_Width;
					adjust_Left = adjust_Left / 2;
					OffsetRect(&source_Bit_Rect,adjust_Left,0);
				}

				if (new_Display == TRUE) {
					if ((source_Bit_Rect.left > g_Last_Rect.left)
						|| (source_Bit_Rect.top > g_Last_Rect.top)
						|| (source_Bit_Rect.right < g_Last_Rect.right)
						|| (source_Bit_Rect.bottom < g_Last_Rect.bottom)) {
						
						/*  */
						/* Erase if areas not equal or inside bounds of previous drawing location */
						/*  */
						ForeColor(blackColor);
						BackColor(blackColor);
						EraseRect (&((current_Graphics_Window_Ptr)->portRect));
						ForeColor(blackColor);
						BackColor(whiteColor);
					}
				
					g_Last_Rect = source_Bit_Rect;
					if (g_Last_Rect.right > (current_Graphics_Window_Ptr)->portRect.right) {
						g_Last_Rect.right = (current_Graphics_Window_Ptr)->portRect.right;
					}

					if (g_Last_Rect.bottom > (current_Graphics_Window_Ptr)->portRect.bottom) {
						g_Last_Rect.bottom = (current_Graphics_Window_Ptr)->portRect.bottom;
					}

					if (g_Last_Rect.top < (current_Graphics_Window_Ptr)->portRect.top) {
						g_Last_Rect.top = (current_Graphics_Window_Ptr)->portRect.top;
					}

					if (g_Last_Rect.left < (current_Graphics_Window_Ptr)->portRect.left) {
						g_Last_Rect.left = (current_Graphics_Window_Ptr)->portRect.left;
					}

				}
				
				DrawPicture(source_Pict_Hdl, &source_Bit_Rect);
				
				/*  */
				/* Release resource */
				/*  */
				if (source_Pict_Hdl != nil) {
					ReleaseResource((Handle)source_Pict_Hdl);
				}
			}
			
		}
		else if (g_Slides_Ptr->slides[g_Active_Pict_Index].type_Of_Source == k_FROM_FILE) {
														
#if (k_USE_BUFFERED_FILE_IO == 1)
		source_File_Size = 0;
		if (Open_Well_Defined_File_Using_Buffer(DATA_FORK
										, &(g_Slides_Ptr->slides[g_Active_Pict_Index].file_Info.name[0])
										, 0
										, (g_Slides_Ptr->slides[g_Active_Pict_Index].file_Info.vRefNum)
										, (g_Slides_Ptr->slides[g_Active_Pict_Index].file_Info.parID)
										, &source_Pict_File_Ref
										, (512L)
										, k_PICT_FILE_DRAWING_BUFFER_SIZE
										, k_PICT_FILE_DRAWING_BUFFER_SIZE
										, &source_File_Size
										, SAFE_TRUE
										, &os_Error) != TRUE) {
			picture_Failure = SAFE_TRUE;

			if (os_Error == noErr) {
				os_Error = -1;
			}
		}
#else
	os_Error = FSpOpenDF(&(g_Slides_Ptr->slides[g_Active_Pict_Index].file_Info),fsCurPerm,&source_Pict_File_Ref);
	if (os_Error != noErr) {
		picture_Failure = SAFE_TRUE;
	}
#endif


			if (os_Error == noErr){
					
				/*  */
				/* Only needed really for CopyBits calls but why not be certain. */
				/*  */
				ForeColor(blackColor);
				BackColor(whiteColor);

				os_Error = Get_File_Picture_Rect(source_Pict_File_Ref, &source_Bit_Rect);
				if (os_Error != noErr) {
					picture_Failure = SAFE_TRUE;
					goto Normal_Function_Exit;
				}
				
				/*  */
				/* source_Bit_Rect is ready! nor we can re-read and re-process the PICT */
				/*  */
				if (os_Error == noErr){
				
													
					source_Bit_Width = source_Bit_Rect.right - source_Bit_Rect.left;
					source_Bit_Height = source_Bit_Rect.bottom - source_Bit_Rect.top;
	
					SetRect(&source_Bit_Rect, 0, 0, source_Bit_Width, source_Bit_Height);
					
					if (Reset_Window_Size_If_Needed(&source_Bit_Rect, &resize_Occured, from_Official_Update) != SAFE_TRUE) {
				
						Set_View_Port (current_Graphics_Window_Ptr);

						/*  */
						/* No consequence if failure */
						/*  */
					}
					else {
				
						Set_View_Port (current_Graphics_Window_Ptr);
					
						if (resize_Occured == SAFE_TRUE) {
							/*  */
							/* Reset the forced update infor if windows changed in size */
							/*  */
							if (from_Official_Update != TRUE) {
								InvalRect(&((current_Graphics_Window_Ptr)->portRect));
								
								if (region_Swapped == SAFE_TRUE) {
									ClipRect(&((current_Graphics_Window_Ptr)->portRect));
								}
							}
							/*  */
							/* We need to change clip rect! */
							/* the saved version is evil now */
							/* Done at bottom */
							/*  */
							
							ForeColor(blackColor);
							BackColor(blackColor);
							EraseRect (&((current_Graphics_Window_Ptr)->portRect));
							ForeColor(blackColor);
							BackColor(whiteColor);
						}
					}
					
					dest_Width = (current_Graphics_Window_Ptr)->portRect.right - (current_Graphics_Window_Ptr)->portRect.left;
					dest_Height = (current_Graphics_Window_Ptr)->portRect.bottom - (current_Graphics_Window_Ptr)->portRect.top;
					
					if (source_Bit_Height <= dest_Height){
						/*  */
						/* Recenter destination Rect height*/
						/*  */
						adjust_Top = dest_Height - source_Bit_Height;
						adjust_Top = adjust_Top / 2;
						
						OffsetRect(&source_Bit_Rect,0,adjust_Top);
					}
					
					if (source_Bit_Width <= dest_Width){
						/*  */
						/* Recenter destination Rect width */
						/*  */
						adjust_Left = dest_Width - source_Bit_Width;
						adjust_Left = adjust_Left / 2;
						OffsetRect(&source_Bit_Rect,adjust_Left,0);
					}
					
					if (new_Display == TRUE) {
					
						if ((source_Bit_Rect.left > g_Last_Rect.left)
							|| (source_Bit_Rect.top > g_Last_Rect.top)
							|| (source_Bit_Rect.right < g_Last_Rect.right)
							|| (source_Bit_Rect.bottom < g_Last_Rect.bottom)) {
							
							/*  */
							/* Erase if areas not equal or inside bounds of previous drawing location */
							/*  */
							ForeColor(blackColor);
							BackColor(blackColor);
							EraseRect (&((current_Graphics_Window_Ptr)->portRect));
							ForeColor(blackColor);
							BackColor(whiteColor);
						}
					
						g_Last_Rect = source_Bit_Rect;
						if (g_Last_Rect.right > (current_Graphics_Window_Ptr)->portRect.right) {
							g_Last_Rect.right = (current_Graphics_Window_Ptr)->portRect.right;
						}
	
						if (g_Last_Rect.bottom > (current_Graphics_Window_Ptr)->portRect.bottom) {
							g_Last_Rect.bottom = (current_Graphics_Window_Ptr)->portRect.bottom;
						}
	
						if (g_Last_Rect.top < (current_Graphics_Window_Ptr)->portRect.top) {
							g_Last_Rect.top = (current_Graphics_Window_Ptr)->portRect.top;
						}
	
						if (g_Last_Rect.left < (current_Graphics_Window_Ptr)->portRect.left) {
							g_Last_Rect.left = (current_Graphics_Window_Ptr)->portRect.left;
						}
	
					}
					
					/* Draw the contents of the PICT file into the current port */
	
					/*  */
					/* Release File and reopen to guarantee flush */
					/*  */
#if (k_USE_BUFFERED_FILE_IO == 1)
					if (Close_File_With_Buffer(source_Pict_File_Ref
												, FLUSH_BUFFER
												, &os_Error) != TRUE) {
						/*  */
						/* Hmmm. Not expected. Ignore anyway. */
						/*  */
						Perform_OK_Dialog("\pProblem with Close_File_With_Buffer (2)");
						picture_Failure = SAFE_TRUE;
					}
					source_Pict_File_Ref = 0;
#else
	os_Error = FSClose(source_Pict_File_Ref);
	source_Pict_File_Ref = 0;
	
	if (os_Error != noErr) {
		picture_Failure = SAFE_TRUE;
	}
#endif

					
					if (os_Error == noErr) {
#if (k_USE_BUFFERED_FILE_IO == 1)
						if (Open_Well_Defined_File_Using_Buffer(DATA_FORK
														, &(g_Slides_Ptr->slides[g_Active_Pict_Index].file_Info.name[0])
														, 0
														, (g_Slides_Ptr->slides[g_Active_Pict_Index].file_Info.vRefNum)
														, (g_Slides_Ptr->slides[g_Active_Pict_Index].file_Info.parID)
														, &source_Pict_File_Ref
														, (512L)
														, k_PICT_FILE_DRAWING_BUFFER_SIZE
														, k_PICT_FILE_DRAWING_BUFFER_SIZE
														, &source_File_Size
														, SAFE_TRUE
														, &os_Error) != TRUE) {
							Perform_OK_Dialog("\pProblem with Open_Well_Defined_File_Using_Buffer (1)");
	
							if (os_Error == noErr) {
								os_Error = -1;
							}
						}
#else
	os_Error = FSpOpenDF(&(g_Slides_Ptr->slides[g_Active_Pict_Index].file_Info),fsCurPerm,&source_Pict_File_Ref);
#endif
						if (os_Error == noErr) {

							os_Error = Draw_File_Picture(source_Pict_File_Ref, &source_Bit_Rect );
							if (os_Error != noErr) {
								Perform_OK_Dialog("\pNote: Draw_File_Picture failed");
							}
							
							/*  */
							/* Release File */
							/*  */
#if (k_USE_BUFFERED_FILE_IO == 1)
							if (Close_File_With_Buffer(source_Pict_File_Ref
														, FLUSH_BUFFER
														, &os_Error) != TRUE) {
								/*  */
								/* Hmmm. Not expected. Ignore anyway. */
								/*  */
								Perform_OK_Dialog("\pProblem with Close_File_With_Buffer (1)");
							}
#else
							os_Error = FSClose(source_Pict_File_Ref);
#endif
							source_Pict_File_Ref = 0;
						}

					} /* NOT if (Close_File_With_Buffer(source_Pict_File_Ref */
									
				} /* if (os_Error == noErr) */
				
			} /* if (os_Error == noErr) */
			
		} /* else if (g_Slides_Ptr->slides[g_Active_Pict_Index].type_Of_Source == k_FROM_FILE) */
		else {
			ForeColor(blackColor);
			BackColor(blackColor);
			EraseRect (&((current_Graphics_Window_Ptr)->portRect));
			ForeColor(blackColor);
			BackColor(whiteColor);
		}
		
	}
	else {
		picture_Failure = SAFE_TRUE;
	}



/* ===
	if (status_Globals_State) {
		if (End_Use_Of_Handle((Handle) active_Window_List[GENERAL_INFO_WINDOW_INDEX], status_Globals_State) != SAFE_TRUE) {
			goto Error_Handler;
		}
	}
=== */


	function_Result = SAFE_TRUE;
	goto Normal_Function_Exit;

Error_Handler:
	function_Result = SAFE_FALSE;
		
Normal_Function_Exit:

	if (picture_Failure == SAFE_TRUE) {
		ForeColor(blackColor);
		BackColor(blackColor);
		EraseRect (&((current_Graphics_Window_Ptr)->portRect));
		ForeColor(blackColor);
		BackColor(whiteColor);
		g_Active_Pict_Index = g_Clear_Screen_Index;
	}
	
	if (from_Official_Update != TRUE) {
		ValidRect(&((current_Graphics_Window_Ptr)->portRect));
	}
	
	if (source_Pict_File_Ref != 0) {
#if (k_USE_BUFFERED_FILE_IO == 1)
		if (Close_File_With_Buffer(source_Pict_File_Ref
									, FLUSH_BUFFER
									, &os_Error) != TRUE) {
			/*  */
			/* Hmmm. Not expected. Ignore anyway. */
			/*  */
			Perform_OK_Dialog("\pProblem with Close_File_With_Buffer (2)");
			picture_Failure = SAFE_TRUE;
		}
		source_Pict_File_Ref = 0;
#else
		os_Error = FSClose(source_Pict_File_Ref);
		source_Pict_File_Ref = 0;
		if (os_Error != noErr) {
			/* ignore */
		}
#endif
	}

	if (region_Swapped == SAFE_TRUE) {
		if (resize_Occured == SAFE_TRUE) {
			ClipRect(&((current_Graphics_Window_Ptr)->portRect));
		}
		else {
			SetClip(save_Region);
		}
		
		DisposeRgn(save_Region);
	}

	return(function_Result);
}
/*=**************************************************************************=*/


/*=**************************************************************************=*/
Safe_Boolean Add_Resources_Into_List(void)
{
	Safe_Boolean		function_Result;
	CARDINAL_32	total_Picts;
	CARDINAL_32	missing_Slides;
	OSErr				os_Error;
	PicHandle			source_Pict_Hdl;
	INTEGER_32		source_Bit_Width;
	INTEGER_32		source_Bit_Height;
	INTEGER_16		source_Pict_Id;
	Rect				source_Bit_Rect;
	INTEGER_32		loop;
	CARDINAL_32	total_Found;
	INTEGER_16		resource_Id;
	ResType			resource_Type;
	Str255				temp_String;
	Safe_Boolean		is_Correct_Size;
/* END VAR */
	
	
	function_Result = SAFE_FALSE;
	
	total_Picts = Count1Resources('PICT');
	if (ResError() != noErr) {
		goto Error_Handler;
	}
	
	if (total_Picts < 1) {
		function_Result = SAFE_TRUE;
		goto Normal_Function_Exit;
	}
	
	missing_Slides = 0;
	total_Found = 0;
	for(loop = k_RESOURCE_PICT_BASE; ((total_Found < total_Picts) && (missing_Slides < 20)); loop++) {
	
		source_Pict_Id = loop;
		
		source_Pict_Hdl = nil;
		source_Pict_Hdl = GetPicture(source_Pict_Id);
		if ((ResError() != noErr) || (source_Pict_Hdl == nil)) {
			missing_Slides += 1;
			/*  */
			/* Release resource */
			/*  */
			if (source_Pict_Hdl != nil) {
				ReleaseResource((Handle)source_Pict_Hdl);
			}
		}
		else {
			/*  */
			/* Add to our data base */
			/*  */
	
			GetResInfo((Handle)source_Pict_Hdl
								, &resource_Id
								, &resource_Type
								, &temp_String[0]);
								
			if (ResError() == noErr) {
				/*  */
				/* Do not add to data base unless pict is larger than 64 pixels */
				/* in both directions! Otherwise will mess up logic. Small picts */
				/* are just previews from Photoshop */
				/*  */
				source_Bit_Rect = (*source_Pict_Hdl)->picFrame;
				source_Bit_Width = source_Bit_Rect.right - source_Bit_Rect.left;
				source_Bit_Height = source_Bit_Rect.bottom - source_Bit_Rect.top;
				
				if ((source_Bit_Width > 64) && (source_Bit_Height > 64)) {
					is_Correct_Size = SAFE_TRUE;
				}
			}
			else {
				is_Correct_Size = SAFE_FALSE;
			}
			
								
			if (is_Correct_Size == SAFE_TRUE) {
				total_Found += 1;
				
				g_Slides_Ptr->slides[g_Total_Picts_Available].active = SAFE_TRUE;
				g_Slides_Ptr->slides[g_Total_Picts_Available].type_Of_Source = k_FROM_RESOURCE;
				g_Slides_Ptr->slides[g_Total_Picts_Available].resource_Id = source_Pict_Id;
				g_Slides_Ptr->slides[g_Total_Picts_Available].algorithm_Id = 0;
				
				/*  */
				/* Names MUST be 31 or less characters */
				/*  */
				if (temp_String[0] > 31) {
					temp_String[0] = 31;
				}
				Copy_Pascal_String(&temp_String[0], (g_Slides_Ptr->slides[g_Total_Picts_Available].slide_Name));
			
				g_Total_Picts_Available += 1;
				
	
				/*  */
				/* Release resource */
				/*  */
				if (source_Pict_Hdl != nil) {
					ReleaseResource((Handle)source_Pict_Hdl);
				}

			}
			else {
				missing_Slides += 1;
				/*  */
				/* Release resource */
				/*  */
				if (source_Pict_Hdl != nil) {
					ReleaseResource((Handle)source_Pict_Hdl);
				}
				
			} /* NOT if (ResError() == noErr) */
					
		}
	}
	
	function_Result = SAFE_TRUE;
	goto Normal_Function_Exit;

Error_Handler:
	function_Result = SAFE_FALSE;
		
Normal_Function_Exit:
	return(function_Result);
}
/*=**************************************************************************=*/


/*=**************************************************************************=*/
/* END OF FILE: Slide_Show_Routines.c */
