/* START OF FILE: Smart_Copy_Ticked_Pixels.c */


/*=**************************************************************************=*/
/* [---BEGIN---] FILE HEADER COMMENTS */
/**
 ** SCREEN VIEW = [Adobe Syntax Black-10, Tabs = 4]
 ** PRINTER VIEW = [Adobe Syntax Black-10, Tabs = 4]
 ** =======================================================================
 ** Written by James Merkle
 ** Smart_Copy_Ticked_Pixels.c - Video Routines to support Poor Man's Video Phone
 ** (C) Copyright 1994 James Merkle
 ** All rights reserved, except those wishing to "copy" moderate sections of code or files may do so, so long as
 ** such code is never used directly either in animation-related tools or multimedia tools that are  
 ** used in live performance or live broadcast or recorded mass market broadcast or film. Desktop usage not restricted.
 ** Conventional desktop game or entertainment not restricted.
 ** =======================================================================
 **
 **
 ** ============
 ** Revision History
 ** ============
 **
 **  1995.04.14		(James Merkle)
 **						Initial coding started from scratch
 **
 **
 **
 ** =================
 ** OTHER INFORMATION
 ** =================
 ** 
 ** 
 **
 **
 **/
/* [----END----] FILE HEADER COMMENTS */
/*=**************************************************************************=*/


/*=**************************************************************************=*/
/* [---BEGIN---] #include AREA */


#include "Shell_Assembly.h"
#include "Fast_Image_Compaction.h"
#include "Memory_Utilities.h"
#include "Core_Specific.h"
#include "Error_Utilities.h"

#include "Smart_Copy_Ticked_Pixels.h"

/* [----END----] #include AREA */
/*=**************************************************************************=*/


/*=**************************************************************************=*/
/* [---BEGIN---] #define AREA */


/* [----END----] #define AREA */
/*=**************************************************************************=*/


/*=**************************************************************************=*/
/* [---BEGIN---] TYPE AREA */


/* [----END----] TYPE AREA */
/*=**************************************************************************=*/


/*=**************************************************************************=*/
/* [---BEGIN---] PROTOTYPE AREA */


/* [----END----] PROTOTYPE AREA */
/*=**************************************************************************=*/


/*=**************************************************************************=*/
/* [---BEGIN---] GLOBAL DEFINITION AREA */


/* [----END----] GLOBAL DEFINITION AREA */
/*=**************************************************************************=*/


/*=**************************************************************************=*/
/* [---BEGIN---] LOCAL STATIC DEFINITION AREA */

static CARDINAL_8	*unit_Name = "\pSmart_Copy_Ticked_Pixels";


/* [----END----] LOCAL STATIC DEFINITION AREA */
/*=**************************************************************************=*/


/*=**************************************************************************=*/  
Safe_Boolean Build_New_Smart_Copy_List(CARDINAL_32 total_Height
										, Smart_Copy_Hdl *smart_Copy_Info_Hdl)
{
	Safe_Boolean		function_Result;
	Memory_Allocation	spoof_Hdl_Memory_In_Use;
	CARDINAL_8		*code_Name = "\pBuild_New_Smart_Copy_List";
	CARDINAL_32	size_Needed;
	CARDINAL_32	preferred_Size;
	Scan_Bounds_Ptr		scan_Info;
/* END VAR */


	function_Result = SAFE_FALSE;
	
	if (total_Height < 1) {
		goto Error_Handler;
	}
	
	preferred_Size = sizeof(Smart_Copy_Rec);
	preferred_Size = preferred_Size + ((CARDINAL_32)sizeof(Scan_Span_Rec) * (total_Height - 1UL));
	size_Needed = preferred_Size;
	
	spoof_Hdl_Memory_In_Use = MEMORY_NOT_INITIALIZED;
	
	if (Create_Handle(&preferred_Size
						, (CARDINAL_32)(10240L)
						, ALLOW_PURGING
						, DISALLOW_PARTIAL_AMOUNT
						, (Handle *)smart_Copy_Info_Hdl
						, &spoof_Hdl_Memory_In_Use) != SAFE_TRUE) {
						
		Display_Low_Memory_Error_Message(unit_Name, code_Name, size_Needed);
		goto Error_Handler;
	}
	
	(**smart_Copy_Info_Hdl)->initialized = SAFE_TRUE;
	(**smart_Copy_Info_Hdl)->current_Structure_Size = size_Needed;

	scan_Info = &((**smart_Copy_Info_Hdl)->scan_Info);
	SetRect(&(scan_Info->bounds_Rect), 0, 0, total_Height + 1, 32767);
	SetRect(&(scan_Info->possible_Rect), 0, 0, total_Height + 1, 32767);
	scan_Info->max_Lines = total_Height;

	Clear_New_Smart_Copy_List(**smart_Copy_Info_Hdl);
	
	function_Result = SAFE_TRUE;
	goto Normal_Function_Exit;

Error_Handler:
	function_Result = SAFE_FALSE;
		
Normal_Function_Exit:
	return(function_Result);
		
} /* Safe_Boolean Build_New_Smart_Copy_List... */
/*=**************************************************************************=*/


/*=**************************************************************************=*/ 
Safe_Boolean Delete_New_Smart_Copy_List(Smart_Copy_Hdl *smart_Copy_Info_Hdl)
{
	Safe_Boolean		function_Result;
	Memory_Allocation	spoof_Hdl_Memory_In_Use;
	CARDINAL_8		*code_Name = "\pDelete_New_Smart_Copy_List";
/* END VAR */


	function_Result = SAFE_FALSE;

	if ((**smart_Copy_Info_Hdl)->initialized != SAFE_TRUE) {
		goto Error_Handler;
	}

	spoof_Hdl_Memory_In_Use = MEMORY_INITIALIZED;
	if (Destroy_Handle((Handle *)smart_Copy_Info_Hdl, &spoof_Hdl_Memory_In_Use) != SAFE_TRUE) {
		Display_Serious_Error_Message(unit_Name, code_Name, "\pError with DisposHandle", "\pSystem Corrupted?");
		goto Error_Handler;
	}

	function_Result = SAFE_TRUE;
	goto Normal_Function_Exit;

Error_Handler:
	function_Result = SAFE_FALSE;
		
Normal_Function_Exit:
	return(function_Result);
	
} /* Safe_Boolean Delete_New_Smart_Copy_List... */
/*=**************************************************************************=*/


/*=**************************************************************************=*/ 
Safe_Boolean Clear_New_Smart_Copy_List(Smart_Copy_Rec *smart_Copy_Info_Ptr)
{
	Safe_Boolean		function_Result;
	CARDINAL_32	bytes_To_Erase;
	Ptr					address_To_Erase;
/* END VAR */


	function_Result = SAFE_FALSE;
	
	if (smart_Copy_Info_Ptr->initialized == SAFE_TRUE) {
		bytes_To_Erase = smart_Copy_Info_Ptr->scan_Info.max_Lines;
		bytes_To_Erase = (bytes_To_Erase * (CARDINAL_32)sizeof(Scan_Span_Rec));
		address_To_Erase = (Ptr)&(smart_Copy_Info_Ptr->scan_Info.spans[0]);
		
		Fill_Large_Buffer(address_To_Erase, bytes_To_Erase, 0);
		smart_Copy_Info_Ptr->scan_Info.spans[smart_Copy_Info_Ptr->scan_Info.max_Lines - 1].rightmost_Pixel = 0;
		smart_Copy_Info_Ptr->scan_Info.spans[smart_Copy_Info_Ptr->scan_Info.max_Lines - 1].leftmost_Pixel = 0;
	}
	else {
		goto Error_Handler;
	}

	function_Result = SAFE_TRUE;
	goto Normal_Function_Exit;

Error_Handler:
	function_Result = SAFE_FALSE;
		
Normal_Function_Exit:
	return(function_Result);
		
} /* Safe_Boolean Clear_New_Smart_Copy_List... */
/*=**************************************************************************=*/


/*=**************************************************************************=*/  
Safe_Boolean Add_Span_Entry_To_Smart_Copy_List(Smart_Copy_Rec *smart_Copy_Info_Ptr
															, CARDINAL_32 vertical_Line_Affected
															, CARDINAL_32 horizontal_Address_Start
															, CARDINAL_32 horizontal_Address_Stop)
{
	Safe_Boolean		function_Result;
/* END VAR */


	function_Result = SAFE_FALSE;





	function_Result = SAFE_TRUE;
	goto Normal_Function_Exit;

Error_Handler:
	function_Result = SAFE_FALSE;
		
Normal_Function_Exit:
	return(function_Result);
		
} /* Safe_Boolean Add_Span_Entries_To_Smart_Copy_List... */
/*=**************************************************************************=*/


/*=**************************************************************************=*/  
Safe_Boolean Add_Picture_Spans_To_Smart_Copy_List(Smart_Copy_Rec *smart_Copy_Info_Ptr
																,  Scan_Bounds_Ptr source_Bounds_Info
																, CARDINAL_32	horizontal_Offset
																, CARDINAL_32	vertical_Offset)
{
	Safe_Boolean			function_Result;
	Scan_Bounds_Ptr		composite_Bounds_Info;
	Rect					source_Rect;
	Rect					destination_Rect;
/* END VAR */


	function_Result = SAFE_FALSE;

	if ((smart_Copy_Info_Ptr)->initialized != SAFE_TRUE) {
		goto Error_Handler;
	}
	
	composite_Bounds_Info = &((smart_Copy_Info_Ptr)->scan_Info);	
	
	/*  */
	/* First, check if is entirely out of bounds and needs no drawing. */
	/* Maybe the picture is out of bounds because the view port is */
	/* much smaller than the offscreen work area. */
	/*  */
	
	source_Rect = source_Bounds_Info->bounds_Rect;
	OffsetRect(&source_Rect, horizontal_Offset, vertical_Offset);
	destination_Rect = composite_Bounds_Info->bounds_Rect;
	
	if (horizontal_Offset >= destination_Rect.right) {
		goto Nothing_Visible_To_Draw_Exit;
	}
	
	if (vertical_Offset >= destination_Rect.bottom) {
		goto Nothing_Visible_To_Draw_Exit;
	}
	
	if ((source_Bounds_Info->bounds_Rect.right + horizontal_Offset) < destination_Rect.left) {
		goto Nothing_Visible_To_Draw_Exit;
	}
	
	if ((source_Bounds_Info->bounds_Rect.bottom + vertical_Offset) < destination_Rect.top) {
		goto Nothing_Visible_To_Draw_Exit;
	}
	
	/*  */
	/* Check if horizontal clipping will be needed. */
	/* Check if vertical clipping will be needed. */
	/*  */
	
	
	/*  */
	/* Increment any line that has rightmost pixel > 0 */
	/*  */
	
/* ==
		destination_Horizontal_Start = 0;

		clipping_Needed = FALSE;	

	Destination_Vertical_Start = 0;
=== */

	
/* ====
	scan_Info = &((smart_Copy_Info_Ptr)->scan_Info);
	SetRect(&(scan_Info->bounds_Rect), 0, 0, total_Height + 1, 32767);

	scan_Line_Bounds_Hdl
=== */


/* ====

typedef struct {
	CARDINAL_16	rightmost_Pixel;
	CARDINAL_16	leftmost_Pixel;
} Scan_Span_Rec, *Scan_Span_Ptr;

typedef struct {
	Rect				bounds_Rect;
	Rect				possible_Rect;
	CARDINAL_32	max_Lines;
	Scan_Span_Rec	spans[1];
} Scan_Bounds_Rec, *Scan_Bounds_Ptr;

==== */





	function_Result = SAFE_TRUE;
	goto Normal_Function_Exit;

Error_Handler:
	function_Result = SAFE_FALSE;
		
Nothing_Visible_To_Draw_Exit:

Normal_Function_Exit:
	return(function_Result);
		
} /* Safe_Boolean Add_Picture_Spans_To_Smart_Copy_List... */
/*=**************************************************************************=*/


/*=**************************************************************************=*/
/* END OF FILE: Smart_Copy_Ticked_Pixels.c */






