/* START OF FILE: Spin_O_Rama.c */


/*=**************************************************************************=*/
/* [---BEGIN---] FILE HEADER COMMENTS */
/**
 ** SCREEN VIEW = [Adobe Syntax Black-10, Tabs = 4]
 ** PRINTER VIEW = [Adobe Syntax Black-10, Tabs = 4]
 ** =======================================================================
 ** Written by James Merkle
 ** Spin_O_Rama.c - Routines for Spin fil display window and control
 ** (C) Copyright 1994 James Merkle
 ** All rights reserved, except those wishing to "copy" moderate sections of code or files may do so, so long as
 ** such code is never used directly either in animation-related tools or multimedia tools that are  
 ** used in live performance or live broadcast or recorded mass market broadcast or film. Desktop usage not restricted.
 ** Conventional desktop game or entertainment not restricted.
 ** =======================================================================
 **
 **
 ** ============
 ** Revision History
 ** ============
 **
 **  1994.02.01		(James Merkle)
 **						Initial coding started
 **
 **
 **
 **
 ** =================
 ** OTHER INFORMATION
 ** =================
 **  bounce - hackup...
 **
 **
 **
 ** Frame rate, color conversion still frames any size batch. No Crash, Director easy.
 **
 ** 
 ** maintain velocity
 ** use PICS files (no delay)
 ** count pics, compare sizes (memory), max bounds, max depth
 ** -- color preference. then validity, 
 ** STATS report frames, max depth
 ** REDUCE - drop extraneous bounds, drop frames (except for equator), drop altitudes, drop depth (32>16>8), reduce rez (1/4)
 ** Query save script?
 **
 ** 
 **
 **/
/* [----END----] FILE HEADER COMMENTS */
/*=**************************************************************************=*/


/*=**************************************************************************=*/
/* [---BEGIN---] #include AREA */

#include "About.h"

#include "Error_Utilities.h"
#include "Memory_Utilities.h"
#include "Program_Globals.h"
#include "Shell_Assembly.h"
#include "String_Utilities.h"

#include "Dreaming_Popup_Dialogs.h"
#include "Vertical_Blanking_Interrupt.h"
#include "Graphics_Demos_Support.h"
#include "Shell_Dialogs.h"
#include "Graphics_Primitives.h"
#include "Window_Environment.h"

#include "Interface_Utilities.h"
#include "Spin_O_Rama_Files.h"
#include "Dreaming_Animation.h"

#include "Spin_O_Rama_Load_Import.h"
#include "OS_Utilities.h"
#include "Core_Globals.h"
#include "Core_Specific.h"
#include "Dreaming_Stuff.h"
#include "Core_Globals.h"
#include "Command_Redirector.h"
#include "Fast_Bit_Copy.h"
#include "Video_Backdrop_Background.h"
#include "Play_Movie_Fast_Benchmark.h"

#include "Spin_O_Rama.h"

/* [----END----] #include AREA */
/*=**************************************************************************=*/


/*=**************************************************************************=*/
/* [---BEGIN---] #define AREA */

#define COLOR_PORT_PORTBITS_CHECK 		0xC000

#define k_MESSAGE_0_PLAY_MOVIE				0
#define k_MESSAGE_1_STOP_MOVIE				1
#define k_MESSAGE_2_START_INTERACTIVE		2
#define k_MESSAGE_3_STOP_INTERACTIVE		3
#define k_MESSAGE_4_PLAY_MOVIE_NO_LOOP		4
#define k_MESSAGE_5_PLAY_MOVIE_FROM_0_NO_LOOP		5
#define k_MESSAGE_6_PLAY_BACK_INTERACTIVE			6

#define k_MAX_DATA_RUNS									(200UL)

/* [----END----] #define AREA */
/*=**************************************************************************=*/


/*=**************************************************************************=*/
/* [---BEGIN---] TYPE AREA */

/* ===
	typedef struct {
		CARDINAL_32	total_Size;
		FCB				*file_Control;
		BTCB				*file_Tree_Control;
		SAfe_Boolean		fully_Mapped;
		CARDINAL_32	total_spans;
		Data_Run			data_Runs[k_MAX_DATA_RUNS];
	} Volume_Map_Rec, *Volume_Map_Ptr;
=== */



/* [----END----] TYPE AREA */
/*=**************************************************************************=*/


/*=**************************************************************************=*/
/* [---BEGIN---] PROTOTYPE AREA */
															
Safe_Boolean	Load_PICS_Sequence_For_Spinning(CARDINAL_32 index_Used
															, Safe_Boolean *user_Abort);

Safe_Boolean	Convert_Spin_File_Frames(void);

/* [----END----] PROTOTYPE AREA */
/*=**************************************************************************=*/


/*=**************************************************************************=*/
/* [---BEGIN---] GLOBAL DEFINITION AREA */

Point				g_Last_Spinning_Point_Position;
CARDINAL_32	g_Last_Spinning_Sound_Level;
CARDINAL_32	g_Time_Since_Last_Sound_Check;
CARDINAL_32	g_Time_Since_Last_Sound_Change;
Safe_Boolean		g_Cursor_Probably_Still_Hidden;
Safe_Boolean		g_Interactive_Motion_Disabled;

Safe_Boolean		g_Spin_File_Importation_Active;

Last_Image_Rec	g_Last_Image;
CARDINAL_32	g_Last_Movie_Frame_Paused;
Safe_Boolean		g_Loop_Movie_Forever;


/* [----END----] GLOBAL DEFINITION AREA */
/*=**************************************************************************=*/


/*=**************************************************************************=*/
/* [---BEGIN---] LOCAL STATIC DEFINITION AREA */

static CARDINAL_8	*unit_Name;

	Safe_Boolean		g_Program_Message_Busy;
	Safe_Boolean		g_Program_Message_New_Message;
	CARDINAL_32	g_Program_Message;
	
	Safe_Boolean		g_Set_Spin_Window_Extra_Gigantic;

	Safe_Boolean 	g_Set_Spin_Window_Max_Monitor_Size;
	Safe_Boolean 	g_Menu_Bar_Obscured;
	INTEGER_16 		g_Menu_Bar_Previous_Height;
	RgnHandle		g_Menu_Bar_Previous_Region;		 	
	RgnHandle		g_Corner_Regions;

/* [----END----] LOCAL STATIC DEFINITION AREA */
/*=**************************************************************************=*/


/*=**************************************************************************=*/
Safe_Boolean Initialize_Spin_O_Rama(void)
{
	CARDINAL_8			*static_Unit_Name = "\pSpin_O_Rama";
/* END VAR */

/*  */
/* Global persistant structures are created here */
/*  */

	dream.spin_Objects.total_Spin_Objects = 0;
	g_Last_Spinning_Sound_Level = 0x7FFFFFFF;
	g_Time_Since_Last_Sound_Check = 0;
	g_Time_Since_Last_Sound_Change = 0;
	g_Last_Spinning_Point_Position.h = -1;
	g_Last_Spinning_Point_Position.v = -1;
	g_Last_Image.valid_Information = SAFE_FALSE;
	g_Last_Image.avoidance_Image_Index = k_BOGUS_INDEX_FRAME;
	
	g_Program_Message_Busy = SAFE_FALSE;
	g_Program_Message = 0;
	
	g_Cursor_Probably_Still_Hidden = SAFE_FALSE;
	g_Interactive_Motion_Disabled = SAFE_FALSE;
	g_Spin_File_Importation_Active = SAFE_FALSE;
	g_Set_Spin_Window_Extra_Gigantic = SAFE_FALSE;
	
	g_Loop_Movie_Forever = SAFE_FALSE;
	g_Last_Movie_Frame_Paused = 0;

/*  */
/* g_Set_Spin_Window_Max_Monitor_Size was */
/* already initialized in menu startup */
/*  */
/* ===
	g_Set_Spin_Window_Max_Monitor_Size = SAFE_FALSE;
=== */

	unit_Name = static_Unit_Name;
	return(SAFE_TRUE);
}
/*=**************************************************************************=*/


/*=**************************************************************************=*/
Safe_Boolean Destroy_Spin_O_Rama(void)
{
	Safe_Boolean		function_Result;
	OSErr				os_Error;
/* END VAR */

/*  */
/* Global persistant structures are deleted here */
/*  */	

	if (g_Last_Image.valid_Information == SAFE_TRUE) {


	}

	function_Result = SAFE_TRUE;
	return(function_Result);
}
/*=**************************************************************************=*/


/*=**************************************************************************=*/
Safe_Boolean Reset_Mouse_Settings_For_Spin_Window(void)
{
	WindowPtr		current_Graphics_Window_Ptr;
	SignedByte		status_Globals_State;
	Rect				global_Detination_Rect;
	GrafPtr			saved_Port;
	Point				offset_Point_For_Shield_Cursor;
	Safe_Boolean		function_Result;
/* END VAR */


	function_Result = SAFE_TRUE;
	/*  */
	/* this is only meant to be called if invoked from within */
	/* the event loop for switching back into app. it resets everything */
	/*  */

	if ((active_Window_List[MAIN_SPINNING_WINDOW_INDEX] == nil)
		|| ((**active_Window_List[MAIN_SPINNING_WINDOW_INDEX]).structure_Initialized != SAFE_TRUE)) {
		
	}
	else {
		/*  */
		/* Lock down because structure content addresses are being passed */
		/*  */
		if (Begin_Use_Of_Handle((Handle) active_Window_List[MAIN_SPINNING_WINDOW_INDEX]
									, MOVE_HIGH_REQUIRED
									, &status_Globals_State) != SAFE_TRUE) {
									
			goto Error_Handler;
		}

		current_Graphics_Window_Ptr	= (**active_Window_List[MAIN_SPINNING_WINDOW_INDEX]).status_Window;
		GetPort(&saved_Port);
		Set_View_Port(current_Graphics_Window_Ptr);
		
		/*  */
		/* Hack Hack Hack warning, this is meant to safety reset mouse. */
		/* but it might be vital to worry about miny little mouse from regular puppet window */
		/*  */
		InitCursor();
		
		global_Detination_Rect = (current_Graphics_Window_Ptr)->portRect;
		LocalToGlobal(&TOP_LEFT(global_Detination_Rect));
		LocalToGlobal(&BOTTOM_RIGHT(global_Detination_Rect));
		offset_Point_For_Shield_Cursor.h = 0;
		offset_Point_For_Shield_Cursor.v = 0;
		ShieldCursor(&global_Detination_Rect,offset_Point_For_Shield_Cursor);
		g_Cursor_Probably_Still_Hidden = SAFE_TRUE;
		
		Set_View_Port(saved_Port);

		if (End_Use_Of_Handle((Handle) active_Window_List[MAIN_SPINNING_WINDOW_INDEX], status_Globals_State) != SAFE_TRUE) {
			goto Error_Handler;
		}
		
	}

	function_Result = SAFE_TRUE;
	goto Normal_Function_Exit;
		
Error_Handler:
	function_Result = SAFE_FALSE;
		
Normal_Function_Exit:

	return(function_Result);
	
} /* Safe_Boolean Reset_Mouse_Settings_For_Spin_Window ... */
/*=**************************************************************************=*/
	
	
/*=**************************************************************************=*/
Safe_Boolean	Select_Spin_Window_Display_First_Time(void)
/*  */
/* This can, and is, called more than just one time.  It is */
/* called whenever a file is opened */
/*  */
{
	Safe_Boolean		function_Result;
	SignedByte		status_Globals_State;
	WindowPtr		current_Graphics_Window_Ptr;
	CARDINAL_8		*code_Name;
	CARDINAL_32	control_Index;
	INTEGER_32		item_Number;
	INTEGER_16		item_Num;
	INTEGER_16		item_Type;
	Handle			item_Handle;
	OSErr				os_Error;
	Rect				item_Rect;
	Scrolling_List_Ptr	scroll_Ptr;
	DialogPtr			temp_Dialog;
	Str255				temp_String;
/* END VAR */


	function_Result = SAFE_FALSE;
	code_Name = "\pSelect_Spin_Window_Display_First_Time";
	
	/*  */
	/* Do initialization check here if needed */
	/*  */
	Select_Graphics_Window(MAIN_SPINNING_WINDOW_INDEX);
	
	/*  */
	/* Lock down because structure content addresses are being passed */
	/*  */
	if (Begin_Use_Of_Handle((Handle) active_Window_List[MAIN_SPINNING_WINDOW_INDEX]
								, MOVE_HIGH_REQUIRED
								, &status_Globals_State) != SAFE_TRUE) {
								
		goto Error_Handler;
	}

	current_Graphics_Window_Ptr	= (**active_Window_List[MAIN_SPINNING_WINDOW_INDEX]).status_Window;

	SetWTitle(current_Graphics_Window_Ptr, (StringPtr)WINDOW_TITLE_FOR_SPINNING_WINDOW);

	if (Redraw_Spinning_Window_Completely(FALSE, FALSE) == SAFE_TRUE) {
		/*  */
		/* Save window to a PICT for use during updates */
		/*  */
	}
	
										
	if (End_Use_Of_Handle((Handle) active_Window_List[MAIN_SPINNING_WINDOW_INDEX], status_Globals_State) != SAFE_TRUE) {
		goto Error_Handler;
	}
		
	function_Result = SAFE_TRUE;
	goto Normal_Function_Exit;

Error_Handler:
	function_Result = SAFE_FALSE;
		
Normal_Function_Exit:
	return(function_Result);
		
		
} /* Safe_Boolean	Select_Spin_Window_Display_First_Time... */
/*=**************************************************************************=*/


/*=**************************************************************************=*/
Safe_Boolean	Get_Current_Volume_For_Spinfile(CARDINAL_32	*the_256_Percentage)
	/*  */
	/* Draws whole frame, not just deviance */
	/*  */
{
	CARDINAL_32	max_Items;
	CARDINAL_32	total_Items_Seen;
	INTEGER_32		current_Time;
	INTEGER_32		voice_Used;
	Safe_Boolean		function_Result;
	Boolean			use_Keyboard_Control;
	INTEGER_16		voice;
	CARDINAL_32	keyboard_Amplitude;
	Boolean			sound_Emitted;
/* END VAR */


	function_Result = SAFE_FALSE;
	*the_256_Percentage = 0;

	sound_Emitted = FALSE;
	voice_Used = 0;
	if (g_Voices.initialized == SAFE_TRUE) {
		if (g_Voices.total_Voices_Active > 0) {
			max_Items = g_Voices.total_Voices_Active;
			total_Items_Seen = 0;
			for (voice=0; ((voice < k_MAXIMUM_VOICE_MONITORS) && (total_Items_Seen < max_Items) && (sound_Emitted != TRUE)); voice++) {
				if (g_Voices.voice_Memory_Initialized[voice] == SAFE_TRUE) {
					total_Items_Seen += 1;
					sound_Emitted = TRUE;
					voice_Used = voice;
				}
			}
		}
	}

	if (sound_Emitted != TRUE) {
		*the_256_Percentage = 0;
		goto Normal_Function_Exit;
	}
							
	/*  */
	/* We should only use keyboard amplitudes if desparate */
	/* and if in a special keyboard monitoring mode selected in the */
	/* menu */
	/*  */
	use_Keyboard_Control = FALSE;
	
	if (use_Keyboard_Control == TRUE) {
		if (Fetch_Keyboard_Amplitude(&keyboard_Amplitude) == TRUE) {
			*the_256_Percentage = (keyboard_Amplitude << 8) / 9;
		}
		else {
			*the_256_Percentage = 0;
		}
	}
	else {
		*the_256_Percentage = (**(g_Voices.voice[voice_Used])).current_Interpolated_Signal;
		
	} /* NOT if (Fetch_Keyboard_Amplitude(&keyboard_Amplitude) == TRUE) */


	/*  */
	/* Adjust gain for Labview card */
	/*  */
	if ((**(g_Voices.voice[voice_Used])).audio_Source == NATIONAL_INSTRUMENTS_DA) {					
		if ((**(g_Voices.voice[voice_Used])).amplification_Response_Curve != (SPEED_DIGITS_REAL)0.0) {
			Convert_Audio_To_Non_Linear_Gain(*the_256_Percentage, &((**(g_Voices.voice[voice_Used])).amplification_Response_Curve), the_256_Percentage);
		}
		if (*the_256_Percentage > 255) {
			*the_256_Percentage = 255;
		}
	}

	function_Result = SAFE_TRUE;
	goto Normal_Function_Exit;
	
Error_Handler:
	function_Result = SAFE_FALSE;
		
Normal_Function_Exit:
	return(function_Result);
	
} /* Safe_Boolean	Get_Current_Volume_Frame... */
/*=**************************************************************************=*/


/*=**************************************************************************=*/
void Clear_Cached_Spin_Window_Placement_Structures(void)
{
		/*  */
		/* Mark cached information dirty on redraws. */
		/* WARNING : Could be less efficient, but is done for */
		/* sake of moving widow and having structures realign */
		/*  */
		g_Last_Image.use_Fast_Copy = SAFE_FALSE;
		
}
/*=**************************************************************************=*/


/*=**************************************************************************=*/
Safe_Boolean	Redraw_Spinning_Window_Title_Bar_If_Possible(Safe_Boolean restore_Titlebar)
	/*  */
	/* Allows filling title bar with Chroma Color if needed, or  */
	/* Can also Inval the Titlebar if needs redraw */
	/*  */
{	
	GrafPtr			window_Manager_Port;
	WindowPtr		current_Graphics_Window_Ptr;
	BitMap			*destination_Bitmap;
	Safe_Boolean		function_Result;
	Ptr					destination_Byte_Address;
	Ptr					start_Address;
	SignedByte		status_Globals_State;
	Boolean			ports_Swapped;
	Boolean			devices_Swapped;
	Rect				global_Rect;
	Rect 				test_Rect;
	Point				local_point;
	GrafPtr			saved_Port;
	GDHandle			saved_Device;
	CARDINAL_8		*code_Name;
	RGBColor			plain_Flood_Fill_Color;
/* END VAR */


	ports_Swapped = FALSE;
	devices_Swapped = FALSE;

	GetPort(&saved_Port);
	saved_Device = GetGDevice();
	
	function_Result = SAFE_FALSE;
	code_Name = "\pRedraw_Spinning_Window_Completely";

	if ((active_Window_List[MAIN_SPINNING_WINDOW_INDEX] == nil)
		|| ((**active_Window_List[MAIN_SPINNING_WINDOW_INDEX]).structure_Initialized != SAFE_TRUE)) {
		
		goto Error_Handler;
	}
	else {
	
		/*  */
		/* Lock down because structure content addresses are being passed */
		/*  */
		if (Begin_Use_Of_Handle((Handle) active_Window_List[MAIN_SPINNING_WINDOW_INDEX]
									, MOVE_HIGH_REQUIRED
									, &status_Globals_State) != SAFE_TRUE) {
									
			goto Error_Handler;
		}

		current_Graphics_Window_Ptr	= (**active_Window_List[MAIN_SPINNING_WINDOW_INDEX]).status_Window;
		destination_Bitmap = &(current_Graphics_Window_Ptr->portBits);
	}
	
	Set_View_Port(current_Graphics_Window_Ptr);
	ports_Swapped = TRUE;


	if (restore_Titlebar == SAFE_TRUE) {
	/* ===
		window_Rect = (*(*current_Graphics_Window_Ptr).visRgn)->rgnBBox;
		window_Rect.bottom = window_Rect.top + 20;
		RectRgn( (*current_Graphics_Window_Ptr).visRgn, &qd.screenBits.bounds );
		InvalRect( &qd.screenBits.bounds );
	=== */
	
		if (g_Menu_Bar_Obscured != SAFE_TRUE) {
			if (g_Preferred_Monitor_Values_Valid == SAFE_TRUE) {
			
			/* ===
				GetWMgrPort(&window_Manager_Port);
				Set_View_Port(window_Manager_Port);
				test_Rect = qd.screenBits.bounds;
				
				test_Rect.bottom = test_Rect.top + GetMBarHeight() + 8L;
				InvalRect( &test_Rect );
			=== */
				
			}
		}
	}
	else {
		if (g_Menu_Bar_Obscured == SAFE_TRUE) {
			if (g_Preferred_Monitor_Values_Valid == SAFE_TRUE) {
				GetWMgrPort(&window_Manager_Port);
				
				Set_View_Port(window_Manager_Port);
				
				Set_View_Device(g_Preferred_Monitor_Graphics_Device);
				devices_Swapped = TRUE;
				
			/* ===
				Index2Color((g_Chroma_Index_Optimized_Long & 0x00FF), &plain_Flood_Fill_Color);
				RGBForeColor(&plain_Flood_Fill_Color);
				PaintRgn(g_Menu_Bar_Previous_Region);
				RGBForeColor(&g_Shell.library.color_Black);
			=== */
			
				destination_Byte_Address = GetPixBaseAddr(g_Preferred_Monitor_Video_Pix_Map_Handle);
				start_Address = destination_Byte_Address;
				if (destination_Byte_Address != nil) {				
					*(CARDINAL_32 *)&destination_Byte_Address += 
										(CARDINAL_32)((CARDINAL_32)((*g_Preferred_Monitor_Video_Pix_Map_Handle)->rowBytes 
										& (~COLOR_PORT_PORTBITS_CHECK))
										* (CARDINAL_32)(g_Menu_Bar_Previous_Height + 4));
										
					Fill_Large_Buffer(start_Address
								, ((CARDINAL_32)destination_Byte_Address - (CARDINAL_32)start_Address)
								, g_Chroma_Index_Optimized_Long);
				}
				
				
			/* ===
				g_Preferred_Monitor_Video_Pix_Map_Handle
				g_Preferred_Monitor_Max_Rect
				g_Menu_Bar_Previous_Region
			=== */
				;
			
			}
		}
	}


No_Draw_Needed:
Out_Of_Bounds:

	if (End_Use_Of_Handle((Handle) active_Window_List[MAIN_SPINNING_WINDOW_INDEX], status_Globals_State) != SAFE_TRUE) {
		goto Error_Handler;
	}

	function_Result = SAFE_TRUE;
	goto Normal_Function_Exit;

Error_Handler:
	function_Result = SAFE_FALSE;
		
Normal_Function_Exit:

	if (devices_Swapped == TRUE) {
		Set_View_Device(saved_Device);
	}
	
	if (ports_Swapped == TRUE) {
		Set_View_Port(saved_Port);
	}

	return(function_Result);
		
} /* Safe_Boolean	Redraw_Spinning_Window_Title_Bar_If_Possible(Safe_Boolean restore_Titlebar ... */
/*=**************************************************************************=*/


/*=**************************************************************************=*/
Safe_Boolean	Redraw_Spinning_Window_Completely(Boolean actual_System_Update, Boolean partial_Update_Allowed)
	/*  */
	/* Draws whole frame, not just deviance */
	/*  */
{
	Safe_Boolean		function_Result;
	SignedByte		status_Globals_State;
	Rect				global_Rect;
	Point				local_point;
	BitMap			*destination_Bitmap;
	CARDINAL_8		intensity;
	CARDINAL_8		horizontal_Angle;
	CARDINAL_8		vertical_Angle;
	CARDINAL_8		roll_Angle;
	CARDINAL_32	temp_Value;
	CARDINAL_32	measured_Sound_Level;
	Boolean			same_Probable_Frame;
	Point				global_Point;
	Rect				destination_Rect;
	Rect				source_Rect;
	Rect				active_Area_Rect;
	Scrolling_List_Ptr	scroll_Ptr;
	INTEGER_32		control_Index;
	INTEGER_16		item_Number;
	FontInfo			font_Specs;
	WindowPtr		current_Graphics_Window_Ptr;
	Spin_Object_Handle		spin_Object_Handle;
	Boolean					found_Location;
	INTEGER_16		index;
	INTEGER_16		position;
	INTEGER_32		source_Bit_Height;
	INTEGER_32		source_Bit_Width;
	Rect				possibly_Larger_Rect;
	CARDINAL_32	denominator;
	CARDINAL_32	numerator;
	Boolean			mouse_Is_Out_Of_Bounds;
	Rect				current_Rect;
	INTEGER_32		current_Bit_Width;
	INTEGER_32		current_Bit_Height;
	INTEGER_32		true_Deviation_From_Left_Side;
	Point				offset_Point_For_Shield_Cursor;
	Safe_Boolean 	resize_Occured;
	Str255				note_String;
	OSErr				os_Error;
	Rect				temp_Rect;
	Safe_Boolean		refresh_Global_Fast_Copy_Information;
	Boolean			ports_Swapped;
	Boolean			devices_Swapped;
	Boolean			check_Validity_Of_Structures;
	Rect				global_Detination_Rect;
	GrafPtr			saved_Port;
	GDHandle			saved_Device;
	RGBColor			plain_Flood_Fill_Color;
	CARDINAL_32	current_Time;
	CARDINAL_8		*code_Name;
/* END VAR */


	ports_Swapped = FALSE;
	devices_Swapped = FALSE;
	
	if ((actual_System_Update == TRUE) && (g_Spin_File_Importation_Active == SAFE_TRUE)) {
		/*  */
		/* HACK HACK : Avoid update during imporation unless deliberate */
		/*  */
		function_Result = SAFE_TRUE;
		goto Normal_Function_Exit;
	}

	GetPort(&saved_Port);
	saved_Device = GetGDevice();
	
	function_Result = SAFE_FALSE;
	code_Name = "\pRedraw_Spinning_Window_Completely";
	
	/*  */
	/* If confident and fully debugged check_Validity_Of_Structures */
	/* could be set to FALSE */
	/*  */
	check_Validity_Of_Structures = TRUE;

	if ((active_Window_List[MAIN_SPINNING_WINDOW_INDEX] == nil)
		|| ((**active_Window_List[MAIN_SPINNING_WINDOW_INDEX]).structure_Initialized != SAFE_TRUE)) {
		
		SysBeep(20); 
		Assign_String(ns_Shell_Globals.error_Description_String, "\pError. Improperly initialized window structure");
		Assign_String(ns_Shell_Globals.error_Note_String, "\pProgrammer flaw");
		if (actual_System_Update != TRUE) {
			Display_Error_Message(unit_Name, code_Name);
		}
		Setup_Error_Handling();
		goto Error_Handler;
	}
	else {
	
		/*  */
		/* Lock down because structure content addresses are being passed */
		/*  */
		if (Begin_Use_Of_Handle((Handle) active_Window_List[MAIN_SPINNING_WINDOW_INDEX]
									, MOVE_HIGH_REQUIRED
									, &status_Globals_State) != SAFE_TRUE) {
									
			goto Error_Handler;
		}

		current_Graphics_Window_Ptr	= (**active_Window_List[MAIN_SPINNING_WINDOW_INDEX]).status_Window;
		destination_Bitmap = &(current_Graphics_Window_Ptr->portBits);
	}
	
	Set_View_Port(current_Graphics_Window_Ptr);
	ports_Swapped = TRUE;


	found_Location = FALSE;
	
	if (g_Dreaming_Module_Globals_Initialized == SAFE_TRUE) {
		if (dream.spin_Objects.total_Spin_Objects > 0) {
			found_Location = FALSE;
			for (index = 0; ((index < k_MAX_ACTIVE_SPIN_OBJECTS) && (found_Location != TRUE)); index++) {
				if (dream.spin_Objects.spin_Object_Memory_In_Use[index] == MEMORY_INITIALIZED) {
					if ((*dream.spin_Objects.spin_Object[index])->isolated_Port_Drawing_Active == SAFE_TRUE) {
						found_Location = TRUE;
						position = index;	
					}			
				}
			}
		}
	}
			

	if (actual_System_Update == TRUE) {
	
		/*  */
		/* Mark cached information dirty on redraws. */
		/* WARNING : Could be less efficient, but is done for */
		/* sake of moving window and having structures realign */
		/*  */
		g_Last_Image.use_Fast_Copy = SAFE_FALSE;
		
		/*  */
		/* Hack HAck Hack, if window update let shield cursor be reset. */
		/*  */
		g_Cursor_Probably_Still_Hidden = SAFE_FALSE;

		ForeColor(blackColor);
		BackColor(whiteColor);

		if (found_Location == TRUE) {
			spin_Object_Handle = dream.spin_Objects.spin_Object[position];
			
			if ((*spin_Object_Handle)->isolated_Port_Drawing_Active != SAFE_TRUE) {
				Assign_String(&note_String[0], "\pError, the arbitrarily chosen first SpinFile in memory is not meant to be used for direct Spin Window Useage! Program needs enhancement.");
				goto Error_Handler_With_Message;
			}
	
			source_Rect =  (**((*spin_Object_Handle)->port_Info.spin_Object_Port_1.offscreen_Port.color_Port.portPixMap)).bounds;
			destination_Rect = source_Rect;
			current_Rect = ((WindowPeek)(current_Graphics_Window_Ptr))->port.portRect;
			
			Index2Color((g_Chroma_Index_Optimized_Long & 0x00FF), &plain_Flood_Fill_Color);
			RGBBackColor(&plain_Flood_Fill_Color);
			EraseRect(&(current_Rect));
			RGBBackColor(&g_Shell.library.color_White);
			
			current_Bit_Width = (current_Rect).right - (current_Rect).left;
			current_Bit_Height = (current_Rect).bottom - (current_Rect).top;
			
			source_Bit_Width = source_Rect.right - source_Rect.left;
			source_Bit_Height = source_Rect.bottom - source_Rect.top;
			
			/*  */
			/* Compare all directions */
			/*  */
			if ((current_Bit_Height != source_Bit_Height) || (current_Bit_Width != source_Bit_Width)){
				if ((g_Set_Spin_Window_Max_Monitor_Size != SAFE_TRUE) || (g_Set_Spin_Window_Extra_Gigantic != SAFE_TRUE)){
				
					if (Stop_Any_Video_Background_Display() != SAFE_TRUE) {
						/*  */
						/* Ignore problems */
						/*  */
					}
				
					if (Reset_Any_Window_Size_If_Needed(MAIN_SPINNING_WINDOW_INDEX
																, &source_Rect
																, &resize_Occured
																, actual_System_Update) != SAFE_TRUE) {
						/*  */
						/* Ignore problems */
						/*  */
					}
					
					if (g_Set_Spin_Window_Extra_Gigantic == SAFE_TRUE) {
						g_Set_Spin_Window_Extra_Gigantic = SAFE_FALSE;
						(*spin_Object_Handle)->port_Info.onscreen_Offscreen_Pixel_X = 0;
						(*spin_Object_Handle)->port_Info.onscreen_Offscreen_Pixel_Y = 0;
					}
										
					destination_Rect = source_Rect;
				}
				else {
					/*  */
					/* Just ensure that destination rect is where it ought to be */
					/*  */
					
					/*  */
					/* Always! Always! readjust X offset based on true window needs */
					/*  */
					temp_Rect =  ((WindowPeek)(current_Graphics_Window_Ptr))->port.portRect;
					LocalToGlobal(&TOP_LEFT(temp_Rect));
					true_Deviation_From_Left_Side = (temp_Rect.left & 0x0000000F);
					
					/*  */
					/* Recalculate source and destination rects for window update */
					/*  */
					destination_Rect = source_Rect;
					
					/*  */
					/* Always! Always! readjust X offset based on true window needs */
					/*  */

					OffsetRect(&destination_Rect
							, (*spin_Object_Handle)->port_Info.onscreen_Offscreen_Pixel_X - true_Deviation_From_Left_Side
							, (*spin_Object_Handle)->port_Info.onscreen_Offscreen_Pixel_Y);
				}
			}
			else {
				/*  */
				/* Window and offscreen graphics match,  */
				/* Therefore could be cooincidence, but if not then resize now */
				/*  */
				if (g_Set_Spin_Window_Max_Monitor_Size == SAFE_TRUE) {
				
					possibly_Larger_Rect = source_Rect;
					if (Expand_Rect_To_Largest_Reasonable_Size(&possibly_Larger_Rect) != SAFE_TRUE) {
						/*  */
						/* Ignore errors, not reasonable to fail in implementation */
						/*  */
					}
					else {
						/*  */
						/* Compare two structures, should work in ANSI C ? */
						/*  */
						
						if (Rect_Is_Same_Rect(&possibly_Larger_Rect, &source_Rect) != SAFE_TRUE) {
						
							/*  */
							/* Hmmm. Try setting spin window to larger size */
							/*  */
							
							if (Stop_Any_Video_Background_Display() != SAFE_TRUE) {
								/*  */
								/* Ignore problems */
								/*  */
							}

							if (Reset_Any_Window_Size_If_Needed(MAIN_SPINNING_WINDOW_INDEX
																		, &possibly_Larger_Rect
																		, &resize_Occured
																		, actual_System_Update) != SAFE_TRUE) {
								/*  */
								/* Ignore problems */
								/*  */
								Set_View_Port(current_Graphics_Window_Ptr);

								Index2Color((g_Chroma_Index_Optimized_Long & 0x00FF), &plain_Flood_Fill_Color);
								RGBBackColor(&plain_Flood_Fill_Color);
								EraseRect(&(((WindowPeek)(current_Graphics_Window_Ptr))->port.portRect));
								RGBBackColor(&g_Shell.library.color_White);
							}
							else {
								/*  */
								/* Erase damned white pixels of newer resized window */
								/*  */
								Set_View_Port(current_Graphics_Window_Ptr);

								Index2Color((g_Chroma_Index_Optimized_Long & 0x00FF), &plain_Flood_Fill_Color);
								RGBBackColor(&plain_Flood_Fill_Color);
								/*  */
								/* Make Window not clipped and force it to get cleared */
								/*  */
								EraseRect(&(((WindowPeek)(current_Graphics_Window_Ptr))->port.portRect));
								RGBBackColor(&g_Shell.library.color_White);

								/*  */
								/* Set X and Y offsets, remembering to be friendly to cache lines */
								/*  */
								
								source_Rect =  ((WindowPeek)(current_Graphics_Window_Ptr))->port.portRect;
								if ( (*((*spin_Object_Handle)->spin_Info))->max_Bounds.right < source_Rect.right) {
									(*spin_Object_Handle)->port_Info.onscreen_Offscreen_Pixel_X = (source_Rect.right - (*((*spin_Object_Handle)->spin_Info))->max_Bounds.right);
									(*spin_Object_Handle)->port_Info.onscreen_Offscreen_Pixel_X = (*spin_Object_Handle)->port_Info.onscreen_Offscreen_Pixel_X >> 1;
									(*spin_Object_Handle)->port_Info.onscreen_Offscreen_Pixel_X = (*spin_Object_Handle)->port_Info.onscreen_Offscreen_Pixel_X & 0xFFFFFFF0;
								}
								
								if ((*((*spin_Object_Handle)->spin_Info))->max_Bounds.bottom < source_Rect.bottom) {
									(*spin_Object_Handle)->port_Info.onscreen_Offscreen_Pixel_Y = (source_Rect.bottom -  (*((*spin_Object_Handle)->spin_Info))->max_Bounds.bottom);
								}
								
								/*  */
								/* Always! Always! readjust X offset based on true window needs */
								/*  */
								LocalToGlobal(&TOP_LEFT(source_Rect));
								true_Deviation_From_Left_Side = (source_Rect.left & 0x0000000F);
								
								/*  */
								/* Recalculate source and destination rects for window update */
								/*  */
								source_Rect =  (**((*spin_Object_Handle)->port_Info.spin_Object_Port_1.offscreen_Port.color_Port.portPixMap)).bounds;
								destination_Rect = source_Rect;
								
								/*  */
								/* Always! Always! readjust X offset based on true window needs */
								/*  */

								OffsetRect(&destination_Rect
										, (*spin_Object_Handle)->port_Info.onscreen_Offscreen_Pixel_X - true_Deviation_From_Left_Side
										, (*spin_Object_Handle)->port_Info.onscreen_Offscreen_Pixel_Y);
								
								g_Set_Spin_Window_Extra_Gigantic = SAFE_TRUE;
							}
						}
					}
				}
			}

			CopyBits ((const BitMap *)*(((*spin_Object_Handle)->port_Info.spin_Object_Port_1).offscreen_Port.color_Port.portPixMap)
						, &(current_Graphics_Window_Ptr->portBits)
						, &source_Rect
						, &destination_Rect
						, srcCopy
						, nil);
					
		}
		else {
		
			ForeColor(blackColor);
			BackColor(whiteColor);
		/* ===
			BackPat(&(qd.black));
		=== */
			Index2Color((g_Chroma_Index_Optimized_Long & 0x00FF), &plain_Flood_Fill_Color);
			RGBBackColor(&plain_Flood_Fill_Color);
			EraseRect(&(current_Graphics_Window_Ptr->portRect));
			RGBBackColor(&g_Shell.library.color_White);
		}
	}
	else {
	
		
		if (found_Location == TRUE) {

			GetMouse(&local_point);
			
			global_Rect = (**((WindowPeek)(**active_Window_List[MAIN_SPINNING_WINDOW_INDEX]).status_Window)->contRgn).rgnBBox;
			
			global_Point = local_point;
			LocalToGlobal(&global_Point);

			if (PtInRect(global_Point, &global_Rect) != TRUE) {
				mouse_Is_Out_Of_Bounds = TRUE;
				
				/* ===
					goto Out_Of_Bounds;
				=== */
			}
			else {
				mouse_Is_Out_Of_Bounds = FALSE;
			}

			spin_Object_Handle = dream.spin_Objects.spin_Object[position];
			
			ForeColor(blackColor);
			BackColor(whiteColor);

			if (partial_Update_Allowed == FALSE) {
				Index2Color((g_Chroma_Index_Optimized_Long & 0x00FF), &plain_Flood_Fill_Color);
				RGBBackColor(&plain_Flood_Fill_Color);
				EraseRect(&(current_Graphics_Window_Ptr->portRect));
				RGBBackColor(&g_Shell.library.color_White);
			}
			
			if ((*spin_Object_Handle)->initialized == SAFE_TRUE) {
				if ((*spin_Object_Handle)->spin_Object_Remaining_To_Load != SAFE_TRUE) {
									
																
					if (g_Last_Image.valid_Information == SAFE_TRUE) {
						if (g_Last_Image.last_Spinfile_Index != position) {
							g_Last_Image.valid_Information = SAFE_FALSE;
						}
					}
					
					if ((*spin_Object_Handle)->isolated_Port_Drawing_Active != SAFE_TRUE) {
						Assign_String(&note_String[0], "\pError, the arbitrarily chosen first SpinFile in memory is not meant to be used for direct Spin Window Useage! Program needs enhancement.");
						goto Error_Handler_With_Message;
					}

					source_Rect =  (**((*spin_Object_Handle)->port_Info.spin_Object_Port_1.offscreen_Port.color_Port.portPixMap)).bounds;
				
					current_Rect = ((WindowPeek)current_Graphics_Window_Ptr)->port.portRect;
					current_Bit_Width = (current_Rect).right - (current_Rect).left;
					current_Bit_Height = (current_Rect).bottom - (current_Rect).top;
					
					source_Bit_Width = source_Rect.right - source_Rect.left;
					source_Bit_Height = source_Rect.bottom - source_Rect.top;
					
					/*  */
					/* Compare all directions */
					/*  */
					if ((current_Bit_Height != source_Bit_Height) || (current_Bit_Width != source_Bit_Width)){
					
						if ((g_Set_Spin_Window_Max_Monitor_Size != SAFE_TRUE) || (g_Set_Spin_Window_Extra_Gigantic != SAFE_TRUE)){
							if (Stop_Any_Video_Background_Display() != SAFE_TRUE) {
								/*  */
								/* Ignore problems */
								/*  */
							}

							if (Reset_Any_Window_Size_If_Needed(MAIN_SPINNING_WINDOW_INDEX
																		, &source_Rect
																		, &resize_Occured
																		, actual_System_Update) != SAFE_TRUE) {
								/*  */
								/* Ignore problems */
								/*  */
							}		
							if (g_Set_Spin_Window_Extra_Gigantic == SAFE_TRUE) {
								g_Set_Spin_Window_Extra_Gigantic = SAFE_FALSE;
								(*spin_Object_Handle)->port_Info.onscreen_Offscreen_Pixel_X = 0;
								(*spin_Object_Handle)->port_Info.onscreen_Offscreen_Pixel_Y = 0;
							}
						}
					}
			
					/*  */
					/* This is the spot!!! This is where intensity is compared to see if */
					/* file is multi-intensity or flat, and proper band is selected. */
					/*  */
					intensity = 0;
					if ((*spin_Object_Handle)->max_Intensity > 0) {
					
						current_Time = (CARDINAL_32)TickCount();
						if (current_Time >= g_Time_Since_Last_Sound_Check + k_TICK_DELAY_BETWEEN_SPIN_SOUND_CHECKS) {
							if (Get_Current_Volume_For_Spinfile(&measured_Sound_Level) != SAFE_TRUE) {
								measured_Sound_Level = 0;
							}
					
							if (measured_Sound_Level != g_Last_Spinning_Sound_Level) {
								g_Last_Spinning_Sound_Level = measured_Sound_Level;
								g_Time_Since_Last_Sound_Change = current_Time;
							}
							g_Time_Since_Last_Sound_Check = current_Time;
						}
						else {
							measured_Sound_Level = g_Last_Spinning_Sound_Level;
						}

						intensity = ((measured_Sound_Level * ((*spin_Object_Handle)->max_Intensity + 1)) >> 8);
					}

					denominator = (CARDINAL_32)(global_Rect.right - global_Rect.left);
					/*  */
					/* The next line ALSO inverts the axis horizontally due to comlaint of coordinate */
					/* system in filming (was camera based not object based, and now */
					/* that multiple objects in a frame was bad idea. Works in base 0.*/
					/*  */

					if (mouse_Is_Out_Of_Bounds == TRUE) {
						if (global_Point.h < global_Rect.left) {
							temp_Value = 1;
						}
						else if (global_Point.h >= global_Rect.right) {
							temp_Value = (*spin_Object_Handle)->max_Horizontal_Angle;
						}
						else {
							numerator = ((CARDINAL_32)local_point.h * (*spin_Object_Handle)->max_Horizontal_Angle);
						
							/*  */
							/* The next line properly rounds (50% = up) without using floating point */
							/*  */
							temp_Value = ((numerator + (denominator >>1)) / denominator);
						}
					}
					else {
						numerator = ((CARDINAL_32)local_point.h * (*spin_Object_Handle)->max_Horizontal_Angle);
					
						/*  */
						/* The next line properly rounds (50% = up) without using floating point */
						/*  */
						temp_Value = ((numerator + (denominator >>1)) / denominator);
					}

					horizontal_Angle = temp_Value;
					
					denominator =  (CARDINAL_32)(global_Rect.bottom - global_Rect.top);
					
					if (mouse_Is_Out_Of_Bounds == TRUE) { 
						if (global_Point.v < global_Rect.top) {
							vertical_Angle =  (*spin_Object_Handle)->max_Vertical_Angle;
						}
						else if (global_Point.v >= global_Rect.bottom) {
							vertical_Angle = 0;
						}
						else {
							/*  */
							/* The next line ALSO inverts the axis horizontally due to comlaint of coordinate */
							/* system in filming (was camera based not object based, and now */
							/* that multiple objects in a frame was bad idea. Works in base 0.*/
							/*  */
							numerator = ((denominator - (CARDINAL_32)local_point.v) * ((*spin_Object_Handle)->max_Vertical_Angle / 5UL));
						
							/*  */
							/* The next line properly rounds (50% = up) without using floating point */
							/*  */
							temp_Value = ((numerator + (denominator >>1)) / denominator);
							
							vertical_Angle = temp_Value;
							vertical_Angle = vertical_Angle * 5UL;
						}
					}
					else {
						/*  */
						/* The next line ALSO inverts the axis horizontally due to comlaint of coordinate */
						/* system in filming (was camera based not object based, and now */
						/* that multiple objects in a frame was bad idea. Works in base 0.*/
						/*  */
						numerator = ((denominator - (CARDINAL_32)local_point.v) * ((*spin_Object_Handle)->max_Vertical_Angle / 5UL));
					
						/*  */
						/* The next line properly rounds (50% = up) without using floating point */
						/*  */
						temp_Value = ((numerator + (denominator >>1)) / denominator);
						
						vertical_Angle = temp_Value;
						vertical_Angle = vertical_Angle * 5UL;
					}
					
					
					
					roll_Angle = 0;
					
					refresh_Global_Fast_Copy_Information = SAFE_FALSE;

					same_Probable_Frame = FALSE;
					if (g_Last_Image.valid_Information != SAFE_TRUE) {
						g_Last_Image.last_Spinfile_Index = position;
						g_Last_Image.avoidance_Image_Index = k_BOGUS_INDEX_FRAME;
						g_Last_Image.valid_Information = SAFE_TRUE;
						g_Last_Image.intensity = intensity;
						g_Last_Image.horizontal_Angle = horizontal_Angle;
						g_Last_Image.vertical_Angle = vertical_Angle;
						g_Last_Image.roll_Angle = roll_Angle;
						
						g_Last_Image.use_Fast_Copy = SAFE_FALSE;
						
						refresh_Global_Fast_Copy_Information = SAFE_TRUE;
					}
					else {
						if (g_Last_Image.last_Spinfile_Index == position) {
							if (g_Last_Image.horizontal_Angle == horizontal_Angle) {
								if (g_Last_Image.vertical_Angle == vertical_Angle) {
									if ((g_Last_Image.roll_Angle == roll_Angle)  && (g_Last_Image.intensity == intensity)){
										same_Probable_Frame = TRUE;
									}
								}
							}
						}
						
						if (same_Probable_Frame != TRUE) {
							g_Last_Image.last_Spinfile_Index = position;
							g_Last_Image.valid_Information = SAFE_TRUE;
							g_Last_Image.intensity = intensity;
							g_Last_Image.horizontal_Angle = horizontal_Angle;
							g_Last_Image.vertical_Angle = vertical_Angle;
							g_Last_Image.roll_Angle = roll_Angle;
						}
						else {
							goto No_Draw_Needed;
						}
					} /* if (g_Last_Image.valid_Information != SAFE_TRUE)  */
					
					
					
					if ((*spin_Object_Handle)->file_Reference == 0) {
						os_Error = HOpen((*spin_Object_Handle)->spin_Object_Location.vRefNum
														, (*spin_Object_Handle)->spin_Object_Location.parID
														, &((*spin_Object_Handle)->spin_Object_Location.name[0])
														, fsRdPerm
														, &((*spin_Object_Handle)->file_Reference));
														
						if (os_Error != noErr) {
							Concat_Many(note_String, 3,"\pFail to open file. HOpen for file ", &((*spin_Object_Handle)->spin_Object_Location.name[0]), "\p ID = ");
							Concat_Long_To_String(note_String, os_Error, NO_LIMIT);
							goto Error_Handler_With_Message;
						}
						else {
						/* ===
							if (Recalculate_File_Volume_Blocks((*spin_Object_Handle)->file_Reference, &(g_Last_Image.file_Blocks)) == SAFE_TRUE) {
							
							}
						==== */
						}
					}
					
					source_Rect =  (**((*spin_Object_Handle)->port_Info.spin_Object_Port_1.offscreen_Port.color_Port.portPixMap)).bounds;
					destination_Rect = source_Rect;
					
					
					InvalRect(&((current_Graphics_Window_Ptr)->portRect));
										

			/*  */
			/* HACK TEST CODE: set to not erase if testing. */
			/*  */
		/* ===
			(*spin_Object_Handle)->port_Info.frame_Needing_Erasure.valid_Information = SAFE_FALSE;
		=== */

			

					if (Draw_Spin_Object_From_File((*spin_Object_Handle)->file_Reference
									, &((*spin_Object_Handle)->port_Info.spin_Object_Port_1)
									, &((*spin_Object_Handle)->port_Info.spin_Object_Port_2)
									, (*spin_Object_Handle)->spin_Info
									, &((*spin_Object_Handle)->port_Info.frame_Needing_Erasure)
									, intensity
									, horizontal_Angle
									, vertical_Angle
									, roll_Angle
									, check_Validity_Of_Structures
									, &(g_Last_Image.avoidance_Image_Index)
									, (*spin_Object_Handle)->port_Info.offset_Offscreen_Pixel_X
									, (*spin_Object_Handle)->port_Info.offset_Offscreen_Pixel_Y) == SAFE_TRUE) {
									
						Set_View_Port(current_Graphics_Window_Ptr);
						
					/* ===
						Set_View_Device(((*spin_Object_Handle)->port_Info.spin_Object_Port_1.offscreen_GDevice_Hdl);
						devices_Swapped = TRUE;
					==== */
		
						InvalRect(&destination_Rect);
						MoveTo(0,0);

						if ((refresh_Global_Fast_Copy_Information == SAFE_TRUE) || (g_Last_Image.use_Fast_Copy != SAFE_TRUE)) {
							if (g_Preferred_Monitor_Values_Valid == SAFE_TRUE) {
							
								/*  */
								/* Build cached structure for video drawing. */
								/* The first 3 values will be force updated each animation frame */
								/* So any representaztive frame is OK */
								/*  */
													
								global_Detination_Rect = destination_Rect;
								LocalToGlobal(&TOP_LEFT(global_Detination_Rect));
								LocalToGlobal(&BOTTOM_RIGHT(global_Detination_Rect));
								
								if (g_Cursor_Probably_Still_Hidden != SAFE_TRUE) {
									offset_Point_For_Shield_Cursor.h = 0;
									offset_Point_For_Shield_Cursor.v = 0;
								
									ShieldCursor(&global_Detination_Rect,offset_Point_For_Shield_Cursor);
									g_Cursor_Probably_Still_Hidden = SAFE_TRUE;
								}
	
								if (g_Set_Spin_Window_Max_Monitor_Size == SAFE_TRUE) {
/* === 
			backdrop_Size_Rect = (**(((*spin_Object_Handle)->port_Info.spin_Object_Port_1).color_Port.portPixMap)).bounds;
			stored_Bit_Depth = (**(((*spin_Object_Handle)->port_Info.spin_Object_Port_1).color_Port.portPixMap)).pixelSize;
	==== */
									active_Area_Rect = (*((*spin_Object_Handle)->spin_Info))->max_Bounds;
									OffsetRect(&active_Area_Rect, global_Detination_Rect.left, global_Detination_Rect.top);
									
									if (g_Set_Spin_Window_Extra_Gigantic == SAFE_TRUE) {
									
										/*  */
										/* Every time we get a possible window movement */
										/* so every time we have to subtract offset */
										/*  */

										temp_Rect = (current_Graphics_Window_Ptr)->portRect;
										LocalToGlobal(&TOP_LEFT(temp_Rect));
										true_Deviation_From_Left_Side = (temp_Rect.left & 0x0000000F);
									/* ===
										OffsetRect(&destination_Rect
													, -(true_Deviation_From_Left_Side)
													, 0);
									==== */

										OffsetRect(&active_Area_Rect
													, (*spin_Object_Handle)->port_Info.onscreen_Offscreen_Pixel_X - true_Deviation_From_Left_Side
													, (*spin_Object_Handle)->port_Info.onscreen_Offscreen_Pixel_Y);
									}
									else {
										OffsetRect(&active_Area_Rect
												, (*spin_Object_Handle)->port_Info.offset_Offscreen_Pixel_X
												, (*spin_Object_Handle)->port_Info.offset_Offscreen_Pixel_Y);
									}
	
									if (Build_Draw_Bits_Raw_Parameters((CARDINAL_32)*(((*spin_Object_Handle)->port_Info.spin_Object_Port_1).image_Handle)
											,((**(((*spin_Object_Handle)->port_Info.spin_Object_Port_1).offscreen_Port.color_Port.portPixMap)).rowBytes & (~COLOR_PORT_PORTBITS_CHECK))
											, (*((*spin_Object_Handle)->spin_Info))->cell_Info[0].compression_Type
											, (*spin_Object_Handle)->port_Info.offset_Offscreen_Pixel_X
											, (*spin_Object_Handle)->port_Info.offset_Offscreen_Pixel_Y
											, g_Preferred_Monitor_Video_Pix_Map_Handle
											, &active_Area_Rect
											, &g_Preferred_Monitor_Max_Rect
											, g_Preferred_Monitor_Depth
											, &g_Last_Image.copy_Options_Rec) == SAFE_TRUE) {
										g_Last_Image.use_Fast_Copy = SAFE_TRUE;
									}
								
								} /* if (g_Set_Spin_Window_Max_Monitor_Size == SAFE_TRUE) */
								else {
									if (Build_Draw_Bits_Raw_Parameters((CARDINAL_32)*(((*spin_Object_Handle)->port_Info.spin_Object_Port_1).image_Handle)
											, (*((*spin_Object_Handle)->spin_Info))->cell_Info[0].picture_Row_Byte_Offset
											, (*((*spin_Object_Handle)->spin_Info))->cell_Info[0].compression_Type
										/* ===
											, (*spin_Object_Handle)->port_Info.offset_Offscreen_Pixel_X
											, (*spin_Object_Handle)->port_Info.offset_Offscreen_Pixel_Y
										==== */
											, 0 
											, 0
											, g_Preferred_Monitor_Video_Pix_Map_Handle
											, &global_Detination_Rect
											, &g_Preferred_Monitor_Max_Rect
											, g_Preferred_Monitor_Depth
											, &g_Last_Image.copy_Options_Rec) == SAFE_TRUE) {
										g_Last_Image.use_Fast_Copy = SAFE_TRUE;
									}
								
								} /* if (g_Set_Spin_Window_Max_Monitor_Size == SAFE_TRUE) */
								
								g_Last_Image.copy_Options_Rec.disable_Interrupts = 1;
							}
						}

						/*  */
						/* Transfer to screen at VBL Time */
						/*  */
						if (g_Preferred_Monitor_Values_Valid == SAFE_TRUE) {
							Wait_For_Next_Video_Monitor_VBL(g_Preferred_Monitor_Slots);
						}
						
						/* ===
							OffsetRect(&source_Rect
								, position_In_Scene_Point->h
								, position_In_Scene_Point->v);
						==== */
							
						if (g_Last_Image.use_Fast_Copy != SAFE_TRUE) {
						
							if (g_Set_Spin_Window_Extra_Gigantic == SAFE_TRUE) {
								destination_Rect.left += (*spin_Object_Handle)->port_Info.onscreen_Offscreen_Pixel_X;
								destination_Rect.right += (*spin_Object_Handle)->port_Info.onscreen_Offscreen_Pixel_X;
								destination_Rect.top += (*spin_Object_Handle)->port_Info.onscreen_Offscreen_Pixel_Y;
								destination_Rect.bottom += (*spin_Object_Handle)->port_Info.onscreen_Offscreen_Pixel_Y;
								CopyBits ((const BitMap *)*(((*spin_Object_Handle)->port_Info.spin_Object_Port_1).offscreen_Port.color_Port.portPixMap)
											, destination_Bitmap
											, &source_Rect
											, &destination_Rect
											, srcCopy
											, nil);
							}
							else {
								CopyBits ((const BitMap *)*(((*spin_Object_Handle)->port_Info.spin_Object_Port_1).offscreen_Port.color_Port.portPixMap)
											, destination_Bitmap
											, &source_Rect
											, &destination_Rect
											, srcCopy
											, nil);
							}

						}
						else {
							g_Last_Image.copy_Options_Rec.compression = (*((*spin_Object_Handle)->spin_Info))->cell_Info[position].compression_Type;
						
						
						/* HACK HACK HACK */
					/* ==
							g_Last_Image.copy_Options_Rec.compression = k_EVEN_ROW_DOUBLE_DRAWING;
						=== */
						
							if (g_Set_Spin_Window_Max_Monitor_Size == SAFE_TRUE) {
								g_Last_Image.copy_Options_Rec.source_Row_Bytes = ((**(((*spin_Object_Handle)->port_Info.spin_Object_Port_1).offscreen_Port.color_Port.portPixMap)).rowBytes & (~COLOR_PORT_PORTBITS_CHECK));
							}
							else {
								g_Last_Image.copy_Options_Rec.source_Row_Bytes = (*((*spin_Object_Handle)->spin_Info))->cell_Info[position].picture_Row_Byte_Offset;
							}
							
							g_Last_Image.copy_Options_Rec.source_Base_Address = (CARDINAL_32)*(((*spin_Object_Handle)->port_Info.spin_Object_Port_1).image_Handle);
							
							
							Draw_Bits_Raw_To_Monitor_Using_Params(&g_Last_Image.copy_Options_Rec);
						}
									

/* =======
	if (g_Preferred_Monitor_Values_Valid == SAFE_TRUE) {
		Draw_Bits_Raw_To_Monitor(CARDINAL_32	source_Base_Address;
				, CARDINAL_32	source_Row_Bytes
				, CARDINAL_32	source_Extra_Offset_X
				, CARDINAL_32	source_Extra_Offset_Y
				, g_Preferred_Monitor_Video_Pix_Map_Handle
				, Rect				*initial_Destination_Rect
				, &g_Preferred_Monitor_Max_Rect
				, g_Preferred_Monitor_Depth)
	}
========= */


						ValidRect(&((current_Graphics_Window_Ptr)->portRect));
						
					}
					else{
						ValidRect(&((current_Graphics_Window_Ptr)->portRect));
					} /* NOT if (Draw_Spin_Object_From_File(g_Last_Image.file_Reference */
				}
			}
			
			
		} /* if (found_Location == TRUE) */
		else {
			ForeColor(blackColor);
			BackColor(whiteColor);
		/* ===
			BackPat(&(qd.black));
		=== */
			
			Index2Color((g_Chroma_Index_Optimized_Long & 0x00FF), &plain_Flood_Fill_Color);
			RGBBackColor(&plain_Flood_Fill_Color);
			EraseRect(&(current_Graphics_Window_Ptr->portRect));
			RGBBackColor(&g_Shell.library.color_White);
		
			TextFont(geneva);
			TextSize(9);
			GetFontInfo(&font_Specs);
			
		} /* if (found_Location != TRUE) */
	
	
	} /* NOT if (actual_System_Update == TRUE) */

No_Draw_Needed:
Out_Of_Bounds:

	if (End_Use_Of_Handle((Handle) active_Window_List[MAIN_SPINNING_WINDOW_INDEX], status_Globals_State) != SAFE_TRUE) {
		goto Error_Handler;
	}

	function_Result = SAFE_TRUE;
	goto Normal_Function_Exit;

Error_Handler_With_Message:
	function_Result = SAFE_FALSE;
	if (actual_System_Update != TRUE) {
		Display_Serious_Error_Message(unit_Name, code_Name, note_String, "\p");
	}
	goto Normal_Function_Exit;

Error_Handler:
	function_Result = SAFE_FALSE;
		
Normal_Function_Exit:

	if (devices_Swapped == TRUE) {
		Set_View_Device(saved_Device);
	}
	
	if (ports_Swapped == TRUE) {
		Set_View_Port(saved_Port);
	}

	return(function_Result);
		
} /* Safe_Boolean	Redraw_Spinning_Window_Completely ... */
/*=**************************************************************************=*/


/*=**************************************************************************=*/
Safe_Boolean	Recompress_Spin_Object_BASE(Recompress_Spin_Object_Rec	*params_For_Recompress_Spin_Object)
	/*  */
	/* DORMANT ROUTINE FOR EXPORTING MOVIE INTO PICTS AGAIN */ 
	/* Draws whole frame, not just deviance */
	/*  */
{
	Safe_Boolean		function_Result;
/* END VAR */

	/*  */
	/* Modify routine to convert pictures when displaying them?? */
	/*  */
	
	if (Benchmark_Spinning_Window_Completely(FALSE, FALSE) != SAFE_TRUE) {
		/*  */
		/* Ignore errors */
		/*  */
	}
		
} /* Safe_Boolean	Convert_Spin_File_Frames ... */
/*=**************************************************************************=*/


/*=**************************************************************************=*/
Safe_Boolean Process_Periodic_Async_Spin_Tasks(void)
{
	Safe_Boolean		function_Result;
	CARDINAL_32	item_Number;
	Scrolling_List_Ptr	scroll_Ptr;
	CARDINAL_32	command;
	INTEGER_16		loop;
	KeyMap			*key_Map_Ptr;
	CARDINAL_8		key_Map[16];
	CARDINAL_16	keyboard_Scan_Codes[9];
	INTEGER_16		total_Codes;
	Boolean			a_Key_Found;
/* END VAR */


	function_Result = SAFE_FALSE;

	if ((active_Window_List[MAIN_SPINNING_WINDOW_INDEX] == nil)
		|| ((**active_Window_List[MAIN_SPINNING_WINDOW_INDEX]).structure_Initialized != SAFE_TRUE)) {
		goto Error_Handler;
	}

	if (dream.spin_Objects.total_Spin_Objects > 0) {
			
		
		if (Check_Keyboard_For_Key(CONTROL_KEYBOARD_CODE) == SAFE_TRUE) {
			if (Check_Keyboard_For_Key(SHIFT_OR_LEFT_SHIFT_KEYBOARD_CODE) == SAFE_TRUE) {
			
				function_Result = FALSE;
				key_Map_Ptr = (KeyMap *)(&key_Map[0]);
				GetKeys(*key_Map_Ptr);
					
				total_Codes = 7;
				keyboard_Scan_Codes[0] = k_KEYCODE_ASCII_A;
				keyboard_Scan_Codes[1] = k_KEYCODE_ASCII_S;
				keyboard_Scan_Codes[2] = k_KEYCODE_ASCII_D;
				keyboard_Scan_Codes[3] = k_KEYCODE_ASCII_F;
				keyboard_Scan_Codes[4] = k_KEYCODE_ASCII_G;
				keyboard_Scan_Codes[5] = k_KEYCODE_ASCII_H;
				keyboard_Scan_Codes[6] = k_KEYCODE_ASCII_I;
				
				a_Key_Found = FALSE;
				
				for (loop = 0; ((loop < total_Codes) && (a_Key_Found != TRUE)); loop++) {
					if ((( key_Map[keyboard_Scan_Codes[loop]>>3] >> (keyboard_Scan_Codes[loop] & 7) ) & 1) != 0) {
						a_Key_Found = TRUE;
						command = loop;
					}
				}
				
				if (a_Key_Found == TRUE) {
				
					if (g_Program_Message_Busy != SAFE_TRUE) {
						g_Program_Message_Busy = SAFE_TRUE;
						g_Program_Message_New_Message = SAFE_TRUE;
						g_Program_Message = command;
					}
				}
			}
		}
		
		

		/*  */
		/* This scheduler should be in a good location */
		/*  */
		if (g_Program_Message_Busy == SAFE_TRUE) {
			if (g_Program_Message_New_Message == SAFE_TRUE) {
				g_Program_Message_New_Message = SAFE_FALSE;
				switch(g_Program_Message) {
				
					case k_MESSAGE_0_PLAY_MOVIE:
						g_Loop_Movie_Forever = SAFE_TRUE;
						if (Benchmark_Spinning_Window_Completely(TRUE, FALSE) != SAFE_TRUE) {
						
						}
						
						 /* New idea */
						g_Interactive_Motion_Disabled = SAFE_FALSE;
						
						g_Loop_Movie_Forever = SAFE_FALSE;
						
						g_Program_Message_Busy = SAFE_FALSE;
						break;
					
					case k_MESSAGE_1_STOP_MOVIE:
						g_Interactive_Motion_Disabled = SAFE_TRUE;

						g_Program_Message_Busy = SAFE_FALSE;
						break;
					
					case k_MESSAGE_2_START_INTERACTIVE:
						g_Interactive_Motion_Disabled = SAFE_FALSE;

						g_Program_Message_Busy = SAFE_FALSE;
						break;
					
					case k_MESSAGE_3_STOP_INTERACTIVE:
						g_Interactive_Motion_Disabled = SAFE_TRUE;

						g_Program_Message_Busy = SAFE_FALSE;
						break;
						
						
					case k_MESSAGE_4_PLAY_MOVIE_NO_LOOP:
						g_Loop_Movie_Forever = SAFE_FALSE;
						if (Benchmark_Spinning_Window_Completely(TRUE, FALSE) != SAFE_TRUE) {
						
						}
						
						 /* New idea */
						g_Interactive_Motion_Disabled = SAFE_FALSE;
						
						g_Program_Message_Busy = SAFE_FALSE;
						break;
					
					
					case k_MESSAGE_5_PLAY_MOVIE_FROM_0_NO_LOOP:
						g_Loop_Movie_Forever = SAFE_FALSE;
						g_Last_Movie_Frame_Paused = 0;
						if (Benchmark_Spinning_Window_Completely(TRUE, FALSE) != SAFE_TRUE) {
						
						}
						g_Last_Movie_Frame_Paused = 0;
						g_Interactive_Motion_Disabled = SAFE_TRUE;
						
						g_Program_Message_Busy = SAFE_FALSE;
						break;
					
					case k_MESSAGE_6_PLAY_BACK_INTERACTIVE:
						g_Program_Message_Busy = SAFE_FALSE;
						break;
					
					default:
						g_Program_Message_Busy = SAFE_FALSE;
					
				} /* switch(g_Program_Message) */
			}
		} /* if (g_Program_Message_Busy == SAFE_TRUE) */
	
	} /* if (dream.spin_Objects.total_Spin_Objects > 0)  */

	function_Result = SAFE_TRUE;
	goto Normal_Function_Exit;

Error_Handler:
	function_Result = SAFE_FALSE;
		
Normal_Function_Exit:
	return(function_Result);
		
} /* Safe_Boolean	Process_Periodic_Async_Spin_Tasks ... */
/*=**************************************************************************=*/


/*=**************************************************************************=*/
/* END OF FILE: Spin_O_Rama.c */
