/* START OF FILE: Spin_O_Rama_Cache.c */


/*=**************************************************************************=*/
/* [---BEGIN---] FILE HEADER COMMENTS */
/**
 ** SCREEN VIEW = [Adobe Syntax Black-10, Tabs = 4]
 ** PRINTER VIEW = [Adobe Syntax Black-10, Tabs = 4]
 ** =======================================================================
 ** Written by James Merkle
 ** Spin_O_Rama_Cache.c - Routines for Spin file display caching, FIFO simple style (ring buffer)
 ** (C) Copyright 1995 James Merkle
 ** All rights reserved, except those wishing to "copy" moderate sections of code or files may do so, so long as
 ** such code is never used directly either in animation-related tools or multimedia tools that are  
 ** used in live performance or live broadcast or recorded mass market broadcast or film. Desktop usage not restricted.
 ** Conventional desktop game or entertainment not restricted.
 ** =======================================================================
 **
 **
 ** ============
 ** Revision History
 ** ============
 **
 **  1995.01.09		(James Merkle)
 **						Initial coding started
 **
 **
 **
 ** =================
 ** OTHER INFORMATION
 ** =================
 **
 ** This ring buffer has occlusion of older elements as needed.
 **
 **/
/* [----END----] FILE HEADER COMMENTS */
/*=**************************************************************************=*/


/*=**************************************************************************=*/
/* [---BEGIN---] #include AREA */

#include "Error_Utilities.h"
#include "Memory_Utilities.h"

#include "Shell_Dialogs.h"
#include "Interface_Utilities.h"
#include "Ring_Buffer_Cache.h"
#include "Ring_Buffer_Cache_Setup.h"

#include "Spin_O_Rama_Cache.h"

/* [----END----] #include AREA */
/*=**************************************************************************=*/


/*=**************************************************************************=*/
/* [---BEGIN---] #define AREA */

#define k_MODERATELY_ACCEPTABLE_CACHE				(7000000L)
#define k_MAXIMUM_ACCEPTABLE_CACHE					(100UL *1024UL * 1024L)
#define k_MINIMUM_ACCEPTABLE_CACHE					(600UL * 1024L)

/* HACK HACK HACK HACK */
/* HACK HACK HACK HACK */
/* HACK HACK HACK HACK */
/*  */
/* This is best if set to only a meg if no scenes are used, but if scenes are used */
/* it is important to have 3 buffers of about a meg each and they are not */
/* created until after this is created, se leave some more space! */
/* Those buffers are about 3 * the normal screen view and are */
/* again multiplied by 3 of them. 9 * 640 * 480 = 2764800 bytes. An additional */
/* 900K is added to that amount to allow a bare bones scene and backdrop to */
/* load. That total is "3686400" or 3600K. */
/*  */
#define k_MINIMUM_EXTRA_APP_RAM_REMAINING		(3600L * 1024L)

#define k_MINIMUM_EXTRA_TEMP_RAM_REMAINING		(280L * 1024L)

/* [----END----] #define AREA */
/*=**************************************************************************=*/


/*=**************************************************************************=*/
/* [---BEGIN---] TYPE AREA */


/* [----END----] TYPE AREA */
/*=**************************************************************************=*/


/*=**************************************************************************=*/
/* [---BEGIN---] PROTOTYPE AREA */
					

/* [----END----] PROTOTYPE AREA */
/*=**************************************************************************=*/


/*=**************************************************************************=*/
/* [---BEGIN---] GLOBAL DEFINITION AREA */

Safe_Boolean					g_Spin_Ring_Buffer_Initialized;
Ring_Buffer_Cache_Ptr		g_Spin_Cache_Master_Ptr;

/* [----END----] GLOBAL DEFINITION AREA */
/*=**************************************************************************=*/


/*=**************************************************************************=*/
/* [---BEGIN---] LOCAL STATIC DEFINITION AREA */

static CARDINAL_8			*unit_Name;

Ring_Buffer_Cache_Hdl			g_Spin_Cache_Master_Handle;
Memory_Allocation				g_Spin_Cache_Master_Memory_Initialized;

/* [----END----] LOCAL STATIC DEFINITION AREA */
/*=**************************************************************************=*/


/*=**************************************************************************=*/
Safe_Boolean Initialize_Spin_O_Rama_Cache(Safe_Boolean use_Temporary_Memory)
{
	CARDINAL_8			*static_Unit_Name = "\pSpin_O_Rama_Cache";
	SignedByte 			original_State;
	Safe_Boolean			function_Result;
	CARDINAL_8			*code_Name;
/* END VAR */


	unit_Name = static_Unit_Name;
	code_Name = "\pInitialize_Spin_O_Rama_Cache";
	
/*  */
/* Global persistant structures are created here */
/*  */
	g_Spin_Cache_Master_Handle = nil;
	g_Spin_Cache_Master_Memory_Initialized = MEMORY_NOT_INITIALIZED;
	g_Spin_Cache_Master_Ptr = nil;
	
	g_Spin_Ring_Buffer_Initialized = SAFE_FALSE;

	if (Create_New_Ring_Cache(&g_Spin_Cache_Master_Handle
						, &g_Spin_Cache_Master_Memory_Initialized
						, use_Temporary_Memory) != SAFE_TRUE) {
		goto Error_Handler;
	}

	if (Begin_Use_Of_Handle((Handle)g_Spin_Cache_Master_Handle
									, MOVE_HIGH_REQUIRED
									, &original_State) != SAFE_TRUE) {
		goto Error_Handler;
	}
	
	/*  */
	/* Handle kept around and is locked for duration of usage */
	/* so a global pointer is relied on for speed */
	/* Also data is written into buffer and read from buffer at interrupt level */
	/* asyncronously */
	/*  */
	g_Spin_Cache_Master_Ptr = *g_Spin_Cache_Master_Handle;

	g_Spin_Ring_Buffer_Initialized = SAFE_TRUE;
	
	function_Result = SAFE_TRUE;
	goto Normal_Function_Exit;
	
Error_Handler:
	function_Result = SAFE_FALSE;
		
Normal_Function_Exit:
	return(function_Result);
	
} /* Safe_Boolean Initialize_Spin_O_Rama_Cache(Safe_Boolean use_Temporary_Memory) */
/*=**************************************************************************=*/


/*=**************************************************************************=*/
Safe_Boolean Reset_And_Open_Spin_Cache_If_Needed(void)
{
	Safe_Boolean			function_Result;
	CARDINAL_8			*code_Name;
/* END VAR */


	code_Name = "\pReset_And_Open_Spin_Cache_If_Needed";

	function_Result = SAFE_FALSE;
	if ((g_Spin_Ring_Buffer_Initialized == SAFE_TRUE) && (g_Spin_Cache_Master_Ptr->actively_Caching != SAFE_TRUE)) {
	
		if (Reset_And_Open_Ring_Buffer_If_Needed(g_Spin_Cache_Master_Ptr
														, k_MINIMUM_EXTRA_TEMP_RAM_REMAINING
														, k_MAXIMUM_ACCEPTABLE_CACHE
														, k_MINIMUM_ACCEPTABLE_CACHE
														, k_MODERATELY_ACCEPTABLE_CACHE
														, k_MINIMUM_EXTRA_APP_RAM_REMAINING) != SAFE_TRUE) {
			goto Error_Handler;
		}
		
	} /* if ((g_Spin_Ring_Buffer_Initialized == SAFE_TRUE) ... */

	function_Result = SAFE_TRUE;
	goto Normal_Function_Exit;
	
Error_Handler:

	Display_Serious_Error_Message(unit_Name
								, code_Name
								, "\pProblem creating large disk RAM cache"
								, "\pCaching is not activated, yet is vital for program to function. Aborting.");
								
	function_Result = SAFE_FALSE;
		
Normal_Function_Exit:

	return(function_Result);
	
} /* Safe_Boolean Reset_And_Open_Spin_Cache_If_Needed ... */
/*=**************************************************************************=*/


/*=**************************************************************************=*/
Safe_Boolean Destroy_Spin_O_Rama_Cache(void)
{
	Safe_Boolean		function_Result;
	OSErr				os_Error;
	CARDINAL_8			*code_Name;
/* END VAR */


	code_Name = "\pDestroy_Spin_O_Rama_Cache";

	/*  */
	/* Global persistant structures are deleted here */
	/*  */	
	function_Result = SAFE_TRUE;
	
	if (g_Spin_Ring_Buffer_Initialized == SAFE_TRUE) {

		if (Destroy_Ring_Buffer_Cache(&g_Spin_Cache_Master_Handle, &g_Spin_Cache_Master_Memory_Initialized) != SAFE_TRUE) {
			Display_Serious_Error_Message(unit_Name, code_Name, "\pError with Destroy_Handle", "\pSystem Corrupted?");
			function_Result = SAFE_FALSE;
		}
		g_Spin_Cache_Master_Ptr = nil;
		
		g_Spin_Ring_Buffer_Initialized = SAFE_FALSE;
	}

	return(function_Result);
	
} /* Safe_Boolean Destroy_Spin_O_Rama_Cache ... */
/*=**************************************************************************=*/


/*=**************************************************************************=*/
/* END OF FILE: Spin_O_Rama_Cache.c */
