/* START OF FILE: Spin_O_Rama_Files.c */


/*=**************************************************************************=*/
/* [---BEGIN---] FILE HEADER COMMENTS */
/**
 ** SCREEN VIEW = [Adobe Syntax Black-10, Tabs = 4]
 ** PRINTER VIEW = [Adobe Syntax Black-10, Tabs = 4]
 ** =======================================================================
 ** Written by James Merkle
 ** Spin_O_Rama_Files.c - FIle management routines to load or convert Spin_Object PICTs
 ** (C) Copyright 1994 James Merkle
 ** All rights reserved, except those wishing to "copy" moderate sections of code or files may do so, so long as
 ** such code is never used directly either in animation-related tools or multimedia tools that are  
 ** used in live performance or live broadcast or recorded mass market broadcast or film. Desktop usage not restricted.
 ** Conventional desktop game or entertainment not restricted.
 ** =======================================================================
 **
 **
 ** ============
 ** Revision History
 ** ============
 **
 **  1994.02.01		(James Merkle)
 **						Initial coding started
 **
 **
 **
 **
 ** =================
 ** OTHER INFORMATION
 ** =================
 **  
 **
 **/
/* [----END----] FILE HEADER COMMENTS */
/*=**************************************************************************=*/


/*=**************************************************************************=*/
/* [---BEGIN---] #include AREA */

#include "Core_Universal.h" /* Non_Specific_Support_Routines */
#include "Core_Specific.h"
#include "Core_Globals.h"
#include "String_Utilities.h"
#include "Error_Utilities.h"
#include "File_Resource_Utilities.h"
#include "Shell_Dialogs.h"
#include "Shell_Assembly.h"
#include "Shell_Globals.h"
#include "Volume_Utilities.h"
#include "Window_Environment.h"
#include "SysHooks.h"
#include "Cursor_Utilities.h"
#include "Dreaming_Scene_Files.h"
#include "Interface_Utilities.h"
#include "Buffered_File_Routines.h"
#include "Copy_Compacted_Picture.h"

#include "Spin_O_Rama_Load_Import.h"
#include "Copy_Compacted_Silloette.h"

#include "Ring_Buffer_Cache.h"
#include "Video_Backdrop_Background.h"

#include "Dreaming_Stuff.h"
#include "Dreaming_Color_Stuff.h"
#include "Dreaming_File_Loading.h"

#include "Simple_Ring_Cache.h"

#include "Spin_O_Rama.h"
#include "Spin_O_Rama_Cache.h"
#include "Spin_O_Rama_Files.h"

/* [----END----] #include AREA */
/*=**************************************************************************=*/


/*=**************************************************************************=*/
/* [---BEGIN---] #define AREA */

#define	k_EXTRA_PAD_AMOUNT								8

#define COLOR_PORT_PORTBITS_CHECK 		0xC000

#define	k_SPIN_OBJECT_HEADER_TEMPLATE_NAME			"\pSpin"
#define	k_SPIN_OBJECT_HEADER_TEMPLATE_ID					3007

/*  */
/* Only allow 2 * 128 bits, or 32 bytes */
/* for all file versions. For k_NO_COMPRESSION, will just result in */
/*a little waste for some frame sizes close to 32K modulo. Otherwise */
/* this are is vital for treble safe flags indicating read status */
/* in the k_EVEN_ODD_ROW_COMPRESSION technology */
/* They are placed after AFTER total sizes stated in the structure. */
/*  */
#define	k_COMPRESSION_STRUCTURE_PADDING					(32L)

#define k_SEARCH_BUFFER_SIZE		(16384L)
#define k_MAXIMUM_FILE_MATCHES		10

#define	k_SPIN_OBJECT_PALLETE_RESOURCE_BASE		128
#define	k_SPIN_OBJECT_CLUT_RESOURCE_BASE			128

/*  */
/* Start all file structures and cells on 32K chunk boundaries for */
/* array parallel speed */
/*  */
#define	k_DISK_MAJOR_DIVISION			(32UL * 1024UL)
#define	k_DISK_END_FRAME_DIVISION	(2UL * 1024UL)

#define	k_BASE_REGION_RESOURCE_ID		k_BASE_SPIN_OBJECT_RESOURCE_ID

/*  */
/* For this version, total items is the same as the RAM-based limit */
/*  */
#define 	k_MAX_FILE_STAGE_ITEMS		k_MAX_STAGE_ITEMS

/*  */
/* There are no stages in loading the spin_Objects in this version, but the */
/* code is ready */
/*  */
#define	k_LAST_BACKGROUND_STAGE		1

#define	k_MAXIMUM_DEVIATION_POSSIBLE_PLUS_ONE	(0x7FFFFFFFUL)

/* [----END----] #define AREA */
/*=**************************************************************************=*/


/*=**************************************************************************=*/
/* [---BEGIN---] TYPE AREA */


/* [----END----] TYPE AREA */
/*=**************************************************************************=*/


/*=**************************************************************************=*/
/* [---BEGIN---] PROTOTYPE AREA */
																																	
Boolean Locate_Frame(Spin_Pict_Info_Handle	spin_Info
								, CARDINAL_8 intensity
								, CARDINAL_8 horizontal_Angle
								, CARDINAL_8 vertical_Angle
								, CARDINAL_8 roll_Angle
								, CARDINAL_32 *index);

Boolean Locate_Best_Frame(Spin_Pict_Info_Handle	spin_Info
								, CARDINAL_8 intensity
								, CARDINAL_8 horizontal_Angle
								, CARDINAL_8 vertical_Angle
								, CARDINAL_8 roll_Angle
								, CARDINAL_32 *index);

Safe_Boolean	Initialize_Empty_Spin_Object(CARDINAL_32 *located_Position);

/* [----END----] PROTOTYPE AREA */
/*=**************************************************************************=*/


/*=**************************************************************************=*/
/* [---BEGIN---] GLOBAL DEFINITION AREA */


/* [----END----] GLOBAL DEFINITION AREA */
/*=**************************************************************************=*/


/*=**************************************************************************=*/
/* [---BEGIN---] LOCAL STATIC DEFINITION AREA */

static CARDINAL_8	*unit_Name;
Safe_Boolean				g_Cache_Misalign_Warning_Issued;

/* [----END----] LOCAL STATIC DEFINITION AREA */
/*=**************************************************************************=*/


/*=**************************************************************************=*/
Safe_Boolean Initialize_Spin_Object_Files(void)
{
	CARDINAL_8			*static_Unit_Name = "\pSpin_Object_Files";
/* END VAR */

/*  */
/* Global persistant structures are created here */
/*  */

	g_Cache_Misalign_Warning_Issued = SAFE_FALSE;
	
	unit_Name = static_Unit_Name;
	return(SAFE_TRUE);
}
/*=**************************************************************************=*/


/*=**************************************************************************=*/
Safe_Boolean Destroy_Spin_Object_Files(void)
{
	Safe_Boolean		function_Result;
/* END VAR */

/*  */
/* Global persistant structures are deleted here */
/*  */

	function_Result = SAFE_TRUE;
	return(function_Result);
	
}
/*=**************************************************************************=*/		

		
/*=**************************************************************************=*/		
Safe_Boolean	Draw_Spin_Object_From_Closed_File(FSSpec *desired_File
								, Off_Screen_Port_Rec *port_Ptr
								, Off_Screen_Port_Rec *optional_Background_Port_Ptr
								, Spin_Pict_Info_Handle	spin_Info
								, Previously_Drawn_Frame_Ptr	frame_To_Erase_First
								, CARDINAL_8 intensity
								, CARDINAL_8 horizontal_Angle
								, CARDINAL_8 vertical_Angle
								, CARDINAL_8 roll_Angle
								, INTEGER_32		offset_Offscreen_Pixel_X
								, INTEGER_32		offset_Offscreen_Pixel_Y)
{
	Safe_Boolean		function_Result;
	INTEGER_16		picture_File_Reference;
	OSErr					os_Error;
	CARDINAL_8		*code_Name;
	Str255					note_String;
	GrafPtr				saved_Original_Port;
	CARDINAL_32 	avoidance_Image_Index;
/* END VAR */


	code_Name = "\pDraw_Spin_Object_From_Closed_File";
	GetPort(&saved_Original_Port);
	Set_View_Port((GrafPtr)&(port_Ptr->offscreen_Port.color_Port));


	/*  */
	/* Now open data fork and fill it with our structure */
	/* The data fork is used because we might want to read */
	/* portions in during interrupts, as a background task, or */
	/* because we need to load it into memory in a specific */
	/* spot and there may not be enough room for two copies */
	/* to exchange from a loaded monster resource. It is true */
	/* that system 7 allows partial resource loading, but some */
	/* resource compressors, and other things fail when these are used. */
	/*  */									
	os_Error = HOpen((*desired_File).vRefNum
									, (*desired_File).parID
									, &((*desired_File).name[0])
									, fsRdPerm
									, &picture_File_Reference);
									
	if (os_Error != noErr) {
		Concat_Many(note_String, 3,"\pHOpen for file ", &((*desired_File).name[0]), "\p ID = ");
		Concat_Long_To_String(note_String, os_Error, NO_LIMIT);
		goto Error_Handler_With_Message;
	} 

	avoidance_Image_Index = k_BOGUS_INDEX_FRAME;
	function_Result = 	Draw_Spin_Object_From_File(picture_File_Reference
								, port_Ptr
								, optional_Background_Port_Ptr
								, spin_Info
								, frame_To_Erase_First
								, intensity
								, horizontal_Angle
								, vertical_Angle
								, roll_Angle
								, TRUE
								, &avoidance_Image_Index
								, offset_Offscreen_Pixel_X
								, offset_Offscreen_Pixel_Y);
								
	os_Error = FSClose(picture_File_Reference);
	if ( os_Error != noErr) {
		/*  */
		/* Ignore, this was just an abortion tactic anyway */
		/*  */
	}
	
	goto Normal_Function_Exit;

Error_Handler_With_Message:
	function_Result = SAFE_FALSE;
	Display_Serious_Error_Message(unit_Name, code_Name, note_String, "\p");
	goto Normal_Function_Exit;

Error_Handler:
	Display_Serious_Error_Message(unit_Name, code_Name, "\pInternal problem drawing spin_Object", "\p");
	function_Result = SAFE_FALSE;
		
Normal_Function_Exit:
					
	Set_View_Port(saved_Original_Port);
	return(function_Result);
	
} /* Safe_Boolean	Draw_Spin_Object_From_Closed_File... */
/*=**************************************************************************=*/		

		
/*=**************************************************************************=*/		
Boolean Locate_Frame(Spin_Pict_Info_Handle	spin_Info
								, CARDINAL_8 intensity
								, CARDINAL_8 horizontal_Angle
								, CARDINAL_8 vertical_Angle
								, CARDINAL_8 roll_Angle
								, CARDINAL_32 *index)
{
	Boolean							function_Result;
	Spin_Pict_Info_Ptr			spin;
	CARDINAL_32				last_Index;
	CARDINAL_32				loop;


	spin = (*spin_Info);
	*index = 0;
	last_Index = spin->array_Actual_Count;
	for (loop = 0; (loop < last_Index); loop++) {
		if (spin->cell_Info[loop].vertical_Angle == vertical_Angle) {
			if (spin->cell_Info[loop].horizontal_Angle == horizontal_Angle) {
				if (spin->cell_Info[loop].roll_Angle == roll_Angle) {
					if (spin->cell_Info[loop].activation == intensity) {
						*index = loop;
						break;
					}
				}
			}
		}
	}
	
	if (loop >= last_Index) {
		function_Result = FALSE;
	}
	else {
		function_Result = TRUE;
	}
	
	if (function_Result != TRUE) {
		if (horizontal_Angle == 0) {
			/*  */
			/* Hack Hack Hack : Bug fix for file format that starts at "1" */
			/* instead of "0" for horizontal even though I desire it to start at "0". */
			/* Retry search substituting "1" */
			/*  */
			horizontal_Angle = 1;
			
			spin = (*spin_Info);
			*index = 0;
			last_Index = spin->array_Actual_Count;
			for (loop = 0; (loop < last_Index); loop++) {
				if (spin->cell_Info[loop].vertical_Angle == vertical_Angle) {
					if (spin->cell_Info[loop].horizontal_Angle == horizontal_Angle) {
						if (spin->cell_Info[loop].roll_Angle == roll_Angle) {
							if (spin->cell_Info[loop].activation == intensity) {
								*index = loop;
								break;
							}
						}
					}
				}
			}
			
			if (loop >= last_Index) {
				function_Result = FALSE;
			}
			else {
				function_Result = TRUE;
			}
		}
	}
	
	return(function_Result);
}
/*=**************************************************************************=*/		

		
/*=**************************************************************************=*/		
Boolean Locate_Best_Frame(Spin_Pict_Info_Handle	spin_Info
								, CARDINAL_8 intensity
								, CARDINAL_8 horizontal_Angle
								, CARDINAL_8 vertical_Angle
								, CARDINAL_8 roll_Angle
								, CARDINAL_32 *index)
{
	Spin_Pict_Info_Ptr			spin;
	Spin_Cell_Index				*orientation;
	CARDINAL_32				last_Index;
	CARDINAL_32				loop;
	CARDINAL_32				lowest_Vertical_Deviation_Found;
	CARDINAL_32				best_Vertical_Index_So_Far;
	CARDINAL_32				lowest_Horizontal_Deviation_Found;
	CARDINAL_32				best_Horizontal_Index_So_Far;
	CARDINAL_32				deviation;
	CARDINAL_32				this_Value;
	Boolean							function_Result;
	Boolean							horizontal_Match;
	Boolean							vertical_Match;
/* END VAR */


	horizontal_Match = FALSE;
	vertical_Match = FALSE;

	spin = (*spin_Info);
	*index = 0;
	last_Index = spin->array_Actual_Count;
	for (loop = 0; (loop < last_Index); loop++) {
		if (spin->cell_Info[loop].horizontal_Angle == horizontal_Angle) {
		
			horizontal_Match = TRUE;
			
			if (spin->cell_Info[loop].vertical_Angle == vertical_Angle) {
			
				vertical_Match = TRUE;
			
				if (spin->cell_Info[loop].roll_Angle == roll_Angle) {
					if (spin->cell_Info[loop].activation == intensity) {
						*index = loop;
						break;
					}
				}
			}
		}
	}
	
	if (loop >= last_Index) {
		function_Result = FALSE;
	}
	else {
		function_Result = TRUE;
	}
	
	/* - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - */
	/* - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - */
	
	if (function_Result != TRUE) {
	
		if (horizontal_Angle == 0) {
			/*  */
			/* Technique "1" : */
			/* Bug fix for file format that starts at "1" */
			/* instead of "0" for horizontal even though I desire it to start at "0". */
			/* Retry search substituting "1" */
			/*  */
			horizontal_Angle = 1;
			
			spin = (*spin_Info);
			*index = 0;
			last_Index = spin->array_Actual_Count;
			for (loop = 0; (loop < last_Index); loop++) {
				if (spin->cell_Info[loop].vertical_Angle == vertical_Angle) {
					if (spin->cell_Info[loop].horizontal_Angle == horizontal_Angle) {
						if (spin->cell_Info[loop].roll_Angle == roll_Angle) {
							if (spin->cell_Info[loop].activation == intensity) {
								*index = loop;
								break;
							}
						}
					}
				}
			}
			
			if (loop >= last_Index) {
				function_Result = FALSE;
			}
			else {
				function_Result = TRUE;
			}
		} /* if (horizontal_Angle == 0) */


		if (function_Result != TRUE) {
			/*  */
			/* Technique "2" :*/
			/* If not found and horizontal_Match was made but */
			/* vertical_Match was not then try to optimize for lowest matching */
			/* vertical angle (NOTE: not rounded up for speed) */
			/*  */
			
			if ((horizontal_Match == TRUE) && (vertical_Match != TRUE)) {
				spin = (*spin_Info);
				*index = 0;
				last_Index = spin->array_Actual_Count;
				lowest_Vertical_Deviation_Found = k_MAXIMUM_DEVIATION_POSSIBLE_PLUS_ONE;
				for (loop = 0; (loop < last_Index); loop++) {
					if (spin->cell_Info[loop].horizontal_Angle == horizontal_Angle) {
					
						orientation = &spin->cell_Info[loop];
						
						if (orientation->activation == intensity) {
							if (orientation->roll_Angle == roll_Angle) {
							
								/*  */
								/* Matches criteria so far exept for vertical_Angle */
								/*  */
								
								this_Value = orientation->vertical_Angle;
								
								if (this_Value == vertical_Angle) {
									*index = loop;
									break;
								}
								else {
									 if (this_Value >= vertical_Angle) {
										deviation = this_Value - vertical_Angle;
										if (deviation < lowest_Vertical_Deviation_Found) {
											lowest_Vertical_Deviation_Found = deviation;
											best_Vertical_Index_So_Far = loop;
										}
									}
									else {
										deviation = vertical_Angle -this_Value;
										if (deviation < lowest_Vertical_Deviation_Found) {
											lowest_Vertical_Deviation_Found = deviation;
											best_Vertical_Index_So_Far = loop;
										}
									}
								}
							}
						}
					}
				}
				
				if (loop >= last_Index) {
					/*  */
					/* Take best match, as long as lowest_Vertical_Deviation_Found */
					/* is < k_MAXIMUM_DEVIATION_POSSIBLE_PLUS_ONE */
					/*  */
					if (lowest_Vertical_Deviation_Found < k_MAXIMUM_DEVIATION_POSSIBLE_PLUS_ONE) {
						*index = best_Vertical_Index_So_Far;
						function_Result = TRUE;
					}
				}
				else {
					function_Result = TRUE;
				}
			} /* if ((horizontal_Match == TRUE) && (vertical_Match != TRUE)) */
		
		} /* if (function_Result != TRUE) */
		
	/* - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - */
	/* - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - */
	
		if (function_Result != TRUE) {
			/*  */
			/* Technique "3" :*/
			/* If not found and vertical_Match was made but */
			/* horizontal_Match was not then try to optimize for lowest matching */
			/* horizontal angle (NOTE: not rounded up for speed) */
			/*  */
			
			if ((horizontal_Match != TRUE) && (vertical_Match == TRUE)) {
				spin = (*spin_Info);
				*index = 0;
				last_Index = spin->array_Actual_Count;
				lowest_Horizontal_Deviation_Found = k_MAXIMUM_DEVIATION_POSSIBLE_PLUS_ONE;
				for (loop = 0; (loop < last_Index); loop++) {
					if (spin->cell_Info[loop].vertical_Angle == vertical_Angle) {
					
						orientation = &spin->cell_Info[loop];
						
						if (orientation->activation == intensity) {
							if (orientation->roll_Angle == roll_Angle) {
							
								/*  */
								/* Matches criteria so far exept for horizontal_Angle */
								/*  */
								
								this_Value = orientation->horizontal_Angle;
								
								if (this_Value == horizontal_Angle) {
									*index = loop;
									break;
								}
								else {
									 if (this_Value >= horizontal_Angle) {
										deviation = this_Value - horizontal_Angle;
										if (deviation < lowest_Horizontal_Deviation_Found) {
											lowest_Horizontal_Deviation_Found = deviation;
											best_Horizontal_Index_So_Far = loop;
										}
									}
									else {
										deviation = horizontal_Angle -this_Value;
										if (deviation < lowest_Horizontal_Deviation_Found) {
											lowest_Horizontal_Deviation_Found = deviation;
											best_Horizontal_Index_So_Far = loop;
										}
									}
								}
							}
						}
					}
				}
				
				if (loop >= last_Index) {
					/*  */
					/* Take best match, as long as lowest_Horizontal_Deviation_Found */
					/* is < k_MAXIMUM_DEVIATION_POSSIBLE_PLUS_ONE */
					/*  */
					if (lowest_Horizontal_Deviation_Found < k_MAXIMUM_DEVIATION_POSSIBLE_PLUS_ONE) {
						*index = best_Horizontal_Index_So_Far;
						function_Result = TRUE;
					}
				}
				else {
					function_Result = TRUE;
				}
			} /* if ((horizontal_Match == TRUE) && (vertical_Match != TRUE)) */
		
		} /* if (function_Result != TRUE) */
		
	/* - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - */
	/* - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - */
	
#if (7 ==11)
		if (function_Result != TRUE) {
			/*  */
			/* Technique "4" :*/
			/* If not found and neither vertical_Match nor horizontal_Match */
			/* then try and find closest match possible amoung multiple optimums. */
			/* Note that one of two closest matches will not be taken unless other */
			/* balanced optimums exist. */ 
			/*  */
			
			if ((horizontal_Match != TRUE) && (vertical_Match == TRUE)) {
				spin = (*spin_Info);
				*index = 0;
				last_Index = spin->array_Actual_Count;
				lowest_Horizontal_Deviation_Found = k_MAXIMUM_DEVIATION_POSSIBLE_PLUS_ONE;
				lowest_Vertical_Deviation_Found = k_MAXIMUM_DEVIATION_POSSIBLE_PLUS_ONE;
				for (loop = 0; (loop < last_Index); loop++) {
				
					orientation = &spin->cell_Info[loop];
					
					if (orientation->activation == intensity) {
						if (orientation->roll_Angle == roll_Angle) {
						
							/*  */
							/* Matches criteria so far exept for horizontal_Angle */
							/* and vertical angle. Get the optimums ready. */
							/*  */
							
							this_Value = orientation->horizontal_Angle;
							
							 if (this_Value >= horizontal_Angle) {
								deviation = this_Value - horizontal_Angle;
								if (deviation < lowest_Horizontal_Deviation_Found) {
									lowest_Horizontal_Deviation_Found = deviation;
									best_Horizontal_Index_So_Far = loop;
								}
							}
							else {
								deviation = horizontal_Angle -this_Value;
								if (deviation < lowest_Horizontal_Deviation_Found) {
									lowest_Horizontal_Deviation_Found = deviation;
									best_Horizontal_Index_So_Far = loop;
								}
							}
							
							xxxx
							
						}
					}
				}
				}
				
				if (loop >= last_Index) {
					/*  */
					/* Take best match, as long as lowest_Horizontal_Deviation_Found */
					/* is < k_MAXIMUM_DEVIATION_POSSIBLE_PLUS_ONE */
					/*  */
					if (lowest_Horizontal_Deviation_Found < k_MAXIMUM_DEVIATION_POSSIBLE_PLUS_ONE) {
						*index = best_Horizontal_Index_So_Far;
						function_Result = TRUE;
					}
				}
				else {
					function_Result = TRUE;
				}
			} /* if ((horizontal_Match == TRUE) && (vertical_Match != TRUE)) */
		
		} /* if (function_Result != TRUE) */
		
#endif /* #if (7 ==11) */

	/* - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - */
	/* - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - */
				
	} /* if (function_Result != TRUE) */
	
	return(function_Result);
}
/*=**************************************************************************=*/		

		
/*=**************************************************************************=*/		
void Get_Max_Ranges(Spin_Pict_Info_Handle	spin_Info
								, CARDINAL_32 *intensity
								, CARDINAL_32 *horizontal_Angle
								, CARDINAL_32 *vertical_Angle
								, CARDINAL_32 *roll_Angle)
{
	Spin_Pict_Info_Ptr			spin;
	CARDINAL_32				last_Index;
	CARDINAL_32				loop;
	CARDINAL_16				highest_Intensity;
	CARDINAL_16				highest_Horizontal;
	CARDINAL_16				highest_Vertical;
	CARDINAL_16				highest_Roll;
/* END VAR */

	highest_Intensity = 0;
	highest_Horizontal = 0;
	highest_Vertical = 0;
	highest_Roll = 0;
	
	spin = (*spin_Info);
	last_Index = spin->array_Actual_Count;
	for (loop = 0; (loop < last_Index); loop++) {
	
		if (spin->cell_Info[loop].vertical_Angle > highest_Vertical) {
			highest_Vertical = spin->cell_Info[loop].vertical_Angle;
		}
		
		if (spin->cell_Info[loop].horizontal_Angle > highest_Horizontal) {
			highest_Horizontal = spin->cell_Info[loop].horizontal_Angle;
		}
		
		if (spin->cell_Info[loop].roll_Angle > highest_Roll) {
			highest_Roll = spin->cell_Info[loop].roll_Angle;
		}
		
		if (spin->cell_Info[loop].activation > highest_Intensity) {
			highest_Intensity = spin->cell_Info[loop].activation;
		}
	}
	
	*intensity = highest_Intensity;
	*horizontal_Angle = highest_Horizontal;
	*vertical_Angle = highest_Vertical;
	*roll_Angle = highest_Roll;
	
}
/*=**************************************************************************=*/		

		
/*=**************************************************************************=*/		
void Get_Periodicity_Ranges(Spin_Pict_Info_Handle	spin_Info
											, CARDINAL_16		highest_Horizontal
											, CARDINAL_16		highest_Vertical
											, CARDINAL_16		highest_Roll
											, CARDINAL_32		*horizontal_Angle_Periodicity
											, CARDINAL_32		*vertical_Angle_Periodicity
											, CARDINAL_32		*roll_Angle_Periodicity)
{
	Spin_Pict_Info_Ptr			spin;
	CARDINAL_32				last_Index;
	CARDINAL_32				loop;
/* END VAR */

	/*  */
	/* HACK AHCK HACK UNFINISHED */
	/* This computes the periodicity, if any, between the measurements */
	/* of angles present in the file. It does this by creating an array of */
	/* 4 thousand angles (not 64K angles as it should) and strikes *&/
	/* thorugh the bytes as a sieve of eratosthenes (kind of). */
	/* when doen it goes thorugh the array and measures the first two */
	/* items located and then ENSURES that all the other entries are a multiple */
	/* of the periodicity. The periodicty is based of if "0" and all entries are modulo */
	/* of that assumed periodicity. the pupose of this routine is to filter data input */
	/* round all control requests by truncating to divisor. It is not critical that the */
	/* files be uniform the result is that the period may be smaller. */
	/* 9% of the CPU is spent looking for optimal puppet frames in a throtough search */
	/* in a large test puppet, with the cache installed it is 0% for cache */
	/* of 64 last frames. */
	/*  */
	
	*horizontal_Angle_Periodicity = 1;
	*vertical_Angle_Periodicity = 1;
	*roll_Angle_Periodicity = 1;
	

	/*  */
	/* First create byte flag field */
	/*  */
	
	/*  */
	/* Do the search three times one for each angle */
	/*  */
	
	/*  */
	/* Nullify field to all 0's */
	/*  */
	
	/*  */
	/* Mark each item located. Frames are not sorted biased to any angle */
	/*  */

	spin = (*spin_Info);
	last_Index = spin->array_Actual_Count;
	for (loop = 0; (loop < last_Index); loop++) {
	
	/* ==
			highest_Vertical = spin->cell_Info[loop].vertical_Angle;
			highest_Horizontal = spin->cell_Info[loop].horizontal_Angle;
			highest_Roll = spin->cell_Info[loop].roll_Angle;
		=== */
			
	}
	
	/*  */
	/* Delete array */
	/*  */
	
	*horizontal_Angle_Periodicity = 1;
	*vertical_Angle_Periodicity = 1;
	*roll_Angle_Periodicity = 1;

} /* void Get_Periodicity_Ranges */
/*=**************************************************************************=*/		

		
/*=**************************************************************************=*/		
Safe_Boolean	Draw_Spin_Object_From_File(INTEGER_16		file_Reference
								, Off_Screen_Port_Rec *port_Ptr
								, Off_Screen_Port_Rec *optional_Background_Port_Ptr
								, Spin_Pict_Info_Handle	spin_Info
								, Previously_Drawn_Frame_Ptr	frame_To_Erase_First
								, CARDINAL_8 intensity
								, CARDINAL_8 horizontal_Angle
								, CARDINAL_8 vertical_Angle
								, CARDINAL_8 roll_Angle
								, Boolean	check_Validity
								, CARDINAL_32 *avoidance_Image_Index
								, INTEGER_32		offset_Offscreen_Pixel_X
								, INTEGER_32		offset_Offscreen_Pixel_Y)
{
	Rect						backdrop_Size_Rect;
	CARDINAL_32	index;
	CARDINAL_32	previous_Index;
	CARDINAL_32	stored_Bit_Depth;
	CARDINAL_32	data_Start_Offset;
	CARDINAL_32	picture_Row_Byte_Offset;
	CARDINAL_32	file_Size;
	INTEGER_32		transfer_Count;
	INTEGER_32		transfer_Count_Copy;
	CARDINAL_32	previous_Data_Start_Offset;
	INTEGER_32		previous_Frame_Total_Size;
	CARDINAL_32 	simple_Key_2;
	CARDINAL_32 	composite_Search_Key;
	CARDINAL_32 	storage_Location_To_Use;
	CARDINAL_32 	old_Storage_Location_To_Use;
	CARDINAL_32	raw_32_Bit_Word_1;
	CARDINAL_32	raw_32_Bit_Word_2;
	CARDINAL_32	memory_Needed;
	CARDINAL_32	handle_Size;
	CARDINAL_16	compression_Type;
	CARDINAL_16	previous_Compression_Type;
	CARDINAL_32	*row_Raw_Data_Ptr;
	CARDINAL_32	*skip_List_Data_Ptr;
	CARDINAL_32	*skip_List_Master_List_Ptr;
	CARDINAL_32	*destination_Word_Ptr;
	CARDINAL_32	*scan_Line_Bounds_Ptr;
	CARDINAL_32	optional_Background_Row_Byte_Offset;
	CARDINAL_32	actual_Ticks_Delayed;
	Boolean				data_Ready_To_Fetch;
	Boolean				old_Frame_Ready_To_Fetch;
	Boolean				primary_Data_Needs_Loading;
	Boolean				async_Read_Is_Useful;
	Rect						optional_Background_Rect;
	CARDINAL_32	*optional_Background_Word_Ptr;
	IOParam				async_Params;
	compression_Data_Header_Ptr	compression_Info;
	CARDINAL_8		*code_Name;
	OSErr					os_Error;
	Safe_Boolean		function_Result;
	Str255					error_String;
	Str255					note_String;
/* END VAR */


	code_Name = "\pDraw_Spin_Object_From_File";
	function_Result = SAFE_FALSE;
	async_Params.ioResult = 0;
	
	/*  */
	/* First see if it can accept another frame without resizing handle */
	/* otherwise abort. Enhace soon. HACK HACK HACK */
	/*  */
	if ((*spin_Info)->valid != SAFE_TRUE) {
		Display_Serious_Error_Message(unit_Name, code_Name, "\pBad pointer for cell info.", "\p");
		goto Error_Handler;
	}

	/* = = = = = = = = = = = = = = = = = = = = = = = = = = = = = = = = = = = = = = = = = = = = = = = = = = = = = = = = */
	/* = = = = = = = = = = = = = = = = = = = = = = = = = = = = = = = = = = = = = = = = = = = = = = = = = = = = = = = = */
	/* = = = = = = = = = = = = = = = = = = = = = = = = = = = = = = = = = = = = = = = = = = = = = = = = = = = = = = = = */
	
	composite_Search_Key = intensity;
	composite_Search_Key = (composite_Search_Key << 8) | horizontal_Angle;
	composite_Search_Key = (composite_Search_Key << 8) | vertical_Angle;
	composite_Search_Key = (composite_Search_Key << 8) | roll_Angle;
	
	/*  */
	/* 1995.03.27 JAM Enhancement */
	/* Trust that all handles are from same storage location. */
	/* By not dereferencing it allows RAM to move around. Bad? */
	/* Will indexed data ever need to move? */
	/*  */
	simple_Key_2 = (CARDINAL_32)StripAddress(spin_Info);
	
	if (Locate_Best_Frame_From_Simple_Ring_Cache(composite_Search_Key, simple_Key_2, &index) != SAFE_TRUE) {
				
		if (Locate_Best_Frame(spin_Info
									, intensity
									, horizontal_Angle
									, vertical_Angle
									, roll_Angle
									, &index) != TRUE) {
									
			goto Skip_Frame_To_Draw;				
		}
		
		/*  */
		/* Save into our cache for later access */
		/*  */
		if (Record_Frame_Into_Simple_Ring_Cache(composite_Search_Key, simple_Key_2, index) != SAFE_TRUE) {
			/*  */
			/* Cannot actually fail */
			/*  */
		}
	}
	
	if (*avoidance_Image_Index == index) {
		goto Skip_Frame_To_Draw;
	}
	
	compression_Type = (*spin_Info)->cell_Info[index].compression_Type;
	
	
	switch(compression_Type) {
		case k_NO_COMPRESSION:
		
		break;
	
		case k_MERKLE_TIME_COMPRESSION:
			/*  */
			/* "g_Spin_Ring_Buffer_Initialized" should be checked? */
			/* not really needed. but if is not SAFE_TRUE, then set */
			/* g_Spin_Cache_Master_Ptr to point to nil for safety. */
			/*  */
			if ((g_Spin_Ring_Buffer_Initialized != SAFE_TRUE) || (g_Spin_Cache_Master_Ptr == nil)){
				goto Error_Handler;
			}
		break;
		
		case k_EVEN_ODD_ROW_COMPRESSION:
		default:
			Display_Serious_Error_Message(unit_Name, code_Name, "\pThis version does not handle this type of compressed file", "\pNeeds enhancement");
			goto Error_Handler;
	} /* switch((*spin_Info)->cell_Info[index].compression_Type) */
			
	if (check_Validity == TRUE) {
	
		if (port_Ptr->image_Handle != nil) {
			backdrop_Size_Rect = (**(port_Ptr->offscreen_Port.color_Port.portPixMap)).bounds;
			picture_Row_Byte_Offset = ((**(port_Ptr->offscreen_Port.color_Port.portPixMap)).rowBytes & (~COLOR_PORT_PORTBITS_CHECK));
			stored_Bit_Depth = (**(port_Ptr->offscreen_Port.color_Port.portPixMap)).pixelSize;
		}
		else {
			Display_Serious_Error_Message(unit_Name, code_Name, "\pBad port for cell info extraction.", "\pProgrammer flaw");
			goto Error_Handler;
		}
				
		if ((*spin_Info)->cell_Info[index].color_Depth != (*spin_Info)->max_Depth) {
			Display_Serious_Error_Message(unit_Name, code_Name, "\pColor depth incompatible for cell in file.", "\pNeeds enhancement");
			goto Error_Handler;
		}
	
		if ((*spin_Info)->cell_Info[index].color_Depth != stored_Bit_Depth) {
			Display_Serious_Error_Message(unit_Name, code_Name, "\pColor depth incompatible for cell in file with destination.", "\pNeeds enhancement");
			goto Error_Handler;
		}
		
		if (compression_Type != k_MERKLE_TIME_COMPRESSION) {
			if ((*spin_Info)->cell_Info[index].picture_Row_Byte_Offset != picture_Row_Byte_Offset) {
				Display_Serious_Error_Message(unit_Name, code_Name, "\pRow Size Offset incompatible for cell in file with destination.", "\pNeeds enhancement (Turn off Max Window for now)");
				goto Error_Handler;
			}
			
			/*  */
			/* HACK HACK HACK we should check if memory being overwritten in */
			/* k_MERKLE_TIME_COMPRESSION */
			/*  */
			raw_32_Bit_Word_1 = *(CARDINAL_32 *)&((*spin_Info)->cell_Info[index].max_Picture_Bounds);
			raw_32_Bit_Word_2 = *(CARDINAL_32 *)&((*spin_Info)->max_Bounds);
			if (raw_32_Bit_Word_1 != raw_32_Bit_Word_2) {
				Display_Serious_Error_Message(unit_Name, code_Name, "\pPicture bounds incompatible for cell in file.", "\pNeeds enhancement");
				goto Error_Handler;
			}
			
			if (raw_32_Bit_Word_1 != *(CARDINAL_32 *)&backdrop_Size_Rect) {
				Display_Serious_Error_Message(unit_Name, code_Name, "\pPicture bounds incompatible with destination for cell in file.", "\pNeeds enhancement (Turn off Max Window for now)");
				goto Error_Handler;
			}
		} /* if (compression_Type != k_MERKLE_TIME_COMPRESSION) */
				
	} /* if (check_Validity == TRUE)  */
	
	data_Start_Offset = (*spin_Info)->cell_Info[index].data_Start_Offset;
	transfer_Count_Copy = (*spin_Info)->cell_Info[index].data_Total_Size;
	
	/*  */
	/* composite_Search_Key and simple_Key_2 are already ready */
	/* from earlier on in this routine */
	/*  */

	/* = = = = = = = = = = = = = = = = = = = = = = = = = = = = = = = = = = = = = = = = = = = = = = = = = = = = = = = = */
	
	data_Ready_To_Fetch = TRUE;
	if (Retrieve_Element_From_Buffer(g_Spin_Cache_Master_Ptr
												, composite_Search_Key
												,simple_Key_2
												,&storage_Location_To_Use) != SAFE_TRUE) {
														
		if (Prepare_Location_For_Buffer_Write(g_Spin_Cache_Master_Ptr
											, composite_Search_Key
											, simple_Key_2
											, transfer_Count_Copy
											, &storage_Location_To_Use) == SAFE_TRUE) {
											
			data_Ready_To_Fetch = FALSE;
			
		}
		else {
			Display_Serious_Error_Message(unit_Name, code_Name, "\pPrepare_Location_For_Buffer_Write failed to find node", "\pBad programming or cache size?");
			goto Error_Handler;
		}
	}
	
	/* = = = = = = = = = = = = = = = = = = = = = = = = = = = = = = = = = = = = = = = = = = = = = = = = = = = = = = = = */

	
	switch(compression_Type) {
		case k_NO_COMPRESSION:
		
			/*  */
			/* Issue an asynchronous READ request during silloette erasure */
			/*  */
			if (data_Ready_To_Fetch != TRUE) {
			
				os_Error = SetFPos(file_Reference,fsFromStart,data_Start_Offset);
				if (os_Error != noErr) {
					Display_Serious_Error_Message(unit_Name, code_Name, "\pSetFPos failed to locate data", "\pBad file");
					goto Error_Handler;
				}
			
				transfer_Count = transfer_Count_Copy;
				os_Error = FSRead(file_Reference, &transfer_Count, (Ptr)storage_Location_To_Use);
				if (os_Error != noErr) {
	
					Assign_String(note_String , "\pFSRead failed to read data. Error = ");
					Concat_Long_To_String(note_String, os_Error, NO_LIMIT);
				
					Display_Serious_Error_Message(unit_Name, code_Name, note_String, "\pBad file or drive?");
					goto Error_Handler;
				}
			}
		
			Block_Move_No_Flush((Ptr)storage_Location_To_Use, (Ptr)*port_Ptr->image_Handle, transfer_Count_Copy);
			
			if (frame_To_Erase_First != nil) {
				frame_To_Erase_First->valid_Information = SAFE_FALSE;
			}
		break;
	
		case k_MERKLE_TIME_COMPRESSION:
			/*  */
			/* Instead of just drawing it, we should decompress it */
			/*  */
			
			backdrop_Size_Rect = (**(port_Ptr->offscreen_Port.color_Port.portPixMap)).bounds;
			picture_Row_Byte_Offset = ((**(port_Ptr->offscreen_Port.color_Port.portPixMap)).rowBytes & (~COLOR_PORT_PORTBITS_CHECK));
			stored_Bit_Depth = (**(port_Ptr->offscreen_Port.color_Port.portPixMap)).pixelSize;
			
			/*  */
			/* Several possibilities are possible */
			/* 1 - no previous frame appears required to erase first, draw if in cache */
			/* 2 - no previous frame appears required to erase first, fetch & draw if not in cache */
			/* 3 - erase old frame and location and ASYNC load next frame if no collision, then draw */
			/* 4 - erase old frame from cache and then load next frame if needed, draw it*/
			/* 5 - erase old frame by loading and then load next frame if needed, draw it*/
			/*  */
						
			if ((frame_To_Erase_First != nil) 
				&& (optional_Background_Port_Ptr != nil) 
				&& (frame_To_Erase_First->valid_Information == SAFE_TRUE)) {
										
				/*  */
				/* Delete old frame first if needed, and reload key if needed and load */
				/* next frame asyncronously if possible while erasing to save a little time */
				/*  */
				
				if (frame_To_Erase_First->port_Ptr == port_Ptr) {
				
					previous_Index = frame_To_Erase_First->previous_Index;
		
			/* ===
					frame_To_Erase_First->offset_Offscreen_Pixel_X = offset_Offscreen_Pixel_X;
					frame_To_Erase_First->offset_Offscreen_Pixel_Y = offset_Offscreen_Pixel_Y;
			==== */
			
					if (data_Ready_To_Fetch != TRUE) {
						primary_Data_Needs_Loading = TRUE;
					}
					else {
						primary_Data_Needs_Loading = FALSE;
					}
	
					previous_Compression_Type = (*spin_Info)->cell_Info[previous_Index].compression_Type;
					previous_Data_Start_Offset = (*spin_Info)->cell_Info[previous_Index].data_Start_Offset;
					previous_Frame_Total_Size = (*spin_Info)->cell_Info[previous_Index].data_Total_Size;
					
					old_Frame_Ready_To_Fetch = TRUE;
	
					if (Retrieve_Element_From_Buffer(g_Spin_Cache_Master_Ptr
																, frame_To_Erase_First->search_Key_1
																, frame_To_Erase_First->search_Key_2
																,&old_Storage_Location_To_Use) != SAFE_TRUE) {
																
						if (Prepare_Location_For_Buffer_Write(g_Spin_Cache_Master_Ptr
															, frame_To_Erase_First->search_Key_1
															, frame_To_Erase_First->search_Key_2
															, previous_Frame_Total_Size
															, &old_Storage_Location_To_Use) == SAFE_TRUE) {
															
							old_Frame_Ready_To_Fetch = FALSE;
							
						}
						else {
							Display_Serious_Error_Message(unit_Name, code_Name, "\pPrepare_Location_For_Buffer_Write failed to find node", "\pBad programming or cache size?");
							goto Error_Handler;
						}
					}
					
					if (previous_Compression_Type == k_MERKLE_TIME_COMPRESSION) {
					
						if (old_Frame_Ready_To_Fetch != TRUE) {
							/*  */
							/* Reload the old picture, and recheck is required of previously located cache value*/
							/*  */
							os_Error = SetFPos(file_Reference, fsFromStart, previous_Data_Start_Offset);
							if (os_Error != noErr) {
								Display_Serious_Error_Message(unit_Name, code_Name, "\pSetFPos failed to locate data", "\pBad file");
								goto Error_Handler;
							}
			
							transfer_Count = previous_Frame_Total_Size;
							os_Error = FSRead(file_Reference, &transfer_Count, (Ptr)old_Storage_Location_To_Use);
							if (os_Error != noErr) {
				
								Assign_String(note_String , "\pFSRead failed to read data. Error = ");
								Concat_Long_To_String(note_String, os_Error, NO_LIMIT);
							
								Display_Serious_Error_Message(unit_Name, code_Name, note_String, "\pBad file or drive?");
								goto Error_Handler;
							}
							
							/*  */
							/* Damn it. We must re-prepare a cache range for initial frame. */
							/* Whether or not data_Ready_To_Fetch is ready? Yes if it is ready then */
							/* the locate will merely locate the raw uninitialized data, else repoint to good info*/
							/*  */
							if (Retrieve_Element_From_Buffer(g_Spin_Cache_Master_Ptr
																		, composite_Search_Key
																		,simple_Key_2
																		,&storage_Location_To_Use) != SAFE_TRUE) {
																				
								if (Prepare_Location_For_Buffer_Write(g_Spin_Cache_Master_Ptr
																	, composite_Search_Key
																	, simple_Key_2
																	, transfer_Count_Copy
																	, &storage_Location_To_Use) == SAFE_TRUE) {
																	
									data_Ready_To_Fetch = FALSE;
									
								}
								else {
									Display_Serious_Error_Message(unit_Name, code_Name, "\pPrepare_Location_For_Buffer_Write failed to find node", "\pBad programming or cache size?");
									goto Error_Handler;
								}
							}
							else {
								if (primary_Data_Needs_Loading == TRUE) {
									data_Ready_To_Fetch = FALSE;
								}
								else {
									data_Ready_To_Fetch = TRUE;
								}
							}
							
						}
						else {
							/*  */
							/* Was in cache already. Good, its ready to use for erasure. */
							/*  */
						}
						
						
						if (data_Ready_To_Fetch != TRUE) {
						
							async_Read_Is_Useful = TRUE;
							if (storage_Location_To_Use <= old_Storage_Location_To_Use) {
								if ((CARDINAL_32)(storage_Location_To_Use
										+ (CARDINAL_32)transfer_Count_Copy
										+ k_SAFETY_ALIGNMENT_CACHE_PAD_MAX) >= old_Storage_Location_To_Use) {
									async_Read_Is_Useful = FALSE;
								}
							}
							else {
								if ((CARDINAL_32)(old_Storage_Location_To_Use
										+ (CARDINAL_32)previous_Data_Start_Offset
										+ k_SAFETY_ALIGNMENT_CACHE_PAD_MAX) >= storage_Location_To_Use) {
									async_Read_Is_Useful = FALSE;
								}
							}
							
							if (async_Read_Is_Useful == TRUE) {
							
								/*  */
								/* NOTE: This is enhanced for Async reading! */
								/*  */
								async_Params.ioRefNum = file_Reference;
								async_Params.ioPosMode = fsFromStart;
								async_Params.ioPosMode = (async_Params.ioPosMode | k_FORCE_NO_CACHE_BIT);
								async_Params.ioCompletion = nil;
								async_Params.ioResult = 1;
								async_Params.ioBuffer = (Ptr)storage_Location_To_Use;
								async_Params.ioReqCount = transfer_Count_Copy;
								async_Params.ioPosOffset = data_Start_Offset;
								
								os_Error = PBRead((ParmBlkPtr)&async_Params,	TRUE);
								if (os_Error != noErr) {
									Assign_String(note_String , "\pFSRead failed to read data. Error = ");
									Concat_Long_To_String(note_String, os_Error, NO_LIMIT);
								
									Display_Serious_Error_Message(unit_Name, code_Name, note_String, "\pBad file or drive?");
									goto Error_Handler;
								}
				
							}
							
						} /* if (data_Ready_To_Fetch != TRUE) */
						else {
							async_Read_Is_Useful = FALSE;
						}								
	
						/*  */
						/* While next frame is loading, if needed, erase previous frame */
						/*  */
						
						compression_Info = (compression_Data_Header_Ptr)old_Storage_Location_To_Use;
			
						if (compression_Info->sentinel_1 != SAFE_TRUE) {
							goto Error_Handler;
						}

						row_Raw_Data_Ptr = (CARDINAL_32 *)((CARDINAL_32)compression_Info + compression_Info->row_Raw_Data_Start_Offset);
						skip_List_Data_Ptr = (CARDINAL_32 *)((CARDINAL_32)compression_Info + compression_Info->skip_List_Data_Start_Offset);
						skip_List_Master_List_Ptr = (CARDINAL_32 *)((CARDINAL_32)compression_Info + compression_Info->skip_List_Master_List_Start_Offset);
						scan_Line_Bounds_Ptr = (CARDINAL_32 *)((CARDINAL_32)compression_Info + compression_Info->scan_Line_Bounds_Offset); 
									
						/*  */
						/* Avoid GetPixBaseAddr for a little speed */
						/*  */
						destination_Word_Ptr = (CARDINAL_32 *)*(port_Ptr->image_Handle);
									
						if (destination_Word_Ptr == nil) {
							goto Error_Handler;
						}
						
						*(CARDINAL_32 *)&destination_Word_Ptr += 
											(CARDINAL_32)((CARDINAL_32)(picture_Row_Byte_Offset)
											* (CARDINAL_32)(frame_To_Erase_First->offset_Offscreen_Pixel_Y));
					
						*(CARDINAL_32 *)&destination_Word_Ptr += 
												(((CARDINAL_32)(frame_To_Erase_First->offset_Offscreen_Pixel_X) 
													* (CARDINAL_32)(stored_Bit_Depth)) >> 3);
			
		
						optional_Background_Rect = (**(optional_Background_Port_Ptr->offscreen_Port.color_Port.portPixMap)).bounds;
						optional_Background_Row_Byte_Offset = ((**(optional_Background_Port_Ptr->offscreen_Port.color_Port.portPixMap)).rowBytes & (~COLOR_PORT_PORTBITS_CHECK));
									
						if ((optional_Background_Rect.left >= backdrop_Size_Rect.left)
							&& (optional_Background_Rect.right <= backdrop_Size_Rect.right)
							&& (optional_Background_Rect.top >= backdrop_Size_Rect.top)
							&& (optional_Background_Rect.bottom <= backdrop_Size_Rect.bottom)) {
							
							optional_Background_Word_Ptr = (CARDINAL_32 *)*(optional_Background_Port_Ptr->image_Handle);
						
						
					/* ===
							*(CARDINAL_32 *)&optional_Background_Word_Ptr += 
												(CARDINAL_32)((CARDINAL_32)(optional_Background_Row_Byte_Offset)
												* (CARDINAL_32)(frame_To_Erase_First->offset_Offscreen_Pixel_Y));
						
							*(CARDINAL_32 *)&optional_Background_Word_Ptr += 
													(((CARDINAL_32)(frame_To_Erase_First->offset_Offscreen_Pixel_X) 
														* (CARDINAL_32)(stored_Bit_Depth)) >> 3);
					==== */
			
							/*  */
							/* This is background erasure step it could be faster, but the way it */
							/* works it erases the entire object filling in from a source picture */
							/*  */
							Fill_Compressed_Span_With_Simple_Silloette(row_Raw_Data_Ptr
																			, skip_List_Data_Ptr
																			, skip_List_Master_List_Ptr
																			, destination_Word_Ptr
																			, picture_Row_Byte_Offset
																			, optional_Background_Word_Ptr
																			, optional_Background_Row_Byte_Offset);
	
						}
		
						
						/*  */
						/* Spin and wait for async read request to complete. */
						/* It was started before erasure was initiated. */
						/*  */
						if (async_Read_Is_Useful == TRUE) {
						
							/*  */
							/* Delay in a dead loop while waiting for file to finish loading. */
							/* Hopefully this is not too detrimental to overall speed */
							/*  */
							while (async_Params.ioResult > 0) {
								Delay(1, (INTEGER_32 *)&actual_Ticks_Delayed);
							}
							
							if (async_Params.ioResult < 0) {
								data_Ready_To_Fetch = FALSE;

								/*  */
								/* Error! Data not read! */
								/*  */
								Assign_String(note_String , "\pFSRead failed to asyncronously read read data. Error = ");
								Concat_Long_To_String(note_String, os_Error, NO_LIMIT);
							
								Display_Serious_Error_Message(unit_Name, code_Name, note_String, "\pBad file or drive?");
								goto Error_Handler;
							}
							else {
								data_Ready_To_Fetch = TRUE;
							}
						}
					
					} /* if (previous_Compression_Type == k_MERKLE_TIME_COMPRESSION) */
				
				} /* if (frame_To_Erase_First->port_Ptr == port_Ptr) */
				
			} /* 	if ((optional_Background_Port_Ptr != nil) && (frame_To_Erase_First->valid_Information == SAFE_TRUE)) */
			
			
			/*  */
			/* Old frame is deleted */
			/* Load frame syncronously only if not already loaded from async */
			/* stage */
			/*  */
			
			if (data_Ready_To_Fetch != TRUE) {

				os_Error = SetFPos(file_Reference,fsFromStart,data_Start_Offset);
				if (os_Error != noErr) {
					Display_Serious_Error_Message(unit_Name, code_Name, "\pSetFPos failed to locate data", "\pBad file");
					goto Error_Handler;
				}

				transfer_Count = transfer_Count_Copy;
				os_Error = FSRead(file_Reference, &transfer_Count, (Ptr)storage_Location_To_Use);
				if (os_Error != noErr) {
	
					Assign_String(note_String , "\pFSRead failed to read data. Error = ");
					Concat_Long_To_String(note_String, os_Error, NO_LIMIT);
				
					Display_Serious_Error_Message(unit_Name, code_Name, note_String, "\pBad file or drive?");
					goto Error_Handler;
				}
			}


			/*  */
			/* Fetch the addresses of out compacted picture information */
			/*  */
			compression_Info = (compression_Data_Header_Ptr)storage_Location_To_Use;
			
			if (compression_Info->sentinel_1 != SAFE_TRUE) {
				goto Error_Handler;
			}
			
			row_Raw_Data_Ptr = (CARDINAL_32 *)((CARDINAL_32)compression_Info + compression_Info->row_Raw_Data_Start_Offset);
			skip_List_Data_Ptr = (CARDINAL_32 *)((CARDINAL_32)compression_Info + compression_Info->skip_List_Data_Start_Offset);
			skip_List_Master_List_Ptr = (CARDINAL_32 *)((CARDINAL_32)compression_Info + compression_Info->skip_List_Master_List_Start_Offset);
			scan_Line_Bounds_Ptr = (CARDINAL_32 *)((CARDINAL_32)compression_Info + compression_Info->scan_Line_Bounds_Offset); 
			
			
			/*  */
			/* Avoid GetPixBaseAddr for a little speed */
			/*  */
			destination_Word_Ptr = (CARDINAL_32 *)*(port_Ptr->image_Handle);
						
			if (destination_Word_Ptr == nil) {
				goto Error_Handler;
			}
			
			*(CARDINAL_32 *)&destination_Word_Ptr += 
								(CARDINAL_32)((CARDINAL_32)(picture_Row_Byte_Offset)
								* (CARDINAL_32)offset_Offscreen_Pixel_Y);
		
			*(CARDINAL_32 *)&destination_Word_Ptr += 
									(((CARDINAL_32)offset_Offscreen_Pixel_X 
										* (CARDINAL_32)(stored_Bit_Depth)) >> 3);

			/*  */
			/* HACK HACK HACK : This should be only changing pixels that need */
			/* erasing ! Not all the pixels! */
			/*  */
			/*  */
			/*  */
			/*  */
			/*  */
			/* First we clear all the RAM for the destination */
			/* HACK HACK HACK : Get correct chroma value and smear into other bytes. */
			/* also, prefecth image_Handle size to avoid calling trap always */
			/*  */
/* ===
			handle_Size = GetHandleSize(port_Ptr->image_Handle);
			if (MemError() != noErr) {
				goto Error_Handler;
			}
		=== */
	/* ===== 
			Fill_Large_Buffer((Ptr)*(port_Ptr->image_Handle), handle_Size, g_Chroma_Index_Optimized_Long);
	==== */
	
	
	
			/*  */
			/* Warn if not cache line aligned */
			/*  */
			
			if (((CARDINAL_32)row_Raw_Data_Ptr & 0x000F) != 0) {
				if (g_Cache_Misalign_Warning_Issued != SAFE_TRUE) {
				
					Get_NS_String(k_DREAM_ERROR_STRINGS, k_ERROR_SOURCE_NOT_ALIGNED_TO_16_BYTE, error_String);
					Perform_OK_Dialog(error_String);
								
					g_Cache_Misalign_Warning_Issued = SAFE_TRUE;
				}	
			}
			
			if (((CARDINAL_32)destination_Word_Ptr & 0x000F) != 0) {
				if (g_Cache_Misalign_Warning_Issued != SAFE_TRUE) {
				
					Get_NS_String(k_DREAM_ERROR_STRINGS, k_ERROR_DESTINATION_NOT_ALIGNED_TO_16_BYTE, error_String);
					Perform_OK_Dialog(error_String);
				
					g_Cache_Misalign_Warning_Issued = SAFE_TRUE;
				}	
			}
	
			
			Copy_Bits_From_Compressed_Pict_With_Region_NEW(row_Raw_Data_Ptr
													, skip_List_Data_Ptr
													, skip_List_Master_List_Ptr
													, destination_Word_Ptr
													, picture_Row_Byte_Offset);

			if (frame_To_Erase_First != nil) {
				frame_To_Erase_First->valid_Information = SAFE_TRUE;
				frame_To_Erase_First->search_Key_1 = composite_Search_Key;
				frame_To_Erase_First->search_Key_2 = simple_Key_2;
				frame_To_Erase_First->previous_Index = index;
				frame_To_Erase_First->port_Ptr = port_Ptr;
				frame_To_Erase_First->offset_Offscreen_Pixel_X = offset_Offscreen_Pixel_X;
				frame_To_Erase_First->offset_Offscreen_Pixel_Y = offset_Offscreen_Pixel_Y;
				frame_To_Erase_First->previous_Index = index;
			}
			
		break;
		
		case k_EVEN_ODD_ROW_COMPRESSION:
		default:
			if (frame_To_Erase_First != nil) {
				frame_To_Erase_First->valid_Information = SAFE_FALSE;
			}
			Display_Serious_Error_Message(unit_Name, code_Name, "\pThis version does not handle this type of compressed file", "\pNeeds enhancement");
			goto Error_Handler;
	} /* switch((*spin_Info)->cell_Info[index].compression_Type) */
	
	
	function_Result = SAFE_TRUE;
	
	goto Normal_Function_Exit;

Error_Handler:
	Display_Serious_Error_Message(unit_Name, code_Name, "\pInternal problem drawing spin_Object", "\p");
	function_Result = SAFE_FALSE;

Skip_Frame_To_Draw:
	function_Result = SAFE_FALSE;
		
Normal_Function_Exit:
					
	return(function_Result);
	
} /* Safe_Boolean	Draw_Spin_Object_From_File... */
/*=**************************************************************************=*/


/*=**************************************************************************=*/
Safe_Boolean	Export_Spin_Object_Sequence(Export_Spin_Object_Rec	*params_For_Export_Spin_Object)
{
	INTEGER_16		scrap_Volume_Reference_Number;
	INTEGER_32		destination_Directory_ID;
	INTEGER_32		directory_Reference;
	INTEGER_16		file_Reference;
	INTEGER_16		resource_ID;
	Rect				spin_Object_Size_Rect;
	INTEGER_16		stored_Bit_Depth;
	Spin_Object_Ptr		current_Spin_Object;
	Safe_Boolean		function_Result;
	CARDINAL_32	temp_Size;
	CARDINAL_32	file_Creator;
	CARDINAL_32	file_Type;
	CARDINAL_32	resource_Type;
	CARDINAL_32	file_Minimum_Size;
	WDPBRec			working_Directory_PB;
	INTEGER_16		working_Directory;
	Safe_Boolean		working_Directory_In_Use;
	Boolean			problem_Getting_Picture;
	PicHandle			picture_Handle;
	INTEGER_32		volume_Space;
	CARDINAL_8		*code_Name;
	Str255				resource_Comment;
	Str255				note_String;
	Str255				volume_Name;
	Str255				work_String_2;
	OSErr				os_Error;
	INTEGER_32		transfer_Count;
	INTEGER_32		transfer_Count_Copy;
/* END VAR */


	code_Name = "\pSave_Spin_Object_Sequence";
	function_Result = SAFE_FALSE;
	working_Directory_In_Use = SAFE_FALSE;
	problem_Getting_Picture = FALSE;
	picture_Handle = nil;
		
	/*  */
	/* current_Spin_Object uses a handle assumed to always be locked */
	/*  */
	current_Spin_Object = *(dream.spin_Objects.spin_Object[params_For_Export_Spin_Object->desired_Spin_Object]);
	
	if ((current_Spin_Object->initialized != SAFE_TRUE) 
		|| ((current_Spin_Object->special_Effect_Spin_Object != 0) && (current_Spin_Object->port_Info.spin_Object_Port_1.initialized != SAFE_TRUE))) {

		Get_NS_String(k_SPIN_STRINGS, k_ERROR_SPIN_OBJECT_UNSAVABLE, work_String_2);
		Perform_OK_Dialog(work_String_2);
		goto Error_Handler;
	}

	if ((dream.spin_Objects.spin_Object_Memory_In_Use[params_For_Export_Spin_Object->desired_Spin_Object]) != SAFE_TRUE) {
		function_Result = SAFE_FALSE;
		goto Normal_Function_Exit;
	}
	
	file_Creator= '8BIM';
	file_Type = 'SCRN';
	note_String[0] = 0;
	
	os_Error = GetVInfo((Get_Drive_Number(params_For_Export_Spin_Object->desired_File.vRefNum)), volume_Name, &scrap_Volume_Reference_Number, (INTEGER_32 *)&volume_Space);
	if (os_Error != noErr) {
		Concat_Many(note_String, 3, "\pGetVInfo for volume ", volume_Name, "\p ID = ");
		Concat_Long_To_String(note_String, os_Error, NO_LIMIT);
		goto Error_Handler_With_Message;
	} /* if (os_Error != noErr) */
		

	switch (current_Spin_Object->port_Info.spin_Object_Port_1.color_Type) {
		case k_COLOR_QUICKDRAW:
		case k_32_BIT_COLOR_QUICKDRAW:
		
			if (current_Spin_Object->port_Info.spin_Object_Port_1.image_Handle != nil) {
				spin_Object_Size_Rect = (**(current_Spin_Object->port_Info.spin_Object_Port_1.offscreen_Port.color_Port.portPixMap)).bounds;
				stored_Bit_Depth = (**(current_Spin_Object->port_Info.spin_Object_Port_1.offscreen_Port.color_Port.portPixMap)).pixelSize;
			}
			else {
				/*  */
				/* Cannot get here. paranoia code */
				/*  */
				SetRect(&(spin_Object_Size_Rect), 0,0,1,1);
				stored_Bit_Depth = 8;
			}
		break;
		
		case k_ORIGINAL_QUICKDRAW:
		default:
			spin_Object_Size_Rect = current_Spin_Object->port_Info.spin_Object_Port_1.offscreen_Port.old_Style_Port.portBits.bounds;
			stored_Bit_Depth = 1;
	} /* switch (current_Spin_Object->port_Info.spin_Object_Port_1.color_Type) */
	
					
	if (spin_Object_Size_Rect.left < current_Spin_Object->spin_Object_Background_Rect.left) {
		spin_Object_Size_Rect.left = current_Spin_Object->spin_Object_Background_Rect.left;
	}
					
	if (spin_Object_Size_Rect.top < current_Spin_Object->spin_Object_Background_Rect.top) {
		spin_Object_Size_Rect.top = current_Spin_Object->spin_Object_Background_Rect.top;
	}
					
	if (spin_Object_Size_Rect.right > current_Spin_Object->spin_Object_Background_Rect.right) {
		spin_Object_Size_Rect.right = current_Spin_Object->spin_Object_Background_Rect.right;
	}
					
	if (spin_Object_Size_Rect.bottom > current_Spin_Object->spin_Object_Background_Rect.bottom) {
		spin_Object_Size_Rect.bottom = current_Spin_Object->spin_Object_Background_Rect.bottom;
	}
	
	if (EmptyRect(&(spin_Object_Size_Rect)) == TRUE) {
		Display_Serious_Error_Message(unit_Name, code_Name, "\pNo Valid Rect", "\pInversed points?");
		goto Error_Handler;
	}

	file_Minimum_Size = 0;
	
	if (current_Spin_Object->special_Effect_Spin_Object != 0) {
		/*  */
		/* Nothing to do */
		/*  */
		Perform_OK_Dialog("\pSorry. The spin_Object cannot be exported because it is algorithmic");
		function_Result = SAFE_TRUE;
		goto Normal_Function_Exit;
	}

	if (current_Spin_Object->special_Effect_Spin_Object == 0) {
		
		if (Copy_Bitmap_Into_Pict_Command(&(current_Spin_Object->port_Info.spin_Object_Port_1)
																	, &spin_Object_Size_Rect
																	, stored_Bit_Depth
																	, &picture_Handle
																	, FALSE) != SAFE_TRUE) {
			goto Error_Handler;
		}
				
		if (EmptyRect(&(**picture_Handle).picFrame) == TRUE) {
			problem_Getting_Picture = TRUE;
		}
		else {
			/*  */
			/* Add in the sizes of Picture */
			/*  */
			temp_Size = GetHandleSize((Handle)picture_Handle);
			if (MemError() != noErr) {
				goto Error_Handler;
			}
			else {
				file_Minimum_Size += temp_Size;
			}
		}
	}
	
	if (((CARDINAL_32)file_Minimum_Size + (CARDINAL_32)MINIMUM_DISK_SPACE_TO_PRESERVE) > volume_Space) {

			Concat_Many(note_String, 3, "\p", volume_Name, "\p is too full for the file. (");
			Make_String_Long_String(work_String_2, (CARDINAL_8 *)note_String, volume_Space, NO_LIMIT,"\p)bytes free, (" );
			Make_String_Long_String(note_String
										, (CARDINAL_8 *)work_String_2
										, (file_Minimum_Size + MINIMUM_DISK_SPACE_TO_PRESERVE)
										, NO_LIMIT
										,"\p)bytes needed.  Find additional space for file." );

		goto Error_Handler_With_Message;
	}

	os_Error = HCreate(params_For_Export_Spin_Object->desired_File.vRefNum
									, params_For_Export_Spin_Object->desired_File.parID
									, params_For_Export_Spin_Object->desired_File.name
									, file_Creator
									, file_Type);
						
	if (os_Error == dupFNErr) {
		os_Error = HDelete(params_For_Export_Spin_Object->desired_File.vRefNum
									, params_For_Export_Spin_Object->desired_File.parID
									, params_For_Export_Spin_Object->desired_File.name);
		if (os_Error != noErr) {
			Concat_Many(note_String, 3,"\pFSDelete for file ", params_For_Export_Spin_Object->desired_File.name, "\p ID = ");
			Concat_Long_To_String(note_String, os_Error, NO_LIMIT);
			goto Error_Handler_With_Message;
		}

		os_Error = HCreate(params_For_Export_Spin_Object->desired_File.vRefNum
									, params_For_Export_Spin_Object->desired_File.parID
									, params_For_Export_Spin_Object->desired_File.name
							, file_Creator
							, file_Type);
	} /* if (os_Error == dupFNErr) */

	if (os_Error != noErr) {
		Concat_Many(note_String, 3,"\pCreate for file ", params_For_Export_Spin_Object->desired_File.name, "\p ID = ");
		Concat_Long_To_String(note_String, os_Error, NO_LIMIT);
		goto Error_Handler_With_Message;
	} 

	HCreateResFile(params_For_Export_Spin_Object->desired_File.vRefNum
									, params_For_Export_Spin_Object->desired_File.parID
									, params_For_Export_Spin_Object->desired_File.name);
	os_Error = ResError();
	if (os_Error != noErr) {
		Concat_Many(note_String, 3,"\pCreateResFile for file ", params_For_Export_Spin_Object->desired_File.name, "\p ID = ");
		Concat_Long_To_String(note_String, os_Error, NO_LIMIT);
		goto Error_Handler_With_Message;
	} 

	/*  */
	/* Crappy deficient OpenRFPerm needs ancient Working Directory */
	/*  */
	working_Directory_PB.ioCompletion = nil;
	working_Directory_PB.ioNamePtr = nil;
	working_Directory_PB.ioWDProcID = FILE_CREATOR;
	working_Directory_PB.ioVRefNum = params_For_Export_Spin_Object->desired_File.vRefNum;
	working_Directory_PB.ioWDDirID = params_For_Export_Spin_Object->desired_File.parID;
	
	os_Error = PBOpenWD(&working_Directory_PB, FALSE);
	if (os_Error != noErr) {
		goto Error_Handler;
	}
	working_Directory_In_Use = SAFE_TRUE;
	working_Directory = working_Directory_PB.ioVRefNum;

	/*=--------------------------------------------------------------------------=*/
	/* 'OpenRFPerm' used to prevent having to use 'SetVol' */
	/* 'StripAddress' called as per tech notes and THINK reference */
	/*=--------------------------------------------------------------------------=*/
	file_Reference = OpenRFPerm(((StringPtr)(StripAddress(
																	&(params_For_Export_Spin_Object->desired_File.name[0]))))
																	, working_Directory
																	, fsRdWrPerm );

	os_Error = ResError();
	if ((file_Reference == -1) || ( os_Error != noErr)) {
		Display_Serious_Error_Message(unit_Name, code_Name, "\pOS Error OpenResFile.", "\pNo resource fork?");
		goto Error_Handler;
	}

	if (problem_Getting_Picture != TRUE) {
		resource_Type = 'PICT';
		resource_ID = 128;

		if (Add_Resource_Handle((Handle)picture_Handle
												, resource_Type
												, resource_ID
												, "\pSpin_Object picture") != SAFE_TRUE) {
			goto Error_Handler;
		}
	}

	CloseResFile(file_Reference);
	os_Error = ResError();
	if ( os_Error != noErr) {
		Display_Serious_Error_Message(unit_Name, code_Name, "\pOS Error CloseResFile.", "\p");
	}

	file_Reference = 0;




	function_Result = SAFE_TRUE;
	if (problem_Getting_Picture == TRUE) {
		goto Error_Handler_No_Memory;
	}
	
	goto Normal_Function_Exit;

Error_Handler_No_Memory:
	function_Result = SAFE_FALSE;
	Display_Serious_Error_Message(unit_Name, code_Name, "\pInternal problem saving Spin_Object", "\pNo memory");
	goto Normal_Function_Exit;

Error_Handler_With_Message:
	function_Result = SAFE_FALSE;
	Display_Serious_Error_Message(unit_Name, code_Name, note_String, "\p");
	goto Normal_Function_Exit;

Error_Handler:
	Display_Serious_Error_Message(unit_Name, code_Name, "\pInternal problem saving spin_Object", "\p");
	function_Result = SAFE_FALSE;
		
Normal_Function_Exit:
		
	/*  */
	/* Placed here to ensure not accidentally ever left allocated */
	/*  */
	if (working_Directory_In_Use == SAFE_TRUE) {
		os_Error = CloseWD(working_Directory);
		if (os_Error != noErr) {
			goto Error_Handler;
		}
	}
		
	/*  */
	/* Delete picture if any used */
	/*  */
	if (current_Spin_Object->special_Effect_Spin_Object != 0) {
		if (picture_Handle != nil) {
			KillPicture(picture_Handle);
		}
	}
	
	return(function_Result);
	
} /* Safe_Boolean	Export_Spin_Object_Sequence... */
/*=**************************************************************************=*/


/*=**************************************************************************=*/
Safe_Boolean	Save_Spin_Object_Sequence(Save_Spin_Object_Rec	*params_For_Save_Spin_Object)
{
	INTEGER_16		scrap_Volume_Reference_Number;
	INTEGER_32		destination_Directory_ID;
	INTEGER_32		directory_Reference;
	INTEGER_16		file_Reference;
	WDPBRec			working_Directory_PB;
	INTEGER_16		working_Directory;
	Safe_Boolean		working_Directory_In_Use;
	Boolean			problem_Getting_Picture;
	Rect				rect_To_Save;
	CTabHandle		temp_Clut_Handle;
	PicHandle			picture_Handle;
	OSErr				os_Error;
	INTEGER_32		volume_Space;
	CARDINAL_32	memory_Needed;
	CARDINAL_32	file_Creator;
	CARDINAL_32	file_Type;
	CARDINAL_32	resource_Type;
	CARDINAL_32	file_Minimum_Size;
	CARDINAL_32	temp_Size;
	INTEGER_16		resource_ID;
	Spin_Object_Ptr		current_Spin_Object;
	Safe_Boolean		function_Result;
	CARDINAL_32	preferred_Size;

	Spin_Object_File_Header_Ptr				header;
	Spin_Object_File_Header_Hdl				resource_To_Create_Hdl;
	Memory_Allocation							resource_To_Create_Hdl_Memory_Initialized;
	
	Handle				template_Hdl;
	INTEGER_16		loop;
	INTEGER_16		index;
	CARDINAL_32	current_File_Offset;
	CARDINAL_8		*code_Name;
	Str255					resource_Comment;
	Str255					note_String;
	Str255					volume_Name;
	Str255					work_String_2;
	
	INTEGER_32		transfer_Count;
	INTEGER_32		transfer_Count_Copy;
/* END VAR */


	code_Name = "\pSave_Spin_Object_Sequence";
	function_Result = SAFE_FALSE;
	working_Directory_In_Use = SAFE_FALSE;
	resource_To_Create_Hdl_Memory_Initialized = MEMORY_NOT_INITIALIZED;
	problem_Getting_Picture = FALSE;
	picture_Handle = nil;
		
	/*  */
	/* current_Spin_Object uses a handle assumed to always be locked */
	/*  */
	current_Spin_Object = *(dream.spin_Objects.spin_Object[params_For_Save_Spin_Object->desired_Spin_Object]);
	
	if ((current_Spin_Object->initialized != SAFE_TRUE) 
		|| ((current_Spin_Object->special_Effect_Spin_Object != 0) && (current_Spin_Object->port_Info.spin_Object_Port_1.initialized != SAFE_TRUE))) {

		Get_NS_String(k_SPIN_STRINGS, k_ERROR_SPIN_OBJECT_UNSAVABLE, work_String_2);
		Perform_OK_Dialog(work_String_2);
		goto Error_Handler;
	}

	if ((dream.spin_Objects.spin_Object_Memory_In_Use[params_For_Save_Spin_Object->desired_Spin_Object]) != SAFE_TRUE) {
		function_Result = SAFE_FALSE;
		goto Normal_Function_Exit;
	}
	
	
	file_Creator= FILE_CREATOR;
	file_Type = k_SPIN_OBJECT_FILE_TYPE;
	note_String[0] = 0;
	
	os_Error = GetVInfo((Get_Drive_Number(params_For_Save_Spin_Object->desired_File.vRefNum)), volume_Name, &scrap_Volume_Reference_Number, (INTEGER_32 *)&volume_Space);
	if (os_Error != noErr) {
		Concat_Many(note_String, 3, "\pGetVInfo for volume ", volume_Name, "\p ID = ");
		Concat_Long_To_String(note_String, os_Error, NO_LIMIT);
		goto Error_Handler_With_Message;
	} /* if (os_Error != noErr) */
		
	preferred_Size = sizeof(Spin_Object_File_Header_Rec);
	resource_To_Create_Hdl = nil;
	resource_To_Create_Hdl_Memory_Initialized = MEMORY_NOT_INITIALIZED;

	if (Create_Handle(&preferred_Size
						, (CARDINAL_32)(10240L)
						, ALLOW_PURGING
						, DISALLOW_PARTIAL_AMOUNT
						, (Handle *)&resource_To_Create_Hdl
						, &resource_To_Create_Hdl_Memory_Initialized) != SAFE_TRUE) {
						
		Display_Low_Memory_Error_Message(unit_Name, code_Name, sizeof(Spin_Object_File_Header_Rec));
		
		function_Result = SAFE_FALSE;
		goto Error_Handler;
	}

	/*  */
	/* Start a section, handle not locked: not calling memory move indirectly */
	/*  */
	header = (Spin_Object_File_Header_Ptr)*resource_To_Create_Hdl;
	header->valid = SAFE_TRUE;
	header->version = k_SPIN_OBJECT_VERSION_NOW;
	header->backward_Compatible_Version = k_SPIN_OBJECT_VERSION_NOW;
	header->spin_Object_Size_Rect = current_Spin_Object->spin_Object_Background_Rect;

	/*  */
	/* header->file_Name will be reset to new name of file */
	/*  if file name is changed and change takes place now. */
	/*  */
	Copy_Pascal_String(&params_For_Save_Spin_Object->desired_File.name[0] , &current_Spin_Object->spin_Object_Name[0]);

	switch (current_Spin_Object->port_Info.spin_Object_Port_1.color_Type) {
		case k_COLOR_QUICKDRAW:
		case k_32_BIT_COLOR_QUICKDRAW:
		
			if (current_Spin_Object->port_Info.spin_Object_Port_1.image_Handle != nil) {
				header->spin_Object_Size_Rect = (**(current_Spin_Object->port_Info.spin_Object_Port_1.offscreen_Port.color_Port.portPixMap)).bounds;
				header->stored_Bit_Depth = (**(current_Spin_Object->port_Info.spin_Object_Port_1.offscreen_Port.color_Port.portPixMap)).pixelSize;
			}
			else {
				/*  */
				/* Cannot get here. paranoia code */
				/*  */
				SetRect(&(header->spin_Object_Size_Rect), 0,0,1,1);
				header->stored_Bit_Depth = 8;
			}
		break;
		
		case k_ORIGINAL_QUICKDRAW:
		default:
			header->spin_Object_Size_Rect = current_Spin_Object->port_Info.spin_Object_Port_1.offscreen_Port.old_Style_Port.portBits.bounds;
			header->stored_Bit_Depth = 1;
	} /* switch (current_Spin_Object->port_Info.spin_Object_Port_1.color_Type) */
	
					
	if (header->spin_Object_Size_Rect.left < current_Spin_Object->spin_Object_Background_Rect.left) {
		header->spin_Object_Size_Rect.left = current_Spin_Object->spin_Object_Background_Rect.left;
	}
					
	if (header->spin_Object_Size_Rect.top < current_Spin_Object->spin_Object_Background_Rect.top) {
		header->spin_Object_Size_Rect.top = current_Spin_Object->spin_Object_Background_Rect.top;
	}
					
	if (header->spin_Object_Size_Rect.right > current_Spin_Object->spin_Object_Background_Rect.right) {
		header->spin_Object_Size_Rect.right = current_Spin_Object->spin_Object_Background_Rect.right;
	}
					
	if (header->spin_Object_Size_Rect.bottom > current_Spin_Object->spin_Object_Background_Rect.bottom) {
		header->spin_Object_Size_Rect.bottom = current_Spin_Object->spin_Object_Background_Rect.bottom;
	}
	
	if (EmptyRect(&(header->spin_Object_Size_Rect)) == TRUE) {
		Display_Serious_Error_Message(unit_Name, code_Name, "\pNo Valid Rect", "\pInversed points?");
		goto Error_Handler;
	}
					
	/*  */
	/* Clean up strings for file tidiness */
	/*  */	
	for (loop = 0; (loop < 64) ; loop++) {
		header->file_Name[loop] = 0;
	}
			
	Copy_Pascal_String(current_Spin_Object->spin_Object_Name
		, header->file_Name);
			
	rect_To_Save = header->spin_Object_Size_Rect;
				
	/*  */
	/* Determine free space, first get header size */
	/*  */
	file_Minimum_Size = GetHandleSize((Handle)resource_To_Create_Hdl);
	if (MemError() != noErr) {
		goto Error_Handler;
	}
	
	/*  */
	/* End the section not calling memory move indirectly */
	/*  */

	/*  */
	/* Memory can indirectly move now and "header" needs */
	/* to be locked momentarily? No, forget it, just */
	/* reset right before Copy_Bitmap_Into_Pict_Command */
	/*  */
	MoveHHi((Handle)resource_To_Create_Hdl);
	if (MemError() != noErr) {
		goto Error_Handler;
	}
	
	HLock((Handle)resource_To_Create_Hdl);
	if (MemError() != noErr) {
		goto Error_Handler;
	}
	
	header = (Spin_Object_File_Header_Ptr)*resource_To_Create_Hdl;

	if (current_Spin_Object->special_Effect_Spin_Object == 0) {
		
		if (Copy_Bitmap_Into_Pict_Command(&(current_Spin_Object->port_Info.spin_Object_Port_1)
																	, &(header->spin_Object_Size_Rect)
																	, header->stored_Bit_Depth
																	, &picture_Handle
																	, TRUE) != SAFE_TRUE) {
			goto Error_Handler;
		}
				
		if (EmptyRect(&(**picture_Handle).picFrame) == TRUE) {
			problem_Getting_Picture = TRUE;
		}
		else {
			/*  */
			/* Add in the sizes of Picture */
			/*  */
			temp_Size = GetHandleSize((Handle)picture_Handle);
			if (MemError() != noErr) {
				goto Error_Handler;
			}
			else {
				file_Minimum_Size += temp_Size;
			}
		}
	}
	
	if (((CARDINAL_32)file_Minimum_Size + (CARDINAL_32)MINIMUM_DISK_SPACE_TO_PRESERVE) > volume_Space) {

			Concat_Many(note_String, 3, "\p", volume_Name, "\p is too full for the file. (");
			Make_String_Long_String(work_String_2, (CARDINAL_8 *)note_String, volume_Space, NO_LIMIT,"\p)bytes free, (" );
			Make_String_Long_String(note_String
										, (CARDINAL_8 *)work_String_2
										, (file_Minimum_Size + MINIMUM_DISK_SPACE_TO_PRESERVE)
										, NO_LIMIT
										,"\p)bytes needed.  Find additional space for file." );

		goto Error_Handler_With_Message;
	}

	os_Error = HCreate(params_For_Save_Spin_Object->desired_File.vRefNum
									, params_For_Save_Spin_Object->desired_File.parID
									, params_For_Save_Spin_Object->desired_File.name
									, file_Creator
									, file_Type);
						
	if (os_Error == dupFNErr) {
		os_Error = HDelete(params_For_Save_Spin_Object->desired_File.vRefNum
									, params_For_Save_Spin_Object->desired_File.parID
									, params_For_Save_Spin_Object->desired_File.name);
		if (os_Error != noErr) {
			Concat_Many(note_String, 3,"\pFSDelete for file ", params_For_Save_Spin_Object->desired_File.name, "\p ID = ");
			Concat_Long_To_String(note_String, os_Error, NO_LIMIT);
			goto Error_Handler_With_Message;
		}

		os_Error = HCreate(params_For_Save_Spin_Object->desired_File.vRefNum
									, params_For_Save_Spin_Object->desired_File.parID
									, params_For_Save_Spin_Object->desired_File.name
							, file_Creator
							, file_Type);
	} /* if (os_Error == dupFNErr) */

	if (os_Error != noErr) {
		Concat_Many(note_String, 3,"\pCreate for file ", params_For_Save_Spin_Object->desired_File.name, "\p ID = ");
		Concat_Long_To_String(note_String, os_Error, NO_LIMIT);
		goto Error_Handler_With_Message;
	} 

	HCreateResFile(params_For_Save_Spin_Object->desired_File.vRefNum
									, params_For_Save_Spin_Object->desired_File.parID
									, params_For_Save_Spin_Object->desired_File.name);
	os_Error = ResError();
	if (os_Error != noErr) {
		Concat_Many(note_String, 3,"\pCreateResFile for file ", params_For_Save_Spin_Object->desired_File.name, "\p ID = ");
		Concat_Long_To_String(note_String, os_Error, NO_LIMIT);
		goto Error_Handler_With_Message;
	} 

	/*  */
	/* Crappy deficient OpenRFPerm needs ancient Working Directory */
	/*  */
	working_Directory_PB.ioCompletion = nil;
	working_Directory_PB.ioNamePtr = nil;
	working_Directory_PB.ioWDProcID = FILE_CREATOR;
	working_Directory_PB.ioVRefNum = params_For_Save_Spin_Object->desired_File.vRefNum;
	working_Directory_PB.ioWDDirID = params_For_Save_Spin_Object->desired_File.parID;
	
	os_Error = PBOpenWD(&working_Directory_PB, FALSE);
	if (os_Error != noErr) {
		goto Error_Handler;
	}
	working_Directory_In_Use = SAFE_TRUE;
	working_Directory = working_Directory_PB.ioVRefNum;

	/*=--------------------------------------------------------------------------=*/
	/* 'OpenRFPerm' used to prevent having to use 'SetVol' */
	/* 'StripAddress' called as per tech notes and THINK reference */
	/*=--------------------------------------------------------------------------=*/
	file_Reference = OpenRFPerm(((StringPtr)(StripAddress(
																	&(params_For_Save_Spin_Object->desired_File.name[0]))))
																	, working_Directory
																	, fsRdWrPerm );

	os_Error = ResError();
	if ((file_Reference == -1) || ( os_Error != noErr)) {
		Display_Serious_Error_Message(unit_Name, code_Name, "\pOS Error OpenResFile.", "\pNo resource fork?");
		goto Error_Handler;
	}

	resource_Type = k_SPIN_OBJECT_MAIN_RESOURCE_TYPE;
	resource_ID = k_BASE_SPIN_OBJECT_RESOURCE_ID;
	Get_NS_String(k_DREAM_STRINGS, k_DONT_EDIT_RESOURCE_COMMENT, resource_Comment);

	if (Add_Resource_Handle((Handle)resource_To_Create_Hdl
											, resource_Type
											, resource_ID
											, resource_Comment) != SAFE_TRUE) {
		goto Error_Handler;
	}

	if (current_Spin_Object->special_Effect_Spin_Object == 0) {
		if (problem_Getting_Picture != TRUE) {
			resource_Type = 'PICT';
			resource_ID = k_BASE_SPIN_OBJECT_RESOURCE_ID;
			Get_NS_String(k_DREAM_STRINGS, k_RESOURCE_IS_JUST_THUMBNAIL_COMMENT, resource_Comment);

			if (Add_Resource_Handle((Handle)picture_Handle
													, resource_Type
													, resource_ID
													, resource_Comment) != SAFE_TRUE) {
				goto Error_Handler;
			}
		}
	}

	/*  */
	/* Now add the template for the header */
	/*  */

	resource_Type = 'TMPL';
	resource_ID = k_SPIN_OBJECT_HEADER_TEMPLATE_ID;
	
	template_Hdl = GetResource(resource_Type, resource_ID);
	if (template_Hdl != nil) {
	
		DetachResource(template_Hdl);
		os_Error = ResError();
		if ( os_Error != noErr) {
			Display_Serious_Error_Message(unit_Name, code_Name, "\pOS Error DetachResource.", "\p");
			goto Error_Handler;
		} 
	
		if (Add_Resource_Handle((Handle)template_Hdl
												, resource_Type
												, resource_ID
												, k_SPIN_OBJECT_HEADER_TEMPLATE_NAME) != SAFE_TRUE) {
			goto Error_Handler;
		}
		
		DisposeHandle(template_Hdl);
		os_Error = MemError();
		if ( os_Error != noErr) {
			Display_Serious_Error_Message(unit_Name, code_Name, "\pOS Error DisposeHandle.", "\p");
			goto Error_Handler;
		} 
		
	} /* if (template_Hdl != nil) */
	
	CloseResFile(file_Reference);
	os_Error = ResError();
	if ( os_Error != noErr) {
		Display_Serious_Error_Message(unit_Name, code_Name, "\pOS Error CloseResFile.", "\p");
	}

	file_Reference = 0;

	/*  */
	/* Data Fork not FULLY created!!! It is filled in using another step */
	/*  */

	/* = = = = = = = = = = = = = = = = = = = = = = = = = = = = = = = = = = = = = = = = = = = = = = = = = = = = = = = = */
	/* = = = = = = = = = = = = = = = = = = = = = = = = = = = = = = = = = = = = = = = = = = = = = = = = = = = = = = = = */
	/* = = = = = = = = = = = = = = = = = = = = = = = = = = = = = = = = = = = = = = = = = = = = = = = = = = = = = = = = */

	/*  */
	/* Now open data fork and fill it with our structure */
	/* The data fork is used because we might want to read */
	/* portions in during interrupts, as a background task, or */
	/* because we need to load it into memory in a specific */
	/* spot and there may not be enough room for two copies */
	/* to exchange from a loaded monster resource. It is true */
	/* that system 7 allows partial resource loading, but some */
	/* resource compressors, and other things fail when these are used. */
	/*  */									
	os_Error = HOpen(params_For_Save_Spin_Object->desired_File.vRefNum
									, params_For_Save_Spin_Object->desired_File.parID
									, &(params_For_Save_Spin_Object->desired_File.name[0])
									, fsRdWrPerm
									, &file_Reference);
									
	if (os_Error != noErr) {
		Concat_Many(note_String, 3,"\pHOpen for file ", &(params_For_Save_Spin_Object->desired_File.name[0]), "\p ID = ");
		Concat_Long_To_String(note_String, os_Error, NO_LIMIT);
		goto Error_Handler_With_Message;
	} 
	
	
	if (current_Spin_Object->spin_Info_Memory_In_Use != MEMORY_INITIALIZED) {
		goto Error_Handler;
	}
	
	if ((**(current_Spin_Object->spin_Info)).valid != SAFE_TRUE) {
		goto Error_Handler;
	}
		
	memory_Needed = sizeof(Spin_Pict_Info_Rec);
	memory_Needed += ((**(current_Spin_Object->spin_Info)).array_Actual_Count) * sizeof(Spin_Cell_Index);
	
	transfer_Count = memory_Needed;
	os_Error = FSWrite(file_Reference, &transfer_Count, (Ptr)*(current_Spin_Object->spin_Info));
	if (os_Error != noErr) {

		os_Error = FSClose(file_Reference);
		if ( os_Error != noErr) {
			/*  */
			/* Ignore, this was just an abortion tactic anyway */
			/*  */
		}

		Concat_Many(note_String, 3,"\pFSWrite for file ", &(params_For_Save_Spin_Object->desired_File.name[0]), "\p ID = ");
		Concat_Long_To_String(note_String, os_Error, NO_LIMIT);
		goto Error_Handler_With_Message;
	}

	os_Error = FSClose(file_Reference);
	if ( os_Error != noErr) {
		Display_Serious_Error_Message(unit_Name, code_Name, "\pOS Error FSClose.", "\p");
	}
	file_Reference = 0;

	/* = = = = = = = = = = = = = = = = = = = = = = = = = = = = = = = = = = = = = = = = = = = = = = = = = = = = = = = = */
	/* = = = = = = = = = = = = = = = = = = = = = = = = = = = = = = = = = = = = = = = = = = = = = = = = = = = = = = = = */
	/* = = = = = = = = = = = = = = = = = = = = = = = = = = = = = = = = = = = = = = = = = = = = = = = = = = = = = = = = */


	function_Result = SAFE_TRUE;
	if (problem_Getting_Picture == TRUE) {
		goto Error_Handler_No_Memory;
	}
	
	goto Normal_Function_Exit;

Error_Handler_No_Memory:
	function_Result = SAFE_FALSE;
	Display_Serious_Error_Message(unit_Name, code_Name, "\pInternal problem saving Spin_Object", "\pNo memory");
	goto Normal_Function_Exit;

Error_Handler_With_Message:
	function_Result = SAFE_FALSE;
	Display_Serious_Error_Message(unit_Name, code_Name, note_String, "\p");
	goto Normal_Function_Exit;

Error_Handler:
	Display_Serious_Error_Message(unit_Name, code_Name, "\pInternal problem saving spin_Object", "\p");
	function_Result = SAFE_FALSE;
		
Normal_Function_Exit:
	
	if (resource_To_Create_Hdl_Memory_Initialized == MEMORY_INITIALIZED) {
		if (Destroy_Handle((Handle *)&resource_To_Create_Hdl
							, &resource_To_Create_Hdl_Memory_Initialized) != SAFE_TRUE) {
							
			Display_Serious_Error_Message(unit_Name, code_Name, "\pError with Destroy_Handle", "\pSystem Corrupted?");
		}
	}
	
	/*  */
	/* Placed here to ensure not accidentally ever left allocated */
	/*  */
	if (working_Directory_In_Use == SAFE_TRUE) {
		os_Error = CloseWD(working_Directory);
		if (os_Error != noErr) {
			goto Error_Handler;
		}
	}
		
	/*  */
	/* Delete picture if any used */
	/*  */
	if (current_Spin_Object->special_Effect_Spin_Object != 0) {
		if (picture_Handle != nil) {
			KillPicture(picture_Handle);
		}
	}
	
	return(function_Result);
	
} /* Safe_Boolean	Save_Spin_Object_Sequence... */
/*=**************************************************************************=*/		

		
/*=**************************************************************************=*/		
Safe_Boolean	Append_Spin_Object_To_File(FSSpec *desired_File
								, Off_Screen_Port_Rec *port_Ptr
								, Spin_Pict_Info_Handle	spin_Info
								, CARDINAL_8 intensity
								, CARDINAL_8 horizontal_Angle
								, CARDINAL_8 vertical_Angle
								, CARDINAL_8 roll_Angle
								, CARDINAL_16	compression_Type
								, Handle row_Raw_Data_Hdl
								, Handle skip_List_Data_Hdl
								, Handle skip_List_Master_List_Hdl
								, Handle scan_Line_Bounds_Hdl)
{
	Safe_Boolean		function_Result;
	Rect						backdrop_Size_Rect;
	CARDINAL_32	array_Actual_Count;
	CARDINAL_32	stored_Bit_Depth;
	CARDINAL_32	data_Start_Offset;
	CARDINAL_32	picture_Row_Byte_Offset;
	CARDINAL_32	file_Size;
	CARDINAL_32	start_Index_Point;
	OSErr					os_Error;
	Ptr						data_To_Write_Pointer;
	INTEGER_16		file_Reference;
	INTEGER_16		loops_Remaining;
	IOParam				flush_File_Params;
	INTEGER_32		transfer_Count;
	INTEGER_32		transfer_Count_Copy;
	CARDINAL_32	raw_32_Bit_Word_1;
	CARDINAL_32	raw_32_Bit_Word_2;
	CARDINAL_32	memory_Needed;
	compression_Data_Header_Rec		compression_Data;
	CARDINAL_8		*code_Name;
	Str255					note_String;
	CARDINAL_8		scrap_Buffer_Padding[k_DISK_END_FRAME_DIVISION];
/* END VAR */


	code_Name = "\pAppend_Spin_Object_To_File";
	function_Result = SAFE_FALSE;

	/*  */
	/* First see if it can accept another frame without resizing handle */
	/* otherwise abort. Enhace soon. HACK HACK HACK */
	/*  */
	if ((*spin_Info)->valid != SAFE_TRUE) {
		Display_Serious_Error_Message(unit_Name, code_Name, "\pBad pointer for cell info.", "\p");
		goto Error_Handler;
	}

	if ((*spin_Info)->array_Actual_Count > (*spin_Info)->array_Max_Count_Size) {
		Display_Serious_Error_Message(unit_Name, code_Name, "\pNo more room for another cell in file.", "\pNeeds enhancement");
		goto Error_Handler;
	}

	switch(compression_Type) {
		case k_NO_COMPRESSION:
		case k_MERKLE_TIME_COMPRESSION:
		case k_EVEN_ODD_ROW_COMPRESSION:
			break;
		
		default:
			goto Error_Handler;
	} /* switch(compression_Type) */


	/* = = = = = = = = = = = = = = = = = = = = = = = = = = = = = = = = = = = = = = = = = = = = = = = = = = = = = = = = */
	/* = = = = = = = = = = = = = = = = = = = = = = = = = = = = = = = = = = = = = = = = = = = = = = = = = = = = = = = = */
	/* = = = = = = = = = = = = = = = = = = = = = = = = = = = = = = = = = = = = = = = = = = = = = = = = = = = = = = = = */

	
	transfer_Count_Copy = 0;
	
	switch(compression_Type) {
	
		case k_NO_COMPRESSION:
			switch (port_Ptr->color_Type) {
				case k_COLOR_QUICKDRAW:
				case k_32_BIT_COLOR_QUICKDRAW:
					if (port_Ptr->image_Handle != nil) {
						transfer_Count_Copy = GetHandleSize(port_Ptr->image_Handle);
						os_Error = MemError();
						if ( os_Error != noErr) {
							Display_Serious_Error_Message(unit_Name, code_Name, "\pBad Handle for Port.", "\p");
							goto Error_Handler;
						}
					}
				break;
				
				case k_ORIGINAL_QUICKDRAW:
				default:
					if (port_Ptr->image_Handle != nil) {
						transfer_Count_Copy = GetHandleSize(port_Ptr->image_Handle);
						os_Error = MemError();
						if ( os_Error != noErr) {
							Display_Serious_Error_Message(unit_Name, code_Name, "\pBad Handle for Port.", "\p");
							goto Error_Handler;
						}
					}
			} /* switch (port_Ptr->color_Type) */
		break; /*k_NO_COMPRESSION */
	
		case k_MERKLE_TIME_COMPRESSION:
				transfer_Count_Copy = 0;
		break; /* k_MERKLE_TIME_COMPRESSION */
	
		case k_EVEN_ODD_ROW_COMPRESSION:
			goto Error_Handler;
		break; /* k_EVEN_ODD_ROW_COMPRESSION */
		
		default:
			goto Error_Handler;
			
	} /* switch(compression_Type) */
		
	if (port_Ptr->image_Handle != nil) {
		backdrop_Size_Rect = (**(port_Ptr->offscreen_Port.color_Port.portPixMap)).bounds;
		picture_Row_Byte_Offset = ((**(port_Ptr->offscreen_Port.color_Port.portPixMap)).rowBytes & (~COLOR_PORT_PORTBITS_CHECK));
		stored_Bit_Depth = (**(port_Ptr->offscreen_Port.color_Port.portPixMap)).pixelSize;
	}
	else {
		Display_Serious_Error_Message(unit_Name, code_Name, "\pBad port for cell info extraction.", "\pProgrammer flaw");
		goto Error_Handler;
	}
	
	array_Actual_Count = (*spin_Info)->array_Actual_Count;
	
			
	if (array_Actual_Count == 0) {
		/*  */
		/* Almost enough for 2 thousand frames */
		/*  */
		data_Start_Offset = (k_DISK_MAJOR_DIVISION * 3UL);
	}
	else {
		data_Start_Offset = (*spin_Info)->cell_Info[array_Actual_Count - 1].data_Start_Offset;
		data_Start_Offset += (*spin_Info)->cell_Info[array_Actual_Count -1].data_Total_Size;
	
		/*  */
		/* Only allow 2 * 128 bits, or 32 bytes */
		/* for all file versions. For k_NO_COMPRESSION, will just result in */
		/*a little waste for some frame sizes close to 32K modulo. Otherwise */
		/* this is vital for treble safe flags indicating read status */
		/* in the k_EVEN_ODD_ROW_COMPRESSION technology */
		/*  */
	/* ====
		data_Start_Offset += k_COMPRESSION_STRUCTURE_PADDING;
	===== */
		
		data_Start_Offset += k_DISK_MAJOR_DIVISION - 1;
		/*  */
		/* Round to next 32K limit for start */
		/*  */
		data_Start_Offset = (data_Start_Offset & ~(k_DISK_MAJOR_DIVISION - 1UL));
	}


	(*spin_Info)->cell_Info[array_Actual_Count].data_Start_Offset = data_Start_Offset;
	(*spin_Info)->cell_Info[array_Actual_Count].data_Total_Size = transfer_Count_Copy;
	(*spin_Info)->cell_Info[array_Actual_Count].max_Picture_Bounds = backdrop_Size_Rect;
	(*spin_Info)->cell_Info[array_Actual_Count].picture_Row_Byte_Offset = picture_Row_Byte_Offset;
	(*spin_Info)->cell_Info[array_Actual_Count].color_Depth = stored_Bit_Depth;
	(*spin_Info)->cell_Info[array_Actual_Count].compression_Type = compression_Type;
	(*spin_Info)->cell_Info[array_Actual_Count].activation = intensity;
	(*spin_Info)->cell_Info[array_Actual_Count].horizontal_Angle = horizontal_Angle;
	(*spin_Info)->cell_Info[array_Actual_Count].vertical_Angle = vertical_Angle;
	(*spin_Info)->cell_Info[array_Actual_Count].roll_Angle = roll_Angle;
		
	if ((*spin_Info)->cell_Info[array_Actual_Count].color_Depth != (*spin_Info)->max_Depth) {
		Display_Serious_Error_Message(unit_Name, code_Name, "\pColor depth incompatible for cell in file.", "\pNeeds enhancement");
		goto Error_Handler;
	}

	raw_32_Bit_Word_1 = *(CARDINAL_32 *)&((*spin_Info)->cell_Info[array_Actual_Count].max_Picture_Bounds);
	raw_32_Bit_Word_2 = *(CARDINAL_32 *)&((*spin_Info)->max_Bounds);
	if (raw_32_Bit_Word_1 != raw_32_Bit_Word_2) {
		Display_Serious_Error_Message(unit_Name, code_Name, "\pPicture bounds incompatible for cell in file.", "\pNeeds enhancement");
		goto Error_Handler;
	}

	/*  */
	/* Now open data fork and fill it with our structure */
	/* The data fork is used because we might want to read */
	/* portions in during interrupts, as a background task, or */
	/* because we need to load it into memory in a specific */
	/* spot and there may not be enough room for two copies */
	/* to exchange from a loaded monster resource. It is true */
	/* that system 7 allows partial resource loading, but some */
	/* resource compressors, and other things fail when these are used. */
	/*  */									
	os_Error = HOpen((*desired_File).vRefNum
									, (*desired_File).parID
									, &((*desired_File).name[0])
									, fsRdWrPerm
									, &file_Reference);
									
	if (os_Error != noErr) {
		Concat_Many(note_String, 3,"\pHOpen for file ", &((*desired_File).name[0]), "\p ID = ");
		Concat_Long_To_String(note_String, os_Error, NO_LIMIT);
		goto Error_Handler_With_Message;
	} 


	if (compression_Type == k_MERKLE_TIME_COMPRESSION ) {
	
		loops_Remaining = 6;
		
		/*  */
		/* Create and write out a 3*16 = 48 byte stucture. (At the offset start)*/
		/* This preserves 16 byte alignment for first item in struct). */
		/*  */
		compression_Data.sentinel_1 = SAFE_TRUE;
		compression_Data.sentinel_2 = SAFE_TRUE;
		compression_Data.row_Raw_Data_Size = 0;
		compression_Data.row_Raw_Data_Start_Offset = 0;
		compression_Data.skip_List_Data_Size = 0;
		compression_Data.skip_List_Data_Start_Offset = 0;
		compression_Data.skip_List_Master_List_Size = 0;
		compression_Data.skip_List_Master_List_Start_Offset = 0;
		compression_Data.scan_Line_Bounds_Size = 0;
		compression_Data.scan_Line_Bounds_Offset = 0;
		compression_Data.pad_Bytes_For_48_Part_1 = 0;
		compression_Data.pad_Bytes_For_48_Part_2 = 0;
		compression_Data.pad_Bytes_For_48_Part_3 = 0;
		
		transfer_Count_Copy = GetHandleSize(scan_Line_Bounds_Hdl);
		if (transfer_Count_Copy == 0) {
			goto Error_Handler;
		}
		
		if (sizeof(compression_Data) != 48) {
			Display_Serious_Error_Message(unit_Name, code_Name, "\pImproper compiler used (padding/packing not normal", "\pProgramming Error");
			goto Error_Handler;
		}
		
		/*  */
		/* Fill in compression_Data, and calculate locations of */
		/* each item relative to this data and including rounding up */
		/* to nearest paragragh (128 bits) for each item stored. */
		/*  */
		compression_Data.row_Raw_Data_Size = GetHandleSize(row_Raw_Data_Hdl);
		if (compression_Data.row_Raw_Data_Size == 0) {
			Display_Serious_Error_Message(unit_Name, code_Name, "\pBad Handle for compression.", "\p");
			goto Error_Handler;
		}
		
		compression_Data.skip_List_Data_Size = GetHandleSize(skip_List_Data_Hdl);
		if (compression_Data.skip_List_Data_Size == 0) {
			goto Error_Handler;
		}
		
		compression_Data.skip_List_Master_List_Size = GetHandleSize(skip_List_Master_List_Hdl);
		if (compression_Data.skip_List_Master_List_Size == 0) {
			goto Error_Handler;
		}
		
		compression_Data.scan_Line_Bounds_Size = GetHandleSize(scan_Line_Bounds_Hdl);
		if (compression_Data.scan_Line_Bounds_Size == 0) {
			goto Error_Handler;
		}


		if ((data_Start_Offset & 0x0000000F) != 0) {
			Display_Serious_Error_Message(unit_Name, code_Name, "\pBad offset for data_Start_Offset.", "\pBad programming");
			goto Error_Handler;
		}

		
		/*  */
		/* Set relative offsets (these are not directly tied to (data_Start_Offset) */
		/* though it is assumed that "data_Start_Offset" is at least aligned */
		/* already to a paragraph (though it is actually aligned to 32K!) */
		/* k_DISK_MAJOR_DIVISION for high speed reading in parallel array. */
		/*  */
		compression_Data.row_Raw_Data_Start_Offset = sizeof(compression_Data);
		
		compression_Data.skip_List_Data_Start_Offset = compression_Data.row_Raw_Data_Start_Offset;
		compression_Data.skip_List_Data_Start_Offset += compression_Data.row_Raw_Data_Size;
		/*  */
		/* Ensure alignmnet to paragraph bounds for move16 and other */
		/* purposes */
		/*  */
		compression_Data.skip_List_Data_Start_Offset += 15UL;
		compression_Data.skip_List_Data_Start_Offset = (compression_Data.skip_List_Data_Start_Offset & 0xFFFFFFF0);
		
		
		compression_Data.skip_List_Master_List_Start_Offset = compression_Data.skip_List_Data_Start_Offset;
		compression_Data.skip_List_Master_List_Start_Offset += compression_Data.skip_List_Data_Size;
		/*  */
		/* Ensure alignmnet to paragraph bounds for move16 and other */
		/* purposes */
		/*  */
		compression_Data.skip_List_Master_List_Start_Offset += 15UL;
		compression_Data.skip_List_Master_List_Start_Offset = (compression_Data.skip_List_Master_List_Start_Offset & 0xFFFFFFF0);
		
		
		compression_Data.scan_Line_Bounds_Offset = compression_Data.skip_List_Master_List_Start_Offset;
		compression_Data.scan_Line_Bounds_Offset += compression_Data.skip_List_Master_List_Size;
		/*  */
		/* Ensure alignmnet to paragraph bounds for move16 and other */
		/* purposes */
		/*  */
		compression_Data.scan_Line_Bounds_Offset += 15UL;
		compression_Data.scan_Line_Bounds_Offset = (compression_Data.scan_Line_Bounds_Offset & 0xFFFFFFF0);
	
		/*  */
		/* Store total exact estimated size into control structure */
		/*  */
		transfer_Count_Copy = compression_Data.scan_Line_Bounds_Offset;
		transfer_Count_Copy += compression_Data.scan_Line_Bounds_Size;
		
		/*  */
		/* Size is pesky, We should round up to nearest harmonious offset. */
		/* Right now it is byte aligned for tail and 512 byte is better and better still is */
		/* 2K or 32 K but to save space we stick to 2K for bias of Barracuda dual headed parallel drive. */
		/*  */
		
		transfer_Count_Copy += (k_DISK_END_FRAME_DIVISION - 1UL);
		/*  */
		/* Round to next 2K limit for start */
		/*  */
		transfer_Count_Copy = (transfer_Count_Copy & ~(k_DISK_END_FRAME_DIVISION - 1UL));
		
		(*spin_Info)->cell_Info[array_Actual_Count].data_Total_Size = transfer_Count_Copy;				
	}
	else {
		loops_Remaining = 1;
	}
	
	
	while (loops_Remaining > 0) {
		
		if (compression_Type == k_MERKLE_TIME_COMPRESSION ) {
			/*  */
			/* Each time through loop copy one of four handles and */
			/* always pre-align data to start on 16 byte bounds just as */
			/* real handles do. First handle is 512 byte aligned but only needs */
			/* 128 bit for certain (move16). Therfore we stuff 4 long words at front */
			/* for extra info actually we stuff another 
			/*  */
			switch (loops_Remaining) {
				case 6:
					/*  */
					/* Write Header, Then write first handle in next loop */
					/*  */
					transfer_Count_Copy = sizeof(compression_Data);
					data_To_Write_Pointer = (Ptr)&compression_Data;
					/*  */
					/* "data_Start_Offset" is already pointing to proper start. */
					/*  */
					start_Index_Point = data_Start_Offset;
				break;

				case 5:
					transfer_Count_Copy = compression_Data.row_Raw_Data_Size;
					/*  */
					/* "data_Start_Offset" needs incrementing */
					/*  */
					data_Start_Offset = start_Index_Point + compression_Data.row_Raw_Data_Start_Offset;
					data_To_Write_Pointer = (Ptr)*row_Raw_Data_Hdl;
				break;

				case 4:
					transfer_Count_Copy = compression_Data.skip_List_Data_Size;
					data_Start_Offset = start_Index_Point + compression_Data.skip_List_Data_Start_Offset;
					data_To_Write_Pointer = (Ptr)*skip_List_Data_Hdl;
				break;

				case 3:
					transfer_Count_Copy = compression_Data.skip_List_Master_List_Size;
					data_Start_Offset = start_Index_Point + compression_Data.skip_List_Master_List_Start_Offset;
					data_To_Write_Pointer = (Ptr)*skip_List_Master_List_Hdl;
				break;

				case 2:
					transfer_Count_Copy = compression_Data.scan_Line_Bounds_Size;
					data_Start_Offset = start_Index_Point + compression_Data.scan_Line_Bounds_Offset;
					data_To_Write_Pointer = (Ptr)*scan_Line_Bounds_Hdl;
				break;
				
				case 1:
					data_Start_Offset = start_Index_Point 
														+ compression_Data.scan_Line_Bounds_Offset 
														+ compression_Data.scan_Line_Bounds_Size;

					transfer_Count_Copy = ((*spin_Info)->cell_Info[array_Actual_Count].data_Total_Size 
																+ start_Index_Point)
																	- data_Start_Offset;
															
					Fill_Word_Buffer((Ptr)&scrap_Buffer_Padding[0], (k_DISK_END_FRAME_DIVISION >> 1), 0x0000FFFF);
					data_To_Write_Pointer = (Ptr)&scrap_Buffer_Padding[0];
				break;
				
				default:
				Display_Serious_Error_Message(unit_Name, code_Name, "\pBad write stage for cell in file.", "\pProgrammer error");
				goto Error_Handler;
			
			} /* switch (loops_Remaining) */			
				
	
		} /* if (compression_Type == k_MERKLE_TIME_COMPRESSION ) */
		else {
			data_To_Write_Pointer = (Ptr)*port_Ptr->image_Handle;
		/* ===
			transfer_Count_Copy = transfer_Count_Copy;
		=== */
		} /* NOT  if (compression_Type == k_MERKLE_TIME_COMPRESSION ) */
		
		os_Error = GetEOF(file_Reference, (INTEGER_32 *)&file_Size);
		if (os_Error != noErr) {
			Concat_Many(note_String, 3,"\pGet file size failed for file ", (*desired_File).name, "\p. Disk full?, ID = ");
			Concat_Long_To_String(note_String, os_Error, NO_LIMIT);
			goto Error_Handler_With_Message;
		}
		
		if (file_Size < (data_Start_Offset + transfer_Count_Copy)) {
			/*  */
			/* Should use PBAllocContig first for entire file. */
			/* Oh well */
			/*  */
		
			os_Error = SetEOF(file_Reference, (data_Start_Offset + transfer_Count_Copy));
			if (os_Error != noErr) {
				Concat_Many(note_String, 3,"\pFile expansion failed for file ", (*desired_File).name, "\p. Disk full?, ID = ");
				Concat_Long_To_String(note_String, os_Error, NO_LIMIT);
				goto Error_Handler_With_Message;
			}
		}
	
		transfer_Count = transfer_Count_Copy;
		if (transfer_Count != 0) {
		
			os_Error = SetFPos(file_Reference,fsFromStart,data_Start_Offset);
			if (os_Error != noErr) {
				Concat_Many(note_String, 3,"\pSetFPos for file ", (*desired_File).name, "\p ID = ");
				Concat_Long_To_String(note_String, os_Error, NO_LIMIT);
				goto Error_Handler_With_Message;
			}
		
			os_Error = FSWrite(file_Reference, &transfer_Count, data_To_Write_Pointer);
			if (os_Error != noErr) {
			
				Concat_Many(note_String, 3,"\pFSWrite failed for file ", &((*desired_File).name[0]), "\p ID = ");
				Concat_Long_To_String(note_String, os_Error, NO_LIMIT);
				
				os_Error = FSClose(file_Reference);
				if ( os_Error != noErr) {
					/*  */
					/* Ignore, this was just an abortion tactic anyway */
					/*  */
				}
		
				goto Error_Handler_With_Message;
			}
		}
		else {
			os_Error = noErr;
		}
		
		loops_Remaining = loops_Remaining - 1;
	} /* 	while (loops_Remaining > 0) */


	/*  */
	/* Before closing we could write the newest appended file control info */
	/*  */
	os_Error = SetFPos(file_Reference,fsFromStart,0);
	if (os_Error != noErr) {
		Concat_Many(note_String, 3,"\pSetFPos for file ", (*desired_File).name, "\p ID = ");
		Concat_Long_To_String(note_String, os_Error, NO_LIMIT);
		goto Error_Handler_With_Message;
	}

	memory_Needed = sizeof(Spin_Pict_Info_Rec);
	memory_Needed += ((**(spin_Info)).array_Actual_Count) * (CARDINAL_32)sizeof(Spin_Cell_Index);
	
	transfer_Count = memory_Needed;
	os_Error = FSWrite(file_Reference, &transfer_Count, (Ptr)*(spin_Info));
	if (os_Error != noErr) {

		os_Error = FSClose(file_Reference);
		if ( os_Error != noErr) {
			/*  */
			/* Ignore, this was just an abortion tactic anyway */
			/*  */
		}

		Concat_Many(note_String, 3,"\pFSWrite for file ", &((*desired_File).name[0]), "\p ID = ");
		Concat_Long_To_String(note_String, os_Error, NO_LIMIT);
		goto Error_Handler_With_Message;
	}
	
	/*  */
	/* Note : Asyncronous file flush request at end of file usage */
	/*  */
	/*  */
	/*  */
	/*  */
	/*  */
	/*  */
	/* Make Async One day soon if possible */
	/* But make params GLOBAL and governed if done so. */
	/*  */
	flush_File_Params.ioCompletion = nil;
	flush_File_Params.ioRefNum = file_Reference;
	os_Error = PBFlushFile((ParmBlkPtr)&flush_File_Params, FALSE);
	if ((os_Error != noErr) && (os_Error != 1)){
		Display_Serious_Error_Message(unit_Name, code_Name, "\pOS Error PBFlushFile.", "\pAsync close not allowed?");
	}

	os_Error = FSClose(file_Reference);
	if ( os_Error != noErr) {
		Display_Serious_Error_Message(unit_Name, code_Name, "\pOS Error FSClose.", "\p");
	}
	file_Reference = 0;
	
	(*spin_Info)->array_Actual_Count += 1;

	/* = = = = = = = = = = = = = = = = = = = = = = = = = = = = = = = = = = = = = = = = = = = = = = = = = = = = = = = = */
	/* = = = = = = = = = = = = = = = = = = = = = = = = = = = = = = = = = = = = = = = = = = = = = = = = = = = = = = = = */
	/* = = = = = = = = = = = = = = = = = = = = = = = = = = = = = = = = = = = = = = = = = = = = = = = = = = = = = = = = */


	function_Result = SAFE_TRUE;
	
	goto Normal_Function_Exit;

Error_Handler_No_Memory:
	function_Result = SAFE_FALSE;
	Display_Serious_Error_Message(unit_Name, code_Name, "\pInternal problem saving Spin_Object", "\pNo memory");
	goto Normal_Function_Exit;

Error_Handler_With_Message:
	function_Result = SAFE_FALSE;
	Display_Serious_Error_Message(unit_Name, code_Name, note_String, "\p");
	goto Normal_Function_Exit;

Error_Handler:
	Display_Serious_Error_Message(unit_Name, code_Name, "\pInternal problem saving spin_Object", "\p");
	function_Result = SAFE_FALSE;
		
Normal_Function_Exit:
					
	return(function_Result);
	
} /* Safe_Boolean	Append_Spin_Object_To_File... */
/*=**************************************************************************=*/		

		
/*=**************************************************************************=*/		
Safe_Boolean Load_Some_More_Spin_Object(void)
/*  */
/* Called as a slow idle loop event task distributed as part */
/* of background I/O while backgrounds are loading. */
/* Governor should have tuning based on whether any animation */
/* is really occuring before deciding how often t call this routine */
/*  */
{
	Safe_Boolean		function_Result;
	CARDINAL_8		*code_Name;
	Spin_Object_Ptr		current_Spin_Object;
	Boolean				still_Searching;
	CARDINAL_16	index_Used;
	CARDINAL_16	current_Spin_Object_Index;
/* END VAR */


	code_Name = "\pLoad_Some_More_Spin_Object";
	function_Result = SAFE_FALSE;

	if (dream.spin_Objects.spin_Objects_Still_Loading == 0) {
		function_Result = SAFE_TRUE;
		goto Normal_Function_Exit;
	}


	/*  */
	/* Search for a Spin_Object to load some data into, ugly but not too slow */
	/* a tradeoff for now */
	/*  */
	still_Searching = TRUE;
	current_Spin_Object_Index = 0;
	for(index_Used = 0; ((index_Used < k_MAX_ACTIVE_SPIN_OBJECTS) && (still_Searching == TRUE)); index_Used++) {
		
		if (dream.spin_Objects.spin_Object_Memory_In_Use[index_Used] == SAFE_TRUE) {
		current_Spin_Object = *(dream.spin_Objects.spin_Object[index_Used]);
			if (current_Spin_Object->initialized == SAFE_TRUE) {
			
				/*  */
				/* NOTE: Also check and compare here for background load needed */
				/* before skipping over them */
				/*  */
			
				if (current_Spin_Object->spin_Object_Remaining_To_Load == SAFE_TRUE) {
				
					/*  */
					/* Found a potentially good Spin_Object to work from */
					/*  */			
					
					if (current_Spin_Object->spin_Object_Loading_Last_Index >= k_LAST_BACKGROUND_STAGE) {
						/*  */
						/* No more items left to load. skip to next Spin_Object */
						/* Correction made here. no complaint issued though */
						/*  */
						current_Spin_Object->spin_Object_Remaining_To_Load = SAFE_FALSE;
					}
					else {
						still_Searching = FALSE;
						current_Spin_Object_Index = index_Used;
					}
				}
				else {
						/*  */
						/* This one is already laoded */
						/*  */
				}
			}
		}
	} /* for(index_Used = 0; ... */
	
	if (still_Searching == TRUE) {
		/*  */
		/* Kill task, nothing left to safely load */
		/*  */
		dream.spin_Objects.spin_Objects_Still_Loading = 0;

		function_Result = SAFE_TRUE;
		goto Normal_Function_Exit;
	}
	
	
	/*  */
	/* Load some more spin_Object done here. In this version, */
	/* we merely promote the status to closure.
	/*  */
	current_Spin_Object->spin_Object_Remaining_To_Load = SAFE_FALSE;
	current_Spin_Object->spin_Object_Loading_Last_Index = k_LAST_BACKGROUND_STAGE;
	

	function_Result = SAFE_TRUE;
	goto Normal_Function_Exit;
	
Error_Handler:
	function_Result = SAFE_FALSE;

Normal_Function_Exit:

	return(function_Result);
	
} /* Safe_Boolean Load_Some_More_Spin_Object(void) */
/*=**************************************************************************=*/
	
	
/*=**************************************************************************=*/
Safe_Boolean	Initialize_Empty_Spin_Object(CARDINAL_32 *located_Position)
{
	Safe_Boolean		function_Result;
	CARDINAL_32	memory_Requested;
	INTEGER_16		index;
	INTEGER_16		position;
	Spin_Object_Ptr		current_Spin_Object;
	SignedByte			original_State;
	CARDINAL_8		*code_Name;
	Boolean				found_Location;
/* END VAR */


	code_Name = "\pInitialize_Empty_Spin_Object";
	function_Result = SAFE_FALSE;
	*located_Position = 0;
	
	if (g_Dreaming_Module_Globals_Initialized == SAFE_TRUE) {
		if (dream.spin_Objects.total_Spin_Objects < k_MAX_ACTIVE_SPIN_OBJECTS) {
			found_Location = FALSE;
			for (index = 0; ((index < k_MAX_ACTIVE_SPIN_OBJECTS) && (found_Location != TRUE)); index++) {
				if (dream.spin_Objects.spin_Object_Memory_In_Use[index] != MEMORY_INITIALIZED) {
					found_Location = TRUE;
					position = index;					
				}
			}
			
			if (found_Location != TRUE) {
				goto No_More_Spin_Objects_Error_Handler;
			}
			else {
			
				dream.spin_Objects.spin_Object[position] = nil;
				dream.spin_Objects.spin_Object_Memory_In_Use[position] = MEMORY_NOT_INITIALIZED;
				memory_Requested = sizeof(Spin_Object_Rec);
				if (Create_Handle( (CARDINAL_32 *)&memory_Requested
										, (CARDINAL_32)0L
										, ALLOW_PURGING
										, DISALLOW_PARTIAL_AMOUNT
										, (Handle *)&(dream.spin_Objects.spin_Object[position])
										, &(dream.spin_Objects.spin_Object_Memory_In_Use[position])) != SAFE_TRUE ) {
										
					Perform_OK_Dialog("\pMemory error allocating buffer_Memory (not enough?) (4)");
		
					goto Normal_Function_Exit;
				}
				
				if (Begin_Use_Of_Handle((Handle)dream.spin_Objects.spin_Object[position]
										, MOVE_HIGH_REQUIRED
										, &original_State) != SAFE_TRUE) {
					/*  */
					/* Ignore error in implementation */
					/*  */
				}
				
				/*  */
				/* Because locked, we can access using a pointer */
				/*  */		
				current_Spin_Object = *(dream.spin_Objects.spin_Object[position]);
				
				current_Spin_Object->isolated_Port_Drawing_Active = SAFE_FALSE;
				current_Spin_Object->port_Info.spin_Object_Port_1.initialized = SAFE_FALSE;
				current_Spin_Object->port_Info.spin_Object_Port_2.initialized = SAFE_FALSE;
				current_Spin_Object->spin_Info_Memory_In_Use = MEMORY_NOT_INITIALIZED;
				
				current_Spin_Object->file_Reference = 0;
				
				current_Spin_Object->spin_Info = nil;
				
				/*  */
				/* Gets properly set later onward */
				/*  */
				SetRect(&(current_Spin_Object->spin_Object_Background_Rect), 0, 0, 0, 0);
				
				/*  */
				/* Mostly paranoia only */
				/*  */
			/* ==
				current_Spin_Object->version = k_SPIN_OBJECT_VERSION_NOW;
				current_Spin_Object->backward_Compatible_Version = k_SPIN_OBJECT_BASE_VERSION_NOW;
				current_Spin_Object->spin_Object_Location.
			=== */

				current_Spin_Object->special_Effect_Spin_Object = 0;
				current_Spin_Object->spin_Object_Name[0] = 0;
				current_Spin_Object->spin_Object_Remaining_To_Load = SAFE_FALSE;
				current_Spin_Object->spin_Object_Loading_Last_Index = 0;
				
				current_Spin_Object->port_Info.offset_Offscreen_Pixel_X = 0;
				current_Spin_Object->port_Info.offset_Offscreen_Pixel_Y = 0;
				
				current_Spin_Object->initialized = SAFE_FALSE;
				
				*located_Position = position;
				function_Result = SAFE_TRUE;
				
			}
		
		}

	} /* if (g_Dreaming_Module_Globals_Initialized ... */

	if (function_Result != SAFE_TRUE) {
		goto Error_Handler;
	}	
	
	goto Normal_Function_Exit;
	
No_More_Spin_Objects_Error_Handler:
	Display_Serious_Error_Message(unit_Name, code_Name, "\pNo more Spin_Objects can be loaded", "\pDelete some");

Error_Handler:
	function_Result = SAFE_FALSE;
			
Normal_Function_Exit:
	return(function_Result);
	
} /* Safe_Boolean	Initialize_Empty_Spin_Object... */
/*=**************************************************************************=*/
	
	
/*=**************************************************************************=*/
Safe_Boolean	Destroy_A_Spin_Object(CARDINAL_32 spin_Object_To_Destroy)
{
	Safe_Boolean		function_Result;
	INTEGER_16		index;
	CARDINAL_8		*code_Name;
	Boolean				found_Location;
/* END VAR */


	code_Name = "\pDestroy_A_Spin_Object";
	function_Result = SAFE_FALSE;
	
	if (g_Dreaming_Module_Globals_Initialized != SAFE_TRUE) {
		goto Error_Handler;
	}	
	
	if (dream.spin_Objects.total_Spin_Objects < 1) {
		goto Error_Handler;
	}
	
	/*  */
	/* In future add logic to not delete spin_Object if depended upon */
	/*  */
	
	if ((**(dream.spin_Objects.spin_Object[spin_Object_To_Destroy])).spin_Info_Memory_In_Use == MEMORY_INITIALIZED) {
		if (Destroy_Handle((Handle *)&((**(dream.spin_Objects.spin_Object[spin_Object_To_Destroy])).spin_Info)
											, &((**(dream.spin_Objects.spin_Object[spin_Object_To_Destroy])).spin_Info_Memory_In_Use)) != SAFE_TRUE) {
			Display_Serious_Error_Message(unit_Name, code_Name, "\pError with DisposHandle", "\pSystem Corrupted?");
			goto Error_Handler;
		}
	}
	
	 if ((**(dream.spin_Objects.spin_Object[spin_Object_To_Destroy])).special_Effect_Spin_Object == 0) {
	 
		 if ((**(dream.spin_Objects.spin_Object[spin_Object_To_Destroy])).port_Info.spin_Object_Port_1.initialized == SAFE_TRUE) {
		 	if (Close_Offscreen_Pict_Port(&((**(dream.spin_Objects.spin_Object[spin_Object_To_Destroy])).port_Info.spin_Object_Port_1)) != SAFE_TRUE) {
				/*  */
				/* might as well complain about it? No not in this version */
				/*  */
				/* ===
		 		goto Error_Handler;
		 		==== */
		 	}
		 }
		
		 if ((**(dream.spin_Objects.spin_Object[spin_Object_To_Destroy])).port_Info.spin_Object_Port_2.initialized == SAFE_TRUE) {
		 	if (Close_Offscreen_Pict_Port(&((**(dream.spin_Objects.spin_Object[spin_Object_To_Destroy])).port_Info.spin_Object_Port_2)) != SAFE_TRUE) {
				/*  */
				/* might as well complain about it? No not in this version */
				/*  */
				/* ===
		 		goto Error_Handler;
		 		==== */
		 	}
		 }
		
		if (Destroy_Handle((Handle *)&(dream.spin_Objects.spin_Object[spin_Object_To_Destroy])
											, &(dream.spin_Objects.spin_Object_Memory_In_Use[spin_Object_To_Destroy])) != SAFE_TRUE) {
			Display_Serious_Error_Message(unit_Name, code_Name, "\pError with DisposHandle", "\pSystem Corrupted?");
			goto Error_Handler;
		}
		
		dream.spin_Objects.total_Spin_Objects = dream.spin_Objects.total_Spin_Objects -1;
		
		if (dream.spin_Objects.total_Spin_Objects == 0) {

			if (g_Spin_Ring_Buffer_Initialized == SAFE_TRUE) {
			
				/*  */	
				/* Closing down cache is a temporary and reversable manuever. */
				/*  */	
				if (Close_Down_Ring_Cache_If_Possible(g_Spin_Cache_Master_Ptr) != SAFE_TRUE) {
					/*  */
					/* might as well complain about it? No not in this version */
					/*  */
				}
			}
		}
	}
	
	
	function_Result = SAFE_TRUE;
	goto Normal_Function_Exit;

Error_Handler:
	function_Result = SAFE_FALSE;

Normal_Function_Exit:

	return(function_Result);

} /* Safe_Boolean	Destroy_A_Spin_Object( ... */
/*=**************************************************************************=*/
	
	
/*=**************************************************************************=*/
Safe_Boolean	Convert_Spin_Object_File_Immediate_Style(Convert_Spin_Object_Rec	*params_For_Convert_Spin_Object)
{
	Safe_Boolean		function_Result;
	Load_Spin_Object_Rec				fake;
/* END VAR */

	/*  */
	/* Window for update/import BETTER BE ONSCREEN SOMEWHERE */
	/*  */
	function_Result = Load_Convert_Spin_Object_File_Immediate_Style(&fake
																							, params_For_Convert_Spin_Object
																							, TRUE);
																																														
	return(function_Result);
}
/*=**************************************************************************=*/
	
	
/*=**************************************************************************=*/
Safe_Boolean	Load_Spin_Object_File_Immediate_Style(Load_Spin_Object_Rec	*params_For_Load_Spin_Object)
{
	Safe_Boolean		function_Result;
	Convert_Spin_Object_Rec				fake;
/* END VAR */

	function_Result = Load_Convert_Spin_Object_File_Immediate_Style(params_For_Load_Spin_Object
																							, &fake
																							, FALSE);
																							
	return(function_Result);
}
/*=**************************************************************************=*/		

		
/*=**************************************************************************=*/
Safe_Boolean	Load_Convert_Spin_Object_File_Immediate_Style(Load_Spin_Object_Rec	*params_For_Load_Spin_Object
													, Convert_Spin_Object_Rec	*params_For_Convert_Spin_Object
													, Boolean		importation_Desired)
{
	CARDINAL_32		index_Used;
	Safe_Boolean			user_Abort;
	CARDINAL_32 		located_Position;
	Safe_Boolean			function_Result;
	CARDINAL_8			*code_Name;
/* END VAR */


	code_Name = "\pLoad_Convert_Spin_Object_File_Immediate_Style";
	function_Result = SAFE_FALSE;
	
	if (Initialize_Empty_Spin_Object(&located_Position) == SAFE_TRUE) {
	
		params_For_Load_Spin_Object->desired_Spin_Object = located_Position;
		params_For_Convert_Spin_Object->desired_Spin_Object = located_Position;
		
		if (Load_Or_Import_Spin_Object_File(params_For_Load_Spin_Object
												, params_For_Convert_Spin_Object
												, importation_Desired) != SAFE_TRUE) {
		
			if (Destroy_A_Spin_Object(located_Position) != SAFE_TRUE) {
				/*  */
				/* Not a big problem. We are quitting anyway. */
				/*  */
				Display_Serious_Error_Message(unit_Name, code_Name, "\pDestroy_A_Spin_Object failed", "\pCorruption?");
			}
			
			goto Error_Handler;
		}
		else {
			/*  */
			/* Add to counter. Finished with it. */
			/* Already incremented */
			/*  */
			dream.spin_Objects.spin_Objects_Still_Loading += 1;
			
			/*  */
			/* Cycle through loading the Items in the Spin_Object. Normally done in stages */
			/* in the event loop */
			/*  */
			while (dream.spin_Objects.spin_Objects_Still_Loading > 0) {
			
				if (Load_Some_More_Spin_Object() != SAFE_TRUE) {
					/*  */
					/* Kill task, nothing left to safely load */
					/*  */
					dream.spin_Objects.spin_Objects_Still_Loading = 0;
				}
				
			}
	
			/*  */
			/* If This spin_Object that was loaded just replaced another.... */
			/* Naw, impracticle to swap atomically. */
			/*  */
			
		
		} /* NOT if (Load_Spin_Object_File(located_Position, ... */
	}
	else {
		goto No_More_Spin_Objects_Error_Handler;
	}
	
	/*  */
	/* This should be moved out in future versions unless window actually desired */
	/* to be visible just merely because file loaded */
	/*  */
/* ===
	if (Select_Spin_Window_Display_First_Time() != SAFE_TRUE) {
	}
=== */
		
	/*  */
	/* Something was loaded, enable all items */
	/*  */
	if (Reset_Menu_Control_Selectability() != SAFE_TRUE) {
		goto Error_Handler;
	}

	if (importation_Desired == TRUE) {
		Copy_Pascal_String(params_For_Convert_Spin_Object->output_Converted_File.name
				, ((**(dream.spin_Objects.spin_Object[located_Position])).spin_Object_Name));
	
		params_For_Convert_Spin_Object->desired_File = params_For_Convert_Spin_Object->output_Converted_File;
	}
	else {
	
	}


	function_Result = SAFE_TRUE;
	goto Normal_Function_Exit;
	
No_More_Spin_Objects_Error_Handler:
	Display_Serious_Error_Message(unit_Name, code_Name, "\pNo more Spin_Objects can be added", "\pRemove some");

Error_Handler:
	function_Result = SAFE_FALSE;

Normal_Function_Exit:

	return(function_Result);
	
} /* Safe_Boolean	Load_Convert_Spin_Object_File_Immediate_Style */
/*=**************************************************************************=*/
	
	
/*=**************************************************************************=*/				
Safe_Boolean	Specify_Spin_Object_To_Load(FSSpec		*file_Specification
															, Safe_Boolean *user_Abort)
{
	INTEGER_16 		number_Of_Types;
	Boolean				user_Clicked_Ok;
	INTEGER_32		source_Directory_ID;
	INTEGER_16		source_Volume_Reference;
	SFTypeList 			type_List;
	SFReply				sf_Reply;
	Safe_Boolean		function_Result;
/* END VAR */


	function_Result = SAFE_FALSE;
	*user_Abort = SAFE_FALSE;

	number_Of_Types = 1;
	type_List[0] = k_SPIN_OBJECT_FILE_TYPE;

	user_Clicked_Ok = Perform_Get_File_Dialog(number_Of_Types
										, &(type_List)
										, &sf_Reply);
											
	if ((user_Clicked_Ok != TRUE) || (sf_Reply.good != TRUE)) {
		/*  */
		/* User Abort, no file name */
		/*  */
		*user_Abort = SAFE_TRUE;
		function_Result = SAFE_FALSE;
		goto Normal_Function_Exit;
	}
	
	Get_Last_Volume_Reference_And_Directory_ID(&source_Volume_Reference, &source_Directory_ID);

	file_Specification->vRefNum = source_Volume_Reference;
	file_Specification->parID = source_Directory_ID;
	Copy_Pascal_String(sf_Reply.fName, file_Specification->name);
		
	function_Result = SAFE_TRUE;
	goto Normal_Function_Exit;

Error_Handler:
	function_Result = SAFE_FALSE;
		
Normal_Function_Exit:
	return(function_Result);
	
} /* Safe_Boolean	Specify_Spin_Object_To_Load... */
/*=**************************************************************************=*/
	
	
/*=**************************************************************************=*/				
Safe_Boolean	Specify_Spin_Object_Pict_To_Import(FSSpec		*file_Specification
															, Safe_Boolean	*file_Is_Pict_File_Type
															, INTEGER_16		*resource_Index
															, Safe_Boolean *user_Abort
															, CARDINAL_32	*selected_Pict_Width
															, CARDINAL_32	*selected_Pict_Height)
{
	INTEGER_16 		number_Of_Types;
	INTEGER_16		input_File_Reference;
	Boolean				user_Clicked_Ok;
	INTEGER_32		source_Directory_ID;
	INTEGER_16		source_Volume_Reference;
	SFTypeList 			type_List;
	INTEGER_16		save_Ref;
	SFReply				sf_Reply;
	Safe_Boolean		resource_Sources_Swapped;
	OSErr					os_Error;
	INTEGER_16		resource_Id;
	ResType				resource_Type;
	PictInfo				picture_Info;
	Safe_Boolean		function_Result;
	INTEGER_32		default_Index;
	INTEGER_32		lowest_Index;
	INTEGER_32		highest_Index;
	CARDINAL_8		*code_Name;
	Rect						selected_Pict_Rect;
	Handle				temp_Resource;
	Boolean				still_Expecting_Pict_Resources;
	CARDINAL_32	total_Picts;
	Str255					message_String;
	Str255					temp_String;
/* END VAR */


	function_Result = SAFE_FALSE;
	code_Name = "\pSpecify_Spin_Object_Pict_To_Import";
	*user_Abort = SAFE_FALSE;
	input_File_Reference = 0;
	resource_Sources_Swapped = SAFE_FALSE;
	*file_Is_Pict_File_Type = SAFE_FALSE;
	temp_Resource = nil;

	number_Of_Types = -1;
	type_List[0] = 0;

	user_Clicked_Ok = Perform_Get_File_Dialog(number_Of_Types
										, &(type_List)
										, &sf_Reply);
											
	if ((user_Clicked_Ok != TRUE) || (sf_Reply.good != TRUE)) {
		/*  */
		/* User Abort, no file name */
		/*  */
		*user_Abort = SAFE_TRUE;
		function_Result = SAFE_FALSE;
		goto Normal_Function_Exit;
	}
	
	Get_Last_Volume_Reference_And_Directory_ID(&source_Volume_Reference, &source_Directory_ID);

	file_Specification->vRefNum = source_Volume_Reference;
	file_Specification->parID = source_Directory_ID;
	Copy_Pascal_String(sf_Reply.fName, file_Specification->name);
	
	/*  */
	/* Verify That a PICT resource exists to import or not */
	/*  */
	
	save_Ref = CurResFile();
	
	/*  */
	/* Open for read only permission */
	/*  */
	still_Expecting_Pict_Resources = TRUE;
	
	input_File_Reference = HOpenResFile (source_Volume_Reference
						, source_Directory_ID
						, sf_Reply.fName
						, fsRdPerm );
						
	if ((input_File_Reference == -1) || (ResError() != noErr)) {
		still_Expecting_Pict_Resources = FALSE;
	}
	
	total_Picts = 0;
	
	if (still_Expecting_Pict_Resources == TRUE) {												
		UseResFile(input_File_Reference);
		os_Error = ResError();
		if (os_Error != noErr) {
			still_Expecting_Pict_Resources = FALSE;
		}
		else {
		
			resource_Sources_Swapped = SAFE_TRUE;
				
			total_Picts = Count1Resources('PICT');
			if (ResError() != noErr) {
				still_Expecting_Pict_Resources = FALSE;
			}
			else {
				
				if (total_Picts < 2) {
					still_Expecting_Pict_Resources = FALSE;
				}
			}
		}
	}
	
	if (still_Expecting_Pict_Resources != TRUE) {
		/*  */
		/* Aha! now we try to look at the data fork instead. */
		/*  */
		if (resource_Sources_Swapped == SAFE_TRUE) {
			
			UseResFile(save_Ref);
			if (ResError() != noErr) {
				/*  */
				/* This should be recoverable from, enhance one day  */
				/*  */
			}
			resource_Sources_Swapped = SAFE_FALSE;
		}

		if (input_File_Reference != 0) {
			CloseResFile(input_File_Reference);
			if (ResError() != noErr) {

			}
			input_File_Reference = 0;
		}


		if (Guarantee_File_Is_PICT_Type(source_Volume_Reference
						, source_Directory_ID
						, sf_Reply.fName
						, (INTEGER_32 *)selected_Pict_Width
						, (INTEGER_32 *)selected_Pict_Height) == SAFE_TRUE) {
			total_Picts = 1;
			*file_Is_Pict_File_Type = SAFE_TRUE;
		}
		else {
			total_Picts = 0;
		}

	}/* if (still_Expecting_Pict_Resources != TRUE) */	
	
	if (total_Picts < 1) {
		Get_NS_String(k_DREAM_STRINGS, k_ERROR_NO_PICT_RESOURCES_IN_FILE, message_String);
		Perform_OK_Dialog(message_String);
		goto Error_Handler;
	}
	
	if (*file_Is_Pict_File_Type == SAFE_TRUE) {
		Get_NS_String(k_SPIN_STRINGS, k_ERROR_WAS_PICT_FILE_INSTEAD_OF_PICS, message_String);
		Perform_OK_Dialog(message_String);
		
		if (Count_PICT_File_Neighbors((file_Specification), &total_Picts) != SAFE_TRUE) {
			Get_NS_String(k_DREAM_STRINGS, k_SORRY_WRONG_FILE_INDEX, message_String);
			Perform_OK_Dialog(message_String);
			goto Error_Handler;
		}

/* ===
		if (total_Picts < 2) {
			Get_NS_String(k_SPIN_STRINGS, k_ERROR_DID_NOT_CONTAIN_MULTIPLE_PICS, message_String);
			Perform_OK_Dialog(message_String);
			goto Error_Handler;
		}
	=== */
		
	}
	else {
		if ((total_Picts < 2) || (still_Expecting_Pict_Resources != TRUE)){
			Get_NS_String(k_SPIN_STRINGS, k_ERROR_DID_NOT_CONTAIN_MULTIPLE_PICS, message_String);
			Perform_OK_Dialog(message_String);
			goto Error_Handler;
		}
		
		lowest_Index = 1;
		highest_Index = total_Picts;
		default_Index = lowest_Index;
	
		temp_Resource = Get1IndResource ('PICT', default_Index);
		if (ResError() != noErr) {
			Display_Serious_Error_Message(unit_Name
																, code_Name
																, "\pInternal problem with resources"
																, "\pCorrupted?");
			goto Error_Handler;
		}
			
		GetResInfo(temp_Resource
							, &resource_Id
							, &resource_Type
							, &temp_String[0]);
							
		if (ResError() != noErr) {
			goto Error_Handler;
		}
			
		*resource_Index = resource_Id;
		
		os_Error = GetPictInfo((PicHandle)temp_Resource
						, &picture_Info
						, (0)
						, 0
						, popularMethod
						, 0);
	
			
		/*  */
		/* Using the first frame (The master puppet frame) */
		/* set limits on the puppet about to be created if needed */
		/* Note though that cropping is arbitrary in this version */
		/* and is not self center adjusting */
		/*  */
		selected_Pict_Rect = (*(PicHandle)temp_Resource)->picFrame;
		*selected_Pict_Width = selected_Pict_Rect.right - selected_Pict_Rect.left;
		*selected_Pict_Height = selected_Pict_Rect.bottom - selected_Pict_Rect.top;
	/* ====
		if (os_Error == noErr) {
			*selected_Pict_Depth = picture_Info.depth;
		}
		else {
			goto Error_Handler;
		}
	=== */

	} /* NOT if (*file_Is_Pict_File_Type == SAFE_TRUE)  */
	
		
	/*  */
	/* At this point we have our pict size info obtained from either */
	/* a PICT resource or a PICT file */
	/*  */
	
	if (*selected_Pict_Width > g_Window_Limits.specific.display_Item_Width_Max) {
		selected_Pict_Rect.right = selected_Pict_Rect.right 
										- (*selected_Pict_Width - g_Window_Limits.specific.display_Item_Width_Max);
		*selected_Pict_Width = g_Window_Limits.specific.display_Item_Width_Max;
		
		Get_NS_String(k_SPIN_STRINGS, k_NOTE_SPIN_OBJECT_WIDTH_WILL_CLIP, message_String);
		Perform_OK_Dialog(message_String);
		
	}
	
	if (*selected_Pict_Height > g_Window_Limits.specific.display_Item_Height_Max) {
		selected_Pict_Rect.bottom = selected_Pict_Rect.bottom 
										- (*selected_Pict_Height - g_Window_Limits.specific.display_Item_Height_Max);
		*selected_Pict_Height = g_Window_Limits.specific.display_Item_Height_Max;
		
		Get_NS_String(k_SPIN_STRINGS, k_NOTE_SPIN_OBJECT_HEIGHT_WILL_CLIP, message_String);
		Perform_OK_Dialog(message_String);
		
	}

	if ((*selected_Pict_Width < g_Window_Limits.specific.display_Item_Width_Min)
		|| (*selected_Pict_Height < g_Window_Limits.specific.display_Item_Height_Min)){
		
		Get_NS_String(k_SPIN_STRINGS, k_ERROR_SPIN_OBJECT_TOO_SMALL, message_String);
		Perform_OK_Dialog(message_String);
		
		goto Error_Handler;
	}
		
	/*  */
	/* Put away the resource, we extracted the information we wanted */
	/*  */
	if (still_Expecting_Pict_Resources == TRUE) {
		ReleaseResource(temp_Resource);
		if (ResError() != noErr) {
			goto Error_Handler;
		}
	}
	
	if (resource_Sources_Swapped == SAFE_TRUE) {
		
		UseResFile(save_Ref);
		if (ResError() != noErr) {
			/*  */
			/* This should be recoverable from, enhance one day  */
			/*  */
		}
		resource_Sources_Swapped = SAFE_FALSE;
	}

	if (input_File_Reference != 0) {
		CloseResFile(input_File_Reference);
		if (ResError() != noErr) {

		}
		input_File_Reference = 0;
	}
		
	function_Result = SAFE_TRUE;
	goto Normal_Function_Exit;

Error_Handler:
	function_Result = SAFE_FALSE;
		
Normal_Function_Exit:

	if (input_File_Reference != 0) {
		CloseResFile(input_File_Reference);
	}

	if (resource_Sources_Swapped == SAFE_TRUE) {
		CloseResFile(input_File_Reference);
		if (ResError() != noErr) {
			/*  */
			/* Ignore day  */
			/*  */
		}
		
		UseResFile(save_Ref);
		if (ResError() != noErr) {
			/*  */
			/* This should be recoverable from, enhance one day  */
			/*  */
		}
		resource_Sources_Swapped = SAFE_FALSE;
	}

	return(function_Result);
	
} /* Safe_Boolean	Specify_Spin_Object_Pict_To_Import... */
/*=**************************************************************************=*/
	
	
/*=**************************************************************************=*/				
Safe_Boolean	Specify_Spin_Object_To_Save(FSSpec		*file_Specification
															, Str63				*assumed_File_Name
															, Safe_Boolean *user_Abort)
{
	INTEGER_16 		number_Of_Types;
	Boolean				user_Clicked_Ok;
	Str63					original_Name;
	Str63					suffix_String;
	Str255					message_String;
	SFTypeList 			type_List;
	SFReply				sf_Reply;
	Safe_Boolean		function_Result;
/* END VAR */


	function_Result = SAFE_FALSE;
	*user_Abort = SAFE_FALSE;

	Get_NS_String(k_SPIN_STRINGS, k_SAVE_SPIN_OBJECT_PROMPT, message_String);
	Get_NS_String(k_SPIN_STRINGS, k_SAVE_SPIN_OBJECT_FILE_SUFFIX, suffix_String);

	Assign_String(original_Name, *assumed_File_Name);
	Strip_Off_Popular_Suffixes(&original_Name);
	/*  */
	/* Suffix not appended if just space character */
	/*  */
	if ((suffix_String[0] != 1) || (suffix_String[1] != ASCII_SPACE)) {
		Concat_String(original_Name, suffix_String);
	}
	
	if(original_Name[0] >31) {
		original_Name[0] = 31;
	}


	user_Clicked_Ok = Perform_Put_File_Dialog(message_String
										, original_Name
										, &sf_Reply);
	
	if ((user_Clicked_Ok != TRUE) || (sf_Reply.good != TRUE)) {
		/*  */
		/* User Abort, no file name */
		/*  */
		*user_Abort = SAFE_TRUE;
		function_Result = SAFE_FALSE;
		goto Normal_Function_Exit;
	}

	Get_Last_Volume_Reference_And_Directory_ID(&(file_Specification->vRefNum)
																	, &(file_Specification->parID));
																	
	Copy_Pascal_String(sf_Reply.fName, file_Specification->name);
		
	function_Result = SAFE_TRUE;
	goto Normal_Function_Exit;

Error_Handler:
	function_Result = SAFE_FALSE;
		
Normal_Function_Exit:
	return(function_Result);
	
} /* Safe_Boolean	Specify_Spin_Object_To_Save... */
/*=**************************************************************************=*/
	
	
/*=**************************************************************************=*/				
Safe_Boolean	Specify_Spin_Object_To_Export(FSSpec		*file_Specification
															, Str63				*assumed_File_Name
															, Safe_Boolean *user_Abort)
{
	INTEGER_16 		number_Of_Types;
	Boolean				user_Clicked_Ok;
	Str63					original_Name;
	Str63					suffix_String;
	Str255					message_String;
	SFTypeList 			type_List;
	SFReply				sf_Reply;
	Safe_Boolean		function_Result;
/* END VAR */


	function_Result = SAFE_FALSE;
	*user_Abort = SAFE_FALSE;

	Get_NS_String(k_SPIN_STRINGS, k_EXPORT_SPIN_OBJECT_PROMPT, message_String);
	Get_NS_String(k_SPIN_STRINGS, k_EXPORT_SPIN_OBJECT_SPIN_OBJECT_FILE_SUFFIX, suffix_String);

	Assign_String(original_Name, *assumed_File_Name);
	Strip_Off_Popular_Suffixes(&original_Name);
	/*  */
	/* Suffix not appended if just space character */
	/*  */
	if ((suffix_String[0] != 1) || (suffix_String[1] != ASCII_SPACE)) {
		Concat_String(original_Name, suffix_String);
	}
	
	if(original_Name[0] >31) {
		original_Name[0] = 31;
	}


	user_Clicked_Ok = Perform_Put_File_Dialog(message_String
										, original_Name
										, &sf_Reply);
	
	if ((user_Clicked_Ok != TRUE) || (sf_Reply.good != TRUE)) {
		/*  */
		/* User Abort, no file name */
		/*  */
		*user_Abort = SAFE_TRUE;
		function_Result = SAFE_FALSE;
		goto Normal_Function_Exit;
	}

	Get_Last_Volume_Reference_And_Directory_ID(&(file_Specification->vRefNum)
																	, &(file_Specification->parID));
																	
	Copy_Pascal_String(sf_Reply.fName, file_Specification->name);
		
	function_Result = SAFE_TRUE;
	goto Normal_Function_Exit;

Error_Handler:
	function_Result = SAFE_FALSE;
		
Normal_Function_Exit:
	return(function_Result);
	
} /* Safe_Boolean	Specify_Spin_Object_To_Export... */
/*=**************************************************************************=*/


/*=**************************************************************************=*/
/* END OF FILE: Spin_O_Rama_Files.c */
