/* START OF FILE: Spin_O_Rama_Load_Import.c */


/*=**************************************************************************=*/
/* [---BEGIN---] FILE HEADER COMMENTS */
/**
 ** SCREEN VIEW = [Adobe Syntax Black-10, Tabs = 4]
 ** PRINTER VIEW = [Adobe Syntax Black-10, Tabs = 4]
 ** =======================================================================
 ** Written by James Merkle
 ** Spin_O_Rama_Load_Import.c - Routines for Spin file loading and importation
 ** (C) Copyright 1995 James Merkle
 ** =======================================================================
 **
 **
 ** ============
 ** Revision History
 ** ============
 **
 **  1995.01.09		(James Merkle)
 **						Initial coding started
 **
 **
 **
 ** =================
 ** OTHER INFORMATION
 ** =================
 **
 ** This ring buffer has occlusion of older elements as needed.
 **
 **/
/* [----END----] FILE HEADER COMMENTS */
/*=**************************************************************************=*/


/*=**************************************************************************=*/
/* [---BEGIN---] #include AREA */

#include "Error_Utilities.h"
#include "Memory_Utilities.h"

#include "String_Utilities.h"
#include "Dreaming_Color_Stuff.h"
#include "SysHooks.h"
#include "Spin_O_Rama.h"

#include "OS_Utilities.h"
#include "Cursor_Utilities.h"

#include "Buffered_File_Routines.h"
#include "Direct_Background_Removal.h"
#include "Fast_Image_Compaction.h"
#include "Video_Backdrop_Background.h"
#include "Window_Environment.h"

#include "Shell_Assembly.h"
#include "Shell_Interface.h"

#include "Picture_Spooling.h"
#include "Shell_Dialogs.h"
#include "Interface_Utilities.h"
#include "Spin_O_Rama_Files.h"
#include "File_Utilities.h"
#include "Dialog_Of_Background_Removal.h"

#include "Core_Globals.h"
#include "Shell_Globals.h"
#include "Core_Specific.h"
#include "Core_Globals.h"

#include "Ring_Buffer_Cache.h"
#include "Graphics_Primitives_Support.h"
#include "Graphics_Primitives.h"
#include "Spin_O_Rama_Cache.h"

#include "Spin_O_Rama_Load_Import.h"


/* [----END----] #include AREA */
/*=**************************************************************************=*/


/*=**************************************************************************=*/
/* [---BEGIN---] #define AREA */

/*  */
/* Always force Dither if possible. Some Quadras do it by default though.*/
/*  */
#define k_FORCE_FULL_MAC_DITHER		1

#define k_ALLOW_AUTO_MOUTH_AUTO_EMOTION		0


/*  */
/* Probably safer if set to "0" but set to "1" anyway so that */
/* progress can be seen */
/*  */
#define k_FORCE_FALSE_UPDATE_WHILE_IMPORTING		1

/*  */
/* Four types of background elimination strategies exist */
/*  */
#define k_DO_NO_EXTRACTION													0
#define k_EXTRACT_LOW_DEPTH_FROM_SOURCE								1
#define k_EXTRACT_HIGH_DEPTH_FROM_SOURCE								2
#define k_EXTRACT_HIGH_DEPTH_FROM_COPY									3
#define k_EXTRACT_HIGH_DEPTH_FROM_COPY_IF_NEEDED					4
#define k_COPY_ALL_PIXELS_BUT_DITHER_TO_INDEXED_FROM_DEEP		5

/*  */
/* HACK HACK HACK Instead of 65536UL, try a different number */
/*  */
#define k_PICT_FILE_DRAWING_BUFFER_SIZE		((110L * 1024L) + 512UL)

#define k_USE_DIRECT_COLOR_AS_INPUT			SAFE_TRUE
#define k_USE_INDEXED_COLOR_AS_INPUT		SAFE_FALSE

#define k_PARANOIA_VIDEO_BORDER_X		(2L);
#define k_PARANOIA_VIDEO_BORDER_Y		(2L);

/* [----END----] #define AREA */
/*=**************************************************************************=*/


/*=**************************************************************************=*/
/* [---BEGIN---] TYPE AREA */


/* [----END----] TYPE AREA */
/*=**************************************************************************=*/


/*=**************************************************************************=*/
/* [---BEGIN---] PROTOTYPE AREA */
					
															
Safe_Boolean Get_PICT_File_Neighbor_File_Spec(FSSpec *desired_File, CARDINAL_32 pict_Index, FSSpec *located_File);


Safe_Boolean Load_Raw_Pict_Fork_From_Spin_File_And_Draw(FSSpec *file_Info
																		, CARDINAL_16	desired_Color_Depth
																		, Off_Screen_Port_Rec *offscreen_Ptr
																		, Spin_Pict_Info_Handle	spin_Info
																		, CARDINAL_32	cell_Offset);

Safe_Boolean Load_Spin_File_Basic_Parts(FSSpec *file_Info
											, CARDINAL_16	desired_Color_Depth
											, Spin_Pict_Info_Handle	spin_Info);

Safe_Boolean	Specify_PICT_File_To_Load(FSSpec		*file_Specification
															, Safe_Boolean *user_Abort);
															
Safe_Boolean Ask_User_If_Auto_Search(FSSpec *near_Neighbor_File
												, CARDINAL_32	total_Estimated_Images_In_Each_Directory
												, Boolean	*automatic_Imporation_Desired
												, Boolean	*automatic_Imporation_Functioning);

Safe_Boolean Get_PICT_File_Neighbor_File_Spec_Next_Folder(FSSpec *near_Neighbor_File
												, CARDINAL_32	total_Estimated_Images_In_Each_Directory
												, Safe_Boolean	calculate_Full_Hierarchy
												, CARDINAL_32	*hierachy_Depth_Above
												, FSSpec			*located_Neighbor_File);

/* [----END----] PROTOTYPE AREA */
/*=**************************************************************************=*/


/*=**************************************************************************=*/
/* [---BEGIN---] GLOBAL DEFINITION AREA */


/* [----END----] GLOBAL DEFINITION AREA */
/*=**************************************************************************=*/


/*=**************************************************************************=*/
/* [---BEGIN---] LOCAL STATIC DEFINITION AREA */

static CARDINAL_8			*unit_Name = "\pSpin_O_Rama_Load_Import";


/* [----END----] LOCAL STATIC DEFINITION AREA */
/*=**************************************************************************=*/
	
	
/*=**************************************************************************=*/
Safe_Boolean Count_PICT_File_Neighbors(FSSpec *desired_File, CARDINAL_32 *total_Found)
{
	HFileParam			parameters;	
	Safe_Boolean			function_Result;	
	Boolean					finished_Searching;
	OSErr						os_Error;	
	INTEGER_32			index;
	Str255						file_Name;
/* END VAR */


	function_Result = SAFE_TRUE;
	*total_Found = 0;
	
	parameters.ioVRefNum = desired_File->vRefNum;
	parameters.ioNamePtr = &file_Name[0];	/* buffer to receive name */
	
	finished_Searching = FALSE;
	for(index=1; (finished_Searching != TRUE) ; index++) {	/* indexing loop */
		
		parameters.ioDirID = desired_File->parID;
		
		parameters.ioFDirIndex = index;

		os_Error = PBHGetFInfo((HParmBlkPtr)&parameters, FALSE);
		if (os_Error != noErr) {
			if (os_Error != fnfErr) {
				function_Result = SAFE_FALSE;
			}
			finished_Searching = TRUE;
		}
		else {
			if ((parameters.ioFlAttrib & 16) == 0) {
				if (parameters.ioFlFndrInfo.fdType == 'PICT') {
					*total_Found += 1;
				}
			}
		}
	}
	
	if (function_Result != SAFE_TRUE) {
		if (*total_Found > 0) {
			function_Result = SAFE_TRUE;
		}
	}
	
	return(function_Result);
	
}
/*=**************************************************************************=*/
	
	
/*=**************************************************************************=*/				
Safe_Boolean	Specify_PICT_File_To_Load(FSSpec		*file_Specification
															, Safe_Boolean *user_Abort)
{
	INTEGER_16 		number_Of_Types;
	Boolean				user_Clicked_Ok;
	INTEGER_32		source_Directory_ID;
	INTEGER_16		source_Volume_Reference;
	SFTypeList 			type_List;
	SFReply				sf_Reply;
	Safe_Boolean		function_Result;
/* END VAR */


	function_Result = SAFE_FALSE;
	*user_Abort = SAFE_FALSE;

	number_Of_Types = 1;
	type_List[0] = 'PICT';

	user_Clicked_Ok = Perform_Get_File_Dialog(number_Of_Types
										, &(type_List)
										, &sf_Reply);
											
	if ((user_Clicked_Ok != TRUE) || (sf_Reply.good != TRUE)) {
		/*  */
		/* User Abort, no file name */
		/*  */
		*user_Abort = SAFE_TRUE;
		function_Result = SAFE_FALSE;
		goto Normal_Function_Exit;
	}
	
	Get_Last_Volume_Reference_And_Directory_ID(&source_Volume_Reference, &source_Directory_ID);

	file_Specification->vRefNum = source_Volume_Reference;
	file_Specification->parID = source_Directory_ID;
	Copy_Pascal_String(sf_Reply.fName, file_Specification->name);
		
	function_Result = SAFE_TRUE;
	goto Normal_Function_Exit;

Error_Handler:
	function_Result = SAFE_FALSE;
		
Normal_Function_Exit:
	return(function_Result);
	
} /* Safe_Boolean	Specify_Spin_Object_To_Load... */
/*=**************************************************************************=*/
	
	
/*=**************************************************************************=*/
Safe_Boolean Get_PICT_File_Neighbor_File_Spec(FSSpec *desired_File, CARDINAL_32 pict_Index, FSSpec *located_File)
{
	HFileParam			parameters;	
	Safe_Boolean			function_Result;	
	Boolean					finished_Searching;		
	INTEGER_32			index;
	OSErr						os_Error;
	CARDINAL_32		total_Found;
	Str255						file_Name;
/* END VAR */


	function_Result = SAFE_FALSE;
	total_Found = 0;
	
	parameters.ioVRefNum = desired_File->vRefNum;
	parameters.ioNamePtr = &file_Name[0];	/* buffer to receive name */
	
	finished_Searching = FALSE;
	
	for (index=1; (finished_Searching != TRUE) ; index++) {
		
		parameters.ioDirID = desired_File->parID;
		
		parameters.ioFDirIndex = index;

		os_Error = PBHGetFInfo((HParmBlkPtr)&parameters, FALSE);
		
		if (os_Error != noErr) {
			if (os_Error != fnfErr) {
				function_Result = SAFE_FALSE;
			}
			finished_Searching = TRUE;
		}
		else {
			if ((parameters.ioFlAttrib & 16) == 0) {
				if (parameters.ioFlFndrInfo.fdType == 'PICT') {
					total_Found += 1;
					
					if (total_Found == pict_Index) {
				
						*located_File = *desired_File;
						Copy_Pascal_String(file_Name, located_File->name);
				
						function_Result = SAFE_TRUE;
						goto Normal_Function_Exit;
					}
				}
			}
		}
	}
		
Normal_Function_Exit:

	return(function_Result);
	
} /* Safe_Boolean Get_PICT_File_Neighbor_File_Spec */
/*=**************************************************************************=*/
	
	
/*=**************************************************************************=*/
Safe_Boolean Expand_Rect_To_Largest_Reasonable_Size(Rect *offscreen_Rect)
{
Safe_Boolean	function_Result;
/* END VAR */


	function_Result = SAFE_FALSE;

	if (g_Set_Spin_Window_Max_Monitor_Size == SAFE_TRUE) {
		if (g_Preferred_Monitor_Values_Valid == SAFE_TRUE) {
			if (offscreen_Rect->right < (g_Preferred_Monitor_Max_Rect.right - g_Preferred_Monitor_Max_Rect.left)) {
				offscreen_Rect->right = (g_Preferred_Monitor_Max_Rect.right - g_Preferred_Monitor_Max_Rect.left);
				
				/*  */
				/* HACK HACK TEMP paranoia only for AV video */
				/*  */
				offscreen_Rect->right = offscreen_Rect->right - k_PARANOIA_VIDEO_BORDER_X; 
			}
			
			if (g_Preferred_Monitor_Max_Rect_Contains_Menubar == SAFE_TRUE) {
				if (offscreen_Rect->bottom < (g_Preferred_Monitor_Max_Rect.bottom - g_Preferred_Monitor_Max_Rect.top - GetMBarHeight())) {
					offscreen_Rect->bottom = (g_Preferred_Monitor_Max_Rect.bottom - g_Preferred_Monitor_Max_Rect.top - GetMBarHeight());
					/*  */
					/* HACK HACK TEMP paranoia only for AV video */
					/*  */
					offscreen_Rect->bottom = offscreen_Rect->bottom - k_PARANOIA_VIDEO_BORDER_Y;
				}
			}
			else {
				if (offscreen_Rect->bottom < (g_Preferred_Monitor_Max_Rect.bottom - g_Preferred_Monitor_Max_Rect.top)) {
					offscreen_Rect->bottom = (g_Preferred_Monitor_Max_Rect.bottom - g_Preferred_Monitor_Max_Rect.top);
					/*  */
					/* HACK HACK TEMP paranoia only for AV video */
					/*  */
					offscreen_Rect->bottom = offscreen_Rect->bottom - k_PARANOIA_VIDEO_BORDER_Y; 
				}
			}
		}
	}
	
	function_Result = SAFE_TRUE;
	
	return(function_Result);
}
/*=**************************************************************************=*/

	
/*=**************************************************************************=*/
Safe_Boolean Get_PICT_File_Neighbor_File_Spec_Next_Folder(FSSpec *near_Neighbor_File
												, CARDINAL_32	total_Estimated_Images_In_Each_Directory
												, Safe_Boolean	calculate_Full_Hierarchy
												, CARDINAL_32	*hierachy_Depth_Above
												, FSSpec			*located_Neighbor_File)
{
	Safe_Boolean			function_Result;	
	CInfoPBRec			parameters;	
	CInfoPBRec			volume_parameters;	
	Boolean				finished_Searching;		
	Boolean				found_Item;		
	INTEGER_32			index;
	CARDINAL_32		grandparents_Directory;
	CARDINAL_32		parents_Directory;
	CARDINAL_32		next_Sibling_Directory;
	OSErr					os_Error;
	FSSpec				wild_Guess_Neighbor_File;
	Str63					file_Name;
	Str63					directory_Name;
	Str63					next_directory_Name;
	Str63					parent_Directory_Name;
/* END VAR */


	function_Result = SAFE_FALSE;
	*hierachy_Depth_Above = 0;

	/* buffer to get/receive name */
	
	parameters.hFileInfo.ioNamePtr = &file_Name[0];	
	Copy_Pascal_String(&(near_Neighbor_File->name[0]), &file_Name[0]);
	parameters.hFileInfo.ioDirID = near_Neighbor_File->parID;
	parameters.hFileInfo.ioVRefNum = near_Neighbor_File->vRefNum;
	parameters.hFileInfo.ioCompletion = nil;
	parameters.hFileInfo.ioFDirIndex = 0;

	finished_Searching = TRUE;
	os_Error = PBHGetFInfo((HParmBlkPtr)&parameters, FALSE);
	if (os_Error == noErr) {
		if ((parameters.hFileInfo.ioFlAttrib & 16) == 0) {
			if (parameters.hFileInfo.ioFlFndrInfo.fdType == 'PICT') {
				finished_Searching = FALSE;
			}
		}
	}
	else {
		goto Error_Handler;
	}

	if (finished_Searching == TRUE) {
		goto Error_Handler;
	}
	
	/*  */
	/* ioDrParID, ioDrDirID */
	/*  */
	directory_Name[0] = 0;
	volume_parameters.dirInfo.ioCompletion = nil;
	volume_parameters.dirInfo.ioVRefNum = parameters.dirInfo.ioVRefNum;
	volume_parameters.dirInfo.ioNamePtr = &directory_Name[0];	
	volume_parameters.dirInfo.ioDrDirID = near_Neighbor_File->parID;
	volume_parameters.dirInfo.ioFDirIndex = -1; /* no name */
		
	os_Error = PBGetCatInfo(&volume_parameters,FALSE);
	if (os_Error != noErr) {
		goto Error_Handler;
	}
	parents_Directory = volume_parameters.dirInfo.ioDrParID;
	
	Copy_Pascal_String(&directory_Name[0], &next_directory_Name[0]);
	Increment_Numerical_Suffix(&next_directory_Name[0]);
	
	/*  */
	/* The steps involved are going through and examining */
	/* relative possibility for either 1 or 2 or 3 level folder hierarchy. */
	/*  */
	file_Name[0] = 0;
	parameters.dirInfo.ioNamePtr = &file_Name[0];	/* buffer to get/receive name */
	parameters.dirInfo.ioDrDirID = parents_Directory;
	parameters.dirInfo.ioVRefNum = near_Neighbor_File->vRefNum;
	parameters.dirInfo.ioCompletion = nil;
	parameters.dirInfo.ioFDirIndex = 0;
	
	finished_Searching = FALSE;
	found_Item = FALSE;
	for(index=1; (finished_Searching != TRUE) ; index++) {	/* indexing loop */
		
		parameters.dirInfo.ioDrDirID = parents_Directory;
		
		parameters.dirInfo.ioFDirIndex = index;

		os_Error = PBGetCatInfo(&parameters, FALSE);
		if (os_Error != noErr) {
			if (os_Error != fnfErr) {
				function_Result = SAFE_FALSE;
			}
			finished_Searching = TRUE;
		}
		else {
			if ((parameters.dirInfo.ioFlAttrib & 16) != 0) {

				if (Compare_Pascal_Strings(file_Name,next_directory_Name) == SAFE_TRUE) {
					finished_Searching = TRUE;
					found_Item = TRUE;
				}
			}
		}
	}
	
	if (found_Item != TRUE) {
		/*  */
		/* We have to travel up to grandparents folder and then climb back down. */
		/*  */
		/*  */
		/* Store current name as target. Increment the name */
		/* Final digits. Search for a match of the new name */
		/*  */
		
		parent_Directory_Name[0] = 0;
		volume_parameters.dirInfo.ioNamePtr = &parent_Directory_Name[0];	/* buffer to get/receive name */
		volume_parameters.dirInfo.ioDrDirID = parents_Directory;
		volume_parameters.dirInfo.ioVRefNum = parameters.dirInfo.ioVRefNum;
		volume_parameters.dirInfo.ioCompletion = nil;
		volume_parameters.dirInfo.ioFDirIndex = -1; /* no name */
		
		os_Error = PBGetCatInfo(&volume_parameters,FALSE);
		if (os_Error != noErr) {
			goto Error_Handler;
		}
		
		grandparents_Directory = volume_parameters.dirInfo.ioDrParID;
		
		Copy_Pascal_String(&parent_Directory_Name[0], &next_directory_Name[0]);
		Increment_Numerical_Suffix(&next_directory_Name[0]);
				
		/*  */
		/* We just travelled up, now travel down to next incremented parent folder */
		/*  */

		parameters.dirInfo.ioNamePtr = &file_Name[0];
		file_Name[0] = 0;
		parameters.dirInfo.ioDrDirID = grandparents_Directory;
		parameters.dirInfo.ioVRefNum = near_Neighbor_File->vRefNum;
		parameters.dirInfo.ioCompletion = nil;
		parameters.dirInfo.ioFDirIndex = 0;
		
		finished_Searching = FALSE;
		found_Item = FALSE;
		for(index=1; (finished_Searching != TRUE) ; index++) {	
			
			parameters.dirInfo.ioDrDirID = grandparents_Directory;
			
			parameters.dirInfo.ioFDirIndex = index;
			
			os_Error = PBGetCatInfo(&parameters, FALSE);
			if (os_Error != noErr) {
				if (os_Error != fnfErr) {
					function_Result = SAFE_FALSE;
				}
				finished_Searching = TRUE;
			}
			else {
				if ((parameters.dirInfo.ioFlAttrib & 16) != 0) {
					if (Compare_Pascal_Strings(file_Name,next_directory_Name) == SAFE_TRUE) {
						finished_Searching = TRUE;
						found_Item = TRUE;
					}
				}
			}
		}
	
		if (found_Item != TRUE) {
			/*  */
			/* Remove from considereation. Not locatable. Perhaps we */
			/* are done? */
			/*  */
			goto Error_Handler;
		}
		
		/*  */
		/* Create new FSSpec for first item that matches in new sibling folder */
		/* Travel down into next folder and search for arbitrary PREFIX */
		/*  */
	
		next_Sibling_Directory = parameters.dirInfo.ioDrDirID;
		
		parameters.dirInfo.ioNamePtr = &file_Name[0];
		file_Name[0] = 0;
		parameters.dirInfo.ioDrDirID = next_Sibling_Directory;
		parameters.dirInfo.ioVRefNum = near_Neighbor_File->vRefNum;
		parameters.dirInfo.ioCompletion = nil;
		parameters.dirInfo.ioFDirIndex = 0;
		
		Truncate_Without_Numerical_Suffix(&directory_Name[0]);
		
		finished_Searching = FALSE;
		found_Item = FALSE;
		for(index=1; (finished_Searching != TRUE) ; index++) {	
			
			parameters.dirInfo.ioDrDirID = next_Sibling_Directory;
			
			parameters.dirInfo.ioFDirIndex = index;
	
			os_Error = PBGetCatInfo(&parameters, FALSE);
			if (os_Error != noErr) {
				if (os_Error != fnfErr) {
					function_Result = SAFE_FALSE;
				}
				finished_Searching = TRUE;
			}
			else {
				if ((parameters.dirInfo.ioFlAttrib & 16) != 0) {
					Truncate_Without_Numerical_Suffix(&file_Name[0]);
					if (Compare_Pascal_Strings(file_Name,directory_Name) == SAFE_TRUE) {
						finished_Searching = TRUE;
						found_Item = TRUE;
					}
				}
			}
		}
		
		if (found_Item == TRUE) {
		
			wild_Guess_Neighbor_File.vRefNum = near_Neighbor_File->vRefNum;
			wild_Guess_Neighbor_File.parID = parameters.dirInfo.ioDrDirID;
			wild_Guess_Neighbor_File.name[0] = 1;
			wild_Guess_Neighbor_File.name[1] = 'X';
	
			if (Get_PICT_File_Neighbor_File_Spec(&wild_Guess_Neighbor_File
													, 1
													, located_Neighbor_File) != SAFE_TRUE) {
				goto Error_Handler;
			}
			else {
			
				*hierachy_Depth_Above = 2;
			}
		}
		else {
			/*  */
			/* Error! No useful files found at lower root! Retry by taking first */
			/* arbitrary folder in section? */
			/*  */
			goto Error_Handler;
		}

	}
	else {
		
		/*  */
		/* Create new FSSpec for first item that matches in new sibling folder */
		/*  */
		
		wild_Guess_Neighbor_File.vRefNum = near_Neighbor_File->vRefNum;
		wild_Guess_Neighbor_File.parID = parameters.dirInfo.ioDrDirID;
		wild_Guess_Neighbor_File.name[0] = 1;
		wild_Guess_Neighbor_File.name[1] = 'X';

		if (Get_PICT_File_Neighbor_File_Spec(&wild_Guess_Neighbor_File
												, 1
												, located_Neighbor_File) != SAFE_TRUE) {
			goto Error_Handler;
		}
		else {
		
			*hierachy_Depth_Above = 1;
		}
	}


	function_Result = SAFE_TRUE;
	goto Normal_Function_Exit;
	
Error_Handler:
	function_Result = SAFE_FALSE;

Normal_Function_Exit:
	return(function_Result);
}
/*=**************************************************************************=*/
	
	
/*=**************************************************************************=*/
Safe_Boolean Ask_User_If_Auto_Search(FSSpec *start_Point_File_Pointer
												, CARDINAL_32	total_Estimated_Images_In_Each_Directory
												, Boolean	*automatic_Imporation_Desired
												, Boolean	*automatic_Imporation_Functioning)
{
	CARDINAL_32		hierachy_Depth_Above;
	Safe_Boolean			function_Result;
	FSSpec				located_Neighbor_File;
	FSSpec				near_Neighbor_File;
	Str255					message_String;
	Str255					temp_String_1;
	Str255					temp_String_2;
	OSErr					os_Error;
/* END VAR */


	function_Result = SAFE_FALSE;
		
	near_Neighbor_File = *start_Point_File_Pointer;
	if (Get_PICT_File_Neighbor_File_Spec_Next_Folder(&near_Neighbor_File
									, total_Estimated_Images_In_Each_Directory
									, SAFE_TRUE
									, &hierachy_Depth_Above
									, &located_Neighbor_File) != SAFE_TRUE) {
		goto Error_Handler;
	}
	
	
	/*  */
	/* Ask user if OK to use hierarchy, because files found in an adjacent folder */
	/*  */
	
	Get_NS_String(k_SPIN_STRINGS, k_IMPORT_SPIN_OBJECT_ALLOW_HIERARCHY_PART_1, temp_String_1);
	Get_NS_String(k_SPIN_STRINGS, k_IMPORT_SPIN_OBJECT_ALLOW_HIERARCHY_PART_2, temp_String_2);
	Concat_Many(message_String, 3,temp_String_1, start_Point_File_Pointer->name, temp_String_2);

	/*  */
	/* Get Directory Name */
	/*  */	
	Get_Parents_Directory_Name(&near_Neighbor_File, &temp_String_1);
	Concat_String(message_String, temp_String_1);

	Get_NS_String(k_SPIN_STRINGS, k_IMPORT_SPIN_OBJECT_ALLOW_HIERARCHY_PART_3, temp_String_1);
	Concat_String(message_String, temp_String_1);

/* =======
	Get_NS_String(k_SPIN_STRINGS, k_IMPORT_SPIN_OBJECT_ALLOW_HIERARCHY_PART_4, temp_String_1);
	Concat_String(message_String, temp_String_1);
	
	Get_Grand_Parents_Directory_Name(&near_Neighbor_File, &temp_String_1);
	Concat_String(message_String, temp_String_1);

	Get_NS_String(k_SPIN_STRINGS, k_IMPORT_SPIN_OBJECT_ALLOW_HIERARCHY_PART_5, temp_String_1);
	Concat_String(message_String, temp_String_1);
====== */

	if (Perform_Yes_No_Dialog(message_String) != TRUE) {
		goto Error_Handler;
	}

	function_Result = SAFE_TRUE;
	goto Normal_Function_Exit;
	
Error_Handler:
	*automatic_Imporation_Desired = FALSE;
	*automatic_Imporation_Functioning = FALSE;
	
	function_Result = SAFE_FALSE;

Normal_Function_Exit:
	return(function_Result);
	
} /* Safe_Boolean Ask_User_If_Auto_Search */
/*=**************************************************************************=*/
	
	
/*=**************************************************************************=*/
Safe_Boolean Load_Spin_File_Basic_Parts(FSSpec *file_Info
											, CARDINAL_16	desired_Color_Depth
											, Spin_Pict_Info_Handle	spin_Info)
{
	Spin_Object_File_Header_Ptr				header;
	SignedByte 									header_Original_State;
	Spin_Object_File_Header_Hdl				spin_Object_Header_Hdl;
	Memory_Allocation							spin_Object_Header_Memory_In_Use;
	INTEGER_16									total_Puppet_Resources;
	Safe_Boolean		function_Result;
	Safe_Boolean		resource_Sources_Swapped;
	INTEGER_16		input_File_Reference;
	INTEGER_16		save_Ref;
	CARDINAL_8		*code_Name;																		
	Str255				note_String;
	Str255				message_String;
/* END VAR */


	code_Name = "\pLoad_Spin_File_Basic_Parts";
	function_Result = SAFE_FALSE;
	spin_Object_Header_Memory_In_Use = MEMORY_NOT_INITIALIZED;
	resource_Sources_Swapped = SAFE_FALSE;
																		

	/*  */
	/* We need to open up the file's resource fork and get the information */
	/* to tell us the port size. We also need the information for */
	/* the version to see if compatible or not */
	/*  */
	
	save_Ref = CurResFile();
	input_File_Reference = HOpenResFile (file_Info->vRefNum
									, file_Info->parID
									, file_Info->name
									, fsRdPerm );
						
	if ((input_File_Reference == -1) || (ResError() != noErr)) {
		goto Error_Handler;
	}
	
	
	total_Puppet_Resources = Count1Resources(k_SPIN_OBJECT_MAIN_RESOURCE_TYPE);
	if (ResError() != noErr) {
		goto Error_Handler;
	}
	
	if (total_Puppet_Resources < 1) {
		Get_NS_String(k_DREAM_STRINGS, k_SORRY_WRONG_FILE_INDEX, message_String);
		Perform_OK_Dialog(message_String);
		goto Error_Handler;
	}


	spin_Object_Header_Hdl = (Spin_Object_File_Header_Hdl)Get1Resource(k_SPIN_OBJECT_MAIN_RESOURCE_TYPE
																			, k_BASE_SPIN_OBJECT_RESOURCE_ID);
	if ((ResError() != noErr) || (spin_Object_Header_Hdl == nil)) {
		goto Error_Handler;
	}
	spin_Object_Header_Memory_In_Use = MEMORY_INITIALIZED;
	HNoPurge((Handle)spin_Object_Header_Hdl);
	if (MemError() != noErr) {
		/*  */
		/* This cannot fail, but check anyway */
		/*  */
		goto Error_Handler;
	}
		
	if ((**spin_Object_Header_Hdl).valid != SAFE_TRUE) {
		Copy_Pascal_String("\pThis file has corrupted resources and cannot be used safely", message_String);
		Perform_OK_Dialog(message_String);
		goto Error_Handler;
	}
	
	if (Begin_Use_Of_Handle((Handle)spin_Object_Header_Hdl
									, MOVE_HIGH_NOT_REQUIRED
									, &header_Original_State) != SAFE_TRUE) {
		goto Error_Handler;
	}
	
	header = *spin_Object_Header_Hdl;

	if ((header->backward_Compatible_Version & 0xFFFF0000) > (k_SPIN_OBJECT_VERSION_NOW & 0xFFFF0000)) {
		Copy_Pascal_String("\pThis file is too new to be used safely", message_String);
		Perform_OK_Dialog(message_String);
		goto Error_Handler;
	}
	
	if ((header->backward_Compatible_Version & 0xFFFF0000) < (k_SPIN_OBJECT_BASE_VERSION_NOW & 0xFFFF0000)) {
		Copy_Pascal_String("\pThis file is too old to be used safely", message_String);
		Perform_OK_Dialog(message_String);
		goto Error_Handler;
	}
	
	if (desired_Color_Depth != header->stored_Bit_Depth) {
		Copy_Pascal_String("\pThis file has unanticipated color depth. This program cannot convert it.", message_String);
		Perform_OK_Dialog(message_String);
		goto Error_Handler;
	}
	
	if (spin_Object_Header_Memory_In_Use == MEMORY_INITIALIZED) {
		ReleaseResource((Handle)spin_Object_Header_Hdl);
		if (ResError() != noErr) {
			/*  */
			/* Ignore error, we are exiting anyway  */
			/*  */
		}
		spin_Object_Header_Memory_In_Use = MEMORY_NOT_INITIALIZED;
	}
		
	if (resource_Sources_Swapped == SAFE_TRUE) {
		CloseResFile(input_File_Reference);
		if (ResError() != noErr) {

		}
		
		UseResFile(save_Ref);
		if (ResError() != noErr) {
			/*  */
			/* This should be recoverable from, enhance one day  */
			/*  */
		}
		resource_Sources_Swapped = SAFE_FALSE;
	}
			
	if ((**spin_Info).valid != SAFE_TRUE) {
		goto Error_Handler;
	}
	
	if ((**spin_Info).array_Actual_Count < 1) {
		goto Error_Handler;
	}
			
	if ((**spin_Info).cell_Info[0].color_Depth != desired_Color_Depth) {
		Concat_Many(note_String, 3,"\pNon-matching type of color depth for file ", file_Info->name, "\p Color Depth = ");
		Concat_Long_To_String(note_String, (**spin_Info).cell_Info[0].color_Depth, NO_LIMIT);
		goto Error_Handler_With_Message;
	}
		

	function_Result = SAFE_TRUE;
	goto Normal_Function_Exit;
		
		
Fatal_Memory_Handler:

Error_Handler_With_Message:
	function_Result = SAFE_FALSE;
	Display_Serious_Error_Message(unit_Name, code_Name, note_String, "\p");
	goto Normal_Function_Exit;
	
Error_Handler:
	Display_Serious_Error_Message(unit_Name, code_Name, "\pInternal problem getting Spin_Object", "\p");
	function_Result = SAFE_FALSE;
		
Normal_Function_Exit:

	if (spin_Object_Header_Memory_In_Use == MEMORY_INITIALIZED) {
		ReleaseResource((Handle)spin_Object_Header_Hdl);
		if (ResError() != noErr) {
			/*  */
			/* Ignore error, we are exiting anyway  */
			/*  */
		}
		spin_Object_Header_Memory_In_Use = MEMORY_NOT_INITIALIZED;
	}
		
	if (resource_Sources_Swapped == SAFE_TRUE) {
		CloseResFile(input_File_Reference);
		if (ResError() != noErr) {

		}
		
		UseResFile(save_Ref);
		if (ResError() != noErr) {
			/*  */
			/* This should be recoverable from, enhance one day  */
			/*  */
		}
		resource_Sources_Swapped = SAFE_FALSE;
	}
	
	return(function_Result);
	
}	/* Safe_Boolean Load_Spin_File_Basic_Parts */
/*=**************************************************************************=*/
	
	
/*=**************************************************************************=*/
Safe_Boolean Load_Raw_Pict_Fork_From_Spin_File_And_Draw(FSSpec *file_Info
																		, CARDINAL_16	desired_Color_Depth
																		, Off_Screen_Port_Rec *offscreen_Ptr
																		, Spin_Pict_Info_Handle	spin_Info
																		, CARDINAL_32	cell_Offset)
	/*  */
	/* This draws a raw data stream of frozen pixels a new port of same size.*/
	/* This Allocates a handle and fills it with bitmap image and uses color*/
	/* It is similar to another verswion called "Load_Raw_Pict_Fork_From_Backdrop" */
	/* in another file */
	/*  */
{
	Spin_Object_File_Header_Ptr				header;
	SignedByte 										header_Original_State;
	Spin_Object_File_Header_Hdl				spin_Object_Header_Hdl;
	Memory_Allocation							spin_Object_Header_Memory_In_Use;
	INTEGER_16										total_Puppet_Resources;
	INTEGER_32							total_Bytes_Contiguous_Now;
	INTEGER_32							total_Bytes_Now;
	INTEGER_32 							total_Bytes_Contiguous_Later;
	INTEGER_32							total_Bytes_Later;
	Rect						source_Bit_Rect;
	CARDINAL_32	handle_Size;
	INTEGER_16		current_Pict_Sequence;
	GrafPtr				saved_Port;
	GDHandle			saved_Device;
	Safe_Boolean		function_Result;
	OSErr					os_Error;
	Rect						offscreen_Rect;
	Safe_Boolean		resource_Sources_Swapped;
	INTEGER_16		input_File_Reference;
	INTEGER_16		save_Ref;
	INTEGER_32		transfer_Count;
	INTEGER_32		transfer_Count_Copy;
	INTEGER_32		file_Item_Offset;
	Boolean				still_Searching;
	CARDINAL_8		*code_Name;
	CARDINAL_32	picture_Row_Byte_Offset;
	CARDINAL_16	compression_Type;
																		
	CTabHandle		offscreen_Color_Table_Handle;
	CGrafPtr			offscreen_Graph_Port;
	GDHandle			*offscreen_GDevice_Ptr;
	Handle			*image_Handle_Ptr;

	Str255					note_String;
	Str255					message_String;
/* END VAR */


	code_Name = "\pLoad_Raw_Pict_Fork_From_Spin_File_And_Draw";
	file_Item_Offset = 0;
	function_Result = SAFE_FALSE;
	spin_Object_Header_Memory_In_Use = MEMORY_NOT_INITIALIZED;
	resource_Sources_Swapped = SAFE_FALSE;
																		
	offscreen_Color_Table_Handle = offscreen_Ptr->color_Table_Handle;
	offscreen_Graph_Port = (CGrafPtr)&(offscreen_Ptr->offscreen_Port.color_Port);
	offscreen_GDevice_Ptr = &(offscreen_Ptr->offscreen_GDevice_Hdl);
	image_Handle_Ptr = &(offscreen_Ptr->image_Handle);

	/*  */
	/* We need to open up the file's resource fork and get the information */
	/* to tell us the port size. We also need the information for */
	/* the version to see if compatible or not */
	/*  */
	
	save_Ref = CurResFile();
	input_File_Reference = HOpenResFile (file_Info->vRefNum
									, file_Info->parID
									, file_Info->name
									, fsRdPerm );
						
	if ((input_File_Reference == -1) || (ResError() != noErr)) {
		goto Error_Handler;
	}
	
	
	total_Puppet_Resources = Count1Resources(k_SPIN_OBJECT_MAIN_RESOURCE_TYPE);
	if (ResError() != noErr) {
		goto Error_Handler;
	}
	
	if (total_Puppet_Resources < 1) {
		Get_NS_String(k_DREAM_STRINGS, k_SORRY_WRONG_FILE_INDEX, message_String);
		Perform_OK_Dialog(message_String);
		goto Error_Handler;
	}

	if (Determine_Absolute_Free_Memory_Hook(&total_Bytes_Contiguous_Now
													,&total_Bytes_Now
													,&total_Bytes_Contiguous_Later
													,&total_Bytes_Later) != TRUE) {
		goto Error_Handler;
	}

	spin_Object_Header_Hdl = (Spin_Object_File_Header_Hdl)Get1Resource(k_SPIN_OBJECT_MAIN_RESOURCE_TYPE
																			, k_BASE_SPIN_OBJECT_RESOURCE_ID);
	if ((ResError() != noErr) || (spin_Object_Header_Hdl == nil)) {
		goto Error_Handler;
	}
	spin_Object_Header_Memory_In_Use = MEMORY_INITIALIZED;
	HNoPurge((Handle)spin_Object_Header_Hdl);
	if (MemError() != noErr) {
		/*  */
		/* This cannot fail, but check anyway */
		/*  */
		goto Error_Handler;
	}
		
	if ((**spin_Object_Header_Hdl).valid != SAFE_TRUE) {
		Copy_Pascal_String("\pThis file has corrupted resources and cannot be used safely", message_String);
		Perform_OK_Dialog(message_String);
		goto Error_Handler;
	}
	
	if (Begin_Use_Of_Handle((Handle)spin_Object_Header_Hdl
									, MOVE_HIGH_NOT_REQUIRED
									, &header_Original_State) != SAFE_TRUE) {
		goto Error_Handler;
	}
	
	header = *spin_Object_Header_Hdl;

	if ((header->backward_Compatible_Version & 0xFFFF0000) > (k_SPIN_OBJECT_VERSION_NOW & 0xFFFF0000)) {
		Copy_Pascal_String("\pThis file is too new to be used safely", message_String);
		Perform_OK_Dialog(message_String);
		goto Error_Handler;
	}
	
	if ((header->backward_Compatible_Version & 0xFFFF0000) < (k_SPIN_OBJECT_BASE_VERSION_NOW & 0xFFFF0000)) {
		Copy_Pascal_String("\pThis file is too old to be used safely", message_String);
		Perform_OK_Dialog(message_String);
		goto Error_Handler;
	}
	
	if (desired_Color_Depth != header->stored_Bit_Depth) {
		Copy_Pascal_String("\pThis file has unanticipated color depth. This program cannot convert it.", message_String);
		Perform_OK_Dialog(message_String);
		goto Error_Handler;
	}
	
	offscreen_Rect = header->spin_Object_Size_Rect;

	/*  */
	/* Set up the rectangle for the off-screen graphics environments */
	/*  */
	SetRect(&offscreen_Rect, 0, 0, (offscreen_Rect.right - offscreen_Rect.left), (offscreen_Rect.bottom - offscreen_Rect.top));
	
	if (g_Set_Spin_Window_Max_Monitor_Size == SAFE_TRUE) {
		if (Expand_Rect_To_Largest_Reasonable_Size(&offscreen_Rect) != SAFE_TRUE) {
			/*  */
			/* Ignore errors, not reasonable to fail in implementation */
			/*  */
		}
	}

	
	os_Error= Create_Off_Screen_Port(&offscreen_Rect
											, desired_Color_Depth
											, offscreen_Color_Table_Handle
											, offscreen_Graph_Port
											, offscreen_GDevice_Ptr
											, image_Handle_Ptr);

	if (os_Error != noErr) {
		goto Error_Handler;
	}

	GetPort(&saved_Port);
	saved_Device = GetGDevice();
	
	Set_View_Port((GrafPtr)offscreen_Graph_Port);
	Set_View_Device(*offscreen_GDevice_Ptr);
	
	/*  */
	/* For safety to clear out the memory */
	/*  */
	EraseRect(&offscreen_Rect); 
	
	/*  */
	/* Only needed really for CopyBits calls but why not be certain. */
	/*  */
	ForeColor(blackColor);
	BackColor(whiteColor);
	
	/*  */
	/* Load data into memory here */
	/*  */
	Set_View_Port(saved_Port);
	Set_View_Device(saved_Device);
		
	
	if (spin_Object_Header_Memory_In_Use == MEMORY_INITIALIZED) {
		ReleaseResource((Handle)spin_Object_Header_Hdl);
		if (ResError() != noErr) {
			/*  */
			/* Ignore error, we are exiting anyway  */
			/*  */
		}
		spin_Object_Header_Memory_In_Use = MEMORY_NOT_INITIALIZED;
	}
		
	if (resource_Sources_Swapped == SAFE_TRUE) {
		CloseResFile(input_File_Reference);
		if (ResError() != noErr) {

		}
		
		UseResFile(save_Ref);
		if (ResError() != noErr) {
			/*  */
			/* This should be recoverable from, enhance one day  */
			/*  */
		}
		resource_Sources_Swapped = SAFE_FALSE;
	}
		
	/*  */
	/* Now open data fork and get our large bit map structure. */
	/* The data fork is used because we might want to read */
	/* portions in during interrupts, as a background task, or */
	/* because we need to load it into memory in a specific */
	/* spot and there may not be enough room for two copies */
	/* to exchange from a loaded monster resource. It is true */
	/* that system 7 allows partial resource loading, but some */
	/* resource compressors, and other things fail when these are used. */
	/*  */

	
	if ((**spin_Info).valid != SAFE_TRUE) {
		goto Error_Handler;
	}
	
	if ((**spin_Info).array_Actual_Count < 1) {
		goto Error_Handler;
	}
	
	if (((**spin_Info).array_Actual_Count - 1 ) < cell_Offset) {
		goto Error_Handler;
	}
		
	if ((**spin_Info).cell_Info[cell_Offset].color_Depth != desired_Color_Depth) {
		Concat_Many(note_String, 3,"\pNon-matching type of color depth for file ", file_Info->name, "\p Color Depth = ");
		Concat_Long_To_String(note_String, (**spin_Info).cell_Info[cell_Offset].color_Depth, NO_LIMIT);
		goto Error_Handler_With_Message;
	}
	
	
	offscreen_Ptr->initialized =  SAFE_TRUE; 
	offscreen_Ptr->color_Type = k_COLOR_QUICKDRAW;
	Lock_Video_Memory_For_Lazy_But_Safe_Use(offscreen_Ptr);
		
	current_Pict_Sequence = 0;
	still_Searching = FALSE;
	
	/*  */
	/* Fill in frame of data */
	/*  */
	compression_Type = (**spin_Info).cell_Info[cell_Offset].compression_Type;
	switch(compression_Type) {
		case k_NO_COMPRESSION:
		
			os_Error = HOpen(file_Info->vRefNum
											,file_Info->parID
											,file_Info->name
											,fsRdPerm
											,&input_File_Reference);
											
			if (os_Error != noErr) {
				Concat_Many(note_String, 3,"\pHOpen for file ", file_Info->name, "\p ID = ");
				Concat_Long_To_String(note_String, os_Error, NO_LIMIT);
				goto Error_Handler_With_Message;
			} 
			
			os_Error = SetFPos(input_File_Reference,fsFromStart,(**spin_Info).cell_Info[cell_Offset].data_Start_Offset);
			if (os_Error != noErr) {
				Concat_Many(note_String, 3,"\pSetFPos for file ", file_Info->name, "\p ID = ");
				Concat_Long_To_String(note_String, os_Error, NO_LIMIT);
				goto Error_Handler_With_Message;
			}
		
			transfer_Count = (**spin_Info).cell_Info[cell_Offset].data_Total_Size;
		
			handle_Size = GetHandleSize(*image_Handle_Ptr);
			if (MemError() != noErr) {
				Display_Serious_Error_Message(unit_Name, code_Name, "\pGetHandleSize (1)", "\p");
			}
			else {
			
				if (transfer_Count > handle_Size) {
					transfer_Count = handle_Size;
				}
				
				os_Error = FSRead(input_File_Reference, &transfer_Count, (Ptr)*(*image_Handle_Ptr));
				if (os_Error != noErr) {
					Concat_Many(note_String, 3,"\pFSRead for file ", file_Info->name, "\p ID = ");
					Concat_Long_To_String(note_String, os_Error, NO_LIMIT);
					goto Error_Handler_With_Message;
				}
			}
			
			os_Error = FSClose(input_File_Reference);
			if ( os_Error != noErr) {
				Display_Serious_Error_Message(unit_Name, code_Name, "\pOS Error FSClose.", "\p");
			}

		break;
	
		case k_MERKLE_TIME_COMPRESSION:
			/*  */
			/* Assume file caching already active or this will fail */
			/*  */
			
		/* ====
			if (Draw_Spin_Object_From_Closed_File(file_Info
								, offscreen_Ptr
								, nil
								, spin_Info
								, nil
								, (**spin_Info).cell_Info[cell_Offset].activation
								, (**spin_Info).cell_Info[cell_Offset].horizontal_Angle
								, (**spin_Info).cell_Info[cell_Offset].vertical_Angle
								, (**spin_Info).cell_Info[cell_Offset].roll_Angle
								, offset_Offscreen_Pixel_X
								, offset_Offscreen_Pixel_Y
								) != SAFE_TRUE) {
				Display_Serious_Error_Message(unit_Name, code_Name, "\pDraw_Spin_Object_From_Closed_File failed", "\pNeeds enhancement or buffers unready?");
				goto Error_Handler;
			}
		=== */
		break;
		
		case k_EVEN_ODD_ROW_COMPRESSION:
		default:
			Display_Serious_Error_Message(unit_Name, code_Name, "\pThis version does not handle this type of compressed file", "\pNeeds enhancement");
			goto Error_Handler;
	} /* switch((*spin_Info)->cell_Info[index].compression_Type) */

	

	function_Result = SAFE_TRUE;
	goto Normal_Function_Exit;
		
		
Fatal_Memory_Handler:

Error_Handler_With_Message:
	function_Result = SAFE_FALSE;
	Display_Serious_Error_Message(unit_Name, code_Name, note_String, "\p");
	goto Normal_Function_Exit;
	
Error_Handler:
	Display_Serious_Error_Message(unit_Name, code_Name, "\pInternal problem getting Spin_Object", "\p");
	function_Result = SAFE_FALSE;
		
Normal_Function_Exit:

	if (spin_Object_Header_Memory_In_Use == MEMORY_INITIALIZED) {
		ReleaseResource((Handle)spin_Object_Header_Hdl);
		if (ResError() != noErr) {
			/*  */
			/* Ignore error, we are exiting anyway  */
			/*  */
		}
		spin_Object_Header_Memory_In_Use = MEMORY_NOT_INITIALIZED;
	}
		
	if (resource_Sources_Swapped == SAFE_TRUE) {
		CloseResFile(input_File_Reference);
		if (ResError() != noErr) {

		}
		
		UseResFile(save_Ref);
		if (ResError() != noErr) {
			/*  */
			/* This should be recoverable from, enhance one day  */
			/*  */
		}
		resource_Sources_Swapped = SAFE_FALSE;
	}
	
	return(function_Result);
	
}	/* Safe_Boolean Load_Raw_Pict_Fork_From_Spin_File_And_Draw */
/*=**************************************************************************=*/
	
	
/*=**************************************************************************=*/
Safe_Boolean	Load_Or_Import_Spin_Object_File(Load_Spin_Object_Rec	*params_For_Load_Spin_Object
													, Convert_Spin_Object_Rec	*params_For_Convert_Spin_Object
													, Boolean		importation_Desired)
/*  */
/* Big Bad Juicy routine with a billion linear steps. */
/* Imports and loads picture data, often resulting in dithering and */
/* background removal, and 3D file organization. */
/* Gets assistance from deep offscreens. Does PICT resource and */
/* sequential PICT file imporation from folders. Does Pict file spooling. */
/* does dithering. Allows offscreen deep memory for workspace. */
/* Invokes my Pict compression routines that can encode picture for saving */
/* Disk & RAM storage. Saves compressed pict data if any.  */
/* Is used for a variety of file importation and conversion methods and */
/* is used for simple file loading and is used for self-sufficient drawing file loads. */
/* In Pascal this giant routine would have been simple to design in several stages */
/* in an encapsulated hierarchy, but C is a shitty macroassembler so read it */
/* and weep. I want it this way. Linear and long. */
/*  */
{

	Spin_Object_File_Header_Ptr				header;
	SignedByte 								header_Original_State;
	Spin_Object_File_Header_Hdl				spin_Object_Header_Hdl;
	Memory_Allocation					spin_Object_Header_Memory_In_Use;
	FSSpec										*file_Info;
	CTabHandle								temp_Clut_Handle;
	Resource_Identifier					*first_Item_Ptr;
	Safe_Boolean 							*end_Sentinel_Ptr;
	Off_Screen_Ptr 							offscreen_Ptr;
	Rect												offscreen_Rect;
	
	Off_Screen_Port_Rec						remove_Background_Port;
	Safe_Boolean								remove_Background_Port_Initialized;
	CARDINAL_16							preferred_Background_Port_Depth;
	Rect										preferred_Background_Port_Rect;
	
	CARDINAL_16							desired_Bit_Depth;

	Handle			row_Raw_Data_Hdl;
	Handle			skip_List_Data_Hdl;
	Handle			skip_List_Master_List_Hdl;
	Handle			scan_Line_Bounds_Hdl;

	CARDINAL_32	total_Items_Desired;
	CARDINAL_32	memory_Preferred;
	CARDINAL_32	memory_Needed;
	CARDINAL_32	array_Actual_Count;
	CARDINAL_32	max_Count;
	CARDINAL_32	index;
	
	RgnHandle 		mask;
	INTEGER_32		temp_Mouth_Value;
	Boolean			finished_Importing;
	INTEGER_32		transfer_Count;
	INTEGER_32		transfer_Count_Copy;
	WindowPtr		current_Graphics_Window_Ptr;
	CARDINAL_32	hierachy_Depth_Above;
	Safe_Boolean		function_Result;
	Safe_Boolean		resource_Sources_Swapped;
	Safe_Boolean		file_Is_Pict_File;
	Safe_Boolean		valid_File;
	Safe_Boolean		load_For_Isolated_Self_Sufficient_Drawing;
	INTEGER_16		save_Ref;
	INTEGER_16		item_To_Fetch;
	INTEGER_32		source_Pict_Id;
	Safe_Boolean		file_Is_Open;
	PicHandle			source_Pict_Hdl;
	INTEGER_32		source_Bit_Width;
	INTEGER_32		source_Bit_Height;
	Rect				source_Bit_Rect;
	Rect				temp_Rect;
	Rect				debug_Rect;
	GrafPtr			saved_Port;
	GDHandle			saved_Device;
	GrafPtr			original_Saved_Port;
	GDHandle			original_Saved_Device;
	GrafPtr			temp_Saved_Port;
	GDHandle			temp_Saved_Device;
	INTEGER_32		temp_Vertical_Value;
	INTEGER_16		input_File_Reference;
	INTEGER_16		resource_To_Load;
	Spin_Cell_Index_Ptr	spin_Cell;
	CARDINAL_32	cell_Offset;
	OSErr					os_Error;
	Save_Spin_Object_Rec	save_Spin_Object_Params;
	INTEGER_16		source_Pict_File_Ref;
	INTEGER_32		total_Bytes_Contiguous_Now;
	INTEGER_32		total_Bytes_Now;
	INTEGER_32 		total_Bytes_Contiguous_Later;
	INTEGER_32		total_Bytes_Later;
	CARDINAL_32	temp_Value_32;
	INTEGER_32		source_File_Size;
	FSSpec 				near_Neighbor_File;
	FSSpec 				located_Neighbor_File;
	CTabHandle		temp_Clut_Table;
	Spin_Object_Ptr		current_Spin_Object;
	INTEGER_32		loop;
	INTEGER_32		total_Puppet_Resources;
	INTEGER_32		total_Images_To_Load;
	CARDINAL_32	scrap_Color_Depth;
	INTEGER_32		scrap_Bit_Width;
	INTEGER_32		scrap_Bit_Height;
	INTEGER_32		total_Images_In_Each_Directory;
	CARDINAL_32	compression_Type;
	CARDINAL_32	handle_Size;
	INTEGER_16		current_Pict_Sequence;
	INTEGER_16		input_Data_File_Reference; 
	Safe_Boolean 	user_Abort;
	Safe_Boolean		original_Interactive_Motion;
	Boolean			still_Searching;
	Safe_Boolean		abort_This_Round_And_Allow_Reselect;
	Boolean			temp_Boolean;
	Boolean			user_Warned_About_Lone_Files;
	RGBColor			plain_Flood_Fill_Color;
	RgnHandle 		saved_Clip;
	RgnHandle 		subtracted_Area_Clip;
	Boolean			automatic_Imporation_Desired;
	Boolean			automatic_Imporation_Functioning;
	Boolean			continue_With_File_Usage;
	CARDINAL_8		*code_Name;
	CARDINAL_8		import_Background_Extraction;
	CARDINAL_8 		intensity;
	CARDINAL_8		 horizontal_Angle;
	CARDINAL_8 		vertical_Angle;
	CARDINAL_8 		roll_Angle;
	GrafPort 				simple_Offscreen_Graph_Port;
	Handle 				simple_Image_Handle;
	
	Str255					message_Compress_String;
	Str255					note_String;
	Str255					message_String;
	Str255					status_String;
	Str255					temp_String_2;
	Str255					temp_String;
	Str255					current_Full_Path_Name;
	SignedByte 			original_Handle_State_1;
	SignedByte 			original_Handle_State_2;
	CARDINAL_32		debug_Old_Address;
	CARDINAL_32		debug_New_Address;
	Safe_Boolean			temp_Result;
/* END VAR */


	code_Name = "\pLoad_Or_Import_Spin_Object_File";
	
	g_Background_Removal_Options_Ready = SAFE_FALSE;
	
	function_Result = SAFE_FALSE;
	file_Is_Pict_File = SAFE_FALSE;
	input_Data_File_Reference = 0;
	remove_Background_Port_Initialized = SAFE_FALSE;
	import_Background_Extraction = k_DO_NO_EXTRACTION;

	original_Interactive_Motion = g_Interactive_Motion_Disabled;
	g_Interactive_Motion_Disabled = SAFE_FALSE;
	g_Spin_File_Importation_Active = SAFE_TRUE;
	
	load_For_Isolated_Self_Sufficient_Drawing = SAFE_TRUE;
	
	simple_Image_Handle = nil;
	row_Raw_Data_Hdl = nil;
	skip_List_Data_Hdl = nil;
	skip_List_Master_List_Hdl = nil;
	scan_Line_Bounds_Hdl = nil;
	
	user_Warned_About_Lone_Files = FALSE;

	spin_Object_Header_Memory_In_Use = MEMORY_NOT_INITIALIZED;
			
	resource_Sources_Swapped = SAFE_FALSE;
	
	total_Images_In_Each_Directory = 1;

	GetPort(&original_Saved_Port);
	original_Saved_Device = GetGDevice();
										
	if (importation_Desired != TRUE) {
		params_For_Convert_Spin_Object->desired_File = params_For_Load_Spin_Object->desired_File;
		params_For_Convert_Spin_Object->desired_Spin_Object = params_For_Load_Spin_Object->desired_Spin_Object;
		
		params_For_Convert_Spin_Object->desired_Resource_Index = k_BASE_SPIN_OBJECT_RESOURCE_ID;
		
		load_For_Isolated_Self_Sufficient_Drawing = params_For_Load_Spin_Object->load_File_With_Self_Contained_Large_Ports;
		
		/*  */
		/* Useless paranoia */
		/*  */
		Copy_Pascal_String(&(params_For_Convert_Spin_Object->desired_File.name[0]), params_For_Convert_Spin_Object->desired_Name);
		params_For_Convert_Spin_Object->selected_Pict_Width = 64;
		params_For_Convert_Spin_Object->selected_Pict_Height = 64;
	}
	else {
		if (params_For_Convert_Spin_Object->file_Is_Pict_File_Type == SAFE_TRUE) {
			/*  */
			/* This file is a PICT file, not a PICT resource to load. */
			/*  */
			file_Is_Pict_File = SAFE_TRUE;
		}
	}
										
	/*  */
	/* Open for read only permission */
	/*  */
	if (file_Is_Pict_File != SAFE_TRUE) {
		save_Ref = CurResFile();
		input_File_Reference = HOpenResFile (params_For_Convert_Spin_Object->desired_File.vRefNum
										, params_For_Convert_Spin_Object->desired_File.parID
										, params_For_Convert_Spin_Object->desired_File.name
										, fsRdPerm );
							
		if ((input_File_Reference == -1) || (ResError() != noErr)) {
			goto Error_Handler;
		}
	}
	
	/*  */
	/* 1995.05.14 JAM BUG FIX: was not locked down and holds grafports! */
	/*  */
	MoveHHi((Handle)dream.spin_Objects.spin_Object[params_For_Convert_Spin_Object->desired_Spin_Object]);
	os_Error = MemError();
	if ((os_Error != noErr) && (os_Error != -117)){
		Display_Serious_Error_Message(unit_Name, code_Name, "\pMoveHHi failed", "\pBad programming (corrupted?)");
		goto Error_Handler;
	}
	HLock((Handle)dream.spin_Objects.spin_Object[params_For_Convert_Spin_Object->desired_Spin_Object]);
	if (MemError() != noErr) {
		Display_Serious_Error_Message(unit_Name, code_Name, "\pHLock failed", "\pBad programming (corrupted?)");
		goto Error_Handler;
	}

	/*  */
	/* current_Spin_Object uses a handle assumed to always be locked */
	/*  */
	current_Spin_Object = *(dream.spin_Objects.spin_Object[params_For_Convert_Spin_Object->desired_Spin_Object]);
	
	if ((current_Spin_Object->initialized == SAFE_TRUE) 
		&& (current_Spin_Object->port_Info.spin_Object_Port_1.initialized == SAFE_TRUE)) {
		Get_NS_String(k_SPIN_STRINGS, k_OLD_SPIN_OBJECT_NOT_DISPOSED_YET_ERROR, message_String);
		Perform_OK_Dialog(message_String);
		goto Error_Handler;
	}
	
	if (file_Is_Pict_File != SAFE_TRUE) {
	
		UseResFile(input_File_Reference);
		os_Error = ResError();
		if (os_Error != noErr) {
			Display_Serious_Error_Message(unit_Name, code_Name, "\pInternal using resource", "\pCorrupted?");
			goto Error_Handler;
		}
		
		resource_Sources_Swapped = SAFE_TRUE;
			
		/*  */
		/* Load images into memory, allocating a graphics port */
		/* big enough to hold the images into one tall graphics device port */
		/*  */
			
		if (importation_Desired != TRUE) {
			total_Puppet_Resources = Count1Resources(k_SPIN_OBJECT_MAIN_RESOURCE_TYPE);
			if (ResError() != noErr) {
				goto Error_Handler;
			}
			
			if (total_Puppet_Resources < 1) {
				Get_NS_String(k_DREAM_STRINGS, k_SORRY_WRONG_FILE_INDEX, message_String);
				Perform_OK_Dialog(message_String);
				goto Error_Handler;
			}
			
		}
		else {
		
			total_Images_To_Load = Count1Resources('PICT');
			if (ResError() != noErr) {
				goto Error_Handler;
			}
		
			if (total_Images_To_Load < 1) {
				Get_NS_String(k_DREAM_STRINGS, k_SORRY_WRONG_FILE_INDEX, message_String);
				Perform_OK_Dialog(message_String);
				goto Error_Handler;
			}
		}
		
		
	} /* if (file_Is_Pict_File != SAFE_TRUE) */
	else {
	
		if (Count_PICT_File_Neighbors(&(params_For_Convert_Spin_Object->desired_File), (CARDINAL_32 *)&total_Images_To_Load) != SAFE_TRUE) {
			Get_NS_String(k_DREAM_STRINGS, k_SORRY_WRONG_FILE_INDEX, message_String);
			Perform_OK_Dialog(message_String);
			goto Error_Handler;
		}

	
		if (user_Warned_About_Lone_Files != TRUE) {
			if (total_Images_To_Load < 2) {
				Get_NS_String(k_DREAM_STRINGS, k_NUMBER_OF_FILES_SOLO_AND_NOT_PICTS, message_String);
				Perform_OK_Dialog(message_String);
			}
			user_Warned_About_Lone_Files = TRUE;
		}
		
		total_Images_In_Each_Directory = total_Images_To_Load;
	
	} /* NOT if (file_Is_Pict_File != SAFE_TRUE) */
		
	if (Determine_Absolute_Free_Memory_Hook(&total_Bytes_Contiguous_Now
													,&total_Bytes_Now
													,&total_Bytes_Contiguous_Later
													,&total_Bytes_Later) != TRUE) {
		goto Error_Handler;
	}
		
	if ((dream.spin_Objects.spin_Object_Memory_In_Use[params_For_Convert_Spin_Object->desired_Spin_Object]) != SAFE_TRUE) {
		function_Result = SAFE_FALSE;
		goto Normal_Function_Exit;
	}
		

	if (importation_Desired != TRUE) {

		spin_Object_Header_Hdl = (Spin_Object_File_Header_Hdl)Get1Resource(k_SPIN_OBJECT_MAIN_RESOURCE_TYPE
																				, k_BASE_SPIN_OBJECT_RESOURCE_ID);
		if ((ResError() != noErr) || (spin_Object_Header_Hdl == nil)) {
			goto Error_Handler;
		}
		spin_Object_Header_Memory_In_Use = MEMORY_INITIALIZED;
		HNoPurge((Handle)spin_Object_Header_Hdl);
		if (MemError() != noErr) {
			/*  */
			/* This cannot fail, but check anyway */
			/*  */
			goto Error_Handler;
		}
			
		if ((**spin_Object_Header_Hdl).valid != SAFE_TRUE) {
			Copy_Pascal_String("\pThis file has corrupted resources and cannot be used safely", message_String);
			Perform_OK_Dialog(message_String);
			goto Error_Handler;
		}
		
		if (Begin_Use_Of_Handle((Handle)spin_Object_Header_Hdl
										, MOVE_HIGH_NOT_REQUIRED
										, &header_Original_State) != SAFE_TRUE) {
			goto Error_Handler;
		}
		
		header = *spin_Object_Header_Hdl;
		
	}
	else {
		/*  */
		/* Paranoia only. 'spin_Object_Header_Memory_In_Use' is flag*/
		/*  */
		spin_Object_Header_Hdl = nil;
		header = nil;
	}
	
	current_Spin_Object->spin_Info_Memory_In_Use = MEMORY_NOT_INITIALIZED;
	current_Spin_Object->spin_Info = nil;
	
	current_Spin_Object->file_Reference = 0;

	
	if (importation_Desired == TRUE) {
		
		memory_Needed = sizeof(Spin_Pict_Info_Rec);
		memory_Preferred = total_Images_To_Load;
		/*  */
		/* Double space for puppets and add one frame */
		/*  */
		max_Count = ((memory_Preferred *2UL) + 1UL);
		if (max_Count < 2000) {
			max_Count = 2000;
		}
		memory_Needed += (max_Count * sizeof(Spin_Cell_Index));
		memory_Preferred = memory_Needed;
		
		if (Create_Handle(&memory_Preferred
							, (CARDINAL_32)(10240L)
							, ALLOW_PURGING
							, DISALLOW_PARTIAL_AMOUNT
							, (Handle *)&(current_Spin_Object->spin_Info)
							, &(current_Spin_Object->spin_Info_Memory_In_Use)) != SAFE_TRUE) {
							
			Display_Low_Memory_Error_Message(unit_Name, code_Name, memory_Needed);
			function_Result = SAFE_FALSE;
			goto Normal_Function_Exit;
		}
		
		Fill_Byte_Memory_With_Byte((Ptr)*(current_Spin_Object->spin_Info), memory_Needed, 0xFFFF);
		/*  */
		/* Initialize most fields */
		/*  */
		(**(current_Spin_Object->spin_Info)).array_Max_Count_Size = max_Count;
		(**(current_Spin_Object->spin_Info)).valid = SAFE_TRUE;
		(**(current_Spin_Object->spin_Info)).version = k_SPIN_OBJECT_VERSION_NOW;
		(**(current_Spin_Object->spin_Info)).max_Depth = 0;
		SetRect(&((**(current_Spin_Object->spin_Info)).max_Bounds), 0, 1, 0, 1);
		(**(current_Spin_Object->spin_Info)).array_Actual_Count = 0;
		
		/*  */
		/* Paranoia clear import cell fields */
		/*  */
	
		for(index = 0; (index < total_Images_To_Load); index++ ) {
			spin_Cell = &((**(current_Spin_Object->spin_Info)).cell_Info[index]);
			spin_Cell->data_Start_Offset = 0;
			spin_Cell->data_Total_Size = 0;
			spin_Cell->compression_Type = k_NO_COMPRESSION;
			spin_Cell->color_Depth = 0;
			spin_Cell->activation = 0;
			spin_Cell->horizontal_Angle = 0;
			spin_Cell->vertical_Angle = 0;
			spin_Cell->roll_Angle = 0;
			spin_Cell->picture_Row_Byte_Offset = 0;
			spin_Cell->max_Picture_Bounds.top = 0;
			spin_Cell->max_Picture_Bounds.left = 0;
			spin_Cell->max_Picture_Bounds.right = 0;
			spin_Cell->max_Picture_Bounds.bottom = 0;
		}
		
	} /* if (importation_Desired == TRUE)  */
	else {
	
		file_Info = &(params_For_Convert_Spin_Object->desired_File);
		
		/*  */
		/* Now open data fork and get our pict cell map structure. */
		/* The data fork is used because we might want to read */
		/* portions in during interrupts, as a background task, or */
		/* because we need to load it into memory in a specific */
		/* spot, or because hand made array structure across two drives. */
		/*  */
	
		os_Error = HOpen(file_Info->vRefNum
										,file_Info->parID
										,file_Info->name
										,fsRdPerm
										,&input_Data_File_Reference);
		if (os_Error != noErr) {
			Concat_Many(note_String, 3,"\pHOpen for file ", file_Info->name, "\p ID = ");
			Concat_Long_To_String(note_String, os_Error, NO_LIMIT);
			goto Error_Handler_With_Message;
		} 
		
		transfer_Count = sizeof(Safe_Boolean);
		os_Error = FSRead(input_Data_File_Reference, &transfer_Count, (Ptr)&valid_File);
		if (os_Error != noErr) {
			Concat_Many(note_String, 3,"\pFSRead for file ", file_Info->name, "\p ID = ");
			Concat_Long_To_String(note_String, os_Error, NO_LIMIT);
			goto Error_Handler_With_Message;
		}
		
		if (valid_File != SAFE_TRUE) {
			Concat_Many(note_String, 2,"\pBad 'validity' flag for file. Closed wrong? Was file ", file_Info->name);
			Concat_Long_To_String(note_String, os_Error, NO_LIMIT);
			goto Error_Handler_With_Message;
		}
		
		transfer_Count = sizeof(CARDINAL_32);
		os_Error = FSRead(input_Data_File_Reference, &transfer_Count, (Ptr)&array_Actual_Count);
		if (os_Error != noErr) {
			Concat_Many(note_String, 3,"\pFSRead for file ", file_Info->name, "\p ID = ");
			Concat_Long_To_String(note_String, os_Error, NO_LIMIT);
			goto Error_Handler_With_Message;
		}
		
		/*  */
		/* Reset back to start. Will read in whole structure */
		/*  */
		os_Error = SetFPos(input_Data_File_Reference,fsFromStart,0);
		if (os_Error != noErr) {
			Concat_Many(note_String, 3,"\pSetFPos for file ", file_Info->name, "\p ID = ");
			Concat_Long_To_String(note_String, os_Error, NO_LIMIT);
			goto Error_Handler_With_Message;
		}
				
		total_Images_To_Load = array_Actual_Count;
		memory_Needed = sizeof(Spin_Pict_Info_Rec);
		memory_Needed += (array_Actual_Count) * sizeof(Spin_Cell_Index);
		memory_Preferred = memory_Needed;
		
		if (Create_Handle(&memory_Preferred
							, (CARDINAL_32)(10240L)
							, ALLOW_PURGING
							, DISALLOW_PARTIAL_AMOUNT
							, (Handle *)&(current_Spin_Object->spin_Info)
							, &(current_Spin_Object->spin_Info_Memory_In_Use)) != SAFE_TRUE) {
							
			Display_Low_Memory_Error_Message(unit_Name, code_Name, memory_Needed);
			function_Result = SAFE_FALSE;
			goto Normal_Function_Exit;
		}
		
		Fill_Byte_Memory_With_Byte((Ptr)*(current_Spin_Object->spin_Info), memory_Needed, 0xFFFF);
		/*  */
		/* Initialize most fields */
		/*  */
		
		transfer_Count = memory_Needed;
		os_Error = FSRead(input_Data_File_Reference, &transfer_Count, (Ptr)*(current_Spin_Object->spin_Info));
		if (os_Error != noErr) {
			Concat_Many(note_String, 3,"\pFSRead for file ", file_Info->name, "\p ID = ");
			Concat_Long_To_String(note_String, os_Error, NO_LIMIT);
			goto Error_Handler_With_Message;
		}
		
		os_Error = FSClose(input_Data_File_Reference);
		if ( os_Error != noErr) {
			Display_Serious_Error_Message(unit_Name, code_Name, "\pOS Error FSClose.", "\p");
		}
		input_Data_File_Reference = 0;
		
	} /* NOT if (importation_Desired == TRUE)  */
	

	/*  */
	/* Initialize most fields */
	/*  */
	current_Spin_Object->initialized = SAFE_TRUE;
	
	current_Spin_Object->isolated_Port_Drawing_Active = SAFE_FALSE;
	current_Spin_Object->port_Info.spin_Object_Port_1.initialized = SAFE_FALSE;
	current_Spin_Object->port_Info.spin_Object_Port_2.initialized = SAFE_FALSE;
		
	SetRect(&(current_Spin_Object->spin_Object_Background_Rect), 0, 0, 1, 1);
	current_Spin_Object->special_Effect_Spin_Object = 0;
	current_Spin_Object->spin_Object_Name[0] = 0;
	current_Spin_Object->spin_Object_Remaining_To_Load = SAFE_FALSE;
	current_Spin_Object->spin_Object_Loading_Last_Index = 0;
	
	current_Spin_Object->port_Info.offset_Offscreen_Pixel_X = 0;
	current_Spin_Object->port_Info.offset_Offscreen_Pixel_Y = 0;
	
	current_Spin_Object->port_Info.onscreen_Offscreen_Pixel_X = 0;
	current_Spin_Object->port_Info.onscreen_Offscreen_Pixel_Y = 0;
	
	os_Error = FSMakeFSSpec(params_For_Convert_Spin_Object->desired_File.vRefNum
									, params_For_Convert_Spin_Object->desired_File.parID
									, params_For_Convert_Spin_Object->desired_File.name
									,&(current_Spin_Object->spin_Object_Location));
	if (os_Error != noErr) {
		goto Error_Handler;
	}
	
	if (importation_Desired == TRUE) {

		if (g_Background_Removal_Options_Ready != SAFE_TRUE) {
			g_Background_Removal_Options_Ready = SAFE_TRUE;
			if (Get_Background_Removal_Options(&g_Options
													, 0
													, desired_Bit_Depth) != TRUE) {
				/*  */
				/* Do not honor cancel command. */
				/*  */
			}
		}
		
		current_Spin_Object->spin_Object_Name[0] = 0;
		SetRect (&offscreen_Rect
						, 0
						, 0
						, params_For_Convert_Spin_Object->selected_Pict_Width
						, params_For_Convert_Spin_Object->selected_Pict_Height );

	/* ===
		if (g_Set_Spin_Window_Max_Monitor_Size == SAFE_TRUE) {
			if (Expand_Rect_To_Largest_Reasonable_Size(&offscreen_Rect) != SAFE_TRUE) {
			}
		}
	==== */

		/*  */
		/* Properly handled below  */
		/*  */
		desired_Bit_Depth = 8;
		
		Copy_Pascal_String(params_For_Convert_Spin_Object->desired_Name
										, current_Spin_Object->spin_Object_Name);
		Strip_Off_Popular_Suffixes(&(current_Spin_Object->spin_Object_Name));

	}
	else {
	
		Copy_Pascal_String(header->file_Name, current_Spin_Object->spin_Object_Name);
	
		if ((header->backward_Compatible_Version & 0xFFFF0000) > (k_SPIN_OBJECT_VERSION_NOW & 0xFFFF0000)) {
			Copy_Pascal_String("\pThis file is too new to be used safely", message_String);
			Perform_OK_Dialog(message_String);
			goto Error_Handler;
		}
		
		if ((header->backward_Compatible_Version & 0xFFFF0000) < (k_SPIN_OBJECT_BASE_VERSION_NOW & 0xFFFF0000)) {
			Copy_Pascal_String("\pThis file is too old to be used safely", message_String);
			Perform_OK_Dialog(message_String);
			goto Error_Handler;
		}
		
		offscreen_Rect = header->spin_Object_Size_Rect;
		desired_Bit_Depth = header->stored_Bit_Depth;
	
		if (g_Set_Spin_Window_Max_Monitor_Size == SAFE_TRUE) {
			if (Expand_Rect_To_Largest_Reasonable_Size(&offscreen_Rect) != SAFE_TRUE) {
				/*  */
				/* Ignore errors, not reasonable to fail in implementation */
				/*  */
			}
		}
	
	} /* NOT if (importation_Desired == TRUE) */
	
	current_Spin_Object->spin_Object_Background_Rect = offscreen_Rect;
		
	/*  */
	/* Now create the offscreen picture!  */
	/*  */
	
	 if (current_Spin_Object->port_Info.spin_Object_Port_1.initialized == SAFE_TRUE) {
	 	if (Close_Offscreen_Pict_Port(&(current_Spin_Object->port_Info.spin_Object_Port_1)) != SAFE_TRUE) {
			/*  */
			/* might as well complain about it. */
			/*  */
	 		goto Error_Handler;
	 	}
	 }
	 
	 if (current_Spin_Object->port_Info.spin_Object_Port_2.initialized == SAFE_TRUE) {
	 	if (Close_Offscreen_Pict_Port(&(current_Spin_Object->port_Info.spin_Object_Port_2)) != SAFE_TRUE) {
			/*  */
			/* might as well complain about it. */
			/*  */
	 		goto Error_Handler;
	 	}
	 }
	 
	/* ==== ==== ==== ====  ==== ==== ==== ==== ==== ==== ==== ====  ==== ==== ==== ====  */
	/* ==== ==== ==== ====  ==== ==== ==== ==== ==== ==== ==== ====  ==== ==== ==== ====  */
	 
	 
	/* ==== ==== ==== ====  ==== ==== ==== ==== ==== ==== ==== ====  ==== ==== ==== ====  */
	/* ==== ==== ==== ====  ==== ==== ==== ==== ==== ==== ==== ====  ==== ==== ==== ====  */
	 
	 if (load_For_Isolated_Self_Sufficient_Drawing == SAFE_TRUE) {
	 
		current_Spin_Object->isolated_Port_Drawing_Active = SAFE_TRUE;
		
		offscreen_Ptr = &(current_Spin_Object->port_Info.spin_Object_Port_1);
		offscreen_Ptr->initialized = SAFE_FALSE;
		offscreen_Ptr->offscreen_GDevice_Hdl = nil;
		offscreen_Ptr->image_Handle = nil;
		offscreen_Ptr->color_Table_Handle = nil;
		offscreen_Ptr->color_Type = k_COLOR_QUICKDRAW;		 
	
		if ((importation_Desired == TRUE) && ((g_Preferred_Monitor_Depth > 8) && (g_Preferred_Monitor_Depth <= 48))){
				
			Get_NS_String(k_SPIN_STRINGS
							, k_IMPORT_SPIN_OBJECT_ALLOW_NON_256_COLOR
							, message_String);
			
			Concat_Long_To_String(message_String, g_Preferred_Monitor_Depth, NO_LIMIT);
				
			if (Perform_Yes_No_Dialog(message_String) != TRUE) {
				desired_Bit_Depth = 8;
				offscreen_Ptr->color_Table_Handle = GetCTable(k_DEFAULT_256_COLOR_SYSTEM_CLUT);
			}
			else {
				desired_Bit_Depth = g_Preferred_Monitor_Depth;
				if (Create_Clut_For_Direct_Mapping(g_Preferred_Monitor_Depth, &(offscreen_Ptr->color_Table_Handle)) != SAFE_TRUE) {
		 			goto Error_Handler;
				}
			}		
		}
		else {
			if ((g_Preferred_Monitor_Depth > 8) && (g_Preferred_Monitor_Depth <= 48)) {
				desired_Bit_Depth = g_Preferred_Monitor_Depth;
				if (Create_Clut_For_Direct_Mapping(g_Preferred_Monitor_Depth, &(offscreen_Ptr->color_Table_Handle)) != SAFE_TRUE) {
		 			goto Error_Handler;
				}
			}
			else {
				desired_Bit_Depth = 8;
				offscreen_Ptr->color_Table_Handle = GetCTable(k_DEFAULT_256_COLOR_SYSTEM_CLUT);
			}
		}
		
		if (offscreen_Ptr->color_Table_Handle == nil) {
				Display_Serious_Error_Message(unit_Name
																	, code_Name
																	, "\pProblem Loading System CLUT (2)"
																	, "\pCorrupted?");
			goto Error_Handler;
		}	
	
		MoveHHi((Handle)offscreen_Ptr->color_Table_Handle);
		if (MemError() != noErr) {
				Display_Serious_Error_Message(unit_Name
																	, code_Name
																	, "\pProblem locking handle (2.0)"
																	, "\pCorrupted!");
			goto Error_Handler;
		}
		
		HLock((Handle)offscreen_Ptr->color_Table_Handle);
		if (MemError() != noErr) {
				Display_Serious_Error_Message(unit_Name
																	, code_Name
																	, "\pProblem locking handle (2.1)"
																	, "\pCorrupted!");
			goto Error_Handler;
		}
		
		if (Determine_Absolute_Free_Memory_Hook(&total_Bytes_Contiguous_Now
														,&total_Bytes_Now
														,&total_Bytes_Contiguous_Later
														,&total_Bytes_Later) != TRUE) {
			goto Error_Handler;
		}
		
		memory_Needed = desired_Bit_Depth;
		memory_Needed = memory_Needed * (CARDINAL_32)(offscreen_Rect.right - offscreen_Rect.left);
		memory_Needed = memory_Needed * (CARDINAL_32)(offscreen_Rect.bottom - offscreen_Rect.top);
		memory_Needed = memory_Needed >> 3;
		
		if (total_Bytes_Contiguous_Later < (k_MINIMUM_RAM_TO_PRESERVE_FOR_BIG_STUFF + memory_Needed)) {
			total_Bytes_Contiguous_Later = MaxMem(&total_Bytes_Now);
			if (total_Bytes_Contiguous_Later < (k_MINIMUM_RAM_TO_PRESERVE_FOR_BIG_STUFF + memory_Needed)) {
				total_Bytes_Contiguous_Later = MaxMem(&total_Bytes_Now);
		
				Get_NS_String(k_DREAM_ERROR_STRINGS, k_ERROR_NOT_ENOUGH_MEMORY_TO_SAFELY_LOAD, note_String);
				Make_String_Long_String(&message_String[0], &note_String[0], memory_Needed, NO_LIMIT, "\p bytes.");
				Perform_OK_Dialog(message_String);
				goto Error_Handler;
			}
		}
	
		
	 } /*  if (load_For_Isolated_Self_Sufficient_Drawing == SAFE_TRUE) */
	 else {
	 
		current_Spin_Object->isolated_Port_Drawing_Active = SAFE_FALSE;
		offscreen_Ptr = (Off_Screen_Ptr)0xFFFFFFFF;
		desired_Bit_Depth = g_Preferred_Monitor_Depth;
		
	 } /*  NOT if (load_For_Isolated_Self_Sufficient_Drawing == SAFE_TRUE) */
	 

	

	if (((importation_Desired != TRUE)) && (header->spin_Object_Special_Effect == SAFE_TRUE)) {

		 if (load_For_Isolated_Self_Sufficient_Drawing == SAFE_TRUE) {
			os_Error= Create_Off_Screen_Port(&offscreen_Rect
																	, desired_Bit_Depth
																	, offscreen_Ptr->color_Table_Handle
																	, (CGrafPtr)&(offscreen_Ptr->offscreen_Port.color_Port)
																	, &(offscreen_Ptr->offscreen_GDevice_Hdl)
																	, &(offscreen_Ptr->image_Handle));
			
			if (os_Error != noErr) {
				Copy_Pascal_String("\pA graphics port for this puppet file failed to be created (memory?)", message_String);
				Perform_OK_Dialog(message_String);
				goto Error_Handler;
			}
			
			/*  */
			/* Redesign to avoid locked crap one day */
			/*  */
			
			offscreen_Ptr->initialized =  SAFE_TRUE; 
			offscreen_Ptr->color_Type = k_COLOR_QUICKDRAW;
			
			Lock_Video_Memory_For_Lazy_But_Safe_Use(offscreen_Ptr);
	
		 }
		/* ==== ==== ==== ====  ==== ==== ==== ==== ==== ==== ==== ====  ==== ==== ==== ====  */


		current_Spin_Object->special_Effect_Spin_Object = header->effect_Type;
		current_Spin_Object->spin_Object_Remaining_To_Load = SAFE_FALSE;
		current_Spin_Object->spin_Object_Loading_Last_Index = 0;
		
		/*  */
		/* Fill with solid effect color */
		/*  */
		/* ===
			"current_Spin_Object->spin_Object_Background_Rect"
		==== */
		
	} /* if (((importation_Desired != TRUE)) && (header->spin_Object_Special_Effect == SAFE_TRUE))  */
	else{
		current_Spin_Object->special_Effect_Spin_Object = 0;
		current_Spin_Object->spin_Object_Remaining_To_Load = SAFE_TRUE;
		current_Spin_Object->spin_Object_Loading_Last_Index = 0;
		dream.spin_Objects.spin_Objects_Still_Loading += 1;
		
		automatic_Imporation_Desired = FALSE;
		automatic_Imporation_Functioning = FALSE;

		/*  */
		/* The data can now loaded, preferably in stages */
		/* so that animation can proceed */
		/*  */
		
		/*  */
		/* At this point the other two stages have been completed */
		/* All that remains is to create the port and then */
		/* load the pict into memory. */
		/*  */
		
		 if (load_For_Isolated_Self_Sufficient_Drawing == SAFE_TRUE) {
			offscreen_Ptr = &(current_Spin_Object->port_Info.spin_Object_Port_1);
		}
		else {
			offscreen_Ptr = (Off_Screen_Ptr)0xFFFFFFFF;
		}
		
		if (file_Is_Pict_File != SAFE_TRUE) {
	
			if (importation_Desired == TRUE) {
				resource_To_Load = params_For_Convert_Spin_Object->desired_Resource_Index;
				
				/*  */
				/* 1994.06.05 JAM This better create all needed items! */
				/*  */
				if (Load_Pict_Into_Color_Bit_Map(resource_To_Load
																, desired_Bit_Depth
																, offscreen_Ptr->color_Table_Handle
																, (CGrafPtr)&(offscreen_Ptr->offscreen_Port.color_Port)
																, &(offscreen_Ptr->offscreen_GDevice_Hdl)
																, &(offscreen_Ptr->image_Handle)) != SAFE_TRUE) {
					goto Error_Handler;
				}
				
			}
			else {
				
				resource_To_Load = 0;
				cell_Offset = 0;
				
				 if (load_For_Isolated_Self_Sufficient_Drawing != SAFE_TRUE) {
				 
					if (Load_Spin_File_Basic_Parts(&(params_For_Load_Spin_Object->desired_File)
													, desired_Bit_Depth
													, current_Spin_Object->spin_Info) != SAFE_TRUE) {
						goto Error_Handler;
					}
					
				 }
				 else {
					/*  */
					/* Load_Raw_Pict_Fork_From_Spin_File_And_Draw needs to complete */
					/*  the graph port opening wihin it. (just a couple fields) */
					/* But it is not done if the port is not meant to be used. */
					/*  */
					if (Load_Raw_Pict_Fork_From_Spin_File_And_Draw(&(params_For_Load_Spin_Object->desired_File)
																	, desired_Bit_Depth
																	, offscreen_Ptr
																	, current_Spin_Object->spin_Info
																	, cell_Offset) != SAFE_TRUE) {
																
						/*  */
						/* Failure. Load default spin_Object (not a PICT file?) */
						/*  */
						
						resource_To_Load = k_BASE_SPIN_OBJECT_RESOURCE_ID;
						if (Load_Pict_Into_Color_Bit_Map(resource_To_Load
																, desired_Bit_Depth
																, offscreen_Ptr->color_Table_Handle
																, (CGrafPtr)&(offscreen_Ptr->offscreen_Port.color_Port)
																, &(offscreen_Ptr->offscreen_GDevice_Hdl)
																, &(offscreen_Ptr->image_Handle)) != SAFE_TRUE) {
																	
							Display_Serious_Error_Message(unit_Name
																				, code_Name
																				, "\pProblem Loading pict resources (Memory?) ,2"
																				, "\pMemory not available?");
							goto Error_Handler;
						}			
					}
				 
				 } /*  if (load_For_Isolated_Self_Sufficient_Drawing == SAFE_TRUE) */
								
			}
		} /* if (file_Is_Pict_File != SAFE_TRUE) */
		else {

			/*  */
			/* Only reaches here is importing from picts, not the normal load. */
			/*  */

			resource_To_Load = 0;

			if (Load_Pict_File_Into_Color_Bit_Map(&(params_For_Convert_Spin_Object->desired_File)
															, desired_Bit_Depth
															, offscreen_Ptr->color_Table_Handle
															, (CGrafPtr)&(offscreen_Ptr->offscreen_Port.color_Port)
															, &(offscreen_Ptr->offscreen_GDevice_Hdl)
															, &(offscreen_Ptr->image_Handle)) != SAFE_TRUE) {
				/*  */
				/* If it fails just load the deafult background from App resources*/
				/*  */
				resource_To_Load = k_BASE_SPIN_OBJECT_RESOURCE_ID;
				if (Load_Pict_Into_Color_Bit_Map(resource_To_Load
																, desired_Bit_Depth
																, offscreen_Ptr->color_Table_Handle
																, (CGrafPtr)&(offscreen_Ptr->offscreen_Port.color_Port)
																, &(offscreen_Ptr->offscreen_GDevice_Hdl)
																, &(offscreen_Ptr->image_Handle)) != SAFE_TRUE) {
					goto Error_Handler;
				}
			}
			else {
				/*  */
				/* Set "automatic_Imporation_Desired" */
				/* because it is a knockdown flag from here on down. */
				/*  */
				automatic_Imporation_Desired = TRUE;
				automatic_Imporation_Functioning = TRUE;
			
				if (Create_Pathname_For_Directory(near_Neighbor_File.parID
										, near_Neighbor_File.vRefNum
										, 3
										, &current_Full_Path_Name[0]) != SAFE_TRUE) {
										
					/*  */
					/* Ignore errors, there should not be any */
					/*  */
					current_Full_Path_Name[0] = 0;
				}
				
			}
						
		} /* NOT if (file_Is_Pict_File != SAFE_TRUE) */

		/*  */
		/* 3 of four previous routines still need this step */
		/* "Load_Raw_Pict_Fork_From_Spin_File_And_Draw" was exception */
		/*  */
		 if (load_For_Isolated_Self_Sufficient_Drawing == SAFE_TRUE) {
			offscreen_Ptr->initialized =  SAFE_TRUE; 
			offscreen_Ptr->color_Type = k_COLOR_QUICKDRAW;
			Lock_Video_Memory_For_Lazy_But_Safe_Use(offscreen_Ptr);
		}
		

		/*  */
		/* Done loading */
		/*  */
		current_Spin_Object->spin_Object_Remaining_To_Load = SAFE_FALSE;
		current_Spin_Object->spin_Object_Loading_Last_Index = 0;
		
		if (importation_Desired == TRUE) {
			current_Spin_Object->spin_Object_Needs_Saving = SAFE_TRUE;
		}
		else {
			current_Spin_Object->spin_Object_Needs_Saving = SAFE_FALSE;
		}
				
		if (dream.spin_Objects.spin_Objects_Still_Loading > 0) {
			dream.spin_Objects.spin_Objects_Still_Loading = dream.spin_Objects.spin_Objects_Still_Loading - 1;
		}
		

			
		/* ==== ==== ==== ====  ==== ==== ==== ==== ==== ==== ==== ====  ==== ==== ==== ====  */
		/* ==== ==== ==== ====  ==== ==== ==== ==== ==== ==== ==== ====  ==== ==== ==== ====  */
		/*  */
		/* Done with dither into bitmap, but how about load into highest bit depth */
		/* so that background removal may take place? */
		/*  */

		if ((importation_Desired == TRUE) && (import_Background_Extraction == k_DO_NO_EXTRACTION)) {
			if (Perform_Yes_No_Dialog(
				"\pShould the semi-solid hued background be removed from this sequence, if present?") == TRUE) {
								
				if (desired_Bit_Depth > 8) {
					import_Background_Extraction = k_EXTRACT_HIGH_DEPTH_FROM_SOURCE;
				}
				else {
				
					if (Guarantee_File_Is_PICT_Type_And_Parse_Depth(params_For_Convert_Spin_Object->desired_File.vRefNum
						, params_For_Convert_Spin_Object->desired_File.parID
						, &(params_For_Convert_Spin_Object->desired_File.name[0])
						, &scrap_Bit_Width
						, &scrap_Bit_Height
						, SAFE_TRUE
						, &scrap_Color_Depth) != SAFE_TRUE) {
						/*  */
						/* IGNORE PROBLEM , merely assume 16 bit */
						/* Indeterminant */
						/*  */
						import_Background_Extraction = k_EXTRACT_HIGH_DEPTH_FROM_COPY_IF_NEEDED;
					}
					else {
						if (scrap_Color_Depth <= 8) {
							import_Background_Extraction = k_EXTRACT_LOW_DEPTH_FROM_SOURCE;
						}
						else {
							import_Background_Extraction = k_EXTRACT_HIGH_DEPTH_FROM_COPY;
						}
					}
				}
			}
		}

		/* ==== ==== ==== ====  ==== ==== ==== ==== ==== ==== ==== ====  ==== ==== ==== ====  */
		/* ==== ==== ==== ====  ==== ==== ==== ==== ==== ==== ==== ====  ==== ==== ==== ====  */

	} /* NOT if (((importation_Desired != TRUE)) && (header->spin_Object_Special_Effect == SAFE_TRUE))  */
	
	
	/* ==== ==== ==== ====  ==== ==== ==== ==== ==== ==== ==== ====  ==== ==== ==== ====  */

	 if (load_For_Isolated_Self_Sufficient_Drawing == SAFE_TRUE) {

		if (Copy_Off_Screen_Port(&(current_Spin_Object->port_Info.spin_Object_Port_1)
													, &(current_Spin_Object->port_Info.spin_Object_Port_2)) != SAFE_TRUE) {
			Copy_Pascal_String("\pA graphics port for this puppet file failed to be created (memory?)", message_String);
			Perform_OK_Dialog(message_String);
			goto Error_Handler;
		}
		
		/*  */
		/* Erase foreground (primary) composite background with */
		/* chroma value */
		/*  */
		handle_Size = GetHandleSize(current_Spin_Object->port_Info.spin_Object_Port_1.image_Handle);
		if (handle_Size != 0) {
			Fill_Large_Buffer((Ptr)*(current_Spin_Object->port_Info.spin_Object_Port_1.image_Handle)
								, handle_Size, g_Chroma_Index_Optimized_Long);
		}
		
		/*  */
		/* Erase secondary (Background) composite background with */
		/* chroma value, it might be used as source in partial playback mode. */
		/*  */
		handle_Size = GetHandleSize(current_Spin_Object->port_Info.spin_Object_Port_2.image_Handle);
		if (handle_Size != 0) {
			Fill_Large_Buffer((Ptr)*(current_Spin_Object->port_Info.spin_Object_Port_2.image_Handle)
								, handle_Size, g_Chroma_Index_Optimized_Long);
		}
		
	} /*  if (load_For_Isolated_Self_Sufficient_Drawing == SAFE_TRUE) */
		
	/*  */
	/* 1995.01.28 JAM ENHANCEMENT */
	/* If imporation is being done AND if backgrounds are being removed */
	/* then determine if background should be removed from normal */
	/* import buffer or from an intermediate high depth buffer. After */
	/* background mask ready, use mask to preserve pixels in target and */
	/* set background to uniform color (Chroma Key Blue) for simplicity */ 
	/*  */
	

	/* ==== ==== ==== ====  ==== ==== ==== ==== ==== ==== ==== ====  ==== ==== ==== ====  */
	
	
	 if (importation_Desired == TRUE) {
		(**(current_Spin_Object->spin_Info)).max_Depth = desired_Bit_Depth;
		(**(current_Spin_Object->spin_Info)).max_Bounds = offscreen_Rect;
		(**(current_Spin_Object->spin_Info)).array_Actual_Count = 0;
	 }
	
				
	if (spin_Object_Header_Memory_In_Use == MEMORY_INITIALIZED) {
		ReleaseResource((Handle)spin_Object_Header_Hdl);
		if (ResError() != noErr) {
			/*  */
			/* Ignore error, we are exiting anyway  */
			/*  */
		}
		spin_Object_Header_Memory_In_Use = MEMORY_NOT_INITIALIZED;
	}
		
	if (resource_Sources_Swapped == SAFE_TRUE) {
		CloseResFile(input_File_Reference);
		if (ResError() != noErr) {
			goto Error_Handler;
		}
		
		UseResFile(save_Ref);
		if (ResError() != noErr) {
			/*  */
			/* This should be recoverable from, enhance one day  */
			/*  */
		}
		resource_Sources_Swapped = SAFE_FALSE;
	}

	/*  */
	/* Add to counter. Finished with it. */
	/*  */
	dream.spin_Objects.total_Spin_Objects += 1;
	
	/*  */
	/* We could compare against total count and ASSUME it is */
	/* already started but that would be incorrect */
	/* Though WOULD eliminate errors on each load attempt, */
	/* but would have hard time self repairing? */
	/*  */
	if (Reset_And_Open_Spin_Cache_If_Needed() != SAFE_TRUE) {
		/*  */
		/* might as well complain about it? No not in this version */
		/*  */

		goto Error_Handler;
	}
	
	/*  */
	/* If importing, write out each frame to file */
	/*  */
	if (importation_Desired == TRUE) {
	
		Get_NS_String(k_DREAM_STRINGS
					, k_LOADING_AND_CONVERTING_TO_CURRENT_COLORS
					, message_String); 

		Get_NS_String(k_DREAM_STRINGS
					, k_COMPRESSING_AND_SAVING_FRAME
					, message_Compress_String); 
									
		Get_NS_String(k_DREAM_STRINGS
					, k_REMAINING_FRAMES
					, temp_String);
					

			
		/*  */
		/* First, force save result file. Writes assuming 0 files*/
		/*  */

		save_Spin_Object_Params.desired_Spin_Object = params_For_Convert_Spin_Object->desired_Spin_Object;
		save_Spin_Object_Params.desired_File = params_For_Convert_Spin_Object->output_Converted_File;
		save_Spin_Object_Params.async_Task_Desired = SAFE_FALSE;

		if (Save_Spin_Object_Sequence(&save_Spin_Object_Params) != SAFE_TRUE) {
			Display_Serious_Error_Message(unit_Name, code_Name, "\pSave_Spin_Object_Sequence failed", "\pDisk Space, programming error?");
			goto Error_Handler;
		}

		os_Error = FSMakeFSSpec(params_For_Convert_Spin_Object->output_Converted_File.vRefNum
									, params_For_Convert_Spin_Object->output_Converted_File.parID
									, params_For_Convert_Spin_Object->output_Converted_File.name
									,&(current_Spin_Object->spin_Object_Location));

		
		near_Neighbor_File = params_For_Convert_Spin_Object->desired_File;

		/* ====== ====== ====== ====== ====== ====== ====== ====== ====== */
		/* ====== ====== ====== ====== ====== ====== ====== ====== ====== */
		
#if !defined(k_FORCE_FULL_MAC_DITHER)
#error "UNDEFINED PARAMETER!"
#endif
#if  (k_FORCE_FULL_MAC_DITHER == 1)
			if (import_Background_Extraction == k_DO_NO_EXTRACTION) {
				if (desired_Bit_Depth <= 8) {
					import_Background_Extraction = k_COPY_ALL_PIXELS_BUT_DITHER_TO_INDEXED_FROM_DEEP;
				}
			}
#endif


			/*  */
			/* Prepare extraction buffer if needed */
			/*  */
			
		switch(import_Background_Extraction) {
									
			case k_EXTRACT_LOW_DEPTH_FROM_SOURCE:
			case k_EXTRACT_HIGH_DEPTH_FROM_SOURCE:
			break;
		
			case k_EXTRACT_HIGH_DEPTH_FROM_COPY:
			case k_EXTRACT_HIGH_DEPTH_FROM_COPY_IF_NEEDED:
			case k_COPY_ALL_PIXELS_BUT_DITHER_TO_INDEXED_FROM_DEEP:

				if (remove_Background_Port_Initialized != SAFE_TRUE) {
					preferred_Background_Port_Depth = 32;
					preferred_Background_Port_Rect = ((**(current_Spin_Object->port_Info.spin_Object_Port_1.offscreen_Port.color_Port.portPixMap)).bounds);
					
					if (Create_Clut_For_Direct_Mapping(preferred_Background_Port_Depth, &(remove_Background_Port.color_Table_Handle)) != SAFE_TRUE) {
					 	goto Error_Handler;
					}
					
					if (Create_Off_Screen_Port(&preferred_Background_Port_Rect
												, preferred_Background_Port_Depth
												, remove_Background_Port.color_Table_Handle
												, (CGrafPtr)&(remove_Background_Port.offscreen_Port.color_Port)
												, &(remove_Background_Port.offscreen_GDevice_Hdl)
												, &(remove_Background_Port.image_Handle)) != noErr) {
					 	DisposeHandle((Handle)remove_Background_Port.color_Table_Handle);
					 	if (MemError() != noErr) {
							/*  */
							/* Ignore */
							/*  */
					 	}
					 	
						Perform_OK_Dialog("\pNote: Only a 16 bit color mask screen is being used instead of a 24 bit screen (low RAM). Should be OK.");

						/*  */
						/* Try a second time, this time with 16 bit color */
						/*  */
						preferred_Background_Port_Depth = 16;											
						if (Create_Clut_For_Direct_Mapping(preferred_Background_Port_Depth, &(remove_Background_Port.color_Table_Handle)) != SAFE_TRUE) {
						 	goto Error_Handler;
						}
						if (Create_Off_Screen_Port(&preferred_Background_Port_Rect
													, preferred_Background_Port_Depth
													, remove_Background_Port.color_Table_Handle
													, (CGrafPtr)&(remove_Background_Port.offscreen_Port.color_Port)
													, &(remove_Background_Port.offscreen_GDevice_Hdl)
													, &(remove_Background_Port.image_Handle)) != noErr) {
						 	DisposeHandle((Handle)remove_Background_Port.color_Table_Handle);
						 	if (MemError() != noErr) {
								/*  */
								/* Ignore */
								/*  */
						 	}
							goto Error_Handler;
						}
						
					} /* if (Create_Off_Screen_Port(&preferred_Background_Port_Rect ... */
					
					remove_Background_Port.initialized =  SAFE_TRUE; 
					remove_Background_Port.color_Type = k_COLOR_QUICKDRAW;
					Lock_Video_Memory_For_Lazy_But_Safe_Use(&remove_Background_Port);
					remove_Background_Port_Initialized = SAFE_TRUE;
					
				} /* if (remove_Background_Port_Initialized != SAFE_TRUE) ... */
			break;
			
		
			case k_DO_NO_EXTRACTION:
			default:
				; /* NULL */
				
		} /* switch(import_Background_Extraction) */
		
		/* ====== ====== ====== ====== ====== ====== ====== ====== ====== */
		/* ====== ====== ====== ====== ====== ====== ====== ====== ====== */
		
		
		/*  */
		/* This is where a display of available art and description of */
		/* possibility that multiple files and folders could be imported */
		/* a hierarchy is used, but before this can occur we should examine */
		/* all the folders first */
		/* First check if folder name of selected folder is similar to others at higher level */
		/* strip suffix and use as template. Allowable deviations are *./
		/* (e01, m01) or (m01), or (nothing) or (v01, m01) */
		/*  */
				
		/* ====== ====== ====== ====== ====== ====== ====== ====== ====== */
		/* ====== ====== ====== ====== ====== ====== ====== ====== ====== */
		
		/*  */
		/* Set "automatic_Imporation_Desired" */
		/* because it is a knockdown flag from here on down. */
		/*  */
		
		if (automatic_Imporation_Desired == TRUE) {
		
			if (Ask_User_If_Auto_Search(&near_Neighbor_File
											, (CARDINAL_32)total_Images_In_Each_Directory
											, &automatic_Imporation_Desired
											, &automatic_Imporation_Functioning) != SAFE_TRUE) {
				automatic_Imporation_Desired = FALSE;
				automatic_Imporation_Functioning = FALSE;
			}
			else {
			
				if (Create_Pathname_For_Directory(near_Neighbor_File.parID
										, near_Neighbor_File.vRefNum
										, 3
										, &current_Full_Path_Name[0]) != SAFE_TRUE) {
										
					/*  */
					/* Ignore errors, there should not be any */
					/*  */
					current_Full_Path_Name[0] = 0;
				}
			}
		}

		/* ====== ====== ====== ====== ====== ====== ====== ====== ====== */
		/* ====== ====== ====== ====== ====== ====== ====== ====== ====== */

		finished_Importing = TRUE;
		temp_Vertical_Value = 0;
		temp_Mouth_Value = 0;
		
		do { /* while (finished_Importing != TRUE) */
		
			abort_This_Round_And_Allow_Reselect = SAFE_FALSE;

			temp_Mouth_Value = temp_Mouth_Value + 1;
			
			if ((automatic_Imporation_Desired == TRUE) 
				&& (automatic_Imporation_Functioning == TRUE)){
				
				/*  */
				/* NULL */
				/*  */
			}
			else {
				if (Get_Long_Dialog("\pPlease specify sound activation level, for this file (1-50). '1' is closed mouth/silence."
										, &temp_Mouth_Value
										, 1
										, 50) != TRUE) {
					abort_This_Round_And_Allow_Reselect = SAFE_TRUE;
				}
			}
			
			if (abort_This_Round_And_Allow_Reselect == SAFE_TRUE) {
				/*  */
				/* NULL */
				/*  */
			}
			else {
				intensity = temp_Mouth_Value -1;
				
				horizontal_Angle = 0;
				vertical_Angle = 0;
				roll_Angle = 0;
				
				/*  */
				/* Ask for vertical elevation file level */
				/*  */
				
				if ((automatic_Imporation_Desired == TRUE) 
					&& (automatic_Imporation_Functioning == TRUE)){
					
					/*  */
					/* NULL */
					/*  */
				}
				else {
					temp_Vertical_Value = temp_Vertical_Value + 1;
					if (Get_Long_Dialog("\pPlease specify elevation, in units of 5 degrees, for this file (1-72). '1' is lowest angle view."
											, &temp_Vertical_Value
											, 1
											, 72) != TRUE) {
						abort_This_Round_And_Allow_Reselect = SAFE_TRUE;
					}
					temp_Vertical_Value = temp_Vertical_Value - 1;
				}
								
				vertical_Angle = temp_Vertical_Value;
				vertical_Angle = vertical_Angle * 5UL;
				
				Animate_Cursor(STANDARD_ACUR_ID, DO_NOT_USE_VBL);
				
				/*  */
				/* Now validate that the following are initialized for imminent use */
				/*  */
			}
				
				
			 if (abort_This_Round_And_Allow_Reselect != SAFE_TRUE) {
			 
				for(index = 0; (index < total_Images_To_Load); index++ ) {
					/*  */
					/* params_For_Convert_Spin_Object->desired_Resource_Index is ignored for safety */
					/* in case gaps exist in sequence */
					/*  */
					source_Pict_Id = 1;
					source_Pict_Id += index;
		
					horizontal_Angle = horizontal_Angle + 1;
					
					Convert_Long_To_String((total_Images_To_Load - index), temp_String_2, NO_LIMIT);
					Concat_Many(status_String, 3, message_String, temp_String_2, temp_String);
					
					if ((automatic_Imporation_Desired == TRUE) 
						&& (automatic_Imporation_Functioning == TRUE)){
						
						Concat_String(status_String, "\p (within ");
						Concat_String(status_String, &current_Full_Path_Name[0]);
						Concat_String(status_String, "\p )");
					}
					
					Display_Message_Low(status_String);
					
					
					if (file_Is_Pict_File != SAFE_TRUE) {
					
						source_Pict_Hdl = nil;
						source_Pict_Hdl = (PicHandle)(Get1IndResource('PICT' , source_Pict_Id));
						if ((ResError() != noErr) || (source_Pict_Hdl == nil)) {
							Stop_Animating_Cursor();
							Dispose_Message();
			
							goto Error_Handler;
						}
					
						source_Bit_Rect = (*source_Pict_Hdl)->picFrame;
						source_Bit_Width = source_Bit_Rect.right - source_Bit_Rect.left;
						source_Bit_Height = source_Bit_Rect.bottom - source_Bit_Rect.top;
					
						GetPort(&saved_Port);
						saved_Device = GetGDevice();
						
						Set_View_Port((GrafPtr)&(offscreen_Ptr->offscreen_Port.color_Port));
						Set_View_Device(offscreen_Ptr->offscreen_GDevice_Hdl);
						MoveTo(0,0);
						
						/*  */
						/* For safety to clear out the memory */
						/*  */
						EraseRect(&offscreen_Rect); 
						
						/*  */
						/* Only needed really for CopyBits calls but why not be certain. */
						/*  */
						ForeColor(blackColor);
						BackColor(whiteColor);
						
						SetRect(&source_Bit_Rect, 0, 0, source_Bit_Width, source_Bit_Height);
						DrawPicture(source_Pict_Hdl, &source_Bit_Rect);
						
						Set_View_Port(saved_Port);
						Set_View_Device(saved_Device);
						
						/*  */
						/* Release resource */
						/*  */
						if (source_Pict_Hdl != nil) {
							ReleaseResource((Handle)source_Pict_Hdl);
						}
					}
					else {
					
						if (Get_PICT_File_Neighbor_File_Spec(&near_Neighbor_File
													, (index + 1UL)
													, &located_Neighbor_File) != SAFE_TRUE) {
													
							Stop_Animating_Cursor();
							Dispose_Message();
			
							Display_Serious_Error_Message(unit_Name, code_Name, "\pGet_PICT_File_Neighbor_File_Spec failed", "\pDisk Space, programming error?");
							goto Error_Handler;
						}
					
						if (Guarantee_File_Is_PICT_Type(located_Neighbor_File.vRefNum
											, located_Neighbor_File.parID
											, &(located_Neighbor_File.name[0])
											, &source_Bit_Width
											, &source_Bit_Height) == SAFE_TRUE) {
											
							
							GetPort(&saved_Port);
							saved_Device = GetGDevice();
							
													
							
							if (Open_Well_Defined_File_Using_Buffer(DATA_FORK
															, &(located_Neighbor_File.name[0])
															, 0
															, (located_Neighbor_File.vRefNum)
															, (located_Neighbor_File.parID)
															, &source_Pict_File_Ref
															, (1024L)
															, k_PICT_FILE_DRAWING_BUFFER_SIZE
															, k_PICT_FILE_DRAWING_BUFFER_SIZE
															, &source_File_Size
															, SAFE_TRUE
															, &os_Error) != TRUE) {
		
								Stop_Animating_Cursor();
								Dispose_Message();
			
								Perform_OK_Dialog("\pProblem with Open_Well_Defined_File_Using_Buffer (1)");
								goto Error_Handler;
								if (os_Error == noErr) {
									os_Error = -1;
								}
							}
							
							if ((os_Error == noErr) && (source_File_Size != 0)){
							
								file_Is_Open = SAFE_TRUE;
								
								temp_Rect = (**(offscreen_Ptr->offscreen_Port.color_Port.portPixMap)).bounds;
								if (((temp_Rect.top > temp_Rect.bottom) || (temp_Rect.top != 0)) || (temp_Rect.right > 2400)){
									Display_Serious_Error_Message(unit_Name
																	, code_Name
																	, "\pProblem with port for compression. (0) (bounds abnormal! mem moved?)"
																	, "\pCorrupted?");
									goto Error_Handler;
								}
								
								
								/* ====== ====== ====== ====== ====== ====== ====== ====== ====== */
								/* ====== ====== ====== ====== ====== ====== ====== ====== ====== */
								/* ====== ====== ====== ====== ====== ====== ====== ====== ====== */
								/* HACK HACK HACK : This is the temp area where backgrounds are removed */
					
								/*  */
								/* Paranoia clearing only */
								/*  */
								row_Raw_Data_Hdl = nil;
								skip_List_Data_Hdl = nil;
								skip_List_Master_List_Hdl = nil;
								scan_Line_Bounds_Hdl = nil;

								switch(import_Background_Extraction) {
															
									case k_EXTRACT_LOW_DEPTH_FROM_SOURCE:
									case k_EXTRACT_HIGH_DEPTH_FROM_SOURCE:
										
										temp_Value_32 = (*(offscreen_Ptr->offscreen_Port.color_Port.portPixMap))->pixelSize;
										if ((temp_Value_32 < 16) || (temp_Value_32 > 48)) {
											Perform_OK_Dialog("\pDEBUG MESSAGE: Stopping PICT import. Bad offscreen pointer");
											goto Error_Handler_For_Draw_Pict_Import;
										}
										
										Set_View_Port((GrafPtr)&(offscreen_Ptr->offscreen_Port.color_Port));
										Set_View_Device(offscreen_Ptr->offscreen_GDevice_Hdl);
										MoveTo(0,0);
										
										/*  */
										/* Only needed really for CopyBits calls but why not be certain. */
										/*  */
										ForeColor(blackColor);
										BackColor(whiteColor);
										
										/*  */
										/* For safety to clear out the memory */
										/*  */
										EraseRect(&offscreen_Rect); 
										
										SetRect(&source_Bit_Rect, 0, 0, source_Bit_Width, source_Bit_Height);
										
										os_Error = Draw_File_Picture(source_Pict_File_Ref, &source_Bit_Rect);
										if (os_Error != noErr) {
											goto Error_Handler_For_Draw_Pict_Import;
										}
										/* ====== ====== ====== ====== ====== ====== ====== ====== ====== */
																					
										/*  */
										/* Release File */
										/*  */
										if (Close_File_With_Buffer(source_Pict_File_Ref
																	, FLUSH_BUFFER
																	, &os_Error) != TRUE) {
											/*  */
											/* Hmmm. Not expected. Ignore anyway. */
											/*  */
											Stop_Animating_Cursor();
											Dispose_Message();
					
											Perform_OK_Dialog("\pProblem with Close_File_With_Buffer (1)");
											goto Error_Handler;
										}
										source_Pict_File_Ref = 0;
										file_Is_Open = SAFE_FALSE;
										/* ====== ====== ====== ====== ====== ====== ====== ====== ====== */
									break;
								
									case k_EXTRACT_HIGH_DEPTH_FROM_COPY:
									case k_EXTRACT_HIGH_DEPTH_FROM_COPY_IF_NEEDED:
									
										Set_View_Port((GrafPtr)&(remove_Background_Port.offscreen_Port.color_Port));
										Set_View_Device(remove_Background_Port.offscreen_GDevice_Hdl);
										MoveTo(0,0);
										
										/*  */
										/* Only needed really for CopyBits calls but why not be certain. */
										/*  */
										ForeColor(blackColor);
										BackColor(whiteColor);
										
										/*  */
										/* For safety to clear out the memory */
										/*  */
										EraseRect(&offscreen_Rect); 
										
										SetRect(&source_Bit_Rect, 0, 0, source_Bit_Width, source_Bit_Height);
										
										os_Error = Draw_File_Picture(source_Pict_File_Ref, &source_Bit_Rect);
										if (os_Error != noErr) {
											goto Error_Handler_For_Draw_Pict_Import;
										}
										/* ====== ====== ====== ====== ====== ====== ====== ====== ====== */
										
												
										/*  */
										/* Release File */
										/*  */
										if (Close_File_With_Buffer(source_Pict_File_Ref
																	, FLUSH_BUFFER
																	, &os_Error) != TRUE) {
											/*  */
											/* Hmmm. Not expected. Ignore anyway. */
											/*  */
											Stop_Animating_Cursor();
											Dispose_Message();
					
											Perform_OK_Dialog("\pProblem with Close_File_With_Buffer (1)");
											goto Error_Handler;
										}
										source_Pict_File_Ref = 0;
										file_Is_Open = SAFE_FALSE;
										/* ====== ====== ====== ====== ====== ====== ====== ====== ====== */
										
	
									break; /* k_EXTRACT_HIGH_DEPTH_FROM_COPY */
	
									case k_COPY_ALL_PIXELS_BUT_DITHER_TO_INDEXED_FROM_DEEP:
										
										Set_View_Port((GrafPtr)&(remove_Background_Port.offscreen_Port.color_Port));
										Set_View_Device(remove_Background_Port.offscreen_GDevice_Hdl);
										MoveTo(0,0);
										
										/*  */
										/* Only needed really for CopyBits calls but why not be certain. */
										/*  */
										ForeColor(blackColor);
										BackColor(whiteColor);
										
										/*  */
										/* For safety to clear out the memory */
										/*  */
										EraseRect(&offscreen_Rect); 
										
										SetRect(&source_Bit_Rect, 0, 0, source_Bit_Width, source_Bit_Height);
										
										os_Error = Draw_File_Picture(source_Pict_File_Ref, &source_Bit_Rect);
										if (os_Error != noErr) {
											goto Error_Handler_For_Draw_Pict_Import;
										}
										/* ====== ====== ====== ====== ====== ====== ====== ====== ====== */
																					
										/*  */
										/* Release File */
										/*  */
										if (Close_File_With_Buffer(source_Pict_File_Ref
																	, FLUSH_BUFFER
																	, &os_Error) != TRUE) {
											/*  */
											/* Hmmm. Not expected. Ignore anyway. */
											/*  */
											Stop_Animating_Cursor();
											Dispose_Message();
					
											Perform_OK_Dialog("\pProblem with Close_File_With_Buffer (1)");
											goto Error_Handler;
										}
										source_Pict_File_Ref = 0;
										file_Is_Open = SAFE_FALSE;
										/* ====== ====== ====== ====== ====== ====== ====== ====== ====== */
	
									break;
									
								
									case k_DO_NO_EXTRACTION:
									default:
										Set_View_Port((GrafPtr)&(offscreen_Ptr->offscreen_Port.color_Port));
										Set_View_Device(offscreen_Ptr->offscreen_GDevice_Hdl);
										MoveTo(0,0);
										
										/*  */
										/* Only needed really for CopyBits calls but why not be certain. */
										/*  */
										ForeColor(blackColor);
										BackColor(whiteColor);
										
										/*  */
										/* For safety to clear out the memory */
										/*  */
										EraseRect(&offscreen_Rect); 
										
										SetRect(&source_Bit_Rect, 0, 0, source_Bit_Width, source_Bit_Height);
										
										os_Error = Draw_File_Picture(source_Pict_File_Ref, &source_Bit_Rect);
										if (os_Error != noErr) {
											goto Error_Handler_For_Draw_Pict_Import;
										}
										/* ====== ====== ====== ====== ====== ====== ====== ====== ====== */
																					
										/*  */
										/* Release File */
										/*  */
										if (Close_File_With_Buffer(source_Pict_File_Ref
																	, FLUSH_BUFFER
																	, &os_Error) != TRUE) {
											/*  */
											/* Hmmm. Not expected. Ignore anyway. */
											/*  */
											Stop_Animating_Cursor();
											Dispose_Message();
					
											Perform_OK_Dialog("\pProblem with Close_File_With_Buffer (1)");
											goto Error_Handler;
										}
										source_Pict_File_Ref = 0;
										file_Is_Open = SAFE_FALSE;
										/* ====== ====== ====== ====== ====== ====== ====== ====== ====== */
										
								} /* switch(import_Background_Extraction) */
								
								
								
								/*  */
								/* Data is in proper buffer and is partially processed. */
								/* Finish it */
								/*  */
								switch(import_Background_Extraction) {
															
									case k_EXTRACT_LOW_DEPTH_FROM_SOURCE:
									case k_EXTRACT_HIGH_DEPTH_FROM_SOURCE:
									
										/* ====== ====== ====== ====== ====== ====== ====== ====== ====== */
									break;
								
									case k_EXTRACT_HIGH_DEPTH_FROM_COPY:
									case k_EXTRACT_HIGH_DEPTH_FROM_COPY_IF_NEEDED:
									
								/* ===
										DebugStr("\pIGNORE: Check for corruption (2)");
									=== */
										
										Set_View_Port((GrafPtr)&(offscreen_Ptr->offscreen_Port.color_Port));
										Set_View_Device(offscreen_Ptr->offscreen_GDevice_Hdl);
										/*  */
										/* Only needed really for CopyBits calls but why not be certain. */
										/*  */
										ForeColor(blackColor);
										BackColor(whiteColor);
										
										/*  */
										/* CopyBits moves memory but these unlocked handles are */
										/* safe to pass I presume */
										/*  */
										CopyBits((const BitMap *)*(remove_Background_Port.offscreen_Port.color_Port.portPixMap)
												, (const BitMap *)*(offscreen_Ptr->offscreen_Port.color_Port.portPixMap)
												, &offscreen_Rect
												, &offscreen_Rect
												, (srcCopy | ditherCopy)
												, nil);

										/*  */
										/* Temp restore in case error messages */
										/*  */
							/* ===			
										Set_View_Port(saved_Port);
										Set_View_Device(saved_Device);
							=== */		
										Set_View_Port((GrafPtr)&(remove_Background_Port.offscreen_Port.color_Port));
										Set_View_Device(remove_Background_Port.offscreen_GDevice_Hdl);
										
							
										/*  */
										/* This calculates the mask of no background pixels */
										/*  */
										simple_Image_Handle = nil;
										if (Create_Bit_Mask_For_Port_Image((GrafPtr)&(remove_Background_Port.offscreen_Port.color_Port)
																		, &offscreen_Rect
																		, &simple_Offscreen_Graph_Port
																		, &simple_Image_Handle) != SAFE_TRUE) {
																		
											if (simple_Image_Handle != nil) {
											
												/*  */
												/* Destroy Bit map, it was created. */
												/*  */
												if (Destroy_Simple_BW_Bit_Map(SAFE_FALSE
																					, &simple_Image_Handle
																					, &simple_Offscreen_Graph_Port
																					, nil) != SAFE_TRUE) {
													simple_Image_Handle = nil;
	
													/*  */
													/* Ignore err, exiting soon anyway */
													/*  */
													Perform_OK_Dialog("\pDestroy_Simple_BW_Bit_Map");
													goto Error_Handler;
												}
												else {
													simple_Image_Handle = nil;
												}
											} /* (simple_Image_Handle != nil) */


											Set_View_Device(saved_Device);
											Set_View_Port(saved_Port);
											Display_Serious_Error_Message(unit_Name
																			, code_Name
																			, "\pProblem calling Create_Bit_Mask_For_Port_Image"
																			, "\p(Not enough memory in Application heap?)");
											goto Error_Handler_For_Compression;
	
										}
										else {																										
										
											Set_View_Device(offscreen_Ptr->offscreen_GDevice_Hdl);
											Set_View_Port((GrafPtr)&(offscreen_Ptr->offscreen_Port.color_Port));
											PenNormal();
											ForeColor(blackColor);
											BackColor(whiteColor);
	
											MoveTo(0,0);
											
	
											/*  */
											/* Warning : First step before filling in a chroma key color */
											/* is to strip out chroma match */
											/* either "k_USE_DIRECT_COLOR_AS_INPUT" or */
											/* "k_USE_INDEXED_COLOR_AS_INPUT" */
											/*  */
											if (Filter_Color_From_Image_Completely(offscreen_Ptr
																						, 0
																						, (g_Chroma_Index_Optimized_Long & 0x00FF)
																						, k_USE_INDEXED_COLOR_AS_INPUT
																						, &offscreen_Rect) != SAFE_TRUE) {
												/*  */
												/* Ignore error for now. Though errors are possible if low memory. */
												/*  */
											}
											
											Index2Color((g_Chroma_Index_Optimized_Long & 0x00FF), &plain_Flood_Fill_Color);

											RGBForeColor(&plain_Flood_Fill_Color);
											CopyBits((BitMapPtr)&(simple_Offscreen_Graph_Port.portBits)
														, (const BitMap *)*(offscreen_Ptr->offscreen_Port.color_Port.portPixMap)	
														, &offscreen_Rect
														, &offscreen_Rect
														, notSrcOr
														, nil);
	
											RGBForeColor(&g_Shell.library.color_Black);
	
	
											Set_View_Device(saved_Device);
											Set_View_Port(saved_Port);
											Convert_Long_To_String((total_Images_To_Load - index), temp_String_2, NO_LIMIT);
											Concat_Many(status_String, 3, message_Compress_String, temp_String_2, temp_String);
											Display_Message_Low(status_String);
											
											/*  */
											/* Now compress the bitmap of our choice with the mask of our */
											/* choice. Apples mask was used to prove source of corruption if */
											/* any. */
											/*  */
											
											row_Raw_Data_Hdl = nil;
											skip_List_Data_Hdl = nil;
											skip_List_Master_List_Hdl = nil;
											scan_Line_Bounds_Hdl = nil;
											
											
											temp_Rect = (**(offscreen_Ptr->offscreen_Port.color_Port.portPixMap)).bounds;
											if (((temp_Rect.top > temp_Rect.bottom) || (temp_Rect.top != 0)) || (temp_Rect.right > 2400)){
												Display_Serious_Error_Message(unit_Name
																				, code_Name
																				, "\pProblem with port for compression. (bounds abnormal!)"
																				, "\pCorrupted?");
												goto Error_Handler;
											}
	
											Set_View_Device(offscreen_Ptr->offscreen_GDevice_Hdl);
											Set_View_Port((GrafPtr)&(offscreen_Ptr->offscreen_Port.color_Port));
	
											debug_Old_Address = (CARDINAL_32)*(offscreen_Ptr->offscreen_Port.color_Port.portPixMap);
											
											/*  */
											/* Gotta lock these unlocked handles during routine call */
											/* similar to CopyBits problem but I know I will move memory. */
											/*  */
											temp_Result = Begin_Use_Of_Handle((Handle)(offscreen_Ptr->offscreen_Port.color_Port.portPixMap)
																, SAFE_FALSE
																, &original_Handle_State_1);
											if (temp_Result != SAFE_TRUE) {
												goto Error_Handler;
											}

											if (Compact_Region_And_Bitmap((const BitMap *)*(offscreen_Ptr->offscreen_Port.color_Port.portPixMap)
																					, &offscreen_Rect
																					, 0
																					, 0
																					, SAFE_FALSE
																					, nil
																					, (BitMapPtr)&(simple_Offscreen_Graph_Port.portBits)
																					, &row_Raw_Data_Hdl
																					, &skip_List_Data_Hdl
																					, &skip_List_Master_List_Hdl
																					, &scan_Line_Bounds_Hdl) != SAFE_TRUE) {
																					
												goto Error_Handler_For_Compression;
											}
											
											temp_Result = End_Use_Of_Handle((Handle)(offscreen_Ptr->offscreen_Port.color_Port.portPixMap)
																	, original_Handle_State_1);
											if (temp_Result != SAFE_TRUE) {
												goto Error_Handler;
											}
	
											
											debug_New_Address = (CARDINAL_32)*(offscreen_Ptr->offscreen_Port.color_Port.portPixMap);
											if (debug_New_Address != debug_Old_Address) {
												Display_Serious_Error_Message(unit_Name
																				, code_Name
																				, "\pCaution: Handles for pixmaps are not locked down!"
																				, "\pPoor programming may result in serious problems.");
												goto Error_Handler;
											}
											
											temp_Rect = (**(offscreen_Ptr->offscreen_Port.color_Port.portPixMap)).bounds;
											if (((temp_Rect.top > temp_Rect.bottom) || (temp_Rect.top != 0)) || (temp_Rect.right > 2400)){
												Display_Serious_Error_Message(unit_Name
																				, code_Name
																				, "\pProblem with port for compression. 2, (bounds abnormal!)"
																				, "\pCorrupted?");
												goto Error_Handler;
											}
											
											if (skip_List_Master_List_Hdl == nil) {
												Set_View_Device(saved_Device);
												Set_View_Port(saved_Port);
												Display_Serious_Error_Message(unit_Name
																				, code_Name
																				, "\pProblem calling Compact_Region_And_Bitmap"
																				, "\p(Not enough memory in Application heap? : Nil skip_List_Master_List_Hdl)");
												goto Error_Handler_For_Compression;
											}
	
										}
										
										/* ====== ====== ====== ====== ====== ====== ====== ====== ====== */
									break; /* k_EXTRACT_HIGH_DEPTH_FROM_COPY */
	
									case k_COPY_ALL_PIXELS_BUT_DITHER_TO_INDEXED_FROM_DEEP:
										Set_View_Port((GrafPtr)&(offscreen_Ptr->offscreen_Port.color_Port));
										Set_View_Device(offscreen_Ptr->offscreen_GDevice_Hdl);
										/*  */
										/* Only needed really for CopyBits calls but why not be certain. */
										/*  */
										ForeColor(blackColor);
										BackColor(whiteColor);
										
										CopyBits((const BitMap *)*(remove_Background_Port.offscreen_Port.color_Port.portPixMap)
												, (const BitMap *)*(offscreen_Ptr->offscreen_Port.color_Port.portPixMap)
												, &offscreen_Rect
												, &offscreen_Rect
												, (srcCopy | ditherCopy)
												, nil);

									break; /* k_COPY_ALL_PIXELS_BUT_DITHER_TO_INDEXED_FROM_DEEP */

									case k_DO_NO_EXTRACTION:
									default:
										; /* NULL */
								} /* switch(import_Background_Extraction) */
								
								
							}
							
							/*  */
							/* Vital to set back!!! */
							/*  */
							Set_View_Device(saved_Device);
							Set_View_Port(saved_Port);
							
						}
						else {
							Stop_Animating_Cursor();
							Dispose_Message();
			
							Display_Serious_Error_Message(unit_Name, code_Name, "\pGuarantee_File_Is_PICT_Type failed", "\pprogramming error?");
							goto Error_Handler;
						
						}
		
					
					
					
					} /* NOT if (file_Is_Pict_File != SAFE_TRUE) */
					
					
					/*  */
					/* Now iterate and load pict resources into already existant port. */
					/* and then write each one out using append feature */
					/*  */		
	
					/*  */
					/* Destroy Bit map, it was created. */
					/*  */
					if (simple_Image_Handle != nil) {
						if (Destroy_Simple_BW_Bit_Map(SAFE_FALSE
															, &simple_Image_Handle
															, &simple_Offscreen_Graph_Port
															, nil) != SAFE_TRUE) {
							simple_Image_Handle = nil;
				
							/*  */
							/* Ignore err, exiting soon anyway */
							/*  */
							Perform_OK_Dialog("\pDestroy_Simple_BW_Bit_Map");
							goto Error_Handler;
						}
						else {
							simple_Image_Handle = nil;
						}
					}
					
					/*  */
					/* Implicitly trust setting of "skip_List_Master_List_Hdl" to */
					/* determine if this frame is saved compressed */
					/*  */
					if (skip_List_Master_List_Hdl == nil) {
					
						if ((import_Background_Extraction == k_DO_NO_EXTRACTION)
							|| (import_Background_Extraction == k_COPY_ALL_PIXELS_BUT_DITHER_TO_INDEXED_FROM_DEEP)) {
							compression_Type = k_NO_COMPRESSION;
						}
						else {
							Stop_Animating_Cursor();
							Dispose_Message();
			
							Display_Serious_Error_Message(unit_Name, code_Name, "\pProgramming flaw. (skip_List_Master_List_Hdl not set)", "\plow memory? Try again");
							goto Error_Handler;
						}
						
					}
					else {
						compression_Type = k_MERKLE_TIME_COMPRESSION;
					}
	
	
					
					if (Append_Spin_Object_To_File(&(params_For_Convert_Spin_Object->output_Converted_File)
										, offscreen_Ptr
										, current_Spin_Object->spin_Info
										, intensity
										, horizontal_Angle
										, vertical_Angle
										, roll_Angle
										, compression_Type
										, row_Raw_Data_Hdl
										, skip_List_Data_Hdl
										, skip_List_Master_List_Hdl
										, scan_Line_Bounds_Hdl) != SAFE_TRUE) {
										
						Stop_Animating_Cursor();
						Dispose_Message();
		
						Display_Serious_Error_Message(unit_Name, code_Name, "\pAppend_Spin_Object_To_File failed", "\pDisk Space, programming error?");
						goto Error_Handler;
					}
					
					if (row_Raw_Data_Hdl != nil) {
						DisposeHandle(row_Raw_Data_Hdl);
						if (MemError() != noErr) {
							goto Error_Handler;
						}
					}
					
					if (skip_List_Data_Hdl != nil) {
						DisposeHandle(skip_List_Data_Hdl);
						if (MemError() != noErr) {
							goto Error_Handler;
						}
					}
					
					if (skip_List_Master_List_Hdl != nil) {
						DisposeHandle(skip_List_Master_List_Hdl);
						if (MemError() != noErr) {
							goto Error_Handler;
						}
					}
					
					if (scan_Line_Bounds_Hdl != nil) {
						DisposeHandle(scan_Line_Bounds_Hdl);
						if (MemError() != noErr) {
							goto Error_Handler;
						}
					}
					
					if (Check_Keyboard_For_Key(k_KEYCODE_ASCII_PERIOD) == SAFE_TRUE) {
						if (Check_Keyboard_For_Key(k_KEYCODE_COMMAND) == SAFE_TRUE) {
							Stop_Animating_Cursor();
							Dispose_Message();
							Get_NS_String(k_SPIN_STRINGS, k_ERROR_USER_CANCELLED_IMPORT, message_String);
							Perform_OK_Dialog(message_String);
							
							index = total_Images_To_Load;
						}
					}
					
										
		/* ====== ====== ====== ====== ====== ====== ====== ====== ====== */

#if !defined(k_FORCE_FALSE_UPDATE_WHILE_IMPORTING)
#error "UNDEFINED PARAMETER!"
#endif
#if  (k_FORCE_FALSE_UPDATE_WHILE_IMPORTING == 1)
					
					GetPort(&temp_Saved_Port);
					temp_Saved_Device = GetGDevice();
			
					Stop_Animating_Cursor();
					Dispose_Message();

					Get_Max_Ranges(current_Spin_Object->spin_Info
										, &current_Spin_Object->max_Intensity
										, &current_Spin_Object->max_Horizontal_Angle
										, &current_Spin_Object->max_Vertical_Angle
										, &current_Spin_Object->max_Roll_Angle);
										
					Get_Periodicity_Ranges(current_Spin_Object->spin_Info
										, current_Spin_Object->max_Intensity
										, current_Spin_Object->max_Horizontal_Angle
										, current_Spin_Object->max_Vertical_Angle
										, &current_Spin_Object->horizontal_Angle_Periodicity
										, &current_Spin_Object->vertical_Angle_Periodicity
										, &current_Spin_Object->roll_Angle_Periodicity);
					
					current_Spin_Object->minimum_Intensity_Representing_Idle = 0;
			
					/*  */
					/* Erase foreground (primary) composite background with */
					/* chroma value. ALSO erase damned scratch area. */
					/*  */
					handle_Size = GetHandleSize(current_Spin_Object->port_Info.spin_Object_Port_1.image_Handle);
					if (handle_Size != 0) {
						Fill_Large_Buffer((Ptr)*(current_Spin_Object->port_Info.spin_Object_Port_1.image_Handle)
											, handle_Size, g_Chroma_Index_Optimized_Long);
					}
			
					current_Spin_Object->port_Info.frame_Needing_Erasure.valid_Information = SAFE_FALSE;
					if (Draw_Spin_Object_From_Closed_File(&(current_Spin_Object->spin_Object_Location)
										, &(current_Spin_Object->port_Info.spin_Object_Port_1)
										, nil
										, current_Spin_Object->spin_Info
										, &(current_Spin_Object->port_Info.frame_Needing_Erasure)
										, intensity
										, horizontal_Angle
										, vertical_Angle
										, roll_Angle
										, current_Spin_Object->port_Info.offset_Offscreen_Pixel_X
										, current_Spin_Object->port_Info.offset_Offscreen_Pixel_Y) != SAFE_TRUE) {
						Display_Serious_Error_Message(unit_Name, code_Name, "\pDraw_Spin_Object_From_Closed_File failed (1)", "\pNeeds enhancement or buffers unready?");
						goto Error_Handler;
					}
					
					/*  */
					/* Disable sentinel first */
					/*  */
					g_Spin_File_Importation_Active = SAFE_FALSE;
					if (Redraw_Spinning_Window_Completely(TRUE, FALSE) != SAFE_TRUE) {
						/*  */
						/* Ignore, failures not critical*/
						/*  */
					}
					g_Spin_File_Importation_Active = SAFE_TRUE;
									
					SetPort(temp_Saved_Port);
					Set_View_Device(temp_Saved_Device);

		
#endif /* #if  (k_FORCE_FALSE_UPDATE_WHILE_IMPORTING == 1) */
		/* ====== ====== ====== ====== ====== ====== ====== ====== ====== */
		
					
				} /* for(index = 0; (index < total_Images_To_Load); index++ ) */
			} /* if (abort_This_Round_And_Allow_Reselect == SAFE_TRUE) */
			Stop_Animating_Cursor();
			Dispose_Message();
		
		
			finished_Importing = TRUE;

			if (file_Is_Pict_File == SAFE_TRUE) {
			
				if (abort_This_Round_And_Allow_Reselect == SAFE_TRUE) {
					temp_Boolean = Perform_Yes_No_Dialog("\pThat File Set was Canceled. Do you want to add another slice of 3D pictures from another directory?");
				}
				else {
								
					if (automatic_Imporation_Desired == TRUE) {
					
						near_Neighbor_File = located_Neighbor_File;
						if (Get_PICT_File_Neighbor_File_Spec_Next_Folder(&near_Neighbor_File
														, total_Images_In_Each_Directory
														, SAFE_FALSE
														, &hierachy_Depth_Above
														, &located_Neighbor_File) != SAFE_TRUE) {
							automatic_Imporation_Functioning = FALSE;			
						}
						
						if (automatic_Imporation_Functioning != TRUE) {
							temp_Boolean = Perform_Yes_No_Dialog("\pDo you want to add another slice of 3D pictures from another directory?");
						}
						else {
							near_Neighbor_File = located_Neighbor_File;
						}
					}
					
				}
			
				if ((temp_Boolean == TRUE) || (automatic_Imporation_Desired == TRUE)) {
					/*  */
					/* Get neighbor and validate and set counter and reloop */
					/* after getting new height */
					/*  */
					
					continue_With_File_Usage = FALSE;
					
					if ((automatic_Imporation_Desired == TRUE) 
							&& (automatic_Imporation_Functioning == TRUE)){
							
						/*  */
						/* Near neighbor file is already active */
						/*  */
						continue_With_File_Usage = TRUE;
						
						if (Create_Pathname_For_Directory(near_Neighbor_File.parID
												, near_Neighbor_File.vRefNum
												, 3
												, &current_Full_Path_Name[0]) != SAFE_TRUE) {
												
							/*  */
							/* Ignore errors, there should not be any */
							/*  */
							current_Full_Path_Name[0] = 0;
						}
						
						if (hierachy_Depth_Above == 1) {
							/*  */
							/* No change in height needed */
							/* No change in intensity needed */
							/*  */
						}
						else if (hierachy_Depth_Above == 2) {
						
							/*  */
							/* Need to change height */
							/*  */
							temp_Mouth_Value = 0;
							temp_Vertical_Value += 1;
						}
						else {
							/*  */
							/* Only 2 levels supported so far (X, Y, Activation)
							/*  */
							temp_Mouth_Value = 0;
							temp_Vertical_Value += 1;

							
							goto Error_Handler;
							
						}
					}
					else {
						near_Neighbor_File = params_For_Convert_Spin_Object->desired_File;
	
						if (Specify_PICT_File_To_Load(&near_Neighbor_File
																, &user_Abort) != SAFE_TRUE) {
																
							continue_With_File_Usage = FALSE;
							
						} /* if (Specify_PICT_File_To_Load(&near_Neighbor_File ... */
						else {
							continue_With_File_Usage = TRUE;
						}
					}
					
					if (continue_With_File_Usage == TRUE) {					
						
						if (Count_PICT_File_Neighbors(&(near_Neighbor_File), (CARDINAL_32 *)&total_Images_To_Load) != SAFE_TRUE) {
							Get_NS_String(k_DREAM_STRINGS, k_SORRY_WRONG_FILE_INDEX, message_String);
							Perform_OK_Dialog(message_String);
							finished_Importing = TRUE;
						}
						else {
						
							if (total_Images_To_Load < 2) {
								if (user_Warned_About_Lone_Files != TRUE) {
									if (total_Images_To_Load < 2) {
										Get_NS_String(k_DREAM_STRINGS, k_NUMBER_OF_FILES_SOLO_AND_NOT_PICTS, message_String);
										Perform_OK_Dialog(message_String);
									}
									user_Warned_About_Lone_Files = TRUE;
								}
								
								finished_Importing = FALSE;
							}
							else {
								finished_Importing = FALSE;
								
								if (total_Images_In_Each_Directory != total_Images_To_Load) {
									Get_NS_String(k_DREAM_STRINGS, k_NUMBER_OF_FILES_DOES_NOT_MATCH_NEIGHBORS, message_String);
									Perform_OK_Dialog(message_String);
								}
	
							}
						}
					}
					
				}
			} /* if (file_Is_Pict_File == SAFE_TRUE) */
			
					
		} while (finished_Importing != TRUE);
		
		
		/* ====== ====== ====== ====== ====== ====== ====== ====== ====== */
		/* ====== ====== ====== ====== ====== ====== ====== ====== ====== */
		/* ====== ====== ====== ====== ====== ====== ====== ====== ====== */
		/* ====== ====== ====== ====== ====== ====== ====== ====== ====== */


	} /* if (importation_Desired == TRUE) */

	Get_Max_Ranges(current_Spin_Object->spin_Info
						, &current_Spin_Object->max_Intensity
						, &current_Spin_Object->max_Horizontal_Angle
						, &current_Spin_Object->max_Vertical_Angle
						, &current_Spin_Object->max_Roll_Angle);
			
	Get_Periodicity_Ranges(current_Spin_Object->spin_Info
								, current_Spin_Object->max_Intensity
								, current_Spin_Object->max_Horizontal_Angle
								, current_Spin_Object->max_Vertical_Angle
								, &current_Spin_Object->horizontal_Angle_Periodicity
								, &current_Spin_Object->vertical_Angle_Periodicity
								, &current_Spin_Object->roll_Angle_Periodicity);

	current_Spin_Object->minimum_Intensity_Representing_Idle = 0;
	
	/*  */
	/* Erase foreground (primary) composite background with */
	/* chroma value. ALSO erase damned scratch area. */
	/*  */
	
	if (load_For_Isolated_Self_Sufficient_Drawing == SAFE_TRUE) {
		handle_Size = GetHandleSize(current_Spin_Object->port_Info.spin_Object_Port_1.image_Handle);
		if (handle_Size != 0) {
			Fill_Large_Buffer((Ptr)*(current_Spin_Object->port_Info.spin_Object_Port_1.image_Handle)
								, handle_Size
								, g_Chroma_Index_Optimized_Long);
		}
	}
	
	current_Spin_Object->port_Info.frame_Needing_Erasure.valid_Information = SAFE_FALSE;

			
	if (importation_Desired != TRUE) {
		if (g_Set_Spin_Window_Max_Monitor_Size == SAFE_TRUE) {
			/*  */
			/* Expand_Rect_To_Largest_Reasonable_Size may have */
			/* been called so readjust offsets of item */
			/* Move base of drawing to bottom center. Note that center */
			/* is not to break a 16 byte offset thus modulo 16 byte for horizontal */
			/*  */
			if (load_For_Isolated_Self_Sufficient_Drawing == SAFE_TRUE) {

				if (current_Spin_Object->port_Info.spin_Object_Port_1.initialized) {
					offscreen_Rect = ((**(current_Spin_Object->port_Info.spin_Object_Port_1.offscreen_Port.color_Port.portPixMap)).bounds);
					if ((*current_Spin_Object->spin_Info)->max_Bounds.right < offscreen_Rect.right) {
						current_Spin_Object->port_Info.offset_Offscreen_Pixel_X = (offscreen_Rect.right - (*current_Spin_Object->spin_Info)->max_Bounds.right);
						current_Spin_Object->port_Info.offset_Offscreen_Pixel_X = current_Spin_Object->port_Info.offset_Offscreen_Pixel_X >> 1;
						current_Spin_Object->port_Info.offset_Offscreen_Pixel_X = current_Spin_Object->port_Info.offset_Offscreen_Pixel_X & 0xFFFFFFF0;
					}
					
					if ((*current_Spin_Object->spin_Info)->max_Bounds.bottom < offscreen_Rect.bottom) {
						current_Spin_Object->port_Info.offset_Offscreen_Pixel_Y = (offscreen_Rect.bottom - (*current_Spin_Object->spin_Info)->max_Bounds.bottom);
					}
				}
			}
		}
	} /* if (importation_Desired == TRUE) */

	if (importation_Desired != TRUE) {
		/*  */
		/* 1995.01.28 JAM ENHANCEMENT, not vital */
		/*  */

		current_Spin_Object->port_Info.frame_Needing_Erasure.valid_Information = SAFE_FALSE;
		
		if (load_For_Isolated_Self_Sufficient_Drawing == SAFE_TRUE) {
			if (Draw_Spin_Object_From_Closed_File(&(current_Spin_Object->spin_Object_Location)
								, &(current_Spin_Object->port_Info.spin_Object_Port_1)
								, nil
								, current_Spin_Object->spin_Info
								, &(current_Spin_Object->port_Info.frame_Needing_Erasure)
								, (**(current_Spin_Object->spin_Info)).cell_Info[0].activation
								, (**(current_Spin_Object->spin_Info)).cell_Info[0].horizontal_Angle
								, (**(current_Spin_Object->spin_Info)).cell_Info[0].vertical_Angle
								, (**(current_Spin_Object->spin_Info)).cell_Info[0].roll_Angle
								, current_Spin_Object->port_Info.offset_Offscreen_Pixel_X
								, current_Spin_Object->port_Info.offset_Offscreen_Pixel_Y) != SAFE_TRUE) {
				Display_Serious_Error_Message(unit_Name, code_Name, "\pDraw_Spin_Object_From_Closed_File failed (1)", "\pNeeds enhancement or buffers unready?");
				goto Error_Handler;
			}
		}
	}


	function_Result = SAFE_TRUE;
	goto Normal_Function_Exit;
	
	
Error_Handler_For_Draw_Pict_Import:
	Set_View_Port(saved_Port);
	Set_View_Device(saved_Device);

	Stop_Animating_Cursor();
	Dispose_Message();

	Perform_OK_Dialog("\pNote: Draw_File_Picture failed");
	function_Result = SAFE_FALSE;
	goto Normal_Function_Exit;
	
	
Error_Handler_For_Compression:
	Set_View_Port(saved_Port);
	Set_View_Device(saved_Device);

	Stop_Animating_Cursor();
	Dispose_Message();

	Display_Serious_Error_Message(unit_Name, code_Name, "\pNote: Compression failed", "\pPerhaps insufficient memory.");
	function_Result = SAFE_FALSE;
	goto Normal_Function_Exit;


Error_Handler_With_Message:
	function_Result = SAFE_FALSE;
	Display_Serious_Error_Message(unit_Name, code_Name, note_String, "\p");
	goto Normal_Function_Exit;
	
Error_Handler:
	Display_Serious_Error_Message(unit_Name, code_Name, "\pInternal problem getting Spin_Object", "\p");
	function_Result = SAFE_FALSE;
		
Normal_Function_Exit:

	if (remove_Background_Port_Initialized == SAFE_TRUE) {
		 if (remove_Background_Port.initialized == SAFE_TRUE) {
		 	if (Close_Offscreen_Pict_Port(&(remove_Background_Port)) != SAFE_TRUE) {
				/*  */
				/* Ignore error, we are exiting anyway  */
				/*  */
		 	}
		 }
	}

	if (spin_Object_Header_Memory_In_Use == MEMORY_INITIALIZED) {
		ReleaseResource((Handle)spin_Object_Header_Hdl);
		if (ResError() != noErr) {
			/*  */
			/* Ignore error, we are exiting anyway  */
			/*  */
		}
		spin_Object_Header_Memory_In_Use = MEMORY_NOT_INITIALIZED;
	}
		
	if (resource_Sources_Swapped == SAFE_TRUE) {
		CloseResFile(input_File_Reference);
		if (ResError() != noErr) {

		}
		
		UseResFile(save_Ref);
		if (ResError() != noErr) {
			/*  */
			/* This should be recoverable from, enhance one day  */
			/*  */
		}
		resource_Sources_Swapped = SAFE_FALSE;
	}
	
	if (input_Data_File_Reference != 0) {
		os_Error = FSClose(input_Data_File_Reference);
		if ( os_Error != noErr) {
			Display_Serious_Error_Message(unit_Name, code_Name, "\pOS Error FSClose.", "\p");
		}
	}

	SetPort(original_Saved_Port);
	Set_View_Device(original_Saved_Device);
	
	g_Interactive_Motion_Disabled = original_Interactive_Motion;
	g_Spin_File_Importation_Active = SAFE_FALSE;

	return(function_Result);
	
} /* Safe_Boolean	Load_Or_Import_Spin_Object_File... */
/*=**************************************************************************=*/


/*=**************************************************************************=*/
/* END OF FILE: Spin_O_Rama_Load_Import.c */
