/* START OF FILE: Stage_View_Manager.c */


/*=**************************************************************************=*/
/* [---BEGIN---] FILE HEADER COMMENTS */
/**
 ** SCREEN VIEW = [Adobe Syntax Black-10, Tabs = 4]
 ** PRINTER VIEW = [Adobe Syntax Black-10, Tabs = 4]
 ** =======================================================================
 ** Written by James Merkle
 ** Stage_View_Manager.c - Backdrop Routines to support Poor Man's Video Phone so that cropping can be ignored.
 ** (C) Copyright 1994 James Merkle
 ** All rights reserved, except those wishing to "copy" moderate sections of code or files may do so, so long as
 ** such code is never used directly either in animation-related tools or multimedia tools that are  
 ** used in live performance or live broadcast or recorded mass market broadcast or film. Desktop usage not restricted.
 ** Conventional desktop game or entertainment not restricted.
 ** =======================================================================
 **
 **
 ** ============
 ** Revision History
 ** ============
 **
 **  1995.05.31		(James Merkle)
 **						Initial coding started from scratch
 **
 **
 **
 ** =================
 ** OTHER INFORMATION
 ** =================
 ** 
 ** This large offscreen area allows panning if needed within the frame. It also allows
 ** ability to jiggle main view if needed by a few pixels for "bomb explosion" on stage.
 ** The main purpose is to allow offscreen drawing of puppets without worrying about
 ** writing crop routines (I think it can only crop vertical directions in algorithm right now
 ** It will also aid in scrolling text layers and such. 
 ** 
 **
 **/
/* [----END----] FILE HEADER COMMENTS */
/*=**************************************************************************=*/


/*=**************************************************************************=*/
/* [---BEGIN---] #include AREA */


#include "Application_Limits.h"
#include "Dreaming_Stuff.h"
#include "Memory_Utilities.h"
#include "Shell_Assembly.h"
#include "Dreaming_Animation.h"
#include "Window_Environment.h"
#include "Core_Globals.h"
#include "Video_Backdrop_Background.h"
#include "Interface_Utilities.h"
#include "Error_Utilities.h"
#include "String_Utilities.h"
#include "Shell_Dialogs.h"
#include "Graphics_Primitives.h"

#include "Stage_View_Manager.h"

/* [----END----] #include AREA */
/*=**************************************************************************=*/


/*=**************************************************************************=*/
/* [---BEGIN---] #define AREA */


#define k_FILL_NORMAL					0
#define k_FILL_DEFAULT_RESOURCE		1
#define k_FILL_SOLID_BLACK				2
#define k_FILL_CURRENT_CHROMA		3

/*  */
/* If we get too many updates per second this will help make it faster */
/* by only updating animation art if required due to proper */
/* passage of time. Should probably be set to "2" */
/*  */
#define k_FREQUENT_UPDATE_TIME_GOVERNOR	2

/*  */
/* For safety the clip region can be reset to entire destination port on screen */
/* updating by force if needed. This is done using the following */
/* boolean flag. It is only used when copying back the entire window*/
/* and if QuickDraw is not being used could be ignored entirely */
/* THIS IS NOT NEEDED */
/*  */
#define k_SWAP_CLIP_REGIONS_DESIRED_PREFERENCE		(FALSE)

/*  */
/* If "True" CopyBits is avoided, and our own routines to draw to screen are used. */
/* This is marginal benefit. On PowerPC until PowerPC blit code written this is a */
/* hindrance */ 
/* THIS IS USEFUL */
/*  */
#define k_AVOID_COPYBITS_FOR_LARGE_COPY_PREFERENCE		(FALSE)


/* [----END----] #define AREA */
/*=**************************************************************************=*/


/*=**************************************************************************=*/
/* [---BEGIN---] TYPE AREA */


/* [----END----] TYPE AREA */
/*=**************************************************************************=*/


/*=**************************************************************************=*/
/* [---BEGIN---] PROTOTYPE AREA */

Safe_Boolean Delete_Frontmost_Item(Stage_View_Ptr stage_View, CARDINAL_16	buffer_To_Erase_From);

void Indicate_Next_Mix_Buffer_To_Start_Using(void);


/* [----END----] PROTOTYPE AREA */
/*=**************************************************************************=*/


/*=**************************************************************************=*/
/* [---BEGIN---] GLOBAL DEFINITION AREA */


/* [----END----] GLOBAL DEFINITION AREA */
/*=**************************************************************************=*/


/*=**************************************************************************=*/
/* [---BEGIN---] LOCAL STATIC DEFINITION AREA */

static CARDINAL_8	*unit_Name;


/* [----END----] LOCAL STATIC DEFINITION AREA */
/*=**************************************************************************=*/


/*=**************************************************************************=*/
Safe_Boolean Initialize_Stage_View_Manager(void)
{
	CARDINAL_8		*static_Unit_Name = "\pStage_View_Manager";
	Safe_Boolean		function_Result;
	SIMPLE_LOOP	loop;
/* END VAR */


/*  */
/* Global persistant structures are created here */
/* Actually are initialized when needed. */
/*  */

	dream.stage_View.stage_View_Buffers_Initialized = SAFE_FALSE;
	dream.stage_View.display_Phase = k_DISPLAY_SYSTEM_TOTALLY_DAMAGED_UNREADY;
	dream.stage_View.display_Minor_Phase = 0;
	dream.stage_View.next_Buffer_To_Use = 0;
	
	for (loop = 0; (loop < k_BUFFERS_REQUIRED); loop++) {
		dream.stage_View.mix_View_Area_Port_Hdl[loop] = nil;
		dream.stage_View.mix_View_Area_Port_Ptr[loop] = nil;
		dream.stage_View.mix_View_Area_Port_Memory_Initialized[loop] = MEMORY_NOT_INITIALIZED;
	}
	
	dream.stage_View.double_Buffered_Data[0].total_Remaining_Erasable_Items = 0;
	dream.stage_View.double_Buffered_Data[1].total_Remaining_Erasable_Items = 0;
	
	dream.stage_View.special_View_Mode = k_FILL_DEFAULT_RESOURCE;
	SetRect(&(dream.stage_View.stage_Active_View_Rect), 0, 0, 1, 1);
	SetRect(&(dream.stage_View.stage_View_Buffers_Total_Rect), 0, 0, 1, 1);
	
	dream.stage_View.last_Visible_Buffer_Update_Request = 0;
	dream.stage_View.double_Buffered_Data[0].total_Remaining_Erasable_Items = 0;
	dream.stage_View.double_Buffered_Data[1].total_Remaining_Erasable_Items = 0;
	
	dream.stage_View.initialized = SAFE_TRUE;
	unit_Name = static_Unit_Name;
	
	function_Result = SAFE_TRUE;
	return(function_Result);
	
} /* Safe_Boolean Initialize_Stage_View_Manager... */
/*=**************************************************************************=*/


/*=**************************************************************************=*/
Safe_Boolean Draw_Current_Stage_Background_Layers(Stage_View_Ptr stage_View
																, CARDINAL_32 mix_Buffer_To_Use
																, Safe_Boolean		*something_Changed_In_View)
{
	Safe_Boolean		function_Result;
/* END VAR */

	mix_Buffer_To_Use = mix_Buffer_To_Use + k_MIX_AREA_BUFFER_0;
	*something_Changed_In_View = SAFE_FALSE;


	/*  */
	/* Traverse list of scene and look for movie Rect items that*/
	/* need updating. These movies are the ones that draw at */
	/* lowest level of scene, do not draw other layers just the */
	/* real background layers. */
	/* Not implemented yet */
	/*  */





	function_Result = SAFE_TRUE;
	return(function_Result);
	
} /* Safe_Boolean Draw_Current_Stage_Background_Layers... */
/*=**************************************************************************=*/


/*=**************************************************************************=*/
void Indicate_Next_Mix_Buffer_To_Start_Using(void)
{
CARDINAL_32	total_Items_For_Debug;
/* END VAR */

	/*  */
	/* Debug stuff */
	/*  */
#if (7==11)
	total_Items_For_Debug = dream.stage_View.double_Buffered_Data[dream.stage_View.next_Buffer_To_Use].total_Remaining_Erasable_Items;
	if (total_Items_For_Debug < 3) {
		DebugStr("\pTEST!!! Ignore crap.");
	}
#endif
	
	/*  */
	/* Advance to next buffer */
	/*  */
	dream.stage_View.next_Buffer_To_Use += 1;
	if (dream.stage_View.next_Buffer_To_Use + k_BASE_OFFSET_FOR_MIX >= k_BUFFERS_REQUIRED) {
		dream.stage_View.next_Buffer_To_Use = 0;
	}
	
	dream.stage_View.display_Phase = k_DISPLAY_PHASE_IDLE_START_ERASE;
	dream.stage_View.display_Minor_Phase = 0;
}
/*=**************************************************************************=*/


/*=**************************************************************************=*/
Safe_Boolean Copy_Composition_Buffer_To_Screen(Stage_View_Ptr stage_View
															, CARDINAL_32 mix_Buffer_To_Use
															, GrafPtr	current_Graphics_Window_Ptr
															, INTEGER_32		offset_X
															, INTEGER_32		offset_Y)
{
	Off_Screen_Port_Rec *port_Ptr;
	const BitMap 	*destination_Bitmap;
	CARDINAL_32	temp_Hack_Real_Seed;
	CARDINAL_32	window_Clut_Seed;
	GrafPtr			saved_Port;
	GDHandle			saved_Device;
	Rect				source_Rect;
	Rect				destination_Rect;
	RgnHandle		save_Region;
	Safe_Boolean		region_Swapped;
	CARDINAL_16	save_Region_Index;
	Safe_Boolean		function_Result;
	Safe_Boolean		color_Table_Seeds_Spoofed;
	Boolean			swap_Clip_Regions_Desired;
	Boolean			use_Direct_Screen_Access;
	Boolean			port_Was_Set;
	CARDINAL_8		*code_Name;
/* END VAR */


	code_Name = "\pCopy_Composition_Buffer_To_Screen";
	use_Direct_Screen_Access = k_AVOID_COPYBITS_FOR_LARGE_COPY_PREFERENCE;
	
	swap_Clip_Regions_Desired = k_SWAP_CLIP_REGIONS_DESIRED_PREFERENCE;
	region_Swapped = SAFE_FALSE;

	
	/*  */
	/* This is not always next_Buffer_To_Use */
	/*  */
	mix_Buffer_To_Use = mix_Buffer_To_Use + k_MIX_AREA_BUFFER_0;
	
	GetPort(&saved_Port);
	saved_Device = GetGDevice();
			
	
	port_Ptr = (stage_View->mix_View_Area_Port_Ptr[mix_Buffer_To_Use]);
	Set_View_Port(current_Graphics_Window_Ptr);
	
/* ===
	destination_Bitmap = &(current_Graphics_Window_Ptr->portBits);
=== */
	/*  */
	/* Save Mac some work and try setting to pixmap instead */
	/*  */
	destination_Bitmap = (const BitMap *)*(((CWindowPeek)current_Graphics_Window_Ptr)->port.portPixMap);
	
	source_Rect = stage_View->stage_Active_View_Rect;
	destination_Rect = source_Rect;
	OffsetRect(&destination_Rect, (-destination_Rect.left + offset_X), (-destination_Rect.top + offset_Y));
	
/* ====
	Set_View_Device((stage_View->mix_View_Area_Port_Ptr[mix_Buffer_To_Use]->offscreen_GDevice_Hdl));
==== */
	/*  */
	/* Use main screen device not the offscreen_GDevice_Hdl of the source port. */
	/* Otherwise nothing will draw. */
	/*  */
	if (g_Preferred_Monitor_Values_Valid == SAFE_TRUE) {
		Set_View_Device(g_Preferred_Monitor_Graphics_Device);
	}

	port_Was_Set = TRUE;

	/*  */
	/*  */
	/*  */
	/*  */
	/*  */
	/* HACK HACK HACK, ignore VBL tearing? */
	/*  */
/* ===
	if (g_Preferred_Monitor_Values_Valid == SAFE_TRUE) {
		Wait_For_Next_Video_Monitor_VBL(g_Preferred_Monitor_Slots);
	}
===== */

	if ((use_Direct_Screen_Access == TRUE)
		&& ((port_Ptr->color_Type == k_COLOR_QUICKDRAW) 
		|| (port_Ptr->color_Type == k_32_BIT_COLOR_QUICKDRAW))) {
		/*  */
		/* Danger:  beware of moving pixmap memory, though Copybits may in theory as well */
		/*  */
		Draw_Copied_Bits((const BitMap *)*(port_Ptr->offscreen_Port.color_Port.portPixMap)
				, destination_Bitmap
				, &source_Rect
				, &destination_Rect
				, srcCopy
				, nil);
	}
	else {
		/*  */
		/* Update Normally using copybits, use_Direct_Screen_Access not desired */
		/*  */
	
		/*  */
		/* TEMP HACK TO SCREW WITH CLIP REGION! */
		/*  */
		if (swap_Clip_Regions_Desired == TRUE) {
			if (Get_Clip_Region_From_Pool(&save_Region, &save_Region_Index) == SAFE_TRUE) {
				InvalRect(&((current_Graphics_Window_Ptr)->portRect));
				GetClip(save_Region);
				ClipRect(&((current_Graphics_Window_Ptr)->portRect));
				region_Swapped = SAFE_TRUE;
			}
			else {
				/*  */
				/* Ignore errors in this version */
				/*  */
			}
		}
		
		/*  */
		/* Tell quickdraw not to try matching */
		/*  */
		
#if !defined(k_FORCE_NO_QUICKDRAW_CLUT_MATCH)
#error "UNDEFINED PARAMETER!"
#endif
#if (k_FORCE_NO_QUICKDRAW_CLUT_MATCH == 1)
			Set_Color_Match_For_Pixel_Copy(FALSE, port_Ptr->offscreen_Port.color_Port.portPixMap);
			
			color_Table_Seeds_Spoofed = SAFE_FALSE;
			temp_Hack_Real_Seed = (**(**(port_Ptr->offscreen_Port.color_Port.portPixMap)).pmTable).ctSeed;
			window_Clut_Seed = (**(**(((CWindowPeek)current_Graphics_Window_Ptr)->port.portPixMap)).pmTable).ctSeed;
			if (temp_Hack_Real_Seed != window_Clut_Seed) {
				color_Table_Seeds_Spoofed = SAFE_TRUE;
				(**(**(port_Ptr->offscreen_Port.color_Port.portPixMap)).pmTable).ctSeed = window_Clut_Seed;
			}
#endif
	
		switch (port_Ptr->color_Type) {
			case k_COLOR_QUICKDRAW:
			case k_32_BIT_COLOR_QUICKDRAW:
						
				if (use_Direct_Screen_Access == TRUE){
					Draw_Copied_Bits((const BitMap *)*(port_Ptr->offscreen_Port.color_Port.portPixMap)
							, destination_Bitmap
							, &source_Rect
							, &destination_Rect
							, srcCopy
							, nil);
								
				}
				else {				
					CopyBits ((const BitMap *)*(port_Ptr->offscreen_Port.color_Port.portPixMap)
								, destination_Bitmap
								, &source_Rect
								, &destination_Rect
								, srcCopy
								, nil);
				}
			break;
			
			case k_ORIGINAL_QUICKDRAW:
			default:
			
				CopyBits (&(port_Ptr->offscreen_Port.old_Style_Port.portBits)
								, destination_Bitmap
								, &source_Rect
								, &destination_Rect
								, srcCopy
								, nil);
													
		} /*  switch (g_Indicator.color_Type) */
	
	#if (k_FORCE_NO_QUICKDRAW_CLUT_MATCH == 1)
	
		if (color_Table_Seeds_Spoofed == SAFE_TRUE) {
			(**(**(port_Ptr->offscreen_Port.color_Port.portPixMap)).pmTable).ctSeed
				= temp_Hack_Real_Seed;
		}
	
		Set_Color_Match_For_Pixel_Copy(TRUE, port_Ptr->offscreen_Port.color_Port.portPixMap);
	#endif
	} /* NOT if ((use_Direct_Screen_Access == TRUE) */
	
	/*  */
	/* This is the one position where "next_Buffer_To_Use" is incremented. */
	/*  */
	Indicate_Next_Mix_Buffer_To_Start_Using();
		
	function_Result = SAFE_TRUE;
	goto Normal_Function_Exit;
	
Error_Handler:
	function_Result = SAFE_FALSE;
	Display_Serious_Error_Message(unit_Name, code_Name, "\pUnexpected serious error", "\pMemory too low?");

		
Normal_Function_Exit:


	if (region_Swapped == SAFE_TRUE) {
		region_Swapped = SAFE_FALSE;

		ValidRect(&((current_Graphics_Window_Ptr)->portRect));
		SetClip(save_Region);
		if (Return_Clip_Region_To_Pool(save_Region_Index) != SAFE_TRUE) {
			/* Ignore error, routine is exiting */
		}
	}

	if (port_Was_Set == TRUE) {
		Set_View_Device(saved_Device);
		Set_View_Port(saved_Port);
	}

	return(function_Result);
	
} /* Safe_Boolean Copy_Composition_Buffer_To_Screen... */
/*=**************************************************************************=*/


/*=**************************************************************************=*/
Safe_Boolean Process_Animation_Tasks_For_Main_Stage(Stage_View_Ptr stage_View)
/*  */
/* This is the simple state machine for the animation phases */
/* to allow psuedo-multitasking if called iteratively.*/
/* Max time consumtion unlimited but hopefully 1 tick. Minimum */
/* is always negligible hopefully. */
/*  */
{
	CARDINAL_32	time_Now;
	SIMPLE_LOOP	loop;
	Safe_Boolean		function_Result;
	CARDINAL_8		phase;
	CARDINAL_8		*code_Name;
/* END VAR */


	function_Result = SAFE_FALSE;
	code_Name = "\pProcess_Animation_Tasks_For_Main_Stage";

	/*  */
	/* Drop to 8 bit index for speed in possible hash jump? */
	/*  */
	phase = stage_View->display_Phase;
	
	switch (phase) {
	
		case k_DISPLAY_PHASE_IDLE_START_ERASE:
			/*  */
			/* This phase means that the screen is updated and that nothing */
			/* special has started yet. It is a good idea to start erasing the */
			/* old sprite regions in preparation for a new animation frame. */
			/* the sprites might need to be reloaded from disk if LRU system *&/
			/* not sorking correctly. In fact the buffers are not Last Recenty Used */
			/* but instead it is a ring buffer of Last Recently Loaded. This is not the same */
			/* and reloads may be more common than desired. But is the buffer */
			/* is large it is not a terrible problem */
			/* This state is useful to allow the screen to be redrawn again if needed. */
			/* If the Mac STARTS ASKING for too many updates per second (weird hacks) then */
			/* instead of animating so often perhaps consider dwelling on this state */
			/* for at least two ticks unless forced in animation earlier. This allows hundreds */
			/* of updates per second if pathological condition demands it, otherwise only */
			/* 15 to 30 updates per second might result. */
			/* On average it is bad to hang around on this too long and better to */
			/* jump into erase active stage. */
			/*  */
			
			time_Now =  (CARDINAL_32)TickCount();
			if (stage_View->last_Visible_Buffer_Update_Request + k_FREQUENT_UPDATE_TIME_GOVERNOR 
							<= time_Now) {
				stage_View->display_Phase += 1;
				stage_View->display_Minor_Phase = 0;
			}
			else {
				/*  */
				/* Tick wrap/ corruption/ timewarp are considered in equation */
				/*  */
				
				if (time_Now < stage_View->last_Visible_Buffer_Update_Request) {
					stage_View->last_Visible_Buffer_Update_Request = 0;
					stage_View->display_Phase += 1;
					stage_View->display_Minor_Phase = 0;
				}
				else {
					/*  */
					/* Before deciding to sit and not destroy the update buffer */
					/* decide if there are two update buffers and therfore the */
					/* code allows stale updates ? */
					/*  */
					
					if (k_BUFFERS_REQUIRED > 2) {
					
						/*  */
						/* This version is two phase double buffered. Therefore if a */
						/* sudden update request keeps occuring a second buffer */
						/* probably exists so deletion of this buffer now is not a big problem */
						/* it is a big problem if we should be doing deletion only during */
						/* disk seeks however. */ 
						/*  */
						
						stage_View->display_Phase += 1;
						stage_View->display_Minor_Phase = 0;
					}
					else {
						/*  */
						/* This has not been tested (single buffer code not finished yet*/
						/*  */
						; /* NULL DESIRED */
					}
				}
			}
		break;
		
		case k_DISPLAY_PHASE_ERASE_ACTIVE:
			/*  */
			/* This phase means that it is OK to start erasing any layers still */
			/* needing erasure. Traditionally, you start from forground backward. */
			/* this allows multilayer sprite systems with embedded background */
			/* updating to restore original pane when done in strict reverse order, */
			/* it also allows simple culling of covered regions for speed, if overlap */
			/* optimization is considerred. Our model is primitive though, and does not */
			/* yet contain any cookie cutter optimizations. Start erasing sprite layers. */
			/* remember that some might regretfully need disk seeks. */
			/* Those could be special cased but instead they are brute force delayed. */
			/*  */
			if ((k_BUFFERS_REQUIRED > 2) 
				&& (stage_View->double_Buffered_Data[stage_View->next_Buffer_To_Use].total_Remaining_Erasable_Items == 0)){
			
				/*  */
				/* Only reaches here if there is absolutely nothing to erase left */
				/*  */
				
				stage_View->display_Phase += 1;
				stage_View->display_Minor_Phase = 0;
			}
			else {
				/*  */
				/* NOTE: Single phase buffer code Not tested yet. */
				/*  */
				
				if (stage_View->double_Buffered_Data[stage_View->next_Buffer_To_Use].total_Remaining_Erasable_Items > 0) {
				
					/*  */
					/* Start deleting if indicators show that there is stuff to delete */
					/*  */

					stage_View->display_Minor_Phase += 1;

					if (Delete_Frontmost_Item(stage_View, stage_View->next_Buffer_To_Use) != SAFE_TRUE){
						/*  */
						/* Add warning in future version */
						/*  */
					}
	
				} /* if (stage_View->double_Buffered_Data[ ... */
				
				if (stage_View->double_Buffered_Data[stage_View->next_Buffer_To_Use].total_Remaining_Erasable_Items <= 0) {
					stage_View->display_Phase += 1;
					stage_View->display_Minor_Phase = 0;
				}
			} /* if (k_BUFFERS_REQUIRED > 2) ... */
		break;
		
		case k_DISPLAY_PHASE_READY_TO_COMPOSITE:
			/*  */
			/* This phase means that there is a nice pristine offscreen buffer */
			/* with all static background art ready to mess up. If the static */
			/* background art needs to be slow panned AGAINST the view of other */
			/* items now is a good place to do it (time consuming though. */
			/* If there is a QuickTime/QuickestTime frame to show this is the */
			/* place to display it also. After that stuff is done, the digital sound */
			/* to be used for the next animation cycle of the talking puppets will be */
			/* grabbed and new heads will be drawn.*/
			/* Note that this stage is skipped if old art active (that jumps to */
			/* k_DISPLAY_PHASE_COMPOSITING_ACTIVE after dropping in some art) */
			/*  */
			
			stage_View->display_Phase += 1;
			stage_View->display_Minor_Phase = 0;
		break;
		
		case k_DISPLAY_PHASE_COMPOSITING_ACTIVE:
			/*  */
			/* This phase means that it is OK to start drawing all layers. */
			/* Drawing must be from back to front. */
			/* We will be hung in this section waiting for disk seeks quit often. */
			/* A better technology might be to draw a previous buffer to screen */
			/* while waiting for disk loads, or decompressing a Quiktime frame, or */
			/* doing something such as erasing a second animation buffer */
			/* while the hard disk is seeking. */
			/* Those delay tactics result in a slight system drag/delay with a */
			/* larger sound delay. Sound was determined before selection of */
			/* animation frames was determined to procede and previous art */
			/* was already scrubbed in our area. If a second buffer frame is made */
			/* it allows slightly more parallel activity, but it causes the ring buffer */
			/* for the art to be busier with stale hits more frequently (to erase two */
			/* frames back. Disk drive heads more 125 times per second, and the array */
			/* we are using is a true Async 18.3 MB per second array on Q840 using */
			/* 2 Seagate 12450W 10.3 MB/sec drives in 32K stripped array. */
			/*  */
			
			stage_View->display_Phase += 1;
			stage_View->display_Minor_Phase = 0;
		break;
		
		case k_DISPLAY_PHASE_READY_TO_BLAST_PORT:
			/*  */
			/* This phase means that all art in a buffer is finished */
			/* and is ready to draw to screen. It also means we are probably waiting */
			/* For VBL phase to allow copy bits transfer to start. */
			/* If so, then some tiny processing of other data (such as erasing a frame) */
			/* is desired if in buffer and ready and small? */
			/* Usually we will just draw as soon as this state is achieved. */
			/*  */
			
			/*  */
			/* Honor "stage_View->draw_System_Style_Update_Flag" */
			/*  */
			
			stage_View->display_Phase += 1;
			stage_View->display_Minor_Phase = 0;
			if (stage_View->draw_System_Style_Update_Flag == SAFE_TRUE) {
				stage_View->draw_System_Style_Update_Flag = SAFE_FALSE;
			}
		break;
		
		case k_DISPLAY_PHASE_BLASTING_PORT_ACTIVE:
			/*  */
			/* This phase means that there is a copybits in progress */
			/* only interrupt time services will ever see this state. */
			/* because copybits is done in one thread and blocks other */
			/* actions. */
			/*  */
			
			/*  */
			/* Do not do anything. This is just a state marker. */
			/* However if we do get here it is a bug! And that means that */
			/* there was no draw action to take the last state */
			/* "k_DISPLAY_PHASE_READY_TO_BLAST_PORT" so just reset to */
			/* idle state ! */
			/*  */
			stage_View->display_Phase = k_DISPLAY_PHASE_IDLE_START_ERASE;
			stage_View->display_Minor_Phase = 0;
		break;
		
		case k_DISPLAY_REQUIRES_TOTAL_BUFFER_REBUILD:
			/*  */
			/* This phase means that the buffers need rebuilding (probably do). */
			/* Changing the color depth or default disply device might do this. */
			/* Changing the display window to a larger size or much smaller size */
			/* would do this too. */
			/*  */
			
			if (Destroy_Stage_View_Manager() != SAFE_TRUE) {
				/*  */
				/* Ignore errors of this type */
				/*  */
			}
			
			if (Initialize_Stage_View_Manager() != SAFE_TRUE) {
				/*  */
				/* Ignore errors of this type */
				/*  */
			}
			
			if (Reset_Stage_View_Manager_If_Needed() != SAFE_TRUE) {
				/*  */
				/* This is a serious error that is likely to happen if low memory */
				/*  */
				goto Error_Handler;
			}
			
			/*  */
			/* Note: going merely to next phase is actually wasteful */
			/* because it is redundant in current design */
			/*  */
			stage_View->display_Phase += 1;
			stage_View->display_Minor_Phase = 0;
		break;
		
		case k_DISPLAY_REQUIRES_TOTAL_BUFFER_REDRAW:
			/*  */
			/* This phase means that either the background art */
			/* changed or something was alterred that requires the background */
			/* buffers to be redrawn completely. */
			/* Changing the color depth or default disply device might do this. */
			/* Changing the display window to a larger size or much smaller size */
			/* would do this too. */
			/*  */
			
			/*  */
			/* Should probably clear to default color or something */
			/*  */
			for (loop = 0; (loop < k_BUFFERS_REQUIRED);  loop++) {
			
				if (Force_Clear_Stage_To_Default_Data(SAFE_FALSE
									, dream.stage_View.mix_View_Area_Port_Ptr[loop]) != SAFE_TRUE) {
					 goto Error_Handler;
				}
			}
			dream.stage_View.double_Buffered_Data[0].total_Remaining_Erasable_Items = 0;
			dream.stage_View.double_Buffered_Data[1].total_Remaining_Erasable_Items = 0;

			stage_View->display_Phase += 1;
			stage_View->display_Minor_Phase = 0;
		break;
		
		case k_DISPLAY_REQUIRES_VISIBLE_BUFFER_REDRAW:
			/*  */
			/* This phase means that there is a request for a visible screen redraw. */
			/* If it is not possible to do this using a stale buffer, (common problem) */
			/* then we need to jump immediately to a redraw cycle state */
			/* and process each step in order, yet completing with a knockdown flag */
			/* that is reset upon drawing conclusion.
			/*  */
			stage_View->last_Visible_Buffer_Update_Request = TickCount();
			
			stage_View->display_Phase = k_DISPLAY_PHASE_ERASE_ACTIVE;
			stage_View->display_Minor_Phase = 0;
			
			stage_View->draw_System_Style_Update_Flag = SAFE_TRUE;
		break;
		
		case k_DISPLAY_SYSTEM_TEMPORARY_FROZEN_AND_IDLE:
			/*  */
			/* This phase means that there is some problem or that the */
			/* display sytem is not being used and is not needed. */
			/* This is used when direct to screen animation is being performed */
			/* at a faster update rate than the normal multistage animation layers. Do nothing. */
			/*  */
		break;
		
		case k_DISPLAY_SYSTEM_TOTALLY_DAMAGED_UNREADY:
			/*  */
			/* This phase means that there is some problem or that the */
			/* display sytem is not being used anymore or is not needed anymore. */
			/* It also is the starting state when the program is loaded but the buffers */
			/* are not created yet. Do nothing. */
			/*  */
		break;
		
		default:
			goto Error_Handler;
	} /* switch (stage_View->display_Phase) */

	function_Result = SAFE_TRUE;
	goto Normal_Function_Exit;
	
Error_Handler:
	function_Result = SAFE_FALSE;
	Display_Serious_Error_Message(unit_Name, code_Name, "\pUnexpected serious error", "\pMemory too low?");

		
Normal_Function_Exit:
	return(function_Result);

} /* Safe_Boolean Process_Animation_Tasks_For_Main_Stage(Stage_View_Ptr stage_View)... */
/*=**************************************************************************=*/


/*=**************************************************************************=*/
Safe_Boolean	Add_Spin_Item_To_Removal_List(Stage_View_Ptr stage_View
														, CARDINAL_32	composite_Search_Key
														, CARDINAL_32	spin_Info_Address
														, CARDINAL_32	previous_Index
														, INTEGER_16	file_Reference
														, INTEGER_16 offset_X
														, INTEGER_16 offset_Y)
/*  */
/* Adds to active stack-like list, so future deletions are possible.*/
/*  */
{
	Item_Location_List_Ptr		list_Ptr;
	Item_Information_Ptr		item_Ptr;
	Safe_Boolean		function_Result;
/* END VAR */


	list_Ptr = &(stage_View->double_Buffered_Data[stage_View->next_Buffer_To_Use]);
	
	if (list_Ptr->total_Remaining_Erasable_Items < k_MAX_STAGE_ITEMS) {
		item_Ptr = &list_Ptr->item_To_Erase[list_Ptr->total_Remaining_Erasable_Items];
		item_Ptr->item_Type = k_ITEM_TYPE_SPIN_OBJECT;
		item_Ptr->offset_X = offset_X;
		item_Ptr->offset_Y = offset_Y;
		
		item_Ptr->info.compacted_Pict.composite_Search_Key = composite_Search_Key;
		item_Ptr->info.compacted_Pict.spin_Info_Address = (CARDINAL_32)spin_Info_Address;
		item_Ptr->info.compacted_Pict.previous_Index = previous_Index;
		item_Ptr->info.compacted_Pict.file_Reference = file_Reference;
		list_Ptr->total_Remaining_Erasable_Items += 1;
	}

	function_Result = SAFE_TRUE;
	return(function_Result);
	
} /* Safe_Boolean	Add_Spin_Item_To_Removal_List(Stage_View_Ptr stage_View... */
/*=**************************************************************************=*/


/*=**************************************************************************=*/
Safe_Boolean	Add_Old_Puppet_To_Removal_List(Stage_View_Ptr stage_View
														, Ptr		existing_Puppet_Ptr
														, CARDINAL_16	activation_Frame
														, INTEGER_16 offset_X
														, INTEGER_16 offset_Y
														, Point scene_Offset)
/*  */
/* Adds to active stack-like list, so future deletions are possible.*/
/*  */
{
	Item_Location_List_Ptr		list_Ptr;
	Item_Information_Ptr		item_Ptr;
	Safe_Boolean		function_Result;
/* END VAR */


	list_Ptr = &(stage_View->double_Buffered_Data[stage_View->next_Buffer_To_Use]);
	
	if (list_Ptr->total_Remaining_Erasable_Items < k_MAX_STAGE_ITEMS) {
		item_Ptr = &list_Ptr->item_To_Erase[list_Ptr->total_Remaining_Erasable_Items];
		item_Ptr->item_Type = k_ITEM_TYPE_SPIN_OBJECT;
		item_Ptr->offset_X = offset_X;
		item_Ptr->offset_Y = offset_Y;
		
		item_Ptr->info.region_Pict.existing_Puppet_Ptr = existing_Puppet_Ptr;
		item_Ptr->info.region_Pict.activation_Frame = activation_Frame;
		item_Ptr->info.region_Pict.scene_Offset = scene_Offset;
		list_Ptr->total_Remaining_Erasable_Items += 1;
		
	} /* if (list_Ptr->total_Remaining_Erasable_Items < k_MAX_STAGE_ITEMS) */
	
	function_Result = SAFE_TRUE;
	return(function_Result);
	
} /* Safe_Boolean	Add_Old_Puppet_To_Removal_List(Stage_View_Ptr stage_View... */
/*=**************************************************************************=*/


/*=**************************************************************************=*/
Safe_Boolean	Add_Rect_Area_To_Removal_List(Stage_View_Ptr stage_View
														, Rect	*rect_Area
														, INTEGER_16 offset_X
														, INTEGER_16 offset_Y)
/*  */
/* Adds to active stack-like list, so future deletions are possible.*/
/*  */
{
	Item_Location_List_Ptr		list_Ptr;
	Item_Information_Ptr		item_Ptr;
	Safe_Boolean		function_Result;
/* END VAR */


	list_Ptr = &(stage_View->double_Buffered_Data[stage_View->next_Buffer_To_Use]);
	
	if (list_Ptr->total_Remaining_Erasable_Items < k_MAX_STAGE_ITEMS) {
		item_Ptr = &list_Ptr->item_To_Erase[list_Ptr->total_Remaining_Erasable_Items];
		item_Ptr->item_Type = k_ITEM_TYPE_SPIN_OBJECT;
		item_Ptr->offset_X = offset_X;
		item_Ptr->offset_Y = offset_Y;
		
		item_Ptr->info.arbitrary_Rectangle.modified_Rect_Area = *rect_Area;
		list_Ptr->total_Remaining_Erasable_Items += 1;
		
	} /* if (list_Ptr->total_Remaining_Erasable_Items < k_MAX_STAGE_ITEMS) */
	
	function_Result = SAFE_TRUE;
	return(function_Result);
	
} /* Safe_Boolean	Add_Rect_Area_To_Removal_List(Stage_View_Ptr stage_View... */
/*=**************************************************************************=*/


/*=**************************************************************************=*/
Safe_Boolean Delete_Frontmost_Item(Stage_View_Ptr stage_View, CARDINAL_16	buffer_To_Erase_From)
/*  */
/* This will properly set and restore the port. It is called when asyn */
/* data reads are waiting for a different mx buffer to draw, and is called */
/* if a frame is used and it is not already cleared. */
/* Therefore it is a useful routine that strategically gives the CPU */
/* valuable work to do when waiting for a async read to compleat. */
/*  */
{
	Off_Screen_Ptr				port_Ptr;
	Item_Location_List_Ptr		list_Ptr;
	Item_Information_Ptr		item_Ptr;
	GrafPtr			saved_Port;
	GDHandle			saved_Device;
	Safe_Boolean		function_Result;
	INTEGER_32		offset_X;
	INTEGER_32		offset_Y;
	Boolean			port_Was_Set;
/* END VAR */

	port_Was_Set = FALSE;

	list_Ptr = &(stage_View->double_Buffered_Data[stage_View->next_Buffer_To_Use]);
	
	if (list_Ptr->total_Remaining_Erasable_Items > 0) {
	
		GetPort(&saved_Port);
		saved_Device = GetGDevice();
				
		/*  */
		/* This is not always next_Buffer_To_Use */
		/*  */
		
		port_Ptr = (dream.stage_View.mix_View_Area_Port_Ptr[buffer_To_Erase_From + k_BASE_OFFSET_FOR_MIX]);
		
		Set_View_Port((GrafPtr)&(port_Ptr->offscreen_Port.color_Port));
		Set_View_Device((port_Ptr->offscreen_GDevice_Hdl));
		port_Was_Set = TRUE;

		/*  */
		/* Decrement counter of scene items that can probably be deleted */
		/* This counter is not double validated so it might be forced low early */
		/* if nothing is found. */
		/*  */
		list_Ptr->total_Remaining_Erasable_Items = list_Ptr->total_Remaining_Erasable_Items - 1;

		item_Ptr = &list_Ptr->item_To_Erase[list_Ptr->total_Remaining_Erasable_Items];
		
		offset_X = item_Ptr->offset_X;
		offset_Y = item_Ptr->offset_Y;
		
		switch(item_Ptr->item_Type) {
		
			case k_ITEM_TYPE_SPIN_OBJECT: 	
				if (Erase_Spin_Item_From_Mix_Buffer(dream.stage_View.mix_View_Area_Port_Ptr[buffer_To_Erase_From + k_BASE_OFFSET_FOR_MIX]
														, dream.stage_View.mix_View_Area_Port_Ptr[k_BACKGROUND_BUFFER]
														, item_Ptr->info.compacted_Pict.composite_Search_Key
														, item_Ptr->info.compacted_Pict.spin_Info_Address
														, item_Ptr->info.compacted_Pict.previous_Index
														, item_Ptr->info.compacted_Pict.file_Reference
														, offset_X
														, offset_Y) != SAFE_TRUE) {
					/*  */
					/* Ignore error in implementation */
					/*  */
				}
				
				
			break;
		
			case k_ITEM_TYPE_OLD_PUPPET_OBJECT:
				if (Erase_Old_Item_From_Mix_Buffer(dream.stage_View.mix_View_Area_Port_Ptr[buffer_To_Erase_From + k_BASE_OFFSET_FOR_MIX]
														, (Puppet_Ptr)item_Ptr->info.region_Pict.existing_Puppet_Ptr
														, item_Ptr->info.region_Pict.activation_Frame
														, item_Ptr->info.region_Pict.scene_Offset
														, offset_X
														, offset_Y) != SAFE_TRUE) {
					/*  */
					/* Ignore error in implementation */
					/*  */
				}
			break;
		
			case k_ITEM_TYPE_ARBITRARY_RECT_OBJECT:
			/* ==
				item_Ptr->info.arbitrary_Rectangle.modified_Rect_Area
			=== */
			break;
		
			default:
				; /* NULL */
				
		} /* switch(item_Ptr->item_Type) */
						
	} /* if (list_Ptr->total_Remaining_Erasable_Items >= 0) */

	function_Result = SAFE_TRUE;

	if (port_Was_Set == TRUE) {
		Set_View_Device(saved_Device);
		Set_View_Port(saved_Port);
	}

	return(function_Result);
	
} /* Safe_Boolean Delete_Frontmost_Item(Stage_View_Ptr stage_View)... */
/*=**************************************************************************=*/


/*=**************************************************************************=*/
Safe_Boolean Reset_Stage_View_Manager_If_Needed(void)
{
	Off_Screen_Ptr 	offscreen_Ptr;
	Rect				offscreen_Rect;
	OSErr				os_Error;
	CARDINAL_16	desired_Color_Depth;
	Safe_Boolean		function_Result;
	CARDINAL_32	max_Width;
	CARDINAL_32	max_Height;
	CARDINAL_32	preferred_Size;
	SIMPLE_LOOP	loop;
	CARDINAL_8		*code_Name;
	Str255				message_String;
/* END VAR */


	code_Name = "\pReset_Stage_View_Manager_If_Needed";

/*  */
/* Global persistant structures are created here */
/* Actually are initialized when needed. */
/*  */

	if (dream.stage_View.initialized == SAFE_TRUE) {
	
		if (dream.stage_View.stage_View_Buffers_Initialized != SAFE_TRUE) {
		
			dream.stage_View.double_Buffered_Data[0].total_Remaining_Erasable_Items = 0;
			dream.stage_View.double_Buffered_Data[1].total_Remaining_Erasable_Items = 0;

			for (loop = 0; (loop < k_BUFFERS_REQUIRED); loop++) {
				dream.stage_View.mix_View_Area_Port_Hdl[loop] = nil;
				dream.stage_View.mix_View_Area_Port_Ptr[loop] = nil;
				dream.stage_View.mix_View_Area_Port_Memory_Initialized[loop] = MEMORY_NOT_INITIALIZED;
			
				dream.stage_View.special_View_Mode = k_FILL_DEFAULT_RESOURCE;
				
				/*  */
				/* Create first port structure */
				/*  */
				preferred_Size = sizeof(Off_Screen_Port_Rec);
				
				if (Create_Handle(&preferred_Size
									, (CARDINAL_32)(10240L)
									, ALLOW_PURGING
									, DISALLOW_PARTIAL_AMOUNT
									, (Handle *)&(dream.stage_View.mix_View_Area_Port_Hdl[loop])
									, &dream.stage_View.mix_View_Area_Port_Memory_Initialized[loop]) != SAFE_TRUE) {
									
					Display_Low_Memory_Error_Message(unit_Name, code_Name, sizeof(Primary_Status_Record));
					goto Error_Handler;
				}
				
				/*  */
				/* It is CRITICAL that this port be locked down permanently. */
				/*  */
				MoveHHi((Handle)dream.stage_View.mix_View_Area_Port_Hdl[loop]);
				os_Error = MemError();
				if ((os_Error != noErr) && (os_Error != -117)){
					goto Error_Handler;
				}
				HLock((Handle)dream.stage_View.mix_View_Area_Port_Hdl[loop]);
				if (MemError() != noErr) {
					goto Error_Handler;
				}
				
				dream.stage_View.mix_View_Area_Port_Ptr[loop] = *dream.stage_View.mix_View_Area_Port_Hdl[loop];
				
			} /* for (loop = 0; (loop < k_BUFFERS_REQUIRED); loop++) */
			
			
			
			/*  */
			/* Create Memory for a port (structure copied later on)*/
			/*  */
			
			SetRect(&(dream.stage_View.stage_Active_View_Rect)
						, 0
						, 0
						, g_Window_Limits.main_Window_Width
						, g_Window_Limits.main_Window_Height);
			
			/*  */
			/* Height is only 150% the normal height. (25% above and below) */
			/* The total offscreen rect is now about 3 times the normal area. */
			/* This should be large enough with cropping space for most work */
			/*  */
			max_Height = g_Window_Limits.main_Window_Height + (g_Window_Limits.main_Window_Height >> 1);

			max_Width = g_Window_Limits.main_Window_Width;
			max_Width = max_Width + max_Width;
			
			/*  */
			/* Perfectly center rect. Align to paragraph boundary (128 bit cache line bounds) */
			/*  */
			OffsetRect(&(dream.stage_View.stage_Active_View_Rect)
									, (((g_Window_Limits.main_Window_Width >> 1) + 15) & 0xFFFFFFF0)
									, (g_Window_Limits.main_Window_Height >> 2));

			/*  */
			/* Mix buffer twice as wide, but add in extra 16 pixels for alignment safety */
			/*  */
			max_Width += 16;
			
			SetRect(&(dream.stage_View.stage_View_Buffers_Total_Rect)
						, 0
						, 0
						, max_Width
						, max_Height);

			
			offscreen_Rect = dream.stage_View.stage_View_Buffers_Total_Rect;
			offscreen_Ptr = dream.stage_View.mix_View_Area_Port_Ptr[k_BACKGROUND_BUFFER];
			offscreen_Ptr->initialized = SAFE_FALSE;
			offscreen_Ptr->offscreen_GDevice_Hdl = nil;
			offscreen_Ptr->image_Handle = nil;
			offscreen_Ptr->color_Table_Handle = nil;
			offscreen_Ptr->color_Type = k_COLOR_QUICKDRAW;
			
			desired_Color_Depth = g_Preferred_Monitor_Depth;
			
			if (desired_Color_Depth > 8) {
				if (Create_Clut_For_Direct_Mapping(desired_Color_Depth, &(offscreen_Ptr->color_Table_Handle)) != SAFE_TRUE) {
				 	goto Error_Handler;
				}
			}
			else {
				desired_Color_Depth = 8;
				offscreen_Ptr->color_Table_Handle = GetCTable(k_DEFAULT_256_COLOR_SYSTEM_CLUT);
				if (offscreen_Ptr->color_Table_Handle == nil) {
				 	goto Error_Handler;
				}
			}

			os_Error= Create_Off_Screen_Port(&offscreen_Rect
													, desired_Color_Depth
													, offscreen_Ptr->color_Table_Handle
													, (CGrafPtr)&(offscreen_Ptr->offscreen_Port.color_Port)
													, &(offscreen_Ptr->offscreen_GDevice_Hdl)
													, &(offscreen_Ptr->image_Handle));

			if (os_Error != noErr) {
				 goto Error_Handler;
			}
						
			/*  */
			/* Redesign to avoid locked crap one day */
			/*  */
			Lock_Video_Memory_For_Lazy_But_Safe_Use(offscreen_Ptr);
			offscreen_Ptr->initialized =  SAFE_TRUE; 
			
			dream.stage_View.stage_View_Buffers_Initialized = SAFE_TRUE;
			
			/*  */
			/* Duplicate the port exactly as many times as needed. */
			/*  */
			for (loop = 1; (loop < k_BUFFERS_REQUIRED);  loop++) {
				if (Copy_Off_Screen_Port(offscreen_Ptr
											, dream.stage_View.mix_View_Area_Port_Ptr[loop]) != SAFE_TRUE) {
					Copy_Pascal_String("\pA graphics port for the offscreen mix area failed to be created (low memory?)", message_String);
					Perform_OK_Dialog(message_String);
					goto Error_Handler;
				}
			}
			
			/*  */
			/* Should probably clear to default color or something */
			/*  */
			for (loop = 0; (loop < k_BUFFERS_REQUIRED);  loop++) {
				if (Force_Clear_Stage_To_Default_Data(SAFE_FALSE
									, dream.stage_View.mix_View_Area_Port_Ptr[loop]) != SAFE_TRUE) {
					 goto Error_Handler;
				}
			}
			
			dream.stage_View.double_Buffered_Data[0].total_Remaining_Erasable_Items = 0;
			dream.stage_View.double_Buffered_Data[1].total_Remaining_Erasable_Items = 0;
			
		} /* if (dream.stage_View.stage_View_Buffers_Initialized != SAFE_TRUE) */
		
	} /* if (dream.stage_View.stage_View_Buffers_Initialized == SAFE_TRUE) */

	function_Result = SAFE_TRUE;
	goto Normal_Function_Exit;
	
Error_Handler:
	function_Result = SAFE_FALSE;
	Display_Serious_Error_Message(unit_Name, code_Name, "\pError with creation", "\pMemory too low?");

		
Normal_Function_Exit:
	return(function_Result);
	
} /* Safe_Boolean Initialize_Stage_View_Manager... */
/*=**************************************************************************=*/


/*=**************************************************************************=*/
Safe_Boolean Destroy_Stage_View_Manager(void)
{
	Safe_Boolean		function_Result;
	SIMPLE_LOOP	loop;
	CARDINAL_8		*code_Name;
/* END VAR */


	code_Name = "\pDestroy_Stage_View_Manager";
	function_Result = SAFE_FALSE;
	
	
	/*  */
	/* Global persistant structures are deleted here */
	/*  */
	
	if (dream.stage_View.initialized == SAFE_TRUE) {
	
		dream.stage_View.stage_View_Buffers_Initialized = SAFE_FALSE;
		
		for (loop = 0; (loop < k_BUFFERS_REQUIRED); loop++) {
			if (dream.stage_View.mix_View_Area_Port_Memory_Initialized[loop] == SAFE_TRUE) {
				/*  */
				/* Close down the port, assume port active if handle is set. /
				/*  */
				if ((*dream.stage_View.mix_View_Area_Port_Hdl[loop])->initialized == SAFE_TRUE) {
				 	if (Close_Offscreen_Pict_Port((*dream.stage_View.mix_View_Area_Port_Hdl[loop])) != SAFE_TRUE) {
						/*  */
						/* complain about it */
						/*  */
				 		goto Error_Handler;
				 	}
			 	}
			 	else {
				 	goto Error_Handler;
			 	}
	
				/*  */
				/* Destroy port memory */
				/*  */
				if (Destroy_Handle((Handle *)(&dream.stage_View.mix_View_Area_Port_Hdl[loop])
							, &(dream.stage_View.mix_View_Area_Port_Memory_Initialized[loop])) != SAFE_TRUE) {
					Display_Serious_Error_Message(unit_Name, code_Name, "\pError with DisposHandle", "\pSystem Corrupted?");
					goto Error_Handler;
				}
				dream.stage_View.mix_View_Area_Port_Ptr[loop] = nil;
			}
		}
				
	}
	else {
		goto Error_Handler;
	}

	function_Result = SAFE_TRUE;
	goto Normal_Function_Exit;
	
Error_Handler:
	function_Result = SAFE_FALSE;
	Display_Serious_Error_Message(unit_Name, code_Name, "\pError with closing down", "\pSystem Corrupted?");

		
Normal_Function_Exit:
	return(function_Result);
}
/*=**************************************************************************=*/

	
/*=**************************************************************************=*/  
Safe_Boolean	Force_Clear_Stage_To_Default_Data(Safe_Boolean visible_Portion_Only
											, 	Off_Screen_Ptr	port_Ptr)
{
	CARDINAL_32	handle_Size;
	CARDINAL_32	fill_Data;
	RGBColor			plain_Flood_Fill_Color;
	GDHandle			saved_Device;
	Safe_Boolean		function_Result;
	Safe_Boolean		port_Was_Set;
	CARDINAL_8		*code_Name;
	GrafPtr			saved_Original_Port;
	PenState			saved_Pen_State;
/* END VAR */


	code_Name = "\pForce_Clear_Stage_To_Default_Data";
	function_Result = SAFE_FALSE;
	port_Was_Set = SAFE_FALSE;

	saved_Device = GetGDevice();
	GetPort(&saved_Original_Port);
	Set_View_Port((GrafPtr)&(port_Ptr->offscreen_Port.color_Port));
	Set_View_Device(port_Ptr->offscreen_GDevice_Hdl);
	GetPenState (&saved_Pen_State);
	port_Was_Set = SAFE_TRUE;

	if (visible_Portion_Only == SAFE_TRUE) {

		switch (dream.stage_View.special_View_Mode) {
		
			case k_FILL_DEFAULT_RESOURCE:
			case k_FILL_SOLID_BLACK:
				ForeColor(blackColor);
				BackColor(blackColor);
				BackPat(&(qd.black));
				EraseRect(&(dream.stage_View.stage_Active_View_Rect));
				BackColor(whiteColor);
			break;
						
			case k_FILL_CURRENT_CHROMA:
				Index2Color((g_Chroma_Index_Optimized_Long & 0x00FF), &plain_Flood_Fill_Color);
				RGBBackColor(&plain_Flood_Fill_Color);
				EraseRect(&(dream.stage_View.stage_Active_View_Rect));
				RGBBackColor(&g_Shell.library.color_White);
			break;
			
			case k_FILL_NORMAL:
				ForeColor(blackColor);
				BackColor(blackColor);
				BackPat(&(qd.black));
				EraseRect(&(dream.stage_View.stage_Active_View_Rect));
				BackColor(whiteColor);
			break;
			
			default:
			goto Error_Handler;
		}

	}
	else {
	
		handle_Size = GetHandleSize(port_Ptr->image_Handle);
		if (MemError() != noErr) {
			goto Error_Handler;
		}

		switch (dream.stage_View.special_View_Mode) {
		
			case k_FILL_DEFAULT_RESOURCE:
				fill_Data = 0;
			break;
			
			case k_FILL_SOLID_BLACK:
				fill_Data = 0;
			break;
			
			case k_FILL_CURRENT_CHROMA:
				fill_Data = g_Chroma_Index_Optimized_Long;
			break;
			
			case k_FILL_NORMAL:
				fill_Data = 0;
			break;
			
			default:
			goto Error_Handler;
		}

		Fill_Large_Buffer(*(port_Ptr->image_Handle)
					, handle_Size
					, fill_Data);

	}

	function_Result = SAFE_TRUE;
	goto Normal_Function_Exit;
	
Error_Handler:
	Display_Serious_Error_Message(unit_Name, code_Name, "\pProblem encountered", "\p");
	function_Result = SAFE_FALSE;
		
Normal_Function_Exit:
	if (port_Was_Set == SAFE_TRUE) {
		SetPenState (&saved_Pen_State);
		Set_View_Device(saved_Device);
		Set_View_Port(saved_Original_Port);
	}
	return(function_Result);
	
} /* Safe_Boolean	Force_Clear_Stage_To_Default_Data... */
/*=**************************************************************************=*/


/*=**************************************************************************=*/
/* END OF FILE: Stage_View_Manager.c */






