/* START OF FILE: Stage_View_Manager.h */


/*=**************************************************************************=*/
/* [---BEGIN---] FILE HEADER COMMENTS */
/**
 ** SCREEN VIEW = [Adobe Syntax Black-10, Tabs = 4]
 ** PRINTER VIEW = [Adobe Syntax Black-10, Tabs = 4]
 ** =======================================================================
 ** Written by James Merkle
 ** Stage_View_Manager.c - Routines to support Poor Man's Video Phone
 ** (C) Copyright 1994 James Merkle
 ** =======================================================================
 **
 **
 ** ============
 ** Revision History
 ** ============
 **
 **  Refer to "Stage_View_Manager.c"
 **						
 **
 ** ====================
 ** OTHER INFORMATION
 ** ====================
 ** 
 **
 **/
/* [----END----] FILE HEADER COMMENTS */
/*=**************************************************************************=*/



/*=**************************************************************************=*/
#ifndef _H_Stage_View_Manager

#define _H_Stage_View_Manager
/*=**************************************************************************=*/


/*=**************************************************************************=*/
/* [---BEGIN---] #include AREA */

#include "Shell_Globals.h"
#include "Off_Screen_Color.h"

/* [----END----] #include AREA */
/*=**************************************************************************=*/


/*=**************************************************************************=*/
/* [---BEGIN---] #define AREA */

/*  */
/* These are the display stages for the animation display buffer */
/*  */

#define k_DISPLAY_PHASE_IDLE_START_ERASE				0
#define k_DISPLAY_PHASE_ERASE_ACTIVE					1
#define k_DISPLAY_PHASE_READY_TO_COMPOSITE		2
#define k_DISPLAY_PHASE_COMPOSITING_ACTIVE			3
#define k_DISPLAY_PHASE_READY_TO_BLAST_PORT		4
#define k_DISPLAY_PHASE_BLASTING_PORT_ACTIVE		5
#define k_DISPLAY_REQUIRES_TOTAL_BUFFER_REBUILD				6
#define k_DISPLAY_REQUIRES_TOTAL_BUFFER_REDRAW				7
#define k_DISPLAY_REQUIRES_VISIBLE_BUFFER_REDRAW				8
#define k_DISPLAY_SYSTEM_TOTALLY_DAMAGED_UNREADY			9
#define k_DISPLAY_SYSTEM_TEMPORARY_FROZEN_AND_IDLE		10

#define k_ITEM_TYPE_SPIN_OBJECT					0
#define k_ITEM_TYPE_OLD_PUPPET_OBJECT			1
#define k_ITEM_TYPE_ARBITRARY_RECT_OBJECT	2

/*  */
/* Set to "1" if old style fast direct to screen non-compressed */
/* simple animation is desired. Normally the full blown 3D */
/* version is desired */
/*  */
#define k_ONLY_ALLOW_DIRECT_SCREEN_ANIMATION		0


/* [----END----] #define AREA */
/*=**************************************************************************=*/

                  
/*=**************************************************************************=*/
/* [---BEGIN---] TYPE AREA */


typedef struct {
	/*  */
	/* Composite cache key is intensity,horizontal_Angle,vertical_Angle, roll_Angle */
	/*  */
	CARDINAL_32	composite_Search_Key;
	
	/*  */
	/* Part of the key used in the cache data base is the */
	/* index of the spin file record itself. That is how it works */
	/* best. */
	/*  */
	CARDINAL_32	spin_Info_Address;
	
	CARDINAL_32	previous_Index;
	
	INTEGER_16		file_Reference;
} Compacted_Pict_Rec;


typedef struct {
	Ptr					existing_Puppet_Ptr;
	CARDINAL_16	activation_Frame;
	Point				scene_Offset;
} Region_Normal_Pict_Rec;

typedef struct {
	Rect			modified_Rect_Area;
} Painted_Area_Rec;

union Union_Position_Record{
	Compacted_Pict_Rec			compacted_Pict;
	Region_Normal_Pict_Rec	region_Pict;
	Painted_Area_Rec			arbitrary_Rectangle;
};

typedef struct {
	CARDINAL_16					item_Type;
	INTEGER_16						offset_X;
	INTEGER_16						offset_Y;
	CARDINAL_16						reserved_1;
	union Union_Position_Record		info;
} Item_Information, *Item_Information_Ptr;

typedef struct {
	CARDINAL_16		total_Remaining_Erasable_Items;
	Item_Information	item_To_Erase[k_MAX_STAGE_ITEMS];
} Item_Location_List, *Item_Location_List_Ptr;

typedef struct {
	Safe_Boolean		initialized;
	Safe_Boolean		stage_View_Buffers_Initialized;
		
	/*  */
	/* The composite area is exactly the same size as the static mix */
	/* area, for simplicity, but each part is not updated in full as often. */
	/* It is a classic two buffer sprite model, plus destination view. */
	/* There are actually two mix areas now this allows erasure of one */
	/* while the other is being rendered or heads are seeking or data */
	/* transferring */
	/*  */
	Off_Screen_Ptr				*mix_View_Area_Port_Hdl[k_BUFFERS_REQUIRED];
	Off_Screen_Ptr				mix_View_Area_Port_Ptr[k_BUFFERS_REQUIRED];
	Memory_Allocation			mix_View_Area_Port_Memory_Initialized[k_BUFFERS_REQUIRED];
	
	Item_Location_List			double_Buffered_Data[(k_BUFFERS_REQUIRED - k_BASE_OFFSET_FOR_MIX)];
	
	/*  */
	/* Structure indicating dirty portions needing updating to screen */
	/* would be placed here. */
	/*  */
	
	Rect				stage_Active_View_Rect;
	Rect				stage_View_Buffers_Total_Rect;
	CARDINAL_16	special_View_Mode;
	CARDINAL_16	display_Phase;
	CARDINAL_32	display_Minor_Phase;
	CARDINAL_16	next_Buffer_To_Use;
	CARDINAL_32	last_Visible_Buffer_Update_Request;
	Safe_Boolean		draw_System_Style_Update_Flag;
	
} Stage_View_Rec, *Stage_View_Ptr;

/* [----END----] TYPE AREA */
/*=**************************************************************************=*/


/*=**************************************************************************=*/
/* [---BEGIN---] PROTOTYPE AREA */

extern Safe_Boolean Initialize_Stage_View_Manager(void);

extern Safe_Boolean Reset_Stage_View_Manager_If_Needed(void);

extern Safe_Boolean Destroy_Stage_View_Manager(void);

extern Safe_Boolean	Force_Clear_Stage_To_Default_Data(Safe_Boolean visible_Portion_Only
											, 	Off_Screen_Ptr	port_Ptr);

extern Safe_Boolean Process_Animation_Tasks_For_Main_Stage(Stage_View_Ptr stage_View_Ptr);

extern Safe_Boolean	Add_Old_Puppet_To_Removal_List(Stage_View_Ptr stage_View
														, Ptr		existing_Puppet_Ptr
														, CARDINAL_16	activation_Frame
														, INTEGER_16 offset_X
														, INTEGER_16 offset_Y
														, Point scene_Offset);

extern Safe_Boolean	Add_Spin_Item_To_Removal_List(Stage_View_Ptr stage_View
														, CARDINAL_32	composite_Search_Key
														, CARDINAL_32	spin_Info_Address
														, CARDINAL_32	previous_Index
														, INTEGER_16	file_Reference
														, INTEGER_16 offset_X
														, INTEGER_16 offset_Y);

extern Safe_Boolean	Add_Rect_Area_To_Removal_List(Stage_View_Ptr stage_View
														, Rect	*rect_Area
														, INTEGER_16 offset_X
														, INTEGER_16 offset_Y);

extern Safe_Boolean Draw_Current_Stage_Background_Layers(Stage_View_Ptr stage_View
																, CARDINAL_32 mix_Buffer_To_Use
																, Safe_Boolean		*something_Changed_In_View);

extern Safe_Boolean Copy_Composition_Buffer_To_Screen(Stage_View_Ptr stage_View
															, CARDINAL_32 mix_Buffer_To_Use
															, GrafPtr	current_Graphics_Window_Ptr
															, INTEGER_32		offset_X
															, INTEGER_32		offset_Y);


/* [----END----] PROTOTYPE AREA */
/*=**************************************************************************=*/


/*=**************************************************************************=*/
/* [---BEGIN---] extern DECLARATIONS */


/* [----END----] extern DECLARATIONS */
/*=**************************************************************************=*/


/*=**************************************************************************=*/
/* [---BEGIN---] LOCAL STATIC DEFINITION AREA */


/* [----END----] LOCAL STATIC DEFINITION AREA */
/*=**************************************************************************=*/


/*=**************************************************************************=*/
#endif
/*=**************************************************************************=*/
/* END OF FILE: Stage_View_Manager.h */