/* START OF FILE: main.c */


/* [---BEGIN---] FILE HEADER COMMENTS */
/**
 ** SCREEN VIEW = [Adobe Syntax Black-10, Tabs = 6]
 ** PRINTER VIEW = [Adobe Syntax Black-10, Tabs = 6]
 ** =======================================================================
 ** Written by  Jim Merkle
 ** main.c - simple test environment. (Setup for Poor Man's Video Phone)
 ** (C) Copyright 1991 by Jim Merkle
 ** All rights reserved, except those wishing to "copy" moderate sections of code or files may do so, so long as
 ** such code is never used directly either in animation-related tools or multimedia tools that are  
 ** used in live performance or live broadcast or recorded mass market broadcast or film. Desktop usage not restricted.
 ** Conventional desktop game or entertainment not restricted.
 ** =======================================================================
 **
 **
 ** =============
 ** Revision History
 ** =============
 **
 **  1991.10.31		(James Merkle)
 **						Initial coding started 
 **
 **  1993.02.03		(James Merkle)
 **						Revised for  Poor Man's Video Phone
 **
 **
 ** ====================
 ** OTHER INFORMATION
 ** ====================
 **
 **/
/* [----END----] FILE HEADER COMMENTS */


/*=**************************************************************************=*/
/* [---BEGIN---] FILE INCLUDE AREA */

#include "Core_Globals.h"
#include "Core_Specific.h"
#include "Graphics_Primitives.h"
#include "Vertical_Blanking_Interrupt.h"

#include "OS_Extra_Utilities.h"

#include "Shell_Dialogs.h"
#include "Error_Utilities.h"
#include "Memory_Utilities.h"
#include "Initialize_Shell.h"
#include "Shell_Assembly.h"
#include "String_Utilities.h"
#include "Shell_Globals.h"
#include "Time_Utilities.h"

/* [----END----] FILE INCLUDE AREA */
/*=**************************************************************************=*/


/*=**************************************************************************=*/
/* [---BEGIN---] DEFINE AREA */

#define GG_COMMANDS_STRING_LIST_ID			529
#define WARNING_TEST_CODE_STRING_INDEX		1

/* [----END----] DEFINE AREA */
/*=**************************************************************************=*/


/*=**************************************************************************=*/
/* [---BEGIN---] TYPE AREA */

/* [----END----] TYPE AREA */
/*=**************************************************************************=*/


/*=**************************************************************************=*/
/* [---BEGIN---] PROTOTYPE AREA */

void Perform_Test_Arbitrary_Development_Code(void);
static Safe_Boolean Setup_Program_Menus(void);
Safe_Boolean Setup_Environment(void);


/* [----END----] PROTOTYPE AREA */
/*=**************************************************************************=*/


/*=**************************************************************************=*/
/* [---BEGIN---] GLOBAL DEFINITION AREA */

	
/* [----END----] GLOBAL DEFINITION AREA */
/*=**************************************************************************=*/


/*=**************************************************************************=*/
/* [---BEGIN---] LOCAL STATIC DEFINITION AREA */

static		 CARDINAL_8			*unit_Name = "\pmain";

/* [----END----] LOCAL STATIC DEFINITION AREA */
/*=**************************************************************************=*/


/*=**************************************************************************=*/
void Perform_Test_Arbitrary_Development_Code(void)
{
	Str255 	temp_String;
	Boolean	work_Boolean;
	
/* END VAR */

	Get_NS_String(GG_COMMANDS_STRING_LIST_ID, WARNING_TEST_CODE_STRING_INDEX, temp_String);
	if (Perform_Yes_No_Cancel_Dialog(temp_String, &work_Boolean) == TRUE) {
		if (work_Boolean == TRUE) {
			


			
		}
	}
					
	Normal_Function_Exit :
	
	; /* required in C */
}/* Perform_Test_Arbitrary_Development_Code */
/*=**************************************************************************=*/


/*=**************************************************************************=*/
Safe_Boolean Setup_Environment(void)
{
	CARDINAL_8		*code_Name;
	CARDINAL_32	current_Index;
	INTEGER_16		result_Code;
	INTEGER_16		index;
	SysEnvRec		system_Environment;
	Safe_Boolean		function_Result;
/* END VAR */

	code_Name = "\pSetup_Environment";
	function_Result = SAFE_FALSE;
	

	/*=--------------------------------------------------------------------=*/
	/* Init global variables */
	/*=--------------------------------------------------------------------=*/
	Reset_Global_Library_Values_To_Defaults(&g_Shell.library);
	
	/*=--------------------------------------------------------------------=*/
	/* Init global variables used by interface */
	/*=--------------------------------------------------------------------=*/
	Reset_Global_Interface_Values_To_Defaults(&g_Shell.interface);


	if (ns_Shell_Globals.has_Color_QuickDraw_Available != TRUE) {
		/*  */
		/* If no color QD, we must exit, because of direct video access (uncommon) */
		/*  */
		SysBeep(50);
		Perform_OK_Dialog("\pProgram Requires Color QuickDraw.  Program will abort.");	
		g_Shell.interface.abort_Program = TRUE;
		goto Error_Handler;
	}
	
	if (ns_Shell_Globals.system_7_Features_Available != TRUE) {
		/*  */
		/* If not system 7 full featured we need to cancel because of specialized color use */
		/*  */
		SysBeep(50);
		Perform_OK_Dialog("\pProgram Requires System 7 or greater due to color demands.  Program will abort.");	
		g_Shell.interface.abort_Program = TRUE;
		goto Error_Handler;
	}
	

	/*=--------------------------------------------------------------------=*/
	/* To make the Random sequences truly random, we need to make the seed start*/
	/*	at a different number.  An easy way to do this is to put the current time*/
	/*	and date into the seed.  Since it is always incrementing the starting seed*/
	/*	will always be different.  Dont for each call of Random, or the sequence*/
	/*	will no longer be random.  Only needed once, here in the init. */
	/*=--------------------------------------------------------------------=*/
	GetDateTime((CARDINAL_32 *)&(qd.randSeed));

	/*=--------------------------------------------------------------------=*/
	/*  Load and display the menubar (enabling items done elsewhere) */
	/*=--------------------------------------------------------------------=*/
	if (Setup_Program_Menus() != SAFE_TRUE) {
		goto Error_Handler;
	}
		
	DrawMenuBar();
	
	/*=--------------------------------------------------------------------=*/
	/*  Init global structures for direct video access (uncommon) */
	/*=--------------------------------------------------------------------=*/
	if (Create_Graphics_Primitives_Data_Structures() == FALSE) {
		goto Error_Handler;
	}
	
	/*=--------------------------------------------------------------------=*/
	/* Vital to initialize window list (nils are important)	*/
	/*=--------------------------------------------------------------------=*/
	for (index = 0; (index < (TOTAL_DEFINED_WINDOW_TYPE)); index++) {
		active_Window_List[index] = nil;
	}


	function_Result = SAFE_TRUE;
	goto END_OF_FUNCTION;

Error_Handler:
	function_Result = SAFE_FALSE;

END_OF_FUNCTION:
	return(function_Result);
	
} /* Setup_Environment */
/*=**************************************************************************=*/


/*=**************************************************************************=*/
static Safe_Boolean Setup_Program_Menus(void)
{
	CARDINAL_8		*code_Name;
	Safe_Boolean		function_Result;
	INTEGER_16		index;
/* END VAR */


	code_Name = "\pSetup_Program_Menus";
	function_Result = SAFE_FALSE;


	g_Shell.interface.main_Menu_Bar = GetNewMBar(SHELL_MENU_BAR_ID);
	if (g_Shell.interface.main_Menu_Bar == nil) {
		goto Missing_Resource_Error_Handler;
	}


	SetMenuBar(g_Shell.interface.main_Menu_Bar);
	/*=--------------------------------------------------------------------=*/
	/* SetMenuBar allocates is own memory copy, dispose of */
	/*  ours because theoretically no longer similar*/
	/*=--------------------------------------------------------------------=*/
	DisposHandle(g_Shell.interface.main_Menu_Bar);
	
	
	/*=--------------------------------------------------------------------=*/
	/* refetch a complete new memory copy of menu references */
	/*=--------------------------------------------------------------------=*/
	g_Shell.interface.main_Menu_Bar = GetMenuBar();
	if (g_Shell.interface.main_Menu_Bar == nil) {
		goto Error_Handler;
	}
	g_Shell.interface.main_Menu_Bar_Memory_In_Use = MEMORY_INITIALIZED;
	
	for (index =0; (index < MENU_COUNT); index++) {
		g_Shell.interface.program_Menus[index] = nil;
	}
	
	g_Shell.interface.program_Menus[APPLE_MENU] = GetMHandle(APPLE_MENU_ID);
	if (g_Shell.interface.program_Menus[APPLE_MENU] == nil) {
		goto Missing_Resource_Error_Handler;
	}
		
	AddResMenu(g_Shell.interface.program_Menus[APPLE_MENU], 'DRVR');

	g_Shell.interface.program_Menus[FILE_MENU] = GetMHandle(FILE_MENU_ID);
	if (g_Shell.interface.program_Menus[FILE_MENU] == nil) {
		goto Missing_Resource_Error_Handler;
	}
	
	g_Shell.interface.program_Menus[EDIT_MENU] = GetMHandle(EDIT_MENU_ID);
	if (g_Shell.interface.program_Menus[EDIT_MENU] == nil) {
		goto Missing_Resource_Error_Handler;
	}
			
	g_Shell.interface.program_Menus[SPECIAL_MENU] = GetMHandle(SPECIAL_MENU_ID);
	if (g_Shell.interface.program_Menus[SPECIAL_MENU] == nil) {
		goto Missing_Resource_Error_Handler;
	}
		
	g_Shell.interface.program_Menus[ITEMS_MENU] = GetMHandle(ITEMS_MENU_ID);
	if (g_Shell.interface.program_Menus[ITEMS_MENU] == nil) {
		goto Missing_Resource_Error_Handler;
	}
		
	g_Shell.interface.program_Menus[BACKDROPS_MENU] = GetMHandle(BACKDROPS_MENU_ID);
	if (g_Shell.interface.program_Menus[BACKDROPS_MENU] == nil) {
		goto Missing_Resource_Error_Handler;
	}
		
	g_Shell.interface.program_Menus[WINDOWS_MENU] = GetMHandle(WINDOWS_MENU_ID);
	if (g_Shell.interface.program_Menus[WINDOWS_MENU] == nil) {
		goto Missing_Resource_Error_Handler;
	}
		
	g_Shell.interface.program_Menus[AUDIO_MENU] = GetMHandle(AUDIO_MENU_ID);
	if (g_Shell.interface.program_Menus[AUDIO_MENU] == nil) {
		goto Missing_Resource_Error_Handler;
	}

	g_Shell.interface.program_Menus[NETWORK_MENU] = GetMHandle(NETWORK_MENU_ID);
	if (g_Shell.interface.program_Menus[NETWORK_MENU] == nil) {
		goto Missing_Resource_Error_Handler;
	}

	g_Shell.interface.program_Menus[SLIDE_MENU] = GetMHandle(SLIDE_MENU_ID);
	if (g_Shell.interface.program_Menus[SLIDE_MENU] == nil) {
		goto Missing_Resource_Error_Handler;
	}

	g_Shell.interface.program_Menus[DEBUG_MENU] = GetMHandle(DEBUG_MENU_ID);
	if (g_Shell.interface.program_Menus[DEBUG_MENU] == nil) {
		goto Missing_Resource_Error_Handler;
	}
		

	/*  
	
	GetResource('mctb', APPLE_MENU);
	SetMcInfo(.... 
	
	*/
	
	
	/*=--------------------------------------------------------------------=*/
	/* DrawMenuBar; is done elseware */
	/*=--------------------------------------------------------------------=*/
	
	function_Result = SAFE_TRUE;
	goto Normal_Function_Exit;
	
	
Missing_Resource_Error_Handler:
	Display_Serious_Error_Message(unit_Name, code_Name, "\pError due to missing menu resource", "\pDamage to resources (curious user, virus, etc.)");
	
Error_Handler:
	function_Result = SAFE_FALSE;

Normal_Function_Exit:

	return(function_Result);
	
} /* static Safe_Boolean Setup_Program_Menus(void) */
/*=**************************************************************************=*/


/*=**************************************************************************=*/
void main(void)
{
	INTEGER_16			stack_Based_Item_To_Measure;
	CARDINAL_32		desired_Stack;
/* END VAR */


/*=--------------------------------------------------------------------=*/
/* NOTE THAT IN CApplication, InitToolbox is not called, either is MaxApplZone ANY MORE !! */
/* in a TCL program SetApplLimit is done in "Class_Application::InitMemory" */
/* Otherwise initialization in non-TCL is as follows... */
/*=--------------------------------------------------------------------=*/

	InitGraf(&(qd.thePort));
	InitFonts();
	InitWindows();
	InitMenus();
	TEInit();
	InitDialogs((INTEGER_32)nil);
	InitCursor();
/*	FlushEvents(everyEvent, 0); */ /*== Think C "ANSI" does not call this ==*/
	
	ns_Shell_Globals.address_Of_Top_Of_Dynamic_Stack_Portion =  (CARDINAL_32)&stack_Based_Item_To_Measure;

	if (Load_Desired_Memory_Options(&ns_Shell_Globals.total_Dynamic_Stack_Desired_In_Options, &ns_Shell_Globals.total_Application_Heap_Desired_In_Options) != TRUE) {
		/*=--------------------------------------------------------------------------=*/
		/* the error from Load_Desired_Memory_Options reports can be */
		/* ignored since it sets default limits anyway, if resource is bad or missing */
		/*=--------------------------------------------------------------------------=*/
	}

	desired_Stack = ns_Shell_Globals.address_Of_Top_Of_Dynamic_Stack_Portion - ns_Shell_Globals.total_Dynamic_Stack_Desired_In_Options;
	if ((desired_Stack % 2) == 1)
		desired_Stack -= 1;
		
	SetApplLimit((Ptr)(desired_Stack));

	MaxApplZone();
	MoreMasters();
	MoreMasters();
	MoreMasters();
	MoreMasters();
	
	ns_Shell_Globals.address_Of_Top_Of_Dynamic_Stack_Portion =  (CARDINAL_32)&stack_Based_Item_To_Measure;


	g_Shell.interface.abort_Program = FALSE;
	
	if(Initialize_Shell() == FALSE) {
		goto Error_Handler;
	}


	if ( Setup_Environment() != SAFE_TRUE ) {
		Perform_OK_Dialog("\pProgram now quitting. Something Wrong with 'Setup_Environment'");	
		goto Application_Error_Handler;
	} /*  NOT if (Setup_Environment() == SAFE_TRUE)*/	

		
/*======
	Perform_Test_Arbitrary_Development_Code();
======= */	
	
	
	if (Initialize_Structures_Allowing_VBL_Slot_Monitoring() == FALSE) {
		Perform_OK_Dialog("\pProgram now quitting. Something Wrong with 'Initialize_Structures_Allowing_VBL_Slot_Monitoring'");	
		goto Application_Error_Handler;
	}
	
	
	Run_Mac_Version_Of_Program();

	
	if (Destroy_Structures_Allowing_VBL_Slot_Monitoring() == FALSE) {
		/* ignore error */
	}
	
	if (Destroy_Graphics_Primitives_Data_Structures() == FALSE) {
		/* ignore error */
	}
	
	
Application_Error_Handler:
	
	if(De_Initialize_Shell() == FALSE) {
		/* ignore error */
	}
	
Error_Handler:
; /* NULL*/
	
} /* void main(void) */
/*=**************************************************************************=*/


/*=**************************************************************************=*/

