///--------------------------------------------------------------------------------------
//	Frame.c
//
//	Created:	Sunday, November 15, 1992 at 2:37:37 AM
//	By:		Tony Myles
//
//	Copyright:  1991-93 Tony Myles, All rights reserved worldwide
//
//	Description:	implementation of the frame stuff
///--------------------------------------------------------------------------------------

#include "Interface_Utilities.h"

#ifndef __MEMORY__
#include <Memory.h>
#endif

#ifndef __RESOURCES__
#include <Resources.h>
#endif

#ifndef __ERRORS__
#include <Errors.h>
#endif

#include "Sprite_World_Common.h"

#ifndef __SPRITEWORLDUTILS__
#include "Sprite_World_Utilities.h"
#endif

#ifndef __FRAME__
#include "Sprite_World_Frame.h"
#endif


///--------------------------------------------------------------------------------------
//	SWCreateFrame
///--------------------------------------------------------------------------------------

SW_PASCAL OSErr SWCreateFrame(
	FramePtr* newFrameP,
	CGrafPtr srcGrafP,
	Rect* frameRect)
{
	OSErr err = noErr;
	FramePtr tempFrameP;
	short pixelSize;

	*newFrameP = NULL;

	tempFrameP = (FramePtr)NewPtrClear((Size)sizeof(FrameRec));

	if (tempFrameP != NULL)
	{
		tempFrameP->isColor = SWHasColorQuickDraw();

		if (srcGrafP == NULL)
		{
			if (tempFrameP->isColor)
			{
				err = SWCreateBestCGrafPort(&tempFrameP->framePort.colorGrafP, frameRect);
			}
			else
			{
				err = SWCreateGrafPort(&tempFrameP->framePort.monoGrafP, frameRect);
			}
		}
		else
		{
			tempFrameP->framePort.colorGrafP = srcGrafP;
		}

		if (err == noErr)
		{
			pixelSize = tempFrameP->isColor ?
					(**tempFrameP->framePort.colorGrafP->portPixMap).pixelSize : 1;			

			tempFrameP->frameRect = tempFrameP->framePort.colorGrafP->portRect;

			tempFrameP->frameRowBytes = tempFrameP->isColor ?
					(**tempFrameP->framePort.colorGrafP->portPixMap).rowBytes & 0x7FFF :
					tempFrameP->framePort.monoGrafP->portBits.rowBytes;

			tempFrameP->frameRowLongs = (tempFrameP->frameRect.right -
					tempFrameP->frameRect.left) >> 2;

				//  the following calculation assumes 32 bits in a long word! 
			tempFrameP->leftAlignFactor = ~((32 / pixelSize) - 1);
			tempFrameP->rightAlignFactor = (32 / pixelSize) - 1;

			*newFrameP = tempFrameP;
		}
	}
	else
	{
		err = MemError();
	}

	if (err != noErr)
	{
		if (tempFrameP != NULL)
		{
			if (tempFrameP->framePort.colorGrafP != NULL)
			{
				if (tempFrameP->isColor)
				{
					SWDisposeCGrafPort(tempFrameP->framePort.colorGrafP);
				}
				else
				{
					SWDisposeGrafPort((GrafPtr)tempFrameP->framePort.monoGrafP);
				}
			}

			DisposePtr((Ptr)tempFrameP);
		}
	}

	return err;
}


///--------------------------------------------------------------------------------------
//	SWCreateFrameFromCIconResource
///--------------------------------------------------------------------------------------

SW_PASCAL OSErr SWCreateFrameFromCIconResource(
	FramePtr* newFrameP,
	short iconResID,
	MaskType maskType)
{
	OSErr err;
	GrafPtr savePort;
	FramePtr tempFrameP;
	CIconHandle cIconH;
	RgnHandle maskRgn;
	Rect frameRect;

	*newFrameP = NULL;
	tempFrameP = NULL;
	cIconH = NULL;

	GetPort(&savePort);

	err = SWGetCIcon(&cIconH, iconResID);

	if (err == noErr)
	{
		HNoPurge((Handle)cIconH);

		frameRect = (**cIconH).iconPMap.bounds;

		err = SWCreateFrame(&tempFrameP, NULL, &frameRect);
	}

	if (err == noErr)
	{
		Set_View_Port(tempFrameP->framePort.monoGrafP);

		SWPlotCIcon(cIconH, &frameRect);
	}

	if (err == noErr)
	{
			// make a region mask
		if ((maskType & kRegionMask) != 0)
		{
			err = SWCreateRegionFromCIconMask(&maskRgn, cIconH);

			if (err == noErr)
			{
				SWSetFrameMaskRgn(tempFrameP, maskRgn);
			}
		}
	}

	if (err == noErr)
	{
			// make a pixel mask
		if ((maskType & kPixelMask) != 0)
		{
			if (tempFrameP->isColor)
			{
				err = SWCreateCGrafPortFromCIconMask(&tempFrameP->maskPort.colorGrafP, cIconH);
			}
			else
			{
				err = SWCreateGrafPortFromCIconMask(&tempFrameP->maskPort.monoGrafP, cIconH);
			}
		}
	}

	if (cIconH != NULL)
	{
		SWDisposeCIcon(cIconH);
	}

	if (err == noErr)
	{
		*newFrameP = tempFrameP;
	}

	if (err != noErr)
	{
			// an error occurred so dispose of anything we managed to create
		if (tempFrameP != NULL)
		{
			SWDisposeFrame(tempFrameP);
		}
	}

	Set_View_Port(savePort);

	return err;
}


///--------------------------------------------------------------------------------------
//	SWCreateFrameFromPictResource
///--------------------------------------------------------------------------------------

SW_PASCAL OSErr SWCreateFrameFromPictResource(
	FramePtr* newFrameP,
	short pictResID,
	short maskResID,
	MaskType maskType)
{
	OSErr err;
	PicHandle newPictH;
	FramePtr tempFrameP;
	RgnHandle maskRgn;
	Rect frameRect;

	tempFrameP = NULL;
	*newFrameP = NULL;

	newPictH = GetPicture(pictResID);

	if (newPictH != NULL)
	{
		frameRect.top = 0;
		frameRect.left = 0;
		frameRect.right = (**newPictH).picFrame.right - (**newPictH).picFrame.left;
		frameRect.bottom = (**newPictH).picFrame.bottom - (**newPictH).picFrame.top;

		err = SWCreateFrame(&tempFrameP, NULL, &frameRect);

		if (err == noErr)
		{
			Set_View_Port(tempFrameP->framePort.monoGrafP);

			DrawPicture(newPictH, &frameRect);
		}

			// make a region mask
		if (((maskType & kRegionMask) != 0) && (err == noErr))
		{
			err = SWCreateRegionFromPictResource(&maskRgn, maskResID);

			if (err == noErr)
			{
				SWSetFrameMaskRgn(tempFrameP, maskRgn);
			}
		}

			// make a pixel mask
		if (((maskType & kPixelMask) != 0) && (err == noErr))
		{
			if (tempFrameP->isColor)
			{
				err = SWCreateCGrafPortFromPictResource(&tempFrameP->maskPort.colorGrafP, maskResID);
			}
			else
			{
				err = SWCreateGrafPortFromPictResource(&tempFrameP->maskPort.monoGrafP, maskResID);
			}
		}

		if (err == noErr)
		{
			*newFrameP = tempFrameP;
		}
		else
		{
				// an error occurred so dispose of anything we managed to create
			if (tempFrameP != NULL)
			{
				SWDisposeFrame(tempFrameP);
			}
		}

		ReleaseResource((Handle)newPictH);
	}
	else
	{
		err = ResError();

		if (err == noErr)
		{
			err = resNotFound;
		}
	}

	return err;
}


///--------------------------------------------------------------------------------------
//	SWDisposeFrame
///--------------------------------------------------------------------------------------

SW_PASCAL void SWDisposeFrame(
	FramePtr oldFrameP)
{
	if (oldFrameP != NULL)
	{
		if (oldFrameP->framePort.colorGrafP != NULL)
		{
			if (oldFrameP->isColor)
			{
				SWDisposeCGrafPort(oldFrameP->framePort.colorGrafP);
			}
			else
			{
				SWDisposeGrafPort(oldFrameP->framePort.monoGrafP);
			}

			oldFrameP->framePort.colorGrafP = NULL;
		}

		if (oldFrameP->maskRgn != NULL)
		{
			DisposeRgn(oldFrameP->maskRgn);

			oldFrameP->maskRgn = NULL;
		}

		if (oldFrameP->maskPort.colorGrafP != NULL)
		{
			if (oldFrameP->isColor)
			{
				SWDisposeCGrafPort(oldFrameP->maskPort.colorGrafP);
			}
			else
			{
				SWDisposeGrafPort(oldFrameP->maskPort.monoGrafP);
			}

			oldFrameP->maskPort.colorGrafP = NULL;
		}

		DisposePtr((Ptr)oldFrameP);
	}
}


///--------------------------------------------------------------------------------------
//	SWSetFrameMaskRgn
///--------------------------------------------------------------------------------------

SW_PASCAL void SWSetFrameMaskRgn(
	FramePtr srcFrameP,
	RgnHandle maskRgn)
{
	srcFrameP->maskRgn = maskRgn;

	srcFrameP->offsetPoint.h = (**maskRgn).rgnBBox.left;
	srcFrameP->offsetPoint.v = (**maskRgn).rgnBBox.top;
}


///--------------------------------------------------------------------------------------
//	SWGetFrameMaskRgn
///--------------------------------------------------------------------------------------

SW_PASCAL RgnHandle SWGetFrameMaskRgn(
	FramePtr srcFrameP)
{
	return srcFrameP->maskRgn;
}


///--------------------------------------------------------------------------------------
//	SWLockFrame
///--------------------------------------------------------------------------------------

SW_PASCAL void SWLockFrame(
	FramePtr srcFrameP)
{
	srcFrameP->isFrameLocked = true;

	if (srcFrameP->isColor)
	{
		PixMapHandle pixMapH;

		pixMapH = srcFrameP->framePort.colorGrafP->portPixMap;
		HLock((Handle)pixMapH);

		srcFrameP->framePix.pixMapP = *pixMapH;
		srcFrameP->frameBaseAddr = (unsigned long *)(**pixMapH).baseAddr;

		if (srcFrameP->maskPort.colorGrafP != NULL)
		{
			pixMapH = srcFrameP->maskPort.colorGrafP->portPixMap;
			HLock((Handle)pixMapH);

			srcFrameP->isMaskLocked = true;

			srcFrameP->maskPix.pixMapP = *pixMapH;

			srcFrameP->maskBaseAddr = (unsigned long *)(**pixMapH).baseAddr;
		}
	}
	else
	{
		srcFrameP->framePix.bitMapP = &srcFrameP->framePort.monoGrafP->portBits;
		srcFrameP->frameBaseAddr = (unsigned long *)srcFrameP->framePix.bitMapP->baseAddr;

		if (srcFrameP->maskPort.monoGrafP != NULL)
		{
			srcFrameP->maskPix.bitMapP = &srcFrameP->maskPort.monoGrafP->portBits;
			srcFrameP->maskBaseAddr = (unsigned long *)srcFrameP->maskPix.bitMapP->baseAddr;
		}
	}
}


///--------------------------------------------------------------------------------------
//	SWUnlockFrame
///--------------------------------------------------------------------------------------

SW_PASCAL void SWUnlockFrame(
	FramePtr srcFrameP)
{
	if (srcFrameP->isColor)
	{
		if (srcFrameP->framePort.colorGrafP != NULL)
		{
			HUnlock((Handle)srcFrameP->framePort.colorGrafP->portPixMap);
		}

		if (srcFrameP->maskPort.colorGrafP != NULL)
		{
			HUnlock((Handle)srcFrameP->maskPort.colorGrafP->portPixMap);
		}
	}

	srcFrameP->isFrameLocked = false;
	srcFrameP->isMaskLocked = false;

	srcFrameP->framePix.pixMapP = NULL;
	srcFrameP->frameBaseAddr = NULL;
	srcFrameP->maskPix.pixMapP = NULL;
	srcFrameP->maskBaseAddr = NULL;
}


///--------------------------------------------------------------------------------------
//	SWCopyFrame
///--------------------------------------------------------------------------------------

SW_PASCAL void SWCopyFrame(
	FramePtr srcFrameP,
	FramePtr dstFrameP)
{
	if (srcFrameP->isFrameLocked)
	{
		CopyBits(srcFrameP->framePix.bitMapP,
					dstFrameP->framePix.bitMapP,
					&srcFrameP->frameRect,
					&srcFrameP->frameRect,
					srcCopy, srcFrameP->maskRgn);
	}
}


///--------------------------------------------------------------------------------------
//	SWSetFrameRect
///--------------------------------------------------------------------------------------

SW_PASCAL void SWSetFrameRect(
	FramePtr srcFrameP,
	Rect* newFrameRect)
{
	srcFrameP->frameRect = *newFrameRect;
	srcFrameP->frameRowLongs = (newFrameRect->right - newFrameRect->left) >> 2;
}


