/* START OF FILE: Shell_Globals.h */


/*=**************************************************************************=*/
/* [---BEGIN---] FILE HEADER COMMENTS */
/**
 ** SCREEN VIEW = [Adobe Syntax Black-10, Tabs = 6]
 ** PRINTER VIEW = [Adobe Syntax Black-10, Tabs = 6]
 ** =======================================================================
 ** NS_Shell, The Shell From Hell
 ** Written by Jim Wolff, and Jim Merkle
 ** Shell_Globals.h - Header for Shell_Globals.c
 ** current version (C) Copyright 1992 James Merkle
 ** =======================================================================
 **
 **
 ** ============
 ** Revision History
 ** ============
 **
 **  1991.11.14		(James Merkle)
 **						Added 'SCROLL_BAR_WIDTH'
 **
 **
 **
 ** =================
 ** OTHER INFORMATION
 ** =================
 ** 
 **
 **/
/* [----END----] FILE HEADER COMMENTS */
/*=**************************************************************************=*/


/*=**************************************************************************=*/
#ifndef _H_Shell_Globals

#define _H_Shell_Globals
/*=**************************************************************************=*/


/*=**************************************************************************=*/
/* [---BEGIN---] FILE INCLUDE AREA */



/* [----END----] FILE INCLUDE AREA */
/*=**************************************************************************=*/


/*=**************************************************************************=*/
/* [---BEGIN---] DEFINE AREA */

typedef enum {
	SAFE_FALSE = 23039,
	SAFE_TRUE
} Safe_Boolean;

/*for controlling the true sizes of the "short, int, and long" integer sizes */
#define INTEGER_8	char
#define INTEGER_16	short int
#define INTEGER_32	long 

/* not utilized within this program */
#define INTEGER_64		($ $ $ $)

#define CARDINAL_8	unsigned char
#define CARDINAL_16	unsigned short int
#define CARDINAL_32	unsigned long

/*  */
/* There are only 6 types of basic floating point types in */
/* computer science. These tradeoff two preferences of */
/* three qualities, speed, accuracy, storage space. */
/* Amazingly, the Mac provides all of these but most C */
/* compilers do not yet provide base types for them. */
/* I've been prepared (and been using) these for over 6 years */
/*  */
#define SPEED_DIGITS_REAL	double
#define SPEED_SPACE_REAL	double
#define DIGITS_SPEED_REAL	double
#define DIGITS_SPACE_REAL	double
#define SPACE_SPEED_REAL	float
#define SPACE_DIGITS_REAL	float

#define CARDINAL_8_MAX		255
#define CARDINAL_16_MAX	((unsigned int)0xFFFF)
#define CARDINAL_32_MAX	((unsigned long)0xFFFFFFFF)

/*  */
/* This must always allow at least 32767 and may be signed. */
/* It may be defined as 32 or 64 bit on RISC archetectures */
/* that are biased for iterative loops and indexing using 32 */
/* bit values. For all Motorola 68K use "INTEGER_16" */
/*  */
#define SIMPLE_LOOP	short int


#define INTEGER_8_MIN		(~127)
#define INTEGER_8_MAX		127
#define INTEGER_16_MIN		(~32767)
#define INTEGER_16_MAX		32767
#define INTEGER_32_MIN		(~2147483647)
#define INTEGER_32_MAX		2147483647


#define SYSTEM_0605			2
#define SCROLL_BAR_WIDTH	(16L)


/*=--------------------------------------------------------------------------=*/
/* ASCII control character definitions */
/*=--------------------------------------------------------------------------=*/
#define ASCII_NUL_NULL_IDLE								0x00
#define ASCII_SOH_START_OF_HEADING					0x01
#define ASCII_STX_START_OF_TEXT						0x02
#define ASCII_ETX_END_OF_TEXT							0x03
#define ASCII_EOT_END_OF_TRANSMISSION				0x04
#define ASCII_ENQ_ENQUIRY								0x05
#define ASCII_ACK_ACKNOWLEDGE						0x06
#define ASCII_BEL_BELL_BEEP_FLEEP						0x07
#define ASCII_BS_BACKSPACE								0x08
#define ASCII_HT_HORIZONTAL_TAB						0x09
#define ASCII_LF_LINE_FEED								0x0A
#define ASCII_VT_VERTICAL_TAB							0x0B
#define ASCII_FF_FORM_FEED_TOP_OF_PAGE			0x0C
#define ASCII_CR_CARRIAGE_RETURN						0x0D
#define ASCII_SO_SHIFT_OUT								0x0E
#define ASCII_SI_SHIFT_IN									0x0F
#define ASCII_DLE_DATA_LINK_ESCAPE					0x10
#define ASCII_DC1_DEVICE_CONTROL_1_XON			0x11
#define ASCII_DC2_DEVICE_CONTROL_2					0x12
#define ASCII_DC3_DEVICE_CONTROL_3_XOFF			0x13
#define ASCII_DC4_DEVICE_CONTROL_4					0X14
#define ASCII_NAK_NEGATIVE_ACKNOWLEDGE			0x15
#define ASCII_SYN_SYNCHRONOUS_IDLE					0x16
#define ASCII_ETB_END_OF_TRANSMISSION_BLOCK		0x17
#define ASCII_CAN_CANCEL								0x18
#define ASCII_EM_END_OF_MEDIUM						0x19
#define ASCII_SUB_SUBSTITUE								0x1A
#define ASCII_ESC_ESCAPE_PREFIX_ALTMODE			0x1B
#define ASCII_FS_FILE_SEPARATOR						0x1C
#define ASCII_GS_GROUP_SEPARATOR					0x1D
#define ASCII_RS_RECORD_SEPARATOR					0x1E
#define ASCII_US_UNIT_SEPARATOR						0x1F

#define ASCII_NULL										ASCII_NUL_NULL_IDLE
#define ASCII_START_OF_HEADING					ASCII_SOH_START_OF_HEADING
#define ASCII_START_OF_TEXT							ASCII_STX_START_OF_TEXT
#define ASCII_END_OF_TEXT							ASCII_ETX_END_OF_TEXT
#define ASCII_END_OF_TRANSMISSION				ASCII_EOT_END_OF_TRANSMISSION
#define ASCII_ENQUIRY									ASCII_ENQ_ENQUIRY
#define ASCII_ACKNOWLEDGE							ASCII_ACK_ACKNOWLEDGE
#define ASCII_BELL_BEEP_FLEEP						ASCII_BEL_BELL_BEEP_FLEEP
#define ASCII_BACKSPACE								ASCII_BS_BACKSPACE
#define ASCII_HORIZONTAL_TAB						ASCII_HT_HORIZONTAL_TAB
#define ASCII_LINE_FEED								ASCII_LF_LINE_FEED
#define ASCII_VERTICAL_TAB							ASCII_VT_VERTICAL_TAB
#define ASCII_FORM_FEED_TOP_OF_PAGE			ASCII_FF_FORM_FEED_TOP_OF_PAGE
#define ASCII_CARRIAGE_RETURN						ASCII_CR_CARRIAGE_RETURN
#define ASCII_SHIFT_OUT								ASCII_SO_SHIFT_OUT
#define ASCII_SHIFT_IN									ASCII_SI_SHIFT_IN
#define ASCII_DATA_LINK_ESCAPE						ASCII_DLE_DATA_LINK_ESCAPE
#define ASCII_DEVICE_CONTROL_1_XON				ASCII_DC1_DEVICE_CONTROL_1_XON
#define ASCII_DEVICE_CONTROL_2					ASCII_DC2_DEVICE_CONTROL_2
#define ASCII_DEVICE_CONTROL_3_XOFF				ASCII_DC3_DEVICE_CONTROL_3_XOFF
#define ASCII_DEVICE_CONTROL_4					ASCII_DC4_DEVICE_CONTROL_4
#define ASCII_NEGATIVE_ACKNOWLEDGE			ASCII_NAK_NEGATIVE_ACKNOWLEDGE
#define ASCII_SYNCHRONOUS_IDLE					ASCII_SYN_SYNCHRONOUS_IDLE
#define ASCII_END_OF_TRANSMISSION_BLOCK		ASCII_ETB_END_OF_TRANSMISSION_BLOCK
#define ASCII_CANCEL									ASCII_CAN_CANCEL
#define ASCII_END_OF_MEDIUM						ASCII_EM_END_OF_MEDIUM
#define ASCII_SUBSTITUE								ASCII_SUB_SUBSTITUE
#define ASCII_ESCAPE_PREFIX_ALTMODE				ASCII_ESC_ESCAPE_PREFIX_ALTMODE
#define ASCII_FILE_SEPARATOR						ASCII_FS_FILE_SEPARATOR
#define ASCII_GROUP_SEPARATOR					ASCII_GS_GROUP_SEPARATOR
#define ASCII_RECORD_SEPARATOR					ASCII_RS_RECORD_SEPARATOR
#define ASCII_UNIT_SEPARATOR						ASCII_US_UNIT_SEPARATOR

/*=--------------------------------------------------------------------------=*/
/* Alternate ASCII control character definitions */
/*=--------------------------------------------------------------------------=*/
#define ASCII_CR							ASCII_CR_CARRIAGE_RETURN

/*=--------------------------------------------------------------------------=*/
/* Other ASCII character definitions */
/*=--------------------------------------------------------------------------=*/
#define ASCII_SPACE					0x20
#define ASCII_DOUBLE_QUOTE			0x22
#define ASCII_OCTOTHORP			0x23
#define ASCII_DOLLAR					0x24
#define ASCII_LEFT_PARENTHESIS		0x28
#define ASCII_RIGHT_PARENTHESIS	0x29
#define ASCII_COMMA					0x2C
#define ASCII_HYPHEN					0x2D
#define ASCII_PERIOD					0x2E
#define ASCII_ZERO						0x30
#define ASCII_NINE						0x39
#define ASCII_AT							0x40
#define ASCII_LESS_THAN				0x3C
#define ASCII_GREATER_THAN			0x3E
#define ASCII_LEFT_BRACKET			0x5B
#define ASCII_RIGHT_BRACKET			0x5D
#define ASCII_LEFT_BRACE				0x7B
#define ASCII_VERTICAL_BAR			0x7C
#define ASCII_RIGHT_BRACE			0x7D

/*=--------------------------------------------------------------------------=*/
/* Keyboard character definitions */
/*=--------------------------------------------------------------------------=*/
#define ENTER_KEY						ASCII_ETX_END_OF_TEXT
#define LEFT_ARROW_KEY				ASCII_FS_FILE_SEPARATOR
#define RIGHT_ARROW_KEY			ASCII_GS_GROUP_SEPARATOR
#define UP_ARROW_KEY				ASCII_RS_RECORD_SEPARATOR
#define DOWN_ARROW_KEY			ASCII_US_UNIT_SEPARATOR

#define RETURN_KEYBOARD_CODE					0x24
#define ENTER_KEYBOARD_CODE						0x4C
#define TILDE_KEYBOARD_CODE						0x32
#define ESCAPE_KEYBOARD_CODE					0x35
/*=--------------------------------------------------------------------------=*/
/* SHIFT_OR_LEFT_SHIFT_KEYBOARD_CODE is always 38, right shift */
/* can sometimes be 3C only on extended keyboard */
/*=--------------------------------------------------------------------------=*/
#define SHIFT_OR_LEFT_SHIFT_KEYBOARD_CODE 	0x38
#define OPTION_KEYBOARD_CODE 					0x3A
#define k_KEYCODE_COMMAND 						0x37
#define CONTROL_KEYBOARD_CODE 					0x3B


/*=--------------------------------------------------------------------------=*/
/* Numeric key scan codes. */
/*=--------------------------------------------------------------------------=*/
#define k_KEYCODE_1				0x12
#define k_KEYCODE_2				0x13
#define k_KEYCODE_3				0x14
#define k_KEYCODE_4				0x15
#define k_KEYCODE_5				0x17
#define k_KEYCODE_6				0x16
#define k_KEYCODE_7				0x1A
#define k_KEYCODE_8				0x1C
#define k_KEYCODE_9				0x19
#define k_KEYCODE_0				0x1D

/*=--------------------------------------------------------------------------=*/
/* other scan codes (useful for physical arrangements regardless of keyboard type). */
/*=--------------------------------------------------------------------------=*/

#define k_KEYCODE_ASCII_HYPHEN	0x1B

#define k_KEYCODE_ASCII_Q		0x0C
#define k_KEYCODE_ASCII_W		0x0D
#define k_KEYCODE_ASCII_E		0x0E
#define k_KEYCODE_ASCII_R		0x0F
#define k_KEYCODE_ASCII_T		0x11
#define k_KEYCODE_ASCII_Y		0x10
#define k_KEYCODE_ASCII_U		0x20
#define k_KEYCODE_ASCII_I			0x22
#define k_KEYCODE_ASCII_O		0x1F
#define k_KEYCODE_ASCII_P		0x23
#define k_KEYCODE_ASCII_A		0x00
#define k_KEYCODE_ASCII_S		0x01
#define k_KEYCODE_ASCII_D		0x02
#define k_KEYCODE_ASCII_F		0x03
#define k_KEYCODE_ASCII_G		0x05
#define k_KEYCODE_ASCII_H		0x04
#define k_KEYCODE_ASCII_J		0x26
#define k_KEYCODE_ASCII_K		0x28
#define k_KEYCODE_ASCII_L		0x25
#define k_KEYCODE_ASCII_Z		0x06
#define k_KEYCODE_ASCII_X		0x07
#define k_KEYCODE_ASCII_C		0x08
#define k_KEYCODE_ASCII_V		0x09
#define k_KEYCODE_ASCII_B		0x0B
#define k_KEYCODE_ASCII_N		0x2D
#define k_KEYCODE_ASCII_M		0x2E
#define k_KEYCODE_COMMA			0x2B
#define k_KEYCODE_ASCII_PERIOD	0x2F
#define k_KEYCODE_ASCII_P			0x23
#define k_KEYCODE_ASCII_SPACE		0x31


#define KEY_CODE_FOR_OVERRIDING_SAFETY	OPTION_KEYBOARD_CODE


#define NS_SHELL_STRING_LIST_ID												1001
#define QUESTION_STRING_INDEX														1
#define CHOICE_A_STRING_INDEX														2
#define CHOICE_B_STRING_INDEX														3
#define INVALID_RANGE_STRING_INDEX												4
#define FREE_MEMORY_STRING_INDEX												5
#define DATE_STRING_INDEX															6
#define TIME_STRING_INDEX															7
#define LOW_MEM_BYTES_NEEDED_1ST_STRING_INDEX							8
#define LOW_MEM_BYTES_NEEDED_2ND_STRING_INDEX							9
#define LOW_MEM_HELP_COMMENT_STRING_INDEX								10
#define IMPROPER_MEMORY_STRUCTURES_NOT_INITIALIZED						11
#define PROGRAMMATICAL_ERROR_STRING_INDEX								12


	/*  */
	/*  */
	/*  */
	/*  */
	/*  */
	/*  */



/* These are used with the Options_Type structure (typedef'd below) in order to interpt the */
/* options_Usage_Flags_1 field correctly. */

#define USAGE_FLAG_1_USE_BUFFERED_IO_BIT		7
#define USAGE_FLAG_1_USES_TCL_ROUTINES_BIT	6
#define USAGE_FLAG_1_USE_MOTIF_BIT				5
#define USAGE_FLAG_1_GLOBALS_ARE_A4_BIT		4
#define USAGE_FLAG_1_RESERVED_1_BIT				3
#define USAGE_FLAG_1_RESERVED_2_BIT				2
#define USAGE_FLAG_1_RESERVED_3_BIT				1
#define USAGE_FLAG_1_RESERVED_4_BIT				0

#define USAGE_FLAG_1_USE_BUFFERED_IO_MASK		0x80
#define USAGE_FLAG_1_USES_TCL_ROUTINES_MASK	0x40
#define USAGE_FLAG_1_USE_MOTIF_MASK				0x20
#define USAGE_FLAG_1_GLOBALS_ARE_A4_MASK		0x10
#define USAGE_FLAG_1_RESERVED_1_MASK				0x08
#define USAGE_FLAG_1_RESERVED_2_MASK				0x04
#define USAGE_FLAG_1_RESERVED_3_MASK				0x02
#define USAGE_FLAG_1_RESERVED_4_MASK				0x01


/*  */
/* Place definitions of these here because not proper part of Mac */
/* Toolbox. */
/*  */
enum {
	k_OK_Item = 1,
	k_Cancel_Item
};


#if (defined(__MWERKS__))
#define k_MW_ASM	asm
#else 
#define k_MW_ASM
#endif

	/*  */
	/*  */
	/*  */
	/*  */
	/*  */
	/*  */

	/*  */
	/* When compiled for direct CPU access in THINK we must set */
	/* this to 1 instead of "0" */
	/*  */
#define k_ENABLE_DIRECT_FPU_ACCESS			0


/*  */
/* Normally set to "1" to indicate usage of "k_DEFAULT_256_COLOR_SYSTEM_CLUT" */
/*  */
#define k_USE_DEFAULT_CLUT_RESOURCE			1
#define k_DEFAULT_256_COLOR_SYSTEM_CLUT		1602

#define k_DEFAULT_2_COLOR_SYSTEM_CLUT		1604



	/*  */
	/* Allows faster PICT file reading */
	/*  */
#define k_USE_BUFFERED_FILE_IO		1

	/*  */
	/* Set to "1" if pointers to functions that can be placed in traditional */
	/* Mac OS structures and no Mixed Mode Manager is needed at all. */
	/* (On full PowerPC you need to abandon traditional */
	/* Mac system for jumping and need to create memory structures */
	/* for each jump and need to have then eventually be discarded either */
	/* explicitely or by having heap closed.) */
	/* Code fragments in 68K are pathetic because loading and unloading */
	/* code resources becomes less friendly and system gets pigged down and */
	/* a jump takes about 5 times as long to complete */
	/*  */
#define ORIGINAL_MAC_SYSTEM			1

	/*  */
	/* Set to "0" if compiling in a driver, code fragment, component, etc. */
	/* This will suppress error messages in utility routines and */
	/* possibly compile them out. */
	/*  */
#define k_USER_ERROR_INTERFACE_ALLOWED		1


#define GLOBALS_ARE_A4_RELATIVE		0
#define k_USES_A5_FOR_GLOBALS		1

#define PROGRAM_MINIMUM_MEMORY_TO_PRESERVE	(100UL * 1024UL)


/*  */
/* Leave about 32K on volumes at all times */
/*  */
#define	MINIMUM_DISK_SPACE_TO_PRESERVE	(32768UL)

/*  */
/* For interprogram use. This should match APPL creator ID */
/*  */
#define FILE_CREATOR				'jDoD'

	/*  */
	/* This does not need to be set if appication is meant to be small as possible. */
	/* It is for picture buffer. */
	/*  */
#define k_USE_RING_BUFFER_FOR_STORAGE		1



/* [----END----] DEFINE AREA */
/*=**************************************************************************=*/


/*=**************************************************************************=*/
/* [---BEGIN---] TYPE AREA */

typedef struct {
	CARDINAL_32	options_Version;
	CARDINAL_32	options_Additional_Stack_Space;
	CARDINAL_32	options_Minimum_Heap;
	CARDINAL_32	options_Applications_Use_1;
	CARDINAL_32	options_Applications_Use_2;
	CARDINAL_8		options_Usage_Flags_1;	
} Options_Type, *Options_Type_Ptr, **Options_Type_Hdl;

typedef struct {
	Options_Type	operating_Conditions;
	
	INTEGER_16	system_Volume_Reference;
	INTEGER_16	system_Resource_Reference;
	INTEGER_16	application_Volume_Reference;
	INTEGER_16	application_Directory_Reference;
	INTEGER_16	application_Resource_Reference;
	INTEGER_32	application_Directory_ID;
	
	OSErr			os_Error;
	SysEnvRec	system_Environment;
	
	Str255			error_Description_String;
	Str255			error_Note_String;
	
	Boolean		abort_Program;
	Boolean		gestalt_Available;
	Boolean		has_Color_QuickDraw_Available;
	Boolean		has_FPU_Available;
	Boolean		system_7_Features_Available;
	Boolean		temporary_Memory_Supported;
	Boolean		temporary_Memory_Is_Real;
	Boolean		temporary_Memory_Tracked;
	Boolean		memory_Dispatch_Routines_Available;
	Boolean		wait_Next_Event_Available; 
	Boolean		cpu_Requires_Even_Data_Bus_Address;
	Boolean		has_Real_68K_Available;
	
	CARDINAL_32	address_Of_Top_Of_Dynamic_Stack_Portion;
	CARDINAL_32	total_Dynamic_Stack_Desired_In_Options;
	CARDINAL_32	total_Application_Heap_Desired_In_Options;
	CARDINAL_32	ticks_Since_Last_Fatal_Error_Dialog;
} Shell_Globals_Rec;

/*=--------------------------------------------------------------------------=*/
/* Had to move to this enum here to avoid requiring superfluous */
/* additions of "Memory_Utilities.h" all the time */
/*=--------------------------------------------------------------------------=*/
typedef enum {
	MEMORY_NOT_INITIALIZED = SAFE_FALSE,
	MEMORY_INITIALIZED = SAFE_TRUE
} Memory_Allocation;


/* [----END----] TYPE AREA */
/*=**************************************************************************=*/


/*=**************************************************************************=*/
/* [---BEGIN---] PROTOTYPE AREA */

/* [----END----] PROTOTYPE AREA */
/*=**************************************************************************=*/


/*=**************************************************************************=*/
/* [---BEGIN---] EXTERN DECLARATION AREA */

extern Shell_Globals_Rec	ns_Shell_Globals;

/* [----END----] EXTERN DECLARATION AREA */
/*=**************************************************************************=*/


/*=**************************************************************************=*/
/* [---BEGIN---] LOCAL STATIC DEFINITION AREA */

/* [----END----] LOCAL STATIC DEFINITION AREA */
/*=**************************************************************************=*/


/*=**************************************************************************=*/
#endif
/*=**************************************************************************=*/


/* END OF FILE: Shell_Globals.h */