/* START OF FILE: Command_Dialogs.c */


/*=**************************************************************************=*/
/* [---BEGIN---] FILE HEADER COMMENTS */
/**
 ** SCREEN VIEW = [Adobe Syntax Black-10, Tabs = 4]
 ** PRINTER VIEW = [Adobe Syntax Black-10, Tabs = 4]
 ** =======================================================================
 ** Written by James Merkle
 ** Command_Dialogs.c - Interactive Dialogs for parameter setup in commands
 ** (C) Copyright 1994 James Merkle
 ** All rights reserved, except those wishing to "copy" moderate sections of code or files may do so, so long as
 ** such code is never used directly either in animation-related tools or multimedia tools that are  
 ** used in live performance or live broadcast or recorded mass market broadcast or film. Desktop usage not restricted.
 ** Conventional desktop game or entertainment not restricted.
 ** =======================================================================
 **
 **
 ** ============
 ** Revision History
 ** ============
 **
 **  1994.02.01		(James Merkle)
 **						Initial coding started
 **
 **
 **
 **
 ** =================
 ** OTHER INFORMATION
 ** =================
 **  
 **
 **/
/* [----END----] FILE HEADER COMMENTS */
/*=**************************************************************************=*/


/*=**************************************************************************=*/
/* [---BEGIN---] #include AREA */

#include "Dreaming_Popup_Dialogs.h"
#include "Shell_Assembly.h"
#include "Automatic_Dialogs.h"
#include "String_Utilities.h"
#include "Error_Utilities.h"
#include "Shell_Dialogs.h"
#include "Dreaming_Color_Stuff.h"
#include "Dreaming_Backdrop_Files.h"
#include "Spin_O_Rama.h"
#include "Dreaming_Scene_Files.h"
#include "Video_Backdrop_Background.h"
#include "String_Utilities.h"
#include "Object_Inventory.h"
#include "Slide_Show_Routines.h"
#include "Spin_O_Rama_Files.h"


#include "Command_Dialogs.h"

/* [----END----] #include AREA */
/*=**************************************************************************=*/


/*=**************************************************************************=*/
/* [---BEGIN---] #define AREA */

#define GET_NEW_PUPPET_DIALOG_ID				7000
#define GET_NEW_PUPPET_OUTLINE_ITEM			10

#define IMPORT_OPTIONS_DIALOG_ID				7001
#define IMPORT_OPTIONS_OUTLINE_ITEM			76

#define k_FILE_NAME_MAXIMUM						31
#define k_TINY_NICKNAME_NAME_MAXIMUM		14


#define k_UNTITLED_NAME_STRING					"\pUntitled"

/* [----END----] #define AREA */
/*=**************************************************************************=*/


/*=**************************************************************************=*/
/* [---BEGIN---] TYPE AREA */

/*  */
/* Shared by Puppet and Scene Name editing */
/*  */
typedef struct {
	Item_Variables_Header	header;
	Edit_Text_Item			input_Text;
	Edit_Text_Item			descriptive_Name_Text;
	Edit_Text_Item			file_Name_Text;
	Edit_Text_Item			tiny_Name_Text;
	User_Item					cancel_Button_Outline;
} Get_New_Scene_Item_Variables_Record;


typedef struct {
	Item_Variables_Header	header;
	Boolean_Item			use_Special_Colors;
	Boolean_Item			import_Background;
	Boolean_Item			compress_To_256_Colors;
	Boolean_Item			ask_For_Special_CLUT;
	
	Boolean_Item			mode_1_Emotion;
	Boolean_Item			mode_2_Emotion;
	Boolean_Item			mode_3_Emotion;
	Boolean_Item			mode_4_Emotion;
	Boolean_Item			mode_5_Emotion;
	
	Edit_Text_Item			mode_1_Emotion_Text;
	Edit_Text_Item			mode_2_Emotion_Text;
	Edit_Text_Item			mode_3_Emotion_Text;
	Edit_Text_Item			mode_4_Emotion_Text;
	Edit_Text_Item			mode_5_Emotion_Text; /* 17 */

	Boolean_Item			simple_Position_Vertical;
	Boolean_Item			multiple_Position_Vertical;
	Boolean_Item			custom_Position_Vertical;
	Boolean_Item			c1112_Position_Vertical;
	Boolean_Item			c112_Position_Vertical;
	Boolean_Item			c12_Position_Vertical;
	Boolean_Item			c1_Position_Vertical;
	Boolean_Item			c2_Position_Vertical;
	Boolean_Item			c4_Position_Vertical;
	Edit_Text_Item			simple_Position_Vertical_Value; /* 27 */
	Edit_Text_Item			multiple_Position_Stop_Vertical_Value;
	Edit_Text_Item			multiple_Position_Start_Vertical_Value;
	Edit_Text_Item			custom_Position_Vertical_Center_Value;
	Edit_Text_Item			custom_Position_Vertical_1_Value;
	Edit_Text_Item			custom_Position_Vertical_2_Value;
	Edit_Text_Item			custom_Position_Vertical_3_Value;
	Edit_Text_Item			custom_Position_Vertical_4_Value;
	Edit_Text_Item			custom_Position_Vertical_5_Value; /* 35 */


	Boolean_Item			simple_Position_Horizontal; /* 36 (33rd item) */
	Boolean_Item			multiple_Position_Horizontal;
	Boolean_Item			custom_Position_Horizontal;
	Boolean_Item			c1112_Position_Horizontal;
	Boolean_Item			c112_Position_Horizontal;
	Boolean_Item			c12_Position_Horizontal;
	Boolean_Item			c1_Position_Horizontal;
	Boolean_Item			c2_Position_Horizontal;
	Boolean_Item			c4_Position_Horizontal;
	
	Edit_Text_Item			simple_Position_Horizontal_Value;
	Edit_Text_Item			multiple_Position_Stop_Horizontal_Value;
	Edit_Text_Item			multiple_Position_Start_Horizontal_Value;
	Edit_Text_Item			custom_Position_Horizontal_Center_Value;
	Edit_Text_Item			custom_Position_Horizontal_1_Value;
	Edit_Text_Item			custom_Position_Horizontal_2_Value;
	Edit_Text_Item			custom_Position_Horizontal_3_Value;
	Edit_Text_Item			custom_Position_Horizontal_4_Value;
	Edit_Text_Item			custom_Position_Horizontal_5_Value;


	Boolean_Item			simple_Position_Roll; /* 54 (51st item) */
	Boolean_Item			multiple_Position_Roll;
	Boolean_Item			custom_Position_Roll; /* 56 */
	Boolean_Item			c1112_Position_Roll;
	Boolean_Item			c112_Position_Roll;
	Boolean_Item			c12_Position_Roll;
	Boolean_Item			c1_Position_Roll;
	Boolean_Item			c2_Position_Roll;
	Boolean_Item			c4_Position_Roll;
	
	Edit_Text_Item			simple_Position_Roll_Value;
	Edit_Text_Item			multiple_Position_Stop_Roll_Value;
	Edit_Text_Item			multiple_Position_Start_Roll_Value;
	Edit_Text_Item			custom_Position_Roll_Center_Value;
	Edit_Text_Item			custom_Position_Roll_1_Value;
	Edit_Text_Item			custom_Position_Roll_2_Value;
	Edit_Text_Item			custom_Position_Roll_3_Value;
	Edit_Text_Item			custom_Position_Roll_4_Value;
	Edit_Text_Item			custom_Position_Roll_5_Value;
	
	Edit_Text_Item			number_Of_PICTS_String; /* 83 */
	User_Item					cancel_Button_Outline; /* 76 (70th item) */
} Import_Options_Item_Variables_Record;


/* [----END----] TYPE AREA */
/*=**************************************************************************=*/


/*=**************************************************************************=*/
/* [---BEGIN---] PROTOTYPE AREA */
																							
Boolean Get_New_Scene_Information(CARDINAL_8 *scene_File_Name
								, CARDINAL_8 *scene_Short_Name
								, CARDINAL_8 *scene_Description);

Safe_Boolean Select_Scene_Size(Rect	*scene_Size);

Safe_Boolean Initialize_Defaults_For_Import(Import_Options_Ptr	options);

Safe_Boolean Validate_And_Copy_Contents(Import_Options_Item_Variables_Record *items
													, Import_Options_Ptr options);


/* [----END----] PROTOTYPE AREA */
/*=**************************************************************************=*/


/*=**************************************************************************=*/
/* [---BEGIN---] GLOBAL DEFINITION AREA */


/* [----END----] GLOBAL DEFINITION AREA */
/*=**************************************************************************=*/


/*=**************************************************************************=*/
/* [---BEGIN---] LOCAL STATIC DEFINITION AREA */

static CARDINAL_8	*unit_Name;

/* [----END----] LOCAL STATIC DEFINITION AREA */
/*=**************************************************************************=*/


/*=**************************************************************************=*/
Safe_Boolean Initialize_Command_Dialogs(void)
{
	CARDINAL_8			*static_Unit_Name = "\pCommand_Dialogs";
/* END VAR */

/*  */
/* Global persistant structures are created here */
/*  */

	unit_Name = static_Unit_Name;
	return(SAFE_TRUE);
}
/*=**************************************************************************=*/


/*=**************************************************************************=*/
Safe_Boolean Destroy_Command_Dialogs(void)
{
	Safe_Boolean		function_Result;
/* END VAR */

/*  */
/* Global persistant structures are deleted here */
/*  */

	function_Result = SAFE_TRUE;
	return(function_Result);
}
/*=**************************************************************************=*/


/*=**************************************************************************=*/
Safe_Boolean Select_Scene_Size(Rect	*scene_Size)
{
	Safe_Boolean		function_Result;
/* END VAR */
	
	
	function_Result = SAFE_TRUE;
		
	/*  */
	/* HACK HACK HACK : Ask user for size */
	/*  */
	SetRect(scene_Size, 0, 0, 640, 390);
	
	return(function_Result);
}
/*=**************************************************************************=*/


/*=**************************************************************************=*/
Boolean Get_New_Puppet_Information(CARDINAL_8 *puppet_File_Name
								, CARDINAL_8 *puppet_Short_Name
								, CARDINAL_8 *puppet_Description)
{
	Str255			message;
	Boolean		function_Result;
/* END VAR */

	Copy_Pascal_String(&(k_UNTITLED_NAME_STRING[0]), puppet_File_Name);
	Copy_Pascal_String(&(k_UNTITLED_NAME_STRING[0]), puppet_Short_Name);
	Copy_Pascal_String(&(k_UNTITLED_NAME_STRING[0]), puppet_Description);

	Get_NS_String(k_DREAM_STRINGS, k_ENTER_PUPPET_NAMES_PROMPT, message);

	function_Result = Edit_Fancy_Filename_Information(message
								, puppet_File_Name
								, puppet_Short_Name
								, puppet_Description);
																
	return(function_Result);						
	
} /* Boolean Get_New_Puppet_Information ... */
/*=**************************************************************************=*/


/*=**************************************************************************=*/
Boolean Get_New_Scene_Information(CARDINAL_8 *scene_File_Name
								, CARDINAL_8 *scene_Short_Name
								, CARDINAL_8 *scene_Description)
{
	Str255			message;
	Boolean		function_Result;
/* END VAR */


	Copy_Pascal_String(&(k_UNTITLED_NAME_STRING[0]), scene_File_Name);
	Copy_Pascal_String(&(k_UNTITLED_NAME_STRING[0]), scene_Short_Name);
	Copy_Pascal_String(&(k_UNTITLED_NAME_STRING[0]), scene_Description);

	function_Result = TRUE;
/* ====	
	Get_NS_String(k_DREAM_STRINGS, k_ENTER_SCENE_NAMES_PROMPT, message);

	function_Result = Edit_Fancy_Filename_Information(message
								, scene_File_Name
								, scene_Short_Name
								, scene_Description);
==== */
								
	return(function_Result);						
	
} /* Boolean Get_New_Scene_Information ... */
/*=**************************************************************************=*/


/*=**************************************************************************=*/
Safe_Boolean Initialize_Defaults_For_Import(Import_Options_Ptr	options)
{

	options->use_Special_Colors = FALSE;
	options->import_Background = FALSE;
	options->compress_To_256_Colors = FALSE;
	options->ask_For_Special_CLUT = FALSE;
	
	options->mode_Emotion_Value = 0;
	options->mode_Emotion_Str255[0] = 0;
	Copy_Pascal_String("\pDefault", options->mode_Emotion_Str255);
	
	options->position_Type_Vertical = 0;
	
	options->simple_Position_Vertical_Value = 180;
	options->multiple_Position_Stop_Vertical_Value = 180;
	options->multiple_Position_Start_Vertical_Value = 180;
	options->custom_Position_Vertical_Center_Value = 180;
	options->custom_Position_Vertical_1_Value = 1;
	options->custom_Position_Vertical_2_Value = 1;
	options->custom_Position_Vertical_3_Value = 1;
	options->custom_Position_Vertical_4_Value = 1;
	options->custom_Position_Vertical_5_Value = 2;

	options->position_Type_Horizontal = 0;
	
	options->simple_Position_Horizontal_Value = 180;
	options->multiple_Position_Stop_Horizontal_Value = 180;
	options->multiple_Position_Start_Horizontal_Value = 180;
	options->custom_Position_Horizontal_Center_Value = 180;
	options->custom_Position_Horizontal_1_Value = 1;
	options->custom_Position_Horizontal_2_Value = 1;
	options->custom_Position_Horizontal_3_Value = 1;
	options->custom_Position_Horizontal_4_Value = 1;
	options->custom_Position_Horizontal_5_Value = 2;

	options->position_Type_Roll = 0;
	
	options->simple_Position_Roll_Value = 180;
	options->multiple_Position_Stop_Roll_Value = 180;
	options->multiple_Position_Start_Roll_Value = 180;
	options->custom_Position_Roll_Center_Value = 180;
	options->custom_Position_Roll_1_Value = 1;
	options->custom_Position_Roll_2_Value = 1;
	options->custom_Position_Roll_3_Value = 1;
	options->custom_Position_Roll_4_Value = 1;
	options->custom_Position_Roll_5_Value = 2;

	return(SAFE_TRUE);
}
/*=**************************************************************************=*/


/*=**************************************************************************=*/
Safe_Boolean Validate_And_Copy_Contents(Import_Options_Item_Variables_Record *items
													, Import_Options_Ptr options)
{
	Safe_Boolean 	function_Result;
	CARDINAL_32	current_Value;
	CARDINAL_16	string_Length;
	CARDINAL_8		*string_Ptr;
	Boolean 			good_Input;
/* END VAR */


	good_Input = TRUE;
	
	/*  */
	/* Convert all numerical strings first */
	/* Compare numerical values for acceptability */
	/*  */
	
	string_Length = Strip_Outside_Controls(items->simple_Position_Vertical_Value.edit_String)[0];
	if ((string_Length < 1) || (string_Length > 3)) {
		good_Input = FALSE;
	}
	
	string_Length = Strip_Outside_Controls(items->multiple_Position_Stop_Vertical_Value.edit_String)[0];
	if ((string_Length < 1) || (string_Length > 3)) {
		good_Input = FALSE;
	}
	
	string_Length = Strip_Outside_Controls(items->multiple_Position_Start_Vertical_Value.edit_String)[0];
	if ((string_Length < 1) || (string_Length > 3)) {
		good_Input = FALSE;
	}
	
	string_Length = Strip_Outside_Controls(items->custom_Position_Vertical_Center_Value.edit_String)[0];
	if ((string_Length < 1) || (string_Length > 3)) {
		good_Input = FALSE;
	}
	
	string_Length = Strip_Outside_Controls(items->custom_Position_Vertical_1_Value.edit_String)[0];
	if (string_Length != 1) {
		good_Input = FALSE;
	}
	
	string_Length = Strip_Outside_Controls(items->custom_Position_Vertical_2_Value.edit_String)[0];
	if (string_Length != 1) {
		good_Input = FALSE;
	}
	
	string_Length = Strip_Outside_Controls(items->custom_Position_Vertical_3_Value.edit_String)[0];
	if (string_Length != 1) {
		good_Input = FALSE;
	}
	
	string_Length = Strip_Outside_Controls(items->custom_Position_Vertical_4_Value.edit_String)[0];
	if (string_Length != 1) {
		good_Input = FALSE;
	}
	
	string_Length = Strip_Outside_Controls(items->custom_Position_Vertical_5_Value.edit_String)[0];
	if (string_Length != 1) {
		good_Input = FALSE;
	}
	
	if (Validate_Ordinal_String_And_Copy(items->simple_Position_Vertical_Value.edit_String
								, 360
								, &(options->simple_Position_Vertical_Value)) != TRUE) {
		good_Input = FALSE;
	}
	
	if (Validate_Ordinal_String_And_Copy(items->multiple_Position_Stop_Vertical_Value.edit_String
								, 360
								, &(options->simple_Position_Vertical_Value)) != TRUE) {
		good_Input = FALSE;
	}
	
	if (Validate_Ordinal_String_And_Copy(items->multiple_Position_Start_Vertical_Value.edit_String
								, 360
								, &(options->simple_Position_Vertical_Value)) != TRUE) {
		good_Input = FALSE;
	}
	
	if (Validate_Ordinal_String_And_Copy(items->custom_Position_Vertical_Center_Value.edit_String
								, 360
								, &(options->simple_Position_Vertical_Value)) != TRUE) {
		good_Input = FALSE;
	}
	
	if (Validate_Ordinal_String_And_Copy(items->custom_Position_Vertical_1_Value.edit_String
								, 9
								, &(options->simple_Position_Vertical_Value)) != TRUE) {
		good_Input = FALSE;
	}
	
	if (Validate_Ordinal_String_And_Copy(items->custom_Position_Vertical_2_Value.edit_String
								, 9
								, &(options->simple_Position_Vertical_Value)) != TRUE) {
		good_Input = FALSE;
	}
	
	if (Validate_Ordinal_String_And_Copy(items->custom_Position_Vertical_3_Value.edit_String
								, 9
								, &(options->simple_Position_Vertical_Value)) != TRUE) {
		good_Input = FALSE;
	}
	
	if (Validate_Ordinal_String_And_Copy(items->custom_Position_Vertical_4_Value.edit_String
								, 9
								, &(options->simple_Position_Vertical_Value)) != TRUE) {
		good_Input = FALSE;
	}
	
	if (Validate_Ordinal_String_And_Copy(items->custom_Position_Vertical_5_Value.edit_String
								, 9
								, &(options->simple_Position_Vertical_Value)) != TRUE) {
		good_Input = FALSE;
	}
	


	
	if (Validate_Ordinal_String_And_Copy(items->simple_Position_Horizontal_Value.edit_String
								, 360
								, &(options->simple_Position_Horizontal_Value)) != TRUE) {
		good_Input = FALSE;
	}
	
	if (Validate_Ordinal_String_And_Copy(items->multiple_Position_Stop_Horizontal_Value.edit_String
								, 360
								, &(options->simple_Position_Horizontal_Value)) != TRUE) {
		good_Input = FALSE;
	}
	
	if (Validate_Ordinal_String_And_Copy(items->multiple_Position_Start_Horizontal_Value.edit_String
								, 360
								, &(options->simple_Position_Horizontal_Value)) != TRUE) {
		good_Input = FALSE;
	}
	
	if (Validate_Ordinal_String_And_Copy(items->custom_Position_Horizontal_Center_Value.edit_String
								, 360
								, &(options->simple_Position_Horizontal_Value)) != TRUE) {
		good_Input = FALSE;
	}
	
	if (Validate_Ordinal_String_And_Copy(items->custom_Position_Horizontal_1_Value.edit_String
								, 9
								, &(options->simple_Position_Horizontal_Value)) != TRUE) {
		good_Input = FALSE;
	}
	
	if (Validate_Ordinal_String_And_Copy(items->custom_Position_Horizontal_2_Value.edit_String
								, 9
								, &(options->simple_Position_Horizontal_Value)) != TRUE) {
		good_Input = FALSE;
	}
	
	if (Validate_Ordinal_String_And_Copy(items->custom_Position_Horizontal_3_Value.edit_String
								, 9
								, &(options->simple_Position_Horizontal_Value)) != TRUE) {
		good_Input = FALSE;
	}
	
	if (Validate_Ordinal_String_And_Copy(items->custom_Position_Horizontal_4_Value.edit_String
								, 9
								, &(options->simple_Position_Horizontal_Value)) != TRUE) {
		good_Input = FALSE;
	}
	
	if (Validate_Ordinal_String_And_Copy(items->custom_Position_Horizontal_5_Value.edit_String
								, 9
								, &(options->simple_Position_Horizontal_Value)) != TRUE) {
		good_Input = FALSE;
	}
	


	
	if (Validate_Ordinal_String_And_Copy(items->simple_Position_Roll_Value.edit_String
								, 360
								, &(options->simple_Position_Roll_Value)) != TRUE) {
		good_Input = FALSE;
	}
	
	if (Validate_Ordinal_String_And_Copy(items->multiple_Position_Stop_Roll_Value.edit_String
								, 360
								, &(options->simple_Position_Roll_Value)) != TRUE) {
		good_Input = FALSE;
	}
	
	if (Validate_Ordinal_String_And_Copy(items->multiple_Position_Start_Roll_Value.edit_String
								, 360
								, &(options->simple_Position_Roll_Value)) != TRUE) {
		good_Input = FALSE;
	}
	
	if (Validate_Ordinal_String_And_Copy(items->custom_Position_Roll_Center_Value.edit_String
								, 360
								, &(options->simple_Position_Roll_Value)) != TRUE) {
		good_Input = FALSE;
	}
	
	if (Validate_Ordinal_String_And_Copy(items->custom_Position_Roll_1_Value.edit_String
								, 9
								, &(options->simple_Position_Roll_Value)) != TRUE) {
		good_Input = FALSE;
	}
	
	if (Validate_Ordinal_String_And_Copy(items->custom_Position_Roll_2_Value.edit_String
								, 9
								, &(options->simple_Position_Roll_Value)) != TRUE) {
		good_Input = FALSE;
	}
	
	if (Validate_Ordinal_String_And_Copy(items->custom_Position_Roll_3_Value.edit_String
								, 9
								, &(options->simple_Position_Roll_Value)) != TRUE) {
		good_Input = FALSE;
	}
	
	if (Validate_Ordinal_String_And_Copy(items->custom_Position_Roll_4_Value.edit_String
								, 9
								, &(options->simple_Position_Roll_Value)) != TRUE) {
		good_Input = FALSE;
	}
	
	if (Validate_Ordinal_String_And_Copy(items->custom_Position_Roll_5_Value.edit_String
								, 9
								, &(options->simple_Position_Roll_Value)) != TRUE) {
		good_Input = FALSE;
	}
	
	

	string_Length = Strip_Outside_Controls(items->mode_1_Emotion_Text.edit_String)[0];
	if ((string_Length < 1) || (string_Length > 31)) {
		good_Input = FALSE;
	}
	
	string_Length = Strip_Outside_Controls(items->mode_2_Emotion_Text.edit_String)[0];
	if ((string_Length < 1) || (string_Length > 31)) {
		good_Input = FALSE;
	}
	
	string_Length = Strip_Outside_Controls(items->mode_3_Emotion_Text.edit_String)[0];
	if ((string_Length < 1) || (string_Length > 31)) {
		good_Input = FALSE;
	}
	
	string_Length = Strip_Outside_Controls(items->mode_4_Emotion_Text.edit_String)[0];
	if ((string_Length < 1) || (string_Length > 31)) {
		good_Input = FALSE;
	}
	
	string_Length = Strip_Outside_Controls(items->mode_5_Emotion_Text.edit_String)[0];
	if ((string_Length < 1) || (string_Length > 31)) {
		good_Input = FALSE;
	}

	/*  */
	/* Assign numerical values */
	/*  */

	/*  */
	/* Copy strings compare lengths first. */
	/*  */

	/*  */
	/* Copy strings, truncate if needed */
	/*  */

	/*  */
	/* Copy over checkbox booleans */
	/*  */
	
	options->use_Special_Colors = items->use_Special_Colors.checked;
	options->import_Background = items->import_Background.checked;
	options->compress_To_256_Colors = items->compress_To_256_Colors.checked;
	options->ask_For_Special_CLUT = items->ask_For_Special_CLUT.checked;

	options->mode_Emotion_Value = 0;
	options->mode_Emotion_Str255[0] = 0;
	Copy_Pascal_String("\pDefault", options->mode_Emotion_Str255);

	if (items->mode_1_Emotion.checked != FALSE) {
		options->mode_Emotion_Value = 0;
		Copy_Pascal_String(items->mode_1_Emotion_Text.edit_String, options->mode_Emotion_Str255);
	}
	else if (items->mode_2_Emotion.checked != FALSE) {
		options->mode_Emotion_Value = 1;
		Copy_Pascal_String(items->mode_2_Emotion_Text.edit_String, options->mode_Emotion_Str255);
	}
	else if (items->mode_3_Emotion.checked != FALSE) {
		options->mode_Emotion_Value = 3;
		Copy_Pascal_String(items->mode_3_Emotion_Text.edit_String, options->mode_Emotion_Str255);
	}
	else if (items->mode_4_Emotion.checked != FALSE) {
		options->mode_Emotion_Value = 4;
		Copy_Pascal_String(items->mode_4_Emotion_Text.edit_String, options->mode_Emotion_Str255);
	}
	else if (items->mode_5_Emotion.checked != FALSE) {
		options->mode_Emotion_Value = 5;
		Copy_Pascal_String(items->mode_5_Emotion_Text.edit_String, options->mode_Emotion_Str255);
	}
	else {
		good_Input = FALSE;
	}


	if  (good_Input != TRUE) {
		goto Error_Handler;
	}	

	/*  */
	/* Convert radio button selection groups select first choice */
	/*  */			

	
	if (items->simple_Position_Vertical.checked != FALSE) {
		options->position_Type_Vertical = 0;
	}
	else if (items->multiple_Position_Vertical.checked != FALSE) {
		options->position_Type_Vertical = 1;
	}
	else if (items->custom_Position_Vertical.checked != FALSE) {
		options->position_Type_Vertical = 2;
	}
	else if (items->c1112_Position_Vertical.checked != FALSE) {
		options->position_Type_Vertical = 3;
	}
	else if (items->c112_Position_Vertical.checked != FALSE) {
		options->position_Type_Vertical = 4;
	}
	else if (items->c12_Position_Vertical.checked != FALSE) {
		options->position_Type_Vertical = 5;
	}
	else if (items->c1_Position_Vertical.checked != FALSE) {
		options->position_Type_Vertical = 6;
	}
	else if (items->c2_Position_Vertical.checked != FALSE) {
		options->position_Type_Vertical = 7;
	}
	else if (items->c4_Position_Vertical.checked != FALSE) {
		options->position_Type_Vertical = 8;
	}
	else {
		good_Input = FALSE;
	}

	if (items->simple_Position_Horizontal.checked != FALSE) {
		options->position_Type_Horizontal = 0;
	}
	else if (items->multiple_Position_Horizontal.checked != FALSE) {
		options->position_Type_Horizontal = 1;
	}
	else if (items->custom_Position_Horizontal.checked != FALSE) {
		options->position_Type_Horizontal = 2;
	}
	else if (items->c1112_Position_Horizontal.checked != FALSE) {
		options->position_Type_Horizontal = 3;
	}
	else if (items->c112_Position_Horizontal.checked != FALSE) {
		options->position_Type_Horizontal = 4;
	}
	else if (items->c12_Position_Horizontal.checked != FALSE) {
		options->position_Type_Horizontal = 5;
	}
	else if (items->c1_Position_Horizontal.checked != FALSE) {
		options->position_Type_Horizontal = 6;
	}
	else if (items->c2_Position_Horizontal.checked != FALSE) {
		options->position_Type_Horizontal = 7;
	}
	else if (items->c4_Position_Horizontal.checked != FALSE) {
		options->position_Type_Horizontal = 8;
	}
	else {
		good_Input = FALSE;
	}


	if (items->simple_Position_Roll.checked != FALSE) {
		options->position_Type_Roll = 0;
	}
	else if (items->multiple_Position_Roll.checked != FALSE) {
		options->position_Type_Roll = 1;
	}
	else if (items->custom_Position_Roll.checked != FALSE) {
		options->position_Type_Roll = 2;
	}
	else if (items->c1112_Position_Roll.checked != FALSE) {
		options->position_Type_Roll = 3;
	}
	else if (items->c112_Position_Roll.checked != FALSE) {
		options->position_Type_Roll = 4;
	}
	else if (items->c12_Position_Roll.checked != FALSE) {
		options->position_Type_Roll = 5;
	}
	else if (items->c1_Position_Roll.checked != FALSE) {
		options->position_Type_Roll = 6;
	}
	else if (items->c2_Position_Roll.checked != FALSE) {
		options->position_Type_Roll = 7;
	}
	else if (items->c4_Position_Roll.checked != FALSE) {
		options->position_Type_Roll = 8;
	}
	else {
		good_Input = FALSE;
	}


	if  (good_Input != TRUE) {
		goto Error_Handler;
	}	






	function_Result = SAFE_TRUE;
	goto Normal_Function_Exit;
	
Error_Handler:
	function_Result = SAFE_FALSE;
			
Normal_Function_Exit:


	return(function_Result);
}
/*=**************************************************************************=*/


/*=**************************************************************************=*/
Boolean Get_Import_Options(Import_Options_Ptr	options, CARDINAL_32 total_Picts)
{
	Import_Options_Item_Variables_Record	items;
	
	INTEGER_8		*work_String_Ptr;
	Str255				work_String;
	INTEGER_16		error_Code;
	INTEGER_32		length_1_Entered;
	Str32				value_String;
	Boolean			valid_Input;
	Boolean			keep_Inquiring;
	Boolean			function_Result;
/* END VAR */


	function_Result = FALSE;
	
	/*  */
	/* Allows all pointers to be set to "nil" lazy style */
	/*  */
	Fill_Byte_Memory_With_Byte((Ptr)&items, sizeof(items), 0);
	
#if !defined(ORIGINAL_MAC_SYSTEM)
#error "UNDEFINED PARAMETER!"
#endif
#if (ORIGINAL_MAC_SYSTEM == 0)
Dialogs.h
XXXX 		
	items.cancel_Button_Outline.setup_Function = (ProcPtr)Setup_Outline_New;
	items.cancel_Button_Outline.update_Function = (ProcPtr)Update_Outline;
#else
	items.cancel_Button_Outline.setup_Function = (void *)Setup_Outline_New;
	items.cancel_Button_Outline.update_Function = (void *)Update_Outline;
#endif 
	
	
	items.cancel_Button_Outline.action_Function = NULL;


	if (Initialize_Defaults_For_Import(options) != SAFE_TRUE) {
		goto Error_Handler;
	}
	/*  */
	/* NOTE: Initialize_Defaults_For_Import filled in defaults */
	/* you may modify here */
	/*  */

	items.use_Special_Colors.enabled = TRUE;
	items.import_Background.enabled = TRUE;
	items.compress_To_256_Colors.enabled = TRUE;
	items.ask_For_Special_CLUT.enabled = TRUE;
	items.mode_1_Emotion.enabled = TRUE;
	items.mode_2_Emotion.enabled = TRUE;
	items.mode_3_Emotion.enabled = TRUE;
	items.mode_4_Emotion.enabled = TRUE;
	items.mode_5_Emotion.enabled = TRUE;

	switch(options->mode_Emotion_Value) {
		case 0:
			items.mode_1_Emotion.checked = TRUE;
			break;
			
		case 1:
			items.mode_2_Emotion.checked = TRUE;
			break;
			
		case 2:
			items.mode_3_Emotion.checked = TRUE;
			break;
			
		case 3:
			items.mode_4_Emotion.checked = TRUE;
			break;
			
		case 4:
			items.mode_5_Emotion.checked = TRUE;
			break;
			
		default:
			items.mode_1_Emotion.checked = TRUE;
			break;
	} /* switch(options->mode_Emotion_Value) */

	items.mode_1_Emotion_Text.enabled = TRUE;
	items.mode_2_Emotion_Text.enabled = TRUE;
	items.mode_3_Emotion_Text.enabled = TRUE;
	items.mode_4_Emotion_Text.enabled = TRUE;
	items.mode_5_Emotion_Text.enabled = TRUE;
	items.simple_Position_Vertical.enabled = TRUE;
	items.multiple_Position_Vertical.enabled = TRUE;
	items.custom_Position_Vertical.enabled = TRUE;
	items.c1112_Position_Vertical.enabled = TRUE;
	items.c112_Position_Vertical.enabled = TRUE;
	items.c12_Position_Vertical.enabled = TRUE;
	items.c1_Position_Vertical.enabled = TRUE;
	items.c2_Position_Vertical.enabled = TRUE;
	items.c4_Position_Vertical.enabled = TRUE;
	items.simple_Position_Vertical_Value.enabled = TRUE;
	items.multiple_Position_Stop_Vertical_Value.enabled = TRUE;
	items.multiple_Position_Start_Vertical_Value.enabled = TRUE;
	items.custom_Position_Vertical_Center_Value.enabled = TRUE;
	items.custom_Position_Vertical_1_Value.enabled = TRUE;
	items.custom_Position_Vertical_2_Value.enabled = TRUE;
	items.custom_Position_Vertical_3_Value.enabled = TRUE;
	items.custom_Position_Vertical_4_Value.enabled = TRUE;
	items.custom_Position_Vertical_5_Value.enabled = TRUE;
	items.simple_Position_Horizontal.enabled = TRUE;
	items.multiple_Position_Horizontal.enabled = TRUE;
	items.custom_Position_Horizontal.enabled = TRUE;
	items.c1112_Position_Horizontal.enabled = TRUE;
	items.c112_Position_Horizontal.enabled = TRUE;
	items.c12_Position_Horizontal.enabled = TRUE;
	items.c1_Position_Horizontal.enabled = TRUE;
	items.c2_Position_Horizontal.enabled = TRUE;
	items.c4_Position_Horizontal.enabled = TRUE;
	items.simple_Position_Horizontal_Value.enabled = TRUE;
	items.multiple_Position_Stop_Horizontal_Value.enabled = TRUE;
	items.multiple_Position_Start_Horizontal_Value.enabled = TRUE;
	items.custom_Position_Horizontal_Center_Value.enabled = TRUE;
	items.custom_Position_Horizontal_1_Value.enabled = TRUE;
	items.custom_Position_Horizontal_2_Value.enabled = TRUE;
	items.custom_Position_Horizontal_3_Value.enabled = TRUE;
	items.custom_Position_Horizontal_4_Value.enabled = TRUE;
	items.custom_Position_Horizontal_5_Value.enabled = TRUE;
	items.simple_Position_Roll.enabled = TRUE;
	items.multiple_Position_Roll.enabled = TRUE;
	items.custom_Position_Roll.enabled = TRUE;
	items.c1112_Position_Roll.enabled = TRUE;
	items.c112_Position_Roll.enabled = TRUE;
	items.c12_Position_Roll.enabled = TRUE;
	items.c1_Position_Roll.enabled = TRUE;
	items.c2_Position_Roll.enabled = TRUE;
	items.c4_Position_Roll.enabled = TRUE;
	items.simple_Position_Roll_Value.enabled = TRUE;
	items.multiple_Position_Stop_Roll_Value.enabled = TRUE;
	items.multiple_Position_Start_Roll_Value.enabled = TRUE;
	items.custom_Position_Roll_Center_Value.enabled = TRUE;
	items.custom_Position_Roll_1_Value.enabled = TRUE;
	items.custom_Position_Roll_2_Value.enabled = TRUE;
	items.custom_Position_Roll_3_Value.enabled = TRUE;
	items.custom_Position_Roll_4_Value.enabled = TRUE;
	items.custom_Position_Roll_5_Value.enabled = TRUE;
	
	items.use_Special_Colors.checked = FALSE;
	items.import_Background.checked = FALSE;
	items.compress_To_256_Colors.checked = FALSE;
	items.ask_For_Special_CLUT.checked = FALSE;
			
	items.simple_Position_Vertical.checked = TRUE;
	items.multiple_Position_Vertical.checked = FALSE;
	items.custom_Position_Vertical.checked = FALSE;
	items.c1112_Position_Vertical.checked = FALSE;
	items.c112_Position_Vertical.checked = FALSE;
	items.c12_Position_Vertical.checked = FALSE;
	items.c1_Position_Vertical.checked = FALSE;
	items.c2_Position_Vertical.checked = FALSE;
	items.c4_Position_Vertical.checked = FALSE;
	
	Copy_Pascal_String("\p180", items.simple_Position_Vertical_Value.edit_String);
	Copy_Pascal_String("\p180", items.multiple_Position_Stop_Vertical_Value.edit_String);
	Copy_Pascal_String("\p180", items.multiple_Position_Start_Vertical_Value.edit_String);
	Copy_Pascal_String("\p180", items.custom_Position_Vertical_Center_Value.edit_String);
	Copy_Pascal_String("\p1", items.custom_Position_Vertical_1_Value.edit_String);
	Copy_Pascal_String("\p1", items.custom_Position_Vertical_2_Value.edit_String);
	Copy_Pascal_String("\p1", items.custom_Position_Vertical_3_Value.edit_String);
	Copy_Pascal_String("\p1", items.custom_Position_Vertical_4_Value.edit_String);
	Copy_Pascal_String("\p2", items.custom_Position_Vertical_5_Value.edit_String);
	
	items.simple_Position_Horizontal.checked = TRUE;
	items.multiple_Position_Horizontal.checked = FALSE;
	items.custom_Position_Horizontal.checked = FALSE;
	items.c1112_Position_Horizontal.checked = FALSE;
	items.c112_Position_Horizontal.checked = FALSE;
	items.c12_Position_Horizontal.checked = FALSE;
	items.c1_Position_Horizontal.checked = FALSE;
	items.c2_Position_Horizontal.checked = FALSE;
	items.c4_Position_Horizontal.checked = FALSE;
	
	Copy_Pascal_String("\p180", items.simple_Position_Horizontal_Value.edit_String);
	Copy_Pascal_String("\p180", items.multiple_Position_Stop_Horizontal_Value.edit_String);
	Copy_Pascal_String("\p180", items.multiple_Position_Start_Horizontal_Value.edit_String);
	Copy_Pascal_String("\p180", items.custom_Position_Horizontal_Center_Value.edit_String);
	Copy_Pascal_String("\p1", items.custom_Position_Horizontal_1_Value.edit_String);
	Copy_Pascal_String("\p1", items.custom_Position_Horizontal_2_Value.edit_String);
	Copy_Pascal_String("\p1", items.custom_Position_Horizontal_3_Value.edit_String);
	Copy_Pascal_String("\p1", items.custom_Position_Horizontal_4_Value.edit_String);
	Copy_Pascal_String("\p2", items.custom_Position_Horizontal_5_Value.edit_String);
	
	items.simple_Position_Roll.checked = TRUE;
	items.multiple_Position_Roll.checked = FALSE;
	items.custom_Position_Roll.checked = FALSE;
	items.c1112_Position_Roll.checked = FALSE;
	items.c112_Position_Roll.checked = FALSE;
	items.c12_Position_Roll.checked = FALSE;
	items.c1_Position_Roll.checked = FALSE;
	items.c2_Position_Roll.checked = FALSE;
	items.c4_Position_Roll.checked = FALSE;
	
	Copy_Pascal_String("\p180", items.simple_Position_Roll_Value.edit_String);
	Copy_Pascal_String("\p180", items.multiple_Position_Stop_Roll_Value.edit_String);
	Copy_Pascal_String("\p180", items.multiple_Position_Start_Roll_Value.edit_String);
	Copy_Pascal_String("\p180", items.custom_Position_Roll_Center_Value.edit_String);
	Copy_Pascal_String("\p1", items.custom_Position_Roll_1_Value.edit_String);
	Copy_Pascal_String("\p1", items.custom_Position_Roll_2_Value.edit_String);
	Copy_Pascal_String("\p1", items.custom_Position_Roll_3_Value.edit_String);
	Copy_Pascal_String("\p1", items.custom_Position_Roll_4_Value.edit_String);
	Copy_Pascal_String("\p2", items.custom_Position_Roll_5_Value.edit_String);

	Copy_Pascal_String("\pHappy", items.mode_1_Emotion_Text.edit_String);
	Copy_Pascal_String("\pSad", items.mode_2_Emotion_Text.edit_String);
	Copy_Pascal_String("\pAngry", items.mode_3_Emotion_Text.edit_String);
	Copy_Pascal_String("\pFrightened", items.mode_4_Emotion_Text.edit_String);
	Copy_Pascal_String("\pDisgusted", items.mode_5_Emotion_Text.edit_String);

	items.mode_1_Emotion_Text.edit_String_Edit_Length = 31;
	items.mode_2_Emotion_Text.edit_String_Edit_Length = 31;
	items.mode_3_Emotion_Text.edit_String_Edit_Length = 31;
	items.mode_4_Emotion_Text.edit_String_Edit_Length = 31;
	items.mode_5_Emotion_Text.edit_String_Edit_Length = 31;
	
	items.simple_Position_Vertical_Value.edit_String_Edit_Length = 3;
	items.multiple_Position_Stop_Vertical_Value.edit_String_Edit_Length = 3;
	items.multiple_Position_Start_Vertical_Value.edit_String_Edit_Length = 3;
	items.custom_Position_Vertical_Center_Value.edit_String_Edit_Length = 3;
	items.custom_Position_Vertical_1_Value.edit_String_Edit_Length = 1;
	items.custom_Position_Vertical_2_Value.edit_String_Edit_Length = 1;
	items.custom_Position_Vertical_3_Value.edit_String_Edit_Length = 1;
	items.custom_Position_Vertical_4_Value.edit_String_Edit_Length = 1;
	items.custom_Position_Vertical_5_Value.edit_String_Edit_Length = 1;
	
	items.simple_Position_Horizontal_Value.edit_String_Edit_Length = 3;
	items.multiple_Position_Stop_Horizontal_Value.edit_String_Edit_Length = 3;
	items.multiple_Position_Start_Horizontal_Value.edit_String_Edit_Length = 3;
	items.custom_Position_Horizontal_Center_Value.edit_String_Edit_Length = 3;
	items.custom_Position_Horizontal_1_Value.edit_String_Edit_Length = 1;
	items.custom_Position_Horizontal_2_Value.edit_String_Edit_Length = 1;
	items.custom_Position_Horizontal_3_Value.edit_String_Edit_Length = 1;
	items.custom_Position_Horizontal_4_Value.edit_String_Edit_Length = 1;
	items.custom_Position_Horizontal_5_Value.edit_String_Edit_Length = 1;
	
	items.simple_Position_Roll_Value.edit_String_Edit_Length = 3;
	items.multiple_Position_Stop_Roll_Value.edit_String_Edit_Length = 3;
	items.multiple_Position_Start_Roll_Value.edit_String_Edit_Length = 3;
	items.custom_Position_Roll_Center_Value.edit_String_Edit_Length = 3;
	items.custom_Position_Roll_1_Value.edit_String_Edit_Length = 1;
	items.custom_Position_Roll_2_Value.edit_String_Edit_Length = 1;
	items.custom_Position_Roll_3_Value.edit_String_Edit_Length = 1;
	items.custom_Position_Roll_4_Value.edit_String_Edit_Length = 1;
	items.custom_Position_Roll_5_Value.edit_String_Edit_Length = 1;

	items.mode_1_Emotion_Text.edit_Field_Type = string_Field;
	items.mode_2_Emotion_Text.edit_Field_Type = string_Field;
	items.mode_3_Emotion_Text.edit_Field_Type = string_Field;
	items.mode_4_Emotion_Text.edit_Field_Type = string_Field;
	items.mode_5_Emotion_Text.edit_Field_Type = string_Field;
	items.simple_Position_Vertical_Value.edit_Field_Type = integer_Field;
	items.multiple_Position_Stop_Vertical_Value.edit_Field_Type = integer_Field;
	items.multiple_Position_Start_Vertical_Value.edit_Field_Type = integer_Field;
	items.custom_Position_Vertical_Center_Value.edit_Field_Type = integer_Field;
	items.custom_Position_Vertical_1_Value.edit_Field_Type = integer_Field;
	items.custom_Position_Vertical_2_Value.edit_Field_Type = integer_Field;
	items.custom_Position_Vertical_3_Value.edit_Field_Type = integer_Field;
	items.custom_Position_Vertical_4_Value.edit_Field_Type = integer_Field;
	items.custom_Position_Vertical_5_Value.edit_Field_Type = integer_Field;
	items.simple_Position_Horizontal_Value.edit_Field_Type = integer_Field;
	items.multiple_Position_Stop_Horizontal_Value.edit_Field_Type = integer_Field;
	items.multiple_Position_Start_Horizontal_Value.edit_Field_Type = integer_Field;
	items.custom_Position_Horizontal_Center_Value.edit_Field_Type = integer_Field;
	items.custom_Position_Horizontal_1_Value.edit_Field_Type = integer_Field;
	items.custom_Position_Horizontal_2_Value.edit_Field_Type = integer_Field;
	items.custom_Position_Horizontal_3_Value.edit_Field_Type = integer_Field;
	items.custom_Position_Horizontal_4_Value.edit_Field_Type = integer_Field;
	items.custom_Position_Horizontal_5_Value.edit_Field_Type = integer_Field;
	items.multiple_Position_Stop_Roll_Value.edit_Field_Type = integer_Field;
	items.multiple_Position_Start_Roll_Value.edit_Field_Type = integer_Field;
	items.custom_Position_Roll_Center_Value.edit_Field_Type = integer_Field;
	items.custom_Position_Roll_1_Value.edit_Field_Type = integer_Field;
	items.custom_Position_Roll_2_Value.edit_Field_Type = integer_Field;
	items.custom_Position_Roll_3_Value.edit_Field_Type = integer_Field;
	items.custom_Position_Roll_4_Value.edit_Field_Type = integer_Field;
	items.custom_Position_Roll_5_Value.edit_Field_Type = integer_Field;
	
	
	items.number_Of_PICTS_String.enabled = FALSE;
	NumToString(total_Picts,(CARDINAL_8 *)value_String);
	Copy_Pascal_String(value_String, items.number_Of_PICTS_String.edit_String);
	items.number_Of_PICTS_String.edit_String_Edit_Length = 4;

	current_Outline_Item_Number = IMPORT_OPTIONS_OUTLINE_ITEM;
	
	valid_Input = FALSE;
	keep_Inquiring = TRUE;
	
	do {					/* while (valid_Input == FALSE); */
		if (Perform_Dialog(IMPORT_OPTIONS_DIALOG_ID, (Item_Variables_Ptr)&items) == TRUE) {
		
			if (items.header.item_Hit == k_OK_Item) {
			
				if (Validate_And_Copy_Contents(&items, options) == SAFE_TRUE) {
					valid_Input = TRUE;
					keep_Inquiring = FALSE;
				}
				else {
					/*  */
					/* Redundant, but placed here for debugging breaks */
					/*  */
					valid_Input = FALSE;
					keep_Inquiring = TRUE;
				}
								
			} /* if (items.header.item_Hit == OK) */
			else {
				; /* NULL */
				
			} /* NOT if (items.header.item_Hit == OK) */
				
		} /* if (Perform_Dialog(IMPORT_OPTIONS_DIALOG_ID, (Item_Variables_Ptr)&items) */
		else {
			valid_Input = FALSE;
			keep_Inquiring = FALSE;
		}
	
	} while (keep_Inquiring == TRUE);


	if (valid_Input != TRUE) {
		goto Error_Handler;
	}

	function_Result = TRUE;
	goto Normal_Function_Exit;
	
Error_Handler:
	function_Result = FALSE;
			
Normal_Function_Exit:

	return(function_Result);
	
} /* Boolean Get_Import_Options(... */
/*=**************************************************************************=*/


/*=**************************************************************************=*/
Boolean Edit_Fancy_Filename_Information(Str255 message
								, CARDINAL_8 *item_File_Name
								, CARDINAL_8 *item_Short_Name
								, CARDINAL_8 *item_Description)
{
	Get_New_Scene_Item_Variables_Record	items;
	
	INTEGER_8		*work_String_Ptr;
	Str255				work_String;
	INTEGER_16		error_Code;
	INTEGER_32		length_1_Entered;
	Boolean			valid_Input;
	Boolean			function_Result;
	Safe_Boolean 	good_Input;
/* END VAR */


	function_Result = FALSE;
	
	
#if !defined(ORIGINAL_MAC_SYSTEM)
#error "UNDEFINED PARAMETER!"
#endif
#if (ORIGINAL_MAC_SYSTEM == 0)
Dialogs.h
XXXX 		
	items.cancel_Button_Outline.setup_Function = (ProcPtr)Setup_Outline_New;
	items.cancel_Button_Outline.update_Function = (ProcPtr)Update_Outline;
#else
	items.cancel_Button_Outline.setup_Function = (void *)Setup_Outline_New;
	items.cancel_Button_Outline.update_Function = (void *)Update_Outline;
#endif 
	
	
	items.cancel_Button_Outline.action_Function = NULL;

	items.input_Text.enabled = FALSE;
	Copy_Pascal_String(message, items.input_Text.edit_String);
	items.input_Text.edit_String_Edit_Length = 255;
	items.input_Text.edit_Field_Type = string_Field;

	items.descriptive_Name_Text.enabled = TRUE;
	Copy_Pascal_String(item_Description, items.descriptive_Name_Text.edit_String);
	items.descriptive_Name_Text.edit_String_Edit_Length = 255;
	items.descriptive_Name_Text.edit_Field_Type = string_Field;
	
	items.file_Name_Text.enabled = TRUE;
	Copy_Pascal_String(item_File_Name, items.file_Name_Text.edit_String);
	items.file_Name_Text.edit_String_Edit_Length = k_FILE_NAME_MAXIMUM;
	items.file_Name_Text.edit_Field_Type = string_Field;

	items.tiny_Name_Text.enabled = TRUE;
	Copy_Pascal_String(item_Short_Name, items.tiny_Name_Text.edit_String);
	items.tiny_Name_Text.edit_String_Edit_Length = k_TINY_NICKNAME_NAME_MAXIMUM;
	items.tiny_Name_Text.edit_Field_Type = string_Field;

	current_Outline_Item_Number = GET_NEW_PUPPET_OUTLINE_ITEM;
	
	valid_Input = FALSE;
	
	do {														/* while (valid_Input == FALSE); */
		if (Perform_Dialog(GET_NEW_PUPPET_DIALOG_ID, (Item_Variables_Ptr)&items) == TRUE) {
		
			if (items.header.item_Hit == k_OK_Item) {
				length_1_Entered = Strip_Outside_Controls(items.file_Name_Text.edit_String)[0];
				
				good_Input = SAFE_FALSE;
				if ((length_1_Entered >= 1) && (length_1_Entered <= k_FILE_NAME_MAXIMUM)) {
					length_1_Entered = Strip_Outside_Controls(items.tiny_Name_Text.edit_String)[0];
					if ((length_1_Entered >= 1) && (length_1_Entered <= k_TINY_NICKNAME_NAME_MAXIMUM)) {
						good_Input = SAFE_TRUE;
					}
				}
				
				if (good_Input == SAFE_TRUE) {

					if(Compare_Pascal_Strings_Ignoring_Case(
							&(items.descriptive_Name_Text.edit_String[0])
							, &(k_UNTITLED_NAME_STRING[0])) == SAFE_TRUE) {
						good_Input = SAFE_FALSE;
					}
				
					if(Compare_Pascal_Strings_Ignoring_Case(
							&(items.file_Name_Text.edit_String[0])
							, &(k_UNTITLED_NAME_STRING[0])) == SAFE_TRUE) {
						good_Input = SAFE_FALSE;
					}
				
					if(Compare_Pascal_Strings_Ignoring_Case(
							&(items.tiny_Name_Text.edit_String[0])
							, &(k_UNTITLED_NAME_STRING[0])) == SAFE_TRUE) {
						good_Input = SAFE_FALSE;
					}
				
					if (good_Input == SAFE_TRUE) {
						valid_Input = TRUE;
						function_Result = TRUE;
						
						Copy_Pascal_String(&(items.descriptive_Name_Text.edit_String[0]), item_Description);
						Copy_Pascal_String(&(items.file_Name_Text.edit_String[0]), item_File_Name);
						Copy_Pascal_String(&(items.tiny_Name_Text.edit_String[0]), item_Short_Name);
					}
					else{
						SysBeep(20);
						
						Get_NS_String(k_DREAM_STRINGS, k_ERROR_UNTITLED_NOT_ALLOWED_PROMPT, work_String);
						
						if (Perform_Boolean_Dialog(work_String) == FALSE) {
							valid_Input = TRUE;
							function_Result = FALSE;
						}
						
					} /* NOT if (good_Input == SAFE_TRUE) */
				}
				else{
					SysBeep(20);
					
					Get_NS_String(k_DREAM_STRINGS, k_BAD_STRING_LENGTHS_PROMPT, work_String);
					
					if (Perform_Boolean_Dialog(work_String) == FALSE) {
						valid_Input = TRUE;
						function_Result = FALSE;
					}
					
				} /* NOT if (good_Input == SAFE_TRUE) */
				
			} /* if (items.header.item_Hit == OK) */
			else {
				function_Result = TRUE;
				
			} /* NOT if (items.header.item_Hit == OK) */
				
		} /* if (Perform_Dialog(GET_NEW_PUPPET_DIALOG_ID, (Item_Variables_Ptr)&items) */
		else
			valid_Input = TRUE;
	
	} while (valid_Input == FALSE);

	return(function_Result);
	
} /* Boolean Get_New_Scene(Str255 message... */
/*=**************************************************************************=*/


/*=**************************************************************************=*/
Safe_Boolean Load_Puppet_Control_Preparation(Load_Puppet_Rec	*params_For_Load_Puppet)
{
	Safe_Boolean			function_Result;
/* END VAR */
	
	
	function_Result = SAFE_TRUE;
	params_For_Load_Puppet->async_Task_Desired = SAFE_FALSE;
	
	
	/*  */
	/* Add code to get FSSpec from memory based list */
	/*  */
	
	
	function_Result = SAFE_FALSE;
	
	return(function_Result);
}
/*=**************************************************************************=*/


/*=**************************************************************************=*/
Safe_Boolean Load_Puppet_Menu_Preparation(Load_Puppet_Rec	*params_For_Load_Puppet)
{
	Safe_Boolean			function_Result;
/* END VAR */
	
	
	function_Result = SAFE_TRUE;
	
	params_For_Load_Puppet->async_Task_Desired = SAFE_FALSE;
		
	return(function_Result);
}
/*=**************************************************************************=*/


/*=**************************************************************************=*/
Safe_Boolean Close_Puppet_Control_Preparation(Close_Puppet_Rec	*params_For_Close_Puppet)
{
	Safe_Boolean			function_Result;
/* END VAR */
	
	
	function_Result = SAFE_TRUE;
	
	params_For_Close_Puppet->async_Task_Desired = SAFE_FALSE;
	
	
	return(function_Result);
}
/*=**************************************************************************=*/


/*=**************************************************************************=*/
Safe_Boolean Close_Puppet_Menu_Preparation(Close_Puppet_Rec	*params_For_Close_Puppet)
{
	Safe_Boolean			function_Result;
/* END VAR */
	
	
	function_Result = SAFE_TRUE;
	
	params_For_Close_Puppet->async_Task_Desired = SAFE_FALSE;
	
	
	return(function_Result);
}
/*=**************************************************************************=*/


/*=**************************************************************************=*/
Safe_Boolean Save_Puppet_Control_Preparation(Save_Puppet_Rec	*params_For_Save_Puppet)
{
	Safe_Boolean			function_Result;
	CARDINAL_32		resulting_Index;
	Safe_Boolean 		valid_Result;
/* END VAR */
	
	
	function_Result = SAFE_TRUE;
	
	params_For_Save_Puppet->async_Task_Desired = SAFE_FALSE;
	
	if (Get_Dream_Popup_Menu_Choice(k_PUPPET_SELECTION
											, &resulting_Index
											, &valid_Result) != SAFE_TRUE) {
		function_Result = SAFE_FALSE;
	}
	else {
		if (valid_Result != SAFE_TRUE) {
			function_Result = SAFE_FALSE;
		}
		else {
			params_For_Save_Puppet->desired_Item = resulting_Index;
		}
	}
	
	return(function_Result);
}
/*=**************************************************************************=*/


/*=**************************************************************************=*/
Safe_Boolean Save_Puppet_Menu_Preparation(Save_Puppet_Rec	*params_For_Save_Puppet)
{
	Safe_Boolean			function_Result;
	CARDINAL_32		resulting_Index;
	Safe_Boolean 		valid_Result;
/* END VAR */
	
	
	function_Result = SAFE_TRUE;
	
	params_For_Save_Puppet->async_Task_Desired = SAFE_FALSE;
	
	if (Get_Dream_Popup_Menu_Choice(k_PUPPET_SELECTION
											, &resulting_Index
											, &valid_Result) != SAFE_TRUE) {
		function_Result = SAFE_FALSE;
	}
	else {
		if (valid_Result != SAFE_TRUE) {
			function_Result = SAFE_FALSE;
		}
		else {
			params_For_Save_Puppet->desired_Item = resulting_Index;
		}
	}
	
	return(function_Result);
}
/*=**************************************************************************=*/


/*=**************************************************************************=*/
Safe_Boolean Import_Puppet_Control_Preparation(Import_Puppet_Rec	*params_For_Import_Puppet)
{
	Safe_Boolean			function_Result;
/* END VAR */
	
	
	function_Result = SAFE_TRUE;
	
	params_For_Import_Puppet->async_Task_Desired = SAFE_FALSE;
	
	if (Get_New_Puppet_Information(params_For_Import_Puppet->puppet_File_Name
								, params_For_Import_Puppet->puppet_Short_Name
								, params_For_Import_Puppet->puppet_Description) != TRUE) {
		function_Result = SAFE_FALSE;
	}
	
	params_For_Import_Puppet->import_Source = k_IMPORT_FROM_PICS_SEQUENCE;

	
	return(function_Result);
}
/*=**************************************************************************=*/


/*=**************************************************************************=*/
Safe_Boolean Import_Puppet_Menu_Preparation(Import_Puppet_Rec	*params_For_Import_Puppet)
{
	Safe_Boolean			function_Result;
/* END VAR */
	
	
	function_Result = SAFE_TRUE;
	
	params_For_Import_Puppet->async_Task_Desired = SAFE_FALSE;
	
	
	if (Get_New_Puppet_Information(params_For_Import_Puppet->puppet_File_Name
								, params_For_Import_Puppet->puppet_Short_Name
								, params_For_Import_Puppet->puppet_Description) != TRUE) {
		function_Result = SAFE_FALSE;
	}
	
	params_For_Import_Puppet->import_Source = k_IMPORT_FROM_PICS_SEQUENCE;


	return(function_Result);
}
/*=**************************************************************************=*/


/*=**************************************************************************=*/
Safe_Boolean Create_Puppet_From_Spin_File_Control_Preparation(Import_Puppet_Rec	*params_For_Import_Puppet)
{
	Safe_Boolean			function_Result;
/* END VAR */
	
	
	function_Result = SAFE_TRUE;
	
	params_For_Import_Puppet->async_Task_Desired = SAFE_FALSE;
	
	if (Get_New_Puppet_Information(params_For_Import_Puppet->puppet_File_Name
								, params_For_Import_Puppet->puppet_Short_Name
								, params_For_Import_Puppet->puppet_Description) != TRUE) {
		function_Result = SAFE_FALSE;
	}
	params_For_Import_Puppet->import_Source = k_IMPORT_FROM_SPIN_FILE;

	
	return(function_Result);
}
/*=**************************************************************************=*/


/*=**************************************************************************=*/
Safe_Boolean Create_Puppet_From_Spin_File_Menu_Preparation(Import_Puppet_Rec	*params_For_Import_Puppet)
{
	Safe_Boolean			function_Result;
/* END VAR */
	
	
	function_Result = SAFE_TRUE;
	
	params_For_Import_Puppet->async_Task_Desired = SAFE_FALSE;
	
	
	if (Get_New_Puppet_Information(params_For_Import_Puppet->puppet_File_Name
								, params_For_Import_Puppet->puppet_Short_Name
								, params_For_Import_Puppet->puppet_Description) != TRUE) {
		function_Result = SAFE_FALSE;
	}
	params_For_Import_Puppet->import_Source = k_IMPORT_FROM_SPIN_FILE;

	return(function_Result);
}
/*=**************************************************************************=*/


/*=**************************************************************************=*/
Safe_Boolean Add_Another_Spin_File_To_Puppet_Control_Preparation(Add_Another_Spin_File_Rec	*params_For_Add_Another_Spin_File_To_Puppet)
{
	Safe_Boolean			function_Result;
	CARDINAL_32		resulting_Index;
	Safe_Boolean 		valid_Result;
	Str255					message;
/* END VAR */
	
	
	function_Result = SAFE_TRUE;
	
	params_For_Add_Another_Spin_File_To_Puppet->async_Task_Desired = SAFE_FALSE;
	
	if (Get_Dream_Popup_Menu_Choice(k_PUPPET_SELECTION
											, &resulting_Index
											, &valid_Result) != SAFE_TRUE) {
		function_Result = SAFE_FALSE;
	}
	else {
		if (valid_Result != SAFE_TRUE) {
			function_Result = SAFE_FALSE;
		}
		else {
		
			if ((dream.puppet_Memory_Initialized[resulting_Index] != MEMORY_INITIALIZED)
				|| ((*dream.puppet[resulting_Index])->puppet_File_Is_Old_Self_Contained_Type == SAFE_TRUE)) {
				
				Get_NS_String(k_DREAM_PROMPT_STRINGS, k_ERROR_PUPPET_IS_NOT_SPIN_FILE_TYPE, message);
				Perform_OK_Dialog(&message[0]);
			
				function_Result = SAFE_FALSE;
			}
			else {
				params_For_Add_Another_Spin_File_To_Puppet->desired_Item = resulting_Index;
			}
		}
	}

	return(function_Result);
}
/*=**************************************************************************=*/


/*=**************************************************************************=*/
Safe_Boolean Add_Another_Spin_File_To_Puppet_Menu_Preparation(Add_Another_Spin_File_Rec	*params_For_Add_Another_Spin_File_To_Puppet)
{
	Safe_Boolean			function_Result;
	CARDINAL_32		resulting_Index;
	Safe_Boolean 		valid_Result;
	Str255					message;
/* END VAR */
	
	
	function_Result = SAFE_TRUE;
	
	params_For_Add_Another_Spin_File_To_Puppet->async_Task_Desired = SAFE_FALSE;
	
	if (Get_Dream_Popup_Menu_Choice(k_PUPPET_SELECTION
											, &resulting_Index
											, &valid_Result) != SAFE_TRUE) {
		function_Result = SAFE_FALSE;
	}
	else {
		if (valid_Result != SAFE_TRUE) {
			function_Result = SAFE_FALSE;
		}
		else {
			if ((dream.puppet_Memory_Initialized[resulting_Index] != MEMORY_INITIALIZED)
				|| ((*dream.puppet[resulting_Index])->puppet_File_Is_Old_Self_Contained_Type == SAFE_TRUE)) {
				
				Get_NS_String(k_DREAM_PROMPT_STRINGS, k_ERROR_PUPPET_IS_NOT_SPIN_FILE_TYPE, message);
				Perform_OK_Dialog(&message[0]);
			
				function_Result = SAFE_FALSE;
			}
			else {
				params_For_Add_Another_Spin_File_To_Puppet->desired_Item = resulting_Index;
			}
		}
	}

	return(function_Result);
}
/*=**************************************************************************=*/


/*=**************************************************************************=*/
Safe_Boolean Export_Puppet_Control_Preparation(Export_Puppet_Rec	*params_For_Export_Puppet)
{
	Safe_Boolean			function_Result;
	CARDINAL_32		resulting_Index;
	Safe_Boolean 		valid_Result;
/* END VAR */
	
	
	function_Result = SAFE_TRUE;
	
	params_For_Export_Puppet->async_Task_Desired = SAFE_FALSE;
	
	if (Get_Dream_Popup_Menu_Choice(k_PUPPET_SELECTION
											, &resulting_Index
											, &valid_Result) != SAFE_TRUE) {
		function_Result = SAFE_FALSE;
	}
	else {
		if (valid_Result != SAFE_TRUE) {
			function_Result = SAFE_FALSE;
		}
		else {
			params_For_Export_Puppet->desired_Item = resulting_Index;
		}
	}
	
	return(function_Result);
}
/*=**************************************************************************=*/


/*=**************************************************************************=*/
Safe_Boolean Export_Puppet_Menu_Preparation(Export_Puppet_Rec	*params_For_Export_Puppet)
{
	Safe_Boolean			function_Result;
	CARDINAL_32		resulting_Index;
	Safe_Boolean 		valid_Result;
/* END VAR */
	
	
	function_Result = SAFE_TRUE;
	
	params_For_Export_Puppet->async_Task_Desired = SAFE_FALSE;
	
	if (Get_Dream_Popup_Menu_Choice(k_PUPPET_SELECTION
											, &resulting_Index
											, &valid_Result) != SAFE_TRUE) {
		function_Result = SAFE_FALSE;
	}
	else {
		if (valid_Result != SAFE_TRUE) {
			function_Result = SAFE_FALSE;
		}
		else {
			params_For_Export_Puppet->desired_Item = resulting_Index;
		}
	}
	
	return(function_Result);
}
/*=**************************************************************************=*/


/*=**************************************************************************=*/
Safe_Boolean Import_Prop_Control_Preparation(Import_Prop_Rec	*params_For_Import_Prop)
{
	Safe_Boolean			function_Result;
/* END VAR */
	
	
	function_Result = SAFE_TRUE;
	
	params_For_Import_Prop->async_Task_Desired = SAFE_FALSE;
	
	if (Get_New_Puppet_Information(params_For_Import_Prop->prop_File_Name
								, params_For_Import_Prop->prop_Short_Name
								, params_For_Import_Prop->prop_Description) != TRUE) {
		function_Result = SAFE_FALSE;
	}

	return(function_Result);
}
/*=**************************************************************************=*/


/*=**************************************************************************=*/
Safe_Boolean Import_Prop_Menu_Preparation(Import_Prop_Rec	*params_For_Import_Prop)
{
	Safe_Boolean			function_Result;
/* END VAR */
	
	
	function_Result = SAFE_TRUE;
	
	params_For_Import_Prop->async_Task_Desired = SAFE_FALSE;
	
	if (Get_New_Puppet_Information(params_For_Import_Prop->prop_File_Name
								, params_For_Import_Prop->prop_Short_Name
								, params_For_Import_Prop->prop_Description) != TRUE) {
		function_Result = SAFE_FALSE;
	}

	return(function_Result);
}
/*=**************************************************************************=*/


/*=**************************************************************************=*/
Safe_Boolean Export_Prop_Control_Preparation(Export_Prop_Rec	*params_For_Export_Prop)
{
	Safe_Boolean			function_Result;
/* END VAR */
	
	
	function_Result = SAFE_TRUE;
	
	params_For_Export_Prop->async_Task_Desired = SAFE_FALSE;
	
	
	return(function_Result);
}
/*=**************************************************************************=*/


/*=**************************************************************************=*/
Safe_Boolean Export_Prop_Menu_Preparation(Export_Prop_Rec	*params_For_Export_Prop)
{
	Safe_Boolean			function_Result;
/* END VAR */
	
	
	function_Result = SAFE_TRUE;
	
	params_For_Export_Prop->async_Task_Desired = SAFE_FALSE;
	
	
	return(function_Result);
}
/*=**************************************************************************=*/


/*=**************************************************************************=*/
Safe_Boolean Time_Animation_Control_Preparation(Time_Animation_Rec	*params_For_Time_Animation)
{
	Safe_Boolean			function_Result;
	CARDINAL_32		resulting_Index;
	Safe_Boolean 		valid_Result;
	Boolean				three_D_Test;
/* END VAR */
	
	
	function_Result = SAFE_TRUE;
	
	params_For_Time_Animation->async_Task_Desired = SAFE_FALSE;
	
	three_D_Test = FALSE;
	if (dream.spin_Objects.total_Spin_Objects > 0) {
		if ((active_Window_List[MAIN_SPINNING_WINDOW_INDEX] != nil)
			&& ((**active_Window_List[MAIN_SPINNING_WINDOW_INDEX]).structure_Initialized == SAFE_TRUE)) {
			three_D_Test = TRUE;
		}
	}

	if (three_D_Test == TRUE) {
		params_For_Time_Animation->desired_Item =0;
	} /* if (three_D_Test != TRUE) */
	else {
		
		if (Get_Dream_Popup_Menu_Choice(k_ANIMATED_PUPPET_IN_SCENE_SELECTION
												, &resulting_Index
												, &valid_Result) != SAFE_TRUE) {
			function_Result = SAFE_FALSE;
		}
		else {
			if (valid_Result != SAFE_TRUE) {
				function_Result = SAFE_FALSE;
			}
			else {
				params_For_Time_Animation->desired_Item = resulting_Index;
			}
		}
	}
	
	
	return(function_Result);
}
/*=**************************************************************************=*/


/*=**************************************************************************=*/
Safe_Boolean Time_Animation_Menu_Preparation(Time_Animation_Rec	*params_For_Time_Animation)
{
	Safe_Boolean			function_Result;
	CARDINAL_32		resulting_Index;
	Safe_Boolean 		valid_Result;
	Boolean				three_D_Test;
/* END VAR */
	
	
	function_Result = SAFE_TRUE;
	
	params_For_Time_Animation->async_Task_Desired = SAFE_FALSE;
	
	three_D_Test = FALSE;
	if (dream.spin_Objects.total_Spin_Objects > 0) {
		if ((active_Window_List[MAIN_SPINNING_WINDOW_INDEX] != nil)
			&& ((**active_Window_List[MAIN_SPINNING_WINDOW_INDEX]).structure_Initialized == SAFE_TRUE)) {
			three_D_Test = TRUE;
		}
	}

	if (three_D_Test == TRUE) {
		params_For_Time_Animation->desired_Item =0;
	} /* if (three_D_Test != TRUE) */
	else {
	
		if (Get_Dream_Popup_Menu_Choice(k_ANIMATED_PUPPET_IN_SCENE_SELECTION
												, &resulting_Index
												, &valid_Result) != SAFE_TRUE) {
			function_Result = SAFE_FALSE;
		}
		else {
			if (valid_Result != SAFE_TRUE) {
				function_Result = SAFE_FALSE;
			}
			else {
				params_For_Time_Animation->desired_Item = resulting_Index;
			}
		}
	}
	
	return(function_Result);
}
/*=**************************************************************************=*/


/*=**************************************************************************=*/
Safe_Boolean Set_Life_Jiggle_Control_Preparation(Set_Life_Jiggle_Rec	*params_For_Set_Life_Jiggle)
{
	Safe_Boolean			function_Result;
	CARDINAL_32		resulting_Index;
	Safe_Boolean 		valid_Result;
	INTEGER_32 			selected_Value;
	Str255					message;
/* END VAR */
	
	
	function_Result = SAFE_TRUE;
	
	params_For_Set_Life_Jiggle->async_Task_Desired = SAFE_FALSE;
	params_For_Set_Life_Jiggle->horizontal_Jiggle_Maximum = params_For_Set_Life_Jiggle->horizontal_Jiggle_Maximum;
	params_For_Set_Life_Jiggle->vertical_Jiggle_Maximum = params_For_Set_Life_Jiggle->vertical_Jiggle_Maximum;
	params_For_Set_Life_Jiggle->jiggle_Enabled = params_For_Set_Life_Jiggle->jiggle_Enabled;
	
	Get_NS_String(k_DREAM_PROMPT_STRINGS, k_ENTER_MAXIMUM_JITTER_DISTANCE_VERTICAL, message);

	selected_Value = dream.vertical_Jiggle_Maximum;
	if (Get_Long_Dialog(&message[0]
								, &selected_Value
								, 0
								, k_MAXIMUM_VERTICAL_JIGGLE_PIXELS) == TRUE) {
		params_For_Set_Life_Jiggle->vertical_Jiggle_Maximum = selected_Value;
	}
	else {
		function_Result = SAFE_FALSE;
	}
	
	if (function_Result == SAFE_TRUE) {
		Get_NS_String(k_DREAM_PROMPT_STRINGS, k_ENTER_MAXIMUM_JITTER_DISTANCE_HORIZONTAL, message);

		selected_Value = dream.horizontal_Jiggle_Maximum;
		if (Get_Long_Dialog(&message[0]
									, &selected_Value
									, 0
									, k_MAXIMUM_HORIZONTAL_JIGGLE_PIXELS) == TRUE) {
			params_For_Set_Life_Jiggle->horizontal_Jiggle_Maximum = selected_Value;
		}
		else {
			function_Result = SAFE_FALSE;
		}
	}

	if (function_Result == SAFE_TRUE) {
		if ((params_For_Set_Life_Jiggle->horizontal_Jiggle_Maximum != 0) || (params_For_Set_Life_Jiggle->vertical_Jiggle_Maximum != 0)) {
			params_For_Set_Life_Jiggle->jiggle_Enabled = SAFE_TRUE;
		}
		else {
			params_For_Set_Life_Jiggle->jiggle_Enabled = SAFE_FALSE;
		}
	}	
	
	return(function_Result);
}
/*=**************************************************************************=*/


/*=**************************************************************************=*/
Safe_Boolean Set_Life_Jiggle_Menu_Preparation(Set_Life_Jiggle_Rec	*params_For_Set_Life_Jiggle)
{
	Safe_Boolean			function_Result;
	CARDINAL_32		resulting_Index;
	Safe_Boolean 		valid_Result;
	INTEGER_32 			selected_Value;
	Str255					message;
/* END VAR */
	
	
	function_Result = SAFE_TRUE;
	
	params_For_Set_Life_Jiggle->async_Task_Desired = SAFE_FALSE;
	params_For_Set_Life_Jiggle->horizontal_Jiggle_Maximum = params_For_Set_Life_Jiggle->horizontal_Jiggle_Maximum;
	params_For_Set_Life_Jiggle->vertical_Jiggle_Maximum = params_For_Set_Life_Jiggle->vertical_Jiggle_Maximum;
	params_For_Set_Life_Jiggle->jiggle_Enabled = params_For_Set_Life_Jiggle->jiggle_Enabled;
	
	Get_NS_String(k_DREAM_PROMPT_STRINGS, k_ENTER_MAXIMUM_JITTER_DISTANCE_VERTICAL, message);

	selected_Value = dream.vertical_Jiggle_Maximum;
	if (Get_Long_Dialog(&message[0]
								, &selected_Value
								, 0
								, k_MAXIMUM_VERTICAL_JIGGLE_PIXELS) == TRUE) {
		params_For_Set_Life_Jiggle->vertical_Jiggle_Maximum = selected_Value;
	}
	else {
		function_Result = SAFE_FALSE;
	}
	
	if (function_Result == SAFE_TRUE) {
		Get_NS_String(k_DREAM_PROMPT_STRINGS, k_ENTER_MAXIMUM_JITTER_DISTANCE_HORIZONTAL, message);

		selected_Value = dream.horizontal_Jiggle_Maximum;
		if (Get_Long_Dialog(&message[0]
									, &selected_Value
									, 0
									, k_MAXIMUM_HORIZONTAL_JIGGLE_PIXELS) == TRUE) {
			params_For_Set_Life_Jiggle->horizontal_Jiggle_Maximum = selected_Value;
		}
		else {
			function_Result = SAFE_FALSE;
		}
	}

	if (function_Result == SAFE_TRUE) {
		if ((params_For_Set_Life_Jiggle->horizontal_Jiggle_Maximum != 0) || (params_For_Set_Life_Jiggle->vertical_Jiggle_Maximum != 0)) {
			params_For_Set_Life_Jiggle->jiggle_Enabled = SAFE_TRUE;
		}
		else {
			params_For_Set_Life_Jiggle->jiggle_Enabled = SAFE_FALSE;
		}
	}	
	
	return(function_Result);
}
/*=**************************************************************************=*/


/*=**************************************************************************=*/
Safe_Boolean Close_Backdrop_Control_Preparation(Close_Backdrop_Rec	*params_For_Close_Backdrop)
{
	Safe_Boolean			function_Result;
	CARDINAL_32		resulting_Index;
	Safe_Boolean 		valid_Result;
/* END VAR */
	
	
	function_Result = SAFE_TRUE;
	
	params_For_Close_Backdrop->async_Task_Desired = SAFE_FALSE;
	
	if (Get_Dream_Popup_Menu_Choice(k_BACKDROP_SELECTION
											, &resulting_Index
											, &valid_Result) != SAFE_TRUE) {
		function_Result = SAFE_FALSE;
	}
	else {
		if (valid_Result != SAFE_TRUE) {
			function_Result = SAFE_FALSE;
		}
		else {
			params_For_Close_Backdrop->desired_Backdrop = resulting_Index;
		}
	}	
	
	return(function_Result);
}
/*=**************************************************************************=*/


/*=**************************************************************************=*/
Safe_Boolean Close_Backdrop_Menu_Preparation(Close_Backdrop_Rec	*params_For_Close_Backdrop)
{
	Safe_Boolean			function_Result;
	CARDINAL_32		resulting_Index;
	Safe_Boolean 		valid_Result;
/* END VAR */
	
	
	function_Result = SAFE_TRUE;
	
	params_For_Close_Backdrop->async_Task_Desired = SAFE_FALSE;
	
	if (Get_Dream_Popup_Menu_Choice(k_BACKDROP_SELECTION
											, &resulting_Index
											, &valid_Result) != SAFE_TRUE) {
		function_Result = SAFE_FALSE;
	}
	else {
		if (valid_Result != SAFE_TRUE) {
			function_Result = SAFE_FALSE;
		}
		else {
			params_For_Close_Backdrop->desired_Backdrop = resulting_Index;
		}
	}	
	
	return(function_Result);
}
/*=**************************************************************************=*/


/*=**************************************************************************=*/
Safe_Boolean Load_Backdrop_Control_Preparation(Load_Backdrop_Rec	*params_For_Load_Backdrop)
{
	Safe_Boolean			function_Result;
	Safe_Boolean 		user_Abort;
/* END VAR */
	
	
	function_Result = SAFE_TRUE;
	
	params_For_Load_Backdrop->async_Task_Desired = SAFE_FALSE;
	
	if (Specify_Backdrop_To_Load(&(params_For_Load_Backdrop->desired_File)
											, &user_Abort) != SAFE_TRUE) {
		function_Result = SAFE_FALSE;
	}
	else {
		if (user_Abort == SAFE_TRUE) {
			function_Result = SAFE_FALSE;
		}
	}	
	
	return(function_Result);
}
/*=**************************************************************************=*/


/*=**************************************************************************=*/
Safe_Boolean Load_Backdrop_Menu_Preparation(Load_Backdrop_Rec	*params_For_Load_Backdrop)
{
	Safe_Boolean			function_Result;
	Safe_Boolean 		user_Abort;
/* END VAR */
	
	
	function_Result = SAFE_TRUE;
	
	params_For_Load_Backdrop->async_Task_Desired = SAFE_FALSE;

	if (Specify_Backdrop_To_Load(&(params_For_Load_Backdrop->desired_File)
											, &user_Abort) != SAFE_TRUE) {
		function_Result = SAFE_FALSE;
	}
	else {
		if (user_Abort == SAFE_TRUE) {
			function_Result = SAFE_FALSE;
		}
	}
	
	return(function_Result);
}
/*=**************************************************************************=*/


/*=**************************************************************************=*/
Safe_Boolean Save_Backdrop_Control_Preparation(Save_Backdrop_Rec	*params_For_Save_Backdrop)
{
	Safe_Boolean			function_Result;
	Safe_Boolean 		user_Abort;
	CARDINAL_32		resulting_Index;
	Safe_Boolean 		valid_Result;
/* END VAR */
	
	
	function_Result = SAFE_FALSE;
	
	params_For_Save_Backdrop->async_Task_Desired = SAFE_FALSE;
	
	if (Get_Dream_Popup_Menu_Choice(k_BACKDROP_SELECTION
											, &resulting_Index
											, &valid_Result) != SAFE_TRUE) {
		goto Error_Handler;
	}
	else {
		if (valid_Result != SAFE_TRUE) {
			goto Error_Handler;
		}
		else {
			params_For_Save_Backdrop->desired_Backdrop = resulting_Index;
		}
	}	
	
	if (dream.backdrops.backdrop_Memory_In_Use[resulting_Index] != MEMORY_INITIALIZED) {
		goto Error_Handler;
	}

	if (Specify_Backdrop_To_Save(&(params_For_Save_Backdrop->desired_File)
											, &((**(dream.backdrops.backdrop[resulting_Index])).backdrop_Name)
											, &user_Abort) != SAFE_TRUE) {
		goto Error_Handler;
	}
	else {
		if (user_Abort == SAFE_TRUE) {
			goto Error_Handler;
		}
	}
	
	Copy_Pascal_String(params_For_Save_Backdrop->desired_File.name
			, ((**(dream.backdrops.backdrop[resulting_Index])).backdrop_Name));

	function_Result = SAFE_TRUE;
	goto Normal_Function_Exit;
	
Error_Handler:
	function_Result = SAFE_FALSE;
			
Normal_Function_Exit:
	return(function_Result);
}
/*=**************************************************************************=*/


/*=**************************************************************************=*/
Safe_Boolean Save_Backdrop_Menu_Preparation(Save_Backdrop_Rec	*params_For_Save_Backdrop)
{
	Safe_Boolean			function_Result;
	Safe_Boolean 		user_Abort;
	CARDINAL_32		resulting_Index;
	Safe_Boolean 		valid_Result;
/* END VAR */
	
	
	function_Result = SAFE_FALSE;
	
	params_For_Save_Backdrop->async_Task_Desired = SAFE_FALSE;
	
	if (Get_Dream_Popup_Menu_Choice(k_BACKDROP_SELECTION
											, &resulting_Index
											, &valid_Result) != SAFE_TRUE) {
		goto Error_Handler;
	}
	else {
		if (valid_Result != SAFE_TRUE) {
			goto Error_Handler;
		}
		else {
			params_For_Save_Backdrop->desired_Backdrop = resulting_Index;
		}
	}	
	
	if (dream.backdrops.backdrop_Memory_In_Use[resulting_Index] != MEMORY_INITIALIZED) {
		goto Error_Handler;
	}

	if (Specify_Backdrop_To_Save(&(params_For_Save_Backdrop->desired_File)
											, &((**(dream.backdrops.backdrop[resulting_Index])).backdrop_Name)
											, &user_Abort) != SAFE_TRUE) {
		goto Error_Handler;
	}
	else {
		if (user_Abort == SAFE_TRUE) {
			goto Error_Handler;
		}
	}
	
	Copy_Pascal_String(params_For_Save_Backdrop->desired_File.name
			, ((**(dream.backdrops.backdrop[resulting_Index])).backdrop_Name));

	function_Result = SAFE_TRUE;
	goto Normal_Function_Exit;
	
Error_Handler:
	function_Result = SAFE_FALSE;
			
Normal_Function_Exit:
	return(function_Result);
	
}
/*=**************************************************************************=*/


/*=**************************************************************************=*/
Safe_Boolean Convert_Backdrop_Control_Preparation(Convert_Backdrop_Rec	*params_For_Convert_Backdrop)
{
	Safe_Boolean			function_Result;
	Safe_Boolean 		user_Abort;
/* END VAR */
	
	
	function_Result = SAFE_TRUE;
	
	params_For_Convert_Backdrop->desired_Name[0] = 0;
	params_For_Convert_Backdrop->async_Task_Desired = SAFE_FALSE;
	function_Result = Specify_Pict_To_Import(&(params_For_Convert_Backdrop->desired_File)
															, &(params_For_Convert_Backdrop->file_Is_Pict_File_Type)
															, &(params_For_Convert_Backdrop->desired_Resource_Index)
															, &user_Abort
															, &(params_For_Convert_Backdrop->selected_Pict_Width)
															, &(params_For_Convert_Backdrop->selected_Pict_Height));
	
	if (function_Result == SAFE_TRUE) {
		Copy_Pascal_String(params_For_Convert_Backdrop->desired_File.name
				, (params_For_Convert_Backdrop->desired_Name));
	}
	
	return(function_Result);
}
/*=**************************************************************************=*/


/*=**************************************************************************=*/
Safe_Boolean Convert_Backdrop_Menu_Preparation(Convert_Backdrop_Rec	*params_For_Convert_Backdrop)
{
	Safe_Boolean			function_Result;
	Safe_Boolean 		user_Abort;
/* END VAR */
	
	
	function_Result = SAFE_TRUE;
	
	params_For_Convert_Backdrop->desired_Name[0] = 0;
	params_For_Convert_Backdrop->async_Task_Desired = SAFE_FALSE;
	function_Result = Specify_Pict_To_Import(&(params_For_Convert_Backdrop->desired_File)
															, &(params_For_Convert_Backdrop->file_Is_Pict_File_Type)
															, &(params_For_Convert_Backdrop->desired_Resource_Index)
															, &user_Abort
															, &(params_For_Convert_Backdrop->selected_Pict_Width)
															, &(params_For_Convert_Backdrop->selected_Pict_Height));
	
	if (function_Result == SAFE_TRUE) {
		Copy_Pascal_String(params_For_Convert_Backdrop->desired_File.name
				, (params_For_Convert_Backdrop->desired_Name));
	}
	
	return(function_Result);
}
/*=**************************************************************************=*/


/*=**************************************************************************=*/
Safe_Boolean Export_Backdrop_Control_Preparation(Export_Backdrop_Rec	*params_For_Export_Backdrop)
{
	Safe_Boolean			function_Result;
	Safe_Boolean 		user_Abort;
	CARDINAL_32		resulting_Index;
	Safe_Boolean 		valid_Result;
/* END VAR */
	
	
	function_Result = SAFE_FALSE;
	
	params_For_Export_Backdrop->async_Task_Desired = SAFE_FALSE;
	
	if (Get_Dream_Popup_Menu_Choice(k_BACKDROP_SELECTION
											, &resulting_Index
											, &valid_Result) != SAFE_TRUE) {
		goto Error_Handler;
	}
	else {
		if (valid_Result != SAFE_TRUE) {
			goto Error_Handler;
		}
		else {
			params_For_Export_Backdrop->desired_Backdrop = resulting_Index;
		}
	}	
	
	if (dream.backdrops.backdrop_Memory_In_Use[resulting_Index] != MEMORY_INITIALIZED) {
		goto Error_Handler;
	}

	if (Specify_Backdrop_To_Export(&(params_For_Export_Backdrop->desired_File)
											, &((**(dream.backdrops.backdrop[resulting_Index])).backdrop_Name)
											, &user_Abort) != SAFE_TRUE) {
		goto Error_Handler;
	}
	else {
		if (user_Abort == SAFE_TRUE) {
			goto Error_Handler;
		}
	}
	
	function_Result = SAFE_TRUE;
	goto Normal_Function_Exit;
	
Error_Handler:
	function_Result = SAFE_FALSE;
			
Normal_Function_Exit:
	return(function_Result);
}
/*=**************************************************************************=*/


/*=**************************************************************************=*/
Safe_Boolean Export_Backdrop_Menu_Preparation(Export_Backdrop_Rec	*params_For_Export_Backdrop)
{
	Safe_Boolean			function_Result;
	Safe_Boolean 		user_Abort;
	CARDINAL_32		resulting_Index;
	Safe_Boolean 		valid_Result;
/* END VAR */
	
	
	function_Result = SAFE_FALSE;
	
	params_For_Export_Backdrop->async_Task_Desired = SAFE_FALSE;
	
	if (Get_Dream_Popup_Menu_Choice(k_BACKDROP_SELECTION
											, &resulting_Index
											, &valid_Result) != SAFE_TRUE) {
		goto Error_Handler;
	}
	else {
		if (valid_Result != SAFE_TRUE) {
			goto Error_Handler;
		}
		else {
			params_For_Export_Backdrop->desired_Backdrop = resulting_Index;
		}
	}	
	
	if (dream.backdrops.backdrop_Memory_In_Use[resulting_Index] != MEMORY_INITIALIZED) {
		goto Error_Handler;
	}

	if (Specify_Backdrop_To_Export(&(params_For_Export_Backdrop->desired_File)
											, &((**(dream.backdrops.backdrop[resulting_Index])).backdrop_Name)
											, &user_Abort) != SAFE_TRUE) {
		goto Error_Handler;
	}
	else {
		if (user_Abort == SAFE_TRUE) {
			goto Error_Handler;
		}
	}
	
	function_Result = SAFE_TRUE;
	goto Normal_Function_Exit;
	
Error_Handler:
	function_Result = SAFE_FALSE;
			
Normal_Function_Exit:
	return(function_Result);
}
/*=**************************************************************************=*/


/*=**************************************************************************=*/
Safe_Boolean Close_Spin_Object_Control_Preparation(Close_Spin_Object_Rec	*params_For_Close_Spin_Object)
{
	Safe_Boolean			function_Result;
	CARDINAL_32		resulting_Index;
	Safe_Boolean 		valid_Result;
/* END VAR */
	
	
	function_Result = SAFE_TRUE;
	
	params_For_Close_Spin_Object->async_Task_Desired = SAFE_FALSE;
	
	if (Get_Dream_Popup_Menu_Choice(k_SPIN_OBJECT_SELECTION
											, &resulting_Index
											, &valid_Result) != SAFE_TRUE) {
		function_Result = SAFE_FALSE;
	}
	else {
		if (valid_Result != SAFE_TRUE) {
			function_Result = SAFE_FALSE;
		}
		else {
			params_For_Close_Spin_Object->desired_Spin_Object = resulting_Index;
		}
	}	
	
	return(function_Result);
}
/*=**************************************************************************=*/


/*=**************************************************************************=*/
Safe_Boolean Close_Spin_Object_Menu_Preparation(Close_Spin_Object_Rec	*params_For_Close_Spin_Object)
{
	Safe_Boolean			function_Result;
	CARDINAL_32		resulting_Index;
	Safe_Boolean 		valid_Result;
/* END VAR */
	
	
	function_Result = SAFE_TRUE;
	
	params_For_Close_Spin_Object->async_Task_Desired = SAFE_FALSE;
	
	if (Get_Dream_Popup_Menu_Choice(k_SPIN_OBJECT_SELECTION
											, &resulting_Index
											, &valid_Result) != SAFE_TRUE) {
		function_Result = SAFE_FALSE;
	}
	else {
		if (valid_Result != SAFE_TRUE) {
			function_Result = SAFE_FALSE;
		}
		else {
			params_For_Close_Spin_Object->desired_Spin_Object = resulting_Index;
		}
	}	
	
	return(function_Result);
}
/*=**************************************************************************=*/


/*=**************************************************************************=*/
Safe_Boolean Load_Spin_Object_Control_Preparation(Load_Spin_Object_Rec	*params_For_Load_Spin_Object)
{
	Safe_Boolean			function_Result;
	Safe_Boolean 		user_Abort;
/* END VAR */
	
	
	function_Result = SAFE_TRUE;
	
	params_For_Load_Spin_Object->async_Task_Desired = SAFE_FALSE;
	params_For_Load_Spin_Object->load_File_With_Self_Contained_Large_Ports = SAFE_TRUE;
	
	if (Specify_Spin_Object_To_Load(&(params_For_Load_Spin_Object->desired_File)
											, &user_Abort) != SAFE_TRUE) {
		function_Result = SAFE_FALSE;
	}
	else {
		if (user_Abort == SAFE_TRUE) {
			function_Result = SAFE_FALSE;
		}
	}	
	
	return(function_Result);
}
/*=**************************************************************************=*/


/*=**************************************************************************=*/
Safe_Boolean Load_Spin_Object_Menu_Preparation(Load_Spin_Object_Rec	*params_For_Load_Spin_Object)
{
	Safe_Boolean			function_Result;
	Safe_Boolean 		user_Abort;
/* END VAR */
	
	
	function_Result = SAFE_TRUE;
	
	params_For_Load_Spin_Object->async_Task_Desired = SAFE_FALSE;
	params_For_Load_Spin_Object->load_File_With_Self_Contained_Large_Ports = SAFE_TRUE;

	if (Specify_Spin_Object_To_Load(&(params_For_Load_Spin_Object->desired_File)
											, &user_Abort) != SAFE_TRUE) {
		function_Result = SAFE_FALSE;
	}
	else {
		if (user_Abort == SAFE_TRUE) {
			function_Result = SAFE_FALSE;
		}
	}
	
	return(function_Result);
}
/*=**************************************************************************=*/


/*=**************************************************************************=*/
Safe_Boolean Save_Spin_Object_Control_Preparation(Save_Spin_Object_Rec	*params_For_Save_Spin_Object)
{
	Safe_Boolean			function_Result;
	Safe_Boolean 		user_Abort;
	CARDINAL_32		resulting_Index;
	Safe_Boolean 		valid_Result;
/* END VAR */
	
	
	function_Result = SAFE_FALSE;
	
	params_For_Save_Spin_Object->async_Task_Desired = SAFE_FALSE;
	
	if (Get_Dream_Popup_Menu_Choice(k_SPIN_OBJECT_SELECTION
											, &resulting_Index
											, &valid_Result) != SAFE_TRUE) {
		goto Error_Handler;
	}
	else {
		if (valid_Result != SAFE_TRUE) {
			goto Error_Handler;
		}
		else {
			params_For_Save_Spin_Object->desired_Spin_Object = resulting_Index;
		}
	}	
	
	if (dream.spin_Objects.spin_Object_Memory_In_Use[resulting_Index] != MEMORY_INITIALIZED) {
		goto Error_Handler;
	}

	if (Specify_Spin_Object_To_Save(&(params_For_Save_Spin_Object->desired_File)
											, &((**(dream.spin_Objects.spin_Object[resulting_Index])).spin_Object_Name)
											, &user_Abort) != SAFE_TRUE) {
		goto Error_Handler;
	}
	else {
		if (user_Abort == SAFE_TRUE) {
			goto Error_Handler;
		}
	}
	
	Copy_Pascal_String(params_For_Save_Spin_Object->desired_File.name
			, ((**(dream.spin_Objects.spin_Object[resulting_Index])).spin_Object_Name));

	function_Result = SAFE_TRUE;
	goto Normal_Function_Exit;
	
Error_Handler:
	function_Result = SAFE_FALSE;
			
Normal_Function_Exit:
	return(function_Result);
}
/*=**************************************************************************=*/


/*=**************************************************************************=*/
Safe_Boolean Save_Spin_Object_Menu_Preparation(Save_Spin_Object_Rec	*params_For_Save_Spin_Object)
{
	Safe_Boolean			function_Result;
	Safe_Boolean 		user_Abort;
	CARDINAL_32		resulting_Index;
	Safe_Boolean 		valid_Result;
/* END VAR */
	
	
	function_Result = SAFE_FALSE;
	
	params_For_Save_Spin_Object->async_Task_Desired = SAFE_FALSE;
	
	if (Get_Dream_Popup_Menu_Choice(k_SPIN_OBJECT_SELECTION
											, &resulting_Index
											, &valid_Result) != SAFE_TRUE) {
		goto Error_Handler;
	}
	else {
		if (valid_Result != SAFE_TRUE) {
			goto Error_Handler;
		}
		else {
			params_For_Save_Spin_Object->desired_Spin_Object = resulting_Index;
		}
	}	
	
	if (dream.spin_Objects.spin_Object_Memory_In_Use[resulting_Index] != MEMORY_INITIALIZED) {
		goto Error_Handler;
	}

	if (Specify_Spin_Object_To_Save(&(params_For_Save_Spin_Object->desired_File)
											, &((**(dream.spin_Objects.spin_Object[resulting_Index])).spin_Object_Name)
											, &user_Abort) != SAFE_TRUE) {
		goto Error_Handler;
	}
	else {
		if (user_Abort == SAFE_TRUE) {
			goto Error_Handler;
		}
	}
	
	Copy_Pascal_String(params_For_Save_Spin_Object->desired_File.name
			, ((**(dream.spin_Objects.spin_Object[resulting_Index])).spin_Object_Name));

	function_Result = SAFE_TRUE;
	goto Normal_Function_Exit;
	
Error_Handler:
	function_Result = SAFE_FALSE;
			
Normal_Function_Exit:
	return(function_Result);
	
}
/*=**************************************************************************=*/


/*=**************************************************************************=*/
Safe_Boolean Convert_Spin_Object_Control_Preparation(Convert_Spin_Object_Rec	*params_For_Convert_Spin_Object)
{
	Safe_Boolean			function_Result;
	Safe_Boolean 		user_Abort;
/* END VAR */
	
	
	function_Result = SAFE_TRUE;
	
	params_For_Convert_Spin_Object->desired_Name[0] = 0;
	params_For_Convert_Spin_Object->async_Task_Desired = SAFE_FALSE;
	function_Result = Specify_Spin_Object_Pict_To_Import(&(params_For_Convert_Spin_Object->desired_File)
															, &(params_For_Convert_Spin_Object->file_Is_Pict_File_Type)
															, &(params_For_Convert_Spin_Object->desired_Resource_Index)
															, &user_Abort
															, &(params_For_Convert_Spin_Object->selected_Pict_Width)
															, &(params_For_Convert_Spin_Object->selected_Pict_Height));
	
	if (function_Result == SAFE_TRUE) {
		Copy_Pascal_String(params_For_Convert_Spin_Object->desired_File.name
				, (params_For_Convert_Spin_Object->desired_Name));
	}
	
	return(function_Result);
}
/*=**************************************************************************=*/


/*=**************************************************************************=*/
Safe_Boolean Convert_Spin_Object_Menu_Preparation(Convert_Spin_Object_Rec	*params_For_Convert_Spin_Object)
{
	Safe_Boolean			function_Result;
	Safe_Boolean 		user_Abort;
/* END VAR */
	
	
	function_Result = SAFE_TRUE;
	
	params_For_Convert_Spin_Object->desired_Name[0] = 0;
	params_For_Convert_Spin_Object->async_Task_Desired = SAFE_FALSE;
	function_Result = Specify_Spin_Object_Pict_To_Import(&(params_For_Convert_Spin_Object->desired_File)
															, &(params_For_Convert_Spin_Object->file_Is_Pict_File_Type)
															, &(params_For_Convert_Spin_Object->desired_Resource_Index)
															, &user_Abort
															, &(params_For_Convert_Spin_Object->selected_Pict_Width)
															, &(params_For_Convert_Spin_Object->selected_Pict_Height));
	
	if (function_Result == SAFE_TRUE) {
		Copy_Pascal_String(params_For_Convert_Spin_Object->desired_File.name
				, (params_For_Convert_Spin_Object->desired_Name));
	}
	
	return(function_Result);
}
/*=**************************************************************************=*/


/*=**************************************************************************=*/
Safe_Boolean Export_Spin_Object_Control_Preparation(Export_Spin_Object_Rec	*params_For_Export_Spin_Object)
{
	Safe_Boolean			function_Result;
	Safe_Boolean 		user_Abort;
	CARDINAL_32		resulting_Index;
	Safe_Boolean 		valid_Result;
/* END VAR */
	
	
	function_Result = SAFE_FALSE;
	
	params_For_Export_Spin_Object->async_Task_Desired = SAFE_FALSE;
	
	if (Get_Dream_Popup_Menu_Choice(k_SPIN_OBJECT_SELECTION
											, &resulting_Index
											, &valid_Result) != SAFE_TRUE) {
		goto Error_Handler;
	}
	else {
		if (valid_Result != SAFE_TRUE) {
			goto Error_Handler;
		}
		else {
			params_For_Export_Spin_Object->desired_Spin_Object = resulting_Index;
		}
	}	
	
	if (dream.spin_Objects.spin_Object_Memory_In_Use[resulting_Index] != MEMORY_INITIALIZED) {
		goto Error_Handler;
	}

	if (Specify_Spin_Object_To_Export(&(params_For_Export_Spin_Object->desired_File)
											, &((**(dream.spin_Objects.spin_Object[resulting_Index])).spin_Object_Name)
											, &user_Abort) != SAFE_TRUE) {
		goto Error_Handler;
	}
	else {
		if (user_Abort == SAFE_TRUE) {
			goto Error_Handler;
		}
	}
	
	function_Result = SAFE_TRUE;
	goto Normal_Function_Exit;
	
Error_Handler:
	function_Result = SAFE_FALSE;
			
Normal_Function_Exit:
	return(function_Result);
}
/*=**************************************************************************=*/


/*=**************************************************************************=*/
Safe_Boolean Export_Spin_Object_Menu_Preparation(Export_Spin_Object_Rec	*params_For_Export_Spin_Object)
{
	Safe_Boolean			function_Result;
	Safe_Boolean 		user_Abort;
	CARDINAL_32		resulting_Index;
	Safe_Boolean 		valid_Result;
/* END VAR */
	
	
	function_Result = SAFE_FALSE;
	
	params_For_Export_Spin_Object->async_Task_Desired = SAFE_FALSE;
	
	if (Get_Dream_Popup_Menu_Choice(k_SPIN_OBJECT_SELECTION
											, &resulting_Index
											, &valid_Result) != SAFE_TRUE) {
		goto Error_Handler;
	}
	else {
		if (valid_Result != SAFE_TRUE) {
			goto Error_Handler;
		}
		else {
			params_For_Export_Spin_Object->desired_Spin_Object = resulting_Index;
		}
	}	
	
	if (dream.spin_Objects.spin_Object_Memory_In_Use[resulting_Index] != MEMORY_INITIALIZED) {
		goto Error_Handler;
	}

	if (Specify_Spin_Object_To_Export(&(params_For_Export_Spin_Object->desired_File)
											, &((**(dream.spin_Objects.spin_Object[resulting_Index])).spin_Object_Name)
											, &user_Abort) != SAFE_TRUE) {
		goto Error_Handler;
	}
	else {
		if (user_Abort == SAFE_TRUE) {
			goto Error_Handler;
		}
	}
	
	function_Result = SAFE_TRUE;
	goto Normal_Function_Exit;
	
Error_Handler:
	function_Result = SAFE_FALSE;
			
Normal_Function_Exit:
	return(function_Result);
}
/*=**************************************************************************=*/


/*=**************************************************************************=*/
Safe_Boolean Recompress_Spin_Object_Control_Preparation(Recompress_Spin_Object_Rec	*params_For_Recompress_Spin_Object)
{
	Safe_Boolean			function_Result;
	Safe_Boolean 		user_Abort;
	CARDINAL_32		resulting_Index;
	Safe_Boolean 		valid_Result;
/* END VAR */
	
	
	function_Result = SAFE_FALSE;
	
	params_For_Recompress_Spin_Object->async_Task_Desired = SAFE_FALSE;
	
	if (Get_Dream_Popup_Menu_Choice(k_SPIN_OBJECT_SELECTION
											, &resulting_Index
											, &valid_Result) != SAFE_TRUE) {
		goto Error_Handler;
	}
	else {
		if (valid_Result != SAFE_TRUE) {
			goto Error_Handler;
		}
		else {
			params_For_Recompress_Spin_Object->desired_Spin_Object = resulting_Index;
		}
	}	
	
	if (dream.spin_Objects.spin_Object_Memory_In_Use[resulting_Index] != MEMORY_INITIALIZED) {
		goto Error_Handler;
	}

	if (Specify_Spin_Object_To_Export(&(params_For_Recompress_Spin_Object->desired_File)
											, &((**(dream.spin_Objects.spin_Object[resulting_Index])).spin_Object_Name)
											, &user_Abort) != SAFE_TRUE) {
		goto Error_Handler;
	}
	else {
		if (user_Abort == SAFE_TRUE) {
			goto Error_Handler;
		}
	}
	
	function_Result = SAFE_TRUE;
	goto Normal_Function_Exit;
	
Error_Handler:
	function_Result = SAFE_FALSE;
			
Normal_Function_Exit:
	return(function_Result);
}
/*=**************************************************************************=*/


/*=**************************************************************************=*/
Safe_Boolean Recompress_Spin_Object_Menu_Preparation(Recompress_Spin_Object_Rec	*params_For_Recompress_Spin_Object)
{
	Safe_Boolean			function_Result;
	Safe_Boolean 		user_Abort;
	CARDINAL_32		resulting_Index;
	Safe_Boolean 		valid_Result;
/* END VAR */
	
	
	function_Result = SAFE_FALSE;
	
	params_For_Recompress_Spin_Object->async_Task_Desired = SAFE_FALSE;
	
	if (Get_Dream_Popup_Menu_Choice(k_SPIN_OBJECT_SELECTION
											, &resulting_Index
											, &valid_Result) != SAFE_TRUE) {
		goto Error_Handler;
	}
	else {
		if (valid_Result != SAFE_TRUE) {
			goto Error_Handler;
		}
		else {
			params_For_Recompress_Spin_Object->desired_Spin_Object = resulting_Index;
		}
	}	
	
	if (dream.spin_Objects.spin_Object_Memory_In_Use[resulting_Index] != MEMORY_INITIALIZED) {
		goto Error_Handler;
	}

	if (Specify_Spin_Object_To_Export(&(params_For_Recompress_Spin_Object->desired_File)
											, &((**(dream.spin_Objects.spin_Object[resulting_Index])).spin_Object_Name)
											, &user_Abort) != SAFE_TRUE) {
		goto Error_Handler;
	}
	else {
		if (user_Abort == SAFE_TRUE) {
			goto Error_Handler;
		}
	}
	
	function_Result = SAFE_TRUE;
	goto Normal_Function_Exit;
	
Error_Handler:
	function_Result = SAFE_FALSE;
			
Normal_Function_Exit:
	return(function_Result);
}
/*=**************************************************************************=*/


/*=**************************************************************************=*/
Safe_Boolean Show_Controls_Control_Preparation(Show_Controls_Rec	*params_For_Show_Controls)
{
	Safe_Boolean			function_Result;
/* END VAR */
	
	
	function_Result = SAFE_TRUE;
	
	params_For_Show_Controls->async_Task_Desired = SAFE_FALSE;
	
	
	return(function_Result);
}
/*=**************************************************************************=*/


/*=**************************************************************************=*/
Safe_Boolean Show_Controls_Menu_Preparation(Show_Controls_Rec	*params_For_Show_Controls)
{
	Safe_Boolean			function_Result;
/* END VAR */
	
	
	function_Result = SAFE_TRUE;
	
	params_For_Show_Controls->async_Task_Desired = SAFE_FALSE;
	
	
	return(function_Result);
}
/*=**************************************************************************=*/


/*=**************************************************************************=*/
Safe_Boolean Show_Status_Control_Preparation(Show_Status_Rec	*params_For_Show_Status)
{
	Safe_Boolean			function_Result;
/* END VAR */
	
	
	function_Result = SAFE_TRUE;
	
	params_For_Show_Status->async_Task_Desired = SAFE_FALSE;
	
	
	return(function_Result);
}
/*=**************************************************************************=*/


/*=**************************************************************************=*/
Safe_Boolean Show_Status_Menu_Preparation(Show_Status_Rec	*params_For_Show_Status)
{
	Safe_Boolean			function_Result;
/* END VAR */
	
	
	function_Result = SAFE_TRUE;
	
	params_For_Show_Status->async_Task_Desired = SAFE_FALSE;
	
	
	return(function_Result);
}
/*=**************************************************************************=*/


/*=**************************************************************************=*/
Safe_Boolean Show_Item_List_Control_Preparation(Show_Item_List_Rec	*params_For_Show_Item_List)
{
	Safe_Boolean			function_Result;
/* END VAR */
	
	
	function_Result = SAFE_TRUE;
	
	params_For_Show_Item_List->async_Task_Desired = SAFE_FALSE;
	
	
	return(function_Result);
}
/*=**************************************************************************=*/


/*=**************************************************************************=*/
Safe_Boolean Show_Item_List_Menu_Preparation(Show_Item_List_Rec	*params_For_Show_Item_List)
{
	Safe_Boolean			function_Result;
/* END VAR */
	
	
	function_Result = SAFE_TRUE;
	
	params_For_Show_Item_List->async_Task_Desired = SAFE_FALSE;
	
	
	return(function_Result);
}
/*=**************************************************************************=*/


/*=**************************************************************************=*/
Safe_Boolean Setup_Chromakey_Video_Control_Preparation(Setup_Chromakey_Video_Rec	*params_For_Setup_Chromakey_Video)
{
	Safe_Boolean			function_Result;
	INTEGER_32			selected_Value;
	Str255					message;
/* END VAR */
	
	
	function_Result = SAFE_TRUE;
	
	params_For_Setup_Chromakey_Video->async_Task_Desired = SAFE_FALSE;
	
	selected_Value = k_PREFERRED_CHROMA_RESERVED_CHANNEL;
	
	Get_NS_String(k_DREAM_STRINGS, k_CHOOSE_A_CHROMA_KEY_256_COLOR, message);
	
	if (Get_Long_Dialog(&message[0]
								, &selected_Value
								, 0
								, 255) == TRUE) {
		params_For_Setup_Chromakey_Video->chroma_1 = selected_Value;
		
/* ==
		params_For_Setup_Chromakey_Video->window_Index = TEST_DREAMING_WINDOW_INDEX;
=== */
		params_For_Setup_Chromakey_Video->window_Index = MAIN_SPINNING_WINDOW_INDEX;
		
		params_For_Setup_Chromakey_Video->chroma_2 = 0;
		params_For_Setup_Chromakey_Video->chroma_3 = 0;

		g_Chroma_Index_Optimized_Long = selected_Value;
		g_Chroma_Index_Optimized_Long = g_Chroma_Index_Optimized_Long << 8;
		g_Chroma_Index_Optimized_Long |= selected_Value;
		g_Chroma_Index_Optimized_Long = g_Chroma_Index_Optimized_Long << 8;
		g_Chroma_Index_Optimized_Long |= selected_Value;
		g_Chroma_Index_Optimized_Long = g_Chroma_Index_Optimized_Long << 8;
		g_Chroma_Index_Optimized_Long |= selected_Value;

	}
	else {
		function_Result = SAFE_FALSE;
	}
	
	return(function_Result);
}
/*=**************************************************************************=*/


/*=**************************************************************************=*/
Safe_Boolean Setup_Chromakey_Video_Menu_Preparation(Setup_Chromakey_Video_Rec	*params_For_Setup_Chromakey_Video)
{
	Safe_Boolean			function_Result;
	INTEGER_32			selected_Value;
	Str255					message;
/* END VAR */
	
	
	function_Result = SAFE_TRUE;
	
	params_For_Setup_Chromakey_Video->async_Task_Desired = SAFE_FALSE;
	
	selected_Value = k_PREFERRED_CHROMA_RESERVED_CHANNEL;
	
	Get_NS_String(k_DREAM_STRINGS, k_CHOOSE_A_CHROMA_KEY_256_COLOR, message);
	
	if (Get_Long_Dialog(&message[0]
								, &selected_Value
								, 0
								, 255) == TRUE) {
		params_For_Setup_Chromakey_Video->chroma_1 = selected_Value;

/* ==
		params_For_Setup_Chromakey_Video->window_Index = TEST_DREAMING_WINDOW_INDEX;
=== */
		params_For_Setup_Chromakey_Video->window_Index = MAIN_SPINNING_WINDOW_INDEX;

		params_For_Setup_Chromakey_Video->chroma_2 = 0;
		params_For_Setup_Chromakey_Video->chroma_3 = 0;
	}
	else {
		function_Result = SAFE_FALSE;
	}
	
	return(function_Result);
}
/*=**************************************************************************=*/


/*=**************************************************************************=*/
Safe_Boolean Repostition_For_Broadcast_Control_Preparation(Repostition_For_Broadcast_Rec	*params_For_Repostition_For_Broadcast)
{
	Safe_Boolean			function_Result;
/* END VAR */
	
	
	function_Result = SAFE_TRUE;
	
	params_For_Repostition_For_Broadcast->async_Task_Desired = SAFE_FALSE;
	
	
	return(function_Result);
}
/*=**************************************************************************=*/


/*=**************************************************************************=*/
Safe_Boolean Repostition_For_Broadcast_Menu_Preparation(Repostition_For_Broadcast_Rec	*params_For_Repostition_For_Broadcast)
{
	Safe_Boolean			function_Result;
/* END VAR */
	
	
	function_Result = SAFE_TRUE;
	
	params_For_Repostition_For_Broadcast->async_Task_Desired = SAFE_FALSE;
	
	
	return(function_Result);
}
/*=**************************************************************************=*/


/*=**************************************************************************=*/
Safe_Boolean Setup_Broadcast_Control_Preparation(Setup_Broadcast_Rec	*params_For_Setup_Broadcast)
{
	Safe_Boolean			function_Result;
/* END VAR */
	
	
	function_Result = SAFE_TRUE;
	
	params_For_Setup_Broadcast->async_Task_Desired = SAFE_FALSE;
	
	
	return(function_Result);
}
/*=**************************************************************************=*/


/*=**************************************************************************=*/
Safe_Boolean Setup_Broadcast_Menu_Preparation(Setup_Broadcast_Rec	*params_For_Setup_Broadcast)
{
	Safe_Boolean			function_Result;
/* END VAR */
	
	
	function_Result = SAFE_TRUE;
	
	params_For_Setup_Broadcast->async_Task_Desired = SAFE_FALSE;
	
	
	return(function_Result);
}
/*=**************************************************************************=*/


/*=**************************************************************************=*/
Safe_Boolean Associate_Voices_Control_Preparation(Associate_Voices_Rec	*params_For_Associate_Voices)
{
	Safe_Boolean			function_Result;
/* END VAR */
	
	
	function_Result = SAFE_TRUE;
	
	params_For_Associate_Voices->async_Task_Desired = SAFE_FALSE;
	
	
	return(function_Result);
}
/*=**************************************************************************=*/


/*=**************************************************************************=*/
Safe_Boolean Associate_Voices_Menu_Preparation(Associate_Voices_Rec	*params_For_Associate_Voices)
{
	Safe_Boolean			function_Result;
/* END VAR */
	
	
	function_Result = SAFE_TRUE;
	
	params_For_Associate_Voices->async_Task_Desired = SAFE_FALSE;
	
	
	return(function_Result);
}
/*=**************************************************************************=*/


/*=**************************************************************************=*/
Safe_Boolean Monitor_Remote_Control_Preparation(Monitor_Remote_Rec	*params_For_Monitor_Remote)
{
	Safe_Boolean			function_Result;
/* END VAR */
	
	
	function_Result = SAFE_TRUE;
	
	params_For_Monitor_Remote->async_Task_Desired = SAFE_FALSE;
	
	
	return(function_Result);
}
/*=**************************************************************************=*/


/*=**************************************************************************=*/
Safe_Boolean Monitor_Remote_Menu_Preparation(Monitor_Remote_Rec	*params_For_Monitor_Remote)
{
	Safe_Boolean			function_Result;
/* END VAR */
	
	
	function_Result = SAFE_TRUE;
	
	params_For_Monitor_Remote->async_Task_Desired = SAFE_FALSE;
	
	
	return(function_Result);
}
/*=**************************************************************************=*/


/*=**************************************************************************=*/
Safe_Boolean Monitor_Microphone_Control_Preparation(Monitor_Microphone_Rec	*params_For_Monitor_Microphone)
{
	Safe_Boolean			function_Result;
/* END VAR */
	
	
	function_Result = SAFE_TRUE;
	
	params_For_Monitor_Microphone->async_Task_Desired = SAFE_FALSE;
	params_For_Monitor_Microphone->select_Any_Input = SAFE_FALSE;
	
	return(function_Result);
}
/*=**************************************************************************=*/


/*=**************************************************************************=*/
Safe_Boolean Monitor_Microphone_Menu_Preparation(Monitor_Microphone_Rec	*params_For_Monitor_Microphone)
{
	Safe_Boolean			function_Result;
/* END VAR */
	
	
	function_Result = SAFE_TRUE;
	
	params_For_Monitor_Microphone->async_Task_Desired = SAFE_FALSE;
	params_For_Monitor_Microphone->select_Any_Input = SAFE_FALSE;
	
	
	return(function_Result);
}
/*=**************************************************************************=*/


/*=**************************************************************************=*/
Safe_Boolean Monitor_Using_Keyboard_Control_Preparation(Monitor_Using_Keyboard_Rec	*params_For_Monitor_Using_Keyboard)
{
	Safe_Boolean			function_Result;
/* END VAR */
	
	
	function_Result = SAFE_TRUE;
	
	params_For_Monitor_Using_Keyboard->async_Task_Desired = SAFE_FALSE;
	
	
	return(function_Result);
}
/*=**************************************************************************=*/


/*=**************************************************************************=*/
Safe_Boolean Monitor_Using_Keyboard_Menu_Preparation(Monitor_Using_Keyboard_Rec	*params_For_Monitor_Using_Keyboard)
{
	Safe_Boolean			function_Result;
/* END VAR */
	
	
	function_Result = SAFE_TRUE;
	
	params_For_Monitor_Using_Keyboard->async_Task_Desired = SAFE_FALSE;
	
	
	return(function_Result);
}
/*=**************************************************************************=*/


/*=**************************************************************************=*/
Safe_Boolean Monitor_Sampler_Control_Preparation(Monitor_Sampler_Rec	*params_For_Monitor_Sampler)
{
	Safe_Boolean			function_Result;
/* END VAR */
	
	
	function_Result = SAFE_TRUE;
	
	params_For_Monitor_Sampler->async_Task_Desired = SAFE_FALSE;
	
	
	return(function_Result);
}
/*=**************************************************************************=*/


/*=**************************************************************************=*/
Safe_Boolean Monitor_Sampler_Menu_Preparation(Monitor_Sampler_Rec	*params_For_Monitor_Sampler)
{
	Safe_Boolean			function_Result;
/* END VAR */
	
	
	function_Result = SAFE_TRUE;
	
	params_For_Monitor_Sampler->async_Task_Desired = SAFE_FALSE;
	
	
	return(function_Result);
}
/*=**************************************************************************=*/


/*=**************************************************************************=*/
Safe_Boolean Monitor_Phonebridge_Control_Preparation(Monitor_Phonebridge_Rec	*params_For_Monitor_Phonebridge)
{
	Safe_Boolean			function_Result;
/* END VAR */
	
	
	function_Result = SAFE_TRUE;
	
	params_For_Monitor_Phonebridge->async_Task_Desired = SAFE_FALSE;
	
	
	return(function_Result);
}
/*=**************************************************************************=*/


/*=**************************************************************************=*/
Safe_Boolean Monitor_Phonebridge_Menu_Preparation(Monitor_Phonebridge_Rec	*params_For_Monitor_Phonebridge)
{
	Safe_Boolean			function_Result;
/* END VAR */
	
	
	function_Result = SAFE_TRUE;
	
	params_For_Monitor_Phonebridge->async_Task_Desired = SAFE_FALSE;
	
	
	return(function_Result);
}
/*=**************************************************************************=*/


/*=**************************************************************************=*/
Safe_Boolean Reset_Connection_Control_Preparation(Reset_Connection_Rec	*params_For_Reset_Connection)
{
	Safe_Boolean			function_Result;
/* END VAR */
	
	
	function_Result = SAFE_TRUE;
	
	params_For_Reset_Connection->async_Task_Desired = SAFE_FALSE;
	
	
	return(function_Result);
}
/*=**************************************************************************=*/


/*=**************************************************************************=*/
Safe_Boolean Reset_Connection_Menu_Preparation(Reset_Connection_Rec	*params_For_Reset_Connection)
{
	Safe_Boolean			function_Result;
/* END VAR */
	
	
	function_Result = SAFE_TRUE;
	
	params_For_Reset_Connection->async_Task_Desired = SAFE_FALSE;
	
	
	return(function_Result);
}
/*=**************************************************************************=*/


/*=**************************************************************************=*/
Safe_Boolean Disconnect_Self_Control_Preparation(Disconnect_Self_Rec	*params_For_Disconnect_Self)
{
	Safe_Boolean			function_Result;
/* END VAR */
	
	
	function_Result = SAFE_TRUE;
	
	params_For_Disconnect_Self->async_Task_Desired = SAFE_FALSE;
	
	
	return(function_Result);
}
/*=**************************************************************************=*/


/*=**************************************************************************=*/
Safe_Boolean Disconnect_Self_Menu_Preparation(Disconnect_Self_Rec	*params_For_Disconnect_Self)
{
	Safe_Boolean			function_Result;
/* END VAR */
	
	
	function_Result = SAFE_TRUE;
	
	params_For_Disconnect_Self->async_Task_Desired = SAFE_FALSE;
	
	
	return(function_Result);
}
/*=**************************************************************************=*/


/*=**************************************************************************=*/
Safe_Boolean Show_Zones_Control_Preparation(Show_Zones_Rec	*params_For_Show_Zones)
{
	Safe_Boolean			function_Result;
/* END VAR */
	
	
	function_Result = SAFE_TRUE;
	
	params_For_Show_Zones->async_Task_Desired = SAFE_FALSE;
	
	
	return(function_Result);
}
/*=**************************************************************************=*/


/*=**************************************************************************=*/
Safe_Boolean Show_Zones_Menu_Preparation(Show_Zones_Rec	*params_For_Show_Zones)
{
	Safe_Boolean			function_Result;
/* END VAR */
	
	
	function_Result = SAFE_TRUE;
	
	params_For_Show_Zones->async_Task_Desired = SAFE_FALSE;
	
	
	return(function_Result);
}
/*=**************************************************************************=*/


/*=**************************************************************************=*/
Safe_Boolean Create_Scene_Control_Preparation(Create_Scene_Rec	*params_For_Create_Scene)
{
	Safe_Boolean			function_Result;
/* END VAR */
	
	
	function_Result = SAFE_TRUE;
	
	params_For_Create_Scene->async_Task_Desired = SAFE_FALSE;
	
	if (Get_New_Scene_Information(params_For_Create_Scene->scene_File_Name
								, params_For_Create_Scene->scene_Short_Name
								, params_For_Create_Scene->scene_Description) != TRUE) {
		function_Result = SAFE_FALSE;
	}
	else {
		if (Select_Scene_Size(&(params_For_Create_Scene->desired_Scene_Size)) != SAFE_TRUE) {
			function_Result = SAFE_FALSE;
		}
	}

	return(function_Result);
}
/*=**************************************************************************=*/


/*=**************************************************************************=*/
Safe_Boolean Create_Scene_Menu_Preparation(Create_Scene_Rec	*params_For_Create_Scene)
{
	Safe_Boolean			function_Result;
/* END VAR */
	
	
	function_Result = SAFE_TRUE;
	
	params_For_Create_Scene->async_Task_Desired = SAFE_FALSE;
	
	if (Get_New_Scene_Information(params_For_Create_Scene->scene_File_Name
								, params_For_Create_Scene->scene_Short_Name
								, params_For_Create_Scene->scene_Description) != TRUE) {
		function_Result = SAFE_FALSE;
	}
	else {
		if (Select_Scene_Size(&(params_For_Create_Scene->desired_Scene_Size)) != SAFE_TRUE) {
			function_Result = SAFE_FALSE;
		}
	}

	return(function_Result);
}
/*=**************************************************************************=*/


/*=**************************************************************************=*/
Safe_Boolean Open_Scene_Control_Preparation(Open_Scene_Rec	*params_For_Open_Scene)
{
	Safe_Boolean			function_Result;
	Safe_Boolean 		user_Abort;
/* END VAR */
	
	
	function_Result = SAFE_TRUE;
	
	params_For_Open_Scene->async_Task_Desired = SAFE_FALSE;
	
	if (Specify_Scene_To_Load(&(params_For_Open_Scene->desired_File)
											, &user_Abort) != SAFE_TRUE) {
		function_Result = SAFE_FALSE;
	}
	else {
		if (user_Abort == SAFE_TRUE) {
			function_Result = SAFE_FALSE;
		}
	}
	
	return(function_Result);
}
/*=**************************************************************************=*/


/*=**************************************************************************=*/
Safe_Boolean Open_Scene_Menu_Preparation(Open_Scene_Rec	*params_For_Open_Scene)
{
	Safe_Boolean			function_Result;
	Safe_Boolean 		user_Abort;
/* END VAR */
	
	
	function_Result = SAFE_TRUE;
	
	params_For_Open_Scene->async_Task_Desired = SAFE_FALSE;
	
	if (Specify_Scene_To_Load(&(params_For_Open_Scene->desired_File)
											, &user_Abort) != SAFE_TRUE) {
		function_Result = SAFE_FALSE;
	}
	else {
		if (user_Abort == SAFE_TRUE) {
			function_Result = SAFE_FALSE;
		}
	}
		
	return(function_Result);
}
/*=**************************************************************************=*/


/*=**************************************************************************=*/
Safe_Boolean Close_Scenes_Control_Preparation(Close_Scenes_Rec	*params_For_Close_Scenes)
{
	Safe_Boolean			function_Result;
	CARDINAL_32		resulting_Index;
	Safe_Boolean 		valid_Result;
/* END VAR */
	
	
	function_Result = SAFE_TRUE;
	
	params_For_Close_Scenes->async_Task_Desired = SAFE_FALSE;
	
	if (Get_Dream_Popup_Menu_Choice(k_SCENE_SELECTION
											, &resulting_Index
											, &valid_Result) != SAFE_TRUE) {
		function_Result = SAFE_FALSE;
	}
	else {
		if (valid_Result != SAFE_TRUE) {
			function_Result = SAFE_FALSE;
		}
		else {
			params_For_Close_Scenes->desired_Scene = resulting_Index;
		}
	}	
	
	return(function_Result);
}
/*=**************************************************************************=*/


/*=**************************************************************************=*/
Safe_Boolean Close_Scenes_Menu_Preparation(Close_Scenes_Rec	*params_For_Close_Scenes)
{
	Safe_Boolean			function_Result;
	CARDINAL_32		resulting_Index;
	Safe_Boolean 		valid_Result;
/* END VAR */
	
	
	function_Result = SAFE_TRUE;
	
	params_For_Close_Scenes->async_Task_Desired = SAFE_FALSE;
	
	if (Get_Dream_Popup_Menu_Choice(k_SCENE_SELECTION
											, &resulting_Index
											, &valid_Result) != SAFE_TRUE) {
		function_Result = SAFE_FALSE;
	}
	else {
		if (valid_Result != SAFE_TRUE) {
			function_Result = SAFE_FALSE;
		}
		else {
			params_For_Close_Scenes->desired_Scene = resulting_Index;
		}
	}	
	
	return(function_Result);
}
/*=**************************************************************************=*/


/*=**************************************************************************=*/
Safe_Boolean Close_All_Scenes_Control_Preparation(Close_All_Scenes_Rec	*params_For_Close_All_Scenes)
{
	Safe_Boolean			function_Result;
/* END VAR */
	
	
	function_Result = SAFE_TRUE;
	
	params_For_Close_All_Scenes->async_Task_Desired = SAFE_FALSE;

	return(function_Result);
}
/*=**************************************************************************=*/


/*=**************************************************************************=*/
Safe_Boolean Close_All_Scenes_Menu_Preparation(Close_All_Scenes_Rec	*params_For_Close_All_Scenes)
{
	Safe_Boolean			function_Result;
/* END VAR */
	
	
	function_Result = SAFE_TRUE;
	
	params_For_Close_All_Scenes->async_Task_Desired = SAFE_FALSE;
	
	return(function_Result);
}
/*=**************************************************************************=*/


/*=**************************************************************************=*/
Safe_Boolean Save_Scene_Control_Preparation(Save_Scene_Rec	*params_For_Save_Scene)
{
	Safe_Boolean			function_Result;
	Safe_Boolean 		user_Abort;
	CARDINAL_32		resulting_Index;
	Safe_Boolean 		valid_Result;
	Str255					message;
/* END VAR */
	
	
	function_Result = SAFE_FALSE;
	
	params_For_Save_Scene->async_Task_Desired = SAFE_FALSE;
	
	if (Get_Dream_Popup_Menu_Choice(k_SCENE_SELECTION
											, &resulting_Index
											, &valid_Result) != SAFE_TRUE) {
		goto Error_Handler;
	}
	else {
		if (valid_Result != SAFE_TRUE) {
			goto Error_Handler;
		}
		else {
			params_For_Save_Scene->desired_Scene = resulting_Index;
		}
	}	
	
	if (dream.play.scene_Memory_In_Use[resulting_Index] != MEMORY_INITIALIZED) {
		goto Error_Handler;
	}
	
	if (Compare_Pascal_Strings_Ignoring_Case(&((**(dream.play.scene[resulting_Index])).file_Name[0]), &(k_UNTITLED_NAME_STRING[0])) == SAFE_TRUE) {
		Get_NS_String(k_DREAM_STRINGS, k_ENTER_SCENE_NAMES_PROMPT, message);
	
		if (Edit_Fancy_Filename_Information(message
									, (**(dream.play.scene[resulting_Index])).file_Name
									, (**(dream.play.scene[resulting_Index])).screen_Short_Name
									, (**(dream.play.scene[resulting_Index])).screen_Name) != TRUE) {
			goto Error_Handler;
		}
	}
	
	if (Specify_Scene_To_Save(&(params_For_Save_Scene->desired_File)
											, &((**(dream.play.scene[resulting_Index])).file_Name)
											, &user_Abort) != SAFE_TRUE) {
		goto Error_Handler;
	}
	else {
		if (user_Abort == SAFE_TRUE) {
			goto Error_Handler;
		}
	}
	
	Copy_Pascal_String(params_For_Save_Scene->desired_File.name
			, (**(dream.play.scene[resulting_Index])).file_Name);


	function_Result = SAFE_TRUE;
	goto Normal_Function_Exit;
	
Error_Handler:
	function_Result = SAFE_FALSE;
			
Normal_Function_Exit:
	return(function_Result);
}
/*=**************************************************************************=*/


/*=**************************************************************************=*/
Safe_Boolean Save_Scene_Menu_Preparation(Save_Scene_Rec	*params_For_Save_Scene)
{
	Safe_Boolean			function_Result;
	Safe_Boolean 		user_Abort;
	CARDINAL_32		resulting_Index;
	Safe_Boolean 		valid_Result;
	Str255					message;
/* END VAR */
	
	
	function_Result = SAFE_FALSE;
	
	params_For_Save_Scene->async_Task_Desired = SAFE_FALSE;
	
	if (Get_Dream_Popup_Menu_Choice(k_SCENE_SELECTION
											, &resulting_Index
											, &valid_Result) != SAFE_TRUE) {
		goto Error_Handler;
	}
	else {
		if (valid_Result != SAFE_TRUE) {
			goto Error_Handler;
		}
		else {
			params_For_Save_Scene->desired_Scene = resulting_Index;
		}
	}	
	
	if (dream.play.scene_Memory_In_Use[resulting_Index] != MEMORY_INITIALIZED) {
		goto Error_Handler;
	}

	if (Compare_Pascal_Strings_Ignoring_Case(&((**(dream.play.scene[resulting_Index])).file_Name[0]), &(k_UNTITLED_NAME_STRING[0])) == SAFE_TRUE) {
		Get_NS_String(k_DREAM_STRINGS, k_ENTER_SCENE_NAMES_PROMPT, message);
	
		if (Edit_Fancy_Filename_Information(message
									, (**(dream.play.scene[resulting_Index])).file_Name
									, (**(dream.play.scene[resulting_Index])).screen_Short_Name
									, (**(dream.play.scene[resulting_Index])).screen_Name) != TRUE) {
			goto Error_Handler;
		}
	}

	if (Specify_Scene_To_Save(&(params_For_Save_Scene->desired_File)
											, &((**(dream.play.scene[resulting_Index])).file_Name)
											, &user_Abort) != SAFE_TRUE) {
		goto Error_Handler;
	}
	else {
		if (user_Abort == SAFE_TRUE) {
			goto Error_Handler;
		}
	}
	
	Copy_Pascal_String(params_For_Save_Scene->desired_File.name
			, (**(dream.play.scene[resulting_Index])).file_Name);
			
	function_Result = SAFE_TRUE;
	goto Normal_Function_Exit;
	
Error_Handler:
	function_Result = SAFE_FALSE;
			
Normal_Function_Exit:
	return(function_Result);
	
}
/*=**************************************************************************=*/


/*=**************************************************************************=*/
Safe_Boolean Save_Scene_As_Control_Preparation(Save_Scene_As_Rec	*params_For_Save_Scene_As)
{
	Safe_Boolean			function_Result;
	Safe_Boolean 		user_Abort;
	CARDINAL_32		resulting_Index;
	Safe_Boolean 		valid_Result;
	Str255					message;
/* END VAR */
	
	
	function_Result = SAFE_FALSE;
	
	params_For_Save_Scene_As->async_Task_Desired = SAFE_FALSE;
	
	if (Get_Dream_Popup_Menu_Choice(k_SCENE_SELECTION
											, &resulting_Index
											, &valid_Result) != SAFE_TRUE) {
		goto Error_Handler;
	}
	else {
		if (valid_Result != SAFE_TRUE) {
			goto Error_Handler;
		}
		else {
			params_For_Save_Scene_As->desired_Scene = resulting_Index;
		}
	}	
	
	if (dream.play.scene_Memory_In_Use[resulting_Index] != MEMORY_INITIALIZED) {
		goto Error_Handler;
	}


	Get_NS_String(k_DREAM_STRINGS, k_ENTER_SCENE_NAMES_PROMPT, message);

	if (Edit_Fancy_Filename_Information(message
								, (**(dream.play.scene[resulting_Index])).file_Name
								, (**(dream.play.scene[resulting_Index])).screen_Short_Name
								, (**(dream.play.scene[resulting_Index])).screen_Name) != TRUE) {
		goto Error_Handler;
	}

	if (Specify_Scene_To_Save(&(params_For_Save_Scene_As->desired_File)
											, &((**(dream.play.scene[resulting_Index])).file_Name)
											, &user_Abort) != SAFE_TRUE) {
		goto Error_Handler;
	}
	else {
		if (user_Abort == SAFE_TRUE) {
			goto Error_Handler;
		}
	}
	
	Copy_Pascal_String(params_For_Save_Scene_As->desired_File.name
			, (**(dream.play.scene[resulting_Index])).file_Name);


	function_Result = SAFE_TRUE;
	goto Normal_Function_Exit;
	
Error_Handler:
	function_Result = SAFE_FALSE;
			
Normal_Function_Exit:
	return(function_Result);
}
/*=**************************************************************************=*/


/*=**************************************************************************=*/
Safe_Boolean Save_Scene_As_Menu_Preparation(Save_Scene_As_Rec	*params_For_Save_Scene_As)
{
	Safe_Boolean			function_Result;
	Safe_Boolean 		user_Abort;
	CARDINAL_32		resulting_Index;
	Safe_Boolean 		valid_Result;
	Str255					message;
/* END VAR */
	
	
	function_Result = SAFE_FALSE;
	
	params_For_Save_Scene_As->async_Task_Desired = SAFE_FALSE;
	
	if (Get_Dream_Popup_Menu_Choice(k_SCENE_SELECTION
											, &resulting_Index
											, &valid_Result) != SAFE_TRUE) {
		goto Error_Handler;
	}
	else {
		if (valid_Result != SAFE_TRUE) {
			goto Error_Handler;
		}
		else {
			params_For_Save_Scene_As->desired_Scene = resulting_Index;
		}
	}	
	
	if (dream.play.scene_Memory_In_Use[resulting_Index] != MEMORY_INITIALIZED) {
		goto Error_Handler;
	}

	if (Specify_Scene_To_Save(&(params_For_Save_Scene_As->desired_File)
											, &((**(dream.play.scene[resulting_Index])).file_Name)
											, &user_Abort) != SAFE_TRUE) {
		goto Error_Handler;
	}
	else {
		if (user_Abort == SAFE_TRUE) {
			goto Error_Handler;
		}
	}
	
	Copy_Pascal_String(params_For_Save_Scene_As->desired_File.name
			, (**(dream.play.scene[resulting_Index])).file_Name);

	Get_NS_String(k_DREAM_STRINGS, k_ENTER_SCENE_NAMES_PROMPT, message);

	if (Edit_Fancy_Filename_Information(message
								, (**(dream.play.scene[resulting_Index])).file_Name
								, (**(dream.play.scene[resulting_Index])).screen_Short_Name
								, (**(dream.play.scene[resulting_Index])).screen_Name) != TRUE) {
		goto Error_Handler;
	}

	function_Result = SAFE_TRUE;
	goto Normal_Function_Exit;
	
Error_Handler:
	function_Result = SAFE_FALSE;
			
Normal_Function_Exit:
	return(function_Result);
}
/*=**************************************************************************=*/


/*=**************************************************************************=*/
Safe_Boolean Page_Setup_Control_Preparation(Page_Setup_Rec	*params_For_Page_Setup)
{
	Safe_Boolean			function_Result;
/* END VAR */
	
	
	function_Result = SAFE_TRUE;
	
	params_For_Page_Setup->async_Task_Desired = SAFE_FALSE;
	
	
	return(function_Result);
}
/*=**************************************************************************=*/


/*=**************************************************************************=*/
Safe_Boolean Page_Setup_Menu_Preparation(Page_Setup_Rec	*params_For_Page_Setup)
{
	Safe_Boolean			function_Result;
/* END VAR */
	
	
	function_Result = SAFE_TRUE;
	
	params_For_Page_Setup->async_Task_Desired = SAFE_FALSE;
	
	
	return(function_Result);
}
/*=**************************************************************************=*/


/*=**************************************************************************=*/
Safe_Boolean Page_Print_Control_Preparation(Page_Print_Rec	*params_For_Page_Print)
{
	Safe_Boolean			function_Result;
/* END VAR */
	
	
	function_Result = SAFE_TRUE;
	
	params_For_Page_Print->async_Task_Desired = SAFE_FALSE;
	
	
	return(function_Result);
}
/*=**************************************************************************=*/


/*=**************************************************************************=*/
Safe_Boolean Page_Print_Menu_Preparation(Page_Print_Rec	*params_For_Page_Print)
{
	Safe_Boolean			function_Result;
/* END VAR */
	
	
	function_Result = SAFE_TRUE;
	
	params_For_Page_Print->async_Task_Desired = SAFE_FALSE;
	
	
	return(function_Result);
}
/*=**************************************************************************=*/


/*=**************************************************************************=*/
Safe_Boolean Print_Saved_Files_Control_Preparation(Print_Saved_Files_Rec	*params_For_Print_Saved_Files)
{
	Safe_Boolean			function_Result;
/* END VAR */
	
	
	function_Result = SAFE_TRUE;
	
	params_For_Print_Saved_Files->async_Task_Desired = SAFE_FALSE;
	
	
	return(function_Result);
}
/*=**************************************************************************=*/


/*=**************************************************************************=*/
Safe_Boolean Print_Saved_Files_Menu_Preparation(Print_Saved_Files_Rec	*params_For_Print_Saved_Files)
{
	Safe_Boolean			function_Result;
/* END VAR */
	
	
	function_Result = SAFE_TRUE;
	
	params_For_Print_Saved_Files->async_Task_Desired = SAFE_FALSE;
	
	
	return(function_Result);
}
/*=**************************************************************************=*/


/*=**************************************************************************=*/
Safe_Boolean Add_Item_Control_Preparation(Add_Item_Rec	*params_For_Add_Item)
{
	Safe_Boolean			function_Result;
	CARDINAL_32		resulting_Index;
	Safe_Boolean 		valid_Result;
/* END VAR */

	
	function_Result = SAFE_FALSE;
	
	params_For_Add_Item->async_Task_Desired = SAFE_FALSE;
	
	if (Get_Dream_Popup_Menu_Choice(k_PUPPET_FILE_SELECTION
											, &resulting_Index
											, &valid_Result) != SAFE_TRUE) {
		goto Error_Handler;
	}
	else {
		if (valid_Result != SAFE_TRUE) {
			goto Error_Handler;
		}
		else {
			params_For_Add_Item->desired_Item = resulting_Index;
			function_Result = SAFE_TRUE;
		}
	}	

	goto Normal_Function_Exit;
	
Error_Handler:
	function_Result = SAFE_FALSE;
			
Normal_Function_Exit:
	return(function_Result);
}
/*=**************************************************************************=*/


/*=**************************************************************************=*/
Safe_Boolean Add_Item_Menu_Preparation(Add_Item_Rec	*params_For_Add_Item)
{
	Safe_Boolean			function_Result;
	CARDINAL_32		resulting_Index;
	Safe_Boolean 		valid_Result;
/* END VAR */

	
	function_Result = SAFE_FALSE;
	
	params_For_Add_Item->async_Task_Desired = SAFE_FALSE;
	
	if (Get_Dream_Popup_Menu_Choice(k_PUPPET_FILE_SELECTION
											, &resulting_Index
											, &valid_Result) != SAFE_TRUE) {
		goto Error_Handler;
	}
	else {
		if (valid_Result != SAFE_TRUE) {
			goto Error_Handler;
		}
		else {
			params_For_Add_Item->desired_Item = resulting_Index;
			function_Result = SAFE_TRUE;
		}
	}	

	goto Normal_Function_Exit;
	
Error_Handler:
	function_Result = SAFE_FALSE;
			
Normal_Function_Exit:
	return(function_Result);
}
/*=**************************************************************************=*/


/*=**************************************************************************=*/
Safe_Boolean Remove_Item_Control_Preparation(Remove_Item_Rec	*params_For_Remove_Item)
{
	Safe_Boolean			function_Result;
	CARDINAL_32		resulting_Index;
	Safe_Boolean 		valid_Result;
/* END VAR */
	
	
	function_Result = SAFE_FALSE;
	
	if (g_Dreaming_Module_Globals_Initialized == SAFE_TRUE) {
		if (dream.play.total_Scenes > 0) {
			params_For_Remove_Item->async_Task_Desired = SAFE_FALSE;
			
			if (Get_Dream_Popup_Menu_Choice(k_ANIMATED_PUPPET_IN_SCENE_SELECTION
													, &resulting_Index
													, &valid_Result) != SAFE_TRUE) {
				function_Result = SAFE_FALSE;
			}
			else {
				if (valid_Result == SAFE_TRUE) {
					params_For_Remove_Item->desired_Scene= dream.current_Scene;
					params_For_Remove_Item->desired_Item = resulting_Index;
					function_Result = SAFE_TRUE;
				}
			}
		}
	}
	
	return(function_Result);
}
/*=**************************************************************************=*/


/*=**************************************************************************=*/
Safe_Boolean Remove_Item_Menu_Preparation(Remove_Item_Rec	*params_For_Remove_Item)
{
	Safe_Boolean			function_Result;
	CARDINAL_32		resulting_Index;
	Safe_Boolean 		valid_Result;
/* END VAR */
	
	
	function_Result = SAFE_FALSE;
	
	if (g_Dreaming_Module_Globals_Initialized == SAFE_TRUE) {
		if (dream.play.total_Scenes > 0) {
			params_For_Remove_Item->async_Task_Desired = SAFE_FALSE;
			
			if (Get_Dream_Popup_Menu_Choice(k_ANIMATED_PUPPET_IN_SCENE_SELECTION
													, &resulting_Index
													, &valid_Result) != SAFE_TRUE) {
				function_Result = SAFE_FALSE;
			}
			else {
				if (valid_Result == SAFE_TRUE) {
					params_For_Remove_Item->desired_Scene= dream.current_Scene;
					params_For_Remove_Item->desired_Item = resulting_Index;
					function_Result = SAFE_TRUE;
				}
			}
		}
	}
	
	return(function_Result);
}
/*=**************************************************************************=*/


/*=**************************************************************************=*/
Safe_Boolean Resposition_Item_Control_Preparation(Resposition_Item_Rec	*params_For_Resposition_Item)
{
	Safe_Boolean			function_Result;
/* END VAR */
	
	
	function_Result = SAFE_TRUE;
	
	params_For_Resposition_Item->async_Task_Desired = SAFE_FALSE;
	
	
	return(function_Result);
}
/*=**************************************************************************=*/


/*=**************************************************************************=*/
Safe_Boolean Resposition_Item_Menu_Preparation(Resposition_Item_Rec	*params_For_Resposition_Item)
{
	Safe_Boolean			function_Result;
/* END VAR */
	
	
	function_Result = SAFE_TRUE;
	
	params_For_Resposition_Item->async_Task_Desired = SAFE_FALSE;
	
	
	return(function_Result);
}
/*=**************************************************************************=*/


/*=**************************************************************************=*/
Safe_Boolean Change_Backdrop_Control_Preparation(Change_Backdrop_Rec	*params_For_Change_Backdrop)
{
	Safe_Boolean			function_Result;
	CARDINAL_32		resulting_Index;
	Safe_Boolean 		valid_Result;
/* END VAR */

	
	function_Result = SAFE_FALSE;
	
	params_For_Change_Backdrop->async_Task_Desired = SAFE_FALSE;
	
	if (Get_Dream_Popup_Menu_Choice(k_BACKDROP_FILE_SELECTION
											, &resulting_Index
											, &valid_Result) != SAFE_TRUE) {
		goto Error_Handler;
	}
	else {
		if (valid_Result != SAFE_TRUE) {
			goto Error_Handler;
		}
		else {
			params_For_Change_Backdrop->desired_Backdrop = resulting_Index;
			function_Result = SAFE_TRUE;
		}
	}	

	goto Normal_Function_Exit;
	
Error_Handler:
	function_Result = SAFE_FALSE;
			
Normal_Function_Exit:
	return(function_Result);
}
/*=**************************************************************************=*/


/*=**************************************************************************=*/
Safe_Boolean Change_Backdrop_Menu_Preparation(Change_Backdrop_Rec	*params_For_Change_Backdrop)
{
	Safe_Boolean			function_Result;
	CARDINAL_32		resulting_Index;
	Safe_Boolean 		valid_Result;
/* END VAR */

	
	function_Result = SAFE_FALSE;
	
	params_For_Change_Backdrop->async_Task_Desired = SAFE_FALSE;
	
	if (Get_Dream_Popup_Menu_Choice(k_BACKDROP_FILE_SELECTION
											, &resulting_Index
											, &valid_Result) != SAFE_TRUE) {
		goto Error_Handler;
	}
	else {
		if (valid_Result != SAFE_TRUE) {
			goto Error_Handler;
		}
		else {
			params_For_Change_Backdrop->desired_Backdrop = resulting_Index;
			function_Result = SAFE_TRUE;
		}
	}	

	goto Normal_Function_Exit;
	
Error_Handler:
	function_Result = SAFE_FALSE;
			
Normal_Function_Exit:
	return(function_Result);
}
/*=**************************************************************************=*/


/*=**************************************************************************=*/
Safe_Boolean Play_Audio_File_Control_Preparation(Play_Audio_File_Rec	*params_For_Play_Audio_File)
{
	Safe_Boolean			function_Result;
	CARDINAL_32		resulting_Index;
	Safe_Boolean 		valid_Result;
/* END VAR */

	
	function_Result = SAFE_FALSE;
	
	params_For_Play_Audio_File->async_Task_Desired = SAFE_FALSE;
	
	if (Get_Dream_Popup_Menu_Choice(k_AUDIO_OBJECT_FILE_SELECTION
											, &resulting_Index
											, &valid_Result) != SAFE_TRUE) {
		goto Error_Handler;
	}
	else {
		if (valid_Result != SAFE_TRUE) {
			goto Error_Handler;
		}
		else {
			params_For_Play_Audio_File->desired_Sound_Index = resulting_Index;
			function_Result = SAFE_TRUE;
		}
	}	

	goto Normal_Function_Exit;
	
Error_Handler:
	function_Result = SAFE_FALSE;
			
Normal_Function_Exit:
	return(function_Result);
}
/*=**************************************************************************=*/


/*=**************************************************************************=*/
Safe_Boolean Play_Audio_File_Menu_Preparation(Play_Audio_File_Rec	*params_For_Play_Audio_File)
{
	Safe_Boolean			function_Result;
	CARDINAL_32		resulting_Index;
	Safe_Boolean 		valid_Result;
/* END VAR */

	
	function_Result = SAFE_FALSE;
	
	params_For_Play_Audio_File->async_Task_Desired = SAFE_FALSE;
	
	if (Get_Dream_Popup_Menu_Choice(k_AUDIO_OBJECT_FILE_SELECTION
											, &resulting_Index
											, &valid_Result) != SAFE_TRUE) {
		goto Error_Handler;
	}
	else {
		if (valid_Result != SAFE_TRUE) {
			goto Error_Handler;
		}
		else {
			params_For_Play_Audio_File->desired_Sound_Index = resulting_Index;
			function_Result = SAFE_TRUE;
		}
	}	

	goto Normal_Function_Exit;
	
Error_Handler:
	function_Result = SAFE_FALSE;
			
Normal_Function_Exit:
	return(function_Result);
}
/*=**************************************************************************=*/


/*=**************************************************************************=*/
extern Safe_Boolean Swap_Voice_For_Audio_File_Control_Preparation(Swap_Voice_For_Audio_File_Rec	*params_For_Swap_Voice_For_Audio_File)
{
	Safe_Boolean			function_Result;
	CARDINAL_32		resulting_Index;
	Safe_Boolean 		valid_Result;
/* END VAR */

	
	function_Result = SAFE_FALSE;
	
	params_For_Swap_Voice_For_Audio_File->async_Task_Desired = SAFE_FALSE;
	
	if (Get_Dream_Popup_Menu_Choice(k_AUDIO_OBJECT_FILE_SELECTION
											, &resulting_Index
											, &valid_Result) != SAFE_TRUE) {
		goto Error_Handler;
	}
	else {
		if (valid_Result != SAFE_TRUE) {
			goto Error_Handler;
		}
		else {
			params_For_Swap_Voice_For_Audio_File->desired_Sound_Index = resulting_Index;
			function_Result = SAFE_TRUE;
		}
	}	

	/*  */
	/* Not implemented yet, just take first item */
	/* "k_ITEM_IN_SCENE_SELECTION" needs to be written still */
	/* and it needs to be fed a values created from a scene selector such as */
	/* from "k_SCENE_SELECTION". this command is less awkward when fed */
	/* from control state structure instead of menu dialogs. */
	/*  */
	params_For_Swap_Voice_For_Audio_File->select_Arbitrary_First_Item = SAFE_TRUE;
	params_For_Swap_Voice_For_Audio_File->desired_Scene = 999;
	params_For_Swap_Voice_For_Audio_File->desired_Item = 999;

	goto Normal_Function_Exit;
	
Error_Handler:
	function_Result = SAFE_FALSE;
			
Normal_Function_Exit:
	return(function_Result);
}
/*=**************************************************************************=*/


/*=**************************************************************************=*/
extern Safe_Boolean Swap_Voice_For_Audio_File_Menu_Preparation(Swap_Voice_For_Audio_File_Rec	*params_For_Swap_Voice_For_Audio_File)
{
	Safe_Boolean			function_Result;
	CARDINAL_32		resulting_Index;
	Safe_Boolean 		valid_Result;
/* END VAR */

	
	function_Result = SAFE_FALSE;
	
	params_For_Swap_Voice_For_Audio_File->async_Task_Desired = SAFE_FALSE;
	
	if (Get_Dream_Popup_Menu_Choice(k_AUDIO_OBJECT_FILE_SELECTION
											, &resulting_Index
											, &valid_Result) != SAFE_TRUE) {
		goto Error_Handler;
	}
	else {
		if (valid_Result != SAFE_TRUE) {
			goto Error_Handler;
		}
		else {
			params_For_Swap_Voice_For_Audio_File->desired_Sound_Index = resulting_Index;
			function_Result = SAFE_TRUE;
		}
	}
	
	/*  */
	/* Not implemented yet, just take first item */
	/* "k_ITEM_IN_SCENE_SELECTION" needs to be written still */
	/* and it needs to be fed a values created from a scene selector such as */
	/* from "k_SCENE_SELECTION". This command is less awkward when fed */
	/* from control state structure instead of menu dialogs. */
	/*  */
	params_For_Swap_Voice_For_Audio_File->select_Arbitrary_First_Item = SAFE_TRUE;
	params_For_Swap_Voice_For_Audio_File->desired_Scene = 999;
	params_For_Swap_Voice_For_Audio_File->desired_Item = 999;

	goto Normal_Function_Exit;
	
Error_Handler:
	function_Result = SAFE_FALSE;
			
Normal_Function_Exit:
	return(function_Result);
}
/*=**************************************************************************=*/


/*=**************************************************************************=*/
extern Safe_Boolean Swap_Assumed_Voice_For_Audio_File_Control_Preparation(Swap_Voice_For_Audio_File_Rec	*params_For_Swap_Voice_For_Audio_File)
{
	CARDINAL_32		index;
	CARDINAL_32		match_Count;
	CARDINAL_32		audio_Index;
	Safe_Boolean			function_Result;
	Boolean			found_Something;
/* END VAR */

	
	function_Result = SAFE_FALSE;
	
	params_For_Swap_Voice_For_Audio_File->async_Task_Desired = SAFE_FALSE;
	found_Something = FALSE;
	
	match_Count = 0;
	for (index = 0; ((index < k_MAXIMUM_DATABASE_SOUND_OBJECTS) 
									&& (match_Count < g_Audio_Files.total_Available)); index++) {
									
		if ((**g_Audio_Files.info.sounds.handle).sound_Objects[index].active == SAFE_TRUE) {
			match_Count += 1;
			if ((**g_Audio_Files.info.sounds.handle).sound_Objects[index].type_Of_Source == k_FROM_FILE) {
				/*  */
				/* This won't exit, but the last item in the list located will be the one we want. */
				/*  */
				found_Something = TRUE;
				audio_Index = index;
			}
		}
	}
	
	if (found_Something == TRUE) {
		params_For_Swap_Voice_For_Audio_File->desired_Sound_Index = audio_Index;
		params_For_Swap_Voice_For_Audio_File->select_Arbitrary_First_Item = SAFE_TRUE;
		params_For_Swap_Voice_For_Audio_File->desired_Scene = 999;
		params_For_Swap_Voice_For_Audio_File->desired_Item = 999;
	}
	else {
		goto Error_Handler;
	}

	function_Result = SAFE_TRUE;
	goto Normal_Function_Exit;
	
Error_Handler:
	function_Result = SAFE_FALSE;
			
Normal_Function_Exit:
	return(function_Result);
}
/*=**************************************************************************=*/


/*=**************************************************************************=*/
extern Safe_Boolean Swap_Assumed_Voice_For_Audio_File_Menu_Preparation(Swap_Voice_For_Audio_File_Rec	*params_For_Swap_Voice_For_Audio_File)
{
	CARDINAL_32		index;
	CARDINAL_32		match_Count;
	CARDINAL_32		audio_Index;
	Safe_Boolean			function_Result;
	Boolean			found_Something;
/* END VAR */

	
	function_Result = SAFE_FALSE;
	
	params_For_Swap_Voice_For_Audio_File->async_Task_Desired = SAFE_FALSE;
	
	match_Count = 0;
	found_Something = FALSE;

	for (index = 0; ((index < k_MAXIMUM_DATABASE_SOUND_OBJECTS) 
									&& (match_Count < g_Audio_Files.total_Available)); index++) {
									
		if ((**g_Audio_Files.info.sounds.handle).sound_Objects[index].active == SAFE_TRUE) {
			match_Count += 1;
			if ((**g_Audio_Files.info.sounds.handle).sound_Objects[index].type_Of_Source == k_FROM_FILE) {
				/*  */
				/* This won't exit, but the last item in the list located will be the one we want. */
				/*  */
				found_Something = TRUE;
				audio_Index = index;
			}
		}
	}
	
	if (found_Something == TRUE) {
		params_For_Swap_Voice_For_Audio_File->desired_Sound_Index = audio_Index;
		params_For_Swap_Voice_For_Audio_File->select_Arbitrary_First_Item = SAFE_TRUE;
		params_For_Swap_Voice_For_Audio_File->desired_Scene = 999;
		params_For_Swap_Voice_For_Audio_File->desired_Item = 999;
	}
	else {
		goto Error_Handler;
	}

	function_Result = SAFE_TRUE;
	goto Normal_Function_Exit;
	
Error_Handler:
	function_Result = SAFE_FALSE;
			
Normal_Function_Exit:
	return(function_Result);
}
/*=**************************************************************************=*/


/*=**************************************************************************=*/
Safe_Boolean Rescan_Files_Control_Preparation(Rescan_Files_Rec	*params_For_Rescan_Files)
{
	Safe_Boolean			function_Result;
/* END VAR */
	
	
	function_Result = SAFE_TRUE;
	
	params_For_Rescan_Files->async_Task_Desired = SAFE_FALSE;
	
	
	return(function_Result);
}
/*=**************************************************************************=*/


/*=**************************************************************************=*/
Safe_Boolean Rescan_Files_Menu_Preparation(Rescan_Files_Rec	*params_For_Rescan_Files)
{
	Safe_Boolean			function_Result;
/* END VAR */
	
	
	function_Result = SAFE_TRUE;
	
	params_For_Rescan_Files->async_Task_Desired = SAFE_FALSE;
	
	
	return(function_Result);
}
/*=**************************************************************************=*/


/*=**************************************************************************=*/
Safe_Boolean Connect_Remote_Control_Preparation(Connect_Remote_Rec	*params_For_Connect_Remote)
{
	Safe_Boolean			function_Result;
/* END VAR */
	
	
	function_Result = SAFE_TRUE;
	
	params_For_Connect_Remote->async_Task_Desired = SAFE_FALSE;

	return(function_Result);
}
/*=**************************************************************************=*/


/*=**************************************************************************=*/
Safe_Boolean Connect_Remote_Menu_Preparation(Connect_Remote_Rec	*params_For_Connect_Remote)
{
	Safe_Boolean			function_Result;
/* END VAR */
	
	
	function_Result = SAFE_TRUE;
	
	params_For_Connect_Remote->async_Task_Desired = SAFE_FALSE;
	
	return(function_Result);
}
/*=**************************************************************************=*/


/*=**************************************************************************=*/
Safe_Boolean Disconnect_Remote_Control_Preparation(Disconnect_Remote_Rec	*params_For_Disconnect_Remote)
{
	Safe_Boolean			function_Result;
/* END VAR */
	
	
	function_Result = SAFE_TRUE;
	
	params_For_Disconnect_Remote->async_Task_Desired = SAFE_FALSE;

	return(function_Result);
}
/*=**************************************************************************=*/


/*=**************************************************************************=*/
Safe_Boolean Disconnect_Remote_Menu_Preparation(Disconnect_Remote_Rec	*params_For_Disconnect_Remote)
{
	Safe_Boolean			function_Result;
/* END VAR */
	
	
	function_Result = SAFE_TRUE;
	
	params_For_Disconnect_Remote->async_Task_Desired = SAFE_FALSE;
	
	return(function_Result);
}
/*=**************************************************************************=*/


/*=**************************************************************************=*/
Safe_Boolean Select_Remote_Control_Preparation(Select_Remote_Rec	*params_For_Select_Remote)
{
	Safe_Boolean			function_Result;
	CARDINAL_32		resulting_Index;
	Safe_Boolean 		valid_Result;
/* END VAR */
	
	
	function_Result = SAFE_TRUE;
	
	params_For_Select_Remote->async_Task_Desired = SAFE_FALSE;

	if (Get_Dream_Popup_Menu_Choice(k_REMOTE_SLIDE_SELECTION
											, &resulting_Index
											, &valid_Result) != SAFE_TRUE) {
		function_Result = SAFE_FALSE;
	}
	else {
		if (valid_Result != SAFE_TRUE) {
			function_Result = SAFE_FALSE;
		}
		else {
			params_For_Select_Remote->desired_Slide = resulting_Index;
		}
	}	

	return(function_Result);
}
/*=**************************************************************************=*/


/*=**************************************************************************=*/
Safe_Boolean Select_Remote_Menu_Preparation(Select_Remote_Rec	*params_For_Select_Remote)
{
	Safe_Boolean			function_Result;
	CARDINAL_32		resulting_Index;
	Safe_Boolean 		valid_Result;
/* END VAR */
	
	
	function_Result = SAFE_TRUE;
	
	params_For_Select_Remote->async_Task_Desired = SAFE_FALSE;
	
	if (Get_Dream_Popup_Menu_Choice(k_REMOTE_SLIDE_SELECTION
											, &resulting_Index
											, &valid_Result) != SAFE_TRUE) {
		function_Result = SAFE_FALSE;
	}
	else {
		if (valid_Result != SAFE_TRUE) {
			function_Result = SAFE_FALSE;
		}
		else {
			params_For_Select_Remote->desired_Slide = resulting_Index;
		}
	}	

	return(function_Result);
}
/*=**************************************************************************=*/


/*=**************************************************************************=*/
Safe_Boolean Select_Next_Remote_Control_Preparation(Select_Next_Remote_Rec	*params_For_Select_Next_Remote)
{
	Safe_Boolean			function_Result;
/* END VAR */
	
	
	function_Result = SAFE_TRUE;
	
	params_For_Select_Next_Remote->async_Task_Desired = SAFE_FALSE;

	return(function_Result);
}
/*=**************************************************************************=*/


/*=**************************************************************************=*/
Safe_Boolean Select_Next_Remote_Menu_Preparation(Select_Next_Remote_Rec	*params_For_Select_Next_Remote)
{
	Safe_Boolean			function_Result;
/* END VAR */
	
	
	function_Result = SAFE_TRUE;
	
	params_For_Select_Next_Remote->async_Task_Desired = SAFE_FALSE;
	
	return(function_Result);
}
/*=**************************************************************************=*/


/*=**************************************************************************=*/
Safe_Boolean Select_Previous_Remote_Control_Preparation(Select_Previous_Remote_Rec	*params_For_Select_Previous_Remote)
{
	Safe_Boolean			function_Result;
/* END VAR */
	
	
	function_Result = SAFE_TRUE;
	
	params_For_Select_Previous_Remote->async_Task_Desired = SAFE_FALSE;

	return(function_Result);
}
/*=**************************************************************************=*/


/*=**************************************************************************=*/
Safe_Boolean Select_Previous_Remote_Menu_Preparation(Select_Previous_Remote_Rec	*params_For_Select_Previous_Remote)
{
	Safe_Boolean			function_Result;
/* END VAR */
	
	
	function_Result = SAFE_TRUE;
	
	params_For_Select_Previous_Remote->async_Task_Desired = SAFE_FALSE;
	
	return(function_Result);
}
/*=**************************************************************************=*/


/*=**************************************************************************=*/
Safe_Boolean Rescan_Remote_Control_Preparation(Rescan_Remote_Rec	*params_For_Rescan_Remote)
{
	Safe_Boolean			function_Result;
/* END VAR */
	
	
	function_Result = SAFE_TRUE;
	
	params_For_Rescan_Remote->async_Task_Desired = SAFE_FALSE;

	return(function_Result);
}
/*=**************************************************************************=*/


/*=**************************************************************************=*/
Safe_Boolean Rescan_Remote_Menu_Preparation(Rescan_Remote_Rec	*params_For_Rescan_Remote)
{
	Safe_Boolean			function_Result;
/* END VAR */
	
	
	function_Result = SAFE_TRUE;
	
	params_For_Rescan_Remote->async_Task_Desired = SAFE_FALSE;
	
	return(function_Result);
}
/*=**************************************************************************=*/


/*=**************************************************************************=*/
Safe_Boolean Update_Remote_List_Control_Preparation(Update_Remote_List_Rec	*params_For_Update_Remote_List)
{
	Safe_Boolean			function_Result;
/* END VAR */
	
	
	function_Result = SAFE_TRUE;
	
	params_For_Update_Remote_List->async_Task_Desired = SAFE_FALSE;

	return(function_Result);
}
/*=**************************************************************************=*/


/*=**************************************************************************=*/
Safe_Boolean Update_Remote_List_Menu_Preparation(Update_Remote_List_Rec	*params_For_Update_Remote_List)
{
	Safe_Boolean			function_Result;
/* END VAR */
	
	
	function_Result = SAFE_TRUE;
	
	params_For_Update_Remote_List->async_Task_Desired = SAFE_FALSE;
	
	return(function_Result);
}
/*=**************************************************************************=*/


/*=**************************************************************************=*/
Safe_Boolean Kill_Remote_Control_Preparation(Kill_Remote_Rec	*params_For_Kill_Remote)
{
	Safe_Boolean			function_Result;
/* END VAR */
	
	
	function_Result = SAFE_TRUE;
	
	params_For_Kill_Remote->async_Task_Desired = SAFE_FALSE;

	return(function_Result);
}
/*=**************************************************************************=*/


/*=**************************************************************************=*/
Safe_Boolean Kill_Remote_Menu_Preparation(Kill_Remote_Rec	*params_For_Kill_Remote)
{
	Safe_Boolean			function_Result;
/* END VAR */
	
	
	function_Result = SAFE_TRUE;
	
	params_For_Kill_Remote->async_Task_Desired = SAFE_FALSE;
	
	return(function_Result);
}
/*=**************************************************************************=*/


/*=**************************************************************************=*/
Safe_Boolean Select_Slide_Menu_Preparation(Select_Slide_Rec	*params_For_Select_Slide)
{
	Safe_Boolean			function_Result;
	CARDINAL_32		resulting_Index;
	Safe_Boolean 		valid_Result;
/* END VAR */
	
	
	function_Result = SAFE_TRUE;
	
	params_For_Select_Slide->async_Task_Desired = SAFE_FALSE;
	
	if (Get_Dream_Popup_Menu_Choice(k_SLIDE_SELECTION
											, &resulting_Index
											, &valid_Result) != SAFE_TRUE) {
		function_Result = SAFE_FALSE;
	}
	else {
		if (valid_Result != SAFE_TRUE) {
			function_Result = SAFE_FALSE;
		}
		else {
			params_For_Select_Slide->desired_Slide = resulting_Index;
		}
	}	
	
	return(function_Result);
}
/*=**************************************************************************=*/


/*=**************************************************************************=*/
Safe_Boolean Select_Slide_Control_Preparation(Select_Slide_Rec	*params_For_Select_Slide)
{
	Safe_Boolean			function_Result;
	CARDINAL_32		resulting_Index;
	Safe_Boolean 		valid_Result;
/* END VAR */
	
	
	function_Result = SAFE_TRUE;
	
	params_For_Select_Slide->async_Task_Desired = SAFE_FALSE;
	
	if (Get_Dream_Popup_Menu_Choice(k_SLIDE_SELECTION
											, &resulting_Index
											, &valid_Result) != SAFE_TRUE) {
		function_Result = SAFE_FALSE;
	}
	else {
		if (valid_Result != SAFE_TRUE) {
			function_Result = SAFE_FALSE;
		}
		else {
			params_For_Select_Slide->desired_Slide = resulting_Index;
		}
	}	
	
	return(function_Result);
}
/*=**************************************************************************=*/


/*=**************************************************************************=*/
Safe_Boolean Change_Window_Size_Menu_Preparation(Change_Window_Size_Rec	*params_For_Change_Window_Size)
{
	Safe_Boolean			function_Result;
	CARDINAL_32		resulting_Index;
	Safe_Boolean 		valid_Result;
/* END VAR */
	
	
	function_Result = SAFE_TRUE;
	
	params_For_Change_Window_Size->async_Task_Desired = SAFE_FALSE;
	
	
	if (Get_Dream_Popup_Menu_Choice(k_WINDOW_SIZE_SELECTION
											, &resulting_Index
											, &valid_Result) != SAFE_TRUE) {
		function_Result = SAFE_FALSE;
	}
	else {
		if (valid_Result != SAFE_TRUE) {
			function_Result = SAFE_FALSE;
		}
		else {
			params_For_Change_Window_Size->window_Type = resulting_Index;
			params_For_Change_Window_Size->window_Index = GENERAL_INFO_WINDOW_INDEX;
			SetRect(&(params_For_Change_Window_Size->new_Rect), 0, 0, 1, 1);
		}
	}	
	
	return(function_Result);
}
/*=**************************************************************************=*/


/*=**************************************************************************=*/
Safe_Boolean Change_Window_Size_Control_Preparation(Change_Window_Size_Rec	*params_For_Change_Window_Size)
{
	Safe_Boolean			function_Result;
	CARDINAL_32		resulting_Index;
	Safe_Boolean 		valid_Result;
/* END VAR */
	
	
	function_Result = SAFE_TRUE;
	
	params_For_Change_Window_Size->async_Task_Desired = SAFE_FALSE;
	
	if (Get_Dream_Popup_Menu_Choice(k_WINDOW_SIZE_SELECTION
											, &resulting_Index
											, &valid_Result) != SAFE_TRUE) {
		function_Result = SAFE_FALSE;
	}
	else {
		if (valid_Result != SAFE_TRUE) {
			function_Result = SAFE_FALSE;
		}
		else {
			params_For_Change_Window_Size->window_Type = resulting_Index;
			params_For_Change_Window_Size->window_Index = GENERAL_INFO_WINDOW_INDEX;
			SetRect(&(params_For_Change_Window_Size->new_Rect), 0, 0, 1, 1);
		}
	}	
	
	return(function_Result);
}
/*=**************************************************************************=*/


/*=**************************************************************************=*/
Safe_Boolean Open_Slide_Control_Preparation(Open_Slide_Rec	*params_For_Open_Slide)
{
	Safe_Boolean			function_Result;
	Safe_Boolean 		user_Abort;
/* END VAR */
	
	
	function_Result = SAFE_TRUE;
	
	params_For_Open_Slide->async_Task_Desired = SAFE_FALSE;
	
/* ====
	if (Specify_Slide_To_Load(&(params_For_Open_Slide->desired_File)
											, &user_Abort) != SAFE_TRUE) {
		function_Result = SAFE_FALSE;
	}
	else {
		if (user_Abort == SAFE_TRUE) {
			function_Result = SAFE_FALSE;
		}
	}
==== */

	
	return(function_Result);
}
/*=**************************************************************************=*/


/*=**************************************************************************=*/
Safe_Boolean Open_Slide_Menu_Preparation(Open_Slide_Rec	*params_For_Open_Slide)
{
	Safe_Boolean			function_Result;
	Safe_Boolean 		user_Abort;
/* END VAR */
	
	
	function_Result = SAFE_TRUE;
	
	params_For_Open_Slide->async_Task_Desired = SAFE_FALSE;
/* ====
	if (Specify_Slide_To_Load(&(params_For_Open_Slide->desired_File)
											, &user_Abort) != SAFE_TRUE) {
		function_Result = SAFE_FALSE;
	}
	else {
		if (user_Abort == SAFE_TRUE) {
			function_Result = SAFE_FALSE;
		}
	}
==== */
		
	return(function_Result);
}
/*=**************************************************************************=*/


/*=**************************************************************************=*/
Safe_Boolean Close_Slides_Control_Preparation(Close_Slides_Rec	*params_For_Close_Slides)
{
	Safe_Boolean			function_Result;
	CARDINAL_32		resulting_Index;
	Safe_Boolean 		valid_Result;
/* END VAR */
	
	
	function_Result = SAFE_TRUE;
	
	params_For_Close_Slides->async_Task_Desired = SAFE_FALSE;
	
/* ===
	if (Get_Dream_Popup_Menu_Choice(k_SCENE_SELECTION
											, &resulting_Index
											, &valid_Result) != SAFE_TRUE) {
		function_Result = SAFE_FALSE;
	}
	else {
		if (valid_Result != SAFE_TRUE) {
			function_Result = SAFE_FALSE;
		}
		else {
			params_For_Close_Slides->desired_Slide = resulting_Index;
		}
	}	
=== */
	
	return(function_Result);
}
/*=**************************************************************************=*/


/*=**************************************************************************=*/
Safe_Boolean Close_Slides_Menu_Preparation(Close_Slides_Rec	*params_For_Close_Slides)
{
	Safe_Boolean			function_Result;
	CARDINAL_32		resulting_Index;
	Safe_Boolean 		valid_Result;
/* END VAR */
	
	
	function_Result = SAFE_TRUE;
	
	params_For_Close_Slides->async_Task_Desired = SAFE_FALSE;
	
/* ===
	if (Get_Dream_Popup_Menu_Choice(k_SCENE_SELECTION
											, &resulting_Index
											, &valid_Result) != SAFE_TRUE) {
		function_Result = SAFE_FALSE;
	}
	else {
		if (valid_Result != SAFE_TRUE) {
			function_Result = SAFE_FALSE;
		}
		else {
			params_For_Close_Slides->desired_Slide = resulting_Index;
		}
	}	
=== */
	
	return(function_Result);
}
/*=**************************************************************************=*/


/*=**************************************************************************=*/
Safe_Boolean Close_All_Slides_Control_Preparation(Close_All_Slides_Rec	*params_For_Close_All_Slides)
{
	Safe_Boolean			function_Result;
/* END VAR */
	
	
	function_Result = SAFE_TRUE;
	
	params_For_Close_All_Slides->async_Task_Desired = SAFE_FALSE;

	return(function_Result);
}
/*=**************************************************************************=*/


/*=**************************************************************************=*/
Safe_Boolean Close_All_Slides_Menu_Preparation(Close_All_Slides_Rec	*params_For_Close_All_Slides)
{
	Safe_Boolean			function_Result;
/* END VAR */
	
	
	function_Result = SAFE_TRUE;
	
	params_For_Close_All_Slides->async_Task_Desired = SAFE_FALSE;
	
	return(function_Result);
}
/*=**************************************************************************=*/


/*=**************************************************************************=*/
Safe_Boolean Select_Scene_Control_Preparation(Select_Scene_Rec	*params_For_Select_Scene)
{
	Safe_Boolean			function_Result;
	CARDINAL_32		resulting_Index;
	Safe_Boolean 		valid_Result;
/* END VAR */
	
	
	function_Result = SAFE_TRUE;
	
	params_For_Select_Scene->async_Task_Desired = SAFE_FALSE;

	if (Get_Dream_Popup_Menu_Choice(k_SCENE_SELECTION
											, &resulting_Index
											, &valid_Result) != SAFE_TRUE) {
		function_Result = SAFE_FALSE;
	}
	else {
		if (valid_Result != SAFE_TRUE) {
			function_Result = SAFE_FALSE;
		}
		else {
			params_For_Select_Scene->desired_Scene = resulting_Index;
		}
	}	

	return(function_Result);
}
/*=**************************************************************************=*/


/*=**************************************************************************=*/
Safe_Boolean Select_Scene_Menu_Preparation(Select_Scene_Rec	*params_For_Select_Scene)
{
	Safe_Boolean			function_Result;
	CARDINAL_32		resulting_Index;
	Safe_Boolean 		valid_Result;
/* END VAR */
	
	
	function_Result = SAFE_TRUE;
	
	params_For_Select_Scene->async_Task_Desired = SAFE_FALSE;
	
	if (Get_Dream_Popup_Menu_Choice(k_SCENE_SELECTION
											, &resulting_Index
											, &valid_Result) != SAFE_TRUE) {
		function_Result = SAFE_FALSE;
	}
	else {
		if (valid_Result != SAFE_TRUE) {
			function_Result = SAFE_FALSE;
		}
		else {
			params_For_Select_Scene->desired_Scene = resulting_Index;
		}
	}	

	return(function_Result);
}
/*=**************************************************************************=*/


/*=**************************************************************************=*/
Safe_Boolean Load_Or_Select_Scene_Control_Preparation(Load_Or_Select_Scene_Rec	*params_For_Load_Or_Select_Scene)
{
	Safe_Boolean			function_Result;
	CARDINAL_32		resulting_Index;
	Safe_Boolean 		valid_Result;
/* END VAR */
	
	
	function_Result = SAFE_TRUE;
	
	params_For_Load_Or_Select_Scene->async_Task_Desired = SAFE_FALSE;

	if (Get_Dream_Popup_Menu_Choice(k_SCENE_FILE_SELECTION
											, &resulting_Index
											, &valid_Result) != SAFE_TRUE) {
		function_Result = SAFE_FALSE;
	}
	else {
		if (valid_Result != SAFE_TRUE) {
			function_Result = SAFE_FALSE;
		}
		else {
			params_For_Load_Or_Select_Scene->desired_Scene = resulting_Index;
		}
	}	

	return(function_Result);
}
/*=**************************************************************************=*/


/*=**************************************************************************=*/
Safe_Boolean Load_Or_Select_Scene_Menu_Preparation(Load_Or_Select_Scene_Rec	*params_For_Load_Or_Select_Scene)
{
	Safe_Boolean			function_Result;
	CARDINAL_32		resulting_Index;
	Safe_Boolean 		valid_Result;
/* END VAR */
	
	
	function_Result = SAFE_TRUE;
	
	params_For_Load_Or_Select_Scene->async_Task_Desired = SAFE_FALSE;
	
	if (Get_Dream_Popup_Menu_Choice(k_SCENE_FILE_SELECTION
											, &resulting_Index
											, &valid_Result) != SAFE_TRUE) {
		function_Result = SAFE_FALSE;
	}
	else {
		if (valid_Result != SAFE_TRUE) {
			function_Result = SAFE_FALSE;
		}
		else {
			params_For_Load_Or_Select_Scene->desired_Scene = resulting_Index;
		}
	}	

	return(function_Result);
}
/*=**************************************************************************=*/


/*=**************************************************************************=*/
Safe_Boolean Select_Next_Scene_Control_Preparation(Select_Next_Scene_Rec	*params_For_Select_Next_Scene)
{
	Safe_Boolean			function_Result;
/* END VAR */
	
	
	function_Result = SAFE_TRUE;
	
	params_For_Select_Next_Scene->async_Task_Desired = SAFE_FALSE;

	return(function_Result);
}
/*=**************************************************************************=*/


/*=**************************************************************************=*/
Safe_Boolean Select_Next_Scene_Menu_Preparation(Select_Next_Scene_Rec	*params_For_Select_Next_Scene)
{
	Safe_Boolean			function_Result;
/* END VAR */
	
	
	function_Result = SAFE_TRUE;
	
	params_For_Select_Next_Scene->async_Task_Desired = SAFE_FALSE;
	
	return(function_Result);
}
/*=**************************************************************************=*/


/*=**************************************************************************=*/
Safe_Boolean Select_Previous_Scene_Control_Preparation(Select_Previous_Scene_Rec	*params_For_Select_Previous_Scene)
{
	Safe_Boolean			function_Result;
/* END VAR */
	
	
	function_Result = SAFE_TRUE;
	
	params_For_Select_Previous_Scene->async_Task_Desired = SAFE_FALSE;

	return(function_Result);
}
/*=**************************************************************************=*/


/*=**************************************************************************=*/
Safe_Boolean Select_Previous_Scene_Menu_Preparation(Select_Previous_Scene_Rec	*params_For_Select_Previous_Scene)
{
	Safe_Boolean			function_Result;
/* END VAR */
	
	
	function_Result = SAFE_TRUE;
	
	params_For_Select_Previous_Scene->async_Task_Desired = SAFE_FALSE;
	
	return(function_Result);
}
/*=**************************************************************************=*/


/*=**************************************************************************=*/
Safe_Boolean Select_Transition_Control_Preparation(Select_Transition_Rec	*params_For_Select_Transition)
{
	Safe_Boolean			function_Result;
	CARDINAL_32		resulting_Index;
	Safe_Boolean 		valid_Result;
/* END VAR */
	
	
	function_Result = SAFE_TRUE;
	
	params_For_Select_Transition->async_Task_Desired = SAFE_FALSE;

	if (Get_Dream_Popup_Menu_Choice(k_TRANSITION_SELECTION
											, &resulting_Index
											, &valid_Result) != SAFE_TRUE) {
		function_Result = SAFE_FALSE;
	}
	else {
		if (valid_Result != SAFE_TRUE) {
			function_Result = SAFE_FALSE;
		}
		else {
			params_For_Select_Transition->transition_Desired = resulting_Index;
		}
	}	

	return(function_Result);
}
/*=**************************************************************************=*/


/*=**************************************************************************=*/
Safe_Boolean Select_Transition_Menu_Preparation(Select_Transition_Rec	*params_For_Select_Transition)
{
	Safe_Boolean			function_Result;
	CARDINAL_32		resulting_Index;
	Safe_Boolean 		valid_Result;
/* END VAR */
	
	
	function_Result = SAFE_TRUE;
	
	params_For_Select_Transition->async_Task_Desired = SAFE_FALSE;

	if (Get_Dream_Popup_Menu_Choice(k_TRANSITION_SELECTION
											, &resulting_Index
											, &valid_Result) != SAFE_TRUE) {
		function_Result = SAFE_FALSE;
	}
	else {
		if (valid_Result != SAFE_TRUE) {
			function_Result = SAFE_FALSE;
		}
		else {
			params_For_Select_Transition->transition_Desired = resulting_Index;
		}
	}	

	return(function_Result);
}
/*=**************************************************************************=*/


/*=**************************************************************************=*/
Safe_Boolean Select_First_Transition_Control_Preparation(Select_First_Transition_Rec	*params_For_Select_First_Transition)
{
	Safe_Boolean			function_Result;
	CARDINAL_32		resulting_Index;
	Safe_Boolean 		valid_Result;
/* END VAR */
	
	
	function_Result = SAFE_TRUE;
	
	params_For_Select_First_Transition->async_Task_Desired = SAFE_FALSE;

	if (Get_Dream_Popup_Menu_Choice(k_FIRST_TRANSITION_SELECTION
											, &resulting_Index
											, &valid_Result) != SAFE_TRUE) {
		function_Result = SAFE_FALSE;
	}
	else {
		if (valid_Result != SAFE_TRUE) {
			function_Result = SAFE_FALSE;
		}
		else {
			params_For_Select_First_Transition->transition_Desired = resulting_Index;
		}
	}	

	return(function_Result);
}
/*=**************************************************************************=*/


/*=**************************************************************************=*/
Safe_Boolean Select_First_Transition_Menu_Preparation(Select_First_Transition_Rec	*params_For_Select_First_Transition)
{
	Safe_Boolean			function_Result;
	CARDINAL_32		resulting_Index;
	Safe_Boolean 		valid_Result;
/* END VAR */
	
	
	function_Result = SAFE_TRUE;
	
	params_For_Select_First_Transition->async_Task_Desired = SAFE_FALSE;

	if (Get_Dream_Popup_Menu_Choice(k_FIRST_TRANSITION_SELECTION
											, &resulting_Index
											, &valid_Result) != SAFE_TRUE) {
		function_Result = SAFE_FALSE;
	}
	else {
		if (valid_Result != SAFE_TRUE) {
			function_Result = SAFE_FALSE;
		}
		else {
			params_For_Select_First_Transition->transition_Desired = resulting_Index;
		}
	}	

	return(function_Result);
}
/*=**************************************************************************=*/


/*=**************************************************************************=*/
Safe_Boolean Activate_Twitch_Control_Preparation(Activate_Twitch_Rec	*params_For_Activate_Twitch)
{
	Safe_Boolean			function_Result;
/* END VAR */
	
	
	function_Result = SAFE_TRUE;
	
	params_For_Activate_Twitch->async_Task_Desired = SAFE_FALSE;
	/*  */
	/* HACK: Not implemented yet. */
	/*  */
	params_For_Activate_Twitch->desired_Scene = 0;
	params_For_Activate_Twitch->desired_Item = 0;
	params_For_Activate_Twitch->desired_Twitch = 0;

	return(function_Result);
}
/*=**************************************************************************=*/


/*=**************************************************************************=*/
Safe_Boolean Activate_Twitch_Menu_Preparation(Activate_Twitch_Rec	*params_For_Activate_Twitch)
{
	Safe_Boolean			function_Result;
/* END VAR */
	
	
	function_Result = SAFE_TRUE;
	
	params_For_Activate_Twitch->async_Task_Desired = SAFE_FALSE;
	/*  */
	/* HACK: Not implemented yet. */
	/*  */
	params_For_Activate_Twitch->desired_Scene = 0;
	params_For_Activate_Twitch->desired_Item = 0;
	params_For_Activate_Twitch->desired_Twitch = 0;
	
	return(function_Result);
}
/*=**************************************************************************=*/


/*=**************************************************************************=*/
Safe_Boolean Monitor_Real_MIDI_Control_Preparation(Monitor_Real_MIDI_Rec	*params_For_Monitor_Real_MIDI)
{
	Safe_Boolean			function_Result;
/* END VAR */

	
	function_Result = SAFE_TRUE;
	
	params_For_Monitor_Real_MIDI->async_Task_Desired = SAFE_FALSE;
	

	goto Normal_Function_Exit;
	
Error_Handler:
	function_Result = SAFE_FALSE;
			
Normal_Function_Exit:
	return(function_Result);
}
/*=**************************************************************************=*/


/*=**************************************************************************=*/
Safe_Boolean Monitor_Real_MIDI_Menu_Preparation(Monitor_Real_MIDI_Rec	*params_For_Monitor_Real_MIDI)
{
	Safe_Boolean			function_Result;
/* END VAR */

	
	function_Result = SAFE_TRUE;
	
	params_For_Monitor_Real_MIDI->async_Task_Desired = SAFE_FALSE;
	

	goto Normal_Function_Exit;
	
Error_Handler:
	function_Result = SAFE_FALSE;
			
Normal_Function_Exit:
	return(function_Result);
}
/*=**************************************************************************=*/


/*=**************************************************************************=*/
Safe_Boolean Record_Titled_AIFF_File_Control_Preparation(Record_Titled_AIFF_File_Rec	*params_For_Record_Titled_AIFF_File)
{
	Safe_Boolean			function_Result;
/* END VAR */

	
	function_Result = SAFE_TRUE;
	
	params_For_Record_Titled_AIFF_File->async_Task_Desired = SAFE_FALSE;
	

	goto Normal_Function_Exit;
	
Error_Handler:
	function_Result = SAFE_FALSE;
			
Normal_Function_Exit:
	return(function_Result);
}
/*=**************************************************************************=*/


/*=**************************************************************************=*/
Safe_Boolean Record_Titled_AIFF_File_Menu_Preparation(Record_Titled_AIFF_File_Rec	*params_For_Record_Titled_AIFF_File)
{
	Safe_Boolean			function_Result;
/* END VAR */

	
	function_Result = SAFE_TRUE;
	
	params_For_Record_Titled_AIFF_File->async_Task_Desired = SAFE_FALSE;
	

	goto Normal_Function_Exit;
	
Error_Handler:
	function_Result = SAFE_FALSE;
			
Normal_Function_Exit:
	return(function_Result);
}
/*=**************************************************************************=*/


/*=**************************************************************************=*/
Safe_Boolean Record_From_Any_To_Titled_AIFF_Control_Preparation(Record_From_Any_To_Titled_AIFF_Rec	*params_For_Record_From_Any_To_Titled_AIFF)
{
	Safe_Boolean			function_Result;
/* END VAR */

	
	function_Result = SAFE_TRUE;
	
	params_For_Record_From_Any_To_Titled_AIFF->async_Task_Desired = SAFE_FALSE;
	

	goto Normal_Function_Exit;
	
Error_Handler:
	function_Result = SAFE_FALSE;
			
Normal_Function_Exit:
	return(function_Result);
}
/*=**************************************************************************=*/


/*=**************************************************************************=*/
Safe_Boolean Record_From_Any_To_Titled_AIFF_Menu_Preparation(Record_From_Any_To_Titled_AIFF_Rec	*params_For_Record_From_Any_To_Titled_AIFF)
{
	Safe_Boolean			function_Result;
/* END VAR */

	
	function_Result = SAFE_TRUE;
	
	params_For_Record_From_Any_To_Titled_AIFF->async_Task_Desired = SAFE_FALSE;
	

	goto Normal_Function_Exit;
	
Error_Handler:
	function_Result = SAFE_FALSE;
			
Normal_Function_Exit:
	return(function_Result);
}
/*=**************************************************************************=*/


/*=**************************************************************************=*/
Safe_Boolean Record_From_Any_To_Untitled_AIFF_Control_Preparation(Record_From_Any_To_Untitled_AIFF_Rec	*params_For_Record_From_Any_To_Untitled_AIFF)
{
	Safe_Boolean			function_Result;
/* END VAR */

	
	function_Result = SAFE_TRUE;
	
	params_For_Record_From_Any_To_Untitled_AIFF->async_Task_Desired = SAFE_FALSE;
	

	goto Normal_Function_Exit;
	
Error_Handler:
	function_Result = SAFE_FALSE;
			
Normal_Function_Exit:
	return(function_Result);
}
/*=**************************************************************************=*/


/*=**************************************************************************=*/
Safe_Boolean Record_From_Any_To_Untitled_AIFF_Menu_Preparation(Record_From_Any_To_Untitled_AIFF_Rec	*params_For_Record_From_Any_To_Untitled_AIFF)
{
	Safe_Boolean			function_Result;
/* END VAR */

	
	function_Result = SAFE_TRUE;
	
	params_For_Record_From_Any_To_Untitled_AIFF->async_Task_Desired = SAFE_FALSE;
	

	goto Normal_Function_Exit;
	
Error_Handler:
	function_Result = SAFE_FALSE;
			
Normal_Function_Exit:
	return(function_Result);
}
/*=**************************************************************************=*/


/*=**************************************************************************=*/
Safe_Boolean Stop_Recording_Any_Open_AIFF_Files_Control_Preparation(Stop_Recording_Any_Open_AIFF_Files_Rec	*params_For_Stop_Recording_Any_Open_AIFF_Files)
{
	Safe_Boolean			function_Result;
/* END VAR */

	
	function_Result = SAFE_TRUE;
	
	params_For_Stop_Recording_Any_Open_AIFF_Files->async_Task_Desired = SAFE_FALSE;
	

	goto Normal_Function_Exit;
	
Error_Handler:
	function_Result = SAFE_FALSE;
			
Normal_Function_Exit:
	return(function_Result);
}
/*=**************************************************************************=*/


/*=**************************************************************************=*/
Safe_Boolean Stop_Recording_Any_Open_AIFF_Files_Menu_Preparation(Stop_Recording_Any_Open_AIFF_Files_Rec	*params_For_Stop_Recording_Any_Open_AIFF_Files)
{
	Safe_Boolean			function_Result;
/* END VAR */

	
	function_Result = SAFE_TRUE;
	
	params_For_Stop_Recording_Any_Open_AIFF_Files->async_Task_Desired = SAFE_FALSE;
	

	goto Normal_Function_Exit;
	
Error_Handler:
	function_Result = SAFE_FALSE;
			
Normal_Function_Exit:
	return(function_Result);
}
/*=**************************************************************************=*/


/*=**************************************************************************=*/
Safe_Boolean Play_Last_Untitled_AIFF_Control_Preparation(Play_Last_Untitled_AIFF_Rec	*params_For_Play_Last_Untitled_AIFF)
{
	Safe_Boolean			function_Result;
/* END VAR */

	
	function_Result = SAFE_TRUE;
	
	params_For_Play_Last_Untitled_AIFF->async_Task_Desired = SAFE_FALSE;
	
	goto Normal_Function_Exit;
	
Error_Handler:
	function_Result = SAFE_FALSE;
			
Normal_Function_Exit:
	return(function_Result);
}
/*=**************************************************************************=*/


/*=**************************************************************************=*/
Safe_Boolean Play_Last_Untitled_AIFF_Menu_Preparation(Play_Last_Untitled_AIFF_Rec	*params_For_Play_Last_Untitled_AIFF)
{
	Safe_Boolean			function_Result;
/* END VAR */

	
	function_Result = SAFE_TRUE;
	
	params_For_Play_Last_Untitled_AIFF->async_Task_Desired = SAFE_FALSE;
	

	goto Normal_Function_Exit;
	
Error_Handler:
	function_Result = SAFE_FALSE;
			
Normal_Function_Exit:
	return(function_Result);
}
/*=**************************************************************************=*/


/*=**************************************************************************=*/
Safe_Boolean Stop_Playback_Of_All_AIFFs_Control_Preparation(Stop_Playback_Of_All_AIFFs_Rec	*params_For_Stop_Playback_Of_All_AIFFs)
{
	Safe_Boolean			function_Result;
/* END VAR */

	
	function_Result = SAFE_TRUE;
	
	params_For_Stop_Playback_Of_All_AIFFs->async_Task_Desired = SAFE_FALSE;
	

	goto Normal_Function_Exit;
	
Error_Handler:
	function_Result = SAFE_FALSE;
			
Normal_Function_Exit:
	return(function_Result);
}
/*=**************************************************************************=*/


/*=**************************************************************************=*/
Safe_Boolean Stop_Playback_Of_All_AIFFs_Menu_Preparation(Stop_Playback_Of_All_AIFFs_Rec	*params_For_Stop_Playback_Of_All_AIFFs)
{
	Safe_Boolean			function_Result;
/* END VAR */

	
	function_Result = SAFE_TRUE;
	
	params_For_Stop_Playback_Of_All_AIFFs->async_Task_Desired = SAFE_FALSE;
	

	goto Normal_Function_Exit;
	
Error_Handler:
	function_Result = SAFE_FALSE;
			
Normal_Function_Exit:
	return(function_Result);
}
/*=**************************************************************************=*/


/*=**************************************************************************=*/
/* END OF FILE: Command_Dialogs.c */
