/* START OF FILE: Command_Executor.c */


/*=**************************************************************************=*/
/* [---BEGIN---] FILE HEADER COMMENTS */
/**
 ** SCREEN VIEW = [Adobe Syntax Black-10, Tabs = 4]
 ** PRINTER VIEW = [Adobe Syntax Black-10, Tabs = 4]
 ** =======================================================================
 ** Written by James Merkle
 ** Command_Executor.c - Main control dispatch area
 ** (C) Copyright 1994 James Merkle
 ** All rights reserved, except those wishing to "copy" moderate sections of code or files may do so, so long as
 ** such code is never used directly either in animation-related tools or multimedia tools that are  
 ** used in live performance or live broadcast or recorded mass market broadcast or film. Desktop usage not restricted.
 ** Conventional desktop game or entertainment not restricted.
 ** =======================================================================
 **
 **
 ** ============
 ** Revision History
 ** ============
 **
 **  1994.02.01		(James Merkle)
 **						Initial coding started
 **
 **
 **
 **
 ** =================
 ** OTHER INFORMATION
 ** =================
 **  
 **
 **/
/* [----END----] FILE HEADER COMMENTS */
/*=**************************************************************************=*/


/*=**************************************************************************=*/
/* [---BEGIN---] #include AREA */

#include "Error_Utilities.h"
#include "Memory_Utilities.h"
#include "Program_Globals.h"
#include "Dreaming_Popup_Dialogs.h"
#include "Shell_Assembly.h"
#include "Shell_Dialogs.h"
#include "Shell_Globals.h"
#include "String_Utilities.h"
#include "Window_Utilities.h"
#include "Remote_Control_Full.h"

#include "Network_Zone_Control.h"
#include "Dreaming_Audio.h"
#include "Dreaming_Stuff.h"
#include "Dreaming_Color_Stuff.h"
#include "Dreaming_File_Loading.h"
#include "Dreaming_Backdrop_Files.h"
#include "Spin_O_Rama_Files.h"
#include "Command_Menu.h"
#include "Command_Executor.h"
#include "Command_Dialogs.h"
#include "Command_Redirector.h"
#include "Dreaming_Scene_Files.h"
#include "Spin_O_Rama.h"
#include "Object_Inventory.h"
#include "Slide_Show_Routines.h"
#include "Dreaming_PhoneBridge.h"

#include "Dreaming_Animation.h"
#include "Video_Backdrop_Background.h"
#include "Play_Movie_Fast_Benchmark.h"
#include "Dreaming_Audio_Record.h"

#include "Core_Globals.h"
#include "Core_Specific.h"
#include "Graphics_Demos_Support.h"
#include "Graphics_Primitives.h"
#include "Dreaming_Analog_Card.h"
#include "Graphics_Primitives_Support.h"
#include "Sound_Play.h"

#include "Command_Executor.h"

/* [----END----] #include AREA */
/*=**************************************************************************=*/


/*=**************************************************************************=*/
/* [---BEGIN---] #define AREA */


/* [----END----] #define AREA */
/*=**************************************************************************=*/


/*=**************************************************************************=*/
/* [---BEGIN---] TYPE AREA */


/* [----END----] TYPE AREA */
/*=**************************************************************************=*/


/*=**************************************************************************=*/
/* [---BEGIN---] PROTOTYPE AREA */
															
void Display_Not_Implemented_Warning(Str255	routine);

/* [----END----] PROTOTYPE AREA */
/*=**************************************************************************=*/


/*=**************************************************************************=*/
/* [---BEGIN---] GLOBAL DEFINITION AREA */


/* [----END----] GLOBAL DEFINITION AREA */
/*=**************************************************************************=*/


/*=**************************************************************************=*/
/* [---BEGIN---] LOCAL STATIC DEFINITION AREA */

static CARDINAL_8	*unit_Name;

/* [----END----] LOCAL STATIC DEFINITION AREA */
/*=**************************************************************************=*/


/*=**************************************************************************=*/
Safe_Boolean Initialize_Command_Executor(void)
{
	CARDINAL_8			*static_Unit_Name = "\pCommand_Executor";
/* END VAR */

/*  */
/* Global persistant structures are created here */
/*  */

	unit_Name = static_Unit_Name;
	return(SAFE_TRUE);
}
/*=**************************************************************************=*/


/*=**************************************************************************=*/
Safe_Boolean Destroy_Command_Executor(void)
{
	Safe_Boolean		function_Result;
/* END VAR */

/*  */
/* Global persistant structures are deleted here */
/*  */

	function_Result = SAFE_TRUE;
	return(function_Result);
}
/*=**************************************************************************=*/


/*=**************************************************************************=*/
void Display_Not_Implemented_Warning(Str255	routine)
{
	; /* NULL */
}
/*=**************************************************************************=*/


/*=**************************************************************************=*/
Safe_Boolean Create_Puppet_From_Spin_File(Import_Puppet_Rec		*params_For_Import_Puppet)
{
	Safe_Boolean			function_Result;
/* END VAR */

	
	function_Result = Import_Puppet(params_For_Import_Puppet);
	
	return(function_Result);

} /* Safe_Boolean Create_Puppet_From_Spin_File(... */
/*=**************************************************************************=*/


/*=**************************************************************************=*/
Safe_Boolean Add_Another_Spin_File_To_Puppet(Add_Another_Spin_File_Rec		*params_For_Add_Another_Spin_File_To_Puppet)
{
	Safe_Boolean			function_Result;
	Safe_Boolean			abort_Desired;
/* END VAR */


	if (Define_Puppet_Using_Spin_File(params_For_Add_Another_Spin_File_To_Puppet->desired_Item
											, &abort_Desired
											, SAFE_FALSE) != SAFE_TRUE) {
	
		if (abort_Desired == SAFE_TRUE) {
			/*  */
			/* Ignore, we are quitting */
			/*  */
			function_Result = SAFE_TRUE;
			goto Normal_Function_Exit;
		}
		else {
			goto Error_Handler;
		}
	}
	
	function_Result = SAFE_TRUE;
	goto Normal_Function_Exit;
	
Error_Handler:
	function_Result = SAFE_FALSE;
		
Normal_Function_Exit:
	return(function_Result);

} /* Safe_Boolean Create_Puppet_From_Spin_File(... */
/*=**************************************************************************=*/


/*=**************************************************************************=*/
Safe_Boolean Load_Puppet(Load_Puppet_Rec	*params_For_Load_Puppet)
{
	Safe_Boolean			function_Result;
	SignedByte			status_Globals_State;
	WindowPtr			current_Graphics_Window_Ptr;
/* END VAR */
	
	
	Select_Graphics_Window(TEST_DREAMING_WINDOW_INDEX);
	
	/*  */
	/* Lock down because structure content addresses are being passed */
	/*  */
	if (Begin_Use_Of_Handle((Handle) active_Window_List[TEST_DREAMING_WINDOW_INDEX]
								, MOVE_HIGH_REQUIRED
								, &status_Globals_State) != SAFE_TRUE) {
								
		goto Error_Handler;
	}

	current_Graphics_Window_Ptr	= (**active_Window_List[TEST_DREAMING_WINDOW_INDEX]).status_Window;

	SetWTitle(current_Graphics_Window_Ptr, (StringPtr)WINDOW_TITLE_FOR_TEST_DREAMING);
	
	ForeColor(blackColor);
	BackColor(blackColor);
	BackPat(&(qd.black));
/* ===
	EraseRect(&(current_Graphics_Window_Ptr->portRect));
=== */
	BackColor(whiteColor);
					
	if (Load_A_Puppet_Into_Any_Scene(FALSE, k_IMPORT_FROM_NORMAL_LOAD) != SAFE_TRUE) {
		goto Error_Handler;
	}

	/*  */
	/* Save window to a PICT for use during updates */
	/*  */
	Support_Update_By_Getting_Pict(&((**active_Window_List[TEST_DREAMING_WINDOW_INDEX]).update_Pict)
										,&((**active_Window_List[TEST_DREAMING_WINDOW_INDEX]).status_Window));
										
	/*  */
	/* Set window to a not use a PICT for use during updates */
	/*  */
	(**active_Window_List[TEST_DREAMING_WINDOW_INDEX]).update_Pict_Permissable_To_Use = SAFE_FALSE;
	(**active_Window_List[TEST_DREAMING_WINDOW_INDEX]).update_Pict = nil;
										
	if (End_Use_Of_Handle((Handle) active_Window_List[TEST_DREAMING_WINDOW_INDEX], status_Globals_State) != SAFE_TRUE) {
		goto Error_Handler;
	}
	
	if (Reset_Menu_Control_Selectability() != SAFE_TRUE) {
		/*  */
		/* Ignore error not vital */
		/*  */
	}
	
	function_Result = SAFE_TRUE;
	goto Normal_Function_Exit;
	
Error_Handler:
	function_Result = SAFE_FALSE;
			
Normal_Function_Exit:
	return(function_Result);
}
/*=**************************************************************************=*/


/*=**************************************************************************=*/
Safe_Boolean Close_Puppet(Close_Puppet_Rec	*params_For_Close_Puppet)
{
	Safe_Boolean			function_Result;
/* END VAR */
	
	
	function_Result = SAFE_TRUE;
	
	if (dream.active_Puppets_Total != 0) {
	
		if (dream.puppet_Memory_Initialized[params_For_Close_Puppet->desired_Item] == MEMORY_INITIALIZED) {
			if (Delete_A_Loaded_Puppet(params_For_Close_Puppet->desired_Item) != SAFE_TRUE) {
				function_Result = SAFE_FALSE;
			}	
		}		
			
		if (Reset_Menu_Control_Selectability() != SAFE_TRUE) {
			/*  */
			/* Not normal to ever fail */
			/*  */
			function_Result = SAFE_FALSE;
		}
	}
	
	return(function_Result);
}
/*=**************************************************************************=*/


/*=**************************************************************************=*/
Safe_Boolean Save_Puppet(Save_Puppet_Rec	*params_For_Save_Puppet)
{
	Safe_Boolean		function_Result;
	Safe_Boolean 	user_Abort;
/* END VAR */
	
	if (Save_Puppet_Sequence(params_For_Save_Puppet->desired_Item, &user_Abort) != SAFE_TRUE) {
		/*  */
		/* ignore all forms of errors including user cancel */
		/*  */
	}
			
	function_Result = SAFE_TRUE;
	goto Normal_Function_Exit;
	
Error_Handler:
	function_Result = SAFE_FALSE;
			
Normal_Function_Exit:
	return(function_Result);
}
/*=**************************************************************************=*/


/*=**************************************************************************=*/
Safe_Boolean Import_Puppet(Import_Puppet_Rec	*params_For_Import_Puppet)
{
	Safe_Boolean			function_Result;
	SignedByte			status_Globals_State;
	WindowPtr			current_Graphics_Window_Ptr;
/* END VAR */
	
	
	Select_Graphics_Window(TEST_DREAMING_WINDOW_INDEX);
	
	/*  */
	/* Lock down because structure content addresses are being passed */
	/*  */
	if (Begin_Use_Of_Handle((Handle) active_Window_List[TEST_DREAMING_WINDOW_INDEX], MOVE_HIGH_REQUIRED, &status_Globals_State) != SAFE_TRUE) {
		goto Error_Handler;
	}

	current_Graphics_Window_Ptr	= (**active_Window_List[TEST_DREAMING_WINDOW_INDEX]).status_Window;

	SetWTitle(current_Graphics_Window_Ptr, (StringPtr)WINDOW_TITLE_FOR_TEST_DREAMING);
	
	ForeColor(blackColor);
	BackColor(blackColor);
	BackPat(&(qd.black));
	EraseRect(&(current_Graphics_Window_Ptr->portRect));
	BackColor(whiteColor);
					
	if (Load_A_Puppet_Into_Any_Scene(TRUE, params_For_Import_Puppet->import_Source) != SAFE_TRUE) {
		goto Error_Handler;
	}

	/*  */
	/* Set window to a not use a PICT for use during updates */
	/*  */
	(**active_Window_List[TEST_DREAMING_WINDOW_INDEX]).update_Pict_Permissable_To_Use = SAFE_FALSE;
	(**active_Window_List[TEST_DREAMING_WINDOW_INDEX]).update_Pict = nil;
														
	if (End_Use_Of_Handle((Handle) active_Window_List[TEST_DREAMING_WINDOW_INDEX], status_Globals_State) != SAFE_TRUE) {
		goto Error_Handler;
	}
	
	if (Reset_Menu_Control_Selectability() != SAFE_TRUE) {
		/*  */
		/* Ignore error not vital */
		/*  */
	}

	function_Result = SAFE_TRUE;
	goto Normal_Function_Exit;
	
Error_Handler:
	function_Result = SAFE_FALSE;
			
Normal_Function_Exit:
	return(function_Result);
}
/*=**************************************************************************=*/


/*=**************************************************************************=*/
Safe_Boolean Export_Puppet(Export_Puppet_Rec	*params_For_Export_Puppet)
{
	Safe_Boolean		function_Result;
	Safe_Boolean 	user_Abort;
/* END VAR */
	
		if (Export_Puppet_Sequence(params_For_Export_Puppet->desired_Item, &user_Abort) != SAFE_TRUE) {
		/*  */
		/* ignore all forms of errors including user cancel */
		/*  */
	}
			
	function_Result = SAFE_TRUE;
	goto Normal_Function_Exit;
	
Error_Handler:
	function_Result = SAFE_FALSE;
			
Normal_Function_Exit:
	return(function_Result);
}
/*=**************************************************************************=*/


/*=**************************************************************************=*/
Safe_Boolean Import_Prop(Import_Prop_Rec	*params_For_Import_Prop)
{
	Safe_Boolean			function_Result;
/* END VAR */
	
	
	function_Result = SAFE_TRUE;
	/*  */
	/* HACK: Not implemented yet. */
	/*  */
	Display_Not_Implemented_Warning("\pImport_Prop");
	
	return(function_Result);
}
/*=**************************************************************************=*/


/*=**************************************************************************=*/
Safe_Boolean Export_Prop(Export_Prop_Rec	*params_For_Export_Prop)
{
	Safe_Boolean			function_Result;
/* END VAR */
	
	
	function_Result = SAFE_TRUE;
	/*  */
	/* HACK: Not implemented yet. */
	/*  */
	Display_Not_Implemented_Warning("\pExport_Prop");
	
	return(function_Result);
}
/*=**************************************************************************=*/


/*=**************************************************************************=*/
Safe_Boolean Time_Animation(Time_Animation_Rec	*params_For_Time_Animation)
{
	Safe_Boolean			function_Result;
	SignedByte			status_Globals_State;
	WindowPtr			current_Graphics_Window_Ptr;
	 INTEGER_16			index_Into_Scene;
	Boolean				three_D_Test;
/* END VAR */

	three_D_Test = FALSE;
	if (dream.spin_Objects.total_Spin_Objects > 0) {
		if ((active_Window_List[MAIN_SPINNING_WINDOW_INDEX] != nil)
			&& ((**active_Window_List[MAIN_SPINNING_WINDOW_INDEX]).structure_Initialized == SAFE_TRUE)) {
			three_D_Test = TRUE;
		}
	}

	if (three_D_Test == TRUE) {
		Select_Graphics_Window(MAIN_SPINNING_WINDOW_INDEX);
		if (Benchmark_Spinning_Window_Completely(FALSE, TRUE) == SAFE_TRUE) {
		
		}
	} /* if (three_D_Test != TRUE) */
	else {

		Select_Graphics_Window(TEST_DREAMING_WINDOW_INDEX);
		/*  */
		/* Lock down for no real reason except paranoia? */
		/*  */
		if (Begin_Use_Of_Handle((Handle) active_Window_List[TEST_DREAMING_WINDOW_INDEX], MOVE_HIGH_REQUIRED, &status_Globals_State) != SAFE_TRUE) {
			goto Error_Handler;
		}
	
		current_Graphics_Window_Ptr	= (**active_Window_List[TEST_DREAMING_WINDOW_INDEX]).status_Window;
		
		if ((dream.play.scene_Memory_In_Use[dream.current_Scene]) == MEMORY_INITIALIZED) {
					
			if (Find_Puppet_Position_In_Scene(dream.current_Scene
																	, params_For_Time_Animation->desired_Item
																	, &index_Into_Scene) == SAFE_TRUE) {
			
				if (Benchmark_Animation_Frames(params_For_Time_Animation->desired_Item, dream.current_Scene, index_Into_Scene) != SAFE_TRUE) {
				
				
				}
			}
		}
		
		if (End_Use_Of_Handle((Handle) active_Window_List[TEST_DREAMING_WINDOW_INDEX], status_Globals_State) != SAFE_TRUE) {
			goto Error_Handler;
		}
	}
	
	function_Result = SAFE_TRUE;
	goto Normal_Function_Exit;
	
Error_Handler:
	function_Result = SAFE_FALSE;
			
Normal_Function_Exit:
	return(function_Result);
}
/*=**************************************************************************=*/


/*=**************************************************************************=*/
Safe_Boolean Set_Life_Jiggle(Set_Life_Jiggle_Rec	*params_For_Set_Life_Jiggle)
{
	Safe_Boolean			function_Result;
	Safe_Boolean			result;
	Boolean				change_Made;
	INTEGER_32			loop;
	INTEGER_16			 current_Stage;
	INTEGER_32			index;
	INTEGER_32			scenes_Found;
	Boolean				stages_Finished;
	INTEGER_32			located_Items;
	Puppet_Ptr			current_Puppet;
	CARDINAL_8			*code_Name;
/* END VAR */
	
	
	function_Result = SAFE_TRUE;
	code_Name = "\pSet_Life_Jiggle";

	/*  */
	/* Simulate shaky camera or active puppet */
	/*  */
		
	change_Made = TRUE;
	if (dream.horizontal_Jiggle_Maximum == params_For_Set_Life_Jiggle->horizontal_Jiggle_Maximum) {
		if (dream.vertical_Jiggle_Maximum == params_For_Set_Life_Jiggle->vertical_Jiggle_Maximum) {
			if (dream.jiggle_Active == params_For_Set_Life_Jiggle->jiggle_Enabled) {
				change_Made = FALSE;
			}
		}
	}
	
	if (change_Made == TRUE) {
	
	
		dream.horizontal_Jiggle_Maximum = params_For_Set_Life_Jiggle->horizontal_Jiggle_Maximum;
		dream.vertical_Jiggle_Maximum = params_For_Set_Life_Jiggle->vertical_Jiggle_Maximum;
		dream.jiggle_Active = params_For_Set_Life_Jiggle->jiggle_Enabled;
		
		if ((dream.horizontal_Jiggle_Maximum != 0) || (dream.vertical_Jiggle_Maximum != 0)) {
			if (g_Dreaming_Module_Globals_Initialized == SAFE_TRUE) {

				/*  */
				/* Make a copy of "total_Lazy_Active_Region" so that it */
				/* can be modifed larger for jiggle effect */
				/*  */
	
				located_Items = 0;
				for (loop = 0; ((loop < k_MAXIMUM_PUPPETS) && (located_Items < dream.active_Puppets_Total)); loop++) {			
					if (dream.puppet_Memory_Initialized[loop] == MEMORY_INITIALIZED) {
						located_Items += 1;
						
						current_Puppet = *(dream.puppet[loop]);
						
						if (Check_If_Legal_Direct_Art_Or_Complain(current_Puppet) != SAFE_TRUE) {
							goto Error_Handler;
						}
						
						if (current_Puppet->direct_Art.total_Lazy_Active_Region != nil) {
							CopyRgn(current_Puppet->direct_Art.total_Lazy_Active_Region, current_Puppet->direct_Art.original_Total_Lazy_Active_Region);
							InsetRgn(current_Puppet->direct_Art.total_Lazy_Active_Region, -(dream.horizontal_Jiggle_Maximum), -(dream.vertical_Jiggle_Maximum));
						}
					}
				}
				
				/*  */
				/* We must cycle through scenes now updating wider or narrower background data */
				/*  */
				scenes_Found = 0;
				for (index = 0; ((index < k_MAX_ACTIVE_SCENES) && (scenes_Found < dream.play.total_Scenes)); index++) {
					if (dream.play.scene_Memory_In_Use[index] == MEMORY_INITIALIZED) {
						scenes_Found += 1;
						
						current_Stage = 0;
						do {
							result = Add_Background_Of_Scene_To_Puppets(index
																					, current_Stage
																					, &stages_Finished);
							current_Stage += 1;															
						} while ((result == SAFE_TRUE) && (stages_Finished != TRUE));
						
						if (result != SAFE_TRUE) {
							/*  */
							/* Ignore. */
							/*  */
							Display_Serious_Error_Message(unit_Name, code_Name, "\punexpected problem (Add_Background_Of_Scene_To_Puppets failed) ", "\pMemory?");
						}
					}
				}
			}
		}
	}
	
	function_Result = SAFE_TRUE;
	goto Normal_Function_Exit;
	
Error_Handler:
	function_Result = SAFE_FALSE;
			
Normal_Function_Exit:

	return(function_Result);
}
/*=**************************************************************************=*/


/*=**************************************************************************=*/
Safe_Boolean Close_Backdrop(Close_Backdrop_Rec	*params_For_Close_Backdrop)
{
	Safe_Boolean			function_Result;
/* END VAR */
	
	
	function_Result = SAFE_TRUE;
	/*  */
	/* HACK: Not implemented yet. */
	/*  */
	Display_Not_Implemented_Warning("\pClose_Backdrop");
	
	return(function_Result);
}
/*=**************************************************************************=*/


/*=**************************************************************************=*/
Safe_Boolean Load_Backdrop(Load_Backdrop_Rec	*params_For_Load_Backdrop)
{
	Safe_Boolean			function_Result;
/* END VAR */
	
	
	function_Result = Load_Backdrop_File_Immediate_Style(params_For_Load_Backdrop);
	
	return(function_Result);
}
/*=**************************************************************************=*/


/*=**************************************************************************=*/
Safe_Boolean Save_Backdrop(Save_Backdrop_Rec	*params_For_Save_Backdrop)
{
	Safe_Boolean			function_Result;
/* END VAR */
	
	function_Result = Save_Backdrop_Sequence(params_For_Save_Backdrop);
	
	return(function_Result);
}
/*=**************************************************************************=*/


/*=**************************************************************************=*/
Safe_Boolean Convert_Backdrop(Convert_Backdrop_Rec	*params_For_Convert_Backdrop)
{
	Safe_Boolean			function_Result;
/* END VAR */
	
	
	function_Result = Convert_Backdrop_File_Immediate_Style(params_For_Convert_Backdrop);
	
	return(function_Result);
}
/*=**************************************************************************=*/


/*=**************************************************************************=*/
Safe_Boolean Export_Backdrop(Export_Backdrop_Rec	*params_For_Export_Backdrop)
{
	Safe_Boolean			function_Result;
/* END VAR */
	
	
	function_Result = Export_Backdrop_Sequence(params_For_Export_Backdrop);
	
	return(function_Result);
}
/*=**************************************************************************=*/


/*=**************************************************************************=*/
Safe_Boolean Close_Spin_Object(Close_Spin_Object_Rec	*params_For_Close_Spin_Object)
{
	Safe_Boolean			function_Result;
/* END VAR */
	
	
	function_Result = SAFE_TRUE;
	/*  */
	/* HACK: Not implemented yet. */
	/*  */
	Display_Not_Implemented_Warning("\pClose_Spin_Object");
	
	return(function_Result);
}
/*=**************************************************************************=*/


/*=**************************************************************************=*/
Safe_Boolean Load_Spin_Object(Load_Spin_Object_Rec	*params_For_Load_Spin_Object)
{
	Safe_Boolean			function_Result;
/* END VAR */
	
	if (Select_Spin_Window_Display_First_Time() != SAFE_TRUE) {
		/*  */
		/* might as well complain about it? No not in this version, though very bad */
		/*  */
	}

	function_Result = Load_Spin_Object_File_Immediate_Style(params_For_Load_Spin_Object);
	
	return(function_Result);
}
/*=**************************************************************************=*/


/*=**************************************************************************=*/
Safe_Boolean Save_Spin_Object(Save_Spin_Object_Rec	*params_For_Save_Spin_Object)
{
	Safe_Boolean			function_Result;
/* END VAR */
	
	function_Result = Save_Spin_Object_Sequence(params_For_Save_Spin_Object);
	
	return(function_Result);
}
/*=**************************************************************************=*/


/*=**************************************************************************=*/
Safe_Boolean Convert_Spin_Object(Convert_Spin_Object_Rec	*params_For_Convert_Spin_Object)
{
	Safe_Boolean			function_Result;
/* END VAR */
	
	if (Select_Spin_Window_Display_First_Time() != SAFE_TRUE) {
		/*  */
		/* might as well complain about it? No not in this version, though very bad */
		/*  */
	}

	function_Result = Convert_Spin_Object_File_Immediate_Style(params_For_Convert_Spin_Object);
	
	return(function_Result);
}
/*=**************************************************************************=*/


/*=**************************************************************************=*/
Safe_Boolean Export_Spin_Object(Export_Spin_Object_Rec	*params_For_Export_Spin_Object)
{
	Safe_Boolean			function_Result;
/* END VAR */
	
	
	function_Result = Export_Spin_Object_Sequence(params_For_Export_Spin_Object);
	
	return(function_Result);
}
/*=**************************************************************************=*/


/*=**************************************************************************=*/
Safe_Boolean Recompress_Spin_Object(Recompress_Spin_Object_Rec	*params_For_Recompress_Spin_Object)
{
	Safe_Boolean			function_Result;
/* END VAR */
	
	
	function_Result = Recompress_Spin_Object_BASE(params_For_Recompress_Spin_Object);
	
	return(function_Result);
}
/*=**************************************************************************=*/


/*=**************************************************************************=*/
Safe_Boolean Select_Scene(Select_Scene_Rec	*params_For_Select_Scene)
{
	Safe_Boolean			function_Result;
	CARDINAL_32		previous_Scene;
/* END VAR */
	
	
	function_Result = SAFE_TRUE;
	
	if (dream.current_Scene != params_For_Select_Scene->desired_Scene) {
	
		previous_Scene = dream.current_Scene;
		dream.current_Scene = params_For_Select_Scene->desired_Scene;
		
		if (Redraw_Idle_Screen_Completely(FALSE, FALSE) != SAFE_TRUE) {
			function_Result = SAFE_FALSE;
			
			dream.current_Scene = previous_Scene;
			if (Redraw_Idle_Screen_Completely(FALSE, FALSE) != SAFE_TRUE) {
			}
		}
		else {
			dream.previously_Displayed_Scene = previous_Scene;
		}
	}
	
	return(function_Result);
}
/*=**************************************************************************=*/


/*=**************************************************************************=*/
Safe_Boolean Load_Or_Select_Scene(Load_Or_Select_Scene_Rec	*params_For_Load_Or_Select_Scene)
{
	Safe_Boolean			function_Result;
	Open_Scene_Rec		params_For_Open_Scene;
	CARDINAL_32		scene_To_Use;
	Str255					name_String;
	CARDINAL_32		index;
	Boolean				still_Searching;
	CARDINAL_32		match_Count;
/* END VAR */
	
	
	function_Result = SAFE_FALSE;
		
	name_String[0] = 0;
	match_Count = 0;
			
	if (g_Dreaming_Module_Globals_Initialized == SAFE_TRUE) {
		still_Searching = TRUE;
		for (index = 0; ((index < k_MAXIMUM_DATABASE_SCENES) 
										&& (match_Count < g_Remote_Scenes.total_Available)
										&& (still_Searching == TRUE)); index++) {
										
										
				if ((**g_Remote_Scenes.info.scenes.handle).scenes[index].active == SAFE_TRUE) {

					if (match_Count == params_For_Load_Or_Select_Scene->desired_Scene) {
	
						if ((**g_Remote_Scenes.info.scenes.handle).scenes[index].type_Of_Source == k_FROM_FILE) {
							still_Searching = FALSE;

							if ((**g_Remote_Scenes.info.scenes.handle).scenes[index].active == SAFE_TRUE) {
							
								params_For_Open_Scene.desired_Scene = params_For_Load_Or_Select_Scene->desired_Scene;
								params_For_Open_Scene.desired_File = (**g_Remote_Scenes.info.scenes.handle).scenes[index].file_Info;
								params_For_Open_Scene.async_Task_Desired = SAFE_TRUE;
								
								function_Result = Load_Scene_File_Immediate_Style(&params_For_Open_Scene);
								
								if (function_Result == SAFE_TRUE) {
								
          			/* =========
									previous_Scene = dream.current_Scene;
									dream.current_Scene = params_For_Load_Or_Select_Scene->desired_Scene;
									
									if (Redraw_Idle_Screen_Completely(FALSE, FALSE) != SAFE_TRUE) {
										function_Result = SAFE_FALSE;
										
										dream.current_Scene = previous_Scene;
										if (Redraw_Idle_Screen_Completely(FALSE, FALSE) != SAFE_TRUE) {
										}
									}
				================== */
				
								}

							}
						}
					}
					match_Count += 1;
					
				}
			}
	}	
	
	return(function_Result);

}
/*=**************************************************************************=*/


/*=**************************************************************************=*/
Safe_Boolean Select_Next_Scene(Select_Next_Scene_Rec	*params_For_Select_Next_Scene)
{
	Safe_Boolean			function_Result;
/* END VAR */
	
						
	function_Result = Advance_Scene(1);
	
	return(function_Result);
}
/*=**************************************************************************=*/


/*=**************************************************************************=*/
Safe_Boolean Select_Previous_Scene(Select_Previous_Scene_Rec	*params_For_Select_Previous_Scene)
{
	Safe_Boolean			function_Result;
/* END VAR */
	
						
	function_Result = Advance_Scene(0);
	
	return(function_Result);
}
/*=**************************************************************************=*/


/*=**************************************************************************=*/
Safe_Boolean Show_Controls(Show_Controls_Rec	*params_For_Show_Controls)
{
	Safe_Boolean			function_Result;
/* END VAR */
	
						
	function_Result = SAFE_TRUE;
	
	return(function_Result);
}
/*=**************************************************************************=*/


/*=**************************************************************************=*/
Safe_Boolean Show_Status(Show_Status_Rec	*params_For_Show_Status)
{
	Safe_Boolean			function_Result;
/* END VAR */
	
	
	function_Result = SAFE_TRUE;
	/*  */
	/* HACK: Not implemented yet. */
	/*  */
	Display_Not_Implemented_Warning("\pShow_Status");
	
	return(function_Result);
}
/*=**************************************************************************=*/


/*=**************************************************************************=*/
Safe_Boolean Show_Item_List(Show_Item_List_Rec	*params_For_Show_Item_List)
{
	Safe_Boolean			function_Result;
/* END VAR */
	
	
	function_Result = SAFE_TRUE;
	/*  */
	/* HACK: Not implemented yet. */
	/*  */
	Display_Not_Implemented_Warning("\pShow_Item_List");
	
	return(function_Result);
}
/*=**************************************************************************=*/


/*=**************************************************************************=*/
Safe_Boolean Setup_Chromakey_Video(Setup_Chromakey_Video_Rec	*params_For_Setup_Chromakey_Video)
{
	Safe_Boolean			function_Result;
/* END VAR */
	
	
	function_Result = Start_Video_Display_Setup(params_For_Setup_Chromakey_Video);
	
	return(function_Result);
}
/*=**************************************************************************=*/


/*=**************************************************************************=*/
Safe_Boolean Setup_Broadcast(Setup_Broadcast_Rec	*params_For_Setup_Broadcast)
{
	Safe_Boolean			function_Result;
/* END VAR */
	
	
	function_Result = SAFE_TRUE;
	/*  */
	/* HACK: Not implemented yet. */
	/*  */
	Display_Not_Implemented_Warning("\pSetup_Broadcast");
	
	return(function_Result);
}
/*=**************************************************************************=*/


/*=**************************************************************************=*/
Safe_Boolean Associate_Voices(Associate_Voices_Rec	*params_For_Associate_Voices)
{
	Safe_Boolean			function_Result;
/* END VAR */
	
	
	function_Result = SAFE_TRUE;
	/*  */
	/* HACK: Not implemented yet. */
	/*  */
	Display_Not_Implemented_Warning("\pAssociate_Voices");
	
	return(function_Result);
}
/*=**************************************************************************=*/


/*=**************************************************************************=*/
Safe_Boolean Repostition_For_Broadcast(Repostition_For_Broadcast_Rec	*params_For_Repostition_For_Broadcast)
{
	Safe_Boolean			function_Result;
/* END VAR */
	
	function_Result = SAFE_TRUE;
	/*  */
	/* HACK: Not implemented yet. */
	/*  */
	Display_Not_Implemented_Warning("\pRepostition_For_Broadcast");
	
	return(function_Result);
}
/*=**************************************************************************=*/


/*=**************************************************************************=*/
Safe_Boolean Monitor_Remote(Monitor_Remote_Rec	*params_For_Monitor_Remote)
{
	Safe_Boolean			function_Result;
/* END VAR */
	

	if (g_Voices.monitor_Network_As_Speech == SAFE_TRUE) {
		SetItemMark(g_Shell.interface.program_Menus[AUDIO_MENU]
					, MENU_COMMAND_MONITOR_REMOTE
					, noMark);

		if (Stop_All_Sound_Monitoring_Of_Type(REMOTE_NETWORK_FEED) != SAFE_TRUE) {
			/*  */
			/* Ignore error */
			/*  */
		}
		g_Voices.monitor_Network_As_Speech = SAFE_FALSE;
	}
	else {
		g_Voices.monitor_Network_As_Speech = SAFE_TRUE;
		SetItemMark(g_Shell.interface.program_Menus[AUDIO_MENU]
					, MENU_COMMAND_MONITOR_REMOTE
					, checkMark);

		if (Attach_Default_Inputs_To_All_Scene_Puppets(dream.play.default_Start_Audio_Point) != SAFE_TRUE) {
			/*  */
			/* Ignore Errors */
			/*  */
		}
		
	}
	function_Result = SAFE_TRUE;
	
	return(function_Result);
}
/*=**************************************************************************=*/


/*=**************************************************************************=*/
Safe_Boolean Monitor_Microphone(Monitor_Microphone_Rec	*params_For_Monitor_Microphone)
{
	Safe_Boolean			function_Result;
	Recording_Ptr 		voice_Ptr;
	Safe_Boolean			recording_Activated;
/* END VAR */

	
	if (g_Voices.monitor_Microphone_For_Speech == SAFE_TRUE) {
		SetItemMark(g_Shell.interface.program_Menus[AUDIO_MENU]
					, MENU_COMMAND_MONITOR_MICROPHONE
					, noMark);
				
				
		if (Stop_All_Sound_Monitoring_Of_Type(PRIMARY_SOUND_INPUT) != SAFE_TRUE) {
			/*  */
			/* Ignore error */
			/*  */
		}
		g_Voices.monitor_Microphone_For_Speech = SAFE_FALSE;
	}
	else {
		g_Voices.monitor_Microphone_For_Speech = SAFE_TRUE;
		SetItemMark(g_Shell.interface.program_Menus[AUDIO_MENU]
					, MENU_COMMAND_MONITOR_MICROPHONE
					, checkMark);
										
		if (Get_Sound_Input_Voice(PRIMARY_SOUND_INPUT, (Recording_Ptr *)&voice_Ptr) != SAFE_TRUE) {
		
			if (Stop_All_Sound_Monitoring_Of_Type(PRIMARY_SOUND_INPUT) != SAFE_TRUE) {
				/*  */
				/* Ignore error */
				/*  */
			}
		
			recording_Activated = SAFE_FALSE;
		}
		else {

			if (Start_Sound_Input_Monitoring(voice_Ptr, &recording_Activated, 0, params_For_Monitor_Microphone->select_Any_Input) != SAFE_TRUE) {
				recording_Activated = SAFE_FALSE;
				voice_Ptr->initialized = SAFE_FALSE;
			}
		}
		
		if (recording_Activated != SAFE_TRUE) {
			SysBeep(10);
			g_Voices.monitor_Microphone_For_Speech = SAFE_FALSE;
			SetItemMark(g_Shell.interface.program_Menus[AUDIO_MENU]
						, MENU_COMMAND_MONITOR_MICROPHONE
						, noMark);
		}

		if (Attach_Default_Inputs_To_All_Scene_Puppets(dream.play.default_Start_Audio_Point) != SAFE_TRUE) {
			/*  */
			/* Ignore Errors */
			/*  */
		}
			
	} /* if (g_Voices.monitor_Microphone_For_Speech == SAFE_TRUE) */

	function_Result = SAFE_TRUE;
	
	return(function_Result);
}
/*=**************************************************************************=*/


/*=**************************************************************************=*/
Safe_Boolean Monitor_Sampler(Monitor_Sampler_Rec	*params_For_Monitor_Sampler)
{
	Safe_Boolean			function_Result;
	Safe_Boolean			recording_Activated;
/* END VAR */
	
	
	/*  */
	/* Looks like this routine resets and closes NATIONAL_INSTRUMENTS_DA */
	/* and restarts it if it is already openned. */
	/*  */
	
	if (g_Voices.monitor_Sampler_For_Speech == SAFE_TRUE) {
	
		SetItemMark(g_Shell.interface.program_Menus[AUDIO_MENU]
					, MENU_COMMAND_MONITOR_SAMPLER
					, noMark);

		if (Stop_All_Sound_Monitoring_Of_Type(NATIONAL_INSTRUMENTS_DA) != SAFE_TRUE) {
			/*  */
			/* Ignore error */
			/*  */
		}

		g_Voices.monitor_Sampler_For_Speech = SAFE_FALSE;
	}
	else {
	
		g_Voices.monitor_Sampler_For_Speech = SAFE_TRUE;
		SetItemMark(g_Shell.interface.program_Menus[AUDIO_MENU]
					, MENU_COMMAND_MONITOR_SAMPLER
					, checkMark);
					
					
		recording_Activated = SAFE_TRUE;
		if (Get_Sampler_Voice(NATIONAL_INSTRUMENTS_DA) != SAFE_TRUE) {
			recording_Activated = SAFE_FALSE;
		}
		else {

			if (Start_Sampler_Monitoring(&recording_Activated) != SAFE_TRUE) {
			
				if (Stop_All_Sound_Monitoring_Of_Type(NATIONAL_INSTRUMENTS_DA) != SAFE_TRUE) {
					/*  */
					/* Ignore error */
					/*  */
				}
			
				recording_Activated = SAFE_FALSE;
			}
		}
		
		if (recording_Activated != SAFE_TRUE) {
			SysBeep(10);
			g_Voices.monitor_Sampler_For_Speech = SAFE_FALSE;
			SetItemMark(g_Shell.interface.program_Menus[AUDIO_MENU]
						, MENU_COMMAND_MONITOR_SAMPLER
						, noMark);
		}

		if (Attach_Default_Inputs_To_All_Scene_Puppets(dream.play.default_Start_Audio_Point) != SAFE_TRUE) {
			/*  */
			/* Ignore Errors */
			/*  */
		}
		
	} /* if (g_Voices.monitor_Sampler_For_Speech == SAFE_TRUE)  */
	
	function_Result = SAFE_TRUE;
	
	return(function_Result);
}
/*=**************************************************************************=*/


/*=**************************************************************************=*/
Safe_Boolean Monitor_Phonebridge(Monitor_Phonebridge_Rec	*params_For_Monitor_Phonebridge)
{
	Safe_Boolean			function_Result;
/* END VAR */
	

	if (g_Voices.Monitor_Phonebridge_For_Sound == SAFE_TRUE) {
		SetItemMark(g_Shell.interface.program_Menus[AUDIO_MENU]
					, MENU_COMMAND_MONITOR_PHONEBRIDGE
					, noMark);

		if (Stop_All_Sound_Monitoring_Of_Type(	TELEBRIDGE_SID_HYBRID_INPUT) != SAFE_TRUE) {
			/*  */
			/* Ignore error */
			/*  */
		}
		
		Shutdown_Phone_Bridge();
		
		g_Voices.Monitor_Phonebridge_For_Sound = SAFE_FALSE;
	}
	else {

		if (Initialize_PhoneBridge() == SAFE_TRUE) {
			g_Voices.Monitor_Phonebridge_For_Sound = SAFE_TRUE;
			SetItemMark(g_Shell.interface.program_Menus[AUDIO_MENU]
						, MENU_COMMAND_MONITOR_PHONEBRIDGE
						, checkMark);
	
			if (Attach_Default_Inputs_To_All_Scene_Puppets(dream.play.default_Start_Audio_Point) != SAFE_TRUE) {
				/*  */
				/* Ignore Errors */
				/*  */
			}
		}

	}
	function_Result = SAFE_TRUE;
	
	return(function_Result);
}
/*=**************************************************************************=*/


/*=**************************************************************************=*/
Safe_Boolean Monitor_Using_Keyboard(Monitor_Using_Keyboard_Rec	*params_For_Monitor_Using_Keyboard)
{
	Safe_Boolean			function_Result;
/* END VAR */
	

	if (g_Voices.monitor_Keyboard_As_Speech == SAFE_TRUE) {
		g_Voices.monitor_Keyboard_As_Speech = SAFE_FALSE;
		SetItemMark(g_Shell.interface.program_Menus[AUDIO_MENU]
					, MENU_COMMAND_MONITOR_MANUAL
					, noMark);
	}
	else {
		g_Voices.monitor_Keyboard_As_Speech = SAFE_TRUE;
		SetItemMark(g_Shell.interface.program_Menus[AUDIO_MENU]
					, MENU_COMMAND_MONITOR_MANUAL
					, checkMark);
	}
	function_Result = SAFE_TRUE;
	
	return(function_Result);
}
/*=**************************************************************************=*/


/*=**************************************************************************=*/
Safe_Boolean Monitor_Real_MIDI(Monitor_Real_MIDI_Rec	*params_For_Monitor_Real_MIDI)
{
	Safe_Boolean			function_Result;
	Safe_Boolean			result;
/* END VAR */
	
	
	function_Result = SAFE_TRUE;

	if (g_Voices.monitor_MIDI_For_Sound == SAFE_TRUE) {
		SetItemMark(g_Shell.interface.program_Menus[AUDIO_MENU]
					, MENU_COMMAND_MONITOR_REAL_MIDI
					, noMark);

		if (Stop_All_Sound_Monitoring_Of_Type(MIDI_INPUT) != SAFE_TRUE) {
			/*  */
			/* Ignore error */
			/*  */
		}
		
	/* ===
		Shutdown_Midi_Tapping();
	=== */
		
		g_Voices.monitor_MIDI_For_Sound = SAFE_FALSE;
	}
	else {

	/* ===
		result = Start_MIDI_Tapping();
	=== */
	
		result = SAFE_TRUE;
		
		if (result == SAFE_TRUE) {
			g_Voices.monitor_MIDI_For_Sound = SAFE_TRUE;
			SetItemMark(g_Shell.interface.program_Menus[AUDIO_MENU]
						, MENU_COMMAND_MONITOR_REAL_MIDI
						, checkMark);
	
			if (Attach_Default_Inputs_To_All_Scene_Puppets(dream.play.default_Start_Audio_Point) != SAFE_TRUE) {
				/*  */
				/* Ignore Errors */
				/*  */
			}
		}
		else {
			function_Result = SAFE_FALSE;
		}
	}
	
	

	/*  */
	/*  */
	/*  */
	/*  */
	/*  */
	/*  */
	/* HACK: Not implemented yet. */
	/*  */
	Display_Not_Implemented_Warning("\pMonitor_Real_MIDI");
	
	
	
		
	return(function_Result);
	
} /* Safe_Boolean Monitor_Real_MIDI ... */
/*=**************************************************************************=*/


/*=**************************************************************************=*/
Safe_Boolean Reset_Connection(Reset_Connection_Rec	*params_For_Reset_Connection)
{
	Safe_Boolean			function_Result;
/* END VAR */
	
	
	function_Result = SAFE_TRUE;
	/*  */
	/* HACK: Not implemented yet. */
	/*  */
	Display_Not_Implemented_Warning("\pReset_Connection");
	
	return(function_Result);
}
/*=**************************************************************************=*/


/*=**************************************************************************=*/
Safe_Boolean Disconnect_Self(Disconnect_Self_Rec	*params_For_Disconnect_Self)
{
	Safe_Boolean			function_Result;
/* END VAR */
	
	
	function_Result = SAFE_TRUE;
	/*  */
	/* HACK: Not implemented yet. */
	/*  */
	Display_Not_Implemented_Warning("\pDisconnect_Self");
	
	return(function_Result);
}
/*=**************************************************************************=*/


/*=**************************************************************************=*/
Safe_Boolean Show_Zones(Show_Zones_Rec	*params_For_Show_Zones)
{
	Safe_Boolean			function_Result;
	SignedByte			status_Globals_State;
	WindowPtr			current_Graphics_Window_Ptr;
/* END VAR */
	
	
				
	Select_Graphics_Window(SHOW_NETWORK_ZONES_WINDOW_INDEX);
	/*  */
	/* Lock down for no real reason except paranoia? */
	/*  */
	if (Begin_Use_Of_Handle((Handle) active_Window_List[SHOW_NETWORK_ZONES_WINDOW_INDEX], MOVE_HIGH_REQUIRED, &status_Globals_State) != SAFE_TRUE) {
		goto Error_Handler;
	}

	current_Graphics_Window_Ptr	= (**active_Window_List[SHOW_NETWORK_ZONES_WINDOW_INDEX]).status_Window;

	if (Start_Network_Zone_Management(current_Graphics_Window_Ptr) != SAFE_TRUE) {
		/*  */
		/* Ignore error in this version */
		/*  */
	}
	else {
	
	
	} /* if (Start_Network_Zone_Management() != SAFE_TRUE) */
	
	function_Result = SAFE_TRUE;
	goto Normal_Function_Exit;
	
Error_Handler:
	function_Result = SAFE_FALSE;
			
Normal_Function_Exit:
	return(function_Result);
}
/*=**************************************************************************=*/


/*=**************************************************************************=*/
Safe_Boolean Create_Scene(Create_Scene_Rec	*params_For_Create_Scene)
{
	Safe_Boolean			function_Result;
/* END VAR */
	
	function_Result = Create_New_Scene(params_For_Create_Scene);
	
	return(function_Result);
}
/*=**************************************************************************=*/


/*=**************************************************************************=*/
Safe_Boolean Open_Scene(Open_Scene_Rec	*params_For_Open_Scene)
{
	Safe_Boolean			function_Result;
/* END VAR */
	
	
	function_Result = Load_Scene_File_Immediate_Style(params_For_Open_Scene);
	
	return(function_Result);
}
/*=**************************************************************************=*/


/*=**************************************************************************=*/
Safe_Boolean Close_Scenes(Close_Scenes_Rec	*params_For_Close_Scenes)
{
	Boolean				already_Updated;
	Safe_Boolean			function_Result;
	CARDINAL_32		located_Scene;
	CARDINAL_32		index;
	Boolean				found_Location;
/* END VAR */
	
	
	function_Result = SAFE_TRUE;
	already_Updated = FALSE;
	
	/*  */
	/* In future dump memory if useful to do so (puppets, backgrounds, unused stuff only) */
	/*  */
	
	/*  */
	/* Select a remaining scene */
	/*  */
	if (dream.current_Scene == params_For_Close_Scenes->desired_Scene) {
		if (dream.play.total_Scenes > 1) {
		
			found_Location = FALSE;
			
	/* ===
			located_Scenes = 0;
			for (scene_To_Use = 0; ((scene_To_Use < k_MAX_ACTIVE_SCENES) && (located_Scenes < dream.play.total_Scenes)); scene_To_Use++) {
				if (dream.play.scene_Memory_In_Use[scene_To_Use] == MEMORY_INITIALIZED) {
	==== */
			
			for (index = 0; ((index < k_MAX_ACTIVE_SCENES) && (found_Location != TRUE)); index++) {
				if (dream.play.scene_Memory_In_Use[index] == MEMORY_INITIALIZED) {
				
					if (index != params_For_Close_Scenes->desired_Scene) {
						found_Location = TRUE;
						located_Scene = index;
					}			
				}
			}
		
			if (found_Location == TRUE) {
				dream.current_Scene = located_Scene;
				if (Redraw_Idle_Screen_Completely(FALSE, FALSE) != SAFE_TRUE) {
					function_Result = SAFE_FALSE;
					dream.current_Scene = 0; /* not good error but exotic */
				}
				already_Updated = TRUE;
			}
			else {
				dream.current_Scene = 0; /* not perfect */
			}

		}
		else {
			dream.current_Scene = 0; /* not perfect */
		}
	}
	
	
	if (Destroy_A_Scene(params_For_Close_Scenes->desired_Scene) != SAFE_TRUE) {
		function_Result = SAFE_FALSE;
	}
	else {
		/*  */
		/* If no more scenes then close up window */
		/*  */
		if (dream.play.total_Scenes == 0) {

		/*  */
		/*  */
		/*  */
		/*  */
		/* HACK HACK  : add soon */
		/*  */
		
		}
	}

	if (already_Updated != TRUE) {
		if (Redraw_Idle_Screen_Completely(FALSE, FALSE) != SAFE_TRUE) {
			/*  */
			/* ignore */
			/*  */
		}
	}
	
	if (Reset_Menu_Control_Selectability() != SAFE_TRUE) {
		/*  */
		/* Not normal to ever fail */
		/*  */
		function_Result = SAFE_FALSE;
	}

	return(function_Result);
}
/*=**************************************************************************=*/


/*=**************************************************************************=*/
Safe_Boolean Close_All_Scenes(Close_All_Scenes_Rec	*params_For_Close_All_Scenes)
{
	Close_Scenes_Rec	params_For_Close_Scenes;
	CARDINAL_32		scene_To_Ignore;
	CARDINAL_32		scene_Index;
	CARDINAL_32		scenes_Found;
	CARDINAL_32		total_To_Search;
	Safe_Boolean			function_Result;
/* END VAR */
	
	
	function_Result = SAFE_TRUE;
	scenes_Found = 0;
	total_To_Search = dream.play.total_Scenes;
	
	scene_To_Ignore = dream.current_Scene;
	
	for (scene_Index = 0; ((scene_Index < k_MAX_ACTIVE_SCENES) 
			&& (scenes_Found < total_To_Search)); scene_Index++) {
			
		if (dream.play.scene_Memory_In_Use[scene_Index] == MEMORY_INITIALIZED) {
			scenes_Found += 1;
			
			if (scene_Index != scene_To_Ignore) {

				params_For_Close_Scenes.desired_Scene =scene_Index;
				params_For_Close_Scenes.async_Task_Desired =SAFE_FALSE;
				
				if (Close_Scenes(&params_For_Close_Scenes) != SAFE_TRUE) {
					function_Result = SAFE_FALSE;
				}
			}
		}
	}					
	
	return(function_Result);
}
/*=**************************************************************************=*/


/*=**************************************************************************=*/
Safe_Boolean Save_Scene(Save_Scene_Rec	*params_For_Save_Scene)
{
	Safe_Boolean			function_Result;
/* END VAR */
	
	
	function_Result = Save_Scene_Sequence(params_For_Save_Scene);
	
	return(function_Result);
}
/*=**************************************************************************=*/


/*=**************************************************************************=*/
Safe_Boolean Save_Scene_As(Save_Scene_As_Rec	*params_For_Save_Scene_As)
{
	Safe_Boolean			function_Result;
	Save_Scene_Rec		params_For_Save_Scene;
/* END VAR */
	

	params_For_Save_Scene.desired_File = params_For_Save_Scene_As->desired_File;
	params_For_Save_Scene.async_Task_Desired = params_For_Save_Scene_As->async_Task_Desired;
	params_For_Save_Scene.desired_Scene = params_For_Save_Scene_As->desired_Scene;
	
	function_Result = Save_Scene_Sequence(&params_For_Save_Scene);

	return(function_Result);
}
/*=**************************************************************************=*/


/*=**************************************************************************=*/
Safe_Boolean Page_Setup(Page_Setup_Rec	*params_For_Page_Setup)
{
	Safe_Boolean			function_Result;
/* END VAR */
	
	
	function_Result = SAFE_TRUE;
	/*  */
	/* HACK: Not implemented yet. */
	/*  */
	Display_Not_Implemented_Warning("\pPage_Setup");
	
	return(function_Result);
}
/*=**************************************************************************=*/


/*=**************************************************************************=*/
Safe_Boolean Page_Print(Page_Print_Rec	*params_For_Page_Print)
{
	Safe_Boolean			function_Result;
/* END VAR */
	
	
	function_Result = SAFE_TRUE;
	/*  */
	/* HACK: Not implemented yet. */
	/*  */
	Display_Not_Implemented_Warning("\pPage_Print");
	
	return(function_Result);
}
/*=**************************************************************************=*/


/*=**************************************************************************=*/
Safe_Boolean Add_Item(Add_Item_Rec	*params_For_Add_Item)
{
	Safe_Boolean			function_Result;
	CARDINAL_32		scene_To_Use;
	Str255					name_String;
	CARDINAL_32		index;
	Boolean				still_Searching;
	CARDINAL_32		match_Count;
/* END VAR */
	
	
	function_Result = SAFE_FALSE;
	
	scene_To_Use = dream.current_Scene;
	
	name_String[0] = 0;

	match_Count = 0;
			
	if (g_Dreaming_Module_Globals_Initialized == SAFE_TRUE) {
		still_Searching = TRUE;
		for (index = 0; ((index < k_MAXIMUM_DATABASE_PUPPETS) 
										&& (match_Count < g_Puppets.total_Available)
										&& (still_Searching == TRUE)); index++) {
										
										
			if ((**g_Puppets.info.puppets.handle).puppets[index].active == SAFE_TRUE) {
				if (match_Count == params_For_Add_Item->desired_Item) {

					if ((**g_Puppets.info.puppets.handle).puppets[index].type_Of_Source == k_FROM_FILE) {
						still_Searching = FALSE;

						Copy_Pascal_String((**g_Puppets.info.puppets.handle).puppets[index].puppet_Name, name_String);

						if ((**g_Puppets.info.puppets.handle).puppets[index].active == SAFE_TRUE) {
						
							function_Result = Add_Item_Immediately_To_Scene(scene_To_Use
																				, &name_String[0]);
						}
					}
				}
				
				match_Count += 1;
			}
		}
	}	
	
	return(function_Result);
}
/*=**************************************************************************=*/


/*=**************************************************************************=*/
Safe_Boolean Remove_Item(Remove_Item_Rec	*params_For_Remove_Item)
{
	Safe_Boolean			function_Result;
/* END VAR */
	
		
	function_Result = Remove_Item_Immediately_From_Scene(params_For_Remove_Item->desired_Scene
															, params_For_Remove_Item->desired_Item);
	
	return(function_Result);
}
/*=**************************************************************************=*/


/*=**************************************************************************=*/
Safe_Boolean Print_Saved_Files(Print_Saved_Files_Rec	*params_For_Print_Saved_Files)
{
	Safe_Boolean			function_Result;
/* END VAR */
	
	
	function_Result = SAFE_TRUE;
	/*  */
	/* HACK: Not implemented yet. */
	/*  */
	Display_Not_Implemented_Warning("\pPrint_Saved_Files");
	
	return(function_Result);
}
/*=**************************************************************************=*/


/*=**************************************************************************=*/
Safe_Boolean Resposition_Item(Resposition_Item_Rec	*params_For_Resposition_Item)
{
	Safe_Boolean			function_Result;
/* END VAR */
	
	
	function_Result = SAFE_TRUE;
	/*  */
	/* HACK: Not implemented yet. */
	/*  */
	Display_Not_Implemented_Warning("\pResposition_Item");
	
	return(function_Result);
}
/*=**************************************************************************=*/


/*=**************************************************************************=*/
Safe_Boolean Play_Audio_File(Play_Audio_File_Rec	*params_For_Play_Audio_File)
{
	Safe_Boolean			function_Result;
	CARDINAL_32		scene_To_Use;
	CARDINAL_32		index;
	Boolean				still_Searching;
	FSSpec				*files_Spec_Ptr;
	Async_AIFF_Ptr		 file_Control_Ptr;
	CARDINAL_32		match_Count;
/* END VAR */
	
	
	function_Result = SAFE_FALSE;

/* ===
	params_For_Play_Audio_File->desired_Sound_Index
==== */

	match_Count = 0;
			
	if (g_Dreaming_Module_Globals_Initialized == SAFE_TRUE) {
		still_Searching = TRUE;
		for (index = 0; ((index < k_MAXIMUM_DATABASE_SOUND_OBJECTS) 
										&& (match_Count < g_Audio_Files.total_Available)
										&& (still_Searching == TRUE)); index++) {
										
			if ((**g_Audio_Files.info.sounds.handle).sound_Objects[index].active == SAFE_TRUE) {
				if (match_Count == params_For_Play_Audio_File->desired_Sound_Index) {

					if ((**g_Audio_Files.info.sounds.handle).sound_Objects[index].type_Of_Source == k_FROM_FILE) {
						still_Searching = FALSE;

						if ((**g_Audio_Files.info.sounds.handle).sound_Objects[index].active == SAFE_TRUE) {
							files_Spec_Ptr = &((**g_Audio_Files.info.sounds.handle).sound_Objects[params_For_Play_Audio_File->desired_Sound_Index].file_Info);
							/*  */
							/* Do not bother installing a tap nor keeping track of file progress. */
							/* For here on its just automatically handled. */
							/*  */
							function_Result = Play_AIFF_Audio_File_Asyncronously(files_Spec_Ptr, SAFE_FALSE, &file_Control_Ptr);
						}
					}
				}
				match_Count += 1;
			}
		}
	}	
	
	return(function_Result);
}
/*=**************************************************************************=*/


/*=**************************************************************************=*/
Safe_Boolean Swap_Voice_For_Audio_File(Swap_Voice_For_Audio_File_Rec	*params_For_Swap_Voice_For_Audio_File)
{
	/*  */
	/* This routine swaps voices for all items matching a specific voice to */
	/* temporarily use an AIFF voice until a sound file finishes playing or is */
	/* stopped. This routine will intelligently pre-enmpt and close */
	/* a sound file in use already if its main item is already busy */
	/* playing another sound file. The previous sound is not stopped */
	/* until the next is started. This prevents any loss of volume intensity. */
	/* The sound being played is being tapped using a very very sophisticated */
	/* tap on a component right above the file play double-back callbacks */
	/* a little further up in the "MoreDataRtn". The data may be */
	/* compressed with the primitive types of compression apple uses but */
	/* that is not a concern for the files expected to be used. */
	/* The only good way to tap a channel is to target all the components for */
	/* output and intercept after the mixer or target the mixer itself (illegal and impossible */
	/* without renaming the original mixer to anything besides 'mixr'. */
	/* another tap on a channel is planned soon for tapping speech manager */
	/* eith hyper-accurate phonetic matching. */
	/* It uses "select_Arbitrary_First_Item" flag to double up this routines feature set. */
	/*  */
	Safe_Boolean			function_Result;
	CARDINAL_32		scene_To_Use;
	CARDINAL_32		index;
	Boolean				still_Searching;
	FSSpec				*files_Spec_Ptr;
	Async_AIFF_Ptr		 file_Control_Ptr;
	CARDINAL_32		match_Count;
	CARDINAL_32		max_Items;
	CARDINAL_32		total_Items_Seen;

	CARDINAL_16 	puppet;
	INTEGER_16 		position;
	Stage_Item_Ptr	item_Ptr;
	Boolean			found;
	SIMPLE_LOOP	loop;
	INTEGER_16		items_Found;
	Scene_Ptr			scene;
	CARDINAL_32	total_Scenes;
	CARDINAL_32	total_To_Search;
	
	CARDINAL_32	original_Voice_Index;
	CARDINAL_32	new_Voice_Position;
	Voice_Hdl			new_Voice;
	Voice_Hdl			original_Voice;
	Safe_Boolean		previous_Voice_Needs_Save;
	Safe_Boolean		file_Is_Playing;
/* END VAR */
	
	
	function_Result = SAFE_FALSE;

	match_Count = 0;
			
	if (g_Dreaming_Module_Globals_Initialized == SAFE_TRUE) {
	
		/*  */
		/* If we are supposed to arbitrarily get a puppet then do so */
		/*  */
		
		found = FALSE;
		if (params_For_Swap_Voice_For_Audio_File->select_Arbitrary_First_Item == SAFE_TRUE) {
			if (dream.play.scene_Memory_In_Use[dream.current_Scene] == MEMORY_INITIALIZED) {
				scene = *(dream.play.scene[dream.current_Scene]);
				if ((*scene).total_Items_In_Scene > 0) {
					items_Found = 0;
					total_To_Search = (*scene).total_Items_In_Scene;
					
					for (loop = 0; (((loop < k_MAX_STAGE_ITEMS) && (found != TRUE) && (items_Found < total_To_Search))); loop++) {
						if ((*scene).item[loop].valid_Pointer == SAFE_TRUE) {
							items_Found += 1;
							
							item_Ptr = &(*scene).item[loop];
							/*  */
							/* Just affix to this item and assume it is supposed to be able to talk */
							/*  */
							found = TRUE;
							position = loop;
							
							
						}
					}
				}
			}
		}
		else {
			if (dream.play.scene_Memory_In_Use[params_For_Swap_Voice_For_Audio_File->desired_Scene] == MEMORY_INITIALIZED) {
				scene = *(dream.play.scene[params_For_Swap_Voice_For_Audio_File->desired_Scene]);
				if ((*scene).total_Items_In_Scene > 0) {
					loop = params_For_Swap_Voice_For_Audio_File->desired_Item;
					if ((*scene).item[loop].valid_Pointer == SAFE_TRUE) {	
						item_Ptr = &(*scene).item[loop];
						/*  */
						/* Just affix to this item and assume it is supposed to be able to talk */
						/*  */
						found = TRUE;
						position = loop;
					}
				}
			}
		}
	
	
		/*  */
		/* An item to redirect sound through has been found. */
		/*  */
		if (found == TRUE) {
		
			still_Searching = TRUE;
			for (index = 0; ((index < k_MAXIMUM_DATABASE_SOUND_OBJECTS) 
											&& (match_Count < g_Audio_Files.total_Available)
											&& (still_Searching == TRUE)); index++) {
											
				if ((**g_Audio_Files.info.sounds.handle).sound_Objects[index].active == SAFE_TRUE) {
				
					if (match_Count == params_For_Swap_Voice_For_Audio_File->desired_Sound_Index) {
	
						if ((**g_Audio_Files.info.sounds.handle).sound_Objects[index].type_Of_Source == k_FROM_FILE) {
							still_Searching = FALSE;
	
							if ((**g_Audio_Files.info.sounds.handle).sound_Objects[index].active == SAFE_TRUE) {
							
								files_Spec_Ptr = &((**g_Audio_Files.info.sounds.handle).sound_Objects[
															params_For_Swap_Voice_For_Audio_File->desired_Sound_Index].file_Info);
								/*  */
								/* Install a tap to keep track of file progress. */
								/* When file is finished the original voice will be restored, if any. */
								/*  */
								function_Result = Play_AIFF_Audio_File_Asyncronously(files_Spec_Ptr
																								, SAFE_TRUE
																								, &file_Control_Ptr);
																								
								if (function_Result == SAFE_TRUE) {
								
									/*  */
									/* If file started to play, then swap audio channel to puppet */
									/* RIGHT NOW, and kill other files that might be playing */
									/* through that puppet if any. Save the old voice if it is not a */
									/* audio file and it will be swapped back later when last file played is */
									/* finished. */
									/*  */
									previous_Voice_Needs_Save = SAFE_FALSE;
									if (item_Ptr->controllers_Attached == SAFE_TRUE) {
										if ((**item_Ptr->voice).initialized == SAFE_TRUE) {
											previous_Voice_Needs_Save = SAFE_TRUE;
										}
									}
									
									if (previous_Voice_Needs_Save == SAFE_TRUE) {
									
										if ((**item_Ptr->voice).audio_Source == LOCAL_AUDIO_FILE_PLAYING) {
						
											/*  */
											/* Immediately issue close and kill request to already playing sound. */
											/* This also disposes of its memory */
											/* We will swap to another sound file immediately. */
											/*  */
											if (g_AIFF_Structures_Opened > 0) {
												if ((**item_Ptr->voice).info.audio_File.initialized == SAFE_TRUE) {
													if ((**item_Ptr->voice).info.audio_File.tap_Possibly_Active == SAFE_TRUE) {
									
														if (Stop_Playing_AIFF_Asynchronously(&g_Playback[(**item_Ptr->voice).info.audio_File.playback_Index]) != SAFE_TRUE) {
															/*  */
															/* Ignore error, if any. Problems of this type sort themselves out properly. */
															/*  */
														}
													}
													(**item_Ptr->voice).info.audio_File.tap_Possibly_Active = SAFE_FALSE;
												}
											}
											
											/*  */
											/* Because we need to save an active voice, that means we are */
											/* replacing a voice. This means we need to identify all the */
											/* Items that share the same voice characteristics as the selected item */
											/* and simultaneously swap all of them to the old previous source. */
											/* This also closes the memory to the handle */
											/*  */
											original_Voice = item_Ptr->voice;
											if (Revert_All_Voices_In_Items_To_Paused_Original_Sources(original_Voice) != SAFE_TRUE) {
												/*  */
												/* Ignore error, if any in this version. */
												/*  */
											}
											
											/*  */
											/* Close and kill that closed voice. It is fully detatched now from all items. */
											/* REMEMBER, item_Ptr->voice now points to something old and unknown now. */
											/* But that little thing is about to be stored and be swapped */
											/* in just a moment, as well as all other items using this same source. */
											/*  */
											max_Items = g_Voices.total_Voices_Active;
											total_Items_Seen = 0;
											for (original_Voice_Index=0; ((original_Voice_Index < k_MAXIMUM_VOICE_MONITORS)
																			&& (total_Items_Seen < max_Items)) ; original_Voice_Index++) {
																			
												if (g_Voices.voice_Memory_Initialized[original_Voice_Index] == SAFE_TRUE) {
													total_Items_Seen += 1;
													if (*g_Voices.voice[original_Voice_Index] == *original_Voice) {
														if (Shutdown_Voice(original_Voice_Index, LOCAL_AUDIO_FILE_PLAYING) != SAFE_TRUE) {
															/*  */
															/* Ignore error, if any in this version. */
															/*  */
														}
													}
												}
											}
											
										} /* if ((**item_Ptr->voice).audio_Source == LOCAL_AUDIO_FILE_PLAYING) */
										
									} /* if (previous_Voice_Needs_Save == SAFE_TRUE) */
									
									/*  */
									/* The voice of the item may have just been restored to "no voice connected. */
									/* therefore we must reset the previous_Voice_Needs_Save flag */
									/* again based on new information. */
									/*  */
									previous_Voice_Needs_Save = SAFE_FALSE;
									if (item_Ptr->controllers_Attached == SAFE_TRUE) {
										if ((**item_Ptr->voice).initialized == SAFE_TRUE) {
											previous_Voice_Needs_Save = SAFE_TRUE;
										}
									}

									/*  */
									/* Now create a new voice input that will exist for the duration */
									/* of the sound file playback and be closed when audio voice  is swapped back */
									/* when file is noticed finished. It will hold the information */
									/* on the file we are playing. It may have already finished playing */
									/* it it was microscopic so test validity flags for it before trusting it.*/
									/*  */
									
									if (Create_Sound_Input_Voice(LOCAL_AUDIO_FILE_PLAYING
																	, &new_Voice_Position) == SAFE_TRUE) {
										new_Voice = g_Voices.voice[new_Voice_Position];
										
										file_Is_Playing = SAFE_FALSE;
										if (Find_Structure_Index_For_AIFF_Pointer(file_Control_Ptr, &(**new_Voice).info.audio_File.playback_Index) != SAFE_TRUE) {
											/*  */
											/* If not found, assume file just finished and was incredibly tiny. No need to track it */
											/*  */
											; /* NULL */
										}
										else {
											if (file_Control_Ptr->structure_Initialized == SAFE_TRUE) {
												if (file_Control_Ptr->playing_AIFF_Now == SAFE_TRUE) {
													file_Is_Playing = SAFE_TRUE;
													(**new_Voice).info.audio_File.tap_Possibly_Active = SAFE_TRUE;
												}
											}
										}										
										
										/*  */
										/* If "previous_Voice_Needs_Save" is set that means we need to add */
										/* the information regarding the new voice for the AIFF into */
										/* possibly more than one item in more than one scene. */
										/* Otherwise we jsut add it to this sole item. It is more common */
										/* to have to swap it for many items. */
										/*  */
										
										(**new_Voice).info.audio_File.previous_Open_Voice_Index_Valid = SAFE_FALSE;
										
										/*  */
										/* No need to track file if not playing. */
										/*  */
										if (file_Is_Playing == SAFE_TRUE) {
											if (previous_Voice_Needs_Save == SAFE_TRUE) {
												original_Voice = item_Ptr->voice;
												
												max_Items = g_Voices.total_Voices_Active;
												total_Items_Seen = 0;
												for (original_Voice_Index=0; ((original_Voice_Index < k_MAXIMUM_VOICE_MONITORS) 
																					&& (total_Items_Seen < max_Items)) ; original_Voice_Index++) {
																					
													if (g_Voices.voice_Memory_Initialized[original_Voice_Index] == SAFE_TRUE) {
														total_Items_Seen += 1;
													
														if (*g_Voices.voice[original_Voice_Index] == *original_Voice) {
															(**new_Voice).info.audio_File.previous_Open_Voice_Index = original_Voice_Index;
															(**new_Voice).info.audio_File.previous_Open_Voice_Index_Valid = SAFE_TRUE;
															
															/*  */
															/* Swap all items in all scenes that share this same voice */
															/* at this moment. */
															/*  */
															if (Swap_Or_Delete_An_Input_From_Any_Scene_Puppets(g_Voices.voice[original_Voice_Index]
																											, SAFE_TRUE
																											, new_Voice) != SAFE_TRUE) {
																
																/*  */
																/* Ignore Error. */
																/*  */
															}
														}
													}
												}
											} /* if (previous_Voice_Needs_Save == SAFE_TRUE) */
											else {
												/*  */
												/* This item was currently silent, so just add this new voice to just this one item*/
												/*  */
												item_Ptr->voice = new_Voice;
												item_Ptr->controllers_Attached = SAFE_TRUE;
											}
											
										} /* if (file_Is_Playing == SAFE_TRUE) */
																				
									}
									else {
										/*  */
										/* Probably low memory. Oh well. Give up trying to track with a voice. */
										/* Oddly enough, audio is already playing and emitting and will clean up */
										/* after itself. */
										/*  */
									}
																		
									/*  */
									/* Everything is finished! */
									/*  */
																		
								} /* if (function_Result == SAFE_TRUE) */
								
							} /* if ((**g_Audio_Files.info..sounds.handle).sound_Objects[index].active == SAFE_TRUE) */
							
						}
						
					} /* if (match_Count == params_For_Swap_Voice_For_Audio_File->desired_Sound_Index) */
					
					match_Count += 1;
					
				} /* if ((**g_Audio_Files.info.sounds.handle).sound_Objects[index].active == SAFE_TRUE) */
				
			}
			
		} /* if (found == TRUE) */
	}	
	
	return(function_Result);
	
} /* Safe_Boolean Swap_Voice_For_Audio_File... */
/*=**************************************************************************=*/


/*=**************************************************************************=*/
Safe_Boolean Change_Backdrop(Change_Backdrop_Rec	*params_For_Change_Backdrop)
{
	Safe_Boolean			function_Result;
	CARDINAL_32		scene_To_Use;
	Str255					name_String;
	CARDINAL_32		index;
	Boolean				still_Searching;
	CARDINAL_32		match_Count;
/* END VAR */
	
	
	function_Result = SAFE_FALSE;

	name_String[0] = 0;
/* ===
	params_For_Change_Backdrop->desired_Backdrop
	desired_File
==== */

	scene_To_Use = dream.current_Scene;

	match_Count = 0;
			
	if (g_Dreaming_Module_Globals_Initialized == SAFE_TRUE) {
		still_Searching = TRUE;
		for (index = 0; ((index < k_MAXIMUM_DATABASE_BACKDROPS) 
										&& (match_Count < g_Backdrops.total_Available)
										&& (still_Searching == TRUE)); index++) {
										
			if ((**g_Backdrops.info.backdrops.handle).backdrops[index].active == SAFE_TRUE) {
				if (match_Count == params_For_Change_Backdrop->desired_Backdrop) {

					if ((**g_Backdrops.info.backdrops.handle).backdrops[index].type_Of_Source == k_FROM_FILE) {
						still_Searching = FALSE;

						Copy_Pascal_String((**g_Backdrops.info.backdrops.handle).backdrops[index].backdrop_Name, name_String);
						if ((**g_Backdrops.info.backdrops.handle).backdrops[index].active == SAFE_TRUE) {
							function_Result = Add_Background_Immediately_To_Scene(scene_To_Use, &((**g_Backdrops.info.backdrops.handle).backdrops[params_For_Change_Backdrop->desired_Backdrop].backdrop_Name[0]));
						}
					}
				}
				match_Count += 1;
			}
		}
	}	
	
	return(function_Result);
}
/*=**************************************************************************=*/


/*=**************************************************************************=*/
Safe_Boolean Advance_New_Slide(Advance_Slide_Rec *params_For_Advance_Slide)
{
	Safe_Boolean			function_Result;
/* END VAR */
	
	function_Result = Advance_Slide(params_For_Advance_Slide->desired_Direction);

	return(function_Result);
}
/*=**************************************************************************=*/


/*=**************************************************************************=*/
Safe_Boolean Select_New_Slide(Select_Slide_Rec *params_For_Select_Slide)
{
	Safe_Boolean			function_Result;
/* END VAR */
	
	function_Result = Select_Slide(params_For_Select_Slide->desired_Slide);

	return(function_Result);
}
/*=**************************************************************************=*/


/*=**************************************************************************=*/
Safe_Boolean Hide_Slide_Window(void)
{
	Safe_Boolean			function_Result;
	Select_Slide_Rec 		params_For_Select_Slide;
/* END VAR */
	
	
	params_For_Select_Slide.desired_Slide = k_NO_WINDOW;
	function_Result = Select_Slide(params_For_Select_Slide.desired_Slide);

	return(function_Result);
}
/*=**************************************************************************=*/


/*=**************************************************************************=*/
Safe_Boolean Open_Slide(Open_Slide_Rec	*params_For_Open_Slide)
{
	Safe_Boolean			function_Result;
/* END VAR */
	
	function_Result = SAFE_TRUE;
/* ==
	function_Result = Load_Slide_File_Immediate_Style(params_For_Open_Slide);
=== */

	return(function_Result);
}
/*=**************************************************************************=*/


/*=**************************************************************************=*/
Safe_Boolean Close_Slides(Close_Slides_Rec	*params_For_Close_Slides)
{
	Safe_Boolean			function_Result;
	CARDINAL_32		located_Slide;
	CARDINAL_32		index;
	Boolean				found_Location;
/* END VAR */
	
	
	function_Result = SAFE_TRUE;
	
	/*  */
	/* In future dump memory if useful to do so (puppets, backgrounds, unused stuff only) */
	/*  */
	
	/*  */
	/* Select a remaining scene */
	/*  */
	
#if (7 == 11)

	if (dream.current_Slide == params_For_Close_Slides->desired_Slide) {
		if (dream.play.total_Slides > 1) {
		
			found_Location = FALSE;
			for (index = 0; ((index < k_MAX_ACTIVE_SCENES) && (found_Location != TRUE)); index++) {
				if (dream.play.scene_Memory_In_Use[index] == MEMORY_INITIALIZED) {
					if (index != params_For_Close_Slides->desired_Slide) {
						found_Location = TRUE;
						located_Slide = index;
					}			
				}
			}
		
			if (found_Location == TRUE) {
				dream.current_Slide = located_Slide;
				if (Redraw_Idle_Screen_Completely(FALSE,FALSE) != SAFE_TRUE) {
					function_Result = SAFE_FALSE;
					dream.current_Slide = 0; /* not good error but exotic */
				}
			}
			else {
				dream.current_Slide = 0; /* not perfect */
			}

		}
		else {
			dream.current_Slide = 0; /* not perfect */
		}
	}
	
	
	if (Destroy_A_Slide(params_For_Close_Slides->desired_Slide) != SAFE_TRUE) {
		function_Result = SAFE_FALSE;
	}
	else {
		/*  */
		/* If no more scenes then close up window */
		/*  */
		if (dream.play.total_Slides == 0) {

		/*  */
		/*  */
		/*  */
		/*  */
		/* HACK HACK  : add soon */
		/*  */
		
		}
	}
	
	if (Reset_Menu_Control_Selectability() != SAFE_TRUE) {
		/*  */
		/* Not normal to ever fail */
		/*  */
		function_Result = SAFE_FALSE;
	}
	
#endif /*if (7 == 11) */

	return(function_Result);
}
/*=**************************************************************************=*/


/*=**************************************************************************=*/
Safe_Boolean Close_All_Slides(Close_All_Slides_Rec	*params_For_Close_All_Slides)
{
	Safe_Boolean			function_Result;
/* END VAR */
	
	
	function_Result = SAFE_TRUE;
	/*  */
	/* HACK: Not implemented yet. */
	/*  */
	Display_Not_Implemented_Warning("\pClose_All_Slides");
	
	return(function_Result);
}
/*=**************************************************************************=*/


/*=**************************************************************************=*/
Safe_Boolean Change_Window_Size(Change_Window_Size_Rec	*params_For_Change_Window_Size)
{
	Safe_Boolean		function_Result;
	INTEGER_16		window_Index;
	Primary_Status_Hdl		status_Globals_Hdl;
/* END VAR */

	
	function_Result = SAFE_TRUE;
	
	window_Index= params_For_Change_Window_Size->window_Index;
	status_Globals_Hdl = active_Window_List[window_Index];
	
	if ((status_Globals_Hdl != nil)
		&& ((**status_Globals_Hdl).structure_Initialized == SAFE_TRUE)) {

		if (Change_Window_Size_BASE(params_For_Change_Window_Size) ==  SAFE_TRUE) {
			if ((**status_Globals_Hdl).window_Is_Hidden != SAFE_TRUE) {
				if (Redraw_General_Screen_Completely(FALSE) != SAFE_TRUE) {
				
				}
			}
		}

	}
	else {
		function_Result = SAFE_FALSE;
	}
	
	
	return (function_Result);
}
/*=**************************************************************************=*/


/*=**************************************************************************=*/
Safe_Boolean Connect_Remote(Connect_Remote_Rec	*params_For_Connect_Remote)
{
	Safe_Boolean			function_Result;
/* END VAR */
	
	
	function_Result = Setup_Remote_Control_Full();
	
	return(function_Result);
}
/*=**************************************************************************=*/


/*=**************************************************************************=*/
Safe_Boolean Disconnect_Remote(Disconnect_Remote_Rec	*params_For_Disconnect_Remote)
{
	Safe_Boolean			function_Result;
/* END VAR */
	
	
	function_Result = Destroy_Remote_Control_Full();
	
	return(function_Result);
}
/*=**************************************************************************=*/


/*=**************************************************************************=*/
Safe_Boolean Hide_Remote_Slide_Window(void)
{
	Safe_Boolean			function_Result;
	Select_Remote_Rec 	params_For_Select_Remote;
/* END VAR */
	
	
	params_For_Select_Remote.desired_Slide = k_NO_WINDOW;
	function_Result = Send_Remote_Message(k_SELECT_ITEM_AE, (Ptr)&params_For_Select_Remote);

	return(function_Result);
}
/*=**************************************************************************=*/


/*=**************************************************************************=*/
Safe_Boolean Select_Remote(Select_Remote_Rec	*params_For_Select_Remote)
{
	Safe_Boolean			function_Result;
	CARDINAL_32		item;
/* END VAR */
	
	
	function_Result = Send_Remote_Message(k_SELECT_ITEM_AE, (Ptr)params_For_Select_Remote);
	
	return(function_Result);
}
/*=**************************************************************************=*/


/*=**************************************************************************=*/
Safe_Boolean Select_Next_Remote(Select_Next_Remote_Rec	*params_For_Select_Next_Remote)
{
	Safe_Boolean			function_Result;
/* END VAR */
	
	function_Result = Send_Remote_Message(k_SELECT_NEXT_ITEM_AE, nil);
	
	return(function_Result);
}
/*=**************************************************************************=*/


/*=**************************************************************************=*/
Safe_Boolean Select_Previous_Remote(Select_Previous_Remote_Rec	*params_For_Select_Previous_Remote)
{
	Safe_Boolean			function_Result;
/* END VAR */
	
	
	function_Result = Send_Remote_Message(k_SELECT_PREVIOUS_ITEM_AE, nil);
	
	return(function_Result);
}
/*=**************************************************************************=*/


/*=**************************************************************************=*/
Safe_Boolean Rescan_Remote(Rescan_Remote_Rec	*params_For_Rescan_Remote)
{
	Safe_Boolean			function_Result;
/* END VAR */
	
	
	function_Result = SAFE_TRUE;
	/*  */
	/* HACK: Not implemented yet. */
	/*  */
	Display_Not_Implemented_Warning("\pShow_Status");
	
	return(function_Result);
}
/*=**************************************************************************=*/


/*=**************************************************************************=*/
Safe_Boolean Update_Remote_List(Update_Remote_List_Rec	*params_For_Update_Remote_List)
{
	Safe_Boolean			function_Result;
/* END VAR */
	
	
	function_Result = SAFE_TRUE;
	/*  */
	/* HACK: Not implemented yet. */
	/*  */
	Display_Not_Implemented_Warning("\pShow_Status");
	
	return(function_Result);
}
/*=**************************************************************************=*/


/*=**************************************************************************=*/
Safe_Boolean Kill_Remote(Kill_Remote_Rec	*params_For_Kill_Remote)
{
	Safe_Boolean			function_Result;
/* END VAR */
	
	
	function_Result = SAFE_TRUE;
	/*  */
	/* HACK: Not implemented yet. */
	/*  */
	Display_Not_Implemented_Warning("\pShow_Status");
	
	return(function_Result);
}
/*=**************************************************************************=*/


/*=**************************************************************************=*/
Safe_Boolean Select_Transition(Select_Transition_Rec	*params_For_Select_Transition)
{
	Safe_Boolean			function_Result;
/* END VAR */
	
	
	function_Result = SAFE_TRUE;
	dream.play.transition_Effect = params_For_Select_Transition->transition_Desired - 1;
		
	return(function_Result);
}
/*=**************************************************************************=*/


/*=**************************************************************************=*/
Safe_Boolean Select_First_Transition(Select_First_Transition_Rec	*params_For_Select_First_Transition)
{
	Safe_Boolean			function_Result;
/* END VAR */
	
	
	function_Result = SAFE_TRUE;
	dream.play.transition_Effect_For_Introductions = params_For_Select_First_Transition->transition_Desired - 1;
	
	return(function_Result);
}
/*=**************************************************************************=*/


/*=**************************************************************************=*/
Safe_Boolean Activate_Twitch(Activate_Twitch_Rec	*params_For_Activate_Twitch)
{
	Safe_Boolean			function_Result;
/* END VAR */
	
	
	function_Result = SAFE_TRUE;
	/*  */
	/* HACK: Not implemented yet. */
	/*  */
	Display_Not_Implemented_Warning("\pShow_Status");
	
	return(function_Result);
}
/*=**************************************************************************=*/


/*=**************************************************************************=*/
void Make_Scene_Default_Scene(void)
{

	if (dream.play.total_Scenes > 0) {
		dream.main_Active_Scene_Specified = SAFE_TRUE;
		dream.main_Active_Scene = dream.current_Scene;
	}

}
/*=**************************************************************************=*/


/*=**************************************************************************=*/
Safe_Boolean Record_Titled_AIFF_File(Record_Titled_AIFF_File_Rec	*params_For_Record_Titled_AIFF_File)
{
	Safe_Boolean			function_Result;
/* END VAR */
	
	
	function_Result = SAFE_TRUE;
	/*  */
	/* HACK: Not implemented yet. */
	/*  */
	Display_Not_Implemented_Warning("\pRecord_Titled_AIFF_File");
		
	return(function_Result);
}
/*=**************************************************************************=*/


/*=**************************************************************************=*/
Safe_Boolean Record_From_Any_To_Titled_AIFF(Record_From_Any_To_Titled_AIFF_Rec	*params_For_Record_From_Any_To_Titled_AIFF)
{
	Safe_Boolean			function_Result;
/* END VAR */
	
	
	function_Result = SAFE_TRUE;
	/*  */
	/* HACK: Not implemented yet. */
	/*  */
	Display_Not_Implemented_Warning("\pRecord_From_Any_To_Titled_AIFF");
		
	return(function_Result);
}
/*=**************************************************************************=*/


/*=**************************************************************************=*/
Safe_Boolean Record_From_Any_To_Untitled_AIFF(Record_From_Any_To_Untitled_AIFF_Rec	*params_For_Record_From_Any_To_Untitled_AIFF)
{
	Safe_Boolean			function_Result;
/* END VAR */
	
	
	function_Result = Start_Recording_From_First_Talking_Item_In_Current_Scene(SAFE_TRUE);
		
	return(function_Result);
}
/*=**************************************************************************=*/


/*=**************************************************************************=*/
Safe_Boolean Stop_Recording_Any_Open_AIFF_Files(Stop_Recording_Any_Open_AIFF_Files_Rec	*params_For_Stop_Recording_Any_Open_AIFF_Files)
{
	Safe_Boolean			function_Result;
/* END VAR */
	
	
	function_Result = Stop_Recording_To_Any_Open_Audio_Files();
		
	return(function_Result);
}
/*=**************************************************************************=*/


/*=**************************************************************************=*/
Safe_Boolean Play_Last_Untitled_AIFF(Play_Last_Untitled_AIFF_Rec	*params_For_Play_Last_Untitled_AIFF)
{
	FSSpec				*files_Spec_Ptr;
	Async_AIFF_Ptr		 file_Control_Ptr;
	CARDINAL_32		index;
	CARDINAL_32		match_Count;
	CARDINAL_32		audio_Index;
	Safe_Boolean			function_Result;
	Boolean				found_Something;
/* END VAR */
	
	
	function_Result = SAFE_FALSE;

	found_Something = FALSE;
	match_Count = 0;
	for (index = 0; ((index < k_MAXIMUM_DATABASE_SOUND_OBJECTS) 
									&& (match_Count < g_Audio_Files.total_Available)); index++) {
									
		if ((**g_Audio_Files.info.sounds.handle).sound_Objects[index].active == SAFE_TRUE) {
			match_Count += 1;
			if ((**g_Audio_Files.info.sounds.handle).sound_Objects[index].type_Of_Source == k_FROM_FILE) {
				/*  */
				/* This won't exit, but the last item in the list located will be the one we want. */
				/*  */
				found_Something = TRUE;
				audio_Index = index;
			}
		}
	}
	
	if (found_Something == TRUE) {
		/*  */
		/* No tap is installed to keep track of file progress. */
		/*  */
		files_Spec_Ptr = &((**g_Audio_Files.info.sounds.handle).sound_Objects[audio_Index].file_Info);
		file_Control_Ptr = nil;
		function_Result = Play_AIFF_Audio_File_Asyncronously(files_Spec_Ptr
																		, SAFE_FALSE
																		, &file_Control_Ptr);
	}
		
	return(function_Result);
}
/*=**************************************************************************=*/


/*=**************************************************************************=*/
Safe_Boolean Stop_Playback_Of_All_AIFFs(Stop_Playback_Of_All_AIFFs_Rec	*params_For_Stop_Playback_Of_All_AIFFs)
{
	Safe_Boolean			function_Result;
/* END VAR */
	
	
	function_Result = SAFE_TRUE;

	Stop_Playing_Any_AIFF_Asynchronously(SAFE_FALSE);
		
	return(function_Result);
}
/*=**************************************************************************=*/


/*=**************************************************************************=*/
Safe_Boolean Rescan_Files(Rescan_Files_Rec	*params_For_Rescan_Files)
{
	Safe_Boolean			function_Result;
/* END VAR */
	
	
	function_Result = SAFE_TRUE;
	/*  */
	/* HACK: Not implemented yet. */
	/*  */
	Display_Not_Implemented_Warning("\pRescan_Files");
		
	return(function_Result);
}
/*=**************************************************************************=*/


/*=**************************************************************************=*/
/* END OF FILE: Command_Executor.c */
