/* START OF FILE: Command_Menu.c */


/*=**************************************************************************=*/
/* [---BEGIN---] FILE HEADER COMMENTS */
/**
 ** SCREEN VIEW = [Adobe Syntax Black-10, Tabs = 4]
 ** PRINTER VIEW = [Adobe Syntax Black-10, Tabs = 4]
 ** =======================================================================
 ** Written by James Merkle
 ** Command_Menu.c - Menu Command dispatcher
 ** (C) Copyright 1994 James Merkle
 ** All rights reserved, except those wishing to "copy" moderate sections of code or files may do so, so long as
 ** such code is never used directly either in animation-related tools or multimedia tools that are  
 ** used in live performance or live broadcast or recorded mass market broadcast or film. Desktop usage not restricted.
 ** Conventional desktop game or entertainment not restricted.
 ** =======================================================================
 **
 **
 ** ============
 ** Revision History
 ** ============
 **
 **  1994.02.01		(James Merkle)
 **						Initial coding started
 **
 **
 **
 **
 ** =================
 ** OTHER INFORMATION
 ** =================
 **  
 **
 **/
/* [----END----] FILE HEADER COMMENTS */
/*=**************************************************************************=*/


/*=**************************************************************************=*/
/* [---BEGIN---] #include AREA */

#include "About.h"

#include "Error_Utilities.h"
#include "Memory_Utilities.h"
#include "Program_Globals.h"
#include "Shell_Assembly.h"
#include "String_Utilities.h"

#include "Graphics_Demos_Support.h"
#include "Graphics_Primitives.h"
#include "Interface_Utilities.h"

#include "Core_Globals.h"
#include "Core_Specific.h"
#include "Dreaming_Stuff.h"
#include "Core_Globals.h"
#include "Command_Redirector.h"
#include "Spin_O_Rama.h"

#include "Command_Executor.h"
#include "Graphics_Primitives_Support.h"

#include "Command_Menu.h"

/* [----END----] #include AREA */
/*=**************************************************************************=*/


/*=**************************************************************************=*/
/* [---BEGIN---] #define AREA */


/* [----END----] #define AREA */
/*=**************************************************************************=*/


/*=**************************************************************************=*/
/* [---BEGIN---] TYPE AREA */


/* [----END----] TYPE AREA */
/*=**************************************************************************=*/


/*=**************************************************************************=*/
/* [---BEGIN---] PROTOTYPE AREA */
															

/* [----END----] PROTOTYPE AREA */
/*=**************************************************************************=*/


/*=**************************************************************************=*/
/* [---BEGIN---] GLOBAL DEFINITION AREA */


/* [----END----] GLOBAL DEFINITION AREA */
/*=**************************************************************************=*/


/*=**************************************************************************=*/
/* [---BEGIN---] LOCAL STATIC DEFINITION AREA */

static CARDINAL_8	*unit_Name;

/* [----END----] LOCAL STATIC DEFINITION AREA */
/*=**************************************************************************=*/


/*=**************************************************************************=*/
Safe_Boolean Initialize_Command_Menu(void)
{
	CARDINAL_8			*static_Unit_Name = "\pCommand_Menu";
/* END VAR */

/*  */
/* Global persistant structures are created here */
/*  */

	unit_Name = static_Unit_Name;
	return(SAFE_TRUE);
}
/*=**************************************************************************=*/


/*=**************************************************************************=*/
Safe_Boolean Destroy_Command_Menu(void)
{
	Safe_Boolean		function_Result;
/* END VAR */

/*  */
/* Global persistant structures are deleted here */
/*  */

	function_Result = SAFE_TRUE;
	return(function_Result);
}
/*=**************************************************************************=*/


/*=**************************************************************************=*/
Safe_Boolean Perform_Menu_Command(INTEGER_32 mResult, Boolean *quit_Program)
{
	CARDINAL_8		*code_Name;
	INTEGER_32 		theMenu, theItem;
	WindowPtr		current_Graphics_Window_Ptr;
	SignedByte		status_Globals_State;
	Str255				daName;
	GrafPtr 			savePort;
	Safe_Boolean		function_Result;
/* END VAR */


	code_Name = "\pPerform_Menu_Command";
	function_Result = SAFE_FALSE;

	theItem = LoWord(mResult);
	theMenu = HiWord(mResult);
	
	switch (theMenu) {
	
	/*  */
	/*APPLE_MENU_ID*/
	/*  */
		case APPLE_MENU_ID:
			if (theItem == ABOUT_PROGRAM_ITEM) {
				Show_About_Program_Dialog();
			}
			else {
				GetItem(g_Shell.interface.program_Menus[APPLE_MENU] , theItem,daName);
				GetPort(&savePort);
				(void) OpenDeskAcc(daName);
				Set_View_Port(savePort);
			}
			break;
			
	/*  */
	/*FILE_MENU_ID*/
	/*  */
		case FILE_MENU_ID:
			switch (theItem) {
										
				case MENU_COMMAND_NEW_SCENE:
					Create_Scene_Using_Menu();
					break;
										
				case MENU_COMMAND_OPEN_SCENE:
					Open_Scene_Using_Menu();
					break;
										
				case MENU_COMMAND_CLOSE_SCENE:
					Close_Scenes_Using_Menu();
					break;
										
				case MENU_COMMAND_CLOSE_ALL_SCENE:
					Close_All_Scenes_Using_Menu();
					break;
										
				case MENU_COMMAND_SAVE_SCENE:
					Save_Scene_Using_Menu();
					break;
										
				case MENU_COMMAND_SAVE_SCENE_AS:
					Save_Scene_As_Using_Menu();
					break;
										
				case MENU_COMMAND_PAGE_SETUP:
					Page_Setup_Using_Menu();
					break;
										
				case MENU_COMMAND_PAGE_PRINT:
					Page_Print_Using_Menu();
					break;
										
				case MENU_COMMAND_PRINT_SAVED_FILES:
					Print_Saved_Files_Using_Menu();
					break;
										
				case MENU_COMMAND_ADD_ITEM:
					Add_Item_Using_Menu();
					break;
										
				case MENU_COMMAND_REMOVE_ITEM:
					Remove_Item_Using_Menu();
					break;
										
				case MENU_COMMAND_REPOSITION_ITEM:
					Resposition_Item_Using_Menu();
					break;
										
				case MENU_COMMAND_LOAD_SELECT_SCENE:
					Load_Or_Select_Scene_Using_Menu();
					break;
										
				case MENU_COMMAND_SELECT_SCENE:
					Select_Scene_Using_Menu();
					break;
										
				case MENU_COMMAND_SELECT_NEXT_SCENE:
					Select_Next_Scene_Using_Menu();
					break;
										
				case MENU_COMMAND_SELECT_PREVIOUS_SCENE:
					Select_Previous_Scene_Using_Menu();
					break;
										
				case MENU_COMMAND_CHANGE_BACKDROP:
					Change_Backdrop_Using_Menu();
					break;
										
				case MENU_COMMAND_RESCAN_FILES:
					Rescan_Files_Using_Menu();
					break;
										
				case MENU_COMMAND_QUIT:
					*quit_Program = TRUE;
					break;
					
				default:
					break;
				} /*  FILE_MENU_ID: */
			break;
			
	/*  */
	/*EDIT_MENU_ID*/
	/*  */
		case EDIT_MENU_ID:
			switch (theItem) {
			
				case MENU_COMMAND_UNDO:
					break;
			
				case MENU_COMMAND_CUT:
					break;
			
				case MENU_COMMAND_COPY:
					break;
			
				case MENU_COMMAND_PASTE:
					break;
			
				case MENU_COMMAND_CLEAR:
					break;
			
				case MENU_COMMAND_SELECT_ALL:
					break;
			
				case MENU_COMMAND_FIND:
					break;
			
				default:
					break;
				} /*  EDIT_MENU_ID: */
			break;
			
	/*  */
	/*BACKDROPS_MENU_ID*/
	/*  */
		case BACKDROPS_MENU_ID:
			switch (theItem) {
			
				case MENU_COMMAND_CLOSE_BACKDROP:
					Close_Backdrop_Using_Menu();
					break;
					
				case MENU_COMMAND_LOAD_BACKDROP:
					Load_Backdrop_Using_Menu();
					break;
					
				case MENU_COMMAND_SAVE_BACKDROP:
					Save_Backdrop_Using_Menu();
					break;
					
				case MENU_COMMAND_CONVERT_BACKDROP:
					Convert_Backdrop_Using_Menu();
					break;
					
				case MENU_COMMAND_EXPORT_BACKDROP:
					Export_Backdrop_Using_Menu();
					break;
					
				default:
					break;
				} /*  BACKDROPS_MENU: */
			break;
			
			
	/*  */
	/* ITEMS_MENU_ID*/
	/*  */
		case ITEMS_MENU_ID:
			switch (theItem) {
			
				case MENU_COMMAND_LOAD_PUPPET:
					Load_Puppet_Using_Menu();
					break;

				case MENU_COMMAND_CLOSE_PUPPET:
					Close_Puppet_Using_Menu();
					break;
					
				case MENU_COMMAND_SAVE_PUPPET:
					Save_Puppet_Using_Menu();
					break;
					
				case MENU_COMMAND_IMPORT_PUPPET:
					Import_Puppet_Using_Menu();
					break;
					
				case MENU_COMMAND_EXPORT_PUPPET:
					Export_Puppet_Using_Menu();
					break;
						
				case MENU_COMMAND_IMPORT_PROP:
					Import_Prop_Using_Menu();
					break;
						
				case MENU_COMMAND_EXPORT_PROP:
					Export_Prop_Using_Menu();
					break;
					
				case MENU_COMMAND_CREATE_PUPPET_VIA_SPINFILE:
					Create_Puppet_From_Spin_File_Using_Menu();
					break;
					
				case MENU_COMMAND_ADD_ANPOTHER_SPINFILE:
					Add_Another_Spin_File_To_Puppet_Using_Menu();
					break;

				case MENU_COMMAND_TIME_VOICE:
					Time_Animation_Using_Menu();
					break;
						
				case MENU_COMMAND_LOAD_SPIN_OBJECT:
					Load_Spin_Object_Using_Menu();
					break;

				case MENU_COMMAND_CLOSE_SPIN_OBJECT:
					Close_Spin_Object_Using_Menu();
					break;
					
				case MENU_COMMAND_SAVE_SPIN_OBJECT:
					Save_Spin_Object_Using_Menu();
					break;
					
				case MENU_COMMAND_IMPORT_SPIN_OBJECT:
					Convert_Spin_Object_Using_Menu();
					break;
					
				case MENU_COMMAND_EXPORT_SPIN_OBJECT:
					Export_Spin_Object_Using_Menu();
					break;
						
				case MENU_COMMAND_RECOMPRESS_SPIN_OBJECT:
					Recompress_Spin_Object_Using_Menu();
					break;
						
				case MENU_COMMAND_SET_LIFE_JIGGLE:
					Set_Life_Jiggle_Using_Menu();
					break;
					
				case MENU_COMMAND_SET_SPIN_WINDOW_SIZE:
				
						if (g_Set_Spin_Window_Max_Monitor_Size == SAFE_TRUE) {
							g_Set_Spin_Window_Max_Monitor_Size = SAFE_FALSE;
						}
						else {
							g_Set_Spin_Window_Max_Monitor_Size = SAFE_TRUE;
						}
											
						if (g_Set_Spin_Window_Max_Monitor_Size == SAFE_TRUE) {
							SetItemMark(g_Shell.interface.program_Menus[ITEMS_MENU], MENU_COMMAND_SET_SPIN_WINDOW_SIZE, checkMark);
						}
						else {
							SetItemMark(g_Shell.interface.program_Menus[ITEMS_MENU], MENU_COMMAND_SET_SPIN_WINDOW_SIZE, noMark);
						}
						
						if (Redraw_Spinning_Window_Completely(TRUE, FALSE) != SAFE_TRUE) {
							/*  */
							/* Ignore error at this stage, and let it tumble out normally */
							/*  */
						}
					break;
						
				default:
					break;
				} /*  ITEMS_MENU_ID: */
			break;
			
	/*  */
	/* NETWORK_MENU_ID*/
	/*  */
		case NETWORK_MENU_ID:
			switch (theItem) {
			
				case MENU_COMMAND_CONNECT:
					Connect_Remote_Using_Menu();
					break;
			
				case MENU_COMMAND_DISCONNECT:
					Disconnect_Remote_Using_Menu();
					break;
			
				case MENU_COMMAND_REMOTE_SELECT_SLIDE:
					Select_Remote_Using_Menu();
					break;
			
				case MENU_COMMAND_REMOTE_SELECT_NEXT_SLIDE:
					Select_Next_Remote_Using_Menu();
					break;
			
				case MENU_COMMAND_REMOTE_SELECT_PREVIOUS_SLIDE:
					Select_Previous_Remote_Using_Menu();
					break;
			
				case MENU_COMMAND_REMOTE_HIDE_WINDOW:
					Hide_Remote_Slide_Window();
					break;

				case MENU_COMMAND_REMOTE_RESCAN_FILES:
					Rescan_Remote_Using_Menu();
					break;
								
				case MENU_COMMAND_REMOTE_UPDATE_LIST:
					Update_Remote_List_Using_Menu();
					break;
			
				case MENU_COMMAND_REMOTE_QUIT:
					Kill_Remote_Using_Menu();
					break;
			
				case MENU_COMMAND_RESET_CONNECTION:
					Reset_Connection_Using_Menu();
					break;
				
				case MENU_COMMAND_DISCONNECT_SELF:
					Disconnect_Self_Using_Menu();
					break;
				
				case MENU_COMMAND_SHOW_ZONES:
					Show_Zones_Using_Menu();
					break;
					
				default:
					break;
				} /*  NETWORK_MENU: */
			break;
			
	/*  */
	/* WINDOWS_MENU_ID*/
	/*  */
		case WINDOWS_MENU_ID:
			switch (theItem) {
			
				case MENU_COMMAND_SET_SCENE_AS_DEFAULT:
					Make_Scene_Default_Scene();
					break;
					
				case MENU_COMMAND_SHOW_CONTROLS:
					Show_Controls_Using_Menu();
					break;
					
				case MENU_COMMAND_SHOW_STATUS:
					Show_Status_Using_Menu();
					break;
					
				case MENU_COMMAND_SHOW_ITEM_LIST:
					Show_Item_List_Using_Menu();
					break;
					
				case MENU_COMMAND_SETUP_CHROMAKEY_LIST:
					Setup_Chromakey_Video_Using_Menu();
					break;
					
				case MENU_COMMAND_SELECT_TRANSITION_EFFECT:
					Select_Transition_Using_Menu();
					break;
					
				case MENU_COMMAND_FIRST_TRANSITION_EFFECT:
					Select_First_Transition_Using_Menu();
					break;
					
				case MENU_COMMAND_REPOSITION_BROADCAST:
					Repostition_For_Broadcast_Using_Menu();
					break;
					
				case MENU_COMMAND_BROADCAST_POSITIONS:
					Setup_Broadcast_Using_Menu();
					break;
					
				case MENU_COMMAND_PLEASE_STAND_BY:
					Setup_Broadcast_Using_Controller();
					break;
					
				default:
					break;
				} /*  WINDOWS_MENU: */
			break;


	/*  */
	/* AUDIO_MENU_ID */
	/*  */
		case AUDIO_MENU_ID:
			switch (theItem) {
			
				case MENU_COMMAND_ASSOCIATE_TO_PUPPET:
					Associate_Voices_Using_Menu();
					break;
					
				case MENU_COMMAND_MONITOR_REMOTE:
					Monitor_Remote_Using_Menu();
					break;
					
				case MENU_COMMAND_MONITOR_MICROPHONE:
					Monitor_Microphone_Using_Menu();
					break;
					
				case MENU_COMMAND_MONITOR_SAMPLER:
					Monitor_Sampler_Using_Menu();
					break;
					
				case MENU_COMMAND_MONITOR_MANUAL:
					Monitor_Using_Keyboard_Using_Menu();
					break;
										
				case MENU_COMMAND_MONITOR_PHONEBRIDGE:
					Monitor_Phonebridge_Using_Menu();
					break;			
					
				case MENU_COMMAND_MONITOR_REAL_MIDI:
					Monitor_Real_MIDI_Using_Menu();
					break;			
					
				case MENU_COMMAND_RECORD_TITLED_AIFF_FILE:
					Record_Titled_AIFF_File_Using_Menu();
					break;			

				case MENU_COMMAND_RECORD_AIFF_FILE:
					Record_From_Any_To_Titled_AIFF_Using_Menu();
					break;			

				case MENU_COMMAND_RECORD_UNTITLED_AIFF_FILE:
					Record_From_Any_To_Untitled_AIFF_Using_Menu();
					break;			

				case MENU_COMMAND_STOP_RECORDING_AIFF_FILE:
					Stop_Recording_Any_Open_AIFF_Files_Using_Menu();
					break;			
					
				case MENU_COMMAND_PLAY_RECENT_AIFF_FILE:
					Play_Last_Untitled_AIFF_Using_Menu();
					break;			
					
				case MENU_COMMAND_STOP_PLAYING_AIFF_FILE:
					Stop_Playback_Of_All_AIFFs_Using_Menu();
					break;			

				case MENU_COMMAND_SWAP_VOICE_FOR_AIFF_FILE:
					Swap_Voice_For_Audio_File_Using_Menu();
					break;			

				case MENU_COMMAND_SWAP_ASSUMED_VOICE_FOR_AIFF_FILE:
					Swap_Assumed_Voice_For_Audio_File_Using_Menu();
					break;			

				case MENU_COMMAND_PLAY_AIFF_FILE:
					Play_Audio_File_Using_Menu();
					break;			

				default:
					break;
				} /*  AUDIO_MENU_ID: */
			break;

	/*  */
	/* SLIDE_MENU_ID */
	/*  */
		case SLIDE_MENU_ID:
			switch (theItem) {
										
										
				case MENU_COMMAND_ADD_SLIDE:
					Open_Slide_Using_Menu();
					break;
										
				case MENU_COMMAND_REMOVE_SLIDE:
					Close_Slides_Using_Menu();
					break;
										
				case MENU_COMMAND_REMOVE_ALL_SLIDES:
					Close_All_Slides_Using_Menu();
					break;
																				
				case MENU_COMMAND_SELECT_SLIDE:
					Select_Slide_Using_Menu();
					break;
																			
				case MENU_COMMAND_SELECT_NEXT_SLIDE:
					Advance_Slide_Using_Menu(1);
					break;
																				
				case MENU_COMMAND_SELECT_PREVIOUS_SLIDE:
					Advance_Slide_Using_Menu(0);
					break;
							
				case MENU_COMMAND_HIDE_WINDOW:
					Hide_Slide_Window();
					break;
							
				case MENU_COMMAND_SET_WINDOW_SIZE:
					Change_Window_Size_Using_Menu();
					break;
					
				case MENU_COMMAND_RESCAN_FILES:
					Rescan_Files_Using_Menu();
					break;
										
				case MENU_COMMAND_QUIT:
					*quit_Program = TRUE;
					break;
					
				default:
					break;
				} /*  SLIDE_MENU_ID: */
			break;



	/*  */
	/*DEBUG_MENU_ID*/
	/*  */
		case DEBUG_MENU_ID:
			switch (theItem) {
			
				case MENU_COMMAND_SET_DEBUG_1:
						if (g_Debugging_Options[0] == SAFE_TRUE) {
							g_Debugging_Options[0] = SAFE_FALSE;
							SetItemMark(g_Shell.interface.program_Menus[DEBUG_MENU]
										, MENU_COMMAND_SET_DEBUG_1
										, noMark);
						}
						else {
							g_Debugging_Options[0] = SAFE_TRUE;
							SetItemMark(g_Shell.interface.program_Menus[DEBUG_MENU]
										, MENU_COMMAND_SET_DEBUG_1
										, checkMark);
						}
					break;
					
				case MENU_COMMAND_SET_DEBUG_2:
						if (g_Debugging_Options[1] == SAFE_TRUE) {
							g_Debugging_Options[1] = SAFE_FALSE;
							SetItemMark(g_Shell.interface.program_Menus[DEBUG_MENU]
										, MENU_COMMAND_SET_DEBUG_2
										, noMark);
						}
						else {
							g_Debugging_Options[1] = SAFE_TRUE;
							SetItemMark(g_Shell.interface.program_Menus[DEBUG_MENU]
										, MENU_COMMAND_SET_DEBUG_2
										, checkMark);
						}
					break;
					
				case MENU_COMMAND_SET_DEBUG_3:
						if (g_Debugging_Options[2] == SAFE_TRUE) {
							g_Debugging_Options[2] = SAFE_FALSE;
							SetItemMark(g_Shell.interface.program_Menus[DEBUG_MENU]
										, MENU_COMMAND_SET_DEBUG_3
										, noMark);
						}
						else {
							g_Debugging_Options[2] = SAFE_TRUE;
							SetItemMark(g_Shell.interface.program_Menus[DEBUG_MENU]
										, MENU_COMMAND_SET_DEBUG_3
										, checkMark);
						}
					break;
					
				case MENU_COMMAND_SET_DEBUG_4:
						if (g_Debugging_Options[3] == SAFE_TRUE) {
							g_Debugging_Options[3] = SAFE_FALSE;
							SetItemMark(g_Shell.interface.program_Menus[DEBUG_MENU]
										, MENU_COMMAND_SET_DEBUG_4
										, noMark);
						}
						else {
							g_Debugging_Options[3] = SAFE_TRUE;
							SetItemMark(g_Shell.interface.program_Menus[DEBUG_MENU]
										, MENU_COMMAND_SET_DEBUG_4
										, checkMark);
						}
					break;
					
			
				default:
					break;
				} /*  DEBUG_MENU: */
			break;
			
	/*  */
	/* SPECIAL_MENU_ID*/
	/*  */
		case SPECIAL_MENU_ID:
			switch (theItem) {
			
				case MENU_COMMAND_SET_DRAWING_PREFERENCE:
						if (g_Shell.library.use_Home_Made_Graphics_Primitives == TRUE) {
							g_Shell.library.use_Home_Made_Graphics_Primitives = FALSE;
						}
						else {
							g_Shell.library.use_Home_Made_Graphics_Primitives = TRUE;
						}
					
						if (g_Shell.library.use_Home_Made_Graphics_Primitives == TRUE) {
							SetItemMark(g_Shell.interface.program_Menus[SPECIAL_MENU], MENU_COMMAND_SET_DRAWING_PREFERENCE, checkMark);
						}
						else {
							SetItemMark(g_Shell.interface.program_Menus[SPECIAL_MENU], MENU_COMMAND_SET_DRAWING_PREFERENCE, noMark);
						}
					break;
					

					
				case MENU_COMMAND_SET_WINDOW_ALIGNMENT:
						if (g_Shell.library.force_Window_Alignment_To_32_Bits == TRUE) {
							g_Shell.library.force_Window_Alignment_To_32_Bits = FALSE;
							SetItemMark(g_Shell.interface.program_Menus[SPECIAL_MENU], MENU_COMMAND_SET_WINDOW_ALIGNMENT, noMark);
						}
						else {
							g_Shell.library.force_Window_Alignment_To_32_Bits = TRUE;
							SetItemMark(g_Shell.interface.program_Menus[SPECIAL_MENU], MENU_COMMAND_SET_WINDOW_ALIGNMENT, checkMark);
						}
					break;
					
				case MENU_COMMAND_GENERAL:
				
						Select_Graphics_Window(GENERAL_INFO_WINDOW_INDEX);
						/*  */
						/* Lock down because structure content addresses are being passed */
						/*  */
						if (Begin_Use_Of_Handle((Handle) active_Window_List[GENERAL_INFO_WINDOW_INDEX], MOVE_HIGH_REQUIRED, &status_Globals_State) != SAFE_TRUE) {
							goto Error_Handler;
						}


						current_Graphics_Window_Ptr	= (**active_Window_List[GENERAL_INFO_WINDOW_INDEX]).status_Window;

						EraseRect(&(current_Graphics_Window_Ptr->portRect));
						Generate_Primary_Display();
						
						/*  */
						/* save window to a PICT for use during updates */
						/*  */
						Support_Update_By_Getting_Pict(&((**active_Window_List[GENERAL_INFO_WINDOW_INDEX]).update_Pict)
															,&((**active_Window_List[GENERAL_INFO_WINDOW_INDEX]).status_Window));
					
						(**active_Window_List[GENERAL_INFO_WINDOW_INDEX]).update_Pict_Permissable_To_Use = SAFE_TRUE;							
															
						if (End_Use_Of_Handle((Handle) active_Window_List[GENERAL_INFO_WINDOW_INDEX], status_Globals_State) != SAFE_TRUE) {
							goto Error_Handler;
						}
					break;
										
				case MENU_COMMAND_TEST_RECT:
						Select_Graphics_Window(TEST_RECT_WINDOW_INDEX);
						
						/*  */
						/* Lock down because structure content addresses are being passed */
						/*  */
						if (Begin_Use_Of_Handle((Handle) active_Window_List[TEST_RECT_WINDOW_INDEX], MOVE_HIGH_REQUIRED, &status_Globals_State) != SAFE_TRUE) {
							goto Error_Handler;
						}


						current_Graphics_Window_Ptr	= (**active_Window_List[TEST_RECT_WINDOW_INDEX]).status_Window;

						SetWTitle(current_Graphics_Window_Ptr, (StringPtr)WINDOW_TITLE_FOR_TEST_RECT);
						EraseRect(&(current_Graphics_Window_Ptr->portRect));
						if (Generate_Test_Rect_Display() != TRUE) { 
							goto Error_Handler;
						}
						
						/*  */
						/* save window to a PICT for use during updates */
						/*  */
						Support_Update_By_Getting_Pict(&((**active_Window_List[TEST_RECT_WINDOW_INDEX]).update_Pict)
															,&((**active_Window_List[TEST_RECT_WINDOW_INDEX]).status_Window));
														
						(**active_Window_List[TEST_RECT_WINDOW_INDEX]).update_Pict_Permissable_To_Use = SAFE_TRUE;							
						if (End_Use_Of_Handle((Handle) active_Window_List[TEST_RECT_WINDOW_INDEX], status_Globals_State) != SAFE_TRUE) {
							goto Error_Handler;
						}
					break;
										
				case MENU_COMMAND_TEST_CIRCLE:
						Select_Graphics_Window(TEST_CIRCLE_INDEX);
						
						/*  */
						/* Lock down because structure content addresses are being passed */
						/*  */
						if (Begin_Use_Of_Handle((Handle) active_Window_List[TEST_CIRCLE_INDEX], MOVE_HIGH_REQUIRED, &status_Globals_State) != SAFE_TRUE) {
							goto Error_Handler;
						}


						current_Graphics_Window_Ptr	= (**active_Window_List[TEST_CIRCLE_INDEX]).status_Window;

						SetWTitle(current_Graphics_Window_Ptr, (StringPtr)WINDOW_TITLE_FOR_TEST_CIRCLE);
						EraseRect(&(current_Graphics_Window_Ptr->portRect));
						if (Generate_Test_Circle_Display() != TRUE) { 
							goto Error_Handler;
						}
						
						/*  */
						/* save window to a PICT for use during updates */
						/*  */
						Support_Update_By_Getting_Pict(&((**active_Window_List[TEST_CIRCLE_INDEX]).update_Pict)
															,&((**active_Window_List[TEST_CIRCLE_INDEX]).status_Window));
															
						(**active_Window_List[TEST_CIRCLE_INDEX]).update_Pict_Permissable_To_Use = SAFE_TRUE;							
						if (End_Use_Of_Handle((Handle) active_Window_List[TEST_CIRCLE_INDEX], status_Globals_State) != SAFE_TRUE) {
							goto Error_Handler;
						}
					break;
					
										
				case MENU_COMMAND_TEST_FILLED_CIRCLE:
						Select_Graphics_Window(TEST_FILLED_CIRCLE_INDEX);
						
						/*  */
						/* Lock down because structure content addresses are being passed */
						/*  */
						if (Begin_Use_Of_Handle((Handle) active_Window_List[TEST_FILLED_CIRCLE_INDEX], MOVE_HIGH_REQUIRED, &status_Globals_State) != SAFE_TRUE) {
							goto Error_Handler;
						}


						current_Graphics_Window_Ptr	= (**active_Window_List[TEST_FILLED_CIRCLE_INDEX]).status_Window;

						SetWTitle(current_Graphics_Window_Ptr, (StringPtr)WINDOW_TITLE_FOR_TEST_FILLED_CIRCLE);
						EraseRect(&(current_Graphics_Window_Ptr->portRect));
						if (Generate_Test_Filled_Circle_Display() != TRUE) { 
							goto Error_Handler;
						}
						
						/*  */
						/* save window to a PICT for use during updates */
						/*  */
						Support_Update_By_Getting_Pict(&((**active_Window_List[TEST_FILLED_CIRCLE_INDEX]).update_Pict)
															,&((**active_Window_List[TEST_FILLED_CIRCLE_INDEX]).status_Window));
															
						(**active_Window_List[TEST_FILLED_CIRCLE_INDEX]).update_Pict_Permissable_To_Use = SAFE_TRUE;							
						if (End_Use_Of_Handle((Handle) active_Window_List[TEST_FILLED_CIRCLE_INDEX], status_Globals_State) != SAFE_TRUE) {
							goto Error_Handler;
						}
					break;
					
					
				case MENU_COMMAND_TEST_ELIPSES:
						Select_Graphics_Window(TEST_ELIPSE_WINDOW_INDEX);
						
						/*  */
						/* Lock down because structure content addresses are being passed */
						/*  */
						if (Begin_Use_Of_Handle((Handle) active_Window_List[TEST_ELIPSE_WINDOW_INDEX], MOVE_HIGH_REQUIRED, &status_Globals_State) != SAFE_TRUE) {
							goto Error_Handler;
						}

						current_Graphics_Window_Ptr	= (**active_Window_List[TEST_ELIPSE_WINDOW_INDEX]).status_Window;

						SetWTitle(current_Graphics_Window_Ptr, (StringPtr)WINDOW_TITLE_FOR_TEST_ELIPSES);
						EraseRect(&(current_Graphics_Window_Ptr->portRect));
						if (Generate_Test_Elipses_Display() != TRUE ) { 
							goto Error_Handler;
						}
						
						/*  */
						/* Save window to a PICT for use during updates */
						/*  */
						Support_Update_By_Getting_Pict(&((**active_Window_List[TEST_ELIPSE_WINDOW_INDEX]).update_Pict)
															,&((**active_Window_List[TEST_ELIPSE_WINDOW_INDEX]).status_Window));
															
						(**active_Window_List[TEST_ELIPSE_WINDOW_INDEX]).update_Pict_Permissable_To_Use = SAFE_TRUE;							
						if (End_Use_Of_Handle((Handle) active_Window_List[TEST_ELIPSE_WINDOW_INDEX], status_Globals_State) != SAFE_TRUE) {
							goto Error_Handler;
						}
					break;
					
					
				case MENU_COMMAND_TEST_FILLED_ELIPSES:
						Select_Graphics_Window(TEST_FILLED_ELIPSE_WINDOW_INDEX);
						
						/*  */
						/* Lock down because structure content addresses are being passed */
						/*  */
						if (Begin_Use_Of_Handle((Handle) active_Window_List[TEST_FILLED_ELIPSE_WINDOW_INDEX], MOVE_HIGH_REQUIRED, &status_Globals_State) != SAFE_TRUE) {
							goto Error_Handler;
						}

						current_Graphics_Window_Ptr	= (**active_Window_List[TEST_FILLED_ELIPSE_WINDOW_INDEX]).status_Window;

						SetWTitle(current_Graphics_Window_Ptr, (StringPtr)WINDOW_TITLE_FOR_TEST_FILLED_ELIPSES);
						EraseRect(&(current_Graphics_Window_Ptr->portRect));
						if (Generate_Test_Filled_Elipses_Display() != TRUE ) { 
							goto Error_Handler;
						}
						
						/*  */
						/* Save window to a PICT for use during updates */
						/*  */
						Support_Update_By_Getting_Pict(&((**active_Window_List[TEST_FILLED_ELIPSE_WINDOW_INDEX]).update_Pict)
															,&((**active_Window_List[TEST_FILLED_ELIPSE_WINDOW_INDEX]).status_Window));
															
						(**active_Window_List[TEST_FILLED_ELIPSE_WINDOW_INDEX]).update_Pict_Permissable_To_Use = SAFE_TRUE;							
						if (End_Use_Of_Handle((Handle) active_Window_List[TEST_FILLED_ELIPSE_WINDOW_INDEX], status_Globals_State) != SAFE_TRUE) {
							goto Error_Handler;
						}
					break;
					
				case MENU_COMMAND_TEST_LINE:
						Select_Graphics_Window(TEST_LINE_WINDOW_INDEX);
						
						/*  */
						/* Lock down because structure content addresses are being passed */
						/*  */
						if (Begin_Use_Of_Handle((Handle) active_Window_List[TEST_LINE_WINDOW_INDEX], MOVE_HIGH_REQUIRED, &status_Globals_State) != SAFE_TRUE) {
							goto Error_Handler;
						}

						current_Graphics_Window_Ptr	= (**active_Window_List[TEST_LINE_WINDOW_INDEX]).status_Window;

						SetWTitle(current_Graphics_Window_Ptr, (StringPtr)WINDOW_TITLE_FOR_TEST_LINE);
						EraseRect(&(current_Graphics_Window_Ptr->portRect));
						if (Generate_Test_Line_Display() != TRUE) {  
							goto Error_Handler;
						}
						
						/*  */
						/* Save window to a PICT for use during updates */
						/*  */
						Support_Update_By_Getting_Pict(&((**active_Window_List[TEST_LINE_WINDOW_INDEX]).update_Pict)
															,&((**active_Window_List[TEST_LINE_WINDOW_INDEX]).status_Window));
															
						(**active_Window_List[TEST_LINE_WINDOW_INDEX]).update_Pict_Permissable_To_Use = SAFE_TRUE;							
						if (End_Use_Of_Handle((Handle) active_Window_List[TEST_LINE_WINDOW_INDEX], status_Globals_State) != SAFE_TRUE) {
							goto Error_Handler;
						}
					break;
					
				case MENU_COMMAND_FRAMED_POLYGONS:
						Select_Graphics_Window(FRAMED_POLYGONS_WINDOW_INDEX);
						
						/*  */
						/* Lock down because structure content addresses are being passed */
						/*  */
						if (Begin_Use_Of_Handle((Handle) active_Window_List[FRAMED_POLYGONS_WINDOW_INDEX], MOVE_HIGH_REQUIRED, &status_Globals_State) != SAFE_TRUE) {
							goto Error_Handler;
						}

						current_Graphics_Window_Ptr	= (**active_Window_List[FRAMED_POLYGONS_WINDOW_INDEX]).status_Window;

						SetWTitle(current_Graphics_Window_Ptr, (StringPtr)WINDOW_TITLE_FOR_FRAMED_POLYGONS);
						EraseRect(&(current_Graphics_Window_Ptr->portRect));
						if (Generate_Demo_Clipped_Outline_Display() != TRUE) {
							goto Error_Handler;
						}
						
						/*  */
						/* Save window to a PICT for use during updates */
						/*  */
						Support_Update_By_Getting_Pict(&((**active_Window_List[FRAMED_POLYGONS_WINDOW_INDEX]).update_Pict)
															,&((**active_Window_List[FRAMED_POLYGONS_WINDOW_INDEX]).status_Window));
															
						(**active_Window_List[FRAMED_POLYGONS_WINDOW_INDEX]).update_Pict_Permissable_To_Use = SAFE_TRUE;							
						if (End_Use_Of_Handle((Handle) active_Window_List[FRAMED_POLYGONS_WINDOW_INDEX], status_Globals_State) != SAFE_TRUE) {
							goto Error_Handler;
						}
					break;
					
				case MENU_COMMAND_TIME_LINE_SPEED:
						Select_Graphics_Window(TIME_LINE_SPEED_WINDOW_INDEX);
						
						/*  */
						/* Lock down because structure content addresses are being passed */
						/*  */
						if (Begin_Use_Of_Handle((Handle) active_Window_List[TIME_LINE_SPEED_WINDOW_INDEX], MOVE_HIGH_REQUIRED, &status_Globals_State) != SAFE_TRUE) {
							goto Error_Handler;
						}

						current_Graphics_Window_Ptr	= (**active_Window_List[TIME_LINE_SPEED_WINDOW_INDEX]).status_Window;

						SetWTitle(current_Graphics_Window_Ptr, (StringPtr)WINDOW_TITLE_FOR_TIME_LINE_SPEED);
						EraseRect(&(current_Graphics_Window_Ptr->portRect));
						
						if (Generate_Test_Line_Pattern_Speed_Display() != TRUE) {
							goto Error_Handler;
						}
						
						/*  */
						/* Save window to a PICT for use during updates */
						/*  */
						Support_Update_By_Getting_Pict(&((**active_Window_List[TIME_LINE_SPEED_WINDOW_INDEX]).update_Pict)
															,&((**active_Window_List[TIME_LINE_SPEED_WINDOW_INDEX]).status_Window));
															
						(**active_Window_List[TIME_LINE_SPEED_WINDOW_INDEX]).update_Pict_Permissable_To_Use = SAFE_TRUE;							
						if (End_Use_Of_Handle((Handle) active_Window_List[TIME_LINE_SPEED_WINDOW_INDEX], status_Globals_State) != SAFE_TRUE) {
							goto Error_Handler;
						}
					break;
			
					
				case MENU_COMMAND_TEST_THREE_D:
						Select_Graphics_Window(TEST_THREE_D_WINDOW_INDEX);
						
						/*  */
						/* Lock down because structure content addresses are being passed */
						/*  */
						if (Begin_Use_Of_Handle((Handle) active_Window_List[TEST_THREE_D_WINDOW_INDEX], MOVE_HIGH_REQUIRED, &status_Globals_State) != SAFE_TRUE) {
							goto Error_Handler;
						}

						current_Graphics_Window_Ptr	= (**active_Window_List[TEST_THREE_D_WINDOW_INDEX]).status_Window;

						SetWTitle(current_Graphics_Window_Ptr, (StringPtr)WINDOW_TITLE_FOR_THREE_D);
						EraseRect(&(current_Graphics_Window_Ptr->portRect));
						
						if (Generate_Test_Three_D_Display() != TRUE) {
							goto Error_Handler;
						}
						
						/*  */
						/* Save window to a PICT for use during updates */
						/*  */
						Support_Update_By_Getting_Pict(&((**active_Window_List[TEST_THREE_D_WINDOW_INDEX]).update_Pict)
															,&((**active_Window_List[TEST_THREE_D_WINDOW_INDEX]).status_Window));
															
						(**active_Window_List[TEST_THREE_D_WINDOW_INDEX]).update_Pict_Permissable_To_Use = SAFE_TRUE;							
						if (End_Use_Of_Handle((Handle) active_Window_List[TEST_THREE_D_WINDOW_INDEX], status_Globals_State) != SAFE_TRUE) {
							goto Error_Handler;
						}
					break;
												
				default:
					break;
				} /*  SPECIAL_MENU_ID: */
			break;
	/*  */
	
	
	
		default:
			break;
		} /* switch (theMenu) */
		
	HiliteMenu(0);
	
	function_Result = SAFE_TRUE;
	goto Normal_Function_Exit;
	
Error_Handler:
	function_Result = SAFE_FALSE;
	
	SysBeep(20); 
	Assign_String(ns_Shell_Globals.error_Description_String, "\pRequested Command failed");
	Assign_String(ns_Shell_Globals.error_Note_String, "\pYou should quit soon.");
	
	Display_Error_Message(unit_Name, code_Name);
	Setup_Error_Handling(); 

Normal_Function_Exit:

	return(function_Result);
	

} /* static Safe_Boolean Perform_Menu_Command(INTEGER_32 mResult) */
/*=**************************************************************************=*/


/*=**************************************************************************=*/
/* END OF FILE: Command_Menu.c */
