/* [Core_Specific.c] START OF FILE */

/* [---BEGIN---] FILE HEADER COMMENTS */

/**
 ** SCREEN VIEW = Adobe Syntax Black-10, Tabs=6
 ** PRINTER VIEW = Adobe Syntax Black -10, Tabs=0.30 inch
 ** ========================================================================
 ** Core_Specific
 ** Written by Jim Merkle
 ** [Core_Specific.c] - support for Poor Man's Video Phone that invoke Mac toolbox or NS-Shell
 ** (C) Copyright 1991 by Jim Merkle
 ** All rights reserved, except those wishing to "copy" moderate sections of code or files may do so, so long as
 ** such code is never used directly either in animation-related tools or multimedia tools that are  
 ** used in live performance or live broadcast or recorded mass market broadcast or film. Desktop usage not restricted.
 ** Conventional desktop game or entertainment not restricted.
 ** ========================================================================
 **
 **
 ** =============
 ** Revision History
 ** =============
 **
 **  1991.11.25		(James Merkle)
 **						Initial coding started, routines transferred from 'Non_Specific_Support_Routines.h' and modified
 **
 **  1992.03.03		(James Merkle)
 **						Development shell completed, windows reposition themselves, all graphics seem stable and fine.
 **
 **  1994.02.01		(James Merkle)
 **						Most of this routine customized for use in Poor Man's Video Phone.
 **
 **  1994.06.04		(James Merkle)
 **						BUG FIX : Window creation not paired with window indices.
 **						Fixed by adding Get_Program_Window_Using_NS_Window_Index
 **
 **
 ** =================
 ** OTHER INFORMATION
 ** =================
 **
 **
 **
 **/

/* [----END----] FILE HEADER COMMENTS */
/*=**************************************************************************=*/



/*=**************************************************************************=*/
/* [---BEGIN---] FILE INCLUDE AREA */

#include <AppleEvents.h>
#include <Processes.h>
#include <PPCToolBox.h>
#include <EPPC.h>
#include <Notification.h>
#include <LoMem.h>

#include <stdio.h>
#include <limits.h>
#include <stdlib.h>
#include <string.h>
#include <profile.h>


/*#include "Automatic_Dialogs.h" */
#include "About.h"
#include "Cursor_Utilities.h"
#include "Command_Control.h"
#include "Spin_O_Rama.h"
#include "Spin_O_Rama_Cache.h"
#include "Error_Utilities.h"
#include "Memory_Utilities.h"
#include "Program_Globals.h"
#include "Shell_Assembly.h"
#include "Shell_Dialogs.h"
#include "Shell_Globals.h"
#include "String_Utilities.h"
#include "Window_Utilities.h"
#include "Window_Environment.h"
#include "Physical_File_Mapping.h"
#include "MOD_Utilities.h"

#include "Network_Zone_Control.h"
#include "Dreaming_Audio.h"
#include "Dreaming_Stuff.h"
#include "Dreaming_Color_Stuff.h"
#include "Dreaming_File_Loading.h"
#include "Dreaming_Backdrop_Files.h"
#include "Spin_O_Rama_Files.h"
#include "Interface_Utilities.h"

#include "Command_Menu.h"
#include "Command_Executor.h"
#include "Command_Dialogs.h"
#include "Command_Redirector.h"
#include "Object_Inventory.h"
#include "Off_Screen_Color.h"
#include "Dreaming_Popup_Dialogs.h"
#include "Dreaming_Scene_Files.h"
#include "Slide_Show_Routines.h"
#include "Dreaming_PhoneBridge.h"
#include "Simple_Ring_Cache.h"

#include "Remote_Control_Full.h"
#include "Remote_Control_Standard.h"
#include "Video_Backdrop_Background.h"

#include "Dreaming_Animation.h"
#include "Dreaming_Audio_Record.h"

#include "Sound_Record.h"
#include "Sound_Play.h"
#include "Sound_Tap_Channel.h"

#include "Core_Globals.h"
#include "Graphics_Demos_Support.h"
#include "Graphics_Primitives.h"
#include "Dreaming_Analog_Card.h"
#include "Graphics_Primitives_Support.h"

#include "Core_Specific.h" /* Specific_Support_Routines */



/* [----END----] FILE INCLUDE AREA */
/*=**************************************************************************=*/


/*=**************************************************************************=*/
/* [---BEGIN---] DEFINE AREA */


/*  */
/* When set to "1" a file will be created on disk showing timing */
/* from profiler for most routines. It uses microseconds (either VIA or */
/* more commonly the OS Microsecond trap). */
/*  */
#define k_USE_THINK_PROFILER		0


#define k_OFFSET_RESOURCE 	'Offs'														
#define k_WINDOW_OFFSET_RES_ID				128

/*  */
/* If this is set to more than 0, a great deal of burning of CPU */
/* speed can be sufferred from but at least it is a good way to */
/* limit the number of times the program spends processing */
/* any type of events */
/* 1 is reasonable, currently set to "2" */
/*  */
#define k_TICKS_TO_DELAY_EVENT_CHECK		2

/*  */
/* k_DEFAULT_YIELD_TIME has usually set to level "0" */
/* and k_DEACTIVATED_YIELD_TIME has been set to "1" */
/* But if talking from background is not imporatant it can be raised to */
/* "2" or "3" instead of "1" */
/*  */
#define k_DEFAULT_YIELD_TIME					0
#define k_DEACTIVATED_YIELD_TIME				1

#define	k_TINY_CURSOR							(1000L)

/* [----END----] DEFINE AREA */
/*=**************************************************************************=*/


/*=**************************************************************************=*/
/* [---BEGIN---] TYPE AREA */



/* [----END----] TYPE AREA */
/*=**************************************************************************=*/


/*=**************************************************************************=*/
/* [---BEGIN---] PROTOTYPES-------------------------- */
#pragma mark PROTOTYPE_AREA

/*------------ LOCAL PROTOTYPES -------------------*/

void Update_Spin_Window_If_Needed(void);

static void NS_Call_Back_For_Window_Update( Handle window_Globals, EventRecord	*event_Ptr);

void Update_Graphics_Window(Primary_Status_Hdl status_Globals_Hdl);

void Update_Graphics_Window_Using_Pict(Primary_Status_Hdl		status_Globals_Hdl);

Safe_Boolean Destroy_Status_Window_Structures(Primary_Status_Hdl	*status_Globals_Hdl);

void Close_Status_Window_Fully(Primary_Status_Hdl	status_Globals_Hdl);

Safe_Boolean Create_Status_Window_Structures(Primary_Status_Hdl	*status_Globals_Hdl);

Safe_Boolean Setup_Status_Window(Primary_Status_Hdl	status_Globals_Hdl, CARDINAL_8	type_Of_Window_To_Create);

void	Manage_Disk_Event(EventRecord *macEvent);

Safe_Boolean Close_Down_Program(void);

Safe_Boolean Perform_Periodic_Tasks(void);

Safe_Boolean Initialize_Starting_Menu_States(void);

void Refresh_Quick_Draw_Globals(void);

Safe_Boolean Get_Program_Window_Using_NS_Window_Index(CARDINAL_16 window_Index_For_NS_Shell
																	, CARDINAL_32 *window_Index);

void Alter_Cursor_If_Needed(void);

void Warn_About_Slow_Emulated_Animation(void);

Safe_Boolean Inform_Window_For_Pre_Drag_Actions(INTEGER_32		window_Index);

Safe_Boolean Inform_Window_For_Post_Drag_Actions(INTEGER_32		window_Index);

/* [----END----] PROTOTYPES-------------------------- */
/*=**************************************************************************=*/


/*=**************************************************************************=*/
/* [---BEGIN---] GLOBAL DEFINITION AREA */

	Primary_Status_Hdl		active_Window_List[TOTAL_DEFINED_WINDOW_TYPE];
	
	/*  */
	/* These are only for temporary setting of 
	/*  */
	GDHandle 				g_Initial_Scrap_Graphics_Device;
	GrafPtr	 				g_Initial_Scrap_Port;


	CARDINAL_32			g_Last_Tick_Time;

/* [----END----] GLOBAL DEFINITION AREA */
/*=**************************************************************************=*/


/*=**************************************************************************=*/
/* [---BEGIN---] LOCAL STATIC DEFINITION AREA */

static CARDINAL_8				*unit_Name = "\pCore_Specific";

/* [----END----] LOCAL STATIC DEFINITION AREA */
/*=**************************************************************************=*/


/*=**************************************************************************=*/
void Show_About_Program_Dialog(void)
{
/* END VAR */

	doAbout();
	
} /* Show_About_Program_Dialog */
/*=**************************************************************************=*/


/*=**************************************************************************=*/
void Alter_Cursor_If_Needed(void)
{
Rect		global_Rect;
Point		local_point;
/* END VAR */


	if (active_Window_List[TEST_DREAMING_WINDOW_INDEX] != nil) {
		if ((**active_Window_List[TEST_DREAMING_WINDOW_INDEX]).structure_Initialized == SAFE_TRUE) {
	
			GetMouse(&local_point);
			
			global_Rect = (**((WindowPeek)(**active_Window_List[TEST_DREAMING_WINDOW_INDEX]).status_Window)->contRgn).rgnBBox;
			
			LocalToGlobal(&local_point);
	
			if (PtInRect(local_point, &global_Rect) == TRUE) {
				if (last_Cursor_ID != k_TINY_CURSOR) {
					Change_Cursor(k_TINY_CURSOR);
				}
			}
			else {
				if (last_Cursor_ID != 0) {
					Arrow_Cursor();
				}
			}
		}
		else {
			if (last_Cursor_ID != 0) {
				Arrow_Cursor();
			}
		}
	}

} /* void Alter_Cursor_If_Needed(void) */
/*=**************************************************************************=*/


/*=**************************************************************************=*/
void Update_Spin_Window_If_Needed(void)
{
Rect		global_Rect;
Point		local_point;
Boolean	update_Called;
CARDINAL_32	current_Time;
CARDINAL_32	measured_Sound_Level;
/* END VAR */

	if (active_Window_List[MAIN_SPINNING_WINDOW_INDEX] != nil) {
		if ((**active_Window_List[MAIN_SPINNING_WINDOW_INDEX]).structure_Initialized == SAFE_TRUE) {
		
			update_Called = FALSE;
			GetMouse(&local_point);
			
			global_Rect = (**((WindowPeek)(**active_Window_List[MAIN_SPINNING_WINDOW_INDEX]).status_Window)->contRgn).rgnBBox;
			LocalToGlobal(&local_point);
	
			if (PtInRect(local_point, &global_Rect) == TRUE) {
			
				if ((local_point.h != g_Last_Spinning_Point_Position.h) 
					|| (local_point.v != g_Last_Spinning_Point_Position.v)) {
				
					if (g_Interactive_Motion_Disabled != SAFE_TRUE) {
						update_Called = TRUE;
						
						current_Time = (CARDINAL_32)TickCount();
						if (current_Time >= (g_Time_Since_Last_Sound_Check + k_TICK_DELAY_BETWEEN_SPIN_SOUND_CHECKS)) {
							if (Get_Current_Volume_For_Spinfile(&measured_Sound_Level) != SAFE_TRUE) {
								measured_Sound_Level = 0;
							}
					
							if (measured_Sound_Level != g_Last_Spinning_Sound_Level) {
								g_Last_Spinning_Sound_Level = measured_Sound_Level;
								g_Time_Since_Last_Sound_Change = current_Time;
							}
							g_Time_Since_Last_Sound_Check = current_Time;
						}
						
						if (Redraw_Spinning_Window_Completely(FALSE, TRUE) != SAFE_TRUE) {
												
						}
					}
				
					g_Last_Spinning_Point_Position = local_point;
				}
			}
			else {
				if (g_Interactive_Motion_Disabled == SAFE_TRUE){
					ShowCursor();
					g_Cursor_Probably_Still_Hidden = SAFE_FALSE;
				}
			}
			
			/*  */
			/* If No update forced from mouse movement alone, then check */
			/* if sound level changed and if so then force update */
			/*  */
			if (update_Called != TRUE) {
	
				current_Time = (CARDINAL_32)TickCount();
				if (current_Time >= g_Time_Since_Last_Sound_Check + k_TICK_DELAY_BETWEEN_SPIN_SOUND_CHECKS) {
					if (Get_Current_Volume_For_Spinfile(&measured_Sound_Level) != SAFE_TRUE) {
						measured_Sound_Level = 0;
					}
			
					if (measured_Sound_Level != g_Last_Spinning_Sound_Level) {
						g_Last_Spinning_Sound_Level = measured_Sound_Level;
						g_Time_Since_Last_Sound_Change = current_Time;
						
						if (Redraw_Spinning_Window_Completely(FALSE, TRUE) != SAFE_TRUE) {
		
						}
					}
					g_Time_Since_Last_Sound_Check = current_Time;
				}
			}
			
		} /* ... if ((**active_Window_List[MAIN_SPINNING_WINDOW_INDEX]).structure_Initialized */
	}

} /* void Alter_Cursor_If_Needed(void) */
/*=**************************************************************************=*/


/*=**************************************************************************=*/
void Update_Graphics_Window_Using_Pict(Primary_Status_Hdl		status_Globals_Hdl)
{
	GrafPtr			saved_Port;
	Str255				work_String;
	INTEGER_16		work_Short;
/* END VAR */


	GetPort(&saved_Port);
	
	Set_View_Port((**status_Globals_Hdl).status_Window);

/*==
	TextFont(geneva);
	TextSize(9);
	TextMode(srcCopy);
== */		

	/* Stuff not dependant on PICT stuff */

	if ((**status_Globals_Hdl).update_Pict_Permissable_To_Use == SAFE_TRUE) {
		Support_Update_By_Putting_Pict(&((**status_Globals_Hdl).update_Pict)
											, &((**status_Globals_Hdl).status_Window));
	}	
		
	Set_View_Port(saved_Port);
	
} /* Update_Graphics_Window_Using_Pict */
/*=**************************************************************************=*/


/*=**************************************************************************=*/
Safe_Boolean Get_Program_Window_Using_NS_Window_Index(CARDINAL_16 window_Index_For_NS_Shell
																	, CARDINAL_32 *window_Index)
{
	Safe_Boolean			function_Result;
	Primary_Status_Ptr	window_Info_Ptr;
	Boolean				still_Searching;
	INTEGER_16			current_Window_Index;
/* END VAR */


	/*  */
	/* This uses the more "system oriented" NS-Shell structure index */
	/* to find application windows index [a position dependent index] */
	/*  */
	*window_Index = 0;
	still_Searching = TRUE;
	for (current_Window_Index = 0
				; ((current_Window_Index < TOTAL_DEFINED_WINDOW_TYPE) && (still_Searching == TRUE))
				; current_Window_Index++ ) {
	
		if ((active_Window_List[current_Window_Index] != nil)
			&& ((**active_Window_List[current_Window_Index]).structure_Initialized == SAFE_TRUE)) {
		
			window_Info_Ptr = (*active_Window_List[current_Window_Index]);
			if (window_Info_Ptr->window_NS_Shell_ID_Valid == SAFE_TRUE) {
				if (window_Info_Ptr->window_NS_Shell_ID == window_Index_For_NS_Shell) {
				
					still_Searching =  FALSE;
					*window_Index = current_Window_Index;
				}
			}
			
		} /* if ((active_Window_List[current_Window_Index] != nil) ... */
		
	} /* for (current_Window_Index = 0 ... */
	
	if (still_Searching == SAFE_TRUE) {
		goto Error_Handler;
	}
	
	function_Result = SAFE_TRUE;
	goto Normal_Function_Exit;

Error_Handler:
	function_Result = SAFE_FALSE;

Normal_Function_Exit:
	return(function_Result);
	
} /* Safe_Boolean Get_Program_Window_Using_NS_Window_Index ... */
/*=**************************************************************************=*/


/*=**************************************************************************=*/
void Update_Graphics_Window(Primary_Status_Hdl status_Globals_Hdl)
{
	GrafPtr			saved_Port;
	Str255				work_String;
	INTEGER_16		work_Short;
	CARDINAL_8				*code_Name;
/* END VAR */


	code_Name = "\pUpdate_Graphics_Window";
	GetPort(&saved_Port);
	Set_View_Port((**status_Globals_Hdl).status_Window);

	switch ((**status_Globals_Hdl).type_Of_Window_For_Updates) {
	
		case ABOUT_PROGRAM_WINDOW_INDEX:
			break;
		
		case GENERAL_INFO_WINDOW_INDEX:
			if (Redraw_General_Screen_Completely(TRUE) != SAFE_TRUE) {
			}
			break;
		
/* =========
		case TEST_RECT_WINDOW_INDEX:
		case TEST_CIRCLE_INDEX:
		case TEST_FILLED_CIRCLE_INDEX:
		case TEST_ELIPSE_WINDOW_INDEX:
		case TEST_FILLED_ELIPSE_WINDOW_INDEX:
		case TEST_LINE_WINDOW_INDEX:
		case FRAMED_POLYGONS_WINDOW_INDEX:
		case TIME_LINE_SPEED_WINDOW_INDEX:
		case TEST_THREE_D_WINDOW_INDEX:
			break;
==== */

		case TEST_DREAMING_WINDOW_INDEX:
			if (Redraw_Idle_Screen_Completely(TRUE, FALSE) != SAFE_TRUE) {
				/*  */
				/* Ignore error at this stage, and let it tumble out normally */
				/*  */
			}
			break;
			
		case MAIN_CONTROLLER_WINDOW_INDEX:
			if (Redraw_Controller_Completely(TRUE) != SAFE_TRUE) {
				/*  */
				/* Ignore error at this stage, and let it tumble out normally */
				/*  */
			}
			break;
			
		case MAIN_SPINNING_WINDOW_INDEX:
			if (Redraw_Spinning_Window_Completely(TRUE, FALSE) != SAFE_TRUE) {
				/*  */
				/* Ignore error at this stage, and let it tumble out normally */
				/*  */
			}
			break;
						
	
		case SHOW_NETWORK_ZONES_WINDOW_INDEX:
			if (Update_Network_Zone_Window() != SAFE_TRUE) {
				/*  */
				/* Ignore error at this stage, and let it tumble out normally */
				/*  */
			}
			break;
	
		default:
		/*=--------------------------------------------------------------------------=*/
		/* !!!!!!! This is not expected, but just in case...*/
		/*=--------------------------------------------------------------------------=*/
		SysBeep(20);
		Display_Serious_Error_Message(unit_Name, code_Name, "\pError with Updating window", "\pBad Programming (new case)?");
	}
	
		
	Set_View_Port(saved_Port);
	
} /* Update_Graphics_Window... */
/*=**************************************************************************=*/


/*=**************************************************************************=*/
static void NS_Call_Back_For_Window_Update( Handle window_Globals
													,  EventRecord	*event_Ptr)
/*=--------------------------------------------------------------------------=*/
/* Caution: the calling convention must be : { void My_Routine_XXXX( Handle window_Globals,  EventRecord	*event_Ptr) } to work! */
/*=--------------------------------------------------------------------------=*/
{
	Primary_Status_Hdl	status_Globals_Hdl;
	GrafPtr				saved_Port;
	WindowPtr			event_Window;
	Point					click_Point;
	INTEGER_16			part_Code;
	ControlHandle		active_Control;
/* END VAR */

	status_Globals_Hdl = (Primary_Status_Hdl)window_Globals;
	
	switch(event_Ptr->what) {
		case updateEvt:
		
				event_Window = (WindowPtr)event_Ptr->message;
				if (event_Window == (**status_Globals_Hdl).status_Window) {
					GetPort(&saved_Port);
					Set_View_Port((**status_Globals_Hdl).status_Window);
					BeginUpdate((**status_Globals_Hdl).status_Window);
					
					if ((**status_Globals_Hdl).update_Events_Permissible == SAFE_TRUE) {
						if ((**status_Globals_Hdl).update_Pict_Permissable_To_Use == SAFE_TRUE) {
							Update_Graphics_Window_Using_Pict(status_Globals_Hdl);
						}
						else {
							Update_Graphics_Window(status_Globals_Hdl);
						}
					}
					
					EndUpdate((**status_Globals_Hdl).status_Window);
					Set_View_Port(saved_Port);
				}
				else {
					/*=--------------------------------------------------------------------------=*/
					/* !!!!!!! This is impossible, but just in case...*/
					/*=--------------------------------------------------------------------------=*/
					SysBeep(20);
					Display_Serious_Error_Message(unit_Name, "\pNS_Call_Back_For_Window_Update", "\pError with Updating window", "\pBad Programming?");
				}
			break;
						
		default:
			break;
			
	} /* switch(event.what) */
	
} /* static void NS_Call_Back_For_Window_Update() */
/*=**************************************************************************=*/


/*=**************************************************************************=*/
Safe_Boolean Create_Status_Window_Structures(Primary_Status_Hdl	*status_Globals_Hdl )
{
	Safe_Boolean				function_Result;
	CARDINAL_16			work_Cardinal_16;
	CARDINAL_32			preferred_Size;
	Memory_Allocation		status_Globals_Hdl_Memory_Initialized;
	Str255						message_String;
	CARDINAL_8				*code_Name;
/* END VAR */


	code_Name = "\pCreate_Status_Window";

	function_Result = SAFE_FALSE;
	

		preferred_Size = sizeof(Primary_Status_Record);
		status_Globals_Hdl_Memory_Initialized = MEMORY_NOT_INITIALIZED;
		*status_Globals_Hdl = nil;
		
		if (Create_Handle(&preferred_Size
							, (CARDINAL_32)(10240L)
							, ALLOW_PURGING
							, DISALLOW_PARTIAL_AMOUNT
							, (Handle *)status_Globals_Hdl
							, &status_Globals_Hdl_Memory_Initialized) != SAFE_TRUE) {
							
			Display_Low_Memory_Error_Message(unit_Name, code_Name, sizeof(Primary_Status_Record));
			function_Result = SAFE_FALSE;
			goto Normal_Function_Exit;
		}
		
		(***status_Globals_Hdl).structure_Initialized = SAFE_TRUE;
		
	function_Result = SAFE_TRUE;
		
Normal_Function_Exit:

	return(function_Result);

} /* Safe_Boolean Create_Status_Window() */
/*=**************************************************************************=*/


/*=**************************************************************************=*/
Safe_Boolean Setup_Status_Window(Primary_Status_Hdl	status_Globals_Hdl, CARDINAL_8	type_Of_Window_To_Create)
{
	Safe_Boolean		function_Result;
	CARDINAL_16	window_NS_Shell_ID;
	INTEGER_16		window_Resorce_ID;
	RgnHandle		tempRgn;
	INTEGER_16		os_Error;
	Rect				source_Rect;
	Rect				destination_Rect;
	Rect				temp_Rect;
	PicHandle			current_PICT;
	Safe_Boolean		status_Handle_Locked;
	Safe_Boolean		is_Dialog_Not_Window;
	Boolean			allow_PICT_Snapshots_For_Updating;
	Boolean			avoid_Color_Window_Type;
	SignedByte		status_Globals_State;
	Handle			offset_Handle;
	Safe_Boolean		allow_Drawing_To_Direct_Memory;

	Rect				global_Rect_Location;
	INTEGER_16		video_Monitor_Driver_Ref;
	INTEGER_8		video_Monitor_Slot;
	INTEGER_8		video_Monitor_ID;
	CARDINAL_32	video_Optional_Base_Address;
	PixMapHandle	video_Pix_Map_Handle;
	GDHandle			video_Graphics_Device_Handle;
	INTEGER_32		bits_Per_Pixel;
	Safe_Boolean		slot_Information_Valid;
	Safe_Boolean		other_Screens_Found_Not_8_Bit;
	Safe_Boolean		something_Found;
	Str255				error_String;
	CARDINAL_16	vertical_Offset;
/* END VAR */

	function_Result = SAFE_FALSE;
	status_Handle_Locked = SAFE_FALSE;

	if ((**status_Globals_Hdl).structure_Initialized != SAFE_TRUE) {
		function_Result = SAFE_FALSE;
		goto Normal_Function_Exit;
	}

	/*  */
	/*      Set Drag Rect   */
	/*  */
	tempRgn = GetGrayRgn();
	HLock ((Handle) tempRgn);
	if (MemError() != noErr) {
		function_Result = SAFE_FALSE;
		goto Normal_Function_Exit;
	}
	
	(**status_Globals_Hdl).drag_Rect = (**tempRgn).rgnBBox;
	
	HUnlock ((Handle) tempRgn);
	if (MemError() != noErr) {
		function_Result = SAFE_FALSE;
		goto Normal_Function_Exit;
	}

	(**status_Globals_Hdl).drag_Rect.top = GetMBarHeight();
	
	(**status_Globals_Hdl).user_Aborted = SAFE_FALSE;
	(**status_Globals_Hdl).user_Abort_Caution = SAFE_TRUE;
	(**status_Globals_Hdl).window_NS_Shell_ID_Valid = SAFE_FALSE;
	(**status_Globals_Hdl).draw_To_Screen_Direct_Plausable = SAFE_FALSE;
	(**status_Globals_Hdl).global_Device_Startup_Max_Rect_Valid = SAFE_FALSE;
	(**status_Globals_Hdl).update_Events_Permissible = SAFE_FALSE;

	
	/*  */
	/* Clear for no important reason, just tidiness */
	/*  */
	(**status_Globals_Hdl).geneva_Bold_9_Height = 0;
	(**status_Globals_Hdl).geneva_Plain_9_Height = 0;
	(**status_Globals_Hdl).geneva_Plain_10_Height = 0;

	is_Dialog_Not_Window = SAFE_FALSE;
	
	switch (type_Of_Window_To_Create) {
		
		case ABOUT_PROGRAM_WINDOW_INDEX:
				window_Resorce_ID = WORKBENCH_WINDOW_ID;
			break;
		
		case GENERAL_INFO_WINDOW_INDEX:
				window_Resorce_ID = GENERAL_INFO_WINDOW_ID;
			break;
		
		
		case TEST_RECT_WINDOW_INDEX:
				window_Resorce_ID = WORKBENCH_WINDOW_ID;
			break;
		
		case TEST_CIRCLE_INDEX:
				window_Resorce_ID = WORKBENCH_WINDOW_ID;
			break;
		
		case TEST_FILLED_CIRCLE_INDEX:
				window_Resorce_ID = WORKBENCH_WINDOW_ID;
			break;
		
		case TEST_ELIPSE_WINDOW_INDEX:
				window_Resorce_ID = WORKBENCH_WINDOW_ID;
			break;
		
		case TEST_FILLED_ELIPSE_WINDOW_INDEX:
				window_Resorce_ID = WORKBENCH_WINDOW_ID;
			break;
		
		case TEST_LINE_WINDOW_INDEX:
				window_Resorce_ID = WORKBENCH_WINDOW_ID;
			break;
		
		case FRAMED_POLYGONS_WINDOW_INDEX:
				window_Resorce_ID = WORKBENCH_WINDOW_ID;
			break;
		
		case TIME_LINE_SPEED_WINDOW_INDEX:
				window_Resorce_ID = WORKBENCH_WINDOW_ID;
			break;
		
		case TEST_THREE_D_WINDOW_INDEX:
				window_Resorce_ID = WORKBENCH_WINDOW_ID;
			break;
		
		case TEST_DREAMING_WINDOW_INDEX:
				window_Resorce_ID = DREAMING_WINDOW_ID;
			break;

		case MAIN_CONTROLLER_WINDOW_INDEX:
				window_Resorce_ID = MAIN_CONTROLLER_WINDOW_ID;
			break;
			
		case MAIN_SPINNING_WINDOW_INDEX:
				window_Resorce_ID = MAIN_SPINNING_WINDOW_ID;
			break;
					
		case SHOW_NETWORK_ZONES_WINDOW_INDEX:
				window_Resorce_ID = NETWORK_ZONE_WINDOW_ID;
				is_Dialog_Not_Window = SAFE_TRUE;
			break;
		
		default:
				window_Resorce_ID = WORKBENCH_WINDOW_ID;
			break;
	} /* switch (type_Of_Window_To_Create) */
	
	
	switch (type_Of_Window_To_Create) {

		case MAIN_CONTROLLER_WINDOW_INDEX:
			avoid_Color_Window_Type = TRUE;
		break;
	
		default:
			avoid_Color_Window_Type = FALSE;
	} /* switch (type_Of_Window_To_Create) */

	GetPort(&(**status_Globals_Hdl).saved_Port_Before_Data_Copy); /* MUST BE DESKTOP TO BE SAFE AT CONCLUSION!!!! */
	
	/*=--------------------------------------------------------------------------=*/
	/* 1993.04.29 JAM Added because of superstition of moving handle problem */
	/*=--------------------------------------------------------------------------=*/
	if (Begin_Use_Of_Handle((Handle)status_Globals_Hdl, MOVE_HIGH_NOT_REQUIRED, &status_Globals_State) != SAFE_TRUE) {
		goto Error_Handler;
	}
	status_Handle_Locked = SAFE_TRUE;

	if (ns_Shell_Globals.has_Color_QuickDraw_Available == TRUE) {
		if (is_Dialog_Not_Window == SAFE_TRUE) {
			(**status_Globals_Hdl).status_Window = GetNewDialog(window_Resorce_ID, nil, (WindowPtr)-1);
		}
		else {
			if (avoid_Color_Window_Type == TRUE) {
				(**status_Globals_Hdl).status_Window = GetNewWindow(window_Resorce_ID, nil, (WindowPtr)-1);
			}
			else {
				(**status_Globals_Hdl).status_Window = GetNewCWindow(window_Resorce_ID, nil, (WindowPtr)-1);
			}
		}
	}
	else {
	
		if (is_Dialog_Not_Window == SAFE_TRUE) {
			(**status_Globals_Hdl).status_Window = GetNewDialog(window_Resorce_ID, nil, (WindowPtr)-1);
		}
		else {
			(**status_Globals_Hdl).status_Window = GetNewWindow(window_Resorce_ID, nil, (WindowPtr)-1);
		}
	}
		
	os_Error = ResError();
	if (( (**status_Globals_Hdl).status_Window == nil) || (os_Error != noErr)){
		function_Result = SAFE_FALSE;
		goto Normal_Function_Exit;
	}
		
	(**status_Globals_Hdl).window_Is_Hidden = SAFE_FALSE;

/* ==
	Set_View_Port((**status_Globals_Hdl).status_Window);
	EraseRect(&( (*((WindowPeek)((**status_Globals_Hdl).status_Window))).port.portRect));
=== */
		
	/*=--------------------------------------------------------------------------=*/
	/* 1993.04.29 JAM moved up here for logic and performance */
	/*=--------------------------------------------------------------------------=*/
	if (Insert_Non_TCL_Window_Into_NS_Shell( (WindowPeek)((**status_Globals_Hdl).status_Window)
													,  &NS_Call_Back_For_Window_Update /* update_Routine */
													,  nil /* user_Shutdown_Routine */
													,  nil /* window_Event_Processor_Routine */
													, SAFE_TRUE /* updates_Desired */
													, SAFE_FALSE /* shutdown_Desired */
													, SAFE_FALSE /* window_Events_Desired*/
													, SAFE_FALSE /*also_TCL_Window_Compliant */
													, (Handle)status_Globals_Hdl
													, &window_NS_Shell_ID) != SAFE_TRUE) {
		function_Result = SAFE_FALSE; /* user will be notified outside */
		goto Normal_Function_Exit;
	}
	
	(**status_Globals_Hdl).window_NS_Shell_ID_Valid = SAFE_TRUE;
	(**status_Globals_Hdl).window_NS_Shell_ID = window_NS_Shell_ID;
	
	/*  */
	/* Position windows newly created */
	/*  */
	switch (type_Of_Window_To_Create) {
		case TEST_DREAMING_WINDOW_INDEX:
		case GENERAL_INFO_WINDOW_INDEX:
		case MAIN_SPINNING_WINDOW_INDEX:

				
		Find_A_8_Bit_Non_System_Screen(&global_Rect_Location
													, &video_Monitor_Driver_Ref
													, &video_Monitor_Slot
													, &video_Monitor_ID
													, &video_Optional_Base_Address
													, &video_Pix_Map_Handle
													, &video_Graphics_Device_Handle
													, &bits_Per_Pixel
													, &slot_Information_Valid
													, &other_Screens_Found_Not_8_Bit
													, &something_Found);
			

		offset_Handle	=	Get1Resource(k_OFFSET_RESOURCE, k_WINDOW_OFFSET_RES_ID);
		if (offset_Handle == nil) {
			vertical_Offset = 1;
		}
		else {
			vertical_Offset = *(CARDINAL_8 *)*offset_Handle;
			if (vertical_Offset > 200) {
				vertical_Offset = 200;
			}
			ReleaseResource(offset_Handle);
		}
		
		/*  */
		/* Set anytime resource is read. (two windows use this value) */
		/*  */
		dream.main_Window_Preferred_Top_Offset = vertical_Offset;
		
		if (type_Of_Window_To_Create == GENERAL_INFO_WINDOW_INDEX) {
			vertical_Offset += dream.main_Window_Preferred_Height;
			vertical_Offset += 2;
		}
										
		if (something_Found == SAFE_TRUE) {
		
			(**status_Globals_Hdl).global_Device_Startup_Max_Rect = global_Rect_Location;
			(**status_Globals_Hdl).global_Device_Startup_Max_Rect_Valid = SAFE_TRUE;
			(**status_Globals_Hdl).global_Device_Startup_Max_Rect_Contains_Menubar = SAFE_FALSE;
						
			g_Preferred_Monitor_Values_Valid = SAFE_TRUE;
			g_Preferred_Monitor_Depth = bits_Per_Pixel;
			g_Preferred_Monitor_Slots = video_Monitor_Slot;
			g_Preferred_Monitor_Video_Pix_Map_Handle = video_Pix_Map_Handle;
			g_Preferred_Monitor_Max_Rect = global_Rect_Location;
			g_Preferred_Monitor_Max_Rect_Contains_Menubar = SAFE_FALSE;
			g_Preferred_Monitor_Graphics_Device = video_Graphics_Device_Handle;


			Center_Window_High_As_Possible_Offscreen((**status_Globals_Hdl).status_Window, global_Rect_Location, (1 + vertical_Offset));
		}
		else {
		
			if (other_Screens_Found_Not_8_Bit == SAFE_TRUE) {
				/*  */
				/* Bitch about offscreen not 8 bit and system screen being used*/
				/*  */
				if (type_Of_Window_To_Create == TEST_DREAMING_WINDOW_INDEX) {
					Get_NS_String(k_DREAM_ERROR_STRINGS, k_ERROR_OFFSCREEN_NOT_8_BIT_SO_SYSTEM_USED, error_String);
					Perform_OK_Dialog(error_String);
				}	
			}
			
			Get_System_Screen_Data(&global_Rect_Location
										, &video_Monitor_Driver_Ref
										, &video_Monitor_Slot
										, &video_Monitor_ID
										, &video_Optional_Base_Address
										, &video_Pix_Map_Handle
										, &video_Graphics_Device_Handle
										, &bits_Per_Pixel
										, &slot_Information_Valid
										, &something_Found);
													
			if ((something_Found != SAFE_TRUE) || (bits_Per_Pixel != 8)){
				/*  */
				/* Bitch about not 8 bit */
				/*  */
				if (type_Of_Window_To_Create == TEST_DREAMING_WINDOW_INDEX) {
					Get_NS_String(k_DREAM_ERROR_STRINGS, k_ERROR_SYSTEM_SCREEN_IF_USED_MUST_BE_256_COLOR, error_String);
					Perform_OK_Dialog(error_String);
				}	
			}
																
			(**status_Globals_Hdl).global_Device_Startup_Max_Rect_Contains_Menubar = SAFE_TRUE;
			
			if (something_Found == SAFE_TRUE) {
				(**status_Globals_Hdl).global_Device_Startup_Max_Rect = global_Rect_Location;
				(**status_Globals_Hdl).global_Device_Startup_Max_Rect_Valid = SAFE_TRUE;
				
				g_Preferred_Monitor_Values_Valid = SAFE_TRUE;
				g_Preferred_Monitor_Depth = bits_Per_Pixel;
				g_Preferred_Monitor_Slots = video_Monitor_Slot;
				g_Preferred_Monitor_Video_Pix_Map_Handle = video_Pix_Map_Handle;
				g_Preferred_Monitor_Max_Rect = global_Rect_Location;
				g_Preferred_Monitor_Max_Rect_Contains_Menubar = SAFE_TRUE;
				g_Preferred_Monitor_Graphics_Device = video_Graphics_Device_Handle;

			}
			
			if (type_Of_Window_To_Create == MAIN_SPINNING_WINDOW_INDEX) {
				Center_Window_High_As_Possible((**status_Globals_Hdl).status_Window, 1 + vertical_Offset - k_SPIN_HEIGHT_READJUST);
			}
			else {
				Center_Window_High_As_Possible((**status_Globals_Hdl).status_Window, 1 + vertical_Offset);
			}

		}
		break;
		
		
		case MAIN_CONTROLLER_WINDOW_INDEX:
		default:
			Center_Window((**status_Globals_Hdl).status_Window);
	}
	
	/*  */
	/* Insert window placement via preferences read at start here */
	/*  */
		
	(**status_Globals_Hdl).update_Pict_Permissable_To_Use = SAFE_FALSE;
	(**status_Globals_Hdl).update_Pict = nil;
	(**status_Globals_Hdl).type_Of_Window_For_Updates = type_Of_Window_To_Create;
		
	Set_View_Device(g_Initial_Scrap_Graphics_Device);
	Set_View_Port(g_Initial_Scrap_Port);
	
	/*  */
	/* Set background erasing color for debugging */
	/*  */
	ForeColor(blackColor);
	BackColor(whiteColor);

	/*  */
	/* 1994.04.08 JAM Changed SelectWindow to avoid bug in */
	/* THINK debugger + Mac OS that ruins menubar */
	/* drawing in black if offscreen window with black background color */
	/* selected. */
	/* Assholes at Apple! One interesting thing is to not call "SelectWindow" */
	/* until "ShowWindow" is called */
	/* Problem may have to do with bad graphics state or pen modes */
	/*  */
	
	ShowWindow((**status_Globals_Hdl).status_Window);
	SelectWindow((**status_Globals_Hdl).status_Window);
	
	Set_View_Port((**status_Globals_Hdl).status_Window);
	EraseRect(&( (*((WindowPeek)((**status_Globals_Hdl).status_Window))).port.portRect));
	
	/*  */
	/* Very helpful for repositioning windows and such */
	/* Needs to be called after set port */
	/*  */
	temp_Rect = ((WindowPeek)(**status_Globals_Hdl).status_Window)->port.portRect;
	LocalToGlobal(&TOP_LEFT(temp_Rect));
	LocalToGlobal(&BOTTOM_RIGHT(temp_Rect));
	(**status_Globals_Hdl).window_Starting_Port_Rect_Position = temp_Rect;
	
	allow_Drawing_To_Direct_Memory = SAFE_FALSE;
	
	switch (type_Of_Window_To_Create) {
		case ABOUT_PROGRAM_WINDOW_INDEX:
		case GENERAL_INFO_WINDOW_INDEX:
		case MAIN_CONTROLLER_WINDOW_INDEX:
			break;
			
		case MAIN_SPINNING_WINDOW_INDEX:
			allow_Drawing_To_Direct_Memory = SAFE_TRUE;
			break;
			
		case TEST_RECT_WINDOW_INDEX:
		case TEST_CIRCLE_INDEX:
		case TEST_FILLED_CIRCLE_INDEX:
		case TEST_ELIPSE_WINDOW_INDEX:
		case TEST_FILLED_ELIPSE_WINDOW_INDEX:
		case TEST_LINE_WINDOW_INDEX:
		case FRAMED_POLYGONS_WINDOW_INDEX:
		case TIME_LINE_SPEED_WINDOW_INDEX:
		case TEST_THREE_D_WINDOW_INDEX:
			allow_Drawing_To_Direct_Memory = SAFE_TRUE;
			break;
			
		case TEST_DREAMING_WINDOW_INDEX:
			allow_Drawing_To_Direct_Memory = SAFE_TRUE;
			break;
			
		case SHOW_NETWORK_ZONES_WINDOW_INDEX:
			break;
		
		default:
			break;
	} /* swtich (type_Of_Window_To_Create) */

	(**status_Globals_Hdl).draw_To_Screen_Direct_Plausable = allow_Drawing_To_Direct_Memory;

	if ((**status_Globals_Hdl).draw_To_Screen_Direct_Plausable == SAFE_TRUE) {
		Reposition_Along_Byte_Coordinates((**status_Globals_Hdl).status_Window, TRUE);
		if (Reset_Current_Window_Structures(TRUE, FALSE) != TRUE) {
			/*  */
			/* HACK HACK : need to raise panic alert */
			/*  */
		};
	}
	else {
		Reposition_Along_Byte_Coordinates((**status_Globals_Hdl).status_Window, FALSE);
	}

	/*=--------------------------------------------------------------------------=*/
	/* "window_Handle_Index" keeps track of how many of a type are already open, vital */
	/*=--------------------------------------------------------------------------=*/
	(**status_Globals_Hdl).window_Handle_Index = type_Of_Window_To_Create;
	
	allow_PICT_Snapshots_For_Updating = TRUE;
	
	switch (type_Of_Window_To_Create) {
		
		case ABOUT_PROGRAM_WINDOW_INDEX:
			break;
		
		case GENERAL_INFO_WINDOW_INDEX:
			allow_PICT_Snapshots_For_Updating = FALSE;
			if (Redraw_General_Screen_Completely(FALSE) != SAFE_TRUE) {
			}
			break;
		
		case TEST_RECT_WINDOW_INDEX:
			break;
		
		case TEST_CIRCLE_INDEX:
			break;
		
		case TEST_FILLED_CIRCLE_INDEX:
			break;
		
		case TEST_ELIPSE_WINDOW_INDEX:
			break;
		
		case TEST_FILLED_ELIPSE_WINDOW_INDEX:
			break;
		
		case TEST_LINE_WINDOW_INDEX:
			break;
		
		case FRAMED_POLYGONS_WINDOW_INDEX:
			break;
		
		case TIME_LINE_SPEED_WINDOW_INDEX:
			break;
		
		case TEST_THREE_D_WINDOW_INDEX:
			break;
		
		case TEST_DREAMING_WINDOW_INDEX:
			allow_PICT_Snapshots_For_Updating = FALSE;
			break;

		case MAIN_CONTROLLER_WINDOW_INDEX:
			allow_PICT_Snapshots_For_Updating = FALSE;
			break;

		case MAIN_SPINNING_WINDOW_INDEX:
			allow_PICT_Snapshots_For_Updating = FALSE;
			break;
		
		case SHOW_NETWORK_ZONES_WINDOW_INDEX:
			allow_PICT_Snapshots_For_Updating = FALSE;
			break;
		
		default:
			break;
			
	} /* switch (type_Of_Window_To_Create) */

		
	/*  */
	/* save window to a PICT for use during updates */
	/*  */
	if (allow_PICT_Snapshots_For_Updating == TRUE) {
		Support_Update_By_Getting_Pict(&((**status_Globals_Hdl).update_Pict)
											,&((**status_Globals_Hdl).status_Window));
											
		(**status_Globals_Hdl).update_Pict_Permissable_To_Use = SAFE_TRUE;								
	}
	
	(**status_Globals_Hdl).update_Events_Permissible = SAFE_TRUE;


	function_Result = SAFE_TRUE;
	goto Normal_Function_Exit;
	
Error_Handler:	
	function_Result = SAFE_FALSE;
	
Normal_Function_Exit:

	/*  */
	/* 1993.04.29 JAM Added because of superstition of moving handle problem */
	/*  */
	if (status_Handle_Locked == SAFE_TRUE) {
		if (End_Use_Of_Handle((Handle)status_Globals_Hdl, status_Globals_State) != SAFE_TRUE) {
			/*  */
			/* Ignore error */
			/*  */
		}
	}

	return(function_Result);

} /* Safe_Boolean Setup_Status_Window() */
/*=**************************************************************************=*/
								
								
/*=**************************************************************************=*/
void Close_Status_Window_Fully(Primary_Status_Hdl	status_Globals_Hdl)
{
		Set_View_Port((**status_Globals_Hdl).saved_Port_Before_Data_Copy);
		if (Remove_Non_TCL_Window_From_NS_Shell( (WindowPeek)((**status_Globals_Hdl).status_Window)) != SAFE_TRUE) {
			/* user was notified */
		}

		DisposeWindow((**status_Globals_Hdl).status_Window);
		Update_Shell_Windows();
		
} /* Close_Status_Window_Fully(void) */
/*=**************************************************************************=*/


/*=**************************************************************************=*/
Safe_Boolean Destroy_Status_Window_Structures(Primary_Status_Hdl	*status_Globals_Hdl)
{
	Safe_Boolean			function_Result;
	CARDINAL_8			*code_Name;
	CARDINAL_32		*scrap_Cardinal_32_Ptr;
	Memory_Allocation	status_Globals_Hdl_Memory_Initialized;
/* END VAR */


	code_Name = "\pDestroy_Status_Window_Structures";

	function_Result = SAFE_FALSE;
	

	if ((***status_Globals_Hdl).update_Pict_Permissable_To_Use == SAFE_TRUE) {
		if ((***status_Globals_Hdl).update_Pict != nil) {
			KillPicture(((***status_Globals_Hdl).update_Pict));
				if (MemError() != noErr) {
				Display_Serious_Error_Message(unit_Name, code_Name, "\pError with KillPicture", "\pSystem Corrupted?");
				goto Error_Handler;
			}
		}
	}
	
	/*=--------------------------------------------------------------------------=*/
	/* "window_Handle_Index" keeps track of how many of a type are already open, vital to set to nil*/
	/*=--------------------------------------------------------------------------=*/
	scrap_Cardinal_32_Ptr = (CARDINAL_32 *)(&(active_Window_List[(***status_Globals_Hdl).window_Handle_Index]));

	status_Globals_Hdl_Memory_Initialized = MEMORY_INITIALIZED;
	if (Destroy_Handle((Handle *)status_Globals_Hdl, &status_Globals_Hdl_Memory_Initialized) != SAFE_TRUE) {
		Display_Serious_Error_Message(unit_Name, code_Name, "\pError with DisposHandle", "\pSystem Corrupted?");
		goto Error_Handler;
	}
	
	/*=--------------------------------------------------------------------------=*/
	/* It was theoretically possible that "status_Globals_Hdl" == "scrap_Cardinal_32_Ptr", */
	/* that is why "scrap_Cardinal_32_Ptr" was made first before setting to nil. */
	/*=--------------------------------------------------------------------------=*/
	*scrap_Cardinal_32_Ptr = 0;
	
	function_Result = SAFE_TRUE;
	goto Normal_Function_Exit;
	
	
Error_Handler:
	function_Result = SAFE_FALSE;

Normal_Function_Exit:

	return(function_Result);
		
} /* Safe_Boolean Destroy_Status_Window_Structures(Primary_Status_Hdl	*status_Globals_Hdl ... */
/*=**************************************************************************=*/


/*=**************************************************************************=*/
Safe_Boolean Change_Any_Window_Size_BASE(Change_Window_Size_Rec	*params_For_Change_Window_Size)
{
	Safe_Boolean		function_Result;
	GrafPtr			window_Manager_Port;
	WindowPtr		current_Graphics_Window_Ptr;
	GrafPtr			saved_Update_Port;
	Rect				valid_Screen_Area;
	INTEGER_16		window_Index;
	INTEGER_16		width;
	INTEGER_16		height;
	INTEGER_32		left_Global;
	INTEGER_32		top_Global;
	Rect						temp_Rect;
	Rect						desired_Screen_Area;
	Primary_Status_Hdl		status_Globals_Hdl;
	Safe_Boolean				window_Needs_Unhide;
/* END VAR */

	
	function_Result = SAFE_TRUE;
	window_Needs_Unhide = SAFE_FALSE;
	GetPort(&saved_Update_Port);

	window_Index= params_For_Change_Window_Size->window_Index;
	status_Globals_Hdl = active_Window_List[window_Index];
	
	if ((status_Globals_Hdl != nil)
		&& ((**status_Globals_Hdl).structure_Initialized == SAFE_TRUE)) {
		current_Graphics_Window_Ptr	= (**status_Globals_Hdl).status_Window;
	}
	else {
		goto Error_Handler;
	}

	GetWMgrPort(&window_Manager_Port);
/* ==
	valid_Screen_Area = window_Manager_Port->portRect;
	valid_Screen_Area.top = valid_Screen_Area.top + GetMBarHeight();
=== */

	valid_Screen_Area = (qd.screenBits).bounds;

	switch(params_For_Change_Window_Size->window_Type) {
		
		case k_WINDOW_SIZE_DIRECT_SETTING:
			width = params_For_Change_Window_Size->new_Rect.right - params_For_Change_Window_Size->new_Rect.left;
			height = params_For_Change_Window_Size->new_Rect.bottom - params_For_Change_Window_Size->new_Rect.top;
		break;
		
		case k_WINDOW_SIZE_640_390:
			width = 640;
			height = 390;
		break;
		
		case k_WINDOW_SIZE_638_455:
			width = 638;
			height = 455;
		break;
		
		case k_WINDOW_SIZE_640_480:
			width = 640;
			height = 480;
		break;
		
		case k_WINDOW_SIZE_960_720:
			width = 960;
			height = 720;
		break;
		
		case k_WINDOW_SIZE_FILLSCREEN:
			width = 640;
			height = 390;
		break;
		
		case k_WINDOW_SIZE_DEFAULT:
		default:
			/*  */
			/* Sizes will be reset properly for "k_WINDOW_SIZE_DEFAULT" soon */
			/*  */
			width = 640;
			height = 70;
	}
	
	if (params_For_Change_Window_Size->window_Type == k_WINDOW_SIZE_DEFAULT) {
		temp_Rect = (**status_Globals_Hdl).window_Starting_Port_Rect_Position;
		width = temp_Rect.right - temp_Rect.left;
		height = temp_Rect.bottom - temp_Rect.top;
		
		/*  */
		/*  */
		/*  */
		/*  */
		/*  */
		/* HACK HACK HACK : Need to find out largest legal workspace. */
		/* what command? what function? How? */
		/* For now, negate usefulness of valid_Screen_Area (our sanity checker ) */
		/*  */
			valid_Screen_Area = temp_Rect;
		
		/*  */
		/*  */
		/*  */
		
		if ((valid_Screen_Area.left <= temp_Rect.right) && (valid_Screen_Area.right >= temp_Rect.left)) {
			/*  */
			/* Good, one side or other in bounds so far */
			/*  */
			if ((valid_Screen_Area.top <= temp_Rect.bottom) && (valid_Screen_Area.bottom >= temp_Rect.top)) {
				/*  */
				/* Good, window top portion in bounds (though maybe not on a monitor */
				/*  */
				
				ForeColor(blackColor);
				BackColor(blackColor);
				EraseRect (&(((**status_Globals_Hdl).status_Window)->portRect));
				ForeColor(blackColor);
				BackColor(whiteColor);
				
				if ((**status_Globals_Hdl).window_Is_Hidden != SAFE_TRUE) {
					ShowHide((**status_Globals_Hdl).status_Window, FALSE);
					window_Needs_Unhide = SAFE_TRUE;
				}
								
				(**status_Globals_Hdl).update_Events_Permissible = SAFE_FALSE;
				MoveWindow((**status_Globals_Hdl).status_Window, temp_Rect.left, temp_Rect.top, FALSE);
				(**status_Globals_Hdl).update_Events_Permissible = SAFE_TRUE;
			}
		}
		
		if ((**status_Globals_Hdl).window_Is_Hidden != SAFE_TRUE) {
			SizeWindow(current_Graphics_Window_Ptr,width,height,TRUE);
		}
		else {
			SizeWindow(current_Graphics_Window_Ptr,width,height,FALSE);
		}
	}
	else {
		/*  */
		/* Center to center of "startup" screen if possible, else center left right */
		/*  */
		if ((**status_Globals_Hdl).global_Device_Startup_Max_Rect_Valid == SAFE_TRUE) {
		
			desired_Screen_Area = (**status_Globals_Hdl).global_Device_Startup_Max_Rect;
			
		/*  */
		/*  */
		/*  */
		/*  */
		/*  */
		/* HACK HACK HACK : Need to find out largest legal workspace. */
		/* what command? what function? How? */
		/* For now, negate usefulness of valid_Screen_Area (our sanity checker ) */
		/*  */
			valid_Screen_Area = desired_Screen_Area;
		
		/*  */
		/*  */
		/*  */

			
			if ((desired_Screen_Area.left >= valid_Screen_Area.left)
				&& (desired_Screen_Area.top >= valid_Screen_Area.top)
				&& (desired_Screen_Area.right <= valid_Screen_Area.right)
				&& (desired_Screen_Area.bottom <= valid_Screen_Area.bottom)) {
				/*  */
				/* Nothing odd has happened. Continue */
				/*  */
				left_Global = desired_Screen_Area.left + ((desired_Screen_Area.right - desired_Screen_Area.left) / 2) - (width / 2);
			/* ===
				if (left_Global < desired_Screen_Area.left) {
					left_Global = desired_Screen_Area.left;
				}
			==== */
	
				if ((**status_Globals_Hdl).global_Device_Startup_Max_Rect_Contains_Menubar == SAFE_TRUE) {
					desired_Screen_Area.top = desired_Screen_Area.top + GetMBarHeight();
				}
				
				top_Global = desired_Screen_Area.top;
				
				/*  */
				/* We will not be centering vertically in this implementation */
				/* But we will instead be honoring a resource preference */
				/* positive or negative */
				/*  */
				top_Global += dream.main_Window_Preferred_Top_Offset;

			/* ===
				top_Global = top_Global + ((desired_Screen_Area.bottom - desired_Screen_Area.top) / 2) - (height / 2);
			==== */
			
				/*  */
				/* TOP MUST be inside if on a Macintosh */
				/*  */
				if (top_Global < desired_Screen_Area.top) {
					top_Global = desired_Screen_Area.top;
				}
				
				if (window_Index == MAIN_SPINNING_WINDOW_INDEX) {
					Clear_Cached_Spin_Window_Placement_Structures(); 
				}
				
				if ((**status_Globals_Hdl).window_Is_Hidden != SAFE_TRUE) {
					ShowHide((**status_Globals_Hdl).status_Window, FALSE);
					window_Needs_Unhide = SAFE_TRUE;
				}
				
				/*  */
				/* Move window, then reposition */
				/*  */
				
				if ((**status_Globals_Hdl).window_Is_Hidden != SAFE_TRUE) {
					SizeWindow(current_Graphics_Window_Ptr,width,height,TRUE);
				}
				else {
					SizeWindow(current_Graphics_Window_Ptr,width,height,FALSE);
				}

				if (window_Index == MAIN_SPINNING_WINDOW_INDEX) {
					Center_Window_High_As_Possible((**status_Globals_Hdl).status_Window
							, 1 + dream.main_Window_Preferred_Top_Offset - k_SPIN_HEIGHT_READJUST);
				}
				else {
					(**status_Globals_Hdl).update_Events_Permissible = SAFE_FALSE;
					MoveWindow((**status_Globals_Hdl).status_Window, left_Global, top_Global, FALSE);
					(**status_Globals_Hdl).update_Events_Permissible = SAFE_TRUE;
				}
				
			/* ===
				ForeColor(blackColor);
				BackColor(blackColor);
				EraseRect (&(((**status_Globals_Hdl).status_Window)->portRect));
				ForeColor(blackColor);
				BackColor(whiteColor);
			=== */
								
			}
			else {
				/*  */
				/* Monitor configuration changed! Ignore cached values. Rewrite */
				/* reset code one day? */
				/*  */
				; /* NULL */
			}
		}
	}
	
	if (window_Needs_Unhide == SAFE_TRUE){
		ShowHide((**status_Globals_Hdl).status_Window, TRUE);
	}
	
	if ((**status_Globals_Hdl).draw_To_Screen_Direct_Plausable == SAFE_TRUE) {
	
	/* ===
		global_Rect = (**((WindowPeek)(**status_Globals_Hdl).status_Window)->contRgn).rgnBBox;
		MoveWindow((**status_Globals_Hdl).status_Window, temp_Rect.left, temp_Rect.top, FALSE);
	=== */
	
		Reposition_Along_Byte_Coordinates((**status_Globals_Hdl).status_Window, TRUE);
	}
	else {
		Reposition_Along_Byte_Coordinates((**status_Globals_Hdl).status_Window, FALSE);
	}

	function_Result = SAFE_TRUE;
	goto Normal_Function_Exit;
	
Error_Handler:
	function_Result = SAFE_FALSE;

Normal_Function_Exit:
	Set_View_Port(saved_Update_Port);
	return(function_Result);
	
} /* Safe_Boolean Change_Any_Window_Size_BASE ... */
/*=**************************************************************************=*/


/*=**************************************************************************=*/
Safe_Boolean	Reset_Any_Window_Size_If_Needed(CARDINAL_16 window_Index
													, Rect *source_Bit_Rect
													, Safe_Boolean *resize_Occured
													, Boolean from_Official_Update)
{
	Safe_Boolean						function_Result;
	Change_Window_Size_Rec		params_For_Change_Window_Size;
	INTEGER_32						source_Bit_Width;
	INTEGER_32						source_Bit_Height;
	Rect								current_Rect;
	INTEGER_32						current_Bit_Width;
	INTEGER_32						current_Bit_Height;
	Primary_Status_Hdl				status_Globals_Hdl;
	Boolean							needs_Resize;
/* END VAR */
	
	
	function_Result = SAFE_FALSE;
	*resize_Occured = SAFE_FALSE;

	source_Bit_Width = (*source_Bit_Rect).right - (*source_Bit_Rect).left;
	source_Bit_Height = (*source_Bit_Rect).bottom - (*source_Bit_Rect).top;

	status_Globals_Hdl = active_Window_List[window_Index];
	
	if ((status_Globals_Hdl != nil)
		&& ((**status_Globals_Hdl).structure_Initialized == SAFE_TRUE)) {
	}
	else {
		goto Error_Handler;
	}

	needs_Resize = FALSE;
	
	
	/*  */
	/* Check if window is already less than k_PROMPT_MAX_CUTOFF. It should be */
	/* if normal prompt small window of 70 pixels high (or 100 )*/
	/*  */
	
	current_Rect = ((WindowPeek)(**status_Globals_Hdl).status_Window)->port.portRect;
	current_Bit_Width = (current_Rect).right - (current_Rect).left;
	current_Bit_Height = (current_Rect).bottom - (current_Rect).top;
	
	/*  */
	/* Compare all directions */
	/*  */
	if ((current_Bit_Height != source_Bit_Height) || (current_Bit_Width != source_Bit_Width)){
		needs_Resize = TRUE;
		params_For_Change_Window_Size.window_Type = k_WINDOW_SIZE_DIRECT_SETTING;
		SetRect(&(params_For_Change_Window_Size.new_Rect), 0, 0, source_Bit_Width, source_Bit_Height);
	}

/* ===
	if (from_Official_Update== TRUE) {
	
	}
==== */
	
	if (needs_Resize == TRUE) {
		params_For_Change_Window_Size.window_Index = window_Index;
				
		if (Change_Any_Window_Size_BASE(&params_For_Change_Window_Size) != SAFE_TRUE) {
			goto Error_Handler;
		}
		*resize_Occured = SAFE_TRUE;
		
		/*  */
		/* Only bother worrying about two type of windows normally (for speed) */
		/*  */
		if (Reset_Video_Background_Rect_For_Window_Move(window_Index) != SAFE_TRUE) {
		
		}
		
		/*  */
		/* Bring to foreground even if was not forground already */
		/*  */
	/* ==
		SelectWindow((**status_Globals_Hdl).status_Window);
	=== */
		
	} /* if (needs_Resize == TRUE) */
	
	function_Result = SAFE_TRUE;
	goto Normal_Function_Exit;

Error_Handler:
	function_Result = SAFE_FALSE;
		
Normal_Function_Exit:
	return(function_Result);
	
} /* Reset_Any_Window_Size_If_Needed ... */
/*=**************************************************************************=*/


/*=**************************************************************************=*/
void	Manage_Disk_Event(EventRecord *macEvent)
{
/*  */
/* Note: embedded define for DoDiskEvent */
/* So that position can be controlled in System 6 */
/*  */
#define DISK_INITIALIZATION_DIALOG_ID		-6047

	Point		corner;
	DialogPtr	disk_Initialization_Dialog;
	OSErr		os_Error;
	GrafPtr	saved_Port;
/* END VAR */
	
	
	if (HiWord(macEvent->message) != noErr) {
	
		/*  */
		/* A disk insert error happened. Ask user if they wish to format. */
		/*  */

		GetPort(&saved_Port);
		disk_Initialization_Dialog = GetNewDialog(DISK_INITIALIZATION_DIALOG_ID, NULL, NULL);
		
		if (disk_Initialization_Dialog != nil) {
			Set_View_Port(disk_Initialization_Dialog);
			
			HideWindow(disk_Initialization_Dialog);
			Center_Window_Upper_One_Third(disk_Initialization_Dialog);
			
			corner.h = disk_Initialization_Dialog->portRect.left;
			corner.v = disk_Initialization_Dialog->portRect.top;
			
			LocalToGlobal(&corner);
			DisposDialog(disk_Initialization_Dialog);
			Set_View_Port(saved_Port);
		}
		else {
			/*  */
			/* Dialog might be centerable in System 7 using -1,-1 but in System 6 */
			/* It does not appear possible without our own dialog. */
			/* In Finder 6.1 Apple uses (112,80) */
			/*  */
			SetPt(&corner, 112, 80);
		}

		os_Error = DIBadMount(corner, macEvent->message);
		
		Set_View_Port(saved_Port);
	}
	
} /* void	Manage_Disk_Event(EventRecord *macEvent) */
/*=**************************************************************************=*/


/*=**************************************************************************=*/
Safe_Boolean Select_Graphics_Window(INTEGER_32 current_Window_Index)
{		
	WindowPtr			current_Graphics_Window_Ptr;
	CARDINAL_8			*code_Name;
	Safe_Boolean			function_Result;
	SignedByte			status_Globals_State;
/* END VAR */

	code_Name = "\pSelect_Graphics_Window";
	

	if (active_Window_List[current_Window_Index] == nil) {
	
		/*  */
		/* If window hasn't been opened before then open it */
		/*  */
		
		if (Create_Status_Window_Structures(&active_Window_List[current_Window_Index]) != SAFE_TRUE) {
			goto Error_Handler;
		}	
		else {
			if (Setup_Status_Window(active_Window_List[current_Window_Index], current_Window_Index) != SAFE_TRUE) {
				goto Error_Handler;
			}
			
			g_Shell.interface.current_Graphics_Window = current_Window_Index;
			
			/*  */
			/*For safety test , lock high for duration of program -- NOT */
			/* 1994.06.29 JAM needed lock in movewindow. Fix requirement one day */
			/* Watch code closer */
			/*  */
			if (Begin_Use_Of_Handle((Handle) active_Window_List[current_Window_Index], MOVE_HIGH_REQUIRED, &status_Globals_State) != SAFE_TRUE) {
				goto Error_Handler;
			}
		}
		
	}
	else {
		/*  */
		/* If window is already open then make it the front most window */
		/*  */
		
		if ((**active_Window_List[current_Window_Index]).structure_Initialized != SAFE_TRUE) {
			SysBeep(20); 
			Assign_String(ns_Shell_Globals.error_Description_String, "\pError. Improperly initialized window structure");
			Assign_String(ns_Shell_Globals.error_Note_String, "\pProgrammer flaw");
			Display_Error_Message(unit_Name, code_Name);
			Setup_Error_Handling(); 
		}
		else {
			
			current_Graphics_Window_Ptr	= (**active_Window_List[current_Window_Index]).status_Window;
			g_Shell.interface.current_Graphics_Window = current_Window_Index;

		
			SelectWindow (current_Graphics_Window_Ptr);
			Set_View_Port (current_Graphics_Window_Ptr);
			/* EraseRect (&((current_Graphics_Window_Ptr)->portRect)); */
		}
		
	} /* NOT if (active_Window_List[current_Window_Index] == nil) */
	
	
	function_Result = SAFE_TRUE;
	goto Normal_Function_Exit;
	
	
Error_Handler:
	function_Result = SAFE_FALSE;

Normal_Function_Exit:
	;	
	
} /* Select_Graphics_Window */
/*=**************************************************************************=*/


/*=**************************************************************************=*/
Safe_Boolean Initialize_Starting_Menu_States(void)
/*  */
/* Only expected to be run once per invocation */
/*  */
{
	Safe_Boolean				function_Result;
/* END VAR */


	function_Result = SAFE_FALSE;
	
	EnableItem(g_Shell.interface.program_Menus[SPECIAL_MENU], MENU_COMMAND_SET_DRAWING_PREFERENCE);
	if (g_Shell.library.use_Home_Made_Graphics_Primitives == TRUE) {
		SetItemMark(g_Shell.interface.program_Menus[SPECIAL_MENU], MENU_COMMAND_SET_DRAWING_PREFERENCE, checkMark);
	}
	else {
		SetItemMark(g_Shell.interface.program_Menus[SPECIAL_MENU], MENU_COMMAND_SET_DRAWING_PREFERENCE, noMark);
	}
	
	if (g_Set_Spin_Window_Max_Monitor_Size == SAFE_TRUE) {
		SetItemMark(g_Shell.interface.program_Menus[ITEMS_MENU], MENU_COMMAND_SET_SPIN_WINDOW_SIZE, checkMark);
	}
	else {
		SetItemMark(g_Shell.interface.program_Menus[ITEMS_MENU], MENU_COMMAND_SET_SPIN_WINDOW_SIZE, noMark);
	}
	EnableItem(g_Shell.interface.program_Menus[ITEMS_MENU], MENU_COMMAND_SET_SPIN_WINDOW_SIZE);

	EnableItem(g_Shell.interface.program_Menus[SPECIAL_MENU], MENU_COMMAND_SET_WINDOW_ALIGNMENT);
	if (g_Shell.library.force_Window_Alignment_To_32_Bits == TRUE) {
		SetItemMark(g_Shell.interface.program_Menus[SPECIAL_MENU], MENU_COMMAND_SET_WINDOW_ALIGNMENT, checkMark);
	}
	else {
		SetItemMark(g_Shell.interface.program_Menus[SPECIAL_MENU], MENU_COMMAND_SET_WINDOW_ALIGNMENT, noMark);
	}
	
	
	EnableItem(g_Shell.interface.program_Menus[SPECIAL_MENU], MENU_COMMAND_GENERAL);
	EnableItem(g_Shell.interface.program_Menus[SPECIAL_MENU], MENU_COMMAND_TEST_RECT);
	EnableItem(g_Shell.interface.program_Menus[SPECIAL_MENU], MENU_COMMAND_TEST_CIRCLE);
	EnableItem(g_Shell.interface.program_Menus[SPECIAL_MENU], MENU_COMMAND_TEST_FILLED_CIRCLE);
	EnableItem(g_Shell.interface.program_Menus[SPECIAL_MENU], MENU_COMMAND_TEST_ELIPSES);
	EnableItem(g_Shell.interface.program_Menus[SPECIAL_MENU], MENU_COMMAND_TEST_FILLED_ELIPSES);
	EnableItem(g_Shell.interface.program_Menus[SPECIAL_MENU], MENU_COMMAND_TEST_LINE);
	EnableItem(g_Shell.interface.program_Menus[SPECIAL_MENU], MENU_COMMAND_FRAMED_POLYGONS);
	EnableItem(g_Shell.interface.program_Menus[SPECIAL_MENU], MENU_COMMAND_TIME_LINE_SPEED);
	
	EnableItem(g_Shell.interface.program_Menus[SPECIAL_MENU], MENU_COMMAND_TEST_THREE_D);
	
/* ==	
	EnableItem(g_Shell.interface.program_Menus[WINDOWS_MENU], MENU_COMMAND_SHOW_CONTROLS);
== */
	
	EnableItem(g_Shell.interface.program_Menus[WINDOWS_MENU], MENU_COMMAND_SELECT_TRANSITION_EFFECT);
	EnableItem(g_Shell.interface.program_Menus[WINDOWS_MENU], MENU_COMMAND_FIRST_TRANSITION_EFFECT);

/* ===
	EnableItem(g_Shell.interface.program_Menus[NETWORK_MENU], MENU_COMMAND_RESET_CONNECTION);
	EnableItem(g_Shell.interface.program_Menus[NETWORK_MENU], MENU_COMMAND_DISCONNECT_SELF);
	EnableItem(g_Shell.interface.program_Menus[NETWORK_MENU], MENU_COMMAND_SHOW_ZONES);
=== */

	EnableItem(g_Shell.interface.program_Menus[FILE_MENU] , MENU_COMMAND_NEW_SCENE);
	EnableItem(g_Shell.interface.program_Menus[FILE_MENU] , MENU_COMMAND_OPEN_SCENE);
	EnableItem(g_Shell.interface.program_Menus[FILE_MENU] , MENU_COMMAND_LOAD_SELECT_SCENE);
	

	EnableItem(g_Shell.interface.program_Menus[FILE_MENU] , MENU_COMMAND_SELECT_SCENE);
	EnableItem(g_Shell.interface.program_Menus[FILE_MENU] , MENU_COMMAND_SELECT_NEXT_SCENE);
	EnableItem(g_Shell.interface.program_Menus[FILE_MENU] , MENU_COMMAND_SELECT_PREVIOUS_SCENE);
	
	EnableItem(g_Shell.interface.program_Menus[FILE_MENU] , MENU_COMMAND_RESCAN_FILES);
	
	EnableItem(g_Shell.interface.program_Menus[FILE_MENU] , MENU_COMMAND_QUIT);

	EnableItem(g_Shell.interface.program_Menus[BACKDROPS_MENU] , MENU_COMMAND_LOAD_BACKDROP);
	EnableItem(g_Shell.interface.program_Menus[BACKDROPS_MENU] , MENU_COMMAND_CONVERT_BACKDROP);

	EnableItem(g_Shell.interface.program_Menus[SLIDE_MENU] , MENU_COMMAND_SELECT_SLIDE);
	EnableItem(g_Shell.interface.program_Menus[SLIDE_MENU] , MENU_COMMAND_SELECT_NEXT_SLIDE);
	EnableItem(g_Shell.interface.program_Menus[SLIDE_MENU] , MENU_COMMAND_SELECT_PREVIOUS_SLIDE);
	EnableItem(g_Shell.interface.program_Menus[SLIDE_MENU] , MENU_COMMAND_HIDE_WINDOW);
	
	EnableItem(g_Shell.interface.program_Menus[SLIDE_MENU] , MENU_COMMAND_SET_WINDOW_SIZE);
	
	EnableItem(g_Shell.interface.program_Menus[AUDIO_MENU] , MENU_COMMAND_MONITOR_REAL_MIDI);	
	EnableItem(g_Shell.interface.program_Menus[AUDIO_MENU] , MENU_COMMAND_RECORD_TITLED_AIFF_FILE);
	EnableItem(g_Shell.interface.program_Menus[AUDIO_MENU] , MENU_COMMAND_RECORD_AIFF_FILE);
	EnableItem(g_Shell.interface.program_Menus[AUDIO_MENU] , MENU_COMMAND_RECORD_UNTITLED_AIFF_FILE);
	EnableItem(g_Shell.interface.program_Menus[AUDIO_MENU] , MENU_COMMAND_STOP_RECORDING_AIFF_FILE);
	EnableItem(g_Shell.interface.program_Menus[AUDIO_MENU] , MENU_COMMAND_PLAY_AIFF_FILE);
	EnableItem(g_Shell.interface.program_Menus[AUDIO_MENU] , MENU_COMMAND_PLAY_RECENT_AIFF_FILE);
	EnableItem(g_Shell.interface.program_Menus[AUDIO_MENU] , MENU_COMMAND_STOP_PLAYING_AIFF_FILE);
	EnableItem(g_Shell.interface.program_Menus[AUDIO_MENU] , MENU_COMMAND_SWAP_VOICE_FOR_AIFF_FILE);
	EnableItem(g_Shell.interface.program_Menus[AUDIO_MENU] , MENU_COMMAND_SWAP_ASSUMED_VOICE_FOR_AIFF_FILE);


	/*  */
	/* Unique to this implementation */
	/*  */
	if (Enable_Minimal_Puppet_Controls() != SAFE_TRUE) {
		goto Error_Handler;
	}	

	/*  */
	/* Sets flags, uses globals that should be initialized by now */
	/*  */
	if (Reset_Debugging_Controls() != SAFE_TRUE) {
		goto Error_Handler;
	}
	
	/*  */
	/* Sets flags, uses globals that should be initialized by now */
	/*  */
	if (Enable_Audio_Controls() != SAFE_TRUE) {
		goto Error_Handler;
	}
		
	function_Result = SAFE_TRUE;
	goto Normal_Function_Exit;

Error_Handler:
	function_Result = SAFE_FALSE;

Normal_Function_Exit:
	return(function_Result);
}
/*=**************************************************************************=*/


/*=**************************************************************************=*/
Safe_Boolean Initialize_Set_Spin_Window_Max_Monitor(void)
{
	
	g_Set_Spin_Window_Max_Monitor_Size = SAFE_FALSE;
	
	g_Menu_Bar_Obscured = SAFE_FALSE;
	g_Menu_Bar_Previous_Height = 0;
	g_Corner_Regions = nil;
	g_Menu_Bar_Previous_Region = nil;
}
/*=**************************************************************************=*/


/*=**************************************************************************=*/
void Square_Desk_Top_Corners(GDHandle device)
{
	Rect rect_Size;
	INTEGER_32 quickDraw;
/* END VAR */


	Gestalt(gestaltQuickdrawVersion,&quickDraw);
	if(quickDraw<gestalt8BitQD){
		Refresh_Quick_Draw_Globals();
		rect_Size=qd.screenBits.bounds;
		rect_Size.top+=GetMBarHeight();
	}else{
		if(device==NULL){
			goto Normal_Function_Exit;
		}
		rect_Size=(*device)->gdRect;
		if(device==GetMainDevice()) {
			rect_Size.top+=GetMBarHeight();
		}
	}
	g_Corner_Regions=NewRgn();
	RectRgn(g_Corner_Regions,&rect_Size);
	DiffRgn(g_Corner_Regions,GetGrayRgn(),g_Corner_Regions);
	if(EmptyRgn(g_Corner_Regions))return;
	UnionRgn(GetGrayRgn(),g_Corner_Regions,GetGrayRgn());
	PaintBehind(FrontWindow(),g_Corner_Regions);
	CalcVisBehind(FrontWindow(),g_Corner_Regions);
	
Normal_Function_Exit:
	; /* NULL */
}
/*=**************************************************************************=*/


/*=**************************************************************************=*/
void Restore_Desk_Top_Curved_Corners(GDHandle device)
/*  */
/* Restore rounding to this screen. */
/* If NULL then applies to all screens. */
/*  */
{
/* END VAR */

	if(g_Corner_Regions!=NULL){
//		Refresh_Quick_Draw_Globals();			// Make sure qd is valid.
//		FillRgn(g_Corner_Regions,qd.black);	// don't know what port it belongs to
		DiffRgn(GetGrayRgn(),g_Corner_Regions,GetGrayRgn());
		DisposeRgn(g_Corner_Regions);
		g_Corner_Regions=NULL;
	}
	
Normal_Function_Exit:
	; /* NULL */
}
/*=**************************************************************************=*/


/*=**************************************************************************=*/
void Refresh_Quick_Draw_Globals(void)
{
	/*  */
	/* HACK HACK HACK : This should not be null rounine. */
	/*  */
	; /* NULL */
}
/*=**************************************************************************=*/


/*=**************************************************************************=*/
Safe_Boolean Hide_Menu_Bar(void)
{	
	Rect rect_Size;
	CARDINAL_32 quick_Draw_Version;
/* END VAR */


	if (g_Preferred_Monitor_Values_Valid == SAFE_TRUE) {
		if (g_Preferred_Monitor_Max_Rect_Contains_Menubar == SAFE_TRUE) {		
			if (g_Menu_Bar_Obscured != SAFE_TRUE) {
			/* ===
				g_Set_Spin_Window_Max_Monitor_Size = SAFE_FALSE;
			=== */
				g_Menu_Bar_Previous_Region=NewRgn();
				Refresh_Quick_Draw_Globals();
				
				g_Menu_Bar_Previous_Height = GetMBarHeight();
				
				if (g_Menu_Bar_Previous_Height > 0) {
					rect_Size=qd.screenBits.bounds;
					rect_Size.bottom=rect_Size.top+g_Menu_Bar_Previous_Height;
					RectRgn(g_Menu_Bar_Previous_Region,&rect_Size);
					MBarHeight =0;
					UnionRgn(GetGrayRgn(),g_Menu_Bar_Previous_Region,GetGrayRgn());
					PaintOne(NULL,g_Menu_Bar_Previous_Region);
					CalcVisBehind(FrontWindow(),g_Menu_Bar_Previous_Region);
				}
				
				Square_Desk_Top_Corners(g_Preferred_Monitor_Graphics_Device);
				g_Menu_Bar_Obscured = SAFE_TRUE;
				
				if (Redraw_Spinning_Window_Title_Bar_If_Possible(SAFE_FALSE) != SAFE_TRUE) {
					/*  */
					/* Ignore error. Unlikly to fail in design */
					/*  */
				}
			}
		}
	}
}
/*=**************************************************************************=*/


/*=**************************************************************************=*/
Safe_Boolean Restore_Menu_Bar(void)
{
/* END VAR */


	if (g_Menu_Bar_Obscured == SAFE_TRUE) {
	/* ===
		g_Set_Spin_Window_Max_Monitor_Size = SAFE_FALSE;
	=== */
		if (g_Preferred_Monitor_Values_Valid == SAFE_TRUE) {
			Restore_Desk_Top_Curved_Corners(g_Preferred_Monitor_Graphics_Device);
		}
	
		MBarHeight=g_Menu_Bar_Previous_Height;
		DiffRgn(GetGrayRgn(),g_Menu_Bar_Previous_Region,GetGrayRgn());
		DrawMenuBar();
		CalcVisBehind(FrontWindow(),g_Menu_Bar_Previous_Region);
		DisposeRgn(g_Menu_Bar_Previous_Region);
		g_Menu_Bar_Previous_Region=NULL;
	
		g_Menu_Bar_Obscured = SAFE_FALSE;
		
		if (Redraw_Spinning_Window_Title_Bar_If_Possible(SAFE_TRUE) != SAFE_TRUE) {
			/*  */
			/* Ignore error. Unlikly to fail in design */
			/*  */
		}

	}
}
/*=**************************************************************************=*/


/*=**************************************************************************=*/
void Warn_About_Slow_Emulated_Animation(void)
{
	Str255		error_String;
/* END VAR */

	
	Get_NS_String(k_DREAM_ERROR_STRINGS, k_NOTE_MOST_ANIMATION_IS_NOT_POWERPC_NATIVE, error_String);
	Perform_OK_Dialog(error_String);
}
/*=**************************************************************************=*/


/*=**************************************************************************=*/
Safe_Boolean Inform_Window_For_Pre_Drag_Actions(INTEGER_32		window_Index)
{
	Safe_Boolean		function_Result;
	WindowPtr		current_Graphics_Window_Ptr;
/* END VAR */


	function_Result = SAFE_FALSE;
	switch (window_Index) {
		
		case GENERAL_INFO_WINDOW_INDEX:
				; /* NULL, OK to procede */
			break;
		
		case TEST_DREAMING_WINDOW_INDEX:
		default:
			function_Result = SAFE_TRUE;
			goto Normal_Function_Exit;
	}
	
	
	if ((active_Window_List[window_Index] == nil)
		|| ((**active_Window_List[window_Index]).structure_Initialized != SAFE_TRUE)) {
		goto Error_Handler;
	}
	
	current_Graphics_Window_Ptr	= (**active_Window_List[window_Index]).status_Window;



	function_Result = SAFE_TRUE;
	goto Normal_Function_Exit;
		
Error_Handler:
	function_Result = SAFE_FALSE;
		
Normal_Function_Exit:

	return(function_Result);
	
} /* Safe_Boolean Inform_Window_For_Pre_Drag_Actions ... */
/*=**************************************************************************=*/


/*=**************************************************************************=*/
Safe_Boolean Inform_Window_For_Post_Drag_Actions(INTEGER_32		window_Index)
{
	Safe_Boolean		function_Result;
	WindowPtr		current_Graphics_Window_Ptr;
/* END VAR */


	function_Result = SAFE_FALSE;
	switch (window_Index) {
		
		case GENERAL_INFO_WINDOW_INDEX:
				; /* NULL, OK to procede */
			break;
		
		case TEST_DREAMING_WINDOW_INDEX:
		default:
			function_Result = SAFE_TRUE;
			goto Normal_Function_Exit;
	}
	
	if ((active_Window_List[window_Index] == nil)
		|| ((**active_Window_List[window_Index]).structure_Initialized != SAFE_TRUE)) {
		goto Error_Handler;
	}
	
	current_Graphics_Window_Ptr	= (**active_Window_List[window_Index]).status_Window;



	function_Result = SAFE_TRUE;
	goto Normal_Function_Exit;
		
Error_Handler:
	function_Result = SAFE_FALSE;
		
Normal_Function_Exit:

	return(function_Result);
	
} /* Safe_Boolean Inform_Window_For_Pre_Drag_Actions ... */
/*=***************************************************************************/


/*=**************************************************************************=*/
Safe_Boolean Perform_Periodic_Tasks(void)
{
	Safe_Boolean				function_Result;
	INTEGER_16				scene_To_Use;
/* END VAR */

	
	
	Update_Spin_Window_If_Needed();
	
	if (g_Forced_Digitizer_Window_Update_Required == SAFE_TRUE) {
		Force_Video_Background_Full_Reupdate_If_Needed();
		g_Forced_Digitizer_Window_Update_Required = SAFE_FALSE;
	}

	
	if (Update_Sound_Input_Monitoring() != SAFE_TRUE) {
		/*  */
		/* Ignore errors in this implementation */
		/*  */
	}
	
	if (g_Voices.phone_Rec.phoneBridge_Enabled == TRUE) {
		if (g_Voices.phone_Rec.phoneBridge_Ringing == TRUE) {
			Take_Phone_On_Or_Off_Hook(FALSE);
			g_Voices.phone_Rec.phoneBridge_Ringing = FALSE;
		}
	}
	
	Close_Any_Sound_Files_If_Needed();

	if (g_Dreaming_Module_Globals_Initialized == SAFE_TRUE) {
		
		if (dream.play.total_Scenes > 0) {
			scene_To_Use = dream.current_Scene;
			
			if (dream.play.scene_Memory_In_Use[scene_To_Use] == MEMORY_INITIALIZED) {
				if ((**(dream.play.scene[scene_To_Use])).initialized == SAFE_TRUE) {
					
					if (Revise_Current_Scene(TEST_DREAMING_WINDOW_INDEX, scene_To_Use) != SAFE_TRUE) {
						/*  */
						/* Ignore errors in this implementation */
						/*  */
					}
				}
			}
		}
		
		/*  */
		/*  */
		/*  */
		/* HACK HACK HACK This should only be called as soon as bacground is drawn */
		/*  */
		if (Setup_Pallette_Animation(TEST_DREAMING_WINDOW_INDEX) != SAFE_TRUE) {
			/*  */
			/* Ignore errors in this implementation */
			/*  */
		}

		/*  */
		/*  */
		/*  */
		/* HACK HACK HACK This should only be called if background desires animation */
		/*  */
		if (Animate_Color_Table_For_Window(TEST_DREAMING_WINDOW_INDEX) != SAFE_TRUE) {
			/*  */
			/* Ignore errors in this implementation */
			/*  */
		}
		
		if (g_Scanning_Needs_Run == SAFE_TRUE) {
			Animate_Cursor(STANDARD_ACUR_ID, DO_NOT_USE_VBL);
			if (Start_Scanning_For_Files(k_ALL_SELECTIONS) != SAFE_TRUE) {
				/* Ignore */
			}
			g_Scanning_Needs_Run = SAFE_FALSE;
			
			if (g_Scanning_Active != SAFE_TRUE) {
				Perform_Startup_Actions_For_Scan_Completion();
				Stop_Animating_Cursor();
			}
		}
	
		if (g_Scanning_Active == SAFE_TRUE) {
			if (Continue_Scanning_For_Files() != SAFE_TRUE) {
				/* Ignore */
			}

			if (g_Scanning_Active == SAFE_TRUE) {
				Animate_Cursor_Tickle();
			}
			else {
				Perform_Startup_Actions_For_Scan_Completion();
				Stop_Animating_Cursor();
			}
		}
		
		if (Proccess_Response_Queue_If_Needed() != SAFE_TRUE) {
			/* Ignore */
		}
		
	} /* if (g_Dreaming_Module_Globals_Initialized == SAFE_TRUE) */
	
	/*  */
	/*  */
	/*  */
	/*  */
	/*  */
	/* HACK HACK HACK */
	/*  */
	/* Update network zone checking code */
	/*  */
	Revise_Network_Zone_Management();
		
	function_Result = SAFE_TRUE;
	goto Normal_Function_Exit;

Error_Handler:
	function_Result = SAFE_FALSE;

Normal_Function_Exit:
	return(function_Result);
	
} /* Safe_Boolean Perform_Periodic_Tasks ... */
/*=**************************************************************************=*/


/*=**************************************************************************=*/
Safe_Boolean Run_Mac_Version_Of_Program(void)
{
	CARDINAL_32			desired_Stack;
	SignedByte				status_Globals_State;
	WindowPtr				current_Graphics_Window_Ptr;
	Primary_Status_Hdl		current_Window_Globals_Hdl;
	CARDINAL_32			window_Index_For_NS_Shell;
	
	Safe_Boolean				function_Result;
	Safe_Boolean				window_Newly_Activated;
	Rect						temp_Rect;
	Boolean					there_Is_An_Event_Waiting;
	EventRecord				event_Record;
	WindowPtr				event_Window;
	INTEGER_16				window_Part;
	CARDINAL_32			window_Index;
	
	Boolean					check_For_Events;
	CARDINAL_32			current_Tick_Time;

	Boolean					track;
	char						key;
	CARDINAL_8				*code_Name;
/* END VAR */


	code_Name = "\pRun_Mac_Version_Of_Program";
	function_Result = SAFE_FALSE;

	g_Dreaming_Module_Globals_Initialized = SAFE_FALSE;
	g_Voice_Globals_Initialized = SAFE_FALSE;
	g_Digital_Audio_Card_In_Use = SAFE_FALSE;
	
	/*  */
	/* Just for debugging and safe temp port setting */
	/*  */
	GetPort(&g_Initial_Scrap_Port);
	g_Initial_Scrap_Graphics_Device = GetGDevice();


	g_Preferred_Monitor_Values_Valid = SAFE_FALSE;
	/*  */
	/* Set to some bogus values */
	/*  */
	g_Preferred_Monitor_Depth = 8;
	g_Preferred_Monitor_Slots = 0xFF;
	SetRect(&g_Preferred_Monitor_Max_Rect,0,0,1,1);
	g_Preferred_Monitor_Max_Rect_Contains_Menubar = SAFE_FALSE;
	g_Preferred_Monitor_Video_Pix_Map_Handle = nil;
	
	if (Initialize_Set_Spin_Window_Max_Monitor() != SAFE_TRUE) {

	}
	
	g_Window_Limits.valid_Information = SAFE_FALSE;
	
	if (Initialize_Window_Limits_If_Needed(&g_Window_Limits) != SAFE_TRUE) {

	}

	if (Initialize_Remote_Control_Standard() != SAFE_TRUE) {
		goto Error_Handler;
	}
	
	if (Initialize_Remote_Control_Full() != SAFE_TRUE) {
		goto Error_Handler;
	}

	if (Initialize_Physical_File_Mapping() != SAFE_TRUE) {
		goto Error_Handler;
	}
	

	/*  */
	/* Unique to this implementation */
	/*  */
	if (Initialize_Dreaming_If_Needed() != SAFE_TRUE) {
		goto Error_Handler;
	}
	
	
	if (Initialize_Starting_Menu_States() != SAFE_TRUE) {
		goto Error_Handler;
	}

	if (Initialize_MOD_Utilities() != SAFE_TRUE) {
		goto Error_Handler;
	}

	/*  */
	/* Unique to this implementation */
	/*  */
	if (Initialize_Network_Zone_Management() != SAFE_TRUE) {
		goto Error_Handler;
	}	
	
	if (Initialize_Command_Executor() != SAFE_TRUE) {
		goto Error_Handler;
	}
	
	if (Initialize_Command_Dialogs() != SAFE_TRUE) {
		goto Error_Handler;
	}
	
	if (Initialize_Command_Redirector() != SAFE_TRUE) {
		goto Error_Handler;
	}
	
	if (Initialize_Object_Inventory() != SAFE_TRUE) {
		goto Error_Handler;
	}
	
	if (Initialize_Sound_Record() != SAFE_TRUE) {
		goto Error_Handler;
	}
	
	if (Initialize_Dreaming_Audio_Record() != SAFE_TRUE) {
		goto Error_Handler;
	}
	
	if (Initialize_Sound_Tap_Channel() != SAFE_TRUE) {
		goto Error_Handler;
	}	
	
	if (Initialize_Sound_Play((Ptr)&Inform_Main_Routine_Audio_File_Is_Closing) != SAFE_TRUE) {
		goto Error_Handler;
	}	
	
	if (Initialize_Dreaming_Backdrop_Files() != SAFE_TRUE) {
		goto Error_Handler;
	}
	
	if (Initialize_Simple_Ring_Cache() != SAFE_TRUE) {
		goto Error_Handler;
	}
	
	if (Initialize_Spin_Object_Files() != SAFE_TRUE) {
		goto Error_Handler;
	}
	
	if (Initialize_Stage_View_Manager() != SAFE_TRUE) {
		goto Error_Handler;
	}
	
	if (Initialize_Dreaming_Scene_Files() != SAFE_TRUE) {
		goto Error_Handler;
	}	
	
	if (Initialize_Slide_Show_Routines() != SAFE_TRUE) {
		goto Error_Handler;
	}
	
	if (Initialize_Command_Control() != SAFE_TRUE) {
		goto Error_Handler;
	}
	
	if (Initialize_Spin_O_Rama_Cache(SAFE_TRUE) != SAFE_TRUE) {
		goto Error_Handler;
	}
	
	if (Initialize_Spin_O_Rama() != SAFE_TRUE) {
		goto Error_Handler;
	}
	
	if (Initialize_Video_Backdrop_Background() != SAFE_TRUE) {
		goto Error_Handler;
	}
	

	/*  */
	/* Note : Warn user about slowdown if not 68K */
	/*  */
	if (ns_Shell_Globals.has_Real_68K_Available != TRUE) {
		Warn_About_Slow_Emulated_Animation();
	}


	g_Shell.library.application_Is_Frontmost = SAFE_TRUE;
	if (g_Shell.library.application_Is_Frontmost == SAFE_TRUE) {
		g_Shell.interface.yield_Time = k_DEFAULT_YIELD_TIME;
	}
	else {
		g_Shell.interface.yield_Time = k_DEACTIVATED_YIELD_TIME;
	}

	
	/*  */
	/* HACK HACK HACK  DISABLE UNTIL COMPLETED */
	/*  */
/* ===
	if (Select_Controller_And_Display_First_Time() != SAFE_TRUE) {
		goto Error_Handler;
	}
=== */


	/*  */
	/* HACK HACK HACK */
	/*  */
/* ===
	if (Select_Spin_Window_Display_First_Time() != SAFE_TRUE) {
		goto Error_Handler;
	}
===== */

	if (Create_Status_Window_Structures(&active_Window_List[GENERAL_INFO_WINDOW_INDEX]) != SAFE_TRUE) {
		goto Error_Handler;
	}	
	else {
		if (Setup_Status_Window(active_Window_List[GENERAL_INFO_WINDOW_INDEX], GENERAL_INFO_WINDOW_INDEX) != SAFE_TRUE) {
			goto Error_Handler;
		}
		
		/*  */
		/*For safety test , lock high for duration of program -- NOT */
		/* 1994.06.29 JAM YES YES YES!  (MoveWindow not locked once move ram */
		/*  */
		if (Begin_Use_Of_Handle((Handle) active_Window_List[GENERAL_INFO_WINDOW_INDEX], MOVE_HIGH_REQUIRED, &status_Globals_State) != SAFE_TRUE) {
			goto Error_Handler;
		}
	}
	
	/*  */
	/* Select the status window as the current window before starting */
	/* Tests search logic only */
	/*  */
	if (Fetch_NS_Window_Index_Using_WindowPeek((WindowPeek)((**active_Window_List[GENERAL_INFO_WINDOW_INDEX]).status_Window)
						, &window_Index_For_NS_Shell
						, (Handle *)&current_Window_Globals_Hdl) != SAFE_TRUE) {
		SysBeep(20); 
		Assign_String(ns_Shell_Globals.error_Description_String, "\pProgram flaw. Fetch_NS_Window_Index_Using_WindowPeek failed");
		Assign_String(ns_Shell_Globals.error_Note_String, "\pReboot Soon ?");
		Display_Error_Message(unit_Name, code_Name);
		Setup_Error_Handling(); 
		goto Error_Handler;
	}
	
	if (Get_Program_Window_Using_NS_Window_Index(window_Index_For_NS_Shell
				, &window_Index) != SAFE_TRUE) {
		SysBeep(20); 
		Assign_String(ns_Shell_Globals.error_Description_String, "\pProgram flaw. Get_Program_Index_Using_NS_Window_Index failed");
		Assign_String(ns_Shell_Globals.error_Note_String, "\pReboot Soon ?");
		Display_Error_Message(unit_Name, code_Name);
		Setup_Error_Handling(); 
		goto Error_Handler;
	}

	g_Shell.interface.current_Graphics_Window	= window_Index;	
	current_Graphics_Window_Ptr	= (**active_Window_List[GENERAL_INFO_WINDOW_INDEX]).status_Window;

#if (k_USE_THINK_PROFILER == 1)
	InitProfile(800, 30);
	_profile = FALSE;
	_trace = FALSE;
/* ===
	start_VIA_timer();
=== */
	_profile = FALSE;
	_trace = FALSE;
	freopen("profiler report.note", "w", stdout);
#endif


	if (g_Shell.interface.abort_Program == TRUE ) {
		g_Shell.interface.quit_Program = TRUE;
	}


	current_Tick_Time = 0;
	g_Last_Tick_Time = TickCount();
	
	while (g_Shell.interface.quit_Program != TRUE) {

		current_Tick_Time = TickCount();
		/*  */
		/* 1994.04.22 Adjustment */
		/*  */
		if ((CARDINAL_32)(g_Last_Tick_Time + (CARDINAL_32)k_TICKS_TO_DELAY_EVENT_CHECK) 
			<= (current_Tick_Time )) {

			check_For_Events = TRUE;
			g_Last_Tick_Time = current_Tick_Time;
		}
		else {
			check_For_Events = FALSE;
		}

		if ((check_For_Events == TRUE) && (g_Shell.interface.quit_Program != TRUE)) {
		
#if (k_USE_THINK_PROFILER == 1)
	_profile = TRUE;
#endif

			Alter_Cursor_If_Needed();
			
			if (Process_Periodic_Async_List_Tasks() != SAFE_TRUE) {
			
			}

			if (Process_Periodic_Async_Spin_Tasks() != SAFE_TRUE) {
			
			}
			
#if (k_USE_THINK_PROFILER == 1)
	_profile = FALSE;
#endif

			if (ns_Shell_Globals.wait_Next_Event_Available == TRUE) {
				/*  */
				/* Organized for prioritizing scanning */
				/*  */

				if (g_Scanning_Active == SAFE_TRUE) {
					g_Shell.interface.yield_Time = k_SEARCH_ACTIVE_YIELD_TIMEOUT;
				}
				else {
					if (g_Shell.library.application_Is_Frontmost == SAFE_TRUE) {
						g_Shell.interface.yield_Time = k_DEFAULT_YIELD_TIME;
					}
					else {
						g_Shell.interface.yield_Time = k_DEACTIVATED_YIELD_TIME;
					}
				}
				there_Is_An_Event_Waiting = WaitNextEvent(everyEvent, &event_Record, g_Shell.interface.yield_Time, nil);
			}
			else {
				there_Is_An_Event_Waiting = GetNextEvent(everyEvent, &event_Record);
				if (there_Is_An_Event_Waiting != TRUE)
					SystemTask();
			}
		
			if (there_Is_An_Event_Waiting == TRUE) {
									
				switch (event_Record.what) {
				
				case nullEvent:
					/*  */
					/* For speed, because we might have just been invoked */
					/* after blocking for a while on "WaitNextEvent", calling */
					/* "Perform_Periodic_Tasks" seems logical here but the loop */
					/* will repeat normally and process it at the top end */
					/*  */
					if (Perform_Periodic_Tasks() != SAFE_TRUE) {
						/*  */
						/* Ignore errors in this implementation */
						/*  */
					}
					break;
				
				case mouseDown:
				
					event_Window = (WindowPtr)(event_Record.message);
					window_Part = FindWindow(event_Record.where, &event_Window);
					
					if ((window_Part != inDesk) && (window_Part != inSysWindow) && (window_Part != inMenuBar)){
						
						if (event_Window == current_Graphics_Window_Ptr) {
							g_Shell.interface.current_Graphics_Window	= window_Index;	
							window_Newly_Activated = SAFE_FALSE;
						}
						else {
							
							if (Fetch_NS_Window_Index_Using_WindowPeek((WindowPeek)event_Window
												, &window_Index_For_NS_Shell
												, (Handle *)&current_Window_Globals_Hdl) != SAFE_TRUE) {
								SysBeep(20); 
								Assign_String(ns_Shell_Globals.error_Description_String, "\pProgram flaw. Fetch_NS_Window_Index_Using_WindowPeek failed");
								Assign_String(ns_Shell_Globals.error_Note_String, "\pReboot Soon ?");
								Display_Error_Message(unit_Name, code_Name);
								Setup_Error_Handling(); 
								goto Error_Handler;
							}
							
							if (Get_Program_Window_Using_NS_Window_Index(window_Index_For_NS_Shell
										, &window_Index) != SAFE_TRUE) {
								SysBeep(20); 
								Assign_String(ns_Shell_Globals.error_Description_String, "\pProgram flaw. Get_Program_Index_Using_NS_Window_Index failed");
								Assign_String(ns_Shell_Globals.error_Note_String, "\pReboot Soon ?");
								Display_Error_Message(unit_Name, code_Name);
								Setup_Error_Handling(); 
								goto Error_Handler;
							}
								
							g_Shell.interface.current_Graphics_Window	= window_Index;	
							current_Graphics_Window_Ptr	= event_Window;
							window_Newly_Activated = SAFE_TRUE;
						}
					}
					
					
					switch (window_Part) {
					
					case inDesk:
						break;
						
					case inSysWindow:
						SystemClick(&event_Record, event_Window);
						break;
					
					case inMenuBar:
						if (Perform_Menu_Command(MenuSelect(event_Record.where), &g_Shell.interface.quit_Program) != SAFE_TRUE) {
							
							SysBeep(20); 
							Assign_String(ns_Shell_Globals.error_Description_String, "\pPerform_Menu_Command failed");
							Assign_String(ns_Shell_Globals.error_Note_String, "\pProgram should be aborted");
							Display_Error_Message(unit_Name, code_Name);
							Setup_Error_Handling(); 
							
							if (Perform_Yes_No_Dialog("\pMay the program terminate itself now?") == TRUE) {
								g_Shell.interface.abort_Program = TRUE;
								g_Shell.interface.quit_Program = TRUE;
							}
							else {
								g_Shell.interface.abort_Program = FALSE;
								g_Shell.interface.quit_Program = FALSE;
							}
						}
						break;
							
					case inContent:
							SelectWindow (event_Window);
							
							if (window_Newly_Activated == SAFE_TRUE) {
								Set_View_Port(event_Window);
								if ((**current_Window_Globals_Hdl).draw_To_Screen_Direct_Plausable == SAFE_TRUE) {
									if (Reset_Current_Window_Structures(TRUE, FALSE) != TRUE) {
										/*  */
										/* HACK HACK : need to raise panic alert */
										/*  */
									}
								}
							}
							
							if (Interpret_Mouse_In_Content(current_Window_Globals_Hdl, event_Window, &event_Record) != SAFE_TRUE) {
								/*  */
								/* Not expected to error by design. Not vital. */
								/*  */
							}
						break;
						
					case inDrag:
							/*  */
							/* Officially "SelectWindow" should only be called in content only*/
							/* but since not active, I decided to put this here for tests anyway. */
							/* Remove at a later date! */
							/*  */
							if (window_Newly_Activated == SAFE_TRUE) {
								SelectWindow(event_Window);
							}
					
							/* temp_window_Globals_Hdl = (**nS_Shell_Window_Manager_Hdl).NS_Window_Data_Record[g_Shell.interface.current_Graphics_Window].window_Globals_For_Routines; */
							if ((**current_Window_Globals_Hdl).structure_Initialized != SAFE_TRUE) {
								/*  */
								/* !!! Wierd, should handle this error in near future!! */
								/*  */
							}
							else {
								temp_Rect = (**current_Window_Globals_Hdl).drag_Rect;
								DragWindow (event_Window, event_Record.where, &temp_Rect);
								/*  */
								/* For the direct to screen writing routines, this doesn't need to */
								/* be done here, but at least it is honest */
								/*  */
								
								/*  */
								/* HACK HACK reposition based on settings in menu, also */
								/* clear cached video options */
								/*  */
								if (window_Index == MAIN_SPINNING_WINDOW_INDEX) {
								
									Clear_Cached_Spin_Window_Placement_Structures(); 
								
									if ((**current_Window_Globals_Hdl).draw_To_Screen_Direct_Plausable == SAFE_TRUE) {
										Reposition_Along_Byte_Coordinates(event_Window, TRUE);
									}
									else {
										Reposition_Along_Byte_Coordinates(event_Window, FALSE);
									}
								
								}
								else {
									if ((**current_Window_Globals_Hdl).draw_To_Screen_Direct_Plausable == SAFE_TRUE) {
										Reposition_Along_Byte_Coordinates(event_Window, TRUE);
									}
									else {
										Reposition_Along_Byte_Coordinates(event_Window, FALSE);
									}
								}
								
								Set_View_Port(event_Window);

								if ((**current_Window_Globals_Hdl).draw_To_Screen_Direct_Plausable == SAFE_TRUE) {
									if (Reset_Current_Window_Structures(TRUE , FALSE) != TRUE) {
										/*  */
										/* HACK HACK : need to raise panic alert */
										/*  */
									};
								}
								
								/*  */
								/* Only bother worrying about two type of windows normally (for speed) */
								/* HACK HACK HACK, only do something if the window actually has moved!! */
								/*  */
								if (Reset_Video_Background_Rect_For_Window_Move(window_Index) != SAFE_TRUE) {
								
								}
							} 
							
						break;
						
					case inGrow:
						break;
						
					case inGoAway:
						track = TrackGoAway (event_Window, event_Record.where);
						
						if (track != FALSE) {
							/*  */
							/* Officially "SelectWindow" should only be called in content only*/
							/* but since not active, I decided to put this here for tests anyway. */
							/* Remove at a later date! */
							/*  */
							if (window_Newly_Activated == SAFE_TRUE) {
								SelectWindow (event_Window);
							}
									
							Close_Status_Window_Fully(current_Window_Globals_Hdl);
							
							if (Destroy_Status_Window_Structures(&current_Window_Globals_Hdl) != SAFE_TRUE) {
								g_Shell.interface.abort_Program = TRUE;
							}
	
	
							if (window_Index == g_Shell.interface.current_Graphics_Window) {
								g_Shell.interface.current_Graphics_Window	= 0;	
								current_Graphics_Window_Ptr = nil;
							}
								
						}
						break;
						
					case inZoomIn:
						break;
						
					case inZoomOut:
						break;
						
					default:
						break;
						
					} /* case mouseDown: */
				break;
				
			case keyDown:
			case autoKey:
				key = event_Record.message & charCodeMask;
				if ( event_Record.modifiers & cmdKey ) {
					if ( event_Record.what == keyDown ) {
						if (Perform_Menu_Command(MenuKey(key), &g_Shell.interface.quit_Program) != SAFE_TRUE) {
	
							SysBeep(20); 
							Assign_String(ns_Shell_Globals.error_Description_String, "\pPerform_Menu_Command failed");
							Assign_String(ns_Shell_Globals.error_Note_String, "\pProgram should be aborted");
							Display_Error_Message(unit_Name, code_Name);
							Setup_Error_Handling(); 
							
							if (Perform_Yes_No_Dialog("\pMay the program terminate itself now?") == TRUE) {
								g_Shell.interface.abort_Program = TRUE;
								g_Shell.interface.quit_Program = TRUE;
							}
							else {
								g_Shell.interface.abort_Program = FALSE;
								g_Shell.interface.quit_Program = FALSE;
							}
	
						}
					}
				}
				else {
					if (Interpret_Key_Command(event_Record.message, event_Record.modifiers) != SAFE_TRUE) {
						/*  */
						/* Ignore. No beep wanted */
						/*  */
					}
				}
				break;
				
			case updateEvt:
				event_Window = (WindowPtr)(event_Record.message);
	
				if (event_Window == current_Graphics_Window_Ptr) {
					g_Shell.interface.current_Graphics_Window	= window_Index;	
				}
				else {
					
					if (Fetch_NS_Window_Index_Using_WindowPeek((WindowPeek)event_Window
										, &window_Index_For_NS_Shell
										, (Handle *)&current_Window_Globals_Hdl) != SAFE_TRUE) {
						SysBeep(20); 
						Assign_String(ns_Shell_Globals.error_Description_String, "\pProgram flaw. Fetch_NS_Window_Index_Using_WindowPeek failed");
						Assign_String(ns_Shell_Globals.error_Note_String, "\pReboot Soon ?");
						Display_Error_Message(unit_Name, code_Name);
						Setup_Error_Handling(); 
						goto Error_Handler;
					}
					else {
					
						if (Get_Program_Window_Using_NS_Window_Index(window_Index_For_NS_Shell
									, &window_Index) != SAFE_TRUE) {
							SysBeep(20); 
							Assign_String(ns_Shell_Globals.error_Description_String, "\pProgram flaw. Get_Program_Index_Using_NS_Window_Index failed");
							Assign_String(ns_Shell_Globals.error_Note_String, "\pReboot Soon ?");
							Display_Error_Message(unit_Name, code_Name);
							Setup_Error_Handling(); 
							goto Error_Handler;
						}
								
						current_Graphics_Window_Ptr = event_Window;
						g_Shell.interface.current_Graphics_Window	= window_Index;	
					} /* NOT if (Fetch_NS_Window_Index_Using_WindowPeek(( ... */
					
				} /* if (event_Window == current_Graphics_Window_Ptr) */
				
				/*  */
				/* Update Specific Work Window */
				/*  */
				Update_Single_Non_TCL_Shell_Window(&event_Record);
				break;
				
				
				
			case diskEvt:
					Manage_Disk_Event(&event_Record);
				break;
				
				
				
			case activateEvt:
			
						
/* too many activates per second... why HACK HACK HACK ?*/


			
				event_Window = (WindowPtr)(event_Record.message);
	
				if (event_Window == current_Graphics_Window_Ptr) {
					g_Shell.interface.current_Graphics_Window	= window_Index;	
				}
				else {
					
					if (Fetch_NS_Window_Index_Using_WindowPeek((WindowPeek)event_Window
										, &window_Index_For_NS_Shell
										, (Handle *)&current_Window_Globals_Hdl) != SAFE_TRUE) {
						SysBeep(20); 
						Assign_String(ns_Shell_Globals.error_Description_String, "\pProgram flaw. Fetch_NS_Window_Index_Using_WindowPeek failed");
						Assign_String(ns_Shell_Globals.error_Note_String, "\pReboot Soon ?");
						Display_Error_Message(unit_Name, code_Name);
						Setup_Error_Handling(); 
						goto Error_Handler;
					}
					else {
						if (Get_Program_Window_Using_NS_Window_Index(window_Index_For_NS_Shell
									, &window_Index) != SAFE_TRUE) {
							SysBeep(20); 
							Assign_String(ns_Shell_Globals.error_Description_String, "\pProgram flaw. Get_Program_Index_Using_NS_Window_Index failed");
							Assign_String(ns_Shell_Globals.error_Note_String, "\pReboot Soon ?");
							Display_Error_Message(unit_Name, code_Name);
							Setup_Error_Handling(); 
							goto Error_Handler;
						}
								
						current_Graphics_Window_Ptr = event_Window;
						g_Shell.interface.current_Graphics_Window	= window_Index;	
	
					} /* NOT if (Fetch_NS_Window_Index_Using_WindowPeek(( ... */
					
				} /* NOT if (event_Window == current_Graphics_Window_Ptr) */
				
				if ((event_Record.modifiers & activeFlag) != 0) {
					/*  */
					/* Activate the window*/
					/*  */
					/* SelectWindow (event_Window); */
					
					Set_View_Port(event_Window); /* needed? */
					
					(**current_Window_Globals_Hdl).suspended = FALSE;
				}
				else {
					/*  */
					/* Mark window as deactivated*/
					/*  */
					(**current_Window_Globals_Hdl).suspended = TRUE;
				}				
				break;
	
			case app4Evt:
				if ((event_Record.message & 0x01000000) != 0) { 
						/*  */
						/* Suspend *OR* Resume*/
						/*  */
						
					if ((event_Record.message & 0x00000001) != 0) { 

						/*  */
						/* Resume, set the cursor and activate front window (none) */
						/*  */
						InitCursor();

											
						g_Shell.library.application_Is_Frontmost = SAFE_TRUE;
												
						if (current_Graphics_Window_Ptr != nil ) {
							if ((**active_Window_List[g_Shell.interface.current_Graphics_Window]).window_Is_Hidden != SAFE_TRUE) {
								ShowWindow(current_Graphics_Window_Ptr);
							}						
							Set_View_Port(current_Graphics_Window_Ptr);
						}
						
						if (Reset_Mouse_Settings_For_Spin_Window() != SAFE_TRUE) {
						
						}

					}
					else { 
						/*  */
						/* Suspend, deactivate the front window (none) */
						/*  */
						g_Shell.library.application_Is_Frontmost = SAFE_FALSE;
						
						/*  */
						/* If Cursor is not normal then reset it */
						/*  */
						if (last_Cursor_ID != 0) {
							Arrow_Cursor();
						}
						
						/*  */
						/* Some programs (Finder) do not set anti-Shield cursor or showcursor*/
						/* On desktop hit swap outs, though other intermediate process */
						/* manager swapping programs do. We might be in a spin file so */
						/* swap out */
						/*  */
						InitCursor();

				/*====			
						HideWindow(active_Window.window_List[g_Shell.interface.g_Shell.interface.current_Graphics_Window].window_Ptr);
					=== */		
					
					/*  */
					/*  */
					/*  */
					/*  */
					/*  */
				/*===
					Restore_Screen_Colors_To_Environment();
				===== */
					
					}
				}
				else {
					if ((event_Record.message & 0xFA000000) == 0xFA000000) { 
						/*  */
						/* Maintain Cursor, It has moved*/
						/*  */
						/* NULL */
					}
				}
				break;
				
							
			case kHighLevelEvent:
					/*  */
					/* EPPC offers "kHighLevelEvent = 23 " */
					/*  */
					Manage_High_Level_Event(&event_Record);
				break;
				
				
			default:
				break;
				
			} /* switch (event_Record.what) */
			
			if (g_Shell.interface.abort_Program == TRUE )
				g_Shell.interface.quit_Program = TRUE;
	
		} /* if (there_Is_An_Event_Waiting == TRUE)  */	
	
	} /* if ((check_For_Events == TRUE) && (g_Shell.interface.quit_Program != TRUE)) */
	
#if (k_USE_THINK_PROFILER == 1)
	_profile = TRUE;
#endif
	
	/*  */
	/* Process all the fast high priority stuff outside of the normal event loop */
	/*  */
	if (Perform_Periodic_Tasks() != SAFE_TRUE) {
		/*  */
		/* Ignore errors in this implementation */
		/*  */
	}
		
#if (k_USE_THINK_PROFILER == 1)
	_profile = FALSE;
#endif
		
	} /* while ( g_Shell.interface.quit_Program != TRUE) */
/*  */
			
#if (k_USE_THINK_PROFILER == 1)
	_profile = FALSE;
	 _trace = FALSE;
	DumpProfile();
#endif
			
	if (Close_Down_Program() != SAFE_TRUE) {
		/*  */
		/* Ignore error we are quitting */
		/*  */
	}


	function_Result = SAFE_TRUE;
	goto Normal_Function_Exit;
	
	
Error_Handler:
	function_Result = SAFE_FALSE;
	Display_Serious_Error_Message(unit_Name, code_Name, "\pError with Initialization of program", "\pQuitting.");


Normal_Function_Exit:

	return(function_Result);


} /* void Run_Mac_Version_Of_Program(void) */
/*=**************************************************************************=*/


/*=**************************************************************************=*/
Safe_Boolean Close_Down_Program(void)
/*  */
/* Order of destruction is important and is reverse order of creation */
/*  */
{
	CARDINAL_8		*code_Name;
	Safe_Boolean		function_Result;
	INTEGER_32		current_Window_Index;
/* END VAR */


	code_Name = "\pClose_Down_Program";
	function_Result = SAFE_FALSE;
			

	if (Stop_Recording_To_Any_Open_Audio_Files() != SAFE_TRUE) {
		/*  */
		/* Ignore errors in this implementation */
		/*  */
	}

	Shutdown_Phone_Bridge();

	if (Shutdown_Sound_Input_Monitoring() != SAFE_TRUE) {
		/*  */
		/* Ignore errors in this implementation */
		/*  */
	}

	if (Destroy_MOD_Utilities() != SAFE_TRUE) {
		/*  */
		/* Ignore errors in this implementation */
		/*  */
	}

	/*  */
	/* Stop network before windows are closed */
	/*  */
	Shutdown_Network_Zone_Management();
	
	/*  */
	/* Does nothing unless needed */
	/*  */
	Restore_Menu_Bar();

	/*  */
	/* This area closes down GENERAL_INFO_WINDOW_INDEX and all */
	/* other windows, releasing memory for niceness sake */
	/*  */
	for (current_Window_Index = 0
				; (current_Window_Index < TOTAL_DEFINED_WINDOW_TYPE)
				; current_Window_Index++ ) {
	
		if ((active_Window_List[current_Window_Index] != nil)
			&& ((**active_Window_List[current_Window_Index]).structure_Initialized == SAFE_TRUE)) {
		
			Close_Status_Window_Fully(active_Window_List[current_Window_Index]);
			
			if (Destroy_Status_Window_Structures(&active_Window_List[current_Window_Index]) != SAFE_TRUE) {
				g_Shell.interface.abort_Program = TRUE;
				
				Display_Serious_Error_Message(unit_Name, code_Name, "\pError Closing down window", "\pQuitting.");
				/*  */
				/* Ignore error we are quitting */
				/*  */
			}
			
		} /* if ((active_Window_List[current_Window_Index] != nil) ... */
		
	} /* for (current_Window_Index = 0 ... */

	if (Destroy_Command_Control() != SAFE_TRUE) {
		/*  */
		/* Ignore error we are quitting */
		/*  */
	}
	
	if (Destroy_Video_Backdrop_Background() != SAFE_TRUE) {
		/*  */
		/* Ignore error we are quitting */
		/*  */
	}
	
	if (Destroy_Spin_O_Rama() != SAFE_TRUE) {
		/*  */
		/* Ignore error we are quitting */
		/*  */
	}
	
	if (Destroy_Spin_O_Rama_Cache() != SAFE_TRUE) {
		/*  */
		/* Ignore error we are quitting */
		/*  */
	}
	
	if (Destroy_Slide_Show_Routines() != SAFE_TRUE) {
		/*  */
		/* Ignore error we are quitting */
		/*  */
	}
	
	if (Destroy_Command_Executor() != SAFE_TRUE) {
		/*  */
		/* Ignore error we are quitting */
		/*  */
	}
	
	if (Destroy_Command_Dialogs() != SAFE_TRUE) {
		/*  */
		/* Ignore error we are quitting */
		/*  */
	}
	
	if (Destroy_Command_Redirector() != SAFE_TRUE) {
		/*  */
		/* Ignore error we are quitting */
		/*  */
	}
	
	if (Destroy_Sound_Play() != SAFE_TRUE) {
		/*  */
		/* Ignore error we are quitting */
		/*  */
	}
	
	if (Destroy_Sound_Tap_Channel() != SAFE_TRUE) {
		/*  */
		/* Ignore error we are quitting */
		/*  */
	}

	if (Destroy_Dreaming_Audio_Record() != SAFE_TRUE) {
		/*  */
		/* Ignore error we are quitting */
		/*  */
	}
	
	if (Destroy_Sound_Record() != SAFE_TRUE) {
		/*  */
		/* Ignore error we are quitting */
		/*  */
	}
	
	if (Destroy_Object_Inventory() != SAFE_TRUE) {
		/*  */
		/* Ignore error we are quitting */
		/*  */
	}
	
	if (Destroy_Dreaming_Backdrop_Files() != SAFE_TRUE) {
		/*  */
		/* Ignore error we are quitting */
		/*  */
	}

	if (Destroy_Spin_Object_Files() != SAFE_TRUE) {
		/*  */
		/* Ignore error we are quitting */
		/*  */
	}
	
	if (Destroy_Simple_Ring_Cache() != SAFE_TRUE) {
		/*  */
		/* Ignore error we are quitting */
		/*  */
	}
	
	if (Destroy_Dreaming_Scene_Files() != SAFE_TRUE) {
		/*  */
		/* Ignore error we are quitting */
		/*  */
	}
	
	if (Destroy_Stage_View_Manager() != SAFE_TRUE) {
		/*  */
		/* Ignore error we are quitting */
		/*  */
	}

	if (Destroy_Physical_File_Mapping() != SAFE_TRUE) {
		/*  */
		/* Ignore error we are quitting */
		/*  */
	}

	if (Destroy_Remote_Control_Full() != SAFE_TRUE) {
		/*  */
		/* Ignore error we are quitting */
		/*  */
	}

	if (Destroy_Remote_Control_Standard() != SAFE_TRUE) {
		/*  */
		/* Ignore error we are quitting */
		/*  */
	}
	
	if (Destroy_Dream_Information() != SAFE_TRUE) {
		/*  */
		/* Ignore error we are quitting */
		/*  */
	}
		
	
	if (g_Digital_Audio_Card_In_Use == SAFE_TRUE ) {
		if (Stop_Sampling_From_Card() != SAFE_TRUE) {
		
		}
		Display_Serious_Error_Message(unit_Name, code_Name, "\pError, Program left digital audio card running", "\pQuitting anyway.");
	}

	
	function_Result = SAFE_TRUE;
	goto Normal_Function_Exit;
	
Error_Handler:
	Display_Serious_Error_Message(unit_Name, code_Name, "\pProblem encountered", "\p");
	function_Result = SAFE_FALSE;
		
Normal_Function_Exit:

	return(function_Result);
	
} /* Safe_Boolean Close_Down_Program(void) */
/*=**************************************************************************=*/


/*=**************************************************************************=*/
/* END OF FILE: Core_Specific.c */
