/* START OF FILE: Remote_Control_Full.c */


/*=**************************************************************************=*/
/* [---BEGIN---] FILE HEADER COMMENTS */
/**
 ** SCREEN VIEW = [Adobe Syntax Black-10, Tabs = 4]
 ** PRINTER VIEW = [Adobe Syntax Black-10, Tabs = 4]
 ** =======================================================================
 ** Written by James Merkle
 ** Remote_Control_Full.c - Accepts control from menu, keyboard, remote control packets, control bar
 ** (C) Copyright 1994 James Merkle
 ** All rights reserved, except those wishing to "copy" moderate sections of code or files may do so, so long as
 ** such code is never used directly either in animation-related tools or multimedia tools that are  
 ** used in live performance or live broadcast or recorded mass market broadcast or film. Desktop usage not restricted.
 ** Conventional desktop game or entertainment not restricted.
 ** =======================================================================
 **
 **
 ** ============
 ** Revision History
 ** ============
 **
 **  1994.02.01		(James Merkle)
 **						Initial coding started
 **
 **
 **
 **
 ** =================
 ** OTHER INFORMATION
 ** =================
 **  Attach "Proccess_Response_Queue_If_Needed" to periodic task loop!
 **
 **/
/* [----END----] FILE HEADER COMMENTS */
/*=**************************************************************************=*/


/*=**************************************************************************=*/
/* [---BEGIN---] #include AREA */

#include <AppleEvents.h>
#include <Processes.h>
#include <PPCToolBox.h>
#include <EPPC.h>
#include <Notification.h>

#include "Core_Globals.h"
#include "Core_Specific.h"
#include "OS_Utilities.h"
#include "String_Utilities.h"
#include "Command_Redirector.h"

#include "Slide_Show_Routines.h"
#include "Error_Utilities.h"
#include "Command_Executor.h"
#include "Remote_Control_Standard.h"
#include "Shell_Dialogs.h"

#include "Remote_Control_Full.h"


/* [----END----] #include AREA */
/*=**************************************************************************=*/


/*=**************************************************************************=*/
/* [---BEGIN---] #define AREA */

/*  */
/* This value is the number of simultaneous links to other apps */
/*  */
#define k_REMOTE_CONNECTIONS_MAX							1


/*  */
/* No harm in being too large or too small. 6 seems reasonable */
/*  */
#define k_MAX_OPTIONAL_KEYWORDS			(6L)

/*  */
/* Only two options sent in selectg slide slide */
/*  */
#define k_TOTAL_OPTIONS_FOR_SELECT			(3L)


#define k_SECONDS_FOR_HIGH_SPEED_ANSWER_REQUEST		(14L)
#define k_SECONDS_FOR_REGULAR_REQUEST				(14L)


/*  */
/* g_Outstanding_Resend_Atttempts incremented up to this limit */
/*  */
#define k_MAX_PPC_SEND_ATTEMPTS		5
#define k_MAX_BUFFERED_MESSAGES		10

/*  */
/* These are used in the PPC browser */
/*  */
#define k_PROGRAM_LINK_STRING "\pChoose a Simple Display program to link to:"
#define k_GENERAL_APPLICATION_LIST_STRING "\pPrograms"

/*  */
/* These are used in the PPC browser */
/*  */
#define k_PPC_AE_SIMPLE_CLASS				'misc'
#define k_PPC_AE_SIMPLE_RETURN_PATH	'dosc'

#define k_SELF_EVENT_CLASS								FILE_CREATOR
#define k_REMOTE_CONTROL_EVENT						(0x1200)
#define k_REMOTE_CONTROL_RESPONSE_EVENT		(0x1300)

#define k_CONNECTION_NONE										0
#define k_CONNECTION_REMOTE_PPC_APPLE_EVENT			1
#define k_CONNECTION_REMOTE_DATAGRAM_LISTENER		2
#define k_CONNECTION_REMOTE_MODEM_STREAM			3
#define k_CONNECTION_REMOTE_ATP_OR_ADSP_STYLE		4
#define k_CONNECTION_REMOTE_LOCAL_APPLE_EVENT		5
#define k_CONNECTION_REMOTE_SELF_TEST					6


#define k_COMMAND_SELECT_DIRECTION_TYPE				'Drm1'
#define k_COMMAND_SELECT_COMPOSITE_TYPE				'Drm2'
	
/*  */
/* There may be limits to maximum event item size for TEXT */
/* This limits how much memory we try to create for response list passing */
/*  */
#define k_MAXIMUM_ALLOWED_RAW_TEXT_MESSAGE		(32600L)

#define k_MASK_HIGH_FLAGS_FOR_SELECT_OVERLOAD				(0x40000000)
#define k_MASK_HIGH_FLAGS_FOR_CALLER_SELECT_OVERLOAD		(0x80000000)

/* [----END----] #define AREA */
/*=**************************************************************************=*/


/*=**************************************************************************=*/
/* [---BEGIN---] TYPE AREA */


typedef struct {
	TargetID				identifier;
	AEDesc				apple_Event_Descriptor;
} Remote_PPC_AE_Rec;

typedef struct {
	TargetID				identifier;
} Local_AE_Rec;

union Union_Connect {
	Remote_PPC_AE_Rec	remote_PPC;
	Local_AE_Rec				local_Connect;
};

typedef struct {
	Safe_Boolean		connected_Once;
	Safe_Boolean		connected_Now;
	Safe_Boolean		needs_Cleanup_At_Quit;
	CARDINAL_16	connection_Type;
	CARDINAL_32	last_Contact_Tick_Time;
	union Union_Connect		connect;
} Remote_Connect_Record, *Remote_Connect_Ptr;

/* [----END----] TYPE AREA */
/*=**************************************************************************=*/


/*=**************************************************************************=*/
/* [---BEGIN---] PROTOTYPE AREA */
															

pascal OSErr Handler_For_Remote_Command_Apple_Event(
						AppleEvent *answer_Document_Apple_Event
						, AppleEvent *reply
						, INTEGER_32 handlerRefCon);
						
pascal OSErr Handler_For_Remote_Response_Apple_Event(
						AppleEvent *answer_Document_Apple_Event
						, AppleEvent *reply
						, INTEGER_32 handlerRefCon);
						
OSErr Initialize_Special_AE_Handlers(void);

Safe_Boolean Connect_Remote_If_Possible(Remote_Connect_Ptr remote_Info, Safe_Boolean *connected);

Safe_Boolean Connect_Remote_Using_Apple_Events(Remote_Connect_Ptr	remote_Info
															, Safe_Boolean *connected);

Safe_Boolean Disconnect_Remote_Completely(Remote_Connect_Ptr		remote_Info);

Safe_Boolean Send_Message_PPC_AE(AEDesc  *apple_Event_Descriptor
													, CARDINAL_16		message_Type
													, Ptr parameters
													, Safe_Boolean *answered);

Safe_Boolean Affirm_Connection_With_Response(Remote_Connect_Ptr		remote_Info
													, Safe_Boolean *answered);

Safe_Boolean Send_Remote_Command(Remote_Connect_Ptr		remote_Info
													, CARDINAL_32	command
													, Ptr parameters
													, Safe_Boolean *answered);

Safe_Boolean Add_Response_To_Response_Queue( CARDINAL_32 reply_Message, TargetID *address_Identifier);

/*  */
/* The following are specialized commands for this implementation */
/*  */


Safe_Boolean Add_Parameters_For_Command_Select(AEDesc			*optional_Parameter_List
															, CARDINAL_32	*optional_Parameter_List_Length
															, CARDINAL_32	maximum_Available
															, AppleEvent		*message
															, AEKeyword		*optional_Parameter_Keywords);


Safe_Boolean Fill_Parameters_For_Command_Select(CARDINAL_32 *optional_Parameter_List_Length
														, AppleEvent			*message
														, CARDINAL_32		*optional_Arguments_Length
														, AEDesc				*extra_Optional_Argument
														, Safe_Boolean		*extra_Optional_Argument_Initialized
														, AEKeyword			*optional_Parameter_Keywords
														, CARDINAL_16 		message_Type
														, Ptr 					parameters);


Safe_Boolean Add_Parameters_For_Response_Select(AEDesc			*optional_Parameter_List
															, CARDINAL_32	*optional_Parameter_List_Length
															, CARDINAL_32	maximum_Available
															, AppleEvent		*message
															, AEKeyword		*optional_Parameter_Keywords);

Safe_Boolean Fill_Parameters_For_Response_Select(CARDINAL_32 *optional_Parameter_List_Length
														, AppleEvent			*message
														, CARDINAL_32		*optional_Arguments_Length
														, AEDesc				*extra_Optional_Arguments
														, Safe_Boolean		*extra_Optional_Argument_Initialized
														, AEKeyword			*optional_Parameter_Keywords
														, CARDINAL_16 		message_Type);

/* [----END----] PROTOTYPE AREA */
/*=**************************************************************************=*/


/*=**************************************************************************=*/
/* [---BEGIN---] GLOBAL DEFINITION AREA */


/* [----END----] GLOBAL DEFINITION AREA */
/*=**************************************************************************=*/


/*=**************************************************************************=*/
/* [---BEGIN---] LOCAL STATIC DEFINITION AREA */

static CARDINAL_8	*unit_Name;

Safe_Boolean	g_Possibly_Connected;
Remote_Connect_Record	g_Remote_Connect[k_REMOTE_CONNECTIONS_MAX];

CARDINAL_32	g_Total_Outstanding_Messages_Total;
TargetID			g_Outstanding_Message_Targets[k_MAX_BUFFERED_MESSAGES];
CARDINAL_32	g_Outstanding_Messages[k_MAX_BUFFERED_MESSAGES];
CARDINAL_32	g_Outstanding_Resend_Atttempts[k_MAX_BUFFERED_MESSAGES];


/* [----END----] LOCAL STATIC DEFINITION AREA */
/*=**************************************************************************=*/


/*=**************************************************************************=*/
Safe_Boolean Initialize_Remote_Control_Full(void)
{
	CARDINAL_8			*static_Unit_Name = "\pInitialize_Remote_Control_Full";
	Safe_Boolean			function_Result;
	INTEGER_16			loop;
/* END VAR */

/*  */
/* Global persistant structures are created here */
/*  */

	function_Result = SAFE_FALSE;
	unit_Name = static_Unit_Name;
	

	/*  */
	/* Its vital to clear outgoing list of messages */
	/*  */
	g_Total_Outstanding_Messages_Total = 0;
	for (loop = 0; (loop < k_MAX_BUFFERED_MESSAGES); loop++) {
		g_Outstanding_Messages[loop] = 0;
		g_Outstanding_Resend_Atttempts[loop] = 0;
	}
	
	if (Initialize_Special_AE_Handlers() != noErr) {
		/*  */
		/* Ignore error in this version. Not expected to cause trouble */
		/* by design. (Not Fatal if AE not available) */
		/*  */
	}
	
	if (Disable_Remote_Menu() != SAFE_TRUE) {
		goto Error_Handler;
	}
	
	g_Possibly_Connected = SAFE_FALSE;
	
	function_Result = SAFE_TRUE;
	goto Normal_Function_Exit;
	
Error_Handler:
	function_Result = SAFE_FALSE;

Normal_Function_Exit:

	return(function_Result);
}
/*=**************************************************************************=*/


/*=**************************************************************************=*/
Safe_Boolean Setup_Remote_Control_Full(void)
{
	CARDINAL_8			*static_Unit_Name = "\pRemote_Control_Full";
	Safe_Boolean 		connected;
	Safe_Boolean 		result;
	Safe_Boolean			function_Result;
	Safe_Boolean 		answered;
	INTEGER_16			loop;
	Boolean				still_Connecting;
	Boolean				current_Iteration_Set_Global;
/* END VAR */

/*  */
/* Global persistant structures are created here */
/*  */

	function_Result = SAFE_FALSE;
	unit_Name = static_Unit_Name;
	

	/*  */
	/* Paranoia reset if needed */
	/*  */
	if (g_Possibly_Connected == SAFE_TRUE) {
		if (Destroy_Remote_Control_Full() != SAFE_TRUE) {
			/*  */
			/* Ignore problems */
			/*  */
		}
	}

	g_Possibly_Connected = SAFE_FALSE;

	still_Connecting = TRUE;
	for (loop=0; (loop < k_REMOTE_CONNECTIONS_MAX); loop++) {
		result = Connect_Remote_If_Possible(&(g_Remote_Connect[loop]), &connected);
		if (result != SAFE_TRUE) {
			still_Connecting = FALSE;
			/*  */
			/* Heed error as if it was fatal. Only fatal errors passed back */
			/*  */
			goto Error_Handler;
		}
		else {
			if (connected == SAFE_TRUE) {
				/*  */
				/* We got at least one so far! */
				/* Double verify by 'pinging it once' */
				/*  */
				current_Iteration_Set_Global = FALSE;
				if (g_Possibly_Connected != SAFE_TRUE) {
					g_Possibly_Connected = SAFE_TRUE;
					current_Iteration_Set_Global = TRUE;
				}

				if (Affirm_Connection_With_Response(&(g_Remote_Connect[loop])
												, &answered) != SAFE_TRUE) {
					/*  */
					/* Heed error as if it was fatal. Only fatal errors passed back */
					/*  */
					if (current_Iteration_Set_Global == TRUE) {
						g_Possibly_Connected = SAFE_FALSE;
					}
					goto Error_Handler;
				}

				if (answered != SAFE_TRUE) {
					if (Disconnect_Remote_Completely(&(g_Remote_Connect[loop])) != SAFE_TRUE) {
						/*  */
						/* Ignore norifying about problems, we are quitting anyway */
						/*  */
						; /* NULL */
					}
					if (current_Iteration_Set_Global == TRUE) {
						g_Possibly_Connected = SAFE_FALSE;
					}
				}

			} /* if (connected == SAFE_TRUE) */
			
			still_Connecting = FALSE;
		}

	} /* for (loop=0; ... */
	

	if (g_Possibly_Connected == SAFE_TRUE) {
		if (Enable_Remote_Menu() != SAFE_TRUE) {
			/*  */
			/* Ignore error in this version. */
			/*  */
		}
	}
	
	function_Result = SAFE_TRUE;
	goto Normal_Function_Exit;
	
Error_Handler:
	function_Result = SAFE_FALSE;

Normal_Function_Exit:

	return(function_Result);
}
/*=**************************************************************************=*/


/*=**************************************************************************=*/
Safe_Boolean Destroy_Remote_Control_Full(void)
{
	Safe_Boolean		function_Result;
	INTEGER_16		loop;
/* END VAR */

	/*  */
	/* Global persistant structures are deleted here */
	/*  */
	
	/*  */
	/* Close all connections. Dispose AE communication structures */
	/*  */
	if (g_Possibly_Connected == SAFE_TRUE) {
	
		if (Disable_Remote_Menu() != SAFE_TRUE) {
			/*  */
			/* Ignore error in this version. */
			/*  */
		}
		
		for (loop=0; (loop < k_REMOTE_CONNECTIONS_MAX); loop++) {
	
			if (Disconnect_Remote_Completely(&(g_Remote_Connect[loop])) != SAFE_TRUE) {
				/*  */
				/* Ignore norifying about problems, we are quitting anyway */
				/*  */
				; /* NULL */
			}
		}
	}
	g_Possibly_Connected = SAFE_FALSE;
	
	function_Result = SAFE_TRUE;
	goto Normal_Function_Exit;
	
Error_Handler:
	function_Result = SAFE_FALSE;

Normal_Function_Exit:

	return(function_Result);
}
/*=**************************************************************************=*/


/*=**************************************************************************=*/
Safe_Boolean Enable_Remote_Menu(void)
{
	Safe_Boolean			function_Result;
/* END VAR */


	DisableItem(g_Shell.interface.program_Menus[NETWORK_MENU] , MENU_COMMAND_CONNECT);
	
	EnableItem(g_Shell.interface.program_Menus[NETWORK_MENU] , MENU_COMMAND_DISCONNECT);
	EnableItem(g_Shell.interface.program_Menus[NETWORK_MENU] , MENU_COMMAND_REMOTE_SELECT_SLIDE);
	EnableItem(g_Shell.interface.program_Menus[NETWORK_MENU] , MENU_COMMAND_REMOTE_SELECT_NEXT_SLIDE);
	EnableItem(g_Shell.interface.program_Menus[NETWORK_MENU] , MENU_COMMAND_REMOTE_SELECT_PREVIOUS_SLIDE);
	EnableItem(g_Shell.interface.program_Menus[NETWORK_MENU] , MENU_COMMAND_REMOTE_HIDE_WINDOW);
	EnableItem(g_Shell.interface.program_Menus[NETWORK_MENU] , MENU_COMMAND_REMOTE_RESCAN_FILES);
	EnableItem(g_Shell.interface.program_Menus[NETWORK_MENU] , MENU_COMMAND_REMOTE_UPDATE_LIST);
	EnableItem(g_Shell.interface.program_Menus[NETWORK_MENU] , MENU_COMMAND_REMOTE_QUIT);
	
	function_Result = SAFE_TRUE;
	return(function_Result);

}
/*=**************************************************************************=*/


/*=**************************************************************************=*/
Safe_Boolean Disable_Remote_Menu(void)
{
	Safe_Boolean			function_Result;
/* END VAR */


	EnableItem(g_Shell.interface.program_Menus[NETWORK_MENU] , MENU_COMMAND_CONNECT);
	
	DisableItem(g_Shell.interface.program_Menus[NETWORK_MENU] , MENU_COMMAND_DISCONNECT);
	DisableItem(g_Shell.interface.program_Menus[NETWORK_MENU] , MENU_COMMAND_REMOTE_SELECT_SLIDE);
	DisableItem(g_Shell.interface.program_Menus[NETWORK_MENU] , MENU_COMMAND_REMOTE_SELECT_NEXT_SLIDE);
	DisableItem(g_Shell.interface.program_Menus[NETWORK_MENU] , MENU_COMMAND_REMOTE_SELECT_PREVIOUS_SLIDE);
	DisableItem(g_Shell.interface.program_Menus[NETWORK_MENU] , MENU_COMMAND_REMOTE_HIDE_WINDOW);
	DisableItem(g_Shell.interface.program_Menus[NETWORK_MENU] , MENU_COMMAND_REMOTE_RESCAN_FILES);
	DisableItem(g_Shell.interface.program_Menus[NETWORK_MENU] , MENU_COMMAND_REMOTE_UPDATE_LIST);
	DisableItem(g_Shell.interface.program_Menus[NETWORK_MENU] , MENU_COMMAND_REMOTE_QUIT);
	
	function_Result = SAFE_TRUE;
	return(function_Result);

}
/*=**************************************************************************=*/


/*=**************************************************************************=*/
Safe_Boolean Disconnect_Remote_Completely(Remote_Connect_Ptr		remote_Info)
{
	OSErr				os_Error;
	Safe_Boolean		function_Result;
	PPCClosePBRec	close_Parameters;
/* END VAR */


	function_Result = SAFE_FALSE;

	if (remote_Info->connected_Once == SAFE_TRUE) {
		if (remote_Info->needs_Cleanup_At_Quit == SAFE_TRUE) {
			switch(remote_Info->connection_Type) {
			
				case k_CONNECTION_REMOTE_PPC_APPLE_EVENT:

/* ====
				close_Parameters.portRefNum = remote_Info->connect.remote_PPC....
				os_Error = PPCClose(&close_Parameters, FALSE);
	            	if (os_Error == noErr) {

				}
====== */

				os_Error = AEDisposeDesc(&(remote_Info->connect.remote_PPC.apple_Event_Descriptor));
				if (os_Error != noErr) {
					goto Error_Handler;
				}
				break;
				
				case k_CONNECTION_NONE:
				case k_CONNECTION_REMOTE_DATAGRAM_LISTENER:
				case k_CONNECTION_REMOTE_MODEM_STREAM:
				case k_CONNECTION_REMOTE_ATP_OR_ADSP_STYLE:
				case k_CONNECTION_REMOTE_LOCAL_APPLE_EVENT:
				case k_CONNECTION_REMOTE_SELF_TEST:
				break;
				
				default:
					goto Error_Handler;
			}	
		}
	}

	function_Result = SAFE_TRUE;
	goto Normal_Function_Exit;
	
Error_Handler:
	function_Result = SAFE_FALSE;

Normal_Function_Exit:

	return(function_Result);
	
} /* Safe_Boolean Disconnect_Remote_For_Shutdown ... */
/*=**************************************************************************=*/


/*=**************************************************************************=*/
Safe_Boolean Add_Response_To_Response_Queue( CARDINAL_32 reply_Message, TargetID *address_Identifier)
{
	Safe_Boolean		function_Result;
/* END VAR */


	function_Result = SAFE_FALSE;
		
	if (g_Total_Outstanding_Messages_Total < k_MAX_BUFFERED_MESSAGES) {
		g_Outstanding_Messages[g_Total_Outstanding_Messages_Total] = reply_Message;
		g_Outstanding_Message_Targets[g_Total_Outstanding_Messages_Total] = *address_Identifier;	
		g_Outstanding_Resend_Atttempts[g_Total_Outstanding_Messages_Total] = 0;
		g_Total_Outstanding_Messages_Total += 1;
	}
	
	function_Result = SAFE_TRUE;
	goto Normal_Function_Exit;
	
Error_Handler:
	function_Result = SAFE_FALSE;

Normal_Function_Exit:

	return(function_Result);
}
/*=**************************************************************************=*/


/*=**************************************************************************=*/
Safe_Boolean Proccess_Response_Queue_If_Needed(void)
{
	OSErr				os_Error;
	Safe_Boolean 	response_Sent;
	Safe_Boolean		function_Result;
	Safe_Boolean		reply_Event_Memory_Initialized;
	AEDesc 			apple_Event_Reply;
	CARDINAL_32	reply_Message;
/* END VAR */


	function_Result = SAFE_FALSE;
	reply_Event_Memory_Initialized = SAFE_FALSE;

	if ((g_Possibly_Connected == SAFE_TRUE) && (g_Total_Outstanding_Messages_Total != 0)) {
		
		reply_Message = g_Outstanding_Messages[(g_Total_Outstanding_Messages_Total - 1)];

		apple_Event_Reply.descriptorType = typeNull;
		
		/*  */
		/* Check if message regarding target */
		/*  */

		os_Error = AECreateDesc(typeTargetID
									, (Ptr)&(g_Outstanding_Message_Targets[(g_Total_Outstanding_Messages_Total - 1)])
									, sizeof(g_Outstanding_Message_Targets[(g_Total_Outstanding_Messages_Total - 1)])
									, &(apple_Event_Reply));

		if (os_Error == noErr) {
			if (apple_Event_Reply.descriptorType != typeNull) {
				reply_Event_Memory_Initialized = SAFE_TRUE;

				/*  */
				/* Reply messages are the best place to return program status */
				/* as well as indicate success of command */
				/* For example it can acknowledge raw command to select */
				/* the next item by actually returning what the item selected is */
				/*  */
				if (Send_Message_PPC_AE(&apple_Event_Reply
								, reply_Message
								, nil
								, &response_Sent) != SAFE_TRUE) {
																
					/*  */
					/* Responses are permissible to fail, though not probable */
					/* Response left in queue however to resend */
					/*  */
					if (g_Outstanding_Resend_Atttempts[(g_Total_Outstanding_Messages_Total - 1)] > k_MAX_PPC_SEND_ATTEMPTS) {
						g_Total_Outstanding_Messages_Total = (g_Total_Outstanding_Messages_Total - 1);
	
						SysBeep(20);
					/* ===
						function_Result = errAEEventNotHandled;
						goto Error_Handler;
					==== */
	
					}
					else {
						/*  */
						/* Increment failure count */
						/*  */
						g_Outstanding_Resend_Atttempts[(g_Total_Outstanding_Messages_Total - 1)] += 1;
					}
				}
				else {				
					g_Total_Outstanding_Messages_Total = (g_Total_Outstanding_Messages_Total - 1);
				}
			}
		}
	}
	
	function_Result = SAFE_TRUE;
	goto Normal_Function_Exit;
	
Error_Handler:
	function_Result = SAFE_FALSE;

Normal_Function_Exit:

	if (reply_Event_Memory_Initialized == SAFE_TRUE) {
		os_Error = AEDisposeDesc(&apple_Event_Reply);
		if (os_Error != noErr) {
			; /* NULL */
		}	
	}

	return(function_Result);

} /* Safe_Boolean Proccess_Response_Queue_If_Needed */
/*=**************************************************************************=*/


/*=**************************************************************************=*/
#pragma mark -
Safe_Boolean Add_Parameters_For_Command_Select(AEDesc			*optional_Parameter_List
															, CARDINAL_32	*optional_Parameter_List_Length
															, CARDINAL_32	maximum_Available
															, AppleEvent		*message
															, AEKeyword		*optional_Parameter_Keywords)
{
	Safe_Boolean		function_Result;
	Boolean			succeded;
	INTEGER_16		loop;
	OSErr				os_Error;
/* END VAR */


	function_Result = SAFE_FALSE;
	
	/*  */
	/* Add an OPTIONAL parameter description to list */
	/* We would nomally loop through this stage multiple times per */
	/* optional attribute */
	/*  */
	
	/*  */
	/* Safety Iniitalize */
	/*  */
	for (loop = 0; (loop < k_TOTAL_OPTIONS_FOR_SELECT); loop++) {
		optional_Parameter_Keywords[loop] = keyDirectObject;
	}
	
	/*  */
	/* Setup list of optional stuff to extract */
	/*  */
	optional_Parameter_Keywords[0] = keyDirectObject;
	optional_Parameter_Keywords[1] = k_COMMAND_SELECT_DIRECTION_TYPE;
	optional_Parameter_Keywords[2] = k_COMMAND_SELECT_COMPOSITE_TYPE;
	
	
	succeded = TRUE;
	for (loop = 0; ((loop < k_TOTAL_OPTIONS_FOR_SELECT) && (succeded == TRUE)); loop++) {
		succeded = FALSE;

		
		if (*optional_Parameter_List_Length >= maximum_Available) {
			/*  */
			/* Too many parameters for object? Tumble out */
			/*  */
			; /* NULL */
		}
		else {
						
			os_Error = AEPutPtr(optional_Parameter_List
									, (*optional_Parameter_List_Length + 1)
									, typeKeyword
									, &(optional_Parameter_Keywords[*optional_Parameter_List_Length])
									, sizeof((optional_Parameter_Keywords[*optional_Parameter_List_Length])));
									
			*optional_Parameter_List_Length += 1;

			if (os_Error == noErr) {
				succeded = TRUE;

			} /* NOT if (os_Error != noErr) */
		
		} /* if (optional_Parameter_List_Length... */
		
	} /* for (loop = 0 */
		
	if (succeded == TRUE) {
		/*  */
		/* Add optional list */
		/*  */
		os_Error = AEPutAttributeDesc(message
											, keyOptionalKeywordAttr
											, optional_Parameter_List);
	
		if (os_Error == noErr) {
			function_Result = SAFE_TRUE;
		}	
	}
	
	goto Normal_Function_Exit;
	
Error_Handler:
	function_Result = SAFE_FALSE;

Normal_Function_Exit:

	return(function_Result);
	
} /* Safe_Boolean Add_Parameters_For_Command_Select ... */
/*=**************************************************************************=*/


/*=**************************************************************************=*/
Safe_Boolean Fill_Parameters_For_Command_Select(CARDINAL_32 *optional_Parameter_List_Length
														, AppleEvent			*message
														, CARDINAL_32		*optional_Arguments_Length
														, AEDesc				*extra_Optional_Argument
														, Safe_Boolean		*extra_Optional_Argument_Initialized
														, AEKeyword			*optional_Parameter_Keywords
														, CARDINAL_16 		message_Type
														, Ptr 					parameters)
{
	Safe_Boolean		function_Result;
	Boolean			succeded;
	CARDINAL_32	optional_Test_Value;
	CARDINAL_32	optional_Direction_Value;
	Str255				optional_Test_String;
	OSErr				os_Error;
/* END VAR */


	function_Result = SAFE_FALSE;
	succeded = TRUE;

	if (succeded == TRUE) {
		succeded = FALSE;
		optional_Test_String[0] = 0;
		Copy_Pascal_String("\pThis is a test", &optional_Test_String[0]);
				
		succeded = FALSE;
		os_Error = AECreateDesc(typeChar
										, (Ptr)&optional_Test_String[1]
										, optional_Test_String[0]
										, &(extra_Optional_Argument[*optional_Arguments_Length]));
									
		if (os_Error != noErr) {
			/*  */
			/* wrong parameter length of object? Tumble out */
			/*  */
			; /* NULL */
		}
		else {
			extra_Optional_Argument_Initialized[*optional_Arguments_Length] = SAFE_TRUE;
			*optional_Arguments_Length += 1;
			
			if (*optional_Arguments_Length <= *optional_Parameter_List_Length) {
				/*  */
				/* create chunk of memory for optional attrribute and add to event to send */
				/*  */
				os_Error = AEPutParamDesc(message
											, optional_Parameter_Keywords[2]
											, &(extra_Optional_Argument[*optional_Arguments_Length  - 1]));
			
				if (os_Error == noErr) {
					succeded = TRUE;
				}
			}
								
		} /* NOT if (os_Error != noErr) */
			
			
	} /* if (succeded == TRUE) */														
														
	if (succeded == TRUE) {
	
		optional_Test_Value = FILE_CREATOR;
		succeded = FALSE;
		os_Error = AECreateDesc(typeLongInteger
										, (Ptr)&optional_Test_Value
										, sizeof(optional_Test_Value)
										, &(extra_Optional_Argument[*optional_Arguments_Length]));
									
		if (os_Error != noErr) {
			/*  */
			/* wrong parameter length of object? Tumble out */
			/*  */
			; /* NULL */
		}
		else {
			extra_Optional_Argument_Initialized[*optional_Arguments_Length] = SAFE_TRUE;
			*optional_Arguments_Length += 1;
			
			if (*optional_Arguments_Length <= *optional_Parameter_List_Length) {
				/*  */
				/* create chunk of memory for optional attrribute and add to event to send */
				/*  */
				os_Error = AEPutParamDesc(message
											, optional_Parameter_Keywords[0]
											, &(extra_Optional_Argument[*optional_Arguments_Length  - 1]));
			
				if (os_Error == noErr) {
					succeded = TRUE;
				}
			}
								
		} /* NOT if (os_Error != noErr) */
	
	} /* if (succeded == TRUE) */

	
	if (succeded == TRUE) {
		succeded = FALSE;
		
		optional_Direction_Value = 0;
		
		switch (message_Type) {
			case k_SELECT_NEXT_ITEM_AE:
				optional_Direction_Value = 1;
			break;
			
			case k_SELECT_PREVIOUS_ITEM_AE:
				optional_Direction_Value = 0;
			break;
			
			case k_SELECT_ITEM_AE:
				optional_Direction_Value = k_MASK_HIGH_FLAGS_FOR_SELECT_OVERLOAD;
				optional_Direction_Value |= ((Select_Remote_Rec *)parameters)->desired_Slide;
			break;

			case k_SELECT_CALLER_ITEM_AE:
				optional_Direction_Value = k_MASK_HIGH_FLAGS_FOR_CALLER_SELECT_OVERLOAD;
				optional_Direction_Value |= ((Select_Caller_Scene_Rec *)parameters)->desired_Caller;
			break;
			
			default:
				/*  */
				/* Should just quit if not accounted for */
				/*  */
				goto Error_Handler;
		}		
				
		succeded = FALSE;
		os_Error = AECreateDesc(typeLongInteger
										, (Ptr)&optional_Direction_Value
										, sizeof(optional_Direction_Value)
										, &(extra_Optional_Argument[*optional_Arguments_Length]));
									
		if (os_Error != noErr) {
			/*  */
			/* wrong parameter length of object? Tumble out */
			/*  */
			; /* NULL */
		}
		else {
			extra_Optional_Argument_Initialized[*optional_Arguments_Length] = SAFE_TRUE;
			*optional_Arguments_Length += 1;
			
			if (*optional_Arguments_Length <= *optional_Parameter_List_Length) {
				/*  */
				/* create chunk of memory for optional attrribute and add to event to send */
				/*  */
				os_Error = AEPutParamDesc(message
											, optional_Parameter_Keywords[1]
											, &(extra_Optional_Argument[*optional_Arguments_Length  - 1]));
			
				if (os_Error == noErr) {
					succeded = TRUE;
				}
			}
								
		} /* NOT if (os_Error != noErr) */
			
			
	} /* if (succeded == TRUE) */
										

	if (succeded == TRUE) {
		function_Result = SAFE_TRUE;
	}
	
	goto Normal_Function_Exit;
	
Error_Handler:
	function_Result = SAFE_FALSE;

Normal_Function_Exit:

	return(function_Result);

} /* Safe_Boolean Fill_Parameters_For_Command_Select ... */
/*=**************************************************************************=*/


/*=**************************************************************************=*/
Safe_Boolean Add_Parameters_For_Response_Select(AEDesc			*optional_Parameter_List
															, CARDINAL_32	*optional_Parameter_List_Length
															, CARDINAL_32	maximum_Available
															, AppleEvent		*message
															, AEKeyword		*optional_Parameter_Keywords)
{
	Safe_Boolean		function_Result;
	Boolean			succeded;
	INTEGER_16		loop;
	OSErr				os_Error;
/* END VAR */


	function_Result = SAFE_FALSE;
	
	/*  */
	/* Add an OPTIONAL parameter description to list */
	/* We would nomally loop through this stage multiple times per */
	/* optional attribute */
	/*  */
	
	/*  */
	/* Safety Iniitalize */
	/*  */
	for (loop = 0; (loop < k_TOTAL_OPTIONS_FOR_SELECT); loop++) {
		optional_Parameter_Keywords[loop] = keyDirectObject;
	}
	
	/*  */
	/* Setup list of optional stuff to extract */
	/*  */
	optional_Parameter_Keywords[0] = keyDirectObject;
	optional_Parameter_Keywords[1] = k_COMMAND_SELECT_DIRECTION_TYPE;
	optional_Parameter_Keywords[2] = k_COMMAND_SELECT_COMPOSITE_TYPE;
	
	
	succeded = TRUE;
	for (loop = 0; ((loop < k_TOTAL_OPTIONS_FOR_SELECT) && (succeded == TRUE)); loop++) {
		succeded = FALSE;

		
		if (*optional_Parameter_List_Length >= maximum_Available) {
			/*  */
			/* Too many parameters for object? Tumble out */
			/*  */
			; /* NULL */
		}
		else {
						
			os_Error = AEPutPtr(optional_Parameter_List
									, (*optional_Parameter_List_Length + 1)
									, typeKeyword
									, &(optional_Parameter_Keywords[*optional_Parameter_List_Length])
									, sizeof((optional_Parameter_Keywords[*optional_Parameter_List_Length])));
									
			*optional_Parameter_List_Length += 1;

			if (os_Error == noErr) {
				succeded = TRUE;

			} /* NOT if (os_Error != noErr) */
		
		} /* if (optional_Parameter_List_Length... */
		
	} /* for (loop = 0 */
		
	if (succeded == TRUE) {
		/*  */
		/* Add optional list */
		/*  */
		os_Error = AEPutAttributeDesc(message
											, keyOptionalKeywordAttr
											, optional_Parameter_List);
	
		if (os_Error == noErr) {
			function_Result = SAFE_TRUE;
		}	
	}
	
	goto Normal_Function_Exit;
	
Error_Handler:
	function_Result = SAFE_FALSE;

Normal_Function_Exit:

	return(function_Result);
	
} /* Safe_Boolean Add_Parameters_For_Response_Select ... */
/*=**************************************************************************=*/


/*=**************************************************************************=*/
Safe_Boolean Fill_Parameters_For_Response_Select(CARDINAL_32 *optional_Parameter_List_Length
														, AppleEvent			*message
														, CARDINAL_32		*optional_Arguments_Length
														, AEDesc				*extra_Optional_Argument
														, Safe_Boolean		*extra_Optional_Argument_Initialized
														, AEKeyword			*optional_Parameter_Keywords
														, CARDINAL_16 		message_Type)
{
	Safe_Boolean		function_Result;
	Safe_Boolean		creation_Result;
	Boolean			succeded;
	CARDINAL_32	optional_Test_Value;
	CARDINAL_32	optional_Direction_Value;
	CARDINAL_32	preferred_Size;
	CARDINAL_32	available_Bytes;
	CARDINAL_32 	utilized_Bytes;
	OSErr				os_Error;
	Handle					temp_List_Hdl;
	Memory_Allocation		temp_List_Memory_Initialized;
	CARDINAL_8				*code_Name;
/* END VAR */



	code_Name = "\pFill_Parameters_For_Response_Select";
	function_Result = SAFE_FALSE;
	succeded = TRUE;
	temp_List_Memory_Initialized = MEMORY_NOT_INITIALIZED;

	if (succeded == TRUE) {
		succeded = FALSE;
		
		temp_List_Hdl = nil;
		preferred_Size = k_MAXIMUM_ALLOWED_RAW_TEXT_MESSAGE;
		if (Create_Handle(&preferred_Size
							, (10240)
							, DISALLOW_PURGING
							, DISALLOW_PARTIAL_AMOUNT
							, (Handle *)&temp_List_Hdl
							, &temp_List_Memory_Initialized) != SAFE_TRUE) {
							
			Display_Low_Memory_Error_Message(unit_Name, code_Name, k_MAXIMUM_ALLOWED_RAW_TEXT_MESSAGE);
			function_Result = SAFE_FALSE;
			goto Normal_Function_Exit;
		}
		
/* ===		
		if (Create_Handle(&preferred_Size
							, (10240)
							, ALLOW_PURGING
							, ALLOW_PARTIAL_AMOUNT
							, (Handle *)&temp_List_Hdl
							, &temp_List_Memory_Initialized) != SAFE_TRUE) {
							
			Display_Low_Memory_Error_Message(unit_Name, code_Name, k_MAXIMUM_ALLOWED_RAW_TEXT_MESSAGE);
			function_Result = SAFE_FALSE;
			goto Normal_Function_Exit;
		}
	=== */
		
		/*  */
		/* We took biggest we could (up to our limit). Adjust. */
		/*  */
		available_Bytes = preferred_Size;
		
		/*  */
		/* No reason to lock handle if no segment loading, and routine trusted */
		/*  */
		creation_Result = Create_Text_List_Of_Slides((CARDINAL_8 *)*temp_List_Hdl
											, available_Bytes
											, &utilized_Bytes);
		
		if (creation_Result != SAFE_TRUE) {
			goto Error_Handler;
		}

		succeded = FALSE;
		os_Error = AECreateDesc(typeChar
										, (Ptr)*temp_List_Hdl
										, utilized_Bytes
										, &(extra_Optional_Argument[*optional_Arguments_Length]));
									
		if (os_Error != noErr) {
			/*  */
			/* wrong parameter length of object? Tumble out */
			/*  */
			; /* NULL */
		}
		else {
			extra_Optional_Argument_Initialized[*optional_Arguments_Length] = SAFE_TRUE;
			*optional_Arguments_Length += 1;
			
			if (*optional_Arguments_Length <= *optional_Parameter_List_Length) {
				/*  */
				/* create chunk of memory for optional attrribute and add to event to send */
				/*  */
				os_Error = AEPutParamDesc(message
											, optional_Parameter_Keywords[2]
											, &(extra_Optional_Argument[*optional_Arguments_Length  - 1]));
			
				if (os_Error == noErr) {
					succeded = TRUE;
				}
			}
								
		} /* NOT if (os_Error != noErr) */
						
	} /* if (succeded == TRUE) */														
														
	if (succeded == TRUE) {
	
		optional_Test_Value = FILE_CREATOR;
		succeded = FALSE;
		os_Error = AECreateDesc(typeLongInteger
										, (Ptr)&optional_Test_Value
										, sizeof(optional_Test_Value)
										, &(extra_Optional_Argument[*optional_Arguments_Length]));
									
		if (os_Error != noErr) {
			/*  */
			/* wrong parameter length of object? Tumble out */
			/*  */
			; /* NULL */
		}
		else {
			extra_Optional_Argument_Initialized[*optional_Arguments_Length] = SAFE_TRUE;
			*optional_Arguments_Length += 1;
			
			if (*optional_Arguments_Length <= *optional_Parameter_List_Length) {
				/*  */
				/* create chunk of memory for optional attrribute and add to event to send */
				/*  */
				os_Error = AEPutParamDesc(message
											, optional_Parameter_Keywords[0]
											, &(extra_Optional_Argument[*optional_Arguments_Length  - 1]));
			
				if (os_Error == noErr) {
					succeded = TRUE;
				}
			}
								
		} /* NOT if (os_Error != noErr) */
	
	} /* if (succeded == TRUE) */

	
	if (succeded == TRUE) {
		succeded = FALSE;
		if (message_Type == k_SELECT_NEXT_ITEM_AE) {
			optional_Direction_Value = 1;
		}
		else {
			optional_Direction_Value = 0;
		}
				
		succeded = FALSE;
		os_Error = AECreateDesc(typeLongInteger
										, (Ptr)&optional_Direction_Value
										, sizeof(optional_Direction_Value)
										, &(extra_Optional_Argument[*optional_Arguments_Length]));
									
		if (os_Error != noErr) {
			/*  */
			/* wrong parameter length of object? Tumble out */
			/*  */
			; /* NULL */
		}
		else {
			extra_Optional_Argument_Initialized[*optional_Arguments_Length] = SAFE_TRUE;
			*optional_Arguments_Length += 1;
			
			if (*optional_Arguments_Length <= *optional_Parameter_List_Length) {
				/*  */
				/* create chunk of memory for optional attrribute and add to event to send */
				/*  */
				os_Error = AEPutParamDesc(message
											, optional_Parameter_Keywords[1]
											, &(extra_Optional_Argument[*optional_Arguments_Length  - 1]));
			
				if (os_Error == noErr) {
					succeded = TRUE;
				}
			}
								
		} /* NOT if (os_Error != noErr) */
			
			
	} /* if (succeded == TRUE) */
										

	if (succeded == TRUE) {
		function_Result = SAFE_TRUE;
	}
	
	goto Normal_Function_Exit;
	
Error_Handler:
	function_Result = SAFE_FALSE;

Normal_Function_Exit:

	if (temp_List_Memory_Initialized == MEMORY_INITIALIZED) {
		if (Destroy_Handle((Handle *)&temp_List_Hdl
							, &temp_List_Memory_Initialized) != SAFE_TRUE) {
	
			Display_Serious_Error_Message(unit_Name, code_Name, "\pError with Destroy_Handle", "\pSystem Corrupted?");
		}
	}

	return(function_Result);

} /* Safe_Boolean Fill_Parameters_For_Response_Select ... */
/*=**************************************************************************=*/


/*=**************************************************************************=*/
#pragma mark -
Safe_Boolean Send_Message_PPC_AE(AEDesc  *apple_Event_Descriptor
													, CARDINAL_16		message_Type
													, Ptr parameters
													, Safe_Boolean *answered)
/*  */
/* NOTE: Inneficient routine: indirectly hits memory manager for event creation */
/*  */
{
	OSErr				os_Error;
	Safe_Boolean		function_Result;
	AppleEvent		message;
	
	CARDINAL_32	optional_Arguments_Length;
	CARDINAL_32	loop;
	AEDesc			extra_Optional_Arguments[k_MAX_OPTIONAL_KEYWORDS];
	Safe_Boolean		extra_Optional_Argument_Initialized[k_MAX_OPTIONAL_KEYWORDS];
	AEKeyword		optional_Parameter_Keywords[k_MAX_OPTIONAL_KEYWORDS];
	AEDesc			optional_Parameter_List;
	CARDINAL_32	optional_Parameter_List_Length;
	AppleEvent		reply;
	DescType			actual_Description;
	INTEGER_32		actual_Size;
	Safe_Boolean		message_Initialized;
	Safe_Boolean		optional_Parameter_List_Initialized;
	Safe_Boolean		send_Event_Message;
	CARDINAL_32	timeout_Ticks;
	INTEGER_32		response_Value;
	INTEGER_32		spoof_Value;
	Str255				error_Data_String_Storage;
/* END VAR */


	function_Result = SAFE_FALSE;
	*answered = SAFE_FALSE;
	
	/*  */
	/* These are all important knock down flags*/
	/*  */
	message_Initialized = SAFE_FALSE;
	send_Event_Message = SAFE_FALSE;
	optional_Parameter_List_Initialized = SAFE_FALSE;
	optional_Arguments_Length = 0;
	reply.descriptorType = typeNull;
	timeout_Ticks = kAEDefaultTimeout;
	
	switch (message_Type) {
	
		case k_SIMPLE_ANSWER_AE:
			/*  */
			/* If the goddamn system can't respond in three seconds */
			/* then I don't care about trying to establish efficient "high-speed-response" */
			/* link. Normal commands are given 15 seconds or more though. */
			/*  */
			timeout_Ticks = (60L * k_SECONDS_FOR_HIGH_SPEED_ANSWER_REQUEST);
			os_Error = AECreateAppleEvent(kCoreEventClass
												, kAEAnswer
												, apple_Event_Descriptor
												, kAutoGenerateReturnID
												, kAnyTransactionID
												, &message);
            	if (os_Error == noErr) {
				message_Initialized = SAFE_TRUE;
				send_Event_Message = SAFE_TRUE;
			}
		break; /* k_SIMPLE_ANSWER_AE */
				
				
		case k_SELECT_NEXT_ITEM_AE:
		case k_SELECT_PREVIOUS_ITEM_AE:
		case k_SELECT_ITEM_AE:
			timeout_Ticks = (60L * k_SECONDS_FOR_REGULAR_REQUEST);
			os_Error = AECreateAppleEvent(k_SELF_EVENT_CLASS
												, k_REMOTE_CONTROL_EVENT
												, apple_Event_Descriptor
												, kAutoGenerateReturnID
												, kAnyTransactionID
												, &message);
												
            	if (os_Error == noErr) {
				message_Initialized = SAFE_TRUE;
			}
            	
            	if (os_Error == noErr) {
				
				/*  */
				/* Make an OPTIONAL parameter Description empty list */
				/*  */
				os_Error = AECreateList(nil, 0, FALSE, &optional_Parameter_List);
				if (os_Error != noErr) {
					/*  */
					/* wrong parameter length of object? Tumble out */
					/*  */
					; /* NULL */
				}
				else {
					optional_Parameter_List_Initialized = SAFE_TRUE;
					optional_Parameter_List_Length = 0;
					
					if  (Add_Parameters_For_Command_Select(&optional_Parameter_List
															, &optional_Parameter_List_Length
															, k_MAX_OPTIONAL_KEYWORDS
															, &message
															, &(optional_Parameter_Keywords[0])) == SAFE_TRUE) {
															
						/*  */
						/* If everything seems good still, add the optional attributes */
						/* Add an OPTIONAL parameter */
						/* related to "k_TOTAL_OPTIONS_FOR_SELECT" */
						/*  */
						optional_Arguments_Length = 0;
						
						if  (Fill_Parameters_For_Command_Select(&optional_Parameter_List_Length
																, &message
																, &optional_Arguments_Length
																, &(extra_Optional_Arguments[0])
																, &(extra_Optional_Argument_Initialized[0])
																, &(optional_Parameter_Keywords[0])
																, message_Type
																, parameters) == SAFE_TRUE) {
	
							send_Event_Message = SAFE_TRUE;
						}
					}
					
				} /* NOT if (os_Error != noErr) */
				
			} /* if (os_Error == noErr) */
												
		break; /* k_SELECT_NEXT_ITEM_AE ... */
		
				

		case k_ACKNOWLEDGE_SIMPLE_ANSWER_AE:
		case k_ACKNOWLEDGE_SELECT_NEXT_ITEM_AE:
		case k_ACKNOWLEDGE_SELECT_PREVIOUS_ITEM_AE:
		
			timeout_Ticks = (60L * k_SECONDS_FOR_REGULAR_REQUEST);
			os_Error = AECreateAppleEvent(k_SELF_EVENT_CLASS
												, k_REMOTE_CONTROL_RESPONSE_EVENT
												, apple_Event_Descriptor
												, kAutoGenerateReturnID
												, kAnyTransactionID
												, &message);
												
            	if (os_Error == noErr) {
				message_Initialized = SAFE_TRUE;
			}
            	
            	if (os_Error == noErr) {
				
				/*  */
				/* Make an OPTIONAL parameter Description empty list */
				/*  */
				os_Error = AECreateList(nil, 0, FALSE, &optional_Parameter_List);
				if (os_Error != noErr) {
					/*  */
					/* wrong parameter length of object? Tumble out */
					/*  */
					; /* NULL */
				}
				else {
					optional_Parameter_List_Initialized = SAFE_TRUE;
					optional_Parameter_List_Length = 0;
					
					if  (Add_Parameters_For_Response_Select(&optional_Parameter_List
															, &optional_Parameter_List_Length
															, k_MAX_OPTIONAL_KEYWORDS
															, &message
															, &(optional_Parameter_Keywords[0])) == SAFE_TRUE) {
															
						/*  */
						/* If everything seems good still, add the optional attributes */
						/* Add an OPTIONAL parameter */
						/* related to "k_TOTAL_OPTIONS_FOR_SELECT" */
						/*  */
						optional_Arguments_Length = 0;
						
						if  (Fill_Parameters_For_Response_Select(&optional_Parameter_List_Length
																, &message
																, &optional_Arguments_Length
																, &(extra_Optional_Arguments[0])
																, &(extra_Optional_Argument_Initialized[0])
																, &(optional_Parameter_Keywords[0])
																, message_Type) == SAFE_TRUE) {
	
							send_Event_Message = SAFE_TRUE;
						}
					}
					
				} /* NOT if (os_Error != noErr) */
				
			} /* if (os_Error == noErr) */
												
		break; /* k_ACKNOWLEDGE_SIMPLE_ANSWER_AE ... */
	

		default:
			goto Error_Handler;
	}
	
	if (send_Event_Message == SAFE_TRUE) {

		reply.descriptorType = typeNull;
		
		if (message_Type == k_SIMPLE_ANSWER_AE) {
			os_Error = AESend(&message
							, &reply
							, (kAEWaitReply | kAENeverInteract)
							, kAEHighPriority /* kNormalPriority */
							, timeout_Ticks
							, nil
							, nil );
		}
		else {
			os_Error = AESend(&message
							, &reply
							, (kAEQueueReply | kAENeverInteract)
							, kAENormalPriority
							, timeout_Ticks
							, nil
							, nil );
		}
							
		if (os_Error == noErr) {
		
			/*  */
			/* Now we must get the REAL return value in keyErrorNumber */
			/* if it is possible */
			/*  */
			actual_Description = -1;
			os_Error = AEGetParamPtr(&reply
									, keyErrorNumber
									, typeLongInteger
									, &actual_Description
									, &response_Value
									, sizeof(response_Value)
									, &actual_Size);
									
			if (((os_Error == noErr) && (response_Value == noErr)) 
				|| (os_Error == errAEDescNotFound)){
				
				/*  */
				/* Proof it was sent! Now validation of return result remains to be done. */
				/* Now we can check the REAL return value from response value */
				/* if this was sent with "wait for reply"? */
				/*  */
				actual_Description = -1;
				os_Error = AEGetParamPtr(&reply
									, keyDirectObject
									, typeLongInteger
									, &actual_Description
									, &response_Value
									, sizeof(response_Value)
									, &actual_Size);
				if (os_Error == noErr) {
					if (response_Value == noErr) {
						/*  */
						/* Goody! At least this message was sent or sent/answered */
						/*  */
						*answered = SAFE_TRUE;
					}
					else {
						/*  */
						/* Event was not sent AND received without some type of problem. */
						/* "response_Value" contains result. */
						/*  */
						; /* NULL */
					}
				}
				else {
					if (os_Error == errAEDescNotFound) {
						/*  */
						/* Damn! No way to extract information regarding reply! */
						/* unfortunately we will have to assume it succeeded! */
						/* The default path for remote PPC connection with "Answer" */
						/* and attributes (kAEWaitReply + kAENeverInteract) hits here always */
						/* when successful. OK though if not sent as "kAEWaitReply"*/
						/*  */
						
						/*  */
						/* HACK HACK HACK */
						/*  */
						
						*answered = SAFE_TRUE;
					}
					else {
						/*  */
						/* Exotic failure in system. OK though if not sent as "kAEWaitReply"*/
						/*  */
						; /* NULL */
					}
				}
			}
			else {
				/*  */
				/* AEGetParamPtr failed, but may have failed because a */
				/* complex, fancy error status was returned, and that the status */
				/* is held in the non-simple portion of the response (non-direct parameter) */
				/* More checking is needed before assuming failure of Apple event send. */
				/* This used to be type international-char in winter of 1992 proposed */
				/* changes, but the new IM Interapplication Communication reverted to */
				/* simple text again. So we try to get simple text string only. */
				/* It is still assumed though that a FAILURE does exist, despite it being */
				/* encoded as a string instead of a number */
				/*  */
				error_Data_String_Storage[0] = 0;
				actual_Description = -1;
				os_Error = AEGetParamPtr(&reply
									, keyErrorString
									, typeChar
									, &actual_Description
									, &error_Data_String_Storage[1]
									, 255
									, &actual_Size);
				if (os_Error == noErr) {
					if (actual_Size > 255) {
						/*  */
						/* Danger! Serious unwanted truncation of reply about to happen */
						/* that may break a multibyte string token in half ! */
						/*  */
						actual_Size = 255;
					}
					
					/*  */
					/* Send occured probably, failure occured, most of result is in string. */
					/*  */
					error_Data_String_Storage[0] = actual_Size;
					

				}
				else {
					/*  */
					/* Exotic failure, Send may have not occured. OK though */
					/* if not sent as "kAEWaitReply"*/
					/*  */
					; /* NULL */
				}
				
			} /* NOT if (os_Error == noErr) */
	
		} /* if (os_Error == noErr) */
	
	} /* if (os_Error == noErr) */
	
		
	function_Result = SAFE_TRUE;
	goto Normal_Function_Exit;
	
Error_Handler:
	function_Result = SAFE_FALSE;

Normal_Function_Exit:

	/*  */
	/* Delete the damned Apple event we created */
	/*  */
	if (message_Initialized == SAFE_TRUE) {
		os_Error = AEDisposeDesc(&message);
		if (os_Error != noErr) {
			; /* NULL */
		}	
	}
	
	/*  */
	/* Delete the piece of RAM we hold optional attribute lists */
	/*  */
	if (optional_Parameter_List_Initialized == SAFE_TRUE) {
		os_Error = AEDisposeDesc(&optional_Parameter_List);
		if (os_Error != noErr) {
			; /* NULL */
		}	
	}
	
	/*  */
	/* Delete the piece of RAM we created to add an long word */
	/*  */
	if (optional_Arguments_Length > 0) {
		for (loop = 0; (loop < optional_Arguments_Length) ; loop++) {
			if (extra_Optional_Argument_Initialized[loop] == SAFE_TRUE) {
				os_Error = AEDisposeDesc(&(extra_Optional_Arguments[loop]));
				if (os_Error != noErr) {
					; /* NULL */
				}	
			}
		}
	}
	
	/*  */
	/* Delete the RAM we created if reply was genereated */
	/*  */
	if (reply.descriptorType != typeNull) {
		os_Error = AEDisposeDesc(&reply);
		if (os_Error != noErr) {
			; /* NULL */
		}	
	}

	return(function_Result);
	
} /* Safe_Boolean Send_Message_PPC_AE ... */
/*=**************************************************************************=*/


/*=**************************************************************************=*/
Safe_Boolean Affirm_Connection_With_Response(Remote_Connect_Ptr		remote_Info
													, Safe_Boolean *answered)
{
	OSErr				os_Error;
	Safe_Boolean		function_Result;
	Safe_Boolean		success;
/* END VAR */


	function_Result = SAFE_FALSE;
	*answered = SAFE_FALSE;

	if (remote_Info->connected_Once == SAFE_TRUE) {
		if (remote_Info->connected_Now == SAFE_TRUE) {
			switch(remote_Info->connection_Type) {
			
				case k_CONNECTION_REMOTE_PPC_APPLE_EVENT:
					if (Send_Message_PPC_AE(&(remote_Info->connect.remote_PPC.apple_Event_Descriptor), k_SIMPLE_ANSWER_AE, nil ,&success) != SAFE_TRUE) {
						success = SAFE_FALSE;
					}
					if (success == SAFE_TRUE) {
						*answered = SAFE_TRUE;
					}
					else {
						/*  */
						/* This is grave enough to negate "remote_Info->connected_Now" ? */
						/*  */
						; /* NULL */
					}
				break;
				
				case k_CONNECTION_NONE:
				case k_CONNECTION_REMOTE_DATAGRAM_LISTENER:
				case k_CONNECTION_REMOTE_MODEM_STREAM:
				case k_CONNECTION_REMOTE_ATP_OR_ADSP_STYLE:
				case k_CONNECTION_REMOTE_LOCAL_APPLE_EVENT:
				case k_CONNECTION_REMOTE_SELF_TEST:
				break;
				
				default:
					goto Error_Handler;
			}	
		}
	}

	function_Result = SAFE_TRUE;
	goto Normal_Function_Exit;
	
Error_Handler:
	function_Result = SAFE_FALSE;

Normal_Function_Exit:

	return(function_Result);
	
} /* Safe_Boolean Disconnect_Remote_For_Shutdown ... */
/*=**************************************************************************=*/


/*=**************************************************************************=*/
Safe_Boolean Connect_Remote_If_Possible(Remote_Connect_Ptr remote_Info, Safe_Boolean *connected)
{
	OSErr				os_Error;
	Safe_Boolean		function_Result;
/* END VAR */


	function_Result = SAFE_FALSE;
	*connected = SAFE_FALSE;
	
	remote_Info->connected_Once = SAFE_FALSE;
	
	/*  */
	/* Paranoia initialize */
	/*  */
	remote_Info->connected_Now = SAFE_FALSE;
	remote_Info->connection_Type = k_CONNECTION_NONE;
	remote_Info->last_Contact_Tick_Time = 0;
	remote_Info->needs_Cleanup_At_Quit = SAFE_FALSE;

	if (Connect_Remote_Using_Apple_Events(remote_Info, connected) != SAFE_TRUE) {
		goto Error_Handler;
	}
	
	if (*connected != SAFE_TRUE) {
		/*  */
		/* If failure, we could try other strategies, hunt process IDs local */
		/* eastablish DDP socket listeners, request Modem serial I/O, etc. */
		/* as far as we are concerned though, exit. */
		/*  */
		; /* NULL */
	}

	function_Result = SAFE_TRUE;
	goto Normal_Function_Exit;
	
Error_Handler:
	function_Result = SAFE_FALSE;

Normal_Function_Exit:

	return(function_Result);
}
/*=**************************************************************************=*/


/*=**************************************************************************=*/
Safe_Boolean Connect_Remote_Using_Apple_Events(Remote_Connect_Ptr	remote_Info
															, Safe_Boolean *connected)
{
	OSErr					os_Error;
	Safe_Boolean			function_Result;
	Str255					prompt_String;
	Str255					applications_String;
	LocationNameRec	location;
	PortInfoRec			port_Information;
	ConstStr32Param	network_Object_NBP_Template;
/* END VAR */


	function_Result = SAFE_FALSE;
	*connected = SAFE_FALSE;

	Copy_Pascal_String(&k_PROGRAM_LINK_STRING[0], prompt_String);
	Copy_Pascal_String(&k_GENERAL_APPLICATION_LIST_STRING[0],  applications_String);
						
	os_Error = PPCBrowser(prompt_String 
								, applications_String
								, FALSE
								, &location
								, &port_Information
								, nil
								, nil);
	
	if (os_Error == noErr) {
		remote_Info->connect.remote_PPC.identifier.location = location;
		remote_Info->connect.remote_PPC.identifier.name = port_Information.name;
		
		remote_Info->needs_Cleanup_At_Quit = SAFE_FALSE;

		remote_Info->connect.remote_PPC.apple_Event_Descriptor.descriptorType = typeNull;
		os_Error = AECreateDesc(typeTargetID
									, (Ptr)&(remote_Info->connect.remote_PPC.identifier)
									, sizeof(remote_Info->connect.remote_PPC.identifier)
									, &(remote_Info->connect.remote_PPC.apple_Event_Descriptor));
		
		if (os_Error == noErr) {
			if (remote_Info->connect.remote_PPC.apple_Event_Descriptor.descriptorType != typeNull) {
				remote_Info->needs_Cleanup_At_Quit = SAFE_TRUE;
			}
			*connected = SAFE_TRUE;
			remote_Info->connected_Once = SAFE_TRUE;
			remote_Info->connected_Now = SAFE_TRUE;
			remote_Info->connection_Type = k_CONNECTION_REMOTE_PPC_APPLE_EVENT;
			remote_Info->last_Contact_Tick_Time = TickCount();
		}
		else {
			*connected = SAFE_FALSE;
		}
	}
	
	function_Result = SAFE_TRUE;
	goto Normal_Function_Exit;
	
Error_Handler:
	function_Result = SAFE_FALSE;

Normal_Function_Exit:

	return(function_Result);
}
/*=**************************************************************************=*/


/*=**************************************************************************=*/
#pragma mark -
pascal OSErr Handler_For_Remote_Response_Apple_Event(
						AppleEvent *answer_Document_Apple_Event
						, AppleEvent *reply
						, INTEGER_32 handlerRefCon)
{
	OSErr				os_Error;
	OSErr				function_Result;
	DescType			returned_Type;
	CARDINAL_32	actual_Size;
	CARDINAL_32	parameter;
	INTEGER_32		direction;
/* END VAR */


	function_Result = -1;
		
	/*  */
	/* Check if valid params ready. */
	/*  */
	os_Error = Determine_If_Required_Params_Done(answer_Document_Apple_Event);
	if (os_Error != noErr) {
	
		/*  */
		/* I do not have documentation on core required apple event */
		/* of type Answer. I do not know what to take off. Ignore os_Error */
		/* indicating we forgot to process something */
		/*  */
		if (os_Error != errAEEventNotHandled) {
			function_Result = os_Error;
			goto Error_Handler;
		}
	}
				
	os_Error = AEGetParamPtr(answer_Document_Apple_Event
									, keyDirectObject
									, typeLongInteger
									, &returned_Type
									, &parameter
									, sizeof(CARDINAL_32)
									, (Size *)&actual_Size);
									
	if (os_Error != noErr) {
	
		SysBeep(20);
		SysBeep(20);
		
		function_Result = errAEEventNotHandled;
		goto Error_Handler;
	}
	
	
	if (parameter == FILE_CREATOR) {
		parameter = 0;
		/*  */
		/* Cool! Data was received */
		/*  */
		os_Error = AEGetParamPtr(answer_Document_Apple_Event
										, k_COMMAND_SELECT_DIRECTION_TYPE
										, typeLongInteger
										, &returned_Type
										, &parameter
										, sizeof(CARDINAL_32)
										, (Size *)&actual_Size);
										
		if (os_Error != noErr) {
		
			SysBeep(20);
			
			function_Result = errAEEventNotHandled;
			goto Error_Handler;
		}
		
		returned_Type = typeChar;
		os_Error = AESizeOfParam(answer_Document_Apple_Event
										, k_COMMAND_SELECT_COMPOSITE_TYPE
										, &returned_Type
										, (Size *)&actual_Size);
		if (os_Error == noErr) {
										
			if (g_Recent_Remote_Slides_Memory_In_Use == MEMORY_INITIALIZED) {
				if (actual_Size > k_MAXIMUM_CACHED_REMOTE_LIST_SIZE) {
					actual_Size = k_MAXIMUM_CACHED_REMOTE_LIST_SIZE;
				}
				
				os_Error = AEGetParamPtr(answer_Document_Apple_Event
											, k_COMMAND_SELECT_COMPOSITE_TYPE
											, typeChar
											, &returned_Type
											, *g_Recent_Remote_Slides_Hdl
											, actual_Size
											, (Size *)&actual_Size);
				if (os_Error == noErr) {
					g_Recent_Remote_Slides_Length = actual_Size;
					g_Recent_Remote_Slides_Valid = SAFE_TRUE;
				}

			}
		}
		
		direction = parameter;
	}
	

	function_Result = noErr;
	goto Normal_Function_Exit;

Error_Handler:

Normal_Function_Exit:
	return(function_Result);
	
} /* pascal OSErr Handler_For_Remote_Response_Apple_Event... */
/*=**************************************************************************=*/


/*=**************************************************************************=*/
pascal OSErr Handler_For_Remote_Command_Apple_Event(
						AppleEvent *answer_Document_Apple_Event
						, AppleEvent *reply
						, INTEGER_32 handlerRefCon)
{
	OSErr				os_Error;
	OSErr				function_Result;
	DescType			returned_Type;
	CARDINAL_32	actual_Size;
	INTEGER_16		source_Of_Event;
	Str255				optional_Test_String;
	CARDINAL_32	parameter;
	CARDINAL_32	reply_Message;	
	AEDesc 			apple_Event_Reply;
	
	TargetID			address_Identifier;	
	
	Safe_Boolean 	response_Sent;
	Safe_Boolean		reply_Event_Memory_Initialized;
	CARDINAL_32	direction;
	Str255				prompt_String;
/* END VAR */


	function_Result = -1;
	reply_Event_Memory_Initialized = SAFE_FALSE;

		
	/*  */
	/* Check if valid params ready. */
	/*  */
	os_Error = Determine_If_Required_Params_Done(answer_Document_Apple_Event);
	if (os_Error != noErr) {
	
		/*  */
		/* I do not have documentation on core required apple event */
		/* of type Answer. I do not know what to take off. Ignore os_Error */
		/* indicating we forgot to process something */
		/*  */
		if (os_Error != errAEEventNotHandled) {
			function_Result = os_Error;
			goto Error_Handler;
		}
	}
	
			
	os_Error = AEGetParamPtr(answer_Document_Apple_Event
									, keyDirectObject
									, typeLongInteger
									, &returned_Type
									, &parameter
									, sizeof(CARDINAL_32)
									, (Size *)&actual_Size);
									
	if (os_Error != noErr) {
	
		SysBeep(20);
		SysBeep(20);
		
		function_Result = errAEEventNotHandled;
		goto Error_Handler;
	}
	
	
	if (parameter == FILE_CREATOR) {
		parameter = 0;
		/*  */
		/* Cool! Data was received */
		/*  */
		os_Error = AEGetParamPtr(answer_Document_Apple_Event
										, k_COMMAND_SELECT_DIRECTION_TYPE
										, typeLongInteger
										, &returned_Type
										, &parameter
										, sizeof(CARDINAL_32)
										, (Size *)&actual_Size);
										
		if (os_Error != noErr) {
		
			SysBeep(20);
			
			function_Result = errAEEventNotHandled;
			goto Error_Handler;
		}
		
		returned_Type = typeChar;
		os_Error = AESizeOfParam(answer_Document_Apple_Event
										, k_COMMAND_SELECT_COMPOSITE_TYPE
										, &returned_Type
										, (Size *)&actual_Size);
		if (os_Error == noErr) {
										
			if (actual_Size > 255) {
				actual_Size = 255;
			}
			
			os_Error = AEGetParamPtr(answer_Document_Apple_Event
										, k_COMMAND_SELECT_COMPOSITE_TYPE
										, typeChar
										, &returned_Type
										, &optional_Test_String[1]
										, actual_Size
										, (Size *)&actual_Size);
			if (os_Error == noErr) {
				optional_Test_String[0] = actual_Size;
			}
			else {
				optional_Test_String[0] = 0;
			}
										
		}
		
		direction = parameter;
		
		/*  */
		/* All information is ready and valid. Act on it. */
		/*  */
		if ((direction & k_MASK_HIGH_FLAGS_FOR_CALLER_SELECT_OVERLOAD) == k_MASK_HIGH_FLAGS_FOR_CALLER_SELECT_OVERLOAD) {
			/*  */
			/* Specific slide selected */
			/*  */
			direction = (direction & (~k_MASK_HIGH_FLAGS_FOR_CALLER_SELECT_OVERLOAD));
		/* ===
			Select_Caller(direction);
		=== */
		}
		else if ((direction & k_MASK_HIGH_FLAGS_FOR_SELECT_OVERLOAD) == k_MASK_HIGH_FLAGS_FOR_SELECT_OVERLOAD) {
			/*  */
			/* Specific slide selected */
			/*  */
			direction = (direction & (~k_MASK_HIGH_FLAGS_FOR_SELECT_OVERLOAD));
			Select_Slide(direction);
		}
		else {
			/*  */
			/* Select adjacent elements */
			/*  */
			Advance_Slide_Using_Controller(direction);
		}
		
	}
	else {
		function_Result = errAEEventNotHandled;
		goto Error_Handler;
	}
	
	/*  */
	/* Send message indicating new status */
	/* We want keyEventSourceAttr, keyOriginalAddressAttr '*/
	/*  */
	os_Error = AEGetAttributePtr(answer_Document_Apple_Event
								, keyEventSourceAttr
								, typeShortInteger
								, &returned_Type
								, &source_Of_Event
								, sizeof(source_Of_Event)
								, (Size *)&source_Of_Event);
								
	if (os_Error == noErr) {
		
		/*  */
		/* Event is not from this app. We need to send reply */
		/* Otherwise just manipulate globals */
		/*  */
		returned_Type = typeTargetID;
		os_Error = AEGetAttributePtr(answer_Document_Apple_Event
									, keyOriginalAddressAttr
									, typeTargetID
									, &returned_Type
									, &address_Identifier
									, (sizeof(address_Identifier))
									, (Size *)&actual_Size);
	
	
	
		if (os_Error == noErr) {
				
			apple_Event_Reply.descriptorType = typeNull;
			
			/*  */
			/* Check if message regarding target */
			/*  */
			os_Error = AECreateDesc(typeTargetID
										, (Ptr)&(address_Identifier)
										, sizeof(address_Identifier)
										, &(apple_Event_Reply));
						
			if (os_Error == noErr) {
				if (apple_Event_Reply.descriptorType != typeNull) {
					reply_Event_Memory_Initialized = SAFE_TRUE;
					
					if (direction == 1) {
						reply_Message = k_ACKNOWLEDGE_SELECT_NEXT_ITEM_AE;
					}
					else {
						reply_Message = k_ACKNOWLEDGE_SELECT_PREVIOUS_ITEM_AE;
					}

					if (Add_Response_To_Response_Queue(reply_Message, &address_Identifier) != SAFE_TRUE) {
						/*  */
						/* Ignore. It is busy or full perhaps. not critical. */
						/*  */
					}
				}
			}
		}
		else {
		
			Make_String_Long_String(&prompt_String[0]
								, "\pYou either have old Apple Event v1.0, or else a serious error occurred snatching 'keyOriginalAddressAttr' address!! Error was"
								, os_Error
								, NO_LIMIT
								, "\p. User Suggestion: Sell your Apple stock soon! (or upgrade to 1.0.1)");
					
			SysBeep(20);
			Perform_OK_Dialog(prompt_String);
		}
	
	} /* if (os_Error == noErr) ... */
	else {	
		function_Result = errAEEventNotHandled;
		goto Error_Handler;
	}
	

	function_Result = noErr;
	goto Normal_Function_Exit;

Error_Handler:

Normal_Function_Exit:

	if (reply_Event_Memory_Initialized == SAFE_TRUE) {
		os_Error = AEDisposeDesc(&apple_Event_Reply);
		if (os_Error != noErr) {
			; /* NULL */
		}	
	}
	
	return(function_Result);
	
} /* pascal OSErr Handler_For_Remote_Command_Apple_Event... */
/*=**************************************************************************=*/


/*=**************************************************************************=*/
Safe_Boolean Send_Remote_Message(CARDINAL_32	command, Ptr parameters)
{
	Safe_Boolean		function_Result;
	Safe_Boolean		answered;
	INTEGER_16		loop;
/* END VAR */

	function_Result = SAFE_FALSE;
	
/* ===
	switch(command) {
		case k_CUSTOM_EVENT_1_AE:
		break;
		
		default:
			;
	}
=== */

	if (g_Possibly_Connected == SAFE_TRUE) {
	
		/*  */
		/* Send message to all, even though not practical or always desired */
		/*  */
		for (loop=0; (loop < k_REMOTE_CONNECTIONS_MAX); loop++) {
	
			if (Send_Remote_Command(&(g_Remote_Connect[loop])
											, command
											, parameters
											, &answered) != SAFE_TRUE) {
				/*  */
				/* Heed error as if it was fatal. Only fatal errors passed back */
				/*  */
				goto Error_Handler;
			}
	
			if (answered == SAFE_TRUE) {
				/*  */
				/* Set true if something worked at all */
				/*  */
				function_Result = SAFE_TRUE;
			}
	
		}
	}
	
	goto Normal_Function_Exit;
	
Error_Handler:
	function_Result = SAFE_FALSE;

Normal_Function_Exit:

	return(function_Result);
}
/*=**************************************************************************=*/


/*=**************************************************************************=*/
Safe_Boolean Send_Remote_Command(Remote_Connect_Ptr		remote_Info
													, CARDINAL_32	command
													, Ptr parameters
													, Safe_Boolean *answered)
{
	OSErr				os_Error;
	Safe_Boolean		function_Result;
	Safe_Boolean		success;
/* END VAR */


	function_Result = SAFE_FALSE;
	*answered = SAFE_FALSE;
	
	if (remote_Info->connected_Once == SAFE_TRUE) {
		if (remote_Info->connected_Now == SAFE_TRUE) {
			switch(remote_Info->connection_Type) {
			
				case k_CONNECTION_REMOTE_PPC_APPLE_EVENT:
					if (Send_Message_PPC_AE(&(remote_Info->connect.remote_PPC.apple_Event_Descriptor)
													, command
													, parameters
													, &success) != SAFE_TRUE) {
						success = SAFE_FALSE;
					}
					if (success == SAFE_TRUE) {
						*answered = SAFE_TRUE;
					}
					else {
						/*  */
						/* This is grave enough to negate "remote_Info->connected_Now" ? */
						/*  */
						; /* NULL */
					}
				break;
				
				case k_CONNECTION_NONE:
				case k_CONNECTION_REMOTE_DATAGRAM_LISTENER:
				case k_CONNECTION_REMOTE_MODEM_STREAM:
				case k_CONNECTION_REMOTE_ATP_OR_ADSP_STYLE:
				case k_CONNECTION_REMOTE_LOCAL_APPLE_EVENT:
				case k_CONNECTION_REMOTE_SELF_TEST:
				break;
				
				default:
					goto Error_Handler;
			}	
		}
	}

	function_Result = SAFE_TRUE;
	goto Normal_Function_Exit;
	
Error_Handler:
	function_Result = SAFE_FALSE;

Normal_Function_Exit:

	return(function_Result);
	
} /* Safe_Boolean Disconnect_Remote_For_Shutdown ... */
/*=**************************************************************************=*/


/*=**************************************************************************=*/
OSErr Initialize_Special_AE_Handlers(void)
{
OSErr					function_Result;
OSErr					install_Result;
Boolean 				apple_Events_Ready;
AEEventHandlerUPP	handler_Proc_Pointer;
INTEGER_32	 		response;
/* END VAR */


	function_Result = -1;	
	
	apple_Events_Ready = FALSE;
	if (Gestalt_Available() == TRUE) {
		function_Result = Gestalt(gestaltAppleEventsAttr, &response);
		if (function_Result == noErr) {
			apple_Events_Ready = TRUE;
		}
	}
	
	if (apple_Events_Ready != TRUE) {
		goto Error_Handler;
	}

	
	/*  */
	/* Add proprietary apple event. Used for remote contol applications. */
	/*  */
	
#if (ORIGINAL_MAC_SYSTEM == 0)
XXXX Do correctly
	handler_Proc_Pointer = (AEEventHandlerUPP)&Handler_For_Remote_Command_Apple_Event;
#else
	handler_Proc_Pointer = (AEEventHandlerUPP)&Handler_For_Remote_Command_Apple_Event;
#endif 
	function_Result = AEInstallEventHandler(
							k_SELF_EVENT_CLASS
							, k_REMOTE_CONTROL_EVENT
			            		, handler_Proc_Pointer
							, 0
							, FALSE);
	if (function_Result != noErr) {
		goto Error_Handler;
	}
	
#if (ORIGINAL_MAC_SYSTEM == 0)
XXXX Do correctly
	handler_Proc_Pointer = (AEEventHandlerUPP)&Handler_For_Remote_Response_Apple_Event;
#else
	handler_Proc_Pointer = (AEEventHandlerUPP)&Handler_For_Remote_Response_Apple_Event;
#endif 
	function_Result = AEInstallEventHandler(
							k_SELF_EVENT_CLASS
							, k_REMOTE_CONTROL_RESPONSE_EVENT
			            		, handler_Proc_Pointer
							, 0
							, FALSE);
	if (function_Result != noErr) {
		goto Error_Handler;
	}
	
	function_Result = noErr;
	goto Normal_Function_Exit;

Error_Handler:

Normal_Function_Exit:
	return(function_Result);

}
/*=**************************************************************************=*/


/*=**************************************************************************=*/
/* END OF FILE: Remote_Control_Full.c */
