/* START OF FILE: Remote_Control_Standard.c */


/*=**************************************************************************=*/
/* [---BEGIN---] FILE HEADER COMMENTS */
/**
 ** SCREEN VIEW = [Adobe Syntax Black-10, Tabs = 4]
 ** PRINTER VIEW = [Adobe Syntax Black-10, Tabs = 4]
 ** =======================================================================
 ** Written by James Merkle
 ** Remote_Control_Standard.c - Accepts control from menu, keyboard, remote control packets, control bar
 ** (C) Copyright 1994 James Merkle
 ** All rights reserved, except those wishing to "copy" moderate sections of code or files may do so, so long as
 ** such code is never used directly either in animation-related tools or multimedia tools that are  
 ** used in live performance or live broadcast or recorded mass market broadcast or film. Desktop usage not restricted.
 ** Conventional desktop game or entertainment not restricted.
 ** =======================================================================
 **
 **
 ** ============
 ** Revision History
 ** ============
 **
 **  1994.02.01		(James Merkle)
 **						Initial coding started
 **
 **
 **
 **
 ** =================
 ** OTHER INFORMATION
 ** =================
 **  
 **
 **/
/* [----END----] FILE HEADER COMMENTS */
/*=**************************************************************************=*/


/*=**************************************************************************=*/
/* [---BEGIN---] #include AREA */

#include <AppleEvents.h>
#include <Processes.h>
#include <PPCToolBox.h>
#include <EPPC.h>
#include <Notification.h>

#include "Core_Globals.h"
#include "OS_Utilities.h"
#include "String_Utilities.h"
#include "Command_Redirector.h"
#include "Remote_Control_Standard.h"


/* [----END----] #include AREA */
/*=**************************************************************************=*/


/*=**************************************************************************=*/
/* [---BEGIN---] #define AREA */



/* [----END----] #define AREA */
/*=**************************************************************************=*/


/*=**************************************************************************=*/
/* [---BEGIN---] TYPE AREA */


/* [----END----] TYPE AREA */
/*=**************************************************************************=*/


/*=**************************************************************************=*/
/* [---BEGIN---] PROTOTYPE AREA */
															

pascal OSErr Handler_For_Quit_Apple_Event (AppleEvent *quit_Apple_Event
						, AppleEvent *reply
						, INTEGER_32 handlerRefCon);
						
pascal OSErr Handler_For_Open_Application_Apple_Event (AppleEvent *open_Application_Apple_Event
						, AppleEvent *reply
						, INTEGER_32 handlerRefCon);
						
pascal OSErr Handler_For_Open_Document_Apple_Event (AppleEvent *open_Document_Apple_Event
						, AppleEvent *reply
						, INTEGER_32 handlerRefCon);
						
pascal OSErr Handler_For_Print_Document_Apple_Event (AppleEvent *print_Document_Apple_Event
						, AppleEvent *reply
						, INTEGER_32 handlerRefCon);

pascal OSErr Handler_For_Answer_Apple_Event (AppleEvent *answer_Document_Apple_Event
						, AppleEvent *reply
						, INTEGER_32 handlerRefCon);

OSErr Initialize_AE_Handlers(void);

Safe_Boolean Create_Apple_Event_Proc_Pointers(void);


/* [----END----] PROTOTYPE AREA */
/*=**************************************************************************=*/


/*=**************************************************************************=*/
/* [---BEGIN---] GLOBAL DEFINITION AREA */

Safe_Boolean			g_Apple_Events_Possible;

/* [----END----] GLOBAL DEFINITION AREA */
/*=**************************************************************************=*/


/*=**************************************************************************=*/
/* [---BEGIN---] LOCAL STATIC DEFINITION AREA */

static CARDINAL_8	*unit_Name;

AEEventHandlerUPP		g_Open_Application_Apple_Event_UPP;
AEEventHandlerUPP		g_Open_Document_Apple_Event_UPP;
AEEventHandlerUPP		g_Quit_Apple_Event_UPP;
AEEventHandlerUPP		g_Print_Application_Apple_Event_UPP;
AEEventHandlerUPP		g_Answer_Apple_Event_UPP;

/* [----END----] LOCAL STATIC DEFINITION AREA */
/*=**************************************************************************=*/


/*=**************************************************************************=*/
Safe_Boolean Initialize_Remote_Control_Standard(void)
{
	CARDINAL_8			*static_Unit_Name = "\pRemote_Control_Standard";
	Safe_Boolean			function_Result;
/* END VAR */

/*  */
/* Global persistant structures are created here */
/*  */

	function_Result = SAFE_FALSE;
	unit_Name = static_Unit_Name;
	g_Apple_Events_Possible = SAFE_FALSE;
	
	if (Initialize_AE_Handlers() != noErr) {
		/*  */
		/* Ignore error in this version. Not expected to cause trouble */
		/* by design. (Not Fatal if AE not available) */
		/*  */
	}
	
	function_Result = SAFE_TRUE;
	goto Normal_Function_Exit;
	
Error_Handler:
	function_Result = SAFE_FALSE;

Normal_Function_Exit:

	return(function_Result);
}
/*=**************************************************************************=*/


/*=**************************************************************************=*/
Safe_Boolean Destroy_Remote_Control_Standard(void)
{
	Safe_Boolean		function_Result;
/* END VAR */

	/*  */
	/* Global persistant structures are deleted here */
	/*  */
	
	function_Result = SAFE_TRUE;
	goto Normal_Function_Exit;
	
Error_Handler:
	function_Result = SAFE_FALSE;

Normal_Function_Exit:

	return(function_Result);
}
/*=**************************************************************************=*/


/*=**************************************************************************=*/
void Manage_High_Level_Event(EventRecord *macEvent)
{
/*  */
/* This routine handles the high-level event specified by anEvent.  The only */
/* high-level events that this application handles are AppleEvents, so I just */
/* pass the high-level event to AEProcessAppleEvent.  AEProcessAppleEvent calls */
/* the appropriate AppleEvent handler routine to handle that particular kind of */
/* AppleEvent. */
/*  */
	OSErr	error;
/* END VAR */

	error = AEProcessAppleEvent((const EventRecord *)macEvent);
	if (error != noErr) {
		/*  */
		/* Ignore error. Nothing can be done about it. */
		/*  */
	}

} /* void Manage_High_Level_Event(EventRecord *macEvent) */
/*=**************************************************************************=*/


/*=**************************************************************************=*/
pascal OSErr Handler_For_Quit_Apple_Event (AppleEvent *quit_Apple_Event
						, AppleEvent *reply
						, INTEGER_32 handlerRefCon)
/*  */
/* Handler_For_Quit_Apple_Event */
/* This is the AppleEvent handler for the 'quit' AppleEvent as passed in the*/
/* quit_Apple_Event parameter by the AppleEvent Manager.  "quit_Program" is*/
/* set which causes this application to quit at the start of the next*/
/* iteration of the main event loop.*/
/* Though the quit AppleEvent doesnt contain any parameters, the standard thing*/
/* to do in reaction to any AppleEvent is to check to see if there are any*/
/* required parameters in the AppleEvent that this routine doesnt recognize.*/
/* DoneRequiredParms checks for this condition and returns an error if there are*/
/* in fact required parameters in the AppleEvent or if some other error occurs*/
/* during the check.*/
/*  */
{
	OSErr		error;
	OSErr		function_Result;
/* END VAR */


	function_Result = noErr;
	
	/*  */
	/* Program will now eventually quit */
	/*  */
	g_Shell.interface.quit_Program = TRUE;
	
/*  */
/* quit AE has no parms, but check in case the client requires any */
/*  */

	error = Determine_If_Required_Params_Done(quit_Apple_Event);
	if (error != noErr) {
		function_Result = error;
		goto Error_Handler;
	}

	function_Result = noErr;
	goto Normal_Function_Exit;

Error_Handler:

Normal_Function_Exit:
	return(function_Result);
	
} /* OSErr Handler_For_Quit_Apple_Event (... */
/*=**************************************************************************=*/


/*=**************************************************************************=*/
pascal OSErr Handler_For_Open_Application_Apple_Event (AppleEvent *open_Application_Apple_Event
						, AppleEvent *reply
						, INTEGER_32 handlerRefCon)
{
	OSErr		error;
	OSErr		function_Result;
/* END VAR */

	function_Result = -1;
	
	/*  */
	/* Check if valid params ready. */
	/*  */
	error = Determine_If_Required_Params_Done(open_Application_Apple_Event);
	if (error != noErr) {
		function_Result = error;
		goto Error_Handler;
	}
	
	/*  */
	/* Do nothing in this version. Simple stub routine */
	/*  */

	function_Result = noErr;
	goto Normal_Function_Exit;

Error_Handler:

Normal_Function_Exit:
	return(function_Result);
	
} /* OSErr Handler_For_Open_Application_Apple_Event (... */
/*=**************************************************************************=*/


/*=**************************************************************************=*/
pascal OSErr Handler_For_Open_Document_Apple_Event (AppleEvent *open_Document_Apple_Event
						, AppleEvent *reply
						, INTEGER_32 handlerRefCon)
{
	OSErr		error;
	OSErr		function_Result;
/* END VAR */

	function_Result = -1;
	
	/*  */
	/* Check if valid params ready. */
	/*  */
	error = Determine_If_Required_Params_Done(open_Document_Apple_Event);
	if (error != noErr) {
		function_Result = error;
		goto Error_Handler;
	}
	
	/*  */
	/* Do nothing in this version. Simple stub routine */
	/*  */

	function_Result = noErr;
	goto Normal_Function_Exit;

Error_Handler:

Normal_Function_Exit:
	return(function_Result);
	
} /* OSErr Handler_For_Open_Document_Apple_Event (... */
/*=**************************************************************************=*/


/*=**************************************************************************=*/
pascal OSErr Handler_For_Print_Document_Apple_Event (AppleEvent *print_Document_Apple_Event
						, AppleEvent *reply
						, INTEGER_32 handlerRefCon)
/*  */
/* This operation is functionally identical to the ODOC event, with the additon */
/* of calling your print routine.  */
/*  */
{
	OSErr		error;
	OSErr		function_Result;
/* END VAR */


	function_Result = -1;
	
	/*  */
	/* Check if valid params ready. */
	/*  */
	error = Determine_If_Required_Params_Done(print_Document_Apple_Event);
	if (error != noErr) {
		function_Result = error;
		goto Error_Handler;
	}
	
	/*  */
	/* Do nothing in this version. Simple stub routine */
	/*  */

	function_Result = noErr;
	goto Normal_Function_Exit;

Error_Handler:

Normal_Function_Exit:
	return(function_Result);
	
} /* OSErr Handler_For_Print_Document_Apple_Event (... */
/*=**************************************************************************=*/


/*=**************************************************************************=*/
pascal OSErr Handler_For_Answer_Apple_Event (AppleEvent *answer_Document_Apple_Event
						, AppleEvent *reply
						, INTEGER_32 handlerRefCon)
/*  */
/* This is the 'ansr', or answer, handler.  You need this if you ever want to */
/* use QueueReply as an option to AESend (which you will) since the reply  */
/* is going to be coming in through your event loop like any other  */
/* EXCEPT if the event is coming from yourself to yourself via the faster */
/* method of a kCurrentProcess Process Serial Number constant, and then */
/* waitnext event is skipped completely and this command is called directly */
/* and since it is direct, answers will not queue, they would reply instantly and synchronously. */
/* This used to be a core apple event that was mandatory, not it is */
/* the only non-mandatory core apple event but it should be mandatory. */
/*  */
{
	OSErr				os_Error;
	OSErr				function_Result;
	DescType			returned_Type;
	CARDINAL_32	actual_Size;
	CARDINAL_32	parameter;
/* END VAR */


	function_Result = -1;
		
	/*  */
	/* Check if valid params ready. */
	/*  */
	os_Error = Determine_If_Required_Params_Done(answer_Document_Apple_Event);
	if (os_Error != noErr) {
	
		if (os_Error == errAEEventNotHandled) {
			/*  */
			/* I do not have documentation on core required apple event */
			/* of type Answer. I do not know what to take off. Ignore os_Error */
			/* indicating we forgot to process something */
			/*  */
			; /* NULL */
		}
		else {
			/*  */
			/* For Answer, we just ignore errors */
			/*  */
		/* ===
			function_Result = os_Error;
			goto Error_Handler;
		==== */
		}
	}
		
	/*  */
	/* Do nothing in this version. Simple stub routine */
	/*  */

	function_Result = noErr;
	goto Normal_Function_Exit;

Error_Handler:

Normal_Function_Exit:
	return(function_Result);
	
} /* pascal OSErr Handler_For_Answer_Apple_Event (... */
/*=**************************************************************************=*/


/*=**************************************************************************=*/
OSErr Determine_If_Required_Params_Done (AppleEvent *passed_Apple_Event)
/*  */
/* Determine_If_Required_Params_Done */
/* Determine_If_Required_Params_Done checks to see if the AppleEvent specified by the*/
/* passed_Apple_Event parameter has any required parameters that we havent yet*/
/* processed.  If there arent any left, then noErr is returned.  If there are*/
/* required parameters that havent been processed yet, then errAEEventNotHandled*/
/* is returned.  If any other errors occur, then that error code is returned.*/
/*  */
{
	OSErr		error;
	OSErr		function_Result;
	DescType	typeCode; /* Type of AppleEvent attribute found; ignored */
	Size		actualSize; /* Actual size of parameters; ignored */
/* END VAR */


	function_Result = noErr;
	
/*  */
/* Are there any required parameters in AppleEvent we didnt process? */
/*  */

	error = AEGetAttributePtr(	(const AppleEvent *)passed_Apple_Event
									, keyMissedKeywordAttr
									, typeWildCard
									, &typeCode
									, nil
									, 0
									, &actualSize);
/*  */
/* Huh????*/
/*  */
	if (error == noErr) {
		/*  */
		/* There was at least one required parameter we didnt process */
		/*  */
		function_Result = errAEEventNotHandled;
		goto Error_Handler;
	}
	else {
		if (error == errAEDescNotFound) {
			/*  */
			/* No required parameters left, so no error */
			/*  */
			function_Result = noErr;
			goto Error_Handler;
		}
		else {
			/*  */
			/* Some other error happened */
			/*  */
			function_Result = error;
			goto Error_Handler;
		}
	}

	function_Result = noErr;
	goto Normal_Function_Exit;

Error_Handler:

Normal_Function_Exit:
	return(function_Result);
	
} /* OSErr Determine_If_Required_Params_Done (AppleEvent quit_Apple_Event) */
/*=**************************************************************************=*/


/*=**************************************************************************=*/
Safe_Boolean Create_Apple_Event_Proc_Pointers(void)
/*  */
/* This is only called if PoerPC compilaer archetecture is used */
/* Motorola style uses immediately loaded/locked segments and pointers */
/* set right before calls. */
/*  */
{
	Safe_Boolean		function_Result;
/* END VAR */
	
	
	function_Result = SAFE_FALSE;

	g_Open_Application_Apple_Event_UPP = (AEEventHandlerUPP) NewRoutineDescriptor(
					(ProcPtr)(&Handler_For_Open_Application_Apple_Event)
					, uppAEEventHandlerProcInfo
					, GetCurrentISA());
	
	g_Open_Document_Apple_Event_UPP = (AEEventHandlerUPP) NewRoutineDescriptor(
					(ProcPtr)(&Handler_For_Open_Document_Apple_Event)
					, uppAEEventHandlerProcInfo
					, GetCurrentISA());
	
	g_Quit_Apple_Event_UPP = (AEEventHandlerUPP) NewRoutineDescriptor(
					(ProcPtr)(&Handler_For_Quit_Apple_Event)
					, uppAEEventHandlerProcInfo
					, GetCurrentISA());
	
	g_Print_Application_Apple_Event_UPP = (AEEventHandlerUPP) NewRoutineDescriptor(
					(ProcPtr)(&Handler_For_Print_Document_Apple_Event)
					, uppAEEventHandlerProcInfo
					, GetCurrentISA());
	
	g_Answer_Apple_Event_UPP = (AEEventHandlerUPP) NewRoutineDescriptor(
					(ProcPtr)(&Handler_For_Answer_Apple_Event)
					, uppAEEventHandlerProcInfo
					, GetCurrentISA());
	
	function_Result = SAFE_TRUE;
	goto Normal_Function_Exit;
		
Error_Handler:
	function_Result = SAFE_FALSE;

Normal_Function_Exit:

	return(function_Result);
		
} /* Safe_Boolean Create_Apple_Event_Proc_Pointers */
/*=**************************************************************************=*/


/*=**************************************************************************=*/
OSErr Initialize_AE_Handlers(void)
{
OSErr					function_Result;
OSErr					install_Result;
Boolean 				apple_Events_Ready;
AEEventHandlerUPP	handler_Proc_Pointer;
INTEGER_32	 		response;
/* END VAR */


	function_Result = -1;	
	g_Apple_Events_Possible = SAFE_FALSE;
	
	apple_Events_Ready = FALSE;
	if (Gestalt_Available() == TRUE) {
		function_Result = Gestalt(gestaltAppleEventsAttr, &response);
		if (function_Result == noErr) {
			apple_Events_Ready = TRUE;
		}
	}
	
	if (apple_Events_Ready != TRUE) {
		goto Error_Handler;
	}

	/*  */
	/* The following series of calls installs all our AppleEvent Handlers. */
	/* These handlers are added to the application event handler list that  */
	/* the AppleEvent manager maintains.  So, whenever an AppleEvent happens */
	/* and we call AEProcessEvent, the AppleEvent manager will check our */
	/* list of handlers and dispatch to it if there is one. */
	/*  */


	/*  */
	/* Install the four required Apple Events */
	/*  */
	
#if !defined(ORIGINAL_MAC_SYSTEM)
#error "UNDEFINED PARAMETER!"
#endif
#if (ORIGINAL_MAC_SYSTEM == 0)
 		handler_Proc_Pointer = g_Open_Application_Apple_Event_UPP;
#else
 		handler_Proc_Pointer = (AEEventHandlerUPP)&Handler_For_Open_Application_Apple_Event;
#endif 

	function_Result = AEInstallEventHandler(
							kCoreEventClass
							, kAEOpenApplication
			            		, handler_Proc_Pointer
							, 0
							, FALSE);
	if (function_Result != noErr) {
		goto Error_Handler;
	}
	
#if (ORIGINAL_MAC_SYSTEM == 0)
	handler_Proc_Pointer = g_Open_Document_Apple_Event_UPP;
#else
	handler_Proc_Pointer = (AEEventHandlerUPP)&Handler_For_Open_Document_Apple_Event;
#endif 

	function_Result = AEInstallEventHandler(
							kCoreEventClass
							, kAEOpenDocuments
			            		, handler_Proc_Pointer
							, 0
							, FALSE);
	if (function_Result != noErr) {
		goto Error_Handler;
	}
	
#if (ORIGINAL_MAC_SYSTEM == 0)
	handler_Proc_Pointer = g_Quit_Apple_Event_UPP;
#else
	handler_Proc_Pointer = (AEEventHandlerUPP)&Handler_For_Quit_Apple_Event;
#endif 

	function_Result = AEInstallEventHandler(
							kCoreEventClass
							, kAEQuitApplication
			            		, handler_Proc_Pointer
							, 0
							, FALSE);
	if (function_Result != noErr) {
		goto Error_Handler;
	}
	
	
#if (ORIGINAL_MAC_SYSTEM == 0)
	handler_Proc_Pointer = g_Print_Application_Apple_Event_UPP;
#else
	handler_Proc_Pointer = (AEEventHandlerUPP)&Handler_For_Print_Document_Apple_Event;
#endif 

	function_Result = AEInstallEventHandler(
							kCoreEventClass
							, kAEPrintDocuments
			            		, handler_Proc_Pointer
							, 0
							, FALSE);
	if (function_Result != noErr) {
		goto Error_Handler;
	}
	
	/*  */
	/* Good. The four required Apple Events are setup and ready. */
	/* Set global flag */
	/*  */
	g_Apple_Events_Possible = SAFE_FALSE;
	
	/*  */
	/* Add additional abandoned 5th core apple event. Yes it IS a */
	/* core event, and it was part of the core spec originally. Support and use it */
	/* as a detector that Apple event processes are installed to some degree. */
	/* Otherwise, there is no legitimate safe "NOP" test. Stupid Apple for dropping */
	/* this critical core event. At least we intelligently use it and know of its existence. */
	/*  */
#if (ORIGINAL_MAC_SYSTEM == 0)
	handler_Proc_Pointer = g_Answer_Apple_Event_UPP;
#else
	handler_Proc_Pointer = (AEEventHandlerUPP)&Handler_For_Answer_Apple_Event;
#endif 

	function_Result = AEInstallEventHandler(
							kCoreEventClass
							, kAEAnswer
			            		, handler_Proc_Pointer
							, 0
							, FALSE);
	if (function_Result != noErr) {
		goto Error_Handler;
	}
	
	function_Result = noErr;
	goto Normal_Function_Exit;

Error_Handler:

Normal_Function_Exit:
	return(function_Result);

}
/*=**************************************************************************=*/


/*=**************************************************************************=*/
/* END OF FILE: Remote_Control_Standard.c */
